
local m_moduleLocation = select(2, ...)
local _, m_widgetId = m_moduleLocation:match("(.*/)(.*)") -- example
local m_models = require("apps.integrations.model"):GetModels(m_widgetId)
module(..., package.seeall, inconcert.orbit.application.Decorate)
local utils = require "shared.apps.utils"
m_htmlRender = nil
m_appLocation = nil
local m_modelName = "ExampleState"-- Aca definimos el nombre del model al cual accede por defecto el controller

--[[
	Aca definimos la lista de funciones que mapearan con las URLs, Ej:
		http://i6.inconcert.cc/inconcert/apps/integrations/el_nombre_del_widget/states/list_all
		http://i6.inconcert.cc/inconcert/apps/integrations/el_nombre_del_widget/states/get
]]--
function GetMapUris ()
	--LogDebug("[INTEGRATIONS.WIDGETS.EXAMPLE.STATES:GetMapUris] -------------------------------------------------------------------------------------> ")
	local definition = {
		{url = "/get/(.+)" , methodType = "GET" , fn = Get    },
		{url = "/list_all/", methodType = "POST", fn = ListAll},
		{url = "/save/"    , methodType = "POST", fn = Save   },
		{url = "/delete/"  , methodType = "POST", fn = Delete }
	}
	return definition
end

local function GetModel (entityName)
	--LogDebug("[INTEGRATIONS.EXAMPLE.STATES:GetModel]---> entityName %s", entityName)
	assert(type(entityName) == "string")
	if not m_models or not m_models[entityName] then
		local model = require "apps.integrations.model"
		model.CreateModelTable()
		m_models = model:GetModels(m_widgetId)
	end
	local entityModel = m_models[entityName]
	assert(type(entityModel) == "table")
	return entityModel
end

function Get (web, stateId)
	local vcc = inconcert.authUser.virtualContactCenter
	LogInfo("[INTEGRATIONS.EXAMPLE.STATES:Get] ----> Retrieving stateId %s, State %s, vcc %s", stateId, (inconcert.authUser.userName.."@"..vcc), vcc)

	local keyspace = utils.GetDatasource():CassandraConnection("Integrations")
	local state = GetModel(m_modelName).Get(vcc, stateId, keyspace)
	keyspace:Close()

	if utils.TableIsEmpty(state) then
		web.status = 404
		return json.encode({result = "NOT_FOUND"})
	end

	return json.encode(state)
end

function ListAll (web)
	local postData = web.request
	LogInfo("[INTEGRATIONS.EXAMPLE.STATES.ListAll] ---> List States -  posted data: '%s'", web.input["post_data"])
	local keyspace = utils.GetDatasource():CassandraConnection("Integrations")
	local states = GetModel(m_modelName).List(keyspace)
	keyspace:Close()
	local function CanAdd (state)
		if utils.TableIsEmpty(state) then
			return false
		end
		return true
	end
	local filteredstates = {}
	for _, state in pairs(states or {}) do
		if CanAdd(state) then
			table.insert(filteredstates, state)
		end
	end
	LogInfo("[INTEGRATIONS.EXAMPLE.STATES.ListAll] states %s", json.encode(filteredstates))
	return filteredstates
end

function Save (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName .. "@" .. vcc
	local ok, state = pcall(json.decode, web.input["post_data"])

	LogInfo("[INTEGRATIONS.EXAMPLE.STATES.SAVE] ---> Saving State - user '%s', vcc '%s'. posted data: '%s'", userId, vcc, web.input["post_data"])

	if (state.Id or ""):len() == 0 then
		web.status = 400
		local err = {result = "INVALID_DATA", type = "MSSING FIELD ID"}
		LogError("[INTEGRATIONS.EXAMPLE.STATES.SAVE] ERROR Saving State - user '%s', vcc '%s'. err: '%s'", userId, vcc, json.encode(err))
		return err
	end
	if (state.Name or ""):len() == 0 then
		web.status = 400
		local err = {result = "INVALID_DATA", type = "MSSING FIELD NAME"}
		LogError("[INTEGRATIONS.EXAMPLE.STATES.SAVE] ERROR Saving State - user '%s', vcc '%s'. err: '%s'", userId, vcc, json.encode(err))
		return err
	end

	state.Vcc = vcc
	local keyspace = utils.GetDatasource():CassandraConnection("Integrations")
	GetModel(m_modelName).Save(vcc, state, keyspace)
	keyspace:Close()
	LogInfo("[INTEGRATIONS.EXAMPLE.STATES.SAVE] SUCCESS Saved State - user '%s', vcc '%s'.", userId, vcc)

	return {result = "OK"}
end

function Delete (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName .. "@" .. vcc
	local ok, state = pcall(json.decode, web.input["post_data"])

	LogInfo("[INTEGRATIONS.EXAMPLE.STATES.Delete] ---> Deleting State - state '%s'", web.input["post_data"])

	if (state.Id or ""):len() == 0 then
		web.status = 400
		local err = {result = "INVALID_DATA", type = "MSSING FIELD ID"}
		LogError("[INTEGRATIONS.EXAMPLE.STATES.Delete] ERROR Deleting State - err: '%s'", json.encode(err))
		return err
	end

	local keyspace = utils.GetDatasource():CassandraConnection("Integrations")
	local currentUser = GetModel(m_modelName).Get(vcc, state.Id, keyspace)
	if utils.TableIsEmpty(currentUser) then
		keyspace:Close()
		web.status = 404
		local err = {result = "NOT_FOUND", type = "STATE NOT FOUND"}
		LogError("[INTEGRATIONS.EXAMPLE.STATES.Delete] ERROR Deleting State - err: '%s'", json.encode(err))
		return err
	end

	local fullStateId = utils.AppendVcc(state.Id, vcc)
	GetModel(m_modelName).Remove(fullStateId, keyspace)
	keyspace:Close()
	LogInfo("[INTEGRATIONS.EXAMPLE.STATES.Delete] SUCCESS Deleted State - State '%s'.", fullStateId)

	return {result = "OK"}
end
