local apiModel = require "api.model"
local models = apiModel:GetModels()
local utils = require "shared.apps.utils"
local api_utils = require "api.utils"
local admCampaignController = require("apps.administration.controllers.campaigns")

module(..., package.seeall, inconcert.orbit.application.Decorate)

local function Search (web, request)
	local result = admCampaignController.Search(web)
	return api_utils.replyOk(web, result)
end

local function GetCampaignMembers (web, request)
	local result = admCampaignController.GetOnlyMembers(web)
	return api_utils.replyOk(web, result)
end

local function Get (web, campaignId)
	local result = admCampaignController.Get(web, campaignId)
	if result.result then
		LogError("[API:campaigns:Get] %s - %s", campaignId, result.result)
		return api_utils.replyError(web, web.status, result.result)
	end
	return api_utils.replyOk(web, { campaign = result })
end

local function GetDispositions (web, campaignId)
	local vcc = inconcert.authUser.virtualContactCenter
	local campaign = models.Campaigns.GetCampaign(vcc, campaignId, nil, { "dispositionCodes" })
	if not campaign then
		LogError("[API:campaigns:GetDispositions] %s - campaign not found", campaignId)
		return api_utils.replyNotFound(web, "campaign not found")
	end
	return api_utils.replyOk(web, { dispositions = campaign.dispositionCodes or {} })
end

local function GetAccountTemplateMessages (web, request)
	local account = models.Campaigns.GetCampaignAccount(request.campaign, request.account, nil, { "HSMConfig" })
	if not account then
		LogError("[API:campaigns:GetAccountTemplateMessages] %s - account not found", request.account)
		return api_utils.replyNotFound(web, "account not found")
	end
	local messages = account.HSMConfig and account.HSMConfig.TemplateMessages or {}
	return api_utils.replyOk(web, { messages = messages })
end

local function GetConfiguredChannels (web, request)
	local vcc = inconcert.authUser.virtualContactCenter
	local campaignsAccounts
	if request.campaign then
		campaignsAccounts = { [request.campaign] = models.Campaigns.GetCampaignAccounts(request.campaign) or {} }
	else
		campaignsAccounts = models.Campaigns.GetVccAccounts(vcc)
	end

	local channels = {}
	for campaignId, accounts in pairs(campaignsAccounts) do
		for accountId, account in pairs(accounts) do
			if not request.enabled or account.state == "1" then
				channels[account.proxyType] = channels[account.proxyType] or {}
				channels[account.proxyType][campaignId] = channels[account.proxyType][campaignId] or {}
				table.insert(channels[account.proxyType][campaignId], accountId)
			end
		end
	end
	return api_utils.replyOk(web, { channels = channels })
end

local function SaveIntegrationCCI (web, request)
	local result = admCampaignController.SaveIntegrationCCI(web)
	return api_utils.replyOk(web, result)
end

--[[----------------------------------------------------------------------
	Mapeo de funciones que quedan disponibles en el módulo para luego ser utilizadas en el mapeo de funciones con las urls (/definition/ARCHIVO.lua función "urlLinkedFunction")
--------------------------------------------------------------------------]]
_M.Get                        = Get
_M.GetDispositions            = GetDispositions
_M.GetAccountTemplateMessages = GetAccountTemplateMessages
_M.GetConfiguredChannels      = GetConfiguredChannels
_M.GetCampaignMembers         = GetCampaignMembers
_M.Search                     = Search
_M.SaveIntegrationCCI         = SaveIntegrationCCI
-------------------------------------------------------------------------

function MapUris( m, authFilter, prefix )
	local f = authFilter
	local p = prefix
	local Log = require("apps.agent.requestFilters").LogRequest
	local jsonValidator = utils.JsonValidator
	local vr = utils.JsonValidatorReply
	local r = utils.JsonReply

	-- mapeo de URLs dinamicas (definidas en \api\definition\full.lua)
	local replyPostFn = Log .. f .. vr
	local replyGetFn = Log .. f .. r
	local apiType = "campaigns"
	api_utils:MapDynamicURIs(apiType, m, _M, replyPostFn, replyGetFn)
	--
	m_htmlRender = htmlRender
end
