local utils = require "shared.apps.utils"
local api_utils = require "api.utils"

local model_perm = require("shared.permissions.models.permissions")
local admGroupsController = require("apps.administration.controllers.groups")
local m_adminModel
module(..., package.seeall, inconcert.orbit.application.Decorate)

local function Get (web, permission)
	local vcc = inconcert.authUser.virtualContactCenter
	local result = model_perm.LoadAllGroupPermissions(vcc)
	if result.result then
		LogError("[API:groups:Get] - %s", result.result)
		return api_utils.replyError(web, web.status, result.result)
	end
	local groups 
	if permission and permission ~= '' then
		groups = {}
		for group in pairs(result) do
			if result[group].Permissions and result[group].Permissions[permission:lower()] then
				groups[group] = result[group]
			end
		end
	else
		groups = result
	end
	return api_utils.replyOk(web, { groups = groups })
end

local function GetGroupsExportInfo (web)
	local vcc = inconcert.authUser.virtualContactCenter
	m_adminModel = m_adminModel or require "apps.administration.model"
	if not m_adminModel.IsCreatedModel() then
		m_adminModel:BuildModels()
	end

	local result = admGroupsController.GetGroupsExportInfo(web)
	LogInfo("[API:Groups:GetGroupsExportInfo] VCC %s - result %s", vcc, json.encode(result))
	if result.result then
		return api_utils.replyError(web, web.status, result.result)
	end
	return api_utils.replyOk(web, { groups = result })
end

--[[----------------------------------------------------------------------
	Mapeo de funciones que quedan disponibles en módulo para luego ser utilizadas en el mapeo de funciones con las urls (/definition/ARCHIVO.lua función "urlLinkedFunction")
--------------------------------------------------------------------------]]
_M.Get = Get
_M.GetGroupsExportInfo = GetGroupsExportInfo
-------------------------------------------------------------------------

function MapUris( m, authFilter, prefix )
	local f = authFilter
	local p = prefix
	local Log = require("apps.agent.requestFilters").LogRequest
	local jsonValidator = utils.JsonValidator
	local vr = utils.JsonValidatorReply
	local r = utils.JsonReply
	-- mapeo de URLs dinamicas (definidas en \api\definition\full.lua)
	local replyPostFn = Log .. f .. vr
	local replyGetFn = Log .. f .. r
	local apiType = "groups"
	api_utils:MapDynamicURIs(apiType, m, _M, replyPostFn, replyGetFn)
	--
	m_htmlRender = htmlRender
end
