local md5 = require "md5"
local lua_globaltimer = require "lua_globaltimer"
local utils = require "shared.apps.utils"
local api_utils = require "api.utils"
local m_outboundEngineClient = require "apps.outbound_engine.outboundengineclient"
module(..., package.seeall, inconcert.orbit.application.Decorate)

local appName = "outbound_engine_batch"

local function GetActionStatus (web, request)
	request.vcc = inconcert.authUser.virtualContactCenter
	request.apiDefinition = nil
	local result = m_outboundEngineClient.GetClient():GetBatchActionStatus(json.encode(request))
    if result.result ~= "OK" then
		LogError("[API:%s:GetBatchActionStatus] %s - %s", appName, json.encode(request), json.encode(result))
		return api_utils.replyServerError(web, result)
	end
	return api_utils.replyOk(web, result.status)
end

local function Create (web, request)
	request.vcc = inconcert.authUser.virtualContactCenter
	request.apiDefinition = nil
	local result = m_outboundEngineClient.GetClient():CreateBatch(json.encode(request))
    if result.result ~= "OK" then
		LogError("[API:%s:Create] %s - %s", appName, json.encode(request), json.encode(result))
		return api_utils.replyServerError(web, result)
	end
	return api_utils.replyOk(web, result)
end

local function Update (web, request)
	request.vcc = inconcert.authUser.virtualContactCenter
	request.apiDefinition = nil
	local result = m_outboundEngineClient.GetClient():UpdateBatch(json.encode(request))
    if result.result ~= "OK" then
		LogError("[API:%s:Update] %s - %s", appName, json.encode(request), json.encode(result))
		return api_utils.replyServerError(web, result)
	end
	return api_utils.replyOk(web, result)
end

local function Destroy (web, request)
	request.vcc = inconcert.authUser.virtualContactCenter
	request.apiDefinition = nil
	local result = m_outboundEngineClient.GetClient():DestroyBatch(json.encode(request))
    if result.result ~= "OK" then
		LogError("[API:%s:Destroy] %s - %s", appName, json.encode(request), json.encode(result))
		return api_utils.replyServerError(web, result)
	end
	return api_utils.replyOk(web, result)
end

local function AddContacts (web, request)
	request.vcc = inconcert.authUser.virtualContactCenter
	request.apiDefinition = nil
	local result = m_outboundEngineClient.GetClient():AddContacts(json.encode(request))
    if result.result ~= "OK" then
		LogError("[API:%s:AddContacts] %s - %s", appName, json.encode(request), json.encode(result))
		return api_utils.replyServerError(web, result)
	end
	return api_utils.replyOk(web, result)
end

local function Pause (web, request)
	request.vcc = inconcert.authUser.virtualContactCenter
	request.apiDefinition = nil
	local result = m_outboundEngineClient.GetClient():PauseContacts(json.encode(request))
    if result.result ~= "OK" then
		LogError("[API:%s:Pause] %s - %s", appName, json.encode(request), json.encode(result))
		return api_utils.replyServerError(web, result)
	end
	return api_utils.replyOk(web, result)
end

local function Resume (web, request)
	request.vcc = inconcert.authUser.virtualContactCenter
	request.apiDefinition = nil
	local result = m_outboundEngineClient.GetClient():ResumeContacts(json.encode(request))
    if result.result ~= "OK" then
		LogError("[API:%s:Resume] %s - %s", appName, json.encode(request), json.encode(result))
		return api_utils.replyServerError(web, result)
	end
	return api_utils.replyOk(web, result)
end

local function Cancel (web, request)
	request.vcc = inconcert.authUser.virtualContactCenter
	request.apiDefinition = nil
	request.cause = "API"
	local result = m_outboundEngineClient.GetClient():CancelContacts(json.encode(request))
    if result.result ~= "OK" then
		LogError("[API:%s:Cancel] %s - %s", appName, json.encode(request), json.encode(result))
		return api_utils.replyServerError(web, result)
	end
	return api_utils.replyOk(web, result)
end

local function Finish (web, request)
	request.vcc = inconcert.authUser.virtualContactCenter
	request.apiDefinition = nil
	request.cause = "API"
	local result = m_outboundEngineClient.GetClient():FinishContacts(json.encode(request))
    if result.result ~= "OK" then
		LogError("[API:%s:Finish] %s - %s", appName, json.encode(request), json.encode(result))
		return api_utils.replyServerError(web, result)
	end
	return api_utils.replyOk(web, result)
end

local function Remove (web, request)
	request.vcc = inconcert.authUser.virtualContactCenter
	request.apiDefinition = nil
	request.cause = "API"
	local result = m_outboundEngineClient.GetClient():RemoveContacts(json.encode(request))
    if result.result ~= "OK" then
		LogError("[API:%s:Remove] %s - %s", appName, json.encode(request), json.encode(result))
		return api_utils.replyServerError(web, result)
	end
	return api_utils.replyOk(web, result)
end

local function Recycle (web, request)
	request.vcc = inconcert.authUser.virtualContactCenter
	request.apiDefinition = nil
	local result = m_outboundEngineClient.GetClient():RecycleContacts(json.encode(request))
    if result.result ~= "OK" then
		LogError("[API:%s:Recycle] %s - %s", appName, json.encode(request), json.encode(result))
		return api_utils.replyServerError(web, result)
	end
	return api_utils.replyOk(web, result)
end

local function Schedule (web, request)
	request.vcc = inconcert.authUser.virtualContactCenter
	request.apiDefinition = nil
	request.cause = "API"
	local result = m_outboundEngineClient.GetClient():ScheduleContacts(json.encode(request))
    if result.result ~= "OK" then
		LogError("[API:%s:Schedule] %s - %s", appName, json.encode(request), json.encode(result))
		return api_utils.replyServerError(web, result)
	end
	return api_utils.replyOk(web, result)
end

local function ChangePriority (web, request)
	request.vcc = inconcert.authUser.virtualContactCenter
	request.apiDefinition = nil
	local result = m_outboundEngineClient.GetClient():ChangePriorityContacts(json.encode(request))
    if result.result ~= "OK" then
		LogError("[API:%s:ChangePriority] %s - %s", appName, json.encode(request), json.encode(result))
		return api_utils.replyServerError(web, result)
	end
	return api_utils.replyOk(web, result)
end

local function SetEngineType (web, request)
	request.vcc = inconcert.authUser.virtualContactCenter
	request.apiDefinition = nil
	local result = m_outboundEngineClient.GetClient():SetEngineTypeContacts(json.encode(request))
    if result.result ~= "OK" then
		LogError("[API:%s:SetEngineType] %s - %s", appName, json.encode(request), json.encode(result))
		return api_utils.replyServerError(web, result)
	end
	return api_utils.replyOk(web, result)
end

local function SetAgent (web, request)
	request.vcc = inconcert.authUser.virtualContactCenter
	request.apiDefinition = nil
	local result = m_outboundEngineClient.GetClient():SetContactsAgent(json.encode(request))
    if result.result ~= "OK" then
		LogError("[API:%s:SetAgent] %s - %s", appName, json.encode(request), json.encode(result))
		return api_utils.replyServerError(web, result)
	end
	return api_utils.replyOk(web, result)
end

--[[----------------------------------------------------------------------
	Mapeo de funciones que quedan disponibles en módulo para luego ser utilizadas en el mapeo de funciones con las urls (/definition/ARCHIVO.lua función "urlLinkedFunction")
--------------------------------------------------------------------------]]
_M.GetActionStatus              = GetActionStatus
_M.Create                       = Create
_M.Update                       = Update
_M.Destroy                      = Destroy
_M.AddContacts                  = AddContacts
_M.Pause                  		= Pause
_M.Resume                  		= Resume
_M.Cancel                  		= Cancel
_M.Finish                  		= Finish
_M.Remove                  		= Remove
_M.Recycle                  	= Recycle
_M.Schedule                  	= Schedule
_M.ChangePriority               = ChangePriority
_M.SetEngineType                = SetEngineType
_M.SetAgent                		= SetAgent
-------------------------------------------------------------------------

function MapUris (m, authFilter, prefix)
	local f = authFilter
	local p = prefix
	local Log = require("apps.agent.requestFilters").LogRequest
	local jsonValidator = utils.JsonValidator
	local vr = utils.JsonValidatorReply
	local r = utils.JsonReply
	-- mapeo de URLs dinamicas (definidas en \api\definition\full.lua)
	local replyPostFn = Log .. f .. vr
	local replyGetFn = Log .. f .. r
	local apiType = "outbound_engine_batch"
	api_utils:MapDynamicURIs(apiType, m, _M, replyPostFn, replyGetFn)
	--
	m_htmlRender = htmlRender
end
