local md5 = require "md5"
local lua_globaltimer = require "lua_globaltimer"
local utils = require "shared.apps.utils"
local api_utils = require "api.utils"
local m_outboundEngineClient = require "apps.outbound_engine.outboundengineclient"
module(..., package.seeall, inconcert.orbit.application.Decorate)

local appName = "outbound_engine_process"

local function Get (web, processId)
	local status, process = utils.GetOEApi().native.processes.GetProcess(processId)
	if not status then
		LogError("[API:%s:Get] %s - process not found", appName, processId)
		return api_utils.replyNotFound(web, { result = "NOT_FOUND", reason = "process not found" })
	end
	return api_utils.replyOk(web, { process = process })
end

local function Start (web, _request, processId)
    local params = { 
		processId = processId,
		vcc = inconcert.authUser.virtualContactCenter
	}
	local result = m_outboundEngineClient.GetClient():StartProcess(json.encode(params))
    if result.result ~= "OK" then
		LogError("[API:%s:Start] %s - %s", appName, processId, json.encode(result))
		return api_utils.replyServerError(web, result)
	end
	return api_utils.replyOk(web, result)
end

local function Stop (web, _request, processId)
    local params = { 
		processId = processId,
		vcc = inconcert.authUser.virtualContactCenter
	}
	local result = m_outboundEngineClient.GetClient():StopProcess(json.encode(params))
    if result.result ~= "OK" then
		LogError("[API:%s:Stop] %s - %s", appName, processId, json.encode(result))
		return api_utils.replyServerError(web, result)
	end
	return api_utils.replyOk(web, result)
end

--[[----------------------------------------------------------------------
	Mapeo de funciones que quedan disponibles en módulo para luego ser utilizadas en el mapeo de funciones con las urls (/definition/ARCHIVO.lua función "urlLinkedFunction")
--------------------------------------------------------------------------]]
_M.Get                          = Get
_M.Start                        = Start
_M.Stop                         = Stop
-------------------------------------------------------------------------

function MapUris (m, authFilter, prefix)
	local f = authFilter
	local p = prefix
	local Log = require("apps.agent.requestFilters").LogRequest
	local jsonValidator = utils.JsonValidator
	local vr = utils.JsonValidatorReply
	local r = utils.JsonReply
	-- mapeo de URLs dinamicas (definidas en \api\definition\full.lua)
	local replyPostFn = Log .. f .. vr
	local replyGetFn = Log .. f .. r
	local apiType = "outbound_engine_process"
	api_utils:MapDynamicURIs(apiType, m, _M, replyPostFn, replyGetFn)
	--
	m_htmlRender = htmlRender
end
