local apiModel = require "api.model"
local models = apiModel:GetModels()
local utils = require "shared.apps.utils"
local api_utils = require "api.utils"
local agent =  require "apps.agent.agent"
local m_adminModel, m_adminStates, m_adminChannels, m_agentService
module(..., package.seeall, inconcert.orbit.application.Decorate)

local function Get (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local keyspace = nil
	local mustSantizeInfo = true
	local vccInfo = models.VCCs.Get(vcc, keyspace, mustSantizeInfo)
	return api_utils.replyOk(web, { vcc = vccInfo })
end

local function GetStates (web)
	local userVcc = inconcert.authUser.virtualContactCenter
	m_adminModel = m_adminModel or require "apps.administration.model"
	if not m_adminModel.IsCreatedModel() then
		m_adminModel:BuildModels()
	end
	m_adminStates = m_adminStates or require "apps.administration.controllers.states"

	local states = m_adminStates.GetAllStates()
	LogInfo("[Api.VCC.GetStates] states %s", json.encode(states))
	local idStates = {}
	if states then
		for key, value in pairs(states) do
			if value.vcc == "system" or value.vcc == userVcc then
				table.insert(idStates, key)
			end
		end
	end
	return api_utils.replyOk(web, { states = idStates})
end

local function GetChannels (web)
	m_adminModel = m_adminModel or require "apps.administration.model"
	if not m_adminModel.IsCreatedModel() then
		m_adminModel:BuildModels()
	end
	local ksAgent = utils.GetDatasource():CassandraConnection("Agent")
	m_adminChannels = m_adminChannels or require "apps.administration.controllers.channels"
	local channels = m_adminChannels.get_availables_channels(ksAgent) or {}
	--LogDebug("[Api.VCC.GetChannels] channels %s", json.encode(channels))
	local idChannels = {}
	for key, _ in pairs(channels or {}) do
		if agent.hasLicensesForChannel(key, ksAgent) then
			table.insert(idChannels, key)
		end
	end
	ksAgent:Close()
	return api_utils.replyOk(web, { channels = idChannels})
end

local function GetAvailableAgents (web)
	local result
	local channels = {}
	local status, request = api_utils.validateGet(web, "API:GetAvailableAgents", {
		{	id = "vcc"       ,  required = true ,	type = "string" },
		{	id = "campaign"  ,  required = true ,	type = "string" },
		{	id = "channel"   ,  required = false,	type = "string", default = nil },
		{	id = "idAttLevel",  required = false,	type = "string", default = nil },
	})

	if not status then
		return request
	end

	m_agentService = m_agentService or require "apps.agent.agentservice"
	status, result = m_agentService.GetAvailableAgents(request.vcc, request.campaign, request.channel, request.idAttLevel)

	if not status then
		return api_utils.replyServerError(web, result)
	end
	
	return api_utils.replyOk(web, { availableAgents = result } )
end

--[[----------------------------------------------------------------------
	Mapeo de funciones que quedan disponibles en módulo para luego ser utilizadas en el mapeo de funciones con las urls (/definition/ARCHIVO.lua función "urlLinkedFunction")
--------------------------------------------------------------------------]]
_M.Get = Get
_M.GetStates = GetStates
_M.GetChannels = GetChannels
_M.GetAvailableAgents = GetAvailableAgents
-------------------------------------------------------------------------

function MapUris( m, authFilter, prefix )
	local f = authFilter
	local p = prefix
	local Log = require("apps.agent.requestFilters").LogRequest
	local jsonValidator = utils.JsonValidator
	local vr = utils.JsonValidatorReply
	local r = utils.JsonReply
	-- m:dispatch_get ( Log .. f .. r .. check(checkParams, Get), p .. "/get/" )
	--
	local replyPostFn = Log .. f .. vr
	local replyGetFn = Log .. f .. r
	local apiType = "vcc"
	api_utils:MapDynamicURIs(apiType, m, _M, replyPostFn, replyGetFn)
	--
	m_htmlRender = htmlRender
end
