local api_utils = require "api.utils"

module((...), package.seeall)

local m_apiDefaultConfig = api_utils.GetApiURLsDefaultConfig()
local entityType = "repository"
local baseUri = string.format("/%s/", entityType)

local m_apiDefinition = {
	info = {
		name = entityType
	},
	items = {}
}

------------------------------------------------
--[[
	Exists repo file NNNNNNNNNNNNN
		url: /repository/is_found/ 
		method: GET
]]--
local definition = {
	entityType = entityType,
	id = "7bde4ac4-c7d2-48d9-b45e-c0d195ce63c9",
	name = "ValidateRepoFileExists",
	description = "Valid if the repository file exists",
	url = baseUri .. "is_found/",
	method = "GET",
	header = {
		Authorization = "....",
		["Content-Type"] = "application/json"
	},
	body = {},
	response = {},
	responseErrors = {},
	config = {
		urlLinkedFunction = "Exists",
		permission = m_apiDefaultConfig.permissions.read,
		rateLimits = api_utils.GetApiURLRateLimit,
		resourceType = "low", -- low, medium_light, medium, high
	}
}

table.insert(m_apiDefinition.items, definition)
------------------------------------------------

------------------------------------------------
--[[
	Delete NNNNNNNNNNNNN
		url: /repository/erase/
		method: POST
]]--

local definition =	{
	entityType = entityType,
	id = "77031997-da4c-40a3-a7ae-6be43713f711",
	name = "DeleteRepoFiles",
	description = "Delete Repository Files",
	url = baseUri .. "erase/",
	method = "POST",
	header = {
		Authorization = "....",
		["Content-Type"] = "application/json"
	},
	body = {
		{id = "id",      required = true,  type = "string" },
		{id = "section", required = false, type = "number", default = -1 }
	},
	response = {},
	responseErrors = {},
	config = {
		urlLinkedFunction = "Delete",
		permission = m_apiDefaultConfig.permissions.write,
		rateLimits = api_utils.GetApiURLRateLimit,
		resourceType = "low", -- low, medium_light, medium, high
	}
}

table.insert(m_apiDefinition.items, definition)
------------------------------------------------

------------------------------------------------
--[[
	BatchDelete NNNNNNNNNNNNN
		url: /repository/batch_delete/
		method: POST
]]--

local definition =	{
	entityType = entityType,
	id = "fc1d0301-b674-4845-b194-e8e611bb1dc1",
	name = "DeleteBatchRepoFiles",
	description = "Delete Batch Repository Files",
	url = baseUri .. "batch_delete/",
	method = "POST",
	header = {
		Authorization = "....",
		["Content-Type"] = "application/json"
	},
	body = {
		{
			id = "files",		required = true,	type = "table", array = true, children = {
				{	id = "id",			required = true,	type = "string" },
				{	id = "section",		required = false,	type = "number", default = -1 },
			}
		}
	},
	response = {},
	responseErrors = {},
	config = {
		urlLinkedFunction = "BatchDelete",
		permission = m_apiDefaultConfig.permissions.write,
		rateLimits = api_utils.GetApiURLRateLimit,
		resourceType = "low", -- low, medium_light, medium, high
	}
}

table.insert(m_apiDefinition.items, definition)
------------------------------------------------

--[[
	Get
		url: /repository/download/            asdfasd asdf asdfasdfadsf
		method: GET
		wsapi: true
]]--
local definition =	{
	entityType = entityType,
	id = "790eb4e1-f41c-4d0f-8b70-c744ad7a3a2b",
	name = "GetRepositoryFile",
	description = "Get Repository File",
	url = baseUri .. "download/(.*)",
	method = "GET",
	wsapi = true,
	header = {
		Authorization = "....",
		["Content-Type"] = "application/json"
	},
	body = {},
	response = {},
	responseErrors = {},
	config = {
		permission = m_apiDefaultConfig.permissions.read,
		rateLimits = api_utils.GetApiURLRateLimit,
		resourceType = "low", -- low, medium_light, medium, high,
		urlLinkedFunction = "Get"
	}
}

table.insert(m_apiDefinition.items, definition)
------------------------------------------------

--[[
	Add
		url: /repository/new/            asdfasd asdf asdfasdfadsf
		method: GET
		wsapi: true
]]--
local definition =	{
	entityType = entityType,
	id = "af51fd60-cb6e-497a-b22d-5d71232a4f69",
	name = "UploadRepositoryFile",
	description = "Upload Repository File",
	url = baseUri .. "new/(.*)",
	method = "POST",
	wsapi = true,
	header = {
		Authorization = "....",
		["Content-Type"] = "application/json"
	},
	body = {},
	response = {},
	responseErrors = {},
	config = {
		permission = m_apiDefaultConfig.permissions.write,
		rateLimits = api_utils.GetApiURLRateLimit,
		resourceType = "low", -- low, medium_light, medium, high,
		urlLinkedFunction = "Add"
	}
}

table.insert(m_apiDefinition.items, definition)
------------------------------------------------

function Get ()
	local m_apiDefinition = m_apiDefinition
	return m_apiDefinition
end