module(..., package.seeall, inconcert.orbit.application.Decorate)

local m_models = require("apps.administration.model"):GetModels()
local m_addressesBookModel
local utils = require "shared.apps.utils"
local permissions = require "shared.permissions.permissions"
local m_moduleLocation = select(2, ...)
local notificationsEngine = require("apps.agent.notifications")
local agentNotifications = require("apps.agent.agentNotifications")
local m_validations = require("apps.administration.controllers.validations.addresses_book").GetDefinitions()
require "lua_telephonyproxyclient"

m_htmlRender = nil
m_appLocation = nil

local m_telephonyProxyClientInstance = nil
local function GetTelephonyProxyClient ()
	if not m_telephonyProxyClientInstance then
		m_telephonyProxyClientInstance = lua_telephonyproxyclient.LuaTelephonyProxyClient(
			inconcert.environment.apps.agent.cluster, "webhandler")
	end
	m_telephonyProxyClientInstance:ConfigureCassandra({
		{
			host = inconcert.environment.apps.agent.cassandra.ip,
			port = inconcert.environment.apps.agent.cassandra.port
		}
	})
	m_telephonyProxyClientInstance:ConfigureTimeClient(
		inconcert.environment.apps.agent.timeServers)
	return m_telephonyProxyClientInstance
end

local m_addressesBook = {}

function m_addressesBook.GetModel ()
	if m_addressesBookModel then
		return m_addressesBookModel
	end
	m_addressesBookModel = m_models.Contacts.GetAddressesBookModel()
	return m_addressesBookModel
end

function m_addressesBook.Render (web)
	_M.response.DontCache(web)
	local env = {_T = m_htmlRender.extension.makeTextTranslator()}
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc

	LogInfo("[ADMINISTRATION.CONTACTS.ADDRESSES_BOOK:Render] Render form Addresses Book for user %s, group %s", userId, inconcert.authUser.userGroup)

	local htmlFile = m_appLocation .. "/pages/" .. "frm_contacts_addresses_book.html"

	local content = m_htmlRender:RenderPageLP(htmlFile, web, env)
	if not content then
		LogError("[ADMINISTRATION.CONTACTS.ADDRESSES_BOOK:Render] HTML Addresses Book does not exist: %s", htmlFile)
	end

	return content
end

function m_addressesBook.Get (web, addressId)
	local vcc = inconcert.authUser.virtualContactCenter
	local fullUserId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	--LogDebug("[ADMINISTRATION.CONTACTS.ADDRESSES_BOOK:Get] Retrieving Addresses Book %s, user %s.", addressId, fullUserId, vcc)

	local addressIdWithVcc = (addressId .. "@" .. vcc):upper()
	local success, addressesBook = m_addressesBook.GetModel().Get(addressIdWithVcc)
	if not success then
		local err = addressesBook
		web.status = 400
		return {result = "ERROR_RETRIEVING_ADDRESSES_BOOK", err = err}
	end

	addressesBook.Id = addressIdWithVcc
	return addressesBook
end

function m_addressesBook.Search (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local fullUserId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local startTime = os.time()

	LogInfo("[ADMINISTRATION.CONTACTS.ADDRESSES_BOOK:SEARCH] 1 - (START) Search rows for user '%s', group '%s'. Posted data: '%s'", fullUserId, inconcert.authUser.userGroup, web.input["post_data"])

	local filter = {vcc = vcc}
	local success, addressesBook = m_addressesBook.GetModel().Search(filter)
	if not success then
		local err = addressesBook
		web.status = 400
		return {result = "ERROR_RETRIEVING_ADDRESSES_BOOK", err = err}
	end
	--LogDebug(" [ADMINISTRATION.CONTACTS.ADDRESSES_BOOK:SEARCH] a - addressesBook total %s", #(addressesBook or {}))

	local totalUnfilteredRows = utils.GetRowCount(addressesBook)
	local totalAdded = 0
	local filteredRows = {}


	LogInfo("[ADMINISTRATION.CONTACTS.ADDRESSES_BOOK:SEARCH] 2 - Filtering rows")

	local postData = web.request
	for _, address in ipairs(addressesBook) do
		local addressId = utils.GetId(address.AddressId)
		address.Id = addressId
		address.Name = address.Name or addressId
		local hasMatch = utils.MatchFilter(address, postData)
		if hasMatch then
			totalAdded = totalAdded + 1
			table.insert(filteredRows, address)
		end
	end

	LogInfo("[ADMINISTRATION.CONTACTS.ADDRESSES_BOOK:SEARCH] 3 - Sorting rows")

	local response = {
		rows = filteredRows,
		last = "",
		total_rows = totalAdded,
		total_unfiltered_rows = totalUnfilteredRows
	}
	local responseParams = {
		MakeSortingValue = function(row)
			assert(row)
			return tostring(row.Name or row.Id):upper()
		end
	}
	local resultData = utils.MakeSearchResponse(response, postData, responseParams)

	local endTime = os.time()
	local elapsedTime = endTime - startTime
	LogInfo("[ADMINISTRATION.CONTACTS.ADDRESSES_BOOK:SEARCH] 4 - (END) Search rows. Retrieve %d of %d matched rows. Total Unfiltered Rows: %d. ELAPSED Time '%s' seconds.", #resultData.rows, resultData.total_rows, resultData.total_unfiltered_rows, elapsedTime)

	return resultData
end

--[[
]]--
function m_addressesBook.ValidatePhoneAddress (addressInfo)
	local vcc = inconcert.authUser.virtualContactCenter
	local client = GetTelephonyProxyClient()
	local result = client:ValidatePhone({
		VCC = vcc,
		number = addressInfo.Number,
		originCountryCode = addressInfo.Country or "",
		originAreaCode = addressInfo.Area or "",
		withTimezone = false,
		allowInvalid = false,
		withTrace = false
	})
	if result.result ~= "OK" then
		result.result = "INVALID_PHONE_NUMBER"
		return nil, result
	end

	addressInfo.Country = result.data.countryCode
	addressInfo.Area = result.data.areaCode
	addressInfo.Number = result.data.number
	addressInfo.GlobalNumber = "+" .. result.data.dialGlobal
	return addressInfo
end

--[[
]]--
function m_addressesBook.ValidateAddress (addressInfo)
	if addressInfo.Type == "Phone" then
		return m_addressesBook.ValidatePhoneAddress(addressInfo)
	else
		return addressInfo
	end
end

--[[
]]--
function m_addressesBook.Save (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc

	--LogDebug("[ADMINISTRATION.CONTACTS.ADDRESSES_BOOK:Save] %s User is Updating Addresses Book '%s'. ", userId, web.input["post_data"])

	local addressId, addressInfo = next(web.request)
	local isNew = addressInfo.isNew and true or false
	local addressInfo, err = m_addressesBook.ValidateAddress(addressInfo)
	if err then
		LogError("[ADMINISTRATION.CONTACTS.ADDRESSES_BOOK:Save] Error Validating Addresses Book '%s', err: %s",
				 json.encode(addressInfo), json.encode(err))
		web.status = 400
		return err
	end

	if isNew then
		local alreadyExist, err = m_addressesBook.GetModel().GetAddressByRawInfo(addressInfo)
		if alreadyExist then
			LogError("[ADMINISTRATION.CONTACTS.ADDRESSES_BOOK:Save] Unable to create Addresses Book. Already Exist Id '%s', addressInfo '%s'. ", currentAddressIdWithVcc, json.encode(addressInfo))
			web.status = 400
			return {result = "ALREADY_EXIST_ADDRESSES_BOOK"}
		end
		if err and err.errType then
			LogError("[ADMINISTRATION.CONTACTS.ADDRESSES_BOOK:Save] Unable to create Addresses Book. Error: %s. ", json.encode(err))
			web.status = 400
			return {result = err.errType or "ERROR_SAVING_ADDRESSES_BOOK"}
		end
	end
	--LogDebug("[ADMINISTRATION.CONTACTS.ADDRESSES_BOOK:Save] addressId '%s' addressInfo '%s'. ", addressId, json.encode(addressInfo))

	local success, response, err = m_addressesBook.GetModel().Save(addressInfo)
	if not success then
		local errType = response
		web.status = 400
		return {result = errType or "ERROR_SAVING_ADDRESSES_BOOK"}
	end

	-- OK
	--notifications
	notificationsEngine.SendNotification(isNew and "AddressesBookCreated" or "AddressesBookUpdated", 0, {addressInfo = addressInfo})

	return {result = "OK"}--, idMapping = idMapping}
end

--[[
]]
function m_addressesBook.Remove (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	--LogDebug("[ADMINISTRATION.CONTACTS.ADDRESSES_BOOK:Remove] %s User is Removing Addresses Book '%s'. ", userId, web.input["post_data"])

	local addressId = web.request[1]
	if not addressId then
		web.status = 400
		LogError("[ADMINISTRATION.CONTACTS.ADDRESSES_BOOK:Remove] Unable to Remove Addresses Book '%s'. ", addressId)
		return {result = "ERROR_REMOVING_ADDRESSES_BOOK"}
	end

	local addressIdWithVcc = (addressId .. "@" .. vcc):upper()
	local success, err = m_addressesBook.GetModel().Remove(addressIdWithVcc)
	if not success then
		web.status = 400
		return {result = "ERROR_REMOVING_ADDRESSES_BOOK", err = err}
	end

	-- OK

	--notifications
	notificationsEngine.SendNotification("AddressesBookDeleted", 0, {addressId = addressIdWithVcc})

	return {result = "OK"}
end

function MapUris (module, htmlRender, prefix, appLocation)
	local LogRequest = require("apps.agent.requestFilters").LogRequest

	module:dispatch_post(utils.JsonValidatorReply..LogRequest.. m_addressesBook.Search, prefix.."/addresses_book/search/")
	module:dispatch_get(utils.JsonReply..LogRequest..permissions:CheckPermission("administration", "view_contacts_addresses_book", m_addressesBook.Get), prefix.."/addresses_book/get/(.+)")
	module:dispatch_post(utils.JsonValidatorReply..LogRequest..permissions:CheckPermission("administration", "save_contacts_addresses_book",
			utils.WebJsonValidatorData(
				m_validations.basic,
				m_addressesBook.Save
			)
		),
		prefix.."/addresses_book/save/"
	)

	module:dispatch_post(utils.JsonValidatorReply..LogRequest..permissions:CheckPermission("administration", "delete_contacts_addresses_book", m_addressesBook.Remove), prefix.."/addresses_book/delete/")
	module:dispatch_get(m_addressesBook.Render, prefix .. "/addresses_book/frm/render/")

	m_htmlRender = htmlRender
	m_appLocation = appLocation
end
