local m_models = require("apps.administration.model"):GetModels()
local utils = require "shared.apps.utils"
local permissions = require "shared.permissions.permissions"
local relational_db = require("shared.relational_databases.sql"):Init()
local m_validations = require("apps.administration.controllers.validations.databases").GetDefinitions()
require "LuaOrm.Date"
require "uuid"
local audits = require "audits"
local wsapi = require "wsapi"
wsapi.request = require "wsapi.request"
module(..., package.seeall, inconcert.orbit.application.Decorate)
m_htmlRender = nil
m_appLocation = nil


local function loadDataSource()
	local mysql_config = inconcert.environment.apps.administration.mysql
	if not mysql_config or not mysql_config.ndb_nodes then
		return
	end
	

	local ndb_nodes_table = {}
	for key, value, state in mysql_config["ndb_nodes"]:childs() do
		if state ~= "deleted" then
			table.insert(ndb_nodes_table, {host = value.host })
		end
	end
	-- validar que existe al menos un host
	if ndb_nodes_table[1] and mysql_config.ndb_nodes[1].host then
		return {
			ndb_nodes = ndb_nodes_table,
			Username = mysql_config.user or "root",
			Password = mysql_config.password or "",
			Port = mysql_config.port,
			Type = "mysql"
		}
	end
end

local ConnectionData = function (dbName, testConections, fnCallback)
	
	--1 valido que existan datos del config de host
	local dataSource = loadDataSource()
	if not dataSource then
		return
	end
	local ndb_nodes =  dataSource.ndb_nodes
	dataSource.Database = dbName or ""

	if dbName then
		local database = m_models.Databases.GetDatabases(dbName)
		if not database or not next(database) or not database.UserId then
			return 
		end
		dataSource.Username = database.UserId
	end

	--[[
		2. Para todos los host 
		2.1 Si 'testConections': valido que existan conexion a todos los host (Para stored procedures, trigger, functions) 
		2.2 Si no existe 'testConections', devuelvo la primera conexion disponible (Tablas)
		2.3 Si existe 'fnCallback', ejecuto funcion
	]]--. 
	local connectionData
	for _, value in ipairs(ndb_nodes) do
		dataSource.Server = value.host
		
		LogInfo("[ADMINISTRATION.DATABASES:ConnectionData] Try to connect dataSource %s testConections %s", json.encode(dataSource), testConections)
		connectionData = relational_db.Connect(dataSource)
		if not connectionData and testConections then
			return nil
		elseif connectionData and not testConections then
			if fnCallback then
				return fnCallback(connectionData)
			else
				return connectionData
			end
		elseif connectionData and testConections and fnCallback then
			local fnResult = fnCallback(connectionData)
			if fnResult.result ~= "OK"  then
				return fnResult
			end
		end
		--Conexion exitosa -> Paso a probar la sigt
		relational_db.Disconnect(connectionData)
	end
	--Todos los test de conexion con exito
	return connectionData
end

local function ReturnError(web, status, err, connectionData)
	web.status = status or 500
	if connectionData then
		relational_db.Disconnect(connectionData)
	end
	return err
end

local formatErrorReason = function (strError, line)
	--LogInfo("[ADMINISTRATION.DATABASES:formatErrorReason] strError %s",strError)
	_T = m_htmlRender.extension.makeTextTranslator()
	local vcc = inconcert.authUser.virtualContactCenter
	--1 Error not exist table
	local regex_not_table = string.match(strError, "Table '" .. vcc .. "_[A-Za-z0-9._]+' doesn't exist")
	if regex_not_table then
		return _T("Table does not exist")
	end
	local regex_not_procedure = string.match(strError, "PROCEDURE " .. vcc .. "_[A-Za-z0-9._]+ does not exist")
	if regex_not_procedure then
		return _T("Procedure does not exist")
	end

	if string.match(strError, "You have an error in your SQL syntax;") then
		return _T("You have an error in your SQL syntax") .. (line and (_T(" at line ") .. line) or "")
	end

	if string.match(strError, "Identifier name '" .. vcc .. "_[A-Za-z0-9._]+' is too long") then
		return _T("Identifier name is too long")
	end

	if string.match(strError, " command denied to user '.*'@'.*' for ") then
		local replace = string.gsub(strError,"'.*'@","'*****'@")
		return _T("Access denied." .. replace)
	end

	if string.match(strError, " command denied to user '.*'@'.*' for ") then
		local replace = string.gsub(strError,"'.*'@","'*****'@")
		return _T("Access denied." .. replace)
	end

	if string.match(strError, "PROCEDURE [A-Za-z0-9._]+ already exists") then
		return _T("Procedure already exists.")
	end

	if string.match(strError, "Table '[A-Za-z0-9._]+' already exists") then
		return _T("Table already exists.")
	end

	if string.match(strError, "Unknown storage engine '[A-Za-z0-9._]+'") then
		return string.gsub(strError,"Unknown storage engine", _T("Unknown storage engine"))
	end

	return _T(strError)
end

local function ExecuteQuery(connection, query)
	local result, err = relational_db.Query(connection, query)
	if not result then
		local line =  nil
		local reason = nil
		if err and err[1] and err[1].message then
			line = string.match(err[1].message, "[0-9]+$")
			reason = string.match(err[1].message, "[^\]]*$")
		end
		return false, {line = line, reason = formatErrorReason(reason, line)}
	end
	return result
end

local function GetSizingTables(connection, dbName)
	local sql_query_sizing = "SELECT ROUND(SUM(data_length + index_length) / 1024 / 1024, 2) AS sizing FROM information_schema.TABLES where table_schema = '".. dbName:lower() .."';"
	local result_sizing = relational_db.Query(connection, sql_query_sizing)

	if not result_sizing then
		return false
	end
	local sizing =  json.encode(result_sizing.rows and result_sizing.rows[1] and result_sizing.rows[1].sizing or 0)
	--LogInfo("[ADMINISTRATION.DATABASES:GetSizingTables] sizing %s",sizing)
	return true, sizing == "null" and 0 or sizing
end

local function CountTablesAndStoredProcedures(connection, dbName)
	local sql_query_tables = "SELECT COUNT(*) as count from Information_Schema.Tables where TABLE_TYPE = 'BASE TABLE' and table_schema = '".. (dbName):lower() .."';"
	local result_tables = relational_db.Query(connection, sql_query_tables)
	local sql_query_stored_proc = "SELECT COUNT(routine_definition) as count from Information_Schema.ROUTINES where routine_schema = '".. (dbName):lower() .."';"
	local result_stored_proc = relational_db.Query(connection, sql_query_stored_proc)
	
	local num_tables, num_sp
	if not result_tables or not result_stored_proc then
		return false
	end
	num_tables = result_tables.rows and result_tables.rows[1] and result_tables.rows[1].count or 0
	num_sp = result_stored_proc.rows and result_stored_proc.rows[1] and result_stored_proc.rows[1].count or 0
	--LogInfo("[ADMINISTRATION.DATABASES:CountTablesAndStoredProcedures] num_tables %s num_sp %s", num_tables, num_sp)
	return true,  num_tables, num_sp
end

--Returns all databases in the VCC
local function Search(web)
	local vcc = inconcert.authUser.virtualContactCenter
	local fullUserId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	--LogInfo("[ADMINISTRATION.DATABASES:SEARCH] 1 - (START) Search rows for user '%s', group '%s'. Posted data: '%s'", fullUserId, inconcert.authUser.userGroup, web.input["post_data"])
	local keyspace = utils.GetDatasource():CassandraConnection( "Agent" )
	local postData = web.input["post_data"]
	--Get all databases from model
	local rows = m_models.Databases.GetDatabases(nil, keyspace, vcc)
	local totalUnfilteredRows = utils.GetRowCount(rows)
	local totalAdded = 0
	local filteredRows = {}

	local connectionData = ConnectionData()
	if not connectionData then
		return ReturnError(web, nil, {result = "ERROR_CONNECTION"})
	end

	for databaseId, database in pairs(rows) do

		if utils.MatchFilter(database, postData) then
			totalAdded = totalAdded + 1
			local dbId = vcc .. "_" .. databaseId
			local success, num_tables, num_sp = CountTablesAndStoredProcedures(connectionData, dbId)
			local success_sizing, sizing = GetSizingTables(connectionData, dbId)
			if success or success_sizing then
				database["Tables"] = num_tables
				database["Procedures"] = num_sp
				database["Sizing"] = sizing
			else
				local err = "ERROR_DATABASE"
				web.status = 500
				return err
			end
			table.insert(filteredRows, database)
		end
	end

	--LogInfo("[ADMINISTRATION.DATABASES:SEARCH] 3 - Sorting rows")

	--include only rows that belong to logged user vcc
	local response = {
		rows = filteredRows,
		last = "",
		total_rows = totalAdded,
		total_unfiltered_rows = totalUnfilteredRows
	}
	local responseParams = {
		MakeSortingValue = function(row)
			assert(row)
			return tostring(row.Name or row.Id):upper()
		end
	}
	local resultData = utils.MakeSearchResponse(response, postData, responseParams)

	--LogInfo("[ADMINISTRATION.DATABASES:SEARCH] 4 - (END) Search rows. Retrieve %d of %d matched rows. Total Unfiltered Rows: %d", #resultData.rows, resultData.total_rows, resultData.total_unfiltered_rows)
	keyspace:Close()
	relational_db.Disconnect(connectionData)
	return resultData
end

local function GetTablesInfo(connectionData, dbNameMysql, tableName)
	local sql_query_tables = "select TABLE_NAME, COLUMN_NAME, COLUMN_DEFAULT, IS_NULLABLE, DATA_TYPE, COLUMN_TYPE, COLUMN_KEY, EXTRA from information_schema.columns where table_schema = '".. dbNameMysql .."'"..(tableName and (" AND TABLE_NAME='".. tableName .."'") or "").. " order by ORDINAL_POSITION;"
	return relational_db.Query(connectionData, sql_query_tables)
end

local function GetSPInfo(connectionData, dbNameMysql, spName)
	local sql_query_sp = 'SELECT R.ROUTINE_NAME as name, R.ROUTINE_DEFINITION as body, (case when P.SPECIFIC_NAME != R.ROUTINE_NAME then "EMPTY" else P.parameter_name end) as param_name, (case when P.SPECIFIC_NAME != R.ROUTINE_NAME then "EMPTY" else P.parameter_mode end) as param_mode, (case when P.SPECIFIC_NAME != R.ROUTINE_NAME then "EMPTY" else P.DTD_IDENTIFIER end) as param_identifier FROM INFORMATION_SCHEMA.ROUTINES R LEFT JOIN INFORMATION_SCHEMA.parameters P on P.specific_schema = R.routine_schema WHERE R.ROUTINE_SCHEMA = "'.. dbNameMysql 
	..'" AND R.ROUTINE_TYPE = "PROCEDURE"' .. (spName and (' and R.ROUTINE_NAME="'.. spName ..'"') or "")..' order by P.ordinal_position;'
	return relational_db.Query(connectionData, sql_query_sp)
end

local function getSPInfoForAudit(connectionData, dbNameMysql, spName)

	local result_query_sp, err = GetSPInfo(connectionData, dbNameMysql, spName)
	if err then
		return result_query_sp, err
	end
	
	local stored_procedures = {}
	for _, value in ipairs(result_query_sp.rows or {}) do
		local field = {
			Name = value.name,
			Body = value.body or "",
		}
		local param_list = {}
		if value.param_name and type(value.param_name) =="string" and value.param_name ~= "EMPTY" then
			local param_mode = value.param_mode and type(value.param_mode) =="string" and value.param_mode ~= "EMPTY" and value.param_mode or ""
			local param_identifier = value.param_identifier and type(value.param_identifier) =="string" and value.param_identifier ~= "EMPTY" and value.param_identifier or ""
			param_list = {
				Name = value.param_name,
				Mode = param_mode,
				Identifier = param_identifier
			}
		end

		if not stored_procedures[value.name] then
			stored_procedures[value.name] = field
		end
		if not utils.TableIsEmpty(param_list) then
			if not stored_procedures[value.name].Params then
				stored_procedures[value.name].Params = {}
			end
			table.insert(stored_procedures[value.name].Params, param_list)
		end
	end
	return stored_procedures
end

local function Get(web, databaseId)
	local vcc = inconcert.authUser.virtualContactCenter
	local keyspace = utils.GetDatasource():CassandraConnection( "Agent" )
	local dbNameMysql = vcc .. "_" .. databaseId:lower()
	local database = m_models.Databases.GetDatabases(dbNameMysql, keyspace)
	--LogInfo("[ADMINISTRATION.DATABASES:Get] database %s: %s", databaseId, json.encode(database))
	keyspace:Close()
	
	--1. Valido si existe db
	if not next(database) then
		return ReturnError(web, 404, {result = "NOT_FOUND"})
	end

	--2. Cargo los datos de conexion y conecto
	local connectionData = ConnectionData(dbNameMysql)
	if not connectionData then
		return ReturnError(web, nil, {result = "CONNECTION_ERROR"})
	end

	--3. 
	local result_query_tables = GetTablesInfo(connectionData, dbNameMysql)
	if not result_query_tables then
		return ReturnError(web, nil, {result = "ERROR"}, connectionData)
	end

	local sql_query_sp = 'SELECT R.ROUTINE_NAME as name, R.ROUTINE_DEFINITION as body, (case when P.SPECIFIC_NAME != R.ROUTINE_NAME then "EMPTY" else P.parameter_name end) as param_name, (case when P.SPECIFIC_NAME != R.ROUTINE_NAME then "EMPTY" else P.parameter_mode end) as param_mode, (case when P.SPECIFIC_NAME != R.ROUTINE_NAME then "EMPTY" else P.DTD_IDENTIFIER end) as param_identifier FROM INFORMATION_SCHEMA.ROUTINES R LEFT JOIN INFORMATION_SCHEMA.parameters P on P.specific_schema = R.routine_schema WHERE R.ROUTINE_SCHEMA = "'.. dbNameMysql ..'" AND R.ROUTINE_TYPE = "PROCEDURE" order by P.ordinal_position;'
	local result_query_sp, err = relational_db.Query(connectionData, sql_query_sp)
	if err then
		return ReturnError(web, nil, {result = "ERROR"}, connectionData)
	end

	--[[
		Cambio el formato a:
		tables = {
			TABLE_NAME = {
				COLUMN_NAME = {
						Id = uuid.new()
						not_null = true o false,
						type = "",
						primary_key = "",
						lenght = ""
				}
			}
		}
	]]--
	local tables = {}
	for _, value in ipairs(result_query_tables.rows or {}) do
		
		--No agregar field timestamp (para Depuracion)
		if value.COLUMN_NAME ~= "ic_timestamp" then
			local id =  uuid.new()
			local field = {
				type = value.DATA_TYPE,
				not_null = value.IS_NULLABLE == "NO",
				primary_key = value.COLUMN_KEY == "PRI",
				length = string.match(value.COLUMN_TYPE, "[0-9,]+"),
				name = value.COLUMN_NAME
			}

			if not tables[value.TABLE_NAME] then
				tables[value.TABLE_NAME] = {}
			end
			tables[value.TABLE_NAME][id] = field
		end
	end

	database.tables = tables

	local stored_procedures = {}
	for _, value in ipairs(result_query_sp.rows or {}) do
		local field = {
			Name = value.name,
			Body = value.body or ""
		}
		if value.param_name and type(value.param_name) =="string" and value.param_name ~= "EMPTY" then
			local param_mode = value.param_mode and type(value.param_mode) =="string" and value.param_mode ~= "EMPTY" and value.param_mode .. " " or ""
			local param_identifier = value.param_identifier and type(value.param_identifier) =="string" and value.param_identifier ~= "EMPTY" and " " .. value.param_identifier or ""
			field.Param_list = param_mode .. value.param_name .. param_identifier
		else
			field.Param_list = ""
		end

		if stored_procedures[value.name] and stored_procedures[value.name].Param_list ~= "" then
			field.Param_list = stored_procedures[value.name].Param_list ..", " .. field.Param_list
		end
		
		stored_procedures[value.name] = field
	end

	database.stored_procedures = stored_procedures

	local success, sizing = GetSizingTables(connectionData, dbNameMysql)
	if not success then
		return ReturnError(web, nil, {result = "ERROR"}, connectionData)
	end
	database.Sizing = sizing
	
	return database
end

local function getTableInfoForAudit(connectionData, dbName, tableName )

	local query_Table, error_table = GetTablesInfo(connectionData,dbName, tableName)
	if not error_table then
		local columnTables = {}
		for _, value in ipairs(query_Table.rows or {}) do
			if value.COLUMN_NAME ~= "ic_timestamp" then
				local field = {
					type = value.DATA_TYPE,
					not_null = value.IS_NULLABLE == "NO",
					primary_key = value.COLUMN_KEY == "PRI",
					length = string.match(value.COLUMN_TYPE, "[0-9,]+"),
				}
				columnTables[value.COLUMN_NAME] = field
			end
		end

		local data = not utils.TableIsEmpty(columnTables) and {
			Name = tableName,
			Columns = columnTables
		} or nil
		return true, data
	else
		return nil, error_table
	end
end

local function UpdateTable(web)
	
	local vcc = inconcert.authUser.virtualContactCenter
	local postData = json.decode(web.input["post_data"])
    --LogInfo("[ADMINISTRATION.DATABASES.UpdateDatabase], postData %s", web.input["post_data"])

	if utils.TableIsEmpty(postData) or not postData.DataBase or not postData.TableName or not postData.TableFields then
		return ReturnError(web, 404, {result="NOT_FOUND"})
	end

	local dbNameMysql = vcc .. "_" .. postData.DataBase:lower()
	local connectionData = ConnectionData(dbNameMysql)

	if not connectionData then
		return ReturnError(web, nil, {result = "ERROR_CONNECTION", error = {reason = "Failed to establish connection to one or more of the database nodes."}})
	end

	local tb_old_success, tb_old_resp
	if not postData.isMigration then
		tb_old_success, tb_old_resp = getTableInfoForAudit(connectionData, dbNameMysql, postData.TableName)
	end
	
	--Update, remove or add table fields
	local formatFieldsByAction = function (tableFields)
		--Retorna la los campos organizados por action, para actualizar o agregar multiple columnas
		local fieldByAction = {}
		for _, value in pairs(tableFields) do
			if value.action == "DELETE" then
				if not fieldByAction.delete then
					fieldByAction.delete = {}
				end
				table.insert(fieldByAction.delete, value.name)
			elseif value.action == "ADD" then
				if not fieldByAction.add then
					fieldByAction.add = {}
				end
				value.action = nil
				fieldByAction.add[value.name] = value
			else
				--Update
				if not fieldByAction.update then
					fieldByAction.update = {}
				end
				value.action = nil
				fieldByAction.update[value.name] = value
			end 
		end
		return fieldByAction
	end

	local fieldsByAction = formatFieldsByAction(postData.TableFields)
	--LogInfo("[ADMINISTRATION.DATABASES.UpdateDatabase], fieldsByAction %s", json.encode(fieldsByAction))

	--Transaction Begin
	connectionData.cnn:setautocommit(false)
	
	--Elimino los fields
	local  DeleteTableFields = function(fields)
		for _, key in ipairs(fields) do
			local query = "ALTER TABLE " .. postData.TableName .." DROP COLUMN " .. key ..";"
			local success, err = ExecuteQuery(connectionData, query)
			if not success then
				return success, err
			end
		end
		return true
	end
	
	local resultDelete, errDel = DeleteTableFields(fieldsByAction.delete or {})
	if not resultDelete then
		connectionData.cnn:rollback()
		connectionData.cnn:setautocommit(true)
		return ReturnError(web, nil, {result = "ERROR", error = errDel}, connectionData)
	end

	--Se actualizan los fields existentes
	local UpdTableFields = function(fields)
		local middleQuery = ""
		
		for _, value in pairs(fields) do
			middleQuery = middleQuery .. string.format(" MODIFY %s", value.name)
			
			if value.type and value.type ~= "" then
				middleQuery = middleQuery .. string.format(" %s", value.type)
			end
			
			if value.length and value.length ~= "" then
				middleQuery = middleQuery .. "(" .. value.length ..")"
			end

			if value.not_null then
				middleQuery = middleQuery .. string.format(" %s", "NOT NULL")
			end

			if value.primary_key then
				middleQuery = middleQuery ..", ADD primary key (" .. value.name .. ")"
			end

			middleQuery = middleQuery .. ","
		end
		local query = string.format("ALTER TABLE %s%s;", postData.TableName, string.sub(middleQuery, 1, #middleQuery - 1))
		
		local success, err = ExecuteQuery(connectionData, query)
		if not success then
			return success, err
		end
		return true
	end
	
	if fieldsByAction.update then
		local resultUpd, errUpd = UpdTableFields(fieldsByAction.update)
		if not resultUpd then
			connectionData.cnn:rollback()
			connectionData.cnn:setautocommit(true)
			return ReturnError(web, nil, {result = "ERROR", error = errUpd}, connectionData)
		end
	end

	--Se agregan los fields
	local AddTableFields = function(fields)
		local middleQuery = ""
		
		--multiples primary keys
		local pkeys_str = ""
		for _, value in pairs(fields) do
			middleQuery = middleQuery .. string.format(" %s%s", not postData.Create and "ADD COLUMN " or "", value.name)

			if value.type and value.type ~= "" then
				middleQuery = middleQuery .. string.format(" %s", value.type)
			end

			if value.length and value.length ~= "" then
				middleQuery = middleQuery .. string.format("(%s)", value.length)
			end

			if value.not_null then
				middleQuery = middleQuery .. string.format(" %s", "NOT NULL")
			end
			--Guardo las key en un string.
			if value.primary_key then
				pkeys_str = pkeys_str .. value.name ..","
			end

			middleQuery = middleQuery .. ","
		end
		
		middleQuery = string.sub(middleQuery, 1, #middleQuery - 1)
		local query = ""
		if postData.Create then

			-- Agrego timestamp para Depuracion
			middleQuery = middleQuery .. ", ic_timestamp TIMESTAMP DEFAULT CURRENT_TIMESTAMP"

			--Agrego las key
			if pkeys_str ~= "" then
				pkeys_str = string.sub(pkeys_str, 1, #pkeys_str - 1)
				pkeys_str = ", primary key (" .. pkeys_str .. ")"
			end
			middleQuery = middleQuery .. pkeys_str
			query = string.format("CREATE TABLE %s(%s) ENGINE=NDBCLUSTER;", postData.TableName, middleQuery)
		else
			--Agrego las key
			if pkeys_str ~= "" then
				pkeys_str = string.sub(pkeys_str, 1, #pkeys_str - 1)
				pkeys_str = ", ADD primary key (" .. pkeys_str .. ")"
			end
			middleQuery = middleQuery .. pkeys_str
			query = string.format("ALTER TABLE %s%s;", postData.TableName, middleQuery)
		end
		
		local success, err = ExecuteQuery(connectionData, query)
		if not success then
			return success, err
		end
		return true
	end
	
	if fieldsByAction.add then
		local resultAdd, errAdd = AddTableFields(fieldsByAction.add)
		if not resultAdd then
			connectionData.cnn:rollback()
			connectionData.cnn:setautocommit(true)
			return ReturnError(web, nil, {result = "ERROR", error = errAdd}, connectionData)
		end
	end

	if not postData.isMigration then
		local tb_curr_success, tb_curr_resp = getTableInfoForAudit(connectionData, dbNameMysql, postData.TableName)
		if tb_old_success and tb_curr_success then
			--Agrego auditoria de Eliminar Tabla
			local ksHistorical = utils.GetDatasource():CassandraConnection("Historical")
			local audit_params = {
				vcc = vcc,
				entity = "Databases",
				user = inconcert.authUser.userName,
				fieldKey = postData.TableName,
				subEntity = "Tables",
				oldFields = tb_old_resp,
				newFields = tb_curr_resp,
				connection = ksHistorical

			}
			audits.Create(audit_params)
			ksHistorical:Close()
		end
	end
	
	connectionData.cnn:setautocommit(true)
	relational_db.Disconnect(connectionData)
	return {result = "OK"}
end

local function CreateUserPermissions(params)
	local connection = params.connectionData
	local dbName	 = params.database
	local userId	 = params.userId
	local password = connection.datasource.Password

	local sql_create_user = string.format("CREATE USER '%s'@'%s' IDENTIFIED WITH mysql_native_password BY '%s';", userId,"%", password)
	local sql_grant_permissions = string.format("GRANT ALL PRIVILEGES ON %s.* TO '%s'@'%s';", dbName, userId, "%")
	local result, error = ExecuteQuery(connection, sql_create_user)
	if error then
		return result, error
	end
	result, error = ExecuteQuery(connection, sql_grant_permissions)
	if error then
		return result, error
	end
	result, error = ExecuteQuery(connection, "FLUSH PRIVILEGES;")
	if error then
		return result, error
	end
	return result
end

local function DeleteUserPermissions(params)
	local connection = params.connectionData
	local dbName	 = params.database
	local userId	 = params.userId

	local sql_drop_user = string.format("DROP USER '%s'@'%s';", userId, "%")
	local sql_revoke_permissions = string.format("REVOKE ALL PRIVILEGES ON %s.* FROM '%s'@'%s';", dbName, userId, "%")
	local result, error = ExecuteQuery(connection, sql_revoke_permissions)
	if error then
		return result, error
	end
	result, error = ExecuteQuery(connection, sql_drop_user)
	if error then
		return result, error
	end
	return result
end

local function CreateDatabase(web)
	local vcc = inconcert.authUser.virtualContactCenter
	local keyspace = utils.GetDatasource():CassandraConnection( "Agent" )

	local postData = json.decode(web.input["post_data"])
   -- LogInfo("[ADMINISTRATION.DATABASES.CreateDatabase], postData %s", web.input["post_data"])

	local connectionData = ConnectionData()
	if not connectionData then
		return ReturnError(web, nil, {result = "ERROR_CONNECTION", error = {reason = "Failed to establish connection to one or more of the database nodes."}})
	end
	local dbName = vcc.. "_" .. postData.Name:lower()
	local exits_db = m_models.Databases.GetDatabases(dbName, keyspace)
	--LogInfo("[ADMINISTRATION.DATABASES.CreateDatabase], exits_db %s", json.encode(exits_db))
	if next(exits_db) then
		return ReturnError(web, 200, {result = "DATABASE_ALREADY_EXIST"})
	end	


	--Se crea usuario con privilegios sobre la db
	local userId = utils.Uuid(true)
	local _, error_user = CreateUserPermissions({
		userId = userId,
		database = dbName,
		connectionData = connectionData
	})

	if error_user then
		return ReturnError(web, nil, {result = "ERROR_CREATE_DATABASE", error = error_user}, connectionData)
	end

	--Execute comando create dbN
	local query = "CREATE DATABASE IF NOT EXISTS " .. dbName .. ";"
	local _, error = ExecuteQuery(connectionData, query)

	if error then
		--. Se elimina el usuario y privilegios de administracion de la basedato
		DeleteUserPermissions({
			userId = userId,
			database = dbName,
			connectionData = connectionData
		})
		return ReturnError(web, nil, {result = "ERROR_CREATE_DATABASE", error = error}, connectionData)
	end

	local dbData = {
		Id = postData.Name:lower(),
		Name = postData.Name,
		Description = postData.Description,
		UserId		= userId,
		CreationDate = LuaOrm.Date():toString("%Y-%m-%d %H:%M:%S")
	}

	local databases = {
		[dbName] = dbData
	}
	m_models.Databases.SaveDatabase(databases, keyspace)

	if not postData.isMigration then
		local keyspaceAudit = utils.GetDatasource():CassandraConnection("Historical")
		local audit_params = {
			vcc = vcc,
			entity = "Databases",
			user = inconcert.authUser.userName,
			fieldKey = postData.Name:lower(),
			newFields = dbData,
			connection = keyspaceAudit
		}
		audits.Create(audit_params)
		keyspaceAudit:Close()
	end
	
	keyspace:Close()
	relational_db.Disconnect(connectionData)
	return {result = "OK"}
end

local function DeleteDatabase(web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc
	local postData = json.decode(web.input["post_data"])
	--LogInfo("[ADMINISTRATION.DATABASES.RemoveDatabase], user %s, vcc %s, database: '%s'", userId, vcc, json.encode(postData))

	--1. Se cargan los datos de conexion y se conecta a la db
	local connectionData = ConnectionData()
	if not connectionData then
		return ReturnError(web, nil, {result = "ERROR_CONNECTION"})
	end

	--2. Se ejecuta query para eliminar la db
	local keyspace = utils.GetDatasource():CassandraConnection( "Agent" )
	local dbName = vcc.. "_" .. postData[1]:lower()
	local exits_db = m_models.Databases.GetDatabases(dbName)
	LogInfo("[ADMINISTRATION.DATABASES.CreateDatabase], exits_db %s", json.encode(exits_db))
	if not next(exits_db) then
		return ReturnError(web, 200, {result = "DATABASE_NOT_EXISTS"})
	end	

	local query = "DROP DATABASE " .. dbName .. ";"
	local result_query = relational_db.Query(connectionData, query)

	if not result_query then
		return ReturnError(web, nil, {result = "ERROR_DROP_DATABASE"})
	end
	--3. Se elimina el usuario y privilegios de administracion de la basedato
	DeleteUserPermissions({
		userId = exits_db.UserId,
		database = dbName,
		connectionData = connectionData
	})

	--4. Se elimina la info de la base en cassandra
	local database = {
		dbName
	}
	m_models.Databases.DeleteDatabase(database, keyspace)
	local keyspaceAudit = utils.GetDatasource():CassandraConnection( "Historical" )
	local audit_params = {
		vcc = vcc,
		entity = "Databases",
		user = inconcert.authUser.userName,
		fieldKey = exits_db.Id,
		oldFields = exits_db,
		connection = keyspaceAudit
	}
	audits.Create(audit_params)

	keyspace:Close()
	keyspaceAudit:Close()
	relational_db.Disconnect(connectionData)
	return {result = "OK"}
end

--Eliminar tabla o Stored Procedures
local function DeleteStoredProcedure  (connectionData, Name, web)
	
	local sql_query_del = "DROP PROCEDURE IF EXISTS " .. Name .. ";"
	--1. Se ejecuta query
	local _, err = ExecuteQuery(connectionData, sql_query_del)
	if err then
		return ReturnError(web, nil, {result = "ERROR", error = err}, connectionData)
	end

	return {result = "OK"}
end

local function DeleteTableSP(web)
	local postData = web.request
	local vcc = inconcert.authUser.virtualContactCenter
	--LogInfo("[[ADMINISTRATION.DATABASES.DeleteTableSP] postData %s", json.encode(postData))
	if not postData.Name or not postData.Database or not postData.Type then
		return ReturnError(web, 404, {result="NOT_FOUND"})
	end
	local dbName = vcc.. "_" .. postData.Database:lower()
	--1 Si es Stored procedure
	if  postData.Type == "SP" then
		-- test all hosts conexions
		local testConnect = ConnectionData(dbName, true)
		if not testConnect then
			return ReturnError(web, nil, {result = "ERROR_CONNECTION", error = {reason=formatErrorReason("Failed to establish connection to one or more of the database nodes.")}})
		end
		
		local connectGetSp = ConnectionData(dbName)
		if not connectGetSp then
			return ReturnError(web, nil, {result = "ERROR_CONNECTION", error = {reason=formatErrorReason("Failed to establish connection to one or more of the database nodes.")}})
		end
		local sp, err = getSPInfoForAudit(connectGetSp,dbName,postData.Name)
		LogInfo("[[ADMINISTRATION.DATABASES.DeleteTableSP] spData %s", json.encode(sp))
		relational_db.Disconnect(connectGetSp)
		if err or utils.TableIsEmpty(sp) then
			return ReturnError(web, nil, {result = "ERROR", error = {reason=formatErrorReason("Procedure does not exist")}})
		end

		local returnCallBack =  ConnectionData(dbName, true, function(connectionData)
			return DeleteStoredProcedure(connectionData, postData.Name, web)
		end)

		if not returnCallBack or (returnCallBack.result and returnCallBack.result ~="OK" ) then
			return returnCallBack
		end
		
		if not postData.isMigration then
			--Agrego auditoria de Eliminar SP
			local keyspaceAudit = utils.GetDatasource():CassandraConnection("Historical")
			if sp[postData.Name] then
				local audit_params = {
					vcc = vcc,
					entity = "Databases",
					user = inconcert.authUser.userName,
					fieldKey = postData.Name,
					subEntity = "Procedures",
					oldFields = sp[postData.Name],
					connection = keyspaceAudit
				}
				audits.Create(audit_params)
			end
			keyspaceAudit:Close()
		end
		
		return returnCallBack
	end

	--1. Se cargan los datos de conexion y se conecta a la db
	
	local connectionData = ConnectionData(dbName)
	if not connectionData then
		return ReturnError(web, nil, {result = "ERROR_CONNECTION", reason=formatErrorReason("Failed to establish connection to one or more of the database nodes.")})
	end

	--Busco la cantidad actual de valores en la tabla
	--Solo permitir Eliminar si es menor al limite permitido (RowsNumber)
	local sql_query_count = "select count(*) as count from ".. postData.Name .. " limit " .. tostring(tonumber( postData.RowsNumber) + 1) ..";"
	local result_count, err = ExecuteQuery(connectionData, sql_query_count)
	if err then
		return ReturnError(web, nil, {result = "ERROR", error = {reason = formatErrorReason("Error getting total values from table")}})
	end

	local count = result_count.rows and result_count.rows[1] and result_count.rows[1].count or 0
	if tonumber(count) > tonumber(postData.RowsNumber) then
		return ReturnError(web, nil, {result = "ERROR", error = {reason = formatErrorReason("Deleting is not allowed when the number of values is greater than the configured limit.")}})
	end

	local tb_success, tb_resp
	if not postData.isMigration then
		tb_success, tb_resp = getTableInfoForAudit(connectionData, dbName, postData.Name)
	end
	
	
	local sql_query_del = "DROP TABLE IF EXISTS " .. postData.Name .. ";"

	--2. Se ejecuta query
	local _, err = relational_db.Query(connectionData, sql_query_del)
	if err then
		return ReturnError(web, nil, {result = "ERROR"}, connectionData)
	end

	if tb_success and not postData.isMigration then
		--Agrego auditoria de Eliminar Tabla
		local keyspaceAudit = utils.GetDatasource():CassandraConnection( "Historical" )
		local audit_params = {
			vcc = vcc,
			entity = "Databases",
			user = inconcert.authUser.userName,
			fieldKey = postData.Name,
			subEntity = "Tables",
			oldFields = tb_resp,
			connection = keyspaceAudit
		}
		audits.Create(audit_params)
		keyspaceAudit:Close()
	end

	
	relational_db.Disconnect(connectionData)
	return {result = "OK"}
end

local function isQueryCreateTable(query)
	local patt = "^%s*(CREATE%s*TABLE)%s*[A-Za-z0-9_]+%s*[(]"
	return string.match(query:upper(), patt)
end

local function isCreateStoredProcedure(query)
	local patt = "^%s*(CREATE%s*PROCEDURE)%s*[A-Za-z0-9_]+%s*[(]"
	return string.match(query:upper(), patt)
end

local function getLastCreateTable(connectionData, dbNameMysql)

	if not connectionData or not dbNameMysql then
		return ""
	end
	local sql_query_tables = "select TABLE_NAME from information_schema.tables where table_schema = '".. dbNameMysql .."' order by CREATE_TIME desc limit 1;"
	local result, error = relational_db.Query(connectionData, sql_query_tables)
	if not error and result.rows and result.rows[1] then
		return result.rows[1].TABLE_NAME
	end
	return ""
end

local function getLastCreateSP(connectionData, dbNameMysql)
	if not connectionData or not dbNameMysql then
		return ""
	end
	local sql_query_sp = 'SELECT ROUTINE_NAME as Name FROM INFORMATION_SCHEMA.ROUTINES WHERE ROUTINE_SCHEMA = "'.. dbNameMysql ..'" AND ROUTINE_TYPE = "PROCEDURE" order by CREATED desc limit 1;'
	local result, error = relational_db.Query(connectionData, sql_query_sp)
	if not error and result.rows and result.rows[1] then
		return result.rows[1].Name
	end
	return ""
end

local function ProcessStoredProcedure(web, Options, query, connectionData)

	--Datos que vienen del lado del cliente
	local old_query_procedure
	if Options and Options.Data then
		old_query_procedure = "Create procedure " .. Options.Data.Name .."(" .. Options.Data.Param_list ..")" .. Options.Data.Body
	
		-- Drop del Procedure
		local drop_query = "Drop procedure if exists ".. Options.Data.Name ..";"
		local _, err = ExecuteQuery(connectionData, drop_query)
		if err then
			return ReturnError(web, nil, {result = "ERROR_DROP_SP", error = {reason = formatErrorReason("Error when trying to delete the procedure to edit")}}, connectionData)
		end
	end

	--Ejecuto la query del create
	local _, err = ExecuteQuery(connectionData, query)
	if err then
	--SI da error al crear y es edicion de sp, ejecuto query del create del antiguo SP
		if old_query_procedure then
			local _, errOldPS = ExecuteQuery(connectionData, old_query_procedure)
			if errOldPS then
				return ReturnError(web, nil, {result = "ERROR_CREATING_SP_DROPPED_PREV", error = err}, connectionData)
			end
		end
		return ReturnError(web, nil, {result = "ERROR_CREATING_SP", error = err}, connectionData)
	end
	return {result = "OK"}
end

local function isQuerySelect(query)
	local patt = "SELECT.*FROM.*;%s*$";
	local replace = string.gsub(query,"\n","")
	local result = replace:upper():match(patt)
	LogInfo("[[ADMINISTRATION.DATABASES.isQuerySelect] result %s", result)
	return result and true or nil
end

local function CheckNodeConnections (web)
	LogInfo("[[ADMINISTRATION.DATABASES.CheckNodeConnections]")
	local testConnect = ConnectionData(nil, true)
	if not testConnect then
		return ReturnError(web, nil, {result = "ERROR_CONNECTION", error = {reason=formatErrorReason("Failed to establish connection to one or more of the database nodes.")}})
	end
	web.status = 200
	return {result="OK"}
end

local function ExecuteCommand(web)
	local postData = web.request
	local vcc = inconcert.authUser.virtualContactCenter
	LogInfo("[[ADMINISTRATION.DATABASES.ExecuteCommand] postData %s", json.encode(postData))
	if not postData.Query or not postData.Database then
		return ReturnError(web, 404, {result="NOT_FOUND"})
	end

	local keyspaceAudit 
	if not postData.isMigration then
		keyspaceAudit = utils.GetDatasource():CassandraConnection( "Historical" )
	end
	
	--Fn para agregar auditoria de Ejecucion
	local function CreateExecAuditReport(query)
		if not isQuerySelect(query) then
			local audit_params = {
				vcc = vcc,
				entity = "Databases",
				user = inconcert.authUser.userName,
				fieldKey = postData.Database:lower(),
				subEntity = "QueryExecution",
				operation = "Execute",
				newFields = {
					Query = string.gsub(query,"\n+"," ")
				},
				connection = keyspaceAudit
			}
			audits.Create(audit_params)
		end
	end

	local dbName = vcc.. "_" .. postData.Database:lower()
	local isCreateTable = isQueryCreateTable(postData.Query)
	LogInfo("[[ADMINISTRATION.DATABASES.ExecuteCommand] postData.Query %s", json.encode(postData.Query))
	--Si es create table debo agregar columna ic_timestamp
	if isCreateTable then
		local query = string.sub(postData.Query, 1, #postData.Query - 2) -- Elimino la secuencia );
		query = query .. ", ic_timestamp TIMESTAMP DEFAULT CURRENT_TIMESTAMP) ENGINE=NDBCLUSTER;" -- agrego la columna timestamp
		postData.Query = query
	elseif isCreateStoredProcedure(postData.Query) then
		 --Si es SP, chequeo todas las conexiones primero, y ejecuto query para cada una.
			local testAllConnection = ConnectionData(dbName, true --[[Check all connections]])
			if testAllConnection then
				local isEditSP = postData.Options and  postData.Options.Action and  postData.Options.Action == "Edit"
				and postData.Options.Data or nil
				local connectGetSp = ConnectionData(dbName)
				local old_sp_err, old_sp_resp
				local spName = ""
				local options
				--Si es edit SP, busco los datos de auditorias que ya estan y formateados a la estructura de audit
				if isEditSP then
					spName = postData.Options.Data.Name
					options = postData.Options
					if connectGetSp and not postData.isMigration then
						old_sp_resp, old_sp_err =  getSPInfoForAudit(connectGetSp, dbName, spName)
					end
				end
				local returnProc = ConnectionData(dbName, true, function (connectionData)
					return ProcessStoredProcedure(web, options, postData.Query, connectionData )
				end)
				if not returnProc or (returnProc.result and returnProc.result ~="OK" ) then
					return returnProc
				end

				if not postData.isMigration then
					--Agrego auditoria Para SP
					local newSpName = getLastCreateSP(connectGetSp, dbName)
					if spName ~= "" or newSpName ~= "" then
						local last_sp_resp, last_sp_error = getSPInfoForAudit(connectGetSp, dbName, newSpName)
						if not last_sp_error then
							local audit_params = {
								vcc = vcc,
								entity = "Databases",
								user = inconcert.authUser.userName,
								fieldKey = isEditSP and spName or newSpName,
								subEntity = "Procedures",
								oldFields = not old_sp_err and old_sp_resp or nil,
								newFields = last_sp_resp,
								connection = keyspaceAudit
							}
							audits.Create(audit_params)
						end
					end
				end
				relational_db.Disconnect(connectGetSp)
				return returnProc
			else
				return ReturnError(web, nil, {result = "ERROR_CONNECTION", error = {reason = formatErrorReason("Failed to establish connection to one or more of the database nodes.")}})
			end
	end
	
	local connectionData = ConnectionData(dbName)
	if not connectionData then
		if not postData.isMigration then
			keyspaceAudit:Close()
		end
		return ReturnError(web, nil, {result = "ERROR_CONNECTION", error = {reason = formatErrorReason("Failed to establish connection to one or more of the database nodes.")}})
	end

	--1. Se ejecuta query
	local sql_result, err = ExecuteQuery(connectionData, postData.Query)
	if err then
		if not postData.isMigration then
			keyspaceAudit:Close()
		end
		return ReturnError(web, nil, {result = "ERROR", error = err or {}}, connectionData)
	end

	--2. Se formatea resultado para mostrar en grilla.
	local format_column = {}
	for index, value in ipairs(sql_result.columns or {}) do
		value.type = ""
		value.id = value.name
		if value.name ~= "ic_timestamp" then
			table.insert(format_column, value)
		end
	end
	sql_result.columns = format_column

	--Agrego auditoria de Ejecucion
	if not postData.isMigration then
		if isCreateTable then
			local tableName = getLastCreateTable(connectionData, dbName)
			if tableName ~="" then
			   local last_tb_success, last_tb_resp = getTableInfoForAudit(connectionData, dbName, tableName)
			   if last_tb_success then
				   local audit_params = {
					   vcc = vcc,
					   entity = "Databases",
					   user = inconcert.authUser.userName,
					   fieldKey = tableName,
					   subEntity = "Tables",
					   newFields = last_tb_resp,
					   connection = keyspaceAudit
				   }
				   audits.Create(audit_params)	
			   end
			end
		else
			CreateExecAuditReport(postData.Query)
		end

		keyspaceAudit:Close()
	end

	relational_db.Disconnect(connectionData)
	return {result="OK", data = sql_result}
end

--Export to csv
local function ExportTableValues(wsapi_env)
	local request = (wsapi.request.new(wsapi_env)).params["data"]
	local query = json.decode(request)
	local vcc = inconcert.authUser.virtualContactCenter
	--LogInfo("[ADMINISTRATION.DATABASES.ExportTableValues] request %s", request)
	local TableName = query.TableName
	local Database = query.Database
	local Limit = query.Limit

	local function ReturnErr (err, connectionData)
		-- Error
		wsapi_env.status = 500
		local response = wsapi.response.new(200, {["Content-Type"]= "text/plain"})
		response:write(json.encode(err))
		response.headers["Response"] = json.encode(err)
		response.status = err.httpCode or 500
		if connectionData then
			relational_db.Disconnect(connectionData)
		end
		return response:finish()
	end

	if not TableName or not Database or not Limit then
		return ReturnErr({
			result = "NOT_FOUND",
			httpCode = 404,
			reason = formatErrorReason("Table or database name not found")
		})
	end

	local dbName = vcc.. "_" .. Database:lower()
	local connectionData = ConnectionData(dbName)
	if not connectionData then
		return ReturnErr({result = "ERROR_CONNECTION", reason=formatErrorReason("Failed to establish connection to one or more of the database nodes.")})
	end

	--1. Se ejecuta query
	local query = "Select * from ".. TableName .. " limit " .. Limit ..";"
	local sql_result, err = ExecuteQuery(connectionData, query)
	if err then
		return ReturnErr({result = "ERROR", error = err or nil}, connectionData)
	end
	--LogInfo("[ADMINISTRATION.DATABASES.ExportTableValues] sql_result %s", json.encode(sql_result))
	local infoToExport = {}

	if sql_result.rows and #sql_result.rows > 0 then
		infoToExport = sql_result.rows
	else
		wsapi_env.status = 404
		local response = wsapi.response.new(200, {["Content-Type"]= "text/plain"})
		response:write(json.encode({result = "ERROR_NOT_FOUND_ITEMS", reason=formatErrorReason("No results found.")}))
		response.headers["Response"] = json.encode({result = "ERROR_NOT_FOUND_ITEMS", reason=formatErrorReason("No results found.")})
		response.status = 404
		return response:finish()
	end

	--Valido los valores NULL y los cambio

	for _, row in ipairs(infoToExport) do
		for key, value in pairs(row) do

			--Valores tipo null
			if type(value) == "function" then
				row[key] = "NULL"
			end
		end
	end
	
	local csv = utils.Table2Csv(infoToExport, nil, {ic_timestamp = true}, ",")

	local updateDate = LuaOrm.Date():toString("%Y-%m-%d %H:%M:%S")
	local suggestedFileName = string.format("%s(%s).CSV", "table_" ..TableName .."_values_".. vcc, updateDate)
	local headers = {}
	headers["Response"]            = "defaultError"
	headers["Content-Type"]        = "application/octet-stream"
	headers["Content-Length"]      = #csv
	headers["Response"]            = "ok"
	headers["Content-Disposition"] = [[attachment; filename="]] .. suggestedFileName .. '"'
	headers["Set-Cookie"]          = "download_started=true; Path=/" -- used by the jquery download plugin

	--LogInfo("[ADMINISTRATION.DATABASES.ExportTableValues] CSV File Name '%s'.", suggestedFileName)
	--Crear registro de auditoria
	local keyspaceAudit = utils.GetDatasource():CassandraConnection( "Historical" )
	local audit_params = {
		vcc = vcc,
		entity = "Databases",
		user = inconcert.authUser.userName,
		fieldKey = TableName,
		subEntity = "Tables",
		operation = "Export",
		newFields = {
			Action = "Export",
			CreationDate = updateDate,
			TotalValues = #infoToExport
		},
		connection = keyspaceAudit
	}
	audits.Create(audit_params)
	keyspaceAudit:Close()
	relational_db.Disconnect(connectionData)
	local coroutineIter = 0
	return 200, headers, coroutine.wrap(function()
		coroutineIter = coroutineIter + 1
		coroutine.yield(csv)
	end)

end

--Import from csv
local function ImportTableValues(web)

	local postData = web.request
	local vcc = inconcert.authUser.virtualContactCenter
	--LogInfo("[[ADMINISTRATION.DATABASES.ImportTableValues] postData %s", json.encode(postData))
	local TableName = postData.TableName
	local Database = postData.Database
	local Override = postData.Override
	local TableValues = postData.tableValues and postData.tableValues.values
	local TableColumns = postData.tableValues and postData.tableValues.columns
	local TableLimit = postData.tableValues and postData.tableValues.rowsLimit

	if not TableName or not Database or not TableValues or not TableColumns or not TableLimit then
		return ReturnError(web, 404, {result = "NOT_FOUND"})
	end

	local dbName = vcc.. "_" .. Database:lower()
	local connectionData = ConnectionData(dbName)
	if not connectionData then
		return ReturnError(web, nil, {result = "CONNECTION_ERROR",error = {reason=formatErrorReason("Failed to establish connection to one or more of the database nodes.")}})
	end

	local sql_query_count = "select count(*) as count from ".. postData.TableName .. " limit " .. TableLimit..";"
	local resultQuery, err = ExecuteQuery(connectionData, sql_query_count)
	if err then
		return ReturnError(web, nil, {result = "ERROR", error = {reason = "Error getting total values from table"}})
	end

	local count = resultQuery.rows and resultQuery.rows[1] and resultQuery.rows[1].count or 0
	local isExceeds = (tonumber(count) + #TableValues ) >= tonumber(TableLimit)
	--LogInfo("[[ADMINISTRATION.DATABASES.ImportTableValues] count %s isExceeds %s", count, isExceeds)
	if (not Override or tostring(Override) == "false" ) and isExceeds then
			return ReturnError(web, nil, {result = "ERROR", error = {reason = formatErrorReason("The sum of the values to insert and the values in the table exceeds the allowed limit.")}})
	end

	--Transaction Begin
	connectionData.cnn:setautocommit(false)

	if Override then
		local query_insert = "DELETE FROM ".. TableName .. ";"
		local sql_result, err = ExecuteQuery(connectionData, query_insert)
		if err then
			connectionData.cnn:setautocommit(true)
			return ReturnError(web, nil, {result = "ERROR", error = {reason = "Error removing rows, before insert."}}, connectionData)
		end	
	end

	local columns= ""	
	local allValues = ""
	local endColumn
	for _, val in ipairs(TableValues) do
		local intValues = ""

		for _, col in ipairs(TableColumns) do
			if not endColumn then
				columns = columns .. col .. ","
			end

			local value = val[col:lower()]
			if not value or value == "NULL" then
				value = value == "NULL" and "NULL" or ""
			else
				value = '"' .. value .. '"'
			end
			intValues = intValues .. value ..","
		end

		if intValues ~= "" then
			intValues = "(" ..  string.sub(intValues, 1, #intValues - 1) .. ")"
			allValues = allValues .. intValues .. ","
		end
		endColumn = true
	end
	columns = string.sub(columns, 1, #columns - 1)
	allValues = string.sub(allValues, 1, #allValues - 1) 

	local query_insert = "INSERT INTO ".. TableName .."(".. columns ..")  VALUES" .. allValues .. ";"
	local sql_result, err = ExecuteQuery(connectionData, query_insert)
	if err then
		connectionData.cnn:rollback()
		connectionData.cnn:setautocommit(true)
		return ReturnError(web, nil, {result = "ERROR", error = err or {}}, connectionData)
	end

	--Transaction End
	connectionData.cnn:setautocommit(true)

	--Crear registro de auditoria
	local ksHistorical = utils.GetDatasource():CassandraConnection("Historical")
	local audit_params = {
		vcc = vcc,
		entity = "Databases",
		user = inconcert.authUser.userName,
		fieldKey = TableName,
		subEntity = "Tables",
		operation = "Import",
		newFields = {
			Action = "Import",
			Override = Override or "false",
			CreationDate = LuaOrm.Date():toString("%Y-%m-%d %H:%M:%S"),
			TotalValues = #TableValues
		},
		connection = ksHistorical

	}
	audits.Create(audit_params)
	ksHistorical:Close()

	relational_db.Disconnect(connectionData)
	return {result="OK"}
end
local function getConnectionData (web)
	return utils.JSON_Encode(ConnectionData());
end 

local function Render (web)
	_M.response.DontCache(web)
	local env = { _T = m_htmlRender.extension.makeTextTranslator() }
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc
	local html_file = m_appLocation .. "/pages/frm_databases_content.html"
	local content = m_htmlRender:RenderPageLP(html_file, web, env)
	if not content then
		LogError("[AUDIT:Render] HTML Databases does not exist: %s", html_file)
	end
	return content
end

function MapUris(module, htmlRender, prefix, appLocation)
	local LogRequest = require("apps.agent.requestFilters").LogRequest

	module:dispatch_post(utils.JsonValidatorReply..LogRequest..Search, prefix.."/search/")
	module:dispatch_get(utils.JsonReply..LogRequest.. Get, prefix.."/get/(.+)")
	module:dispatch_get(utils.JsonReply..LogRequest.. CheckNodeConnections, prefix.."/check_node_connections/")
	module:dispatch_post(utils.JsonValidatorReply..LogRequest..permissions:CheckPermission("administration", "savedatabase", UpdateTable), prefix.."/update/")
	module:dispatch_post(utils.JsonValidatorReply..LogRequest..permissions:CheckPermission("administration", "deletedatabase", DeleteDatabase), prefix.."/delete/")
	module:dispatch_post(utils.JsonValidatorReply..LogRequest..permissions:CheckPermission("administration", "savedatabase",
			utils.WebJsonValidatorData(
				m_validations.basic,
				CreateDatabase
			)
		),
		prefix.."/create/"
	)
	module:dispatch_post(utils.JsonValidatorReply..LogRequest..permissions:CheckPermission("administration", "deletedatabase", DeleteTableSP), prefix.."/delete_table_sp/")
	module:dispatch_post(utils.JsonValidatorReply..LogRequest..permissions:CheckPermission("administration", "savedatabase", ExecuteCommand), prefix.."/console/")
	module:dispatch_post(utils.JsonValidatorReply..LogRequest..permissions:CheckPermission("administration", "savedatabase", ImportTableValues), prefix .. "/table_import/")
	module:dispatch_get(utils.JsonReply..LogRequest..permissions:CheckPermission("administration", "savedatabase", getConnectionData), prefix .. "/connection_data/")

	module:dispatch_wsapi(ExportTableValues, prefix .. "/table_export/")
	-- frm audit render
	module:dispatch_get ( Render, prefix .. "/frm/render/")

	m_htmlRender = htmlRender
	m_appLocation = appLocation
end
