local m_models = require("apps.administration.model"):GetModels()

module(..., package.seeall, inconcert.orbit.application.Decorate)

local m_moduleLocation = select(2, ...)
local utils = require "shared.apps.utils"
local agentservice = require "apps.agent.agentservice"
local admUserController = require("apps.administration.controllers.users")
local permissions = require "shared.permissions.permissions"
local model_perm = require "shared.permissions.models.permissions"
local admCampaignController = require("apps.administration.controllers.campaigns")
local notificationsEngine = require("apps.agent.notifications")
local agentNotifications = require("apps.agent.agentNotifications")
local audits = require "audits"
local wsapi = require "wsapi"
wsapi.request = require "wsapi.request"
require "LuaOrm.Date"
local utf8 = require "icu.utf8"

m_htmlRender = nil
m_appLocation = nil

function MapUris (module, htmlRender, prefix, appLocation)

	local LogRequest = require("apps.agent.requestFilters").LogRequest

	module:dispatch_post(utils.JsonValidatorReply..LogRequest.. Search, prefix.."/search/")
	module:dispatch_get(utils.JsonReply..LogRequest..permissions:CheckPermission("administration", "viewgroups", Get), prefix.."/get/(.+)")
	module:dispatch_get(utils.JsonReply..LogRequest..permissions:CheckPermission("administration", "viewgroups", GetAllSystemGroups), prefix.."/get_system_groups/")
	module:dispatch_get(utils.JsonReply..LogRequest..GetAllGroups, prefix.."/get_all_groups/(.+)")
	module:dispatch_wsapi(ExportGroupsInfoCsv, prefix.."/export/")

	module:dispatch_post(utils.JsonValidatorReply..LogRequest..permissions:CheckPermission("administration", "savegroup", SaveGroups), prefix.."/save/")
	module:dispatch_post(utils.JsonValidatorReply..LogRequest..permissions:CheckPermission("administration", "deletegroup", RemoveGroups), prefix.."/delete/")
	module:dispatch_post(utils.JsonValidatorReply..LogRequest..permissions:CheckPermission("administration", "savegroup", RestoreDefault), prefix .. "/restore_default/")
	module:dispatch_post(utils.JsonValidatorReply..LogRequest..permissions:CheckPermission("administration", "savegroup", ValidateOverrides), prefix .. "/validate_override/")

	module:dispatch_get(Render, prefix .. "/frm/render/")

	m_htmlRender = htmlRender
	m_appLocation = appLocation
end

function Render ( web )
	_M.response.DontCache(web)
	local env = { _T = m_htmlRender.extension.makeTextTranslator() }
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc

	LogInfo("[ADMINISTRATION.GROUPS:Render] Render form Roles for user %s, group %s", userId, inconcert.authUser.userGroup)

	local html_file = m_appLocation .. "/pages/frm_role_content.html"
	local content = m_htmlRender:RenderPageLP(html_file, web, env)
	if not content then
		LogError("[ADMINISTRATION.GROUPS:Render] HTML Roles does not exist: %s", html_file)
	end
	return content
end

--Get Audit data
local function GetGroupAuditData(keyspace)
	
	local vcc = inconcert.authUser.virtualContactCenter
	local all_groups = {}
	local has_close_keyspace = false
	if not keyspace then
		keyspace = utils.GetDatasource():CassandraConnection( "Agent" )
		has_close_keyspace = true
	end
	local system_groups = m_models.Groups.GetGroups("system", nil, keyspace) -- system groups
	local vcc_groups = m_models.Groups.GetGroups(vcc, nil, keyspace) -- vcc groups
	
	if not system_groups and not vcc_groups then
		if has_close_keyspace then
			keyspace:Close()
		end
		return nil
	end

	for groupId, group in pairs(system_groups or {}) do
		local full_id = groupId .. "@system"
		local group_info = {
			Id = full_id,
			Name = group.Name,
			Description = group.Description,
		}
		group_info.permissions = model_perm.GetUserGroupPermissions(groupId, keyspace, vcc) or {}
		all_groups[ full_id ] = group_info
	end

	for groupId, group in pairs(vcc_groups or {}) do
		local full_id = groupId .. "@" .. vcc
			local group_info = {
				Id = full_id,
				Name = group.Name,
				Description = group.Description,
			}
			group_info.permissions = model_perm.GetUserGroupPermissions(groupId, keyspace, vcc) or {}
			all_groups[ full_id ] = group_info
	end

	if has_close_keyspace then
		keyspace:Close()
	end
	
	return all_groups
end

--Returns all groups in the VCC
function Search ( web )
	local vcc = inconcert.authUser.virtualContactCenter
	local fullUserId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local startTime = os.time()

	LogInfo("[ADMINISTRATION.GROUPS:SEARCH] 1 - (START) Search rows for user '%s', group '%s'. Posted data: '%s'", fullUserId, inconcert.authUser.userGroup, web.input["post_data"])

	local keyspace = utils.GetDatasource():CassandraConnection( "Agent" )
	local has_allegro = utils.HasAllegroIntegration( vcc, keyspace )

	--Get system groups from model
	-- Si es un VCC de integracion con allegro, no permite realizar override de los roles del sistema.
	local system_groups = has_allegro and {} or m_models.Groups.GetGroups( "system", nil, keyspace )
	--Get vcc groups from model
	local vcc_groups = m_models.Groups.GetGroups( vcc, nil, keyspace )

	local totalUnfilteredRows = utils.GetRowCount(system_groups) + utils.GetRowCount(vcc_groups)
	local totalAdded = 0
	local filteredRows = {}
	local postData = web.request

	LogInfo("[ADMINISTRATION.GROUPS:SEARCH] 2 - Filtering rows")

	local function filter ( groups, is_custom)
		for group_id, group in pairs( groups ) do
			group_id = utils.GetId( group_id )
			group.Id = group_id
			group.Name = group.Name or group_id
			if utils.MatchFilter( group, postData ) then
				if tonumber(group.IsFullSystemAccess or "0") == 0 then
					totalAdded = totalAdded + 1
					local is_modified = model_perm.IsModifiedSystemGroup( group_id, keyspace ).modified
					group.IsModified = is_modified and "1" or "0"
					table.insert(filteredRows, group)
				else
					totalUnfilteredRows = totalUnfilteredRows - 1
				end
			end
		end
	end

	filter(system_groups)
	filter(vcc_groups, true)
	keyspace:Close()

	LogInfo("[ADMINISTRATION.GROUPS:SEARCH] 3 - Sorting rows")
	--include only rows that belong to logged user vcc
	local response = {
		rows = filteredRows,
		last = "",
		total_rows = totalAdded,
		total_unfiltered_rows = totalUnfilteredRows
	}
	local responseParams = {
		MakeSortingValue = function(row)
			assert(row)
			return tostring(row.Name or row.Id):upper()
		end
	}
	local resultData = utils.MakeSearchResponse(response, postData, responseParams)

	local endTime = os.time()
	local elapsedTime = endTime - startTime
	LogInfo("[ADMINISTRATION.GROUPS:SEARCH] 4 - (END) Search rows. Retrieve %d of %d matched rows. Total Unfiltered Rows: %d. ELAPSED Time '%s' seconds.", #resultData.rows, resultData.total_rows, resultData.total_unfiltered_rows, elapsedTime)

	return resultData
end

local function SetGroupPermissions (group, group_id, is_system, keyspace, applicationsActions)
	local group_permissions

	local function set()
		group.grouppermissions = group_permissions or utils.EMPTY_OBJECT
		group.categorized_permissions = model_perm.Categorize(group.grouppermissions, keyspace, applicationsActions)
		if group.IsSystemGroup == "1" then
			local response = model_perm.IsModifiedSystemGroup( group_id, keyspace )
			group.default_system_group_permissions = response.default_system_group_permissions
			group.IsModified = response.modified and "1" or "0"
		end
	end

	local vcc
	if is_system then
		vcc = "system"
		local response = model_perm.IsModifiedSystemGroup( group_id, keyspace )
		if response.modified then
			group_permissions = response.group_permissions
			set()
			return
		end
	else
		vcc = inconcert.authUser.virtualContactCenter
	end

	group_permissions = model_perm.GetGroupPermissions( group_id .. "@" .. vcc, keyspace )
	set()
end
 
function Get ( web, groupId )
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc
	LogInfo("[ADMINISTRATION.GROUPS:Get] Retrieving groupId %s, user %s, vcc %s", groupId, userId, vcc)

	local keyspace = utils.GetDatasource():CassandraConnection("Agent")
	local applicationsActions = permissions.GetApplicationsActions(keyspace, vcc)
	local groups

	for _, domain in pairs( { vcc, "system" } ) do
		groups = m_models.Groups.GetGroups(domain, groupId, keyspace)
		if next(groups) then
			-- full access system role validation
			if tonumber(groups.IsFullSystemAccess or "0") == 1 then
				keyspace:Close()
				web.status = 403
				local reason = string.format( "Role Name %s - Unable to retrive System Role.", groups.Name )
				return { result = "UNABLE_TO_RETRIEVE_SYSTEM_ROLE_PERMISSIONS", reason = reason }
			end
			--
			groups.Name = groups.Name or groupId
			SetGroupPermissions(groups, groupId, domain == "system", keyspace, applicationsActions)
			break
		end
		groups = false
	end

	keyspace:Close()
	if groups then
		-- OK
		LogInfo("[ADMINISTRATION.GROUPS:Get] Success Retrieved groupId %s, user %s, vcc %s.", groupId, userId, vcc)
		return groups
	end
	-- ERROR
	LogError("[ADMINISTRATION.GROUPS:Get] Error Retrieving groupId %s, user %s, vcc %s. Role NOT FOUND.", groupId, userId, vcc)
	web.status = 404
	return {result = "NOT_FOUND"}
end

--[[
	Retorna los roles/permisos del Sistema del VCC, menos el "full-access-system"
]]--
function GetAllSystemGroups (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local _T = m_htmlRender.extension.makeTextTranslator()
	local keyspace = utils.GetDatasource():CassandraConnection( "Agent" )
	local groups = m_models.Groups.GetGroups("system", nil, keyspace)
	local applicationsActions = permissions.GetApplicationsActions(keyspace, vcc)
	--[[
	]]--
	local function CanInclude (group)
		if tonumber(group.IsFullSystemAccess or "0") == 1 then return false end
		return true
	end
	local groupsPermissions = {}
	for groupId, group in pairs(groups) do
		-- full-access-system role validation
		if CanInclude(group) then
			group.IdLabel = _T(group.Id)
			group.NameLabel = _T(group.Name)
			SetGroupPermissions(group, groupId, true, keyspace, applicationsActions)
			groupsPermissions[groupId] = group
		end
	end
	keyspace:Close()
	return groupsPermissions
end

function GetAllGroups (web, includeAllUsers)
	local vcc = inconcert.authUser.virtualContactCenter
	local all_groups = {}
	local _T = m_htmlRender.extension.makeTextTranslator()
	local keyspace = utils.GetDatasource():CassandraConnection( "Agent" )
	local system_groups = m_models.Groups.GetGroups("system", nil, keyspace) -- system groups
	local applicationsActions = permissions.GetApplicationsActions(keyspace, vcc)

	-- system groups
	for groupId, group in pairs(system_groups) do
		SetGroupPermissions(group, groupId, true, keyspace, applicationsActions)
		group.IdLabel = _T(group.Id)
		group.NameLabel = _T(group.Name)
		all_groups[ groupId ] = group
	end

	-- vcc groups
	local vcc_groups = m_models.Groups.GetGroups(vcc, nil, keyspace)
	for groupId, group in pairs(vcc_groups) do
		SetGroupPermissions(group, groupId, false, keyspace, applicationsActions)
		group.IdLabel = _T(group.Id)
		group.NameLabel = _T(group.Name)
		all_groups[ groupId ] = group
	end

	if includeAllUsers == "1" then
		--Agregamos un grupo ficticio allusers, para que si no hay sincronizacion los usuarios se busquen por el filtro asociado a este grupo
		all_groups["allusers"] = {
			Description = "All Users",
			Id = "allusers",
			IdLabel = _T("allusers"),
			IsDeployer = "0",
			IsFullSystemAccess = "0",
			IsSystemGroup = "1",
			Name = "allusers",
			NameLabel = _T("allusers")
		}
	end

	keyspace:Close()
	return all_groups
end

function ExportGroupsInfoCsv(wsapi_env)
	local vcc = inconcert.authUser.virtualContactCenter
	local groupsInfo = GetGroupsExportInfo(wsapi_env)
	_T = m_htmlRender.extension.makeTextTranslator()
	local function ReturnErr (err)
		-- Error
		wsapi_env.status = 500
		local response = wsapi.response.new(200, {["Content-Type"]= "text/plain"})
		response:write(json.encode(err))
		response.headers["Response"] = "defaultError"
		response.status = err.httpCode or 500
		return response:finish()
	end

	if groupsInfo.result then
		return ReturnErr(groupsInfo.result)
	end

	local orderHeaders = {
		_T("Name"),
		_T("Description"),
		_T("Creation Date"),
		_T("Permissions")
	}

	local groupsInfoTranslate = {}
	for _, value in pairs(groupsInfo) do
		local userData = {
			[_T("Name")] = value.Name,
			[_T("Description")] = value.Description,
			[_T("Creation Date")] = value.creationDate,
			[_T("Permissions")] = value.permissions ~= "" and string.gsub(value.permissions, '"', '""' ) or ""
		}
	
		table.insert(groupsInfoTranslate, userData)
	end

	local csv = utils.Table2Csv(groupsInfoTranslate, nil, nil, "|", orderHeaders)
	csv = string.gsub('"'..csv..'"', "|", '","' )
	local LN = "\n"
	csv = string.gsub(csv, LN, '"'..LN..'"')

	local updateDate = LuaOrm.Date():toString("%Y-%m-%d %H:%M:%S")
	local suggestedFileName = string.format("%s(%s).CSV", "groups_export_info_".. vcc, updateDate)
	local headers = {}
	headers["Response"]            = "defaultError"
	headers["Content-Type"]        = "application/octet-stream"
	headers["Content-Length"]      = #csv
	headers["Response"]            = "ok"
	headers["Content-Disposition"] = [[attachment; filename="]] .. suggestedFileName .. '"'
	headers["Set-Cookie"]          = "download_started=true; Path=/" -- used by the jquery download plugin

	LogInfo("[Users:ExportGroupsInfoCsv] ExportGroupsInfoCsv. CSV File Name '%s'.", suggestedFileName)

	local coroutineIter = 0
	return 200, headers, coroutine.wrap(function()
		coroutineIter = coroutineIter + 1
		coroutine.yield(csv)
	end)

end

function GetGroupsExportInfo (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local all_groups = {}
	local keyspace = utils.GetDatasource():CassandraConnection( "Agent" )
	local system_groups = m_models.Groups.GetGroups("system", nil, keyspace) -- system groups
	local vcc_groups = m_models.Groups.GetGroups(vcc, nil, keyspace) -- vcc groups
	
	if not next(system_groups) and not next(vcc_groups) then
		keyspace:Close()
		web.status = 404
		return {result = "NOT_FOUND"}
	end

	for groupId, group in pairs(system_groups) do
		local group_info = {
			Name = group.Name,
			creationDate = group.creationDate,
			Description = group.Description,
		}
		group_info.permissions = json.encode(model_perm.GetUserGroupPermissions(groupId, keyspace, vcc)) or ""
		all_groups[ groupId ] = group_info
	end

	for groupId, group in pairs(vcc_groups) do

		if not all_groups[groupId] then
			local group_info = {
				Name = group.Name,
				creationDate = group.creationDate,
				Description = group.Description,
			}
			group_info.permissions = json.encode(model_perm.GetUserGroupPermissions(groupId, keyspace, vcc)) or ""
			all_groups[ groupId ] = group_info
		end
	end

	keyspace:Close()
	return all_groups
end

local function ChangeAvailableUsersStates (fullGroupId, group, keyspace, applicationsActions)
	local users, count = m_models.Users.GetUsersByRole( fullGroupId, keyspace )
	LogInfo("[GROUPS:ChangeAvailableUsersStates] groupId: %s, users: %s", fullGroupId, utils.GetRowCount(users))
	if count == 0 then
	-- si no tiene usuarios que pertenzcan a este rol, me voy
		LogInfo("[ADMINISTRATION.GROUPS:ChangeAvailableUsersStates] Change Available users states. The role has no users assigned.")
		return
	end
	-- si existen usuarios que pertenezcan a este rol, mando cambiar su estatus.
	local vcc = inconcert.authUser.virtualContactCenter
	for fullUserId in pairs(users) do
		local user = m_models.Users.GetUser(vcc, utils.GetId(fullUserId), keyspace)
		if next(user) then
			user.grouppermissions = group.grouppermissions
			admUserController.ChangeUserStateToOnLine(user, keyspace, applicationsActions)
		else
			LogError("[ADMINISTRATION.GROUPS:ChangeAvailableUsersStates] User %s does not exist.", fullUserId)
		end
	end
end

--[[
	Add/Modifies groups in a vcc
	posted data must have the form:	
{
	groupId1 = {
		Name = "group1",
		Description = "group description 1"
	},
	groupId2 = {
		Name = "group2",
		Description = "group description 2"
	},
	...
}

]]

-- Esta funcion valida que existan los permisos "savegroup" de asignar permisos (frm permission),
--  y la accion "saveuser" de override en el frm de usuarios.
--
local function IsEnabledPermissionToAssign (groupId, newGroupPermissions, keyspace, applicationsActions)
	local vcc = inconcert.authUser.virtualContactCenter
	local vccUsers = m_models.Users.GetVCCUsers( vcc, keyspace )
	local user_columns = { "UserGroup", "Enabled" }
	local has_permission = false

	LogInfo("[ADMINISTRATION.GROUPS:IsEnabledPermissionToAssign] Validating Permissions to actions Save group 'savegroup' and Save Users 'saveuser' ")

	local validate = function( appId, actionId )
		local has_permission_to_action = false
		for userId, user in pairs(vccUsers) do
			if user and ( user.Status == nil or tonumber(user.Status) == 1 ) then
				local groupPermissions
				if groupId == user.UserGroup then
				-- esto se realiza porque los permisos del grupo del usuario han cambiado, y debe validar contra los nuevos
				-- que llegan en el POST
					groupPermissions = newGroupPermissions
				end
				has_permission_to_action = model_perm.IsPermissionEnabledForSpecificUser(
					appId              , -- [[ application             --]
					actionId           , -- [[ action                  --]
					userId             ,
					user.UserGroup     ,
					nil                , -- [[ userVccPermissions      --]
					groupPermissions   , -- [[groupPermissions         --]
					nil                , -- [[userCampaignsPermissions --]
					keyspace           ,
					applicationsActions
				)

				if has_permission_to_action then
					break
				end
			end
		end
		return has_permission_to_action
	end

	has_permission = validate( "administration", "savegroup" )
	if has_permission then
		has_permission = validate( "administration", "saveuser" )
		if not has_permission then
			LogWarning("[ADMINISTRATION.GROUPS:IsEnabledPermissionToAssign] Not has permission to Save Users action 'saveuser'.")
		end
	else
		LogWarning("[ADMINISTRATION.GROUPS:IsEnabledPermissionToAssign] Not has permission to Save Groups action 'savegroup'.")
	end

	return has_permission
end

local function HasGroupPermission (groupPermissions)
	if type(groupPermissions) ~= "table" then
		return false
	end
	return next(groupPermissions) ~= nil
end

-- Valida si existen usuarios que pertenezcan al rol que se encuentren en ALS.
-- si se encuentran en ALs debe de validar si el permisos modificado tiene acceso a la aplicacion "interaction"
-- si no tiene acceso, falla
local function InConflictUsersByGroupWithAttentionLevels (users, group_id, newGroupPermissions, keyspace, applicationsActions)
	local result = {
		success = true
	}
	LogInfo("[ADMINISTRATION.GROUPS:InConflictUsersByGroupWithAttentionLevels] Attention levels validation. Validate if exist user by group '%s' in conflict. Without permission to INTERACTION application.", group_id)

	-- recorro la lista de users
	local user_with_als = {}
	for full_user_id in pairs( users ) do
		local als_by_user = m_models.Campaigns.GetAttentionLevelsByUser( full_user_id, keyspace )
		local has_als = not utils.TableIsEmpty(als_by_user)
		--LogDebug(" full_user_id: '%s', has_als: '%s'", full_user_id, has_als)
		if has_als then
			user_with_als[full_user_id] = als_by_user
		end
	end

	-- recorro la lista de users que tengan attention levels
	for full_user_id, als_by_user in pairs( user_with_als ) do
		-- aca realizo la validacion si el usuario tiene permisos para la app "INTERACTION"
		local hasPermissionToAppInteraction = model_perm.HasPermissionToApplication("interaction", full_user_id, group_id, nil, newGroupPermissions, nil, keyspace, nil, nil, nil, applicationsActions)
		--LogDebug("  -- full_user_id: '%s', group_id: '%s', perm: '%s'.", full_user_id, group_id, hasPermissionToAppInteraction)
		if not hasPermissionToAppInteraction then
			result.success = false
			result.count_users = (result.count_users or 0) + 1
			result.role_id = group_id
			result.problematic_users = result.problematic_users or {}
			result.problematic_users[ full_user_id ] = als_by_user
		end
	end

	return result
end

-- Valida si existen usuarios que pertenezcan al rol, que tengan interacciones tomadas.
-- Si tiene interacciones tomadas valida si el permisos modificado tiene acceso a la aplicacion "interaction"
--   y si no tiene acceso, falla
local function InConflictUsersByGroupWithAssignedInteractions (users, group_id, newGroupPermissions, keyspace, applicationsActions)
	local result = {
		success = true
	}
	LogInfo("[ADMINISTRATION.GROUPS:InConflictUsersByGroupWithAssignedInteractions] Assigned Interactions validation. Validate if exist user by group '%s' in conflict. Without permission to INTERACTION application.", group_id)

	-- recorro la lista de users
	for full_user_id in pairs( users ) do
		local simple_user_id = utils.GetId(full_user_id)
		local assigned_interactions_count = admUserController.CountAssignedInteractions(simple_user_id, nil, keyspace)
		--LogDebug("group_id: %s, simple_user_id: %s, assigned_interactions_count: %d", group_id, simple_user_id, assigned_interactions_count)
		if assigned_interactions_count > 0 then
			local hasPermissionToAppInteraction = model_perm.HasPermissionToApplication("interaction", full_user_id, group_id, nil, newGroupPermissions, nil, keyspace, nil, nil, nil, applicationsActions)
			--LogDebug("  -- full_user_id: '%s', group_id: '%s', perm: '%s'.", full_user_id, group_id, hasPermissionToAppInteraction)
			if not hasPermissionToAppInteraction then
				result.success = false
				result.count_users = (result.count_users or 0) + 1
				result.role_id = group_id
				result.problematic_users = result.problematic_users or {}
				result.problematic_users[ full_user_id ] = assigned_interactions_count
			end
		end
	end

	return result
end

local function IsSystemGroup (groupId, existingSystemGroups, keyspace)
	if not existingSystemGroups then
		existingSystemGroups = m_models.Groups.GetGroups( "system", nil, keyspace )
	end
	if not existingSystemGroups then
		return false
	end
	if existingSystemGroups[ groupId ] then
		return true
	end
	return false
end

--[[
  Esta funcion se encarga de eliminar los overrides a nivel Campana, especificamente los de los Members en la campana ( override Nietos ), que se hayan recibido en el request (accion previa del usuario, decidiendo que quiere removerlos)
  Los overrides Nietos son los overrides de Members de la campana, y son llamados Nietos ya que son override de override.
  Es decir tenemos los roles de permisos (abuelo), el override de roles dentro de la campana (padre), y el override de members dentro de la campana (nieto)
--]]
local function CleanUpCampaignMembersOverrides (members_overrides, keyspace, ksHistorical)
	if not members_overrides then
		return
	end

	local vcc = inconcert.authUser.virtualContactCenter

	LogInfo("[ADMINISTRATION.GROUPS:CleanUpCampaignMembersOverrides] 3 - CleanUp Campaign Members Overrides %s ", json.encode( members_overrides ) )

	local applicationsActions = permissions.GetApplicationsActions(keyspace, vcc)

	for group_id, info in pairs( members_overrides ) do
		-- si del lado del cliente se ha decido remover los override nietos!!!
		if info.type == "REMOVE" then
			local overrides = json.decode( info.members_overrides )
			-- recorro las campanas que tienen override en los members
			for campaign_id, members in pairs( overrides ) do
				-- obtengo los overrides de permisos de la campana de todos sus members ( de la BD ) 

				local members_permissions
				local encoded_permissions = m_models.Campaigns.GetSpecificColumn( campaign_id, "permissions", keyspace )
				--LogDebug("encoded_permissions: %s", encoded_permissions)
				if type(encoded_permissions) == "string" then
					local permissions = json.decode(encoded_permissions)
					members_permissions = permissions.members
				end

				local changed = false
				if members_permissions then
				-- recorro los members de la campana ( loos que me pasaron en el request )
					for member_id, apps in pairs( members ) do
						-- si existe el member del request "members" en el que se obtuvo de la BD "members_permissions"
						--	 entro y procedo a recorrer la lista de apps que me llegaron en el request
						if members_permissions[ member_id ] then
						-- recorro las acciones del request
							for app_id, actions in pairs( apps ) do
							-- si la app coincide con la que que se obtuvo de la BD, procedo a recorrer las acciones.
								if members_permissions[ member_id ].actions[ app_id ] then
								-- ahora procedo a recorrer las acciones del request, y elimino el override del member de la lista de la BD
									for action_id, new_state in pairs( actions ) do
										members_permissions[ member_id ].actions[ app_id ][ action_id ] = nil
										changed = true
									end
								end
							end
						end
					end

					if changed then
					-- si exsitieron cambios, tengo que validar que no queden apps sin acciones de override, y grupos de apps sin ninguna app con acciones de overide.
					-- si se da el caso antes mencionado, tengo que eliminar tambien de los override la app y/o la lista de apps
						for member_id, apps in pairs( members_permissions ) do
							local count_apps = 0
							local count_removed_apps = 0
							-- cuento la cantidad de aplicaciones y acciones que existen con override, la cantidad de apps removidas
							for app_id, apps_actions in pairs( apps.actions ) do
								count_apps = count_apps + 1
								local count_actions_in_app = 0
								for action_id, actions in pairs( apps_actions ) do
									count_actions_in_app = count_actions_in_app + 1
								end
								if count_actions_in_app == 0 then
								-- si la app no tiene acciones, la elimino de los overrides
									members_permissions[ member_id ].actions[ app_id ] = nil
									count_removed_apps = count_removed_apps + 1
								end
							end
							if count_apps == count_removed_apps then
							-- si no existe ninngua app con override tengo que eliminar la lista de contenedor de acciones por app
								members_permissions[ member_id ].actions = nil
								members_permissions[ member_id ] = nil
							end
						end
						-- aca guardo los nuevos overrides de members para la campana especifica
						LogInfo("[ADMINISTRATION.GROUPS:CleanUpCampaignMembersOverrides] CleanUp Campaign Members Overrides - New overrides for campaign '%s' = '%s'.", campaign_id, json.encode( members_permission ) )
						admCampaignController.SaveAllPermissionsOnCampaign(
							campaign_id,
							nil, -- role permissions
							members_permissions,
							nil, -- new_members
							keyspace,
							applicationsActions,
							ksHistorical
						)
					end
				end
			end
		end
	end
end

--[[
  Esta funcion se encarga de eliminar los overrides a nivel Campana, especificamente los Roles en la campana ( override Hijos )
--]]
local function CleanUpCampaignRoleOverrides (groups, keyspace, ksHistorical)
	local vcc = inconcert.authUser.virtualContactCenter

	LogInfo("[ADMINISTRATION.GROUPS:CleanUpCampaignRoleOverrides] 2 - CleanUp Campaigns Roles Overrides")

	local applicationsActions = permissions.GetApplicationsActions(keyspace, vcc)

	-- recorro la lista de grupos que se van a guardar y valido si tienen override ( siempre es un solo grupo, pero se realizo el request, para que pueda recibir varios grupos en un mismo request )
	for _, group in pairs( groups ) do
		local roles = {}
		local is_system_group = IsSystemGroup( group.Id, nil, keyspace )
		local domain = ( is_system_group and "system" or vcc )
		local full_group_id = utf8.lower( group.Id .. "@" .. domain )
		local group_permissions = group.grouppermissions

		-- si el grupo tiene permisos asignados, procedo a realizar a la validacion de overrides
		if HasGroupPermission( group_permissions ) then

			local campaigns = m_models.Campaigns.GetVCCCampaigns( vcc, keyspace )
			-- recorro la lista de campanas del vcc
			for campaign_id in pairs( campaigns ) do

				local overrides
				local encoded_permissions = m_models.Campaigns.GetSpecificColumn( campaign_id, "permissions", keyspace )
				--LogDebug("encoded_permissions: %s", encoded_permissions)
				if type(encoded_permissions) == "string" then
					local permissions = json.decode(encoded_permissions)
					overrides = permissions
				end

				-- si la campana tiene override grupos definidos, continuo
				if overrides and overrides.groups then
					local changed = false

					-- recorro los overrides de roles de la campana, buscando el role que se esta procesando
					for full_role_id, group_overrides in pairs( overrides.groups ) do
						-- si encuentro el rol continuo
						if full_role_id == full_group_id then
							local role = roles[ full_role_id ] -- me quedo con la lista de roles, para no ir contra cassandra todas las veces por cada campana.
							if not role then
								role = m_models.Groups.GetGroups( domain, group.Id, keyspace )
								next( role )
							end
							roles[ full_role_id ] = role -- no importa que se pise una y otra vez!!

							-- si tiene acciones definidas (overrides) continua
							if group_overrides.actions then
								for app_id, actions in pairs( group_overrides.actions ) do
									local group_app_perm = group_permissions[ app_id ]
									if not group_app_perm then
										overrides.groups[ full_role_id ].actions[ app_id ] = nil
										changed = true
									else
										-- ahora recorro las acciones del override, comparo contra las del grupo y elimino del override roles de la campana si es necesario
										for action_id, enabled in pairs( actions ) do
											if enabled then
											-- si las 2 acciones estan en true, elimino el override.
											-- es decir, si la accion del role coincide en true con el override del role de la campana, no cumple funcion y lo elimino
												if group_app_perm[ action_id ] == enabled then
													overrides.groups[ full_role_id ].actions[ app_id ][ action_id ] = nil
													changed = true
												end
											else
											-- si el override tiene el flag en false de la accion (que la misma esta desactivada),
											-- y si el grupo no tiene una accion, la misma no llega (queda en nil), entonces ahi, procedo a eliminarla del override
												if group_app_perm[ action_id ] == nil then
													overrides.groups[ full_role_id ].actions[ app_id ][ action_id ] = nil
													changed = true
												end
											end
										end
									end
								end
							end
						end
					end

					if changed then
					-- si exsitieron cambios, tengo que validar que no queden apps sin acciones de override, y grupos de apps sin ninguna app con acciones de overide.
					-- si se da el caso antes mencionado, tengo que eliminar tambien de los override la app y/o la lista de apps
						for group_id, apps in pairs( overrides.groups ) do
							local count_apps = 0
							local count_removed_apps = 0
							-- cuento la cantidad de aplicaciones y acciones que existen con override, la cantidad de apps removidas
							for app_id, apps_actions in pairs( apps.actions ) do
								count_apps = count_apps + 1
								local count_actions_in_app = 0
								for action_id, actions in pairs( apps_actions ) do
									count_actions_in_app = count_actions_in_app + 1
								end
								if count_actions_in_app == 0 then
								-- si la app no tiene acciones, la elimino de los overrides
									overrides.groups[ group_id ].actions[ app_id ] = nil
									count_removed_apps = count_removed_apps + 1
								end
							end
							if count_apps == count_removed_apps then
							-- si no existe ninngua app con override tengo que eliminar la lista de contenedor de acciones por app
								overrides.groups[ group_id ].actions = nil
								overrides.groups[ group_id ] = nil
							end
						end
						-- aca guardo los nuevos overrides de members para la campana especifica
						LogInfo("[ADMINISTRATION.GROUPS:CleanUpCampaignRoleOverrides] CleanUp Campaign Roles Overrides - New overrides for campaign '%s' = '%s'.", campaign_id, json.encode( overrides.groups ) )

						admCampaignController.SaveAllPermissionsOnCampaign(
							campaign_id,
							overrides.groups,
							nil, -- members permissions
							nil, -- new_members
							keyspace,
							applicationsActions,
							ksHistorical
						)
					end
				end
			end
		end
	end

end

--[[
  Esta funcion se encarga de eliminar los overrides a nivel VCC, especificamente los overrides de Usuarios ( override Hijos )
--]]
local function CleanUpUserOverrides (groups, keyspace)
	local vcc = inconcert.authUser.virtualContactCenter

	LogInfo("[ADMINISTRATION.GROUPS:CleanUpUserOverrides] 1 - CleanUp VCC Users Overrides")

	-- recorro la lista de grupos que se van a guardar y valido si tienen override ( siempre es un solo grupo, pero se realizo el request, para que pueda recibir varios grupos en un mismo request )
	for _, group in pairs( groups ) do
		local roles = {}
		local is_system_group = IsSystemGroup( group.Id, nil, keyspace )
		local domain = ( is_system_group and "system" or vcc )
		local full_group_id = utf8.lower( group.Id .. "@" .. domain )
		local group_permissions = group.grouppermissions

		-- si el grupo tiene permisos asignados, procedo a realizar la validacion de overrides
		if HasGroupPermission( group_permissions ) then

			-- obtengo la lista de usuarios pertenecientes al grupo
			local users = m_models.Users.GetUsersByRole( full_group_id, keyspace )

			-- recorro la lista de users
			for full_user_id in pairs( users ) do
				local overrides = model_perm.GetUserPermissions( full_user_id, keyspace )

				-- si el usuario tiene override grupos definidos, continuo

				if overrides and overrides.actions then
					local changed = false
					-- si tiene acciones definidas (overrides) continua
					for app_id, actions in pairs( overrides.actions ) do
						local group_app_perm = group_permissions[ app_id ]
						if not group_app_perm then
							overrides.actions[ app_id ] = nil
							changed = true
						else
							-- ahora recorro las acciones del override, comparo contra las del grupo y elimino del override roles de la campana si es necesario

							for action_id, enabled in pairs( actions ) do
								if enabled then
									-- si las 2 acciones estan en true, elimino el override.
									-- es decir, si la accion del role coincide en true con el override del user, no cumple funcion y lo elimino
									if group_app_perm[ action_id ] == enabled then
										overrides.actions[ app_id ][ action_id ] = nil
										changed = true
									end
								else
									-- si el override tiene el flag en false de la accion (que la misma esta desactivada),
									-- y si el grupo no tiene una accion, la misma no llega (queda en nil), entonces ahi, procedo a eliminarla del override
									if group_app_perm[ action_id ] == nil then
										overrides.actions[ app_id ][ action_id ] = nil
										changed = true
									end
								end
							end
						end
					end

					if changed then
					-- si exsitieron cambios, tengo que validar que no queden apps sin acciones de override, y grupos de apps sin ninguna app con acciones de override.
					-- si se da el caso antes mencionado, tengo que eliminar tambien de los override la app y/o la lista de apps
						local count_apps = 0
						local count_removed_apps = 0
							-- cuento la cantidad de aplicaciones y acciones que existen con override, la cantidad de apps removidas
						for app_id, apps_actions in pairs( overrides.actions ) do
							count_apps = count_apps + 1
							local count_actions_in_app = 0
							for action_id, actions in pairs( apps_actions ) do
								count_actions_in_app = count_actions_in_app + 1
							end
							if count_actions_in_app == 0 then
								-- si la app no tiene acciones, la elimino de los overrides
								overrides.actions[ app_id ] = nil
								count_removed_apps = count_removed_apps + 1
							end
						end
						if count_apps == count_removed_apps then
						-- si no existe ninngua app con override tengo que eliminar la lista de contenedor de acciones por app
							overrides.actions = nil
							overrides.has_override = nil
						end
						-- aca guardo los nuevos overrides de members para la campana especifica
						LogInfo("[ADMINISTRATION.GROUPS:CleanUpUserOverrides] CleanUp VCC Users Overrides - New overrides for user '%s' = '%s'.", full_user_id, json.encode( overrides ) )
						model_perm.SaveUserPermissions( full_user_id, overrides, keyspace )
					end
				end
			end
		end
	end
end

local function CleanUpOverrides (groups, members_overrides, keyspace, ksHistorical)
	CleanUpUserOverrides( groups, keyspace )
	CleanUpCampaignRoleOverrides( groups, keyspace, ksHistorical )
	CleanUpCampaignMembersOverrides( members_overrides, keyspace, ksHistorical)
end

local function Save (groupsToSave, keyspace, ksHistorical)
	local vcc = inconcert.authUser.virtualContactCenter

	local function AlreadyRoleName (list, groupId, groupName)
		local repeated = utils.FilterEntities(list, {Name = groupName})
		for id in pairs(repeated) do
			if id:lower() == groupId:lower() then
				return true
			end
		end
		return false
	end

	--Get all Groups from model
	local existingCustomGroups = m_models.Groups.GetGroups( vcc, nil, keyspace )
	--LogDebug("[ADMINISTRATION.GROUPS:Save] --> existingCustomGroups: %s.", json.encode(existingCustomGroups))
	local existingSystemGroups = m_models.Groups.GetGroups( "system", nil, keyspace )
	--LogDebug("[ADMINISTRATION.GROUPS:Save] --> existingSystemGroups: %s.", json.encode(existingSystemGroups))

	local idMapping = {}
	local groups = {}
	local members_overrides
	local applicationsActions = permissions.GetApplicationsActions(keyspace, vcc)
	local roles_with_full_id = {}
	--[[
		En este Loop realizamos las diferentes validaciones sobre la lista de Groups, para determinar si es posible realizar la Actualización.
	]]--
	for groupId, group in pairs( groupsToSave ) do
		if type(group.Description) ~= "string" then
			local reason = string.format( "group %s doesnt have a description", groupId )
			return false, { result = "MUST_HAVE_A_DESCRIPTION", reason = reason }
		end
		if type(group.Name) ~= "string" then
			local reason = string.format( "group %s doesnt have a Name", groupId )
			return false, { result = "MUST_HAVE_A_NAME", reason = reason }
		end
		if not HasGroupPermission(group.grouppermissions) then
			local reason = string.format( "group %s doesnt have a group permissions", groupId )
			return false, { result = "MUST_HAVE_A_PERMISSIONS", reason = reason }
		end
		if not IsEnabledPermissionToAssign(groupId, group.grouppermissions, keyspace, applicationsActions) then
			local reason = string.format( "By modifying %s group permissions, No user has permissions to assign permissions.", groupId )
			return false, { result = "NO_USER_HAS_PERMISSIONS_TO_ASSIGN_PERMISSIONS", reason = reason }
		end

		-- full access system role validation
		if existingSystemGroups[groupId] then
			if tonumber(existingSystemGroups[groupId].IsFullSystemAccess or "0") == 1 then
				local reason = string.format( "Role Name %s - Unable to update System Role.", group.Name )
				return false, { result = "UNABLE_TO_UPDATE_SYSTEM_ROLE_PERMISSIONS", reason = reason }
			end
		end

		local users, count = m_models.Users.GetUsersByRole( groupId, keyspace )
		--LogDebug("group_id: %s, count: %d", group_id, count)
		if count >= 1 then
			local result = InConflictUsersByGroupWithAssignedInteractions(users, groupId, group.grouppermissions, keyspace, applicationsActions)
			if not result.success then
				local reason = string.format( "Assigned interactions conflicts. %d users belogns to group '%s' without permission to INTERACTION application. Problematic info: '%s'", result.count_users, groupId, json.encode(result.problematic_users) )
				return false, { result = "SOME_USERS_HAS_INTERACTIONS_WITHOUT_PERMISSION_TO_INTERACTION_APP", reason = reason, raw_error = result }
			end
			local result = InConflictUsersByGroupWithAttentionLevels(users, groupId, group.grouppermissions, keyspace, applicationsActions)
			if not result.success then
				local reason = string.format( "Attention levels conflicts. %d users belogns to group '%s' without permission to INTERACTION application. Problematic info: '%s'", result.count_users, groupId, json.encode(result.problematic_users) )
				return false, { result = "ATTENTION_LEVELS_CONFLICT_USER_WITHOUT_PERMISSION_TO_INTERACTION_APP", reason = reason, raw_error = result }
			end
		end

		LogInfo("[ADMINISTRATION.GROUPS:Save] group.__override: %s", group.__override)
		if group.__override then
			members_overrides = members_overrides or {}
			members_overrides[ groupId ] = group.__override
			group.__override = nil
		end

		--If this is a new entity, try to find an unused Id for it
		local isNewGroup
		if group.isNew then
			idMapping[groupId] = groupId
			group.isNew = nil
			isNewGroup = true
			-- Add creation date
			group.creationDate = LuaOrm.Date():toString()
		else
			--modification of an existing entity, must ensure that the entity already exists
			local exists = false
			utils.FilterEntities(existingCustomGroups, function(id) exists = exists or id == groupId end)
			if not exists then
				utils.FilterEntities(existingSystemGroups, function(id) exists = exists or id == groupId end)
				if not exists then
					local reason = string.format( "Group %s does not exist", groupId )
					return false, { result = "ID_DOES_NOT_EXIST", reason = reason }
				end
			end
		end
		--Check repeated groups (those with same Id or Name)
		-- custom role validation
		if isNewGroup and AlreadyRoleName(existingCustomGroups, groupId, group.Name) then
			local reason = string.format( "Name %s already exists in custom permissions.", group.Name )
			return false, { result = "MUST_HAVE_UNIQUE_NAME", reason = reason }
		end
		-- system role validation
		if isNewGroup and AlreadyRoleName(existingSystemGroups, groupId, group.Name) then
			local reason = string.format( "Name %s already exists in system permissions.", group.Name )
			return false, { result = "NAME_ALREADY_EXISTS_IN_SYSTEM_PERMISSIONS", reason = reason }
		end

		------------------------------------
		group.Id = groupId

		local is_system_group = IsSystemGroup( groupId, existingSystemGroups, keyspace )
		if is_system_group then
			group.IsSystemGroup = "1"
			-- group.Name = nil
			-- group.Description = nil
			existingSystemGroups[ groupId ] = group
			roles_with_full_id[groupId .. "@system"] = group
		else
			group.IsSystemGroup = "0"
			existingCustomGroups[ groupId ] = group
			roles_with_full_id[groupId .. "@"..vcc] = group
		end

		groups[ groupId ] = group
	end

	--[[
		Funcion que almacena los permisos en la CF "GroupPermissions"
	]]--
	local override_groups = {}

	local function SavePermissions ( groupId, domain, group, is_system_group )
		local fullGroupId = groupId .. "@" .. domain

		local appId = "administration"
		if group and group.grouppermissions and group.grouppermissions[ appId ] and group.grouppermissions[ appId ][ "create_campaign" ] then
			-- categorizo las acciones para poder mapear su busqueda
			local categorized_permissions = model_perm.Categorize(group.grouppermissions, keyspace, applicationsActions)
			if categorized_permissions[ appId ] then
				local currentAppActions = model_perm.GetApplicationActions( appId, keyspace )

				local function add_related_actions( actions )
					for creation_action_id in pairs( actions ) do
						local action_def = currentAppActions[ creation_action_id ]
						-- si tienen acciones relacionadas, les termino dando permisos.
						if action_def and action_def.RelationsActions then
							if action_def.RelationType == "rol_to_user" then
								for _, relation_action_id in pairs( action_def.RelationsActions ) do
									LogInfo("[ADMINISTRATION.GROUPS:Save] Role '%s' Builtin relation actions for app %s, actions: %s --> %s", fullGroupId, appId, action_def.ActionId, relation_action_id)
									group.grouppermissions[ appId ][ relation_action_id ] = true
								end
							end
						end
					end
				end
				-- aca obtengo las acciones de save
				local campaign_creation_actions = categorized_permissions[ appId ][ "save" ] or {}
				add_related_actions( campaign_creation_actions )
				-- aca obtengo las acciones de delete
				campaign_creation_actions = categorized_permissions[ appId ][ "delete" ] or {}
				add_related_actions( campaign_creation_actions )
			end
		end

		model_perm.DeleteGroupPermissions( fullGroupId, keyspace )
		model_perm.SaveGroupPermissions( fullGroupId, group.grouppermissions, keyspace )
		ChangeAvailableUsersStates(fullGroupId, group, keyspace, applicationsActions)
		override_groups = override_groups or {}
		override_groups[ fullGroupId ] = {}
		override_groups[ fullGroupId ][ "Id" ] = groupId
		override_groups[ fullGroupId ][ "grouppermissions" ] = group.grouppermissions

		if is_system_group then
			system_group = m_models.Groups.GetGroups( "system", groupId, keyspace )
			if next( system_group ) then
			-- estos valores no se pueden modificar, son siempre los por default del role
				group.Name = system_group.Name
				group.Description = system_group.Name
			end
		end
	end

	-- Elimina de la CF "GroupPermissions"
	local function ErasePermissions (groupId)
	-- delete role permissions
		local fullGroupId = groupId .. "@" .. vcc
		model_perm.DeleteGroupPermissions( fullGroupId, keyspace )
		LogInfo("[ADMINISTRATION.GROUPS:Save] - Modified deleted of the '%s' Role in vcc '%s.", groupId, vcc )
	end

	-- Elimina de la CF "Groups"
	local function EraseRole (groupId)
		-- remove role
		local success, err = m_models.Groups.RemoveGroups( vcc, { groupId }, keyspace )
		if not success then
			LogWarning("[ADMINISTRATION.GROUPS:Save] Removing roles error: %s.", json.encode(err))
		end
	-- delete role permissions
		ErasePermissions(groupId)
	end

	local administrationActions
	local function CanValidateFn (attr)
		-- Esta funcion valida si la accion (attr), de la app "administration", se debe de validar dentro la funcion [utils.StructuralCompare].
		--  La misma es invocada por esta, funcion de utils.
		--  En si valida si la accion se almacena unicamente si la misma NO visible unicamente para los overrides.
		if not attr then return true end
		if not administrationActions then
			administrationActions = model_perm.GetApplicationActions("administration", keyspace)
			if not administrationActions then
				return true
			end
		end
		local action = administrationActions[attr]
		if not action then return true end
		if not action.Domain then return true end
		if tonumber(action.Domain.only_override) ~= 1 then return true end
		--LogDebug("[CanValidateFn]--> Exclude action '%s' of administration app, because this action is a only override.", attr)
		-- Si llega aca quiere decir que no se debe de validar, ya que es solo para override esta accion
		return false
	end

	--[[
		En este Loop, comenzamos la Actualización contra la CF "GroupPermissions"
	]]--
	local notifications = {} -- En esta tabla, se van almacenando las notificaciones que se realizarán a los Usuarios, por los cambios de permisos!!
	local groupsToRollback = {} -- En es utilizada para realizar el Rollback, en ella van quedando las modificaciones de los Roles, como estaban antes de ser modificados, etc!!
	local customGroupsToUpdate -- Acá se almacenan los roles que se actualizaran, los mismos son enviados para que se actualice la CF "Groups" y se realicen los Request hacia las Integraciones "Allegro", "Helpdesk"
	local onlyUpdateIntegration = false
	local old_groups_audit_data = GetGroupAuditData(keyspace) -- Obtengo los datos a auditar de los grupos
	for groupId, group in pairs(groups) do
		local status
		local hasNotifyUsers = false

		local is_system_group = IsSystemGroup(groupId, existingSystemGroups, keyspace)
		if group.grouppermissions then
			local domain = vcc
			if is_system_group then
			-- Si es un ROL del sistema
			--
				local default_system_group_permissions = model_perm.GetGroupPermissions(groupId .. "@system", keyspace)
				local new_system_group_permissions = group.grouppermissions
				local hasChangedSystemGroup = not utils.StructuralCompare(default_system_group_permissions, new_system_group_permissions, CanValidateFn)

				if hasChangedSystemGroup then
				--	LogDebug("1")
					LogInfo("[ADMINISTRATION.GROUPS:Save] + System Role '%s' has changed, your default permissions !!!", groupId)

					local current_custom_group_permissions = model_perm.GetGroupPermissions(groupId .. "@" .. domain, keyspace)
					if not HasGroupPermission(current_custom_group_permissions) then
						current_custom_group_permissions = default_system_group_permissions
					end
					-- Me quedo con la información para realizar un posible rollback del permiso del sistema que ha sido modificado!!
					LogInfo("[ADMINISTRATION.GROUPS:Save] Store System '%s' Group Permissions for Rollback. System Role has changed.", groupId)
					local new_custom_group_permissions = group.grouppermissions
					groupsToRollback[groupId] = {
						is_system = true,
						has_changed = true,
						current_custom_group_permissions = current_custom_group_permissions,
						new_custom_group_permissions = new_custom_group_permissions,
						default_system_group_permissions = default_system_group_permissions,
						groupInfo = group
					}

					--LogDebug("1.1- default_system_group_permissions : %s", json.encode(default_system_group_permissions))
					--LogDebug("1.2- new_system_group_permissions     : %s", json.encode(new_system_group_permissions))
					SavePermissions(groupId, domain, group, true)
					status = "SAVED"
					hasNotifyUsers = true
					-- TODO: Cuando se realiza este change al role del sistema, se tienen que cambiar todos los usuarios en la CF "UsersByGroup", para que esten asignados al role del sistema modificado!!
				else
					--LogDebug("2")
					LogInfo("[ADMINISTRATION.GROUPS:Save] - System Role '%s' not has changed, your default permissions !!!", groupId)

					local response = model_perm.IsModifiedSystemGroup( groupId, keyspace )
					if response.modified then
						local current_custom_group_permissions = model_perm.GetGroupPermissions( groupId .. "@" .. domain, keyspace )
						if HasGroupPermission(current_custom_group_permissions) then
							LogInfo("[ADMINISTRATION.GROUPS:Save] Store System '%s' Group Permissions for Rollback. System Role not has changed.", groupId)
							local new_custom_group_permissions = group.grouppermissions
							groupsToRollback[groupId] = {
								is_system = true,
								has_changed = false,
								current_custom_group_permissions = current_custom_group_permissions,
								new_custom_group_permissions = new_custom_group_permissions,
								default_system_group_permissions = default_system_group_permissions,
								groupInfo = group
							}
						end
						EraseRole(groupId)
						status = "DELETED"
						onlyUpdateIntegration = true
						hasNotifyUsers = true
					end
				end
			else
			-- Si es un ROL del custom
			--
				--LogDebug("3")
				LogInfo("[ADMINISTRATION.GROUPS:Save] Updating Custom Role '%s' !!!", groupId)
				local current_custom_group_permissions = model_perm.GetGroupPermissions( groupId .. "@" .. domain, keyspace )
				local new_custom_group_permissions = group.grouppermissions

				SavePermissions(groupId, domain, group, false)
				status = "SAVED"

				local hasChangedCustomGroup = not utils.StructuralCompare(current_custom_group_permissions, new_custom_group_permissions)
				if hasChangedCustomGroup then
					LogInfo("[ADMINISTRATION.GROUPS:Save] Store Custom '%s' Group Permissions for Rollback.", groupId)
					groupsToRollback[groupId] = {
						is_system = false,
						has_changed = true,
						current_custom_group_permissions = current_custom_group_permissions,
						new_custom_group_permissions = new_custom_group_permissions,
						groupInfo = group
					}
					hasNotifyUsers = true
				end
			end
		end

		if status == "SAVED" or onlyUpdateIntegration then
			customGroupsToUpdate = customGroupsToUpdate or {}
			group.IsDeployer = "0"
			group.IsFullSystemAccess = "0"
			customGroupsToUpdate[groupId] = group
		end

		if hasNotifyUsers then
			table.insert(notifications, {
				role_id = groupId,
				is_system_group = is_system_group
			})
		end
	end

	--[[
		Función que realiza Rollback de la CF "GroupPermissions" (overrides), si es que la actualización de la CF "Groups" Falla.
		En sí, la falla NO es contra la CF "Groups", sino cuando se actualiza contra "Allegro" o "Helpdesk".
		Si falla la actualización contra una Integración, tiene que realizar el Rollback.
	]]--
	local function RollbackPermissions ()
		LogInfo("[ADMINISTRATION.GROUPS:Save] Rollback Permissions custom roles...")

		for groupId, group in pairs(groupsToRollback) do
			--LogDebug("-->groupId: '%s' is_system: '%s'", groupId, group.is_system)
			if group.is_system then
				if group.has_changed then
					--LogDebug("A")

					local hasChangedCurrentPermission = not utils.StructuralCompare(group.current_custom_group_permissions, group.new_custom_group_permissions, CanValidateFn)
					if hasChangedCurrentPermission then
						--LogDebug("A.1")
						local hasChangedSystemPermission = not utils.StructuralCompare(group.default_system_group_permissions, group.current_custom_group_permissions, CanValidateFn)
						if hasChangedSystemPermission then
							-- Si el Rol del sistema ya tenía un override y un siguiente override ha fallado.
							LogInfo("[ADMINISTRATION.GROUPS:Save] Rollbacking System '%s' Group Permissions. Role has had more than one overwrite.", groupId)
							--LogDebug("A.1.1")
							group.groupInfo.grouppermissions = group.current_custom_group_permissions
							SavePermissions(groupId, vcc, group.groupInfo, true)
						else
							--LogDebug("A.1.2")
							-- Si el Rol del sistema ya tenía un override y un siguiente override ha fallado.
							LogInfo("[ADMINISTRATION.GROUPS:Save] Rollbacking System '%s' Group Permissions. Reload System Permissions.", groupId)
							ErasePermissions(groupId)
						end
					else
						--LogDebug("A.2")
						LogDebug("[ADMINISTRATION.GROUPS:Save] Rollbacking System '%s' Group Permissions. POR ACA NO DEBERIA PASAR!!. REVISAR SI PARA ACA PASA", groupId)
					end
				else
					--LogDebug("B")
					local has_changed = not utils.StructuralCompare(group.current_custom_group_permissions, group.new_custom_group_permissions, CanValidateFn)
					--LogDebug("B1 response: '%s'", has_changed)
					if has_changed then
						-- Si el Rol del sistema tenía un override y se han modificado los permisos con los mismos valores que el tiene el Rol del sistema.
						LogInfo("[ADMINISTRATION.GROUPS:Save] Rollbacking System '%s' Group Permissions. The Permissions of the Custom Role are identical to those of the System Role.", groupId)
					--LogDebug("B2")
						group.groupInfo.grouppermissions = group.current_custom_group_permissions
						SavePermissions(groupId, vcc, group.groupInfo, true)
					end
				end
			else
				--LogDebug("C")
				LogInfo("[ADMINISTRATION.GROUPS:Save] Rollbacking Custom '%s' Group Permissions ...", groupId)
				group.groupInfo.grouppermissions = group.current_custom_group_permissions
				SavePermissions(groupId, vcc, group.groupInfo, false)
			end
		end
	end

	--[[
		Función encargada de notificar a los Usuarios los cambios de roles!!
		La misma es invocada si la Actualización se ha realizado correctamente.
	]]--
	local function NotifyUsers ()
		LogInfo("[ADMINISTRATION.GROUPS:Save] Notify Users who have Changed the Permissions of Role...")
		for _, notification in ipairs(notifications) do
			agentNotifications.NotifyAgents({
				eventType = "administrator_role_permissions_change",
				agentsLevel = "ROLE_USERS",
				pollerMessage = notification
			}, keyspace)
		end
	end

	if customGroupsToUpdate then
		LogInfo("[ADMINISTRATION.GROUPS:Save] Saving custom roles")
		local success, err = m_models.Groups.SaveGroups(vcc, customGroupsToUpdate, onlyUpdateIntegration, keyspace)
		if not success then
			LogError("[ADMINISTRATION.GROUPS:Save] Saving custom roles error: %s.", json.encode(err))
			RollbackPermissions()
			return false, err
		end
		NotifyUsers()
	end

	CleanUpOverrides(override_groups, members_overrides, keyspace, ksHistorical)

	--Crear registros de auditoria para los grupos salvados
	local current_groups_audit_data = GetGroupAuditData(keyspace)
	LogInfo("[ADMINISTRATION.GROUPS:Save] Create Audit record - roles %s", json.encode(roles_with_full_id))
	local audit_params = {
		vcc = vcc,
		entity = "Roles",
		user = inconcert.authUser.userName
	}
	--Se crea registro de auditoria
	local connectionHistorical
	for groupFullId, _ in pairs(roles_with_full_id) do
		if not connectionHistorical then
			connectionHistorical = utils.GetDatasource():CassandraConnection( "Historical" )
		end
		audit_params.fieldKey = groupFullId:lower()
		audit_params.oldFields = old_groups_audit_data[groupFullId] or nil
		audit_params.newFields = current_groups_audit_data[groupFullId] or nil
		audit_params.connection = connectionHistorical
		audits.Create(audit_params)
	end
	if connectionHistorical then
		connectionHistorical:Close()
	end
	return true, idMapping
end

function SaveGroups (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc
	local groupsToSave = web.request
	local groupsToNotify = json.decode(json.encode(groupsToSave))

	LogInfo("[ADMINISTRATION.GROUPS:SaveGroups] Saving roles - user %s, vcc %s", userId, vcc )

	local ksHistorical = utils.GetDatasource():CassandraConnection("Historical")
	local keyspace = utils.GetDatasource():CassandraConnection( "Agent" )
	local success, idMapping = Save( groupsToSave, keyspace, ksHistorical )
	keyspace:Close()
	ksHistorical:Close()

	if not success then
		local err = idMapping
		LogError("[ADMINISTRATION.GROUPS:SaveGroups] Error Saving roles, reason: '%s'. Posted data: '%s'", (err.reason or err.message_error), web.input["post_data"])
		web.status = 400
		return err
	end

	LogInfo("[ADMINISTRATION.GROUPS:SaveGroups] Succesuffuly saved roles.")

	--notifications
	for groupId, group in pairs(groupsToNotify) do
		if groupsToSave[groupId] and groupsToSave[groupId].IsSystemGroup == "1" then
			notificationsEngine.SendNotification("SystemRoleOverrided", 0, { role = group })
		else
			notificationsEngine.SendNotification(group.isNew and "RoleCreated" or "RoleUpdated", 0, { role = group })
		end
	end

	return { result = "OK", idMapping = idMapping }
end

--[[
  Esta funcion/uri se encarga de verificar si los overrides de campana / members (override Nietos) colisionan con los override de su abuelo.
  Es decir si tenemos una determinada accion que se realiza override role / campaign y luego se realiza override en member / campaign (Nieto)
  Luego se modifica el permiso original (Abuelo), llevandolo a un estado diferente al de su nieto, se tiene que informar al cliente que esto ocurre, para que este decida, si mantiene el override del nieto, o lo remueve.
  
  Ej:
							role   camp role  camp member

Antes de modificar			true      false      true     
Despues de modificar				false      false      true   (cambia) Es diferente tengo que informar al cliente 

Antes de modificar			true       --        false    
Despues de modificar				false       --        false  (NO cambia) No tengo que informar al cliente

Antes de modificar			 --       true       false    
Despues de modificar				true       true        false  (cambia)  Es diferente tengo que informar al cliente

Antes de modificar			 --        --        true     
Despues de modificar				 true        --        true   (NO cambia)  No tengo que informar al cliente
  
--]]
function ValidateOverrides (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName .. "@" .. vcc
	local group = web.request.group

	LogInfo("[ADMINISTRATION.GROUPS:ValidateOverrides] Validate Overrides Campaigns Members Permission. user %s, vcc %s, post_data: %s", userId, vcc, web.input["post_data"])

	local count_changed = 0
	local overrided_permissions
	-- esta funcion se encarga de agregar en la lista de overrides que se tiene que informar al cliente
	local AddChange = function( campaign_id, member_id, app_id, action_id, new_status )
		LogInfo("[ADMINISTRATION.GROUPS:ValidateOverrides] Changed parent of parent in override - app: '%s', action: '%s' = '%s' on campaign: '%s', member: '%s'", app_id, action_id, new_status, campaign_id, member_id )
		count_changed = count_changed + 1
		overrided_permissions = overrided_permissions or {}
		overrided_permissions[ campaign_id ] = overrided_permissions[ campaign_id ] or {}
		overrided_permissions[ campaign_id ][ member_id ] = overrided_permissions[ campaign_id ][ member_id ] or {}
		overrided_permissions[ campaign_id ][ member_id ][ app_id ] = overrided_permissions[ campaign_id ][ member_id ][ app_id ] or {}
		overrided_permissions[ campaign_id ][ member_id ][ app_id ][ action_id ] = new_status
	end

	if group then
		local keyspace = utils.GetDatasource():CassandraConnection( "Agent" )
		local is_system_group = IsSystemGroup( group.Id )
		local full_group_id = utf8.lower( group.Id .. "@" .. ( is_system_group and "system" or vcc ) )
		local group_permissions = group.grouppermissions

		if not group_permissions and is_system_group then
			group_permissions = model_perm.GetGroupPermissions( full_group_id, keyspace )
		end

		-- si el grupo tiene permisos
		if HasGroupPermission( group_permissions ) then
			local members_by_role = {}
			local campaigns = m_models.Campaigns.GetVCCCampaigns( vcc, keyspace )
			-- recorro la lista de campanas, para buscar el override de members (nietos)
			for campaign_id in pairs( campaigns ) do

				local full_campaign_id = utils.AppendVcc(campaign_id, vcc)
				local overrides
				local encoded_permissions = m_models.Campaigns.GetSpecificColumn( full_campaign_id, "permissions", keyspace )
				--LogDebug("[ValidateOverrides] encoded_permissions: %s", encoded_permissions)
				if type(encoded_permissions) == "string" then
					overrides = json.decode(encoded_permissions)
				end

				if overrides and overrides.members then
				-- si tenemos override de members
					for member_id, member_overrides in pairs( overrides.members ) do
						local full_member_id =  member_id .. "@" .. vcc
						local member = members_by_role[ full_member_id ] -- a esto lo realizo para no tener que ir todas las veces contra cassandra
						if not member then
							member = m_models.Users.GetUser( vcc, member_id, keyspace, {"UserGroup", "VccGroup"} )
						end
						local mUserGroup = member.UserGroup or ""
						local mVccGroup = member.VccGroup or ""
						if member and full_group_id == utf8.lower( mUserGroup .. "@" .. mVccGroup ) then
							members_by_role[ full_member_id ] = member -- no importa que se pise una y otra vez!!
							if member_overrides.actions then
								for app_id, actions in pairs( member_overrides.actions ) do
									local group_app_perm = group_permissions[ app_id ]
									for action_id, enabled in pairs( actions ) do
										if enabled then
											if not group_app_perm or group_app_perm[ action_id ] == nil then
											-- si son diferentes las agrego en la lista de override que colisionan, con su nuevo estado
												AddChange( campaign_id, member_id, app_id, action_id, false )
											end
										else
											if group_app_perm and group_app_perm[ action_id ] == true then
											-- si son diferentes las agrego en la lista de override que colisionan, con su nuevo estado
												AddChange( campaign_id, member_id, app_id, action_id, true )
											end
										end
									end
								end
							end
						end
					end
				end
			end
		end

		keyspace:Close()
	end

	local response = { result = "OK", override_changed = false }

	if count_changed > 0 then
	-- en el response informo al cliente que tiene override que colisionan
		response.override_changed         = true
		response.count_changed            = count_changed
		response.override_actions_changed = json.encode( overrided_permissions )
		LogInfo("[ADMINISTRATION.GROUPS:ValidateOverrides]  Has changed in conflict Overrides Campaigns Members Permission.\n Count conflicts %d, \n %s", count_changed, response.override_actions_changed )
	else
		LogInfo("[ADMINISTRATION.GROUPS:ValidateOverrides] Not changed Overrides Campaigns Members Permission")
	end

	return response
end

--[[
Remove groups from a vcc
postedData must be of the form: ["groupId1", "groupId2", ...]
]]
function RemoveGroups (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[ADMINISTRATION.GROUPS:RemoveGroups] user %s, vcc %s, post_data: %s", userId, vcc, web.input["post_data"])

	if "system" == vcc:lower() then
		web.status = 403
		return { result = "UNABLE_TO_REMOVE_SYSTEM_ROLE_PERMISSIONS", reason = reason }
	end

	local roles = web.request
	local keyspace = utils.GetDatasource():CassandraConnection( "Agent" )
	local roles_with_full_id = {}
	-- validate remove groups
	for _, roleId in ipairs(roles) do
		local currentRole = m_models.Groups.GetGroups(vcc, roleId, keyspace)
		--LogDebug("[ADMINISTRATION.GROUPS:RemoveGroups]==> roleId %s, currentRole %s", roleId, json.encode(currentRole or {}))
		if not next(currentRole) then
			keyspace:Close()
			web.status = 404
			local reason = LogError("[ADMINISTRATION.GROUPS:RemoveGroups] The role %s to does not exist", roleId)
			return {result = "ROLE DOES NOT EXIST", reason = reason}
		end
		-- full access system role validation
		if tonumber(currentRole.IsSystemGroup or "1") == 1 then
			keyspace:Close()
			web.status = 403
			local reason = string.format( "Role Name %s - Unable to remove System Role.", currentRole.Name )
			return { result = "UNABLE_TO_REMOVE_SYSTEM_ROLE_PERMISSIONS", reason = reason }
		end
		local full_role_id = utils.AppendVcc(roleId, vcc)
		table.insert(roles_with_full_id, full_role_id)
		local users, count = m_models.Users.GetUsersByRole( full_role_id, keyspace )
		--LogDebug("full_role_id: %s, users: %s", full_role_id, json.encode(users))
		if count > 0 then
			keyspace:Close()
			web.status = 400
			LogError("[ADMINISTRATION.GROUPS:RemoveGroups] The role %s to delete is currently in use", roleId)
			return {result = "ROLE_BEING_USED", reason = "The role '" .. roleId .. "' to delete is currently in use."}
		end
	end

	--Obtengo los datos de auditoria de los roles a eliminar
	local current_audit_data = GetGroupAuditData(keyspace)
	-- remove groups
	local success, err = m_models.Groups.RemoveGroups(vcc, roles, keyspace)
	if not success then
		keyspace:Close()
		web.status = 400
		LogError("[ADMINISTRATION.GROUPS:RemoveGroups] Removing roles error: %s.", json.encode(err))
		return err
	end

	-- delete group permissions and add audit data
	local connectionHistorical
	for _, roleId in ipairs(roles) do
		model_perm.DeleteGroupPermissions(roleId .. "@" .. vcc, keyspace)

		--Agrego registro de auditoria
		if current_audit_data[roleId .. "@" .. vcc] then
			if not connectionHistorical then
				connectionHistorical = utils.GetDatasource():CassandraConnection( "Historical" )
			end
			audits.Create({
				vcc = vcc,
				entity = "Roles",
				user = inconcert.authUser.userName,
				fieldKey =   (roleId .. "@" .. vcc):lower(),
				oldFields = current_audit_data[roleId .. "@" .. vcc],
				connection = connectionHistorical
			})
		end
	end
	if connectionHistorical then
		connectionHistorical:Close()
	end

	keyspace:Close()

	--notifications
	for _, roleId in ipairs(roles) do
		notificationsEngine.SendNotification("RoleDeleted", 0, { roleId = roleId })
	end
	
	return {result = "OK"}
end

function RestoreDefault (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc

	LogInfo("[ADMINISTRATION.GROUPS:RestoreDefault] Restoring Default Permission in system role. user %s, vcc %s, post_data: %s", userId, vcc, web.input["post_data"])

	local role = web.request.group
	local keyspace = utils.GetDatasource():CassandraConnection( "Agent" )
	role.grouppermissions = model_perm.GetGroupPermissions( role.Id .. "@system", keyspace )
	local roles = {}
	roles[ role.Id ] = role
	local ksHistorical = utils.GetDatasource():CassandraConnection("Historical")
	local success, err = Save( roles, keyspace, ksHistorical )
	keyspace:Close()
	ksHistorical:Close()

	if not success then
		LogError("[ADMINISTRATION.GROUPS:RestoreDefault] Error Restored system role, reason: '%s'. Posted data: '%s'", err.reason, web.input["post_data"])
		web.status = 400
		return err
	end

	LogInfo("[ADMINISTRATION.GROUPS:RestoreDefault] Succesuffuly restored system role.")

	--notifications
	notificationsEngine.SendNotification("SystemRoleRestored", 0, { roleId = role })

	return {result = "OK"}
end