local m_models = require("apps.administration.model"):GetModels()
module(..., package.seeall, inconcert.orbit.application.Decorate)
require "uuid"
local utils = require "shared.apps.utils"
local permissions = require "shared.permissions.permissions"
local repositoryController = require("apps.repository")
require "LuaOrm.Date"
local wsapi = require "wsapi"
wsapi.request = require "wsapi.request"

local m_moduleLocation = select(2, ...)

local function StartRepository()
	repositoryController.ManualStart(inconcert.environment.apps.agent.cassandra.ip, inconcert.environment.apps.agent.cassandra.port)
end

local function RemoveRepositoryFiles (filesToRemoveFromRepository)
	LogInfo("[ADMINISTRATION.SHARED_FILES:RemoveRepositoryFiles] Remove Repository Batch Files Content: %s", json.encode(filesToRemoveFromRepository))
	StartRepository()
	repositoryController.deleteBatchFiles(filesToRemoveFromRepository)
end

m_htmlRender = nil
m_appLocation = nil

local function Render (web)
	_M.response.DontCache(web)
	local env = { _T = m_htmlRender.extension.makeTextTranslator() }
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc

	LogInfo("[ADMINISTRATION.SHARED_FILES:Render] Render form shared_files for user %s, group %s", userId, inconcert.authUser.userGroup)

	local html_file = m_appLocation .. "/pages/frm_shared_file_content.html"
	local content = m_htmlRender:RenderPageLP(html_file, web, env)
	if not content then
		LogError("[ADMINISTRATION.SHARED_FILES:Render] HTML shared_files does not exist: %s", html_file)
	end
	return content
end

local function DeleteVCCProcessBatch (batchProcessId, vcc)
	local success = m_models.SharedFiles.DeleteVCCProcessBatch(batchProcessId, vcc)
	if success then
		LogInfo("[ADMINISTRATION.SHARED_FILES:DeleteVCCProcessBatch] Deleting Process batch Id '%s' vcc: '%s'.", batchProcessId, vcc)
		return true
	end
	LogInfo("[ADMINISTRATION.SHARED_FILES:DeleteVCCProcessBatch] Process does not exist. Process batch Id '%s' vcc: '%s'.", batchProcessId, vcc)
	return false
end

--Returns all files in the VCC
local function Search (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local fullUserId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local startTime = os.time()

	LogInfo("[ADMINISTRATION.SHARED_FILES:Search] 1 - (START) Search rows for user '%s', group '%s'. Posted data: '%s'", fullUserId, inconcert.authUser.userGroup, web.input["post_data"])

	local postData = web.request
-- local myPostData = [[
-- 	{"last":"","limit":16,"numberPage":1,"query":{"tree":{"operator":"OR","items":[{"Language":"en","childs":{"operator":"OR","items":[{"AgentVisible":"1"},{"AccountMessagesVisible":"1"},{"FlowDesignerVisible":"1"}]}}]},"OR":[]},"columns":["Id","Type","Name","Description","LastUsageInInteractions","RepositoryId","AgentVisible","AccountMessagesVisible","FlowDesignerVisible","Language"]}
-- ]]
--LogDebug("---> myPostData: %s", myPostData)
--postData = json.decode(myPostData)

	-- abro conexión a cassandra que sera utilizada en todas las conexiones a cassandra.
	local keyspace = utils.GetDatasource():CassandraConnection("Agent")

	--Get all files from model
	local retrieveWithRelations = true
	local rows = m_models.SharedFiles.GetFiles(vcc, nil, keyspace, retrieveWithRelations)

	--LogDebug("[ADMINISTRATION.SHARED_FILES:Search] 1.1 - All rows '%s'", json.encode(rows))

	local totalUnfilteredRows = utils.GetRowCount(rows)

	LogInfo("[ADMINISTRATION.SHARED_FILES:Search] 2 - Filtering rows")

	postData.query = postData.query or {}

	--LogDebug("[ADMINISTRATION.SHARED_FILES:Search] 2.1 - totalUnfilteredRows '%s'", totalUnfilteredRows)
	local totalAdded = 0
	local totalSize = 0

	--postData.query = {}
	local treeQuery = postData.query.tree or {}
	local AND_Query = postData.query.AND or {}
	local OR_Query = postData.query.OR or {}

--	local AND_Query = {}
-- table.insert(AND_Query, {CampaignId = "CAMPI6TEL"})
-- table.insert(AND_Query, {CampaignId = "campi6tel2"})

-- table.insert(AND_Query, {FlowId = "8181e38d-8210-4712-b111-f45797e74968"})
-- table.insert(AND_Query, {FlowId = "31404D18912672D1A8B1C800F9299E57"})

	local isEmptyTreeQuery = utils.TableIsEmpty(treeQuery)
	local isEmptyOR_Query  = utils.TableIsEmpty(OR_Query)
	local isEmptyAND_Query = utils.TableIsEmpty(AND_Query)

	local filteredRows = {}
	local function Add (fileData)
		totalAdded = totalAdded + 1
		table.insert(filteredRows, fileData)
	end

	local flowsByName -- map de flows por name, por si se ha buscado por flow name, tiene que ir a buscar el ID del mismo.

	local function CustomMatch (filterValue, fieldId, row)
		filterValue = (filterValue or ""):lower()
		LogDebug(" CustomMatch - fieldId: '%s', row: '%s', filterValue: '%s'.", fieldId, row, filterValue)
		if fieldId ~= "CampaignId" and fieldId ~= "FlowId" and fieldId ~= "FlowName" then
			--LogDebug("CustomMatch 1")
			return nil
		end
		--LogDebug("CustomMatch 2")
		if not row.Relations then
			--LogDebug("CustomMatch 3")
			return false
		end
		if fieldId == "CampaignId" then
			--LogDebug("CustomMatch  CampaignId 4")
			for _, campaignAccountMessages in pairs(row.Relations.CampaignAccountMessages or {}) do
				local campaignIdWithoutVcc = utils.GetId(campaignAccountMessages.campaignId):lower()
				if campaignIdWithoutVcc == filterValue then
					--LogDebug("CustomMatch CampaignId  SIII 4.1")
					return true, campaignIdWithoutVcc
				end
			end
		elseif fieldId == "FlowId" or fieldId == "FlowName" then
			--LogDebug("CustomMatch  FlowId 5")
			if fieldId == "FlowName" then
				-- Si ha buscado por FlowName tiene hago un mapeo para luego ir a buscar el Id del Flow.
				if not flowsByName then
					flowsByName = flowsByName or {}
					local flows = m_models.ProcessFlows.GetFlows(vcc, nil, {"Id", "Name"}, keyspace)
					for _, flowInfo in pairs(flows or {}) do
						flowsByName[flowInfo.Name:lower()] = flowInfo
					end
				end
				local flowInfo = flowsByName[filterValue]
				if flowInfo and flowInfo.Id then
					filterValue = flowInfo.Id:lower()
					fieldId = "FlowId"
				end
			end
			--LogDebug("-----------------> fieldId-> '%s' =  '%s'", fieldId, filterValue)
			for flowId in pairs(row.Relations.FlowDesigner or {}) do
				flowId = flowId:lower()
				if flowId == filterValue then
					--LogDebug("CustomMatch FlowId  SIII 5.1")
					return true, flowId
				end
			end
		end
		--LogDebug("CustomMatch END")
		return false
	end

	local i = 0
	for rowId, row in pairs(rows) do
		i = i  + 1
		totalSize = totalSize + tonumber(row.Size)
		row.Id = (row.Id or rowId):lower()
		row.Name = row.Name or rowId

		--LogDebug("-- (%d) -------", i )
		--LogDebug("-START rowId %s--------", rowId)
		if isEmptyTreeQuery and isEmptyOR_Query and isEmptyAND_Query then
			--LogDebug("-1 - ADDDD")
			Add(row)
		else
			local mustAdd

			if not isEmptyAND_Query then
				--LogDebug("0 - voy por el AND de las campañas y flow ---------------------")
				--LogDebug("  0.1 - row: %s", json.encode(row))
				local andResult = false
				--LogDebug("  0.2 - AND_Query: %s", json.encode(AND_Query))
				for _, rowQuery in ipairs(AND_Query) do
					local filterInfo = {
						query = rowQuery or {}
					}
					----LogDebug("  0.3 - row: %s", json.encode(row))
					--LogDebug("  0.4 - filterInfo: %s", json.encode(filterInfo))
					if utils.MatchFilter(row, filterInfo, "AND", nil, CustomMatch) then
						andResult = true
						break
					-- else
					-- 	andResult = false
					-- 	break
					end
				end
				mustAdd = andResult
			end

			if mustAdd ~= false then
				if not isEmptyTreeQuery then
					--LogDebug("1 - voy por el TREE ---------------------")
					--LogDebug("  1.1 - row: %s", json.encode(row))
					local passedTreeFilter = utils.TreeFilter(row, treeQuery)
					--LogDebug("  1.2 - passedTreeFilter: %s", passedTreeFilter)
					if passedTreeFilter then
						mustAdd = true
					else
						mustAdd = false
					end
				end
			end
			if mustAdd ~= false then
				--local logicalOperator = "OR"
				--LogDebug("2 - voy por los OR --------------------")
				if not isEmptyOR_Query then
					local orResult = false
					--LogDebug("  2.0 - OR_Query: %s", json.encode(OR_Query))
					for _, rowQuery in ipairs(OR_Query) do
						local filterInfo = {
							query = rowQuery or {}
						}
						----LogDebug("  2.1 - row: %s", json.encode(row))
						--LogDebug("  2.2 - filterInfo: %s", json.encode(filterInfo))
						if utils.MatchFilter(row, filterInfo, "AND") then
							orResult = true
							break
						end
					end
					mustAdd = orResult
				end
			end

			if mustAdd then
				--LogDebug("3 - AGREGO")
				Add(row)
			end
			--LogDebug("-END rowId %s--------", rowId)
			--LogDebug("---------")
			--LogDebug("")
			--LogDebug("")
		end
	end

	LogInfo("[ADMINISTRATION.SHARED_FILES:Search] 3 - Sorting rows")
	--LogDebug("[ADMINISTRATION.SHARED_FILES:Search] 3.1 - filteredRows '%s'", json.encode(filteredRows))

	--include only rows that belong to logged user vcc
	local response = {
		rows = filteredRows,
		last = "",
		total_rows = totalAdded,
		total_unfiltered_rows = totalUnfilteredRows,
		total_size = totalSize,
		size_limit_mb = m_models.SharedFiles.GetSizeLimit(vcc, keyspace)
	}
	-- cierro la conexión utilizada en todos los accesos al keyspace "Agent"
	keyspace:Close()

	local responseParams = {
		MakeSortingValue = function(row)
			assert(row)
			return tostring(row.Name or row.Id):upper()
		end
	}
	local resultData = utils.MakeSearchResponse(response, postData, responseParams)

	local endTime = os.time()
	local elapsedTime = endTime - startTime
	LogInfo("[ADMINISTRATION.SHARED_FILES:Search] 4 - (END) Search rows. Retrieve %d of %d matched rows. Total Unfiltered Rows: %d. ELAPSED Time '%s' seconds.", #resultData.rows, resultData.total_rows, resultData.total_unfiltered_rows, elapsedTime)
	--LogDebug("[ADMINISTRATION.SHARED_FILES:Search] 5 - (END) Search rows. DATA: %s.", json.encode(resultData.rows))

	return resultData
end

local function Get (web, fileId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local file = m_models.SharedFiles.GetFiles(vcc, fileId)
	LogInfo("[ADMINISTRATION.SHARED_FILES:Get] Retrieving file '%s', , vcc '%s', content: '%s'.", fileId, vcc, json.encode(file))
	if next(file) then
		-- success
		file.Name = file.Name or fileId
		return file
	end
	-- error
	web.status = 404
	return {result = "NOT_FOUND"}
end

local function GetFilesByIds (web)
	local vcc = inconcert.authUser.virtualContactCenter
	LogInfo("[ADMINISTRATION.SHARED_FILES:GetFilesByIds] Loading vcc '%s' files by ids. Posted data: '%s'", vcc, web.input["post_data"])
	local postData = web.request
	local vccFiles = m_models.SharedFiles.GetFiles(vcc)
	local files = {}
	for _, vccFile in pairs(vccFiles or {}) do
		if postData[vccFile.Id:lower()] then
			files[vccFile.Id:lower()] = vccFile
		end
	end
	--LogDebug("[ADMINISTRATION.SHARED_FILES:GetFilesByIds] Loaded vcc '%s' files by ids. Posted data: '%s'", vcc, json.encode(files))
	return files
end

local function ValidateSharedFiles (vcc, files, keyspace)
	local vccConfiguredAvailableSpaceBytes = m_models.SharedFiles.GetSizeLimit(vcc, keyspace) * 1024 * 1024
	local vccAvailableSpaceBytes = vccConfiguredAvailableSpaceBytes
	local filesByName = {}

	-- 0.5 Valido que NO exista ningun otro proceso de batch en curso (exportacion, alta, baja)
	local isAnyProcessBatch = m_models.SharedFiles.IsAnyProcessBatchInProgressInVCC(vcc)
	if isAnyProcessBatch then
		-- Error overflow
		local msg = "Unable to Export. There another process batch in progress"
		local err = {
			result = "ERROR",
			result = "THERE ANOTHER PROCESS BATCH IN PROGRESS",
--			code = -105,
			reason = msg
		}
		LogError("[ADMINISTRATION.SHARED_FILES:ValidateSharedFiles] Shared Files Error. %s", json.encode(err))
		-- Is Error
		return 400, err
	end

	-- 0.6 - Creo el status del proceso en redis.
	local currentProcessBatch = {
		Id = uuid.new(),
		VCC = vcc,
		StartDate = LuaOrm.Date():toString(),
		User = inconcert.authUser.userName,
--		TotalFiles = utils.GetRowCount(files),
		Status = "RUNNING",
		Action = "VALIDATE CREATION OR UPDATE",
		SubAction = "LOADING CURRENT FILES FROM DB"
	}
	m_models.SharedFiles.UpdateVCCStatusProcessBatch(currentProcessBatch)

	-- 1 - Al espacio disponible que se ha configurado en el VCC, le resto el espacio de los archivos que ya existen en la BD.
	local existingFiles = m_models.SharedFiles.GetFiles(vcc, nil, keyspace) or {}
	for fileId, fileData in pairs(existingFiles) do
		filesByName[fileData.Name] = fileId
		vccAvailableSpaceBytes = vccAvailableSpaceBytes - tonumber(fileData.Size)
	end

	local vccCalculatedAvailableSpaceBytes = vccAvailableSpaceBytes
	local totalRows = 0
	local errors = {}
	files.isNew = nil

	-- 0.7 - Actualizo el status del proceso
	currentProcessBatch.SubAction = "VALIDATING ALL POSTED FILES"
	m_models.SharedFiles.UpdateVCCStatusProcessBatch(currentProcessBatch)

	local updateStatusEvery = 10
	local i = 0

	-- 2 - Realizo validaciones.
	local filesToSave = {}
	local filesToRemoveFromRepository

	for fileId, fileData in pairs(files) do
		fileId = fileId:lower()
		totalRows = totalRows + 1
		local err
		if type(fileData.Description) ~= "string" or fileData.Description == "" then
			LogError("[ADMINISTRATION.SHARED_FILES:ValidateSharedFiles] File %s doesnt have a description", fileId)
			err = {result = "MUST_HAVE_A_DESCRIPTION", reason = "File must have a description"}
		end
		if type(fileData.Name) ~= "string" or fileData.Name == "" then
			LogError("[ADMINISTRATION.SHARED_FILES:ValidateSharedFiles] File %s doesnt have a Name", fileId)
			err = {result = "MUST_HAVE_A_NAME", reason = "File must have a name"}
		end
		if type(fileData.RepositoryId) ~= "string" or fileData.RepositoryId == "" then
			LogError("[ADMINISTRATION.SHARED_FILES:ValidateSharedFiles] File %s doesnt have a repository id", fileId)
			err = {result = "MUST_HAVE_A_REPOSITORY_ID", reason = "File must have a repository id"}
		end
		if type(fileData.Type) ~= "string" or fileData.Type == "" then
			LogError("[ADMINISTRATION.SHARED_FILES:ValidateSharedFiles] File %s doesnt have a file type", fileId)
			err = {result = "MUST_HAVE_A_TYPE", reason = "File must have a file type"}
		end
		if type(fileData.ContentType) ~= "string" or fileData.ContentType == "" then
			LogError("[ADMINISTRATION.SHARED_FILES:ValidateSharedFiles] File %s doesnt have a content type", fileId)
			err = {result = "MUST_HAVE_A_CONTENT_TYPE", reason = "File must have a content type"}
		end
		if tostring(fileData.AccountMessagesVisible) ~= "1" and tostring(fileData.AgentVisible) ~= "1" and tostring(fileData.FlowDesignerVisible) ~= "1" then
			LogError("[ADMINISTRATION.SHARED_FILES:ValidateSharedFiles] You must select at least one Visibility option. File %s.", fileId)
			err = {result = "MUST_SELECT_AT_LEAST_ONE_VISIBILITY_OPTION", reason = "You must select at least one visibility option."}
		end

		local fileSizeBytes = tonumber(fileData.Size)
		if not fileSizeBytes or fileSizeBytes <= 0 then
			LogError("[ADMINISTRATION.SHARED_FILES:ValidateSharedFiles] File %s doesnt have a size", fileId)
			err = {result = "MUST_HAVE_A_SIZE", reason = "File must have a size"}
		end
		fileData.Size = fileSizeBytes
		vccCalculatedAvailableSpaceBytes = vccCalculatedAvailableSpaceBytes - fileSizeBytes

		if vccAvailableSpaceBytes - fileSizeBytes < 0 then
			LogError("[ADMINISTRATION.SHARED_FILES:ValidateSharedFiles] File %s by itself exceeds the available space", fileId)
			err = {result = "HAS_FILE_NOT_SPACE", reason = "File by itself exceeds the available space"}
		elseif vccCalculatedAvailableSpaceBytes < 0 then
			LogError("[ADMINISTRATION.SHARED_FILES:ValidateSharedFiles] File %s exceeds available space", fileId)
			err = {result = "HAS_BATCH_FILES_NOT_SPACE", reason = "File exceeds avaible space"}
		end

		if filesByName[fileData.Name] and fileData.is_new then
			LogError("[ADMINISTRATION.SHARED_FILES:ValidateSharedFiles] File %s name %s already exists", fileId, fileData.Name)
			err = {result = "NAME_ALREADY_EXISTS", reason = "File name already exists", fileId = filesByName[fileData.Name]}
		end

		if err then
			-- Is Error
			err.fileData = fileData
			err.id = fileId
			table.insert(errors, err)
		else
			-- Is Ok
			local currentFile = existingFiles[fileId]
			if currentFile then
				-- Actualizando un sharedFile
				fileData.Section = currentFile.Section or -1
				-- Si estoy modificando el archivo de shared files, valido si ha subido un nuevo Archivo (si ha cambiado en el Repositorio).
				if currentFile.RepositoryId ~= fileData.RepositoryId then
					-- Si el archivo SharedFile ha cambiado de archivo en el Repositorio, agrego el archivo viejo en un array para que pueda ser eliminado del Repositorio.
					filesToRemoveFromRepository = filesToRemoveFromRepository or {}
					table.insert(filesToRemoveFromRepository, {
						FileId  = currentFile.RepositoryId,
						Section = currentFile.Section or -1
					})
				end
			else
				-- Nuevo un sharedFile
				fileData.Section = fileData.Section or -1
			end

			fileData.Extension = utils.GetFileExtension(fileData.Name):lower()
			fileData.UpdatedTime = os.time()
			fileData.is_new = nil
			filesByName[fileData.Name] = fileId
			filesToSave[fileId] = fileData
		end

		-- Actualizo el status del process batch, cada N loops
		i = i + 1
		if i == updateStatusEvery then
			currentProcessBatch.NumberProcessBatchRows = totalRows
			currentProcessBatch.SubAction = "VALIDATING POSTED FILE"
			m_models.SharedFiles.UpdateVCCStatusProcessBatch(currentProcessBatch)
			i = 0
		end
	end

	local moduleImports = require "apps.administration.controllers.imports"
	local MAXIMUM_NUMBER_OF_FILES_IN_IMPORT = moduleImports.Public().SharedFilesGetMaxNumberSupportedFilesImport()

	if totalRows > MAXIMUM_NUMBER_OF_FILES_IN_IMPORT then
		-- Error overflow
		local msg = string.format("The upload must not exceed %d files. The current upload contains %d files.", MAXIMUM_NUMBER_OF_FILES_IN_IMPORT, totalRows)
		local err = {
			result = "ERROR",
			result = "UPLOAD EXCEEDS THE LIMIT OF FILES SUPPORTED",
			code = -104,
			reason = msg,
			maximumNumberRowsInImport = MAXIMUM_NUMBER_OF_FILES_IN_IMPORT,
			fileRows = totalRows,
			fileData = {}
		}
		LogError("[ADMINISTRATION.SHARED_FILES:ValidateSharedFiles] Shared Files Error. %s", json.encode(err))

		currentProcessBatch.Status = "FAIL"
		m_models.SharedFiles.UpdateVCCStatusProcessBatch(currentProcessBatch)

		-- Is Error
		return 400, err
	end

	--LogDebug("[ADMINISTRATION.SHARED_FILES:ValidateSharedFiles] -------------> Errors: %s", json.encode(errors))

	if #errors > 0 then
		LogError("[ADMINISTRATION.SHARED_FILES:ValidateSharedFiles] Total Validation errors: %d", #errors)
		-- Is Error
		return 400, errors
	end

	-- Is Ok
	return 200, {result = "OK", currentProcessBatch = currentProcessBatch, filesToSave = filesToSave, filesToRemoveFromRepository = filesToRemoveFromRepository, vcc = vcc}
end

--[[

]]--
local function UpdateSharedFiles (vcc, files, pKsAgent)
	assert(type(vcc) == "string")
	assert(type(files) == "table")
	LogInfo("[ADMINISTRATION.SHARED_FILES:UpdateSharedFiles] Updating Shared Files: %s", json.encode(files))
	local keyspace = pKsAgent or utils.GetDatasource():CassandraConnection("Agent")
	-- 1 - realizo las validaciones necesarias.
	local status, result = ValidateSharedFiles(vcc, files, keyspace)
	if status == 200 then
		-- Ok
		-- 2 - almaceno en las CF de Shared Files
		local filesToSave = result.filesToSave
		m_models.SharedFiles.SaveFiles(vcc, filesToSave, keyspace)
		result.filesToSave = nil
		if result.filesToRemoveFromRepository then
			-- 3 - si se han actualizado los archivos subiendo de shared files y su archivo en el repo ha cambiado, tengo que eliminar el viejo.
			LogInfo("[ADMINISTRATION.SHARED_FILES:SaveSharedFiles] Deleting old files from Repository.")
			RemoveRepositoryFiles(result.filesToRemoveFromRepository)
			result.filesToRemoveFromRepository = nil
		end
		if result.currentProcessBatch then
			-- 4 - elimino el proceso de batch del VCC
			DeleteVCCProcessBatch(result.currentProcessBatch.Id, vcc)
		end
	else
		-- error
		if result.result ~= "THERE ANOTHER PROCESS BATCH IN PROGRESS" then
			DeleteVCCProcessBatch(nil, vcc)
		end
	end
	if not pKsAgent then
		keyspace:Close()
	end
	return status, result
end

--[[
  Utilizado por la migracion de VCCs de Allegro a Nativos, via el controller de init.lua de la app "repository".
]]--
function ExternalUpdateSharedFiles (vcc, files, pKsAgent)
	return UpdateSharedFiles(vcc, files, pKsAgent)
end

--[[
	Add/Modifies files in a vcc
	posted data must have the form:
{
	fileId1 = {
		Description = "file description 1",
	},
	fileId2 = {
		Description = "file description 2",
	},
	...
}

]]
local function SaveSharedFiles (web)
	local vcc = inconcert.authUser.virtualContactCenter
	LogInfo("[ADMINISTRATION.SHARED_FILES:SaveSharedFiles] Saving Shared Files -> user '%s', vcc '%s', files: '%s'.", inconcert.authUser.userName, vcc, web.input["post_data"])
	local files = web.request
	local status, result = UpdateSharedFiles(vcc, files)
	web.status = status
	return result
end

local function ValidateSharedFilesWeb (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[ADMINISTRATION.SHARED_FILES:ValidateSharedFilesWeb] Validate Files, userId %s, post data:  %s", userId, web.input["post_data"])

	local files = web.request
	local keyspace = utils.GetDatasource():CassandraConnection("Agent")
	local status, result = ValidateSharedFiles(vcc, files, keyspace)
	keyspace:Close()

	if status == 200 then
		if result.currentProcessBatch then
			DeleteVCCProcessBatch(result.currentProcessBatch.Id, vcc)
		end
	else
		if result.result ~= "THERE ANOTHER PROCESS BATCH IN PROGRESS" then
			DeleteVCCProcessBatch(nil, vcc)
		end
	end

	web.status = status
	return result
end

--[[

]]
local function StartDeleteSharedFiles (files, vcc, ksAgent)
	assert(type(files) == "table")
	assert(type(vcc) == "string")
	assert(ksAgent)

	LogInfo("[ADMINISTRATION.SHARED_FILES:StartDeleteSharedFiles] Remove Shared Files -> user '%s', vcc '%s', files: '%s'.", inconcert.authUser.userName, vcc, json.encode(files))

	-- 0.5 Valido que NO exista ningun otro proceso de batch en curso (exportacion, alta, baja)
	local isAnyProcessBatch = m_models.SharedFiles.IsAnyProcessBatchInProgressInVCC(vcc)
	if isAnyProcessBatch then
		local msg = string.format("Unable to Export. There another process batch in progress.")
		LogError("[ADMINISTRATION.SHARED_FILES:StartDeleteSharedFiles] Removing Shared Files Error. %s", msg)
		response.code = "THERE ANOTHER PROCESS BATCH IN PROGRESS"
		return false, response
	end

	-- 0.6 - Creo el status del proceso en redis.
	local currentProcessBatch = {
		Id = uuid.new(),
		VCC = vcc,
		StartDate = LuaOrm.Date():toString(),
		User = inconcert.authUser.userName,
--		TotalFiles = utils.GetRowCount(files),
		Status = "RUNNING",
		Action = "VALIDATE DELETE",
		SubAction = "LOADING CURRENT FILES FROM DB"
	}
	m_models.SharedFiles.UpdateVCCStatusProcessBatch(currentProcessBatch)

	--LogDebug(" 1 - files %s", json.encode(files))

	local filesToRemoveFromRepository = {}
	local filesToRemoveFromSharedFiles = {}

	local function AddRemoveSharedFile (fileId)
		table.insert(filesToRemoveFromSharedFiles, fileId)
		local fileData = m_models.SharedFiles.GetFiles(vcc, fileId, ksAgent)
		if fileData and fileData.RepositoryId and fileData.RepositoryId ~= "" then
			table.insert(filesToRemoveFromRepository, {
				FileId  = fileData.RepositoryId,
				Section = fileData.Section or -1
			})
		else
			LogWarning("[ADMINISTRATION.SHARED_FILES:StartDeleteSharedFiles] Corrupt Shared File or Not Exist '%s'. File content: %s", fileId, json.encode(fileData))
		end
	end

	local updateStatusEvery = 10
	local i = 0
	local errors = {}
	local cachedFlowsWithErrors = {} -- cacheo los flows con error, asi no los voy a buscar a la BD todas las veces.

	--[[
		Funcion que valida si el Shared File se encuentra en Uso en alguna interacción, y si ha pasado un determinado tiempo (60 días) para así poder eliminar el Archivo.
	]]--
	local function IsUsedInInteractions (vcc, fileId)
		local response = {isUsed = false, canRemove = false}
		local lastUsageInInteractions = m_models.SharedFiles.GetLastUsageInInteractions(vcc, fileId)
		--LogDebug("[ADMINISTRATION.SHARED_FILES:StartDeleteSharedFiles] -----> fileId '%s' lastUsageInInteractions  %s", fileId, lastUsageInInteractions)

		if not lastUsageInInteractions then
			-- no utilizdo
			response.canRemove = true -- se puede eliminar
			return response
		end

		-- SI es utilizado
		response.isUsed = true

		local MAX_TIME_DAYS = 60 -- Tiempo máximo en el cual puede estar el Shared File en uso en la Interaccion, antes de que se pueda eliminar el mismo.
		local MAX_TIME_SEC = (((MAX_TIME_DAYS * 24) * 60) * 60)
		local elapsedTimeSec = utils.GetElapsedTimeSec(lastUsageInInteractions)

		--LogDebug("[ADMINISTRATION.SHARED_FILES:StartDeleteSharedFiles] ELAPSED Time '%s' minutes of use of the Shared File '%s' in some interaction. MaxTime %s days", (elapsedTimeSec / 60), fileId, MAX_TIME_DAYS)
		--LogDebug("-------------> elapsedTimeSec '%s' <= MAX_TIME_SEC '%s'  = CanRemove %s", elapsedTimeSec, MAX_TIME_SEC, elapsedTimeSec > MAX_TIME_SEC)
		if elapsedTimeSec > MAX_TIME_SEC then
			-- SI se puede eliminar
			response.canRemove = true
		end

		return response
	end

	for fileNumber, fileId in pairs(files) do
		-- valida el uso contra redis (por ejemplo cuando el mismo se ha incluído en una interacción de chat, mail, etc)
		local respUsedInteractions = IsUsedInInteractions(vcc, fileId)
		--LogDebug("[ADMINISTRATION.SHARED_FILES:StartDeleteSharedFiles] -----> %d fileId '%s' isUsedInInteractions  %s", fileNumber, fileId, json.encode(respUsedInteractions))

		if respUsedInteractions.canRemove then
			-- valida contra la relacion de (campaign account messages, flow desinger, etc)
			local sharedFileRelation = m_models.SharedFiles.GetSharedFilesRelations(vcc, fileId, ksAgent)
			--LogDebug("[ADMINISTRATION.SHARED_FILES:StartDeleteSharedFiles] -----> File '%s' used in sharedFileRelation %s", fileId, json.encode(sharedFileRelation or {}))

			local isUsed = false

			if not utils.TableIsEmpty(sharedFileRelation) then
				local totalUsedInAccountMessages = utils.Cardinality(sharedFileRelation.CampaignAccountMessages)
				if totalUsedInAccountMessages > 0 then
					isUsed = true
					LogWarning("[ADMINISTRATION.SHARED_FILES:StartDeleteSharedFiles] Unable to remove file '%s'. File in used in %d Campaign Accounts Messages: %s", fileId, totalUsedInAccountMessages, json.encode(sharedFileRelation.CampaignAccountMessages or {}))
					table.insert(errors, {
						fileId = fileId,
						errType = "FILE USED IN CAMPAIGN ACCOUNTS",
						totalUsedInAccountMessages = totalUsedInAccountMessages,
						errInfo = sharedFileRelation.CampaignAccountMessages
					})
				end
				local totalUsedInFlowDesigner = utils.Cardinality(sharedFileRelation.FlowDesigner)
				if totalUsedInFlowDesigner > 0 then
					-- Obtengo los nombres de los Flows a partir de us ID.
					isUsed = true
					for flowId in pairs(sharedFileRelation.FlowDesigner) do
						local cachedFlowWithErr = cachedFlowsWithErrors[flowId]
						if not cachedFlowWithErr then
							local flow = m_models.ProcessFlows.GetFlows(vcc, flowId, {"Id", "Name"}, ksAgent)
							cachedFlowWithErr = flow
							cachedFlowsWithErrors[flowId] = cachedFlowWithErr
						end
						sharedFileRelation.FlowDesigner[flowId].flowName = cachedFlowWithErr.Name
					end
					LogWarning("[ADMINISTRATION.SHARED_FILES:StartDeleteSharedFiles] Unable to remove file '%s'. File in used in %d Flow Designer: %s", fileId, totalUsedInFlowDesigner, json.encode(sharedFileRelation.FlowDesigner or {}))
					table.insert(errors, {
						fileId = fileId,
						errType = "FILE USED IN FLOW DESIGNER",
						totalUsedInFlowDesigner = totalUsedInFlowDesigner,
						errInfo = sharedFileRelation.FlowDesigner
					})
				end
			end

			if not isUsed then
				-- Si no es utilizado en las CFs relacionadas lo agrego en la lista de SharedFiles a remover
				AddRemoveSharedFile(fileId)
			end
		else
			table.insert(errors, {
				fileId = fileId,
				errType = "FILE USED IN INTERACTIONS",
				lastUsageInInteractions = lastUsageInInteractions
			})
			LogWarning("[ADMINISTRATION.SHARED_FILES:StartDeleteSharedFiles] Unable to remove file '%s'. File in used on Interactions. lastUsageInInteractions: %s", fileId, lastUsageInInteractions)
		end
		-- Actualizo el status del process batch, cada N loops
		i = i + 1
		if i == updateStatusEvery then
			currentProcessBatch.NumberProcessBatchRows = fileNumber
			currentProcessBatch.SubAction = "GET FILE"
			m_models.SharedFiles.UpdateVCCStatusProcessBatch(currentProcessBatch)
			i = 0
		end
	end

	--LogDebug(" 2 - filesToRemoveFromSharedFiles %s", json.encode(filesToRemoveFromSharedFiles))
	--LogDebug(" 3 - filesToRemoveFromRepository %s", json.encode(filesToRemoveFromRepository))
	--LogDebug(" 4 - errors %s", json.encode(errors))

	if #errors == #files then
		-- Elimino el status del current process
		DeleteVCCProcessBatch(currentProcessBatch.Id, vcc)
		LogError("[ADMINISTRATION.SHARED_FILES:StartDeleteSharedFiles] No file could be deleted.")
		response.code = "NO FILE COULD BE DELETED"
		response.errors = errors
		return false, response
	end

	if #filesToRemoveFromSharedFiles == 0 then
		-- Elimino el status del current process
		DeleteVCCProcessBatch(currentProcessBatch.Id, vcc)
		LogError("[ADMINISTRATION.SHARED_FILES:StartDeleteSharedFiles] The shared files to delete do not exist in the database.")
		response.code = "SHARED FILES TO DELETE DO NOT EXIST IN DATABASE"
		response.errors = errors
		return false, response
	end

	-- Remuevo de las CFs de Shared Files
	currentProcessBatch.SubAction = "REMOVING SHARED FILES"
	m_models.SharedFiles.UpdateVCCStatusProcessBatch(currentProcessBatch)
	m_models.SharedFiles.RemoveFiles(vcc, filesToRemoveFromSharedFiles, ksAgent)

	-- Remuevo de las CFs del Repositorio
	currentProcessBatch.SubAction = "REMOVING REPOSITORY FILES"
	m_models.SharedFiles.UpdateVCCStatusProcessBatch(currentProcessBatch)
	RemoveRepositoryFiles(filesToRemoveFromRepository)

	-- Elimino el status del current process
	DeleteVCCProcessBatch(currentProcessBatch.Id, vcc)

	if #errors > 0 then
		LogWarning("[ADMINISTRATION.SHARED_FILES:StartDeleteSharedFiles] %d files could not be deleted.", #errors)
		response.code = "SOME FILES COULD NOT BE DELETED"
		response.errors = errors
		response.totalDeleted = #filesToRemoveFromSharedFiles
		response.totalNotDeleted = #errors
		return false, response
	end

	-- OK complete
	return true
end

--[[
	Utilizado por el rollback de VCCs de Nativos a Allegro, via el controller de la app "/apps/provisioning/migrations/campaigns.lua".
]]--
function ExternalDeleteSharedFiles (files, vcc, ksAgent)
	return StartDeleteSharedFiles(files, vcc, ksAgent)
end

--[[
Remove files from a vcc
postedData must be of the form: ["fileId1", "fileId2", ...]
]]
local function RemoveSharedFiles (web)
	local vcc = inconcert.authUser.virtualContactCenter
	LogInfo("[ADMINISTRATION.SHARED_FILES:RemoveSharedFiles] Remove Shared Files -> user '%s', vcc '%s', files: '%s'.", inconcert.authUser.userName, vcc, web.input["post_data"])
	local files = web.request
	local ksAgent = utils.GetDatasource():CassandraConnection("Agent")
	local success, response = StartDeleteSharedFiles(files, vcc, ksAgent)
	ksAgent:Close()
	if not success then
		-- error
		web.status = 500
		return response
	end
	-- OK complete
	web.status = 200
	return {result = "OK"}
end

local function DownloadSharedFile (wsapi_env, fileId)
	local vcc = inconcert.authUser.virtualContactCenter
	StartRepository()
	return repositoryController.retrieveFile(wsapi_env,
		function(request)
			LogInfo("[ADMINISTRATION.SHARED_FILES:DownloadSharedFile] Download Shared File -> user '%s', vcc '%s', file: '%s'.", inconcert.authUser.userName, vcc, fileId)
			local fileData = m_models.SharedFiles.GetFiles(vcc, fileId)
			if not next(fileData) then
				return 404, "not found"
			end
			LogDebug("[ADMINISTRATION.SHARED_FILES:DownloadSharedFile] fileData: '%s'.", json.encode(fileData))
			request.params.id = fileData.RepositoryId
			request.params.name = fileData.Name
			request.params.section = fileData.Section or -1
			request.params.part = -1
			return 200
		end
	)
end

local function ExportSharedFileToJSON (wsapi_env, fileId)
	local vcc = inconcert.authUser.virtualContactCenter
	StartRepository()
	LogInfo("[ADMINISTRATION.SHARED_FILES:ExportSharedFileToJSON] Export Shared File to JSON -> user '%s', vcc '%s', file: '%s'.", inconcert.authUser.userName, vcc, fileId)
	local fileData = m_models.SharedFiles.GetFiles(vcc, fileId)
	if not next(fileData) then
		web.status = 404
		return {result = "NOT_FOUND"}
	end
	local base64Content = true
	local repositoryFile, err = repositoryController.getFile(fileData.RepositoryId, fileData.Section, -1, base64Content)
	if not repositoryFile then
		web.status = 404
		return {result = "NOT_FOUND_IN_REPOSITORY", reason = err}
	end
	fileData.content = repositoryFile.content
	web.headers["Content-Disposition"] = [[attachment; filename="]] .. fileData.Name .. '.json"'
	return fileData
end

--[[
	Funcion que comprime a ZIP los archivos del BATCH
]]--
local function Compress (files)
	local vcc = inconcert.authUser.virtualContactCenter
	local SOURCE_ROOT_PATH_TEMPORAL_FILES = "/tmp/"

	LogInfo("[ADMINISTRATION.SHARED_FILES:Compress] (START) Compressing")

	-- 1 - Creo la estructura BASE Temporal de directorios /tmp/shared_files/export/.
	--
	local sharedFilesPath = SOURCE_ROOT_PATH_TEMPORAL_FILES .. "sharedFiles/"
	local command = "mkdir " .. sharedFilesPath
	--LogDebug("0.2 --> command " .. tostring(command))
	os.execute(command)

	local exportSharedFilesPath = sharedFilesPath .. "export/"
	command = "mkdir " .. exportSharedFilesPath
	--LogDebug("0.3 --> command " .. tostring(command))
	os.execute(command)

	-- 2 - Genero el nombre del archivo ZIP (prefijo + fecha y hora en utc + el vcc) 
	--      SHARED_FILES_2021-06-15_12-00-00_UTC_VCC_TU_VCC
	--
	local fileName = ("SHARED_FILES_%s_UTC_VCC_%s"):format(
		LuaOrm.Date():toString("%Y-%m-%d_%H-%M-%S"),
		vcc:upper()
	)

	local zipDir = exportSharedFilesPath .. fileName .. "/"
	command = "mkdir " .. zipDir
	os.execute(command)

	-- 3 - Crea la estructura DESTINO de exportacion actual /tmp/shared_files/export/SHARED_FILES_2021-06-15_12-00-00_UTC_VCC_TU_VCC/files/.
	--
	local zipFilesDir = zipDir .. "files/"
	command = "mkdir " .. zipFilesDir
	LogInfo("[ADMINISTRATION.SHARED_FILES:Compress] Compressing .. creating temporal directory: %s", zipFilesDir)
	os.execute(command)

	-- 4 - Creando los archivos Temporales en el directorio Temporal, que luego seran compactados.
	--
	for i, file in ipairs(files) do
		local fullFileName = zipFilesDir .. file.Name
		--LogDebug("fullFileName %s", fullFileName)
		utils.CreateFile(fullFileName, file.Content)
	end
	LogInfo("[ADMINISTRATION.SHARED_FILES:Compress] Compressing.. created temporal %s files to compress.", #files)

	-- 5 - Creo el archivo "files.CSV" con la información de todos los archivos a exportar que quedaran en el archivo comprimdio .ZIP.
	--     A la definicion de las columnas permitidas del CSV las obtiene desde el controller "importer"
	--     El archivo "files.CSV" quedara dentro del archivo .ZIP en el directorio raíz.
	local moduleImports = require "apps.administration.controllers.imports"
	local csvAllowedColumns = moduleImports.Public().SharedFilesGetMappingFields("HASH_BY_CASSANDRA_FIELD_NAME")
	local csv = utils.Table2Csv(files, csvAllowedColumns)
	if csv then
		local csvFullFileName = zipDir .. "files.csv"
		utils.CreateFile(csvFullFileName, csv)
	end

	-- 6 - Cominezo con la compresion de los archivos.
	--
	local loadContent = true
	local success, zipFileInfo = utils.CompressFiles(fileName, zipDir, loadContent)

	if not success then
		-- 7 - Remuevo el directorio temporal
		utils.RemoveDirectory(zipDir)
		LogError("[ADMINISTRATION.SHARED_FILES:Compress] (END) Compressing Error: %s", json.encode(err))
		local err = zipContent
		return false, err
	end

	LogInfo("[ADMINISTRATION.SHARED_FILES:Compress] (END) Compressing OK")
	return true, zipFileInfo
end

--......

-- El cantidad máxima de archivos que se pueden incluir en una exportación.
local MAXIMUM_NUMBER_OF_FILES_IN_EXPORT = 500 -- El valor -1 no valida un máximo
-- El cantidad máxima de archivos que se pueden incluir en una exportación.
local MAXIMUM_COMPRESS_FILE_SIZE_BYTES = 50 * 1024 * 1024 -- El valor -1 no valida un máximo

local function ValidateAndCompressExportFiles (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local fullUserId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	StartRepository()

	local startTime = os.time()

	LogInfo("[ADMINISTRATION.SHARED_FILES:ValidateAndCompressExportFiles] 1 - (START) Validate & Compress Export Files.")
	--LogDebug("[ADMINISTRATION.SHARED_FILES:ValidateAndCompressExportFiles] Validating Exporting Shared Files: web.request: '%s'", json.encode(web.request or {}))

	local response = {
		success = false
	}
	-- 0.5 Valido que NO exista ningun otro proceso de batch en curso (exportacion, alta, baja)
	local isAnyProcessBatch = m_models.SharedFiles.IsAnyProcessBatchInProgressInVCC(vcc)
	if isAnyProcessBatch then
		local msg = string.format("Unable to Export. There another process batch in progress.")
		LogError("[ADMINISTRATION.SHARED_FILES:ValidateAndCompressExportFiles] Shared Files Error. %s", msg)
		response.code = "THERE ANOTHER PROCESS BATCH IN PROGRESS"
		web.status = 500
		return response
	end

	local postData = web.request

	-- 1 - valido la que no supere los N archivos a exportar.
	local totalRows = #postData.files
	if MAXIMUM_NUMBER_OF_FILES_IN_EXPORT > -1 and totalRows > MAXIMUM_NUMBER_OF_FILES_IN_EXPORT then
		-- Error overflow
		local msg = string.format("The export must not exceed %d files. The current export contains %d files.", MAXIMUM_NUMBER_OF_FILES_IN_EXPORT, totalRows)
		LogError("[ADMINISTRATION.SHARED_FILES:ValidateAndCompressExportFiles] Shared Files Error. %s", msg)
		response.code = "EXPORT EXCEEDS THE LIMIT OF FILES SUPPORTED"
		response.message = msg
		response.maxNumberOfFilesInExport = MAXIMUM_NUMBER_OF_FILES_IN_EXPORT
		response.totalRows = totalRows
		web.status = 500
		return response
	end

	-- 2 - los archivos recibidos en el request, que deberan ser validados, exportados y compactados en un ZIP.
	local filesToCompress = postData.files
	local files = {}

	local agentKeySpace = utils.GetDatasource():CassandraConnection("Agent")
	local repoKeySpace = utils.GetDatasource():CassandraConnection("Repository")

	local currentProcessBatch = {
		Id = uuid.new(),
		VCC = vcc,
		StartDate = LuaOrm.Date():toString(),
		User = inconcert.authUser.userName,
		TotalFiles = #filesToCompress,
		Status = "RUNNING",
		Action = "VALIDATE EXPORTATION",
		SubAction = "STARTING"
	}
	m_models.SharedFiles.UpdateVCCStatusProcessBatch(currentProcessBatch)

	local updateStatusEvery = 10
	local i = 0

	for _, fileId in ipairs(filesToCompress) do
		-- 2.1 - obtengo la informacion desde FileShared
		local fileData = m_models.SharedFiles.GetFiles(vcc, fileId, agentKeySpace)
		if not next(fileData) then
			agentKeySpace:Close()
			repoKeySpace:Close()
			LogError("[ADMINISTRATION.SHARED_FILES:ValidateAndCompressExportFiles] Unable to Export. File '%s' does not exist in File Shared.", fileId)

			currentProcessBatch.Status = "FAIL"
			m_models.SharedFiles.UpdateVCCStatusProcessBatch(currentProcessBatch)

			response.code = "SHARED FILE NOT FOUND"
			response.file = fileId
			web.status = 404
			return response
		end

		-- 2.2 - obtengo el contenido del archivo desde el repositorio
		local base64Content = false
		local repositoryFile, err = repositoryController.getFile(fileData.RepositoryId, fileData.Section, -1, base64Content, repoKeySpace)
		if not repositoryFile then
			agentKeySpace:Close()
			repoKeySpace:Close()
			LogError("[ADMINISTRATION.SHARED_FILES:ValidateAndCompressExportFiles] Unable to Export. File '%s' does not exist in repository.", fileData.RepositoryId)

			currentProcessBatch.Status = "FAIL"
			m_models.SharedFiles.UpdateVCCStatusProcessBatch(currentProcessBatch)

			response.code = "REPOSITORY FILE NOT FOUND"
			response.file = fileId
			response.repositoryId = fileData.RepositoryId
			web.status = 404
			return response
		end

		fileData.Name = fileData.Name or fileId
		fileData.Content = repositoryFile.content

		table.insert(files, fileData)

		-- Actualizo el status del process batch, cada N loops
		i = i + 1
		if i == updateStatusEvery then
			currentProcessBatch.NumberProcessBatchRows = #files
			currentProcessBatch.SubAction = "VALIDATING FILES"
			m_models.SharedFiles.UpdateVCCStatusProcessBatch(currentProcessBatch)
			i = 0
		end
	end

	agentKeySpace:Close()
	repoKeySpace:Close()

	-- 3 - Envio a comprimir los archivos.
	currentProcessBatch.SubAction = "COMPRESSING"
	m_models.SharedFiles.UpdateVCCStatusProcessBatch(currentProcessBatch)

	local success, zipFileInfo = Compress(files)
	zipFileInfo.content = nil -- tengo que borrar el content, ya que el mismo se obtiene en la función "StartExportSharedFileToZIP"

	if not success then
		local err = zipFileInfo
		LogError("[ADMINISTRATION.SHARED_FILES:ValidateAndCompressExportFiles] Unable to Export. Err: %s.", json.encode(err))

		currentProcessBatch.Status = "FAIL"
		m_models.SharedFiles.UpdateVCCStatusProcessBatch(currentProcessBatch)

		response.code = "UNABLE TO COMPRESS FILES TO EXPORT"
		response.err = err
		web.status = 500
		return response
	end

	-- 4 - valido que el tamaño del archivo ZIP NO supere el tamaño máximo permitido.
	if MAXIMUM_COMPRESS_FILE_SIZE_BYTES > -1 and zipFileInfo.size > MAXIMUM_COMPRESS_FILE_SIZE_BYTES then
		-- Error overflow
		--
		-- Remuevo el directorio temporal
		utils.RemoveDirectory(zipFileInfo.directory)
		local msg = string.format("The export ZIP must not exceed %d Bytes. The current export ZIP is of %d Bytes.", MAXIMUM_COMPRESS_FILE_SIZE_BYTES, zipFileInfo.size)
		LogError("[ADMINISTRATION.SHARED_FILES:ValidateAndCompressExportFiles] Shared Files Error. %s", msg)

		currentProcessBatch.Status = "FAIL"
		m_models.SharedFiles.UpdateVCCStatusProcessBatch(currentProcessBatch)

		response.code = "EXPORT EXCEEDS THE COMPRESS FILE SIZE LIMIT SUPPORTED"
		response.message = msg
		response.maxFileSizeBytes = MAXIMUM_COMPRESS_FILE_SIZE_BYTES
		response.fileSizeBytes =  zipFileInfo.size
		web.status = 500
		return response
	end

	-- 5 - Ok --
	web.status = 200

	local endTime = os.time()
	local elapsedTime = endTime - startTime
	response.zipInfo = zipFileInfo
	response.success = true
	response.zipInfo = zipFileInfo
	response.batchProcessId = currentProcessBatch.Id
	-- 6 - actualizo el current status
	currentProcessBatch.SubAction = "END"
	m_models.SharedFiles.UpdateVCCStatusProcessBatch(currentProcessBatch)

	LogInfo("[ADMINISTRATION.SHARED_FILES:ValidateAndCompressExportFiles] 2 - (END) Validation Compress Exporting total of %s shared files. ELAPSED Time '%s' seconds. Response: %s", totalRows, elapsedTime, json.encode(response))

	return response
end

--[[
	Funcion que realiza la Exportacion en BATCH comprimiendo a ZIP.
]]--
function StartExportSharedFileToZIP (wsapi_env, withContentType)
	local vcc = inconcert.authUser.virtualContactCenter
	local request = wsapi.request.new(wsapi_env)
	local response = wsapi.response.new(200, {["Content-Type"] = "application/octet-stream"})
	local MakeResponse = function(status, reason) return reason or "" end

	local headers = {}
	local startTime = os.time()

	LogInfo("[ADMINISTRATION.SHARED_FILES:StartExportSharedFileToZIP] 1 - (START) Exporting Shared Files")
--LogDebug("[ADMINISTRATION.SHARED_FILES:StartExportSharedFileToZIP] Exporting Shared Files: request: '%s'", json.encode(request or {}))

	local input_readers = {
		GET = function()
			--LogDebug("ADMINISTRATION.SHARED_FILES] GET data: '%s'", request.params.data)
			return pcall(json.decode, request.params.data)
		end
	}

	if not input_readers[request.method] then
		LogError("[ADMINISTRATION.SHARED_FILES:StartExportSharedFileToZIP] Unable to Export. Invalid request.")

		local currentProcessBatch = {Status = "FAIL", VCC = vcc}
		m_models.SharedFiles.UpdateVCCStatusProcessBatch(currentProcessBatch)

		response.headers["Response"] = "INVALID_REQUEST"
		response.status = 400
		response:write("")
		return response:finish()
	end

	local res, queryParams = input_readers[request.method]()
	if not res or not queryParams then
		LogError("[ADMINISTRATION.SHARED_FILES:StartExportSharedFileToZIP] Unable to Export. Invalid request, paramaters does not exist.")

		local currentProcessBatch = {Status = "FAIL", VCC = vcc}
		m_models.SharedFiles.UpdateVCCStatusProcessBatch(currentProcessBatch)

		response.headers["Response"] = "INVALID_REQUEST"
		response.status = 400
		response:write("")
		return response:finish()
	end

	-- La busco en REDIS
	local encodedProcessBatchStatus = m_models.SharedFiles.GetVCCStatusProcessBatch(vcc)
	local postedProcessBatchId = queryParams.batchProcessId
	--LogDebug("---> BD postedProcessBatchId: '%s', running batch status: '%s'", postedProcessBatchId, encodedProcessBatchStatus)
	local ok, currentProcessBatch = pcall(json.decode, encodedProcessBatchStatus)
	if not ok then
		currentProcessBatch = nil
	end
	if not currentProcessBatch or (currentProcessBatch.Id or ""):lower() ~= postedProcessBatchId:lower() then
		LogError("[ADMINISTRATION.SHARED_FILES:StartExportSharedFileToZIP] Unable to Export. Process batch Id '%s' does not exist.", postedProcessBatchId)

		local currentProcessBatch = {Status = "FAIL", VCC = vcc}
		m_models.SharedFiles.UpdateVCCStatusProcessBatch(currentProcessBatch)

		response.headers["Response"] = "PROCESS BATCH ID DOES NOT EXIST"
		response.status = 400
		response:write("")
		return response:finish()
	end

	local zipInfo = queryParams.zipInfo
	if not zipInfo then
		LogError("[ADMINISTRATION.SHARED_FILES:StartExportSharedFileToZIP] Unable to Export. Invalid request, zip file info does not exist.")

		local currentProcessBatch = {Status = "FAIL", VCC = vcc}
		m_models.SharedFiles.UpdateVCCStatusProcessBatch(currentProcessBatch)

		response.headers["Response"] = "INVALID_REQUEST"
		response.status = 400
		response:write("")
		return response:finish()
	end

	-- Abro el archivo comprimido.
	local success, contentZipFile = utils.ReadFile(zipInfo.fullFileName)
	--LogDebug("--> contentZipFile: %s", contentZipFile)
	if not success then
		local err = contentZipFile
		LogError("[ADMINISTRATION.SHARED_FILES:StartExportSharedFileToZIP] Unable to Export. Err: %s.", json.encode(err))

		local currentProcessBatch = {Status = "FAIL", VCC = vcc}
		m_models.SharedFiles.UpdateVCCStatusProcessBatch(currentProcessBatch)

		response.headers["Response"] = "UNABLE_TO_OPEN_COMPRESSED_EXPORT_FILE"
		response.status = 500
		response:write("")
		return response:finish()
	end

	-- Remuevo el directorio temporal
	utils.RemoveDirectory(zipInfo.directory)

	-- nota: estos headers response estan mal! para señalar el status se usa, justamente, el status
	headers["Response"]            = "defaultError"
	headers["Content-Type"]        = "application/octet-stream"
	headers["Content-Length"]      = zipInfo.size
	headers["Response"]            = "ok"
	headers["Content-Disposition"] = [[attachment; filename="]] .. zipInfo.fileName .. '"'
	headers["Set-Cookie"]          = "exported_started=true; Path=/" -- used by the jquery download plugin

	local endTime = os.time()
	local elapsedTime = endTime - startTime
	LogInfo("[ADMINISTRATION.SHARED_FILES:StartExportSharedFileToZIP] 2 - (END) Exporting total of %s shared files. Zip File Name '%s' Size '%s'. ELAPSED Time '%s' seconds.", totalRows, zipInfo.fileName, zipInfo.size, elapsedTime)

	local coroutineIter = 0
	return 200, headers, coroutine.wrap(function()
		coroutineIter = coroutineIter + 1
		--LogDebug("[ADMINISTRATION.SHARED_FILES:StartExportSharedFileToZIP] coroutine.yield - iter:%d", coroutineIter)
		coroutine.yield(contentZipFile)
	end)
end

--[[
	,,,,,,,,,,,,,,
]]--
function EndedExportSharedFileToZIP (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local postData = web.request
	LogInfo("[ADMINISTRATION.SHARED_FILES:EndedExportSharedFileToZIP] Ended Compress Export Files. web.request %s", web.input["post_data"])
	DeleteVCCProcessBatch(postData.batchProcessId, vcc)
	return {}
end

local function GetUsedSpace (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local existingFiles = m_models.SharedFiles.GetFiles(vcc)
	local usedSpace = 0
	for fileId, fileData in pairs(existingFiles) do
		usedSpace = usedSpace + tonumber(fileData.Size)
	end
	return {result = "OK", used_space = usedSpace}
end

function MapUris (module, htmlRender, prefix, appLocation)
	local logR = require("apps.agent.requestFilters").LogRequest
	local jr = utils.JsonReply
	local jvr = utils.JsonValidatorReply
	local Perm = function (permId, fn)
		return permissions:CheckPermission("administration", permId, fn)
	end

	module:dispatch_get  (Render                                                              , prefix .. "/frm/render/"             )
	module:dispatch_get  (jr   .. logR..GetUsedSpace                                          , prefix .. "/get_used_space/(.*)"     )
	module:dispatch_get  (jr   .. logR..Perm("view_shared_files", Get)                        , prefix .. "/get/(.+)"                )
	module:dispatch_wsapi(logR .. DownloadSharedFile                                          , prefix .. "/download/(.+)"           )
	module:dispatch_post (jvr  .. logR .. Search                                              , prefix .. "/search/"                 )
	module:dispatch_post (jvr  .. logR .. GetFilesByIds                                       , prefix .. "/get_files_by_ids/"       )
	module:dispatch_post (jvr  .. logR .. Perm("save_shared_files", SaveSharedFiles)          , prefix .. "/save/"                   )
	module:dispatch_post (jvr  .. logR .. Perm("save_shared_files", ValidateSharedFilesWeb)   , prefix .. "/validate/"               )
	module:dispatch_post (jvr  .. logR .. Perm("delete_shared_files", RemoveSharedFiles)      , prefix .. "/delete/"                 )
	-- export
	module:dispatch_post (jvr  .. logR .. ValidateAndCompressExportFiles                      , prefix .. "/can_export/"             )
	module:dispatch_wsapi(logR .. StartExportSharedFileToZIP                                  , prefix .. "/export_to_compressed/"   )
	module:dispatch_post (jvr  .. logR .. EndedExportSharedFileToZIP                          , prefix .. "/ended_compressed_export/")

	module:dispatch_get  (jr   .. logR .. Perm("export_shared_files", ExportSharedFileToJSON) , prefix .. "/export/(.+)"             )

	m_htmlRender = htmlRender
	m_appLocation = appLocation
end
