local m_models = require("apps.administration.model"):GetModels()
module(..., package.seeall, inconcert.orbit.application.Decorate)
local utils = require "shared.apps.utils"
local permissions = require "shared.permissions.permissions"
local notificationsEngine = require("apps.agent.notifications")
local m_moduleLocation = select(2, ...)
local m_validations = require("apps.administration.controllers.validations.skill").GetDefinitions()
m_htmlRender = nil
m_appLocation = nil

function MapUris(module, htmlRender, prefix, appLocation)
	local LogRequest = require("apps.agent.requestFilters").LogRequest

	module:dispatch_post(utils.JsonValidatorReply..LogRequest..Search, prefix.."/search/")
	module:dispatch_get(utils.JsonReply..LogRequest..permissions:CheckPermission("administration", "viewskill", Get), prefix.."/get/(.+)")
	module:dispatch_post(utils.JsonValidatorReply..LogRequest..permissions:CheckPermission("administration", "saveskill",
			utils.WebJsonValidatorData(
				m_validations.basic,
				SaveSkills
			)
		),
		prefix.."/save/"
	)
	module:dispatch_post(utils.JsonValidatorReply..LogRequest..permissions:CheckPermission("administration", "deleteskill", RemoveSkills), prefix.."/delete/")
	module:dispatch_get(utils.JsonReply..LogRequest.. RemoveValidation, prefix.."/delete_validation/(.+)")
	module:dispatch_get(Render, prefix .. "/frm/render/")

	m_htmlRender = htmlRender
	m_appLocation = appLocation
end

function Render(web)
	_M.response.DontCache(web)

	local env = { _T = m_htmlRender.extension.makeTextTranslator() }
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc

	LogInfo("Render form Skills for user %s, group %s", userId, inconcert.authUser.userGroup)

	local html_file = m_appLocation .. "/pages/frm_skill_content.html"
	local content = m_htmlRender:RenderPageLP(html_file, web, env)
	if not content then
		LogError("HTML Skills does not exist: %s", html_file)
	end
	return content
end

--Returns all skills in the VCC
function Search(web)
	local _T = m_htmlRender.extension.makeTextTranslator()
	local vcc = inconcert.authUser.virtualContactCenter
	local fullUserId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local startTime = os.time()

	LogInfo("[ADMINISTRATION.SKILLS:SEARCH] 1 - (START) Search rows for user '%s', group '%s'. Posted data: '%s'", fullUserId, inconcert.authUser.userGroup, web.input["post_data"])

	local postData = web.request
	--Get all skill from model
	local rows = m_models.Skills.GetSkills(vcc)
	local totalUnfilteredRows = utils.GetRowCount(rows)
	local totalAdded = 0
	local filteredRows = {}

	--include only skills that belong to logged user vcc
	LogInfo("[ADMINISTRATION.SKILLS:SEARCH] 2 - Filtering rows")

	for skillId, skill in pairs(rows) do
		skillId = utils.GetId(skillId)
		skill.Id = skillId
		skill.Name = skill.Name or skillId
		skill.DefaultValue = tonumber(skill.DefaultValue) or 0
		skill.NameLabel = _T(skill.Name)
		skill.IdLabel = _T(skill.Id)
		if utils.MatchFilter(skill, postData) then
			totalAdded = totalAdded + 1
			table.insert(filteredRows, skill)
		end
	end

	LogInfo("[ADMINISTRATION.SKILLS:SEARCH] 3 - Sorting rows")

	--include only rows that belong to logged user vcc
	local response = {
		rows = filteredRows,
		last = "",
		total_rows = totalAdded,
		total_unfiltered_rows = totalUnfilteredRows
	}
	local responseParams = {
		MakeSortingValue = function(row)
			assert(row)
			return tostring(row.Name or row.Id):upper()
		end
	}
	local resultData = utils.MakeSearchResponse(response, postData, responseParams)

	local endTime = os.time()
	local elapsedTime = endTime - startTime
	LogInfo("[ADMINISTRATION.SKILLS:SEARCH] 4 - (END) Search rows. Retrieve %d of %d matched rows. Total Unfiltered Rows: %d. ELAPSED Time '%s' seconds.", #resultData.rows, resultData.total_rows, resultData.total_unfiltered_rows, elapsedTime)

	return resultData
end

function Get(web, skillId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc
	LogInfo("Retrieving skillId %s, user %s, vcc %s", skillId, userId, vcc)
	local skills = m_models.Skills.GetSkills(vcc, skillId)
	LogInfo("Skill %s: %s", skillId, json.encode(skills))

	if next(skills) then
		skills.Name = skills.Name or skillId
		return skills
	else
		web.status = 404
		return {result = "NOT_FOUND"}
	end
end

--[[
	Add/Modifies skills in a vcc
	posted data must have the form:
{
	skillId1 = {
		Description = "skill description 1",
		DefaultValue = "1", //must be a numeric string
	},
	skillId2 = {
		Description = "skill description 2",
		DefaultValue = "4", 
	},
	...
}

]]
function SaveSkills(web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc

	LogInfo("[ADMINISTRATION.SaveSkills], user %s, vcc %s", userId, vcc)

	local keyspace = utils.GetDatasource():CassandraConnection( "Agent" )
	--Get all skill from model
	local existingSkills = m_models.Skills.GetSkills(vcc, nil, keyspace)
	local postData = web.request
	local idMapping = {}

	local skills = {}
	local newSkills = {}
	for skillId, skill in pairs(postData) do
		if type(skill.Description) ~= "string" then
			keyspace:Close()
			web.status = 400
			LogError("[ADMINISTRATION.SaveSkills] Skill %s doesnt have a description", skillId)
			return {result = "MUST_HAVE_A_DESCRIPTION", reason = "Skill must have a description"}
		end
		if type(skill.DefaultValue) ~= "number" then
			keyspace:Close()
			web.status = 400
			LogError("[ADMINISTRATION.SaveSkills] Skill %s must have an integer DefaultValue", skillId)
			return {result = "MUST_HAVE_DEFAULT", reason = "Skill must have a valid DefaultValue"}
		end
		if type(skill.Name) ~= "string" then
			keyspace:Close()
			web.status = 400
			LogError("[ADMINISTRATION.SaveSkills] Skill %s doesnt have a Name", skillId)
			return {result = "MUST_HAVE_A_NAME", reason = "Skill must have a name"}
		end

		--If this is a new entity, try to find an unused Id for it
		if skill.isNew then
			local n = 0
			repeat
				n = n + 1
				local repeated = utils.FilterEntities(existingSkills, function(id) return id == utils.Id_n(skillId, n) end)
			until not next(repeated)
			idMapping[skillId] = utils.Id_n(skillId, n)
			skillId = idMapping[skillId]
			skill.isNew = nil
			newSkills[skillId] = true
		else
			--modification of an existing entity, must ensure that the entity already exists
			local exists = false
			utils.FilterEntities(existingSkills, function(id) exists = exists or id == skillId end)
			if not exists then
				keyspace:Close()
				web.status = 400
				LogError("[ADMINISTRATION.SaveSkills] Skill %s does not exist", skillId)
				return {result = "ID_DOES_NOT_EXIST", reason = "Id does not exist"}
			end
		end

		--Check repeated skills (those with same Id or Name)
		local repeated = utils.FilterEntities(existingSkills, {Name = skill.Name})
		for id in pairs(repeated) do
			if id ~= skillId then
				keyspace:Close()
				web.status = 400
				LogError("[ADMINISTRATION.SaveSkills] Skill %s has repeated Name or Id with %s", skillId, id)
				return {result = "MUST_HAVE_UNIQUE_NAME", reason = "Skill must have an unique name"}
			end
		end

		existingSkills[skillId] = skill
		skills[skillId] = skill
	end

	local result
	local success, err = m_models.Skills.SaveSkills(vcc, skills, keyspace)
	if success then
		result = {result = "OK", idMapping = idMapping}
		
		--notifications
		for skillId, skill in pairs(skills) do
			skill.Id = skillId
			notificationsEngine.SendNotification(newSkills[skillId] and "SkillCreated" or "SkillUpdated", 0, 
				{ skill = skill })
		end
	else
		web.status = 400
		LogError("[ADMINISTRATION.SaveSkills] Saving skills error: %s.", json.encode(err))
		result = err
	end

	keyspace:Close()
	return result
end

local function IsUsedSkill(skill_id)

	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc

	LogInfo("[ADMINISTRATION.IsUsedSkill] Validating is used skill removing skill '%s'. User '%s'.", skill_id, userId)

	local keyspace = utils.GetDatasource():CassandraConnection( "Agent" )

	local skills = m_models.Skills.GetSkills(vcc, skill_id, keyspace)

	local result = { success = false }

	if not next(skills) then
		keyspace:Close()
		LogError("[ADMINISTRATION.IsUsedSkill] - Validation Error: '%s' skill not found.", skill_id)
		result.result = "NOT_FOUND"
		return result
	end

	-- 1 - valido si se encuentra en uso en algun usuario
	local agents_skills = m_models.Skills.GetAllAgentSkills(vcc, keyspace)
	local user_used_skills = {}

	for user_id, skill in pairs(agents_skills) do
		for id, value in pairs(skill) do
			user_used_skills[id] = user_used_skills[id] or {}
			user_used_skills[id][user_id] = { user_id = user_id }
		end
	end

	local user_used_skill = user_used_skills[skill_id]
	-- 2 - valido si se encuentra en uso en alguna account de alguna campana
	local accounts_skill = m_models.Skills.GetAccountsBySkill( vcc, skill_id, keyspace )
	keyspace:Close()

	local count_users = utils.GetRowCount(user_used_skill)
	local count_accounts = utils.GetRowCount(accounts_skill)

	if count_users > 0 and count_accounts > 0 then
		local msg = LogWarning("[ADMINISTRATION.IsUsedSkill] - Skill '%s'. Is being used for '%d' Users: '%s'.\n'%d' Accounts: '%s'", skill_id, count_users, json.encode(user_used_skill), count_accounts, json.encode(accounts_skill) )
		result.warning = {
			info = "SKILL_BEING_USED_ON_USERS_AND_ACCOUNTS",
			count_users = count_users,
			count_accounts = count_accounts,
			reason = msg
		}
	elseif count_users > 0 then
		local msg = LogWarning("[ADMINISTRATION.IsUsedSkill] - Skill '%s'. Is being used for '%d' Users: '%s'.", skill_id, count_users, json.encode(user_used_skill) )
		result.warning = {
			info = "SKILL_BEING_USED_ON_USERS",
			count_users = count_users,
			reason = msg
		}
	elseif count_accounts > 0 then
		local msg = LogWarning("[ADMINISTRATION.IsUsedSkill] - Skill '%s'. Is being used for '%d' Accounts: '%s'.", skill_id, count_accounts, json.encode(accounts_skill) )
		result.warning = {
			info = "SKILL_BEING_USED_ON_ACCOUNTS",
			count_accounts = count_accounts,
			reason = msg
		}
	else
		LogInfo("[ADMINISTRATION.IsUsedSkill] - Skill '%s', is not used.", skill_id)
		result.success = true
	end

	return result
end

--[[
Remove skills from a vcc
postedData must be of the form: ["skillId1", "skillId2", ...]
]]
function RemoveSkills(web)
	local vcc = inconcert.authUser.virtualContactCenter

	LogInfo("[ADMINISTRATION.RemoveSkills] -> user '%s', vcc '%s', skills: '%s'.", inconcert.authUser.userName, vcc, web.input["post_data"])

	local skills_to_remove = web.request

	if utils.HasAllegroIntegration( vcc ) then
		for _, skill_id in ipairs(skills_to_remove) do
			local result = IsUsedSkill(skill_id)
			if not result.success then
				web.status = 400
				return result
			end
		end
	end

	local success, err = m_models.Skills.RemoveSkills( vcc, skills_to_remove )
	local result

	if success then
		result = {result = "OK"}
		
		--notifications
		for _, skillId in pairs(skills_to_remove) do
			notificationsEngine.SendNotification("SkillDeleted", 0, { skillId = skillId })
	end
	else
		web.status = 400
		LogError("[ADMINISTRATION.RemoveSkills] Removing skills error: %s.", json.encode(err))
		result = err
	end

	return result
end

function RemoveValidation(web, skill_id)

	local result = IsUsedSkill(skill_id)

	if result.success then
		result.result = "OK"
		return result
	end
	if result.result == "NOT_FOUND" then
		web.status = 404
		return result
	end
	result.result = "OK"

	return result
end
