module(..., package.seeall, inconcert.orbit.application.Decorate)
local utils = require "shared.apps.utils"
local permissions = require "shared.permissions.permissions"
local http = require("socket.http")
local ltn12 = require("ltn12")
local wsapi = require "wsapi"
local icu = require("icu")
local m_moduleLocation = select(2, ...)
-- TODO: ver de resolver la ip y puerto de otra forma
local m_spellchecker_URI = [[http://127.0.0.1:9009/inconcert/apps/spellchecker/spellcheck/%s]]
m_htmlRender = nil
m_appLocation = nil

function MapUris(module, htmlRender, prefix, appLocation)

	local LogRequest = require("apps.agent.requestFilters").LogRequest

	module:dispatch_post(utils.JsonValidatorReply..LogRequest..Search, prefix .. "/search/")
	module:dispatch_get(utils.JsonReply..LogRequest..permissions:CheckPermission("administration", "view_vcc_spell_checker", load_vcc_custom_words), prefix.."/load_words/(.*)")
	module:dispatch_post(utils.JsonValidatorReply..LogRequest..permissions:CheckPermission("administration", "save_vcc_spell_checker", save_word), prefix.."/save_word/")
	module:dispatch_post(utils.JsonValidatorReply..LogRequest..permissions:CheckPermission("administration", "delete_vcc_spell_checker", remove_word), prefix.."/delete_word/")

	module:dispatch_get(Render, prefix .. "/frm/render/")

	m_htmlRender = htmlRender
	m_appLocation = appLocation
end


local function request_load_language_custom_words( language )

	local vcc = inconcert.authUser.virtualContactCenter
	-- TODO: ver la ip y puerto posta del spellchecker
	-- sample: http://i6.inconcert.cc/inconcert/apps/spellchecker/spellcheck/inconcert
	local uri = string.format(m_spellchecker_URI, vcc)
	-- sample: "lang=es&driver=PSpell&action=get_custom_words"
	local body = string.format([[lang=%s&driver=%s&action=%s]], language, "PSpell", "get_custom_words")

	LogInfo("[spellchecker:request_load_language_custom_words] Load vcc custom words for language url: '%s', params: '%s'", uri, body)

	local response = {}
	local result, code, headers, statusline = socket.http.request({
		url = uri,
		method = "POST",
		source = ltn12.source.string(body),
		headers = {
			["Content-Length"] = #body,
			["Content-Type"] = "text/plain"
		},
		--respuesta
		sink = ltn12.sink.table(response),
	})

	LogInfo("[spellchecker:request_load_language_custom_words] Loaded vcc custom words for language result: '%s', code: '%s' headers: '%s' , statusline: '%s'", result, code, json.encode(headers or {}), statusline)

	if code ~= 200 then
		return {}, code
	end
	-- Aca los datos me llegan encodeados por ejemplo en Alfabeto latino "ISO8859-1", a esto asi me lo envia el request desde el spellchecker.
	--   entonces a partir de esto tengo que convertirlo a UTF8 para poder enviarlo al lado del cliente, y para que el json.encode no reviente. he dicho!!!
	-- Ej: de como me esta llegando y como lo tengo que enviar.
	--  llega la palabra "cany�ana" si a esto lo paso derecho al json.encode explota, entonces lo convierte a utf8 y queda
	--  algo asi: "canyñana", y luego de que pasa por el json.encode queda asi: "cany\u00F1ana"
	--  que es finalmente como se termina enviando al lado del cliente.
	--
	-- TODO: tengo que revisarlo bien, porque NO esta bien. Ver con Ignacio

	local function get_encoding( info )
		-- el enconding esta llegando en el request que retorna el spell, revisar!!
		return info.encoding -- "ISO8859-1"
	end
	local encoded_words = table.concat(response, "")
	local words = json.decode(encoded_words)
	-- aca convierto a UTF8
	encoded_words = icu.convert(encoded_words, get_encoding(words), "UTF-8")
	local words = json.decode(encoded_words)

	return words.custom_words or {}, code
end

function Render(web)
	_M.response.DontCache(web)
	local env = { _T = m_htmlRender.extension.makeTextTranslator() }
	local vcc = inconcert.authUser.virtualContactCenter
	local user_id = inconcert.authUser.userName.."@"..vcc

	LogInfo("Render form Spell Checker for user %s, role %s", user_id, inconcert.authUser.userGroup)

	local html_file = m_appLocation .. "/pages/frm_spell_checker_content.html"
	local content = m_htmlRender:RenderPageLP(html_file, web, env)
	if not content then
		LogError("HTML Spell Checker does not exist: %s", html_file)
	end
	return content
end

--Returns all Spell Checker in the VCC
function Search( web )
	local _T = m_htmlRender.extension.makeTextTranslator()
	local vcc = inconcert.authUser.virtualContactCenter
	local user_id = inconcert.authUser.userName.."@"..vcc

	LogInfo("Search VCC Spell Checker for user '%s', role '%s'.", user_id, inconcert.authUser.userGroup)

	local languages = {
		{ id = "en" , name = _T("English")    , description = _T("Custom english language")    , total_words = 0 },
		{ id = "es" , name = _T("Spanish")    , description = _T("Custom spanish language")    , total_words = 0 },
		{ id = "pt" , name = _T("Portuguese") , description = _T("Custom portuguese language") , total_words = 0 },
		{ id = "fr" , name = _T("French")     , description = _T("Custom french language")     , total_words = 0 }
	}

	local post_data = web.request
	-- se fija si tiene que contar la cantidad de palabras en el diccionario
	local must_count_words = false
	for _, column in ipairs(post_data.columns) do
		if column == "total_words" then
			must_count_words = true
			break
		end
	end
	if must_count_words then
		for _, language in pairs(languages) do
			local words, code = request_load_language_custom_words( language.id )
			if code == 200 then
				language.total_words = #words
			end
		end
	end
	--LogDebug("Spellchecker LANGUAGES: %s", json.encode(languages))

	--include only Spell Checker that belong to logged user vcc
	local response = {
		rows = languages,
		last = "",
		total_rows = utils.GetRowCount( languages )
	}

	post_data.__order_attr_name = "name"
	return utils.MakeSearchResponse( response, post_data )
end

function load_vcc_custom_words( web, language )

	--	Esta funcion retorna la palabras customs para language / vcc y el formato retornado es algo asi:
	-- {
		-- {name="hola"},
		-- {name="manana"},
		-- {name="perro"},
		-- {name="gato"},
		-- {name="loro"},
		-- {name="estupido"},
		-- {name="genio"},
		-- {name="mejor"},
		-- {name="peor"},
		-- {name="rocio"},
		-- {name="ellio"}
	-- }

	local vcc = inconcert.authUser.virtualContactCenter
	local user_id = inconcert.authUser.userName.."@"..vcc
	LogInfo("[spellchecker:load_vcc_custom_words] User '%s' Retrieving custom words for language '%s'.", user_id, language)

	local words, code = request_load_language_custom_words( language )

	if code == 200 then
		if next(words) then
			local formatted_words = {}
			for _, word in ipairs(words) do
				table.insert(formatted_words, { name = word })
			end
			return formatted_words
		else
			return {}
		end
	else
		web.status = code
		LogError("[spellchecker:load_vcc_custom_words] Error loading spell checker custom words for vcc: '%s'. Error code: '%s'", vcc, code)
		return {result = "NOT_FOUND"}
	end
end

function save_word( web )

	local vcc = inconcert.authUser.virtualContactCenter
	local user_id = inconcert.authUser.userName.."@"..vcc

	LogInfo( "[spellchecker:save_word], user '%s'. post_data: %s", user_id, web.input["post_data"] )

	local language = web.request.language
	local new_word = web.request.new_word.name
	-- optional
	local old_word = web.request.old_word and web.request.old_word.name

	if new_word == old_word then
		LogError("[spellchecker:save_word] Error saving custom word. Old word: '%s' is equal to new word: '%s', language: '%s', vcc: '%s'. Error code: '%s'", old_word, new_word, language, vcc, code)
		web.status = 400
		return {result = "ERROR_UPDATING_EQUALS_WORDS"}
	end

	-- TODO: ver la ip y puerto posta del spellchecker
	-- sample: http://i6.inconcert.cc/inconcert/apps/spellchecker/spellcheck/inconcert
	local uri = string.format(m_spellchecker_URI, vcc)
	local body
	if old_word then
	-- update word
		-- sample: "lang=es&driver=PSpell&action=add_word&word=rocio&old_word=rocio1"
		body = string.format([[lang=%s&driver=%s&action=%s&word=%s&old_word=%s]], language, "PSpell", "add_word", wsapi.util.url_encode(new_word), wsapi.util.url_encode(old_word))
	else
	-- new word
		-- sample: "lang=es&driver=PSpell&action=add_word&word=rocio"
		body = string.format([[lang=%s&driver=%s&action=%s&word=%s]], language, "PSpell", "add_word", wsapi.util.url_encode(new_word))
	end

	LogInfo("[spellchecker:save_word] Request: '%s', data: '%s'", uri, body)

	local response = {}
	local result, code, headers, statusline = socket.http.request({
		url = uri,
		method = "POST",
		source = ltn12.source.string(body),
		headers = {
			["Content-Length"] = #body,
			["Content-Type"] = "text/plain"
		},
		--respuesta
		sink = ltn12.sink.table(response)
	})

	local encoded_response = table.concat(response, "")
	LogInfo("[spellchecker:save_word] Request-> result: '%s', code: '%s', , headers: '%s' , statusline: '%s', response: '%s'.", result,  code, json.encode(headers or {}), statusline, encoded_response)

	local success
	success, response = pcall(json.decode, encoded_response)

	if code == 200 then
		if response.status == "ok" then
			LogInfo("[spellchecker:save_word] Successfully saved custom word. Language: '%s', vcc: '%s'. New word: '%s', Old word: '%s'.", language, vcc, new_word, old_word)
			return { result = "OK" }
		else
			web.status = 400
			LogError("[spellchecker:save_word] Error saving custom word. Language: '%s', vcc: '%s'. New word: '%s', Old word: '%s'. Error: '%s'", language, vcc, new_word, old_word, encoded_response)

			return { result = response.reason or "NOT_FOUND" }
		end
	else
		code = type(code) == "number" and code or "400"
		web.status = code
		LogError("[spellchecker:save_word] Error saving custom word. Language: '%s', vcc: '%s'. New word: '%s', Old word: '%s'. Error code: '%s'", language, vcc, new_word, old_word, code)
		return {result = "NOT_FOUND"}
	end
end

--[[
Remove language words from a vcc
]]
function remove_word( web )

	local vcc = inconcert.authUser.virtualContactCenter
	local user_id = inconcert.authUser.userName.."@"..vcc

	LogInfo( "[spellchecker:remove_word] Removing custom word. User '%s'. post_data: %s", user_id, web.input["post_data"] )

	local language = web.request.language
	local word = web.request.word
	-- TODO: ver la ip y puerto posta del spellchecker
	-- sample: http://i6.inconcert.cc/inconcert/apps/spellchecker/spellcheck/inconcert
	local uri = string.format(m_spellchecker_URI, vcc)
	-- sample: "lang=es&driver=PSpell&action=remove_word&word=rocio"
	local body = string.format([[lang=%s&driver=%s&action=%s&word=%s]], language, "PSpell", "remove_word", word)

	LogInfo("[spellchecker:remove_word] Request: '%s', data: '%s'", uri, body)

	local response = {}
	local result, code, headers, statusline = socket.http.request({
		url = uri,
		method = "POST",
		source = ltn12.source.string(body),
		headers = {
			["Content-Length"] = #body,
			["Content-Type"] = "text/plain"
		},
		--respuesta
		sink = ltn12.sink.table(response)
	})

	LogInfo("[spellchecker:remove_word] Request-> result: '%s', code: '%s', , headers: '%s' , statusline: '%s'", result,  code, json.encode(headers or {}), statusline)

	if code == 200 then
		LogInfo("[spellchecker:remove_word] Successfully removed custom word: '%s', language: '%s', vcc: '%s'.", word, language, vcc)
		return { result = "OK" }
	else
		web.status = code
		LogError("[spellchecker:remove_word] Error removing custom word: '%s', language: '%s', vcc: '%s'. Error code: '%s'", word, language, vcc, code)
		return {result = "NOT_FOUND"}
	end
end
