module(..., package.seeall, inconcert.orbit.application.Decorate)

local Ticketek = require "ticketek"
local utils = require "shared.apps.utils"
local nozzle = require "nozzle"
local helpers = require "ticketek.helpers"
local permissions = require "shared.permissions.permissions"
-- yeah, I know...
local agentCampaignModel = require "apps.agent.models.campaigns"

---
-- Filter that performs structure validation on the request
--
local validator = nozzle.filter{ input = function(self, web)
	local request = web.request
	--LogDebug("[ADMINISTRATION.TICKETS:validator] request: %s", json.encode(request or {}))
	if not request.description then
		web.status = 400
		return { status = false, code = "E_INVALID_ARG", msg = "missing description" }
	elseif not request.number_type then
		web.status = 400
		return { status = false, code = "E_INVALID_ARG", msg = "missing number type" }
	elseif web.request.number_type ~= "autoincrement" and web.request.number_type ~= "date" then
		web.status = 400
		return { status = false, code = "E_INVALID_ARG", msg = "invalid number type" }
	end
end}

---
-- Begin of REST? api

---
--
local function GetList (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local post_data = web.input["post_data"]
	LogInfo("[ADMINISTRATION.TICKETS:GETLIST] 1 - Retrieving ticket emitters for vcc '%s'. post_data: %s", vcc, post_data)
	_M.response.DontCache(web)
	local tickets = Ticketek.get_by_vcc(vcc)
	return tickets
end

local function Search (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local startTime = os.time()
	local post_data = web.input["post_data"]
	LogInfo("[ADMINISTRATION.TICKETS:SEARCH] 1 - Retrieving ticket emitters for vcc '%s'. post_data: %s", vcc, post_data)
	_M.response.DontCache(web)
	local tickets = Ticketek.get_by_vcc(vcc)
	local postData = web.request
	local totalUnfilteredRows = utils.GetRowCount(tickets)
	local totalAdded = 0
	local filteredRows = {}
	local hashColumns = utils.ArrayToHash(postData.columns or {}) or {}
	LogInfo("[ADMINISTRATION.TICKETS:SEARCH] 2 - Filtering rows")

	for _, ticket in pairs(tickets) do
		ticket.Id = utils.GetId(ticket.id)
		ticket.Description = ticket.description
		if utils.MatchFilter(ticket, postData) then

			local emitter
			if hashColumns.number_type then
				emitter = Ticketek.get(ticket.id)
				ticket.number_type = emitter and emitter.number_type
			end
			if hashColumns.preview then
				emitter = emitter or Ticketek.get(ticket.id)
				if emitter and emitter.number_type and emitter.number_type then
					local min, max = Ticketek.preview(emitter)
					ticket.preview = {
						minimum = min, maximum = max
					}
				end
			end
			
			totalAdded = totalAdded + 1
			table.insert(filteredRows, ticket)
		end
	end

	LogInfo("[ADMINISTRATION.TICKETS:SEARCH] 3 - Sorting rows")

	--include only rows that belong to logged user vcc
	local response = {
		rows = filteredRows,
		last = "",
		total_rows = totalAdded,
		total_unfiltered_rows = totalUnfilteredRows
	}
	local responseParams = {
		MakeSortingValue = function(row)
			assert(row)
			return tostring(row.Description):lower()
		end
	}
	local resultData = utils.MakeSearchResponse(response, postData, responseParams)

	local endTime = os.time()
	local elapsedTime = endTime - startTime
	LogInfo("[ADMINISTRATION.TICKETS:SEARCH] 4 - (END) Search rows. Retrieve %d of %d matched rows. Total Unfiltered Rows: %d. ELAPSED Time '%s' seconds.", #resultData.rows, resultData.total_rows, resultData.total_unfiltered_rows, elapsedTime)

	return resultData
end

local function GetById (web, emitter_id)
	_M.response.DontCache(web)
	local vcc = inconcert.authUser.virtualContactCenter
	-- checks that the vcc of the emitter matches the vcc of the user
	if helpers.extract_vcc(emitter_id) ~= vcc then
		web.status = 403
		return { status = false, code = "E_NOT_AUTHORIZED", msg = "not authorized" }
	end
	return Ticketek.get(emitter_id)
end

---
--
local function AddNew (web)
	local vcc = inconcert.authUser.virtualContactCenter
	LogInfo("[ADMINISTRATION.TICKEST:AddNew] Adding new ticket emitter '%s' in vcc '%s'", emitter_id, vcc)
	local emitter, err = Ticketek.new(vcc, {
		description = web.request.description,
		number_type = web.request.number_type,
		number_width = web.request.number_width,
		base_number = web.request.base_number,
		prefix = web.request.prefix,
		suffix = web.request.suffix
	})
	return { status = true, emitter = emitter }
end

---
--
local function Update (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local emitter_id = web.request.id
	if not emitter_id or type(emitter_id) ~= "string" then
		web.status = 400
		return { status = false, code = "E_INVALID_ARG", msg = "missing id" }
	end
	LogInfo("[ADMINISTRATION.TICKEST:Update] Updating ticket emitter '%s' in vcc '%s'", emitter_id, vcc)
	local new_data = web.request
	new_data.isNew = nil -- hay que quitar este atributo ya que sirve solo client side
	local emitter = Ticketek.update(vcc, emitter_id, new_data)
	return { status = true, emitter = emitter }
end

---
-- TODO: improve. we are (ab)using a controller of the 'agent' application to get a list of all campaigns and see if
-- any account is using the given emitter.
-- Ideally, I should have a Lua library that deals with campaigns, accounts, etc, and use that instead of accessing
-- the guts of another application.
--
local function Delete (web, emitter_id)
	local vcc = inconcert.authUser.virtualContactCenter

	-- checks that the vcc of the emitter matches the vcc of the user
	if helpers.extract_vcc(emitter_id) ~= vcc then
		web.status = 403
		return { status = false, code = "E_NOT_AUTHORIZED", msg = "not authorized" }
	end

	LogInfo("Deleting ticket emitter '%s' in vcc '%s'", emitter_id, vcc)

	-- This is shit. Must not use a controller of another application.
	-- TODO: armar una libreria de manejo de campañas
	local campaigns = agentCampaignModel:FindAll()
	if campaigns then
		for campaign_id, campaign in pairs(campaigns) do
			for _, account in ipairs(campaign.accounts) do
				local ticket_emitter = account.TicketEmitter
				if ticket_emitter and ticket_emitter.id == emitter_id then
					local msg = LogWarning("The emitter is in use by '%s' account '%s' in campaign '%s'",
						account.proxyType, account.name, campaign.Description)
					return { status = false, code = "E_EMITTER_IN_USE", msg = msg }
				end
			end
		end
	end

	local ok, err = Ticketek.delete(emitter_id)
	return { status = ok, message = err }
end

---
-- Returns a preview of what does the emitted values look like.
-- The preview consists of the minimum and maximum value and a table with warnings.
-- atm, it just warns if the emitter has a width that would clip the result
--
local function Preview (web, emitter_data)
	LogInfo("Previewing a ticket emitter with the following data: '%s'", web.input.post_data)

	if not web.request.number_type then
		web.status = 400
		return { status = false, code = "E_INVALID_ARG", msg = "missing number type" }
	end
	if web.request.number_type ~= "autoincrement" and web.request.number_type ~= "date" then
		web.status = 400
		return { status = false, code = "E_INVALID_ARG", msg = "invalid number type" }
	end

	local min, max, warnings = Ticketek.preview(web.request)
	return { status = true, minimum = min, maximum = max, warnings = warnings }
end

---
-- Makes sure the description is not duplicated, to avoid confusion.
-- emitter_id is optional (this is the case when we're creating a new emitter)
--
local function CheckDescriptionDuplicates (web, description, emitter_id)
	local vcc = inconcert.authUser.virtualContactCenter
	LogInfo("Checking that description '%s' is not duplicated on vcc '%s'", description, vcc)

	local emitters = Ticketek.get_by_vcc(vcc)
	description = description:lower()
	for _, emitter in ipairs(emitters or {}) do
		if emitter.description:lower() == description and emitter.id ~= emitter_id then
			return { available = false }
		end
	end

	return {
		available = true
	}
end

--
-- End of REST? api
---

---
-- Builds a filter that checks wether we have a given permission
--
local function make_filtered_permission (permission)
	return nozzle.filter{ input = function(_, web)
		LogDebug("aaaaaaaaaaaaaaaaaaaaaaa")
		if not permissions.IsPermissionEnabledForLoggedUser("administration", permission) then
			web.status = 403
			return { status = false, code = "E_NOT_AUTHORIZED", msg = "not authorized", result = "FORBIDDEN" }
		end
	end}
end

---
--
function MapUris (module, htmlRender, prefix)
	local LogRequest = require("apps.agent.requestFilters").LogRequest

	local check_can_save = make_filtered_permission("save_tickets")
	local check_can_view = make_filtered_permission("view_tickets")
	local check_can_delete = make_filtered_permission("delete_tickets")

	module:dispatch_post ( LogRequest .. utils.JsonReply .. GetList, prefix .. "/list" )

	module:dispatch_post ( LogRequest .. utils.JsonValidatorReply .. Search, prefix .. "/search" )

	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. validator .. check_can_save .. AddNew, prefix .. "/new" )
	module:dispatch_get ( LogRequest .. utils.JsonReply .. check_can_view .. GetById, prefix .. "/get/(.+)" )
	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. validator.. check_can_save .. Update, prefix .. "/update" )
	module:dispatch_post( LogRequest .. utils.JsonReply .. check_can_delete .. Delete, prefix .. "/delete/(.+)" )

	module:dispatch_post( LogRequest .. utils.JsonValidatorReply .. Preview, prefix .. "/preview" )
	module:dispatch_get ( LogRequest .. utils.JsonReply .. CheckDescriptionDuplicates, prefix .. "/is_available/(.+)/(.*)" )
end
