local m_models = require("apps.administration.model"):GetModels()
module(..., package.seeall, inconcert.orbit.application.Decorate)
local utils = require "shared.apps.utils"
local permissions = require "shared.permissions.permissions"
local agentservice = require "apps.agent.agentservice"
local authFilter = (require "authfilter")("agent")
local utf8 = require "icu.utf8"
require "uuid"
require "LuaOrm.Date"
local agentUsers = require "apps.agent.controllers.users"
local agentInteractions = require "apps.agent.controllers.interactions"
local agentUsersModels = require("apps.agent.models.users")
local model_perm = require "shared.permissions.models.permissions"
local notificationsEngine = require("apps.agent.notifications")
local agentNotifications = require("apps.agent.agentNotifications")
local Charsets = require "shared.relational_databases.charsets"
local m_moduleLocation = select(2, ...)
local wsapi = require "wsapi"
wsapi.request = require "wsapi.request"
local audits = require "audits"
local m_validations = require("apps.administration.controllers.validations.user").GetDefinitions()
m_htmlRender = nil
m_appLocation = nil

function MapUris(module, htmlRender, prefix, appLocation)
	local LogRequest = require("apps.agent.requestFilters").LogRequest

	module:dispatch_post(utils.JsonValidatorReply .. LogRequest.. Search, prefix .. "/search/")
	module:dispatch_get(utils.JsonReply .. LogRequest .. permissions:CheckPermission("administration", "viewusers", Get), prefix .. "/get/(.+)")

	module:dispatch_post(utils.JsonReply .. LogRequest .. permissions:CheckPermission("administration", "saveuser",
			utils.WebJsonValidatorData(
				m_validations.basic,
				Save
			)
		),
		prefix .. "/save/"
	)

	module:dispatch_post(utils.JsonValidatorReply .. LogRequest .. permissions:CheckPermission("administration", "deactivate_user", Activate), prefix .. "/activate/")
	module:dispatch_post(utils.JsonValidatorReply .. LogRequest .. permissions:CheckPermission("administration", "deactivate_user", Deactivate), prefix .. "/deactivate/")
	module:dispatch_get(Render, prefix .. "/frm/render/")
	module:dispatch_wsapi(ExportInfoCsv, prefix .. "/export/(.*)")

	m_htmlRender = htmlRender
	m_appLocation = appLocation
end

local function GetModel (entityName)
	assert(type(entityName) == "string")
	if not m_models or not m_models[entityName] then
	-- Si el modelo no se ha creado aun, debo crearlo.
	--  Esto sucede en algunas ocasiones si es que me han invocado desde otra app Ej: "Allegro" y aun no se ha creado el modelo.
	--  Entonces tengo que crear el modelo.
		local model = require "apps.administration.model"
		model.CreateModelTable()
		m_models = model:GetModels()
	end
	local entityModel = m_models[entityName]
	assert(type(entityModel) == "table")
	return entityModel
end

function Render (web)
	_M.response.DontCache(web)
	local env = { _T = m_htmlRender.extension.makeTextTranslator() }
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc

	LogInfo("Render form User for user %s, group %s", userId, inconcert.authUser.userGroup)

	local html_file = m_appLocation .. "/pages/frm_user_content.html"
	local content = m_htmlRender:RenderPageLP(html_file, web, env)
	if not content then
		LogError("HTML User does not exist: %s", html_file)
	end
	return content
end

function GetUserAuditData (fullUserId, keyspace, usersOverridePermissions, vcc, userColumns)
	assert(type(fullUserId) == "string")
	assert(type(keyspace) == "userdata")
	assert(type(usersOverridePermissions) == "table")
	return GetModel("Users").GetUserAuditData(fullUserId, keyspace, vcc, usersOverridePermissions, userColumns) or {}	
end

function ExportInfoCsv(wsapi_env)
	local vcc = inconcert.authUser.virtualContactCenter
	_T = m_htmlRender.extension.makeTextTranslator()
	local request = wsapi.request.new(wsapi_env)
	local show_inactive = request.params["show_inactive"]
	LogInfo("[Users:ExportUsersInfoCsv] ExportUsersInfoCsv. show_inactive '%s'.", show_inactive)
	local usersInfo = GetExportInfo(wsapi_env)
	local function ReturnErr (err)
		-- Error
		wsapi_env.status = 500
		local response = wsapi.response.new(200, {["Content-Type"]= "text/plain"})
		response:write(json.encode(err))
		response.headers["Response"] = "defaultError"
		response.status = err.httpCode or 500
		return response:finish()
	end

	if usersInfo.result then
		return ReturnErr(usersInfo.result)
	end

	local orderHeaders = {
		_T("User"),
		_T("Name"),
		_T("Surname"),
		_T("Profile"),
		_T("Creation Date"),
		_T("Last Login"),
		_T("Status"),
		_T("Permission Overrides"),
		_T("Campaigns"),
	}

	local userInfoTranslate = {}
	for _, value in pairs(usersInfo) do
		local arrayCampaigns = {}
		for key in pairs(value.campaigns or {}) do
			table.insert( arrayCampaigns, utils.GetId(key))
		end
		local userCampaigns = #arrayCampaigns > 0 and table.concat(arrayCampaigns, ";") or ""

		local userData = {
			[_T("User")] = value.user,
			[_T("Name")] = value.name,
			[_T("Surname")] = value.FirstSurname,
			[_T("Profile")] = value.UserGroup,
			[_T("Creation Date")] = LuaOrm.Date(value.creationDate):toDifferentTimezoneString(inconcert.authUser.timezone, "%Y-%m-%d %H:%M:%S"),
			[_T("Last Login")] = LuaOrm.Date(value.LastLogin):toDifferentTimezoneString(inconcert.authUser.timezone, "%Y-%m-%d %H:%M:%S"),
			[_T("Status")] = _T(value.Status),
			[_T("Permission Overrides")] = value.overridePermissions ~= "" and string.gsub(value.overridePermissions, '"', '""' ) or "",
			[_T("Campaigns")] = userCampaigns
		}
		
		if value.Status == "Active" or tostring(show_inactive) == "1"  then
			table.insert(userInfoTranslate, userData)
		end
	end

	local csv = Charsets.Init().fromutf8(utils.Table2Csv(userInfoTranslate, nil, nil, "|", orderHeaders))
	csv = string.gsub('"'..csv..'"', "|", '","' )
	local LN = "\n"
	csv = string.gsub(csv, LN, '"'..LN..'"')
	LogInfo("[Users:ExportUsersInfoCsv] ExportUsersInfoCsv. CSV data '%s'.", csv)
	local updateDate = LuaOrm.Date(LuaOrm.Date()):toDifferentTimezoneString(inconcert.authUser.timezone, "%Y-%m-%d %H:%M:%S")
	local suggestedFileName = string.format("%s(%s).CSV", "users_export_info_".. vcc, updateDate)
	local headers = {}
	headers["Response"]            = "defaultError"
	headers["Content-Type"]        = "application/octet-stream"
	headers["Content-Length"]      = #csv
	headers["Response"]            = "ok"
	headers["Content-Disposition"] = [[attachment; filename="]] .. suggestedFileName .. '"'
	headers["Set-Cookie"]          = "download_started=true; Path=/" -- used by the jquery download plugin

	LogInfo("[Users:ExportUsersInfoCsv] ExportUsersInfoCsv. CSV File Name '%s'.", suggestedFileName)

	local coroutineIter = 0
	return 200, headers, coroutine.wrap(function()
		coroutineIter = coroutineIter + 1
		coroutine.yield(csv)
	end)

end

function GetExportInfo(web)
	LogInfo("Get Export Info")
	local vcc = inconcert.authUser.virtualContactCenter
	local keyspace = utils.GetDatasource():CassandraConnection( "Agent" )
	local users = GetModel("Users").GetVCCUsers(vcc, keyspace)

	if not next(users) then
		keyspace:Close()
		web.status = 404
		return {result = "NOT_FOUND"}
	end

	local usersOverridePermissions = model_perm.GetUsersOverridePermissions(vcc, keyspace)
	local info = {}
	for _, value in pairs(users) do
		if value.Id then
			local fullUserId = value.Id .. "@" .. vcc
			local user_obj = {
				user = fullUserId,
				name = value.Name,
				FirstSurname = value.FirstSurname,
				UserGroup = value.UserGroup,
				creationDate = value.creationDate,
				LastLogin = LuaOrm.Date(tonumber(value.LastLoginSuccessTimestamp)):toString()
			}
			if (not value.StatusReason or value.StatusReason == "") and value.Status == "0" then
				value.StatusReason = "ADMIN"
			end
			if value.Status == "0" then
				user_obj.Status = value.StatusReason == "ADMIN" and "Inactive" or "Locked"
			else
				user_obj.Status = "Active"
			end
			user_obj.overridePermissions = value.HasRoleOverride == "1" and json.encode(usersOverridePermissions[fullUserId].actions) or ""
			user_obj.campaigns = GetModel("Campaigns").GetUserCampaigns(vcc, value.Id, keyspace) or utils.EMPTY_OBJECT
			table.insert(info, user_obj)
		end
	end

	keyspace:Close()
	return info
end

local function isUserExpired (vccInfo, user)
	if not vccInfo or not user or not next(vccInfo) or not next(user) then
		return nil
	end
	if vccInfo.Id ~= "system" and not authFilter:HasExternalAuth(vccInfo) and vccInfo.Security.authentication.passwordChecks and user.PasswordChangeTimestamp then
		local now = os.time()
		local passwordExpiration = tonumber(vccInfo.Security.authentication.passwordChecks.expirationDays) * 24 * 60 * 60
		local passwordExpirationDate = (tonumber(user.PasswordChangeTimestamp) + passwordExpiration) or now
		if passwordExpiration > 0 and passwordExpirationDate < now then
			--LogDebug("[ADMINISTRATION.USERS:isUserExpired] User %s password has expired", user.Id)
			return true
		end	
	end
	return nil
end

function Get (web, userId)
	local vcc = inconcert.authUser.virtualContactCenter
	LogInfo("[ADMINISTRATION.USERS:Get] Retrieving userId %s, user %s, vcc %s", userId, (inconcert.authUser.userName.."@"..vcc), vcc)

	local keyspace = utils.GetDatasource():CassandraConnection( "Agent" )
	local user = GetModel("Users").GetUser(vcc, userId, keyspace)

	if not next(user) then
		local reason = LogError("[ADMINISTRATION.USERS:Get] User '%s' not found.", userId)
		keyspace:Close()
		web.status = 404
		return {result = "NOT_FOUND", reason = reason}
	end

	if utils.IsUserFullSystemAccess(user) then
		local reason = LogError("[ADMINISTRATION.USERS:Get] You do not have permissions to obtain users with full access to the system. User: %s", userId)
		keyspace:Close()
		web.status = 403
		return {result = "ACCESS FORBIDDEN", reason = reason}
	end

	user.Password = nil
	local userPermissions = model_perm.GetUserPermissions(userId.."@"..vcc, keyspace)
	user.custom_permissions = userPermissions or utils.EMPTY_OBJECT

	-- categorized user permissions
	local applicationsActions = permissions.GetApplicationsActions(keyspace, vcc)
	user.categorized_permissions = model_perm.Categorize(user.custom_permissions.actions, keyspace, applicationsActions)

	user.skills = GetModel("Skills").GetAgentSkills(vcc, userId, keyspace) or utils.EMPTY_OBJECT

	-- los override de permisos de la campana (member y roles) categorizados
	user.user_campaigns = utils.Remap(GetModel("Campaigns").GetUserCampaigns(vcc, userId), utils.GetId) or utils.EMPTY_OBJECT
	local override_camp_perm = model_perm.GetUserPermissionsOnCampaigns( userId, nil --[[ specific_user_group --]] , user.user_campaigns, keyspace )
	local categorized_override_camp_perm = model_perm.CategorizeCampaignsPermissions(override_camp_perm, keyspace, applicationsActions)
	user.categorized_camp_permissions = categorized_override_camp_perm

	user.Status = user.Status or "1"

	if (not user.StatusReason or user.StatusReason == "") and user.Status == "0" then
		user.StatusReason = "ADMIN"
	end

	local vccInfo = GetModel("VCCs").Get(vcc, keyspace, {"Security"}, true)
	user.isExpired = isUserExpired(vccInfo, user)

	local statuses = agentservice.GetAgentsStatuses({user}, vcc) or {}
	local fullUserId = user.Id .. "@" .. vcc
	user.status = statuses[fullUserId]

	keyspace:Close()
	return user
end

local function MakeAllegroOtherInfo (httpVars, action)
	-- LogDebug(" 1- httpVars %s", httpVars)
	-- LogDebug(" 2- httpVars.HTTP_X_ICC_RID '%s'", httpVars.HTTP_X_ICC_RID)
	-- LogDebug(" 3- httpVars.HTTP_X_REAL_IP '%s'", httpVars.HTTP_X_REAL_IP)
	local allegroOtherInfo = {}
	if not httpVars then
		return allegroOtherInfo
	end
	allegroOtherInfo.remoteServerIP = httpVars.HTTP_X_REAL_IP
	allegroOtherInfo.action = action
	return allegroOtherInfo
end

--[[
	keyspace: keyspace Agent
	userId: usuario
	vcc: vcc
	--Se obtienen las campannas del usuario y los AL de las campannas del que el usuario es miembro.
	--Solicitado desde Api
]]--
function GetUserCampaignsAls (userId,vcc, keyspace, userCampaigns)
	vcc = vcc or inconcert.authUser.virtualContactCenter
	if not userId then
		return 
	end
	local campaigns = {}
	local userCampaigns = userCampaigns or GetModel("Campaigns").GetUserCampaigns(vcc, userId, keyspace) or {}
	for campaignId in pairs(userCampaigns) do
		--Busco los ALs
		local campaignAls = GetModel("Campaigns").GetAttentionLevels(campaignId, keyspace) or {}
		local simple_camp_id = utils.GetId(campaignId)
		campaigns[simple_camp_id] = {
			UserAttentionLevels = {},
			id = simple_camp_id
		}
		for al, alValue in pairs(campaignAls) do
			if alValue.users and alValue.users[userId] then
				table.insert( campaigns[simple_camp_id].UserAttentionLevels, al)
			end
		end
	end
	return campaigns
end

function SearchUsers (info, keyspace)
	local vcc = info.vcc or inconcert.authUser.virtualContactCenter
	local currentUserId = inconcert.authUser.userName
	local fullUserId = utils.AppendVcc(currentUserId, inconcert.authUser.virtualContactCenter)
	local startTime = os.time()

	 LogInfo("[ADMINISTRATION.USERS:SearchUsers] 1 - Search users for user %s, group %s (START). Posted data: '%s'", fullUserId, inconcert.authUser.userGroup, json.encode(info))

	local wantEnabled = ""
	local wantApplication
	if info.query then
		--want enabled
		wantEnabled = info.query.Status
		if wantEnabled == nil then
			wantEnabled = ""
		end
		info.query.Status = nil
		--want application
		--Se empieza a soportar filtrado de multiples Aplicaciones
		if type(info.query.Application) == "string" and info.query.Application ~= "" then
			wantApplication = {info.query.Application}
		elseif type(info.query.Application) == "table" and #info.query.Application > 0 then
			wantApplication = info.query.Application
		end
		info.query.Application = nil
		--empty
		if utils.TableIsEmpty(info.query) then
			info.query = nil
		end
	end

	local applicationsActions = nil
	local cachedCampaignsPermissions = nil
	local usersOverridePermissions = nil
	local cachedRolesPermissions = {}

	--[[
	]]--
	local function customFilter (userId, user, userCampaigns)
		if wantEnabled ~= "" and tonumber(user.Status) ~= tonumber(wantEnabled) then
			return false
		end
		if tonumber(user.IsSystemUser or "0") ~= 0 then
			return false
		end
		if utils.IsUserFullSystemAccess(user) then
			return false
		end
		if wantApplication then
			-- Se realizan diversos "caches" para que la función "model_perm.HasPermissionToApplication" NO tenga
			--  que realizar ciertos cálculos repetitivos (Que no cambian) para diferentes usuarios.
			cachedRolesPermissions[user.UserGroup] = user.grouppermissions
										or cachedRolesPermissions[user.UserGroup]
										or model_perm.GetUserGroupPermissions(user.UserGroup, keyspace)
			usersOverridePermissions = usersOverridePermissions
										or model_perm.GetUsersOverridePermissions(vcc, keyspace)
			cachedCampaignsPermissions = cachedCampaignsPermissions
										or model_perm.GetAllVCCCampaignsPermissions(vcc, keyspace)
			applicationsActions = applicationsActions or permissions.GetApplicationsActions(keyspace, vcc)
			local userCampaignsPermissions = model_perm.MakeGroupedPermissionsByCampaigns(
				{
					get_campaign_permissions = function(campaignId)
						return cachedCampaignsPermissions[campaignId]
					end,
					specific_user_group = user.UserGroup,
					user_campaigns = userCampaigns,
					user_id = userId
				}
			)
			local fullUserId = utils.AppendVcc(userId, vcc)
			local userOverrideVccPermissions = usersOverridePermissions[fullUserId] or utils.EMPTY_OBJECT
			if not permissions.HasPermissionToApplication(
					wantApplication,
					userId,
					user.UserGroup,
					nil, --fullCampaignId,
					keyspace,
					{
						applicationsActions       = applicationsActions,
						group_permissions         = cachedRolesPermissions[user.UserGroup],
						campaigns_permissions     = cachedCampaignsPermissions,
						user_campaigns            = userCampaigns,
						user_vcc_permissions      = userOverrideVccPermissions,
						user_campaign_permissions = userCampaignsPermissions,
						customPermissions         = nil
					},
					info.skipCanValidationPermission
				)
			then
				return false
			end
		end
		-- si, lo debe de incluir
		return true
	end

	local hashColumns = utils.ArrayToHash(info.columns or {}) or {}
	local mustUsersStatus = (not info.columns or hashColumns["status"]) and true or false
	local rowsPerPage = tonumber(info.limit)
	info.numberPage = info.numberPage or 1
	--local startCurrentProcessTime = startTime
	-- LogInfo("[ADMINISTRATION.USERS:SearchUsers] 2 - Search users - (load users of cassandra) page: '%s', rows per page: '%s'", info.numberPage, rowsPerPage)

	local closeKeyspace = false
	if not keyspace then
		keyspace = utils.GetDatasource():CassandraConnection("Agent")
		closeKeyspace = true
	end
	local vccUsers = GetModel("Users").GetVCCUsers(vcc, keyspace)
	local vccInfo = GetModel("VCCs").Get(vcc, keyspace, {"Security"}, true)

	--[[local userColumns = info.columns and utils.DeepCopy(info.columns)
	if userColumns then
		table.insert(userColumns, "IsSystemUser")
	end]]--

	--LogInfo("[ADMINISTRATION.USERS:SearchUsers] 3 - Search users - (end loaded users of cassandra). ELAPSED Time '%s' seconds.", utils.GetElapsedTimeSec(startCurrentProcessTime))
	--startCurrentProcessTime = os.time()

	--LogInfo("[ADMINISTRATION.USERS:SearchUsers] 4 - Search users - (filtering users)")
	local totalAdded = 0
	local filteredRows = {}

	for userSimpleId, userInfo in pairs(vccUsers or {}) do
		userInfo.Enabled = userInfo.Enabled or "1"
		if (not userInfo.StatusReason or userInfo.StatusReason == "") and userInfo.Status == "0" then
			userInfo.StatusReason = "ADMIN"
		end

		userInfo.Password = nil

		--Obtengo los ide de campaigns del user si necesito validar permisos sobre una APP
		local userCampaigns
		if wantApplication then
			userCampaigns = GetModel("Campaigns").GetUserCampaigns(vcc, userSimpleId, keyspace)
		end
		if utils.MatchFilter(userInfo, info) and customFilter(userSimpleId, userInfo, userCampaigns) then
			--Datos solicitados desde la APiSdk (por WFM)
			if hashColumns["Campaigns"] then
				userInfo.Campaigns = GetUserCampaignsAls(userSimpleId, vcc, keyspace, userCampaigns)
			end

			userInfo.isExpired = isUserExpired(vccInfo, userInfo)
			totalAdded = totalAdded + 1
			--LogDebug("[ADMINISTRATION.USERS:SearchUsers]   + %d added User '%s' ", totalAdded, userSimpleId)
			table.insert(filteredRows, userInfo)
		-- else
			-- LogDebug("[ADMINISTRATION.USERS:SearchUsers]   - NOT ADD  user: '%s'",  json.encode(userInfo))
		end
	end

	if closeKeyspace then
		keyspace:Close()
	end

	--LogInfo("[ADMINISTRATION.USERS:SearchUsers] 5 - Search users - Sorting and Make search response. Specifics columns. ELAPSED Time '%s' seconds.", utils.GetElapsedTimeSec(startCurrentProcessTime))
	--startCurrentProcessTime = os.time()

	local response = {
		rows = filteredRows,
		total_rows = totalAdded,
		total_unfiltered_rows = utils.GetRowCount(vccUsers)
	}

	local responseParams = {
		MakeSortingValue = function(row)
			assert(row)
			return string.format("%s %s", tostring(row.Name or ""), tostring(row.FirstSurname or "")):upper()
		end
	}

	local searchResponse = utils.MakeSearchResponse(response, info, responseParams)

	if mustUsersStatus and searchResponse and searchResponse.rows and #searchResponse.rows > 0 then
		--LogInfo("[ADMINISTRATION.USERS:SearchUsers] 5.1 - Search users - (Agent service get agent status of %d users). ELAPSED Time '%s' seconds.", #searchResponse.rows, utils.GetElapsedTimeSec(startCurrentProcessTime))
		--startCurrentProcessTime = os.time()
		local statuses = agentservice.GetAgentsStatuses(searchResponse.rows, vcc) or {}
		for i, user in ipairs( searchResponse.rows ) do
			local userStatus = statuses[user.Id .. "@" .. vcc]
			user.status = userStatus and {
				state = userStatus.state,
				lastStatusChangeTime = userStatus.lastStatusChangeTime,
				lastLoginTime = userStatus.lastLoginTime,
				interactions = userStatus.interactions,
			} or { state = "OFFLINE" }
		end
	--else
		-- LogInfo("[ADMINISTRATION.USERS:SearchUsers] 5.1 - Search users - Not retrieve Agent service statuses. ELAPSED Time '%s' seconds.", utils.GetElapsedTimeSec(startCurrentProcessTime))
		--startCurrentProcessTime = os.time()
	end

	--LogDebug("[ADMINISTRATION.USERS:SearchUsers] 5.5 - Search - Detail Users of Page: '%s'. ELAPSED Time '%s' seconds.", json.encode(searchResponse.rows), utils.GetElapsedTimeSec(startCurrentProcessTime))

	LogInfo("[ADMINISTRATION.USERS:SearchUsers] 6 - Search users (END). Retrieve %d users of %d. ELAPSED Time '%s' seconds.", #searchResponse.rows, response.total_rows, utils.GetElapsedTimeSec(startTime))

	return searchResponse
end

--Search all users in the VCC
function Search (web)
	local postData = web.request
	return SearchUsers(postData)
end

local function SavePermissions (userId, userPermissions, keyspace)
	local vcc = inconcert.authUser.virtualContactCenter
	LogInfo("Save custom permission for user %s, group %s", userId, inconcert.authUser.userGroup)
	model_perm.SaveUserPermissions(userId .. "@" .. vcc, userPermissions, keyspace)
	return true
end

local function ValidateCustomPermissions (user, customPermissions, keyspace)
	if type(customPermissions) ~= "table" then
		return true -- retorno ok
	end

	-- 1)- busco los gruppermissions segun el grupo, como aca no se si el grupo pertenece a un grupo custom
	--	o a un grupo de sistema, primero busco en uno y luego en otro.
	local userGroup = user.UserGroup
	local groupPermissions = model_perm.GetUserGroupPermissions( userGroup, keyspace )
	if utils.TableIsEmpty(groupPermissions) then
		return false -- fallo
	end

	--[[
		2) Aca valido que el Usuario "administrador" que esta realizando una actualizacion de los permisos (override) de si mismo,
			que NO se quede sin el permiso de Actualizar Usuarios "saveuser" que se encuentra en la App "administration" bajo la entidad "users".
		Con esto nos aseguramos que al menos quede un usuario que tenga permisos para actualizar y crear Usuarios.
	]]--
	if customPermissions.actions then
		local fullUserId = utils.AppendVcc(user.Id, inconcert.authUser.virtualContactCenter):lower()
		local loggedUserId = utils.GetCurrentLoggedUser():lower()
		if loggedUserId == fullUserId:lower() then
			if not customPermissions.actions.administration
				or customPermissions.actions.administration.saveuser == false
			then
				return false, {result = "UNABLE TO REMOVE YOUR OWN PERMISSIONS TO UPDATE USERS", reason = "You cannot remove your own permissions to update users."}
			end
		end
	end

	-- 3)- aca recorro la lista de custom recibidos en el request, y busco dentro de los permisos existentes en la BD.
	--	Si la accion custom que se recibe es activa (enabled=true) retorno ok porque quiere decir que al menos hay una accion activa,
	--	o si se encuentra dentro de los permisos de la app en la BD, y lo que se recibe
	--	indica que la accion se debe desactivar (enabled=false), lo que hago es poner en nil la accion existente
	--	de la BD. Esto me sirve para despues recorrer los permisos de la BD que quedaron (**),
	--	y si existe al menos una en true, salgo de la funcion retornando ok. De lo contrario retorno false
	--	porque me han desactivado todas las acciones para este grupo.
	for appId, permissions in pairs( customPermissions.actions or {} ) do
		for actionId, enabled in pairs( permissions or {} ) do
			if enabled == true then
				return true -- al menos una accion custom activa retorno ok
			elseif enabled == false then
				if groupPermissions[appId] and groupPermissions[appId][actionId] == true then
					groupPermissions[appId][actionId] = nil -- dejo en nil en la lista
				end
			end
		end
	end

	-- 4)- (**) aca hago la validacion mencionada en el comentario anterior 3).
	for appId, permissions in pairs(groupPermissions) do
		for actionId, enabled in pairs(permissions) do
			if enabled == true then
				return true
			end
		end
	end

	return false
end

--[[
	Esta funcion le cambia el state a "BREAK" a los usuarios que se encuentren en estado "AVAILABLE"
	y que ya no tengan permisos sobre la aplicacion "INTERACTION".
--]]
function ChangeUserStateToOnLine (user, keyspace, applicationsActions)
	if type(user) ~= "table" then
		return
	end
	local vcc = inconcert.authUser.virtualContactCenter
	local userName = utils.GetId(user.Id)
	if not agentservice.IsAgentLoggedIn(userName, vcc) then
	-- si el usuario no esta logueado, me voy
		return
	end

	local statuses = agentservice.GetAgentsStatuses({user}, vcc) or {}
	local fullUserId = userName .. "@" .. vcc
	local status = statuses[fullUserId]

	if type(status) == "table" and type(status.state) == "string" and utf8.upper(status.state) == "ACTIVE" then
	-- aca ingreso si el usuario se encuentra en el estado "Available (ACTIVE)"
		local groupPermissions = user.grouppermissions
		local userPermissions = user.custom_permissions -- TODO: #4639

		-- aca realizo la validacion si el usuario tiene permisos para la app "INTERACTION"
		local hasPermissionToAppInteraction = model_perm.HasPermissionToApplication("interaction", userName, user.UserGroup, userPermissions, groupPermissions, nil, keyspace, nil, nil, nil, applicationsActions)

		if hasPermissionToAppInteraction then
		-- si tiene permisos para esta app, me voy
			return
		end
		-- obtengo el token del usuario al cual le voy a cambiar el estado
		local userSessionId = authFilter:GetTokenFromUser(userName, vcc)
		if not userSessionId then
			return
		end
		-- lo cambio al estado "BREAK"
		local state = "BREAK"
		LogInfo("[USER:ChangeUserStateToOnLine] Changing user %s@%s status to %s on session id %s", userName, vcc, state, userSessionId)
		local result = agentservice.ChangeAgentState(userSessionId, userName, vcc, state)
		if result.success then
			-- notifico via poller al agente, para que realice las operaciones pertinentes del lado del cliente.
			agentNotifications.NotifyAgents({
				scope = "agent",
				eventType = "interaction_permissions_removed",
				pollerEventType = "server_changed_state",
				agents = { fullUserId },
				pollerMessage = {
					state = state,
					type = "NOT_ALLOWED_TO_RECEIVE_CONVERSATIONS"
				}
			}, keyspace)
		end
	end
end

local function ValidateSaveLoggedUser (user, keyspace)
	if type(user) ~= "table" then
		return false
	end

	local vcc = inconcert.authUser.virtualContactCenter
	local userName = utils.GetId(user.Id)
	local isLoggedUser = agentservice.IsAgentLoggedIn(userName, vcc)

	local currentUser = GetModel("Users").GetUser(vcc, userName, keyspace, { "UserGroup" })
	if not next(currentUser) then
	-- si el usuario no existe en la BD me voy
		return true, isLoggedUser
	end

	if isLoggedUser and utf8.upper(user.UserGroup) ~= utf8.upper(currentUser.UserGroup) then
		local reason = string.format("Unable to modify role %s of the %s logged user.", user.UserGroup, user.Id)
		return false, {result = "UNABLE TO MODIFY ROLE OF THE LOGGED USER", reason = reason}
	end

	return true, isLoggedUser
end

--[[
]]--
function SaveOneUser (user, keyspace, applicationsActions, allegroOtherInfo, connectionHistorical)
	assert(type(user) == "table")
	assert(keyspace)

	local supportedLanguages = { es = true, en = true, fr = true, pt = true}
	local ValidateLanguage = function( user )
		user.LanguageId = user.LanguageId or "es"
		return supportedLanguages[user.LanguageId:lower()]
	end

	local vcc = inconcert.authUser.virtualContactCenter
	local isLoggedUser
	local is_new = user.isNew
	local old_user_permissions
	local new_user_permissions = user.custom_permissions
	local fullUserId = user.Id .. "@" .. vcc

	if not ValidateLanguage(user) then
		local reason = LogError("Error Saving User: '%s', unsupported language: '%s'", user.Id, user.LanguageId)
		local result = {result = "UNSUPPORTED_APP_LANGUAGE", reason = reason }
		return false, result
	end

	if (user.UserGroup or ""):lower() == "full-system-access" then
		user.VccGroup = "system"
		local superUser = GetModel("Users").GetSuperUser(vcc, user.Id, keyspace)
		if not superUser then
			-- Si es un Super Usuario debo almacenarlo en la CF SuperUsers.
			local allSuperUsers = GetModel("Users").GetVCCSuperUsers(vcc, keyspace)
			local total = utils.GetRowCount(allSuperUsers)
			--LogDebug("--> total: %s - allSuperUsers: %s", total, json.encode(allSuperUsers))
			local MAXIMUM_SUPER_USERS_PER_VCC = 10
			if total >= MAXIMUM_SUPER_USERS_PER_VCC then
				local reason = LogError("[ADMINISTRATION.USERS:SaveOneUser] Unable to Save Super User '%s'. You have reached the maximum number of super users allowed in the VCC. Maximum allowed: %s", fullUserId, MAXIMUM_SUPER_USERS_PER_VCC)
				local result = {result = "UNABLE ADD SUPER USER - REACHED THE MAXIMUM ALLOWED", reason = reason, maxAllowed = MAXIMUM_SUPER_USERS_PER_VCC}
				return false, result
			end
			user.isNew = true
			is_new = true
		end
	end
	if is_new then
		user.UserGroup = user.UserGroup or "agent"
		user.newUser = true
		user.isNew = nil
	else
		local hasChange, result = ValidateSaveLoggedUser(user, keyspace)
		if not hasChange then
			LogError(result.reason)
			return false, result
		end

		-- valido si el usuario ha cambiado de Rol
		local old_user = GetModel("Users").GetUser(vcc, user.Id, keyspace, {"UserGroup", "VccGroup"})
		if next(old_user) then
			old_user.Id = user.Id
			old_user.VCC = vcc
			-- if utils.IsUserFullSystemAccess(old_user) then
			-- 	local reason = LogError("Error Saving User: '%s'. Unable to update user of full system access.", user.Id)
			-- 	local result = {result = "UNABLE TO UPDATE USER - FULL SYSTEM ACCESS", reason = reason }
			-- 	return false, result
			-- end
			if old_user.UserGroup ~= user.UserGroup then
				local has_permission_to_app_interaction = nil -- como referencia
				-- 1 - valida si el usuario tiene interacciones asignadas, y si al cambiar de role, sigue teniendo permisos sobre la app "interaction"
				local assigned_count = CountAssignedInteractions( user.Id, nil, keyspace )
				if assigned_count > 0 then
					has_permission_to_app_interaction = model_perm.HasPermissionToApplication("interaction", fullUserId, user.UserGroup, nil, nil, nil, keyspace, nil, nil, nil, applicationsActions)
					if not has_permission_to_app_interaction then
						-- si no tiene permisos sobre la app interaction falla
						local reason = string.format("User has %d assigned interactions. Unable to modify role permission of user '%s', new role '%s' not have permission to interaction app.", assigned_count, user.Id, user.UserGroup)
						local result = {result = "USER_HAS_INTERACTIONS_ROLE_HAVE_NOT_PERMISSION_TO_INTERACTION_APP", reason = reason, assigned_interactions = assigned_count}
						LogError(result.reason)
						return false, result
					end
				end
				-- 2- si cambio de rol valido, si el usuario pertenecia al menos a un att level.
				local is_user_in_attention_levels = GetModel("Campaigns").IsUserInAttentionLevels( user.Id, keyspace )
				if is_user_in_attention_levels then
					-- si pertenecia a algun attention level, valido si el nuevo rol que ha seleccionado tiene permisos sobre la app "interaction"
					if has_permission_to_app_interaction == nil then -- si es nil es porque no tiene interacciones tomadas (1)
						has_permission_to_app_interaction = model_perm.HasPermissionToApplication("interaction", fullUserId, user.UserGroup, nil, nil, nil, keyspace, nil, nil, nil, applicationsActions)
					end
					if not has_permission_to_app_interaction then
						-- si no tiene permisos sobre la app interaction falla
						local reason = string.format("Attention levels conflict. Unable to modify role permission of user '%s', new role '%s' not have permission to interaction app.", user.Id, user.UserGroup)
						local result = {result = "USER_IN_ATTENTION_LEVELS_ROLE_HAVE_NOT_PERMISSION_TO_INTERACTION_APP", reason = reason}
						LogError(result.reason)
						return false, result
					end
				end
			end
		end

		isLoggedUser = result
		old_user_permissions = model_perm.GetUserPermissions( fullUserId, keyspace )
	end

	--modified users are always enabled
	user.Status = "1"

	local success, err = ValidateCustomPermissions( user, new_user_permissions, keyspace )
	if not success then
		err = err or {result = "GROUP_NOT_HAS_ENABLED_ACTIONS", reason = "Group does not has enabled actions"}
		LogError("[ADMINISTRATION.USERS:SaveOneUser] %s.", err.reason)
		return false, err
	end

	local usersOverridePermissions = model_perm.GetUsersOverridePermissions(vcc, keyspace)
	fullUserId = utils.AppendVcc(user.Id, vcc)
	local old_audit_user = GetUserAuditData(fullUserId, keyspace, usersOverridePermissions)

	-- Por compatibilidad voy contra el controller Users de la App "AGENT"
	success, result = agentUsers.UpdateUser(user, keyspace, applicationsActions, allegroOtherInfo)
	if not success then
		--error
		if type(result) == "table" then
			return false, result
		end
		return false, {result = result}
	end

	--success
	fullUserId = utils.AppendVcc(user.Id, vcc)
	-- Si es un Super Usuario debo almacenarlo en la CF SuperUsers.
	local isUserFullSystemAccess = utils.IsUserFullSystemAccess(user)
	if isUserFullSystemAccess then
		GetModel("Users").SaveSuperUser(vcc, user, keyspace)
	end
	-- change el user status
	ChangeUserStateToOnLine(user, keyspace, applicationsActions)
	if new_user_permissions then
		if new_user_permissions.has_override == "1" then
			if not SavePermissions( user.Id, new_user_permissions, keyspace ) then
				return {result = "USER_HAS_NOT_PERMISSION", reason = "User has no rigth to grant permission if he has'nt got it in first place"}
			end
		else
			new_user_permissions = nil
			model_perm.DeleteUserPermissions( fullUserId, keyspace )
		end
	end

	if isLoggedUser then
		local has_changed_permissions = not utils.StructuralCompare( new_user_permissions, old_user_permissions )
		if has_changed_permissions then
			-- notifico via poller al agente, que los permisos custom del usuario para que realice las operaciones pertinentes del lado del cliente.
			agentNotifications.NotifyAgents({
				eventType = "administrator_user_permissions_change",
				agents = { fullUserId },
				pollerMessage = {
					type = "USER_CHANGE_PERMISSIONS"
				}
			}, keyspace)
		end
	end

	fullUserId = utils.AppendVcc(user.Id, vcc)
	local new_audit_user = GetUserAuditData(fullUserId, keyspace, usersOverridePermissions)

	if old_audit_user or new_audit_user then
		audits.Create({
			vcc = vcc,
			entity = "User",
			user = inconcert.authUser.userName,
			fieldKey = user.Id,
			oldFields = old_audit_user and next(old_audit_user) and old_audit_user or nil,
			newFields = new_audit_user and next(new_audit_user) and new_audit_user or nil,
			connection = connectionHistorical
		})
	end

	-- success
	return true
end

function Save (web)
	local allegroOtherInfo = MakeAllegroOtherInfo(web.vars, "SAVE")
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName .. "@" .. vcc
	local ok, users = pcall(json.decode, web.input["post_data"])

	LogInfo("[ADMINISTRATION.USERS.SAVE] Saving Users - user '%s', vcc '%s'. posted data: '%s'", userId, vcc, web.input["post_data"])
	if not ok or not users then
		LogError("Error in Save Users: '%s', posted data: %s", users, web.input["post_data"])
		web.status = 400
		web.headers["Content-Type"] = "application/json"
		return json.encode {result = "INVALID_REQUEST"}
	end

	local keyspace = utils.GetDatasource():CassandraConnection("Agent")
	local applicationsActions = permissions.GetApplicationsActions(keyspace, vcc)
	local connectionHistorical = utils.GetDatasource():CassandraConnection( "Historical" ) -- para crear record de audit
	for id, user in pairs(users) do
		local success, result = SaveOneUser(user, keyspace, applicationsActions, allegroOtherInfo, connectionHistorical)
		if not success then
			connectionHistorical:Close()
			keyspace:Close()
			web.status = 400
			return result
		end
	end
	connectionHistorical:Close()
	keyspace:Close()
	return {result = "OK"}
end

function Deactivate (web)
	local allegroOtherInfo = MakeAllegroOtherInfo(web.vars, "DEACTIVATE")
	local loggedUserId = inconcert.authUser.userName.."@"..inconcert.authUser.virtualContactCenter
	local memberId = tostring(web.request.memberId):lower()

	LogInfo("User %s is deactivating user %s", loggedUserId, memberId)

	if not memberId then
		LogError("Unable to deactivate user %s. Invalid post data.", memberId)
		web.status = 400
		return {result = "NO_MEMBER_SPECIFIED", reason = "You need to specifiy a memberId"}
	end

	local simpleMember, memberVCC = utils.SeparateVCC(memberId)
	memberVCC = memberVCC or inconcert.authUser.virtualContactCenter

	local isLoggedIn = agentservice.IsAgentLoggedIn(simpleMember, memberVCC)
	if isLoggedIn then
		LogError("Unable to deactivate user %s. User is logged in.", memberId)
		web.status = 400
		return {result = "USER_IS_LOGGED", reason = "User is logged in"}
	end

	local keyspace = utils.GetDatasource():CassandraConnection( "Agent" )
	--Check that user hasn't assigned interactions
	local assigned_count = CountAssignedInteractions(simpleMember, nil, keyspace, memberVCC)

	if assigned_count > 0 then
		keyspace:Close()
		LogError("Unable to deactivate user %s. User has assigned interactions.", memberId)
		web.status = 400
		return {result = "USER_HAS_INTERACTIONS", reason = "User has assigned interactions"}
	end
	--get old user
	local usersOverridePermissions = model_perm.GetUsersOverridePermissions(memberVCC, keyspace)
	local fullUserId = utils.AppendVcc(simpleMember, memberVCC)
	local oldUser = GetUserAuditData(fullUserId, keyspace, usersOverridePermissions)

	local modelUsers = GetModel("Users")
	local success, info = modelUsers.EnableUser(memberVCC, simpleMember, false, nil, nil, keyspace, allegroOtherInfo)

	if not success then
		local err = info
		keyspace:Close()
		LogError("Unable to deactivate user %s. Err: '%s'.", memberId, json.encode(err))
		web.status = 400
		return err
	end

	local vccInfo = GetModel("VCCs").Get(memberVCC, keyspace, {"Security"}, true)
	info.isExpired = isUserExpired(vccInfo, info)
	info.PasswordChangeTimestamp = nil

	--Create audit record
	local saved_user = GetUserAuditData(fullUserId, keyspace, usersOverridePermissions)
	if oldUser and saved_user then
		local ksHistorical = utils.GetDatasource():CassandraConnection("Historical")
		audits.Create({
			vcc = memberVCC,
			entity = "User",
			user = inconcert.authUser.userName,
			fieldKey = simpleMember,
			oldFields = oldUser,
			newFields = saved_user,
			connection = ksHistorical
		})
		ksHistorical:Close()
	end

	keyspace:Close()
	--notifications
	notificationsEngine.SendNotification("UserDisabled", 0, { userId = memberId })

	return {result = "OK", info = info}
end

function Activate (web)
	local allegroOtherInfo = MakeAllegroOtherInfo(web.vars, "ACTIVATE")
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc
	local memberId = tostring(web.request.memberId):lower()

	LogInfo("User %s is activating user %s", userId, memberId)

	if not memberId then
		LogError("Unable to activate user %s. Invalid post data.", memberId)
		web.status = 400
		return {result = "NO_MEMBER_SPECIFIED", reason = "You need to specifiy a memberId"}
	end

	local keyspace = utils.GetDatasource():CassandraConnection( "Agent" )

	local Id = utils.GetId(memberId)
	--try to reserve peer
	local ok, peerData
	if not utils.HasAllegroIntegration( vcc, keyspace ) then
		ok, peerData = agentUsers.ReserveAvailablePeer(vcc, keyspace)
		if not ok then
			keyspace:Close()
			LogError("Unable to activate user %s. Can not reserve peer %s.", memberId, peerData)
			web.status = 400
			return {result = "CAN_NOT_RESERVE_PEER", reason = peerData}
		end
	end

	local mail = GetModel("Users").GetMailFromAddresses(vcc, Id, keyspace)
	if mail then
		local owner = GetModel("Users").MailOwner(mail, keyspace)
		if owner and owner ~= Id.."@"..vcc then
			keyspace:Close()
			LogError("Unable to activate user %s. E-mail specified is already taken, please specify another one.", memberId)
			web.status = 400
			return {result = "MAIL_ALREADY_TAKEN", reason = "E-mail specified is already taken, please specify another one"}
		end
	end

	--get old audit user data
	local usersOverridePermissions = model_perm.GetUsersOverridePermissions(vcc, keyspace)
	local fullUserId = utils.AppendVcc(Id, vcc)
	local oldUser = GetUserAuditData(fullUserId, keyspace, usersOverridePermissions)

	LogInfo("User %s is activating user %s, with new mail %s", userId, Id, mail)
	local success, info = GetModel("Users").EnableUser(vcc, Id, true, mail, peerData, keyspace, allegroOtherInfo)

	if peerData then
		agentUsersModels.UnlockPeer(peerData.id)
	end

	if not success then
		local err = info
		keyspace:Close()
		LogError("Unable to activate user %s. Err: '%s'.", memberId, json.encode(err))
		web.status = 400
		return err
	end

	local vccInfo = GetModel("VCCs").Get(vcc, keyspace, {"Security"}, true)
	info.isExpired = isUserExpired(vccInfo, {PasswordChangeTimestamp = info.PasswordChangeTimestamp})
	info.PasswordChangeTimestamp = nil
	
	--Create audit record
	local saved_user = GetUserAuditData(fullUserId, keyspace, usersOverridePermissions)
	if oldUser and saved_user then
		local ksHistorical = utils.GetDatasource():CassandraConnection("Historical")
		audits.Create({
			vcc = vcc,
			entity = "User",
			user = inconcert.authUser.userName,
			fieldKey = Id,
			oldFields = oldUser,
			newFields = saved_user,
			connection = ksHistorical
		})
		ksHistorical:Close()
	end
	keyspace:Close()

	--notifications
	notificationsEngine.SendNotification("UserEnabled", 0, { userId = memberId })
	
	return {result = "OK", info = info}
end

--[[
	optionals: campaign_id, keyspace
--]]
function CountAssignedInteractions (simple_user_id, campaign_id, keyspace, vcc)
	simple_user_id = utils.GetId(simple_user_id)
	local user_interactions = agentInteractions.RetrieveUserInteractions(simple_user_id, campaign_id, keyspace, vcc)
	local count = #user_interactions
	if count > 0 then
		LogInfo("[Users:CountAssignedInteractions] User '%s' has '%d' interactions taken for '%s' campaign.", simple_user_id, count, campaign_id)
	end
	return count
end

local function __MakeVCCStatusHelpDeskIntegrationsId ()
	local key = string.format(
		"HelpDeskIntegration:VCC:%s:Users:Updated:Status",
		inconcert.authUser.virtualContactCenter
	)
	--LogDebug("[__MakeVCCStatusHelpDeskIntegrationsId]------------> key: '%s'", key)
	return key
end

local function GetVCCStatusHelpDeskIntegration ()
	local key = __MakeVCCStatusHelpDeskIntegrationsId()
	local datasource = GetModel("Users").GetDataSource()
	return datasource.redis:get(key)
end

function UpdateVCCStatusHelpDeskIntegrations (integration)
	assert(integration)
	local key = __MakeVCCStatusHelpDeskIntegrationsId()
	local datasource = GetModel("Users").GetDataSource()
	if integration.Status == "RUNNING" then
		datasource.redis:set(key, json.encode(integration))
		-- La key expira al minuto
		local MINUTE_IN_SEC = 60
		datasource.redis:expire(key, MINUTE_IN_SEC)
	else
		datasource.redis:del(key)
		if GetVCCStatusHelpDeskIntegration() then
			LogWarning("[UpdateVCCStatusHelpDeskIntegrations] Unable to delete ASDFFFFFFFFFFFFFFFFFFFFF status '%s'", key)
			return false
		end
	end
	return key
end

function IsAnyHelpDeskIntegrationInProgress ()
	local encodedStatus = GetVCCStatusHelpDeskIntegration()
	if not encodedStatus then
		return false
	end
	return true, encodedStatus
end

--[[
	Esta función se encarga de actualizar la lista de Usuarios en la integración HELPDESK.
	Es decir, a partir de la información de los usuarios de la BD de i6, actualiza la BD de HelpDesk.

	Esta función puede recibir la lista de usuarios a actualizar, si es que queremos actualizar alguno o algunos usuarios en particular,
		o si la lista de usuarios recibida es vacía, se procederá a actualizar todos los Usuarios del VCC.

	La misma es invocada del script la app "provisioning" controllers "vccs".
]]--
function UpdateHelpDeskIntegrations (data)
	local fullUserId = utils.AppendVcc(inconcert.authUser.userName, inconcert.authUser.virtualContactCenter)

	LogInfo("[ADMINISTRATION.USERS.UpdateHelpDeskIntegrations] Updating Users In Integrations. User '%s'. Data: '%s'", fullUserId, json.encode(data))
	local vcc = data.vcc
	assert(vcc)
	local users = data.users or {}

	if not utils.HasHelpDeskIntegration(vcc) then
		LogError("[ADMINISTRATION.USERS:UpdateHelpDeskIntegrations] Unable to Update Users Integrations. Vcc '%s' has not configured integration with HELPDESK.", vcc)
		return false, {result = "WITHOUT_HELPDESK_INTEGRATION", reason = "VCC without HELPDESK integration"}
	end

	local inProgress, encodedStatus = IsAnyHelpDeskIntegrationInProgress()
	if inProgress then
		LogError("[ADMINISTRATION.USERS:UpdateHelpDeskIntegrations] Unable to Update Users of VCC '%s' with HelpDesk Integrations. Already exist another update running. Info: %s", vcc, encodedStatus)
		return false, {result = "ALREADY_EXIST_ANOTHER_UPDATE_RUNNING", reason = "Already exist another update running.", info = json.decode(encodedStatus)}
	end

	-- Obtiene todos los usuarios que se encuentren en estado ENABLED
	local info = {
		Enabled = "1",
		query = {},
		numberPage = nil,
		limit = nil,
		vcc = vcc
	}
	local keyspace = utils.GetDatasource():CassandraConnection("Agent")
	local response = SearchUsers(info, keyspace)
	local totalPostedUsers = #users

	LogInfo("[ADMINISTRATION.USERS.UpdateHelpDeskIntegrations] VCC '%s'. Total DATABASE Users '%s'. Total Posted Users '%s'.", vcc, #response.rows, (totalPostedUsers == 0 and "ALL" or totalPostedUsers))

	-- Si la cantidad de Usuarios que se deben actualizar (los recibidos en el POST)
	--  se encuentra vacía, quiere decir que tiene que actualizar todos los usuarios del VCC.
	local forceInclude = totalPostedUsers == 0
	-- Ahora hash de usuarios, para una validación más eficiente.
	local hashUsersToInculde = {}
	for _, userId in pairs(users) do
		hashUsersToInculde[userId:lower()] = true
	end
	-- Función que realiza la validación para saber si el usuario se debe incluir en la actualización.
	local function MustIncludeUser (user)
		if forceInclude then
			return true
		end
		return hashUsersToInculde[user.Id:lower()]
	end
	--Funcion que valida que el Usuario a Postear a HELPDESK, tengo los campos necesarios.
	local function IsValidUser (user)
		local res = {success = true}
		if not user.Mail or #user.Mail == 0 then
			res.success = false
			res.err = {result = "DOES NOT HAVE EMAIL", reason = "User does not have an assigned e-mail"}
			LogError("[ADMINISTRATION.USERS.UpdateHelpDeskIntegrations] Validation ERROR. User '%s', %s.", user.Id, res.err.reason)
		end
		return res
	end
	-- Ahora cargo la lista de Usuarios que se tienen que actualizar.
	-- En si se valida la lista de usuarios de la BD contra la recibida en el POST.
	local validationErrorsList = {} -- lista utilizada unicamente para retorna al cliente si es que la validacion ha fallado
	local validationSuccessList = {} -- lista utilizada unicamente para retorna al cliente si es que la validacion ha fallado
	local usersToUpdate = {} -- lista de usuarios a actulizar contra HELPDESK (Usuarios con todos los detalles). Esta lista sirve si es que TODOS los usuarios han superado la validación.

	for _, user in ipairs(response.rows) do
		--LogDebug(" %d - user: '%s'", _, json.encode(user))
		if MustIncludeUser(user) then
			--LogDebug("[ADMINISTRATION.USERS.UpdateHelpDeskIntegrations] + INCLUDE User '%s'.", user.Id)
			local res = IsValidUser(user)
			if res.success then
				user.isNew = false
				-- Lista de usuarios a actualizar, si es que pasan toda la validación.
				table.insert(usersToUpdate, user)
			-- actualizo la lista de usuarios que SI pasaron la validación
				table.insert(validationSuccessList, {
					userId = user.Id
				})
			else
			-- actualizo la lista de usuarios que NO pasaron la validación
				table.insert(validationErrorsList, {
					userId = user.Id,
					err = res.err
				})
			end
		--else
			--LogDebug("[ADMINISTRATION.USERS.UpdateHelpDeskIntegrations] - CAN NOT INCLUDE User '%s'.", user.Id)
		end
	end

	if #validationErrorsList > 0 then
		-- La actualización se cancelará por completo, ya que los usuarios tienen que tener todos los campos correctamente para poder ser actualizados en HELPDESK
		keyspace:Close()
		LogError("[ADMINISTRATION.USERS:UpdateHelpDeskIntegrations] Unable to Update Users Integrations in Vcc '%s'/HelpDesk. Validation Error.", vcc)
		return false, {result = "VALIDATION_ERROR", reason = "", validation = {success = validationSuccessList, errors = validationErrorsList}}
	end

	local updatedSuccessList = {}
	local updateErrorsList = {}
	local agentUserModel = require "apps.agent.models.users"

	-- Marco el start del proceso en redis
	local updateInfo = {
		Id = uuid.new(),
		VCC = vcc,
		StartDate = LuaOrm.Date():toString(),
		User = inconcert.authUser.userName,
		Status = "RUNNING"
	}
	UpdateVCCStatusHelpDeskIntegrations(updateInfo)
	updateInfo.TotalRows = #usersToUpdate
	--

	local updateStatusEvery = 5
	local i = 0
	local forceCancel = false

	for index, user in ipairs(usersToUpdate) do
		if forceCancel then
			break
		end
		-- Guarda en HelpDesk el usuario
		LogInfo("[ADMINISTRATION.USERS:UpdateHelpDeskIntegrations] Syncrhonize %d of %d Users of VCC '%s' with HelpDesk. User '%s'", index, updateInfo.TotalRows, vcc, user.Id)
		local success, helpDeskInfo = agentUserModel.UpdateHelpDeskUser(user, vcc, keyspace)
		if success then
		-- actualizo la lista de actualizaciones correctas
			table.insert(updatedSuccessList, {
				userId = user.Id
			})
		else
			local err
			if helpDeskInfo.sub_type and (helpDeskInfo.sub_type.code or ""):lower() == "timeout" then
				-- Si es un error por timeout, quiere decir que no se ha podido comunicar con HelpDesk, entonces debemos cancelar la actualizacion de todos los usuarios.
				forceCancel = true
				err = {
					result = "CONNECTION TIMEOUT",
					reason = "The connection time against HelpDesk has expired."
				}
			end
		-- actualizo la lista de usuarios que NO se han actualizado, porque HELPDESK ha retornado error.
			table.insert(updateErrorsList, {
				userId = user.Id,
				err = err
			})
		end
		-- Actualizo el status de la actualizacion, cada N loops (usuarios actualizados contra helpdesk)
		i = i + 1
		if i == updateStatusEvery then
			updateInfo.NumberUpdatedRows = #updatedSuccessList
			updateInfo.NumberFailedRows = #updateErrorsList
			updateInfo.LastUpdateDate = LuaOrm.Date():toString()
			UpdateVCCStatusHelpDeskIntegrations(updateInfo)
			i = 0
			-- local encodedStatus = GetVCCStatusHelpDeskIntegration()
			-- LogDebug("----> index %d,  status: '%s'", index, encodedStatus)
		end
	end

	keyspace:Close()

	-- Finalizo el status en redis
	updateInfo.Status = "FINISHED"
	UpdateVCCStatusHelpDeskIntegrations(updateInfo)

	if #updateErrorsList > 0 then
		-- La actualización se realizó de forma parcial, ya que la actualización de algunos usuarios ha fallado en HELPDESK.
		LogWarning("[ADMINISTRATION.USERS:UpdateHelpDeskIntegrations] Only some Users have been updated in Vcc '%s'/HelpDesk.", vcc)
		return false, {result = "ONLY SOME USERS UPDATED", reason = "Only some Users have been updated.", update = {success = updatedSuccessList, errors = updateErrorsList}}
	else
		LogInfo("[ADMINISTRATION.USERS:UpdateHelpDeskIntegrations] All users have been updated in Vcc '%s'/HelpDesk.", vcc)
		return true, {result = "OK", update = {success = updatedSuccessList, errors = updateErrorsList}}
	end
end
