local m_definitions = {
	base_campaign = {
		isSimple = true,
		fields   = {
			{
				field_id = "Id",
				validations = {
					id_without_dots = "Only letters, numbers and dashes are allowed in the campaign Id.",
					html_malicious_code = "Possible malicious code in the campaign Id.",
					maxLength           = {
						--[[
							El largo maximo permitido de las campanas es:
								64 + 1 + 20 = 85

								64 = El largo maximo permitido en el nombre de la campana
								1  = El largo del @
								20 = El largo maximo permitido de el ID del VCC en la CF VCCs.
							
							Ej:
								camp1_larga_3456789012345678901234567890123456789012345678901234@vcc_with_name_largo1
						]]--
						max    = 85,
						reason = "Allowed %s characters max length in campaign Id. Current length is %s characters."
					}
				}
			},
			{
				field_id = "Name",
				validations = {
					id_without_dots     = "Only letters, numbers and dashes are allowed in the campaign Name.",
					html_malicious_code = "Possible malicious code in the campaign Name.",
					maxLength           = {
						max    = 64,
						reason = "Allowed %s characters max length in campaign Name. Current length is %s characters."
					}
				}
			},
			{
				field_id = "Description",
				validations = {
					html_malicious_code = "Possible malicious code in the campaign Description."
				}
			}
		}
	},
	attention_levels = {
		isSimple = true,
		GetEntity = function(data)
			return data.attention_levels
		end,
		fields   = {
			{
				field_id = "name",
				validations = {
					id = "Only letters, numbers, dashes and dots are allowed in the attention level Name.",
					html_malicious_code = "Possible malicious code in the attention level Name."
				}
			},
			{
				field_id = "description",
				validations = {
					html_malicious_code = "Possible malicious code in the attention level Description."
				}
			}
		}
	},
	disposition_codes = {
		isSimple = true,
		GetEntity = function(data)
			return data.dispositionCodes
		end,
		fields   = {
			{
				field_id = "name",
				inDeep = true,
				validations = {
					id_with_spaces_customs_characters = "Only letters, spaces, numbers and dashes are allowed in the Name.",
					html_malicious_code = "Possible malicious code in the disposition code Name."
				}
			}
		}
	},
	rule = {
		isSimple = true,
		GetEntity = function(data)
			return data.rules
		end,
		fields   = {
			{
				field_id = "name",
				validations = {
					id = "Only letters, numbers, dashes and dots are allowed in the rule Name.",
					html_malicious_code = "Possible malicious code in the rule Name."
				}
			},
			{
				field_id = "id",
				validations = {
					id = "Only letters, numbers, dashes and dots are allowed in the rule Id.",
					html_malicious_code = "Possible malicious code in the rule Id."
				}
			},
			{
				field_id = "description",
				validations = {
					html_malicious_code = "Possible malicious code in the rule Description."
				}
			}
		}
	},
	label = {
		isSimple = true,
		GetEntity = function(data)
			return data.labels
		end,
		fields   = {
			{
				field_id = "name",
				validations = {
					id = "Only letters, numbers, dashes and dots are allowed in the label Name.",
					html_malicious_code = "Possible malicious code in the label Name."
				}
			},
			{
				field_id = "id",
				validations = {
					id = "Only letters, numbers, dashes and dots are allowed in the label Id.",
					html_malicious_code = "Possible malicious code in the label Id."
				}
			}
		}
	},
	account = {
		isSimple = true,
		GetEntity = function(data)
			return data.account
		end,
		fields   = {
			{
				field_id = "id",
				validations = {
					maxLength           = {
						--[[
							El largo maximo permitido de las cuentas es:
								NN + 1 + 64 + 1 + 20 + 1 + 64 = 200

								NN  = El prefijo de la tipo de canal de cuenta lo dejamos aleatoria ya que depende del tipo de canal.
								1   = El separador con el Nombre de la Campana.
								1   = El largo del @ que separa con el ID del VCC.
								20  = El largo maximo permitido de el ID del VCC en la CF VCCs.
								1   = El largo del separado entre el VCC y el Name de la cuenta.
								64  = El Name de la cuenta.

							Ejemplo de largo de una cuenta de WebChat:
								WEBCHAT_camp1_larga_3456789012345678901234567890123456789012345678901234@vcc_with_name_largo1_mywebchat_123456789012345678901234567890123456789012345678901234
						]]--
						max    = 200,
						reason = "Allowed %s characters max length in account Id. Current length is %s characters."
					}
				}
			},
			{
				field_id    = "name",
				validations = {
					maxLength           = {
						max    = 64,
						reason = "Allowed %s characters max length in account Name. Current length is %s characters."
					}
					--[[
						Estas 2 validaciones quedan comentadas ya que son validas para todas las cuentas, menos para las cuentas de EMAIL.
						Ya que aca se debe validar si es un correo electronico valido.

						Para relizar este tipo de validacion, hay que analizar cual es la mejor opcion.
						Seguramente que en lugar de ser un texto, debería ser una tabla con funciones para que determine que tipo de validacion hay que realizar.

						A esta validacion la deberia de determinar la rocka helpers.

						Pero como dije hay que analizarlo bien.
					]]--
					-- id_without_dots     = "Only letters, numbers and dashes are allowed in the account Name.",
					-- html_malicious_code = "Possible malicious code in the account Name.",
				}
			}
		}
	}
}

local function FindFiledValidation (fieldType, fieldName)
	local definitionType = m_definitions[fieldType]
	if not definitionType then return end
	local fieldValidation
	for _, def in ipairs(definitionType.fields or {}) do
		if def.field_id == fieldName then
			fieldValidation = def
			break
		end
	end
	return fieldValidation, definitionType
end

return {
	FindFiledValidation = FindFiledValidation,
	GetDefinitions      = function() return m_definitions end
}
