local m_models = require("apps.administration.model"):GetModels()
local m_agentModels = require("apps.agent.model"):GetModels()
local m_agentCampController = require("apps.agent.controllers.campaigns")
module(..., package.seeall, inconcert.orbit.application.Decorate)
local utils = require "shared.apps.utils"
local permissions = require "shared.permissions.permissions"
local m_moduleLocation = select(2, ...)
m_htmlRender = nil
m_appLocation = nil


--[[ ---------------------------------------------------------------------------------------
	Accounts
--------------------------------------------------------------------------------------------]]

local function RenderAccounts (web)
	_M.response.DontCache(web)
	local env = { _T = m_htmlRender.extension.makeTextTranslator() }
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc

	LogInfo("Render form Accounts for user %s, role %s", userId, inconcert.authUser.userGroup)

	local html_file = m_appLocation .. "/pages/frm_vcc_content.html"
	local content = m_htmlRender:RenderPageLP(html_file, web, env)
	if not content then
		LogError("HTML Accounts does not exist: %s", html_file)
	end
	return content
end

--Returns all campaigns in the VCC without filtering
local function GetAllCampaigns ( web )
	local vcc = inconcert.authUser.virtualContactCenter
	local fullUserId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[ADMINISTRATION.VCCS:GETALLCAMPAIGNS] 1 - VCC CAMPAIGNS for user '%s', group '%s'. VCC: '%s'", fullUserId, inconcert.authUser.userGroup, vcc)

	return m_models.Campaigns.GetVCCCampaigns( vcc ) or {}
end

--Returns all accounts in the VCC without filtering
local function GetAllAccounts ( web )
	local vcc = inconcert.authUser.virtualContactCenter
	local fullUserId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[ADMINISTRATION.VCCS:GETALLACCOUNTS] 1 - VCC ACCOUNTS for user '%s', group '%s'. VCC: '%s'", fullUserId, inconcert.authUser.userGroup, vcc)

	return m_models.VCCs.GetAccounts( vcc ) or {}
end

--Returns all accounts in the VCC
local function SearchAccounts ( web )
	local vcc = inconcert.authUser.virtualContactCenter
	local fullUserId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local startTime = os.time()

	LogInfo("[ADMINISTRATION.VCCS:SEARCHACCOUNTS] 1 - (START) Search VCC ACCOUNTS for user '%s', group '%s'. Posted data: '%s'", fullUserId, inconcert.authUser.userGroup, web.input["post_data"])


	local postData = web.request

	local accountType = ( postData.special_values.accountType or "" ):upper()
	--Get all account from model
	local accounts = m_models.VCCs.GetAccounts( vcc, accountType ) or {}
	local totalUnfilteredRows = utils.GetRowCount(accounts)
	local totalAdded = 0
	local filteredRows = {}

	LogInfo("[ADMINISTRATION.VCCS:SEARCHACCOUNTS] 2 - Filtering VCC ACCOUNTS")

	for accountId, account in pairs( accounts ) do
		account.Id = account.uid or accountId
		account.Name = account.Name or account.Id
		if utils.MatchFilter( account, postData ) then
			totalAdded = totalAdded + 1
			table.insert(filteredRows, account)
		end
	end

	LogInfo("[ADMINISTRATION.VCCS:SEARCHACCOUNTS] 3 - Sorting VCC ACCOUNTS")

	--include only rows that belong to logged user vcc
	local response = {
		rows = filteredRows,
		last = "",
		total_rows = totalAdded,
		total_unfiltered_rows = totalUnfilteredRows
	}
	local responseParams = {
		MakeSortingValue = function(row)
			assert(row)
			return tostring(row.Name or row.Id):upper()
		end
	}
	local resultData = utils.MakeSearchResponse(response, postData, responseParams)

	local endTime = os.time()
	local elapsedTime = endTime - startTime
	LogInfo("[ADMINISTRATION.VCCS:SEARCHACCOUNTS] 4 - (END) Search VCC ACCOUNTS. Retrieve %d of %d matched rows. Total Unfiltered Rows: %d. ELAPSED Time '%s' seconds.", #resultData.rows, resultData.total_rows, resultData.total_unfiltered_rows, elapsedTime)

	return resultData
end

local function GetAccount ( web, accountType, accountId )
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc

	LogInfo("[ADMINISTRATION.VCCS:GETACCOUNT] Retrieving VCC accountId %s, type:%s, user %s, vcc %s", accountId, accountType, userId, vcc)

	local accounts = m_models.VCCs.GetAccounts( vcc, accountType, accountId ) or {}
	LogInfo("Account %s: %s", accountId, json.encode(accounts))

	if next(accounts) then
		accounts.Name = accounts.Name or accountId
		return accounts
	else
		web.status = 404
		return {result = "NOT_FOUND"}
	end
end

--[[
	Add/Modifies accounts in a vcc
	posted data must have the form:
{
	accountId1 = {
		Description = "account description 1",
		DefaultValue = "1", //must be a numeric string
	},
	accountId2 = {
		Description = "account description 2",
		DefaultValue = "4", 
	},
	...
}

]]
local function SaveAccounts ( web )
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc

	LogInfo( "Save VCC Accounts, user %s, vcc %s. post_data: %s", userId, vcc, web.input["post_data"] )

	local postData = web.request

	local keyspace = utils.GetDatasource():CassandraConnection( "Agent" )

	local validate = function( account, is_new )
		if is_new then
			-- 1 - valido contra las mismas accounts a nivel vcc
			local existingAccount = m_models.VCCs.GetAccounts( vcc, account.proxyType, account.uid, keyspace )
			if existingAccount then
				local err = {
					result = "ACCOUNT_ALREADY_EXISTS_IN_CURRENT_VCC",
					reason = string.format( "Unable to save vcc account. Account id: '%s', type: '%s', Already exists in current VCC: %s", account.uid, account.proxyType, vcc )
				}
				return false, err
			end

			-- 2 - valido contra las accounts de las campanas
			local responseAccount, campaign_id = m_agentModels.Campaigns.FindAccountByUID( account, keyspace )
			if responseAccount then
				local campaign_name = m_models.Campaigns.GetSpecificColumn( campaign_id, "Name", keyspace ) or campaign_id
				local err = {
					result = "ACCOUNT_IS_CONFIGURED_IN_A_CAMPAIGN",
					campaign = campaign_name,
					reason = string.format( "Unable to save vcc account. Account '%s' type: '%s', is configured in '%s' campaign.", account.uid, account.proxyType, tostring(campaign_name) )
				}
				return false, err
			end
		end

		-- 3 - realizo validaciones especificas del tipo de account.
		--      - Que el "user" de la configuracion "outgoing", no se encuentre utilizado.
		if account.proxyType == "MAIL" then
			return m_agentCampController.MailAccountValidation( account, nil, keyspace )
		end

		return true
	end

	local accounts = {}
	for accountId, account in pairs(postData) do
		local success, err = validate( account, account.isNew )
		if not success then
			keyspace:Close()
			web.status = err.web_status or 400
			LogError( err.reason )
			return err
		end

		if account.isNew then
			account.isNew = nil
		end

		accounts[accountId] = account
	end

	m_models.VCCs.SaveAccounts( vcc, accounts, keyspace )

	keyspace:Close()
	return { result = "OK" }
end

local function InUseAccount ( vcc_accountId, ks_agent, ks_summarizer )
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName .. "@" .. vcc

	LogInfo("Validate in use Outgoing Account %s, user %s, vcc %s.", vcc_accountId, userId, vcc)

	local vcc_accountId = vcc_accountId:upper()

	-- 1 - valido si se encuentra en uso en alguna campana/account de la campana/mensaje de la misma (ejemplo: mensaje de finalizacion)
	local vccCampaigns = m_models.Campaigns.GetVCCCampaigns( vcc, ks_agent )

	for campaignId in pairs( vccCampaigns ) do
		local accounts = m_models.Campaigns.GetCampaignAccounts( campaignId, ks_agent ) or {}
		for _, account in pairs( accounts or {} ) do
			local campaignIdAtVCC = utils.AppendVcc( campaignId , vcc)
			local account_messages = m_models.Campaigns.GetAccountMessages( campaignIdAtVCC, account.id, ks_agent )
			for message_id, message in pairs( account_messages or {} ) do
				if ( message.outgoing_mail or "" ):upper() == vcc_accountId then
					return true, string.format( "Unable to delete '%s' vcc account. Account is being used in campaign '%s', account '%s', message '%s'.",
						vcc_accountId, campaignId, account.id, message_id )
				end
			end
		end
	end

	-- 2 - valido si se encuentra en uso en los schedules
	local schedules = utils.GetAllColumns({ keyspace = ks_summarizer or "SummarizerData", columnFamily = "ReportSchedules", startKey = "", endKey = "" })
	for sch_id, sch in pairs(schedules) do
		if vcc == sch.Vcc and ( sch.EmailFrom or "" ):upper() == vcc_accountId then
			return true, string.format( "Unable to delete '%s' vcc account. Account is being used in schedule '%s'.", vcc_accountId, sch_id )
		end
	end

	-- no se encuentra en uso
	return false
end

--[[
Remove accounts from a vcc
postedData must be of the form: ["accountId1", "accountId2", ...]
]]
local function RemoveAccounts ( web )
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc

	LogInfo("Remove VCC Outgoing Accounts, user %s, vcc %s", userId, vcc)

	local ks_agent = utils.GetDatasource():CassandraConnection( "Agent" )
	local ks_summarizer = utils.GetDatasource():CassandraConnection( "SummarizerData" )

	local validate = function( accountId )
		local in_use, reason = InUseAccount( accountId, ks_agent, ks_summarizer )
		if in_use then
			local err = {
				result = "ACCOUNT_BEING_USED",
				reason = reason
			}
			return false, err
		end
		return true
	end

	local postData = web.request
	local accountType = postData.accountType
	for _, accountId in ipairs( postData.rows ) do
		local success, err = validate( accountId )
		if not success then
			ks_agent:Close()
			ks_summarizer:Close()
			web.status = err.web_status or 400
			LogError( err.reason )
			return err
		end
		local result, err = m_models.VCCs.RemoveAccounts( vcc, accountType, accountId, ks_agent )
		if not result then
			ks_agent:Close()
			ks_summarizer:Close()
			local reason = "Error deleting VCC Account '"..tostring(accountId).."'. Account does not exist. Error type:" .. tostring(err)
			LogError(reason)
			web.status = 400
			return { result = "ACCOUNT_DOES_NOT_EXIST", reason = reason }
		end
	end

	ks_agent:Close()
	ks_summarizer:Close()
	return { result = "OK" }
end

--[[ ---------------------------------------------------------------------------------------
	Clients Windows Layout Design
--------------------------------------------------------------------------------------------]]

local function RenderClientsDesign (web)
	_M.response.DontCache(web)
	local env = { _T = m_htmlRender.extension.makeTextTranslator() }
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc

	LogInfo("[ADMINISTRATION.VCCS:RenderClientsDesign] Render form Clients Design for user %s, role %s", userId, inconcert.authUser.userGroup)

	local html_file = m_appLocation .. "/pages/frm_vcc_clients_design_content.html"
	local content = m_htmlRender:RenderPageLP(html_file, web, env)
	if not content then
		LogError("[ADMINISTRATION.VCCS:RenderClientsDesign] HTML Clients Design does not exist: %s", html_file)
	end
	return content
end

--Returns all clients Layouts in the VCC
local function SearchClientsDesign (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local fullUserId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local startTime = os.time()

	LogInfo("[ADMINISTRATION.VCCS:SearchClientsDesign] 1 - (START) Search VCC Clients Design for user '%s', group '%s'. Posted data: '%s'", fullUserId, inconcert.authUser.userGroup, web.input["post_data"])

	local postData = web.request

	--Get all account from model
	local clientsLayouts = m_models.VCCs.LoadClientsLayouts(vcc) or {}
--LogDebug("vcc '%s' clients layouts: %s", vcc, json.encode(clientsLayouts))
	local totalUnfilteredRows = utils.GetRowCount(clientsLayouts)
	local totalAdded = 0
	local filteredRows = {}

	LogInfo("[ADMINISTRATION.VCCS:SearchClientsDesign] 2 - Filtering VCC CLIENTS DESIGN")

	for layoutId, layout in pairs(clientsLayouts) do
		local layoutInfo = {
			Id = layoutId
		}
		layoutInfo.Name = layoutInfo.Id
		if utils.MatchFilter(layoutInfo, postData) then
			totalAdded = totalAdded + 1
			table.insert(filteredRows, layoutInfo)
		end
	end

	LogInfo("[ADMINISTRATION.VCCS:SearchClientsDesign] 3 - Sorting VCC CLIENTS DESIGN")

	--include only rows that belong to logged user vcc
	local response = {
		rows = filteredRows,
		last = "",
		total_rows = totalAdded,
		total_unfiltered_rows = totalUnfilteredRows
	}

	local resultData = utils.MakeSearchResponse(response, postData)

	local endTime = os.time()
	local elapsedTime = endTime - startTime
	LogInfo("[ADMINISTRATION.VCCS:SearchClientsDesign] 4 - (END) Search VCC CLIENTS DESIGN. Retrieve %d of %d matched rows. Total Unfiltered Rows: %d. ELAPSED Time '%s' seconds.", #resultData.rows, resultData.total_rows, resultData.total_unfiltered_rows, elapsedTime)

	return resultData
end

local function GetClientsDesign (web, layoutId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc

	LogInfo("[ADMINISTRATION.VCCS:GetClientsDesign] Retrieving VCC Clients Design %s, user %s, vcc %s", layoutId, userId, vcc)

	local layout = m_models.VCCs.GetClientLayout(vcc, layoutId) or {}
	--LogInfo("layoutId %s: %s", layoutId, json.encode(layout))

	if next(layout) then
		return layout
	else
		web.status = 404
		return {result = "NOT_FOUND"}
	end
end

local function SaveClientsDesign (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc

	LogInfo("[ADMINISTRATION.VCCS:SaveClientsDesign] Save VCC Clients Design, user %s, vcc %s.", userId, vcc)
	--LogDebug("[ADMINISTRATION.VCCS:SaveClientsDesign] Save VCC Clients post_data: %s", web.input["post_data"])

	local keyspace = utils.GetDatasource():CassandraConnection("Agent")

	local Validate = function(layout)
		return true
	end

	local postData = web.request

	local layouts = {}
	for layoutId, layout in pairs(postData) do
		if type(layout) == "table" then
			local success, err = Validate(layout)
			if not success then
				keyspace:Close()
				web.status = err.web_status or 400
				LogError(err.reason)
				return err
			end
			if layout.isNew then
				layout.isNew = nil
			end
			layouts[layoutId] = layout
		end
	end

	m_models.VCCs.SaveClientsLayouts(vcc, layouts, keyspace)

	keyspace:Close()
	return {result = "OK"}
end

local function RemoveClientsDesign ( web )
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName.."@"..vcc

	LogInfo("[ADMINISTRATION.VCCS:RemoveClientsDesign] Remove VCC Clients Design, user %s, vcc %s. Posted data: '%s'", userId, vcc,web.input["post_data"])

	local ksAgent = utils.GetDatasource():CassandraConnection("Agent")

	local Validate = function(layoutId)
		return true
	end

	local layouts = web.request
	for _, layoutId in ipairs(layouts) do
		local success, err = Validate(layoutId)
		if not success then
			ksAgent:Close()
			web.status = err.web_status or 400
			LogError(err.reason)
			return err
		end
		local result, err = m_models.VCCs.RemoveClientsLayout(vcc, layoutId, ksAgent)
		if not result then
			ksAgent:Close()
			local reason = "Error deleting VCC Clients Design '"..tostring(layoutId).."'. Clients Design does not exist. Error type:" .. tostring(err)
			LogError(reason)
			web.status = 400
			return {result = "LAYOUT_DOES_NOT_EXIST", reason = reason}
		end
	end

	ksAgent:Close()
	return {result = "OK"}
end

-------------------------------------------------------------------------------------------------------------

function MapUris (module, htmlRender, prefix, appLocation)
	local LogRequest = require("apps.agent.requestFilters").LogRequest
	--VCC Campaigns
	module:dispatch_get(utils.JsonReply..LogRequest..permissions:CheckPermission("administration", "viewcampaigns", GetAllCampaigns), prefix.."/campaigns/get_all_campaigns/")	

	-- VCC Accounts
	module:dispatch_get(RenderAccounts, prefix .. "/accounts/frm/render/")
	module:dispatch_post(utils.JsonValidatorReply..LogRequest..SearchAccounts, prefix.."/accounts/search/")
	module:dispatch_get(utils.JsonReply..LogRequest..permissions:CheckPermission("administration", "view_vcc_accounts", GetAccount), prefix.."/accounts/get/([^/]+)/(.*)")
	module:dispatch_post(utils.JsonValidatorReply..LogRequest..permissions:CheckPermission("administration", "save_vcc_accounts", SaveAccounts), prefix.."/accounts/save/")
	module:dispatch_post(utils.JsonValidatorReply..LogRequest..permissions:CheckPermission("administration", "delete_vcc_accounts", RemoveAccounts), prefix.."/accounts/delete/")
	module:dispatch_get(utils.JsonReply..LogRequest..permissions:CheckPermission("administration", "view_vcc_accounts", GetAllAccounts), prefix.."/accounts/get_all_accounts/")

	-- VCC Client Widnows Layout Redesign
	module:dispatch_get(RenderClientsDesign, prefix .. "/clients_design/frm/render/")
	module:dispatch_post(utils.JsonValidatorReply..LogRequest..SearchClientsDesign, prefix.."/clients_design/search/")
	module:dispatch_get(utils.JsonReply..LogRequest..permissions:CheckPermission("administration", "view_clients_windows_layout_design", GetClientsDesign), prefix.."/clients_design/get/(.*)")
	module:dispatch_post(utils.JsonValidatorReply..LogRequest..permissions:CheckPermission("administration", "save_clients_windows_layout_design", SaveClientsDesign), prefix.."/clients_design/save/")
	module:dispatch_post(utils.JsonValidatorReply..LogRequest..permissions:CheckPermission("administration", "delete_clients_windows_layout_design", RemoveClientsDesign), prefix.."/clients_design/delete/")

	m_htmlRender = htmlRender
	m_appLocation = appLocation
end
