require "inconcert.translators"
local lua_cassandra = require "lua_cassandra"
require "redis"
local WrapRedis = require "wrapredis"

local utils = require "shared.apps.utils"

local m_moduleLocation = select(2, ...)

module(..., package.seeall)

local m_dataSource
local m_models = {}
local m_isCreated = false

local function InitDatasource ()
	local redisConfig = utils.GetConnectionConfig("redis", "administration")
	if not redisConfig then
		error("+Unable to initialize administration app, connection to redis failed")
	end
	LogInfo("[ADMINISTRATION.MODEL:InitDatasource] Connecting to redis on host: '%s', port:'%s'.", redisConfig.host, redisConfig.port)
	--LogDebug("[ADMINISTRATION.MODEL:InitDatasource] --> Connecting to redis pwd:'%s'.", redisConfig.pwd)
	--start in nonblocking mode
	local redis = WrapRedis(redisConfig.host, redisConfig.port, redisConfig.pwd)
	if not redis then
		error("-Unable to initialize administration app, connection to redis failed")
	end
	--init local datasource table
	m_dataSource = {
		cassandra = lua_cassandra,
		redis = redis
	}
end

function GetDatasource ()
	if not m_dataSource then
		InitDatasource()
	end
	return m_dataSource
end

function GetRedisClient ()
	return assert(m_dataSource.redis)
end

--recupera la lista de modelos del directorio models
local function GetModelsFromFilesystem ()
	local modelsDir = m_moduleLocation .. "/models"
	if not utils.isDirectory(modelsDir) then
		return {}
	end
	local models = {}
	for file in lfs.dir(modelsDir) do
		if file ~= "." and file ~= ".." then
			local f = modelsDir..'/'..file
			local attr = lfs.attributes (f)
			if attr.mode ~= "directory" then
				local modelName = string.match(file, [[(.+).lua]])
				if (modelName) then
					models[modelName] = modelName
				end
			end
		end
	end
	return models
end

--///////////////////////////////////////////////////////////////////////////////////
-- arma la tabla de modelos definidos
function CreateModelTable()
	--LogInfo("[ADMINISTRATION.MODEL:CreateModelTable] Creating cassandra models")

	local datasource = GetDatasource()
	if not m_isCreated then
		for key, controllerDef in pairs(GetModelsFromFilesystem()) do
			--LogInfo("[ADMINISTRATION:CreateModelTable] Creating cassandra model: '%s'", key)
			local model = require("apps.administration.models." .. key)
			if model.GetDefinition then
				m_models[model.GetName()] = model.GetDefinition(datasource)
			end
		end
		m_isCreated = true
	else
		--if model modules were already loaded just overwrite their inner datasource
		--with the newly changed one
		for modelName, model in pairs(m_models) do
			model.GetDefinition(datasource)
		end
	end
end

--///////////////////////////////////////////////////////////////////////////////////
-- 
function IsCreatedModel ()
	return m_isCreated
end

--///////////////////////////////////////////////////////////////////////////////////
-- 
function GetModels()
	return m_models
end

--///////////////////////////////////////////////////////////////////////////////////
-- 
function BuildModels (self)
	InitDatasource()
	utils.SetupConnections({ cassandra = lua_cassandra })
	CreateModelTable()
	LogInfo("Administration model built")
	return true
end
