module(..., package.seeall)

local utils = require "shared.apps.utils"
local m_datasource

function GetName()
	return "AgentNotifications"
end

function GetDefinition(datasource)
	m_datasource = datasource
	return _M
end

function GetAgentScopeNotifications(scopeId, vcc, keyspace)
	vcc = vcc or "system"
	local scopes = utils.GetAllColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "AgentNotifications",
		superColumns = true,
		key = vcc,
		columns = {scopeId}
	}
	if scopes[scopeId] then
		return scopes[scopeId]
	elseif vcc ~= "system" then
		return GetAgentNotifications(scopeId, "system", keyspace)
	else
		return nil
	end
end

function SaveAgentNotifications(vcc, scopes, keyspace)
	utils.SaveRows {
		keyspace = keyspace or "Agent",
		columnFamily = "AgentNotifications",
		superColumns = true,
		rows = {
			[vcc] = scopes
		}
	}
	return true
end

function RemoveAgentNotifications(vcc, scopes, keyspace)
	utils.RemoveColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "AgentNotifications",
		superColumns = true,
		rows = {
			[vcc] = scopes
		}
	}
	return true
end

local function __QueueAgentMessage(redis, scope, eventType, reference, level, agent_id, message)
	local queue_id = ("AMQ::Agent::%s::queue"):format(agent_id)
	local notifications_queue_id = ("AMQ::Agent::%s::notifications_queue"):format(agent_id)
	local notifications_references_queue_id = ("AMQ::Agent::%s::notifications_references_queue"):format(agent_id)
	local key_next_id = ("AMQ::Agent::%s::next_id"):format(agent_id)
	
	local notification_config = redis:hget("authFilter:userNotifications:AGENT", agent_id) or ""
	local update_reference = false
	if notification_config ~= "" then
		if level == "ALWAYS" then
			update_reference = true
		elseif reference and level == "REFERENCE" then
			if redis:zscore(notifications_references_queue_id, reference) then
				notification_config = ""
			else
				update_reference = true
			end
		elseif level == "GLOBAL" then
			if redis:exists(notifications_queue_id) then
				notification_config = ""
			end
		end
	end
	
	local id = redis:get(key_next_id ) or 0
	redis:zadd(queue_id, id, message)
	if notification_config ~= "" then
		redis:zadd(notifications_queue_id, id, "")
		if reference and update_reference then
			redis:zadd(notifications_references_queue_id, id, reference)
		end
	end
	redis:incr(key_next_id)
	redis:publish("AMQ::Agent::channels::messages_available", agent_id)

	return id, notification_config
end

function QueueAgentsMessage(scope, eventType, reference, level, agents, message)
	local notifiedAgents = {}
	m_datasource.redis:transaction({watch = key_next_id, cas = true, retry = math.huge}, function(redis)
		for _, agentAtVcc in pairs(agents) do
			local messageId, notificationConfig = __QueueAgentMessage(redis, scope, eventType, reference, level, agentAtVcc, message)
			notifiedAgents[agentAtVcc] = {
				messageId = messageId,
				notificationConfig = notificationConfig
			}
		end
	end)
	local serverAgents = {}
	for agentAtVcc, agentResult in pairs(notifiedAgents) do
		if agentResult.notificationConfig ~= "" then
			local notificationConfig = json.decode(agentResult.notificationConfig)
			notificationConfig.messageId = agentResult.messageId
			serverAgents[agentAtVcc] = notificationConfig
		end
	end
	return { notifiedAgents = notifiedAgents, serverAgents = serverAgents }
end

function CleanNotifications(userName, vcc)
	m_datasource.redis:del(("AMQ::Agent::%s@%s::queue"):format(userName, vcc))
	m_datasource.redis:del(("AMQ::Agent::%s@%s::notifications_queue"):format(userName, vcc))
	m_datasource.redis:del(("AMQ::Agent::%s@%s::notifications_references_queue"):format(userName, vcc))
end
