module(..., package.seeall)

local utils = require "shared.apps.utils"
local m_datasource

function GetName()
	return "Channels"
end

function GetDefinition (datasource)
	m_datasource = datasource
	return _M
end

function GetInteractionType ( key, keyspace )
	-- sample: "FACEBOOK@inconcert@camp1@agent10"
	assert(type(key) == "string")

	return utils.GetAllColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "InteractionTypes",
		key = key,
		rowCount = 10000
	}
end

function GetInteractionTypeByChannelSystemVCC ( channel, keyspace )
	-- sample: "FACEBOOK@system"

	assert(type(channel) == "string")

	channel = channel:upper()

	-- 1 creo la key channel, del system VCC
	local key_system_vcc_channel = channel .. "@system"
	-- Busco los valores sin override del channel (los valores del sistema)
	local system_vcc_result = GetInteractionType( key_system_vcc_channel )
	if not utils.TableIsEmpty(system_vcc_result) then
		return system_vcc_result
	end

	return
end

function GetInteractionTypeByChannelCurrentVCC ( channel, keyspace )
	-- sample: "FACEBOOK@inconcert"

	assert(type(channel) == "string")

	channel = channel:upper()
	local vcc = inconcert.authUser.virtualContactCenter
	-- 1 creo la key channel, del current VCC
	local key_current_vcc_channel = channel .. "@" .. vcc
	-- Busco el override del channel en el vcc
	local vcc_result = GetInteractionType( key_current_vcc_channel )
	if not utils.TableIsEmpty(vcc_result) then
		return vcc_result
	end

	return
end

function GetInteractionTypeByChannelCampaign ( channel, simple_campaign_id, keyspace )
	-- sample: "FACEBOOK@inconcert@camp1"

	assert(type(channel) == "string")
	assert(type(simple_campaign_id) == "string")

	channel = channel:upper()
	local vcc = inconcert.authUser.virtualContactCenter
	-- 1 creo la key channel, del override de la campana
	local key_campaign = channel .. "@" .. vcc .. "@" .. simple_campaign_id
	-- Busco el override de la campana segun el channel y el vcc
	local campaign_result = GetInteractionType( key_campaign )
	if not utils.TableIsEmpty(campaign_result) then
		return campaign_result
	end

	return
end

function GetInteractionTypeByChannelCampaignMember ( channel, simple_campaign_id, simple_user_id, keyspace )
	-- sample: "FACEBOOK@inconcert@camp1@agent1"

	assert(type(channel) == "string")
	assert(type(simple_campaign_id) == "string")
	assert(type(simple_user_id) == "string")

	channel = channel:upper()
	local vcc = inconcert.authUser.virtualContactCenter
	-- 1 creo la key channel, del override del member de la campana
	local key_user = channel .. "@" .. vcc .. "@" .. simple_campaign_id .. "@" .. simple_user_id
	-- Busco el override del member de la campana segun el channel y el vcc
	local user_result = GetInteractionType( key_user )
	if not utils.TableIsEmpty(user_result) then
		return user_result
	end

	return
end

function GetInteractionTypes ( keyspace )
	return utils.GetAllColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "InteractionTypes",
		startKey = "",
		endKey = "",
		rowCount = 20000
	}
end

function GetInteractionTypesByCampaign( full_campaign_id, keyspace )
	return utils.GetAllColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "InteractionTypesByCampaign",
		key = full_campaign_id,
		rowCount = 10000
	}
end

function GetInteractionTypesByUser ( full_user_id, keyspace )
	return utils.GetAllColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "InteractionTypesByUser",
		key = full_user_id,
		rowCount = 10000
	}
end

function SaveInteractionTypes ( channel_id, interaction_types, full_camp_id, full_user_id, keyspace )
	-- 1 - aca guardo los datos posta!!
	utils.SaveRows({
		keyspace = keyspace or "Agent",
		columnFamily = "InteractionTypes",
		rows = {
			[ channel_id ] = interaction_types
		}
	})

	-- 2 - aca guardo por indice (campanas)
	if full_camp_id then
		local data = { campaign = full_camp_id }
		if full_user_id then
			data.user = full_user_id
		end
		utils.SaveRows({
			keyspace = keyspace or "Agent",
			columnFamily = "InteractionTypesByCampaign",
			rows = {
				[ full_camp_id ] = { [ channel_id ] = data }
			}
		})
	end

	-- 3 - aca guardo por indice (users)
	if full_user_id then
		local data = { user = full_user_id }
		if full_camp_id then
			data.campaign = full_camp_id
		end
		utils.SaveRows({
			keyspace = keyspace or "Agent",
			columnFamily = "InteractionTypesByUser",
			rows = {
				[ full_user_id ] = { [ channel_id ] = data }
			}
		})
	end
end

function RemoveInteractionTypes ( key, full_camp_id, full_user_id, has_remove_all, keyspace )
	-- 1 - aca remuevo los datos posta!!
	utils.RemoveColumns({
		keyspace = keyspace or "Agent",
		columnFamily = "InteractionTypes",
		rows = { key }
	})

	if full_camp_id or full_user_id then
		local rows_to_remove
		-- 2 - aca remuevo del indice (campanas)
		--  Si llega a venir el Id del usuario, no remuevo del indice por campana, que esta clave tiene que estar como clave en el indice x campana
		if full_camp_id then
			if has_remove_all then
			-- remueve toda la columna
				rows_to_remove = { full_camp_id }
			else
			-- remueve unicamente la clave especifica
				rows_to_remove = { [ full_camp_id ] = { key } }
			end

			utils.RemoveColumns({
				keyspace = keyspace or "Agent",
				columnFamily = "InteractionTypesByCampaign",
				rows = rows_to_remove
			})
		end

		-- 3 - aca remuevo del indice (users)
		if full_user_id then
			if has_remove_all then
			-- remueve toda la columna
				rows_to_remove = { full_user_id }
			else
			-- remueve unicamente la clave especifica
				rows_to_remove = { [ full_user_id ] = { key } }
			end
			utils.RemoveColumns({
				keyspace = keyspace or "Agent",
				columnFamily = "InteractionTypesByUser",
				rows = rows_to_remove
			})
		end
	end
end
