local utils = require "shared.apps.utils"
module(..., package.seeall)

local m_datasource

function GetName ()
	return "Imports"
end

function GetDefinition (datasource)
	m_datasource = datasource
	return _M
end

local function GetDataSource ()
	if not m_datasource then
		local model = require "apps.administration.model"
		model.BuildModels(true)
	end
	return m_datasource
end

local function __MakeVCCStatusImportationId ()
	local key = string.format(
		"Importations:VCC:%s:Users:Importation:Status",
		inconcert.authUser.virtualContactCenter
	)
	--LogDebug("[__MakeVCCStatusImportationId]------------> key: '%s'", key)
	return key
end

function UpdateVCCStatusImportation (importation)
	assert(importation)
	local key = __MakeVCCStatusImportationId()
	if importation.Status == "RUNNING" then
		GetDataSource().redis:set(key, json.encode(importation))
		-- La key expira al minuto
		local MINUTE_IN_SEC = 60
		GetDataSource().redis:expire(key, MINUTE_IN_SEC)
	else
		GetDataSource().redis:del(key)
		if GetVCCStatusImportation() then
			LogWarning("[UpdateVCCStatusImportation] Unable to delete importation status '%s'", key)
			return false
		end
	end
	return key
end

function GetVCCStatusImportation ()
	local key = __MakeVCCStatusImportationId()
	return GetDataSource().redis:get(key)
end

function IsAnyImportInProgressInVCC ()
	local currentImportation = GetVCCStatusImportation()
	if not currentImportation then
		return false
	end
	return true
end

local function __MakeUploadedFileId (fileInfo)
	assert(type(fileInfo) == "table")
	assert(type(fileInfo.FileId) == "string")
	local key = string.format(
		"Importations:VCC:%s:Users:ImportationId:%s",
		inconcert.authUser.virtualContactCenter,
		fileInfo.FileId
	)
	--LogDebug("[__MakeUploadedFileId]------------> key: '%s'", key)
	return key
end

function SetTemporalUploadedFile (fileInfo, temporalFile)
	assert(fileInfo)
	assert(temporalFile)

	CollectAllMemory()
	local key = __MakeUploadedFileId(fileInfo)
	GetDataSource().redis:set(key, temporalFile)
	CollectAllMemory()
	-- La key expira a los 30 minutos
	local MINUTES_30_IN_SEC = 1800
	GetDataSource().redis:expire(key, MINUTES_30_IN_SEC)

	return key
end

function GetTemporalUploadedFile (fileInfo)
	assert(fileInfo)

	local key = __MakeUploadedFileId(fileInfo)
	return GetDataSource().redis:get(key)
end

function DeleteTemporalUploadedFile (fileInfo)
	assert(fileInfo)

	local key = __MakeUploadedFileId(fileInfo)
	GetDataSource().redis:del(key)
	CollectAllMemory()
	-- valida si el archivo se ha eliminado correctamente
	if GetTemporalUploadedFile(fileInfo) then
		LogWarning("[DeleteTemporalUploadedFile] Unable to delete temporal file: '%s'", key)
		return false
	end
	return true
end

function SaveUserImportation (importation, keyspace)
	assert(type(importation) == "table")
	assert(type(importation.Id) == "string")

	local rows = {}
	rows[importation.Id] = importation

	utils.SaveRows({
		keyspace     = keyspace or "Agent",
		columnFamily = "UsersImportations",
		rows         = rows
	})

	UpdateVCCStatusImportation(importation)
end

function LoadUsersImportations (keyspace)
	local vcc = inconcert.authUser.virtualContactCenter
	return utils.GetAllColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "UsersImportations",
		startKey = "",
		endKey = "",
		rowCount = 10000,
		CanAddRow= function(row)
			if (row.VCC or ""):lower() == vcc:lower() then
				return true
			end
			return false
		end
	}
end

function DeleteUserImportation (importationId, keyspace)
	assert(type(importationId) == "string")

	utils.RemoveColumns({
		keyspace = keyspace or "Agent",
		columnFamily = "UsersImportations",
		rows = {importationId}
	})

	return true
end
