module(..., package.seeall)
local utils = require "shared.apps.utils"
local m_datasource
local m_helpers = require "helpers"

function GetName ()
	return "SharedFiles"
end

local function GetDataSource ()
	if not m_datasource then
		local model = require "apps.administration.model"
		model.BuildModels(true)
	end
	return m_datasource
end

function GetDefinition (datasource)
	m_datasource = datasource
	return _M
end

function GetSizeLimit(vcc, keyspace)
	if m_helpers.utils.HasLicenseSystem() then
		local options = utils.Get_m_helpers_agentservice_options()
		local vccInfo = m_helpers.agentservice.GetVccLicense(options, vcc)
		return vccInfo.SharedFilesSpaceQuota or 0
	end
	local limit_s = utils.GetColumnValue {
		keyspace     = keyspace or "Agent",
		columnFamily = "VCCs",
		key          = vcc,
		column       = "SharedFilesSpaceQuota"
	}
	return limit_s ~= "" and tonumber(limit_s) or 0
end

function GetLastUsageInInteractions (vcc, fileId)
	assert(vcc)
	assert(fileId)
	fileId = fileId:lower()
	local lastUse = m_datasource.redis:get("SharedFileLastUse:" .. vcc .. ":" .. fileId)
	if lastUse ~= "" then
		return tonumber(lastUse)
	else
		return nil
	end
end

function RemoveLastUse(vcc, fileId)
	assert(vcc)
	assert(fileId)
	fileId = fileId:lower()
	m_datasource.redis:del("SharedFileLastUse:" .. vcc .. ":" .. fileId)
end

function GetFiles (vcc, fileId, keyspace, retrieveWithRelations)
	fileId = fileId and fileId:lower() or nil
	local files = utils.GetAllColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "SharedFiles",
		superColumns = true,
		key = vcc,
		columns = fileId and {fileId} or {}
	} or {}
	local relations
	if retrieveWithRelations then
		relations = GetSharedFilesRelations(vcc, files, keyspace)
	end
	for fileId, fileData in pairs(files) do
		fileData.LastUsageInInteractions = GetLastUsageInInteractions(vcc, fileId)
		if retrieveWithRelations and relations then
			fileData.Relations = relations[fileId:lower()]
		end
	end
	if fileId then
		return files[fileId] or {}
	end
	return files
end

function GetSharedFilesRelations (vcc, fileId, keyspace)
	--LogDebug("GetSharedFilesRelations 1 -> vcc %s fileId %s", vcc, fileId)
	local columns = {}
	if type(fileId) == "string" then
		fileId = fileId and fileId:lower() or nil
		columns = {fileId}
	elseif type(fileId) == "table" then
		local files = fileId
		for id, file in pairs(files) do
			table.insert(columns, id:lower())
		end
	end
	--LogDebug("GetSharedFilesRelations 2 -> columns %s", json.encode(columns))

	local relations = utils.GetAllColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "SharedFilesRelations",
		superColumns = true,
		key = vcc,
		columns = columns --fileId and {fileId} or {}
	} or {}

	--LogDebug("GetSharedFilesRelations 3 -> relations %s", json.encode(relations))

	if type(fileId) == "string" then
		return relations[fileId] or {}
	end
	return relations
end

local function GetSharedFilesAccountsMessagesRelations (accountMessageId, keyspace)
	assert(accountMessageId)
	local relation = utils.GetAllColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "SharedFilesAccountsMessagesRelations",
		key = accountMessageId
	} or {}
	return relation
end

local function GetSharedFilesFlowsDesignerRelations (flowId, keyspace)
	assert(flowId)
	local relation = utils.GetAllColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "SharedFilesFlowsDesignerRelations",
		key = flowId
	} or {}
	return relation
end

local function SaveSharedFilesRelations (vcc, files, keyspace)
	files = files or {}
	--LogDebug("SaveSharedFilesRelations: 1 - vcc %s files %s", vcc, json.encode(files))
	local currentVccSharedFilesRelations = GetSharedFilesRelations(vcc, nil, keyspace) or {}
	--LogDebug("SaveSharedFilesRelations: 2 - vcc %s currentVccSharedFilesRelations %s", vcc, json.encode(currentVccSharedFilesRelations))

	local campaignAccountMessages, flowDesigner
	for fileId, info in pairs(files) do
		--LogDebug("SaveSharedFilesRelations:   2.1  updating   fileId %s info %s", fileId, json.encode(info))

		if not currentVccSharedFilesRelations[fileId] then
			--LogDebug("SaveSharedFilesRelations:    2.2  updating  ")
			currentVccSharedFilesRelations[fileId] = info

			-- La CF CampaignAccountMessages, o la CF ProcessFlow
			for relationCF, relationInfo in pairs(info) do
				if not currentVccSharedFilesRelations[fileId][relationCF] then
					--currentVccSharedFilesRelations[fileId][relationCF] = relationInfo
				else
					-- La key de CampaignAccountMessages ej: CALL_campi6tel@vcc1_TELi6a_campi6tel@vcc1
					for id, relationData in pairs(relationInfo) do
					--	if not currentVccSharedFilesRelations[fileId][relationCF][id] then
						if relationCF == "CampaignAccountMessages" then
							local accountMessageId = id
							campaignAccountMessages = campaignAccountMessages or {}
							campaignAccountMessages[accountMessageId] = campaignAccountMessages[accountMessageId] or {}
							campaignAccountMessages[accountMessageId][fileId] = {vcc = vcc}
							--LogDebug("a---> campaignAccountMessages %s", json.encode(campaignAccountMessages))
						elseif relationCF == "FlowDesigner" then
							local flowId = id
							flowDesigner = flowDesigner or {}
							flowDesigner[flowId] = flowDesigner[flowId] or {}
							flowDesigner[flowId][fileId] = {vcc = vcc}
							--LogDebug("b---> flowDesigner %s", json.encode(flowDesigner))
						else
							assert(false, "Unrecognized SharedFile Relation " .. tostring(relationCF))
						end
					end
				end
			end
		else
			--LogDebug("SaveSharedFilesRelations:    2.3  updating  ")
			-- La CF CampaignAccountMessages
			for relationCF, relationInfo in pairs(info) do
				--LogDebug("SaveSharedFilesRelations:    2.3.1  relationCF %s  relationInfo %s ", relationCF, json.encode(relationInfo))
				if 	utils.TableIsEmpty(relationInfo) then
					--LogDebug("SaveSharedFilesRelations:      2.3.1.1")
					currentVccSharedFilesRelations[fileId][relationCF] = {}
				else
					--LogDebug("SaveSharedFilesRelations:      2.3.1.2")
					if not currentVccSharedFilesRelations[fileId][relationCF] then
						currentVccSharedFilesRelations[fileId][relationCF] = relationInfo
					else
						-- La key de CampaignAccountMessages ej: CALL_campi6tel@vcc1_TELi6a_campi6tel@vcc1
						for id, relationData in pairs(relationInfo) do
							if relationCF == "CampaignAccountMessages" then
								local accountMessageId = id
								if not currentVccSharedFilesRelations[fileId][relationCF][accountMessageId] then
									currentVccSharedFilesRelations[fileId][relationCF][accountMessageId] = relationInfo[accountMessageId]
								else
									currentVccSharedFilesRelations[fileId][relationCF][accountMessageId].messages = relationInfo[accountMessageId].messages
								end
								--
								campaignAccountMessages = campaignAccountMessages or {}
								campaignAccountMessages[accountMessageId] = campaignAccountMessages[accountMessageId] or {}
								campaignAccountMessages[accountMessageId][fileId] = {vcc = vcc}
								--LogDebug("a---> campaignAccountMessages %s", json.encode(campaignAccountMessages))

							elseif relationCF == "FlowDesigner" then

								local flowId = id
								flowDesigner = flowDesigner or {}
								flowDesigner[flowId] = flowDesigner[flowId] or {}
								flowDesigner[flowId][fileId] = {vcc = vcc}

								local flowId = id
								if not currentVccSharedFilesRelations[fileId][relationCF][flowId] then
									currentVccSharedFilesRelations[fileId][relationCF][flowId] = relationInfo[flowId]
								else
									currentVccSharedFilesRelations[fileId][relationCF][flowId].activities = relationInfo[flowId].activities
								end
								--
								flowDesigner = flowDesigner or {}
								flowDesigner[flowId] = flowDesigner[flowId] or {}
								flowDesigner[flowId][fileId] = {vcc = vcc}
								--LogDebug("b---> flowDesigner %s", json.encode(flowDesigner))
							else
								assert(false, "Unrecognized SharedFile Relation " .. tostring(relationCF))
								-- Flow Designer, etc
--								currentVccSharedFilesRelations[fileId][relationCF][id] = relationInfo[id]
							end
						end
					end
				end
			end
		end
	end

	--LogDebug("SaveSharedFilesRelations: 3 - vcc %s currentVccSharedFilesRelations %s", vcc, json.encode(currentVccSharedFilesRelations))
	--LogDebug("SaveSharedFilesRelations: 4 - vcc %s campaignAccountMessages %s", vcc, json.encode(campaignAccountMessages))
	--LogDebug("SaveSharedFilesRelations: 5 - vcc %s flowDesigner %s", vcc, json.encode(flowDesigner))

	utils.SaveRows {
		keyspace = keyspace or "Agent",
		columnFamily = "SharedFilesRelations",
		superColumns = true,
		rows = {
			[vcc] = currentVccSharedFilesRelations
		}
	}

	if campaignAccountMessages then
		utils.SaveRows {
			keyspace = keyspace or "Agent",
			columnFamily = "SharedFilesAccountsMessagesRelations",
			rows = campaignAccountMessages
		}
	end

	if flowDesigner then
		utils.SaveRows {
			keyspace = keyspace or "Agent",
			columnFamily = "SharedFilesFlowsDesignerRelations",
			rows = flowDesigner
		}
	end
end

local function RemoveSharedFilesAccountsMessagesRelations (rows, keyspace)
	--LogDebug("----> RemoveSharedFilesAccountsMessagesRelations - rows %s", rows)
	utils.RemoveColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "SharedFilesAccountsMessagesRelations",
		rows = rows
	}
end

local function RemoveSharedFilesFlowDesignerRelations (rows, keyspace)
	--LogDebug("----> RemoveSharedFilesFlowDesignerRelations - rows %s", rows)
	utils.RemoveColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "SharedFilesFlowsDesignerRelations",
		rows = rows
	}
end

local function RemoveSharedFilesRelations (vcc, filesToRemove, keyspace)
	utils.RemoveColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "SharedFilesRelations",
		superColumns = true,
		rows = {
			[vcc] = filesToRemove
		}
	}
end

function SaveFiles (vcc, files, keyspace)
	utils.SaveRows {
		keyspace = keyspace or "Agent",
		columnFamily = "SharedFiles",
		superColumns = true,
		rows = {
			[vcc] = files
		}
	}

--	SaveSharedFilesRelations(vcc, files, keyspace)
end

function RemoveFiles (vcc, filesToRemove, keyspace)
	utils.RemoveColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "SharedFiles",
		superColumns = true,
		rows = {
			[vcc] = filesToRemove
		}
	}
	for _, fileId in pairs(filesToRemove) do
		RemoveLastUse(vcc, fileId)
	end
	RemoveSharedFilesRelations(vcc, filesToRemove, keyspace)
end

----
--  PROCESS FUNCTIONS
--
local function __MakeVCCStatusProcessBatchId (vcc)
	local key = string.format(
		"ProcessBatch:VCC:%s:SharedFiles:Status",
		vcc or inconcert.authUser.virtualContactCenter
	)
	--LogDebug("[__MakeVCCStatusProcessBatchId]------------> key: '%s'", key)
	return key
end

function UpdateVCCStatusProcessBatch (processBatch)
	assert(processBatch)
	assert(processBatch.VCC)
	local vcc = processBatch.VCC
	local key = __MakeVCCStatusProcessBatchId(vcc)
	if processBatch.Status == "RUNNING" then
		GetDataSource().redis:set(key, json.encode(processBatch))
		-- La key expira a los 30 segundos
		local TIME_IN_SEC = 30
		GetDataSource().redis:expire(key, TIME_IN_SEC)
	else
		GetDataSource().redis:del(key)
		if GetVCCStatusProcessBatch(vcc) then
			LogWarning("[UpdateVCCStatusProcessBatch] Unable to delete process batch status '%s'", key)
			return false
		end
	end
	return key
end

function GetVCCStatusProcessBatch (vcc)
	assert(vcc)
	local key = __MakeVCCStatusProcessBatchId(vcc)
	return GetDataSource().redis:get(key)
end

function IsAnyProcessBatchInProgressInVCC (vcc)
	assert(vcc)
	local currentProcessBatch = GetVCCStatusProcessBatch(vcc)
	if not currentProcessBatch then
		return false
	end
	--LogDebug("[IsAnyProcessBatchInProgressInVCC]------------> currentProcessBatch: '%s'", json.encode(currentProcessBatch))
	return true
end

function DeleteVCCProcessBatch (batchProcessId, vcc)
	-- La busco en REDIS
	assert(vcc)
	local encodedProcessBatchStatus = GetVCCStatusProcessBatch(vcc)
	--LogDebug("[ADMINISTRATION.SHARED_FILES:DeleteVCCProcessBatch] BD batchProcessId: '%s', running batch status: '%s'", batchProcessId, encodedProcessBatchStatus)
	local ok, currentProcessBatch = pcall(json.decode, encodedProcessBatchStatus)
	if not ok then
		return false
	end
	if not batchProcessId or (currentProcessBatch.Id or ""):lower() == batchProcessId:lower() then
	--	LogInfo("[ADMINISTRATION.SHARED_FILES:DeleteVCCProcessBatch] Deleting Process batch Id '%s'.", batchProcessId)
		local currentProcessBatch = {Status = "ENDED", VCC = vcc}
		UpdateVCCStatusProcessBatch(currentProcessBatch)
		return true
	end
	--LogInfo("[ADMINISTRATION.SHARED_FILES:DeleteVCCProcessBatch] Unable to delete. Process  %s does not exist.", batchProcessId)
	return false
end

--[[
{
  "vcc": "vcc1",
  "campaignId": "campi6tel@vcc1",
  "messages": {
    "welcome": {
      "id": "welcome",
      "type": "file",
      "options": [
        {
          "fileInformation": {
            "fileSize": 869328,
            "section": "-1",
            "id": "45032668357A9E45A54CF828843BC5B6",
            "contentType": "audio/wav",
            "fileName": "emmmma1.wav",
            "format": "wav"
          },
          "type": "file",
          "Name": "emmmma1.wav",
          "Id": "emmmma1.wav",
          "MessageType": "audio"
        },
        {
          "message": "my tts",
          "type": "custom",
          "Name": "534B4CB3C5EBCB2E7F3E4719D792C315",
          "MessageSubType": "CUSTOM",
          "Id": "534B4CB3C5EBCB2E7F3E4719D792C315",
          "MessageType": "tts"
        },
        {
          "message": "{CAMPAIGN}",
          "type": "custom",
          "Id": "E18632155BAED2742E3F48E9A7B86BF0",
          "VariableId": "{CAMPAIGN}",
          "Engine": "TTS",
          "MessageType": "variable"
        },
        {
          "message": "{CAMPAIGN}",
          "type": "file",
          "Id": "ECC77DF7A9FD56574EBEB8B294A9F829",
          "VariableId": "{CAMPAIGN}",
          "Engine": "FILE",
          "MessageType": "variable"
        }
      ],
      "isComposite": true
    }
  },
  "accountId": "CALL_campi6tel@vcc1_TELi6a"
}
--]]
function SaveSharedFilesAccountMessagesRelations (info, keyspace)
	LogInfo("[ADMINISTRATION.SHARED_FILES.SaveSharedFilesAccountMessagesRelations] Updating Shared Files relations with Campaign Account Messages.")
	--LogDebug("entra - info %s", json.encode(info))

	local files
	for _, message in pairs(info.messages or {}) do
		for _, option in ipairs(message.options or {}) do
			local fromBd = false
			if option.MessageType:lower() == "audio" then
				files = files or {}

				local fieldId = option.Id:lower()
				--LogDebug("       audio %s message.id %s", fieldId, message.id)

				local fileRelation
				if files[fieldId] then
					fromBd = false
					fileRelation = files[fieldId]
				else
					fromBd = true
					fileRelation = GetSharedFilesRelations(info.vcc, fieldId, keyspace) or {}
				end

				fileRelation.CampaignAccountMessages = fileRelation.CampaignAccountMessages or {}
				local accountMessagesRelation = fileRelation.CampaignAccountMessages[info.messagesAccountId] or {}
				
				if fromBd then
					-- si lo obtiene de la BD debo eliminar todo y cargar de cero!
					accountMessagesRelation.messages = {}
				end

				accountMessagesRelation.campaignId = info.campaignId
				accountMessagesRelation.accountId = info.accountId
				accountMessagesRelation.messagesAccountId = info.messagesAccountId
				accountMessagesRelation.messages = accountMessagesRelation.messages or {}
				accountMessagesRelation.messages[message.id] = true

				fileRelation.CampaignAccountMessages[info.messagesAccountId] = accountMessagesRelation

				files[fieldId] = fileRelation

				--LogDebug("       audio %s fileRelation %s", fieldId, json.encode(fileRelation))
			end
		end
	end

	if files then
		-- Almaceno la relacion: si tiene archivos de audio de shared files relacionados.
		SaveSharedFilesRelations(info.vcc, files, keyspace)
	else
		-- Elimino la relacion: si tiene archivos de audio de shared files relacionados.
		local relation = GetSharedFilesAccountsMessagesRelations(info.messagesAccountId, keyspace)
		--LogDebug("   relation %s", json.encode(relation or {}))
		local filesToRemove = {}
		for fileId, relationInfo in pairs(relation or {}) do
			local fileRelations = GetSharedFilesRelations(info.vcc, fileId, keyspace)
			--LogDebug("     fileRelations %s", json.encode(fileRelations or {}))
			if fileRelations.CampaignAccountMessages then
				--LogDebug("       fileRelations.CampaignAccountMessages %s", json.encode(fileRelations.CampaignAccountMessages or {}))
				--LogDebug("       info.messagesAccountId %s", info.messagesAccountId)
				fileRelations.CampaignAccountMessages[info.messagesAccountId] = nil
			end
			files = files or {}
			files[fileId] = fileRelations
			table.insert(filesToRemove, fileId)
		end

		-- relacion con la CF "SharedFilesRelations"
		utils.SaveRows {
			keyspace = keyspace or "Agent",
			columnFamily = "SharedFilesRelations",
			superColumns = true,
			rows = {
				[info.vcc] = files
			}
		}

		if not utils.TableIsEmpty(filesToRemove) then
			-- relacion con la CF "SharedFilesAccountsMessagesRelations"
			RemoveSharedFilesAccountsMessagesRelations({info.messagesAccountId}, keyspace)
		end
	end
end

--[[
	{
		"vcc": "vcc1",
		"sharedFiles": {
			"9CC68F48F406B80CC072E28CD35B60BF": {
				"2": {
					"activity_8": {
					"type": "send_chat_message",
					"description": "",
					"sharedFiles": [
						"Ring1aaaaaaaaaaa4.wav"
					]
					},
					"activity_6": {
					"type": "send_chat_message",
					"description": "",
					"sharedFiles": [
						"Ring09.wav"
					]
					}
				}
			}
		}
	}
]]--
function SaveSharedFilesFlowsDesginerRelations (info, keyspace)
	LogInfo("[ADMINISTRATION.SHARED_FILES.SaveSharedFilesFlowsDesginerRelations] Updating Shared Files relations with Flow Designer.")
--	LogDebug("entra - info %s", json.encode(info))

	local function IsMessageTypeSharedFile (fileInfo)
		local fileId
		if type(fileInfo) == "table" then
		--	LogDebug("      ---> fileInfo: %s", json.encode(fileInfo))
			if fileInfo.MessageType == "audio" then
				fileId = fileInfo.Id
			end
		else
			fileId = fileInfo
		end
		return fileId and fileId:lower()
	end

	local files

	local function CreateRelate (flowId, fieldId, activityId)
		if not fieldId then
			return
		end
		files = files or {}
		local fromBd = false

		local fileRelation
		if files[fieldId] then
			fromBd = false
			fileRelation = files[fieldId]
		else
			fromBd = true
			fileRelation = GetSharedFilesRelations(info.vcc, fieldId, keyspace) or {}
		end

		fileRelation.FlowDesigner = fileRelation.FlowDesigner or {}
		local flowDesignerRelation = fileRelation.FlowDesigner[flowId] or {}

		if fromBd then
			-- si lo obtiene de la BD debo eliminar todo y cargar de cero!
			flowDesignerRelation.activities = {}
		end

		flowDesignerRelation.flowId = flowId
		flowDesignerRelation.activities = flowDesignerRelation.activities or {}
		flowDesignerRelation.activities[activityId] = true

		fileRelation.FlowDesigner[flowId] = flowDesignerRelation
		files[fieldId] = fileRelation

		--LogDebug("       fieldId %s fileRelation %s", fieldId, json.encode(fileRelation))
	end

	for flowId, flowModelInfo in pairs(info.sharedFiles or {}) do
		for modelId, activities in pairs(flowModelInfo) do
			for activityId, activity in pairs(activities) do
				for i, fileInfo in ipairs(activity.sharedFiles) do
					if type(fileInfo) == "string" then
						local fieldId = fileInfo
						--LogDebug("       1 - fieldId: %s, activityId %s, activity.type: '%s'", fieldId or "NIL", activityId, activity.type)
						CreateRelate(flowId, fieldId, activityId)
					elseif type(fileInfo) == "table" then
						for _, file in pairs(fileInfo) do
							local fieldId = IsMessageTypeSharedFile(file)
							--LogDebug("       2 - fieldId: %s, activityId %s, activity.type: '%s'", fieldId or "NIL", activityId, activity.type)
							if fieldId then
								CreateRelate(flowId, fieldId, activityId)
							end
						end
					end
				end
			end
		end
	end

	if files then
		-- Almaceno la relacion: si tiene archivos en shared files relacionados.
		SaveSharedFilesRelations(info.vcc, files, keyspace)
	else
		-- Elimino la relacion: si tiene archivos de shared files relacionados.
		local function DeleteFlowRelations (flowId)
			local relation = GetSharedFilesFlowsDesignerRelations(flowId, keyspace)
			--LogDebug("  flowId: %s --> relation %s", flowId, json.encode(relation or {}))
			local filesToRemove = {}
			for fileId, relationInfo in pairs(relation or {}) do
				local fileRelations = GetSharedFilesRelations(info.vcc, fileId, keyspace)
				--LogDebug("     fileRelations %s", json.encode(fileRelations or {}))
				if fileRelations.FlowDesigner then
					--LogDebug("       fileRelations.FlowDesigner %s", json.encode(fileRelations.FlowDesigner or {}))
					--LogDebug("       flowId %s", flowId)
					fileRelations.FlowDesigner[flowId] = nil
				end
				files = files or {}
				files[fileId] = fileRelations
				table.insert(filesToRemove, fileId)
			end
	
			-- relacion con la CF "SharedFilesRelations"
			utils.SaveRows {
				keyspace = keyspace or "Agent",
				columnFamily = "SharedFilesRelations",
				superColumns = true,
				rows = {
					[info.vcc] = files
				}
			}

			if not utils.TableIsEmpty(filesToRemove) then
				-- relacion con la CF "SharedFilesFlowsDesignerRelations"
				RemoveSharedFilesFlowDesignerRelations({flowId}, keyspace)
			end
		end

		if info.flowId then
			--LogDebug("1")
			DeleteFlowRelations(info.flowId)
		else
			--LogDebug("2")
			for flowId in pairs(info.sharedFiles or {}) do
				DeleteFlowRelations(flowId)
			end
		end
	end

	--LogDebug("sale - files %s", json.encode(files or {}))
end

function RemoveSharedFilesFlowsDesginerRelations (info, keyspace)
	LogInfo("[ADMINISTRATION.SHARED_FILES.RemoveSharedFilesFlowsDesginerRelations] Removing Shared Files relations with Flow Designer.")
	SaveSharedFilesFlowsDesginerRelations(info, keyspace)
end