module(..., package.seeall)

local utils = require "shared.apps.utils"
local m_datasource
local allegro_wkg = require "apps.allegro.controllers.workgroups"

function GetName()
	return "Workgroups"
end

function GetDefinition(datasource)
	m_datasource = datasource
	return _M
end

function GetWorkgroups(vcc, workgroupId, keyspace)
	vcc = (vcc or ""):lower()
	local vccWorkGroups = utils.GetAllColumns({
		keyspace = keyspace or "Agent",
		columnFamily = "Workgroup",
		superColumns = true,
		key = vcc,
		columns = {}
	}) or {}
	--LogDebug("1=========> Vcc '%s' WorkGroups: %s", vcc, json.encode(vccWorkGroups))
	if workgroupId then
		-- un workgroup especifico del vcc
		local workGroupInfo = {}
		workgroupId = workgroupId:lower()
		for vccWorkGroupId, vccWorkGroup in pairs(vccWorkGroups) do
			if vccWorkGroupId:lower() == workgroupId then
				workGroupInfo = vccWorkGroup
				break
			end
		end
		--LogDebug("2=========> Vcc '%s' WorkgroupId: '%s', WorkGroup Info: %s", vcc, workgroupId, json.encode(workGroupInfo))
		return workGroupInfo
	end
	-- todos los workgroups del VCC
	return vccWorkGroups
end


--[[
	-- Guarda en Allegro los workgroups
]]--
local function UpdateAllegroWorkgroupsIn55( workgroups, vcc, keyspace )

	if not utils.HasAllegroIntegration( vcc, keyspace ) then
		LogInfo("[ADMINISTRATION:UpdateAllegroWorkgroupsIn55] - Allegro integration. Updating workgroups '%s'. Vcc '%s' has not configured integration with allegro.", json.encode(workgroups), vcc)
		return false, { type = "WITHOUT_ALLEGRO_INTEGRATION" }
	end

	local success, err = allegro_wkg.Save( workgroups, vcc )
	if not success then
		LogError("[ADMINISTRATION:UpdateAllegroWorkgroupsIn55] - Allegro integration error. Updating workgroups '%s'. Error: '%s'.", json.encode(workgroups), json.encode(err))
		return false, { type = "ALLEGRO_INTEGRATION_ERROR", err = err }
	end

	return true
end

function SaveWorkgroups(vcc, workgroups, keyspace)

	-- Guarda en Allegro los workgroups
	-- Debo mandar a actualizar aca, en este lugar, antes de guardar algo en cassandra, asi me evito el Rollback y toda esas cosas!!!
	--
	local success, err = UpdateAllegroWorkgroupsIn55( workgroups, vcc, keyspace )
	if not success then
		-- Solo tiene que retornar sin guardar en cassandra, si el error es de tipo "ALLEGRO_INTEGRATION_ERROR",
		--  ya que otro tipo de error, puede ser que no tenga integracion con Allegro y por lo tanto en si, no es un error y debe de continuar
		--  la actualizacion en i6.
		if err and err.type == "ALLEGRO_INTEGRATION_ERROR" then
			if close_ks then keyspace:Close() end
			return false, { result = err.type }
		end
	end

	utils.SaveRows {
		keyspace = keyspace or "Agent",
		columnFamily = "Workgroup",
		superColumns = true,
		rows = {
			[vcc] = workgroups
		}
	}

	return true
end

--[[
	-- Delete en Allegro los workgroups
]]--
local function DeleteAllegroWorkgroupsIn55( workgroups, keyspace )

	local vcc = inconcert.authUser.virtualContactCenter

	if not utils.HasAllegroIntegration( vcc, keyspace ) then
		LogInfo("[ADMINISTRATION:DeleteAllegroWorkgroupsIn55] - Allegro integration. Deleting workgroups '%s'. Vcc '%s' has not configured integration with allegro.", json.encode(workgroups), vcc)
		return false, { type = "WITHOUT_ALLEGRO_INTEGRATION" }
	end

	local success, err = allegro_wkg.Delete( workgroups, vcc )
	if not success then
		if err.type == "ALLEGRO_INTEGRATION_ERROR" then
			LogError("[ADMINISTRATION:DeleteAllegroWorkgroupsIn55] - Allegro integration error. Deleting workgroups '%s'. Response: '%s'.", json.encode(workgroups), json.encode(err))
		else
			LogInfo("[ADMINISTRATION:DeleteAllegroWorkgroupsIn55] - Allegro integration. Deleting workgroups '%s'. Response: '%s'.", json.encode(workgroups), json.encode(err))
		end
		return false, err
	end

	return true
end

function RemoveWorkgroups(vcc, workgroups, keyspace)

	local success, err = DeleteAllegroWorkgroupsIn55( workgroups, keyspace )
	if not success then
		-- Solo tiene que retornar sin eliminar el workgroup en i6, si el error es de tipo "ALLEGRO_INTEGRATION_ERROR",
		--  ya que otro tipo de error, puede ser que no tenga integracion con Allegro y por lo tanto en si, no es un error y debe de continuar la eliminacion en i6.
		if err and err.type == "ALLEGRO_INTEGRATION_ERROR" then
			keyspace:Close()
			return false, { result = err.type, err = (err.err or err.sub_type) }
		end
	end

	utils.RemoveColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "Workgroup",
		superColumns = true,
		rows = {
			[vcc] = workgroups
		}
	}

	return true
end
