/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 11 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

//	"use strict";

	const format = inConcert.Utils.format;
	const UNLIMITED_VALUE = -1;

	var __get_cbo_value = function( $option_selected, $value )
	{
		var type = $option_selected.val();
		if ( type === "unlimited" ) {
			return UNLIMITED_VALUE;
		}
		// if ( type === "without" ) {
			// return 0;
		// }
		var value = $value.val();
		if ( value.trim().length === 0 ) {
			return "NOT_CONFIGURED";
		}
		return parseInt( value, 10 );
	};

	var __disable_txt_values = function( self, $option, $value, force_disable, default_value, $options_container)
	{
		var disabled, is_required;

		if ( self.parent_frm.is_readonly() ) {
			disabled = true;
			is_required = false;
			self.parent_frm.readonly_control( $value, disabled, is_required );
		}
		else {
			var previous_value;
			var type = $option.val();

			if ( type === "limited" ) {
				if ( force_disable ) {
					disabled = true;
					is_required = false;
				}
				else {
					disabled = false;
					is_required = true;
				}
				previous_value = $options_container.attr( "previous_value" );
				if ( previous_value !== undefined ) {
					$value.val( previous_value );
				}
				else {
					if ( default_value !== undefined && $value.val().trim().length === 0 ) {
						$value.val( default_value );
					}
				}
			}
			else {
				disabled = true;
				is_required = false;
				previous_value = $value.val();
				if ( previous_value.trim().length > 0 ) {
					previous_value = parseInt( previous_value, 10 );
					if ( previous_value < 1 ) {
						previous_value = 0;
					}
					$options_container.attr( "previous_value" , previous_value );
				}
				$value.val("");
			}
			self.parent_frm.disable_control( $value, disabled, is_required );
		}
		$value.focus();
	};

	var __set_cbo_value = function( self, def, $txt, value, definition )
	{
		var type;
		if ( value === undefined ) {
			value = "";
			type = "unlimited";
		}
		else {
			value = parseInt( value, 10 );
			// if ( value === 0 ) {
				// type = "without";
				// value = "";
			// }
			// else
			if ( value > 0 ) {
				type = "limited";
			}
			else { // < 0
				type = "unlimited";
				value = "";
			}
		}
		const $option = $(`#${def.cbo}_${type}`);
		$option.click();
		$txt.val( value );
		var default_value;
		var force_disable = definition.is_disabled(definition);
		__disable_txt_values( self,$option, $txt, force_disable, default_value, $(`#${def.cbo}_container`));

		return type === "unlimited";
	};

	var __validate_changed_values = function( self )
	{
		var has_changed = false;
		$.each( self.ctrls, function( channel, def )
		{
			if ( !self.channels || !self.channels[ channel ] ) {
				return true; // continue
			}
			var can_configure = def.can_configure( def );
			if ( !can_configure || def.is_disabled(def) ) {
				return true; // continue
			}

			const $selected_option = self.__$get_option_selected($(`#${def.cbo}_container`));
			var original_value = self.channels[ channel ].attentionCapability;

			if ( original_value === UNLIMITED_VALUE.toString() && $selected_option.val() === "unlimited" ) {
				original_value = "";
			}

			var actual_value = $("#" + def.txt ).val();
			if ( original_value !== actual_value ) {
				has_changed = true;
				return false; // break
			}
		});
		// esto esta implementado unicamente en la configuracion a nivel VCC
		if ( typeof self.parent_frm.disable_restore_button === "function" ) {
			self.parent_frm.disable_restore_button( !has_changed );
		}
	};

	//---------------------------------------------------------------------
	// capability handle class
	function adm_capability_handle( params )
	{
		var self = this;

		this.override_type = params.override_type || "NONE";
		this.prefix_id     = params.prefix_id;
		this.container_id  = params.container_id;
		this.parent_frm    = params.parent_frm;
		this.parent_type   = params.parent_type;
		this.proxies       = params.proxies || {};
		this.is_required   = true;

		var can_configure = function( info ) {
			return self.proxies[ info.proxy ];
		};
		var has_licensed = function( info ) {
			return inConcert.app.hasLicensesForChannel(info.proxy);
		};
		var is_disabled = function( info ) {
			if ( !has_licensed(info) ) {
				return true;
			}
			//return self.override_type === "NONE" ? false : true;
			return false;
		};
		var can_enable = function(info, proxyType) {
			proxyType = (proxyType || "").toUpperCase();
			if (info.proxy === proxyType) {
				return true;
			}
			if (info.relationProxies && info.relationProxies[proxyType]) {
				if (typeof(info.relationProxies[proxyType]) === "function") {
					return info.relationProxies[proxyType]();
				}
				return true;
			}
			return false;
		};

		const has_allegro_integration = inConcert.app.has_allegro_integration();

		// el orden que se encuentra aca, tiene que ser el mismo orden en el cual se encuentra en el $html.
		// esto porque cuando se ingresa al frm, el foco se le da al primer txt que se encuentre enabled
		// esta busqueda se realiza iterando esta lista "this.ctrls".
		this.ctrls = {
			WEBCONTACT: {
				proxy         : "WEBCONTACT",
				cbo           : this.prefix_id + "_contact_form_attention_capability_type",
				txt           : this.prefix_id + "_contact_form_attention_capability",
				name          : "<%=_T('Contact Form')%>",
				$html         : this.$get_channel_html,
				is_disabled   : is_disabled,
				has_licensed  : has_licensed,
				can_configure : can_configure,
				can_enable    : can_enable,
				initial_state : this.initial_state,
			},
			FACEBOOK: {
				proxy         : "FACEBOOK",
				cbo           : this.prefix_id + "_facebook_attention_capability_type",
				txt           : this.prefix_id + "_facebook_attention_capability",
				name          : "<%=_T('Facebook')%>",
				$html         : this.$get_channel_html,
				is_disabled   : is_disabled,
				has_licensed  : has_licensed,
				can_configure : can_configure,
				can_enable    : can_enable,
				initial_state : this.initial_state
			},
			INSTAGRAM: {
				proxy         : "INSTAGRAM",
				cbo           : this.prefix_id + "_instagram_attention_capability_type",
				txt           : this.prefix_id + "_instagram_attention_capability",
				name          : "<%=_T('Instagram')%>",
				$html         : this.$get_channel_html,
				is_disabled   : is_disabled,
				has_licensed  : has_licensed,
				can_configure : can_configure,
				can_enable    : can_enable,
				initial_state : this.initial_state
			},
			YOUTUBE: {
				proxy         : "YOUTUBE",
				cbo           : this.prefix_id + "_youtube_attention_capability_type",
				txt           : this.prefix_id + "_youtube_attention_capability",
				name          : "<%=_T('Youtube')%>",
				$html         : this.$get_channel_html,
				is_disabled   : is_disabled,
				has_licensed  : has_licensed,
				can_configure : can_configure,
				can_enable    : can_enable,
				initial_state : this.initial_state
			},
			APPSTORE: {
				proxy         : "APPSTORE",
				cbo           : this.prefix_id + "_appstore_attention_capability_type",
				txt           : this.prefix_id + "_appstore_attention_capability",
				name          : "<%=_T('Appstore')%>",
				$html         : this.$get_channel_html,
				is_disabled   : is_disabled,
				has_licensed  : has_licensed,
				can_configure : can_configure,
				can_enable    : can_enable,
				initial_state : this.initial_state
			},
			GOOGLEPLAY: {
				proxy         : "GOOGLEPLAY",
				cbo           : this.prefix_id + "_googleplay_attention_capability_type",
				txt           : this.prefix_id + "_googleplay_attention_capability",
				name          : "<%=_T('Googleplay')%>",
				$html         : this.$get_channel_html,
				is_disabled   : is_disabled,
				has_licensed  : has_licensed,
				can_configure : can_configure,
				can_enable    : can_enable,
				initial_state : this.initial_state
			},
			RCS: {
				proxy         : "RCS",
				cbo           : this.prefix_id + "_rcs_attention_capability_type",
				txt           : this.prefix_id + "_rcs_attention_capability",
				name          : "<%=_T('Rcs')%>",
				$html         : this.$get_channel_html,
				is_disabled   : is_disabled,
				has_licensed  : has_licensed,
				can_configure : can_configure,
				initial_state : this.initial_state,
				can_enable    : can_enable,
				hidden        : true
			},
			MAIL: {
				proxy         : "MAIL",
				cbo           : this.prefix_id + "_mail_attention_capability_type",
				txt           : this.prefix_id + "_mail_attention_capability",
				name          : "<%=_T('E-mail')%>",
				$html         : this.$get_channel_html,
				is_disabled   : is_disabled,
				has_licensed  : has_licensed,
				can_configure : can_configure,
				can_enable    : can_enable,
				initial_state : this.initial_state
			},
			TWITTER: {
				proxy         : "TWITTER",
				cbo           : this.prefix_id + "_twitter_attention_capability_type",
				txt           : this.prefix_id + "_twitter_attention_capability",
				name          : "<%=_T('Twitter')%>",
				$html         : this.$get_channel_html,
				is_disabled   : is_disabled,
				has_licensed  : has_licensed,
				can_configure : can_configure,
				can_enable    : can_enable,
				initial_state : this.initial_state
			},
			CHAT: {
				proxy         : "CHAT",
				cbo           : this.prefix_id + "_chat_attention_capability_type",
				txt           : this.prefix_id + "_chat_attention_capability",
				name          : "<%=_T('Web Chat')%>",
				$html         : this.$get_channel_html,
				is_disabled   : is_disabled,
				has_licensed  : has_licensed,
				can_configure : can_configure,
				can_enable    : can_enable,
				initial_state : this.initial_state
			},
			SMS: {
				proxy         : "SMS",
				cbo           : this.prefix_id + "_sms_attention_capability_type",
				txt           : this.prefix_id + "_sms_attention_capability",
				name          : "<%=_T('SMS')%>",
				$html         : this.$get_channel_html,
				is_disabled   : is_disabled,
				has_licensed  : has_licensed,
				can_configure : can_configure,
				can_enable    : can_enable,
				initial_state : this.initial_state
			},
			WHATSAPP: {
				proxy         : "WHATSAPP",
				cbo           : this.prefix_id + "_whatsapp_attention_capability_type",
				txt           : this.prefix_id + "_whatsapp_attention_capability",
				name          : "<%=_T('Whatsapp')%>",
				$html         : this.$get_channel_html,
				is_disabled   : is_disabled,
				has_licensed  : has_licensed,
				can_configure : can_configure,
				can_enable    : can_enable,
				initial_state : this.initial_state
			},
			CALL: {
				proxy         : "CALL",
				cbo           : this.prefix_id + "_call_attention_capability_type",
				txt           : this.prefix_id + "_call_attention_capability",
				name          : "<%=_T('Call')%>",
				$html         : this.$get_channel_html,
				is_disabled   : function() { return true; },
				has_licensed  : has_licensed,
				can_configure : can_configure,
				can_enable    : function() { return false; },
				initial_state : this.disable_initial_state
			},
			VOICEMAIL: {
				proxy         : "VOICEMAIL",
				cbo           : this.prefix_id + "_voicemail_attention_capability_type",
				txt           : this.prefix_id + "_voicemail_attention_capability",
				name          : "<%=_T('VoiceMail')%>",
				$html         : this.$get_channel_html,
				relationProxies: {
					CALL: function() {
						if (has_allegro_integration) {
							return false;
						}
						return true;
					}
				},
				is_disabled   : has_allegro_integration ? function() { return true; } : is_disabled,
				has_licensed  : has_licensed,
				can_configure : can_configure,
				can_enable    : can_enable,
				initial_state : has_allegro_integration ? this.disable_initial_state : this.initial_state
			},
			IM: {
				proxy         : "IM",
				cbo           : this.prefix_id + "_im_attention_capability_type",
				txt           : this.prefix_id + "_im_attention_capability",
				name          : "<%=_T('IM')%>",
				$html         : this.$get_channel_html,
				is_disabled   : is_disabled,
				has_licensed  : has_licensed,
				can_configure : can_configure,
				can_enable    : can_enable,
				initial_state : this.initial_state,
				hidden        : true
			}
		};

		return true;
	}

	if ( typeof inConcert.adm_capability_handle !== "function" ) {
		inConcert.adm_capability_handle = adm_capability_handle;
	}

	adm_capability_handle.prototype.initialize = function()
	{
		this.build_markup( true /*cleanup container*/);
		this.set_events();
	};

	adm_capability_handle.prototype.initial_state = function( info )
	{
		if ( !inConcert.app.hasLicensesForChannel(info.proxy) ) {
			return "DISABLED";
		}
		return "ENABLED";
	};

	adm_capability_handle.prototype.disable_initial_state = function()
	{
		return "DISABLED";
	};

	adm_capability_handle.prototype.build_markup = function( cleanup_container )
	{
		var self = this;

		var $container = $("#" + this.container_id);
		$container.hide();
		if ( cleanup_container ) {
			$container.html("");
		}

		$.each( this.ctrls, function( _, def )
		{
			var can_configure = def.can_configure( def );
			if ( !can_configure ) {
				return true; // continue
			}
			var $capability = def.$html( def, self.is_required);
			$container.append( $capability );

			var state = "DISABLED";
			if ( def.initial_state(def) === "ENABLED" ) {
				state = "ENABLED";
			}
			self.enable_channel( state, $capability );
		});
		$container.show();
	};

	adm_capability_handle.prototype.$get_channel_html = function( info, is_required)
	{

		var $html = $( "".concat(
			"<div class='channel-card-container' ctrl_type='channel_container' data-proxy_config='", info.proxy, "' hidden>",
					"<div class='form-group'>",
					"<label for='", info.cbo, "_title' id='", info.cbo, "_title' class='capability-channel-title-container capabilityHandleChannel", info.proxy,"'>", info.name ,"</label>",
					"<span id='", info.cbo, "_readonly_limit_title' class='capability-channel-limit-container' hidden>",
					"</span>",
					"<div id='", info.cbo, "_container' class='capability-channel-radio-buttons-container'>",
					"<div class='form-radiobutton'>",
					"<label for='", info.cbo, "_unlimited' class='form-radiobutton-label'>",
					"<input type='radio' value='unlimited' name='", is_required? info.cbo:"", "' id='", info.cbo, "_unlimited' class='form-radiobutton-input'>",
					"<span><%=_T('Unlimited')%></span>",
									"</label>",
								"</div>",
							"<div class='form-radiobutton' >",
							"<label for='", info.cbo, "_limited' class='form-radiobutton-label'>",
								"<input type='radio' value='limited' name='", is_required? info.cbo :'', "' id='", info.cbo, "_limited' class='form-radiobutton-input'>",
									"<span><%=_T('Limited')%></span>",
									"</label>",
								"</div>",
							"<div id='", info.txt, "_container' class='form-group' style='display:none'>",
							"<input id='", info.txt, "' class='form-control' type='text' ctrl_type='input' placeholder='<%=_T('Limit')%>'/>",
								"<i id='", info.txt, "_validation' class='formFieldValidation formFieldError' hidden ></i>",
								"</div>",
							"</div>",
						"<i id='", info.txt, "_title_validation' class='formFieldValidation formFieldError' hidden ></i>",
					"</div>",
			"</div>"
		));

		$html.find(`#${info.cbo}_unlimited`).prop("checked", is_required);

		if (info.hidden) {
			$html.hide();
		}
		return $html;
	};

	adm_capability_handle.prototype.get_id = function( id_type )
	{
		switch(id_type)
		{
			case "ENABLE_OVERRIDE":
				return this.prefix_id + "enable_override";
			default:
				throw("adm_capability_handle.get_id -> Unrecognized id type'" + id_type + "'");
		}
	};

	adm_capability_handle.prototype.editable = function()
	{
		var self = this;
		var frm = this.parent_frm;

		if ( this.override_type !== "NONE" ) {
			var $chk_enabled = $("#" + this.get_id("ENABLE_OVERRIDE"));
			frm.readonly_control( $chk_enabled, false /*readonly*/ );
		}

		$.each( this.ctrls, function( channel, def )
		{
			if ( !self.channels || !self.channels[ channel ] ) {
				return true; // continue
			}
			var can_configure = def.can_configure( def );
			if ( !can_configure || def.is_disabled(def) ) {
				return true; // continue
			}

			self.readonly_channel( $(`#${def.cbo}_container`), undefined, def.proxy, false);
		});
	};

	adm_capability_handle.prototype.readonly = function()
	{
		var self = this;
		var frm = this.parent_frm;

		if ( this.override_type !== "NONE" ) {
			// TODO: Ver el porque no funciona el readonly para este control cuando se utiliza de la seccion de members de la campana(con un timeout si funciona)
			var $chk_enabled = $("#" + this.get_id("ENABLE_OVERRIDE"));
			frm.readonly_control( $chk_enabled, true /*readonly*/ );
		}

		$.each( this.ctrls, function( channel, def )
		{
			if ( !self.channels || !self.channels[ channel ] ) {
				return true; // continue
			}
			var can_configure = def.can_configure( def );
			if ( !can_configure || def.is_disabled(def) ) {
				return true; // continue
			}

			self.readonly_channel($(`#${def.cbo}_container`), undefined, def.proxy);
		});

		this.__set_limit_title();
	};

	adm_capability_handle.prototype.can_enable_channel = function(config)
	{
		if (!config) {
			return false;
		}
		return config.has_licensed(config);
	};

	adm_capability_handle.prototype.__$get_option_selected = function( $capability){
		return $capability.find('input[type="radio"]:checked');
	};

	adm_capability_handle.prototype.readonly_channel = function( $capability, is_required, proxy, readonly = true){

		this.readonly_channel_ctrls($capability, readonly, is_required);
		proxy = proxy || $capability.data("proxy_config");
		const config = this.ctrls[proxy];
		const $readonly_limit_title = $(`#${config.cbo}_readonly_limit_title`);
		const $options_container = $(`#${config.cbo}_container`);
		if (readonly) {
			$options_container.hide();
			$readonly_limit_title.show();
		}
		else{
			$readonly_limit_title.hide();
			$options_container.show();
		}
	};

	adm_capability_handle.prototype.__set_limit_title = function(){
		
		const self = this;
		$.each( this.ctrls, function( channel, def )
		{
			if ( !self.channels || !self.channels[ channel ] ) {
				return true; // continue
			}
			const $readonly_limit_title = $(`#${def.cbo}_readonly_limit_title`);
			const attentionCapability = self.channels[ channel ].attentionCapability;
			const limit_title = attentionCapability && parseInt(attentionCapability) > -1 ? `<%=_T('Limit')%> ${ attentionCapability}`:"<%=_T('Unlimited')%>";
			$readonly_limit_title.html(limit_title);
		});
	};

	adm_capability_handle.prototype.readonly_channel_ctrls = function( $capability, readonly, is_required){
		const self = this;
		$capability.find("input").each(function() {
			is_required = $(this).attr("ctrl_type") === "input" ? is_required: false;
			self.parent_frm.readonly_control( $(this), readonly, is_required);
		});
	};

	adm_capability_handle.prototype.enable_channel = function( state, $capability)
	{
		var disable = true;
		var proxy = $capability.data("proxy_config");
		var config = this.ctrls[proxy];
		if ( this.parent_frm.is_readonly() ) {
			return this.readonly_channel($capability);
		}
		else {
			if ( state === "ENABLED" ) {
				if ( this.can_enable_channel(config) ) {
					disable = false;
				}
			}
		}

		// Si no esta habilitado, no se muestra.
		$capability[disable ? "hide" : "show"]();
		return !disable;
	};

	adm_capability_handle.prototype.set_events = function()
	{
		var self = this;

		var $container = $("#" + this.container_id);
		var event_selector;

		$.each( this.ctrls, function( _, def )
		{
			var can_configure = def.can_configure( def );
			if ( !can_configure ) {
				return true; // continue
			}

			var $txt = $("#" + def.txt );
						
			const $options_container = $(`#${def.cbo}_container`);
			const $radios = $options_container.find(`input[type="radio"]`);
			$radios.off("click")
			.on("click", function() {
				const $this = $(this);
				const type = $this.val();
				if (self.is_required) {
					$(`#${def.txt}_container`)[type === "limited"? "show": "hide"]();
				}
				else{
					const is_checked = $this.attr("is_checked") === "true";
					$this.prop("checked", !is_checked);
					$this.attr("is_checked", !is_checked);

					$options_container.find(`input[type="radio"][value!="${type}"]`).prop("checked", false);
					$options_container.find(`input[type="radio"][value!="${type}"]`).attr("is_checked", false);
					$(`#${def.txt}_container`)[type === "limited" && !is_checked ? "show": "hide"]();
				}

				var default_value;
				var force_disable = def.is_disabled(def);
				__disable_txt_values( self, $(this), $txt, force_disable, default_value, $options_container);
				__validate_changed_values( self );
			});


			var changed_timeout;
			event_selector = "[id='" + def.txt + "']";
			$container.off("keyup", event_selector)
				.on("keyup", event_selector, function ( event ) {
					if ( changed_timeout ) {
						window.clearTimeout( changed_timeout );
						changed_timeout = undefined;
					}
					// enter o escape
					if ( event.keyCode === 13 || event.keyCode === 27 ) {
						return;
					}
					// se encuentra en un timeout de 500 ms, para que no realice la validacion de forma inmediata.
					var elapsed_time_ms = 500;
					changed_timeout = window.setTimeout(function() {
						window.clearTimeout( changed_timeout );
						changed_timeout = undefined;
						__validate_changed_values( self );
					}, elapsed_time_ms);
				});
		});
	};

	adm_capability_handle.prototype.load_from_campaign = function( callback )
	{
		var self = this;
		callback = callback || $.noop;

		var campaign = this.parent_frm.get_entity_instance();
		if ( !campaign ) {
			return;
		}
		campaign.get_attention_capability(
			function( success, channels ) {
				self.set( success, channels );
				callback( success, channels );
			}
		);
	};

	adm_capability_handle.prototype.get = function()
	{
		var self = this;
		var data = { is_override: false, channels: null };

		if ( this.override_type !== "NONE" ) {
			data.is_override = true;
			var $chk_override = $( "#" + this.get_id("ENABLE_OVERRIDE") );
			if ( !$chk_override.prop("checked") ) {
				data.has_override = false;
				return data;
			}
			data.has_override = true;
		}

		var channels = this.channels || {};
		$.each( this.ctrls, function( channel, def )
		{
			var can_configure = def.can_configure( def );
			if ( !can_configure || !def.has_licensed(def) ) {
				return true; // continue
			}
			var $option_selected = self.__$get_option_selected($(`#${def.cbo}_container`));
			if ( !self.is_required && (!$option_selected || $option_selected.length === 0) ) {
				return true; // continue
			}
			var $txt = $("#" + def.txt );
			channels[ channel ] = channels[ channel ] || {};
			channels[ channel ].attentionCapability = __get_cbo_value( $option_selected, $txt );
		});
		this.channels = channels;
		data.channels = channels;

		return data;
	};

	adm_capability_handle.prototype.set = function( success, channels, can_rebuild_mark, callback = $.noop)
	{
// TODO: #4636 - Esto no debe de estar aca!!.
//   Este set events, NO se tiene que invocar aca; a esto lo realiza de forma automatica
//   la clase base "conf_form", pero si ahora no lo realizo asi, tengo un problema en el formulario de campanas.
//   Entonces, hasta que esto se encuentre en el ultimo build de mcwarren queda aca, y lo arreglo ahi,
//   en el ultimo build de mcwarren.
//   Como esta se esta tocando en el build 1962, el ultimo congelado a la fecha, prefiero tocar puntal y que esto funcione, para lo que se necesita, y luego que este en mcwarren, revisar y arreglar correctamente.
//   Revisar en "conf_form.prototype.ready", donde se realiza el set(..) del frm.

		var self = this;

		if ( can_rebuild_mark ) {
			this.initialize();
		}
		else {
			this.set_events();
		}

		if ( !success ) {
			callback( false );
			return false;
		}

		this.channels = channels || {};
		var txt_focus_id;
		var at_least_one_configured = false;

		$.each( this.ctrls, function( channel, def )
		{
			if ( !self.channels || !self.channels[ channel ] ) {
				return true; // continue
			}
			var can_configure = def.can_configure( def );
			if ( !can_configure || !def.has_licensed(def) ) {
				return true; // continue
			}
			at_least_one_configured = true;
			const $options_container = $(`#${def.cbo}_container`);
			$options_container.removeAttr("previous_value");

			var $txt = $("#" + def.txt );
			var value = channels[ channel ].attentionCapability;
			var is_unlimited = __set_cbo_value( self, def, $txt, value, def );
			if ( txt_focus_id === undefined && !is_unlimited ) {
				txt_focus_id = def.txt;
			}
		});

		this.__set_limit_title();

		if ( this.override_type !== "NONE" ) {
			var $chk_override = $( "#" + this.get_id("ENABLE_OVERRIDE") );
			var enabled = at_least_one_configured;
			$chk_override.prop("checked", enabled).trigger("change");
		}

		if ( txt_focus_id ) {
			$("#" + txt_focus_id ).focus();
		}

		callback( true );
		return true;
	};

	adm_capability_handle.prototype.get_old_values = function()
	{
		return this.channels;
	};

	adm_capability_handle.prototype.validate = function()
	{
		var result = false;
		var validation_def = this.get_validation_def();
		this.parent_frm.validate( function( success ) {
			result = success;
		}, validation_def );
		return result;
	};

	adm_capability_handle.prototype.get_validation_def = function()
	{
		var self = this;

		var validate = function( channel, def ) {
			var response = {
				success: true
			};

			if ( self.override_type !== "NONE" ) {
				var $chk_enabled = $("#" + self.get_id("ENABLE_OVERRIDE"));
				if (!$chk_enabled.prop("checked")) {
					return response;
				}
			}

			var $txt = $("#" + self.ctrls[ channel ].txt );
			var value = $txt.val();

			const $options_container = $(`#${self.ctrls[ channel ].cbo}_container`);
			const $option_selected = self.__$get_option_selected($options_container);
			if ( $option_selected && $option_selected.val() === "limited") {
				if ( value.trim().length === 0 ) {
					response.success = false;
					response.message = def.messages.REQUIRED;
					return response;
				}
				var is_number = inConcert.Utils.validate( value, "number" );
				if ( !is_number.success ) {
					response.success = false;
					response.message = def.messages.NOT_A_NUMBER;
					return response;
				}
				// JSLINT / JSHINT: fallan pero necesito comparar por si viene un numero dentro de un string.
				/* jshint -W116 */
				if ( Math.floor( value ) != value ) {
					response.success = false;
					response.message = def.messages.NOT_A_NUMBER;
					return response;
				}

				var range = def.range || { from: 1, to: 1000 }; // rango por default
				value = parseInt( value, 10 );

				if ( value < range.from || value > range.to ) {
					response.success = false;
					response.message = format( def.messages.RANGE, range.from, range.to );
					return response;
				}
			}

			return response;
		};

		var definitions = [
			{
				proxy: "WEBCONTACT",
				control_id: `${this.ctrls.WEBCONTACT.cbo}_title`,
				messages: {
					custom: {
						type:  "custom",
						valid: function() {
							let response = {success : true, message : "" };
							if (self.is_required) {
								const $option_selected = self.__$get_option_selected($(`#${self.ctrls.WEBCONTACT.cbo}_container`));
								if (!$option_selected || $option_selected.length === 0) {
									response.success = false;
									response.message = "<%=_T('Please select the contact form attention capability configuration.')%>";
								}
							}
							return response;
						}
					}
				}
			},
			{
				proxy: "WEBCONTACT",
				control_id: this.ctrls.WEBCONTACT.txt,
				messages: {
					custom: {
						type:  "custom",
						valid: function() {
							return validate( "WEBCONTACT",
								{
								//	range: { from: 1, to: 50 },
									messages: {
										REQUIRED     : "<%=_T('Please enter the contact form attention capability.')%>",
										NOT_A_NUMBER : "<%=_T('Only numbers are allowed in the contact form attention capability.')%>",
										RANGE        : "<%=_T('Please enter contact form attention capability between {0} and {1}.')%>"
									}
								}
							);
						}
					}
				}
			},
			{
				proxy: "FACEBOOK",
				control_id: `${this.ctrls.FACEBOOK.cbo}_title`,
				messages: {
					custom: {
						type:  "custom",
						valid: function() {
							let response = {success : true, message : "" };
							if (self.is_required) {
								const $option_selected = self.__$get_option_selected($(`#${self.ctrls.FACEBOOK.cbo}_container`));
								if (!$option_selected || $option_selected.length === 0) {
									response.success = false;
									response.message = "<%=_T('Please select the facebook attention capability configuration.')%>";
								}
							}
							return response;
						}
					}
				}
			},
			{
				proxy: "FACEBOOK",
				control_id: this.ctrls.FACEBOOK.txt,
				messages: {
					custom: {
						type:  "custom",
						valid: function() {
							return validate( "FACEBOOK",
								{
									messages: {
										REQUIRED     : "<%=_T('Please enter the facebook attention capability.')%>",
										NOT_A_NUMBER : "<%=_T('Only numbers are allowed in the facebook attention capability.')%>",
										RANGE        : "<%=_T('Please enter facebook attention capability between {0} and {1}.')%>"
									}
								}
							);
						}
					}
				}
			},
			{
				proxy: "INSTAGRAM",
				control_id: `${this.ctrls.INSTAGRAM.cbo}_title`,
				messages: {
					custom: {
						type:  "custom",
						valid: function() {
							let response = {success : true, message : "" };
							if (self.is_required) {
								const $option_selected = self.__$get_option_selected($(`#${self.ctrls.INSTAGRAM.cbo}_container`));
								if (!$option_selected || $option_selected.length === 0) {
									response.success = false;
									response.message = "<%=_T('Please select the instagram attention capability configuration.')%>";
								}
							}
							return response;
						}
					}
				}
			},
			{
				proxy: "INSTAGRAM",
				control_id: this.ctrls.INSTAGRAM.txt,
				messages: {
					custom: {
						type:  "custom",
						valid: function() {
							return validate( "INSTAGRAM",
								{
									messages: {
										REQUIRED     : "<%=_T('Please enter the instagram attention capability.')%>",
										NOT_A_NUMBER : "<%=_T('Only numbers are allowed in the instagram attention capability.')%>",
										RANGE        : "<%=_T('Please enter instagram attention capability between {0} and {1}.')%>"
									}
								}
							);
						}
					}
				}
			},
			{
				proxy: "YOUTUBE",
				control_id: `${this.ctrls.YOUTUBE.cbo}_title`,
				messages: {
					custom: {
						type:  "custom",
						valid: function() {
							let response = {success : true, message : "" };
							if (self.is_required) {
								const $option_selected = self.__$get_option_selected($(`#${self.ctrls.YOUTUBE.cbo}_container`));
								if (!$option_selected || $option_selected.length === 0) {
									response.success = false;
									response.message = "<%=_T('Please select the youtube attention capability configuration.')%>";
								}
							}
							return response;
						}
					}
				}
			},
			{
				proxy: "YOUTUBE",
				control_id: this.ctrls.YOUTUBE.txt,
				messages: {
					custom: {
						type:  "custom",
						valid: function() {
							return validate( "YOUTUBE",
								{
									messages: {
										REQUIRED     : "<%=_T('Please enter the youtube attention capability.')%>",
										NOT_A_NUMBER : "<%=_T('Only numbers are allowed in the youtube attention capability.')%>",
										RANGE        : "<%=_T('Please enter youtube attention capability between {0} and {1}.')%>"
									}
								}
							);
						}
					}
				}
			},
			{
				proxy: "APPSTORE",
				control_id: `${this.ctrls.APPSTORE.cbo}_title`,
				messages: {
					custom: {
						type:  "custom",
						valid: function() {
							let response = {success : true, message : "" };
							if (self.is_required) {
								const $option_selected = self.__$get_option_selected($(`#${self.ctrls.APPSTORE.cbo}_container`));
								if (!$option_selected || $option_selected.length === 0) {
									response.success = false;
									response.message = "<%=_T('Please select the appstore attention capability configuration.')%>";
								}
							}
							return response;
						}
					}
				}
			},
			{
				proxy: "APPSTORE",
				control_id: this.ctrls.APPSTORE.txt,
				messages: {
					custom: {
						type:  "custom",
						valid: function() {
							return validate( "APPSTORE",
								{
									messages: {
										REQUIRED     : "<%=_T('Please enter the appstore attention capability.')%>",
										NOT_A_NUMBER : "<%=_T('Only numbers are allowed in the appstore attention capability.')%>",
										RANGE        : "<%=_T('Please enter appstore attention capability between {0} and {1}.')%>"
									}
								}
							);
						}
					}
				}
			},
			{
				proxy: "GOOGLEPLAY",
				control_id: `${this.ctrls.GOOGLEPLAY.cbo}_title`,
				messages: {
					custom: {
						type:  "custom",
						valid: function() {
							let response = {success : true, message : "" };
							if (self.is_required) {
								const $option_selected = self.__$get_option_selected($(`#${self.ctrls.GOOGLEPLAY.cbo}_container`));
								if (!$option_selected || $option_selected.length === 0) {
									response.success = false;
									response.message = "<%=_T('Please select the googleplay attention capability configuration.')%>";
								}
							}
							return response;
						}
					}
				}
			},
			{
				proxy: "GOOGLEPLAY",
				control_id: this.ctrls.GOOGLEPLAY.txt,
				messages: {
					custom: {
						type:  "custom",
						valid: function() {
							return validate( "GOOGLEPLAY",
								{
									messages: {
										REQUIRED     : "<%=_T('Please enter the googleplay attention capability.')%>",
										NOT_A_NUMBER : "<%=_T('Only numbers are allowed in the googleplay attention capability.')%>",
										RANGE        : "<%=_T('Please enter googleplay attention capability between {0} and {1}.')%>"
									}
								}
							);
						}
					}
				}
			},
			{
				proxy: "RCS",
				control_id: `${this.ctrls.RCS.cbo}_title`,
				messages: {
					custom: {
						type:  "custom",
						valid: function() {
							let response = {success : true, message : "" };
							if (self.is_required) {
								const $option_selected = self.__$get_option_selected($(`#${self.ctrls.RCS.cbo}_container`));
								if (!$option_selected || $option_selected.length === 0) {
									response.success = false;
									response.message = "<%=_T('Please select the rcs attention capability configuration.')%>";
								}
							}
							return response;
						}
					}
				}
			},
			{
				proxy: "RCS",
				control_id: this.ctrls.RCS.txt,
				messages: {
					custom: {
						type:  "custom",
						valid: function() {
							return validate( "RCS",
								{
									messages: {
										REQUIRED     : "<%=_T('Please enter the rcs attention capability.')%>",
										NOT_A_NUMBER : "<%=_T('Only numbers are allowed in the rcs attention capability.')%>",
										RANGE        : "<%=_T('Please enter rcs attention capability between {0} and {1}.')%>"
									}
								}
							);
						}
					}
				}
			},
			{
				proxy: "IM",
				control_id: `${this.ctrls.IM.cbo}_title`,
				messages: {
					custom: {
						type:  "custom",
						valid: function() {
							let response = {success : true, message : "" };
							if (self.is_required) {
								const $option_selected = self.__$get_option_selected($(`#${self.ctrls.IM.cbo}_container`));
								if (!$option_selected || $option_selected.length === 0) {
									response.success = false;
									response.message = "<%=_T('Please select the IM attention capability configuration.')%>";
								}
							}
							return response;
						}
					}
				}
			},
			{
				proxy: "IM",
				control_id: this.ctrls.IM.txt,
				messages: {
					custom: {
						type:  "custom",
						valid: function() {
							return validate( "IM",
								{
									messages: {
										REQUIRED     : "<%=_T('Please enter the IM attention capability.')%>",
										NOT_A_NUMBER : "<%=_T('Only numbers are allowed in the IM attention capability.')%>",
										RANGE        : "<%=_T('Please enter IM attention capability between {0} and {1}.')%>"
									}
								}
							);
						}
					}
				}
			},
			{
				proxy: "MAIL",
				control_id: `${this.ctrls.MAIL.cbo}_title`,
				messages: {
					custom: {
						type:  "custom",
						valid: function() {
							let response = {success : true, message : "" };
							if (self.is_required) {
								const $option_selected = self.__$get_option_selected($(`#${self.ctrls.MAIL.cbo}_container`));
								if (!$option_selected || $option_selected.length === 0) {
									response.success = false;
									response.message = "<%=_T('Please select the e-mail attention capability configuration.')%>";
								}
							}
							return response;
						}
					}
				}
			},
			{
				proxy: "MAIL",
				control_id: this.ctrls.MAIL.txt,
				messages: {
					custom: {
						type:  "custom",
						valid: function() {
							return validate( "MAIL",
								{
									messages: {
										REQUIRED     : "<%=_T('Please enter the e-mail attention capability.')%>",
										NOT_A_NUMBER : "<%=_T('Only numbers are allowed in the e-mail attention capability.')%>",
										RANGE        : "<%=_T('Please enter e-mail attention capability between {0} and {1}.')%>"
									}
								}
							);
						}
					}
				}
			},
			{
				proxy: "TWITTER",
				control_id: `${this.ctrls.TWITTER.cbo}_title`,
				messages: {
					custom: {
						type:  "custom",
						valid: function() {
							let response = {success : true, message : "" };
							if (self.is_required) {
								const $option_selected = self.__$get_option_selected($(`#${self.ctrls.TWITTER.cbo}_container`));
								if (!$option_selected || $option_selected.length === 0) {
									response.success = false;
									response.message = "<%=_T('Please select the twitter attention capability configuration.')%>";
								}
							}
							return response;
						}
					}
				}
			},
			{
				proxy: "TWITTER",
				control_id: this.ctrls.TWITTER.txt,
				messages: {
					custom: {
						type:  "custom",
						valid: function() {
							return validate( "TWITTER",
								{
									messages: {
										REQUIRED     : "<%=_T('Please enter the twitter attention capability.')%>",
										NOT_A_NUMBER : "<%=_T('Only numbers are allowed in the twitter attention capability.')%>",
										RANGE        : "<%=_T('Please enter twitter attention capability between {0} and {1}.')%>"
									}
								}
							);
						}
					}
				}
			},
			{
				proxy: "CHAT",
				control_id: `${this.ctrls.CHAT.cbo}_title`,
				messages: {
					custom: {
						type:  "custom",
						valid: function() {
							let response = {success : true, message : "" };
							if (self.is_required) {
								const $option_selected = self.__$get_option_selected($(`#${self.ctrls.CHAT.cbo}_container`));
								if (!$option_selected || $option_selected.length === 0) {
									response.success = false;
									response.message = "<%=_T('Please select the web chat attention capability configuration.')%>";
								}
							}
							return response;
						}
					}
				}
			},
			{
				proxy: "CHAT",
				control_id: this.ctrls.CHAT.txt,
				messages: {
					custom: {
						type:  "custom",
						valid: function() {
							return validate( "CHAT",
								{
									messages: {
										REQUIRED     : "<%=_T('Please enter the web chat attention capability.')%>",
										NOT_A_NUMBER : "<%=_T('Only numbers are allowed in the chat attention capability.')%>",
										RANGE        : "<%=_T('Please enter web chat attention capability between {0} and {1}.')%>"
									}
								}
							);
						}
					}
				}
			},
			{
				proxy: "SMS",
				control_id: `${this.ctrls.SMS.cbo}_title`,
				messages: {
					custom: {
						type:  "custom",
						valid: function() {
							let response = {success : true, message : "" };
							if (self.is_required) {
								const $option_selected = self.__$get_option_selected($(`#${self.ctrls.SMS.cbo}_container`));
								if (!$option_selected || $option_selected.length === 0) {
									response.success = false;
									response.message = "<%=_T('Please select the SMS attention capability configuration.')%>";
								}
							}
							return response;
						}
					}
				}
			},
			{
				proxy: "SMS",
				control_id: this.ctrls.SMS.txt,
				messages: {
					custom: {
						type:  "custom",
						valid: function() {
							return validate( "SMS",
								{
									messages: {
										REQUIRED     : "<%=_T('Please enter the SMS attention capability.')%>",
										NOT_A_NUMBER : "<%=_T('Only numbers are allowed in the SMS attention capability.')%>",
										RANGE        : "<%=_T('Please enter SMS attention capability between {0} and {1}.')%>"
									}
								}
							);
						}
					}
				}
			},
			{
				proxy: "WHATSAPP",
				control_id: `${this.ctrls.WHATSAPP.cbo}_title`,
				messages: {
					custom: {
						type:  "custom",
						valid: function() {
							let response = {success : true, message : "" };
							if (self.is_required) {
								const $option_selected = self.__$get_option_selected($(`#${self.ctrls.WHATSAPP.cbo}_container`));
								if (!$option_selected || $option_selected.length === 0) {
									response.success = false;
									response.message = "<%=_T('Please select the WHATSAPP attention capability configuration.')%>";
								}
							}
							return response;
						}
					}
				}
			},
			{
				proxy: "WHATSAPP",
				control_id: this.ctrls.WHATSAPP.txt,
				messages: {
					custom: {
						type:  "custom",
						valid: function() {
							return validate( "WHATSAPP",
								{
									messages: {
										REQUIRED     : "<%=_T('Please enter the WHATSAPP attention capability.')%>",
										NOT_A_NUMBER : "<%=_T('Only numbers are allowed in the WHATSAPP attention capability.')%>",
										RANGE        : "<%=_T('Please enter WHATSAPP attention capability between {0} and {1}.')%>"
									}
								}
							);
						}
					}
				}
			}
		];

		if (!inConcert.app.has_allegro_integration()) {
			definitions.push({
				proxy: "VOICEMAIL",
				control_id: `${this.ctrls.VOICEMAIL.cbo}_title`,
				messages: {
					custom: {
						type:  "custom",
						valid: function() {
							let response = {success : true, message : "" };
							if (self.is_required) {
								const $option_selected = self.__$get_option_selected($(`#${self.ctrls.VOICEMAIL.cbo}_container`));
								if (!$option_selected || $option_selected.length === 0) {
									response.success = false;
									response.message = "<%=_T('Please select the voicemail attention capability configuration.')%>";
								}
							}
							return response;
						}
					}
				}
			});

			definitions.push({
				proxy: "VOICEMAIL",
				control_id: this.ctrls.VOICEMAIL.txt,
				messages: {
					custom: {
						type:  "custom",
						valid: function() {
							return validate( "VOICEMAIL",
								{
									messages: {
										REQUIRED     : "<%=_T('Please enter the voicemail attention capability.')%>",
										NOT_A_NUMBER : "<%=_T('Only numbers are allowed in the voicemail attention capability.')%>",
										RANGE        : "<%=_T('Please enter voicemail attention capability between {0} and {1}.')%>"
									}
								}
							);
						}
					}
				}
			});
		}

		var validation_def = [];
		$.each( definitions, function( _, def )
		{
			var ctrl_def = self.ctrls[ def.proxy ];
			if ( ctrl_def ) {
				var is_configured = ctrl_def.can_configure( def );
				if ( is_configured ) {
					validation_def.push( def );
				}
			}
		});

		return validation_def;
	};


	//---------------------------------------------------------------------
	// capability override handle class
	function adm_capability_handle_override( params )
	{
		var success = adm_capability_handle_override.uber.constructor.call(this, params);
		if (!success) {
			return;
		}
		this.is_required = typeof params.is_required === "boolean" ? params.is_required : true;
		this.panel_id    = params.panel_id; // este panel y los metodos de show, on_show, hide, on_hide, son utilizados unicamente en el override de members
		return true;
	}

	inConcert.Utils.inherit( adm_capability_handle_override, inConcert.adm_capability_handle );

	if ( typeof inConcert.adm_capability_handle_override !== "function" ) {
		inConcert.adm_capability_handle_override = adm_capability_handle_override;
	}

	adm_capability_handle_override.prototype.initial_state = function(_)
	{
		return "DISABLED";
	};

	// estos metodos de show, on_show, hide, on_hide, son utilizados unicamente en el override de members
	adm_capability_handle_override.prototype.show = function( callback )
	{
		callback = callback || $.noop;
		if ( !this.panel_id ) {
			callback();
			return;
		}
		$("#" + this.panel_id).show();
		this.on_show( callback );
	};

	adm_capability_handle_override.prototype.hide = function( callback )
	{
		callback = callback || $.noop;
		if ( !this.panel_id ) {
			callback();
			return;
		}
		$("#" + this.panel_id).hide();
		this.on_hide( callback );
	};

	adm_capability_handle_override.prototype.on_show = function( callback )
	{
		callback();
	};

	adm_capability_handle_override.prototype.on_hide = function( callback )
	{
		this.parent_frm.hide_messages( 3000 /* timeout */ );
		callback();
	};

	adm_capability_handle_override.prototype.set_events = function()
	{
		var self = this;

		adm_capability_handle_override.uber.set_events.call( this ); //invokes the base class method

		var $container = $("#" + this.container_id);
		var event_selector = "[id='" + this.get_id("ENABLE_OVERRIDE") + "']";

		$container.off("change", event_selector)
			.on("change", event_selector, function ()
			{
				var $this = $( this );
				var enable = $this.prop("checked");
				var $capabilities = $container.find("[ctrl_type='channel_container']");
				$.each( $capabilities, function( _, capability )
				{
					var $capability = $( capability );
					var state = enable ? "ENABLED" : "DISABLED";
					self.enable_channel( state, $capability);
				});
			});
	};

	adm_capability_handle_override.prototype.can_enable_channel = function(config)
	{
		var can_enabled = adm_capability_handle_override.uber.can_enable_channel.call(this, config); //invokes the base class method
		if (!can_enabled) {
			return false;
		}
		var campaign = this.parent_frm.get_entity_instance();
		if (!campaign) {
			return false;
		}
		var accounts = campaign.accounts;
		if (!$.isArray(accounts) || accounts.length === 0) {
			return false;
		}
		var enabled = false;
		$.each( accounts, function( _, account )
		{
			if (config.can_enable(config, account.proxyType)) {
				// existe al menos una cuenta configurada de este channel en la campana, entonces dejo enabled la opcion de override
				enabled = true;
				return false; // break
			}
		});
		return enabled;
	};

	adm_capability_handle_override.prototype.build_markup = function( cleanup_container )
	{
		var $container = $("#" + this.container_id);
		if ( cleanup_container ) {
			$container.html("");
			cleanup_container = false;
		}

		var id_chk_override = this.get_id("ENABLE_OVERRIDE");
		const $chk_override = $("".concat(
			"<div class='col-12'>",
				"<div class='d-flex justify-content-between header-form-switch'>",
					"<div class='card-header'>",
						"<h5>",
							"<%=_T('Override User Capability')%>",
						"</h5>",
						"<span class='fa fa-question-circle' title='", "<%=_T('Allows you to specify the user permissions within this campaign. It will overwrite the one established within the User entity.')%>", "'></span>",
						"<i id='", id_chk_override, "_validation' class='formFieldValidation formFieldError' hidden ></i>",
					"</div>",
					"<div class='form-group mb-0'>",
						"<div class='form-check form-switch'>",
							"<label for='", id_chk_override, "' class='form-check-label form-check-label-title'>",
								"<input id='", id_chk_override, "' class='form-check-input' type='checkbox'>",
								"<span></span>",
							"</label>",
						"</div>",
					"</div>",
				"</div>",
			"</div>",
			
		));

		if (this.override_type === "CAMPAIGN-MEMBER") {
			if (this.parent_frm.frm.is_allegro_campaign()) {
				$chk_override.hide();
			}
		}

		// insert checkbox enable override
		$container.html($chk_override);

		adm_capability_handle_override.uber.build_markup.call(this, cleanup_container); //invokes the base class method
	};

	adm_capability_handle_override.prototype.get_validation_def = function()
	{
		var self = this;
		var validation_def = adm_capability_handle_override.uber.get_validation_def.call(this); //invokes the base class method

		if ( this.override_type === "CAMPAIGN-MEMBER" ) {
			var id_chk_override = this.get_id("ENABLE_OVERRIDE");

			validation_def.push({
				control_id: id_chk_override,
				messages: {
					custom: {
						type:  "custom",
						valid: function() {
							var response = {
								success: true
							};
							if ( !$("#" + id_chk_override).prop("checked") ) {
								// is ok
								return response;
							}
							var count_enabled_channels = 0;
							var count_override_channels = 0;
							$.each(self.ctrls, function(_,config)
							{
								if ( self.can_enable_channel(config) ) {
									count_enabled_channels++;
									var $selected_options = self.__$get_option_selected($(`#${config.cbo}_container`));
									if ( $selected_options && $selected_options.length > 0 ) {
										count_override_channels++;
									}
								}
							});
							if ( count_enabled_channels === 0 ) {
								// is error
								response.success = false;
								response.message = "<%=_T('No configurable channels.')%>";
								return response;
							}
							else {
								if ( count_override_channels === 0 ) {
									// is error
									response.success = false;
									response.message = "<%=_T('Is required that at least one capability override.')%>";
									return response;
								}
							}
							// is success
							return response;
						}
					}
				}
			});
		}

		return validation_def;
	};

}());
