
/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:false, browser:true, indent:false, maxerr:50, -W014, esversion: 11 */

/* global inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	"use strict";

	const format = inConcert.Utils.format;

	var get_subgrid_value = function(row, name) {
		var original_value = row[name] !== undefined ? row[name] : "";
		var resp = {
			original_value: original_value,
			value: original_value
		};
		return resp;
	};

	let m_flow_processes, m_map_flow_processes;
	const loadFowProcesses = function() {
		const flowAPI = inConcert.app.getFlowDesignerAPI();
		flowAPI.searchFlows({}, function(success, result) {
			if (success) {
				m_flow_processes = result;
				m_map_flow_processes = inConcert.Utils.arrayToMap(m_flow_processes.rows || [],
					function(data) {
						return data;
					},
					function(data) {
						// el ID
						return (data.Id || "").toLowerCase();
					}
				);
			}
		});
	};

	//------------------------------------------------------
	// campaigns navigation config
	var campaigns_config = function(base_uri)
	{
		const get_campaign_name = function(campaign)
		{
			let campaign_name = campaign.Name;
			if (typeof campaign_name !== "string" || campaign_name.trim().length > 0) {
				campaign_name = campaign.Name;
			}
			else {
				campaign_name = campaign.Id;
			}
			return {
				original_value: campaign_name,
				value: campaign_name
			};
		};

		const make_delete_warning_detail = function(errors)
		{
			let detail = "";
			const make_detail_list = function(list, title, calculate_detail_fn) {
				let i = 0;
				for (let rule_name in list || {}) {
					if (list.hasOwnProperty(rule_name)) {
						let item = list[rule_name];
						if (detail) {
							detail = detail + "\n";
						}
						if (i === 0) {
							detail = detail + "    " + title + "<%=_T(':')%>";
							detail = detail + "\n";
						}
						let one_detail = calculate_detail_fn(item);
						detail = detail + one_detail;
						i++;
					}
				}
			};

			const get_action_name = function(action_name) {
				let actions = {
					"transfer": "<%=_T('Transfer')%>"
				};
				return actions[action_name] || action_name;
			};

			for (let related_camp_id in errors || {}) {
				if (errors.hasOwnProperty(related_camp_id)) {
					if (detail) {
						detail = detail + "\n";
					}
					detail = detail + format("<%=_T('Campaign: {0}')%>", related_camp_id);
					let err = errors[related_camp_id];

					if (err.acdRelatedCampaigns && err.acdRelatedCampaigns.action) {
						detail = detail + "\n";
					 	detail = detail + "    " + "<%=_T('ACD Timeout')%>" + "<%=_T(':')%>";
					 	detail = detail + "\n";
					 	detail = detail + "       - " + format("<%=_T('Action: {0} to account: {1}.')%>", get_action_name(err.acdRelatedCampaigns.action), err.acdRelatedCampaigns.to.accountName);
					}

					make_detail_list(err.rulesRelatedCampaigns,
						"<%=_T('Rules')%>",
						function(item) {
							return "       - " + format("<%=_T('Rule {0} action: {1} to account: {2}.')%>", item.ruleName, get_action_name(item.action), item.to.accountName);
						}
					);
				}
			}
			return detail;
		};

		const get_message_error = function(type, err, name)
		{
			let message;
			if (type === "delete") {
				let err_messages = {
					"ALLEGRO_INTEGRATION_ERROR"                                   : "<%=_T('Unable to delete {0} campaign. Allegro integration error.')%>",
					"FORBIDDEN"                                                   : "<%=_T('Unable to delete {0} campaign. You dont have permission.')%>",
					"HAS_ACTIVE_POSTS"                                            : "<%=_T('Campaign {0} has active social network posts. Close them before delete it.')%>",
					"HAS_ACTIVE_BATCHES"                                          : "<%=_T('Campaign {0} has active outgoing batches. Close them before delete it.')%>",
					"CAMPAIGN IS USED IN RULES OF OTHER CAMPAIGNS"                : "<%=_T('It is in use in the Rules of other Campaigns')%>",
					"CAMPAIGN IS USED IN ACD TIMEOUT OF OTHER CAMPAIGNS"          : "<%=_T('It is in use in the ACD timeout of other Campaigns')%>",
					"CAMPAIGN IS USED IN ACD TIMEOUT AND RULES OF OTHER CAMPAIGNS": "<%=_T('It is in use in the ACD timeout AND Rules of other Campaigns')%>",
					"CAMPAIGN HAS AN OUTBOUND PROCESS CONFIGURED THAT IS RUNNING" : "<%=_T('Unable to delete {0} campaign. The Campaign has an Outbound Process configured that is Running. Process: {1}.')%>",
					"CAMPAIGN HAS AN OUTBOUND PROCESS CONFIGURED"                 : "<%=_T('Unable to delete {0} campaign. The Campaign has an Outbound Process configured. Process: {1}.')%>"
				};
				let err_type = "";
				if (typeof err === "object") {
					if (err.errors) {
						let popup_message = err_messages[(err.result || "").toUpperCase()] || "<%=_T('Is in use')%>";
						let title, detail, msg_type;
						title = format("<%=_T('Unable to delete {0} campaign')%>", name);
						detail = make_delete_warning_detail(err.errors);
						msg_type = "Error";
						return {
							can_show_popup_message: true,
							title  : title,
							message: popup_message + "<%=_T(':')%>",
							detail : detail,
							type   : msg_type
						};
					}
					//no está duplicado, puede venir doble
					err_type = err.result || "";
					if (typeof err_type === "object") {
						err_type = err.result || "";
					}
				}
				err_type = err_type.toUpperCase();
				message = err_messages[err_type] || "<%=_T('Error deleting {0} campaign.')%>";
				if (err.rawError &&
						(err_type === "CAMPAIGN HAS AN OUTBOUND PROCESS CONFIGURED THAT IS RUNNING" || 
						err_type === "CAMPAIGN HAS AN OUTBOUND PROCESS CONFIGURED")) {
					message = format(message, name, err.rawError.processId);
				}
				else {
					message = format(message, name);
				}
			}
			return message;
		};

		var validateMessengerPages = function(campaign, data, callback)
		{
			var hasFacebookAccount = false;
			$.each(campaign.accounts || [], function(_, account) {
				if (account.proxyType === "FACEBOOK") {
					hasFacebookAccount = true;
					return false;
				}
			});
			if (!hasFacebookAccount) {
				return callback(true);
			}
			window.setTimeout(function() {
				$.inconcertPopup.Show(
					"<%=_T('Confirmation')%>",
					"<%=_T('Facebook accounts will be deleted. Would you like to restore accounts pages messenger configuration?')%>",
					$.inconcertPopup.ButtonSets.BS_YESNOCANCEL,
					function(response) {
						switch(response.toLowerCase()) {
							case "yes":
								data.mustCleanMessenger = true;
								return callback(true);
							case "no":
								data.mustCleanMessenger = false;
								return callback(true);
							default:
								return callback(false);
						}
					},
					undefined,
					{
						popup_id        : "conf_inconcertPopup",
						container_class : "confirmationPopUp",
						overlay_class   : "confirmationOverlay"
					}
				);
			}, 50);
		};

		var erase_previous_confirmation = function(campaignId, callback = $.noop)
		{
			
			inConcert.invoke({
				url: base_uri + "campaigns/get/" + campaignId,
				type: "GET",
				dataType: "json",
				contentType: "application/json",
				success: function(campaign)	{
					var data = {};
					validateMessengerPages(campaign, data, function(status, error) {
						if (!status) {
							return callback(status, error);
						}
						callback(true, data);
					});
				},
				error: function(_, error) {
					let message;
					const name = inConcert.app.get_id(campaignId);
					if (error.result === "NOT_FOUND") {
						message = format("<%=_T('Unable to delete {0} campaign. This campaign does not exist.')%>", name);
					}
					else {
						message = format("<%=_T('Unable to delete {0} campaign')%>", name);
					}
					callback(false, message);
				}
			});
		};

		// is action enabled
		var is_action_enabled = function(action, row)
		{
			/*
			 * Valores a retornar:
			 *    1  = La accion se debe visualizar y estar Enabled.
			 *    0  = La accion se debe visualizar y estar Disabled.
			 *    -1 = La accion no se genera en el markup. (No aparece el boton de la accion).
			 */
			 
			return row.Id === inConcert.app.get_internal_campaign_id() && action.toLowerCase() === "delete" ? 0 : 1;
		};

		let view_config;
		/*

		*/
		const on_search = (view, success, _, warn = [])=> {
			view_config.with_error = false;
			if (!success) {
				return;
			}
			const resp = inConcert.app.validateAllegroWarningsCampaigns(warn);
			if (resp.campaignsWithServerDownErrors.length === 0)
				return;
			const campaignsIds = resp.campaignsWithServerDownErrors.map(item => item.campaignId).join(",");
			const message = format("<%=_T('There is a users synchronization problem with Allegro in {0} campaign/s: {1}.<br>You have to edit the campaign, go to the Users section, and save the list.')%>", warn.length, campaignsIds);
			view.show_status_message(message, "Error", false);
			view_config.with_error = true;
		};

		var get_channel = function(row)
		{
			let channels = row.Thresholds;
			const all_channels = inConcert.app.load_vcc_channels();
			let value =  ( Object.keys(channels)).map(function(channel) {
				return {
					tooltip : (all_channels[channel] ? all_channels[channel].name : channel),
					class_name: "AddressesBookChannels" + channel
				};
			});
			if (value.length === 0) {
				value.push({tooltip:"<%=_T('Not contains')%>", class_name:"CLASS_ICON_NOT_FOUND"});
			 }

			return {
				original_value: value,
				value: value
			};
		};

		var exists_als = function(row)
		{
			let als = row.AttentionLevels;
			let value =  Object.keys(als).length > 0;

			return {
				original_value: value,
				value: value
			};
		};

		var get_tooltip_als = function(row) {
			return Object.keys(row.AttentionLevels).length > 0? "<%=_T('Yes')%>": "<%=_T('No')%>";
		};

		var exists_dispositio_codes = function(row)
		{
			let value = (row.DispositionCodes || []).length > 0;

			return {
				original_value: value,
				value: value
			};
		};

		var get_tooltip_dispositio_codes = function(row) {
			return (row.DispositionCodes || []).length > 0? "<%=_T('Yes')%>": "<%=_T('No')%>";
		};

		var get_total_accounts = function(row)
		{
			let accounts = row.Accounts;
			let value = Object.keys(accounts).length;
			return {
				original_value: value,
				value: value
			};
		};

		var get_total_members = function(row)
		{
			let value = Object.keys(row.Members).length;
			return {
				original_value: value,
				value: value
			};
		};

		view_config = {
			id: "campaigns",
			name: "<%=_T('Campaigns')%>",
			permissions: {
				view: [ 
					"viewcampaigns",
					"view_campaigns_basic",
					"view_campaigns_accounts",
					"view_campaigns_channels",
					"view_campaigns_attention_level",
					"view_campaigns_members",
					"view_campaigns_labels",
					"view_campaigns_schedule",
					"view_campaigns_dispositions",
					"view_campaigns_canned",
					"view_campaigns_cc_integration",
					"view_campaigns_rules",
					"view_campaigns_roles_permissions",
					"view_campaigns_interaction_notifications"
				],
				save: "create_campaign" ,
				erase: "deletecampaign" ,
				permission_type: "campaign",
				get_campaign_id: function( view, row ) {
					return row[ view.row_id ]; // row.Id sale de (*)
				}
			},
			make_delete_warning_detail: function(errors) {
				return make_delete_warning_detail(errors);
			},
			selected: true,
			class_name: "itemCampaign",
			i_class_name: "far fa-bullhorn",
			view: {
				render: "/inconcert/apps/configuration/pages/view_content_default.<%=_L%>.html#grunt-cache-bust",
				title: "<%=_T('Campaigns')%>",
				tagline: "<%=_T('Tagline for Campaigns')%>",
				new_button_name: "<%=_T('Create campaign')%>",
				row_id: "Id", // sale de aca (*)
				display_name: "Name",
				change_nav_sections: true,
				paginator: {
					paged_text_format : "<%=_T('{page_row_start}-{page_row_end} of {many}')%>"
				},
				messages: {
					erase: "<%=_T('Do you want to delete campaign {0} ?')%>",
					err: get_message_error
				},
				uri: {
					erase:  base_uri + "campaigns/delete/",
					search: base_uri + "campaigns/search/"
				},
				erase_previous_confirmation: erase_previous_confirmation,
				source: ["Id", "Name", "Description", "UpdateMembersResult", "Members", "Accounts", "Thresholds", "AttentionLevels", "DispositionCodes"],
				columns: [
					{id: "_number"    , name: "", type: "counter"   },
					{id: "Name"       , name: "<%=_T('Name')%>", get_value: get_campaign_name},
					{id: "Description", name: "<%=_T('Description')%>"},
					{ id: "Channels"    , name: "<%=_T('Channels')%>"    , get_value: get_channel,   type: "multi_image" ,
						cell_class_name : "AddressesBookIconColumn d-flex", without_tooltip: true },
					{id: "Accounts", name: "<%=_T('Accounts')%>", get_value: get_total_accounts},
					{id: "Members", name: "<%=_T('Members_')%>", get_value: get_total_members},
					{id: "AttentionLevels", name: "<%=_T('Levels')%>", type: "boolean", get_tooltip_header:"<%=_T('Attention Levels')%>", get_tooltip: get_tooltip_als, get_value: exists_als},
					{id: "DispositionCodes", name: "<%=_T('Codes')%>", type: "boolean", get_tooltip_header:"<%=_T('Disposition Codes')%>", get_tooltip: get_tooltip_dispositio_codes, get_value: exists_dispositio_codes},
					{id: "_action"    , name: "", 
						system_action: {edit: true, erase: true},
						is_enabled: is_action_enabled
					}
				],
				filter: {fields: {"Name": {control: null}, "Description": {control: null}}},
				realtime: null,
				on_ready: function() {
					if (inConcert.app.has_allegro_integration()) {
						if (view_config.with_error) {
							return {
								action: "REFRESH_VIEW"
							};
						}
					}
					// Se necesitan cargar los procesos flow en la primera vista que se cargue de administracion.
					// Asi en la vista de Shared Files no se visualiza el GUID de los flows.
					loadFowProcesses();
				},
				//
				on_search: on_search
/*
// El siguiente es un ejemplo de como implementar las actualizaciones realtime para las vistas.
// al definir lo siguiente, y cuando alguien publique un cambio a "pub_id", el evento llegara a la funcion "callback"
// faltaria implementar en conf_view la actualizacion correspondiente. ( No se realiza ahora para no perder tiempo en algo que no se ha pedido)
				realtime: {
					"view_campaign_added_campaign": {
						pub_id: "/application/campaigns/added",
						callback: function( data ) {
							window.alert( " (view campaign) /application/campaigns/added " + data );
						}
					},
					"view_campaign_updated_campaign": {
						pub_id: "/application/campaigns/updated",
						callback: function( data ) {
							window.alert( " (view campaign) /application/campaigns/updated " + data );
						}
					},
					"view_campaign_removed_campaign": {
						pub_id: "/application/campaigns/removed",
						callback: function( data ) {
							window.alert( " (view campaign) /application/campaigns/removed " + data );
						}
					},
					"view_campaign_updated_accounts": {
						pub_id: "/application/campaigns/update_accounts",
						callback: function( data ) {
							window.alert( " (view campaign) /application/campaigns/update_accounts " + data );
						}
					}
				}
*/
			},
			form: {instance: inConcert.adm_form_campaigns}
		};

		return view_config;
	};

	//------------------------------------------------------
	// users navigation config
	var users_config = function(base_uri)
	{
		var user_config;

		var get_user_status = function(status)
		{
			var state = "OFFLINE";
			if (typeof status !== "object") {
				return state;
			}
			if (typeof status.state === "string" && status.state.trim().length > 0) {
				state = status.state;
			}
			return state;
		};

		var get_role_name = function( user )
		{
			var roles;
			inConcert.app.loadAllRoles(
				function( success, loaded_roles ) {
					if ( success ) {
						roles = loaded_roles;
					}
				},
				true /* from cache */
			);

			var user_role = user.UserGroup;
			var value = {
				original_value: user_role,
				value: user_role
			};

			user_role = ( user_role || "" ).toUpperCase();
			$.each( roles || {} , function( _, role ) {
				if ( user_role === ( role.Id || "" ).toUpperCase() ) {
					value.value = role.NameLabel || value.value;
					return false; // break
				}
			});

			return value;
		};

		const get_full_name  = function( {Name, FirstSurname} ) {
			const full_name = `${Name} ${FirstSurname}`;
			return {
				original_value: full_name,
				value         : full_name
			};
		};

		const get_Status_tooltip = function( user )
		{			
			if(user.Status === "1" || user.StatusReason === undefined) {
				return  "<%=_T('Enabled')%>";
			}
			const know_Status_descriptions = {
				"ADMIN": "<%=_T('Disabled by an administrator')%>",
				"MAX_LOGIN_RETRIES": "<%=_T('Disabled due to maximum login failed retries reached')%>",
				"SYNC": "<%=_T('Disabled by external synchronization process')%>"
			};
			const translated_Status_description = know_Status_descriptions[user.StatusReason];
			return translated_Status_description || "<%=_T('Disabled')%>";
		};

		const get_user_tooltip = function( user )
		{
			if (user.isExpired) {
				return "<%=_T('User locked. The password has expired.')%>";
			}
			return user.Id;
		};
		

/*
		// get user avatar
		var get_user_avatar = function(row)
		{
			return {
				original_value: "",
				value: "<img src='" + inConcert.app.getBaseDomain() + "/inconcert/apps/agent/users/get_avatar/" + row.Id + "' width='24' height='24'>"
			};
		};
*/
		// is action enabled
		const is_action_enabled = function(action, row)
		{
			/*
			 * Valores a retornar:
			 *    1  = La accion se debe visualizar y estar Enabled.
			 *    0  = La accion se debe visualizar y estar Disabled.
			 *    -1 = La accion no se genera en el markup. (No aparece el boton de la accion).
			 */

			var has_permission_to_activate_or_deactivate = function()
			{
				return inConcert.permissions.to_action( "administration" /*app*/, "deactivate_user" );
			};
			switch(action.toLowerCase()) {
				case "activate":
					if ( !has_permission_to_activate_or_deactivate() ) {
						return -1;
					}
				// si el usuario se encuentra available, no muestra el boton para activarlo.
					if (row.Status === "1" || row.Status === undefined) {
						return -1;
					}
					break;

				case "deactivate":
					if ( !has_permission_to_activate_or_deactivate() ) {
						return -1;
					}
					if (row.Status === undefined) {
						return 1;
					}
				// muestra el boton para activar si el usuario se encuentra offline.
					var status = get_user_status(row.status);
					if (status.toUpperCase() !== "OFFLINE") {
						return 0;
					}
				// si el usuario se encuentra desactivado, no muestra el boton para desactivarlo.
					if (row.Status === "0") {
						return -1;
					}
					break;
			}
			return 1;
		};

		var swap_button_action = function(new_action, $row)
		{
			if (!$row) {
				return;
			}
			var current_action = (new_action === "deactivate" ? "activate" : "deactivate");
			var columns = user_config.view.columns;
			var column_custom_action = null;
			$.each(columns, function(_, column) {
				if (column.custom_action) {
					column_custom_action = column.custom_action;
					return false; // break
				}
			});
			var action_config = column_custom_action[new_action];
			var $action_button = $row.find("input[action='" + current_action + "']");
			$action_button
				.attr("action", new_action)
				.val(action_config.name);
		};

		// execute custom action
		var execute_custom_action = function(action_id, $row, view, _ /*$btn*/, execute_callback = $.noop)
		{
			var row_id = $row.attr("row_id");
			var display_name = $row.attr("row_name");
			var uri = base_uri;
			var confirmation_message, waiting_message, success_message, error_message;
			action_id = action_id.toLowerCase();
			if (action_id === "activate") {
				uri = uri + "users/activate/";
				confirmation_message = "<%=_T('Do you want to activate User {0} ?')%>";
				waiting_message = "<%=_T('Please wait, activating User {0}')%>";
				success_message = "<%=_T('User {0} was successfully activated.')%>";
				error_message = "<%=_T('Error activating User {0}.')%>";
			}
			else {
				uri = uri + "users/deactivate/";
				confirmation_message = "<%=_T('Do you want to deactivate User {0} ?')%>";
				waiting_message = "<%=_T('Please wait, deactivating User {0}')%>";
				success_message = "<%=_T('User {0} was successfully deactivated.')%>";
				error_message = "<%=_T('Error deactivating User {0}.')%>";
			}
			confirmation_message = format(confirmation_message, display_name);
			waiting_message = format(waiting_message, display_name);
			success_message = format(success_message, display_name);
			error_message = format(error_message, display_name);

			view.dashboard.hide_status_message();

			$.inconcertPopup.Show(
				"<%=_T('Confirm')%>",
				confirmation_message,
				jQuery.inconcertPopup.ButtonSets.BS_YESNO,
				function(response) {
					if (response.toLowerCase() !== "yes") {
						return;
					}
					view.dashboard.show_waiting(waiting_message);
					var request_response = function(success, error)
					{
						var message, message_type, timeout_ms;
						if (success) {
							message = success_message;
							const response = error;
							message_type = "Info";
							var entity;
							if (action_id === "deactivate") {
								var filter_values = view.get_filter_values();
								if ( filter_values.Status === "1" ) {
									view.erase_grid_row( $row );
								}
								else {
									swap_button_action("activate", $row);
									entity = {Status: "0", isExpired: (response && response.info && response.info.isExpired) || false};
									view.update_row( entity, $row );
								}
							}
							else {
								swap_button_action("deactivate", $row);
								entity = {Status: "1", isExpired: (response && response.info && response.info.isExpired) || false};
								view.update_row( entity, $row );
							}
						}
						else {

							switch ( error.result.toUpperCase() )
							{
								case "FORBIDDEN":
									error_message = "<%=_T('Unable to activate / deactivate {0} user. You dont have permission.')%>";
									error_message = format(error_message, display_name);
									break;

							// deactiviating
								case "USER_IS_LOGGED":
									error_message = "<%=_T('Unable to deactivate user {0}. User is logged in.')%>";
									error_message = format(error_message, display_name);
									break;

								case "USER_HAS_INTERACTIONS":
									error_message = "<%=_T('Unable to deactivate user {0}. User has assigned conversations.')%>";
									error_message = format(error_message, display_name);
									break;

							//activating
								case "CAN_NOT_RESERVE_PEER":
									error_message = "<%=_T('Unable to activate user {0}. Can not reserve peer.')%>";
									error_message = format(error_message, display_name);
									break;

								case "USER_HAS_NOT_PREVIOUS_MAIL":
									error_message = "<%=_T('Unable to activate user {0}. User has no previous e-mail.')%>";
									error_message = format(error_message, display_name);
									break;
								
								case "HELPDESK_INTEGRATION_ERROR":
									const vcc_id = inConcert.app.get_vcc_id();
									const message_error = error.message_error.replace("@"+vcc_id,"")
									error_message = `${error_message} ${message_error}`;
									break;

								case "ALLEGRO_INTEGRATION_ERROR":
									error_message = "<%=_T('Unable to {0} user {1}. Allegro integration error.')%>";
									error_message = format(error_message, action_id, display_name);
									break;

								case "MAIL_ALREADY_TAKEN":
									var config = {
										focus: "adm_form_user_mail",
										message: {
											mail: "<%=_T('Please enter the e-mail address for activate the user.')%>",
											update_button: {
												id: view.dashboard.prefix_id + "form_button_save",
												caption: "<%=_T('Update & Activate')%>"
											}
										}
									};
									view.grid_execute_action("edit", $row, config);
									execute_callback(success, error);
									return;
							}

							message = error_message;
							message_type = "Error";
							timeout_ms = false; //infinite
						}
						view.show_status_message(message, message_type, timeout_ms);
						execute_callback(success, error);
					};

					// ejecuto el request para el cambio de estado
					inConcert.invoke({
						url: uri,
						type: "POST",
						data: JSON.stringify({
							memberId: row_id
						}),
						dataType: "json",
						contentType: "application/json",
						success: function(response) {
							request_response(true, response);
						},
						error: function(_, text_status) {
							var err;
							try {
								err = JSON.parse(text_status);
								err = err.result;
							}
							catch(e) {
								err = text_status;
							}
							request_response(false, err);
						}
					});

				},
				undefined,
				{
					popup_id        : "conf_inconcertPopup",
					container_class : "confirmationPopUp",
					overlay_class   : "confirmationOverlay"
				}
			);
		};

		var get_row_classes = function( row )
		{
			let classes;
			if ( row.HasRoleOverride === "1" ) {
				classes = "override";
			}

			if ( row.isExpired) {
				classes = classes? classes + " userLocked": "userLocked";
			}
			
			return classes;
		};

		// Aqui decido si instancio la clase base de frm users, o la clase hija de integracion con allegro. 
		var frm_instance;
		if ( inConcert.app.has_allegro_integration() ) {
			frm_instance = inConcert.adm_form_users_allegro;
		}
		else {
			frm_instance = inConcert.adm_form_users;
		}

		const can_show_importations = function() {
			return inConcert.permissions.to_action("administration", "import_users")
				&& inConcert.permissions.to_action("administration", "saveuser");
		};

		const can_export_users = function() {
			return inConcert.permissions.to_action("administration", "export_users");
		};

		const show_importations = function(view) {
			if (!can_show_importations()) {
				return view.show_status_message(
					"<%=_T('Unable to show importations: You dont have permission.')%>",
					"Error",
					5000
				);
			}
			view.render_popup({
				super_popup: true
			});
		};

		var export_users = function(view) {
			if (!can_export_users()) {
				var error_message = "<%=_T('Unable to export: You dont have permission.')%>";
				view.show_status_message( error_message, "Error", 5000 );
				return;
			}
			const waiting_message = "<%=_T('Please wait, exporting Users')%>";
			view.dashboard.show_waiting(waiting_message);
			const show_inactive = view.$get_dom_element_by_id("filter_custom_Status").prop("checked")?"1":"0";
			const uri = base_uri + "users/export/?show_inactive=" + show_inactive;
			$.fileDownload(uri, {
				httpMethod : "GET",
				cookieName : "download_started"
			})
			.done(function() {
				view.dashboard.hide_waiting();
			})
			.fail(function(error) {
				const error_message = "<%=_T('Unable to export')%>";
				view.dashboard.hide_waiting();
				view.show_status_message( error_message, "Error", 5000 );
				console.error(error);
				}
			);
		};

		user_config = {
			id: "users",
			name: "<%=_T('Users')%>",
			permissions: {
				view    : "viewusers",
				save    : "saveuser",
				erase   : "deactivate_user",
				permission_type: "vcc"
			},
			class_name: "itemUsers",
			i_class_name: "far fa-user-friends",
			view: {
				render: "/inconcert/apps/configuration/pages/view_content_default.<%=_L%>.html#grunt-cache-bust",
				title: "<%=_T('Users')%>",
				tagline: "<%=_T('Tagline for Users')%>",
				new_button_name: "<%=_T('Create user')%>",
				row_id: "Id",
				display_name: "Id",
				paginator: {
					paged_text_format : "<%=_T('{page_row_start}-{page_row_end} of {many}')%>"
				},
				header: {
					hide_new_button   : inConcert.app.HasExternalUsersSync()
				},
				global_buttons: [
					{
						id: "importations",
						name: "<%=_T('Import')%>",
						className : "buttonColorWhite pageButtonNoIcon",
						enabled: can_show_importations,
						run: function(view) {
							show_importations(view);
						},
						before: false
					},
					{
						id: "exports",
						name: "<%=_T('Export')%>",
						className : "buttonColorWhite pageButtonNoIcon pageButtonRightSeparator",
						enabled: can_export_users,
						run: function(view) {
							export_users(view);
						},
						before: false
					}
				],
				messages: {
					//erase: "<%=_T('Do you want to delete user {0}')%>"
				},
				uri: {
					//erase:  base_uri + "users/delete/",
					search: base_uri + "users/search/"
				},
				source: ["Id", "Name", "FirstSurname", "UserGroup", "Status", "HasRoleOverride", "StatusReason", "isExpired"],
				columns: [
					{id: "_number"    , name: "", type: "counter"   },
//					{id: "_avatar"      , name: "", get_value: get_user_avatar}, //Atencion: esta linea esta comentada pero no eliminar
					{id: "Id"         , name: "<%=_T('Id')%>", get_tooltip: get_user_tooltip},
					{id: "FullName", name: "<%=_T('Full Name')%>" , get_value: get_full_name},
					{id: "UserGroup", name: "<%=_T('Permission')%>", get_value: get_role_name ,	
						get_tooltip: function(user) {
							let hasRoleOverrideTooltip = "";
							let hasRoleOverrideData = user.HasRoleOverride;
							if (hasRoleOverrideData === "1") {
								hasRoleOverrideTooltip = "<%=_T('Role Permissions')%>";
							}
							return hasRoleOverrideTooltip ;
						},
					},
					{id: "Status", name: "<%=_T('Active')%>", type: "boolean", class_name: "booleanColumnTitle", get_tooltip: get_Status_tooltip},
					{id: "_action",
//						class_name: "gridColumnThreeActions",
						system_action: {edit: true},
						custom_action: {
							activate: {
								name: "<%=_T('Activate')%>",
								execute: execute_custom_action,
								class_name: "activate activateUser"
							},
							deactivate: {
								name: "<%=_T('Deactivate')%>",
								execute: execute_custom_action,
								class_name: "deactivate deactivateUser"
							}
						},
						is_enabled: is_action_enabled
					}
				],
				rows: {
					get_row_classes: get_row_classes
				},
				filter: {
					fields: {
						"Id": {control: null},
						"Name": {control: null},
						"FirstSurname": {control: null},
						"UserGroup": {control: null},
						"Status": {control: "switchbox",
						name: "<%=_T('Show Inactive')%>", default_value: "",
							get_value: function($control) {
								// (1) Si el checkbox de Enabled, se encuentra marcado tiene que mostrar tanto los Enabled = True como los Enabled = False (Show Inactive).
								// (2) Si el checkbox de Enabled, se encuentra desmarcado, solo tiene que mostrar los Enabled = True.
								if ($control.prop("checked")) {
									// retornando un string vacio muestra (1)
									return "";
								}
								else {
									// retornando un string vacio muestra (2)
									return "1";
								}
							}
						}
					}
				},
				realtime: null,
				on_update_view: function(_, $row)
				{
					// siempre que se actualiza un usuario, el mismo pasa a estar activo, entonces el boton de la accion
					//	pasa a ser "deactivate".
					var new_action = "deactivate";
					swap_button_action(new_action, $row);
				}
			},
			form: {instance: frm_instance}
		};

		return user_config;
	};

	//------------------------------------------------------
	// roles navigation config
	var roles_config = function(base_uri)
	{
		var get_message_error = function(type, err, name)
		{
			var message;
			if (type === "delete") {
				var err_messages = {
					"ALLEGRO_INTEGRATION_ERROR" : "<%=_T('Unable to delete {0} permission. Allegro integration error.')%>",
					"FORBIDDEN"                 : "<%=_T('Unable to delete {0} permission. You dont have permission.')%>",
					"ROLE_BEING_USED"           : "<%=_T('The permission {0} to delete is currently in use. You must release it before deletion.')%>"
				};
				if (typeof err === "object") {
					err = err.result || "";
				}
				message = err_messages[err.toUpperCase()] || "<%=_T('Error deleting {0} permission.')%>";
				message = format(message, name);
			}
			return message;
		};

		// is action enabled
		var is_action_enabled = function( action, row )
		{
			/*
			 * Valores a retornar:
			 *    1  = La accion se debe visualizar y estar Enabled.
			 *    0  = La accion se debe visualizar y estar Disabled.
			 *    -1 = La accion no se genera en el markup. (No aparece el boton de la accion).
			 */
			if ( action.toLowerCase() === "delete" ) {
				if ( row.IsSystemGroup === "1" ) {
					return 0;
				}
			}
			return 1;
		};

		var get_row_classes = function( row )
		{
			if ( row.IsSystemGroup !== "1" ) {
				return;
			}
			if ( row.IsModified !== "1" ) {
				return;
			}
			return "modified";
		};

		var can_export_roles = function() {
			return inConcert.permissions.to_action("administration", "exportgroup");
		};

		var export_roles = function(view) {
			if (!can_export_roles()) {
				var error_message = "<%=_T('Unable to export: You dont have permission.')%>";
				view.show_status_message( error_message, "Error", 5000 );
				return;
			}
			const waiting_message = "<%=_T('Please wait, exporting roles')%>";
			view.dashboard.show_waiting(waiting_message);
			const uri = base_uri + "groups/export/";
			$.fileDownload(uri, {
				httpMethod : "GET",
				cookieName : "download_started"
			})
			.done(function() {
				view.dashboard.hide_waiting();
			})
			.fail(function(error) {
				const error_message = "<%=_T('Unable to export')%>";
				view.dashboard.hide_waiting();
				view.show_status_message( error_message, "Error", 5000 );
				console.error(error);
				}
			);
		};

		const get_tooltip = function(row, type)
		{
			let msg = "";

			switch(type)
			{
				case "IsSystemGroup":
					
					let opertaion_id = (row.IsSystemGroup || "").toUpperCase();
					switch (opertaion_id) {
						case "0":
							msg = "<%=_T('Not system role')%>";
							break;
						case "1":
							msg = "<%=_T('System role')%>";
							break;
						default:
							break;
					}
				break;
			}

			return msg;
		};

		return {
			id: "roles",
			name: "<%=_T('Roles and permissions')%>", 
			permissions: {
				view    : "viewgroups",
				save    : "savegroup",
				erase   : "deletegroup",
				permission_type: "vcc"
			},
			class_name: "itemPermissions",
			i_class_name: "far fa-key",
			view: {
				render: "/inconcert/apps/configuration/pages/view_content_default.<%=_L%>.html#grunt-cache-bust",
				title: "<%=_T('Roles and permissions')%>", 
				tagline: "<%=_T('Tagline for Permissions')%>",
				new_button_name: "<%=_T('Create permission')%>",
				row_id: "Id",
				display_name: "Name",
				paginator: {
					paged_text_format : "<%=_T('{page_row_start}-{page_row_end} of {many}')%>"
				},
				global_buttons: [
					{
						id: "exports",
						name: "<%=_T('Export Roles')%>",
						className : "buttonColorWhite pageButtonNoIcon pageButtonRightSeparator",
						enabled: can_export_roles,
						run: function(view) {
							export_roles(view);
						},
						before: false
					}
				],
				messages: {
					erase: "<%=_T('Do you want to delete permission {0} ?')%>",
					err: get_message_error
				},
				uri: {
					erase:  base_uri + "groups/delete/",
					search: base_uri + "groups/search/"
				},
				request: {
					timeout: 55000 // Lo dejo en 55 segundos ya que el timeout por defecto en NGINX esta en 60 segundos
				},
				source: ["Id", "Name", "Description", "IsSystemGroup", "IsModified"],
				columns: [
					{ id: "_number"       , name: "", type: "counter"   },
					{ id: "Name"          , name: "<%=_T('Name')%>"},
					{ id: "Description"   , name: "<%=_T('Description')%>"},
					{ id: "ModifiedPermission"   , name: "<%=_T('Modified Permissions')%>", type: "boolean", class_name: "booleanColumnTitle",
						get_tooltip: function( user ) {
						if (user.IsModified === "1" && user.IsSystemGroup === "1") {
							return "<%=_T('Modified Permissions')%>";
						}
						return user.Id;
					}},
					{ id: "IsSystemGroup" , name: "<%=_T('System Permission')%>", type: "boolean", class_name: "booleanColumnTitle", get_tooltip: function(row) {
						return get_tooltip(row, "IsSystemGroup");
					}},
					{ id: "_action"       , name: "",
						system_action: { edit: true, erase: true },
						is_enabled: is_action_enabled
					}
				],
				rows: {
					get_row_classes: get_row_classes
				},
				filter: {fields: {"Name": {control: null}, "Description": {control: null}}},
				realtime: null
			},
			form: { instance: inConcert.adm_form_roles }
		};
	};

	//------------------------------------------------------
	// states navigation config
	var states_config = function(base_uri)
	{
		var get_message_error = function(type, err, name)
		{
			var message;
			if (type === "delete") {
				var err_messages = {
					"ALLEGRO_INTEGRATION_ERROR" : "<%=_T('Unable to delete {0} custom state. Allegro integration error.')%>",
					"FORBIDDEN"                 : "<%=_T('Unable to delete {0} custom state. You dont have permission.')%>"
				};
				if (typeof err === "object") {
					err = err.result || "";
				}
				message = err_messages[err.toUpperCase()] || "<%=_T('Error deleting {0} custom state.')%>";
				message = format(message, name);
			}
			return message;
		};

		var erase_previous_validation = function(state_id, callback)
		{
			callback = callback || $.noop;

			inConcert.invoke({
				url: base_uri + "states/delete_validation/" + state_id,
				type: "GET",
				dataType: "json",
				contentType: "application/json",
				success: function(response)
				{
					var message;
					if (response && response.warning) {
						if (response.warning.info === "STATE_BEING_USED_ON_INITIAL_STATE") {
							message = "<%=_T('Warning, {0} state is being used on initial state. Also want to delete ?')%>";
						}
					}
					callback(true, message);
				},
				error: function(_, error) {
					var message;
					if (error.result === "NOT_FOUND") {
						message = "<%=_T('Unable to delete {0} state. This state does not exist.')%>";
					}
					else {
						message = "<%=_T('Unable to delete {0} state.')%>";
					}
					callback(false, message);
				}
			});
		};

		// Aqui decido si instancio la clase base de frm states, o la clase hija de integracion con allegro. 
		var frm_instance;
		if ( inConcert.app.has_allegro_integration() ) {
			frm_instance = inConcert.adm_form_states_allegro;
		}
		else {
			frm_instance = inConcert.adm_form_states;
		}

		return {
			id: "states",
			name: "<%=_T('Custom States')%>",
			permissions: {
				view    : "viewuserstates",
				save    : "saveuserstate",
				erase   : "deleteuserstate",
				permission_type: "vcc"
			},
			class_name: "itemStates",
			i_class_name: "far fa-flag",
			view: {
				render: "/inconcert/apps/configuration/pages/view_content_default.<%=_L%>.html#grunt-cache-bust",
				title: "<%=_T('Custom States')%>",
				tagline: "<%=_T('Tagline for Custom States')%>",
				new_button_name: "<%=_T('Create custom status')%>",
				row_id: "Id",
				display_name: "Name",
				paginator: {
					paged_text_format : "<%=_T('{page_row_start}-{page_row_end} of {many}')%>"
				},
				messages: {
					erase: "<%=_T('Do you want to delete state {0} ?')%>",
					err: get_message_error
				},
				uri: {
					erase:  base_uri + "states/delete/",
					search: base_uri + "states/search/"
				},
				erase_previous_validation: erase_previous_validation,
				source: ["Id", "Name", "Description"],
				columns: [
					{id: "_number"    , name: "", type: "counter"   },
					/*
						Dejo comentado, si quisieramos habilitar el delete en batch, tenemos que descomentar y tambien la defincion de "adm_form_states.js" para este frm.
					*/
					// {id: "_select", name: "", type: "row_selector",
					// 	is_checked: function(/*row*/) {
					// 		return false;
					// 	}
					// },
					{id: "Name"       , name: "<%=_T('Name')%>"   },
					{id: "Description", name: "<%=_T('Description')%>"},
					{id: "_action"    , name: "", system_action: {edit: true, erase: true}}
				],
				filter: {fields: {"Name": {control: null}, "Description": {control: null}}},
				realtime: null
				/*
					Dejo comentado, si quisieramos habilitar el delete en batch, tenemos que descomentar y tambien la defincion de "adm_form_states.js" para este frm.
				*/
				// header_actions: {
				// 	system_actions: {
				// 		batch_actions: {caption: null}
				// 	}
				// }
			},
			form: {instance: frm_instance}
		};
	};

	//------------------------------------------------------
	// workgroups navigation config
	var workgroups_config = function(base_uri)
	{
		var get_message_error = function(type, err, name)
		{
			var message;
			if (type === "delete") {
				var err_messages = {
					"ALLEGRO_INTEGRATION_ERROR"     : "<%=_T('Unable to delete {0} workgroup. Allegro integration error.')%>",
					"FORBIDDEN"                     : "<%=_T('Unable to delete {0} workgroup. You dont have permission.')%>",
					"WORKGROUP_BEING_USED_ON_USERS" : "<%=_T('Unable to delete {0} workgroup. It’s assigned to user.')%>"
				};
				if (typeof err === "object") {
					err = err.result || "";
				}
				message = err_messages[err.toUpperCase()] || "<%=_T('Error deleting {0} workgroup.')%>";
				message = format(message, name);
			}
			return message;
		};

		const get_totals = function(row, type) {
			let resp = {class_name : ""};

			switch(type)
			{
				case "Agents":
					resp.value = Object.keys(row.Agents || {}).length;
					resp.original_value = row.Agents;
				break;
				case "Supervisors":
					resp.value = Object.keys(row.Supervisors || {}).length;
					resp.original_value = row.Supervisors;
				break;
				default:
					break;
			}

			return resp;
		};

		const get_tooltip_supervisors = function(row) {
			let campaigns_validated = {};
			let campaigns = [];
			   $.each(row.Supervisors || [], function(_, data) {
				   const workgroupCampaigns = data.WorkgroupCampaigns;
				   $.each(workgroupCampaigns || [], function(id) {
						if (!campaigns_validated[id]) {
						   campaigns_validated[id] = true;
						   campaigns.push(id);
						}
				   });
				   
			   });
			   return campaigns.join();
		};
		
		return {
			id: "workgroups",
			name: "<%=_T('Workgroups')%>",
			permissions: {
				view    : "viewworkgroup",
				save    : "saveworkgroup",
				erase   : "deleteworkgroup",
				permission_type: "vcc"
			},
			class_name: "itemWorkgroups",
			i_class_name: "far fa-users-class",
			view: {
				render: "/inconcert/apps/configuration/pages/view_content_default.<%=_L%>.html#grunt-cache-bust",
				title: "<%=_T('Workgroups')%>",
				tagline: "<%=_T('Tagline for Workgroups')%>",
				new_button_name: "<%=_T('Create workgroup')%>",
				row_id: "Id",
				display_name: "Name",
				paginator: {
					paged_text_format : "<%=_T('{page_row_start}-{page_row_end} of {many}')%>"
				},
				messages: {
					erase: "<%=_T('Do you want to delete workgroup {0} ?')%>",
					err: get_message_error
				},
				uri: {
					erase:  base_uri + "workgroups/delete/",
					search: base_uri + "workgroups/search/"
				},
				source: ["Id", "Name", "Description", "Agents", "Supervisors"],
				columns: [
					{id: "_number"    , name: "", type: "counter"     },
					{id: "Name"       , name: "<%=_T('Name')%>"       },
					{id: "Description", name: "<%=_T('Description')%>"},
					{id: "Agents", name: "<%=_T('Members')%>", get_value: function(row) {
						return get_totals(row, "Agents");
					}},
					{id: "Supervisors", name: "<%=_T('Supervisors')%>", get_value: function(row) {
						return get_totals(row, "Supervisors");
					}, get_tooltip: get_tooltip_supervisors},
					{id: "_action"    , name: "" , system_action: {edit: true, erase: true}}
				],
				filter: {fields: {"Name": {control: null}, "Description": {control: null}}},
				realtime: null
			},
			form: {instance: inConcert.adm_form_workgroups}
		};
	};

	//------------------------------------------------------
	// skills navigation config
	var skills_config = function(base_uri)
	{
		var get_message_error = function(type, err, name)
		{
			var message;
			if (type === "delete") {

				var err_messages = {
					"SKILL_BEING_USED_ON_USERS_AND_ACCOUNTS" : function() {
						return format(
							"<%=_T('Unable to delete {0} skill. Skill to delete is currently in used for {1} Users and {2} Accounts. You must release it before deletion.')%>",
							name,
							err.warning.count_users,
							err.warning.count_accounts
						);
					},
					"SKILL_BEING_USED_ON_USERS" : function() {
						return format(
							"<%=_T('Unable to delete {0} skill. Skill to delete is currently in used for {1} Users. You must release it before deletion.')%>",
							name,
							err.warning.count_users
						);
					},
					"SKILL_BEING_USED_ON_ACCOUNTS" : function() {
						return format(
							"<%=_T('Unable to delete {0} skill. Skill to delete is currently in used for {1} Accounts. You must release it before deletion.')%>",
							name,
							err.warning.count_accounts
						);
					},
					"FORBIDDEN": function() {
						return format( "<%=_T('Unable to delete {0} skill. You dont have permission.')%>", name );
					},
					"ALLEGRO_INTEGRATION_ERROR": function() {
						return format( "<%=_T('Unable to delete {0} skill. Allegro integration error.')%>", name );
					}
				};

				var err_id;
				if (typeof err === "object" && typeof err.warning === "object") {
					err_id = err.warning.info || "";
				}
				else {
					err_id = err;
				}
				err_id = err_id.toUpperCase();

				if ( err_messages[err_id] ) {
					message = err_messages[err_id]();
				}
				else {
					message = format( "<%=_T('Error deleting {0} skill.')%>", name );
				}
			}
			return message;
		};

		var erase_previous_validation = function(skill_id, callback)
		{
			callback = callback || $.noop;

			if ( inConcert.app.has_allegro_integration() ) {
				// Si el VCC tiene integracion con allegro, no se permite eliminar skills utilizados.
				// La validacion y el error lo tira la funcion/uri de delete.
				callback(true);
				return;
			}

			inConcert.invoke({
				url: base_uri + "skills/delete_validation/" + skill_id,
				type: "GET",
				dataType: "json",
				contentType: "application/json",
				success: function(response)
				{
					var message;
					if (response && response.warning) {
						if (response.warning.info === "SKILL_BEING_USED_ON_USERS_AND_ACCOUNTS") {
							message = format("<%=_T('Warning, {0} skill is being used for {1} users and {2} accounts. Also want to delete ?')%>",
								skill_id,
								response.warning.count_users,
								response.warning.count_accounts
							);
						}
						else if (response.warning.info === "SKILL_BEING_USED_ON_USERS") {
							message = format("<%=_T('Warning, {0} skill is being used for {1} users. Also want to delete ?')%>",
								skill_id,
								response.warning.count_users
							);
						}
						else if (response.warning.info === "SKILL_BEING_USED_ON_ACCOUNTS") {
							message = format("<%=_T('Warning, {0} skill is being used for {1} accounts. Also want to delete ?')%>",
								skill_id,
								response.warning.count_accounts
							);
						}
					}
					callback(true, message);
				},
				error: function(_, error) {
					var message;
					if (error.result === "NOT_FOUND") {
						message = "<%=_T('Unable to delete {0} skill. This state does not exist.')%>";
					}
					else {
						message = "<%=_T('Unable to delete {0} skill.')%>";
					}
					callback(false, message);
				}
			});
		};

		return {
			id: "skills",
			name: "<%=_T('Skills')%>",
			permissions: {
				view    : "viewskill",
				save    : "saveskill",
				erase   : "deleteskill",
				permission_type: "vcc"
			},
			class_name: "itemSkills",
			i_class_name: "far fa-award",
			view: {
				render: "/inconcert/apps/configuration/pages/view_content_default.<%=_L%>.html#grunt-cache-bust",
				title: "<%=_T('Skills')%>",
				tagline: "<%=_T('Tagline for Skills')%>",
				new_button_name: "<%=_T('Create skill')%>",
				row_id: "Id",
				display_name: "Name",
				paginator: {
					paged_text_format : "<%=_T('{page_row_start}-{page_row_end} of {many}')%>"
				},
				messages: {
					erase: "<%=_T('Do you want to delete skill {0} ?')%>",
					err: get_message_error
				},
				uri: {
					erase:  base_uri + "skills/delete/",
					search: base_uri + "skills/search/"
				},
				erase_previous_validation: erase_previous_validation,
				source: ["Id", "Name", "Description", "DefaultValue"],
				columns: [
					{id: "_number"      , name: "", type: "counter"                           },
					{id: "Name"         , name: "<%=_T('Name')%>"                             },
					{id: "DefaultValue" , name: "<%=_T('Default Value')%>"                    },
					{id: "Description"  , name: "<%=_T('Description')%>"                      },
					{id: "_action"      , name: "", system_action: {edit: true, erase: true}  }
				],
				filter: {fields: {"Name": {control: null}, "Description": {control: null}, "DefaultValue": {control: null}}},
				realtime: null
			},
			form: {instance: inConcert.adm_form_skills}
		};
	};

	//------------------------------------------------------
	// Audit navigation config
	const audit_config = function(base_uri)
	{
		const get_vs_filter_values = function(rows, attrId) {
			let values = [];
			for (let i = 0; i < (rows || []).length; i++) {
				let row = rows[i];
				let rowId;
				if (typeof attrId === "function") {
					rowId = attrId(row);
				}
				else {
					attrId = attrId || "id";
					rowId = row[attrId];
				}
				if (rowId) {
					values.push(rowId);
				}
				else {
					throw("row.id is required");
				}
			}
			return values;
		};

		const get_value = function(row, type)
		{
			let resp = {};

			switch(type)
			{
				case "subEntity":
					const list = {
						Accounts : "<%=_T('Accounts')%>",
						Members : "<%=_T('Users')%>",
						DispositionCodes : "<%=_T('Disposition Codes')%>",
						Integrations : "<%=_T('Integrations')%>",
						Thresholds : "<%=_T('Thresholds')%>",
						Rules : "<%=_T('Rules')%>",
						Labels : "<%=_T('Labels')%>",
						AttentionLevels : "<%=_T('Attention Levels')%>",
						Scheduler : "<%=_T('Scheduler')%>",
						Templates : "<%=_T('Templates')%>",
						Notifications : "<%=_T('Notifications')%>",
						Channels : "<%=_T('Channels')%>",
						Procedures : "<%=_T('Procedures')%>",
						Tables : "<%=_T('Tables')%>",
						QueryExecution : "<%=_T('Query Execution')%>",

					};
					resp.value = list[row.subEntity] || row.subEntity;
					resp.class_name = "";
					resp.original_value = row.subEntity;
				break;
				case "operation":
					let msg = "<%=_T('Create')%>";
					let class_name = "gridCreated";
					let opertaion_id = (row.operation || "").toUpperCase();
					switch (opertaion_id) {
						case "DELETE":
							msg = "<%=_T('Delete')%>";
							class_name = "gridDeleted";
							break;
						case "UPDATE":
							msg = "<%=_T('Update')%>";
							class_name = "gridUpdated";
							break;
						case "IMPORT":
							msg = "<%=_T('Import')%>";
							class_name = "gridImported";
							break;
						case "EXPORT":
							msg = "<%=_T('Export')%>";
							class_name = "gridExported";
							break;
						case "EXECUTE":
							msg = "<%=_T('Execute')%>";
							class_name = "gridExecuted";
							break;
						default:
							break;
					}
					resp.value = msg;
					resp.class_name = class_name;
					resp.original_value = row.operation;
				break;
			}

			return resp;
		};

		const can_export_audits_record = function () {
			return inConcert.permissions.to_action("administration", "exportaudits");
		};

		const export_audit = function (view) {
			view.dashboard.show_waiting("<%=_T('Please wait, exporting')%>");
			const uri = base_uri + "audit/export/";
			var error_message = "<%=_T('Unable to export')%>";
			$.fileDownload(uri, {
					httpMethod : "GET",
					cookieName : "download_started",
					data      : {
						query: JSON.stringify(view.current_query)
					}
				})
				.done(function() {
					view.dashboard.hide_waiting();
				})
				.fail(function(error) {
					if(typeof error === "string" && error === "ERROR_NOT_FOUND"){
						view.show_status_message( "<%=_T('No results found.')%>", "Info", 5000 );
					}
					else{
						view.show_status_message( error_message, "Error", 5000 );
					}
					view.dashboard.hide_waiting();
					
				});
		};

		const get_search_err = function(err) {
			if (err.result === "AUDIT ROWS CANNOT BE FILTERED - A MAXIMUM RANGE OF DAYS IS ALLOWED") {
				return format("<%=_T('Audit rows cannot be filtered. A maximum range of {0} days is allowed.')%>", err.maxAllowedDays);
			}
		};

		return {
			id: "audit",
			name: "<%=_T('Audit')%>",
			permissions: {
				view    : "viewaudits",
				permission_type: "vcc"
			},
			class_name: "itemAudit",
			i_class_name: "far fa-clipboard-list-check",
			view: {
				render: "/inconcert/apps/configuration/pages/view_content_default.<%=_L%>.html#grunt-cache-bust",
				title: "<%=_T('Audit')%>",
				tagline: "<%=_T('Tagline for Audit')%>",
				className : "filterVisualSearch auditTable",
				header: {
					hide_new_button  : true,
					hide_search_ctrls: true
				},
				row_id: "Id",
				display_name: "keyField",
				paginator: {
					paged_text_format : "<%=_T('{page_row_start}-{page_row_end} of {many}')%>",
					//  show_at_least : 3,
					//  rows_per_page : 5
				},
				global_buttons: [
					{
						id: "exports",
						name: "<%=_T('Export')%>",
						className : "pageButtonGeneric buttonColorOrange pageButtonNew",
						enabled: can_export_audits_record,
						run: function(view) {
							export_audit(view);
						},
						before: false
					}
				],
				messages: {
					err: function() {
						// TODO: Hacer algo ?
					}
				},
				uri: {
					search: base_uri + "audit/filter/"
				},
				source: [], //["entity", "keyField"],
				columns: [
					{id: "_number"    , name: "", type: "counter" },
					{id: "displayTime", name: "<%=_T('Date')%>"},
					{id: "user"       , name: "<%=_T('User')%>"},
					{id: "entity"     , name: "<%=_T('Entity')%>",  get_value: function(row) {
						const items = {
							"Campaign": "<%=_T('Campaign')%>",
							"User"    : "<%=_T('User')%>",
							"Roles"   : "<%=_T('Roles')%>",
							"Security": "<%=_T('Security')%>",
							"Databases": "<%=_T('Databases')%>"
						};
						const translatedValue = items[row.entity] || row.entity;
						const value = {
							original_value: row.entity,
							value         : translatedValue
						};
						return value;
					}},
					{id: "keyField", name: "<%=_T('Key')%>"},
					{id: "subEntity"  , name: "<%=_T('SubEntity')%>", get_value: function(row) {
						return get_value(row, "subEntity");
					}},
					{id: "operation", class_name:"operation_col", name: "<%=_T('Operation')%>", type: "chip", get_value: function(row) {
						return get_value(row, "operation");
					}},

					{id: "_action" , name: "" , system_action: {edit: false, erase: false, split_screen: {
						name      : "<%=_T('Differences')%>",
						tooltip   : "<%=_T('Show JSON Differences')%>",
						class_name: "differences"
					}}}
				],
				filter: {
					type: {
						search: {
							id: "visual-search",
							conteiner_destination: {type: "header" /*body*/}
						}
					},
					fields: {
					// 1-  los filtros del Visual-Search -------------------
					//
						"user"       : {id: "user"       , label : "<%=_T('User')%>", logical_operator: "OR", type: "VISUAL-SEARCH"},
						"fieldKey"   : {id: "fieldKey"   , label : "<%=_T('Key')%>" ,logical_operator: "OR" , type: "VISUAL-SEARCH",
						transform_name_value: function(item, filter_value) {
							filter_value.fieldKey = item.toLowerCase();
							return filter_value;
						}
					},
						"initialDate": {id: "initialDate", label : "<%=_T('Start Date')%>" + "(" + "<%=_T('yyyy-mm-dd')%>" + ")", logical_operator: "OR", type: "VISUAL-SEARCH"},
						"endDate"    : {id: "endDate"    , label : "<%=_T('End Date')%>" + "(" + "<%=_T('yyyy-mm-dd')%>" + ")"  , logical_operator: "OR", type: "VISUAL-SEARCH"},
						"period": {
							id: "period",
							label: "<%=_T('Period')%>",
							logical_operator: "OR",
							type: "VISUAL-SEARCH",
							values: function () {
								const list = [
									{id: "<%=_T('Now')%>"           },
									{id: "<%=_T('Yesterday')%>"     },
									{id: "<%=_T('The last week')%>" },
									{id: "<%=_T('The last month')%>"}
								];
								return get_vs_filter_values(list);
							},
							transform_name_value: function(item, filter_value) {
									const list = {
										["<%=_T('Now')%>"] : "NOW" ,
										["<%=_T('Yesterday')%>"]: "YESTERDAY",
										["<%=_T('The last week')%>"]: "LAST WEEK",
										["<%=_T('The last month')%>"]: "MONTH",
									};
									let new_item = list[item] || item;
									filter_value.period = new_item;
								return filter_value;
							}
						},
						"entity": {
							id: "entity",
							label: "<%=_T('Entity')%>",
							logical_operator: "OR",
							type: "VISUAL-SEARCH",
							values: function () {
								const list = [
									{ id: "<%=_T('Campaign')%>" },
									{ id: "<%=_T('User')%>"   },
									{ id: "<%=_T('Roles')%>"    },
									{ id: "<%=_T('Security')%>" },
									{ id: "<%=_T('Databases')%>"}
								];
								return get_vs_filter_values(list);
							},
							transform_name_value: function(item, filter_value) {
									const list = {
										["<%=_T('Campaign')%>"] : "Campaign" ,
										["<%=_T('User')%>"]: "User",
										["<%=_T('Roles')%>"]: "Roles",
										["<%=_T('Security')%>"]: "Security",
										["<%=_T('Databases')%>"]: "Databases",
									};
									let new_item = list[item] || item;
									filter_value.entity = new_item;
								return filter_value;
							}
						},
						"subEntity": {
							id: "subEntity",
							label: "<%=_T('SubEntity')%>",
							logical_operator: "OR",
							type: "VISUAL-SEARCH",
							values: function () {
								const list = [
									{ id: "<%=_T('Accounts')%>" },
									{ id: "<%=_T('Users')%>" },
									{ id: "<%=_T('Disposition Codes')%>" },
									{ id: "<%=_T('Permissions')%>" },
									{ id: "<%=_T('Integrations')%>" },
									{ id: "<%=_T('Thresholds')%>" },
									{ id: "<%=_T('Rules')%>" },
									{ id: "<%=_T('Labels')%>" },
									{ id: "<%=_T('Attention Levels')%>" },
									{ id: "<%=_T('Scheduler')%>" },
									{ id: "<%=_T('Templates')%>" },
									{ id: "<%=_T('Notifications')%>" },
									{ id: "<%=_T('Channels')%>" },
									{ id: "<%=_T('Procedures')%>" },
									{ id: "<%=_T('Tables')%>" },
									{ id: "<%=_T('Query Execution')%>" }
								];
								return get_vs_filter_values(list);
							},
							transform_name_value: function(item, filter_value) {
									const list = {
										["<%=_T('Accounts')%>"] : "Accounts" ,
										["<%=_T('Users')%>"]: "Members",
										["<%=_T('Disposition Codes')%>"]: "DispositionCodes",
										["<%=_T('Integrations')%>"]: "Integrations",
										["<%=_T('Thresholds')%>"]: "Thresholds",
										["<%=_T('Rules')%>"]: "Rules",
										["<%=_T('Labels')%>"]: "Labels",
										["<%=_T('Attention Levels')%>"]: "AttentionLevels",
										["<%=_T('Scheduler')%>"]: "Scheduler",
										["<%=_T('Templates')%>"]: "Templates",
										["<%=_T('Notifications')%>"]: "Notifications",
										["<%=_T('Channels')%>"]: "Channels",
										["<%=_T('Procedures')%>"]: "Procedures",
										["<%=_T('Tables')%>"]: "Tables",
										["<%=_T('Query Execution')%>"]: "QueryExecution",
										["<%=_T('Permissions')%>"]: "Permissions",
									};
									let new_item = list[item] || item;
									filter_value.subEntity = new_item;
								return filter_value;
							}
						},
						"operation": {
							id: "operation",
							label: "<%=_T('Operations')%>",
							logical_operator: "OR",
							type: "VISUAL-SEARCH",
							values: function () {
								const list = [
									{ id: "<%=_T('Create')%>" },
									{ id: "<%=_T('Delete')%>" },
									{ id: "<%=_T('Update')%>" },
									{ id: "<%=_T('Import')%>" },
									{ id: "<%=_T('Export')%>" },
									{ id: "<%=_T('Execute')%>" }
								];
								return get_vs_filter_values(list);
							},
							transform_name_value: function(item, filter_value) {
									const list = {
										["<%=_T('Create')%>"] : "Create" ,
										["<%=_T('Delete')%>"]: "Delete",
										["<%=_T('Update')%>"]: "Update",
										["<%=_T('Import')%>"]: "Import",
										["<%=_T('Export')%>"]: "Export",
										["<%=_T('Execute')%>"]: "Execute",
									};
									let new_item = list[item] || item;
									filter_value.operation = new_item;
								return filter_value;
							}
						},

					},
					on_changed_filter : function () {
						const $custom_btns= $("#conf_form_custom_buttons");
						if (!$custom_btns.is(":hidden")) {
							$("#adm_form_audit_back_to_list_btn").click();
						}
					},
					not_duplicate_filters : true,
				},
				callbacks: {
					get_search_err: (err) => {return get_search_err(err);}
				},
				realtime: null
			},
			form: {
				instance: inConcert.adm_form_audit
			},
			on_leave: function() {
				// Si cambia entre app, elimino la clase 'splittedScenarioConfiguration' pq esta trayendo problemas.
				//Si esta la seccion de botones customs (back_to_list_btn), lanzo el evento click para que realize los cambios.
				const $custom_btns= $("#conf_form_custom_buttons");
				if (!$custom_btns.is(":hidden")) {
					$("#adm_form_audit_back_to_list_btn").click();
				}
			}
		};
	};

	//------------------------------------------------------
	// Databases navigation config
	var databases_config = function(base_uri){
		var get_message_error = function(type, err, name)
		{
			let message;
			if (type === "delete") {
				var err_messages = {
					"FORBIDDEN": "<%=_T('Unable to delete {0} database. You dont have permission.')%>"
				};
				if (typeof err === "object") {
					err = err.result || "";
				}
				message = err_messages[err.toUpperCase()] || "<%=_T('Error deleting {0} database.')%>";
				message = format(message, name);
			}
			return message;
		};

		const config_db = inConcert.app.getDetabasesConfig();

		var get_value = function(row) {

			const percent = Math.ceil( parseFloat(row.Sizing) / config_db.Sizing * 100 ).toString();
			const value = format("".concat('<div class="progress databaseSizingContainer {0}">',
											'<div class="progress-bar progress-bar-striped active" role="progressbar" aria-valuenow="0" aria-valuemin="0" aria-valuemax="100" style="width:{1}%;"></div>',
									'</div>'), (percent >= 80)? " runningOutFreeSpace":"", percent);
			const column_class_name = "databaseSizingColumn";
			return {
				value : value,
				column_class_name : column_class_name
			};
		};

		var get_tooltip = function(row) {
			return  format("<%=_T('{0} of {1} consumed megabyte (MB)')%>", row.Sizing, config_db.Sizing.toString());
		};

		var enable_disable_new_button = function(total_rows, view) {
			const config_db = inConcert.app.getDetabasesConfig();
			const btn_create = view.$get_dom_element_by_id("view_button_new");
			if(total_rows < config_db.LimitNumber){
				btn_create.prop("disabled", false);
				btn_create.prop("title", "");
			}
			else{
				btn_create.prop("disabled", true);
				btn_create.prop("title", `<%=_T('Can not create database, exceeds limit')%> (${config_db.LimitNumber}).`);
			}				
		};

		return {
			id: "databases",
			name: "<%=_T('Databases')%>",
			permissions: {
				view    : "viewdatabases",
				save    : "savedatabase",
				erase   : "deletedatabase",
				permission_type: "vcc"
			},
			class_name: "itemDatabases",
			i_class_name: "far fa-database",
			view: {
				title: "<%=_T('Databases')%>",
				tagline: "<%=_T('Tagline for Databases')%>",
				className : "dataBasesTable",
				new_button_name: "<%=_T('Create database')%>",
				row_id: "Id",
				display_name: "Name",
				paginator: {
					paged_text_format : "<%=_T('{page_row_start}-{page_row_end} of {many}')%>",
					hide_container : true
				},
				canEnableNewButton: function(view) {
					const config_db = inConcert.app.getDetabasesConfig();
					return view.count_main_rows() < config_db.LimitNumber;
				},
				actions: {
					erase: {
						callback_after_erase: function(_, view) {
							enable_disable_new_button(view.count_main_rows() - 1, view);
						}
					}
				},
				on_search: function(view, success, rows) {
					if (success && rows) {
						enable_disable_new_button(rows.length, view);
					}
				},
				messages: {
					erase: "<%=_T('Do you want to delete database {0} ?')%>",
					err: get_message_error
				},
				uri: {
					erase:  base_uri + "databases/delete/",
					search: base_uri + "databases/search/"
				},
				source: ["Id", "Name", "Description"],
				columns: [
					{ id: "_number"       , name: "", type: "counter"   },
					{ id: "Name"          , name: "<%=_T('Name')%>" },
					{ id: "Description"   , name: "<%=_T('Description')%>"},
					{ id: "CreationDate"  , name: "<%=_T('Creation Date')%>",
						type: "date",
						date_config: {utc: false, type_date: "date", format_date: "ymd", separator_date: "-"},
					},
					{ id: "Tables"   	  , name: "<%=_T('Tables')%>"},
					{ id: "Procedures"    , name: "<%=_T('Procedures')%>"},
					{ id: "Sizing"    	  , name: "<%=_T('Used/Open Space')%>", get_value: function(row) {
						return get_value(row);
					},
					get_tooltip: function(row){
						return get_tooltip(row);
					},
					render_html_value: true
					},
					{ id: "_action"       , name: "",
						system_action: { edit: true, erase: true }
					}
				]
			},
			form: { instance: inConcert.adm_form_databases
			}
		};
	};
	//------------------------------------------------------
	// tickets navigation config
	var tickets_config = function(base_uri)
	{
		var get_message_error = function(type, err, name)
		{
			var message;
			if (type === "delete") {
				var err_messages = {
					"FORBIDDEN" : "<%=_T('Unable to delete emitter {0}. You dont have permission.')%>",
					"E_EMITTER_IN_USE" : "<%=_T('The ticket emitter {0} is currently in use. You must release it before deletion.')%>"
				};
				if (typeof err === "object") {
					err = err.result || "";
				}
				message = err_messages[err.toUpperCase()] || "<%=_T('Error deleting {0} ticket.')%>";
				message = format(message, name);
			}
			return message;
		};

		var get_value_preview = function (ticket) {
			let value = ticket && ticket.preview && ticket.preview.minimum || "";
			return {
				value : value,
				original_value : value,
				column_class_name : ""
			};
		};

		// this is the object that dashboard.get_current_action_obj manipulates
		return {
			id: "tickets",
			name: "<%=_T('Tickets issuer')%>",
			permissions: {
				view    : "view_tickets",
				save    : "save_tickets",
				erase   : "delete_tickets",
				permission_type: "vcc"
			},
			class_name: "itemTickets",
			i_class_name: "far fa-ticket",
			view: {
				render: "/inconcert/apps/configuration/pages/view_content_default.<%=_L%>.html#grunt-cache-bust",
				title: "<%=_T('Tickets issuer')%>",
				tagline: "<%=_T('Tagline for tickets')%>",
				new_button_name: "<%=_T('Create ticket issuer')%>",
				row_id: "id",
				display_name: "description",
				paginator: {
					paged_text_format : "<%=_T('{page_row_start}-{page_row_end} of {many}')%>"
				},
				messages: {
					erase: "<%=_T('Do you want to delete ticket emitter {0} ?')%>",
					err: get_message_error
				},
				uri: {
					erase:  base_uri + "tickets/delete/",
					search: base_uri + "tickets/search"
				},
				source: ["id", "description", "number_type", "preview"],
				columns: [
					{id: "description", name: "<%=_T('Description')%>"},
					{id: "number_type", name: "<%=_T('Sequence type')%>",  get_value: function(row) {
						const items = {
							"autoincrement": "<%=_T('Autoincrement')%>",
							"date"         : "<%=_T('Date')%>"
						};
						const translatedValue = items[row.number_type] || row.number_type;
						return {
							original_value: row.number_type,
							value         : translatedValue
						};
					}},
					{id: "preview", name: "<%=_T('Sample ticket')%>", type:"chip", get_value:get_value_preview},
					{id: "_action"    , name: "" , system_action: {edit: true, erase: true}}
				],
				filter: {fields: {"Description": {control: null}}},
				realtime: null
			},
			form: {instance: inConcert.adm_form_tickets}
		};
	};

	//------------------------------------------------------
	// channels navigation config
	var channels_config = function()
	{
		return {
			id: "channels",
			name: "<%=_T('Attention Capability')%>",
			permissions: {
				view    : "view_attention_capability",
				save    : "save_attention_capability",
				permission_type: "vcc"
			},
			class_name: "itemChannels",
			i_class_name: "far fa-hand-holding-magic",
			only_frm: true,
			view: {
				header:{
					hide_new_button: true
				},
				render: "/inconcert/apps/configuration/pages/view_content_default.<%=_L%>.html#grunt-cache-bust"
			},
			form: {
				instance: inConcert.adm_form_channels,
				cancel_button: {
				//	caption       : "<%=_T('Restore')%>",
					initial_state : "enabled"
				}
			}
		};
	};

	//------------------------------------------------------
	// accounts navigation config
	var accounts_config = function(base_uri)
	{
		var get_message_error = function(type, err, name)
		{
			var message;
			if (type === "delete") {
				var err_messages = {
					"ACCOUNT_DOES_NOT_EXIST" : "<%=_T('Unable to delete {0} outgoing account. Account does not exist.')%>",
					"ACCOUNT_BEING_USED"     : "<%=_T('The outgoing account {0} to delete is currently in use. You must release it before deletion.')%>",
					"FORBIDDEN"              : "<%=_T('Unable to delete {0} outgoing account. You dont have permission.')%>"
				};
				if (typeof err === "object") {
					err = err.result || "";
				}
				message = err_messages[err.toUpperCase()] || "<%=_T('Error deleting {0} outgoing account.')%>";
				message = format(message, name);
			}
			return message;
		};

		return {
			id: "accounts",
			name: "<%=_T('Outgoing E-mail Accounts')%>",
			permissions: {
				view    : "view_vcc_accounts",
				save    : "save_vcc_accounts",
				erase   : "delete_vcc_accounts",
				permission_type: "vcc"
			},
			class_name: "itemOutgoingAccounts",
			i_class_name: "far fa-envelope-open",
			view: {
				render: "/inconcert/apps/configuration/pages/view_content_default.<%=_L%>.html#grunt-cache-bust",
				title: "<%=_T('Outgoing E-mail Accounts')%>",
				tagline: "<%=_T('Tagline for Outgoing Accounts')%>",
				new_button_name: "<%=_T('Create output account')%>",
				row_id: "uid",
				display_name: "uid",
				paginator: {
					paged_text_format : "<%=_T('{page_row_start}-{page_row_end} of {many}')%>"
				},
				messages: {
					erase: "<%=_T('Do you want to delete outgoing account {0} ?')%>",
					err: get_message_error
				},
				uri: {
					erase:  base_uri + "vccs/accounts/delete/",
					search: base_uri + "vccs/accounts/search/"
				},
				source: [ "uid", "defaultName" ],
				columns: [
					{ id: "_number"      , name: "", type: "counter"                            },
					{ id: "uid"          , name: "<%=_T('Account')%>"                           },
					{ id: "defaultName"  , name: "<%=_T('Display Name')%>"                      },
					{ id: "_action"      , name: "" , system_action: { edit: true, erase: true }}
				],
				filter: { 
					fields : {
						"uid"         : { control: null },
						"defaultName" : { control: null } 
					},
					special_values : function() {
						return { accountType: "MAIL" };
					}
				},
				erase_values: function( rows ) {
					return {
						rows        : rows,
						accountType : "MAIL"
					};
				},
				realtime: null
			},
			form: { instance: inConcert.adm_form_vcc_accounts }
		};
	};

	//------------------------------------------------------
	// spell checker navigation config
	var spell_checker_config = function(base_uri)
	{
		return {
			id: "spell_checker",
			name: "<%=_T('Spell Checker')%>",
			permissions: {
				view    : "view_vcc_spell_checker",
				save    : "save_vcc_spell_checker",
				erase   : "delete_vcc_spell_checker",
				permission_type: "vcc"
			},
			class_name: "itemSpellChecker",
			i_class_name: "far fa-spell-check",
			view: {
				render: "/inconcert/apps/configuration/pages/view_content_default.<%=_L%>.html#grunt-cache-bust",
				title: "<%=_T('Spell Checker')%>",
				paginator: {
					paged_text_format : "<%=_T('{page_row_start}-{page_row_end} of {many}')%>"
				},
				header: {
					hide_new_button   : true,
					hide_search_ctrls : true
				},
				row_id: "id",
				display_name: "name",
				uri: {
					search: base_uri + "spell_checker/search/"
				},
				source: [ "id", "name", "description", "total_words" ],
				columns: [
					{ id: "_number"      , name: "", type: "counter"                             },
					{ id: "name"         , name: "<%=_T('Name')%>"                               },
					{ id: "description"  , name: "<%=_T('Description')%>"                        },
					{ id: "total_words"  , name: "<%=_T('Words')%>"                              },
					{ id: "_action"      , name: "" , system_action: { edit: true, erase: false }}
				],
				realtime: null
			},
			form: { instance: inConcert.adm_form_spell_checker }
		};
	};

	//------------------------------------------------------
	// signatures navigation config
	var signatures_config = function(base_uri)
	{
		var get_message_error = function(type, err, name)
		{
			var message;
			if (type === "delete") {
				var err_messages = {
					"SIGNATURE_BEING_USED_FOR_ACCOUNT" : function() {
						return format(
							"<%=_T('Unable to delete {0} signature. Signature to delete is currently in use for {1} account, campaign {2}. You must release it before deletion.')%>",
							name,
							err.account,
							err.campaign_id
						);
					},
					"FORBIDDEN": function() {
						return format( "<%=_T('Unable to delete {0} signature. You dont have permission.')%>", name );
					}
				};

				var err_id;
				if (typeof err === "object") {
					err_id = err.result || "";
				}
				else {
					err_id = err;
				}
				err_id = err_id.toUpperCase();

				if ( err_messages[err_id] ) {
					message = err_messages[err_id]();
				}
				else {
					message = format( "<%=_T('Error deleting {0} signature.')%>", name );
				}
			}
			return message;
		};

		var erase_previous_validation = function(signature_id, callback)
		{
			callback = callback || $.noop;

			inConcert.invoke({
				url: base_uri + "signatures/delete_validation/" + signature_id,
				type: "GET",
				dataType: "json",
				contentType: "application/json",
				success: function(response)
				{
					var message;
					if (response && response.warning) {
						if (response.warning.info === "SIGNATURE_BEING_USED_ON_ACCOUNTS") {
							message = format("<%=_T('Warning, {0} signature is being used for {1} accounts. Also want to delete ?')%>", 
									signature_id,
									response.warning.count_accounts
								);
						}
					}
					callback(true, message);
				},
				error: function(_, error) {
					var message;
					if (error.result === "NOT_FOUND") {
						message = "<%=_T('Unable to delete {0} signature. This state does not exist.')%>";
					}
					else {
						message = "<%=_T('Unable to delete {0} signature.')%>";
					}
					callback(false, message);
				}
			});
		};

		// this is the object that dashboard.get_current_action_obj manipulates
		return {
			id: "signatures",
			name: "<%=_T('Signatures')%>",
			permissions: {
				view    : "view_signatures",
				save    : "save_signatures",
				erase   : "delete_signatures",
				permission_type: "vcc"
			},
			class_name: "itemSignatures",
			i_class_name: "far fa-signature",
			view: {
				render: "/inconcert/apps/configuration/pages/view_content_default.<%=_L%>.html#grunt-cache-bust",
				title: "<%=_T('Signatures')%>",
				tagline: "<%=_T('Tagline for signatures')%>",
				new_button_name: "<%=_T('Create signature')%>",
				row_id: "Id",
				display_name: "Name",
				paginator: {
					paged_text_format : "<%=_T('{page_row_start}-{page_row_end} of {many}')%>"
				},
				messages: {
					erase: "<%=_T('Do you want to delete signature {0} ?')%>",
					err: get_message_error
				},
				uri: {
					erase:  base_uri + "signatures/delete/",
					search: base_uri + "signatures/search/"
				},
				erase_previous_validation: erase_previous_validation,
				source: ["Id", "Name", "Description"],
				columns: [
					{id: "_number"      , name: "", type: "counter"                           },
					{id: "Name"         , name: "<%=_T('Name')%>"                             },
					{id: "Description"  , name: "<%=_T('Description')%>"                      },
					{id: "_action"      , name: "" , system_action: {edit: true, erase: true} }
				],
				filter: {fields: {"Name": {control: null}, "Description": {control: null}}},
				realtime: null
			},
			form: {instance: inConcert.adm_form_signatures}
		};
	};

	//------------------------------------------------------
	// ttsasr_engine navigation config

	var ttsasr_engine_config = function(base_uri)
	{
		return {
			id: "ttsasr_engine",
			name: "<%=_T('TTS/ASR Engines')%>",
			permissions: {
				view    : "view_ttsasr_engine",
				save    : "save_ttsasr_engine",
				permission_type: "vcc"
			},
			class_name: "itemTtsAsrEngines",
			i_class_name: "far fa-sensor-on",
			view: {
				render: "/inconcert/apps/configuration/pages/view_content_default.<%=_L%>.html#grunt-cache-bust",
				title: "<%=_T('TTS/ASR Engines')%>",
				paginator: {
					paged_text_format : "<%=_T('{page_row_start}-{page_row_end} of {many}')%>"
				},
				header: {
					hide_new_button   : true,
					hide_search_ctrls : true
				},
				row_id: "Id",
				display_name: "engineDesc",
				uri: {
					search: base_uri + "ttsasr_engines/search/"
				},
				source: ["Id", "engineId", "engineDesc", "profileId", "profileDesc", "providerId", "providerDesc", "providerType", "endpoint", "voices", "grammars"],
				columns: [
					{id: "_number"     , name: "", type: "counter"        },
					{id: "engineDesc"  , name: "<%=_T('Engine')%>"        },
					{id: "profileDesc" , name: "<%=_T('Profile')%>"       },
					{id: "providerDesc", name: "<%=_T('Provider')%>"      },
					{id: "providerType", name: "<%=_T('Type')%>"          },
					{id: "_action"     , name: "" , system_action: {edit: true, erase: false}}
				],
				realtime: null
			},
			form: {instance: inConcert.adm_form_ttsasr_engines}
		};
	};

	//------------------------------------------------------
	// Routing Groups navigation config
	var routing_groups_config = function(base_uri)
	{
		var get_message_error = function(type, err, name)
		{
			let message;
			if (type === "delete") {
				var err_messages = {
					"ALLEGRO_INTEGRATION_ERROR": "<%=_T('Unable to delete {0} routing group. Allegro integration error.')%>",
					"FORBIDDEN"                : "<%=_T('Unable to delete {0} routing group. You dont have permission.')%>",
					"-2"                       : "<%=_T('Unable to delete {0} routing group. The routing group you tried to delete is currently in use.')%>"
				};
				if (typeof err === "object") {
					if (typeof err.err === "object") {
						err = (typeof err.err.result === "object") ? err.err.result.result : err.err.result;
						err = (err || "").toString();
					}
					else {
						err = err.type || "";
					}
				}
				message = err_messages[err.toUpperCase()] || "<%=_T('Error deleting {0} routing group.')%>";
				message = format(message, name);
			}
			return message;
		};

		return {
			id: "routing groups",
			name: "<%=_T('Routing Groups')%>",
			can_show: function()
			{
				var has_allegro_integration = inConcert.app.has_allegro_integration();
				if (has_allegro_integration) {
					return true;
				}
				return false;
			},
			permissions: {
				view    : "viewroutinggroups",
				save    : "saveroutinggroups",
				erase   : "deleteroutinggroups",
				permission_type: "vcc"
			},
			class_name: "itemRoutingGroups",
			i_class_name: "far fa-flux-capacitor",
			view: {
				header: {
					hide_new_button: false,
					hide_total_rows: true
				},
				paginator: {
					paged_text_format : "<%=_T('{page_row_start}-{page_row_end} of {many}')%>"
				},
				render: "/inconcert/apps/configuration/pages/view_content_default.<%=_L%>.html#grunt-cache-bust",
				title: "<%=_T('Routing Groups')%>",
				tagline: "<%=_T('Tagline for Routing Groups')%>",
				new_button_name: "<%=_T('New Routing Group')%>",
				row_id: "Id",
				display_name: "Id",
				messages: {
					erase: "<%=_T('Do you want to delete routing group {0} ?')%>",
					err: get_message_error
				},
				uri: {
					erase:  base_uri + "routing_groups/delete/",
					search: base_uri + "routing_groups/search/"
				},
				source: ["Id", "DisplayCapacity"],
				columns: [
					{id: "_number"        , name: "", type: "counter"  },
					{id: "Id"             , name: "<%=_T('Name')%>"    },
					{id: "DisplayCapacity", name: "<%=_T('Capacity')%>"},
					{id: "_action"        , name: "" , system_action: {edit: true, erase: true}}
				],
				filter: {
					fields: {
						"Id"                               : {control: null},
						"OutgoingRoutingGroupTrunk.TrunkId": {control: null}
					}
				},
				realtime: null,
				// subgrilla definicion
				sub_grid: {
					data_source: "OutgoingRoutingGroupTrunks", // aca vienen los registros de la subgrilla
					colapsed: true,
					messages: function( type ) {
						var message;
						if ( type === "no_records_found" ) {
							message = "<%=_T('No rows found.')%>";
						}
						return message;
					},
					columns: [
						// las columns del colspan
						{id: "_number", name: "", type: "counter"},
						{
							id       : "TrunkId",
							name     : "<%=_T('Trunk Name')%>",
							get_value: function( row ) { return get_subgrid_value(row, "TrunkId"); }
						},
						{
							id       : "DisplayTechnology",
							name     : "<%=_T('Type')%>",
							get_value: function( row ) { return get_subgrid_value(row, "DisplayTechnology"); }
						},
						{
							id       : "DisplayCapacity",
							name     : "<%=_T('Assigned Capacity')%>",
							get_value: function( row ) { return get_subgrid_value(row, "DisplayCapacity"); }
						}
					]
				}
			},
			form: {instance: inConcert.adm_form_routing_groups}
		};
	};

	//------------------------------------------------------
	// api_users navigation config

	var canShowTokensKey = function() {
		return inConcert.permissions.to_action("administration", "api_users_key");
	};

	var showTokensKey = function(view) {
		if (!canShowTokensKey()) {
			var error_message = "<%=_T('Unable to show tokens key: You dont have permission.')%>";
			view.show_status_message( error_message, "Error", 5000 );
			return;
		}
		view.render_popup({ type : "api_users_key" });
	};

	var canShowIpFilters = function() {
		return inConcert.permissions.to_action("administration", "api_ip_filters");
	};

	var showIpFilters = function(view) {
		if (!canShowIpFilters()) {
			var error_message = "<%=_T('Unable to show IP filters: You dont have permission.')%>";
			view.show_status_message( error_message, "Error", 5000 );
			return;
		}
		view.render_popup({ type : "api_ip_filters" });
	};

	var api_users_config = function(base_uri)
	{
		var get_message_error = function(type, err, name)
		{
			var message;
			if (type === "delete") {
				var err_messages = {
					"FORBIDDEN"                     : "<%=_T('Unable to delete {0} user. You dont have permission.')%>"
				};
				if (typeof err === "object") {
					err = err.result || "";
				}
				message = err_messages[err.toUpperCase()] || "<%=_T('Error deleting {0} user.')%>";
				message = format(message, name);
			}
			return message;
		};

		return {
			id: "api_users",
			name: "<%=_T('API Users')%>",
			permissions: {
				view    : "view_api_users",
				save    : "save_api_users",
				erase   : "delete_api_users",
				permission_type: "vcc"
			},
			class_name: "itemApiUsers",
			i_class_name: "far fa-puzzle-piece",
			view: {
				render: "/inconcert/apps/configuration/pages/view_content_default.<%=_L%>.html#grunt-cache-bust",
				title: "<%=_T('API Users')%>",
				tagline: "<%=_T('Tagline for API Users')%>",
				new_button_name: "<%=_T('Create API user')%>",
				row_id: "Id",
				display_name: "Id",
				paginator: {
					paged_text_format : "<%=_T('{page_row_start}-{page_row_end} of {many}')%>"
				},
				messages: {
					erase: "<%=_T('Do you want to delete user {0} ?')%>",
					err: get_message_error
				},
				uri: {
					erase:  base_uri + "api_users/delete/",
					search: base_uri + "api_users/search/"
				},
				source: ["Id", "Type"],
				columns: [
					{id: "_number"    , name: "", type: "counter" },
					{id: "Id"		  , name: "<%=_T('Id')%>"     },
					{id: "Type"		  , name: "<%=_T('Type')%>"   },
					{id: "_action"    , name: "" , system_action: {edit: true, erase: true}}
				],
				filter: { fields: { "Id": {control: null} } },
				realtime: null,
				global_buttons : [
					{
						id : "api_users_key",
						name : "<%=_T('Tokens Key')%>",
						className : "buttonColorWhite pageButtonNoIcon",
						enabled : canShowTokensKey,
						run : function(view) {
							showTokensKey(view);
						},
						before: false
					},
					{
						id : "api_ip_filters",
						name : "<%=_T('IP Filters')%>",
						className : "buttonColorWhite pageButtonNoIcon",
						enabled : canShowIpFilters,
						run : function(view) {
							showIpFilters(view);
						},
						before: false
					}
				]
			},
			form: {instance: inConcert.adm_form_api_users}
		};
	};

	//------------------------------------------------------
	// shared_files navigation config

	var shared_files_config = function()
	{
		var get_message_error = function(type, err, name)
		{
			var message;
			if (type === "delete") {
				let err_messages = {
					"FORBIDDEN": function() {
						return format("<%=_T('Unable to delete {0} file. You dont have permission.')%>", name);
					},
					"SHARED FILES TO DELETE DO NOT EXIST IN DATABASE": function() {
						return format("<%=_T('Unable to delete {0} file. The file to delete does not exist in the database.')%>", name);
					},
					"NO FILE COULD BE DELETED": function() {
						let error = (err.errors || [])[0];
						if (error) {
							if (error.errType === "FILE USED IN CAMPAIGN ACCOUNTS") {
								if (error.totalUsedInAccountMessages === 1) {
									let accountMsgId = Object.keys(error.errInfo)[0];
									let campErr = error.errInfo[accountMsgId];
									let campaignId = campErr.campaignId || "";
									campaignId = inConcert.app.get_id(campaignId);
									return format("<%=_T('Unable to delete {0} file. File is in use in the {1} campaign.')%>", name, campaignId);
								}
								return format("<%=_T('Unable to delete {0} file. File is in use in {1} campaigns.')%>", name, error.totalUsedInAccountMessages);
							}
							else if (error.errType === "FILE USED IN FLOW DESIGNER") {
								if (error.totalUsedInFlowDesigner === 1) {
									let flowId = Object.keys(error.errInfo)[0];
									let campErr = error.errInfo[flowId];
									let flowName = campErr.flowName || campErr.flowId || "";
									return format("<%=_T('Unable to delete {0} file. File is in use in the {1} flow process.')%>", name, flowName);
								}
								return format("<%=_T('Unable to delete {0} file. File is in use in {1} flow processes.')%>", name, error.totalUsedInFlowDesigner);
							}
						}
						return format("<%=_T('Unable to delete {0} file. File is in use.')%>", name);
					}
				};

				var err_id;
				if (typeof err === "object") {
					err_id = err.result || err.code || "";
				}
				else {
					err_id = err;
				}
				err_id = err_id.toUpperCase();

				if (err_messages[err_id]) {
					message = err_messages[err_id]();
				}
				else {
					message = format( "<%=_T('Error deleting {0} file.')%>", name );
				}
			}
			return message;
		};

		const filesAPI = inConcert.app.getSharedFilesAPI();

		// execute custom action
		var execute_custom_action = function(action_id, $row, view)
		{
			var file_id = $row.attr( "row_id" );
			var display_name = $row.attr("row_name");
			switch (action_id) {
				case "download":
					filesAPI.downloadFile(file_id);
					break;
				case "export":
					if (!view.form.has_permission_to_save()) {
						view.show_status_message(
							format( "<%=_T('Unable to export {0} file. You dont have permission.')%>", display_name),
							"Error",
							5000
						);
						return;
					}
					filesAPI.startExportFilesToCompressed([file_id], function(success, err) {
						let message, messageType;
						if (success) {
							message = format("<%=_T('File {0} successfully exported.')%>", display_name);
							messageType = "Info";
						}
						else {
							message = err.message || format("<%=_T('Unable to export file {0}.')%>", display_name);
							messageType = "Error";
						}
						view.show_status_message(
							message,
							messageType,
							5000
						);
					});
					break;
			}
		};

		const get_vs_filter_values = function(rows, attrId) {
			let values = [];
			for (let i = 0; i < (rows || []).length; i++) {
				let row = rows[i];
				let rowId;
				if (typeof attrId === "function") {
					rowId = attrId(row);
				}
				else {
					attrId = attrId || "id";
					rowId = row[attrId];
				}
				if (rowId) {
					values.push(rowId);
				}
				else {
					throw("row.id is required");
				}
			}
			return values;
		};

		const get_language = function(row)
		{
			let value = {
				original_value: row.Language,
				value         : row.Language
			};
			let languages = inConcert.shared_files_languages;
			for (let i = 0; i < languages.length; i++) {
				if (languages[i].id === row.Language) {
					value.value = languages[i].label;
					break;
				}
			}
			value.value = `<span class="sharedFileTitleLi"><%=_T("Language")%></span><span class="sharedFileDataLi">${value.value}</span>`;
			return value;
		};

		const get_labels = function(row)
		{
			let labels = "", html_labels = "";
			const template_label_html = '<span class="sharedFileDataLi">{0}</span>';
			let anyVisible = false;

			if (parseInt(row.AgentVisible, 10) === 1) {
				let agent_label = "<%=_T('Agent')%>";
				labels = agent_label;
				html_labels = format(template_label_html, agent_label);
				anyVisible = true;
			}
			if (parseInt(row.AccountMessagesVisible, 10) === 1) {
				if (labels.length > 0) {
					labels = labels + " ";
				}
				let account_msg_label = "<%=_T('Account Messages')%>";
				labels = labels + account_msg_label;
				html_labels = html_labels + format(template_label_html, account_msg_label);
				anyVisible = true;
			}
			if (parseInt(row.FlowDesignerVisible, 10) === 1) {
				if (labels.length > 0) {
					labels = labels + " ";
				}
				let flow_designer_label = "<%=_T('Process Flows')%>";
				labels = labels + flow_designer_label;
				html_labels = html_labels + format(template_label_html, flow_designer_label);
				anyVisible = true;
			}

			let value = {
				original_value: labels,
				value         : anyVisible? '<span class="sharedFileTitleLi"><%=_T("Visibility")%></span>' + html_labels : html_labels
			};
			return value;
		};

		const get_usage = function(row)
		{
			const format_name = function(name) {
				return format("<span>{0}</span>", name);
			};
			const format_value = function(val) {
				return format("<strong>{0}</strong>", val);
			};
			const format_name_value = function(name, val) {
				return format_name(name) + format_value(val);
			};
			const format_row_value = function(row) {
				return format("<label>{0}</label>", row);
			};

			const max_char_per_items = 300;
			const never_used = "<%=_T('Not used')%>";
			const conversations_label = "<%=_T('Agent')%>" + "<%=_T(':')%>";

			let tooltip = "", html_value = "";
			let widthData = false;
			if (row.LastUsageInInteractions) {
				// usado en interacciones
				let time_ms = row.LastUsageInInteractions * 1000;
				time_ms = inConcert.Utils.formatDate(time_ms, "date-short-year-short-hour");
				tooltip = conversations_label +  time_ms;
				html_value = format_row_value(format_name_value(conversations_label, time_ms));
				widthData = true;
			}
			else {
				// sin uso en conversaciones
				tooltip = conversations_label + never_used;
				html_value = format_row_value(format_name_value(conversations_label, never_used));
			}

			const campaigns_label = "<%=_T('Campaigns')%>" + "<%=_T(':')%>";
			const flows_labels = "<%=_T('Process Flows')%>" + "<%=_T(':')%>";
			if (!row.Relations) {
				// sin uso en campanas
				tooltip = tooltip + "\n" + campaigns_label + never_used;
				html_value = html_value + format_row_value(format_name_value(campaigns_label, never_used));
				// sin uso en flows
				tooltip = tooltip + "\n" + flows_labels + never_used;
				html_value = html_value + format_row_value(format_name_value(flows_labels, never_used));
			}
			else {
				// relacion con las campanas (usado en campaigns accounts)
				let totalCampaigns = 0;
				let campaignAccountMessages = row.Relations.CampaignAccountMessages || {};
				let html_campaigns = "";
				let value_campaigns = "";
				let hidden_campaigns = [];

				for (let accountMsgId in campaignAccountMessages) {
					if (campaignAccountMessages.hasOwnProperty(accountMsgId)) {
						totalCampaigns++;
						if (totalCampaigns === 1) {
							value_campaigns = value_campaigns + campaigns_label;
						}
						if (totalCampaigns > 1) {
							value_campaigns = value_campaigns + ", ";
						}
						let info = campaignAccountMessages[accountMsgId];
						let campiagnWithOutId = inConcert.app.get_id(info.campaignId);
						value_campaigns = value_campaigns + campiagnWithOutId;
						// Valida que el valor de la celda para las campanas NO supere el maximo permitido.
						if (value_campaigns.length <= max_char_per_items) {
							if (!html_campaigns) {
								html_campaigns = format_name(campaigns_label);
							}
							html_campaigns = html_campaigns + format_value(campiagnWithOutId);
						}
						else {
							// las campanas que queda ocultas
							hidden_campaigns.push(campiagnWithOutId);
						}
					}
				}
				if (value_campaigns) {
					tooltip = tooltip + "\n" + value_campaigns;
				}
				if (hidden_campaigns.length > 0) {
					// si hay campaigns ocutlas, visualizo el mensaje de que existen N campanas ocultas.
					let more = format("<%=_T('and in {0} more.')%>", hidden_campaigns.length);
					html_campaigns = html_campaigns + format_value(more);
				}
				if (html_campaigns) {
					html_value = html_value + format_row_value(html_campaigns);
					widthData = true;
				}
				else {
					// Sin uso en campanas
					tooltip = tooltip + "\n" + campaigns_label + never_used;
					html_value = html_value + format_row_value(format_name_value(campaigns_label, never_used));
				}

				// relacion con los flujos (usado en el flow designer)
				let totalFlowDesigner = 0;
				let flowDesigner = row.Relations.FlowDesigner || {};
				let html_flows = "";
				let value_flows = "";
				let hidden_flows = [];

				for (let flowId in flowDesigner) {
					if (flowDesigner.hasOwnProperty(flowId)) {
						totalFlowDesigner++;
						if (totalFlowDesigner === 1) {
							value_flows = value_flows + flows_labels;
						}
						if (totalFlowDesigner > 1) {
							value_flows = value_flows + ", ";
						}
						let flow_name;
						if (m_map_flow_processes) {
							let info = m_map_flow_processes[flowId.toLowerCase()];
							flow_name = info && info.Name ? info.Name : flowId;
						}
						let flowDesc = (flow_name || flowId);
						value_flows = value_flows + flowDesc;
						// Valida que el valor de la celda para los Flows NO supere el maximo permitido.
						if (value_flows.length <= max_char_per_items) {
							if (!html_flows) {
								html_flows = format_name(flows_labels);
							}
							html_flows = html_flows + format_value(flowDesc);
						}
						else {
							// los flows que queda oculoas
							hidden_flows.push(flowDesc);
						}
					}
				}
				if (value_flows) {
					tooltip = tooltip + "\n" + value_flows;
				}
				if (hidden_flows.length > 0) {
					// si hay flows ocutlos, visualizo el mensaje de que existen N flows ocultos.
					let more = format("<%=_T('and in {0} more.')%>", hidden_flows.length);
					html_flows = html_flows + format_value(more);
				}
				if (html_flows) {
					html_value = html_value + format_row_value(html_flows);
					widthData = true;
				}
				else {
					// sin uso en flows
					tooltip = tooltip + "\n" + flows_labels + never_used;
					html_value = html_value + format_row_value(format_name_value(flows_labels, never_used));
				}
			}

			let cell_value = {
				original_value: tooltip,
				value         : html_value,
				column_class_name : widthData? "widthData":"widthNoData"
			};

			return cell_value;
		};

		// is action enabled
		const is_action_enabled = function(action, row) {
			/*
			 * Valores a retornar:
			 *    1  = La accion se debe visualizar y estar Enabled.
			 *    0  = La accion se debe visualizar y estar Disabled.
			 *    -1 = La accion no se genera en el markup. (No aparece el boton de la accion).
			 */
			if (action.toLowerCase() === "play") {
				if ((row.Type || "").toLowerCase() !== "audio") {
					return -1;
				}
			}
			return 1;
		};

		// this is the object that dashboard.get_current_action_obj manipulates
		return {
			id: "shared_files",
			name: "<%=_T('Shared Files')%>",
			permissions: {
				view           : "view_shared_files",
				save           : "save_shared_files",
				erase          : "delete_shared_files",
				"export"       : "export_shared_files",
				permission_type: "vcc"
			},
			class_name: "itemSharedFiles",
			i_class_name: "far fa-cloud-upload-alt",
			view: {
				render: "/inconcert/apps/configuration/pages/view_content_default.<%=_L%>.html#grunt-cache-bust",
				title: "<%=_T('Shared Files')%>",
				tagline: "<%=_T('Tagline for Shared Files')%>",
				className : "sharedFilesView",
				new_button_name: "<%=_T('Upload File')%>",
				new_action: {
					with_contextual_menu: true
				},
				markup_with_divs: true, // Con este flag la grilla se crea con DIV+UL+LI,etc en lugar de ser creada con TABLE+THEAD+TBODY,etc
				row_id: "Id",
				display_name: "Name",
				header_actions: {
					system_actions: {
						batch_actions: {caption: null},
						items_mode   : "gallery",
						on_set_items_mode: function (mode, view) {
							const $view_grid_header_title =  view.$get_dom_element_by_id("view_grid_header_title");
							let title = mode ==="gallery" ?"<%=_T('File Gallery')%>": "<%=_T('File List')%>";
							$view_grid_header_title.html(title);
						}
					}
				},
				paginator: {
					show_at_least : 47,
					rows_per_page : 50,
					paged_text_format : "<%=_T('{page_row_start}-{page_row_end} of {many}')%>"
				},
				messages: {
					total_selected_rows: "<%=_T('Total selected files: {0}.')%>",
					no_selected_rows   : "<%=_T('You currently have no selected files.')%>",
					erase              : "<%=_T('Do you want to delete file {0} ?')%>",
					err                : get_message_error
				},
				uri: {
					erase: function(file_id, _, callback) {
						let files = [];
						files.push(file_id);
						return filesAPI.deleteFiles(files, function(status, result) {
							inConcert.shared_files_maintenance_mng.prototype.reloadSizeLimit(function() {
								callback(status, result);
							});
						});
					},
					search: function(callback, filter, view) {
						const $view_grid_header_title =  view.$get_dom_element_by_id("view_grid_header_title");
						$view_grid_header_title.parent().show();
						return filesAPI.searchFiles(filter,
								function(status, result) {
								inConcert.shared_files_maintenance_mng.prototype.loadSizeLimit(status, result, $view_grid_header_title);
								callback(status, result);
							}
						);
					}
				},
				source: ["Id", "Type", "Name", "Description", "LastUsageInInteractions", "RepositoryId", "AgentVisible", "AccountMessagesVisible", "FlowDesignerVisible", "Language", "Relations", "Section", "Format"],
				columns: [
					{id: "_number"    , name: "", type: "counter"},
					{id: "_select", name: "", type: "row_selector",
						class_name : "sharedFileSelectorColumn",
						cell_class_name: "sharedFileSelectorColumn",
						is_checked: function(/*row*/) {
							return false;
						},
						on_selected_row: function($row, is_selected) {
							$row.removeClass("selectedRow").addClass(is_selected? "selectedRow":"");
						}
					},
					{id: "Type"       , name: "", type: "image",
						class_name : "sharedFileIconColumn",
						cell_class_name: "sharedFileIconColumn",
						get_value: function(file) {
							let image_url = inConcert.shared_files_maintenance_mng.prototype.getImageUrl(file);
							return { 
								class_name       : "fileType" + file.Type,
								image_url        : image_url
							};
						}
					},
					{id: "Name"                   , name: "<%=_T('Name')%>"       , class_name: "sharedFileNameColumn"        , cell_class_name: "sharedFileNameColumn"                                 },
					{id: "Description"            , name: "<%=_T('Description')%>", class_name: "sharedFileDescriptionColumn" , cell_class_name: "sharedFileDescriptionColumn"                          },
					{id: "Language"               , name: "<%=_T('Language')%>"   , class_name: "sharedFileLanguageColumn"    , cell_class_name: "sharedFileLanguageColumn"    , get_value: get_language, render_html_value : true},
					{id: "_Labels"                , name: "<%=_T('Visibility')%>" , class_name: "sharedFileAgentVisibleColumn", cell_class_name: "sharedFileAgentVisibleColumn", get_value: get_labels  ,
						render_html_value : true
					},
					{id: "LastUsageInInteractions", name: "<%=_T('Used')%>"       , class_name: "sharedFileLastUseColumn"     , cell_class_name: "sharedFileLastUseColumn"     , get_value: get_usage   ,
						render_html_value : true
					},
					{ id: "_action"    , name: "" ,
						class_name: "sharedFileActionsColumn",
						cell_class_name: "sharedFileActionsColumn",
						system_action: {
							edit: {
								tooltip: "<%=_T('Edit')%>"
							},
							erase: {
								tooltip: "<%=_T('Delete')%>"
							}
						},
						custom_action: {
							play: {
								tooltip: "<%=_T('Play')%>",
								execute: function(_, $row, view, $button) {
									let row_data = $row.data("row_data");
									let file_info = {
										id     : row_data.RepositoryId,
										section: row_data.Section || "-1",
										format : row_data.Format || "wav"
									};
									$button.data("file_info", file_info);
									view.dashboard.player_start($button);
								},
								class_name: "start"
							},
							download: {
								tooltip: "<%=_T('Download')%>",
								execute: execute_custom_action,
								class_name: "download"
							},
							export: {
								tooltip: "<%=_T('Export')%>",
								execute: execute_custom_action,
								class_name: "export"
							}
						},
						is_enabled: is_action_enabled
					}
				],
				filter: {
					type: {
						search: {
							id: "visual-search",
							conteiner_destination: {type: "header" /*body*/}
						},
						tree_filter: {
							enabled      : true,
							title        : "<%=_T('Files')%>",
							initial_state: {colapsed: true, selected_all: true},
							get_tree     : function() {
								let languages =inConcert.shared_files_languages.reduce(function(options, option) {
									options.push({value : option.id, label : option.label, entity_id: "Language"});
									return options;
								}, []);
								let tree = {
									id   : "root",
									label: "<%=_T('Total Files')%>",
									childs: languages
								};
								tree.childs.forEach(item => {
									let visibilty =inConcert.shared_files_visibilty.reduce(function(options, option) {
										options.push({value : option.id, label : option.label, entity_id: "Visibility"});
										return options;
									}, []);
									item.childs = visibilty;
								});
								return tree;
							}
						}
					},
					fields: {
					// 1-  los filtros del Visual-Search -------------------
					//
						"Name"       : {id: "Name", label: "<%=_T('Name')%>", default: true, logical_operator: "OR", type: "VISUAL-SEARCH"},
						"Description": {id: "Description", label : "<%=_T('Description')%>", logical_operator: "OR", type: "VISUAL-SEARCH"},
						"Type": {
							id: "Type",
							label: "<%=_T('File Type')%>",
							logical_operator: "OR",
							type: "VISUAL-SEARCH",
							values: function () {
								return get_vs_filter_values(inConcert.shared_files_types);
							}
						},
						"CampaignId": {
							id: "CampaignId",
							label: "<%=_T('Campaign')%>",
							logical_operator: "AND",
							type: "VISUAL-SEARCH",
							values: function () {
								let campaigns = inConcert.app.getCampaigns();
								return get_vs_filter_values(campaigns, function(row) {
									return inConcert.app.get_id(row.Id);
								});
							}
						},
						"FlowName": {
							id: "FlowName",
							label: "<%=_T('Process Flows')%>",
							logical_operator: "AND",
							type: "VISUAL-SEARCH",
							values: function () {
								let rows = m_flow_processes && m_flow_processes.rows ? m_flow_processes.rows : [];
								return get_vs_filter_values(rows, "Name");
							}
						},
					// 2-  los filtros del Tree -------------------
					//
						"Language": {id: "Language", label : "<%=_T('Language')%>", logical_operator: "OR", type: "TREE",
							values: function () {
								return get_vs_filter_values(inConcert.shared_files_languages);
							}
						},
						"Visibility": {id: "Visibility", label : "<%=_T('Visibility')%>", logical_operator: "OR", type: "TREE",
							values: function () {
								let visibilty =inConcert.shared_files_visibilty.reduce(function(options, option) {
									options.push({id : option.id, label : option.label});
									return options;
								}, []);
								return get_vs_filter_values(visibilty);
							},
							can_include_filter_value: function(/*item*/) {
								// Siempre se incluye el valor a retornar, sin importar si se ha o no seleccionado el item en el tree.
								//  Esto es asi, ya que del lado del server se debe evaular siempre por los valores 1 o 0 (logical_operator AND)
								return true;
							},
							transform_name_value: function(item, filter_value) {
								let definition = {
									"agent": {
										name : "AgentVisible"
									},
									"account messages": {
										name : "AccountMessagesVisible"
									},
									"flow designer": {
										name : "FlowDesignerVisible"
									}
								};
								let name_value = definition[item.value];
								// Cuando el checkbox del filtro se encuentra desmarcado, se debe postear un valor distinto a "0",
								// ya que sino retorna los que no tiene visibilidad y no es lo que se necesita.
								filter_value[name_value.name] = item.selected ? "1" : false;

								return filter_value;
							}
						}//,
						// "Used": {id: "Used", label : "<%=_T('Used')%>", // LasUsed
						// 	values: function () {
						// 		return ["all", "yes", "no"];
						// 	}
						// }
					}
				},
				on_ready: function() {
					loadFowProcesses();
				},
				realtime: null
			},
			form: {instance: inConcert.adm_form_shared_files}
		};
	};

	//------------------------------------------------------
	// apps launcher navigation config
	var apps_launcher_config = function(base_uri)
	{
		var get_message_error = function(type, err, name)
		{
			let message;
			if (type === "delete") {
				const err_messages = {
					"ERROR_DELETING_ACTION": function() {
						return format( "<%=_T('Unable to delete {0} Application integration. Error deleting application action.')%>", name );
					},
					"FORBIDDEN": function() {
						return format( "<%=_T('Unable to delete {0} Application integration. You dont have permission.')%>", name );
					},
					"ALLEGRO_INTEGRATION_ERROR": function() {
						return format( "<%=_T('Unable to delete {0} Application integration. Allegro integration error.')%>", name );
					}
				};
				let err_id;
				if (typeof err === "object") {
					err_id = err.result || "";
				}
				else {
					err_id = err;
				}
				err_id = err_id.toUpperCase();

				if ( err_messages[err_id] ) {
					message = err_messages[err_id]();
				}
				else {
					message = format( "<%=_T('Error deleting {0} Application integration.')%>", name );
				}
			}
			return message;
		};

		return {
			id: "apps_launcher",
			name: "<%=_T('Application integration')%>",
			permissions: {
				view    : "view_apps_launcher",
				save    : "save_apps_launcher",
				erase   : "delete_apps_launcher",
				permission_type: "vcc"
			},
			class_name: "itemAppsLauncher",
			i_class_name: "far fa-external-link",
			view: {
				render: "/inconcert/apps/configuration/pages/view_content_default.<%=_L%>.html#grunt-cache-bust",
				title: "<%=_T('Application integration')%>",
				tagline: "<%=_T('Tagline for Applications integration')%>",
				new_button_name: "<%=_T('Integrate new application')%>",
				row_id: "Id",
				display_name: "Name",
				paginator: {
					paged_text_format : "<%=_T('{page_row_start}-{page_row_end} of {many}')%>"
				},
				messages: {
					erase: "<%=_T('Do you want to delete Application integration {0} ?')%>",
					err: get_message_error
				},
				uri: {
					erase:  base_uri + "apps_launcher/delete/",
					search: base_uri + "apps_launcher/search/"
				},
				source: ["Id", "Name", "Description", "Data"],
				columns: [
					{id: "_number"      , name: "", type: "counter"     },
					{id: "Name"         , name: "<%=_T('Name')%>"       },
					{id: "Description"  , name: "<%=_T('Description')%>"},
					{id: "Application"  , name: "<%=_T('Application')%>" ,
						get_value: function(app_launcher) {
							let value = "";
							if (app_launcher.ApplicationId) {
							// Cuando regresamo de editar viene la aplicacion en este atributo.
								value = app_launcher.ApplicationId;
							}
							else if (app_launcher.Data && app_launcher.Data.webdesigner) {
							// Cuando viene del servidor, llega en este objeto.
								value = app_launcher.Data.webdesigner.projectId || "";
							}
							return {
								original_value: value, value: value
							};
						}
					},
					{id: "_action"      , name: "", system_action: {edit: true, erase: true}}
				],
				filter: {fields: {"Name": {control: null}, "Description": {control: null}, "DefaultValue": {control: null}}},
				realtime: null
			},
			form: {instance: inConcert.adm_form_apps_launcher}
		};
	};

	//------------------------------------------------------
	// apps_notifications navigation config

	var apps_notifications_config = function(base_uri)
	{
		var get_message_error = function(type, err, name)
		{
			var message;
			if (type === "delete") {
				var err_messages = {
					"FORBIDDEN": "<%=_T('Unable to delete app {0} notifications. You dont have permission.')%>"
				};
				if (typeof err === "object") {
					err = err.result || "";
				}
				message = err_messages[err.toUpperCase()] || "<%=_T('Error deleting app {0} notifications.')%>";
				message = format(message, name);
			}
			return message;
		};
		
		var is_action_enabled = function(action, row)
		{
			if (row.System && action.toLowerCase() === "delete") {
				return 0;
			}
			return 1;
		};

		return {
			id: "apps_notifications",
			name: "<%=_T('Apps Notifications')%>",
			can_show: function() {
				return inConcert.app.has_configured_notifications_in_vcc();
			},
			permissions: {
				view    : "view_apps_notifications",
				save    : "save_apps_notifications",
				erase   : "delete_apps_notifications",
				permission_type: "vcc"
			},
			class_name: "itemAppsNotifications",
			i_class_name: "far fa-bell-on",
			view: {
				render: "/inconcert/apps/configuration/pages/view_content_default.<%=_L%>.html#grunt-cache-bust",
				title: "<%=_T('Apps Notifications')%>",
				tagline: "<%=_T('Tagline for Apps Notifications')%>",
				new_button_name: "<%=_T('Link app to notify')%>",
				row_id: "Id",
				display_name: "Id",
				paginator: {
					paged_text_format : "<%=_T('{page_row_start}-{page_row_end} of {many}')%>"
				},
				messages: {
					erase: "<%=_T('Do you want to delete user {0} ?')%>",
					err: get_message_error
				},
				uri: {
					erase:  base_uri + "apps_notifications/delete/",
					search: base_uri + "apps_notifications/search/"
				},
				source: ["Id", "Description", "Url", "AllCampaigns", "System"],
				columns: [
					{id: "_number"     , name: "", type: "counter" },
					{id: "Id"          , name: "<%=_T('Id')%>"     },
					{id: "Description" , name: "<%=_T('Description')%>"   },
					{id: "Url"         , name: "<%=_T('Url')%>"   },
					{id: "AllCampaigns", name: "<%=_T('All Campaigns')%>", type: "boolean", class_name: "booleanColumnTitle"  ,
					 get_tooltip: function( app ) {	
						var allCampaigns = "";
						let allCamps = app.AllCampaigns;

						if (allCamps ==="0") {
							allCampaigns = "<%=_T('No')%>";
						}		
						if (allCamps === "1") {
							allCampaigns = "<%=_T('Yes')%>";
						}

						return allCampaigns;	
					}
					},
					{id: "System"      , name: "<%=_T('System App')%>", type: "boolean", class_name: "booleanColumnTitle"  ,
					get_tooltip: function(app){
						return app.System ? "<%=_T('Yes')%>" : "<%=_T('No')%>";
					}
				 },
					{id: "_action"     , name: "" , system_action: {edit: true, erase: true},
						is_enabled: is_action_enabled
					}
				],
				filter: { fields: { "Id": {control: null}, "Description": {control: null}, "Url": {control: null} } },
				realtime: null
			},
			form: {instance: inConcert.adm_form_apps_notifications}
		};
	};

	//------------------------------------------------------
	// vcc clients layouts navigation config
	var clients_layouts_config = function(base_uri)
	{
		const all_channels = inConcert.app.load_vcc_channels();
		var get_layout_name = function(row)
		{
			let original_value = row.Id;
			let translated_value = all_channels[original_value] ? all_channels[original_value].name : "<%=_T('All')%>";
			return {
				original_value: original_value,
				value: translated_value
			};
		};

		var get_message_error = function(type, err, name)
		{
			var message;
			if (type === "delete") {
				var err_messages = {
					"LAYOUT_DOES_NOT_EXIST": "<%=_T('Unable to delete {0} design. Design does not exist.')%>",
					"FORBIDDEN"            : "<%=_T('Unable to delete {0} design. You dont have permission.')%>"
				};
				if (typeof err === "object") {
					err = err.result || "";
				}
				message = err_messages[err.toUpperCase()] || "<%=_T('Error deleting {0} design.')%>";
				message = format(message, name);
			}
			return message;
		};

		return {
			id: "clients_windows_design",
			name: "<%=_T('Windows Design')%>",
			permissions: {
				view    : "view_clients_windows_layout_design",
				save    : "save_clients_windows_layout_design",
				erase   : "delete_clients_windows_layout_design",
				permission_type: "vcc"
			},
			class_name: "itemClientsLayout",
			i_class_name: "far fa-window",
			view: {
				render: "/inconcert/apps/configuration/pages/view_content_default.<%=_L%>.html#grunt-cache-bust",
				title: "<%=_T('Windows Design')%>",
				tagline: "<%=_T('Tagline for Clients Windows Design')%>",
				className : "windowsDesignTable",
				new_button_name: "<%=_T('New Client Window Design')%>",
				row_id: "Name",
				display_name: "Name",
				paginator: {
					paged_text_format : "<%=_T('{page_row_start}-{page_row_end} of {many}')%>"
				},
				messages: {
					erase: "<%=_T('Do you want to delete clients windows design {0} ?')%>",
					err: get_message_error
				},
				uri: {
					erase:  base_uri + "vccs/clients_design/delete/",
					search: base_uri + "vccs/clients_design/search/"
				},
				source: [ "Id", "Name" ],
				columns: [
					{ id: "Type"         , name: "", type: "multi_image" , cell_class_name : "WindowsDesignIconColumn", without_tooltip: true },
					{ id: "Name"         , name: "<%=_T('Channel name')%>", get_value: get_layout_name  },
					{ id: "_action"      , name: "" , system_action: { edit: true, erase: true }}
				],
				filter: { 
					fields: {
						"Name": { control: null }
					}
				},
				realtime: null
			},
			form: { instance: inConcert.adm_form_vcc_layouts }
		};
	};

	//------------------------------------------------------
	// vcc contacts addresses book navigation config
	var contacts_addresses_book = function(base_uri)
	{
		const all_channels = inConcert.app.load_vcc_channels();
		var get_channel = function(row)
		{
			let original_value = row.Channels;
			let value = (row.Channels || []).map(function(channel) {
				return {
					tooltip : (all_channels[channel] ? all_channels[channel].name : channel),
					class_name: "AddressesBookChannels" + channel
				};
			});

			return {
				original_value: original_value,
				value: value
			};
		};

		const addresses_type = {
			"Mail" : "<%=_T('E-mail')%>",
			"Phone": "<%=_T('Phone')%>"
		};

		var get_address_type = function(row)
		{
			let original_value = row.Type;
			let translated_value = addresses_type[original_value] || original_value;
			return {
				original_value: original_value,
				value: translated_value
			};
		};

		var get_address = function(row)
		{
			let original_value, display_value;
			if (row.Type.toUpperCase() === "MAIL") {
				original_value = row.address;
				display_value = row.address;
			}
			else {
				if (row.GlobalNumber) {
					original_value = row.GlobalNumber;
					display_value = original_value;
				} else {
					original_value = row.Number;
					display_value = original_value;
	
					if (row.Area) {
						display_value = row.Area + " " + display_value;
					}
					if (row.Country) {
						display_value = row.Country + " " + display_value;
					}
				}
			}
			return {
				original_value: original_value,
				value: display_value
			};
		};

		var get_message_error = function(type, err, name)
		{
			var message;
			if (type === "delete") {
				var err_messages = {
					"CONTACT_ADDRESS_BOOK_DOES_NOT_EXIST": "<%=_T('Unable to delete {0} address. Address does not exist.')%>",
					"FORBIDDEN"                          : "<%=_T('Unable to delete {0} address. You dont have permission.')%>"
				};
				if (typeof err === "object") {
					err = err.result || "";
				}
				message = err_messages[err.toUpperCase()] || "<%=_T('Error deleting {0} address.')%>";
				message = format(message, name);
			}
			return message;
		};

		return {
			id: "contacts_addresses_book",
			name: "<%=_T('Contacts - Addresses Book')%>",
			permissions: {
				view    : "view_contacts_addresses_book",
				save    : "save_contacts_addresses_book",
				erase   : "delete_contacts_addresses_book",
				permission_type: "vcc"
			},
			class_name: "itemAddressesBook",
			i_class_name: "far fa-address-book",
			view: {
				render: "/inconcert/apps/configuration/pages/view_content_default.<%=_L%>.html#grunt-cache-bust",
				title: "<%=_T('Addresses Book')%>",
				tagline: "<%=_T('Tagline for Address')%>",
				new_button_name: "<%=_T('Create address')%>",
				row_id: "AddressId",
				display_name: "Name",
				paginator: {
					paged_text_format : "<%=_T('{page_row_start}-{page_row_end} of {many}')%>"
				},
				messages: {
					erase: "<%=_T('Do you want to delete Address {0} ?')%>",
					err: get_message_error
				},
				uri: {
					erase:  base_uri + "contacts/addresses_book/delete/",
					search: base_uri + "contacts/addresses_book/search/"
				},
				source: [ "AddressId", "Name", "address", "Number", "Type", "Channels", "Description", "Area", "Country", "GlobalNumber" ],
				columns: [
					//{ id: "_number"     , name: "", type: "counter"                                    },
					{ id: "Name"        , name: "<%=_T('Name')%>", class_name: "AddressesBookFirstColumnHeader", cell_class_name: "AddressesBookFirstColumnRow"    }, 
					{ id: "Type"        , name: "<%=_T('Type')%>", get_value: get_address_type		   },
					/*{id: "Type"       , name: "<%=_T('Type')%>", type: "image", 
					class_name : "AddressesBookIconColumnHeader", 
					get_value: function( addresses_book ) {
						return { class_name : "AddressesBookType" + addresses_book.Type, column_class_name : "AddressesBookIconColumn" };
					}
					},*/
					{ id: "Address"     , name: "<%=_T('Address')%>"     , get_value: get_address      },
					{ id: "Channels"    , name: "<%=_T('Channels')%>"    , get_value: get_channel,   type: "multi_image" ,
						cell_class_name : "AddressesBookIconColumn d-flex", without_tooltip: true },
					//{ id: "Description" , name: "<%=_T('Description')%>"                               },
					{ id: "_action"     , name: "" , system_action: { edit: true, erase: true }        }
				],
				filter: { 
					fields: {
						"Name"       : { control: null },
						"Type"       : { control: null },
						"address"    : { control: null },
						"Number"     : { control: null },
						"Area"       : { control: null },
						"Country"    : { control: null },
						"Channels"   : { control: null },
						"Description": { control: null }
					}
				},
				realtime: null
			},
			form: { instance: inConcert.adm_form_contacts_addresses_book }
		};
	};

	//------------------------------------------------------
	// security navigation config

	var security_config = function()
	{
		return {
			id: "security",
			name: "<%=_T('Security')%>",
			can_show: function()
			{
				return true;
			},
			permissions: {
				view: "view_security",
				save: "save_security",
				permission_type: "vcc"
			},
			class_name: "itemSecurity",
			i_class_name: "far fa-shield-check",
			only_frm: true,
			view: {
				header:{
					hide_new_button: true
				},
				render: "/inconcert/apps/configuration/pages/view_content_default.<%=_L%>.html#grunt-cache-bust"
			},
			form: {
				instance: inConcert.adm_form_security,
				save_button: {
					update_caption: "<%=_T('Update')%>"
				},
				cancel_button: {
					initial_state : "enabled"
				}
			}
		};
	};	

		 //------------------------------------------------------
    // Phone Validator

	var phone_validator = function()
	{
		return {
			id: "phone_validator",
			name: "<%=_T('Phone Number Rules')%>",
			permissions: {
				view    : "view_phone_validator",
                save    : "save_phone_validator",
                erase   : "delete_phone_validator",
				permission_type: "vcc"
			},
			class_name: "itemPhoneValidator",
			i_class_name: "far fa-phone-volume",
			only_frm: true,
			view: {
				header:{
					hide_new_button: true
				},
				render: "/inconcert/apps/configuration/pages/view_content_default.<%=_L%>.html#grunt-cache-bust"
			},
			form: {
				instance: inConcert.adm_form_phone_validator
			}
		};
	};

	//------------------------------------------------------
	// get all navigation config
	var get_navigation_config = function()
	{
		var base_uri = inConcert.app.getBaseDomain() + "/inconcert/apps/administration/";
		return {
			campaigns              : campaigns_config(base_uri)         ,
			users                  : users_config(base_uri)             ,
			roles                  : roles_config(base_uri)             ,
			states                 : states_config(base_uri)            ,
			workgroups             : workgroups_config(base_uri)        ,
			skills                 : skills_config(base_uri)            ,
			tickets                : tickets_config(base_uri)           ,
			channels               : channels_config(base_uri)          ,
			audit                  : audit_config(base_uri)             ,
			databases			   : databases_config(base_uri)         ,
			accounts               : accounts_config(base_uri)          ,
			spell_checker          : spell_checker_config(base_uri)     ,
			signatures             : signatures_config(base_uri)        ,
			routing_groups         : routing_groups_config(base_uri)    ,
			ttsasr_engines         : ttsasr_engine_config(base_uri)     ,
			shared_files           : shared_files_config(base_uri)      ,
			api_users              : api_users_config(base_uri)         ,
			apps_launcher          : apps_launcher_config(base_uri)     ,
			apps_notifications     : apps_notifications_config(base_uri),
			clients_layouts        : clients_layouts_config(base_uri)   ,
			contacts_addresses_book: contacts_addresses_book(base_uri)  ,
			phone_validator		   : phone_validator(base_uri)			,
			security               : security_config(base_uri)
		};
	};

	function adm_config() {}

	adm_config.prototype.get_app_config = function()
	{
		return {
			id         : "administration",
			name       : "<%=_T('Administration')%>",
			selected   : true,
			class_name : "appAdministration",
			main_html  : "/inconcert/apps/configuration/pages/conf_main.<%=_L%>.html#grunt-cache-bust",
			navigation : get_navigation_config(),
			get_skin: function() {
				return "skinOCCNew";
			},
			can_show   : function() {
				return inConcert.permissions.to_category( "administration", "view" );
			},
			css        : [
				{ id: "adm_messages_css"                            , href: "/inconcert/apps/administration/css/administrationRules.css#grunt-cache-bust"                       },
				{ id: "adm_whatsapp_groups_css"                     , href: "/inconcert/apps/administration/css/administrationWhatsappGroups.css#grunt-cache-bust"              },
				{ id: "skin_occ_internal_forms_css"                 , href: "/inconcert/apps/configuration/css/skin-2021/masterInternalPageForms.css#grunt-cache-bust"          },
				//{ id: "skin_occ_internal_page_layout_css"           , href: "/inconcert/apps/configuration/css/skin-2021/masterInternalPageLayout.css#grunt-cache-bust"         },
				//{ id: "skin_occ_provisioning_master_css"            , href: "/inconcert/apps/provisioning/css/skin-2021/provisioningMaster.css#grunt-cache-bust"                },
				{ id: "skin_occ_adm_permissions_css"                , href: "/inconcert/apps/administration/css/skin-2021/administrationPermissions.css#grunt-cache-bust"       },
				{ id: "skin_occ_adm_disposition_css"               	, href: "/inconcert/apps/administration/css/skin-2021/administrationManagementResults.css#grunt-cache-bust" },
				{ id: "skin_occ_adm_messenger_css"                 	, href: "/inconcert/apps/administration/css/skin-2021/administrationPageMessenger.css#grunt-cache-bust"     },
				{ id: "skin_occ_adm_shared_files_css"              	, href: "/inconcert/apps/administration/css/skin-2021/sharedFiles.css#grunt-cache-bust"                     },
				{ id: "skin_occ_master_buttons_css"                 , href: "/inconcert/apps/agent/css/skin-2021/masterButtons.css#grunt-cache-bust"                            },
				{ id: "skin_occ_inbox_master_css"                   , href: "/inconcert/apps/agent/css/skin-2021/inboxMaster.css#grunt-cache-bust"                              },
				{ id: "skin_occ_master_quick_interactions_css"      , href: "/inconcert/apps/agent/css/skin-2021/masterQuickInteractions.css#grunt-cache-bust"                  },
				{ id: "skin_occ_messages_tooltips_css"              , href: "/inconcert/apps/agent/css/skin-2021/genericMessagesAndTooltips.css#grunt-cache-bust"               },
				{ id: "skin_occ_adm_master_css"                     , href: "/inconcert/apps/administration/css/skin-2021/administrationMaster.css#grunt-cache-bust"            },
				{ id: "skin_occ_schedule_config_css"                , href: "/inconcert/apps/administration/css/skin-2021/administrationScheduleConfig.css#grunt-cache-bust"    },
				{ id: "skin_occ_inbox_master_popups_css"            , href: "/inconcert/apps/agent/css/skin-2021/inboxMasterPopUps.css#grunt-cache-bust"                        },
				{ id: "skin_occ_master_client_workgroups_window_css", href: "/inconcert/apps/agent/css/skin-2021/masterClientWorkgroupsWindow.css#grunt-cache-bust"             },
				{ id: "skin_occ_master_client_window_css"           , href: "/inconcert/apps/agent/css/skin-2021/masterWindowCommon.css#grunt-cache-bust"                       },
				{ id: "skin_occ_master_client_entities_window_css"  , href: "/inconcert/apps/agent/css/skin-2021/masterClientEntitiesWindow.css#grunt-cache-bust"               },
				{ id: "skin_occ_custom_visual_search_css"           , href: "/inconcert/apps/agent/css/skin-2021/customVisualSearch.css#grunt-cache-bust"                       }
			]
		};
	};

	if (typeof inConcert.adm_config !== "function") {
		inConcert.adm_config = adm_config;
	}
}());

