/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 6 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	"use strict";

	//---------------------------------------------------------------------
	// form api users class
	function adm_form_api_users(params)
	{
		/* jshint validthis: true*/
		const success = adm_form_api_users.uber.constructor.call(this, params);
		if (!success) {
			return;
		}

		this.new_title = "<%=_T('Create API user')%>";
		this.update_title = "<%=_T('Update API User {0}')%>";
		this.view_title = "<%=_T('View API User {0}')%>";
		this.tagline = "<%=_T('Tagline form API User!!!')%>";

		this.html_file_name = "/inconcert/apps/administration/api_users/frm/render/";
		this.save_uri       = "/inconcert/apps/administration/api_users/save/";
		this.get_uri        = "/inconcert/apps/administration/api_users/get/";
		
		this.password_has_changed = null;

		return true;
	}

	inConcert.Utils.inherit(adm_form_api_users, inConcert.conf_form);

	if (typeof inConcert.adm_form_api_users !== "function") {
		inConcert.adm_form_api_users = adm_form_api_users;
	}

	const apiUserTypes = [
		{ id : "read", label : "<%=_T('Read')%>" },
		{ id : "write", label : "<%=_T('Read & Write')%>" },
		{ id : "full", label : "<%=_T('Full Access')%>" }
	];

	const expirationUnits = [
		{ id : "minutes", label : "<%=_T('Minutes')%>" },
		{ id : "hours", label : "<%=_T('Hours')%>" },
		{ id : "days", label : "<%=_T('Days')%>" }
	];

	adm_form_api_users.prototype.initialize = function()
	{
		this.password_has_changed = null;
	};

	adm_form_api_users.prototype.load_data = function(callback_complete_requests)
	{
		this.fill_combo($("#adm_form_api_user_type"), false, apiUserTypes, "id", "label");
		this.fill_combo($("#adm_form_api_user_expiration_unit"), false, expirationUnits, "id", "label");

		$("#adm_form_api_user_expiration")
			.off("change")
			.on("change", function() {
				var checked = $(this).is(":checked");
				$("#adm_form_api_user_expiration_unit, #adm_form_api_user_expiration_time")
					.prop("disabled", !checked);
				$("#adm_form_api_user_expiration_time_label")
					.toggleClass("", !checked)
					.toggleClass("required", checked);
			});

		adm_form_api_users.uber.load_data.call(this, callback_complete_requests);
	};
	
	adm_form_api_users.prototype.set_events = function()
	{
		adm_form_api_users.uber.set_events.call( this );
		var self = this;

		if (this.is_readonly()) {
			return;
		}
		
		// Esta funcion muestra el textbox de confirmacion de pwd.
		var show_pwd_confirm = function()
		{
			if (self.password_has_changed) {
				return;
			}
			self.password_has_changed = true;
			$("#adm_form_api_user_pwd_confirmation_section").show();
		};
		//---------------
		// eventos (paste, keypress, keydown) del text de pwd que habilitan el textbox de confirmación de pwd
		$("#adm_form_api_user_password")
			.on("paste", null, show_pwd_confirm)	//evento paste, ya sea desde el menu contextual o presionando ctrl-v
			.on("keypress", show_pwd_confirm)		//este evento captura las teclas presionandas que modifican el valor, menos las teclas "backspace" y "delete"
			.on("keydown", function(event) {
				/*
					En el evento keydown capturo y proceso las teclas que se han presionado y que modifican el texto existente.
						-keyCode 8  = tecla "backspace" y -keyCode 46 = tecla "delete"
				*/
				if (event.keyCode === 8 || event.keyCode === 46) {
					show_pwd_confirm();
				}
			}
		);		
	};

	adm_form_api_users.prototype.get = function()
	{
		let user = {
			Id: $("#adm_form_api_user_id").val(),
			Type: $("#adm_form_api_user_type").val(),
			ExpirationTime: 0,
			ExpirationUnit: ""
		};
		if (this.password_has_changed) {
			user.Password = $("#adm_form_api_user_password").val();
		}
		if ($("#adm_form_api_user_expiration").is(":checked")) {
			user.ExpirationUnit = $("#adm_form_api_user_expiration_unit").val();
			user.ExpirationTime = parseInt($("#adm_form_api_user_expiration_time").val());
		}
		if (!this.is_new) {
			let action = this.dashboard.get_current_action_obj();
			let id = action.view.row_id;
			user[id] = this.$current_row.attr("row_id");
		}
		return user;
	};

	adm_form_api_users.prototype.set = function(success, user)
	{
		if (!success) {
			return;
		}
		$("#adm_form_api_user_id").val(user.Id);
		$("#adm_form_api_user_password").val("****");
		$("#adm_form_api_user_type").val(user.Type);
		$("#adm_form_api_user_expiration")
			.prop("checked", user.ExpirationTime !== "0")
			.change();
		$("#adm_form_api_user_expiration_unit").val(user.ExpirationUnit || "minutes");
		$("#adm_form_api_user_expiration_time").val(user.ExpirationTime !== "0" && user.ExpirationTime || "");
	};

	adm_form_api_users.prototype.empty_controls = function()
	{
		$("#adm_form_api_user_id").val("").focus();
		$("#adm_form_api_user_password").val("");
		$("#adm_form_api_user_type").val("");
		$("#adm_form_api_user_expiration")
			.prop("checked", false)
			.change();
		$("#adm_form_api_user_expiration_unit").val("");
		$("#adm_form_api_user_expiration_time").val("");
	};

	adm_form_api_users.prototype.err_save_messages = function(err)
	{
		let is_custom_message = false;
		let message = null;

		if ( typeof err === "object" ) {
			let error_type = err.result.toUpperCase();
			if (error_type === "ID_DOES_NOT_EXIST" || error_type === "ID_ALREADY_EXISTS") {
				$("#adm_form_api_user_id").focus();
				if (error_type === "ID_ALREADY_EXISTS") {
					message = "<%=_T('Already exists another user with same id')%>";
				}
			}
			else if (error_type === "MUST_HAVE_A_PASSWORD") {
				$("#adm_form_api_user_password").focus();
			}
			else if (error_type === "MUST_HAVE_A_TYPE") {
				$("#adm_form_api_user_type").focus();
			}
			else if (error_type === "MUST_HAVE_A_EXPIRATION_TIME") {
				$("#adm_form_api_user_expiration_time").focus();
			}
			if (message) {
				is_custom_message = true;
				err = message;
			}
		}

		adm_form_api_users.uber.err_save_messages.call( this, err, is_custom_message ); //invokes the base class method
	};

	adm_form_api_users.prototype.get_validation_def = function()
	{
		var self = this;
		var definitions = [
			{
				control_id: "adm_form_api_user_id",
				messages: {
					required: "<%=_T('Please enter the user id.')%>",
					id:    "<%=_T('Only letters, numbers, dashes and dots are allowed in the user id.')%>",
					range: {
						between: { from: 8, to: 255 },
						get_value: function(user_id) {
							return user_id.length.toString();
						},
						message: "<%=_T('Length of User Id must be between {0} and {1} characters.')%>"
					}
				}
			},
			{
				control_id: "adm_form_api_user_password",
				can_validate: function() {
					return self.is_new || self.password_has_changed;
				},
				messages: {
					required: "<%=_T('Please enter the password.')%>",
					range: {
						between: { from: 8, to: 15 },
						get_value: function(user_pwd) {
							return user_pwd.length.toString();
						},
						message: "<%=_T('Password length must be between {0} and {1} characters.')%>"
					},
					custom: {
						type:  "custom",
						valid: function() {
							var response = {
								success: true
							};
							var pwd = $("#adm_form_api_user_password").val();
							var pwd_confirm = $("#adm_form_api_user_pwd_confirmation").val();
							if (pwd !== pwd_confirm) {
								response.success = false;
								response.message = "<%=_T('Passwords do not match.')%>";
							}
							return response;
						}
					}
				}
			}
		];
		return definitions;
	};

	adm_form_api_users.prototype.validate = function( save_callback, validation_def )
	{
		save_callback = save_callback || $.noop;

		if ($("#adm_form_api_user_expiration").is(":checked")) {
			var expirationTime = parseInt($("#adm_form_api_user_expiration_time").val());
			if (isNaN(expirationTime) || expirationTime <= 0) {
				this.validation.show_validation_error($("#adm_form_api_user_expiration_time"), 
					"<%=_T('Must enter an expiration time.')%>");
				return save_callback(false);
			}
		}

		adm_form_api_users.uber.validate.call(this, save_callback, validation_def);
	};

	adm_form_api_users.prototype.save = function(callback)
	{
		adm_form_api_users.uber.save.call( this, callback );  //invokes the base class method
	};

	adm_form_api_users.prototype.ready = function()
	{
		adm_form_api_users.uber.ready.call( this );  //invokes the base class method
		
		//inicializo el plugin password strength
		$("#adm_form_api_user_password").inconcert_password_strength(
		{
			veredictWords: {
				empty: {
					text: "<%=_T('Empty')%>",
					className: "passwordMessage passwordEmpty"
				},
				very_weak: {
					text: "<%=_T('Very Weak')%>",
					className: "passwordMessage passwordVeryWeak"
				},
				weak: {
					text: "<%=_T('Weak')%>",
					className: "passwordMessage passwordWeak"
				},
				mediocre: {
					text: "<%=_T('Mediocre')%>",
					className: "passwordMessage passwordMediocre"
				},
				strong: {
					text: "<%=_T('Strong')%>",
					className: "passwordMessage passwordStrong"
				},
				stronger: {
					text: "<%=_T('Stronger')%>",
					className: "passwordMessage passwordStronger"
				}
			}
		});

		var $id = $("#adm_form_api_user_id");
		if (!this.is_new) {
			this.readonly_control($id, true);
		}
		$id.focus();
	};
	
	adm_form_api_users.prototype.popup_show = function( params )
	{
		adm_form_api_users.uber.popup_show.call( this, params );
		switch(params && params.type || "") {
			case "api_users_key":
				$("#adm_form_api_users_key_input").focus();
				break;
			case "api_ip_filters":
				$("#adm_form_api_ip_filters_input").focus();
				break;
		}
	};
	
	adm_form_api_users.prototype.popup_html = function( params )
	{
		switch(params && params.type || "") {
			case "api_users_key":
				return "".concat(
				'<div class="internalPopUpContainer apiUsersKeyPopup formPopUp popUp520w">',
					'<div class="internalPopUpOverlay"></div>',
					'<div class="internalPopUp">',

						// title
						'<div class="internalPopUpTitle">',
							'<span><%=_T("Tokens Key")%></span>',
							//'<button id="adm_form_api_users_key_cancel_times" class="internalPopUpCloseButton"><i class="far fa-times"></i></button>',
						'</div>',

						'<div class="internalPopUpMiddle pageFormContainer">',

							// mensaje de error
							'<div id="adm_form_api_users_key_message" class="genericFormMessagesContainer messageError" hidden></div>',

							// key input
							'<div class="card no-card">',
								'<div class="card-block">',
									'<div class="row">',
										'<div class="col-12">',
											'<div class="form-group ">',
												'<label for="adm_form_api_users_key_input" class="required"> <%=_T("Key")%><%=_T(":")%></label>',
												'<div class="d-flex flex-row">',
													'<input id="adm_form_api_users_key_input" class="form-control" type="text" />',
													'<button id="adm_form_api_users_key_regenerate" class="pageButtonGeneric buttonColorWhite iconOnlyBtn align-self-end"><span class="far fa-sync"></span></button>',
												'</div>',
												'<i id="adm_form_api_users_key_input_validation" class="formFieldValidation formFieldError" hidden></i>',
											'</div>',
										'</div>',
									'</div>',
									// disclaimer
									'<div class="row">',
										'<div class="col-12">',
											'<div class="inlineMessagesContainer messageInfo"><%=_T("Authentication tokens are encrypted with this key. By regenerating it, existing tokens will no longer be valid.")%></div>',
										'</div>',
									'</div>',
								'</div>',
							'</div>',
						'</div>',

						// MODAL FOOTER BUTTONS
						'<div class="internalPopUpButtons">',
							"<button id='adm_form_api_users_key_update' class='pageButtonGeneric buttonColorBlue float-right'>" + "<%=_T('Confirm')%>" + "</button>",
							"<button id='adm_form_api_users_key_cancel' class='pageButtonGeneric buttonColorWhite float-right'>" + "<%=_T('Cancel')%>" + "</button>",
						'</div>',
					"</div>",
				"</div>");
			case "api_ip_filters":
				return "".concat(
				'<div class="internalPopUpContainer apiIpFiltersPopup popUp520w">',
					'<div class="internalPopUpOverlay"></div>',
					'<div class="internalPopUp">',

						// title
						'<div class="internalPopUpTitle">',
							'<span><%=_T("Filter by IP")%></span>',
						'</div>',

						// 
						'<div class="internalPopUpMiddle pageFormContainer">',
							'<div class="row">',
								'<div class="col-12">',
									'<div id="adm_form_api_ip_filters_message" class="genericFormMessagesContainer messageError" style="display:none;"></div>',
									'<textarea id="adm_form_api_ip_filters_input" class="formTextarea"  placeholder="// ' + '<%=_T("Enter one ip number per line.")%> // &#10; &#10; // <%=_T("For the search to match 1 or more numbers of an IP, use an * (asterisk) in the position that the variable numbers would go.")%> //' + '">',
									'</textarea>',
								'</div>',
							'</div>',
						'</div>',

						// MODAL FOOTER BUTTONS
						'<div class="internalPopUpButtons">',
							"<button id='adm_form_api_ip_filters_update' class='pageButtonGeneric buttonColorBlue float-right'>" + "<%=_T('Confirm')%>" + "</button>",
							"<button id='adm_form_api_ip_filters_cancel' class='pageButtonGeneric buttonColorWhite float-right'>" + "<%=_T('Cancel')%>" + "</button>",
						'</div>',

					"</div>",
				"</div>");
			default:
				return "";
		}
	};


	adm_form_api_users.prototype.popup_set_events = function( params )
	{
		var self = this;
		switch(params && params.type || "") {
			case "api_users_key":
				
				$("#adm_form_api_users_key_regenerate")
					.off("click")
					.on("click", function() {
						$("#adm_form_api_users_key_input").val(inConcert.Utils.generateGuid());
					});
					
				$("#adm_form_api_users_key_cancel")
					.off("click")
					.on("click", function() {
						self.popup_hide();
					});

				$("#adm_form_api_users_key_cancel_times")
					.off("click")
					.on("click", function() {
						self.popup_hide();
					});
					
				$("#adm_form_api_users_key_update")
					.off("click")
					.on("click", function() {
						var key = $("#adm_form_api_users_key_input").val();
						if (!key) {
							$("#adm_form_api_users_key_message")
								.show()
								.text("<%=_T('Must enter a key.')%>");
							return;
						}
						if (key === $("#adm_form_api_users_key_input").data("original_key")) {
							self.popup_hide();
							return;
						}
						$("#adm_form_api_users_key_message").hide();
						$.inconcertPopup.Show(
							"<%=_T('Confirmation')%>",
							"<%=_T('Existent tokens will be invalidated. Do you like to continue?')%>",
							$.inconcertPopup.ButtonSets.BS_YESNOCANCEL,
							function(response) {
								switch(response.toLowerCase()) {
									case "yes":
										inConcert.invoke({
											url: inConcert.app.getBaseDomain() + "/inconcert/apps/administration/api_users/set_key/",
											type: "POST",
											data: JSON.stringify({
												key : key
											}),
											dataType: "json",
											contentType: "application/json",
											success: function(result)
											{
												self.popup_hide();
											},
											error: function(_, error) {
												$("#adm_form_api_users_key_message")
													.show()
													.text("<%=_T('Error saving key.')%>");
											}
										});
										break;
								}
							},
							undefined,
							{
								popup_id        : "conf_inconcertPopup",
								container_class : "confirmationPopUp",
								overlay_class   : "confirmationOverlay"
							}
						);
					});
					
				break;
			case "api_ip_filters":
					
				$("#adm_form_api_ip_filters_cancel")
					.off("click")
					.on("click", function() {
						self.popup_hide();
					});

				$("#adm_form_api_ip_filters_cancel_times")
					.off("click")
					.on("click", function() {
						self.popup_hide();
					});
					
				$("#adm_form_api_ip_filters_update")
					.off("click")
					.on("click", function() {
						var ips_a = ($("#adm_form_api_ip_filters_input").val() || "").trim().split("\n");
						var ips = [];
						$.each(ips_a, function(_, ip) {
							ips.push(ip.trim());
						});
						$("#adm_form_api_ip_filters_message").hide();
						inConcert.invoke({
							url: inConcert.app.getBaseDomain() + "/inconcert/apps/administration/api_users/set_ip_filters/",
							type: "POST",
							data: JSON.stringify({
								ip_filters : ips
							}),
							dataType: "json",
							contentType: "application/json",
							success: function(result)
							{
								self.popup_hide();
							},
							error: function(_, error) {
								$("#adm_form_api_ip_filters_message")
									.show()
									.text("<%=_T('Error saving filters.')%>");
							}
						});
					});
					
				break;
		}
	};
	
	adm_form_api_users.prototype.popup_load_data = function( params, callback )
	{
		var self = this;
		switch(params && params.type || "") {
			case "api_users_key":
				inConcert.invoke({
					url: inConcert.app.getBaseDomain() + "/inconcert/apps/administration/api_users/get_key/",
					type: "GET",
					dataType: "json",
					contentType: "application/json",
					success: function(result)
					{
						$("#adm_form_api_users_key_input")
							.val(result.key)
							.data("original_key", result.key);
						callback();
					},
					error: function(_, error) {
						callback();
						inConcert.app.m_mainWindow.showBannerMessage({ message: "<%=_T('Error getting key.')%>", timeout: 5000 });
						self.popup_hide();
					}
				});
				break;
			case "api_ip_filters":
				inConcert.invoke({
					url: inConcert.app.getBaseDomain() + "/inconcert/apps/administration/api_users/get_ip_filters/",
					type: "GET",
					dataType: "json",
					contentType: "application/json",
					success: function(result)
					{
						var ips_s = "";
						$.each(result.ip_filters, function(_, ip) {
							ips_s += (ips_s? "\n" : "") + ip;
						});
						$("#adm_form_api_ip_filters_input").val(ips_s);
						callback();
					},
					error: function(_, error) {
						callback();
						inConcert.app.m_mainWindow.showBannerMessage({ message: "<%=_T('Error getting ip filters.')%>", timeout: 5000 });
						self.popup_hide();
					}
				});
				break;
			default:
				callback();
		}
	};
}());
