/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 9 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	//---------------------------------------------------------------------
	// form apps_launcher class
	function adm_form_apps_launcher(params)
	{
		const success = adm_form_apps_launcher.uber.constructor.call(this, params);
		if (!success) {
			return;
		}

		this.new_title = "<%=_T('Integrate new application')%>";
		this.update_title = "<%=_T('Edit Application integration {0}')%>";
		this.view_title = "<%=_T('View Application integration {0}')%>";
		this.tagline = "<%=_T('Tagline form Application integration!!!')%>";

		this.html_file_name = "/inconcert/apps/administration/apps_launcher/frm/render/";
		this.save_uri       = "/inconcert/apps/administration/apps_launcher/save/";
		this.get_uri        = "/inconcert/apps/administration/apps_launcher/get/";

		this.m_grd_roles = null;
		this.m_sorted_roles = [];
		this.__new_grid();

		return true;
	}

	inConcert.Utils.inherit(adm_form_apps_launcher, inConcert.conf_form);

	if (typeof inConcert.adm_form_apps_launcher !== "function") {
		inConcert.adm_form_apps_launcher = adm_form_apps_launcher;
	}

	adm_form_apps_launcher.prototype.get = function()
	{
		let app_launcher = {
			Name            : $("#adm_form_apps_launcher_name").val(),
			Description     : $("#adm_form_apps_launcher_desc").val(),
			ApplicationId   : $("#adm_form_apps_launcher_application_id").val(),
			windowMode      : this.get_window_mode(),
			RolesPermissions: this.get_roles_permissions()
		};
		const action = this.dashboard.get_current_action_obj();
		const id = action.view.row_id;
		if (!this.is_new) {
			app_launcher[id] = this.$current_row.attr("row_id");
		}
		else {
			app_launcher[id] = app_launcher.Name;
		}
		return app_launcher;
	};

	adm_form_apps_launcher.prototype.set = function(success, app_launcher)
	{
		if (!success) {
			return;
		}
		this.m_grd_roles.initialize($.noop, this.m_sorted_roles);

		const $name = $("#adm_form_apps_launcher_name");
		$name.val(app_launcher.Name);

		if (app_launcher.Data && app_launcher.Data.webdesigner) {
			$("#adm_form_apps_launcher_application_id").val(app_launcher.Data.webdesigner.projectId);
			if (app_launcher.Data && app_launcher.Data.webdesigner.windowMode) {
				$("#adm_form_apps_launcher_window_open_mode").val(app_launcher.Data.webdesigner.windowMode.mode);
				if (app_launcher.Data.webdesigner.windowMode.mode === "NORMAL") {
					$("#adm_form_apps_launcher_window_open_mode_normal_size").show();
					if (app_launcher.Data.webdesigner.windowMode.size) {
						$("#adm_form_apps_launcher_window_open_mode_normal_width").val(app_launcher.Data.webdesigner.windowMode.size.width);
						$("#adm_form_apps_launcher_window_open_mode_normal_height").val(app_launcher.Data.webdesigner.windowMode.size.height);
					}
				}
			}
		}
		$("#adm_form_apps_launcher_desc").val(app_launcher.Description);
	};

	adm_form_apps_launcher.prototype.empty_controls = function()
	{
		this.m_grd_roles.initialize($.noop, this.m_sorted_roles);
		$("#adm_form_apps_launcher_desc").val("");
		$("#adm_form_apps_launcher_application_id").val("");

		const $name = $("#adm_form_apps_launcher_name");
		$name.val("").focus();
		$("#adm_form_apps_launcher_window_open_mode").val("").trigger("change");
		$("#adm_form_apps_launcher_window_open_mode_normal_width").val("");
		$("#adm_form_apps_launcher_window_open_mode_normal_height").val("");
	};

	adm_form_apps_launcher.prototype.get_window_mode = function()
	{
		const $combo = $("#adm_form_apps_launcher_window_open_mode");
		let window_mode = {
			mode: $combo.val()
		};

		if (window_mode.mode === "NORMAL") {
			let width = $("#adm_form_apps_launcher_window_open_mode_normal_width").val().trim();
			let height = $("#adm_form_apps_launcher_window_open_mode_normal_height").val().trim();
			if (width.length > 0 && height.length > 0) {
				window_mode.size = {
					width: width,
					height: height
				};
			}
		}

		return window_mode;
	};

	adm_form_apps_launcher.prototype.get_roles_permissions = function()
	{
		const all_roles = this.m_grd_roles.get_items();
		const selected_roles = this.m_grd_roles.get_selected_items();
		let roles_to_include = [];
		let has_enabled_roles = {};

		// 1 - agrego los enabled
		all_roles.forEach(role => {
			for (let i = 0; i < selected_roles.length; i++) {
				let selected_role = selected_roles[i];
				if (selected_role.Id === role.Id ) {
					// Lo marco como seleccionado
					let role_to_include = {
						RoleId : role.Id,
						Enabled: true
					};
					// Lo incluyo
					roles_to_include.push(role_to_include);
					has_enabled_roles[role.Id.toLowerCase()] = true;
					break;
				}
			}
		});

		// 2 - agrego los que eran enabled y pasaron a estar disabled.
		if (!this.is_creating()) {
			const current_app_launcher = this.get_entity_instance();
			if (current_app_launcher && current_app_launcher.RolesPermissions) {
				current_app_launcher.RolesPermissions.forEach(role => {
					if (!has_enabled_roles[role.RoleId.toLowerCase()]) {
						// Lo marco como NO seleccionado
						let role_to_include = {
							RoleId : role.RoleId,
							Enabled: false
						};
						// Lo incluyo
						roles_to_include.push(role_to_include);
					}
				});
			}
		}
		return roles_to_include;
	};

	adm_form_apps_launcher.prototype.err_save_messages = function(err)
	{
		if ( typeof err === "object" ) {
			const error_type = err.result.toUpperCase();
			if (error_type === "MUST_HAVE_A_NAME" || error_type === "MUST_HAVE_UNIQUE_NAME") {
				$("#adm_form_apps_launcher_name").focus();
			}
			else if (error_type === "MUST_HAVE_A_DESCRIPTION") {
				$("#adm_form_apps_launcher_desc").focus();
			}
		}
		const is_custom_message = false;
		adm_form_apps_launcher.uber.err_save_messages.call(this, err, is_custom_message); //invokes the base class method
	};

	adm_form_apps_launcher.prototype.get_validation_def = function()
	{
		const definitions = [
			{
				control_id: "adm_form_apps_launcher_name",
				messages: {
					required : "<%=_T('Please enter the Name.')%>",
					id       : "<%=_T('Only letters, numbers, dashes and dots are allowed in the Name.')%>"
				}
			},
			{
				control_id: "adm_form_apps_launcher_desc",
				messages: {
					required : "<%=_T('Please enter the Description.')%>"
				}
			},
			{
				control_id: "adm_form_apps_launcher_window_open_mode",
				messages: {
					required : "<%=_T('Please select the Window Mode.')%>"
				}
			},
			{
				control_id: "adm_form_apps_launcher_application_id",
				messages: {
					required : "<%=_T('Please select the Application Id.')%>"
				}
			}
		];
		return definitions;
	};

	adm_form_apps_launcher.prototype.load_data = function(callback_complete_requests)
	{
		const self = this;
		const tweety_api = inConcert.app.getTweetyAPI();

		this.requests = [
			{
				request_function: function(success, response) {
					return tweety_api.load_generic_projects(success, response);
				},
				success: function(applications) {
					self.fill_applications(applications);
				}
			},
			{
				request_function: function(success /*, response */) {
					return inConcert.app.loadRolesThatAllowOverwriting(success);
				},
				success: function(roles) {
					self.fill_roles(roles);
				}
			},
		];
		adm_form_apps_launcher.uber.load_data.call(this, callback_complete_requests);  //invokes the base class method

		this.fill_combo_window_open_mode();
	};

	adm_form_apps_launcher.prototype.fill_applications = function(applications)
	{
		const $application = $("#adm_form_apps_launcher_application_id");
		if (!applications) {
			$application.html("");
			return;
		}
		const is_blank_first_item = true;
		const attr_id = "id";
		const attr_name = "description";
		this.fill_combo($application, is_blank_first_item, applications, attr_id, attr_name);
	};

	adm_form_apps_launcher.prototype.fill_roles = function(roles)
	{
		// el hash de grupos se pasa a un array ordenado alfabeticamente
		this.m_sorted_roles = inConcert.Utils.hash_to_sorted_array(roles,
			function(role) {
				return role.Name? role.NameLabel : role.IdLabel;
			}
		);
		if (this.is_creating()) {
			this.m_grd_roles.initialize($.noop, this.m_sorted_roles);
		}
	};

	adm_form_apps_launcher.prototype.__new_grid = function()
	{
		const self = this;

		this.m_grd_roles = new inConcert.conf_form_paginator({
			paginator_name: "ADM_ROLES",
			owner: this,
			frm: this,
			row_id: "Id",
			display_name: "Id",
			rows_per_page: 2500,
			show_at_least: 2499,
			paginator_type: "ALL_ROWS_CLIENT_SIDE",
			sortable: true,
			without_paginator: true,
			loading_waiting_msg: "<%=_T('Please wait, loading Roles ...')%>",
			ids: {
				grid: "adm_form_apps_launcher_roles_list",
				paginator: "adm_form_apps_launcher_roles_paginator",
				not_results_found: "adm_form_apps_launcher_roles_not_results_found"
			},
			strings: {
				not_results_found: "<%=_T('No Roles found.')%>"
			},
			columns: [
				{id: "_number"    , name: "", type: "counter"},
				{id: "_select"    , name: "", type: "row_selector",
					is_checked: function(item) {
						if (self.is_creating()) {
							return false;
						}
						const entity = self.get_entity_instance();
						let selected = false;
						let assigned_roles = entity.RolesPermissions || [];
						for (let i = 0; i < assigned_roles.length; i++) {
							let assigned_role = assigned_roles[i];
							if (assigned_role.RoleId.toLowerCase() === item.Id.toLowerCase()) {
								selected = true;
								break;
							}
						}
						return selected;
					}
				},
				{id: "Id", name: "<%=_T('Name')%>"},
				{id: "Description", name: "<%=_T('Description')%>"}
			],
			callbacks: {}
		});
	};

	adm_form_apps_launcher.prototype.set_events = function()
	{
		$("#adm_form_apps_launcher_window_open_mode")
			.off("change")
			.on("change", function() {
				const mode = $(this).val();
				$("#adm_form_apps_launcher_window_open_mode_normal_size")[mode === "MAXIMIZED" ? "hide" : "show"]();
			});

		if (this.is_readonly()) {
			return;
		}
		this.m_grd_roles.set_events();
	};

	adm_form_apps_launcher.prototype.ready = function()
	{
		adm_form_apps_launcher.uber.ready.call( this );  //invokes the base class method

		const $name = $("#adm_form_apps_launcher_name");
		if (!this.is_new) {
			this.readonly_control($name, true);
			$("#adm_form_apps_launcher_desc").focus();
			return;
		}
		$name.focus();
	};

	adm_form_apps_launcher.prototype.err_save_messages = function(err)
	{
		let is_custom_message = false;
		var get_error_description = function(err_type)
		{
			is_custom_message = true;
			let message;
			switch (err_type)
			{
				case "ACTION_ALREADY_EXIST":
					message = "<%=_T('Unable to save. Application integration already exists.')%>";
					$("#adm_form_apps_launcher_name").focus();
				break;

				default:
					is_custom_message = false;
					message = "<%=_T('Unable to save. Server error.')%>";
			}
			return message;
		};

		switch (typeof err)
		{
			case "object":
				var error_type = err.result.toUpperCase();
				err = get_error_description(error_type);
			break;

			case "string":
				err = get_error_description(err.toUpperCase());
			break;
		}

		adm_form_apps_launcher.uber.err_save_messages.call(this, err, is_custom_message);  //invokes the base class method
	};

	adm_form_apps_launcher.prototype.fill_combo_window_open_mode = function()
	{
		let list = [
			{id: "MAXIMIZED", description: "<%=_T('Maximized')%>"},
			{id: "NORMAL"   , description: "<%=_T('Normal')%>"   }
		];
		const $combo = $("#adm_form_apps_launcher_window_open_mode");
		const is_blank_first_item = true;
		const attr_id = "id";
		const attr_name = "description";
		const sorted = true;

		this.fill_combo($combo, is_blank_first_item, list, attr_id, attr_name, sorted);
	};

}());
