/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, esversion: 11 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

//	"use strict";
	const format = inConcert.Utils.format;

	//---------------------------------------------------------------------
	// form campaigns class
	function adm_form_campaigns(params)
	{
		let success = adm_form_campaigns.uber.constructor.call(this, params);
		if (!success) {
			return;
		}
		const self = this;

		this.new_title = "<%=_T('Create campaign')%>";
		this.update_title = "<%=_T('Update campaign {0}')%>";
		this.view_title = "<%=_T('View campaign {0}')%>";
		this.tagline = "<%=_T('Tagline form Campaign!!!')%>";

		this.html_file_name = "/inconcert/apps/administration/campaigns/frm/render/";
		this.save_uri =       "/inconcert/apps/administration/campaigns/save/";
		this.get_uri =        "/inconcert/apps/administration/campaigns/get/";
		this.permissions_type = "campaign";

		this.classes_name_frm_container = "campaignFormPage";
		this.include_save_and_continue = false;
		this.requests = [];
		this.sorted_timezones = null; // Este array se carga en la funcion definida en clase base "conf_form.fill_timezones"
		this.campaign = null;
		this.roles = null;
		this.sorted_roles = null;

		const perm = this.dashboard.permissions;
		const app_id = "administration";

		this.proxies_desc = inConcert.app.loadTranslatedChannels();
		this.im_protocols = null;

		const has_allegro_integration = inConcert.app.has_allegro_integration();

		var __custom_can_show_i6_section = function ( /*campaign_id*/ )
		{
			if (!has_allegro_integration) {
				return true;
			}
			if (self.is_creating()) {
				return true;
			} 
			const campaign = self.get_entity_instance();
			if (!campaign) {
				return false;
			}
			if (self.is_allegro_campaign()) {
				return false;
			}
			return true;
		};

		var __custom_can_show_allegro_section = function ( /*campaign_id*/ )
		{
			if (!has_allegro_integration) {
				return false;
			}
			if (self.is_creating()) {
				return false;
			}
			return self.is_allegro_campaign();
		};

		var __can_view_section = function ( section, campaign_id /* optional */ )
		{
			if ( !campaign_id ) {
				campaign_id = self.campaign ? self.campaign.Id : undefined;
			}

			if(campaign_id === inConcert.app.get_internal_campaign_id() && !["basic", "members", "schedule"].find(optionId => optionId === section.frm.section_id)) {
				return false;
			}
			let has_permission = perm.to_action( app_id, section.action_id, "VIEW", campaign_id, true /* no_log */, true /* all overrides */ );
			if (!has_permission) {
				return false;
			}
			if (!section.custom_can_show) {
				return true;
			}
			return section.custom_can_show( campaign_id );
		};

		var __get_instance = function ( section_id, campaign_id /* optional */ )
		{
			if ( !campaign_id ) {
				campaign_id = self.campaign ? self.campaign.Id : undefined;
			}

			switch(section_id)
			{
				case "basic":
					return inConcert.adm_form_campaigns_section_basic;
				case "accounts":
					return inConcert.adm_form_campaigns_section_accounts;
				case "channels":
					return inConcert.adm_form_campaigns_section_channels;
				case "members":
					return inConcert.adm_form_campaigns_section_members;
				case "groups":
					return inConcert.adm_form_campaigns_section_groups;
				case "schedule":
					return inConcert.adm_form_campaigns_section_schedule;
				case "integration_cci":
					return inConcert.adm_form_campaigns_section_cci;
				case "integration_cci_allegro":
					return inConcert.adm_form_campaigns_section_cci_allegro;
				case "disposition":
					return inConcert.adm_form_campaigns_section_dispositions;
				case "labels":
					return inConcert.adm_form_campaigns_section_labels;
				case "attention_level":
					return inConcert.adm_form_campaigns_section_attention_level;
				case "canned_messages":
					return inConcert.adm_form_campaigns_section_canned_messages;
				case "rules":
					return inConcert.adm_form_campaigns_section_rules;
				case "interaction_notifications":
					return inConcert.adm_form_campaigns_section_interaction_notifications;
				default:
					throw "adm_form_campaigns.__get_instance. Unrecognized instance: " + section_id;
			}
		};

		const configured_interaction_notifications_in_vcc = inConcert.app.has_configured_notifications_in_vcc();

		// definicion de las secciones del formulario.
		this.sections = {
			"basic": {
				name           : "<%=_T('Basic Information')%>",
				action_id      : "view_campaigns_basic",
				i_class_name   : "far fa-info-circle",
				no_form_display_name  : true, 
				instance       : function( campaign_id ) { return __get_instance( "basic", campaign_id ); },
				initial_state  : "enabled",
				selected       : true,
				can_show       : function( section, campaign_id ) { return __can_view_section( section, campaign_id ); }
			},
			"accounts": {
				name           : "<%=_T('Accounts')%>",
				action_id      : "view_campaigns_accounts",
				i_class_name     : "far fa-comment",
				instance       : function( campaign_id ) { return __get_instance( "accounts", campaign_id ); },
				can_show       : function( section, campaign_id ) { return __can_view_section( section, campaign_id ); }
			},
			"channels": {
				name           : "<%=_T('Channels')%>",
				action_id      : "view_campaigns_channels",
				i_class_name     : "far fa-exchange",
				instance       : function( campaign_id ) { return __get_instance( "channels", campaign_id ); },
				can_show       : function( section, campaign_id ) { return __can_view_section( section, campaign_id ); }
			},
			"members": {
				name           : "<%=_T('Members')%>",
				action_id      : "view_campaigns_members",
				i_class_name     : "far fa-user-friends",
				instance       : function( campaign_id ) { return __get_instance( "members", campaign_id ); },
				can_show       : function( section, campaign_id ) { return __can_view_section( section, campaign_id ); }
			},
			"groups": {
				name           : "<%=_T('Permissions')%>",
				action_id      : "view_campaigns_roles_permissions",
				i_class_name     : "far fa-key",
				instance       : function( campaign_id ) { return __get_instance( "groups", campaign_id ); },
				can_show       : function( section, campaign_id ) { return __can_view_section( section, campaign_id ); }
			},
			"schedule": {
				name           : "<%=_T('Schedule')%>",
				action_id      : "view_campaigns_schedule",
				i_class_name     : "far fa-clock",
				instance       : function( campaign_id ) { return __get_instance( "schedule", campaign_id ); },
				can_show       : function( section, campaign_id ) { return __can_view_section( section, campaign_id ); }
			},
			"integration_cci": {
				name           : "<%=_T('CC Integration')%>",
				action_id      : "view_campaigns_cc_integration",
				i_class_name     : "far fa-atom-alt",
				instance       : function( campaign_id ) { return __get_instance( "integration_cci", campaign_id ); },
				can_show       : function( section, campaign_id ) { return __can_view_section( section, campaign_id ); },
				custom_can_show: function( campaign_id ) { return __custom_can_show_i6_section( campaign_id ); }
			},
			"integration_cci_allegro": {
				name           : "<%=_T('CC Integration')%>",
				action_id      : "view_campaigns_cc_integration",
				i_class_name     : "far fa-atom-alt",
				instance       : function( campaign_id ) { return __get_instance( "integration_cci_allegro", campaign_id ); },
				initial_state  : "hidden",
				can_show       : function( section, campaign_id ) { return __can_view_section( section, campaign_id ); },
				custom_can_show: function( campaign_id ) { return __custom_can_show_allegro_section( campaign_id ); }
			},
			"disposition": {
				name           : "<%=_T('Dispositions')%>",
				action_id      : "view_campaigns_dispositions",
				i_class_name     : "fas fa-flag-checkered",
				instance       : function( campaign_id ) { return __get_instance( "disposition", campaign_id ); },
				can_show       : function( section, campaign_id ) { return __can_view_section( section, campaign_id ); }
			},
			"labels": {
				name           : "<%=_T('Labels')%>",
				action_id      : "view_campaigns_labels",
				i_class_name     : "far fa-tag",
				instance       : function( campaign_id ) { return __get_instance( "labels", campaign_id ); },
				can_show       : function( section, campaign_id ) { return __can_view_section( section, campaign_id ); }
			},
			"interaction_notifications": {
				name           : "<%=_T('Notifications')%>",
				action_id      : "view_campaigns_interaction_notifications",
				i_class_name     : "far fa-bell-on",
				instance       : function( campaign_id ) { return __get_instance( "interaction_notifications", campaign_id ); },
				can_show       : function( section, campaign_id ) { return __can_view_section( section, campaign_id ); },
				custom_can_show: function( _ ) { return configured_interaction_notifications_in_vcc; }
			},
		// Configuracion utilizada por campanas que NO tienen integracion con Allegro
			"attention_level": {
				name           : "<%=_T('Attention Levels')%>",
				action_id      : "view_campaigns_attention_level",
				i_class_name     : "far fa-poll-people",
				instance       : function( campaign_id ) { return __get_instance( "attention_level", campaign_id ); },
				can_show       : function( section, campaign_id ) { return __can_view_section( section, campaign_id ); },
				custom_can_show: function( campaign_id ) { return __custom_can_show_i6_section( campaign_id ); }
			},
			"canned_messages": {
				name           : "<%=_T('Canned Messages')%>",
				action_id      : "view_campaigns_canned",
				i_class_name     : "far fa-file-alt",
				instance       : function( campaign_id ) { return __get_instance( "canned_messages", campaign_id ); },
				can_show       : function( section, campaign_id ) { return __can_view_section( section, campaign_id ); },
				custom_can_show: function( campaign_id ) { return __custom_can_show_i6_section( campaign_id ); }
			},
			"rules": {
				name           : "<%=_T('Rules')%>",
				action_id      : "view_campaigns_rules",
				i_class_name     : "far fa-tasks",
				instance       : function( campaign_id ) { return __get_instance( "rules", campaign_id ); },
				can_show       : function( section, campaign_id ) { return __can_view_section( section, campaign_id ); },
				custom_can_show: function( campaign_id ) { return __custom_can_show_i6_section( campaign_id ); }
			}
		};
		this.create_sections(this);

		return true;
	}

	inConcert.Utils.inherit( adm_form_campaigns, inConcert.conf_form );

	if ( typeof inConcert.adm_form_campaigns !== "function" ) {
		inConcert.adm_form_campaigns = adm_form_campaigns;
	}

	adm_form_campaigns.prototype.get = function()
	{
		let campaign_obj;
		if (this.is_new) {
			let campaign_def = this.sections.basic.frm.get();
			campaign_obj = this.new_campaign(campaign_def);
		}
		else {
			campaign_obj = this.campaign;
			$.each(this.sections, function(_, section) {
				if (!section.frm) {
					return true; // continue
				}
				section.frm.get(campaign_obj);
			});
		}
		return campaign_obj;
	};

	adm_form_campaigns.prototype.uri_get_entity = function()
	{
		if (this.is_new) {
			return "";
		}
		var id = this.$current_row.attr("row_id");
		id = inConcert.Utils.getIdWithoutVCC(id);
		var uri = inConcert.app.getBaseDomain() + this.get_uri + id;
		return uri;
	};

	adm_form_campaigns.prototype.new_campaign = function(campaign)
	{
		if (typeof campaign !== "object") {
			return;
		}
		this.campaign = null;
		this.campaign = new inConcert.Campaign(campaign, undefined, "administration");
		return this.campaign;
	};

	adm_form_campaigns.prototype.get_entity_id = function()
	{
		if (this.campaign) {
			return this.campaign.Id;
		}
	};

	adm_form_campaigns.prototype.get_entity_instance = function()
	{
		return this.campaign;
	};

	adm_form_campaigns.prototype.set = function(success, campaign)
	{
		if (!success || !campaign) {
			return;
		}
		this.new_campaign(campaign);
	};

	adm_form_campaigns.prototype.__get_server_error_description = function(err)
	{
		let response = { is_custom_message: true, message: undefined, result: err.result, reason: err.reason };

		switch (err.result) {
			case "ALLEGRO_INTEGRATION_ERROR":
				response.is_custom_message = true;
				response.message = "<%=_T('Allegro integration error.')%>";
			break;
			case "ERROR_CAMPS_EXCEEDS_LIMIT_ALLOWED":
				response.is_custom_message = true;
				const limitCamp = inConcert.app.getMaxConfigurationsAllowed("MaxCampaigns");
				response.message = format("<%=_T('Can not create campaigns, exceeds limit allowed {0}.')%>", limitCamp);
			break;
			case "ERROR_CAMPS_USERS_EXCEEDS_LIMIT_ALLOWED":
				response.is_custom_message = true;
				const limitCampUsers = inConcert.app.getMaxConfigurationsAllowed("MaxCampaignsUsers");
				response.message = "<%=_T('Error adding campaign.')%>" + " "+ format("<%=_T('The total number of users assigned to the campaigns exceeds the limit allowed {0}.')%>", limitCampUsers);
			break;
			case "INVALID_REQUEST":
				response.is_custom_message = true;
				response.message = "<%=_T('Invalid request.')%>";
			break;
			default:
				response.is_custom_message = false;
				response.message = err.err_type;
		}

		return response;
	};

	adm_form_campaigns.prototype.err_save_messages = function( err )
	{
		if (typeof err !== "object") {
			adm_form_campaigns.uber.err_save_messages.call( this, err, false /* custom message*/ );  //invokes the base class method
			return;
		}

		var response = this.__get_server_error_description(err);

		adm_form_campaigns.uber.err_save_messages.call( this, response.message? response.message: response, response.is_custom_message );

		var error_id = ( err.result || "" ).toUpperCase();
		if ( error_id === "FORBIDDEN" || error_id === "MUST_HAVE_A_NAME" || error_id === "MUST_HAVE_UNIQUE_NAME") {
			$("#adm_form_campaign_name").focus();
		}
		else if ( error_id === "MUST_HAVE_A_DESCRIPTION") {
			$("#adm_form_campaign_desc").focus();
		}
	};

	adm_form_campaigns.prototype.get_validation_def = function()
	{
		var definitions = [];
		$.each(this.sections, function( _, section )
		{
			if (!section.frm) {
				return true; // continue
			}
			section.frm.get_validation_def(definitions);
		});
		return definitions;
	};

	adm_form_campaigns.prototype.fill_roles = function(roles)
	{
		var self = this;
		this.roles = {};
		$.each(roles, function( _, role )
		{
			self.roles[role.Id.toLowerCase()] = role;
		});

		var sort_roles = function()
		{
			// el hash de grupos se pasa a un array ordenado alfabeticamente, y se filtra segun la funcion "has_permissions"
			self.sorted_roles = inConcert.Utils.hash_to_sorted_array(self.roles,
				function(role) {
					return role.Name? role.NameLabel : role.IdLabel;
				});
		};

		sort_roles();
	};

	adm_form_campaigns.prototype.get_role = function(role_id)
	{
		if (!this.roles || typeof role_id !== "string") {
			return;
		}
		return this.roles[role_id.toLowerCase()];
	};

	adm_form_campaigns.prototype.get_sorted_roles = function()
	{
		return this.sorted_roles;
	};

	adm_form_campaigns.prototype.fill_vcc_mail_addresses = function( vcc_mail_addresses )
	{
		var self = this;
		this.vcc_mail_addresses = {};
		$.each( vcc_mail_addresses || {}, function( _, mail )
		{
			self.vcc_mail_addresses[ mail.uid ] = mail;
		});
	};

	adm_form_campaigns.prototype.get_vcc_mail_addresses = function()
	{
		return this.vcc_mail_addresses;
	};

	adm_form_campaigns.prototype.get_im_protocol = function( protocol )
	{
		if ( !protocol ) {
			return "";
		}
		if ( !this.im_protocols[ protocol ] ) {
			return protocol;
		}
		return this.im_protocols[ protocol ];
	};

	adm_form_campaigns.prototype.load_im_protocols = function()
	{
		var self = this;
		inConcert.app.loadIMProtcols( function( success, protocols )
		{
			if ( !success ) {
				return;
			}
			self.im_protocols = {};
			$.each( protocols || {}, function( _, protocol_info )
			{
				self.im_protocols[ protocol_info.id ] = protocol_info.description;
			});
		});
	};

	adm_form_campaigns.prototype.getAccountDescription = function( params )
	{
		if ( !params ) {
			return "";
		}
		var account = params.account;
		if ( !account || !this.campaign ) {
			return "";
		}

		var message_type = params.message_type || "ONLY_ACCOUNT_DESCRIPTION";
		var account_desc = this.campaign.getAccountDescription( account );

		if ( message_type === "ONLY_ACCOUNT_DESCRIPTION" ) {
			return account_desc;
		}
		else if ( message_type === "WITH_PROXY" ) {
			var proxy_desc = this.proxies_desc[ account.proxyType.toUpperCase() ];
			if ( !proxy_desc ) {
				return account_desc;
			}
			return format( "{0} ({1})", account_desc, proxy_desc );
		}
		else if ( message_type === "WITH_SPECIFIC_PROXY" ) {
			if ( account.proxyType.toUpperCase() === "IM" ) {
				var protocol_description = this.get_im_protocol( account.protocol );
				if ( !protocol_description ) {
					return account_desc;
				}
				return format( "{0} ({1})", account_desc, protocol_description );
			}
			else {
				return account_desc;
			}
		}
	};

	adm_form_campaigns.prototype.load_events_suscribe = function()
	{
		var self = this;

		this.events_to_suscribe = {
			"form_campaign_updated_accounts": {
				pub_id: "/application/campaigns/update_accounts",
				callback: function( data ) {
					self.sections.accounts.frm.realtime_update( "update_accounts", data );
				}
			}
		};
	};

	adm_form_campaigns.prototype.load_data = function(callback_complete_requests)
	{
		var self = this;
		this.requests = [];

		this.requests.push({
			request_function: function(success/*, response*/) {
				return inConcert.app.loadAllRoles( success );
			},
			success: function(roles) {
				self.fill_roles(roles);
			}
		});

		var base_uri = inConcert.app.getBaseDomain() + "/inconcert/apps/administration/";
		this.requests.push(
			{
				url: base_uri + "vccs/accounts/search/",
				type: "POST",
				data: { columns: ["uid"], special_values: { accountType: "MAIL" } },
				success: function( vcc_accounts )
				{
					self.fill_vcc_mail_addresses( vcc_accounts.rows );
				},
				error: function() {
					var message = "<%=_T('Error retrieving vcc addresses.')%>";
					var timeout_ms = 5000;
					self.show_status_message( message, "Error", timeout_ms );
				}
			}
		);

		$.each(this.sections, function( _, section)
		{
			if (!section.frm) {
				return true; // continue
			}
			section.frm.attach_load_request(self.requests);
		});

		adm_form_campaigns.uber.load_data.call( this, callback_complete_requests ); //invokes the base class method
	};

	adm_form_campaigns.prototype.save_execute = function( after_edit, campaign /*, hash_id */ )
	{
		const self = this;

		this.dashboard.hide_back_button();
		const callback_response = function(success, response)
		{
			let err;
			if ( success ) {
				if (response) {
					campaign.allPermissionsOnCampaign = response.allPermissionsOnCampaign;
				}
			}
			else{
				err = response;
			}
			if ( !after_edit ) {
				after_edit = self.is_new ? "SAVE_AND_NOT_EXIT" : "SAVE_WITHOUT_ACTION";
			}
			
			self.on_saved(success, campaign, after_edit, err, function() {
				self.dashboard.hide_waiting();
			});
		};

		campaign.save_basic(callback_response, this.is_new);
	};

	adm_form_campaigns.prototype.on_saved = function( success, entity, after_save, err, callback)
	{
		var self = this;

		callback = callback || $.noop;

		var pending_events = 0;
		var on_saved = function() {
			if ( pending_events === 0 ) {
				adm_form_campaigns.uber.on_saved.call(
					self,
					success,
					entity,
					after_save,
					err,
					function(success) {
						callback(success);
					}
				); //invokes the base class method
			}
		};

		if ( !success ) {
			on_saved();
			return;
		}

		if ( this.is_new ) {
			let permissions = entity.allPermissionsOnCampaign;
			delete entity.allPermissionsOnCampaign;
			self.campaign.setAllPermissions( permissions );
			inConcert.app.update_campaign_permissions( {
				campaignId: entity.Id,
				permissions: permissions
			} );
			inConcert.permissions.load_logged_user_permissions(function(){
				on_saved();
			});
		}
		else {
			on_saved();
		}

		var section = this.sections[ this.current_section ];
		if ( section && section.frm && section.frm.readonly_campaign_name ) {
			section.frm.readonly_campaign_name();
		}
	};

	adm_form_campaigns.prototype.initialize = function()
	{
		adm_form_campaigns.uber.initialize.call( this ); //invokes the base class method
	};

	adm_form_campaigns.prototype.dispose = function()
	{
		adm_form_campaigns.uber.dispose.call( this ); //invokes the base class method
		this.campaign = null;
	};

	adm_form_campaigns.prototype.set_events = function()
	{
		adm_form_campaigns.uber.set_events.call( this ); //invokes the base class method
	};

	adm_form_campaigns.prototype.ready = function()
	{
		adm_form_campaigns.uber.ready.call( this ); //invokes the base class method
	};

	adm_form_campaigns.prototype.is_allegro_campaign = function()
	{
		const campaign = this.get_entity_instance();
		if (!campaign) {
			return false;
		}
		if (campaign.isAllegro()) {
			return true;
		}
		return false;
	};

}());
