/*  jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
  strict:true, undef:true, unused:true, curly:true, browser:true, indent:false, maxerr:50, esversion: 9 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	var format = inConcert.Utils.format;

	/*---------------------------------------------------------------------
	CLASS USERS SEARCH POPUP.
	------------------------
		Realiza la busqueda y asignacion de usuarios que pertenecen al nivel de atencion.
		Los mismos se visualizaran en el frm de niveles de atencion.
	*/
	function users_search_poup( params )
	{
		this.parent = params.parent;
		this.section = this.parent.parent;
		this.id_container               = "adm_form_campaign_attention_level_edition_popup_container";
		this.id_hide_popup              = "adm_form_campaign_attention_level_edition_popup_hide";
		this.id_hide_popup_times		= "adm_form_campaign_attention_level_edition_popup_hide_times";
		this.id_update_users_list       = "adm_form_campaign_attention_level_edition_popup_update_list";
		this.id_err_message             = "adm_form_campaign_attention_level_edition_popup_message";
		this.id_waiting                 = "adm_form_campaign_attention_level_edition_popup_waiting";
		this.id_search_text             = "adm_form_campaign_attention_level_edition_popup_search_text";
		this.id_search_list             = "adm_form_campaign_attention_level_edition_popup_search_list";
		this.id_search_not_result_found = "adm_form_campaign_attention_level_edition_popup_search_not_results_found";
		this.id_total_rows              = "adm_form_campaign_attention_level_edition_popup_total_rows";
		this.id_check_all               = "adm_form_campaign_attention_level_edition_popup_check_all";

		this.total_rows = 0;
		this.selected_rows = 0;
		this.members = null;

		this.initialize();
	}

	users_search_poup.prototype.initialize = function()
	{
		this.set_events();
	};

	users_search_poup.prototype.make_members_markup = function( selected_users, callback )
	{
		var self = this;

		callback = callback || $.noop;
		var members_section = this.section.frm.get_frm_section( "members" );
		if ( !members_section ) {
			callback();
			return;
		}

		var add_rows = function( members )
		{
			self.hide_not_results_found();
			$("#" + self.id_search_list).find("li[user_data]").remove();
			var users = [];
			$.each(members,
				function(_, member)
				{
					if (member.apps_permisson.interaction)
					{
						var is_member = member.Role ? true : false;
						if (!is_member) {
							return true; // continue
						}
						var member_without_vcc = inConcert.Utils.getIdWithoutVCC( member.Id );
						var get_user_name = function() {
							if ( !member.Name ) {
								return member_without_vcc;
							}
							if ( typeof member.FirstSurname !== "string" || member.FirstSurname.trim().length === 0 ) {
								return member.Name;
							}
							return member.Name + " " + member.FirstSurname;
						};
						users.push({
							id   : member_without_vcc,
							name : get_user_name(),
							apps_permisson : member.apps_permisson
						});
					}
				}
			);
			if ( !users || users.length === 0 ) {
				self.show_not_results_found();
				callback();
				return;
			}
			self.add_rows_users( users, selected_users );
			$("#" + self.id_check_all).prop( "checked", users.length === Object.keys(selected_users).length );
			callback();
		};

		if ( this.members ) {
			add_rows( this.members );
		}
		else {
			this.show_waiting( "<%=_T('Wait moment please, loading users')%>" );
			var campaign = this.section.frm.campaign;
			campaign.getFullUserList(function( members, error )
			{
				if ( !error ) {
					members = inConcert.Utils.sortArray(members, "Id", "asc");
					add_rows( members );
					self.members = members;
				}
				else {
					var default_message = "<%=_T('Error retrieving campaign users configuration.')%>";
					var messages_error = {
						FORBIDDEN: "<%=_T('You not have permissions for retrieving campaign members configuration.')%>"
					};
					var err_code = ( error.result || "" ).toUpperCase();
					var message = messages_error[ err_code ] || default_message;
					self.section.frm.show_status_message( message, "Error" );
				}
				// self.set_load(false);
				self.hide_waiting();
			});
		}
	};

	users_search_poup.prototype.show = function( selected_users )
	{
		this.hide_errors();
		this.disable_ctrls( true );
		this.total_rows = 0;
		this.selected_rows = 0;
		if ( !this.section.has_permission_to_section() || !this.section.has_permission_to_save() ) {
			var message = "<%=_T('Unable to update users list: You dont have permissions for this action.')%>";
			this.section.frm.show_status_message( message, "Error", 5000 );
			return;
		}
		this.make_members_markup( selected_users );
		$("#" + this.id_container).show();
	};

	users_search_poup.prototype.hide = function()
	{
		$("#" + this.id_container).hide();
		$("#" + this.id_search_text).val("");

		this.parent.on_show();
	};

	users_search_poup.prototype.show_not_results_found = function()
	{
		$("#" + this.id_search_list).hide();
		$("#" + this.id_search_not_result_found).show();
		// $("#" + this.id_total_rows).hide();
	};

	users_search_poup.prototype.hide_not_results_found = function()
	{
		$("#" + this.id_search_not_result_found).hide();
		$("#" + this.id_search_list).show();
		// $("#" + this.id_total_rows).show();
	};

	users_search_poup.prototype.hide_errors = function()
	{
		$("#" + this.id_err_message).hide();
		$("#" + this.id_container).find(".formFieldError").hide();
	};

	users_search_poup.prototype.show_waiting = function( message )
	{
		this.hide_errors();
		var $waiting = $("#" + this.id_waiting);
		$waiting.find("[waiting_message]").text(message || "");
		$waiting.show();
	};

	users_search_poup.prototype.hide_waiting = function()
	{
		$("#" + this.id_waiting).hide();
	};

	users_search_poup.prototype.get_selected_users = function( only_visible )
	{
		var selected_users = {};
		var count = 0;
		$("#" + this.id_search_list + " li[user_data]").each(function( _, member )
		{
			var $member = $(member);
			if ( !only_visible || (only_visible && $member.is(":visible")) ) {
				var $chk = $member.find("input[selector='1'][type='checkbox']");
				var checked = $chk.prop("checked");
				if ( checked ) {
					count++;
					var user = $chk.data("user_info");
					selected_users[ user.id ] = {
						id   : user.id,
						name : user.name || user.id
					};
				}
			}
		});
		return {
			count : count,
			users : selected_users
		};
	};

	users_search_poup.prototype.get_opt_user_id = function( user_id )
	{
		var normalized_id = inConcert.Utils.getNormalizedId( user_id );
		var opt_id = this.id_search_list + "_opt_user_" + normalized_id;
		return opt_id;
	};

	users_search_poup.prototype.add_row_user = function( user, selected_users, $list_container )
	{
		var self = this;
		if ( !user ) {
			return;
		}

		// aca dejo checked segun lo que vino del frm
		var is_selected_user = false;
		$.each( selected_users || {}, function(i, selected_user) {
			if ( user.id === selected_user.id ) {
				is_selected_user = true;
				self.selected_rows++;
				delete selected_users[i]; // remuevo de los selected, para que no valide contra este mismo usuario
				return false; // break
			}
		});

		var unique_id = user.id.trim().toLowerCase();
		var $make_new_row = function()
		{
			var user_id = user.id;
			var user_name = user.name || user_id;
			var opt_dom_id = self.get_opt_user_id( user_id );

			var $user = $(
				"<li user_data='true' class='formListItem' search_text='" + unique_id + user_name + "'>" +
					"<input id='" + opt_dom_id + "' user_id='" + unique_id + "' class='formListCheck' selector='1' type='checkbox' />" +
					"<div class='formItemText'>" +
						"<label for='" + opt_dom_id + "' user_id='true' unique_id='" + unique_id + "'>" + user_id  + "</label>" +
					"</div>" +
					"<div class='formItemText'>" +
						"<label for='" + opt_dom_id + "' user_name='true' user_name='" + user_name + "'>" + user_name + "</label>" +
					"</div>" +
				"</li>");
			$user.find(".formListCheck").prop("checked", is_selected_user).data("user_info", user);
			return $user;
		};

		var $new_row = $make_new_row();
		$list_container.append($new_row);
	};

	users_search_poup.prototype.update_total_rows_msg = function( from_cache )
	{
		$("#" + this.id_total_rows).text( format("<%=_T('{0} of {1}')%>", this.count_selected_rows( from_cache ), this.count_rows()) );
	};

	users_search_poup.prototype.add_rows_users = function( users, selected_users )
	{
		var self = this;

		var $list_container = $("#" + this.id_search_list);
		var $parent = $list_container.parent();

		$list_container.detach();

		inConcert.Utils.foreach_timeout({
			rows : users,
			waiting_message : "<%=_T('Please wait, adding user, {0} of {1}.')%>",
			append_fn : function( user ) {
				self.add_row_user( user, selected_users, $list_container );
			},
			waiting_fn : function( message ) {
				self.show_waiting( message );
			},
			complete_fn : function() {
				$parent.append( $list_container );
				self.hide_waiting();
				self.disable_ctrls( false );
				self.update_total_rows_msg();
				$("#" + self.id_search_text).focus();
			}
		});
	};

	users_search_poup.prototype.count_rows = function()
	{
		if ( this.total_rows === 0 ) {
			this.total_rows = $("#" + this.id_search_list).find("li[user_data]").length;
		}
		return this.total_rows;
	};

	users_search_poup.prototype.count_selected_rows = function( from_cache )
	{
		if ( !from_cache ) {
			var selected = this.get_selected_users();
			this.selected_rows = selected.count;
		}
		return this.selected_rows;
	};

	users_search_poup.prototype.disable_ctrls = function( disabled )
	{
		if ( !disabled ) {
			disabled = this.count_rows() === 0;
		}
		this.section.frm.disable_control($("#" + this.id_search_text), disabled);
		this.section.frm.disable_control($("#" + this.id_update_users_list), disabled);
	};

	users_search_poup.prototype.make_search_regexp = function( word )
	{
		if (typeof word !== "string") {
			return "";
		}
		var search_pattern = inConcert.Utils.removeSpecialChars(word);
		return new RegExp(search_pattern.replace(/[^\w\s]/gi, ""), "i");
	};

	users_search_poup.prototype.filter = function( text )
	{
		var self = this;
		var $list_container = $("#" + this.id_search_list);
		if ( text.trim().length === 0 ) {
		// si no hay texto buscado en el filtro, no filtro la row
			this.hide_not_results_found();
			$list_container.find("li[user_data]").show();
			return;
		}
		var total_rows = this.count_rows();
		var hidden_rows = 0;
		$list_container.find("li[user_data]").each(function(_, row)
		{
			var $row = $(row);
			var row_value = $row.attr("search_text");
			var search_regexp = self.make_search_regexp(text);
			if (row_value.search(search_regexp) === -1) {
				$row.hide();
				hidden_rows++;
			}
			else {
				$row.show();
			}
		});
		if ( total_rows === hidden_rows ) {
			this.show_not_results_found();
		}
		else {
			this.hide_not_results_found();
		}
	};

	users_search_poup.prototype.set_events = function()
	{
		var self = this;

		// evento keyup del text de search
		$("#" + this.id_search_text)
			.off("keyup")
			.on("keyup", function() {
				var text = $(this).val();
				if ( self.timeout ) {
					window.clearTimeout( self.timeout );
					self.timeout = null;
				}
				self.timeout = window.setTimeout( function() {
					self.filter( text );
					window.clearTimeout( self.timeout );
					self.timeout = null;
					self.update_total_rows_msg();
				}, 500 );
			});
		// click sobre el boton update para actualizar la lista
		$("#" + this.id_update_users_list)
			.off("click")
			.on("click", function() {
				self.hide_errors();
				var selected = self.get_selected_users();
				self.hide();
				self.parent.update_users_list( selected.users );
			});
		// click en el boton para cerrar el popup y descartar cambios
		$("#" + this.id_hide_popup)
			.off("click")
			.on("click", function() {
				self.hide();
			});
		$("#" + this.id_hide_popup_times)
			.off("click")
			.on("click", function() {
				self.hide();
			});
		// click en el check all
		$("#" + this.id_check_all)
			.off("click")
			.on("click", function() {
				var checked_all = $(this).prop("checked");
				$("#" + self.id_search_list).find("li[user_data='true']:visible input[selector='1'][type='checkbox']").prop("checked", checked_all);
				self.update_total_rows_msg();
			});
		// evento click de los checkboxes de la lista de users.
		var event_selector = "input[selector='1'][type='checkbox']";
		$("#" + this.id_search_list)
			.off("click", event_selector)
			.on("click", event_selector, function() {
				if ( $(this).prop("checked") ) {
					self.selected_rows++;
				}
				else {
					self.selected_rows--;
				}
				$("#" + self.id_check_all).prop("checked", self.count_rows() === self.selected_rows );
				self.update_total_rows_msg( true /* desde el cache */ );
			});
	};
	//-----------------------------------


	/*---------------------------------------------------------------------
	CLASS USERS FRM.
	---------------
		Esta clase es la encargada de realizar el Alta/Modificacion client/side desde el frm de niveles de atencion.
		Aqui se encuentra el nombre, la descripcion y la lista de members que pertenecen a nivel atencion
	*/

	function users_frm( params )
	{
		this.parent = params.parent;
	// dom ids
		// frm
		this.id_frm_container              = "adm_form_campaign_attention_level_edition";
		// inputs
		this.id_attention_name             = "adm_form_campaign_attention_level_edition_name";
		this.id_attention_desc             = "adm_form_campaign_attention_level_edition_desc";
		// grid
		this.id_users_list_container       = "adm_form_campaign_attention_level_edition_users_list_container";
		this.id_users_list                 = "adm_form_campaign_attention_level_edition_users_list";
		this.id_users_not_result_found     = "adm_form_campaign_attention_level_edition_users_not_results_found";
		this.id_users_popup_show           = "adm_form_campaign_attention_level_edition_popop_show";
		this.id_conflict_buttons           = "adm_form_campaign_attention_level_edition_conflict_buttons";
		this.id_show_users_conflict        = "adm_form_campaign_attention_level_edition_only_conflict";
		this.id_remove_all_users_conflict  = "adm_form_campaign_attention_level_edition_remove_all_conflict";
		// buttons actions
		this.id_frm_actions                = "adm_form_campaign_attention_level_edition_actions";
		this.id_create                     = "adm_form_campaign_attention_level_edition_create";
		this.id_cancel                     = "adm_form_campaign_attention_level_edition_close";

		// instancia del poup de search de users/members
		this.search_popup = new users_search_poup({
			parent : this
		});
		this.initialize();
	}

	users_frm.prototype.initialize = function() { this.set_events(); };

	users_frm.prototype.show_not_result_found = function()
	{
		$("#" + this.id_users_list_container).hide();
		$("#" + this.id_conflict_buttons).hide();
		$("#" + this.id_users_not_result_found).show();
	};

	users_frm.prototype.hide_not_result_found = function()
	{
		$("#" + this.id_users_not_result_found).hide();
		$("#" + this.id_users_list_container).show();
		$("#" + this.id_conflict_buttons).show();
	};

	// retorna:
	//          { "rocio": { "id": "rocio" }, "ayelen": { "id": "ayelen" }, "de_mello": { "id": "de_mello" }, "aquino": { "id": "aquino" } }
	//
	users_frm.prototype.get_rows_users = function()
	{
		var users = {};
		var $list = $("#" + this.id_users_list);
		if ( !$list.is(":visible") ) {
			return users;
		}
		$list.find("li[user_data]").each(function(_, user_data)
		{
			var $user = $(user_data);
			var user = $user.data("user_info");
			users[ user.id ] = {
				id : user.id
			};
		});
		return users;
	};

	users_frm.prototype.get = function()
	{
		var id;
		if ( this.parent.$row_editing ) {
			id = this.parent.$row_editing.attr("attention_level_id");
		}
		else {
			id = inConcert.Utils.generateGuid();
		}
		return {
			id          : id,
			name        : $("#" + this.id_attention_name).val(),
			description : $("#" + this.id_attention_desc).val(),
			users       : this.get_rows_users()
		};
	};

	users_frm.prototype.set = function( attention_level )
	{
		attention_level = attention_level || {};
		$("#" + this.id_attention_name).val( attention_level.name );
		$("#" + this.id_attention_desc).val( attention_level.description );
		this.add_rows_users( attention_level.users, "add" );
	};

	users_frm.prototype.hide_solve_actions_users_in_conflicts = function()
	{
		$("#" + this.id_show_users_conflict).hide();
		$("#" + this.id_remove_all_users_conflict).hide();
	};

	users_frm.prototype.show_message_users_in_conflicts = function( conflicts )
	{
		var self = this;
		var $list = $("#" + this.id_users_list);
		if ( !$list.is(":visible") || !conflicts ) {
			return;
		}
		$.each( conflicts.users, function( user_id, err_detail )
		{
			var opt_dom_id = self.get_opt_user_id( user_id );
			var $user = $("#" + opt_dom_id);
			$user.find("[error]").show();
			var $show_err = $user.find("[show_err]");
			$show_err.data("err_detail", err_detail).show();
		});

		$("#" + this.id_show_users_conflict).html( "<%=_T('Show only in conflict')%>" ).show();
		$("#" + this.id_remove_all_users_conflict).show();
	};

	users_frm.prototype.toggle_only_users_in_conflicts = function()
	{
		var $list = $("#" + this.id_users_list);
		if ( !$list.is(":visible") ) {
			return;
		}
		var $errors = [];
		var $hidden_users = [];
		$list.find("li[user_data]").each(function(_, user_data)
		{
			var $user = $(user_data);
			if ( !$user.is(":visible") ) {
				$hidden_users.push( $user );
			}
			var $err = $user.find("[error]");
			if ( !$err.is(":visible") ) {
				$errors.push( $user );
			}
		});

		var $btn_hide_user_in_conflict = $("#" + this.id_show_users_conflict);
		var action;
		if ( $hidden_users.length > 0 ) {
			action = "show";
			$btn_hide_user_in_conflict.html( "<%=_T('Show only in conflict')%>" );
		}
		else {
			action = "hide";
			$btn_hide_user_in_conflict.html( "<%=_T('Show all users')%>" );
		}

		$.each( $errors, function( _, $user ) { $user[ action ](); });
	};

	users_frm.prototype.remove_all_users_in_conflicts = function()
	{
		var self = this;
		var $list = $("#" + this.id_users_list);
		if ( !$list.is(":visible") ) {
			return;
		}
		this.parent.show_waiting( "<%=_T('Please wait, removing users in conflicts.')%>" );
		// a este codigo lo meto en un timeout para que pueda ser visualizado el waiting
		window.setTimeout(function()
		{
			$list.find("li[user_data]").each(function(_, user_data)
			{
				var $user = $(user_data);
				var $err = $user.find("[error]");
				if ( $err.is(":visible") ) {
					self.remove_user( $user, false );
				}
				else {
					$user.show(); // a esto lo hago x las dudas que se encuentre el filtro aplicado.
				}
			});
			self.show_or_hide_grid();
			$("#" + self.id_frm_container).find(".formFieldValidation").hide();
			self.hide_solve_actions_users_in_conflicts();
			self.parent.frm.hide_messages();
			self.parent.hide_waiting();
		});
	};

	users_frm.prototype.show = function( current_attention_level, callback )
	{
		callback = callback || $.noop;
		this.set( current_attention_level );
		// var $create = $("#" + this.id_create);

		// if ( !current_attention_level ) {
		// // si presione el boton para crear uno nuevo
		// 	$create.html("<%=_T('Create')%>");
		// }
		// else {
		// // si presione el boton para editar
		// 	$create.val("<%=_T('Update')%>");
		// }
		$("#" + this.id_frm_container).show();
		$("#" + this.id_frm_actions).show();

		callback();
		this.on_show();
	};

	users_frm.prototype.on_show = function()
	{
		this.parent.frm.add_css_class("pageFormForVisibleButtons");
		$("#" + this.id_attention_name).focus();
	};

	users_frm.prototype.hide = function()
	{
		this.parent.hide_attention_level_frm_edition();
	};

	users_frm.prototype.on_hide = function()
	{
		this.hide_solve_actions_users_in_conflicts();
		$("#" + this.id_frm_container).hide();
		$("#" + this.id_frm_actions).hide();
		this.parent.frm.remove_css_class("pageFormForVisibleButtons");
		//this.search_popup.members = null;
	};

	users_frm.prototype.get_opt_user_id = function( user_id )
	{
		var normalized_id = inConcert.Utils.getNormalizedId( user_id || "" );
		var opt_id = this.id_users_list + "_opt_user_" + normalized_id;
		return opt_id;
	};

	users_frm.prototype.can_add_row = function( user )
	{
		var $row = $("#" + this.get_opt_user_id( user.id ) );
		return $row.length === 0;
	};

	users_frm.prototype.add_row_user = function( user, apply_effect, $list_container )
	{
		var self = this;
		if ( !user ) {
			return;
		}
		if ( !this.can_add_row( user ) ) {
			return;
		}
		var unique_id = user.id.trim().toLowerCase();
		var $make_new_row = function()
		{
			var user_id = user.id;
			var user_name = user.name || user_id;
			var display = "";
			if ( apply_effect ) {
				display = " hidden ";
			}
			var opt_dom_id = self.get_opt_user_id( user_id );
			var $user = $(
				"<li id='" + opt_dom_id + "' user_data='true' class='formListItem position-relative'" + display + ">" +
					"<i error='true' class='formFieldValidation formFieldError' hidden ></i>" +
					"<label user_id='true' class='formItemText' unique_id='" + unique_id + "'>" + user_id  + "</label>" +
					"<label user_name='true' class='formItemText' user_name='" + user_name + "'>" + user_name + "</label>" +
					"<button show_err='true' class='buttonColorGray gridActionButton view' title='"+ "<%=_T('Show Error')%>" + "' hidden><i class='far fa-exclamation-triangle'></i><%=_T('Show Error')%></button>" +
					"<button action_del='true' class='buttonColorGray gridActionButton delete last' title='"+ "<%=_T('Remove')%>" + "'><i class='buttons-icons-delete'></i><%=_T('Remove')%></button>" +
				"</li>");
			$user.data("user_info", user);
			return $user;
		};

		var $new_row = $make_new_row();
		$list_container.append($new_row);

		if (apply_effect && $new_row) {
			// aca realizo el fade in
			$new_row.fadeIn("slow");
		}
	};

	users_frm.prototype.count_rows = function()
	{
		return $("#" + this.id_users_list).find("li[user_data]").length;
	};

	users_frm.prototype.show_or_hide_grid = function()
	{
		if ( this.count_rows() === 0 ) {
			this.show_not_result_found();
		}
		else {
			this.hide_not_result_found();
		}
	};

	users_frm.prototype.add_rows_users = function( users, from )
	{
		var self = this;

		var $list_container = $("#" + this.id_users_list);
		var $parent = $list_container.parent();
		$list_container.detach();
		$list_container.find("li[user_data]").remove();

		users = inConcert.Utils.hash_to_sorted_array(users || {},
			function(user) {
				return user.id.toLowerCase();
			}
		);

		inConcert.Utils.foreach_timeout({
			rows : users,
			waiting_message : from === "add" ? "<%=_T('Please wait, adding user, {0} of {1}.')%>" : "<%=_T('Please wait, updating users.')%>",
			append_fn: function( user ) {
				self.add_row_user( user, undefined, $list_container );
			},
			waiting_fn: function( message ) {
				self.parent.show_waiting( message );
			},
			complete_fn: function() {
				$parent.append( $list_container );
				self.parent.hide_waiting();
				self.show_or_hide_grid();
			}
		});
	};

	users_frm.prototype.update_users_list = function( users )
	{
		$("#" + this.id_frm_container).find(".formFieldValidation").hide();
		this.hide_solve_actions_users_in_conflicts();
		this.add_rows_users( users, "update" );
	};

	users_frm.prototype.validate = function()
	{
		var self = this;

		this.parent.frm.hide_messages();
		this.hide_solve_actions_users_in_conflicts();

		var show_error = function(message_error, $control)
		{
			self.parent.frm.show_status_message( message_error, "Error", false );
			var error_id = $control.attr("id") + "_validation";
			var $error_indicator = $("#" + error_id);
			if ($error_indicator) {
				$error_indicator.show();
			}
			if ($control) {
				$control.focus();
			}
		};

		var $name = $("#" + this.id_attention_name );
		var att_level_info = this.get();
		var new_name = att_level_info.name.trim().toLowerCase();

		var validation_result = inConcert.Utils.validate( new_name, "required" );
		if ( !validation_result.success ) {
			// valida que el name tenga al menos un caracter
			show_error("<%=_T('Name is required.')%>", $name);
			return false;
		}

		validation_result = inConcert.Utils.validate( new_name, "id" );
		if ( !validation_result.success ) {
			// valida que el name tenga al menos un caracter
			show_error("<%=_T('Only letters, numbers, dashes and dots are allowed in the Name.')%>", $name);
			return false;
		}

		validation_result = inConcert.Utils.validate(new_name, "html_malicious_code");
		if ( !validation_result.success ) {
			// valida codigo malicioso
			show_error("<%=_T('Possible malicious code in the field value.')%>", $name);
			return false;
		}

		var is_new = this.parent.$row_editing ? false : true;
		var $rows = this.parent.$get_rows();
		var already = function()
		{
			var edit_att_level_id;
			if ( !is_new ) {
				edit_att_level_id = self.parent.$row_editing.attr("attention_level_id");
			}
			var already_exist = false;
			$rows.each(function( _, row )
			{
				var $row = $(row);
				if ( edit_att_level_id !== undefined ) {
				// si estoy editando, no valido sobre el mismo row.
					var row_att_level_id = $row.attr("attention_level_id");
					if (row_att_level_id === edit_att_level_id) {
						return true; // continue
					}
				}
				var current_name = $row.find( self.parent.selector_row_id ).text();
				already_exist = current_name.trim().toLowerCase() === new_name;
				if (already_exist) {
					show_error("<%=_T('Name already exists.')%>", $name);
					return false; // break
				}
			});
			return already_exist;
		};

		// valida que si estoy agregando un nuevo attention levels, el mismo ya no exista.
		if (already()) {
			return false;
		}

		if (att_level_info.description.trim().length === 0) {
			// valida que el description tenga al menos un caracter
			show_error("<%=_T('Description is required.')%>", $("#" + this.id_attention_desc));
			return false;
		}

		validation_result = inConcert.Utils.validate(att_level_info.description, "html_malicious_code");
		if ( !validation_result.success ) {
			// valida codigo malicioso
			show_error("<%=_T('Possible malicious code in the field value.')%>", $("#" + this.id_attention_desc));
			return false;
		}

		$("#" + this.id_frm_container).find(".formFieldValidation").hide();
		$name.val("");
		$("#" + this.id_attention_desc).val("");

		// validation successfully
		return att_level_info;
	};

	users_frm.prototype.remove_user = function( $user_to_remove, apply_effect )
	{
		var self = this;
		if ( apply_effect ) {
			$user_to_remove.fadeOut("slow", function() {
				$user_to_remove.remove();
				self.show_or_hide_grid();
			});
		}
		else {
			$user_to_remove.remove();
		}
	};

	users_frm.prototype.show_user_conflict_detail = function( conflicts )
	{
		var $popup = this.parent.$get_dom_element_by_id( "view_popup" );

		$popup.html(
			"<div class='internalPopUpContainer attentionMembersPopup attentionLevelConflict formPopUp popUp520w'>" +
				"<div class='internalPopUpOverlay'></div>" +
				"<div class='internalPopUp'>" +
					// waiting
					"<div id='adm_form_campaign_att_levels_popup_waiting' class='popupWaitingContainer' hidden>" +
						"<div waiting_message='true' class='popupWaitingMessage'></div>" +
						"<div class='popupWaitingAnimation'></div>" +
						"<div class='popupWaitingOverlay'></div>" +
					"</div>" +

					// title
					"<div class='internalPopUpTitle'>" +
						"<span id='adm_form_campaign_att_levels_popup_title'></span>" +
						"<button close_popup='true' class='internalPopUpCloseButton'><i class='far fa-times'></i></button>" +
					"</div>" +

					// content
					"<div class='internalPopUpMiddle'>" +
						"<div class='row'>" +
							"<div class='col-12'>" +
								"<ul id='adm_form_campaign_att_levels_popup_list' class='formList formListAsGrid'>" +
									"<li class='formListTitle'>" +
										"<label head='true' class='formListTitleText'>" + "<%=_T('Accounts')%>" + "</label>" +
										"<label head='true' class='formListTitleText'>" + "<%=_T('Attention levels')%>" + "</label>" +
									"</li>" +
								"</ul>" +
							"</div>" +
						"</div>" +
					"</div>" +
					// MODAL FOOTER BUTTONS
					'<div class="internalPopUpButtons">',
						"<button close_popup='true' class='pageButtonGeneric buttonColorWhite float-right'>" + "<%=_T('Cancel')%>" + "</button>",
					'</div>',
				"</div>" +
			"</div>"
		);

		$popup.show();

		var $list = $("#adm_form_campaign_att_levels_popup_list");

		var count_accounts = 0;
		var count_att_levels = 0;

		var add_account = function( account )
		{
			count_accounts++;
			var att_levels = "";

			var sorted_att_levels = inConcert.Utils.hash_to_sorted_array(account.att_levels,
				function(att_level) {
					return att_level.name.toLowerCase();
				}) || {};

			$.each(sorted_att_levels, function( _, att_level )
			{
				count_att_levels++;
				if ( att_levels.length > 0 ) {
					att_levels = att_levels + ", ";
				}
				att_levels = att_levels + att_level.name;
			});
			var $account = $(
				"<li class='formListItem'>" +
					"<div class='formItemText'>" +
						"<label account_id='true'>" + account.name + "</label>" +
					"</div>" +
					"<div class='formItemText'>" +
						"<label>" + att_levels + "</label>" +
					"</div>" +
				"</li>");

			$list.append($account);
		};

		var $waiting = $("#adm_form_campaign_att_levels_popup_waiting");
		var sorted_accounts = inConcert.Utils.hash_to_sorted_array(conflicts.accounts,
			function(account) {
				return account.name.toLowerCase();
			}) || {};

		inConcert.Utils.foreach_timeout({
			rows : sorted_accounts,
			waiting_message : "<%=_T('Please wait, adding accounts, {0} of {1}.')%>",
			append_fn : function( account ) {
				add_account( account );
			},
			waiting_fn : function( message ) {
				$waiting.find("[waiting_message]").text(message || "");
				$waiting.show();
			},
			complete_fn : function() {
				var title;
				if ( count_accounts === 1 ) {
					title = "<%=_T('{0} user has {1} account in conflict with {2} assigned attention levels.')%>";
				}
				else {
					title = "<%=_T('{0} user has {1} accounts in conflict with {2} assigned attention levels.')%>";
				}
				title = format(title, conflicts.user_id, count_accounts, count_att_levels);
				$("#adm_form_campaign_att_levels_popup_title").text(title);
				$waiting.hide();
			}
		});

	};

	users_frm.prototype.hide_user_conflict_detail = function()
	{
		var $popup = this.parent.$get_dom_element_by_id( "view_popup" );
		$popup.hide();
	};

	users_frm.prototype.set_events = function()
	{
		var self = this;

		$("#" + this.id_create)
			.off("click")
			.on("click", function() {
				self.parent.add_or_update_attention_level();
			});
		$("#" + this.id_cancel)
			.off("click")
			.on("click", function() {
				self.hide();
				self.parent.hide_messages();
			});

		// boton new que abre el popup para poder actualizar la lista de members
		$("#" + this.id_users_popup_show)
			.off("click")
			.on("click", function() {
				var users = self.get_rows_users();
				self.search_popup.show( users );
			});

		// boton que visualiza unicamente los usuarios en conflicto
		$("#" + this.id_show_users_conflict)
			.off("click")
			.on("click", function() {
				self.toggle_only_users_in_conflicts();
			});

		// boton que visualiza unicamente los usuarios en conflicto
		$("#" + this.id_remove_all_users_conflict)
			.off("click")
			.on("click", function() {
				self.remove_all_users_in_conflicts();
			});

		// grilla acciones boton delete
		var $list_container = $("#" + this.id_users_list);
		// boton delete de la lista de members
		var event_selector = "button[action_del]";
		$list_container
			.off("click", event_selector)
			.on("click", event_selector, function()
			{
				var $user = $(this).parent();
				var message = format(
					"<%=_T('Do you want to remove the user ({0}: {1}) ?')%>",
					$user.find("[user_id='true']").text(),
					$user.find("[user_name='true']").text()
				);
				$.inconcertPopup.Show(
					"<%=_T('Confirm')%>",
					message,
					$.inconcertPopup.ButtonSets.BS_YESNO,
					function( response )
					{
						if (response.toLowerCase() !== "yes") {
							return;
						}
						if ( !self.parent.has_permission_to_section() || !self.parent.has_permission_to_save() ) {
							var message = "<%=_T('Unable to remove users: You dont have permissions for this action.')%>";
							self.parent.frm.show_status_message( message, "Error", 5000 );
							return;
						}
						self.remove_user( $user, true );
					},
					undefined,
					{
						popup_id        : "conf_inconcertPopup",
						container_class : "confirmationPopUp",
						overlay_class   : "confirmationOverlay"
					}
				);
			});

		event_selector = "button[show_err]";
		$list_container
			.off("click", event_selector)
			.on("click", event_selector, function()
			{
				var $show_err = $(this);
				var err_detail = $show_err.data("err_detail");
				self.show_user_conflict_detail( err_detail );
			});

		event_selector = "button[close_popup='true']";
		var $popup = this.parent.$get_dom_element_by_id( "view_popup" );
		$popup
			.off("click", event_selector)
			.on("click", event_selector, function()
			{
				self.hide_user_conflict_detail();
			});
	};

	//
	//-----------------------------------------


	//---------------------------------------------------------------------
	// 
	/*---------------------------------------------------------------------
	CLASS ATTENTION LEVELS SECTION 
	------------------------
		Esta clase hereda de la clase "conf_form_section" utilizado en las diferentes secciones de la campana.

		Es la encargada de mostrar la lista de niveles de atencion, y de realizar el ABM de estos.
	*/
	function adm_form_campaigns_section_attention_level(params)
	{
		var success = adm_form_campaigns_section_attention_level.uber.constructor.call(this, params);
		if (!success) {
			return;
		}
		// permisos
		this.view_action_id    = "view_campaigns_attention_level";   // view action id de permisos
		this.save_action_id    = "save_campaign_attention_level";   // save action id de permisos
		this.erase_action_id   = "delete_campaign_attention_level"; // delete action id de permisos
		this.has_perm_to_save  = null;
		this.has_perm_to_erase = null;

		// dom ids
		// 
		this.id_list_container           = "adm_form_campaign_attention_level_list_container";
		this.att_levels                  = "adm_form_campaign_attention_level_list";
		this.id_new_att_level            = "adm_form_campaign_attention_level_new";
		this.att_levels_not_result_found = "adm_form_campaign_attention_level_not_results_found";


		// selectors - row
		this.selector_row_id       = "label[col_id]";
		this.selector_row_desc     = "label[col_desc]";
		this.selector_row_users    = "label[col_users]";
		this.selector_row_accounts = "label[col_accounts]";

		this.$row_editing = null; // Aca se encuentra el row html de jquery que se esta editando
		this.all_attention_levels = {}; // todos los attention levels de la campana (utilizado para la busqueda general)
		this.list_to_save = {}; // los added/updated attention levels que se agregan en el popup y se deben guardar al cerrar el mismo. TODO: Cambiar !! feo muy feo, pero de momento no encontre otra forma para hacerlo rapido y sin romper todo lo que esta armado.

		this.users_frm = null; // instancia de la clase que maneja la actualizacion de members para los niveles de atencion.

		return true;
	}

	inConcert.Utils.inherit( adm_form_campaigns_section_attention_level, inConcert.conf_form_section );

	if (typeof inConcert.adm_form_campaigns_section_attention_level !== 'function') {
		inConcert.adm_form_campaigns_section_attention_level = adm_form_campaigns_section_attention_level;
	}

	adm_form_campaigns_section_attention_level.prototype.dispose = function()
	{
		this.has_perm_to_save  = null;
		this.has_perm_to_erase = null;
	};

	// Alta en el hash de todos los attention levels
	adm_form_campaigns_section_attention_level.prototype.add_attention_level_to_full_list = function(id, attention_level)
	{
		this.all_attention_levels[id] = attention_level;
	};

	// Actualiza en el hash de todos los attention levels
	adm_form_campaigns_section_attention_level.prototype.update_attention_level_on_full_list = function(id, attention_level)
	{
		if (!this.all_attention_levels.hasOwnProperty(id)) {
			return;
		}
		this.all_attention_levels[id] = attention_level;
	};

	// Baja en el hash todos los attention levels
	adm_form_campaigns_section_attention_level.prototype.erase_attention_level_on_full_list = function(id)
	{
		if (!this.all_attention_levels.hasOwnProperty(id)) {
			return;
		}
		delete this.all_attention_levels[id];
	};

	adm_form_campaigns_section_attention_level.prototype.get_attention_level_from_full_list = function(id)
	{
		if (!id) {
			return {};
		}
		return this.all_attention_levels[id];
	};

	// Alta en el hash attention_level que se han modificado en el popup y se enviaran a guardar al server.
	adm_form_campaigns_section_attention_level.prototype.update_list_to_save = function( attention_level, is_new)
	{
		this.list_to_save = this.list_to_save || {};
		this.list_to_save[attention_level.id] = { is_new: is_new || false, attention_level: attention_level };
	};

	// Vacia el hash que mantiene la lista de attention levels a guardar en el server.
	adm_form_campaigns_section_attention_level.prototype.empty_list_to_save = function()
	{
		this.list_to_save = {};
	};

	// Valida si el hash de attention levels a guardar se encuentra vacio.
	adm_form_campaigns_section_attention_level.prototype.is_empty_list_to_save = function()
	{
		return $.isEmptyObject(this.list_to_save);
	};

	adm_form_campaigns_section_attention_level.prototype.$get_row = function(id)
	{
		if ( typeof id !== "string" ) {
			id = "";
		}
		return $("#" + this.att_levels).find("li[row_data][attention_level_id='" + id + "']");
	};

	adm_form_campaigns_section_attention_level.prototype.$get_rows = function()
	{
		return $("#" + this.att_levels).find("li[row_data]");
	};

	// agrega un row ordenado alfabeticamente
	adm_form_campaigns_section_attention_level.prototype.add_sorted_row = function(new_name, $new_row)
	{
		var self = this;
		var $grid = $("#" + this.att_levels);
		var new_sorted_key = this.make_sorted_key(new_name);
		var added = false;
		$grid.find("li[row_data]").each(function( _, row )
		{
			var $row = $(row);
			var id = $row.attr("attention_level_id");
			var att_level_data = self.get_attention_level_from_full_list(id);
			var current_sorted_key = self.make_sorted_key(att_level_data.name);
			if (new_sorted_key < current_sorted_key) {
				$row.before($new_row);
				added = true;
				return false; // break
			}
		});
		if (!added) {
			$grid.append($new_row);
		}
	};

	adm_form_campaigns_section_attention_level.prototype.has_permission_to_delete = function()
	{
		if ( this.has_perm_to_erase !== null ) {
			return this.has_perm_to_erase;
		}
		this.has_perm_to_erase = adm_form_campaigns_section_attention_level.uber.has_permission_to_delete.call( this );
		return this.has_perm_to_erase;
	};

	adm_form_campaigns_section_attention_level.prototype.get_row_users_desc = function( users_list )
	{
		var total = Object.keys( users_list || {} ).length;
		if ( total > 0 ) {
			return total;
		}
		else {
			return "<%=_T('No selected')%>";
		}
	};

	adm_form_campaigns_section_attention_level.prototype.on_account_update = function()
	{
		$("#" + this.att_levels).find("li[row_data]").remove();
		const no_hide_waiting = true;
		this.load_campaign_attention_levels(no_hide_waiting);
	};

	adm_form_campaigns_section_attention_level.prototype.on_account_delete = function()
	{
		this.on_account_update();
	};

	adm_form_campaigns_section_attention_level.prototype.get_row_accounts_desc = function( attention_level )
	{
		var campaign = this.frm.campaign;
		var assigned_accounts = 0;
		$.each(campaign.accounts || {}, function( _, account )
		{
			var account_att_levels = account.AttentionLevels || {};
			if ( typeof account_att_levels === "string" ) {
				account_att_levels = JSON.parse(account_att_levels);
			}
			$.each(account_att_levels, function( _, data )
			{
				if ( attention_level.id === data.attention_level_id ) {
					assigned_accounts++;
					return false; // break
				}
			});
		});
		if ( assigned_accounts === 0 ) {
			return "<%=_T('No assigned')%>";
		}
		return assigned_accounts;
	};

	adm_form_campaigns_section_attention_level.prototype.add_row = function(attention_level, is_new)
	{
		var self = this;
		if (!$.isPlainObject(attention_level)) {
			return;
		}
		var id = attention_level.id || inConcert.Utils.generateGuid();
		var name = this.frm.sanitize_value(attention_level.name || "");
		var description = this.frm.sanitize_value(attention_level.description || "");
		var $row = this.$get_row(id);

		var already_attention_level = $row.length > 0;
		if ( already_attention_level ) {
		// si el attention_level a agregar ya existe en la lista de la GUI me voy
			return;
		}
		var get_actions = function()
		{
			var has_perm_to_save  = self.has_permission_to_save();
			var has_perm_to_erase = self.has_permission_to_delete();
			if ( !has_perm_to_save && ! has_perm_to_erase ) {
				return "";
			}
			var erase_html = "";
			if ( has_perm_to_erase ) { 
				erase_html = "<button action_del='true' class='buttonColorGray gridActionButton delete last' title='"+ "<%=_T('Delete')%>" + "'><i class='buttons-icons-delete'></i>" + "<%=_T('Delete')%>" + "</button>";
			}
			var save_html = "";
			if ( has_perm_to_save ) {
				save_html = "<button action_edit='true' class='buttonColorGray gridActionButton edit' title='"+ "<%=_T('Edit')%>" + "'><i class='buttons-icons-edit'></i>" + "<%=_T('Edit')%>" + "</button>";
			}
			return "".concat( save_html, erase_html);
		};
		var $row_html = $("".concat(
			"<li row_data='true' attention_level_id='", id, "' class='viewListItem'>",
				"<label col_id='true' class='viewListItemText' title='",name,"'>", name, "</label>",
				"<label col_desc='true' class='viewListItemText' title='",description,"'>", description, "</label>",
				"<label col_users='true' class='viewListItemText'>", this.get_row_users_desc( attention_level.users ), "</label>",
				"<label col_accounts='true' class='viewListItemText'>", this.get_row_accounts_desc( attention_level ), "</label>",
				get_actions(),
			"</li>")
		);

		if (!is_new) {
			$("#" + this.att_levels).append($row_html);
		}
		else {
			this.add_sorted_row(name, $row_html); // agregar ordenada
		}
		// add attention_level in full hash
		this.add_attention_level_to_full_list(id, attention_level);
		// update message found
		this.show_or_hide_not_results_found();
	};

	adm_form_campaigns_section_attention_level.prototype.update_row = function( new_row_data )
	{
		if ( !$.isPlainObject( new_row_data ) ) {
			return;
		}
		var $current_row = this.$get_row( new_row_data.id );
		// name
		var $name = $( $current_row.find( this.selector_row_id ) );
		var old_name = $name.text().toLowerCase();
		var new_name = ( new_row_data.name || "" ).toLowerCase();

		// actualiza la lista completa de attention levels
		this.update_attention_level_on_full_list( new_row_data.id, new_row_data );
		$name.text( new_row_data.name );
		// celda description
		var $desc = $( $current_row.find( this.selector_row_desc ) );
		$desc.text( new_row_data.description );
		// celda users list
		var $users = $( $current_row.find( this.selector_row_users ) );
		$users.text( this.get_row_users_desc( new_row_data.users ) );
		// celda accounts list
		var $accounts = $( $current_row.find( this.selector_row_accounts ) );
		$accounts.text( this.get_row_accounts_desc( new_row_data ) );

		if ( old_name !== new_name ) {
			// si el Nombre cambio, tengo que cambiarlo de posicion dentro de la lista (alfabeticamente)
			var $row = $current_row;
			this.$row_editing.remove();
			this.add_sorted_row( new_name, $row );
		}
		this.show_or_hide_not_results_found();
	};

	adm_form_campaigns_section_attention_level.prototype.make_sorted_key = function(name)
	{
		return ( name || "" ).toLowerCase();
	};

	adm_form_campaigns_section_attention_level.prototype.show_not_results_found = function( message )
	{
		$("#" + this.att_levels_not_result_found)
			.text( message || "<%=_T('No attention levels found.')%>" )
			.show();

		$("#" + this.att_levels).hide();
	};

	adm_form_campaigns_section_attention_level.prototype.hide_not_results_found = function()
	{
		$("#" + this.att_levels).show();
		$("#" + this.att_levels_not_result_found).hide();
	};

	adm_form_campaigns_section_attention_level.prototype.show_or_hide_not_results_found = function()
	{
		var count = Object.keys( this.all_attention_levels || {} ).length;
		if ( count > 0 ) {
			this.hide_not_results_found();
		}
		else {
			this.show_not_results_found();
		}
	};

	adm_form_campaigns_section_attention_level.prototype.set_attention_levels = function(attention_levels, no_hide_waiting)
	{
		var self = this;

		$("#" + this.att_levels).find("li[row_data]").remove();
		this.all_attention_levels = {};

		if (!$.isPlainObject(attention_levels)) {
			this.show_or_hide_not_results_found();
			return;
		}

		var sorted_attention_level = inConcert.Utils.hash_to_sorted_array(attention_levels,
			function(attention_level) {
				return self.make_sorted_key(attention_level.name);
			}) || {};

		if ( sorted_attention_level.length > 0 ) {
			$.each(sorted_attention_level, function( _, item )
			{
				item.id = item.Id;
				delete item.Id; // la funcion hash to sorted array, mete el id, lo pelo porque aca no lo necesito.
				self.add_row(item);
			});
		}
		this.show_or_hide_not_results_found();
		if (!no_hide_waiting) {
			this.frm.dashboard.hide_waiting();
		}
	};

	adm_form_campaigns_section_attention_level.prototype.after_save_attention_levels = function(success, message_error)
	{
		var self = this;
		var attention_level_list = this.list_to_save;

		if (success) {
			if (this.$row_editing) {
			// update actual
				$.each(attention_level_list, function( _, att_level_info )
				{
					self.update_row( att_level_info.attention_level );
				});
			}
			else {
			// add all new
				var is_new = true;
				$.each(attention_level_list, function( _, att_level_info )
				{
					if ( att_level_info.is_new ) {
						self.add_row( att_level_info.attention_level, is_new );
					}
					else {
						self.update_row( att_level_info.attention_level );
					}
				});
			}
			this.empty_list_to_save();
			this.$row_editing = null;
			var message = "<%=_T('Attention levels have been saved successfully.')%>";
			this.frm.show_status_message(
				message ,                /* mensaje          */
				"Info"  ,                /* tipo de mensaje  */
				3000                     /* timeout ms       */);
		}
		else {
			this.frm.show_status_message(
				message_error,           /* mensaje                                            */
				"Error",                 /* tipo de mensaje                                    */
				false,                   /* timeout ms (false = inifito)                       */
				false,                   /* scroll to top (false = no se mueve)                */
				{                        /* muestra la accion personalizada (en este caso retry) */
					message: "<%=_T('Retry')%>",
					callback: function(){
						self.save_attention_levels();
					}
				}
			);
		}
	};

	adm_form_campaigns_section_attention_level.prototype.save_attention_levels = function()
	{
		var self = this;
		this.hide_attention_level_frm_edition();

		var attention_level_list = this.list_to_save;
		if ( $.isEmptyObject( attention_level_list ) ) {
			return;
		}
		this.saving = true;

		this.frm.dashboard.show_waiting("<%=_T('Please wait until attention levels are saved.')%>");
		// lo meto dentro de un timeout minimo, para que se pueda ver el waiting, ya que si la grilla tiene muchos registros
		//	parece que queda la pantalla congelada, porque no da el tiempo para que se vea el waiting
		window.setTimeout(function()
		{
			var attention_levels = [];
			$.each(attention_level_list, function(id, att_level_info)
			{
				var attention_level = att_level_info.attention_level;
				attention_level.id = attention_level.id || id;
				attention_levels.push(attention_level);
			});

			self.frm.campaign.saveAttentionLevels(attention_levels, function(error)
			{
				var success = !error;
				var message_error;
				if ( !success ) {
					if (typeof error === "object") {
						error = ( error.result || "").toUpperCase();
					}
					switch (error)
					{
						case "INVALID_REQUEST":
							message_error = "<%=_T('Invalid request.')%>";
							break;

						case "ERROR_ALS_EXCEEDS_LIMIT_ALLOWED":
							const limitAls = inConcert.app.getMaxConfigurationsAllowed("MaxAttsLevel");
							message_error = format("<%=_T('Can not create attention levels, exceeds limit allowed {0}.')%>", limitAls);
							break;
						
						case "ERROR_ALS_USERS_EXCEEDS_LIMIT_ALLOWED":
							const limitAlsUsers = inConcert.app.getMaxConfigurationsAllowed("MaxAttsLevelUsers");
							message_error = format("<%=_T('The total number of users assigned to the att. levels exceeds the limit allowed {0}.')%>", limitAlsUsers);
							break;

						case "CAMPAIGN_DOES_NOT_EXIST":
							message_error = "<%=_T('Unable to save campaign attention levels, campaign does not exist...')%>";
							break;

						case "FORBIDDEN":
							message_error = "<%=_T('You dont have permissions for save campaign attention levels.')%>";
							break;

						case "PENDING_INT":
							message_error = "<%=_T('Attention levels users have been updated but some users cannot be removed because they have active interactions.')%>";
							break;

						default:
							message_error = "<%=_T('Failed to save attention levels, please try again in a few minutes...')%>";
					}
				}
				self.after_save_attention_levels( success, message_error );
				self.saving = false;
			});
		}, 50);
	};

	adm_form_campaigns_section_attention_level.prototype.remove_attention_levels = function(attention_levels, callback)
	{
		var self = this;
		callback = callback || $.noop;

		if ( !$.isArray( attention_levels ) || attention_levels.length === 0 ) {
			callback(false);
			return;
		}

		this.frm.dashboard.show_waiting("<%=_T('Please wait until attention levels are deleted.')%>");
		// lo meto dentro de un timeout minimo, para que se pueda ver el waiting, ya que si la grilla tiene muchos registros
		//	parece que queda la pantalla congelada, porque no da el tiempo para que se vea el waiting
		window.setTimeout(function()
		{
			self.frm.campaign.delete_attention_levels(attention_levels, function(error)
			{
				var success = !error;
				var message, type, timeout_ms;

				if ( success ) {
					message = "<%=_T('Attention levels have been deleted successfully.')%>";
					type = "Info";
					timeout_ms = 3000;
				}
				else {
					if (typeof error !== "object") {
						error = {result: error};
					}

					var err_info;
					var load_account_al_desc = function (error) {
						var account = self.frm.campaign.getAccount(error.account_id);
						err_info = {
							account: self.frm.getAccountDescription({account: account, message_type: "WITH_PROXY"}),
							al: self.get_attention_level_from_full_list(error.att_level_id).name
						};
					};

					switch ((error.result || "").toUpperCase())
					{
						case "INVALID_REQUEST":
							message = "<%=_T('Invalid request.')%>";
							break;
						case "CAMPAIGN_DOES_NOT_EXIST":
							message = "<%=_T('Unable to delete attention levels. Campaign does not exist...')%>";
							break;
						case "FORBIDDEN":
							message = "<%=_T('You dont have permissions for delete attention levels.')%>";
							break;
						case "NO_CONFIGURED_ATTENTION_LEVELS":
							message = "<%=_T('Unable to delete attention levels. No configured attention leveles in campaign.')%>";
							break;
						case "NO_ATTENTION_LEVELES_TO_DELETE":
							message = "<%=_T('Unable to delete attention levels. No exist assigned attention leveles.')%>";
							break;
						case "ATTENTION_LEVEL_IN_USE_IN_ACCOUNT":
							load_account_al_desc(error);
							message = format("<%=_T('Unable to delete attention level. {0} distribution attention level is used on {1} account.')%>", err_info.al, err_info.account);
							break;
						case "ATTENTION_LEVEL_WITH_INTERACTIONS_IN_WAITING":
							load_account_al_desc(error);
							if (error.number_waiting_conversations === 1) {
								message = format("<%=_T('Unable to delete attention level. Attention level {0} in account {1} has {2} conversation waiting.')%>", err_info.al, err_info.account, error.number_waiting_conversations);
							}
							else {
								message = format("<%=_T('Unable to delete attention level. Attention level {0} in account {1} has {2} conversations waiting.')%>", err_info.al, err_info.account, error.number_waiting_conversations);
							}
							break;
						default:
							message = "<%=_T('Failed to delete attention levels, please try again in a few minutes...')%>";
					}
					type = "Error";
					timeout_ms = false; // infinito
				}

				self.frm.show_status_message(
					message,        /* mensaje          */
					type,           /* tipo de mensaje  */
					timeout_ms      /* timeout ms       */
				);
				callback( success );
			});
		}, 50);
	};

	adm_form_campaigns_section_attention_level.prototype.build_header_custom_buttons = function()
	{
		var html_buttons =
			"<button id='" + this.id_new_att_level + "' class='pageButtonGeneric buttonColorPrimary pageButtonNew'>" + "<%=_T('Create attention level')%>"  + "</button>";

		adm_form_campaigns_section_attention_level.uber.build_header_custom_buttons.call(this, html_buttons);
	};

	adm_form_campaigns_section_attention_level.prototype.add_or_update_attention_level = function()
	{
		var att_level_info = this.users_frm.validate();
		if ( !att_level_info ) {
			return;
		}
		var is_new = !this.$row_editing;
		this.update_list_to_save( att_level_info, is_new );
		this.save_attention_levels();
	};

	adm_form_campaigns_section_attention_level.prototype.show_attention_level_frm_edition = function()
	{
		var self = this;
		this.frm.hide_messages();

		var att_level_data;
		if ( this.$row_editing ) {
		// si presione el boton para editar
			var id = this.$row_editing.attr("attention_level_id");
			att_level_data = this.get_attention_level_from_full_list(id);
		}

		if ( !this.has_permission_to_section() || !this.has_permission_to_save() ) {
			var message;
			if ( !att_level_data ) {
				message =  "<%=_T('Unable to view attention level: You dont have permissions for this action.')%>";
			}
			else {
				message = format( "<%=_T('Unable to view {0} attention level: You dont have permissions for this action.')%>", att_level_data.name );
			}
			this.$row_editing = null;
			this.frm.show_status_message( message, "Error", 5000 );
			return;
		}

		this.empty_list_to_save();
		// show frm
		this.users_frm.show( att_level_data, function() {
			$("#" + self.id_list_container).hide();
			$("#" + self.id_new_att_level).hide();
		});

		this.set_edit_mode(true);
		this.frm.change_title_section(this.section_id, att_level_data?.name);
	};

	adm_form_campaigns_section_attention_level.prototype.before_jump_to_other_menunav = function(){
		if (this.users_frm) {
			this.users_frm.hide();
			this.frm.hide_messages();	
		}
	};

	adm_form_campaigns_section_attention_level.prototype.hide_attention_level_frm_edition = function()
	{
		this.users_frm.on_hide();
		$("#" + this.id_list_container).show();
		$("#" + this.id_new_att_level).show();
		if ( this.is_empty_list_to_save() ) {
			this.$row_editing = null;
		}
		this.set_edit_mode(false);
	};

	adm_form_campaigns_section_attention_level.prototype.set_events_frm_abm = function()
	{
		var self = this;
		// new message event
		var event_selector = "#" + this.id_new_att_level;
		var $header_buttons = this.frm.dashboard.$get_dom_element_by_id( "form_custom_buttons" ); //conf_form_custom_buttons
		$header_buttons
			.off("click", event_selector)
			.on("click", event_selector, function() {
				self.show_attention_level_frm_edition();
			});
	};

	adm_form_campaigns_section_attention_level.prototype.set_events = function()
	{
		var self = this;

		var $grid = $("#" + this.att_levels);
		var rows_event_selector = "li[row_data]";

		// boton delete de la lista de attention levels
		var event_selector =  rows_event_selector + " button[action_del]";
		$grid
			.off("click", event_selector)
			.on("click", event_selector, function()
			{
				self.frm.hide_messages();
				self.empty_list_to_save();
				self.$row_editing = null;
				var $row = $(this).parent();
				var id = $row.attr("attention_level_id");
				var attention_level = self.get_attention_level_from_full_list(id);
				var message = format(
					"<%=_T('Do you want to delete attention level {0} ?')%>",
					attention_level.name);

				$.inconcertPopup.Show(
					"<%=_T('Confirm')%>",
					message,
					$.inconcertPopup.ButtonSets.BS_YESNO,
					function(response) {
						if (response.toLowerCase() !== "yes") {
							return;
						}
						var attention_levels_list = [];
						attention_levels_list.push(attention_level.id);
						// envia a remover al server
						self.remove_attention_levels(
							attention_levels_list,
							function(success) {
								if (!success) {
									return;
								}
								// elimino del hash de attention levels
								self.erase_attention_level_on_full_list(attention_level.id);
								$row.fadeOut("slow", function() {
									$row.remove();
									self.show_or_hide_not_results_found();
								});
							});
					},
					undefined,
					{
						popup_id        : "conf_inconcertPopup",
						container_class : "confirmationPopUp",
						overlay_class   : "confirmationOverlay"
					}
				);
			});

		// doble click sobre la row edita la misma
		$grid
			.off("dblclick", rows_event_selector)
			.on("dblclick", rows_event_selector, function(event) {
				event.stopPropagation();
				self.$row_editing = $(this);
				self.show_attention_level_frm_edition();
			});

		// boton edit de la lista de parametros
		event_selector = rows_event_selector + " button[action_edit]";
		$grid
			.off("click", event_selector)
			.on("click", event_selector, function() {
				self.$row_editing = $(this).parent();
				self.show_attention_level_frm_edition();
			});

		// eventos del abm del frm
		this.set_events_frm_abm();
	};

	adm_form_campaigns_section_attention_level.prototype.load_campaign_attention_levels = function(no_hide_waiting)
	{
		var self = this;
		this.frm.campaign.getAttentionLevels(function(attention_levels, error)
		{
			if (error) {
				var message_type = "Error";
				var timeout_ms = false; // infinite
				self.frm.show_status_message("<%=_T('Error retrieving campaign attention levels.')%>", message_type, timeout_ms);
				return;
			}
			self.set_attention_levels(attention_levels, no_hide_waiting);
		});
	};

	adm_form_campaigns_section_attention_level.prototype.initialize = function()
	{
		this.has_perm_to_save  = null;
		this.has_perm_to_erase = null;
		this.users_frm = new users_frm( {
			parent: this
		});
		this.load_campaign_attention_levels();
	};

	adm_form_campaigns_section_attention_level.prototype.set = function()
	{
		this.initialize();
	};

	adm_form_campaigns_section_attention_level.prototype.on_show = function()
	{
		adm_form_campaigns_section_attention_level.uber.on_show.call(this, {show: false});
		this.frm.dashboard.$get_dom_element_by_id("form_buttons_actions").hide();
	};

	adm_form_campaigns_section_attention_level.prototype.on_hide = function()
	{
		adm_form_campaigns_section_attention_level.uber.on_hide.call( this );
		this.hide_attention_level_frm_edition();
	};

	adm_form_campaigns_section_attention_level.prototype.editable = function()
	{
		adm_form_campaigns_section_attention_level.uber.editable.call( this );
	};

	adm_form_campaigns_section_attention_level.prototype.readonly = function() {
		adm_form_campaigns_section_attention_level.uber.readonly.call( this );
	};

}());
