/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 9 */

/* global inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

//	const FORMAT = inConcert.Utils.format;
	const _T = inConcert.i18n.getString;

	const m_supported_address_type = {
		"MAIL" : {"MAIL": true},
		"PHONE": {"CALL": true, "SMS": true, "WHATSAPP": true}
	};

	//---------------------------------------------------------------------
	// form contact addresses book class
	function adm_form_contacts_addresses_book(params)
	{
		let success = adm_form_contacts_addresses_book.uber.constructor.call(this, params);
		if ( !success ) {
			return;
		}

		this.new_title      = "<%=_T('Create Contact Address')%>";
		this.update_title   = "<%=_T('Edit Contact Address {0}')%>";
		this.view_title     = "<%=_T('View Addresses Book {0}')%>";
		this.tagline        = "<%=_T('Tagline for Addresses Book')%>";

		this.html_file_name = "/inconcert/apps/administration/contacts/addresses_book/frm/render/";
		this.save_uri       = "/inconcert/apps/administration/contacts/addresses_book/save/";
		this.get_uri        = "/inconcert/apps/administration/contacts/addresses_book/get/";

		this.include_save_and_continue = true;

		this.telephone_codes = inConcert.app.getInstanceTelephonyCodes();
		this.sorted_countries = null;

		return true;
	}

	inConcert.Utils.inherit(adm_form_contacts_addresses_book, inConcert.conf_form);

	if ( typeof inConcert.adm_form_contacts_addresses_book !== "function" ) {
		inConcert.adm_form_contacts_addresses_book = adm_form_contacts_addresses_book;
	}

	adm_form_contacts_addresses_book.prototype.get = function()
	{
		let addresses_book = {
			Name       : $("#adm_form_contacts_addresses_book_name").val(),
			Description: $("#adm_form_contacts_addresses_book_desc").val(),
			Type       : $("#adm_form_contacts_addresses_book_address_type").val()
		};

		addresses_book.Channels = this.get_channels_by_address_type(addresses_book.Type);

		if (addresses_book.Type.toUpperCase() === "MAIL") {
			addresses_book.address = $("#adm_form_contacts_addresses_book_email_address_id").val();
		}
		else {
			addresses_book.Number  = $("#adm_form_contacts_addresses_book_phone_number").val();
			addresses_book.Country = $("#adm_form_contacts_addresses_book_country_code").val();
			let area = $("#adm_form_contacts_addresses_book_area_code").val();
			if (area !== "__BLANK") {
				addresses_book.Area = area;
			}
		}

		const action = this.dashboard.get_current_action_obj();
		const id = action.view.row_id;
		if (!this.is_new) {
			addresses_book[id] = this.$current_row.attr("row_id");
			const old_addresses_book = this.get_entity_instance();
			if (old_addresses_book) {
				addresses_book.OldAddressInfo = old_addresses_book;
			}
		}

		return addresses_book;
	};

	adm_form_contacts_addresses_book.prototype.get_channels_by_address_type = function(address_type = "") {
		let channels = [];
		address_type = address_type.toUpperCase();
		if (!m_supported_address_type[address_type]) {
			return channels;
		}
		const $channels = $("#adm_form_contacts_addresses_book_channels").find("[for_address='" + address_type + "']");
		$.each($channels, function(_, channel) {
			const $channel = $(channel);
			const $chk = $channel.find("[selector]");
			if ($chk.prop("checked")) {
				let channel_id = ($chk.attr("channel_id") || "").toUpperCase();
				channels.push(channel_id);
			}
		});
		return channels;
	};

	adm_form_contacts_addresses_book.prototype.set = function(success, addresses_book)
	{
		if (!success) {
			return;
		}
		$("#adm_form_contacts_addresses_book_name").val(addresses_book.Name).focus();
		$("#adm_form_contacts_addresses_book_desc").val(addresses_book.Description);
		$("#adm_form_contacts_addresses_book_address_type").val(addresses_book.Type).trigger("change");

		if (addresses_book.Type.toUpperCase() === "MAIL") {
			$("#adm_form_contacts_addresses_book_email_address_id").val(addresses_book.address);
		}
		else {
			$("#adm_form_contacts_addresses_book_phone_number").val(addresses_book.Number);
			$("#adm_form_contacts_addresses_book_country_code").val(addresses_book.Country);
			this.fill_combo_areas(addresses_book.Country);
			$("#adm_form_contacts_addresses_book_area_code").val(addresses_book.Area);
		}
	};

	adm_form_contacts_addresses_book.prototype.set_channels_by_address_type = function(addresses_book)
	{
		const $addresses_book_channels = $("#adm_form_contacts_addresses_book_channels");
		$addresses_book_channels.find("[selector]").prop("checked", false);

		const $channels_by_address_type = $("#adm_form_contacts_addresses_book_channels").find("[for_address='" + (addresses_book.Type.toUpperCase()) + "']");

		for (let i = 0; i < addresses_book.Channels.length; i++) {
			let channel = addresses_book.Channels[i].toUpperCase();
			let $chk = $channels_by_address_type.find("[channel_id='" + channel + "'");
			$chk.prop("checked", true);
		}
	};

	adm_form_contacts_addresses_book.prototype.empty_controls = function()
	{
		$("#adm_form_contacts_addresses_book_name").val("").focus();
		$("#adm_form_contacts_addresses_book_desc").val("");
		// email
		$("#adm_form_contacts_addresses_book_email_address_id").val("");
		// phone
		$("#adm_form_contacts_addresses_book_phone_number").val("");
		$("#adm_form_contacts_addresses_book_country_code").val("").trigger("change");
		$("#adm_form_contacts_addresses_book_area_code").val("");
	};

	adm_form_contacts_addresses_book.prototype.ready_for_save = function()
	{
		adm_form_contacts_addresses_book.uber.ready_for_save.call(this); //invokes the base class method
		this.only_modified();
	};

	adm_form_contacts_addresses_book.prototype.readonly = function()
	{
		adm_form_contacts_addresses_book.uber.readonly.call(this); //invokes the base class method
		this.only_modified();
		const $addresses_book_channels = $("#adm_form_contacts_addresses_book_channels");
		$addresses_book_channels.find("[selector]").prop("disabled", true);
	};

	adm_form_contacts_addresses_book.prototype.only_modified = function()
	{
		const $address_type = $("#adm_form_contacts_addresses_book_address_type");
		$address_type.trigger("change");
		let is_readonly;

		if (!this.is_new) {
			let addresses_book = this.get_entity_instance();
			this.set_channels_by_address_type(addresses_book);
			is_readonly = true;
		}
		else {
			is_readonly = false;
		}
		this.readonly_control($address_type, is_readonly, true);
		$("#adm_form_contacts_addresses_book_name").focus();
	};

	adm_form_contacts_addresses_book.prototype.err_save_messages = function(err)
	{
		let is_custom_message = false;

		const get_error_description = function(err_type)
		{
			is_custom_message = true;
			let message;
			switch (err_type) {
				case "ALREADY_EXIST_ADDRESSES_BOOK":
					is_custom_message = true;
					message = "<%=_T('Unable to save Address Book. Address already exist.')%>";
				break;
				case "INVALID_PHONE_NUMBER":
					is_custom_message = true;
					message = "<%=_T('Unable to save Address Book. Invalid phone number.')%>";
				break;
				default:
					is_custom_message = false;
					message = "<%=_T('Unable to save Address Book. Server error')%>";
			}
			return message;
		};

		switch (typeof err)
		{
			case "object":
				let error_type = err.result.toUpperCase();
				err = get_error_description(error_type);
			break;

			case "string":
				err = get_error_description(err.toUpperCase());
			break;
		}

		adm_form_contacts_addresses_book.uber.err_save_messages.call(this, err, is_custom_message); //invokes the base class method
	};

	adm_form_contacts_addresses_book.prototype.load_data = function(callback_complete_requests)
	{
		const self = this;

		this.fill_addresses_type([
			{id: "Mail" , desc: "<%=_T('E-mail')%>"},
			{id: "Phone", desc: "<%=_T('Phone')%>"}
		]);

		this.requests = [
			// Countries
			{
				request_function: function(success) {
					return self.telephone_codes.loadCountries(success);
				},
				success: function(countries) {
					self.fill_countries(countries);
				}
			}
		];

	 	adm_form_contacts_addresses_book.uber.load_data.call(this, callback_complete_requests); //invokes the base class method
	 };

	 adm_form_contacts_addresses_book.prototype.fill_addresses_type = function(list)
	 {
		 const $combo = $("#adm_form_contacts_addresses_book_address_type");
		 this.fill_combo($combo, false /* is_blank_first_item */, list, "id" /* attr_id */, "desc" /* attr_name */, false /* sorted */, undefined /*can_add_item*/, "trunk_type", /* data_id */);
	 };

	 adm_form_contacts_addresses_book.prototype.toggle_address_type = function(address_type = "")
	 {
		address_type = address_type.toUpperCase();

		const $channels_section = $("#adm_form_contacts_addresses_book_channels_section");
		const $channels_not_results_found = $("#adm_form_contacts_addresses_book_channels_not_results_found");
		const $email_address_panel = $("#adm_form_contacts_addresses_book_email_address_panel");
		const $phone_address_panel = $("#adm_form_contacts_addresses_book_phone_address_panel");


		if (!m_supported_address_type[address_type]) {
			$channels_section.hide();
			$channels_not_results_found.show();
			$email_address_panel.hide();
			$phone_address_panel.hide();
			return;
		}

		const $channels = $channels_section.find("[for_address");
		$.each($channels, function(_, channel) {
			const $channel = $(channel);
			const for_address = ($channel.attr("for_address") || "").toUpperCase();
			if (for_address === address_type) {
				$channel.show();
			}
			else {
				$channel.hide();
			}
		});
		$channels_section.show();
		$channels_not_results_found.hide();

		if (address_type === "MAIL") {
			$email_address_panel.show();
			$phone_address_panel.hide();
		}
		else {
			// PHONE
			$email_address_panel.hide();
			$phone_address_panel.show();
		}
	 };

	 adm_form_contacts_addresses_book.prototype.set_events = function()
	 {
		 const self = this;
	 
		 adm_form_contacts_addresses_book.uber.set_events.call(this); //invokes the base class method
 
		 const $address_type= $("#adm_form_contacts_addresses_book_address_type");
		 $address_type
			 .off("change")
			 .on("change", function() {
				 let address_type = $address_type.val();
				 self.toggle_address_type(address_type);
			 });

		const $country = $("#adm_form_contacts_addresses_book_country_code");
		$country.change(function() {
			const country_code =$country.val();
			window.setTimeout(function() {
				self.fill_combo_areas(country_code);
			}, 50);
		});
	 };

	 adm_form_contacts_addresses_book.prototype.fill_countries = function(countries)
	 {
		 const self = this;
 
		 var build_sorted_countries = function( countries_map ) {
			 if ($.isArray(self.sorted_countries)) {
				 return self.sorted_countries;
			 }
			 self.sorted_countries = [];
			 $.each(countries_map, function(country_id, country) {
				 self.sorted_countries.push( { code: country_id, name: _T(country.name) });
			 });
			 return inConcert.Utils.sortArray(self.sorted_countries, "name");
		 };
 
		 const $combo = $("#adm_form_contacts_addresses_book_country_code");
		 $combo.html("");
		 this.combo_add_value($combo, "", "", true);
 
		 let sorted_countries = build_sorted_countries( countries );
		 $.each( sorted_countries, function(_, country) {
			 self.combo_add_value($combo, country.code, country.name, false);
		 });
	 };

	 adm_form_contacts_addresses_book.prototype.__toggle_combo_areas = function(enable)
	 {
		const $area = $("#adm_form_contacts_addresses_book_area_code");
		const $area_container = $("#adm_form_contacts_addresses_book_area_code_container");
		if (enable) {
			$area.prop("disabled", false);
			$area_container.show();
		}
		else{
			$area.prop("disabled", true);
			$area_container.hide();
		}
	 };
 
	 adm_form_contacts_addresses_book.prototype.fill_combo_areas = function(country_code)
	 {
		const areas = this.telephone_codes.getAreasByCode(country_code);
		const $area = $("#adm_form_contacts_addresses_book_area_code");
		if ( !areas ) {
			$area.html("");
			this.__toggle_combo_areas(false);
			return;
		}
		const attr_name = "name";
		const attr_id = "code";
		
		this.fill_combo($area, false, areas, attr_id, attr_name);
		this.__toggle_combo_areas(Object.keys(areas).length > 1);
	 };

	 adm_form_contacts_addresses_book.prototype.get_validation_def = function()
	 {
		const self = this;

		const address_type = $("#adm_form_contacts_addresses_book_address_type").val().toUpperCase();
		var can_validate_mail_address_info = function() {
			if (address_type === "MAIL") {
				return true;
			}
			return false;
		};
		var can_validate_phone_address_info = function() {
			if (address_type === "PHONE") {
				return true;
			}
			return false;
		};

		const definitions = [
			{
				control_id: "adm_form_contacts_addresses_book_name",
				messages: {
					required: "<%=_T('Please enter Address Name.')%>"
				}
			},
			{
				control_id: "adm_form_contacts_addresses_book_desc",
				messages: {
					html_malicious_code: "<%=_T('Possible malicious code in the field value.')%>"
				}
			},
			{
				control_id: "adm_form_contacts_addresses_book_address_type",
				messages: {
					required: "<%=_T('Please select Address Type.')%>"
				}
			},

			// Channels Validation
			{
				control_id: "adm_form_contacts_addresses_book_channels",
				messages: {
					custom: {
						type : "custom",
						valid: function() {
							let response = {
								success: true
							};
							let channels = self.get_channels_by_address_type(address_type);
							if (channels.length === 0) {
								response.success = false;
								response.message = "<%=_T('Please select some Channel.')%>";
							}
							return response;
						}
					}
				}
			},

			// EMail Address Validation
			{
				control_id: "adm_form_contacts_addresses_book_email_address_id",
				can_validate: function() {
					return can_validate_mail_address_info();
				},
				messages: {
					required: "<%=_T('Please enter the e-mail Address.')%>",
					email   : "<%=_T('Please enter a Valid E-mail Address.')%>"
				}
			},
			// Phone Address Validation
			{
				control_id: "adm_form_contacts_addresses_book_phone_number",
				can_validate: function() {
					return can_validate_phone_address_info();
				},
				messages: {
					required: "<%=_T('Please enter the Phone Number Address.')%>",
					digits  : "<%=_T('Please enter a Valid Phone Number Address.')%>"
				}
			}
		];
		return definitions;
	};

	adm_form_contacts_addresses_book.prototype.on_saved = function(success, entity, after_save, err, callback, not_show_grid_message)
	{
		after_save = after_save || "SAVE_AND_REFRESH_VIEW";
		adm_form_contacts_addresses_book.uber.on_saved.call(this, success, entity, after_save, err, callback, not_show_grid_message); //invokes the base class method
	};

}());

