/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 11 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	var FORMAT = inConcert.Utils.format;

	//---------------------------------------------------------------------
	// form roles class
	function adm_form_roles(params)
	{
		var success = adm_form_roles.uber.constructor.call(this, params);
		if (!success) {
			return;
		}

		var self = this;
		this.new_title = "<%=_T('Create permission')%>";
		this.update_title = "<%=_T('Edit Permission {0}')%>";
		this.view_title = "<%=_T('View permission {0}')%>";
		this.tagline = "<%=_T('Tagline form Permission!!!')%>";

		this.html_file_name = "/inconcert/apps/administration/groups/frm/render/";
		this.save_uri = "/inconcert/apps/administration/groups/save/";
		this.get_uri = "/inconcert/apps/administration/groups/get/";

		this.requests = [];
		this.container_id  = "#adm_form_role_permission_list_2014";
		this.perm_gui = new inConcert.perm_gui({
			container_id: this.container_id,
			callback: function( data ) {
				self.perm_callback( data );
			}
		});
		this.is_system_role = null;
		this.default_system_group_permissions = null;
		this.m_external_filters = null;

		return true;
	}

	inConcert.Utils.inherit(adm_form_roles, inConcert.conf_form);

	if (typeof inConcert.adm_form_roles !== "function") {
		inConcert.adm_form_roles = adm_form_roles;
	}

	
	// callback de la clase de permisos cuando, se chequea / deschequea algun permiso ( action/app/grupo )
	adm_form_roles.prototype.perm_callback = function( data )
	{
		if ( data.type !== "mark_app" ) {
			return;
		}

		switch ( data.app_id )
		{
			case "administration":
				if (data.marked) {
					this.perm_gui.mark_specific_checkbox(false,"group","administration","admin_restricted","click");
				}
			
			break;

			default:
				return;
		}
	};

	adm_form_roles.prototype.get = function()
	{
		let externalUsersAuthType = inConcert.app.get_vcc_info().ExternalUsersAuthType;
		let externalUsersSyncMethod = inConcert.app.get_vcc_info().ExternalUsersSyncMethod;
		let externalAuthFilter;
		switch(externalUsersAuthType) {
			case "LDAP":
				externalAuthFilter = $("#adm_form_role_external_auth_filter").val();
				break;
			case "OPENID":
				if(externalUsersSyncMethod === "AZUREAD") {
					externalAuthFilter = { id: $("#adm_form_role_azuread_group").val(), displayName: $("#adm_form_role_azuread_group  option:selected").text() };
				}
				break;
		}

		var role = {
			Name:             $("#adm_form_role_name").val(),
			Description:      $("#adm_form_role_desc").val(),
			grouppermissions: this.perm_gui.get_permissions(),
			ExternalAuthFilter: externalAuthFilter
		};
		var action = this.dashboard.get_current_action_obj();
		var id = action.view.row_id;
		if ( !this.is_new ) {
			role[ id ] = this.$current_row.attr( "row_id" );
		}
		return role;
	};

	adm_form_roles.prototype.set = function( success, role )
	{
		if ( !success ) {
			return;
		}
		$( "#adm_form_role_name" ).val( role.Name );
		$( "#adm_form_role_desc" ).val( role.Description );

		if ( role.IsSystemGroup === "1" ) {
			this.show_system_ctrls( role.IsModified );
			this.is_system_role = true;
			this.default_system_group_permissions = role.default_system_group_permissions;
		}
		else {
			this.show_not_system_ctrls();
			this.is_system_role = false;
		}

		this.perm_gui.mark_actions( role.grouppermissions );

		if(inConcert.app.HasExternalUsersSync()) {
			let externalUsersSyncFilters = this.external_group_filters;
			let externalUsersAuthType = inConcert.app.get_vcc_info().ExternalUsersAuthType;
			let externalUsersSyncMethod = inConcert.app.get_vcc_info().ExternalUsersSyncMethod;
			let external_auth_filter_obj = externalUsersSyncFilters[this.is_new ? "" : role.Id];
			switch(externalUsersAuthType) {
				case "LDAP":
					$("#adm_form_role_external_auth_filter").val(external_auth_filter_obj);
					break;

				case "OPENID":
					if(!this.is_new && externalUsersSyncMethod === "AZUREAD" && external_auth_filter_obj) {
						let $option = $("<option value='" + external_auth_filter_obj.id + "' selected='selected' >" + external_auth_filter_obj.displayName + "</option>");
						$("#adm_form_role_azuread_group").append($option);
					}
					break;					
			}		
		}

		if ( this.is_new ) {
			return;
		}

		if (role.grouppermissions.administration && role.grouppermissions.administration.admin_restricted) {
			// Disable the checkboxes for  administration app / view users section and his childs.
			this.perm_gui.disable_specific_group(
				true             , /* disabled */
				"administration" , /* app */
				"viewusers"     , /* group id */
				true               /* aplicar cambios a todos los hijos */
			);
			// Disable the checkboxes for  administration app / view permissions section and his childs.
			this.perm_gui.disable_specific_group(
				true             , /* disabled */
				"administration" , /* app */
				"viewgroups"     , /* group id */
				true               /* aplicar cambios a todos los hijos */
			);
		}
	};

	adm_form_roles.prototype.empty_controls = function()
	{
		$("#adm_form_role_name").val("").focus();
		$("#adm_form_role_desc").val("");
		$("#adm_form_role_system_groups option[value='blank']").prop("selected", true);
		var checked = false;
		this.perm_gui.unmark_all_actions( checked );
		this.is_system_role = false;
	};

	adm_form_roles.prototype.readonly = function()
	{
		adm_form_roles.uber.readonly.call( this );  //invokes the base class method
		this.perm_gui.readonly();
	};

	adm_form_roles.prototype.err_save_messages = function(err)
	{
		var is_custom_message = false;
		var get_error_description = function(err_type, raw_error)
		{
			is_custom_message = true;
			var message;
			switch (err_type)
			{
				case "MUST_HAVE_A_PERMISSIONS":
					message = "<%=_T('Please select some permissions.')%>";
				break;

				case "NAME_ALREADY_EXISTS_IN_SYSTEM_PERMISSIONS":
					message = "<%=_T('Name already exist in system permissions.')%>";
				break;

				case "NO_USER_HAS_PERMISSIONS_TO_ASSIGN_PERMISSIONS":
					message = "<%=_T([[Unable to remove the App \'Administration.Users.Create/Modify\' permission. If you remove it, no User will be able to create or update Users.]])%>";
				break;

				case "SOME_USERS_HAS_INTERACTIONS_WITHOUT_PERMISSION_TO_INTERACTION_APP":
					message = FORMAT("<%=_T('{0} {1} has assigned conversations, without permission to interaction application.')%>",
						raw_error.count_users, raw_error.count_users === 1 ? "<%=_T('user')%>" : "<%=_T('users')%>"
					);
				break;

				case "ATTENTION_LEVELS_CONFLICT_USER_WITHOUT_PERMISSION_TO_INTERACTION_APP":
					message = FORMAT("<%=_T('Attention levels conflict. {0} {1} without permission to interaction application.')%>",
						raw_error.count_users, raw_error.count_users === 1 ? "<%=_T('user')%>" : "<%=_T('users')%>"
					);
				break;

				case "ALLEGRO_INTEGRATION_ERROR":
					message = "<%=_T('Error updating information on Allegro integration...')%>";
				break;

				case "HELPDESK_INTEGRATION_ERROR":
					const vcc_id = inConcert.app.get_vcc_id();
					const message_error = err.message_error.replace("@"+vcc_id,"")
					message = "<%=_T('Error updating information on HelpDesk integration...')%>" + message_error;
				break;

				case "UNABLE_TO_UPDATE_SYSTEM_ROLE_PERMISSIONS":
					message = "<%=_T([[Unable to update system role permissions.]])%>";
				break;

				default:
					is_custom_message = false;
					message = "<%=_T('Server error!!')%>";
			}
			return message;
		};

		switch (typeof err)
		{
			case "object":
				var error_type = err.result.toUpperCase();
				if (error_type === "MUST_HAVE_A_NAME" || error_type === "MUST_HAVE_UNIQUE_NAME") {
					$("#adm_form_role_name").focus();
				}
				else if (err.result === "MUST_HAVE_A_DESCRIPTION") {
					$("#adm_form_role_desc").focus();
				}
				else {
					err = get_error_description(error_type, err.raw_error);
				}
			break;

			case "string":
				err = get_error_description(err.toUpperCase());
			break;
		}

		adm_form_roles.uber.err_save_messages.call( this, err, is_custom_message );  //invokes the base class method
	};

	adm_form_roles.prototype.get_validation_def = function()
	{
		var self = this;
		var definitions = [
			{
				control_id: "adm_form_role_name",
				messages: {
					required: "<%=_T('Please enter the Permission Name.')%>",
					id:    "<%=_T('Only letters, numbers, dashes and dots are allowed in the Permission Name.')%>"
				}
			},
			{
				control_id: "adm_form_role_desc",
				messages: {
					required: "<%=_T('Please enter the Permission description.')%>"
				}
			},
			{
				control_id: "adm_form_role_permission_list_2014",
				messages: {
					custom: {
						type:  "custom",
						valid: function() {
							return self.perm_gui.validate( "marked-all-actions" );
						}
					}
				}
			},
			{
				control_id: "adm_form_role_external_auth_filter",
				can_validate: function() { return inConcert.app.get_vcc_info().ExternalUsersAuthType === "LDAP"; },
				messages: {
					required: "<%=_T('Please enter the External Authorization filter.')%>",
					custom: {
						type:  "custom",
						valid: function() {
							let response = {
								success: true
							};	
							if(self.is_new && inConcert.app.HasExternalUsersSync()) {
								let selectedRol = $("#adm_form_role_name").val();
								let selectedFilterValue = $("#adm_form_role_external_auth_filter").val();
								for (var [key, value] of Object.entries(self.external_group_filters)) {
									if(value === selectedFilterValue && selectedRol !== key) {
										response.success = false;
										response.message = FORMAT("<%=_T('Selected filter is in use by role {0}')%>", key);	
										return response;
									}
								}
							}
							return response;
						}
					}
				}
			},
			{
				control_id: "adm_form_role_azuread_group",
				can_validate: function() { return inConcert.app.get_vcc_info().ExternalUsersAuthType === "OPENID" && inConcert.app.get_vcc_info().ExternalUsersSyncMethod === "AZUREAD"; },
				messages: {
					required: "<%=_T('Please enter the Azure AD Group.')%>",
					custom: {
						type:  "custom",
						valid: function() {
							let response = {
								success: true
							};	
							if(self.is_new && inConcert.app.HasExternalUsersSync()) {
								let selectedRol = $("#adm_form_role_name").val();
								let externalUsersAuthType = inConcert.app.get_vcc_info().ExternalUsersAuthType;
								let externalUsersSyncMethod = inConcert.app.get_vcc_info().ExternalUsersSyncMethod;
								if(externalUsersAuthType !== "OPENID" || externalUsersSyncMethod !== "AZUREAD") {
									return response;
								}
								let selectedFilterValue = $("#adm_form_role_azuread_group").val();
								for (var [key, value] of Object.entries(self.external_group_filters)) {
									if(value.id === selectedFilterValue && selectedRol !== key) {
										response.success = false;
										response.message = FORMAT("<%=_T('Selected Azure AD Group is in use by role {0}')%>", key);	
										return response;
									}
								}
							}
							return response;
						}
					}
				}
			}
		];
		return definitions;
	};

	adm_form_roles.prototype.save = function( callback )
	{
		var self = this;
		this.validate_override(
			true, /* include client permissions */
			function( override_action, overrides ) {
				var entity;
				if ( override_action ) {
					entity = self.get();
					entity.__override = {
						type              : override_action,
						members_overrides : overrides
					};
				}
				adm_form_roles.uber.save.call( self, callback, entity );  //invokes the base class method
			}
		);
	};

	adm_form_roles.prototype.on_saved = function( success, entity, after_save, err )
	{
		if ( !this.is_new && this.is_system_role ) {
			entity.IsSystemGroup = "1";
			if ( entity.IsModified === undefined ) {
				var differences = this.perm_gui.get_overrides( this.default_system_group_permissions );
				entity.IsModified = differences.has_override;
			}
		}
		adm_form_roles.uber.on_saved.call( this, success, entity, after_save, err ); //invokes the base class method
		inConcert.app.loadAllRoles(); // reload los cacheados
	};

	adm_form_roles.prototype.validate_override = function( has_include_permissions, callback_save )
	{
		var self = this;

		callback_save = callback_save || $.noop;

		var role = $("#adm_form_role_name").val();

		var callback = function( success, response )
		{

			if ( !success ) {
				self.dashboard.hide_waiting();

				var message;
				var error_type = ( response.result || "" ).toUpperCase();

				if ( error_type === "FORBIDDEN" ) {
					message = FORMAT( "<%=_T('Error saving {0} permissions. You do not have permission.')%>", role );
				}
				else {
					message = FORMAT( "<%=_T('Error saving {0} permissions. Validating override permissions.')%>", role );
				}

				var timeout_ms = false; // infinite
				self.show_status_message( message, "Error", timeout_ms );
				return;
			}

			if ( !response.override_changed ) {
				callback_save();
				return;
			}
			//callback_save
			$.inconcertPopup.Show(
				"<%=_T('Confirm')%>",
				"<%=_T('Some overrides already exists in some campaign users, which belongs to new permissions.<br>Remove or keep overrides ?')%>",
				null,
				null,
				[
					{
						label: "<%=_T('Remove')%>",
						onClick: function() {
							callback_save( "REMOVE", response.override_actions_changed );
						},
						className: "buttonColorBlack pageButtonGeneric" 
					},
					{
						label: "<%=_T('Keep')%>",
						onClick: function() {
							callback_save( "KEEP", response.override_actions_changed );
						},
						className: "buttonColorBlack pageButtonGeneric" 
					},
					{
						label: "<%=_T('Cancel')%>",
						onClick: function() { /* cancela */ },
						className: "buttonColorGray pageButtonGeneric" 
					}
				],
				{
					popup_id        : "conf_inconcertPopup",
					container_class : "confirmationPopUp",
					overlay_class   : "confirmationOverlay"
				}
			);

		};

		var data = {
			group: this.get()
		};

		if ( !has_include_permissions ) {
			data.group.grouppermissions = undefined;
		}

		if ( this.is_creating() ) {
			var action = this.dashboard.get_current_action_obj();
			data.group.Id = data.group[ action.view.display_name ];
		}

		var api = this.dashboard.get_api();
		// se utiliza el mismo metodo de "save" ya que es lo mismo. Un post generico que realiza el request
		api.save({
			uri      : inConcert.app.getBaseDomain() + "/inconcert/apps/administration/groups/validate_override/",
			data     : data,
			callback : callback
		});
	};

	adm_form_roles.prototype.restore_default = function()
	{
		var self = this;

		var restore = function( entity ) {
			var role = $("#adm_form_role_name").val();
			self.dashboard.show_waiting( FORMAT( "<%=_T('Please wait, restoring default permissions to {0} system permissions.')%>", role ) );

			var callback = function( success/*, response*/ )
			{
				var message;
				var timeout_ms = 5000;

				if ( success ) {
					entity.IsModified = "0"; // si se restauran los valores x default queda como no modificado
					self.on_saved( true, entity );
					self.dashboard.hide_waiting();
					self.close();
					message = FORMAT( "<%=_T('Have been successfully restored {0} system permissions.')%>", role );
					self.show_status_message( message, "Info", timeout_ms );
					return;
				}

				self.dashboard.hide_waiting();
				message = FORMAT( "<%=_T('Error restoring {0} system permissions.')%>", role );
				self.show_status_message( message, "Error", timeout_ms );
			};

			var data = {
				group: entity
			};
			data.group.grouppermissions = undefined;

			var api = self.dashboard.get_api();
			// se utiliza el mismo metodo de "save" ya que es lo mismo. Un post generico que realiza el request
			api.save({
				uri     : inConcert.app.getBaseDomain() + "/inconcert/apps/administration/groups/restore_default/",
				data    : data,
				callback: callback
			});
		};

		this.validate_override(
			false, /* include client permissions */
			function( override_action, overrides ) {
				var entity = self.get();
				if ( override_action ) {
					entity.__override = {
						type              : override_action,
						members_overrides : overrides
					};
				}
				restore( entity );
			}
		);
	};

	// Esta funcion carga el combo de system groups
	adm_form_roles.prototype.fill_system_groups = function( system_groups )
	{
		var $system_groups = $("#adm_form_role_system_groups");
		if (typeof system_groups !== "object") {
			$system_groups.html("");
			return;
		}
		// funcion que agrega una opcion en el combo
		var add_option = function(value, name, permissions, selected)
		{
			var html_selected = "";
			if (selected) {
				html_selected = " selected='selected' ";
			}
			var $option = $("<option value='" + value + "' " + html_selected + ">" + name + "</option>");
			$option.data("permissions", permissions);
			$system_groups.append($option);
		};

		// primero agrego las opciones "none" y "full". Para que desmarquen y marquen todas las opciones en los checkboxs.
		var selected = true;
		add_option("blank", "", "none", selected);
		add_option("none", "<%=_T('None')%>", "none");
		add_option("full", "<%=_T('Full')%>", "full");

		// funcion que valida si el grupo tiene algun permiso (accion).
		// si no tiene ninguna, retorna false para que el grupo no aparezca en el combo
		var has_permissions = function(group)
		{
			if (typeof group !== "object" || typeof group.grouppermissions !== "object") {
				return false;
			}
			var count = 0;
			$.each(group.grouppermissions, function(/* app_id, permission */)
			{
				count++;
			});
			return count > 0;
		};

		// el hash de grupos se pasa a un array ordenado alfabeticamente, y se filtra segun la funcion "has_permissions"
		var sorted_system_groups = inConcert.Utils.hash_to_sorted_array(system_groups,
			function(group) {
				return group.Name? group.NameLabel : group.IdLabel;
			},
			has_permissions);

		// ahora se recorre el array ordenado y se agregan los items al combo.
		$.each(sorted_system_groups, function( _, group )
		{
			var name = group.Name? group.NameLabel : group.IdLabel;
			add_option(group.Id, name, group.grouppermissions);
		});
	};

	adm_form_roles.prototype.loadExternalAuthoritationEntities = function(callback = $.noop)
	{
		inConcert.invoke({
			url: inConcert.app.getBaseDomain() + "/inconcert/apps/administration/security/get_vcc_externalentities/",
			dataType: "json",
			success: function( response )
			{
				callback(response.success, response.result || response.error);
			},
			error: function(_, response)
			{
				callback(false, response.error);
			}
		});
	};

	adm_form_roles.prototype.load_data = function(callback_complete_requests)
	{
		var self = this;

		self.requests = [
			{
				// esto tiene que estar en una funcion dentro de otra, porque sino
				//	dentro de "load_apps_actions, no accede al this que corresponde.
				request_function: function(success) {
					//realiza un return del resultado del air.invoke.
					//	el mismo es utilizado por si se desea cancelar el request.
					return inConcert.permissions.load_apps_actions(success);
				},
				success: function(application_actions) {
					self.perm_gui.build_apps_permissions(application_actions);
				}
			},
			{
				request_function: function(success, response) {
					return inConcert.app.loadSystemGroups(success, response);
				},
				success: function(system_groups) {
					self.fill_system_groups( system_groups );
				}
			}
		];

		if(inConcert.app.HasExternalUsersSync()) {
			self.requests.push(
				{
					request_function: function(success, response) {
						return inConcert.app.getExternalGroupFilters(success, response);
					},
					success: function(response) {
						self.external_group_filters = response;
					}
				}
			);
			if (this.is_new) {
				self.requests.push(		
					{
						request_function: function(success, response) {
							return self.loadExternalAuthoritationEntities(success, response);
						},
						success: function(data) {
							self.fill_external_filters(data);
							self.fill_azuread_groups(data);
						},
						error: function(error_msg) {
							var timeout_ms = 5000;
							self.show_status_message(error_msg, "Error", timeout_ms);
						}
					}
				);
			}
		}

		adm_form_roles.uber.load_data.call( this, callback_complete_requests );  //invokes the base class method
	};

	adm_form_roles.prototype.set_events = function()
	{
		var self = this;

		adm_form_roles.uber.set_events.call( this );  //invokes the base class method

		// seteo los eventos del container de acciones
		self.perm_gui.set_events();

		// evento change del combo de system groups
		$( "#adm_form_role_system_groups" )
			.off( "change" )
			.on( "change", function( /* e */ ) {
				var $option_selected = $( "option:selected", this );
				var grouppermissions = $option_selected.data( "permissions" );
				// Este timeout es agregado para que no quede la opcion del combo congelada
				//	mientras se marcan las acciones de los checkboxs
				window.setTimeout( function() {
					self.perm_gui.disable_specific_group(false, "administration" , "viewusers" , true);
					self.perm_gui.disable_specific_group(false, "administration" , "viewgroups", true);
					self.perm_gui.mark_actions( grouppermissions );
				}, 100);
			});

		// evento click. restaura los default del grupo del sistema
		$( "#conf_form_buttons_actions" )
			.off("click", "#adm_form_role_restore_default" )
			.on( "click", "#adm_form_role_restore_default", function( /* e */ ) {
				self.restore_default();
			});

		$("#adm_form_role_external_auth_filter")
			.off("click")
			.on("click", function() {
				self.show_external_filters();
			});			
	};

	adm_form_roles.prototype.show_system_ctrls = function( is_modified_system_role )
	{
		this.readonly_control( $("#adm_form_role_name"), true /* disabled */ );
		this.readonly_control( $("#adm_form_role_desc"), true /* disabled */ );

		var $cancel = this.dashboard.$get_dom_element_by_id( "form_button_cancel" );
		$cancel.siblings( ".auxContainerButtons" ).remove();

		if ( !this.is_new && !this.is_readonly() && is_modified_system_role === "1" ) {
			var $restore = $( "<div class='auxContainerButtons2 mr-1'><input id='adm_form_role_restore_default' is_system_domain='true' class='pageButtonGeneric buttonColorGray' type='button' value='" + "<%=_T('Restore Default')%>" + "' / ></div>" );
			$cancel.before( $restore );
		}

		this.dashboard.$get_dom_element_by_id( "form_data_container div[is_system_domain='true']" ).show();
		this.dashboard.$get_dom_element_by_id( "form_data_container div[is_system_domain='false']" ).hide();
	};

	adm_form_roles.prototype.show_not_system_ctrls = function()
	{
		var readonly_name = false;
		if ( !this.is_new && inConcert.app.has_allegro_integration() ) {
		// IDEM al comentario (**)
			readonly_name = true;
		} 
		this.readonly_control( $("#adm_form_role_name"), readonly_name /* disabled */ , true /* is_required */ );
		this.readonly_control( $("#adm_form_role_desc"), false /* disabled */ , true /* is_required */ );

		$( "#adm_form_role_restore_default" ).remove();
		this.dashboard.$get_dom_element_by_id( "form_data_container div[is_system_domain='false']" ).show();
		this.dashboard.$get_dom_element_by_id( "form_data_container div[is_system_domain='true']" ).hide();
	};

	adm_form_roles.prototype.initialize = function()
	{
		this.show_not_system_ctrls();
		this.perm_gui.initialize();
	};

	adm_form_roles.prototype.ready = function()
	{
		this.initialize();
		adm_form_roles.uber.ready.call( this );  //invokes the base class method

		let hasExternalUsersSync = inConcert.app.HasExternalUsersSync();
		let externalUsersAuthType = inConcert.app.get_vcc_info().ExternalUsersAuthType;
		let externalUsersSyncMethod = inConcert.app.get_vcc_info().ExternalUsersSyncMethod;

		var showExternalAuthFilter = hasExternalUsersSync && externalUsersAuthType === "LDAP";
		var showExternalAzureAdGroup = hasExternalUsersSync && externalUsersAuthType === "OPENID" && externalUsersSyncMethod === "AZUREAD";

		$("#adm_form_role_external_auth_filter_section").toggle(showExternalAuthFilter);
		$("#adm_form_role_azuread_group_section").toggle(showExternalAzureAdGroup);
		
		var isRequiredExternalAuthFilter = showExternalAuthFilter && this.is_new;
		var isRequiredAzureAdGroup = showExternalAzureAdGroup && this.is_new;

		this.readonly_control( $("#adm_form_role_external_auth_filter"), !isRequiredExternalAuthFilter, isRequiredExternalAuthFilter );
		this.readonly_control( $("#adm_form_role_azuread_group"), !isRequiredAzureAdGroup, isRequiredAzureAdGroup );

		var $name = $("#adm_form_role_name");
		if (!this.is_new) {
		// (**) Se deberia crear una nueva clase para que se implemente todo lo necesario de AllegroIntegraton!!!
		// Pero, como por ahora es lo unico que es diferente, realizo este IF en su lugar, Si hoy o manana hay que realizar mas cambios,
		//   entonces si o si, hay que crear un nuevo js/clase que herede de esta, e implementar lo que sea necesario para AllegroIntegration. He dicho!!!
			if ( inConcert.app.has_allegro_integration() ) {
				this.readonly_control($name, true);
				$("#adm_form_role_desc").focus();
				return;
			}
		}

		$name.focus();
	};

	adm_form_roles.prototype.fill_external_filters = function(entities) {
		var self = this;
		// entities es un objeto cuyas propiedades son las entradas de ldap, desordenadas y a un mismo nivel
		// La clase GenericTreePopup espera los elementos como un array, en el que cada elemento tendría un campo string "name" y un campo array "childs"
		this.m_external_filters = [];

		// Obtiene un array anidado, en forma de arbol, con toda la descendencia de una entrada de ldap
		var getChilds;
		getChilds = function(nodeLevels) {
			if(nodeLevels.length > 0) {
				let node = {
					name: nodeLevels[nodeLevels.length - 1]
				};
				nodeLevels.pop();
				let childs = getChilds(nodeLevels);
				if(childs) {
					node.childs = [];
					node.childs.push(childs);
				}
				return node;
			}
		};

		//Inserta el primer nodoque no exista ya en arr, junto con sus hijos
		let insertFirstNonExistingNode;
		insertFirstNonExistingNode = function(arr, node) {
			let indexFound = arr.findIndex(e => e.name === node.name);
			if(indexFound >= 0) {
				if(arr[indexFound].childs && node.childs && node.childs[0]) {
					insertFirstNonExistingNode(arr[indexFound].childs, node.childs[0]);
				}
			}
			else {
				arr.push(node);
			}
		};

		// Hacer el reverse de cada propiedad de entities para poder ordenar 
		let reversedResult = {};
		for (const property in entities) {
			if (entities.hasOwnProperty(property)) {
				reversedResult[property.split("").reverse().join("")] = entities[property];
			}
		}

		// Obtenemos un objeto con sus properties ordenadas descendente, para que aparezcan los descendientesde nivel más profundo antes que su progenitor
		let sortedResult = {};
		Object.keys(reversedResult).sort(function (a, b) {
			if (a < b) {
				return 1;
			}
			if (a > b) {
				return -1;
			}
			// a must be equal to b
			return 0;
		}).map(i=>sortedResult[i]=entities[i]);

		for (const itemReversed in sortedResult) {
			if (sortedResult.hasOwnProperty(itemReversed)) {
				//Deshacemos el reverse una vez ordenado el resultado
				let item = itemReversed.split("").reverse().join("");
				// Extraer del item el numero de separadores (n) de path de ldap tiene (,). El node debe ir en el subnivel n del array
				let nodeLevels = item.split(",");
				let node = {
					name: nodeLevels[nodeLevels.length - 1]
				};
				nodeLevels.pop();
				let childs = getChilds(nodeLevels);
				if(childs) {
					node.childs = [];
					node.childs.push(childs);
				}
				insertFirstNonExistingNode(self.m_external_filters, node);
			}
		}	
	};

	adm_form_roles.prototype.show_external_filters = function()
	{
		var self = this;
		if(this.is_new && inConcert.app.HasExternalUsersSync()) {
			if(!Array.isArray(self.m_external_filters) || self.m_external_filters.length === 0) {
				return;
			}
			const externalFiltersTreePopup = new inConcert.GenericTreePopup();

			externalFiltersTreePopup.show({
				title: "<%=_T('Select a LDAP level')%>",
				parentNode:       $("#adm_form_role_external_auth_filter"),
				dispositionCodes: self.m_external_filters,
				permissionToComment: false,
				callbacks: {
					on_finish : function( response ) {
						if (response && response.disposition && response.disposition.treePath && Array.isArray(response.disposition.treePath)) {
							// Insertar en el filter correspondiente lo seleccionado en el arbol
							let selectedFilterValue = "";
							response.disposition.treePath.slice().reverse().forEach(element => selectedFilterValue+= (selectedFilterValue.length ? "," : "") + element);
							$("#adm_form_role_external_auth_filter").val(selectedFilterValue);
						}
					},
					onOpen: $.noop,
					onClose: $.noop
				}
			});
		}
	};

	// Esta funcion carga el combo de Azure AD groups
	adm_form_roles.prototype.fill_azuread_groups = function(groups)
	{	
		const $azuread_group = $("#adm_form_role_azuread_group");
		if(groups && inConcert.app.get_vcc_info().ExternalUsersSyncMethod === "AZUREAD") {
			this.fill_combo( $azuread_group, false, groups, "id", "displayName", true );
			return;
		}
		$azuread_group.html("");
	};

}());
