/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:false, browser:true, indent:false, maxerr:50, -W083, -W014, esversion: 11 */
/* global inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

//	"use strict";

	const FORMAT = inConcert.Utils.format;

	const jQUERY_SELECTOR                      = (n, v)=> `[${n}='${v}']`;
	const jQUERY_SELECTOR_LDAP_GROUP_FILTER    = (v)=> jQUERY_SELECTOR("LDAP_GROUP_FILTER", v);
	const jQUERY_SELECTOR_LDAP_GROUP_NAME      = (v)=> jQUERY_SELECTOR("LDAP_GROUP_NAME", v);
	const jQUERY_SELECTOR_AZUREAD_GROUP_FILTER = (v)=> jQUERY_SELECTOR("AZUREAD_GROUP_FILTER", v);

	//---------------------------------------------------------------------
	// form security class
	function adm_form_security(params)
	{
		const self = this;
		const success = adm_form_security.uber.constructor.call(this, params);
		if (!success) {
			return;
		}
		this.view_title        = "<%=_T('Security')%>";
		this.update_title      = "<%=_T('Security')%>";
		this.update_ok_message = "<%=_T('Security was successfully updated.')%>";
		this.html_file_name    = "/inconcert/apps/administration/security/frm/render/";

		this.telephone_codes = inConcert.app.getInstanceTelephonyCodes();
		this.countries_by_code = null;
		this.sorted_countries = null;

		// Instancia de la clase los TABs
		this.main_tabs = new inConcert.conf_form_tab({
			parent        : this,
			frm           : this.frm,
			container_id  : "adm_form_security_updating_tabs",
			must_intialize: false,
			tabs_list   : {
				security: {
					id              : "security",
					name            : "<%=_T('Security')%>",
					tab_container_id: "adm_form_security_updating_security_tab_container",
					selected        : true,
					on_show         : function() {
						self.on_show_tab("security");
					}
				}
			}
		});

		this.get_uri = "/inconcert/apps/administration/security/get/";
		this.requests = [];

		return true;
	}

	inConcert.Utils.inherit( adm_form_security, inConcert.conf_form );

	if ( typeof inConcert.adm_form_security !== "function" ) {
		inConcert.adm_form_security = adm_form_security;
	}

	adm_form_security.prototype.initialize = function()
	{
		this.main_tabs.initialize();
		this.main_tabs.show_tabs_container();
	};

	adm_form_security.prototype.get = function()
	{		
		this.get_security_captcha();
		this.get_security_authentication();

		return this.m_security;
	};

	adm_form_security.prototype.get_security_captcha = function()
	{
		const $captcha_enabled = $("#adm_form_security_captcha_enabled");
		this.m_security.captcha.enabled = $captcha_enabled.prop("checked") ? "1" : "0";
		if (this.m_security.captcha.enabled !== "1") {
			return;
		}

		const security_captcha_type = $("#adm_form_security_captcha_type").val();
		if (security_captcha_type !== "CUSTOM") {
			this.m_security.captcha.publicKey = "";
			this.m_security.captcha.secretKey = "";
			return;
		}

		const $public_key = $("#adm_form_security_captcha_public_key");
		const $secret_key = $("#adm_form_security_captcha_secret_key");
		this.m_security.captcha.publicKey = $public_key.val();
		this.m_security.captcha.secretKey = $secret_key.val();
	};

	adm_form_security.prototype.get_security_authentication = function()
	{
		var self = this;
		const type = $("#adm_form_security_authentication_type").val();
		this.m_security.authentication = {
			type : type
		};
		//BASIC
		if(type === "BASIC" || type === "MFA") {
			this.m_security.authentication.passwordChecks = {};
			this.m_security.authentication.passwordChecks.minLen = parseInt($("#adm_form_security_authentication_passwordChecks_minLen").val());
			this.m_security.authentication.passwordChecks.maxLen = parseInt($("#adm_form_security_authentication_passwordChecks_maxLen").val());
			this.m_security.authentication.passwordChecks.expirationDays = parseInt($("#adm_form_security_authentication_passwordChecks_expirationDays").val());
			this.m_security.authentication.passwordChecks.blockOnLoginFailures = parseInt($("#adm_form_security_authentication_passwordChecks_blockOnLoginFailures").val());
			this.m_security.authentication.passwordChecks.history = parseInt($("#adm_form_security_authentication_passwordChecks_history").val());
			this.m_security.authentication.passwordChecks.minUpperChars = parseInt($("#adm_form_security_authentication_passwordChecks_minUpperChars").val());
			this.m_security.authentication.passwordChecks.minLowerChars = parseInt($("#adm_form_security_authentication_passwordChecks_minLowerChars").val());
			this.m_security.authentication.passwordChecks.minSpecialChars = parseInt($("#adm_form_security_authentication_passwordChecks_minSpecialChars").val());
			this.m_security.authentication.passwordChecks.minNumericChars = parseInt($("#adm_form_security_authentication_passwordChecks_minNumericChars").val());
			this.m_security.authentication.passwordChecks.expirationNotificationDays = parseInt($("#adm_form_security_authentication_passwordChecks_expirationNotificationDays").val());
			this.m_security.authentication.passwordChecks.changeOnFirstLogin = $("#adm_form_security_authentication_passwordChecks_changeOnFirstLogin").prop("checked");
		}
		//MFA
		this.m_security.mfa = {
			enabled : type === "MFA" ? "1" : "0"
		};
		if (this.m_security.mfa.enabled === "1") {
			this.m_security.mfa.type = $("#adm_form_security_authentication_mfa_type").val();
			switch(this.m_security.mfa.type) {
				case "MAIL": {
					this.m_security.mfa.account = $("#adm_form_security_authentication_mfa_mail_account").val();
					this.m_security.mfa.keyLength = parseInt($("#adm_form_security_authentication_mfa_mail_length").val());
					this.m_security.mfa.expirationSeconds = 
						parseInt($("#adm_form_security_authentication_mfa_mail_expiration").val()) * 60;
					this.m_security.mfa.subject = $("#adm_form_security_authentication_mfa_mail_subject").val();
					const parts = this.m_mailControls.mfa.get_body_parts();
					this.m_security.mfa.body = JSON.stringify([ parts[0] ]);
					this.m_security.mfa.attachments = JSON.stringify(this.m_mailControls.mfa.get_repository_images_list());
					break;
				}
			}
		}
		//LDAP
		this.m_security.ldap = {
			enabled : type === "LDAP" ? "1" : "0"
		};
		if (this.m_security.ldap.enabled === "1") {
			this.m_security.ldap.server = $("#adm_form_security_authentication_ldap_server").val();
			this.m_security.ldap.adminUserDN = $("#adm_form_security_authentication_ldap_adminUserDN").val();
			this.m_security.ldap.useTLS = $("#adm_form_security_authentication_ldap_useTLS").prop("checked");
			this.m_security.ldap.baseDN = $("#adm_form_security_authentication_ldap_baseDN").val();
			this.m_security.ldap.userIdAttributte = $("#adm_form_security_authentication_ldap_userIdAttributte").val();
			
			const $password =  $("#adm_form_security_authentication_ldap_password"); 
			const jwtToken  = inConcert.Utils.GetJwtToken($password.val());
			this.m_security.ldap.adminPassword = jwtToken;

			const $ldap_panel = $("#adm_form_security_authentication_ldap_panel");

			this.m_security.ldap.sync_enabled = $("#adm_form_security_authentication_ldap_sync_enabled").prop("checked");
			this.m_security.ldap.groupFilters = {};
			if(this.m_security.ldap.sync_enabled) {
				this.m_security.ldap.timezone = $("#adm_form_security_authentication_ldap_sync_timezone").val();
				this.m_security.ldap.language = $("#adm_form_security_authentication_ldap_sync_language").val();
				this.m_security.ldap.country = $("#adm_form_security_authentication_ldap_sync_country").val();
				this.m_security.ldap.area = $("#adm_form_security_authentication_ldap_sync_area").val();
				this.m_security.ldap.default_campaign = $("#adm_form_security_authentication_ldap_sync_default_campaign").val();
				this.m_security.ldap.initial_state = $("#adm_form_security_authentication_ldap_sync_initial_state").val();

				$ldap_panel.find("[LDAP_ROW_GROUP_NAME]").each(function(_, item) {
					const $item = $(item);
					const group = $item.attr("ldap_row_group_name");
					if (group !== "allusers") {
						const $ldapGroupFilter = $item.find("[ldap_group_filter]");
						self.m_security.ldap.groupFilters[group] = $ldapGroupFilter.val();
					}
				});
			}
			else {
				const group ="allusers";
				const selector = jQUERY_SELECTOR_LDAP_GROUP_FILTER(group);
				const filter = $ldap_panel.find(selector).val();
				this.m_security.ldap.groupFilters[group] = filter;
			}
		}
		//OPENID
		this.m_security.openid = {
			enabled : type === "OPENID" ? "1" : "0"
		};
		if (this.m_security.openid.enabled === "1") {
			this.m_security.openid.openid_configuration_url = $("#adm_form_security_authentication_openid_configuration_url").val();
			this.m_security.openid.sync_method = $("#adm_form_security_authentication_openid_sync_method").val();
			if(this.m_security.openid.sync_method === "AZUREAD") {
				this.m_security.openid.client_id = $("#adm_form_security_authentication_azuread_sync_client_id").val();	

				const $password =  $("#adm_form_security_authentication_azuread_sync_client_secret"); 
				const jwtToken  = inConcert.Utils.GetJwtToken($password.val());
				this.m_security.openid.adminPassword = jwtToken;

				this.m_security.openid.sync_enabled = $("#adm_form_security_authentication_azuread_sync_enabled").prop("checked");
				this.m_security.openid.groupFilters = {};
				this.m_security.openid.timezone = $("#adm_form_security_authentication_azuread_sync_timezone").val();
				this.m_security.openid.language = $("#adm_form_security_authentication_azuread_sync_language").val();
				this.m_security.openid.country = $("#adm_form_security_authentication_azuread_sync_country").val();
				this.m_security.openid.area = $("#adm_form_security_authentication_azuread_sync_area").val();
				this.m_security.openid.default_campaign = $("#adm_form_security_authentication_azuread_sync_default_campaign").val();
				this.m_security.openid.initial_state = $("#adm_form_security_authentication_azuread_sync_initial_state").val();

				const $azuread_panel = $("#adm_form_security_authentication_azuread_sync_panel");
				$azuread_panel.find("[AZUREAD_ROW_GROUP_NAME]").each(function(_, item) {
					const $item = $(item);
					const group = $item.attr("AZUREAD_ROW_GROUP_NAME");
					if (group !== "allusers") {
						const $filter = $item.find("[AZUREAD_GROUP_FILTER]");
						const $cboSelectedOption = $filter.find("option:selected");
						self.m_security.openid.groupFilters[group] = {
							id         : $filter.val(),
							displayName: $cboSelectedOption.text()
						};
					}
				});
			}
		}
	};

	adm_form_security.prototype.set = function(success, security)
	{
		if (!success) {
			// TODO: ALGO ?
			return;
		}
		if(!security) {
			security = this.m_security;
		}
		this.set_security_captcha(security);
		this.set_security_authentication(security);
		this.m_security = security;
		return true;
	};

	adm_form_security.prototype.set_security_captcha = function(security)
	{
		const $security_captcha_type = $("#adm_form_security_captcha_type");
		const $captcha_enabled = $("#adm_form_security_captcha_enabled");

		if (!security || !security.captcha) {
			$security_captcha_type.val("DEFAULT").trigger("change");
			$captcha_enabled.prop("checked", false).trigger("change");
			return;
		}
		const enabled = security.captcha.enabled === "1";
		$captcha_enabled.prop("checked", enabled);

		const public_key = (security.captcha.publicKey || "");
		const secret_key = (security.captcha.secretKey || "");

		if (public_key.length > 0 && secret_key.length > 0) {
			$security_captcha_type.val("CUSTOM").trigger("change");
		}
		else {
			$security_captcha_type.val("DEFAULT").trigger("change");
		}

		$security_captcha_type.trigger("change");

		const $public_key = $("#adm_form_security_captcha_public_key");
		const $secret_key = $("#adm_form_security_captcha_secret_key");
		$public_key.val(public_key);
		$secret_key.val(secret_key);
		this.toggle_security_captcha_enabled();
		this.toggle_security_captcha_custom_enabled();
	};

	adm_form_security.prototype.set_security_authentication = function(security)
	{
		const authentication = security && security.authentication || {};
		const passwordChecks = authentication.passwordChecks || {};
		$("#adm_form_security_authentication_type").val(authentication.type);
		switch(authentication.type) {
			case "BASIC":
			case "MFA":
				$("#adm_form_security_authentication_passwordChecks_minLen").val(passwordChecks.minLen);
				$("#adm_form_security_authentication_passwordChecks_maxLen").val(passwordChecks.maxLen);
				$("#adm_form_security_authentication_passwordChecks_expirationDays").val(passwordChecks.expirationDays);
				$("#adm_form_security_authentication_passwordChecks_blockOnLoginFailures").val(passwordChecks.blockOnLoginFailures);
				$("#adm_form_security_authentication_passwordChecks_history").val(passwordChecks.history);
				$("#adm_form_security_authentication_passwordChecks_minUpperChars").val(passwordChecks.minUpperChars);
				$("#adm_form_security_authentication_passwordChecks_minLowerChars").val(passwordChecks.minLowerChars);
				$("#adm_form_security_authentication_passwordChecks_minSpecialChars").val(passwordChecks.minSpecialChars);
				$("#adm_form_security_authentication_passwordChecks_minNumericChars").val(passwordChecks.minNumericChars);
				$("#adm_form_security_authentication_passwordChecks_expirationNotificationDays").val(passwordChecks.expirationNotificationDays);
				$("#adm_form_security_authentication_passwordChecks_changeOnFirstLogin").prop("checked", passwordChecks.changeOnFirstLogin);

				const mfa = security && security.mfa || {};
				if (mfa.enabled === "1") {
					$("#adm_form_security_authentication_mfa_type").val(mfa.type);
					switch(mfa.type) {
						case "MAIL": {
							$("#adm_form_security_authentication_mfa_mail_account").val(mfa.account);
							$("#adm_form_security_authentication_mfa_mail_length").val(mfa.keyLength);
							$("#adm_form_security_authentication_mfa_mail_expiration").val(mfa.expirationSeconds / 60);
							$("#adm_form_security_authentication_mfa_mail_subject").val(mfa.subject);
							this.m_mailControls.mfa.set_body_parts(JSON.parse(mfa.body));
							break;
						}
					}
				}
				break;

			case "LDAP":
				const ldap = security && security.ldap || {};
				const $ldap_panel = $("#adm_form_security_authentication_ldap_panel");

				const setLdapValues = (group)=> {
					let selector = jQUERY_SELECTOR_LDAP_GROUP_NAME(group);
					const $item = $ldap_panel.find(selector);
					$item.val(group);
					selector = jQUERY_SELECTOR_LDAP_GROUP_FILTER(group);
					const $filter = $ldap_panel.find(selector);
					$filter.val(ldap.groupFilters ? ldap.groupFilters[group] : "");
				};

				$("#adm_form_security_authentication_ldap_server").val(ldap.server);
				$("#adm_form_security_authentication_ldap_adminUserDN").val(ldap.adminUserDN);
				$("#adm_form_security_authentication_ldap_useTLS").prop("checked", ldap.useTLS);
				$("#adm_form_security_authentication_ldap_baseDN").val(ldap.baseDN);
				$("#adm_form_security_authentication_ldap_userIdAttributte").val(ldap.userIdAttributte);
				$("#adm_form_security_authentication_ldap_password").val(ldap.adminPassword);
				$("#adm_form_security_authentication_ldap_sync_enabled").prop("checked", ldap.sync_enabled);
				if(ldap.sync_enabled) {
					$("#adm_form_security_authentication_ldap_sync_timezone").val(ldap.timezone);
					$("#adm_form_security_authentication_ldap_sync_language").val(ldap.language);
					$("#adm_form_security_authentication_ldap_sync_country").val(ldap.country);
					this.fill_security_authentication_ldap_sync_areas(ldap.country);
					$("#adm_form_security_authentication_ldap_sync_area").val(ldap.area);
					$("#adm_form_security_authentication_ldap_sync_default_campaign").val(ldap.default_campaign);
					$("#adm_form_security_authentication_ldap_sync_initial_state").val(ldap.initial_state);
					for (const group in ldap.groupFilters) {
						if (ldap.groupFilters.hasOwnProperty(group)) {
							setLdapValues(group);
						}
					}
				}
				else {
					setLdapValues("allusers");
				}

				break;

			case "OPENID":
				const openid = security && security.openid || {};
				$("#adm_form_security_authentication_openid_configuration_url").val(openid.openid_configuration_url);
				$("#adm_form_security_authentication_openid_sync_method").val(openid.sync_method);
				$("#adm_form_security_authentication_azuread_sync_panel").toggle(openid.sync_method === "AZUREAD");
				if(openid.sync_method === "AZUREAD") {
					$("#adm_form_security_authentication_azuread_sync_client_id").val(openid.client_id);
					$("#adm_form_security_authentication_azuread_sync_client_secret").val(openid.adminPassword);
					$("#adm_form_security_authentication_azuread_sync_enabled").prop("checked", openid.sync_enabled);
					$("#adm_form_security_authentication_azuread_sync_timezone").val(openid.timezone);
					$("#adm_form_security_authentication_azuread_sync_language").val(openid.language);
					$("#adm_form_security_authentication_azuread_sync_country").val(openid.country);
					this.fill_security_authentication_azuread_sync_areas(openid.country);
					$("#adm_form_security_authentication_azuread_sync_area").val(openid.area);
					$("#adm_form_security_authentication_azuread_sync_default_campaign").val(openid.default_campaign);
					$("#adm_form_security_authentication_azuread_sync_initial_state").val(openid.initial_state);
				}
				break;
		}
		this.toggle_security_authentication();
	};	

	adm_form_security.prototype.on_show_tab = function(tab_id)
	{
		if (tab_id === "security") {
			return this.main_tabs.switch_tab("security");
		}
		throw("adm_form_security.prototype.on_show_tab - Unrecognized Tab " + tab_id);
	};

	adm_form_security.prototype.__show_error_panel = function()
	{
		$("#adm_form_security_error_container").show();
		$("#adm_form_security_updating_container").hide();
	};

	adm_form_security.prototype.show_or_hide_required_message = function()
	{
		let must_show = false;
		if (this.is_creating()) {
			must_show = true;
		}
		else {
			const selected_tab_id = this.main_tabs.get_selected_tab_id();
			must_show = selected_tab_id === "security" ? true : false;
		}
		adm_form_security.uber.show_or_hide_required_message.call(this, must_show); //invokes the base class method
	};

	adm_form_security.prototype.uri_get_entity = function()
	{
		return inConcert.app.getBaseDomain() + this.get_uri;
	};

	adm_form_security.prototype.get_validation_def = function()
	{
		const self = this;
		let definitions = [];

		var can_validate_security_captcha_fn = function() {
			if (!$("#adm_form_security_captcha_enabled").prop("checked")) {
				return false;
			}
			const security_captcha_type = $("#adm_form_security_captcha_type").val();
			return security_captcha_type === "CUSTOM";
		};

		var can_validate_security_authentication_basic_fn = function() {
			return $("#adm_form_security_authentication_type").val() === "BASIC" ||
				$("#adm_form_security_authentication_type").val() === "MFA";
		};

		var can_validate_security_authentication_mfa_mail_fn = function() {
			return $("#adm_form_security_authentication_type").val() === "MFA" &&
				$("#adm_form_security_authentication_mfa_type").val() === "MAIL";
		};

		var can_validate_security_authentication_ldap_fn = function() {
			return $("#adm_form_security_authentication_type").val() === "LDAP";
		};

		var can_validate_security_authentication_azuread_fn = function() {
			return $("#adm_form_security_authentication_type").val() === "OPENID" && $("#adm_form_security_authentication_openid_sync_method").val() === "AZUREAD";
		};

		var on_security_error = function() {
			self.main_tabs.switch_tab("security");
		};

		var sumOfMinimums = function() {
			const minOfLengths = 4; //Minimum length for both minLen and maxLen

			const minLowerChars = parseInt((/^-?\d+$/.test($("#adm_form_security_authentication_passwordChecks_minLowerChars").val()) && $("#adm_form_security_authentication_passwordChecks_minLowerChars").val()) || 0);
			const minUpperChars = parseInt((/^-?\d+$/.test($("#adm_form_security_authentication_passwordChecks_minUpperChars").val()) && $("#adm_form_security_authentication_passwordChecks_minUpperChars").val())|| 0);
			const minSpecialChars = parseInt((/^-?\d+$/.test($("#adm_form_security_authentication_passwordChecks_minSpecialChars").val()) && $("#adm_form_security_authentication_passwordChecks_minSpecialChars").val()) || 0);
			const minNumericChars = parseInt((/^-?\d+$/.test($("#adm_form_security_authentication_passwordChecks_minNumericChars").val()) && $("#adm_form_security_authentication_passwordChecks_minNumericChars").val()) || 0);
			const sumOfMin = minLowerChars + minUpperChars + minSpecialChars + minNumericChars;
			return sumOfMin < minOfLengths ? minOfLengths : sumOfMin;
		};

		var getMinLen = function() {
			const minOfLengths = sumOfMinimums(); //Minimum length for both minLen and maxLen

			const minLen = parseInt((/^-?\d+$/.test($("#adm_form_security_authentication_passwordChecks_minLen").val()) && $("#adm_form_security_authentication_passwordChecks_minLen").val()) || 0);
			return minLen < minOfLengths ? minOfLengths : minLen;
		};

		const getMaxExpirationNotificationDays = function() {
			const $expirationDays = $("#adm_form_security_authentication_passwordChecks_expirationDays");
			const expirationDays = parseInt($expirationDays.val());
			return expirationDays - 1 < 0 ? 0 : expirationDays - 1;
		};	

		definitions.push(
		// Custom Security Captcha
			{
			// public key
				control_id: "adm_form_security_captcha_public_key",
				can_validate: can_validate_security_captcha_fn,
				messages: {
					required: "<%=_T('Public Key is required for Security Captcha.')%>"
				},
				on_error: on_security_error
			},
			{
			// Secret key
				control_id: "adm_form_security_captcha_secret_key",
				can_validate: can_validate_security_captcha_fn,
				messages: {
					required: "<%=_T('Secret Key is required for Security Captcha.')%>"
				},
				on_error: on_security_error
			},

		// Basic authentication
			{
			// Minimum upper character count
				control_id: "adm_form_security_authentication_passwordChecks_minUpperChars",
				can_validate: can_validate_security_authentication_basic_fn,
				messages: {
					required: "<%=_T('Minimum upper character count is required.')%>",
					range: {
						between: { from: 0, to: 5 },
						message: "<%=_T('Minimum upper character count is out of range. Please enter a value between {0} and {1}.')%>"
					}
				},
				on_error: on_security_error
			},
			{
			// Minimum lower character count
				control_id: "adm_form_security_authentication_passwordChecks_minLowerChars",
				can_validate: can_validate_security_authentication_basic_fn,
				messages: {
					required: "<%=_T('Minimum lower character count is required.')%>",
					range: {
						between: { from: 0, to: 5 },
						message: "<%=_T('Minimum lower character count is out of range. Please enter a value between {0} and {1}.')%>"
					}
				},
				on_error: on_security_error
			},
			{
			// Minimum special character count
				control_id: "adm_form_security_authentication_passwordChecks_minSpecialChars",
				can_validate: can_validate_security_authentication_basic_fn,
				messages: {
					required: "<%=_T('Minimum special character count is required.')%>",
					range: {
						between: { from: 0, to: 5 },
						message: "<%=_T('Minimum special character count is out of range. Please enter a value between {0} and {1}.')%>"
					}
				},
				on_error: on_security_error
			},
			{
			// Minimum numeric character count
				control_id: "adm_form_security_authentication_passwordChecks_minNumericChars",
				can_validate: can_validate_security_authentication_basic_fn,
				messages: {
					required: "<%=_T('Minimum numeric character count is required.')%>",
					range: {
						between: { from: 0, to: 5 },
						message: "<%=_T('Minimum numeric character count is out of range. Please enter a value between {0} and {1}.')%>"
					}
				},
				on_error: on_security_error
			},
			{
			// Minimum password length
				control_id: "adm_form_security_authentication_passwordChecks_minLen",
				can_validate: can_validate_security_authentication_basic_fn,
				messages: {
					required: "<%=_T('Minimum password length is required.')%>",
					range: {
						between: { from: sumOfMinimums(), to: 20 },
						message: "<%=_T('Minimum password length is out of range. Please enter a value between {0} and {1}.')%>"
					}
				},
				on_error: on_security_error
			},
			{
			// Maximum password length
				control_id: "adm_form_security_authentication_passwordChecks_maxLen",
				can_validate: can_validate_security_authentication_basic_fn,
				messages: {
					required: "<%=_T('Maximum password length is required.')%>",
					range: {
						between: { from: getMinLen(), to: 20 },
						message: "<%=_T('Maximum password length is out of range. Please enter a value between {0} and {1}.')%>"
					}
				},
				on_error: on_security_error
			},
			{
			// Password expiration
				control_id: "adm_form_security_authentication_passwordChecks_expirationDays",
				can_validate: can_validate_security_authentication_basic_fn,
				messages: {
					required: "<%=_T('Password expiration is required (0 = disabled).')%>",
					range: {
						between: { from: 0, to: 60 },
						message: "<%=_T('Password expiration is out of range. Please enter a value between {0} and {1} days (0 = disabled).')%>"
					}
				},
				on_error: on_security_error
			},
			{
			// Password expiration notification
				control_id: "adm_form_security_authentication_passwordChecks_expirationNotificationDays",
				can_validate: function() {
					return can_validate_security_authentication_basic_fn() && $("#adm_form_security_authentication_passwordChecks_expirationDays").val() > 0;
				},
				messages: {
					required: "<%=_T('Password expiration notification is required (0 = disabled).')%>",
					range: {
						between: { from: 0, to: getMaxExpirationNotificationDays() },
						message: "<%=_T('Password expiration notification is out of range. Please enter a value between {0} and {1} days (0 = disabled).')%>"
					}
				},
				on_error: on_security_error
			},			
			{
			// Block on login failures
				control_id: "adm_form_security_authentication_passwordChecks_blockOnLoginFailures",
				can_validate: can_validate_security_authentication_basic_fn,
				messages: {
					required: "<%=_T('Block on login failures is required (0 = disabled).')%>",
					range: {
						between: { from: 0, to: 10 },
						message: "<%=_T('Block on login failures is out of range. Please enter a value between {0} and {1} (0 = disabled).')%>"
					}
				},
				on_error: on_security_error
			},
			{
			// Password history
				control_id: "adm_form_security_authentication_passwordChecks_history",
				can_validate: can_validate_security_authentication_basic_fn,
				messages: {
					required: "<%=_T('Password history is required (0 = disabled).')%>",
					range: {
						between: { from: 0, to: 20 },
						message: "<%=_T('Password history is out of range. Please enter a value between {0} and {1} (0 = disabled).')%>"
					}
				},
				on_error: on_security_error
			},


		// Security MFA
			{
			// mail account
				control_id: "adm_form_security_authentication_mfa_mail_account",
				can_validate: can_validate_security_authentication_mfa_mail_fn,
				messages: {
					required: "<%=_T('An outgoing account is required.')%>"
				},
				on_error: on_security_error
			},
			{
			// Secret key
				control_id: "adm_form_security_authentication_mfa_mail_length",
				can_validate: can_validate_security_authentication_mfa_mail_fn,
				messages: {
					required: "<%=_T('Security code length is required.')%>",
					range: {
						between: { from: 4, to: 10 },
						message: "<%=_T('Security code length is out of range. Please enter a value between {0} and {1}.')%>"
					}
				},
				on_error: on_security_error
			},
			{
			// Expiration minutes
				control_id: "adm_form_security_authentication_mfa_mail_expiration",
				can_validate: can_validate_security_authentication_mfa_mail_fn,
				messages: {
					required: "<%=_T('Expiration minutes are required.')%>",
					range: {
						between: { from: 5, to: 30 },
						message: "<%=_T('Expiration minutes are out of range. Please enter a value between {0} and {1} minutes.')%>"
					}
				},
				on_error: on_security_error
			},
			{
			// Subject
				control_id: "adm_form_security_authentication_mfa_mail_subject",
				can_validate: can_validate_security_authentication_mfa_mail_fn,
				messages: {
					required: "<%=_T('Security e-mail subject is required.')%>"
				},
				on_error: on_security_error
			},
			{
			// Body
				control_id: "adm_form_security_authentication_mfa_mail_body",
				can_validate: can_validate_security_authentication_mfa_mail_fn,
				messages: {
					valid: function() {
						let response = {
							success: false
						};
						self.m_mailControls.mfa.validate(function(validation_result) {
							response = validation_result;
						});
						return response;
					}
				},
				on_error: on_security_error
			}
		);
		if(can_validate_security_authentication_ldap_fn()) {
			this.ldap_get_validation_def(definitions);
		}
		if(can_validate_security_authentication_azuread_fn()) {
			this.azuread_get_validation_def(definitions);
		}

		return definitions;
	};

	adm_form_security.prototype.save_execute = function(after_save, entity)
	{
		const self = this;

		var callback = function(success, response) {
			self.on_saved(success, entity, after_save, response);
		};

		this.m_security.isNew = undefined;

		inConcert.invoke({
			url: inConcert.app.getBaseDomain() + "/inconcert/apps/administration/security/update/",
			type   : "PUT",
			data        : JSON.stringify(this.m_security),
			dataType    : "json",
			contentType : "application/json",
			success: function( response )
			{
				callback(true, response);
			},
			error: function(/*err_type, text_status*/)
			{
				callback(false);
			}
		});
	};

	adm_form_security.prototype.on_saved = function(success, entity, after_save, response)
	{
		adm_form_security.uber.on_saved.call(this, success, entity, after_save, response); //invokes the base class method

		this.dashboard.hide_waiting();
	};

	adm_form_security.prototype.set_events = function()
	{
		const self = this;
		adm_form_security.uber.set_events.call(this); //invokes the base class method

		// authentication type
		$("#adm_form_security_authentication_type")
			.off("change")
			.on("change", function() {
				self.toggle_security_authentication();
			});

		// openid sync method
		$("#adm_form_security_authentication_openid_sync_method")
			.off("change")
			.on("change", function() {
				$("#adm_form_security_authentication_azuread_sync_panel").toggle($(this).val() === "AZUREAD");
			});

		// ldap sync enabled
		const $ldap_panel = $("#adm_form_security_authentication_ldap_panel");
		$("#adm_form_security_authentication_ldap_sync_enabled")
			.off("change")
			.on("change", function() {
				self.toggle_security_authentication_ldap_sync_enabled();
				if ($(this).prop("checked")) {
					const selector = jQUERY_SELECTOR_LDAP_GROUP_FILTER("allusers");
					$ldap_panel.find(selector).val("");
				}
				else {
					$ldap_panel.find("[LDAP_ROW_GROUP_NAME] [LDAP_GROUP_FILTER]").val("");
				}
				$("#adm_form_security_authentication_ldap_sync_country").trigger("change");
			});

		// azuread sync enabled
		$("#adm_form_security_authentication_azuread_sync_enabled")
			.off("change")
			.on("change", function() {
				self.reload_security_authentication_azuread_groups();
			});

		// ldap country
		$("#adm_form_security_authentication_ldap_sync_country")
			.off("change")
			.on("change", function() {
				self.fill_security_authentication_ldap_sync_areas($(this).val());
			});

		// azuread country
		$("#adm_form_security_authentication_azuread_sync_country")
			.off("change")
			.on("change", function() {
				self.fill_security_authentication_azuread_sync_areas($(this).val());
			});

		// captcha enabled
		$("#adm_form_security_captcha_enabled")
			.off("change")
			.on("change", function() {
				self.toggle_security_captcha_enabled();
			});

		// captcha custom type enabled
		$("#adm_form_security_captcha_type")
			.off("change")
			.on("change", function() {
				self.toggle_security_captcha_custom_enabled();
			});

		// mfa custom type enabled
		$("#adm_form_security_authentication_mfa_type")
			.off("change")
			.on("change", function() {
				self.toggle_security_authentication_mfa_type();
			});		

		// expirationDays 
		$("#adm_form_security_authentication_passwordChecks_expirationDays")
			.off("blur")
			.on("blur", function() {
				let $expirationNotificationDays = $("#adm_form_security_authentication_passwordChecks_expirationNotificationDays");
				self.disable_control($expirationNotificationDays, $( this ).val() === "0", true);
				$expirationNotificationDays.val($( this ).val() === "0" ? "0" : $expirationNotificationDays.val());
			});	
			
		$("#adm_form_security_authentication_edit")
			.off("click")
			.on("click", function() {
				self.set_readonly(false);
				self.reopen_view({
					extra_parameters:{
						show_view_readonly: false
					}
				});
			});
	};

	adm_form_security.prototype.no_close = function()
	{
		this.set(true);
		this.readonly();
		this.set_title(this.view_title);
		$("#adm_form_security_authentication_edit").show();
	};

	adm_form_security.prototype.fill_security_authentication_types = function()
	{
		const types = [
			{id: "BASIC", description: "<%=_T('Basic')%>"},
			{id: "MFA", description: "<%=_T('MFA')%>"}
		];
		if (!inConcert.app.has_allegro_integration()) {
			types.push({id: "LDAP", description: "<%=_T('LDAP')%>"});
			types.push({id: "OPENID", description: "<%=_T('OpenId')%>"});
		}
		this.fill_combo($("#adm_form_security_authentication_type"), false, types, "id", "description");
	};

	/*
		La funcion 'fill_combo_app_languages' se debe de invocar a la clase BASE ya que esta definida en clase base "conf_form".
	*/
	adm_form_security.prototype.fill_security_authentication_languages = function()
	{
		//LDAP
		this.fill_combo_app_languages($("#adm_form_security_authentication_ldap_sync_language"), false);
		//AZURE
		this.fill_combo_app_languages($("#adm_form_security_authentication_azuread_sync_language"), false);
	};

	adm_form_security.prototype.fill_security_captcha_types = function()
	{
		const captcha_type_list = [
			{id: "DEFAULT", description: "<%=_T('Default')%>"},
			{id: "CUSTOM" , description: "<%=_T('Custom')%>" }
		];
		const $security_captcha_type = $("#adm_form_security_captcha_type");
		const is_blank_first_item = false;
		const attr_id = "id";
		const attr_name = "description";
		this.fill_combo($security_captcha_type, is_blank_first_item, captcha_type_list, attr_id, attr_name);
	};

	adm_form_security.prototype.fill_security_authentication_mfa_types = function()
	{
		const types = [
			{id: "MAIL", description: "<%=_T('E-mail')%>"}
		];
		this.fill_combo($("#adm_form_security_authentication_mfa_type"), false, types, "id", "description");
	};

	adm_form_security.prototype.fill_security_authentication_mfa_mail_accounts = function(accounts)
	{
		this.fill_combo($("#adm_form_security_authentication_mfa_mail_account"), false, accounts, "id", "name");
	};

	adm_form_security.prototype.fill_security_authentication_ldap_common = function()
	{
		var self = this;

		var can_validate_security_authentication_ldap_fn = function() {
			return $("#adm_form_security_authentication_type").val() === "LDAP";
		};
		
		var on_security_error = function() {
			self.main_tabs.switch_tab("security");
		};

		this.ldap_common_validation_definitions = [];
		this.ldap_common_validation_definitions.push(
			{
			// server
				control_id: "adm_form_security_authentication_ldap_server",
				can_validate: can_validate_security_authentication_ldap_fn,
				messages: {
					required: "<%=_T('Server IP or domain is required.')%>"
				},
				on_error: on_security_error
			},
			{
			// Admin User DN
				control_id: "adm_form_security_authentication_ldap_adminUserDN",
				can_validate: can_validate_security_authentication_ldap_fn,
				messages: {
					required: "<%=_T('User RDN is required.')%>"
				},
				on_error: on_security_error
			},
			{
			// Admin User Password
				control_id: "adm_form_security_authentication_ldap_password",
				can_validate: can_validate_security_authentication_ldap_fn,
				messages: {
					required: "<%=_T('Password is required.')%>"
				},
				on_error: on_security_error
			},
			{
			// Base DN
				control_id: "adm_form_security_authentication_ldap_baseDN",
				can_validate: can_validate_security_authentication_ldap_fn,
				messages: {
					required: "<%=_T('Base DN is required.')%>"
				},
				on_error: on_security_error
			},
			{
			// Base DN
				control_id: "adm_form_security_authentication_ldap_userIdAttributte",
				can_validate: can_validate_security_authentication_ldap_fn,
				messages: {
					required: "<%=_T('User identifier attribute is required.')%>",
					custom: {
						type : "custom",
						valid: function() {
							let response = {
								success: true
							};

							const $userIdAttributte =  $("#adm_form_security_authentication_ldap_userIdAttributte");
							let userIdAttributte = $userIdAttributte.val();
							let userIdAttributteRegExp = /^[a-zA-Z0-9\-]+$/;
							if(!userIdAttributte.match(userIdAttributteRegExp)) {
								response.success = false;
								response.message = "<%=_T('Only letters, numbers and mid dashes are allowed in the User identifier attribute.')%>";
							}

							return response;
						}
					}					
				},
				on_error: on_security_error
			},
		);
	};

	adm_form_security.prototype.fill_security_authentication_azuread_common = function()
	{
		var self = this;

		var can_validate_security_authentication_azuread_fn = function() {
			return $("#adm_form_security_authentication_type").val() === "OPENID" && $("#adm_form_security_authentication_openid_sync_method").val() === "AZUREAD";
		};
		
		var on_security_error = function() {
			self.main_tabs.switch_tab("security");
		};

		this.azuread_common_validation_definitions = [];
		this.azuread_common_validation_definitions.push(
			{
			// url
				control_id: "adm_form_security_authentication_openid_configuration_url",
				can_validate: can_validate_security_authentication_azuread_fn,
				messages: {
					required: "<%=_T('OpenId configuration URL is required.')%>"
				},
				on_error: on_security_error
			},
			{
			// Client Id
				control_id: "adm_form_security_authentication_azuread_sync_client_id",
				can_validate: can_validate_security_authentication_azuread_fn,
				messages: {
					required: "<%=_T('Client Id is required.')%>"
				},
				on_error: on_security_error
			},
			{
			// Client secret
				control_id: "adm_form_security_authentication_azuread_sync_client_secret",
				can_validate: can_validate_security_authentication_azuread_fn,
				messages: {
					required: "<%=_T('Client secret is required.')%>"
				},
				on_error: on_security_error
			},
		);
	};

	adm_form_security.prototype.fill_security_authentication_ldap_groups = function(groups)
	{
		const self = this;

		const $ldap_panel = $("#adm_form_security_authentication_ldap_panel");

		const inputFilter_onclick = function() {
			let selectedFilterId = $(this).attr("id");
			self.validate(function( success ) {
				if (!success) {
					return;
				}
				const $password =  $("#adm_form_security_authentication_ldap_password"); 
				const jwtToken  = inConcert.Utils.GetJwtToken($password.val());
				var data = {
					server : $("#adm_form_security_authentication_ldap_server").val(),
					baseDN : $("#adm_form_security_authentication_ldap_baseDN").val(),
					adminUserDN : $("#adm_form_security_authentication_ldap_adminUserDN").val(),
					password : jwtToken,
					useTLS : $("#adm_form_security_authentication_ldap_useTLS").prop("checked"),
					source: "ldap"
				};
				var validation_def = {
					control_id: "adm_form_security_authentication_ldap_server",
					can_validate: true,
					messages: {
						custom: {
							has_error: false,
							error_message: "",
							type : "custom",
							valid: function() {
								let response = {
									success: true
								};
								if(this.has_error) {
									response.success = false;
									response.message = this.error_message;
								}
								return response;
							}
						}
					}
				};
				var definitions = [];
				definitions.push(validation_def);

				self.load_external_entities(data, function(success, response) {
					if(!success) {
						validation_def.messages.custom.has_error = true;
						validation_def.messages.custom.error_message = response;
						self.validate(function(/*success*/) {}, definitions);
						console.log(response);
						return;
					}
					if(!response.success) {
						validation_def.messages.custom.has_error = true;
						validation_def.messages.custom.error_message = response.error;
						self.validate(function(/*success*/) {}, definitions);
						console.log(response.error);
						return;
					}
					if(!response.result || response.result.length === 0) {
						validation_def.messages.custom.has_error = true;
						validation_def.messages.custom.error_message =  "<%=_T('No results returned')%>";
						self.validate(function(/*success*/) {}, definitions);
						return;
					}
					
					// response.result es un objeto cuyas propiedades son las entradas de ldap, desordenadas y a un mismo nivel
					// La clase GenericTreePopup espera los dispositionCodes como un array, en el que cada elemento tendría un campo string "name" y un campo array "childs"
					var resultAsDispositionCodesFormat = [];

					// Obtiene un array anidado, en forma de arbol, con toda la descendencia de una entrada de ldap
					var getChilds;
					getChilds = function(nodeLevels) {
						if(nodeLevels.length > 0) {
							let node = {
								name: nodeLevels[nodeLevels.length - 1]
							};
							nodeLevels.pop();
							let childs = getChilds(nodeLevels);
							if(childs) {
								node.childs = [];
								node.childs.push(childs);
							}
							return node;
						}
					};

					//Inserta el primer nodo, junto con sus hijos, que no exista ya en arr 
					var insertFirstNonExistingNode = function(arr, node) {
						let indexFound = arr.findIndex(e => e.name === node.name);
						if(indexFound >= 0) {
							if(arr[indexFound].childs && node.childs && node.childs[0]) {
								insertFirstNonExistingNode(arr[indexFound].childs, node.childs[0]);
							}
						}
						else {
							arr.push(node);
						}
					};

					// Hacer el reverse de cada propiedad de response.result para poder ordenar 
					let reversedResult = {};
					for (const property in response.result) {
						if(response.result.hasOwnProperty(property)) {
							reversedResult[property.split("").reverse().join("")] = response.result[property];
						}
					}

					// Obtenemos un objeto con sus properties ordenadas descendente, para que aparezcan los descendientesde nivel más profundo antes que su progenitor
					let sortedResult = {};
					Object.keys(reversedResult).sort(function (a, b) {
						if (a < b) {
							return 1;
						}
						if (a > b) {
							return -1;
						}
						// a must be equal to b
						return 0;
					}).map(i=>sortedResult[i]=response.result[i]);

					for (const itemReversed in sortedResult) {
						if(sortedResult.hasOwnProperty(itemReversed)) {
							//Deshacemos el reverse una vez ordenado el resultado
							let item = itemReversed.split("").reverse().join("");
			        		// Extraer del item el numero de separadores (n) de path de ldap tiene (,). El node debe ir en el subnivel n del array
							let nodeLevels = item.split(",");
							let node = {
								name: nodeLevels[nodeLevels.length - 1]
							};
							nodeLevels.pop();
							let childs = getChilds(nodeLevels);
			        		if(childs) {
			        			node.childs = [];
			        			node.childs.push(childs);
			        		}

			        		insertFirstNonExistingNode(resultAsDispositionCodesFormat, node);
			        	}
					}

					const ldapTreePopup = new inConcert.GenericTreePopup();

					ldapTreePopup.show({
						title: "<%=_T('Select a LDAP level')%>",
						parentNode:       $("#" + selectedFilterId),
						dispositionCodes: resultAsDispositionCodesFormat,
						permissionToComment: false,
						callbacks: {
							on_finish : function( response ) {
								if (response && response.disposition && response.disposition.treePath && Array.isArray(response.disposition.treePath)) {
									// Insertar en el filter correspondiente lo seleccionado en el arbol
									let selectedFilterValue = "";
									response.disposition.treePath.slice().reverse().forEach(element => selectedFilterValue+= (selectedFilterValue.length ? "," : "") + element);

									// No permitir filtros duplicados
									const ldap_filter_validation_definition = {
										control_id: selectedFilterId,
										can_validate: true,
										messages: {
											custom: {
												valid: function() {
													const response = {
														success: true
													};
													$ldap_panel.find("[LDAP_ROW_GROUP_NAME]").each(function(_, item) {
														const $item = $(item);
														const $ldapGroupFilter = $item.find("[ldap_group_filter]");
														const filter = $ldapGroupFilter.val();
														const filterId = $ldapGroupFilter.attr("id");
														if (filterId !== selectedFilterId && filter === selectedFilterValue) {
															response.success = false;
															response.message = "<%=_T('A filter can only be used once')%>";	
															return response;
														}
													});
													return response;
												}
											}
										}
									};

									const ldap_filter_validation_definitions = [];
									ldap_filter_validation_definitions.push(ldap_filter_validation_definition);

									self.validate(function( success ) {	
										if(success) {
											$("#" + selectedFilterId).val(selectedFilterValue);
										}
									}, ldap_filter_validation_definitions);
								}
							},
							onOpen: $.noop,
							onClose: $.noop
						}
					});
				});
			}, self.ldap_common_validation_definitions);
		};		

		for (const group in groups) {
			if(groups.hasOwnProperty(group)) {
				var groupId = "adm_form_security_authentication_ldap_group_" + group;

				var labelGroup = document.createElement("label");
				labelGroup.setAttribute("for", groupId);
				labelGroup.innerHTML = "<%=_T('Group')%>";

				var inputGroup = document.createElement("input");
				inputGroup.classList.add("form-control");
				inputGroup.setAttribute("id", groupId);
				inputGroup.setAttribute("type", "text");
				inputGroup.setAttribute("disabled", "true");
				inputGroup.setAttribute("LDAP_GROUP_NAME", group);
				inputGroup.value = group;

				var formGroup = document.createElement("div");
				formGroup.classList.add("form-group");
				formGroup.classList.add("customEditable");
				formGroup.appendChild(labelGroup);
				formGroup.appendChild(inputGroup);

				var colGroup = document.createElement("div");
				colGroup.classList.add("col-3");
				colGroup.appendChild(formGroup);

				var labelFilter = document.createElement("label");
				labelFilter.setAttribute("for", groupId + "_filter");
				labelFilter.innerHTML = "<%=_T('Filter')%>";

				var filterId = groupId + "_filter"; 
				var inputFilter = document.createElement("input");
				inputFilter.classList.add("form-control");
				inputFilter.setAttribute("id", filterId);
				inputFilter.setAttribute("type", "text");
				//inputFilter.setAttribute("disabled", "true");
				inputFilter.setAttribute("LDAP_GROUP_FILTER", group);

				var validationFilter = document.createElement("i");
				validationFilter.classList.add("formFieldValidation");
				validationFilter.classList.add("formFieldError");
				validationFilter.setAttribute("id", filterId + "_validation");

				var formFilter = document.createElement("div");
				formFilter.classList.add("form-group");
				//formFilter.classList.add("customEditable");
				formFilter.appendChild(labelFilter);
				formFilter.appendChild(inputFilter);
				formFilter.appendChild(validationFilter);

				var colFilter = document.createElement("div");
				colFilter.classList.add("col-6");
				colFilter.appendChild(formFilter);

				inputFilter.onclick = inputFilter_onclick;

				var row = document.createElement("div");
				row.classList.add("row");
				row.setAttribute("LDAP_ROW_GROUP_NAME", group);
				row.appendChild(colGroup);
				row.appendChild(colFilter);
	
				let $ldap_sync_common = document.getElementById("adm_form_security_authentication_ldap_sync_common");
				document.getElementById("adm_form_security_authentication_ldap_sync_panel").insertBefore(row, $ldap_sync_common);
			}
		}
	};

	adm_form_security.prototype.fill_security_authentication_azuread_groups = function(groups)
	{
		const self = this;	

		for (const group in groups) {
			if(groups.hasOwnProperty(group) && group !== "allusers") {
				var groupId = "adm_form_security_authentication_azuread_group_" + group;

				var labelGroup = document.createElement("label");
				labelGroup.setAttribute("for", groupId);
				labelGroup.innerHTML = "<%=_T('OCC Group')%><%=_T(':')%>";

				var inputGroup = document.createElement("input");
				inputGroup.classList.add("form-control");
				inputGroup.setAttribute("id", groupId);
				inputGroup.setAttribute("type", "text");
				inputGroup.setAttribute("disabled", "true");
				inputGroup.setAttribute("AZUREAD_GROUP_NAME", group);
				inputGroup.value = group;

				var formGroup = document.createElement("div");
				formGroup.classList.add("form-group");
				formGroup.classList.add("customEditable");
				formGroup.appendChild(labelGroup);
				formGroup.appendChild(inputGroup);

				var colGroup = document.createElement("div");
				colGroup.classList.add("col-3");
				colGroup.appendChild(formGroup);

				var labelFilter = document.createElement("label");
				labelFilter.setAttribute("for", groupId + "_filter");
				labelFilter.innerHTML = "<%=_T('Azure AD Group')%><%=_T(':')%>";

				var filterId = groupId + "_filter"; 
				var inputFilter = document.createElement("select");
				inputFilter.classList.add("form-control");
				inputFilter.setAttribute("id", filterId);
				inputFilter.setAttribute("AZUREAD_GROUP_FILTER", group);

				var validationFilter = document.createElement("i");
				validationFilter.classList.add("formFieldValidation");
				validationFilter.classList.add("formFieldError");
				validationFilter.setAttribute("id", filterId + "_validation");

				var formFilter = document.createElement("div");
				formFilter.classList.add("form-group");
				formFilter.appendChild(labelFilter);
				formFilter.appendChild(inputFilter);
				formFilter.appendChild(validationFilter);

				var colFilter = document.createElement("div");
				colFilter.classList.add("col-3");
				colFilter.appendChild(formFilter);

				var row = document.createElement("div");
				row.classList.add("row");
				row.setAttribute("AZUREAD_ROW_GROUP_NAME", group);
				row.appendChild(colGroup);
				row.appendChild(colFilter);
	
				let $azuread_sync_common = document.getElementById("adm_form_security_authentication_azuread_sync_common");
				document.getElementById("adm_form_security_authentication_azuread_sync_panel").insertBefore(row, $azuread_sync_common);
			}
		}

		inConcert.invoke({
			url: inConcert.app.getBaseDomain() + "/inconcert/apps/administration/security/get_vcc_externalentities/",
			type   : "GET",
			success: function( response )
			{
				const groupFilters = self.m_security && self.m_security.openid && self.m_security.openid.groupFilters || {};
				const $azuread_panel = $("#adm_form_security_authentication_azuread_sync_panel");
				$azuread_panel.find("[AZUREAD_ROW_GROUP_NAME]").each(function(_, item) {
					const $item = $(item);
					const group = $item.attr("AZUREAD_ROW_GROUP_NAME");
					const selector = jQUERY_SELECTOR_AZUREAD_GROUP_FILTER(group);
					const $filter = $azuread_panel.find(selector);
					self.fill_combo($filter, false, response.result, "id", "displayName");
					$filter.val(groupFilters[group] && groupFilters[group].id);
				});
			}
		});
	};

	adm_form_security.prototype.fill_security_authentication_ldap_sync_areas = function( country_code ){
		this.fill_combo_areas(
			country_code,
			$("#adm_form_security_authentication_ldap_sync_area"),
			false
		);
	};
	
	adm_form_security.prototype.fill_security_authentication_azuread_sync_areas = function( country_code ){
		this.fill_combo_areas(
			country_code,
			$("#adm_form_security_authentication_azuread_sync_area"),
			false
		);
	};

	adm_form_security.prototype.fill_security_authentication_ldap_sync_campaigns = function(campaigns)
	{
		var $combo = $("#adm_form_security_authentication_ldap_sync_default_campaign");
		this.fill_combo($combo, false, campaigns, "id", "description");
	};

	adm_form_security.prototype.fill_security_authentication_azuread_sync_campaigns = function(campaigns)
	{
		var $combo = $("#adm_form_security_authentication_azuread_sync_default_campaign");
		this.fill_combo($combo, false, campaigns, "id", "description");
	};

	adm_form_security.prototype.fill_security_authentication_ldap_sync_states = function()
	{
		var $combo = $("#adm_form_security_authentication_ldap_sync_initial_state");
		this.fill_combo($combo, false, inConcert.app.getAvailableStates(), "Id", "Description");
	};

	adm_form_security.prototype.fill_security_authentication_azuread_sync_states = function()
	{
		var $combo = $("#adm_form_security_authentication_azuread_sync_initial_state");
		this.fill_combo($combo, false, inConcert.app.getAvailableStates(), "Id", "Description");
	};

	adm_form_security.prototype.fill_accounts = function(accounts = {})
	{
		this.fill_security_authentication_mfa_mail_accounts(accounts.MAIL || []);
	};	

	adm_form_security.prototype.fill_mail_controls = function()
	{
		this.m_mailControls = {
			mfa : new inConcert.MailManagement({
				owner: this,
				auto_init: true,
				ids: {
					body     : "adm_form_security_authentication_mfa_mail_body",
					container: "adm_form_security_authentication_mfa_mail_body_container"
				},
				callbacks: {
					loadVariables: function(sorted, callback = $.noop) {
						let variables = [];
						// {CODE}, {VCC}, {AGENT}, {AGENTNAME}, {AGENT_COMPLETE_NAME}
						variables.push({name: "<%=_T('User Id')%>"           , id: "{AGENT}"});
						variables.push({name: "<%=_T('User Name')%>"         , id: "{AGENTNAME}"});
						variables.push({name: "<%=_T('User Complete Name')%>", id: "{AGENT_COMPLETE_NAME}"});
						variables.push({name: "<%=_T('Code')%>"              , id: "{CODE}"});
						variables.push({name: "<%=_T('Contact Center')%>"    , id: "{VCC}"});
						if (sorted) {
							variables = inConcert.Utils.sortArray(variables, "name", "asc");
						}
						callback(true, variables);
					}
				}
			})
		};
		//default values
		$("#adm_form_security_authentication_mfa_mail_subject").val("<%=_T('Omnichannel login code')%>");
		this.m_mailControls.mfa.set("<%=_T('Enter the code {CODE} to login.')%>");
	};
	
	adm_form_security.prototype.fill_ldap_common_controls = function()
	{
		this.fill_security_authentication_ldap_common();
		this.fill_security_authentication_ldap_sync_states();
	};
	
	adm_form_security.prototype.fill_azuread_common_controls = function()
	{
		this.fill_security_authentication_azuread_common();
		this.fill_security_authentication_azuread_sync_states();
	};
	
	adm_form_security.prototype.fill_groups_grid = function(groups = {})
	{
		this.fill_security_authentication_ldap_groups(groups || []);
		this.fill_security_authentication_azuread_groups(groups || []);
	};

	adm_form_security.prototype.fill_all_ctrls_timezones = function(timezones = {})
	{
		// LDAP
		this.fill_timezones(timezones, $("#adm_form_security_authentication_azuread_sync_timezone"));
		// AZURED
		this.fill_timezones(timezones, $("#adm_form_security_authentication_ldap_sync_timezone"));
	};	

	adm_form_security.prototype.fill_campaigns = function(campaigns = {})
	{
		//Remove internal and not assignable campaigns as default campaign for a user
		delete campaigns.internal_calls;
		delete campaigns.sandbox;
		this.fill_security_authentication_ldap_sync_campaigns(campaigns || []);
		this.fill_security_authentication_azuread_sync_campaigns(campaigns || []);
	};

	adm_form_security.prototype.fill_all_ctrls_coutries = function(countries)
	{
		// LDAP
		this.fill_countries( countries, $("#adm_form_security_authentication_ldap_sync_country") );
		// AZURE
		this.fill_countries( countries, $("#adm_form_security_authentication_azuread_sync_country") );
	};

	adm_form_security.prototype.loadAccounts = function(callback = $.noop)
	{
		inConcert.invoke({
			url: inConcert.app.getBaseDomain() + "/inconcert/apps/administration/vccs/accounts/get_all_accounts/",
			type   : "GET",
			success: function( response )
			{
				callback(true, response);
			},
			error: function(/*err_type, text_status*/)
			{
				callback(false);
			}
		});
	};

	adm_form_security.prototype.loadCampaigns = function(callback = $.noop)
	{
		inConcert.invoke({
			url: inConcert.app.getBaseDomain() + "/inconcert/apps/administration/vccs/campaigns/get_all_campaigns/",
			type   : "GET",
			success: function( response )
			{
				callback(true, response);
			},
			error: function(/*err_type, text_status*/)
			{
				callback(false);
			}
		});
	};

	adm_form_security.prototype.load_data = function(callback_complete_requests)
	{
		var self = this;
		self.fill_security_captcha_types();
		self.fill_security_authentication_types();
		self.fill_security_authentication_mfa_types();
		self.fill_security_authentication_languages();
		self.fill_mail_controls();
		self.fill_ldap_common_controls();
		self.fill_azuread_common_controls();
		if (!$.isArray(self.requests)) {
			return;
		}

		self.requests.push(
			{
				request_function: function(success) {
					return self.loadAccounts(success);
				},
				success: function(accounts) {
					self.fill_accounts(accounts);
				}
			},
			{
				request_function: function(success) {
					return inConcert.app.loadAllRoles( success, false /*not from cache*/, "1" /*include_allusers*/ );
				},
				success: function(roles) {
					self.fill_groups_grid(roles);
				}
			},
			{
				request_function: function(success) {
					return inConcert.app.loadTimezones(success);
				},
				success: function(response) {
					self.fill_all_ctrls_timezones(response); 
				}
			},
			{
				request_function: function(success) {
					return self.telephone_codes.loadCountries(success);
				},
				success: function(countries) {
					self.fill_all_ctrls_coutries(countries);
				}
			},
			{
				request_function: function(success) {
					return self.loadCampaigns(success);
				},
				success: function(campaigns) {
					self.fill_campaigns(campaigns);
				}
			}
		);		

		adm_form_security.uber.load_data.call( this, callback_complete_requests );  //invokes the base class method
	};

	adm_form_security.prototype.toggle_security_authentication = function()
	{
		const type = $("#adm_form_security_authentication_type").val();
		const passwordChecksEnabled = type === "BASIC" || type === "MFA";
		const mfaEnabled = type === "MFA";
		const ldapEnabled = type === "LDAP";
		const openIdEnabled = type === "OPENID";
		$("#adm_form_security_authentication_passwordChecks_panel").toggle(passwordChecksEnabled);

		const $expirationDays = $("#adm_form_security_authentication_passwordChecks_expirationDays");
		const $expirationNotificationDays = $("#adm_form_security_authentication_passwordChecks_expirationNotificationDays");
		this.disable_control($expirationNotificationDays, passwordChecksEnabled && $expirationDays.val() === "0", passwordChecksEnabled);
		$expirationNotificationDays.val(passwordChecksEnabled && $expirationDays.val() === "0" ? "0" : $expirationNotificationDays.val());

		$("#adm_form_security_authentication_mfa_panel").toggle(mfaEnabled);
		$("#adm_form_security_authentication_ldap_panel").toggle(ldapEnabled);
		$("#adm_form_security_authentication_openid_panel").toggle(openIdEnabled);	
		this.toggle_security_authentication_ldap_sync_enabled();
		this.fill_security_authentication_openid_sync_methods(openIdEnabled);
		$("#adm_form_security_authentication_openid_sync_method").trigger("change");		
	};	

	adm_form_security.prototype.toggle_security_authentication_ldap_sync_enabled = function()
	{
		const is_enabled_ldap_sync = $("#adm_form_security_authentication_ldap_sync_enabled").prop("checked");
		const $ldap_panel = $("#adm_form_security_authentication_ldap_sync_panel");
		$ldap_panel.find("[LDAP_ROW_GROUP_NAME]").each(function(_, element) {
			const $groupRow = $(element);
			const group = $groupRow.attr("ldap_row_group_name");
			const showGroup = (group === "allusers" && !is_enabled_ldap_sync) || (group !== "allusers" && is_enabled_ldap_sync);
			$groupRow.toggle(showGroup);
		});
		$("#adm_form_security_authentication_ldap_sync_common").toggle(is_enabled_ldap_sync);
	};

	adm_form_security.prototype.toggle_security_captcha_enabled = function()
	{
		const is_required = false;
		// security captcha enabled
		const is_enabled_security_captcha = $("#adm_form_security_captcha_enabled").prop("checked");
		// security captcha type
		const $security_captcha_type = $("#adm_form_security_captcha_type");
		this.disable_control($security_captcha_type, !is_enabled_security_captcha, is_required);
		$security_captcha_type.trigger("change");
		// catpcha fields config
		const $security_captcha_basic_fields_config = $("#adm_form_security_captcha_basic_fields_config");
		$security_captcha_basic_fields_config[is_enabled_security_captcha ? "show" : "hide"]();
	};

	adm_form_security.prototype.toggle_security_captcha_custom_enabled = function()
	{
		const is_required = true;
		// security captcha enabled
		const is_enabled_security_captcha = $("#adm_form_security_captcha_enabled").prop("checked");
		// security captcha type
		const $security_captcha_type = $("#adm_form_security_captcha_type");
		const security_captcha_type = $security_captcha_type.val();
		const is_enabled_custom_security_captcha = is_enabled_security_captcha && security_captcha_type === "CUSTOM";
		// catpcha fields config
		const $security_captcha_basic_fields_config = $("#adm_form_security_captcha_basic_fields_config");
		$security_captcha_basic_fields_config.find("[captcha_custom_config]")[is_enabled_custom_security_captcha ? "show" : "hide"]();
		// captcha public_key
		const $public_key = $("#adm_form_security_captcha_public_key");
		this.disable_control($public_key, !is_enabled_custom_security_captcha, is_required);
		// captcha secret_key
		const $secret_key = $("#adm_form_security_captcha_secret_key");
		this.disable_control($secret_key, !is_enabled_custom_security_captcha, is_required);
	};

	adm_form_security.prototype.toggle_security_authentication_mfa_type = function()
	{
		const type = $("#adm_form_security_authentication_mfa_type").val();
		$("#adm_form_security_authentication_mfa_panel")
			.children("[mfa_section]")
			.hide()
			.filter(FORMAT("[mfa_section={0}]", type))
				.show();
		switch(type) {
			case "MAIL": {
				const $mailAccount = $("#adm_form_security_authentication_mfa_mail_account");
				$mailAccount.val($mailAccount.val() || $mailAccount.children(":first").val());
				break;
			}
		}
	};

	adm_form_security.prototype.load_external_entities = function(data, callback)
	{
		callback = callback || $.noop;

		inConcert.invoke({
			url: inConcert.app.getBaseDomain() + "/inconcert/apps/administration/security/loadexternalentities/",
			type: "POST",
			data: JSON.stringify(data),
			dataType: "json",
			contentType: "application/json",
			success: function( response )
			{
				callback(true, response);
			},
			error: function(/*err_type, text_status*/)
			{
				callback(false);
			}
		});		
	};

	adm_form_security.prototype.ldap_get_validation_def = function (definitions) {
		const self = this;
		this.ldap_common_validation_definitions.forEach(definition => definitions.push(definition));

		const on_error_fn = function() { self.main_tabs.switch_tab("security"); };
		const $ldap_panel = $("#adm_form_security_authentication_ldap_panel");
		const get_group_definition = function(group) {
			let groupId = "adm_form_security_authentication_ldap_group_" + group;
			let filterId = groupId + "_filter"; 
			return {
				control_id: filterId,
				can_validate: true,
				messages: {
					required: FORMAT("<%=_T('Filter for group {0} is required.')%>", group),
					custom: {
						valid: function() {
							const response = {
								success: true
							};
							const filters = [];
							$ldap_panel.find("[LDAP_ROW_GROUP_NAME]").each(function(_, item) {
								const $item = $(item);
								const selectedGroup = $item.attr("ldap_row_group_name");
								const $ldapGroupFilter = $item.find("[ldap_group_filter]");
								const selectedFilterValue = $ldapGroupFilter.val();
								if(selectedFilterValue !== "") {
									const selectedFilterId = $ldapGroupFilter.attr("id");
									if(filters[selectedFilterValue] && filters[selectedFilterValue] !== selectedFilterId) {
										response.success = false;
										response.message = FORMAT("<%=_T('Selected filter is in use by role {0}')%>", selectedGroup);
										return false;
									}
									filters[selectedFilterValue] = selectedFilterId;
								}
							});
						}
					}
				},
				on_error: on_error_fn
			};
		};

		let ldap_sync_validation_definitions = [];	
		if($("#adm_form_security_authentication_ldap_sync_enabled").prop("checked")) {

			var can_validate_security_authentication_ldap_sync_fn = function() {
				return $("#adm_form_security_authentication_type").val() === "LDAP";
			};

			$ldap_panel.find("[LDAP_GROUP_NAME]").each(function(_, item) {
				const $item = $(item);
				const group = $item.val();
				if (group === "allusers")
					return;
				ldap_sync_validation_definitions.push(
					get_group_definition(group)
				);
			});

			ldap_sync_validation_definitions.push(
				{
				// timezone
					control_id: "adm_form_security_authentication_ldap_sync_timezone",
					can_validate: can_validate_security_authentication_ldap_sync_fn,
					messages: {
						required: "<%=_T('Time Zone is required.')%>"
					},
					on_error: on_error_fn
				},
				{
				// language
					control_id: "adm_form_security_authentication_ldap_sync_language",
					can_validate: can_validate_security_authentication_ldap_sync_fn,
					messages: {
						required: "<%=_T('Language is required.')%>"
					},
					on_error: on_error_fn
				},
				{
				// country
					control_id: "adm_form_security_authentication_ldap_sync_country",
					can_validate: can_validate_security_authentication_ldap_sync_fn,
					messages: {
						required: "<%=_T('Country is required.')%>"
					},
					on_error: on_error_fn
				},
				{
				// area
					control_id: "adm_form_security_authentication_ldap_sync_area",
					can_validate: can_validate_security_authentication_ldap_sync_fn,
					messages: {
						required: "<%=_T('Area code is required.')%>"
					},
					on_error: on_error_fn
				},
				{
				// campaign
					control_id: "adm_form_security_authentication_ldap_sync_default_campaign",
					can_validate: can_validate_security_authentication_ldap_sync_fn,
					messages: {
						required: "<%=_T('Default Campaign is required.')%>"
					},
					on_error: on_error_fn
				},
				{
				// state
					control_id: "adm_form_security_authentication_ldap_sync_initial_state",
					can_validate: can_validate_security_authentication_ldap_sync_fn,
					messages: {
						required: "<%=_T('Initial State is required.')%>"
					},
					on_error: on_error_fn
				}
			);
		}
		else {
			ldap_sync_validation_definitions.push(
				get_group_definition("allusers")
			);
		}

		ldap_sync_validation_definitions.forEach(definition => definitions.push(definition));
	};

	adm_form_security.prototype.azuread_get_validation_def = function (definitions) {
		const self = this;
		this.azuread_common_validation_definitions.forEach(definition => definitions.push(definition));

		const on_error_fn = function() { self.main_tabs.switch_tab("security"); };
		const $azured_panel = $("#adm_form_security_authentication_azuread_sync_panel");

		const get_group_definition = function(group) {
			let groupId = "adm_form_security_authentication_azuread_group_" + group;
			let filterId = groupId + "_filter"; 
			return {
				control_id: filterId,
				can_validate: true,
				messages: {
					required: FORMAT("<%=_T('Azure AD Group for group {0} is required.')%>", group),
					custom: {
						valid: function() {
							const response = {
								success: true
							};
							const filters = [];
							$azured_panel.find("[AZUREAD_ROW_GROUP_NAME]").each(function(_, item) {
								const $item = $(item);
								const selectedGroup = $item.attr("AZUREAD_ROW_GROUP_NAME");
								const $filter = $item.find("[AZUREAD_GROUP_FILTER]");
								const selectedFilterValue = $filter.val();
								if(selectedFilterValue !== "") {
									const selectedFilterId = $filter.attr("id");
									if(filters[selectedFilterValue] && filters[selectedFilterValue] !== selectedFilterId) {
										response.success = false;
										response.message = FORMAT("<%=_T('Selected Azure AD Group is in use by role {0}')%>", selectedGroup);
										return false;
									}
									filters[selectedFilterValue] = selectedFilterId;
								}
							});
							return response;
						}
					}
				},
				on_error: on_error_fn
			};
		};

		let azure_sync_validation_definitions = [];	
		$azured_panel.find("[AZUREAD_GROUP_NAME]").each(function(_, item) {
			const $item = $(item);
			const group = $item.val();
			if (group === "allusers")
				return;
			azure_sync_validation_definitions.push(
				get_group_definition(group)
			);
		});

		const can_validate_security_authentication_azuread_sync_fn = function() {
			return $("#adm_form_security_authentication_type").val() === "OPENID" && $("#adm_form_security_authentication_openid_sync_method").val() === "AZUREAD";
		};

		azure_sync_validation_definitions.push(
			{
			// timezone
				control_id: "adm_form_security_authentication_azuread_sync_timezone",
				can_validate: can_validate_security_authentication_azuread_sync_fn,
				messages: {
					required: "<%=_T('Time Zone is required.')%>"
				},
				on_error: on_error_fn
			},
			{
			// language
				control_id: "adm_form_security_authentication_azuread_sync_language",
				can_validate: can_validate_security_authentication_azuread_sync_fn,
				messages: {
					required: "<%=_T('Language is required.')%>"
				},
				on_error: on_error_fn
			},
			{
			// country
				control_id: "adm_form_security_authentication_azuread_sync_country",
				can_validate: can_validate_security_authentication_azuread_sync_fn,
				messages: {
					required: "<%=_T('Country is required.')%>"
				},
				on_error: on_error_fn
			},
			{
			// area
				control_id: "adm_form_security_authentication_azuread_sync_area",
				can_validate: can_validate_security_authentication_azuread_sync_fn,
				messages: {
					required: "<%=_T('Area code is required.')%>"
				},
				on_error: on_error_fn
			},
			{
			// campaign
				control_id: "adm_form_security_authentication_azuread_sync_default_campaign",
				can_validate: can_validate_security_authentication_azuread_sync_fn,
				messages: {
					required: "<%=_T('Default Campaign is required.')%>"
				},
				on_error: on_error_fn
			},
			{
			// state
				control_id: "adm_form_security_authentication_azuread_sync_initial_state",
				can_validate: can_validate_security_authentication_azuread_sync_fn,
				messages: {
					required: "<%=_T('Initial State is required.')%>"
				},
				on_error: on_error_fn
			}
		);

		azure_sync_validation_definitions.forEach(definition => definitions.push(definition));
	};

	adm_form_security.prototype.fill_security_authentication_openid_sync_methods = function(openIdEnabled)
	{
		var types = [];
		if(openIdEnabled) {
			types.push({id: "AZUREAD", description: "<%=_T('Azure AD API Graph')%>"});
		}
		this.fill_combo($("#adm_form_security_authentication_openid_sync_method"), false, types, "id", "description");
	};

	adm_form_security.prototype.reload_security_authentication_azuread_groups = function()
	{
		const self = this;

		//clean filters
		const $azuread_panel = $("#adm_form_security_authentication_azuread_sync_panel");
		$azuread_panel.find("[AZUREAD_ROW_GROUP_NAME]").each(function(_, item) {
			const $item = $(item);
			const group = $item.attr("AZUREAD_ROW_GROUP_NAME");
			const selector = jQUERY_SELECTOR_AZUREAD_GROUP_FILTER(group);
			const $filter = $azuread_panel.find(selector);
			$filter.empty();
		});
		let validation_def = {
			control_id: "adm_form_security_authentication_openid_configuration_url",
			can_validate: true,
			messages: {
				custom: {
					has_error: false,
					error_message: "",
					type : "custom",
					valid: function() {
						let response = {
							success: true
						};
						if(this.has_error) {
							response.success = false;
							response.message = this.error_message;
						}
						return response;
					}
				}
			}
		};
		let definitions = [];
		definitions.push(validation_def);

		const $azuread_sync_country = $("#adm_form_security_authentication_azuread_sync_country");
		const $chk = $("#adm_form_security_authentication_azuread_sync_enabled");
		if (!$chk.prop("checked")) {
			$azuread_sync_country.trigger("change");
			return;
		}
		this.validate(function( success ) {
			if (!success) {
				$chk.prop("checked", false);
				return;
			}
			const $password =  $("#adm_form_security_authentication_azuread_sync_client_secret"); 
			const jwtToken  = inConcert.Utils.GetJwtToken($password.val());							
			var data = {
				vcc: inConcert.app.getDomain(),
				openid_configuration_url : $("#adm_form_security_authentication_openid_configuration_url").val(),
				client_id : $("#adm_form_security_authentication_azuread_sync_client_id").val(),
				password : jwtToken,
				source: "azuread"
			};
			self.dashboard.show_waiting("<%=_T('Recovering Azure AD groups')%>");
			self.load_external_entities(data,
				function(success, response) {
					self.dashboard.hide_waiting();
					if (!success) {
						validation_def.messages.custom.has_error = true;
						validation_def.messages.custom.error_message = response;
						self.validate(function(_) {}, definitions);
						console.log(response);
						$chk.prop("checked", false);
						return;
					}
					if (!response.success) {
						validation_def.messages.custom.has_error = true;
						validation_def.messages.custom.error_message = "<%=_T('Azure AD Error:')%>" + " (" + response.error + ")";
						self.validate(function(_) {}, definitions);
						console.log(response.error);
						$chk.prop("checked", false);
						return;
					}
					if (!response.result || response.result.length === 0) {
						validation_def.messages.custom.has_error = true;
						validation_def.messages.custom.error_message =  "<%=_T('No results returned')%>";
						self.validate(function(_) {}, definitions);
						$chk.prop("checked", false);
						return;
					}
					const groupFilters = self.m_security && self.m_security.openid && self.m_security.openid.groupFilters || {};
					$azuread_panel.find("[AZUREAD_ROW_GROUP_NAME]").each(function(_, item) {
						const $item = $(item);
						const group = $item.attr("AZUREAD_ROW_GROUP_NAME");
						const selector = jQUERY_SELECTOR_AZUREAD_GROUP_FILTER(group);
						const $filter = $azuread_panel.find(selector);
						self.fill_combo($filter, false, response.result, "id", "displayName");
						$filter.val(groupFilters[group] && groupFilters[group].id);
					});
				});
			},
			this.azuread_common_validation_definitions
		);

		$azuread_sync_country.trigger("change");
	};

	adm_form_security.prototype.build_footer_custom_buttons = function()
	{
		const html_buttons = '<button id="adm_form_security_authentication_edit" class="pageButtonGeneric pageButtonNew buttonColorOrange" type="button" hidden style="display: none;"><i class="fal fa-pencil"></i><%=_T("Edit")%></button>';
		adm_form_security.uber.build_footer_custom_buttons.call(this, html_buttons);
	};

	adm_form_security.prototype.ready_for_save = function()
	{
		adm_form_security.uber.ready_for_save.call(this);
		if (this.instance_config?.show_view_readonly !== false) {
			this.no_close();
		}
	};

}());
