/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 9 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	"use strict";

	const FORMAT = inConcert.Utils.format;

	//---------------------------------------------------------------------
	// form shared files class
	function adm_form_shared_files(params)
	{
		/* jshint validthis: true*/
		const success = adm_form_shared_files.uber.constructor.call(this, params);
		if (!success) {
			return;
		}

		this.maintenance_mng = new inConcert.shared_files_maintenance_mng({
			owner              : this,
			container_id       : "adm_form_shared_file_container",
			prefix_id          : "adm_form_shared_file_",
			allowsMultipleFiles: true,
			maximum_number_of_files_in_import: 100
		});

		this.new_title      = "<%=_T('New File')%>";
		this.update_title   = "<%=_T('Update File {0}')%>";
		this.view_title     = "<%=_T('View File {0}')%>";
		this.tagline        = "<%=_T('Tagline form Shared File!!!')%>";
		this.include_save_and_continue = false;
		this.html_file_name = "/inconcert/apps/administration/shared_files/frm/render/";
		this.save_uri       = "/inconcert/apps/administration/shared_files/save/";
		this.get_uri        = "/inconcert/apps/administration/shared_files/get/";

		return true;
	}

	inConcert.Utils.inherit(adm_form_shared_files, inConcert.conf_form);

	if (typeof inConcert.adm_form_shared_files !== "function") {
		inConcert.adm_form_shared_files = adm_form_shared_files;
	}
	adm_form_shared_files.prototype.initialize = function()
	{
		this.maintenance_mng.initialize();
	};

	adm_form_shared_files.prototype.load_data = function(callback_complete_requests)
	{
		this.maintenance_mng.load_data();
		adm_form_shared_files.uber.load_data.call(this, callback_complete_requests);
	};

	adm_form_shared_files.prototype.set_events = function()
	{
		adm_form_shared_files.uber.set_events.call(this);
		if (this.is_readonly()) {
			return;
		}
		this.maintenance_mng.set_events();
	};

	adm_form_shared_files.prototype.get = function()
	{
		let file_info = this.maintenance_mng.get();
		if (!this.is_creating()) {
			return file_info;
		}
		let array_files = file_info;
		let files = inConcert.Utils.arrayToMap(array_files || [],
			function(file) {
				return file;
			},
			function(file) {
				// el ID
				return (file.Name || "").toLowerCase();
			}
		);
		return files;
	};

	adm_form_shared_files.prototype.set = function(success, file)
	{
		this.maintenance_mng.setMultipleFiles(false);
		this.maintenance_mng.set(success, file);
	};

	adm_form_shared_files.prototype.empty_controls = function()
	{
		this.maintenance_mng.empty_controls();
	};

	adm_form_shared_files.prototype.on_close = function(action)
	{
		this.maintenance_mng.setMultipleFiles(true);
		this.maintenance_mng.on_close(action);
		adm_form_shared_files.uber.on_close.call(this, action);
	};

	adm_form_shared_files.prototype.err_save_messages = function(err)
	{
		this.maintenance_mng.err_save_messages(err);
	};

	adm_form_shared_files.prototype.get_validation_def = function()
	{
		return this.maintenance_mng.get_validation_def();
	};

	adm_form_shared_files.prototype.validate = function( save_callback, validation_def )
	{
		save_callback = save_callback || $.noop;
		let success = this.maintenance_mng.validate();
		if (!success) {
			return save_callback(false);
		}
		adm_form_shared_files.uber.validate.call(this, save_callback, validation_def);
	};

adm_form_shared_files.prototype.save = function(callback)
{
	adm_form_shared_files.uber.save.call( this, callback ); //invokes the base class method
};

	adm_form_shared_files.prototype.ready = function()
	{
		adm_form_shared_files.uber.ready.call(this); //invokes the base class method
		let file_type, info;
		if (this.instance_config) {
			file_type = this.instance_config.action_id; // audio, image, file, video
			info = this.instance_config.info;
			if (file_type === "telephony_audio") {
				file_type = "audio";
				info = { ...(info || {}), telephonyAudio: true };
			}
		}
		this.maintenance_mng.ready(file_type, info);
		if (this.is_creating()) {
			let file_type_info = this.maintenance_mng.getFileTypeInfo(file_type);
			if (file_type_info && file_type_info.label) {
				let title = FORMAT("<%=_T('New {0}')%>", file_type_info.label);
				this.set_title(title);
			}
		}
	};

	adm_form_shared_files.prototype.get_hash_id = function(entity)
	{
		return entity.Id;
	};

	adm_form_shared_files.prototype.popup_show = function(params)
	{
		adm_form_shared_files.uber.popup_show.call(this, params);
		this.maintenance_mng.popup_show(params);
	};

	adm_form_shared_files.prototype.popup_html = function(params)
	{
		return this.maintenance_mng.popup_html(params);
	};

	adm_form_shared_files.prototype.popup_set_events = function(params)
	{
		this.maintenance_mng.popup_set_events(params);
	};

	adm_form_shared_files.prototype.popup_error = function(type, error) {
		let $error;
		switch(type) {
			case "import_files":
				$error = $("#adm_form_import_shared_files_message");
				break;
		}
		$error.toggle(!!error).text(error || "");
	};

	adm_form_shared_files.prototype.popup_hide = function(has_been_imported)
	{
		adm_form_shared_files.uber.popup_hide.call(this); //invokes the base class method
		if (has_been_imported) {
			const $btn_search = this.$get_dom_element_by_id("filter_button");
			$btn_search.trigger("click");
		}
	};

	adm_form_shared_files.prototype.__make_delete_warning_detail = function(warnings)
	{
		let detail = "";

		const make_detail_fields = function(errInfo, attrName) {
			if (!errInfo || !attrName) {
				return;
			}
			for (let id in errInfo || {}) {
				if (errInfo.hasOwnProperty(id)) {
					let err = errInfo[id];
					let name;
					if (typeof attrName === "function") {
						name = attrName(err, id);
					}
					else {
						name = err.hasOwnProperty(attrName) ? err[attrName] : id;
					}
					detail = detail + "\n";
					detail = detail + "    " + name;
				}
			}
		};

		for (let i = 0; i < (warnings || []).length; i++) {
			let warning = warnings[i];

			if (detail) {
				detail = detail + "\n";
			}
			detail = detail + warning.fileId + " - ";

			if (warning.errType === "FILE USED IN INTERACTIONS") {
				detail = detail + "<%=_T('File used in conversations')%>";
			}
			else if (warning.errType === "FILE USED IN CAMPAIGN ACCOUNTS") {
				if (warning.totalUsedInAccountMessages === 1) {
					detail = detail + "<%=_T('File used in accounts messages from 1 campaign')%>";
				}
				else {
					detail = detail + FORMAT("<%=_T('File used in accounts messages from {0} campaigns')%>", warning.totalUsedInAccountMessages);
				}
				make_detail_fields(warning.errInfo, function(err, id) {
					if (!err.campaignId) {
						return id;
					}
					return inConcert.app.get_id(err.campaignId);
				});
			}
			else if (warning.errType === "FILE USED IN FLOW DESIGNER") {
				if (warning.totalUsedInFlowDesigner === 1) {
					detail = detail + "<%=_T('File used in 1 flow process')%>";
				}
				else {
					detail = detail + FORMAT("<%=_T('File used in {0} flow processes')%>", warning.totalUsedInFlowDesigner);
				}
				make_detail_fields(warning.errInfo, "flowName");
			}
			else {
				detail = detail + "<%=_T('File used')%>";
			}
		}

		return detail;
	};

	adm_form_shared_files.prototype.execute_batch_delete_action = function(selected_grid_items, callback = $.noop) {
		const self = this;
		let files_to_delete = [];
		selected_grid_items.forEach(item => {
			files_to_delete.push(item.Id);
		});
		const get_message_error = function(result) {
			let err_code = (result.code || "").toUpperCase();
			if (err_code === "THERE ANOTHER PROCESS BATCH IN PROGRESS") {
				return "<%=_T('Unable to complete action. There is another batch process in progress.')%>";
			}
			else if (err_code === "SHARED FILES TO DELETE DO NOT EXIST IN DATABASE") {
				return "<%=_T('Unable to complete action. The files to delete do not exist in the database.')%>";
			}
			else if (err_code === "NO FILE COULD BE DELETED" || err_code === "SOME FILES COULD NOT BE DELETED") {
				let title, message, msg_type;
				if (err_code === "NO FILE COULD BE DELETED") {
					// NONE DELETED FILES
					title = "<%=_T('No files could be deleted, all are in use.')%>";
					message = "<%=_T('The files that have not been deleted are')%>";
					msg_type = "error";
					result.must_refresh_grid = false;
				}
				else {
					// PARTIAL DELETED FILES
					title = "<%=_T('Not all Files have been deleted.')%>";
					message = FORMAT("<%=_T('{0} files have been deleted and {1} could not be deleted. The files that have not been deleted are')%>", result.totalDeleted, result.totalNotDeleted);
					msg_type = "warn";
					result.must_refresh_grid = true;
				}
				let detail = self.__make_delete_warning_detail(result.errors);
				self.show_popup_message({
					messages: {
						title  : title,
						message: message + "<%=_T(':')%>",
						detail : detail,
						type   : msg_type
					}
				});
				// sin mensaje de error en el titulo de la vista
				return "";
			}

			// el error sale generico desde conf_form.js
			return;
		};
		this.maintenance_mng.deleteFiles(files_to_delete, function(success, result) {
			if (success) {
				callback(true, result);
				return;
			}
			result.message = get_message_error(result);
			callback(false, result);
		});
	};

	adm_form_shared_files.prototype.get_batch_menu_options = function() {
		const self = this;
		const filesAPI = inConcert.app.getSharedFilesAPI();

		let system_menu_actions = adm_form_shared_files.uber.get_batch_menu_options.call(this);
		let custom_menu_options = [
			{
				id     : "export",
				name   : "<%=_T('Export')%>",
				execute: function() {
					self.execute_batch_action(function(selected_items, callback = $.noop) {
						let files_to_export = [];
						selected_items.forEach(item => {
							files_to_export.push(item.Id);
						});
						filesAPI.startExportFilesToCompressed(files_to_export, function(success, err) {
							let resp = err || {};
							if (success) {
								resp.message = "<%=_T('Selected items have been exported.')%>";
							}
							else {
								resp.message = err.message || "<%=_T('Unable to export selected items.')%>";
							}
							callback(success, resp);
						});
					});
				}
			}
		];
		return custom_menu_options.concat(system_menu_actions);
	};

	adm_form_shared_files.prototype.get_new_menu_options = function(callback_execute = $.noop) {
		const self = this;
		let options = [
			{
				id     : "audio",
				name   : "<%=_T('Audio')%>",
				execute: function() {
					callback_execute("audio");
				}
			},
			{
				id     : "telephony_audio",
				name   : "<%=_T('Telephony Audio')%>",
				execute: function() {
					callback_execute("telephony_audio");
				}
			},
			{
				id     : "image",
				name   : "<%=_T('Image')%>",
				execute: function() {
					callback_execute("image");
				}
			},
			{
				id     : "video",
				name   : "<%=_T('Video')%>",
				execute: function() {
					callback_execute("video");
				}
			},
			{
				id     : "file",
				name   : "<%=_T('File')%>",
				execute: function() {
					callback_execute("file");
				}
			},
			{
				id     : "csv",
				name   : "<%=_T('From CSV')%>",
				execute: function() {
					self.__open_csv_poopup(callback_execute);
				}
			}
		];
		return options;
	};

	adm_form_shared_files.prototype.__open_csv_poopup = function(callback_execute = $.noop) {
		this.dashboard.view.render_popup({
			type: "import_csv_file",
			super_popup: false,
			callbacks: {
				uploaded: function(success, type, files) {
					if (success) {
						type = type || "image";
						let info = {
							from: "csv",
							files: files
						};
						callback_execute(type, info);
					}
				}
			}
		});
	};

	adm_form_shared_files.prototype.popup_on_render = function(params)
	{
		this.maintenance_mng.popup_on_render(params);
	};

}());
