/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 9 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	const FORMAT = inConcert.Utils.format;
	const LIMIT_TIME_FROM = 1;
	const LIMIT_TIME_TO = 1400;

	//---------------------------------------------------------------------
	// form allegro states class
	function adm_form_states_allegro(params)
	{
		const success = adm_form_states_allegro.uber.constructor.call(this, params);
		if (!success) {
			return;
		}
		return true;
	}

	inConcert.Utils.inherit(adm_form_states_allegro, inConcert.adm_form_states);

	if (typeof inConcert.adm_form_states_allegro !== "function") {
		inConcert.adm_form_states_allegro = adm_form_states_allegro;
	}

	adm_form_states_allegro.prototype.apply_initial_config = function()
	{
		adm_form_states_allegro.uber.apply_initial_config.call(this); //invokes the base class method
		if (this.is_new) {
			this.empty_controls();
		}
	};

	adm_form_states_allegro.prototype.get = function()
	{
		let state = adm_form_states_allegro.uber.get.call(this); //invokes the base class method
		state.Enabled = $("#adm_form_state_is_active").prop("checked") ? 1 : 0;
		state.Timeout = $("#adm_form_state_limit").prop("checked") ? $("#adm_form_state_limit_time").val() : 0;
		return state;
	};

	adm_form_states_allegro.prototype.set = function(success, state)
	{
		if (!success) {
			return;
		}
		adm_form_states_allegro.uber.set.call(this, success, state); //invokes the base class method

		this.readonly_control($("#adm_form_state_name"), true);
		$("#adm_form_state_is_active").prop("checked", (state.Enabled || "0").to_boolean());

		const $limit = $("#adm_form_state_limit");
		const timeout = Number(state.Timeout) || 0;
		if (timeout > 0) {
			$("#adm_form_state_limit_time").val(timeout);
			$limit.prop("checked", true);
		}
		else {
			$limit.prop("checked", false);
		}
	};

	adm_form_states_allegro.prototype.ready_for_update = function()
	{
		adm_form_states_allegro.uber.ready_for_update.call(this);
		const $limit = $("#adm_form_state_limit");
		$limit.trigger("change");
		window.setTimeout(function() {$("#adm_form_state_desc").focus();});
	};

	adm_form_states_allegro.prototype.empty_controls = function()
	{
		adm_form_states_allegro.uber.empty_controls.call(this); //invokes the base class method
		$("#adm_form_state_is_active").prop("checked", true);
		$("#adm_form_state_limit").prop("checked", false).trigger("change");
		window.setTimeout(function() {$("#adm_form_state_name").focus();});
	};

	adm_form_states_allegro.prototype.get_validation_def = function()
	{
		let definitions = adm_form_states_allegro.uber.get_validation_def.call(this); //invokes the base class method
		definitions.push({
			control_id: "adm_form_state_limit_time",
			messages: {
				required: "<%=_T('Please enter the State Time Limit.')%>",
				range: {
					between: {
						from : LIMIT_TIME_FROM,
						to   : LIMIT_TIME_TO
					},
					message: "<%=_T('Please enter the State Time Limit, between {0} to {1} minutes.')%>"
				}
			}
		});
		return definitions;
	};

	adm_form_states_allegro.prototype.err_save_messages = function(err)
	{
		let response = {};

		if (typeof err === "object") {
			if ( err.result === "ALLEGRO_INTEGRATION_ERROR" ) {
				const allegro_err = err.err || {};
				if (allegro_err.sub_type.err === "UNABLE_TO_UPDATE_ALLEGRO_SYSTEM_STATE") {
					response.message = "<%=_T('Unable to create State. You have used a system name.')%>";
				}
				response.message = response.message || "<%=_T('Error updating information on Allegro integration...')%>";
				$("#adm_form_state_desc").focus();
				response.is_custom_message = true;
			}
			else if ( err.result === "ALLEGRO_INTEGRATION_ERROR_STATE_LIMIT_OUT_OF_RANGE" ) {
				response.message = FORMAT("<%=_T('State Time Limit out of range. Please enter values between {0} to {1} minutes.')%>", LIMIT_TIME_FROM, LIMIT_TIME_TO);
				$("#adm_form_state_limit_time").focus();
				response.is_custom_message = true;
			}
			else if ( err.result === "ALLEGRO_INTEGRATION_OVERFLOW_NAME" ) {
				response.message = "<%=_T('Unable to save. Name exceeds the allowed length.')%>";
				$("#adm_form_state_name").focus();
				response.is_custom_message = true;
			}
		}

		if ( response.is_custom_message ) {
			adm_form_states_allegro.uber.err_save_messages.call( this, response.message, response.is_custom_message ); //invokes the base class method
		}
		else {
			adm_form_states_allegro.uber.err_save_messages.call( this, err, false ); //invokes the base class method
		}
	};

}());
