/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 6 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	"use strict";
	
	let format = inConcert.Utils.format;

	//---------------------------------------------------------------------
	// form api users class
	function adm_form_ttsasr_engines(params)
	{
		/* jshint validthis: true*/
		const success = adm_form_ttsasr_engines.uber.constructor.call(this, params);
		if (!success) {
			return;
		}

		this.new_title = "<%=_T('Create TTS/ASR Engine')%>";
		this.update_title = "<%=_T('Edit TTS/ASR Engine {0}')%>";
		this.view_title = "<%=_T('View TTS/ASR Engine {0}')%>";
		this.tagline = "<%=_T('Tagline for TTS/ASR Engine!!!')%>";

		this.html_file_name = "/inconcert/apps/administration/ttsasr_engines/frm/render/";
		this.save_uri       = "/inconcert/apps/administration/ttsasr_engines/save/";
		this.get_uri        = "/inconcert/apps/administration/ttsasr_engines/get/";

		this.engine = null;
		
		this.voices_mng = new inConcert.AdminTtsVoicesMng({
			owner       : this,
			prefix      : "adm_form_ttsasr_engines_voices",
			container_id: "adm_form_ttsasr_engines_voices_panel"
		});
		
		this.grammars_mng = new inConcert.AdminAsrGrammarsMng({
			owner       : this,
			prefix      : "adm_form_ttsasr_engines_grammars",
			container_id: "adm_form_ttsasr_engines_grammars_panel"
		});
		
		this.m_grid_recognitionLanguages = new inConcert.conf_form_paginator({
			owner: this,
			frm: this,
			row_id: "id",
			display_name: "id",
			rows_per_page: 15,
			show_at_least: 14,
			paginator_type: "ALL_ROWS_CLIENT_SIDE",
			without_paginator: false,
			loading_waiting_msg: "<%=_T('Please wait, loading Recognition Languages ...')%>",
			ids: {
				paginator: "adm_form_ttsasr_engines_recognitionLanguages_paginator",
				grid: "adm_form_ttsasr_engines_recognitionLanguages_list",
				not_results_found: "adm_form_ttsasr_engines_recognitionLanguages_not_results_found"
			},
			strings: {
				not_results_found: "<%=_T('No Recognition Languages found.')%>",
			},
			columns: [
				{id: "_number"           , name: "", type: "counter"                            },
				{id: "id"                , name: "<%=_T('Id')%>"                                },
				{id: "LanguageName"      , name: "<%=_T('Language Name')%>"                     },
				{
					id: "_action",
					name: "",
					system_action: {
						edit: false,
						erase: false
					}
				}
			]
		});
		
		this.sorted_volumes = [
			{id: "silent" , name: "silent"},
			{id: "x-soft" , name: "x-soft"},
			{id: "soft"   , name: "soft"},
			{id: "medium" , name: "medium"},
			{id: "load"   , name: "load"},
			{id: "x-loud" , name: "x-loud"},
			{id: "default", name: "default"}
		];
		
		this.sorted_rates  = [
			{id: "x-slow" , name: "x-slow"},
			{id: "slow"   , name: "slow"},
			{id: "medium" , name: "medium"},
			{id: "fast"   , name: "fast"},
			{id: "x-fast" , name: "x-fast"},
			{id: "default", name: "default"}
		];
		
		return true;
	}

	inConcert.Utils.inherit(adm_form_ttsasr_engines, inConcert.conf_form);

	if (typeof inConcert.adm_form_ttsasr_engines !== "function") {
		inConcert.adm_form_ttsasr_engines = adm_form_ttsasr_engines;
	}
	
	adm_form_ttsasr_engines.prototype.initialize = function(callback_complete_requests)
	{
		this.voices_mng.initialize();
		this.grammars_mng.initialize();
	};

	adm_form_ttsasr_engines.prototype.load_data = function(callback_complete_requests)
	{
		this.__fill_prosodyVolume();
		this.__fill_prosodyRate();
		adm_form_ttsasr_engines.uber.load_data.call(this, callback_complete_requests);
	};

	adm_form_ttsasr_engines.prototype.__fill_prosodyVolume = function()
	{
		this.voices_mng.fill_prosodyVolume();
	};

	adm_form_ttsasr_engines.prototype.__fill_prosodyRate = function()
	{
		this.voices_mng.fill_prosodyRate();
	};

	adm_form_ttsasr_engines.prototype.get = function()
	{
		this.engine.voices = this.voices_mng.get();
		this.engine.grammars = this.grammars_mng.get();
		//No se necesita setear this.engine.recognitionLanguages porque no permitimos editarlos aqui
		
		return this.engine;
	};

	adm_form_ttsasr_engines.prototype.set = function(success, engine)
	{
		if (!success) {
			return;
		}
		$("#adm_form_ttsasr_engines_description").val(engine.engineDesc);
		$("#adm_form_ttsasr_engines_profile").val(engine.profileDesc);
		$("#adm_form_ttsasr_engines_provider").val(engine.providerDesc);
		$("#adm_form_ttsasr_engines_type").val(engine.providerType);
		this.engine = engine;
		
		this.voices_mng.set(engine);
		this.grammars_mng.set(engine);
		this.m_grid_recognitionLanguages.initialize($.noop, engine.recognitionLanguages);
		
		//Ocultamos lo que no proceda
		if(engine.providerType === "TTS") {
			$("#adm_form_ttsasr_engines_grammars_panel").hide();
			$("#adm_form_ttsasr_engines_recognitionLanguages_panel").hide();
		}
		if(engine.providerType === "ASR") {
			$("#adm_form_ttsasr_engines_voices_panel").hide();
		}
		
		//TODO: Deshabilitar el boton de nueva grammar para speechmatic
		if(engine.engineId === "speechmatic") {
			this.readonly_control($("#adm_form_ttsasr_engines_grammars_new"), true);
		}
	};
	
	adm_form_ttsasr_engines.prototype.can_save_entity = function(engine) 
	{
		return !engine.System;
	};

	adm_form_ttsasr_engines.prototype.empty_controls = function()
	{
		
	};

	adm_form_ttsasr_engines.prototype.err_save_messages = function(err)
	{	
		//TODO: Acomodar a mensajes que procedan
		let is_custom_message = false;
		let message = null;

		if ( typeof err === "object" ) {
			let error_type = err.result.toUpperCase();
			if (error_type === "ID_DOES_NOT_EXIST" || error_type === "ID_ALREADY_EXISTS") {
				if (error_type === "ID_ALREADY_EXISTS") {
					message = "<%=_T('Already exists another TTS/ASR Engine with same id')%>";
				}
			}
			else if (error_type === "MUST_HAVE_A_DESCRIPTION") {
				$("#adm_form_ttsasr_engines_description").focus();
			}
			if (message) {
				is_custom_message = true;
				err = message;
			}
		}

		adm_form_ttsasr_engines.uber.err_save_messages.call( this, err, is_custom_message ); //invokes the base class method
	};

	adm_form_ttsasr_engines.prototype.get_validation_def = function()
	{
		let definitions = [
			{
				control_id: "adm_form_ttsasr_engines_description",
				messages: {
					required: "<%=_T('Please enter the TTS/ASR Engine description.')%>"
				}
			}
		];
		return definitions;
	};

	adm_form_ttsasr_engines.prototype.validate = function( save_callback, validation_def )
	{
		save_callback = save_callback || $.noop;
		
		//if (!$("[notification_type]:checked").length) {
		//	this.validation.show_validation_error(null, 
		//		"<%=_T('Must select almost one event.')%>");
		//	return save_callback(false);
		//}

		adm_form_ttsasr_engines.uber.validate.call(this, save_callback, validation_def);
	};

	adm_form_ttsasr_engines.prototype.ready = function()
	{
		adm_form_ttsasr_engines.uber.ready.call( this );  //invokes the base class method

		this.readonly_control($("#adm_form_ttsasr_engines_description"), true);
		this.readonly_control($("#adm_form_ttsasr_engines_profile"), true);
		this.readonly_control($("#adm_form_ttsasr_engines_provider"), true);
		this.readonly_control($("#adm_form_ttsasr_engines_type"), true);
	};
	
	adm_form_ttsasr_engines.prototype.set_events = function()
	{
		adm_form_ttsasr_engines.uber.set_events.call(this); //invokes the base class method

		this.voices_mng.set_events();
		this.grammars_mng.set_events();
	};
	
}());
