/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 9 */

/* global inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	const format = inConcert.Utils.format;

	//---------------------------------------------------------------------
	// form accounts class
	function adm_form_vcc_accounts( params )
	{
		const self = this;
		const success = adm_form_vcc_accounts.uber.constructor.call( this, params );
		if ( !success ) {
			return;
		}

		this.new_title      = "<%=_T('Create output account')%>";
		this.update_title   = "<%=_T('Edit Output Account {0}')%>";
		this.view_title     = "<%=_T('View Outgoing Account {0}')%>";
		this.tagline        = "<%=_T('Tagline form Outgoing Account!!!')%>";

		this.html_file_name = "/inconcert/apps/administration/vccs/accounts/frm/render/";
		this.save_uri       = "/inconcert/apps/administration/vccs/accounts/save/";
		this.get_uri        = "{0}/inconcert/apps/administration/vccs/accounts/get/{1}/{2}";

		this.mail_hnd = new inConcert.Administration.MailAccountHnd({
			owner       : this, 
			parent      : { frm: this, is_readonly: this.is_readonly,
				has_permission_to_save: function() { return self.has_permission_to_save(); }
			},
			prefix_id   : "adm_form_vcc_accounts_mail_",
			config_tabs : "outgoing",
			validate_name_before_incomming_section: true
		});

		return true;
	}

	inConcert.Utils.inherit( adm_form_vcc_accounts, inConcert.conf_form );

	if ( typeof inConcert.adm_form_vcc_accounts !== "function" ) {
		inConcert.adm_form_vcc_accounts = adm_form_vcc_accounts;
	}

	adm_form_vcc_accounts.prototype.initialize = function()
	{
		this.mail_hnd.initialize();
		if ( this.is_creating() ) {
			this.mail_hnd.set_account( null );
		}
	};

	adm_form_vcc_accounts.prototype.set_events = function()
	{
		adm_form_vcc_accounts.uber.set_events.call( this );  //invokes the base class method
		this.mail_hnd.set_events();
	};

	adm_form_vcc_accounts.prototype.readonly = function()
	{
		adm_form_vcc_accounts.uber.readonly.call( this );
		this.mail_hnd.readonly();
	};

	adm_form_vcc_accounts.prototype.set = function( success, account )
	{
		if (!success) {
			return;
		}
		this.mail_hnd.set_account( account );
		this.mail_hnd.edit();
		this.mail_hnd.on_show();
	};

	adm_form_vcc_accounts.prototype.empty_controls = function()
	{
		this.mail_hnd.set_account( null );
		this.mail_hnd.empty_controls();
	};

	adm_form_vcc_accounts.prototype.err_save_messages = function( err )
	{
		var is_custom_message = false;
		var get_error_description = function(err_type)
		{
			is_custom_message = true;
			var campaign_err = err.campaign || "";
			var mail_user_url_err = err.user || "";
			var mail_err = err.mail || "";

			if ( ( err.url || "" ).length > 0 ) {
				mail_user_url_err = mail_user_url_err + " - " + err.url;
			}

			var message;
			switch (err_type)
			{
				case "ACCOUNT_ALREADY_EXISTS_IN_CURRENT_VCC":
					message = "<%=_T('Unable to save. Outgoing Account already exists.')%>";
					$("#adm_form_vcc_accounts_mail_account").focus();
				break;

				case "ACCOUNT_IS_CONFIGURED_IN_A_CAMPAIGN":
					message = format( "<%=_T('Unable to save. Outgoing Account is configured in {0} campaign.')%>", campaign_err );
					$("#adm_form_vcc_accounts_mail_account").focus();
				break;

				case "MAIL_OUTGOING_USER_IN_USE_IN_ANOTHER_ACCOUNT_CURRENT_CAMPAIGN":
					message = format( "<%=_T('Unable to save. Outgoing user/server {0} in use in account {1} of campaign {2}.')%>", mail_user_url_err, mail_err, campaign_err );
					$("#adm_form_vcc_accounts_mail_user_outgoing").focus();
				break;

				case "MAIL_OUTGOING_USER_IN_USE_IN_ANOTHER_CAMPAIGN":
					message = format( "<%=_T('Unable to save. Outgoing user/server {0} in use in account {1} of campaign {2}.')%>", mail_user_url_err, mail_err, campaign_err );
					$("#adm_form_vcc_accounts_mail_user_outgoing").focus();
				break;

				case "MAIL_OUTGOING_USER_IN_USE_IN_ANOTHER_VCC":
					message = format( "<%=_T('Unable to save. Outgoing user/server {0} in use in another company. For inquiries contact our technical service.')%>", mail_user_url_err );
					$("#adm_form_vcc_accounts_mail_user_outgoing").focus();
				break;

				case "MAIL_OUTGOING_USER_IN_USE_VCCS_ACCOUNTS":
					message = format( "<%=_T('Unable to save. Outgoing user/server {0} in use in outgoing account {1} of vcc.')%>", mail_user_url_err, mail_err );
					$("#adm_form_vcc_accounts_mail_user_outgoing").focus();
				break;

				default:
					is_custom_message = false;
					message = "<%=_T('Unable to save Outgoing Account. Server error!!')%>";
			}
			return message;
		};

		switch (typeof err)
		{
			case "object":
				var error_type = err.result.toUpperCase();
				err = get_error_description(error_type);
			break;

			case "string":
				err = get_error_description(err.toUpperCase());
			break;
		}

		adm_form_vcc_accounts.uber.err_save_messages.call( this, err, is_custom_message );  //invokes the base class method
	};

	adm_form_vcc_accounts.prototype.save = function( callback )
	{
		var account_info = this.mail_hnd.validate();
		if ( !account_info.success ) {
			return;
		}
		var account = account_info.account;

		adm_form_vcc_accounts.uber.save.call( this, callback, account );  //invokes the base class method
	};

	adm_form_vcc_accounts.prototype.ready = function()
	{
		adm_form_vcc_accounts.uber.ready.call( this );  //invokes the base class method
		this.mail_hnd.on_show();
	};

	adm_form_vcc_accounts.prototype.uri_get_entity = function()
	{
		if (this.is_new) {
			return "";
		}
		var name = this.$current_row.attr("row_id");
		var uri = format( this.get_uri, inConcert.app.getBaseDomain(), "mail", name );
		return uri;
	};
}());

