/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 9 */

/* global inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	var format = inConcert.Utils.format;

	function permissions_gui( params )
	{
		if ( !params || typeof params.container_id !== "string" ) {
			throw( "permission gui container_id is required" );
		}

		this.permissions        = new inConcert.perm();
		this.container_id       = params.container_id;
		this.raw_app_actions    = null;
		this.group_permissions  = null;
		this.tree_apps_actions  = null;
		this.overrideable_apps  = null;   // overrideable apps sobre el current override (campaign o vcc)
		this.callback           = params.callback || $.noop;
		this.initialize();

		return true;
	}

	if ( typeof inConcert.perm_gui !== "function" ) {
		inConcert.perm_gui = permissions_gui;
	}

	permissions_gui.prototype.initialize = $.noop;

	permissions_gui.prototype.get_perm_instance = function()
	{
		// retorna la instacia que validan los permisos del current grupo
		return this.permissions;
	};

	permissions_gui.prototype.can_add_app = function()
	{
		return true;
	};

	permissions_gui.prototype.can_add_action = function( app_id, action_id )
	{
		var raw_app = this.get_raw_app( app_id );
		if ( !raw_app || !raw_app.Actions  ) {
			return false;
		}
		var raw_action = raw_app.Actions[ action_id ];
		if ( !raw_action ) {
			return false;
		}
		return raw_action.Domain.only_override !== "1";
	};

	permissions_gui.prototype.belogne_to_override_type = function()
	{
		return true;
	};

	permissions_gui.prototype.is_configurable = function( app_id, action_id )
	{
		var raw_app = this.get_raw_app( app_id );
		if ( !raw_app || !raw_app.Actions  ) {
			return false;
		}
		var raw_action = raw_app.Actions[ action_id ];
		if ( !raw_action ) {
			return false;
		}
		if ( !raw_action.Domain ) {
			return true;
		}
		if ( !raw_action.Domain.hasOwnProperty( "not_is_configurable" ) ) {
			return true;
		}
		// si tiene el valor en "1", no es configurable la accion
		var is_configurable = raw_action.Domain.not_is_configurable !== "1";
		return is_configurable;
	};

	permissions_gui.prototype.get_raw_app = function( app_id )
	{
		return this.raw_app_actions[ app_id ];
	};

	permissions_gui.prototype.is_overrideable_app_in_current_override = function( app_id )
	{
	// valida si la app es overrideable en el current override. Ejemplo si estoy realizando override a nivel vcc o a nivel campaign
		if ( !this.is_overrideable_app( app_id ) ) {
			return false;
		}
		if ( !this.overrideable_apps ) {
			return false;
		}
		return this.overrideable_apps[ app_id ];
	};

	permissions_gui.prototype.is_overrideable_app = function( app_id )
	{
		var raw_app = this.get_raw_app( app_id );
		if ( !raw_app ) {
			return false;
		}
		return raw_app.IsOverrideable === "1";
	};

	permissions_gui.prototype.get_element_id = function( type, app_id, action_id )
	{
		if ( !type || !app_id ) {
			throw("Permissions - get element id error. Parameters is required.");
		}
		var prefix = this.container_id.substring( 1 );
		if ( type === "app" ) {
			return prefix + "_opt_app_" + app_id;
		}
		else if ( type === "group" ) {
			return prefix + "_opt_group_" + app_id + "_" + action_id;
		}
		else if ( type === "action" ) {
			return prefix + "_opt_action_" + app_id + "_" +  action_id;
		}
		else {
			throw("Permissions - get element id error. Parameter type '" + type + "' unidentified.");
		}
	};

	permissions_gui.prototype.get_jquery_selector = function( name, app_id, parent_id )
	{
		/*
		 * Esta funcion retorna un string con el selector de jquery solicitado.
		 * Sirve para no tener que estar repitiendo los selectores mas de una vez, en diferentes lugares y/o funciones.
		 */
		if (typeof name !== "string") {
			throw("jquery selector is not a string.");
		}
		/*
		 *Siempre todo comienza desde el id contenedor
		 *	-> this.container_id = "#...."
		 */

		var selector;
		// base selectors

		switch( name.toLowerCase() )
		{
			case "all-apps":
			// selector para todos chks de las aplicaciones .
				selector = this.container_id  + " .moduleCheckbox";
			break;

			case "all-groups":
			// selector para todos chks de las groups de las apps.
				selector = this.container_id  + " .subModuleCheckbox";
			break;

			case "all-actions":
			// selector para todos chks de las acciones de las apps.
				selector = this.container_id  + " .itemCheckbox";
			break;

			case "all-groups-specific-app":
			// selector para todos chks de las groups de una aplicacion especifica.(ENABLED y disabled)
				selector = this.container_id  + " .subModuleCheckbox[app_id='" + ( app_id || "" ) + "']";
			break;

			case "all-actions-specific-app":
			// selector para todos chks de las acciones de una aplicacion especifica.(ENABLED y disabled)
				selector = this.container_id  + " .itemCheckbox[app_id='" + ( app_id || "" ) + "']";
			break;

			case "all-childs-groups-specific-app-group-parent-is-app":
			// selector para todos chks de las groups de una aplicacion especifica.(ENABLED y disabled)
				selector = this.container_id  + " .subModuleCheckbox[parent_id='" + ( app_id || "" ) + "'][parent_type='app']";
			break;

			case "all-childs-groups-specific-app-group":
			// selector para todos chks de las grupos de una aplicacion especifica. (ENABLED y disabled)
				selector = this.container_id  + " .subModuleCheckbox[parent_id='" + ( parent_id || "" ) + "']";
			break;

			case "all-childs-enabled-groups-specific-app-group":
			// selector para todos chks de los grupos (ENABLED) de una aplicacion especifica.
				selector = this.container_id  + " .subModuleCheckbox[parent_id='" + ( parent_id || "" ) + "']" + ":enabled";
			break;

			case "all-childs-actions-specific-app-group":
			// selector para todos chks de las acciones de una aplicacion especifica.(ENABLED y disabled)
				selector = this.container_id  + " .itemCheckbox[parent_id='" + ( parent_id || "" ) + "']";
			break;

			case "all-childs-enabled-actions-specific-app-group":
			// selector para todos chks de las acciones (ENABLED) de una aplicacion especifica.
				selector = this.container_id  + " .itemCheckbox[parent_id='" + ( parent_id || "" ) + "']" + ":enabled";
			break;

			default:
				throw("jquery selector '" + name + "' does not exist.");
		}

		// retorno el selector
		return selector;
	};

	permissions_gui.prototype.get_permissions = function()
	{
		var self = this;
		var group_permissions = {};

		// aca recorro todas las acciones y agrego las que se encuentren en checked a la lista de aplicacion que corresponda
		$( this.container_id + " div[app_item='true']" ).each( function( i, app_item )
		{
			var $app_item = $( app_item );
			var $chk_app = $app_item.find( "input[type='checkbox']" );
			var is_checked_app = $chk_app.prop( "checked" );

			if ( !is_checked_app ) {
			// si la app no esta marcada, no agrega ninguna accion
				return true; // continue
			}

			var app_id = $app_item.attr( "app_id" );
			$app_item.find( "div[group_item='true'], div[action_item='true']" ).each(
				function( i, action )
				{
					var $action = $( action );
					var $chk_action = $action.find( "input[type='checkbox']" );
					var is_checked_act = $chk_action.prop( "checked" );
					if ( is_checked_act ) {
					// si la accion se encuentra marcada la agrega a la lista
						var action_id = $action.attr( "action_id" );
						group_permissions[ app_id ] = group_permissions[ app_id ] || {};
						group_permissions[ app_id ][ action_id ] = true;
					}
				});
		});

		// si no se ha marcado ningun permiso se va y no continua con las validaciones
		if ( Object.keys( group_permissions ).length === 0 ) {
			return group_permissions;
		}

		// aca valida que tambien se encuentre con permisos la accion padre, de la accion en curso a la cual se ha
		// asignado permiso.
		// si no tiene permisos la tiene que agregar en la lista de acciones a las cuales se le revocara el permiso.
		var revoked_permissions = {};
		$.each( group_permissions, function( app_id, actions )
		{
			var tree_node = self.tree_apps_actions[ app_id ];
			$.each( actions, function( action_id )
			{
				var found_action = self.find_action_in_tree( tree_node, action_id );
				var parent_action = found_action.parent;
				if ( parent_action && !actions[ parent_action.ActionId ] ) {
					revoked_permissions[ app_id ] = revoked_permissions[ app_id ] || {};
					revoked_permissions[ app_id ][ action_id ] = true;
				}
			});
		});

		// recorro la lista de acciones a la cuales se le ha revocado el permiso, y la elimino de forma definitiva
		// de los permisos del grupo.
		$.each( revoked_permissions, function( app_id, actions ) {
			$.each( actions, function( action_id ) {
				// aca elimino la accion
				delete group_permissions[ app_id ][ action_id ];
				if ( Object.keys( group_permissions[ app_id ] ).length === 0 ) {
				// si la app no tiene ninguna accion, tambien elimino la app de la lista de permisos
					delete group_permissions[ app_id ];
				}
			});
		});

		return group_permissions;
	};

	permissions_gui.prototype.is_checked_app = function( app_id )
	{
		var dom_app_id = this.get_element_id( "app", app_id ) || "";
		var $chk_app = $( "#" + dom_app_id );
		if ( $chk_app.length === 0 ) {
			throw( "permissions - checkbox app id '" + dom_app_id + "' does not exist" );
		}
		return $chk_app.prop( "checked" );
	};

	permissions_gui.prototype.is_checked_group = function( app_id, group_id )
	{
		var dom_group_id = this.get_element_id( "group", app_id, group_id )  || "";
		var $chk_group = $( "#" + dom_group_id );
		if ( $chk_group.length === 0 ) {
			throw( "permissions - checkbox group app id '" + dom_group_id + "' does not exist" );
		}
		return $chk_group.prop( "checked" );
	};

	// funcion recursiva que busca una accion determinada en el tree
	permissions_gui.prototype.find_action_in_tree = function( tree_nodes, action_to_find, level )
	{
		if ( tree_nodes[ action_to_find ] ) {
			if ( typeof level === "number" ) {
				tree_nodes[ action_to_find ].level = level;
			}
			return tree_nodes[ action_to_find ];
		}
		if ( typeof level === "number" ) {
			level++;
		}
		for ( var action_id in tree_nodes ) {
			var node = tree_nodes[ action_id ];
			if ( node.childs ) {
				var found_action = this.find_action_in_tree( node.childs, action_to_find, level );
				if ( found_action ) {
					return found_action;
				}
			}
		}
		return false;
	};

	/*
	Esta funcion lo que realiza es generar un tree con la jerarquia de applications->groups->actions, etc etc.
	Esto es asi porque las acciones en la BD se guardan de forma plana una lista de acciones por app sin tener una 
	jerarquia de arbol.
	Aca nosotros necesitamos verla de forma de tree, por eso se realiza esta conversion a tree.
	*/
	permissions_gui.prototype.make_tree_permissions = function( app_actions )
	{
		var self = this;

		if ( typeof app_actions !== "object" || this.tree_apps_actions !== null ) {
			return;
		}
		var cloned_app_actions = inConcert.Utils.cloneObject( app_actions );
		this.tree_apps_actions = {};

		var pending_apps_actions = {}; // aca se almacenan las acciones que quedan pendientes a agregar en el tree

		var add_child_action = function( app_id, parent_action, child_action_id, child_action )
		{
			if ( !self.can_add_action( app_id, child_action_id ) ) {
				return false;
			}
			parent_action.childs = parent_action.childs || {};
			child_action.level = parent_action.level + 1;
			child_action.parent = parent_action;
			parent_action.childs[ child_action_id ] = child_action;
			return true;
		};

		// esta funcion comienza a generar el tree de acciones para una aplicacion
		var make_tree = function( app_id, actions )
		{
			pending_apps_actions[ app_id ] = {};
			self.tree_apps_actions[ app_id ] = {};
			var tree_node = self.tree_apps_actions[ app_id ];

			$.each( actions, function( action_id, action )
			{
				if ( !self.can_add_action( app_id, action_id ) ) {
					return true; // continue
				}

				if ( !action.hasOwnProperty( "DependensOnAction" ) ) {
				// si la accion NO depende ( no es hija ) de ninguna otra, la inserto directo al tree node
					tree_node[ action_id ] = action;
					tree_node[ action_id ].level = 1;
				}
				else {
				// si la accion depende de otra ( es hija )
					var action_to_find = action.DependensOnAction;
					var found_action = self.find_action_in_tree( tree_node, action_to_find, 1 /* level */ );
					if ( !found_action ) {
					// si no encuentra la accion la agrega en las acciones/grupos pendientes a insertar en el tree
						pending_apps_actions[ app_id ][ action_to_find ] = pending_apps_actions[ app_id ][ action_to_find ] || {};
						pending_apps_actions[ app_id ][ action_to_find ][ action_id ] = action;
					}
					else {
					// si la encuentra la inserta como hija
						add_child_action( app_id, found_action, action_id, action );
					}
				}
			});
		};

		// recorro la lista de aplicaciones y armo el tree para cada una de ellas
		$.each( cloned_app_actions, function( app_id, app ) {
			if ( self.can_add_app( app_id ) ) {
				make_tree( app_id, app.Actions );
			}
		});

		// queda en un bulce procesando las acciones pendientes, hasta que ya no quede ninguna accion pendiente.
		//  o sea que ya se hayan agregado todas las acciones/grupos que estuvieran pendientes al tree.
		//  esto tiene que estar en este while ya que las acciones pendientes puedan estar en desorden y procesarse antes
		//  un hijo que un padre.
		var before_count_actions_pending = 0;
		while ( true ) {
			var count_actions_pending = 0;

			// recorre las apps con acciones pendientes
			$.each( pending_apps_actions, function( app_id, pending_actions )
			{
				// recorre las acciones pendientes de la aplicacion
				$.each( pending_actions, function( pend_act_id, child_actions )
				{
					count_actions_pending++;
					var tree_node = self.tree_apps_actions[ app_id ];
					var found_action = self.find_action_in_tree( tree_node, pend_act_id, 1 /* level */ ); // busca en el tree la accion padre
					if ( found_action ) {
					// si existe la accion padre, inserta los hijos que se recorren en el loop
						$.each( child_actions || {}, function( child_pend_act_id, pend_act ) {
							add_child_action( app_id, found_action, child_pend_act_id, pend_act );
						});
						// aca elimino la accion pendiente del loop
						delete pending_apps_actions[ app_id ][ pend_act_id ];
						if ( Object.keys( pending_apps_actions[ app_id ] ).length === 0 ) {
						// si la app no tiene mas acciones pendientes, elimino la accion pendiente del loop
							delete pending_apps_actions[ app_id ];
						}
					}
				});
			});

			if ( count_actions_pending === 0 ) {
			// si no quedan acciones pendientes salgo del while porque el tree se ha terminado de crear
				break;
			}
			if ( before_count_actions_pending === count_actions_pending ) {
			// TODO: #4639 Revisar. Si entra aca, es porque hay un error de codigo y hay que revisar, pero con esto evito que quede en loop para siempre y se cuelgue el navegador
				throw("Error - the loop is not interrupted!!!. Actions pendings: " + JSON.stringify(pending_apps_actions || {}));
			}
			before_count_actions_pending = count_actions_pending;
		}
	};

	permissions_gui.prototype.build_apps_permissions = function( app_actions )
	{
		var self = this;

		this.overrideable_apps = [];
		this.raw_app_actions = app_actions;

		if (typeof app_actions !== "object") {
			return;
		}

		this.make_tree_permissions( app_actions );

		var major_level = 0;
		var find_major_level = function( action, reset )
		{
			if ( reset ) {
				major_level = 0;
			}
			if ( major_level < action.level ) {
				major_level = action.level;
			}
			var childs = action.childs || {};
			$.each( childs, function( i, child_action ) {
				if ( child_action.childs ) {
					find_major_level( child_action );
				}
			});
		};

		// esta funcion adjunta el markup con las acciones y grupos a la aplicacion
		var attach_actions = function( app_id, $app )
		{
			var tree_actions = self.tree_apps_actions[ app_id ];
			if ( !tree_actions || Object.keys( tree_actions ).length === 0 ) {
				return false;
			}
			// funcion recursiva
			var fill = function( tree_acts, $actions_container, level, current_action_id )
			{
				level++;
				var $actions = {};
				var $groups = {};

				var attrs_parent;
				if ( current_action_id ) {
					attrs_parent = " parent_id='" + current_action_id + "' parent_type='action' ";
				}
				else {
					attrs_parent = " parent_id='" + app_id + "' parent_type='app' ";
				}

				$.each( tree_acts || {}, function( action_id, action )
				{
					if ( action.childs || level === 1 ) {
					// este es markup de un grupo contenedor de acciones
						var opt_group_id = self.get_element_id( "group", app_id, action_id );
						var other_classes = "";
						if ( level === 1 ) {
							find_major_level( action, true );
							if ( major_level >= 2 ) {
							// agrego clases especiales
								other_classes = "especialSubModule";
							}
						}
						var $group = $( "".concat(
							"<div action_id='", action_id, "' group_item='true' is_action_container='true' class='subModule ", other_classes, " unselected'>",
								"<div class='subModuleTitle'>",
									"<input id='", opt_group_id, "' action_id='", action_id, "' app_id='", app_id, "' chk_type='group' ", attrs_parent, " class='subModuleCheckbox' type='checkbox' />",
									"<label for='", opt_group_id, "' class='subModuleTitleText'>", action.DescriptionLabel , "<i class='itemBooleanImage'></i>", "</label>",
								"</div>",
								"<div actions_container='true' class='subModuleContent'></div>",
							"</div>")
						);

						var is_configurable = self.is_configurable( app_id, action_id );
						var belogne_to_override_type = self.belogne_to_override_type( app_id, action_id );
						if ( !is_configurable  || !belogne_to_override_type ) {
						// si el grupo pertenece a otro override, no lo tengo que visualizar como accion, y lo tengo que visualizar como un simple agrupador (sin chk)

							// el chk
							var $chk = $group.find( "#" + opt_group_id );
							self.mark_checkbox( $chk, true );
							$chk.off("click")
								.attr( "only_container", true )
								.hide();

							// el lbl
							var $lbl = $group.find( ".subModuleTitleText[for='" + opt_group_id + "']" );
							$lbl.removeAttr("for");

							// el div del title
							var $title = $group.find( ".subModuleTitle" );
							$title.addClass("noFunctionality");
						}

						$groups[ action_id ] = { Name: action.Description, Label: action.DescriptionLabel, $html: $group, childs: action.childs, action_id: action.ActionId };
					}
					else {
					// este es el markup de una accion
						var opt_action_id = self.get_element_id( "action", app_id, action_id );
						var $action = $( "".concat(
							"<div action_id='", action_id, "' action_item='true' is_action_container='false' class='subModuleItem unselected'>",
								"<input id='", opt_action_id, "' action_id='", action_id, "' app_id='", app_id, "' chk_type='action' ", attrs_parent, " class='itemCheckbox' type='checkbox' />",
								"<label for='", opt_action_id, "' class='itemText'>", action.DescriptionLabel, "<i class='itemBooleanImage'></i>", "</label>",
							"</div>")
						);
						$actions[ action_id ] = { Name: action.Description, Label: action.DescriptionLabel, $html: $action, action_id: action.ActionId };
					}
				});

				// 1 - el hash de grupos de acciones ordenado alfabeticamente
				var $sorted_groups = inConcert.Utils.hash_to_sorted_array(
					$groups,
					function( group ) {
					//	( * ) aca tengo que ordenar descendente de mayor a menor,
					//		por la cantidad de hijos que tenga primero cada grupo,
					//		luego si tiene la misma cantidad, los ordena alfabeticamente ascendente

						// --------------------------------
						// TODO: LENGA LENGA, esto sale del comentario, porque se necesita primero este si o si,
						//   tengo que mirar bien y resolverlo de otra forma, por ahora queda asi.
						if ( app_id === "interaction" && group.action_id === "int_layout" ) {
							return 1000000; // un numero disparatado para que lo ponga primero!! Revisar y cambiar 
						}
						// --------------------------------

						// obtengo la cantidad de hijos y lo multiplico por 1000 
						var weight = Object.keys( group.childs || {} ).length * 1000;
						// ahora obtengo el primer codigo ascii de la palabra traducida y la multiplico por -1 para que quede negativo
						//  y se lo reste al valor anterior.
						weight = weight + ( group.Label.toUpperCase().charCodeAt( 0 ) * -1 );
						return weight;
					},
					undefined,
					{ order: "desc" }
				);

				$.each( $sorted_groups || {}, function( i, group ) {
					$actions_container.append( group.$html );
					// voy por sus hijos
					fill( group.childs, group.$html.find( "div[actions_container]" ), level, group.action_id );
				});

				// 2 - el hash de acciones ordenado alfabeticamente
				var $sorted_actions = inConcert.Utils.hash_to_sorted_array( $actions,
					function( action ) {
						return action.NameLabel;
					});
				$.each( $sorted_actions || {}, function( i, action ) {
					$actions_container.append( action.$html );
				});
			};
			// comienzo a cargar el tree de acciones
			fill( tree_actions, $app, 0 /* tree level */ );

			return true;
		};

		var filter_app = function( app )
		{
			// FIXME: by now we disable analytics until we get it's data from the reports.
			if ( app.Id === "analytics" ) {
				return true;
			}
			if ( !self.can_add_app( app.Id ) ) {
				return true;
			}
			return false;
		};

		// el hash de aplicaciones se pasa a un array ordeando alfabeticamente
		var sorted_app_actions = inConcert.Utils.hash_to_sorted_array( app_actions,
			function(app) {
				return app.NameLabel;
			});

		// aca recorro las aplicaciones
		$.each( sorted_app_actions, function( i, app )
		{
			if ( filter_app( app ) ) {
				return true; // continue
			}
			var title = app.NameLabel;
			// este es markup de una aplicacion contenedora de grupos y acciones
			var opt_app_id = self.get_element_id( "app", app.Id );
			var $app = $(
				"<div app_id='" + app.Id + "' app_item='true' class='formPermissionModule unselected'>" +
					"<div class='moduleTitle'>" +
						"<input id='"  + opt_app_id + "' app_id='" + app.Id + "' chk_type='app' class='moduleCheckbox' type='checkbox' />" +
						"<label for='" + opt_app_id + "' class='moduleTitleText'>" + title + "</label>" +
						"<i class='itemBooleanImage'></i>" +
					"</div>" +
					"<div actions_container='true' class='moduleContent'></div>" +
				"</div>"
			);
			// adjunto las acciones de la aplicacion
			var has_attached_actions = attach_actions( app.Id, $app.find("div[actions_container]") );

			if ( has_attached_actions ) {
				$( self.container_id ).append( $app );
				self.overrideable_apps[ app.Id ] = true; 
			}
		});

		this.build_specific_markup();
	};

	permissions_gui.prototype.build_specific_markup = $.noop;

	permissions_gui.prototype.mark_all_actions = function()
	{
		var checked = true;
		// marco los checkbox de aplicaciones
		$( this.get_jquery_selector( "all-apps" ) ).prop( "checked", !checked ).trigger( "click" );
	};

	permissions_gui.prototype.unmark_all_actions = function()
	{
		var self = this;
		var checked = false;
		// // desmarco los checkbox de aplicaciones
		$( this.get_jquery_selector( "all-apps" ) ).prop( "checked", !checked ).trigger( "click" );
		// desmarco los checkbox de grupos y acciones acciones
		$.each( this.raw_app_actions || {}, function( app_id ) {
			self.unmark_all_app_actions( app_id );
		});
	};

	permissions_gui.prototype.mark_all_app_actions = function( app_id )
	{
		var checked = true;
		$( this.get_jquery_selector( "all-childs-groups-specific-app-group-parent-is-app", app_id ) ).prop( "checked", !checked ).trigger( "click" );
	};

	permissions_gui.prototype.unmark_all_app_actions = function( app_id )
	{
		var checked = false;
		$( this.get_jquery_selector( "all-childs-groups-specific-app-group-parent-is-app", app_id ) ).prop( "checked", !checked ).trigger( "click" );
	};

	permissions_gui.prototype.mark_all_app_group_actions = function( app_id, group_id /* action id*/ )
	{
		var checked = true;
		$( this.get_jquery_selector( "all-childs-enabled-groups-specific-app-group", app_id, group_id ) ).prop( "checked", !checked ).trigger( "click" );
		$( this.get_jquery_selector( "all-childs-enabled-actions-specific-app-group", app_id, group_id ) ).prop( "checked", !checked ).trigger( "click" );
	};

	permissions_gui.prototype.unmark_all_app_group_actions = function( app_id, group_id /* action id*/ )
	{
		var checked = false;
		$( this.get_jquery_selector( "all-childs-enabled-groups-specific-app-group", app_id, group_id ) ).prop( "checked", !checked ).trigger( "click" );
		$( this.get_jquery_selector( "all-childs-enabled-actions-specific-app-group", app_id, group_id ) ).prop( "checked", !checked ).trigger( "click" );
	};

	permissions_gui.prototype.disable_all_actions = function( disabled )
	{
		var $apps_chks = $( this.get_jquery_selector( "all-apps" ) );
		var $groups_chks = $( this.get_jquery_selector( "all-groups" ) );
		var $actions_chks = $( this.get_jquery_selector( "all-actions" ));

		// activo/desactivo los checkbox de aplicaciones
		$apps_chks.prop( "disabled", disabled );
		// activo/desactivo los checkbox de grupos
		$groups_chks.prop( "disabled", disabled );
		// desmarco los checkbox de acciones
		$actions_chks.prop( "disabled", disabled );

		if ( disabled ) {
			$groups_chks.addClass("readonly");
		}
		else {
			$groups_chks.removeClass("readonly");
		}
	};

	permissions_gui.prototype.disable_specific_app = function( disabled, app_id, apply_to_children )
	{
		var opt_app_id = this.get_element_id( "app", app_id );
		var $app_chk = $( "#" + opt_app_id );
		if ( $app_chk.length === 0 ) {
			return;
		}

		var apply_changes = function( $ctrls ) {
			$ctrls.prop( "disabled", disabled );
			if ( disabled ) {
				$ctrls.addClass("readonly");
			}
			else {
				$ctrls.removeClass("readonly");
			}
		};

		// activo / desactivo el grupo
		apply_changes( $app_chk );

		if ( apply_to_children ) {
			var $groups_chks =$( this.get_jquery_selector( "all-groups-specific-app", app_id ) );
			// activo/desactivo los checkbox hijos (grupos)
			apply_changes( $groups_chks );

			var $actions_chks = $( this.get_jquery_selector( "all-actions-specific-app", app_id ) );
			// activo/desactivo los checkbox hijos (acciones)
			apply_changes( $actions_chks );
		}
	};

	permissions_gui.prototype.disable_specific_group = function( disabled, app_id, group_id, apply_to_children )
	{
		var opt_group_id = this.get_element_id( "group", app_id, group_id );
		var $group_chk = $( "#" + opt_group_id );
		if ( $group_chk.length === 0 ) {
			return;
		}

		var apply_changes = function( $ctrls ) {
			$ctrls.prop( "disabled", disabled );
			if ( disabled ) {
				$ctrls.addClass("readonly");
			}
			else {
				$ctrls.removeClass("readonly");
			}
		};

		// activo / desactivo el grupo
		apply_changes( $group_chk );

		if ( apply_to_children ) {
			var $children_chks = $( this.get_jquery_selector( "all-childs-actions-specific-app-group", app_id, group_id ) );
			// activo/desactivo los checkbox hijos del grupo
			apply_changes( $children_chks );
		}
	};

	permissions_gui.prototype.get_role_unmarked_actions = function()
	{
		var self = this;

		var unmarked_actions = {};
		if ( !this.group_permissions ) {
			return unmarked_actions;
		}

		var fill = function( app_id, apps_actions, app_permissions ) {
			$.each( apps_actions || {}, function( i, action )
			{
				var action_id = action.ActionId;

				if ( !app_permissions[ action_id ] ) {
					if ( action.RelationsActions ) {
						var has_add = false;
						$.each( action.RelationsActions,
							function( _, relation_info )
							{
								if ( relation_info.type === "child" ) {
									has_add = true;
									return false; // break
								}
							}
						);

						if ( has_add ) {
							if ( !unmarked_actions[ app_id ] ) {
								unmarked_actions[ app_id ] = [];
							}
							unmarked_actions[ app_id ].push( { action_id: action_id, checked: false } );
						}

						return true; // continue
					}
				}

				if ( action.childs ) {
					fill( app_id, action.childs, app_permissions );
				}
			});
		};

		$.each( this.group_permissions, function( app_id, app_permissions )
		{
			var apps_actions = self.tree_apps_actions[ app_id ];
			if ( !apps_actions ) {
				return true; // continue
			}
			fill( app_id, apps_actions, app_permissions );
		});

		return unmarked_actions;
	};

	permissions_gui.prototype.mark_actions = function( grouppermissions, callback )
	{
		var self = this;

		callback = callback || $.noop;

		if ( typeof grouppermissions !== "object" ) {
			this.group_permissions = null;
			switch( grouppermissions ) {
				case "none":
					this.unmark_all_actions();
					break;
				case "full":
					this.mark_all_actions();
					break;
			}
			callback();
			return;
		}
		else {
			this.group_permissions = grouppermissions;
		}

		this.unmark_all_actions();

		var mark = function( list, exec_trigger, is_array ) {
			$.each( list, function( app_id, permissions )
			{
				var can_mark_app = false;
				$.each( permissions, function( action_id, checked )
				{
					if ( is_array ) {
						action_id = checked.action_id;
						checked   = checked.checked;
					}
					if (action_id === "admin_restricted" && exec_trigger) {
						exec_trigger = false;
					}
				// marca las acciones
					var success = self.mark_specific_checkbox( checked, "action", app_id, action_id, exec_trigger );
					if ( success ) {
						can_mark_app = true;
						return true; // continue
					}
				// marca los grupos
					success = self.mark_specific_checkbox( checked, "group", app_id, action_id, exec_trigger );
					if ( success ) {
						can_mark_app = true;
						return true; // continue
					}
				});
				if ( can_mark_app ) {
				// marca la app
					self.mark_specific_checkbox( true, "app", app_id );
				}
			});
		};

		mark( grouppermissions );

		// queda dentro de un callback asi, da tiempo para que seteen eventos, etc
		window.setTimeout(function() {
			var role_unmarked_actions = self.get_role_unmarked_actions();
			mark( role_unmarked_actions, true, true );
			callback();
		}, 150);
	};

	permissions_gui.prototype.count_total_actions = function()
	{
		var $groups_chks = $( this.get_jquery_selector( "all-groups" ) );
		var $actions_chks = $( this.get_jquery_selector( "all-actions" ) );

		var count = $groups_chks.length + $actions_chks.length;
		return count;
	};

	permissions_gui.prototype.count_marked_actions = function()
	{
		var self = this;
		var count = 0;
		var $apps_chks = $( this.get_jquery_selector( "all-apps" ) );

		$.each( $apps_chks || {}, function( i, app )
		{
			var $app = $( app );
			// si la app se encuentra marcada
			if ( $app.prop( "checked" ) ) {
				// si la app no se encuentra en los default de permiso ( o sea no tiene permiso a esta app por defecto )
				var app_id = $app.attr( "app_id" );
				count = count + self.count_marked_actions_by_specific_app( app_id );
			}
		});

		return count;
	};

	permissions_gui.prototype.count_unmarked_actions = function()
	{
		var $groups_chks = $( this.get_jquery_selector( "all-groups" ) );
		var $actions_chks = $( this.get_jquery_selector( "all-actions" ) );
		var filter_selector = ":not(:checked)";

		var count = $groups_chks.filter( filter_selector ).length + $actions_chks.filter( filter_selector ).length;
		return count;
	};

	permissions_gui.prototype.count_marked_actions_by_specific_app = function( app_id )
	{
		var $groups_chks = $( this.get_jquery_selector( "all-groups-specific-app", app_id ) );
		var $actions_chks = $( this.get_jquery_selector( "all-actions-specific-app", app_id ) );
		var filter_selector = ":checked";

		var count = $groups_chks.filter( filter_selector ).length + $actions_chks.filter( filter_selector ).length;
		return count;
	};

	permissions_gui.prototype.count_marked_actions_by_specific_group = function( app_id, group_id )
	{
		var $groups_chks = $( this.get_jquery_selector( "all-childs-groups-specific-app-group", app_id, group_id ) );
		var $actions_chks = $( this.get_jquery_selector( "all-childs-actions-specific-app-group", app_id, group_id ) );
		var filter_selector = ":checked";

		var count = $groups_chks.filter( filter_selector ).length + $actions_chks.filter( filter_selector ).length;
		return count;
	};

	permissions_gui.prototype.validate = function(type, extra_info)
	{
		var self = this;
		if (typeof type !== "string") {
			throw("Permissions - Validation type is required.");
		}
		var response = { success: true };

		var apps, count, permissions;
		switch(type)
		{
			case "marked-all-actions":
				count = this.count_marked_actions();
				if ( count === 0 ) {
					response.success = false;
					response.message = "<%=_T('Please select some permissions.')%>";
				}
			break;

			case "mark-at-least-a-action-of-the-app":
				var $apps_chks = $( this.get_jquery_selector( "all-apps" ) );

				$.each( $apps_chks || {}, function( i, app )
				{
					var $app = $( app );
					// si la app se encuentra marcada
					if ( $app.prop( "checked" ) ) {
						// si la app no se encuentra en los default de permiso ( o sea no tiene permiso a esta app por defecto )
						var app_id = $app.attr( "app_id" );
						var count = self.count_marked_actions_by_specific_app( app_id );
						if ( count === 0 ) {
							var app_name = app_id;
							var raw_app = self.get_raw_app( app_id );
							if ( raw_app && raw_app.Name ) {
								app_name = raw_app.Name;
							}
							response.success = false;
							response.message = format( "<%=_T('Please select some permission on {0} application.')%>", app_name );
							return false; //break
						}
					}
				});

			break;

			case "marked-especific-actions":
				response.success = false;
				apps = extra_info.allowed_apps;
				permissions = this.get_permissions() || {};
				$.each(apps || {}, function(app_id, actions) {
					if (!permissions.hasOwnProperty(app_id)) {
						return true; // continue;
					}
					$.each(actions || {}, function(i, action_id) {
						if (permissions[app_id].hasOwnProperty(action_id)) {
							response.success = true;
							return false; // break;
						}
					});
				});
			break;

			default:
				throw("Permissions - Validation type '" + type + "' is unknown!");
		}

		return response;
	};

	// esta funcion es la que realiza el checked/unchecked de los checkbox de acciones/grupos/apps
	permissions_gui.prototype.mark_specific_checkbox = function( marked, type, app_id, action_id, exec_trigger )
	{
		var opt_id = this.get_element_id( type, app_id, action_id );
		var $chk = $( "#" + opt_id );
		if ( $chk.length > 0 ) {
			this.mark_checkbox( $chk, marked, exec_trigger );
			return true;
		}
		return false;
	};

	permissions_gui.prototype.mark_checkbox = function( $chk, checked, exec_trigger )
	{
		var mark_selected = function( $container ) {
			if ( checked ) {
				$container
					.removeClass( "unselected" )
					.addClass( "selected" );
			}
			else {
				$container
					.removeClass( "selected" )
					.addClass( "unselected" );
			}
		};

		var chk_type = $chk.attr( "chk_type" );
		$chk.prop( "checked", checked );
		var $parent = $chk.parent();

		// mark selected / unselected
		if ( chk_type === "app" || chk_type === "group" ) {
		// apps o groups
			mark_selected( $parent.parent() );
		}
		else {
		// actions
			mark_selected( $parent );
		}

		if ( exec_trigger ) {
		// se debe de ejecutar 2 veces el click para que luego ejecutar de aplicar los procesos necesarios, vuela el chk a su mismo estado)
		// esto solo sirve para las referencias cruzadas, que son medio keko, me tiene loco
			$chk.trigger( "click" );
			$chk.trigger( "click" );
		}
	};

	// esta funcion se encarga de marcar/desmarcar acciones relacionadas (referencia cruzada), cuando se marca algun chk determinado
	// valida contra relaciones client-side
	permissions_gui.prototype.mark_unmark_related_chk = function( app_id, action_id, action_type )
	{
		var self = this;
		if ( !app_id || !action_id ) {
			return;
		}
		action_type = action_type || "action"; // action or group
		var raw_app = this.get_raw_app( app_id );
		if ( !raw_app || !raw_app.Actions ) {
			return;
		}
		var raw_action = raw_app.Actions[ action_id ];
		if ( !raw_action ) {
			return;
		}
		if ( raw_action.RelationType !== "client-side" || !raw_action.RelationsActions ) {
			return;
		}

		var dom_action_id = this.get_element_id( action_type, app_id, action_id )  || "";
		var $current_chk = $( "#" + dom_action_id );
		if ( $current_chk.length === 0 ) {
			throw( "permissions - checkbox action app id '" + dom_action_id + "' does not exist" );
		}

		let behavior = function ( $related_chk, related_info )
		{
			if (related_info.behavior !== null && related_info.behavior === "1")
			{
				if ( !$current_chk.prop( "checked" ) )
				{
					if (related_info.useactive && related_info.useactive === "1") {
						$related_chk.prop("disabled", false);
					}
					
					if (related_info.usecheck && related_info.usecheck === "1") {
						if ( !$related_chk.prop( "checked" ) ) {
							$related_chk.trigger( "click" );
						}
					}
				}
				else 
				{
					if (related_info.usecheck && related_info.usecheck === "1") {
						if ( $related_chk.prop( "checked" ) ) {
							$related_chk.trigger( "click" );
						}
					}

					if (related_info.useactive && related_info.useactive === "1") {
						$related_chk.prop("disabled", true);
					}
				}
			}
			else if (related_info.behavior !== null && related_info.behavior === "2")
			{
				if ( !$current_chk.prop( "checked" ) ) {
					$related_chk.prop("disabled", false);
				}
				else {
					if ( $related_chk.prop( "checked" ) ) {
						self.mark_checkbox( $related_chk, false /* unchecked */ , false /* execute trigger click */ , true /* not mark override */ );
					}
					$related_chk.prop("disabled", true);
				}
			}
			else
			{
				if ( !$current_chk.prop( "checked" ) ) {
					if ( $related_chk.prop( "checked" ) ) {
						$related_chk.trigger( "click" );
					}
					$related_chk.prop("disabled", true);
				}
				else {
					$related_chk.prop("disabled", false);
				}
			}
		};

		$.each( raw_action.RelationsActions,
			function( relation_action_id, relation_info )
			{
				if ( relation_info.type === "child" )
				{
					// si la accion relacionado depende de esta
					var $child_chk = null;
					// primero busco a ver si es una accion
					var dom_relation_action_id = self.get_element_id( "action", relation_info.app, relation_action_id )  || "";
					$child_chk = $( "#" + dom_relation_action_id );
					if ( $child_chk.length === 0 ) {
					//si no es una accion, busco si es un agrupador
						dom_relation_action_id = self.get_element_id( "group", relation_info.app, relation_action_id )  || "";
						$child_chk = $( "#" + dom_relation_action_id );
						if ( $child_chk.length === 0 ) {
							throw( "permissions - checkbox action app id '" + dom_relation_action_id + "' does not exist" );
						}
					}

					behavior( $child_chk, relation_info );
				}
				else if ( relation_info.type === "app" && relation_info.app === relation_action_id )
				{
					let $app_chk   = null;
					let opt_app_id = self.get_element_id( "app", relation_info.app );

					$app_chk = $( "#" + opt_app_id );
					if ( $app_chk.length === 0 ) {
						throw( "permissions - checkbox app id '" + opt_app_id + "' does not exist" );
					}

					behavior( $app_chk, relation_info );
				}
			});
	};

	permissions_gui.prototype.set_events = function()
	{
		var self = this;

		var check_event = function( chk ) {
			var $chk = $( chk );
			var checked = $chk.prop( "checked" );
			self.mark_checkbox( $chk, checked );

			var chk_type = $chk.attr( "chk_type" );
			if ( chk_type === "action" || chk_type === "group" ) {
			// si es una accion o grupo, tiene que marcar las referencias cruzadas
				var app_id = $chk.attr( "app_id" );
				var action_id = $chk.attr( "action_id" ); // action o group
				self.mark_unmark_related_chk( app_id, action_id, chk_type );
			}
			return checked;
		};

		var mark_app = function( app_id, marked ) {
			var dom_app_id = self.get_element_id( "app", app_id );
			var chk_app = document.getElementById( dom_app_id );
			var $chk_app = $( chk_app );
			$chk_app.prop( "checked", marked );
			check_event( chk_app );
		};

		var mark_group = function( app_id, parent_id, marked ) {
			var dom_group_id = self.get_element_id( "group", app_id, parent_id );
			var chk_group = document.getElementById( dom_group_id );
			var $chk_group = $( chk_group );
			$chk_group.prop( "checked", marked );
			check_event( chk_group );
			return $chk_group;
		};

		// funcion recursiva que se encarga de marcar o desmarcar los chk de grupos parent ( si corresponde )
		var mark_parent = function( $chk )
		{
			var count;
			var parent_type = $chk.attr( "parent_type" );
			var app_id = $chk.attr( "app_id" );

			if ( parent_type === "app" ) {
			// la app
				count = self.count_marked_actions_by_specific_app( app_id );
				if ( count === 0 ) {
					if ( self.is_checked_app( app_id ) ) {
						mark_app( app_id, false /* unmarked */ );
					}
				}
				return;
			}

			// los grupos
			var $parent_chk_group;
			var parent_id = $chk.attr( "parent_id" );
			count = self.count_marked_actions_by_specific_group( app_id, parent_id );
			if ( count > 0 ) {
				if ( !self.is_checked_group( app_id, parent_id ) ) {
					$parent_chk_group = mark_group( app_id, parent_id, true /* marked */ );
				}
			}
			if ( $parent_chk_group ) {
				mark_parent( $parent_chk_group );
			}
		};

		// TODO: Revisar estas 2 funciones.
		var timeout_id;
		var click_on; // app id, or group id, or action id

		var initialize_event_timeout = function( current_action ) 
		{
		// (revisar) timeout necesario para que no se mezcle el marcar padres e hijos de una accion
			if ( click_on !== "APP" && click_on === undefined ) {
				var timeout_ms = 50; // user click timeout
				click_on = current_action || "APP";
				if ( timeout_id ) {
					window.clearTimeout( timeout_id );
				}
				timeout_id = window.setTimeout(function() {
					click_on = undefined;
					timeout_id = undefined;
				}, timeout_ms);
			}
		};

		var start_mark_parent = function( app_id, action_id, $chk )
		{
			if ( click_on === "APP" ) {
				return;
			}
			// solo comienza a marcar los parents, si el evento se ejecuto a partir de la misma accion del usuario (*)
			// o si la action del click del usuario es hija de la current action (**)
			if ( click_on === action_id ) {
				// (*)
				mark_parent( $chk );
			}
			else {
				var tree_node = self.tree_apps_actions[ app_id ];
				var user_click_action = self.find_action_in_tree( tree_node, click_on );
				var current_action = self.find_action_in_tree( tree_node, action_id );
				// atencion, cuando mas arriba del arbol se encuentra la accion, el valor del attr "level" es menor.
				// Ej: app:adm->action_a->action_b
				//	action_a.level = 1 y action_b.level = 2
				if ( user_click_action.level > current_action.level ) {
				// (**)
					mark_parent( $chk );
				}
			}
		};

		// evento click de cada opcion global de los checkboxes
		$(this.get_jquery_selector( "all-apps" ))
			.off("click")
			.on("click", function()
				{
					initialize_event_timeout();

					var checked = check_event( this );
					var app_id = $( this ).attr( "app_id" );
					if ( checked ) {
						self.mark_all_app_actions( app_id );
					}
					else {
						self.unmark_all_app_actions( app_id );
					}
					self.callback( {
						type   : "mark_app" ,
						app_id : app_id     ,
						marked : checked
					});
				});

		// evento click de cada opcion global de los checkboxes
		$(this.get_jquery_selector( "all-groups" ))
			.off("click")
			.on("click", function()
				{
					var checked = check_event( this );
					var $chk = $( this );
					var app_id = $chk.attr( "app_id" );
					var action_id = $chk.attr( "action_id" ); // group id

					initialize_event_timeout( action_id );
					if ( checked ) {
						self.mark_all_app_group_actions( app_id, action_id );
					}
					else {
						self.unmark_all_app_group_actions( app_id, action_id );
					}
					start_mark_parent( app_id, action_id, $chk );
				});

		// evento click de cada opcion avanzada de los checkboxes (save, view, delete)
		$(this.get_jquery_selector( "all-actions" ))
			.off("click")
			.on("click", function()
				{
					var $chk = $( this );
					var app_id = $chk.attr( "app_id" );
					var action_id = $chk.attr( "action_id" ); // action id

					initialize_event_timeout( action_id );
					check_event( this );
					start_mark_parent( app_id, action_id, $chk );
				});
	};

	permissions_gui.prototype.readonly = function() {
		this.disable_all_actions( true /* disabled */ );
	};

	permissions_gui.prototype.editable = function() {
		this.disable_all_actions( false /* disabled */ );
	};

	/*
	 * - Esta funcion retorna las acciones personalizadas.
	 *   - Las acciones extras que se han agregado o quitado al grupo.
	 *   - Le agrega un true a las acciones nuevas y un false a las acciones que se han quitado.
	 *   Ej:
	 *      {administration: {savegroup: true, saveuser: false}}
	 *   - Retorna lo anterior en el caso en que se le haya agregado a los permisos la accion de savegroup
	 *     y que le haya quitado permisos de saveusers.
	 */
	permissions_gui.prototype.get_overrides = function( current_permissions /* optional */)
	{
		var self = this;

		current_permissions = current_permissions || this.group_permissions;

		var selected_permissions = this.get_permissions(); // obtengo todas las acciones marcadas
		// Esta funcion elimina de la lista de acciones seleccionadas.
		var delete_selected = function( app_id, action_id )
		{
			if ( selected_permissions.hasOwnProperty( app_id ) ) {
				delete selected_permissions[ app_id ][ action_id ];
			}
		};

		var apps_differences = {}; // esta es la lista de apps que se han deschequeado de forma manual
		var actions_differences = {}; // esta es la lista que contiene las diferencias encontradas entre las acciones. La lista que se retornara.

		//Esta funcion agrega una diferencia a la lista, con sus estado (enabled/disabled)
		var add_action_difference = function( app_id, action_id, state ) {
			actions_differences[ app_id ] = actions_differences[ app_id ] || {};
			actions_differences[ app_id ][ action_id ] = state;
		};

		// esta funcion desactiva todas las acciones de la app ( marca en false las que se encuntran en true )
		var disabled_all_app_actions = function( app_id, actions ) {
			// recorro las acciones de las app
			$.each( actions, function( action_id, current_state ) {
				if ( current_state === true ) {
					apps_differences[ app_id ] = false;
					add_action_difference( app_id, action_id, false /* disabled */ );
				}
			});
		};

		var enabled = true;
		var disabled = false;
		// esta funcion es la que realiza la busqueda de diferencias entre ambas listas.
		// recibe:
		//     - "source_list"  - la lista origen que se ha de recorrer.
		//     - "compare_list" - la lista con la cual se compara la lista anterior.
		//     - "state"        - (true/false) el estado con el cual se agregara en la lista de diferencias.
		//     - "has_delete"   - (true/false) si viene en true, elimina de la lista de selected.
		var find_differences = function( source_list, compare_list, state, has_delete )
		{
			// recorro la lista origen por app
			$.each( source_list, function( app_id, app )
			{
				if ( !self.is_overrideable_app_in_current_override( app_id ) ) {
					return true; // continue
				}
				var is_checked_app = self.is_checked_app( app_id );
				if ( !is_checked_app ) {
					disabled_all_app_actions( app_id, app );
					return true; // continue
				}

				// recorro las acciones de las app
				$.each( app, function( action_id )
				{
					if ( !self.can_add_action( app_id, action_id ) ) {
						return true; // continue
					}
					if ( compare_list.hasOwnProperty( app_id ) )
					{
					//si la app existe en ambas listas tengo que comparar si la accion existe.
						if ( !compare_list[ app_id ].hasOwnProperty( action_id ) )
						{
						// si la accion existe en la lista origen, y no existe en la a comparar, agrego a la lista de diferencias.
							add_action_difference( app_id, action_id, state );
							if ( has_delete ) {
							// elimina de la lista de selected. Esto sirve para que en la proxima invocacion a esta funcion,
							// no se vuelva a recorrer la misma accion seleccionada.
								delete_selected( app_id, action_id );
							}
						}
					}
					else {
					// si la app no existe en la lista a comprar, agrego a la lista de diferencias.
						add_action_difference( app_id, action_id, state );
						if ( has_delete ) {
						// elimina de la lista de selected. Esto sirve para que en la proxima invocacion a esta funcion,
						// no se vuelva a recorrer la misma accion seleccionada.
							delete_selected( app_id, action_id );
						}
					}
				});
			});
		};

		var has_delete = true;
		// primero recorro las acciones del grupo, y valido contra las seleccionadas.
		//  en el caso de que encuentre diferencias, agrega la accion como disabled, y mando borrar de las selected.
		find_differences( current_permissions, selected_permissions  , disabled, has_delete );
		// segundo recorro las acciones seleccionadas, y valido contra las acciones del grupo.
		//  en el caso de que encuentre diferencias, agrega la accion como enabled.
		find_differences( selected_permissions, current_permissions, enabled );


		// en esta parte lo que hago es marcar los overrides de los permisos de las aplicaciones.
		// solo los en true.
		// ejemplo si por defecto no teniamos permisos sobre una app, y la misma se marca de forma manual por el usuario
		// mas alguna accion ( al menos una accion)
		var $apps_chks = $( this.get_jquery_selector( "all-apps" ) );
		$.each( $apps_chks || {}, function( i, app )
		{
			var $app = $( app );
			// si la app se encuentra marcada
			if ( $app.prop( "checked" ) ) {
				// si la app no se encuentra en los default de permisos ( o sea no tiene permiso a esta app por defecto )
				var app_id = $app.attr( "app_id" );
				if ( !current_permissions[ app_id ] ) {
				// aca valido que al menos haya marcado una accion dentro de la aplicacion para poder agregarla como override.
					var count_marked_actions = self.count_marked_actions_by_specific_app( app_id );
					if ( count_marked_actions > 0 ) {
						apps_differences[ app_id ] = true;
					}
				}
			}
		});

		var get_override = function( list ) { return Object.keys( list ).length > 0 ? list : null; };

		var apps_override    = get_override( apps_differences       );
		var actions_override = get_override( actions_differences    );

		return {
			has_override : ( apps_override !== null || actions_override !== null ) ? "1" : "0", // tiene que ir asi por cassandra
			apps         : apps_override,    // override de apps
			actions      : actions_override  // override de actions
		};
	};

	//GUI override class ---------------------------------------------------
	//
	function permissions_gui_override( params )
	{
		var success = permissions_gui_override.uber.constructor.call( this, params );
		if ( !success ) {
			return;
		}

		this.cleanup_external_container_id = params.cleanup_external_container_id;
		var container_id          = params.cleanup_external_container_id || this.container_id.substring( 1 ); // sanitize id, sin el # de jquery
		this.cleanup_id           = container_id + "_override_cleanup";
		this.cleanup_container_id = this.cleanup_id + "_container";
		this.override_type        = params.override_type;

		this.group_permissions    = null;
		this.override_actions     = null;

		return true;
	}

	inConcert.Utils.inherit( permissions_gui_override, permissions_gui );

	if ( typeof inConcert.perm_gui_override !== "function" ) {
		inConcert.perm_gui_override = permissions_gui_override;
	}

	permissions_gui_override.prototype.initialize = function()
	{
		permissions_gui_override.uber.initialize.call( this );

		this.group_permissions = null;
		this.override_actions = null;
		var container_class = "exceptionPage";
		$( this.container_id ).removeClass( container_class ).addClass( container_class );
	};

	permissions_gui_override.prototype.can_add_app = function( app_id )
	{
		var is_overrideable = this.is_overrideable_app( app_id );
		return is_overrideable;
	};

	permissions_gui_override.prototype.can_add_action = function( app_id, action_id )
	{
		var is_overrideable_app = this.is_overrideable_app( app_id );
		if ( !is_overrideable_app ) {
			return false;
		}
		var is_overrideable_action = this.is_overrideable_action( app_id, action_id );
		if ( !is_overrideable_action ) {
			return false;
		}
		var belogne_to_override_type = this.belogne_to_override_type( app_id, action_id );
		if ( belogne_to_override_type ) {
			return true;
		}
		var contain_another_override = this.contain_another_override( app_id, action_id );
		return contain_another_override;
	};

	permissions_gui_override.prototype.is_overrideable_action = function( app_id, action_id )
	{
		if ( !this.is_overrideable_app( app_id ) ) {
			return false;
		}
		var raw_app = this.get_raw_app( app_id );
		if ( !raw_app || !raw_app.Actions ) {
			return false;
		}
		var raw_action = raw_app.Actions[ action_id ];
		if ( !raw_action ) {
			return false;
		}
		return raw_action.Domain.is_overrideable === "1";
	};

	permissions_gui_override.prototype.belogne_to_override_type = function( app_id, action_id )
	{
		if ( !this.is_overrideable_app( app_id ) ) {
			return false;
		}
		var raw_app = this.get_raw_app( app_id );
		if ( !raw_app || !raw_app.Actions  ) {
			return false;
		}
		var raw_action = raw_app.Actions[ action_id ];
		if ( !raw_action ) {
			return false;
		}
		return ( raw_action.Domain.override_type || "" ).toLowerCase() === ( this.override_type || "" ).toLowerCase();
	};

	permissions_gui_override.prototype.contain_another_override = function( app_id, action_id )
	{
		if ( !this.is_overrideable_app( app_id ) ) {
			return false;
		}
		var raw_app = this.get_raw_app( app_id );
		if ( !raw_app || !raw_app.Actions  ) {
			return false;
		}
		var raw_action = raw_app.Actions[ action_id ];
		if ( !raw_action ) {
			return false;
		}
		return raw_action.Domain.contain_another_override === "1";
	};

	permissions_gui_override.prototype.disable_all_actions = function( disabled )
	{
		permissions_gui_override.uber.disable_all_actions.call( this, disabled );
		// el boton remove override
		$( "#" + this.cleanup_id ).prop( "disabled", disabled );
	};

	permissions_gui_override.prototype.mark_difference_actions = function( differences )
	{
		var self = this;

		if ( typeof differences !== "object" ) {
			this.override_actions = null;
			return;
		}
		this.override_actions = differences.actions;

		$.each( differences.apps || {}, function( app_id, state ) {
			self.mark_specific_checkbox( state, "app", app_id );
		});

		$.each( differences.actions || {}, function( app_id, permissions )
		{
			if ( differences.apps && differences.apps[ app_id ] === false ) {
				return true; // continue
			}
			$.each( permissions, function( action_id, checked )
			{
				var success = self.mark_specific_checkbox( checked, "action", app_id, action_id );
				if ( success ) {
					self.mark_unmark_related_chk( app_id, action_id, "action" );
					return true; // continue
				}

				success = self.mark_specific_checkbox( checked, "group", app_id, action_id );
				if ( success ) {
					self.mark_unmark_related_chk( app_id, action_id, "group" );
					return true; // continue
				}
			});
		});
	};

	permissions_gui_override.prototype.mark_all_actions = function()
	{
		var checked = true;
		// marco / desmarco los checkbox de aplicaciones
		$( this.get_jquery_selector( "all-apps"    ) ).removeClass( "modified" ).prop( "checked", !checked ).trigger( "click" );
		// marco / desmarco los checkbox de grupos
		$( this.get_jquery_selector( "all-groups"  ) ).removeClass( "modified" ).prop( "checked", !checked ).trigger( "click" );
		// marco / desmarco los checkbox de acciones
		$( this.get_jquery_selector( "all-actions" ) ).removeClass( "modified" ).prop( "checked", !checked ).trigger( "click" );
	};

	permissions_gui_override.prototype.unmark_all_actions = function()
	{
		var self = this;

		var un_mark_all = function( $chks ) {
			$.each( $chks || {}, function( i, chk )
			{
				var $chk = $( chk );
				if ( $chk.prop( "checked" ) ) { //&& $chk.attr( "only_container" ) ) {
					self.mark_checkbox( $chk, false /* unchecked */ , false /* execute trigger click */ , true /* not mark override */ );
				}
			});
		};

		// marco / desmarco los checkbox de aplicaciones
		var $apps_chks = $( this.get_jquery_selector( "all-apps" ) );
		un_mark_all( $apps_chks );

		var $groups_chks = $( this.get_jquery_selector( "all-groups" ) );
		un_mark_all( $groups_chks );

		var $actions_chks = $( this.get_jquery_selector( "all-actions" ));
		un_mark_all( $actions_chks );
	};

	permissions_gui_override.prototype.validate = function( type, extra_info )
	{
		return permissions_gui_override.uber.validate.call( this, type, extra_info );
	};

	permissions_gui_override.prototype.is_override_app = function( app_id, app_checked )
	{
		if ( !this.group_permissions ) {
			return false;
		}
		var app_perm = this.group_permissions[ app_id ];
		if ( !app_perm ) {
			return app_checked;
		}
		return !app_checked;
	};

	permissions_gui_override.prototype.is_override_action = function( app_id, action_id, action_checked )
	{
		if ( !this.group_permissions ) {
			return false;
		}
		var app_perm = this.group_permissions[ app_id ];
		if ( !app_perm ) {
			return action_checked;
		}

		if ( app_perm[ action_id ] ) {
			return app_perm[ action_id ] !== action_checked;
		}
		else {
			return action_checked === true;
		}
	};

	permissions_gui_override.prototype.mark_checkbox = function( $chk, checked, exec_trigger, not_mark_override )
	{
		var self = this;

		permissions_gui_override.uber.mark_checkbox.call( this, $chk, checked, exec_trigger );

		var chk_type = $chk.attr( "chk_type" );

		var mark_override = function( $container )
		{
			$container.removeClass( "modified" );
			if ( not_mark_override ) {
				return;
			}
			var app_id = $chk.attr( "app_id" );
			var can_mark_override = false;
			if ( chk_type === "app" ) {
			// override de aplicaciones
				can_mark_override = self.is_override_app( app_id, checked );
			}
			else {
				// override de acciones y grupos
				var action_id = $chk.attr( "action_id" );
				can_mark_override = self.is_override_action( app_id, action_id, checked );
			}

			if ( can_mark_override ) {
				$container.addClass( "modified" );
			}
		};

		var $parent = $chk.parent();
		mark_override( $parent );
	};

	permissions_gui_override.prototype.build_specific_markup = function()
	{
		var $html_actions;

		$( "#" + this.cleanup_container_id ).remove();

		if ( this.cleanup_external_container_id ) {
			$html_actions = $( "".concat(
				"<div class='customPageButtonContainer' id='", this.cleanup_container_id , "'>",
					"<button id='", this.cleanup_id, "' type='button' class='pageButtonGeneric buttonColorWhite pageButtonNoIcon'><%=_T('Remove Overrides')%></button>",
				"</div>"
			));
			$( "#" + this.cleanup_external_container_id ).before ( $html_actions );
		}
		else {
			$html_actions = $( "".concat(
				"<div id='", this.cleanup_container_id , "' class='formInternalButtonContainer'>",
					"<input id='", this.cleanup_id, "' value='", "<%=_T('Remove Overrides')%>", "' type='button' class='formInternalButton' />",
				"</div>"
			));
			$( this.container_id ).before( $html_actions );
		}
	};

	permissions_gui_override.prototype.set_events = function()
	{
		permissions_gui_override.uber.set_events.call( this );

		var self = this;

		$( "#" + this.cleanup_id )
			.off("click")
			.on("click", function() {
				self.unmark_all_actions();
				self.mark_actions( self.group_permissions );
			});
	};

	permissions_gui_override.prototype.readonly = function()
	{
		permissions_gui_override.uber.readonly.call( this );
		$( "#" + this.cleanup_container_id ).hide();
	};

}());