/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, esversion: 11 */

/* global $, inConcert */

if ( !this.inConcert ) {
	this.inConcert = {};
}

// try improve namespacing a bit
if ( !this.inConcert.Administration ) {
	this.inConcert.Administration = {};
}

if ( !this.inConcert.Administration.Campaigns ) {
	this.inConcert.Administration.Campaigns = {};
}

/**
 * @namespace
 * @name AppstoreAccountPanel
 * @author Sebastián Moreno (smoreno@inconcertcc.com)
 */
(function () {

	'use strict';

	var format = inConcert.Utils.format;

	/**
	 * Creates a new Appstore account panel instance.
	 *
	 * @constructor
	 * @constructor
	 * @memberOf AppstoreAccountPanel
	 */
	function AppstoreAccountPanel(params) {
		this.prefix_id = "adm_form_campaign_accounts_appstore_";
		AppstoreAccountPanel.uber.constructor.call(this, params);

		this.messages_class = null; // Appstore no tiene mensajes automaticos ya que da problemas.
		this.container_id = '#adm_form_campaign_accounts_appstore_panel';

		this.language = '#adm_form_campaign_accounts_appstore_language';
		this.distribution = '#adm_form_campaign_accounts_appstore_distribution';
		this.priority = '#adm_form_campaign_accounts_appstore_priority';
		this.name = '#adm_form_campaign_accounts_appstore_name';
		this.appId = '#adm_form_campaign_accounts_appstore_app_id';
		
		this.country_popup = "#adm_form_campaign_accounts_appstore_countries_popup";
		this.country_new = "#adm_form_campaign_accounts_appstore_countries_new_country";
		this.country_code = "#adm_form_campaign_accounts_appstore_countries_popup_code";
		this.country_popup_create = "#adm_form_campaign_accounts_appstore_countries_popup_create";
		this.country_popup_continue = "#adm_form_campaign_accounts_appstore_countries_popup_create_and_continue";
		this.country_popup_cancel = "#adm_form_campaign_accounts_appstore_countries_popup_cancel";
		this.country_list = "#adm_form_campaign_accounts_appstore_countries_parameters";

		this.attention_levels_container = '#adm_form_campaign_accounts_appstore_attention_levels_config';
		this.ticket_management_container = '#adm_form_campaign_accounts_appstore_tickets_mng_panel';
		this.contact_management_container = '#adm_form_campaign_accounts_appstore_contact_mng_panel';
		this.thresholds_container = '#adm_form_campaign_accounts_appstore_thresholds_panel';

		this.account_save_id = '#adm_form_campaign_accounts_save';

		this.tagline_new      = "<%=_T('New Appstore account.')%>";
		this.tagline_edit     = "<%=_T('Edit Appstore account {0}.')%>";
		this.tagline_readonly = "<%=_T('Appstore account {0}.')%>";
	}

	inConcert.Utils.inherit( AppstoreAccountPanel, inConcert.adm_form_campaigns_section_accounts_base_panel );

	if ( typeof inConcert.Administration.Campaigns.AppstoreAccountPanel !== 'function' ) {
		inConcert.Administration.Campaigns.AppstoreAccountPanel = AppstoreAccountPanel;
	}

	/**
	 * Called when the form is being initialized
	 *
	 * @public
	 */
	AppstoreAccountPanel.prototype.initialize = function(){
		AppstoreAccountPanel.uber.initialize.call(this);
		this.tabs.disable_tab( 'messages' );
	};

	AppstoreAccountPanel.prototype.has_qna_maker       = function() {return true;};
	/*AppstoreAccountPanel.prototype.has_flow_management = function() {return true;};
	AppstoreAccountPanel.prototype.has_flow_inactivity = function() {return true;};*/

	/**
	 * Initializes thresholds management
	 *
	 * @public
	 */
	AppstoreAccountPanel.prototype.initialize_thresholds = function()
	{
		// build thresholds panel
		AppstoreAccountPanel.uber.initialize_thresholds.call( this,
			'APPSTORE',
			{
				parent    : this.parent,
				prefix_id : 'adm_form_campaign_accounts_appstore_thresholds'
			}
		);
	};

	AppstoreAccountPanel.prototype.initialize_qna_maker_mng = function()
	{
		// build qna_maker management panel
		AppstoreAccountPanel.uber.initialize_qna_maker_mng.call(
			this,
			{
				type         : "APPSTORE"                                                  ,
				parent       : this                                                    ,
				container_id : "adm_form_campaign_accounts_appstore_qna_maker_mng_panel",
				prefix_id    : "adm_form_campaign_accounts_appstore_qna_maker_mng"
			}
		);
	};

	/**
	 * Initializes contact management
	 *
	 * @public
	 */
	AppstoreAccountPanel.prototype.initialize_contact_mng = function(){
		// build contact management panel
		AppstoreAccountPanel.uber.initialize_contact_mng.call(this,
			{
				type         : 'APPSTORE'                                              ,
				parent       : this                                                   ,
				auto_add_type: 'BOTH'                                                 , // incoming y outgoing
				container_id : 'adm_form_campaign_accounts_appstore_contact_mng_panel' ,
				prefix_id    : 'adm_form_campaign_accounts_appstore_contact_mng'
			}
		);
	};

	AppstoreAccountPanel.prototype.initialize_flows_mng = function()
	{
		// build flows designer management panel
		AppstoreAccountPanel.uber.initialize_flows_mng.call(
			this,
			{
				type         : 'appstore_BOT'                                                    ,
				parent       : this                                                         ,
				container_id : 'adm_form_campaign_accounts_appstore_flows_designer_mng_panel',
				prefix_id    : 'adm_form_campaign_accounts_appstore_flows_designer_mng'
			}
		);
	};

	/**
	 * Initializes ticket management
	 *
	 * @public
	 */
	AppstoreAccountPanel.prototype.initialize_ticket_emitter = function(){
		// build ticket management section
		AppstoreAccountPanel.uber.initialize_ticket_emitter.call(this,
			{
				type         : 'APPSTORE'                                              ,
				parent       : this                                                   ,
				auto_add_type: 'BOTH'                                                 , // incoming y outgoing
				container_id : 'adm_form_campaign_accounts_appstore_tickets_mng_panel' ,
				prefix_id    : 'adm_form_campaign_accounts_appstore_tickets_mng'
			}
		);
	};
	
	/**
	 * Called when the user enter to account edition
	 *
	 * @public
	 */
	AppstoreAccountPanel.prototype.edit = function(account)
	{
		AppstoreAccountPanel.uber.edit.call( this, account );
		if ( !this.current_account ) {
			return;
		}
		this.m_appId = this.current_account.uid;
		$(this.name).val(this.current_account.name);
		$(this.appId).val(this.current_account.uid);
		this.parent.frm.readonly_control($(this.appId), true);
		this.tabs.enable_tab( 'messages' );
		this.setCountries(this.current_account.countries);
	};

	/**
	 * Method to initialize tabs (basic & messages)
	 *
	 * @public
	 */
	AppstoreAccountPanel.prototype.initialize_tabs = function() {
		AppstoreAccountPanel.uber.initialize_tabs.call(this,
			{
				parent       : this                              ,
				frm          : this.parent.frm                   ,
				container_id : 'adm_form_campaign_accounts_tabs' ,
				tabs_list    : {
					basic : {
						id               : "basic",
						name             : "<%=_T('Appstore Config')%>",
						tab_container_id : 'adm_form_campaign_accounts_appstore_panel_tab_basic',
						selected         : true
					}
				}
			}
		);
	};

	AppstoreAccountPanel.prototype.initialize_skills = function()
	{
		AppstoreAccountPanel.uber.initialize_skills.call(
			this,
			{
				container_id : 'adm_form_campaign_accounts_appstore_skills_config',
				parent_frm   : this.parent,
				parent_type  : 'FORM-SECTION'
			}
		);
	};

	AppstoreAccountPanel.prototype.initialize_attention_levels = function()
	{
		AppstoreAccountPanel.uber.initialize_attention_levels.call(
			this,
			{
				container_id : 'adm_form_campaign_accounts_appstore_attention_levels_config',
				parent_frm   : this.parent,
				parent_type  : 'FORM-SECTION',
				proxy_type   : 'APPSTORE'
			}
		);
	};

	/**
	 * Validates Appstore account fields.
	 *
	 * @todo Validate and save should be separated actions
	 *
	 * @public
	 *
	 * @return {Object}  result
	 * @return {Boolean} result.success
	 * @return {Boolean} result.is_new
	 *
	 * @return {Object}  [result.account]
	 * @return {String}  [result.account.id]
	 * @return {String}  [result.account.proxyType]
	 * @return {String}  [result.account.name]
	 * @return {String}  [result.account.CCAlgorithm]
	 *
	 * @return {String}  [result.error_message]
	 */
	AppstoreAccountPanel.prototype.validate = function()
	{
		var is_new = this.is_new();

		var result = {
			success: false,
			is_new: is_new
		};

		this.hide_marked_errors();

		// NAME - validation
		var $name = $(this.name);
		if (!this.validate_field($name, "required", "<%=_T('Account name is required.')%>")) {
			return result;
		}
		if (!this.validate_field($name, "alpha", "<%=_T('Only letters, numbers, dashes and dots are allowed in the Account Name.')%>")) {
			return result;
		}

		// APP ID - validation
		let $appId = $(this.appId);
		if (!this.validate_field($appId, "required", "<%=_T('App id is required.')%>")) {
			return result;
		}
		if (!this.validate_field($appId, "alpha", "<%=_T('Only letters, numbers, dashes and dots are allowed in the Account App id.')%>")) {
			return result;
		}
		
		// COUNTRIES - validation
		let countries = this.getCountries();
		if (!countries.length) {
			this.parent.frm.show_status_message( "<%=_T('Must enter almost one country.')%>", 'Error', 5000 );
			return result;
		}

		var $distribution = $(this.distribution);
		if (!this.validate_field($distribution, 'required', "<%=_T('Distribution is required.')%>")) {
			return result;
		}

		// PRIORITY - validation
		var $priority = $(this.priority);
		var has_att_levels = this.attention_levels_handle.count_selected() > 0;
		if (!has_att_levels && $priority.val().trim().length > 0) {
			var range = { between: { from: 0, to: 99 } }; // range attention_level value validation
			if (!this.validate_field($priority, 'range', format("<%=_T('Account priority value out of range. Please enter account priority value between {0} and {1}.')%>", range.between.from, range.between.to ), range )) {
				return result;
			}
		}

		// LANGUAGE - validation
		var $language = $(this.language);
		if (!this.validate_field($language, "required", "<%=_T('Language is required.')%>")) {
			return result;
		}

		// valido los skills ----------
		var result_skills = this.validate_skills();
		if ( !result_skills.success ) {
			this.show_validation_error( result_skills.$ctrl, result_skills.message );
			return result;
		}

		// valido los attention levels ----------
		var result_attention_levels = this.validate_attention_levels();
		if ( !result_attention_levels.success ) {
			this.show_validation_error(
				result_attention_levels.$ctrl,
				result_attention_levels.message,
				false, //timeout_ms
				true, //scrolling
				result_attention_levels.show_custom_button
			);
			return result;
		}

		// validate ticket configuration
		var result_ticket_emitter = this.validate_ticket_emitter();
		if (!result_ticket_emitter.success) {
			this.show_validation_error( result_ticket_emitter.$ctrl, result_ticket_emitter.error_message );
			return result;
		}

		// valido los contact mng ----------
		var result_contact_mng = this.validate_contact_mng();
		if ( !result_contact_mng.success ) {
			this.show_validation_error( result_contact_mng.$ctrl, result_contact_mng.error_message );
			return result;
		}

		// valido los flows mng ----------
		var result_flows_mng = this.validate_flows_mng();
		if ( !result_flows_mng.success ) {
			this.show_validation_error( result_flows_mng.$ctrl, result_flows_mng.error_message );
			return result;
		}

		// valido los qna maker mng ----------
		var result_qna_maker_mng = this.validate_qna_maker_mng();
		if ( !result_qna_maker_mng.success ) {
			this.show_validation_error( result_qna_maker_mng.$ctrl, result_qna_maker_mng.error_message );
			return result;
		}

		// valido los thresholds ----------
		var result_th = this.validate_thresholds();
		if ( !result_th.success ) {
			this.show_validation_error( result_th.$ctrl, result_th.error_message );
			return result;
		}

		// valido los messages de la account ----------
		var result_messages = this.validate_messages();
		if ( !result_messages.success ) {
			this.show_validation_error( undefined, result_messages.error_message );
			return result;
		}

		// all good, now save it.
		// TO-DO: This should be separeted
		var account = {};
		if (is_new) {
			this.m_appId =  $appId.val();
			var campaign = this.parent.frm.get_entity_instance();
			// TO-DO: Why we need to create the account id client side?
			account.id = 'APPSTORE_' + campaign.Id + '_' + this.m_appId;
		}
		else {
			account.id = this.current_account.id;
		}
		
		let name = $name.val();

		account.proxyType          = 'APPSTORE';
		account.name               = name;
		account.uid                = this.m_appId;
		account.screen_name        = name;
		account.countries		   = countries;
		account.CCAlgorithm        = $distribution.val();
		account.language           = $language.val();
		account.TicketEmitter      = result_ticket_emitter.result;
		account.Skills             = this.skills_handle.get();
		account.Priority           = $priority.val();
		account.AttentionLevels    = this.attention_levels_handle.get();
		account.GUI                = this.get_gui_config();
		account.ActionOnAcdWaitTimeout = result_th.ActionOnAcdWaitTimeout;
		account.InactivityTimeout = result_th.InactivityTimeout;
		account.ignoreAgentAssignedFromReOpen = $(this.ignoreAgent).prop("checked")? "1" : "0";
		this.contact_mng_instance.attach_data(account);
		if (this.flow_mng_instance) {
			this.flow_mng_instance.attach_data( account );
		}
		if (this.qna_maker_mng_instance) {
			this.qna_maker_mng_instance.attach_data(account);
		}

		result.account = account;
		result.success = true;

		return result;
	};

	/**
	 * Called once the panel is initialized, we can attach buttons events here.
	 *
	 * @public
	 */
	AppstoreAccountPanel.prototype.set_events = function()
	{
		AppstoreAccountPanel.uber.set_events.call( this );
		this.setCountryEvents();
	};

	AppstoreAccountPanel.prototype.readonly = function()
	{
		AppstoreAccountPanel.uber.readonly.call( this );
		this.disableCountryActions(true);
	};
	
	/* COUNTRIES */
	
	AppstoreAccountPanel.prototype.setCountryEvents = function() {
		let self = this;
		self.loadCountriesList();
		
		$(self.country_new)
			.off("click")
			.on("click", function() {
				self.newCountry();
			});
		
		$(self.country_popup_create)
			.off("click")
			.on("click", function() {
				self.saveCountry();
			});
		
		$(self.country_popup_continue)
			.off("click")
			.on("click", function() {
				self.saveCountry(true);
			});
		
		$(self.country_popup_cancel)
			.off("click")
			.on("click", function() {
				self.hideCountryPopup();
			});
	};
	
	AppstoreAccountPanel.prototype.disableCountryActions = function() {
		this.toggleCountryActions(false);
	};
	
	AppstoreAccountPanel.prototype.enableCountryActions = function() {
		this.toggleCountryActions(true);
	};
	
	AppstoreAccountPanel.prototype.toggleCountryActions = function(enabled) {
		$(this.country_new).prop("disabled", !enabled);
		$(this.country_list).find("[country_action]").prop("disabled", !enabled);
	};
	
	AppstoreAccountPanel.prototype.loadCountriesList = function() {
		let countries = inConcert.Utils.getSortedCountries();
		this.parent.frm.fill_combo($(this.country_code), false, countries, "id", "label");
	};
	
	AppstoreAccountPanel.prototype.getCountries = function() {
		let countries = [];
		$(this.country_list).children("[country_item]").each(function() {
			countries.push($(this).data("country"));
		});
		countries.sort(function(a, b) {
			return a.id.localeCompare(b.id);
		});
		return countries;
	};
	
	AppstoreAccountPanel.prototype.setCountries = function(countries = []) {
		let self = this;
		countries.sort(function(a, b) {
			return a.id.localeCompare(b.id);
		});
		$(this.country_list).append(countries.map(function(country) {
			return self.makeCountryItem(country);
		}));
	};
	
	AppstoreAccountPanel.prototype.appendCountry = function(country) {
		this.setCountries([country]);
	};
	
	AppstoreAccountPanel.prototype.existsCountry = function(countryCode) {
		return this.$countryItem(countryCode).length > 0;
	};
	
	AppstoreAccountPanel.prototype.$countryItem = function(countryCode) {
		return $(this.country_list).find(format("[country_item='{0}']", countryCode));
	};
	
	AppstoreAccountPanel.prototype.makeCountryItem = function(country) {
		let self = this;
		
		let $country = $(format("".concat(
			"<li country_item='{0}' class='formListItem'>",
				"<label class='formItemText nameColumn'>{0}</label>",
				"<label class='formItemText valueColumn'>{1}</label>",
				"<button country_action='remove' class='buttonColorGray gridActionButton delete last'><i class='buttons-icons-delete'></i>{2}</button>",
			"</li>"
		), country.id, window.inConcertAllCountries[country.id] || country.id, "<%=_T('Delete')%>"));
		$country.data("country", country);
		
		$country.find("[country_action='remove']").click(function() {
			self.removeCountry($(this).parent().attr("country_item"));
		});
		
		return $country;
	};
	
	AppstoreAccountPanel.prototype.removeCountry = function(country) {
		let countryCode = $.isPlainObject(country)? country.id : country;
		this.$countryItem(countryCode).remove();
	};
	
	AppstoreAccountPanel.prototype.newCountry = function() {
		this.cleanCountryPopup();
		this.showCountryPopup();
	};
	
	AppstoreAccountPanel.prototype.cleanCountryPopup = function() {
		$(this.country_code).val($(this.country_code + " option:first").val());
	};
	
	AppstoreAccountPanel.prototype.showCountryPopup = function() {
		this.disableCountryActions();
		$(this.country_popup).show();
	};
	
	AppstoreAccountPanel.prototype.hideCountryPopup = function() {
		this.enableCountryActions();
		$(this.country_popup).hide();
	};
	
	AppstoreAccountPanel.prototype.saveCountry = function(saveAndContinue = false) {
		let country = {
			id : $(this.country_code).val()
		};
		if (!this.existsCountry(country.id)) {
			this.appendCountry(country);
		}
		
		if (saveAndContinue) {
			this.cleanCountryPopup();
		}
		else {
			this.hideCountryPopup();
		}
	};
	
}());
