/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 11 */

/* global $, inConcert, _ */

if (!this.inConcert) {
	this.inConcert = {};
}

// try improve namespacing a bit
if ( !this.inConcert.Administration ) {
	this.inConcert.Administration = {};
}

if ( !this.inConcert.Administration.Campaigns ) {
	this.inConcert.Administration.Campaigns = {};
}

(function () {

	"use strict";

	const format = inConcert.Utils.format;

	//---------------------------------------------------------------------
	// frm account edition (clase base)
	function account_panel( params )
	{
		/* jshint validthis: true*/
		this.parent = params.section;
		this.current_account         = null;
		this.tagline_new             = "<%=_T('New account.')%>";
		this.tagline_edit            = "<%=_T('Edit account {0}.')%>";
		this.tagline_readonly        = "<%=_T('Account {0}.')%>";
		this.thresholds_instance     = null;
		this.contact_mng_instance    = null;
		this.flow_mng_instance       = null;
		this.skills_handle           = null;
		this.attention_levels_handle = null;
		this.messages_class          = null;
		this.messages_instance       = null;
		this.qna_maker_mng_instance  = null;
		this.hsm_mng_instance        = null;
		this.blacklists_mng_instance = null;
		// ids common
		this.gui_container_id        = "#" + this.prefix_id + "gui_container";
		this.gui_window_open_id      = "#" + this.prefix_id + "gui_window_open_mode";
		this.ignoreAgent       = "#" + this.prefix_id + "ignore_agent_assigned";

		return true;
	}

	if ( typeof inConcert.adm_form_campaigns_section_accounts_base_panel !== "function" ) {
		inConcert.adm_form_campaigns_section_accounts_base_panel = account_panel;
	}

	account_panel.prototype.has_blacklists           = function() { return false; };
	account_panel.prototype.has_hsm_maker            = function() { return false; };
	account_panel.prototype.has_qna_maker            = function() { return false; };
	account_panel.prototype.has_flow_management      = function() { return false; };
	account_panel.prototype.has_flow_inactivity      = function() { return false; };
	account_panel.prototype.get_flow_inactivity_time = function() { 
		return {
			maxHours : 24,
			disclaimer : "<%=_T('Interaction will be closed after configured inactivity time (max: 24 hours).')%>",
			error : "<%=_T('Inactivity time must be a number and less than 24 hours.')%>"
		};
	};
	account_panel.prototype.get_account_name        = function() { return ""; };
	account_panel.prototype.save_account            = null; // si no se define en cada account, se toma el generico
	account_panel.prototype.select_tab_panel        = $.noop;
	account_panel.prototype.show_message_after_save = function(_ /*account*/, callback = $.noop) { callback(); };
	account_panel.prototype.empty_controls          = function() {
		$(this.distribution).val("");
		$(this.priority).val("");
		$(this.language).val("");
		$(this.gui_window_open_id).val("NORMAL");
		const $ignoreAgent = $(this.ignoreAgent);
		if ($ignoreAgent) {
			$ignoreAgent.prop("checked", false);
		}
	};
	account_panel.prototype.is_enable_menu_option   = function( callback )
	{
		// Si es allegro campaign, es decir tiene una cuenta de telefonia/allegro, las opciones para generar cuentas quedan enabled en false.
		//  La clase de call allegro, hace override esta funcion e implementa su propia logica para visualizar o no esta opcion.
		//
		var is_allegro_campaign = this.parent.__is_allegro_campaign();
		var is_enabled = !is_allegro_campaign;
		callback({ enabled: is_enabled });
	};

	account_panel.prototype.on_saved = function( new_account, current_account, callback = $.noop)
	{
		const self = this;

		if (current_account) {
			// Si estoy actualizando
			callback();
			return;
		}
		// Despues de crear una nueva cuenta:

		// Actualizo los ALs en su seccion.
		let attention_level_section = this.parent.frm.get_frm_section( "attention_level" );
		if (attention_level_section) {
			attention_level_section.on_account_update(new_account);
		}

		// Suscribo un evento, para que cuando se actualice el objeto campana, via poller, se oculten o muestren secciones
		// Ej: Si tiene integracion con Allegro, tiene que ocultar las secciones, Rules, Attention levels y canned messages.
		//     Como tambien tiene que, realizar un switch entre las secciones de Integracion, ya que una se encuentra oculta.
		let campaign_id = this.get_campaign_id();
		if (this.pubsub_added_account) {
			inConcert.app.messagebus.unsubscribe(this.pubsub_added_account);
		}
		this.pubsub_added_account = inConcert.app.messagebus.subscribe(
			"/application/campaign/" + campaign_id + "/account/added/",
			function(_) {
				if (self.pubsub_added_account) {
					inConcert.app.messagebus.unsubscribe(self.pubsub_added_account);
				}
				self.parent.frm.show_sections(callback);
			}
		);

		if (this.messages_instance) {
			this.messages_instance.on_saved();
		}
	};

	account_panel.prototype.get_gui_config = function()
	{
		const GUI = {
			window_open_mode: $(this.gui_window_open_id).val()
		};
		return GUI;
	};

	account_panel.prototype.edit = function( acc /* optional */ )
	{
		var self = this;
		this.set_account(acc);
		if ( !acc ) {
			return;
		}
		$( this.distribution ).val( acc.CCAlgorithm );
		$( this.language ).val( acc.language );
		$( this.priority ).val( acc.Priority );
		if (acc.GUI) {
			$(this.gui_window_open_id).val(acc.GUI.window_open_mode);
		}
		this.contact_mng_instance.set(acc);
		this.ticket_emitter_instance.set_account(acc);

		if ( this.flow_mng_instance ) {
			this.flow_mng_instance.set(acc);
		}
		if ( this.attention_levels_handle ) {
			this.attention_levels_handle.set( acc.AttentionLevels );
		}
		if ( this.skills_handle ) {
			this.skills_handle.set( acc.Skills );
			this.show_or_hide_skills_section();
		}
		if ( this.survey_management ) {
			this.survey_management.set_account(acc);
		}
		if ( this.messages_instance ) {
			this.parent.frm.campaign.getAccountMessages(
				acc.id,
				function( message, error )
				{
					if ( error ) {
						var message_type = "Error";
						var timeout_ms = 5000;
						self.parent.frm.show_status_message("<%=_T('Error retrieving campaign account messages.')%>", message_type, timeout_ms);
						return;
					}
					self.messages_instance.set_current_config( message );
					self.parent.frm.dashboard.hide_waiting();
				}
			);
		}
		if (this.qna_maker_mng_instance) {
			this.qna_maker_mng_instance.set(acc);
		}
		if (this.hsm_mng_instance) {
			this.hsm_mng_instance.set(acc);
		}
		if (this.blacklists_mng_instance) {
			this.blacklists_mng_instance.set(acc);
		}
		const $ignoreAgent = $(this.ignoreAgent);
		if ($ignoreAgent) {
			const ignoreAgentAssignedFromReOpen = acc.hasOwnProperty("ignoreAgentAssignedFromReOpen") && acc.ignoreAgentAssignedFromReOpen === "1" ? true : false;
			$ignoreAgent.prop("checked", ignoreAgentAssignedFromReOpen);
		}
		
	};

	account_panel.prototype.show = function(account)
	{
		this.set_account(account);
		this.initialize();
		if (!this.is_new()) {
		// esta editando una account existente
			this.edit( account );
		}
		else {
			this.empty_controls();
		}
		$("#adm_form_campaign_accounts_list_panel").hide();
		$(this.container_id).show();
		$("#adm_form_campaign_accounts_edit_buttons_panel").show();
		this.on_show();
	};

	account_panel.prototype.fill_distribution = function( list )
	{
		if ( typeof this.distribution !== "string" ) {
			return;
		}
		var is_blank_first_item = true;
		var attr_id = "id";
		var attr_name = "description";
		var $combo = $( this.distribution );
		this.parent.frm.fill_combo( $combo, is_blank_first_item, list, attr_id, attr_name );
	};

	account_panel.prototype.fill_languages = function( list )
	{
		if ( typeof this.language !== "string" ) {
			return;
		}
		var is_blank_first_item = true;
		var attr_id = "id";
		var attr_name = "name";
		var $combo = $( this.language );
		this.parent.frm.fill_combo( $combo, is_blank_first_item, list, attr_id, attr_name );
	};

	account_panel.prototype.fill_signatures = function( list )
	{
		if ( typeof this.signature !== "string" ) {
			return;
		}
		var is_blank_first_item = true;
		var attr_id = "Id";
		var attr_name = "Name";
		var $combo = $( this.signature );
		this.parent.frm.fill_combo( $combo, is_blank_first_item, list, attr_id, attr_name );
	};

	account_panel.prototype.fill_font_family = function( list )
	{
		if ( typeof this.font_family !== "string" ) {
			return;
		}
		var is_blank_first_item = true;
		var attr_id = "id";
		var attr_name = "name";
		var $combo = $( this.font_family );
		var sorted = true;
		this.parent.frm.fill_combo( $combo, is_blank_first_item, list, attr_id, attr_name, sorted );
	};

	account_panel.prototype.fill_font_size = function( list )
	{
		if ( typeof this.font_size !== "string" ) {
			return;
		}
		var is_blank_first_item = true;
		var attr_id = "id";
		var attr_name = "name";
		var $combo = $( this.font_size );
		this.parent.frm.fill_combo( $combo, is_blank_first_item, list, attr_id, attr_name );
	};

	account_panel.prototype.fill_gui_window_mode = function( list )
	{
		if ( typeof this.gui_window_open_id !== "string" ) {
			return;
		}
		const is_blank_first_item = false;
		const attr_id = "id";
		const attr_name = "description";
		const sorted = true;
		const $combo = $(this.gui_window_open_id);
		this.parent.frm.fill_combo($combo, is_blank_first_item, list, attr_id, attr_name, sorted);
		$combo.val("NORMAL");
	};

	account_panel.prototype.on_show = function()
	{
		this.parent.show_or_hide_required_message(true);
		var tagline = "";
		if (!this.is_new()) {
		// esta editando una account existente
			tagline = this.tagline_edit;
			if ( this.parent.is_readonly() ) {
				tagline = this.tagline_readonly;
			}
			var account_name = this.get_account_name();
			tagline = format( tagline, "<span>\"" + account_name + "\"</span>" );
			this.parent.frm.change_title_section(undefined, account_name);
		}
		else {
			tagline = this.tagline_new;
		}
		this.parent.frm.set_tagline( tagline );
		this.tabs.show_tabs_container();
		this.show_or_hide_skills_section();
		this.parent.frm.dashboard.hide_waiting();
	};

	account_panel.prototype.on_hide = function()
	{
		this.tabs.hide_tabs_container();
		this.set_account(null);
	};

	account_panel.prototype.editable = function()
	{
		this.parent.editable();

		if ( this.contact_mng_instance ) {
			this.contact_mng_instance.editable();
		}
		if ( this.flow_mng_instance ) {
			this.flow_mng_instance.editable();
		}
		if ( this.ticket_emitter_instance ) {
			this.ticket_emitter_instance.editable();
		}
		if ( this.survey_management ) {
			this.survey_management.editable();
		}
		if ( this.thresholds_instance ) {
			if ( !this.parent.is_readonly() ) {
				this.readonly_thresholds();
			}
		}
		if ( this.messages_instance ) {
			this.messages_instance.editable();
		}
		if ( this.skills_handle ) {
			this.skills_handle.editable();
		}
		if ( this.attention_levels_handle ) {
			this.attention_levels_handle.editable();
		}
		if (this.qna_maker_mng_instance) {
			this.qna_maker_mng_instance.editable();
		}
		if (this.hsm_mng_instance) {
			this.hsm_mng_instance.editable();
		}
		if (this.blacklists_mng_instance) {
			this.blacklists_mng_instance.editable();
		}
	};

	account_panel.prototype.readonly = function( internal_tab )
	{
		var members = _.filter([
			this.thresholds_instance,
			this.contact_mng_instance,
			this.flow_mng_instance,
			this.ticket_emitter_instance,
			this.messages_instance,
			this.survey_management,
			this.skills_handle,
			this.attention_levels_handle,
			this.qna_maker_mng_instance,
			this.hsm_mng_instance,
			this.blacklists_mng_instance
		], _.isObject);
		_.invoke(members, "readonly");
		this.parent.readonly( internal_tab );
	};

	account_panel.prototype.disable = function( internal_tab )
	{
		var members = _.filter([
			this.thresholds_instance,
			this.contact_mng_instance,
			this.flow_mng_instance,
			this.ticket_emitter_instance,
			this.messages_instance,
			this.survey_management,
			this.skills_handle,
			this.attention_levels_handle,
			this.qna_maker_mng_instance,
			this.hsm_mng_instance,
			this.blacklists_mng_instance
		], _.isObject);
		_.invoke(members, "disable");
		this.parent.disable( internal_tab );
	};

	account_panel.prototype.get_campaign_id = function ()
	{
		var campaign_id;
		var entity = this.parent.frm.get_entity_instance();
		if ( entity !== null && entity !== undefined ) {
			campaign_id = entity.Id;
		}
		return campaign_id;
	};

	account_panel.prototype.is_new = function ()
	{
		return !this.current_account;
	};

	account_panel.prototype.get_account = function()
	{
		return this.current_account;
	};

	account_panel.prototype.set_account = function ( account )
	{
		this.current_account = account || null;
	};

	account_panel.prototype.load = function(callback = $.noop)
	{
		callback();
	};

	account_panel.prototype.initialize = function()
	{
		this.set_events();

		this.initialize_tabs();
		this.initialize_thresholds();
		this.initialize_messages();
		this.initialize_contact_mng();
		this.initialize_flows_mng();
		this.initialize_ticket_emitter();
		this.initialize_survey_management();
		this.initialize_skills();
		this.initialize_attention_levels();
		this.initialize_qna_maker_mng();
		this.initialize_hsm_mng();
		this.initialize_blacklists_mng();
		this.initialize_client_window_config();

		var is_readonly = this.parent.is_readonly();
		if ( is_readonly ) {
			this.readonly();
		}
		else {
			this.editable();
		}
	};

	account_panel.prototype.initialize_client_window_config = function()
	{
		if (!this.gui_container_id) {
			return;
		}
		const $gui_container = $(this.gui_container_id);
		if ($gui_container.length === 0) {
			return;
		}
		let cbo_id = this.prefix_id + "gui_window_open_mode";
		let markup = "" +
			// "<h2 class='formSubTitle'>" +
			// 	"<label class='subTitleText'><%=_T([[Client Window]])%></label>" +
			// 	"<label class='subTitleHelp'>/<%=_T([[You can configure how the window is displayed]])%></label>" +
			// "</h2>" +
			"<div class='form-group'>" +
				"<label for='" + cbo_id + "'><%=_T([[Client window display]])%><%=_T([[:]])%></label>" +
				"<select id='" + cbo_id + "' class='form-control'></select>" +
			"</div>";

		$gui_container.html(markup);

		const list = [
			{ id: "MAXIMIZED", description: "<%=_T('Maximized')%>" },
			{ id: "NORMAL"   , description: "<%=_T('Normal')%>"    }
		];
		this.fill_gui_window_mode(list);
	};

	account_panel.prototype.initialize_messages = function()
	{
		if ( !this.messages_class ) {
			return;
		}
		this.messages_instance = new this.messages_class({
			parent : this,
			frm    : this.parent.frm 
		});
	};

	account_panel.prototype.get_messages_instance = function()
	{
		return this.messages_instance;
	};

	account_panel.prototype.validate_messages = function()
	{
		var response = { success: true };
		if ( !this.messages_class ) {
			return response;
		}
		var messages_inst = this.get_messages_instance();
		var validate_result = messages_inst.validate();
		if ( validate_result.error ) {
			this.tabs.switch_tab( "messages" );
			validate_result.$ctrl.focus();
			response.success = false;
			response.error_message = validate_result.error_message || "<%=_T('Error validating account messages, please verify the marked fields')%>";
		}
		return response;
	};

	account_panel.prototype.initialize_thresholds = function( type, parameters )
	{
		if (this.current_account && this.current_account.id) {
			parameters.account_id = this.current_account.id;
		}

		var thresholds_external_config = new inConcert.adm_thresholds_external_config();
		this.thresholds_instance = thresholds_external_config.initialize( type, parameters );
	};

	account_panel.prototype.get_thresholds_instance = function()
	{
		return this.thresholds_instance;
	};

	account_panel.prototype.validate_thresholds = function(account_config = {})
	{
		if ( !this.thresholds_instance ) {
			return { success: true };
		}
		var result = this.thresholds_instance.validate(account_config);
		if ( !result.success ) {
			if ( result.$ctrl ) {
				result.$ctrl.focus();
			}
		}
		return result;
	};

	account_panel.prototype.validate_contact_mng = function()
	{
		if ( !this.contact_mng_instance ) {
			return { success: true };
		}
		var result = this.contact_mng_instance.validate();
		if ( !result.success ) {
			if ( result.$ctrl ) {
				result.$ctrl.focus();
			}
		}
		return result;
	};

	account_panel.prototype.validate_qna_maker_mng = function()
	{
		if (!this.qna_maker_mng_instance) {
			return { success: true };
		}
		var result = this.qna_maker_mng_instance.validate();
		if (!result.success) {
			if (result.$ctrl) {
				result.$ctrl.focus();
			}
		}
		return result;
	};

	account_panel.prototype.validate_hsm_mng = function()
	{
		if (!this.hsm_mng_instance) {
			return { success: true };
		}
		var result = this.hsm_mng_instance.validate();
		if (!result.success) {
			if (result.$ctrl) {
				result.$ctrl.focus();
			}
		}
		return result;
	};

	account_panel.prototype.validate_flows_mng = function()
	{
		if ( !this.flow_mng_instance ) {
			return { success: true };
		}
		var result = this.flow_mng_instance.validate();
		if ( !result.success ) {
			if ( result.$ctrl ) {
				result.$ctrl.focus();
			}
		}
		return result;
	};

	account_panel.prototype.validate_ticket_emitter = function ()
	{
		if (!this.ticket_emitter_instance) {
			return { success: true, result: {} };
		}
		var result = this.ticket_emitter_instance.validate();
		if (!result.success && result.$ctrl) {
			result.$ctrl.focus();
		}
		return result;
	};

	account_panel.prototype.validate_survey = function ()
	{
		if (!this.survey_management) {
			return { success: true, result: {} };
		}
		var result = this.survey_management.validate();
		if (!result.success && result.$ctrl) {
			result.$ctrl.focus();
		}
		return result;
	};

	account_panel.prototype.validate_skills = function()
	{
		if ( !this.skills_handle ) {
			return { success: true };
		}
		return this.skills_handle.validate();
	};

	account_panel.prototype.validate_attention_levels = function()
	{
		if ( !this.attention_levels_handle ) {
			return { success: true };
		}
		return this.attention_levels_handle.validate();
	};

	account_panel.prototype.validate_blacklists = function()
	{
		if (!this.blacklists_mng_instance) {
			return { success: true };
		}
		const result = this.blacklists_mng_instance.validate();
		if (!result.success) {
			if (result.$ctrl) {
				result.$ctrl.focus();
			}
		}
		return result;
	};

	account_panel.prototype.can_configure_thresholds = function( /*internal_tab*/ )
	{
		// SI puede configurar
		return true;
	};

	account_panel.prototype.on_hide_all_tabs_thresholds = $.noop;

	account_panel.prototype.enable_thresholds = function( internal_tab )
	{
		let account_id = (this.current_account && this.current_account.id) ? this.current_account.id : undefined;

		this.thresholds_instance.editable( internal_tab, account_id );
		this.hide_thresholds_disclaimer();
	};

	account_panel.prototype.disable_thresholds = function( force_disabled, internal_tab )
	{
		if ( force_disabled || !this.can_configure_thresholds(internal_tab) ) {
			this.thresholds_instance.disable( internal_tab );
			this.show_thresholds_disclaimer();
		}
		else {
			this.enable_thresholds( internal_tab );
		}
	};

	account_panel.prototype.readonly_thresholds = function( force_disabled, internal_tab )
	{
		if ( force_disabled || !this.can_configure_thresholds(internal_tab) ) {
			this.thresholds_instance.readonly( internal_tab );
			this.show_thresholds_disclaimer();
		}
		else {
			this.enable_thresholds( internal_tab );
		}
	};

	account_panel.prototype.show_thresholds_disclaimer = function()
	{
		var thresholds_section_id = ( this.thresholds_instance.section_id || "" );
		var $disclaimer = $( "#" + thresholds_section_id ).find( "[thresholds_disclaimer='true']" );
		if ( $disclaimer.length > 0 ){
			$disclaimer.show();
		}
	};

	account_panel.prototype.hide_thresholds_disclaimer = function()
	{
		var thresholds_section_id = ( this.thresholds_instance.section_id || "" );
		var $disclaimer = $( "#" + thresholds_section_id ).find( "[thresholds_disclaimer='true']" );
		if ( $disclaimer.length > 0 ){
			$disclaimer.hide();
		}
	};

	account_panel.prototype.initialize_contact_mng = function( parameters )
	{
		parameters = parameters || {
			type   : "UNDEFINED",
			parent : this
		};
		this.contact_mng_instance = new inConcert.Administration.Campaigns.ContactManagement( parameters );
		this.contact_mng_instance.initialize();
	};

	account_panel.prototype.initialize_qna_maker_mng = function(parameters)
	{
		if (!this.has_qna_maker()) {
			return;
		}
		parameters = parameters || {
			type   : "UNDEFINED",
			parent : this
		};
		this.qna_maker_mng_instance = new inConcert.Administration.Campaigns.QnAMakerManagement(parameters);
		if (!this.qna_maker_mng_instance.has_configured_qna_maker_in_vcc()) {
			return;
		}
		this.qna_maker_mng_instance.initialize();
	};

	account_panel.prototype.initialize_hsm_mng = function(parameters)
	{
		if (!this.has_hsm_maker()) {
			return;
		}
		parameters = parameters || {
			type   : "UNDEFINED",
			parent : this
		};
		if(parameters.parent.is_new()) {
			if(this.hsm_mng_instance && this.hsm_mng_instance.container_id) {
				$( this.hsm_mng_instance.container_id ).html( "" );
			}
			this.hsm_mng_instance = null;
		}
		else
		{
			if(parameters.parent.current_account && parameters.parent.current_account.integrationType === "ICC_CLOUD_API") {
				this.hsm_mng_instance = new inConcert.Administration.Campaigns.HSM_Management_cloud_api(parameters);
			}
			else {
				this.hsm_mng_instance = new inConcert.Administration.Campaigns.HSM_Management(parameters);
			}
			this.hsm_mng_instance.initialize();
		}
	};

	account_panel.prototype.initialize_flows_mng = function( parameters )
	{
		if (!this.has_flow_management()) {
			return;
		}
		parameters = parameters || {
			type   : "UNDEFINED",
			parent : this
		};
		parameters.inactivity = this.has_flow_inactivity() ? this.get_flow_inactivity_time() : null;
		this.flow_mng_instance = new inConcert.Administration.Campaigns.FlowManagement( parameters );
		this.flow_mng_instance.initialize();
	};

	account_panel.prototype.initialize_ticket_emitter = function( parameters )
	{
		parameters = parameters || {
			type   : "UNDEFINED",
			parent : this
		};
		this.ticket_emitter_instance = new inConcert.Administration.Campaigns.TicketEmitterManagement( parameters );
		this.ticket_emitter_instance.initialize();
	};

	account_panel.prototype.initialize_survey_management = function( parameters )
	{
		if( !parameters ) {
			return;
		}
		this.survey_management = new inConcert.Administration.Campaigns.SurveyManagement( parameters );
		this.survey_management.initialize();
	};

	account_panel.prototype.initialize_blacklists_mng = function(parameters)
	{
		if (!this.has_blacklists()) {
			return;
		}
		parameters = parameters || {
			type  : "UNDEFINED",
			parent: this,
			frm   : this.parent.frm
		};
		parameters.help = parameters.help;
		this.blacklists_mng_instance = new inConcert.conf_form_blacklists(parameters);
		this.blacklists_mng_instance.initialize();
	};

	account_panel.prototype.initialize_tabs = function( parameters )
	{
		parameters = parameters || {};
		this.tabs = new inConcert.conf_form_tab( parameters );
	};

	account_panel.prototype.initialize_skills = function( parameters )
	{
		parameters = parameters || {};
		parameters.title = parameters.title || "<%=_T('Select Agents skills available for weight in distribution')%>";
		parameters.help = parameters.help || "";
		parameters.input_ctrls = {
			skill_value    : "COMBO",
			skill_required : true
		};
		// instancia del handle de skills (asignacion)
		this.skills_handle = new inConcert.adm_skills_handle( parameters );
		this.skills_handle.initialize();
	};

	account_panel.prototype.initialize_attention_levels = function( parameters )
	{
		parameters = parameters || {};
		parameters.title = parameters.title || "<%=_T('Select Attention Levels available for conversations')%>";
		parameters.help = parameters.help || "<%=_T('If more than one Attention Level selected, please select one of them as the receiver of incoming conversations activating the control placed on the Distribution column.')%>";
		parameters.help += " " + "<%=_T('Priority by date allows when transferring to a attention level to prioritize the interaction by date of creation or current transfering.')%>";
		parameters.account_priority_id = this.priority;

		// instancia del handle de niveles de atencion (asignacion)
		this.attention_levels_handle = new inConcert.attention_levels_handle( parameters );
		this.attention_levels_handle.initialize();
	};

	account_panel.prototype.hide_marked_errors = function ()
	{
		$(this.container_id).find(".formFieldError").hide();
	};

	account_panel.prototype.mark_error = function( $fieldNode )
	{
		var validationFieldId = $fieldNode.attr("id") + "_validation";
		var validationField = $("#" + validationFieldId);
		if (validationField) {
			validationField.show();
		}
		$fieldNode.focus();
		return false;
	};

	account_panel.prototype.is_valid = function( value, type, options )
	{
		return inConcert.Utils.validate( value, type, options );
	};

	account_panel.prototype.before_saving = function(callback)
	{
		callback = callback || $.noop;

		if (!this.messages_instance) {
			callback(true);
			return;
		}

		this.messages_instance.start_upload_images(function(success) {
			callback(success);
		});
	};

	account_panel.prototype.validate_field = function( $ctrl, validation_type, message, options, callback_err, timeout_ms, $custom_message_container )
	{
		return this.parent.frm.validation.validate_field( $ctrl, validation_type, message, options, callback_err, timeout_ms, $custom_message_container );
	};

	account_panel.prototype.show_validation_error = function( $ctrl, message, timeout_ms, scrolling, show_custom_button, $custom_message_container )
	{
		return this.parent.frm.validation.show_validation_error( $ctrl, message, timeout_ms, scrolling, show_custom_button, $custom_message_container );
	};

	account_panel.prototype.validate = function()
	{
		return {
			success : true                 ,
			account : this.current_account ,
			is_new  : this.is_new()
		};
	};

	account_panel.prototype.show_or_hide_skills_section = function()
	{
		if ( ($(this.distribution).val()) && ($(this.distribution).val().toUpperCase() === "SKILL") ) {
			this.skills_handle.show_container();
			$(".adm_form_campaign_accounts_skills_config_hr").removeClass("d-none");
		}
		else {
			this.skills_handle.hide_container();
			$(".adm_form_campaign_accounts_skills_config_hr").addClass("d-none");
		}
	};

	account_panel.prototype.on_change_distribution = function()
	{
		this.show_or_hide_skills_section();
	};

	account_panel.prototype.set_events = function()
	{
		var self = this;
		$(this.distribution)
			.off("click")
			.on("click", function() {
				const value = $(this).val();
				self.on_change_distribution(value);
			});
	};

}());
