/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:false, browser:true, indent:false, maxerr:50, -W014, esversion: 11 */

/* global $, inConcert, Mustache */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	const FORMAT = inConcert.Utils.format;

	//---------------------------------------------------------------------
	// DIDs Pool Management
	function AdministrationAccountDidsPoolMng(params)
	{
		if (!params) {
			throw("AdministrationAccountDidsPoolMng - 'params' is required");
		}
		if (!params.owner) {
			throw("AdministrationAccountDidsPoolMng - 'params.owner' is required");
		}
		if (!params.prefix) {
			throw("AdministrationAccountDidsPoolMng - 'params.prefix' is required");
		}
		if (!params.container_id) {
			throw("AdministrationAccountDidsPoolMng - 'params.container_id' is required");
		}
		this.owner = params.owner;
		this.owner_name = params.owner_name || "";
		this.prefix = params.prefix + "_";
		this.container_id = params.container_id;
		this.repated_dids = {};
		return true;
	}

	if (typeof inConcert.AdministrationAccountDidsPoolMng !== "function") {
		inConcert.AdministrationAccountDidsPoolMng = AdministrationAccountDidsPoolMng;
	}

	AdministrationAccountDidsPoolMng.prototype.initialize = function()
	{
		this.__make_popup_html();
		this.empty_controls();
		this.set_events();
		this.alert_removed = false;
	};

	AdministrationAccountDidsPoolMng.prototype.set_existing_dids = function(dids)
	{
		this.repated_dids = dids;
	};

	AdministrationAccountDidsPoolMng.prototype.__make_popup_html = function()
	{
		const prefix = this.prefix;
		const html = `
		<div class="row">
			<div class="col-10">
				<div class="card-header">
					<h5><%=_T([[DIDs Package]])%></h5>
				</div>
			</div>
			<div class="col-2">
				<button id="${prefix}btn_repeated" class="pageButtonGeneric buttonColorWhite w-100 float-none" disabled="true"><%=_T([[View repeated DIDs]])%></button>
			</div>
		</div>
		<div class="row">
			<div class="col-12">
				<div id="${prefix}alert_message_container" class="inlineMessagesContainer DIDsMessagesContainer messageWarning mb-0">
					<span id="${prefix}alert_message_message"></span>
					<a id="${prefix}alert_message_close"><i class="far fa-times"></i></a>
				</div>
			</div>
		</div>
		<div class="row">
			<div class="col-10">
				<div class="card-block">
					<div id="${prefix}table_container" class="DIDsManagementTableContainer">
						<table cellpadding="0" cellspacing="0" class="DIDsManagementTable">
							<thead>
								<tr>
									<th id="${prefix}header_state_active"><%=_T([[Active]])%> (0)</th>
									<th id="${prefix}header_state_disabled"><%=_T([[Disabled]])%> (0)</th>
									<th id="${prefix}header_state_rejected"><%=_T([[Rejected]])%> (0)</th>
								</tr>
							</thead>
							<tbody>
								<tr>
									<td>
										<div id="${prefix}column_state_active"></div>
									</td>
									<td>
										<div id="${prefix}column_state_disabled"></div>
									</td>
									<td>
										<div id="${prefix}column_state_rejected"></div>
									</td>
								</tr>
							</tbody>
						</table>
					</div>
				</div>
			</div>
			<div class="col-2">
				<div class="card-header dids-card-header">
					<h5>
						<%=_T([[Add DIDs]])%>
						<i class="fa fa-question-circle" title="<%=_T([[You can add multiple DIDs separated by ;.]])%>"></i>
					</h5>
					<span id="${prefix}header_panel_configuracion">(0)</span>
				</div>
				<div class="card-block dids-card-block">
					<div class="row h-100">
						<div class="col-12">
							<div class="form-group">
								<textarea id="${prefix}configuration" class="form-control" placeholder="<%=_T([[Add the ID(s).]])%>"></textarea>
							</div>
						</div>
					</div>
				</div>
				<div class="dids-card-footer">
					<button id="${prefix}btn_add" class="pageButtonGeneric buttonColorBlue w-100" disabled="true"><%=_T([[Add DIDs]])%></button>
				</div>
			</div>
		</div>
		<div class="row">
			<div class="col-12">
				<div id="${prefix}actions_container" class="formGroupsActionsFooterContainer" style="display:none;">
					<div class="formGroupsActionsFooterLeftContainer">
						<h5><span id="${prefix}actions_selected_number">4</span><%=_T([[Selected items]])%></h5>
						<h6 id="${prefix}actions_submessage"></h6>
					</div>
					<div class="formGroupsActionsFooterRightContainer">
						<button id="${prefix}btn_cancel" class="pageButtonGeneric buttonColorTertiaryLight"><%=_T([[Cancel]])%></button>
						<button id="${prefix}btn_delete" class="buttonToggle buttonToggleOutlined"><i class="far fa-trash-alt"></i></button>
						<button id="${prefix}btn_move" class="form-control" data-toggle="dropdown" title="<%=_T([[A Disabled or Rejected/Blocked DID can be moved to Active. An Active DID can be moved to Disabled.]])%>">
							<%=_T([[Move to]])%> 
						</button>
						<div id="${prefix}action_move_to_container" class="dropdown-menu btnDropDownMoreActions">
							<a id="${prefix}action_move_to_active" class="dropdown-item" type="button"><%=_T([[Active]])%></a>
							<a id="${prefix}action_move_to_disabled"  class="dropdown-item" type="button"><%=_T([[Disabled]])%></a>
						</div>
					</div>
				</div>
			</div>
		</div>
		<div id="${prefix}repeated_popup" class="internalPopUpContainer formPopUp popUp520w" hidden>
			<div class="internalPopUpOverlay"></div>
			<div class="internalPopUp">
				<div class="internalPopUpTitle">
					<span><%=_T([[Repeated DIDs]])%></span>
				</div>
				<div class="internalPopUpMiddle pageFormContainer">
					<div class="card no-card">
						<div class="card-block">
							<div class="row">
								<div class="col-12">
									<table id="${prefix}repeated_popup_table" class="repeatedDIDsTable">
										<thead>
											<th><%=_T([[Name]])%></th>
											<th><%=_T([[Description]])%></th>
											<th><%=_T([[Accounts]])%></th>
										</thead>
									</table>
								</div>
							</div>
						</div>
					</div>
				</div>
				<div class="internalPopUpButtons">
					<button id="${prefix}repeated_popup_close" class='pageButtonGeneric buttonColorWhite float-right'><%=_T([[Close]])%></button>
				</div>
			</div>
		</div>
		`;
		$(`#${this.container_id}`).html(html);
	};

	AdministrationAccountDidsPoolMng.prototype.dispose = function() {};

	AdministrationAccountDidsPoolMng.prototype.validate = function()
	{
		const dids = this.__get_data_dids();
		return { success: true, data: { dids: dids } };
	};

	AdministrationAccountDidsPoolMng.prototype.set = function(dids, accountName)
	{
		this.owner_name = accountName;
		this.__make_markup(dids);
	};

	AdministrationAccountDidsPoolMng.prototype.empty_controls = function()
	{
		//Clean all
		$(`#${this.prefix}configuration`).val("");
		$(`#${this.prefix}column_state_active`).html("");
		$(`#${this.prefix}column_state_disabled`).html("");
		$(`#${this.prefix}column_state_rejected`).html("");
		this.__update_counter_of_dids_by_status();
	};

	AdministrationAccountDidsPoolMng.prototype.__get_data_dids = function(confirm_add)
	{
		confirm_add = typeof confirm_add ==="function"? confirm_add: function() {
			return true;	
		};
		const $input_dids = $(`#${this.prefix}table_container input[type=checkbox]`);
		let list_dids = [];
		$input_dids.each(function() {
			const $did = $(this);
			if (confirm_add($did)) {
				list_dids.push($did.data());
			}
		});
		return list_dids;
	};

	AdministrationAccountDidsPoolMng.prototype.__$get_selected_dids = function()
	{
		const $selected_dids = $(`#${this.prefix}table_container input[type=checkbox]`);
		let list_selected = [];
		$selected_dids.each(function() {
			const $this = $(this);
			if ($this.prop("checked")) {
				list_selected.push($this);
			}
		});
		return list_selected;
	};

	AdministrationAccountDidsPoolMng.prototype.__update_actions_container = function(cont_selected = 0)
	{
		$(`#${this.prefix}actions_container`)[cont_selected > 0? "show" : "hide"]();
		$(`#${this.prefix}actions_selected_number`).text(cont_selected);
		$(`#${this.prefix}actions_submessage`).text(FORMAT("{0} {1} DIDs","<%=_T([[You are about to configure]])%>", cont_selected));
	};

	AdministrationAccountDidsPoolMng.prototype.__update_counter_of_dids_by_status = function(list_selected)
	{
		const obj_dids_by_status = this.__get_object_dids_data_by_status(list_selected);
		$(`#${this.prefix}header_state_active`).text(FORMAT("{0} ({1})","<%=_T([[Active]])%>", obj_dids_by_status.active.length));
		$(`#${this.prefix}header_state_disabled`).text(FORMAT("{0} ({1})","<%=_T([[Disabled]])%>", obj_dids_by_status.disabled.length));
		$(`#${this.prefix}header_state_rejected`).text(FORMAT("{0} ({1})","<%=_T([[Rejected]])%>", obj_dids_by_status.rejected.length));
		this.__check_repeated();
	};

	AdministrationAccountDidsPoolMng.prototype.__get_object_dids_data_by_status = function(list_selected)
	{
		list_selected = list_selected || this.__get_data_dids();
		let list_active = [];
		let list_disabled = [];
		let list_rejected = []; 
		
		$.each(list_selected , function(_, did) {
			
			if (did.status === "active") {
				list_active.push(did);
			}
			if (did.status === "disabled") {
				list_disabled.push(did);
			}
			if (did.status === "rejected") {
				list_rejected.push(did);
			}
		});

		return {
			active: list_active,
			rejected: list_rejected,
			disabled: list_disabled
		};
	};

	AdministrationAccountDidsPoolMng.prototype.__show_actions_move_to = function(list_data_selected)
	{
		list_data_selected = list_data_selected || this.__get_data_dids(function($did) {
			return $did.prop("checked");
		});
		const $btn_move = $(`#${this.prefix}btn_move`);
		$btn_move.prop("disabled", true);
		if (!list_data_selected.length) {
			return;
		}
		const $active_option = $(`#${this.prefix}action_move_to_active`);
		$active_option.hide();
		const $disabled_option = $(`#${this.prefix}action_move_to_disabled`);
		$disabled_option.hide();
		
		const all_active = list_data_selected.every(function(data) {
			return data.status === "active";
		});

		if (all_active) {
			//Muestro deshabilitados
			$disabled_option.show(); 
			$btn_move.prop("disabled", false);
			return;
		} 

		const all_rejected_disabled = list_data_selected.every(function(data) {
			return data.status === "rejected" || data.status === "disabled";
		});

		if (all_rejected_disabled) {
			//Muestro activos
			$active_option.show();
			$btn_move.prop("disabled", false);
			return;
		}
		
	};

	AdministrationAccountDidsPoolMng.prototype.__on_click_did_input = function()
	{
		const list_data_selected = this.__get_data_dids(function($did) {
			return $did.prop("checked");
		});

		this.__update_actions_container(list_data_selected.length);

		this.__show_actions_move_to(list_data_selected);
	};

	AdministrationAccountDidsPoolMng.prototype.__validate_dids_to_add = function(list_did)
	{
		const validated = list_did.every(function(elem){
			let validation_result = inConcert.Utils.validate( elem, "number");
			return validation_result.success;
		});
		if (!validated) {
			const message = "<%=_T([[Only numeric values of did are allowed.]])%>";
			this.owner.parent.frm.show_status_message(message, "Error", 7000);
		}

		return validated;
	};

	AdministrationAccountDidsPoolMng.prototype.set_events = function()
	{
		const self = this;
		//Borrar dids de la tabla por el boton close(x) en el hover
		const $table_container = $(`#${this.prefix}table_container`);
		$table_container
		.off("click",`#${this.prefix}hover_btn_close`)
		.on("click", `#${this.prefix}hover_btn_close`, function(e) {
			$(this).parent().remove();
			const list_selected = self.__get_data_dids(function($did) {
				return $did.prop("checked");
			});
			
			let cont_selected = list_selected.length;
			self.__update_actions_container(cont_selected);
			self.__update_counter_of_dids_by_status();
		});

		//Deseleccionar todos los dids marcados (Boton cancel)
		const $btn_cancel = $(`#${this.prefix}btn_cancel`);
		$btn_cancel
		.off("click")
		.on("click", function(e) {
			const $input_dids = $(`#${self.prefix}table_container input[type=checkbox]`);
			$input_dids.each(function() {
				$(this).prop("checked", false);
			});
			self.__update_actions_container();
		});

		//Eliminar todos los dids seleccionados
		const $btn_delete = $(`#${this.prefix}btn_delete`);
		$btn_delete
		.off("click")
		.on("click", function(e) {
			self.__remove_selected_inputs();
			self.__update_actions_container();
			self.__update_counter_of_dids_by_status();
		});

		//mover los dids seleccionados a un estado
		const $btn_move_to_active = $(`#${this.prefix}action_move_to_active`);
		$btn_move_to_active
		.off("click")
		.on("click", function(e) {
			self.__move_selected_dids_to("active");
		});

		const $btn_move_to_disabled = $(`#${this.prefix}action_move_to_disabled`);
		$btn_move_to_disabled
		.off("click")
		.on("click", function(e) {
			self.__move_selected_dids_to("disabled");
		}); 

		//Hide-Show text area de dids
		const event_fn = function() {
			const values = $(this).val().replaceAll(/[\s\r\t]+/g, "");
			let dids = values.split(";");
			if (dids.length) {
				dids = dids.filter(item => {
					return item !== "";
				});
			}

			const number = values === "" || !dids.length ? 0 : dids.length;
			const $header_number_dids = $(`#${self.prefix}header_panel_configuracion`);
			$header_number_dids.text(`(${number})`);
			$btn_add.prop("disabled", number === 0);
		};
		const field_events_fn = {
			keyup: event_fn,
			page: event_fn,
			change: event_fn
		};

		const $configuration = $(`#${this.prefix}configuration`);
		$configuration
		.off("keyup", "page", "change" )
		.on(field_events_fn);

		//Boton Agregar dids. Solo visible si existe data en el text area.
		const $btn_add = $(`#${this.prefix}btn_add`);
		$btn_add
		.off("click")
		.on("click", function(e) {
			
			//Se limpian lo espacios, tab, enter.
			const values = $configuration.val().replaceAll(/[\s\r\t]+/g, "");
			let array_dids = values.split(";");
			
			if (!self.__validate_dids_to_add(array_dids)) {
				return;
			}
			
			if (array_dids.length) {
				array_dids = array_dids.filter((item, index) => {
					return item !== "" && array_dids.indexOf(item) === index;
				});
			}

			let already_added = [];
			const current_dids_obj = inConcert.Utils.arrayToMap(self.__get_data_dids() || [],
				function(did) {
					return did;
				},
				function(did) {
					return did.number;
				}
			);
			
			$.each(array_dids, function(_, did) {
				if (current_dids_obj[did]) {
					already_added.push(did);
				}
				else{
					self.__add_markup_did_by_state({
						status:"active",
						number: did 
					});
				}
			});
			$configuration.val("");
			$configuration.trigger("keyup");
			self.__update_counter_of_dids_by_status();
			if (already_added.length) {
				const message = FORMAT("<%=_T([[DIDs ({0}) already exists, are not added.]])%>", already_added.toString());
				self.owner.parent.frm.show_status_message(message, "Warning", 10000);
			}
		});

		$(`#${this.prefix}alert_message_close`)
			.off("click")
			.on("click", () => {
				this.alert_removed = true;
				this.__hide_alert();
			});

		$(`#${this.prefix}repeated_popup_close`)
			.off("click")
			.on("click", () => $(`#${this.prefix}repeated_popup`).hide());
	};

	AdministrationAccountDidsPoolMng.prototype.__enable_disable_inputs = function(enable, state)
	{
		const id = state? `${this.prefix}column_state_${state}`: `${this.prefix}table_container`;
		const $input_dids = $("#"+ id +" input[type=checkbox]");
		$input_dids.each(function() {
			$(this).prop("disable", !enable);
		});
	};

	/*
		Un DID podria pasar a Deshabilitado solo desde Activo. 
		Un DID Deshabilitado podria pasar a Activo.
		Un DID Rechazado/Bloqueado podria pasarlo a Activo.
	*/
	AdministrationAccountDidsPoolMng.prototype.__move_selected_dids_to = function(moveToState)
	{
		const self = this;
		const list_selected = self.__$get_selected_dids();
		$.each(list_selected, function(_, did) {
			const $did =  $(did);
			const data = $did.data();
			$did.parent().remove();
			self.__add_markup_did_by_state({
				status: moveToState,
				number: data.number
			});
		});
		self.__update_actions_container();
		self.__update_counter_of_dids_by_status();
	};

	AdministrationAccountDidsPoolMng.prototype.__remove_selected_inputs = function()
	{
		const $input_dids = $(`#${this.prefix}table_container input[type=checkbox]`);
		$input_dids.each(function() {
			if ($(this).prop("checked")) {
				$(this).parent().remove();
			}
		});
	}; 

	AdministrationAccountDidsPoolMng.prototype.__make_markup = function(didsPool)
	{
		if (!didsPool.length) {
			return;
		}
		this.empty_controls();
		const self = this;
		$.each(didsPool, function(_, elem) {
			self.__add_markup_did_by_state(elem);
		});

		this.__update_counter_of_dids_by_status();
	};

	AdministrationAccountDidsPoolMng.prototype.__add_markup_did_by_state = function(elem)
	{
		const $container = $(`#${this.prefix}column_state_${elem.status}`);
		const self = this;
		const $did = $(Mustache.render(("").concat(
					'<label id="{{prefix}}did_input_{{did}}_container" for="{{prefix}}did_input_{{did}}" class="form-check-label did-pool-element">',
						'<input id="{{prefix}}did_input_{{did}}" type="checkbox" class="form-check-input"/>',
						'<span>{{did}}</span>',
						'<button id="{{prefix}}hover_btn_close">',
							'<i class="far fa-times"></i>',
						'</button>',
					'</label>'), {
						did: elem.number,
						prefix: this.prefix,
					}));
			$container.append($did);
			const $input_did = $(`#${this.prefix}did_input_${elem.number}`);
			$input_did.data(elem);
			$input_did
				.off("click")
				.on("click", function(e) {
					self.__on_click_did_input();
				});
	};

	AdministrationAccountDidsPoolMng.prototype.__check_repeated = function()
	{
		$(".did-pool-element").removeClass("repeated").prop("title", "");
		const repeatedDids = this.__get_data_dids()
			.map((did) => this.__check_repeated_did(did))
			.filter((did) => did);
		if (!repeatedDids.length) {
			this.__hide_alert();
			$(`#${this.prefix}btn_repeated`).prop("disabled", true).off("click");
			return;
		}

		this.__show_alert(`
			<%=_T([[There are DIDs that are part of another account.]])%>
			<b><%=_T([[Already exists in the DID pool or is part of an inbound DNIS]])%></b>
		`);
		$(`#${this.prefix}btn_repeated`)
			.prop("disabled", false)
			.off("click")
			.on("click", () => this.__show_repeated_popup(repeatedDids));
		repeatedDids.forEach((did) =>
			$(`#${this.prefix}did_input_${did.number}_container`).addClass("repeated").prop("title", did.tooltip)
		);
	};

	AdministrationAccountDidsPoolMng.prototype.__check_repeated_did = function(did)
	{
		let repeated = this.repated_dids[did.number];
		if (!repeated) return null;

		repeated = Object.keys(repeated)
			.filter((accountName) => accountName !== this.owner_name)
			.map((accountName) => ({ accountName, place: repeated[accountName] }));
		if (!repeated.length) return null;

		return {
			...did,
			repeated,
			tooltip : repeated.some((account) => account.place === "pool")
				? "<%=_T([[Did already exists on another account pool]])%>"
				: "<%=_T([[Did already exists on another account inbound DNIS]])%>"

		};
	};

	AdministrationAccountDidsPoolMng.prototype.__show_repeated_popup = function(repeatedDids)
	{
		$(".repeated_popup_row").remove();
		$(`#${this.prefix}repeated_popup_table`).append(
			repeatedDids.map((repeatedDid) => this.__make_repeated_popup_did_rows(repeatedDid))
		);
		$(`#${this.prefix}repeated_popup`).show();
	};

	AdministrationAccountDidsPoolMng.prototype.__make_repeated_popup_did_rows = function(repeatedDid)
	{
		const byPlace = repeatedDid.repeated.reduce((byPlace, account) => {
			byPlace[account.place] = byPlace[account.place] || [];
			byPlace[account.place].push(account.accountName);
			return byPlace;
		}, {});
		return Object.keys(byPlace).sort().map((place) =>
			this.__make_repeated_popup_did_row(repeatedDid.number, place, byPlace[place].sort()));
	};

	AdministrationAccountDidsPoolMng.prototype.__make_repeated_popup_did_row = function(number, place, accounts)
	{
		const description = place === "pool"
			? "<%=_T([[Did already exists on another account pool]])%>"
			: "<%=_T([[Did already exists on another account inbound DNIS]])%>";
		const accountNames = accounts.join(", ");
		return `
			<tr class="repeated_popup_row">
				<td>${number}</td>
				<td>${description}</td>
				<td>${accountNames}</td>
			</tr>
		`;
	};

	AdministrationAccountDidsPoolMng.prototype.__show_alert = function(message)
	{
		if (this.alert_removed) return;

		$(`#${this.prefix}alert_message_message`).html(message);
		$(`#${this.prefix}alert_message_container`).show();
	};

	AdministrationAccountDidsPoolMng.prototype.__hide_alert = function()
	{
		$(`#${this.prefix}alert_message_container`).hide();
	};

}());
