/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 11 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

// try improve namespacing a bit
if ( !this.inConcert.Administration ) {
	this.inConcert.Administration = {};
}

if ( !this.inConcert.Administration.Campaigns ) {
	this.inConcert.Administration.Campaigns = {};
}

(function () {

	"use strict";

	var format = inConcert.Utils.format;

	var mark_chk = function( ctrl, checked ) { $( ctrl ).prop( "checked", checked ); };

	var get_chk_bool_to_string  = function( chk_id ) {
		var $chk = $( chk_id );
		if ( $chk.length === 0 ) {
			return undefined;
		}
		var checked = $chk.prop("checked");
		return Number( checked ).toString();
	};

	//---------------------------------------------------------------------
	// frm im account edition
	function im_account_panel( params )
	{
		/* jshint validthis: true*/
		this.prefix_id = "adm_form_campaign_accounts_im_";
		var success = im_account_panel.uber.constructor.call( this, params );
		if (!success) {
			return;
		}

		this.container_id             = "#adm_form_campaign_accounts_im_panel";

		this.protocol                 = "#adm_form_campaign_accounts_im_protocol";
		this.user                     = "#adm_form_campaign_accounts_im_user";
		this.pwd                      = "#adm_form_campaign_accounts_im_pwd";
		this.alias                    = "#adm_form_campaign_accounts_im_alias";
		this.distribution             = "#adm_form_campaign_accounts_im_distribution";
		this.language                 = "#adm_form_campaign_accounts_im_language";
		this.priority                 = "#adm_form_campaign_accounts_im_priority";
		this.auto_accept_friends      = "#adm_form_campaign_accounts_im_auto_accept_friends";
		this.auto_add_contacts        = "#adm_form_campaign_accounts_im_auto_add_contacts";
		this.close_session_on_timeout = "#adm_form_campaign_accounts_im_close_session_on_timeout";
		this.close_session_minutes    = "#adm_form_campaign_accounts_im_close_session_minutes";

		this.messages_class = inConcert.adm_form_campaigns_im_account_messages;
		this.pwd_mask = "X X X X X";

		this.tagline_new      = "<%=_T('New IM account.')%>";
		this.tagline_edit     = "<%=_T('Edit IM account {0}.')%>";
		this.tagline_readonly = "<%=_T('IM account {0}.')%>";

		return true;
	}

	inConcert.Utils.inherit( im_account_panel, inConcert.adm_form_campaigns_section_accounts_base_panel );

	if ( typeof inConcert.Administration.Campaigns.IMAccountPanel !== "function" ) {
		inConcert.Administration.Campaigns.IMAccountPanel = im_account_panel;
	}

	im_account_panel.prototype.has_qna_maker = function() {return true;};

	im_account_panel.prototype.initialize = function()
	{
		var self = this;
		im_account_panel.uber.initialize.call( this );

		inConcert.app.loadIMProtcols( function( success, list ) {
			if ( !success ) {
				return;
			}
			self.fill_protocol( list );
		});

		var disabled = true, is_required = true;
		if ( !this.parent.is_readonly() ) {
			this.parent.frm.disable_control( $( this.close_session_minutes ), disabled, is_required );
		}

		disabled = false;
		this.parent.frm.disable_control( $( this.protocol ), disabled, is_required );
		this.parent.frm.disable_control( $( this.user ), disabled, is_required );

		// cuando ingreso reseteo el flag de que la pwd ha sido modificada.
		$( this.pwd ).attr( "pwd_changed", "0" );
	};

	im_account_panel.prototype.initialize_thresholds = function()
	{
		// build thresholds panel
		im_account_panel.uber.initialize_thresholds.call(
			this,
			"IM",
			{
				parent    : this.parent,
				prefix_id : "adm_form_campaign_accounts_im_thresholds"
			}
		);
	};

	im_account_panel.prototype.initialize_contact_mng = function()
	{
		// build contact management panel
		im_account_panel.uber.initialize_contact_mng.call(
			this,
			{
				type         : "IM",
				parent       : this ,
				auto_add_type: "BOTH", // incoming & outgoing
				container_id : "adm_form_campaign_accounts_im_contact_mng_panel",
				prefix_id    : "adm_form_campaign_accounts_im_contact_mng"
			}
		);
	};

	im_account_panel.prototype.initialize_flows_mng = function()
	{
		// build flows designer management panel
		im_account_panel.uber.initialize_flows_mng.call(
			this,
			{
				type         : "IM"                                                    ,
				parent       : this                                                    ,
				container_id : "adm_form_campaign_accounts_im_flows_designer_mng_panel",
				prefix_id    : "adm_form_campaign_accounts_im_flows_designer_mng"
			}
		);
	};

	im_account_panel.prototype.initialize_ticket_emitter = function()
	{
		im_account_panel.uber.initialize_ticket_emitter.call(
			this,
			{
				type         : "IM",
				parent       : this,
				container_id : "adm_form_campaign_accounts_im_tickets_mng_panel",
				prefix_id    : "adm_form_campaign_accounts_im_tickets_mng"
			}
		);
	};

	im_account_panel.prototype.initialize_qna_maker_mng = function()
	{
		// build qna_maker management panel
		im_account_panel.uber.initialize_qna_maker_mng.call(
			this,
			{
				type         : "IM"                                               ,
				parent       : this                                               ,
				container_id : "adm_form_campaign_accounts_im_qna_maker_mng_panel",
				prefix_id    : "adm_form_campaign_accounts_im_qna_maker_mng"
			}
		);
	};

	im_account_panel.prototype.initialize_tabs = function()
	{
		// build tabs
		im_account_panel.uber.initialize_tabs.call(
			this,
			{
				parent       : this,
				frm          : this.parent.frm,
				container_id : "adm_form_campaign_accounts_tabs",
				tabs_list    : {
					basic : {
						id               : "basic",
						name             : "<%=_T('IM Config')%>",
						tab_container_id : "adm_form_campaign_accounts_im_panel_tab_basic",
						selected         : true
					},
					messages : {
						id               : "messages",
						name             : "<%=_T('Messages')%>",
						tab_container_id : "adm_form_campaign_accounts_im_panel_tab_messages"
					}
				}
			}
		);
	};

	im_account_panel.prototype.initialize_skills = function()
	{
		im_account_panel.uber.initialize_skills.call(
			this,
			{
				container_id : "adm_form_campaign_accounts_im_skills_config",
				parent_frm   : this.parent,
				parent_type  : "FORM-SECTION"
			}
		);
	};

	im_account_panel.prototype.initialize_attention_levels = function()
	{
		im_account_panel.uber.initialize_attention_levels.call(
			this,
			{
				container_id : "adm_form_campaign_accounts_im_attention_levels_config",
				parent_frm   : this.parent,
				parent_type  : "FORM-SECTION",
				proxy_type   : "IM"
			}
		);
	};

	im_account_panel.prototype.fill_protocol = function( list )
	{
		var is_blank_first_item = true;
		var attr_id = "id";
		var attr_name = "description";
		var $combo = $( this.protocol );
		this.parent.frm.fill_combo( $combo, is_blank_first_item, list, attr_id, attr_name );
	};

	im_account_panel.prototype.edit = function(account)
	{
		im_account_panel.uber.edit.call( this, account );

		if ( !this.current_account ) {
			return;
		}
		var acc = this.current_account;
		var user = acc.user;

		if ( acc.protocol === "prpl-jabber" ) {
			var match_user = user.match( new RegExp( "^(.+)@([^@]+)$" ) );
			if ( match_user ) {
				user = match_user[ 1 ];
			}
		}

		var $protocol = $( this.protocol );
		var $user = $( this.user );
		var disabled = true;
		this.parent.frm.readonly_control( $protocol, disabled );
		this.parent.frm.readonly_control( $user, disabled );

		$protocol.val( acc.protocol );
		$user.val( user );
		$( this.pwd ).val( this.pwd_mask );
		$( this.alias ).val( acc.alias );

		mark_chk( this.auto_accept_friends, acc.autoAcceptFriends === "1" );
		mark_chk( this.auto_add_contacts, acc.autoAddContacts === "1" );

		var close_session_seconds = parseInt( acc.interactionTimeout, 10 );
		var has_configured_close_session = close_session_seconds > 0;
		mark_chk( this.close_session_on_timeout, has_configured_close_session );
		if ( has_configured_close_session ) {
			$( this.close_session_on_timeout ).trigger( "change" );
			$( this.close_session_minutes ).val( close_session_seconds / 60 );
		}
	};

	im_account_panel.prototype.empty_controls = function()
	{
		im_account_panel.uber.empty_controls.call( this );

		$( this.protocol ).val( "" );
		$( this.user ).val( "" );
		$( this.pwd ).val( "" );
		$( this.alias ).val( "" );

		mark_chk( this.auto_accept_friends, false );
		mark_chk( this.auto_add_contacts, false );
		mark_chk( this.close_session_on_timeout, false );
		$( this.close_session_on_timeout ).trigger( "change" );
	};

	im_account_panel.prototype.set_events = function()
	{
		im_account_panel.uber.set_events.call( this );

		if ( this.parent.is_readonly() ) {
			return;
		}

		var self = this;

		var $close_session_minutes = $( this.close_session_minutes );
		$( this.close_session_on_timeout ).off( "change" )
			.on( "change", function() {
				var disabled = !$( this ).prop( "checked" );
				var is_required = true;
				self.parent.frm.disable_control( $close_session_minutes, disabled, is_required );
				if ( !disabled ) {
					$close_session_minutes.focus();
				}
			});

		var pwd_events_fn = {
			blur: function() {
				var $this = $( this );
				var pwd_changed = $this.attr( "pwd_changed" );
				if ( pwd_changed !== "1" ) {
					$this.val( self.pwd_mask );
				}
			},
			focus: function() {
				var $this = $( this );
				var pwd_changed = $this.attr( "pwd_changed" );
				if ( pwd_changed !== "1" ) {
					$this.val( "" );
				}
			},
			keyup: function( event ) {
				var filtered_control_keys = inConcert.Utils.getControlCharacters();
				filtered_control_keys[ 17 ] = false; //CTRL is allowed for pasting
				if ( filtered_control_keys [ event.keyCode ] ) {
					return;
				}
				$( this ).attr( "pwd_changed", "1" );
			}
		};

		$( this.pwd )
			.off( "blur", "focus", "keyup" )
			.on( pwd_events_fn );
	};

	im_account_panel.prototype.on_show = function()
	{
		im_account_panel.uber.on_show.call( this );
		$( this.protocol ).focus();
	};

	im_account_panel.prototype.validate = function()
	{
		var self = this;

		var is_new = this.is_new();
		var account_info = {
			success: false,
			account: null,
			is_new:  is_new,
			error_message: null
		};

		var error_in_basic_tab = function() {
			self.tabs.switch_tab( "basic" );
			return account_info;
		};

		this.hide_marked_errors(); // oculto todos los errors de los campos

		// NAME - validation
		var $protocol = $( this.protocol );
		if ( !this.validate_field( $protocol, "required", "<%=_T('Please select the instant messaging protocol to be used.')%>" ) ) {
			return error_in_basic_tab();
		}
		var protocol = $protocol.val();

		// user (mail) - validation
		var $user = $( this.user );
		if ( !this.validate_field( $user, "required", "<%=_T('User is required.')%>" ) ) {
			return error_in_basic_tab();
		}
		var user = $user.val();
		var result_im = this.is_valid( user, "IMUser");
		if ( !result_im.success ) {
			if ( !this.validate_field( $user, "email", "<%=_T('User is not a valid.')%>" ) ) {
				return error_in_basic_tab();
			}
		}
		else {
			// if the user does not insert @gmail at the end, and try to fix it
			if ( protocol === "prpl-jabber" ) {
				user = user + "@gmail.com";
			}
		}

		// pwd - validation
		var $pwd = $( this.pwd );
		if ( !this.validate_field( $pwd, "required", "<%=_T('Password is required.')%>" ) ) {
			return error_in_basic_tab();
		}
		var pwd = null;
		if ( $pwd.attr( "pwd_changed" ) === "1" ) {
			pwd = $pwd.val();
		}

		// DISTRIBUTION - validation
		var $distribution = $( this.distribution );
		if ( !this.validate_field( $distribution, "required", "<%=_T('Distribution is required.')%>" ) ) {
			return error_in_basic_tab();
		}

		// LANGUAGE - validation
		var $language = $(this.language);
		if (!this.validate_field($language, "required", "<%=_T('Language is required.')%>")) {
			return error_in_basic_tab();
		}

		// PRIORITY - validation
		var $priority = $(this.priority);
		var has_att_levels = this.attention_levels_handle.count_selected() > 0;
		if (!has_att_levels && $priority.val().trim().length > 0) {
			var range = { between: { from: 0, to: 99 } }; // range attention_level value validation
			if (!this.validate_field($priority, "range", format("<%=_T('Account priority value out of range. Please enter account priority value between {0} and {1}.')%>", range.between.from, range.between.to ), range )) {
				return error_in_basic_tab();
			}
		}

		// advanced config - validation

		// close on timeout
		var close_session_seconds = 0;

		if ( $( this.close_session_on_timeout ).prop( "checked" ) ) {
			var $close_session_minutes = $( this.close_session_minutes );
			if (!this.validate_field( $close_session_minutes, "required", "<%=_T('Close session minutes is required.')%>")) {
				return error_in_basic_tab();
			}
			if (!this.validate_field( $close_session_minutes, "digits", "<%=_T('Close session minutes is not a number.')%>")) {
				return error_in_basic_tab();
			}
			var close_session_minutes = $close_session_minutes.val();
			var opts = { between : { from: 1, to: 30 } };
			var result_range = this.is_valid( close_session_minutes, "range", opts );
			if ( !result_range.success ) {
				var error_msg = format("<%=_T('Please enter Close session minutes between {0} and {1}.')%>", opts.between.from, opts.between.to );
				this.show_validation_error( $close_session_minutes, error_msg );
				return error_in_basic_tab();
			}
			close_session_seconds = parseInt( close_session_minutes, 10 ) * 60;
		}

		// valido los skills ----------
		var result_skills = this.validate_skills();
		if ( !result_skills.success ) {
			this.show_validation_error( result_skills.$ctrl, result_skills.message );
			return error_in_basic_tab();
		}

		// valido los attention levels ----------
		var result_attention_levels = this.validate_attention_levels();
		if ( !result_attention_levels.success ) {
			this.show_validation_error(
				result_attention_levels.$ctrl,
				result_attention_levels.message,
				false, //timeout_ms
				true, //scrolling
				result_attention_levels.show_custom_button
			);
			return error_in_basic_tab();
		}

		// valido los thresholds ----------
		var result_th = this.validate_thresholds();
		if ( !result_th.success ) {
			this.show_validation_error( result_th.$ctrl, result_th.error_message );
			return error_in_basic_tab();
		}

		// valido los contact mng ----------
		var result_contact_mng = this.validate_contact_mng();
		if ( !result_contact_mng.success ) {
			this.show_validation_error( result_contact_mng.$ctrl, result_contact_mng.error_message );
			return error_in_basic_tab();
		}

		// valido los flows mng ----------
		var result_flows_mng = this.validate_flows_mng();
		if ( !result_flows_mng.success ) {
			this.show_validation_error( result_flows_mng.$ctrl, result_flows_mng.error_message );
			return error_in_basic_tab();
		}

		// valido los qna maker mng ----------
		var result_qna_maker_mng = this.validate_qna_maker_mng();
		if ( !result_qna_maker_mng.success ) {
			this.show_validation_error( result_qna_maker_mng.$ctrl, result_qna_maker_mng.error_message );
			return error_in_basic_tab();
		}

		// valido los messages de la account ----------
		var result_messages = this.validate_messages();
		if ( !result_messages.success ) {
			this.show_validation_error( undefined, result_messages.error_message );
			return account_info;
		}

		// valido la configuracion de tickets
		var result_ticket_emitter = this.validate_ticket_emitter();
		if (!result_ticket_emitter.success) {
			this.show_validation_error( result_ticket_emitter.$ctrl, result_ticket_emitter.error_message );
			return error_in_basic_tab();
		}

		//------------------------------------------------------
		// paso la validacion correctamente, ahora a guardar!!!

		var account = {};
		if ( is_new ) {
			var uid = user + "@" + protocol;

			var campaign = this.parent.frm.campaign;
			account.id = "IM_" + campaign.Id + "_" + uid;

			account.protocol    = protocol;
			account.user        = user;
			account.name        = user;
			account.uid         = uid;
			account.type        = "IM";
			account.proxyType   = "IM";
		}
		else {
			account.id          = this.current_account.id;
			account.protocol    = this.current_account.protocol;
			account.user        = this.current_account.user;
			account.name        = this.current_account.name;
			account.uid         = this.current_account.uid;
			account.type        = this.current_account.type;
			account.proxyType   = this.current_account.proxyType;
		}

		account.password           = pwd;
		account.alias              = $( this.alias ).val();
		account.language           = $language.val();
		account.CCAlgorithm        = $distribution.val();
		account.autoAddContacts    = get_chk_bool_to_string( this.auto_add_contacts );
		account.autoAcceptFriends  = get_chk_bool_to_string( this.auto_accept_friends );
		account.interactionTimeout = close_session_seconds.toString();
		account.TicketEmitter      = result_ticket_emitter.result;
		account.Skills             = this.skills_handle.get();
		account.Priority           = $priority.val();
		account.AttentionLevels    = this.attention_levels_handle.get();
		account.GUI                = this.get_gui_config();
		this.contact_mng_instance.attach_data( account );
		if (this.flow_mng_instance) {
			this.flow_mng_instance.attach_data( account );
		}
		if (this.qna_maker_mng_instance) {
			this.qna_maker_mng_instance.attach_data(account);
		}

		account_info.account = account;
		account_info.success = true;

		return account_info;
	};

	im_account_panel.prototype.get_account_name = function()
	{
		return this.current_account.name;
	};

}());
