/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, esversion: 11 */

/* global $, inConcert */

if ( !this.inConcert ) {
	this.inConcert = {};
}

// try improve namespacing a bit
if ( !this.inConcert.Administration ) {
	this.inConcert.Administration = {};
}

if ( !this.inConcert.Administration.Campaigns ) {
	this.inConcert.Administration.Campaigns = {};
}

/**
 * @namespace
 * @name TwitterAccountPanel
 * @author Sebastián Moreno (smoreno@inconcertcc.com)
 */
(function () {

	'use strict';

	var format = inConcert.Utils.format;

	/**
	 * Creates a new Twitter account panel instance.
	 *
	 * @constructor
	 * @constructor
	 * @memberOf TwitterAccountPanel
	 */
	function TwitterAccountPanel(params){
		this.prefix_id = "adm_form_campaign_accounts_twitter_";
		TwitterAccountPanel.uber.constructor.call(this, params);

		this.messages_class = inConcert.adm_form_campaigns_twitter_account_messages; 	//TODO twitter no tiene mensajes automaticos ya que da problemas.
		this.container_id = '#adm_form_campaign_accounts_twitter_panel';
		this.authorize_id = '#adm_form_campaign_accounts_twitter_account_auth';
		this.authorize_img_id = '#adm_form_campaign_accounts_twitter_account_auth_img';

		this.language = '#adm_form_campaign_accounts_twitter_language';
		this.distribution = '#adm_form_campaign_accounts_twitter_distribution';
		this.priority = '#adm_form_campaign_accounts_twitter_priority';
		this.username = '#adm_form_campaign_accounts_twitter_display_name';

		this.attention_levels_container = '#adm_form_campaign_accounts_twitter_attention_levels_config';
		this.ticket_management_container = '#adm_form_campaign_accounts_twitter_tickets_mng_panel';
		this.contact_management_container = '#adm_form_campaign_accounts_twitter_contact_mng_panel';
		this.thresholds_container = '#adm_form_campaign_accounts_twitter_thresholds_panel';

		this.account_save_id = '#adm_form_campaign_accounts_save';

		this.tagline_new      = "<%=_T('New Twitter account.')%>";
		this.tagline_edit     = "<%=_T('Edit Twitter account {0}.')%>";
		this.tagline_readonly = "<%=_T('Twitter account {0}.')%>";
	}

	inConcert.Utils.inherit( TwitterAccountPanel, inConcert.adm_form_campaigns_section_accounts_base_panel );

	if ( typeof inConcert.Administration.Campaigns.TwitterAccountPanel !== 'function' ) {
		inConcert.Administration.Campaigns.TwitterAccountPanel = TwitterAccountPanel;
	}

	/**
	 * Called when the form is being initialized
	 *
	 * @public
	 */
	TwitterAccountPanel.prototype.initialize = function(){
		TwitterAccountPanel.uber.initialize.call(this);

		// TO-DO: Find a prettier way to do this.
		$(this.account_save_id).prop('disabled', true);

		var $account_name = $( this.username );
		this.parent.frm.readonly_control( $account_name, true /* disabled */ );

		//this.tabs.disable_tab( 'messages' );
	};

	TwitterAccountPanel.prototype.has_qna_maker       = function() {return true;};
	TwitterAccountPanel.prototype.has_flow_management = function() {return true;};
	TwitterAccountPanel.prototype.has_flow_inactivity = function() {return true;};

	/**
	 * Initializes thresholds management
	 *
	 * @public
	 */
	TwitterAccountPanel.prototype.initialize_thresholds = function()
	{
		// build thresholds panel
		TwitterAccountPanel.uber.initialize_thresholds.call( this,
			'TWITTER',
			{
				parent    : this.parent,
				prefix_id : 'adm_form_campaign_accounts_twitter_thresholds'
			}
		);
	};

	/**
	 * Initializes survey management
	 *
	 * @public
	 */
	 TwitterAccountPanel.prototype.initialize_survey_management = function()
	{
		TwitterAccountPanel.uber.initialize_survey_management.call(
			this,
			{
				type         : "TWITTER",
				parent       : this,
				container_id : "adm_form_campaign_accounts_twitter_survey_panel",
				prefix_id    : "adm_form_campaign_accounts_twitter_survey"
			}
		);
	};

	TwitterAccountPanel.prototype.initialize_qna_maker_mng = function()
	{
		// build qna_maker management panel
		TwitterAccountPanel.uber.initialize_qna_maker_mng.call(
			this,
			{
				type         : "TWITTER"                                                  ,
				parent       : this                                                    ,
				container_id : "adm_form_campaign_accounts_twitter_qna_maker_mng_panel",
				prefix_id    : "adm_form_campaign_accounts_twitter_qna_maker_mng"
			}
		);
	};

	/**
	 * Initializes contact management
	 *
	 * @public
	 */
	TwitterAccountPanel.prototype.initialize_contact_mng = function(){
		// build contact management panel
		TwitterAccountPanel.uber.initialize_contact_mng.call(this,
			{
				type         : 'TWITTER'                                              ,
				parent       : this                                                   ,
				auto_add_type: 'BOTH'                                                 , // incoming y outgoing
				container_id : 'adm_form_campaign_accounts_twitter_contact_mng_panel' ,
				prefix_id    : 'adm_form_campaign_accounts_twitter_contact_mng'
			}
		);
	};

	TwitterAccountPanel.prototype.initialize_flows_mng = function()
	{
		// build flows designer management panel
		TwitterAccountPanel.uber.initialize_flows_mng.call(
			this,
			{
				type         : 'TWITTER_BOT'                                                    ,
				parent       : this                                                         ,
				container_id : 'adm_form_campaign_accounts_twitter_flows_designer_mng_panel',
				prefix_id    : 'adm_form_campaign_accounts_twitter_flows_designer_mng'
			}
		);
	};

	/**
	 * Initializes ticket management
	 *
	 * @public
	 */
	TwitterAccountPanel.prototype.initialize_ticket_emitter = function(){
		// build ticket management section
		TwitterAccountPanel.uber.initialize_ticket_emitter.call(this,
			{
				type         : 'TWITTER'                                              ,
				parent       : this                                                   ,
				auto_add_type: 'BOTH'                                                 , // incoming y outgoing
				container_id : 'adm_form_campaign_accounts_twitter_tickets_mng_panel' ,
				prefix_id    : 'adm_form_campaign_accounts_twitter_tickets_mng'
			}
		);
	};

	/**
	 * Toggle every control, used from init and edit mode.
	 *
	 * @param  {Boolean} on_or_off - Toggle or not.
	 */
	TwitterAccountPanel.prototype.__toggle_controls = function(on_or_off)
	{
		// Los divs que contienen el attr 'row_toggle', se ocultan/muestran cuando la app es autorizada!!
		$(this.container_id).find("[row_toggle]").toggle(on_or_off);
		$(this.account_save_id).prop('disabled', !on_or_off);
		$(this.authorize_id).text(!on_or_off ? "<%=_T('Authorize Application')%>" : "<%=_T('Re-Authorize Application')%>");
		// $(this.authorize_img_id).toggleClass('formImageTrue', on_or_off).toggleClass('formImageFalse', !on_or_off);
	};

	/**
	 * Clears every control
	 *
	 * @todo This name is wrong, must be "clear controls" or something.
	 * @public
	 */
	TwitterAccountPanel.prototype.empty_controls = function() {
		TwitterAccountPanel.uber.empty_controls.call( this );
		this.__toggle_controls(false);
	};
	
	/**
	 * Called when the user enter to account edition
	 *
	 * @public
	 */
	TwitterAccountPanel.prototype.edit = function(account)
	{
		TwitterAccountPanel.uber.edit.call( this, account );
		if ( !this.current_account ) {
			return;
		}
		this.m_user_id = this.current_account.uid;
		this.m_screen_name = this.current_account.screen_name;
		this.m_oauth_verifier = this.current_account.oauth_verifier;
		this.m_token_key = this.current_account.tokenKey;
		this.m_token_secret = this.current_account.tokenSecret;
		this.m_consumerKey = this.current_account.twitterApp.consumerKey;
		this.m_consumerSecret = this.current_account.twitterApp.consumerSecret;
		$(this.username).val(this.current_account.name);

		this.__toggle_controls(true);
		this.tabs.enable_tab( 'messages' );
	};

	TwitterAccountPanel.prototype.get_account_name = function()
	{
		return this.current_account.name;
	};

	/**
	 * Method to initialize tabs (basic & messages)
	 *
	 * @public
	 */
	TwitterAccountPanel.prototype.initialize_tabs = function() {
		TwitterAccountPanel.uber.initialize_tabs.call(this,
			{
				parent       : this                              ,
				frm          : this.parent.frm                   ,
				container_id : 'adm_form_campaign_accounts_tabs' ,
				tabs_list    : {
					basic : {
						id               : "basic",
						name             : "<%=_T('Twitter Config')%>",
						tab_container_id : 'adm_form_campaign_accounts_twitter_panel_tab_basic',
						selected         : true
					},
					messages : {
						id               : "messages",
						name             : "<%=_T('Messages')%>",
						tab_container_id : "adm_form_campaign_accounts_twitter_panel_tab_messages" 
					}
				}
			}
		);
	};

	TwitterAccountPanel.prototype.initialize_skills = function()
	{
		TwitterAccountPanel.uber.initialize_skills.call(
			this,
			{
				container_id : 'adm_form_campaign_accounts_twitter_skills_config',
				parent_frm   : this.parent,
				parent_type  : 'FORM-SECTION'
			}
		);
	};

	TwitterAccountPanel.prototype.initialize_attention_levels = function()
	{
		TwitterAccountPanel.uber.initialize_attention_levels.call(
			this,
			{
				container_id : 'adm_form_campaign_accounts_twitter_attention_levels_config',
				parent_frm   : this.parent,
				parent_type  : 'FORM-SECTION',
				proxy_type   : 'TWITTER'
			}
		);
	};

	/**
	 * Validates Twitter account fields.
	 *
	 * @todo Validate and save should be separated actions
	 *
	 * @public
	 *
	 * @return {Object}  result
	 * @return {Boolean} result.success
	 * @return {Boolean} result.is_new
	 *
	 * @return {Object}  [result.account]
	 * @return {String}  [result.account.id]
	 * @return {String}  [result.account.proxyType]
	 * @return {String}  [result.account.name]
	 * @return {String}  [result.account.CCAlgorithm]
	 *
	 * @return {String}  [result.error_message]
	 */
	TwitterAccountPanel.prototype.validate = function()
	{
		var is_new = this.is_new();

		var result = {
			success: false,
			is_new: is_new
		};

		this.hide_marked_errors();

		var $distribution = $(this.distribution);
		if (!this.validate_field($distribution, 'required', "<%=_T('Distribution is required.')%>")) {
			return result;
		}

		// PRIORITY - validation
		var $priority = $(this.priority);
		var has_att_levels = this.attention_levels_handle.count_selected() > 0;
		if (!has_att_levels && $priority.val().trim().length > 0) {
			var range = { between: { from: 0, to: 99 } }; // range attention_level value validation
			if (!this.validate_field($priority, 'range', format("<%=_T('Account priority value out of range. Please enter account priority value between {0} and {1}.')%>", range.between.from, range.between.to ), range )) {
				return result;
			}
		}

		// LANGUAGE - validation
		var $language = $(this.language);
		if (!this.validate_field($language, "required", "<%=_T('Language is required.')%>")) {
			return result;
		}

		// valido los skills ----------
		var result_skills = this.validate_skills();
		if ( !result_skills.success ) {
			this.show_validation_error( result_skills.$ctrl, result_skills.message );
			return result;
		}

		// valido los attention levels ----------
		var result_attention_levels = this.validate_attention_levels();
		if ( !result_attention_levels.success ) {
			this.show_validation_error(
				result_attention_levels.$ctrl,
				result_attention_levels.message,
				false, //timeout_ms
				true, //scrolling
				result_attention_levels.show_custom_button
			);
			return result;
		}

		// validate ticket configuration
		var result_ticket_emitter = this.validate_ticket_emitter();
		if (!result_ticket_emitter.success) {
			this.show_validation_error( result_ticket_emitter.$ctrl, result_ticket_emitter.error_message );
			return result;
		}

		// valido los contact mng ----------
		var result_contact_mng = this.validate_contact_mng();
		if ( !result_contact_mng.success ) {
			this.show_validation_error( result_contact_mng.$ctrl, result_contact_mng.error_message );
			return result;
		}

		// valido los flows mng ----------
		var result_flows_mng = this.validate_flows_mng();
		if ( !result_flows_mng.success ) {
			this.show_validation_error( result_flows_mng.$ctrl, result_flows_mng.error_message );
			return result;
		}

		// valido los qna maker mng ----------
		var result_qna_maker_mng = this.validate_qna_maker_mng();
		if ( !result_qna_maker_mng.success ) {
			this.show_validation_error( result_qna_maker_mng.$ctrl, result_qna_maker_mng.error_message );
			return result;
		}

		// valido los thresholds ----------
		var result_th = this.validate_thresholds();
		if ( !result_th.success ) {
			this.show_validation_error( result_th.$ctrl, result_th.error_message );
			return result;
		}

		// valido los messages de la account ----------
		var result_messages = this.validate_messages();
		if ( !result_messages.success ) {
			this.show_validation_error( undefined, result_messages.error_message );
			return result;
		}

		// valido la configuracion de survey ----------
		var result_survey_management = this.validate_survey();
		if (!result_survey_management.success) {
			this.show_validation_error(result_survey_management.$ctrl, result_survey_management.error_message);
			return result;
		}


		// all good, now save it.
		// TO-DO: This should be separeted
		var account = {};
		if (is_new) {
			var campaign = this.parent.frm.get_entity_instance();
			// TO-DO: Why we need to create the account id client side?
			account.id = 'TWITTER_' + campaign.Id + '_' + this.m_user_id;
		}
		else {
			account.id = this.current_account.id;
		}

		account.proxyType          = 'TWITTER';
		account.name               = this.m_screen_name;
		account.uid                = this.m_user_id;
		account.screen_name        = this.m_screen_name;
		account.CCAlgorithm        = $distribution.val();
		account.language           = $language.val();
		account.TicketEmitter      = result_ticket_emitter.result;
		account.Skills             = this.skills_handle.get();
		account.Priority           = $priority.val();
		account.AttentionLevels    = this.attention_levels_handle.get();
		account.GUI                = this.get_gui_config();
		account.ActionOnAcdWaitTimeout = result_th.ActionOnAcdWaitTimeout;
		account.InactivityTimeout  = result_th.InactivityTimeout;
		account.Survey             = result_survey_management.result;
		account.ignoreAgentAssignedFromReOpen = $(this.ignoreAgent).prop("checked")? "1" : "0";
		this.contact_mng_instance.attach_data(account);
		if (this.flow_mng_instance) {
			this.flow_mng_instance.attach_data( account );
		}
		if (this.qna_maker_mng_instance) {
			this.qna_maker_mng_instance.attach_data(account);
		}

		// TO-DO: There is no validation of this data server side,
		// seems wrong to me.
		account.tokenKey = this.m_token_key;
		account.tokenSecret = this.m_token_secret;
		account.oauth_verifier = this.m_oauth_verifier;

		account.twitterApp = {
			consumerKey : this.m_consumerKey,
			consumerSecret : this.m_consumerSecret
		};

		result.account = account;
		result.success = true;

		return result;
	};

	/**
	 * Called when our Twitter app is authorized.
	 *
	 * @param  {Object} data
	 * @param  {String} data.oauth_token
	 * @param  {String} data.oauth_token_secret
	 * @param  {String} data.oauth_verifier
	 * @param  {String} data.user_id
	 * @param  {String} data.screen_name
	 */
	TwitterAccountPanel.prototype.__on_app_authorized = function(data, callback = $.noop) {
		this.m_token_key = data.oauth_token;
		this.m_token_secret = data.oauth_token_secret;
		this.m_oauth_verifier = data.oauth_verifier;
		this.m_screen_name = data.screen_name;
		this.m_user_id = data.user_id;
		this.m_consumerKey = data.consumerKey;
		this.m_consumerSecret = data.consumerSecret;
		$(this.username).val(data.screen_name);
		this.__toggle_controls(true);
		this.tabs.enable_tab( 'messages' );
		callback(true);
	};

	/**
	 * Called once the panel is initialized, we can attach buttons events here.
	 *
	 * @public
	 */
	TwitterAccountPanel.prototype.set_events = function()
	{
		TwitterAccountPanel.uber.set_events.call( this );

		let self = this;

		$(this.authorize_id).unbind('click').bind('click', function() {
			if ( !self.parent.has_permission_to_save() ) {
				let message = "<%=_T('You do not have permissions for save account.')%>";
				self.parent.frm.show_status_message( message, 'Error', 5000 );
				return;
			}
			let campaign = self.parent.frm.get_entity_instance();
			let postback_data = JSON.stringify({ 
				campaignId : campaign.Id, 
				origin: window.location.origin,
				host : window.location.host
			});
			let parameters = '?&post_data=' + encodeURIComponent(postback_data);

			let href = inConcert.app.getBaseDomain() + '/inconcert/apps/oauth/twitter/authorize_user/' + parameters;
			let window_options = "height=600,width=800,status=yes,toolbar=yes,menubar=no,location=yes";
			let authorization_window = window.open(href, "_blank", window_options);

			if (window.TwitterAccountPanel_listener) {
				window.removeEventListener("message", window.TwitterAccountPanel_listener);
				window.TwitterAccountPanel_listener = undefined;
			}

			let close_popup_window = function( can_add ) {
				let timeout_ms = can_add ? 1500 : undefined;
				window.setTimeout( function() {
					authorization_window.close();
					authorization_window = undefined;
				}, timeout_ms );
			};

			let listener = function(ev) {
				if (ev.origin.search( window.location.host ) > 0 && ev.data && ev.data.authorization === "TWITTER") {
					self.__on_app_authorized(ev.data, close_popup_window);
					window.removeEventListener("message", window.TwitterAccountPanel_listener);
					window.TwitterAccountPanel_listener = undefined;
				}
			};

			window.TwitterAccountPanel_listener = listener;
			window.addEventListener("message", window.TwitterAccountPanel_listener, false);
		});
	};

	TwitterAccountPanel.prototype.editable = function()
	{
		TwitterAccountPanel.uber.editable.call( this );
		$( this.authorize_id ).show();
	};

	TwitterAccountPanel.prototype.readonly = function()
	{
		TwitterAccountPanel.uber.readonly.call( this );
		$( this.authorize_id ).hide();
	};
	
	TwitterAccountPanel.prototype.on_saved = function(new_account, current_account, callback = $.noop)
	{
		TwitterAccountPanel.uber.on_saved.call(this, new_account, current_account, callback);

		var self = this;

		if ( new_account.Survey ) {
			var project_id = new_account.Survey.id;
			var token = new_account.Survey.Token;

			if ( !project_id ) { return; }

			var tweety = inConcert.app.getTweetyAPI();

			return tweety.set_authfilter( project_id, token,
				function( success )
				{
					if( !success ) {
						var error_message = "<%=_T('Error setting auth filter to survey.')%>";
						var timeout_ms = 3000;
						self.parent.frm.show_status_message( error_message, "Error", timeout_ms );
					}
				}
			);
		}
	};
}());
