/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 11 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

//	"use strict";

	var format = inConcert.Utils.format;

	//---------------------------------------------------------------------
	// attention levels usage handle class (accounts/campaign)
	//
	function attention_levels_handle( params )
	{
		this.container_id        = params.container_id;
		this.account_priority_id = params.account_priority_id;
		this.parent_frm          = params.parent_frm;
		this.parent_type         = params.parent_type;
		this.title               = params.title || "";
		this.help                = params.help || "";
		this.proxy_type          = params.proxy_type;

		this.list_id    = this.container_id + "_list";
		this.message_id = params.container_id + "_message_not_results_found";

		// estos retries son para el set de datos cuando se edita en el frm de accounts
		//  ya que el DOM de los attention levels quizas aun no se haya generado ( se genera en la respuesta del request )
		this.count_setting_retries = 0;
		this.max_setting_retries = 10;
		this.request_status = "NONE";

		return true;
	}

	if ( typeof inConcert.attention_levels_handle !== "function" ) {
		inConcert.attention_levels_handle = attention_levels_handle;
	}

	attention_levels_handle.prototype.initialize = function()
	{
		this.build_container_html();
		this.set_events();
		this.load_data();
	};

	attention_levels_handle.prototype.load_data = function()
	{
		var self = this;
		this.request_status = "RUNNING";
		var campaign = this.parent_frm.get_entity_instance();
		campaign.getAttentionLevels(function( attention_levels, err )
		{
			if ( err ) {
			//ERROR
				self.request_status = "ERROR";
				// Pequeno timeout para que se pueda visualizar el mensaje de error, ya que este request se tuvo generar desacoplado al handle de request
				// de los forms de las clases bases de "configuration"
				window.setTimeout(function() {
					var timeout_ms = false; //infinite
					self.parent_frm.show_status_message( "<%=_T('Error retrieving attention levels list.')%>", "Error", timeout_ms);
				}, 3000);
				return;
			}
			// SUCCESS
			self.build_att_levels_html( attention_levels );
			self.request_status = "COMPLETED";
		});
	};

	attention_levels_handle.prototype.build_container_html = function()
	{
		var help = this.help.length > 0 ? (this.help) : "";

		$("#" + this.container_id).html(
			"<div class='row'>" +
				"<div class='col-12'>" +
					"<div class='card-header'>" +
						"<h5 class='mb-0'>" + this.title + "</h5>" +
						"<p>" + help + "</p>" +
					"</div>" +
				"</div>" +
			"</div>" +
			"<div class='row'>" +
				"<div class='col-12'>" +
					"<ul id='" + this.list_id + "' class='formList formListAsGrid formListWithCheckbox mt-0'></ul>" +
					"<div id='" + this.message_id + "' class='inlineMessagesContainer messageInfo mt-0' hidden></div>" +
				"</div>" +
			"</div>"
		);
	};

	attention_levels_handle.prototype.get_opt_id = function( attention_level_id )
	{
		var normalized_id = inConcert.Utils.getNormalizedId( attention_level_id );
		var opt_id = this.list_id + "_opt_att_level_" + normalized_id;
		return opt_id;
	};

	attention_levels_handle.prototype.get_opt_distribution_id = function( attention_level_id )
	{
		return "distribution_" + this.get_opt_id( attention_level_id );
	};

	attention_levels_handle.prototype.get_opt_priority_id = function( attention_level_id )
	{
		return "priority_" + this.get_opt_id( attention_level_id );
	};

	attention_levels_handle.prototype.get_opt_priority_date_id = function( attention_level_id )
	{
		return "priority_date_" + this.get_opt_id( attention_level_id );
	};

	attention_levels_handle.prototype.fill_combos = function()
	{
		var self = this;
		this.combo_priority_date = [
			{ name: "<%=_T('Transference')%>"    , value : "transfer"},
			{ name: "<%=_T('Creation')%>" , value : "creation" }
		];
		$("#" + this.list_id).find("select[priority_date='true']").each(function(_, cbo) {
			self.parent_frm.fill_combo( $(cbo), false, self.combo_priority_date, "value", "name", false, undefined, undefined, undefined, "transfer");
		});
	};

	attention_levels_handle.prototype.build_att_levels_html = function( attention_levels )
	{
		var self = this;

		if (!$.isPlainObject(attention_levels)) {
			this.hide_not_attention_levels_found();
			return;
		}

		var sorted_attention_levels = inConcert.Utils.hash_to_sorted_array(attention_levels,
			function(attention_level) {
				return ( attention_level.name || "" ).toLowerCase();
			}) || {};

		var html_attention_levels = "";
		var has_attention_levels = sorted_attention_levels.length > 0;

		if ( has_attention_levels ) {
			this.hide_not_attention_levels_found();

			// header
			html_attention_levels = html_attention_levels +
				"<li class='formListTitle'>" +
					"<div class='firstItemCheckbox'></div>" +
					"<label class='formListTitleText nameColumn'>"  + "<%=_T('Name')%>"             + "</label>" +
					"<label class='formListTitleText valueColumn'>" + "<%=_T('Description')%>"      + "</label>" +
					"<label class='formListTitleText valueColumn formItemInput100 checkboxImageCol'>" + "<%=_T('Distribution')%>"     + "</label>" +
					"<label class='formListTitleText valueColumn formItemInput150'>" + "<%=_T('Priority (0-99)')%>" + "</label>" +
					"<label class='formListTitleText valueColumn formItemInput150'>" + "<%=_T('Priority date')%>"      + "</label>" +
				"</li>";

			var get_ctrls = function( attention_level )
			{
				var html_actions = "";

				// radio button (default)
				var distribution_id = self.get_opt_distribution_id( attention_level.id );
				var radio_group = self.proxy_type + "_att_level_default_distribution";
				html_actions = html_actions + 
					"<div class='formItemText formItemInput100 checkboxImageCol'>" +
						"<div class='form-check form-check-only'>" +
							"<input class='formListRadio' id='" + distribution_id + "' type='radio' name='" + radio_group + "' disabled='true' />" +
						"</div>" +
					"</div>";


				// txt priorization
				var priority_id = self.get_opt_priority_id( attention_level.id );
				html_actions = html_actions +
					"<div class='formItemInput150'>" +
						"<input id='" + priority_id + "' class='formListField form-control' type='text' />"+
					"</div>";
				
				// priority_date
				const attention_level_priority_date = attention_level.priority_date || "transfer";
				const priority_date_id = self.get_opt_priority_date_id( attention_level.id );
				html_actions = html_actions +
					"<div class='formItemInput150'>" +
						"<select id='" + priority_date_id + "' class='form-control' priority_date='true' disabled='true' default_value='" + attention_level_priority_date + "'></select>"+
					"</div>";

				return html_actions;
			};

			// lista de attention levels
			$.each(sorted_attention_levels, function( _, attention_level )
			{
				var attention_level_id = self.get_opt_id( attention_level.id );
				var attention_level_name = attention_level.name || attention_level.id;
				var attention_level_desc = attention_level.description || "";

				html_attention_levels = html_attention_levels +
					"<li class='formListItem' attention_level_id='" + attention_level.id + "'>" +
						"<input id='" + attention_level_id + "' attention_level_id='" + attention_level.Id + "' attention_level_name='" + attention_level_name + "' class='formListCheck' selector='1' type='checkbox' />" +
						"<div class='formItemText'>" +
							"<label for='" + attention_level_id + "'>" + attention_level_name + "</label>" +
						"</div>" +
						"<div class='formItemText'>" +
							"<label for='" + attention_level_id + "'>" + attention_level_desc + "</label>" +
						"</div>" +
						get_ctrls( attention_level ) +
						// "<div attention_level_value_container='true' class='formItemAttentionLevelValue' hidden>" + get_ctrls( attention_level ) + "</div>" +
					"</li>";
			});
		}
		else {
			this.show_not_attention_levels_found();
		}

		$("#" + this.list_id).html( html_attention_levels );

		if ( has_attention_levels) {
			this.fill_combos();
		}
	};

	attention_levels_handle.prototype.show_container = function()
	{
		$("#" + this.container_id).show();
	};

	attention_levels_handle.prototype.hide_container = function()
	{
		$("#" + this.container_id).hide();
	};

	attention_levels_handle.prototype.show_not_attention_levels_found = function( message )
	{
		$("#" + this.message_id)
			.text( message || "<%=_T('No attention levels found.')%>" )
			.show();
	};

	attention_levels_handle.prototype.hide_not_attention_levels_found = function()
	{
		$("#" + this.message_id).hide();
	};

	attention_levels_handle.prototype.editable = function()
	{
		// editable chks de la lista
		$("#" + this.list_id + " input[type='checkbox']")
			.prop("disabled", false)
			.removeClass("readonly");

		// editable los divs contenedores de textos en la lista
		$("#" + this.list_id + " .formItemText")
			.removeClass("readonly");

		// editable los inputs con los valores de los attention levels en la lista
		$("#" + this.list_id + " .formListField.form-control")
			.prop("disabled", false)
			.removeClass("readonly");
	};

	attention_levels_handle.prototype.readonly = function()
	{
		var self = this;
		if ( this.request_status === "NONE" || this.request_status === "RUNNING" ) {
		//  Si el DOM de los attention levels quizas aun no se haya generado ( se genera en la respuesta del request )
			window.setTimeout(function() {
				self.readonly();
			}, 500);
			return;
		}
		else if ( this.request_status === "COMPLETED" ) {
			// readonly chks de la lista
			$("#" + this.list_id + " input[type='checkbox']")
				.prop("disabled", true)
				.addClass("readonly");

			// readonly los divs contenedores de textos en la lista
			$("#" + this.list_id + " .formItemText")
				.addClass("readonly");

			// readonly los inputs con los valores de los attention levels en la lista
			$("#" + this.list_id + " .formListField.form-control")
				.prop("disabled", true)
				.addClass("readonly");

		}
		// El otro estado es "ERROR" pero no hace nada con el, ya que el mensaje de error ya ha sido visualizado en la respuesta del request.
	};

	attention_levels_handle.prototype.set_events = function()
	{
		var self = this;

		var event_selector = "input[selector='1'][type='checkbox']";
		$("#" + this.list_id)
			.off("click", event_selector)
			.on("click", event_selector, function()
			{
				var $chk = $(this);
				self.on_mark_unmark_att_level( $chk );
			});

		event_selector = "input[close_popup='true']";
		var $popup = this.parent_frm.$get_dom_element_by_id( "view_popup" );
		$popup
			.off("click", event_selector)
			.on("click", event_selector, function()
			{
				self.hide_conflicts();
			});
	};

	attention_levels_handle.prototype.get = function()
	{
		var self = this;
		var assigned_attention_levels = {};

		$("#" + this.list_id + " input[selector='1'][type='checkbox']").each(function( _, attention_level )
		{
			var $chk = $(attention_level);
			var checked = $chk.prop("checked");
			if ( checked ) {
				var attention_level_id = $chk.attr("attention_level_id");
				var $distribution = $("#" + self.get_opt_distribution_id( attention_level_id ));
				var $priority = $("#" + self.get_opt_priority_id( attention_level_id ));
				var $priority_date = $("#" + self.get_opt_priority_date_id( attention_level_id ));
				var assigned_attention_level = {
					attention_level_id : attention_level_id,
					distribution       : $distribution.prop("checked"),
					priority           : $priority.val(),
					priority_date	   : $priority_date.val()
				};
				assigned_attention_levels[attention_level_id] = assigned_attention_level;
			}
		});

		return assigned_attention_levels;
	};

	attention_levels_handle.prototype.set = function( assigned_attention_levels )
	{
		var self = this;
		if ( !assigned_attention_levels ) {
			return;
		}
		if ( this.request_status === "NONE" || this.request_status === "RUNNING" ) {
		// estos retries son para el set de datos cuando se edita en el frm de accounts
		//  ya que el DOM de los attention levels quizas aun no se haya generado ( se genera en la respuesta del request )
			if ( this.count_setting_retries <= this.max_setting_retries ) {
				this.count_setting_retries++;
				window.setTimeout(function() {
					self.set( assigned_attention_levels );
				}, 500);
			}
			return;
		}
		else if ( this.request_status === "COMPLETED" ) {

			var checked = true;
			$.each(assigned_attention_levels, function( _, data )
			{
				// chk selected
				var chk_id = self.get_opt_id(data.attention_level_id);
				var $chk = $("#" + chk_id);
				$chk.prop("checked", checked);
				// muestra los controles de input
				self.on_mark_unmark_att_level( $chk );
				// el radio button distribution
				if ( data.distribution ) {
					var distribution_id = self.get_opt_distribution_id(data.attention_level_id);
					var $distribution = $("#" + distribution_id);
					$distribution.prop("checked", true);
				}
				// el textbox priority
				var priority_id = self.get_opt_priority_id(data.attention_level_id);
				var $priority = $("#" + priority_id);
				$priority.val(data.priority);

				//priority date
				const priority_date_id = self.get_opt_priority_date_id(data.attention_level_id);
				const $priority_date = $("#" + priority_date_id);
				$priority_date.val(data.priority_date || "transfer");
			});
		}
		// El otro estado es "ERROR" pero no hace nada con el, ya que el mensaje de error ya ha sido visualizado en la respuesta del request.
	};

	attention_levels_handle.prototype.show_or_hide_attention_level_value = function( $chk )
	{
		if ( !$chk || $chk.length === 0 ) {
			return;
		}
		var checked = $chk.prop("checked");
		// var $value_container = $chk.parent().find("[attention_level_value_container]");
		// if ( checked ) {
		// 	$value_container.show();
		// }
		// else {
		// 	$value_container.hide();
		// }
		var att_level_id = $chk.attr("attention_level_id");
		var disabled = this.parent_frm.is_readonly() ? true : !checked;
		// distribution
		var $distribution = $("#" + this.get_opt_distribution_id(att_level_id));
		$distribution.prop("disabled", disabled);
		// priority
		var $proritiy = $("#" + this.get_opt_priority_id(att_level_id));
		$proritiy.prop("disabled", disabled);

		// priority date
		var $proritiy_date = $("#" + this.get_opt_priority_date_id(att_level_id));
		$proritiy_date.prop("disabled", disabled);

	};

	attention_levels_handle.prototype.count_selected = function()
	{
		return $("#" + this.list_id + " input[selector='1'][type='checkbox']:checked").length;
	};

	attention_levels_handle.prototype.on_mark_unmark_att_level = function( $chk )
	{
		this.show_or_hide_attention_level_value( $chk );
		// TODO: ver modo readonly
		var disabled = this.count_selected() > 0;
		var $account_priority = $(this.account_priority_id);
		var account_priority_val = $account_priority.val();
		var keep_value = true;
		this.parent_frm.disable_control($account_priority, disabled, undefined, undefined, keep_value);
		if ( $chk.prop("checked" ) ) {
			// si ha marcado un AL debe de tomar el valor por defecto la prioridad de la cuenta.
			var attention_level_id = $chk.attr("attention_level_id");
			var $priority = $("#" + this.get_opt_priority_id( attention_level_id ));
			if ( $priority.val().trim().length === 0 ) {
				$priority.val(account_priority_val);
			}
		}
	};

	attention_levels_handle.prototype.show_conflicts = function( conflicts )
	{
		var $popup = this.parent_frm.$get_dom_element_by_id( "view_popup" );

		$popup.html(
			"<div class='internalPopUpContainer attentionLevelConflict'>" +
				"<div class='internalPopUpOverlay'></div>" +
				"<div class='internalPopUp'>" +
					"<div class='internalPopUpTitle'>" + conflicts.title + "</div>" +
					"<div class='internalPopUpMiddle'>" +

						"<div id='adm_form_campaign_assigned_att_levels_popup_waiting' class='popupWaitingContainer' hidden>" +
							"<div waiting_message='true' class='popupWaitingMessage'></div>" +
							"<div class='popupWaitingAnimation'></div>" +
							"<div class='popupWaitingOverlay'></div>" +
						"</div>" +

						"<div class='formListAuxContainer'>" +
							"<ul id='adm_form_campaign_assigned_att_levels_popup_list' class='formList'>" +
								"<li class='formListTitle'>" +
									"<label head='true' class='formTitleText'>" + "<%=_T('User')%>" + "</label>" +
									"<label head='true' class='formTitleText'>" + "<%=_T('Attention levels')%>" + "</label>" +
								"</li>" +
							"</ul>" +
						"</div>" +
					"</div>" +

					"<div class='internalPopUpButtons internal'>" +
						"<input close_popup='true' class='pageButtonGeneric buttonColorGray' type='button' value='" + "<%=_T('Close')%>" + "' />" +
					"</div>" +
				"</div>" +
			"</div>"
		);

		$popup.show();

		var $list = $("#adm_form_campaign_assigned_att_levels_popup_list");

		var add_user = function( user )
		{
			var att_levels = "";
			$.each(user.att_levels_by_user, function( _, att_level )
			{
				if ( att_levels.length > 0 ) {
					att_levels = att_levels + ", ";
				}
				att_levels = att_levels + att_level.name;
			});
			var $user = $(
				"<li class='formListItem'>" +
					"<div class='formItemText'>" +
						"<label user_id='true'>" + user.user_id + "</label>" +
					"</div>" +
					"<div class='formItemText'>" +
						"<label>" + att_levels + "</label>" +
					"</div>" +
				"</li>");

			$list.append($user);
		};

		var $waiting = $("#adm_form_campaign_assigned_att_levels_popup_waiting");
		var sorted_users = inConcert.Utils.hash_to_sorted_array(conflicts.users,
			function(user) {
				return user.user_id.toLowerCase();
			}) || {};

		inConcert.Utils.foreach_timeout({
			rows : sorted_users,
			waiting_message : "<%=_T('Please wait, adding user, {0} of {1}.')%>",
			append_fn : function( user ) {
				add_user( user );
			},
			waiting_fn : function( message ) {
				$waiting.find("[waiting_message]").text(message || "");
				$waiting.show();
			},
			complete_fn : function() {
				$waiting.hide();
			}
		});
	};

	attention_levels_handle.prototype.hide_conflicts = function()
	{
		var $popup = this.parent_frm.$get_dom_element_by_id( "view_popup" );
		$popup.hide();
	};

	attention_levels_handle.prototype.validate = function()
	{
		var self = this;
		var response = { success: true };

		if ( this.count_selected() === 0 ) {
			return response;
		}

		var validate_field = function( $ctrl, type, msg_err, options ) {
			// required validacion
			var value = $ctrl.val();
			var result = self.parent_frm.valid( value, type, options );
			if ( !result.success ) {
				result.$ctrl = $("#" + self.container_id);
				result.message = msg_err;
				$ctrl.focus();
			}
			return result;
		};
		var range = { between: { from: 0, to: 99} }; // range attention_level value validation

		var selected_att_levels = [];
		var selected_distribution = false;
		$("#" + this.list_id + " input[selector='1'][type='checkbox']:checked").each(function( _, attention_level )
		{
			var attention_level_id = $(attention_level).attr("attention_level_id");
			var att_level_name = $(attention_level).attr("attention_level_name");

			selected_att_levels.push( { id: attention_level_id, name: att_level_name } );

		// 1- distribution validacion
			if (!selected_distribution) {
				selected_distribution = $("#" + self.get_opt_distribution_id( attention_level_id ) ).prop("checked");
			}
		// 2- priority validacion
			var $priority = $("#" + self.get_opt_priority_id( attention_level_id ) );

			if ( $priority.val().trim().length > 0 ) {
				// numeric validacion
				response = validate_field( $priority, "digits", format("<%=_T('{0} attention level value is not a number.')%>", att_level_name) );
				if ( !response.success ) {
					return false; // break
				}
				// range validacion
				response = validate_field( $priority, "range", format("<%=_T('{0} attention level value out of range. Please enter attention level value between {1} and {2}.')%>", att_level_name, range.between.from, range.between.to ), range );
				if ( !response.success ) {
					return false; // break
				}
			}
		});

		if ( !selected_distribution && response.success ) {
		// distribution error
			response.$ctrl = $("#" + this.container_id);
			response.success = false;
			response.message = "<%=_T('Attention levels distribution is required.')%>";
		}

		return response;
	};
}());
