/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 11 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

// try improve namespacing a bit
if ( !this.inConcert.Administration ) {
	this.inConcert.Administration = {};
}

if ( !this.inConcert.Administration.Campaigns ) {
	this.inConcert.Administration.Campaigns = {};
}

(function () {

	"use strict";

	var mark_chk       = function( ctrl, checked ) { $( ctrl ).prop( "checked", checked ); };
	var is_enabled_chk = function( option ) { return !option || option === "1"; };

	var get_chk_bool_to_string  = function( chk_id ) {
		var $chk = $( chk_id );
		if ( $chk.length === 0 ) {
			return undefined;
		}
		var checked = $chk.prop("checked");
		return Number( checked ).toString();
	};

	//---------------------------------------------------------------------
	// contact management
	function contact_management( params )
	{
		params = params || {};
		/* jshint validthis: true*/
		this.parent = params.parent;
		this.section = this.parent.parent;

		var prefix_id = params.prefix_id; // ejemplo de prefijo: adm_form_campaign_accounts_contact_form, etc

		this.current_contact_mng = null;

		// container
		this.container_id                         = "#" + params.container_id                                ;

		// auto
		this.contact_mng                          = "#" + prefix_id + "_contact_mng"                         ;

		// auto add
		//this.auto_add_contact_lbl                 = "#" + prefix_id + "_auto_add_contact_lbl"                ;
		this.auto_add_inc_contact                 = "#" + prefix_id + "_auto_add_incoming_contact"           ;
		this.auto_add_out_contact                 = "#" + prefix_id + "_auto_add_outgoing_contact"           ;

		// merge
		this.merge                                = "#" + prefix_id + "_merge"                               ;
		this.merge_options                        = "#" + prefix_id + "_merge_options"                       ;

		this.auto_add_type                        = params.auto_add_type;

		this.list_merge_criteria = [
			{ id: "KEEP_OLD"            , name: "<%=_T('Keep Old')%>"                            } ,
			{ id: "KEEP_NEW"            , name: "<%=_T('Keep New')%>"                            }
		];

		return true;
	}

	if (typeof inConcert.Administration.Campaigns.ContactManagement !== "function") {
		inConcert.Administration.Campaigns.ContactManagement = contact_management;
	}

	contact_management.prototype.initialize = function()
	{
		this.current_contact_mng = null;
		this.make_html();
		this.fill_data();
		this.set_events();
		this.set_default_values();
	};

	contact_management.prototype.fill_data = function( )
	{
		// cargo el combo merge options
		this.section.frm.fill_combo(
			$( this.merge_options ),    /* cbo merge options       */
			true,                       /* add blank first element */
			this.list_merge_criteria,   /* list                    */
			"id",                       /* attr id                 */
			"name"                      /* attr name               */
		);
	};

	contact_management.prototype.validate = function()
	{
		var result = { success: false };

		if ( $( this.contact_mng ).prop( "checked" ) ) {
			if ( $( this.merge ).prop( "checked" ) ) {
				var $merge_options = $( this.merge_options );
				var error_message = "<%=_T('Contact merge criteria option is required.')%>";
				if ( !this.parent.validate_field( $merge_options, "required", error_message ) ) {
					result.error_message = error_message;
					result.$ctrl = $merge_options;
					return result;
				}
			}
		}

		result.success = true;
		return result;
	};

	contact_management.prototype.get = function( keep_old_data )
	{
		var enabled_contact_mng = $( this.contact_mng ).prop( "checked" );

		if ( keep_old_data && !enabled_contact_mng ) {
		// esto es un lenga lenga raro, que no me gusta pero lo hago igual!!! Es para que mantenga la configuracion en el server aunque, el flag de Enabled = 0
			if ( this.current_contact_mng ) {
				this.current_contact_mng.Enabled = get_chk_bool_to_string( this.contact_mng );
				return this.current_contact_mng;
			}
		}

		return {
			Enabled                : get_chk_bool_to_string( this.contact_mng          ),
			AutoAddIncomingContact : get_chk_bool_to_string( this.auto_add_inc_contact ),
			AutoAddOutgoingContact : get_chk_bool_to_string( this.auto_add_out_contact ),
			MergeCriteria          : $( this.merge_options ).val()
		};
	};

	contact_management.prototype.attach_data = function( acc )
	{
		if ( !acc ) {
			return;
		}
		acc.ContactManagement = this.get( true /* keep_old_data */ );
		return acc;
	};

	contact_management.prototype.update_current_contact_mng = function()
	{
		this.current_contact_mng = this.get();
	};

	contact_management.prototype.set_ctrls_values = function( contact_mng )
	{
		if ( !contact_mng ) {
			return;
		}
		// auto add
		mark_chk( this.auto_add_inc_contact , ( is_enabled_chk( contact_mng.AutoAddIncomingContact ) ) );
		mark_chk( this.auto_add_out_contact , ( is_enabled_chk( contact_mng.AutoAddOutgoingContact ) ) );

		// merge
		var merge_criteria = contact_mng.MergeCriteria || "";
		$( this.merge_options ).val( merge_criteria );
		var has_merge = merge_criteria.length > 0;
		this.disable_merge_opts( !has_merge /* !merge = disabled */ );

		$( this.merge ).prop( "checked", has_merge );
	};

	contact_management.prototype.has_configured_contact_mng = function( acc )
	{
		if ( !acc ) {
			return false;
		}
		if ( acc.ContactManagement && acc.ContactManagement.Enabled === "1" ) {
			return true;
		}
		return false;
	};

	contact_management.prototype.set = function( acc )
	{
		if ( !acc ) {
			return;
		}

		this.current_contact_mng = acc.ContactManagement;

		var $contact_mng = $( this.contact_mng );
		if ( !this.has_configured_contact_mng( acc ) ) {
			$contact_mng.prop( "checked", false );
			this.disable_contact_mng( true );
			return false;
		}

		$contact_mng.prop( "checked", true );
		this.disable_contact_mng( false );
		this.set_ctrls_values( this.current_contact_mng );
	};

	contact_management.prototype.disable_contact_mng = function( disabled )
	{
		var self = this;

		var is_readonly = this.section.is_readonly();
		disabled = is_readonly ? true : disabled;

		var ds_ctrl = function( $ctrl, required ) {
			if ( is_readonly ) {
				self.section.frm.readonly_control( $ctrl, disabled, required );
			}
			else {
				self.section.frm.disable_control( $ctrl, disabled, required );
			}
		};

		// var ds_lbl = function( $ctrl, required ) {
		// 	if ( is_readonly ) {
		// 		self.section.frm.readonly_label( $ctrl, disabled, required );
		// 	}
		// 	else {
		// 		self.section.frm.disable_label( $ctrl, disabled, required );
		// 	}
		// };

		var $inc = $( this.auto_add_inc_contact );
		var $out = $( this.auto_add_out_contact );
		var $merge = $( this.merge );

		ds_ctrl( $inc );
		ds_ctrl( $out );
		//ds_lbl( $( this.auto_add_contact_lbl ) );

		if ( disabled ) {
			$inc.prop( "checked", false );
			$out.prop( "checked", false );
			$merge.prop( "checked", false );
			this.disable_merge_opts( disabled );
		}
		ds_ctrl( $merge );

		if ( is_readonly ) {
			ds_ctrl( $( this.contact_mng ) );
		}
	};

	contact_management.prototype.disable_merge_opts = function( disabled )
	{
		var self = this;

		disabled = this.section.is_readonly() ? true : disabled;

		var ds_ctrl = function( $cbo, required ) {
			if ( self.section.is_readonly() ) {
				self.section.frm.readonly_control( $cbo, disabled, required );
			}
			else {
				self.section.frm.disable_control( $cbo, disabled, required );
			}
		};

		var $cbo = $( this.merge_options );
		ds_ctrl( $cbo, true /* required */ );

		if ( disabled ) {
			if ( !this.section.is_readonly() ) {
				$cbo.val( "" );
			}
			this.parent.hide_marked_errors();
		}
	};

	contact_management.prototype.set_default_values = function()
	{
		mark_chk( this.auto_add_inc_contact , false );
		mark_chk( this.auto_add_out_contact , false );
		$( this.merge_options ).val( "" );
		$( this.contact_mng ).prop( "checked", true );
		$( this.contact_mng ).trigger( "click" );
	};

	contact_management.prototype.editable = function()
	{
		var self = this;

		var ds_ctrl = function( $ctrl, ds, required ) {
			self.section.frm.disable_control( $ctrl, ds, required );
		};

		// var ds_lbl = function( $ctrl, ds, required ) {
		// 	self.section.frm.disable_label( $ctrl, ds, required );
		// };

		var $contact_mng = $( this.contact_mng );
		ds_ctrl( $contact_mng, false );

		var checked = $contact_mng.prop( "checked" );
		var disabled = !checked;

		//ds_lbl( $( this.auto_add_contact_lbl ), disabled );
		ds_ctrl( $( this.auto_add_inc_contact ), disabled );
		ds_ctrl( $( this.auto_add_out_contact ), disabled );

		var $merge = $( this.merge );
		ds_ctrl( $merge, disabled );
		checked = $merge.prop( "checked" );
		disabled = !checked;

		this.disable_merge_opts( disabled );
	};

	contact_management.prototype.readonly = function()
	{
		var self = this;

		var ds_ctrl = function( $ctrl, ds, required ) {
			self.section.frm.readonly_control( $ctrl, ds, required );
		};

		// var ds_lbl = function( $ctrl, ds, required ) {
		// 	self.section.frm.readonly_label( $ctrl, ds, required );
		// };

		ds_ctrl( $( this.contact_mng ), true /* disabled */ );
		//ds_lbl( $( this.auto_add_contact_lbl ), true /* disabled */ );
		ds_ctrl( $( this.auto_add_inc_contact ), true /* disabled */ );
		ds_ctrl( $( this.auto_add_out_contact ), true /* disabled */ );
		this.disable_merge_opts( true /* disabled */ );
		ds_ctrl( $( this.merge ), true /* disabled */ );
	};

	contact_management.prototype.sanitize_id = function( id )
	{
		// remueve el # de jquery
		return id.substring( 1 );
	};

	contact_management.prototype.make_html = function()
	{
		var self = this;
		var s_id = this.sanitize_id;
		//var auto_add_contact_lbl = s_id( this.auto_add_contact_lbl );
		var auto_add_inc_contact = s_id( this.auto_add_inc_contact );
		var auto_add_out_contact = s_id( this.auto_add_out_contact );

		var get_html_auto_add = function()
		{
			
			var html_incoming = "".concat(
					"<div class='form-group d-flex flex-row mr-2 mr-xl-3'>",
						"<div class='form-check'>",
							"<label for='", auto_add_inc_contact, "' class='form-check-label form-check-label-title'>",
								"<input id='", auto_add_inc_contact, "' class='form-check-input' type='checkbox' />",
								"<%=_T('Add automatically Incoming Contact')%>", 
							"</label>",
						"</div>",
					"</div>",
			);

			var html_outgoing = "".concat(
					"<div class='form-group d-flex flex-row'>",
						"<div class='form-check'>",
							"<label for='", auto_add_out_contact, "' class='form-check-label form-check-label-title'>",
								"<input id='", auto_add_out_contact, "' class='form-check-input' type='checkbox' />",
								"<%=_T('Add automatically Outgoing Contact')%>", 
							"</label>",
						"</div>",
					"</div>",
			);

			switch( self.auto_add_type ) {
				case "BOTH":
					return html_incoming + html_outgoing;

				case "ONLY_INCOMING":
					return html_incoming;

				default: // ONLY_OUTGOING
					return html_outgoing;
			}
		};

		var merge            = s_id( this.merge            );
		var merge_options    = s_id( this.merge_options    );
		var contact_mng      = s_id( this.contact_mng      );


		$( this.container_id ).html( "".concat(
		// titulo
			// "<h2 class='formSubTitle'>",
			// 	"<label class='subTitleText'>", "<%=_T('Contact Management')%>", "</label>",
			// 	"<label class='subTitleHelp'>/", "<%=_T('Allows you to save & manage contacts in our system.')%>", "</label>",
			// "</h2>",

		// Contact Management ( Enabled / Disabled )
			"<div class='row'>",
				"<div class='col-12'>",
					"<div class='form-group'>",
						"<div class='form-check'>",
							"<label for='", contact_mng, "' class='form-check-label form-check-label-title'>",
								"<input id='", contact_mng, "' class='form-check-input' type='checkbox' />",
								"<strong>", "<%=_T('Enable Contact Management inside system')%>", "</strong>", 
							"</label>",
						"</div>",
					"</div>",
				"</div>",
			"</div>",

		// generacion automatica (inc / out)
			"<div class='row'>",
				"<div class='col-12 d-flex flex-row'>",
					// "<label id='", auto_add_contact_lbl, "'>", "<%=_T('Auto. Add Contact')%>", "<%=_T(':')%>", "</label>",
					get_html_auto_add(),
				"</div>",
			"</div>",

		// criterios de merge
			"<div class='row'>",
				"<div class='col-6 d-flex flex-row'>",
					"<div class='form-group'>",
						"<div class='form-check form-check-right-input '>",
							"<label for='", merge, "' class='form-check-label form-check-label-title'>",
								"<input id='", merge, "' class='form-check-input' type='checkbox' />",
								"<%=_T('Enable automatic merge')%>", 
							"</label>",
						"</div>",
					"</div>",

					"<div class='form-group flex-grow-1 position-relative'>",
						"<label for='", merge_options, "' class='required'>", "<%=_T('Merge Criteria')%>", "<%=_T(':')%>", "</label>",
						"<select id='", merge_options, "' class='form-control'></select>",
						"<i id='", merge_options, "_validation' class='formFieldValidation formFieldError' hidden ></i>",
					"</div>",
				"</div>",
			"</div>",

		// disclaimer
			// "<div style='display:none' class='tabDisclaimer formFieldsetMessagesContainer messageInfo'>", "<%=_T('DISCLAIMER MESSAGE !!!.')%>", "</div>"
		));

	};

	contact_management.prototype.set_events = function()
	{
		var self = this;

		if ( this.section.is_readonly() ) {
			return;
		}

		$( this.contact_mng ).off( "click" )
			.on( "click", function() {
				var disabled = !$( this ).prop( "checked" );
				if ( !disabled ) {
					if ( self.current_contact_mng ) {
						self.set_ctrls_values( self.current_contact_mng );
					}
				}
				else {
				// actualizo la configuracion del current contact mng cuando estoy editando una account existente.
					self.update_current_contact_mng();
				}

				self.disable_contact_mng( disabled );
			});

		$( this.merge ).off( "click" )
			.on( "click", function() {
				var disabled = !$( this ).prop( "checked" );
				self.disable_merge_opts( disabled );
			});
	};

}());
