/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 11 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

// try improve namespacing a bit
if ( !this.inConcert.Administration ) {
	this.inConcert.Administration = {};
}

if ( !this.inConcert.Administration.Campaigns ) {
	this.inConcert.Administration.Campaigns = {};
}

(function () {
	"use strict";
	const FORMAT = inConcert.Utils.format;

	//---------------------------------------------------------------------
	// flows management
	function flow_management( params )
	{
		params = params || {};
		/* jshint validthis: true*/
		this.parent = params.parent;
		this.section = this.parent.parent;
		this.proxy_type = params.type;
		let prefix_id = params.prefix_id; // ejemplo de prefijo: adm_form_campaign_accounts_flows_form, etc

		this.current_flow_info = null;
		// container
		this.container_id  = "#" + params.container_id;
		// process
		this.flows_process = "#" + prefix_id + "_flows_process";
		// inactivity
		this.inactivity = params.inactivity;
		this.inactivity_section = "#" + prefix_id + "_flows_inactivity_section";
		this.inactivity_unit     = "#" + prefix_id + "_flows_inactivity_unit";
		this.inactivity_time     = "#" + prefix_id + "_flows_inactivity_time";

		this.types = [
			{
				id : "on_schedule",
				name : "<%=_T('Incoming')%>",
				description : "<%=_T('Incoming on schedule flow.')%>"
			},
			{
				id : "out_of_schedule",
				name : "<%=_T('Out of Schedule')%>",
				description : "<%=_T('Incoming out of schedule flow.')%>"
			},
			// {
			// 	id : "dialer",
			// 	name : "<%=_T('Dialer')%>",
			// 	description : "<%=_T('Outbound dialing flows.')%>",
			// 	types: ["CALL"]
			// }
		].filter((flowType) => !flowType.types || flowType.types.includes(this.proxy_type));

		this.FlowsByType = [];
		this.BotsToTransferFromAgent = [];
		this.process_to_transfer_during_interaction_popup = new inConcert.PopupWindow({
			ownerId   : this.sanitize_id(this.flows_process + "_process_to_transfer_during_interaction_list_container"), // container id
			eventName : this.sanitize_id(this.flows_process + "_process_to_transfer_during_interaction_list_container")  // evento click personalizado
		});

		this.__account = null;
		this.onDataLoaded = params.onDataLoaded || $.noop;

		return true;
	}

	if (typeof inConcert.Administration.Campaigns.FlowManagement !== "function") {
		inConcert.Administration.Campaigns.FlowManagement = flow_management;
	}

	flow_management.prototype.initialize = function()
	{
		this.current_flow_info = null;
		this.make_html();
		this.fill_data();
		this.set_events();
		this.set_default_values();
		this.hide_transfer_list();
	};

	flow_management.prototype.fill_data = function()
	{
		const self = this;
		const flowAPI = inConcert.app.getFlowDesignerAPI();
		const params = {proxyType: this.proxy_type};

		flowAPI.loadFlowsByType(params,
			function(success, flows) {
				if (!success) {
					let account_description = self.section.frm.getAccountDescription( { account: self.__account, message_type: "WITH_PROXY" } );
					self.section.frm.show_status_message(FORMAT("<%=_T('Error Retrieving Flows By Type {0}.')%>", account_description), "Error", false);
					return;
				}
				// cargo el combo flows options
				self.types.forEach(type => self.section.frm.fill_combo(
					$(self.flows_process + "_" + type.id), /* cbo flows options       */
					true,           /* add blank first element */
					flows,          /* list                    */
					"Id",           /* attr id                 */
					"Name",         /* attr name               */
					true            /* sorted                  */
				));
				let $ul = $(self.flows_process + "_process_to_transfer_during_interaction_list");
				var $liHeaders = "" + 
					"<li>" +
						"<label class='pl-0 py-2 d-block'><%=_T('Agent can transfer to process')%></label>" +
					"</li>";
				$ul.append($liHeaders);
				let sorted_flows = inConcert.Utils.sortArray(flows, "Name");
				sorted_flows.forEach(function(flow) {
					self.FlowsByType[flow.Id] = flow;
					var $li = "" + 
						"<li id='" + flow.Id + "_li'>" + "<div class='form-check'>" +
							"<input id='" + flow.Id + "' class='form-check-input' type='checkbox' process='" + flow.Name + "'/>" +
							"<label for='" + flow.Id + "' class='form-check-label form-check-label-title'>" + flow.Name + "</label>" +
						"</div> </li>";
					$ul.append($li);
					//Ocultamos el li si ya lo tenemos seleccionado
					if(self.BotsToTransferFromAgent.find(bot => bot.id === flow.Id) !== undefined) {
						$("#" + flow.Id + "_li").hide();
					}
				});
				//Hay que definir aqui este evento, porque los elementos li se crean de forma asincronca
				$(self.flows_process + "_process_to_transfer_during_interaction_list_container input")
					.off("click")
					.on("click", function()
					{
						let selected_processes = "";
						let selection = $(self.flows_process + "_process_to_transfer_during_interaction_list_container").find(":checked");
						for (let i = 0; i < selection.length; i++) {
							selected_processes += (selected_processes.length ? ", " : "") + $(selection[i]).next().text();
						}
						$(self.flows_process + "_process_to_transfer_during_interaction_selection .formVisualComboValue").text(selected_processes);
					});
					
				self.init_ctrls();
			}
		);
	};

	flow_management.prototype.validate = function()
	{
		let result = { success: false };
		let hasFlows = this.types.some(type => !!$(this.flows_process + "_" + type.id).val());

		if (hasFlows && this.inactivity) {
			let inactivityTime = parseInt($(this.inactivity_time).val());
			let inactivityTimeUnit = $(this.inactivity_unit).val();
			let inactivityTimeSeconds;
			switch(inactivityTimeUnit) {
				case "minutes":
					inactivityTimeSeconds = inactivityTime * 60;
					break;
				case "hours":
					inactivityTimeSeconds = inactivityTime * 60 * 60;
					break;
				case "days":
					inactivityTimeSeconds = inactivityTime * 24 * 60 * 60;
					break;
			}
			if (!inactivityTimeSeconds || inactivityTimeSeconds < 0 || 
				inactivityTimeSeconds > this.inactivity.maxHours * 60 * 60
			) {
				result.error_message = this.inactivity.error;
				result.$ctrl = $(this.inactivity_time);
				return result;
			}
		}

		result.success = true;
		return result;
	};

	flow_management.prototype.get = function()
	{
		const self = this;
		let inactivityTimeUnit, inactivityTimeSeconds;
		if (this.inactivity) {
			inactivityTimeUnit = $(this.inactivity_unit).val();
			let inactivityTime = parseInt($(this.inactivity_time).val());
			switch(inactivityTimeUnit) {
				case "minutes":
					inactivityTimeSeconds = inactivityTime * 60;
					break;
				case "hours":
					inactivityTimeSeconds = inactivityTime * 60 * 60;
					break;
				case "days":
					inactivityTimeSeconds = inactivityTime * 24 * 60 * 60;
					break;
			}
		}
		let info = self.types.reduce(function(info, type) {
			let process = $(self.flows_process + "_" + type.id).val();
			if (process) {
				info[type.id] = {
					Process : process,
					InactivitySeconds : inactivityTimeSeconds,
					InactivityTimeUnit : inactivityTimeUnit
				};
			}
			return info;
		}, {});
		return info;
	};

	flow_management.prototype.attach_data = function( acc )
	{
		if ( !acc ) {
			return;
		}
		acc.FlowInformation = this.get();
		acc.BotsToTransferFromAgent = this.BotsToTransferFromAgent;
		return acc;
	};

	flow_management.prototype.update_current_flows_mng = function()
	{
		this.current_flow_info = this.get();
	};

	flow_management.prototype.set_ctrls_values = function( flows_mng )
	{
		if ( !flows_mng ) {
			return;
		}
		const self = this;
		let inactivityTimeUnit, inactivityTimeSeconds;
		this.types.forEach(function(type) {
			let typeData = flows_mng[type.id];
			if (typeData) {
				$(self.flows_process + "_" + type.id).val(typeData.Process).trigger("change");
				inactivityTimeUnit = inactivityTimeUnit || typeData.InactivityTimeUnit;
				inactivityTimeSeconds = inactivityTimeSeconds || typeData.InactivitySeconds;
			}
		});
		this.BotsToTransferFromAgent.forEach(function(bot) {
			self.AddBotToTransferFromAgent(bot);
		});
		//Visualiza u oculta el contenedor o el mensaje not found
		this.ShowOrHideBotsToTransfer();
		//
		if (this.inactivity) {
			inactivityTimeUnit = inactivityTimeUnit || "minutes";
			inactivityTimeSeconds = inactivityTimeSeconds || (30 * 60);
			let inactivityTime;
			switch(inactivityTimeUnit) {
				case "minutes":
					inactivityTime = inactivityTimeSeconds / 60;
					break;
				case "hours":
					inactivityTime = inactivityTimeSeconds / 60 / 60;
					break;
				case "days":
					inactivityTime = inactivityTimeSeconds / 24 / 60 / 60;
					break;
			}
			$(this.inactivity_time).val(inactivityTime);
			$(this.inactivity_unit).val(inactivityTimeUnit);
		}
		this.onDataLoaded();
	};

	flow_management.prototype.set = function( acc )
	{
		if ( !acc ) {
			this.__account = null;
			return;
		}

		this.__account = acc;
		this.current_flow_info = acc.FlowInformation;
		this.BotsToTransferFromAgent = acc.BotsToTransferFromAgent || [];
	};

	flow_management.prototype.init_ctrls = function()
	{
		this.set_ctrls_values(this.current_flow_info);
	};

	flow_management.prototype.disable_flows_mng = function( disabled )
	{
		const self = this;
		const is_readonly = this.section.is_readonly();
		disabled = is_readonly ? true : disabled;

		const ds_ctrl = function($ctrl) {
			if ( is_readonly ) {
				self.section.frm.readonly_control( $ctrl, true, false );
			}
			else {
				self.section.frm.disable_control( $ctrl, disabled, true );
			}
		};

		this.types.forEach(type => ds_ctrl($(this.flows_process + "_" + type.id)));
		
		if (this.inactivity) {
			ds_ctrl($(this.inactivity_time));
			ds_ctrl($(this.inactivity_unit));
		}
	};

	flow_management.prototype.set_default_values = function()
	{
		this.types.forEach(type => $(this.flows_process + "_" + type.id).val(""));
		if (this.inactivity) {
			$( this.inactivity_time ).val("30");
			$( this.inactivity_unit ).val("minutes");
		}
	};

	flow_management.prototype.editable = function()
	{
		const self = this;

		let ds_ctrl = function( $ctrl, ds, required ) {
			self.section.frm.disable_control( $ctrl, ds, required );
		};

		this.types.forEach(type => ds_ctrl($(this.flows_process + "_" + type.id)));
		
		if (this.inactivity) {
			ds_ctrl($(this.inactivity_time));
			ds_ctrl($(this.inactivity_unit));
		}
	};

	flow_management.prototype.readonly = function()
	{
		const self = this;

		let ds_ctrl = function( $ctrl, ds, required ) {
			self.section.frm.readonly_control( $ctrl, ds, required );
		};

		this.types.forEach(type => ds_ctrl($(this.flows_process + "_" + type.id), true));
		if (this.inactivity) {
			ds_ctrl($(this.inactivity_time));
			ds_ctrl($(this.inactivity_unit));
		}
	};

	flow_management.prototype.sanitize_id = function( id )
	{
		// remueve el # de jquery
		return id.substring( 1 );
	};

	flow_management.prototype._is_transfer_visible = function()
	{
		const availables_channels = {
			CALL       : true,
			CHAT       : false,
			FACEBOOK   : false,
			INSTAGRAM  : false,
			YOUTUBE	   : false,
			APPSTORE   : false,
			GOOGLEPLAY : false,
			RCS	   	   : false,
			IM         : false,
			MAIL       : false,
			SMS        : false,
			WHATSAPP   : false,
			TWITTER    : false,
			WEBCONTACT : false,
			VOICEMAIL  : false
		};
		return availables_channels[this.proxy_type];
	};

	flow_management.prototype.hide_transfer_list = function () {
		
		if (!this._is_transfer_visible()) { 
			$(this.container_id).find("[row_transfer_list]").hide();
		}
	};

	flow_management.prototype.make_html = function()
	{
		let s_id = this.sanitize_id;
		let flows_process      = s_id( this.flows_process );
		let inactivity_unit    = s_id( this.inactivity_unit );
		let inactivity_time    = s_id( this.inactivity_time );
		let inactivity_section = s_id( this.inactivity_section );
		let days = this.inactivity && this.inactivity.maxHours > 72;

		let typesHtml = this.types.reduce((html, type) => html.concat(
			"<div class='col-4'>",
				"<div class='form-group' title='", type.description , "'>",
					"<label for='", flows_process + "_" + type.id, "'>", type.name, "<%=_T(':')%>", "</label>",
					"<select id='", flows_process + "_" + type.id, "' class='form-control'></select>",
					"<i id='", flows_process + "_" + type.id, "_validation' class='formFieldValidation formFieldError' hidden ></i>",
				"</div>",
			"</div>"
		), "");

		let processToTransferHtml = this.make_process_to_transfer_list_html();

		$( this.container_id ).html( "".concat(
		"<div class='card'>",	
			"<div class='card-block'>",	
				// titulo
				"<div class='row'>",
					"<div class='col-12'>",
						"<div class='card-header'>",
							"<h5><%=_T('Flow Management')%></h5>",
						"</div>",
					"</div>",
				"</div>",
				"<div class='row'>",
					// process
					typesHtml,
				"</div>",
				//inactivity
				this.inactivity ? "".concat(
					"<div id='", inactivity_section, "' class='row' style='display:none;'>",
						"<div class='col-4'>",
							"<div class='form-group flex-grow-1 w-100 position-relative'>",
								"<label for='", inactivity_time, "'>", "<%=_T('Inactivity Time')%>", "<%=_T(':')%> <span class='fa fa-question-circle'  title='", this.inactivity.disclaimer, "'></span> </label>",
								"<input id='", inactivity_time, "' class='form-control formFieldHour' type='text'>",
								"<i id='", inactivity_time, "_validation' class='formFieldValidation formFieldError' hidden ></i>",
							"</div>",
						"</div>",
						"<div class='col-4'>",
							"<div class='form-group flex-grow-1 w-100 align-self-end'>",
								"<label></label>",
								"<select id='", inactivity_unit, "' class='form-control'>",
									"<option value='minutes'><%=_T('Minutes')%></option>",
									"<option value='hours'><%=_T('Hours')%></option>",
									days ? "<option value='days'><%=_T('Days')%></option>" : "",
								"</select>",
							"</div>",
						"</div>",
					"</div>"
			) : "",
				"</div>",
			"</div>",
			"<div row_transfer_list >"+
					"<div class='card'>"+
						"<div class='card-block'>"+
							// process to transfer
							processToTransferHtml,
						"</div>"+
					"</div>"+
			"</div>",
		// disclaimer
		"<div style='display:none' class='tabDisclaimer formFieldsetMessagesContainer messageInfo'>", "<%=_T('DISCLAIMER MESSAGE !!!.')%>", "</div>"
		));
	};

	flow_management.prototype.set_events = function()
	{
		const self = this;

		if ( this.section.is_readonly() ) {
			return;
		}

		$(this.flows_process + "_process_to_transfer_during_interaction_selector")
			.off("click")
			.on("click", function()
			{
				let $list = $(self.flows_process + "_process_to_transfer_during_interaction_list_container");
				if ($list.is(":visible")) {
					self.process_to_transfer_during_interaction_popup.hide();
				}
				else {
					// Show popup
					self.process_to_transfer_during_interaction_popup.show();
				}
			}
		);

		$(this.flows_process + "_add_selected_processes")
			.off("click")
			.on("click", function() {
				self.process_to_transfer_during_interaction_popup.hide();
				let selections = $(self.flows_process + "_process_to_transfer_during_interaction_list_container").find(":checked");
				let returnToAgent = $(self.flows_process + "_go_back_to_agent_after_transfer_to_process").prop("checked");
				//Insertar los que no existan
				$.each(selections, function(_, selection) {
					let dom_id = $(selection).attr("id");
					if (!self.BotsToTransferFromAgent.find(bot => bot.id === dom_id)) {
						self.BotsToTransferFromAgent.push({id: dom_id, returnToAgent: returnToAgent});
					}
					//Ocultarlo en la lista de bots disponibles
					$("#" + dom_id + "_li").hide();
				});
				//Vaciar contenedor de labels
				$(self.flows_process + "_selected_processes_to_transfer").empty();
				//Insertar en contenedor de labels
				self.BotsToTransferFromAgent.forEach(function(bot) {
					self.AddBotToTransferFromAgent(bot);
				});
				//Visualiza u oculta el contenedor o el mensaje not found
				self.ShowOrHideBotsToTransfer();
				//
				$(self.flows_process + "_process_to_transfer_during_interaction_selection .formVisualComboValue").text("");
				$(self.flows_process + "_process_to_transfer_during_interaction_list_container input").prop("checked", false);
			}
		);

		this.types.forEach(function(type) {
			$(self.flows_process + "_" + type.id).off("change").on("change", function() {
				if (self.inactivity) {
					let must_shown_inactivity_section = false;
					self.types.forEach(function(type) {
						let value = $(self.flows_process + "_" + type.id).val();
						if (value) {
							must_shown_inactivity_section = true;
						}
					});
					const $inactivity_section = $(self.inactivity_section);
					if (must_shown_inactivity_section) {
						$inactivity_section.show();
					}
					else {
						$inactivity_section.hide();
					}
				}
			});
		});
	};

	flow_management.prototype.make_process_to_transfer_list_html = function()
	{
		let dom_prefix_id = this.sanitize_id( this.flows_process );

		return "" +
			"<div class='row'>"+
				"<div class='col-12'>"+
					"<div class='card-header'>"+
						"<h5><%=_T('Process Flows available for agents to transfer')%></h5>"+
					"</div>"+
				"</div>"+
			"</div>"+
			"<div class='row'>"+
				"<div class='col-6 d-flex flex-column'>"+
					"<div class='form-group'>"+
						"<label>" + "<%=_T('During interaction')%>" + "<%=_T(':')%>" + "</label>" +
						"<div id='" + dom_prefix_id + "_process_to_transfer_during_interaction_selection' class='formVisualCombo'>" +
							"<div id='" + dom_prefix_id + "_process_to_transfer_during_interaction_selector' class='form-control'>" +
								"<label class='formVisualComboValue pl-0'>" + "</label>" +
							"</div>" +
							"<div id='" + dom_prefix_id + "_process_to_transfer_during_interaction_list_container' class='formVisualComboList' style='display:none;'>" +
								"<ul id='" + dom_prefix_id + "_process_to_transfer_during_interaction_list'>" +
								"</ul>" +
							"</div>" +
						"</div>" +
						"<i class='formFieldValidation formFieldValidationError' hidden></i>" +
					"</div>"+
					"<div class='form-group pt-2 pb-2'>" +
						"<div class='form-check'>" +
							"<label for='" + dom_prefix_id + "_go_back_to_agent_after_transfer_to_process' class='form-check-label'>" +
								"<input id='" + dom_prefix_id + "_go_back_to_agent_after_transfer_to_process' type='checkbox' class='form-check-input' name='adm_account_protocol' value='Go back to agent' />" +
								"<span>" + "<%=_T('Send back to Agent')%>" + " <i class='fa fa-question-circle' title='" + "<%=_T('The system will reassign the interaction to an agent when process flow ends.')%>" + "' >" + "</i></span>" +
							"</label>" +
						"</div>" +
					"</div>"+
					"<button id='" + dom_prefix_id + "_add_selected_processes' class='pageButtonGeneric buttonColorWhite align-self-start mb-3'>" +
						"<%=_T('Add Flows')%>" + 
					"</button>" +
				"</div>" +
				"<div class='col-6 d-flex flex-column'>" +
					"<div class='form-group mb-0'>" +
						"<label>" + "<%=_T('Flows Added')%>" + "<%=_T(':')%>" + "</label>" +
					"</div>" +
					"<div class='card-outlined-internal form-group-pill-card-outlined flex-grow-1'>" +
						"<div id='" + dom_prefix_id + "_processes_to_transfer_not_found' class='inlineMessagesContainer messageInfo'>" +
							"<%=_T('No flows found.')%>" +
						"</div>" +
						"<div class='form-group-pill-container' id='" + dom_prefix_id + "_selected_processes_to_transfer' style='display:none;'></div>" +
					"</div>" +
				"</div>"+
			"</div>";
	};

	flow_management.prototype.ShowOrHideBotsToTransfer = function()
	{
		const $processes_to_transfer_not_found = $(this.flows_process + "_processes_to_transfer_not_found");
		const $processes_to_transfer = $(this.flows_process + "_selected_processes_to_transfer");
		if ((this.BotsToTransferFromAgent || []).length === 0) {
			$processes_to_transfer_not_found.show();
			$processes_to_transfer.hide();
			return;
		}
		$processes_to_transfer_not_found.hide();
		$processes_to_transfer.show();
	};

	flow_management.prototype.AddBotToTransferFromAgent = function(bot)
	{
		const self = this;
		if (!this.FlowsByType[bot.id]) {
			return;
		}
		const $processes_to_transfer = $(this.flows_process + "_selected_processes_to_transfer");
		const $lbl = "" +
			"<label id='" + bot.id + "_label' class='form-group-pill'>" +
				"<span>" + this.FlowsByType[bot.id].Name + " - " + (bot.returnToAgent ? "<%=_T('Go back to agent')%>" : "<%=_T('End interaction')%>") + "</span>" +
				"<button id='" + bot.id + "_remove' class='pillButtonDelete'></button>" +
			"</label>";
		$processes_to_transfer.append($lbl);

		$("#" + bot.id + "_remove").click(function() {
			for (var i = self.BotsToTransferFromAgent.length - 1; i >= 0; --i) {
				if ((self.BotsToTransferFromAgent[i].id + "_remove") === $(this).prop("id")) {
					//Mostrar en la lista de bots disponibles
					$("#" + self.BotsToTransferFromAgent[i].id + "_li").show();
					//Eliminar del array
					self.BotsToTransferFromAgent.splice(i,1);
				}
			}
			//Eliminar del contenedor de labels
			$(this).parent().remove();
			//Visualiza u oculta el contenedor o el mensaje not found
			self.ShowOrHideBotsToTransfer();
		});
	};
}());
