/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, -W083, esversion: 11 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

// try improve namespacing a bit
if ( !this.inConcert.Administration ) {
	this.inConcert.Administration = {};
}

if ( !this.inConcert.Administration.Campaigns ) {
	this.inConcert.Administration.Campaigns = {};
}

(function () {

//	"use strict";

	const format = inConcert.Utils.format;
	const _T = inConcert.i18n.getString;
	const ERROR_TIMEOUT_MS = 5000;

	const bool_to_string  = function(enabled) {
		return Number(enabled).toString();
	};

	//---------------------------------------------------------------------
	//hsm Buttons
	function hsm_management_button( prefix_id) {
		this.prefix_id = prefix_id;
	}

	function showWaiting(waitingContainerId, show, message) {
		let $loading = $(waitingContainerId);

		if(show) {
			$loading
				.show()
				.find("[waiting_message]")
				.text(message);
		}
		else {
			$loading.hide();
		}
	}

	hsm_management_button.prototype.__makeButton = function (hsmButton) {
		this.hsmButton = hsmButton || {};
		const self = this;
		var $template = $(this.__makeButtontemplate());

		$template.find("[hsm_btn_value]").val(this.hsmButton.payload || "");

		$template.find("[hsmButton_add]").click(function () {
			var $row = $template;
			// Up to 3 quick replay buttons and upto 1 call_to_action buttons
			let max_buttons = $("#" + self.prefix_id + "_edit_popup_button_type").val() === "quick_reply" ? 3 : 1;
			if ($row.find("[hsm_btn_value]").val().trim() !== "" && ($row.siblings().length + 1) < max_buttons) {
				var $newRow = self.__makeButton();
				$row.after($newRow);
				$row.parent().scrollTo($newRow);
				$newRow.find("[hsm_btn_value]").focus();
			}
			else {
				$row.find("[hsm_btn_value]").focus();
			}
		});

		$template.find("[hsmButton_remove]").click(function() {
			var $row = $template;
			var $siblings = $row.siblings();
			if ($siblings.length) {
				if ($row.prev().length) {
					$row.prev().find("[hsm_btn_value]").focus();
				}
				else {
					$row.next().find("[hsm_btn_value]").focus();
				}
				$row.remove();
			}
			else {
				$row.find("[hsm_btn_value]").val("");
				$row.find("[hsm_btn_value]").focus();
			}
		});

		return $template;
	};

	hsm_management_button.prototype.__getDataButtons = function () {
		let data = [];
		let button_type = $("#" + this.prefix_id + "_edit_popup_button_type").val();
		let hsm_buttons_type = this.m_current_hsm && this.m_current_hsm.hsm_buttons_type ? this.m_current_hsm.hsm_buttons_type : $("#" + this.prefix_id + "_edit_popup_button_type").val();
		let button_list_id = this.prefix_id + (hsm_buttons_type === "call_to_action" ? "_cta_buttons" : "_qr_buttons");
		$("#" + button_list_id + " li").each(function () {
			const $this = $(this);
			let button = {};
			button.type = button_type;
			button.payload = $this.find("[hsm_btn_value]").val();
			if ($this.find("[hsm_btn_value]").val().trim().length) {
				data.push(button);
			}
		});
		return data;
	};

	hsm_management_button.prototype.__deleteButtons = function () {
		$("#" + this.prefix_id + "_cta_buttons").html("");
		$("#" + this.prefix_id + "_qr_buttons").html("");
	};

	hsm_management_button.prototype.__makeButtons = function(data) {
		const self = this;
		this.__deleteButtons();
		var hsmButtons = data? data.hsm_buttons || []: [];	
		let hsm_buttons_type = data? data.hsm_buttons_type : $("#" + this.prefix_id + "_edit_popup_button_type").val();
		let button_list_id = this.prefix_id + (hsm_buttons_type === "call_to_action" ? "_cta_buttons" : "_qr_buttons");
		var $buttonsHsm = $("#" + button_list_id);
		var $list = [];

		$.each(hsmButtons, function(_, button) {
			$list.push(self.__makeButton(button));
		});
		
		if (!$list.length) {
			$list.push(self.__makeButton());
		}

		$buttonsHsm.append($list);
	};

	hsm_management_button.prototype.__makeButtontemplate = function () {
		let payloadLabel = $("#" + this.prefix_id + "_edit_popup_button_type").val() === "call_to_action" ? "<%=_T('Insert URL ending')%>" : "<%=_T('Insert payload')%>";
		return "".concat(
			"<li hsmButton class='row ruleField ruleFieldLimit'>",
				"<div class='col-12'>",
					"<div class='ruleFieldContent'>",
						"<div class='form-group flex-grow-1 mr-2'>",
							"<label>", payloadLabel, "<%=_T(':')%></label>",
							"<input hsm_btn_value type='text' class='form-control'>",
						"</div>",
						"<div class='form-group d-flex align-items-end'>",
							"<button hsmButton_remove class='pageButtonGeneric ruleFieldDelete mr-2' type='button'>",
								"<i class='far fa-minus-circle mr-0'></i>",
							"</button>",
							"<button hsmButton_add class='pageButtonGeneric ruleFieldAdd mr-2' type='button'>",
								"<i class='far fa-plus mr-0'></i>",
							"</button>",
						"</div>",
					"</div>",
				"</div>",
			"</li>");
	};

	//---------------------------------------------------------------------
	// hsm management
	function hsm_management(params)
	{
		params = params || {};
		/* jshint validthis: true*/
		this.parent = params.parent;
		this.section = this.parent.parent;
		this.frm = this.section.frm;

		this.prefix_id = params.prefix_id; // ejemplo de prefijo: adm_form_campaign_accounts_hsm_form, etc

		this.current_hsm_mng = null;

		// container
		this.container_id = "#" + params.container_id;
		// enable/disable
		this.hsm_mng = "#" + this.prefix_id + "_hsm_mng";
		this.m_integration_type = null;
		this.m_grd_hsm_list = null;
		this.rows_per_page = 10;
		this.show_at_least = this.rows_per_page - 1;
		this.m_is_new_hsm = null;
		this.m_current_hsm = null;

		// Instancia de la clase los TABs
		this.main_tabs = new inConcert.conf_form_tab({
			parent        : this,
			frm           : this.frm,
			container_id  : this.prefix_id + "_languages_tabs",
			must_intialize: false,
			tabs_list   : {
				en: {
					id              : "en",
					name            : "<%=_T('English')%>",
					tab_container_id: this.prefix_id + "_languages_english_tab_container",
					selected        : true
				},
				es: {
					id              : "es",
					name            : "<%=_T('Spanish')%>",
					tab_container_id: this.prefix_id + "_languages_spanish_tab_container"
				},
				fr: {
					id              : "fr",
					name            : "<%=_T('French')%>",
					tab_container_id: this.prefix_id + "_languages_french_tab_container"
				},
				pt: {
					id              : "pt",
					name            : "<%=_T('Portuguese')%>",
					tab_container_id: this.prefix_id + "_languages_portuguese_tab_container"
				}
			}
		});
		this.hsmButtons = new hsm_management_button(this.prefix_id);
		return true;
	}

	if (typeof inConcert.Administration.Campaigns.HSM_Management !== "function") {
		inConcert.Administration.Campaigns.HSM_Management = hsm_management;
	}

	hsm_management.prototype.initialize = function()
	{
		this.current_hsm_mng = null;
		this.make_html();
		this.__initialize_grid();
		this.main_tabs.initialize();
		this.main_tabs.show_tabs_container();
		this.set_events();
		this.set_default_codes();
	};

	hsm_management.prototype.__initialize_grid = function()
	{
		const self = this;

		const container_id = this.sanitize_id(this.container_id);

		this.m_grd_hsm_list = new inConcert.conf_form_paginator({
			paginator_name: "HSM_LIST",
			owner: this,
			frm: this.frm,
			auto_initialize: true,
			row_id: "id",
			display_name: "id",
			rows_per_page: this.rows_per_page,
			show_at_least: this.show_at_least,
			paginator_type: "ALL_ROWS_CLIENT_SIDE",
			add_with_enter: false,
			ids: {
				grid: this.prefix_id + "_list",
				paginator: this.prefix_id + "_paginator",
				not_results_found: this.prefix_id + "_not_results_found",
				container_new_btn: container_id,
				new_item: this.prefix_id + "_popup_new",
				//popup0 (add hsm)
				popup: this.prefix_id + "_edit_popup",
				popup_msg: this.prefix_id + "_edit_popup_message",
				popup_create: this.prefix_id + "_edit_popup_create",
				popup_create_and_continue: this.prefix_id + "_edit_popup_create_and_continue",
				popup_title: this.prefix_id + "_edit_popup_title",
				popup_close: this.prefix_id + "_edit_popup_close",
				popup_close_times: this.prefix_id + "_edit_popup_close_times",
				//popup_input_id: this.prefix_id + "_edit_popup_id",
				popup_waiting: this.prefix_id + "_edit_popup_waiting"
			},
			strings: {
				//popup0 (add hsm)
				not_results_found         : "<%=_T('No HSM Messages Found.')%>",
				can_remove                : "<%=_T('Do you want to delete HSM Message: {0} ?')%>",
				title_new                 : "<%=_T('New HSM')%>",
				title_update              : "<%=_T('Edit HSM')%>",
				button_create             : "<%=_T('Add')%>",
				button_create_and_continue: "<%=_T('Add & Continue')%>"
			},
			columns: [
				{id: "_number"  , name: "", type: "counter"},
				{id: "id"        , name: "<%=_T('Id')%>"},
				{id: "groupName" , name: "<%=_T('Namespace')%>"},
				{id: "en" , name: "<%=_T('English')%>", get_value: function(item) {
					return self.get_grid_value(item, "en");
				}},
				{id: "es" , name: "<%=_T('Spanish')%>", get_value: function(item) {
					return self.get_grid_value(item, "es");
				}},
				{id: "fr" , name: "<%=_T('French')%>", get_value: function(item) {
					return self.get_grid_value(item, "fr");
				}},
				{id: "pt" , name: "<%=_T('Portuguese')%>", get_value: function(item) {
					return self.get_grid_value(item, "pt");
				}},					
				{
					id: "_action",
					name: "",
					system_action: {
						edit: {
							must_show: function() {
								return true;
							}
						},
						erase: true
					}
				}
			],
			callbacks: {
				save: function(callback, must_create_and_contiune, current_popup) {
					callback = callback || $.noop;
					self.add_or_update_message(callback, must_create_and_contiune, current_popup);
				},
				edit: function(row, current_popup) {
					self.on_open_popup(row, current_popup);
				},
				remove: function(row, callback = $.noop) {
					self.remove(row, callback);
				},
				on_closed_popup: function(current_popup) {
					self.on_closed_popup(current_popup);
				}
			},
			validate: {
				required_msg: "<%=_T('Must add at least one HSM Message.')%>",
				required: true
			}
		});
	};

	hsm_management.prototype.__fill_grid = function(list)
	{
		list = list || {};
		this.m_grd_hsm_list.fill_rows($.noop, list);
	};

	const HSM_TYPES = {
		ICC_CLOUD_API: [
			{ id: "hsm", label: "<%=_T('HSM')%>" },
			{ id: "media_simple", label: "<%=_T('Media Template')%>" },
			{ id: "interactive_template", label: "<%=_T('Interactive Template')%>" }
		],
		BUSINESS: [
			{ id: "hsm", label: "<%=_T('HSM')%>" },
			{ id: "media_simple", label: "<%=_T('Media Template')%>" },
			{ id: "interactive_template", label: "<%=_T('Interactive Template')%>" }
		],
		WAVY: [
			{ id: "hsm", label: "<%=_T('HSM')%>" },
			{ id: "media_simple", label: "<%=_T('Media Template')%>" },
			{ id: "interactive_template", label: "<%=_T('Interactive Template')%>" }
		],
		SMOOCH: [
			{ id: "hsm", label: "<%=_T('HSM')%>" },
			{ id: "media_simple", label: "<%=_T('Media Template')%>" },
			{ id: "interactive_template", label: "<%=_T('Interactive Template')%>" }
		],
		INFOBIP: [
			{ id: "hsm", label: "<%=_T('HSM')%>" },
			{ id: "media_simple", label: "<%=_T('Media Template')%>" },
			{ id: "interactive_template", label: "<%=_T('Interactive Template')%>" }
		]
	};

	const HSM_BUTTON_TYPES = {
		ICC_CLOUD_API: [
			{ id: "quick_reply", label: "<%=_T('Quick Reply')%>" },
			{ id: "call_to_action", label: "<%=_T('Call To Action')%>" }
		],
		BUSINESS: [
			{ id: "quick_reply", label: "<%=_T('Quick Reply')%>" }
		],
		WAVY: [
			{ id: "quick_reply", label: "<%=_T('Quick Reply')%>" }
		],
		SMOOCH: [
			{ id: "quick_reply", label: "<%=_T('Quick Reply')%>" }
		],
		INFOBIP: [
			{ id: "quick_reply", label: "<%=_T('Quick Reply')%>" }
		]		
	};

	hsm_management.prototype.on_open_popup = function(item)
	{
		const self = this;
		this.m_is_new_hsm = item ? false: true;
		this.m_current_hsm = item;

		this.m_mediaUrlFromRepository = !this.m_is_new_hsm ? item.mediaUrlFromRepository : false;
		this.m_mediaUrlFileId = !this.m_is_new_hsm ? item.mediaUrlFileId : null;
		this.m_mediaFileName = !this.m_is_new_hsm ? item.mediaFileName : null;
		this.m_mediaContentType = !this.m_is_new_hsm ? item.mediaContentType : null;
		this.m_mediaUrlType = !this.m_is_new_hsm ? item.mediaUrlType : null;

		const $type = $("#" + this.prefix_id + "_edit_popup_type");
		const types = HSM_TYPES[this.m_integration_type];
		const button_types = HSM_BUTTON_TYPES[this.m_integration_type];
		const $button_type = $("#" + self.prefix_id + "_edit_popup_button_type");
		
		const $media = $("#" + this.prefix_id + "_media_template_container").find("[hsm_media_template_value]");
		this.section.frm.readonly_control( $media, this.m_mediaUrlFileId, false );

		const media_types = [
			{ id: "whatsapp_cloud_image", label: "<%=_T('Image')%>" },
			{ id: "whatsapp_cloud_video", label: "<%=_T('Video')%>"  },
			{ id: "whatsapp_cloud_document" , label: "<%=_T('File')%>"   }
		];
		const $media_type = $("#" + this.prefix_id + "_edit_popup_media_type");

		if (types) {
			this.frm.fill_combo($type, false, types, "id", "label");
			this.frm.fill_combo($media_type, false, media_types, "id", "label");
			$type.parent().show();
			$type.off("change")
				.on("change", function () {
					if (item) {
						item.hsm_buttons = [];
					}
					self.frm.fill_combo($button_type, false, button_types, "id", "label");
					let isButtonTypeReadonly = $(this).val() !== "interactive_template" || self.m_integration_type !== "ICC_CLOUD_API";
					let isButtonTypeRequired = !isButtonTypeReadonly;
					self.frm.readonly_control($button_type, isButtonTypeReadonly, isButtonTypeRequired);
					$button_type
						.val(self.m_current_hsm && self.m_current_hsm.hsm_buttons_type ? self.m_current_hsm.hsm_buttons_type : "quick_reply")
						.change()
						.parent().toggle($(this).val() === "interactive_template");
					if ($(this).val() === "interactive_template") {
						$("#" + self.prefix_id + "_interactive_template_container").show();
						$("#" + self.prefix_id + "_media_template_container").hide();
					}
					else if($(this).val() === "media_simple")
					{
						$("#" + self.prefix_id + "_media_template_container").show();
						self.toggle_popup_media_from_shared_file();
						$("#" + self.prefix_id + "_interactive_template_container").hide();
					}
					else {
						$("#" + self.prefix_id + "_interactive_template_container").hide();
						$("#" + self.prefix_id + "_media_template_container").hide();
					}
				});

			$button_type.off("change")
				.on("change", function () {
					$("#" + self.prefix_id + "_quick_reply_buttons_container").toggle($(this).val() === "quick_reply" || self.m_integration_type !== "ICC_CLOUD_API");
					$("#" + self.prefix_id + "_call_to_action_buttons_container").toggle($(this).val() === "call_to_action" && self.m_integration_type === "ICC_CLOUD_API");
					if(self.m_current_hsm) {
						self.m_current_hsm.hsm_buttons_type = $(this).val();
					}
					self.hsmButtons.__makeButtons(self.m_current_hsm);					
				});

			self.$getMediaUrlButton()
				.off("click")
				.on("click", function() {
					const $parent = $(this);
					self.show_popup_shared_file($parent);
				}
			);

			self.$delMediaUrlButton()
				.off("click")
				.on("click", function() {
					const $media = $("#" + self.prefix_id + "_media_template_container").find("[hsm_media_template_value]");
					$media.val("");
					self.m_mediaUrlFromRepository = false;
					self.m_mediaUrlFileId = null;
					self.m_mediaFileName = null;
					self.m_mediaContentType = null;
					self.m_mediaUrlType = null;
					self.section.frm.readonly_control( $media, false, false );
				}
			);

		}
		else {
			$type.parent().hide();
			$("#" + self.prefix_id + "_interactive_template_container").hide();
			$("#" + self.prefix_id + "_media_template_container").hide();
			if (item) {
				item.hsm_buttons = [];
				item.type = "";
			}
		}

		this.clean_up_controls();
		
		if (this.m_is_new_hsm) {
			// nuevo
			$button_type.parent().toggle(false);
			return;
		}
		// editando

		this.hsmButtons.__makeButtons(item);
		this.frm.fill_combo($button_type, false, button_types, "id", "label");
		let isButtonTypeReadonly = item.type !== "interactive_template" || this.m_integration_type !== "ICC_CLOUD_API";
		let isButtonTypeRequired = !isButtonTypeReadonly;
		this.frm.readonly_control($button_type, isButtonTypeReadonly, isButtonTypeRequired);
		$button_type
			.val(item.hsm_buttons_type)
			.change()
			.parent().toggle(item.type === "interactive_template");
		if (item.type === "interactive_template") {
			$("#" + self.prefix_id + "_interactive_template_container").show();
			$("#" + self.prefix_id + "_quick_reply_buttons_container").toggle(item.hsm_buttons_type === "quick_reply" || self.m_integration_type !== "ICC_CLOUD_API");
			$("#" + self.prefix_id + "_call_to_action_buttons_container").toggle(item.hsm_buttons_type === "call_to_action" && self.m_integration_type === "ICC_CLOUD_API");	
		}
		else if(item.type === "media_simple") {
			$("#" + self.prefix_id + "_media_template_container").show();
			$("#" + self.prefix_id + "_media_template_container").find("[hsm_media_template_value]").val( item.mediaUrl || "");
		}
		else {
			$("#" + self.prefix_id + "_interactive_template_container").hide();
			$("#" + self.prefix_id + "_media_template_container").hide();
		}

		const $id = $("#" + this.prefix_id + "_edit_popup_id");
		this.frm.readonly_control($id, true);
		$id.val(item.id);

		const $group_name = $("#" + this.prefix_id + "_edit_popup_group_name");
		$group_name.val(item.groupName).focus();
		
		$type.val(item.type);

		const $container = $(this.container_id);

		for (let lang in item.languages || {}) {
			if (item.languages.hasOwnProperty(lang)) {
				const languange = item.languages[lang];
				const $lang_messages_selector = $container.find("[lang_message_selector='" + lang + "']");
				$lang_messages_selector.prop("checked", true).trigger("change");
				const $status = $container.find("[lang_status_selector='" + lang + "']");
				$status.text(_T(languange.status));
				const $value = $("#" + $lang_messages_selector.attr("related_ctrl"));
				$value.val(languange.text);
				const $code = $("#" + $lang_messages_selector.attr("related_ctrl_code"));
				$code.val(languange.code || lang);
			}
		}

		const $params = $("#" + this.prefix_id + "_edit_popup_params");
		let params = (item.params || []).join("\n");
		$params.val(params);

		if(media_types.filter(media_type => media_type.id === item.mediaUrlType).length > 0) {
			$media_type.val(item.mediaUrlType);
		}
		self.toggle_popup_media_from_shared_file();
	};

	hsm_management.prototype.on_closed_popup = function(/*current_popup*/)
	{
		this.m_is_new_hsm = null;
		this.m_current_hsm = null;
	};

	hsm_management.prototype.remove = function(hsm, callback = $.noop)
	{
		if (!hsm) {
			throw("hsm_management.prototype.remove. Undefined hsm.");
		}
		callback(true);
	};

	hsm_management.prototype.add_or_update_message = function(callback, _, current_popup)
	{
		const self = this;
		callback = callback || $.noop;

		let response = this.validate_hsm_to_add_or_update(current_popup);
		if (!response.success) {
			callback(false);
			return;
		}

		function after_get_templates (success, error, templates, callback) {
			showWaiting("#" + self.prefix_id + "_edit_popup_waiting", false);
			let msg_response = {
				msg : "",
				type: "Info"
			};

			if(success) {
				if(self.parent.constructor.name === "WHATSAPPAccountPanel" && self.m_integration_type === "ICC_CLOUD_API") {
					if(templates) {
						for (const language in response.item.languages) {
							if (Object.hasOwn(response.item.languages, language)) {
								let templateFound = templates.find(template => {
									return template.name === response.item.id && response.item.languages[language].code === template.language;
								});
								if(!templateFound) {
									error = "<%=_T('Template not found for language')%>" + ": " + response.item.languages[language].code;
									response = {success: false, $ctrl: $("#" + self.prefix_id + "_edit_popup_id"), item: null, msg: _T(error)};
									self.show_ctrl_err(response);
									return;
								}
								response.item.languages[language].status = templateFound.status;
							}
						}
					}
					else {
						response = {success: false, $ctrl: $("#" + self.prefix_id + "_edit_popup_id"), item: null, msg: _T(error)};
						self.show_ctrl_err(response);
						return;
					}
				}

				if (self.m_is_new_hsm) {
					msg_response.msg = "<%=_T('HSM Message was successfully added.')%>";
					let must_add_many = false;
					callback(true, response.item, msg_response, must_add_many);
				}
				else {
					let cloned_hsm = inConcert.Utils.cloneObject(response.item);
					msg_response.msg = "<%=_T('HSM Message was successfully updated.')%>";
					self.m_current_hsm = cloned_hsm;
					callback(true, self.m_current_hsm, msg_response);
				}

				self.clean_up_controls();
			}
			else {
				response = {success: false, $ctrl: $("#" + self.prefix_id + "_edit_popup_id"), item: null, msg: _T(error)};
				self.show_ctrl_err(response);
			}
		}

		if (this.parent.constructor.name === "WHATSAPPAccountPanel" && this.m_integration_type === "ICC_CLOUD_API") {
			//Retrieve whatsapp templates for current waba, to set status for each configured language
			showWaiting("#" + this.prefix_id + "_edit_popup_waiting", true, "<%=_T('Retrieving whatsapp templates')%>");

			inConcert.invoke({
				url: inConcert.app.getBaseDomain() + "/inconcert/apps/administration/campaigns/whatsapp_get_message_templates/",
				type: "POST",
				data: JSON.stringify({
					campaignId : self.frm.campaign.Id,
					accountId : (self.parent.current_account && self.parent.current_account.id) ? self.parent.current_account.id : self.parent.m_authorizingId,
					wabaId: $(self.parent.inconcertCloudApiWaba).val()
					//templateId: $("#" + self.prefix_id + "_edit_popup_id").val()
				}),
				dataType: "json",
				contentType: "application/json",
				success: function(response) {
					let success = response.result === "OK";
					let err = response.reason || "<%=_T('Error retrieving templates')%>";
					after_get_templates(success, err, response.templates, callback);
				},
				error: function(_, text_status) {
					var err = text_status.reason || "<%=_T('Invalid Template Id.')%>";
					after_get_templates(false, err, undefined, callback);
				}
			});
		}
		else {
			after_get_templates(true, undefined, undefined, callback);
		}
	};

	hsm_management.prototype.clean_up_controls = function()
	{
		const self = this;
		const $id = $("#" + this.prefix_id + "_edit_popup_id");
		const $group_name = $("#" + this.prefix_id + "_edit_popup_group_name");
		const $type = $("#" + this.prefix_id + "_edit_popup_type");
		const $params = $("#" + this.prefix_id + "_edit_popup_params");

		this.frm.readonly_control($id, false, true);
		$id.val("").focus();
		$group_name.val("");
		$type.val($("#" + this.prefix_id + "_edit_popup_type option:first").val());
		$params.val("");

		const $lang_messages_selector = $(this.container_id).find("[lang_message_selector]");
		$lang_messages_selector.prop("checked", false).trigger("change");
		const $lang_status_selector = $(this.container_id).find("[lang_status_selector]");
		$lang_status_selector.text("");

		$.each($lang_status_selector, function(_, lang_status_selector) {
			const $selector = $(lang_status_selector);
			$selector.parent().toggle(self.parent.constructor.name === "WHATSAPPAccountPanel" && self.m_integration_type === "ICC_CLOUD_API");
		});

		this.main_tabs.switch_tab("en"); // dejo seleccionado el primer tab

		this.hsmButtons.__deleteButtons();
		$("#" + this.prefix_id + "_interactive_template_container").hide();
		$("#" + this.prefix_id + "_media_template_container").hide();
		$("#" + this.prefix_id + "_media_template_container").find("[hsm_media_template_value]").val("");
	};

	hsm_management.prototype.validate_hsm_to_add_or_update = function()
	{
		this.hide_all_errors();
		let response = {success: false, $ctrl: null, item: null, msg: "<%=_T('Unable to Add Message.')%>"};

		const $id = $("#" + this.prefix_id + "_edit_popup_id");
		let id = $id.val();

		let validation_result = inConcert.Utils.validate(id, "required");
		if (!validation_result.success) {
			// valida que el name tenga al menos un caracter
			response.msg = "<%=_T('HSM Id is required.')%>";
			response.$ctrl = $id;
			this.show_ctrl_err(response);
			return response;
		}

		validation_result = inConcert.Utils.validate(id, "alpha");
		if (!validation_result.success) {
			// solo letras, numeros y puntos
			response.msg = "<%=_T('Only letters, numbers, dashes and dots are allowed in the HSM Id.')%>";
			response.$ctrl = $id;
			this.show_ctrl_err(response);
			return response;
		}

		const $group_name = $("#" + this.prefix_id + "_edit_popup_group_name");
		let groupName = $group_name.val();
		validation_result = inConcert.Utils.validate(groupName, "required");
		if ( !validation_result.success ) {
			// valida que el name tenga al menos un caracter
			response.msg = "<%=_T('HSM Namespace is required.')%>";
			response.$ctrl = $group_name;
			this.show_ctrl_err(response);
			return response;
		}
		
		const $type = $("#" + this.prefix_id + "_edit_popup_type");
		let type = $type.val();

		let total_languages = 0;
		let languages = {};
		const $lang_messages_selector = $(this.container_id).find("[lang_message_selector]");
		$.each($lang_messages_selector, function(_, selector) {
			const $selector = $(selector);
			if ($selector.prop("checked") === true) {
				total_languages++;
				const $value = $("#" + $selector.attr("related_ctrl"));
				const $code = $("#" + $selector.attr("related_ctrl_code"));
				const lang = $selector.attr("lang_message_selector");
				languages[lang.toLowerCase()] = {
					text: $value.val(),
					code: $code.val() || lang.toLowerCase(),
					$ctrl: $value
				};
				if(this.m_integration_type !== "ICC_CLOUD_API") {
					languages[lang.toLowerCase()].status = "APPROVED";
				}	
			}		
		});

		if (total_languages === 0) {
			// valida que al menos hayan ingresado un mensaje (es, fr, en)
			response.msg = "<%=_T('HSM Messages is required.')%>";
			response.$ctrl = null;//$group_name;
			this.show_ctrl_err(response);
			return response;
		}

		for (let lang in languages) {
			if (languages.hasOwnProperty(lang)) {
				let language = languages[lang];
				validation_result = inConcert.Utils.validate(language.text, "required");
				if (!validation_result.success) {
					// valida que el mensaje no este vacio (es, fr, en, pt)
					this.main_tabs.switch_tab(lang);
					response.msg = "<%=_T('HSM Message is required.')%>";
					response.$ctrl = language.$ctrl;
					this.show_ctrl_err(response);
					return response;
				}
				delete language.$ctrl;
			}
		}

		const $params = $("#" + this.prefix_id + "_edit_popup_params");
		let parsed_params = $params.val();
		parsed_params = parsed_params.split("\n");
		let params = [];
		parsed_params.forEach(param => {
			if (param.trim().length > 0) {
				params.push(param);
			}
		});

		let $hsm_buttons_type = $("#" + this.prefix_id + "_edit_popup_button_type");
		let hsm_buttons_type = type === "interactive_template" && this.m_integration_type === "ICC_CLOUD_API" ? $hsm_buttons_type.val() : "quick_reply";
		let hsm_buttons = this.hsmButtons.__getDataButtons();
		let item = {
			id: id.toLowerCase(), // el Id tiene que ir en minusculas porque asi lo requiere la config de HSM de Wavy
			groupName: groupName,
			groupId: groupName,
			type : type || "",
			status: "approved",
			available: true,
			params: params,
			languages: languages,
			hsm_buttons_type: hsm_buttons_type,
			hsm_buttons: hsm_buttons
		};

		item.mediaUrl = type === "media_simple"? $("#" + this.prefix_id + "_media_template_container").find("[hsm_media_template_value]").val():"";
		item.mediaUrlFromRepository = this.m_mediaUrlFromRepository || false;
		item.mediaUrlFileId = this.m_mediaUrlFileId;
		item.mediaFileName = this.m_mediaFileName;
		item.mediaContentType = this.m_mediaContentType;
		item.mediaUrlType = this.m_mediaUrlType;

		response.success = true;
		response.msg = "";
		response.item = item;

		return response;
	};

	hsm_management.prototype.hide_all_errors = function()
	{
		this.m_grd_hsm_list.hide_popup_msg();
	};

	hsm_management.prototype.show_ctrl_err = function(err_info)
	{
		if (!err_info) {
			return;
		}
		const $control = err_info.$ctrl;
		if ($control) {
			const error_id = $control.attr("id") + "_validation";
			const $error_indicator = $("#" + error_id);
			if ($error_indicator) {
				$error_indicator.show();
			}
			$control.focus();
		}
		this.m_grd_hsm_list.show_popup_msg(err_info.msg, "Error", ERROR_TIMEOUT_MS);
	};

	hsm_management.prototype.validate = function()
	{
		const result = {success: true};
		return result;
	};

	hsm_management.prototype.get = function()
	{
		const enabled = $(this.hsm_mng).prop("checked");
		const config = this.__get(enabled);
		return config;
	};

	hsm_management.prototype.__get = function(enabled)
	{
		let config = {Enabled: null, TemplateMessages: {}};

		if (!this.supports_integration(this.m_integration_type)) {
			config.Enabled = bool_to_string(false);
			return config;
		}

		config.Enabled = bool_to_string(enabled);
		if (config.Enabled === "1") {
			config.TemplateMessages = this.get_config_list();
		}
		return config;
	};

	/*
		{
			"chatclub_southcone_welcome_v1": {
				"id": "chatclub_southcone_welcome_v1", 
				"groupId": "1e1a3508_c587_e845_d431_b6764aae2e52", 
				"groupName": "whatsapp:hsm:ecommerce:movile", 
				"status": "approved", 
				"available": true, 
				"params": [
					"company_name", 
					"text"
				], 
				"languages": {
					"es": {
						"text": "Hola {{1}} bienvenido a {{2}}, este es nuestro nuevo canal de WhatsApp!"
					}
				}
			}, 
			"chatclub_welcome_redirect": {
				"id": "chatclub_welcome_redirect", 
				"groupId": "1e1a3508_c587_e845_d431_b6764aae2e52", 
				"groupName": "whatsapp:hsm:ecommerce:movile", 
				"status": "approved", 
				"available": true, 
				"params": [ ], 
				"languages": {
					"pt": {
						"text": "Agora você também poderá receber nossas mensagens por aqui."
					}
				}
			}
		}
	*/
	hsm_management.prototype.get_config_list = function()
	{
		let hsm_list = {};

		let items = this.m_grd_hsm_list.get_items();
		if (items && items.length > 0) {
			items.forEach(item => {
				const id = item.id;
				hsm_list[id] = item;
			});
		}

		return hsm_list;
	};

	hsm_management.prototype.attach_data = function( acc )
	{
		if (!acc) {
			return;
		}
		acc.HSMConfig = this.get(true /* keep_old_data */);
		return acc;
	};

	hsm_management.prototype.update_current_hsm_mng = function(enabled)
	{
		this.current_hsm_mng = this.__get(enabled);
	};

	hsm_management.prototype.set_ctrls_codes = function()
	{
		const hsm_mng = this.current_hsm_mng || {};
		this.__fill_grid(hsm_mng.TemplateMessages || {});
	};

	hsm_management.prototype.has_configured_hsm_mng = function(acc)
	{
		if (!acc) {
			return false;
		}
		if (acc.HSMConfig && acc.HSMConfig.Enabled === "1") {
			return true;
		}
		return false;
	};

	hsm_management.prototype.set = function(acc)
	{
		if (!acc) {
			return;
		}
		this.current_hsm_mng = acc.HSMConfig || null;
		const $hsm_mng = $(this.hsm_mng);

		if (!this.has_configured_hsm_mng(acc)) {
			$hsm_mng.prop("checked", false);
			this.disable_hsm_mng(true);
		}
		else {
			$hsm_mng.prop("checked", true);
			this.disable_hsm_mng(false);
		}

		this.set_ctrls_codes();
	};

	hsm_management.prototype.disable_hsm_mng = function(disabled)
	{
		const $grid_container = $("#" + this.prefix_id + "_grid_container");
		const $new = $("#" + this.prefix_id + "_popup_new");
		if (disabled) {
			$grid_container.hide();
			$new.hide();
		}
		else {
			$grid_container.show();
			$new.show();
		}
	};

	hsm_management.prototype.set_default_codes = function()
	{
		$( this.hsm_mng ).prop( "checked", true );
		$( this.hsm_mng ).trigger( "click" );
	};

	hsm_management.prototype.editable = function()
	{
		const self = this;
		var ds_ctrl = function( $ctrl, ds, required ) {
			self.section.frm.disable_control( $ctrl, ds, required );
		};
		const $hsm_mng = $( this.hsm_mng );
		ds_ctrl( $hsm_mng, false );
	};

	hsm_management.prototype.readonly = function()
	{
		const self = this;
		const ds_ctrl = function( $ctrl, ds, required ) {
			self.section.frm.readonly_control( $ctrl, ds, required );
		};
		ds_ctrl( $( this.hsm_mng ), true /* disabled */ );
	};

	hsm_management.prototype.sanitize_id = function( id )
	{
		// remueve el # de jquery
		return id.substring( 1 );
	};

	hsm_management.prototype.show = function()
	{
		$(this.container_id).show();
	};

	hsm_management.prototype.hide = function()
	{
		$(this.container_id).hide();
	};

	hsm_management.prototype.toggle = function()
	{
		if (this.supports_integration(this.m_integration_type)) {
			this.show();
		}
		else {
			this.hide();
		}
	};

	hsm_management.prototype.set_integration_type = function(integration_type)
	{
		this.m_integration_type = (integration_type || "").toUpperCase();
		this.toggle();
	};

	hsm_management.prototype.make_html = function()
	{
		const hsm_mng_id = this.sanitize_id(this.hsm_mng);
		$( this.container_id ).html( "".concat(
		// titulo
			// "<h2 class='formSubTitle'>",
			// 	"<label class='subTitleText'>", "<%=_T('HSM Config')%>", "</label>",
			// 	"<label class='subTitleHelp'>/ ", "<%=_T('You can configure HSM Messages for the available languages.')%>", "</label>",
			// 	"<i id='" + this.prefix_id + "_popup_list_validation' class='formFieldValidation formFieldError' hidden></i>",
			// 	"<input id='" + this.prefix_id + "_popup_new' class='pageButtonGeneric buttonColorBlack internalButtons' type='button' value='<%=_T('New HSM')%>' style='display:none;' />",
			// "</h2>",

		// hsm management ( Enabled / Disabled )
			"<div class='row align-items-center'>",
				"<div class='col-6 py-2'>",
					"<div class='form-group mb-0'>",
						"<div class='form-check'>",
							"<label for='", hsm_mng_id, "' class='form-check-label form-check-label-title'>",
								"<input id='", hsm_mng_id, "' class='form-check-input' type='checkbox' />",
								"<strong>", "<%=_T('Enable HSM Message Configuration')%>", "</strong>", 
							"</label>",
						"</div>",
					"</div>",
				"</div>",
				"<div class='col-6'>",
					"<button id='" + this.prefix_id + "_popup_new' class='pageButtonGeneric buttonColorWhite pageButtonNew' style='display:none;'><i class='far fa-plus'></i>", "<%=_T('New HSM')%>" ,"</button>",
				"</div>",
			"</div>",
			// grid
			"<div id='", this.prefix_id , "_grid_container' style='display:none;'>",
				// Table
				"<div class='row mt-4'>",
					"<div class='col-12'>",
						"<table id='", this.prefix_id , "_list' cellpadding='0' cellspacing='0' border='0' class='viewGrid mt-0'>",
							"<thead class='viewGridHeader'></thead>",
							"<tbody class='viewGridBody'></tbody>",
						"</table>",
						"<div id='", this.prefix_id , "_not_results_found' class='inlineMessagesContainer messageInfo' style='display:none'></div>",
						"<div id='", this.prefix_id , "_paginator'></div>",
					"</div>",
				"</div>",
				// start - edit hsm popup
				"<div id='" + this.prefix_id + "_edit_popup' class='internalPopUpContainer popUp940w formPopUp hsmManagementPopUp' style='display:none;'>",
					"<div class='internalPopUpOverlay'></div>",
					"<div class='internalPopUp'>",
						// waiting
						"<div id='" + this.prefix_id + "_edit_popup_waiting' class='internalWaitingContainer' style='display:none;'>",
							"<div waiting_message='true' class='internalWaitingMessage'></div>",
							"<div class='internalWaitingAnimation'></div>",
							"<div class='internalWaitingOverlay'></div>",
						"</div>",
						// title
						"<div id='" + this.prefix_id + "_edit_popup_title' class='internalPopUpTitle'>",
							"<span popup_title_text>", "<%=_T('Authorize Application')%>", "</span>"  +
							"<button popup_close id='" + this.prefix_id + "_edit_popup_close_times' class='internalPopUpCloseButton'><i class='far fa-times'></i></button>",
						"</div>",
						// content
						"<div class='internalPopUpMiddle pageFormContainer'>",
							// mensaje de error
							"<div id='" + this.prefix_id + "_edit_popup_message' class='genericFormMessagesContainer messageError' style='display:none;'>FIELD ERROR MESSAGE</div>",
							// form
							"<div class='no-card'>",
								"<div class='card-block'>",
									"<div class='row'>",
										"<div class='col-6 d-flex flex-column'>",
											"<div class='form-group'>",
												"<label for='" + this.prefix_id + "_edit_popup_id' class='required'><%=_T('Id')%><%=_T(':')%></label>",
												"<input id='" + this.prefix_id + "_edit_popup_id' class='form-control' type='text' maxlength='50' placeholder='", "<%=_T('Enter the id')%>...", "' />",
												"<i id='" + this.prefix_id + "_edit_popup_id_validation' class='formFieldValidation formFieldError' hidden></i>",
											"</div>",
											"<div class='form-group'>",
												"<label for='" + this.prefix_id + "_edit_popup_group_name' class='required'><%=_T('Namespace')%><%=_T(':')%></label>",
												"<input id='" + this.prefix_id + "_edit_popup_group_name' class='form-control' type='text' maxlength='50' placeholder='", "<%=_T('Enter the namespace')%>...", "' />",
												"<i id='" + this.prefix_id + "_edit_popup_group_name_validation' class='formFieldValidation formFieldError' hidden></i>",
											"</div>",
											"<div class='form-group' title='<%=_T('Select interactive template for call to action or quick reply buttons')%>'>",
												"<label for='" + this.prefix_id + "_edit_popup_type' class='required'><%=_T('Type')%><%=_T(':')%></label>",
												"<select id='" + this.prefix_id + "_edit_popup_type' class='form-control' type='text' maxlength='50'>",
												"</select>",
												"<i id='" + this.prefix_id + "_edit_popup_type_validation' class='formFieldValidation formFieldError' hidden></i>",
											"</div>",
											"<div class='form-group' title='<%=_T('Select buttons type')%>' style='display:none;'>",
												"<label for='" + this.prefix_id + "_edit_popup_button_type' class='required'><%=_T('Buttons')%><%=_T(':')%></label>",
												"<select id='" + this.prefix_id + "_edit_popup_button_type' class='form-control' type='text' maxlength='50'>",
												"</select>",
												"<i id='" + this.prefix_id + "_edit_popup_button_type_validation' class='formFieldValidation formFieldError' hidden></i>",
											"</div>",
										"</div>",
										"<div class='col-6 d-flex flex-column'>",
											"<div class='form-group flex-grow-1'>",
												"<label for='" + this.prefix_id + "_edit_popup_params'><%=_T('Parameters')%><%=_T(':')%> <span class='fa fa-question-circle' title='", "<%=_T('Press enter to add parameters')%>" ,"' > </span></label>",
												"<textarea id='" + this.prefix_id + "_edit_popup_params' class='form-control fullColHeight'></textarea>",
												"<i id='" + this.prefix_id + "_edit_popup_params_validation' class='formFieldValidation formFieldError' hidden></i>",
											"</div>",
										"</div>",
									"</div>",
									"<div class='row'>",
										"<div class='col-12'>",
											"<div class='formTabsMainContainer'>",
												// TABS
												"<div id='", this.prefix_id, "_languages_tabs' class='formFatherFieldset'></div>",
												// TAB ENGLISH CONTAINER
												"<div id='", this.prefix_id, "_languages_english_tab_container' class='formTabsBody' style='display:none;'>",
													"<div class='row'>",
														"<div class='col-6'>",
															"<div class='form-group'>",
																"<div class='form-check'>" ,
																	"<label for='" , this.prefix_id , "_edit_popup_lang_en_selector' class='form-check-label form-check-label-title'>" ,
																		"<input id='" , this.prefix_id , "_edit_popup_lang_en_selector' class='form-check-input' type='checkbox' lang_message_selector='en' related_ctrl='" , this.prefix_id , "_edit_popup_lang_en_message' related_ctrl_code='" , this.prefix_id , "_edit_popup_lang_en_code' />" ,
																		"<%=_T('Enable language')%>", 
																	"</label>" ,
																"</div>" ,
															"</div>",
														"</div>",
													"</div>",
													"<div class='row pt-2'>",
														"<div class='col-6'>",
															"<div class='form-group'>",
																"<label for='" , this.prefix_id , "_edit_popup_lang_en_code' ><%=_T('Code')%><%=_T(':')%></label>",
																"<input id='" , this.prefix_id , "_edit_popup_lang_en_code' class='form-control mt-1' type='text' maxlength='50' placeholder='", "<%=_T('Enter the code')%>..." ,"' />",
															"</div>",
															"<div id='" , this.prefix_id , "_edit_popup_lang_en_code_status_container'", " class='form-group'>",
																"<span class='fa fa-question-circle' title='<%=_T('Only Active language templates can be used')%>'></span>",
																"<label for='" , this.prefix_id , "_edit_popup_lang_en_code_status' ><%=_T('Status')%><%=_T(':')%> </label>",
																"<label id='" , this.prefix_id , "_edit_popup_lang_en_code_status' lang_status_selector='en' ></label>",
															"</div>",															
														"</div>",
														"<div class='col-6'>",
															"<div class='form-group'>",
																"<label for='" , this.prefix_id , "_edit_popup_lang_en_message' ><%=_T('Text')%><%=_T(':')%></label>",
																"<textarea id='" , this.prefix_id , "_edit_popup_lang_en_message' class='form-control rowsBased mt-1' rows='4' type='text'></textarea>",
																"<i id='" , this.prefix_id , "_edit_popup_lang_en_message_validation' class='formFieldValidation formFieldError' hidden></i>",
															"</div>",
														"</div>",
													"</div>",
												"</div>",
												// TAB SPANISH CONTAINER
												"<div id='", this.prefix_id, "_languages_spanish_tab_container' class='formTabsBody' style='display:none;'>",
													"<div class='row'>",
														"<div class='col-6'>",
															"<div class='form-group'>",
																"<div class='form-check'>" ,
																	"<label for='" , this.prefix_id , "_edit_popup_lang_es_selector' class='form-check-label form-check-label-title'>" ,
																		"<input id='" , this.prefix_id , "_edit_popup_lang_es_selector' class='form-check-input' type='checkbox' lang_message_selector='es' related_ctrl='" , this.prefix_id , "_edit_popup_lang_es_message' related_ctrl_code='" , this.prefix_id , "_edit_popup_lang_es_code' />" ,
																		"<%=_T('Enable language')%>", 
																	"</label>" ,
																"</div>" ,
															"</div>",
														"</div>",
													"</div>",
													"<div class='row pt-2'>",
														"<div class='col-6'>",
															"<div class='form-group'>",
																"<label for='" , this.prefix_id , "_edit_popup_lang_es_code' ><%=_T('Code')%><%=_T(':')%></label>",
																"<input id='" , this.prefix_id , "_edit_popup_lang_es_code' class='form-control mt-1' type='text' maxlength='50' placeholder='", "<%=_T('Enter the code')%>..." ,"' />",
															"</div>",
															"<div id='" , this.prefix_id , "_edit_popup_lang_es_code_status_container'", " class='form-group'>",
																"<span class='fa fa-question-circle' title='<%=_T('Only Active language templates can be used')%>'></span>",
																"<label for='" , this.prefix_id , "_edit_popup_lang_es_code_status' ><%=_T('Status')%><%=_T(':')%></label>",
																"<label id='" , this.prefix_id , "_edit_popup_lang_es_code_status' lang_status_selector='es' ></label>",
															"</div>",
														"</div>",
														"<div class='col-6'>",
															"<div class='form-group'>",
																"<label for='" , this.prefix_id , "_edit_popup_lang_es_message' ><%=_T('Text')%><%=_T(':')%></label>",
																"<textarea id='" , this.prefix_id , "_edit_popup_lang_es_message' class='form-control rowsBased' rows='4' type='text'></textarea>",
																"<i id='" , this.prefix_id , "_edit_popup_lang_es_message_validation' class='formFieldValidation formFieldError' hidden></i>",
															"</div>",
														"</div>",
													"</div>",
												"</div>",
												// TAB FRENCH CONTAINER
												"<div id='", this.prefix_id, "_languages_french_tab_container' class='formTabsBody' style='display:none;'>",
													"<div class='row'>",
														"<div class='col-6'>",
															"<div class='form-group'>",
																"<div class='form-check'>" ,
																	"<label for='" , this.prefix_id , "_edit_popup_lang_fr_selector' class='form-check-label form-check-label-title'>" ,
																		"<input id='" , this.prefix_id , "_edit_popup_lang_fr_selector' class='form-check-input' type='checkbox' lang_message_selector='fr' related_ctrl='" , this.prefix_id , "_edit_popup_lang_fr_message' related_ctrl_code='" , this.prefix_id , "_edit_popup_lang_fr_code' />" ,
																		"<%=_T('Enable language')%>", 
																	"</label>" ,
																"</div>" ,
															"</div>",
														"</div>",
													"</div>",
													"<div class='row pt-2'>",
														"<div class='col-6'>",
															"<div class='form-group'>",
																"<label for='" , this.prefix_id , "_edit_popup_lang_fr_code' ><%=_T('Code')%><%=_T(':')%></label>",
																"<input id='" , this.prefix_id , "_edit_popup_lang_fr_code' class='form-control mt-1' type='text' maxlength='50' placeholder='", "<%=_T('Enter the code')%>..." ,"' />",
															"</div>",
															"<div id='" , this.prefix_id , "_edit_popup_lang_fr_code_status_container'", " class='form-group'>",
																"<span class='fa fa-question-circle' title='<%=_T('Only Active language templates can be used')%>'></span>",
																"<label for='" , this.prefix_id , "_edit_popup_lang_fr_code_status' ><%=_T('Status')%><%=_T(':')%></label>",
																"<label id='" , this.prefix_id , "_edit_popup_lang_fr_code_status' lang_status_selector='fr' ></label>",
															"</div>",
														"</div>",
														"<div class='col-6'>",
															"<div class='form-group'>",
																"<label for='" , this.prefix_id , "_edit_popup_lang_fr_message' ><%=_T('Text')%><%=_T(':')%></label>",
																"<textarea id='" , this.prefix_id , "_edit_popup_lang_fr_message' class='form-control rowsBased' rows='4' type='text'></textarea>",
																"<i id='" , this.prefix_id , "_edit_popup_lang_fr_message_validation' class='formFieldValidation formFieldError' hidden></i>",
															"</div>",
														"</div>",
													"</div>",
												"</div>",
												// TAB PORTUGUESE CONTAINER
												"<div id='", this.prefix_id, "_languages_portuguese_tab_container' class='formTabsBody' style='display:none;'>",
													"<div class='row'>",
														"<div class='col-6'>",
															"<div class='form-group'>",
																"<div class='form-check'>" ,
																	"<label for='" , this.prefix_id , "_edit_popup_lang_pt_selector' class='form-check-label form-check-label-title'>" ,
																		"<input id='" , this.prefix_id , "_edit_popup_lang_pt_selector' class='form-check-input' type='checkbox' lang_message_selector='pt' related_ctrl='" , this.prefix_id , "_edit_popup_lang_pt_message' related_ctrl_code='" , this.prefix_id , "_edit_popup_lang_pt_code' />" ,
																		"<%=_T('Enable language')%>", 
																	"</label>" ,
																"</div>" ,
															"</div>",
														"</div>",
													"</div>",
													"<div class='row pt-2'>",
														"<div class='col-6'>",
															"<div class='form-group'>",
																"<label for='" , this.prefix_id , "_edit_popup_lang_pt_code' ><%=_T('Code')%><%=_T(':')%></label>",
																"<input id='" , this.prefix_id , "_edit_popup_lang_pt_code' class='form-control mt-1' type='text' maxlength='50' placeholder='", "<%=_T('Enter the code')%>..." ,"' />",
															"</div>",
															"<div id='" , this.prefix_id , "_edit_popup_lang_pt_code_status_container'", " class='form-group'>",
																"<span class='fa fa-question-circle' title='<%=_T('Only Active language templates can be used')%>'></span>",
																"<label for='" , this.prefix_id , "_edit_popup_lang_pt_code_status' ><%=_T('Status')%><%=_T(':')%></label>",
																"<label id='" , this.prefix_id , "_edit_popup_lang_pt_code_status' lang_status_selector='pt' ></label>",
															"</div>",
														"</div>",
														"<div class='col-6'>",
															"<div class='form-group'>",
																"<label for='" , this.prefix_id , "_edit_popup_lang_pt_message' ><%=_T('Text')%><%=_T(':')%></label>",
																"<textarea id='" , this.prefix_id , "_edit_popup_lang_pt_message' class='form-control rowsBased' rows='4' type='text'></textarea>",
																"<i id='" , this.prefix_id , "_edit_popup_lang_pt_message_validation' class='formFieldValidation formFieldError' hidden></i>",
															"</div>",
														"</div>",
													"</div>",
												"</div>",
											"</div>",
										"</div>",
									"</div>",
									// END TABS
									"<div id='", this.prefix_id, "_media_template_container' class='row' style='display:none;'>",
										"<div class='col-12'>",
											"<div class='card-header'>",
												"<h5>",
													" <%=_T('Media template') %> ",
													"<span class='fa fa-question-circle' title='<%=_T('Add a media url, if required by the HSM')%>'></span>",
												"</h5 > ",
											"</div>",
											"<div class='card-block'>",
												"<div class='row ruleField ruleFieldLimit'>",
													"<div class='col-12'>",
														"<div class='ruleFieldContent'>",
															"<div class='form-group flex-grow-1 mr-2'>",
																"<label><%=_T('Insert media url')%><%=_T(':')%></label>",
																"<input hsm_media_template_value type='text' class='form-control'>",
															"</div>",
														"</div>",
													"</div>",
												"</div>",
												"<div id='" + this.prefix_id + "_media_sharedfiles_selector' class='row ruleField ruleFieldLimit' style='display:none;'>",
													"<div class='col-12'>",
														"<div class='ruleFieldContent'>",
															"<div class='form-group flex-grow-1 mr-2'>",
																"<label for='" + this.prefix_id + "_edit_popup_media_type'><%=_T('You can get an url from a file of type')%><%=_T(':')%></label>",
																"<select id='" + this.prefix_id + "_edit_popup_media_type' class='form-control' type='text' maxlength='50'>",
																"</select>",
																"<i id='" + this.prefix_id + "_edit_popup_media_type_validation' class='formFieldValidation formFieldError' hidden></i>",															
															"</div>",
															"<div class='form-group d-flex align-items-end'>",
																"<button id='", this.prefix_id, "_media_template_getFile' class='pageButtonGeneric buttonColorWhite iconOnlyBtn mr-1' type='button' title='" + "<%=_T('Get url from file')%>" + "'>",
																	"<i class='far fa-paperclip'></i>",
																"</button>",		
																"<button id='", this.prefix_id, "_media_template_delFile' class='pageButtonGeneric buttonColorWhite iconOnlyBtn mr-2' type='button' title='" + "<%=_T('Clean url')%>" + "'>",
																	"<i class='far fa-eraser'></i>",
																"</button>",
															"</div>",
														"</div>",
													"</div>",
												"</div>",
											"</div>",
										"</div>",
									"</div>",
									"<div id='", this.prefix_id, "_interactive_template_container' style='display:none;'>",
										"<div id='", this.prefix_id, "_call_to_action_buttons_container' class='row' style='display:none;'>",
											"<div class='col-12'>",
												"<div class='card-header'>",
													"<h5>",
														" <%=_T('Call to action buttons') %> ",
														"<span class='fa fa-question-circle' title='<%=_T('Add up to 1 call to action buttons if required by the HSM')%>'></span>",
													"</h5 > ",
												"</div>",
												"<div class='card-block'>",
													"<ul id='" , this.prefix_id , "_cta_buttons' class='hsmManagementButtonsList'>",
													"</ul>",
												"</div>",
											"</div>",
										"</div>",
										"<div id='", this.prefix_id, "_quick_reply_buttons_container' class='row' style='display:none;'>",
											"<div class='col-12'>",
												"<div class='card-header'>",
													"<h5>",
														" <%=_T('Quick reply buttons') %> ",
														"<span class='fa fa-question-circle' title='<%=_T('Add up to 3 quick reply buttons if required by the HSM')%>'></span>",
													"</h5 > ",
												"</div>",
												"<div class='card-block'>",
													"<ul id='" , this.prefix_id , "_qr_buttons' class='hsmManagementButtonsList'>",
													"</ul>",
												"</div>",
											"</div>",
										"</div>",
									"</div>",
									//END BUTTONS
								"</div>",
							"</div>",
						"</div>",
						// MODAL FOOTER BUTTONS
						"<div class='internalPopUpButtons'>",
							"<button id='" , this.prefix_id , "_edit_popup_close' class='pageButtonGeneric buttonColorWhite float-right'>" , "<%=_T('Cancel')%>" , "</button>" ,
							"<button id='" , this.prefix_id , "_edit_popup_create_and_continue' class='pageButtonGeneric buttonColorBlue float-right'>" , "<%=_T('Confirm & Continue')%>" , "</button>" ,
							"<button id='" , this.prefix_id , "_edit_popup_create' class='pageButtonGeneric buttonColorBlue float-right'>" , "<%=_T('Confirm')%>" , "</button>" ,
						"</div>" +
					"</div>",
				"</div>",
				// end - edit hsm popup
			"</div>",
		// disclaimer
		"<div class='row'>" ,
				"<div class='col-12'>" ,
					"<div style='display:none' class='inlineMessagesContainer messageInfo'>", "<%=_T('DISCLAIMER MESSAGE !!!.')%>", "</div>",
				"</div>",
		"</div>",
		));
	};

	hsm_management.prototype.set_events = function()
	{
		const self = this;

		if ( this.section.is_readonly() ) {
			return;
		}

		$(this.hsm_mng)
			.off("change")
			.on("change", function() {
				// Para cloud API se puede solicitar una URL publica de un file de nuestro repo. 
				//	Esa URL contiene el phoneId seleccionado. En una cuenta nueva se podria modificar el phone seleccionado DESPUES de haber generado URLs publicas, lo que llevaria a confusion al usuario
				if(self.parent.constructor.name === "WHATSAPPAccountPanel" && self.m_integration_type === "ICC_CLOUD_API" && self.parent.is_new()) {
					self.parent.show_validation_error( $(self.parent.inconcertCloudApiWaba), "<%=_T('Must save account first.')%>");
					$(this).prop("checked", false);
					return;
				}
				const disabled = !$(this).prop("checked");
				if (!disabled) {
					if (self.current_hsm_mng) {
						self.set_ctrls_codes();
					}
				}
				else {
				// -actualizo la configuracion del current hsm_mng cuando estoy editando una account existente.
				// -siempre se tiene que marcar como "enabled" para que los valores puedan ser cacheados como estaban
					self.update_current_hsm_mng(true);
				}
				self.disable_hsm_mng(disabled);
			}
		);

		const $container = $(this.container_id);
		let event_selector = "[lang_message_selector]";
		$container
			.off("change", event_selector)
			.on("change", event_selector, function() {
				const $chk = $(this);
				self.disable_chk_txt_codes($chk);
			}
		);
	};

	hsm_management.prototype.supports_integration = function(integration_type)
	{
		const supported_integrations = { WEB : false };
		return supported_integrations[integration_type] !== false;
	};

	hsm_management.prototype.disable_chk_txt_codes = function($chk)
	{
		let disabled, is_required;
		const checked = $chk.prop("checked");
		const $value = $("#" + $chk.attr("related_ctrl"));
		const $code = $("#" + $chk.attr("related_ctrl_code"));
		const lang_code = $chk.attr("lang_message_selector");

		if (checked) {
			disabled = false;
			is_required = true;
			$code.val(lang_code);
		}
		else {
			disabled = true;
			is_required = false;
			$value.val("");
			$code.val("");
		}
		this.frm.disable_control($value, disabled, is_required);
		this.frm.disable_control($code, disabled, false);
		$value.focus();
	};

	hsm_management.prototype.validate_required = function()
	{
		let response = {
			success: true
		};
		if (!$(this.hsm_mng).prop("checked")) {
			return response;
		}
		let success = this.m_grd_hsm_list.start_validation().success;
		if (!success) {
			response.success = false;
			response.message = this.m_grd_hsm_list.get_validation_message_err("required");
		}
		return response;
	};

	hsm_management.prototype.$getMediaUrlButton = function() {
		return $("#" + this.prefix_id + "_media_template_getFile");
	};

	hsm_management.prototype.$delMediaUrlButton = function() {
		return $("#" + this.prefix_id + "_media_template_delFile");
	};

	hsm_management.prototype.show_popup_shared_file = function($parent, _, item)
	{
		const self = this;
		if (self.parent.constructor.name !== "WHATSAPPAccountPanel" || self.m_integration_type !== "ICC_CLOUD_API") {
			return;
		}

		const media_type = $("#" + this.prefix_id + "_edit_popup_media_type").val();
		const media_type_text = $("#" + this.prefix_id + "_edit_popup_media_type option:selected").text();
		this.m_sharedFilesPopupList = null;
		this.m_sharedFilesPopupList = new inConcert.SharedFilesPopupList({
			from_app            : "administration",
			owner               : this,
			popupId             : "adm_form_campaign_accounts_messages_shared_file_popup_container",
			search_files_type_id: media_type,
			abm_files_type_id   : media_type,
			visibility          : {
				AgentVisible          : true,
				AccountMessagesVisible: false,
				FlowDesignerVisible   : true
			},
			callbacks: {
				on_show: function() {},
				on_close: function(selected_file) {
					if (!selected_file) {
						// Si cerro el popup por cancelar
						return;
					}
					const $media = $("#" + self.prefix_id + "_media_template_container").find("[hsm_media_template_value]");
					// Si cerro el popup por guardar
					let allowedContentTypes = {
						"image/png" : true,
						"image/jpeg" : true,
						"video/mp4" : true,
						"application/pdf" : true
					};
					let errorMsg;
					const show_shared_file_error = function() {
						self.show_ctrl_err({
							success: false, 
							$ctrl: $media,
							item: null,
							msg: errorMsg
						});
						$media.val( "");
						self.section.frm.readonly_control( $media, false, false );
						self.m_mediaUrlFromRepository = false;
						self.m_mediaUrlFileId = null;
						self.m_mediaFileName = null;
						self.m_mediaContentType = null;
						self.m_mediaUrlType = null;
						$("#" + self.prefix_id + "_edit_popup").find(".internalPopUpMiddle").animate({scrollTop: 0}, 250, "linear");
					};
					if(!allowedContentTypes[selected_file.ContentType]) {
						errorMsg = format("<%=_T('Invalid media: {0}. Valid formats for {1} type: {2}')%>", selected_file.ContentType, _T(media_type_text), inConcert.FileUploaderTypes[media_type].extensions);
						show_shared_file_error();
						return;
					}
					// Aqui es donde hay que llamar al server para que asocie una URL publica al shared file y nos la devuelva
					let campaign = self.parent.parent.frm.get_entity_instance();
					inConcert.invoke({
						url: inConcert.app.getBaseDomain() + "/inconcert/apps/administration/campaigns/whatsapp_get_cloud_api_public_repository_file_url/",
						type: "POST",
						data: JSON.stringify({
							campaignId : campaign.Id,
							accountId : (self.parent.current_account && self.parent.current_account.id) ? self.parent.current_account.id : self.parent.m_authorizingId,
							fileId: selected_file.RepositoryId,
							fileName: selected_file.Name
						}),
						dataType: "json",
						contentType: "application/json",
						success: function(response) {
							if(response.result && response.result === "OK") {
								$media.val( response.publicFileUrl || "");
								self.section.frm.readonly_control( $media, true, false );
								self.m_mediaUrlFromRepository = true;
								self.m_mediaUrlFileId = selected_file.RepositoryId;
								self.m_mediaFileName = selected_file.Name;
								self.m_mediaContentType = selected_file.ContentType;
								self.m_mediaUrlType = media_type;
							}
							else {
								errorMsg = response.reason && _T(response.reason) || "<%=_T('Unknown error')%>";
								show_shared_file_error();
							}
						},
						error: function(_, text_status) {
							errorMsg = text_status.reason && _T(text_status.reason) || "<%=_T('Unknown error')%>";
							show_shared_file_error();
						}
					});
				}
			}
		});

		const media_type_options = {
			whatsapp_cloud_image : "image",
			whatsapp_cloud_video : "video",
			whatsapp_cloud_document : "file"
		};
		this.m_sharedFilesPopupList.show({
			parentNode: $parent,
			option_id : media_type_options[media_type],
			can_create_new_file: true
		});
		if (item) {
			let file = inConcert.Utils.cloneObject(item);
			$.extend(true, file, item.RawData || {});
			delete file.RawData;
			this.m_sharedFilesPopupList.edit_file(file);
		}
	};	

	hsm_management.prototype.toggle_popup_media_from_shared_file = function()
	{
		const $type = $("#" + this.prefix_id + "_edit_popup_type");
		const show_media_from_shared_file = $type.val() === "media_simple" && this.parent.constructor.name === "WHATSAPPAccountPanel" && this.m_integration_type === "ICC_CLOUD_API";
		const $mediaSharedfileSelector = $("#" + this.prefix_id + "_media_sharedfiles_selector");
		$mediaSharedfileSelector.toggle(show_media_from_shared_file);
	};

	hsm_management.prototype.get_grid_value = function(item, field_id)
	{
		const default_status = "<%=_T('N/A')%>";
		const value = {
			original_value: default_status,
			value         : default_status,
			search_value  : default_status
		};

		const item_value = item.languages && item.languages[field_id] && item.languages[field_id].status;
		if (item_value === undefined) {
			return value;
		}

		value.original_value = _T(item_value);
		value.value = _T(item_value);
		value.search_value = _T(item_value);

		return value;
	};
}());
