/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, -W083, esversion: 11 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

// try improve namespacing a bit
if ( !this.inConcert.Administration ) {
	this.inConcert.Administration = {};
}

if ( !this.inConcert.Administration.Campaigns ) {
	this.inConcert.Administration.Campaigns = {};
}

(function () {

//	"use strict";

	const format = inConcert.Utils.format;
	const _T = inConcert.i18n.getString;
	const ERROR_TIMEOUT_MS = 5000;
	const ERROR_TIMEOUT_EXTENDED_MS = 30000;
	const AUTHENTICATION_DEFAULT_MESSAGE = {
		en: "Your verification code is *{{1}}*.",
		es: "Su código de verificación es *{{1}}*.",
		fr: "Votre code de vérification est *{{1}}*.",
		pt: "Seu código de verificação é *{{1}}*."
	};
	const AUTHENTICATION_DEFAULT_SECURITY_RECOMENDATION_TEXT = {
		en: " For your safety, please do not share it.",
		es: " Por su seguridad, por favor no lo comparta.",
		fr: " Pour votre sécurité, merci de ne pas le partager.",
		pt: " Para sua segurança, não compartilhe."
	};
	const AUTHENTICATION_DEFAULT_CODE_EXPIRATION_MINUTES = 10;
	const AUTHENTICATION_DEFAULT_FOOTER_TEXT = {
		en: "This code expires in {0} minutes.",
		es: "Este código caduca en {0} minutos.",
		fr: "Ce code expire dans {0} minutes.",
		pt: "Este código expira em {0} minutos."
	};
	const AUTHENTICATION_DEFAULT_ONE_TAP_TEXT = {
		en: "Autofill",
		es: "Autocompletar",
		fr: "Remplissage automatique",
		pt: "Preenchimento automático"
	};

	const bool_to_string  = function(enabled) {
		return Number(enabled).toString();
	};

	//---------------------------------------------------------------------
	//hsm Buttons
	function hsm_management_cloud_api_button( prefix_id, frm) {
		this.prefix_id = prefix_id;
		this.frm = frm;
		this.buttons_counter = 0;
	}

	function showWaiting(waitingContainerId, show, message) {
		let $loading = $(waitingContainerId);

		if(show) {
			$loading
				.show()
				.find("[waiting_message]")
				.text(message);
		}
		else {
			$loading.hide();
		}
	}

	hsm_management_cloud_api_button.prototype.__makeButton = function (hsmButton, templateCategory, lang_code) {
		const self = this;
		this.buttons_counter = this.buttons_counter + 1;
		var $template = hsmButton && hsmButton.type === "QUICK_REPLY" ? $(this.__makeQuickReplyButtontemplate()) : $(this.__makeCallToActionButtontemplate());

		$template.find("[hsm_btn_text]").val(hsmButton.text || "");

		const btn_action_types = [];
		if(templateCategory === "AUTHENTICATION") {
			btn_action_types.push({ id: "COPY_CODE", label: "<%=_T('Copy code')%>"   });
			btn_action_types.push({ id: "ONE_TAP", label: "<%=_T('Autofill')%>" });

			$template.find("[hsm_btn_text_disclaimer]").show();
		}
		else {
			btn_action_types.push({ id: "PHONE_NUMBER", label: "<%=_T('Call to phone')%>"   });
			btn_action_types.push({ id: "URL", label: "<%=_T('Navigate to')%>" });

			$template.find("[hsm_btn_text_disclaimer]").hide();
		}

		if(hsmButton.type === "URL" || hsmButton.type === "PHONE_NUMBER") {			
			this.frm.fill_combo($template.find("[hsm_btn_action_type]"), false, btn_action_types, "id", "label");

			$template.find("[hsm_btn_action_type]")
				.off("change")
				.on("change", function () {
				const action_type = $(this).val();
				let isOneTap = action_type === "ONE_TAP";
				$template.find("[hsmButton_add]").toggle(templateCategory !== "AUTHENTICATION");
				$template.find("[hsmButton_remove]").toggle(templateCategory !== "AUTHENTICATION");
				$template.find("[hsm_btn_url]")
					.val(hsmButton.url || (templateCategory === "AUTHENTICATION" ? format("https://www.whatsapp.com/otp/code/?otp_type={0}", action_type) + (isOneTap ? "&cta_display_name=" + ($template.find("[hsm_btn_autofill_text]").val() || AUTHENTICATION_DEFAULT_ONE_TAP_TEXT[lang_code]) + "&package_name=" + $template.find("[hsm_btn_package_name]").val() + "&signature_hash=" + $template.find("[hsm_btn_signature_hash]").val() : "") + "&code=otp{{1}}" : ""))
					.parent()
					.toggle(action_type === "URL" && templateCategory !== "AUTHENTICATION");
				$template.find("[hsm_btn_phone]")
					.val(hsmButton.type === "PHONE_NUMBER" ? hsmButton.phone_number || "" : "")
					.parent()
					.toggle(action_type === "PHONE_NUMBER");
				$template.find("[hsm_btn_autofill_text]")
					.val(isOneTap ? hsmButton.autofill_text || inConcert.Utils.getUrlParameter(hsmButton.url, "cta_display_name") || "" : "")
					.prop("disabled", !isOneTap)
					.parent()
					.toggle(templateCategory === "AUTHENTICATION");
				$template.find("[hsm_btn_package_name]")
					.val(isOneTap ? hsmButton.package_name || inConcert.Utils.getUrlParameter(hsmButton.url, "package_name") || "" : "")
					.prop("disabled", !isOneTap)
					.parent()
					.toggle(templateCategory === "AUTHENTICATION");			
				$template.find("[hsm_btn_signature_hash]")
					.val(isOneTap ? hsmButton.signature_hash || inConcert.Utils.getUrlParameter(hsmButton.url, "signature_hash") || "" : "")
					.prop("disabled", !isOneTap)
					.parent()
					.toggle(templateCategory === "AUTHENTICATION");
			});

			$template.find("[hsm_btn_autofill_text]")
				.off("change")
				.on("change", function () {
					const action_type = $template.find("[hsm_btn_action_type]").val();
					let isOneTap = action_type === "ONE_TAP";
					$template.find("[hsm_btn_url]")
						.val(templateCategory === "AUTHENTICATION" ? format("https://www.whatsapp.com/otp/code/?otp_type={0}", action_type) + (isOneTap ? "&cta_display_name=" + ($(this).val() || AUTHENTICATION_DEFAULT_ONE_TAP_TEXT[lang_code]) + "&package_name=" + $template.find("[hsm_btn_package_name]").val() + "&signature_hash=" + $template.find("[hsm_btn_signature_hash]").val() : "") + "&code=otp{{1}}" : "");
			});

			$template.find("[hsm_btn_package_name]")
				.off("change")
				.on("change", function () {
					const action_type = $template.find("[hsm_btn_action_type]").val();
					let isOneTap = action_type === "ONE_TAP";
					$template.find("[hsm_btn_url]")
						.val(templateCategory === "AUTHENTICATION" ? format("https://www.whatsapp.com/otp/code/?otp_type={0}", action_type) + (isOneTap ? "&cta_display_name=" + ($template.find("[hsm_btn_autofill_text]").val() || AUTHENTICATION_DEFAULT_ONE_TAP_TEXT[lang_code]) + "&package_name=" + $(this).val() + "&signature_hash=" + $template.find("[hsm_btn_signature_hash]").val() : "") + "&code=otp{{1}}" : "");
			});

			$template.find("[hsm_btn_signature_hash]")
				.off("change")
				.on("change", function () {
					const action_type = $template.find("[hsm_btn_action_type]").val();
					let isOneTap = action_type === "ONE_TAP";
					$template.find("[hsm_btn_url]")
						.val(templateCategory === "AUTHENTICATION" ? format("https://www.whatsapp.com/otp/code/?otp_type={0}", action_type) + (isOneTap ? "&cta_display_name=" + ($template.find("[hsm_btn_autofill_text]").val() || AUTHENTICATION_DEFAULT_ONE_TAP_TEXT[lang_code]) + "&package_name=" + $template.find("[hsm_btn_package_name]").val() + "&signature_hash=" + $(this).val() : "") + "&code=otp{{1}}" : "");
			});
			
			$template.find("[hsm_btn_action_type]")
				.val(templateCategory === "AUTHENTICATION" ? hsmButton.otp_type || inConcert.Utils.getUrlParameter(hsmButton.url, "otp_type") || "ONE_TAP" : hsmButton.type)
				.change();
		}
		else {
			$template.find("[hsm_btn_url]").val("").parent().hide();
			$template.find("[hsm_btn_package_name]").val("").hide();
			$template.find("[hsm_btn_signature_hash]").val("").hide();
			$template.find("[hsm_btn_phone]").val("").parent().hide();
			$template.find("[hsmButton_add]").parent().show();
			$template.find("[hsmButton_remove]").parent().show();
		}
		
		$template.find("[hsmButton_add]").click(function () {
			var $row = $template;
			// Up to 3 quick replay buttons and upto 2 call_to_action buttons
			let max_buttons = hsmButton.type === "QUICK_REPLY" ? 3 : 2;

			// If new button exceedes maximum, does not create it
			if((self.buttons_counter + 1) > max_buttons) {
				return;	
			}

			if ($row.find("[hsm_btn_text]").val().trim().length === 0) {
				$row.find("[hsm_btn_text]").focus();
				return;
			}
				
			if(hsmButton.type === "PHONE_NUMBER" && $row.find("[hsm_btn_phone]").val().trim().length === 0) {
				$row.find("[hsm_btn_phone]").focus();
				return;
			}
				
			if(hsmButton.type === "URL" && $row.find("[hsm_btn_url]").val().trim().length === 0) {
				$row.find("[hsm_btn_url]").focus();
				return;
			}

			const new_button = {
				type: hsmButton.type === "QUICK_REPLY" ? "QUICK_REPLY" : (hsmButton.type === "URL" ? "PHONE_NUMBER": "URL")
			};				
			var $newRow = self.__makeButton(new_button);
			$row.after($newRow);
			$row.parent().scrollTo($newRow);

			self.frm.readonly_control( $row.parent().find("[hsm_btn_action_type]"), $row.siblings().length > 0, false );

			$newRow.find("[hsm_btn_text]").focus();		
		});

		$template.find("[hsmButton_remove]").click(function() {
			var $row = $template;
			var $siblings = $row.siblings();
			if ($siblings.length) {
				if ($row.prev().length) {
					$row.prev().find("[hsm_btn_text]").focus();
				}
				else {
					$row.next().find("[hsm_btn_text]").focus();
				}
				self.frm.readonly_control( $row.siblings().find("[hsm_btn_action_type]"), $row.siblings().length === 0, false );
				$row.remove();
				self.buttons_counter = self.buttons_counter - 1;
			}
			else {
				$row.find("[hsm_btn_text]").val("");
				$row.find("[hsm_btn_url]").val("");
				$row.find("[hsm_btn_phone]").val("");
				$row.find("[hsm_btn_text]").focus();
			}
		});

		return $template;
	};

	hsm_management_cloud_api_button.prototype.__getDataButtons = function (lang_code) {
		const self = this;
		let data = [];
		let $buttons_type = $("#" + self.prefix_id + "_edit_popup_lang_" + lang_code + "_button_type");
		let buttons_type = $buttons_type.val();
		let button_list_id = this.prefix_id + "_edit_popup_lang_" + lang_code + "_" + buttons_type + "_buttons";
		$("#" + button_list_id + " li").each(function () {
			const $this = $(this);
			let button = {};
			button.type = buttons_type === "quick_reply" ? "QUICK_REPLY" : $this.find("[hsm_btn_action_type]").val();
			button.textCtrl = $this.find("[hsm_btn_text]");
			button.text = button.textCtrl.val();
			if(button.type === "URL") {
				button.urlCtrl = $this.find("[hsm_btn_url]");
				button.url = button.urlCtrl.val();
			}
			else if(button.type === "PHONE_NUMBER") {
				button.phone_numberCtrl = $this.find("[hsm_btn_phone]");
				button.phone_number = button.phone_numberCtrl.val();
			}
			else if(button.type === "COPY_CODE") {
				button.url = $this.find("[hsm_btn_url]").val();
				button.autofill_textCtrl = $this.find("[hsm_btn_autofill_text]");
				button.autofill_text = button.autofill_textCtrl.val();
			}
			else if(button.type === "ONE_TAP") {
				button.package_nameCtrl = $this.find("[hsm_btn_package_name]");
				button.package_name = button.package_nameCtrl.val();
				button.signature_hashCtrl = $this.find("[hsm_btn_signature_hash]");
				button.signature_hash = button.signature_hashCtrl.val();
				button.url = $this.find("[hsm_btn_url]").val();
				button.autofill_textCtrl = $this.find("[hsm_btn_autofill_text]");
				button.autofill_text = button.autofill_textCtrl.val();
			}
			
			data.push(button);
		});
		return data;
	};

	hsm_management_cloud_api_button.prototype.__deleteButtons = function ($buttons) {
		$buttons.html("");
	};

	hsm_management_cloud_api_button.prototype.__makeButtons = function(data) {
		const self = this;
		let buttons = data.buttons;	
		var $buttons = data.button_list;
		this.__deleteButtons($buttons);
		var $list = [];

		$.each(buttons, function(_, button) {
			$list.push(self.__makeButton(button, data.templateCategory, data.lang_code));
		});
		
		if (!$list.length) {
			$list.push(self.__makeButton(undefined, data.templateCategory, data.lang_code));
		}

		$buttons.append($list);

		let disable_hsm_btn_action_type = $buttons.find("[hsm_btn_action_type]").length > 1;
		$buttons.find("[hsm_btn_action_type]").each(function() {
			self.frm.readonly_control($( this ), disable_hsm_btn_action_type, false );
		});
		//self.frm.readonly_control( $buttons.find("[hsm_btn_action_type]"), $buttons.find("[hsm_btn_action_type]").length > 1, false );
	};

	hsm_management_cloud_api_button.prototype.__makeQuickReplyButtontemplate = function () {
		return "".concat(
			"<li hsmButton class='row ruleField ruleFieldLimit'>",
				"<div class='col-12'>",
					"<div class='ruleFieldContent'>",
						"<div class='form-group flex-grow-1 position-relative'>",
							"<label class='required'>", "<%=_T('Text')%>", "<%=_T(':')%></label>",
							"<input id='hsm_qr_btn_text_", this.buttons_counter, "' hsm_btn_text type='text' maxlength='25' class='form-control'>",
							"<i id='hsm_qr_btn_text_", this.buttons_counter, "_validation' class='formFieldValidation formFieldError' hidden></i>",
						"</div>",
						"<div class='form-group d-flex align-items-end'>",
							"<button hsmButton_remove class='pageButtonGeneric ruleFieldDelete' type='button'>",
								"<i class='far fa-minus-circle'></i>",
							"</button>",
							"<button hsmButton_add class='pageButtonGeneric ruleFieldAdd ml-1' type='button'>",
								"<i class='far fa-plus'></i>",
							"</button>",
						"</div>",
					"</div>",
				"</div>",
			"</li>");
	};

	hsm_management_cloud_api_button.prototype.__makeCallToActionButtontemplate = function () {
		return "".concat(
			"<li hsmButton class='row ruleField ruleFieldLimit'>",
				"<div class='col-12'>",
					"<div class='ruleFieldContent'>",
						"<div class='form-group flex-grow-1'>",
							"<label class='required'>", "<%=_T('Action type')%>", "<%=_T(':')%></label>",
							"<select hsm_btn_action_type class='form-control'></select>",
						"</div>",
						"<div class='form-group flex-grow-1 position-relative'>",
							"<label class='required'>", "<%=_T('Text')%>", "<%=_T(':')%>",
								"<span class='fa fa-question-circle' title='", "<%=_T('Enter the text for the copy button.')%>", "' hsm_btn_text_disclaimer ></span>",
							"</label>",
							"<input id='hsm_cta_btn_text_", this.buttons_counter, "' hsm_btn_text type='text' maxlength='25' class='form-control'>",
							"<i id='hsm_cta_btn_text_", this.buttons_counter, "_validation' class='formFieldValidation formFieldError' hidden></i>",
						"</div>",
						"<div class='form-group flex-grow-1 position-relative'>",
							"<label class='required'>", "<%=_T('Autofill Text')%>", "<%=_T(':')%>",
								"<span class='fa fa-question-circle' title='", "<%=_T('Enter the text for the autofill button that will be displayed on Android devices that have the package correctly installed. On devices that are not Android or do not have the package installed, the copy button will be displayed instead.')%>", "'></span>",
							"</label>",
							"<input id='hsm_cta_btn_autofill_text_", this.buttons_counter, "' hsm_btn_autofill_text type='text' maxlength='25' class='form-control'>",
							"<i id='hsm_cta_btn_autofill_text_", this.buttons_counter, "_validation' class='formFieldValidation formFieldError' hidden></i>",
						"</div>",
						"<div class='form-group flex-grow-1 position-relative' style='display:none;'>",
							"<label class='required'>", "<%=_T('Phone number')%>", "<%=_T(':')%>",
								"<span class='fa fa-question-circle' title='", "<%=_T('Enter a full phone number, including the + sign and the country code.')%>", "'></span>",
							"</label>",
							"<input id='hsm_btn_phone_", this.buttons_counter, "' hsm_btn_phone type='text' maxlength='25' class='form-control'>",
							"<i id='hsm_btn_phone_", this.buttons_counter, "_validation' class='formFieldValidation formFieldError' hidden></i>",
						"</div>",
						"<div class='form-group flex-grow-1 position-relative' style='display:none;'>",
							"<label class='required'>", "<%=_T('URL')%>", "<%=_T(':')%>",
								"<span class='fa fa-question-circle' title='", "<%=_T('By adding {{1}} to the end of the url you can create a custom link.')%>", "'></span>",
							"</label>",
							"<input id='hsm_btn_url_", this.buttons_counter, "' hsm_btn_url type='text' maxlength='2000' class='form-control'>",
							"<i id='hsm_btn_url_", this.buttons_counter, "_validation' class='formFieldValidation formFieldError' hidden></i>",
						"</div>",
						"<div class='form-group flex-grow-1 position-relative' style='display:none;'>",
							"<label class='required'>", "<%=_T('Package name')%>", "<%=_T(':')%>",
								"<span class='fa fa-question-circle' title='", "<%=_T('Enter the full package name of the Android application to which autofill button will send the code.')%>", "'></span>",
							"</label>",
							"<input id='hsm_btn_package_name_", this.buttons_counter, "' hsm_btn_package_name type='text' maxlength='2000' class='form-control'>",
							"<i id='hsm_btn_package_name_", this.buttons_counter, "_validation' class='formFieldValidation formFieldError' hidden></i>",
						"</div>",
						"<div class='form-group flex-grow-1 position-relative' style='display:none;'>",
							"<label class='required'>", "<%=_T('Application signature')%>", "<%=_T(':')%>",
								"<span class='fa fa-question-circle' title='", "<%=_T('Enter the application signature hash of the Android application to which autofill button will send the code.')%>", "'></span>",
							"</label>",
							"<input id='hsm_btn_signature_hash_", this.buttons_counter, "' hsm_btn_signature_hash type='text' maxlength='2000' class='form-control'>",
							"<i id='hsm_btn_signature_hash_", this.buttons_counter, "_validation' class='formFieldValidation formFieldError' hidden></i>",
						"</div>",
						"<div class='form-group d-flex align-items-end'>",
							"<button hsmButton_remove class='pageButtonGeneric ruleFieldDelete' type='button'>",
								"<i class='far fa-minus-circle'></i>",
							"</button>",
							"<button hsmButton_add class='pageButtonGeneric ruleFieldAdd ml-1' type='button'>",
								"<i class='far fa-plus'></i>",
							"</button>",
						"</div>",
					"</div>",
				"</div>",
			"</li>");
	};

	//---------------------------------------------------------------------
	// hsm management
	function hsm_management_cloud_api(params)
	{
		params = params || {};
		/* jshint validthis: true*/
		this.parent = params.parent;
		this.section = this.parent.parent;
		this.frm = this.section.frm;

		this.prefix_id = params.prefix_id; // ejemplo de prefijo: adm_form_campaign_accounts_hsm_form, etc

		this.current_hsm_mng = null;

		// container
		this.container_id = "#" + params.container_id;
		// enable/disable
		this.hsm_mng = "#" + this.prefix_id + "_hsm_mng";
		this.m_integration_type = null;
		this.m_grd_hsm_list = null;
		this.rows_per_page = 10;
		this.show_at_least = this.rows_per_page - 1;
		this.m_is_new_hsm = null;
		this.m_current_hsm = null;
		this.waba_templates = [];
		this.useSelectedTemplate = false;

		// Instancia de la clase los TABs
		this.main_tabs = new inConcert.conf_form_tab({
			parent        : this,
			frm           : this.frm,
			container_id  : this.prefix_id + "_languages_tabs",
			must_intialize: false,
			tabs_list   : {
				en: {
					id              : "en",
					name            : "<%=_T('English')%>",
					tab_container_id: this.prefix_id + "_languages_english_tab_container",
					selected        : true
				},
				es: {
					id              : "es",
					name            : "<%=_T('Spanish')%>",
					tab_container_id: this.prefix_id + "_languages_spanish_tab_container"
				},
				fr: {
					id              : "fr",
					name            : "<%=_T('French')%>",
					tab_container_id: this.prefix_id + "_languages_french_tab_container"
				},
				pt: {
					id              : "pt",
					name            : "<%=_T('Portuguese')%>",
					tab_container_id: this.prefix_id + "_languages_portuguese_tab_container"
				}
			}
		});
		this.hsmButtons = {};
		this.hsmButtons.en = new hsm_management_cloud_api_button(this.prefix_id, this.frm);
		this.hsmButtons.es = new hsm_management_cloud_api_button(this.prefix_id, this.frm);
		this.hsmButtons.fr = new hsm_management_cloud_api_button(this.prefix_id, this.frm);
		this.hsmButtons.pt = new hsm_management_cloud_api_button(this.prefix_id, this.frm);

		return true;
	}

	if (typeof inConcert.Administration.Campaigns.HSM_Management_cloud_api !== "function") {
		inConcert.Administration.Campaigns.HSM_Management_cloud_api = hsm_management_cloud_api;
	}

	hsm_management_cloud_api.prototype.initialize = function()
	{
		this.current_hsm_mng = null;
		this.make_html();
		this.__initialize_grid();
		this.main_tabs.initialize();
		this.main_tabs.show_tabs_container();
		this.set_events();
		this.set_default_codes();
	};

	hsm_management_cloud_api.prototype.__initialize_grid = function()
	{
		const self = this;

		const container_id = this.sanitize_id(this.container_id);

		this.m_grd_hsm_list = new inConcert.conf_form_paginator({
			paginator_name: "HSM_LIST",
			owner: this,
			frm: this.frm,
			auto_initialize: true,
			row_id: "id",
			display_name: "id",
			rows_per_page: this.rows_per_page,
			show_at_least: this.show_at_least,
			paginator_type: "ALL_ROWS_CLIENT_SIDE",
			add_with_enter: false,
			ids: {
				grid: this.prefix_id + "_list",
				paginator: this.prefix_id + "_paginator",
				not_results_found: this.prefix_id + "_not_results_found",
				container_new_btn: container_id,
				new_item: this.prefix_id + "_popup_new",
				//popup0 (add hsm)
				popup: this.prefix_id + "_edit_popup",
				popup_msg: this.prefix_id + "_edit_popup_message",
				popup_create: this.prefix_id + "_edit_popup_create",
				popup_create_and_continue: this.prefix_id + "_edit_popup_create_and_continue",
				popup_title: this.prefix_id + "_edit_popup_title",
				popup_close: this.prefix_id + "_edit_popup_close",
				popup_close_times: this.prefix_id + "_edit_popup_close_times",
				popup_waiting: this.prefix_id + "_edit_popup_waiting"
			},
			strings: {
				//popup0 (add hsm)
				not_results_found         : "<%=_T('No HSM Messages Found.')%>",
				can_remove                : "<%=_T('Do you want to delete HSM Message: {0} ?')%>",
				title_new                 : "<%=_T('New HSM')%>",
				title_update              : "<%=_T('Edit HSM')%>",
				button_create             : "<%=_T('Add')%>",
				button_create_and_continue: "<%=_T('Add & Continue')%>"
			},
			columns: [
				{id: "_number"  , name: "", type: "counter"},
				{id: "id"        , name: "<%=_T('Id')%>"},
				{id: "category"  , name: "<%=_T('Category')%>", get_value: function(item) {
					let category_found = categories.find(category => category.id === item.category);
					let value = {
						original_value: item.category,
						value         : category_found ? category_found.label : item.category,
						search_value  : item.category
					};

					return value;
				}},
				{id: "en" , name: "<%=_T('English')%>", get_value: function(item) {
					return self.get_grid_value(item, "en");
				}},
				{id: "es" , name: "<%=_T('Spanish')%>", get_value: function(item) {
					return self.get_grid_value(item, "es");
				}},
				{id: "fr" , name: "<%=_T('French')%>", get_value: function(item) {
					return self.get_grid_value(item, "fr");
				}},
				{id: "pt" , name: "<%=_T('Portuguese')%>", get_value: function(item) {
					return self.get_grid_value(item, "pt");
				}},					
				{
					id: "_action",
					name: "",
					system_action: {
						edit: {
							must_show: function() {
								return true;
							}
						},
						erase: true
					}
				}
			],
			callbacks: {
				save: function(callback, must_create_and_contiune, current_popup) {
					callback = callback || $.noop;
					self.add_or_update_message(callback, must_create_and_contiune, current_popup);
				},
				edit: function(row) {
					self.load_waba_templates(row, row ? false : true);
				},
				remove: function(row, callback = $.noop) {
					self.remove(row, callback);
				},
				on_closed_popup: function(current_popup) {
					self.on_closed_popup(current_popup);
				}
			},
			validate: {
				required_msg: "<%=_T('Must add at least one HSM Message.')%>",
				required: true
			}
		});
	};

	hsm_management_cloud_api.prototype.__fill_grid = function(list)
	{
		list = list || {};
		this.m_grd_hsm_list.fill_rows($.noop, list);
	};

	const language_codes = [
		{
			main_code: "es", localized_codes: [
				{ id: "es", label: "<%=_T('Spanish')%>"        },
				{ id: "es_ES", label: "<%=_T('Spanish (Spain)')%>"             },
				{ id: "es_AR", label: "<%=_T('Spanish (Argentina)')%>"         },
				{ id: "es_MX", label: "<%=_T('Spanish (Mexico)')%>"            }
			]
		},
		{
			main_code: "en", localized_codes: [
				{ id: "en", label: "<%=_T('English')%>"        },
				{ id: "en_US", label: "<%=_T('English (United States)')%>"     },
				{ id: "en_GB", label: "<%=_T('English (United Kingdom)')%>"           }
			]
		},
		{
			main_code: "fr", localized_codes: [
				{ id: "fr", label: "<%=_T('French')%>"                         }
			]
		},
		{
			main_code: "pt", localized_codes: [
				{ id: "pt_PT", label: "<%=_T('Portuguese (Portugal)')%>"       },
				{ id: "pt_BR", label: "<%=_T('Portuguese (Brazil)')%>"         }
			]
		}
	];

	hsm_management_cloud_api.prototype.get_language_codes = function(lang)
	{
		let lang_found = language_codes.find(main_code => main_code.main_code === lang);
		return lang_found ? lang_found.localized_codes : [];
	};	
	
	hsm_management_cloud_api.prototype.get_language_fam = function(lang_code)
	{
		for (let i = 0; i < language_codes.length; i++) {
			let lang_found = language_codes[i].localized_codes.find(lang => lang.id === lang_code);
			return lang_found ? language_codes[i].main_code : "";
		}
	};	

	const editableStatuses = ["APPROVED","REJECTED","PAUSED","INCOMPLETE"];

	const categories = [
		{ id: "UTILITY", label: "<%=_T('Service')%>"                             },
		{ id: "AUTHENTICATION", label: "<%=_T('Authentication')%>"               },
		{ id: "MARKETING", label: "<%=_T('Marketing')%>"                         },
		{ id: "TRANSACTIONAL", label: "<%=_T('Transactional')%>"                 },
		{ id: "OTP", label: "<%=_T('One time password')%>"                       },
		{ id: "SHIPPING_UPDATE", label: "<%=_T('Shipping update')%>"             },
		{ id: "ISSUE_RESOLUTION", label: "<%=_T('Issue resolution')%>"           },
		{ id: "TICKET_UPDATE", label: "<%=_T('Ticket update')%>"                 },
		{ id: "APPOINTMENT_UPDATE", label: "<%=_T('Appointment update')%>"       }
	];

	const editableCategories = {
		UTILITY: true,
		AUTHENTICATION: true,
		MARKETING: true
	};

	const header_formats = [
		{ id: "NONE", label: "<%=_T('None')%>"    },
		{ id: "TEXT", label: "<%=_T('Text')%>"    },
		{ id: "IMAGE" , label: "<%=_T('Image')%>" },
		{ id: "VIDEO" , label: "<%=_T('Video')%>" },
		{ id: "DOCUMENT" , label: "<%=_T('Document')%>" }
	];

	const header_media_types = { 
		IMAGE: "whatsapp_cloud_image",
		VIDEO: "whatsapp_cloud_video",
		DOCUMENT: "whatsapp_cloud_document"
	};

	const button_types = [
		{ id: "none", label: "<%=_T('None')%>"                     },
		{ id: "call_to_action", label: "<%=_T('Call To Action')%>" },
		{ id: "quick_reply" , label: "<%=_T('Quick Reply')%>"      }
	];

	hsm_management_cloud_api.prototype.__fill_combo_templates = function()
	{
		const self = this;
		const $waba_templates = $("#" + this.prefix_id + "_select_popup_id");

		// Subconjunto de templates que ya estan configurados en la account
		const map_used_templates = new Map();
		if(self.current_hsm_mng && self.current_hsm_mng.TemplateMessages) {
			for (const template in self.current_hsm_mng.TemplateMessages) {
				if(Object.hasOwn(self.current_hsm_mng.TemplateMessages, template)) {
					map_used_templates.set(template, true);
				}
			}
		}

		// Subconjunto de templates que puedo usar para crear nuevo hsm
		const available_waba_templates = self.unique_templates_by_name.filter(item => !map_used_templates.get(item.name));
		this.frm.fill_combo($waba_templates, false, available_waba_templates, "name", "name", "name");
	};

	hsm_management_cloud_api.prototype.on_open_popup = function(item)
	{
		const self = this;
		if(self.loading_waba_templates) {
			self.item_to_open = item;
			self.launch_on_open_popup_after_load_templates = true;
			showWaiting("#" + this.prefix_id + "_edit_popup_waiting", true, "<%=_T('Retrieving whatsapp templates')%>");
			return;	
		}
		self.launch_on_open_popup_after_load_templates = false;

		this.m_is_new_hsm = item ? false: true;
		this.m_current_hsm = item;

		this.waba_templates = this.waba_templates || [];
		this.unique_templates_by_name = this.unique_templates_by_name || [];

		const $id = $("#" + this.prefix_id + "_edit_popup_id");
		$(this.container_id).find(".internalPopUpMiddle").animate({scrollTop: 0}, 250, "linear");

		const $category = $("#" + this.prefix_id + "_edit_popup_category");
		this.frm.fill_combo($category, false, categories, "id", "label");

		// fill language dependant combos, except "_code" because it is filled on language checkbox change
		for(const lang_code in this.main_tabs.tabs_list) {
			if(Object.hasOwn(this.main_tabs.tabs_list, lang_code)) {
				const $header_format = $("#" + this.prefix_id + "_edit_popup_lang_" + lang_code + "_header_format");
				this.frm.fill_combo($header_format, false, header_formats, "id", "label");

				const $button_type = $("#" + this.prefix_id + "_edit_popup_lang_" + lang_code + "_button_type");
				this.frm.fill_combo($button_type, false, button_types, "id", "label");
			}
		}

		$category
			.off("change")
			.on("change", function () {
				const $lang_messages_selector = $(self.container_id).find("[lang_message_selector]");
				$.each($lang_messages_selector, function(_, selector) {
					$(selector).prop("checked", false);
					$(selector).change();
				});
			});

		this.clean_up_controls();
		
		const $waba_templates = $("#" + this.prefix_id + "_select_popup_id");
		$waba_templates.parent().toggle(this.m_is_new_hsm);

		const $select_popup_id_container = $("#" + this.prefix_id + "_select_popup_id_container");
		$select_popup_id_container.toggle(this.m_is_new_hsm);
		const $edit_popup_id_container = $("#" + this.prefix_id + "_edit_popup_id_container");
		$edit_popup_id_container.toggle(!this.m_is_new_hsm);

		const $new_template_button = $("#" + this.prefix_id + "_new_template");
		// There is a max templates on a waba limited by whatsapp
		$new_template_button.toggle(this.waba_templates.length < 250);

		const $clean_new_template_button = $("#" + this.prefix_id + "_clean_new_template");
		$clean_new_template_button.toggle(this.m_is_new_hsm);

		if (this.m_is_new_hsm) {
			// nuevo
			self.__fill_combo_templates();

			this.useSelectedTemplate = true;

			$waba_templates.off("change")
				.on("change", function () {
					$id.val($(this).find("option:selected").text());				
					if(self.useSelectedTemplate) {
						const template = self.unique_templates_by_name.find(element => element.name === $id.val());
						$category.val(template ? template.category : "");
						const $lang_messages_selector = $(self.container_id).find("[lang_message_selector]");
						$.each($lang_messages_selector, function(_, selector) {
							$(selector).prop("checked", false);
							$(selector).change();
						});
					}
					self.frm.readonly_control($category, !(self.m_is_new_hsm && !self.useSelectedTemplate), true);
				});

			$new_template_button
				.off("click")
				.on("click", function() {
					self.useSelectedTemplate = false;
					$select_popup_id_container.toggle(false);
					$edit_popup_id_container.toggle(true);	
					$id.val("");
					self.frm.fill_combo($category, false, categories.filter(category => editableCategories[category.id]), "id", "label");

					const $lang_messages_selector = $(self.container_id).find("[lang_message_selector]");
					$.each($lang_messages_selector, function(_, selector) {
						$(selector).prop("checked", false);
						$(selector).change();
					});
					self.frm.readonly_control($category, false, true);
				}
			);		

			$clean_new_template_button
				.off("click")
				.on("click", function() {
					self.useSelectedTemplate = true;
					$select_popup_id_container.toggle(true);
					$edit_popup_id_container.toggle(false);
					$id.val($("#" + self.prefix_id + "_select_popup_id").find("option:selected").text());
					self.frm.fill_combo($category, false, categories, "id", "label");
					const template = self.unique_templates_by_name.find(element => element.name === $id.val());
					$category.val(template ? template.category : "");	
					const $lang_messages_selector = $(self.container_id).find("[lang_message_selector]");
					$.each($lang_messages_selector, function(_, selector) {
						$(selector).prop("checked", false);
						$(selector).change();
					});
					self.frm.readonly_control($category, true, true);	
				}
			);		

			$waba_templates.change();

			return;
		}
		// editando

		this.frm.readonly_control($id, true);
		$id.val(item.id);

		this.frm.readonly_control($category, true, true);		
		$category.val(item.category);

		const $container = $(this.container_id);

		for (let lang in item.languages || {}) {
			if (item.languages.hasOwnProperty(lang)) {
				const $lang_messages_selector = $container.find("[lang_message_selector='" + lang + "']");
				$lang_messages_selector.prop("checked", true).trigger("change");
			}
		}
	};

	hsm_management_cloud_api.prototype.on_closed_popup = function(/*current_popup*/)
	{
		this.m_is_new_hsm = null;
		this.m_current_hsm = null;
	};

	hsm_management_cloud_api.prototype.remove = function(hsm, callback = $.noop)
	{
		const self = this;
		if (!hsm) {
			throw("hsm_management_cloud_api.prototype.remove. Undefined hsm.");
		}
		if(this.current_hsm_mng && this.current_hsm_mng.TemplateMessages && this.current_hsm_mng.TemplateMessages[hsm.id]) {
			self.parent.parent.frm.dashboard.show_waiting("<%=_T('Deleting whatsapp template')%>");

			let templateName = self.current_hsm_mng.TemplateMessages[hsm.id].id;
			inConcert.invoke({
				url: inConcert.app.getBaseDomain() + "/inconcert/apps/administration/campaigns/whatsapp_del_message_templates/",
				type: "POST",
				data: JSON.stringify({
					campaignId : self.frm.campaign.Id,
					accountId : (self.parent.current_account && self.parent.current_account.id) ? self.parent.current_account.id : self.parent.m_authorizingId,
					wabaId: $(self.parent.inconcertCloudApiWaba).val(),
					templateName: templateName
				}),
				dataType: "json",
				contentType: "application/json",
				success: function(response) {
					self.parent.parent.frm.dashboard.hide_waiting();
					if(response.result === "OK") {
						delete self.current_hsm_mng.TemplateMessages[templateName];
						if(response.deleted) {
							for (let i = self.unique_templates_by_name.length - 1; i >= 0; --i) {
								if (self.unique_templates_by_name[i].name === templateName) {
									self.unique_templates_by_name.splice(i,1);
									break;
								}
							}	

							for (let i = self.waba_templates.length - 1; i >= 0; --i) {
								if (self.waba_templates[i].name === templateName) {
									self.waba_templates.splice(i,1);
								}
							}
						}			
						callback(true);
					}
					else {
						let err = response.reason ? _T(response.reason) : "<%=_T('Error deleting whatsapp template')%>";
						self.parent.parent.frm.show_status_message(err, "Error", ERROR_TIMEOUT_MS);
						callback(false);
					}
				},
				error: function(_, text_status) {
					self.parent.parent.frm.dashboard.hide_waiting();
					var err = _T(text_status.reason) || "<%=_T('Error deleting whatsapp template')%>";
					self.parent.parent.frm.show_status_message(err, "Error", ERROR_TIMEOUT_MS);
					callback(false);
				}
			});
		}
		else {		
			callback(true);
		}
	};

	hsm_management_cloud_api.prototype.add_or_update_message = function(callback)
	{
		const self = this;
		callback = callback || $.noop;

		let response = this.validate_hsm_to_add_or_update();
		if (!response.success) {
			callback(false);
			return;
		}

		function after_set_templates (success, error, languages, error_data, callback) {
			showWaiting("#" + self.prefix_id + "_edit_popup_waiting", false);

			let msg_response = {
				msg : "",
				type: "Info"
			};

			let languages_errors = {
				es: {name: "<%=_T('Spanish')%>"   },
				en: {name: "<%=_T('English')%>"   },
				fr: {name: "<%=_T('French')%>"    },
				pt: {name: "<%=_T('Portuguese')%>"},
			};

			if(success) {
				// Aun siendo success === true, en el objeto languages puedo tener algunos languages que se guardaron bien y otros que no. 
				// 	Los que se guardaron bioen tengo que actualizarlos/insertarlos en la grilla
				//	Para los que se produjo error muestro el error
				// Un ejemplo de la estructura de languages:
				/*
					languages: {
						"es": {	-- This occ language family. One of: "es", "en", "fr", "pt"
							success: true or false,
							error: <whatsapp or proxy language template specific error>
							id:	<whatsapp template id>,
							quality_score: <whatsapp template quality score>,
							status: <whatsapp template status>,
							language: <whatsapp template language. One of: "es", "es_ES", "es_AR", "es_MX", "en", "en_GB", "en_US", "fr", "pt_PT", "pt_BR",
							rejected_reason: <whatsapp template rejected reason>,
							urlButtonParam: flag to know if language template has a dynamic call to action button,
							category: <whatsapp template category>
						}
					}
				*/

				let hasErrors = false;
				response = {success: false, $ctrl: undefined, item: response.item, msg: ""};
				
				if(languages) {
					for (const language in languages) {
						if (Object.hasOwn(languages, language)) {
							if(languages[language].success) {
								//Actualizo en el item los datos del language que pueden haber cambiado en el server
								response.item.languages[language].quality_score = languages[language].quality_score;
								response.item.languages[language].status = languages[language].status;
								response.item.languages[language].rejected_reason = languages[language].rejected_reason;
								response.item.languages[language].id = languages[language].id;
								response.item.languages[language].urlButtonParam = languages[language].urlButtonParam || false;
								response.item.languages[language].params = languages[language].params || [];
								response.item.languages[language].text = languages[language].text;
								response.item.languages[language].category = languages[language].category;
								let templateFound = self.waba_templates.find(template => {
									return template.id === languages[language].id;
								});
								if(templateFound) {
									//Actualizo en waba_templates los datos del language que pueden haber cambiado en el server
									templateFound.quality_score = languages[language].quality_score;
									templateFound.status = languages[language].status;
									templateFound.rejected_reason = languages[language].rejected_reason;
									templateFound.id = languages[language].id;
									templateFound.category = languages[language].category;
								}
								else if (self.m_is_new_hsm) {
									//	Si no existia previamente en waba_templates, insertar en self.waba_templates
									self.waba_templates.push(response.item.languages[language]);
								}
							}
							else {
								// Acumular errores de los diferentes languages y mostrarlos al usuario
								response.msg = response.msg + languages_errors[language].name + ": " + _T(languages[language].error) + ".";
								response.timeout = ERROR_TIMEOUT_EXTENDED_MS;	// incremento el tiempo durante el que se muestra el error, ya que en este caso puede llegar a ser largo
								hasErrors = true;
							}
						}
					}

					if(!hasErrors) {				
						if (self.m_is_new_hsm) {
							msg_response.msg = "<%=_T('HSM Message was successfully added.')%>";
							let must_add_many = false;
							callback(true, response.item, msg_response, must_add_many);
						}
						else {
							let cloned_hsm = inConcert.Utils.cloneObject(response.item);
							msg_response.msg = "<%=_T('HSM Message was successfully updated.')%>";
							self.m_current_hsm = cloned_hsm;
							callback(true, self.m_current_hsm, msg_response);
						}
						self.current_hsm_mng.TemplateMessages[response.item.id] = response.item;

						self.clean_up_controls();
						self.__fill_combo_templates();	
						const $category = $("#" + self.prefix_id + "_edit_popup_category");
						self.frm.readonly_control($category, !self.m_is_new_hsm || self.useSelectedTemplate, true);	
						if(self.m_is_new_hsm) {
							$category.val(categories[0].id);
						}
										
						return;
					}
					self.show_ctrl_err(response);
					return;

				}
				else {
					response = {success: false, $ctrl: $("#" + self.prefix_id + "_edit_popup_id"), item: null, msg: _T(error)};
					self.show_ctrl_err(response);
					return;
				}
			}
			else {
				//Default control with error 
				let $ctrl_with_error = $("#" + self.prefix_id + "_edit_popup_id");
				// buscar el tab y el campo que dio error
				if(error_data && error_data.lang_code) { 
					let lang_code = self.get_language_fam(error_data.lang_code);
					if(error_data.error_component === "HEADER") {
						$ctrl_with_error = $("#" + self.prefix_id + "_edit_popup_lang_" + lang_code + "_" + (error_data.error_component_field ? error_data.error_component_field : "header_text"));
					}
					else if(error_data.error_component === "BODY") {
						$ctrl_with_error = $("#" + self.prefix_id + "_edit_popup_lang_" + lang_code + "_" + (error_data.error_component_field ? error_data.error_component_field : "message"));
					}
					else if(error_data.error_component === "BUTTONS" && error_data.button_type && error_data.button_value) {
						let buttons_translation = {
							PHONE_NUMBER: "hsm_btn_phone",
							URL: "hsm_btn_url"
						};
						const button_type_translated = buttons_translation[error_data.button_type];
						const $buttons = $("#" + self.prefix_id + "_edit_popup_lang_" + lang_code + "_call_to_action_buttons_container").find("[" + button_type_translated + "]");
						$.each($buttons, function(_, selector) {
							const $selector = $(selector);
							if($selector.val() === error_data.button_value) {
								$ctrl_with_error = $selector;
							}
						});
					}
					self.main_tabs.switch_tab(lang_code);
				}
				response = {success: false, $ctrl: $ctrl_with_error, item: null, msg: _T(error)};
				self.show_ctrl_err(response);
			}		
		}

		showWaiting("#" + this.prefix_id + "_edit_popup_waiting", true, "<%=_T('Saving whatsapp templates')%>");

		inConcert.invoke({
			url: inConcert.app.getBaseDomain() + "/inconcert/apps/administration/campaigns/whatsapp_set_message_templates/",
			type: "POST",
			data: JSON.stringify({
				campaignId : self.frm.campaign.Id,
				accountId : (self.parent.current_account && self.parent.current_account.id) ? self.parent.current_account.id : self.parent.m_authorizingId,
				wabaId: $(self.parent.inconcertCloudApiWaba).val(),
				data: response.item
			}),
			dataType: "json",
			contentType: "application/json",
			success: function(response) {
				let success = response.result === "OK";
				let err = success ? "" : response.reason || "<%=_T('Error saving whatsapp templates')%>";
				after_set_templates(success, err, response.languages, response.error_data, callback);
			},
			error: function(_, text_status) {
				var err = _T(text_status.reason) || "<%=_T('Error saving whatsapp templates')%>";
				after_set_templates(false, err, undefined, text_status.error_data, callback);
			}
		});

	};

	hsm_management_cloud_api.prototype.clean_up_controls = function()
	{
		const $id = $("#" + this.prefix_id + "_edit_popup_id");
		const $group_name = $("#" + this.prefix_id + "_edit_popup_group_name");
		const $category = $("#" + this.prefix_id + "_edit_popup_category");

		this.frm.readonly_control($id, false, true);
		$id.val("").focus();
		$group_name.val("");
		$category.val("");

		const $lang_messages_selector = $(this.container_id).find("[lang_message_selector]");
		$lang_messages_selector.prop("checked", false).trigger("change");

		this.main_tabs.switch_tab("en"); // dejo seleccionado el primer tab
	};

	hsm_management_cloud_api.prototype.validate_hsm_to_add_or_update = function()
	{
		const self = this;
		this.hide_all_errors();
		let response = {success: false, $ctrl: null, item: null, msg: "<%=_T('Unable to Add Message.')%>"};

		const $id = $("#" + this.prefix_id + "_edit_popup_id");
		let id = $id.val();

		let validation_result = inConcert.Utils.validate(id, "required");
		if (!validation_result.success) {
			// valida que el name tenga al menos un caracter
			response.msg = "<%=_T('HSM Id is required.')%>";
			response.$ctrl = $id;
			this.show_ctrl_err(response);
			return response;
		}

		validation_result = inConcert.Utils.validate(id, "letters_number_underscore");
		if (!validation_result.success) {
			// solo letras, numeros y puntos
			response.msg = "<%=_T('Only letters, numbers, dashes are allowed in the HSM Id.')%>";
			response.$ctrl = $id;
			this.show_ctrl_err(response);
			return response;
		}
		
		const $category = $("#" + this.prefix_id + "_edit_popup_category");
		let category = $category.val();
		validation_result = inConcert.Utils.validate(category, "required");
		if ( !validation_result.success ) {
			// valida que el name tenga al menos un caracter
			response.msg = "<%=_T('Category is required.')%>";
			response.$ctrl = $category;
			this.show_ctrl_err(response);
			return response;
		}

		let total_languages = 0;
		let languages = {};
		const $lang_messages_selector = $(this.container_id).find("[lang_message_selector]");
		$.each($lang_messages_selector, function(_, selector) {
			const $selector = $(selector);
			if ($selector.prop("checked") === true) {
				total_languages++;
				const lang_code = $selector.attr("lang_message_selector");
				const $quality_score = $("#" + self.prefix_id + "_edit_popup_lang_" + lang_code + "_quality_score");
				const $status = $("#" + self.prefix_id + "_edit_popup_lang_" + lang_code + "_code_status");
				const $code = $("#" + self.prefix_id + "_edit_popup_lang_" + lang_code + "_code");
				const $header_format = $("#" + self.prefix_id + "_edit_popup_lang_" + lang_code + "_header_format");
				const $header_text = $("#" + self.prefix_id + "_edit_popup_lang_" + lang_code + "_header_text");
				const $example_header_text = $("#" + self.prefix_id + "_edit_popup_lang_" + lang_code + "_example_header_text");
				const $header_media_url = $("#" + self.prefix_id + "_edit_popup_lang_" + lang_code + "_header_media_url");
				const $security_recomendation = $("#" + self.prefix_id + "_edit_popup_lang_" + lang_code + "_add_security_recomendation");
				const $message = $("#" + self.prefix_id + "_edit_popup_lang_" + lang_code + "_message");
				const $params = $("#" + self.prefix_id + "_edit_popup_lang_" + lang_code + "_params");
				const $code_expiration = $("#" + self.prefix_id + "_edit_popup_lang_" + lang_code + "_add_code_expiration");
				const $code_expiration_minutes = $("#" + self.prefix_id + "_edit_popup_lang_" + lang_code + "_footer_code_expiration_minutes");
				const $footer = $("#" + self.prefix_id + "_edit_popup_lang_" + lang_code + "_footer");				
				const buttons = self.hsmButtons[lang_code].__getDataButtons(lang_code);
						
				languages[lang_code.toLowerCase()] = {
					$quality_score: $quality_score,
					$status: $status,
					$code: $code,
					$header_format: $header_format,
					$header_text: $header_text,
					$example_header_text: $example_header_text,
					$header_media_url: $header_media_url,
					$security_recomendation: $security_recomendation,
					$message: $message,
					$params: $params,
					$code_expiration: $code_expiration,
					$code_expiration_minutes: $code_expiration_minutes,
					$footer: $footer,
					buttons: buttons
				};
				
			}		
		});

		if (total_languages === 0) {
			// valida que al menos hayan ingresado un mensaje (es, fr, en)
			response.msg = "<%=_T('HSM Messages is required.')%>";
			response.$ctrl = null;//$group_name;
			this.show_ctrl_err(response);
			return response;
		}

		let languages_to_save = {};
		for (let lang in languages) {
			if (languages.hasOwnProperty(lang)) {
				let language = languages[lang];
				let language_to_save = {};

				// validate language code
				validation_result = inConcert.Utils.validate(language.$code.val(), "required");
				if (!validation_result.success) {
					this.main_tabs.switch_tab(lang);
					response.msg = "<%=_T('Language code is required.')%>";
					response.$ctrl = language.$code;
					this.show_ctrl_err(response);
					return response;
				}
				language_to_save.id = language.$code.prop("waba_template_id");
				language_to_save.name = id.toLowerCase();
				language_to_save.language = language.$code.val();
				language_to_save.code = language.$code.val();
				language_to_save.category = category;
				language_to_save.quality_score = {
					score: language.$quality_score.val()
				};
				language_to_save.status = language.$status.prop("waba_template_status");
				language_to_save.groupId = this.parent.current_account.message_template_namespace;
				language_to_save.groupName = this.parent.current_account.message_template_namespace;
				language_to_save.components = [];

				// Validate header 
				let header = null;
				let header_format = language.$header_format.val();
				if(header_format === "TEXT") {
					validation_result = inConcert.Utils.validate(language.$header_text.val(), "required");
					if (!validation_result.success) {
						// valida que el texto del header no este vacio
						this.main_tabs.switch_tab(lang);
						response.msg = "<%=_T('Header text is required.')%>";
						response.$ctrl = language.$header_text;
						this.show_ctrl_err(response);
						return response;
					}
					header = header || {};
					header.text = language.$header_text.val();

					// validate header placeholders and params
					let header_placeholders = (language.$header_text.val().match(/{{([^}]*)}}/g) || []);
					if(header_placeholders.length > 1 || (header_placeholders.length === 1 && header_placeholders[0] !== "{{1}}" )) {
						this.main_tabs.switch_tab(lang);
						response.msg = "<%=_T('Only one placeholder is allowed on header')%>";
						response.$ctrl = language.$header_text;
						this.show_ctrl_err(response);
						return response;
					}

					if(header_placeholders.length) {
						validation_result = inConcert.Utils.validate(language.$example_header_text.val(), "required");
						if (!validation_result.success) {
							// valida que parametro del header no este vacio si hay un placeholder definido en el texto del header
							this.main_tabs.switch_tab(lang);
							response.msg = "<%=_T('Header parameter is required.')%>";
							response.$ctrl = language.$example_header_text;
							this.show_ctrl_err(response);
							return response;
						}
						header.header_param = language.$example_header_text.val();
					}
					else if(language.$example_header_text.val()){
						// valida que haya un placeholder definido si hay un parametro de header
						this.main_tabs.switch_tab(lang);
						response.msg = "<%=_T('Header placeholder is required if header parameter is defined.')%>";
						response.$ctrl = language.$header_text;
						this.show_ctrl_err(response);
						return response;
					}
				}
				if(header_format === "VIDEO" || header_format === "DOCUMENT" || header_format === "IMAGE") {
					if(!language.$header_media_url.val().trim().length) {
						this.main_tabs.switch_tab(lang);
						response.msg = "<%=_T('Header media url is required.')%>";
						response.$ctrl = language.$header_media_url;
						this.show_ctrl_err(response);
						return response;
					}
					header = header || {};
					header.mediaUrl = language.$header_media_url.val();
					let sharedFileObj = language.$header_media_url.prop("sharedFile");
					if(sharedFileObj) {
						header.sharedFile = JSON.parse(sharedFileObj);
					}
				}
				if(header) {					
					header.type = "HEADER";
					header.format = header_format;
					language_to_save.components.push(header);
				}

				// validate body text
				validation_result = inConcert.Utils.validate(language.$message.val(), "required");
				if (!validation_result.success) {
					// valida que el mensaje no este vacio (es, fr, en, pt)
					this.main_tabs.switch_tab(lang);
					response.msg = "<%=_T('HSM Message is required.')%>";
					response.$ctrl = language.$message;
					this.show_ctrl_err(response);
					return response;
				}

				// validate body params
				let parsed_params = language.$params.val().split("\n");
				let params = [];
				parsed_params.forEach(param => {
					if (param.trim().length > 0) {
						params.push(param);
					}
				});

				let required_placeholders = (language.$message.val().match(/{{([^}]*)}}/g) || []);
				for (let i = 0; i < required_placeholders.length; i++) {
					let placeholder = required_placeholders[i].replace("{{", "").replace("}}", "");
					if(placeholder !== (i+1).toString()) {
						this.main_tabs.switch_tab(lang);
						response.msg = "<%=_T('Text placeholders must be consecutive numbers starting at one')%>";
						response.$ctrl = language.$message;
						this.show_ctrl_err(response);
						return response;
					}
				}
				if(required_placeholders.length !== params.length) {
					this.main_tabs.switch_tab(lang);
					response.msg = format("<%=_T('The number of placeholders defined in message ({0}) does not match with defined parameters ({1})')%>", required_placeholders.length, params.length);
					response.$ctrl = language.$message;
					this.show_ctrl_err(response);
					return response;
				}
				let body = {
					type: "BODY",
					text: language.$message.val()
				};
				if(params.length) {
					body.params = params;
				}
				if(language.$security_recomendation.prop("checked")) {
					body.add_security_recommendation = true;
				}

				language_to_save.components.push(body);

				// validate code_expiration_minutes
				if(category === "AUTHENTICATION" && language.$code_expiration.prop("checked")) {
					validation_result = inConcert.Utils.validate(language.$code_expiration_minutes.val(), "required");
					if (!validation_result.success) {
						// valida que el tiempo de expiracion del codigo no este vacio (es, fr, en, pt)
						this.main_tabs.switch_tab(lang);
						response.msg = "<%=_T('Code expiration (minutes) is required.')%>";
						response.$ctrl = language.$message;
						this.show_ctrl_err(response);
						return response;
					}

					let code_expiration_minutes_range = { from: 1, to: 90 };
					validation_result = inConcert.Utils.validate(language.$code_expiration_minutes.val(), "range", { between: code_expiration_minutes_range});
					if (!validation_result.success) {
						// valida que el tiempo de expiracion del codigo este en el rango permitido: 1 - 90 (es, fr, en, pt)
						this.main_tabs.switch_tab(lang);
						response.msg = format("<%=_T('Code expiration (minutes) must be a number between {0} and {1}.')%>", code_expiration_minutes_range.from, code_expiration_minutes_range.to);
						response.$ctrl = language.$message;
						this.show_ctrl_err(response);
						return response;
					}
				}
				
				if(language.$footer.val().trim().length) {
					let footer = {
						type: "FOOTER",
						text: language.$footer.val()
					};
					if(language.$code_expiration.prop("checked")) {
						footer.code_expiration_minutes = language.$code_expiration_minutes.val();
					}
					language_to_save.components.push(footer);
				}

				// validate buttons
				if(language.buttons.length) {
					for (let i = 0; i < language.buttons.length; i++) {
						if(!language.buttons[i].text.trim().length) {
							this.main_tabs.switch_tab(lang);
							response.msg = "<%=_T('Button text is required')%>";
							response.$ctrl = language.buttons[i].textCtrl;
							this.show_ctrl_err(response);
							return response;
						}
						switch(language.buttons[i].type) {
							case "URL":
								validation_result = inConcert.Utils.validate(language.buttons[i].urlCtrl.val(), "required");
								if (!validation_result.success) {	
									this.main_tabs.switch_tab(lang);
									response.msg = "<%=_T('Button url is required')%>";
									response.$ctrl = language.buttons[i].urlCtrl;
									this.show_ctrl_err(response);
									return response;
								}
								let button_url_params = (language.buttons[i].urlCtrl.val().match(/{{([^}]*)}}/g) || []);
								if(button_url_params.length > 0) {
									if(button_url_params.length > 1) {
										this.main_tabs.switch_tab(lang);
										response.msg = "<%=_T('Only one variable can be added to the end of the url')%>";
										response.$ctrl = language.buttons[i].urlCtrl;
										this.show_ctrl_err(response);
										return response;
									}
									if(button_url_params[0] !== "{{1}}") {
										this.main_tabs.switch_tab(lang);
										response.msg = "<%=_T('Use {{1}} to add a variable to the end of the url')%>";
										response.$ctrl = language.buttons[i].urlCtrl;
										this.show_ctrl_err(response);
										return response;
									}
								}
								let buttonUrlWithoutParam = language.buttons[i].urlCtrl.val().replace("{{1}}", "");
								validation_result = inConcert.Utils.validate(buttonUrlWithoutParam, "url");
								if (!validation_result.success) {	
									this.main_tabs.switch_tab(lang);
									response.msg = "<%=_T('Button url is invalid')%>";
									response.$ctrl = language.buttons[i].urlCtrl;
									this.show_ctrl_err(response);
									return response;
								}
								break;

							case "PHONE_NUMBER":
								validation_result = inConcert.Utils.validate(language.buttons[i].phone_numberCtrl.val(), "required");
								if (!validation_result.success) {						
									this.main_tabs.switch_tab(lang);
									response.msg = "<%=_T('Button phone is required')%>";
									response.$ctrl = language.buttons[i].phone_numberCtrl;
									this.show_ctrl_err(response);
									return response;
								}
								break;
							case "COPY_CODE":
							case "ONE_TAP":
								language.buttons[i].otp_type = language.buttons[i].type;
								language.buttons[i].type = "URL";
								if(language.buttons[i].otp_type === "ONE_TAP") {
									validation_result = inConcert.Utils.validate(language.buttons[i].autofill_textCtrl.val(), "required");
									if (!validation_result.success) {						
										this.main_tabs.switch_tab(lang);
										response.msg = "<%=_T('Button autofill text is required')%>";
										response.$ctrl = language.buttons[i].autofill_textCtrl;
										this.show_ctrl_err(response);
										return response;
									}
									validation_result = inConcert.Utils.validate(language.buttons[i].package_nameCtrl.val(), "required");
									if (!validation_result.success) {						
										this.main_tabs.switch_tab(lang);
										response.msg = "<%=_T('Button package name is required')%>";
										response.$ctrl = language.buttons[i].package_nameCtrl;
										this.show_ctrl_err(response);
										return response;
									}
									validation_result = inConcert.Utils.validate(language.buttons[i].signature_hashCtrl.val(), "required");
									if (!validation_result.success) {						
										this.main_tabs.switch_tab(lang);
										response.msg = "<%=_T('Button application signature is required')%>";
										response.$ctrl = language.buttons[i].signature_hashCtrl;
										this.show_ctrl_err(response);
										return response;
									}
								}
								break;
						}

						delete language.buttons[i].textCtrl;
						delete language.buttons[i].urlCtrl;
						delete language.buttons[i].phone_numberCtrl;
						delete language.buttons[i].autofill_textCtrl;
						delete language.buttons[i].package_nameCtrl;
						delete language.buttons[i].signature_hashCtrl;
					} 
					let buttons = {
						type: "BUTTONS",
						buttons: language.buttons
					};
					language_to_save.components.push(buttons);
				}

				languages_to_save[lang] = language_to_save;
			}
		}
		
		let item = {
			id: id.toLowerCase(),
			category: category,
			groupId: this.parent.current_account.message_template_namespace,
			groupName: this.parent.current_account.message_template_namespace,
			status: "approved",
			available: true,
			languages: languages_to_save
		};

		response.success = true;
		response.msg = "";
		response.item = item;

		return response;
	};

	hsm_management_cloud_api.prototype.hide_all_errors = function()
	{
		this.m_grd_hsm_list.hide_popup_msg();
	};

	hsm_management_cloud_api.prototype.show_ctrl_err = function(err_info)
	{
		if (!err_info) {
			return;
		}
		const $control = err_info.$ctrl;
		if ($control) {
			const error_id = $control.attr("id") + "_validation";
			const $error_indicator = $("#" + error_id);
			if ($error_indicator) {
				$error_indicator.show();
			}
			$control.focus();
		}
		this.m_grd_hsm_list.show_popup_msg(err_info.msg, "Error", err_info.timeout || ERROR_TIMEOUT_MS);
		$(this.container_id).find(".internalPopUpMiddle").animate({scrollTop: 0}, 250, "linear");
	};

	hsm_management_cloud_api.prototype.validate = function()
	{
		const result = {success: true};
		return result;
	};

	hsm_management_cloud_api.prototype.get = function()
	{
		const enabled = $(this.hsm_mng).prop("checked");
		const config = this.__get(enabled);
		return config;
	};

	hsm_management_cloud_api.prototype.__get = function(enabled)
	{
		let config = {Enabled: null, TemplateMessages: {}};

		if (!this.supports_integration(this.m_integration_type)) {
			config.Enabled = bool_to_string(false);
			return config;
		}

		config.Enabled = bool_to_string(enabled);
		if (config.Enabled === "1") {
			config.TemplateMessages = this.get_config_list();
		}
		return config;
	};

	hsm_management_cloud_api.prototype.get_config_list = function()
	{
		let hsm_list = {};

		let items = this.m_grd_hsm_list.get_items();
		if (items && items.length > 0) {
			items.forEach(item => {
				const id = item.id;
				hsm_list[id] = item;
			});
		}

		return hsm_list;
	};

	hsm_management_cloud_api.prototype.attach_data = function( acc )
	{
		if (!acc) {
			return;
		}
		acc.HSMConfig = this.get(true /* keep_old_data */);
		return acc;
	};

	hsm_management_cloud_api.prototype.update_current_hsm_mng = function(enabled)
	{
		this.current_hsm_mng = this.__get(enabled);
	};

	hsm_management_cloud_api.prototype.refresh_remplates = function(item_to_open)
	{
		const self = this;
		if(this.current_hsm_mng && this.current_hsm_mng.TemplateMessages) {
			for (const templateMessage in this.current_hsm_mng.TemplateMessages) {
				if (Object.hasOwn(this.current_hsm_mng.TemplateMessages, templateMessage)) {
					if(this.current_hsm_mng.TemplateMessages[templateMessage].languages) {
						for (const language in this.current_hsm_mng.TemplateMessages[templateMessage].languages) {
							if (Object.hasOwn(this.current_hsm_mng.TemplateMessages[templateMessage].languages, language)) {
								let language_template = self.waba_templates.find(t => t.id === self.current_hsm_mng.TemplateMessages[templateMessage].languages[language].id);
								if(language_template) {
									this.current_hsm_mng.TemplateMessages[templateMessage].languages[language].status = language_template.status;
									this.current_hsm_mng.TemplateMessages[templateMessage].languages[language].rejected_reason = language_template.rejected_reason;
									if(item_to_open && item_to_open.languages && item_to_open.languages[language] && item_to_open.languages[language].id === self.current_hsm_mng.TemplateMessages[templateMessage].languages[language].id) {
										item_to_open.languages[language].status = language_template.status;
										item_to_open.languages[language].rejected_reason = language_template.rejected_reason;
									}
								}
							}
						}
					}
				}
			}			
		}
	};

	hsm_management_cloud_api.prototype.set_ctrls_codes = function()
	{
		const hsm_mng = this.current_hsm_mng || {};
		this.__fill_grid(hsm_mng.TemplateMessages || {});
	};

	hsm_management_cloud_api.prototype.has_configured_hsm_mng = function(acc)
	{
		if (!acc) {
			return false;
		}
		if (acc.HSMConfig && acc.HSMConfig.Enabled === "1") {
			return true;
		}
		return false;
	};

	hsm_management_cloud_api.prototype.set = function(acc)
	{
		if (!acc) {
			return;
		}
		this.current_hsm_mng = acc.HSMConfig || null;
		const $hsm_mng = $(this.hsm_mng);

		if (!this.has_configured_hsm_mng(acc)) {
			$hsm_mng.prop("checked", false);
			this.disable_hsm_mng(true);
		}
		else {
			$hsm_mng.prop("checked", true);
			this.disable_hsm_mng(false);
		}

		this.load_waba_templates();
	};

	hsm_management_cloud_api.prototype.disable_hsm_mng = function(disabled)
	{
		const $grid_container = $("#" + this.prefix_id + "_grid_container");
		const $new = $("#" + this.prefix_id + "_popup_new");
		if (disabled) {
			$grid_container.hide();
			$new.hide();
		}
		else {
			$grid_container.show();
			$new.show();
		}
	};

	hsm_management_cloud_api.prototype.set_default_codes = function()
	{
		$( this.hsm_mng ).prop( "checked", true );
		$( this.hsm_mng ).trigger( "click" );
	};

	hsm_management_cloud_api.prototype.editable = function()
	{
		const self = this;
		var ds_ctrl = function( $ctrl, ds, required ) {
			self.section.frm.disable_control( $ctrl, ds, required );
		};
		const $hsm_mng = $( this.hsm_mng );
		ds_ctrl( $hsm_mng, false );
	};

	hsm_management_cloud_api.prototype.readonly = function()
	{
		const self = this;
		const ds_ctrl = function( $ctrl, ds, required ) {
			self.frm.readonly_control( $ctrl, ds, required );
		};
		ds_ctrl( $( this.hsm_mng ), true /* disabled */ );
	};

	hsm_management_cloud_api.prototype.sanitize_id = function( id )
	{
		// remueve el # de jquery
		return id.substring( 1 );
	};

	hsm_management_cloud_api.prototype.show = function()
	{
		$(this.container_id).show();
	};

	hsm_management_cloud_api.prototype.hide = function()
	{
		$(this.container_id).hide();
	};

	hsm_management_cloud_api.prototype.toggle = function()
	{
		if (this.supports_integration(this.m_integration_type)) {
			this.show();
		}
		else {
			this.hide();
		}
	};

	hsm_management_cloud_api.prototype.set_integration_type = function(integration_type)
	{
		this.m_integration_type = (integration_type || "").toUpperCase();
		this.toggle();
	};

	hsm_management_cloud_api.prototype.make_html = function()
	{
		const hsm_mng_id = this.sanitize_id(this.hsm_mng);
		$( this.container_id ).html( "".concat(
		// hsm management ( Enabled / Disabled )
			"<div class='row align-items-center'>",
				"<div class='col-6 py-2'>",
					"<div class='form-group mb-0'>",
						"<div class='form-check'>",
							"<label for='", hsm_mng_id, "' class='form-check-label form-check-label-title'>",
								"<input id='", hsm_mng_id, "' class='form-check-input' type='checkbox' />",
								"<strong>", "<%=_T('Enable HSM Message Configuration')%>", "</strong>", 
							"</label>",
						"</div>",
					"</div>",
				"</div>",
				"<div class='col-6'>",
					"<button id='" + this.prefix_id + "_popup_new' class='pageButtonGeneric buttonColorWhite pageButtonNew' style='display:none;'><i class='far fa-plus'></i>", "<%=_T('New HSM')%>" ,"</button>",
				"</div>",
			"</div>",
			// grid
			"<div id='", this.prefix_id , "_grid_container' style='display:none;'>",
				// Table
				"<div class='row mt-4'>",
					"<div class='col-12'>",
						"<table id='", this.prefix_id , "_list' cellpadding='0' cellspacing='0' border='0' class='viewGrid mt-0'>",
							"<thead class='viewGridHeader'></thead>",
							"<tbody class='viewGridBody'></tbody>",
						"</table>",
						"<div id='", this.prefix_id , "_not_results_found' class='inlineMessagesContainer messageInfo' style='display:none'></div>",
						"<div id='", this.prefix_id , "_paginator'></div>",
					"</div>",
				"</div>",
				// start - edit hsm popup
				"<div id='" + this.prefix_id + "_edit_popup' class='internalPopUpContainer popUp940w formPopUp hsmManagementPopUp' style='display:none;'>",
					"<div class='internalPopUpOverlay'></div>",
					"<div class='internalPopUp'>",
						// waiting
						"<div id='" + this.prefix_id + "_edit_popup_waiting' class='internalWaitingContainer' style='display:none;'>",
							"<div waiting_message='true' class='internalWaitingMessage'></div>",
							"<div class='internalWaitingAnimation'></div>",
							"<div class='internalWaitingOverlay'></div>",
						"</div>",
						// title
						"<div id='" + this.prefix_id + "_edit_popup_title' class='internalPopUpTitle'>",
							"<span popup_title_text>", "<%=_T('Authorize Application')%>", "</span>"  +
							"<button popup_close id='" + this.prefix_id + "_edit_popup_close_times' class='internalPopUpCloseButton'><i class='far fa-times'></i></button>",
						"</div>",
						// content
						"<div class='internalPopUpMiddle pageFormContainer'>",
							// mensaje de error
							"<div id='" + this.prefix_id + "_edit_popup_message' class='genericFormMessagesContainer messageError' style='display:none;'>FIELD ERROR MESSAGE</div>",
							// form
							"<div class='no-card'>",
								"<div class='card-block'>",
									"<div id='" + this.prefix_id + "_select_popup_id_container' class='row'>",
										"<div class='col-12 d-flex'>",
											"<div class='form-group flex-grow-1'>",
												"<label for='" + this.prefix_id + "_select_popup_id' class='required'><%=_T('Id')%><%=_T(':')%></label>",
												"<select id='" + this.prefix_id + "_select_popup_id' class='form-control' type='text' maxlength='50'>",
												"</select>",
											"</div>",
											"<div class='form-group d-flex align-items-end'>",
												"<button id='", this.prefix_id, "_new_template' class='pageButtonGeneric buttonColorWhite iconOnlyBtn' type='button' title='" + "<%=_T('Create New Template')%>" + "'>",
													"<i class='far fa-plus'></i>",
												"</button>",
											"</div>",
										"</div>",
									"</div>",
									"<div id='" + this.prefix_id + "_edit_popup_id_container' class='row' style='display:none;'>",
										"<div class='col-12 d-flex'>",	
											"<div class='form-group flex-grow-1 position-relative'>",
												"<label for='" + this.prefix_id + "_edit_popup_id' class='required'><%=_T('Id')%><%=_T(':')%></label>",
												"<input id='" + this.prefix_id + "_edit_popup_id' class='form-control' type='text' maxlength='512' placeholder='", "<%=_T('Enter the id')%>...", "' />",
												"<i id='" + this.prefix_id + "_edit_popup_id_validation' class='formFieldValidation formFieldError' hidden></i>",
											"</div>",
											"<div class='form-group d-flex align-items-end'>",
												"<button id='", this.prefix_id, "_clean_new_template' class='pageButtonGeneric buttonColorWhite iconOnlyBtn' type='button' title='" + "<%=_T('Select Existing Template')%>" + "'>",
													"<i class='far fa-eraser'></i>",
												"</button>",
											"</div>",
										"</div>",
									"</div>",
									"<div class='row'>",
										"<div class='col-3'>",
											"<div class='form-group' title='<%=_T('Select category')%>'>",
												"<label for='" + this.prefix_id + "_edit_popup_category' class='required'><%=_T('Category')%><%=_T(':')%>",
													"<span class='fa fa-question-circle' title='", "<%=_T('Only Service, Marketing and Authentication categories are fully editable. Transactional and One time password categories are editable until 03-31-2023. Any template category can be used to send messages.')%>", "'></span>",
												"</label>",
												"<select id='" + this.prefix_id + "_edit_popup_category' class='form-control' type='text' maxlength='50'>",
												"</select>",
												"<i id='" + this.prefix_id + "_edit_popup_category_validation' class='formFieldValidation formFieldError' hidden></i>",
											"</div>",
										"</div>",
									"</div>",
									"<div class='row'>",
										"<div class='col-12'>",
											"<div class='formTabsMainContainer'>",
												// TABS
												"<div id='", this.prefix_id, "_languages_tabs' class='formFatherFieldset'></div>",

												// TAB ENGLISH CONTAINER
												"<div id='", this.prefix_id, "_languages_english_tab_container' class='formTabsBody' tab_lang_selector='en' style='display:none;'>",
													// language activation
													"<div class='row'>",
														"<div class='col-3'>",
															"<div class='form-group'>",
																"<div class='form-check'>" ,
																	"<label for='" , this.prefix_id , "_edit_popup_lang_en_selector' class='form-check-label form-check-label-title'>" ,
																		"<input id='" , this.prefix_id , "_edit_popup_lang_en_selector' class='form-check-input' type='checkbox' lang_message_selector='en' related_tab='" , this.prefix_id , "_languages_english_tab_container' />" ,
																		"<%=_T('Enable language')%>", 
																	"</label>" ,
																"</div>" ,
															"</div>",
														"</div>",
													"</div>",
													// language code, quality score, status and rejected reason
													"<div class='row'>",
														"<div class='col-3'>",
															"<div class='form-group'>",
																"<label for='" , this.prefix_id , "_edit_popup_lang_en_code' ><%=_T('Code')%><%=_T(':')%></label>",
																"<select id='" , this.prefix_id , "_edit_popup_lang_en_code' class='form-control' lang_code_selector='en'></select>",
															"</div>",
														"</div>",
														"<div class='col-3'>",
															"<div id='" , this.prefix_id , "_edit_popup_lang_en_quality_score_container'", " class='form-group'>",																
																"<label for='" , this.prefix_id , "_edit_popup_lang_en_quality_score' ><%=_T('Quality score')%><%=_T(':')%></label>",
																"<input id='" , this.prefix_id , "_edit_popup_lang_en_quality_score' class='form-control' type='text' lang_quality_score_selector='en' ></input>",
															"</div>",															
														"</div>",
														"<div class='col-3'>",
															"<div id='" , this.prefix_id , "_edit_popup_lang_en_code_status_container'", " class='form-group'>",																
																"<label for='" , this.prefix_id , "_edit_popup_lang_en_code_status' ><%=_T('Status')%><%=_T(':')%>",
																	"<span class='fa fa-question-circle' title='", "<%=_T('Only Active, Rejected and Paused language templates can be edited.')%>", "<%=_T(' ')%>", "<%=_T('Only Active language templates can be used')%>", "<%=_T('.')%>", "'></span>",
																"</label>",
																"<input id='" , this.prefix_id , "_edit_popup_lang_en_code_status' class='form-control' type='text' lang_status_selector='en' ></input>",
															"</div>",															
														"</div>",
														"<div class='col-3'>",
															"<div id='" , this.prefix_id , "_edit_popup_lang_en_rejected_reason_container'", " class='form-group'>",																
																"<label for='" , this.prefix_id , "_edit_popup_lang_en_rejected_reason' ><%=_T('Rejected reason')%><%=_T(':')%></label>",
																"<input id='" , this.prefix_id , "_edit_popup_lang_en_rejected_reason' class='form-control' type='text' lang_rejected_reason_selector='en' ></input>",
															"</div>",															
														"</div>",
													"</div>",
													// header
													"<div class='card-block'>",
														"<div class='row pt-2'>",
															"<div class='col-3'>",
																"<div class='card-block'>",
																	"<div class='form-group'>",
																		"<label for='" , this.prefix_id , "_edit_popup_lang_en_header_format' ><%=_T('Header type')%><%=_T(':')%>",
																			"<span class='fa fa-question-circle' title='", "<%=_T('Optional. Add a title or media content to the header of your message template.')%>", "'></span>",
																		"</label>",
																		"<select id='" , this.prefix_id , "_edit_popup_lang_en_header_format' class='form-control' lang_header_format_selector='en'></select>",
																		"<i id='" , this.prefix_id , "_edit_popup_lang_en_header_format_validation' class='formFieldValidation formFieldError' hidden></i>",
																	"</div>",
																"</div>",
															"</div>",																																								
														"</div>",
														"<div id='" , this.prefix_id , "_edit_popup_lang_en_header_text_container' class='row pt-2' style='display:none;'>",
															"<div class='col-9' >",
																"<div class='form-group'>",
																	"<label for='" , this.prefix_id , "_edit_popup_lang_en_header_text' ><%=_T('Header text')%><%=_T(':')%>",
																		"<span class='fa fa-question-circle' title='", "<%=_T('Add an optional parameter placeholder ({{1}}) to customize header text.')%>", "'></span>",
																	"</label>",
																	"<input id='" , this.prefix_id , "_edit_popup_lang_en_header_text' class='form-control' type='text' maxlength='60' placeholder='", "<%=_T('Enter the header text')%>..." ,"' />",
																	"<i id='" , this.prefix_id , "_edit_popup_lang_en_header_text_validation' class='formFieldValidation formFieldError' hidden></i>",
																"</div>",
															"</div>",	
															"<div class='col-3' >",
																"<div class='form-group'>",
																	"<label for='" , this.prefix_id , "_edit_popup_lang_en_example_header_text' ><%=_T('Parameter')%><%=_T(':')%>",
																		"<span class='fa fa-question-circle' title='", "<%=_T('Enter a header parameter')%>", "'></span>",
																	"</label>",
																	"<input id='" , this.prefix_id , "_edit_popup_lang_en_example_header_text' class='form-control' type='text' maxlength='60' />",
																	"<i id='" , this.prefix_id , "_edit_popup_lang_en_example_header_text_validation' class='formFieldValidation formFieldError' hidden></i>",
																"</div>",
															"</div>",
														"</div>",
														"<div id='" , this.prefix_id , "_edit_popup_lang_en_header_media_container' class='row pt-2' style='display:none;'>",
															"<div class='col-12 d-flex' >",	
																"<div class='form-group flex-grow-1 position-relative'>",
																	"<label for='" , this.prefix_id , "_edit_popup_lang_en_header_media_url' ><%=_T('Header media url')%><%=_T(':')%></label>",
																	"<input id='" , this.prefix_id , "_edit_popup_lang_en_header_media_url' class='form-control' type='text' maxlength='2000' placeholder='", "<%=_T('Insert media url')%>..." ,"' />",
																	"<i id='" , this.prefix_id , "_edit_popup_lang_en_header_media_url_validation' class='formFieldValidation formFieldError' hidden></i>",
																"</div>",
																"<div class='form-group d-flex align-items-end'>",
																	"<button id='", this.prefix_id, "_edit_popup_lang_en_header_media_getFile' class='pageButtonGeneric buttonColorWhite iconOnlyBtn' type='button' lang_header_media_getFile_selector='en' title='" + "<%=_T('Get url from file')%>" + "'>",
																		"<i class='far fa-paperclip'></i>",
																	"</button>",		
																	"<button id='", this.prefix_id, "_edit_popup_lang_en_header_media_delFile' class='pageButtonGeneric buttonColorWhite iconOnlyBtn ml-1' type='button' lang_header_media_delFile_selector='en' title='" + "<%=_T('Clean url')%>" + "'>",
																		"<i class='far fa-eraser'></i>",
																	"</button>",														
																"</div>",
															"</div>",																																							
														"</div>",
													"</div>",
													// body
													"<div class='card-block'>",
														"<div class='row pt-2'>",
															"<label for='" , this.prefix_id , "_edit_popup_lang_en_add_security_recomendation' class='form-check-label'>" ,
																"<input id='" , this.prefix_id , "_edit_popup_lang_en_add_security_recomendation' class='form-check-input' type='checkbox' security_recomendation_selector='en' />" ,
																"<%=_T('Add security recommendation')%>", 
															"</label>" ,
														"</div>",
														"<div class='row pt-2'>",
															"<div class='col-9 d-flex flex-column'>",
																"<div class='form-group'>",
																	"<label for='" , this.prefix_id , "_edit_popup_lang_en_message' ><%=_T('Text')%><%=_T(':')%>",
																		"<span class='fa fa-question-circle' title='", "<%=_T('Enter the message text.')%>", "'></span>",
																	"</label>",
																	"<textarea id='" , this.prefix_id , "_edit_popup_lang_en_message' class='form-control rowsBased' rows='4' type='text' maxlength='1024'></textarea>",
																	"<i id='" , this.prefix_id , "_edit_popup_lang_en_message_validation' class='formFieldValidation formFieldError' hidden></i>",
																"</div>",
															"</div>",
															"<div class='col-3 d-flex flex-column'>",
																"<div class='form-group flex-grow-1 position-relative'>",
																	"<label for='" + this.prefix_id + "_edit_popup_lang_en_params'><%=_T('Parameters')%><%=_T(':')%> <span class='fa fa-question-circle' title='", "<%=_T('Press enter to add parameters')%>" ,"' > </span></label>",
																	"<textarea id='" + this.prefix_id + "_edit_popup_lang_en_params' class='form-control fullColHeight'></textarea>",
																	"<i id='" + this.prefix_id + "_edit_popup_lang_en_params_validation' class='formFieldValidation formFieldError' hidden></i>",
																"</div>",
															"</div>",
														"</div>",
														"<div class='row pt-2'>",
															"<div class='col-12'>",
																"<div class='inlineMessagesContainer messageInfo mt-0'>",
																	"<%=_T('Add consecutive parameter placeholders ({{1}}, {{2}}, ...) to customize the message text. Add as many parameters as defined placeholders.')%>",
																"</div>",
															"</div>",														
														"</div>",
													"</div>",
													//footer
													"<div class='card-block'>",
														"<div class='row pt-2'>",
															"<label for='" , this.prefix_id , "_edit_popup_lang_en_add_code_expiration' class='form-check-label'>" ,
																"<input id='" , this.prefix_id , "_edit_popup_lang_en_add_code_expiration' class='form-check-input' type='checkbox' code_expiration_selector='en' />" ,
																"<%=_T('Code expiration (minutes)')%>", 
																"<input id='" , this.prefix_id , "_edit_popup_lang_en_footer_code_expiration_minutes' class='form-control' type='text' code_expiration_minutes_selector='en' ></input>",
																"<i id='" , this.prefix_id , "_edit_popup_lang_en_footer_code_expiration_minutes_validation' class='formFieldValidation formFieldError' hidden></i>",
															"</label>" ,
														"</div>",
														"<div class='row pt-2'>",
															"<div class='col-12'>",
																"<div class='form-group'>",
																	"<label for='" , this.prefix_id , "_edit_popup_lang_en_footer' ><%=_T('Footer text')%><%=_T(':')%>",
																		"<span class='fa fa-question-circle' title='", "<%=_T('Optional. Add a short line of text to the bottom of your message template.')%>", "'></span>",
																	"</label>",
																	"<input id='" , this.prefix_id , "_edit_popup_lang_en_footer' class='form-control rowsBased' type='text' maxlength='60'></input>",
																	"<i id='" , this.prefix_id , "_edit_popup_lang_en_footer_validation' class='formFieldValidation formFieldError' hidden></i>",
																"</div>",
															"</div>",
														"</div>",
													"</div>",
													// buttons
													"<div class='card-block'>",
														"<div class='row pt-2'>",
															"<div class='col-3'>",
																"<div class='form-group' title='<%=_T('Select buttons type')%>'>",
																	"<label for='" + this.prefix_id + "_edit_popup_lang_en_button_type' class='required'><%=_T('Button type')%><%=_T(':')%>",
																		"<span class='fa fa-question-circle' title='", "<%=_T('Optional. Create buttons that allow customers to reply to your message or take some action.')%>", "'></span>",
																	"</label>",
																	"<select id='" + this.prefix_id + "_edit_popup_lang_en_button_type' class='form-control' lang_button_type_selector='en'>",
																	"</select>",
																	"<i id='" + this.prefix_id + "_edit_popup_lang_en_button_type_validation' class='formFieldValidation formFieldError' hidden></i>",
																"</div>",
															"</div>",															
														"</div>",
														"<div class='row pt-2'>",
															"<div class='col-12'>",
																"<div id='" + this.prefix_id + "_edit_popup_lang_en_button_type_info' class='inlineMessagesContainer messageInfo mt-0' style='display:none'></div>",
															"</div>",
														"</div>",
														"<div id='", this.prefix_id, "_edit_popup_lang_en_call_to_action_buttons_container' class='row' style='display:none;'>",
															"<div class='col-12'>",
																"<div class='card-block'>",
																	"<ul id='" , this.prefix_id , "_edit_popup_lang_en_call_to_action_buttons' class='hsmManagementButtonsList'>",
																	"</ul>",
																"</div>",
															"</div>",
														"</div>",
														"<div id='", this.prefix_id, "_edit_popup_lang_en_quick_reply_buttons_container' class='row' style='display:none;'>",
															"<div class='col-12'>",
																"<div class='card-block'>",
																	"<ul id='" , this.prefix_id , "_edit_popup_lang_en_quick_reply_buttons' class='hsmManagementButtonsList'>",
																	"</ul>",
																"</div>",
															"</div>",
														"</div>",
													"</div>",
												"</div>",
												// TAB SPANISH CONTAINER
												"<div id='", this.prefix_id, "_languages_spanish_tab_container' class='formTabsBody' tab_lang_selector='es' style='display:none;'>",
													// language activation
													"<div class='row'>",
														"<div class='col-3'>",
															"<div class='form-group'>",
																"<div class='form-check'>" ,
																	"<label for='" , this.prefix_id , "_edit_popup_lang_es_selector' class='form-check-label form-check-label-title'>" ,
																		"<input id='" , this.prefix_id , "_edit_popup_lang_es_selector' class='form-check-input' type='checkbox' lang_message_selector='es' related_tab='" , this.prefix_id , "_languages_spanish_tab_container' />" ,
																		"<%=_T('Enable language')%>", 
																	"</label>" ,
																"</div>" ,
															"</div>",
														"</div>",
													"</div>",
													// language code, quality score, status and rejected reason
													"<div class='row'>",
														"<div class='col-3'>",
															"<div class='form-group'>",
																"<label for='" , this.prefix_id , "_edit_popup_lang_es_code' ><%=_T('Code')%><%=_T(':')%></label>",
																"<select id='" , this.prefix_id , "_edit_popup_lang_es_code' class='form-control' lang_code_selector='es'></select>",
															"</div>",
														"</div>",
														"<div class='col-3'>",
															"<div id='" , this.prefix_id , "_edit_popup_lang_es_quality_score_container'", " class='form-group'>",																
																"<label for='" , this.prefix_id , "_edit_popup_lang_es_quality_score' ><%=_T('Quality score')%><%=_T(':')%></label>",
																"<input id='" , this.prefix_id , "_edit_popup_lang_es_quality_score' class='form-control' type='text' lang_quality_score_selector='es' ></input>",
															"</div>",															
														"</div>",
														"<div class='col-3'>",
															"<div id='" , this.prefix_id , "_edit_popup_lang_es_code_status_container'", " class='form-group'>",
																"<label for='" , this.prefix_id , "_edit_popup_lang_es_code_status' ><%=_T('Status')%><%=_T(':')%>",
																	"<span class='fa fa-question-circle' title='", "<%=_T('Only Active, Rejected and Paused language templates can be edited.')%>", "<%=_T('Only Active language templates can be used')%>", "'></span>",
																"</label>",
																"<input id='" , this.prefix_id , "_edit_popup_lang_es_code_status' class='form-control' type='text' lang_status_selector='es' ></input>",
															"</div>",
														"</div>",
														"<div class='col-3'>",
															"<div id='" , this.prefix_id , "_edit_popup_lang_es_rejected_reason_container'", " class='form-group'>",																
																"<label for='" , this.prefix_id , "_edit_popup_lang_es_rejected_reason' ><%=_T('Rejected reason')%><%=_T(':')%></label>",
																"<input id='" , this.prefix_id , "_edit_popup_lang_es_rejected_reason' class='form-control' type='text' lang_rejected_reason_selector='es' ></input>",
															"</div>",															
														"</div>",
													"</div>",
													// header
													"<div class='card-block'>",
														"<div class='row pt-2'>",
															"<div class='col-3'>",
																"<div class='card-block'>",
																	"<div class='form-group'>",
																		"<label for='" , this.prefix_id , "_edit_popup_lang_es_header_format' ><%=_T('Header type')%><%=_T(':')%>",
																			"<span class='fa fa-question-circle' title='", "<%=_T('Optional. Add a title or media content to the header of your message template.')%>", "'></span>",
																		"</label>",
																		"<select id='" , this.prefix_id , "_edit_popup_lang_es_header_format' class='form-control' lang_header_format_selector='es' ></select>",
																		"<i id='" , this.prefix_id , "_edit_popup_lang_es_header_format_validation' class='formFieldValidation formFieldError' hidden></i>",
																	"</div>",
																"</div>",
															"</div>",																																								
														"</div>",
														"<div id='" , this.prefix_id , "_edit_popup_lang_es_header_text_container' class='row pt-2' style='display:none;'>",
															"<div class='col-9' >",
																"<div class='form-group'>",
																	"<label for='" , this.prefix_id , "_edit_popup_lang_es_header_text' ><%=_T('Header text')%><%=_T(':')%>",
																		"<span class='fa fa-question-circle' title='", "<%=_T('Add an optional parameter placeholder ({{1}}) to customize header text.')%>", "'></span>",
																	"</label>",
																	"<input id='" , this.prefix_id , "_edit_popup_lang_es_header_text' class='form-control' type='text' maxlength='60' placeholder='", "<%=_T('Enter the header text')%>..." ,"' />",
																	"<i id='" , this.prefix_id , "_edit_popup_lang_es_header_text_validation' class='formFieldValidation formFieldError' hidden></i>",
																"</div>",
															"</div>",	
															"<div class='col-3' >",
																"<div class='form-group'>",
																	"<label for='" , this.prefix_id , "_edit_popup_lang_es_example_header_text' ><%=_T('Parameter')%><%=_T(':')%>",
																		"<span class='fa fa-question-circle' title='", "<%=_T('Enter a header parameter')%>", "'></span>",
																	"</label>",
																	"<input id='" , this.prefix_id , "_edit_popup_lang_es_example_header_text' class='form-control' type='text' maxlength='60' />",
																	"<i id='" , this.prefix_id , "_edit_popup_lang_es_example_header_text_validation' class='formFieldValidation formFieldError' hidden></i>",
																"</div>",
															"</div>",																																							
														"</div>",
														"<div id='" , this.prefix_id , "_edit_popup_lang_es_header_media_container' class='row pt-2' style='display:none;'>",
															"<div class='col-12 d-flex' >",	
																"<div class='form-group flex-grow-1 position-relative'>",
																	"<label for='" , this.prefix_id , "_edit_popup_lang_es_header_media_url' ><%=_T('Header media url')%><%=_T(':')%></label>",
																	"<input id='" , this.prefix_id , "_edit_popup_lang_es_header_media_url' class='form-control' type='text' maxlength='2000' placeholder='", "<%=_T('Insert media url')%>..." ,"' />",
																	"<i id='" , this.prefix_id , "_edit_popup_lang_es_header_media_url_validation' class='formFieldValidation formFieldError' hidden></i>",
																"</div>",
																"<div class='form-group d-flex align-items-end'>",
																	"<button id='", this.prefix_id, "_edit_popup_lang_es_header_media_getFile' class='pageButtonGeneric buttonColorWhite iconOnlyBtn' type='button' lang_header_media_getFile_selector='es' title='" + "<%=_T('Get url from file')%>" + "'>",
																		"<i class='far fa-paperclip'></i>",
																	"</button>",
																	"<button id='", this.prefix_id, "_edit_popup_lang_es_header_media_delFile' class='pageButtonGeneric buttonColorWhite iconOnlyBtn ml-1' type='button' lang_header_media_delFile_selector='es' title='" + "<%=_T('Clean url')%>" + "'>",
																		"<i class='far fa-eraser'></i>",
																	"</button>",														
																"</div>",
															"</div>",																																							
														"</div>",
													"</div>",
													// body
													"<div class='card-block'>",
														"<div class='row pt-2'>",
															"<label for='" , this.prefix_id , "_edit_popup_lang_es_add_security_recomendation' class='form-check-label'>" ,
																"<input id='" , this.prefix_id , "_edit_popup_lang_es_add_security_recomendation' class='form-check-input' type='checkbox' security_recomendation_selector='es' />" ,
																"<%=_T('Add security recommendation')%>", 
															"</label>" ,
														"</div>",
														"<div class='row pt-2'>",
															"<div class='col-9 d-flex flex-column'>",
																"<div class='form-group'>",
																	"<label for='" , this.prefix_id , "_edit_popup_lang_es_message' ><%=_T('Text')%><%=_T(':')%>",
																		"<span class='fa fa-question-circle' title='", "<%=_T('Enter the message text.')%>", "'></span>",
																	"</label>",
																	"<textarea id='" , this.prefix_id , "_edit_popup_lang_es_message' class='form-control rowsBased' rows='4' type='text' maxlength='1024'></textarea>",
																	"<i id='" , this.prefix_id , "_edit_popup_lang_es_message_validation' class='formFieldValidation formFieldError' hidden></i>",
																"</div>",
															"</div>",
															"<div class='col-3 d-flex flex-column'>",
																"<div class='form-group flex-grow-1 position-relative'>",
																	"<label for='" + this.prefix_id + "_edit_popup_lang_es_params'><%=_T('Parameters')%><%=_T(':')%> <span class='fa fa-question-circle' title='", "<%=_T('Press enter to add parameters')%>" ,"' > </span></label>",
																	"<textarea id='" + this.prefix_id + "_edit_popup_lang_es_params' class='form-control fullColHeight'></textarea>",
																	"<i id='" + this.prefix_id + "_edit_popup_lang_es_params_validation' class='formFieldValidation formFieldError' hidden></i>",
																"</div>",
															"</div>",
														"</div>",
														"<div class='row pt-2'>",
															"<div class='col-12'>",
																"<div class='inlineMessagesContainer messageInfo mt-0'>",
																	"<%=_T('Add consecutive parameter placeholders ({{1}}, {{2}}, ...) to customize the message text. Add as many parameters as defined placeholders.')%>",
																"</div>",
															"</div>",														
														"</div>",														
													"</div>",
													//footer
													"<div class='card-block'>",
														"<div class='row pt-2'>",
															"<label for='" , this.prefix_id , "_edit_popup_lang_es_add_code_expiration' class='form-check-label'>" ,
																"<input id='" , this.prefix_id , "_edit_popup_lang_es_add_code_expiration' class='form-check-input' type='checkbox' code_expiration_selector='es' />" ,
																"<%=_T('Code expiration (minutes)')%>", 
																"<input id='" , this.prefix_id , "_edit_popup_lang_es_footer_code_expiration_minutes' class='form-control' type='text' code_expiration_minutes_selector='es' ></input>",
																"<i id='" , this.prefix_id , "_edit_popup_lang_es_footer_code_expiration_minutes_validation' class='formFieldValidation formFieldError' hidden></i>",
															"</label>" ,
														"</div>",
														"<div class='row pt-2'>",
															"<div class='col-12'>",
																"<div class='form-group'>",
																	"<label for='" , this.prefix_id , "_edit_popup_lang_es_footer' ><%=_T('Footer text')%><%=_T(':')%>",
																		"<span class='fa fa-question-circle' title='", "<%=_T('Optional. Add a short line of text to the bottom of your message template.')%>", "'></span>",
																	"</label>",
																	"<input id='" , this.prefix_id , "_edit_popup_lang_es_footer' class='form-control rowsBased' type='text' maxlength='60'></input>",
																	"<i id='" , this.prefix_id , "_edit_popup_lang_es_footer_validation' class='formFieldValidation formFieldError' hidden></i>",
																"</div>",
															"</div>",
														"</div>",
													"</div>",
													// buttons
													"<div class='card-block'>",
														"<div class='row pt-2'>",
															"<div class='col-3'>",
																"<div class='form-group' title='<%=_T('Select buttons type')%>'>",
																	"<label for='" + this.prefix_id + "_edit_popup_lang_es_button_type' class='required'><%=_T('Button type')%><%=_T(':')%>",
																		"<span class='fa fa-question-circle' title='", "<%=_T('Optional. Create buttons that allow customers to reply to your message or take some action.')%>", "'></span>",
																	"</label>",
																	"<select id='" + this.prefix_id + "_edit_popup_lang_es_button_type' class='form-control' lang_button_type_selector='es'>",
																	"</select>",
																	"<i id='" + this.prefix_id + "_edit_popup_lang_es_button_type_validation' class='formFieldValidation formFieldError' hidden></i>",
																"</div>",
															"</div>",
														"</div>",
														"<div class='row pt-2'>",
															"<div class='col-12'>",
																"<div id='" + this.prefix_id + "_edit_popup_lang_es_button_type_info' class='inlineMessagesContainer messageInfo mt-0' style='display:none'></div>",
															"</div>",
														"</div>",
														"<div id='", this.prefix_id, "_edit_popup_lang_es_call_to_action_buttons_container' class='row' style='display:none;'>",
															"<div class='col-12'>",
																"<div class='card-block'>",
																	"<ul id='" , this.prefix_id , "_edit_popup_lang_es_call_to_action_buttons' class='hsmManagementButtonsList'>",
																	"</ul>",
																"</div>",
															"</div>",
														"</div>",
														"<div id='", this.prefix_id, "_edit_popup_lang_es_quick_reply_buttons_container' class='row' style='display:none;'>",
															"<div class='col-12'>",
																"<div class='card-block'>",
																	"<ul id='" , this.prefix_id , "_edit_popup_lang_es_quick_reply_buttons' class='hsmManagementButtonsList'>",
																	"</ul>",
																"</div>",
															"</div>",
														"</div>",
													"</div>",
												"</div>",
												// TAB FRENCH CONTAINER
												"<div id='", this.prefix_id, "_languages_french_tab_container' class='formTabsBody' tab_lang_selector='fr' style='display:none;'>",
													// language activation
													"<div class='row'>",
														"<div class='col-3'>",
															"<div class='form-group'>",
																"<div class='form-check'>" ,
																	"<label for='" , this.prefix_id , "_edit_popup_lang_fr_selector' class='form-check-label form-check-label-title'>" ,
																		"<input id='" , this.prefix_id , "_edit_popup_lang_fr_selector' class='form-check-input' type='checkbox' lang_message_selector='fr' related_tab='" , this.prefix_id , "_languages_french_tab_container' />" ,
																		"<%=_T('Enable language')%>", 
																	"</label>" ,
																"</div>" ,
															"</div>",
														"</div>",													
													"</div>",
													// language code, status and rejected reason
													"<div class='row'>",
														"<div class='col-3'>",
															"<div class='form-group'>",
																"<label for='" , this.prefix_id , "_edit_popup_lang_fr_code' ><%=_T('Code')%><%=_T(':')%></label>",
																"<select id='" , this.prefix_id , "_edit_popup_lang_fr_code' class='form-control' lang_code_selector='fr'></select>",
															"</div>",
														"</div>",
														"<div class='col-3'>",
															"<div id='" , this.prefix_id , "_edit_popup_lang_fr_quality_score_container'", " class='form-group'>",																
																"<label for='" , this.prefix_id , "_edit_popup_lang_fr_quality_score' ><%=_T('Quality score')%><%=_T(':')%></label>",
																"<input id='" , this.prefix_id , "_edit_popup_lang_fr_quality_score' class='form-control' type='text' lang_quality_score_selector='fr' ></input>",
															"</div>",															
														"</div>",
														"<div class='col-3'>",
															"<div id='" , this.prefix_id , "_edit_popup_lang_fr_code_status_container'", " class='form-group'>",																
																"<label for='" , this.prefix_id , "_edit_popup_lang_fr_code_status' ><%=_T('Status')%><%=_T(':')%>",
																	"<span class='fa fa-question-circle' title='", "<%=_T('Only Active, Rejected and Paused language templates can be edited.')%>", "<%=_T('Only Active language templates can be used')%>", "'></span>",
																"</label>",
																"<input id='" , this.prefix_id , "_edit_popup_lang_fr_code_status' class='form-control' type='text' lang_status_selector='fr' ></input>",
															"</div>",
														"</div>",
														"<div class='col-3'>",
															"<div id='" , this.prefix_id , "_edit_popup_lang_fr_rejected_reason_container'", " class='form-group'>",																
																"<label for='" , this.prefix_id , "_edit_popup_lang_fr_rejected_reason' ><%=_T('Rejected reason')%><%=_T(':')%></label>",
																"<input id='" , this.prefix_id , "_edit_popup_lang_fr_rejected_reason' class='form-control' type='text' lang_rejected_reason_selector='fr' ></input>",
															"</div>",															
														"</div>",
													"</div>",
													// header
													"<div class='card-block'>",
														"<div class='row pt-2'>",
															"<div class='col-3'>",
																"<div class='card-block'>",
																	"<div class='form-group'>",
																		"<label for='" , this.prefix_id , "_edit_popup_lang_fr_header_format' ><%=_T('Header type')%><%=_T(':')%>",
																			"<span class='fa fa-question-circle' title='", "<%=_T('Optional. Add a title or media content to the header of your message template.')%>", "'></span>",
																		"</label>",
																		"<select id='" , this.prefix_id , "_edit_popup_lang_fr_header_format' class='form-control' lang_header_format_selector='fr' ></select>",
																		"<i id='" , this.prefix_id , "_edit_popup_lang_fr_header_format_validation' class='formFieldValidation formFieldError' hidden></i>",
																	"</div>",
																"</div>",
															"</div>",																																								
														"</div>",
														"<div id='" , this.prefix_id , "_edit_popup_lang_fr_header_text_container' class='row pt-2' style='display:none;'>",
															"<div class='col-9' >",
																"<div class='form-group'>",
																	"<label for='" , this.prefix_id , "_edit_popup_lang_fr_header_text' ><%=_T('Header text')%><%=_T(':')%>",
																		"<span class='fa fa-question-circle' title='", "<%=_T('Add an optional parameter placeholder ({{1}}) to customize header text.')%>", "'></span>",
																	"</label>",
																	"<input id='" , this.prefix_id , "_edit_popup_lang_fr_header_text' class='form-control' type='text' maxlength='60' placeholder='", "<%=_T('Enter the header text')%>..." ,"' />",
																	"<i id='" , this.prefix_id , "_edit_popup_lang_fr_header_text_validation' class='formFieldValidation formFieldError' hidden></i>",
																"</div>",
															"</div>",	
															"<div class='col-3' >",
																"<div class='form-group'>",
																	"<label for='" , this.prefix_id , "_edit_popup_lang_fr_example_header_text' ><%=_T('Parameter')%><%=_T(':')%>",
																		"<span class='fa fa-question-circle' title='", "<%=_T('Enter a header parameter')%>", "'></span>",
																	"</label>",
																	"<input id='" , this.prefix_id , "_edit_popup_lang_fr_example_header_text' class='form-control' type='text' maxlength='60' />",
																	"<i id='" , this.prefix_id , "_edit_popup_lang_fr_example_header_text_validation' class='formFieldValidation formFieldError' hidden></i>",
																"</div>",
															"</div>",																																						
														"</div>",
														"<div id='" , this.prefix_id , "_edit_popup_lang_fr_header_media_container' class='row pt-2' style='display:none;'>",
															"<div class='col-12 d-flex' >",	
																"<div class='form-group flex-grow-1 position-relative'>",
																	"<label for='" , this.prefix_id , "_edit_popup_lang_fr_header_media_url' ><%=_T('Header media url')%><%=_T(':')%></label>",
																	"<input id='" , this.prefix_id , "_edit_popup_lang_fr_header_media_url' class='form-control' type='text' maxlength='2000' placeholder='", "<%=_T('Insert media url')%>..." ,"' />",
																	"<i id='" , this.prefix_id , "_edit_popup_lang_fr_header_media_url_validation' class='formFieldValidation formFieldError' hidden></i>",
																"</div>",
																"<div class='form-group d-flex align-items-end'>",
																	"<button id='", this.prefix_id, "_edit_popup_lang_fr_header_media_getFile' class='pageButtonGeneric buttonColorWhite iconOnlyBtn' type='button' lang_header_media_getFile_selector='fr' title='" + "<%=_T('Get url from file')%>" + "'>",
																		"<i class='far fa-paperclip'></i>",
																	"</button>",		
																	"<button id='", this.prefix_id, "_edit_popup_lang_fr_header_media_delFile' class='pageButtonGeneric buttonColorWhite iconOnlyBtn ml-1' type='button' lang_header_media_delFile_selector='fr' title='" + "<%=_T('Clean url')%>" + "'>",
																		"<i class='far fa-eraser'></i>",
																	"</button>",														
																"</div>",
															"</div>",																																							
														"</div>",
													"</div>",
													// body
													"<div class='card-block'>",
														"<div class='row pt-2'>",
															"<label for='" , this.prefix_id , "_edit_popup_lang_fr_add_security_recomendation' class='form-check-label'>" ,
																"<input id='" , this.prefix_id , "_edit_popup_lang_fr_add_security_recomendation' class='form-check-input' type='checkbox' security_recomendation_selector='fr' />" ,
																"<%=_T('Add security recommendation')%>", 
															"</label>" ,
														"</div>",
														"<div class='row pt-2'>",
															"<div class='col-9 d-flex flex-column'>",
																"<div class='form-group'>",
																	"<label for='" , this.prefix_id , "_edit_popup_lang_fr_message' ><%=_T('Text')%><%=_T(':')%>",
																		"<span class='fa fa-question-circle' title='", "<%=_T('Enter the message text.')%>", "'></span>",
																	"</label>",
																	"<textarea id='" , this.prefix_id , "_edit_popup_lang_fr_message' class='form-control rowsBased' rows='4' type='text' maxlength='1024'></textarea>",
																	"<i id='" , this.prefix_id , "_edit_popup_lang_fr_message_validation' class='formFieldValidation formFieldError' hidden></i>",
																"</div>",
															"</div>",
															"<div class='col-3 d-flex flex-column'>",
																"<div class='form-group flex-grow-1 position-relative'>",
																	"<label for='" + this.prefix_id + "_edit_popup_lang_fr_params'><%=_T('Parameters')%><%=_T(':')%> <span class='fa fa-question-circle' title='", "<%=_T('Press enter to add parameters')%>" ,"' > </span></label>",
																	"<textarea id='" + this.prefix_id + "_edit_popup_lang_fr_params' class='form-control fullColHeight'></textarea>",
																	"<i id='" + this.prefix_id + "_edit_popup_lang_fr_params_validation' class='formFieldValidation formFieldError' hidden></i>",
																"</div>",
															"</div>",
														"</div>",
														"<div class='row pt-2'>",
															"<div class='col-12'>",
																"<div class='inlineMessagesContainer messageInfo mt-0'>",
																	"<%=_T('Add consecutive parameter placeholders ({{1}}, {{2}}, ...) to customize the message text. Add as many parameters as defined placeholders.')%>",
																"</div>",
															"</div>",														
														"</div>",
													"</div>",
													// footer
													"<div class='card-block'>",
														"<div class='row pt-2'>",
															"<label for='" , this.prefix_id , "_edit_popup_lang_fr_add_code_expiration' class='form-check-label'>" ,
																"<input id='" , this.prefix_id , "_edit_popup_lang_fr_add_code_expiration' class='form-check-input' type='checkbox' code_expiration_selector='fr' />" ,
																"<%=_T('Code expiration (minutes)')%>", 
																"<input id='" , this.prefix_id , "_edit_popup_lang_fr_footer_code_expiration_minutes' class='form-control' type='text' code_expiration_minutes_selector='fr' ></input>",
																"<i id='" , this.prefix_id , "_edit_popup_lang_fr_footer_code_expiration_minutes_validation' class='formFieldValidation formFieldError' hidden></i>",
															"</label>" ,
														"</div>",
														"<div class='row pt-2'>",
															"<div class='col-12'>",
																"<div class='form-group'>",
																	"<label for='" , this.prefix_id , "_edit_popup_lang_fr_footer' ><%=_T('Footer text')%><%=_T(':')%>",
																		"<span class='fa fa-question-circle' title='", "<%=_T('Optional. Add a short line of text to the bottom of your message template.')%>", "'></span>",
																	"</label>",
																	"<input id='" , this.prefix_id , "_edit_popup_lang_fr_footer' class='form-control rowsBased' type='text' maxlength='60'></input>",
																	"<i id='" , this.prefix_id , "_edit_popup_lang_fr_footer_validation' class='formFieldValidation formFieldError' hidden></i>",
																"</div>",
															"</div>",
														"</div>",
													"</div>",
													// buttons
													"<div class='card-block'>",
														"<div class='row pt-2'>",
															"<div class='col-3'>",
																"<div class='form-group' title='<%=_T('Select buttons type')%>'>",
																	"<label for='" + this.prefix_id + "_edit_popup_lang_fr_button_type' class='required'><%=_T('Button type')%><%=_T(':')%>",
																		"<span class='fa fa-question-circle' title='", "<%=_T('Optional. Create buttons that allow customers to reply to your message or take some action.')%>", "'></span>",
																	"</label>",
																	"<select id='" + this.prefix_id + "_edit_popup_lang_fr_button_type' class='form-control' lang_button_type_selector='fr'>",
																	"</select>",
																	"<i id='" + this.prefix_id + "_edit_popup_lang_fr_button_type_validation' class='formFieldValidation formFieldError' hidden></i>",
																"</div>",
															"</div>",
														"</div>",
														"<div class='row pt-2'>",
															"<div class='col-12'>",
																"<div id='" + this.prefix_id + "_edit_popup_lang_fr_button_type_info' class='inlineMessagesContainer messageInfo mt-0' style='display:none'></div>",
															"</div>",
														"</div>",
														"<div id='", this.prefix_id, "_edit_popup_lang_fr_call_to_action_buttons_container' class='row' style='display:none;'>",
															"<div class='col-12'>",
																"<div class='card-block'>",
																	"<ul id='" , this.prefix_id , "_edit_popup_lang_fr_call_to_action_buttons' class='hsmManagementButtonsList'>",
																	"</ul>",
																"</div>",
															"</div>",
														"</div>",
														"<div id='", this.prefix_id, "_edit_popup_lang_fr_quick_reply_buttons_container' class='row' style='display:none;'>",
															"<div class='col-12'>",
																"<div class='card-block'>",
																	"<ul id='" , this.prefix_id , "_edit_popup_lang_fr_quick_reply_buttons' class='hsmManagementButtonsList'>",
																	"</ul>",
																"</div>",
															"</div>",
														"</div>",
													"</div>",
												"</div>",
												// TAB PORTUGUESE CONTAINER
												"<div id='", this.prefix_id, "_languages_portuguese_tab_container' class='formTabsBody' tab_lang_selector='pt' style='display:none;'>",
													// language activation
													"<div class='row'>",
														"<div class='col-3'>",
															"<div class='form-group'>",
																"<div class='form-check'>" ,
																	"<label for='" , this.prefix_id , "_edit_popup_lang_pt_selector' class='form-check-label form-check-label-title'>" ,
																		"<input id='" , this.prefix_id , "_edit_popup_lang_pt_selector' class='form-check-input' type='checkbox' lang_message_selector='pt' related_tab='" , this.prefix_id , "_languages_portuguese_tab_container' />" ,
																		"<%=_T('Enable language')%>", 
																	"</label>" ,
																"</div>" ,
															"</div>",
														"</div>",
													"</div>",
													// language code, status and rejected reason
													"<div class='row'>",
														"<div class='col-3'>",
															"<div class='form-group'>",
																"<label for='" , this.prefix_id , "_edit_popup_lang_pt_code' ><%=_T('Code')%><%=_T(':')%></label>",
																"<select id='" , this.prefix_id , "_edit_popup_lang_pt_code' class='form-control' lang_code_selector='pt'></select>",
															"</div>",
														"</div>",
														"<div class='col-3'>",
															"<div id='" , this.prefix_id , "_edit_popup_lang_pt_quality_score_container'", " class='form-group'>",																
																"<label for='" , this.prefix_id , "_edit_popup_lang_pt_quality_score' ><%=_T('Quality score')%><%=_T(':')%></label>",
																"<input id='" , this.prefix_id , "_edit_popup_lang_pt_quality_score' class='form-control' type='text' lang_quality_score_selector='pt' ></input>",
															"</div>",															
														"</div>",
														"<div class='col-3'>",
															"<div id='" , this.prefix_id , "_edit_popup_lang_pt_code_status_container'", " class='form-group'>",
																"<label for='" , this.prefix_id , "_edit_popup_lang_pt_code_status' ><%=_T('Status')%><%=_T(':')%>",
																	"<span class='fa fa-question-circle' title='", "<%=_T('Only Active, Rejected and Paused language templates can be edited.')%>", "<%=_T('Only Active language templates can be used')%>", "'></span>",
																"</label>",
																"<input id='" , this.prefix_id , "_edit_popup_lang_pt_code_status' class='form-control' type='text' lang_status_selector='pt' ></input>",
															"</div>",
														"</div>",
														"<div class='col-3'>",
															"<div id='" , this.prefix_id , "_edit_popup_lang_pt_rejected_reason_container'", " class='form-group'>",																
																"<label for='" , this.prefix_id , "_edit_popup_lang_pt_rejected_reason' ><%=_T('Rejected reason')%><%=_T(':')%></label>",
																"<input id='" , this.prefix_id , "_edit_popup_lang_pt_rejected_reason' class='form-control' type='text' lang_rejected_reason_selector='pt' ></input>",
															"</div>",															
														"</div>",
													"</div>",
													// header
													"<div class='card-block'>",
														"<div class='row pt-2'>",
															"<div class='col-3'>",
																"<div class='card-block'>",
																	"<div class='form-group'>",
																		"<label for='" , this.prefix_id , "_edit_popup_lang_pt_header_format' ><%=_T('Header type')%><%=_T(':')%>",
																			"<span class='fa fa-question-circle' title='", "<%=_T('Optional. Add a title or media content to the header of your message template.')%>", "'></span>",
																		"</label>",
																		"<select id='" , this.prefix_id , "_edit_popup_lang_pt_header_format' class='form-control' lang_header_format_selector='pt' ></select>",
																		"<i id='" , this.prefix_id , "_edit_popup_lang_pt_header_format_validation' class='formFieldValidation formFieldError' hidden></i>",
																	"</div>",
																"</div>",
															"</div>",																																								
														"</div>",
														"<div id='" , this.prefix_id , "_edit_popup_lang_pt_header_text_container' class='row pt-2' style='display:none;'>",
															"<div class='col-9' >",
																"<div class='form-group'>",
																	"<label for='" , this.prefix_id , "_edit_popup_lang_pt_header_text' ><%=_T('Header text')%><%=_T(':')%>",
																		"<span class='fa fa-question-circle' title='", "<%=_T('Add an optional parameter placeholder ({{1}}) to customize header text.')%>", "'></span>",
																	"</label>",
																	"<input id='" , this.prefix_id , "_edit_popup_lang_pt_header_text' class='form-control' type='text' maxlength='60' placeholder='", "<%=_T('Enter the header text')%>..." ,"' />",
																	"<i id='" , this.prefix_id , "_edit_popup_lang_pt_header_text_validation' class='formFieldValidation formFieldError' hidden></i>",
																"</div>",
															"</div>",	
															"<div class='col-3' >",
																"<div class='form-group'>",
																	"<label for='" , this.prefix_id , "_edit_popup_lang_pt_example_header_text' ><%=_T('Parameter')%><%=_T(':')%>",
																		"<span class='fa fa-question-circle' title='", "<%=_T('Enter a header parameter')%>", "'></span>",
																	"</label>",
																	"<input id='" , this.prefix_id , "_edit_popup_lang_pt_example_header_text' class='form-control' type='text' maxlength='60' />",
																	"<i id='" , this.prefix_id , "_edit_popup_lang_pt_example_header_text_validation' class='formFieldValidation formFieldError' hidden></i>",
																"</div>",
															"</div>",																																							
														"</div>",
														"<div id='" , this.prefix_id , "_edit_popup_lang_pt_header_media_container' class='row pt-2' style='display:none;'>",
															"<div class='col-12 d-flex' >",	
																"<div class='form-group flex-grow-1 position-relative'>",
																	"<label for='" , this.prefix_id , "_edit_popup_lang_pt_header_media_url' ><%=_T('Header media url')%><%=_T(':')%></label>",
																	"<input id='" , this.prefix_id , "_edit_popup_lang_pt_header_media_url' class='form-control' type='text' maxlength='2000' placeholder='", "<%=_T('Insert media url')%>..." ,"' />",
																	"<i id='" , this.prefix_id , "_edit_popup_lang_pt_header_media_url_validation' class='formFieldValidation formFieldError' hidden></i>",
																"</div>",
																"<div class='form-group d-flex align-items-end'>",
																	"<button id='", this.prefix_id, "_edit_popup_lang_pt_header_media_getFile' class='pageButtonGeneric buttonColorWhite iconOnlyBtn' type='button' lang_header_media_getFile_selector='pt' title='" + "<%=_T('Get url from file')%>" + "'>",
																		"<i class='far fa-paperclip'></i>",
																	"</button>",		
																	"<button id='", this.prefix_id, "_edit_popup_lang_pt_header_media_delFile' class='pageButtonGeneric buttonColorWhite iconOnlyBtn ml-1' type='button' lang_header_media_delFile_selector='pt' title='" + "<%=_T('Clean url')%>" + "'>",
																		"<i class='far fa-eraser'></i>",
																	"</button>",														
																"</div>",
															"</div>",																																							
														"</div>",
													"</div>",
													// body
													"<div class='card-block'>",
														"<div class='row pt-2'>",
															"<label for='" , this.prefix_id , "_edit_popup_lang_pt_add_security_recomendation' class='form-check-label'>" ,
																"<input id='" , this.prefix_id , "_edit_popup_lang_pt_add_security_recomendation' class='form-check-input' type='checkbox' security_recomendation_selector='pt' />" ,
																"<%=_T('Add security recommendation')%>", 
															"</label>" ,
														"</div>",
														"<div class='row pt-2'>",
															"<div class='col-9 d-flex flex-column'>",
																"<div class='form-group'>",
																	"<label for='" , this.prefix_id , "_edit_popup_lang_pt_message' ><%=_T('Text')%><%=_T(':')%>",
																		"<span class='fa fa-question-circle' title='", "<%=_T('Enter the message text.')%>", "'></span>",
																	"</label>",
																	"<textarea id='" , this.prefix_id , "_edit_popup_lang_pt_message' class='form-control rowsBased' rows='4' type='text' maxlength='1024'></textarea>",
																	"<i id='" , this.prefix_id , "_edit_popup_lang_pt_message_validation' class='formFieldValidation formFieldError' hidden></i>",
																"</div>",
															"</div>",
															"<div class='col-3 d-flex flex-column'>",
																"<div class='form-group flex-grow-1 position-relative'>",
																	"<label for='" + this.prefix_id + "_edit_popup_lang_pt_params'><%=_T('Parameters')%><%=_T(':')%> <span class='fa fa-question-circle' title='", "<%=_T('Press enter to add parameters')%>" ,"' > </span></label>",
																	"<textarea id='" + this.prefix_id + "_edit_popup_lang_pt_params' class='form-control fullColHeight'></textarea>",
																	"<i id='" + this.prefix_id + "_edit_popup_lang_pt_params_validation' class='formFieldValidation formFieldError' hidden></i>",
																"</div>",
															"</div>",
														"</div>",
														"<div class='row pt-2'>",
															"<div class='col-12'>",
																"<div class='inlineMessagesContainer messageInfo mt-0'>",
																	"<%=_T('Add consecutive parameter placeholders ({{1}}, {{2}}, ...) to customize the message text. Add as many parameters as defined placeholders.')%>",
																"</div>",
															"</div>",														
														"</div>",
													"</div>",
													// footer
													"<div class='card-block'>",
														"<div class='row pt-2'>",
															"<label for='" , this.prefix_id , "_edit_popup_lang_pt_add_code_expiration' class='form-check-label'>" ,
																"<input id='" , this.prefix_id , "_edit_popup_lang_pt_add_code_expiration' class='form-check-input' type='checkbox' code_expiration_selector='pt' />" ,
																"<%=_T('Code expiration (minutes)')%>", 
																"<input id='" , this.prefix_id , "_edit_popup_lang_pt_footer_code_expiration_minutes' class='form-control' type='text' code_expiration_minutes_selector='pt' ></input>",
																"<i id='" , this.prefix_id , "_edit_popup_lang_pt_footer_code_expiration_minutes_validation' class='formFieldValidation formFieldError' hidden></i>",
															"</label>" ,
														"</div>",
														"<div class='row pt-2'>",
															"<div class='col-12'>",
																"<div class='form-group'>",
																	"<label for='" , this.prefix_id , "_edit_popup_lang_pt_footer' ><%=_T('Footer text')%><%=_T(':')%>",
																		"<span class='fa fa-question-circle' title='", "<%=_T('Optional. Add a short line of text to the bottom of your message template.')%>", "'></span>",
																	"</label>",
																	"<input id='" , this.prefix_id , "_edit_popup_lang_pt_footer' class='form-control rowsBased' type='text' maxlength='60'></input>",
																	"<i id='" , this.prefix_id , "_edit_popup_lang_pt_footer_validation' class='formFieldValidation formFieldError' hidden></i>",
																"</div>",
															"</div>",
														"</div>",
													"</div>",
													// buttons
													"<div class='card-block'>",
														"<div class='row pt-2'>",
															"<div class='col-3'>",
																"<div class='form-group' title='<%=_T('Select buttons type')%>'>",
																	"<label for='" + this.prefix_id + "_edit_popup_lang_pt_button_type' class='required'><%=_T('Button type')%><%=_T(':')%>",
																		"<span class='fa fa-question-circle' title='", "<%=_T('Optional. Create buttons that allow customers to reply to your message or take some action.')%>", "'></span>",
																	"</label>",
																	"<select id='" + this.prefix_id + "_edit_popup_lang_pt_button_type' class='form-control' lang_button_type_selector='pt'>",
																	"</select>",
																	"<i id='" + this.prefix_id + "_edit_popup_lang_pt_button_type_validation' class='formFieldValidation formFieldError' hidden></i>",
																"</div>",
															"</div>",
														"</div>",
														"<div class='row pt-2'>",
															"<div class='col-12'>",
																"<div id='" + this.prefix_id + "_edit_popup_lang_pt_button_type_info' class='inlineMessagesContainer messageInfo mt-0' style='display:none'></div>",
															"</div>",
														"</div>",
														"<div id='", this.prefix_id, "_edit_popup_lang_pt_call_to_action_buttons_container' class='row' style='display:none;'>",
															"<div class='col-12'>",
																"<div class='card-block'>",
																	"<ul id='" , this.prefix_id , "_edit_popup_lang_pt_call_to_action_buttons' class='hsmManagementButtonsList'>",
																	"</ul>",
																"</div>",
															"</div>",
														"</div>",
														"<div id='", this.prefix_id, "_edit_popup_lang_pt_quick_reply_buttons_container' class='row' style='display:none;'>",
															"<div class='col-12'>",
																"<div class='card-block'>",
																	"<ul id='" , this.prefix_id , "_edit_popup_lang_pt_quick_reply_buttons' class='hsmManagementButtonsList'>",
																	"</ul>",
																"</div>",
															"</div>",
														"</div>",
													"</div>",
												"</div>",
											"</div>",
										"</div>",
									"</div>",
									// END TABS
								"</div>",
							"</div>",
						"</div>",
						// MODAL FOOTER BUTTONS
						"<div class='internalPopUpButtons'>",
							"<button id='" , this.prefix_id , "_edit_popup_create_and_continue' class='pageButtonGeneric buttonColorBlue float-right'>" , "<%=_T('Confirm & continue')%>" , "</button>" ,
							"<button id='" , this.prefix_id , "_edit_popup_create' class='pageButtonGeneric buttonColorBlue float-right'>" , "<%=_T('Confirm')%>" , "</button>" ,
							"<button id='" , this.prefix_id , "_edit_popup_close' class='pageButtonGeneric buttonColorWhite float-right'>" , "<%=_T('Cancel')%>" , "</button>" ,
						"</div>" +
					"</div>",
				"</div>",
				// end - edit hsm popup
			"</div>",
		// disclaimer
		"<div class='row'>" ,
				"<div class='col-12'>" ,
					"<div style='display:none' class='inlineMessagesContainer messageInfo'>", "<%=_T('DISCLAIMER MESSAGE !!!.')%>", "</div>",
				"</div>",
		"</div>",
		));
	};

	hsm_management_cloud_api.prototype.set_events = function()
	{
		const self = this;

		if ( this.section.is_readonly() ) {
			return;
		}

		$(this.hsm_mng)
			.off("change")
			.on("change", function() {
				const disabled = !$(this).prop("checked");
				if (!disabled) {
					if (self.current_hsm_mng) {
						self.set_ctrls_codes();
					}
				}
				else {
				// -actualizo la configuracion del current hsm_mng cuando estoy editando una account existente.
				// -siempre se tiene que marcar como "enabled" para que los valores puedan ser cacheados como estaban
					self.update_current_hsm_mng(true);
				}
				self.disable_hsm_mng(disabled);
			}
		);

		const $container = $(this.container_id);

		// Evento de change para el checkbox de cada language
		$container
			.off("change", "[lang_message_selector]")
			.on("change", "[lang_message_selector]", function() {
				const $chk = $(this);
				self.disable_chk_related_tab($chk);
			}
		);

		// Evento de change para el checkbox de recomendación de seguridad de cada language
		$container
			.off("change", "[security_recomendation_selector]")
			.on("change", "[security_recomendation_selector]", function() {
				const $chk = $(this);
				const $category = $("#" + self.prefix_id + "_edit_popup_category");
				const lang_code = $chk.attr("security_recomendation_selector");
				const $message =  $("#" + self.prefix_id + "_edit_popup_lang_" + lang_code + "_message");
				$message.val($category.val() === "AUTHENTICATION" ? ($chk.prop("checked") ? AUTHENTICATION_DEFAULT_MESSAGE[lang_code] + " " + AUTHENTICATION_DEFAULT_SECURITY_RECOMENDATION_TEXT[lang_code] : AUTHENTICATION_DEFAULT_MESSAGE[lang_code]) : $message.val() || "");
			}
		);

		// Evento de change para el checkbox de caducidad del codigo de cada language
		$container
			.off("change", "[code_expiration_selector]")
			.on("change", "[code_expiration_selector]", function() {
				const $chk = $(this);
				const lang_code = $chk.attr("code_expiration_selector");
				const $footer =  $("#" + self.prefix_id + "_edit_popup_lang_" + lang_code + "_footer");
				const $code_expiration_minutes = $("#" + self.prefix_id + "_edit_popup_lang_" + lang_code + "_footer_code_expiration_minutes");
				$code_expiration_minutes.val($chk.prop("checked") ? $code_expiration_minutes.val() || AUTHENTICATION_DEFAULT_CODE_EXPIRATION_MINUTES : "");
				self.frm.disable_control($code_expiration_minutes, !$chk.prop("checked"), $chk.prop("checked"), false, true);
				$footer.val($chk.prop("checked") ? format(AUTHENTICATION_DEFAULT_FOOTER_TEXT[lang_code], $code_expiration_minutes.val() || "") : "");
			}
		);

		// Evento de change para el input de caducidad del codigo de cada language
		$container
			.off("change", "[code_expiration_minutes_selector]")
			.on("change", "[code_expiration_minutes_selector]", function() {
				const $control = $(this);
				const lang_code = $control.attr("code_expiration_minutes_selector");
				const $footer =  $("#" + self.prefix_id + "_edit_popup_lang_" + lang_code + "_footer");
				const $code_expiration = $("#" + self.prefix_id + "_edit_popup_lang_" + lang_code + "_add_code_expiration");
				$footer.val($code_expiration.prop("checked") ? format(AUTHENTICATION_DEFAULT_FOOTER_TEXT[lang_code], $control.val() || "") : "");
			}
		);

		// Evento de change para el combo de cada language
		$container
			.off("change", "[lang_code_selector]")
			.on("change", "[lang_code_selector]", function() {
				let disabled, is_required, keep_value;
				const $category = $("#" + self.prefix_id + "_edit_popup_category");
				const $lang_id = $(this);
				const lang_id = $lang_id.val();
				const lang_code = $lang_id.attr("lang_code_selector");

				const $lang_quality_score = $("#" + self.prefix_id + "_edit_popup_lang_" + lang_code + "_quality_score");
				const $lang_status = $("#" + self.prefix_id + "_edit_popup_lang_" + lang_code + "_code_status");
				const $lang_rejected_reason = $("#" + self.prefix_id + "_edit_popup_lang_" + lang_code + "_rejected_reason");
				const $lang_header_format =  $("#" + self.prefix_id + "_edit_popup_lang_" + lang_code + "_header_format");

				const $security_recomendation = $("#" + self.prefix_id + "_edit_popup_lang_" + lang_code + "_add_security_recomendation");
				const $message =  $("#" + self.prefix_id + "_edit_popup_lang_" + lang_code + "_message");
				const $params =  $("#" + self.prefix_id + "_edit_popup_lang_" + lang_code + "_params");
				const $footer =  $("#" + self.prefix_id + "_edit_popup_lang_" + lang_code + "_footer");
				const $code_expiration = $("#" + self.prefix_id + "_edit_popup_lang_" + lang_code + "_add_code_expiration");
				const $code_expiration_minutes = $("#" + self.prefix_id + "_edit_popup_lang_" + lang_code + "_footer_code_expiration_minutes");
				const $button_type =  $("#" + self.prefix_id + "_edit_popup_lang_" + lang_code + "_button_type");
				const $ca_buttons_container =  $("#" + self.prefix_id + "_edit_popup_lang_" + lang_code + "_call_to_action_buttons_container");
				const $qr_buttons_container =  $("#" + self.prefix_id + "_edit_popup_lang_" + lang_code + "_quick_reply_buttons_container");

				disabled = false;
				is_required = true;
				keep_value = true;

				if(lang_id) {
					let template = null; 
					if(self.m_is_new_hsm) {
						if(self.useSelectedTemplate) {
							const template_name = $("#" + self.prefix_id + "_select_popup_id").find("option:selected").text();
							template = self.waba_templates.find(element => element.name === template_name && element.language === lang_id);	
						}			
					}
					else {
						template = self.m_current_hsm.languages[lang_code];
						const template_name = $("#" + self.prefix_id + "_edit_popup_id").val();
						template = template || self.waba_templates.find(element => element.name === template_name && element.language === lang_id);	
					}

					if(template) {
						$lang_id.prop("waba_template_id", template.id);
						$lang_quality_score.val(_T(template.quality_score ? template.quality_score.score : ""));
						$lang_status.val(_T(template.status));
						$lang_status.prop("waba_template_status", template.status);
						$lang_rejected_reason.val(_T(template.rejected_reason));
						const header = template.components.find(component => component.type === "HEADER");
						const body = template.components.find(component => component.type === "BODY");
						const footer = template.components.find(component => component.type === "FOOTER");
						const buttons = template.components.find(component => component.type === "BUTTONS");

						$lang_header_format.val(header && header.format ? header.format : "NONE");
						// This is not directly informed from API, so we have do this shit: If text has two dots, template has the security recomendation flag enabled
						$security_recomendation.prop("checked", $category.val() === "AUTHENTICATION" && (body.text.match(/\./g) || []).length === 2);
						$message.val(body && body.text ? body.text : "");
						let params = body && body.params ? body.params : (body && body.example && body.example.body_text && Array.isArray(body.example.body_text) && body.example.body_text.length && Array.isArray(body.example.body_text[0]) ? body.example.body_text[0] : []);
						$params.val(params.length ? params.join("\n") : ""); 
						$footer.val(footer && footer.text ? footer.text : "");
						// From footer text, if it contains a number between 1 and 90, and category is AUTHENTICATION, set the $code_expiration checkbox
						// By now there is no other option, because API does not give us the value of this check
						let footer_words = $footer.val().split(" ");
						let code_expiration_minutes = footer_words.find(word => word === parseInt(word).toString() && word >= 1 && word <= 90);
						$code_expiration.prop("checked", $category.val() === "AUTHENTICATION" && code_expiration_minutes);
						$code_expiration_minutes.val(code_expiration_minutes || "");
						$button_type.val(buttons && buttons.buttons ? (buttons.buttons.find(button => button.type === "QUICK_REPLY") ? "quick_reply" : "call_to_action") : "none");

						if(!editableStatuses.includes(template.status)) {
							disabled = true;
							is_required = false;
							keep_value = true;
						}
					}
					else {
						$lang_id.removeProp("waba_template_id");
						$lang_quality_score.val("");
						$lang_status.val("N/A");
						$lang_status.removeProp("waba_template_status");
						$lang_rejected_reason.val("");
						$lang_header_format.val("NONE");
						$security_recomendation.prop("checked", false);
						$message.val($category.val() === "AUTHENTICATION" ? AUTHENTICATION_DEFAULT_MESSAGE[lang_code] : "");
						$params.val("");
						$code_expiration.prop("checked", false);
						$footer.val("");
						$button_type.val($category.val() === "AUTHENTICATION" ? "call_to_action" : "none");
					}
				}
				else {
					disabled = true;
					is_required = false;
					keep_value = false;
					$lang_id.removeProp("waba_template_id");
					$lang_quality_score.val("");
					$lang_status.val("");
					$lang_status.removeProp("waba_template_status");
					$lang_rejected_reason.val("");
					$lang_header_format.val("NONE");
					$security_recomendation.prop("checked", false);
					$message.val("");
					$params.val("");
					$code_expiration.prop("checked", false);
					$footer.val("");
					$button_type.val("none");
				}

				disabled = disabled || !editableCategories[$category.val()];
				is_required = !disabled;
				keep_value = !disabled || (disabled && !editableCategories[$category.val()]);
				let disabled_by_category = disabled || $category.val() === "AUTHENTICATION";

				self.hsmButtons[lang_code].buttons_counter = 0;

				self.frm.disable_control($lang_quality_score, true, false, false, true);
				self.frm.disable_control($lang_status, true, false, false, true);
				self.frm.disable_control($lang_rejected_reason, true, false, false, true);
				self.frm.disable_control($lang_header_format, disabled_by_category, false, false, keep_value);
				$security_recomendation.parent().toggle($category.val() === "AUTHENTICATION");
				
				self.frm.disable_control($security_recomendation, disabled, is_required, false, keep_value);
				self.frm.disable_control($message, disabled_by_category, is_required, false, keep_value);
				self.frm.disable_control($params, disabled, false, false, keep_value);
				$code_expiration.parent().toggle($category.val() === "AUTHENTICATION");
				
				self.frm.disable_control($code_expiration, disabled, is_required, false, keep_value);
				$code_expiration_minutes.parent().toggle($category.val() === "AUTHENTICATION");
				self.frm.disable_control($footer, disabled_by_category, false, false, keep_value);
				self.frm.disable_control($button_type, disabled_by_category, false, false, keep_value);
				$ca_buttons_container.find("[hsm_btn_text], [hsm_btn_autofill_text], [hsm_btn_action_type], [hsm_btn_phone], [hsm_btn_url], [hsmButton_remove], [hsmButton_add]").each(function(){
					self.frm.disable_control($(this), $(this).prop("disabled") || disabled, false, false, keep_value);
				});
				$qr_buttons_container.find("[hsm_btn_text], [hsm_btn_autofill_text], [hsm_btn_action_type], [hsm_btn_phone], [hsm_btn_url], [hsmButton_remove], [hsmButton_add]").each(function(){
					self.frm.disable_control($(this), $(this).prop("disabled") || disabled, false, false, keep_value);
				});
				
				$lang_header_format.change();
				$security_recomendation.change();
				$code_expiration.change();
				$button_type.change();

				$message.focus();
			}
		);

		// Evento de change para el combo de tipo de header de cada language
		$container
			.off("change", "[lang_header_format_selector]")
			.on("change", "[lang_header_format_selector]", function() {
				let disabled = false;
				const $category = $("#" + self.prefix_id + "_edit_popup_category");

				const header_format = $(this).val();
				const lang_code = $(this).attr("lang_header_format_selector");
				const $lang_id = $("#" + self.prefix_id + "_edit_popup_lang_" + lang_code + "_code");
				const lang_id = $lang_id.val();				

				const $header_text_container = $("#" + self.prefix_id + "_edit_popup_lang_" + lang_code + "_header_text_container");
				const $header_text = $("#" + self.prefix_id + "_edit_popup_lang_" + lang_code + "_header_text");
				const $example_header_text = $("#" + self.prefix_id + "_edit_popup_lang_" + lang_code + "_example_header_text");
				const $header_media_container = $("#" + self.prefix_id + "_edit_popup_lang_" + lang_code + "_header_media_container");
				const $header_media_url = $("#" + self.prefix_id + "_edit_popup_lang_" + lang_code + "_header_media_url");
				const $header_media_getFile = $("#" + self.prefix_id + "_edit_popup_lang_" + lang_code + "_header_media_getFile");
				const $header_media_delFile = $("#" + self.prefix_id + "_edit_popup_lang_" + lang_code + "_header_media_delFile");

				let header_text_visible = false;
				let header_text_readonly = true;
				let header_media_url_visible = false;
				let header_media_url_readonly = false;

				if(lang_id) {
					let template = null;
					if(self.m_is_new_hsm) {
						if(self.useSelectedTemplate) {
							const template_name = $("#" + self.prefix_id + "_select_popup_id").find("option:selected").text();
							template = self.waba_templates.find(element => element.name === template_name && element.language === lang_id);
						}
					}
					else {
						template = self.m_current_hsm.languages[lang_code];
						const template_name = $("#" + self.prefix_id + "_edit_popup_id").val();
						template = template || self.waba_templates.find(element => element.name === template_name && element.language === lang_id);	
					}

					let has_shared_file = false;
					if(template) {
						disabled = !editableStatuses.includes(template.status)
						const header = template.components.find(component => component.type === "HEADER");
						if(header) {
							$header_text.val(header.format && header.format === "TEXT" ? header.text : "");
							$example_header_text.val(header.format && header.format === "TEXT" && header.header_param ? header.header_param : (header.example && header.example.header_text && Array.isArray(header.example.header_text) && header.example.header_text.length ? header.example.header_text[0] : ""));
							header.mediaUrl = header.mediaUrl || (header.example && header.example.header_handle && Array.isArray(header.example.header_handle) && header.example.header_handle.length ? header.example.header_handle[0] : "");
							$header_media_url.val(header.mediaUrl);
							has_shared_file = header.sharedFile ? true : false;
						}
						else {
							$header_text.val("");
							$example_header_text.val("");
							$header_media_url.val("");
						}
					}
					else {
						$header_text.val("");
						$example_header_text.val("");
						$header_media_url.val("");
					}

					header_text_visible = header_format === "TEXT";
					header_text_readonly = false;

					header_media_url_visible = header_format === "VIDEO" || header_format === "DOCUMENT" || header_format === "IMAGE";
					header_media_url_readonly = $(this).prop("disabled") || has_shared_file;

					if(self.m_is_new_hsm) { 
						$header_media_url.removeProp("sharedFile");
					}
					else {
						const header = self.m_current_hsm && self.m_current_hsm.languages && self.m_current_hsm.languages[lang_code] && self.m_current_hsm.languages[lang_code].components && self.m_current_hsm.languages[lang_code].components.find(component => component.type === "HEADER");
						if(header && header.sharedFile) {
							$header_media_url.prop("sharedFile", JSON.stringify(header.sharedFile));
						}
						else {
							$header_media_url.removeProp("sharedFile");
						}
					}
				}
				else {
					disabled = true;
					$header_text.val("");
					$header_media_url.val("");
				}

				let disabled_by_category = disabled || !editableCategories[$category.val()];
				$header_text_container.toggle(header_text_visible);
				self.frm.readonly_control( $header_text, disabled_by_category || header_text_readonly, header_text_visible);
				self.frm.readonly_control( $example_header_text, disabled_by_category || header_text_readonly, false);

				$header_media_container.toggle(header_media_url_visible);	
				self.frm.readonly_control( $header_media_url, disabled_by_category || header_media_url_readonly, header_media_url_visible );
				
				self.frm.readonly_control( $header_media_getFile, disabled_by_category, false );
				self.frm.readonly_control( $header_media_delFile, disabled_by_category, false );
				
			});

		// Evento de change para el combo de typo de button de cada language
		$container
			.off("change", "[lang_button_type_selector]")
			.on("change", "[lang_button_type_selector]", function() {
				const button_type = $(this).val();
				const lang_code = $(this).attr("lang_button_type_selector");
				const $lang_id = $("#" + self.prefix_id + "_edit_popup_lang_" + lang_code + "_code");
				const lang_id = $lang_id.val();		
				self.hsmButtons[lang_code].buttons_counter = 0;	

				const $call_to_action_buttons_container = $("#" + self.prefix_id + "_edit_popup_lang_" + lang_code + "_call_to_action_buttons_container");
				$call_to_action_buttons_container.toggle(button_type === "call_to_action");

				const $quick_reply_buttons_container = $("#" + self.prefix_id + "_edit_popup_lang_" + lang_code + "_quick_reply_buttons_container");
				$quick_reply_buttons_container.toggle(button_type === "quick_reply");	

				// default first button
				let buttons_to_make = [
					{
						type: button_type === "quick_reply" ? "QUICK_REPLY" : "URL"
					}
				];

				if(lang_id) {
					let template = null;
					if(self.m_is_new_hsm) {
						if(self.useSelectedTemplate) {
							const template_name = $("#" + self.prefix_id + "_select_popup_id").find("option:selected").text();
							template = self.waba_templates.find(element => element.name === template_name && element.language === lang_id);
						}
					}
					else {
						template = self.m_current_hsm.languages[lang_code];
						const template_name = $("#" + self.prefix_id + "_edit_popup_id").val();
						template = template || self.waba_templates.find(element => element.name === template_name && element.language === lang_id);
					}	

					if(template) {
						const buttons = template.components.find(component => component.type === "BUTTONS");
						if(buttons && !((button_type === "quick_reply" && buttons.buttons.find(button => button.type !== "QUICK_REPLY")) || (button_type !== "quick_reply" && buttons.buttons.find(button => button.type === "QUICK_REPLY")))) {
							buttons_to_make = buttons.buttons;
						}
					}		

					let templateCategory = template ? template.category : $("#" + self.prefix_id + "_edit_popup_category").val();
					
					if(button_type !== "none") {
						const $button_list = $("#" + self.prefix_id + "_edit_popup_lang_" + lang_code + "_" + button_type + "_buttons");
						self.hsmButtons[lang_code].__makeButtons({buttons: buttons_to_make, button_list: $button_list, templateCategory: templateCategory, lang_code: lang_code});
					}

					let button_type_info = "";
					switch(button_type){
						case "quick_reply":
							button_type_info = "<%=_T('Create up to 3 buttons that allow customers to reply to your message.')%>";
							break;
						case "call_to_action":
							button_type_info = templateCategory === "AUTHENTICATION" ? "<%=_T('Define how customers will use the received code.')%>" : "<%=_T('Create up to 2 buttons that allow customers to perform some action.')%>";
							break;
					}
					const $button_type_info = $("#" + self.prefix_id + "_edit_popup_lang_" + lang_code + "_button_type_info");
					$button_type_info.toggle(button_type && button_type !== "none").text(button_type_info);
				}
			}
		);

		// Evento de change para el boton para abrir el popup de shared files de cada language
		$container
			.off("click", "[lang_header_media_getFile_selector]")
			.on("click", "[lang_header_media_getFile_selector]", function() {
				const $parent = $(this);
				self.show_popup_shared_file($parent);
			}
		);

		// Evento de change para el boton para eliminar el shared file de cada language
		$container
			.off("click", "[lang_header_media_delFile_selector]")
			.on("click", "[lang_header_media_delFile_selector]", function() {
				const lang_code = $(this).attr("lang_header_media_delFile_selector");
				const $header_media_url = $("#" + self.prefix_id + "_edit_popup_lang_" + lang_code + "_header_media_url");	
				$header_media_url.val("");
				self.frm.readonly_control( $header_media_url, false, true );
				$header_media_url.removeProp("sharedFile");
			}
		);
	};

	hsm_management_cloud_api.prototype.supports_integration = function(integration_type)
	{
		const supported_integrations = { WEB : false };
		return supported_integrations[integration_type] !== false;
	};

	hsm_management_cloud_api.prototype.disable_chk_related_tab = function($chk)
	{
		const self = this;
		let disabled, is_required;
		const checked = $chk.prop("checked");
		const $related_tab = $("#" + $chk.attr("related_tab"));
		const lang_code = $related_tab.attr("tab_lang_selector");

		const $lang_code =  $("#" + this.prefix_id + "_edit_popup_lang_" + lang_code + "_code");
		
		if (checked) {
			disabled = false;
			is_required = true;
			let template = null; 
			let language_codes = this.get_language_codes(lang_code);

			for (let item of language_codes) {
				let item_template = null;
				if(self.m_is_new_hsm) {
					if(self.useSelectedTemplate) {
						const template_name = $("#" + this.prefix_id + "_select_popup_id").find("option:selected").text();
						item_template = self.waba_templates.find(element => element.name === template_name && element.language === item.id);	
					}			
				}
				else {
					item_template = self.m_current_hsm.languages[item.id];
				}
				if(item_template) {
					// Agrego atributo 'bold' para despues marcar en bold este valor del combo, para indicar que este language existe en el template
					language_codes.find(l => l.id === item.id).bold = true;
				}
				else {
					const template_name = $("#" + self.prefix_id + "_edit_popup_id").val();
					item_template = item_template || self.waba_templates.find(element => element.name === template_name && element.language === item.id);	
				}
				if(!template && item_template) {
					template = item_template;
				}
			}

			this.frm.fill_combo($lang_code, false, language_codes, "id", "label");
			// Ponemos en bold las opciones que existen en el template
			$lang_code.find("option").each(function(){
				let $this = $(this);
				if(language_codes.find(l => l.id === $this.val()).bold) {
					$this.attr("style", "font-weight:bold;");
				}
			});

			if(template) {
				$lang_code.val(template.language || template.code);
			}
			
		}
		else {
			disabled = true;
			is_required = false;
		}
		this.frm.disable_control($lang_code, disabled, is_required);
		$lang_code.change();
	};

	hsm_management_cloud_api.prototype.validate_required = function()
	{
		let response = {
			success: true
		};
		if (!$(this.hsm_mng).prop("checked")) {
			return response;
		}
		let success = this.m_grd_hsm_list.start_validation().success;
		if (!success) {
			response.success = false;
			response.message = this.m_grd_hsm_list.get_validation_message_err("required");
		}
		return response;
	};

	hsm_management_cloud_api.prototype.show_popup_shared_file = function($parent, _, item)
	{
		const self = this;
		const lang_code = $parent.attr("lang_header_media_getFile_selector");
		const header_format = $("#" + this.prefix_id + "_edit_popup_lang_" + lang_code + "_header_format option:selected").val();
		const header_format_text = $("#" + this.prefix_id + "_edit_popup_lang_" + lang_code + "_header_format option:selected").text();
		const media_type = header_media_types[header_format];
		const $header_media_url = $("#" + this.prefix_id + "_edit_popup_lang_" + lang_code + "_header_media_url");
		let allowedContentTypes = {
			"image/png" : true,
			"image/jpeg" : true,
			"video/mp4" : true,
			"application/pdf" : true
		};
		this.m_sharedFilesPopupList = null;
		this.m_sharedFilesPopupList = new inConcert.SharedFilesPopupList({
			from_app            : "administration",
			owner               : this,
			popupId             : "adm_form_campaign_accounts_messages_shared_file_popup_container",
			search_files_type_id: media_type,
			abm_files_type_id   : media_type,
			visibility          : {
				AgentVisible          : true,
				AccountMessagesVisible: false,
				FlowDesignerVisible   : true
			},
			callbacks: {
				on_show: function() {},
				on_close: function(selected_file) {
					if (!selected_file) {
						// Si cerro el popup por cancelar
						return;
					}
					// Si cerro el popup por guardar
					let errorMsg;
					const show_shared_file_error = function() {
						self.show_ctrl_err({
							success: false, 
							$ctrl: $header_media_url,
							item: null,
							msg: errorMsg
						});
						$header_media_url.val("");
						$header_media_url.removeProp("sharedFile");
						self.frm.readonly_control( $header_media_url, false, true );
						$(self.container_id).find(".internalPopUpMiddle").animate({scrollTop: 0}, 250, "linear");
					};
					if(!allowedContentTypes[selected_file.ContentType]) {
						errorMsg = format("<%=_T('Invalid media: {0}. Valid formats for {1} type: {2}')%>", selected_file.ContentType, _T(header_format_text), inConcert.FileUploaderTypes[media_type].extensions);
						show_shared_file_error();
						return;
					}
					// Aqui es donde hay que llamar al server para que asocie una URL publica al shared file y nos la devuelva
					let campaign = self.parent.parent.frm.get_entity_instance();
					inConcert.invoke({
						url: inConcert.app.getBaseDomain() + "/inconcert/apps/administration/campaigns/whatsapp_get_cloud_api_public_repository_file_url/",
						type: "POST",
						data: JSON.stringify({
							campaignId : campaign.Id,
							accountId : (self.parent.current_account && self.parent.current_account.id) ? self.parent.current_account.id : self.parent.m_authorizingId,
							fileId: selected_file.RepositoryId,
							fileName: selected_file.Name
						}),
						dataType: "json",
						contentType: "application/json",
						success: function(response) {
							if(response.result && response.result === "OK") {
								$header_media_url.val( response.publicFileUrl || "");
								const sharedFileObj = JSON.stringify({
									fileId: selected_file.RepositoryId,
									fileName: selected_file.Name,
									contentType: selected_file.ContentType,
									type: media_type
								});
								$header_media_url.prop("sharedFile", sharedFileObj);
								self.frm.readonly_control( $header_media_url, true, false );
							}
							else {
								errorMsg = response.reason && _T(response.reason) || "<%=_T('Unknown error')%>";
								show_shared_file_error();
							}
						},
						error: function(_, text_status) {
							errorMsg = text_status.reason && _T(text_status.reason) || "<%=_T('Unknown error')%>";
							show_shared_file_error();
						}
					});					

				}
			}
		});

		const media_type_options = {
			whatsapp_cloud_image : "image",
			whatsapp_cloud_video : "video",
			whatsapp_cloud_document : "file"
		};
		this.m_sharedFilesPopupList.show({
			parentNode: $parent,
			option_id : media_type_options[media_type],
			can_create_new_file: true
		});
		if (item) {
			let file = inConcert.Utils.cloneObject(item);
			$.extend(true, file, item.RawData || {});
			delete file.RawData;
			this.m_sharedFilesPopupList.edit_file(file);
		}
	};	

	hsm_management_cloud_api.prototype.get_grid_value = function(item, field_id)
	{
		let default_status = "<%=_T('N/A')%>";
		var value = {
			original_value: default_status,
			value         : default_status,
			search_value  : default_status
		};

		var item_value = item.languages && item.languages[field_id] && item.languages[field_id].status;
		if (item_value === undefined) {
			return value;
		}

		value.original_value = _T(item_value);
		value.value = _T(item_value);
		value.search_value = _T(item_value);

		return value;
	};	

	hsm_management_cloud_api.prototype.load_waba_templates = function(item_to_open, is_new)
	{
		const self = this;
		if(!self.parent.current_account) {
			return;
		}
		self.loading_waba_templates = true;
		self.parent.parent.frm.dashboard.show_waiting("<%=_T('Retrieving whatsapp templates')%>");
		inConcert.invoke({
			url: inConcert.app.getBaseDomain() + "/inconcert/apps/administration/campaigns/whatsapp_get_message_templates/",
			type: "POST",
			data: JSON.stringify({
				campaignId : self.frm.campaign.Id,
				accountId : self.parent.current_account.id,
				wabaId: self.parent.current_account.wabaId
			}),
			dataType: "json",
			contentType: "application/json",
			success: function(response) {
				self.waba_templates = response.result === "OK" ? response.templates : [];
				// De los waba_templates, que contiene un elemento por cada par template/language, me quedo con los templates SIN repetir por cada idioma
				self.unique_templates_by_name = [];
				for (let item of self.waba_templates) {
					if(!self.unique_templates_by_name.find(element => element.name === item.name)) {
						self.unique_templates_by_name.push({name:item.name, category:item.category});
					}
				}		
				self.loading_waba_templates = false;	
				if(self.launch_on_open_popup_after_load_templates) {
					showWaiting("#" + self.prefix_id + "_edit_popup_waiting", false);
					self.on_open_popup(self.item_to_open);
				}
				else {
					self.refresh_remplates(item_to_open);
					if(item_to_open || is_new) {
						self.on_open_popup(item_to_open);
					}
					else {
						self.set_ctrls_codes();
					}
				}
				self.parent.parent.frm.dashboard.hide_waiting();
			},
			error: function() {
				self.waba_templates = [];
				self.unique_templates_by_name = [];
				self.loading_waba_templates = false;	
				if(self.launch_on_open_popup_after_load_templates) {
					showWaiting("#" + self.prefix_id + "_edit_popup_waiting", false);
					self.on_open_popup(self.item_to_open);
				}
				else {
					self.refresh_remplates(item_to_open);
					if(item_to_open || is_new) {
						self.on_open_popup(item_to_open);
					}
					else {
						self.set_ctrls_codes();
					}
				}
				self.parent.parent.frm.dashboard.hide_waiting();
			}
		});
	};	

}());
