local oo = require "loop.base"
local dateClass = require "LuaOrm.Date"
require "LuaOrm.Mapper"
require "inconcert.translators"
local jsonEncoder = require "LuaOrm.Serializers.Json"
local rpairs = LuaOrm.Associations.AssociationArray.Iterator

local models = require("apps.agent.model"):GetModels()

local m_moduleLocation = select(2, ...)

module(..., package.seeall, inconcert.orbit.application.Decorate)

m_htmlRender = nil

function MapUris(module, htmlRender, prefix)
	
	local LogRequest = require("apps.agent.requestFilters").LogRequest

	module:dispatch_get(  LogRequest .. GetLocalizedMessages, prefix.."/load_localized_messages/(.*)")
	m_htmlRender = htmlRender
end

local function messageCompleted(poMessage)
	return poMessage.msgid and poMessage.msgstr
end

local function unquoteString(str)
    return (gsub(str,"^([\"\'])(.*)%1$","%2"))
end

local function PO2JSON(baseDir)
	local translatedStrings = {}
	local poFile = string.format( [[%s\LC_MESSAGES\agent.po]], baseDir)
	local f = io.open(poFile, "r")
	if (f) then
		--read the file line by line and start constructing the resulting array
		local poMessage = {}
		local match
		while true do
			local line = f:read("*l")
			if (line == nil) then
				--EOF
				break
			else
				match = string.match(line, "^msgid%s+(.*)")
				if match then
					poMessage.msgid = unquoteString(match)
				else
					match = string.match(line, "^msgstr%s+(.*)")
					if match then
						poMessage.msgstr = unquoteString(match)
						if (messageCompleted(poMessage)) then
							translatedStrings[poMessage.msgid] = poMessage.msgstr
							poMessage = {}
						end
					end
				end
			end
		end
		f:close()
		if (messageCompleted(poMessage)) then
			translatedStrings[poMessage.msgid] = poMessage.msgstr
		end
	end
	return json.encode(translatedStrings)
end

--Given the current user language returns a JSON converted .po with the translated strings
function GetLocalizedMessages(web)
	local appLocation = GetAppLocation()
	local userLanguage = string.lower(inconcert.authUser.languageId)
	local partialMatch = nil
	--go search for a matching directory languages containing a po file to convert
	local languagesDir = appLocation .. "\\languages"
	for dirName in lfs.dir(languagesDir) do
		if dirName ~= "." and dirName ~= ".." then
			local f = languagesDir..'/'..dirName
			local attr = lfs.attributes (f)
			if attr.mode == "directory" then
				--if perfect match, convert it
				if (string.lower(dirName) == userLanguage) then
					return PO2JSON(dirName)
				end
				local match = string.match(dirName, [[(.+).lua]])
				if match then
					partialMatch = dirName
				end
			end
		end
	end
	--finished searching for language to convert, there was no perfect match, it I found a partial one use that
	if (partialMatch) then
		return PO2JSON(partialMatch)
	end
	return nil
end
