local m_models = require("apps.campaign_analytics.model"):GetModels()
module(..., package.seeall, inconcert.orbit.application.Decorate)
local utils = require "shared.apps.utils"
local helpers = require "helpers"
local permissions = require "shared.permissions.permissions"
m_htmlRender = nil
local m_outboundEngineClient = require "apps.outbound_engine.outboundengineclient"
require "lua_telephonyproxyclient"

local LOG_CONTEXT = "agent:outbound_engine"

local m_telephonyProxyClientInstance = nil
local function GetTelephonyProxyClient ()
	if not m_telephonyProxyClientInstance then
		m_telephonyProxyClientInstance = lua_telephonyproxyclient.LuaTelephonyProxyClient(
			inconcert.environment.apps.agent.cluster, "webhandler")
	end
	m_telephonyProxyClientInstance:ConfigureCassandra({
		{
			host = inconcert.environment.apps.agent.cassandra.ip,
			port = inconcert.environment.apps.agent.cassandra.port
		}
	})
	m_telephonyProxyClientInstance:ConfigureTimeClient(
		inconcert.environment.apps.agent.timeServers)
	return m_telephonyProxyClientInstance
end

local function CheckPermission(permission, method)
	return method
	--return permissions:CheckPermission("campaign_analytics", permission, method)
end

function MapUris(m, htmlRender, prefix)
	m_htmlRender = htmlRender
	local LogRequest = require("apps.agent.requestFilters").LogRequest

	local function DoGet(permission, request)
		return utils.JsonReply .. LogRequest .. CheckPermission(permission, request)
	end

	local function DoRequest(permission, request)
		return utils.JsonValidatorReply .. LogRequest .. CheckPermission(permission, request)
	end

	m:dispatch_get(DoGet("", GetCampaigns), prefix.."/get_campaigns/")
	m:dispatch_get(DoGet("", GetCampaignBatches), prefix.."/get_batches/(.+)")
	m:dispatch_get(DoGet("", GetCampaignBatch), prefix.."/get_batch/(.+)/(.+)")
	m:dispatch_get(DoGet("", GetOutboundContact), prefix.."/get_contact/(.+)")
	m:dispatch_post(DoRequest("", GetOutboundContactByName), prefix.."/get_contact_by_name/")
	m:dispatch_post(DoRequest("", GetOutboundContactByAddress), prefix.."/get_contact_by_address/")
	m:dispatch_post(DoRequest("", SearchOutboundContacts), prefix.."/search_contacts/")
	m:dispatch_post(DoRequest("", GetOutboundContactResults), prefix.."/get_contact_results/")
	m:dispatch_post(DoRequest("", StoreOutboundContact), prefix.."/store_contact/")
	m:dispatch_post(DoRequest("", SetOutboundContactAddresses), prefix.."/set_contact_addresses/")
	m:dispatch_post(DoRequest("", SetOutboundContactValues), prefix.."/set_contact_values/")
	m:dispatch_post(DoRequest("", SetOutboundContactSkills), prefix.."/set_contact_skills/")
	m:dispatch_post(DoRequest("", RemoveOutboundContactAddress), prefix.."/remove_contact_address/")
	m:dispatch_post(DoRequest("", EnableOutboundContactAddress), prefix.."/enable_contact_address/")
	m:dispatch_post(DoRequest("", DisableOutboundContactAddress), prefix.."/disable_contact_address/")
	m:dispatch_post(DoRequest("", RescheduleOutboundContact), prefix.."/reschedule_contact/")
	m:dispatch_post(DoRequest("", AddContactToOutboundBatch), prefix.."/add_contact/")
	m:dispatch_post(DoRequest("", PauseOutboundContact), prefix.."/pause_contact/")
	m:dispatch_post(DoRequest("", ResumeOutboundContact), prefix.."/resume_contact/")
	m:dispatch_post(DoRequest("", CancelOutboundContact), prefix.."/cancel_contact/")
	m:dispatch_post(DoRequest("", FinishOutboundContact), prefix.."/finish_contact/")
	m:dispatch_post(DoRequest("", RemoveOutboundContact), prefix.."/remove_contact/")
	m:dispatch_post(DoRequest("", RecycleOutboundContact), prefix.."/recycle_contact/")
	m:dispatch_post(DoRequest("", SetAgentOutboundContact), prefix.."/set_agent_contact/")
	m:dispatch_post(DoRequest("", ChangePriorityOutboundContact), prefix.."/change_contact_priority/")
	m:dispatch_post(DoRequest("", SetEngineTypeOutboundContact), prefix.."/set_contact_engine_type/")
	m:dispatch_post(DoRequest("", GetOutboundEngineActionStatus), prefix.."/action_status/")
	m:dispatch_post(DoRequest("", ValidateAddress), prefix.."/validate_address/")
end

local function ProcessResult(web, method, requestStr, result)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	if result.result ~= "OK" then
		web.status = 500
		LogError("[%s:%s] %s %s - %s", LOG_CONTEXT, method, userId, requestStr, json.encode(result))
	end
	return result
end

local function OeMethod(web, method, request)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local mockIndicators = inconcert.environment.apps.campaign_analytics.mock_indicators
	if mockIndicators then
		request.mockIndicators = true
	end
	local requestStr = json.encode(request)

	LogInfo("[%s:%s] %s %s", LOG_CONTEXT, method, userId, requestStr)
	local client = m_outboundEngineClient.GetClient()
	local result = client[method](client, requestStr)
	return ProcessResult(web, method, requestStr, result)
end

local function UpdateDataIfExists(data, updatedData, field)
	if updatedData[field] ~= nil then
		data[field] = updatedData[field]
	end
	return data
end

local function UpdateProcessData(process, updatedData)
	for channel, channelData in pairs(updatedData.channels or {}) do
		process.channels[channel] = UpdateChannelData(process.channels[channel], channelData)
	end
	process = UpdateDataIfExists(process, updatedData, "maxContactRetries")
	process = UpdateDataIfExists(process, updatedData, "maxContactRetriesDay")
	return process
end

local function MakeGetContactParams(contactId)
	return {
		vcc = inconcert.authUser.virtualContactCenter,
		contactId = contactId
	}
end

local function MakeGetContactByNameParams(data)
	return {
		vcc = inconcert.authUser.virtualContactCenter,
		name = data.name
	}
end

local function MakeGetContactByAddressParams(data)
	return {
		vcc = inconcert.authUser.virtualContactCenter,
		addressType = data.addressType,
		address = data.address,
	}
end

local function MakeSearchContactsParams(data)
	return {
		vcc = inconcert.authUser.virtualContactCenter,
		processId = data.processId,
		campaignId = data.campaignId,
		batchId = data.batchId,
		skillId = data.skillId,
		size = data.size,
		conditions = data.conditions,
		pit = data.pit,
		searchAfter = data.searchAfter,
		keepOpen = data.keepOpen,
		sortTerms = data.sortTerms,
		fields = data.fields,
		fullData = data.fullData,
	}
end

local function MakeGetContactResultsParams(data)
	return {
		vcc = inconcert.authUser.virtualContactCenter,
		processId = data.processId,
		campaignId = data.campaignId,
		batchId = data.batchId,
		contactId = data.contactId,
	}
end

local function MakeStoreContactParams(data)
	local contact = data.contact
	contact.Id = data.contactId
	return {
		vcc = inconcert.authUser.virtualContactCenter,
		contacts = { contact }
	}
end

local function MakeSetContactAddressesParams(data)
	return {
		vcc = inconcert.authUser.virtualContactCenter,
		contactId = data.contactId,
		addresses = data.addresses,
		mix = data.mix,
	}
end

local function MakeSetContactValuesParams(data)
	return {
		vcc = inconcert.authUser.virtualContactCenter,
		contactId = data.contactId,
		simple = data.simple,
		date = data.date,
		numeric = data.numeric,
		text = data.text,
		mix = data.mix,
	}
end

local function MakeSetContactSkillsParams(data)
	return {
		vcc = inconcert.authUser.virtualContactCenter,
		contactId = data.contactId,
		skills = data.skills,
		mix = data.mix,
	}
end

local function MakeRemoveContactAddressParams(data)
	return {
		vcc = inconcert.authUser.virtualContactCenter,
		contactId = data.contactId,
		address = data.address,
		addressType = data.addressType,
	}
end

local function MakeEnableContactAddressParams(data)
	return {
		vcc = inconcert.authUser.virtualContactCenter,
		contactId = data.contactId,
		address = data.address,
		addressType = data.addressType,
	}
end

local function MakeDisableContactAddressParams(data)
	return {
		vcc = inconcert.authUser.virtualContactCenter,
		contactId = data.contactId,
		address = data.address,
		addressType = data.addressType,
	}
end

local function MakeRescheduleContactParams(data)
	return {
		vcc = inconcert.authUser.virtualContactCenter,
		processId = data.processId,
		campaignId = data.campaignId,
		batchId = data.batchId,
		contactId = data.contactId,
		configuration = data.configuration,
		cause = "API"
	}
end

local function MakeAddContactParams(data)
	return {
		vcc = inconcert.authUser.virtualContactCenter,
		processId = data.processId,
		campaignId = data.campaignId,
		batchId = data.batchId,
		scope = {
			contactId = data.contactId,
		},
		configuration = data.configuration,
	}
end

local function MakeContactActionParams(data)
	return {
		vcc = inconcert.authUser.virtualContactCenter,
		processId = data.processId,
		campaignId = data.campaignId,
		scope = {
			batchId = data.batchId,
			contactId = data.contactId,
		},
		cause = "API"
	}
end

local function MakeRecycleContactParams(data)
	local params = MakeContactActionParams(data)
	params.configuration = data.configuration
	return params
end

local function MakeChangePriorityContactParams(data)
	local params = MakeContactActionParams(data)
	params.priority = data.priority
	return params
end

local function MakeSetEngineTypeContactParams(data)
	local params = MakeContactActionParams(data)
	params.engineType = data.engineType
	return params
end

local function MakeSetAgentContactParams(data)
	local params = MakeContactActionParams(data)
	params.agent = data.agent
	return params
end

local function MakeGetOutboundEngineActionStatusParams(data)
	local params = MakeContactActionParams(data)
	params.actionId = data.actionId
	params.scope = nil
	return params
end

function GetCampaigns(web)
	local campaigns = m_outboundEngineClient.GetClient():GetVccCampaignProcesses(
		inconcert.authUser.virtualContactCenter
	)
	return { result = "OK", campaigns = campaigns }
end

function GetCampaignBatches(web, campaignId)
	local batches = m_outboundEngineClient.GetClient():GetCampaignBatches(
		inconcert.authUser.virtualContactCenter, campaignId
	)
	return { result = "OK", batches = batches }
end

function GetCampaignBatch(web, campaignId, batchId)
	local batch = m_outboundEngineClient.GetClient():GetCampaignBatch(
		inconcert.authUser.virtualContactCenter, campaignId, batchId
	)
	return { result = "OK", batch = batch }
end

function GetOutboundContact(web, contactId)
	return OeMethod(web, "GetContact", MakeGetContactParams(contactId))
end

function GetOutboundContactByName(web)
	return OeMethod(web, "GetContactByName", MakeGetContactByNameParams(web.request))
end

function GetOutboundContactByAddress(web)
	return OeMethod(web, "GetContactByAddress", MakeGetContactByAddressParams(web.request))
end

function SearchOutboundContacts(web)
	return OeMethod(web, "SearchContacts", MakeSearchContactsParams(web.request))
end

function GetOutboundContactResults(web)
	return OeMethod(web, "GetContactResults", MakeGetContactResultsParams(web.request))
end

function StoreOutboundContact(web)
	return OeMethod(web, "StoreContacts", MakeStoreContactParams(web.request))
end

function SetOutboundContactAddresses(web)
	return OeMethod(web, "SetContactAddresses", MakeSetContactAddressesParams(web.request))
end

function SetOutboundContactValues(web)
	return OeMethod(web, "SetContactValues", MakeSetContactValuesParams(web.request))
end

function SetOutboundContactSkills(web)
	return OeMethod(web, "SetContactSkills", MakeSetContactSkillsParams(web.request))
end

function RemoveOutboundContactAddress(web)
	return OeMethod(web, "RemoveContactAddress", MakeRemoveContactAddressParams(web.request))
end

function EnableOutboundContactAddress(web)
	return OeMethod(web, "EnableContactAddress", MakeEnableContactAddressParams(web.request))
end

function DisableOutboundContactAddress(web)
	return OeMethod(web, "DisableContactAddress", MakeDisableContactAddressParams(web.request))
end

function RescheduleOutboundContact(web)
	return OeMethod(web, "RescheduleContact", MakeRescheduleContactParams(web.request))
end

function AddContactToOutboundBatch(web)
	return OeMethod(web, "AddContacts", MakeAddContactParams(web.request))
end

function PauseOutboundContact(web)
	return OeMethod(web, "PauseContacts", MakeContactActionParams(web.request))
end

function ResumeOutboundContact(web)
	return OeMethod(web, "ResumeContacts", MakeContactActionParams(web.request))
end

function CancelOutboundContact(web)
	return OeMethod(web, "CancelContacts", MakeContactActionParams(web.request))
end

function FinishOutboundContact(web)
	return OeMethod(web, "FinishContacts", MakeContactActionParams(web.request))
end

function RemoveOutboundContact(web)
	return OeMethod(web, "RemoveContacts", MakeContactActionParams(web.request))
end

function RecycleOutboundContact(web)
	return OeMethod(web, "RecycleContacts", MakeRecycleContactParams(web.request))
end

function ChangePriorityOutboundContact(web)
	return OeMethod(web, "ChangePriorityContacts", MakeChangePriorityContactParams(web.request))
end

function SetEngineTypeOutboundContact(web)
	return OeMethod(web, "SetEngineTypeContacts", MakeSetEngineTypeContactParams(web.request))
end

function GetOutboundEngineActionStatus(web)
	return OeMethod(web, "GetBatchActionStatus", MakeGetOutboundEngineActionStatusParams(web.request))
end

function SetAgentOutboundContact(web)
	return OeMethod(web, "SetContactsAgent", MakeSetAgentContactParams(web.request))
end

function ValidateAddress(web)
	local vcc = inconcert.authUser.virtualContactCenter
	local encodedInfo = web.input["post_data"]

	LogInfo("[%s:ValidateAddress] Validating phone in '%s'. Posted Info: '%s'",
			LOG_CONTEXT, vcc, encodedInfo)

	web.headers["Content-Type"] = "application/json"
	local queryInfo = web.request

	local client = GetTelephonyProxyClient()
	local result = client:ValidatePhone({
		VCC = vcc,
		number = queryInfo.number,
		originCountryCode = queryInfo.country or "",
		originAreaCode = queryInfo.area or "",
		withTimezone = true,
		allowInvalid = false,
		withTrace = true
	})
	if result.result ~= "OK" then
		web.status = 500
	end
	if result.data then
		result.data.result = result.result
		result.data.reason = result.reason
		return result.data
	end

	return result
end
