local avatar = require "apps.agent.avatar"
local agentservice = require "apps.agent.agentservice"
local models = require("apps.agent.model"):GetModels()
local provisioningmanager = require("provisioningmanager")
local mail = require "apps.agent.mail"
local agentModule = require "apps.agent.agent"
local md5 = require "md5"
local utf8 = require "icu.utf8"
require "uuid"
require "LuaOrm.Date"
require "LuaImageProcessing"
local socket = require("socket")
local m_interactions = require "apps.agent.controllers.interactions"
local helpers = require "apps.agent.helpers"
local utils = require "shared.apps.utils"
local administrationWorkgroupsModel = require("apps.administration.models.workgroups")
local administrationGroupsModel = require("apps.administration.models.groups")
local adminCampaignsModel = require("apps.administration.models.campaigns")
local model_perm = require("shared.permissions.models.permissions")
local notificationsEngine = require("apps.agent.notifications")
local agentNotifications = require("apps.agent.agentNotifications")
local authFilter = (require "authfilter")("agent")
local m_moduleLocation = select(2, ...)
local audits = require "audits"
local modelRestrictions = require "apps.agent.models.config_restrictions"
LuaImageProcessing.CascadePath = string.gsub(m_moduleLocation, "controllers", "haarcascades")

local permissions = require "shared.permissions.permissions"
local agentModel = require "apps.agent.model"
local model_vccs = require "apps.agent.models.vccs"
local jwt = require "shared.jwt"

module(..., package.seeall, inconcert.orbit.application.Decorate)

m_htmlRender = nil

function MapUris(module, htmlRender, prefix)
	local LogRequest = require("apps.agent.requestFilters").LogRequest

	module:dispatch_post( LogRequest .. SaveUser, prefix .. "/save/" )
	module:dispatch_get(  LogRequest .. GetAvatar, prefix.."/get_avatar/([^@]*)")
	module:dispatch_post( LogRequest .. permissions:CheckPermission("settings", "changenick", SetStatusMessage), prefix.."/set_status_message/([^.]*)" )
	module:dispatch_post( LogRequest .. utils.JsonReply .. ChangeUserStatus, prefix.."/change_state/([^.]*)" )

	module:dispatch_post( LogRequest .. permissions:CheckPermission("settings", "changeavatar", ChangeAvatar), prefix .. "/change_avatar/([^.]*)" )
	module:dispatch_post( LogRequest .. permissions:CheckPermission("settings", "changeavatar", UploadAvatar), prefix .. "/upload_avatar/([^.]*)" )
	module:dispatch_post( LogRequest .. permissions:CheckPermission("settings", "changeavatar", CropAvatar),   prefix .. "/crop_avatar/([^.]*)" )
	module:dispatch_get(  LogRequest .. GetUploadedAvatar, prefix.."/get_uploaded_avatar/([^.]*)" )

	-- INCO10 - SpeedTest
	module:dispatch_get( LogRequest .. GetClientIp, prefix .. "/get_client_ip/([^.]*)" )
	module:dispatch_get( LogRequest .. GetDownloadSample, prefix .. "/get_download_sample/([^.]*)" )

	m_htmlRender = htmlRender
end

function CountAssignedInteractions(user_name, campaign_id, keyspace)
	local user_interactions = m_interactions.RetrieveUserInteractions(user_name, campaign_id, keyspace)

	LogInfo("\n\nUser %s has %d interactions taken for %s campaign.", user_name, #user_interactions, campaign_id)

	return #user_interactions
end

local function MakeAllegroOtherInfo (httpVars)
--	LogDebug(" 1- httpVars %s", httpVars)
--	LogDebug(" 2- httpVars.HTTP_X_ICC_RID '%s'", httpVars.HTTP_X_ICC_RID)
--	LogDebug(" 3- httpVars.HTTP_X_REAL_IP '%s'", httpVars.HTTP_X_REAL_IP)
	local allegroOtherInfo = {}
	if not httpVars then
		return allegroOtherInfo
	end
	allegroOtherInfo.remoteServerIP = httpVars.HTTP_X_REAL_IP
	return allegroOtherInfo
end

--[[
	VCC     requerido
	ksAgent opcional
	user    opcional
]]--
function ReserveAvailablePeer (VCC, ksAgent, user)
	if not utils.HasAllegroIntegration( VCC, ksAgent ) then
	-- Si NO tenemos integracion con ALLEGRO, no es necesario asignar y/o reservar peer
		LogInfo("[AGENT.USERS:ReserveAvailablePeer] No reserved PEER for VCCs without Allegro integration. VCCs: '%s'.", VCC)
		return true, nil
	end

	-- Si tenemos integracion con ALLEGRO, Mando a almacenar algo sin importar la informacion.
	--   A esto debo hacerlo porque sino el telefono del lado del cliente no funciona correctamente.
	--
	local allegro_peer = user.AllegroIntegration and user.AllegroIntegration.peer
	if utils.TableIsEmpty(allegro_peer) then
		return true, nil
	end

	local fake_peer_name = allegro_peer.peer
	local fake_tech = allegro_peer.technology
	local fake_peer_id = allegro_peer.service .. ", " .. fake_peer_name .. ", " .. fake_tech -- TELEPHONY_192_168_24_236, 2, ASTERISK,IAX2

	local suffix = "_" .. fake_peer_id

	local fake_peer = {
		id               = fake_peer_id,
		address_id       = fake_tech .. "/" .. fake_peer_name,
		name             = fake_peer_name,
		tech             = fake_tech,
		resource_address = "resource_address" .. suffix,
		hostId           = "hostId"           .. suffix,
		rtmpServer       = "rtmpServer"       .. suffix,
		sipServer        = "sipServer"        .. suffix,
		password         = "password"
	}

	LogInfo("[AGENT.USERS:ReserveAvailablePeer] Allegro integration reserve FAKE PEER in i6. PEER: %s.", json.encode(fake_peer))
	return true, fake_peer
end

local function EncodePlainPassword (fullUserId, plainPassword)
	if not fullUserId then
		return false, "failed to build the password, user id is null."
	end
	if not plainPassword then
		return false, "failed to build the password, plainPassword id is null."
	end
	local userId = string.lower(helpers.SeparateVCC(fullUserId))
	local textToEncode = userId .. plainPassword
	local password = md5.sumhexa(textToEncode)
	--LogDebug("textToEncode: '%s', password: '%s'", textToEncode, password)
	return {password = password, textToEncode = textToEncode, plain = plainPassword}
end

local function BuildRandomPassword (fullUserId)
	if not fullUserId then
		return false, "failed to build the password, user id is null."
	end
	LogInfo("[AGENT.USERS:BuildRandomPassword] Creating Random Password for User: '%s'.", fullUserId)
	-- Initialize the pseudo random number generator
	math.randomseed(socket.gettime()*10000)
	local plainPassword = ""
	local numChars = math.random(10, 10)
	for i = 1, numChars do
		plainPassword = plainPassword .. string.char(math.random(97, 122))
	end
	return EncodePlainPassword(fullUserId, plainPassword)
end

-- find group on list (custom or system)
local function GroupExistInList (groupsList, groupId)
	local groups = utils.FilterEntities(groupsList, {Id = groupId})
	local roleId = next(groups)
	if not roleId then
		return false
	end
	local currentRole = groups[roleId]
	return true, currentRole
end

--[[
	Creates user password and sets defaults for several user settings not configurable on creation
	TODO: As soon as configuration screens are created this should be parametrized(timezone, countrycode, etc)
--]]
local function GetNewUserData (user, keyspace)
	if not user then
		return false, "User information is null"
	end
	local vcc = user.VCC or inconcert.authUser.virtualContactCenter
	local success, reservedPeer = ReserveAvailablePeer(vcc, keyspace, user)
	if not success then
		local message = reservedPeer
		return false, message
	end

	if not user.Password then
		local randomPassword, message = BuildRandomPassword(user.Id)
		if not randomPassword then
			return false, message
		end
		user.Password = randomPassword.password
		user.plainPassword = randomPassword
		user.MustChangePassword = "1"
	else
		if user.IsPlainPassword then
			user.IsPlainPassword = nil
			local passwordInfo, message = EncodePlainPassword(user.Id, user.Password)
			if not passwordInfo then
				return false, message
			end
			user.Password = passwordInfo.password
			user.plainPassword = passwordInfo
		end
	end
	if not user.DefaultCampaign then
		if user.groups and user.groups[1] and user.groups[1].id then
			user.DefaultCampaign = user.groups[1].id
		end
	end

	local userGroup, vccGroup
	if not user.UserGroup then
		userGroup = "agent"
		vccGroup = "system"
	else
		local customGroups = administrationGroupsModel.GetGroups(vcc, nil, keyspace)
		userGroup = user.UserGroup
		vccGroup = GroupExistInList(customGroups, userGroup) and vcc or "system"
	end

	user.UserGroup = userGroup
	user.VccGroup = userGroup:lower() == "full-system-access" and "system" or vccGroup
	user.LanguageId = user.LanguageId or "es"
	user.countryCode = user.countryCode or "598"
	user.VCC = vcc
	user.ValidationOrigin = "0"
	user.IsSystemUser = user.IsSystemUser and tostring(user.IsSystemUser) or "0"

	--[[
		Atencion: si se llegan a actualizar los atributos que se asigna o crean a partir del user.Id, se debe revisar y actualizar
		el archivo \helpers\change_users_id.lua en la definicion correspondiente a la SuperCF "AgentAddresses" funcion "TransformValues".
	]]--
	-- addresses
	user.addresses = {
		CALL = reservedPeer,
		CHAT = {
			address_id = user.Id .. "@" .. vcc,
			resource_address = "",
			resourceAddress = "",
			vcc = user.VCC,
			alias =  user.Name,
			username = user.Id,
			address = user.Id
		},
		IM = {
			address_id = user.Id .. "@" .. vcc,
			resource_address = "",
			resourceAddress = "",
			vcc = vcc,
			alias =  user.Name,
			username = user.Id,
			address = user.Id
		}
	}

	if user.Mail then
		user.addresses.MAIL = {
			address_id = user.Mail,
			resource_address = "",
			address = user.Mail
		}
	end

	user.skills = user.skills or {}
	return true
end

--[[
	Sends a welcome notification message to the newly created user, uses a different
	template if user is invited to a group or just created empty
--]]
local function SendWelcomeMessage(user)
	if not user or not user.Mail then
		return false
	end
	if not user.groups then
		return mail.SendNewUserWelcomeMessage(user)
	else
		local inviteGroups = {}
		for _, group in ipairs(user.groups) do
			if group.invite then
				table.insert(inviteGroups, group.id)
			end
		end
		inviteGroups = table.concat(inviteGroups, ", ")
		if #inviteGroups == 0 then
			return mail.SendNewUserWelcomeMessage(user)
		end
		local message = ""
		if user.invite and type(user.invite.message) == "string" then
			message = user.invite.message
		end
		local from = inconcert.authUser.userName .. "@" .. inconcert.authUser.virtualContactCenter
		return mail.SendNewUserAndInviteMessage(from, inviteGroups, user, message)
	end
end

local function ValidatePermissionToApp (appId, userIdAtVCC, userGroup, keyspace, applicationsActions)
	return permissions.HasPermissionToApplication(
		appId,
		userIdAtVCC,
		userGroup,
		nil,
		keyspace,
		{applicationsActions = applicationsActions}
	)
end

local function ValidateUser (user, hasAllegroIntegration, keyspace, applicationsActions, hasHelpDeskIntegration)
	local vcc = user.VCC
	local userIdAtVCC = utils.AppendVcc(user.Id, vcc)

	LogInfo("[AGENT.USERS:ValidateUser] ====> Start Validation User '%s' in VCC '%s' with Allegro Integration: '%s', HelpDesk Integration: '%s'.", userIdAtVCC, vcc, hasAllegroIntegration, hasHelpDeskIntegration)

	local hasPermissionToAppInteraction
	local timeZones = models.Timezones.FindAll(keyspace)
	local telephonesCodes = require("apps.agent.controllers.telephones").GetAllCodes()

	local availablesStates
	local function ValidateInitialState (userInitialState)
		if not userInitialState or #utils.alltrim(userInitialState) == 0 then
			return true
		end
		userInitialState = userInitialState:upper()
		local stateIdWithoutVCC = utils.GetId(userInitialState)
		availablesStates = availablesStates or models.AgentStates.FindAll(vcc, keyspace) or {}

		--LogDebug("[AGENT.USERS:ValidateUser] --> userInitialState: '%s', availablesStates: %s", stateIdWithoutVCC, json.encode(availablesStates))

		local isValid = false
		for stateId, state in pairs(availablesStates) do
			stateId = (state.Id or stateId):upper()
			if stateId == stateIdWithoutVCC then
				isValid = true
				break
			end
		end
		return isValid
	end

	local function ValidateGroup (user, vcc, keyspace)
		local userGroup = user.UserGroup
		if not userGroup then
			return false
		end
		-- validate on custom list
		local customGroupsList = administrationGroupsModel.GetGroups(vcc, nil, keyspace)
		local exist, customGroup = GroupExistInList(customGroupsList, userGroup)
		if exist then
			--LogDebug("[AGENT.USERS:ValidateGroup] Ok - Group '%s' exist in custom list!!", userGroup)
			user.VccGroup = vcc
			return true, customGroup -- ok
		end
		-- validate on system list
		-- para la validacion debo de recuperar todos los roles, incluyendo el rol deployer, ya que es necesario desde provisioning->personal settings.
		local systemGroupsList = administrationGroupsModel.GetGroups("system", nil, keyspace, "INCLUDE_ALL")
		--LogDebug("------systemGroupsList -> %s", json.encode(systemGroupsList or {}))
		local exist, systemGroup = GroupExistInList(systemGroupsList, userGroup)
		if exist then
			--LogDebug("[AGENT.USERS:ValidateGroup] Ok - Group '%s' exist in system list!!", userGroup)
			user.VccGroup = "system"
			return true, systemGroup -- ok
		end
	--		LogError("Group '%s' does not exist!!", userGroup)
		return false
	end

	local userIdRange = {minimum = 8, maximum = 255} -- el rango posible del User Id
	local function IsValidUserIdSyntax (userId)
		local isValidCharacters = userId:find("^[%-_.a-zA-Z0-9]+$")
		if not isValidCharacters then
			LogError("[AGENT.USERS:ValidateUser] Only letters, numbers, dashes and dots are allowed in the user Id.")
			return false, "USER WITH CHARS NOT ALLOWED"
		end
		local userIdLength = #utils.alltrim(userId)
		if userIdLength < userIdRange.minimum or userIdLength > userIdRange.maximum then
			LogError("[AGENT.USERS:ValidateUser] UserId '%s' length must be between %d and %d characters. Length: '%s'", userId, userIdRange.minimum, userIdRange.maximum, userIdLength)
			return false, "USER OUT OF RANGE"
		end
		return true
	end

	local groupEntity -- la Entidad del rol (Group)
	local pwdRange = {minimum = 8, maximum = 15} -- el rango posible de las pwd
	local vccInfo = model_vccs.Get(vcc, keyspace)

	local validationFields = nil
	validationFields = {
	-- PRIMERO los requeridos ---------------
	--
		-- el id de usuario único y válido
		Id = {
			required = true,
			allowedValues = function(value, user)
				if user.newUser then
					--LogInfo("[AGENT.USERS:ValidateUser] Validating syntax of new user id %s at vcc %s", userId, vcc)
					if not hasAllegroIntegration then
						local success, err = IsValidUserIdSyntax(user.Id)
						if not success then
							return false, err
						end
					end
					--LogInfo("[AGENT.USERS:ValidateUser] Checking if user %s already exists", userIdAtVCC)
					if models.Users.UserExists(userIdAtVCC, keyspace) then
						LogError("[AGENT.USERS:ValidateUser] User ID '%s' Not available.", userIdAtVCC)
						return false, "USER ALREADY EXISTS"
					end
				end
				return true
			end
		},
		-- un rol del sistema o personalizado válido
		UserGroup = {
			required = true,
			allowedValues = function(value, user)
				local succes
				success, groupEntity = ValidateGroup(user, vcc, keyspace)
				if not success then
					return false
				end
				--LogDebug("----> groupEntity: '%s'", json.encode(groupEntity))
				hasPermissionToAppInteraction = ValidatePermissionToApp("interaction", userIdAtVCC, user.UserGroup, keyspace, applicationsActions) -- esto se utiliza en otras validaciones
				return true
			end,
			tranformValue = function(user)
				if not groupEntity then
					return
				end
				-- Agrega el Id tal cual se encuentra en la CF "Group", para que conincidan las mayúsculas y minúsculas, ya que en el CSV puede venir de cualquier forma.
				user.UserGroup = groupEntity.Id
			end
		},
		-- el PRIMER NOMBRE del usuario (REQUERIDO)
		Name = {
			required = true,
			tranformValue = function(user)
				user.Name = utils.alltrim(user.Name)
			end
		},
		-- el PRIMER APELLIDO del usuario (REQUERIDO)
		FirstSurname = {
			required = true,
			tranformValue = function(user)
				user.FirstSurname = utils.alltrim(user.FirstSurname)
			end
		},
		-- el SEGUNDO NOMBRE del usuario (NO REQUERIDO)
		LastName = {
			required = false,
			tranformValue = function(user)
				if not user.LastName then
					return
				end
				user.LastName = utils.alltrim(user.LastName)
			end
		},
		-- el SEGUNDO APELLIDO del usuario (NO REQUERIDO)
		LastSurname = {
			required = false,
			tranformValue = function(user)
				if not user.LastSurname then
					return
				end
				user.LastSurname = utils.alltrim(user.LastSurname)
			end
		},
		-- el timezone id
		TimeZoneId = {
			required = true,
			allowedValues = function(value)
				if not value or not timeZones[value:upper()] then
					return false
				end
				return true
			end,
			tranformValue = function(user)
				if not user.TimeZoneId then
					return
				end
				user.TimeZoneId = user.TimeZoneId:upper()
			end
		},
		-- el código de país
		countryCode = {
			required = true,
			allowedValues = function(value)
				if not value or not telephonesCodes[value] then
					return false
				end
				return true
			end
		},
		-- el código de área para un país (areaCode) determinado
		areaCode = {
			required = false, -- La validación de requerido queda desactivada, ya que si no llega un valor en código de área, se asigna un valor por default.
			allowedValues = function(value, user)
				local countryCodes = telephonesCodes[user.countryCode]
				if not countryCodes then
					return false
				end
				if not value then
					-- Si no viene definido un código de área en el usuario, no realizo la validación de código de área, ya que en la función 'setDefaultValue' se le va a asignar un valor por defecto.
					return true
				end
				if not countryCodes.areasByCode or not countryCodes.areasByCode[value] then
					return false
				end
				return true
			end,
			setDefaultValue = function(user)
				if user.areaCode then
					-- Si ya viene definido un código de área, quiere decir que ya paso la validación de la función "allowedValues", entonces me voy.
					return
				end
				-- Si no viene definido un código de área, tengo que asignar uno por defecto.
				-- Voy a seleccionar el primero de la lista según el código country.
				local countryCodes = telephonesCodes[user.countryCode]
				local defaultAreaCode = next(countryCodes.areasByCode or {})
				user.areaCode = defaultAreaCode
				LogInfo("[AGENT.USERS:ValidateUser] User '%s' is assigned the default area '%s' of country '%s'.", user.Id, user.areaCode, user.countryCode)
			end
		},
	-- SEGUNDO los NO requeridos ---------------
	--
		-- la dirección de correo electrónico único dentro del vcc
		Mail = {
			required = function(user)
				if hasHelpDeskIntegration then
					if not user.Mail or #utils.alltrim(user.Mail) == 0 then
					-- si no tiene permisos sobre la app "interaction" las campañas no son requeridas
						LogError("[AGENT.USERS:ValidateUser] Mail is Required in VCC with HelpDesk Integration.")
						return false, "MAIL IS REQUIRED IN VCC WITH HELPDESK INTEGRATION"
					end
				end
				-- ok
				return true
			end,
			allowedValues = function(_, user)
				if not user.Mail or #utils.alltrim(user.Mail) == 0 then
					return true
				end
				local isValidEmail = utils.ValidateEmail(user.Mail)
				if not isValidEmail then
					LogError("[AGENT.USERS:ValidateUser] Invalid format Mail '%s'.", user.Mail)
					return false, "INVALID FORMAT MAIL"
				end
				local ownerUserId = models.Users.FindUserByEmail(user.Mail, vcc, keyspace)
				if ownerUserId and (user.newUser or (not user.newUser and ownerUserId ~= userIdAtVCC)) then
					LogError("[AGENT.USERS:ValidateUser] Mail '%s' not available. '%s' user is the owner.", user.Mail, ownerUserId)
					return false, "MAIL ALREADY EXISTS"
				end
				return true
			end,
			setDefaultValue = function(user)
				if #utils.alltrim(user.Mail) == 0 then
					user.Mail = nil -- si el mail es empty, lo dejo nil para que en la BD no se agregue el campo
				end
			end
		},
		-- el tipo de lenguaje que se le configura al usuario
		Password = {
			required = false,
			mustValidateValues = function(value, user)
				if authFilter:HasExternalAuth(vccInfo) then
					LogInfo("[AGENT.USERS:ValidateUser.mustValidateValues] 1- Password field validation. NO VALIDATE - HAS EXTERNAL AUTH")
					return false -- NO debo validar la pwd
				end
				if user.newUser then
					LogInfo("[AGENT.USERS:ValidateUser.mustValidateValues] 2- Password field validation. YES VALIDATE NEW USER")
					return true -- -SI debo validar la pwd
				end
				if not value then
					LogInfo("[AGENT.USERS:ValidateUser.mustValidateValues] 3- Password field validation. NO VALIDATE - NO PWD VALUE")
					return false -- NO debo validar la pwd
				end
				--Los chequeos de password solo se hacen si cambió el password, si la password ha expirado o es un newUser
				local oldUser = models.Users.Get(userIdAtVCC, nil, keyspace)
				-- si es un nuevo usuario o la pwd ha cambiado
				if oldUser.Password ~= value
					or user.PasswordHasChanged
				then
					LogInfo("[AGENT.USERS:ValidateUser.mustValidateValues] 4 - Password field validation. YES VALIDATE - PASSWORD HAS CHANGED")
					return true, oldUser -- SI valido la pwd
				end
				-- NO debo validar la pwd
				LogInfo("[AGENT.USERS:ValidateUser.mustValidateValues] 5- Password field validation. NO VALIDATE - DEFAULT OPTION")
				return false, oldUser
			end,
			allowedValues = function(value, user)
				--Los chequeos de password solo se hacen con usuarios internos
				--LogDebug("[AGENT.USERS:ValidateUser] 1- Password field validation.")
				local mustValidate, resp = validationFields.Password.mustValidateValues(value, user)
				if not mustValidate then
					--LogDebug("[AGENT.USERS:ValidateUser] 2- Password field validation. mustValidate = FALSE")
					return true
				end
				--LogDebug("[AGENT.USERS:ValidateUser] 3- Password field validation. mustValidate = TRUE")
				local oldUser = resp
				-- valido la password
				local plainPassword = ""
				local patternAllegroInvalidCharacters = "[?=&#]"
				if user.IsPlainPassword then
					--LogDebug("[AGENT.USERS:ValidateUser] 3.1- Password field validation.")
					if not value then
					-- Si es plana tiene que venir si os si, despues se autogenera
						LogError("[AGENT.USERS:ValidateUser] Password is required.")
						return false, "PASSWORD IS REQUIRED"
					end
					-- Si tengo integracion con Allegro no realiza las validaciones de la PWD
					if hasAllegroIntegration then
						-- (Allegro) Se valida que el password no tenga caracteres reservados que se modifican durante la serialización de la URL (#, &, ?, =), porque da error en el login.
						if string.match(value, patternAllegroInvalidCharacters) then
							LogError("[AGENT.USERS:ValidateUser] Allegro error - The password value must not contain reserved characters: %s.", patternAllegroInvalidCharacters)
							return false, "INVALID CHARACTERS IN PASSWORD"
						end
						return true
					end
					plainPassword = value
					--Si viene en blanco no realizo validaciones, porque luego se autogenera fieldId
					if plainPassword == "" then
						return true
					end
					--LogDebug("[AGENT.USERS:ValidateUser] 3.2- Password field validation.")
				else
					if not user.jwtToken then
						--LogDebug("[AGENT.USERS:ValidateUser] 4.1- Password field validation.")
						plainPassword = user.PlainPasswordFromApi or value
					else
						--jwtToken
						--LogDebug("[AGENT.USERS:ValidateUser] 4.2- Password field validation.")
						local jwtToken = user.jwtToken
						local jwtData, jwtErr = jwt.decode(user.jwtToken, md5.sumhexa(inconcert.authUser.authToken), "HS256")
						if jwtErr then
							LogError("[AGENT.USERS:ValidateUser] Error decoding jwtToken: '%s'", jwtErr)
							return false, "INVALID JWT PASSWORD"
						end
						plainPassword = jwtData.password
					end
				end
				if hasAllegroIntegration and string.match(plainPassword, patternAllegroInvalidCharacters) then
					-- (Allegro)  Se valida que el password no tenga caracteres reservados que se modifican durante la serialización de la URL (#, &, ?, =), porque da error en el login.
					LogError("[AGENT.USERS:ValidateUser] Allegro error - The password value must not contain reserved characters: %s.", patternAllegroInvalidCharacters)
					return false, "INVALID CHARACTERS IN PASSWORD"
				end
				-- Para los SuperUsuarios o el primer usuario de un VCC de allegro no se dispone de su pwd en claro, luego no se puede validar
				if utils.IsUserFullSystemAccess(user)
					or (
						user.CreatedBy
						and user.CreatedBy.version == "allegro"
					)
				then
					return true -- No realiza las validaciones de seguridad configuradas en el VCC contra el AuthFilter
				end
				-- authFilter Password validation
				local passwordHistory = oldUser and oldUser.PasswordHistory or {}
				local success, resp = authFilter:IsPasswordValid(vccInfo, plainPassword, value, passwordHistory)
				--LogDebug("[AGENT.USERS:ValidateUser] 5- Password field validation. success: %s, resp: %s", success, json.encode(resp or {}))
				if not success then
					local err = resp
					return false, "PASSWORD EXTERNAL AUTH ERROR", err
				end
				-- ok
				return true
			end,
			setDefaultValue = function(user)
				if not user.IsPlainPassword then
					--LogDebug("[AGENT.USERS:ValidateUser] setDefaultValue 1")
					return
				end
				-- Si es un SuperUsers, es plana y viene por ejemplo llega desde la App "provisioning", no debe asignar valores por defecto, sino que se queda con la pwd que el usuario tiene almacenada.
				if utils.IsUserFullSystemAccess(user) then
					--LogDebug("[AGENT.USERS:ValidateUser] setDefaultValue 2")
					return
				end
				if #utils.alltrim(user.Password) == 0 then
				-- Se debe de autogenerar la PWD
					user.Password = string.sub(user.Id, 1, pwdRange.maximum):lower()
				-- Se le obliga a cambiar la password en el primer inicio de session
					user.MustChangePassword = "1"
					LogInfo("[AGENT.USERS:ValidateUser] Autogenerate User Password. User: '%s', plain password: '%s'. User must change password in the first login.", user.Id, user.Password)
				end
			end
		},
		-- el tipo de lenguaje que se le configura al usuario
		LanguageId = {
			required = false,
			allowedValues = function(value)
				if not value or #utils.alltrim(value) == 0 then
					return true
				end
				local supportedLanguages = {es = true, en = true, fr = true, pt = true}
				if not supportedLanguages[value:lower()] then
					return false
				end
				return true
			end,
			tranformValue = function(user)
				if not user.LanguageId then
					return
				end
				user.LanguageId = user.LanguageId:lower()
			end,
			setDefaultValue = function(user)
				if #utils.alltrim(user.LanguageId) == 0 then
					user.LanguageId = "es"
				end
			end
		},
		-- el estado inicial
		initialState = {
			required = false,
			allowedValues = function(value)
				local success = ValidateInitialState(value)
				if not success then
					return false
				end
				if value and value:upper() == "ACTIVE" then
				-- El estado "ACTIVE" solo es válido si el usuario tiene acceso a la app Interaction
					if not hasPermissionToAppInteraction then
						LogError("[AGENT.USERS:ValidateUser] The user '%s' must have permissions to Interaction application to use ACTIVE system state.", userIdAtVCC)
						return false, "must have permissions to interaction application to use ACTIVE system state"
					end
				end
				return true
			end,
			setDefaultValue = function(user)
				if hasAllegroIntegration or #utils.alltrim(user.initialState) == 0 then
					user.initialState = agentModule.GetUserInitialState(vcc)
					LogInfo("[AGENT.USERS:ValidateUser] Setting Default Value '%s' in field 'initialState'.", user.initialState)
				end
			end
		},
		-- campañas
		groups = {
			required = function(user)
				if not hasPermissionToAppInteraction then
				-- si no tiene permisos sobre la app "interaction" las campañas no son requeridas
					return true
				end
				if type(user.groups) ~= "table" or #user.groups < 1 then
					LogError("[AGENT.USERS:ValidateUser] If '%s' have permission for interaction application, must have some assigned campaign.", userIdAtVCC)
					return false, "if have permission for interaction application, must have some assigned campaign"
				end
				return true
			end,
			allowedValues = function(value)
				local success = true
				local msgErr, errCustomValue
				for _, campInfo in ipairs(user.groups or {}) do
					local campIdWithoutVCC = utils.GetId(campInfo.id)
					local campaignAtVCC = utils.AppendVcc(campIdWithoutVCC, vcc)
					if not models.Campaigns.CampaignExists(campaignAtVCC) then
						LogError("[AGENT.USERS:ValidateUser] Campaign '%s' does not exist.", campaignAtVCC)
						success = false
						msgErr = "CAMPAIGN DOES NOT EXIST"
						errCustomValue = campaignAtVCC
						break
					end
					user.groups[_].id = campIdWithoutVCC:lower()
				end
				return success, msgErr, errCustomValue
			end
		},
		-- la campaña por defecto, solo sirve cuando vienen campañas en la lista "groups"
		DefaultCampaign = {
			allowedValues = function(_, user)
				if not user.DefaultCampaign then
					return true
				end
				if not user.groups then
					-- Si NO vienen campañas asignadas al usuario, elimino la "DefaultCampaign" y no fallo.
					user.DefaultCampaign = nil
					return true
				end
				local existDefaultCampaign = false
				local firstCampaign
				for _, campInfo in ipairs(user.groups) do
					if not firstCampaign then
						firstCampaign = campInfo.id:lower()
					end
					if campInfo.id:lower() == user.DefaultCampaign:lower() then
						existDefaultCampaign = true
						break
					end
				end
				if not existDefaultCampaign then
					LogWarning("[AGENT.USERS:ValidateUser] Default campaign '%s' does not exist. Default campaign is changed for '%s'", user.DefaultCampaign, firstCampaign)
					user.DefaultCampaign = firstCampaign
				end
				return true
			end
		},
		-- Valores utilizados en VCCs con integracion con allegro (peer, etc)
		AllegroIntegration = {
			required = false,
			tranformValue = function(user)
				if not hasAllegroIntegration then
					return
				end
				if not user.AllegroIntegration then
					return
				end
				--LogDebug("[AGENT.USERS:ValidateUser] a- AllegroIntegration '%s'.", json.encode(user.AllegroIntegration or {}))
				if #utils.alltrim(user.AllegroIntegration.peer.service) == 0 or
					#utils.alltrim(user.AllegroIntegration.peer.peer) == 0 or
					#utils.alltrim(user.AllegroIntegration.peer.technology) == 0 then
					-- Si la informacion del PEER no viene completa debo limpiar la misma
					user.AllegroIntegration.peer = {}
				end
				--LogDebug("[AGENT.USERS:ValidateUser] b- AllegroIntegration '%s'.", json.encode(user.AllegroIntegration or {}))
			end
		},
	-- TERCERO los NO requeridos, con valores por defecto, pero que se tiene que almacenar sí o sí en la BD. ---------------
	--
		-- Si el Usuario esta Activo o Desactivo para el VCC
		Enabled = {
			required = false,
			allowedValues = function(_, user)
				if not user.Status or #utils.alltrim(user.Status) == 0 then
					return true
				end
				local enabled = tonumber(user.Status)
				if enabled == 0 or enabled == 1 then
					return true
				end
				return false
			end,
			setDefaultValue = function(user)
				if #utils.alltrim(user.Status or "") == 0 then
					user.Status = "1"
				end
			end
		},
		-- Segundo nombre
		LastName = {
			required = false,
			setDefaultValue = function(user)
				if #utils.alltrim(user.LastName or "") == 0 then
					user.LastName = ""
				end
			end
		},
		-- Segundo apellido
		LastSurname = {
			required = false,
			setDefaultValue = function(user)
				if #utils.alltrim(user.LastSurname or "") == 0 then
					user.LastSurname = ""
				end
			end
		},
		-- Si el Usuario tiene algún override
		HasRoleOverride = {
			required = false,
			setDefaultValue = function(user)
				if #utils.alltrim(user.HasRoleOverride or "") == 0 then
					user.HasRoleOverride = "0"
				end
			end
		}
	}

	local function ReturnRequiredError (fieldId, fieldValue, errMsg)
		local logMsg = errMsg or string.format("field %s is required", fieldId)
		LogError("[AGENT.USERS:ValidateUser] %s.", logMsg)
		local err = {
			result = "FILE IS REQUIRED",
			msg = errMsg,
			fieldId = fieldId,
			fieldValue = tostring(fieldValue)
		}
		return false, err
	end

	local function ReturnAllowedError (fieldId, fieldValue, errMsg)
		local logMsg = (errMsg and type(errMsg) == "string")
			or (errMsg and type(errMsg) == "table" and json.encode(errMsg))
			or string.format("value '%s' is not allowed in field %s", tostring(fieldValue), fieldId)

		LogError("[AGENT.USERS:ValidateUser] fieldId '%s':%s.", fieldId, logMsg)

		local value = type(fieldValue) ~= "table" and tostring(fieldValue) or fieldValue -- si es tabla debe de viajar como objeto
		local err = {
			result = "VALUE NOT ALLOWED",
			msg = errMsg,
			fieldId = fieldId,
			fieldValue = value
		}
		return false, err
	end

	for fieldId, info in pairs(validationFields) do
		local fieldValue = user[fieldId]
		if type(info.required) == "boolean" and info.required then
			if not fieldValue or #utils.alltrim(fieldValue) == 0 then
				return ReturnRequiredError(fieldId, fieldValue)
			end
		elseif type(info.required) == "function" then
			local success, errMsg = info.required(user)
			if not success then
				if errMsg then
					return ReturnRequiredError(fieldId, fieldValue, errMsg)
				end
				return ReturnRequiredError(fieldId, fieldValue)
			end
		end
		if info.allowedValues then
			local isAllowed, errMsg, errCustomValue = info.allowedValues(fieldValue, user)
			if not isAllowed then
				local errValue = errCustomValue or fieldValue
				if errMsg then
					return ReturnAllowedError(fieldId, errValue, errMsg)
				end
				return ReturnAllowedError(fieldId, errValue)
			end
		end
		if info.setDefaultValue then
			info.setDefaultValue(user)
		end
		if info.tranformValue then
			info.tranformValue(user)
		end
	end

	-- SUCCESS
	return true
end

--[[
]]--
function UpdateUser (user, keyspace, applicationsActions, allegroOtherInfo, skipLimitsValidation)
	if not user or type(user.Id) ~= "string" then
		return false, "Invalid request"
	end

	allegroOtherInfo = allegroOtherInfo or {}

	local loggedUserId = utils.AppendVcc(inconcert.authUser.userName, inconcert.authUser.virtualContactCenterc)
	LogDebug("")
	LogInfo("[AGENT.USERS:UpdateUser] User '%s' Update UserInfo: '%s'. allegroOtherInfo: '%s'", loggedUserId, utils.JSON_Encode(user), json.encode(allegroOtherInfo))

	user.Id = utf8.lower(user.Id)
	local userId, userVCC = helpers.SeparateVCC(user.Id)

	--Remove the domain from the Id Field in case it was sent
	user.Id = userId

	local vcc = utf8.lower(inconcert.authUser.virtualContactCenter == "system" and userVCC or inconcert.authUser.virtualContactCenter)
	local userIdAtVCC = userId.."@"..vcc
	user.VCC = vcc

	if (user.UserGroup or ""):lower() == "full-system-access" then
		user.VccGroup = "system"
		local superUser = models.Users.GetSuperUser(user.VCC, user.Id, keyspace)
		if not superUser then
			local allSuperUsers = models.Users.GetVCCSuperUsers(user.VCC, keyspace)
			local total = utils.GetRowCount(allSuperUsers)
			--LogDebug("--> total: %s - allSuperUsers: %s", total, json.encode(allSuperUsers))
			local MAXIMUM_SUPER_USERS_PER_VCC = 10
			if total >= MAXIMUM_SUPER_USERS_PER_VCC then
				local reason = LogError("[AGENT.USERS:UpdateUser] Unable to Save Super User '%s'. You have reached the maximum number of super users allowed in the VCC. Maximum allowed: %s", user.Id, MAXIMUM_SUPER_USERS_PER_VCC)
				-- remuevo la app+controller+funcion de la reason. Esto pos si se tiene que visualizar client side.
				reason = string.gsub(reason, "%[AGENT%.USERS:UpdateUser%]", "")
				local result = {result = "UNABLE ADD SUPER USER - REACHED THE MAXIMUM ALLOWED", reason = reason, maxAllowed = MAXIMUM_SUPER_USERS_PER_VCC}
				return false, result
			end
		end
	end

	if user.DefaultCampaign then
		user.DefaultCampaign = utf8.lower(user.DefaultCampaign)
	end

	local close_keyspace = false
	if type(keyspace) ~= "userdata" then
		keyspace = models.Users.GetCassandra():CassandraConnection("Agent")
		close_keyspace = true
	end

	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc, keyspace)
	local vccInfo = model_vccs.Get(vcc, keyspace)
	local vccAuthentication = vccInfo.Security and vccInfo.Security.authentication or {}
	local passwordChecks = vccAuthentication.passwordChecks or {}

	local oldUser = models.Users.Get(userIdAtVCC, nil, keyspace)
	local updateUserPasswordHistory = function()
		if not user.newUser and oldUser.Password ~= user.Password and user.Password then
			local pwdHistory = oldUser.PasswordHistory or {}
			while pwdHistory and #pwdHistory > passwordChecks.history do
				table.remove(pwdHistory, 1)
			end
			user.PasswordHistory = pwdHistory
		end
	end

	--Valido si agrega workgroup que no supere el limite de la relacion workgroups/users y campaigns/user
	local newCampaigns = {}
	local deletedCampaigns = {}
	utils.ProcessDiffs(
		utils.ArrayToHash((oldUser and oldUser.groups) or {}, "id"),
		utils.ArrayToHash(user.groups or {}, "id"),
		-- Adding
		function(id)
			table.insert( newCampaigns, id )
		end,
		-- Deleting
		function( id )
			table.insert( deletedCampaigns, id )
		end,
		function()end
	)

	local cont_campaigns_user = #newCampaigns - #deletedCampaigns
	LogInfo("[AGENT.USERS:UpdateUser] newCampaigns %s, cont_campaigns_user %s", json.encode(newCampaigns), cont_campaigns_user)
	if cont_campaigns_user > 0 and not skipLimitsValidation then
		-- Se agregaron nuevas campanas
		local success, error = modelRestrictions.ValidateConfigurationsLimits(vcc, "MaxCampaignsUsers", keyspace, cont_campaigns_user)
		if not success then
			if close_keyspace then
				keyspace:Close()
			end
			return false, {result = error or "ERROR_CAMPS_USERS_EXCEEDS_LIMIT_ALLOWED"}
		end
	end

	local is_add_workgroups_user = user.WorkgroupId and (not oldUser or not oldUser.WorkgroupId or oldUser.WorkgroupId == "") or nil
	if is_add_workgroups_user and not skipLimitsValidation then
		local success, error = modelRestrictions.ValidateConfigurationsLimits(vcc, "MaxWorkgroupsUsers", keyspace)
		if not success then
			if close_keyspace then
				keyspace:Close()
			end
			return false, {result = error or "ERROR_WORKGROUPS_USERS_EXCEEDS_LIMIT_ALLOWED"}
		end
	end

	--Before ValidateUser, update user password history, to keep just the already history password items configured in vcc
	if vccAuthentication.type == "MFA" or vccAuthentication.type == "BASIC" then
		updateUserPasswordHistory()
	end

	-- Before validate user we use some old user info just in case this is a password reset from provisioning and we havent that info at this point
	user.CompanyInformation = user.CompanyInformation or oldUser.CompanyInformation
	user.CustomInformation = user.CustomInformation or oldUser.CustomInformation
	user.DateFormat = user.DateFormat or oldUser.DateFormat
	user.DefaultCampaign = user.DefaultCampaign or oldUser.DefaultCampaign
	user.FirstSurname = user.FirstSurname or oldUser.FirstSurname
	user.FreeSecondsAfterInteraction = user.FreeSecondsAfterInteraction or oldUser.FreeSecondsAfterInteraction
	user.HasRoleOverride = user.HasRoleOverride or oldUser.HasRoleOverride
	user.IsSystemUser = user.IsSystemUser or oldUser.IsSystemUser
	user.LanguageId = user.LanguageId or oldUser.LanguageId
	user.LastName = user.LastName or oldUser.LastName
	user.LastSurname = user.LastSurname or oldUser.LastSurname
	user.Mail = user.Mail or oldUser.Mail
	user.Name = user.Name or oldUser.Name
	user.TimeZoneId = user.TimeZoneId or oldUser.TimeZoneId
	user.UserGroup = user.UserGroup or oldUser.UserGroup
	user.ValidationOrigin = user.ValidationOrigin or oldUser.ValidationOrigin
	user.VccGroup = user.VccGroup or oldUser.VccGroup
	if user.WorkgroupId == nil then
		user.WorkgroupId = oldUser.WorkgroupId
	end

	user.areaCode = user.areaCode or oldUser.areaCode
	user.countryCode = user.countryCode or oldUser.countryCode
	user.groups = user.groups or oldUser.groups
	user.initialState = user.initialState or oldUser.initialState
	user.skills = user.skills or oldUser.skills

	local hasHelpDeskIntegration = utils.HasHelpDeskIntegration(vcc, keyspace)
	local success, err = ValidateUser(user, hasAllegroIntegration, keyspace, applicationsActions, hasHelpDeskIntegration)
	if not success then
		if close_keyspace then keyspace:Close() end
		LogError("[AGENT.USERS:UpdateUser] Unable to Update User '%s'. Err: %s", userIdAtVCC, utils.JSON_Encode(err))
		return false, err
	end

	--Remove attribute used for validation
	user.PasswordHasChanged = nil
	user.PlainPasswordFromApi = nil

	if not models.Users.LockUser(user) then -- locking user data before save
		if close_keyspace then keyspace:Close() end
		LogWarning("[AGENT.USERS:UpdateUser] User %s does not own the lock on user id %s, will not save", loggedUserId, user.Id)
		return false, "user identifier in use"
	end

	--get new user data and validation availables resources
	if user.newUser then
		local response_user_data, messageError = GetNewUserData(user, keyspace)
		if not response_user_data then
			if close_keyspace then keyspace:Close() end
			models.Users.UnlockUser(user)
			LogError(messageError)
			return false, "not resources availables"
		end
		if vccAuthentication.type == "MFA" or vccAuthentication.type == "BASIC" then
			-- actualizo el historial de passwords del user
			local pwdHistory = user.PasswordHistory or {}
			table.insert(pwdHistory, user.Password)
			user.PasswordHistory = pwdHistory
			updateUserPasswordHistory()
			-- actualizo la fecha de la nueva password
			user.PasswordChangeTimestamp = tostring(os.time())
			-- reseteo el numero de reintentos de login fallidos
			user.LoginFailedAttemps = "0"
			user.StatusReason = ""
			user.StatusTimestamp = tostring(os.time())
		end
	else
		if not(oldUser.addresses and oldUser.addresses.CALL) or hasAllegroIntegration then
			--LogInfo("[AGENT.USERS:UpdateUser] User %s has no assigned peer, let's find one available", userId)
			local success, reservedPeer = ReserveAvailablePeer(vcc, keyspace, user)
			if not success then
				models.Users.UnlockUser(user)
				if close_keyspace then keyspace:Close() end
				LogError("[AGENT.USERS:UpdateUser] %s", reservedPeer)
				return false, "not resources availables"
			end
			user.addresses = user.addresses or {}
			user.addresses.CALL = reservedPeer
		end

		-- Actualizo la info de los clientes (layout)
		user.Clients = user.Clients or oldUser.Clients

		if user.IsPlainPassword then
			user.IsPlainPassword = nil
			if user.Password then
				local passwordInfo, message = EncodePlainPassword(user.Id, user.Password)
				if not passwordInfo then
					return false, message
				end
				user.Password = passwordInfo.password
				user.plainPassword = passwordInfo
			end
		else
			if vccAuthentication.type == "MFA" or vccAuthentication.type == "BASIC" then
				--Si ha cambiado la password
				if oldUser.Password ~= user.Password and user.Password then
					-- actualizo el historial de passwords del user
					local pwdHistory = oldUser.PasswordHistory or {}
					table.insert(pwdHistory, user.Password)
					user.PasswordHistory = pwdHistory
					updateUserPasswordHistory()
					-- actualizo la fecha de la nueva password
					user.PasswordChangeTimestamp = tostring(os.time())
					-- reseteo el numero de reintentos de login fallidos
					user.LoginFailedAttemps = "0"
					-- desactivo la restriccion de cambio de passsword en el siguiente login 
					user.MustChangePassword = not user.MustChangePassword and "0" or user.MustChangePassword
				end
			end
		end
	end

	--Save into workgroup
	if user.WorkgroupId ~= nil then
		local workgroupsToSave = {}
		--lock workgroups by vcc
		if not models.Users.LockWorkgroup(user) then
			models.Users.UnlockUser(user)
			if close_keyspace then keyspace:Close() end
			LogWarning("[AGENT.USERS:UpdateUser] User %s does not own the lock on workgroups, will not save", loggedUserId)
			return false, "workgroup identifier in use"
		end

		if user.WorkgroupId then
			--Add user to new workgroup
			local workgroup = administrationWorkgroupsModel.GetWorkgroups(vcc, user.WorkgroupId, keyspace)
			if not next(workgroup) then
				LogError("Workgroup %s does not exist", user.WorkgroupId)
				models.Users.UnlockUser(user)
				models.Users.UnlockWorkgroup(user)
				if close_keyspace then keyspace:Close() end
				return false, "workgroup does not exist"
			end
			--LogInfo("[AGENT.USERS:UpdateUser] Adding user %s to workgroup %s", userId, user.WorkgroupId)
			workgroup.Agents[userId] = true
			workgroupsToSave[user.WorkgroupId] = workgroup
		end

		local oldUser = models.Users.Get(userIdAtVCC, nil, keyspace)
		local oldWorkgroupId = oldUser and oldUser.WorkgroupId
		--LogInfo("[AGENT.USERS:UpdateUser] Old WorkgroupId for user %s is %s", userId, oldWorkgroupId)
		if type(oldWorkgroupId) ~= nil and (oldWorkgroupId ~= user.WorkgroupId or user.WorkgroupId == false) then
			--Remove user from old workgroup
			--LogInfo("[AGENT.USERS:UpdateUser] Removing user %s from old workgroup %s", userId, oldWorkgroupId)
			local oldWorkgroup = administrationWorkgroupsModel.GetWorkgroups(vcc, oldWorkgroupId, keyspace)
			if oldWorkgroup and oldWorkgroup.Agents then
				oldWorkgroup.Agents[userId] = nil
				workgroupsToSave[oldWorkgroupId] = oldWorkgroup
			end
		end

		administrationWorkgroupsModel.SaveWorkgroups(vcc, workgroupsToSave, keyspace)
		--unlocks workgroups
		models.Users.UnlockWorkgroup(user)
	end

	--Add creation Date for new User
	if user.newUser then
		user.creationDate = LuaOrm.Date():toString()
	end
	
	--saving user
	local responseSavedUser, msg = models.Users.Save(userId, vcc, user, keyspace, allegroOtherInfo) -- saving user data
	models.Users.UnlockUserData(user) -- unlocking user data after saved
	if not responseSavedUser then
		if close_keyspace then keyspace:Close() end
		return false, msg or "unable to save"
	end

	-- Si es un Super Usuario debo almacenarlo en la CF SuperUsers. Solo si es nuevo, ya que si se esta modificando el mismo ya fue agregado.
	if user.newUser then
		local isUserFullSystemAccess = utils.IsUserFullSystemAccess(user)
		--LogDebug("---------> isUserFullSystemAccess: %s", isUserFullSystemAccess)
		if isUserFullSystemAccess then
			models.Users.SaveSuperUser(vcc, user, keyspace)
		end
	end

	--[[
		Aca valido:
		 - si el usuario a actualizar fue asignado a nuevas campanas.
		 - si esta logueado. si existe integracion con helpdesk.
		 - Si no fue licenciado (helpdesk) previamente.
		 - si tiene permisos de helpdesk tickets y la app interaccion
		 - Si al menos una de las nuevas campanas tiene integracion con helpdesk.

		 Se consume una licencia de helpdesk ya que en el login no se consumio para evitar consumos innecesarios.
	]]--
	if #newCampaigns > 0 and agentservice.IsAgentLoggedIn(userId, vcc) and hasHelpDeskIntegration then
		--LogInfo("[AGENT.USERS:UpdateUser] Validate Consume helpdesk license for user %s --> newCampaigns %s, is logged user and the vcc has helpdesk integration.",userIdAtVCC, #newCampaigns)
		local userData = authFilter:GetLoggedUserInfo(userId, vcc) or {}
		local userHelpdeskIntegration = userData.Licenses and userData.Licenses.AppsIntegrations and userData.Licenses.AppsIntegrations.HelpDesk or {}
		local isUserLicensed = userHelpdeskIntegration.IsLicensed
		--LogInfo("[AGENT.USERS:UpdateUser] Validate Consume helpdesk license for user %s --> newCampaigns %s, is logged user and the vcc has helpdesk integration.",userIdAtVCC, #newCampaigns)
		if not isUserLicensed then
			local custom_permissions = model_perm.GetUserPermissions(userId, keyspace, vcc)
			local hasUserTicketHelpdeskPermission = permissions.IsPermissionEnabledForSpecificUser("apps_integrations", "helpdesk_tickets_in_app_interaction", userIdAtVCC, user.UserGroup, custom_permissions, nil, nil, keyspace, applicationsActions)
			local hasPermissions = hasUserTicketHelpdeskPermission and permissions.HasPermissionToApplication("interaction", userIdAtVCC, user.UserGroup, nil, keyspace, {
				applicationsActions = applicationsActions,
				user_vcc_permissions = custom_permissions}, true)
			if hasPermissions then
				for _, fullCampaignId in ipairs(newCampaigns) do
					local campIntegration = adminCampaignsModel.GetSpecificColumn( fullCampaignId, "Integration", keyspace )
					campIntegration = type(campIntegration) ~= "string" and {} or json.decode(campIntegration)
					local has_campaign_helpdesk = campIntegration and campIntegration.othersIntegrations and campIntegration.othersIntegrations.HELPDESK and campIntegration.othersIntegrations.HELPDESK.Enable
					
					LogInfo("[AGENT.USERS:UpdateUser] Consume helpdesk license for user %s --> campaign %s has_helpdesk %s.", userIdAtVCC, fullCampaignId, has_campaign_helpdesk)
					if has_campaign_helpdesk then
						userData.AppsPermissions = userData.AppsPermissions or {}
						userData.AppsPermissions.helpdesk = true
						--LogInfo("[AGENT.USERS:UpdateUser] Consume helpdesk license --> userData %s.", json.encode(userData))
						local success, noAvailableLicense = authFilter:DecreaseVCCAvailableLicenses({
							username = userId,
							vcc = vcc,
							user = userData
						})
						-- Si success no es true, no significa que exista error.
						if success and not noAvailableLicense then
							--Mando notificacion al usuario de consumo de licencias para que habilite la visualizacion.
							agentNotifications.NotifyAgents({
								vcc = vcc,
								level= "ALWAYS",
								scope = "agent",
								eventType = "consume_helpdesk_licenses",
								agents = { userIdAtVCC },
								pollerMessage = {
									status = "ENABLE" --Para habilitar las opciones de helpdesk del agente
								}
							}, keyspace)
							LogInfo("[AGENT.USERS:UpdateUser] A license helpdesk for user %s was consumed.", userIdAtVCC)
						else
							-- Aqui no se pudo consumir licencia de helpdesk porque no existen disponibles.
							-- Solo se deja log
							LogError("[AGENT.USERS:UpdateUser] Error consumming helpdesk license for user %s, no available licenses", userIdAtVCC)
						end
						break 
					end
				end
			end
		end
	end

	--Actualizo en redis el contador de la relacion workgroups/User y campaigns/users en caso que se agregue o elimine 
	local is_delete_workgroup = (not user.WorkgroupId or user.WorkgroupId == "") and oldUser and oldUser.WorkgroupId and oldUser.WorkgroupId ~= ""
	if is_add_workgroups_user or is_delete_workgroup or cont_campaigns_user ~= 0 then
		-- Actualizo los totales en redis de workgroups
		modelRestrictions.UpdateRedisConfigTotals(vcc, {
			MaxWorkgroupsUsers = (is_add_workgroups_user and 1) or (is_delete_workgroup and -1) or nil,
			MaxCampaignsUsers = cont_campaigns_user
		}, keyspace)
	end

	if user.newUser then
		if user.notify or user.notify == nil then
			if not hasAllegroIntegration then
				local response = SendWelcomeMessage(user)
				LogInfo("[AGENT.USERS:UpdateUser] Sent welcome message response was: %s", response)
			else
				LogInfo("[AGENT.USERS:UpdateUser] Welcome messages are not sent to VCC %s with allegro integration", vcc)
			end
		end
	else
		local isNameChanged = (oldUser.Name ~= user.Name or oldUser.FirstSurname ~= user.FirstSurname or oldUser.LastSurname ~= user.LastSurname or oldUser.LastName ~= user.LastName) and true or nil
		if isNameChanged then
			--Se envia notificacion si se cambio el nombre del usuario a todos los agentes
			agentNotifications.NotifyAgents({
				scope = "administration",
				vcc = vcc,
				eventType = "administrator_user_information_changed",
				agentsLevel = "VCC_USERS",
				pollerMessage = {
					responsible_agent = inconcert.authUser.userName,
					userData = user
				}
			}, keyspace)
			-- Se elimina member cacheado en redis
			agentModel.GetRedisClient():hdel("CachedTeamMembers", userIdAtVCC)
		end
	end
	-- SUCCESS
	LogInfo("[AGENT.USERS:UpdateUser] Successfully Updated User '%s'", userIdAtVCC)

	--notifications
	notificationsEngine.SendNotification(user.newUser and "UserCreated" or "UserUpdated", 0, { user = user })

	if close_keyspace then keyspace:Close() end
	return true
end

function SaveUser (web)
	local allegroOtherInfo = MakeAllegroOtherInfo(web.vars)
	local vcc = inconcert.authUser.virtualContactCenter
	local postedData = web.input["post_data"]
	local res, user = pcall(json.decode, postedData)

	LogInfo("[AGENT.USERS:SaveUser] Saving User. post_data: '%s'", postedData)

	if not res or not user or type(user.Id) ~= "string" then
		web.status = 400
		return "Invalid request"
	end

	local keyspace = models.Users.GetCassandra():CassandraConnection("Agent")
	local applicationsActions = permissions.GetApplicationsActions(keyspace, vcc)
	local success, errType = UpdateUser(user, keyspace, applicationsActions, allegroOtherInfo)
	keyspace:Close()

	if not success then
		web.status = 400
		if type(errType) == "table" then
			return json.encode(errType)
		end
		return errType
	end

	-- SUCCESS
	return ""
end

--[[
	Retrieves the avatar for a specific member in case it's available
]]--
function GetAvatar (web, memberId)
	local vcc = inconcert.authUser.virtualContactCenter
	web.headers["Content-Type"] = "image/png"
	web.headers["Cache-Control"] = "max-age=1800, public, must-revalidate"
	local userAtVCC = memberId .. "@" .. vcc

	LogInfo("[USER:GetAvatar] Retrieving avatar for user %s", userAtVCC)

	local member = models.Users.Get(userAtVCC)
	if member and member.Avatar then
		return member.Avatar
	end

	web.headers["Content-Type"] = "image/svg+xml"
	local function getUserInitialsAndColor(firstName, firstSurname)
		local colors_options = {
    		["A"] = "#1abc9c",
    		["B"] = "#2ecc71",
    		["C"] = "#3498db",
    		["D"] = "#9b59b6",
   			["E"] = "#34495e",
    		["F"] = "#16a085",
    		["G"] = "#27ae60",
    		["H"] = "#2980b9",
    		["I"] = "#8e44ad",
   			["J"] = "#2c3e50",
    		["K"] = "#f1c40f",
    		["L"] = "#e67e22",
    		["M"] = "#e74c3c",
    		["N"] = "#a5a9aa",
    		["O"] = "#95a5a6",
    		["P"] = "#f39c12",
    		["Q"] = "#d35400",
    		["R"] = "#c0392b",
    		["S"] = "#e76c20",
    		["T"] = "#7f8c8d",
    		["U"] = "#504796",
    		["V"] = "#9b2c38",
    		["W"] = "#05983d",
    		["X"] = "#049a41",
    		["Y"] = "#075480",
    		["Z"] = "#810950"
		}

		local initialFirstName = firstName:sub(1, 1):upper()
		local initialFirstSurname = firstSurname:sub(1, 1):upper()
		return initialFirstName .. initialFirstSurname, colors_options[initialFirstName] or "#1abc9c"
	end

	local user_initials, color = getUserInitialsAndColor(member.Name, member.FirstSurname)
	local svg = string.format([[
		<svg xmlns="http://www.w3.org/2000/svg" width="24" height="24">
			<rect width="100" height="100" fill="%s" />
			<text x="50%%" y="50%%" font-weight="400" font-size="12" fill="white" text-anchor="middle" dy=".3em" font-family="Roboto, sans-serif">
                %s
            </text>
        </svg>
    ]], color, user_initials)
	return svg
end

--[[
	Set the status message for the logged user
]]--
function SetStatusMessage (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local name = inconcert.authUser.userName
	local postedData = web.input["post_data"]
	web.headers["Content-Type"] = "application/json"

	LogInfo("User %s on vcc %s requested to change his status message to %s", name, vcc, postedData)

	local data = json.decode(postedData)
	if (data == nil) then
		return json.encode({error = -1})
	end
	local userAtVCC = name .. "@" .. vcc
	local result = models.Users.SetStatusMessage(userAtVCC, data.message)
	return json.encode({result = result})
end

--[[
	Changes agent status
--]]
function ChangeUserStatus (web)
	web.headers["Content-Type"] = "application/json"
	local vcc = inconcert.authUser.virtualContactCenter
	local name = inconcert.authUser.userName
	local sessionId = inconcert.authUser.authToken
	local postedData = web.input["post_data"]

	LogInfo("[AGENT.USERS:ChangeUserStatus] Changing user '%s@%s' status to '%s' on session id '%s'.", name, vcc, postedData, sessionId)

	local state = json.decode(postedData).state
	local result = agentservice.ChangeAgentState(sessionId, name, vcc, state)
	if not result.success then
		web.status = result.httpCode
	end

	return result
end

function ChangeAvatar(web)
	local vcc = inconcert.authUser.virtualContactCenter
	local name = inconcert.authUser.userName

	LogInfo("Change Avatar of the user '%s@%s'", name, vcc)

	web.headers["Content-Type"] = "application/json"

	local postedData = web.input["post_data"]
	local size = web.vars["CONTENT_LENGTH"]

	LogInfo("Changing user %s@%s avatar", name, vcc)
	local result = models["Users"].SaveAvatar(name .. "@" .. vcc, {image = postedData, size = size})
	return json.encode(result)
end

local function CreateTemporaryFile(id, content)
	local path = string.gsub(m_moduleLocation, "controllers", "images/")
	local filename = path .. id .. ".jpg"
	local f = io.open(filename, "wb")
	if not f then
		LogError ("Can not open %s for writing", filename)
		return json.encode({result = "FAIL", reason = "Can not open file " .. filename .. " for writing"})
	end
	f:write(content)
	f:close()
	return filename
end

function GetUploadedAvatar(web, avatarId)
	web.headers["Content-Type"] = "image/png"
	_M.response.DontCache(web)

	local vcc = inconcert.authUser.virtualContactCenter
	local name = inconcert.authUser.userName

	local blob = models["TemporaryFiles"].GetFile(name, vcc, avatarId)
	if not blob then
		--this is in order for the load balancer to try another server
		web.status = 503
		return "File not found here"
	end
	return blob
end

function UploadAvatar(web)
	web.headers["Content-Type"] = "application/json"

	local vcc = inconcert.authUser.virtualContactCenter
	local name = inconcert.authUser.userName

	LogInfo("Upload Avatar of the user '%s@%s'", name, vcc)

	--Genero un Id unico para el avatar
	--Con este Id luego hago referencia a la seccion que el usuario recortara de la foto
	local avatarId = uuid.new()
	local postedData = web.input["post_data"]
	local size = web.vars["CONTENT_LENGTH"]

	models["TemporaryFiles"].SaveFile(name, vcc, avatarId, postedData)

	--send the file to disk in order to operate with it
	local filename = CreateTemporaryFile(avatarId, postedData)

	local image = LuaImageProcessing.Image:new()
	if not image:LoadFromFile(filename) then
		LogError ("Can not open image %s", filename)
		return json.encode({result = "FAIL", reason = "Can not open image"})
	end

	local width = image:Width()
	local height = image:Height()

	mayor = math.max(width, height)
	if mayor > 1024 then
		--La imagen es bastante grande, tengo que achicarla
		local ratio = mayor / 1024
		width = width / ratio
		height = height / ratio

		image, error = image:Resize (width, height)
		if not image then
			LogError ("Error resizing image: %s", filename)
			os.remove(filename)
			return json.encode({result = "FAIL", reason = "Error resizing image"})
		end

		LogInfo("Saving user %s@%s avatar to file %s", name, vcc, filename)
		image:SaveToFile(filename)
		--I need to open the file again and read the content in order to update the information in the distributed storage
		local f = io.open(filename, "rb")
		local modifiedFile = f:read("*a")
		models["TemporaryFiles"].SaveFile(name, vcc, avatarId, modifiedFile)
	end

	local features, err = image:DetectFeatures("frontalface_alt")
	if features then
		if #features == 0 then
			err = "Image does not contains faces"
			features = nil
		end
	end

	local region = nil
	if features then
		--Calculo el area mayor que engloba todas las features
		region = {
			x = width,
			y = height,
			x2 = 0,
			y2 = 0,
		}
		for i, reg in pairs (features) do
			region.x = math.min(reg.x, region.x)
			region.y = math.min(reg.y, region.y)
			region.x2 = math.max(reg.x + reg.width, region.x2)
			region.y2 = math.max(reg.y + reg.height, region.y2)
		end
	else
		LogWarning ("Could not detect features for file %s. Err = %s", filename, err)
	end

	os.remove(filename)
	return json.encode({result = "OK", avatarId = avatarId, region = region, imageSize = {w = width, h=height}})
end

function CropAvatar(web)
	web.headers["Content-Type"] = "application/json"

	local vcc = inconcert.authUser.virtualContactCenter
	local name = inconcert.authUser.userName
	local avatarCoords = json.decode(web.input["post_data"])

	LogInfo("Crop Avatar of the user '%s@%s'", name, vcc)

	if avatarCoords and avatarCoords.id then
		--Me definieron una imagen para cargar y cropear
		local blob = models["TemporaryFiles"].GetFile(name, vcc, avatarCoords.id)
		if not blob then
			--this is in order for the load balancer to search for the file on another server
			web.status = 503
			return "File not found here"
		end
		--send the file to disk in order to crop it 
		local filename = CreateTemporaryFile(avatarCoords.id, blob)
		local image = LuaImageProcessing.Image:new()
		if not image:LoadFromFile(filename) then
			local err = LogError("Error in CropAvatar: Can not open image  %s", filename)
			return json.encode({result = "FAIL", reason = err})
		end

		local image, err = image:CropRegion {
				x = avatarCoords.x,
				y = avatarCoords.y,
				width = avatarCoords.w,
				height = avatarCoords.h,
		}

		if not image then
			LogError("Error in CropAvatar: %s", err)
			return json.encode({result = "FAIL", reason = err})
		end

		image = image:Resize (64, 64)
		image:SaveToFile(filename)

		local f = io.open(filename, "rb")
		local content = f:read ("*a")
		f:close()

		models["Users"].SaveAvatar (name.."@"..vcc, {image = content, size = #content})

		--Elimino el archivo temporal
		os.remove(filename)
		models["TemporaryFiles"].Delete(name, vcc, avatarCoords.id)
		return json.encode({result = "OK"})
	end

	return json.encode({result = "FAIL"})
end

function GetClientIp(web)
	web.headers["Content-Type"] = "text/plain"
	local client_ip = web.vars.HTTP_X_CLIENT_IP

	if not client_ip or client_ip == '' then
		client_ip = web.vars["X-Forwarded-For"]
		if not client_ip or client_ip == '' then
			client_ip = web.vars["REMOTE_ADDR"]
		end
	end

	return client_ip
end

function GetDownloadSample(web)
	web.headers["Content-Type"] = "application/octet-stream"
	local download_size = tonumber(web.input["size"]) or 4096 -- 4 megabytes default

	local random_dev = "/dev/urandom"
	local rand = assert(io.open(random_dev, "rb"))

	-- LogDebug("=====> 1- Opened %s", random_dev)
	-- LogDebug("=====> 2- Reading %d bytes from %s", download_size, random_dev)
	
	local bytes = 0
	local buf = ""
	while bytes <= download_size do
		local chunk = rand:read(1024)
		-- LogDebug("=====> 3- 1024 bytes read from %s", random_dev)
		buf = buf .. chunk
		bytes = bytes + 1024
	end

	-- LogDebug("=====> 3- %d bytes read from %s", download_size, random_dev)

	return buf
end