require "md5"
local http = require("socket.http")
local ltn12 = require("ltn12")
local json = require("json")
local url = require("socket.url")
local utf8 = require "icu.utf8"
local m_gravatar = "http://www.gravatar.com"
local m_moduleLocation = select(2, ...)
local m_defaultSize = 60
local m_defaultExtension = "png"
local utils = require "shared.apps.utils"

module(..., package.seeall, inconcert.orbit.application.Decorate)

function GetRequest (url)
	local agentModel = require "apps.agent.model"
	local redis = agentModel:GetRedisClient()
	local success = utils.TestInternetConnection(redis)
	if not success then
		return false
	end

	local response_body = {}
	local result, code, header = socket.http.request{
		url = url,
		method = "GET",
		sink = ltn12.sink.table(response_body)
	}
	local response = table.concat(response_body,'')

	-- LogDebug("[GRAVATAR:GetRequest] --------> Result. Code: '%s'", code)
	-- LogDebug("[GRAVATAR:GetRequest] --------> -> result: '%s', code: '%s', header: '%s'.", result, code, json.encode(header))
	-- LogDebug("[GRAVATAR:GetRequest] --------> -> response: '%s'", response)
	-- LogDebug("")

	if code == 200 then
		return true, response, tonumber(header["content-length"])
	else
		if code == 404 then
			return false, nil
		else
			if code == "temporary failure in name resolution" then
				utils.UpdateTestInternetConnection(redis, false, code)
			end
			local message = "Error sending gravatar request, result code:".. tostring(code) .. "\nbody:"..response
			return false, message
		end
	end
end

---
--	Returns a gravatar avatar according to specifications on http://en.gravatar.com/site/implement/images/
--
function GetAvatar (email, size, extension)
	local hash = md5.sumhexa(utf8.lower(email))
	local imagesrc = string.format("%s/avatar/%s.%s?s=%s&d=404", m_gravatar, hash, extension or m_defaultExtension, size or m_defaultSize)
	LogDebug("Searching for gravatar avatar on email %s url %s", email, imagesrc)
	local result, image, imageSize = GetRequest(imagesrc)
	if result then
		return { image = image, size = imageSize }
	end
end

--- Returns a gravatar profile according to specifications on http://en.gravatar.com/site/implement/profiles/
--
function GetProfile (email)
	local hash = md5.sumhexa(utf8.lower(email))
	local profileUrl = string.format("%s%s.%s", m_gravatar, hash, "json")
	LogDebug("Searching for gravatar profile on email %s, url %s", email, profileUrl)
	local result, profile = GetRequest(profileUrl)
	if result then
		return json.decode(profile)
	end
end
