require "inconcert.translators"
local lua_cassandra = require "lua_cassandra"
require "redis"
local WrapRedis = require "wrapredis"
local utils = require "shared.apps.utils"
local m_moduleLocation = select(2, ...)
module(..., package.seeall)

local m_dataSource
local m_models = {}
local m_loaded = false
local m_ksCore

function SetKSCore(ksCore)
	m_ksCore = ksCore
end

function GetRedisClient ()
	return assert(m_dataSource.redis)
end

function GetAgentRedisClient ()
	return assert(m_dataSource.agentredis)
end

--recupera la lista de modelos del directorio models
local function GetModelsFromFilesystem()
	local modelsDir = m_moduleLocation .. "/models"
	if not utils.isDirectory(modelsDir) then
		return {}
	end
	local models = {}
	for file in lfs.dir(modelsDir) do
		if file ~= "." and file ~= ".." then
			local f = modelsDir..'/'..file
			local attr = lfs.attributes (f)
			if attr.mode ~= "directory" then
				local modelName = string.match(file, [[(.+).lua]])
				if (modelName) then
					models[modelName] = modelName
				end
			end
		end
	end
	return models
end
	
--///////////////////////////////////////////////////////////////////////////////////
-- arma la tabla de modelos definidos
local function CreateModelTable()
	--LogDebug("[AGENT.MODEL] ENTRA - CreateModelTable. m_loaded: %s", m_loaded)
	if not m_loaded then
		for key, controllerDef in pairs(GetModelsFromFilesystem()) do
			local model = require("apps.agent.models." .. key)
			if model.GetDefinition then
				m_models[model.GetName()] = model.GetDefinition(m_dataSource, m_ksCore)
			end
		end
		m_loaded = true
	else
		--if model modules were already loaded just overwrite their inner datasource
		--with the newly changed one
		for modelName, model in pairs(m_models) do
			model.GetDefinition(m_dataSource, m_ksCore)
		end
	end
	--LogDebug("[AGENT.MODEL] SALE - CreateModelTable")
end
--///////////////////////////////////////////////////////////////////////////////////
-- 
function GetModels()
	return m_models
end

--///////////////////////////////////////////////////////////////////////////////////
-- 
function BuildModels (self)
	local agentConfig = inconcert.environment.apps.agent

	--LogDebug("[AGENT.MODEL] 1 - BuildModels")
	--connect to the redis node 
	local redisConfig = agentConfig.redis	
	LogInfo("Connecting to redis on host: '%s', port:'%s'.", redisConfig.host, redisConfig.port)
	--start in nonblocking mode
	local redis = WrapRedis(redisConfig.host, redisConfig.port, redisConfig.pwd)
	if not redis then
		error("Unable to initialize agent app, connection to redis failed")
	end

	--connect to the agentredis node
	local agentredis
	local agentredisConfig = agentConfig.agentredis	
	if agentredisConfig then
		LogInfo("Connecting to agentredis on %s:%s", agentredisConfig.host, agentredisConfig.port)
		--start in nonblocking mode
		agentredis = WrapRedis(agentredisConfig.host, agentredisConfig.port, agentredisConfig.pwd)
		if not agentredis then
			error("Unable to initialize agent app, connection to agentredis failed")
		end
	end

	--init local datasource table
	m_dataSource = {
		cassandra  = lua_cassandra,
		redis      = redis,
		agentredis = agentredis
	}

	utils.SetupConnections(m_dataSource)

	CreateModelTable()
	--LogDebug("[AGENT.MODEL] 2 - BuildModels")
	return true
end
