module(..., package.seeall)

local m_cachedTimezones
local m_datasource

function GetName ()
	return "Timezones"
end

function GetDefinition (datasource)
	m_datasource = datasource
	return _M
end

function FindAll (keyspace)
	if m_cachedTimezones then
		return m_cachedTimezones
	end
	local closeKeyspace
	if not keyspace then
		keyspace = m_datasource.cassandra:CassandraConnection("Agent")
		closeKeyspace = true
	end
	m_cachedTimezones = {}
	local timezoneList = keyspace:GetRangeSlices("Timezones",  --column family
		"",  --supercolumn
		"", --startkey
		"", --endkey
		{ --predicate
			column_names = {},
			slice_range = {
				start = "",
				finish = "",
				reversed = false,
				count = 100
			}
		},
		1000 --total keys
	)
	if timezoneList then
		for timezoneId, timezoneRow in timezoneList:Iterator() do
			if #timezoneRow > 0 then
				--Ignoro posible registros borrados.
				local timezone = {}
				for k, v in ipairs(timezoneRow) do
					timezone[v.name] = v.value
					if v.name == "HistoricalDaylight" then
						local success, historicalDaylight = pcall(json.decode, v.value)
						if success then
							timezone[v.name] = historicalDaylight
						end
					end
				end
				timezone.Id = timezoneId
				m_cachedTimezones[timezoneId] = timezone
			end
		end
	end
	if closeKeyspace then
		keyspace:Close()
	end
	return m_cachedTimezones
end

function Get (timezoneId, keyspace)
	assert(timezoneId)
	assert(keyspace)
	local timezones = FindAll(keyspace)
	assert(timezones)
	local timezone = timezones[timezoneId]
	assert(timezone, "Timezone " .. timezoneId .. " does not exst !!")
	return timezone
end
