local LuaOrm = { 
	Date = require "LuaOrm.Date"
}
local cache = require "shared.cache.cache"
local httpRequest = require "shared.http.request"
local m_models = require("apps.administration.model"):GetModels()

module(..., package.seeall)

local function GetModel (entityName)
	assert(type(entityName) == "string")
	if not m_models or not m_models[entityName] then
	-- Si el modelo no se ha creado aun, debo crearlo.
	--  Esto sucede en algunas ocasiones si es que me han invocado desde otra app Ej: "Allegro" y aun no se ha creado el modelo.
	--  Entonces tengo que crear el modelo.
		local model = require "apps.administration.model"
		model.CreateModelTable()
		m_models = model:GetModels()
	end
	local entityModel = m_models[entityName]
	assert(type(entityModel) == "table")
	return entityModel
end

local m_apps = cache.New(function(vcc, ksAgent)
	LogInfo("[AGENT.NOTIFICATIONS:SendNotification] FetchNotifications vcc %s", vcc)
	local vccInfo = GetModel("VCCs").Get(vcc, ksAgent)
	--check has notifications
	if not vccInfo or not vccInfo.InteractionNotifications or vccInfo.InteractionNotifications.Enabled ~= "1" then
		return {}
	end
	--get apps
	local apps = GetModel("AppsNotifications").GetAppsNotifications(vcc, nil, ksAgent, vccInfo.AppsIntegrations) or {}
	return apps
end, 30)

local EVENTS_SUBEVENTS = {
	AdminUsers = {
		UserCreated = true,
		UserUpdated = true,
		UserDisabled = true,
		UserEnabled = true
	},
	AdminCampaigns = {
		CampaignCreated = true,
		CampaignUpdated = true,
		CampaignDeleted = true,
		CampaignSchedulerUpdated = true,
		CampaignIntegrationUpdated = true,
		CampaignDispositionsUpdated = true
	},
	AdminAccounts = {
		AccountCreated = true,
		AccountUpdated = true,
		AccountEnabled = true,
		AccountDisabled = true,
		AccountDeleted = true
	},
	AdminAttentionLevels = {
		AttLevelCreated = true,
		AttLevelUpdated = true,
		AttLevelDeleted = true
	},
	AdminCapabilities = {
		VccCapabilitiesUpdated = true,
		CampaignCapabilitiesUpdated = true,
		CampaignUserCapabilitiesUpdated = true
	},
	AdminRoles = {
		RoleCreated = true,
		RoleUpdated = true,
		RoleDeleted = true,
		SystemRoleOverrided = true,
		SystemRoleRestored = true,
		CampaignRolesUpdated = true,
	},
	AdminStates = {
		StateCreated = true,
		StateUpdated = true,
		StateDeleted = true
	},
	AdminSkills = {
		SkillCreated = true,
		SkillUpdated = true,
		SkillDeleted = true
	}
}

SUBEVENTS_EVENTS = {}
for event, subevents in pairs(EVENTS_SUBEVENTS) do
	for subevent, _ in pairs(subevents) do
		SUBEVENTS_EVENTS[subevent] = event
	end
end

local function GetEventApps (eventName, vcc, ksAgent)
	if SUBEVENTS_EVENTS[eventName] then
		eventName = SUBEVENTS_EVENTS[eventName]
	end
	local vccApps = m_apps.get(vcc, ksAgent)

	LogDebug("[AGENT.NOTIFICATIONS:GetEventApps] vccApps: '%s'.", json.encode(vccApps))

	local apps = {}
	for _, app in pairs(vccApps) do
		if app.Events and app.Events[eventName] then
			table.insert(apps, app)
		end
	end

	LogDebug("[AGENT.NOTIFICATIONS:GetEventApps] Load event: '%s' vcc: '%s' apps: '%s'.", eventName, vcc, json.encode(apps))

	return apps
end

function HasNotifications (eventName, vcc)
	if SUBEVENTS_EVENTS[eventName] then
		eventName = SUBEVENTS_EVENTS[eventName]
	end
	local vccApps = m_apps.get(vcc)
	for _, app in pairs(apps) do
		if app.Events and app.Events[eventName] then
			return true
		end
	end
	return false
end

local function SendNotificationToApp (app, encodedEvent)
	LogDebug("[AGENT.NOTIFICATIONS:SendNotificationToApp] 1 ==> Send notification to app: '%s', event: %s.", json.encode(app), encodedEvent)

	--#9325: Using AsyncTasks here as long as we dont expect a result, and to avoid request timeout errors
	local AsyncTasks = require ("AsyncTasks");
	encodedEvent = encodedEvent or ""
	local headers = {}
	headers["Content-Length"] = tostring(#encodedEvent)
	headers["Content-Type"] = "application/json"
	local task = AsyncTasks{
		url = app.Url,
		data = encodedEvent,
		headers = headers
	}
	task:Add("NotificationToApp");
end

function SendNotification (eventName, timestamp, data, ksAgent)
	local vcc = inconcert.authUser.virtualContactCenter

	LogInfo("[AGENT.NOTIFICATIONS:SendNotification] ==> START send notification event: '%s' vcc: '%s' timestamp: '%s'", eventName, vcc, timestamp)
	--LogInfo("[AGENT.NOTIFICATIONS:SendNotification] ==> START send notification data: %s.", json.encode(data or {}))
	local apps = GetEventApps(eventName, vcc, ksAgent)
	if #apps == 0 then
		return
	end
	data = data or {}
	data.clusterID = inconcert.environment.apps.agent.cluster
	data.vcc = vcc
	data.timestampDouble = timestamp or 0
	if data.timestampDouble <= 0 then
		data.timestampDouble = os.time()
	end
	data.timestamp = LuaOrm.Date(data.timestampDouble):toString("%Y-%m-%d %H:%M:%S")
	local event = {
		event = eventName,
		data = data
	}
	local encodedEvent = json.encode(event)
	for _, app in pairs(apps) do
		if app.Url and app.Url ~= "" then
			LogInfo("[AGENT.NOTIFICATIONS:SendNotification] vcc %s event %s app %s url %s", vcc, eventName, app.Id, app.Url)
			local status, err = pcall(SendNotificationToApp, app, encodedEvent)
			if not status then
				LogError("[AGENT.NOTIFICATIONS:SendNotification] vcc %s event %s app %s url %s - %s", vcc, eventName, app.Id, app.Url, err or "error sending notification")
			end
		end
	end
end
