/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
  strict:false, undef:true, unused:true, curly:true, browser:true, indent:false, maxerr:50, smarttabs:true,
  quotmark:false, esversion: 9 */
/* global inConcert, uuid */

//The sole purpose of this file is to work as a ajax wrapper in order to be easily replaced on testability scenarios

// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	var _T = inConcert.i18n.getString;

	/*
		This function will try to parse an error response from an HTTP request.
		Normally, this is a json, but sometimes nginx returns an html, or the WH returns an empty string or null.

		@returns: An object with the parsed json, if we couldn't parse the json, returns an error message inside the object.
	*/
	function getErrorFromResponse(response, jqXHR)
	{
		// nothing to do, response is empty
		if (!response) {
			return { result: "<%=_T('Empty server response')%>" };
		}

		if (jqXHR.getResponseHeader('Content-Type') !== 'application/json') {
		// No headers, or not JSON, just return the response inside the error object.
			return { result: response };
		}

		try {
			return JSON.parse(response);
		} catch (err) {
			var result = 'Error parsing data: ' + response.toString() + ', response type: "' + typeof(response) + '" with error: ' + err;
			inConcert.logger.error({ result: result, response: response, headers: jqXHR.getAllResponseHeaders() });
			return { result: result };
		}
	}

	//builtin error handler
	function defaultErrorHandler(response, textStatus, errorThrown)
	{
		if (textStatus === "abort") {
			return; // nothing to do
		}
		inConcert.logger.error({
			response_type : typeof(response),
			response      : response,
			status_code   : response.status,
			textStatus    : textStatus,
			errorThrown   : errorThrown
		});
	}

	//just invokes ajax request as specified by the caller and gives a default error handler in case none specified
	function invoke(parameters)
	{
		if (!parameters.error) {
			parameters.error = defaultErrorHandler;
		} else {
			var previousError = parameters.error;
			parameters.error = function(jqXHR, textStatus/*, errorThrown*/) {
				if (textStatus === "abort") {
					return; // nothing to do
				}
				var response = { status: jqXHR.status, jqXHR : jqXHR };
				var parsed_error = getErrorFromResponse(jqXHR.responseText, jqXHR);
				previousError(response, parsed_error, jqXHR.error ? jqXHR.error : '' );
			};
		}

		var previousHandler = parameters.beforeSend;
		parameters.beforeSend = function (request) {
			request.setRequestHeader('Content-Encoding', 'utf-8');
			request.setRequestHeader('x-icc-rid', uuid.v4());
			var id = document.cookie;
			var sid = id.match(/.*AUTH_SID_AGENT=([^;]+)/);
			sid = sid && sid[1];
			if (sid && sid.length > 0) {
				sid = decodeURIComponent(sid);
				request.setRequestHeader('x-inconcert-sid', "AGENT:" + sid);
			}
			if (typeof(previousHandler) === 'function') {
				previousHandler(request);
			}
		};

		if(!parameters.timeout) {
			parameters.timeout = 30 * 1000;
		}
		return jQuery.ajax(parameters);
	}

	if (typeof inConcert.invoke !== 'function') {
		inConcert.invoke        = invoke;
		inConcert.secureInvoke  = invoke;
	}

}());
