/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
  strict:false, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark: false,
  smarttabs:true, esversion: 9 */

/* global inConcert, _ , EmojiPicker*/


/*
	Handling GUI events to interact with appstore client session
*/

// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	var m_maxCharsOnTheWall = 1000;
	var m_maxCharsOnTheComment = 8000;
	var FORMAT = inConcert.Utils.format;

	var SCROLL_TO = inConcert.Utils.scroll_to;
	var TIME_SELECTED_COMMENT_MS = 5000;

	var MARK_ALL_READ_BTN_TEXT   = "<%=_T('Mark All As Read')%>";
	var MARK_ALL_UNREAD_BTN_TEXT = "<%=_T('Mark All As Unread')%>";

	var LNK_THREAD_MARK_AS_READ_CLASS   = "threadActionsMarkAsRead";
	var LNK_THREAD_MARK_AS_UNREAD_CLASS = "threadActionsMarkAsUnread";
	var LNK_ORIGIN_MARK_AS_READ_CLASS   = "originActionsMarkAsRead";
	var LNK_ORIGIN_MARK_AS_UNREAD_CLASS = "originActionsMarkAsUnread";
	
	var LNK_LIKE_CLASS   = "actionsLike";
	var LNK_UNLIKE_CLASS = "actionsUnlike";

	var CLASS_DELIVERY_SENDING    = "deliverySending";
	var CLASS_DELIVERY_SUCCESS    = "deliveryOk";
	var CLASS_DELIVERY_FAILED     = "deliveryFail";
	var CLASS_DELIVERY_DISCARDED  = "deliveryDiscarded";
	var CLASS_DELIVERY_DISCARDING = "deliveryDiscarding";

	/**
	Class AppstoreClientWindow.
	Appstore Client Handler
	@param interaction
	@param options
	*/
	function AppstoreClientWindow (interaction, options)
	{
		var self = this;

		//relate the interaction to the view
		this.m_maxCompressedComments = 20;
		this.m_interaction = interaction;
		this.hash_events = {};
		this.update_hash_events_return_diff();
		this.setView("client");
		this.setHorizontalResizingLimits(480);

		var subtype = interaction && interaction.interactionData && interaction.interactionData.subtype;

		if (subtype === "NewReview") {
			this.m_postComment = true;
		}
		else {
			throw "invalid subtype";
		}

		var success = AppstoreClientWindow.uber.constructor.call( this, options );
		if (!success) {
			return;
		}

		if (this.m_privateMessages) {
			this.setView("mail_client");
			this.replyByMail();
		}

		$("#appstoreClientCharacterCounter").text(m_maxCharsOnTheWall);
		this.has_created_main_thread = false;
		this.can_show_all_comments = false;
		this.are_showing_all_messages = false;
		this.loading_all_comments = false;

		if (options.readonly || options.monitor) {
			//cannot comment anything on this state
			$('.appstoreThreadCommentControlContainer').hide();
			//cannot reply in wall
			$("#replyInWall").hide();
		}
		
		$("#postCommentText").off('change').on('change', function() {
			$("#appstoreCommentButton").prop("disabled", !$(this).val());
		}).change();

		$("#appstoreCommentButton").click(function() {
			self.comment_current_post();
		});

		$("#replyInWall").click(function() {
			self.reply_in_wall();
		});

		//load avatar for the contact
		var avatar_uri = inConcert.app.getBaseUrl() + interaction.getAvatarURI();
		$('.interactionContactAvatar object').attr("data", avatar_uri);
		$("#appstoreWallText").keyup(function() {
			self.onCharCountChanged($(this));
		});

		$("#postCommentText").keypress(function() {
			var comment = $(this).val();
			if (comment.length >= m_maxCharsOnTheComment) {
				return false;
			}
		}).bind("paste", null, function() {
			var self = $(this);
			var previousValue = self.val();
			window.setTimeout(function() {
				var actualValue = self.val();
				if (actualValue.length >= m_maxCharsOnTheComment) {
					self.val(previousValue);
				}
			}, 1);
		});

		$("#appstoreGroup").text(this.m_interaction.campaign);

		this.was_being_composed = this.m_interaction.initiative === "OUTBOUND" && this.m_interaction.state === "PENDING";
		var pending_origin_post = this.m_interaction.get_pending_origin_post();
		if ( this.was_being_composed || pending_origin_post ) {
			this.reply_in_wall( pending_origin_post );
			$("#mainContainer").addClass("appstoreStartOutboundInteraction");
			if (!this.m_interaction.contact) {
				this.on_account_changed();
			}
		}
		else {
			var events = this.m_interaction.getEventList();
			if (events && events.length) {
				$("#appstoreDate").text(this.m_interaction.getLocaleDateString(0));
				$("#appstoreFrom").text(this.m_interaction.getSource(0));
			}
			var campaignAddresses = inConcert.app.getCampaignAddresses(this.m_interaction.campaign, "APPSTORE");
			for (var j = 0; j < campaignAddresses.length; j++) {
				if (campaignAddresses[j].id === this.m_interaction.getAccount()) {
					this.campaignAddress = campaignAddresses[j];
				}
			}
		}
		this.m_timer = window.setInterval(function() {
			self.onTimer();
		}, 1000);
	}

	inConcert.Utils.inherit( AppstoreClientWindow, inConcert.ClientWindow );

	AppstoreClientWindow.prototype.initialize = function()
	{
		AppstoreClientWindow.uber.initialize.call( this );
	};

	AppstoreClientWindow.prototype.getMenuContextualActions = function()
	{
		const actions = {
		// acciones solo para el cliente
			attachfile   : true,
			mark_all_read_unread: true,
		// acciones sobre la interaccion
			comment      : true,
			label        : true,
			emit_ticket  : true,
			search_ticket: true,
			queue        : true,
			transfer     : true
		// estas NO tienen que visualizarse en el menu, ya que las mismas se encuentran en la toolbar
		};
		return actions;
	};

	AppstoreClientWindow.prototype.clearInput = function($input) {
		var $input_emoji = $input.siblings(".emoji-wysiwyg-editor");
		if ($input_emoji.length) {
			$input_emoji.html("");
		}
		$input.val("");
	};

	AppstoreClientWindow.prototype.focusInput = function($input) {
		var $input_emoji = $input.siblings(".emoji-wysiwyg-editor");
		if ($input_emoji.length) {
			inConcert.Utils.placeCaretAtEnd($input_emoji.get(0));
		}
		else {
			$input.focus();
		}
	};

	AppstoreClientWindow.prototype.appendToInput = function($input, text) {
		var $input_emoji = $input.siblings(".emoji-wysiwyg-editor");
		if ($input_emoji.length) {
			$input_emoji.html($input_emoji.html() + " " + text);
		}
		$input.val($input.val() + " " + text);
	};

	AppstoreClientWindow.prototype.getWindowDefaultTitle = function()
	{
		return "<%=_T('Appstore Window')%>";
	};

	AppstoreClientWindow.prototype.eventsSuscribe = function()
	{
		AppstoreClientWindow.uber.eventsSuscribe.call( this );
		var self = this;

		if ( this.m_interaction && this.m_interaction.id ) {
			// failure ( POST, COMMENT, PRIVATE MESSAGE )
			var pub_appstore_failure_id =  "/appstore/" + this.m_interaction.id + "/failure";
			this.pubsub_appstore_failure_messages_handle = inConcert.app.messagebus.subscribe(
				pub_appstore_failure_id,
				function( event, type ) {
					self.show_proxy_failure( event, type );
				}
			);
		}
	};

	AppstoreClientWindow.prototype.eventsUnsuscribe = function()
	{
		AppstoreClientWindow.uber.eventsUnsuscribe.call( this );

		if ( this.pubsub_appstore_failure_messages_handle ) {
			inConcert.app.messagebus.unsubscribe( this.pubsub_appstore_failure_messages_handle );
		}
	};

	AppstoreClientWindow.prototype.show_proxy_failure = function( event /*, type*/ )
	{
		var short_msg = this.m_interaction ? this.m_interaction.get_error_message( event.error_code ) : "<%=_T('Appstore error.')%>";
		var long_msg = short_msg + ". " + "<%=_T('Please try again later.')%>";
		this.hideStatusMessage();
		this.displayStatusMessage( long_msg, "Error", true, 5000 );
		this.delivery_failed_message( event, short_msg );
	};

	/*
		Like/unlike an event
	*/
	AppstoreClientWindow.prototype.like = function( like, event_index, callback )
	{
		var self = this;
		callback = callback || $.noop;

		var message;
		if ( like ) {
			message = "<%=_T('Please wait, liking.')%>";
		}
		else {
			message = "<%=_T('Please wait, unliking.')%>";
		}
		this.displayStatusMessage( message, "Process", false );

		this.m_interaction.like(
			like,
			event_index,
			function( success, message ) {
				callback( success );
				self.hideStatusMessage();
				if (!success) {
					self.displayStatusMessage( message, "Error" );
				}
			}
		);
	};

	AppstoreClientWindow.prototype.setEventsHandlers = function()
	{
		var self = this;

		AppstoreClientWindow.uber.setEventsHandlers.call( this ); //invokes the base class method

		if ( !this.m_interaction.isBeingComposed() ) {
			$("#mark_all_read_unread").click( function()
			{
				self.markAllReadUnread();
			});
		}

		var $container = $("#thread_container");

		// reply comment link
		var event_selector = "[reply_comment_link]";
		$container
			.off("click", event_selector)
			.on("click", event_selector, function ()
			{
				if ( !self.canExecuteInteractionAction() ) {
					return;
				}
				var $reply_link = $( this );
				var message_id = $reply_link.data("message_id");
				var name = $reply_link.data("user_name");
				var $comment = $reply_link.parent();

				if ( $comment.hasClass( "unread" ) ) {
					var event_index = $reply_link.data("event_index");
					self.mark_as_read(
						true,
						event_index,
						function( success ) {
							self.on_readed_event( success, $comment );
						}
					);
				}

				self.setReplyToComment( message_id, name );
			});

		// mark as read/unread comment link
		event_selector = "[mark_as_read_unread]";
		$container
			.off("click", event_selector)
			.on("click", event_selector, function ()
			{
				var $mark_read_unread = $( this );

				if ( $mark_read_unread.prop( "disabled" ) ) {
					return;
				}

				var $comment = $mark_read_unread.parent();
				if ( $comment.hasClass( "selected" ) ) {
					return;
				}

				var event_index = $mark_read_unread.data( "event_index" );
				var new_state = $mark_read_unread.data( "event_unread" ) === "unread";
				self.mark_as_read(
					new_state,
					event_index,
					function( success ) {
						self.on_readed_event( success, $comment );
					}
				);
			});

		// mark as read/unread comment link
		event_selector = "[like_unlike]";
		$container
			.off("click", event_selector)
			.on("click", event_selector, function ()
			{
				if ( !self.canExecuteInteractionAction() ) {
					return;
				}
				var $like_unlike = $( this );

				if ( $like_unlike.prop( "disabled" ) ) {
					return;
				}

				var event_index = $like_unlike.data( "event_index" );
				var new_state = $like_unlike.data( "event_like" ) === "like";
				self.like(
					new_state,
					event_index,
					function( success ) {
						self.on_liked_event( success, new_state, $like_unlike.parent() );
					}
				);
			});

		// resend failed comment link
		var on_resend = function( success, message )
		{
			if ( !success ) {
				// Estos son errores genericos. Los errores de appstore se procesan en "AppstoreClientWindow.eventsSuscribe / failure"
				self.hideStatusMessage();
				self.displayStatusMessage( message, "Error" );
				return;
			}
			// El OK se procesa cuando llega el evento "ClientWindow.onInteractionUpdated -> AppstoreClientWindow.clientUpdate"
		};

		event_selector = "[resend_message]";
		$container
			.off("click", event_selector)
			.on("click", event_selector, function ()
			{
				if ( !self.canExecuteInteractionAction() ) {
					return;
				}
				var $resend = $( this );
				var $comment = $resend.parent().parent();
				var fake_event_id = $comment.attr( "data-id" );
				var failed_event = self.m_interaction.get_pending_event( fake_event_id );
				if ( !failed_event ) {
					return;
				}

				self.displayStatusMessage("<%=_T('Resending...')%>", "Process", true);
				self.delivery_sending_message( $comment, "<%=_T('Resending')%>" );

				if ( self.m_interaction.isPrivateMessage() ) {
					self.m_interaction.newPrivateMessage(
						fake_event_id,
						failed_event.message,
						function( success, message ) { on_resend( success, message ); }
					);
				}
				else {
					self.m_interaction.commentPost(
						fake_event_id,
						failed_event.postId,
						failed_event.commentId,
						failed_event.message,
						function( success, message ) { on_resend( success, message ); }
					);
				}
			});

		// discard failed comment link
		event_selector = "[discard_message]";
		$container
			.off("click", event_selector)
			.on("click", event_selector, function ()
			{
				if ( !self.canExecuteInteractionAction() ) {
					return;
				}
				var $discard = $( this );
				var $comment = $discard.parent().parent();
				var fake_event_id = $comment.attr( "data-id" );
				if ( !fake_event_id ) {
					return;
				}
				var failed_event = self.m_interaction.get_pending_event( fake_event_id );
				if ( !failed_event ) {
					return;
				}

				self.displayStatusMessage("<%=_T('Discarding...')%>", "Process", true);
				self.delivery_discarding_message( $comment );

				self.m_interaction.discardFailedEvent( fake_event_id,
					function( success )
					{
						self.hideStatusMessage();
						if ( success ) {
							self.delivery_discarded_message( $comment );
							return;
						}
						self.displayStatusMessage( FORMAT( "<%=_T('Error discarding message {0}')%>", fake_event_id), "Error", true, 5000 );
						self.delivery_failed_message( failed_event );
					}
				);
			});

		// show / hide message error !!!
		var timeout_handle;
		var show_error = function( $error, can_show ) {
			var $err_msg_container = $error.parent().find( ".deliveryFailReasonContainer" );
			if ( can_show ) {
				window.clearTimeout( timeout_handle );
				var show_error_timeout_ms = 200;
				timeout_handle = window.setTimeout( function() { $err_msg_container.show(); }, show_error_timeout_ms );
			}
			else {
				window.clearTimeout( timeout_handle );
				$err_msg_container.hide();
			}
		};

		event_selector = ".deliveryMessageContainer." + CLASS_DELIVERY_FAILED;
		$container
			.off("mouseenter", event_selector)
			.off("mouseleave", event_selector)
			.on("mouseenter", event_selector, function () { show_error( $( this ), true  ); })
			.on("mouseleave", event_selector, function () { show_error( $( this ), false ); });

				/* Gestionar contacto si:
		 1- Esta habilitado la administracion de contactos
		 2- Se puede visualizar los addresses
		*/
		const visualize_address = inConcert.app.canVisualizeAddress(null, this.m_interaction.campaign, null, true);
		if ( this.m_interaction.accountHasContactManagementEnabled( this.from_app ) && visualize_address) {
			//Setup contact manager popup when user click in the contact
			event_selector = ".subEventAuthor";
			$container
				.off("click", event_selector)
				.on("click", event_selector, function ()
				{
					var event_index = $( this ).data( "event_index" );
					if ( !event_index ) {
						return;
					}
					var event = self.m_interaction.getEvent( event_index );
					if ( !event || event.creator !== "contact" ) {
						return;
					}

					var address_normalized = inConcert.app.getContactManager().normalizeContactAddress(self.m_interaction.getRawAddress( event_index ), "Appstore");

					var contact_window = new inConcert.ContactWindow({
						from_app    : self.from_app,
						interaction : self.m_interaction,
						name        : self.m_interaction.getUserName( event ),
						avatar      : self.m_interaction.getEventUserAvatarURI( event ),
						addresses   : [ address_normalized ]
					});
					contact_window.show();
				});
		}
	};

	AppstoreClientWindow.prototype.set_mark_all_unread_read_button_caption = function()
	{
		var caption = this.m_interaction.getUnreadEventCount() > 0 ? MARK_ALL_READ_BTN_TEXT : MARK_ALL_UNREAD_BTN_TEXT;
		$("#mark_all_read_unread").val( caption );
	};

	AppstoreClientWindow.prototype.getScrollBasicInfo = function()
	{
		return {
			$scroll : $("#thread_container"), // el contenedor de la lista con el scroll
			increase_position : -170 // este es un magic number que sirve unicamente en el caso en que "destination='item_position'",
									// decrementa un valor para poder ajustarlo a la pantalla correctamente.
		};
	};

	AppstoreClientWindow.prototype.getCaptionRelatedButton = function()
	{
		return FORMAT( "<%=_T('{0} Unread')%>", this.m_interaction.getUnreadEventCount() );
	};

	AppstoreClientWindow.prototype.getRelatedEvents = function() {
		return this.m_interaction.getUnreadEvents();
	};

	AppstoreClientWindow.prototype.invalidInteraction = function() {
		AppstoreClientWindow.uber.invalidInteraction.call( this );
		$(".interactionActionsButtons").children().hide();
		$(".appstoreCommentControl").hide();
		$(".appstoreThreadCommentControlContainer").hide();
	};

	AppstoreClientWindow.prototype.onInteractionGone = function(interaction) {
		AppstoreClientWindow.uber.onInteractionGone.call( this, interaction );
		$("#postCommentText").prop("disabled", true);
		$("#appstoreComment").prop("disabled", true);
		$("#appstoreWallText").prop("disabled", true);
		$("#appstoreWallButtonPost").prop("disabled", true);
	};

	AppstoreClientWindow.prototype.__setExternalMessage = function(message)
	{
		const self = this;

		if (!this.m_interaction) {
			return;
		}
		message = message || "";

		let $writing_area;
		const was_being_composed = this.m_interaction.initiative === "OUTBOUND" && this.m_interaction.state === "PENDING";
		const pending_origin_post = this.m_interaction.get_pending_origin_post();
		let afterSetFn = $.noop;

		if ( was_being_composed || pending_origin_post ) {
			$writing_area = $("#facebookWallText");
			afterSetFn = function() {
				self.onCharCountChanged($writing_area);
			};
		}
		else {
			if ( this.m_interaction.isPrivateMessage() ) {
				$writing_area = $("#facebookMailText");
			}
			else {
				$writing_area = $("#postCommentText");
			}
		}
		this.appendToInput($writing_area, message);
		this.focusInput($writing_area);
		afterSetFn();
	};

	AppstoreClientWindow.prototype.setCannedMessage = function(canned)
	{
		this.__setExternalMessage(canned.message);
	};

	AppstoreClientWindow.prototype.setQnAMakerAnswer = function(answer)
	{
		this.__setExternalMessage(answer.answer);
	};

	AppstoreClientWindow.prototype.on_account_changed = function()
	{
		AppstoreClientWindow.uber.on_account_changed.call( this );
		
		
	};

	AppstoreClientWindow.prototype.onCharCountChanged = function(textArea) {
		var currentChars = document.getElementById(textArea.attr("id")).value.length;
		var charsDifference = m_maxCharsOnTheWall - currentChars;
		var disabledWallButton = charsDifference < 0;
		$("#appstoreClientCharacterCounter").text(charsDifference)[disabledWallButton ? "addClass" : "removeClass"]("appstoreClientCharacterCounterNegative");
		$("#appstoreWallButtonPost").prop("disabled", disabledWallButton);
	};

	AppstoreClientWindow.prototype.setView = function( view_id, pending_event )
	{
		var class_name;
		var $comment = $("#appstoreComment");
		var $wall = $("#appstoreWallMessage");
		var $private_message = $("#appstorePrivateMessage");

		switch( view_id )
		{
			case "client" :
				class_name = "appstoreClientContainer";
				//hide
				$private_message.hide();
				$wall.hide();
				// show
				$comment.show();
				break;

			case "wall_client" :
				class_name = "appstoreWallClientContainer";
				var $wall_err_container = $("#appstoreWallError");
				$wall_err_container.hide();

				if ( pending_event ) {
					var state_class = "";
					var state = pending_event.state || "";

					switch( state ) 
					{
						case "DELIVERED":
							// NO HAGO NADA!!! ( CLASS_DELIVERY_DELIVERED )
						break;
						case "DISCARDED":
							state_class = CLASS_DELIVERY_DISCARDED;
						break;
						case "DISCARDING":
							state_class = CLASS_DELIVERY_DISCARDING;
						break;
						case "FAILED":
							var short_msg = this.m_interaction.get_error_message( pending_event.error_code );
							$wall_err_container.show();
							$wall_err_container.find("#appstoreWallErrorMessage").text(short_msg);
							state_class = CLASS_DELIVERY_FAILED;
						break;
						case "SENDING":
							state_class = CLASS_DELIVERY_SENDING;
						break;
						default:
							throw "Unrecognized pending event '" + state + "' state";
					}

					class_name += " " + state_class;
				}
				//hide
				$comment.hide();
				$private_message.hide();
				// show
				$wall.show();
				break;

			case "mail_client" :
				class_name = "appstoreMailClientContainer";
				//hide
				$comment.hide();
				$wall.hide();
				// show
				$private_message.show();
				break;

			default:
				throw "AppstoreClientWindow.prototype.setView. Unrecognized view_id: '" + view_id + "'";
		}

		var initiative = this.m_interaction.initiative;

		$("#mainContainer")
			.removeClass()
			.addClass( class_name )
			.addClass( initiative === "OUTBOUND" ? "appstoreOutboundClientContainer" : "appstoreInboundClientContainer" );
	};

	AppstoreClientWindow.prototype.reply_in_wall = function( pending_origin_post )
	{
		var self = this;

		this.setView("wall_client", pending_origin_post);
		$("#replyInWall").prop("disabled", true);

		var text = "";
		var $wall_text = $("#appstoreWallText");
		if ( pending_origin_post ) {
			text = pending_origin_post.message;
			$wall_text.trigger("click");
		}

		document.getElementById("appstoreWallText").value = text;

		$wall_text.focus();
		$("#appstoreClientCharacterCounter").text(m_maxCharsOnTheWall);

		var on_send = function( success, message )
		{
			if ( !success ) {
			// Estos son errores genericos. Los errores de appstore se procesan en "AppstoreClientWindow.eventsSuscribe / failure"
				self.hideStatusMessage();
				self.displayStatusMessage( message, "Error" );
				return;
			}
			// El OK se procesa cuando llega el evento "ClientWindow.onInteractionUpdated -> AppstoreClientWindow.clientUpdate"
		};

		$("#appstoreWallButtonPost")
			.unbind('click')
			.bind('click', function()
			{
				if (!self.canExecuteInteractionAction()) {
					return;
				}
				self.displayStatusMessage("<%=_T('Posting...')%>", "Process", true);
				var message = document.getElementById("appstoreWallText").value;
				var destination = $("#cboAccounts").val();

				var fake_event_id; // tiene que ser pasado como parametro en undefined
				pending_origin_post = self.m_interaction.get_pending_origin_post();
				if ( !pending_origin_post ) {
					self.m_interaction.newPost( fake_event_id, destination, message, on_send );
				}
				else {
					// resend
					fake_event_id = pending_origin_post.fake_event_id;
					self.m_interaction._newPost( fake_event_id, destination, message, on_send );
				}
			}
		);
	};

	/*
	Esta funcion actualiza la informacion del usuario (id, name, avatar), de los eventos pendings
	En las acciones del lado del cliente (delivery_discarding_message y delivery_sending_message)
	Esta funcion no es incovada en las respuestas del delivery

	Es necesaria cuando el evento fallido lo genero un agente diferente al que se encuentra logueado.
	*/
	AppstoreClientWindow.prototype.delivery_update_user_info = function( $pending )
	{
		var $author = $pending.find( ".subEventAuthor" );
		var author_id = $author.data( "event_user_id" );

		if ( typeof author_id !== "string" ) {
			return;
		}
		var logged_user_id = inConcert.app.getCompleteId();
		if ( author_id === logged_user_id ) {
			return;
		}

		$pending.find( ".subEventAvatar object" ).attr( "data", inConcert.app.getLoggedUserAvatar() );
		$author.data( "event_user_id", logged_user_id );
		$author.text( inConcert.app.getLoggedUserName() );
	};

	AppstoreClientWindow.prototype.delivery_success_message = function( $success, success_event )
	{
		var $thread_container = $("#thread_container");

		var $siblings, $last_message, last_message_id;
		var parent_id = success_event.message.parent_id;
		var is_reply = _.isString( parent_id );

		// obtengo el ultimo mensaje del hilo al cual corresponde el evento SUCCESS
		if ( is_reply ) {
			// si es reply, lo obtengo de su hilo
			$siblings = $thread_container.find("[data-parent_id=" + parent_id +"]");
			$last_message = $siblings.last();
			last_message_id = $last_message.attr( "data-id" );
		}
		else {
			// si no es un reply, lo obtengo del hilo principal
			$siblings = $thread_container.find(".appstoreThreadSubEvent");
			$last_message = $siblings.last();
			last_message_id = $last_message.attr( "data-id" );
		}

		var fake_event_id = $success.attr( "data-id" );
		var previous_state = $success.attr( "previous_state" );

		var move_last = ( previous_state === "failed" && fake_event_id !== last_message_id );
		if ( move_last ) {
		// si habia fallado y el ultimo mensaje del hilo al cual pertenece no es el mismo evento,
		// le realizao detach ya que se tiene que agrega al final de su thread ( como ultimo mensaje )
		// (*)
			$success.removeAttr( "previous_state" );
			$success.detach();
		}

		$success
			.removeClass( CLASS_DELIVERY_SENDING )
			.removeClass( CLASS_DELIVERY_FAILED )
			.addClass( CLASS_DELIVERY_SUCCESS )
			.attr( "data-id", success_event.message.id )
			.attr( "data-time", success_event.creationDate.getTime() );

		var $date = $success.find( ".subEventDate" );
		$date.text(inConcert.Utils.userFormatDate(success_event.creationDate, false, true));

		var $delivery_message_container = $success.find( ".deliveryMessageContainer" );
		var $lbl_message = $success.find( ".deliveryMessageText" );
		$lbl_message.text( "<%=_T('Delivered')%>" );

		$delivery_message_container
			.removeClass( CLASS_DELIVERY_SENDING )
			.removeClass( CLASS_DELIVERY_FAILED )
			.addClass( CLASS_DELIVERY_SUCCESS )
			.show();

		var $err_msg_container = $success.find( ".deliveryFailReasonContainer" );
		$err_msg_container.hide();

		var $failed_actions = $success.find( ".deliveryFailActions" );
		$failed_actions.hide();

		if ( move_last ) {
		// como hemos explicado arriba en (*), debemos mover el mensaje a la ultima posicion del hilo al cual pertenece!!!
			if ( is_reply ) {
			// si es reply tiene que validar si tiene hermanos o no, para saber donde ubicarlo!!
				if ( $siblings.length === 0 ) {
					$thread_container.find("[data-id='" + parent_id + "']").after( $success );
				}
				else {
					$siblings.last().after( $success );
				}
			}
			else {
			// si no es reply derecho al final
				$thread_container.append( $success );
			}

			var scroll_data = this.getScrollBasicInfo();
			scroll_data.destination = "item_position";
			scroll_data.$item = $success;
			SCROLL_TO( scroll_data );
		}

		window.setTimeout(function()
		{
			$success.removeClass( CLASS_DELIVERY_SUCCESS );
			$delivery_message_container.removeClass( CLASS_DELIVERY_SUCCESS ).hide();
		}, TIME_SELECTED_COMMENT_MS);
	};

	AppstoreClientWindow.prototype.delivery_sending_message = function( $pending, message )
	{
		var has_failed = $pending.hasClass( CLASS_DELIVERY_FAILED );
		if ( has_failed ) {
			$pending.attr( "previous_state", "failed");
		}

		$pending
			.removeClass( CLASS_DELIVERY_FAILED )
			.addClass( CLASS_DELIVERY_SENDING );

		var $delivery_message_container = $pending.find( ".deliveryMessageContainer" );
		var $lbl_message = $delivery_message_container.find( ".deliveryMessageText" );
		$lbl_message.text( message || "<%=_T('Sending')%>" );
		$delivery_message_container
			.removeClass( CLASS_DELIVERY_FAILED )
			.addClass( CLASS_DELIVERY_SENDING )
			.show();

		var $err_msg_container = $pending.find( ".deliveryFailReasonContainer" );
		$err_msg_container.hide();

		var $failed_actions = $pending.find( ".deliveryFailActions" );
		$failed_actions.hide();

		this.delivery_update_user_info( $pending );
	};

	AppstoreClientWindow.prototype.delivery_failed_message = function( failed_event, err_msg )
	{
		var $pending = $("#thread_container").find("[data-id='" + failed_event.fake_event_id +"']");
		if ( $pending.length === 0 ) {
			return;
		}

		$pending
			.removeClass( CLASS_DELIVERY_SENDING )
			.addClass( CLASS_DELIVERY_FAILED );

		var $delivery_message_container = $pending.find( ".deliveryMessageContainer" );
		var $lbl_message = $delivery_message_container.find( ".deliveryMessageText" );
		$lbl_message.text( "<%=_T('Failed')%>" );
		$delivery_message_container
			.removeClass( CLASS_DELIVERY_SENDING )
			.addClass( CLASS_DELIVERY_FAILED )
			.show();

		var $err_msg_container = $pending.find( ".deliveryFailReasonContainer" );
		var $err_msg = $err_msg_container.find( ".deliveryFailReasonTxt" );
		if ( $err_msg.text().length > 0 || err_msg ) {
			if ( err_msg ) {
				$err_msg.text( err_msg );
			}
		}

		if ( this.isEditable() ) {
			var $failed_actions = $pending.find( ".deliveryFailActions" );
			$failed_actions.show();
		}
	};

	AppstoreClientWindow.prototype.delivery_discarding_message = function( $discarding )
	{
		$discarding
			.removeClass( CLASS_DELIVERY_FAILED )
			.addClass( CLASS_DELIVERY_DISCARDING );

		var $delivery_message_container = $discarding.find( ".deliveryMessageContainer" );
		var $lbl_message = $delivery_message_container.find( ".deliveryMessageText" );
		$lbl_message.text( "<%=_T('Discarding')%>" );
		$delivery_message_container
			.removeClass( CLASS_DELIVERY_FAILED )
			.addClass( CLASS_DELIVERY_DISCARDING )
			.show();

		var $err_msg_container = $discarding.find( ".deliveryFailReasonContainer" );
		$err_msg_container.hide();

		var $failed_actions = $discarding.find( ".deliveryFailActions" );
		$failed_actions.hide();

		this.delivery_update_user_info( $discarding );
	};

	AppstoreClientWindow.prototype.delivery_discarded_message = function( $discarded )
	{
		$discarded
			.removeClass( CLASS_DELIVERY_SENDING )
			.removeClass( CLASS_DELIVERY_DISCARDING )
			.removeClass( CLASS_DELIVERY_FAILED )
			.addClass( CLASS_DELIVERY_DISCARDED );

		var $delivery_message_container = $discarded.find( ".deliveryMessageContainer" );
		var $lbl_message = $delivery_message_container.find( ".deliveryMessageText" );
		$lbl_message.text( "<%=_T('Discarded')%>" );
		$delivery_message_container
			.removeClass( CLASS_DELIVERY_SENDING )
			.removeClass( CLASS_DELIVERY_DISCARDING )
			.removeClass( CLASS_DELIVERY_FAILED )
			.addClass( CLASS_DELIVERY_DISCARDED )
			.show();

		var $err_msg_container = $discarded.find( ".deliveryFailReasonContainer" );
		$err_msg_container.hide();

		var $failed_actions = $discarded.find( ".deliveryFailActions" );
		$failed_actions.hide();
	};

	AppstoreClientWindow.prototype.delivery_remove = function( failed_event )
	{
		var $pending = $("#thread_container").find("[data-id='" + failed_event.fake_event_id +"']");
		if ( $pending.length === 0 ) {
			return;
		}
		$pending.fadeOut( 1000, function(){ 
			$pending.remove();
		});
	};

	AppstoreClientWindow.prototype.on_private_message_created = function( new_event, sorted )
	{
		var self = this;

		this.hideStatusMessage();
		this.updateMessageColapsedComments();

		var incoming_realtime = sorted;
		if ( !incoming_realtime ) {
			this.clearInput($("#appstoreMailText"));
		}

		var $pending;
		if ( new_event.creator.toLowerCase() === "agent" && new_event.fake_event_id ) {
			$pending = $("#thread_container").find("[data-id='" + new_event.fake_event_id +"']");
		}

		if ( $pending && $pending.length > 0 ) {
			this.delivery_success_message( $pending, new_event );
		}
		else {
			var callback = function( _, can_scroll )
			{
				if ( !can_scroll ) {
					return;
				}
				var scroll_data = self.getScrollBasicInfo();
				scroll_data.destination = "bottom";
				SCROLL_TO( scroll_data );
			};
			this.add_new_message_stub( new_event,
				callback,
				sorted
			);
		}
	};

	AppstoreClientWindow.prototype.replyByMail = function() 
	{
		var self = this;
		this.setView("mail_client");
		$("#replyInWall").prop("disabled", false);
		var helperMessage = "<%=_T('Send a private message')%>";
		$("#appstorePrivateMessage > .appstoreInteractionControlTitle").text(helperMessage);
		var $txt = $("#appstoreMailText");
		this.clearInput($txt);
		
		$txt.off('change').on('change', function() {
			$("#appstoreMailButtonSend").prop("disabled", !$(this).val());
		}).change();

		$("#appstoreMailButtonSend").unbind('click').bind('click', function() {
			if (!self.canExecuteInteractionAction()) {
				return;
			}
			self.displayStatusMessage("<%=_T('Sending...')%>", "Process", true);
			
			//mark all as read
			if ($("#mark_all_read_unread").val() === MARK_ALL_READ_BTN_TEXT) {
				$("#mark_all_read_unread").click();
			}
			
			var text = $txt.val();
			var message = { text : text };
			var event = self.get_current_event();
			if ( !event ) {
				return;
			}
			var fake_event_id = self.m_interaction.newPrivateMessage(
				undefined, // fake_event_id
				message,
				function( success, message, error ) {
					if ( success ) {
					// Los success SE REALIZAN CUANDO LLEGA LA ACTUALIZACION DE LA INTERACTION "onUpdateInteraction->clientUpdate"
						return;
					}
					if ( error.agent_err ) {
					// estos son errores que se producen en controller interactions del lado del agente (ejemplo sin licencia para el canal).
					// Los errores de delivery, llegan desde el IR en la funcion "eventsSuscribe"
						event.fake_event_id = fake_event_id;
						self.hideStatusMessage();
						self.displayStatusMessage( message, "Error", true, 5000 );
						self.delivery_remove( event );
					}
				}
			);
			self.add_new_outgoing_fake_event( fake_event_id, message );
		});
	};

	/*
		fake_event_id
		text
		parent_comment - opcional 
		creation_date  - opcional
		state          - opcional
		agent          - opcional
	*/
	AppstoreClientWindow.prototype.make_fake_event = function( fake_data )
	{
		var self = this;

		var get_parent = function()
		{
			var origin_event = self.get_current_event();
			if ( !origin_event ) {
				return "";
			}
			return origin_event.message.id;
		};
		var creation_date = fake_data.creation_date || new Date(); // si no viene la fecha, toma now
		var contact_form = $("#contactFrom").text();

		return {
			outgoing_pending : true,
			state            : fake_data.state || "SENDING",
			agent            : fake_data.agent || inConcert.app.getCompleteId(),
			agentName        : contact_form,
			creationDate     : creation_date,
			parent_id        : get_parent(),
			message : {
				id           : fake_data.fake_event_id,
				timestamp	 : creation_date.getTime() / 1000,
				parent_id    : fake_data.parent_comment,
				text      	 : fake_data.message.text || "",
				userid		 : this.m_interaction.getAccount(),
				name		 : contact_form
			}
		};
	};

	AppstoreClientWindow.prototype.add_new_outgoing_fake_event = function( fake_event_id, message, parent_comment /* opcional */ )
	{
		var self = this;
		var $txt;
		var scroll_data = this.getScrollBasicInfo();
		scroll_data.destination = "bottom";
		scroll_data.callback = function() {
			self.focusInput($txt);
		};

		if ( this.m_interaction.isPrivateMessage() ) {
			$txt = $("#appstoreMailText");
		}
		else {
			$txt = $("#postCommentText");
			if ( parent_comment ) {
				scroll_data.destination = "item_position";
				scroll_data.type = "without_animation";
			}
		}
		this.clearInput($txt);

		var fake_event = this.make_fake_event({
			fake_event_id  : fake_event_id,
			message        : message,
			parent_comment : parent_comment
		});

		this.add_new_message_stub( fake_event, function( $comment )
		{
			if ( scroll_data.destination === "item_position" ) {
				scroll_data.$item = $comment;
			}
			SCROLL_TO( scroll_data );
		});
	};

	AppstoreClientWindow.prototype.afterRenderRelatedMarkup = function() {
		$("#unreadTitle").html(`<%=_T('Unread messages')%> <span>(${this.m_interaction.getUnreadEventCount()})</span>`);
	};

	AppstoreClientWindow.prototype.realtime_add_new_incoming_message = function ( $comment, new_event )
	{
		if ( !$comment || $comment.length === 0 ) {
			return false;
		}

		var can_scroll = false;
		var $container = $("#thread_container");
		var parent_comment = new_event.message.parent_id;
		var is_reply = _.isString( parent_comment );
		var $siblings = is_reply ? 
						$container.find(".replyMessage[data-parent_id='" + parent_comment +"']") :
						$siblings = $container.find(".topLevelMessage");

		if ( $siblings.length === 0 ) {
		// si es el primer evento del hilo principal o del hilo de una respuesta no tiene hermanos, lo inserta primero en la lista
			if ( is_reply ) {
				$container.find("[data-id='" + parent_comment +"']").after( $comment );
			}
			else {
				$container.append( $comment );
			}
		}
		else {
		// Si tiene hermanos, recorre el hilo al cual pertenece, buscando entre sus hermanos el primero que tenga una fecha/hora menor a la del current event.
		// La busqueda es realizada comenzando desde el ultimo ( ya que estan ordenado x fecha ).
			var added = false;
			var new_event_time = new_event.creationDate.getTime();

			for ( var i = $siblings.length - 1; i >= 0; i -- ) {
				if ( !$siblings[ i ] ) {
					continue;
				}
				var $sibling = $( $siblings[ i ] );
				var sibling_time = $sibling.data( "time" );
				if ( sibling_time <= new_event_time ) {
				// si la fecha del hermano es menor o igual al nuevo evento, inserto este nuevo evento despues del hermano
					$sibling.after( $comment );
					added = true;
					break;
				}
			}

			if ( !added ) {
			// si no existe un hermano menor tiene que agregarlo si o si al nodo
				if ( is_reply ) {
					$siblings.last().after( $comment );
				}
				else {
					$container.append( $comment );
				}
			}
		}

		delete new_event.is_new;
		return can_scroll;
	};

	AppstoreClientWindow.prototype.add_new_message_stub = function ( new_event, callback, sorted )
	{
		var self = this;
		callback = callback || $.noop;
		var stub = new_event;
		this.m_interaction.getEvent( 0 ).children.push( stub );
		var event_index = new_event.__index || this.m_interaction.getEventCount();
		var $container = $("#thread_container");

		//
		this.insert_comment_in_dom(
			stub,
			event_index,
			function( $comment )
			{
				var can_scroll = true;
				if ( new_event.is_new && sorted ) {
				// aca se insertan los evento entrantes cuando la pagina es chequeada por el proxy
					can_scroll = self.realtime_add_new_incoming_message( $comment, new_event );
				}
				else {
				// aca se insertan los mensajes de la carga inicial
					var parent_comment = new_event.message.parent_id;
					var postId = self.m_interaction.interactionData.postId;
					if ( _.isString( parent_comment ) && parent_comment !== postId) {
						var $siblings = $container.find("[data-parent_id='" + parent_comment +"']");

						if ( $siblings.length === 0 ) {
							$container.find("[data-id='" + parent_comment +"']").after( $comment );
						}
						else {
							$siblings.last().after( $comment );
						}
					}
					else {
						$container.append( $comment );
					}
				}

				$comment.addClass( "new" );
				window.setTimeout( function() {
					$comment.removeClass( "new" );
				}, TIME_SELECTED_COMMENT_MS );

				callback( $comment, can_scroll );
			}
		);
	};

	AppstoreClientWindow.prototype.setReplyToComment = function( commentId, name )
	{
		var self = this;
		this.m_reply_to_comment = commentId;
		$(".appstoreInteractionControlTitle").show().text( "<%=_T('Reply to')%>" + "<%=_T(':')%>" + " " + name );
		$("#appstoreCommentButton").val( "<%=_T('SEND REPLY')%>" );
		$("#mainContainer").addClass("commentReply");
		self.focusInput($("#postCommentText"));

		$("#cancelReplyToComment").show().unbind('click').bind('click', function() {
			$( this ).hide();
			$(".appstoreInteractionControlTitle").hide();
			$("#appstoreCommentButton").val( "<%=_T('COMMENT')%>" );
			$("#mainContainer").removeClass("commentReply");
			self.focusInput($("#postCommentText"));
			self.m_reply_to_comment = null;
		});
	};

	AppstoreClientWindow.prototype.getMessageColapsedComments = function( increment_count )
	{
		var count = this.m_interaction.getEventList().length + ( increment_count || 0 );
		return FORMAT( "<%=_T('View all {0} comments')%>", count );
	};

	AppstoreClientWindow.prototype.updateMessageColapsedComments = function()
	{
		var $message = $('.appstoreThreadSubEventSummary');
		//if the hidden markup is visible, update it with + 1 comment count
		if ( $message.is(':visible') ) {
			var message = this.getMessageColapsedComments();
			$message.find('a').text( message );
		}
	};

	AppstoreClientWindow.prototype.on_comment_post_created = function( new_event, sorted )
	{
		var self = this;

		this.hideStatusMessage();
		this.updateMessageColapsedComments();

		var incoming_realtime = sorted;
		if ( !incoming_realtime ) {
			self.clearInput($("#postCommentText"));
		}

		var $pending;
		if ( new_event.creator.toLowerCase() === "agent" && new_event.fake_event_id ) {
			$pending = $("#thread_container").find("[data-id='" + new_event.fake_event_id +"']");
		}

		if ( $pending && $pending.length > 0 ) {
		// aca actualizo en pantalla el evento pending/sending.
			this.delivery_success_message( $pending, new_event );
		}
		else {
			var callback = function( $comment, can_scroll )
			{
				if ( can_scroll ) {
					var exist_comment_in_dom = $comment && $comment.length > 0;

					var scroll_data = self.getScrollBasicInfo();
					scroll_data.callback = function() {
						if ( !exist_comment_in_dom ) {
							return;
						}
					};

					if ( exist_comment_in_dom) {
						scroll_data.destination = "item_position";
						scroll_data.type = "without_animation";
						scroll_data.$item = $comment;
					}
					else {
						scroll_data.destination = "bottom";
					}

					SCROLL_TO( scroll_data );
				}

				if ( !incoming_realtime ) {
					$("#cancelReplyToComment").trigger("click");
				}
			};

			this.add_new_message_stub( new_event,
				callback,
				sorted
			);
		}
	};

	AppstoreClientWindow.prototype.comment_current_post = function()
	{
		var self = this;
		if (!this.canExecuteInteractionAction()) {
			return;
		}
		var text = $("#postCommentText").val();
		this.focusInput($("#postCommentText"));
		if (text && text.length > 0) {
			var message = { text : text };
			this.hideStatusMessage();
			this.displayStatusMessage("<%=_T('Sending comment...')%>", "Process", true);
			var event = this.get_current_event();
			if ( !event ) {
				return;
			}
			
			//mark all as read
			if ($("#mark_all_read_unread").val() === MARK_ALL_READ_BTN_TEXT) {
				$("#mark_all_read_unread").click();
			}
			
			//reply to comment, or reply to post

			var postId = this.m_interaction.interactionData.postId;
			var commentId = this.m_reply_to_comment || this.m_interaction.interactionData.commentId || "";

			var fake_event_id = this.m_interaction.commentPost(
				undefined, // fake event id - tiene que ser pasado como parametro en undefined
				postId,
				commentId,
				message,
				function( success, message ) {
					if ( success ) {
					// Los success SE REALIZAN CUANDO LLEGA LA ACTUALIZACION DE LA INTERACTION "onUpdateInteraction->clientUpdate"
						return;
					}
					event.fake_event_id = fake_event_id;
					event.fake_event_id = fake_event_id;
					self.hideStatusMessage();
					self.displayStatusMessage( message, "Error", true, 5000 );
					self.delivery_remove( event );
				}
			);
			this.add_new_outgoing_fake_event( fake_event_id, message, commentId);
		}
	};

	AppstoreClientWindow.prototype.getInteractionRelatedMarkup = function( properties )
	{
		var typeClass = 'appstoreEventType' + (this.m_privateMessages? 'PrivateMessage' : 'Wall');
		var photoClass = properties.event.message.type ? "photoPost": "";
		var readClass = "relatedInteractionUnread";
		var message = inConcert.Utils.escapeAttrib(this.m_interaction.getMessage(properties.sorted_index));
		var message_abstract = inConcert.Utils.linkify( message.length <= 60 ? message : message.substring(0, 60) + '...', {exclude: {email: true}});
		var author_name = this.m_interaction.getUserName(properties.event);
		var genericAvatar = this.getGenericAvatar();
		var avatar = properties.avatarURI || this.m_interaction.getEventUserAvatarURI(properties.event) || genericAvatar;

		return $(''.concat(
			'<div class="relatedInteraction ', readClass, ' ', typeClass, ' ', photoClass, '">',
				'<div class="interactionAvatar">',
					'<object data="', avatar, '" type="image/png" width="48px" height="48px">',
						'<img src="', genericAvatar, '" width="48px" height="48px"/>',
					'</object>',
				'</div>',
				"<div class='interactionInfoContainer'>",
					'<label class="interactionContactName">',
						author_name,
					'</label>',
					'<label class="interactionDate">', this.m_interaction.getLocaleDateString( properties.sorted_index ), '</label>',
					'<label class="interactionAbstract">', message_abstract, '</label>',
				'</div>',
				'<a unread_event_action="true" href="javascript:void(0);" class="interactionButton">',
					'<i class="buttons-icons-chevron-circle-right-solid"></i>',
				'</a>',
			'</div>'));
	};

	AppstoreClientWindow.prototype.getAttachMarkup = function( event_index )
	{
		let self = this;
		let content = self.m_interaction.getMessageContent(event_index);
		let response = {
			disclaimer : "",
			media      : ""
		};
		
		switch(content.type) {
			case "attachment":
				if (!content.mediaFail) {
					switch(content.fileType) {
						case "video":
							response.media = FORMAT("".concat(
								"<a class='appstoreVideoLinkImage' href='{0}' target='_blank'>",
									"<div class='appstoreGalleryImageContainer'>", 
										"<div class='cropImage'>", 
											"<img class='imagePostedContainer' src='{1}'>",
										"</div>", 
									"</div>",
									"<div class='appstoreVideoLogo'></div>",
								"</a>"
							), content.link, content.previewLink);
					}
				}
				else {
					response.disclaimer = "<%=_T('Attachment not available')%>";
				}
				break;
		}
		return response;
	};
	
	AppstoreClientWindow.prototype.__makeReviewStars = function(content)
	{
		let rating = content.rating || 0;
		let ratingFloor = Math.floor(rating);
		let stars = "";
		for (let i = 0; i < 5; i++) {
			let starClassName = i < rating? "fullStar"
				: i < ratingFloor? "halfStar" : "emptyStar";
			stars += FORMAT("".concat(
				"<div class='appstoreReviewStar {0}'>",
				"</div>"
				), starClassName);
		}
		return "".concat(
		"<div class='appstoreReviewStarsContainer'>",
			stars,
		"</div>");
	};
	
	let MEDATA_FIELDS = [
		{ id : "reviewerLanguage", label : "<%=_T('Language')%>" },
		{ id : "device", label : "<%=_T('Device')%>" },
		{ id : "productName", label : "<%=_T('Product')%>" },
		{ id : "manufacturer", label : "<%=_T('Manufacturer')%>" },
		{ id : "deviceClass", label : "<%=_T('Class')%>" },
		{ id : "androidOsVersion", label : "<%=_T('Android Version')%>" },
		{ id : "nativePlatform", label : "<%=_T('Platform')%>" },
		{ id : "screenWidthPx", label : "<%=_T('Screen Width')%>" },
		{ id : "screenHeightPx", label : "<%=_T('Screen Height')%>" },
		{ id : "screenDensityDpi", label : "<%=_T('Screen Dpi')%>" },
		{ id : "glEsVersion", label : "<%=_T('GL Version')%>" },
		{ id : "cpuModel", label : "<%=_T('CPU Model')%>" },
		{ id : "cpuMake", label : "<%=_T('CPU Make')%>" },
		{ id : "ramMb", label : "<%=_T('RAM Mb')%>" }
	];
	
	AppstoreClientWindow.prototype.__makeReviewMetadata = function(content)
	{
		let self = this;
		let metadata = content.deviceMetadata || {};
		if ($.isEmptyObject(metadata)) {
			return "";
		}
		let fields = "";
		let added = {};
		$.each(MEDATA_FIELDS, function(_, field) {
			if (metadata[field.id]) {
				fields += self.__makeReviewMetadataField(field.label, metadata[field.id]);
				added[field.id] = true;
			}
		});
		$.each(metadata, function(key, value) {
			if (!added[key]) {
				fields += self.__makeReviewMetadataField(key, value);
			}
		});
		return "".concat(
			"<div class='appstoreReviewMetadata'>",
				"<label class='appstoreReviewMetadataTitle'>",
					"<%=_T('Metadata')%>",
				"</label>",
				"<button class='appstoreReviewMetadataPicker metadataHidden'>",
				"</button>",
				"<div class='appstoreReviewMetadataFields metadataHidden'>",
					fields,
				"</label>",
			"</div>"
		);
	};
	
	AppstoreClientWindow.prototype.__makeReviewMetadataField = function(label, value) {
		return "".concat(
			"<div class='appstoreReviewMetadataField'>",
				"<label class='appstoreReviewMetadataFieldLabel'>",
					label + "<%=_T(':')%>",
				"</label>",
				"<label class='appstoreReviewMetadataFieldValue'>",
					value,
				"</label>",
			"</div>"
		);
	};
	
	AppstoreClientWindow.prototype.__makeReviewVersion = function(review) {
		return "".concat(
			"<div class='appstoreReviewVersion'>",
				"<label class='appstoreReviewVersionLabel'>",
					"<%=_T('App Version')%><%=_T(':')%>",
				"</label>",
				"<label class='appstoreReviewVersionValue'>",
					review.version,
				"</label>",
			"</div>"
		);
	};
	
	AppstoreClientWindow.prototype.getReviewMarkup = function( event_index )
	{
		let self = this;
		let content = self.m_interaction.getMessageContent(event_index);
		let response = {
			disclaimer : "",
			media      : "",
			className  : ""
		};
		
		if (!content.review) {
			return response;
		}
		
		response.media = "".concat(
			"<div class='appstoreReviewContainer'>",
				self.__makeReviewStars(content.review),
				self.__makeReviewVersion(content.review),
				self.__makeReviewMetadata(content.review),
			"</div>"
		);
		response.className = "withReview";
		return response;
	};
	
	AppstoreClientWindow.prototype.bindReviewMedia = function($comment)
	{
		$comment.find(".appstoreReviewMetadataPicker").click(function() {
			$(this).toggleClass("metadataHidden");
			$(this).siblings(".appstoreReviewMetadataFields").toggleClass("metadataHidden");
		});
	};


	AppstoreClientWindow.prototype.isReadEvent = function( event, event_index )
	{
		if ( typeof event !== "object" ) {
			throw "AppstoreClientWindow.isReadEvent - event is not a object.";
		}
		if ( event.outgoing_pending ) {
			return true;
		}
		if ( !event_index ) {
			return true;
		}
		return this.m_interaction.isRead( event_index );
	};

	AppstoreClientWindow.prototype.insert_comment_in_dom = function( event, event_index, callback = $.noop )
	{
		const self = this;

		event_index = event_index || -1;
		var failed_event;

		if ( event.outgoing_pending && event.timestamp ) {
			if ( event.state === "FAILED" ) {
				failed_event = $.extend( {}, event );
			}
			event = this.make_fake_event({
				fake_event_id  : event.fake_event_id,
				message        : event.message,
				parent_comment : event.parent_id,
				creation_date  : event.creationDate,
				state          : event.state,
				agent          : event.agentAtVCC || event.agent
			});
		}
		var id = event.message.id;
		var parent_id = event.message.parent_id;

		var author_name = this.m_interaction.getUserName( event );
		var author_id   = this.m_interaction.get_user_event_id( event );
		var attach_html = this.getAttachMarkup( event_index );
		var review_html = this.getReviewMarkup( event_index );
		var message_text = inConcert.Utils.linkify( inConcert.Utils.escapeAttrib( this.m_interaction.getMessageText(event.message) ), { exclude: { email: true } } );
		var message_class = (event.message.webhookType === "REFERRAL" || event.message.webhookType === "OPTIN")? "referralMessage" : "";

		var postId = this.m_interaction.get_origin_message_id();
		var has_parent = _.isString( parent_id ) && parent_id !== postId;
		var html_reply = '';

		var class_container, attr_parent_id;
		if ( has_parent ) {
			class_container = "replyMessage";
			attr_parent_id = ' data-parent_id="' + parent_id + '" ';
		}
		else {
			class_container = "topLevelMessage";
			attr_parent_id = '';
		}

		if (event.creator === "contact") {
			class_container += " contactEvent";
		}
		else if (event.creator === "bot") {
			class_container += " botEvent";
		}
		else {
			class_container += " agentEvent";
		}
		class_container += review_html.className;

		var read_unread_lnk = this.get_html_read_unread_link( event, event_index, true );
		//var like_unlike_lnk = this.get_html_like_unlike_link( event, event_index );
		class_container += read_unread_lnk.parent_class;
		//class_container += like_unlike_lnk.parent_class;

		var attr_id =  ' data-id="' + id + '" ';
		var attr_index =  ' data-event_index="' + event_index + '" ';
		var time = ' data-time="' + event.creationDate.getTime() + '" ';

		var genericAvatar = this.getGenericAvatar();
		var avatar = this.m_interaction.getEventUserAvatarURI(event) || genericAvatar;

		var $comment = $( ''.concat(
			'<div class="appstoreThreadSubEvent ', class_container, '" ', attr_id, attr_parent_id, attr_index, time, '>',

				// label de estado
				'<div class="deliveryMessageContainer" hidden >',
					'<label class="deliveryMessageText" ></label>',
				'</div>',

				// tooltip - mensaje de por que fallo
				'<div class="deliveryFailReasonContainer" hidden>',
					'<label class="deliveryFailReasonTxt"></label>',
				'</div>',

				'<div class="subEventMainContainer">',
					'<div class="subEventAvatar">',
						'<object data="', avatar, '" type="image/png" width="24" height="24">',
							'<img src="', genericAvatar, '" width="24" height="24"/>',
						'</object>',
					'</div>',

					'<div class="subEventMessageContainer">',
						'<a href="javascript:void(0);" class="subEventAuthor" data-event_index="', event_index , '" data-event_user_id="', author_id , '">', author_name, '</a>',
						'<label message_text="true" class="subEventMessage ', message_class , '">', message_text, '</label>',
						attach_html.media,
						review_html.media,
					'</div>',

					'<label class="subEventDate">', inConcert.Utils.userFormatDate(event.creationDate), '</label>',
				'</div>',

				'<div class="deliveryFailActions" hidden>',
					'<a resend_message class="threadActionsResendFailed buttonColorBlack" href="javascript:void(0);">', "<%=_T('Resend')%>", '</a>',
					'<a discard_message class="threadActionsDiscardFailed buttonColorGray" href="javascript:void(0);">', "<%=_T('Discard')%>", '</a>',
				'</div>',
				html_reply,
				//like_unlike_lnk.html,
				read_unread_lnk.html,
				attach_html.disclaimer,
			'</div>'
		));

		self.bindReviewMedia($comment);

		if (event.creator === "contact") {
			const $message_text = $comment.find('[message_text]');
			$message_text
				.off('dblclick')
				.on('dblclick', function() {
					self.setMessageInSideBar(message_text);
				}
			);
		}

		// inserta in dom
		callback( $comment );

		if ( event.outgoing_pending ) {
			var state = event.state || "";
			switch( state ) 
			{
				case "DELIVERED":
					// NO HAGO NADA!!! ( delivery_success_message )
				break;
				case "DISCARDED":
					this.delivery_discarded_message( $comment );
				break;
				case "DISCARDING":
					this.delivery_discarding_message( $comment );
				break;
				case "FAILED":
					var short_msg = this.m_interaction.get_error_message( failed_event.error_code );
					this.delivery_failed_message( failed_event, short_msg );
				break;
				case "SENDING":
					this.delivery_sending_message( $comment );
				break;
				default:
					throw "Unrecognized pending event '" + state + "' state";
			}
		}
	};

	// Reorder events so that nested comments are placed right after the original comment
	// But maintaining the chronological order between comments in the same scope
	AppstoreClientWindow.prototype.reorder_nested_events = function ()
	{
		//TODO: Revisar esta funcion, que una linda y hermosa bosta!!
		//		ya lo era, y ahora aun mas!!! Despues que este aprobado el issue de ACK, revisar y realizar refactoring de la misma Ellio!!
		//

		var pending_events = {};
		var count_pending_events = this.m_interaction.count_total_pending_events();

		if ( count_pending_events > 0 ) {

			pending_events = inConcert.Utils.cloneObject( this.m_interaction.get_pending_events() );

			// Es necesario, agregar algunos fake atributos los eventos pending ( creationDate, outgoing_pending, parent_id )
			//
			// creationDate: a partir del timestamp, necesito crear un objeto Date, para que pueda ser ordenado el hash de pending, y para que lo pueda insertar en orden en la lista posta de eventos.
			// outgoing_pending: es necsario para diferenciar los eventos pending de los que no son pending en la lista unificada de sorted_events, para su posterior utilizacion.
			// parent_id: se necesita este atributo para que puedan ser ordenados en su propio hilo de comments.

			$.each( pending_events, function( _, pending_event )
			{
				pending_event.creationDate = new Date( parseInt( pending_event.timestamp, 10 ) * 1000 );
				pending_event.outgoing_pending = true;
				if ( pending_event.reply ) {
					pending_event.parent_id = pending_event.destination;
				}
			});

			// si tengo mas de un evento pending, mando ordenar
			if ( count_pending_events > 1 ) {
				// Ordena el hash por fecha de creacion
				pending_events = inConcert.Utils.sorted_hash( 
					// hash desordenado
					pending_events,

					// funcion que retorna el valor por el cual realizar el orden de los pendings ( fecha de creacion )
					function( pending_event ) { return pending_event.creationDate; },

					// funcion que genera el hash ordenado
					function( target_hash, pending_event ) { target_hash[ pending_event.Id ] = pending_event; return target_hash; }
				);
			}
		}

		var replies_by_top_level = {};
		var events = this.m_interaction.getEventList() || [];
		var i, e, parent_id;
		//first iterate over all events and group each one by parent
		for ( i = 0; i < events.length; i++ ) {
			e = events[i];
			parent_id = e.message.parent_id;
			if ( parent_id ) {
				replies_by_top_level[ parent_id ] = replies_by_top_level[ parent_id ] || [];
				replies_by_top_level[ parent_id ].push(e);
			}
		}

		// agrego los pending de replies a la lista de replies
		var added_pending_events = {};
		$.each( pending_events, function( pending_id, pending_event )
		{
			if ( pending_event.reply ) {
				var parent_id = pending_event.parent_id;
				replies_by_top_level[ parent_id ] = replies_by_top_level[ parent_id ] || [];
				replies_by_top_level[ parent_id ].push( pending_event );
				added_pending_events[ pending_id ] = true;
			}
		});

		// ahora reordeno la lista de replies x fecha
		$.each( replies_by_top_level, function( parent_id, list )
		{
			replies_by_top_level[ parent_id ] = inConcert.Utils.sortArray( list, "creationDate" );
		});
		// hago clean up de los pendings que ya se han agregado
		$.each( added_pending_events, function( pending_id )
		{
			delete pending_events[ pending_id ];
		});


		// esta funcion agrega los pending a nivel de post original, ordenados por fecha
		var add_sorted_pending_events = function( list, current_event, compare_fn )
		{
			var added_pending_events = {};
			$.each( pending_events, function( pending_id, pending_event )
			{
				if ( compare_fn( pending_event, current_event ) ) {
					list.push( pending_event );
					added_pending_events[ pending_id ] = true;
				}
			});

			$.each( added_pending_events, function( pending_id )
			{
				delete pending_events[ pending_id ];
			});
		};

		var sort_pending_fn = function( pending_event, current_event ) { return pending_event.creationDate <= current_event.creationDate; };
		var sorted_events = [];
		var id;
		//second, push each event in the final array
		//if the event has replies, the push each one after it
		for (i = 0; i < events.length; i++) {
			e = events[i];
			id = e.message.id;
			parent_id = e.message.parent_id;

			if ( !parent_id ) {
				add_sorted_pending_events(
					sorted_events,
					e,
					sort_pending_fn
				);
				sorted_events.push(e);
				if (replies_by_top_level[id]) {
					sorted_events = sorted_events.concat(replies_by_top_level[id]);
					delete replies_by_top_level[id];
				}
			}
		}

		// x ulitmo tengo que agregar los pendings a nivel post original que no se han agregado!!
		add_sorted_pending_events(
			sorted_events,
			undefined,
			function() { return true; }
		);

		//these comments does not have any parent
		//we put them at the last place
		for ( id in replies_by_top_level ) {
			if ( replies_by_top_level.hasOwnProperty(id) ) {
				sorted_events = sorted_events.concat(replies_by_top_level[id]);
			}
		}

		return sorted_events;
	};

	AppstoreClientWindow.prototype.canShowRelatedSections = function()
	{
		return this.m_interaction.getUnreadEventCount() > 0;
	};

	AppstoreClientWindow.prototype.closeRelated = function( original_index )
	{
		var self = this;
		var $container = $( "#thread_container" );

		var on_ready = function()
		{
			if ( self.m_currentEvent === 0 ) {
				// cuando se cierra el tab de unread al post origen lo dejo marcado como leido!!!
				var current_event = self.m_interaction.getEvent( self.m_currentEvent );
				if ( self.isReadEvent( current_event, self.m_currentEvent ) ) {
					var id = current_event.message.id;
					var $post = $container.find( "[data-id='" + id + "']" );
					if ( $post && $post.length > 0 ) {
						self.on_readed_event( true, $post, true );
					}
				}
			}
			else {
				// cuando esta listo, siempre tiene que quedar en el evento cero (el post original m_currentEvent = 0)
				// ya que el input para comentar siempre que se cierra el related es sobre el post original
				self.m_currentEvent = 0;
			}
			self.hideStatusMessage();
			if ( self.m_interaction.isPrivateMessage() ) {
				self.focusInput($("#appstoreMailText"));
			}
			else {
				self.focusInput($("#postCommentText"));
			}
		};

		if ( !this.can_show_all_comments ) {
			this.can_show_all_comments = original_index === undefined ? false : true;
		}

		AppstoreClientWindow.uber.closeRelated.call( this, original_index, function()
		{
			if ( original_index === undefined ) {
				on_ready();
				return;
			}
			var current_event = self.m_interaction.get_event_by_original_index( original_index );
			if ( current_event === false || !current_event.message || !current_event.message.id ) {
				on_ready();
				return;
			}
			var id = current_event.message.id;
			var $message = $container.find( "[data-id='" + id + "']" );
			if ( !$message || $message.length === 0 ) {
				on_ready();
				return;
			}

			var $mark_read_unread = $message.find( "[mark_as_read_unread]" );
			if ( $mark_read_unread.length > 0 ) {
				$mark_read_unread.hide();
			}

			if ( self.isEditable() ) {
				if ( !$message.hasClass( "unread" ) ) {
					$message.addClass( "unread" );
				}
			}
			if ( !$message.hasClass( "selected" ) ) {
				$message.addClass( "selected" );
			}

			var scroll_data = self.getScrollBasicInfo();
			scroll_data.destination = "item_position";
			scroll_data.$item = $message;
			scroll_data.callback = function()
			{
				window.setTimeout( function() {
					self.on_readed_event( true /* success */, $message, true /* is read */ );
				}, TIME_SELECTED_COMMENT_MS );

				on_ready();
			};
			SCROLL_TO( scroll_data );
		});
	};

	AppstoreClientWindow.prototype.on_readed_event = function( success, $comment, is_read )
	{
		if ( !success || typeof $comment !== "object" || $comment.length === 0 ) {
			return;
		}

		if ( !this.isEditable() ) {
			$comment.removeClass( "selected" );
			return;
		}

		// thread link class
		var class_mark_as_read = LNK_THREAD_MARK_AS_READ_CLASS;
		var class_mark_as_unread = LNK_THREAD_MARK_AS_UNREAD_CLASS;
		if ( $comment.hasClass( "appstoreThreadOrigin" ) ) {
		// origin link class
			class_mark_as_read = LNK_ORIGIN_MARK_AS_READ_CLASS;
			class_mark_as_unread = LNK_ORIGIN_MARK_AS_UNREAD_CLASS;
		}

		var new_state, new_caption;
		var $mark_read_unread = $comment.find("[mark_as_read_unread]");
		$mark_read_unread.prop( "disabled", false );
		var state = $mark_read_unread.data( "event_unread" );
		if ( is_read || state === "unread" ) {
			new_state = "read";
			new_caption = "<%=_T('Mark as unread')%>";
			$comment.removeClass( "unread selected" );
			$mark_read_unread.removeClass( class_mark_as_read ).addClass( class_mark_as_unread );
		}
		else {
			new_state = "unread";
			new_caption = "<%=_T('Mark as read')%>";
			$comment.addClass( "unread" );
			$mark_read_unread.removeClass( class_mark_as_unread ).addClass( class_mark_as_read );
		}
		$mark_read_unread.data( "event_unread", new_state ).text( new_caption ).show();
	};

	AppstoreClientWindow.prototype.on_liked_event = function( success, liked, $object )
	{
		if ( !success || typeof $object !== "object" || $object.length === 0 ) {
			return;
		}
		
		$object
			.toggleClass("liked", liked)
			.find("[like_unlike]")
				.toggleClass(LNK_UNLIKE_CLASS, liked)
				.toggleClass(LNK_LIKE_CLASS, !liked)
				.data("event_like", liked? "unlike" : "like")
				.html(liked? "<%=_T('Unlike')%>" : "<%=_T('Like')%>");
		
	};

	AppstoreClientWindow.prototype.get_html_read_unread_link = function( event, event_index, is_thread )
	{
		var response = {
			html         : '',
			parent_class : ''
		};

		var lnk_mark_as_read_class = LNK_THREAD_MARK_AS_READ_CLASS;
		var lnk_mark_as_unread_class = LNK_THREAD_MARK_AS_UNREAD_CLASS;
		if ( !is_thread ) {
			lnk_mark_as_read_class = LNK_ORIGIN_MARK_AS_READ_CLASS;
			lnk_mark_as_unread_class = LNK_ORIGIN_MARK_AS_UNREAD_CLASS;
		}

		if ( event.outgoing_pending === true || event.creator === 'agent' ) {
			return response;
		}

		if ( !this.isReadEvent( event, event_index ) ) {
			response.parent_class = ' unread';
			if ( this.isEditable() ) {
				response.html = FORMAT(
					''.concat(
						'<a mark_as_read_unread class="', lnk_mark_as_read_class, '" href="javascript:void(0);"',
								' data-event_index="{0}" data-event_unread="{1}">',
							"<%=_T('Mark as read')%>",
						'</a>'
					),
					event_index,  // data-event_index
					'unread'
				);
			}
		}
		else {
			if ( this.isEditable() ) {
				response.html = FORMAT(
					''.concat(
						'<a mark_as_read_unread class="', lnk_mark_as_unread_class, '" href="javascript:void(0);"',
								' data-event_index="{0}" data-event_unread="{1}">',
							"<%=_T('Mark as unread')%>",
						'</a>'
					),
					event_index,  // data-event_index
					'read'
				);
			}
		}

		return response;
	};

	AppstoreClientWindow.prototype.get_html_like_unlike_link = function( _ /*event*/, event_index )
	{
		var response = {
			html         : '',
			parent_class : ''
		};

		event_index = this.m_interaction.get_sorted_index_by_original_index(event_index);
		if (event_index === false || !this.m_postComment || event_index === 0) {
			return response;
		}

		var liked = this.m_interaction.isLiked(event_index);
		if (liked) {
			response.parent_class = " liked";
		}
		if ( this.isEditable() ) {
			response.html = FORMAT(
				''.concat(
					'<a like_unlike class="buttonColorGray ', liked? LNK_UNLIKE_CLASS : LNK_LIKE_CLASS, '" href="javascript:void(0);"',
							' data-event_index="{0}" data-event_like="{1}">',
						liked? "<%=_T('Unlike')%>" : "<%=_T('Like')%>",
					'</a>'
				),
				event_index,  // data-event_index
				liked? "unlike" : "like"
			);
		}

		return response;
	};

	AppstoreClientWindow.prototype.openCurrentEvent = function(callback = $.noop)
	{
		const self = this;
		
		$("body").addClass("alreadyReplied");

		var execute_callback = function( success ) {
			self.hideStatusMessage();
			callback( success );
		};

		if ( this.are_showing_all_messages || this.loading_all_comments ) {
			execute_callback( false );
			return;
		}

		var $container = $("#thread_container");
		$container.find(".appstoreThreadOrigin").remove();
		$container.find(".appstoreThreadSubEventSummary").remove();
		$container.find(".appstoreThreadSubEvent").remove();

		var firstEventIndex = 0;
		var current_event = this.m_interaction.getEvent( firstEventIndex );
		if ( !current_event || !current_event.message || current_event.message.pending ) {
			execute_callback( false );
			return;
		}

		var attach_html = self.getAttachMarkup( firstEventIndex );
		var review_html = self.getReviewMarkup( firstEventIndex );

		var getMessageMarkup = function() {
			var escAttrib = inConcert.Utils.escapeAttrib;
			var linkify = inConcert.Utils.linkify;
			var messageMarkup = "";
			var content = self.m_interaction.getMessageContent( firstEventIndex );
			//add label
			messageMarkup += '<label>' + linkify(escAttrib(content.text), {exclude: {email: true}}) + '</label>';
			
			return messageMarkup;
		};

		var event_id = current_event.message.id;
		var date_time = new Date( current_event.creationDate.toString() );
		var author_name = this.m_interaction.getUserName( current_event );
		var genericAvatar = this.getGenericAvatar();
		var avatar = this.m_interaction.getEventUserAvatarURI( current_event ) || genericAvatar;
		var read_unread_lnk = this.get_html_read_unread_link( current_event, firstEventIndex, false );
		//var like_unlike_lnk = this.get_html_like_unlike_link( current_event, firstEventIndex );
		var message_class = (current_event.message.webhookType === "REFERRAL" || current_event.message.webhookType === "OPTIN")? "referralMessage" : "";

		var class_container = "appstoreThreadOrigin appstoreEventType" +
			( this.m_interaction.isPrivateMessage() ? 'PrivateMessage' : 'Wall' ) +
			read_unread_lnk.parent_class;
			//+ like_unlike_lnk.parent_class;

		if (current_event.creator === "contact") {
			class_container += " contactEvent";
		}
		else if (current_event.creator === "bot") {
			class_container += " botEvent";
		}
		else {
			class_container += " agentEvent";
		}
		class_container += review_html.className;

		var $root_node = $( ''.concat(
			'<div class="', class_container, '" data-id="', event_id , '">',
				'<div class="appstoreThreadInfoContainer">',
					'<div class="appstoreThreadEventTypeIcon" style="display:none"></div>',
					'<div class="appstoreThreadOriginAvatar">',
						'<object data="', avatar, '" type="image/png">',
							'<img src="', genericAvatar, '"/>',
						'</object>',
					'</div>',
					'<div class="appstoreMainInfoContainer">',
						'<a href="javascript:void(0);" class="originAuthor">', author_name, '</a>',
						'<div class="originMessage ', message_class, '">', getMessageMarkup(), '</div>',
						//like_unlike_lnk.html,
						read_unread_lnk.html,
					'</div>',
					'<label class="originDate">', inConcert.Utils.userFormatDate(date_time), '</label>',
				'</div>',
			'</div>')
		);

		var $thread_container = $container.find( ".appstoreThreadCommentControlContainer" );
		$thread_container.before( $root_node );

		// this window
		if ( this.m_interaction.accountHasContactManagementEnabled( this.from_app ) ) {
			$('.originAuthor').unbind('click').bind('click', function() {
				let avatar_uri = inConcert.app.getBaseUrl() + self.m_interaction.getAvatarURI();
				var contact_window = new inConcert.ContactWindow({
					from_app    : self.from_app,
					interaction : self.m_interaction,
					name        : author_name,
					avatar      : avatar_uri,
					addresses   : current_event.addresses
				});
				contact_window.show();
			});
		}

		var events = this.reorder_nested_events();

		$thread_container.show();

		////
		var hiddenComments = Math.max( events.length - 1 - self.m_maxCompressedComments, 0 );
		var base = self.can_show_all_comments ? 0 : hiddenComments;

		// esta funcion carga el detalle en el header de POST/PM original.
		// la misma es invocada, desde la funcion 'append_messages' una vez que ha finalizado con la carga de todos los comentarios del POST/PM
		//
		var append_header_detail = function()
		{
			if ( !self.can_show_all_comments ) {
				if ( hiddenComments > 0 ) {
					var increment_count = -1; // aca se debe decrementar en uno ya que el post original, no se debe de contar
					var message = self.getMessageColapsedComments( increment_count );
					var $hidden_node = $(
						'<div class="appstoreThreadSubEventSummary">' +
							'<a href="javascript:void(0);">' + message + '</a>' +
						'</div>'
					);
					$root_node.after( $hidden_node );
					$hidden_node.find('a')
						.unbind('click').click(function() {
							self.can_show_all_comments = true;
							self.openCurrentEvent( function( success )
							{
								if ( !success ) {
									return;
								}
								var scroll_data = self.getScrollBasicInfo();
								scroll_data.destination = "bottom";
								SCROLL_TO( scroll_data );
								self.hideStatusMessage();
							});
						});
				}
			}

			self.are_showing_all_messages = base === 0;
			self.loading_all_comments = false;
			
			//if current message has an attachment, show disclaimer
			let $content = $(".appstoreThreadOrigin");
			if ( attach_html.disclaimer ) {
				$content.append($( attach_html.disclaimer ));
			}
			// if current message has media
			if ( attach_html.media ) {
				$content.append($( attach_html.media ));
			}
			// if current message has a review
			if ( review_html.media ) {
				$content.append($( review_html.media ));
				self.bindReviewMedia($content);
			}

			self.has_created_main_thread = true;
			self.m_currentEvent = 0;
			execute_callback( true );
		};

		// a la carga de los mensajes la agrego en un timeout, para que se pueda visualizar el mensaje de waiting.
		// ya que sino el mismo no se visualiza si tiene muchos mensajes y comienza a agregar en el DOM.

		// esta funcion muestra un mensaje informativo en el cual va inidicando la cantidad de mensajes que se van cargando.
		//
		var count_events = events.length;
		var show_loading_message = function( count_added )
		{
			var count_comments  = count_events - 1; // se le debe de restar uno, ya que dentro de los eventos tambien esta el POST original, y en el mensaje se de de informar los comments agregados
			count_added = count_added > count_comments ? count_comments : count_added;
			var message;
			if ( count_added === undefined ) {
				message = "<%=_T('Please wait, loading messages.')%>";
			}
			else {
				message = FORMAT( "<%=_T('Please wait, loading messages, {0} of {1}.')%>", count_added, count_comments );
			}
			self.displayStatusMessage( message, "Process", false );
		};

		// Esta funcion es una funcion recursiva y es la encargada de cargar los mensajes (eventos) en el markup
		// La misma se ejecuta dentro de un timeout minimo, y se va cargando de a unos 50 elementos, luego se invoca nuevamente hasta terminar la carga.
		// Se ejecuta dentro de un timeout, para que el navegador no quede colgado, en el loop y en la carga del DOM, si es que tiene muchos mensajes.
		// Tambien es ejecutada dentro del timeout, porque sino el mensaje de info de la funcion 'show_loading_message', no se visualiza en pantalla cuando comienza el loop.
		// Una vez que termina la carga de los mensajes, invoca a la funcion que carga el detalle en el POST/PM original.
		//
		var max_append_per_loop = 50;
		var pause_between_loop_ms = 10;
		var added = base + 1;

		var append_messages = function( count_added )
		{
			show_loading_message( count_added );

			// timeout que contiene la funcion que carga los mensajes.
			window.setTimeout( function() 
			{
				var added_in_loop = 0;

				for ( var i = added; i < count_events; i++ )
				{
					var current_children_event = events[i];

					self.insert_comment_in_dom(
						current_children_event,
						current_children_event.__index,
						function ($comment)
						{
							var parent_comment = current_children_event.message.parent_id;
							var postId = self.m_interaction.get_origin_message_id();
							if ( _.isString( parent_comment ) && parent_comment !== postId) {
								var $siblings = $container.find("[data-parent_id='" + parent_comment +"']");

								if ( $siblings.length === 0 ) {
									$container.find("[data-id='" + parent_comment +"']").after( $comment );
								}
								else {
									$siblings.last().after( $comment );
								}
							}
							else {
								$container.append( $comment );
							}
						}
					);

					added_in_loop++;
					added++;
					if ( added_in_loop === max_append_per_loop ) {
						break;
					}
				}

				if ( added === count_events ) {
					// FIN. muestro mensaje de que se ha completado la carga, cargo los mensajes en el DOM e invoco a la funcion que carga los datos del detalle.
					// el Append al DOM lo tengo que realizar en un timeout, porque sino el ultimo mensaje de info de que se ha cargado correctamente, no es visualizado.
					show_loading_message( added );
					window.setTimeout(function() {
						append_header_detail();
					}, pause_between_loop_ms);
				}
				else {
					// si aun quedan mensajes por cargar invoca la misma funcion recursiva
					append_messages( added );
				}

			}, pause_between_loop_ms);
		};

		// aca comienza la carga de los mensajes
		this.loading_all_comments = true;
		append_messages();
	};

	AppstoreClientWindow.prototype.update_hash_events_return_diff = function () 
	{
		var self = this;
		var new_events = [];
		if ( !this.m_interaction || this.m_interaction.isBeingComposed() || !$.isArray( this.m_interaction.events ) ) {
			return new_events;
		}
		var events = this.reorder_nested_events();
		$.each( events || [], function( i, event )
		{
			if ( !event || !event.message || !event.message.id ) {
				return true; // continue
			}
			var evt_id = event.message.id.toUpperCase();
			if ( !self.hash_events[ evt_id ] ) {
				if ( i > 0 ) {
					event.is_new = true;
				}
				new_events.push( event );
				self.hash_events[ evt_id ] = true;
			}
		});
		return new_events;
	};

	AppstoreClientWindow.prototype.append_new_events = function () 
	{
		var self = this;
		var new_events = this.update_hash_events_return_diff();
		if ( new_events.length === 0 ) {
			return;
		}

		var append = function()
		{
			$.each( new_events, function( _, new_event )
			{
				var sorted = false;
				if ( new_event.creator === "contact" ) {
					// Siempre que el nuevo evento sea generado x un contacto,
					//    lo tengo que agregar ordenado x fecha en el DOM.
					// Los eventos del agente no se ordenan, ya que los mismo ya fueron agregado en el "delivery sending" y actualizados en el "delivery success/failed"
					sorted = true;
				}
				if ( self.m_interaction.isPrivateMessage() ) {
					self.on_private_message_created( new_event, sorted );
				}
				else {
					self.on_comment_post_created( new_event, sorted );
				}
			});
		};

		if ( this.has_created_main_thread ) {
			append();
		}
		else {
			this.openCurrentEvent();
		}
	};

	AppstoreClientWindow.prototype.clientUpdate = function () 
	{
		if (!this.m_interaction.isValid()) {
			return;
		}
		if ( this.was_being_composed && !this.m_interaction.isBeingComposed() && ( this.m_interaction.events || [] ).length > 0 ) {
			this.close();
			return;
		}

		$("#postCommentText").prop("disabled", false);
		$("#appstoreComment").prop("disabled", false);
		$("#appstoreWallText").prop("disabled", false);
		$("#appstoreWallButtonPost").prop("disabled", false);

		this.append_new_events();
		this.set_mark_all_unread_read_button_caption();
	};

	AppstoreClientWindow.prototype.change_client_state = function( new_state )
	{
		new_state = AppstoreClientWindow.uber.change_client_state.call( this, new_state );

		var $solve_container = $("#appstore_solve_container");

		switch( new_state )
		{
			case "archived":
				$solve_container.hide();
			break;

			case "connected":
				$solve_container.show();
			break;

			case "monitoring":
				$solve_container.hide();
			break;

			case "pending":
				$solve_container.hide();
			break;
		}

		return new_state;
	};

	//when jquery says so, start it all up
	$(document).ready(function()
	{
		//initializates emoji
		window.emojiPicker = new EmojiPicker({
			emojiable_selector: '[data-emojiable=true]',
			assetsPath: '/inconcert/lib/emoji-picker/lib/img/',
			popupButtonClasses: 'emoji-picker-button',
			norealTime: false,
			onPaste: function(pasted) {
				/*
					Este callback se debe especificar para que el texto que se va pegar desde el portapapeles, no incluya ENTERs que no van en el texto
					cuando el origen es por ejemplo una planilla de Excel.
					Aunque el callback simplemente retorna el mismo valor que ha recibido, quien realiza el laburo es el mismo plugins "EMOJI"
					cuando obtiene la información a pegar desde portapapeles en formato 'text/plain' y luego invoca al callback.
					Archivo: "\lib\inconcert\agent\inconcert\lib\emoji-picker\lib\js\jquery.emojiarea.js"
					Ref Issue #7071
				*/
				return pasted;
			}
		});
		window.emojiPicker.discover();

		window.hook_environment();
		inConcert.i18n.updateScreen();

		var callback = function()
		{
			var client = new AppstoreClientWindow(window.interaction,
				{
					archive  : window.archive,
					from_app : window.from_app,
					queued   : window.queued,
					waiting  : window.waiting,
					monitor  : window.monitor,
					readonly : window.readonly || window.archive || window.monitor || window.queued || window.waiting
				}
			);

			window.onReady(client);

			if (window && window.view && window.view instanceof inConcert.ClientWindowProxy) {
				window.view.apply(client);
			}
			window.view = client;
		};

		$("body").addClass("loadingWindow");

		if(window.interaction.isBeingComposed()) {
			callback();
		}
		else {
			window.interaction.inflate(callback);
		}
	});
}());
