/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, -W083, esversion: 11 */

/* global $, inConcert, _*/

// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	'use strict';

	const _T = inConcert.i18n.getString;
	const format = inConcert.Utils.format;

	/**
		Class AssignedInteractionsPanel.
		Subclass of Panel used for assigned and archived interactions.
		@param id
		@param parent_panel
	*/
	function AssignedInteractionsPanel (id, parent_panel)
	{
		//the category should be defined before the base constructor in order to be used properly
		const self = this;

		this.m_parent_panel = parent_panel;
		this.m_category = 'assigned';
		this.m_live_list = {};
		this.m_live_count = 0;
		this.m_active_list = {};
		this.m_active_count = 0;
		this.m_appointment_list = {};
		this.m_appointment_count = 0;
		this.m_main = inConcert.app.getMainWindow();
		this.m_views = {
			ACTIVE     : 0,
			LIVE       : 1,
			ARCHIVE    : 2,
			APPOINTMENT: 3
		};
		this.m_currentView = this.m_views.ACTIVE; //one of active/live/archive
		this.m_timeout_sending_events_list = {};
		//tooltips
		$('#panelAssignedInteractions').inconcert_tooltip( { alignment: 'left-bottom', bind_selector: '[tooltip]' });
		this.m_has_bulk_actions = true;
		AssignedInteractionsPanel.uber.constructor.call(this, {
			id: id,
			name: "<%=_T('Assigned Conversations')%>",
			callback: {
				get_timestamp_type: function() {
					// Esta funcion determina si la vista actual es la ACTIVE, y si es asi el tiempo a mostrar en la columna es la del ultimo evento,
					//   para cualquier otra vista, se visualiza el por default de interaction gui, que es el ultimo estado.
					if (self.m_currentView === self.m_views.ACTIVE) {
						return "last_event_date"; // valor que es utilizado en shared/gui/interaction_gui.js
					}
					return; // interaction gui va a tomar el por default.
				}
			}
		});
		this.updateBatchActions();
		this.m_width = 460;
	}

	inConcert.Utils.inherit(AssignedInteractionsPanel, inConcert.InteractionPanel);

	if (typeof inConcert.AssignedInteractionsPanel !== 'function') {
		inConcert.AssignedInteractionsPanel = AssignedInteractionsPanel;
	}

	AssignedInteractionsPanel.prototype.can_reposition = function() {
		return false;
	};

	AssignedInteractionsPanel.prototype.get_prefix = function( interaction )
	{
		if ( interaction.isFinished() ) {
		// archived
			return 'archived_interaction_';
		}
		// appointment
		if (interaction.isAppointment()) {
			return 'appointment_interaction_';
		}
		// active & live
		return 'assigned_interaction_';
	};

	AssignedInteractionsPanel.prototype.remove_archive_interaction = function( interactionId ) {
		this.__getInteractionNode(interactionId, 'archived_interaction_').remove();
	};

	AssignedInteractionsPanel.prototype.allowed_bulk_actions = function(interaction)
	{
		if (interaction && (interaction.isFinished() || interaction.isAppointment())) {
			// archived and appointment
			return false;
		}
		// active & live
		return AssignedInteractionsPanel.uber.allowed_bulk_actions.call( this, interaction );
	};

	AssignedInteractionsPanel.prototype.reset = function() {
		this.closeAllClients();
		this.m_live_list = {};
		this.m_live_count = 0;
		this.m_active_list = {};
		this.m_active_count = 0;
		this.m_appointment_list = {};
		this.m_appointment_count = 0;
		this.m_windows = {};
		this.loadAllInteractions();
	};

	AssignedInteractionsPanel.prototype.refreshPanel = function() {
		if (this.m_currentView !== this.m_views.ARCHIVE) {
			this.refresh();
		}
	};

	AssignedInteractionsPanel.prototype.on_show_live = function()
	{
		this.collapsed_panel_current("interaction");
	};

	AssignedInteractionsPanel.prototype.show_live = function()
	{
		$('#interactionAssignedScrollList').unbind('scroll.interactionArchive');
		$('#archiveInteractionSearch').unbind('keyup.interactionArchive');
		$('#activeAssignedFilter,#appointmentFilter,#archivedFilter').hide();
		$('#liveAssignedFilter').show();
		$('#interaction_assigned_search_not_found_msg').hide();

		// this is done to show/hide (via css) the batch toolbar
		$('#panelAssignedInteractions')
			.removeClass('offlineContainer')
			.removeClass('appointmentContainer')
			.removeClass('archivedContainer')
			.addClass('liveContainer');

		this.endSearch(1); //turns off current search if executing
		$('#tabAssignedActiveInteractions').removeClass('boxTabNewEvent');
		this.m_currentView = this.m_views.LIVE;
		this.loadLiveInteractions();
		this.updateBatchButtonsStatus();
		this.__update_grid();
	};

	AssignedInteractionsPanel.prototype.loadLiveInteractions = function()
	{
		this.endSearch(1); // finalizo si existe una busqueda previa
		this.startingSearch();
		this.clearListContainer();
		let added_interactions = 0;
		for (let id in this.m_live_list) {
			if (this.m_live_list.hasOwnProperty(id)) {
				let node = this.m_interactions[id];
				if ( this.sortedInsertion(this.m_live_list[id], node) ) {
					added_interactions++;
				}
			}
		}
		$("#interactionQueryBuilder-btn-clean-all").prop("disabled", !inConcert.app.getMainWindow().m_panels.conversations_panel.queryBuilderObj.hasRules());

		//Pills
		let query_search = this.get_query_search();
		let $pillContainer = $('#liveInteractionSearch').find("#liveInteractionSearchPillContainer");
		let $noPill = $pillContainer.find(".inlineMessagesContainer");
		let $pillGroup = $pillContainer.find(".form-group-pill-container");
		$pillGroup.empty();
		if(query_search && query_search.rules && $.isArray(query_search.rules) && query_search.rules.length > 0)
		{
			for (let i = 0; i < query_search.rules.length; i++) {				
				// #12450: Dont add labels to Pills
				if(query_search.rules[i].id === "labels") {
					continue;
				}
				let ruleId = "live_pill_" + i;
				let rulePillText = _T(query_search.rules[i].label || query_search.rules[i].id) + " | " + _T(query_search.rules[i].operator) + " | " + "<strong>" + _T(query_search.rules[i].value) + "</strong>";
				const $lbl = "" +
					"<label id='" + ruleId + "_label' class='form-group-pill'>" +
						"<span>" + rulePillText + "</span>" +
						"<button id='" + ruleId + "_remove' class='pillButtonDelete'></button>" +
					"</label>";
				$pillGroup.append($lbl);
				$pillGroup.show();
				$noPill.hide();

				$("#" + ruleId + "_remove").click(function() {
					let ruleIdx = parseInt(this.id.replace("live_pill_", ""));
					//Eliminar del queryBuilder
					if(inConcert.app.getMainWindow().m_panels.conversations_panel.queryBuilderObj.deleteRule(ruleIdx))
					{
						//Eliminar del contenedor de pills
						$(this).parent().remove();
					}
				});
			}
		}
		else {
			$pillGroup.hide();
			$noPill.show();
		}
		this.endSearch(added_interactions);
	};

	AssignedInteractionsPanel.prototype.onNodeInserted = function(node, interaction)
	{
		const self = this;
		this.updateLabelsAndDisposition(interaction, node);
		this.updateBatchButtonsStatus();
		// bind checkbox for batch actions
		node.find('[int_batch_selector]').bind('click', function(e) {
			self.onNodeSelected(interaction, node);
			// TODO: revisar tema de bulling. Se esta ejecutando y abriendo la ventana.
			e.stopPropagation();
			$("body").trigger('click');
		});
	};

	AssignedInteractionsPanel.prototype.__update_grid = function()
	{
		// aca estoy actualizando el header de la columna "State date" / "update date" dependiendo de la vista.
		const $grid_header = $("#int_assigned_grid_header");
		const $date = $grid_header.find("[header_column_name='last_event_date_or_state_date']");
		if ($date.length > 0) {
			const colmun_caption = this.m_currentView === this.m_views.ACTIVE ? "<%=_T('Update Date')%>" : "<%=_T('State Date')%>";
			$date.find(".interactionColumnTitleText").text(colmun_caption);
		}

		const $campaign_column = $grid_header.find("[header_column_name='campaign']");
		if (this.m_currentView === this.m_views.ARCHIVE) {
			$campaign_column.show();
		}
		else {
			$campaign_column.hide();
		}

		// #18697 Change the name according to the current tab view.
		const $creation_date_column = $grid_header.find("[header_column_name='creation_date']");
		if ($creation_date_column.length > 0) {
			const colmun_caption = this.m_currentView === this.m_views.APPOINTMENT ? "<%=_T('Schedule Date')%>" : "<%=_T('Creation Date')%>";
			$creation_date_column.find(".interactionColumnTitleText").text(colmun_caption);
		}
	};

	AssignedInteractionsPanel.prototype.show_active = function()
	{
		$('#interactionAssignedScrollList').unbind('scroll.interactionArchive');
		$('#archiveInteractionSearch').unbind('keyup.interactionArchive');
		$('#liveAssignedFilter,#appointmentFilter,#archivedFilter').hide();
		$('#activeAssignedFilter').show();
		$('#interaction_assigned_search_not_found_msg').hide();

		// this is done to show/hide (via css) the batch toolbar
		$('#panelAssignedInteractions')
			.removeClass('liveContainer')
			.removeClass('appointmentContainer')
			.removeClass('archivedContainer')
			.addClass('offlineContainer');

		this.m_currentView = this.m_views.ACTIVE;
		this.loadActiveInteractions();
		this.updateBatchButtonsStatus();
		this.__update_grid();
	};

	AssignedInteractionsPanel.prototype.on_show_active = function()
	{
		this.collapsed_panel_current("interaction");
	};

	AssignedInteractionsPanel.prototype.on_show_archive = function()
	{
		this.collapsed_panel_current("interaction");
	};

	AssignedInteractionsPanel.prototype.show_appointment = function()
	{
		$('#interactionAssignedScrollList').unbind('scroll.interactionArchive');
		$('#archiveInteractionSearch').unbind('keyup.interactionArchive');
		$('#liveAssignedFilter,#activeAssignedFilter,#archivedFilter').hide();
		$('#appointmentFilter').show();
		$('#interaction_assigned_search_not_found_msg').hide();

		// this is done to show/hide (via css) the batch toolbar
		$('#panelAssignedInteractions')
			.removeClass('liveContainer')
			.removeClass('archivedContainer')
			.removeClass('offlineContainer')
			.addClass('appointmentContainer');

		this.m_currentView = this.m_views.APPOINTMENT;
		this.loadAppointmentInteractions();
		this.updateBatchButtonsStatus();
		this.__update_grid();
	};

	AssignedInteractionsPanel.prototype.on_show_appointment = function()
	{
		this.collapsed_panel_current("interaction");
	};

	AssignedInteractionsPanel.prototype.interactionPassesFilter = function(interaction)
	{
		var facets = this.get_query_search();

		if(!facets) {
			return false;
		}
		var int_camp = inConcert.Utils.getIdWithoutVCC(interaction.campaign);
		var int_type = interaction.type;
		for (let i = 0; i < facets.rules.length; i++) {
			//condition === AND: Si algun rule no se cumple, return false
			//condition === OR: Si algun rule se cumple return true
			switch(facets.rules[i].id)
			{
				case "campaign":
					switch(facets.rules[i].operator)
					{
						case "equal":
							if(facets.condition === "AND" && facets.rules[i].value !== int_camp){
								return false;								
							}
							else if(facets.condition === "OR" && facets.rules[i].value === int_camp) {
								return true;
							}
							break;

						case "not_equal":
							if(facets.condition === "AND" && facets.rules[i].value === int_camp){
								return false;						
							}
							else if(facets.condition === "OR" && facets.rules[i].value !== int_camp) {
								return true;
							}
							break;

						case "contains":
							if(facets.condition === "AND" && !int_camp.includes(facets.rules[i].value)){
								return false;								
							}
							else if(facets.condition === "OR" && int_camp.includes(facets.rules[i].value)) {
								return true;
							}
							break;

						case "not_contains":
							if(facets.condition === "AND" && int_camp.includes(facets.rules[i].value)){
								return false;								
							}
							else if(facets.condition === "OR" && !int_camp.includes(facets.rules[i].value)) {
								return true;
							}

							break;
					}
					break;

				case "type":
					switch(facets.rules[i].operator)
					{
						case "equal":
							if(facets.condition === "AND" && facets.rules[i].value !== int_type){
								return false;								
							}
							else if(facets.condition === "OR" && facets.rules[i].value === int_type) {
								return true;
							}
							break;

						case "not_equal":
							if(facets.condition === "AND" && facets.rules[i].value === int_type){
								return false;								
							}
							else if(facets.condition === "OR" && facets.rules[i].value !== int_type) {
								return true;
							}
							break;
					}

					break;

				case "labels": //operator is always 'in' for labels: If interaction has at least one of the queried labels, it passes
					if(!interaction.labels) 
					{
						return false;
					}
					let someLabelFoundOnInteraction = false;
					facets.rules[i].value.forEach(function (label) {
						if(interaction.labels[label.toUpperCase()]) {
							someLabelFoundOnInteraction = true;
						}
					});
					
					if(facets.condition === "AND" && !someLabelFoundOnInteraction){
						return false;								
					}
					else {
						return true;
					}
					break;
					
				case "has_draft":
						if (facets.rules[i].operator === "equal" && facets.condition === "AND" && ((facets.rules[i].value === "true" && !interaction.hasDraft()) || (facets.rules[i].value === "false" && interaction.hasDraft()))) {
							return false;
						}
						else if (facets.rules[i].operator === "equal" && facets.condition === "OR" && ((facets.rules[i].value === "true" && interaction.hasDraft()) || (facets.rules[i].value === "false" && !interaction.hasDraft()))) {
							return true;
						}
						break;
			
				case "has_attachments":
					if (facets.rules[i].operator === "equal" && facets.condition === "AND" && ((facets.rules[i].value === "true" && !interaction.hasAttachments()) || (facets.rules[i].value === "false" && interaction.hasAttachments()))) {
						return false;
					}
					else if (facets.rules[i].operator === "equal" && facets.condition === "OR" && ((facets.rules[i].value === "true" && interaction.hasAttachments()) || (facets.rules[i].value === "false" && !interaction.hasAttachments()))) {
						return true;
					}
					break;
				case "initiative":
						if (facets.rules[i].operator === "equal" && facets.condition === "AND" && facets.rules[i].value !== interaction.initiative) {
							return false;
						}
						else if (facets.rules[i].operator === "equal" && facets.condition === "OR" && facets.rules[i].value === interaction.initiative) {
							return true;
						}
						break;
			}
		}

		// Si es AND ha pasado todos los filtros y la interaccion puede ser visualizada. 
		// Si es OR deberia haber salido en alguno de los return previos y la interaccion no puede ser visualizada, excepto que no haya rules, en cuyo caso si pasa el filtro
		return facets.condition === "AND" || facets.rules.length === 0;
	};

	AssignedInteractionsPanel.prototype.loadActiveInteractions = function()
	{
		this.endSearch(1); // finalizo si existe una busqueda previa
		this.startingSearch();
		var $parentNode = this.m_listContainer.parent();
		this.m_listContainer.detach();
		this.clearListContainer();
		var added_interactions = 0;
		const self = this;
		try {
			for (var id in this.m_active_list) {
				if (this.m_active_list.hasOwnProperty(id)) {
					var node = this.m_interactions[id];
					if (!node) {
						$parentNode.append(this.m_listContainer);
						this.endSearch(1);
						throw 'Invalid interaction on list ' + id;
					}
					if ( this.sortedInsertion(this.m_active_list[id], node) ) {
						added_interactions++;
					}
				}
			}
		}
		catch(e) {
			$parentNode.append(this.m_listContainer);
			this.endSearch(1);
			throw e;
		}
		$parentNode.append(this.m_listContainer);

		this.m_listContainer.find("[int_labels_disposition]").each(function(_, element) {
			self.m_interaction_gui.toggleInteractionLabels($(element));
		});
		
		$("#interactionQueryBuilder-btn-clean-all").prop("disabled", !inConcert.app.getMainWindow().m_panels.conversations_panel.queryBuilderObj.hasRules());

		//Pills
		var query_search = this.get_query_search();
		var $pillContainer = $('#activeInteractionSearch').find("#activeInteractionSearchPillContainer");
		var $noPill = $pillContainer.find(".inlineMessagesContainer");
		var $pillGroup = $pillContainer.find(".form-group-pill-container");
		$pillGroup.empty();
		if(query_search && query_search.rules && $.isArray(query_search.rules) && query_search.rules.length > 0)
		{
			for (var i = 0; i < query_search.rules.length; i++) {
				// #12450: Dont add labels to Pills
				if(query_search.rules[i].id === "labels") {
					continue;
				}
				var ruleId = "active_pill_" + i;
				var rulePillText = _T(query_search.rules[i].label || query_search.rules[i].id) + " | " + _T(query_search.rules[i].operator) + " | " + "<strong>" + _T(query_search.rules[i].value) + "</strong>";
				const $lbl = "" +
					"<label id='" + ruleId + "_label' class='form-group-pill'>" +
						"<span>" + rulePillText + "</span>" +
						"<button id='" + ruleId + "_remove' class='pillButtonDelete'></button>" +
					"</label>";
				$pillGroup.append($lbl);
				$pillGroup.show();
				$noPill.hide();

				$("#" + ruleId + "_remove").click(function() {
					var ruleIdx = parseInt(this.id.replace("active_pill_", ""));
					//Eliminar del queryBuilder
					if(inConcert.app.getMainWindow().m_panels.conversations_panel.queryBuilderObj.deleteRule(ruleIdx))
					{
						//Eliminar del contenedor de pills
						$(this).parent().remove();
					}
				});
			}
		}
		else
		{
			$pillGroup.hide();
			$noPill.show();
		}		

		this.endSearch(added_interactions);
	};

	AssignedInteractionsPanel.prototype.loadAppointmentInteractions = function()
	{
		this.endSearch(1); // finalizo si existe una busqueda previa
		this.startingSearch();
		var $parentNode = this.m_listContainer.parent();
		this.m_listContainer.detach();
		this.clearListContainer();
		var added_interactions = 0;

		try {
			for (var id in this.m_appointment_list) {
				if (this.m_appointment_list.hasOwnProperty(id)) {
					var node = this.m_interactions[id];
					if (!node) {
						$parentNode.append(this.m_listContainer);
						this.endSearch(1);
						throw 'Invalid interaction on list ' + id;
					}
					if ( this.sortedInsertion(this.m_appointment_list[id], node) ) {
						added_interactions++;
					}
				}
			}
		}
		catch(e) {
			$parentNode.append(this.m_listContainer);
			this.endSearch(1);
			throw e;
		}
		$parentNode.append(this.m_listContainer);
		$("#interactionQueryBuilder-btn-clean-all").prop("disabled", !inConcert.app.getMainWindow().m_panels.conversations_panel.queryBuilderObj.hasRules());

		//Pills
		var query_search = this.get_query_search();
		var $pillContainer = $('#appointmentInteractionSearch').find("#appointmentInteractionSearchPillContainer");
		var $noPill = $pillContainer.find(".inlineMessagesContainer");
		var $pillGroup = $pillContainer.find(".form-group-pill-container");
		$pillGroup.empty();
		if(query_search && query_search.rules && $.isArray(query_search.rules) && query_search.rules.length > 0)
		{
			for (var i = 0; i < query_search.rules.length; i++) {			
				// #12450: Dont add labels to Pills
				if(query_search.rules[i].id === "labels") {
					continue;
				}
				var ruleId = "appointment_pill_" + i;
				var rulePillText = _T(query_search.rules[i].label || query_search.rules[i].id) + " | " + _T(query_search.rules[i].operator) + " | " + "<strong>" + _T(query_search.rules[i].value) + "</strong>";
				const $lbl = "" +
					"<label id='" + ruleId + "_label' class='form-group-pill'>" +
						"<span>" + rulePillText + "</span>" +
						"<button id='" + ruleId + "_remove' class='pillButtonDelete'></button>" +
					"</label>";
				$pillGroup.append($lbl);
				$pillGroup.show();
				$noPill.hide();

				$("#" + ruleId + "_remove").click(function() {
					var ruleIdx = parseInt(this.id.replace("appointment_pill_", ""));
					//Eliminar del queryBuilder
					if(inConcert.app.getMainWindow().m_panels.conversations_panel.queryBuilderObj.deleteRule(ruleIdx))
					{
						//Eliminar del contenedor de pills
						$(this).parent().remove();
					}
				});
			}
		}
		else
		{
			$pillGroup.hide();
			$noPill.show();
		}		
		this.endSearch(added_interactions);
	};

	AssignedInteractionsPanel.prototype.startingSearch = function() {
		$('#interactionPanelSearchWaiting').show();
		$('#interaction_assigned_search_not_found_msg').hide();
	};

	AssignedInteractionsPanel.prototype.endSearch = function(totalInteractions) {
		$('#interactionPanelSearchWaiting').hide();
		if (totalInteractions === 0) {
			$('#interaction_assigned_search_not_found_msg').show();
		}
	};

	AssignedInteractionsPanel.prototype.$build_interaction_node = function(interaction)
	{
		let config = {};
		config.columns = [
			{id: "attention_time", hide: function(conversation) {
					if (conversation.isFinished()) {
						return false;
					}
					return true;
				}
			}
		];
		return AssignedInteractionsPanel.uber.$build_interaction_node.call(this, interaction, config);
	};
	
	AssignedInteractionsPanel.prototype.onClientClosed = function(interaction) {
		for (const window in this.m_windows) {
			if (window === interaction.id) {
				let total = $("#mainButton" + interaction.type).find('.quickInboxNewEvents').text();
				total = parseInt(total) || 0;
				if (total === 0) {
					$("#mainButton" + interaction.type).find('.quickInboxNewEvents').text('');
				} else {
					if (total === 1  && Object.values(this.m_windows).length >= 1 && interaction.deleted) {
						total = '';
					}else {
						total = total.toString();
					}
					$("#mainButton" + interaction.type).find('.quickInboxNewEvents').text(total);
				}
				
			}
		}
		// TODO: #9778 - CANY - Docked.
		//
		return AssignedInteractionsPanel.uber.onClientClosed.call(this, interaction);
	};

	AssignedInteractionsPanel.prototype.onNewWindow = function(interaction, sessionWindow, on_window_close, from_app, on_window_open)
	{
		const self = this;
		if ( !interaction || !sessionWindow || sessionWindow.client && sessionWindow.client.permanentWindow || this.m_windows[ interaction.id ] ) {
			return;
		}
		this.m_windows[ interaction.id ] = sessionWindow;
		on_window_close = on_window_close || function() {
			// obtengo la interaccion actualizada
			const current_interaction = inConcert.app.getInteractionManager().getInteractions()[ interaction.id ] || interaction;
			self.onClientClosed( current_interaction );
		};
		sessionWindow.on("close", on_window_close);
		const on_ready_window = function() {
			// TODO: #9778 - CANY - Docked.
			self.collapsed_panel_current(from_app, interaction);
			(on_window_open || $.noop)(true, interaction);
		};
		sessionWindow.on("ready", on_ready_window);
		if (interaction.isInternal()) {
			return;
		}
	};

	AssignedInteractionsPanel.prototype.searchArchive = function()
	{
		var self = this;

		var query = this.get_query_search_archive();
		if(!query) {
			return;
		}

		this.clearListContainer();

		this.filter = {
			last : '',
			rows : 60,
			query: query,
			paginating : false,
			output: 'interactions'
		};

		this.startingSearch();
		var bindScroll = function () {
			var onScroll = function() {
				if (self.filter && self.filter.last !== '' && !self.filter.paginating) {
					var scroll = this;
					var scrollTop = scroll.scrollTop;
					var scrollHeight = scroll.scrollHeight;
					var clientHeight = scroll.clientHeight;
					var scrolledPercentage = Math.round((scrollTop / (scrollHeight - clientHeight)) * 100);
					if (scrolledPercentage > 80) {
						$('#interactionAssignedScrollList').unbind('scroll.interactionArchive');
						paginateArchiveFn();
					}
				}
			};
			$('#interactionAssignedScrollList').unbind('scroll.interactionArchive').bind('scroll.interactionArchive', onScroll);
		};

		var insert_in_list_fn = function($new_node) {
			self.m_listContainer.append($new_node);
		};

		var paginateArchiveFn = function() {
			//search using null filter, should return last retrieved contacts
			self.filter.paginating = true;
			inConcert.app.getInteractionManager().searchArchiveInteractions(self.filter, function(_, result) {
				let interactions = result.interactions;
				//if the user changed tabs before the query returned dismiss the results
				if(self.m_currentView !== self.m_views.ARCHIVE) {
					bindScroll();
					return;
				}
				if (self.filter.last === '') { //don't announce NOT FOUND when scrolling
					self.endSearch(interactions ? interactions.length : 0);
				}
				self.filter.last = result.last;
				$("#interactionQueryBuilder-btn-clean-all").prop("disabled", !inConcert.app.getMainWindow().m_panels.conversations_panel.queryBuilderObj.hasRules());
				if (!interactions || interactions.length === 0) {
					self.filter.paginating = false;
					return;
				}

				for (var i = 0; i < interactions.length; i++)
				{
					var interaction = interactions[i];
					var $current_interaction = self.$findInteraction( interaction );
					if ( $current_interaction.length > 0 ) {
						continue;
					}
					var $new_interaction = self.$build_interaction_node(interaction);
					if ( $new_interaction.length === 0 ) {
						continue;
					}
					self.updateInteractionData( interaction, $new_interaction );

					// inserts the interaction or group interactions in the list panel
					self.insertSortedInteractionInPanelList(
						interaction,
						$new_interaction,
						insert_in_list_fn
					);
					self.m_interaction_gui.toggleInteractionLabels($new_interaction.find(`[int_labels_disposition]`));
				}

				//self.updateBatchActions();

				//set_interactions_events();
				self.filter.paginating = false;
				bindScroll();
			});
		};
		bindScroll();
		$('#archiveInteractionSearch').unbind('keyup.interactionArchive');

		paginateArchiveFn();

		//Pills
		var query_search = this.get_query_search_archive();
		var $pillContainer = $('#archiveInteractionSearch').find("#archiveInteractionSearchPillContainer");
		var $noPill = $pillContainer.find(".inlineMessagesContainer");
		var $pillGroup = $pillContainer.find(".form-group-pill-container");
		$pillGroup.empty();
		if(query_search && query_search.rules && $.isArray(query_search.rules) && query_search.rules.length > 0)
		{
			for (var i = 0; i < query_search.rules.length; i++) {
				// #12450: Dont add labels to Pills
				if(query_search.rules[i].id === "labels") {
					continue;
				}				
				var ruleId = "archive_pill_" + i;
				var rulePillText = _T(query_search.rules[i].label || query_search.rules[i].id);
				if(query_search.rules[i].id === 'businessdata') {
					for(let j = 0; j < query_search.rules[i].value.length; j++){
						var businessdataName = query_search.rules[i].value[j].name;
						var businessdataOperator = query_search.rules[i].value[j].operator;
						var businessdataType = query_search.rules[i].value[j].type;
						var businessdataValue = query_search.rules[i].value[j].value;
						var businessdataValueTo = query_search.rules[i].value[j].value_to;
						rulePillText += ' | ' + '<strong>' + businessdataName + '</strong>' + ' (' + businessdataType.toLowerCase() + ') | ' + _T(businessdataOperator) + ' | <strong>' + businessdataValue;
						if(businessdataOperator === 'between') {
							rulePillText += ' , ' + businessdataValueTo;
						}
						rulePillText += '</strong>';
					}
				}
				else {
					rulePillText += ' | ' + _T(query_search.rules[i].operator) + ' | ' + '<strong>';
					rulePillText += query_search.rules[i].pill ? query_search.rules[i].pill : _T(query_search.rules[i].value);
					if(query_search.rules[i].operator === 'between') {
						rulePillText += ' , ' + query_search.rules[i].value_to;
					}
					rulePillText += '</strong>';
				}
				const $lbl = "" +
					"<label id='" + ruleId + "_label' class='form-group-pill'>" +
						"<span>" + rulePillText + "</span>" +
						"<button id='" + ruleId + "_remove' class='pillButtonDelete'></button>" +
					"</label>";
				$pillGroup.append($lbl);
				$pillGroup.show();
				$noPill.hide();
				
				$("#" + ruleId + "_remove").click(function() {
					var ruleIdx = parseInt(this.id.replace("archive_pill_", ""));
					//Eliminar del queryBuilder
					if(inConcert.app.getMainWindow().m_panels.conversations_panel.queryBuilderObj.deleteRule(ruleIdx))
					{
						//Eliminar del contenedor de pills
						$(this).parent().remove();
					}
				});
			}
		}
		else
		{
			$pillGroup.hide();
			$noPill.show();
		}		
	};

	AssignedInteractionsPanel.prototype.show_archive = function() {
		$('#activeAssignedFilter,#liveAssignedFilter,#appointmentFilter').hide();
		$('#archivedFilter').show();
		// this is done to hide (via css) the batch toolbar
		$('#panelAssignedInteractions')
			.removeClass('offlineContainer')
			.removeClass('liveContainer')
			.removeClass('appointmentContainer')
			.addClass('archivedContainer');

		this.m_currentView = this.m_views.ARCHIVE;
		this.clearListContainer();
		this.searchArchive();
		this.__update_grid();
	};

	//clear the current panel from any entry loaded
	AssignedInteractionsPanel.prototype.clearListContainer = function()
	{
		if ( !this.m_listContainer ) {
			return;
		}
		var $interactions = this.m_listContainer.children();
		$interactions.detach();
		$interactions = null;
	};

	//returns whether the interaction should be added to his panel or not
	AssignedInteractionsPanel.prototype.interactionBelongsToPanel = function(interaction) {
		if (interaction && interaction.state === "CREATED" && 
				interaction.interactionData && 
				(interaction.interactionData.internal_consultation_flag || 
					interaction.interactionData.internal_consultation_last_response)
			) 
		{
			return true;
        } else {
			if (interaction.isAbandoned() || interaction.isFinished() || interaction.isQueued()) {
				return false;
			}
			//if it's an interaction assigned to me, belongs to this panel
			var belongs = interaction.iAmRequired() || (interaction.belongsToMe() && interaction.state !== 'RESERVED');
			return belongs;
		}
	};

	AssignedInteractionsPanel.prototype.attachInteractionToGroup = function( groupId, interaction )
	{
		var $interaction = this.$findInteraction( interaction );
		if ( $interaction.length === 0 ) {
			return;
		}
		interaction.setGroupId( groupId );
		$interaction.detach();
		$interaction.attr( 'belongs_to_group', groupId );
		this.sortedInsertion( interaction, $interaction );
	};

	AssignedInteractionsPanel.prototype.openInteractionClient = function(interaction, $interaction, options)
	{
		options = options || {};
		// Si no llega la propiedad "monitor", siempre la ventana se abre como "no monitoreable", si llega (true/false) se respeta lo que viene en como parametro..
		// Todo esto es un asco, si si, ya lo se, porque todas las ventanas de todas las aplicaciones dependen de "assigned panel" y esta mal, pero
		// bueno hay que cambiarlo, y hoy no es el momento.
		options.monitor = options.monitor === undefined ? false : options.monitor;
		return AssignedInteractionsPanel.uber.openInteractionClient.call( this, interaction, $interaction, options );
	};

	AssignedInteractionsPanel.prototype.createOutgoingInteraction = function( interactionType, contact, contact_address, campaigns, parentInteraction, currentSessionClientWindow, interaction_account, options )
	{
		let fromAPI = options && options.fromAPI;
		if(options) {
			delete options.fromAPI;
		}
		var response = {success: false, interaction: false, conflicts: false};
		var im = inConcert.app.getInteractionManager();
		response.conflicts = im.conflictsToCreateOutgoingInteractions(interactionType);
		if (response.conflicts) {
			return response;
		}

		var currentAgent = inConcert.app.getCompleteId();

		var parentInteractionId, groupId;
		if ( parentInteraction ) {
			parentInteractionId = parentInteraction.id;
			groupId = parentInteraction.getGroupId();
			if ( !this.isInSomeGroupInteractions( parentInteraction ) ) {
				groupId = parentInteraction.id;
				this.attachInteractionToGroup( groupId, parentInteraction );
			}
		}

		var interaction = im.createOutgoing({
			fromAPI             : fromAPI,
			type                : interactionType    ,
			contact             : contact            ,
			currentAgent        : currentAgent       ,
			address             : contact_address    ,
			campaigns           : campaigns          ,
			parentInteractionId : parentInteractionId,
			groupId             : groupId            ,
			account             : interaction_account
		}, options);

		if (!interaction) {
			return response;
		}
		
		if(fromAPI) {
			interaction.__setInflated();
		}
		
		if (!currentSessionClientWindow) {
			// TODO: #9778 - CANY - Docked.
			// let hasOpened = this.openInteractionClient(interaction, null);
			let hasOpened = this.openInteractionClient(interaction, undefined, {from_app: "interaction"});
			// ----
			if (!hasOpened) {
				im.remove(interaction);
				return response; // false
			}
		}
		else {
			this.onNewWindow(interaction, currentSessionClientWindow);
		}

		this.addInteraction(interaction);

		response.success = true;
		response.interaction = interaction;
		response.conflicts = false;

		return response;
	};

	AssignedInteractionsPanel.prototype.toggleSearchPanel = function()
	{
		$("#interactionQueryBuilderContainer").toggle();
	};

	AssignedInteractionsPanel.prototype.set_events = function()
	{
		AssignedInteractionsPanel.uber.set_events.call( this );
		this.m_handler.find('.butFormSearch').unbind('click').click(this.toggleSearchPanel);
		this.m_handler.find('.inlineMessagesContainer').unbind('click').click(this.toggleSearchPanel);
	};

	AssignedInteractionsPanel.prototype.notify_desktop_new_interactions = function(info)
	{
		var interactions = info.new_incomming_interactions;
		var desktop_notifications = inConcert.app.getDesktopNotifications();
		desktop_notifications.notify_batch_interactions(interactions);
	};

	AssignedInteractionsPanel.prototype.notify_desktop_new_events_interactions = function(interaction, wasInflated = false)
	{
		var desktop_notifications = inConcert.app.getDesktopNotifications();
		desktop_notifications.notify_events_interaction(interaction, wasInflated);
	};

	AssignedInteractionsPanel.prototype.remove_interaction_desktop_notifications = function(interaction_id)
	{
		var desktop_notifications = inConcert.app.getDesktopNotifications();
		desktop_notifications.remove_interaction_notifications(interaction_id);
	};

	AssignedInteractionsPanel.prototype.getAppointmentCounters = function() {
		var counters = {
			active  : 0,
			inactive: 0
		};
		$.each(this.m_appointment_list, function(_, interaction) {
			if (interaction.isAppointmentActive()) {
				counters.active++;
			}
			else {
				counters.inactive++;
			}
		});
		return counters;
	};

	AssignedInteractionsPanel.prototype.update_labels_counters = function()
	{
		if ( this.m_parent_panel ) {
			this.m_parent_panel.update_counters( {
				active     : this.m_active_count,
				live       : this.m_live_count,
				appointment: this.getAppointmentCounters()
			});
		}
	};

	//called from the base class when an interaction is added to the panel
	AssignedInteractionsPanel.prototype.onInteractionAdded = function( interaction, $new_interaction )
	{
		console.debug("[AssignedInteractionsPanel.prototype.onInteractionAdded] On Added interaction: " + (interaction || {}).id + " to Assigned Panel. initiative: " +  (interaction || {}).initiative + ", state: " + (interaction || {}).state);

		AssignedInteractionsPanel.uber.onInteractionAdded.call( this, interaction, $new_interaction );
		this.increment_internal_list( interaction );

		if ((interaction.isAppointment() && this.m_currentView === this.m_views.APPOINTMENT) ||
			(interaction.isLive()  && !interaction.isAppointment() && this.m_currentView === this.m_views.LIVE) ||
			(!interaction.isLive() && !interaction.isAppointment() && this.m_currentView === this.m_views.ACTIVE)) {
			this.sortedInsertion( interaction, $new_interaction );
		}
		if (interaction.isLive() && this.m_currentView === this.m_views.ACTIVE && interaction.initiative !== 'OUTBOUND') {
			$('#tabAssignedActiveInteractions').addClass('boxTabNewEvent');
		}
		this.update_labels_counters();
		this.update_msg_rows_found();
	};

	AssignedInteractionsPanel.prototype.onInteractionUpdated = function(interaction)
	{
		if (!interaction || !interaction.id || !this.m_windows) {
			return;
		}
		console.debug("[AssignedInteractionsPanel.prototype.onInteractionUpdated] On Updated interaction: " + (interaction || {}).id + " in Assigned Panel. initiative: " +  (interaction || {}).initiative + ", state: " + (interaction || {}).state);

		//interaction left appointments window
		if (this.m_appointment_list[interaction.id] && !interaction.isAppointment()) {
			this.m_appointment_count--;
			delete this.m_appointment_list[interaction.id];
			this.increment_internal_list(interaction);
			this.apply_filter();
			this.update_labels_counters();
			return;
		}
		AssignedInteractionsPanel.uber.onInteractionUpdated.call(this, interaction, (success) => {
			this.notify_desktop_new_events_interactions(interaction, success);
		});
	};

	//triggered when an interaction is removed from the panel
	AssignedInteractionsPanel.prototype.onInteractionRemoved = function(interaction)
	{
		console.debug("[AssignedInteractionsPanel.prototype.onInteractionRemoved] On Removed interaction: " + (interaction || {}).id + " of Assigned Panel. initiative: " +  (interaction || {}).initiative + ", state: " + (interaction || {}).state);
		AssignedInteractionsPanel.uber.onInteractionRemoved.call(this, interaction);
		this.decrement_internal_list(interaction);
		this.update_labels_counters();
		this.setBatchButtonsEvents();
		this.updateBatchButtonsStatus();
		this.update_msg_rows_found();
		this.remove_interaction_desktop_notifications(interaction.id);
		// Publica el evento de remove de la interaction via agent poller. Por el momento solo lo usa el phoneclient.
		inConcert.app.messagebus.publish(`/interactions/removed/${interaction.id}`, interaction);
	};

	AssignedInteractionsPanel.prototype.update_msg_rows_found = function()
	{
		var visible = false;
		if ( this.m_currentView === this.m_views.ACTIVE ) {
			visible = this.m_active_count < 1;
		}
		else if ( this.m_currentView === this.m_views.LIVE ) {
			visible = this.m_live_count < 1;
		}
		else if ( this.m_currentView === this.m_views.APPOINTMENT ) {
			visible = this.m_appointment_count < 1;
		}
		else {
			return; // TODO: Ver que hago con las archivadas
		}

		var $not_found_msg = $('#interaction_assigned_search_not_found_msg');
		if (visible) {
			$not_found_msg.show();
		}
		else {
			$not_found_msg.hide();
		}
	};

	AssignedInteractionsPanel.prototype.clear_timeout_sending_event = function( int_id, fake_event_id )
	{
		if ( !this.m_timeout_sending_events_list[ int_id ] ||
				!this.m_timeout_sending_events_list[ int_id ][ fake_event_id ] ) {
			return;
		}

		window.clearTimeout( this.m_timeout_sending_events_list[ int_id ][ fake_event_id ] );
		this.m_timeout_sending_events_list[ int_id ][ fake_event_id ] = undefined;
	};

	// TODO: Ver si esto hay que realizarlo en el IR ( de momento queda aca ).
	//  En el IR habria que tocar la Network, para que te envie un callback al IR, y ademas habria que validar las interacciones existentes (no las archivadas)
	//  cuando el servicio IR se reinicia. ( como todo esto esta medio complicado de realizarlo en el IR, y como es exlcusivamente de feedback al cliente, lo realizamos en la app "agent" )
	//  En si esta funcion lo que realiza es validar si los eventos pending que se encuentren en el estado "sending", no han expirado.
	//  Si expiran, los marca a los mismos como failed, con el codigo de error "TIMEOUT".
	//  La funcion "clear_timeout_sending_event", tambien va de la mano con esta funcion.
	AssignedInteractionsPanel.prototype.timeout_sending_events = function( interaction, sending_count )
	{
		var self = this;
		var int_id = interaction.id;

		if ( sending_count === 0 ) {
			// realizo cleanup de todos los timeouts existentes (sending)
			$.each( this.m_timeout_sending_events_list[ int_id ] || {}, function( fake_event_id )
			{
				self.clear_timeout_sending_event( int_id, fake_event_id );
			});
			return;
		}
		else {
			// realizo cleanup de los pending events (sending) que ya forman parte de los eventos (sending)
			$.each( this.m_timeout_sending_events_list[ int_id ] || {}, function( fake_event_id )
			{
				var pending_event = interaction.get_pending_event( fake_event_id );
				if ( !pending_event || pending_event.state !== 'SENDING' ) {
					self.clear_timeout_sending_event( int_id, fake_event_id );
				}
			});
		}

		// esta funcion marca el evento pending sending, como pending failed
		var fail_event = function( interaction, fake_event_id )
		{
			self.clear_timeout_sending_event( interaction.id, fake_event_id );
			interaction.failSendingEvent( fake_event_id,
				function( success )
				{
					if ( success ) {
						return;
					}
					inConcert.logger.warn({
						message     : format( "<%=_T('Error to change the state FAIL to: {0} sending event of {1} conversation id {2}.')%>", fake_event_id, interaction.type, int_id ),
						messageType : 'Error'
					});
				}
			);
		};

		var timeout_sending_ms = 1000 * 600; // 10 minutos
		var sending_events = interaction.get_pending_events_by_state( 'SENDING' );

		var now = new Date();
		$.each( sending_events, function( fake_event_id, sending_event )
		{
		// recorro la lista de sending events
			var has_change_state = false;

			if ( sending_event.send_time ) {
				var timeout_date = new Date( ( parseInt( sending_event.send_time, 10 ) * 1000 ) + timeout_sending_ms );
				if ( timeout_date <= now ) {
				// si cuando valido en pending event, ya expiro, falgueo y asi ya invoco a la funcion para que lo marque como pending failed
					has_change_state = true;
				}
				else {
				// si aun, no expiro, lo agrego en la lista de timeouts, previamente, lo elimino al timeout existente, si es que ya existia el mismo
					self.m_timeout_sending_events_list[ int_id ] = self.m_timeout_sending_events_list[ int_id ] || {};

					self.clear_timeout_sending_event( int_id, fake_event_id );

					self.m_timeout_sending_events_list[ int_id ][ fake_event_id ] = window.setTimeout(
						function() {
							fail_event( interaction, fake_event_id );
						},
						timeout_sending_ms
					);
				}
			}
			else {
				has_change_state = true;
			}

			if ( has_change_state ) {
				fail_event( interaction, fake_event_id );
			}
		});
	};

	AssignedInteractionsPanel.prototype.updateInteractionData = function( interaction, $interaction_node )
	{
		AssignedInteractionsPanel.uber.updateInteractionData.call( this, interaction, $interaction_node );

		var $delivery = $interaction_node.find('[int_delivery]');
		var count_pending_events = interaction.count_pending_events_by_states();

		if ((count_pending_events.FAILED  === 0 || count_pending_events.FAILED  === undefined) &&
			(count_pending_events.SENDING === 0 || count_pending_events.SENDING === undefined)) {
			$delivery
				.attr('tooltip', "<%=_T('All sent')%>")
				.removeClass('deliveryFail deliverySending')
				.hide();
		}
		else {
			var classes, tooltip;
			if ( count_pending_events.FAILED > 0 ) {
				classes = 'deliveryFail';
				tooltip = format( "<%=_T('{0} Failed')%>", count_pending_events.FAILED );
			}
			if ( count_pending_events.SENDING > 0 ) {
				classes = ( classes ? classes + ' ' : '' ) + 'deliverySending';
				tooltip = ( tooltip ? tooltip + ', ' : '' ) + format( "<%=_T('{0} Sending')%>", count_pending_events.SENDING );
			}
			$delivery
				.attr('tooltip', tooltip )
				.addClass(classes)
				.show();
		}

		this.timeout_sending_events( interaction, count_pending_events.SENDING );
	};

	AssignedInteractionsPanel.prototype.onCampaignAccountUpdated = function()
	{
		for (var id in this.m_windows || {}) {
			if (this.m_windows.hasOwnProperty(id)) {
				var currentWindow = this.m_windows[id];
				var nativeWindow = currentWindow.getWindow();
				if(!nativeWindow) {
					inConcert.logger.warn({
						method:      'AssignedInteractionsPanel.onCampaignAccountUpdated',
						interaction_id:   id,
						message:     'currentWindow.getWindow() yielded nothing. It was ' + typeof(nativeWindow)
					});
					continue;
				}
				var view = nativeWindow.view;
				if (!view || typeof(view.onCampaignAccountUpdated) === 'undefined') {
					return;
				}
				if(typeof(view.onCampaignAccountUpdated) !== 'function') {
					inConcert.logger.warn({
						method:      'AssignedInteractionsPanel.onCampaignAccountUpdated',
						interaction_id:   id,
						message:     'onCampaignAccountUpdated is not a function. Is a ' + typeof(view.onCampaignAccountUpdated)
					});
					return;
				}
				view.onCampaignAccountUpdated();
			}
		}

		var phone = inConcert.app.getInteractionManager().getChannelClient('CALL');
		if (phone) {
			var phone_view = phone.getView();
			if (phone_view && typeof(phone_view.onInteractionUpdated) === 'function') {
				phone_view.onCampaignAccountUpdated();
			}
		}
	};

	AssignedInteractionsPanel.prototype.set_filter_labels = function( filtered_labels ) {
		this.m_filteredLabels = filtered_labels;
	};

	AssignedInteractionsPanel.prototype.apply_filter = function()
	{
		if ( this.m_currentView === this.m_views.ACTIVE ) {
			this.loadActiveInteractions();
			this.updateBatchButtonsStatus();
		}
		else if ( this.m_currentView === this.m_views.LIVE ) {
			this.loadLiveInteractions();
			this.updateBatchButtonsStatus();
		}
		else if ( this.m_currentView === this.m_views.APPOINTMENT ) {
			this.loadAppointmentInteractions();
			this.updateBatchButtonsStatus();
		}
		else if ( this.m_currentView === this.m_views.ARCHIVE ) {
			this.searchArchive();
		}
		else {
			throw 'AssignedInteractionsPanel.prototype.apply_filter_labels - Unrecognized view_id: ' + this.m_currentView;
		}
	};

	AssignedInteractionsPanel.prototype.onResize = $.noop;

	AssignedInteractionsPanel.prototype.switchWindowInteraction = function(oldWindowId, newInteraction) {
		if (!this.m_windows || !oldWindowId || !newInteraction) {
			return;
		}
		var newWindowId = newInteraction.id;
		var currentWindow = this.m_windows[oldWindowId];
		if (!currentWindow || !currentWindow.getWindow()) {
			return false;
		}
		this.m_windows[newWindowId] = currentWindow;
		delete this.m_windows[oldWindowId];
	};

	AssignedInteractionsPanel.prototype.increment_internal_list = function( interaction )
	{
		var it_id = interaction.id;
		if (interaction.isAppointment()) {
			this.m_appointment_list[ it_id ] = interaction;
			this.m_appointment_count++;
		}
		else if ( interaction.isLive() ) {
			this.m_live_list[ it_id ] = interaction;
			this.m_live_count++;
		}
		else {
			this.m_active_list[ it_id ] = interaction;
			this.m_active_count++;
		}
	};

	AssignedInteractionsPanel.prototype.decrement_internal_list = function( interaction )
	{
		var it_id = interaction.id;
		if ( this.m_live_list[ it_id ] ) {
			this.m_live_count--;
			delete this.m_live_list[ it_id ];
		}
		if ( this.m_active_list[ it_id ] ) {
			this.m_active_count--;
			delete this.m_active_list[ it_id ];
		}
		if ( this.m_appointment_list[ it_id ] ) {
			this.m_appointment_count--;
			delete this.m_appointment_list[ it_id ];
		}
		this.m_interaction_gui.decrement_internal_list(interaction);
	};

	AssignedInteractionsPanel.prototype.get_query_search_archive = function()
	{
		var result = inConcert.app.getMainWindow().m_panels.conversations_panel.queryBuilderObj.getFilter();

		if(result) {
			//labels
			if (this.m_parent_panel) {
				var selected_labels = this.m_parent_panel.get_selected_labels(true /*simple item*/);
				if ( $.isArray(selected_labels) && selected_labels.length > 0 ) {
					var labelsRule = {
						id : "labels",
						operator : "in",
						value : selected_labels
					};
					result.rules.push(labelsRule);
				}
			}
		}

		return result;
	};

	AssignedInteractionsPanel.prototype.get_query_search = function()
	{
		var result = inConcert.app.getMainWindow().m_panels.conversations_panel.queryBuilderObj.getFilter();

		if(result) {
			//labels
			if (this.m_parent_panel) {
				var selected_labels = this.m_parent_panel.get_selected_labels(true /*simple item*/);
				if ( $.isArray(selected_labels) && selected_labels.length > 0 ) {
					var labelsRule = {
						id : "labels",
						operator : "in",
						value : selected_labels
					};
					result.rules.push(labelsRule);
				}
			}
		}

		return result;
	};

	/**
	 * Will requeue all selected interactions on the inbox, assuming previous validation.
	 *
	 * @param  {Function} onNodesRequeued - Callback
	 */
	AssignedInteractionsPanel.prototype.requeueSelectedNodes = function(onNodesRequeued)
	{
		var self = this;
		var interactions = this.getSelectedInteractions().interactions;

		var callbacks = {
			onFinished: function(result) {
				result = self.__extractBatchMessageInformation(result, 'requeue');
				self.m_main.showBannerMessage({
					message:     result.message      ,
					messageType: result.message_type ,
					timeout:     5000
				});
			},
			onPartial : function(successfulNodes){
				if (successfulNodes && successfulNodes.length > 0){
					successfulNodes.forEach(function(interaction){
						var $node = self.__getInboxInteractionNode(interaction.id);
						if ( $node && $node.length > 0 ) {
							self.updateInteractionNode($node);
						}
					});
				}
			},
			onFailure: function(err){
				self.m_main.showBannerMessage({message : err, messageType : 'Error', timeout : 5000});
			}
		};

		var args = [];
		if ( interactions.length > 0 ){
			var interactionBatchFunction = inConcert.app.getInteractionManager().batchRequeue;
			var interactionBatchData = new self.BatchData(interactionBatchFunction, interactions, args, callbacks);
			self.processBatchInteractions(interactionBatchData, function(){
				if (typeof(onNodesRequeued) === 'function'){
					onNodesRequeued.apply(self);
				}
			});
		}
	};

	/**
	 * Will cancel all selected interactions on the inbox, assuming previous validation.
	 *
	 *
	 * @param  {Function} onNodesCanceled - Callback
	 */
	AssignedInteractionsPanel.prototype.cancelSelectedNodes = function(onNodesCanceled)
	{
		var self = this;
		var interactions = this.getSelectedInteractions().interactions;

		var args = [];
		var batchData;

		if (interactions.length > 0){
			var interactionBatchFunction = inConcert.app.getInteractionManager().batchCancel;
			var interactionCallbacks = {
				onFinished : function(){
					self.m_main.showBannerMessage({ message: "<%=_T('All conversations were canceled successfully!')%>", messageType: "Info", timeout : 4000});
				},
				onPartial : function(successfulInteractions, unsuccessfulInteractions){
					if (successfulInteractions && successfulInteractions.length > 0){
						var result = [];
						successfulInteractions.forEach(function(node){
							var stub = inConcert.app.getInteractionManager().updateInteraction(node);
							result.push(stub || {id: node.id, deleted: true});
						});
						inConcert.app.getMainWindow().updateInteractions(result);
					}
					if (unsuccessfulInteractions && unsuccessfulInteractions.length > 0){
						var errorMessage = format("<%=_T('{0} conversations failed to requeue')%>", unsuccessfulInteractions.length);
						self.m_main.showBannerMessage({	message : errorMessage, messageType : 'Error', timeout : 5000});
					}
				},
				onFailure: function(errorMessage){
					self.m_main.showBannerMessage({	message : errorMessage, messageType : 'Error', timeout : 5000});
				}
			};

			batchData = new self.BatchData(interactionBatchFunction, interactions, args, interactionCallbacks);
			self.processBatchInteractions(batchData, function(){
				if (typeof(onNodesCanceled) === 'function'){
					onNodesCanceled.apply(self);
				}
			});
		}
	};

	/**
	 *
	 */
	 AssignedInteractionsPanel.prototype.$getChkSelectAllConversations = function()
	{
		return $("#int_assigned_grid_header").find("[selector='select-all-conversations']");
	};
}());
