/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double */

/* global $, inConcert, console */

// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.

if (!this.inConcert) {
    this.inConcert = {};
}


(function () {

	"use strict";

	var _T = inConcert.i18n.getString;

	/**
	Class AvatarWindow.
	*/
	function AvatarWindow ()
	{
		AvatarWindow.uber.constructor.call(this, {});

		var self = this;
		document.title = "<%=_T('Changing Avatar')%>";

		var avatarUri = inConcert.app.getBaseUrl() + "/users/get_avatar/" + inConcert.app.getLoggedUserId() +"?uid=" + new Date().getTime();

		$("#imgCropPreview").attr("src", avatarUri);

		var cropCoords = null;
		var jcrop_api = null;
		var avatarId = null;

		var show_error = function( err ) {
			var message;
			if ( err === 403 ) { // forbidden
				message = "<%=_T('Unable to change avatar. You not have permission.')%>";
			}
			else {
				message = "<%=_T('Unable to change avatar.')%>";
			}
			inConcert.app.m_mainWindow.showBannerMessage({
				message : message,
				timeout : 5000
			});
		};

		function DestroyJCrop() {
			if (jcrop_api) {
				jcrop_api.destroy();
				jcrop_api = null;
			}
			$("#cropPhotoOk").unbind("click");
			$("#cropPhotoCancel").unbind("click");
			self.close();
		}

		var onError = function ( err ) {
			show_error( err );
			$("#imgUploadWaitAnim").hide();
			$(".changeAvatarProgressTotal").hide();
			DestroyJCrop();
		};

		function showPreview(coords){
			cropCoords = coords;
			if (parseInt(coords.w, 10) > 0)
			{
				var rx = 64 / coords.w;
				var ry = 64 / coords.h;

				if (jcrop_api) {
					var bounds = jcrop_api.getBounds();
					jQuery("#imgCropPreview").css({
						width: Math.round(rx * bounds[0]) + "px",
						height: Math.round(ry * bounds[1]) + "px",
						marginLeft: "-" + Math.round(rx * coords.x) + "px",
						marginTop: "-" + Math.round(ry * coords.y) + "px"
					});
				}
			}
		}

		var $files_picker_ctrl = $( "#fileSelector");
		$files_picker_ctrl.parent().click(function() {
			var $file_picker_ctrl = $( this ).find( "input[type='file']" );
			$file_picker_ctrl[0].click();
		});

		$("#fileSelector").change(function(event) {
			var file = event.target.files[0];
			if (!file) {
				return;
			}
			var xhr = new XMLHttpRequest();
			xhr.open("POST", "/inconcert/apps/agent/users/upload_avatar/", true);
			xhr.onload = function(e) {
				if (e.target.status === 200) {
					onUploadFinished(JSON.parse(e.target.responseText));
				}
				else {
					onError( e.target.status );
				}
			};

			xhr.upload.onprogress = function(e) {
				console.log(e);
				var percentage = "";
				if ( e && !isNaN( e.loaded ) && !isNaN( e.total ) ) {
					percentage = Math.floor( e.loaded / e.total * 100 ) + "%";
				}
				$("#imgUploadWaitAnim").show();
				$(".changeAvatarProgressTotal").show();
				$(".changeAvatarProgressBar").show().width ( percentage );
			};

			xhr.send(file);
		});

		$("#cropPhotoOk").click (function () {
			var hasPermission = inConcert.permissions.to_action( "settings", "changeavatar" );
			if ( !hasPermission ) {
				show_error( 403 ); // forbidden
				return;
			}
			if (!cropCoords) {
				return;
			}

			//En caso de Ok guardo las coordenadas seleccionadas
			var requests = {
				x: cropCoords.x,
				y: cropCoords.y,
				w: cropCoords.w,
				h: cropCoords.h,
				id: avatarId
			};

			var url = inConcert.app.getBaseUrl() + "/users/crop_avatar/";
			inConcert.invoke({
				url: url,
				type: "POST",
				data: JSON.stringify(requests),
				dataType: "json",
				contentType: "application/json",
				success: function( /*response*/ ) {
					inConcert.app.getMainWindow().refreshAvatar();
					DestroyJCrop();
				},
				error: function( response /*, error*/ ) {
					show_error( response.status );
				}
			});
		});

		$("#cropPhotoCancel").click (function (){
			DestroyJCrop();
		});

		var onUploadFinished = function(result) {
			$("#imgUploadWaitAnim").hide();
			$(".changeAvatarProgressTotal").hide();

			if (result && result.result === "OK") {
				avatarId = result.avatarId;
				var imageUrl = inConcert.app.getBaseUrl() + "/users/get_uploaded_avatar/" + result.avatarId;

				$("#show_hide_crop_image").show();
				$("#imgCropPreview").attr("src", imageUrl);
				$("#imgUploadedPhoto").attr("src", imageUrl).bind("load", function () {
					$("#imgUploadedPhoto").unbind("load");
					$("#divUploadedPhoto").show();

					if (jcrop_api) {
						//destroy previous version
						jcrop_api.destroy();
					}
					jcrop_api = $.Jcrop("#imgUploadedPhoto", {
						onSelect: showPreview,
						onChange: showPreview,
						aspectRatio: 1,
						minSize: [32,32],
						trueSize: [result.imageSize.w, result.imageSize.h],
						allowSelect: false
					});

					var region = [result.imageSize.w / 4, result.imageSize.h / 4, 3 * result.imageSize.w / 4, 3 * result.imageSize.h / 4];
					if (result.region) {
						region = [result.region.x, result.region.y, result.region.x2, result.region.y2];
					}
					jcrop_api.animateTo(region);
				});
			}
			else if (result) {
				//TODO: Solo por ahora, no alarmarse

				inConcert.app.m_mainWindow.showBannerMessage({
					messageType : "Error",
					message     : "Error: " + result.reason,
					timeout     : 10000
				});

			}
		};
	}

	//derive the main window class according to execution context
	inConcert.Utils.inherit(AvatarWindow, inConcert.Dialog);

	//when jquery says so, start it all up
	$(document).ready(function() {
		inConcert.i18n.updateScreen();
		window.hook_environment();
		var avatarWindow = new AvatarWindow();
		window.onReady(avatarWindow);
	});
}());
