/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:false, browser:true, indent:false, maxerr:50, quotmark:false, maxlen:200, esversion: 11 */

/* global $, inConcert*/

// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.

if (!this.inConcert) {
	this.inConcert = {};
}

(function () 
{
	/**
	Class Panel.
	Defines a generic panel with its behaviour (open, close, moving, docking, etc)
	*/
	'use strict';
	function Panel (id, name)
	{
		this.m_id = id;
		this.m_name = name || id;
		this.m_visible = true;
		this.m_dockToLeft = true;
		this.m_windows = {};
		this.m_main = inConcert.app.getMainWindow();
		this.m_width = 256; //this is initialized as default but should be redefined in the derived classes
		this.m_handler = id ? jQuery('#' + id) : null;
		this.set_events();
	}

	if (typeof inConcert.Panel !== 'function') {
		inConcert.Panel = Panel;
	}

	Panel.prototype.enter = $.noop;

	Panel.prototype.destroyDOMPanel = $.noop;

	Panel.prototype.set_events = function()
	{
		var self = this;
		if ( this.m_handler ) {
			// TODO: Revisar: esto se encuentra en un timeout, para que termine de iniciarse la clase y sus clases hijas
			window.setTimeout(function() { self.remove_tabs(); }, 200 );
		}
	};

	Panel.prototype.can_reposition = function() {
		return true;
	};

	Panel.prototype.has_permission_to_show_panel = function() {
		if ( this.permission_id ) {
			if ( !inConcert.permissions.to_action_on_any_campaign( 'interaction', this.permission_id ) ) {
				return false;
			}
		}
		if ( !this.has_permission_at_least_one_tab() ) {
			return false;
		}
		return true;
	};

	Panel.prototype.has_permission_to_tab = function( tab_id ) {
		if ( !this.permissions_tabs ) {
			return true;
		}
		var tab = this.permissions_tabs[ tab_id ];
		if ( !tab ) {
			return true;
		}
		if ( !inConcert.permissions.to_action_on_any_campaign( 'interaction', tab.permission_id ) ) {
			return false;
		}
		return true;
	};

	Panel.prototype.remove_tabs = function() 
	{
		var self = this;
		var selected_weight = 99;
		var $selected_tab;
		var total_tabs = 0;
		var total_removed_tabs = 0;
		// En esta lista me quedo con la cantidad de tabs que utilizan cada container.
		// Es el caso de los tabs de team y contacts, que utilizan el mismo div contenedor para mostrar la lista.
		var containers = {};

		$.each( this.permissions_tabs || {}, function( tab_id, tab_config )
		{
			total_tabs++;
			containers[tab_config.id_container_dom] = (containers[tab_config.id_container_dom] || 0) + 1;
			var $tab = $( '#' + tab_config.id_tab_dom || '' );
			if ( !self.has_permission_to_tab( tab_id ) ) {
				$tab.remove();
				total_removed_tabs++;
				containers[tab_config.id_container_dom]--;
			}
			else {
				if ( tab_config.selected_weight < selected_weight ) {
					$selected_tab = $tab;
					selected_weight = tab_config.selected_weight;
				}
			}
		});
		$.each(containers, function(id, used_by_tabs) {
			// si ningun tab lo utiliza, se elimina del DOM el contenedor
			if (used_by_tabs === 0) {
				var $tab_panel = $( '#' + id );
				$tab_panel.remove();
			}
		});
		if (total_tabs > 0 && total_tabs === total_removed_tabs) {
			// si no existe ningun tab, en el panel se destruye el mismo.
			this.destroyDOMPanel();
			return;
		}
		if ( $selected_tab ) {
			$selected_tab.trigger( 'click' );
		}
	};

	Panel.prototype.has_permission_at_least_one_tab = function() {
		var self = this;
		var has_permission = true;
		var count_hidden = 0;
		var count_tabs = 0;

		$.each( this.permissions_tabs || {}, function( tab_id )
		{
			count_tabs++;
			if ( !self.has_permission_to_tab( tab_id ) ) {
				count_hidden++;
			}
		});

		if ( count_tabs > 0 && ( count_tabs === count_hidden ) ) {
			has_permission = false;
		}

		return has_permission;
	};

	//closes the panel, THIS is the button
	Panel.prototype.onClose = function() {
		if (this.m_main.totalOpenedPanels() > 1) {
			var id = $(this).attr('panelId');
			var self = this.m_main.findPanel(id);
			self.m_visible = false;
			self.m_handler.hide();
			this.m_main.repositionPanels();
		}
	};

	//returns whether the panel should be docked to the left or not (ie right)
	Panel.prototype.docksToLeft = function() {
		return this.m_dockToLeft;
	};

	//returns panel id
	Panel.prototype.getId = function() {
		return this.m_id;
	};

	//returns panel name
	Panel.prototype.getName = function() {
		return this.m_name;
	};

	//shows the panel
	Panel.prototype.show = function() {
		if ( this.has_permission_to_show_panel() ) {
			this.m_visible = true;
			this.m_handler.show();
			return true;
		}
		this.hide();
		return false;
	};

	Panel.prototype.onResize = function() {

	};

	//hides the panel
	Panel.prototype.hide = function() {
		this.m_visible = false;
		this.m_handler.hide();
	};

	//indicates if the current panel is visible or not
	Panel.prototype.isVisible = function() {
		return this.m_visible;
	};

	//toggles visibility status of the panel
	Panel.prototype.setVisible = function(flag) {
		this.m_visible = flag;
	};

	//toggles visibility status of the panel
	Panel.prototype.toggleVisibility = function() {
		this.m_visible = !this.m_visible;
	};

	//indicates the possition offseted from the left border
	Panel.prototype.setLeftPosition = function(value) {
		this.m_handler.css('left', value);
	};

	//indicates the possition offseted from the right border
	Panel.prototype.setRightPosition = function(value) {
		this.m_handler.css('right', value);
	};

	//indicates if the current panel is visible or not
	Panel.prototype.getWidth = function() {
		return this.m_width;
	};

	Panel.prototype.onClientClosed = function(interaction) {
		if (!interaction) {
			return;
		}
		console.info("[Panel.prototype.onClientClosed] --> interaction-> this.m_windows['" + interaction.id + "'] = " + this.m_windows[interaction.id]);
		let is_closed_window = true;
		interaction.opened = false;
		if (interaction.type === "CALL") {
			if ($.inArray(interaction.state, ["PENDING", "CANCELED", "CREATED", "ABANDONED"]) >= 0) {
				// Si es una interaccion de telefono de tipo PENDING (Saliente que se esta por crear) o CANCELED publico
				//  el evento para que se escuche en "phone_client.js" (PhoneClient.prototype.switchTo)
				//  y proceda con los pasos a seguir (ejemplo, si se ha cancelado por una entrante)
				console.info("[Panel.prototype.onClientClosed] 1 - Publish event - /live_phone_client_window/interaction/" + interaction.id + ", state: " + interaction.state);
				inConcert.app.messagebus.publish_async("/live_phone_client_window/interaction/" + interaction.id + "/windows_closed/", interaction);
			}
			else {
				console.info("[Panel.prototype.onClientClosed] 2 - NO Publish event - for interaction: " + interaction.id + ", state: " + interaction.state);
			}
		}
		if (typeof(this.m_windows[interaction.id]) !== 'undefined') {
			delete this.m_windows[interaction.id];
			console.info("[Panel.prototype.onClientClosed] --> Deleted this.m_windows['" + interaction.id + "'] ");
		}
		// TODO: #9778 - CANY - Docked.
		this.collapsed_panel_current('interaction', interaction, is_closed_window);
	};

	Panel.prototype.isOpenedClientWindow = function( interaction_id )
	{
		if ( !interaction_id || !this.m_windows ) {
			return false;
		}
		return this.m_windows.hasOwnProperty( interaction_id );
	};

	Panel.prototype.getClientWindow = function( interaction_id )
	{
		if ( !this.isOpenedClientWindow( interaction_id ) ) {
			return;
		}
		var current_window = this.m_windows[ interaction_id ];
		if ( !current_window ) {
			return;
		}
		return current_window.getWindow().view;
	};

	/*
		Trigered when a local change is made to an interaction, it could be an event read, a change on campaign or contact selection, etc.
		If the change made affects the avatar selection process refreshAvatar should be sent in true
	*/
	Panel.prototype.onLocalInteractionUpdate = function(/*interaction, refreshAvatar*/) {
		//Dummy method
	};

	// TODO: #9778 - CANY - Docked.
	Panel.prototype.getClientWindowIFrameContainerId = function(interaction, options)
	{
		if (!inConcert.app.is_enable_docked_windows_interactions()) {
			// NORMAL WINDOWS
			return "iFramesWindowsDailogsContainerForGenericsApps";
		}
		// DOCKED WINDOWS
		if (interaction.isInternal()) {
			return null;
		}
		if (options.id_iframe_windows_dialog_container) {
			return options.id_iframe_windows_dialog_container;
		}
		const defaultAppInteractionContainerId = 'iFramesWindowsDailogsContainerForInteractionApp';
		if (options.from_app !== "interaction") {
			return defaultAppInteractionContainerId;
		}
	//	const panelCurrent = this.get_selected_folder_id();
		const current_panel = options.panel_name;
		if (current_panel === "global_search") {
			return "iFramesWindowsDailogsContainerForInteractionAppGlobalSearchPanel";
		}
		if (interaction.isAssigned() || interaction.isWrappedUp() || interaction.isBeingComposed()) {
			if (interaction.isLive()) {
				return "iFramesWindowsDailogsContainerForInteractionAppLivePanel";
			}
			return "iFramesWindowsDailogsContainerForInteractionAppActivePanel";
		}
		else if (interaction.isFinished()) {
			return "iFramesWindowsDailogsContainerForInteractionAppArchivedPanel";
		}
		else if (interaction.isQueued()) {
			return "iFramesWindowsDailogsContainerForInteractionAppQueuedPanel";
		}
		else if (interaction.isWaiting()) {
			return "iFramesWindowsDailogsContainerForInteractionAppWaitingPanel";
		}
		else if (interaction.isAbandoned()) {
			return "iFramesWindowsDailogsContainerForInteractionAppAbandonedPanel";
		}
		else if (interaction.isAppointment()) {
			return "iFramesWindowsDailogsContainerForInteractionAppCitesPanel";
		}
		return defaultAppInteractionContainerId;
	};

	/**
	Opens the interaction client
	Returns true if the client could be opened successfully (or it was already opened) and false otherwise.
	*/
	Panel.prototype.openInteractionClient = function(interaction, _, options)
	{
		const self = this;
		const currentView = interaction.getView();
		interaction.opened = true;
		if (currentView) {
			console.log("[Panel.prototype.openInteractionClient] --> 1- ACTIVATE currentView.activate() - interactionId: " + interaction.id);
			currentView.activate();
			return true;
		}
		if (this.isOpenedClientWindow(interaction.id)) {
			console.log("[Panel.prototype.openInteractionClient] --> 2- NO OPEN this.isOpenedClientWindow() - interactionId: " + interaction.id);
			return true;
		}

		console.log("[Panel.prototype.openInteractionClient] --> 3- OPEN -Window interaction: " + interaction.id);

		options.from_app = options.from_app || 'interaction';
		let idIFrameContainer = this.getClientWindowIFrameContainerId(interaction, options);

		const openClientView = function(page, windowOptions) {
			windowOptions = $.extend({
				windowID   : interaction.id,
				interaction: interaction,
				page       : page,
				taskbar    : options && options.owner || self,
				from_app   : 'interaction',
				id_iframe_windows_dialog_container: idIFrameContainer,
				size       : {
					// Estas dimensiones deben de coincidir con lo que se ha definido en "client_window.js" ej: "DEFAULT_WINDOW_WIDTH_PX"
					width : 950,
					height: 600
				}
			}, options, windowOptions);
			console.log("[Panel.prototype.openInteractionClient] --> 4- [openClientView] interactionId: " + interaction.id + ", inConcert.app.getWindowManager().createWindow(..)");
			return inConcert.app.getWindowManager().createWindow(windowOptions);
		};

		const onNewWindow = function(session_window, callback) {
			console.log("[Panel.prototype.openInteractionClient] --> 5 [onNewWindow] interactionId: "+ interaction.id +
				", state: " + interaction.state +
				", session_window: " + session_window + 
				", callback: " + callback
			);
			inConcert.logger.guiAction({action_name : 'open_' + interaction.type.toLowerCase() + '_client', window_id: window.id}, interaction);
			callback = callback || $.noop; // a este callback no lo agrego como valor defecto en el parametro de la funcion ya que necesito saber si la funcion se ha enviado como parametro en el log.
			if (!session_window) {
				callback(false);
				return false;
			}
			const on_window_close = options ? options.on_window_close : undefined;
			const on_window_open = options ? options.on_window_open : undefined;
			self.onNewWindow(interaction, session_window, on_window_close, options.from_app, on_window_open);
			callback(true);
			return true;
		};

		this.m_main.updateUnreadCounters();
		let sessionWindow;
		switch (interaction.type) {
			case 'CALL':
			case 'VOICEMAIL':
				/*
				*/
				const returnOpenedFalse = ()=> {
					interaction.opened = false;
					return true;
				};

				const hasAllegroBarAgent = inConcert.app.has_allegro_baragent();
				if (hasAllegroBarAgent) {
					// interaccion de CALL de Allegro, con un telefono asignado al agente.
					if (inConcert.BaragentPolling.isRedialInteraction(interaction))
						return returnOpenedFalse();
					if ($.inArray(interaction.state, ["WAITING", "CANCELED", "ABANDONED", "OPEN"]) >= 0)
						return returnOpenedFalse();
					if (interaction.state === "LOCKED" && options.monitor)
						return returnOpenedFalse();
					if (interaction.isQueued() && (interaction.type !== "VOICEMAIL" || options.from_app !== "supervisor"))
						return returnOpenedFalse();
				}
				else {
					// vcc allegro o nativo.
					if ($.inArray(interaction.state, ["ABANDONED"]) >= 0)
						return returnOpenedFalse();
				}
				if (options.from_app !== "supervisor" && interaction.belongsToMe() &&
					!interaction.isFinished() && !interaction.isAbandoned() && 
					!(interaction.type === "VOICEMAIL" && !hasAllegroBarAgent))
				{
					/*
						Como ahora el telefono NO es un singleton, se obtiene la session en un callback,
						ya que la clase "PhoneClient" necesita que el telefono se encuentre ready para poder inicializar correctamente
						el telefono en las salientes.
					*/
					const phone = inConcert.app.getInteractionManager().getChannelClient('CALL');
					if (phone) {
						phone.openLiveTelephoneWindow(interaction, function(sessionTelephoneWindow, callback) {
							onNewWindow(sessionTelephoneWindow, callback);
						});
						return true;
					}
				}
				else {
					sessionWindow = openClientView('telephonyClientWindow', {
						title   : "Web Phone Archive",
						maximizable: true,
						resizable  : true,
						size: {
							width : 950,
							height: 600
						}
					});
				}
				break;
			case 'INTERNAL_CHAT':
				sessionWindow = openClientView('internalchatClientWindow');
				inConcert.app.m_mainWindow.internalIMWindow = sessionWindow.window;
				break;
			case 'CHAT':
				// Estas dimensiones deben de coincidir con lo que se ha definido en "client_window.js" y 
				//"chat_client_window.js" ej: "DEFAULT_WINDOW_WIDTH_PX"
			 	sessionWindow = openClientView('webchatClientWindow');
			 	break;
			case 'IM':
				sessionWindow = openClientView('imClientWindow');
				break;
			case 'SMS':
				sessionWindow = openClientView('smsClientWindow');
				break;
			case 'WHATSAPP':
				sessionWindow = openClientView('whatsappClientWindow');
				break;
			case 'MAIL':
				// Estas dimensiones deben de coincidir con lo que se ha definido en "mail_client_window.js" ej: "DEFAULT_WINDOW_WIDTH_PX"
				sessionWindow = openClientView('mailClientWindow', {
					size: {
						width : 1100,
						height: 700
					}
				});
				break;
			case 'FACEBOOK':
				sessionWindow = openClientView('facebookClientWindow');
				break;
			case 'INSTAGRAM':
				sessionWindow = openClientView('instagramClientWindow');
				break;
			case 'YOUTUBE':
				sessionWindow = openClientView('youtubeClientWindow');
				break;
			case 'APPSTORE':
				sessionWindow = openClientView('appstoreClientWindow');
				break;
			case 'GOOGLEPLAY':
				sessionWindow = openClientView('googleplayClientWindow');
				break;
			case 'RCS':
				sessionWindow = openClientView('rcsClientWindow');
				break;
			case 'TWITTER':
				sessionWindow = openClientView('twitterClientWindow');
				break;
			case 'WEBCONTACT':
				sessionWindow = openClientView('webcontactClientWindow');
				break;
			default:
				throw ("Panel.openInteractionClient - proxy type unrecognized " + ( interaction.type || "" ) );
		}

		return onNewWindow(sessionWindow);
	};

	Panel.prototype.onNewWindow = function(interaction, sessionWindow, on_window_close, from_app, on_window_open)
	{
		const self = this;
		if ( !interaction || !sessionWindow || sessionWindow.client && sessionWindow.client.permanentWindow || this.m_windows[ interaction.id ] ) {
			return;
		}
		this.m_windows[ interaction.id ] = sessionWindow;
		on_window_close = on_window_close || function() {
			// obtengo la interaccion actualizada
			const current_interaction = inConcert.app.getInteractionManager().getInteractions()[ interaction.id ] || interaction;
			self.onClientClosed( current_interaction );
		};
		sessionWindow.on("close", on_window_close);
		// TODO: #9778 - CANY - Docked.
		const on_ready_window = function() {
			self.collapsed_panel_current(from_app, interaction);
			(on_window_open || $.noop)(true, interaction);
		};
		sessionWindow.on("ready", on_ready_window);
	};

	// TODO: #9778 - CANY - Docked.
	Panel.prototype.collapsed_go_back = function()
	{
	};

	Panel.prototype.get_selected_folder_id = function()
	{
		return inConcert.app.getMainWindow().get_selected_folder_id();
	};

	// TODO: #9778 - CANY - Docked.
	Panel.prototype.must_move_to_live_folder = function(interaction, is_closed_window = false)
	{
		if (!inConcert.app.is_enable_docked_windows_interactions()) {
			// NORMAL WINDOWS
			//console.log("[Panel.prototype.must_move_to_live_folder] 0.5 -NO MOVE - interaction.id: " + interaction.id);
			return false;
		}
		// DOCKED WINDOWS
		if (!interaction) {
			//console.log("[Panel.prototype.must_move_to_live_folder] 1 -NO MOVE - interaction.id: " + interaction.id);
			return false;
		}
		if (is_closed_window) {
			//console.log("[Panel.prototype.must_move_to_live_folder] 2 -NO MOVE - interaction.id: " + interaction.id);
			return false;
		}
		if (interaction.isInternal()) {
			//console.log("[Panel.prototype.must_move_to_live_folder] 3 -NO MOVE - interaction.id: " + interaction.id);
			return false;
		}
		if (!interaction.belongsToMe()) {
			//console.log("[Panel.prototype.must_move_to_live_folder] 4 -NO MOVE - interaction.id: " + interaction.id);
			return false;
		}
		if (!interaction.isLive()) {
			//console.log("[Panel.prototype.must_move_to_live_folder] 5 -NO MOVE - interaction.id: " + interaction.id);
			return false;
		}
		if (interaction.isLocked()) {
			//console.log("[Panel.prototype.must_move_to_live_folder] 6 -SIIIII MOVE - interaction.id: " + interaction.id);
			return true;
		}
		if (interaction.isAssigned()) {
			//console.log("[Panel.prototype.must_move_to_live_folder] 7 -SIIII MOVE - interaction.id: " + interaction.id);
			return true;
		}
		//console.log("[Panel.prototype.must_move_to_live_folder] 8 -NO MOVE - interaction.id: " + interaction.id);
		return false;
	};

	Panel.prototype.select_live_folder = function()
	{
		const mw = inConcert.app.getMainWindow();
		mw.jumpToApp("interaction");
		inConcert.app.getMainWindow().select_live_folder();
	};

	// TODO: #9778 - CANY - Docked.
	Panel.prototype.get_total_interactions_windows_by_panels = function()
	{
		if (!inConcert.app.is_enable_docked_windows_interactions()) {
			// NORMAL WINDOWS
			return false;
		}
		// DOCKED WINDOWS
		let windows_by_panels = {
			active: {total: 0},
			live  : {total: 0},
			archive: {total: 0},
			appointment: {total: 0},
			spam: {total: 0},
			queued: {total: 0},
			waiting: {total: 0},
			abandoned: {total: 0},
			global_search: {total: 0}
		};
		for (let window_id in this.m_windows || {}) {
			if (this.m_windows.hasOwnProperty(window_id)) {
				let interactionWindow = this.m_windows[window_id];
				if (interactionWindow.client) {
					if (interactionWindow.client.m_interaction) {
						let interaction = interactionWindow.client.m_interaction;
						let force_navigation_panel_id = interactionWindow.client.panel_name;
						let info = this.m_main.get_interaction_panel_info(interaction, force_navigation_panel_id);
						if (windows_by_panels[info.navigation_panel_id]) {
							windows_by_panels[info.navigation_panel_id].total++;
							windows_by_panels[info.navigation_panel_id].info = info;
						}
					}
				}
			}
		}
		return windows_by_panels;
	};

	// TODO: #9778 - CANY - Docked.
	Panel.prototype.on_show_folder = function() {
		this.collapsed_panel_current("interaction");
	};

	// TODO: #9778 - CANY - Docked.
	Panel.prototype.collapsed_panel_current = function(from_app, interaction, is_closed_window = false)
	{
		if (!inConcert.app.is_enable_docked_windows_interactions()) {
			// NORMAL WINDOWS
			return false;
		}
		// DOCKED WINDOWS
		if (from_app === 'supervisor') {//sacar parametro
			return;
		}
		//// INTERACTION App
		const collapseUncollapsePanel = function(info, total) {
			if (!info){
				return;
			}
			for (let id in info.iFrames_containers || {}) {
				if (info.iFrames_containers.hasOwnProperty(id)) {
					let iFrame_container = info.iFrames_containers[id];
					if (iFrame_container.selected && total > 0) {
						iFrame_container.$container.show();
					}
					else {
						iFrame_container.$container.hide();
					}
				}
			}
			const $conversations_container_list = info.$panel.find("[conversations_container_list]");
			$conversations_container_list.removeClass('isCollapsed');
			if (total > 0) {
				$conversations_container_list.addClass('isCollapsed');
			}
			return true;
		};

		let isCollapsedUncollapsed = false;
		let total_info = this.get_total_interactions_windows_by_panels();
		const panelCurrent = this.get_selected_folder_id();

		if (total_info && total_info[panelCurrent]) {
			if (total_info[panelCurrent].info) {
				isCollapsedUncollapsed = collapseUncollapsePanel(total_info[panelCurrent].info, total_info[panelCurrent].total);
			}
			else {
				if (interaction) {
					let info = this.m_main.get_interaction_panel_info(interaction);
					isCollapsedUncollapsed = collapseUncollapsePanel(info, 0);
				}
			}
		}

		if (!isCollapsedUncollapsed) {
			let info = this.m_main.get_panel_info(panelCurrent);
			isCollapsedUncollapsed = collapseUncollapsePanel(info, 0);
		}

		if (interaction) {
			if (this.must_move_to_live_folder(interaction, is_closed_window)) {
				this.select_live_folder();
			}
		}
	};
}());