/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50 */

/* global $, inConcert */

// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	/**
	Class CampaignPanel.
	Panel subclass.
	*/

	'use strict';

	var format = inConcert.Utils.format;

	function CampaignPanel (id)
	{
		this.permission_id = 'int_view_campaigns';
		this.m_handler = $('#' + id);
		this.m_width = 256;
		this.m_visible = true;
		this.m_accounts_initializing = 'none';
		this.openCampaignSummary();
		// tooltip - status account images
	}

	if (typeof inConcert.CampaignPanel !== 'function') {
		inConcert.CampaignPanel = CampaignPanel;
	}

	//TODO this should be part of the CampaignPanel class, but a way to access it from the callbacks must be devised
	//in case a campaign node is received it changes the class on the fly
	function calculateCampaignClass(campaign, node) {
		var stateMode = campaign.status ? 'Active' : '';
		//TODO this could/should be a setting for the moderators
		var expandMode = 'Open';
		var campaignClass = 'boxGroup' + stateMode + expandMode;
		if (node) {
			var currentClass = node.attr('class');
			node.removeClass(currentClass).addClass(campaignClass);
		}
		return campaignClass;
	}

	CampaignPanel.prototype.select_tab = function()
	{
		$('#int_shared_panel_search').hide();
		$('#int_panel_contacts').hide();
		$('#int_panel_campaigns').show();
	};

	//in this case THIS is the campaign to expand NOT the panel
	CampaignPanel.prototype.onCampaignExpand = function() {
		var campaignNode = $(this).parent().parent();
		var campaign = campaignNode.data('campaign');
		//TODO save settings expanded/colapsed
		campaign.expanded = !campaign.expanded;
		calculateCampaignClass(campaign, campaignNode);
	};

	CampaignPanel.prototype.openCampaignSummary = function() {
		var campaigns = inConcert.app.getMainWindow().m_application.getCampaigns();
		for (var i = 0; i < campaigns.length; i++) {
			this.addCampaign(campaigns[i]);
		}
	};

	/**
	* Adds a new campaign to the panel.
	*/
	CampaignPanel.prototype.addCampaign = function(campaign)
	{
		//TODO this should be coming from the settings
		campaign.expanded = true;
		var campaignBox = "<div class='" + calculateCampaignClass(campaign) + "' campaignId='" + campaign.Id + "' campaign_container='true'>";
		campaignBox += "<div class='groupName'>";
		campaignBox += "<img src='/inconcert/apps/agent/images/blank.png' class='butExpandBoxGroup' />";
		campaignBox += "<label class='groupNameText'>" + campaign.getDescription() + "<label>";
		campaignBox += "</div><div class='groupExtras'>";
		campaignBox += "<div class='groupExtraActive'><label class='extraInfoLabel'>" + "<%=_T('Active')%>" + "</label><label class='extraInfoNumber'>0</label></div>";
		campaignBox += "<div class='groupExtraWaiting'><label class='extraInfoLabel'>" + "<%=_T('Waiting')%>" + "</label><label class='extraInfoNumber'>0</label></div>";
		campaignBox += "<div class='groupExtraInbox'><label class='extraInfoLabel'>" + "<%=_T('Queued')%>" + "</label><label class='extraInfoNumber'>0</label></div>";
		campaignBox += "<div class='groupAccountStatus'></div>";
		campaignBox += "<div class='groupAccountMessage' style='display:none'>" + "<%=_T('No accounts configured')%>" + "</div>";
		campaignBox += "</div>";
		campaignBox += "</div>";

		var node = jQuery(campaignBox);
		node.data('campaign', campaign);

		node.find('.butExpandBoxGroup').click(this.onCampaignExpand);

		//insert orderly by description
		var campaignsNode = this.m_handler.find('.boxContentScrollArea');
		var campaignList = campaignsNode.children();
		var inserted = false;
		for (var i = 0; i < campaignList.length; i++) {
			var campaignNode = $(campaignList[i]);
			var name = campaignNode.find('.groupNameText').text();
			if (name > campaign.getDescription()) {
				campaignNode.before(node);
				inserted = true;
				break;
			}
		}
		if (!inserted) {
			campaignsNode.append(node);
		}
		this.updateAccountsStatus(campaign);

		this.updateCounters(campaign);
	};

	CampaignPanel.prototype.deleteCampaign = function(campaign) {
		var $campaigns_scroll_area = this.m_handler.find('.boxContentScrollArea');
		var $campaign_area = $campaigns_scroll_area.find('div[campaign_container="true"][campaignId="' + campaign.Id + '"]');
		$campaign_area.remove();
	};

	CampaignPanel.prototype.updateAccountsStatus = function( campaign )
	{
		var self = this;

		if ( !$.isArray(campaign.accounts) ) {
			return;
		}

		var search_token = campaign.Id.replace('@','\\@');
		var $campaign = this.m_handler.find('div[campaignId="' + search_token + '"]');
		if ( $campaign.length === 0 ) {
			return;
		}

		if(campaign.isInternal()) {
			$campaign.find('.groupAccountMessage').remove();
			return;
		}

		// first iterate in order to gather ALL account statuses for each type
		// TO-DO: This should be done in 1 object
		var statusesByType = {};
		var accountsByType = {};
		var accountsNames = {};
		var $statuses = $campaign.find('.groupAccountStatus');

		campaign.accounts.forEach(function(account)
		{
			var statusClass;
			var status = account.status || account.Status;
			var type = account.proxyType.toUpperCase();
			var $interaction_type = $statuses.find('i[interactionType="' + type + '"]');

			if (account.state === '0') {
				statusClass = 'icoAccountInactive';
			}
			else {
				statusClass = 'icoAccountActive';
				if ( !status || typeof( status ) === 'string' || status.status !== 'OK' ) {
					if ( $interaction_type.length === 0 ) {
						if ( self.m_accounts_initializing === 'none' ) {
							self.m_accounts_initializing = 'running';
							// si elemento en el DOM de la account de la campana no existe y aun ha comenzado el initialize de todas las accounts
							// tengo que o marcar como 'running' el initialize.
							// luego de 3 segundos marco como "m_accounts_initializing=end".
							// despues se quien se encarga de quitar la clase 'TU_CLASE_INITIALIZING' (css), es mediante las actualizaciones del loop.
							window.setTimeout(function() {
								self.m_accounts_initializing = 'end';
							}, 3000 );
						}
					}
					if ( self.m_accounts_initializing === 'running' ) {
						statusClass = statusClass + ' icoAccountInitializing';
					}
					else {
						statusClass = ' icoAccountConflict';
					}
				}
			}
			if ( !statusesByType[type] ) {
				statusesByType[type] = {};
				accountsByType[type] = {};
			}

			var state_description = '';
			if ( typeof( status ) === 'string' ) {
				state_description = status;
			}
			else {
				if ( status ) {
					state_description = inConcert.app.getAccountStatusLabel(type, status.status);
				}
				else {
					if ( self.m_accounts_initializing === 'running' ) {
						state_description = "<%=_T('Initializing')%>";
					}
					else {
						state_description = "<%=_T('Not Checked')%>";
					}
				}
			}
			accountsByType[type][account.id] = state_description;
			statusesByType[type][statusClass] = true;
			accountsNames[account.id] = account.name || "<%=_T('(no name)')%>";
		});

		for ( var accountType in statusesByType )
		{
			if ( statusesByType.hasOwnProperty(accountType) )
			{
				var $channel_state = $statuses.find('i[interactionType="' + accountType + '"]');
				var exist_node = $channel_state.length > 0;
				if ( !inConcert.app.hasLicensesForChannel( accountType ) ) {
					if ( exist_node && $channel_state.is(':visible') ) {
						$channel_state.hide();
					}
					continue;
				}
				else {
					if ( exist_node && !$channel_state.is(':visible') ) {
						$channel_state.show();
					}
				}

				var accountClass = 'icoAccount' + accountType;
				for (var accountStatus in statusesByType[accountType]) {
					if (statusesByType[accountType].hasOwnProperty(accountStatus)) {
						accountClass += ' ' + accountStatus;
					}
				}
				var tooltip = '';
				for (var accountId in accountsByType[accountType]) {
					if (accountsByType[accountType].hasOwnProperty(accountId)) {
						if (tooltip.length > 0) {
							tooltip += '\n';
						}
						tooltip += format('{0} ({1}): {2}', accountsNames[accountId], accountType, accountsByType[accountType][accountId]);
					}
				}
				if ( exist_node ) {
					$channel_state
						.removeClass('')
						.removeAttr('title')
						.attr('title', tooltip)
						.attr('class', accountClass);
				}
				else {
					$channel_state = $("<i class='" + accountClass + "' title='" + tooltip + "' interactionType='" + accountType + "'></i>" );
					$statuses.append( $channel_state );
				}
			}
		}

		return this.m_accounts_initializing;
	};

	//update the main counters  independently of interaction type
	CampaignPanel.prototype.updateCounters = function( campaign )
	{
		var search_token = campaign.Id.replace('@','\\@');
		var $campaign = this.m_handler.find("div[campaignId='" + search_token + "']");
		if ( $campaign.length === 0 ) {
			return;
		}
		var counters = campaign.summarizeCampaignInteractions();
		$campaign.find('.groupExtraActive > .extraInfoNumber').text(counters.totalActive);
		$campaign.find('.groupExtraWaiting > .extraInfoNumber').text(counters.totalWaiting);
		$campaign.find('.groupExtraInbox > .extraInfoNumber').text(counters.totalQueued);
		return counters;
	};
}());
