/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:true, curly:true, browser:true, indent:false, maxerr:50, esversion: 11 */

/* global $, inConcert, Mustache, _, EventEmitter, GoldenLayout */

// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.
if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	'use strict';
	const _T = inConcert.i18n.getString;
	const FORMAT = inConcert.Utils.format;

	const tw = function(name) {
		return inConcert.Utils.truncateWord(name, 10, "..");
	};

	/**
	Class CannedPopup.
	*/
	function CannedPopup (params)
	{
		CannedPopup.uber.constructor.call( this );
		this.clickEventName = this.clickEventName + 'CANNED';
		this.owner = params.owner;
		this.ownerId = 'popupCannedInfo';
		this.canReposition = true;
		this.from_app = params.from_app || 'interaction';
		this.canned = params.canned;
		this.can_add = params.can_add;
	}

	inConcert.Utils.inherit(CannedPopup, inConcert.PopupWindow);

	/*******************************************************************
	 * Metodo que crea el markup contenedor de la ventana
	 */
	CannedPopup.prototype.createMarkup = function()
	{
		const self = this;

		var get_html_add_btn = function()
		{
			if (!self.can_add) {
				return "";
			}
			let caption = "<%=_T([[Add]])%>";
			if (self.owner.m_interaction.type === "Chat") {
				let format_type = self.owner.get_canned_format_type(self.canned);
				// El canal de WEB CHAT se visualiza en el formato que se haya configurado en administration.
				if (format_type.id === "HTML") {
					caption = "<%=_T([[Send]])%>";
				}
				caption = "<%=_T([[Add]])%>";
			}
			// El makup del boton del ADD
			return '<button id="insertCanned" class="pageButtonGeneric buttonColorBlue ml-2">' + caption + '</button>';
		};

		$('#' + this.ownerId).remove();
		//create the markup everytime
		let $node = $(''.concat(
			'<div class="systemPopWindow windowContainer windowCannedMessages" style="display:none" id="', this.ownerId, '">',
				'<div class="windowTop">',
					'<div class="windowTitle">', "<%=_T([[Canned Information]])%>", '</div>',
				'</div>',
				'<div class="windowMiddle windowCannedMessagesContainer">',
					'<div class="row">',
						'<div class="col-12 resultDataItem">',
							'<label for="cannedGroup" class="resultLabel">', "<%=_T([[Group]])%>", "<%=_T([[:]])%>", '</label>',
							'<span id="cannedGroup" class="resultInfo"></span>',
						'</div>',
						'<div class="col-12 resultDataItem">',
							'<label for="cannedName" class="resultLabel">', "<%=_T([[Name]])%>", "<%=_T([[:]])%>", '</label>',
							'<span id="cannedName" class="resultInfo"></span>',
						'</div>',
						'<div class="col-12 resultDataItem">',
							'<label for="cannedGroup" class="resultLabel">', "<%=_T([[Message]])%>", "<%=_T([[:]])%>", '</label>',
							'<span id="cannedMessage" class="resultInfo cannedMessageContainer"></span>',
						'</div>',
					'</div>',
				'</div>',
				'<div class="windowBottom">',
					'<button id="closeCannedPopup" class="pageButtonGeneric buttonColorWhite">', "<%=_T([[Close]])%>", '</button>',
					get_html_add_btn(),
				'</div>',
			'</div>'));

		$('body').append($node);
		return $node;
	};

	/*******************************************************************
	 *
	 */
	CannedPopup.prototype.initialize = function()
	{
		this.data.container = this.createMarkup();
		this.loadCanned();
		this.setEvents();
	};

	CannedPopup.prototype.close = function(params) {
		if (this.data && this.data.container) {
			this.data.container.remove();
		}
		this.finalize(params);
	};

	/*******************************************************************
	 * Metodo que setea eventos de los controles
	 */
	CannedPopup.prototype.setEvents = function()
	{
		const self = this;
		//---------------
		$('#insertCanned')
			.unbind('click')
			.click(function() {
				self.data.callbacks.onAdd(self.canned);
				self.close();
			});
		//---------------
		$('#closeCannedPopup').unbind('click').click(function() {
			self.close();
		});
	};

	/*******************************************************************
	 * Metodo que crea el markup de todos los comentarios y los renderiza
	 */
	CannedPopup.prototype.loadCanned = function()
	{
		const self = this;
		$('#cannedGroup').append(this.canned.group);
		$('#cannedName').append(this.canned.name);
		var message = this.canned.message;
		message = this.owner.__replace_vars(message);
		var $html = $('#cannedMessage');
		$html.html(message);
		// Bindeo el click de los links, para que los mismos se abran en una nueva ventana.
		inConcert.Utils.bind_href($html);
		window.setTimeout(function(){
			self.setPosition({
				top_position_config: {
					decrement: 10 // con este valor ajusto el top en 10 px si es que sale fuera de la pantalla.
				}
			});
		}, 20);
	};

	//------------------------------------------------------------------------------------
	/**
	Class QnAPopupDetail.
	*/
	function QnAPopupDetail (params)
	{
		QnAPopupDetail.uber.constructor.call( this );
		this.clickEventName = this.clickEventName + 'QNA';
		this.owner = params.owner;
		this.ownerId = 'popupQnAInfo';
		this.canReposition = true;
		this.from_app = params.from_app || 'interaction';
		this.answer = params.answer;
	}

	inConcert.Utils.inherit(QnAPopupDetail, inConcert.PopupWindow);

	/*******************************************************************
	 * Metodo que crea el markup contenedor de la ventana
	 */
	QnAPopupDetail.prototype.createMarkup = function()
	{
		$('#' + this.ownerId).remove();
		const $node = $(''.concat(
			'<div class="systemPopWindow windowContainer windowQnAMakerMessages" style="display:none;" id="', this.ownerId, '">',
				'<div class="windowTop">',
					"<span id='qnaMakerDetailScore' score='true' class='qnaMakerScoreText'></span>",
					'<div class="windowTitle" id="qnaMakerDetailQuestion">', "<%=_T([[QnA Maker Question Information]])%>",
					'</div>',
				'</div>',
				'<div class="windowMiddle windowQnAMessagesContainer">',
					'<div class="row">',
						// alternatives
						'<div class="col-12 resultDataItem">',
							'<label for="qnaMakerDetailAlterantives" class="resultLabel">', "<%=_T([[Alternative Questions]])%>", '</label>',
							'<span id="qnaMakerDetailAlterantives" class="resultInfo"><span class="qnaMakerAlternativeText">...</span></span>',
						'</div>',
						// answer
						'<div class="col-12 resultDataItem">',
							'<label for="qnaMakerDetailAnswer" class="resultLabel">', "<%=_T([[Answer]])%>", '</label>',
							'<span id="qnaMakerDetailAnswer" class="resultInfo"></span>',
						'</div>',
						// metadata
						'<div class="col-12 resultDataItem">',
							'<label for="qnaMakerDetailMetadata" class="resultLabel">', "<%=_T([[Metadata]])%>", '</label>',
							'<span id="qnaMakerDetailMetadata" class="resultInfo QnAMakerMessageContainer"><span class="qnaMetadataText">...</span></span>',
						'</div>',
					'</div>',
				'</div>',

				'<div class="windowBottom">',
					'<button id="closeQnADetailPopup" class="pageButtonGeneric buttonColorWhite">', "<%=_T([[CLOSE]])%>", '</button>',
				'</div>',
			'</div>'));

		$('body').append($node);
		return $node;
	};

	/*******************************************************************
	 *
	 */
	QnAPopupDetail.prototype.initialize = function()
	{
		this.data.container = this.createMarkup();
		this.loadInfo();
		this.setEvents();
	};

	QnAPopupDetail.prototype.close = function(params) {
		if (this.data && this.data.container) {
			this.data.container.remove();
		}
		this.finalize(params);
	};

	/*******************************************************************
	 * Metodo que setea eventos de los controles
	 */
	QnAPopupDetail.prototype.setEvents = function()
	{
		const self = this;
		//---------------
		$('#closeQnADetailPopup').unbind('click').click(function() {
			self.close();
		});
	};

	/*******************************************************************
	 * Metodo que crea el markup
	 */
	QnAPopupDetail.prototype.loadInfo = function()
	{
		const self = this;

		// question & answer
		$('#qnaMakerDetailQuestion').html(this.answer.questions[0]);
		$('#qnaMakerDetailAnswer').html(this.answer.answer);

		// Score
		const score_class = this.owner.get_qna_maker_score_css_class(this.answer.score);
		const $score = $('#qnaMakerDetailScore');
		$score.addClass(score_class);
		$score.text(this.answer.score);

		// Alternatives
		const $alternatives = $('#qnaMakerDetailAlterantives');
		if (this.answer.questions.length > 1) {
			$alternatives.html("");
			let i = 0;
			this.answer.questions.forEach(question => {
				i++;
				if (i > 1) {
					const $alternative = $("<span class='qnaMakerAlternativeText'></span>");
					$alternative.html(question);
					$alternatives.append($alternative);
				}
			});
		}

		//Metadata
		const $metadata = $('#qnaMakerDetailMetadata');
		(this.answer.metadata || []).forEach((metadata, i) => {
			if (i === 0) {
				$metadata.html("");
			}
			const $name_value = $(
				"<span class='qnaMetadataText'><span metadata_name='true'></span>&nbsp;|&nbsp;<strong metadata_value='true'></strong></span>"
			);
			$name_value.find("[metadata_name]").text(metadata.name);
			$name_value.find("[metadata_value]").text(metadata.value);
			$metadata.append($name_value);
		});

		window.setTimeout(function(){
			self.setPosition({
				top_position_config: {
					decrement: 10 // con este valor ajusto el top en 10 px si es que sale fuera de la pantalla.
				}
			});
		}, 20);
	};

	//-------------------------------------
	//
	const WINDOW_MAXIMIZED = "MAXIMIZED";
	const WINDOW_NORMAL    = "NORMAL";
	const WINDOW_MINIMIZED = "MINIMIZED";
	const RELOAD_TRACKME_TIME_MS = 5000; // 5 segundos

	var ClientSidebar = function(params)
	{
		const self = this;

		this.owner = params.owner;
		this.initialized = false;
		this.already_rendered = false;
		this.is_visible = false;
		this.m_current_window_state = null; // MINIMIZED, MAXIMIZED, NORMAL
		this.m_request_browsing_history_timestamp_ms = null;
		this.m_helpdesk_render_status = null;
		this.m_mas_render = {};
		this.m_isReadyCounter = 0;
		this.m_dynamic_tabs = [];

		// MAXIMUM_SIZE_IN_CLIENT o MAXIMUM_SIZE_IN_SIDEBAR
		this.maximum_size = {
			MAXIMUM_SIZE_IN_CLIENT  : { class_name: 'relatedLogsFocusClient' , selected: true ,  tooltip: "<%=_T([[Maximum size in sidebar]])%>" },
			MAXIMUM_SIZE_IN_SIDEBAR : { class_name: 'relatedLogsFocusSidebar', selected: false,  tooltip: "<%=_T([[Maximum size in client]])%>"  }
		};

		this.m_layout_config = null;

		this.update_information(
			params.campaign_id     ,
			params.account_id      ,
			params.client_id       ,
			params.contact_addresses,
			params.login           ,
			params.pre_survey      ,
			params.params          ,
			params.post_survey     ,
			params.batch           ,
			params.process_flow    ,
			params.token           ,
			params.interaction     ,
			params.layout_config
		);

		this.from_app = params.from_app;
		this.m_interaction_gui = new inConcert.interaction_gui( params.actions,
			{
				from_app: params.from_app,
				owner   : this
			}
		);

		// del layout design se obtiene la informacion correspondiente de los tabs, etc (es para que quede centralizado en un unico lugar)
		this.m_layout_design = new inConcert.LayoutDesign({});

		// list tab container
		this.$m_layout_container = null;
		if (this.m_layout_config) {
			this.$m_layout_container = $("#" + this.m_layout_config.container_id);
			if (this.$m_layout_container.length === 0) {
				this.$m_layout_container = null;
			}
		}

		// list tab container
		this.$m_contactform_container   = null;
		this.$m_conversations_container = null;
		this.$m_browsing_container      = null;
		this.$m_context_container       = null;
		this.$m_canned_container        = null;
		this.$m_qna_maker_container     = null;
		this.$m_assistance_container    = null;

		this.emit_show_id     = 'show_sidebar';
		this.prefix_canned_id = 'client_sidebar_canned_msg_';
		this.m_current_view   = null;
		this.request_members  = null;

		// cached info
		this.m_previous_conversations = null;
		this.m_canned_messages        = null;
		this.m_canned_groups          = null;
		this.m_browsing_history       = null;
		this.m_canned_popup           = null;

		// messages pub handles
		this.pubsub_contact_updated_handle = null;

		// instancia del popup de tabs cerradas.
		this.m_closed_tabs_popup = null;
		this.m_hasBeenInitialisedGoldenLayout = false;

		// Tipos de formatos que soporta el CANAL
		this.m_canned_formats_list = {
			["HTML"]      : {
				id  : "HTML",
				desc: "<%=_T([[Format HTML]])%>",
				channels: {
					supported_in: "SOME",
					allowed: {
						"CHAT": true,
						"MAIL": true
					}
				}
			},
			["TEXT PLAIN"]: {
				id  : "TEXT PLAIN",
				desc: "<%=_T([[Text Plain]])%>",
				channels: {
					supported_in: "ALL"
				}
			}
		};

		var set_callback = function( fn_name ) {
			return typeof params[ fn_name ]  === 'function' ? params[ fn_name ] : $.noop;
		};

		// callbacks
		this.onSidebarShown            = set_callback('onSidebarShown');
		this.onSidebarHidden           = set_callback('onSidebarHidden');
		this.onSidebarChangeTab        = set_callback('onSidebarChangeTab');

		this.onSidebarSetCannedMessage = function(canned) {
			var fn = set_callback('onSidebarSetCannedMessage');
			canned.message = self.__replace_vars(canned.message || '');
			canned.plain_message = self.__replace_vars(canned.plain_message || '');
			fn(canned);
		};

		this.onSidebarSetQnaAMakerAnswer = function(answer) {
			var fn = set_callback('onSidebarSetQnaAMakerAnswer');
			fn(answer);
		};

		this.onSidebarIntialisedLayout = function() {
			if (self.m_hasBeenInitialisedGoldenLayout) {
				//console.debug("ClientSidebar.onSidebarIntialisedLayout --------> HAS BEEN INITIALISED GOLDEN LAYOUT. No invoke callback of client window. interaction: " + self.m_interaction.id);
				return;
			}
			//console.debug("ClientSidebar.onSidebarIntialisedLayout -1-------> Invoking callback of client window. interaction: " + self.m_interaction.id);
			self.m_hasBeenInitialisedGoldenLayout = true;
			var fn = set_callback('onSidebarIntialisedLayout');
			fn();
		};

		this.onShowMinimizeButton = function() {
			var fn = set_callback('onShowMinimizeButton');
			fn();
		};

		/*
			El orden por defecto de los TABs se tiene que respetar, es decir primero tiene que estar el "main", luego las integraciones y el tab de informacion, etc.
			Los tabs de integraciones y el de informacion tiene que estar si o si primeros ya que despues estos son los que tienen que quedar selected.
			Si a esto no lo realizamos el orden de los tabs cambia una vez que se ha renderizado la ventana.
			Tambien este orden por defecto, tiene que coincidir con el orden por defecto la clase "inConcert.LayoutUserDesign" funcion "get_layout_default_tabs_config" en el JavaScript "/shared/layouts/scripts/layout.js"
		*/
		this.m_tabs = {
			// MAIN
			main:{
				tab_container_id: function() {
					return self.m_layout_config ? self.m_layout_config.client_main_id : 'mainContainer';
				},
				id: 'main',
				is_main: true,
				is_dynamic: false,
				method: 'show_main_tab',
				$layout_tab: null,
				it_can_be_shown: function( callback ) {
					callback({status: "SHOW"});
				}
			}
		};
	
		// TAb fijas que NO se pueden mover (Las INTEGRACIONES)
		this.m_fixedTabs = {
			// CC INTEGRATION
			client_sidebar_cc_integration_tab: {
				is_dynamic: false,
				tab_container_id: 'cc_integration_panel',
				class_name: 'integrationContentContainer integration',
				method: 'show_cc_integration_view',
				$layout_tab: null,
				id: 'client_sidebar_cc_integration_tab',
				it_can_be_shown: function( callback ) {
					const can_show = self.can_show_cc_integration();
					const status = can_show ? "SHOW" : "HIDE";
					callback({status: status});
				}
			},
			// MAS INTEGRATION
			client_sidebar_mas_integration_tab: {
				is_dynamic: false,
				tab_container_id: 'mas_integration_panel',
				class_name: 'integrationContentContainer integration',
				method: 'show_mas_integration_view',
				$layout_tab: null,
				id: 'client_sidebar_mas_integration_tab',
				it_can_be_shown: function( callback ) {
					const can_show = self.can_show_mas_integration();
					const status = can_show ? "SHOW" : "HIDE";
					callback({status: status});
				}
			},
			// HELPDESK INTEGRATION
			client_sidebar_helpdesk_integration_tab: {
				is_dynamic: false,
				tab_container_id: 'helpdesk_integration_panel',
				class_name: 'integrationContentContainer integration',
				method: 'show_helpdesk_integration_view',
				$layout_tab: null,
				id: 'client_sidebar_helpdesk_integration_tab',
				it_can_be_shown: function( callback ) {
					const can_show = self.can_show_helpdesk_integration();
					const status = can_show ? "SHOW" : "HIDE";
					callback({status: status});
				}
			}
		};

		// TAb fijas que SI se pueden mover (ContextInfo, etc)
		this.m_moveableTabs = {
			// INFORMATION
			client_sidebar_context_tab: {
				is_dynamic: false,
				tab_container_id: 'context_panel',
				class_name: 'clientSideBarPanels survey',
				method: 'show_context_info',
				$layout_tab: null,
				id: 'client_sidebar_context_tab',
				it_can_be_shown: function( callback ) {
					callback({status: "SHOW"});
				}
			},
			// PREVIOUS CONVERSATIONS
			client_sidebar_conversations_tab: {
				is_dynamic: false,
				tab_container_id: 'conversations_panel',
				class_name: 'clientSideBarPanels conversations',
				method: 'show_conversations_view',
				$layout_tab: null,
				id: 'client_sidebar_conversations_tab',
				it_can_be_shown: function( callback )
				{
					const can_show = self.m_interaction.isBeingComposed() ? false : true;
					const status = can_show ? "SHOW" : "CLOSE";
					callback({status: status});

					if (can_show) {
						if (!self.pubsub_contact_updated_handle) {
							// suscribe to contact changes
							self.pubsub_contact_updated_handle = inConcert.app.messagebus.subscribe(
								'/application/contacts/contact_updated',
								function(/*contact*/)
								{
									if (self.m_interaction.isBeingComposed()) {
										return;
									}
									var client_id; // va undefined porque al contact lo resuelve desde el atributo interaction.contact
									self.get_previous_conversations(self.m_interaction.campaign, client_id,
										function(previous_conversations) {
											self.m_previous_conversations = previous_conversations;
											if (self.m_current_view === 'client_sidebar_conversations_tab') {
												self.show_conversations_view(self.m_current_view);
											}
										}
									);
								}
							);
						}
					}
					else {
						self.events_unsuscribe('CONTACTS');
					}
				}
			},
			// WEB HISTORY
			client_sidebar_browsing_tab: {
				is_dynamic: false,
				tab_container_id: 'browsing_panel',
				class_name: 'clientSideBarPanels browsingHistory',
				method: 'show_browsing_view',
				$layout_tab: null,
				id: 'client_sidebar_browsing_tab',
				it_can_be_shown: function( callback ) {
					if (self.m_interaction.type !== "CHAT") {
						callback({status: "CLOSE"});
						return;
					}
					let status = "HIDE";
					if (self.has_configured_tracking()) {
						status = self.m_browsing_history !== null ? "SHOW" : status;
					}
					callback({status: status});
				}
			},
			// CANNED
			client_sidebar_canned_tab: {
				is_dynamic: false,
				tab_container_id: 'canned_panel',
				class_name: 'clientSideBarPanels cannedMessages',
				method: 'show_canned_view',
				$layout_tab: null,
				id: 'client_sidebar_canned_tab',
				it_can_be_shown: function( callback ) {
					// Webcontact does not have outgoing conversations.
					if (self.m_interaction.type === 'WEBCONTACT') {
						const status = "CLOSE";
						return callback({status: status});
					}
					else if (self.m_interaction.type === "CALL" || self.m_interaction.type === "VOICEMAIL") {
						const status = "CLOSE";
						return callback({status: status});
					}
					else if (self.m_interaction.isFinished() || self.m_interaction.isWrappedUp()) {
						const status = "HIDE";
						return callback({status: status});
					}
					else if (self.m_interaction.isQueued() || self.m_interaction.isWaiting() || self.m_interaction.isAbandoned()) {
						const status = "CLOSE";
						return callback({status: status});
					}
					// Visualizo el Tab
					const status = "SHOW";
					return callback({status: status});
				}
			},
			// QNA
			client_sidebar_qna_maker_tab: {
				is_dynamic: false,
				tab_container_id: 'qna_maker_panel',
				class_name: 'clientSideBarPanels qnaMaker',
				method: 'show_qna_maker_view',
				$layout_tab: null,
				id: 'client_sidebar_qna_maker_tab',
				it_can_be_shown: function( callback = $.noop ) {
					if (!self.has_configured_qna_maker()) {
						const status = "HIDE";
						return callback({status: status});
					}
					else if (self.m_interaction.type === 'WEBCONTACT') {
						// Webcontact does not have outgoing conversations.
						const status = "CLOSE";
						return callback({status: status});
					}
					else if (self.m_interaction.isBeingComposed()) {
						if (self.m_interaction.type === "CALL" || self.m_interaction.type === "VOICEMAIL") {
							const status = "HIDE";
							return callback({status: status});
						}
					}
					else if (self.m_interaction.isFinished() || self.m_interaction.isAbandoned() || self.m_interaction.isWaiting() || self.m_interaction.isQueued()){
						const status = "CLOSE";
						return callback({status: status});
					}
					// Visualizo el Tab
					callback({status: "SHOW"});
				}
			},
			// CONTACT FORM
			client_sidebar_contactform_tab: {
				is_dynamic: false,
				tab_container_id: 'contactform_panel',
				class_name: 'clientSideBarPanels contactForm',
				method: 'show_contactform_view',
				$layout_tab: null,
				id: 'client_sidebar_contactform_tab',
				it_can_be_shown: function( callback ) {
					self.m_interaction.hasContactFormInteractionParent(function(is_contactform_parent_it) {
						const status = is_contactform_parent_it ? "SHOW" : "CLOSE";
						callback({status: status});
					});
				}
			},
            client_sidebar_assistance_tab: {
				is_dynamic: false,
				tab_container_id: 'assistance_panel',
				class_name: 'clientSideBarPanels assistance',
				method: 'show_assistance_info',
				$layout_tab: null,
				id: 'client_sidebar_assistance_tab',
				it_can_be_shown: function( callback ) {
					callback({status: "SHOW"});
				}
			}
		};

		// Tabs fijos que no se pueden move.
		for (let key in this.m_fixedTabs) {
			if (this.m_fixedTabs.hasOwnProperty(key)) {
				let tab = this.m_fixedTabs[key];
				this.m_tabs[key] = tab;
			}
		}

		// Tabs dinamicos (widgets integrations)
		//
		let dynamicMethdFn = function(design_tab) {
			return function() {
				self.show_dynamic_integration_view(design_tab.widget_id);
			};
		};
		let it_can_be_shown = function(design_tab) {
			return function(callback) {
				const can_show = self.can_show_dynamic_integration(design_tab.widget_id);
				const status = can_show ? "SHOW" : "HIDE";
				callback({status: status});
			};
		};

		let design_tabs = this.m_layout_design.get_layout_default_tabs_config();
		for (let i = 0; i < design_tabs.length; i++) {
			let design_tab = design_tabs[i];
			if (!design_tab.is_dynamic) {
				continue;
			}
			this.m_dynamic_tabs.push(design_tab);
			let tab = {
				is_dynamic: true,
				tab_container_id: design_tab.widget_id + '_widget_integration_panel',
				class_name: 'integrationContentContainer integration',
				method: dynamicMethdFn,
				$layout_tab: null,
				id: design_tab.id, //'client_sidebar_helpdesk_integration_tab',
				it_can_be_shown: it_can_be_shown(design_tab)
			};
			this.m_tabs[design_tab.id] = tab;
		}

		// Tabs movibles
		for (let key in this.m_moveableTabs) {
			if (this.m_moveableTabs.hasOwnProperty(key)) {
				let tab = this.m_moveableTabs[key];
				this.m_tabs[key] = tab;
			}
		}

		return true;
	};

	inConcert.Utils.inherit(ClientSidebar, EventEmitter);

	if (typeof inConcert.ClientSidebar !== 'function') {
		inConcert.ClientSidebar = ClientSidebar;
	}

	var contactinfo_groups_tpl = "<div class='contactInfoTitle'>" + "<%=_T([[Contact Info]])%>" + "</div>" +
							"<div class='contactInfoContainer'></div>";

	var login_groups_tpl = "<div class='loginTitle'>" + "<%=_T([[Login]])%>" + "</div>" +
							"<div class='loginContainer'></div>";

	var survey_groups_tpl = "<div class='preSurveyTitle'>" + "<%=_T([[Pre Survey]])%>" + "</div>" +
							"<div class='preSurveyContainer'></div>";

	var params_groups_tpl = "<div class='paramsTitle'>" + "<%=_T([[Parameters]])%>" + "</div>" +
							"<div class='paramsContainer'></div>";

	var post_survey_groups_tpl = "<div class='postSurveyTitle'>" + "<%=_T([[Post Survey]])%>" + "</div>" +
							"<div class='postSurveyContainer'></div>";

	var batch_groups_tpl = "<div class='batchAgentDataTitle'>" + "<%=_T([[Batch Message]])%>" + "</div>" +
							"<div class='batchAgentDataContainer'></div>";

	var process_flow_groups_tpl = "<div class='processFlowDataTitle'>" + "<%=_T([[Bot]])%>" + "</div>" +
							"<div class='processFlowDataContainer'></div>";

	var assistance_groups_tpl = "<div class='assistanceDataContainer'></div>";

	var item_contactinfo_tpl = "<div class='contactInfoItem'>" +
								"<label class='contactInfoName'>{{ name }} </label>" +
								"<label class='contactInfoValue'>{{ value }} </label>" +
							"</div>";

	var item_login_tpl = "<div class='loginItem'>" +
								"<label class='loginQuestion'>{{ question }} </label>" +
								"<label class='loginAnswer'>{{ answer }} </label>" +
							"</div>";

	var item_survey_tpl = "<div class='surveyItem'>" +
								"<label class='surveyQuestion'>{{ question }} </label>" +
								"<label class='surveyAnswer'>{{ answer }} </label>" +
							"</div>";

	var item_params_tpl = "<div class='paramItem'>" +
								"{{#alias}} <label class='paramAlias'> {{ alias }} </label> {{/alias}}" +
								"<label class='paramValue'>{{ value }} </label>" +
							"</div>";

	var item_post_survey_tpl = "<div class='postSurveyItem'>" +
								"<label class='postSurveyQuestion'>{{ question }} </label>" +
								"<label class='postSurveyAnswer'>{{ answer }} </label>" +
							"</div>";

	var item_batch_tpl = "<div class='batchAgentDataItem'>" +
								"<label class='batchAgentDataName'>{{ name }} </label>" +
								"<label class='batchAgentDataValue'>{{ value }} </label>" +
							"</div>";

	var item_process_flow_tpl = "<div class='processFlowDataItem'>" +
								"<label class='processFlowDataName'>{{ name }} </label>" +
								"<label class='processFlowDataValue'>{{ value }} </label>" +
							"</div>";

	var item_process_flow_internal_consultation = "<div id='{{ question_id }}_container' class='assistanceDataInternalConsultation'>" +
		"<div class='assistanceDataContainerData'><label class='assistanceDataName'>{{ name }} </label>" +
		"<label class='assistanceDataValue'>{{ value }} </label></div>" +
		"<div id='{{ question_id }}_progress_total' class='assistanceDataProgressTotal'><div id='{{ question_id }}_progress' class='assistanceDataProgressBar' style='width:{{ progress }}'></div></div>" +
		"<div id='{{ question_id }}_progress_detail' class='assistanceDataProgressDetail'><span class='assistanceDataProgressDetailA'><%=_T([[Time to respond]])%></span><span id='{{ question_id }}_time_left' class='assistanceDataProgressDetailB'>{{ time_left }}</span></div></div>" +
	"<div class='assistanceDataButtonTakeContainer'><button id='{{ question_id }}_button_take' class='pageButtonGeneric buttonColorBlue assistanceDataButtonTake'><%=_T([[Respond to the bot]])%></button></div>";

	var item_process_flow_internal_consultation_history = 
		"<div id='{{ question_id }}_container' class='assistanceDataInternalConsultation'>" +
			"<div class='assistanceDataContainerDataHistoryAgent'>" + 
				"<label class='assistanceDataName'><%=_T([[The bot needs help:]])%></label>" +
				"<label class='assistanceDataValue'>{{ last_question }}</label>" +
			"</div>" +
			"<div class='assistanceDataContainerDataHistorySupervisor'>" + 
				"<label class='assistanceDataName'>{{ last_response_user }} <%=_T([[responded:]])%></label>" +
				"<label class='assistanceDataValue'>{{ last_response }}</label>" +
			"</div>" +
		"</div>";

	var item_process_flow_internal_consultation_response = "<div id='{{ question_id }}_response_container' class='assistanceDataInternalConsultationResponse'>" +
		"<textarea id='{{ question_id }}_response' class='assistanceDataResponse' placeholder='<%=_T([[Write your answer here...]])%>'></textarea>" +
		"<div class='assistanceDataButtonContainer'>" +
		"{{ #allowStoringInKB }}<input id='{{ question_id }}_add_to_qna' type='checkbox' class='assistanceDataCheckbox'/></input>" +
		"<label for='{{ question_id }}_add_to_qna' class='moduleTitleText'><%=_T([[Add answer to knowledge base]])%></label>{{ /allowStoringInKB }}" +
		"<button id='{{ question_id }}_button' class='pageButtonGeneric buttonColorBlue assistanceDataButtonSend'><%=_T([[Send]])%></button></div>" +
	"</div>";

	var item_process_flow_internal_consultation_taken = "<div id='{{ question_id }}_taken_for_another_agent' class='assistanceDataInternalConsultationTaken'>" +
		"<div class='assistanceDataContainerData'>" +
		"<label class='assistanceDataTitle'><div class='inlineMessagesContainer messageInfo'></div> Conversación asistiéndose</label>" +
		"<label class='assistanceDataName'><%=_T([[At this moment]])%> <b>{{ agent_name }}</b> <%=_T([[is helping the bot, you can see his response live or take control of the interaction]])%></label>" +
		"</div></div>";

	var item_process_flow_internal_consultation_time_off = "<div id='{{ question_id }}_time_off' class='assistanceDataInternalConsultationTimeOff'>" +
		"<div class='assistanceDataContainerData'>" +
		"<label class='assistanceDataTitle'><div class='inlineMessagesContainer messageInfo'></div> <%=_T([[Time is up to respond to the bot]])%></label>" +
		"<label class='assistanceDataName'><%=_T([[The interaction will continue with the flow that was defined]])%></label>" +
		"</div></div>";

	var browsing_history_tpl = "{{#params}}" +
									"<div class='activityItem'>" +
										"<label class='activityText'>" +
											"<a class='browsingLink' href='{{ url }}'>{{ url }}</a>" +
											"<span class='browsingDate'>{{ activity_date }}</span>" +
										"</label>" +
									"</div>" +
								"{{/params}}";

	var item_tpl_canned_data = "<div class='d-flex flex-row align-items-center'><span class='cannedFormatType'>{{ format_type }}  </span><span class='cannedTitle'>{{ activity_type }}</span></div><span class='cannedMessagesData'>'{{ message }}'</span>";

	var item_tpl_canned = "<div class='row'>" +
							"<div class='col-12'>" +
								"<div id='{{ id }}' class='cannedItem'>" +
									"<div class='cannedText flex-grow-1'>" +
										item_tpl_canned_data +
									"</div>" +
									"<div class='cannedMessagesActionsColumn'>" +
										"<button action_add='true' class='buttonColorGray gridActionButton' title='{{ add_msg }}'>" +
											"<i class='{{ add_class }}'></i>" + //buttons-icons-add or 
										"</button>" +
										"<button action_show='true' class='buttonColorGray gridActionButton show' title='<%=_T([[Show]])%>' hidden>" +
											"<i class='far fa-eye'></i>" +
										"</button>" +
									"</div>" +
								"</div>" +
							"</div></div>";

	var show_more_canned_messages_tpl = "<div class='row pt-3 mx-0'>" +
											"<div class='col-12'>" +
												"<button id='showMoreCannedButton' class='pageButtonGeneric buttonColorWhite'>" +
													"<i class='far fa-chevron-down'></i>" +
													"<%=_T([[Show more options]])%>" +
												"</button>" +
											"</div>" +
										"</div>";

	var group_conversations_tpl = "<div group_conversation_id='{{ group_conversation_id }}' class='asociatedInteractionsGroup' node_type='group_conversations'></div>";

	var item_conversation_tpl = "<div id='{{ id }}' class='activityItem {{ additional_classes }}' node_type='interaction' belongs_to_group='{{ belongs_to_group }}' >" +
									"<i class='activityType'></i>" +
									"<label class='activityText'>" +
										"<span class='activityAction'>{{ agent }} </span>" +
										"<span class='activityDate'>{{ activity_date }} </span>" +
										"<span class='activityAbstract'>{{ abstract }}</span>" +
									"</label>" +
								"</div>";

	var no_data_available_tpl = "<div class='row noDataAvailable'>" +
									"<div class='col-12'>" +
										"<div class='inlineMessagesContainer messageInfo'>" + "<%=_T([[No data available]])%>" + "</div>" +
									"</div>" +
								"</div>";

	var sidebar_tpl = "<div id='sidebar_container' class='logsContainer' style='display:none;'>" +
						// LINK que realiza el switch del tamanio maximo/minimo de la sidebar.
						"<a id='sidebar_resize' class='logResizeButton' tooltip=''><i></i></a>" +

						"<div id='sidebar_tabs_internal_container' class='logTabsInternalContainer'>" +
							"<div class='logTabsContent'>" +
								// contactform div panel
								"<div id='contactform_panel' tab='contactform' class='clientSideBarPanels' style='display:none;'>" +
									"<div id='contactform_list' class='logContentContainer'></div>" +
								"</div>" +

								// conversations div panel
								"<div id='conversations_panel' tab='conversations' class='clientSideBarPanels' style='display:none;'>" +
									"<div id='conversations_list' class='logContentContainer'></div>" +
								"</div>" +

								// browsing div panel
								"<div id='browsing_panel' tab='browsing' tab='conversations' class='clientSideBarPanels' style='display:none;'>" +
									"<div id='sidebar_browsing_waiting' class='clientLoading' style='display:none;'>" +
										"<div class='clientLoadingAnimation'></div>" +
									"</div>" +
									"<div class='logContentContainer'>" +
										"<button id='sidebar_refresh_trackme' class='buttonColorGray butBrowseListRefresh' title='{{ refresh }}' >" +
											"<i class='buttons-icons-refresh'></i>" +
										"</button>" +
										"<div id='browsing_list' class='browselistContentContainer'></div>" +
									"</div>" +
								"</div>" +

								// context div panel
								"<div id='context_panel' tab='context_info' class='clientSideBarPanels' style='display:none;'>" +
									"<div id='context_list' class='logContentContainer'></div>" +
								"</div>" +

								// assistance div panel
                                "<div id='assistance_panel' tab='assistance' class='clientSideBarPanels' style='display:none;'>" +
                                    "<div id='assistance_list' class='logContentContainer'></div>" +
                                "</div>" +

								// canned div panel
								"<div id='canned_panel' tab='canned' class='clientSideBarPanels' style='display:none;'>"+
									"<div id='canned_list' class='logContentContainer'>"+
										"<div class='cannedMessagesTitle'>" + "<%=_T([[Canned Messages]])%>" + "</div>" +
										"<div class='cannedFilter cannedMessagesFilterContainer'>" +
											"<div class='row'>" +
												"<div class='col-12'>" +
													"<div class='form-group'>" +
														"<div class='cannedCategoryContainer'>" +
															"<select id='client_sidebar_cbo_canned_groups' class='form-control mt-0'></select>" +
														"</div>" +
													"</div>" +
												"</div>" +
											"</div>" +
											"<div class='row'>" +
												"<div class='col-12 d-flex flex-row'>" +
													"<div class='d-flex flex-grow-1'>" +
														"<input class='cannedFilterSearch form-control mt-0' type='text' placeholder='" + "<%=_T('Enter value')%>..." + "' />" +
														"<button class='pageButtonGeneric buttonToggle search butCannedFilter' disabled='true' >" +
															"<i class='far fa-search'></i>" +
														"</button>" +
													"</div>" +
													"<div class='form-group d-flex align-items-end ml-2'>"+
														"<button class='pageButtonGeneric buttonColorWhite iconOnlyBtn search eraser butCannedFilterReset' disabled='true'>" +
															"<i class='far fa-eraser'></i>" +
														"</button>" +
													"</div>" +
												"</div>" +
											"</div>" +
										"</div>" +
									"</div>" +
								"</div>" +

								// cci div panel
								"<div id='cc_integration_panel' tab='cc_integration' class='integrationContentContainer' style='display:none;'>" +
									"<div class='integrationInternalContainer'>" +
										"<div id='cc_integration_view' class='integrationContentScroll'></div>" +
										"<div id='cc_integration_waiting' class='clientLoading'>" +
											"<div class='clientLoadingAnimation'></div>" +
										"</div>" +
										"<div id='cc_integration_error' class='integrationErrorMessageContainer' hidden>" +
											"<div class='helpDeskContentErrorContainer'>" +
												"<div class='helpDeskListErrorMessage'>" +
													"<label error_msg='true' class='integrationErrorMessage'>" + "<%=_T([[Error loading tab]])%>" + "</label>" +
													// "<button load_retry='true' class='helpDeskListRetryButton' type='button' tooltip='", _T("Retry"), "'>" +
													// 	"<i class='TU_CLASE_RCR'></i>" +
													// 	_T("Retry") +
													// "</button>" +
												"</div>" +
												"<div class='helpDeskListErrorIcon'></div>" +
												"<div class='helpDeskListErrorOverlay'></div>" +
											"</div>" +
										"</div>" +
									"</div>" +
								"</div>" +

								// qna_maker div panel
								"<div id='qna_maker_panel' tab='qna_maker' class='clientSideBarPanels' style='display:none;'>"+
									"<div class='logContentContainer qnaMakerContainer'>"+
										"<div class='qnaMessagesTitle'>" + "<%=_T([[QnA Maker]])%>" + "</div>" +
										"<div class='qnaMakerFilter cannedMessagesFilterContainer'>" +
											"<div class='row'>" +
												"<div class='col-12 d-flex flex-row'>" +
													"<div class='form-group flex-grow-1'>" +
														"<input qna_maker_question='true' class='qnaMakerFilterSearch form-control mt-0' type='text' placeholder='" + "<%=_T('Enter value')%>..." + "' />" +
													"</div>" +
													"<div class='form-group d-flex align-items-end ml-2'>"+
														"<button qna_maker_search_anwser='true' class='pageButtonGeneric buttonColorWhite iconOnlyBtn search butQnaMakerFilter' >" +
															"<i class='far fa-search'></i>" +
														"</button>" +
														"<button qna_maker_reset='true' class='pageButtonGeneric buttonColorWhite iconOnlyBtn search eraser butQnaMakerFilterReset'>" +
															"<i class='far fa-eraser'></i>" +
														"</button>" +
													"</div>" +
												"</div>" +
											"</div>" +
										"</div>" +
										// waiting
										"<div id='sidebar_qna_waiting' class='clientLoading' style='display:none;'>" +
											"<div class='clientLoadingAnimation'></div>" +
										"</div>" +
										// list
										"<div id='qna_maker_list' class='qnaMakerListContainer' style='display:none'></div>" +
										"<div class='row' id='qna_maker_no_answer_found'>" +
											"<div class='col-12'>" +
												"<div class='inlineMessagesContainer messageInfo'>" + "<%=_T([[No Answers Found.]])%>" + "</div>" +
											"</div>" +
										"</div>" +
									"</div>" +
								"</div>" +

								// M&S Integration div panel
								"<div id='mas_integration_panel' tab='mas_integration' class='integrationContentContainer' style='display:none;'>" +
									"<div class='integrationInternalContainer'>" +
										"<div id='mas_integration_view' class='integrationContentScroll'>" +
											'<iframe id="mas_integration_iframe" iframe_type="mas_integration" frameborder="0" height="220px" width="404px" style=""></iframe>' +
										"</div>" +
										"<div id='mas_integration_waiting' class='clientLoading'>" +
											"<div class='clientLoadingAnimation'></div>" +
										"</div>" +
										"<div id='mas_integration_error' class='integrationErrorMessageContainer' hidden>" +
											"<div class='helpDeskContentErrorContainer'>" +
												"<div class='helpDeskListErrorMessage'>" +
													"<label error_msg='true' class='integrationErrorMessage'>" + "<%=_T([[Error loading tab]])%>" + "</label>" +
													// "<button load_retry='true' class='helpDeskListRetryButton' type='button' tooltip='", _T("Retry"), "'>" +
													// 	"<i class='TU_CLASE_RCR'></i>" +
													// 	_T("Retry") +
													// "</button>" +
												"</div>" +
												"<div class='helpDeskListErrorIcon'></div>" +
												"<div class='helpDeskListErrorOverlay'></div>" +
											"</div>" +
										"</div>" +
									"</div>" +
								"</div>" +

								// HelpDesk Integration div panel
								"<div id='helpdesk_integration_panel' tab='mas_integration' class='integrationContentContainer' style='display:none;'>" +
									"<div class='integrationInternalContainer'>" +
										"<div id='helpdesk_integration_view' class='integrationContentScroll'></div>" +
										"<div id='helpdesk_integration_waiting' class='clientLoading'>" +
											"<div class='clientLoadingAnimation'></div>" +
										"</div>" +
										"<div id='helpdesk_integration_error' class='integrationErrorMessageContainer' hidden>" +
											"<div class='helpDeskContentErrorContainer'>" +
												"<div class='helpDeskListErrorMessage'>" +
													"<label error_msg='true' class='integrationErrorMessage'>" + "<%=_T([[Error loading tab]])%>" + "</label>" +
													// "<button load_retry='true' class='helpDeskListRetryButton' type='button' tooltip='", _T("Retry"), "'>" +
													// 	"<i class='TU_CLASE_RCR'></i>" +
													// 	_T("Retry") +
													// "</button>" +
												"</div>" +
												"<div class='helpDeskListErrorIcon'></div>" +
												"<div class='helpDeskListErrorOverlay'></div>" +
											"</div>" +
										"</div>" +
									"</div>" +
								"</div>" +

								"<div id='dynamic_integrations_panel'></div>" +

							"</div>" +
						"</div>" +
					"</div>";

	var item_tpl_qna_maker_pair = "" +
        "<div class='row'>" +
            "<div class='col-12'>" +
                "<div qna_anwser_item='true' class='qnaMakerAnswerItem'>" +
					"<div class='d-flex w-100'>" +
						"<div class='qnaMakerText flex-grow-1'>" +
							// ---------
							"<div class='d-flex flex-row align-items-center'>" +
								"<span answer='true' class='qnaMakerScoreText {{ score_class }}'>{{ score }}</span>" +
								"<span question_alternatives='true' class='qnaMakerQuestionText'>{{ question }}</span>" +
							"</div>" + 
						"</div>" +
						// buttons
						"<div qna_actions='true' class='qnaMakerActionsContainer'>" +
							"<button action_add='true' class='buttonColorGray gridActionButton' title='{{ add_msg }}'>" +
								"<i class='{{ add_class }}'></i>" + //buttons-icons-add or 
							"</button>" +
							"<button action_show='true' class='buttonColorGray gridActionButton show' title='{{ view_msg }}'>" +
								"<i class='{{ view_class }}'></i>" + //buttons-icons-monitor or
							"</button>" +
						"</div>" +
					"</div>" +
                     // answer
                    "<span answer='true' class='qnaMakerAnswerText'>'{{ answer }}'</span>" +
					// METADATA
					// "<div metadata='true' class='qnaMakerMetadataContainer' style='display:none;'>" +
					// 	"<div metadata_name_value_list='true' class='qnaMakerMetadataText'></div>" +
					// "</div>" +
                "</div>" +
            "</div>" +
        "</div>";

	ClientSidebar.prototype.build_sidebar = function()
	{
		const sidebar_view = {
			hide                 : "<%=_T([[HIDE]])%>",
			refresh              : "<%=_T([[Update]])%>",

			contactform_name     : tw("<%=_T([[Contact Form]])%>"),
			contactform_tooltip  : "<%=_T([[Contact Form]])%>",
			contactform          : true,

			integration_name     : tw("<%=_T([[Integration]])%>"),
			integration_tooltip  : "<%=_T([[Integration]])%>",
			integration          : true,

			browsing_name        : tw("<%=_T([[Web-Historical]])%>"),
			browsing_tooltip     : "<%=_T([[browsing]])%>",
			browsing             : true,

			canned_name          : tw("<%=_T([[Canned]])%>"),
			canned_tooltip       : "<%=_T([[Canned]])%>",
			canned               : true,

			context_info_name    : tw("<%=_T([[Context]])%>"),
			context_info_tooltip : "<%=_T([[Context]])%>",
			context_info         : true,

			conversations_name   : tw("<%=_T([[Previous Conversations]])%>"),
			conversations_tooltip: "<%=_T([[Previous Conversations]])%>",
			conversations        : true,

			qna_maker_name       : tw("<%=_T([[QnA Maker]])%>"),
			qna_maker_tooltip    : "<%=_T([[QnA Maker]])%>",
			qna_maker            : true,

			assistance_name      : tw("<%=_T([[Assistance]])%>"),
			assistance_tooltip   : "<%=_T([[Assistance]])%>",
			assistance           : true
		};
		// Add initial markup
		$('body').append( Mustache.render( sidebar_tpl, sidebar_view) );

		this.$m_contactform_container   = $('#contactform_list');
		this.$m_conversations_container = $('#conversations_list');
		this.$m_browsing_container      = $('#browsing_list');
		this.$m_context_container       = $('#context_list');
		this.$m_canned_container        = $('#canned_list');
		this.$m_qna_maker_container     = $('#qna_maker_list');
		this.$m_assistance_container    = $('#assistance_list');

		//dynamic_integrations_panel
		let html = "";
		for (let i = 0; i < this.m_dynamic_tabs.length; i++) {
			let dynamic_tab = this.m_dynamic_tabs[i];
			let widget_id = (dynamic_tab.widget_id).toLowerCase();
			if (!this.can_show_dynamic_integration(widget_id)) {
				continue;
			}
			let panel_id = dynamic_tab.widget_id + "_widget_integration_panel";
			let iframe_id = dynamic_tab.name + "_integration_iframe";
			let error_id = dynamic_tab.name + "_integration_error";
			// /inconcert/apps/integrations/3082?widget_id=sample&interaction_id=EL_ID
			let iframe_src = FORMAT("/inconcert/apps/integrations/{0}?widget_id={1}&interaction_id={2}", "<%=_L%>", encodeURIComponent(widget_id), encodeURIComponent(this.m_interaction.id));
			html = html +
				// Dynamic Widget Integration div panel
				"<div id='" + panel_id + "' tab='" + dynamic_tab.name + "' class='integrationContentContainer' style='display:none;'>" +
					"<div class='integrationInternalContainer'>" +
						"<div id='mas_integration_view' class='integrationContentScroll'>" +
							'<iframe id="' + iframe_id + '" iframe_type="' + dynamic_tab.name + '" frameborder="0" height="220px" width="404px" src="' + iframe_src + '"></iframe>' +
						"</div>" +
						"<div id='" + error_id + "' class='integrationErrorMessageContainer' hidden>" +
							"<div class='helpDeskContentErrorContainer'>" +
								"<div class='helpDeskListErrorMessage'>" +
									"<label error_msg='true' class='integrationErrorMessage'>" + "<%=_T([[Error loading tab]])%>" + "</label>" +
								"</div>" +
								"<div class='helpDeskListErrorIcon'></div>" +
								"<div class='helpDeskListErrorOverlay'></div>" +
							"</div>" +
						"</div>" +
					"</div>" +
				"</div>";
		}
		if (html) {
			$("#dynamic_integrations_panel").html(html);
		}

		this.already_rendered = true;
	};

	/*
		Valida si la sidebar y quien la ha instanciado inlcuye el GoldenLayout.
	*/
	ClientSidebar.prototype.hasConfiguredLayout = function()
	{
		// aca valido si la config ha llegado
		if (!this.m_layout_config) {
			console.warn("ClientSidebar.hasConfiguredLayout (1) - No existe configuracion para el layout.");
			return false;
		}
		// valido si se encuentra enabled
		if (!this.m_layout_config.enabled) {
			console.warn("ClientSidebar.hasConfiguredLayout (2) - La configuracion se encuentra Enabled='" + this.m_layout_config.enabled + "'");
			return false;
		}
		// valido si llega la info del id del tab main (el cliente en si)
		if (!this.m_layout_config.client_main_id) {
			console.warn("ClientSidebar.hasConfiguredLayout (3) - La configuracion es Erronea. Atributo client_main_id='" + this.m_layout_config.client_main_id + "'");
			return false;
		}
		// valida que sea un cliente valido.
		const $client_main = $('#' + this.m_layout_config.client_main_id);
		if (!$client_main || $client_main.length === 0) {
			console.warn("ClientSidebar.hasConfiguredLayout (4) - No existe el Contanier Main ='" + this.m_layout_config.client_main_id + "'");
			return false;
		}
		// valida que exista el plugin GoldenLayout
		if (typeof(GoldenLayout) === "undefined") {
			console.warn("ClientSidebar.hasConfiguredLayout (5) - El Plugin GoldenLayout NO Existe.");
			return false;
		}
		return true;
	};

	/*
		En nuestra estructura de Tabs, asigno el jQuery layout_tab, asi podemos realizar cambios al seleccionar un tab.
	*/
	ClientSidebar.prototype.setLayoutComponent = function(tab_id, component)
	{
		if (!this.m_tabs[tab_id]) {
			return;
		}
		this.m_tabs[tab_id].component = component;
	};

/*
FUNCIONES que quedan comentadas para revisar si llega a ser necesario.

*/
	/*
		Esta funcion no retorna el objet jQuery layout_tab que hemos almacenado en nuestra estructura.
	*/
	// ClientSidebar.prototype.getLayoutComponent = function(tab_id)
	// {
	// 	if (!this.m_tabs[tab_id]) {
	// 		return;
	// 	}
	// //	return this.m_tabs[tab_id].component;

	// 	// let components = this.m_layout.root.getComponentsByName(tab_id);
	// 	// if (components) {
	// 	// 	return components[0];
	// 	// }

	// 	// let component = this.m_layout.getComponent("testComponent");
	// 	// return component

	// 	// let components = this.m_layout.root.getComponentsByName("testComponent");
	// 	// if (components) {
	// 	// 	for (let i = 0; i < components.length; i++) {
	// 	// 		let component = components[i];
	// 	// 		console.log("  component " + component + ", component.id " + component.id);
	// 	// 	}
	// 	// 	//return components[0];
	// 	// }

	// 	let container;
	// 	let components = this.m_layout.root.getItemsByType("component");
	// 	if (components) {
	// 		for (let i = 0; i < components.length; i++) {
	// 			let c = components[i];
	// 			console.log("  component " + c + ", component.config.id " + c.config.id);
	// 			if (c.config && c.config.id === tab_id) {
	// 				container = c.container;
	// 				break;
	// 			}
	// 		}
	// 	}
	// 	return container;
	// };

	ClientSidebar.prototype.getLayoutComponent = function(tab_id)
	{
		if (!this.m_tabs[tab_id]) {
			return;
		}
		return this.m_tabs[tab_id].component;
	};

	/*
		En nuestra estructura de Tabs, asigno el jQuery layout_tab, asi podemos realizar cambios al seleccionar un tab.
	*/
	ClientSidebar.prototype.setLayoutTab = function(tab_id, $tab)
	{
		if (!this.m_tabs[tab_id]) {
			return;
		}
		this.m_tabs[tab_id].$layout_tab = $tab;
	};

	/*
		Esta funcion no retorna el objet jQuery layout_tab que hemos almacenado en nuestra estructura.
	*/
	ClientSidebar.prototype.$getLayoutTab = function(tab_id)
	{
		if (!this.m_tabs[tab_id]) {
			return;
		}
		return this.m_tabs[tab_id].$layout_tab;
	};

	/*
		Valida si tiene que almacenar el layout
	*/
	ClientSidebar.prototype.canSaveLayoutStage = function()
	{
		if (!this.hasConfiguredLayout()) {
			return false;
		}
		// alguna validacion de permisos o configuracion si llega a ser necesario.
		return true;
	};

	ClientSidebar.prototype.getLayoutDefaultTabsConfig = function()
	{
		return this.m_layout_design.load_translated_layout_default_tabs_config();
	};

	ClientSidebar.prototype.getLayoutDefaultTabConfig = function(tab_id) {
		let default_tabs_config = this.getLayoutDefaultTabsConfig();

		for (let i = 0; i < default_tabs_config.length; i++) {
			let tab_config = default_tabs_config[i];
			if (tab_config.id === tab_id) {
				return tab_config;
			}
		}
	};

	/*
		La informacion por defecto del layout.
	*/
	ClientSidebar.prototype.getLayoutDefaultConfig = function()
	{
		return this.m_layout_design.load_translated_layout_default_config();
	};

	/*
		La informacion Custom del layout. Ya sea del Usuario o particular del VCC.
	*/
	ClientSidebar.prototype.getLayoutCustomConfig = function()
	{
		const self = this;

		var translate = function(config) {
			return self.m_layout_design.translate_config(config);
		};

		let channel = this.m_interaction.type;
		let user_custom_config = inConcert.app.loadLoggedUserClientsLayoutConfig();
		if (user_custom_config) {
			if (user_custom_config[channel]) {
				return translate(user_custom_config[channel][channel]);
			}
			if (user_custom_config.ALL) {
				return translate(user_custom_config.ALL.ALL);
			}
		}

		let vcc_custom_config = inConcert.app.loadVccClientsLayoutConfig();
		if (!vcc_custom_config) {
			return;
		}
		if (vcc_custom_config[channel]) {
			let layout_config = vcc_custom_config[channel][channel];
			if (layout_config) {
				return translate(layout_config);
			}
		}
		if (vcc_custom_config.ALL) {
			let layout_config = vcc_custom_config.ALL.ALL;
			if (layout_config) {
				return translate(layout_config);
			}
		}

		return;
	};

	/*
		La key con la cual se almacenara en el localStorage
	*/
	ClientSidebar.prototype.makeLayoutStorageKey = function()
	{
		return "i6_users_window_clients_layout";
	};

	/*
		Remueve la key por completo del localStorage (para todos los usuarios)
		Esto se debe utilizar si en algun momento la estructura cambia por algo.
	*/
	ClientSidebar.prototype.removeAllSavedLayoutConfig = function()
	{
		const key = this.makeLayoutStorageKey();
		inConcert.Storage.remove(key);
	};

	ClientSidebar.prototype.canSaveLayoutConfig = function()
	{
		return false;
	};

	/*
		Recupera desde el localStorage la informacion que se ha almacenado para el layout (todos los usuarios)
	*/
	ClientSidebar.prototype.loadSavedLayoutConfig = function()
	{
//		this.removeAllSavedLayoutConfig();
		if (!this.canSaveLayoutConfig()) {
			return false;
		}
		const key = this.makeLayoutStorageKey();
		const users_window_clients_layout = inConcert.Storage.get(key);
		return users_window_clients_layout;
	};

	/*
		Retorna la configuracion del layout para el usuario actual.
			Si existe informacion en el localStorage lo retorna desde ahi.
			Si no existe informacion en el localStorage, retorna la config por default.
	*/
	ClientSidebar.prototype.loadLayoutConfig = function()
	{
		const default_config = this.getLayoutDefaultConfig();

		if (!this.m_interaction) {
			// si no existe una interaccion, retorna la config por defecto (esto no deberia suceder nunca)
			return default_config;
		}

		// obtengo la informacion personalizada, que se ha configurado a nivel VCC o a nivel Usuario.
		const custom_config = this.getLayoutCustomConfig();
		if (custom_config) {
			return this.setLayoutReadonlyConfig(custom_config);
		}
	
		return this.setLayoutReadonlyConfig(default_config);
	};

	/*
		Las ventanas que contengan iFrames tienen que estar readonly, o sea no se pueden mover con el plugin GoldenLayout,
			ya que si se realiza esto el navegador recarga el iFrame y se pierde la informacion.
	*/
	ClientSidebar.prototype.isReadonlyLayoutByChannel = function()
	{
		if (!this.m_interaction) {
			return true;
		}
		if (this.m_interaction.type === "MAIL") {
			return true;
		}
		return false;
	};

	ClientSidebar.prototype.isReadonlyLayout = function()
	{
		if (!this.m_interaction) {
			return true;
		}
		if (this.m_interaction.isBeingComposed()) {
			return true;
		}
		if (this.can_show_cc_integration()) {
			return true;
		}
		if (this.can_show_mas_integration()) {
			return true;
		}
		if (this.can_show_helpdesk_integration()) {
			return true;
		}
		if (this.isShownSomeDynamicWidget()) {
			return true;
		}
		if (this.isReadonlyLayoutByChannel()) {
			return true;
		}
		return false;
	};

	ClientSidebar.prototype.isShownSomeDynamicWidget = function()
	{
		for (let i = 0; i < this.m_dynamic_tabs.length; i++) {
			let dynamic_tab = this.m_dynamic_tabs[i];
			let widget_id = (dynamic_tab.widget_id).toLowerCase();
			if (this.can_show_dynamic_integration(widget_id)) {
				// SI se esta visualizando al menos un Tab dinamico.
				return true;
			}
		}
		// NO se esta visualizando nigun Tab dinamico.
		return false;
	};

	/*
		Funcion encargada de dejar inamovibles los Tabs de GoldenLayout.
		Si, se pueden resizear, pero no se pueden cambiar de lugar o de stack.
	*/
	ClientSidebar.prototype.setLayoutReadonlyConfig = function(layout_config)
	{
		if (!layout_config) {
			return;
		}

		if (!this.isReadonlyLayout()) {
			return layout_config;
		}

		var readonly_tabs = function(content = []) {
			let len = content.length - 1;
			for (let i = len; i >= 0; i--) {
				let item = content[i];
				if (item.type === "component") {
					item.isClosable = false;
				}
				else {
					// Invoco a la funcion recursiva para que busque TABs dentro del Stack
					readonly_tabs(item.content);
				}
			}
		};

		readonly_tabs(layout_config.content);

		if (layout_config.settings) {
			layout_config.settings.reorderEnabled = false;
			layout_config.settings.selectionEnabled = true;
			layout_config.settings.showMaximiseIcon = false;
		}

		return layout_config;
	};

	/*
		Se almacena un JSON en "localStorage" bajo la clave "i6_users_window_clients_layout".
		Estructura del JSON:
{
	-- 1 - UNA LISTA DE USUARIOS
	--
	"agentee2@emma": {
		-- 2 - DENTRO DE CADA USUARIO, LA APP DE I6
		--
		"interaction": {
			-- 3 - DENTRO DE CADA APP DE I6, EL TIPO DE VENTANA (MAIL, SMS, ETC)
			--
			"MAIL": {
				-- 4 - ACA QUEDA ALMACENADA LA CONIGURACION QUE RETORNA "GoldenLayout"
				--
				"settings": {   },
				"dimensions": {   },
				"labels": { },
				"content": [
					{},
					{}
				],
			},
			"FACEBOOK": {},
			"INSTAGRAM": {},
			..
			..
		},
		"supervisor":{
			"MAIL": {},
			"FACEBOOK": {},
			"INSTAGRAM": {},
			..
		}
	},
	"otroagent1@emma": {
		"interaction": {}
	},
	..
	..
	
}
	*/
	ClientSidebar.prototype.saveLayoutConfig = function()
	{
// 	const layout_config = this.m_layout.toConfig();
//	console.log("ClientSidebar.prototype.saveLayoutConfig - layout_config: " + JSON.stringify(layout_config));
		if (!this.canSaveLayoutConfig()) {
			return;
		}
		//Check for localStorage
		if (!typeof inConcert.Storage.is_supported_for_browser()) {
			return;
		}

		// obtengo la informacion que se ha almacenado en localStorage para todos los usuarios.
		let users_window_clients_layout = this.loadSavedLayoutConfig() || {};
		const logged_user_id = inConcert.app.getCompleteId();

		// armo info para el usuario
		users_window_clients_layout[logged_user_id] = users_window_clients_layout[logged_user_id] || {};
		// armo info para el usuario+app
		users_window_clients_layout[logged_user_id][this.from_app] = users_window_clients_layout[logged_user_id][this.from_app] || {};

		// recupero desde el plugin GoldenLayout la configuracion actual
		const user_window_client_layout = this.m_layout.toConfig();
		// armo info para el usuario+app+client (tipo de interaccion, "mail", "facebook", etc)
		users_window_clients_layout[logged_user_id][this.from_app][this.m_interaction.type] = user_window_client_layout;

		// almaceno en el localStorage
		const key = this.makeLayoutStorageKey();
		//console.log("ClientSidebar.prototype.saveLayoutConfig - Saving layout config in localStorage. KEY: '" + key + "', VALUE: " + JSON.stringify(users_window_clients_layout));
		inConcert.Storage.put(key, users_window_clients_layout);
	};

	ClientSidebar.prototype.__count_closed_tabs = function()
	{
		const popup_id = 'sidebar_closed_tabs_name';
		let $menu = $('#' + popup_id);
		const $all_closed_tabs = $menu.find("[tab_id]");
		return $all_closed_tabs.length;
	};

	ClientSidebar.prototype.toggle_popup_tabs_button = function(force_hide)
	{
		const $btn = $("#sidebar_show_popup_include_tabs");
		if (this.__count_closed_tabs() === 0 || force_hide) {
			$btn.hide();
			$(".logContentContainer").removeClass("floatBtnShown");
		}
		else {
			$btn.show();
			$(".logContentContainer").addClass("floatBtnShown");
		}
		if (this.m_closed_tabs_popup) {
			this.m_closed_tabs_popup.finalize();
		}
	};

	ClientSidebar.prototype.__showPopupHiddenTabs = function(params)
	{
		const self = this;

		inConcert.app.m_mainWindow.hideBannerMessage();
		if (this.__count_closed_tabs() === 0) {
			inConcert.app.m_mainWindow.showBannerMessage({message: "<%=_T([[No closed Tabs.]])%>", timeout : 3000});
 			return;
		}

		params.$parent.prop("disabled", true);

		if (this.m_closed_tabs_popup) {
			params.$parent.removeClass("showingMenu");
			this.m_closed_tabs_popup.close();
			this.m_closed_tabs_popup = null;
		}

		const popup_id = 'sidebar_closed_tabs_name';
		let $menu = $('#' + popup_id);

		$menu.detach();
		$('body').append($menu);

		params.$parent.removeClass("showingMenu");

		this.m_closed_tabs_popup = new inConcert.PopupWindow({ownerId: popup_id});
		// Show popup
		this.m_closed_tabs_popup.show(
		{
			ownerId      : popup_id,
			eventName    : popup_id,
			canReposition: true,
			container    : $menu,
			parentNode   : params.$parent,
			callbacks: {
				onOpen: function() {
					$('#' + popup_id).show();
					params.$parent.removeClass("showingMenu").addClass("showingMenu");
					params.$parent.prop("disabled", false);
				},
				onClose: function(callback) {
					callback = callback || $.noop;
					params.$parent.removeClass("showingMenu");
					if (self.m_closed_tabs_popup) {
						self.m_closed_tabs_popup.hide(function() {
							window.setTimeout(function() {
								params.$parent.removeClass("showingMenu");
								params.$parent.prop("disabled", false);
								self.m_closed_tabs_popup = null;
							});
							callback();
						});
						return;
					}
					params.$parent.prop("disabled", false);
					callback();
				}
			}
		});
	};

	ClientSidebar.prototype.__makeLayoutClosedTabs = function()
	{
		const $closed_tabs_container = $("#sidebar_include_tabs_container");
		$closed_tabs_container.removeClass().addClass("tabsMainMenuContainer");
		const $content = $('' +
			// BUTTON que visualiza el popup con tabs cerrados-
			'<button id="sidebar_show_popup_include_tabs" class="addButton" style="display:none;" title="' + "<%=_T([[Include Tabs]])%>" + '">' + 
				'<i class="buttons-icons-add-solid"></i>' +
			'</button>' +
			// UL que contiene los Nombres/IDs de los TABs disponibles que han sido cerrados del GOLDEN LAYOUT
			'<div id="sidebar_closed_tabs_name" class="interactionMenuBlind closedTabsMenu" style="display:none;"></div>' +
			// DIV que contiene el contenido (MARKUP) de los TABs disponibles que han sido cerrados del GOLDEN LAYOUT
			'<div id="sidebar_closed_tabs_content"></div>'
		);
		// renderiza el markup
		$closed_tabs_container.html($content);
	};

	/*
		Esta funcion es la encargada de validar si la sidebar y el plugin del layout estan ready.
		Si estan ready invoca a los callbacks (de quien invoco a la funcion y el callback de la client window para indicarle de que la sidebar esta ready)

		El counter a 3 (NUMBER_IS_READY) es porque para que este listo esta funcion "validateIsReady" se invoca de 3 lados:
			1 -  (*****) El evento callback del plugin "initialised".
			2 - (***)    callback: initialize-> initializeLayout -> initialized      -> "update_sidebar_tabs".
			3 - (****)   callback: initialize-> on "TabDataRetrieved" -> initialized -> "update_sidebar_tabs".
	
		Es decir para estar ready, tiene que haber sido invocada esta funcion 3 veces.
		Esto es asi ya que la validacion de si se muestran los tabs, pueden ser o no asincronicos.
	*/
	ClientSidebar.prototype.validateIsReady = function(callback = $.noop)
	{
		const self = this;
		this.m_isReadyCounter++;
		callback(true);

		const NUMBER_IS_READY = 3;

		if (this.m_isReadyCounter > NUMBER_IS_READY) {
			//console.debug("[ClientSidebar.validateIsReady] --------> Validating IF LAYOUT IS READY. " + this.m_isReadyCounter + " of " + NUMBER_IS_READY + ", interaction: " + this.m_interaction.id);
			return;
		}

		//console.debug("[ClientSidebar.validateIsReady] -A-------> Validating IF LAYOUT IS READY. " + this.m_isReadyCounter + " of " + NUMBER_IS_READY + ", interaction: " + this.m_interaction.id + ", state: " + this.m_interaction.state);

		// En este callback, realizamos hide del waiting del frm, y alguna tarea especifica que cada cliente necesite.
		if (this.m_isReadyCounter === NUMBER_IS_READY) {
			//console.debug("[ClientSidebar.validateIsReady] -B=======================> LAYOUT IS READY. Execute ASYNC (setTimeout) interaction: " + this.m_interaction.id);
			this.m_hnd_is_ready_layout = window.setTimeout(function() {
				//console.debug("[ClientSidebar.validateIsReady] -C =======================> LAYOUT IS READY. Now SYNC Invoking callback. interaction: " + self.m_interaction.id);
				self.onSidebarIntialisedLayout();
			});
		}
		//else {
			//console.debug("[ClientSidebar.validateIsReady] -D-------> Layout IS NOT READY." + this.m_interaction.id);
		//}
	};

	ClientSidebar.prototype.initializeLayout = function(callback = $.noop)
	{
		const self = this;

		//console.debug("");
		//console.debug("-START (initializeLayout)--------------------------------------------------------------------");
		console.info("ClientSidebar.initializeLayout - (START) Instanciando GoldenLayout.");

		if (!this.hasConfiguredLayout()) {
			console.warn("ClientSidebar.initializeLayout - No se puede instanciar GoldenLayout.");
			//console.debug("-ERROR (initializeLayout)--------------------------------------------------------------------");
			//console.debug("");
			callback();
			return;
		}

		if (this.m_layout) {
			callback();
			return;
		}

		this.__makeLayoutClosedTabs();

		$("#sidebar_show_popup_include_tabs")
			.off("click")
			.on("click", function() {
				const params = {
					$parent: $(this)
				};
				self.__showPopupHiddenTabs(params);
			}
		);

		// 1 - Retorna la configuracion, del layout:
		const config = this.loadLayoutConfig();
		//console.debug("ClientSidebar.initializeLayout - Layout config: " + JSON.stringify(config || {}));

		let visibles_tabs = {};

		var initializeLayout = function() {
			// (*) Funcion Recursiva que recorre el tree de configuracion de GoldenLayout.
			//		La misma realiza un filtro de la lista de Tabs disponibles que se obtuvieron en el punto 1)
			//		y las elimina, segun la visibilidad personalizada que se obtuvo en el punto 2).
			var filter = function(content = []) {
				let tabs_in_stack = 0;
				let len = content.length - 1;
				for (let i = len; i >= 0; i--) {
					let item = content[i];
					if (item.type === "component") {
					// si es un componente es decir es un TAB
						let visible_tab = visibles_tabs[item.id];
						let status = visible_tab && visible_tab.status;
						if (status !== "SHOW") {
							// Quito el TAB (componente) si:
							//   El TAB es visible segun la validacion de la sidebar 1), y el mismo NO se encuentra visible dentro de la configuracion personalizada.
							if (status === "CLOSE") {
								content.splice(i, 1);
							}
							else {
								tabs_in_stack++;
							}
						}
						else {
							// Si el mismo es visible, marco como que existe un tab en el stack y lo elimino de la lista temporal de tabs "visibles_tabs".
							tabs_in_stack++;
							//delete visibles_tabs[item.id];
						}
					}
					else {
					// Si NO es un componente (TAB), valido si es un stack (contenedor de TABs)
						let is_stack = false;
						if (item.type === "stack") {
							// Ref #8496 Es necesario eliminar el atributo "activeItemIndex" del stack, ya que sino si el mismo no existe, porque el diseno del tab ha cambiado, 
							// el plugin tira un error throw.
							item.__activeItemIndex = item.activeItemIndex;
							delete item.activeItemIndex;
							is_stack = true;
						}
						// Invoco a la funcion recursiva para que busque TABs dentro del Stack
						let total_tabs_in_stack = filter(item.content);
						if (is_stack && total_tabs_in_stack === 0) {
							// Si es un stack y el mismo NO contienen ningun TAB, lo debo de eliminar de la configuracion.
							// quito el contenedor de TABs (stack)
							content.splice(i, 1);
						}
					}
				}
				return tabs_in_stack;
			};

			// 2 - Retorna la configuracion, del layout:
			//	a - Primero si el usuario tiene configuracion personalizada.
			//	b - Si el vcc tiene configuracion personalizada.
			//	c - Si no tiene ninguna de las dos, se obtiene la configuracion por defecto.

			// 3 - Ahora realizo un filtro (*)
			filter(config.content);

			// Variable que se asigna para saber en que stack se encuentra el Tab Main.
			self.m_main_stack = null;
			//console.log("--> ClientSidebar.prototype.initializeLayout . config: " + JSON.stringify(config));
			self.m_layout = new GoldenLayout(config, self.$m_layout_container);
			self.m_layout.__creating = true;

			/*
				Funcion encargada de agregar en la lista de Tabs disponibles, pero que NO estan visibles en el layout.
			*/
			var addMenuItem = function(tab_id)
			{
				//console.debug("  ClientSidebar.initializeLayout - Agregando Tab '" + tab_id + "' oculto (disponible para ser agregado).");
				const tab_config = self.m_tabs[tab_id];
				if (!tab_config) {
					console.log("ClientSidebar.initializeLayout.addMenuItem - Tab does not exist. tab_id: " + tab_id);
					//throw("error tab config does not exist. " + tab_id);
					return;
				}
				const tab_item_config = self.getLayoutDefaultTabConfig(tab_id);
				if (!tab_item_config) {
					return;
				}
				const tab_container_id = typeof(tab_config.tab_container_id) === "function" ? tab_config.tab_container_id() : tab_config.tab_container_id;

				// Aca meto el contenido del TAB.
				const $tab_info = $("#" + tab_container_id);
				$tab_info.hide();
				$tab_info.detach();
				$("#sidebar_closed_tabs_content").hide().append($tab_info);

				// Ahora meto en la lista de tabs disponibles para que se pueda Insertar al Stack seleccionado.
				const $tab_element = $("<div tab_id='" + tab_container_id + "' class='interactionMenuItem'>" + tab_item_config.tooltip + "</div>");
				$tab_element
					.off("click")
					.on("click", function() {
						if(self.m_layout.selectedItem === null) {
							const message = "<%=_T([[No container of tab selected. Please click the header of container to select which target you want to add new tab to.]])%>";
							self.__show_message(message, "Info", 5000);
						} else {
							self.m_layout.selectedItem.addChild(tab_item_config);
						}
					}
				);
				$('#sidebar_closed_tabs_name').append($tab_element);
				/*
					Este es codigo que activa la forma de realizar el insert de tabs cerrados, mediante drag & drop.
				*/
				// // Por ultimo creo el drag source en el GoldenLayout.
				// self.m_layout.createDragSource($tab_element, tab_item_config);

				// visualiza u oculta el boton que abre el popup con los tabs cerrados.
				self.toggle_popup_tabs_button();
			};

			for (let tab_id in visibles_tabs) {
				if (visibles_tabs.hasOwnProperty(tab_id)) {
					let visible_tab = visibles_tabs[tab_id];
					if (visible_tab.status === "SHOW") {
						addMenuItem(tab_id);
					}
				}
			}

			self.m_layout.on('initialised', function() {
				//console.debug("  ClientSidebar.initializeLayout --------> on('initialised ...");
				self.onShowMinimizeButton();
				self.validateIsReady(); // (*****)
			});

			// aca se guarda el estado, cuando ha cambiado la ubicacion layout, etc.
			self.m_layout.on('stateChanged', function(/*resp*/) {
			//	console.debug("  ClientSidebar.initializeLayout - stateChanged");
							//			for (let name in (resp || {})) {
							//				if (resp.hasOwnProperty(name)) {
							//					console.debug("  ClientSidebar.initializeLayout - " + name + " = " + resp[name]);
							//				}
							//			}
				self.saveLayoutConfig();
			});

			// Evento ejecutado, cuando el stack que se encuentra seleccionado ha cambiado.
			self.m_layout.on('selectionChanged', function(selected_stack) {
				//console.debug("  ClientSidebar.initializeLayout - selectionChanged. selected_stack: " + selected_stack);
				// Si el stack seleccionado es null, quiere decir que no ha quedado ningun stack seleccionado, entonces tengo que dejar seleccionado el stack main.
				if (selected_stack === null) {
					if (self.m_main_stack) {
						self.m_layout.selectItem(self.m_main_stack);
					}
				}
			});
	
			// registramos el componente
			self.m_layout.registerComponent('testComponent', function(container, componentState)
			{
				//console.debug("  ClientSidebar.initializeLayout - registerComponent '" + componentState.id + "'");
				const tab_config = self.m_tabs[componentState.id];
				if (!tab_config) {
					console.log("ClientSidebar.initializeLayout.registerComponent - Tab does not exist. tab_id: " + componentState.id);
					//throw("error tab config does not exist. " + componentState.id);
					return;
				}
				//console.warn('a --> registerComponent container.getState(): ' + container.getState());
				//console.warn('b --> registerComponent container.hide(): ' + container.hide());
				const tab_container_id = typeof(tab_config.tab_container_id) === "function" ? tab_config.tab_container_id() : tab_config.tab_container_id;
				const $tab_container = $("#" + tab_container_id);

				if ($tab_container) {
					$tab_container.show();
					// aca inserto el div del contenido de la solapa de la sidebar o el main del cliente, dentro del html del plugin GoldenLayout
					container.getElement().html($tab_container);
					// aca seteo el jQuery element a nuestra estructura de tabs.
					self.setLayoutComponent(componentState.id, container);
				}
			});

			// cuando se crea el TAB, hacemos esta pisada, para poder tener el evento, cuando se selecciona un TAB,
			// ya que el GoldenLayout no tiene el evento onselecttab o algo asi.
			self.m_layout.on('tabCreated', function(tab)
			{
				tab.element.attr("title", tab.contentItem.config.tooltip);
				const tab_id = tab.contentItem.config.id;

				//console.debug("  ClientSidebar.initializeLayout - on tabCreated '" + tab_id + "'");

				/*
					Cuando se crea el tab main, tengo que quedarme con el stack que al cual pertenece, y dejarlo seleccionado.
				*/
				if (!self.m_main_stack) {
					if ("main" === tab_id.toLowerCase()) {
						const stack = tab.contentItem.parent;
						self.m_main_stack = stack;
						self.m_layout.selectItem(self.m_main_stack);
					}
				}

				// aca seteo el jQuery element a nuestra estructura de tabs.
				self.setLayoutTab(tab_id, tab.element);

				const tab_config = self.m_tabs[tab_id];
				if (!tab_config) {
					console.log("ClientSidebar.initializeLayout.on('tabCreated.. - Tab does not exist. tab_id: " + tab_id);
					//throw("error tab config does not exist. " + tab_id);
					return;
				}
				const tab_container_id = typeof(tab_config.tab_container_id) === "function" ? tab_config.tab_container_id() : tab_config.tab_container_id;

				const $closed_tab_name = $('#sidebar_closed_tabs_name').find("[tab_id='" + tab_container_id + "']");
				if ($closed_tab_name.length > 0) {
					$closed_tab_name.remove();
				}
				tab.closeElement
					.off('click') //unbind the current click handler
					.click(function(){
						//add your own
						if (true) {
							addMenuItem(tab_id);
							tab.contentItem.remove();
						}
					});
				// visualiza u oculta el boton que abre el popup con los tabs cerrados.
				self.toggle_popup_tabs_button();
			});

			let is_created = false;
			self.m_layout.on('stackCreated', function(stack)
			{
				//console.debug("  ClientSidebar.initializeLayout - on stackCreated");
				if (!is_created) {
					// si el stack (tab) NO ha sido creado, realizo "show" de la sidebar.
					self.show();
				}
				// evento cuando el TAB (contenido) se encuentra ACTIVO (visible)
				stack.on('activeContentItemChanged', function(contentItem) { //, a, b) {
					const tab_id = contentItem.config.componentState.id;
					//console.debug('  ClientSidebar.initializeLayout - on activeContentItemChanged ' + tab_id + " " + a + " " + b);
					self.on_tab_selected(tab_id);
					//self.refreshLayout();
				});

				stack
					.header
					.controlsContainer
					.find( '.lm_close' ) //get the close icon
					.off( 'click' ) //unbind the current click handler
					.click(function() {
						//window.alert("stack closed todo implement ?")
						//add your own
						//if( confirm( 'really close self?' ) ) {
							//stack.remove();
						//}
					});
			});

			// iniciamos la instancia del layout.
			self.m_layout.init();
			console.info("ClientSidebar.initializeLayout - (END) Instanciando GoldenLayout.");
			//console.debug("-END (initializeLayout)--------------------------------------------------------------------");
			//console.debug("");
			self.refreshLayout();
			callback();
		};

		// 1 - Recorro la lista de Tabs disponibles en la sidebar y evaluo si son visibles.
		//		Esto como fue siempre, dependiendo del estado de la interaccion, del tipo de interaccion, etc.
		//
		//		La funcion que valida si un tab puede o no ser visible, puede o no ser asincronica. por lo que hay que validar si el total de tabs corresponde con los passed.
		let total_tabs = inConcert.Utils.objectCount(this.m_tabs);
		let passed_tabs = 0;
		const it_can_be_shown = function(tab, id) {
			tab.it_can_be_shown(function(result) {
				passed_tabs++;
				visibles_tabs[id] = {
					status    : result.status,
					tab_config: tab
				};
				if (passed_tabs === total_tabs) {
					initializeLayout();
				}
			});
		};
		for (let id in this.m_tabs) {
			if (this.m_tabs.hasOwnProperty(id)) {
				let tab = this.m_tabs[id];
				// funcion asincronica
				it_can_be_shown(tab, id);
			}
		}
	};

	ClientSidebar.prototype.refreshLayout = function() {
		const self = this;
		if (!this.hasConfiguredLayout()) {
			return;
		}
		if (this.m_hnd_refresh_layout) {
			window.clearTimeout(this.m_hnd_refresh_layout);
			this.m_hnd_refresh_layout = undefined;
		}
		this.m_hnd_refresh_layout = window.setTimeout(function() {
			//console.debug("ClientSidebar.refreshLayout - Refresh layout tabs.");
			if (self.m_layout) {
				self.m_layout.updateSize();
				self.m_hnd_refresh_layout = undefined;
			}
		});
	};

	/**
	 * Interaction has contact information to show?
	 *
	 * @private
	 * @return {Boolean}
	 */
	ClientSidebar.prototype.__interaction_has_contact_information = function(/*interaction*/) {
		return this.m_pre_survey || this.m_params || this.m_post_survey || this.m_batch || this.m_process_flow || this.m_contact_addresses;
	};

	ClientSidebar.prototype.__hide_all_containers = $.noop;

	ClientSidebar.prototype.__$get_tab_waiting = function(tab_id) {
		let tabs_waitings = {
			"browsing": {
				dom_id: "sidebar_browsing_waiting"
			},
			"qna": {
				dom_id: "sidebar_qna_waiting"
			}
		};
		if (!tab_id) {
			return;
		}
		let tab_waiting = tabs_waitings[tab_id];
		if (!tab_waiting) {
			return;
		}
		const $waiting = $("#" + tab_waiting.dom_id);
		if ($waiting.length === 0) {
			return;
		}
		return $waiting;
	};

	ClientSidebar.prototype.__show_waiting = function(tab_id) {
		const $waiting = this.__$get_tab_waiting(tab_id);
		if (!$waiting) {
			return;
		}
		$waiting.show();
	};

	ClientSidebar.prototype.__hide_waiting = function(tab_id) {
		const $waiting = this.__$get_tab_waiting(tab_id);
		if (!$waiting) {
			return;
		}
		$waiting.hide();
	};

	ClientSidebar.prototype.__show_message = function(message, type, timeout_ms) {
		this.owner.displayStatusMessage(message, type, false, timeout_ms);
	};

	ClientSidebar.prototype.__hide_message = function() {
		this.owner.hideStatusMessage();
	};

	/**
	Shows the container used for CCI integration.
	*/
	ClientSidebar.prototype.enable_cc_integration = function() {
		/*add 'withIntegration' here to show embedded CCI integration */
		this.m_current_view = null;
		$('body').removeClass("withIntegration").addClass('withIntegration');
		this.hideWaitingCCIntegration();
	};

	ClientSidebar.prototype.on_error_cc_integration = function() {
		this.m_current_view = null;
		$('body').removeClass("withIntegration").addClass('withIntegration');
		this.hideWaitingCCIntegration();
		$("#cc_integration_waiting").hide();
		$("#cc_integration_view").hide();
		$("#cc_integration_error").show();
	};

	ClientSidebar.prototype.hideWaitingCCIntegration = function() {
		window.setTimeout(function() {
			$("#cc_integration_waiting").hide();
			$("#cc_integration_error").hide();
			$("#cc_integration_view").show();
		}, 350);
	};

	ClientSidebar.prototype.enable_helpdesk_integration = function() {
		/*add 'withIntegration' here to show embedded Helpdesk integration */
		this.m_current_view = null;
		$('body').removeClass("withIntegration").addClass('withIntegration');
		this.hideWaitingHelpDeskIntegration();
	};

	ClientSidebar.prototype.on_error_helpdesk_integration = function() {
		this.m_current_view = null;
		$('body').removeClass("withIntegration").addClass('withIntegration');
		$("#helpdesk_integration_waiting").hide();
		$("#helpdesk_integration_view").hide();
		$("#helpdesk_integration_error").show();
	};

	ClientSidebar.prototype.showWaitingHelpDeskIntegration = function() {
		// $("#helpdesk_integration_waiting").show();
		// $("#helpdesk_integration_view").hide();
	};

	ClientSidebar.prototype.hideWaitingHelpDeskIntegration = function() {
		window.setTimeout(function() {
			$("#helpdesk_integration_waiting").hide();
			$("#helpdesk_integration_error").hide();
			$("#helpdesk_integration_view").show();
		}, 350);
	};

	ClientSidebar.prototype.update_information = function( campaign_id, account_id, client_id, contact_addresses, login, pre_survey, params, post_survey, batch, process_flow, token, interaction, layout_config) {
		var get_data = function( data ) {
			return data && !_.isEmpty(data) ? data : null;
		};
//		console.debug("ClientSidebar.prototype.update_information ");

		this.m_campaign_id      = campaign_id || '';
		this.m_account_id       = account_id || '';
		this.m_client           = client_id  || '';
		this.m_contact_addresses = get_data( contact_addresses );
		this.m_login            = get_data( login            );
		this.m_pre_survey       = get_data( pre_survey       );
		this.m_params           = get_data( params           );
		this.m_post_survey      = get_data( post_survey      );
		this.m_batch            = get_data( batch            );
		this.m_process_flow     = get_data( process_flow     );
		this.m_token            = token || '';
		this.m_interaction      = interaction;
		this.m_layout_config    = layout_config|| this.m_layout_config;

		return true;
	};

	/**
	 * Get previous conversations for a client.
	 *
	 * @todo - Move this function to another file
	 * @param  {string}   campaign_id - The campaign identificator.
	 * @param  {string}   client_id   - The client identificator.
	 * @param  {Function} callback    - Callback is not optional
	 */
	ClientSidebar.prototype.get_previous_conversations = function(_, client_id, callback)
	{
		console.debug("[ClientSidebar.get_previous_conversations]  Loading previous conversations. interaction: " + this.m_interaction.id);

		var query = { 
			condition : "AND",
			fixedStateRule: "FINISHED",
			paginating: false,
			output: "interactions",
			rules: []
		};

		var ruleContact = {};

		if (typeof this.m_interaction.contact === 'string' && this.m_interaction.contact.length > 0) {
			ruleContact.id = "contact";
			ruleContact.operator = "equal";
			ruleContact.value = this.m_interaction.contact;
		}
		else if (typeof client_id === 'string' && client_id.length > 0) {
			ruleContact.id = "client";
			ruleContact.operator = "equal";
			ruleContact.value = client_id;
		}
		else {
			setTimeout(function() { callback([]); }, 0);
			return;
		}

		query.rules.push(ruleContact);
		
		var filter = {
			last  : '',
			start : 0,
			rows  : 100,
			query : query
		};
		//console.debug("[ClientSidebar.get_previous_conversations]  Loading previous conversations. filter: " + JSON.stringify(filter));

		// (Ref issue #6837) El request se ejecuta dentro de un timeout.
		//  Esto se realiza asi, para darle tiempo a ES que actualice el contacto, cuando se ha resuelto un conflicto, mediante un merge.
		var timeout_ms = 1000;
		window.setTimeout(function() {
			inConcert.app.getInteractionManager().searchArchiveInteractions(filter, function(status, result) {
				if (status){
					callback(result.interactions);
				}
				else {
					inConcert.app.m_mainWindow.showBannerMessage({ message : result && result.reason || '', timeout : 5000 });
					callback(null, result);
				}
			},this.from_app);
		}, timeout_ms);
	};

	/**
	 * Get the browsing history the specified client id.
	 *
	 * @todo - Move this function to another file
	 *
	 * @param  {String}   client_id   - The client identificator.
	 * @param  {Function} callback    - Callback is not optional
	 */
	ClientSidebar.prototype.get_browsing_history = function(callback = $.noop) {
		console.debug("[ClientSidebar.get_browsing_history] Loading browsing history. interaction: " + this.m_interaction.id);

		let client_id = this.m_interaction && this.m_interaction.getClientId() || "";

		if (!client_id) {
			setTimeout(function() { callback({}); }, 0);
			return;
		}

		this.m_request_browsing_history_timestamp_ms = new Date().getTime();
		this.m_interaction.getBrowsingHistory(function(history, error) {
			callback(history, error);
		});
	};

	ClientSidebar.prototype.__get_canned_plain_message = function(canned)
	{
		if (typeof canned !== 'object') {
			return "";
		}
		var plain_message = canned.plain_message;
		if (!plain_message) {
			plain_message = inConcert.Utils.htmlBodyToPlainText(canned.message || '');
		}
		return this.__replace_vars(plain_message);
	};

	ClientSidebar.prototype.get_canned_format_type = function(canned)
	{
		if (typeof canned !== 'object') {
			return "";
		}
		let format_type;
		if (this.m_interaction.type === "CHAT" || this.m_interaction.type === "MAIL") {
			// Los canales de WEB CHAT y MAIL se visualizan en el formato que se haya configurado en administration.
			format_type = canned.format;
		}
		else {
			// Todos los demas canales los visualizan en formato TEXT PLAIN.
			format_type = "TEXT PLAIN";
		}
		return this.m_canned_formats_list[format_type] || this.m_canned_formats_list["TEXT PLAIN"];
	};

	ClientSidebar.prototype.__replace_vars = function(message)
	{
		if (typeof message !== 'string') {
			return "";
		}
		message = inConcert.Utils.replace_vars({
			text: message,
			data: {interaction: this.m_interaction, campaign_id: this.m_interaction.campaign}
		});
		return message || "";
	};

	ClientSidebar.prototype.setup_canned_response_search = function()
	{
		const self = this;
		const $search_box    = $('.cannedFilterSearch');
		const $search_button = $('.butCannedFilter');
		const $reset_button  = $('.butCannedFilterReset');
		const $cbo_group     = $('#client_sidebar_cbo_canned_groups');

		var search = function() {
			// hide previous results
			self.$m_canned_container.find('.cannedItem').hide().data('satisfiesFilter', false);
			let list;
			let current_group = $cbo_group.val() || '';
			if (current_group.length === 0) {
				list = self.m_canned_messages; // all canned messages
			}
			else {
				list = self.m_canned_groups[current_group]; // group canneds (solo id)
			}
			let filter_regex = self.make_canned_search_regexp($search_box.val());

			$.each(list || {}, function(canned_id, canned) {
				if (typeof canned === 'boolean') {
					// si la lista viene por grupo, viene solo el id, y tengo que obtener el objeto
					canned = self.m_canned_messages[canned_id];
					if (typeof canned !== 'object') {
						return true; // continue
					}
				}
				let plain_message = self.__get_canned_plain_message(canned);
				let text_to_search = inConcert.Utils.removeSpecialChars(plain_message + canned.name);
				if (text_to_search.search(filter_regex) !== -1) {
					const $row = self.$get_canned_row(canned_id);
					$row.show().data('satisfiesFilter', true);
				}
			});
			const disabled_reset_button = $search_box.val().length === 0 && current_group.length === 0;
			$reset_button.prop('disabled', disabled_reset_button);
			self.truncate_visibility_canned_messages();
		};

		var reset_search = function() {
			$search_box.val('');
			$search_button.prop('disabled', true);

			if ($cbo_group.val() === '') {
				$reset_button.prop('disabled', true);
				$("div[id^='" + self.prefix_canned_id + "']").show().data("satisfiesFilter", true);
				self.truncate_visibility_canned_messages();
			}
			else {
				search();
			}
		};

		$cbo_group
			.unbind('change')
			.bind('change', function() {
				// el search se realiza en el timeout, porque sino la opcion del combo queda colgada mientras se realiza la busqueda
				var elapsed_time_ms = 15;
				window.setTimeout(function() {
					search();
				}, elapsed_time_ms);
			});

		// bind filter click button
		$search_button
			.unbind('click')
			.bind('click', function () {
				search();
			});

		//reset button click handler
		$reset_button
			.unbind('click')
			.bind('click', function () {
				$cbo_group.val('');
				reset_search();
			});

		let search_timeout;
		$search_box
			.unbind('keyup')
			.bind('keyup', function (e) {
				if (search_timeout) {
					window.clearTimeout(search_timeout);
					search_timeout = undefined;
				}
				if ($search_box.val()) {
					if (e.keyCode === 27) { //Escape reset search
						reset_search();
						return;
					}
					// se encuentra en un timeout de 500 ms, para que no realice la busqueda mientras escribis.
					const elapsed_time_ms = 500;
					search_timeout = window.setTimeout(function() {
						window.clearTimeout(search_timeout);
						search_timeout = undefined;
						search();
						$search_button.prop('disabled', false);
					}, elapsed_time_ms);
				} else {
					if (e.keyCode === 8 || e.keyCode === 46) { // backspace and no text, reset search.
						reset_search();
						return;
					}
					$search_button.prop('disabled', true);
				}
			}
		);
	};

	ClientSidebar.prototype.abort_request_member = function() {
		if (!this.request_members) {
			return;
		}
		this.request_members.abort();
		this.request_members = null;
	};

	ClientSidebar.prototype.show_cc_integration_view = function() {
		const $panel = $('#cc_integration_panel');
		const can_show = this.can_show_cc_integration();
		if (can_show) {
			$panel.show(); // show view
			return;
		}
		$panel.hide();
	};

	ClientSidebar.prototype.show_helpdesk_integration_view = function() {
		const $panel = $('#helpdesk_integration_panel');
		const can_show = this.can_show_helpdesk_integration();
		if (can_show) {
			$panel.show(); // show view
			return;
		}
		$panel.hide();
	};

	ClientSidebar.prototype.show_mas_integration_view = function() {
		const $panel = $('#mas_integration_panel');
		const can_show = this.can_show_mas_integration();
		if (can_show) {
			$panel.show(); // show view
			this.__render_mas_integration();
			return;
		}
		$panel.hide();
	};

	ClientSidebar.prototype.show_dynamic_integration_view = function(widget_id) {
		if (!widget_id) {
			return;
		}
		const $panel = $(FORMAT('#{0}_widget_integration_panel', widget_id));
		const can_show = this.can_show_dynamic_integration(widget_id);
		if (can_show) {
			$panel.show(); // show view
			return;
		}
		$panel.hide();
	};

	ClientSidebar.prototype.__render_mas_integration = function(/*must_reload*/)
	{
		const self = this;

		let log_preffix = "-----> [ClientSidebar:__render_mas_integration] ";

		if (this.m_interaction.isBeingComposed()) {
			console.debug(log_preffix + "Unable to render MAS - Outbound Interaction is being Composed. Interaction Id: '" + this.m_interaction.id + "'");
			return;
		}

		//console.debug(log_preffix + "ENTER - Interaction Id: '" + this.m_interaction.id + "' must_reload: " + must_reload);

		if (this.m_mas_render.status === "RENDERING" && this.m_mas_render.interactionId === this.m_interaction.id) {
			//console.debug(log_preffix + "return already rendering. Interaction Id: " + this.m_interaction.id);
			return;
		}
		if (this.m_mas_render.status === "SUCCESS" && this.m_mas_render.interactionId === this.m_interaction.id) {
			//console.debug(log_preffix + "return already render. Interaction Id: " + this.m_interaction.id);
			return;
		}

		const $waiting_container = $("#mas_integration_waiting");
		const $view_container = $("#mas_integration_view");

		var show_waiting = function() {
			//console.debug(log_preffix + "-> show_waiting");
			self.m_mas_render.status = "RENDERING";
			self.m_mas_render.interactionId = self.m_interaction.id;
			$view_container.hide();
			$waiting_container.show();
		};

		var show_integration = function() {
			//console.debug(log_preffix + "-> show_integration");
			self.m_mas_render.status = "SUCCESS";
			self.m_mas_render.interactionId = self.m_interaction.id;
			$waiting_container.hide();
			$view_container.show();
		};

		show_waiting();

		const $iframe = $("#mas_integration_iframe");
		var set_uri_to_iframe = function(uri) {
			//console.debug(log_preffix + "-> set_uri_to_iframe " + uri);
			if (!uri) {
				//console.debug(log_preffix + "-> Invalid MAS Uri");
				uri = "";
			}
			$iframe.attr("src", uri);
			// TODO: revisar si es posible obtener feedback de la carga del iframe.
			show_integration();
		};

		const mas_integration = inConcert.app.get_vcc_app_integration_config("mas");
		if (mas_integration.AppRender && mas_integration.AccessToken) {
			let mas_app_uri = mas_integration.AppRender.Url;
			let mas_access_token = mas_integration.AccessToken.Token;
			mas_app_uri += FORMAT("?interaction_id={0}&token={1}", encodeURIComponent(this.m_interaction.id), encodeURIComponent(mas_access_token));
			set_uri_to_iframe(mas_app_uri);
		}
		else {
			console.warn(log_preffix + "-> Bad MAS Configuration ");
		}
	};

	ClientSidebar.prototype.show_main_tab = function() {
		/*
			Cuando me dan click sobre el tab "main" (lo selecciono), tengo que invocar al metodo "adjust_panel_positions" de la clase base "ClientWindow"
			para que este se encargue de ajustar la posicion top del elemento con la clase "clientOffsetBottom" para que acomode el body segun el header.
			Ref #8469
		*/
		this.owner.adjust_panel_positions();
	};

	ClientSidebar.prototype.show_contactform_view = function() {
		var self = this;
		$('#contactform_panel').show(); // show view

		this.m_interaction.getParentInteraction( function( parent_interaction ) {
			var html_content = '';
			if ( parent_interaction ) {
				var add_html_control = function( name, value )
				{
					html_content = html_content +
						"<label class='activityText'>" +
							"<span class='activityAction'>" + (name || '')  + "</span>" +
							"<label>" + (value || "") + "</label>" +
						"</label>";
				};

				// contact id
				var contact_id = parent_interaction.getClientExternalId() || '';
				if ( contact_id.length > 0 ) {
					add_html_control( "<%=_T([[Contact Id]])%>", contact_id );
				}

				// contact name
				var contact_name = parent_interaction.getClientName() || '';
				if ( contact_name.length > 0 ) {
					add_html_control( "<%=_T([[Contact Name]])%>", contact_name );
				}

				// add addresses
				var addresses = parent_interaction.getExternalContactAddresses();
				$.each( addresses || {}, function( i, address ) {
					add_html_control( address.Type, address.displayable );
				});

				// add fields
				var fields = parent_interaction.getReportFields();
				$.each( fields || {}, function( i, field ) {
					add_html_control( field.displayName, field.value );
				});
			}

			if ( html_content.length === 0 ) {
				html_content = Mustache.render( no_data_available_tpl );
			}

			self.$m_contactform_container.html( html_content );
			if (!self.callback_contactform) {
				//Genero un Callback para que termine de levantar el ContactForm
				self.callback_contactform = true;
				self.wrapped_callback(true, null, "callback show_contactform_view");
			}
		});
	};

	ClientSidebar.prototype.$get_group_conversations = function( $container, conversation )
	{
		if ( !$container || !conversation.getGroupId ) {
			return false;
		}
		var group_conversation_id = conversation.getGroupId();
		return $container.find('[group_conversation_id="' + group_conversation_id + '"]' );
	};

	ClientSidebar.prototype.__sort_conversation_on_group = function( fc /* first conversation */, sc /* second conversation */ )
	{
		var date_compare = function() {
			return fc.currentTimeRaw() < sc.currentTimeRaw();
		};

		if ( fc.type === sc.type ) {
		// to the same type of conversations, the comparison is done by date
			return date_compare();
		}

		// types of conversations with greater weight, are located below
		// Ej: webcontact, go under every conversation type.
		var conversations_weight = {
			'WEBCONTACT': 100
			// "MAIL": 50   // sample
		};
		var fi_weight = conversations_weight[ fc.type ] || 0;
		var si_weight = conversations_weight[ sc.type ] || 0;

		if ( fi_weight === si_weight ) {
		// compared by date conversations with the same weight
			return date_compare();
		}
		// order according to the weight of the type of conversation
		return fi_weight > si_weight;
	};

	ClientSidebar.prototype.__insert_conversation_in_group = function( conversation, $conversation, $container )
	{
		var self = this;
		var belongsToSomeGroup = conversation.belongsToSomeGroup();
		if ( !belongsToSomeGroup ) {
			return false;
		}

		var $group = this.$get_group_conversations( $container, conversation );

		var response = { new_group: false, $group: false, sort_group: false };
		var group_conversation_id = conversation.getGroupId();
		var $interactions_group = $container.find( '[belongs_to_group="' + group_conversation_id + '"]' );

		if ( $interactions_group.length === 0 ) {
			// If there is no interaction in the group. Insert the first group interaction in the panel
			return false;
		}
		else if ( $interactions_group.length === 1 ) {
			$group = $( Mustache.render( group_conversations_tpl, { group_conversation_id: group_conversation_id } ) );
			$group.data( 'interaction', conversation ); // data from the first conversation are used to sort the group node list
			response.new_group = true;

			$.each( $interactions_group, function( i, dom_it ) {
				var $it = $( dom_it );
				$it.detach();
				$it.attr( 'belongs_to_group', group_conversation_id );
				$group.append( $it );
			});
		}

		var inserted = false;
		var group_conversations = $group.children();
		$.each( group_conversations, function( i, group_conversation )
		{
			var $group_conversation = $( group_conversation );
			var current_conversation = $group_conversation.data('interaction');
			if ( self.__sort_conversation_on_group( current_conversation, conversation ) ) {
				$group_conversation.before( $conversation );
				if ( i === 0 ) {
					$group.data('interaction', conversation); // data from the first conversation are used to sort the group node list
					response.sort_group = true;
				}
				inserted = true;
				return false; // break
			}
		});

		if ( !inserted ) {
			$group.append( $conversation );
		}

		response.$group = $group;
		return response;
	};

	ClientSidebar.prototype.__insert_conversation_in_panel_list = function ( conversation, $conversation, $container )
	{
		var group_response = this.__insert_conversation_in_group( conversation, $conversation, $container );
		if ( !group_response ) {
		// insert a simple new conversation in list panel
			$container.append( $conversation );
		}
		else if ( group_response.new_group ) {
		// insert a new group conversation in list panel
			$container.append( group_response.$group );
		}
		// TODO: reordenar grupo en la lista, con los datos de la primera interaccion.
		// group_response.sort_group
	};

	ClientSidebar.prototype.__$build_conversation_node = function( conversation, membersById )
	{
		var self = this;

		var belongs_to_group = '';
		if ( conversation.belongsToSomeGroup() ) {
			belongs_to_group = conversation.getGroupId();
		}

		var get_abstract = function() {
			var abstract_length = 70;
			var conversation_abstract = inConcert.Utils.escapeAttrib( conversation.getExtendedInfo() || '' );
			if ( conversation_abstract.length <= abstract_length ){
				return conversation_abstract;
			}
			return conversation_abstract.substring( 0, abstract_length ) + ' ...';
		};

		var last_agent = conversation.getLastAgentEndsConversation();
		var member = membersById[ last_agent ];

		var normalized_conversation_id = inConcert.Utils.getNormalizedId(conversation.id);

		var conversation_view = {
			id                 : 'client_sidebar_prev_conv_' + normalized_conversation_id        ,
			agent              : member ? member.Name : last_agent                               ,
			activity_date      : this.m_interaction_gui.__get_interaction_last_date(conversation),
			'abstract'         : get_abstract()                                                  ,
			additional_classes : this.m_interaction_gui.get_interaction_classes( conversation )  ,
			belongs_to_group   : belongs_to_group
		};

		var $conversation = $( Mustache.render( item_conversation_tpl, conversation_view ) );
		$conversation.data('interaction', conversation);

		$conversation.dblclick(function()
		{
			inConcert.app.getMainWindow().getPanel('AssignedInteractions').openInteractionClient(
				conversation,
				$(this),
				$.extend( {
					owner    : self,
					from_app : self.from_app
				}, { archive:true } )
			);
			$conversation.removeClass('activityItem').addClass('activityItem');
		});

		return $conversation;
	};

	ClientSidebar.prototype.show_conversations_view = function( current_view_id )
	{
		var self = this;

		$('#conversations_panel').show(); // show view

		var conversations = this.m_previous_conversations || [];
		if ( !conversations || conversations.length === 0 ) {
			this.$m_conversations_container.html(Mustache.render(no_data_available_tpl));
			return;
		}
		this.$m_conversations_container.find(".noDataAvailable").remove();

		this.request_members = inConcert.app.getTeamMembers(
			function(members)
			{
				self.request_members = null;
				if ( !members ) {
					return;
				}
				if (self.m_current_view !== current_view_id){
					return;
				}

				// Si existen en el DOM interacciones previas, las elimino.
				var previous_conversations_prefix = 'div[id^="client_sidebar_prev_conv_"]';
				var $previous_items = self.$m_conversations_container.find(previous_conversations_prefix);
				if ($previous_items.length > 0) {
					$previous_items.remove();
				}

				var membersById = {};
				for(var i = 0; i < members.length; i++) {
					membersById[members[i].Id] = members[i];
				}
				membersById[inConcert.app.getLoggedUserId()] = { Name : "<%=_T([[Me]])%>" };

				// build every previous conversation
				conversations.forEach( function( interaction )
				{
					if ( interaction ) {
						var $conversation_node = self.__$build_conversation_node( interaction, membersById );
						self.__insert_conversation_in_panel_list( interaction, $conversation_node, self.$m_conversations_container );
					}
				});
			},
			true // from cache
		);
	};

	ClientSidebar.prototype.show_browsing_view = function()
	{
		$('#browsing_panel').show(); // show view

		if ( !this.m_browsing_history || !this.m_browsing_history.map){
			this.$m_browsing_container.html(Mustache.render(no_data_available_tpl));
			return;
		}

		var length = this.$m_browsing_container.find('.activityItem').length;
		if (length > 0) {
			return;
		}

		var browsing_list = _.map(this.m_browsing_history, function(item) {
			return {
				url          : item.page,
				activity_date: inConcert.Utils.timeAgo(item.timestamp * 1000)
			};
		});

		if (browsing_list.length === 0) {
			this.$m_browsing_container.html(Mustache.render(no_data_available_tpl));
		} else {
			this.$m_browsing_container.html('');
		}

		// show the most recent first
		browsing_list.reverse();

		this.$m_browsing_container.append(Mustache.render(browsing_history_tpl, { params: browsing_list }));
	};

	ClientSidebar.prototype.show_context_info = function()
	{
		const self = this;

		$('#context_panel').show(); // show view

		var reset_jquery_container = function(jquery_selector, groups_tpl) {
			let $objectContainer = self.$m_context_container.find(jquery_selector);
			if (!$objectContainer.length) {
				self.$m_context_container.append(Mustache.render(groups_tpl));
			} 
			$objectContainer.html("");
			return true;
		};

		if (this.m_contact_addresses) {
		
			reset_jquery_container(".contactInfoContainer",contactinfo_groups_tpl);
			var contactinfo_container = this.$m_context_container.find('.contactInfoContainer');

			// contact id
			var contact_id = this.m_interaction.getClientExternalId() || '';
			if ( contact_id.length > 0 ) {
				contactinfo_container.append(
					Mustache.render( item_contactinfo_tpl, { name : "<%=_T([[Contact Id]])%>", value : contact_id } )
				);
			}

			// contact name
			var contact_name = this.m_interaction.getClientName() || '';
			if ( contact_name.length > 0 ) {
				contactinfo_container.append(
					Mustache.render( item_contactinfo_tpl, { name : "<%=_T([[Contact Name]])%>", value : contact_name } )
				);
			}

			//Validar si ocultar las direciones
			const visualize_address = inConcert.app.canVisualizeAddress(null, this.m_campaign_id, null, true);
			
			// contact addresses
			this.m_contact_addresses.forEach(function( contact_address ) {
				contactinfo_container.append(
					Mustache.render( item_contactinfo_tpl, { name : _T(contact_address.Type), value : visualize_address? contact_address.displayable: "<%=_T([[Hidden Address]])%>" } )
				);
			});
		}

		if (this.m_login) {
			reset_jquery_container(".loginContainer",login_groups_tpl);
			var login_container = this.$m_context_container.find( '.loginContainer' );
			
			this.m_login.forEach( function( field ) {
				if (field.hidden) {
					return;
				}
				login_container.append(
					Mustache.render( item_login_tpl, { question : field.displayName, answer : field.value } )
				);
			});
			if (!login_container.children().length) {
				this.$m_context_container.find( '.loginTitle' ).hide();
			}
		}

		if (this.m_pre_survey) {
			reset_jquery_container(".preSurveyContainer",survey_groups_tpl);
			var survey_container = this.$m_context_container.find('.preSurveyContainer');
			
			this.m_pre_survey.forEach(function(field) {
				if (field.hidden) {
					return;
				}
				survey_container.append(
					Mustache.render( item_survey_tpl, { question : field.displayName, answer : field.value } )
				);
			});
			if (!survey_container.children().length) {
				this.$m_context_container.find( '.preSurveyTitle' ).hide();
			}
		}

		if (this.m_params){
			reset_jquery_container(".paramsContainer .paramItem",params_groups_tpl);
			var params_container = this.$m_context_container.find('.paramsContainer');
			
			this.m_params.forEach(function(field){
				if (field.hidden) {
					return;
				}
				params_container.append(
					Mustache.render( item_params_tpl, { alias: field.displayName, value: field.value } )
				);
			});
			if (!params_container.children().length) {
				this.$m_context_container.find( '.paramsTitle' ).hide();
			}
		}

		if (this.m_post_survey) {
			reset_jquery_container(".postSurveyContainer",post_survey_groups_tpl);
			var pos_survey_container = this.$m_context_container.find('.postSurveyContainer');
			
			this.m_post_survey.forEach(function(field) {
				if (field.hidden) {
					return;
				}
				pos_survey_container.append(
					Mustache.render( item_post_survey_tpl, { question : field.displayName, answer : field.value } )
				);
			});
			if (!pos_survey_container.children().length) {
				this.$m_context_container.find( '.postSurveyTitle' ).hide();
			}
		}

		if (this.m_batch) {
			reset_jquery_container(".batchAgentDataContainer",batch_groups_tpl);
			var batch_container = this.$m_context_container.find('.batchAgentDataContainer');
			
			this.m_batch.forEach(function(field) {
				if (field.hidden) {
					return;
				}
				batch_container.append(
					Mustache.render( item_batch_tpl, { name : field.displayName || field.name, value : field.value } )
				);
			});
			if (!batch_container.children().length) {
				this.$m_context_container.find( '.batchAgentDataTitle' ).hide();
			}
		}

		if (this.m_process_flow && this.m_process_flow.filter(o => o.hidden === false).length > 0) {
			reset_jquery_container(".processFlowDataContainer",process_flow_groups_tpl);
			var process_flow_container = this.$m_context_container.find('.processFlowDataContainer');
			
			this.m_process_flow.forEach(function(field) {
				if (field.hidden) {
					return;
				}
				process_flow_container.append(
					Mustache.render( item_process_flow_tpl, { name : field.displayName || field.name, value : field.value } )
				);
			});

			self.__update_bot_supervisor_information();

			if (!process_flow_container.children().length) {
				this.$m_context_container.find( '.processFlowDataTitle' ).hide();
			}
		}

		if (this.$m_context_container.find('div').length === 0) {
			this.$m_context_container.html( Mustache.render( no_data_available_tpl ) );
		}
	};

	ClientSidebar.prototype.show_assistance_info = function()
	{
		const self = this;

		$('#assistance_panel').show(); // show view

		var reset_jquery_container = function(jquery_selector, groups_tpl) {
			let $objectContainer = self.$m_assistance_container.find(jquery_selector);
			if (!$objectContainer.length) {
				self.$m_assistance_container.append(Mustache.render(groups_tpl));
			} 
			$objectContainer.html("");
			return true;
		};

        reset_jquery_container(".assistanceDataContainer", assistance_groups_tpl);
        self.__update_bot_supervisor_information();

		if (this.$m_assistance_container.find('div').length === 0) {
			this.$m_assistance_container.html( Mustache.render( no_data_available_tpl ) );
		}
	};

	ClientSidebar.prototype.__update_bot_supervisor_information = function () {

		let self = this;

		var assistance_container = self.$m_assistance_container.find('.assistanceDataContainer');

		let internal_consultation_owner_without_vcc = "";
		if (self.m_interaction.interactionData.internal_consultation_owner &&
			self.m_interaction.interactionData.internal_consultation_owner !== undefined &&
			self.m_interaction.interactionData.internal_consultation_owner.split("@").length > 1) {
				internal_consultation_owner_without_vcc = self.m_interaction.interactionData.internal_consultation_owner.match(/(.+)@([^@]+)$/)[1];
		}

		$("#" + self.m_interaction.interactionData.internal_consultation_id + "_container").remove();
		$("#" + self.m_interaction.interactionData.internal_consultation_id + "_response_container").remove();
		$("#" + self.m_interaction.interactionData.internal_consultation_id + "_button_take").parent().remove();
		$("#" + self.m_interaction.interactionData.internal_consultation_id + "_taken_for_another_agent").remove();
		$("#" + self.m_interaction.interactionData.internal_consultation_id + "_time_off").remove();
		
		if (self.m_interaction.interactionData.internal_consultation_flag && self.m_interaction.state === "CREATED") {

			window.setTimeout(function() {
                const assistance_component = self.getLayoutComponent("client_sidebar_assistance_tab");
                self.__select_tab(assistance_component);
            }, 2000);
			let response = self.__calculateTimeLeft(self.m_interaction.interactionData.internal_consultation_startdate, 
				self.m_interaction.interactionData.internal_consultation_timeout);

			self.__updateTimeLeft(self.m_interaction.interactionData.internal_consultation_startdate,
				self.m_interaction.interactionData.internal_consultation_timeout,
				self.m_interaction.interactionData.internal_consultation_id);
			
			if (internal_consultation_owner_without_vcc === inConcert.app.getLoggedUserId()) {
				
				assistance_container.append(					
					Mustache.render(item_process_flow_internal_consultation, {
						name : '<%=_T([[The bot needs help:]])%>',
						value : self.m_interaction.interactionData.internal_consultation_question,
						question_id: self.m_interaction.interactionData.internal_consultation_id,
						time_left: response.time_left,
						progress: response.progress
					})
				);

				$("#" + self.m_interaction.interactionData.internal_consultation_id + "_container").parent().css("display", "grid");
				$("#" + self.m_interaction.interactionData.internal_consultation_id + "_container").parent().css("height", "calc(100% - 50px)");

				if (response.time_left_seconds > 0) {
					
					assistance_container.append(					
						Mustache.render(item_process_flow_internal_consultation_response, {
							question_id: self.m_interaction.interactionData.internal_consultation_id,
                            allowStoringInKB: self.m_interaction.interactionData.internal_consultation_kb ? true : false
						})
					);
								
					$("#" + self.m_interaction.interactionData.internal_consultation_id + "_button_take").parent().hide();

					$("#" + self.m_interaction.interactionData.internal_consultation_id + "_button").click(function() {
						self.__sendMessageInternal();
					});
				} else {
					assistance_container.append(					
						Mustache.render(item_process_flow_internal_consultation_time_off, {
							question_id: self.m_interaction.interactionData.internal_consultation_id
						})
					);
				}
				
			} else if (internal_consultation_owner_without_vcc && internal_consultation_owner_without_vcc !== inConcert.app.getLoggedUserId()) {
				if (self.m_interaction.interactionData.internal_consultation_last_question && 
                    self.m_interaction.interactionData.internal_consultation_last_response) 
                {
                    assistance_container.append(					
                        Mustache.render(item_process_flow_internal_consultation_history, {
                            last_question : self.m_interaction.interactionData.internal_consultation_last_question,
                            last_response : self.m_interaction.interactionData.internal_consultation_last_response,
                            last_response_user : self.m_interaction.interactionData.internal_consultation_last_response_user
                        })
                    );
                }
				assistance_container.append(					
					Mustache.render(item_process_flow_internal_consultation_taken, {
						question_id: self.m_interaction.interactionData.internal_consultation_id,
						agent_name: internal_consultation_owner_without_vcc
					})
				);

			} else {
				assistance_container.append(					
					Mustache.render(item_process_flow_internal_consultation, {
						name : '<%=_T([[The bot needs help:]])%>',
						value : self.m_interaction.interactionData.internal_consultation_question,
						question_id: self.m_interaction.interactionData.internal_consultation_id,
						time_left: response.time_left,
						progress: response.progress
					})
				);

				$("#" + self.m_interaction.interactionData.internal_consultation_id + "_container").parent().css("display", "grid");
				$("#" + self.m_interaction.interactionData.internal_consultation_id + "_container").parent().css("height", "calc(100% - 50px)");
				$("#" + self.m_interaction.interactionData.internal_consultation_id + "_button_take").parent().show();				
				$("#" + self.m_interaction.interactionData.internal_consultation_id + "_button_take").click(function() {
					self.__takeBotInteraction();
				});
			}
		} else if (self.m_interaction.interactionData.internal_consultation_last_question && 
			self.m_interaction.interactionData.internal_consultation_last_response
		) 
		{
            window.setTimeout(function() {
                const assistance_component = self.getLayoutComponent("client_sidebar_assistance_tab");
                self.__select_tab(assistance_component);
            }, 2000);

			assistance_container.append(					
				Mustache.render(item_process_flow_internal_consultation_history, {
					last_question : self.m_interaction.interactionData.internal_consultation_last_question,
					last_response : self.m_interaction.interactionData.internal_consultation_last_response,
					last_response_user : self.m_interaction.interactionData.internal_consultation_last_response_user
				})
			);
			
		}
	}

	ClientSidebar.prototype.__updateTimeLeft = function (startdate, timeout, question_id) {

		var self = this;
		clearInterval(window.bot_supervisor_interval);
		window.bot_supervisor_interval = setInterval(function () {
			let response = self.__calculateTimeLeft(startdate, timeout);
			$("#" + question_id + "_time_left").text(response.time_left);
			$("#" + question_id + "_progress").width(response.progress);

			if (response.time_left_seconds <= 0) {
				$("#" + self.m_interaction.interactionData.internal_consultation_id + "_button_take").parent().remove();
				$("#" + self.m_interaction.interactionData.internal_consultation_id + "_response_container").remove();

				if ($("#" + self.m_interaction.interactionData.internal_consultation_id + "_time_off").length === 0 &&
				$("#" + self.m_interaction.interactionData.internal_consultation_id + "_taken_for_another_agent").length === 0) {
					var assistance_container = self.$m_assistance_container.find('.assistanceDataContainer');
					assistance_container.append(					
						Mustache.render(item_process_flow_internal_consultation_time_off, {
							question_id: self.m_interaction.interactionData.internal_consultation_id
						})
					);
				}
			}
			
		}, 1000);
		
	}

	ClientSidebar.prototype.__calculateTimeLeft = function (startdate, timeout) {

		var diferenciaEnMilisegundos = (timeout * 1000) - (Date.now() - startdate * 1000);
		let time_left = "0 min 0 seg";
		let progress = "100%";
		let time_left_seconds = 0;

		if (diferenciaEnMilisegundos > 0) {
			var porcentajeRestante = (((timeout * 1000) - diferenciaEnMilisegundos) / (timeout * 1000)) * 100;
			var minutos = Math.floor(diferenciaEnMilisegundos / (60 * 1000));
			var segundos = Math.floor((diferenciaEnMilisegundos % (60 * 1000)) / 1000);				
			time_left = `${minutos.toString().padStart(2, '0')} min ${segundos.toString().padStart(2, '0')} seg`;
			progress = porcentajeRestante + "%";
			time_left_seconds = diferenciaEnMilisegundos / 1000;
		}

		return {
			time_left: time_left,
			progress: progress,
			time_left_seconds: time_left_seconds
		}

	}

	ClientSidebar.prototype.__takeBotInteraction = function () {

		const self = this;
		const app_id = this.owner.from_app;

		this.owner.displayStatusMessage("<%=_T('Taking...')%>", "Process", true);

		const take_callback = function( taked, messageError )
		{
			const main_window = inConcert.app.getMainWindow();
			if ( taked ) {
				self.owner.is_archive = false;
				self.owner.is_queued = false;
				self.owner.is_waiting = false;
				self.owner.is_readonly = false;
				self.owner.hideStatusMessage();
				self.owner.displayStatusMessage("<%=_T('Taked successfully')%>", "Info", false, 3000);

				window.setTimeout(function() {
					self.__update_bot_supervisor_information();
				}, 1500);
			}
			else {
				//handle error
				self.owner.hideStatusMessage();
				main_window.showBannerMessage({message: messageError, messageType: "Error", timeout: 5000 }, null, true);
				self.owner.displayStatusMessage(messageError, "Error", false, 5000);

				window.setTimeout(function() {
					self.__update_bot_supervisor_information();
				}, 1500);
			}
		};

		this.m_interaction.take_bot_interaction( take_callback, app_id );
		
	}

	ClientSidebar.prototype.__sendMessageInternal = function () {

		var self = this;
		
		if (self && 
			self.m_interaction &&
			self.m_interaction.type === "CHAT" &&
			self.m_interaction.m_session &&
			self.m_interaction.m_session.m_client) 
		{
			self.m_interaction.m_session.m_client.SendMessageInternal(self.m_interaction.m_session.m_sessionId,
				$("#" + self.m_interaction.interactionData.internal_consultation_id + "_response").val(),
                $("#" + self.m_interaction.interactionData.internal_consultation_id + "_add_to_qna").prop('checked'),
                function() {
					clearInterval(window.bot_supervisor_interval);
					$("#" + self.m_interaction.interactionData.internal_consultation_id + "_response_container").remove();
					$("#" + self.m_interaction.interactionData.internal_consultation_id + "_progress_total").remove();
					$("#" + self.m_interaction.interactionData.internal_consultation_id + "_progress_detail").remove();
					
					window.setTimeout(function() {
						self.__update_bot_supervisor_information();
					}, 1500);
				});
		} else if (self && 
					self.m_interaction && 
					self.m_interaction.type === "WHATSAPP" && 
					self.m_interaction.m_view) {

			self.m_interaction.m_view.SendMessageInternal(
				$("#" + self.m_interaction.interactionData.internal_consultation_id + "_response").val(),
                $("#" + self.m_interaction.interactionData.internal_consultation_id + "_add_to_qna").prop('checked'), 
                function() {
					clearInterval(window.bot_supervisor_interval);
					$("#" + self.m_interaction.interactionData.internal_consultation_id + "_response_container").remove();
					$("#" + self.m_interaction.interactionData.internal_consultation_id + "_progress_total").remove();
					$("#" + self.m_interaction.interactionData.internal_consultation_id + "_progress_detail").remove();
					
					window.setTimeout(function() {
						self.__update_bot_supervisor_information();
					}, 1500);
				});
		} else {
			self.owner.displayStatusMessage("<%=_T('Could not send message at this time, please try again...')%>", "Error", false, 5000);
		}
	}

	ClientSidebar.prototype.__update_flow_information = function () {

		if (!this.m_process_flow) {
			return;
		}

		let $flowContainer = this.$m_context_container.find(".processFlowDataContainer");
		if (!$flowContainer.length) {
			this.$m_context_container.append(Mustache.render(process_flow_groups_tpl));
			$flowContainer = this.$m_context_container.find(".processFlowDataContainer");
		}
		else {
			$flowContainer.html("");
		}
			
		this.m_process_flow.forEach(function (field) {
			if (field.hidden) {
				return;
			}
			$flowContainer.append(
				Mustache.render(item_process_flow_tpl, { name: field.displayName || field.name, value: field.value })
			);
		});

		let $title = $flowContainer.parent();

		if (!$flowContainer.children().length) {
			$title.hide();
		}
		else {
			$title.show();
		}

		if (!this.$m_context_container.find('div').length) {
			this.$m_context_container.html(Mustache.render(no_data_available_tpl));
		}
		else {
			this.$m_context_container.find(".noDataAvailable").remove();
		}
	};

	ClientSidebar.prototype.onInformationUpdated = function()
	{
		if (!this.m_interaction) {
			return;
		}
		
		if (this.m_interaction.isUpdateTransferBot()) {
			this.__update_flow_information();
		}
	};
	ClientSidebar.prototype.update_canned_messages = function(canned_messages, differences)
	{
		var self = this;
		this.m_canned_messages = canned_messages;

		$.each(differences, function(canned_id, difference) {
			switch(difference.type) {
				case 'NEW':
					var canned = difference.row;
					var sorted = true;
					var filter = true;
					self.new_row_canned(canned_id, canned, sorted, filter);
				break;

				case 'UPDATE':
					var udpated_canned = difference.updated_row;
					var old_canned = difference.old_row;
					self.update_row_canned(canned_id, udpated_canned, old_canned);
				break;

				case 'DELETE':
					var deleted_canned = difference.deleted_row;
					self.erase_row_canned(canned_id, deleted_canned.group);
				break;
			}
		});

		this.truncate_visibility_canned_messages();
	};

	ClientSidebar.prototype.add_group_to_canned_combo = function(value_id, selected, canned_id)
	{
		this.m_canned_groups = this.m_canned_groups || {};
		if (!this.m_canned_groups.hasOwnProperty(value_id)) {
			this.m_canned_groups[value_id] = {};
			var $option = $("<option></option>").attr('value', value_id).html(value_id);
			if (selected) {
				$option.attr('selected', 'selected');
			}
			$('#client_sidebar_cbo_canned_groups').append($option);
		}
		if (canned_id) {
			this.m_canned_groups[value_id][canned_id] = true;
		}
		this.canned_enable_or_disable_combo_groups();
	};

	// TO-DO: Translate comments to English.
	ClientSidebar.prototype.update_group_canned_combo = function(canned_id, new_group, old_group)
	{
		if (old_group === new_group) {
			return;
		}
		var exist_old_group_in_hash = this.m_canned_groups.hasOwnProperty(old_group);
		var exist_new_group_in_hash = this.m_canned_groups.hasOwnProperty(new_group);

		if (!exist_old_group_in_hash) {
			if (exist_new_group_in_hash) {
			// 1 - el old no existe y el nuevo si existe.
			/*
				- agrego canned en new hash.
				- me voy.
			 */
				this.m_canned_groups[new_group][canned_id] = true;
				return;
			}
			// 2 - el old no existe y el nuevo no existe.
			/*
				- creo new grupo en hash.
				- agrego canned en new hash.
				- agrego new group en cbo.
				- me voy
			 */
			this.add_group_to_canned_combo(new_group, false, canned_id);
			return;
		}
		else {
			delete this.m_canned_groups[old_group][canned_id];
			if ($.isEmptyObject(this.m_canned_groups[old_group])) {
				if (!exist_new_group_in_hash) {
					// 3 - el old existe (con un item), y el nuevo no existe.
					/*
						- elimino canned del hash del old group.
						- elimino old group del hash.
						- creo new group en hash
						- agrego canned en new hash.
						- actualizo new group en cbo.
					 */
					delete this.m_canned_groups[old_group];
					this.m_canned_groups[new_group] = {};
					this.m_canned_groups[new_group][canned_id] = true;
					$('#client_sidebar_cbo_canned_groups option[value="' + old_group + '"]')
						.attr('value', new_group).html(new_group);
					return;
				}
				else {
					// 4 - el old existe (con un item), y el nuevo si existe.
					/*
						- elimino canned del hash del old group.
						- elimino old group del hash.
						- elimino old group del cbo
						- agrego canned en new hash.
					 */
					delete this.m_canned_groups[old_group];
					this.m_canned_groups[new_group][canned_id] = true;
					$('#client_sidebar_cbo_canned_groups option[value="' + old_group + '"]').remove();
					this.canned_enable_or_disable_combo_groups();
					return;
				}
			}
			else {
				if (!exist_new_group_in_hash) {
					// 5 - el old existe (con mas de un item), y el nuevo no existe.
					/*
						- elimino canned del hash del old group.
						- creo new group en hash
						- agrego canned en new hash.
						- agrego new group del cbo.
					 */
					this.add_group_to_canned_combo(new_group, false, canned_id);
					return;
				}
				else {
					// 6 - el old existe (con mas de un item), y el nuevo si existe.
					/*
						- elimino canned del hash del old group.
						- agrego canned en new hash.
						- me voy.
					 */
					this.m_canned_groups[new_group][canned_id] = true;
					return;
				}
			}
		}
	};

	ClientSidebar.prototype.delete_group_canned_combo = function(canned_id, group)
	{
		if (this.m_canned_groups && this.m_canned_groups.hasOwnProperty(group)) {
			delete this.m_canned_groups[group][canned_id];
			if ($.isEmptyObject(this.m_canned_groups[group])) {
				delete this.m_canned_groups[group];
				$('#client_sidebar_cbo_canned_groups option[value="' + group + '"]').remove();
			}
		}
		this.canned_enable_or_disable_combo_groups();
	};

	ClientSidebar.prototype.sanitize_canned_id = function(canned_id) { return canned_id.replace(new RegExp('^' + this.prefix_canned_id + '(.*?)$', 'i'), '$1'); };
	ClientSidebar.prototype.$get_canned_row    = function(canned_id) { return $('#' + this.prefix_canned_id + canned_id); };

	ClientSidebar.prototype.format_canned_name = function(canned, lower) {
		if (!canned.group) {
			return canned.name;
		}
		var key = (canned.group || '') + ' - ' + (canned.name || '');
		if (!lower) {
			return key;
		}
		return key.toLowerCase();
	};

	// This function will enforce truncated visibility for canned messages
	// That is, it will only show X number of messages in screen at any time
	// Refs #3479
	ClientSidebar.prototype.truncate_visibility_canned_messages = function(max_visible)
	{
		var self = this;
		var DEFAULT_MAX_VISIBLE = 4;
		max_visible = max_visible || DEFAULT_MAX_VISIBLE;

		var visibleCount = 0;
		this.$m_canned_container.find('.cannedItem').each(function(i, e) {
			var $e = $(e);
			if ($e.data('satisfiesFilter')) {
				$e.toggle(visibleCount < max_visible);
				visibleCount++;
			}
		});

		var button = $('#showMoreCannedButton');

		if (visibleCount > max_visible) {
			// must show button with text 'show more messages'
			if (button.length === 0) {
				//create button if not exist
				this.$m_canned_container.append(Mustache.render(show_more_canned_messages_tpl));
				button = $('#showMoreCannedButton');
			}
			button.unbind('click').click(function () {
				self.truncate_visibility_canned_messages(max_visible + DEFAULT_MAX_VISIBLE);
			});
			button.show();
		}
		else {
			$('#showMoreCannedButton').hide();
		}
	};

	ClientSidebar.prototype.add_sorted_row_canned = function(canned_id, canned, $new_row)
	{
		var self = this;
		var added = false;
		var lower = true;
		var new_sorted_key = this.format_canned_name(canned, lower);
		this.$m_canned_container.find('.cannedItem').each(function(i, row)
		{
			var $row = $(row);
			var canned_id = self.sanitize_canned_id($row.attr('id'));
			var canned_data = self.m_canned_messages[canned_id];
			if (canned_data) {
				var current_sorted_key = self.format_canned_name(canned_data, lower);
				if (new_sorted_key <= current_sorted_key) {
					$row.before($new_row);
					added = true;
					return false; // break
				}
			}
		});
		if (!added) {
			this.$m_canned_container.append($new_row);
		}
	};

	ClientSidebar.prototype.make_canned_search_regexp = function(word) {
		if (typeof word !== 'string') {
			return '';
		}
		var search_pattern = inConcert.Utils.removeSpecialChars(word);
		return new RegExp(search_pattern.replace(/[^\w\s]/gi, ''), 'i');
	};

	ClientSidebar.prototype.filter_row_canned = function($row, canned)
	{
		// (filter) - valido si los nuevos valores del row matchean con el filtro
		var $search_combo = $('#client_sidebar_cbo_canned_groups');
		var filter_group = $search_combo.val() || '';
		if (filter_group.length > 0 && filter_group.toUpperCase() !== canned.group.toUpperCase()) {
		// si en el combo de filtro de categoria, existe una seleccionada, y ha cambiado de categoria, esconde la row.
			$row.hide();
			return;
		}
		var search_text = $('.cannedFilterSearch').val();
		if (search_text.length === 0) {
		// si no hay texto buscado en el filtro, no filtro la row
			return;
		}
		var search_regexp = this.make_canned_search_regexp(search_text);
		var plain_message = this.__get_canned_plain_message(canned);
		var text_to_search = inConcert.Utils.removeSpecialChars((plain_message) + (canned.name || ''));
		if (text_to_search.search(search_regexp) === -1) {
		// si el mensaje o el name ha cambiado, y no matchea con el filtro, escondo la row.
			$row.hide();
		}
	};

	ClientSidebar.prototype.can_include_canned = function(canned)
	{
		const self = this;
		if (typeof canned !== 'object') {
			return false;
		}

		const validate_include_sanitized_message = function() {
			const plain_message = self.__get_canned_plain_message(canned);
			if (plain_message.trim().length === 0) {
				return false;
			}
			return true;
		};

		let canned_format_type = canned.format;
		let canned_format_def = this.m_canned_formats_list[canned_format_type];
		if (!canned_format_def) {
			return false;
		}

		let channel = this.m_interaction.type;
		let channelsInfo = canned_format_def.channels;
		if (channelsInfo.supported_in === "SOME") {
			if (channelsInfo.allowed[channel]) {
				return validate_include_sanitized_message();
			}
			return false;
		}
		else if (channelsInfo.supported_in === "ALL") {
			return validate_include_sanitized_message();
		}
		else {
			throw("ClientSidebar.prototype.can_include_canned - Unrecognized canned supported_in: " + channelsInfo.supported_in);
		}
	};

	ClientSidebar.prototype.new_row_canned = function(canned_id, canned, sorted, filter)
	{
		let can_include = this.can_include_canned(canned);
		if (!can_include) {
			return;
		}
		this.add_group_to_canned_combo(canned.group, false, canned_id);
		const plain_message = this.__get_canned_plain_message(canned);
		const format_type = this.get_canned_format_type(canned);
		let canned_view = {
			id           : this.prefix_canned_id + canned_id,
			activity_type: this.format_canned_name(canned),
			message      : inConcert.Utils.truncateWord(plain_message, 150),
			format_type  : format_type.desc
		};

		if (this.m_interaction.type !== "CHAT") {
			// El canal de MAIL siempre se visualiza en formato HTML.
			canned_view.add_msg = "<%=_T([[Add]])%>";
			canned_view.add_class = "far fa-plus";
		}
		else {
			// El canal de WEB CHAT se visualiza en el formato que se haya configurado en administration.
			if (format_type.id === "HTML") {
				canned_view.add_msg = "<%=_T([[Send]])%>";
				canned_view.add_class = "far fa-paper-plane";
			}
			else {
				canned_view.add_msg = "<%=_T([[Add]])%>";
				canned_view.add_class = "far fa-plus";
			}
		}

		const $new_row = $(Mustache.render(item_tpl_canned, canned_view));
		if (sorted) {
			this.add_sorted_row_canned(canned_id, canned, $new_row);
		}
		else {
			this.$m_canned_container.append($new_row);
		}
		const can_show_info_by_channel = {
			MAIL: true,
			CHAT: true
		};
		if (can_show_info_by_channel[this.m_interaction.type]) {
			$new_row.find('[action_show="true"]').show();
		}
		const hidden_add_by_channel = {
			MAIL: true
		};
		if (hidden_add_by_channel[this.m_interaction.type]) {
			$new_row.find('[action_add="true"]').hide();
		}
		$new_row.data('satisfiesFilter', true);
		if (filter) {
			// filtra la row
			this.filter_row_canned($new_row, canned);
		}
	};

	ClientSidebar.prototype.update_row_canned = function(canned_id, new_canned, old_canned)
	{
		let exist_old_canned_in_list  = this.can_include_canned(old_canned);
		let can_include = this.can_include_canned(new_canned);
		if (!can_include) {
			if (exist_old_canned_in_list) {
				this.erase_row_canned(old_canned.canned_id, old_canned.group);
			}
			return;
		}
		if (!exist_old_canned_in_list) {
			const sorted = true;
			const filter = true;
			this.new_row_canned(canned_id, new_canned, sorted, filter);
			return;
		}
		var plain_message = this.__get_canned_plain_message(new_canned);
		var format_type = this.get_canned_format_type(new_canned);
		var canned_view = {
			activity_type: this.format_canned_name(new_canned),
			message      : inConcert.Utils.truncateWord(plain_message, 150),
			format_type  : format_type.desc
		};

		var $row = this.$get_canned_row(canned_id);
		$row.find('.activityText').html(Mustache.render(item_tpl_canned_data, canned_view));

		// filtra la row
		this.filter_row_canned($row, new_canned);

		if ((old_canned.group !== new_canned.group) || (old_canned.name !== new_canned.name)) {
			// si el grupo o el name cambio, tengo que cambiarlo de posicion dentro de la table (alfabeticamente)
			var $removed_row = $row;
			$removed_row.remove();
			this.add_sorted_row_canned(canned_id, new_canned, $row);
		}
		this.update_group_canned_combo(canned_id, new_canned.group, old_canned.group);
	};

	ClientSidebar.prototype.erase_row_canned = function(canned_id, group)
	{
		var $row = this.$get_canned_row(canned_id);
		if ( $row.length > 0 ){
			$row.remove();
		}
		this.delete_group_canned_combo(canned_id, group);
	};

	ClientSidebar.prototype.__open_canned_popup = function ($parent, canned, can_add)
	{
		var self = this;
		this.m_canned_popup = new CannedPopup({from_app: this.from_app, canned: canned, owner: this, can_add: can_add});
		this.m_canned_popup.show({
			parentNode: $parent,
			callbacks: {
				onAdd: function(canned) {
					self.onSidebarSetCannedMessage(canned);
				}
			}
		});
	};

	ClientSidebar.prototype.show_canned_view = function(reload)
	{
		var self = this;

		$('#canned_panel').show(); // show view

		var canned_prefix_selector = 'div[id^="' + this.prefix_canned_id + '"]';
		var previous_canned = this.$m_canned_container.find(canned_prefix_selector);
		if (reload === true) {
			$('.butCannedFilterReset').trigger('click');
			previous_canned.remove();
		}
		else {
			if ( previous_canned.length ) {
				return;
			}
		}

		// Setup search
		this.setup_canned_response_search();
		// build every canned message, order msgs by name
		var arr_sorted_canneds = this.m_canned_messages ? _.sortBy(this.m_canned_messages, function(msg){ return self.format_canned_name(msg, true); }) : {};

		this.m_canned_groups = {};
		$('#client_sidebar_cbo_canned_groups').html('');
		this.add_group_to_canned_combo('', true); // la categoria en blanco

		$.each(arr_sorted_canneds, function(i, canned)
		{
			var canned_id = canned.canned_id || i;
			self.new_row_canned(canned_id, canned);
		});

		this.truncate_visibility_canned_messages();

		var get_canned = function(canned_id) {
			canned_id = self.sanitize_canned_id(canned_id || '');
			return self.m_canned_messages[canned_id];
		};

		var add_event_selector = 'button[action_add]';
		var show_event_selector = 'button[action_show]';

		$('#canned_panel')
			.off('click', add_event_selector)
			.on('click', add_event_selector, function() {
				var canned_id = $(this).parent().parent().attr("id");
				var canned = get_canned(canned_id);
				if (canned) {
					self.onSidebarSetCannedMessage(canned);
				}
			})
			.off('click', show_event_selector)
			.on('click', show_event_selector, function() {
				const $show_button = $(this);
				const $canned_row = $show_button.parent().parent();
				const canned_id = $canned_row.attr("id");
				const canned = get_canned(canned_id);
				if (canned) {
					let can_add = false;
					const $add_button = $canned_row.find(add_event_selector);
					if ($add_button && $add_button.length) {
						can_add = $add_button.is(':visible');
					}
					self.__open_canned_popup($show_button, canned, can_add);
				}
			});

		this.onSidebarChangeTab();
	};

	ClientSidebar.prototype.canned_enable_or_disable_combo_groups = function()
	{
		var $lbl_search_combo = $('#client_sidebar_cbo_canned_groups_lbl');
		var $search_combo = $('#client_sidebar_cbo_canned_groups');
		var groups_length = $search_combo.find( 'option' ).length;
		var is_empty = groups_length <= 1; // un item se toma como empty porque es el group en blanco.

		if ( is_empty ) {
			$lbl_search_combo.addClass('cannedMessagesLabel-ds');
			$search_combo.addClass('readonly').prop( 'disabled', true );
		}
		else {
			$lbl_search_combo.removeClass('cannedMessagesLabel-ds');
			$search_combo.removeClass('readonly').prop( 'disabled', false );
		}
	};

	ClientSidebar.prototype.on_tab_selected = function(tab_id, force_select = false)
	{
		const self = this;

		var change_view = function(view) {
			if (self.m_current_view === tab_id && !force_select) {
				return false;
			}

			const tab_container_id = typeof(view.tab_container_id) === "function" ? view.tab_container_id() : view.tab_container_id;
			const $tab_container = $("#" + tab_container_id);
			if ($tab_container.length > 0 && view.class_name) {
				$tab_container.removeClass().addClass(view.class_name);
			}
			self.m_current_view = tab_id;
			self.__hide_all_containers();

			// TODO: This is out of scope, this *view* function does not know anything about a request
			// this request needs to be triggered on initialize method.
//			self.abort_request_member();

			// show view
			if (typeof view.method === "function") {
				view.method(tab_id);
			}
			else {
				self[view.method](tab_id);
			}
			
			return true;
		};

		let tab_config = this.m_tabs[tab_id];
		if (!tab_config) {
			return false;
		}

		return change_view(this.m_tabs[tab_id]);
	};

	ClientSidebar.prototype.__setup_buttons_handlers = function()
	{
		var self = this;

		$('#sidebar_refresh_trackme').unbind('click').bind('click', function(){
			self.__reloadHistoryBrowsing();
		});

		$('#sidebar_resize').unbind('click').bind('click', function(){
			self.resize();
		});
	};

	ClientSidebar.prototype.update_sidebar_tabs = function(/*invoked_from = "UNRECOGNIZED"*/)
	{
		const self = this;
		const is_readonly = this.isReadonlyLayout();

		//console.debug("");
		//console.debug("-START (update_sidebar_tabs)-------------------------------------------------------------------");
		//console.info("[ClientSidebar.prototype.update_sidebar_tabs] - UPDATING SIDEBAR tabs - (interaction.id: " + this.m_interaction.id + ", state: " + this.m_interaction.state + ") is_readonly " + is_readonly + ", invoked_from: " + invoked_from);

		if (!this.m_layout) {
			console.debug("-ERROR (update_sidebar_tabs)--------------------------------------------------------------------");
			//console.debug("");
			return;
		}

		var get_first_stack = function(content = []) {
			let first_stack;
			let len = content.length - 1;

			for (let i = len; i >= 0; i--) {
				let item = content[i];

				//console.info("     get_first_stack -- item.type " + item.type + ", item.id " + item.id + ", item.content " + (item.content ? item.content.length : "none") + ", item.contentItems " + (item.contentItems ? item.contentItems.length : "none"));

				if (item.type === "stack") {
					first_stack = item;
				}
				else if (item.type === "row") {
					first_stack = get_first_stack(item.contentItems);
				}

				if (first_stack) {
					break;
				}
			}
			return first_stack;
		};

		let passed_tabs = {};
		let visible_tabs = [];
		let i = 0;

		var update_sidebar_tabs = function() 
		{
			for (let passed_tab_id in passed_tabs) {
				if (passed_tabs.hasOwnProperty(passed_tab_id)) {
					let passed_tab = passed_tabs[passed_tab_id];
					let $tab_node = passed_tab.$tab_node;
					let result = passed_tab.result;
					let tab = passed_tab.tab;
					let component = passed_tab.component;
					let is_visible_tab_node = passed_tabs.is_visible_tab_node;

					if ($tab_node) {
						/*
							Cuando el Plugin de GoldenLayout se esta creando "__creating === true", el "status" debe ser "SHOW",
							es decir, se deben de crear todos los TABs visibles, ya que sino luego cambian de orden.
							En una segunda actualizacion de tabs y cuando "__creating === false", ahi si se debe de respetar el valor que venga en status,
							para asi poder ocultar los tabs que sean necesarios.
							El error sino sucede cuando la ventana se debe abrir maximizada (configuracion de la cuenta).
						*/
						if (self.m_layout.__creating) {
							result.status = "SHOW";
						}
						if (is_visible_tab_node) {
							//console.info("      ClientSidebar.prototype.update_sidebar_tabs -   1.01 - show - is already visible. tab.id " + tab.id);
							visible_tabs[tab.id] = visible_tabs[tab.id] || {
								tab_id    : tab.id,
								component : component,
								is_hidden : false,
								is_dynamic: tab.is_dynamic
							};
						}
						else {
							let can_add = true;
							if (component) {
								//console.info("      ClientSidebar.prototype.update_sidebar_tabs -   1.1 - show. tab.id " + tab.id);
								if (is_readonly) {
									component.isClosable = false;
								}
								else {
									component.isClosable = true;
								}
								component.show();
								visible_tabs[tab.id] = {
									tab_id    : tab.id,
									component : component,
									is_hidden : false,
									is_dynamic: tab.is_dynamic
								};
							}
							if (can_add) {
								$tab_node.show();
							}
						}
						// Aca oculto el TAB del layout
						if (result.status !== "SHOW") {
							//console.info("      ClientSidebar.prototype.update_sidebar_tabs -   2 - hide -> tab.id " + tab.id);
							$tab_node.hide();
							visible_tabs[tab.id].is_hidden = true;
						}
					}
				}
			}

			self.validateIsReady(function(is_ready) {
				if (is_ready) {
					// Ahora selecciono el mejor Tab de los disponibles
					self.__select_best_first_tab(visible_tabs, function() {
						// Dejo el callback por si tengo que relaizar algo con los visible_tabs[tab.id].is_hidden = true;
					});
				}
			});
		};

		let total_tabs = inConcert.Utils.objectCount(this.m_tabs);

		$.each(this.m_tabs, function(_, tab)
		{
			let $tab_node, component;

			$tab_node = self.$getLayoutTab(tab.id);
			component = self.getLayoutComponent(tab.id);

			let tab_node_len, is_visible_tab_node;
			if ($tab_node) {
				tab_node_len = $tab_node.length;
				is_visible_tab_node = $tab_node.is(":visible");
			}
			else {
				tab_node_len = 0;
				is_visible_tab_node = false;
			}

			i++;
			//console.info("  - " + i + " - - -  - - - - - - " + tab.id);
			//console.info("    ClientSidebar.prototype.update_sidebar_tabs -   1 - tab.id '" + tab.id +
			//	"', $tab_node '" + tab_node_len +
			//	"', is_visible_tab_node '" + is_visible_tab_node +
			//	"', component '" + component + "'");

			// funcion asincronica.
			tab.it_can_be_shown(function(result)
			{
				//console.info("       --> it_can_be_shown -> result: " + JSON.stringify(result));

				passed_tabs[tab.id] = {
					$tab_node: $tab_node,
					result: result,
					is_visible_tab_node: is_visible_tab_node,
					component: component,
					tab: tab
				};
				// como la funcion "it_can_be_shown" puede o no ser asincronica, tengo que validar que el total de tabs corresponda con los passed para poder continuar.
				if (total_tabs === inConcert.Utils.objectCount(passed_tabs)) {
					update_sidebar_tabs();
				}
			});
		});

		//console.debug("--END (update_sidebar_tabs)-------------------------------------------------------------------");
		//console.debug("");
	};

	/*
		Esta funcion, emula un click sobre algun TAB del plugin GoldenLayout.
		Hubo que revisar el codigo del plugin para implementarlo de forma manual, ya que el mismo tiene un BUG y esto no funciona.
		@param component: es un componente del GoldelLayout.

		Referencias:
			https://github.com/golden-layout/golden-layout/issues/299
			https://github.com/golden-layout/golden-layout/issues/430

		El codigo se reviso en la funcion: "setActiveContentItem" del archivo "\lib\golden-layout\1.5.9\scripts\goldenlayout.js"
	*/
	ClientSidebar.prototype.__select_tab = function(component)
	{
		if (!component || !component.tab) {
			return false;
		}
		component.tab._onTabClick({button: 0, type: "touchstart", forceActivated: true});
		return true;
	};

	ClientSidebar.prototype.__select_main_tab = function(callback = $.noop)
	{
		const self = this;
		const main_component = this.getLayoutComponent("main");
		if (!main_component) {
			return;
		}
		// Pequeno timeout para que de tiempo a la carga del layout (plugin). Quizas el timeout deba aumentar !!
		window.setTimeout(function() {
			self.__select_tab(main_component);
			callback();
		}, 500);
	};

	ClientSidebar.prototype.__select_best_first_tab = function(visible_tabs, callback = $.noop)
	{
		const self = this;

		const select_tab = function(tab_id)
		{
			if (!visible_tabs[tab_id]) {
				return false;
			}
			const component = visible_tabs[tab_id].component;
			const is_selected = self.__select_tab(component);
			if (visible_tabs[tab_id].is_hidden) {
				return false;
			}
			return is_selected;
		};
		/*
			Al map lo debo de pasar a un array en orden inverso a lo que se ha definido.
			Para despues recorrerlo y seleccionarlo (**).
			Si no lo recorro en orden inverso al momento de seleccionarlo, los tabs se mueven de lugar y quedan en cualquier orden.
			Ej: el Tab 'information' que es el primero que debe de visualizarse (sin integraciones), queda en cualquier otro orden.
		*/
		let reversed_visible_tabs = [];
		for (let tab_id in visible_tabs) {
			if (visible_tabs.hasOwnProperty(tab_id)) {
				let tab = visible_tabs[tab_id];
				tab.__tab_id = tab_id;
				reversed_visible_tabs.push(tab);
			}
		}
		reversed_visible_tabs.reverse();
		/*---------------------------------------------------
			(**) Debo recorrer todos los tabs y seleccionarlos, ya que sino se visualizan en blanco.
			Esto es por un supuesto Bug del Plugin.
		---------------------------------------------------*/
		reversed_visible_tabs.forEach(tab => {
			if (tab.__tab_id !== "main"){
				select_tab(tab.__tab_id);
			}
		});
		/*---------------------------------------------------
			Ahora tengo que seleccionar si o si los tabs que tienen que quedar con el focos.
			Segun si se visualizan o no.
		---------------------------------------------------*/
		// CC integration
		if (!select_tab("client_sidebar_cc_integration_tab")) {
			// MAS integration
			if (!select_tab("client_sidebar_mas_integration_tab")) {
				// HelpDesk integration
				if (!select_tab("client_sidebar_helpdesk_integration_tab")) {
					//-------
					// Dynamic Widgets Integrations
					let is_selected_widget = false;
					for (let i = 0; i < reversed_visible_tabs.length; i++) {
						let tab = reversed_visible_tabs[i];
						if (tab.is_dynamic) {
							is_selected_widget = select_tab(tab.__tab_id);
							if (is_selected_widget) {
								break;
							}
						}
					}
					//--------
					if (!is_selected_widget) {
						// context info
						select_tab("client_sidebar_context_tab");
					}
				}
			}
		}
		// Despues de seleccionar el tab de la sidebar, tengo que dejar selected (dar el foco) en el tab main
		//   ya que sino el tab queda en blanco y no se visualizan sus controles
		this.__select_main_tab(callback);
	};

	ClientSidebar.prototype.dispose = function()
	{
		this.events_unsuscribe('ALL');
		if (this.hasConfiguredLayout() && this.m_layout) {
			this.m_layout.destroy();
		}
		if (this.m_layout_design) {
			this.m_layout_design.dispose();
		}
	};

	ClientSidebar.prototype.events_unsuscribe = function( event_id )
	{
		event_id = event_id || 'ALL';

		if (event_id === 'CONTACTS' || event_id === 'ALL') {
			// unsubscribe contact changes
			if (this.pubsub_contact_updated_handle) {
				inConcert.app.messagebus.unsubscribe(this.pubsub_contact_updated_handle);
				this.pubsub_contact_updated_handle = null;
			}
		}
	};

	/**
	 * Initialize method will fetch all the data needed before show anything
	 *
	 * @public
	 * @param  {String}   campaign_id
	 * @param  {String}   client_id
	 * @param  {Function} callback
	 */
	ClientSidebar.prototype.initialize = function(campaign_id, client_id, callback)
	{
		const self = this;
		var at_least_one_success = false;

		//console.info("");
		console.info("ClientSidebar.initialize -1-------> INITIALIZING SIDEBAR - START. Interaction: " + this.m_interaction.id + ", state: " + this.m_interaction.state);

		if (!this.already_rendered) {
			//console.debug("ClientSidebar.initialize -2");
			// Build sidebar markup
			this.build_sidebar();

			/*
				Esta funcion tiene que ser invocada en los callbacks de "initializeLayout" y "TabDataRetrieved", ya que puede llegar a en diferentes tiempos y una necesita de la otra.
			*/
			var initialized = function(invoked_from) {
				/*
				La siguiente linea comentada, queda solo como referencia mientras estamos en la construccion de la nueva interfaz + el golden layout.
				Una vez que se haya culminado el mismo, todo este comentario tiene que ser eliminado.
					Ref: http://redmine.inconcert/issues/8585

					self.initializeLayout();
				*/
				self.update_sidebar_tabs(invoked_from);
				self.__setup_buttons_handlers();
				self.initialized = true;
				self.emit( self.emit_show_id );
				if (self.m_layout) {
					callback(true);
				}
			};

			// Instancia e inicializa el Plugin "Golden Layout" (***)
			this.initializeLayout(function() {
				let invoked_from = "CALLBACK of initializeLayout";
				//console.debug("ClientSidebar.initialize -2-------> INITIALIZING SIDEBAR - " + invoked_from + " - Interaction: " + self.m_interaction.id);
				initialized(invoked_from);
			});
			// update the UI when tab data is available (****)
			this.on('TabDataRetrieved', function() {
				if (self.m_layout) {
					self.m_layout.__creating = false;
				}
				let invoked_from = "CALLBACK of TabDataRetrieved";
				//console.debug("ClientSidebar.initialize -3-------> INITIALIZING SIDEBAR - " + invoked_from + " - Interaction: " + self.m_interaction.id);
				initialized(invoked_from);
			});
		}

		// browsing callback
		var callback_browsing = function(browsing_history, error) {
			at_least_one_success = self.callback_browsing(browsing_history, error, at_least_one_success);
		};

		// canned callback
		var callback_canneds = function(canned_messages, error) {
			if ( error ) {
				self.wrapped_callback(false, error, "callback_canneds 1");
				return false;
			}
			if ( canned_messages && !$.isEmptyObject(canned_messages) ) {
				at_least_one_success = true;
				self.m_canned_messages = canned_messages;
			}
			if (!self.m_interaction.isBeingComposed()) {
				// get user browsing history
				self.get_browsing_history(callback_browsing);
			}
			return true;
		};

		// conversations callback
		var callback_conversations = function(conversations, error) {
			if ( error ) {
				self.wrapped_callback(false, error.result, "callback_conversations 1");
				return;
			}
			if ( conversations ){
				at_least_one_success = true;
				self.m_previous_conversations = conversations;
			}
			if (!self.m_interaction.isFinished()) {
				// go and get canned messages
				var campaign = inConcert.app.getCampaign(campaign_id);
				if ( campaign === null || campaign === undefined ) {
					self.wrapped_callback(false, null, "callback_conversations 2");
					return false;
				}
				campaign.getCannedMessages(callback_canneds);
			}
			else {
				// get user browsing history
				self.get_browsing_history(callback_browsing);
			}
		};

		if ( this.m_interaction.isBeingComposed() ) {
			this.load_canneds(campaign_id, function() {
				self.wrapped_callback(true, null, "callback canned commposed");
			});
		}
		else {
			this.get_previous_conversations(campaign_id, client_id, callback_conversations);
		}
	};

	ClientSidebar.prototype.wrapped_callback = function(success, msg, invoked_from = "unrecognized")
	{
		success = typeof(success) === "boolean" ? success : true;
		msg = msg || "no message";

		let method = success ? "info" : "error";
		console[method]("[ClientSidebar.wrapped_callback]  - success: " + success + ", msg: " + msg + ", invoked_from: " + invoked_from + ", interaction: " + this.m_interaction.id);

		this.emit('TabDataRetrieved');
	};

	ClientSidebar.prototype.has_configured_tracking = function()
	{
		// check if the campaign has tracking configured and the interaction
		// is from that account.
		let campaign = inConcert.app.getCampaign(this.m_campaign_id);
		if (campaign === null || campaign === undefined) {
			return false;
		}
		let has_configured = false;
		campaign.accounts.forEach(account => {
			if (account.Tracking === '1' && this.m_token === account.Token){
				has_configured = true;
			}
		});
		return has_configured;
	};

	ClientSidebar.prototype.has_configured_qna_maker = function(campaign_id /*optional*/)
	{
		if (!inConcert.app.has_configured_qna_maker_in_vcc()) {
			return false;
		}
		campaign_id = campaign_id || this.m_campaign_id;
		let campaign = inConcert.app.getCampaign(this.m_campaign_id);
		if (campaign === null || campaign === undefined) {
			return false;
		}
		let has_configured = false;
		(campaign.accounts || []).forEach(account => {
			if (account.id === this.m_interaction.account) {
				if (account.QnAMaker && account.QnAMaker.Enabled === "1") {
					has_configured = true;
				}
			}
		});
		return has_configured;
	};

	ClientSidebar.prototype.callback_browsing = function(browsing_history, error, at_least_one_success, callback, no_execute_wrapped_callback)
	{
		callback = callback || $.noop;
		at_least_one_success = at_least_one_success || false;

		if (error) {
			if (!no_execute_wrapped_callback) {
				this.wrapped_callback(false, error.result, "callback_browsing 1");
			}
			callback();
			return at_least_one_success;
		}
		if (browsing_history) {
			if (this.has_configured_tracking()) {
				at_least_one_success = true;
				this.m_browsing_history = browsing_history;
			}
		}
		// callback with the at_least_one_success flag
		// this is done to handle the case when we don't
		// have enough info to show the sidebar
		if (!no_execute_wrapped_callback) {
			this.wrapped_callback(at_least_one_success, null, "callback_browsing 2");
		}
		callback();
		return at_least_one_success;
	};

	ClientSidebar.prototype.load_canneds = function( campaign_id, callback )
	{
		console.debug("[ClientSidebar.load_canneds]  Loading canned messages. interaction: " + this.m_interaction.id);
		var self = this;
		callback = callback || $.noop;
		var campaign = inConcert.app.getCampaign(campaign_id);
		if ( campaign === null || campaign === undefined ) {
			callback();
			return false;
		}
		campaign.getCannedMessages(function(canned_messages, error) {
			if ( error ) {
				callback();
				return;
			}
			if ( canned_messages ) {
				self.m_canned_messages = canned_messages;
				self.on_tab_selected('client_sidebar_canned_tab');
				var reload = true;
				self.show_canned_view(reload);
			}
			callback();
		});
	};

	ClientSidebar.prototype.on_changed_campaign_account = function(new_campaign_id/*, new_account_id*/)
	{
		this.load_canneds(new_campaign_id);
		let invoked_from = "creating outgoing - on_changed_campaign_account";
		this.update_sidebar_tabs(invoked_from);
	};

	ClientSidebar.prototype.is_visible_tab = function(tab_id)
	{
		if (!this.m_tabs[tab_id]) {
			return false;
		}
		const $tab = $("#" + tab_id);
		if ($tab.length === 0) {
			return false;
		}
		return $tab.is(":visible");
	};

	ClientSidebar.prototype.total_visible_tabs = function()
	{
		let total_visible_tabs = 0;
		for (let tab_id in this.m_tabs) {
			if (this.m_tabs.hasOwnProperty(tab_id)) {
				if (this.is_visible_tab(tab_id)) {
					total_visible_tabs++;
				}
			}
		}
		return total_visible_tabs;
	};

	ClientSidebar.prototype.__can_show_integration = function()
	{
		if (inConcert.app.isMobile()) {
		/*
			NO visualizo la integracion si:
				Ha iniciado sesion desde un Movil.
		*/
			//console.debug("ClientSidebar.__can_show_integration -----> 1 - NO show - TAB integracion - interaction " + this.m_interaction.id + ". Integrations do not display for mobile.");
			return false;
		}

		if (!this.m_interaction.belongsToMe()) {
		/*
			NO visualizo la integracion si:
				La interaccion no me pertenece, es decir, el usuario NO se encuentra en los arrays (assignedAgents o requiredAgent, o reservedAgents)
		*/
			//console.debug("ClientSidebar.__can_show_integration -----> 2 - NO show - TAB integracion - interaction " + this.m_interaction.id + " does not belongs to me !!!");
			return false;
		}

		if (this.m_interaction.isIntegrationEnabled()) {
		/*
			SI visualizo la integracion si:
				El estado de la interaccion es "TAKEN" o "TRANSFER TAKEN" o "WRAPUP"
		*/
			//console.debug("ClientSidebar.__can_show_integration *-----> 3 - SI SHOW - TAB integracion - interaction " + this.m_interaction.id + ", state: " + this.m_interaction.state);
			return true;
		}

		if (this.m_interaction.isLocked()) {
			/*
				SI visualizo la integracion si:
					El estado de la interaccion es "LOCKED" o "TRANSFER LOCKED" o "CONFERENCE LOCKED"
			*/
			//console.debug("ClientSidebar.__can_show_integration *-----> 4 - SI SHOW (isLocked) - TAB integracion - interaction " + this.m_interaction.id + ", state: " + this.m_interaction.state);
			return true;
		}

		// if (this.m_interaction.isWrappedUp()) {
		// 	console.debug("ClientSidebar.__can_show_integration *----->5 - SI SHOW (isWrappedUp) - TAB integracion - interaction " + this.m_interaction.id + ", state: " + this.m_interaction.state);
		// 	return true;
		// }

		/*
			NO visualizo la integracion si:
		*/
		//console.debug("ClientSidebar.__can_show_integration -----> 6 - NO show - TAB integracion - interaction " + this.m_interaction.id + ", state: " + this.m_interaction.state);
		return false;
	};

	ClientSidebar.prototype.can_show_cc_integration = function()
	{
		if (!this.m_interaction.hasCCIntegration()) {
			//console.debug("ClientSidebar.can_show_cc_integration - 1 - NO show - TAB integracion - campaign " + this.m_interaction.campaign + ", interaction " + this.m_interaction.id);
			return false;
		}
		return this.__can_show_integration();
	};

	ClientSidebar.prototype.can_show_mas_integration = function()
	{
		if (!this.m_interaction.hasIntegrationWithMarketingAndSales()) {
			return false;
		}
		return this.__can_show_integration();
	};

	ClientSidebar.prototype.can_show_helpdesk_integration = function()
	{
		if (!this.m_interaction.hasIntegrationWithHelpDesk()) {
			return false;
		}
		return this.__can_show_integration();
	};

	ClientSidebar.prototype.can_show_dynamic_integration = function(widget_id)
	{
		if (!this.m_interaction.hasIntegrationWithOccWidget(widget_id)) {
			return false;
		}
		return this.__can_show_integration();
	};

	ClientSidebar.prototype.show = function()
	{
//		console.debug("ClientSidebar.prototype.show");
		const self = this;

		var show = function() {
			self.__setup_buttons_handlers();
			self.is_visible = true;
			self.refreshLayout();
			// Callback
			self.onSidebarShown();
		};

		if (this.initialized) {
			show();
		}
		else {
			this.once( this.emit_show_id, function() {
				var elapsed_time_ms = 300;
				window.setTimeout(function() {
					self.removeListener( self.emit_show_id, $.noop );
				},
				elapsed_time_ms);
				show();
			});
		}
	};

	ClientSidebar.prototype.hide = function() {
	};

	ClientSidebar.prototype.resize = function() {
	};

	ClientSidebar.prototype.isVisible = function() {
		return this.is_visible;
	};

	ClientSidebar.prototype.showActions = function() {
		const $canned_add = this.$m_canned_container.find('[action_add]');
		const $qna_add = this.$m_qna_maker_container.find('[action_add]');
		$canned_add.show();
		$qna_add.show();
	};

	ClientSidebar.prototype.hideActions = function() {
		const $canned_add = this.$m_canned_container.find('[action_add]');
		const $qna_add = this.$m_qna_maker_container.find('[action_add]');
		$canned_add.hide();
		$qna_add.hide();
	};

	/*
		El reload del browsing history se realiza cuando:
			1 - El tiempo transcurrido, desde que se realizo el ultimo request de history supera los N segundos (RELOAD_TRACKME_TIME_MS)
	*/
	ClientSidebar.prototype.__reloadHistoryBrowsing = function() {
		const self = this;

		if (!this.m_browsing_history) {
			//console.debug("ClientSidebar.prototype.__reloadHistoryBrowsing - > 1) No se realiza reload history. No se visualiza el TAB History.");
			// No realizo el Reload si: el load de la carga inicial, NO tenia configurado "browsing history" (es decir, si no se esta visualizando la solapa)
			return;
		}
		if (this.m_interaction.type !== "CHAT") {
			//console.debug("ClientSidebar.prototype.__reloadHistoryBrowsing - > 2) No se realiza reload history. No es un interaccion de WEBCHAT.");
			// No realizo el Reload si: NO es una interaccion de webchat
			return;
		}
		if (this.m_interaction.isFinished() || this.m_interaction.isWrappedUp()) {
			//console.debug("ClientSidebar.prototype.__reloadHistoryBrowsing - > 3) No se realiza reload history. La interaccion se encuentra en un estado FINISH o WRAPUP.");
			// No realizo el Reload si: la interaccion se encuentra en los estados de finish o wrapup
			return;
		}
		if (!this.has_configured_tracking()) {
			//console.debug("ClientSidebar.prototype.__reloadHistoryBrowsing - > 4) No se realiza reload history. La cuenta/campana no tiene configruado tracking.");
			// No realizo el Reload si: la cuenta en la campana, NO tiene configurado tracking
			return;
		}

		var end = function(message) {
			window.setTimeout(function() {
				if (message) {
					self.__show_message(message, "Info", 5000);
				}
				self.__hide_waiting("browsing");
			}, 500);
		};

		this.__show_waiting("browsing");

		if (this.m_request_browsing_history_timestamp_ms) {
			// Aca valido el tiempo transcurrido del ultimo request de "browsing history".
			let now = new Date();
			let elapsed_time_ms = now.getTime() - this.m_request_browsing_history_timestamp_ms;
			if (elapsed_time_ms < RELOAD_TRACKME_TIME_MS) {
				//console.debug("ClientSidebar.prototype.__reloadHistoryBrowsing - > 5) No se realiza reload history. No ha transcurrido el Intervalo de tiempo permitido entre request '" + parseInt(RELOAD_TRACKME_TIME_MS / 1000, 10) + "' segundos. Tiempo que ha transcurrido: '" + parseInt(elapsed_time_ms / 1000, 10) + "' segundos.");
				// No realizo el Reload si: desde que se realizo el ultimo request, no han transcurrido N segundos (RELOAD_TRACKME_TIME_MS)
				end("<%=_T([[There are no new records.]])%>");
				return;
			}
		}

		// Bueno, ahora si, realizo el request contra "browsing history"
		this.get_browsing_history(function(browsing_history, error) {
			if (!$.isArray(browsing_history)) {
				//console.debug("ClientSidebar.prototype.__reloadHistoryBrowsing - > A) Si realizo reload, pero NO hay info.");
				end();
				return;
			}
			let $old_items = self.$m_browsing_container.find(".activityItem");
			let total_old_items = $old_items.length;
			let total_current_items = browsing_history.length;

			if (total_old_items === total_current_items) {
				//console.debug("ClientSidebar.prototype.__reloadHistoryBrowsing - > B) Si realizo reload, pero NO hay nuevos items.");
				// NO realizo nada si la cantidad de rows que ha retornado el request, es la misma cantidad de rows que se visualizan en pantalla.
				end("<%=_T([[There are no new records.]])%>");
				return;
			}

			const callback_browsing = function() {
				let total_new_items = total_current_items - total_old_items;
				//console.debug("ClientSidebar.prototype.__reloadHistoryBrowsing - > C) ACTUALIZO LA LISTA. Se agregaran '" + total_new_items + "' nuevos items.");
				const force_select = true; //si ya me encuentro posicionado en el TAB, realizo el select a prepo para que se actualice.
				$old_items.remove(); // elimino los rows que se estaban visualizando, ya que voy a proceder a insertar los nuevos.
				self.on_tab_selected("client_sidebar_browsing_tab", force_select); // aca hago los inserts y demas.
				let message = FORMAT("<%=_T([[{0} new records have been added.]])%>", total_new_items);
				end(message);
			};
			const no_execute_wrapped_callback = true;
			self.callback_browsing(browsing_history, error, null, callback_browsing, no_execute_wrapped_callback);
		});
	};

	ClientSidebar.prototype.onMinimizeWindow = function() {
//		console.debug("ClientSidebar.prototype.onMinimizeWindow - > this.m_current_window_state: " + this.m_current_window_state);
		this.m_current_window_state = WINDOW_MINIMIZED;
	};

	ClientSidebar.prototype.onRestoreWindow = function() {
//		console.debug("ClientSidebar.prototype.onMinimizeWindow - > this.m_current_window_state: " + this.m_current_window_state);
		this.m_current_window_state = WINDOW_NORMAL;
	};

	ClientSidebar.prototype.onMaximizeWindow = function() {
//		console.debug("ClientSidebar.prototype.onMinimizeWindow - > this.m_current_window_state: " + this.m_current_window_state);
		this.m_current_window_state = WINDOW_MAXIMIZED;
	};

	ClientSidebar.prototype.show_qna_maker_view = function()
	{
		const $panel = $('#qna_maker_panel');
		$panel.show(); // show view
		// // Setup search
		this.setup_qna_maker_search();
	};

	ClientSidebar.prototype.showQnaMakerQuestionDetail = function ($parent, answer)
	{
		this.m_qna_popup = new QnAPopupDetail({from_app: this.from_app, answer: answer, owner: this});
		this.m_qna_popup.show({
			parentNode: $parent
		});
	};

	ClientSidebar.prototype.setup_qna_maker_search = function()
	{
		const self = this;

		const $panel = $('#qna_maker_panel');
		const $question = $panel.find('[qna_maker_question]');
		const $search_anwser = $panel.find('[qna_maker_search_anwser]');
		const $reset = $panel.find('[qna_maker_reset]');
		const qnAMakerAPI = inConcert.app.getQnAMakerAPI();

		this.m_hnd_qna_request = null;
		var search = function()
		{
			if (self.m_hnd_qna_request) {
				return;
			}
			const disabled_reset_button = $question.val().length === 0;
			$reset.prop('disabled', disabled_reset_button);

			const question = $question.val().trim();
			if (question.length === 0) {
				return;
			}

			self.__show_waiting("qna");
			let info = {
				question: question
				/* METADATA SAMPLE: si en algun momento necesitamos filtran con metadata, al request hay que agregarle la siguiente estructura */
				/*
					metadata: {
						filters: [
							{name: "ciudad", value: "colonia"},
							{name: "ciudad", value: "montevideo"}
						],
						operationType: "AND"
					}
				*/
			};
			self.m_hnd_qna_request = qnAMakerAPI.generateAnswer(info, function(success, resp)
			{
				self.m_hnd_qna_request = null;
				self.$m_qna_maker_container.html("");
				if (!success) {
					const message = "<%=_T([[Unable to retrieve answer.]])%>";
					self.__show_message(message, "Error", 5000);
					self.__hide_waiting("qna");
					return;
				}
				let total_answers_added = 0;
				let answers = resp.answers || [];
				answers.forEach(answer => {
					if (self.new_row_qna_maker_answer(answer)) {
						total_answers_added++;
					}
				});
				if (total_answers_added === 0) {
					$("#qna_maker_no_answer_found").show();
					self.$m_qna_maker_container.hide();
				}
				else {
					$("#qna_maker_no_answer_found").hide();
					self.$m_qna_maker_container.show();
				}
				self.__hide_waiting("qna");
			});
		};

		var reset_search = function() {
			$question.val('');
			$search_anwser.prop('disabled', true);
			$reset.prop('disabled', true);
			self.$m_qna_maker_container.html("");
			$("#qna_maker_no_answer_found").show();
			self.$m_qna_maker_container.hide();
		};

		// bind filter click button
		$search_anwser
			.unbind('click')
			.bind('click', function () {
				search();
			});

		//reset button click handler
		$reset
			.unbind('click')
			.bind('click', function () {
				$question.val('');
				reset_search();
			});

		let search_timeout;
		$question
			.unbind('keyup')
			.bind('keyup', function (e) {
				if (search_timeout) {
					window.clearTimeout(search_timeout);
					search_timeout = undefined;
				}
				if ($question.val()) {
					if (e.keyCode === 27) { //Escape reset search
						reset_search();
						return;
					}
					if (e.keyCode === 13) { //Enter execute search
						$search_anwser.prop('disabled', false);
						search();
						return;
					}
					// se encuentra en un timeout de 500 ms, para que no realice la busqueda mientras escribis.
					const elapsed_time_ms = 500;
					search_timeout = window.setTimeout(function() {
						window.clearTimeout(search_timeout);
						search_timeout = undefined;
						$search_anwser.prop('disabled', false);
					}, elapsed_time_ms);
				} else {
					if (e.keyCode === 8 || e.keyCode === 46) { // backspace and no text, reset search.
						reset_search();
						return;
					}
					$search_anwser.prop('disabled', true);
				}
			}
		);
	};

	ClientSidebar.prototype.get_qna_maker_score_css_class = function(score)
	{
		let score_class;
		const scores_def = [
			{"score":  20, "class_name": "veryLowScore" },
			{"score":  40, "class_name": "lowScore"     },
			{"score":  60, "class_name": "mediumScore"  },
			{"score":  80, "class_name": "highScore"    },
			{"score": 100, "class_name": "veryHighScore"}
		];
		for (let i = 0; i < scores_def.length; i++) {
			const score_def = scores_def[i];
			if (score <= score_def.score) {
				score_class = score_def.class_name;
				break;
			}
		}
		return score_class;
	};

	ClientSidebar.prototype.new_row_qna_maker_answer = function(answer)
	{
		const self = this;

		if (!answer || answer.score === 0) {
			return false;
		}

		const questions = (answer.questions || []);
		const top_shown_metadata_items = 1;
		let metadata_html = "";
		let metadata_tooltip = "";
		for (let i = 0; i < (answer.metadata || []).length; i++) {
			if (i > 0) {
				metadata_tooltip = metadata_tooltip + "\n";
			}
			let metadata_info = answer.metadata[i];
			if (i < top_shown_metadata_items) {
				let name_value_html = FORMAT("<span class='qnaMetadataText'>{0} | {1}</span>", metadata_info.name, metadata_info.value);
				metadata_html = metadata_html + name_value_html;
			}
			let name_value_tooltip = FORMAT("{0}: {1}", metadata_info.name, metadata_info.value);
			metadata_tooltip = metadata_tooltip + name_value_tooltip; 
		}
		const has_metadata = metadata_html.length > 0;

		const answer_view = {
			answer     : answer.answer,
			question   : inConcert.Utils.htmlBodyToPlainText(questions[0] || "" || ""),
			score      : answer.score,
			add_msg    : "<%=_T([[ADD]])%>",
			add_class  : "far fa-plus",
			view_msg   : "<%=_T([[SHOW]])%>",
			view_class : "far fa-eye",
			score_class: this.get_qna_maker_score_css_class(answer.score)
		};
		const $new_row = $(Mustache.render(item_tpl_qna_maker_pair, answer_view));
		$new_row.data("answer", answer);

		if (has_metadata) {
			const $metadata = $new_row.find("[metadata]");
			$metadata.show();
			metadata_tooltip = "<%=_T([[Metadata]])%>" + "<%=_T([[:]])%>" + "\n" + metadata_tooltip;
			$metadata.attr("title", metadata_tooltip);
			const $metadata_name_value_list = $new_row.find("[metadata_name_value_list]");
			$metadata_name_value_list.html(metadata_html);
		}

		let question_alternatives;
		let i = 0;
		answer.questions.forEach(question => {
			i++;
			if (i > 1) {
				if (i === 2) {
					question_alternatives = inConcert.Utils.htmlBodyToPlainText(question);
				}
				else {
					question_alternatives = question_alternatives + "\n" + inConcert.Utils.htmlBodyToPlainText(question);
				}
			}
		});

		if (question_alternatives) {
			const $question_alternatives = $new_row.find("[question_alternatives]");
			question_alternatives = "<%=_T([[Alternatives]])%>" + "<%=_T([[:]])%>" + "\n" + question_alternatives;
			$question_alternatives.attr("title", question_alternatives);
		}

		this.$m_qna_maker_container.append($new_row);

		const hidden_add_by_channel = {
			CALL: function() {
				return true;
			},
			MAIL: function() {
				return !self.owner.is_composing_mail();
			}
		};
		const hidden_fn = hidden_add_by_channel[this.m_interaction.type];
		if (hidden_fn) {
			if (hidden_fn()) {
				$new_row.find('[action_add="true"]').hide();
			}
		}

		const $panel = $('#qna_maker_panel');
		$panel.show(); // show view

		// // Setup search
		this.setup_qna_maker_search();

		// Eventos de las acciones
		const $qna_actions = $new_row.find("[qna_actions]");
		let event_selector = 'button[action_add]';
		$qna_actions
			.off('click', event_selector)
			.on('click', event_selector, function() {
				const $answer = $(this).parent().parent().parent().parent().parent();
				const answer = $answer.data("answer");
				if (answer) {
					self.onSidebarSetQnaAMakerAnswer(answer);
				}
			});

		event_selector = 'button[action_show]';
		$qna_actions
			.off('click', event_selector)
			.on('click', event_selector, function() {
				const $show_button = $(this);
				const $answer = $show_button.parent().parent().parent().parent().parent();
				const answer = $answer.data("answer");
				if (answer) {
					self.showQnaMakerQuestionDetail($show_button, answer);
				}
			});

		return true;
	};

	ClientSidebar.prototype.set_message = function(message)
	{
		if (!this.m_interaction || this.m_interaction.isFinished() || this.m_interaction.isWrappedUp() || this.m_interaction.isWaiting()) {
			return;
		}
		if (this.m_current_view === 'client_sidebar_qna_maker_tab') {
			const $panel = $('#qna_maker_panel');
			const $question = $panel.find('[qna_maker_question]');
			$question.val(message).trigger('keyup');
		}
	};

}());
