/*global jQuery*/
/*jshint strict:false, quotmark: false, smarttabs: true*/

(function(window, undefined) {

/**
Class FakeClientWindow.
It acts as a proxy in the meantime the actual ClientWindow is being created.
*/
function ClientWindowProxy ()
{
	this.m_queued_calls = [];
}

/**
Clones a value. If it is a object, takes care of its prototype too.
*/
function clone_single_argument(arg)
{
	if(typeof(arg) !== "object" || arg === null) {
		return arg;
	}
	var clone = Object.create( Object.getPrototypeOf(arg) );
	// we use jQuery extend because it deals correctly with cycles and stuff
	jQuery.extend(true, clone, arg);
	return clone;
}

/**
Clones an array of arguments.
*/
function clone_arguments(args)
{
	return args.map(function(elem) {
		return clone_single_argument(elem);
	});
}

/**
For each method name, create a member function to store its arguments.
*/
var method_names = [ "onInteractionUpdated",  "onInteractionGone"];
method_names.forEach(function(name) {

	ClientWindowProxy.prototype[name] = function()
	{
		window.inConcert.logger.info("[ClientWindowProxy] Storing call to " + name);
		var args = Array.prototype.slice.call(arguments);
		this.m_queued_calls.push({
			func: name,
			args: clone_arguments(args)
		});
	};
});

/**
Perform the method calls that we received up to now.
*/
ClientWindowProxy.prototype.apply = function (dest)
{
	this.m_queued_calls.forEach(function(invocation) {
		var member = dest[invocation.func];
		if(typeof(member) !== "function") {
			window.inConcert.logger.error("[ClientWindowProxy] Could not invoke method '" + invocation.func + "' in target object");
			return;
		}
		window.inConcert.logger.info("[ClientWindowProxy] Applying queued call to " + invocation.func);
		member.apply(dest, invocation.args);
	});

	this.m_queued_calls = [];
};


window.inConcert = window.inConcert || {};
window.inConcert.ClientWindowProxy = ClientWindowProxy;

})(window);
