/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, esversion: 11 */

/* global $, inConcert, inconcert, _ */

// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	/**
		Class ContactPanel.
		Panel subclass.
	*/
	'use strict';

	var _T = inConcert.i18n.getString;
	var format = inConcert.Utils.format;

	function ContactPanel (id)
	{
		var self = this;

		this.m_width = 256;
		this.m_visible = true;
		this.m_views = {
			CONTACTS : 0,
			TEAM : 1
		};
		this.m_interaction_gui = new inConcert.interaction_gui( inConcert.attention_actions,
			{
				from_app : 'interaction',
				owner    : this
			}
		);
		this.m_currentView = null; //one of contacts/favorites/team
		this.current_query_filter = {};
		this.current_query_filter[this.m_views.CONTACTS] = null;
		this.current_query_filter[this.m_views.TEAM] = null;
		this.m_main = inConcert.app.getMainWindow();

		// arrays with list of the cached contacts, favorites and team members.
		this.m_cachedContacts = null;
		this.m_cachedFavorites = null;
		this.m_request_members = null;

		//the contact container list is initialized only once to avoid continuos searching
		this.m_handler = $('#' + id);
		this.m_listContainer = this.m_handler.find('.contentContactList');
		this.m_pageStart = '';
		this.m_rowsPerPage = 40;
		this.m_scrollOnPercentage = 80;
		// unbind/bind scroll event.
		this.m_listContainer.unbind('scroll').bind('scroll', function() { self.onScroll(); });
		// number of refresh ticks processed, used to refresh different things on different ticks modulus
		this.m_ticks = 0;

		//tooltips
		$('#btn_reset_contact_list').inconcert_tooltip(); //(bindeo estatico)
		$('#int_panel_contacts').inconcert_tooltip( { bind_selector: '.boxContactActions [tooltip]' } ); // acciones de los botones (bindeo dinamico)
		$('#int_panel_contacts .contactDetailId [tooltip]').inconcert_tooltip(); // acciones dentro del detalle (bindeo estatico)
		$('#int_panel_contacts').inconcert_tooltip( {alignment: 'left-bottom', bind_selector: '.contactDetailAddressActions [tooltip]' } ); // acciones en las interaccions dentro del detalle (bindeo dinamico)
		this.m_handler.inconcert_tooltip( {alignment: 'right-top' , bind_selector: '.contactStatus span[tooltip]' });
		this.m_handler.inconcert_tooltip( {alignment: 'right-top' , bind_selector: '[user_type="member"] .boxContactIdentificator span[tooltip]' });

		//load avaiable address for contact edition combo
		var addresses = [
			{ id : 'Phone'    , label: "<%=_T('Phone')%>"    },
			{ id : 'Mail'     , label: "<%=_T('E-mail')%>"     },
			{ id : 'IM'       , label: "<%=_T('IM')%>"       },
			{ id : 'Twitter'  , label: "<%=_T('Twitter')%>"  },
			{ id : 'Facebook' , label: "<%=_T('Facebook')%>" },
			{ id : 'Instagram' , label: "<%=_T('Instagram')%>" },
			{ id : 'Youtube'   , label: "<%=_T('Youtube')%>"  },
			{ id : 'Appstore'  , label: "<%=_T('Appstore')%>"  },
			{ id : 'Googleplay', label: "<%=_T('Googleplay')%>"  }
		];
		$.each(addresses, function(undefined, value) {
			$('#contactAddressEditType').
				append($('<option></option>').
				attr('value',value.id).
				text(value.label));
		});

		//available IM protocols
		var IMNetworks = [
			{ id : 'prpl-yahoo'  , label : "<%=_T('prpl-yahoo')%>" },
			{ id : 'prpl-msn'    , label : "<%=_T('prpl-msn')%>" },
			{ id : 'prpl-jabber' , label : "<%=_T('prpl-jabber')%>" }
		];
		$.each(IMNetworks, function(undefined, value) {
			$('#contactAddressEditIMNetwork').
				append($('<option></option>').
				attr('value',value.id).
				text(value.label));
		});

		//global handlers
		$('#chkContactPeople').click(function() {
			if ($(this).prop('checked')) {
				$('#chkContactOrganization').prop('checked', false);
				$('#contactDetailCompanySection').show();
				if (self.m_openedContact) {
					self.m_openedContact.IsOrganization = '0';
					if (!self.m_openedContact.isNew) {
						self.saveContact(self.m_openedContact);
					}
				}
			}
		});

		$('#chkContactOrganization').click(function() {
			if ($(this).prop('checked')) {
				$('#chkContactPeople').prop('checked', false);
				$('#contactDetailCompanySection').hide();
				if (self.m_openedContact) {
					self.m_openedContact.IsOrganization = '1';
					if (!self.m_openedContact.isNew) {
						self.saveContact(self.m_openedContact);
					}
				}
			}
		});

		$('#contactDetailCompany').blur(function() {
			var company = $('#contactDetailCompany').val();
			if (self.m_openedContact) {
				if (self.m_openedContact.Company !== company) {
					self.m_openedContact.Company = company;
					if (self.validateBasicData(self.m_openedContact)) {
						if (!self.m_openedContact.isNew) {
							self.saveContact(self.m_openedContact);
						}
					}
					else {
						return false;
					}
				}
			}
		});

		$('#contactDetailNameHeader').blur(function() {
			var name = $('#contactDetailNameHeader').val().trim();
			if (self.m_openedContact) {
				if (self.m_openedContact.Name !== name) {
					var previousName = self.m_openedContact.Name;
					self.m_openedContact.Name = name;
					if (self.validateBasicData(self.m_openedContact)) {
						if (!self.m_openedContact.isNew) {
							self.saveContact(self.m_openedContact);
						}
					}
					else {
						self.m_openedContact.Name = previousName;
						$('#contactDetailNameHeader').val(self.m_openedContact.Name);
						return false;
					}
				}
			}
		});

		$('#contactDetailName').blur(function() {
			var name = $('#contactDetailName').val().trim();
			if (self.m_openedContact) {
				self.m_openedContact.Name = name;
			}
		});

		$('#contactDetailGroupList > input').blur(function() {
			inConcert.app.m_mainWindow.showPopupMessage({
				message: "<%=_T('hat to do here? this is a list')%>",
				messageTitle: ':(',
				popUpTitle: "<%=_T('Oops')%>",
				messageType: 'Error'
			});
		});

		$('#createNewContact').click(function() {
			self.createNewContact();
		});

		var closeDetail = function () {
			self.m_handler.find('.butDetailClose').trigger('click');
		};

		$('#saveNewContact').click(function() {
			if (!self.m_openedContact || !self.validateBasicData(self.m_openedContact)) {
				return;
			}
			$('#contactPanelSearchWaiting').show();
			self.m_openedContact.save(
				function( success, contact ) {
					if ( success ) {
						contact = inConcert.app.getContactManager().create(contact);
						if ( contact ) {
							self.addContact(contact, self.m_views.CONTACTS, true);
							if (self.m_cachedContacts){
								self.m_cachedContacts.push(contact);
							}
							if (self.m_onContactCreatedFn) {
								self.m_onContactCreatedFn(contact);
								self.m_onContactCreatedFn = null;
							}
						}
						$('#contactPanelSearchWaiting').hide();
						closeDetail();
					}
					else {
						var error = contact;
						var message_error;
						if ( error.status === 403 ) {
							message_error = "<%=_T('You not have permission to save contact.')%>";
						}
						else {
							message_error = format("<%=_T('ERROR SAVING CONTACT {0}')%>", error.status );
						}
						inConcert.app.m_mainWindow.showBannerMessage({ message : message_error, timeout : 5000 });
						$("#contactPanelSearchWaiting").hide();
					}
				}
			);
		});

		$('#butContactOptionDelete').click(function() {
			if (!self.m_openedContact  || self.m_openedContact.isNew) {
				return;
			}
			jQuery.inconcertPopup.Show(
				"<%=_T('Confirm')%>",
				_T('Do you want to delete contact "' + self.m_openedContact.Name + '" ?'),
				jQuery.inconcertPopup.ButtonSets.BS_YESNO,
				function(response) {
					if (response.toLowerCase() !== 'yes') {
						return;
					}
					self.m_openedContact.remove( function( success, error ) {
						if ( success ) {
							if (self.m_openedNode) {
								self.m_openedNode.remove();
							}
							self.deleteContactFromCache(self.m_openedContact);
							self.m_handler.find('.butDetailClose').click();
						}
						else {
							var message_error;
							if ( error.status === 403 ) {
								message_error = "<%=_T('You not have permission to delete contact.')%>";
							}
							else {
								message_error = format("<%=_T('ERROR DELETING CONTACT {0}')%>", error.status );
							}
							inConcert.app.m_mainWindow.showBannerMessage({ message : message_error, timeout : 5000 });
						}
					});
				},
				undefined,
				{
					popup_id        : 'contact_panel_inconcertPopup',
					container_class : 'confirmationPopUp',
					overlay_class   : 'confirmationOverlay'
				}
			);
		});

		$('#cancelNewContact').click(function() {
			self.m_handler.find('.butDetailClose').click();
			$('#contactMainErrorMessage').text('').hide();
		});

		$('#addContactAddress').click(function() {
			self.addContactAddress(self.m_openedContact);
		});

		$('#contactAddressEditType').change(function() {
			self.changeAddressType($(this).val());
		});

		$('#chkShowContactArchiveInteractions').click(function() {
			if ($(this).prop('checked')) {
				$('#chkShowContactCurrentInteractions').prop('checked', false);
				self.loadContactArchive(self.m_openedContact);
			}
		});

		$('#chkShowContactCurrentInteractions').click(function() {
			if ($(this).prop('checked')) {
				$('#chkShowContactArchiveInteractions').prop('checked', false);
				self.loadContactCurrentInteractions(self.m_openedContact);
			}
		});

		$('#btn_search_conversation_contact_list').click(function() {
			if ($('#chkShowContactCurrentInteractions').prop('checked')) {
				self.loadContactCurrentInteractions(self.m_openedContact);
			}
			else {
				self.loadContactArchive(self.m_openedContact);
			}
		});

		$('#chkShowTeamMemberArchivedInteractions').click(function() {
			if ($(this).prop('checked')) {
				$('#chkShowTeamMemberCurrentInteractions').prop('checked', false);
				self.loadTeamMemberArchive(self.m_openedMember);
			}
		});

		$('#chkShowTeamMemberCurrentInteractions').click(function() {
			if ($(this).prop('checked')) {
				$('#chkShowTeamMemberArchivedInteractions').prop('checked', false);
				self.loadTeamMemberCurrentInteractions(self.m_openedMember);
			}
		});

		$('#btn_search_conversation_member_list').click(function() {
			if ($('#chkShowTeamMemberCurrentInteractions').prop('checked')) {
				self.loadTeamMemberCurrentInteractions(self.m_openedMember);
			}
			else {
				self.loadTeamMemberArchive(self.m_openedMember);
			}
		});

		//trigger for contact filtered search
		var $txt_search_contact_list = $('#txt_search_contact_list');
		var searchTimerId;

		var triggerSearchFn = function() {
			clearTimeout(searchTimerId);
			self.search(true, undefined, function() {
				$txt_search_contact_list.focus();
			});
		};

		$('#btn_search_contact_list').click(function() {
			switch (self.m_currentView)
			{
				case self.m_views.CONTACTS:
					self.cleanContactsCache();
					break;
			}
			triggerSearchFn();
		});

		$('#btn_reset_contact_list').click(function() {
			$(this).prop('disabled', true);
			$(this).hide();
			$('#btn_search_contact_list').show();
			$txt_search_contact_list.val('');
			$('#btn_search_contact_list').click();
		});

		$txt_search_contact_list.keyup(function(event) {
			// If user press ctrl/tab/command/arrows don't do anything
			var filteredControlKeys = inConcert.Utils.getControlCharacters();
			if (filteredControlKeys[event.keyCode]) {
				return;
			}

			self.enableDisableResetButton();

			// if user press Enter trigger the search inmediatly
			if (event.keyCode === 13) {
				triggerSearchFn();
				return;
			}
			clearTimeout(searchTimerId);
			searchTimerId = setTimeout(triggerSearchFn, 400);
		});

		this.__set_pagination_events();

		//trigger first search when initialized
		this.m_currentView = this.m_views.CONTACTS;
		this.search(true);

		// suscribe to contact changes
		this.pubsub_contact_updated_handle = inConcert.app.messagebus.subscribe(
			'/application/contacts/contact_updated',
			function() {
				self.search(true);
			}
		);
	}

	//Define functions and objects in the namespace in case they don't already exist
	if (typeof inConcert.ContactPanel !== 'function') {
		inConcert.ContactPanel = ContactPanel;
	}

	//on scroll
	ContactPanel.prototype.onScroll = function()
	{
		var self = this.m_main.getPanel('shared').get_tab('CONTACTS');
		if (!self) {
			throw('ContactPanel.prototype.onScrol - Unrecognized CONTACTS panels.');
		}
		if (self.m_currentView !== self.m_views.CONTACTS) {
			return;
		}
		var scroll = this;
		var scrollTop = scroll.scrollTop;
		var scrollHeight = scroll.scrollHeight;
		var clientHeight = scroll.clientHeight;
		var scrolledPercentage = Math.round((scrollTop / (scrollHeight - clientHeight)) * 100);
		if (scrolledPercentage < self.m_scrollOnPercentage || self.m_pageStart === '') {
			return;
		}
		//start from the current loaded
		self.m_listContainer.unbind('scroll');
		var current_filter = self.current_query_filter[self.m_currentView];
		if ( !current_filter ) {
			return;
		}
		var filter = {
			last  : self.m_pageStart,
			rows  : self.m_rowsPerPage,
			query : current_filter.query
		};
		inConcert.app.getContactManager().search(filter, function(contacts, lastContact) {
			if (contacts) {
				// New contacts goes to cache
				self.m_cachedContacts = self.m_cachedContacts.concat(contacts);
				// Append new contacts to the panel
				self.appendContacts(contacts, self.m_views.CONTACTS);
				self.m_pageStart = lastContact;
			}
			self.m_listContainer.bind('scroll', function() {self.onScroll();});
		});
	};

	ContactPanel.prototype.enableDisableResetButton = function()
	{
		var currentFilter = $('#txt_search_contact_list').val();
		if (currentFilter !== '') {
			$('#btn_reset_contact_list').removeAttr('disabled');
			$('#btn_reset_contact_list').show();
			$('#btn_search_contact_list').hide();
		} else {
			$('#btn_reset_contact_list').prop('disabled', true);
			$('#btn_reset_contact_list').hide();
			$('#btn_search_contact_list').show();
		}
	};

	ContactPanel.prototype.showMessageNotFound = function(view, $message_container, waiting_type) {
		var views = this.m_views;
		var message;
		if (waiting_type === 'contact_conversations' || waiting_type === 'member_conversations') {
			message = "<%=_T('No conversations found.')%>";
		}
		else {
			switch (view || this.m_currentView)
			{
				case views.CONTACTS:
					message = "<%=_T('No contacts found.')%>";
					break;
				case views.TEAM:
					message = "<%=_T('No users found.')%>";
					break;
				default:
					message = "<%=_T('No records found.')%>";
			}
		}
		$message_container = $message_container || $('#contactPanelSearchMessages');
		$message_container.text(message).show();
	};

	ContactPanel.prototype.startingSearch = function()
	{
		$('#contactPanelSearchWaiting').show();
		$('#contactPanelSearchMessages').hide();
		$('#int_panel_contacts [waiting_type]').hide();
	};

	ContactPanel.prototype.endSearch = function(totalContacts, view, waiting_type)
	{
		var $message_container = waiting_type ?
			$('#int_panel_contacts [waiting_type="'+ waiting_type + '"]') :
			$('#contactPanelSearchMessages');

		$('#contactPanelSearchWaiting').hide();
		if (totalContacts === 0) {
			this.showMessageNotFound(view, $message_container, waiting_type);
		}
		else {
			$message_container.hide();
		}
	};

	ContactPanel.prototype.select_tab = function( tab_id )
	{
		if ( tab_id === 'contacts' ) {
			this.activateContacts();
		}
		else if ( tab_id === 'my_team' ) {
			this.activateMyTeams();
		}
		else {
			throw('ContactPanel.prototype.select_tab. Unrecognized tab + "' + tab_id + '"');
		}
	};

	//load the contacts in the current view, 'this' is the clicked tab
	ContactPanel.prototype.activateContacts = function()
	{
		$('#int_shared_panel_search').show();
		$('#int_shared_panel_contact_member_paginated_container').hide();
		$('#int_panel_contacts').show();
		$('#int_panel_campaigns').hide();

		// var $txt_search_contact_list = $('#txt_search_contact_list');
		// this.current_query_filter[this.m_currentView] = $txt_search_contact_list.val();
		// this.m_currentView = this.m_views.CONTACTS;
		// $txt_search_contact_list.val(this.current_query_filter[this.m_currentView]).focus();

		this.m_currentView = this.m_views.CONTACTS;
		this.m_listContainer.removeClass('contentTeamMemberList');
		this.clearListContainer();

		//If we have cached contacts, reuse them
		if (this.m_cachedContacts) {
			this.endSearch(this.m_cachedContacts ? this.m_cachedContacts.length : 0);
			this.appendContacts(this.m_cachedContacts, this.m_views.CONTACTS);
		}
		else {
			//Execute default search
			this.search(false);
		}

		this.enableDisableResetButton();
	};

	ContactPanel.prototype.cleanContactsCache = function() {
		this.m_cachedContacts = null;
	};

	ContactPanel.prototype.deleteContactFromCache = function(contact) {
		var self = this;
		if ( !self.m_cachedContacts ){
			return;
		}
		for (var i = 0; i < self.m_cachedContacts.length; i++ ) {
			if (self.m_cachedContacts[i].Id === contact.Id) {
				self.m_cachedContacts.splice(i, 1);
				break;
			}
		}
	};

	ContactPanel.prototype.updateContactCache = function(contact) {
		var self = this;
		if ( !self.m_cachedContacts ){
			return;
		}
		for (var i = 0; i < self.m_cachedContacts.length; i++ ) {
			if (self.m_cachedContacts[i].Id === contact.Id) {
				self.m_cachedContacts.splice(i, 1, contact);
				break;
			}
		}
	};

	ContactPanel.prototype.getContactFromCache = function(contactID) {
		var self = this;
		if ( !self.m_cachedContacts ){
			return;
		}
		for (var i = 0; i < self.m_cachedContacts.length; i++ ) {
			if (self.m_cachedContacts[i].Id === contactID) {
				return self.m_cachedContacts[i];
			}
		}
	};

	ContactPanel.prototype.saveContact = function(contact, callbackFn) {
		//Save the contact via API, and adds it to the local cache
		var self = this;
		contact.save(function(success, savedContact){
			if (success) {
				self.updateContactCache(savedContact);
				var normalizedContactId = 'contactNode_' + (inConcert.Utils.getNormalizedId(savedContact.Id) || savedContact.Id);
				$('#' + normalizedContactId).data('contact', savedContact);
			}
			if (typeof(callbackFn) === 'function') {
				callbackFn(success ? savedContact: false);
			}
		});
	};

	//load the team members in the current view, 'this' is the clicked tab
	ContactPanel.prototype.activateMyTeams = function()
	{
		$('#int_shared_panel_search').show();
		$('#int_shared_panel_contact_member_paginated_container').show();
		$('#int_panel_contacts').show();
		$('#int_panel_campaigns').hide();

		this.endSearch(1);
		// var $txt_search_contact_list = $('#txt_search_contact_list');
		// this.current_query_filter[this.m_currentView] = $txt_search_contact_list.val();
		// this.m_currentView = this.m_views.TEAM;
		// $txt_search_contact_list.val(this.current_query_filter[this.m_currentView]).focus();

		this.m_currentView = this.m_views.TEAM;
		this.m_listContainer.addClass('contentTeamMemberList');
		this.clearListContainer();
		this.search(false);

		this.enableDisableResetButton();
	};

	//refresh contact panel on every tick
	//only team members state times get refreshed
	ContactPanel.prototype.refreshPanel = function() {
		var reloadInSecs = 60;
		if (this.m_currentView !== this.m_views.TEAM) {
			return;
		}
		if (this.m_handler.hasClass('boxContactDetails')) {
			//do not refresh list if viewing member detail
			return;
		}
		this.m_ticks++;
		if (this.m_ticks % reloadInSecs === 0) {
			var show_waiting = false;
			this.search(false, show_waiting);
			return;
		}
		var refreshInSecs = 10;
		if (this.m_ticks % refreshInSecs === 0) {
			this.m_handler.find('.contactStatus').each(function() {
				var statusDate = parseInt($(this).attr('date'), 10);
				var statusName = $(this).attr('status');
				var exactTimeInState = "<%=_T('Never')%>";
				var timeInState = "<%=_T('Never')%>";
				if (statusDate) {
					exactTimeInState = inConcert.Utils.getTimeDifference(new Date(statusDate));
					timeInState = inConcert.Utils.timeAgo(new Date(statusDate));
				}
				var statusMessage = format('<span tooltip="{0}">({1})</span> {2}', exactTimeInState, timeInState, statusName);
				$(this).html(statusMessage);
			});
			return;
		}
	};

	ContactPanel.prototype.disableSearchCtrls = function()
	{
		$('#txt_search_contact_list').prop('disabled', true);
		$('#btn_search_contact_list').prop('disabled', true);
		$('#btn_reset_contact_list').prop('disabled', true);
		$('#btn_reset_contact_list').hide();
	};

	ContactPanel.prototype.enableSearchCtrls = function(callback)
	{
		callback = callback || $.noop;
		$('#txt_search_contact_list').prop('disabled', false);
		$('#btn_search_contact_list').prop('disabled', false);
		this.enableDisableResetButton();
		callback();
	};

	//search for contacts starting from page 0 on the current filter
	ContactPanel.prototype.search = function(is_new_search, show_waiting, callback)
	{
		var self = this;
		callback = callback || $.noop;

		var currentView = this.m_currentView;
		//Starting search animation
		if (show_waiting || show_waiting === undefined) {
			this.startingSearch();
		}
		this.m_listContainer.unbind('scroll');

		var current_filter = this.current_query_filter[this.m_currentView];

		// defaults value set
		if ( !current_filter || is_new_search ) {
			current_filter = {
				page_number  : 1,
				rows_per_page: this.m_rowsPerPage
			};
		}

		// forzar el disabled de los botones mientras esta buscando
		this.__update_current_pagination( true );

		current_filter.query = $('#txt_search_contact_list').val();

		this.current_query_filter[this.m_currentView] = current_filter;

		//this local function fill, sort list of the favorites and team members
		var onMembersSearchCompleted = function(response, sortFn)
		{
			sortFn = sortFn || function() {return response.members;};
			self.m_listContainer.bind('scroll', function() {self.onScroll();});
			if ( !response.members ) {
				return;
			}
			if (self.m_currentView !== currentView) {
				return;
			}
			self.clearListContainer();
			var sorted_list = sortFn(response.members);
			self.appendContacts(sorted_list, currentView, function() {
				self.endSearch(sorted_list.length, currentView);
			});

			var current_filter = self.current_query_filter[self.m_currentView];
			current_filter.count_users_in_pages = response.count_users_in_pages;

			// update pagination buttons
			self.__update_current_pagination();
		};

		switch (currentView)
		{
			case this.m_views.TEAM:
				if ( !this.m_request_members ) {
					// si no existe un request de este tipo en curso continuo 
					this.disableSearchCtrls();
					this.m_request_members = inConcert.app.getTeamMembersWithState(
						current_filter,
						function(success, response)
						{
							if (!success) {
								onMembersSearchCompleted(false);
								self.enableSearchCtrls(callback);
								return;
							}
							self.m_request_members = null;
							onMembersSearchCompleted(response, inConcert.Utils.sortTeamMembersByState);
							self.enableSearchCtrls(callback);
						}
					);
				}
				break;

			case this.m_views.CONTACTS:

				if (this.m_request_members) {
					this.m_request_members.abort();
					this.m_request_members = null;
				}
				this.m_pageStart = '';
				var contactHandler = function(people, lastContact) {
					// validate the view did not change
					if (self.m_currentView === currentView) {
						self.endSearch(people ? people.length : 0);
						self.clearListContainer();
						if (people) {
							self.m_cachedContacts = people;
							self.appendContacts(people, self.m_views.CONTACTS);
							self.m_pageStart = lastContact;
						}
					}
					self.m_listContainer.bind('scroll', function() {self.onScroll();});
					callback();
				};
				var filter = {
					last: '',
					rows: this.m_rowsPerPage,
					query: current_filter.query
				};
				inConcert.app.getContactManager().search(filter, contactHandler);
		}
	};

	ContactPanel.prototype.switchToDetailedView = function(contact, contactNode, headerClass, detailClass) {
		var self = this;
		//switching between listview and detail view
		self.m_handler.removeClass('boxContactList').addClass('boxContactDetails');
		self.m_handler.find('.boxContentFilterArea').hide();
		self.m_handler.find('.contentContactList').hide();
		self.m_handler.find('#contactPanelListActions').hide();
		self.m_handler.find('.boxContentListMessages').hide();
		$('#cancelEditContactAddress').click();
		self.m_handler.find('.' + headerClass).show();
		self.m_handler.find('.' + detailClass).show();
		if (contact.isNew) {
			self.m_handler.find('#contactDetailName').val('');
			self.m_handler.find('#contactDetailCompany').val('');
			self.m_handler.addClass('boxContactNew');
			self.m_handler.find('.' + headerClass).hide();
			self.m_handler.find('#createContactDiv').show();
			self.m_handler.find('#createContactInf').show();
			self.m_handler.find('#contactNewActions').show();
			self.m_handler.find('.contactDetailInformationField-ro').addClass('contactDetailInformationField-ed').removeClass('contactDetailInformationField-ro');
		}
		else {
			self.m_handler.removeClass('boxContactNew');
			self.m_handler.find('.' + headerClass).show();
			self.m_handler.find('#createContactDiv').hide();
			self.m_handler.find('#createContactInf').hide();
			self.m_handler.find('#contactNewActions').hide();
			self.m_handler.find('.contactDetailInformationField-ed').addClass('contactDetailInformationField-ro').removeClass('contactDetailInformationField-ed');
		}
		var fnBackToList = function() {
			if (contactNode) {
				contactNode.find('.boxContactIdentificator').text(contact.Name);
				var company = contact.Company ? contact.Company : "";
				contactNode.find('.contactGroup').text(company);
			}
			$('#contactMainErrorMessage').text('').hide();
			$('#cancelEditContactAddress').click();
			self.m_handler.find('#contactNewActions').hide();
			self.m_handler.removeClass('boxContactDetails').addClass('boxContactList').removeClass('boxContactNew');
			self.m_handler.find('.boxContentFilterArea').show();
			self.m_handler.find('.contentContactList').show();
			self.m_handler.find('#contactPanelListActions').show();
			self.m_openedContact = null;
			self.m_onContactCreatedFn = null;
			self.m_handler.find('.' + headerClass).hide();
			self.m_handler.find('.' + detailClass).hide();
		};
		self.m_handler.find('.butBackList').unbind('click').bind('click', fnBackToList);
		self.m_handler.find('.butDetailClose').unbind('click').bind('click', fnBackToList);
	};

	ContactPanel.prototype.createTeamMemberChart = function(  ) {

		var memberChart = new inconcert.Chart({
			div: 'teamMemberDetailCharts',
			ChartProperties : {title: "<%=_T('Agent State Distribution')%>",
								allowZoom: false,
								streched : false,
								canFetchData: false
								},
			Pagination: {cache: false, showButtons: false},
			Series: {
				Properties: {showSeriesSelector: false},
				Main: {
					Properties : {
						type : 'bar',
						hintString : '[x], [y]'
					}
				}
			},
			OnError: function ( /* errorCode, description */ ) {
				return false;
			}
		});
		var data2 = [];
		for (var i = 0; i < 20; i++) {
			data2 [i] = [i, 0.6*Math.abs(Math.sin(i * 0.2))];
		}
		memberChart.Series.Main.SetDataSource ({source: data2});

	};

	//opens a contact detail
	ContactPanel.prototype.openTeamMemberDetail = function(member, contactNode) {
		var self = this;
		self.switchToDetailedView(member, contactNode, 'teamMemberDetailId', 'contentTeamMemberDetail');

		var online = member.isOnline();
		self.m_openedMember = member;
		var stateClass = 'teamMemberDetail' + inConcert.app.getMainWindow().getStateQualifier(member.status.state);
		self.m_handler.find('.teamMemberDetailId').removeClass('teamMemberDetailAvailable')
												.removeClass('teamMemberDetailBusy')
												.removeClass('teamMemberDetailAway')
												.removeClass('teamMemberDetailOffLine')
												.addClass(stateClass);
		var statusName = online ? inConcert.app.getStatusDescription(member.status.state) : "<%=_T('Offline')%>";
		self.m_handler.find('.teamMemberDetailStatus').text(statusName);

		//load the proper avatar
		var avatarNode = self.m_handler.find('.teamMemberDetailAvatar');
		avatarNode.children().remove();
		avatarNode.append("<img src='" + inConcert.app.getBaseUrl() + "/users/get_avatar/" + member.Id + "' width='52px' height='52px' />");
		self.createTeamMemberChart(member);
		//load always current first in order to get the proper summary
		$('#chkShowTeamMemberCurrentInteractions').prop('checked', true);
		$('#chkShowTeamMemberArchivedInteractions').prop('checked', false);
		self.loadTeamMemberCurrentInteractions(self.m_openedMember);
	};

	ContactPanel.prototype.loadTeamMemberArchive = function(member) {
		var self = this;
		self.startingSearch();
		var interactionsNode = $('#teamMemberDetailInteractions');
		interactionsNode.find('.teamMemberDetailHistoricData').remove();

		var filterText = interactionsNode.find('.teamMemberDetailHistoricFilterArea').find('.contactHistoricSearch').val();
		self.m_memberArchiveFilter = {
			last : '',
			rows : 20,
			query : { contact_name: filterText },
			paginating : false
		};
		var bindScroll = function () {
			var onScroll = function() {
				var scroll = this;
				var scrollTop = scroll.scrollTop;
				var scrollHeight = scroll.scrollHeight;
				var clientHeight = scroll.clientHeight;
				var scrolledPercentage = Math.round((scrollTop / (scrollHeight - clientHeight)) * 100);
				if (scrolledPercentage > 80 && self.m_memberArchiveFilter.last !== '') {
					$('.contentTeamMemberDetail').unbind('scroll.contactArchive');
					self.m_memberArchiveFilter.start = interactionsNode.children().length;
					paginateArchiveFn();
				}
			};
			$('.contentTeamMemberDetail').unbind('scroll.contactArchive').bind('scroll.contactArchive', onScroll);
		};
		var paginateArchiveFn = function() {
			if (!self.m_memberArchiveFilter.paginating) {
				self.m_memberArchiveFilter.paginating = true;
				member.getArchive(self.m_memberArchiveFilter, function(status, result) {
					if (!status) {
						self.endSearch(0, null, 'member_conversations');
						return;
					}
					self.m_memberArchiveFilter.last = result.last;
					let interactions = result.interactions;
					for (var i = 0; i < interactions.length; i++) {
						var markup = self.getMemberInteractionMarkup(interactions[i], true);
						var node = $(markup);
						node.data('interaction', interactions[i]);
						interactionsNode.append(node);
					}
					var list = interactionsNode.find('.teamMemberDetailHistoricData');
					list.unbind('dblclick.openclient');
					list.bind('dblclick.openclient', function() {
						var interaction = $(this).data('interaction');
						inConcert.app.getMainWindow().getPanel('AssignedInteractions').openInteractionClient(interaction, null, {archive:true});
					});
					list.find('[int_comment]').click(function() {
						var interaction = $(this).parent().data('interaction');
						inConcert.logger.guiAction({action_name : 'open_comments_members', window_id: window.id},interaction);
						var comments = new inConcert.CommentsPopup({ from_app: 'interaction' });
						comments.show({
							interaction: interaction,
							parentNode: $(this),
							callbacks: {
								onOpen: function() {
									//mark interaction comments as read
									interaction.readComments();
								}
							}
						});
					});
					self.m_memberArchiveFilter.paginating = false;
					bindScroll();
					self.endSearch(interactions.length, null, 'member_conversations');
				});
			}
			else {
				bindScroll();
			}
		};
		//trigger first page retrieval
		paginateArchiveFn();
		interactionsNode.show();
		bindScroll();
	};

	ContactPanel.prototype.getMemberInteractionMarkup = function(interaction, isArchived) {
		var intClass = this.m_interaction_gui.getChannelClass(interaction);
		intClass += interaction.hasAttachments() ? ' interactionWithAttach ' : '';
		var markup = "<div class='teamMemberDetailHistoricData detailInteractions interaction" + (isArchived ? "Done" : "Active") + " " + intClass + "'>"; //interactionActive interactionDone
		var totalComments = interaction.getComments() ? interaction.getComments().length : '';
		var commentsClass = this.m_interaction_gui.getCommentClass(interaction);
		//TODO bring this back when reopen is done
		if (totalComments !== '' || !isArchived) {
			markup += "<label int_comment='true' class='interactionComments " + commentsClass + "'><span>" + totalComments + "</span></label>";
		}
		markup +="<img src='/inconcert/apps/agent/images/blank.png' class='interactionType' />";
		markup +="<div class='interactionBasicInfo'>";
		markup +="<label class='interactionContactName'>" + inConcert.Utils.escapeAttrib(interaction.getDisplayName()) + "</label>";
		markup +="<label class='interactionDate'>" + interaction.timeSinceLastEvent() + "</label>";
		markup +="</div>";
		markup +="</div>";
		return markup;
	};

	ContactPanel.prototype.loadTeamMemberCurrentInteractions = function(member) {
		var self = this;
		self.startingSearch();
		inConcert.app.getInteractionManager().getMemberInteractions(member, function(interactions) {
			if (!interactions) {
				self.endSearch(0, null, 'member_conversations');
				return;
			}
			var summary = {};
			var summaryLabels = {};
			var interactionsNode = $('#teamMemberDetailInteractions');
			interactionsNode.find('.teamMemberDetailHistoricData').remove();
			for (var i = 0; i < interactions.length; i++) {
				if (summary[interactions[i].type]) {
					summary[interactions[i].type]++;
				}
				else {
					summary[interactions[i].type] = 1;
					summaryLabels[interactions[i].type] = interactions[i].typeLabel;
				}
				var markup = self.getMemberInteractionMarkup(interactions[i], false);
				var node = $(markup);
				node.data('interaction', interactions[i]);
				interactionsNode.append(node);
			}

			$('.teamMemberDetailHistoricData').dblclick(function() {
				var interaction = $(this).data('interaction');
				//open without related node, could be using this maybe? TODO TOCHECK
				inConcert.app.getMainWindow().getPanel('AssignedInteractions').openInteractionClient(interaction, null, {readonly:true});
			});

			interactionsNode.find('[int_comment]').click(function() {
				var interaction = $(this).parent().data('interaction');
				inConcert.logger.guiAction({action_name : 'open_comments_team_members', window_id: window.id},interaction);

				var comments = new inConcert.CommentsPopup({ from_app: 'interaction' });
				comments.show({
					interaction: interaction,
					parentNode: $(this),
					callbacks: {
						onOpen: function() {
							//mark interaction comments as read
							interaction.readComments();
						}
					}
				});
			});

			var summaryParent = $('#teamMemberDetailSummary');
			summaryParent.find('.teamMemberNowSummary').remove();
			for (var intType in summary) {
				if (summary.hasOwnProperty(intType)) {
					var m = "<div class='teamMemberNowSummary'>";
					m += "<img src='/inconcert/apps/agent/images/blank.png' class='teamMemberNowSummaryType summaryType" + intType + "' />";
					m += "<label class='teamMemberNowSummaryText'>" + summaryLabels[intType] + "</label>";
					m += "<label class='teamMemberNowSummaryNumber'>" + summary[intType] + "</label>";
					m += "</div>";
					summaryParent.append(m);
				}
			}
			self.endSearch(interactions.length, null, 'member_conversations');
		});
	};

	ContactPanel.prototype.loadContactCurrentInteractions = function(/*contact*/) {
		// this.startingSearch(); // TODO: descomentar esta linea cuando se implemente la carga de current interactions para los contactos.
		$('#int_panel_contacts .contactDetailHistoric .contactDetailHistoricData').remove();
		this.endSearch(0, null, 'contact_conversations');
	};

	ContactPanel.prototype.loadContactArchive = function(contact) {
		var historyListNode = $('.contactDetailHistoric');
		var self = this;
		self.startingSearch();
		var filterText = historyListNode.find('.contactDetailHistoricFilterArea').find('.contactHistoricSearch').val();
		self.m_archiveFilter = {
			last: '',
			rows : 10,
			query : { contact_name: filterText },
			paginating : false
		};
		historyListNode.find('.contactDetailHistoricData').remove();
		var bindScroll = function () {
			var onScroll = function() {
				var scroll = this;
				var scrollTop = scroll.scrollTop;
				var scrollHeight = scroll.scrollHeight;
				var clientHeight = scroll.clientHeight;
				var scrolledPercentage = Math.round((scrollTop / (scrollHeight - clientHeight)) * 100);
				if (scrolledPercentage > 80 && self.m_archiveFilter.last !== '') {
					$('.contentContactDetail').unbind('scroll.contactArchive');
					paginateArchiveFn();
				}
			};
			$('.contentContactDetail').unbind('scroll.contactArchive').bind('scroll.contactArchive', onScroll);
		};
		var paginateArchiveFn = function() {
			if (!self.m_archiveFilter.paginating) {
				self.m_archiveFilter.paginating = true;
				contact.getArchive(self.m_archiveFilter, function(status, result) {
					if (!status) {
						self.endSearch(0, null, 'contact_conversations');
						return;
					}
					self.m_archiveFilter.last = result.last;
					let interactions = result.interactions;
					for (var i = 0; i < interactions.length; i++) {
						var intClass = self.m_interaction_gui.getChannelClass(interactions[i]);
						intClass += interactions[i].hasAttachments() ? ' interactionWithAttach ' : '';
						var markup = "<div class='contactDetailHistoricData detailInteractions interactionDone " + intClass + "'>";
						markup += '<img src="/inconcert/apps/agent/images/blank.png" class="interactionType"/>';
						var totalComments = interactions[i].getComments() ? interactions[i].getComments().length : '';
						var commentsClass = self.m_interaction_gui.getCommentClass(interactions[i]);
						//TODO bring this back when reopen is done
						if (totalComments !== '') {
							markup += "<label int_comment='true' class='interactionComments " + commentsClass + "'><span>" + totalComments + "</span></label>";
						}
						markup += "<div class='interactionBasicInfo'>";
						markup += "<label class='interactionDate'>" + interactions[i].timeSinceLastEvent() + "</label>";
						var firstAgent = '';
						for (var j = 0; j < interactions[i].steps.length; j++) {
							var step = interactions[i].steps[j];
							if (step.action === 'TAKEN' && step.assignedAgent && step.assignedAgent !== '') {
								firstAgent = step.assignedAgent;
								break;
							}
						}
						markup += "<label class='interactionTeamMemberName'>" + firstAgent + "</label>";
						markup += "</div>";
						markup += "</div>";
						var node = $(markup);
						node.data('interaction', interactions[i]);
						historyListNode.append(node);
					}
					var list = historyListNode.find('.contactDetailHistoricData');
					list.unbind('dblclick.openclient');
					list.bind('dblclick.openclient', function() {
						var interaction = $(this).data('interaction');
						inConcert.app.getMainWindow().getPanel('AssignedInteractions').openInteractionClient(interaction, null, {archive:true});
					});
					var commentArchiveInteraction = list.find('[int_comment]');
					commentArchiveInteraction.unbind('click.comment_archive_contact_detail')
						.bind('click.comment_archive_contact_detail', function() {
							var interaction = $(this).parent().data('interaction');
							inConcert.logger.guiAction({action_name : 'open_comments_archive_contact_detail', window_id: window.id}, interaction);
							var comments = new inConcert.CommentsPopup({ from_app: 'interaction' });
							comments.show({
								interaction: interaction,
								parentNode: $(this),
								callbacks: {
									onOpen: function() {
										//mark interaction comments as read
										interaction.readComments();
									}
								}
							});
					});
					self.m_archiveFilter.paginating = false;
					bindScroll();
					self.endSearch(interactions.length, null, 'contact_conversations');
				});
			}
			else {
				bindScroll();
			}
		};
		//trigger first page retrieval
		paginateArchiveFn();
		historyListNode.show();
		bindScroll();
	};
	//opens a contact detail
	ContactPanel.prototype.openContactDetail = function(contact, contactNode) {
		var self = this;
		self.switchToDetailedView(contact, contactNode, 'contactDetailId', 'contentContactDetail');

		var hasPermission = false;
		$('#butContactOptionDelete, #butContactOptionMerge, #addContactAddress').toggle( hasPermission );

		//load the proper avatar
		var avatarNode = self.m_handler.find('.contactDetailAvatar');
		avatarNode.children().remove();
		avatarNode.append("<img src='" + inConcert.app.getBaseUrl() + contact.getAvatarURI() + "' width='52px' height='52px' />");

		$('#contactDetailNameHeader').val( contact.Name || contact.Id ).attr ('contactId', contact.Id);
		$('#contactDetailName').val(contact.Name).focus();
		var company = contact.Company ? contact.Company : "";
		$('#contactDetailCompany').val(company);

		//Load status text for this sujeto
		//TODO, aca deberia traer el status de IM si existe el texto, o el BEST STATUS MULTI ACCOUNT
		var online = contact.getOnlineStatus();
		var statusText = online ? "<%=_T('Online')%>" : "<%=_T('Offline')%>";
		if (online) {
			self.m_handler.find('.contactDetailId').removeClass('contactDetailUnknown').addClass('contactDetailOnLine');
		}
		else {
			self.m_handler.find('.contactDetailId').removeClass('contactDetailOnLine').addClass('contactDetailUnknown');
		}
		self.m_handler.find('.contactDetailStatus').text(statusText);
		if (contact.IsOrganization === '1') {
			$('#chkContactOrganization').prop('checked', true).click();
		}
		else {
			$('#chkContactPeople').prop('checked', true).click();
		}
		var favoriteAddClass = 'butContactOptionFavorite';
		var favoriteRemoveClass = 'butContactOptionUnFavorite';
		var favoriteMessage = "<%=_T('Favorite')%>";
		var favoriteFn = function() {
			var node = $(this);
			self.favoriteContactFn(node, contact);
		};
		if ( contact.isUserFavorite() ) {
			favoriteAddClass = 'butContactOptionUnFavorite';
			favoriteRemoveClass = 'butContactOptionFavorite';
			favoriteMessage = "<%=_T('UnFavorite')%>";
			favoriteFn = function() {
				var node = $(this);
				self.unFavoriteContactFn(node, contact);
			};
		}
		if ( inConcert.permissions.to_action_on_any_campaign( 'interaction', 'int_favorite_contact' ) ) {
			$('#butContactOptionFavorite')
				.show()
				.removeClass(favoriteRemoveClass)
				.addClass(favoriteAddClass)
				.attr('tooltip', favoriteMessage)
				.unbind('click')
				.bind('click', favoriteFn);
		}
		else {
			$('#butContactOptionFavorite').hide();
		}
		self.m_openedContact = contact;
		self.m_openedNode = contactNode;

		//Load group list for this sujeto
		var groupListNode = self.m_handler.find('#contactGroupList');
		groupListNode.children().remove();
		var campaigns = inConcert.app.getCampaigns();
		var contactCampaignCount = contact.campaigns.length;
		for (var i = 0; i < campaigns.length; i++) {
			var campaign = campaigns[i];
			var userBelongs = false;
			if (contact.isNew && contactCampaignCount === 0) {
				userBelongs = true;
				contact.addToCampaign(campaign.Id);
			}
			else {
				for (var j = 0; j < contact.campaigns.length; j++) {
					if (contact.campaigns[j] === campaign.Id) {
						userBelongs = true;
						break;
					}
				}
			}
			var groupMarkup = '';
			groupMarkup += "<div class='contactGroupListItem' campaignId='" + campaign.Id + "'>";
			groupMarkup +=		"<input class='contactGroupListItemCheck' " + (userBelongs ? "checked" : "") + " type='checkbox' />";
			groupMarkup +=		"<label class='contactGroupListItemLabel'>" + campaign.getDescription() + "</label>";
			groupMarkup += "</div>";
			groupListNode.append(groupMarkup);
		}

		groupListNode.find('.contactGroupListItemCheck').click(function() {
			var campaignId = $(this).parent().attr('campaignId');
			if ($(this).prop('checked')) {
				if (self.m_openedContact.addToCampaign(campaignId) && !self.m_openedContact.isNew) {
					self.saveContact(self.m_openedContact);
				}
			}
			else {
				if (self.m_openedContact.campaigns.length === 1) {
					inConcert.app.m_mainWindow.showPopupMessage({ message : "<%=_T('This buddy must belong at least to one group in order to be contactable')%>",
														messageTitle : "<%=_T('Can not do that for you sorry')%>",
														popUpTitle : "<%=_T('Contact Management Police')%>",
														messageType : 'Info'});
					$(this).prop('checked', true);
				}
				else {
					if (self.m_openedContact.removeFromCampaign(campaignId) && !self.m_openedContact.isNew) {
						self.saveContact(self.m_openedContact);
					}
				}
			}
		});

		//Load address list for this sujeto
		var addressListPanel = self.m_handler.find('.contentContactDetail > .contactDetailAddresses');
		addressListPanel.children().remove();
		_.each(contact.addresses, function(address){
			self.appendAddressNode(addressListPanel, self.getAddressNode(contact, address));
		});

		//load history for this sujeto in case it's not new
		var historyListNode = $('.contactDetailHistoric');
		if (contact.isNew) {
			historyListNode.hide();
		}
		else {
			if ($('#chkShowContactCurrentInteractions').prop('checked')) {
				self.loadContactCurrentInteractions(self.m_openedContact);
			}
			else {
				self.loadContactArchive(self.m_openedContact);
			}
		}
	};

	//handlers the creation of the HTML div and event binding for the address node
	ContactPanel.prototype.appendAddressNode = function(addressListPanel, node) {
		var self = this;

		addressListPanel.append(node);
		$('.butContactDetailDelete')
			.unbind('click.handler')
			.bind('click.handler', function() {
				var addressNode = $(this).parent().parent();
				var address = addressNode.data('address');
				var getMessage = function() {
					var type;
					switch (address.Type) {
						case 'Phone':
							type = "<%=_T('the phone account')%>";
							break;
						case 'Twitter':
							type = "<%=_T('the twitter account')%>";
							break;
						case 'IM':
							type = "<%=_T('the chat account')%>";
							break;
						case 'Facebook':
							type = "<%=_T('the facebook account')%>";
							break;
						case 'Instagram':
							type = "<%=_T('the instagram account')%>";
							break;
						case 'Youtube':
							type = "<%=_T('the youtube account')%>";
							break;
						case 'Appstore':
							type = "<%=_T('the appstore account')%>";
							break;
						case 'Googleplay':
							type = "<%=_T('the googleplay account')%>";
							break;
						case 'Mail':
							type = "<%=_T('the e-mail account')%>";
							break;
						default:
							type = address.Type;
					}
					var message = "<%=_T('Do you want to delete ')%>" + (!type ? '' : type);
					if (self.m_openedContact) {
						var addressName = self.m_openedContact.getAddressDisplayText(address);
						if (addressName) {
							message += ' "' + addressName + '"';
						}
						message += "<%=_T(' of the contact ')%>" + self.m_openedContact.Name;
					}
					message += '?';
					return message;
				};
				$.inconcertPopup.Show(
					"<%=_T('Confirm')%>",
					getMessage(),
					$.inconcertPopup.ButtonSets.BS_YESNO,
					function(response) {
						if (response.toLowerCase() !== 'yes') {
							return;
						}
						self.deleteContactAddress(self.m_openedContact, addressNode);
						if (self.m_openedContact && !self.m_openedContact.isNew) {
							self.saveContact(self.m_openedContact);
						}
					},
					undefined,
					{
						popup_id        : 'contact_panel_inconcertPopup',
						container_class : 'confirmationPopUp',
						overlay_class   : 'confirmationOverlay'
					}
				);
			}
		);
		$('.butContactDetailEdit')
			.unbind('click.handler')
			.bind('click.handler', function() {
				var addressNode = $(this).parent().parent();
				self.addContactAddress(self.m_openedContact, addressNode);
			}
		);
		$('.contactOnAddress')
			.unbind('click.handler')
			.bind('click.handler', function() {
				var addressNode = $(this).parent().parent();
				var address = addressNode.data('address');
				var interactionType = inConcert.app.getContactManager().addressTypeToInteractionType(address.Type);
				self.startInteraction(self.m_openedContact, interactionType, address);
			}
		);
	};

	ContactPanel.prototype.getAddressNode = function(contact, address) {
		switch(address.Type){
			case 'Twitter':
				return this.getTwitterAddressNode(contact, address);
			case 'Facebook':
				return this.getFacebookAddressNode(contact, address);
			case 'Instagram':
				return this.getInstagramAddressNode(contact, address);
			case 'Youtube':
				return this.getYoutubeAddressNode(contact, address);
			case 'Appstore':
				return this.getAppstoreAddressNode(contact, address);
			case 'Googleplay':
				return this.getGoogleplayAddressNode(contact, address);
			case 'Webchat':
				return this.getWebchatAddressNode(contact, address);
			case 'IM':
			case 'Phone':
			case 'Mail':
				// There is no custom address node for this addresses types.
				return this.getDefaultAddressNode(contact, address);
			default:
				inConcert.logger.warn({
					action_name:      'ContactPanel.prototype.getAddressNode',
					action_type:      'contact',
					address_type:     address.Type,
					message:          format('Someone call getAddressNode with an invalid address type: {0} and contact id: {1}', address.Type, contact.Id)
				});
		}
	};

	// Returns a basic jquery node with common actions for an address.
	ContactPanel.prototype.getBasicAddressNode = function(addressType, addressToDisplay) {
		var m = '';
		m += "<div class='contactDetailAddressData'>";
		m +=	"<img src='/inconcert/apps/agent/images/blank.png' class='iconDetailAddressType" + addressType + "'/>";
		m +=	"<input type='field' class='contactDetailAddressField-ro' value='" + addressToDisplay + "'/>";
		m +=	"<div class='contactDetailAddressActions' style='display:none'>";
		m +=	"</div>";
		m += "</div>";
		return $(m);
	};

	ContactPanel.prototype.getDefaultAddressNode = function(contact, address) {
		var addressToDisplay = contact.getAddressDisplayText(address);
		var node = this.getBasicAddressNode(address.Type, addressToDisplay);
		node.data('address', address);

		var tooltip = format("<%=_T('{0} to {1}')%>", address.Type, addressToDisplay);
		node.find('.contactDetailAddressActions')
			.append("<input tooltip='" + tooltip + "' type='button' class='butContactDetailGo" + address.Type + " contactOnAddress' />");

		return node;
	};

	ContactPanel.prototype.getWebchatAddressNode = function(contact, address) {
		var customerId = contact.getAddressDisplayText(address);

		// For now, only valid webchat address is a email address.
		if (customerId.match(inConcert.Utils.getEmailRegexp())) {
			var node = this.getBasicAddressNode('Mail', customerId);
			address.Type = 'Mail';
			address.address = customerId;

			node.find('.contactDetailAddressActions')
				.append("<input tooltip='" + format("<%=_T('{0} to {1}')%>", address.Type, customerId) + "' type='button' class='butContactDetailGoMail contactOnAddress' />");

			node.data('address', address);
			return node;
		}
		else{
			// if we haven't a valid email, return an empty node.
			return $();
		}
	};

	ContactPanel.prototype.getFacebookAddressNode = function(contact, address) {
		var facebookUrl = 'http://www.facebook.com/';
		var facebookUID = contact.getAddressDisplayText(address);
		var node = this.getBasicAddressNode(address.Type, facebookUID);

		node.find('.contactDetailAddressActions')
			.append("<a href='" + facebookUrl + address.UID + "' tooltip='" + format("<%=_T('Open @{0} profile')%>", facebookUID) + "' class='butContactDetailOpenProfileFacebook' />");

		node.data('address', address);
		// return complete node
		return node;
	};

	ContactPanel.prototype.getInstagramAddressNode = function(contact, address) {
		var instagramUrl = 'http://www.instagram.com/';
		var instagramUser = contact.getAddressDisplayText(address);
		var node = this.getBasicAddressNode(address.Type, instagramUser);

		node.find('.contactDetailAddressActions')
			.append("<a href='" + instagramUrl + address.UID + "' tooltip='" + format("<%=_T('Open @{0} profile')%>", instagramUser) + "' class='butContactDetailOpenProfileInstagram' />");

		node.data('address', address);
		// return complete node
		return node;
	};

	ContactPanel.prototype.getYoutubeAddressNode = function(contact, address) {
		var youtubeUrl = 'http://www.youtube.com/';
		var youtubeUser = contact.getAddressDisplayText(address);
		var node = this.getBasicAddressNode(address.Type, youtubeUser);

		node.find('.contactDetailAddressActions')
			.append("<a href='" + youtubeUrl + address.UID + "' tooltip='" + format("<%=_T('Open @{0} profile')%>", youtubeUser) + "' class='butContactDetailOpenProfileYoutube' />");

		node.data('address', address);
		// return complete node
		return node;
	};

	ContactPanel.prototype.getAppstoreAddressNode = function(contact, address) {
		return this.getDefaultAddressNode(contact, address);
	};

	ContactPanel.prototype.getGoogleplayAddressNode = function(contact, address) {
		return this.getDefaultAddressNode(contact, address);
	};

	ContactPanel.prototype.getTwitterAddressNode = function(contact, address) {
		var self = this;
		var twitterUrl = 'http://www.twitter.com/';
		var twitterUser = contact.getAddressDisplayText(address);
		var node = self.getBasicAddressNode(address.Type, twitterUser);

		node.data('address', address);

		node.find('.contactDetailAddressActions')
			.append("<input tooltip='" + format("<%=_T('Send tweet to @{0}')%>", twitterUser) + "' type='button' class='butContactDetailGoTwitter contactOnAddress' />")
			.append("<a href='" + twitterUrl+address.User + "' tooltip='" + format("<%=_T('Open @{0} profile')%>", twitterUser) + "' class='butContactDetailOpenProfileTwitter' />");

		// Generate custom actions (follow/block) for every account on every campaign.
		inConcert.app.getCampaigns().forEach(function(currentCampaign){
			var twitterAccounts = inConcert.app.getCampaignAddresses(currentCampaign.Id, 'TWITTER');
			// Nothing to do without accounts.
			if (twitterAccounts.length === 0){
				return;
			}

			var getGroupMarkup = function(campaignId, campaignName, accountId){
				var m = "";
				m +=	"<div class='contactDetailAddressOptions addressOptionsTwitter' style='display:none'>";
				m +=		"<div id='" + campaignId + "_" + accountId + "' class='contactGroupListItem contactGroupListItemDisabled'>";
				m +=			"<label class='contactGroupListItemLabel'>" + campaignName + " - " + accountId + "</label>";
				m +=			"<div class='contactDetailAddressActions'>";
				m +=				"<input type='button' class='contactDetailAddressOptionsButton contactDetailAddressOptionsButton-ds optionTWFollow' value='" +"<%=_T('Follow')%>" + "'/>";
				m +=				"<input type='button' class='contactDetailAddressOptionsButton contactDetailAddressOptionsButton-ds optionTWBlock' value='" + "<%=_T('Block')%>"+ "'/>";
				m +=			"</div>";
				m +=		"</div>";
				m +=	"</div>";
				return m;
			};

			// Create a markup for every account available
			twitterAccounts.forEach(function(currentAccount){
				// append group actions
				var currentNode = $(getGroupMarkup(currentCampaign.Id, currentCampaign.getDescription(), currentAccount.id));
				node.append(currentNode);

				// Update custom address actions
				self.updateTwitterAddressActions(currentNode, address, currentCampaign.Id, currentAccount.id);
			});
		});

		// return complete node
		return node;
	};

	ContactPanel.prototype.updateTwitterAddressActions = function(node, address, campaignId, accountId){
		var contactManager = inConcert.app.getContactManager();
		var currentNode = node.find('>:first-child');
		var followButton = currentNode.find('.optionTWFollow');
		var blockButton = currentNode.find('.optionTWBlock');

		//
		//	Local function to update the follow/unfollow button status.
		//
		var updateFollowUnfollowButtons = function(){
			contactManager.twitterGetRelationship(address.User, accountId, campaignId, function(result, errorMessage){
				if (result){
					var imFollowingHim = result.relationship.target.followed_by;
					// add class
					currentNode.toggleClass('followingGroup', imFollowingHim);
					currentNode.removeClass('contactGroupListItemDisabled');

					followButton
						.attr('tooltip', format('{0} @{1} on account {2}', imFollowingHim ? "<%=_T('Unfollow')%>" : "<%=_T('Follow')%>", address.User, accountId))
						.val(imFollowingHim ? "<%=_T('Unfollow')%>" : "<%=_T('Follow')%>")
						.removeClass('contactDetailAddressOptionsButton-ds')
						.unbind('click')
						.bind('click', function() {
							contactManager[imFollowingHim ? 'twitterUnfollowUser' : 'twitterFollowUser'](address.User, accountId, campaignId, function(succeeded) {
								if (succeeded) {
									updateFollowUnfollowButtons();
								}
								else{
									inConcert.app.m_mainWindow.showPopupMessage({
										message: "<%=_T('There is a problem trying to follow/unfollow this user, please try again later')%>",
										messageTitle: inConcert.Utils.format("<%=_T('Can not follow/unfollow user {0}')%>", address.User),
										popUpTitle: "<%=_T('Oops')%>",
										messageType: 'Error'
								});
								}
							});
						});
				}
				else{
					// button is disabled by default, just show a tooltip with the message.
					followButton.attr('tooltip', errorMessage);
				}
			});
		};

		//
		//	Local function to update the block/unblock button status.
		//
		//	Note: You can't follow/unfollow a user that is blocked, so I will decide here
		//	if I need to update those buttons.
		//
		var updateBlockUnblockButtons = function(){
			contactManager.twitterBlockExists(address.User, accountId, campaignId, function(imBlockingHim, errorMsg){
				if (!errorMsg){
					// Enable Campaign-Address node.
					currentNode.removeClass('contactGroupListItemDisabled');

					// Update block/unblock status.
					blockButton
						.attr('tooltip', format('{0} @{1} on account {2}', imBlockingHim ? "<%=_T('Unblock')%>" : "<%=_T('Block')%>", address.User, accountId))
						.val(imBlockingHim ? "<%=_T('Unblock')%>" : "<%=_T('Block')%>")
						.removeClass('contactDetailAddressOptionsButton-ds')
						.unbind('click')
						.bind('click', function() {
							contactManager[imBlockingHim ? 'twitterUnBlockUser' : 'twitterBlockUser'](address.User, accountId, campaignId, function(succeeded) {
								if (succeeded) {
									updateBlockUnblockButtons();
								}
								else{
									inConcert.app.m_mainWindow.showPopupMessage({
										message: "<%=_T('There is a problem trying to block/unblock this user, please try again later')%>",
										messageTitle: inConcert.Utils.format("<%=_T('Can not block/unblock user {0}')%>", address.User),
										popUpTitle: "<%=_T('Oops')%>",
										messageType: 'Error'
									});
								}
							});
						});

					// follow/unfollow actions does not apply to blocked users.
					if (!imBlockingHim){
						updateFollowUnfollowButtons();
					}
					else{
						followButton
							.addClass('contactDetailAddressOptionsButton-ds')
							.attr('tooltip', "<%=_T('Action disabled for blocked users')%>");
					}
				}
				else{
					// Show tooltip with error message.
					blockButton.attr('tooltip', _T(errorMsg));

					// Follow/Unfollow actions does not apply if I cannot check if block exists.
					followButton
						.addClass('contactDetailAddressOptionsButton-ds')
						.attr('tooltip', "<%=_T('Check if block exist failed, this is action will be disabled')%>");
				}
			});
		};
		updateBlockUnblockButtons();
	};

	ContactPanel.prototype.deleteContactAddress = function(contact, addressNode) {
		if (addressNode) {
			var address = addressNode.data('address');
			contact.deleteAddress(address);
			addressNode.remove();
		}
	};

	//TODO this should go into contactmanager class
	ContactPanel.prototype.validateContactAddress = function(type) {
		var address = {};
		var message = '';
		var fieldError;
		address.Type = type;
		let User;
		switch (type) {
			case 'Phone':
				var number = $('#contactAddressEditPhone').val();
				//TODO proper number validation
				if (number.length > 5 && number.match(/^[\- \(\)0-9_.]+$/)) {
					address.Number = number;
				}
				else {
					message = "<%=_T('Phone numbers must be at least six digits long')%>";
					address = null;
					fieldError = $('#contactAddressEditPhone');
				}
				break;
			case 'Facebook':
				var UID = $('#contactAddressEditFacebook').val();
				//TODO proper UID validation
				if (UID.length > 0 && UID.match(/^[0-9]+$/)) {
					address.UID = UID;
				}
				else {
					message = "<%=_T('Facebook UID are only numbers')%>";
					address = null;
					fieldError = $('#contactAddressEditFacebook');
				}
				break;
			case 'Instagram':
				User = $('#contactAddressEditInstagram').val();
				// If the ID is the @ character in the first place, remove
				if (User.substr(0, 1) === '@') {
					User = User.substr(1, User.length);
				}
				if (User.length > 0) {
					// no spaces are allowed and no points
					if (!User.match(/^([A-Za-z0-9_]+)$/)) {
						message = "<%=_T('Invalid username! Alphanumerics only')%>";
						address = null;
						fieldError = $('#contactAddressEditInstagram');
					}
					else {
						address.User = User;
						$('#contactAddressEditInstagram').val(User);
					}
				}
				else {
					message = "<%=_T('Username is required.')%>";
					address = null;
					fieldError = $('#contactAddressEditInstagram');
				}
				break;
			case 'Youtube':
				User = $('#contactAddressEditYoutube').val();
				// If the ID is the @ character in the first place, remove
				if (User.substr(0, 1) === '@') {
					User = User.substr(1, User.length);
				}
				if (User.length > 0) {
					// no spaces are allowed and no points
					if (!User.match(/^([A-Za-z0-9_]+)$/)) {
						message = "<%=_T('Invalid username! Alphanumerics only')%>";
						address = null;
						fieldError = $('#contactAddressEditYoutube');
					}
					else {
						address.User = User;
						$('#contactAddressEditYoutube').val(User);
					}
				}
				else {
					message = "<%=_T('Username is required.')%>";
					address = null;
					fieldError = $('#contactAddressEditYoutube');
				}
				break;
			case 'Appstore':
				User = $('#contactAddressEditAppstore').val();
				// If the ID is the @ character in the first place, remove
				if (User.substr(0, 1) === '@') {
					User = User.substr(1, User.length);
				}
				if (User.length > 0) {
					// no spaces are allowed and no points
					if (!User.match(/^([A-Za-z0-9_]+)$/)) {
						message = "<%=_T('Invalid username! Alphanumerics only')%>";
						address = null;
						fieldError = $('#contactAddressEditAppstore');
					}
					else {
						address.User = User;
						$('#contactAddressEditAppstore').val(User);
					}
				}
				else {
					message = "<%=_T('Username is required.')%>";
					address = null;
					fieldError = $('#contactAddressEditAppstore');
				}
				break;
			case 'Googleplay':
				User = $('#contactAddressEditGoogleplay').val();
				// If the ID is the @ character in the first place, remove
				if (User.substr(0, 1) === '@') {
					User = User.substr(1, User.length);
				}
				if (User.length > 0) {
					// no spaces are allowed and no points
					if (!User.match(/^([A-Za-z0-9_]+)$/)) {
						message = "<%=_T('Invalid username! Alphanumerics only')%>";
						address = null;
						fieldError = $('#contactAddressEditGoogleplay');
					}
					else {
						address.User = User;
						$('#contactAddressEditGoogleplay').val(User);
					}
				}
				else {
					message = "<%=_T('Username is required.')%>";
					address = null;
					fieldError = $('#contactAddressEditGoogleplay');
				}
				break;
			case 'Twitter':
				User = $('#contactAddressEditTwitterUser').val();
				// If the ID is the @ character in the first place, remove
				if (User.substr(0, 1) === '@') {
					User = User.substr(1, User.length);
				}
				if (User.length > 0) {
					// no spaces are allowed and no points
					if (!User.match(/^([A-Za-z0-9_]+)$/)) {
						message = "<%=_T('Invalid username! Alphanumerics only')%>";
						address = null;
						fieldError = $('#contactAddressEditTwitterUser');
					}
					else {
						address.User = User;
						$('#contactAddressEditTwitterUser').val(User);
					}
				}
				else {
					message = "<%=_T('Username is required.')%>";
					address = null;
					fieldError = $('#contactAddressEditTwitterUser');
				}
				break;
			case 'IM':
				var protocol = $('#contactAddressEditIMNetwork').val();
				var buddy = $('#contactAddressEditIMbuddy').val();
				if (protocol.length > 0 && buddy.length > 0 && buddy.match(/^[\-a-zA-Z 0-9_@.]+$/)) {
					address.protocol = protocol;
					address.buddy = buddy;
				}
				else {
					message = "<%=_T('Enter a valid buddy id')%>";
					address = null;
					fieldError = $('#contactAddressEditIMbuddy');
				}
				break;
			case 'Mail':
				var email = $('#contactAddressEditEmail').val();
				if (email.match(inConcert.Utils.getEmailRegexp())) {
					address.address = email;
				}
				else {
					message = "<%=_T('Enter a valid e-mail address')%>";
					address = null;
					fieldError = $('#contactAddressEditEmail');
				}
				break;
			default:
				address = null;
		}
		if (!address) {
			$('#contactAddressErrorMessage').text(message).show();
			if (fieldError) {
				fieldError.focus();
			}
		}
		else {
			$('#contactAddressErrorMessage').text('').hide();
		}
		return address;
	};

	/*
		The contact can be created starting from an interaction,
		in that case construct a default address an name suggestion according to that interaction

		Or if you are working with an specific event, you can pass custom contact name, campaigns and addresses.
	*/
	ContactPanel.prototype.createNewContact = function(onContactCreatedFn, interaction, contactName, contactCampaigns, contactAddress) {
		this.m_onContactCreatedFn = onContactCreatedFn;
		var contact = inConcert.app.getContactManager().create();
		if (interaction) {
			contact.Name = interaction.getDisplayName();
			contact.campaigns.push(interaction.campaign);
			var address = interaction.buildContactAddress();
			if (address) {
				contact.addresses.push(address);
			}
		}
		if (contactName){
			contact.Name = contactName;
		}
		if (contactCampaigns){
			contact.campaigns.push(contactCampaigns);
		}
		if (contactAddress){
			contact.addresses.push(contactAddress);
		}
		this.openContactDetail(contact);
	};

	ContactPanel.prototype.setContactAddressText = function(address) {
		if (address) {
			switch (address.Type) {
				case 'Phone':
					$('#contactAddressEditPhone').val(address.Number).focus();
					break;
				case 'Facebook':
					$('#contactAddressEditFacebook').val(address.UID).focus();
					break;
				case 'Instagram':
					$('#contactAddressEditInstagram').val(address.User).focus();
					break;
				case 'Youtube':
					$('#contactAddressEditYoutube').val(address.User).focus();
					break;
				case 'Appstore':
					$('#contactAddressEditAppstore').val(address.User).focus();
					break;
				case 'Googleplay':
					$('#contactAddressEditGoogleplay').val(address.User).focus();
					break;
				case 'Twitter':
					$('#contactAddressEditTwitterUser').val(address.User).focus();
					break;
				case 'IM':
					$('#contactAddressEditIMNetwork').val(address.protocol);
					$('#contactAddressEditIMbuddy').val(address.buddy).focus();
					break;
				case 'Mail':
					$('#contactAddressEditEmail').val(address.address).focus();
					break;
				default:
			}
		}
		else {
			$('.newAddressesFormField').val('');
		}
	};

	ContactPanel.prototype.changeAddressType = function(addressType) {
		$('#contactAddressErrorMessage').hide();
		$('.newAddressesFormData').hide();
		$('.newAddressFormType').show();
		$('.newAddressForm' + addressType).show();
	};

	ContactPanel.prototype.addContactAddress = function(contact, addressNode) {
		var self = this;
		var previousAddress, previousType;
		$('#contactAddressEditionPanel').show();
		if (addressNode) {
			previousAddress = addressNode.data('address');
			previousType = previousAddress.Type;
			$('#contactAddressEditType').val(previousAddress.Type);
		}
		else {
			previousType = 'Phone';
			$('#contactAddressEditType').val(previousType);
		}
		self.changeAddressType(previousType);
		$('#saveContactAddress').unbind('click.permanent').bind('click.permanent', function() {
			var type = $('#contactAddressEditType').val();
			var address = self.validateContactAddress(type);
			if (address) {
				if (addressNode) {
					//update existing
					addressNode.data('address', address);
					if (previousType !== address.Type) {
						addressNode.find('.iconDetailAddressType' + previousType).
									removeClass('iconDetailAddressType' + previousType).
									addClass('iconDetailAddressType' + address.Type);
						addressNode.find('.butContactDetailGo' + previousType).
									removeClass('butContactDetailGo' + previousType).
									addClass('butContactDetailGo' + address.Type);
					}
					addressNode.find('.contactDetailAddressField-ro').val(contact.getAddressDisplayText(address));
				}
				else {
					//new address
					var addressListPanel = self.m_handler.find('.contentContactDetail > .contactDetailAddresses');
					self.appendAddressNode(addressListPanel, self.getAddressNode(contact, address));
				}
				contact.updateAddress(address, previousAddress);
				if (self.m_openedContact && !self.m_openedContact.isNew) {
					self.saveContact(self.m_openedContact);
				}
				$('#contactAddressEditionPanel').hide();
			}
		});
		$('#cancelEditContactAddress').unbind('click').bind('click', function() {
			$('#contactAddressEditionPanel').hide();
			$('#contactAddressErrorMessage').text('').hide();
		});
		self.setContactAddressText(previousAddress);
	};

	//appends the received contacts to the current list scrolling the panel as needed
	ContactPanel.prototype.appendContacts = function(contacts, view, callback)
	{
		var self = this;
		callback = callback || $.noop;
		if ( !$.isArray(contacts)) {
			callback();
			return;
		}
		inConcert.Utils.foreach_timeout({
			rows : contacts,
			append_fn : function( contact ) {
				self.addContact(contact, view);
			},
			complete_fn : function() {
				callback();
			}
		});
	};

	ContactPanel.prototype.startInteraction = function(contact, interactionType, address)
	{
		//build the list to filter
		var panel = this.m_main.getPanel('AssignedInteractions');
		var response = panel.createOutgoingInteraction(interactionType, contact, address, inConcert.app.getCampaigns());
		if (!response.success) {
			if (response.conflicts) {
				inConcert.app.m_mainWindow.showPopupMessage({
					message: response.conflicts.message,
					messageTitle: response.conflicts.title,
					popUpTitle: "<%=_T('Oops')%>",
					messageType: "Error"
				});
			}
			else {
				inConcert.app.m_mainWindow.showPopupMessage({
					message: "<%=_T('If you want to communicate with this contact using this address, please set up an account in one of the contact campaigns.')%>",
					messageTitle: format("<%=_T('Can not {0} this user')%>", interactionType),
					popUpTitle: "<%=_T('Oops')%>",
					messageType: 'Error'
				});
			}
			return null;
		}

		return response.interaction;
	};

	ContactPanel.prototype.onTeamMemberSelected = function(teamMemberNode)
	{
		var self = this;
		var member = teamMemberNode.data('contact');

		//Unselect the current member
		if (this.m_selectedNode) {
			this.m_selectedNode.removeClass('boxContactItemSelect');
			this.m_selectedNode.find('.boxContactActions').remove();
			this.m_selectedNode = null;
		}
		var actionsPanel = teamMemberNode.find('.boxContactActions');
		if (actionsPanel.length === 0 && member) {
			actionsPanel = $("<div class='boxContactActions'><div class='contactInteractions'></div></div>");
			//Removes team member detail button. Issue #3566
			//actionsPanel.prepend('<div class="contactOptions"><input tooltip=' + "<%=_T('Detail')%>" + ' type="button" class="butContactOptionDetail"></div>');
			var addressContainer = actionsPanel.find('.contactInteractions');

			var canStartInternalInteraction;
			canStartInternalInteraction = function(type, permission) {
				let canDoIt = {
					success : true,
					error_message : ""
				};
				let validStates = ["ACTIVE", "NOACD"];
				let validStatesMessage = "<%=_T('Valid states are: ')%>";
				validStates.forEach(function(state, index) {
					let stateName = state;
					let stateObject = inConcert.app.getStateObject(state);
					if (stateObject) {
						stateName = stateObject.Description;
					}
					validStatesMessage += format( "<%=_T('{0}')%>", stateName);
					if(index < (validStates.length - 1) ) {
						validStatesMessage += ", ";
					}
					else {
						validStatesMessage += ".";
					}
				});
				// El Destino debe estar conectado
				if(!(member.status && member.status.state !== "OFFLINE" && member.status.state !== "")) {
					canDoIt.success = false;
					canDoIt.error_message = "<%=_T('User is not connected.')%>";
					return canDoIt;
				}
				if(type === "CHAT") {
					canDoIt.success = inConcert.permissions.to_action_on_any_campaign('interaction', permission);
					canDoIt.error_message = "<%=_T('User has not permissions to receive internal chats.')%>";
					return canDoIt;
				}

				// Validaciones adicionales para llamadas internas

				//Si es allegro no habilitamos el boton de llamada interna (En allegro no hay llamadas internas)
				let vccInfo = inConcert.app.get_vcc_info();
				let hasAllegro = vccInfo && vccInfo.AllegroIntegration && vccInfo.AllegroIntegration.enabled;
				if(hasAllegro) {
					canDoIt.success = false;
					canDoIt.error_message = "<%=_T('User has not permissions to receive internal calls.')%>";
					return canDoIt;
				}

				//No es allegro, seguimos
				
				// Destino
					// Asignacion a la campaña
					if(!member.status || !member.status.campaigns || !member.status.campaigns[inConcert.app.separate_vcc(inConcert.app.get_internal_campaign_id()).id]) {
						canDoIt.success = false;
						canDoIt.error_message = "<%=_T('User has not permissions to receive internal calls.')%>";
						return canDoIt;
					}

				// Origen
					// Asignacion a la campaña
					if(!inConcert.app.getCampaigns().find(campaign => campaign.Id === inConcert.app.get_internal_campaign_id())) {
						canDoIt.success = false;
						canDoIt.error_message = "<%=_T('You are not allowed to make internal calls.')%>";
						return canDoIt;
					}
					// Si el destino no tiene el permiso de supervisor se necesita el permiso de llamada interna para el usuario actual (origen)
					if(!member.isSupervisor && !inConcert.permissions.to_action("interaction", "int_my_teams_start_call")) {
						canDoIt.success = false;
						canDoIt.error_message = "<%=_T('You are not allowed to make internal calls.')%>";
						return canDoIt;
					}

				// Destino: Estados permitidos
				let stateName = member.status.state;
				let stateObject = inConcert.app.getStateObject(member.status.state);
				if (stateObject) {
					stateName = stateObject.Description;
				}
				let member_state_message = format("<%=_T('User state ({0}) is not valid to receive internal calls.')%>", stateName);
				if(!validStates.find(state => state === member.status.state)) {
					canDoIt.success = false;
					canDoIt.error_message = member_state_message + " " + validStatesMessage;
					return canDoIt;
				}

				// Destino: Que no tenga llamada en curso
				if(member.status.interactions) {
					for (var interaction in member.status.interactions) {
						if (Object.prototype.hasOwnProperty.call(member.status.interactions, interaction) && member.status.interactions[interaction].type === "CALL") {		
							canDoIt.success = false;
							canDoIt.error_message = "<%=_T('User has another active call.')%>";
							return canDoIt;
						}
					}
				}

				// Origen: Estados permitidos
				let myCurrentState = inConcert.app.getCurrentStatus();
				stateName = myCurrentState;
				stateObject = inConcert.app.getStateObject(myCurrentState);
				if (stateObject) {
					stateName = stateObject.Description;
				}
				let my_current_state_message = format("<%=_T('Your state ({0}) is not valid to make internal calls.')%>", stateName);
				if(!validStates.find(state => state === myCurrentState)) {
					canDoIt.success = false;
					canDoIt.error_message = my_current_state_message + " " + validStatesMessage;
					return canDoIt;
				}

				// Origen: Que no tenga llamada en curso
				let interactions = inConcert.app.getInteractionManager().getInteractions();
				if (interactions) {
					for (let interaction in interactions) {
						if (Object.prototype.hasOwnProperty.call(interactions, interaction) && interactions[interaction].type === "CALL" &&
							(interactions[interaction].isAssigned() || interactions[interaction].isWrappedUp() || interactions[interaction].isLocked())) {
							canDoIt.success = false;
							canDoIt.error_message = "<%=_T('You can not make a new call when another call is active.')%>";
							return canDoIt;
						}
					}
				}

				return canDoIt;
			};

			var addMemberInteractionButton;
			addMemberInteractionButton = function(params) {
				addressContainer.append("<button tooltip='" + "<%=_T('" + params.tooltip + "')%>" + "' type='button' class='buttonColorWhite " + params.buttonClass + "'><i class='" + params.iconClass + "'></i></button>");
				var button = addressContainer.find('.' + params.buttonClass);
				button.prop('disabled', !(canStartInternalInteraction(params.type, params.permission)).success)
					.click(function()
					{
						//Internal calls require fresh member address information
						let query = {
							page_number: 1,
							query: member.Id,
							rows_per_page: 40
						};
						inConcert.app.getTeamMembersWithState(query, function(success, freshMembers) {
							if(success) {
								var freshMember = freshMembers.members.find(m => m.Id === member.Id);
								if(freshMember) {
									member = freshMember;
									let memberFullName = member.Name + " " + member.FirstSurname;
									var result = canStartInternalInteraction(params.type, params.permission);
									if (!result.success) {
										let error_message = format( "<%=_T('Unable to communicate with {0}.')%>", memberFullName );
										error_message += " " + result.error_message;
										inConcert.app.m_mainWindow.showBannerMessage({ message : error_message, timeout : 5000 });
										return;
									}

									var permissionFn = function() {
										return (canStartInternalInteraction(params.type, params.permission).success);
									};

									var name_and_surname = member.get_name_and_surname();
									params.createInteractionFn(
										member.Id,
										name_and_surname,
										permissionFn,
										function( success, interaction )
										{
											if ( !success || !interaction ) {
												var error = interaction || {};

												if ( error.result === 'FORBIDDEN' ) {
													var error_message = params.error_message;
													inConcert.app.m_mainWindow.showBannerMessage({ message : error_message, timeout : 5000 });
													return;
												}

												// default message
												inConcert.app.getMainWindow().showPopupMessage({
													message      : format( "<%=_T('Unable to communicate with {0}.')%>", name_and_surname ),
													messageTitle : params.messageTitle,
													popUpTitle   : params.popUpTitle,
													messageType  : 'Error'
												});

												return;
											}
											inConcert.app.m_mainWindow.updateInteractions([interaction]);
											inConcert.app.getMainWindow().getPanel('AssignedInteractions').openInteractionClient(interaction, null);
										},
										params.address);
								}
							}
						});
					});
			};

			var addMemberInteractionButtons;
			addMemberInteractionButtons = function() {			
				var interactionManager = inConcert.app.getInteractionManager();
				var implementedInteractionButtonAddresses = {
					CHAT: {
						type: "CHAT",
						permission: 'int_my_teams_start_chat',
						tooltip: "<%=_T('Chat')%>",
						buttonClass: 'butContactInteractionChat',
						iconClass: 'buttons-icons-chat',
						createInteractionFn: interactionManager.createInternalChat.bind(interactionManager),
						error_message: "<%=_T('Unable to start chat: You not have permission for this action.')%>",
						messageTitle: "<%=_T('Unable to start chat.')%>",
						popUpTitle: "<%=_T('Chat not started.')%>"
					},
					CALL: {
						type: "CALL",
						permission: 'int_my_teams_start_call',
						tooltip: "<%=_T('Call')%>",
						buttonClass: 'butContactInteractionCall',
						iconClass: 'buttons-icons-telephone',
						createInteractionFn: interactionManager.createInternalCall.bind(interactionManager),
						error_message: "<%=_T('Unable to start call: You not have permission for this action.')%>",
						messageTitle: "<%=_T('Unable to start call.')%>",
						popUpTitle: "<%=_T('Call not started.')%>",
						address: member.status.addresses.CALL ? member.status.addresses.CALL.address : undefined
					}
				};

				for (var member_address in member.status.addresses) {
					if (member.status.addresses.hasOwnProperty(member_address) && implementedInteractionButtonAddresses[member_address]) {
						addMemberInteractionButton(implementedInteractionButtonAddresses[member_address]);
					}
				}

				var showDetailButton = actionsPanel.find('.butContactOptionDetail');
				showDetailButton.click(function () {
					var $this = $(this);
					$this.prop('disabled', true);
					var memberNode = $(this).parent().parent().parent();
					var member = teamMemberNode.data('contact');
					self.m_main.getPanel('shared').get_tab('CONTACTS').openTeamMemberDetail(member, memberNode);
					$this.prop('disabled', false);
				}).show();

				// Avoid double creation on double click
				if(!self.m_selectedNode) {
					teamMemberNode.append(actionsPanel);
					teamMemberNode.addClass('boxContactItemSelect');
					self.m_selectedNode = teamMemberNode;
				}
			};

			//Internal calls require fresh member address information
			let query = {
				page_number: 1,
				query: member.Id,
				rows_per_page: 40
			};			
			inConcert.app.getTeamMembersWithState(query, function(success, freshMembers) {
				if(success) {
					var freshMember = freshMembers.members.find(m => m.Id === member.Id);
					if(freshMember) {
						member = freshMember;
						addMemberInteractionButtons();
					}
				}
			});
		}
	};

	ContactPanel.prototype.getFavoriteError = function( response )
	{
		var result = response.result;

		switch( result.toUpperCase() ) 
		{
			case 'FORBIDDEN':
				return "<%=_T('Unable to Favorite action: You not have permission for this action.')%>";

			default:
				return "<%=_T('Internal error. Please retry later')%>";
		}
	};

	ContactPanel.prototype.favoriteContactFn = function(node, contactNode) {
		var self = this;
		node.inconcert_tooltip({message: "<%=_T('UnFavorite')%>"});
		var contact =  contactNode.data ? contactNode.data('contact') : contactNode;
		inConcert.app.getContactManager()
			.addToFavorites(contact, function(added, error) {
				if (!added) {
					var message_error = self.getFavoriteError( error );
					inConcert.app.m_mainWindow.showBannerMessage({ message : message_error, timeout : 5000 });
					return;
				}
				self.updatedContact(contact);
		});
	};

	ContactPanel.prototype.unFavoriteContactFn = function(node, contactNode) {
		var self = this;
		node.inconcert_tooltip({message: "<%=_T('Favorite')%>"});
		var contact =  contactNode.data ? contactNode.data('contact') : contactNode;
		inConcert.app.getContactManager()
			.removeFromFavorites(contact, function(removed, error) {
				if (!removed) {
					var message_error = self.getFavoriteError( error );
					inConcert.app.m_mainWindow.showBannerMessage({ message : message_error, timeout : 5000 });
					return;
				}
				self.updatedContact(contact);
		});
	};

	//when the contact is selected the layout with button actions is created as it's first sibling
	//if the panel already exists it's closed
	ContactPanel.prototype.onContactSelected = function(contactNode) {
		var self = this;
		var contact = contactNode.data('contact');

		//Unselected the current contact
		if (self.m_selectedNode) {
			self.m_selectedNode.removeClass('boxContactItemSelect');
			self.m_selectedNode.find('.boxContactActions').remove();
			//TODO this should be on a configuration/per view basis SHOW AVATAR ON LIST
			if (self.m_currentView === self.m_views.CONTACTS) {
				// self.m_selectedNode.find('.contactAvatar').remove();
				self.m_selectedNode.find('.contactGroup').remove();
				self.m_selectedNode.find('.contactStatus').remove();
			}
			self.m_selectedNode = null;
		}

		var actionsPanel = contactNode.find('.boxContactActions');
		var contactInfoPanel = contactNode.find('.contactInfo');
		if (actionsPanel.length === 0 && contact) {

			// TO-DO: Move this and all the code above to mustache templates.
			actionsPanel = $("<div class='boxContactActions'><div class='contactInteractions'></div></div>");
			// TODO, same comment as above, configuration on a perview basis
			if (self.m_currentView === self.m_views.CONTACTS) {
				// contactNode.prepend("<div class='contactAvatar'><img src='" + inConcert.app.getBaseUrl() + contact.getAvatarURI() + "' width='44px' height='44px'/></div>");
				var company = contact.Company ? contact.Company : "";
				contactNode.append("<div class='contactGroup'>" + company + "</div>");
				var status = contactNode.hasClass("boxContactOnLine") ? "<%=_T('Online')%>" : "<%=_T('Offline')%>";
				contactInfoPanel.append("<div class='contactStatus'>" + status + "</div>");
			}

			if ( self.m_currentView !== self.m_views.TEAM ) {
				var favorite_tooltip, favorite_class;
				if ( contact.isUserFavorite() ) {
					favorite_tooltip = "<%=_T('UnFavorite')%>";
					favorite_class = 'butContactOptionUnFavorite';
				}
				else {
					favorite_tooltip = "<%=_T('Favorite')%>";
					favorite_class = 'butContactOptionFavorite';
				}
				var contactActions = ''.concat(
					"<div class='contactOptions'>",
						"<button tooltip='", favorite_tooltip, "' type='button' class='buttonColorWhite ", favorite_class + "'><i class='buttons-icons-favorite'></i></button>",
					"</div>");

				actionsPanel.prepend( contactActions );
			}
			else {
				actionsPanel.prepend("<div class='contactOptions'><button tooltip='" + "<%=_T('Detail')%>" + "' type='button' class='butContactOptionDetail'><i class='buttons-icons-zoomin'></i></button></div>");
			}

			var defaultHandler = function( $btn )
			{
				var cached_contact = self.getContactFromCache( contact.Id );
				var proxy_type = this.type;
				var addresses_list = this.list;
				if ( addresses_list.length === 1 ) {
				// si el contacto tiene una unica address para este proxy, abre la ventana.
					self.startInteraction( cached_contact, proxy_type, addresses_list[0] );
				}
				else {
				// si el contacto tiene mas de una address para este proxy, abre un popup con las addresses en un cbo, para que puede seleccionar una de ellas
				// y luego presionando un boton, abrir el cliente con esa address.
				//
					var popup_id = "age_popup_contact_panel_addresses";
					var $popup = $( 
						"<div id='" + popup_id + "' class='systemPopWindow windowContainer disambiguateWindow' hidden>" +
							"<div class='windowTop'>" +
								"<label class='windowTitle'>" + "<%=_T('Choose Target Address')%>" + "</label>" +
							"</div>" +
							"<div class='windowMiddle'>" +
								"<select id='age_popup_contact_panel_addresses_list' class='addressChooseCombo'></select>" +
							"</div>" +
							"<div class='windowBottom'>" +
								"<input id='age_popup_contact_panel_addresses_open' class='butWindowOk buttonColorBlack' type='button' value='" + "<%=_T('Open')%>" + "' />" +
								"<input id='age_popup_contact_panel_addresses_cancel' class='butWindowClose buttonColorGray' type='button' value='" + "<%=_T('Cancel')%>" + "' />" +
							"</div>" +
						"</div>"
					);
					$("body").append( $popup );

					// agrego las addresses al combo
					var $combo = $('#age_popup_contact_panel_addresses_list');
					$.each( addresses_list, function( index, address )
					{
						var $option = $('<option></option>')
							.attr('value', index)
							.html(address.displayable || address.address);
						if (index === 0) {
							$option.attr('selected', 'selected');
						}
						$combo.append($option);
					});

					// evento click del boton para abrir la ventana del cliente con la address seleccionada
					$('#age_popup_contact_panel_addresses_open')
						.unbind('click')
						.bind('click', function() {
							var index = $combo.val();
							self.startInteraction( cached_contact, proxy_type, addresses_list[ index ] );
							self.contact_addresses_popup.finalize();
						}
					);

					// evento click del boton para cancelar y cerrar el popup
					$('#age_popup_contact_panel_addresses_cancel')
						.unbind('click')
						.bind('click', function() {
							self.contact_addresses_popup.finalize();
						}
					);

					// instancia el popup
					self.contact_addresses_popup = new inConcert.PopupWindow(
						{
							container : $popup,
							ownerId   : popup_id,          // container id
							eventName : 'ev_' + popup_id,  // evento click personalizado
							parentNode: $btn,
							canReposition: true,
							callbacks: {
								onClose : function() {
									window.setTimeout(function() {
										self.contact_addresses_popup = null;
										$popup.remove();
									}, 100);
								}
							}
						}
					);
					// visualiza el popup
					self.contact_addresses_popup.show();
				}
			};
			var addresses = {
				Phone   : [
					{ list: [], type: 'CALL',     tooltip: "<%=_T('Call')%>",     className: 'buttonColorWhite butContactInteractionCall',     handler: defaultHandler, permission_id: 'int_create_outgoing' },
					{ list: [], type: 'SMS',     tooltip: "<%=_T('Sms')%>",     className: 'buttonColorWhite butContactInteractionSms',     handler: defaultHandler, permission_id: 'int_create_outgoing' },
					{ list: [], type: 'RCS',     tooltip: "<%=_T('Rcs')%>",     className: 'buttonColorWhite butContactInteractionRcs',     handler: defaultHandler, permission_id: 'int_create_outgoing' },
					{ list: [], type: 'WHATSAPP',     tooltip: "<%=_T('Whatsapp')%>",     className: 'buttonColorWhite butContactInteractionWhatsapp',     handler: defaultHandler, permission_id: 'int_create_outgoing' }
				],
				IM      : { list: [], type: 'IM',       tooltip: "<%=_T('IM')%>",       className: 'buttonColorWhite butContactInteractionIM',       handler: defaultHandler, permission_id: 'int_create_outgoing' },
				Mail    : { list: [], type: 'MAIL',     tooltip: "<%=_T('E-mail')%>",     className: 'buttonColorWhite butContactInteractionMail',     handler: defaultHandler, permission_id: 'int_create_outgoing' }
				//Twitter : { list: [], type: 'TWITTER',  tooltip: "<%=_T('Twitter')%>",  className: 'buttonColorWhite butContactInteractionTwitter',  handler: defaultHandler, permission_id: 'int_create_outgoing' }
			};
			
			var processAddress = function(address, conf_address) {
				var has_license = inConcert.app.hasLicensesForChannel( conf_address.type );
				if ( has_license ) {
					var has_permission = inConcert.permissions.to_action_on_any_campaign( 'interaction', conf_address.permission_id );
					if ( has_permission ) {
						conf_address.list.push(address);
					}
				}
			};

			_.each(contact.addresses, function(address)
			{
				var conf_address = addresses[address.Type];
				if (!conf_address) {
					return;
				}
				if ($.isArray(conf_address)) {
					_.each(conf_address, function(conf_address_d) {
						processAddress(address, conf_address_d);
					});
				}
				else {
					processAddress(address, conf_address);
				}
			});

			var setHandler = function(container, address) {
				container.find(format('[action_type="new_interaction"][address_type={0}]', address.type)).click(function() {
					address.handler( $(this) );
				});
			};

			//now create the div according to the
			var addressContainer = actionsPanel.find('.contactInteractions');

			// Se deshabilita el boton cuando el permiso int_visualize_address esta deshabilitado
			const visualize_address = inConcert.app.canVisualizeAddress();

			var makeAddressButton = function(address) {
				if (address.list.length > 0) {
					addressContainer.append(format(
						"<button action_type='new_interaction' address_type='{0}' tooltip='{1}' type='button' class='{2}' {3} {4}><i></i></button>",
						address.type, address.tooltip, address.className, !visualize_address? "title='<%=_T('You do not have permission to see the addresses')%>'":"", !visualize_address? "disabled":""));
					setHandler(addressContainer, address);
				}
			};

			_.each(addresses, function(address){

				if ($.isArray(address)) {

					_.each(address, function(address_d) {
						makeAddressButton(address_d);
					});
				}
				else {
					makeAddressButton(address);
				}
			});

			var $favorite = actionsPanel.find('.butContactOptionFavorite');
			var $unfavorite = actionsPanel.find('.butContactOptionUnFavorite');

			if ( inConcert.permissions.to_action_on_any_campaign( 'interaction', 'int_favorite_contact' ) ) {
				$favorite.show()
					.click(function() {
						var node = $(this);
						var contactNode = node.parent().parent().parent();
						self.favoriteContactFn(node, contactNode);
					});

				$unfavorite.show()
					.click(function() {
						var node = $(this);
						var contactNode = node.parent().parent().parent();
						self.unFavoriteContactFn(node, contactNode);
					});

			}
			else {
				$favorite.hide().unbind( 'click' );
				$unfavorite.hide().unbind( 'click' );
			}

			contactNode.append(actionsPanel);
			contactNode.addClass('boxContactItemSelect');
			self.m_selectedNode = contactNode;
		}
	};

	ContactPanel.prototype.getContactMarkup = function(contact) {
		if (typeof(contact) !== 'object') {
			return;
		}
		var online = contact.getOnlineStatus();
		var contactId = inConcert.Utils.getNormalizedId(contact.Id);
		if (!contactId) {
			contactId = contact.Id;
		}
		const contactName = (contact.Name || "").length === 0 || contact.Name.toLowerCase() == "no name"? "<%=_T('No Name')%>" : contact.Name;
		return "<div id='contact_node_" + contactId + "' class='boxContactItem " + (online ? "boxContactOnLine" : "boxContactUnknown") + "'>" +
					"<div class='contactAvatar'><img src='" + inConcert.app.getBaseUrl() + contact.getAvatarURI() + "'/></div>" +
					"<div class='contactInfo'>" +
						"<div class='boxContactIdentificator' title='" + contact.Name + "'>" + contactName + "</div>" +
					"</div>" +
				"</div>";
	};

	ContactPanel.prototype.getTeamMarkup = function(member) {
		var online = member.isOnline();
		var stateClass;
		var statusName = online ? inConcert.app.getStatusDescription(member.status.state) : "<%=_T('Offline')%>";
		stateClass = 'boxContact' + inConcert.app.getMainWindow().getStateQualifier(member.status.state);
		var memberId = inConcert.Utils.getNormalizedId(member.Id);
		if (!memberId) {
			memberId = member.Id;
		}
		var quickMessage = '';
		if (member.StatusMessage) {
			quickMessage = inConcert.Utils.html2text(member.StatusMessage || statusName || '');
		}
		if (!quickMessage || quickMessage.length === 0) {
			quickMessage = statusName;
		}
		var statusDate = parseInt(member.status.lastStatusChangeTime * 1000, 10);
		var exactTimeInState = "<%=_T('Never')%>";
		var timeInState = "<%=_T('Never')%>";
		if (statusDate) {
			exactTimeInState = inConcert.Utils.getTimeDifference(new Date(statusDate));
			timeInState = inConcert.Utils.timeAgo(new Date(statusDate));
		}
		var html_status_message = format("<span>({1})</span> {2}", exactTimeInState, timeInState, statusName);
		var html_member_name = format("<span>{1}</span>", member.get_full_name(), member.get_name_and_surname());

		return "<div id='member_node_" + memberId + "' user_type='member' class='boxContactFavoriteItem " + stateClass + "'>" +
					"<div class='contactAvatar'>" +
						"<img src='" + inConcert.app.getBaseUrl() + "/users/get_avatar/" + member.Id + "' width='21px' height='21px' />" +
					"</div>" +
					"<div class='contactInfo'>" +
						"<div class='boxContactIdentificator'>" + html_member_name + "</div>" +
						"<div class='contactNick'>" + quickMessage + "</div>" +
						"<div class='contactStatus' date='" + statusDate + "' status='" + statusName + "'>" + html_status_message + "</div>" +
					"</div>" +
				"</div>";
	};

	ContactPanel.prototype.validateBasicData = function(contact) {
		var $errorMessage = $('#contactMainErrorMessage');
		$errorMessage.text('').hide();

		var showError = function(message) {
			$errorMessage.text(message).show();
			setTimeout(function() {
				$errorMessage.hide();
			}, 3000);
			return false;
		};

		if (contact.Name.length === 0) {
			return showError ("<%=_T('Please enter the contact name')%>");
		}
		else if (!contact.Name.match(/^[^;]*$/)) {
			return showError("<%=_T('The character ; not allowed in the contact name')%>");
		}
		else if (contact.campaigns.length === 0) {
			return showError ("<%=_T('The contact must belong at least to one group')%>");
		}
		else if (!contact.Company.match(/^[\-a-zA-Z 0-9_.]*$/)) {
			return showError("<%=_T('Only letters, numbers, dashes and dots are allowed in the contact company')%>");
		}
		return true;
	};

	// Adds a contact to the panel if the current view corresponds to the 'view' parameter.
	// The contact is added to the bottom of the list unless parameter 'first' is true.
	ContactPanel.prototype.addContact = function(contact, view, first)
	{
		var self = this;
		if (!contact || view !== this.m_currentView) {
			return;
		}
		if ( view !== this.m_views.CONTACTS && view !== this.m_views.TEAM ) {
			return;
		}
		var onSelectedFn = this.onContactSelected;
		var nodeMarkup;

		// TO-DO: This is too much tightly coupled to the view.
		switch (view) {
			case this.m_views.TEAM:
				nodeMarkup = this.getTeamMarkup(contact);
				onSelectedFn = this.onTeamMemberSelected;
				break;
			case this.m_views.CONTACTS:
				nodeMarkup = this.getContactMarkup(contact);
				break;
			default:
				throw('Undefined contact panel view');
		}

		var contactNode = $(nodeMarkup);

		contactNode.data('contact', contact);

		//only when the user clicks the name the selection is triggered
		contactNode.find('.boxContactIdentificator')
			.unbind('click')
			.bind('click', function(){
				// FIX-ME: Hack ahead, the this scope is all clunky on this file
				onSelectedFn.call(self, contactNode);
		});

		var contactList = this.m_listContainer.children();
		if (contactList) {
			$('#contactPanelSearchMessages').hide();
		}
		if (first) {
			var firstContact = contactList.first();
			if (firstContact[0]) {
				contactNode.insertBefore(firstContact);
			}
			else {
				this.m_listContainer.append(contactNode);
			}
			contactNode.find('.boxContactIdentificator').click();
		}
		else {
			this.m_listContainer.append(contactNode);
		}
		return contactNode;
	};

	//clear the current panel from any entry loaded
	ContactPanel.prototype.clearListContainer = function() {
		var self = this;
		self.m_listContainer.children().remove();
	};

	//Shows a contact in the panel, if it's not already loaded goes to the server to find it..
	ContactPanel.prototype.showContact = function(contactID) {
		var self = this;
		var current_view = self.m_currentView; // we capture the current view to make sure the user don't change this member.

		if (current_view !== self.m_views.CONTACTS) {
			self.activateContacts();
		}

		//iterate over the loaded contacts, if found open that contact detail, if not, go search for it
		var loaded = self.m_listContainer.children();
		for (var i = 0; i < loaded.length; i++) {
			var contactNode = $(loaded[i]);
			var contact = contactNode.data('contact');
			if (contact.Id === contactID) {
				self.openContactDetail(contact, contactNode);
				return;
			}
		}
		inConcert.app.getContactManager().getContact(contactID, function(contact){
			if (contact) {
				var contactNode = self.addContact(contact, current_view);
				self.openContactDetail(contact, contactNode);
			}
		});
	};

	ContactPanel.prototype.updatedContact = function(contact) {
		var self = this;
		if (!contact) {
			return;
		}
		var contactId = inConcert.Utils.getNormalizedId(contact.Id);
		if (!contactId) {
			contactId = contact.Id;
		}
		var contactDetailNode = self.m_handler.find('.contactDetailId');
		var detailFavoriteButton = contactDetailNode.find('#butContactOptionFavorite');

		if ( !inConcert.permissions.to_action_on_any_campaign( 'interaction', 'int_favorite_contact' ) ) {
			detailFavoriteButton.hide();
			return;
		}
		detailFavoriteButton.show();

		var contactNode = $('#contact_node_' + contactId);
		var isFavorite = contact.isUserFavorite();
		//solapa contactos
		if (isFavorite) {
			contactNode.find('.butContactOptionFavorite')
				.attr('tooltip', "<%=_T('UnFavorite')%>")
				.addClass('butContactOptionUnFavorite')
				.removeClass('butContactOptionFavorite')
				.unbind('click')
				.bind('click', function () {
					self.unFavoriteContactFn($(this), contact);
				});

			detailFavoriteButton
				.attr('tooltip', "<%=_T('UnFavorite')%>")
				.addClass('butContactOptionUnFavorite')
				.removeClass('butContactOptionFavorite')
				.unbind('click')
				.bind('click', function () {
					self.unFavoriteContactFn($(this), contact);
				});
		}
		else {
			contactNode.find('.butContactOptionUnFavorite')
				.attr('tooltip', "<%=_T('Favorite')%>")
				.addClass('butContactOptionFavorite')
				.removeClass('butContactOptionUnFavorite')
				.unbind('click')
				.bind('click', function () {
					self.favoriteContactFn($(this), contact);
				});
			detailFavoriteButton
				.attr('tooltip', "<%=_T('Favorite')%>")
				.addClass('butContactOptionFavorite')
				.removeClass('butContactOptionUnFavorite')
				.unbind('click')
				.bind('click', function () {
					self.favoriteContactFn($(this), contact);
				});
		}
	};

	// PAGINATION -------------------
	//
	ContactPanel.prototype.__set_pagination_events = function( search_fn )
	{
		search_fn = search_fn || $.noop;
		var self = this;

		var page_buttons = [
			{ selector: '#int_shared_panel_contact_member_paginated_page_first' , tooltip: _T('First Page')    },
			{ selector: '#int_shared_panel_contact_member_paginated_page_next'  , tooltip: _T('Next Page')     },
			{ selector: '#int_shared_panel_contact_member_paginated_page_prev'  , tooltip: _T('Previous Page') }
		];

		page_buttons.forEach(function(button)
		{
			var $button_node = $( button.selector );

			$button_node
				.unbind('click_navigate_to')
				.bind('click_navigate_to', function()
				{
					var $button = $(this);

					var query = self.current_query_filter[self.m_currentView];

					if ( query )
					{
						var go_to = $button.attr('go_to') || '';

						switch(go_to)
						{
							case 'first':
								query.page_number = 1;
							break;

							case 'previous':
								if ( query.page_number > 1 ) {
									query.page_number--;
								}
							break;

							case 'next':
								query.page_number++;
							break;

							default:
								throw('ContactPanel.prototype.__set_pagination_events: Unrecognized pagination button: "' + go_to + '"');
						}
					}
					// ejecuto la busqueda de los datos de la pagina seleccionada
					var is_new_search = false;
					self.search(is_new_search);
				});

			$button_node
				.attr('tooltip', button.tooltip)
				.unbind('click')
				.bind('click', function(){
					$(this).trigger('click_navigate_to');
				});
		});
	};

	ContactPanel.prototype.__update_current_pagination = function( force_disabled )
	{
		var current_filter = this.current_query_filter[this.m_currentView];
		// update navigation buttons page
		$('#int_shared_panel_contact_member_paginated_page_first').prop('disabled', force_disabled || current_filter.page_number <= 1);
		$('#int_shared_panel_contact_member_paginated_page_prev').prop('disabled', force_disabled || current_filter.page_number <= 1);
		$('#int_shared_panel_contact_member_paginated_page_next').prop('disabled', force_disabled || current_filter.count_users_in_pages < current_filter.rows_per_page );
	};
}());
