/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, esversion: 11 */

/*global $, inConcert, Mustache */

// TO-DO: General TODO, remove long lines as much as possible.
// TO-DO: General TODO, update documentation of public/private me

// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.
if (!this.inConcert) {
	this.inConcert = {};
}

/**
 * @namespace
 * @name ContactBasicInfo
 * @author Sebastián Moreno (smoreno@inconcertcc.com)
 */
(function () {

	'use strict';

	const format = inConcert.Utils.format;
	let m_telephoneCodes = null;

	/**
	 * Creates a new contact info instance.
	 *
	 * @constructor
	 * @memberOf ContactBasicInfo
	 */
	function ContactBasicInfo(view_prefix, contact_window)
	{
		// we allow just one window instance with the same prefix at time.
		this.m_prefix = view_prefix;
		const $previous_window = this.get_contact_window();
		if ($previous_window.length > 0) {
			$previous_window.remove();
		}
		this.m_templates = inConcert.ContactInformationTemplates;
		this.m_contact_window = contact_window;

		m_telephoneCodes = inConcert.app.getInstanceTelephonyCodes();
	}

	//Define functions and objects in the namespace in case they don't already exist
	if (typeof inConcert.ContactWindowBasicInfo !== 'function') {
		inConcert.ContactWindowBasicInfo = ContactBasicInfo;
	}

	/**
	 * Get the main contact window node.
	 *
	 * @public
	 * @return {Object} - The jQuer object for the contact window instantiated
	 */
	ContactBasicInfo.prototype.get_contact_window = function() {
		return $('#contact_window_' + this.m_prefix);
	};

	ContactBasicInfo.prototype.__get_contact_window_container = function() {
		return $('#contact_window_container_' + this.m_prefix);
	};

	ContactBasicInfo.prototype.__get_address_list = function() {
		return $('#contact_window_address_list_'+ this.m_prefix);
	};

	ContactBasicInfo.prototype.__clean_addresses_list = function(cached_list) {
		cached_list = cached_list ? cached_list : this.__get_address_list();
		return cached_list.children('.addressItem').remove();
	};

	ContactBasicInfo.prototype.__get_address_empty_message = function() {
		return $('#contact_window_addresses_empty_' + this.m_prefix);
	};

	ContactBasicInfo.prototype.__get_contact_save_button = function() {
		return $('#contact_window_save_' + this.m_prefix);
	};

	ContactBasicInfo.prototype.__get_contact_change_button = function() {
		return $('#contact_window_change_' + this.m_prefix);
	};

	ContactBasicInfo.prototype.__get_contact_cancel_button = function() {
		return $('#contact_window_cancel_' + this.m_prefix);
	};

	ContactBasicInfo.prototype.__get_contact_delete_button = function() {
		return $('#contact_window_delete_' + this.m_prefix);
	};

	ContactBasicInfo.prototype.__get_contact_favorite_button = function() {
		return $('#contact_window_toggle_favorite_' + this.m_prefix);
	};

	ContactBasicInfo.prototype.__enable_save_create_contact = function() {
		this.__get_contact_save_button().prop('disabled', false);
	};

	ContactBasicInfo.prototype.__disable_save_create_contact = function() {
		this.__get_contact_save_button().prop('disabled', true);
	};

	ContactBasicInfo.prototype.__get_contact_merge_button = function() {
		return $('#contact_window_merge_' + this.m_prefix);
	};

	ContactBasicInfo.prototype.__get_contact_avatar = function() {
		return $('#contact_window_basic_info_avatar_' + this.m_prefix);
	};

	ContactBasicInfo.prototype.__get_contact_name = function() {
		return $('#contact_window_contact_name_' + this.m_prefix);
	};

	ContactBasicInfo.prototype.__get_contact_name_editable = function() {
		return $('#contact_window_name_editable_' + this.m_prefix);
	};

	ContactBasicInfo.prototype.__get_contact_name_edit_btn = function() {
		return $('#contact_window_name_edit_button_' + this.m_prefix);
	};

	ContactBasicInfo.prototype.__get_contact_name_save_btn = function() {
		return $('#contact_window_name_edit_save_' + this.m_prefix);
	};

	ContactBasicInfo.prototype.__get_contact_name_cancel_btn = function() {
		return $('#contact_window_name_edit_cancel_' + this.m_prefix);
	};

	ContactBasicInfo.prototype.__get_contact_external_id = function() {
		return $('#contact_window_external_id_editable_' + this.m_prefix);
	};

	ContactBasicInfo.prototype.__show_address_empty_message = function() {
		var $message = this.__get_address_empty_message();
		$message.show();
	};

	ContactBasicInfo.prototype.__hide_address_empty_message = function() {
		var $message = this.__get_address_empty_message();
		$message.hide();
	};

	/**
	 * Given an address type, will return an object with detailed
	 * actions based on the address.
	 *
	 * @private
	 *
	 * @param  {String} address_type
	 *
	 * @return {Object} address_info
	 * @return {String} address_info.contact
	 * @return {String} address_info.edit
	 * @return {String} address_info.remove
	 * @return {String} address_info.cancel
	 * @return {String} address_info.save
	 */
	ContactBasicInfo.prototype.__get_info_by_address = function(address_type) {
		switch(address_type) {
			case 'IM':
				return {
					contact : "<%=_T('Start a new IM chat to {0}')%>",
					edit : "<%=_T('Edit IM address')%>",
					remove : "<%=_T('Remove IM address')%>",
					cancel : "<%=_T('Cancel edit')%>",
					save : "<%=_T('Save IM address')%>",
					new_message: "<%=_T('IM')%>"
				};
			case 'Twitter':
				return {
					contact : "<%=_T('Tweet to @{0}')%>",
					edit : "<%=_T('Edit Twitter username')%>",
					remove : "<%=_T('Remove Twitter username')%>",
					cancel : "<%=_T('Cancel edit')%>",
					save : "<%=_T('Save Twitter username')%>",
					new_message: "<%=_T('Twitter')%>"
				};
			case 'Facebook':
				return {
					contact : "<%=_T('New post to {0}')%>",
					edit : "<%=_T('Edit Facebook user')%>",
					remove : "<%=_T('Remove Facebook user')%>",
					cancel : "<%=_T('Cancel edit')%>",
					save : "<%=_T('Save Facebook user')%>",
					new_message: "<%=_T('Facebook')%>"
				};
			case 'Instagram':
				return {
					contact : "<%=_T('New post to {0}')%>",
					edit : "<%=_T('Edit Instagram user')%>",
					remove : "<%=_T('Remove Instagram user')%>",
					cancel : "<%=_T('Cancel edit')%>",
					save : "<%=_T('Save Instagram user')%>",
					new_message: "<%=_T('Instagram')%>"
				};
			case 'Youtube':
				return {
					contact : "<%=_T('New post to {0}')%>",
					edit : "<%=_T('Edit Youtube user')%>",
					remove : "<%=_T('Remove Youtube user')%>",
					cancel : "<%=_T('Cancel edit')%>",
					save : "<%=_T('Save Youtube user')%>",
					new_message: "<%=_T('Youtube')%>"
				};
			case 'Appstore':
				return {
					contact : "<%=_T('New post to {0}')%>",
					edit : "<%=_T('Edit Appstore user')%>",
					remove : "<%=_T('Remove Appstore user')%>",
					cancel : "<%=_T('Cancel edit')%>",
					save : "<%=_T('Save Appstore user')%>"
				};
			case 'Googleplay':
				return {
					contact : "<%=_T('New post to {0}')%>",
					edit : "<%=_T('Edit Googleplay user')%>",
					remove : "<%=_T('Remove Googleplay user')%>",
					cancel : "<%=_T('Cancel edit')%>",
					save : "<%=_T('Save Googleplay user')%>"
				};
			case 'Mail':
				return {
					contact : "<%=_T('Write email to {0}')%>",
					edit : "<%=_T('Edit e-mail')%>",
					remove : "<%=_T('Remove e-mail')%>",
					cancel : "<%=_T('Cancel edit')%>",
					save : "<%=_T('Save new e-mail')%>",
					new_message: "<%=_T('E-mail')%>"
				};
			case 'Phone':
			case 'Cellular':
				return {
					buttons : [
						{ type : "CALL", text : "<%=_T('Call')%>" },
						{ type : "SMS", text : "<%=_T('Sms')%>" },
						{ type : "WHATSAPP", text : "<%=_T('Whatsapp')%>" }
					],
					contact : "",
					edit : "<%=_T('Edit phone number')%>",
					remove : "<%=_T('Remove phone number')%>",
					cancel : "<%=_T('Cancel edit')%>",
					save : "<%=_T('Save phone number')%>"
				};
				/* falls through */
			default:
				throw('Invalid address type' + address_type);
		}
	};

	ContactBasicInfo.prototype.__make_phone_custom_key = function(key)
	{
		if (!key) {
			return;
		}
		//return format("CUSTOM_{0}", key).toUpperCase();
		return key;
	};

	ContactBasicInfo.prototype.__get_phone_custom_type = function()
	{
		if (inConcert.app.has_allegro_integration()) {
			return {id: "CUSTOM", desc: "<%=_T('Custom')%>"};
		}
		return;
	};

	ContactBasicInfo.prototype.__get_phone_types = function(kinds)
	{
		let phone_types = {
			CELLULAR: {id: "CELLULAR", desc: "<%=_T('Cellular')%>"},
			FAX     : {id: "FAX"     , desc: "<%=_T('Fax')%>"     },
			HOME    : {id: "HOME"    , desc: "<%=_T('Home')%>"    },
			OFFICE  : {id: "OFFICE"  , desc: "<%=_T('Office')%>"  }
		};

		(kinds || []).forEach(kind => {
			if (kind !== "" && !phone_types[kind]) {
				let phone_custom_type = this.__get_phone_custom_type();
				if (phone_custom_type) {
					let phone_type = phone_custom_type;
					phone_type.desc = phone_type.desc + " '" + kind + "'";
					const custom_key = this.__make_phone_custom_key(kind);
					if (custom_key) {
						phone_type.id = kind;
						phone_types[custom_key] = phone_type;
					}
				}
			}
		});

		return phone_types;
	};

	ContactBasicInfo.prototype.__get_phone_type = function(kind)
	{
		if (typeof(kind) !== "string") {
			console.warn("ContactBasicInfo.prototype.__get_phone_type - Phone kind not as string. Kind: " + kind);
			return;
		}
		kind = kind === "" ? "HOME" : kind; // Si el kind es "" deja el por defecto.
		const phone_types = this.__get_phone_types();
		let phone_type = phone_types[kind.toUpperCase()];
		if (!phone_type) {
			const all_phone_types = this.__get_phone_types([kind]); // incluye los customs
			const custom_key = this.__make_phone_custom_key(kind);
			if (custom_key) {
				phone_type = all_phone_types[custom_key];
				if (!phone_type) {
					console.warn("ContactBasicInfo.prototype.__get_phone_type - Unrecognized phone kind: " + kind);
				}
			}
		}
		return phone_type;
	};

	/**
	 * Will build the markup for an address node
	 *
	 */
	ContactBasicInfo.prototype.$build_address_node = function(address_info)
	{
		const self = this;

		let address_displayable = address_info.displayable;
		let address_type = (address_info.Type || "");
		const is_phone_address = address_type.toUpperCase() === "PHONE";
		let info = this.__get_info_by_address(address_type);
		let multiple_channels = false;
		let address_buttons = "";
		let localNumber = null;
		let phoneNumberWithPrefix = false;

		if (is_phone_address) {
			localNumber = address_info.SimpleNumber || address_info.LocalNumber || null;
			let firstNumber = address_displayable.substring(0, 1);
			if (firstNumber === "+") {
				phoneNumberWithPrefix = "+";
			}
		}

		$.each(info.buttons || [], function(_, button) {
			multiple_channels = true;
			if (!inConcert.app.hasLicensesForChannel(button.type)) {
				return;
			}
			address_buttons += Mustache.render(self.m_templates.contact_address_item, {
				type : button.type,
				text : format(button.text, address_displayable),
				className : button.type.to_camel()
			});
		});

		let $address = $(Mustache.render(this.m_templates.contact_address_item_tpl, {
			address             : localNumber || address_displayable,
			address_type        : address_type,
			contact_tooltip     : format(info.contact, address_displayable),
			edit_tooltip        : format(info.edit   , address_displayable),
			save_tooltip        : format(info.save   , address_displayable),
			cancel_close_tooltip: format(info.cancel , address_displayable),
			delete_tooltip      : format(info.remove , address_displayable),
			actions_tooltip     : "<%=_T('Actions')%>"
		}));

		if (phoneNumberWithPrefix) {
			$address.attr("phoneNumberWithPrefix", "+");
		}

		let kind = this.__get_phone_type(address_info.Kind);
		if (is_phone_address) {
			if (kind) {
				let $kind = $address.find("[phone_kind]");
				$kind.text(kind.desc);
				$kind.attr("kind_id", kind.id);
				$kind.show();
			}
			//if (!inConcert.app.has_allegro_integration()) {
				// countries & area
				let $cbo_countries = $address.find("[address_country]");
				let $cbo_areas = $address.find("[address_area]");
				let params = {
					$cboCountries        : $cbo_countries,
					$cboAreas            : $cbo_areas,
					can_show_country_code: true
				};
				m_telephoneCodes.fillCountriesCombo(params, function(/*success*/) {
					// if (!success) {
					// 	// TODO: hacer algo ?
					// }
					$cbo_countries.val(address_info.Country);
					$cbo_countries.trigger("change");
					$cbo_countries.css("display", "");
					$cbo_areas.val(address_info.Area);
					$cbo_areas.css("display", "");
				});
			//}
		}

		$address.data("address_info", address_info);
		// cargo el combo editable con los phone kind
		if (is_phone_address) {
			let $editable_phone_kind = $address.find('[editable_phone_kind]');
			let cbo_id = $editable_phone_kind.attr("id");
			if (!cbo_id) {
				// Tengo que agregar un ID aleatorio ya que la funcion generica de Utils que ordena los combos necesita un ID en el combo.
				$editable_phone_kind.attr("id", inConcert.Utils.generateGuid());
			}
			let phone_types = this.__get_phone_types([address_info.Kind]);
			inConcert.Utils.fill_combo($editable_phone_kind, false, phone_types, "id", "desc", true);
		}

		const $interactionButton = $address.find("[new_address_interaction_button]");
		if (multiple_channels) {
			$interactionButton.data("multiple_channels", true);
			if (address_buttons) {
				const menuId = inConcert.Utils.generateGuid();
				const $menu = $(Mustache.render(this.m_templates.contact_addresses_menu, {
					id : menuId,
					options : address_buttons
				}));
				$("body").append($menu);
				$interactionButton.data("menu", menuId);
			}
			else {
				$interactionButton.hide();
			}
		}
		else {
			$interactionButton.addClass("butContactInteraction" + inConcert.app.getContactManager().addressTypeToInteractionType(address_type).to_camel());
		}

		$interactionButton.attr("is_multiple_channels", multiple_channels ? "1" : "0");

		return $address;
	};

	/**
	 * Will show or hide basic info containers (initial view)
	 *
	 * TO-DO: This is OK? Its seems pretty odd.
	 * maybe all this info should be in a container?
	 *
	 * @param  {Boolean} showOrHide
	 */
	ContactBasicInfo.prototype.__toggle_contact_basic_info_containers = function(showOrHide)
	{
		$('#contact_window_basic_info_container').toggle(showOrHide);
		$('#contact_window_addressess_container').toggle(showOrHide);
		$('#contact_window_external_id_container').toggle(showOrHide);
		$('#contact_window_contact_actions').toggle(showOrHide);
	};

	/*
		Funcion encargada de abrir el popup/menu de acciones.
		El mismo crea una instancia de "inConcert.PopupWindow" para visualizar el menu.

		Las acciones de cada item (edit, delete y las acciones de create outgoing), se continuna utilizando desde los bindeos que ya existian.
		Esto para no tener que reescribir toda la logica.

		"__setup_address_edition_handlers" (delete y edit)
		"__setup_create_outgoing_handlers" (create outgoing)
	*/
	ContactBasicInfo.prototype.__show_actions_menu =  function($owner)
	{
		const self = this;

		const menu_id = 'contact_address_menu_actions';
		$('#' + menu_id).remove();
		const $menu = $(
			'<div id="' + menu_id + '" class="interactionMenuBlind contactAddressInteractionMenu" style="z-index: 800!important;" hidden></div>'
		);

		// Estas son las acciones del menu
		const menu_options = [
			{id: "open_client", text: "<%=_T('Actions')%>", class_name: "butContactInteraction"},
			{id: "edit"       , text: "<%=_T('Edit')%>"   , class_name: "buttonAddressDelete"                  },
			{id: "remove"     , text: "<%=_T('Remove')%>" , class_name: "buttonAddressEdit"                    }
		];

		$.each(menu_options, function(_, option)
		{
			const $option = $("<div class='interactionMenuItem'></div>");
			$option.attr("action_id", option.id);
			$option.text(option.text);
			$option.addClass(option.class_name);

			const $owner_parent = $owner.parent();
			// Esto es para la accion de crear salientes.
			if (option.id === "open_client") {
				const address_type = $owner_parent.attr("address_type");
				const $hidden_action = $owner_parent.find('[action_id="' + option.id + '"]');
				const is_multiple_channels = $hidden_action.attr("is_multiple_channels");
				if (is_multiple_channels === "0") {
					// Cuando NO se abre el submenu, es decir solo podemos crear un tipo de salientes.
					$option.addClass("butContactInteraction" + inConcert.app.getContactManager().addressTypeToInteractionType(address_type).to_camel());
					const info = self.__get_info_by_address(address_type);
					$option.text(info.new_message || "");
				}
			}

			$option.click(function() {
				const $this = $(this);
				const action_id = $this.attr("action_id");
				const $hidden_action = $owner_parent.find('[action_id="' + action_id + '"]');

				if ($hidden_action.length === 0) {
					// TODO: ALGO ? no deberia suceder
					return;
				}

				self.__set_submenu_action_owner_btn(null);
				const is_multiple_channels = $hidden_action.attr("is_multiple_channels");
				if (is_multiple_channels === "1") {
					// Para el caso del submenu de acciones, tengo que asignar el owner.
					self.__set_submenu_action_owner_btn($this);
					// tambien tengo que flaguear el popup para que NO se cierre de forma automatica cuando se abre el submenu
					self.m_contact_address_menu.setClosedAutomatic(false);
				}
				else {
					self.m_contact_address_menu.setClosedAutomatic(true);
				}
				// ejecuto la accion desde un trigger click que se ha biendado en "__setup_address_edition_handlers" o "__setup_create_outgoing_handlers"
				$hidden_action.trigger("click");
				window.setTimeout(function() {
					// despues de un interavalo deja el popup del menu para que se pueda cerrar de forma automatica.
					self.m_contact_address_menu.setClosedAutomatic(true);
				}, 50);
			});

			// agrego la opcion al menu
			$menu.append($option);
		});

		/*
			Aca es donde instancio y visualizo el popup con el menu de acciones.
		*/
		$('body').append($menu);
		this.m_contact_address_menu = new inConcert.PopupWindow({ownerId: menu_id});
		this.m_contact_address_menu.show(
		{
			ownerId       : menu_id,
			eventName     : menu_id,
			canReposition : true,
			container     : $menu,
			parentNode    : $owner,
			align         : "LEFT",
			callbacks: {
				onOpen: function() {
					$menu.show();
				},
				onClose: function(onClose) {
					onClose = onClose || $.noop;
					if (self.m_contact_address_menu) {
						self.m_contact_address_menu.hide(function() {
							onClose();
						});
					}
					else {
						onClose();
					}
				}
			}
		});
	};

	/**
	 * Setup handlers related to contact addresses edition
	 *
	 * @private

	 	 Los eventos que se bindean seran ejecutados desde la funcion "__show_actions_menu" en el menu de acciones.
	 */
	ContactBasicInfo.prototype.__setup_address_edition_handlers = function()
	{
		const self = this;

		let $actions = this.get_contact_window().find('[address_actions_menu]');
		$actions.click(function() {
			const $button = $(this);
			self.__show_actions_menu($button);
		});

		let $edit = this.get_contact_window().find('.addressItem .buttonAddressEdit');
		let $delete = this.get_contact_window().find('.addressItem .buttonAddressDelete');
		if ( !this.m_contact_window.has_permission_to_save() ) {
			$edit.hide();
			$delete.hide();
			return;
		}

		$delete.click(function() {
			let $address = $(this).parent();
			let $editable_address_field = $address.find('[editable_address]');
			let confirm_options = { message: format("<%=_T('Are you sure you want to delete address: {0} ?')%>", $editable_address_field.val()) };
			self.m_contact_window.__show_confirmation_dialog(confirm_options, {
				on_confirmed : function() {
					$address.remove();
					self.__enable_save_create_contact();
					// we make equal one because the message itself is inside the
					// container
					if (self.__get_address_list().children().length === 1) {
						self.__show_address_empty_message();
					}
				}
			});
		});

		//const has_allegro_integration = inConcert.app.has_allegro_integration();

		$edit.click( function()
		{
			let $edit_button = $(this);
			let $address = $edit_button.parent();
			let $previous_address_node = $address.find('[address_id]');
			let $previous_phone_kind = $address.find('[phone_kind]');
			let $editable_address_field = $address.find('[editable_address]');
			let $editable_phone_kind = $address.find('[editable_phone_kind]');
			let $cbo_countries, $cbo_areas;
			let address_info = $address.data("address_info");
			let is_phone_address = (address_info.Type || "").toUpperCase() === "PHONE";

			// countries & areas
			let can_change_phone_country_and_area = false;
			// if (!has_allegro_integration && is_phone_address) {
			 if (is_phone_address) {
				can_change_phone_country_and_area = true;
				$cbo_countries = $address.find("[address_country]");
				$cbo_areas = $address.find("[address_area]");
				$cbo_countries.prop("disabled", false);
				$cbo_areas.prop("disabled", false);
			}

			// botones "..." de todas las direcciones, que abren su popup de acciones
			$actions = self.get_contact_window().find('[address_actions_menu]');
			$actions.hide();

			// botones save & cancel de la address que se ha editado
			let $save_button = $address.find('.buttonAddressSave');
			let $cancel_button = $address.find('.buttonAddressCancel');

			// hide edit and read-only address
			$previous_address_node.hide();
			$previous_phone_kind.hide();
			$edit_button.hide().siblings("[new_address_interaction_button]").addClass("editMode");

			const __go_back_to_contact_edit = function() {
				// countries & areas
				if (can_change_phone_country_and_area) {
					$cbo_countries.prop("disabled", true);
					$cbo_areas.prop("disabled", true);
				}
				// label and inputs
				$editable_address_field.hide();
				$editable_phone_kind.hide();
				$previous_address_node.show();
				$previous_phone_kind.show();
				// buttons
				$edit_button.siblings("[new_address_interaction_button]").removeClass("editMode");
				$save_button.hide();
				$cancel_button.hide();
				// boton "..." de acciones de todas las direcciones.
				$actions.show();
			};

			// keep the previous address
			let previous_address = $previous_address_node.text();
			$editable_address_field.val(previous_address).show().focus();
			// Phone Kind
			if (is_phone_address) {
				$editable_phone_kind.val(address_info.Kind || "HOME");
				$editable_phone_kind.show();
			}

			// CANCEL address button behavior
			$cancel_button.show().click(function() {
				$previous_address_node.text(previous_address);
				__go_back_to_contact_edit();
			});

			// SAVE address button behavior
			$save_button.show().click( function()
			{
				let new_address = $.trim($editable_address_field.val());
				$previous_address_node.show().text(new_address);
				let phone_kind;
				if (is_phone_address) {
					phone_kind = $editable_phone_kind.val();
					let kind = self.__get_phone_type(phone_kind);
					if (kind) {
						$previous_phone_kind.text(kind.desc);
						$previous_phone_kind.attr("kind_id", kind.id);
					}
				}
				// countries & areas
				if (can_change_phone_country_and_area) {
					address_info.Country = $cbo_countries.val() || "";
					address_info.Area = $cbo_areas.val() || "";
					address_info.LocalNumber = new_address || "";
					const phoneNumberWithPrefix = $address.attr("phoneNumberWithPrefix") || "";
					const firstNumber = new_address.substring(0, 1);
					if (firstNumber === "+") {
						new_address = new_address.substring(1);
					}
					new_address = format(
						"{0}{1}{2}{3}",
						phoneNumberWithPrefix,
						address_info.Country,
						address_info.Area,
						new_address
					);
				}
				let address_normalized = inConcert.app.getContactManager().normalizeContactAddress(new_address, address_info.Type, undefined, phone_kind, address_info);
				for (let name in address_normalized) {
					if (address_normalized.hasOwnProperty(name)) {
						let value = address_normalized[name];
						address_info[name] = value;
					}
				}
				$address.data("address_info", address_info);
				// enable save contact button
				self.__enable_save_create_contact();
				// move back to contact view
				__go_back_to_contact_edit();
			});

			// enable cancel and save when the user makes a modification
			$editable_address_field.change(function() {
				self.__get_contact_cancel_button().show();
				self.show_contact_save_button();
			});

			// default action for enter is save
			$editable_address_field.keyup(function(event) {
				if (event.keyCode === 13) {
					$save_button.trigger('click');
				}
			});
		});
	};

	/**
	 * Handlers for "Add new Address" behavior.
	 *
	 * @private
	 */
	ContactBasicInfo.prototype.__setup_add_new_address_handlers = function()
	{
		const self = this;

		const $contact_window_basic_info_container_main = $('#contact_window_basic_info_container_main');
		const $contact_window_title_container = $('#contact_window_title_container_' + this.m_prefix);

		const $add_new_address_button = $('#contact_window_add_address_button_' + this.m_prefix);
		if ( !this.m_contact_window.has_permission_to_save() ) {
			$add_new_address_button.hide();
			return;
		}

		const $add_address_cancel = $('#contact_window_add_address_cancel_' + this.m_prefix);
		const $add_address_save = $('#contact_window_add_address_save_' + this.m_prefix);

		const $address_type_container = $('#contact_window_add_address_type_container_' + this.m_prefix);
		const $address_type_combo = $('#contact_window_add_address_type_' + this.m_prefix);

		const $phone_kind_container = $('#contact_window_add_address_phone_kind_container_' + this.m_prefix);
		const $phone_kind = $('#contact_window_add_address_phone_kind_' + this.m_prefix);

		const $address_input_container = $('#contact_window_add_address_container_' + this.m_prefix);
		const $address_input = $('#contact_window_add_address_' + this.m_prefix);

		const $previous_addresses_container = this.__get_address_list();
		const $address_actions_container = $('#contact_window_address_actions_container_' + this.m_prefix);

		const $contact_actions_container = $('#contact_window_contact_actions_' + this.m_prefix);
		const $contact_external_id_container = $('#contact_window_external_id_container_' + this.m_prefix);

		const $contact_followers_container = $('#contact_window_followers_total_container_' + this.m_prefix);

		const $country_area_container = $('#contact_window_add_address_phone_country_area_container_' + this.m_prefix);
		const $cbo_countries = $('#contact_window_add_address_phone_country_' + this.m_prefix);
		const $cbo_areas = $('#contact_window_add_address_phone_area_' + this.m_prefix);

		const has_allegro_integration = inConcert.app.has_allegro_integration();
	//	if (!has_allegro_integration) {
			// countries & area
			let params = {
				$cboCountries        : $cbo_countries,
				$cboAreas            : $cbo_areas,
				can_show_country_code: true
			};
			m_telephoneCodes.fillCountriesCombo(params, function(success) {
				// TODO: hacer algo ?
				if (!success) {
					return;
				}
			});
	//	}

		const __move_back_to_contact_view = function() {
			// show needed buttons
			$previous_addresses_container.show();
			$add_new_address_button.show();

			// hide all address related buttons
			$add_address_cancel.hide();
			$add_address_save.hide();
			$address_type_container.hide();
			$phone_kind_container.hide();
			$address_input_container.hide();
			$country_area_container.hide();

			// show global contact actions buttons
			$contact_actions_container.show();
			$contact_external_id_container.show();
			$contact_followers_container.show();

			// hide address actions
			$address_actions_container.hide();

			// back button
			$contact_window_title_container.hide();

			// address container
			$contact_window_basic_info_container_main.show();
		};

		const toggle_phone_info = function() {
			if ($address_type_combo.val().toUpperCase() === "PHONE") {
				$phone_kind_container.show();
				if (!has_allegro_integration) {
					$country_area_container.show();
				}
			}
			else {
				$phone_kind_container.hide();
				$country_area_container.hide();
			}
		};

		const $title = $contact_window_title_container.find("[action_title]");
		// Users clicks "Add New Address" button
		$add_new_address_button.click(function()
		{
			// clear previous inputs
			$address_input.val('');

			// hide buttons and containers
			$previous_addresses_container.hide();
			$add_new_address_button.hide();

			// show new buttons
			$address_type_container.show();
			$address_input_container.show();

			$address_input.focus();
			if ($address_type_combo.val().toUpperCase() === "PHONE") {
				$phone_kind.val("HOME");
			}
			// visualiza/oculta los controles address de telefonia
			toggle_phone_info();

			$add_address_save.show();
			$add_address_cancel.show();

			// show address actions
			$address_actions_container.show();

			// back button
			$title.text("<%=_T('Add Address')%>");
			$contact_window_title_container.show();

			// address container
			$contact_window_basic_info_container_main.hide();

			// hide global contact actions buttons
			$contact_actions_container.hide();
			$contact_external_id_container.hide();
			$contact_followers_container.hide();
		});

		// CANCEL address button
		$add_address_cancel.click(function() {
			__move_back_to_contact_view();
		});

		// back button
		const $back = $contact_window_title_container.find("[action_back]");
		$back.click(function() {
			__move_back_to_contact_view();
		});

		// ADD/SAVE address button
		$add_address_save.click(function() {
			self.m_contact_window.hide_message();
			// append new address to list
			let address = {
				displayable: $address_input.val(),
				Type: $address_type_combo.val(),
			};
			if (address.Type.toUpperCase() === "PHONE") {
				address.Kind = $phone_kind.val();
				//if (!has_allegro_integration) {
					address.Country = $cbo_countries.val() || "";
					if (!address.Country) {
						let message_error = "<%=_T('Please select a country.')%>";
						self.m_contact_window.show_message(message_error, 'error');
						$cbo_countries.focus();
						return;
					}
					address.Area = $cbo_areas.val() || "";
					if (address.Area === "__BLANK") {
						let message_error = "<%=_T('Please select a area.')%>";
						self.m_contact_window.show_message(message_error, 'error');
						$cbo_areas.focus();
						return;
					}
			//	}
			}
			// creo el row para el DOM
			let $address = self.$build_address_node(address);
			self.__get_address_list().append($address);

			// enable save contact button
			self.__enable_save_create_contact();

			// hide no addresses message
			self.__hide_address_empty_message();

			// move back
			__move_back_to_contact_view();

			// setup address actions
			self.__setup_address_edition_handlers();
		});

		// ADDRESS INPUT
		$address_input.keyup(function(event) {
			var address_text = $address_input.val();
			$add_address_save.prop('disabled', !address_text);
			if (event.keyCode === 13) {
				$add_address_save.click();
				return;
			}
		});

		$address_type_combo.change(function() {
			const $this = $(this);
			let address_type = $this.val().toUpperCase();
			// IM is an special case, it needs an extra field for protocol.
			$('#contact_window_add_address_protocol').toggle(address_type === 'IM');
			// visualiza/oculta los controles address de telefonia
			toggle_phone_info();
		});
	};

	/**
	 * Will setup edition handlers for contact name
	 */
	ContactBasicInfo.prototype.__setup_name_edition_handlers = function()
	{
		var self = this;

		var $name_edit = this.__get_contact_name_edit_btn();
		if ( !this.m_contact_window.has_permission_to_save() ) {
			$name_edit.hide();
			return;
		}

		var $name_save = this.__get_contact_name_save_btn();
		var $name_cancel = this.__get_contact_name_cancel_btn();

		var $name_editable_field = this.__get_contact_name_editable();
		var $name_field = this.__get_contact_name();

		var __leave_edit_mode = function() {
			$name_editable_field.add($name_cancel).add($name_save).hide();
			$name_field.parent().add($name_edit).show();
		};

		// EDIT
		$name_edit.click(function() {
			// show/hide buttons and fields
			$name_field.parent().add($name_edit).hide();
			$name_editable_field.add($name_save).add($name_cancel).show();
			$name_editable_field.focus();

			// copy current name
			$name_editable_field.val($name_field.text());
		});

		// SAVE
		$name_save.click(function() {
			var new_name = $.trim($name_editable_field.val());
			$name_field.text(new_name);
			self.__enable_save_create_contact();
			__leave_edit_mode();
		});

		// CANCEL
		$name_cancel.click(function() {
			$name_field.text(self.m_contact_name);
			__leave_edit_mode();
		});

		// default action for *enter* key is save
		$name_editable_field.keyup(function(event) {
			if (event.keyCode === 13) {
				$name_save.trigger('click');
			}
		});
	};

	ContactBasicInfo.prototype.show_contact_save_button = function() {
		var $save_button = this.__get_contact_save_button();
		if ( !this.m_contact_window.has_permission_to_save() ) {
			$save_button.hide();
			return;
		}
		$save_button.show();
		return $save_button;
	};

	ContactBasicInfo.prototype.show_contact_merge_button = function() {
		const $merge_button = this.__get_contact_merge_button();
		if ( this.m_contact_window.is_new_contact() || !this.m_contact_window.has_permission_to_merge() ) {
			$merge_button.hide();
			return;
		}
		$merge_button.show();
		return $merge_button;
	};

	ContactBasicInfo.prototype.show_contact_change_button = function() {
		var $change_button = this.__get_contact_change_button();
		if ( this.m_contact_window.is_new_contact() ||  !this.m_contact_window.has_permission_to_change() ) {
			$change_button.hide();
			return;
		}
		$change_button.show();
		return $change_button;
	};

	ContactBasicInfo.prototype.show_contact_delete_button = function() {
		var $delete_button = this.__get_contact_delete_button();
		if ( this.m_contact_window.is_new_contact() || !this.m_contact_window.has_permission_to_delete() ) {
			$delete_button.hide();
			return;
		}
		$delete_button.show();
		return $delete_button;
	};

	ContactBasicInfo.prototype.show_contact_favorite_button = function() {
		var $favorite_button = this.__get_contact_favorite_button();
		if ( !this.m_contact_window.has_permission_to_favorite() ) {
			$favorite_button.hide();
			return;
		}
		$favorite_button.show();
		return $favorite_button;
	};

	/**
	 * Will setup the contact favorite button behavior.
	 *
	 * @param  {Object} contact
	 * @param  {Object} contact.isUserFavorite
	 */
	ContactBasicInfo.prototype.__setup_contact_fav_handlers = function(contact) {
		if (!contact) {
			return;
		}
		var self = this;

		var $favorite_button = this.show_contact_favorite_button();
		if ( !$favorite_button ) {
			return;
		}
		var campaign_id = this.m_contact_window.m_interaction ? this.m_contact_window.m_interaction.campaign : undefined;

		var __update_fav_button = function() {
			var contact_is_favorite = contact.isUserFavorite();
			$favorite_button
				.removeClass(contact_is_favorite ? 'butInteractionFavorites' : 'butInteractionUnFavorites')
				.addClass(contact_is_favorite ? 'butInteractionUnFavorites' : 'butInteractionFavorites')
				.attr('tooltip', contact_is_favorite ? "<%=_T('Unfavorite Contact')%>" : "<%=_T('Favorite Contact')%>")
				.unbind('click')
				.bind('click', function() {
					var method = contact_is_favorite ? 'removeFromFavorites' : 'addToFavorites';
					inConcert.app.getContactManager()[ method ](
						contact, 
						function( succeeded, error ) {
							if ( succeeded ) {
								__update_fav_button();
								return;
							}
							var message_error;
							switch ( error.result.toLowerCase() ) {
								case 'forbidden':
									message_error = "<%=_T('You not has permission to mark favorite/unfavorite contact.')%>";
									break;
								default:
									message_error = "<%=_T('Server error, marking favorite/unfavortie contact.')%>";
							}
							self.m_contact_window.show_message( message_error, 'error' );
						},
						campaign_id, // optional, used by permissions
						self.m_contact_window.get_app_id()
					);
				});
		};
		__update_fav_button();
	};

	ContactBasicInfo.prototype.__create_outgoing_action = function($address_node, proxyType)
	{
		const self = this;

		const addressType = $address_node.attr('address_type').toUpperCase();
		if (addressType === "TWITTER") {
			return;
		}
		if (typeof this.onCreateOutIntCallback === 'function') {
			this.onCreateOutIntCallback();
		}
		else {
			const address_info = $address_node.data("address_info");
			inConcert.app.getContactManager().buildAddress(addressType, address_info, function(address)
			{
				if (address) {
					const panel = inConcert.app.getMainWindow().getPanel('AssignedInteractions');
					proxyType = proxyType || inConcert.app.getContactManager().addressTypeToInteractionType(address.Type);
					const response = panel.createOutgoingInteraction(proxyType, self.m_contact_window.m_contact || null, address, inConcert.app.getCampaigns());

					// if there is an error, show it to the user.
					if (!response.success) {
						let message_error = response.conflicts ? response.conflicts.message : "<%=_T('Unable to create outgoing conversation, please try again later')%>";
						self.m_contact_window.show_message(message_error, 'error');
					}
				}
			});
		}
	};

	/**
	 * Will setup necessary handlers to create interactions from the contact addresses.
	 *
	 * @private
	 
	 	 Los eventos que se bindean sera ejecutados desde la funcion  "__show_actions_menu" en el menu de acciones.
	 */
	ContactBasicInfo.prototype.__setup_create_outgoing_handlers = function()
	{
		const self = this;

		var canCreate = function($button) {
			const channel = $button.parent().attr('address_type').toUpperCase();
			return channel !== "TWITTER";
		};

		const $window = this.get_contact_window();
		$window.find("[new_address_interaction_button]")
			.filter(function() {
				const $button = $(this);
				if (!canCreate($button)) {
					$button.hide();
					return false;
				}
				return true;
			})
			.unbind('click')
			.bind('click', function() {
				const $button = $(this);
				if (!$button.data("multiple_channels")) {
					self.__create_outgoing_action($button.parent());
					return;
				}
				const menuId = $(this).data("menu");
				let menu = $button.data("menu_object");
				if (menu && menu.isVisible()) {
					$button.data("menu_object", null);
					menu.finalize();
					return;
				}
				const $menu = $("#" + menuId);
				$menu.children()
					.off("click")
					.on("click", function() {
						self.__create_outgoing_action($button.parent(), $(this).attr("new_address_interaction"));
						menu.finalize();
						$button.data("menu_object", null);
					});
				menu = new inConcert.PopupWindow({ ownerId: menuId });
				$button.data("menu_object", menu);
				/*
					Obtengo el objeto jQuery owner que ha ejecutado el submenu.
				*/
				const $parentNode = self.__$get_submenu_action_owner_btn() || $button;
				menu.show({
					ownerId       : menuId,
					eventName     : menuId,
					canReposition : true,
					container     : $menu,
					parentNode    : $parentNode,
					align         : "LEFT",
					callbacks: {
						onOpen: function() {
							$menu.show();
						},
						onClose: function( callback ) {
							callback = callback || $.noop;
							if (menu) {
								menu.hide(function() {
									callback();
								});
							}
							else {
								callback();
							}
						}
					}
				});
			}
		);
	};

	/*
		Funcion que obtiene el objeto jQuery owner (quien ha ejecuatdo) del submenu de acciones
	*/
	ContactBasicInfo.prototype.__$get_submenu_action_owner_btn = function()
	{
		return this.$submenu_action_owner_btn;
	};

	/*
		Funcion que asigna el objeto jQuery owner (quien ha ejecuatdo) del submenu de acciones
	*/
	ContactBasicInfo.prototype.__set_submenu_action_owner_btn = function($owner)
	{
		this.$submenu_action_owner_btn = $owner;
	};

	/**
	 * Will setup edition handlers for external id
	 *
	 * @private
	 */
	ContactBasicInfo.prototype.__setup_external_id_edition_handlers = function()
	{
		var self = this;

		var $external_id_edit = $('#contact_window_external_id_edit_' + this.m_prefix);
		if ( !this.m_contact_window.has_permission_to_save() ) {
			$external_id_edit.hide();
			return;
		}

		var $external_id_save = $('#contact_window_external_id_save_' + this.m_prefix);
		var $external_id_cancel = $('#contact_window_external_id_cancel_' + this.m_prefix);

		var $external_id_editable = this.__get_contact_external_id();
		var previous_external_id = $external_id_editable.val();

		var __leave_edit_mode = function() {
			$external_id_editable.addClass('externalIdField-ro').prop('disabled', true);
			$external_id_save.add($external_id_cancel).hide();
			$external_id_edit.show();
		};

		// EDIT
		$external_id_edit.click(function() {
			$external_id_edit.hide();
			$external_id_editable.removeClass('externalIdField-ro').prop('disabled', false).focus();
			$external_id_save.add($external_id_cancel).show();
		});

		// SAVE
		$external_id_save.click(function() {
			// trim the value and update current input
			var new_id = $.trim($external_id_editable.val());
			if (new_id.length === 0) {
				new_id = "<%=_T('N/A')%>";
			}
			$external_id_editable.val(new_id);
			previous_external_id = new_id;

			// enable save contact button
			self.__enable_save_create_contact();

			// go back
			__leave_edit_mode();
		});

		// CANCEL
		$external_id_cancel.click(function() {
			$external_id_editable.val(previous_external_id);
			__leave_edit_mode();
		});


		// default action for *enter* key is save
		$external_id_editable.keyup(function(event) {
			if (event.keyCode === 13) {
				$external_id_save.trigger('click');
			}
		});
	};

	/**
	 * Will setup the *CANCEL* button behavior. Normally remove the container
	 * passed as parameter, otherwise (if it's called from main view) will remove the main
	 * contact window.
	 *
	 * @param  {Function}  - Callback
	 */
	ContactBasicInfo.prototype.setup_contact_cancel_handlers = function(on_canceled)
	{
		this.__get_contact_cancel_button().click(function() {
			on_canceled();
		});
	};

	/**
	 * Will setup the *SAVE* contact handlers
	 *
	 * @public
	 * @param  {Object}  [contact] - Optional contact object
	 *
	 * @param  {Object}  [callbacks]
	 * @param  {Object}  [callbacks.on_saved]
	 *
	 * @param  {Object}  [confirmation]
	 * @param  {Object}  [confirmation.confirm_function]
	 */
	ContactBasicInfo.prototype.setup_contact_save_handlers = function(contact, callbacks, confirmation)
	{
		const self = this;

		const $save_button = this.show_contact_save_button();
		if ( !$save_button ) {
			return;
		}
		let campaign_id = this.m_contact_window.m_interaction ? this.m_contact_window.m_interaction.campaign : undefined;

		const has_allegro_integration = inConcert.app.has_allegro_integration();

		$save_button.unbind('click').bind('click', function()
		{
			let addresses = [];
			let contact_name = self.__get_contact_name().text();

			// get contact addresses
			self.get_contact_window().find('.addressItem').each(function(undefined, e) {
				const $address = $(e);
				// address is already trimmed
				let address = $address.find('[address_id]').text();
				let address_type = $address.attr('address_type');
				let address_protocol = $address.attr('protocol');
				let phone_kind;
				let address_info = $address.data("address_info");

				if (address_type.toUpperCase() === "PHONE") {
					let $kind = $address.find("[phone_kind]");
					phone_kind = $kind.attr("kind_id");
					if (!has_allegro_integration) {
						address = address_info.displayable;
					}
				}

				let address_normalized = inConcert.app.getContactManager().normalizeContactAddress(address, address_type, address_protocol, phone_kind, address_info);
				addresses.push(address_normalized);
			});

			// update external id
			let external_id = self.__get_contact_external_id().val();
			if (external_id !== 'N/A') {
				contact.externalId = external_id;
			} else {
				delete contact.externalId;
			}

			// update name and addresses
			contact.addresses = addresses;
			contact.Name = contact_name;

			const __save_contact = function()
			{
				const __finish_save = function(success, saved_contact, error) {
					if (success) {
						self.m_contact_window.show_message(format("<%=_T('Success {0} contact')%>", contact.isNew ? "<%=_T('creating')%>" : "<%=_T('saving')%>"), "<%=_T('info')%>", true);
						// TO-DO: Change this calls to a pubsub event
						self.m_contact_window.__on_contact_updated(saved_contact);
					}
					else {
						let message_error;
						switch( ( error.result || '' ).toUpperCase() )
						{
							case 'FORBIDDEN':
								message_error = "<%=_T('You not has permission to save contact.')%>";
								break;

							default:
								message_error = format("<%=_T('Error {0} contact: {1}')%>",
									contact.isNew ? "<%=_T('creating')%>" : "<%=_T('saving')%>",
									error.translatedMessage || error.result
								);
						}
						self.m_contact_window.show_message( message_error, 'error' );
					}
					if (callbacks && typeof(callbacks.on_saved) === 'function') {
						callbacks.on_saved(success, saved_contact);
					}
				};

				// async operation ahead, show waiting
				self.m_contact_window.show_message(format("<%=_T('{0} contact, please wait...')%>", contact.isNew ? "<%=_T('creating')%>" : "<%=_T('saving')%>"), 'process');

				// go and save this contact
				contact.save(
					function(success_save, saved_contact, error) {
						if (success_save && !error) {
							__finish_save(success_save, saved_contact, error);
						} else {
							switch (error.result) {
								case 'E_ADDR_IN_USE':
								case 'E_EXT_ID_IN_USE':
									// we handle different the "address already in use"
									// and "external id already in use" error.
									let conflict = {
										allow_merge : true,
										allow_move : true,
										allow_cancel : true,
										contact_id : error.contact_id,
										contact_external_id : contact.externalId ? contact.externalId : "<%=_T('N/A')%>",
										addresses : contact.addresses
									};

									if (error.result === 'E_ADDR_IN_USE') {
										conflict.field = error.address.displayable;
										conflict.field_type = 'address';
									}
									else if (error.result === 'E_EXT_ID_IN_USE') {
										conflict.field = error.external_id;
										conflict.field_type = 'externalId';
									}
									self.m_contact_window.hide_message();
									conflict.message = format("<%=_T('You already have a contact associated with that address. How do you prefer to continue?')%>", conflict.field_type);
									self.m_contact_window.__show_conflict_window(conflict);
								break;

								case 'E_THERE_ARE_INVALID_ADDRESSES':
									error.translatedMessage = "<%=_T('There are invalid addresses')%>";
									__finish_save(false, saved_contact, error);
								break;

								default:
									__finish_save(false, saved_contact, error);
							}
						}
					},
					campaign_id, // optional, used by permissions
					self.m_contact_window.get_app_id()
				);
			};

			// this function supports confirmation for save, if a confirmation
			// object is passed as argument, we call the confirm_function with the
			// contact to be saved as first parameter and a callback, the callback receive
			// as parameter if the action was confirmed or not.
			if (confirmation) {
				confirmation.confirm_function(contact, function(confirmed) {
					if (confirmed) {
						__save_contact();
					}
				});
			} else {
				__save_contact();
			}
		});
	};

	/**
	 * Build a generic contact info container markup and bind events
	 *
	 * @public
	 *
	 * @param {Object}   contact
	 * @param {String}   contact.Id
	 * @param {String}   contact.Name
	 * @param {Boolean}  contact.isNew
	 * @param {Array}    contact.addresses
	 * @param {Function} contact.getCompleteAvatarURI
	 * @param {String} contact.avatar
	 *
	 * @param {Object}   visual_options
	 * @param {Boolean}  visual_options.remove_cancel
	 * @param {Boolean}  visual_options.remove_merge
	 *
	 * @param {Object}   $container - Where the node will be appended, default is the body
	 */
	ContactBasicInfo.prototype.build_contact_info_container = function(contact, $container)
	{
		const self = this;

		const disable_save_create = !contact.isNew;
		const cancel_close_label = disable_save_create ? "<%=_T('Close')%>" : "<%=_T('Cancel')%>";

		const contact_view = {
			// view details
			view : this.m_prefix,

			// Basic info
			avatar_uri : contact.avatar ? contact.avatar : contact.getCompleteAvatarURI(),
			contact_name : contact.Name,
			contact_id : contact.isNew ? '' : contact.Id,

			// External ID
			external_id_label : "<%=_T('Contact ID')%>",
			external_id : contact.externalId,
			has_external_id : contact.externalId && contact.externalId.length > 0,

			// Save
			save_create_label : contact.isNew ? "<%=_T('CREATE')%>" : "<%=_T('SAVE')%>",
			save_create_tooltip : contact.isNew ? "<%=_T('Create new contact')%>" : "<%=_T('Save contact')%>",
			disable_save_create : disable_save_create,

			// Cancel / Close
			cancel_close_label : cancel_close_label,
			cancel_close_tooltip : "<%=_T('Cancel contact edition')%>",
			hide_cancel_close_btn : contact || false,

			// Delete
			delete_label : "<%=_T('DELETE')%>",
			delete_tooltip : "<%=_T('Delete contact')%>",
			hide_delete_btn : !this.m_contact_window.has_permission_to_delete() || contact.isNew,

			// Merge (always visible)
			merge_label : "<%=_T('MERGE')%>",
			merge_tooltip : contact.isNew ? "<%=_T('Merge data to existing contact')%>" : "<%=_T('Merge contacts')%>",
			hide_merge_btn : $container ? true : false, /* TO-DO: This sucks, I am assuming embedded view based on this, not good. */

			// Change
			change_label : "<%=_T('CHANGE')%>",
			change_tooltip : "<%=_T('This is not the contact I am interacting')%>",
			hide_change_btn : contact.isNew,

			// Favorite
			hide_fav_btn : !this.m_contact_window.has_permission_to_favorite() || contact.isNew,

			// Addresses
			add_address_tooltip : "<%=_T('Add New Address')%>",
			add_address_label : "<%=_T('Add New Address')%>",
			hide_no_addresses_msg : contact.addresses.length > 0,
			no_addresses_msg : "<%=_T('No addresses to show, you can add new ones below.')%>",

			show_followers_container : contact.totalFollowers,
			totalFollowers : contact.totalFollowers || "",
			followersDate : contact.followersDate && inConcert.Utils.getStringFromDate(inConcert.Utils.parseDateTimeString(contact.followersDate), undefined, "date") || ""
		};

		// append the basic markup to the container or to the body
		$container = $container ? $container : $("body");
		let $contact_info = Mustache.render(this.m_templates.contact_info_container_tpl, contact_view);
		$container.append($contact_info);

		// TODO: Make this part of the Mustache template
		// build every address node
		let kinds = [];
		contact.addresses.forEach(function(address) {
			const address_node = self.$build_address_node(address);
			self.__get_address_list().append(address_node);
			if (address.Kind) {
				kinds.push(address.Kind);
			}
		});

		const $phone_kind = $("#contact_window_add_address_phone_kind_" + this.m_prefix);
		const phone_types = this.__get_phone_types(kinds);
		inConcert.Utils.fill_combo($phone_kind, false, phone_types, "id", "desc", true);

		// return an updated version of the node
		return this.get_contact_window();
	};
}());
