/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double */

/* global inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";
	var format = inConcert.Utils.format;

		/*
		ContactList, template to make contacts list
	*/
	function ContactList(manager, properties) {
		this.manager = manager;
		properties = properties || {};
		
		this.addressTypes = properties.addressTypes || null;
		this.addressTypes_d = this.addressTypes? this.addressTypes.reduce(function(map, address) {
			map[address] = true;
			return map;
		}, {}) : null;
		this.filter = properties.filter !== false;
		this.fixedFilter = properties.fixedFilter || null;
		this.scrollPaginator = properties.scrollPaginator !== false;
		this.scrollPaginatorPerc = properties.scrollPaginatorPerc || 80;
		
		this.autoSelectFirst = properties.autoSelectFirst || false;
		this.multiSelect = properties.multiSelect || false;
		this.unSelect = properties.unSelect || false;
		this.avatar = properties.avatar !== false;
		
		this.addressRows = properties.addressRows || false;
		this.addressRowsAutoSelectFirst = properties.addressRowsAutoSelectFirst || false;
		this.addressRowsUnselect = properties.addressRowsUnselect || false;
		this.addressRowsMulti = properties.addressRowsMulti || false;
		
		this.delayMs = properties.delayMs || 400;
		this.rows = properties.rows || 40;
		this.className = "boxContactList " + (properties.className || "");
		this.messages = properties.messages || {
			searching : "<%=_T('Searching contacts')%>",
			empty : "<%=_T('No contacts found.')%>",
			error : "<%=_T('Error searching contacts.')%>"
		};
		this.result = properties.result || null;
	}

	//Define functions and objects in the namespace in case they don't already exist
	if (typeof inConcert.ContactList !== "function") {
		inConcert.ContactList = ContactList;
	}
	
	ContactList.prototype.make = function() {
		this.$body = this.makeBody();
		this.bindEvents();
		return this.$body;
	};
	
	ContactList.prototype.start = function(callback) {
		this.search(false, callback);
	};
	
	ContactList.prototype.makeBody = function() {
		return $("".concat(
			"<div class='", this.className, "'>",
				"<div class='boxContentFilterArea'>",
					"<input contacts_list_filter type='text' placeholder='<%=_T('Enter')%>' class='form-control'>",
					"<button contacts_list_refresh class='pageButtonGeneric buttonColorWhite iconOnlyBtn ml-1' tooltip='", "<%=_T('Search')%>" , "'>",
						"<i class='far fa-search'></i>",
					"</button>",
					"<button contacts_list_reset class='pageButtonGeneric buttonColorWhite iconOnlyBtn ml-1' tooltip='", "<%=_T('Reset')%>" , "'>",
						"<i class='far fa-eraser'></i>",
					"</button>",
				"</div>",
				"<div contacts_list_messages class='inlineMessagesContainer messageInfo mt-4' hidden>",
				"</div>",
				"<div contacts_list_waiting class='boxContentListWaiting' hidden>",
					"<div class='listWaitingAnimation'></div>",
					"<div contacts_list_waiting_message class='listWaitingMessage'></div>",
					"<div class='listWaitingOverlay'></div>",
				"</div>",
				"<div contacts_list class='boxContentScrollArea contentContactList'>",
				"</div>",
			"</div>"
		));
	};
	
	ContactList.prototype.$refresh = function() {
		return this.$body.find("[contacts_list_refresh]");
	};
	
	ContactList.prototype.$reset = function() {
		return this.$body.find("[contacts_list_reset]");
	};
	
	ContactList.prototype.$filter = function() {
		return this.$body.find("[contacts_list_filter]");
	};
	
	ContactList.prototype.$list = function() {
		return this.$body.find("[contacts_list]");
	};
	
	ContactList.prototype.$message = function() {
		return this.$body.find("[contacts_list_messages]");
	};
	
	ContactList.prototype.$waiting = function() {
		return this.$body.find("[contacts_list_waiting]");
	};
	
	ContactList.prototype.$waitingMessage = function() {
		return this.$body.find("[contacts_list_waiting_message]");
	};
	
	ContactList.prototype.bindEvents = function() {
		this.bindRefresh();
		this.bindReset();
		this.bindFilter();
		if (this.scrollPaginator) {
			this.bindScroll();
		}
	};
	
	ContactList.prototype.bindRefresh = function() {
		let self = this;
		self.$refresh()
			.off("click")
			.on("click", function() {
				self.search();
			});
	};
	
	ContactList.prototype.bindReset = function() {
		let self = this;
		self.$reset()
			.off("click")
			.on("click", function() {
				self.reset();
			});
	};
	
	ContactList.prototype.bindFilter = function() {
		let self = this;
		self.$filter()
			.off("keyup")
			.on("keyup", function() {
				self.search(true);
			});
	};
	
	ContactList.prototype.bindScroll = function() {
		let self = this;
		self.$list()
			.off("scroll")
			.on("scroll", function() {
				let scrolledPercentage = Math.round((this.scrollTop / (this.scrollHeight - this.clientHeight)) * 100);
				if (!self.result.allResultsFound && scrolledPercentage > self.scrollPaginatorPerc) {
					self.nextPage();
				}
			});
	};
	
	ContactList.prototype.message = function(message) {
		this.$message()
			.toggle(!!message)
			.text(message || "");
	};
	
	ContactList.prototype.waiting = function(waiting, message) {
		this.$waiting().toggle(!!waiting);
		this.$waitingMessage().text(message || "");
	};
	
	ContactList.prototype.startProcess = function(message) {
		this.message();
		this.waiting(true, message);
		this.processing = true;
	};
	
	ContactList.prototype.stopProcess = function() {
		this.waiting(false);
		this.processing = false;
	};
	
	ContactList.prototype.isProcessing = function() {
		return !!this.processing;
	};
	
	ContactList.prototype.getFilter = function() {
		if (!this.filter || this.fixedFilter) {
			return this.fixedFilter || "";
		}
		return this.$filter().val() || "";
	};
	
	ContactList.prototype.setFilter = function(value) {
		if (!this.filter || this.fixedFilter) {
			return;
		}
		this.$filter().val(value || "");
	};
	
	ContactList.prototype.disableInputs = function() {
		this.$refresh().prop("disabled", true);
		this.$reset().prop("disabled", true);
		this.$filter().prop("disabled", true);
	};
	
	ContactList.prototype.enableInputs = function() {
		this.$refresh().prop("disabled", false);
		this.$reset().prop("disabled", !!this.getFilter());
		this.$filter().prop("disabled", !this.fixedFilter);
	};
	
	ContactList.prototype.reset = function(callback) {
		callback = callback || $.noop;
		if (this.isProcessing()) {
			return callback(false, "bussy");
		}
		this.setFilter("");
		this.search(false, callback);
	};
	
	ContactList.prototype.cleanList = function() {
		this.$list().html("");
	};
	
	ContactList.prototype.search = function(delay, callback) {
		callback = callback || $.noop;
		let self = this;
		
		function task() {
			if (self.isProcessing()) {
				return callback(false, "bussy");
			}
			self.cleanList();
			self.result = null;
			self.__search(callback);
		}
		
		if (self.delayTimeout) {
			window.clearTimeout(self.delayTimeout);
			self.delayTimeout = null;
		}
		if (delay) {
			self.delayTimeout = window.setTimeout(function() {
				self.delayTimeout = null;
				task();
			}, self.delayMs);
		}
		else {
			task();
		}
	};
	
	ContactList.prototype.nextPage = function(callback) {
		callback = callback || $.noop;
		if (this.isProcessing()) {
			return callback(false, "bussy");
		}
		this.__search(callback);
	};
	
	ContactList.prototype.__makeFilter = function() {
		let filter = {
			last : this.result && this.result.last || "",
			rows : this.rows,
			query : this.getFilter(),
			addresses : this.addressTypes? this.addressTypes.map(function(addressType) {
				return { type : addressType };
			}) : null
		};
		return filter;
	};
	
	ContactList.prototype.__search = function(callback) {
		callback = callback || $.noop;
		let self = this;
		let filter = self.__makeFilter();
		self.startProcess(self.messages.searching);
		self.manager.search(filter, function(contacts, last) {
			if (!contacts) {
				self.message(self.messages.error);
				self.stopProcess();
				callback(false, "searching");
				return;
			}
			self.addContacts(contacts, last, function() {
				self.stopProcess();
				callback(true);
			});
		});
	};
	
	ContactList.prototype.addContacts = function(contacts, last, callback) {
		callback = callback || $.noop;
		let self = this;
		self.result = self.result || {
			contacts : [],
			contacts_d : {}
		};
		let currentLength = (self.result.files || []).length;
		self.result.last = last;
		(function addContact(index) {
			if (index === contacts.length) {
				if (!self.result.contacts.length) {
					self.message(self.messages.empty);
				}
				if ((self.result.files || []).length === currentLength) {
					self.result.allResultsFound = true;
				}
				return callback();
			}
			self.addContact(contacts[index], function() {
				addContact(index + 1);
			});
		})(0);
	};
	
	ContactList.prototype.addContact = function(contactRaw, callback) {
		callback = callback || $.noop;
		let self = this;
		self.processContact(contactRaw, function(contact, error) {
			if (!contact) {
				return callback(false, error);
			}
			if (self.result.contacts_d[contact.Id]) {
				return callback(false, "already added");
			}
			let first = !self.result.contacts.length;
			self.result.contacts.push(contact);
			self.result.contacts_d[contact.Id] = contact;
			self.appendContact(contact, first && self.autoSelectFirst? 
				self.makeContactSelectedRow(contact) : self.makeContactRow(contact));
			callback(contact);
		});
	};
	
	ContactList.prototype.processContact = function(contactRaw, callback) {
		callback(contactRaw);
	};
	
	ContactList.prototype.appendContact = function(contact, $contact) {
		this.$list().append($contact);
	};
	
	ContactList.prototype.makeContactRow = function(contact) {
		let self = this;
		let $contact = $(format("".concat(
			"<div list_contact='{0}' class='boxContactItem'>",
				"<div class='boxContactIdentificator' title='{1}'>",
					"{1}",
				"</div>",
			"</div>"
		), contact.Id, contact.Name));
		
		$contact.on("click", function() {
			self.selectContact($(this).attr("list_contact"));
		});
		
		return $contact;
	};
	
	ContactList.prototype.makeContactSelectedRow = function(contact) {
		let self = this;
		let $contact = $(format("".concat(
			"<div list_contact='{0}' selected_contact class='boxContactItem boxContactItemSelect'>",
				"<div class='contactAvatar'>",
					"<img src='{1}'>",
				"</div>",
				"<div class='boxContactIdentificator'>",
					"{2}",
				"</div>",
			"</div>"
		), contact.Id, inConcert.app.getBaseUrl() + contact.getAvatarURI(), contact.Name));
		$contact.append(self.makeContactActions(contact));
		if (this.unSelect) {
			$contact.on("click", function(e) {
				if (self.hasToUnSelectContact($(this), $(e.target))) {
					self.unSelectContact($(this).attr("list_contact"));
				}
			});
		}
		return $contact;
	};
	
	ContactList.prototype.hasToUnSelectContact = function($contact, $target) {
		return true;
	};
	
	ContactList.prototype.selectContact = function(contact) {
		if (!this.multiSelect) {
			this.unSelectContacts();
		}
		contact = this.getContact(contact);
		if (!contact) {
			return;
		}
		this.$contactRow(contact.Id).replaceWith(this.makeContactSelectedRow(contact));
	};
	
	ContactList.prototype.unSelectContact = function(contact) {
		contact = this.getContact(contact);
		if (!contact) {
			return;
		}
		if (this.addressRows && !this.addressRowsMulti) {
			this.unSelectAddresses(contact);
		}
		this.$contactRow(contact.Id).replaceWith(this.makeContactRow(contact));
	};
	
	ContactList.prototype.getSelectedContacts = function() {
		let self = this;
		let contacts = [];
		this.$body.find("[selected_contact]").each(function() {
			contacts.push(self.getContact($(this).attr("list_contact")));
		});
		return contacts;
	};
	
	ContactList.prototype.getSelectedContact = function() {
		let contacts = this.getSelectedContacts();
		return contacts.length? contacts[0] : null;
	};
	
	ContactList.prototype.unSelectContacts = function() {
		let self = this;
		self.getSelectedContacts().map(function(contact) {
			self.unSelectContact(contact);
		});
	};
	
	ContactList.prototype.$contactRow = function(contact) {
		let contactId = typeof(contact) === "string"? contact : contact.Id;
		return this.$body.find(format("[list_contact='{0}']", contactId));
	};
	
	ContactList.prototype.getContact = function(contact) {
		if (!contact || typeof(contact) !== "string") {
			return contact;
		}
		return this.result && this.result.contacts_d[contact] || null;
	};
	
	ContactList.prototype.makeContactActions = function(contact) {
		let actions = [];
		if (this.addressRows) {
			actions.push(this.makeAddressesRows(contact));
		}
		return actions;
	};
	
	ContactList.prototype.makeAddressesRows = function(contact) {
		let self = this;
		let $addresses = $(format("".concat(
			"<div contact_addresses='{0}' class='contactAddressesList'>",
			"</div>"
		), contact.Id));
		let first = true;
		$addresses.append((contact.addresses || []).map(function(address, index) {
			if (self.addressTypes_d && !self.addressTypes_d[address.Type]) {
				return;
			}
			address.contact = contact;
			let $address = first && self.addressRowsAutoSelectFirst? 
				self.makeAddressSelectedRow(contact, address) : self.makeAddressRow(contact, address);
			first = false;
			return $address;
		}));
		return $addresses;
	};
	
	ContactList.prototype.makeAddressRow = function(contact, address) {
		let self = this;
		let $address = $(format("".concat(
			"<div contact_address='{0}' class='contactAddressRow addressType{1}'>",
				"{2}",
			"</div>"
		), address.SanitizedId, address.Type, address.displayable));
		$address.data("contact", contact.Id);
		
		$address.on("click", function() {
			self.selectAddress($(this).data("contact"), $(this).attr("contact_address"));
		});
		
		return $address;
	};
	
	ContactList.prototype.makeAddressSelectedRow = function(contact, address) {
		let self = this;
		let $address = $(format("".concat(
			"<div contact_address='{0}' selected_address class='contactAddressRow addressType{1} contactAddressRowSelected'>",
				"{2}",
			"</div>"
		), address.SanitizedId, address.Type, address.displayable));
		$address.data("contact", contact.Id);
		
		if (this.addressRowsUnselect) {
			$address.on("click", function(e) {
				if (self.hasToUnSelectAddress($(this), $(e.target))) {
					self.unSelectAddress($(this).data("contact"), $(this).attr("contact_address"));
				}
			});
		}
		
		return $address;
	};
	
	ContactList.prototype.hasToUnSelectAddress = function($address, $target) {
		return true;
	};
	
	ContactList.prototype.selectAddress = function(contact, address) {
		if (!this.addressRowsMulti) {
			this.unSelectAddresses(contact);
		}
		contact = this.getContact(contact);
		address = this.getAddress(contact, address);
		if (!contact || !address) {
			return;
		}
		this.$addressRow(contact, address).replaceWith(this.makeAddressSelectedRow(contact, address));
	};
	
	ContactList.prototype.unSelectAddress = function(contact, address) {
		contact = this.getContact(contact);
		address = this.getAddress(contact, address);
		if (!contact || !address) {
			return;
		}
		this.$addressRow(contact, address).replaceWith(this.makeAddressRow(contact, address));
	};
	
	ContactList.prototype.getSelectedAddresses = function(contact) {
		let self = this;
		let addresses = [];
		(contact? this.$contactRow(contact) :this.$body)
			.find("[selected_address]").each(function() {
				addresses.push(self.getAddress($(this).data("contact"), $(this).attr("contact_address")));
			});
		return addresses;
	};
	
	ContactList.prototype.getSelectedAddress = function(contact) {
		let addresses = this.getSelectedAddresses(contact);
		return addresses.length? addresses[0] : null;
	};
	
	ContactList.prototype.unSelectAddresses = function(contact) {
		let self = this;
		self.getSelectedAddresses(contact).map(function(address) {
			self.unSelectAddress(contact, address);
		});
	};
	
	ContactList.prototype.$addressRow = function(contact, address) {
		let addressId = typeof(address) === "string"? address : address.SanitizedId;
		return this.$contactRow(contact).find(format("[contact_address='{0}']", addressId));
	};
	
	ContactList.prototype.getAddress = function(contact, address) {
		if (!address || typeof(address) !== "string") {
			return address;
		}
		contact = this.getContact(contact);
		return (contact && contact.addresses || []).find(function(contactAddress) {
			return contactAddress.SanitizedId === address;
		});
	};

}());
