/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
  strict:false, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark: false,
  smarttabs:true, esversion: 11 */

/* global inConcert, _ , EmojiPicker, CInconcertPlayer */


/*
	Handling GUI events to interact with facebook client session
*/

// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	var m_maxCharsOnTheWall = 1000;
	var m_maxCharsOnTheComment = 8000;
	var FORMAT = inConcert.Utils.format;

	var SCROLL_TO = inConcert.Utils.scroll_to;
	var TIME_SELECTED_COMMENT_MS = 5000;

	var LNK_THREAD_MARK_AS_READ_CLASS   = "threadActionsMarkAsRead";
	var LNK_THREAD_MARK_AS_UNREAD_CLASS = "threadActionsMarkAsUnread";
	var LNK_ORIGIN_MARK_AS_READ_CLASS   = "originActionsMarkAsRead pageButtonGeneric buttonColorWhite";
	var LNK_ORIGIN_MARK_AS_UNREAD_CLASS = "originActionsMarkAsUnread pageButtonGeneric buttonColorWhite";
	
	var LNK_LIKE_CLASS   = "actionsLike";
	var LNK_UNLIKE_CLASS = "actionsUnlike";

	var CLASS_DELIVERY_SENDING    = "deliverySending";
	var CLASS_DELIVERY_SUCCESS    = "deliveryOk";
	var CLASS_DELIVERY_FAILED     = "deliveryFail";
	var CLASS_DELIVERY_DISCARDED  = "deliveryDiscarded";
	var CLASS_DELIVERY_DISCARDING = "deliveryDiscarding";

	/**
	Class FacebookClientWindow.
	Facebook Client Handler
	@param interaction
	@param options
	*/
	function FacebookClientWindow (interaction, options)
	{
		var self = this;

		//relate the interaction to the view
		this.m_maxCompressedComments = 20;
		this.m_interaction = interaction;
		this.hash_events = {};
		this.update_hash_events_return_diff();
		this.setView("client");
		this.setHorizontalResizingLimits(480);

		if (interaction && interaction.interactionData && _.contains(["PageInboxNewMessage", "InboxNewMessage"], interaction.interactionData.subtype)) {
			//this flag must be set up before the contructor call because is needed in the markup build
			this.m_privateMessages = true;
		}
		else if (interaction && interaction.interactionData && interaction.interactionData.subtype === "NewPostComment") {
			this.m_postComment = true;
		}
		else if (interaction && interaction.events && interaction.events.length && interaction.initiative === "INBOUND") {
			this.m_wallMessage = true;
			$("#facebookCommentButton").val("<%=_T([[COMMENT]])%>");
		}
		else if (interaction && interaction.events && interaction.events.length) {
			this.m_createdPost = true;
			//2016-03: Used in lagazy interactions when outbound post comments split was implemented
			//		   TODO: Remove all related with this variable when old interactions expired
			if (interaction.events.length > 1) {
				this.m_legazyPost = true;
			}
		}
		else {
			this.m_newPost = true;
		}

		var success = FacebookClientWindow.uber.constructor.call( this, options );
		if (!success) {
			return;
		}

		if (this.m_privateMessages) {
			this.setView("mail_client");
			this.replyByMail();
		}

		$("#facebookClientCharacterCounter").text(m_maxCharsOnTheWall);
		this.has_created_main_thread = false;
		this.can_show_all_comments = false;
		this.are_showing_all_messages = false;
		this.loading_all_comments = false;

		if (options.readonly || options.monitor) {
			//cannot comment anything on this state
			$('.facebookThreadCommentControlContainer').hide();
			//cannot reply in wall
			$("#replyInWall").hide();
		}

		if (this.m_createdPost) {
			$("#facebook_solve_container").hide();
		}
		
		$("#postCommentText").off('change').on('change', function() {
			$("#facebookCommentButton").prop("disabled", !$(this).val());
			self.Spellcheck();
		}).change();

		$("#facebookCommentButton").click(function() {
			self.comment_current_post();
		});

		$("#replyInWall").click(function() {
			self.reply_in_wall();
		});

		//load avatar for the contact
		var avatar_uri = inConcert.app.getBaseUrl() + interaction.getAvatarURI();
		$('#contactFromAvatar').attr("data", avatar_uri);
		$("#facebookWallText").keyup(function() {
			self.onCharCountChanged($(this));
		});

		$("#postCommentText").keypress(function() {
			var comment = $(this).val();
			if (comment.length >= m_maxCharsOnTheComment) {
				return false;
			}
		}).bind("paste", null, function() {
			var self = $(this);
			var previousValue = self.val();
			window.setTimeout(function() {
				var actualValue = self.val();
				if (actualValue.length >= m_maxCharsOnTheComment) {
					self.val(previousValue);
				}
			}, 1);
		});

		$("#facebookGroup").text(this.m_interaction.campaign);
		//facebook cboAccount shows the page name in case this is a message for the page
		var page = this.m_interaction.interactionData.page;
		if (page) {
			var accountId = this.m_interaction.getAccount();
			var account;
			var campaignAccounts = inConcert.app.getCampaignAddresses(this.m_interaction.campaign, "FACEBOOK");
			for (var i = 0; i < campaignAccounts.length; i++) {
				if (campaignAccounts[i].id === accountId) {
					account = campaignAccounts[i];
					break;
				}
			}
			if (account) {
				var campaign = inConcert.app.getCampaign(this.m_interaction.campaign);
				if (campaign) {
					var pagesNames = campaign.getAccountPages(accountId);
					if (pagesNames) {
						$("#cboAccounts option[value='" + this.m_interaction.account + "']").text(pagesNames[page]+" ("+account.name+")");
					}
				}
			}
		}

		this.was_being_composed = this.m_interaction.initiative === "OUTBOUND" && this.m_interaction.state === "PENDING";
		var pending_origin_post = this.m_interaction.get_pending_origin_post();
		if ( this.was_being_composed || pending_origin_post ) {
			this.reply_in_wall( pending_origin_post );
			$("#mainContainer").addClass("facebookStartOutboundInteraction");
			if (!this.m_interaction.contact) {
				this.on_account_changed();
			}
		}
		else {
			var events = this.m_interaction.getEventList();
			if (events && events.length) {
				$("#facebookDate").text(this.m_interaction.getLocaleDateString(0));
				$("#facebookFrom").text(this.m_interaction.getSource(0));
			}
			var campaignAddresses = inConcert.app.getCampaignAddresses(this.m_interaction.campaign, "FACEBOOK");
			for (var j = 0; j < campaignAddresses.length; j++) {
				if (campaignAddresses[j].id === this.m_interaction.getAccount()) {
					this.campaignAddress = campaignAddresses[j];
				}
			}
		}
		this.m_timer = window.setInterval(function() {
			self.onTimer();
		}, 1000);
	}

	inConcert.Utils.inherit( FacebookClientWindow, inConcert.ClientWindow );

	FacebookClientWindow.prototype.initialize = function()
	{
		this.__initializeButtonsSpell();
		this.createSpellChecker();
		FacebookClientWindow.uber.initialize.call( this );
	};

	FacebookClientWindow.prototype.dispose = function() {
		this.destroySpellChecker();
		FacebookClientWindow.uber.dispose.call( this);
	};

	FacebookClientWindow.prototype.getMenuContextualActions = function()
	{
		const actions = {
		// acciones solo para el cliente
			attachfile   : true,
			mark_all_read_unread: true,
		// acciones sobre la interaccion
			comment      : true,
			label        : true,
			emit_ticket  : true,
			search_ticket: true,
			queue        : true,
			transfer     : true
		// estas NO tienen que visualizarse en el menu, ya que las mismas se encuentran en la toolbar
		};
		return actions;
	};

	FacebookClientWindow.prototype.clearInput = function($input) {
		var $input_emoji = this.$emojiText($input);
		if ($input_emoji.length) {
			$input_emoji.html("");
		}
		$input.val("");
	};

	FacebookClientWindow.prototype.focusInput = function($input) {
		var $input_emoji = this.$emojiText($input);
		if ($input_emoji.length) {
			inConcert.Utils.placeCaretAtEnd($input_emoji.get(0));
		}
		else {
			$input.focus();
		}
	};

	FacebookClientWindow.prototype.enableInput = function(enabled) {
		var $input = this.$input();
		$input.prop("disabled", !enabled);
		this.$emojiText($input).prop("contenteditable", enabled);
		$(".emoji-picker-button").toggleClass("emoji-picker-button-disabled", !enabled);
	};
	
	FacebookClientWindow.prototype.$input = function() {
		const was_being_composed = this.m_interaction.initiative === "OUTBOUND" && this.m_interaction.state === "PENDING";
		const pending_origin_post = this.m_interaction.get_pending_origin_post();
		if ( was_being_composed || pending_origin_post ) {
			return $("#facebookWallText");
		}
		else {
			if ( this.m_interaction.isPrivateMessage() ) {
				return $("#facebookMailText");
			}
			else {
				return $("#postCommentText");
			}
		}
	};
	
	FacebookClientWindow.prototype.$attachmentPicker = function() {
		const was_being_composed = this.m_interaction.initiative === "OUTBOUND" && this.m_interaction.state === "PENDING";
		const pending_origin_post = this.m_interaction.get_pending_origin_post();
		if ( was_being_composed || pending_origin_post ) {
			return $("#sendFilePost");
		}
		else {
			if ( this.m_interaction.isPrivateMessage() ) {
				return $("#sendFilePrivate");
			}
			else {
				return $("#sendFileComment");
			}
		}
	};
	
	FacebookClientWindow.prototype.$recordStart = function() {
		const was_being_composed = this.m_interaction.initiative === "OUTBOUND" && this.m_interaction.state === "PENDING";
		const pending_origin_post = this.m_interaction.get_pending_origin_post();
		if ( was_being_composed || pending_origin_post ) {
			return $("#recordAudioStartButtonPost");
		}
		else {
			if ( this.m_interaction.isPrivateMessage() ) {
				return $("#recordAudioStartButtonPrivate");
			}
			else {
				return $("#recordAudioStartButtonComment");
			}
		}
	};
	
	FacebookClientWindow.prototype.$inputContainer = function() {
		const was_being_composed = this.m_interaction.initiative === "OUTBOUND" && this.m_interaction.state === "PENDING";
		const pending_origin_post = this.m_interaction.get_pending_origin_post();
		if ( was_being_composed || pending_origin_post ) {
			return $("#facebookWallMessage");
		}
		else {
			if ( this.m_interaction.isPrivateMessage() ) {
				return $("#facebookPrivateMessage");
			}
			else {
				return $("#facebookComment");
			}
		}
	};

	FacebookClientWindow.prototype.$emojiText = function($input) {
		return $input.siblings(".emoji-wysiwyg-editor");
	};

	FacebookClientWindow.prototype.$emojiInput = function() {
		return this.$emojiText(this.$input());
	};

	FacebookClientWindow.prototype.appendToInput = function($input, text) {
		var $input_emoji = this.$emojiText($input);
		if ($input_emoji.length) {
			$input_emoji.html($input_emoji.html() + " " + text);
		}
		$input.val($input.val() + " " + text);
	};
	
	FacebookClientWindow.prototype.setInputText = function($input, text) {
		this.clearInput($input);
		this.appendToInput($input, text);
		this.focusInput($input);
		$input.change();
	};

	FacebookClientWindow.prototype.getWindowDefaultTitle = function()
	{
		return "<%=_T([[Facebook Window]])%>";
	};

	FacebookClientWindow.prototype.eventsSuscribe = function()
	{
		FacebookClientWindow.uber.eventsSuscribe.call( this );
		var self = this;

		if ( this.m_interaction && this.m_interaction.id ) {
			// failure ( POST, COMMENT, PRIVATE MESSAGE )
			var pub_fb_failure_id =  "/facebook/" + this.m_interaction.id + "/failure";
			this.pubsub_fb_failure_messages_handle = inConcert.app.messagebus.subscribe(
				pub_fb_failure_id,
				function( event, type ) {
					self.show_proxy_failure( event, type );
				}
			);
		}
	};

	FacebookClientWindow.prototype.eventsUnsuscribe = function()
	{
		FacebookClientWindow.uber.eventsUnsuscribe.call( this );

		if ( this.pubsub_fb_failure_messages_handle ) {
			inConcert.app.messagebus.unsubscribe( this.pubsub_fb_failure_messages_handle );
		}
	};

	FacebookClientWindow.prototype.show_proxy_failure = function( event /*, type*/ )
	{
		var short_msg = this.m_interaction ? this.m_interaction.get_event_error_message( event.error_code, event.error || "" ) : "<%=_T([[Facebook error.]])%>";
		var long_msg = short_msg + ". " + "<%=_T([[Please try again later.]])%>";
		this.hideStatusMessage();
		this.displayStatusMessage( long_msg, "Error", true, 5000 );
		this.delivery_failed_message( event, short_msg );
	};
	
	/*
		Like/unlike an event
	*/
	FacebookClientWindow.prototype.like = function( like, event_index, callback )
	{
		var self = this;
		callback = callback || $.noop;

		var message;
		if ( like ) {
			message = "<%=_T([[Please wait, liking.]])%>";
		}
		else {
			message = "<%=_T([[Please wait, unliking.]])%>";
		}
		this.displayStatusMessage( message, "Process", false );

		this.m_interaction.like(
			like,
			event_index,
			function( success, message ) {
				callback( success );
				self.hideStatusMessage();
				if (!success) {
					self.displayStatusMessage( message, "Error" );
				}
			}
		);
	};
	
	FacebookClientWindow.prototype.buildReplyComment = function($reply_link) {
		let self = this;
		if ( !self.canExecuteInteractionAction() ) {
			return "can not reply comment";
		}
		let message_id = $reply_link.data("message_id");
		let name = $reply_link.data("user_name");
		let $comment = $reply_link.parent();

		if ( $comment.hasClass( "unread" ) ) {
			var event_index = $reply_link.data("event_index");
			self.mark_as_read(
				true,
				event_index,
				function( success ) {
					self.on_readed_event( success, $comment );
				}
			);
		}

		self.setReplyToComment( message_id, name );
	};
	
	FacebookClientWindow.prototype.buildPrivateReply = function($reply_link) {
		let self = this;
		if ( !self.canExecuteInteractionAction() ) {
			return "can not private reply";
		}
		let event_index = $reply_link.data("event_index");
		let name = $reply_link.data("user_name");
		let $comment = $reply_link.parent();

		if ( $comment.hasClass( "unread" ) ) {
			self.mark_as_read(
				true,
				event_index,
				function( success ) {
					self.on_readed_event( success, $comment );
				}
			);
		}

		self.setPrivateReply( event_index, name );
	};
	
	FacebookClientWindow.prototype.likeUnlikeObject = function($like_unlike, callback = $.noop) {
		let self = this;
		let new_state = $like_unlike.data( "event_like" ) === "like";
		
		if (!self.canExecuteInteractionAction() || $like_unlike.prop( "disabled" )) {
			return callback(false, FORMAT("can not {0} object", new_state? "like" : "unlike"));
		}

		let event_index = $like_unlike.data( "event_index" );
		self.like(
			new_state,
			event_index,
			function( success ) {
				self.on_liked_event( success, new_state, $like_unlike.parent() );
				if (success) {
					callback(true);
				}
				else {
					callback(false, FORMAT("error {0} object", new_state? "liking" : "unliking"));
				}
			}
		);
	};

	FacebookClientWindow.prototype.setEventsHandlers = function()
	{
		var self = this;

		FacebookClientWindow.uber.setEventsHandlers.call( this ); //invokes the base class method

		if ( !this.m_interaction.isBeingComposed() ) {

			$("#mark_all_read_unread").click( function()
			{
				self.markAllReadUnread();
			});
		}

		var $container = $("#thread_container");

		// reply comment link
		var event_selector = "[reply_comment_link]";
		$container
			.off("click", event_selector)
			.on("click", event_selector, function ()
			{
				self.buildReplyComment($(this));
			});

		event_selector = "[private_reply_link]";
		$container
			.off("click", event_selector)
			.on("click", event_selector, function ()
			{
				self.buildPrivateReply($(this));
			});

		// mark as read/unread comment link
		event_selector = "[mark_as_read_unread]";
		$container
			.off("click", event_selector)
			.on("click", event_selector, function ()
			{
				var $mark_read_unread = $( this );

				if ( $mark_read_unread.prop( "disabled" ) ) {
					return;
				}

				var $comment = $mark_read_unread.parent();
				if ( $comment.hasClass( "selected" ) ) {
					return;
				}

				var event_index = $mark_read_unread.data( "event_index" );
				var new_state = $mark_read_unread.data( "event_unread" ) === "unread";
				self.mark_as_read(
					new_state,
					event_index,
					function( success ) {
						self.on_readed_event( success, $comment );
					}
				);
			});

		// mark as read/unread comment link
		event_selector = "[like_unlike]";
		$container
			.off("click", event_selector)
			.on("click", event_selector, function ()
			{
				self.likeUnlikeObject($(this));
			});

		// resend failed comment link
		var on_resend = function( success, message )
		{
			if ( !success ) {
				// Estos son errores genericos. Los errores de FB se procesan en "FacebookClientWindow.eventsSuscribe / failure"
				self.hideStatusMessage();
				self.displayStatusMessage( message, "Error" );
				return;
			}
			// El OK se procesa cuando llega el evento "ClientWindow.onInteractionUpdated -> FacebookClientWindow.clientUpdate"
		};

		event_selector = "[resend_message]";
		$container
			.off("click", event_selector)
			.on("click", event_selector, function ()
			{
				if ( !self.canExecuteInteractionAction() ) {
					return;
				}
				var $resend = $( this );
				var $comment = $resend.parent().parent();
				var fake_event_id = $comment.attr( "data-id" );
				var failed_event = self.m_interaction.get_pending_event( fake_event_id );
				if ( !failed_event ) {
					return;
				}

				self.displayStatusMessage("<%=_T([[Resending...]])%>", "Process", true);
				self.delivery_sending_message( $comment, "<%=_T([[Resending]])%>" );

				if ( self.m_interaction.isPrivateMessage() ) {
					self.m_interaction.newPrivateMessage(
						fake_event_id,
						failed_event.page,
						failed_event.message,
						function( success, message ) { on_resend( success, message ); }
					);
				}
				else {
					self.m_interaction.commentPost(
						fake_event_id,
						failed_event.postId,
						failed_event.commentId,
						failed_event.message,
						function( success, message ) { on_resend( success, message ); }
					);
				}
			});

		// discard failed comment link
		event_selector = "[discard_message]";
		$container
			.off("click", event_selector)
			.on("click", event_selector, function ()
			{
				if ( !self.canExecuteInteractionAction() ) {
					return;
				}
				var $discard = $( this );
				var $comment = $discard.parent().parent();
				var fake_event_id = $comment.attr( "data-id" );
				if ( !fake_event_id ) {
					return;
				}
				var failed_event = self.m_interaction.get_pending_event( fake_event_id );
				if ( !failed_event ) {
					return;
				}

				self.displayStatusMessage("<%=_T([[Discarding...]])%>", "Process", true);
				self.delivery_discarding_message( $comment );

				self.m_interaction.discardFailedEvent( fake_event_id,
					function( success )
					{
						self.hideStatusMessage();
						if ( success ) {
							self.delivery_discarded_message( $comment );
							return;
						}
						self.displayStatusMessage( FORMAT( "<%=_T([[Error discarding message {0}]])%>", fake_event_id), "Error", true, 5000 );
						self.delivery_failed_message( failed_event );
					}
				);
			});

		// show / hide message error !!!
		var timeout_handle;
		var show_error = function( $error, can_show ) {
			var $err_msg_container = $error.parent().find( ".deliveryFailReasonContainer" );
			if ( can_show ) {
				window.clearTimeout( timeout_handle );
				var show_error_timeout_ms = 200;
				timeout_handle = window.setTimeout( function() { $err_msg_container.show(); }, show_error_timeout_ms );
			}
			else {
				window.clearTimeout( timeout_handle );
				$err_msg_container.hide();
			}
		};

		event_selector = ".deliveryMessageContainer." + CLASS_DELIVERY_FAILED;
		$container
			.off("mouseenter", event_selector)
			.off("mouseleave", event_selector)
			.on("mouseenter", event_selector, function () { show_error( $( this ), true  ); })
			.on("mouseleave", event_selector, function () { show_error( $( this ), false ); });

				/* Gestionar contacto si:
		 1- Esta habilitado la administracion de contactos
		 2- Se puede visualizar los addresses
		*/
		const visualize_address = inConcert.app.canVisualizeAddress(null, this.m_interaction.campaign, null, true);
		if ( this.m_interaction.accountHasContactManagementEnabled( this.from_app ) && visualize_address) {
			//Setup contact manager popup when user click in the contact
			event_selector = ".subEventAuthor";
			$container
				.off("click", event_selector)
				.on("click", event_selector, function ()
				{
					var event_index = $( this ).data( "event_index" );
					if ( !event_index ) {
						return;
					}
					var event = self.m_interaction.getEvent( event_index );
					if ( !event || event.creator !== "contact" ) {
						return;
					}

					var address_normalized = inConcert.app.getContactManager().normalizeContactAddress(self.m_interaction.getRawAddress( event_index ), "Facebook");
					var contact_window = new inConcert.ContactWindow({
						from_app    : self.from_app,
						interaction : self.m_interaction,
						name        : self.m_interaction.getUserName( event ),
						avatar      : self.m_interaction.getEventUserAvatarURI( event ),
						addresses   : [ address_normalized ]
					});
					contact_window.show();
				});
		}

		var language_timeout = null;
		$('#cboLanguage').change(function()
		{
			if ( language_timeout ) {
				window.clearTimeout(language_timeout);
				language_timeout = null;
			}
			var $cbo = $(this);
			language_timeout = window.setTimeout(function()
			{
				window.clearTimeout(language_timeout);
				language_timeout = null;
				var new_language = $cbo.val();
				self.ChangeSpellcheckLanguage(new_language);
			}, 1000);
		});

		$('#fbButtonSpellcheck').
		off("click")
		.on("click", function () {
			if ( !self.m_spellchecker) {
				return;
			}
			var $spellcheck_feedback = $('#fbSpellcheckFeedback');
			self.m_spellchecker.check(null, function (words) {
				if (_.isArray(words[0]) && words[0].length > 0) {
					$spellcheck_feedback.hide();
				}
				else {
					$spellcheck_feedback.show();
					window.setTimeout(function () {
						$spellcheck_feedback.hide();
					}, 3000);
				}
			});
		});
	};

	FacebookClientWindow.prototype.getScrollBasicInfo = function()
	{
		return {
			$scroll : $("#thread_container"), // el contenedor de la lista con el scroll
			increase_position : -170 // este es un magic number que sirve unicamente en el caso en que "destination='item_position'",
									// decrementa un valor para poder ajustarlo a la pantalla correctamente.
		};
	};

	FacebookClientWindow.prototype.getCaptionRelatedButton = function()
	{
		return FORMAT( "<%=_T([[{0} Unread]])%>", this.m_interaction.getUnreadEventCount() );
	};

	FacebookClientWindow.prototype.getRelatedEvents = function() {
		return this.m_interaction.getUnreadEvents();
	};

	FacebookClientWindow.prototype.invalidInteraction = function() {
		FacebookClientWindow.uber.invalidInteraction.call( this );
		$(".interactionActionsButtons").children().hide();
		$(".facebookCommentControl").hide();
		$(".facebookThreadCommentControlContainer").hide();
	};

	FacebookClientWindow.prototype.onInteractionGone = function(interaction) {
		FacebookClientWindow.uber.onInteractionGone.call( this, interaction );
		$("#postCommentText").prop("disabled", true);
		$("#facebookComment").prop("disabled", true);
		$("#facebookWallText").prop("disabled", true);
		$("#facebookWallButtonPost").prop("disabled", true);
	};

	FacebookClientWindow.prototype.__setExternalMessage = function(message = "")
	{
		const self = this;

		if (!this.m_interaction) {
			return;
		}

		let $writing_area;
		const was_being_composed = this.m_interaction.initiative === "OUTBOUND" && this.m_interaction.state === "PENDING";
		const pending_origin_post = this.m_interaction.get_pending_origin_post();
		let afterSetFn = $.noop;

		if ( was_being_composed || pending_origin_post ) {
			$writing_area = $("#facebookWallText");
			afterSetFn = function() {
				self.onCharCountChanged($writing_area);
			};
		}
		else {
			if ( this.m_interaction.isPrivateMessage() ) {
				$writing_area = $("#facebookMailText");
			}
			else {
				$writing_area = $("#postCommentText");
			}
		}
		this.appendToInput($writing_area, message);
		this.focusInput($writing_area);
		afterSetFn();
	};

	FacebookClientWindow.prototype.on_account_changed = function()
	{
		FacebookClientWindow.uber.on_account_changed.call( this );

		$("#facebookAvailablePages").unbind("change");

		if ( !this.m_interaction.contact && this.m_interaction.initiative === "OUTBOUND" ) {
			$("#facebookAvailablePages").parent().hide();
			$("#facebookAvailablePages").children().remove();
			var accountId = this.m_interaction.getAccount();
			var account;
			var campaignAccounts = inConcert.app.getCampaignAddresses(this.m_interaction.campaign, "FACEBOOK");
			for (var i = 0; i < campaignAccounts.length; i++) {
				if (campaignAccounts[i].id === accountId) {
					account = campaignAccounts[i];
					break;
				}
			}

			if (account) {
				var campaign = inConcert.app.getCampaign(this.m_interaction.campaign);
				if (campaign) {
					var pagesNames = campaign.getAccountPages(accountId);
					if (pagesNames) {
						for (var page in pagesNames) {
							if (pagesNames.hasOwnProperty(page)) {
								$("#facebookAvailablePages").
									append($("<option></option>").
										attr("value",page).
										text(pagesNames[page]));
							}
						}

						$("#facebookAvailablePages").val(" ");
						$("#facebookAvailablePages").parent().show();
						$("#facebookAvailablePages").bind("change", function() {
							var page = $(this).val();
							var pageName = $("#facebookAvailablePages option[value='" + page + "']").text();
							var helperMessage = FORMAT("<%=_T([[Post Something On {0}s Page]])%>", pageName);
							$("#facebookWallMessage > .facebookInteractionControlTitle").text(helperMessage);
						}).change();
					}
				}
			}
		}
	};

	FacebookClientWindow.prototype.onCharCountChanged = function(textArea) {
		var currentChars = document.getElementById(textArea.attr("id")).value.length;
		var charsDifference = m_maxCharsOnTheWall - currentChars;
		var disabledWallButton = charsDifference < 0;
		$("#facebookClientCharacterCounter").text(charsDifference)[disabledWallButton ? "addClass" : "removeClass"]("facebookClientCharacterCounterNegative");
		$("#facebookWallButtonPost").prop("disabled", disabledWallButton);
	};

	FacebookClientWindow.prototype.setView = function( view_id, pending_event )
	{
		var class_name;
		var $comment = $("#facebookComment");
		var $wall = $("#facebookWallMessage");
		var $private_message = $("#facebookPrivateMessage");

		switch( view_id )
		{
			case "client" :
				class_name = "facebookClientContainer";
				//hide
				$private_message.hide();
				$wall.hide();
				// show
				$comment.show();
				break;

			case "wall_client" :
				class_name = "facebookWallClientContainer";
				var $wall_err_container = $("#facebookWallError");
				$wall_err_container.hide();

				if ( pending_event ) {
					var state_class = "";
					var state = pending_event.state || "";

					switch( state ) 
					{
						case "DELIVERED":
							// NO HAGO NADA!!! ( CLASS_DELIVERY_DELIVERED )
						break;
						case "DISCARDED":
							state_class = CLASS_DELIVERY_DISCARDED;
						break;
						case "DISCARDING":
							state_class = CLASS_DELIVERY_DISCARDING;
						break;
						case "FAILED":
							var short_msg = this.m_interaction.get_event_error_message( pending_event.error_code, pending_event.error_message || "" );
							$wall_err_container.show();
							$wall_err_container.find("#facebookWallErrorMessage").text(short_msg);
							state_class = CLASS_DELIVERY_FAILED;
						break;
						case "SENDING":
							state_class = CLASS_DELIVERY_SENDING;
						break;
						default:
							throw "Unrecognized pending event '" + state + "' state";
					}

					class_name += " " + state_class;
				}
				//hide
				$comment.hide();
				$private_message.hide();
				// show
				$wall.show();
				break;

			case "mail_client" :
				class_name = "facebookMailClientContainer";
				//hide
				$comment.hide();
				$wall.hide();
				// show
				$private_message.show();
				break;

			default:
				throw "FacebookClientWindow.prototype.setView. Unrecognized view_id: '" + view_id + "'";
		}

		$("#mainContainer")
			.removeClass()
			.addClass( class_name )
			.addClass("facebookInboundClientContainer");
	};

	FacebookClientWindow.prototype.reply_in_wall = function( pending_origin_post )
	{
		var self = this;

		this.setView("wall_client", pending_origin_post);
		$("#replyInWall").prop("disabled", true);

		var text = "";
		var $wall_text = $("#facebookWallText");
		if ( pending_origin_post ) {
			text = pending_origin_post.message;
			$wall_text.trigger("click");
		}

		document.getElementById("facebookWallText").value = text;

		$wall_text.focus();
		$("#facebookClientCharacterCounter").text(m_maxCharsOnTheWall);

		var on_send = function( success, message )
		{
			if ( !success ) {
			// Estos son errores genericos. Los errores de FB se procesan en "FacebookClientWindow.eventsSuscribe / failure"
				self.hideStatusMessage();
				self.displayStatusMessage( message, "Error" );
				return;
			}
			// El OK se procesa cuando llega el evento "ClientWindow.onInteractionUpdated -> FacebookClientWindow.clientUpdate"
		};

		$("#facebookWallButtonPost")
			.unbind('click')
			.bind('click', function()
			{
				if (!self.canExecuteInteractionAction()) {
					return;
				}
				self.displayStatusMessage("<%=_T([[Posting...]])%>", "Process", true);
				var message = document.getElementById("facebookWallText").value;
				var destination;
				var destinationType = "CONTACT";
				// Facebook doens't let us write wall post on friend's walls, just log error if
				// someone try to do it.
				if (self.m_interaction.initiative === "OUTBOUND" && self.m_interaction.contact) {
					inConcert.logger.error("Facebook doesn't allow to write post on friend's walls");
				}
				var destinationName;
				var page = $.trim($("#facebookAvailablePages").val());
				//first part of the "and" = user didn't select anything: default to profile
				//second part user selected the profile
				if (page !== "" && page !== self.m_interaction.account) {
					destination = page;
					destinationName = $("#facebookAvailablePages option[value='" + page + "']").text();
					destinationType = "PAGE";
				}
				else {
					destination = $("#cboAccounts").val();
					destinationType = "CAMPAIGN";
					var campaign = inConcert.app.getCampaign(self.m_interaction.campaign);
					if (campaign) {
						destinationName = campaign.Name;
					}
				}

				var fake_event_id; // tiene que ser pasado como parametro en undefined
				pending_origin_post = self.m_interaction.get_pending_origin_post();
				if ( !pending_origin_post ) {
					self.m_interaction.newPost( fake_event_id, destination, message, destinationType, destinationName, on_send );
				}
				else {
					// resend
					fake_event_id = pending_origin_post.fake_event_id;
					self.m_interaction._newPost( fake_event_id, destination, message, on_send );
				}
			}
		);
	};

	/*
	Esta funcion actualiza la informacion del usuario (id, name, avatar), de los eventos pendings
	En las acciones del lado del cliente (delivery_discarding_message y delivery_sending_message)
	Esta funcion no es incovada en las respuestas del delivery

	Es necesaria cuando el evento fallido lo genero un agente diferente al que se encuentra logueado.
	*/
	FacebookClientWindow.prototype.delivery_update_user_info = function( $pending )
	{
		var $author = $pending.find( ".subEventAuthor" );
		var author_id = $author.data( "event_user_id" );

		if ( typeof author_id !== "string" ) {
			return;
		}
		var logged_user_id = inConcert.app.getCompleteId();
		if ( author_id === logged_user_id ) {
			return;
		}

		$pending.find( ".subEventAvatar object" ).attr( "data", inConcert.app.getLoggedUserAvatar() );
		$author.data( "event_user_id", logged_user_id );
		$author.text( inConcert.app.getLoggedUserName() );
	};

	FacebookClientWindow.prototype.delivery_success_message = function( $success, success_event )
	{
		var $thread_container = $("#thread_container");

		var $siblings, $last_message, last_message_id;
		var parent_id = success_event.message.parent_id;
		var is_reply = _.isString( parent_id );

		// obtengo el ultimo mensaje del hilo al cual corresponde el evento SUCCESS
		if ( is_reply ) {
			// si es reply, lo obtengo de su hilo
			$siblings = $thread_container.find("[data-parent_id=" + parent_id +"]");
			$last_message = $siblings.last();
			last_message_id = $last_message.attr( "data-id" );
		}
		else {
			// si no es un reply, lo obtengo del hilo principal
			$siblings = $thread_container.find(".facebookThreadSubEvent");
			$last_message = $siblings.last();
			last_message_id = $last_message.attr( "data-id" );
		}

		var fake_event_id = $success.attr( "data-id" );
		var previous_state = $success.attr( "previous_state" );

		var move_last = ( previous_state === "failed" && fake_event_id !== last_message_id );
		if ( move_last ) {
		// si habia fallado y el ultimo mensaje del hilo al cual pertenece no es el mismo evento,
		// le realizao detach ya que se tiene que agrega al final de su thread ( como ultimo mensaje )
		// (*)
			$success.removeAttr( "previous_state" );
			$success.detach();
		}

		$success
			.removeClass( CLASS_DELIVERY_SENDING )
			.removeClass( CLASS_DELIVERY_FAILED )
			.addClass( CLASS_DELIVERY_SUCCESS )
			.attr( "data-id", success_event.message.id )
			.attr( "data-time", success_event.creationDate.getTime() );

		var $date = $success.find( ".subEventDate" );
		$date.text( inConcert.Utils.userFormatDate(success_event.creationDate, false, true));

		var $delivery_message_container = $success.find( ".deliveryMessageContainer" );
		var $lbl_message = $success.find( ".deliveryMessageText" );
		$lbl_message.text( "<%=_T([[Delivered]])%>" );

		$delivery_message_container
			.removeClass( CLASS_DELIVERY_SENDING )
			.removeClass( CLASS_DELIVERY_FAILED )
			.addClass( CLASS_DELIVERY_SUCCESS )
			.show();

		var $err_msg_container = $success.find( ".deliveryFailReasonContainer" );
		$err_msg_container.hide();

		var $failed_actions = $success.find( ".deliveryFailActions" );
		$failed_actions.hide();

		if ( move_last ) {
		// como hemos explicado arriba en (*), debemos mover el mensaje a la ultima posicion del hilo al cual pertenece!!!
			if ( is_reply ) {
			// si es reply tiene que validar si tiene hermanos o no, para saber donde ubicarlo!!
				if ( $siblings.length === 0 ) {
					$thread_container.find("[data-id='" + parent_id + "']").after( $success );
				}
				else {
					$siblings.last().after( $success );
				}
			}
			else {
			// si no es reply derecho al final
				$thread_container.append( $success );
			}

			var scroll_data = this.getScrollBasicInfo();
			scroll_data.destination = "item_position";
			scroll_data.$item = $success;
			SCROLL_TO( scroll_data );
		}

		window.setTimeout(function()
		{
			$success.removeClass( CLASS_DELIVERY_SUCCESS );
			$delivery_message_container.removeClass( CLASS_DELIVERY_SUCCESS ).hide();
		}, TIME_SELECTED_COMMENT_MS);
	};

	FacebookClientWindow.prototype.delivery_sending_message = function( $pending, message )
	{
		var has_failed = $pending.hasClass( CLASS_DELIVERY_FAILED );
		if ( has_failed ) {
			$pending.attr( "previous_state", "failed");
		}

		$pending
			.removeClass( CLASS_DELIVERY_FAILED )
			.addClass( CLASS_DELIVERY_SENDING );

		var $delivery_message_container = $pending.find( ".deliveryMessageContainer" );
		var $lbl_message = $delivery_message_container.find( ".deliveryMessageText" );
		$lbl_message.text( message || "<%=_T([[Sending]])%>" );
		$delivery_message_container
			.removeClass( CLASS_DELIVERY_FAILED )
			.addClass( CLASS_DELIVERY_SENDING )
			.show();

		var $err_msg_container = $pending.find( ".deliveryFailReasonContainer" );
		$err_msg_container.hide();

		var $failed_actions = $pending.find( ".deliveryFailActions" );
		$failed_actions.hide();

		this.delivery_update_user_info( $pending );
	};

	FacebookClientWindow.prototype.delivery_failed_message = function( failed_event, err_msg )
	{
		var $pending = $("#thread_container").find("[data-id='" + failed_event.fake_event_id +"']");
		if ( $pending.length === 0 ) {
			return;
		}

		$pending
			.removeClass( CLASS_DELIVERY_SENDING )
			.addClass( CLASS_DELIVERY_FAILED );

		var $delivery_message_container = $pending.find( ".deliveryMessageContainer" );
		var $lbl_message = $delivery_message_container.find( ".deliveryMessageText" );
		$lbl_message.text( "<%=_T([[Failed]])%>" );
		$delivery_message_container
			.removeClass( CLASS_DELIVERY_SENDING )
			.addClass( CLASS_DELIVERY_FAILED )
			.show();

		var $err_msg_container = $pending.find( ".deliveryFailReasonContainer" );
		var $err_msg = $err_msg_container.find( ".deliveryFailReasonTxt" );
		if ( $err_msg.text().length > 0 || err_msg ) {
			if ( err_msg ) {
				$err_msg.text( err_msg );
			}
		}

		if ( this.isEditable() ) {
			var $failed_actions = $pending.find( ".deliveryFailActions" );
			$failed_actions.show();
		}
	};

	FacebookClientWindow.prototype.delivery_discarding_message = function( $discarding )
	{
		$discarding
			.removeClass( CLASS_DELIVERY_FAILED )
			.addClass( CLASS_DELIVERY_DISCARDING );

		var $delivery_message_container = $discarding.find( ".deliveryMessageContainer" );
		var $lbl_message = $delivery_message_container.find( ".deliveryMessageText" );
		$lbl_message.text( "<%=_T([[Discarding]])%>" );
		$delivery_message_container
			.removeClass( CLASS_DELIVERY_FAILED )
			.addClass( CLASS_DELIVERY_DISCARDING )
			.show();

		var $err_msg_container = $discarding.find( ".deliveryFailReasonContainer" );
		$err_msg_container.hide();

		var $failed_actions = $discarding.find( ".deliveryFailActions" );
		$failed_actions.hide();

		this.delivery_update_user_info( $discarding );
	};

	FacebookClientWindow.prototype.delivery_discarded_message = function( $discarded )
	{
		$discarded
			.removeClass( CLASS_DELIVERY_SENDING )
			.removeClass( CLASS_DELIVERY_DISCARDING )
			.removeClass( CLASS_DELIVERY_FAILED )
			.addClass( CLASS_DELIVERY_DISCARDED );

		var $delivery_message_container = $discarded.find( ".deliveryMessageContainer" );
		var $lbl_message = $delivery_message_container.find( ".deliveryMessageText" );
		$lbl_message.text( "<%=_T([[Discarded]])%>" );
		$delivery_message_container
			.removeClass( CLASS_DELIVERY_SENDING )
			.removeClass( CLASS_DELIVERY_DISCARDING )
			.removeClass( CLASS_DELIVERY_FAILED )
			.addClass( CLASS_DELIVERY_DISCARDED )
			.show();

		var $err_msg_container = $discarded.find( ".deliveryFailReasonContainer" );
		$err_msg_container.hide();

		var $failed_actions = $discarded.find( ".deliveryFailActions" );
		$failed_actions.hide();
	};

	FacebookClientWindow.prototype.delivery_remove = function( failed_event )
	{
		var $pending = $("#thread_container").find("[data-id='" + failed_event.fake_event_id +"']");
		if ( $pending.length === 0 ) {
			return;
		}
		$pending.fadeOut( 1000, function(){ 
			$pending.remove();
		});
	};

	FacebookClientWindow.prototype.on_private_message_created = function( new_event, sorted )
	{
		var self = this;

		this.hideStatusMessage();
		this.updateMessageColapsedComments();

		var incoming_realtime = sorted;
		if ( !incoming_realtime ) {
			this.clearInput($("#facebookMailText"));
		}

		var $pending;
		if ( new_event.creator.toLowerCase() === "agent" && new_event.fake_event_id ) {
			$pending = $("#thread_container").find("[data-id='" + new_event.fake_event_id +"']");
		}

		if ( $pending && $pending.length > 0 ) {
			this.delivery_success_message( $pending, new_event );
		}
		else {
			var callback = function( _, can_scroll )
			{
				if ( !can_scroll ) {
					return;
				}
				var scroll_data = self.getScrollBasicInfo();
				scroll_data.destination = "bottom";
				SCROLL_TO( scroll_data );
			};
			this.add_new_message_stub( new_event,
				callback,
				sorted
			);
		}
	};
	
	FacebookClientWindow.prototype.replyByMailAction = function(callback = $.noop) {
		const self = this;
		if (!this.canExecuteInteractionAction()) {
			return callback(false, "can not send private message");
		}
		this.displayStatusMessage("<%=_T([[Sending...]])%>", "Process", true);

		//mark all as read
		if ($("#mark_all_read_unread").val() === this.MARK_ALL_READ_BTN_TEXT) {
			$("#mark_all_read_unread").click();
		}

		var text = $("#facebookMailText").val();
		var event = self.get_current_event();
		if ( !event ) {
			return callback(false, "event not found");
		}
		var fake_event_id = self.m_interaction.newPrivateMessage(
			undefined, // fake_event_id
			self.m_interaction.interactionData.page || event.message.page,
			text,
			function( success, message, error ) {
				if ( success ) {
				// Los success SE REALIZAN CUANDO LLEGA LA ACTUALIZACION DE LA INTERACTION "onUpdateInteraction->clientUpdate"
					return callback(true);
				}
				if ( error.agent_err ) {
				// estos son errores que se producen en controller interactions del lado del agente (ejemplo sin licencia para el canal).
				// Los errores de delivery, llegan desde el IR en la funcion "eventsSuscribe"
					event.fake_event_id = fake_event_id;
					self.hideStatusMessage();
					self.displayStatusMessage( message, "Error", true, 5000 );
					self.delivery_remove( event );
				}
				callback(false, message || "error sending message");
			}
		);
		self.add_new_outgoing_fake_event( fake_event_id, text );
	};

	FacebookClientWindow.prototype.replyByMail = function() 
	{
		var self = this;
		this.setView("mail_client");
		$("#replyInWall").prop("disabled", false);
		var helperMessage = "<%=_T([[Send a private message]])%>";
		$("#facebookPrivateMessage > .facebookInteractionControlTitle").text(helperMessage);
		var $txt = $("#facebookMailText");
		this.clearInput($txt);
		
		$txt.off('change').on('change', function() {
			$("#facebookMailButtonSend").prop("disabled", !$(this).val());
			self.Spellcheck();
		}).change();

		$("#facebookMailButtonSend").unbind('click').bind('click', function() {
			self.replyByMailAction();
		});
	};

	/*
		fake_event_id
		text
		parent_comment - opcional 
		creation_date  - opcional
		state          - opcional
		agent          - opcional
	*/
	FacebookClientWindow.prototype.make_fake_event = function( fake_data )
	{
		var self = this;

		var get_parent = function()
		{
			var origin_event = self.get_current_event();
			if ( !origin_event ) {
				return "";
			}
			return origin_event.message.id;
		};
		var creation_date = fake_data.creation_date || new Date(); // si no viene la fecha, toma now
		var contact_form = $("#contactID").text();

		return {
			outgoing_pending : true,
			state            : fake_data.state || "SENDING",
			agent            : fake_data.agent || inConcert.app.getCompleteId(),
			agentName        : contact_form,
			creationDate     : creation_date,
			parent_id        : get_parent(),
			message : {
				id           : fake_data.fake_event_id,
				created_time : creation_date.toString(),
				parent_id    : fake_data.parent_comment,
				message      : fake_data.text,
				from : {
					id   : this.m_interaction.getAccount(),
					name : contact_form
				}
			}
		};
	};

	FacebookClientWindow.prototype.add_new_outgoing_fake_event = function( fake_event_id, text, parent_comment /* opcional */ )
	{
		var self = this;
		var $txt;
		var scroll_data = this.getScrollBasicInfo();
		scroll_data.destination = "bottom";
		scroll_data.callback = function() {
			self.focusInput($txt);
		};

		if ( this.m_interaction.isPrivateMessage() ) {
			$txt = $("#facebookMailText");
		}
		else {
			$txt = $("#postCommentText");
			if ( parent_comment ) {
				scroll_data.destination = "item_position";
				scroll_data.type = "without_animation";
			}
		}
		this.clearInput($txt);

		var fake_event = this.make_fake_event({
			fake_event_id  : fake_event_id,
			text           : text,
			parent_comment : parent_comment
		});

		this.add_new_message_stub( fake_event, function( $comment )
		{
			if ( scroll_data.destination === "item_position" ) {
				scroll_data.$item = $comment;
			}
			SCROLL_TO( scroll_data );
		});
	};

	FacebookClientWindow.prototype.afterRenderRelatedMarkup = function() {
		$("#unreadTitle").html(`<%=_T('Unread messages')%> <span>(${this.m_interaction.getUnreadEventCount()})</span>`);
	};

	FacebookClientWindow.prototype.realtime_add_new_incoming_message = function ( $comment, new_event )
	{
		if ( !$comment || $comment.length === 0 ) {
			return false;
		}

		var can_scroll = false;
		var $container = $("#thread_container");
		var parent_comment = new_event.message.parent_id;
		var is_reply = _.isString( parent_comment );
		var $siblings = is_reply ? 
						$container.find(".replyMessage[data-parent_id='" + parent_comment +"']") :
						$siblings = $container.find(".topLevelMessage");

		if ( $siblings.length === 0 ) {
		// si es el primer evento del hilo principal o del hilo de una respuesta no tiene hermanos, lo inserta primero en la lista
			if ( is_reply ) {
				$container.find("[data-id='" + parent_comment +"']").after( $comment );
			}
			else {
				$container.append( $comment );
			}
		}
		else {
		// Si tiene hermanos, recorre el hilo al cual pertenece, buscando entre sus hermanos el primero que tenga una fecha/hora menor a la del current event.
		// La busqueda es realizada comenzando desde el ultimo ( ya que estan ordenado x fecha ).
			var added = false;
			var new_event_time = new_event.creationDate.getTime();

			for ( var i = $siblings.length - 1; i >= 0; i -- ) {
				if ( !$siblings[ i ] ) {
					continue;
				}
				var $sibling = $( $siblings[ i ] );
				var sibling_time = $sibling.data( "time" );
				if ( sibling_time <= new_event_time ) {
				// si la fecha del hermano es menor o igual al nuevo evento, inserto este nuevo evento despues del hermano
					$sibling.after( $comment );
					added = true;
					break;
				}
			}

			if ( !added ) {
			// si no existe un hermano menor tiene que agregarlo si o si al nodo
				if ( is_reply ) {
					$siblings.last().after( $comment );
				}
				else {
					$container.append( $comment );
				}
			}
		}

		delete new_event.is_new;
		return can_scroll;
	};

	FacebookClientWindow.prototype.add_new_message_stub = function ( new_event, callback, sorted )
	{
		var self = this;
		callback = callback || $.noop;
		var stub = new_event;
		this.m_interaction.getEvent( 0 ).children.push( stub );
		var event_index = new_event.__index || this.m_interaction.getEventCount();
		var $container = $("#thread_container");

		//
		this.insert_comment_in_dom(
			stub,
			event_index,
			function( $comment )
			{
				var can_scroll = true;
				if ( new_event.is_new && sorted ) {
				// aca se insertan los evento entrantes cuando la pagina es chequeada por el proxy
					can_scroll = self.realtime_add_new_incoming_message( $comment, new_event );
				}
				else {
				// aca se insertan los mensajes de la carga inicial
					var parent_comment = new_event.message.parent_id;
					var postId = self.m_interaction.interactionData.postId;
					if (!postId) { //old interactions
						postId = self.m_interaction.events[0].message.id;
					}
					if ( _.isString( parent_comment ) && parent_comment !== postId) {
						var $siblings = $container.find("[data-parent_id='" + parent_comment +"']");

						if ( $siblings.length === 0 ) {
							$container.find("[data-id='" + parent_comment +"']").after( $comment );
						}
						else {
							$siblings.last().after( $comment );
						}
					}
					else {
						$container.append( $comment );
					}
				}

				$comment.addClass( "new" );
				window.setTimeout( function() {
					$comment.removeClass( "new" );
				}, TIME_SELECTED_COMMENT_MS );

				callback( $comment, can_scroll );
			}
		);
	};

	FacebookClientWindow.prototype.setReplyToComment = function( commentId, name )
	{
		var self = this;
		this.m_reply_to_comment = commentId;
		$(".facebookInteractionControlTitle").show().text( "<%=_T([[Reply to]])%>" + "<%=_T([[:]])%>" + " " + name );
		$("#facebookInteractionPrivateContainer").show();
		$("#facebookCommentButton").val( "<%=_T([[SEND REPLY]])%>" );
		$("#mainContainer").addClass("commentReply");
		if (this.m_legazyPost) {
			$("#facebook_solve_container").show();
		}
		self.focusInput($("#postCommentText"));

		$("#cancelReplyToComment").show().unbind('click').bind('click', function() {
			$( this ).hide();
			$(".facebookInteractionControlTitle").hide();
			$("#facebookInteractionPrivateContainer").hide();
			$("#facebookCommentButton").val( "<%=_T([[COMMENT]])%>" );
			$("#mainContainer").removeClass("commentReply");
			self.focusInput($("#postCommentText"));
			self.m_reply_to_comment = null;
			if (self.m_legazyPost) {
				$("#facebook_solve_container").hide();
			}
		});
	};

	FacebookClientWindow.prototype.setPrivateReply = function( index, name )
	{
		var self = this;
		this.m_private_reply_index = index;
		$(".facebookInteractionControlTitle").show().text( "<%=_T([[Private message to]])%>" + "<%=_T([[:]])%>" + " " + name );
		$("#facebookInteractionPrivateContainer").show();
		$("#facebookCommentButton").val( "<%=_T([[SEND MESSAGE]])%>" );
		$("#mainContainer").addClass("commentReply");
		if (this.m_legazyPost) {
			$("#facebook_solve_container").show();
		}
		self.focusInput($("#postCommentText"));

		$("#cancelReplyToComment").show().unbind('click').bind('click', function() {
			$( this ).hide();
			$(".facebookInteractionControlTitle").hide();
			$("#facebookInteractionPrivateContainer").hide();
			$("#facebookCommentButton").val( "<%=_T([[COMMENT]])%>" );
			$("#mainContainer").removeClass("commentReply");
			self.focusInput($("#postCommentText"));
			delete self.m_private_reply_index;
			if (self.m_legazyPost) {
				$("#facebook_solve_container").hide();
			}
		});
	};

	FacebookClientWindow.prototype.getMessageColapsedComments = function( increment_count )
	{
		var count = this.m_interaction.getEventList().length + ( increment_count || 0 );
		return FORMAT( "<%=_T([[View all {0} comments]])%>", count );
	};

	FacebookClientWindow.prototype.updateMessageColapsedComments = function()
	{
		var $message = $('.facebookThreadSubEventSummary');
		//if the hidden markup is visible, update it with + 1 comment count
		if ( $message.is(':visible') ) {
			var message = this.getMessageColapsedComments();
			$message.find('a').text( message );
		}
	};

	FacebookClientWindow.prototype.on_comment_post_created = function( new_event, sorted )
	{
		var self = this;

		this.hideStatusMessage();
		this.updateMessageColapsedComments();

		var incoming_realtime = sorted;
		if ( !incoming_realtime ) {
			self.clearInput($("#postCommentText"));
		}

		var $pending;
		if ( new_event.creator.toLowerCase() === "agent" && new_event.fake_event_id ) {
			$pending = $("#thread_container").find("[data-id='" + new_event.fake_event_id +"']");
		}

		if ( $pending && $pending.length > 0 ) {
		// aca actualizo en pantalla el evento pending/sending.
			this.delivery_success_message( $pending, new_event );
		}
		else {
			var callback = function( $comment, can_scroll )
			{
				if ( can_scroll ) {
					var exist_comment_in_dom = $comment && $comment.length > 0;

					var scroll_data = self.getScrollBasicInfo();
					scroll_data.callback = function() {
						if ( !exist_comment_in_dom ) {
							return;
						}
					};

					if ( exist_comment_in_dom) {
						scroll_data.destination = "item_position";
						scroll_data.type = "without_animation";
						scroll_data.$item = $comment;
					}
					else {
						scroll_data.destination = "bottom";
					}

					SCROLL_TO( scroll_data );
				}

				if ( !incoming_realtime ) {
					$("#cancelReplyToComment").trigger("click");
				}
			};

			this.add_new_message_stub( new_event,
				callback,
				sorted
			);
		}
	};

	FacebookClientWindow.prototype.private_reply = function(index, message, callback = $.noop)
	{
		var self = this;
		this.displayStatusMessage("<%=_T([[Sending private message...]])%>", "Process", true);
		this.m_interaction.privateReply(index, message, function( success, message, error ) {
			self.hideStatusMessage();
			if (success) {
				callback(true);
			}
			else {
				self.displayStatusMessage( message, "Error", true, 5000 );
				callback(false, message || "error sending message");
			}
			if (success || error && error.result === "ALREADY_REPLIED") {
				$(FORMAT("[private_reply_link][data-event_index={0}]", index)).remove();
				$("#cancelReplyToComment").click();
				self.clearInput($("#postCommentText"));
			}
		});
	};

	FacebookClientWindow.prototype.isPrivatePostReply = function()
	{
		return typeof this.m_private_reply_index !=="undefined";
	};


	FacebookClientWindow.prototype.sendAttachment = function(option, fileData, callback = $.noop) {
		const self = this;
		if (this.isPrivatePostReply()) {

			if (!this.m_interaction.canPrivateReply(this.m_private_reply_index)) {
				self.displayStatusMessage("<%=_T('Error replying message.')%>", "Error", false, 5000);
				return callback(false);
			}

			this.displayStatusMessage("<%=_T([[Sending private message...]])%>", "Process", true);
			
			let replyCallback = function(success, message, error) {
				if (success) {
					self.hideStatusMessage();
				}
				if (success || error && error.result === "ALREADY_REPLIED") {
					$(FORMAT("[private_reply_link][data-event_index={0}]", self.m_private_reply_index)).remove();
					$("#cancelReplyToComment").click();
					self.clearInput($("#postCommentText"));
				}
				callback(success, message);
			};

			return FacebookClientWindow.uber.sendAttachment.call(this, option, fileData, replyCallback);
			
		}
		FacebookClientWindow.uber.sendAttachment.call(this, option, fileData, callback);
	};

	FacebookClientWindow.prototype.sendSharedFile = function(option, sharedFile, callback = $.noop){
		const self = this;
		if (this.isPrivatePostReply()) {
			if (!this.m_interaction.canPrivateReply(this.m_private_reply_index)) {
				self.displayStatusMessage("<%=_T('Error replying message.')%>", "Error", false, 5000);
				return callback(false);
			}
			this.displayStatusMessage("<%=_T([[Sending private message...]])%>", "Process", true);
			
			let replyCallback = function(success, message, error) {
				if (success) {
					self.hideStatusMessage();
				}
				if (success || error && error.result === "ALREADY_REPLIED") {
					$(FORMAT("[private_reply_link][data-event_index={0}]", self.m_private_reply_index)).remove();
					$("#cancelReplyToComment").click();
					self.clearInput($("#postCommentText"));
				}
				callback(success, message);
			};
			return FacebookClientWindow.uber.sendSharedFile.call(this, option, sharedFile, replyCallback);
		}

		FacebookClientWindow.uber.sendSharedFile.call(this, option, sharedFile, callback);
	};

	FacebookClientWindow.prototype.comment_current_post = function(callback = $.noop)
	{
		var self = this;
		if (!this.canExecuteInteractionAction()) {
			return callback(false, "can not send message");
		}
		var text = $("#postCommentText").val();
		this.focusInput($("#postCommentText"));
		if (!text) {
			return callback(false, "must enter a text");
		}
		var event = this.get_current_event();
		if ( !event ) {
			return callback(false, "event not found");
		}
		
		this.hideStatusMessage();
		this.displayStatusMessage("<%=_T([[Sending comment...]])%>", "Process", true);
		
		//mark all as read
		if ($("#mark_all_read_unread").val() === this.MARK_ALL_READ_BTN_TEXT) {
			$("#mark_all_read_unread").click();
		}
		
		if (this.m_private_reply_index || this.m_private_reply_index === 0) {
			return this.private_reply(this.m_private_reply_index, { text : text }, callback);
		}

		//reply to comment, or reply to post
		//IMPORTANT: for nested comments to work properly, the page in Facebook
		//must be configured with the option "Allow replies to comments on my Page"
		//otherwise it won't work

		var postId = this.m_interaction.interactionData.postId;
		var commentId = this.m_reply_to_comment || this.m_interaction.interactionData.commentId || "";
		if (!postId) { //old interactions
			postId = this.m_interaction.events[0].message.id;
		}

		var fake_event_id = this.m_interaction.commentPost(
			undefined, // fake event id - tiene que ser pasado como parametro en undefined
			postId,
			commentId,
			{ text : text },
			function( success, message ) {
				if ( success ) {
				// Los success SE REALIZAN CUANDO LLEGA LA ACTUALIZACION DE LA INTERACTION "onUpdateInteraction->clientUpdate"
					if (self.m_reply_to_comment) {
						$("#cancelReplyToComment").click();
					}
					return callback(true);
				}
				event.fake_event_id = fake_event_id;
				event.fake_event_id = fake_event_id;
				self.hideStatusMessage();
				self.displayStatusMessage( message, "Error", true, 5000 );
				self.delivery_remove( event );
				callback(false, message || "error sending message");
			}
		);
		this.add_new_outgoing_fake_event( fake_event_id, text, commentId);
	};

	FacebookClientWindow.prototype.getInteractionRelatedMarkup = function( properties )
	{
		let eventType = properties.event.message.type ? properties.event.message.type.toLowerCase() : "wall";
		let typeClass = 'facebookEventType' + (eventType === 'mail' || eventType === 'pagemail' || eventType === 'webhookMessage' ? 'PrivateMessage' : 'Wall');
		let photoClass = (eventType === "photo") ? "photoPost":"";
		let readClass = "relatedInteractionUnread";
		let message = inConcert.Utils.escapeAttrib(this.m_interaction.getMessage(properties.sorted_index));
		let message_abstract = inConcert.Utils.linkify( message.length <= 60 ? message : message.substring(0, 60) + '...', {exclude: {email: true}});
		let author_name = this.m_interaction.getUserName(properties.event);
		let genericAvatar = this.getGenericAvatar();
		let avatar = properties.avatarURI || this.m_interaction.getEventUserAvatarURI(properties.event) || genericAvatar;

		return $(''.concat(
			'<div unread_event="true" class="relatedInteraction ', readClass, ' ', typeClass, ' ', photoClass, '">',
				'<div class="interactionAvatar">',
					'<object data="', avatar, '" type="image/png" width=“36px" height="36px">',
						'<img src="', genericAvatar, '" width="36px" height="36px"/>',
					'</object>',
				'</div>',
				'<div class="interactionInfoContainer">',
					'<label class="interactionContactName">',
						author_name,
					'</label>',
					'<label class="interactionDate">', this.m_interaction.getLocaleDateString( properties.sorted_index ), '</label>',
					'<label class="interactionAbstract">', message_abstract, '</label>',
				'</div>',
				'<a unread_event_action="true" href="javascript:void(0);" class="interactionButton">',
					'<i  class="far fa-chevron-right"></i>',
				'</a>',
			'</div>'));
	};
	
	FacebookClientWindow.prototype.__getButtonMarkup = function(button)
	{
		return FORMAT("".concat(
			"<button class='facebookButton'>",
				"<label class='facebookButtonLabel'>",
					"{0}",
				"</label>",
			"</button>"
		), button.title);
	};
	
	FacebookClientWindow.prototype.__getGalleryElementMarkup = function(element)
	{
		var self = this;
		var imageHtml = element.image_url? FORMAT("".concat(
			"<div class='facebookGalleryElementImageContainer'>",
				"<img class='facebookGalleryElementImage' src='{0}'>",
			"</div>"), element.image_url) : "";
		var buttonsHtml = "";
		$.each(element.buttons || [], function(_, button) {
			buttonsHtml += self.__getButtonMarkup(button);
		});
		return FORMAT("".concat(
			"<div class='facebookGalleryElement'>",
				"{2}",
				"<label class='facebookGalleryElementTitle'>",
					"{0}",
				"</label>",
				"<label class='facebookGalleryElementSubtitle'>",
					"{1}",
				"</label>",
				"<div class='facebookGalleryButtons'>",
					"{3}",
				"</div>",
			"</div>"
		), element.title, element.subtitle || "", imageHtml, buttonsHtml);
	};
	
	FacebookClientWindow.prototype.__getListElementMarkup = function(element)
	{
		var self = this;
		var imageHtml = element.image_url? FORMAT("".concat(
			"<div class='facebookListElementImageContainer'>",
				"<img class='facebookListElementImage' src='{0}'>",
			"</div>"), element.image_url) : "";
		var buttonsHtml = "";
		$.each(element.buttons || [], function(_, button) {
			buttonsHtml += self.__getButtonMarkup(button);
		});
		return FORMAT("".concat(
			"<div class='facebookListElement'>",
				"<label class='facebookListElementTitle'>",
					"{0}",
				"</label>",
				"<label class='facebookListElementSubtitle'>",
					"{1}",
				"</label>",
				"<div class='facebookListButtons'>",
					"{3}",
				"</div>",
				"{2}",
			"</div>"
		), element.title, element.subtitle || "", imageHtml, buttonsHtml);
	};

	FacebookClientWindow.prototype.__getButtonsListMarkup = function(payload) 
	{
		let self = this;
		return (payload.buttons || []).reduce(function(content, button) {
			return content + self.__getButtonMarkup(button);
		}, "<div class='facebookButtonsContainer'>") + "</div>";
	};
	
	FacebookClientWindow.prototype.__getButtonsListMarkup = function(payload) 
	{
		let self = this;
		return (payload.buttons || []).reduce(function(content, button) {
			return content + self.__getButtonMarkup(button);
		}, "<div class='facebookButtonsContainer'>") + "</div>";
	};
	
	FacebookClientWindow.prototype.__getGalleryMarkup = function(payload) 
	{
		let self = this;
		return (payload.elements || []).reduce(function(content, element) {
			return content + self.__getGalleryElementMarkup(element);
		}, FORMAT("<div class='facebookGalleryContainer {0}Gallery'>", payload.image_aspect_ratio)) + "</div>";
	};
	
	FacebookClientWindow.prototype.__getListMarkup = function(payload) 
	{
		let self = this;
		let content = (payload.elements || []).reduce(function(content, element) {
			return content + self.__getListElementMarkup(element);
		}, "<div class='facebookListContainer'>") + "</div>";
		if (!$.isEmptyObject(payload.buttons)) {
			content += payload.buttons.reduce(function(content, button) {
				return content + self.__getButtonMarkup(button);
			}, "<div class='facebookListContainer'>") + "</div>";
		}
		return content;
	};
	
	FacebookClientWindow.prototype.__getTemplateMarkup = function(payload) 
	{
		switch(payload.template_type) {
			case "button":
				return this.__getButtonsListMarkup(payload);
			case "generic":
				return this.__getGalleryMarkup(payload);
			case "list":
				return this.__getListMarkup(payload);
			default:
				return "";
		}
	};
	
	FacebookClientWindow.prototype.__getImageMarkup = function(link) 
	{
		return link? FORMAT("<img class='imagePostedContainer' src='{0}'>", link) : "";
	};
	
	FacebookClientWindow.prototype.__getLinkMarkup = function(type, link, linkName, imageLink) 
	{
		return link? FORMAT("<a class='{0}' href='{1}' target='_blank'>{2}</a>",
			(type === "video"? "facebookVideoLink" : "facebookLink") + (imageLink? "Image" : ""),
			link,
			this.__getImageMarkup(imageLink) || linkName || link) : "";
	};
	
	FacebookClientWindow.prototype.__getAudioMarkup = function(link, mimeType)
	{
		return "".concat(
			"<div class='facebookMedia facebookAudio' file_type='audio' mime_type='", mimeType || "audio/mp3", "'>",
				"<button class='facebookMediaPlay'>",
					"<i class='fas fa-play'></i>",
				"</button>",
				"<button class='facebookMediaPause' hidden>",
					"<i class='fas fa-pause'></i>",
				"</button>",
				"<button class='facebookMediaStop' hidden>",
					"<i class='fas fa-stop'></i>",
				"</button>",
				"<a class='facebookMediaDownload' href='", link, "' target='_blank'>",
					"<i class='far fa-download'></i>",
				"</a>",
			"</div>");
	};
	
	FacebookClientWindow.prototype.__getVideoMarkup = function(link, mimeType)
	{
		return "".concat(
			"<div class='facebookMedia facebookVideo' file_type='video' mime_type='", mimeType || "video/mp4", "'>",
				"<button class='facebookMediaPlay'>",
					"<i class='fas fa-play'></i>",
				"</button>",
				"<a class='facebookMediaDownload' href='", link, "' target='_blank'>",
					"<i class='far fa-download'></i>",
				"</a>",
			"</div>");
	};
	
	FacebookClientWindow.prototype.__getFileMarkup = function(file) {
		// 1 - la url de descarga
		let url = !file.id ? file.url :
			FORMAT(
				"{0}/inconcert/apps/repository/download/?id={1}&section={2}&part={3}&name={4}",
				inConcert.app.getBaseDomain(),
				encodeURIComponent(file.id),             /* file id */
				"-1",                                    /* section */
				"-1",                                    /* part */
				encodeURIComponent(file.name || file.id) /* file name */
			);
		// 2 - la url del preview
		let previewUrl = !file.previewFileId ? file.previewUrl :
			FORMAT(
				"{0}/inconcert/apps/repository/download/?id={1}&section={2}&part={3}&name={4}",
				inConcert.app.getBaseDomain(),
				encodeURIComponent(file.previewFileId), /* file id */
				"-1",                                   /* section */
				"-1"                                    /* part */
			);
		//2020-09-14 DEPRECATED: fix some days before build error (remove after some years)
		file.fileType = file.fileType || file.type.split("/")[0];
		switch(file.fileType) {
			case "image":
				return this.__getImageMarkup(url);
			case "video":
				return file.id? this.__getVideoMarkup(url, file.type) :
					this.__getLinkMarkup("video", url, file.name, previewUrl);
			case "audio":
				return file.id? this.__getAudioMarkup(url, file.type) :
					this.__getLinkMarkup("video", url, file.name, previewUrl);
			default:
				return this.__getLinkMarkup("file", url, file.name, previewUrl);
		}
	};
	
	//2020-09-14 DEPRECATED: replaced with files (remove after some years)
	FacebookClientWindow.prototype.__getAttachmentMarkup = function(attachment) 
	{
		let self = this;
		if ($.isEmptyObject(attachment)) {
			return;
		}
		//deprectated content
		if (attachment.image_data) {
			return this.__getImageMarkup(attachment.image_data.url);
		}
		if (attachment.video_data) {
			return this.__getLinkMarkup("video", attachment.video_data.url, null, attachment.video_data.preview_url);
		}
		if (attachment.file_url) {
			if (attachment.mime_type && attachment.mime_type.indexOf("audio") === 0) {
				return this.__getAudioMarkup(attachment.file_url, attachment.mime_type);
			}
			else {
				return this.__getLinkMarkup("file", attachment.file_url, attachment.name);
			}
		}
		//multiple content
		if (attachment.subattachments) {
			return (attachment.subattachments.data || []).reduce(function(content, subattach) { 
				return content + self.__getAttachmentMarkup(subattach);
			}, "");
		}
		//content
		let media = attachment.media || {};
		let target = attachment.target || {};
		let payload = attachment.payload;
		let type = attachment.type || "share";
		if (type.indexOf("video") >= 0) {
			type = "video";
		}

		let linkName, link, imageLink;

		switch(type) {
			//media
			case "sticker":
			case "photo":
			case "video":
			case "share":
				link = media.source || target.url || attachment.url;
				linkName = attachment.title;
				imageLink = media.image && media.image.src;
				return link? this.__getLinkMarkup(type, link, linkName, imageLink) : this.__getImageMarkup(imageLink);
			//messenger image
			case "image":
				return this.__getImageMarkup(payload.url);
			//messenger file
			case "file":
				if (payload.url) {
					var parts = payload.url.split("/");
					linkName = parts[parts.length - 1].replace(/\?.*/g, "");
					return this.__getLinkMarkup("file", payload.url, linkName);
				}
				return "";
			//messenger location
			case "location":
				link = attachment.url && inConcert.Utils.getUrlParameter(attachment.url, "u");
				return this.__getLinkMarkup("file", link, "<%=_T([[My location]])%>");
			//messenger templates
			case "template":
				return this.__getTemplateMarkup(payload);
			//another media or messenger content
			default:
				link = media.source || target.url || payload.url || attachment.url;
				linkName = attachment.title;
				imageLink = media.image && media.image.src;
				return link? this.__getLinkMarkup(type, link, linkName, imageLink) : this.__getImageMarkup(imageLink);
		}
	};

	FacebookClientWindow.prototype.getAttachmentsMarkup = function( event )
	{
		var self = this;
		// retorna el markup correspondiente a imagenes, etc.
		// tambien retorna un disclaimer si tiene archivos adjuntos.

		var response = {
			disclaimer : "",
			media      : "",
			resend	   : true
		};

		if ( !event || !event.message ) {
			return response;
		}
		
		var message = event.message;
		if (message.message && $.isPlainObject(message.message)) {
			message = message.message;
		}
		
		if (message.msgtype === "attachment" && event.state) {
			response.disclaimer = "<%=_T([[Attachment file]])%>";
			response.resend = false;
			return response;
		}
		
		let files = this.m_interaction.getFiles(event);
		if (files) {
			$.each(files, function(_, file) {
				response.media += self.__getFileMarkup(file);
			});
		}
		else {
			//2020-09-14 DEPRECATED: replaced with files (remove after some years)
			//direct messages shares link
			var shares = message && message.shares && message.shares.data && 
					$.isArray(message.shares.data) && message.shares.data.length && message.shares.data[0];
			if (shares && shares.link) {
				var extension = this.__extractExtension(shares.link);
				if (extension === "png") {
					response.media += FORMAT(
						"<a class='facebookLinkImage' href='{0}' target='_blank'><img class='imagePostedContainer' src='{0}'></a>",
						shares.link);
				}
				else {
					response.media += FORMAT("<a class='{0}' href='{1}' target='_blank'>{2}</a>",
						"facebookLink", shares.link, shares.name || shares.description || "<%=_T([[Link]])%>");
				}
			}
			let attachments = this.m_interaction.getAttachments(event);
			$.each( attachments || [], function(_, attachment) {
				response.media += self.__getAttachmentMarkup(attachment) || "";
			});	
		}
		
		//quickreplies
		if (message && message && message.quick_replies && message.quick_replies.length) {
			var quickReplies = "<div class='facebookQuickrepliesContainer'>";
			$.each(message.quick_replies, function(_, quickReply) {
				var text = "";
				var icon = "";
				switch(quickReply.content_type) {
					case "location":
						text = "<%=_T([[Location]])%>";
						break;
					case "text":
						text = quickReply.title;
						icon = quickReply.image_url || "";
						break;
					case "user_phone_number":
						text = "<%=_T([[User Phone Number]])%>";
						break;
					case "user_email":
						text = "<%=_T([[User E-Mail]])%>";
						break;
					default:
						text = quickReply.title || "<%=_T([[Unrecognized]])%>";
				}
				var iconHtml = icon? FORMAT("".concat(
						"<i class='facebookQuickreplyIcon'>",
							"<img class='facebookQuickreplyIconImage' src='{0}'>",
						"</i>"), icon) : "";
				quickReplies += FORMAT("".concat(
					"<button class='facebookQuickreply'>",
						"{1}",
						"<label class='facebookQuickreplyLabel'>",
							"{0}",
						"</label>",
					"</button>"
				), text, iconHtml);
			});
			quickReplies += "</div>";
			response.media += quickReplies;
		}

		return response;
	};

	FacebookClientWindow.prototype.__extractExtension = function(url)
	{
		var url_a = (url || "").split("?")[0].split(".");
		return url_a.length > 1? url_a[url_a.length - 1].toLowerCase() : "";
	};

	FacebookClientWindow.prototype.isReadEvent = function( event, event_index )
	{
		if ( typeof event !== "object" ) {
			throw "FacebookClientWindow.isReadEvent - event is not a object.";
		}
		if ( event.outgoing_pending ) {
			return true;
		}
		if ( !event_index ) {
			return true;
		}
		return this.m_interaction.isRead( event_index );
	};

	FacebookClientWindow.prototype.insert_comment_in_dom = function( event, event_index, callback = $.noop )
	{
		const self = this;

		event_index = event_index || -1;
		var failed_event;

		if ( event.outgoing_pending && event.timestamp ) {
			if ( event.state === "FAILED" ) {
				failed_event = $.extend( {}, event );
			}
			event = this.make_fake_event({
				fake_event_id  : event.fake_event_id,
				text           : event.message,
				parent_comment : event.parent_id,
				creation_date  : event.creationDate,
				state          : event.state,
				agent          : event.agentAtVCC || event.agent
			});
		}
		var id = event.message.id;
		var parent_id = event.message.parent_id;

		var author_name = this.m_interaction.getUserName( event );
		var author_id   = this.m_interaction.get_user_event_id( event );
		var attach_html = this.getAttachmentsMarkup( event );
		var message_text = inConcert.Utils.linkify( inConcert.Utils.escapeAttrib( this.m_interaction.getMessageText(event.message) ), { exclude: { email: true } } );
		var message_class = (event.message.webhookType === "REFERRAL" || event.message.webhookType === "OPTIN")? "referralMessage" : "";

		var postId = this.m_interaction.get_origin_message_id();
		var has_parent = _.isString( parent_id ) && parent_id !== postId;
		var html_reply = '';

		if (this.isEditable()) {
			if ( event.outgoing_pending !== true && (this.m_wallMessage || this.m_legazyPost) ) {
				html_reply += FORMAT(
					''.concat(
						'<a reply_comment_link class="threadActionsReply" href="javascript:void(0);"',
								' data-event_index="{0}" data-message_id="{1}" data-user_name="{2}">',
							"<%=_T([[Reply]])%>",
						'</a>'
					),
					event_index,						// data-event_index
					has_parent? parent_id : id,			// data-message_id
					author_name							// data-user_name
				);
			}
			if ( this.m_interaction.canPrivateReply(event_index)) {
				html_reply += FORMAT(
					''.concat(
						'<a private_reply_link class="threadActionsPrivateReply" href="javascript:void(0);"',
								' data-event_index="{0}" data-message_id="{1}" data-user_name="{2}" title="', 
								'<%=_T("Reply user by a private message")%>' ,'">',
							"<%=_T([[Private]])%>",
						'</a>'
					),
					event_index,						// data-event_index
					has_parent? parent_id : id,			// data-message_id
					author_name							// data-user_name
				);
			}
		}

		var class_container, attr_parent_id;
		if ( has_parent ) {
			class_container = "replyMessage";
			attr_parent_id = ' data-parent_id="' + parent_id + '" ';
		}
		else {
			class_container = "topLevelMessage";
			attr_parent_id = '';
		}

		if (event.creator === "contact") {
			class_container += " contactEvent";
		}
		else if (event.creator === "bot") {
			class_container += " botEvent";
		}
		else {
			class_container += " agentEvent";
		}

		var read_unread_lnk = this.get_html_read_unread_link( event, event_index, true );
		var like_unlike_lnk = this.get_html_like_unlike_link( event, event_index );

		class_container += read_unread_lnk.parent_class;
		class_container += like_unlike_lnk.parent_class;

		var attr_id =  ' data-id="' + id + '" ';
		var attr_index =  ' data-event_index="' + event_index + '" ';
		var time = ' data-time="' + event.creationDate.getTime() + '" ';

		var genericAvatar = this.getGenericAvatar();
		var avatar = this.m_interaction.getEventUserAvatarURI(event) || genericAvatar;

		var $comment = $( ''.concat(
			'<div class="facebookThreadSubEvent ', class_container, '" ', attr_id, attr_parent_id, attr_index, time, '>',
				'<div class="subEventMainContainer">',
					'<div class="subEventHeaderContainer">',
						// Avatar
						'<div class="subEventAvatar">',
							'<object data="', avatar, '" type="image/png" width="24" height="24">',
								'<img src="', genericAvatar, '" width="24" height="24"/>',
							'</object>',
						'</div>',
						// Name & Date
						'<div class="subEventGralInfo">',
							'<a href="javascript:void(0);" class="subEventAuthor" data-event_index="', event_index , '" data-event_user_id="', author_id , '">', author_name, '</a>',
							'<label class="subEventDate">', inConcert.Utils.userFormatDate(event.creationDate, false, true), '</label>',
						'</div>',
						// Buttons
						like_unlike_lnk.html,
						'<div class="dropdown">',
							'<div class="subEventActionsContainer dropdown-menu dropdown-menu-right">',
								read_unread_lnk.html,
								html_reply,
							'</div>',
							'<button class="buttonToggle buttonToggleSM" type="button" data-toggle="dropdown" aria-expanded="false">',
								'<i class="fas fa-ellipsis-v"></i>',
							'</button>',
						'</div>',
					'</div>',
					'<div class="subEventMessageContainer">',
						'<label message_text="true" class="subEventMessage ', message_class , '">', message_text || attach_html.disclaimer, '</label>',
						attach_html.media,
					'</div>',
				'</div>',
				'<div class="deliveryFailActions" hidden>',
					'<a discard_message class="pageButtonGeneric buttonColorWhite" href="javascript:void(0);">', "<%=_T([[Discard]])%>", '</a>',
					 attach_html.resend? ('<a resend_message class="pageButtonGeneric buttonColorBlue" href="javascript:void(0);">' + "<%=_T([[Resend]])%>" + '</a>') : '',
				'</div>',
				// label de estado
				'<div class="deliveryMessageContainer" hidden >',
					'<label class="deliveryMessageText" ></label>',
				'</div>',
				// tooltip - mensaje de por que fallo
				'<div class="deliveryFailReasonContainer" hidden>',
					'<label class="deliveryFailReasonTxt"></label>',
				'</div>',
				'<div class="subEventDateMailContainer" style="display: none;">',
					'<label class="subEventDate">', inConcert.Utils.userFormatDate(event.creationDate, false, true), '</label>',
				'</div>',
			'</div>'
		));

		this.__processMediaPlayers($comment);

		if (event.creator === "contact") {
			const $message_text = $comment.find('[message_text]');
			$message_text
				.off('dblclick')
				.on('dblclick', function() {
					self.setMessageInSideBar(message_text);
				}
			);
		}

		// inserta in dom
		callback( $comment );

		if ( event.outgoing_pending ) {
			var state = event.state || "";
			switch( state ) 
			{
				case "DELIVERED":
					// NO HAGO NADA!!! ( delivery_success_message )
				break;
				case "DISCARDED":
					this.delivery_discarded_message( $comment );
				break;
				case "DISCARDING":
					this.delivery_discarding_message( $comment );
				break;
				case "FAILED":
					var short_msg = this.m_interaction.get_event_error_message( failed_event.error_code, failed_event.error_message || "" );
					this.delivery_failed_message( failed_event, short_msg );
				break;
				case "SENDING":
					this.delivery_sending_message( $comment );
				break;
				default:
					throw "Unrecognized pending event '" + state + "' state";
			}
		}
	};

	// Reorder events so that nested comments are placed right after the original comment
	// But maintaining the chronological order between comments in the same scope
	FacebookClientWindow.prototype.reorder_nested_events = function ()
	{
		//TODO: Revisar esta funcion, que una linda y hermosa bosta!!
		//		ya lo era, y ahora aun mas!!! Despues que este aprobado el issue de ACK, revisar y realizar refactoring de la misma Ellio!!
		//

		var pending_events = {};
		var count_pending_events = this.m_interaction.count_total_pending_events();

		if ( count_pending_events > 0 ) {

			pending_events = inConcert.Utils.cloneObject( this.m_interaction.get_pending_events() );

			// Es necesario, agregar algunos fake atributos los eventos pending ( creationDate, outgoing_pending, parent_id )
			//
			// creationDate: a partir del timestamp, necesito crear un objeto Date, para que pueda ser ordenado el hash de pending, y para que lo pueda insertar en orden en la lista posta de eventos.
			// outgoing_pending: es necsario para diferenciar los eventos pending de los que no son pending en la lista unificada de sorted_events, para su posterior utilizacion.
			// parent_id: se necesita este atributo para que puedan ser ordenados en su propio hilo de comments.

			$.each( pending_events, function( _, pending_event )
			{
				pending_event.creationDate = new Date( parseInt( pending_event.timestamp, 10 ) * 1000 );
				pending_event.outgoing_pending = true;
				if ( pending_event.reply ) {
					pending_event.parent_id = pending_event.destination;
				}
			});

			// si tengo mas de un evento pending, mando ordenar
			if ( count_pending_events > 1 ) {
				// Ordena el hash por fecha de creacion
				pending_events = inConcert.Utils.sorted_hash( 
					// hash desordenado
					pending_events,

					// funcion que retorna el valor por el cual realizar el orden de los pendings ( fecha de creacion )
					function( pending_event ) { return pending_event.creationDate; },

					// funcion que genera el hash ordenado
					function( target_hash, pending_event ) { target_hash[ pending_event.Id ] = pending_event; return target_hash; }
				);
			}
		}

		var replies_by_top_level = {};
		var events = this.m_interaction.getEventList() || [];
		var i, e, parent_id;
		//first iterate over all events and group each one by parent
		for ( i = 0; i < events.length; i++ ) {
			e = events[i];
			parent_id = e.message.parent_id;
			if ( parent_id ) {
				replies_by_top_level[ parent_id ] = replies_by_top_level[ parent_id ] || [];
				replies_by_top_level[ parent_id ].push(e);
			}
		}

		// agrego los pending de replies a la lista de replies
		var added_pending_events = {};
		$.each( pending_events, function( pending_id, pending_event )
		{
			if ( pending_event.reply ) {
				var parent_id = pending_event.parent_id;
				replies_by_top_level[ parent_id ] = replies_by_top_level[ parent_id ] || [];
				replies_by_top_level[ parent_id ].push( pending_event );
				added_pending_events[ pending_id ] = true;
			}
		});

		// ahora reordeno la lista de replies x fecha
		$.each( replies_by_top_level, function( parent_id, list )
		{
			replies_by_top_level[ parent_id ] = inConcert.Utils.sortArray( list, "creationDate" );
		});
		// hago clean up de los pendings que ya se han agregado
		$.each( added_pending_events, function( pending_id )
		{
			delete pending_events[ pending_id ];
		});


		// esta funcion agrega los pending a nivel de post original, ordenados por fecha
		var add_sorted_pending_events = function( list, current_event, compare_fn )
		{
			var added_pending_events = {};
			$.each( pending_events, function( pending_id, pending_event )
			{
				if ( compare_fn( pending_event, current_event ) ) {
					list.push( pending_event );
					added_pending_events[ pending_id ] = true;
				}
			});

			$.each( added_pending_events, function( pending_id )
			{
				delete pending_events[ pending_id ];
			});
		};

		var sort_pending_fn = function( pending_event, current_event ) { return pending_event.creationDate <= current_event.creationDate; };
		var sorted_events = [];
		var id;
		//second, push each event in the final array
		//if the event has replies, the push each one after it
		for (i = 0; i < events.length; i++) {
			e = events[i];
			id = e.message.id;
			parent_id = e.message.parent_id;

			if ( !parent_id ) {
				add_sorted_pending_events(
					sorted_events,
					e,
					sort_pending_fn
				);
				sorted_events.push(e);
				if (replies_by_top_level[id]) {
					sorted_events = sorted_events.concat(replies_by_top_level[id]);
					delete replies_by_top_level[id];
				}
			}
		}

		// x ulitmo tengo que agregar los pendings a nivel post original que no se han agregado!!
		add_sorted_pending_events(
			sorted_events,
			undefined,
			function() { return true; }
		);

		//these comments does not have any parent
		//we put them at the last place
		for ( id in replies_by_top_level ) {
			if ( replies_by_top_level.hasOwnProperty(id) ) {
				sorted_events = sorted_events.concat(replies_by_top_level[id]);
			}
		}

		return sorted_events;
	};

	FacebookClientWindow.prototype.canShowRelatedSections = function()
	{
		return this.m_interaction.getUnreadEventCount() > 0;
	};

	FacebookClientWindow.prototype.closeRelated = function( original_index )
	{
		var self = this;
		var $container = $( "#thread_container" );

		var on_ready = function()
		{
			if ( self.m_currentEvent === 0 ) {
				// cuando se cierra el tab de unread al post origen lo dejo marcado como leido!!!
				var current_event = self.m_interaction.getEvent( self.m_currentEvent );
				if ( self.isReadEvent( current_event, self.m_currentEvent ) ) {
					var id = current_event.message.id;
					var $post = $container.find( "[data-id='" + id + "']" );
					if ( $post && $post.length > 0 ) {
						self.on_readed_event( true, $post, true );
					}
				}
			}
			else {
				// cuando esta listo, siempre tiene que quedar en el evento cero (el post original m_currentEvent = 0)
				// ya que el input para comentar siempre que se cierra el related es sobre el post original
				self.m_currentEvent = 0;
			}
			self.hideStatusMessage();
			if ( self.m_interaction.isPrivateMessage() ) {
				self.focusInput($("#facebookMailText"));
			}
			else {
				self.focusInput($("#postCommentText"));
			}
		};

		if ( !this.can_show_all_comments ) {
			this.can_show_all_comments = original_index === undefined ? false : true;
		}

		FacebookClientWindow.uber.closeRelated.call( this, original_index, function()
		{
			if ( original_index === undefined ) {
				on_ready();
				return;
			}
			var current_event = self.m_interaction.get_event_by_original_index( original_index );
			if ( current_event === false || !current_event.message || !current_event.message.id ) {
				on_ready();
				return;
			}
			var id = current_event.message.id;
			var $message = $container.find( "[data-id='" + id + "']" );
			if ( !$message || $message.length === 0 ) {
				on_ready();
				return;
			}

			var $mark_read_unread = $message.find( "[mark_as_read_unread]" );
			if ( $mark_read_unread.length > 0 ) {
				$mark_read_unread.hide();
			}

			if ( self.isEditable() ) {
				if ( !$message.hasClass( "unread" ) ) {
					$message.addClass( "unread" );
				}
			}
			if ( !$message.hasClass( "selected" ) ) {
				$message.addClass( "selected" );
			}

			var scroll_data = self.getScrollBasicInfo();
			scroll_data.destination = "item_position";
			scroll_data.$item = $message;
			scroll_data.callback = function()
			{
				window.setTimeout( function() {
					self.on_readed_event( true /* success */, $message, true /* is read */ );
				}, TIME_SELECTED_COMMENT_MS );

				on_ready();
			};
			SCROLL_TO( scroll_data );
		});
	};

	FacebookClientWindow.prototype.on_readed_event = function( success, $comment, is_read )
	{
		if ( !success || typeof $comment !== "object" || $comment.length === 0 ) {
			return;
		}

		if ( !this.isEditable() ) {
			$comment.removeClass( "selected" );
			return;
		}

		// thread link class
		var class_mark_as_read = LNK_THREAD_MARK_AS_READ_CLASS;
		var class_mark_as_unread = LNK_THREAD_MARK_AS_UNREAD_CLASS;
		if ( $comment.hasClass( "facebookThreadOrigin" ) ) {
		// origin link class
			class_mark_as_read = LNK_ORIGIN_MARK_AS_READ_CLASS;
			class_mark_as_unread = LNK_ORIGIN_MARK_AS_UNREAD_CLASS;
		}

		var new_state, new_caption;
		var $mark_read_unread = $comment.find("[mark_as_read_unread]");
		$mark_read_unread.prop( "disabled", false );
		var state = $mark_read_unread.data( "event_unread" );
		if ( is_read || state === "unread" ) {
			new_state = "read";
			new_caption = "<%=_T([[Mark as unread]])%>";
			$comment.removeClass( "unread selected" );
			$mark_read_unread.removeClass( class_mark_as_read ).addClass( class_mark_as_unread );
		}
		else {
			new_state = "unread";
			new_caption = "<%=_T([[Mark as read]])%>";
			$comment.addClass( "unread" );
			$mark_read_unread.removeClass( class_mark_as_unread ).addClass( class_mark_as_read );
		}
		$mark_read_unread.data( "event_unread", new_state ).text( new_caption ).show();
	};

	FacebookClientWindow.prototype.on_liked_event = function( success, liked, $object )
	{
		if ( !success || typeof $object !== "object" || $object.length === 0 ) {
			return;
		}
		$object
			.toggleClass("liked", liked)
			.find("[like_unlike]")
				.toggleClass(LNK_UNLIKE_CLASS, liked)
				.toggleClass(LNK_LIKE_CLASS, !liked)
				.data("event_like", liked? "unlike" : "like")
				.html(liked? "" : "");
	};

	FacebookClientWindow.prototype.get_html_read_unread_link = function( event, event_index, is_thread )
	{
		var response = {
			html         : '',
			parent_class : ''
		};

		var lnk_mark_as_read_class = LNK_THREAD_MARK_AS_READ_CLASS;
		var lnk_mark_as_unread_class = LNK_THREAD_MARK_AS_UNREAD_CLASS;
		if ( !is_thread ) {
			lnk_mark_as_read_class = LNK_ORIGIN_MARK_AS_READ_CLASS;
			lnk_mark_as_unread_class = LNK_ORIGIN_MARK_AS_UNREAD_CLASS;
		}

		if ( event.outgoing_pending === true || event.creator === 'agent' ) {
			return response;
		}

		if ( !this.isReadEvent( event, event_index ) ) {
			response.parent_class = ' unread';
			if ( this.isEditable() ) {
				response.html = FORMAT(
					''.concat(
						'<a mark_as_read_unread class="', lnk_mark_as_read_class, '" href="javascript:void(0);"',
								' data-event_index="{0}" data-event_unread="{1}">',
							"<%=_T([[Mark as read]])%>",
						'</a>'
					),
					event_index,  // data-event_index
					'unread'
				);
			}
		}
		else {
			if ( this.isEditable() ) {
				response.html = FORMAT(
					''.concat(
						'<a mark_as_read_unread class="', lnk_mark_as_unread_class, '" href="javascript:void(0);"',
								' data-event_index="{0}" data-event_unread="{1}">',
							"<%=_T([[Mark as unread]])%>",
						'</a>'
					),
					event_index,  // data-event_index
					'read'
				);
			}
		}

		return response;
	};

	FacebookClientWindow.prototype.get_html_like_unlike_link = function( _ /*event*/, event_index )
	{
		var response = {
			html         : '',
			parent_class : ''
		};

		let subtype = this.m_interaction.getSubtype();
		event_index = this.m_interaction.get_sorted_index_by_original_index(event_index);
		if (event_index === false || subtype === "FanPageRateFeed" || !this.m_wallMessage && !this.m_legazyPost && (!this.m_postComment || event_index === 0)) {
			return response;
		}

		var liked = this.m_interaction.isLiked(event_index);
		if (liked) {
			response.parent_class = " liked";
		}
		if ( this.isEditable() ) {
			let class_name = liked ? LNK_UNLIKE_CLASS : LNK_LIKE_CLASS;
			let attr_action_name = liked ? "unlike" : "like";
			response.html = FORMAT(
				''.concat(
					'<a like_unlike class="pageButtonGeneric iconOnlyBtnSm buttonColorTertiary {0}" href="javascript:void(0);" data-event_index="{1}" data-event_like="{2}">',
						liked ? "" : "",
					'</a>'
				),
				class_name,      // nombre de la clase (css) dependiendo de si es un like o un unlike
				event_index,     // valor del atributo 'data-event_index'
				attr_action_name // valor del atributo 'data-event_like'
			);
		}

		return response;
	};

	FacebookClientWindow.prototype.openCurrentEvent = function(callback = $.noop)
	{
		const self = this;

		var execute_callback = function( success ) {
			self.hideStatusMessage();
			callback( success );
			if (self.__loadDraftMessage) {
				self.__loadDraftMessage = false;
				self.loadDraftMessage();
			}
		};

		if ( this.are_showing_all_messages || this.loading_all_comments ) {
			execute_callback( false );
			return;
		}

		var $container = $("#thread_container");
		$container.find(".facebookThreadOrigin").remove();
		$container.find(".facebookThreadSubEventSummary").remove();
		$container.find(".facebookThreadSubEvent").remove();

		var firstEventIndex = 0;
		var current_event = this.m_interaction.getEvent( firstEventIndex );
		if ( !current_event || !current_event.message || current_event.message.pending ) {
			execute_callback( false );
			return;
		}

		var getMessageMarkup = function() {
			var escAttrib = inConcert.Utils.escapeAttrib;
			var linkify = inConcert.Utils.linkify;
			var messageMarkup = "";
			var content = self.m_interaction.getMessageContent( firstEventIndex );
			//add recomendation
			if (content.recommendation_type) {
				var recomendationMarkup = FORMAT("<div class='facebookPostRecomendation {0}Recomendation'></div>", 
					content.recommendation_type);
				messageMarkup += recomendationMarkup;
			}
			//add label
			messageMarkup += '<label>' + linkify(escAttrib(content.text), {exclude: {email: true}}) + '</label>';
			//add rating
			if (content.rating && !isNaN(content.rating)) {
				var rateMarkup = "<div class='facebookPostRate'>";
				for (var i = 1; i <= 5; i++) {
					rateMarkup += FORMAT("<div class='facebookPostRateStar {0}'></div>",
						i <= content.rating? "positive" : "negative");
				}
				rateMarkup += "</div>";
				messageMarkup += rateMarkup;
			}
			return messageMarkup;
		};

		var event_id = current_event.message.id;
		var date_time = new Date( current_event.creationDate.toString() );
		var author_name = this.m_interaction.getUserName( current_event );
		var genericAvatar = this.getGenericAvatar();
		var avatar = this.m_interaction.getEventUserAvatarURI( current_event ) || genericAvatar;
		var read_unread_lnk = this.get_html_read_unread_link( current_event, firstEventIndex, false );
		var like_unlike_lnk = this.get_html_like_unlike_link( current_event, firstEventIndex );
		var message_class = (current_event.message.webhookType === "REFERRAL" || current_event.message.webhookType === "OPTIN")? "referralMessage" : "";
		var html_reply = '';

		if (this.isEditable()) {
			if ( this.m_interaction.canPrivateReply(firstEventIndex)) {
				html_reply += FORMAT(
					''.concat(
						'<a private_reply_link class="buttonColorGray actionsPrivateReply" href="javascript:void(0);"',
								' data-event_index="{0}" data-message_id="{1}" data-user_name="{2}" title="', 
								'<%=_T("Reply user by a private message")%>' ,'">',
							"<%=_T([[Private]])%>",
						'</a>'
					),
					firstEventIndex,					// data-event_index
					event_id,							// data-message_id
					author_name							// data-user_name
				);
			}
		}

		var class_container = "facebookThreadOrigin facebookEventType" +
			( this.m_interaction.isPrivateMessage() ? 'PrivateMessage' : 'Wall' ) +
			read_unread_lnk.parent_class +
			like_unlike_lnk.parent_class;

		if (current_event.creator === "contact") {
			class_container += " contactEvent";
		}
		else if (current_event.creator === "bot") {
			class_container += " botEvent";
		}
		else {
			class_container += " agentEvent";
		}

		var $root_node = $( ''.concat(
			'<div class="', class_container, '" data-id="', event_id , '">',
				'<div class="facebookThreadInfoContainer">',
					'<div class="facebookThreadEventTypeIcon" style="display:none"></div>',
					'<div class="facebookThreadOriginAvatar">',
						'<object data="', avatar, '" type="image/png">',
							'<img src="', genericAvatar, '"/>',
						'</object>',
					'</div>',
					'<div class="facebookMainInfoContainer">',
						'<a href="javascript:void(0);" class="originAuthor">', author_name, '</a>',
						'<label class="originDate">', inConcert.Utils.userFormatDate(date_time), '</label>',
						'<div class="originMessage ', message_class, '">', getMessageMarkup(), '</div>',
						html_reply,
						like_unlike_lnk.html,
						read_unread_lnk.html,
						'<label class="originDate originDateMail"style="display: none;">', inConcert.Utils.userFormatDate(date_time), '</label>',
					'</div>',
				'</div>',
			'</div>')
		);

		var $thread_container = $container.find( ".facebookThreadCommentControlContainer" );
		$thread_container.before( $root_node );

		// this window
		if ( this.m_interaction.accountHasContactManagementEnabled( this.from_app ) ) {
			$('.originAuthor').unbind('click').bind('click', function() {
				let avatar_uri = inConcert.app.getBaseUrl() + self.m_interaction.getAvatarURI();
				var contact_window = new inConcert.ContactWindow({
					from_app    : self.from_app,
					interaction : self.m_interaction,
					name        : author_name,
					avatar      : avatar_uri,
					addresses   : current_event.addresses
				});
				contact_window.show();
			});
		}

		var events = this.reorder_nested_events();

		$thread_container.show();

		////
		var hiddenComments = Math.max( events.length - 1 - self.m_maxCompressedComments, 0 );
		var base = self.can_show_all_comments ? 0 : hiddenComments;

		// esta funcion carga el detalle en el header de POST/PM original.
		// la misma es invocada, desde la funcion 'append_messages' una vez que ha finalizado con la carga de todos los comentarios del POST/PM
		//
		var append_header_detail = function()
		{
			if ( !self.can_show_all_comments ) {
				if ( hiddenComments > 0 ) {
					var increment_count = -1; // aca se debe decrementar en uno ya que el post original, no se debe de contar
					var message = self.getMessageColapsedComments( increment_count );
					var $hidden_node = $(
						'<div class="facebookThreadSubEventSummary">' +
							'<a href="javascript:void(0);">' + message + '</a>' +
						'</div>'
					);
					$root_node.after( $hidden_node );
					$hidden_node.find('a')
						.unbind('click').click(function() {
							self.can_show_all_comments = true;
							self.openCurrentEvent( function( success )
							{
								if ( !success ) {
									return;
								}
								var scroll_data = self.getScrollBasicInfo();
								scroll_data.destination = "bottom";
								SCROLL_TO( scroll_data );
								self.hideStatusMessage();
							});
						});
				}
			}

			self.are_showing_all_messages = base === 0;
			self.loading_all_comments = false;

			var attach_html = self.getAttachmentsMarkup( current_event );
			//if current message has an attachment, show disclaimer
			if ( attach_html.disclaimer ) {
				$(".facebookThreadOrigin").append($( attach_html.disclaimer ));
			}
			// if current message has an media
			if ( attach_html.media ) {
				$(".facebookThreadOrigin .originMessage").after($( attach_html.media ));
				self.__processMediaPlayers($(".facebookThreadOrigin .originMessage"));
			}

			self.has_created_main_thread = true;
			self.m_currentEvent = 0;
			execute_callback( true );
		};

		// a la carga de los mensajes la agrego en un timeout, para que se pueda visualizar el mensaje de waiting.
		// ya que sino el mismo no se visualiza si tiene muchos mensajes y comienza a agregar en el DOM.

		// esta funcion muestra un mensaje informativo en el cual va inidicando la cantidad de mensajes que se van cargando.
		//
		var count_events = events.length;
		var show_loading_message = function( count_added )
		{
			var count_comments  = count_events - 1; // se le debe de restar uno, ya que dentro de los eventos tambien esta el POST original, y en el mensaje se de de informar los comments agregados
			count_added = count_added > count_comments ? count_comments : count_added;
			var message;
			if ( count_added === undefined ) {
				message = "<%=_T([[Please wait, loading messages.]])%>";
			}
			else {
				message = FORMAT( "<%=_T([[Please wait, loading messages, {0} of {1}.]])%>", count_added, count_comments );
			}
			self.displayStatusMessage( message, "Process", false );
		};

		// Esta funcion es una funcion recursiva y es la encargada de cargar los mensajes (eventos) en el markup
		// La misma se ejecuta dentro de un timeout minimo, y se va cargando de a unos 50 elementos, luego se invoca nuevamente hasta terminar la carga.
		// Se ejecuta dentro de un timeout, para que el navegador no quede colgado, en el loop y en la carga del DOM, si es que tiene muchos mensajes.
		// Tambien es ejecutada dentro del timeout, porque sino el mensaje de info de la funcion 'show_loading_message', no se visualiza en pantalla cuando comienza el loop.
		// Una vez que termina la carga de los mensajes, invoca a la funcion que carga el detalle en el POST/PM original.
		//
		var max_append_per_loop = 50;
		var pause_between_loop_ms = 10;
		var added = base + 1;

		var append_messages = function( count_added )
		{
			show_loading_message( count_added );

			// timeout que contiene la funcion que carga los mensajes.
			window.setTimeout( function() 
			{
				var added_in_loop = 0;

				for ( var i = added; i < count_events; i++ )
				{
					var current_children_event = events[i];

					if(current_children_event.creator === "system"){ //no mostrar mensajes de sistema en el agente (bienvenida, fuera de tiempo, finalizar)
						added++;
						continue;
					}

					self.insert_comment_in_dom(
						current_children_event,
						current_children_event.__index,
						function ($comment)
						{
							var parent_comment = current_children_event.message.parent_id;
							var postId = self.m_interaction.get_origin_message_id();
							if ( _.isString( parent_comment ) && parent_comment !== postId) {
								var $siblings = $container.find("[data-parent_id='" + parent_comment +"']");

								if ( $siblings.length === 0 ) {
									$container.find("[data-id='" + parent_comment +"']").after( $comment );
								}
								else {
									$siblings.last().after( $comment );
								}
							}
							else {
								$container.append( $comment );
							}
						}
					);

					added_in_loop++;
					added++;
					if ( added_in_loop === max_append_per_loop ) {
						break;
					}
				}

				if ( added === count_events ) {
					// FIN. muestro mensaje de que se ha completado la carga, cargo los mensajes en el DOM e invoco a la funcion que carga los datos del detalle.
					// el Append al DOM lo tengo que realizar en un timeout, porque sino el ultimo mensaje de info de que se ha cargado correctamente, no es visualizado.
					show_loading_message( added );
					window.setTimeout(function() {
						append_header_detail();
					}, pause_between_loop_ms);
				}
				else {
					// si aun quedan mensajes por cargar invoca la misma funcion recursiva
					append_messages( added );
				}

			}, pause_between_loop_ms);
		};

		// aca comienza la carga de los mensajes
		this.loading_all_comments = true;
		append_messages();
	};

	FacebookClientWindow.prototype.update_hash_events_return_diff = function () 
	{
		var self = this;
		var new_events = [];
		if ( !this.m_interaction || this.m_interaction.isBeingComposed() || !$.isArray( this.m_interaction.events ) ) {
			return new_events;
		}
		var events = this.reorder_nested_events();
		$.each( events || [], function( i, event )
		{
			if ( !event || !event.message || !event.message.id ) {
				return true; // continue
			}
			var evt_id = event.message.id.toUpperCase();
			if ( !self.hash_events[ evt_id ] ) {
				if ( i > 0 ) {
					event.is_new = true;
				}
				new_events.push( event );
				self.hash_events[ evt_id ] = true;
			}
		});
		return new_events;
	};

	FacebookClientWindow.prototype.append_new_events = function () 
	{
		var self = this;
		var new_events = this.update_hash_events_return_diff();
		if ( new_events.length === 0 ) {
			return;
		}

		var append = function()
		{
			$.each( new_events, function( _, new_event )
			{
				var sorted = false;
				if ( new_event.creator === "contact" ) {
					// Siempre que el nuevo evento sea generado x un contacto,
					//    lo tengo que agregar ordenado x fecha en el DOM.
					// Los eventos del agente no se ordenan, ya que los mismo ya fueron agregado en el "delivery sending" y actualizados en el "delivery success/failed"
					sorted = true;
				}
				if ( self.m_interaction.isPrivateMessage() ) {
					self.on_private_message_created( new_event, sorted );
				}
				else {
					self.on_comment_post_created( new_event, sorted );
				}
			});
		};

		if ( this.has_created_main_thread ) {
			append();
		}
		else {
			this.openCurrentEvent();
		}
	};

	FacebookClientWindow.prototype.clientUpdate = function () 
	{
		if (!this.m_interaction.isValid()) {
			return;
		}
		if ( this.was_being_composed && !this.m_interaction.isBeingComposed() && ( this.m_interaction.events || [] ).length > 0 ) {
			this.close();
			return;
		}

		$("#postCommentText").prop("disabled", false);
		$("#facebookComment").prop("disabled", false);
		$("#facebookWallText").prop("disabled", false);
		$("#facebookWallButtonPost").prop("disabled", false);

		this.append_new_events();
		this.set_mark_all_unread_read_button_caption();
	};

	FacebookClientWindow.prototype.change_client_state = function( new_state )
	{
		new_state = FacebookClientWindow.uber.change_client_state.call( this, new_state );

		var $solve_container = $("#facebook_solve_container");

		switch( new_state )
		{
			case "archived":
				$solve_container.hide();
			break;

			case "connected":
				$solve_container.show();
			break;

			case "monitoring":
				$solve_container.hide();
			break;

			case "pending":
				$solve_container.hide();
			break;
		}

		return new_state;
	};

	FacebookClientWindow.prototype.firstLoadDraftMessage = function() {
		this.__loadDraftMessage = true;
	};

	FacebookClientWindow.prototype.__addDraftTextMessage = function(message, callback) {
		let self = this;
		if (!message.replyEvent && message.replyEvent !== 0) {
			var events = this.m_interaction.getEventList();
			var lastContactEvent = null;

			events.forEach(function(currentEvent) {
				if (currentEvent.creator && currentEvent.creator === "contact") {
					lastContactEvent = currentEvent;
				}
			});
			message.replyEvent = lastContactEvent.__index;
		}
		// Cuando se utiliza desde el API integration y si se ha ejecutado en modo iFrame, la validacion de que el boton se encuentra o no oculto se realiza de la siguiente forma, 
		//   ya que sino el selector de jQuery "....:visible" siempre retorna como si estuviera oculto.
		const $cancelReplyToComment = $("#cancelReplyToComment");
		const isVisibleBtnCancelToComment = inConcert.Utils.is_visible_dom_element($cancelReplyToComment);
		if (isVisibleBtnCancelToComment) {
			$cancelReplyToComment.click();
		}
		if (message.replyComment) {
			const selector = FORMAT("[reply_comment_link][data-event_index={0}]", message.replyEvent);
			const $elements = $(selector);
			if ($elements.length > 0) {
				let error = this.buildReplyComment($elements);
				if (error) {
					return callback(false, error);
				}
			}
			else {
				return callback(false, "can not reply comment");
			}
		}
		else if (message.privateReply) {
			const selector = FORMAT("[private_reply_link][data-event_index={0}]", message.replyEvent);
			const $elements = $(selector);
			if ($elements.length > 0) {
				let error = this.buildPrivateReply($elements);
				if (error) {
					return callback(false, error);
				}
			}
			else {
				return callback(false, "can not private reply");
			}
		}
		else if (message.like || message.unlike) {
			const $btn = this.$btnLikeUnLike(message.replyEvent);
			if ($btn.length === 0) {
				return callback(false, FORMAT("can not {0} message", message.like ? "like" : "unlike"));
			}
		}
		window.setTimeout(function() {
			self.setInputText(self.$input(), message.text || "");
			callback(true);
		}, 1000);
	};

	FacebookClientWindow.prototype.$btnLikeUnLike = function(event_index) {
		const selector = FORMAT("[like_unlike][data-event_index={0}]", event_index || "");
		const $btn = $(selector);
		return $btn;
	};

	FacebookClientWindow.prototype.__sendDraftTextMessage = function(message, callback) {
		if (message.like) {
			if (this.m_interaction.isLiked(message.replyEvent)) {
				callback(true);
			}
			else {
				const $btn = this.$btnLikeUnLike(message.replyEvent);
				this.likeUnlikeObject($btn, callback);
			}
		}
		else if (message.unlike) {
			if (!this.m_interaction.isLiked(message.replyEvent)) {
				callback(true);
			}
			else {
				const $btn = this.$btnLikeUnLike(message.replyEvent);
				this.likeUnlikeObject($btn, callback);
			}
		}
		else if (this.m_interaction.isPrivateMessage()) {
			this.replyByMailAction(callback);
		}
		else {
			this.comment_current_post(callback);
		}
	};
	
	FacebookClientWindow.prototype.__processMediaPlayers = function($message) {
		let self = this;
		$message.find(".facebookMedia").each(function() {
			self.__processMediaPlayer($message, $(this));
		});
	};
	
	FacebookClientWindow.prototype.__processMediaPlayer = function($message, $container) {
		let self = this;
		let player = null;
		let $play = $container.find(".facebookMediaPlay");
		let $pause = $container.find(".facebookMediaPause");
		let $stop = $container.find(".facebookMediaStop");
		let $download = $container.find(".facebookMediaDownload");
		let content = {
			fileType : $container.attr("file_type"),
			contentType : $container.attr("mime_type"),
			link : $download.attr("href")
		};
		$play.click(function() {
			if (!player) {
				let guid = inConcert.Utils.generateGuid();
				$message.find("[message_text]").after("".concat(
					"<div id='", guid, "' class='", content.fileType, "FacebookPlayer'>",
					"</div>"
				));
				$play.prop("disabled", true);
				player = new CInconcertPlayer({
					containerId : guid,
					url : content.link,
					fileFormat : inConcert.Utils.getFileExtensionFromMimeType(content.contentType),
					fileType : content.fileType,
					contentType : content.contentType,
					success : function() {
						player.play();
						if (content.fileType === "video") {
							$play.hide();
						}
						else {
							$play.prop("disabled", false);
							$play.hide();
							$pause.show();
							$stop.show();
						}
					},
					error : function() {
						self.displayStatusMessage("<%=_T([[Error playing audio]])%>", "Error", false, 2000);
					}
				});
				player.onEnded(function() {
					if (content.fileType !== "video") {
						$play.show();
						$pause.hide();
						$stop.hide();
					}
				});
				player.load();
				return;
			}
			player.play();
			$play.hide();
			$pause.show();
			$stop.show();
		});
		if (content.fileType !== "video") {
			$stop.click(function() {
				player.stop();
				$play.show();
				$pause.hide();
				$stop.hide();
			});
			$pause.click(function() {
				player.pause();
				$play.show();
				$pause.hide();
			});
		}
	};

	FacebookClientWindow.prototype.hasAttachment = function() {
		return true;
	};

	FacebookClientWindow.prototype.getAttachmentOptions = function() {
		let options = FacebookClientWindow.uber.getAttachmentOptions.call(this);
		options.image.description = false;
		options.video.description = false;
		options.audio.description = false;
		options.file.description = false;
		options.image.type.extensions = [ "jpg", "jpeg" , "png" ];
		options.video.type.extensions = [ "mp4", "3gpp" ];
		options.audio.type.extensions = [ "aac", "amr", "mp3", "ogg" ];
		options.file.type.extensions = [ "doc", "docx", "pdf", "ppt", "pptx", "xls", "xlsx",
			"7z", "csv", "eml", "ics", "key", "log", "neon", "numbers", "odt", "pages", "pps", "ppsx", 
			"txt", "vcf", "xml", "yml", "yaml" ];
		if (!this.m_interaction.isPrivateMessage()) {
			options.video.disabled = true;
			options.audio.disabled = true;
			options.file.disabled = true;
		}
		return options;
	};

	FacebookClientWindow.prototype.hasDragAndDropFiles = function() {
		return true;
	};

	FacebookClientWindow.prototype.getDragAndDropControl = function() {
		return this.$emojiInput();
	};

	FacebookClientWindow.prototype.hasAudioRecord = function() {
		return this.m_interaction.isPrivateMessage();
	};

	FacebookClientWindow.prototype.onAudioRecordStarted = function() {
		this.$inputContainer().hide();
	};

	FacebookClientWindow.prototype.onAudioRecordStopped = function() {
		this.$inputContainer().show();
	};
	
	FacebookClientWindow.prototype.processAttachmentData = function(data) {
		data.commentId = this.m_reply_to_comment || this.m_interaction.interactionData.commentId || null;
		
		if (this.isPrivatePostReply()) {
			data.PrivatePostReply = {
				index : this.m_private_reply_index,
				event : this.m_interaction.getEvent(this.m_private_reply_index),
			};
		}
		return data;
	};

	FacebookClientWindow.prototype.__$getfbbtnspellcontainer = function() {
		return this.$inputContainer().find("[fbbtnspellcontainer]");
	};

	FacebookClientWindow.prototype.__$getfbincorrectwordlist = function() {
		return $("#" + this.__getIncorrectWordListID());
	};

	FacebookClientWindow.prototype.__initializeButtonsSpell = function() {
		let $containerBtsSpell = this.__$getfbbtnspellcontainer();
		$containerBtsSpell.html("");
		const btns = ("").concat(
				'<button id="fbButtonSpellcheck" title="<%=_T("Check")%>" class="pageButtonGeneric iconOnlyBtn buttonColorWhite"><i class="far fa-spell-check"></i></button>',
				'<div class="languageComboContainer">',
					'<select id="cboLanguage" class="languageCombo"></select>',
				'</div>');
		$containerBtsSpell.append($(btns));
		
	};

	FacebookClientWindow.prototype.ChangeSpellcheckLanguage = function( new_language )
	{
		if( !this.m_spellchecker ) {
			return;
		}
		this.m_spellchecker.config.lang = new_language;
		this.m_spellchecker.webservice.config.lang = new_language;
		this.m_spellchecker.webservice.defaultConfig.data.lang = new_language;

		this.__$getfbincorrectwordlist().hide();
		this.Spellcheck();
	};

	FacebookClientWindow.prototype.HasToSpellcheck = function ()
	{
		if(!this.m_spellchecker) {
			return false;
		}

		var current_text = this.$input().val().trim();
		return current_text !== "";
	};

	FacebookClientWindow.prototype.Spellcheck = function ()
	{
		if(this.HasToSpellcheck()) {
			if (this.m_spellchecker_timeout) {
				this.m_spellchecker_timeout = null;
			}
			const self = this;
			this.m_spellchecker_timeout = setTimeout(() => {
				if (self.m_spellchecker) {
					self.m_spellchecker.check();
				}
			}, 1000);
		}
		else {
			const $incorrect_words = this.__$getfbincorrectwordlist().find('.spellchecker-incorrectwords');
			$incorrect_words.empty().parent().hide();
		}
	};

	FacebookClientWindow.prototype.__getIncorrectWordListID = function() {
		const was_being_composed = this.m_interaction.initiative === "OUTBOUND" && this.m_interaction.state === "PENDING";
		const pending_origin_post = this.m_interaction.get_pending_origin_post();
		if ( was_being_composed || pending_origin_post ) {
			return "fbwallincorrectwordlist";
		}
		else {
			if ( this.m_interaction.isPrivateMessage() ) {
				return "fbprivatemsgincorrectwordlist";
			}
			else {
				return "fbcommentincorrectwordlist";
			}
		}
	};

	FacebookClientWindow.prototype.createSpellChecker = function () {
		if ( !$.hasOwnProperty("SpellChecker")) {
			return false;
		}
		const self = this;
		this.destroySpellChecker();
		const $body = $("body");
		const validate_spellchecker_permissions = function()
		{
			// add to dictionary permission
			const allowed_add_to_dictionary = self.hasPermissionToAction( 'SPELL_CHECKER_ADD_WORD' );
			const $add_dictionary = $body.find('.ignore-forever');
			if ( allowed_add_to_dictionary ) {
				$add_dictionary.show();
			}
			else {
				$add_dictionary.hide();
			}
		};

		const vcc = inConcert.app.get_vcc_id();
		const language = this.get_language();
		var $text = this.$input();
		this.m_spellchecker = new $.SpellChecker($text, {
			lang: language,
			parser: 'text',
			webservice: {
				path: inConcert.app.getBaseDomain() + '/inconcert/apps/spellchecker/spellcheck/' + vcc,
				driver: 'pspell'
			},
			suggestBox: {
				position: 'above'
			},
			incorrectWords: {
				container: "#" + this.__getIncorrectWordListID()
			}
		});

		var $incorrect_words_list = this.__$getfbincorrectwordlist();
		var $incorrect_words = $incorrect_words_list.find('.spellchecker-incorrectwords');
		// Bind spellchecker handler functions
		this.m_spellchecker.on('check.success', function () {
			$incorrect_words.empty().parent().hide();
		});
		
		this.m_spellchecker.on('replace.word', function () {
			// if there are no more words to be replaced, hide the container
			if ($incorrect_words.children().length === 0) {
				$incorrect_words_list.hide();
			}
			const unicodeToImageText = window.emojiPicker.unicodeToImage($text.val());
			const $emoji = self.$emojiInput();
			$emoji.html(unicodeToImageText);
			$emoji.change();
		});

		const $spellcheck_feedback = $('#fbSpellcheckFeedback');
		this.m_spellchecker.on('check.fail', function () {
			if ($text.val().trim() !=="") {
				$incorrect_words_list.show();
				$spellcheck_feedback.hide();
				validate_spellchecker_permissions();
				return;
			}
			$incorrect_words.empty().parent().hide();	
		});

		return true;
	};

	FacebookClientWindow.prototype.destroySpellChecker = function()
	{
		if ( this.m_spellchecker ) {
			this.m_spellchecker.destroy();
			this.m_spellchecker = null;
		}

		if (this.m_spellchecker_timeout) {
			this.m_spellchecker_timeout = null;
		}
	};

	//when jquery says so, start it all up
	$(document).ready(function()
	{
		//initializates emoji
		window.emojiPicker = new EmojiPicker({
			emojiable_selector: '[data-emojiable=true]',
			assetsPath: '/inconcert/lib/emoji-picker/lib/img/',
			popupButtonClasses: 'emoji-picker-button',
			norealTime: false,
			onPaste: function(pasted) {
				/*
					Este callback se debe especificar para que el texto que se va pegar desde el portapapeles, no incluya ENTERs que no van en el texto
					cuando el origen es por ejemplo una planilla de Excel.
					Aunque el callback simplemente retorna el mismo valor que ha recibido, quien realiza el laburo es el mismo plugins "EMOJI"
					cuando obtiene la información a pegar desde portapapeles en formato 'text/plain' y luego invoca al callback.
					Archivo: "\lib\inconcert\agent\inconcert\lib\emoji-picker\lib\js\jquery.emojiarea.js"
					Ref Issue #7071
				*/
				return pasted;
			}
		});
		window.emojiPicker.discover();

		window.hook_environment();
		inConcert.i18n.updateScreen();

		var callback = function()
		{
			var client = new FacebookClientWindow(window.interaction,
				{
					archive  : window.archive,
					from_app : window.from_app,
					queued   : window.queued,
					waiting  : window.waiting,
					monitor  : window.monitor,
					readonly : window.readonly || window.archive || window.monitor || window.queued || window.waiting
				}
			);

			window.onReady(client);

			if (window && window.view && window.view instanceof inConcert.ClientWindowProxy) {
				window.view.apply(client);
			}
			window.view = client;
		};

		if(window.interaction.isBeingComposed()) {
			callback();
		}
		else {
			window.interaction.inflate(callback);
		}
	});
}());
