/* jshint jquery:true, strict: true, quotmark:false
*/

/* global inConcert */

if ( !this.inConcert ) {
	this.inConcert = {};
}

(function () {

	'use strict';

	var _T = inConcert.i18n.getString;
	var format = inConcert.Utils.format;

	function FinalizationUI( options )
	{
		this.client_window        = options.client_window;
		this.container_id         = options.container_id;
		this.finalization_include = options.container_id + '-include';
		this.finalization_mail    = options.container_id + '-mail';
		this.finalization_message = options.container_id + '-message';
		this.current_message_conf = null;
		this.initialize( options.on_ready );
	}

	if ( typeof inConcert.FinalizationUI !== 'function' ) {
		inConcert.FinalizationUI = FinalizationUI;
	}

	FinalizationUI.prototype.initialize = function( on_ready )
	{
		var self = this;
		on_ready = on_ready || $.noop;

		this.can_create( function( success, message ) {
			self.current_message_conf = message;
			if ( !success || !self.has_configured() ) {
				$( '#' + self.container_id ).hide();
				return;
			}
			self.create_markup();
			self.set_events();
			self.set_values();
			on_ready();
		});
	};

	FinalizationUI.prototype.has_configured = function()
	{
		var config = this.current_message_conf;
		if ( !config || config.agent_control !== '1' ) {
			return false;
		}
		return true;
	};

	FinalizationUI.prototype.can_create = function( callback )
	{
		callback = callback || $.noop;

		var interaction = this.client_window.m_interaction;
		if ( !interaction || !interaction.isTaken() || !interaction.belongsToMe() ) {
			callback( false );
			return;
		}
		
		interaction.getFinalizationMessageConfiguration(function(status, message) {
			callback(status, message);
		});
	};

	FinalizationUI.prototype.create_markup = function()
	{
		var $markup = $( ''.concat(
			'<div class="finalizationControlContainer">',
				'<label for="', this.finalization_include, '" class="controlLabel">', "<%=_T('Finalization e-mail')%>", "<%=_T(':')%>", '</label>',
				'<input id="', this.finalization_include, '" class="controlCheck" type="checkbox" />',
				'<input id="', this.finalization_mail, '" class="controlField" type="text" disabled />',
			'</div>'
		));

		$( '#' + this.container_id ).html( $markup ).show();
	};

	FinalizationUI.prototype.load_finalization_destination_address = function( callback )
	{
		callback = callback || $.noop;

		var interaction = this.client_window.m_interaction;
		if ( !interaction ) {
			callback( false );
			return;
		}

		var get_first_address_from_list = function( addresses ) {
			var mail;
			$.each( addresses || {}, function( _, address ) {
				var proxy_type = address.Type || address.type;
				if ( ( proxy_type || "" ).toUpperCase() === "MAIL" ) {
				// me quedo con la primer direccion de mail que tenga el contacto
					mail = address;
					return false; // break
				}
			});
			return mail;
		};

	// 1 - Searching finalization destination address by pre survey.

		if ( interaction.interactionData ) {
			var data = interaction.interactionData.data;
			if ( data && data.contact ) {
				var contact_address = get_first_address_from_list( data.contact.addresses );
				if ( contact_address ) {
					callback( contact_address.address );
					return;
				}
			}
		}

	// 2 - (**) Searching finalization destination address by interaction.contact

		var load_contact_by_external_id = function() {
	// 3 - Searching finalization destination address by external_id
			var external_id = interaction.interactionData &&
				interaction.interactionData.gatheredContactData &&
				interaction.interactionData.gatheredContactData.external_id;

			if ( external_id ) {
				inConcert.app.getContactManager().getContactByExternalId(
					external_id,
					function( contact ) {
						if ( contact ) {
							var contact_address = get_first_address_from_list( contact.addresses );
							if ( contact_address ) {
								callback( contact_address.address );
								return;
							}
						}
						callback( false );
					});
			}
			else {
				callback( false );
			}
		};

		// (**)
		var load_contact_by_id = function()
		{
			var contact_id = interaction.contact;
			// contact id sample: var contact_id = "CHAT6b90a346ee23490da38815d00bfc674f@inconcert";

			if ( contact_id && contact_id.length > 0 ) {
				inConcert.app.getContactManager().getContact(
					contact_id,
					function( contact ) {
						if ( contact ) {
							var contact_address = get_first_address_from_list( contact.addresses );
							if ( contact_address ) {
								callback( contact_address.address );
								return;
							}
						}
						load_contact_by_external_id();
					});
			}
			else {
				load_contact_by_external_id();
			}
		};

		//
		load_contact_by_id();
	};

	FinalizationUI.prototype.set_values = function()
	{
		var self = this;
		if ( !this.has_configured() ) {
			return;
		}
		var interaction = this.client_window.m_interaction;
		if ( !interaction ) {
			return;
		}
		var $include = $( '#' + this.finalization_include );
		var $mail = $( '#' + this.finalization_mail );
		var data = interaction.getFinalizationData();
		if ( data === null ) {
			$include.prop( 'checked', false );
			this.toggle_destination();
			return;
		}
		else if ( typeof data === "object" ) {
			$include.prop( 'checked', data.send );
			this.toggle_destination();
			$mail.val( data.mail_to || '' );
		}
		else {
			this.load_finalization_destination_address( function( mail_to ) {
				if ( mail_to ) {
					$include.prop( 'checked', true );
					self.toggle_destination();
					$mail.val( mail_to );
					self.set_finalization_data();
				}
			});
		}
	};

	FinalizationUI.prototype.show_message = function( message, type, timeout_ms )
	{
		type = type || 'Alert';
		timeout_ms = timeout_ms || 5000;

		this.client_window.displayStatusMessage(
			message,
			type,
			false,
			timeout_ms
		);
	};

	FinalizationUI.prototype.hide_message = function()
	{
		this.client_window.hideStatusMessage();
	};

	FinalizationUI.prototype.validate = function()
	{
		var self = this;

		if ( !$( '#' + this.finalization_include ).prop( 'checked' ) ) {
			return true;
		}

		var $finalization_mail = $( '#' + this.finalization_mail );
		var addresses = $finalization_mail.val();

		var mail_list = {
			to:  {
				addresses : addresses,
				get_error : function( result )
				{
					var message_error;
					var address = inConcert.Utils.truncateWord( result.address );
					switch( result.error_type )
					{
						case 'required_mail':
							message_error = "<%=_T('E-mail address is required.')%>";
							break;

						case 'invalid_mail':
							message_error = format( _T('Finalization e-mail must be a valid e-mail address: \'{0}\'.'), address );
							break;

						case 'invalid_mail_list':
							message_error = format( _T('Finalization e-mail list must be a valid: \'{0}\'. Must use the separator  \'; \'. Example: Contact Name <sample@sample.com>; ... OR sample@sample.com; ...'), address );
							break;

						default:
							throw( 'Invalid validation mail message type' );
					}
					return message_error;
				}
			}
		};

		var success = true;
		var validation_error = function( result ) {
			success = false;
			var email_type = mail_list[ result.type ];
			var message = email_type.get_error( result );
			self.show_message( message );
			$finalization_mail.focus();
		};

		if ( addresses.trim().length === 0 ) {
			validation_error({
				error_type : 'required_mail',
				type       : 'to',
				address    : addresses
			});
		}
		else {
			inConcert.Utils.validateEmails( mail_list, validation_error );
		}

		return success;
	};

	FinalizationUI.prototype.set_finalization_data = function()
	{
		var interaction = this.client_window.m_interaction;
		if ( !interaction ) {
			return;
		}

		var data = { send : false };
		if ( $( '#' + this.finalization_include ).prop( 'checked' ) ) {
			if ( this.validate() ) {
				var mail_to = $( '#' + this.finalization_mail ).val().trim();
				if ( mail_to.length > 0 ) {
					data.send = true;
					data.mail_to = mail_to;
				}
			}
		}

		interaction.setFinalizationData( data );
	};

	FinalizationUI.prototype.toggle_destination = function()
	{
		var $finalization_mail = $( '#' + this.finalization_mail );
		var is_checked = $( '#' + this.finalization_include ).prop( 'checked' );
		if ( is_checked ) {
			$finalization_mail.prop( 'disabled', false );
			$finalization_mail.focus();
		}
		else {
			this.hide_message();
			$finalization_mail.prop( 'disabled', true );
			$finalization_mail.val( '' );
		}
	};

	FinalizationUI.prototype.set_events = function()
	{
		var self = this;

		$( '#' + this.finalization_include )
			.off( 'click' )
			.on( 'click', function()
			{
				self.toggle_destination();
				var is_checked = $( this ).prop( 'checked' );
				if ( !is_checked ) {
					self.set_finalization_data();
				}
			});

		$( '#' + this.finalization_mail )
			.off( 'focusout' )
			.on( 'focusout', function() {
				self.set_finalization_data();
			});
	};
}());