/*jshint browser:true, quotmark:false, smarttabs:true*, esversion: 11/
/*global inConcert, localStorage*/

/**
Storage module, in this case uses adobe air local storage.

Just a key/value store
*/

// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.

if (!this.inConcert) {
	this.inConcert = {};
}


(function () {
	"use strict";

	function Storage() {}
	if (typeof inConcert.Storage !== 'function') {
		inConcert.Storage = new Storage();
	}

	Storage.prototype.is_supported_for_browser = function() {
		//Check for localStorage
		if (!typeof window.localStorage) {
			return false;
		}
		return true
	};

	//stores the key value in the local storage
	Storage.prototype.put = function(key, value) {
		window.localStorage.setItem(key, JSON.stringify(value));
	};

	//retrieves the value for key
	Storage.prototype.get = function(key) {
		var value = window.localStorage.getItem(key);
		try {
			return value ? JSON.parse(value) : null;
		}
		catch(e) {
			return undefined;
		}
	};

	//removes the key
	Storage.prototype.remove = function(key) {
		window.localStorage.removeItem(key);
	};
}());
