/*jslint nomen:false, regexp: false*/
/*jshint nomen:false, unused:true, undef: true, newcap:true, strict:false, smarttabs:true, esversion: 11 */
/*global $, inConcert */

/*
3rd party apps integration.
*/

// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.

/**
Docs

Setting up a parent-child sandbox relationship
http://help.adobe.com/en_US/AIR/1.5/devappshtml/WS5b3ccc516d4fbf351e63e3d118666ade46-7f11.html#WS5b3ccc516d4fbf351e63e3d118676a3fc7-7fea

Scripting between content in different domains
http://help.adobe.com/en_US/AIR/1.5/devappshtml/WS5b3ccc516d4fbf351e63e3d118666ade46-7e5c.html

*/

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	// Prevent middle mouse button clicks.
	// In Chrome, at least with 30.0.1599.101 m, starting a scroll with the middle button will "freeze".
	// Everything would still work except that you can't click anywhere. It seems like the mouseup event
	// got lost. This has something to do with iframes.
	// There are bug reports here:
	//
	//   https://code.google.com/p/chromium/issues/detail?id=308989
	//   https://code.google.com/p/chromium/issues/detail?id=269917
	//
	function preventMiddleButtonClick (element)
	{
		$(element).mousedown(function(e){
			/* 0=left, 1=middle, 2=right*/
			if(e.button == 1) {
				return false;
			}
		});
	}

	function getContextParameterValue(interaction, contact, scope, key, callback, paramName) {
		if (interaction.isAllegro()) {
			if (paramName) {
				let value = interaction.getPopupUrlAttribute(paramName);
				callback(value);
			}
			else {
				interaction.getBaragentAttribute(key, callback);
			}
		}
		else {
			let value = interaction.getReportValue(scope, key, true);
			callback(value);
		}
	}

	//it's assumed there are a lot of built-in parameters that the user can specify without inner knowledge, the back mapping is done here from name to value
	function getParameterValue(interaction, contact, parameterName, callback, paramName) {
		if (typeof(parameterName) === "string" && parameterName.indexOf(".") > 0) {
			const param_a = parameterName.split(".");
			const scope = param_a[0];
			const key = param_a[1];
			return getContextParameterValue(interaction, contact, scope, key, callback, paramName);
		}

		/*
			Esta lista (la del switch) debe coincidir con la lista de la configuracion CCI de la campana
			En el archivo:  \inconcert\apps\administration\scripts\adm_form_campaigns_section_integration_cci.js
			Funcion: cci_base_web_panel.prototype.get_system_interaction_parameters
		*/
		switch (parameterName) {
			//logged user information
			case "{USER}":
				return callback(inConcert.app.getLoggedUserId());
			case "{USER_NAME}":
				return callback(inConcert.app.getLoggedUserName());
			case "{USER_STATUS}":
				return callback(inConcert.app.getLoggedUserStatus());
			case "{USER_AUTH_TOKEN}":
				return callback(inConcert.app.getLoggedUserToken());
			//contact data
			case "{CONTACT_ID}":
				//for integration purposes contact id is not sent with @vcc
				var contactID = contact && contact.Id || interaction.contact;
				if(contactID.match(/^([\-a-zA-Z 0-9_.]+)(?:@(\w+))?/)) {
					contactID = RegExp.$1;
				}
				return callback(contactID);
			case "{CONTACT_NAME}":
				return callback(contact && contact.Name || interaction.contactAddress);
			case "{CONTACT_COMPANY}":
				return callback(contact && contact.Company || "");
			//interaction data
			case "{INTERACTIONID}":
				return callback(interaction.id);
			case "{VIRTUALCC}":
				return callback(inConcert.app.get_vcc_id());
			case "{TIMESTAMP}":
				return callback(interaction.timestamp);
			case "{ADDRESS}":
			case "{CALLERID}":
			case "{FROM}":
				return callback(interaction.contactAddress);
			//agent information
			case "{AGENTID}":
				return callback(inConcert.app.getLoggedUserId());
			//only valid if the interaction is a mail, assumed to refer the first mail
			case "{TO}":
				if (interaction.type === "MAIL") {
					return callback(interaction.getTo(0));
				}
				break;
			case "{SUBJECT}":
				if (interaction.type === "MAIL") {
					return callback(interaction.getSubject(0));
				}
				break;
			//ony valid for im and chat interactions
			case "{FIRST_MESSAGE}":
				if (interaction.type === "IM" || interaction.type === "CHAT") {
					return callback(interaction.interactionData.firstMessage);
				}
				break;
			default:
				return callback(parameterName);
		}
		callback(null);
	}

	/**
	*/
	function clone_interaction_subset (interaction)
	{
		var safe_keys = ["score", "VCC", "proxyPort", "attentionLevel", "contact", "classificationProperties",
						 "agentHistory", "startTimestampReadable", "priority", "steps", "contactAddress",
						 "campaignOffsetMinutes", "type", "interactionData", "proxyIP", "reservedAgents",
						 "requiredAgent", "timestampDouble", "id", "state", "startTimestamp", "campaign",
						 "assignedAgents", "initiative", "timestamp", "analysisTimes", "account", "data"];
		var new_interaction = {};

		safe_keys.forEach(function(key) {
			new_interaction[key] = interaction[key];
		});
		return new_interaction;
	}

	/**
	Walk the steps of an interaction backwards, checking for the 'unanswered' flag.
	Steps with this flag may be either QUEUED or WAITING.
	*/
	function look_for_unanswered_step (interaction)
	{
		for (var j = interaction.steps.length - 1; j >= 0; j--) {
			var step = interaction.steps[j];
			if (step.unanswered) {
				return true;
			}
			else if (step.action === "TAKEN") {
				break;
			}
		}
		return false;
	}

	/**
	* Checks if the interaction's ender field matches the given code.
	* @param ender_code must be a lowercase string
	*/
	function check_ender (interaction, ender_code)
	{
		var ender = interaction.interactionData.ender;
		return (typeof(ender) === "string" && ender.toLowerCase() === ender_code);
	}

	function setBridgeState(state) 
	{
		inConcert.logger.debug("SetBridgeState: '" + state + "'");
		this.m_isBridgeReady = state;
	}
	// --------------------------------------------------------------------


	/*--------------------------------------------------------------------
		WebApplication Class
	----------------------------------------------------------------------*/

	//Application abstract class, every registered application must derive from this class.
	function IntegrationApplication(options)
	{
		this.m_options = options;
		this.m_isBridgeReady = false;
	}

	//authenticates the current user against the bound application
	//trying to reuse the current credentials
	IntegrationApplication.prototype.login = $.noop;

	//virtual funcion, should open the specified contact file on the application
	IntegrationApplication.prototype.open = function(interaction, contact, callbackOnOpen = $.noop) {
		if (this.canOpen(interaction, contact)) {
			this.__open(interaction, contact, callbackOnOpen);
		}
	};
	
	IntegrationApplication.prototype.__open = function(/*interaction, contact, callbackOnOpen = $.noop*/) {};
	
	IntegrationApplication.prototype.canOpen = function(/*interaction, contact*/) { return true; };

	IntegrationApplication.prototype.getParameterValue = getParameterValue;

	IntegrationApplication.prototype.setBridgeState = setBridgeState;

	IntegrationApplication.prototype.isAllegro = function() {
		return inConcert.app.has_allegro_integration() && this.m_options && this.m_options.isAllegro || false;
	};
	// --------------------------------------------------------------------


	/*--------------------------------------------------------------------
		WebApplication Class
	----------------------------------------------------------------------*/

	/**
	Class WebApplication. Allows to integrate a third-party web application in a window that we control.

	A html page (integrationPanel.html) is loaded in our application domain. This html will have an iframe that will
	be used to load the application.
	Once everything is loaded, we perform a "bridge", this allowing code in the web app to call our API.
	The bridge can go both ways. The application can register functions to be called by us.
	The API that is exposed is located at 'integration_bridge.js'.
	*/
	function WebApplication (options)
	{
		WebApplication.uber.constructor.call(this, options);
		if(typeof options.popup !== "object") {
			throw("Missing mandatory 'popup' field");
		}
		this.m_triggered_events = {};
		this.m_queued_events = undefined; // an array that will be created later if needed

		// DOM IDs del DIV Container del iFrame y del elemento iFrame.
		this.m_dom_ids = {
			iframe_container: "cc_integration_view",
			iframe_sandbox  : "cc_integration_sandbox_iframe"
		};
	}

	inConcert.Utils.inherit(WebApplication, IntegrationApplication);

	WebApplication.prototype.getDOMIds = function ()
	{
		return this.m_dom_ids;
	};

	WebApplication.prototype.getiFrameDOMId = function ()
	{
		return this.m_dom_ids.iframe_sandbox;
	};

	WebApplication.prototype.isExternalIntegration = function ()
	{
		if (!this.m_options) {
			return false;
		}
		let integrationType = (this.m_options.type || "").toLowerCase();
		if (integrationType === "ctipopupweb") {
			return true;
		}
		if (integrationType === "helpdesk_integration") {
			return true;
		}
		return false;
	};

	/**
	Loads and sets the binding between the third-party application and our interaction and its optional contact.
	If a window is supplied, assume there is a div with the id 'integration' and set up the iframe as its child.
	Else, open a new popup window.
	@interaction : Interaction.
	@contact : Contact.
	@host_options : undefined / hash with @host_window and @close_handler NativeWindow (ie. the top-level window object)
	*/
	WebApplication.prototype.load = function (interaction, contact, host_options)
	{
		//
		// When the integration is not embedded, load integrationParent.html. When done, create an iframe to host the
		// third-party app. When it has loaded, "bridge" our page and the iframe, so the hosted app can invoke our API.
		// If embedded, "attach" to the hosting element.
		//
		const self = this;

		var onCompleteFn = function()
		{
			var node = self.host_window.document.getElementById(self.m_dom_ids.iframe_container);
			var iFrame =  self.host_window.document.getElementById(self.m_dom_ids.iframe_sandbox);

			console.debug("1 - [WebApplication.prototype.load - onCompleteFn] iframe_container: '" + self.m_dom_ids.iframe_container +
				"', iframe_sandbox: '" + self.m_dom_ids.iframe_sandbox +
				"', iFrame: " + iFrame
			);

			if (!iFrame) {
				iFrame = self.host_window.document.createElement("iframe");
				iFrame.id = self.m_dom_ids.iframe_sandbox;
				iFrame.src = "about:blank";
				iFrame.allowCrossDomainXHR = "true";
				iFrame.onload = "EngageBridge";
				iFrame.frameBorder = "0";
				iFrame.iframe_type = self.m_dom_ids.iframe_sandbox;

				// when embedded, we need to supply the dimensions. We can't rely on the host
				if(self.embedded) {
					iFrame.height = "220px";
					iFrame.width = "404px";
				}
				else {
					iFrame.height = $(self.host_window).height();
					iFrame.width = $(self.host_window).width();
				}

				iFrame.documentRoot = "app:";
				node.appendChild(iFrame);

				var reloadTimes = 0;
				var MAX_RELOAD_TIMES = 5;

				// 'ready' event won't do. we need 'load'
				const $iframe = $(iFrame);

				$iframe.on("load", function()
				{
					console.debug("2 - [WebApplication.prototype.load - on('load') ..] iframe_sandbox: " + self.m_dom_ids.iframe_sandbox);

					if (self.m_options.type === "ctipopup") {
						preventMiddleButtonClick(iFrame.contentWindow.document.body);
					}

					var on_update, on_automatic_wrapup_timeout, on_archived_by_user;

					var remove_events_listener = function(conversation) {
						conversation.removeListener("update", on_update);
						conversation.removeListener("automatic_wrapup_timeout", on_automatic_wrapup_timeout);
						conversation.removeListener("archived_by_user", on_archived_by_user);
					};

					on_automatic_wrapup_timeout = function() {
						var conversation = this;
						inConcert.logger.debug({Integration: "automatic wrapup timeout", id: conversation.id, state: conversation.state});
						remove_events_listener(conversation);
						self.triggerEvent("AutomaticWrapupTimeout", clone_interaction_subset(conversation));
					};
					// escucha el EventEmitter
					interaction.on("automatic_wrapup_timeout", on_automatic_wrapup_timeout);

					on_archived_by_user = function() {
						var conversation = this;
						inConcert.logger.debug({Integration: "user archive", id: conversation.id, user: inConcert.app.getLoggedUserId()});
						remove_events_listener(conversation);
					};
					// escucha el EventEmitter
					interaction.on("archived_by_user", on_archived_by_user);

					on_update = function()
					{
						var conversation = this;

						inConcert.logger.debug({Integration: "state change", id: conversation.id, state: conversation.state});

						// 'this' is the interaction
						// it goes back to the queue or to waiting (in case of circular campaign)
						if(conversation.isQueued() || conversation.state === "WAITING") {
							remove_events_listener(conversation);
							if(look_for_unanswered_step(conversation)) {
								self.triggerEvent("Unanswered", clone_interaction_subset(conversation));
							}
							self.close();
						}
						// Was it closed due to inactivity?
						else if(conversation.state === "WRAPUP" && check_ender(conversation, "inactivity")) {
							self.triggerEvent("AutomaticInactivityTimeout", clone_interaction_subset(conversation));
						}
						else if(conversation.state === "WRAPUP") {
							self.triggerEvent("Wrapup", clone_interaction_subset(conversation));
						} 
						else if(conversation.state === "TAKEN") {
							self.triggerEvent("Taken", clone_interaction_subset(conversation));
						} 
						else if(conversation.state === "TRANSFER TAKEN") {
							self.triggerEvent("Transfer", clone_interaction_subset(conversation));
						} 
						else if(conversation.state === "CONFERENCE TAKEN") {
							// another agent now has the interaction
							self.triggerEvent("ConferenceConsultation", clone_interaction_subset(conversation));
						} 
						else if(conversation.state === "CONFERENCE") {
							self.triggerEvent("Conference", clone_interaction_subset(conversation));
						}
						else if(conversation.state === "TRANSFERRING") {
							// #7100 (IE-Fix) Aca se debe dejar de escuchar los eventos, ya que sino IE falla runtime.
							remove_events_listener(conversation);
						}
						else if(conversation.state === "FINISHED") {
							if (inConcert.app.isInRedial()) {
								return;
							}
							remove_events_listener(conversation);
							self.close();
						}
					};
					// escucha el EventEmitter
					interaction.on("update", on_update);

					// we need to manually engage the bridge
					self.host_window.EngageBridge(self, function(bridge) {
						console.debug("3 - [WebApplication.prototype.load - callback EngageBridge ..] iframe_sandbox: " + self.m_dom_ids.iframe_sandbox);

						(host_options.on_load || $.noop)();

						var bridgeOk = true;

						// bridge is an instance of inConcert.IntegrationBridge
						self.child_bridge = bridge.m_child_bridge;

						const is_external_integration = self.isExternalIntegration();
						if (!is_external_integration && (self.child_bridge && typeof(self.child_bridge.trigger) !== "function")) {
							bridgeOk = false;
						}

						if (!bridgeOk) {
							//something went wrong with the bridge
							if (reloadTimes === MAX_RELOAD_TIMES) {
								inConcert.logger.error("Error loading bridge. Max retry limit reached. Form src = " + iFrame.src);
								return;
							}
							//force reload
							reloadTimes++;
							iFrame.src = iFrame.src;
							return;
						}

						// If there are any queued events, fire them (outside of this callback)
						// After that, trigger an 'update' event so we can handle possible state changes
						// while we were loading.
						if((self.child_bridge || self.m_isBridgeReady) && typeof(self.m_queued_events) !== "undefined") {
							setTimeout(function() {
								inConcert.logger.debug("Bridge is ready, flushing event queue");
								self.m_queued_events.forEach(function(evt) {
									self.triggerEvent(evt.event, evt.data);
								});
								delete self.m_queued_events;

								interaction.emit("update");
							}, 10);
						} else if ((self.child_bridge && !self.child_bridge.IsReady()) || (!self.child_bridge && !self.m_isBridgeReady)) {
							inConcert.logger.debug("Bridge not ready yet, create timer event queue");
							var queued_events_timer  = setInterval(function() {
								if ((self.child_bridge && self.child_bridge.IsReady()) || (!self.child_bridge && self.m_isBridgeReady)) {
									inConcert.logger.debug("Bridge is ready now, cancel timer event queue");
									clearInterval(queued_events_timer);

									if(typeof(self.m_queued_events) !== "undefined") {
										inConcert.logger.debug("Bridge is ready, flushing event queue");
										self.m_queued_events.forEach(function(evt) {
											self.triggerEvent(evt.event, evt.data);
										});
										delete self.m_queued_events;
										interaction.emit("update");
									}
								}
							}, 100);
							interaction.emit("update");
						} else { 
							interaction.emit("update");
							if(interaction.state === "FINISHED") {
								self.close();
							}
						}
					});
				});
			}
			// Abro el formulario/popup
			if( !interaction.isInflated() ) {
				interaction.inflate(function() {
					self.open(interaction, contact, function(success, err) {
						(host_options.on_ready || $.noop)(success, err);
					});
				});
			} else {
				self.open(interaction, contact, function(success, err) {
					(host_options.on_ready || $.noop)(success, err);
				});
			}
		};

		// If a native window was specified, embed the CCI in there. Else, open a new window.
		if(host_options) {
			if(!host_options.host_window) {
				throw ("Missing window in WebApplication.load host_options");
			}
			self.host_window = host_options.host_window;
			self.close_handler = host_options.close_handler;
			self.embedded = true;
			onCompleteFn();
		}
		else {
			var windowSize = { width: 600 , height: 600 };
			var windowOptions = {
				page: "integrationParent",
				size: windowSize,
				visible: true,
				//native: true,
				scrollBarsVisible: false,
				onComplete: onCompleteFn
			};
			const native_prop = "native"; // se agrega asi, para que jshint no de warning!!
			windowOptions[native_prop] = true; // se agrega asi, para que el jcompressor no falle, ya que esto falla windowOptions.native
			var host_window = inConcert.app.getWindowManager().createWindow(windowOptions);
			self.host_window = host_window.getNativeWindow();
			self.close_handler = function() {
				self.host_window.close();
			};
			self.embedded = false;
		}
	};

	/**
	Opens the third party application. Looks up the 'popup' field in our options, resolves the list of parameters
	to be sent and performs the appropiate verb (GET or POST).
	*/
	/*
		popup.url = "http://www.worldofspectrum.org/infoseek.cgi";
		popup.method = "POST";
		popup.parameters =
		{
			regexp: "headoverheels",
			model: "any",
			loadpics: 3
		};
	*/

	WebApplication.prototype.__open = function(interaction, contact, callbackOnOpen = $.noop)
	{
		const self = this;
		const popup = this.m_options.popup;

		inConcert.Utils.asyncReduce(popup.parameters,
			function(parameterList, param, callback, key) {
				getParameterValue(interaction, contact, param, function(value) {
					parameterList[key] = value;
					callback(parameterList);
				}, key);
			},
			function(parameterList) {
				self.__makeURI(interaction, popup, function(success, url) {
					if (!success) {
						callbackOnOpen(false);
						return;
					}
					if (popup.method === "POST") {
						self.postTo(url, parameterList);
					}
					else {
						self.navigateTo(url, parameterList);
					}
					callbackOnOpen(true);
				});
			},
			{}
		);
	};

	WebApplication.prototype.__makeURI = function(interaction, popup_info, callback = $.noop)
	{
		let uri;
		if (!popup_info.url_parts) {
			uri = popup_info.url;
		}
		else {
			const tweety_uri = inConcert.app.getTweetyDesignerUrl();
			uri = tweety_uri + "/" + popup_info.url_parts.appName;
		}
		callback(true, uri);
	};

	WebApplication.prototype.getHostIframe = function()
	{
		// TODO: improve checks
		const iframe = this.host_window.document.getElementById(this.m_dom_ids.iframe_sandbox);
		return iframe;
	};

	WebApplication.prototype.postTo = function(url, parameters)
	{
		var parent = this.getHostIframe();
		var form = parent.contentWindow.document.createElement("form");
		form.setAttribute("method", "post");
		form.setAttribute("action", url);

		for (var p in parameters) {
			if (parameters.hasOwnProperty(p)) {
				var hiddenField = parent.contentWindow.document.createElement("input");
				hiddenField.setAttribute("name", p);
				hiddenField.setAttribute("value", parameters[p]);
				form.appendChild(hiddenField);
			}
		}
		form.submit();
	};

	WebApplication.prototype.navigateTo = function(url, parameters) {
		let paramUrl = "";
		for (let paramName in parameters) {
			if (parameters.hasOwnProperty(paramName)) {
				paramUrl += paramUrl.length > 0 ? "&" : "";
				let paramValue = parameters[paramName];
				paramUrl += paramName + "=" + encodeURIComponent(paramValue);
			}
		}

		if (paramUrl.length > 0) {
			let paramSeparator = "";
			const separatorPos = url.indexOf("?");
			if (separatorPos === -1) {
				paramSeparator = "?";
			}
			else {
				paramSeparator = "&";
			}
			url += paramSeparator + paramUrl;
		}

//		url += paramUrl.length > 0 ? ("?" + paramUrl) : "";

		let iframe = this.getHostIframe();
		iframe.src = url;
	};

	// Send a message to the child iframe
	WebApplication.prototype.sendMessage = function(msg) {
		// Make sure you are sending a string, and to stringify JSON
		var iframe = this.getHostIframe();
		iframe.contentWindow.postMessage(msg, '*');
	};

	/**
	Triggers en event in the integrated application.
	@param event : String is the name of event to be triggered.
	@data : Any is an optional parameter to the event. Due to bridge restrictions, data will be encoded as json.
	*/
	WebApplication.prototype.triggerEvent = function(event, data)
	{
		var encoded_data;
		var bridgeOk = true;

		console.log("WebApplication.prototype.triggerEvent. event: '" + event + "', data: '" + data + "', JSON: " + JSON + "'");

		const is_external_integration = this.isExternalIntegration();
		if (is_external_integration && !this.m_isBridgeReady) {
			bridgeOk = false;

		} else if (!is_external_integration && (typeof(this.child_bridge) === "undefined" || !this.child_bridge.IsReady())) {
			bridgeOk = false;
		}

		if (!bridgeOk) {
			inConcert.logger.info("Queueing event '" + event + "' until the bridge is ready");
			if(typeof(this.m_queued_events) === "undefined") {
				this.m_queued_events = [];
			}
			this.m_queued_events.push({event: event, data: data});
			return;
		}

		// avoid triggering events multiple times
		if(this.m_triggered_events[event]) {
			return;
		}
		this.m_triggered_events[event] = true;

		if(data) {
			encoded_data = JSON.stringify(data);
		}
		try {
			const is_external_integration = this.isExternalIntegration();
			if (!is_external_integration) {
				this.child_bridge.trigger(event, encoded_data);
			} else {
				var message = {
					uniqueId: 0,
					messageId: event,
					result: encoded_data
				};
				this.sendMessage(message);
			}
		}
		catch(e) {
			inConcert.logger.error(e);
		}
	};

	WebApplication.prototype.close = function ()
	{
		this.close_handler();
	};

	WebApplication.prototype.resizeTo = function (width, height)
	{
		// for the time being, we don't resize when embedded
		if(!this.embedded) {
			this.host_window.resizeTo(width, height);
		}
	};
	// --------------------------------------------------------------------


	/*--------------------------------------------------------------------
		CtiPopup Class
	----------------------------------------------------------------------*/

	function CtiPopup (options)
	{
		// For CTI integration, always use the protocol (http or https) the agent is using,
		// no matter what the configured url says. (this is because the agent is using https,
		// the cti integration will fail to load if its url is http)
		//
		if(typeof(options.popup.url) == "string") {
			options.popup.url = window.location.protocol + "//" + options.popup.url.match("^http[s]?://(.+)")[1];
		}

		CtiPopup.uber.constructor.call(this, options);

		if(typeof(this.m_options.popup.parameters) !== "object") {
			this.m_options.popup.parameters = {};
		}
		this.m_options.popup.parameters.__cti = true;
		this.m_options.popup.parameters.__ctiembeed = true;
		this.m_options.popup.parameters.__allegro = inConcert.app.has_allegro_integration();
	}

	inConcert.Utils.inherit(CtiPopup, WebApplication);

	CtiPopup.prototype.canOpen = function(/*interaction, contact*/) {
		if (this.isAllegro()) {
			return !!inConcert.app.getBaragentPolling().getPopupInfo();
		}
		return true;
	};
	// --------------------------------------------------------------------


	/*--------------------------------------------------------------------
		AllegroApplication Class
	----------------------------------------------------------------------*/

	function AllegroApplication (options)
	{
		AllegroApplication.uber.constructor.call(this, options);
	}

	inConcert.Utils.inherit(AllegroApplication, WebApplication);
	
	AllegroApplication.prototype.canOpen = function(/*interaction, contact*/) {
		return !!inConcert.app.getBaragentPolling().getPopupInfo();
	};

	AllegroApplication.prototype.__open = function(interaction, contact, callbackOnOpen = $.noop)
	{
		const self = this;

		if ($("#" + this.m_dom_ids.iframe_sandbox).attr("src") !== "about:blank") {
			inConcert.logger.debug("Popup already open");
			return;
		}

		var makeClientSideUrlParametersValues = function(endCallback) {
			if (!self.m_options) {
				endCallback();
				return;
			}
			if (!self.m_options.popup || !self.m_options.popup) {
				endCallback();
				return;
			}
			const popup = self.m_options.popup;

			inConcert.Utils.asyncReduce(popup.parameters,
				function(parameterList, param, callback, key) {
					getParameterValue(interaction, contact, param, function(value) {
						parameterList[param] = {key: key, value: value};
						callback(parameterList);
					}, key);
				},
				function(parameterList) {
					endCallback(parameterList);
				},
				{}
			);
		};

		var navigateTo = function(url) {
			/*
				Ahora se realiza el replace de los parametros client side.
			*/
			makeClientSideUrlParametersValues(function(parameterList) {
				if (parameterList) {
					for(let name in parameterList) {
						if (parameterList.hasOwnProperty(name)) {
							let info = parameterList[name];
							if (typeof info.value === "string") {
								let regExp = new RegExp(name, "ig");
								let encodedValue = encodeURIComponent(info.value);
								url = url.replace(regExp, encodedValue);
							}
						}
					}
				}
				self.navigateTo(url, {});
				callbackOnOpen(true);
			});
		};

		const allegro_popup = inConcert.app.getBaragentPolling().getPopupInfo();
		switch(allegro_popup.type) {
			case "TWEETY":
				inConcert.app.getAllegroAPI().loginAgentFromToken(function(token, error) {
					if (!token) {
						inConcert.logger.error("Error retrieving and validating allegro auth token: " + error);
						callbackOnOpen(false);
						return;
					}
					let url = inConcert.app.redirectAllegroWebUrl(allegro_popup.url) + "&omnichannel=true&SessionId=" + token;
					navigateTo(url);
				});
				break;
			case "WEB_GET":
				navigateTo(allegro_popup.url);
				break;
			case "WEB_POST":
				var params = {};
				var body = allegro_popup.body.split("&");
				$.each(body, function(_, param) {
					var param_a = param.split("=");
					if (param_a.length === 2) {
						params[param_a[0]] = param_a[1];
					}
				});
				this.postTo(allegro_popup.url, params);
				callbackOnOpen(true);
				break;
			default:
				inConcert.logger.error("Unsupported allegro cti type " + allegro_popup.type);
				callbackOnOpen(false);
				break;
		}
	};
	// --------------------------------------------------------------------


	/*--------------------------------------------------------------------
		CtiAllegroApplication Class
	----------------------------------------------------------------------*/

	function CtiAllegroApplication (options)
	{
		CtiAllegroApplication.uber.constructor.call(this, options);

		if(typeof(this.m_options.popup.parameters) !== "object") {
			this.m_options.popup.parameters = {};
		}
		this.m_options.popup.parameters.__cti = true;
		this.m_options.popup.parameters.__ctiembeed = true;
	}

	inConcert.Utils.inherit(CtiAllegroApplication, AllegroApplication);
	// --------------------------------------------------------------------


	/*--------------------------------------------------------------------
		HelpDeskApplication Class
	----------------------------------------------------------------------*/

	function HelpDeskApplication (options)
	{
		HelpDeskApplication.uber.constructor.call(this, options);

		// DOM IDs del DIV Container del iFrame y del elemento iFrame.
		this.m_dom_ids = {
			iframe_container: "helpdesk_integration_view",
			iframe_sandbox  : "helpdesk_integration_sandbox_iframe"
		};
	}

	inConcert.Utils.inherit(HelpDeskApplication, WebApplication);

	/*
		Sample uri:
			https://168.62.161.232:2443/helpdesk/addiccticket.aspx?logged_user=adminfull@vcc1&token=3B57D6EAD1BC2086F6E6237DF003021C
			https://168.62.161.232:2443/ichelpdeskdesarrollo/addiccticket.aspx?logged_user=adminfull@vcc1&token=3B57D6EAD1BC2086F6E6237DF003021C

		Estos parametros se agregan segun la config del popup
			&VIRTUALCC=vcc1&INTERACTIONID=da6d80d4bd7e45c3a974708b755983cc&AGENTID=adminfull&interaction_id=da6d80d4bd7e45c3a974708b755983cc

		Para pruebas con el test occ Ejemplo Usar ese codigo en la funcion
			let uri = "https://testubu.i6.inconcertcc.com/inconcert/lib/pruebas/sdk/testoccsdk1.html";
			callback(true, uri);
		*/
	HelpDeskApplication.prototype.__makeURI = function(interaction, popup_info, callback = $.noop)
	{
		//  let uri = "https://testubu.i6.inconcertcc.com/inconcert/lib/pruebas/sdk/testoccsdk1.html";
		//  callback(true, uri);

		const helpdesk_api = inConcert.app.getHelpDeskAPI();
		helpdesk_api.getAddTicketsURI(function(success, helpdesk_info) {
			if (!success) {
				let err = helpdesk_info;
				console.debug("HelpDeskApplication.prototype.__makeURI . Error retrieving URI - " + JSON.stringify(err));
				callback(false, err);
				return;
			}
			let helpdesk_app_uri = helpdesk_info.uri;
			callback(true, helpdesk_app_uri);
		});
	};

	//----------------------------------------------------------------------

	/*--------------------------------------------------------------------
		IntegrationManager Class
	----------------------------------------------------------------------*/

	/**
	Class IntegrationManager. Links and knows how to create any 3rd party app binding
	*/
	function IntegrationManager() {}

	if(typeof inConcert.IntegrationManager !== "object") {
		//integration manager is a singleton so it's published already initialized
		//(everyone should see the same registered application list)
		inConcert.IntegrationManager = new IntegrationManager();
	}

	/**
	For a given application name, instantiate the corresponding handler class.
	@param type a string (one of CTIPOPUP, etc)
	@param options an object with stuff (TODO: emprolijar esto)
	options needs to have a 'popup' field i
	*/
	IntegrationManager.prototype.newAppIntegration = function(type, options) {
		var application;
		if (typeof(type) === "undefined") {
			return;
		}
		type = type.toLowerCase();
		options.type = type;
		switch (type) {
			case "salesforce":
				break;
			case "sugarcrm":
				break;
			case "zendesk":
				break;
			case "fogbugz":
				break;
			case "ctipopup":
				application = new CtiPopup(options); /* Tweety app */
				break;
			case "ctiallegropopup":
				application = new CtiAllegroApplication(options); /* Tweety App from allegro */
				break;
			case "ctipopupweb":
				if (options.isAllegro) {
					application = new AllegroApplication(options); /* App from allegro */
				}
				else {
					application = new WebApplication(options); /* Regular web app */
				}
				break;
		/*	--------------------------
			Integraciones con Apps Externas (HELPDESK, etc)
		------------------------------*/
			case "helpdesk_integration":
				options.type = "helpdesk_integration";
				application = new HelpDeskApplication(options); /* HelpDesk app */
				break;

		/*	--------------------------
			El por defaul !!, NO deberia pasar nunca por aca !!!
		------------------------------*/
			default:
				// TODO: options should have a 'popup' field or else it will break
				// add a check for this
				options.type = "ctipopupweb";
				application = new WebApplication(options);
			break;
		}

		return application;
	};
}());
