/*jshint bitwise:true, browser:true, curly:true, eqeqeq:true, noarg:true, noempty:true, strict:true, undef:true, unused:true, esversion: 11 */
/*jshint quotmark:false*/


if (!window.inConcert) {
	window.inConcert = {};
}

/**
Please remember that all objects that crosses the bridge cannot be enumerated. You either pass them encoded as JSON or
extract the desired properties on the receiving side.
*/

(function () {

	"use strict";
/**
Some helpers
*/

/**
*/
function clone_interaction_subset (interaction) {
	var safe_keys = ["score", "VCC", "proxyPort", "attentionLevel", "contact", "classificationProperties",
					 "agentHistory", "startTimestampReadable", "priority", "steps", "contactAddress",
					 "campaignOffsetMinutes", "type", "interactionData", "proxyIP", "reservedAgents",
					 "requiredAgent", "timestampDouble", "id", "state", "startTimestamp", "campaign",
					 "assignedAgents", "initiative", "timestamp", "analysisTimes", "account", "data"];
	var new_interaction = {};

	safe_keys.forEach(function(key) {
		new_interaction[key] = interaction[key];
	});
	return new_interaction;
}

//
// Wrapper function that checks that the given interaction_id correponds to an existing interaction before
// calling the supplied function.
var GetInteraction = function(f) {
	// Return a function whose first argument is always an interaction id and its last argument a callback.
	return function(interaction_id/*, ..., callback*/)
	{
		var callback = arguments[arguments.length - 1];
		if(typeof(callback) !== "function") {
			throw new Error("A callback function must be supplied");
		}

		if(typeof(interaction_id) !== "string") {
			throw new Error("A valid interaction id must be supplied");
		}

		var interactions = this.m_application.getInteractionManager().getInteractions();
		var interaction = interactions[interaction_id];
		if(!interaction) {
			callback(false, "Interaction not found");
		}
		else {
			// Replace interaction_id by interaction and call the wrapped function.
			var args = Array.prototype.slice.call(arguments);
			args[0] = interaction;
			f.apply(this, args);
		}
	};
};


/**
BindEvent.
@param element : window
@param eventName : message
@eventHandler: function to process events
*/
let m_isAlreadyBindedEvent = false;
function BindEvent(element, eventName, eventHandler) {
	if (m_isAlreadyBindedEvent) {
		return;
	}
	if (element.addEventListener) {
		element.addEventListener(eventName, eventHandler, false);
	} else if (element.attachEvent) {
		element.attachEvent('on' + eventName, eventHandler);
	}
	m_isAlreadyBindedEvent = true;
}

// Send a message to the child iframe
function sendMessage(msg, sandboxId) {
	// Make sure you are sending a string, and to stringify JSON
	var content_window = window.document.getElementById(sandboxId).contentWindow;
	content_window.postMessage(msg, '*');
}

function UnifyCriteriaActionResponse(messageId, result) {
	switch (messageId) {
		case "response_interaction_get":
			//return {type: "object", data: clone_interaction_subset(result)};
			return clone_interaction_subset(result);

		//true-false
		case "response_interaction_archive":
		case "response_interaction_comment":
			//return {type: "boolean", data : result};
			return result.toString();

		//array
		case "response_interaction_getmessages":
		case "response_interaction_geteventlist":
		//	return {type: "array", data : result};

		default:
			return result;

	}
}

function ProcessActionMessage (msg, sandboxId, integrationMessageBridge) {
	var message = {
		uniqueId: 0,
		messageId: "",
		Post: function(result) {
			var message = {};
			message.uniqueId = this.uniqueId;
			message.messageId = this.messageId;
			message.result = UnifyCriteriaActionResponse(this.messageId, result);
			sendMessage(message, sandboxId);
		}
	};
	var uMessage = message.Post;
	var callbackMessage = uMessage.bind(message);

	var action = msg.data.action || "";
	var parameters = msg.data.parameters || [];
	action = action.toLowerCase();
		switch (action) {

			case "bridgeready":
				integrationMessageBridge.SetBridgeState(true);
				message.uniqueId = msg.uniqueId || 0;
				message.messageId = "response_" + action;
				callbackMessage("true");
				break;

			//interaction
			case "takenextbest":
				message.uniqueId = msg.uniqueId || 0;
				message.messageId = "response_" + action;
				integrationMessageBridge.TakeNextBest(callbackMessage);
				break;

			case "interaction_requeue":
				message.uniqueId = msg.uniqueId || 0;
				message.messageId = "response_" + action;
				integrationMessageBridge.Interaction_Requeue(parameters.interactionId, callbackMessage);
				break;

			case "interaction_comment":
				message.uniqueId = msg.uniqueId || 0;
				message.messageId = "response_" + action;
				integrationMessageBridge.Interaction_Comment(parameters.interactionId, parameters.comment, callbackMessage);
				break;

			case "interaction_get":
				message.uniqueId = msg.uniqueId || 0;
				message.messageId = "response_" + action;
				integrationMessageBridge.Interaction_Get(parameters.interactionId, callbackMessage);
				break;

			case "interaction_archive":
				message.uniqueId = msg.uniqueId || 0;
				message.messageId = "response_" + action;
				integrationMessageBridge.Interaction_Archive(parameters.interactionId, parameters.disposition_param, parameters.comment, callbackMessage);
				break;

			case "interaction_getparameter":
				message.uniqueId = msg.uniqueId || 0;
				message.messageId = "response_" + action;
				integrationMessageBridge.Interaction_GetParameter(parameters.interactionId, parameters.parameter, callbackMessage);
				break;

			case "interaction_getdispositioncodes":
				message.uniqueId = msg.uniqueId || 0;
				message.messageId = "response_" + action;
				integrationMessageBridge.Interaction_GetDispositionCodes(parameters.interactionId, callbackMessage);
				break;

			//user info
			case "user_getcompanyinformation":
				message.uniqueId = msg.uniqueId || 0;
				message.messageId = "response_" + action;
				callbackMessage(integrationMessageBridge.User_GetCompanyInformation());
				break;

			case "user_getcustominformation":
				message.uniqueId = msg.uniqueId || 0;
				message.messageId = "response_" + action;
				callbackMessage(integrationMessageBridge.User_GetCustomInformation());
				break;

			case "user_getcompletename":
				message.uniqueId = msg.uniqueId || 0;
				message.messageId = "response_" + action;
				callbackMessage(integrationMessageBridge.User_GetCompleteName());
				break;

			case "user_getfirstname":
				message.uniqueId = msg.uniqueId || 0;
				message.messageId = "response_" + action;
				callbackMessage(integrationMessageBridge.User_GetFirstName());
				break;

			case "user_getlastname":
				message.uniqueId = msg.uniqueId || 0;
				message.messageId = "response_" + action;
				callbackMessage(integrationMessageBridge.User_GetLastName());
				break;

			case "user_getfirstsurname":
				message.uniqueId = msg.uniqueId || 0;
				message.messageId = "response_" + action;
				callbackMessage(integrationMessageBridge.User_GetFirstSurname());
				break;

			case "user_getlastsurname":
				message.uniqueId = msg.uniqueId || 0;
				message.messageId = "response_" + action;
				callbackMessage(integrationMessageBridge.User_GetLastSurname());
				break;

			case "user_getname":
				message.uniqueId = msg.uniqueId || 0;
				message.messageId = "response_" + action;
				callbackMessage(integrationMessageBridge.User_GetName());
				break;

			//chat
			case "interaction_getmessages":
				message.uniqueId = msg.uniqueId || 0;
				message.messageId = "response_" + action;
				integrationMessageBridge.Interaction_GetMessages(parameters.interactionId, callbackMessage);
				break;

			//facebook
			//twitter
			case "interaction_geteventlist":
				message.uniqueId = msg.uniqueId || 0;
				message.messageId = "response_" + action;
				integrationMessageBridge.Interaction_GetEventList(parameters.interactionId, callbackMessage);
				break;

			case "window_resizeto":
				integrationMessageBridge.Window_ResizeTo(parameters.width, parameters.height);
				break;

			case "window_close":
				integrationMessageBridge.Window_Close();
				break;

			default:
				break;
		}
}

/**
Bridge constructor.
@param webapplication : inConcert.WebApplication
@param application : inConcert.Application
*/
function Bridge (webapplication, application)
{
	this.m_application = application;
	this.m_webapplication = webapplication;
	let sandboxId = this.m_webapplication.getiFrameDOMId();


	if (this.m_webapplication.m_options.type === "ctipopup") {
	// WEBDESIGNER: Cuando la comunicacion es con el WebDesigner
	//
		var content_window = window.document.getElementById(sandboxId).contentWindow;
		content_window.parentSandboxBridge = this;

		if(typeof(content_window.childSandboxBridge) === "object") {
			var child_engage_fn = content_window.childSandboxBridge.EngageBridge;
			if(typeof(child_engage_fn) === "function") {
				child_engage_fn();
			}
			this.m_child_bridge = content_window.childSandboxBridge;
		}
	}
	else {
		const is_external_integration = this.m_webapplication.isExternalIntegration();
		if (is_external_integration) {
		// URL Externa: Cuando la comunicacion NO es con el WebDesigner, (Ej con HelpDesk o terceros)
		//
			let integrationMessageBridge = this;
			// Listen to message from child window
			BindEvent(window, 'message', function(msg) {
				return ProcessActionMessage(msg, sandboxId, integrationMessageBridge);
			});
		}
		else {
			console.debug("function Bridge -> this.m_webapplication.m_options.type: " + this.m_webapplication.m_options.type);
		}
	}
}

window.inConcert.IntegrationBridge = Bridge;

Bridge.prototype.SetBridgeState = function(state)
{
	this.m_webapplication.setBridgeState(state);
};

Bridge.prototype.TakeNextBest = function(callback)
{
	this.m_application.getInteractionManager().takeNextBest(undefined, undefined, function(response) {
		callback(response);
	});
};

Bridge.prototype.Interaction_Requeue = GetInteraction(function(interaction, callback)
{
	interaction.requeue(callback);
});

Bridge.prototype.Interaction_Comment = GetInteraction(function(interaction, comment, callback)
{
	interaction.comment(comment, callback);
});

Bridge.prototype.Interaction_Get = GetInteraction(function(interaction, callback)
{
	callback(interaction);
});

Bridge.prototype.Interaction_Archive = GetInteraction(function(interaction, disposition_param, comment, callback)
{
	if(typeof(disposition_param) === "undefined") {
		interaction.archive(undefined, comment, callback);
		return;
	}

	var disposition;
	var dispositionTreePath;
	var dialer = interaction.interactionData && interaction.interactionData.dialer || [];	
	dialer.contactName = interaction.interactionData && interaction.interactionData.contactName || "";	

	if(typeof(disposition_param) === "object") {
		if (!disposition_param.treePath) {
			dispositionTreePath = [disposition_param.name];
		} else {
			dispositionTreePath = disposition_param.treePath;
		}
		if (disposition_param.contactCode) {
			dialer.contactCode = disposition_param.contactCode;
		}
		disposition = {
			name: disposition_param.name,
			treePath : dispositionTreePath,
			rescheduleDate: disposition_param.rescheduleDate,
			rescheduleTo: disposition_param.rescheduleTo,
			alternativeNumber: disposition_param.alternativeNumber,
			dialer : dialer
		};
	}
	else {
		dispositionTreePath = [disposition_param];
		disposition = { 
			name: disposition_param,
			treePath : dispositionTreePath, 
			dialer : dialer
		};
	}
	interaction.archive(disposition, comment, callback);
});

Bridge.prototype.Interaction_GetParameter = GetInteraction(function(interaction, parameter, callback)
{
	this.m_webapplication.getParameterValue(interaction, undefined, parameter, function(value) {
		callback(value);
	});
});

Bridge.prototype.Interaction_GetDispositionCodes = GetInteraction(function(interaction, callback)
{
	var campaign = this.m_application.getCampaign(interaction.getCampaign());
	var codes = campaign.getDispositionCodes();
	callback(JSON.stringify(codes));
});


/** 
User info
*/

Bridge.prototype.User_GetCompanyInformation = function(/*callback*/)
{
	return this.m_application.getLoggedUserCompanyInformation();
};

Bridge.prototype.User_GetCustomInformation = function(/*callback*/)
{
	return this.m_application.getLoggedUserCustomInformation();
};

Bridge.prototype.User_GetCompleteName = function(/*callback*/)
{
	return this.m_application.getLoggedUserCompleteName();
};

Bridge.prototype.User_GetFirstName = function(/*callback*/)
{	
	return this.m_application.getLoggedUserFirstName();
};

Bridge.prototype.User_GetLastName = function(/*callback*/)
{
	return this.m_application.getLoggedUserLastName();
};

Bridge.prototype.User_GetFirstSurname = function(/*callback*/)
{
	return this.m_application.getLoggedUserFirstSurname();
};

Bridge.prototype.User_GetLastSurname = function(/*callback*/)
{
	return this.m_application.getLoggedUserLastSurname();
};

Bridge.prototype.User_GetName = function(/*callback*/)
{
	return this.m_application.getLoggedUserName();
};

/**
Returns an array with the method names that the bridge supports.
Shit, this is like COM !
*/
Bridge.prototype.GetMethodNames = function(/*callback*/)
{
	var value = [];
	for(var name in Bridge.prototype) {
		if(Bridge.prototype.hasOwnProperty(name) && name !== "GetMethodNames" && name !== "GetEventNames")
		{
			value.push(name);
		}
	}
	return value;
};

/**
Same as GetMethodNames, but this is an array of events that can be triggered on the Tweety popup.
*/
Bridge.prototype.GetEventNames = function(/*callback*/)
{
	var value = [ "Taken", "Wrapup", "Unanswered", "AutomaticWrapupTimeout", "Transfer", "AutomaticInactivityTimeout" ];
	return value;
};


/**
Chat
*/
Bridge.prototype.Interaction_GetMessages = GetInteraction(function(interaction, callback)
{
	var session = interaction.getSession();
	var messages = session.getMessages() || [];
	callback(messages);
});


/**
facebook
twitter
*/
Bridge.prototype.Interaction_GetEventList = GetInteraction(function(interaction, callback)
{
	var eventList = interaction.getEventList() || [];
	callback(eventList);
});


/**
Window
*/
Bridge.prototype.Window_Close = function()
{
	this.m_webapplication.close();
};

Bridge.prototype.Window_ResizeTo = function(width, height)
{
	this.m_webapplication.resizeTo(width, height);
};

})();