
/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 11 */

/* global inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}
if (!inConcert.interactions) {
	inConcert.interactions = {};
}

/**
 * @namespace
 * @name TelephonyInteraction
 */
(function () {

	"use strict";
	var format = inConcert.Utils.format;

	/**
	 * Class TelephonyInteraction.
	 * Subclass for call conversations.
	 *
	 * @param {Object} rawInteraction
	 */
	function TelephonyInteraction (rawInteraction)
	{
		TelephonyInteraction.uber.constructor.call(this, rawInteraction);
		this.m_number = null;
		this.read = true;
		$.extend(true, this.m_actions, {
			REDIAL          : { interaction : "int_redial"                , supervisor : null                        } ,
			RECORDING       : { interaction : "int_startRecording"        , supervisor : null                        } ,
			HOLD            : { interaction : "int_hold"                  , supervisor : null                        } ,
			SEND_DIGITS     : { interaction : "int_sendDigits"            , supervisor : null                        },
			CONFERENCE      : { interaction : "int_conference"            , supervisor : null                        },
			LISTEN_RECORD   : { interaction : "int_listen_call_records"   , supervisor : "sup_listen_call_records"   },
			DOWNLOAD_RECORD : { interaction : "int_download_call_records" , supervisor : "sup_download_call_records" },
			SPY_CALL_LISTEN : { interaction : null                        , supervisor : "sup_call_spy_listen"       },
			SCREEN_SHARE    : { interaction : "int_screen_share"          , supervisor : null                        }
		});
		//NOTA: capaz que cambia cuando saquemos allegro
		//a nivel de codigo soporta ser dividido en { interaction : true, supervisor : false }
		this.m_actionsWithPhoneRequired = {
			FINISH          : { interaction : true, supervisor : false },
			HANGUP          : { interaction : true, supervisor : false },
			REQUEUE         : { interaction : true, supervisor : false },
			TAKE            : { interaction : true, supervisor : false },
			TRANSFER        : { interaction : true, supervisor : false },
			TRANSFER_QUEUED : { interaction : true, supervisor : false },
			SCHEDULE        : { interaction : true, supervisor : false },
			REDIAL          : { interaction : true, supervisor : false },
			RECORDING       : { interaction : true, supervisor : false },
			HOLD            : { interaction : true, supervisor : false },
			SEND_DIGITS     : { interaction : true, supervisor : false },
			CONFERENCE      : { interaction : true, supervisor : false },
			LISTEN_RECORD   : { interaction : true, supervisor : false },
			DOWNLOAD_RECORD : { interaction : true, supervisor : false },
			SPY_CALL_LISTEN : { interaction : true, supervisor : false },
			SCREEN_SHARE    : { interaction : true, supervisor : false }
		};
	}

	inConcert.Utils.inherit(TelephonyInteraction, inConcert.interactions.base);

	if (typeof inConcert.interactions.telephony !== "function") {
		inConcert.interactions.telephony = TelephonyInteraction;
	}

	// invokes dial
	TelephonyInteraction.prototype.dial = function(params) {
		const {
			accountId,        // solo para telefonia nativa de i6
			attentionLevelId,
			destination,
			callback = $.noop
		} = params;

		const phone = inConcert.app.getInteractionManager().getChannelClient("CALL");
		if (!phone) {
			return callback(false, "phone not found");
		}

		const resp = phone.canStartOutgoingCall();
		if (!resp.success) {
			return callback(false, resp.message);
		}

		const campaignPrefix = inConcert.app.getCampaignPhonePrefix(this.campaign, accountId);
		const makeCall = () => {
			phone.dial({
				interactionID: this.id,
				accountId    : accountId,
				campaignAtVCC: this.campaign,
				prefix       : campaignPrefix,
				destination  : destination,
				contact      : this.contactObject,
				callback     : callback
			});
		};

		if (!attentionLevelId) {
			return makeCall();
		}

		const preInteractionData = {
			attention_level_id: attentionLevelId
		};
		this.setPreInteractionData(preInteractionData, makeCall);
	};

	//Telephony conversation are always considered to be active
	TelephonyInteraction.prototype.isLive = function() {
		return true;
	};

	TelephonyInteraction.prototype.matchPending = function(interaction) {
		var self = this;
		if (!interaction) {
			return false;
		}
		if (interaction.type !== "CALL" || interaction.initiative !== "OUTBOUND" ||
			self.isDialer() || self.state !== "PENDING" || !self.isCreatedByAgent()) 
		{
			return false;
		}
		if (!interaction.interactionData.completeAddress) {
			return false;
		}
		if (this.contactAddress !== interaction.interactionData.completeAddress.DialedNumber) {
			return false;
		}
		return true;
	};

	//returns call state
	TelephonyInteraction.prototype.getCallState = function() {
		return this.m_callState;
	};

	//answers the current conversation if state is valid and phone supports it
	TelephonyInteraction.prototype.answer = $.noop;

	TelephonyInteraction.prototype.accountIsOperational = function() {
		var accountResponse = {
			success: false
		};
		if (!this.campaign) {
			return accountResponse;
		}
		var campaign = inConcert.app.getCampaign(this.campaign);
		if (!campaign) {
			return accountResponse;
		}
		accountResponse.success = true;
		return accountResponse;
	};

	TelephonyInteraction.prototype.transfer = function(target, onTransferred = $.noop) {
		if (!inConcert.app.has_allegro_baragent()) {
			return TelephonyInteraction.uber.transfer.call(this, target, onTransferred);
			//return onTransferred(false, "<%=_T('Telephony conversations cannot be transferred this way')%>");
		}
		switch(target.to) {
			case "campaign":
				return inConcert.BaragentActions.transferToCampaign(target.campaign_id, onTransferred);
			case "member":
				return inConcert.BaragentActions.transferToAgent(target.user, onTransferred);
			case "number":
				return inConcert.BaragentActions.transferToPhone(target.number, onTransferred);
			default:
				return onTransferred(false);
		}
	};

	//ESTO NO SE USO CON ALLEGRO
	TelephonyInteraction.prototype.startAttendedTransfer = function(address, addressType, onStarted) {
		var self = this;
		this.pending = true;
		inConcert.logger.action({
			action_name : "startAttendedTransfer",
			id : this.id,
			target : addressType+":"+address,
			campaign : this.campaign
		});

		var request = {
			id : this.id
		};
		request[addressType] = address;

		var url = this.m_manager.getBaseUrl() + "/interactions/call/start_attended_transfer/";
		inConcert.invoke({
			type       : "POST",
			url        : url,
			data       : JSON.stringify(request),
			dataType   : "json",
			contentType: "application/json",
			success: function() {
				self.pending = false;
				if (typeof(onStarted) === "function") {
					onStarted(true);
				}
			},
			error: function(_, error) {
				self.pending = false;
				inConcert.logger.error("Error in startAttendedTransfer: " + error.result);
				if (typeof(onStarted) !== "function") {
					return;
				}
				var errorString;
				switch (error.result) {
					case "INTERACTION NOT TAKEN":
						errorString = "<%=_T('This call is not taken by anyone')%>";
						break;
					case "INTERACTION NOT TAKEN BY AGENT":
						errorString = "<%=_T('This call is not taken by you')%>";
						break;
					case "CONTACT HUNG UP":
						errorString = "<%=_T('Contact hung-up while transferring')%>";
						break;
					case "AGENT UNAVAILABLE":
						errorString = format("<%=_T('User {0} is not available')%>", address);
						break;
					//case "INVALID_REQUEST":
					//case "INTERNAL_ERROR":
					default:
						errorString = "<%=_T('There was an error, please try again later')%>";
						break;
				}
				onStarted(false, errorString);
			}
		});
	};

	//ESTO NO SE USO CON ALLEGRO
	TelephonyInteraction.prototype.commitAttendedTransfer = function(onComplete) {
		inConcert.logger.action({action_name : "commitAttendedTransfer", id : this.id});

		var request = {
			id : this.id
		};
		var url = this.m_manager.getBaseUrl() + "/interactions/call/commit_attended_transfer/";
		inConcert.invoke({
			type       : "POST",
			url        : url,
			data       : JSON.stringify(request),
			dataType   : "json",
			contentType: "application/json",
			success: function() {
				if (typeof(onComplete) === "function") {
					onComplete(true);
				}
			},
			error: function(_, error) {
				inConcert.logger.error("Error in commitAttendedTransfer: " + error.result);
				if (typeof(onComplete) !== "function") {
					return;
				}
				var errorString;
				if(error.result === "INTERACTION NOT TRANSFER TAKEN") {
					errorString = "<%=_T('Call is not being transferred')%>";
				}
				else {
					// INVALID_REQUEST, INTERNAL_ERROR
					errorString = "<%=_T('Unknown error confirming the transfer')%>";
				}
				onComplete(false, errorString);
			}
		});
	};

	//ESTO NO SE USO CON ALLEGRO
	TelephonyInteraction.prototype.cancelAttendedTransfer = function(onComplete) {
		inConcert.logger.action({action_name : "cancelAttendedTransfer", id : this.id});

		var request = {
			id : this.id
		};
		var url = this.m_manager.getBaseUrl() + "/interactions/call/cancel_attended_transfer/";
		inConcert.invoke({
			type       : "POST",
			url        : url,
			data       : JSON.stringify(request),
			dataType   : "json",
			contentType: "application/json",
			success: function() {
				if (typeof(onComplete) === "function") {
					onComplete(true);
				}
			},
			error: function(_, error) {
				inConcert.logger.error("Error in cancelAttendedTransfer: " + error.result);
				if (typeof(onComplete) !== "function") {
					return;
				}
				var errorString;
				if (error.result === "INTERACTION NOT TRANSFER TAKEN") {
					errorString = "<%=_T('Call is not being transferred')%>";
				}
				else {
					// INVALID_REQUEST, INTERNAL_ERROR
					errorString = "<%=_T('Unknown error cancelling the transfer')%>";
				}
				onComplete(false, errorString);
			}
		});
	};
	
	TelephonyInteraction.prototype.commitAttendedConference = TelephonyInteraction.prototype.commitAttendedTransfer;
	TelephonyInteraction.prototype.cancelAttendedConference = TelephonyInteraction.prototype.cancelAttendedTransfer;

	TelephonyInteraction.prototype.setSession = function(session) {
		this.interactionData = this.interactionData || {
			contactId : session.contactId || null,
			contactName : session.contactName || null
		};
		this.contactAddress = this.contactAddress || (session.initiative === "INBOUND"? session.callerId : session.calledId);
		TelephonyInteraction.uber.setSession.call(this, session);
	};

	TelephonyInteraction.prototype.cleanSession = function() {
		this.interactionData = null;
		this.contactAddress = null;
	};

	TelephonyInteraction.prototype.eventsAreFromTheSameClient = function(/*oncomplete*/) {
		return true;
	};

	/**
	* Calls don't have any events. There is no need to inflate the interaction.
	* TODO: Wouldn't it be better for ClientWindow to ask if the interaction type has events or not?
	*/
	TelephonyInteraction.prototype.getEventList = function()
	{
		return null;
	};
	TelephonyInteraction.prototype.getFirstIncommingEvent = function()
	{
		return null;
	};

	TelephonyInteraction.prototype.getFirstOutgoingEvent = function()
	{
		return null;
	};

	TelephonyInteraction.prototype.eventsAreFromTheSameClient = function() {
		return true;
	};

	TelephonyInteraction.prototype.getUnreadEventCount = function()
	{
		return 0;
	};

	TelephonyInteraction.prototype.getEventCount = function()
	{
		return 0;
	};

	TelephonyInteraction.prototype.supportsEvents = function ()
	{
		return false;
	};
	
	TelephonyInteraction.prototype.setTimestamp = function() {
		var zp = inConcert.Utils.zeropad;
		var now_utc = inConcert.Utils.getNowUTCDateByServer();
		this.timestamp = format("{0}-{1}-{2} {3}:{4}:{5}",
			now_utc.getUTCFullYear(),
			zp(now_utc.getUTCMonth() + 1),
			zp(now_utc.getUTCDate()),
			zp(now_utc.getUTCHours()),
			zp(now_utc.getUTCMinutes()),
			zp(now_utc.getUTCSeconds())
		);
	};

	TelephonyInteraction.prototype.changeState  = function(state, notManual) {
		if (this.state === state) {
			return;
		}
		if (this.state === "TRANSFER TAKEN" || this.state === "CONFERENCE TAKEN" || this.state === "CONFERENCE") {
			this.pending = false;
		}
		this.state = state;
		this.manualState = !notManual && !inConcert.app.has_allegro_baragent();
		this.setTimestamp();
	};

	TelephonyInteraction.prototype.hasPreview = function() {
		if (!inConcert.app.has_allegro_baragent())
			return TelephonyInteraction.uber.hasPreview.call(this);

		var phone = inConcert.app.getInteractionManager().getChannelClient("CALL");
		if (!phone) {
			return false;
		}
		let has_attending = phone.attendingInteraction(this);
		if (!has_attending) {
			return false;
		}
		if (phone._sip_phone.with_preview) {
			return true;
		}
		if (phone._sip_phone.preview) {
			return true;
		}
		return false;
	};

	TelephonyInteraction.prototype.isPreview = function() {
		if (!inConcert.app.has_allegro_baragent())
		return TelephonyInteraction.uber.isPreview.call(this);

		var phone = inConcert.app.getInteractionManager().getChannelClient("CALL");
		return phone && phone.attendingInteraction(this) && phone._sip_phone.preview;
	};

	TelephonyInteraction.prototype.isRecording = function() {
		if (inConcert.app.has_allegro_baragent()) {
			var phone = inConcert.app.getInteractionManager().getChannelClient("CALL");
			return phone && phone.attendingInteraction(this) && phone._sip_phone.recording;
		}
		else {
			let activeLeg = this.getActiveLeg();
			return activeLeg && activeLeg.recording || false;
		}
	};

	TelephonyInteraction.prototype.isInConference = function()
	{
		if (inConcert.app.has_allegro_baragent()) {
			var phone = inConcert.app.getInteractionManager().getChannelClient("CALL");
			return phone && phone.attendingInteraction(this) && phone._sip_phone.in_conference;
		}
		else {
			return this.state === "CONFERENCE";
		}
	};
	
	TelephonyInteraction.prototype.isInConferenceWithoutClient = function() 
	{
		if (inConcert.app.has_allegro_baragent()) {
			return false;
		}
		else {
			if (!this.isInConference()) {
				return false;
			}
			let contactLeg = this.getContactLeg();
			return !contactLeg || contactLeg.status !== "BRIDGED";
		}
	};

	TelephonyInteraction.prototype.isOnHold = function()
	{
		if (inConcert.app.has_allegro_baragent()) {
			var phone = inConcert.app.getInteractionManager().getChannelClient("CALL");
			return phone && phone.attendingInteraction(this) && phone._sip_phone.in_hold;
		}
		else {
			let activeLeg = this.getActiveLeg();
			return activeLeg && activeLeg.hold || false;
		}
	};

	TelephonyInteraction.prototype.isMuted = function() {
		var phone = inConcert.app.getInteractionManager().getChannelClient("CALL");
		return phone && phone.attendingInteraction(this) && phone._sip_phone.muted;
	};

	TelephonyInteraction.prototype.isSwitched = function() {
		if (inConcert.app.has_allegro_baragent()) {
			var phone = inConcert.app.getInteractionManager().getChannelClient("CALL");
			return phone && phone.attendingInteraction(this) && phone._sip_phone.switched;
		}
		else {
			let consultationLeg = this.getConsultationLeg();
			return consultationLeg && consultationLeg.status !== "BRIDGED";
		}
	};
	
	TelephonyInteraction.prototype.isConsulting = function() {
		if (inConcert.app.has_allegro_baragent()) {
			return this.isConferenceConnected();
		}
		else {
			let lastStep = this.getLastStep();
			return lastStep && (lastStep.action === "TRANSFER TAKEN" || lastStep.action === "CONFERENCE TAKEN") &&
				lastStep.from === inConcert.app.getLoggedUserFullId();
		}
	};
	
	TelephonyInteraction.prototype.isIncomingConsultation = function() {
		if (inConcert.app.has_allegro_baragent()) {
			return false;
		}
		else {
			let lastStep = this.getLastStep();
			return lastStep && (lastStep.action === "TRANSFER TAKEN" || lastStep.action === "CONFERENCE TAKEN") &&
				lastStep.to === inConcert.app.getLoggedUserFullId();
		}
	};

	TelephonyInteraction.prototype.isTransferring = function()
	{
		if (!inConcert.app.has_allegro_baragent()) {
			return false; //implementame
		}
		var phone = inConcert.app.getInteractionManager().getChannelClient("CALL");
		return phone && phone.attendingInteraction(this) && phone._sip_phone.isTransferring();
	};

	TelephonyInteraction.prototype.isTransferringConnected = function()
	{
		if (!inConcert.app.has_allegro_baragent()) {
			return false; //implementame
		}
		var phone = inConcert.app.getInteractionManager().getChannelClient("CALL");
		return phone && phone.attendingInteraction(this) && phone._sip_phone.isTransferringConnected();
	};

	TelephonyInteraction.prototype.isConferenceTrying = function()
	{
		if (!inConcert.app.has_allegro_baragent()) {
			return false; //implementame
		}
		var phone = inConcert.app.getInteractionManager().getChannelClient("CALL");
		return phone && phone.attendingInteraction(this) && phone._sip_phone.isConferenceTrying();
	};

	TelephonyInteraction.prototype.isConferenceConnected = function()
	{
		if (!inConcert.app.has_allegro_baragent()) {
			return false; //implementame
		}
		var phone = inConcert.app.getInteractionManager().getChannelClient("CALL");
		return phone && phone.attendingInteraction(this) && phone._sip_phone.isConferenceConnected();
	};

	TelephonyInteraction.prototype.__actionRequiresPhone = function(action, app) {
		var requires = this.m_actionsWithPhoneRequired[action];
		if (requires && typeof requires === "object") {
			return requires[app] || false;
		}
		return requires || false;
	};

	TelephonyInteraction.prototype.hasPermission = function(action, app) {
		if (app === "supervisor" && $.inArray(action, ["TRANSFER"]) >= 0) {
			return false;
		}
		if (this.__actionRequiresPhone(action, app)) {
			var phone = inConcert.app.getInteractionManager().getChannelClient("CALL");
			if (!phone) {
				return false;
			}
		}
		if ($.inArray(action, ["HANGUP", "CONFERENCE", "TRANSFER", "REQUEUE"]) >= 0 && 
				(this.isTransferring() || this.isTransferringConnected() || 
				this.isConferenceTrying() || this.isConferenceConnected())) 
		{
			return false;
		}
		if ($.inArray(action, ["CONFERENCE"]) >= 0 && this.isOnHold()) {
			return false;
		}
		if ($.inArray(action, ["FINISH", "REDIAL"]) >= 0 && this.in_redial) {
			return false;
		}
		if (action === "REDIAL" && !this.isOutgoing()) {
			var campaign = inConcert.app.getCampaign( this.campaign );
			if (!campaign || !campaign.isInboundRedialEnabled(this.getAccount())) {
				return false;
			}
		}
		return TelephonyInteraction.uber.hasPermission.apply(this, [action, app]);
	};

	TelephonyInteraction.prototype.getRecords = function(callback = $.noop)
	{
		const self = this;

		const successCallbackFn = function(success, records) {
			if (!success) {
				callback(false);
				return;
			}
			// Invoco a la clase base.
			TelephonyInteraction.uber.getRecords.call(self, callback, records);
		};

		if (inConcert.app.has_allegro_integration()) {
			// allegro
			inConcert.app.getAllegroAPI().getInteractionRecords(this.id, successCallbackFn);
			return;
		}
		// nativo
		let records = (this.interactionData.records || []).reduce((records, record) => {
			if (!record.Mode || record.Mode === "all" || record.Mode === "audio_only" ) {
				records.push(record);
			}
			if (record.Mode === "all" || record.Mode === "video_only" || record.Mode === "screen_recording") {
				records.push({
					...record,
					Video : true,
					Format : record.VideoFormat
				});
			}
			return records;
		}, []);
		// invoco al callback
		successCallbackFn(true, records);
	};

	TelephonyInteraction.prototype.take = function( callback, app_id )
	{
		if (!inConcert.app.has_allegro_integration()) {
			return TelephonyInteraction.uber.take.call(this, callback, app_id);
		}

		var self = this;
		this.pending = true;

		var result = function(ok, message) {
			self.pending = false;
			if (!ok) {
				inConcert.logger.error({action_name : "take_interaction", app: app_id, error_type : "", error_message : message });
			}
			callback(ok, message);
		};

		if (!this.canBeTaken()) {
			return result(false, "<%=_T('Failed to take conversation, please try again in a few seconds...')%>");
		}

		if (!inConcert.app.canTakeWithCurrentState()) {
			return result(false, format("<%=_T('You can not take this because your current status is {0}')%>", 
				inConcert.app.getLoggedUserStatus()));
		}

		if (!inConcert.app.userHasAddress("CALL")) {
			return result(false, format("<%=_T('You can not take this because have not configured phone')%>", 
				inConcert.app.getLoggedUserStatus()));
		}

		if (!inConcert.AllegroInteractionManager.canTakeAllegroInteractions()) {
			return result(false, "<%=_T('Please check you are not exceeding the maximum amount of conversation of this type in your inbox')%>");
		}

		inConcert.BaragentActions.take(this.id, function(ok) {
			if (!ok) {
				return result(false, "<%=_T('Failed to take conversation, please try again in a few seconds...')%>");
			}
			//Este chequeo se hace porque el take devuelve siempre true (incluso es asincrónico)
			var timeout = 15000;
			(function checkInteraction() {
				if (inConcert.BaragentPolling.hasThisInteraction(self.id)) {
					return result(true);
				}
				timeout -= 300;
				if (timeout < 0) {
					return result(false, "<%=_T('Please check you are not exceeding the maximum amount of conversation of this type in your inbox')%>");
				}
				window.setTimeout(checkInteraction, 300);
			})();
		});
	};

	TelephonyInteraction.prototype.canSchedule = function()
	{
		var is_taken = this.isTaken();
		var is_wrapped_up = this.isWrappedUp();
		var is_abandoned = this.isAbandoned();

		if (is_taken || is_wrapped_up || is_abandoned) {
			var campaign = inConcert.app.getCampaign( this.campaign );
			if (!campaign) {
				return false;
			}
			var account = this.getAccount();
			if (is_taken || is_wrapped_up) {
				return campaign.isCallbackEnabled(account);
			}
			return campaign.isCallbackEnabledForAbandoned(account);
		}

		return false;
	};
	
	TelephonyInteraction.prototype.__scheduleBaragent = function(data, callback)
	{
		let self = this;

		var schedule_callback = function(ok) {
			callback(ok);
			self.pending = false;
		};

		var scheduleAbandoned = function()
		{
			self.pending = true;

			var doSchedule = function(contactId, contactName)
			{
				contactName = encodeURIComponent(contactName);
				if (data.type === "DATE") {
					inConcert.BaragentActions.scheduleAbandoned(self.id, self.getCampaign(), data.number, contactId, contactName,
						data.date,
						schedule_callback
					);
				}
				else {
					inConcert.BaragentActions.callAbandoned(self.id, self.getCampaign(), data.number, contactId, contactName,
						schedule_callback
					);
				}
			};

			if (self.contact) {
				inConcert.app.getContactManager().getContact( self.contact, function( contact ) {
					if (contact) {
						doSchedule(contact.externalId || "", contact.Name);
					}
					else {
						doSchedule("", self.getClientName());
					}
				});
			}
			else {
				doSchedule("", self.getClientName());
			}
		};

		var scheduleCurrent = function()
		{
			self.pending = true;

			if (data.type === "DATE") {
				inConcert.BaragentActions.rescheduleCurrent(data.number, data.date,
					schedule_callback
				);
			}
			else {
				inConcert.BaragentActions.callbackCurrent(data.number,
					schedule_callback
				);
			}
		};

		if (this.isAbandoned()) {
			scheduleAbandoned();
		}
		else if (this.isTaken() || this.isWrappedUp()) {
			scheduleCurrent();
		}
		else {
			callback(false, format("<%=_T('CANT SCHEDULE CONVERSATIONS WITH STATE {0}')%>", this.state));
		}
	};

	TelephonyInteraction.prototype.schedule = function(data, callback = $.noop)
	{
		if (inConcert.app.has_allegro_baragent()) {
			return this.__scheduleBaragent(data, callback);
		}
		data.proxyType = "CALL";
		if (data.overridedNumber) {
			data.addressTable = {
				Number : data.number
			};
		}
		TelephonyInteraction.uber.schedule.call(this, data, callback);
	};

	TelephonyInteraction.prototype.getRedialNumber = function() {
		if (inConcert.app.has_allegro_baragent()) {
			const number = this.interactionData.completeAddress?.DialedNumber ||
				this.contactAddress || "";
			return inConcert.Utils.cleanPhoneNumber(number, { allowPlus: false });
		}
			
		return this.contactAddress;
	};

	TelephonyInteraction.prototype.isPhoneValidToRedial = function() {
		return inConcert.app.validatePhoneNumber(this.getRedialNumber() || "", "CALL");
	};

	TelephonyInteraction.prototype.isTelephonyInteraction = function() {
		return true;
	};
	
	TelephonyInteraction.prototype.hasIVR = function() {
		return this.interactionData && this.interactionData.hasIVR || false;
	};

	TelephonyInteraction.prototype.isInboundCall = function() {
		if (this.initiative === "INBOUND") {
			return true;
		}
		//Esto es por las llamadas de dialer con un flujo de ivr o voiceblaster previo
		//En esos casos, se manda a agente y la llamada llega como una entrante
		return this.hasIVR();
	};

	TelephonyInteraction.prototype.creationDate = function(creation_date)
	{
		if (this.isAppointment()) {
			creation_date = this.getAppointmentDate() 
		}
		return TelephonyInteraction.uber.creationDate.call(this, creation_date);
	};

	TelephonyInteraction.prototype.getBaragentAttribute = function(key, callback) {
		inConcert.app.getBaragentActions().getAttribute(key.toUpperCase(), function(value) { 
			callback(value); 
		});
	};

	TelephonyInteraction.prototype.__loadPopupUrlAttributes = function() {
		if (this.m_popupUrlAttributes) {
			return;
		}
		let popup = inConcert.app.getBaragentPolling().getPopupInfo();
		this.m_popupUrlAttributes = popup && popup.url? inConcert.Utils.getUrlParameters(popup.url) : {};
	};

	TelephonyInteraction.prototype.getPopupUrlAttribute = function(key) {
		this.__loadPopupUrlAttributes();
		return this.m_popupUrlAttributes[key];
	};

	TelephonyInteraction.prototype.getRedialProcess = function() {
		return this.interactionData && this.interactionData.redialProcess || null;
	};

	TelephonyInteraction.prototype.cancelInteraction = function() {
		if (inConcert.app.has_allegro_integration()) {
			return;
		}
		const phone = inConcert.app.getInteractionManager().getChannelClient("CALL");
		if (phone && phone.status() === "DIALING") {
			phone.hangup(this.id);
			inConcert.app.getMainWindow().showBannerMessage({
				messageType: "Alert",
				message    : "<%=_T('Outgoing conversation was canceled, collision with incoming conversation. Please, switch to Online NoACD status to do outgoing conversations')%>..."
			});
		}
	};

	TelephonyInteraction.prototype.__fireInteractionAssigned = function() {
		if (!inConcert.app.has_allegro_integration()) {
			const status = this.getAccountStatus();
			const phone = inConcert.app.getInteractionManager().getChannelClient("CALL");
			if (status.status !== "OK" || !phone) {
				this.reject();
				return;
			}
			const interaction = phone.getInteraction();
			if (phone.status() === "DIALING") {
				if (interaction && interaction.id !== this.id) {
					this.reject();
					return;
				}
				else if (interaction && interaction.id === this.id && this.state === "CANCELED") {
					phone.hangup(this.id);
					inConcert.app.getMainWindow().showBannerMessage({
						messageType: "Alert",
						message    : "<%=_T('Outgoing conversation was canceled, collision with incoming conversation. Please, switch to Online NoACD status to do outgoing conversations')%>..."
					});
					return;
				}
			}
		}
		if (inConcert.app.isInRedial() || this.getRedialProcess()) {
			return;
		}
		TelephonyInteraction.uber.__fireInteractionAssigned.call(this);
	};

	let archivedIds = {};

	TelephonyInteraction.prototype.__fireInteractionArchived = function() {
		let interactionId = this.id;
		if (inConcert.app.isInRedial() || archivedIds[interactionId]) {
			return;
		}
		archivedIds[interactionId] = true;
		window.setTimeout(function() {
			if (archivedIds[interactionId]) {
				delete archivedIds[interactionId];
			}
			if ($.isEmptyObject(archivedIds)) {
				archivedIds = {};
			}
		}, 5000);
		TelephonyInteraction.uber.__fireInteractionArchived.call(this);
	};
	
	TelephonyInteraction.prototype.__fireInteractionCanceled = function() {
		if (inConcert.app.isInRedial()) {
			this.__fireNotification("CallRedialCanceled");
		} else if (inConcert.app.isInAppointment()) {
			this.__fireNotification("CallAppointmentCanceled");
		}
		else {
			TelephonyInteraction.uber.__fireInteractionCanceled.call(this);
		}
	};

	TelephonyInteraction.prototype.__fireInteractionInWrapup = function() {
		//called two times in transfer
		if (inConcert.app.has_allegro_baragent() && this.m_beforeNotificationsData.state === "TRANSFERRING") {
			return;
		}
		TelephonyInteraction.uber.__fireInteractionInWrapup.call(this);
	};

	TelephonyInteraction.prototype.canHangup = function() {
		return !this.isFinished() && !this.isWrappedUp();
	};

	TelephonyInteraction.prototype.hangup = function(callback = $.noop) {
		if (!this.canHangup()) {
			return callback(false, "can not hangup");
		}
		let phone = inConcert.app.getInteractionManager().getChannelClient("CALL");
		if (!phone) {
			return callback(false, "conversation is closed");
		}
		phone.hangup(this, callback);
	};

	TelephonyInteraction.prototype.takeAppointment = function(callback, app_id = "interaction") {
		callback = callback || $.noop;
		const self = this;

		const phone = inConcert.app.getInteractionManager().getChannelClient("CALL");
		if (!phone) {
			return callback(false, "<%=_T('Can not take appointment without a phone.')%>");
		}
		let error = phone.canMakeAnOutgoingCall();
		if (error) {
			return callback(false, error.message);
		}
		if (!inConcert.app.canTakeWithCurrentState()) {
			return callback(false, format("<%=_T('You can not take this because your current status is {0}')%>", 
				inConcert.app.getLoggedUserStatus()));
		}
		this.canTakeAppointment(function(status, reason) {
			if (!status) {
				return callback(false, reason);
			}
			phone.takeAppointment(self, function(status, err) {
				callback(status, err);
			});
		}, app_id);
	};

	TelephonyInteraction.prototype.canTakeAppointment = function(callback, app_id = "interaction") {
		callback = callback || $.noop;
		if (!this.isAppointmentActive()) {
			return callback(false, "<%=_T('Appointment is not active.')%>");
		}
		var getErrorMessage = function(reason)
		{
			reason = (reason || "").toUpperCase();
			switch (reason) {
				case "INVALID_REQUEST":
					return "<%=_T('There was an error, please try again later')%>";

				case "FORBIDDEN":
					return "<%=_T('Unable to create outgoing conversation. You not have permission.')%>";

				case "AGENT_UNAVAILABLE":
					return format("<%=_T('You can not take this because your current status is {0}')%>", inConcert.app.getLoggedUserStatus());

				default:
					return "<%=_T('There was an error, please try again later')%>";
			}
		};

		const url = this.m_manager.getBaseUrl() + "/interactions/can_take_appointment/";
		const request = {
			proxyType    : this.type,
			campaignAtVCC: this.campaign,
			interactionID: this.id,
			app_id       : app_id
		};

		inConcert.invoke({
			type: "POST",
			url: url,
			data: JSON.stringify( request ),
			dataType: "json",
			contentType: "application/json",
			success: function( result ) {
				callback(result.result === "OK", result.reason);
			},
			error: function( _, error ) {
				inConcert.logger.action({ action_name : "can_take_appointment", error_message : error.result });
				let error_message = getErrorMessage(error.result);
				callback(false, error_message);
			}
		});
	};

	TelephonyInteraction.prototype.startRecord = function(onComplete = $.noop) {
		var self = this;
		var request = {
			id : this.id
		};
		let liveClientWindow = self.m_manager.m_interactionClients.CALL._view;
		if (liveClientWindow && (!liveClientWindow.m_interaction || liveClientWindow.m_interaction.isRecording())) {
			liveClientWindow.clientUpdate();
			return onComplete(false);
		}			
		var url = this.m_manager.getBaseUrl() + "/interactions/call/start_record/";
		inConcert.invoke({
			type       : "POST",
			url        : url,
			data       : JSON.stringify(request),
			dataType   : "json",
			contentType: "application/json",
			success: function(result) {
				onComplete(result);
			},
			error: function(_, error) {
				inConcert.logger.error("Error in record: " + error.result);
				onComplete(false, error.result);
			}
		});
	};

	TelephonyInteraction.prototype.stopRecord = function(onComplete = $.noop) {
		var self = this;
		var request = {
			id : this.id
		};
		let liveClientWindow = self.m_manager.m_interactionClients.CALL._view;
		if (liveClientWindow && (!liveClientWindow.m_interaction || !liveClientWindow.m_interaction.isRecording())) {
			liveClientWindow.clientUpdate();
			return onComplete(false);
		}		
		var url = this.m_manager.getBaseUrl() + "/interactions/call/stop_record/";
		inConcert.invoke({
			type       : "POST",
			url        : url,
			data       : JSON.stringify(request),
			dataType   : "json",
			contentType: "application/json",
			success: function(result) {				
				onComplete(result);
			},
			error: function(_, error) {
				inConcert.logger.error("Error in stopRecord: " + error.result);
				onComplete(false, error.result);
			}
		});
	};
	
	TelephonyInteraction.prototype.hold = function(onComplete) {
		var self = this;
		var request = {
			id : this.id
		};
		let liveClientWindow = self.m_manager.m_interactionClients.CALL._view;
		if (liveClientWindow && (!liveClientWindow.m_interaction || liveClientWindow.m_interaction.isOnHold())) {
			liveClientWindow.clientUpdate();
			return;
		}			
		var url = this.m_manager.getBaseUrl() + "/interactions/call/hold/";
		inConcert.invoke({
			type       : "POST",
			url        : url,
			data       : JSON.stringify(request),
			dataType   : "json",
			contentType: "application/json",
			success: function(result) {
				if (typeof(onComplete) === "function") {			
					onComplete(result);
				}
			},
			error: function(_, error) {
				inConcert.logger.error("Error in hold: " + error.result);
				if (typeof(onComplete) !== "function") {
					return;
				}
				onComplete(false, error.result);
			}
		});
	};
	
	TelephonyInteraction.prototype.unhold = function(onComplete) {
		var self = this;
		var request = {
			id : this.id
		};
		let liveClientWindow = self.m_manager.m_interactionClients.CALL._view;
		if (liveClientWindow && (!liveClientWindow.m_interaction || !liveClientWindow.m_interaction.isOnHold())) {
			liveClientWindow.clientUpdate();
			return;
		}		
		var url = this.m_manager.getBaseUrl() + "/interactions/call/unhold/";
		inConcert.invoke({
			type       : "POST",
			url        : url,
			data       : JSON.stringify(request),
			dataType   : "json",
			contentType: "application/json",
			success: function(result) {
				if (typeof(onComplete) === "function") {
					onComplete(result);
				}
			},
			error: function(_, error) {
				inConcert.logger.error("Error in unhold: " + error.result);
				if (typeof(onComplete) !== "function") {
					return;
				}
				onComplete(false, error.result);
			}
		});
	};

	TelephonyInteraction.prototype.reject = function(callback) {
		inConcert.invoke({
			type       : "POST",
			url        : inConcert.app.getInteractionManager().getBaseUrl() + "/interactions/call/close_agent_leg/",
			data       : JSON.stringify({ id : this.id }),
			dataType   : "json",
			contentType: "application/json",
			success: function() {
				if (typeof(callback) === "function") {
					callback(true);
				}
			},
			error: function(_, error) {
				inConcert.logger.error("Error in hangup: " + error.result);
				if (typeof(callback) !== "function") {
					return;
				}
				var errorString;
				if(error.result === "INTERACTION_NOT_FOUND") {
					errorString = "<%=_T('Conversation does not exist')%>";
				}
				else {
					// INVALID_REQUEST, INTERNAL_ERROR
					errorString = "<%=_T('There was an error, please try again later')%>";
				}
				callback(false, errorString);
			}
		});
	};

	TelephonyInteraction.prototype.getLegs = function() {
		return this.interactionData && this.interactionData.legs || {};
	};

	TelephonyInteraction.prototype.isLegActive = function(leg) {
		return leg.status !== "CLOSED" && leg.pendingStatus !== "CLOSED";
	};

	TelephonyInteraction.prototype.getChannelLeg = function(channel) {
		return channel && this.getLegs()[channel] || null;
	};

	TelephonyInteraction.prototype.getLegsBy = function(condition) {
		let legs = this.getLegs();
		return Object.keys(legs)
				.filter(function(channelID) {
				return condition(legs[channelID]);
			})
			.reduce(function(result, channelID) {
				result[channelID] = legs[channelID];
				return result;
			}, {});
	};

	TelephonyInteraction.prototype.getLegBy = function(condition) {
		let legs = this.getLegs();
		let channelID = Object.keys(legs).find(function(channelID) {
			return condition(legs[channelID]);
		});
		return channelID && legs[channelID] || null;
	};

	TelephonyInteraction.prototype.getContactLeg = function() {
		return this.getLegBy(function(leg) {
			return leg.contactLeg;
		});
	};

	TelephonyInteraction.prototype.getAgentLeg = function() {
		let lastStep = this.getLastStep();
		let agent = lastStep ? lastStep.agent || lastStep.from : null;
		return this.getLegBy(function(leg) {
			return leg.agent === agent;
		});
	};

	TelephonyInteraction.prototype.getConsultationLeg = function() {
		let lastStep = this.getLastStep();
		return this.getChannelLeg(lastStep ? lastStep.externalLeg || lastStep.toLeg : null);
	};

	TelephonyInteraction.prototype.getActiveLeg = function() {
		if ((this.isConsulting() && !this.isSwitched()) || this.isIncomingConsultation() || this.isInConferenceWithoutClient()) {
			return this.getConsultationLeg();
		}
		else {
			return this.getContactLeg();
		}
	};

	TelephonyInteraction.prototype.canSwitchConversation = function() {
		return this.isConsulting();
	};
	
	TelephonyInteraction.prototype.switchConversation = function(callback = $.noop) {
		if (!this.canSwitchConversation() || this.isSwitched()) {
			return callback(false);
		}

		let url = this.m_manager.getBaseUrl() + "/interactions/call/switch/";
		inConcert.invoke({
			type       : "POST",
			url        : url,
			data       : JSON.stringify({ id : this.id }),
			dataType   : "json",
			contentType: "application/json",
			success: function(/*result*/) {
				callback(true);
			},
			error: function(_, error) {
				inConcert.logger.error("Error in switch: " + error.result);
				callback(false);
			}
		});
	};

	TelephonyInteraction.prototype.unswitchConversation = function(callback) {
		if (!this.canSwitchConversation() || !this.isSwitched()) {
			return callback(false);
		}

		let url = this.m_manager.getBaseUrl() + "/interactions/call/unswitch/";
		inConcert.invoke({
			type       : "POST",
			url        : url,
			data       : JSON.stringify({ id : this.id }),
			dataType   : "json",
			contentType: "application/json",
			success: function(/*result*/) {
				callback(true);
			},
			error: function(_, error) {
				inConcert.logger.error("Error in unswitch: " + error.result);
				callback(false);
			}
		});
	};

	TelephonyInteraction.prototype.canHold = function(from_app = "interaction") {
		return this.hasPermission("HOLD", from_app) && !this.isIncomingConsultation() && !this.isInConference();
	};

	TelephonyInteraction.prototype.canRecord = function(from_app = "interaction") {
		return this.hasPermission("RECORDING", from_app) && !this.isIncomingConsultation() && !this.isIncomingConference();
	};

	TelephonyInteraction.prototype.isConferenceTaken = function() {
		if (inConcert.app.has_allegro_baragent()) {
			return false;
		}
		return this.state === "CONFERENCE TAKEN";
	};

	TelephonyInteraction.prototype.isAssigned = function() {
		return TelephonyInteraction.uber.isAssigned.call(this) || this.isConferenceTaken() || this.isInConference();
	};
	
	TelephonyInteraction.prototype.isIncomingConference = function() {
		if (inConcert.app.has_allegro_baragent()) {
			return false;
		}
		else {
			let lastStep = this.getLastStep();
			return lastStep && lastStep.action === "CONFERENCE" && lastStep.to === inConcert.app.getLoggedUserFullId();
		}
	};
	
	TelephonyInteraction.prototype.getAccountStatus = function() {
		let phone = inConcert.app.getInteractionManager().getChannelClient("CALL");
		return phone? phone.getAccountStatus() : { status : "phone not found" };
	};

	TelephonyInteraction.prototype.isGone = function() {
		const statesOfGoneInteraction = ["ABANDONED", "CANCELED", "FINISHED"];
		let isGoned = $.inArray(this.state, statesOfGoneInteraction) >= 0;
		console.info("[TelephonyInteraction.isGone] --> IsGoned: '" + isGoned + "', interaction '" + this.id + "', state "  + this.state);
		return isGoned;
	};

	TelephonyInteraction.prototype.hasVideo = function() {
		return this.interactionData && this.interactionData.hasVideo || false;
	};

	TelephonyInteraction.prototype.isWeb = function() {
		return this.interactionData && this.interactionData.source && this.interactionData.source === "webcall" || false;
	};

	TelephonyInteraction.prototype.hasScreenRecording = function() {
		return this.interactionData && this.interactionData.hasScreenRecording || false;
	};

	TelephonyInteraction.prototype.videoDisabled = function() {
		var phone = inConcert.app.getInteractionManager().getChannelClient("CALL");
		return phone && phone.attendingInteraction(this) && phone._sip_phone.videoDisabled;
	};

	TelephonyInteraction.prototype.remoteVideoDisabled = function() {
		const activeLeg = this.getActiveLeg();
		return activeLeg && activeLeg.videoDisabled || false;
	};
	
	TelephonyInteraction.prototype.__notifyAgentVideoChange = function(change, onComplete) {
		const url = this.m_manager.getBaseUrl() + "/interactions/call/agent_video_change/";
		const request = {
			id : this.id,
			change : change
		};
		inConcert.invoke({
			type       : "POST",
			url        : url,
			data       : JSON.stringify(request),
			dataType   : "json",
			contentType: "application/json",
			success: function(result) {
				if (typeof(onComplete) === "function") {			
					onComplete(result);
				}
			},
			error: function(_, error) {
				inConcert.logger.error("Error notyfing video change: " + error.result);
				if (typeof(onComplete) !== "function") {
					return;
				}
				onComplete(false, error.result);
			}
		});
	};
	
	TelephonyInteraction.prototype.disableVideo = function(onComplete) {
		if (!this.hasVideo()) {
			return onComplete(false, "has not phone");
		}
		const phone = inConcert.app.getInteractionManager().getChannelClient("CALL");
		if (!phone || !phone.attendingInteraction(this)) {
			return onComplete(false, "has not phone");
		}
		phone._sip_phone.disableVideo((status, err) => {
			if (!status) {
				return onComplete(status, err);
			}
			this.__notifyAgentVideoChange({ videoDisabled : true }, onComplete);
		});
	};
	
	TelephonyInteraction.prototype.enableVideo = function(onComplete) {
		if (!this.hasVideo()) {
			return onComplete(false, "has not phone");
		}
		const phone = inConcert.app.getInteractionManager().getChannelClient("CALL");
		if (!phone || !phone.attendingInteraction(this)) {
			return onComplete(false, "has not phone");
		}
		phone._sip_phone.enableVideo((status, err) => {
			if (!status) {
				return onComplete(status, err);
			}
			this.__notifyAgentVideoChange({ videoEnabled : true }, onComplete);
		});
	};

	TelephonyInteraction.prototype.screenShare = function(onComplete) {
		if (!this.hasVideo()) {
			return onComplete(false, "has not phone");
		}
		const phone = inConcert.app.getInteractionManager().getChannelClient("CALL");
		if (!phone || !phone.attendingInteraction(this)) {
			return onComplete(false, "has not phone");
		}
		phone._sip_phone.screenShare((status, err) => {
			if (!status) {
				return onComplete(status, err);
			}
			this.__notifyAgentVideoChange({ videoEnabled : true }, onComplete);
			this._isScreenSharing = true;
		});
	};

	TelephonyInteraction.prototype.StopScreenShare = function(onComplete) {
		if (!this.hasVideo()) {
			return onComplete(false, "has not phone");
		}
		const phone = inConcert.app.getInteractionManager().getChannelClient("CALL");
		if (!phone || !phone.attendingInteraction(this)) {
			return onComplete(false, "has not phone");
		}
		phone._sip_phone.StopScreenShare((status, err) => {
			if (!status) {
				return onComplete(status, err);
			}
			this.__notifyAgentVideoChange({ videoEnabled : true }, onComplete);
			this._isScreenSharing = false;
		});
	};

	TelephonyInteraction.prototype.isScreenSharing = function() {
		const phone = inConcert.app.getInteractionManager().getChannelClient("CALL");
		if (!phone || !phone.attendingInteraction(this)) {
			return false;
		}
		return !!this._isScreenSharing;
	};

	TelephonyInteraction.prototype.getClientName = function() {
		let clientName = TelephonyInteraction.uber.getClientName.call(this);
		const visualize_address = inConcert.app.canVisualizeAddress(null, this.campaign, null, true);
		
		if (!this.campaign) {
			return clientName;
		}
		const campaign = inConcert.app.getCampaign(this.campaign);
		
		if(clientName === "" || visualize_address || !campaign){
			return clientName;
		}
		const has_allegro_integration = inConcert.app.has_allegro_integration();

		if(has_allegro_integration)
		{
			let config = null;
			if(this.initiative === "INBOUND"){
				config = campaign.getIncomingConfig(this.getAccount());
				if(config && config.enabled && config.ContactCenter.ContactNameFormat ===""){
					return "<%=_T('Origen : Hidden')%>";			
				}
			}
			else if(this.initiative === "OUTBOUND")
			{
				config = campaign.getOutgoingConfig(this.getAccount());
				if(config && config.enabled && config.ContactCenter && config.ContactCenter.ManualContactNameFormat ===""){
					return "<%=_T('Origen : Hidden')%>";			
				}
			}
		}
		return clientName;
	};

	TelephonyInteraction.prototype.getHumanReadableAddress = function() {
		// Se extiende para ocultar numero si no se tiene los permisos 
		const contactAddress = TelephonyInteraction.uber.getHumanReadableAddress.call(this);
		const visualize_address = inConcert.app.canVisualizeAddress(null, this.campaign, null, true);	
		return visualize_address? contactAddress: "<%=_T('Hidden Address')%>";
	};

	TelephonyInteraction.prototype.getDisplayName = function() {
		let clientName = TelephonyInteraction.uber.getDisplayName.call(this);
		const visualize_address = inConcert.app.canVisualizeAddress(null, this.campaign, null, true);
		
		if (!this.campaign) {
			return clientName;
		}
		const campaign = inConcert.app.getCampaign(this.campaign);
		
		if(clientName === "" || visualize_address || !campaign){
			return clientName;
		}
		const has_allegro_integration = inConcert.app.has_allegro_integration();

		if(has_allegro_integration && clientName.indexOf("Origen : ") >= 0)
		{
			return "<%=_T('Origen : Hidden')%>";
		}
		return clientName;
	};

	

}());

