/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 11 */

/* global inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}
if (!inConcert.interactions) {
	inConcert.interactions = {};
}

/**
 * @namespace
 * @name TwitterInteraction
 */
(function () {

	"use strict";

	/**
	 * Class WebChatInteraction.
	 * Subclass of ChatInteraction
	 *
	 * @param {Object} rawInteraction
	 */
	function WebChatInteraction (rawInteraction)
	{
		WebChatInteraction.uber.constructor.call(this, rawInteraction);
		this.m_number = null;
		$.extend(true, this.m_actions, {
			LISTEN_RECORD   : { interaction : "int_listen_call_records"   , supervisor : "sup_listen_call_records"   },
			DOWNLOAD_RECORD : { interaction : "int_download_call_records" , supervisor : "sup_download_call_records" }
		});
	}

	inConcert.Utils.inherit(WebChatInteraction, inConcert.interactions.chat);

	if (typeof inConcert.interactions.webchat !== "function") {
		inConcert.interactions.webchat = WebChatInteraction;
	}

	WebChatInteraction.prototype.getAddress = function() {
		return this.interactionData.contactName;
	};

	WebChatInteraction.prototype.getDisplayAddress = function() {
		return this.getAddress();
	};

	/**
	* Webchats don't have any events. There is no need to inflate the interaction.
	* TODO: Wouldn't it be better for ClientWindow to ask if the interaction type has events or not?
	*/
	WebChatInteraction.prototype.getEventList = function()
	{
		return null;
	};

	WebChatInteraction.prototype.eventsAreFromTheSameClient = function() {
		return true;
	};

	WebChatInteraction.prototype.getUnreadEventCount = function()
	{
		return 0;
	};

	WebChatInteraction.prototype.getEventCount = function()
	{
		return 0;
	};

	WebChatInteraction.prototype.supportsEvents = function ()
	{
		return false;
	};

	WebChatInteraction.prototype.getTransferrerIds = function ()
	{
		return this.interactionData && this.interactionData.data && $.isArray(this.interactionData.data.transferrerIds)?
			this.interactionData.data.transferrerIds : [];
	};

	WebChatInteraction.prototype.getRecords = function(callback)
	{
		let records = (this.interactionData.records || []).reduce((records, record) => {
			if (record.Mode === "all" || record.Mode === "audio_only") {
				records.push(record);
			}
			if (record.Mode === "all" || record.Mode === "video_only" || record.Mode === "screen_recording") {
				records.push({
					...record,
					Video : true,
					Format : record.VideoFormat
				});
			}
			return records;
		}, []);
		// Invoco a la clase base.
		WebChatInteraction.uber.getRecords.call(this, callback, records);
	};
}());

