/*jshint browser:true, quotmark:false, smarttabs:true, -W014, curly:false, esversion: 11*/
/*global $, window, CheckIfLoginRetryNeeded, DoLogin, grecaptcha, notifyLoginError, inConcert, GetAuthToken*/

const FORMAT = inConcert.Utils.format;
let openIdTimeout;
let openIdPollTimer;

//little hack for IE
if (!String.prototype.trim) {
	String.prototype.trim = function() {
		return $.trim(this);
	};
}

////////////////////////////////////////////////////////////////
///
function IsUserFullSystemAccess (userInfo) {
	userInfo = userInfo || this.m_userInfo;
	if (!userInfo) return false;
	return userInfo.IsFullSystemAccess;
}

////////////////////////////////////////////////////////////////
///
function OnFieldKeyDown (field, evt) {
	var keycode;
	if(window.event) {
		keycode = window.event.keyCode;
	}
	else if (evt) {
		keycode = evt.which;
	}
	else {
		return true;
	}

	// escape o tab los ignoro
	if(keycode === 27 || keycode === 12) {
		return true;
	}

	if (keycode === 13) {
		$("#submitButton").click();
		return true;
	}

	var needed = document.getElementById(field.name + "_needed");
	if(needed) {
		document.body.removeChild(needed);
		field.style.backgroundColor = "";
	}
}

/*
	Funcion que inhabilita/habilita la sección de submit (btn, textboxes, waiting)
*/
function DisableSubmit (disabled, waiting_msg) {
	let isMfa = $("body").is(".mfaMode");
	$("#submitButton").removeClass("loginFormButton-ds").addClass("loginFormButton")[0].disabled = disabled;
	const $username = $("#username");
	const $password = $("#password");
	if (disabled) {
		$username.prop("disabled", true);
		$password.prop("disabled", true);
	}
	else if (!isMfa) {
		$username.prop("disabled", false);
		$password.prop("disabled", false);
	}
	const $waiting_msg = $(".loginWaitingMessage > div");
	const $waiting_container = $waiting_msg.parent();

	if (disabled && waiting_msg) {
		$waiting_msg.text(waiting_msg);
		$waiting_container.show();
	}
	else {
		$waiting_msg.text("");
		$waiting_container.hide();
	}
}

function Translate (message) {
	if (!message) {
		return "";
	}
	const know_messages = {
		"invalid request"             : "<%=_T('Something has gone wrong. Check your data.')%>"                  ,
		"invalid username"            : "<%=_T('Invalid User. Contact your administrator.')%>"                   ,
		"migrating or rollbacking vcc": "<%=_T('administrative tasks are running')%>"                            ,
		"new password error"          : "<%=_T('New password does not meet the system requirements.')%>"         ,
		"password change"             : "<%=_T('Error changing password. Please, contact your administrator.')%>",
		"time to login has expired"   : "<%=_T('Time to login has expired.')%>"
	};
	const translated_msg = know_messages[message.toLowerCase()];
	return translated_msg || message;
}

function hideErrors()
{
	$("#username_container").removeClass("ic-field-error");
	$(".ic-field-error").hide();
	$("#loginStatus").hide();
}

function showError(err)
{
	var message = typeof err === 'string' ? err : err.message;
	let translated_msg = Translate(message);
	if(err.field_id) {
		$("#" + err.field_id + "_validation").text(translated_msg).show().parent().addClass("ic-field-error");
	}
	else {
		$("#loginStatus div").text(translated_msg).parent().show();
	}
	notifyLoginError(message);
}

function makePasswordsCheckUI(passwordChecks)
{
	let makePasswordCheckUI = function(cssClass, text) {
		return $(
			"<div class='" + cssClass + "'>" +
				text +
			"</div>"
			);
	};
	let translatedPasswordChecks = [];
	//passwordLen always goes
	let passwordLenText = FORMAT("<%=_T('Be between {0} and {1} characters long')%>", passwordChecks.LEN.range[0], passwordChecks.LEN.range[1]);

	let passwordLenClass = passwordChecks.LEN.success === true ? "passwordCheckStatusPass" : (passwordChecks.LEN.success === false ? "passwordCheckStatusFail" : "passwordCheckStatusInitial");
	translatedPasswordChecks.push(makePasswordCheckUI(passwordLenClass, passwordLenText));

	if(passwordChecks.LOWERS.range[0] > 0) {
		let minLowerCharsText = FORMAT("<%=_T('Include at least {0} lowercase characters')%>", passwordChecks.LOWERS.range[0]);
		let minLowerCharsLenClass = passwordChecks.LOWERS.success === true ? "passwordCheckStatusPass" : (passwordChecks.LOWERS.success === false ? "passwordCheckStatusFail" : "passwordCheckStatusInitial");
		translatedPasswordChecks.push(makePasswordCheckUI(minLowerCharsLenClass, minLowerCharsText));
	}
	if(passwordChecks.UPPERS.range[0] > 0) {
		let minUpperCharsText = FORMAT("<%=_T('Include at least {0} uppercase characters')%>", passwordChecks.UPPERS.range[0]);
		let minUpperCharsLenClass = passwordChecks.UPPERS.success === true ? "passwordCheckStatusPass" : (passwordChecks.UPPERS.success === false ? "passwordCheckStatusFail" : "passwordCheckStatusInitial");
		translatedPasswordChecks.push(makePasswordCheckUI(minUpperCharsLenClass, minUpperCharsText));
	}
	if(passwordChecks.NUMBERS.range[0] > 0){
		let minNumericCharsText = FORMAT("<%=_T('Include at least {0} numeric characters')%>", passwordChecks.NUMBERS.range[0]);
		let minNumericCharsLenClass = passwordChecks.NUMBERS.success === true ? "passwordCheckStatusPass" : (passwordChecks.NUMBERS.success === false ? "passwordCheckStatusFail" : "passwordCheckStatusInitial");
		translatedPasswordChecks.push(makePasswordCheckUI(minNumericCharsLenClass, minNumericCharsText));
	}
	if(passwordChecks.SYMBOLS.range[0] > 0) {
		let minSpecialCharsText = FORMAT("<%=_T('Include at least {0} special characters')%>", passwordChecks.SYMBOLS.range[0]);
		let minSpecialCharsLenClass = passwordChecks.SYMBOLS.success === true ? "passwordCheckStatusPass" : (passwordChecks.SYMBOLS.success === false ? "passwordCheckStatusFail" : "passwordCheckStatusnitial");
		translatedPasswordChecks.push(makePasswordCheckUI(minSpecialCharsLenClass, minSpecialCharsText));
	}
	if(passwordChecks.HISTORY.range[0] > 0) {
		let historyText = FORMAT("<%=_T('Not have been used in the last {0} passwords')%>", passwordChecks.HISTORY.range[0]);
		let historyClass = passwordChecks.HISTORY.success === true ? "passwordCheckStatusPass" : (passwordChecks.HISTORY.success === false ? "passwordCheckStatusFail" : "passwordCheckStatusInitial");
		translatedPasswordChecks.push(makePasswordCheckUI(historyClass, historyText));
	}
	return translatedPasswordChecks;
}

function ShowPasswordChange(showBackToUserButton, passwordChecks) {
	$("#change_password_container").toggle(true);
	$("#submitButton").addClass("btn-dark");
	$(".loginFormContainer").addClass("change_password_main_container");
	$("#loginTitleContainer").toggle(false);
	$("#loginFormTitle").toggle(false);
	$("#loginFormSubtitle").text("<%=_T('Change Password')%>");
	$("#password_checks_list").html("");
	$("#backToUserButton").toggle(showBackToUserButton);
	for(let checkElem of makePasswordsCheckUI(passwordChecks || this.m_vcc_info.info.vcc.passwordChecks)) {
		$("#password_checks_list").append(checkElem);
	}
	$("#new_password")
		.off("focusin")
		.on("focusin", function() {
			$("#password_checks_container").toggle(true);
		});
	$("#new_password")
		.off("focusout")
		.on("focusout", function() {
			$("#password_checks_container").toggle(false);
		});
	$("#password_expiration_container").toggle(false);
}

function __on_openid_authorized(data, openid_window) {
	window.setTimeout( function() {
		openid_window.close();
		openid_window = undefined;

		if(data.code !== "0") {
			loginResponseSuccess({
					code : -1,
					message : data.message
				});
			DisableSubmit(true);
			$("#backToUserButton").show();
			return;
		}
		
		let urlParams = "hash=true" +
			"&username=" + encodeURIComponent(data.username) +
			"&loginToken=" + data.loginToken + 
			"&callcenter=" + data.callcenter +
			"&app=" + encodeURIComponent(data.app) +
			"&authenticatedByOpenId=true" + 
			"&authenticationFrom=" + encodeURIComponent(data.authenticationFrom); 
	
		var url = document.location.toString().replace(/apps.*/, "apps/agent/login/authenticate/");
		inConcert.invoke({
			url: url,
			type: 'post',
			data: urlParams,
			dataType: "json",
			contentType: "application/x-www-form-urlencoded",
			success: function(response) {
				DisableSubmit(true, response.message);
				loginResponseSuccess({
						code : response.status ? 0 : -1,
						message : response.message,
						languageId : response.languageId
					});
			},
			error: function(response) {
				console.log(response);
			}
		});	
	}, 1500);
}

function OpenOpenIdAuthPopup(url, popupWaitTimeout) {
	var window_options = 'height=600,width=800,status=yes,toolbar=yes,menubar=no,location=yes';
	var openid_window = window.open( url, '_blank', window_options );

	if (window.openid_message_listener) {
		window.removeEventListener("message", window.openid_message_listener);
		window.openid_message_listener = undefined;
	}

	var listener = function( ev )
	{
		if (ev.origin.search( window.location.host ) > 0 && ev.data && ev.data.authorization === "OPENID")
		{
			clearTimeout(openIdTimeout);
			window.clearInterval(openIdPollTimer);
			__on_openid_authorized( ev.data, openid_window );
			window.removeEventListener( 'message', window.openid_message_listener );
			window.openid_message_listener = undefined;
		}
	};

	window.openid_message_listener = listener;
	window.addEventListener("message", window.openid_message_listener, false );

	var onTimeoutOrClosePopup = function(message)
	{
		clearTimeout(openIdTimeout);
		window.clearInterval(openIdPollTimer);
		loginResponseSuccess({
				code : -1,
				message : message
			});
		window.removeEventListener( 'message', window.openid_message_listener );
		window.openid_message_listener = undefined;
		DisableSubmit(true);
		$("#backToUserButton").show();
	};

	//openId popup window timeout
	openIdTimeout = window.setTimeout( function() {
		onTimeoutOrClosePopup("<%=_T('The time to perform authentication has expired')%>");
	}, popupWaitTimeout);

	//openId popup window closig poll check
	openIdPollTimer = window.setInterval(function() {
		if (openid_window.closed !== false) { // !== is required for compatibility with Opera
			onTimeoutOrClosePopup("<%=_T('Authentication popup window closed')%>");
		}
	}, 200);
}

/**
Callback called if an auth token attempt was succesfull
	Funcion que es invocada en el archivo "login_plain.js"
*/
function GetAuthTokenResponseSuccess (success, response) {
	success = success && response.status;
	if (!success) {
		//error
		showError(response.message);
		//En cualquier caso, el boton lo necesitamos
		DisableSubmit(false);
		return;
	}
	// ok
	this.m_userInfo = response.userInfo;
	const isUserFullSystemAccess = IsUserFullSystemAccess();
	const vccInfo = response.info && response.info.vcc ? response.info.vcc : {};
	const $username_container = $("#username_container");

	if (!isUserFullSystemAccess
		&& vccInfo.authentication
		&& vccInfo.authentication.type === "OPENID"
		&& vccInfo.openid
		&& vccInfo.openid.authorization_endpoint
	) {
		const popupWaitTimeout = (vccInfo.openid.expireTokenSec * 1000) || 60000;
		OpenOpenIdAuthPopup(vccInfo.openid.authorization_endpoint, popupWaitTimeout);
		$username_container.toggle(true);
		DisableSubmit(true, "<%=_T('Authenticating user')%>");
		return;
	}
	$username_container.toggle(false);
	this.m_vcc_info = response;
	$("#password_container").toggle(true);
	if (response.mustChangePassword) {
		ShowPasswordChange(false, vccInfo.passwordChecks);
	}
	else if(response.pwdExpirationDays !== undefined) {
		$("#password_expiration_container").toggle(true);
		const hasChangePasswordPermission = response.hasChangePasswordPermission ? true : false;
		const message = FORMAT("<%=_T('Your password will expire in {0} days.')%>", response.pwdExpirationDays);
		$("#password_expiration_title").html(`
			<strong>${message}</strong> ${
				!hasChangePasswordPermission ? "<%=_T('Contact your administrator')%>" : ""
			}
		`);
		$("#password_expiration_subtitle").toggle(hasChangePasswordPermission);
		$("#changePasswordButton").toggle(hasChangePasswordPermission);
	}
	$("#submitButton").text("<%=_T('SIGN IN')%>");
	$("#backToUserButton").toggle(!response.mustChangePassword);
	setTimeout(function() { $('#password').focus(); });
	//En cualquier caso, el boton lo necesitamos
	DisableSubmit(false);
}

/**
Callback called if a login attempt was succesfull
	Funcion que es invocada en el archivo "login_plain.js"
*/
function loginResponseSuccess(response) {
	let isMfa = $("body").is(".mfaMode");
	if (isMfa && !response.retry) {
		revertMfa();
	}

	$(".formFieldWrong").each(function(){
		$(this).removeClass("formFieldWrong");
	});

	if (!response) {
		let message = "<%=_T('Error accessing server. Please try again later')%>";
		showError(message);
		DisableSubmit(false);
		return;
	}
	else if(response.code !== 0) {
		if(CheckIfLoginRetryNeeded(response)) {
			var button = $("form[name='formLogin']").find("#submitButton");
			button[0].disabled = false;	// we need to enable the button before trying to login again, else it will ignore us
			button.click();
			return;
		}
		ShowCaptchaContainer(false);

		if(response.newPasswordError) {
			$("#password_checks_list").html("");
			for(let checkElem of makePasswordsCheckUI(response.newPasswordErrorDetails)) {
				$("#password_checks_list").append(checkElem);
			}
			response.message = {
				message: response.message,
				field_id: "new_password"
			};
			showError(response.message);
			$("#new_password").focus();
		}
		else {
			showError(response.message);
			BackToUserButton();
		}
	}
	else {
		$("#backToUserButton").toggle(false);
		sessionStorage.loggedFromForm = true;
		const currentUrl = document.location;
		let newLocation;
		switch(window.location.hash) {
				case "#/webdesigner":
					const protocolAndDomain = currentUrl.protocol + "//" + currentUrl.host;
					const webDesignerPath = "/webdesigner";
					newLocation = protocolAndDomain + webDesignerPath;
				break;
				case "#/agent":
				default:
					newLocation = currentUrl.toString().replace(/apps.*/, "apps/agent/" + response.languageId);
		}
		document.location = newLocation;
	}
}

/*
	Funcion encargada de validar si el captcha puede ser utilizado, sino retorna error.

*/
function CanRenderCaptcha (captcha_config)
{
	let resp = {success: false};
	// Valida que exista conexion a internet en el navegador, ya que el captcha en el navegador funciona contra los servidores de google,
	// TODO: Esta funcion NO esta funcionando correctamente, hay que ver si se encuentra alguna forma de detectar la conexión a internet.
	// const on_line = inConcert.Utils.test_internet_connection();
	// if (!on_line) {
	// 	resp.status = -307;
	// 	return resp;
	// }

	// Valida que haya configurado (valor obtenido desde la DB del server)
	if (!captcha_config) {
		resp.status = -300;
		return resp;
	}
	// Valida que exista la clave publica.
	if (!captcha_config.publicKey) {
		resp.status = -302;
		return resp;
	}
	// Is ok
	resp.success = true;
	return resp;
}

/*
	Funcion encargada de visualizar u ocultar el markup contenedor del captcha.
*/
function ShowCaptchaContainer (show)
{
	const $submit_container = $("#submit_container");
	const $captcha_container = $("#captcha_container");
	if (show) {
		DisableSubmit(true);
		$captcha_container.show();
		$submit_container.hide();
	}
	else {
		DisableSubmit(false);
		$captcha_container.hide();
		$submit_container.show();
	}
}

let m_captcha_wnd_id = null; // el handle de la instancia del captcha
const m_captcha_div_id = "captcha"; // este div se encuentra en login.html

function ResetCaptcha () {
	if (typeof(grecaptcha) === "undefined") {
		return false;
	}
	try {
		if (m_captcha_wnd_id !== null) {
			grecaptcha.reset(m_captcha_wnd_id);
			return true;
		}
	}
	catch(e) {
	}
	return false;
}

function IncludeCaptchaJS () {
	if (typeof(grecaptcha) !== "undefined") {
		return;
	}
	$("#loginCaptchaJSContainer").html('<script src="https://www.google.com/recaptcha/api.js?onload=onloadCallback&render=explicit" async defer></script>');
}

function RemoveCaptchaJS () {
	$("#loginCaptchaJSContainer").html("");
}

let m_captcha_render_timeout = null; // timeout utilizado para instanciar el captcha con reintentos de forma asincronica.
/*
	Funcion que es invocada en el archivo "authenticaciton.js"
*/
function RenderCaptcha (captcha_config, callback = $.noop)
{
	let resp_render_captcha = CanRenderCaptcha(captcha_config);
	if (!resp_render_captcha.success) {
		RemoveCaptchaJS();
		callback(false, resp_render_captcha);
		return;
	}

	if (m_captcha_render_timeout) {
		window.clearTimeout(m_captcha_render_timeout);
		m_captcha_render_timeout = null;
	}

	IncludeCaptchaJS();

	const timeout_ms = 500;
	let retries = 0;
	const max_retries = 10;
	var CaptchaRenderFn;

	// El captcha render tiene que estar dentro de un timeout con reintentos ya que el objeto "grecaptcha"
	// quizas aun no se haya instanciado.
	//
	CaptchaRenderFn = function()
	{
		retries++;
		if (retries > max_retries) {
			callback(false, {status: -303});
			return;
		}
		m_captcha_render_timeout = window.setTimeout(function()
		{
			if (typeof(grecaptcha) === "undefined") {
				// reintento ya que reCaptcha aun no se ha inicializado.
				CaptchaRenderFn();
				return;
			}

			// Si el captcha ya existe, unicamente tengo que resetearlo y visualizarlo
			const is_reseted = ResetCaptcha();
			if (is_reseted) {
				ShowCaptchaContainer(true);
				return;
			}

			let captchaParams = {
				// requerido
				sitekey: captcha_config.publicKey,
				// opcionales
				theme: captcha_config.theme, // light, dark
				hl: captcha_config.language,
				size: captcha_config.size // normal, compact
			};
			// callbacks
			captchaParams.callback = function(recaptcha_token) {
				callback(true, recaptcha_token);
			};
			captchaParams["expired-callback"] = function(err) {
				callback(false, {status: -305});
			};
			captchaParams["error-callback"] = function(err) {
				callback(false, {status: -306});
			};

			try {
				m_captcha_wnd_id = grecaptcha.render(m_captcha_div_id, captchaParams);
				ShowCaptchaContainer(true);
			}catch(e) {
				console.error("captcha error: " + e.message);
				callback(false, {status: -304, nativeErr: e});
			}

			m_captcha_render_timeout = null;
		}, timeout_ms);
	};

	CaptchaRenderFn();
}

function loginProcessMessageMfa(data) 
{
	$("#mfa_code_container")
		.html("".concat(
			'<label class="mfaCodeMessage" id="mfa_code_message"></label>',
			'<label class="loginFormLabel"><%=_T("Code")%><%=_T(":")%></label>',
			'<input id="mfa_code" class="loginFormInput requiredField" type="text" />'
		))
		.show();
	$("#mfa_code_message").text(data.message);
	$("#mfa_code")
		.pincodeInput({
			inputs : data.length,
			hidedigits : false
		})
		.data("plugin_pincodeInput")
			.focus();
}

function loginProcessMailMfa(data, finalStepFn)
{
	data.message = FORMAT("<%=_T('We sent you an email to {0} with authentication code.')%>", 
		data.address);
	loginProcessMessageMfa(data, finalStepFn);
}

function loginProcessMfa(data, finalStepFn)
{
	hideErrors();
	$("body")
		.addClass("mfaMode")
		.addClass(data.mfa.toLowerCase() + "Mfa")
		.data("mfa", data);
	ShowCaptchaContainer(false);
	$("#username")
		// .css("color", "DarkGray")
		.prop("disabled", true);
	$("#password_container").hide();
	$("#cancelButton")
		.show()
		.off("click")
		.on("click", function() {
			window.location.reload();
		});
	$("#submitButton")
		.show()
		.off("click")
		.on("click", function() {
			loginMfaFinalStep(data, finalStepFn);
		});
	switch(data.mfa) {
		case "MAIL":
			loginProcessMailMfa(data);
			break;
	}
}

function loginMfaMessageFinalStep(data, finalStepFn) 
{
	let code = $("#mfa_code").val();
	if (!code || code.length !== data.length) {
		return loginResponseSuccess({
			code : -1,
			message : "<%=_T('Must enter a code.')%>",
			retry : true
		});
	}
	finalStepFn({
		code : code
	});
}

function loginMfaMailFinalStep(data, finalStepFn)
{
	loginMfaMessageFinalStep(data, finalStepFn);
}

function loginMfaFinalStep(data, finalStepFn)
{
	hideErrors();
	let waiting_msg = "<%=_T('Logging in')%>";
	DisableSubmit(true, waiting_msg);
	switch(data.mfa) {
		case "MAIL":
			loginMfaMailFinalStep(data, finalStepFn);
			break;
	}
}

function revertMessageMfa(data) {
	$("#mfa_code_container")
		.html("")
		.hide();
}

function revertMailMfa(data) {
	revertMessageMfa(data);
}

function revertMfa()
{
	let data = $("body").data("mfa");
	$("body")
		.removeClass("mfaMode")
		.removeClass(data.mfa.toLowerCase() + "Mfa")
		.data("mfa", null);
	$("#password_container").show();
	$("#cancelButton").hide();
	$("#submitButton")
		.off("click")
		.on("click", function() {
			SignIn();
		});
	switch(data.mfa) {
		case "MAIL":
			revertMailMfa(data);
			break;
	}
}

function BackToUserButton()
{
	$("#username_container").toggle();
	$("#password_container").toggle(false);
	$("#change_password_container").toggle(false);
	$(".loginFormContainer").removeClass("change_password_main_container");
	$("#loginTitleContainer").toggle(true);
	$("#loginFormTitle").toggle(true);
	$("#loginFormSubtitle").text("<%=_T('Session login')%>");
	$("#submitButton").removeClass("btn-dark");
	$("#submitButton").text("<%=_T('CONTINUE')%>");
	$("#backToUserButton").toggle(false);
	$("#password_expiration_container").toggle(false);
	$("#password").val("");
	$("#new_password").val("");
	$("#new_password_confirm").val("");
	ShowCaptchaContainer(false);
	this.m_vcc_info = undefined;
	this.m_userInfo = undefined;
}

function Submit () {
	if (this.m_vcc_info) {
		SignIn(this.m_vcc_info);
		return;
	}
	GetVccAuthentication();
}

/*
	Funcion que realiza el primer paso del inicio de sesion.
	Utiliza solo el username y el vcc para recuperar info sobre el tipo de authenticacion que aplica al vcc
*/
function GetVccAuthentication()
{
	hideErrors();

	if ($("#username").val().trim() === "") {
		let message = "<%=_T('Field is required')%>";
		showError(message);
		return false;
	}

	let username = $("#username").val().trim();
	if (username === "admin") {
		username += "@system"; //user admin is assumed to be in system vcc
	}
	let userAndVcc = username.split("@");
	if (userAndVcc.length !== 2) {
		$("#username").next("img").show();
		let message = "<%=_T('Username must include only one @')%>";
		showError(message);
		return false;
	}

	username = userAndVcc[0];
	let vcc = userAndVcc[1];

	GetAuthToken({
		username : username,
		vcc : vcc,
		callback : GetAuthTokenResponseSuccess
	});
}

/*
	Funcion que realiza el inicio de sesion.
	must_auto_login: este parametro es utilizado en "true", en el caso de logins desde el api.
*/
function SignIn (params = {})
{
	let info = params.info;

	//optional params
	let data = params.data || null;
	let autoLogin = params.autoLogin || false;
	let mustChangePassword = params.mustChangePassword;
	
	hideErrors();
	if($("#submitButton")[0].disabled) {
		return false;
	}
	
	if ($("#password").val() === "") {
		$("#password").next("img").show();
		let message = "<%=_T('Field is required')%>";
		showError(message);
		return false;
	}

	if(mustChangePassword) {
		if ($("#new_password").val() === "") {
			$("#new_password").focus().next("img").show();
			let message = "<%=_T('Field is required')%>";
			showError(message);
			return false;
		}
		if ($("#new_password_confirm").val() === "") {
			$("#new_password_confirm").focus().next("img").show();
			let message = "<%=_T('Field is required')%>";
			showError(message);
			return false;
		}
		if ($("#new_password").val() !== $("#new_password_confirm").val()) {
			$("#new_password_confirm").focus().next("img").show();
			let message = "<%=_T('New Password and Password Confirm does not match')%>";
			showError(message);
			return false;
		}
	}

	let username = $("#username").val().trim();
	if (username === "admin") {
		username += "@system"; //user admin is assumed to be in system vcc
	}
	let userAndVcc = username.split("@");
	if (userAndVcc.length !== 2) {
		$("#username").next("img").show();
		let message = "<%=_T('Username must include only one @')%>";
		showError(message);
		return false;
	}

	username = userAndVcc[0];
	let vcc = userAndVcc[1];
	let password = $("#password").val();
	let newPassword = $("#new_password").val();

	let waiting_msg = "<%=_T('Logging in')%>";
	DisableSubmit(true, waiting_msg);
	DoLogin({
		username : username,
		password : password,
		mustChangePassword : mustChangePassword,
		newPassword : newPassword,
		vcc : vcc,
		info : info,
		data : data,
		autoLogin : autoLogin,
		callback : loginResponseSuccess,
		mfaCallback : loginProcessMfa
	});
}

/*
	Funcion que valida si se debe de realiza un auto login.
	Caso de login desde el api.
*/
function MustAutoLogin () {
	let auto_login = {
		enabled: false
	};

	let query = window.location.search.substring(1);
	let vars = query.split("&");

	for (let i=0; i < vars.length; i++) {
		let pair = vars[i].split("=");
		// If first entry with this name
		if (typeof auto_login[pair[0]] === "undefined") {
			auto_login[pair[0]] = decodeURIComponent(pair[1]);
		} 
		// If second entry with this name
		else if (typeof auto_login[pair[0]] === "string") {
			let arr = [ auto_login[pair[0]],decodeURIComponent(pair[1]) ];
			auto_login[pair[0]] = arr;
		} 
		// If third or later entry with this name
		else {
			auto_login[pair[0]].push(decodeURIComponent(pair[1]));
		}
	} 

	if (auto_login.username && auto_login.vcc && auto_login.password) {
		auto_login.enabled = true;
	}
	return auto_login;
}

$(document).ready(function()
{
	//removes potential garbage from previous session
	sessionStorage.removeItem("loggedFromForm");
	sessionStorage.removeItem("alreadyInitialized");

	let api_server = new inConcert.AgentAPIServer("login");
	window.notifyLoginError = function(message = "error doing login", result = "FAIL") {
		api_server.UserLoginFailed(result, message);
	};

	$(".requiredField").bind("keypress", function(event) {
		return OnFieldKeyDown(this, event);
	});

	$(".requiredField:first").focus();

	$("#submitButton").bind("click", function() {
		Submit();
		return false;
	});

	$("#backToUserButton").bind("click", function() {
		BackToUserButton();
		return false;
	});

	$("#cancel_password_change").bind("click", function() {
		BackToUserButton();
		return false;
	});

	$("#changePasswordButton").bind("click", function() {
		ShowPasswordChange();
		return false;
	});

	// TODO: revisar si es que no tiene que pedir el captcha de todas formas!! es decir sin auto login
	const auto_login = MustAutoLogin();
	if (auto_login.enabled) {
		$("#username").val(auto_login.username + "@" + auto_login.vcc);
		$("#password").val(auto_login.password);
		SignIn({
			data : auto_login.data,
			autoLogin : true
		});
	}
});