/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
  strict:false, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark: false,
  smarttabs:true, scripturl:true, esversion: 11 */

/* global inConcert, _, URLSearchParams */

// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	const format = inConcert.Utils.format;

	/**
	Class FoldersPopup.
	*/
	function FoldersPopup (params)
	{
		FoldersPopup.uber.constructor.call( this );
		this.clickEventName = this.clickEventName + "_batch_filter";
		this.ownerId = "mail_folders_popup_" + inConcert.Utils.generateGuid();
		this.folderId = this.ownerId + "_folder_id";
		this.saveId = this.ownerId + "_save";
		this.closeId = this.ownerId + "_close";
		this.params = params;
		this.interaction = params.interaction;
		this.jquery_item_selector = "[row_item]";
		this.jquery_txt_search_selector = "input[txt_search='true']";
		this.selected_item_id = null;
		this.from_app = params.from_app;
		this.cached_campaign_labels = null;
		this.cached_folders = params.cached_folders || null;
		this.canReposition = true;
	}

	inConcert.Utils.inherit(FoldersPopup, inConcert.PopupWindow);

	/*******************************************************************
	 * Metodo que crea el markup contenedor de la ventana
	 */
	FoldersPopup.prototype.createMarkup = function()
	{
		$("#" + this.ownerId).remove();

		var $node = $("".concat(
			"<div id='", this.ownerId, "' class='windowMoveToFolderContainer windowContainer' hidden>",

				"<div id='", this.ownerId, "_waiting' class='windowWaitingContainer' hidden>",
					"<div message='true' class='windowWaitingMessage'></div>",
					"<div class='windowWaitingAnimation'></div>",
					"<div class='windowWaitingOverlay'></div>",
				"</div>",

				"<div class='windowTop'>",
					"<div class='windowTitle windowMoveToFolder'>",
						"<div id='", this.ownerId, "_menu_title' class='windowMoveToFolderTitleText'>", "<%=_T('Move to Folder')%>", "</div>",
						"<img id='", this.ownerId, "_menu_close' src='/inconcert/apps/agent/images/blank.png' class='windowMoveToFolderButClose' hidden />",
					"</div>",
				"</div>",

				"<div id='", this.ownerId, "_menu_message' class='windowContainerMessages messageError' hidden>",
					"<a id='", this.ownerId, "_menu_close_message' class='windowMessagesClose' href='javascript:void(0);'>", "<%=_T('Close')%>", "</a>",
					"<div class='windowMessagesText'>",
						"<label class='windowMessagesTextAux'></label>",
					"</div>",
				"</div>",

				"<div class='windowMiddle'>",
					"<div id='", this.ownerId, "_menu_tab_items' class='windowTabsMenu' hidden>",
						"<span tab_item='items' class='windowTabsMenuItem'>", "<%=_T('Folders')%>", "</span>",
					"</div>",

					// items list container
					"<div tab_container='items' class='windowTargetTransferContainer'>",
						"<input txt_search='true' class='windowTargetSearch' type='text' disabled='true'/>",
						"<div data_container='true' class='windowTargetTransferControl' hidden>",
							"<div list_container='true' class='windowTargetTransferScrollArea'></div>",
							"<div class='windowTargetFooter'>",
								"<div total='true' class='windowTargetTotal'></div>",
								"<div detail='true' class='windowTargetDetailContainer'>",
									"<div detail_list='true' class='windowTargetDetailInfo'></div>",
								"</div>",
							"</div>",
						"</div>",
						"<div id='", this.ownerId, "_current_folder_section' class='moveToFolderTopContainer' hidden>",
							"<label class='moveToFolderTopLabel'>", "<%=_T('Moved To Folder')%>", "<%=_T(':')%>", "</label>",
							"<input id='", this.ownerId, "_current_folder' class='moveToFolderTopInput' type='text' disabled='true' />",
						"</div>",
						"<div not_found_container='true' class='windowTargetNoResults' hidden>",
							"<span class='windowTargetNoResultsText'></span>",
						"</div>",
					"</div>",
				"</div>",

				"<div class='windowBottom'>",
					"<input id='", this.saveId, "' class='buttonColorBlack butWindowOk' disabled='true' type='button' value='", "<%=_T('Move')%>", "'/>",
					"<input id='", this.closeId, "' class='buttonColorGray butWindowClose' type='button' value='", "<%=_T('Close')%>", "'/>",
				"</div>",

			"</div>"));

		$("body").append($node);

		return $node;
	};

	/*******************************************************************
	 *
	 */
	FoldersPopup.prototype.initialize = function()
	{
		this.data.container = this.createMarkup();
		this.parent_save = this.data.callbacks.save || $.noop;
		this.setEvents();
		this.load();
	};

	FoldersPopup.prototype.load = function()
	{
		var self = this;

		this.ShowWaiting();

		var campaign = inConcert.app.getCampaign(this.interaction.campaign);
		if (!campaign) {
			return false;
		}
		campaign.loadLabels(function(labels/*, error*/) {
			self.cached_campaign_labels = labels;
		});

		if (!this.cached_folders) {
			window.setTimeout(function()
			{
				self.interaction.loadFoldersAccount(function(success, list) {
					if (success) {
						self.FillList(list, function() {
							self.cached_folders = list;
							self.OnFinishLoad();
						});
					}
					else {
						var message_error = list;
						self.ShowStatusMessage(message_error, "Error", true, 5000);
						self.HideList();
						self.HideWaiting();
					}
				});
			}, 500);
		}
		else {
			this.FillList(this.cached_folders, function() {
				self.OnFinishLoad();
			});
		}
	};

	FoldersPopup.prototype.ShowCurrentAssigned = function(item)
	{
		if (!item) {
			return;
		}
		var $folder_section = $("#" + this.ownerId + "_current_folder_section");
		var $current_folder = $("#" + this.ownerId + "_current_folder");
		$current_folder.val(this.GetItemName(item));
		$folder_section.show();
	};

	FoldersPopup.prototype.HideCurrentAssigned = function()
	{
		var $folder_section = $("#" + this.ownerId + "_current_folder_section");
		$folder_section.hide();
	};

	FoldersPopup.prototype.FillList = function(list, callback)
	{
		var self = this;

		callback = callback || $.noop;

		var $list = this.$GetTabContainer().find("[list_container='true']");
		$list.html("");

		var last_moved_folder = this.interaction.getLastMovedFolder();
		var can_add = function(item) {
			// En la lista de folders, no muestra el folder a la cual la interaccion fue movida.
			if (last_moved_folder && last_moved_folder.to === item) {
				self.ShowCurrentAssigned(item);
				return false;
			}
			return true;
		};

		this.total_rows = 0;
		var sorted_list = inConcert.Utils.sortArray(list);

		// se carga en un foreach que corre dentro de un timeout para que el navegador no quede colgado
		inConcert.Utils.foreach_timeout({
			rows: sorted_list,
			append_fn : function(item)
			{
				if (!can_add(item)) {
					return;
				}
				self.total_rows++;
				var dom_id = self.GetRadioDomId( item );
				var search_text = self.MakeSearchItemText( item );
				var item_name = self.GetItemName( item );

				var $label = $("".concat(
					"<div item_name='" + item_name + "' row_item='true' for='", dom_id ,"' search_text='", search_text, "' class='targetListItem'>",
						"<input id='", dom_id ,"' selector='true' type='radio' name='entity' class='targetItemCheck' />",
						"<div class='targetItemData entityItemData'>",
							"<label for='", dom_id ,"' class='targetItemName'>", item_name, "</label>",
							"<label for='", dom_id ,"' class='targetItemInfo' style='display:none;'>", item_name, "</label>",
						"</div>",
					"</div>"
				));
				$label.data("item_data", item);
				$list.append($label);
			},
			complete_fn: function() {
				callback();
			}
		});
	};

	FoldersPopup.prototype.GetRadioDomId = function( id )
	{
		return this.ownerId + "_folder_" + inConcert.Utils.getNormalizedId(id);
	};

	FoldersPopup.prototype.GetItemName = function( item )
	{
		return item;
	};

	FoldersPopup.prototype.MakeSearchItemText = function( item )
	{
		return this.GetItemName(item);
	};

	FoldersPopup.prototype.ready = function()
	{
		var current_folder = this.params.current_folder;
		$("#" + this.folderId).val(current_folder).focus();
	};

	FoldersPopup.prototype.close = function(params) {
		this.data.container.remove();
		this.finalize(params);
	};

	/*******************************************************************
	 * Metodo que setea eventos de los controles
	 */
	FoldersPopup.prototype.setEvents = function()
	{
		var self = this;

		//---------------
		$("#" + this.saveId)
			.unbind("click")
			.click(function() {
				self.MoveToFolder();
			});
		//---------------
		$("#" + this.closeId).unbind("click").click(function() {
			self.close();
		});

		var $tab_container = this.$GetTabContainer();

		// click en la row del item
		$tab_container
			.off("click", this.jquery_item_selector)
			.on("click", this.jquery_item_selector, function () {
				self.OnSelectedItem($(this)) ;
			});
		// dbl click en la row del item
		$tab_container
			.off("dblclick", this.jquery_item_selector)
			.on("dblclick", this.jquery_item_selector, function () {
				self.EnableApplyAction();
				self.MoveToFolder();
			});
		// evento keyup del text de search
		$tab_container
			.off("keyup", this.jquery_txt_search_selector)
			.on("keyup", this.jquery_txt_search_selector, function() {
				var text = $(this).val();
				if ( self.timeout ) {
					window.clearTimeout( self.timeout );
					self.timeout = null;
				}
				self.timeout = window.setTimeout( function() {
					self.Filter( text );
					window.clearTimeout( self.timeout );
					self.timeout = null;
				}, 500 );
			});
	};

	FoldersPopup.prototype.GetSelectedItem = function()
	{
		var $tab_container = this.$GetTabContainer();
		var $chk_label = $tab_container.find("div[list_container='true'] input[name='entity']:checked");
		if ($chk_label.length === 0) {
			return;
		}
		var item_data = $chk_label.parent().data("item_data");
		return item_data;
	};

	FoldersPopup.prototype.MoveToFolder = function()
	{
		var self = this;

		var item_data = this.GetSelectedItem();
		if (!item_data) {
			this.ShowStatusMessage("<%=_T('Error in selected item.')%>", "Error", true, 3500);
			return;
		}

		var OnMoved = this.data.callbacks.on_moved || $.noop;
		var item_name = this.GetItemName(item_data);

		var callback = function(success, message)
		{
			var msg_type, time_ms;
			self.HideWaiting();
			if (success) {
				var $current_folder = $("#" + self.ownerId + "_current_folder");
				var from = $current_folder.val();
				var to = item_data;
				OnMoved(from, to);

				self.DisableApplyAction();
				msg_type = "Info";
				time_ms = 1500;
				message  = format("<%=_T('Successfully moved to folder to folder {0}')%>", item_name);

				window.setTimeout(function() {
					self.close();
				}, time_ms + 150); // incremento 150 para que se cierre, 150 ms despues que se oculta el mensaje de forma automatica
			}
			else {
				msg_type = "Error";
				time_ms = 3500;
			}
			self.ShowStatusMessage(message, msg_type, true, time_ms);
		};

		this.ShowWaiting(format("<%=_T('Please wait moving to folder {0}')%>", item_name));

		window.setTimeout(function()
		{
			var data = {
				toFolder: item_data,
				appId   : self.from_app
			};
			var $assign_label = $("#" + self.ownerId + "_assign_label");
			if ($assign_label.length > 0) {
				var label = $assign_label.data("label_data");
				data.labelInfo = {
					label: label,
					action: $assign_label.prop("checked") ? "add" : "remove"
				};
			}
			self.interaction.moveToFolder(data,
				function(success, err) {
					callback(success, err);
				}
			);
		}, 500);
	};

	FoldersPopup.prototype.OnSelectedItem = function( $new_selected_item /* radio buttom */ )
	{
		if ( this.selected_item_id !== null ) {
			$("#" + this.selected_item_id).parent().removeClass("selected");
		}
		this.selected_item_id = $new_selected_item.attr("for");
		$("#" + this.selected_item_id).prop("checked", true).parent().addClass("selected");
		this.ShowDetail();
		this.EnableApplyAction();
	};

	FoldersPopup.prototype.ShowStatusMessage = function(text, type, closeDisabled, timeout_ms )
	{
		var $section = $("#" + this.ownerId + "_menu_message");
		$section.removeClass().addClass("windowContainerMessages").addClass("message" + type);
		$section.show().find(".windowMessagesTextAux").text(text);
		if (timeout_ms) {
			window.setTimeout(function() {
				$section.hide();
			}, timeout_ms);
		}
		var $close = $("#" + this.ownerId + "_menu_close_message");
		if (closeDisabled) {
			$close.hide();
		}
		else {
			$close.show();
		}
	};

	/*******************************************************************
	* funcion que oculta mensaje
	*/
	FoldersPopup.prototype.HideStatusMessage = function() {
		$("#" + this.ownerId + "_menu_message").hide();
	};

	/*******************************************************************
	* Metodo que ...
	* @param message
	*/
	FoldersPopup.prototype.ShowWaiting = function( message )
	{
		message = message || "<%=_T('Loading, wait moment please')%>";
		var $waiting = $("#" + this.ownerId + "_waiting");
		$waiting.find("[message='true']").text(message);
		$waiting.show();
	};

	/*******************************************************************
	* Metodo que ...
	*/
	FoldersPopup.prototype.HideWaiting = function()
	{
		$("#" + this.ownerId + "_waiting").hide();
		this.setPosition();
	};

	FoldersPopup.prototype.Disable = function()
	{
		var $tab_container = this.$GetTabContainer();
		var $search = $tab_container.find(this.jquery_txt_search_selector);
		this.HideList( "<%=_T('Assign Disabled')%>" );
		$search.prop("disabled", true);
	};

	FoldersPopup.prototype.Enable = function()
	{
		var $tab_container = this.$GetTabContainer();
		var $search = $tab_container.find(this.jquery_txt_search_selector);
		this.ShowList();
		$search.prop("disabled", false).focus();
	};

	FoldersPopup.prototype.Filter = function( text )
	{
		var self = this;
		var $tab_container = this.$GetTabContainer();
		var $list_container = $tab_container.find("[list_container='true']");

		if ( text.trim().length === 0 ) {
		// si no hay texto buscado en el filtro, no filtro la row
			$list_container.find(this.jquery_item_selector).show();
			this.ShowList();
			return;
		}
		var hidden_rows = 0;
		$list_container.find(this.jquery_item_selector).each(function(_, row)
		{
			var $row = $(row);
			var row_value = $row.attr("search_text");
			var search_regexp = self.MakeSearchRegexp(text);
			if ( row_value.search(search_regexp) === -1 ) {
				$row.hide();
				hidden_rows++;
			}
			else {
				$row.show();
			}
		});
		if ( this.total_rows === hidden_rows ) {
			this.HideList();
		}
		else {
		// muestra la lista
			this.ShowList();
			this.ShowDetail();
		}
	};

	FoldersPopup.prototype.EnableApplyAction = function()
	{
		$("#" + this.saveId).prop("disabled", false);
	};

	FoldersPopup.prototype.DisableApplyAction = function()
	{
		$("#" + this.saveId).prop("disabled", true);
	};

	FoldersPopup.prototype.MakeSearchRegexp = function( word )
	{
		if (typeof word !== "string") {
			return "";
		}
		var search_pattern = inConcert.Utils.removeSpecialChars(word);
		return new RegExp(search_pattern.replace(/[^\w\s]/gi, ""), "i");
	};

	FoldersPopup.prototype.SetTotals = function()
	{
		var text = this.total_rows === 1 ? "<%=_T('{0} folder.')%>" : "<%=_T('{0} folders.')%>";
		var $tab_container = this.$GetTabContainer();
		$tab_container.find("[total='true']").text(format(text, this.total_rows));
	};

	FoldersPopup.prototype.ShowDetail = function()
	{
		var $detail_list = this.$GetTabContainer().find("[detail_list='true']");
		if ( !$detail_list || $detail_list.length === 0 ) {
			return;
		}
		if ( !this.selected_item_id ) {
			$detail_list.html("");
			return;
		}
		var $item_container = $("#" + this.selected_item_id).parent();
		if ( !$item_container.is(":visible") ) {
			$detail_list.html("");
			return;
		}
		var item_data = $item_container.data("item_data");
		if ( !item_data ) {
			$detail_list.html("");
			return;
		}
		var $detail = this.$MakeDetail(item_data);
		$detail_list.html($detail);
	};

	FoldersPopup.prototype.$MakeDetail = function(item_data)
	{
		var html = "";
		var count_campaigns  = Object.keys(this.cached_campaign_labels || {} ).length;

		if (count_campaigns === 0) {
			return html;
		}
		var folder_id = item_data.toUpperCase();
		var label = this.cached_campaign_labels[folder_id];
		if (!label) {
			return html;
		}
		if (!this.interaction.hasPermission("LABEL", this.from_app)) {
			return html;
		}

		var dom_id = this.ownerId + "_assign_label";
		var $html = $(
			"<div class='assignLabelCampaignContainer'>" +
				"<label for='" + dom_id + "' class='assignLabelCampaignLabel'>" + "<%=_T('Assign Campagin Label')%>" + "</label>" +
				"<input id='" + dom_id +  "' type='checkbox' class='assignLabelCampaignCheckbox' />" + 
			"</div>"
		);

		var $chk = $html.find("[id='" + dom_id + "']");
		$chk.data("label_data", label);
		$chk.prop("checked", (this.interaction.folderId || "").toUpperCase() === folder_id.toUpperCase());
		return $html;
	};

	FoldersPopup.prototype.OnFinishLoad = function()
	{
		this.is_loaded = true;
		this.SetTotals();
		if ( this.total_rows === 0 ) {
			this.Disable();
		}
		else {
			this.Enable();
		}
		this.ShowList();
		this.HideWaiting();
		(this.data.callbacks.on_load || $.noop)(this.cached_folders);
	};

	/*******************************************************************
	* Metodo que ...
	* @param message
	*/
	FoldersPopup.prototype.HideList = function( message )
	{
		message = message || this.not_found_default_msg || "<%=_T('Not found.')%>";
		var $tab_container = this.$GetTabContainer();
		$tab_container.find("[data_container='true']").hide();
		var $message_container = $tab_container.find("[not_found_container='true']");
		$message_container.find("span").text(message);
		$message_container.show();
		this.HideCurrentAssigned();
		this.DisableApplyAction();
	};

	/*******************************************************************
	* Metodo que ...
	*/
	FoldersPopup.prototype.ShowList = function()
	{
		var $tab_container = this.$GetTabContainer();
		var $data = $tab_container.find("[data_container='true']");
		$data.show();
		var $not_found = $tab_container.find("[not_found_container='true']");
		$not_found.hide();
		var last_moved_folder = this.interaction.getLastMovedFolder();
		if (last_moved_folder) {
			this.ShowCurrentAssigned(last_moved_folder.to);
		}
	};

	FoldersPopup.prototype.$GetTabContainer = function()
	{
		var $tab_container = $("#" + this.ownerId).find("[tab_container='items']");
		return $tab_container;
	};


	/////////////////////////////////////////////////////////////////
	const URI_REPOSITORY_GET = "/inconcert/apps/repository/download/";
	var URI_DOMAIN;
	var COMPOSING_MAIL_SIGNATURE_CONTAINER_ID;
	var WHITE_SPACE = " ";
	var CTRLF = "<DIV><BR/></DIV>";
	var COLON = "<%=_T(':')%>";
	var MAX_UPLOAD_IMAGE_SIZE_BYTES  = 524288;  // bytes (512 Kb)
	var MAX_UPLOAD_IMAGES_SIZE_BYTES = 1048576; // bytes (1024 Kb)
	// Estas dimensiones deben de coincidir con lo que se ha definido en "base_panel.js" funcion "openInteractionClient"
	const DEFAULT_WINDOW_HEIGHT_PX = 700;
	const DEFAULT_WINDOW_WIDTH_PX = 1100;

	/**
	Class MailClientWindow.
	Mail Client Handler
	*/
	function MailClientWindow (interaction, options)
	{
		URI_DOMAIN = inConcert.app.getBaseDomain();
		this.m_spellchecker = null;
		this.m_composingMail = false;
		this.m_uploadedFiles = [];
		this.m_deletedCurrentEventAttachments = {}; // aca se encuentran los deletes al realizar forward, si el mail original ya tenia adjuntos(aca van solo esos adjuntos, los nuevos no)
		this.m_forwardOrReplyAttachmentsEvent = null;
		this.m_interaction = interaction;
		this.m_action = null;
		this.m_editor_saved_selection = null;
		this.m_auto_search_popup = null;
		// Must call constructor after having set m_interaction. The base class depends on that.
		var success = MailClientWindow.uber.constructor.call( this, options );
		this.setHorizontalResizingLimits(DEFAULT_WINDOW_WIDTH_PX);
		this.setVerticalResizingLimits(DEFAULT_WINDOW_HEIGHT_PX);
		this.folders_popup = null;
		if (!success) {
			return;
		}
	}

	inConcert.Utils.inherit( MailClientWindow, inConcert.ClientWindow );

	MailClientWindow.prototype.initialize = function( options )
	{
		this.is_spam = options.spam || false;
		
		if(this.m_interaction.interactionData.fromAPI) {
			this.createHtmlEditor(null);
		}

		MailClientWindow.uber.initialize.call( this, options ); //invokes the base class method

		var self = this;

		this.setPeopleFrom();

		if (this.is_readonly === true) { // variable in base class
			$("body").addClass("clientReadOnly");
		}

		if ( this.is_spam === true ) {
			this.is_spam = true;
			this.change_client_state("spam");
		}
		else {
			this.change_client_state("connected");
		}

		this.m_timer = window.setInterval(function() {
			self.onTimer();
		}, 1000);

		if ( options.reply === true ) {
			this.navigateLast();
			if (this.is_visible_related_events()) {
				this.closeRelated();
			}
			this.replyCurrentMail();
		}
		else {
			if (this.m_interaction.state === "PENDING") {
				this.__setComposingMail(true);
				this.updateSideBar();
				var mailTo = this.m_interaction.contactAddress;
				if (this.m_interaction.contact && this.m_interaction.interactionData.contactName) {
					mailTo = format("'{0}' <{1}>", this.m_interaction.interactionData.contactName, this.m_interaction.contactAddress);
				}
				else {
					mailTo = this.m_interaction.getCompleteAddress();
				}
				if (mailTo) {
					$("#mailTo").val(mailTo);
				}
				$("body").removeClass("mailClientRead").addClass("mailClientWrite");
				if(!this.m_interaction.interactionData.fromAPI) {
					this.createHtmlEditor(null);
				}
				window.setTimeout(function() {
					$("#mailTo").focus();
				}, 0);
			}
		}
	};

	MailClientWindow.prototype.getMenuContextualActions = function()
	{
		const actions = {
		// acciones solo para el cliente
			attachfile   : true,
			print        : true,
		// acciones sobre la interaccion
			comment      : true,
			label        : true,
			emit_ticket  : true,
			search_ticket: true,
			queue        : true,
			transfer     : true
		// estas NO tienen que visualizarse en el menu, ya que las mismas se encuentran en la toolbar
		};
		return actions;
	};

	MailClientWindow.prototype.getWindowDefaultTitle = function()
	{
		return "<%=_T('E-mail Window')%>";
	};

	MailClientWindow.prototype.change_client_state = function( new_state )
	{
		new_state = MailClientWindow.uber.change_client_state.call( this, new_state );

		var $body = $("body");
		var $state_flap = $(".clientStateFlap");
		$body.removeClass("clientSpam");

		if ( new_state === "spam" ) {
			$state_flap.text("<%=_T('SPAM')%>");
			$body.addClass("clientSpam");
		}
	};

	MailClientWindow.prototype.onChangedClientState = function(new_state)
	{
		MailClientWindow.uber.onChangedClientState.call(this, new_state);
		this.setEventsByState(new_state);
	};

	MailClientWindow.prototype.clientUnusable = function() {
		MailClientWindow.uber.clientUnusable.call(this); //invokes the base class method
	};

	MailClientWindow.prototype.navigateNext = function() {
		this.m_currentEvent = this.m_interaction.getNextRelatedEventIndex(this.m_currentEvent);
		this.readAndOpenEvent();
	};

	MailClientWindow.prototype.navigatePrev = function() {
		this.m_currentEvent = this.m_interaction.getPreviousRelatedEventIndex(this.m_currentEvent);
		this.readAndOpenEvent();
	};

	MailClientWindow.prototype.navigateLast = function(callback = $.noop) {
		var last_index = this.m_interaction.getAllRelatedEventCount() - 1;
		if (this.m_interaction.is_auto_response_event(last_index)) {
			last_index = this.m_interaction.getPreviousRelatedEventIndex(last_index);
		}
		this.m_currentEvent = last_index;
		this.readAndOpenEvent(undefined, callback);
	};

	MailClientWindow.prototype.navigateEvent = function(index, callback = $.noop) {
		this.m_currentEvent = index;
		this.readAndOpenEvent(undefined, callback);
	};

	MailClientWindow.prototype.navigateToBestEventByChannel = function(callback = $.noop) {
		this.navigateLast(callback);
	};

	MailClientWindow.prototype.updateNavigationButtons = function()
	{
		MailClientWindow.uber.updateNavigationButtons.call( this ); //invokes the base class method

		var $previous = $("#butRelatedPrevious");
		var $next     = $("#butRelatedNext");

		if ( !this.m_interaction.getNextRelatedEventIndex( this.m_currentEvent ) ) {
			$next.prop("disabled", true ).addClass("butRelatedDisabled");
		}
		else {
			$next.prop("disabled", false ).removeClass("butRelatedDisabled");
		}
		var first_index_event = this.m_interaction.getFirstIndexEvent();
		if ( this.m_currentEvent === first_index_event ) {
			$previous.prop("disabled", true ).addClass("butRelatedDisabled");
		}
		else {
			$previous.prop("disabled", false ).removeClass("butRelatedDisabled");
		}
	};

	MailClientWindow.prototype.setEventsHandlers = function(){
		var self = this;

		MailClientWindow.uber.setEventsHandlers.call( this ); //invokes the base class method

		// navigation events
		$("#butRelatedNext")
			.unbind("click.nav")
			.bind("click.nav", function() {
				self.navigateNext();
			});

		$("#butRelatedPrevious")
			.unbind("click.nav")
			.bind("click.nav", function() {
				self.navigatePrev();
			});

		// Los demas eventos de la toolbar se setean en la funcion "setEventsByState", la cual es invocada cuando hay un cambio de estado de la interaccion en la funcion "onChangedClientState".
	};

	/*
		Esta funcion asigna eventos en las acciones de Read y Write dependiendo de la App y del estado de la interaccion.
	*/
	MailClientWindow.prototype.setEventsByState = function(state)
	{
		const self = this;

		var log = function( action_name ) { inConcert.logger.guiAction( { action_name: action_name, window_id: window.id } ); };

		var callback_spam_action = function( ok, err ) {
			if ( ok ) {
				window.setTimeout(
					function() {
						inConcert.app.m_mainWindow.getPanel("SpamInteractions").refreshSpamQueue();
						self.close();
					}, 10 );
				return;
			}
			inConcert.app.m_mainWindow.showBannerMessage({ message : err, timeout : 5000 });
		};

		var app_id = this.from_app;
		var interactionManager = inConcert.app.getInteractionManager();
		var markAsSpam = function() {
			interactionManager.markSpam( [ self.m_interaction.id ], callback_spam_action, app_id );
		};

		var markAsNoSpam = function() {
			interactionManager.markNotSpam( [ self.m_interaction.id ], callback_spam_action, app_id );
		};

		var setReply = function() {
			$("#mailReply").click(function() {
				log("reply");
				self.disableRelatedSections();
				self.replyCurrentMail();
			});
		};

		var setForward = function() {
			$("#mailForward").click(function() {
				log("forward");
				self.disableRelatedSections();
				self.forwardCurrentMail();
			});
		};

		var setReplyAll = function() {
			$("#mailReplyAll").click(function() {
				log("reply_to_all");
				self.disableRelatedSections();
				self.replyCurrentMail(true /* reply all */);
			});
		};

		var setMarkAsSpam = function() {
			$("#markAsSpam").click(function() {
				log("mark_spam");
				markAsSpam();
			});
		};

		var setMarkAsNoSpam = function() {
			$("#markAsNoSpam").click(function() {
				log("mark_nospam ");
				markAsNoSpam();
			});
		};

		var setPrint = function() {
			$("#mailPrint").click(function() {
				log("print e-mail");
				self.print();
			});
		};

		var setDiscard = function() {
			$("#discardCurrentMail").click(function() {
				log("discard");

				const apply_discard = function() {
					if (self.m_interaction.state === "PENDING") {
						self.deleteAttachments();
						self.__setComposingMail(false);
						self.updateSideBar();
						self.openParentClientWindow(true /* close current window */,
							function() {
								self.close();
							});
					}
					else {
						self.discardMail();
						self.enableRelatedSections();
					}	
				};

				if (self.m_interaction.hasDraft()) {
					//Eliminar draft
					self.m_interaction.deleteDrafts(function (success, message, interactionId) {
						if (success) {
							self.__update_draft_text(false, interactionId);
							apply_discard();
						}
						else{
							self.displayStatusMessage(message, "Error", false, 8000);
							inConcert.logger.debug(message);
						}
					});
					return;
				}
				apply_discard();
			});
		};

		var setSend = function() {
			$("#sendMail").click(function() {
				log("send_mail");
				self.send();
			});
		};
		//Validar si se puede mostrar Addresses. Si no se puede, se elimina el evento de buscar addresses y se modifica el placeholder
		// para dar referencia a q ingrese mail manual
		const visualize_address = inConcert.app.canVisualizeAddress(null, this.m_interaction.campaign, null, true);
		if (!visualize_address) {
			$("#mailTOSection").find("#mailTo").attr('placeholder','<%=_T("Enter e-mail")%>');
			$("#mailCCSection").find("#mailCC").attr('placeholder','<%=_T("Enter e-mail")%>');
			$("#mailBCCSection").find("#mailBCC").attr('placeholder','<%=_T("Enter e-mail")%>');
		}

		var setTo = function() {
			if (visualize_address) {
				$("#mailTOSection")
					.off("keyup", "#mailTo")
					.on("keyup", "#mailTo", function(evt) {
						const $mailTo = $(this);
						self.search_contact_keypress(evt, $mailTo);
					})
					.off("click", "#mailTo")
					.on("click", "#mailTo", function() {
						const $mailTo = $(this);
						if (!self.is_composing_mail()) {
							return;
						}
						window.setTimeout(function() {
							let value = $mailTo.val().trim();
							self.search_contact(value, $mailTo);
						});
					}
				);
			}
		};

		var setCC = function() {
			if (visualize_address) {
				$("#mailCCSection")
					.off("keyup", "#mailCC")
					.on("keyup", "#mailCC", function(evt) {
						const $mailCC = $(this);
						self.search_contact_keypress(evt, $mailCC);
					})
					.off("click", "#mailCC")
					.on("click", "#mailCC", function() {
						const $mailCC = $(this);
						if (!self.is_composing_mail()) {
							return;
						}
						window.setTimeout(function() {
							let value = $mailCC.val().trim();
							self.search_contact(value, $mailCC);
						});
					}
				);
			}
		};

		var setBCC = function() {
			$("#toggleBCCSection").click(function() {
				self.toggle_cc_bcc_sections();
			});

			if (visualize_address) {
				$("#mailBCCSection")
					.off("keyup", "#mailBCC")
					.on("keyup", "#mailBCC", function(evt) {
						const $mailBCC = $(this);
						self.search_contact_keypress(evt, $mailBCC);
					})
					.off("click", "#mailBCC")
					.on("click", "#mailBCC", function() {
						const $mailBCC = $(this);
						if (!self.is_composing_mail()) {
							return;
						}
						window.setTimeout(function() {
							let value = $mailBCC.val().trim();
							self.search_contact(value, $mailBCC);
						});
					}
				);
			}
		};

		var setSubjectMore = function() {
			$("#mailSubjectMore").focus(function() {
				$("#mailSubject").focus();
			});
		};

		var setSubject = function() {
			$("#mailSubject")
				.keydown(function(event) {
					if (event.which === 9) {
						// Si presiono la tecla TAB, tengo que darle el foco por codigo al body del control de escritura,
						//   ya que sino el mismo se lo da la toolbar.
						// Tambien tengo que agregar la asignacion del foco dentro de un timeout, porque sino el foco no lo termina asignando correctamente.
						window.setTimeout(function() {
							self.$getBodyEditor().focus();
						});
					}
				});
		};

		// expander events
		var setExpander = function() {
			self.setEventExpander(true);
		};

		// atachments upload
		var setAttachmentsUploader = function() {
			self.initAttachmentsUploader(true);
		};

		// move to folder events
		var setMoveToFolder = function() {
			self.setMoveToFolder(true);
		};

		/*
			Desactivo todos los eventos
		*/
		var deactivateEvents = function() {
			//console.log("E-mail client window - Deactivate events for app " + app_id);
			$("#mailReply").unbind("click");
			$("#mailForward").unbind("click");
			$("#mailReplyAll").unbind("click");
			$("#markAsSpam").unbind("click");
			$("#markAsNoSpam").unbind("click");
			$("#mailPrint").unbind("click");
			$("#discardCurrentMail").unbind("click");
			$("#sendMail").unbind("click");
			$("#mailTOSection").off("keyup", "#mailTo");
			$("#mailCCSection").off("keyup", "#mailCC");
			$("#mailBCCSection").off("keyup", "#mailBCC");
			$("#toggleBCCSection").unbind("click");
			$("#mailSubjectMore").unbind("focus");
			$("#mailSubject").unbind("keydown");
			// expander events
			self.setEventExpander(false);
			// atachments upload
			self.initAttachmentsUploader(false);
			// move to folder
			self.setMoveToFolder(false);
		};

		/*
			En esta definicion se encuentran las funciones que asignan los eventos por aplicacion y por estados de las interacciones. 
		*/
		var statesDefinition = {
			interaction: {
				connected: {
					setEvents: function() {
						//console.log("E-mail client window - Setting events by state: " + state + " on Interaction app.");
						setReply();
						setForward();
						setReplyAll();
						setMarkAsSpam();
						setMarkAsNoSpam();
						setPrint();
						setDiscard();
						setSend();
						setTo();
						setCC();
						setBCC();
						setSubjectMore();
						setSubject();
						setExpander();
						setAttachmentsUploader();
						setMoveToFolder();
					}
				}
			},
			supervisor: {
				archived: {
					setEvents: function() {
						//console.log("E-mail client window - Setting events by state: " + state + " on Supervisor app.");
						setPrint();
					}
				},
				monitoring: {
					setEvents: function() {
						//console.log("E-mail client window - Setting events by state: " + state + " on Supervisor app.");
						setPrint();
						setMarkAsSpam();
						setMoveToFolder();
					}
				},
				queued: {
					setEvents: function() {
						//console.log("E-mail client window - Setting events by state: " + state + " on Supervisor app.");
						setPrint();
					}
				},
				waiting: {
					setEvents: function() {
						//console.log("E-mail client window - Setting events by state: " + state + " on Supervisor app.");
						setPrint();
					}
				}
			}
		};

		// primero y antes que nada desactivo los eventos
		deactivateEvents();

		// segundo asigno los eventos para la app/estado.
		if (statesDefinition[app_id]) {
			var stateEvents = statesDefinition[app_id][state];
			if (stateEvents) {
				stateEvents.setEvents();
			}
		}
	};

	MailClientWindow.prototype.cancelInteraction = function() //Button Cancel for pending
	{
		MailClientWindow.uber.cancelInteraction.call( this );
		/*Eliminar attachment*/
		this.deleteAttachments();
	};

	MailClientWindow.prototype.is_composing_mail = function(){
		return this.m_composingMail;
	};

	MailClientWindow.prototype.dispose = function(){
		MailClientWindow.uber.dispose.call( this ); //invokes the base class method
		this.destroySpellChecker();
	};

	MailClientWindow.prototype.canCloseWindow = function (params) {
		if (this.isConfirmCloseWindows) {
			return false;
		}
		
		if (params && params.interactive && this.m_composingMail) {
			const body = this.get_editor_body() || "";
			let sms = "<%=_T('You will lose all changes if you close the window.')%>";
			let show = false;
			if (typeof this.__isDraftAvailable === "function" && this.__isDraftAvailable()) {
				let resp = this.__validateDraftLimitAndSize({ body: body });

				if (resp.result) {
					//Se cierra client (boton X). Se salva una vez mas, para evitar caso borde.
					this.__onProcessDraft(true);
					return true;
				} else {
					sms = resp.error;
					show = true;
				}
			} else {
				const to = $("#mailTo").val() || "";
				const subject = $("#mailSubject").val() || "";
				const cc = $("#mailCC").val() || "";
				const bcc = $("#mailBCC").val() || "";
				const bodytext = this.$getBodyEditor().text();
				const uploadedFiles = this.m_uploadedFiles || [];
				show = to.trim() !== "" || subject.trim() !== "" || cc.trim() !== "" || bcc.trim() !== "" || bodytext.trim() !== "" || uploadedFiles.length > 0;
			}

			if (show) {
				const self = this;
				this.isConfirmCloseWindows = true;
				$.inconcertPopup.Show(
					"<%=_T('Confirm')%>",
					sms + " " + "<%=_T('Do you want to continue ?')%>",
					$.inconcertPopup.ButtonSets.BS_YESNO,
					function(response) {
						delete self.isConfirmCloseWindows;
						if (response.toLowerCase() === "yes") {
							self.close();
						}
					},
					undefined,
					{
						popup_id       : "conf_inconcertPopup",
						container_class: "confirmationPopUp",
						overlay_class  : "confirmationOverlay"
					}
				);

				return false;
			}
		}
		return true;
	};

	MailClientWindow.prototype.invalidInteraction = function() {
		MailClientWindow.uber.invalidInteraction.call( this ); //invokes the base class method
		this.updateReadActionsButtons("hide");
		$("#mailTo").prop("readonly", true );
		$("#mailSubject").prop("readonly", true );
		$("#mailCC").prop("readonly", true );
		$("#mailBCC").prop("readonly", true );
		$("#mailBodySection").hide();
		this.setContent();
	};

	MailClientWindow.prototype.on_account_changed = function( old_account_id, new_account_id )
	{
		MailClientWindow.uber.on_account_changed.call( this, old_account_id, new_account_id );
		var self = this;

		if (this.m_composingMail ) {
			if ( old_account_id !== new_account_id ) {
				// font
				self.set_font_by_account();

				// signature
				if ( COMPOSING_MAIL_SIGNATURE_CONTAINER_ID ) {
					var $signature_container = $("#" + COMPOSING_MAIL_SIGNATURE_CONTAINER_ID);
					if ( $signature_container.length > 0 ) {
						var signature = this.getSignature();
						$signature_container.html(signature);
					}
				}
			}
		}
	};

	MailClientWindow.prototype.updateReadActionsButtons = function( action /* no es requerida */)
	{
		var $actions = $("#interactionReadActionsButtons").children();

		if ( this.is_client_unusuable || action === "hide" || (this.m_interaction.isFinished() || this.is_readonly || !this.m_interaction.isValid()) ) {
			$actions.hide();
		}
		else {
			$actions.show();
		}
		this.updateSpamButtons();
	};

	MailClientWindow.prototype.updateSpamButtons = function()
	{
		var self = this;

		var $spam   = $("#markAsSpam");
		var $nospam = $("#markAsNoSpam");

		var hide_all = function() { $spam.hide(); $nospam.hide(); };
		hide_all();

		if (this.isHam() || !this.m_interaction) {
			return;
		}

		var show_spam = function() {
			var must_show_mark_spam = self.hasPermissionToAction("MARK_AS_SPAM");
			$spam[must_show_mark_spam ? "show" : "hide"]();
			$nospam.hide();
		};
		var show_nospam = function() {
			var must_show_not_spam  = self.hasPermissionToAction("MARK_AS_NOT_SPAM");
			$nospam[must_show_not_spam ? "show" : "hide"]();
			$spam.hide();
		};

		if (this.is_spam) {
			show_nospam();
			return;
		}
		if (this.m_interaction.initiative === "INBOUND") {
			show_spam();
		}
	};

	MailClientWindow.prototype.getWindowSizeOnSidebarShown = function(){
		var size = { width: null, height: null };
		size.width = DEFAULT_WINDOW_WIDTH_PX;
		return size;
	};

	MailClientWindow.prototype.getWindowSizeOnSidebarHidden = function(){
		var size = { width: null, height: null };
		size.width = DEFAULT_WINDOW_WIDTH_PX;
		return size;
	};

	//Returns true if the interaction can be considered as ham
	//that is: its reopened, or the account is the remitent
	MailClientWindow.prototype.isHam = function() {
		if (!this.m_interaction || !this.m_interaction.steps || !this.m_interaction.events) {
			return false;
		}

		var steps = this.m_interaction.steps;
		for (var step_idx = 0; step_idx < steps.length; step_idx++) {
			if (steps[step_idx].action === "REOPENED") {
				return true;
			}
		}
		var events = this.m_interaction.events;
		for (var event_idx = 0; event_idx < events.length; event_idx++) {
			if (events[event_idx].creator === "agent") {
				return true;
			}
		}

		return false;
	};

	MailClientWindow.prototype.setPeopleFrom = function(current_agent) {
		const self = this;
		const app = inConcert.app;
		const interaction = this.m_interaction;
		const currentEvent = interaction.getEvent(this.m_currentEvent);

		// Build the from header
		// support a callback to create a link
		const setFrom = function(from, accountFrom, linkCallback){
			const avatarURI = function() {
				if (currentEvent && currentEvent.creator === "bot") {
					return "/inconcert/apps/agent/images/botAvatar.png";
				}
				if (accountFrom === app.getLoggedUserId()) {
					return app.getBaseUrl() + "/users/get_avatar/" + accountFrom;
				}
				return interaction.getEventUserAvatarURI( currentEvent );
			}();
			$("#contactFromAvatar").attr("data", avatarURI);

			const visualize_address = inConcert.app.canVisualizeAddress(null, interaction.campaign, null, true);
			$("#contactFromAccount").text(!visualize_address? "<%=_T('Hidden Address')%>" : accountFrom);

			const $iconTopContactId = $("#iconContactId");
			$iconTopContactId.hide();
			if (typeof(linkCallback) === "function" && interaction.accountHasContactManagementEnabled( self.from_app ) && visualize_address){
				$iconTopContactId.show();
			}
			$("#contactTopID").text(interaction.getDisplayName());
			$("#contactID").text(from);
		};

		if (current_agent || (interaction.initiative === "OUTBOUND" && interaction.state === "PENDING")) {
			setFrom(app.getLoggedUserName(), app.getLoggedUserId());
		}
		else {
			if (self.m_currentEvent === -1) {
				// if has more than one unread events, exit the function and opens the related list.
				return;
			}
			if (currentEvent.creator === "agent" && currentEvent.agent) {
				if (currentEvent.agent === app.getCompleteId()) {
					setFrom(app.getLoggedUserName(), app.getLoggedUserId());
				}
				else {
					var parsedAgent = currentEvent.agent.match(new RegExp("(.+)@([^@]+)$"));
					var memberWithoutVCC = parsedAgent[1];
					app.getTeamMembers(function(members) {
						if ( !members ) {
							return;
						}
						var member = members.find(memberWithoutVCC);
						if (member) {
							setFrom(member.Name, memberWithoutVCC);
						}
						else {
							setFrom(currentEvent.name, currentEvent.from);
						}
					}, true);
				}
			}
			else if (currentEvent.creator === "bot") {
				setFrom("<%=_T('Bot')%>", currentEvent.from);
			}
			// creator it's the contact
			else {
				var currentName = currentEvent.name;
				var avatarURI = interaction.getEventUserAvatarURI( currentEvent );
				var eventAddress = _.findWhere(currentEvent.addresses, {"source" : "from"});
				var addresses = eventAddress ? [ eventAddress.address ] : [];
				var eventAddressDisplayable = currentEvent.from;

				// if the interaction have a contact associated, set the proper info to the client window
				if (interaction.contact && interaction.contact.length > 0 && interaction.contactAddress){
					// only if we are working with the same event address
					if (interaction.contactAddress && interaction.contactAddress === eventAddressDisplayable){
						currentName = interaction.interactionData.contactName;
						eventAddressDisplayable = interaction.contactAddress;
					}
				}
				setFrom(currentName, eventAddressDisplayable, function(){
					var contactWindow = new inConcert.ContactWindow({
						from_app: self.from_app,
						interaction : interaction,
						name : currentName,
						avatar : avatarURI,
						addresses : addresses
					});
					contactWindow.show();
				});
			}
		}
	};

	MailClientWindow.prototype.getInteractionRelatedMarkup = function( properties )
	{
		let i = this.m_interaction;
		if ( i.is_auto_response_event( properties.sorted_index ) ) {
			return false;
		}
		// los unread se tienen que validar contra el index posta ( como se encuentran ordenados en el server ) ya que en el cliente los eventos son reordenados x fecha de creacion.
		let readClass = i.isRead( properties.original_index ) ? "relatedInteractionRead" : "relatedInteractionUnread";
		let interactionAbstract = inConcert.Utils.escapeAttrib(i.getExtendedInfo( properties.sorted_index ));

		return $("".concat(
			"<div unread_event='true' class='relatedInteraction ", readClass, "'>",
				"<div class='interactionAvatar'>",
					"<img src='", properties.avatarURI, "' width='40px' height='40px' />",
				"</div>",
				"<div class='interactionInfoContainer'>",
					"<label class='interactionContactName'>", properties.fromName,
						"<span class='interactionContactAccount'>", properties.fromAccount, "</span>",
					"</label>",
					"<label class='interactionDate'>", i.getLocaleDateString( properties.sorted_index ), "</label>",
					"<div class='interactionAbstract'>", interactionAbstract, "</div>",
					"</div>",
					"<a unread_event_action='true' href='javascript:void(0);' class='interactionButton'>",
						'<i class="far fa-chevron-right"></i>',
					"</a>",
				"</div>",
			"</div>"));
	};

	MailClientWindow.prototype.showNavigation = function() {
		$(".interactionNavigation, .interactionNavigation > input").show();
	};

	MailClientWindow.prototype.hideNavigation = function() {
		$(".interactionNavigation, .interactionNavigation > input").hide();
	};

	MailClientWindow.prototype.updateToolbarButtons = function()
	{
		MailClientWindow.uber.updateToolbarButtons.call( this );

		var can_show = this.canMoveToFolder();
		var $moveToFolder = $("#moveMailToFolder");
		if (can_show) {
			$moveToFolder.show();
		}
		else {
			$moveToFolder.hide();
		}
	};

	MailClientWindow.prototype.openFoldersPopup = function(btn_parent)
	{
		var self = this;

		var canMoveToFolder = self.canMoveToFolder();
		if (!canMoveToFolder) {
			return;
		}

		if (this.folders_popup) {
			this.folders_popup.close();
			this.folders_popup = null;
		}

		var current_folder_id = this.m_interaction.folderId;
		this.folders_popup = new FoldersPopup({
			current_folder: current_folder_id,
			interaction: this.m_interaction,
			from_app: this.from_app,
			cached_folders: this.cached_folders
		});

		this.folders_popup.show({
			parentNode: $(btn_parent),
			owner_object: this,
			callbacks: {
				on_load: function(folders) {
					if (folders) {
						self.cached_folders = folders;
					}
				},
				on_moved: function(from, to) {
					if (self.is_monitor) {
					// En el caso en la cual la ventana fue abierta desde la app Supervisor y la misma se encuentra monitorenado,
					//   tengo actualizar la interaccion agregando de forma manual al array de folders, el folder al cual se ha movido.
					// Esto se debe de realizar asi, ya que las interaccion en el supervisor NO se actualizan con el poller y por ende no se actualiza la ventana
					//  cuando existe algún cambio.
					//
					// Esto no es necesario para la app "Interaction"
						var folders = self.m_interaction.folders || [];
						folders.push({
							action: "MANUAL",
							from: from,
							to: to
						});
						self.m_interaction.folders = folders;
					}
				}
			}
		});
	};

	MailClientWindow.prototype.canMoveToFolder = function()
	{
		if (!this.m_interaction) {
			return false;
		}
		var custom_verify = {is_monitor: false};
		if (!this.isEditable(custom_verify)) {
			return false;
		}
		var app_id = this.from_app;
		if (!this.hasPermissionToAction("MOVE_MAIL_TO_FOLDER", app_id)) {
			return false;
		}
		return this.m_interaction.canMoveToFolder();
	};

	MailClientWindow.prototype.setMoveToFolder = function(enabled)
	{
		var self = this;

		var $move = $("#moveMailToFolder");
		$move.off("click");

		if (enabled) {
			$move.on("click", function()
			{
				self.openFoldersPopup(this);
			});
		}
	};

	MailClientWindow.prototype.setEventExpander = function(enabled)
	{
		const self = this;
		//for each "show more" link calculate the width of the contained elements
		//(TO, CC, CCO, Subject, Attachments). If they don't fit in the given space
		//then displaythe "show more" link
		this.$getControlsHeaderMoreInfo().each( function(undefined, element)
		{
			const $this = $(this);
			let field = $this.parent().children(".mailField");
			let childrenWidth = 0;
			if (field.length) {
				childrenWidth = field.attr("scrollWidth");
			} else {
				//special case for attachments (do not have the same class)
				field = $this.parent().children(".mailAttachContainer");
				//summation of chidren
				field.children().each(function(undefined, attachment) {
					// The magic number is used so you can see the link "show more", when the size is slightly smaller
					const attachment_width = $(attachment).width() + 20;
					childrenWidth += attachment_width;
				});
			}

			const $element = $(element);
			if (!enabled || childrenWidth <= field.width()) {
				$element.hide().unbind("click.expander");
				return;
			}

			$element.show()
				.unbind("click.expander")
				.bind("click.expander", function() {
					const $section = $element.parent();
					const placeholders = {
						["mailcc"]: "<%=_T('CC')%>",
						["mailbcc"]: "<%=_T('BCC')%>",
						["mailsubject"]: "<%=_T('Subject')%>"
					};
					let input, value, inputId;
					if ($section.hasClass("mailDataExtended")) {
						$section.removeClass("mailDataExtended");
						$element.attr("title", "<%=_T('+ show more')%>");
						input = $section.find("textarea");
						inputId = input.attr("id");
						if (inputId) {
							const placeholder = placeholders[inputId.toLowerCase()];
							value = document.getElementById(inputId).value;
							input.remove();
							const $inputNode = $("<input " + (!self.m_composingMail ? "readonly" :"") + " class='mailField' type='text' placeholder='" + placeholder + "' id='" + inputId + "' />");
							$inputNode.val(value);
							$element.after($inputNode);
						}
					}
					else {
						$section.addClass("mailDataExtended");
						$element.attr("title", "<%=_T('- show less')%>");
						input = $section.find("input.mailField");
						value = input.val();
						inputId = input.attr("id");
						if (inputId) {
							const placeholder = placeholders[inputId.toLowerCase()];
							input.remove();
							$element.after("<textarea " + (!self.m_composingMail ? "readonly" :"") + " class='mailField' placeholder='" + placeholder + "' id='" + inputId + "'>" + value + "</textarea>");
						}
					}
					self.adjustBodyTop();
				}
			);
		});
	};

	MailClientWindow.prototype.$getBodyEditor = function() { return $("#mailBodySection").find("[contenteditable]"); };

	MailClientWindow.prototype.disableBrowserBuiltInSpellChecker = function()
	{
		var $body = this.$getBodyEditor();
		if ( $body ) {
			$body.attr("spellcheck", "false");
		}
	};

	MailClientWindow.prototype.setTOLabel = function()
	{
		$("#mailTOSection .mailLabel").text(
			this.m_composingMail ?
			"<%=_T('TO (one address):')%>" :
			"<%=_T('TO:')%>"
		);
	};

	/*
		WYSIWYG EDITOR - FORECOLOR PERMISSION

			Para permitir o no el cambio de color de letra, hubo que generar la paleta de colores manualmente en un popup (similar al tipo de letra).
			Esto hubo que realizarlo así, ya que la paleta de colores es built-in del plugin, y no hay forma de inhabilitarla o habilitarla de forma dinámica.
			Tampoco se tiene acceso a esta para utilizarla, entonces para solucionar el problema de validación de permisos, hubo que:

				1 - Agregar el evento "popup" al botón "fore color", y que invoque a esta función "$makeCustomColorPalette".
				2 - En función primero se realiza la validación de permisos.
				3 - Luego genera la paleta de colores ídem a como lo realiza de forma interna el WYSIWYG EDITOR.
				4 - Problema solucionado.

		Referencia: http://redmine.inconcert/issues/7383#change-26917
	*/
	MailClientWindow.prototype.$makeCustomColorPalette = function()
	{
		var $mail_editor = $("#mailBodyContent");

		// si tiene configurado el font color en la cuenta, no se puede cambiar la misma
		var account_font_config = this.getAccountFont();
		if (account_font_config && account_font_config.color.length > 0) {
			$mail_editor.wysiwyg('shell').closePopup();
			var message_error = "<%=_T('Unable to change font color. You not have permission for this action.')%>";
			this.displayStatusMessage(message_error, "Error", false, 2500);
			return;
		}

		// http://stackoverflow.com/questions/17242144/javascript-convert-hsb-hsv-color-to-rgb-accurately
		var HSVtoRGB = function( h, s, v )
		{
			var r, g, b, i, f, p, q, t;
			i = Math.floor(h * 6);
			f = h * 6 - i;
			p = v * (1 - s);
			q = v * (1 - f * s);
			t = v * (1 - (1 - f) * s);
			switch (i % 6)
			{
				case 0: r = v; g = t; b = p; break;
				case 1: r = q; g = v; b = p; break;
				case 2: r = p; g = v; b = t; break;
				case 3: r = p; g = q; b = v; break;
				case 4: r = t; g = p; b = v; break;
				case 5: r = v; g = p; b = q; break;
			}
			var hr = Math.floor(r * 255).toString(16);
			var hg = Math.floor(g * 255).toString(16);
			var hb = Math.floor(b * 255).toString(16);
			return '#' + (hr.length < 2 ? '0' : '') + hr +
						(hg.length < 2 ? '0' : '') + hg +
						(hb.length < 2 ? '0' : '') + hb;
		};

		var $content_colorpalette = function( wysiwygeditor, forecolor )
		{
			var $content = $('<table/>')
							.attr('cellpadding','0')
							.attr('cellspacing','0')
							.attr('unselectable','on');

			var set_color = function() {
				var new_color = this.title;
				if( forecolor ) {
					wysiwygeditor.forecolor( new_color ).closePopup().collapseSelection();
				}
				else {
					wysiwygeditor.highlight( new_color ).closePopup().collapseSelection();
				}
				return false;
			};

			for( var row=1; row < 15; ++row ) // should be '16' - but last line looks so dark
			{
				var $rows = $('<tr/>');
				for( var col=0; col < 25; ++col ) // last column is grayscale
				{
					var color;
					if( col === 24 )
					{
						var gray = Math.floor(255 / 13 * (14 - row)).toString(16);
						var hexg = (gray.length < 2 ? '0' : '') + gray;
						color = '#' + hexg + hexg + hexg;
					}
					else
					{
						var hue        = col / 24;
						var saturation = row <= 8 ? row     /8 : 1;
						var value      = row  > 8 ? (16-row)/8 : 1;
						color = HSVtoRGB( hue, saturation, value );
					}
					$('<td/>').addClass('wysiwyg-toolbar-color')
						.attr('title', color)
						.attr('unselectable','on')
						.css({backgroundColor: color})
						.click(set_color)
						.appendTo( $rows );
				}
				$content.append( $rows );
			}
			return $content;
		};

		var wysiwyg_shell = $mail_editor.wysiwyg('shell');
		return $content_colorpalette(wysiwyg_shell, true);
	};

	MailClientWindow.prototype.createHtmlEditor = function( event_index )
	{
		var self = this;

		this.setTOLabel();

		$("#mailBodyHTML").hide().children().remove();

		// http://www.jqueryrain.com/?0gvIIXi7
		// https://github.com/wysiwygjs/wysiwyg.js
		//
		var $mail_editor = $("#mailBodyContent");

		$mail_editor.wysiwyg({
			classes: 'some-more-classes',
			// 'selection'|'top'|'top-selection'|'bottom'|'bottom-selection'
			toolbar: 'top-selection',
			buttons: {
				// // Smiley plugin
				// smilies: {
					// title: "<%=_T('Smilies')%>",
					// image: '\uf118', // <img src="path/to/image.png" width="16" height="16" alt="" />
					// popup: function( $popup, $button ) {
						// var list_smilies = [
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/afraid.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/amorous.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/angel.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/angry.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/bored.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/cold.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/confused.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/cross.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/crying.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/devil.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/disappointed.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/dont-know.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/drool.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/embarrassed.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/excited.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/excruciating.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/eyeroll.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/happy.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/hot.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/hug-left.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/hug-right.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/hungry.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/invincible.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/kiss.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/lying.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/meeting.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/nerdy.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/neutral.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/party.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/pirate.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/pissed-off.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/question.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/sad.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/shame.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/shocked.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/shut-mouth.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/sick.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/silent.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/sleeping.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/sleepy.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/stressed.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/thinking.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/tongue.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/uhm-yeah.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/wink.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/working.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/bathing.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/beer.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/boy.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/camera.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/chilli.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/cigarette.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/cinema.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/coffee.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/girl.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/console.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/grumpy.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/in_love.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/internet.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/lamp.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/mobile.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/mrgreen.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/musical-note.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/music.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/phone.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/plate.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/restroom.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/rose.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/search.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/shopping.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/star.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/studying.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/suit.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/surfing.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/thunder.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/tv.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/typing.png" width="16" height="16" alt="" />',
								// '<img src="/inconcert/lib/wysiwyg/demo/smiley/writing.png" width="16" height="16" alt="" />'
						// ];
						// var $smilies = $('<div/>').addClass('wysiwyg-plugin-smilies').attr('unselectable','on');
						// $.each( list_smilies, function(index,smiley) {
							// if( index !== 0 ) {
								// $smilies.append(' ');
							// }
							// var $image = $(smiley).attr('unselectable','on');
							// // Append smiley
							// var imagehtml = ' '+$('<div/>').append($image.clone()).html()+' ';
							// $image
								// .css({ cursor: 'pointer' })
								// .click(function(event) {
									// $mail_editor.wysiwyg('shell').insertHTML(imagehtml); // .closePopup(); - do not close the popup
								// })
								// .appendTo( $smilies );
						// });
						// var $container = $mail_editor.wysiwyg('container');
						// $smilies.css({ maxWidth: parseInt($container.width()*0.95)+'px' });
						// $popup.append( $smilies );
						// // Smilies do not close on click, so force the popup-position to cover the toolbar
						// var $toolbar = $button.parents( '.wysiwyg-toolbar' );
						// if( !$toolbar.length ){ // selection toolbar?
							// return ;
						// }
						// return { // this prevents applying default position
							// left: parseInt( ($toolbar.outerWidth() - $popup.outerWidth()) / 2 ),
							// top: $toolbar.hasClass('wysiwyg-toolbar-bottom') ? ($container.outerHeight() - parseInt($button.outerHeight()/4)) : (parseInt($button.outerHeight()/4) - $popup.height())
						// };
						// },
					// showstatic: true,    // wanted on the toolbar
				// },

				//insertsharedimage: {
				//	title: "<%=_T('Insert shared image')%>",
				//	image: '\uf03e', // <img src="path/to/image.png" width="16" height="16" alt="" />
				//	showstatic: true,    // wanted on the toolbar
				//	showselection: false,
				//	popup: function( /*$popup, $button*/ ) { },
				//	click: function( /*$button*/ )
				//	{
				//		self.showSharedImageSelector();
				//	}
			//	},//
				insertimage: {
					title: "<%=_T('Insert image')%>",
					image: '\uf03e', // <img src="path/to/image.png" width="16" height="16" alt="" />
					showstatic: true    // wanted on the toolbar
					//showselection: index == 2 ? true : false    // wanted on selection
				},
				// insertvideo: {
					// title: "<%=_T('Insert video')%>",
					// image: '\uf03d', // <img src="path/to/image.png" width="16" height="16" alt="" />
					// //showstatic: true,    // wanted on the toolbar
					// showselection: index == 2 ? true : false    // wanted on selection
				// },
				insertlink: {
					title: "<%=_T('Insert link')%>",
					image: '\uf08e' // <img src="path/to/image.png" width="16" height="16" alt="" />
				},
				// Fontname plugin
				fontname: {
					title: "<%=_T('Font')%>",
					image: '\uf031', // <img src="path/to/image.png" width="16" height="16" alt="" />
					popup: function( $popup/*, $button*/ )
					{
						// si tiene configurado el font family en la cuenta, no se puede cambiar la misma
						var account_font_config = self.getAccountFont();
						if (account_font_config && account_font_config.family.length > 0) {
							$mail_editor.wysiwyg('shell').closePopup();
							var message_error = "<%=_T('Unable to change font family. You not have permission for this action.')%>";
							self.displayStatusMessage(message_error, "Error", false, 2500);
							return;
						}
						var fonts = inConcert.app.loadFontsList();
						var $list = $('<div/>').addClass('wysiwyg-plugin-list').attr('unselectable', 'on');
						$.each( fonts, function( _, font ) {
							var $link = $('<a/>').attr('href','#')
												.css( 'font-family', font.id )
												.html( font.name )
												.click(function(event) {
													$mail_editor.wysiwyg('shell').fontName(font.id).closePopup();
													// prevent link-href-#
													event.stopPropagation();
													event.preventDefault();
													return false;
												});
							$list.append( $link );
						});
						$popup.append( $list );
					},
					showstatic: true    // wanted on the toolbar
				},
				// Fontsize plugin
				fontsize: {
					title: "<%=_T('Size')%>",
					image: '\uf034', // <img src="path/to/image.png" width="16" height="16" alt="" />
					popup: function( $popup/*, $button*/ )
					{
						// si tiene configurado el font size en la cuenta, no se puede cambiar la misma
						var account_font_config = self.getAccountFont();
						if (account_font_config && account_font_config.size.length > 0) {
							$mail_editor.wysiwyg('shell').closePopup();
							var message_error = "<%=_T('Unable to change font size. You not have permission for this action.')%>";
							self.displayStatusMessage(message_error, "Error", false, 2500);
							return;
						}
						// Hack: http://stackoverflow.com/questions/5868295/document-execcommand-fontsize-in-pixels/5870603#5870603
						//
						var list_fontsizes = [];
						var i;
						for( i=8; i <= 11; ++i ) {
							list_fontsizes.push(i+'px');
						}
						for( i=12; i <= 28; i+=2 ) {
							list_fontsizes.push(i+'px');
						}
						list_fontsizes.push('36px');
						list_fontsizes.push('48px');
						list_fontsizes.push('72px');
						var $list = $('<div/>').addClass('wysiwyg-plugin-list').attr('unselectable','on');
						$.each( list_fontsizes, function( _, size ) {
							var $link = $('<a/>').attr('href','#')
												.html( size )
												.click(function(event) {
													$mail_editor.wysiwyg('shell').fontSize(7).closePopup();
													$mail_editor.wysiwyg('container')
															.find('font[size=7]')
															.removeAttr("size")
															.css("font-size", size);
													// prevent link-href-#
													event.stopPropagation();
													event.preventDefault();
													return false;
												});
							$list.append( $link );
						});
						$popup.append( $list );
					}
					//showstatic: true,    // wanted on the toolbar
					//showselection: true    // wanted on selection
				},
				bold: {
					title: "<%=_T('Bold (Ctrl+B)')%>",
					image: '\uf032', // <img src="path/to/image.png" width="16" height="16" alt="" />
					hotkey: 'b'
				},
				italic: {
					title: "<%=_T('Italic (Ctrl+I)')%>",
					image: '\uf033', // <img src="path/to/image.png" width="16" height="16" alt="" />
					hotkey: 'i'
				},
				underline: {
					title: "<%=_T('Underline (Ctrl+U)')%>",
					image: '\uf0cd', // <img src="path/to/image.png" width="16" height="16" alt="" />
					hotkey: 'u'
				},
				strikethrough: {
					title: "<%=_T('Strikethrough (Ctrl+S)')%>",
					image: '\uf0cc', // <img src="path/to/image.png" width="16" height="16" alt="" />
					hotkey: 's'
				},
				forecolor: {
					title: "<%=_T('Text color')%>",
					image: '\uf1fc', // <img src="path/to/image.png" width="16" height="16" alt="" />
					popup: function( $popup/*, $button*/ )
					{
						var $custom_color_palette = self.$makeCustomColorPalette();
						$popup.append($custom_color_palette);
					}
				},
				highlight: {
					title: "<%=_T('Background color')%>",
					image: '\uf576' // <img src="path/to/image.png" width="16" height="16" alt="" />
				},
				alignleft: {
					title: "<%=_T('Left')%>",
					image: '\uf036', // <img src="path/to/image.png" width="16" height="16" alt="" />
					//showstatic: true,    // wanted on the toolbar
					showselection: false    // wanted on selection
				},
				aligncenter: {
					title: "<%=_T('Center')%>",
					image: '\uf037', // <img src="path/to/image.png" width="16" height="16" alt="" />
					//showstatic: true,    // wanted on the toolbar
					showselection: false    // wanted on selection
				},
				alignright: {
					title: "<%=_T('Right')%>",
					image: '\uf038', // <img src="path/to/image.png" width="16" height="16" alt="" />
					//showstatic: true,    // wanted on the toolbar
					showselection: false    // wanted on selection
				},
				alignjustify: {
					title: "<%=_T('Justify')%>",
					image: '\uf039', // <img src="path/to/image.png" width="16" height="16" alt="" />
					//showstatic: true,    // wanted on the toolbar
					showselection: false    // wanted on selection
				},
				subscript: {
					title: "<%=_T('Subscript')%>",
					image: '\uf12c', // <img src="path/to/image.png" width="16" height="16" alt="" />
					//showstatic: true,    // wanted on the toolbar
					showselection: true    // wanted on selection
				},
				superscript: {
					title: "<%=_T('Superscript')%>",
					image: '\uf12b', // <img src="path/to/image.png" width="16" height="16" alt="" />
					//showstatic: true,    // wanted on the toolbar
					showselection: true    // wanted on selection
				},
				indent: {
					title: "<%=_T('Indent')%>",
					image: '\uf03c', // <img src="path/to/image.png" width="16" height="16" alt="" />
					//showstatic: true,    // wanted on the toolbar
					showselection: false    // wanted on selection
				},
				outdent: {
					title: "<%=_T('Outdent')%>",
					image: '\uf03b', // <img src="path/to/image.png" width="16" height="16" alt="" />
					//showstatic: true,    // wanted on the toolbar
					showselection: false    // wanted on selection
				},
				orderedList: {
					title: "<%=_T('Ordered list')%>",
					image: '\uf0cb', // <img src="path/to/image.png" width="16" height="16" alt="" />
					//showstatic: true,    // wanted on the toolbar
					showselection: false    // wanted on selection
				},
				unorderedList: {
					title: "<%=_T('Unordered list')%>",
					image: '\uf0ca', // <img src="path/to/image.png" width="16" height="16" alt="" />
					//showstatic: true,    // wanted on the toolbar
					showselection: false    // wanted on selection
				},

		// custom buttons -----------------------

			// spell checker
				spell_checker: {
					title: "<%=_T('Spell checker')%>",
					// How should the button look like?
					image:'\uf891', // '<img src="/inconcert/lib/wysiwyg/demo/smiley/afraid.png">',
					html: '<raw html>',
					// What should the button do?
					popup: function( /*$popup, $button*/ ) { },
					click: function( /*$button*/ )
					{
						if ( !self.createSpellChecker() ) {
							return;
						}
						window.setTimeout(function() {
							self.m_spellchecker.check(
								null,
								function( /* words */ ) { /* lo dejo por si tengo que hacer algo. */ }
							);
						}, 150);
					},
					// Where should the button be placed?
					showstatic: true, // on the static toolbar
					showselection: false // on selection toolbar
				}
			},

			// Submit-Button
			submit: {
				title: "<%=_T('Submit')%>",
				image: '\uf00c' // <img src="path/to/image.png" width="16" height="16" alt="" />
			},
			// Other properties
			selectImage: "<%=_T('Click image')%>",
			placeholderUrl: 'www.example.com',
			placeholderEmbed: '<embed/>',
			maxImageSize: [600,200],
			validateUploadImageSize : function( file, event, callback ) {
				self.validate_upload_image( file, event, callback );
			},
			onKeyDown: function( /*key, character, shiftKey, altKey, ctrlKey, metaKey*/ ) {
				// E.g.: submit form on enter-key:
				//if( (key == 10 || key == 13) && !shiftKey && !altKey && !ctrlKey && !metaKey ) {
				//    submit_form();
				//    return false; // swallow enter
				//}
			},
			onKeyPress: function( /*key, character, shiftKey, altKey, ctrlKey, metaKey*/ ) {
			},
			onKeyUp: function( /*key, character, shiftKey, altKey, ctrlKey, metaKey*/ ) {
			},
			onAutocomplete: function( typed, key, character/*, shiftKey, altKey, ctrlKey, metaKey*/ ) {
				if( typed.indexOf('@') === 0 ) // startswith '@'
				{
					var usernames = [
							'Evelyn',
							'Harry',
							'Amelia',
							'Oliver',
							'Isabelle',
							'Eddie',
							'Editha',
							'Jacob',
							'Emily',
							'George',
							'Edison'
						];
					var $list = $('<div/>').addClass('wysiwyg-plugin-list').attr('unselectable','on');
					$.each( usernames, function( _, username ) {
						if( username.toLowerCase().indexOf(typed.substring(1).toLowerCase()) !== 0 ) { // don't count first character '@'
							return;
						}
						var $link = $('<a/>').attr('href','#')
											.text( username )
											.click(function(event) {
												var url = 'http://example.com/user/' + username,
													html = '<a href="' + url + '">@' + username + '</a> ';
												var editor = $mail_editor.wysiwyg('shell');
												// Expand selection and set inject HTML
												editor.expandSelection( typed.length, 0 ).insertHTML( html );
												editor.closePopup().getElement().focus();
												// prevent link-href-#
												event.stopPropagation();
												event.preventDefault();
												return false;
											});
						$list.append( $link );
					});
					if( $list.children().length )
					{
						if( key === 13 )
						{
							$list.children(':first').click();
							return false; // swallow enter
						}
						// Show popup
						else if( character || key === 8 ) {
							return $list;
						}
					}
				}
			},
			onImageUpload: function( /*insert_image*/ ) {
				// A bit tricky, because we can't easily upload a file via
				// '$.ajax()' on a legacy browser without XMLHttpRequest2.
				// You have to submit the form into an '<iframe/>' element.
				// Call 'insert_image(url)' as soon as the file is online
				// and the URL is available.
				// Example server script (written in PHP):
				/*
				<?php
				$upload = $_FILES['upload-filename'];
				// Crucial: Forbid code files
				$file_extension = pathinfo( $upload['name'], PATHINFO_EXTENSION );
				if( $file_extension != 'jpeg' && $file_extension != 'jpg' && $file_extension != 'png' && $file_extension != 'gif' )
					die("Wrong file extension.");
				$filename = 'image-'.md5(microtime(true)).'.'.$file_extension;
				$filepath = '/path/to/'.$filename;
				$serverpath = 'http://domain.com/path/to/'.$filename;
				move_uploaded_file( $upload['tmp_name'], $filepath );
				echo $serverpath;
				*/
				// Example client script (without upload-progressbar):
			/*
				var iframe_name = 'legacy-uploader-' + Math.random().toString(36).substring(2);
				$('<iframe>').attr('name',iframe_name)
							 .load(function() {
								// <iframe> is ready - we will find the URL in the iframe-body
								var iframe = this;
								var iframedoc = iframe.contentDocument ? iframe.contentDocument :
											   (iframe.contentWindow ? iframe.contentWindow.document : iframe.document);
								var iframebody = iframedoc.getElementsByTagName('body')[0];
								var image_url = iframebody.innerHTML;
								insert_image( image_url );
								$(iframe).remove();
							 })
							 .appendTo(document.body);
				var $input = $(this);
				$input.attr('name','upload-filename')
					  .parents('form')
					  .attr('action','/script.php') // accessing cross domain <iframes> could be difficult
					  .attr('method','POST')
					  .attr('enctype','multipart/form-data')
					  .attr('target',iframe_name)
					  .submit();
			*/
			},
			forceImageUpload: false,    // upload images even if File-API is present
			videoFromUrl: function( url ) {
				// Contributions are welcome :-)

				// youtube - http://stackoverflow.com/questions/3392993/php-regex-to-get-youtube-video-id
				var youtube_match = url.match( /^(?:http(?:s)?:\/\/)?(?:[a-z0-9.]+\.)?(?:youtu\.be|youtube\.com)\/(?:(?:watch)?\?(?:.*&)?v(?:i)?=|(?:embed|v|vi|user)\/)([^\?&\"'>]+)/ );
				if( youtube_match && youtube_match[1].length === 11 ) {
					return '<iframe src="//www.youtube.com/embed/' + youtube_match[1] + '" width="640" height="360" frameborder="0" allowfullscreen></iframe>';
				}
				// vimeo - http://embedresponsively.com/
				var vimeo_match = url.match( /^(?:http(?:s)?:\/\/)?(?:[a-z0-9.]+\.)?vimeo\.com\/([0-9]+)$/ );
				if( vimeo_match ) {
					return '<iframe src="//player.vimeo.com/video/' + vimeo_match[1] + '" width="640" height="360" frameborder="0" webkitAllowFullScreen mozallowfullscreen allowFullScreen></iframe>';
				}
				// dailymotion - http://embedresponsively.com/
				var dailymotion_match = url.match( /^(?:http(?:s)?:\/\/)?(?:[a-z0-9.]+\.)?dailymotion\.com\/video\/([0-9a-z]+)$/ );
				if( dailymotion_match ) {
					return '<iframe src="//www.dailymotion.com/embed/video/' + dailymotion_match[1] + '" width="640" height="360" frameborder="0" webkitAllowFullScreen mozallowfullscreen allowFullScreen></iframe>';
				}

				// undefined -> create '<video/>' tag
			}
		});

		this.m_editor_saved_selection = null;
		var $body = this.$getBodyEditor();
		$body.off("focusin")
			.on("focusin", function() {
				self.m_editor_saved_selection = null;
			}
		);
		$body.off("focusout")
			.on("focusout", function() {
				// Cuando el editor pierde el foco, almaceno la posicion donde se encuentra el mismo.
				//  Posteriormente puede ser utilizado en el seteo de canned messages,
				//    ya que sino el canned es agregado siempre al final del body, en lugar de ser agregado donde se encuentra el cursor del editor.
				self.m_editor_saved_selection = self.editor_save_selection();
			}
		);

		// inicia los eventos para realizar resize de las imagenes.
		this.__InitializeImageResize();
		// inicia los eventos para realizar drag & drop de imagenes y archivos desde el file system.
		this.__InitializeFileSystemFilesDragAndDrop();
		// desactiva el corrector ortografico por defecto de los navegadores
		this.disableBrowserBuiltInSpellChecker();

		var content = this.getHTMLQuotedBody(event_index);
		this.__set_body_content(content);

		this.set_font_by_account();
		$("#mailBodySection").show();
		this.adjustBodyTop();
	};

	MailClientWindow.prototype.__set_body_content = function(content)
	{
		var $mail_editor = $("#mailBodyContent");
		$mail_editor.wysiwyg("shell").setHTML(content);
	};

	MailClientWindow.prototype.__append_body_content = function(content)
	{
		var $mail_editor = $("#mailBodyContent");
		let editor = $mail_editor.wysiwyg("shell");
		editor.setHTML(editor.getHTML() + content);
	};

	MailClientWindow.prototype.__prepend_body_content = function(content)
	{
		var $mail_editor = $("#mailBodyContent");
		let editor = $mail_editor.wysiwyg("shell");
		editor.setHTML(content + editor.getHTML());
	};

	MailClientWindow.prototype.__InitializeImageResize = function()
	{
		var $mail_editor = this.$getBodyEditor();
		inConcert.Utils.initialize_image_resize($mail_editor);
	};

	MailClientWindow.prototype.__buildDraftAttachments = function(draft)
	{
		if (!draft || !draft.uploadedFiles || draft.uploadedFiles.length === 0) {
			return;
		}

		this.m_uploadedFiles = draft.uploadedFiles;
		const self = this;
		for (var i = 0; i < this.m_uploadedFiles.length; i++) {
			const file = this.m_uploadedFiles[i];
			const $attachNode = this.makeAttachmentNode(file.name);
			this.bindAttachmentNodeDelete($attachNode, function() {
				self.__onDeleteAttachment(file.id, $attachNode);
			});
			this.appendAttachmentNode($attachNode);
			this.setAttachmentNodeStatus($attachNode, "done");
		}
		this.toggleAttachmentSection(true);
	};

	MailClientWindow.prototype.__buildDraft = function()
	{
		const getCC = function(emails) {
			if (!emails || emails.trim().length === 0) {
				return;
			}
			return emails.trim();
		};

		let draft = {
			to          : $("#mailTo").val().trim() || ""             ,
			subject     : $("#mailSubject").val()     || ""            ,
			cc          : getCC($("#mailCC").val())      || ""         ,
			bcc         : getCC($("#mailBCC").val())    || ""        ,
			body        : this.get_editor_body()       || ""    ,
			user : inConcert.app.getLoggedUserId(),
			uploadedFiles: inConcert.Utils.cloneArray(this.m_uploadedFiles || [])
		};

		if (this.m_action) {
			draft.action = this.m_action;
		}

		if (this.m_currentEvent !== undefined && this.m_currentEvent > -1) {
			draft.currentEvent = this.m_currentEvent;
		}

		return draft;
	};

	MailClientWindow.prototype.__addDraftToClient = function(draft) {
		$("#mailTo").val(draft.to || "");
		$("#mailSubject").val(draft.subject || "");
		$("#mailCC").val(draft.cc || "");
		$("#mailBCC").val(draft.bcc || "");
		if (draft.body) {
			this.__set_body_content(draft.body);
		}
		this.__update_draft_text(true);
	};

	MailClientWindow.prototype.__isDraftAvailable = function() {
		//solo disponible para la App “Interaction” (para la App Supervisor NO) y en la App “Interaction” que queda solo para la bandeja “Active”
		const enableDraft = typeof inConcert.app.getMaxDraftsAllowedToSave === "function" && inConcert.app.getMaxDraftsAllowedToSave();
		return this.from_app && this.from_app === "interaction" && enableDraft && (this.m_interaction.isAssigned() || this.m_interaction.isBeingComposed());
	};

	MailClientWindow.prototype.__getDraftSizeAllowed = function(currentDraft) {
		const kilobytesSize = function (str) {
			return (new Blob([str]).size) / 1000; //llevamos de bytes a kb
		};
		return kilobytesSize(currentDraft.body);
	};

	MailClientWindow.prototype.__initialize_draft = function()
	{
		const draft = this.m_interaction && this.m_interaction.getDraft();
		const logged_user = inConcert.app.getLoggedUserId();
		if (!draft || !this.__isDraftAvailable() || (logged_user !== draft.user)) {
			return;
		}

		if (!this.m_interaction.isBeingComposed()) {
			const d_action = draft.action;
			const d_currentEvent = draft.currentEvent;
			if (d_currentEvent !== undefined && d_action) {
				switch (d_action) {
					case "reply":
						this.navigateEvent(d_currentEvent);
						$("#mailReply").click();
						break;
					case "reply_all":
						this.navigateEvent(d_currentEvent);
						$("#mailReplyAll").click();
						break;
					case "forward":
						this.navigateEvent(d_currentEvent);
						$("#mailForward").click();
						break;
					default:
						return;
				}	
			}
			else
			{
				return;
			}
		}
		this.__addDraftToClient(draft);
		this.__buildDraftAttachments(draft);
	};

	MailClientWindow.prototype.__validateDraftLimitAndSize = function (currentDraft)
	{
		const maxDraftAllowed = inConcert.app.getMaxDraftsAllowedToSave();
		const intWithDraft = inConcert.app.getInteractionManager().getAssignedInteractionsWithDraftByChannel("MAIL");

		/*
		1- Si el numero de interacciones con draft es igual al limite configurado
			y la interaccion actual no esta en el grupo con draft, muestro sms = "<%=_T('blabla')%>"
		2- Si el size del body es mayor al configurado (en kb) muestro sms = "<%=_T('blabla')%>"
		*/
		if (!$.isEmptyObject(intWithDraft) && Object.keys(intWithDraft).length === parseInt(maxDraftAllowed, 10) && !intWithDraft[this.m_interaction.id]) {				
			return {result:false, error : format("<%=_T('Unable to save draft. The maximum number of drafts allowed ({0}) is exceeded.')%>", maxDraftAllowed)};
		}

		const sizeDraft = this.__getDraftSizeAllowed(currentDraft);
		const sizeLimit = parseInt(inConcert.app.getMaxDraftsSizeAllowed(), 10);
		if (sizeDraft > sizeLimit) {
			return {result:false, error : format("<%=_T('Unable to save draft. The draft size ({0}) exceeds the allowed limit ({1}).')%>", Number.isInteger(sizeDraft)? sizeDraft: sizeDraft.toFixed(2), sizeLimit)};
		}
		return {result:true};
	};

	MailClientWindow.prototype.__update_draft_text = function(isAdd, interactionId) {
		const $contact = $("#interactionTopIdContact");
		const $contactDraftText = $contact.find("#draftTextContainer");
		// Se agrega text 'Borrador' en el nodo del panel
		const panel = inConcert.app.getMainWindow().getPanel("AssignedInteractions");
		const interaction = this.m_interaction || {id :interactionId};
		const $interaction = panel.$findInteraction(interaction);
		this.m_interaction_gui.update_int_draft_display($interaction, isAdd);
		// Se agrega text 'Borrador' en el cliente
		if (isAdd) {
			if ($contactDraftText.length > 0) {
				return;
			}
			const $node = $("".concat(
				"<div id='draftTextContainer'>",
				($contact.html().trim() !== "") ? "<span>, </span>" : "",
					"<span class='draftText'>", "<%=_T('Draft')%>", "</span>",
				"</div>"));

				$contact.append($node);
			return;
		}
		if ($contactDraftText.length > 0) {
			$contactDraftText.remove();
		}
	};

	MailClientWindow.prototype.__onProcessDraft = function (isClose) {
		//Si tiene old draft y es igual al current, no hago nada
		const self = this;
		const currentDraft = this.__buildDraft();
		const oldDraft = this.m_interaction.getDraft();

		const areEqualsDrafts = function()
		{
			for (const property in currentDraft) {
				if (currentDraft.hasOwnProperty(property)) {
					if (property === "user") {
						continue;
					}
					if (property === "uploadedFiles") {
						const curr_files = currentDraft[property];
						const old_files = oldDraft[property];
						if (curr_files.length !== old_files.length) {
							return false;
						}
						if (curr_files.length === 0 && old_files.length === 0) {
							continue;
						}
						let curr_files_map = inConcert.Utils.arrayToMap(curr_files || [],
							(file) => file,
							(file) => file.id
						);
						let old_files_map = inConcert.Utils.arrayToMap(old_files || [],
							(file) => file,
							(file) => file.id
						);
						for (let id in curr_files_map) {
							if (curr_files_map.hasOwnProperty(id)) {
								if (!old_files_map.hasOwnProperty(id)) {
									return false;
								}
							}
						}
					}
					else if ( currentDraft[property] !== oldDraft[property])
					{
						return false;
					}
				}
			}
			return true;
		};

		if (oldDraft && areEqualsDrafts()) {
			return;
		}

		if (!isClose) {
			//Validate limit and size
			let resp = this.__validateDraftLimitAndSize(currentDraft); 
			if(!resp.result){
				this.displayStatusMessage(resp.error , "Alert", false, 10000);
				return;
			}
		}
		
		//Enviar a cassandra o IR para salvar
		//Si es true el envio del lado del server, guardar en memoria.
		this.m_interaction.saveDrafts(currentDraft, function (success, message, interactionId) {
			if (!success) {
				inConcert.logger.debug(message);
				if (isClose) {//boton X, muestro generico
					inConcert.app.getMainWindow().showBannerMessage({
						messageType : "Error",
						message     : message,
						timeout     : 5000
					});
				}else{
					self.displayStatusMessage(message, "Error", true, 5000);
				}
			}
			else{
				self.__update_draft_text(true, interactionId);
			}
		});
	};

	MailClientWindow.prototype.__setComposingMail = function(enable)
	{
		const self = this;
		this.m_composingMail = enable;

		if (enable && this.__isDraftAvailable()) {
			
			if (this.nIntervDraft) {
				clearInterval(this.nIntervDraft);
			}

			this.nIntervDraft = setInterval(() => {
				self.__onProcessDraft();
			}, 60 * 1000);
		}
		else if (this.nIntervDraft){
			clearInterval(this.nIntervDraft);
		}
	};

	MailClientWindow.prototype.__InitializeFileSystemFilesDragAndDrop = function()
	{
		const self = this;

		var validate_fn = function(file, callback) {
			if (!file) {
				callback(false);
				return;
			}
			if (file.type.match(/^image/i)) {
				self.editor_restore_selection();
				self.validate_upload_image(file, _, function(success) {
					callback(success);
				});
			}
			else {
				callback(true);
			}
		};

		var custom_drop_fn = function(file/*, e*/) {
			if (!file) {
				return;
			}
			self.uploadAttachmentFile(file);
		};

		const $mail_editor = this.$getBodyEditor();
		inConcert.Utils.initialize_file_system_files_drag_and_drop($mail_editor, validate_fn, custom_drop_fn);
	};

	/*
		Estas 2 funciones "editor_save_selection" y "editor_restore_selection", son utilizadas para guardar y restaurar la posicion
		  donde se encuentra el cursor en el editor, cuando este pierde el foco.

		save/restore selection
		http://stackoverflow.com/questions/13949059/persisting-the-changes-of-range-objects-after-selection-in-html/13950376#13950376
	*/
	MailClientWindow.prototype.editor_save_selection = function()
	{
		var selection;
		if ( window.getSelection )
		{
			selection = window.getSelection();
			if ( selection.rangeCount > 0 ) {
				return selection.getRangeAt(0);
			}
		}
		else if ( document.selection ) {
			selection = document.selection;
			return selection.createRange();
		}
		return null;
	};

	MailClientWindow.prototype.editor_restore_selection = function()
	{
		if ( !this.m_editor_saved_selection ) {
			return;
		}
		if ( window.getSelection ) {
			var sel = window.getSelection();
			sel.removeAllRanges();
			sel.addRange(this.m_editor_saved_selection);
		}
		else if ( document.selection ) {
			this.m_editor_saved_selection.select();
		}
	};

	MailClientWindow.prototype.__setExternalMessage = function(message)
	{
		if (!this.m_composingMail) {
			return;
		}
		this.editor_restore_selection();
		message = message || "";
		$("#mailBodyContent").wysiwyg("shell").insertHTML(message);
		this.$getBodyEditor().focus();
	};

	/*
		El canal de Email soporta "texto sin formato" y "HTML"
	*/
	MailClientWindow.prototype.setCannedMessage = function(canned)
	{
		let message;
		let format_type = this.sidebar.get_canned_format_type(canned);
		if (format_type.id === "TEXT PLAIN") {
			message = canned.plain_message || canned.message;
		}
		else {
			message = canned.message;
		}
		this.__setExternalMessage(message);
	};

	MailClientWindow.prototype.getAccountFont = function()
	{
		var campaign = inConcert.app.getCampaign( this.m_interaction.campaign );
		if (!campaign) {
			return false;
		}
		var account_id = this.m_interaction.getAccount();
		if (!campaign.has_font_configured_in_account( account_id, "MAIL" )) {
			return false;
		}
		var account = campaign.getAccount( account_id, "MAIL" );
		return account.Font;
	};

	// Los estilos de fuentes se aplican al contenedor del nuevo evento que se esta redactando.
	// Cada evento (contenido body del mail) se encuentra dentro de un DIV con el atributo "event-custom-body-style=NN",
	//  donde NN es el numero de evento, a partir de este atributo podemos aplicar los estilos de fuentes que se han configurado para la cuenta.
	//
	MailClientWindow.prototype.set_font_by_account = function()
	{
		var event_index = this.m_interaction.isBeingComposed() ? 0 : this.m_currentEvent + 1;
		var $custom_body_style = this.$getBodyEditor().find("[event-custom-body-style='" + event_index + "']");
		if ($custom_body_style.length === 0) {
			return;
		}

		var default_font = inConcert.app.getDefaultFont();
		var font_family = default_font.id; // default
		var font_size = default_font.size; // default
		var font_color = default_font.color; // default

		var account_font = this.getAccountFont();
		if (account_font) {
			font_family = account_font.family || font_family;
			font_size = account_font.size || font_size;
			font_color = account_font.color || font_color;
		}
		$custom_body_style.css("font-family", font_family);
		$custom_body_style.css("font-size", font_size + "px");
		$custom_body_style.css("color", font_color);
	};

	MailClientWindow.prototype.get_editor_body = function()
	{
		//text area is not coming out with modifications using .val() nor .text()
		return $("#mailBodyContent").wysiwyg("shell").getHTML();
	};

	MailClientWindow.prototype.validate_upload_image = function( file, _, callback )
	{
		var self = this;
		var success = true;
		var msg_error = "<%=_T('')%>"; // NO borrar, thrubelshuting ya que sino el primer _T en esta funcion, no se traduce, ya que se produce un error en la i18nLib, TODO: Validar el porque!!! Tira algo asi: <?lua =_T('') ?>

		if (!file) {
			success = false;
			msg_error = "<%=_T('Unable to upload image. File does not exist.')%>";
			this.displayStatusMessage(msg_error, "Error", false, 5000);
			callback(success);
			return;
		}
		if (!file.type.match(/^image/i)) {
			success = false;
			msg_error = "<%=_T('Unable to upload image. Only images are allowed.')%>";
			this.displayStatusMessage(msg_error, "Error", false, 5000);
			callback(success);
			return;
		}

		var file_size_kb = parseInt(file.size / 1024, 10);

		if ( file.size > MAX_UPLOAD_IMAGE_SIZE_BYTES ) {
		// aca valida que el tamanio del archivo a subir no supere el maximo tamanio soportado por archivos de forma indivudual
			success = false;
			msg_error = format("<%=_T('Unable to upload image of {0} Kb size. Max upload image size is {1} Kb.')%>", file_size_kb , MAX_UPLOAD_IMAGE_SIZE_BYTES / 1024);
			this.displayStatusMessage(msg_error, "Error", false, 5000);
			callback(success);
		}
		else {
			var body = this.get_editor_body();
			var $body = $(body);
			// tengo que agregar en un div contenedor solo para que el $.each pueda iterar entre los elementos
			var $container = $("<div></div>");
			$container.append($body);
			var generate_id = false; // solo debo obtener las imagenes, el id ya fue creado en el upload, ademas aca no lo necesito
			var embedded_images = this.geEmbeddedImagesInBody($container, generate_id);
			var total_size = file.size; // agrego la current file a subir, ya que aun la misma no se encuentra como imagen embebida en el DOM
			$.each(embedded_images, function(_, image_info)
			{
				// Aca esta la magia, paso base64 a binario para que posteriormente obtener el tamano exacto del file.
				// A esta conversion la realiza "canavas to blob":
				//		https://github.com/blueimp/JavaScript-Canvas-to-Blob
				//		/inconcert/lib/canvas-to-blob/js/canvas-to-blob.js
				//
				var base64_data = image_info.src;
				var blob = window.dataURLtoBlob && window.dataURLtoBlob(base64_data);
				total_size = total_size + blob.size; // voy incrementando al current file size, el tamanio de los archivos embebidos.
				if ( total_size > MAX_UPLOAD_IMAGES_SIZE_BYTES ) {
					success = false;
					msg_error = format("<%=_T('Unable to upload image of {0} Kb size. Max upload image size is {1} Kb.')%>", file_size_kb, MAX_UPLOAD_IMAGES_SIZE_BYTES / 1024);
					self.displayStatusMessage(msg_error, "Error", false, 5000);
					return false; // break
				}
			});
			// aca va el ok o el error
			callback(success);
		}
	};

	MailClientWindow.prototype.getHTMLQuotedBody = function(eventIndex)
	{
		var header_html = "";
		var body_html = "";
		var new_event_idex = 0;

		COMPOSING_MAIL_SIGNATURE_CONTAINER_ID = "mail_signature_" + inConcert.Utils.generateGuid();

		var default_font = inConcert.app.getDefaultFont();
		var signature_style_value = format("font-family: {0}; font-size: {1}px; color: {2}",
			default_font.family.id,
			default_font.size,
			default_font.color
		);

		var signature_html =
			CTRLF + CTRLF + CTRLF +
			"<div id='" + COMPOSING_MAIL_SIGNATURE_CONTAINER_ID + "' style='" + signature_style_value + "'>" +
				this.getSignature() +
			"</div>" +
			CTRLF + CTRLF + CTRLF;

		if ( typeof eventIndex === "number" ) {
			new_event_idex = eventIndex + 1;
			var escAttr = inConcert.Utils.escapeAttrib;
			var interaction = this.m_interaction;

			var getSubject = function() {
				var subject = interaction.getSubject(eventIndex);
				return (subject !== null ? escAttr(subject) : "");
			};
			var getFrom = function() {
				var event = interaction.getEvent(eventIndex);
				if (event.agent) {
					var campaign = inConcert.app.getCampaign( interaction.campaign );
					var owner_address = interaction.getAddressByAccount();
					return escAttr( interaction.formatAddress( owner_address, campaign.Name ) );
				}
				return escAttr(interaction.getFrom(eventIndex));
			};
			var getTo = function() {
				var to = interaction.getTo(eventIndex);
				return (to !== null ? escAttr(to) : "");
			};
			var getCC = function() {
				var cc = interaction.getCC(eventIndex);
				if (cc.length === 0) {
					return "";
				}
				return "<%=_T('CC')%>" + COLON + WHITE_SPACE + interaction.getCC(eventIndex) + CTRLF;
			};

			// header -----------------------
			header_html = "".concat(
					"<%=_T('Date')%>"    , COLON, WHITE_SPACE, interaction.getLocaleDateString( eventIndex ), CTRLF,
					"<%=_T('Subject')%>" , COLON, WHITE_SPACE, getSubject(),                                  CTRLF,
					"<%=_T('From')%>"    , COLON, WHITE_SPACE, getFrom(),                                     CTRLF,
					"<%=_T('To')%>"      , COLON, WHITE_SPACE, getTo(),                                       CTRLF,
				getCC(),
				CTRLF, CTRLF
			);
			// body -----------------------
			body_html = "<blockquote>" + this.getEventContentHTML(eventIndex) + "</blockquote>";
		}

		// Quoted -----------------------
		return "".concat(
			// Cada evento (contenido body del mail) se encuentra dentro de un DIV con el atributo "event-custom-body-style=NN",
			//  donde NN es el numero de evento, a partir de esto de este atributo podemos aplicar los estilos de fuentes que se han configurado para la cuenta.
			//
			"<div event-custom-body-style='" + new_event_idex + "'>",
				// signature -----------------------
				signature_html,
				// header -----------------------
				header_html,
				// body -------------------------
				body_html,
			"</div>"
		);
	};

	MailClientWindow.prototype.getSignature = function()
	{
		var campaign = inConcert.app.getCampaign( this.m_interaction.campaign );
		if (!campaign) {
			return "";
		}
		var account_id = this.m_interaction.getAccount();
		if (!campaign.has_signature_configured_in_account( account_id, "MAIL" )) {
			return "";
		}
		var account = campaign.getAccount( account_id, "MAIL" );
		var signature_id = account.Signature;
		var signature = inConcert.app.getMailSignature(signature_id);
		if (!signature) {
			return "";
		}
		signature = inConcert.Utils.replace_vars({
			text : signature.Signature,
			data : {interaction: this.m_interaction, campaign_id: this.m_interaction.campaign}
		});
		return signature;
	};

	MailClientWindow.prototype.getReplySubject = function(index) {
		var subject = this.m_interaction.events[index].subject || "";
		return "<%=_T('Re')%>" + COLON + WHITE_SPACE + subject;
	};

	MailClientWindow.prototype.getQuotedAttachmentsToSend = function($body)
	{
		var self = this;
		var can_include = function(attach)
		{
			if ( attach["content-disposition"] === "inline" ) {
				// 1 - aca filtro las imagenes recibidas, que el Emailproxy ha incluido en la lista de attach como "inline", y que el agente ha decidio eliminar de forma manual la imagen del mail.
				var $image = $body.find("img[src*='" + attach.id + "']");
				if ( $image.length === 0 ) {
					return false;
				}
			}
			else {
				// 2 - aca filtro los current attachments, los que el usuario haya decidido no incluir, no los agrego al array de attachment del nuevo mail (evento)
				if ( self.m_deletedCurrentEventAttachments[ attach.id ] ) {
					return false;
				}
			}
			return true;
		};
		
		var attachments = [];
		$.each( self.m_forwardOrReplyAttachmentsEvent, function( _, attach )
		{
			if ( can_include( attach ) ) {
				attachments.push( attach );
			}
		});
		return attachments;
	};

	MailClientWindow.prototype.addAttachmentsToSend = function(response)
	{
		let attachments = [];
		let $body = $("<div>" + response.body + "</div>");

		//add quoted
		if (this.m_forwardOrReplyAttachmentsEvent ) {
			attachments = attachments.concat(this.getQuotedAttachmentsToSend($body));
		}
		
		//add current
		attachments = attachments.concat(this.m_uploadedFiles);
		
		//add already uploaded (like signatures)
		let attachments_d = attachments.reduce(function(data, attachment) {
			data[attachment.id] = true;
			return data;
		}, {});
		
		let domain = window.location.protocol + "//" + window.location.host;
		let fullPath = domain + URI_REPOSITORY_GET;
		
		$body.find("img[src*='repository']").each(function() {
			let $image = $(this);
			let src = $image.attr("src");
			//clean domain (copied images
			if (src.indexOf(fullPath) >= 0) {
				response.body = response.body.replace(domain, "");
			}
			//process image
			let id = $image.attr("repo_id");
			if (id && !attachments_d[id]) {
				let alreadyAdded = attachments.find(function(attachment) {
					return src.indexOf(attachment.id) >= 0;
				});
				if (!alreadyAdded) {
					attachments.push({
						id  : id,
						name: $image.attr("title") || "NO_NAME",
						type: "image/jpeg",
						"content-disposition" : "inline"
					});
					attachments_d[id] = true;
				}
			}
		});

		response.attachments = attachments;
		return true;
	};

	MailClientWindow.prototype.validateEmailFields = function()
	{
		var self = this;

		var getCC = function(emails) {
			if (!emails || emails.trim().length === 0) {
				return;
			}
			return emails.trim();
		};

		this.destroySpellChecker();

		var $account = $("#cboAccounts");

		var response = {
			// el atributo ".body" se inserta despues de que la validacion sea exitosa.
			account     : $account.val()                        ,
			attention_level_id : this.getSelectedAttentionLevel(),
			from        : $("option:selected", $account ).text(),
			to          : $("#mailTo").val().trim()             ,
			subject     : $("#mailSubject").val()               ,
			cc          : getCC($("#mailCC").val())             ,
			bcc         : getCC($("#mailBCC").val())            ,
			body		: this.get_editor_body()           ,
			contact     : self.m_interaction.contact
		};
		
		if (!this.addAttachmentsToSend(response)) {
			return false;
		}

		var waiting_time_ms = 5000;
		if (response.to.trim().length === 0) {
			self.displayStatusMessage("<%=_T('Please enter the e-mail recipient before sending')%>", "Info", false, waiting_time_ms);
			$("#mailTo").focus();
			return false;
		}

		var list = {
			from: {
				addresses : response.from,
				get_error : function( result )
				{
					var message_error;
					var address = inConcert.Utils.truncateWord( result.address );
					switch( result.error_type )
					{
						case "invalid_mail":
							message_error = format("<%=_T('FROM e-mail account must be a valid e-mail address: {0}.')%>", address );
							break;

						case "invalid_mail_list":
							message_error = format("<%=_T('FROM e-mail account list must be a valid: {0}. Must use the separator ;.  Example: Contact Name <sample@sample.com>; ... OR sample@sample.com; ...')%>", address );

							break;
						default:
							throw("Invalid validation mail message type");
					}
					return message_error;
				},
				container : null
			},
			to:   {
				addresses         : response.to,
				allowed_addresses : 1,
				get_error         : function( result )
				{
					var message_error;
					var address =  inConcert.Utils.truncateWord( result.address );
					var allowed_addresses = result.allowed_addresses;

					switch( result.error_type )
					{
						case "invalid_mail":
							message_error = format("<%=_T('TO e-mail account must be a valid e-mail address: {0}.')%>", address );
							break;

						case "invalid_mail_list":
							message_error = format("<%=_T('TO e-mail account list must be a valid: {0}. Must use the separator ;.  Example: Contact Name <sample@sample.com>; ... OR sample@sample.com; ...')%>", address );
							break;

						case "exceeded_limit_of_allowed_addresses":
							if ( allowed_addresses === 1 ) {
								message_error = format("<%=_T('Only {0} address is allowed in TO e-mail account field.')%>", allowed_addresses );
							}
							else {
								message_error = format("<%=_T('Only {0} addresses is allowed in TO e-mail account field.')%>", allowed_addresses || 1 );
							}

							break;
						default:
							throw("Invalid validation mail message type");
					}
					return message_error;
				},
				container     : $("#mailTo")
			},
			cc:   {
				addresses     : response.cc,
				get_error     : function( result )
				{
					var message_error;
					var address =  inConcert.Utils.truncateWord( result.address );
					switch( result.error_type )
					{
						case "invalid_mail":
							message_error = format("<%=_T('CC e-mail account must be a valid e-mail address: {0}.')%>", address );
							break;

						case "invalid_mail_list":
							message_error = format("<%=_T('CC e-mail account list must be a valid: {0}. Must use the separator ;. Example: Contact Name <sample@sample.com>; ... OR sample@sample.com; ...')%>", address );

							break;
						default:
							throw("Invalid validation mail message type");
					}
					return message_error;
				},
				container     : $("#mailCC")
			},
			bcc:  {
				addresses     : response.bcc,
				get_error     : function( result )
				{
					var message_error;
					var address =  inConcert.Utils.truncateWord( result.address );
					switch( result.error_type )
					{
						case "invalid_mail":
							message_error = format("<%=_T('BCC e-mail account must be a valid e-mail address: {0}.')%>", address );
							break;

						case "invalid_mail_list":
							message_error = format("<%=_T('BCC e-mail account list must be a valid: {0}. Must use the separator ;. Example: Contact Name <sample@sample.com>; ... OR sample@sample.com; ...')%>", address );

							break;
						default:
							throw("Invalid validation mail message type");
					}
					return message_error;
				},
				container     : $("#mailBCC")
			}
		};

		var validationError = function(result) {
			var email_type = list[result.type];
			var message_error = email_type.get_error( result );
			self.displayStatusMessage(message_error, "Error" , false, waiting_time_ms);
			if (email_type.container) {
				email_type.container.focus();
			}
		};
		if (!inConcert.Utils.validateEmails(list, validationError)) {
			return false;
		}

		return response;
	};

	/*
		Esta funcion sube las imagenes embebidas al repo
	*/
	MailClientWindow.prototype.uploadEmbeddedImages = function(embedded_images, callback)
	{
		var self = this;
		callback = callback || $.noop;

		if ( !$.isArray(embedded_images) || embedded_images.length ===0 ) {
			callback(true);
			return;
		}

		var count_atob_err = 0;
		var count_uploaded = 0;
		// esta funcion es invocada por cada archivo que se sube
		var upload_success = function(image_info)
		{
			// marco el file como success
			image_info.upload_status = "success";
			// asigno la uri de get para el archivo recien subido, la misma sera utilizada para reemplazar el attr src del base por la url del repo en el body del mail
			image_info.get_uri = format(
				"{0}?id={1}&section={2}&part={3}"       , /* full uri (uri get + params)  */
				URI_REPOSITORY_GET                      , /* repo uri get                 */
				encodeURIComponent( image_info.repo_id ), /* file id                      */
				"-1"                                    , /* section                      */
				"0"                                       /* part                         */
			);
			// si subieron todos los archivos correctamente, invoco al callback para que termine enviando el mail
			count_uploaded++;
			if ( count_uploaded + count_atob_err === embedded_images.length ) {
				callback(true, embedded_images); // si llega aca se subieron los mails correctamente y envio el mail por fin!!
			}
		};

		// En el caso de que algun archivo de los que se estan subiendo falle, tiene que:
		//	1- Abortar los request en curso que aun no se haya completado.
		//	2- Mandar a eliminar del repo los archivos embebidos previamente subidos.
		var upload_error = function(image_info)
		{
			callback(false, "<%=_T('Error uploading embed images')%>");
			image_info.upload_status = "error";
			var uploaded_list = [];

			$.each(embedded_images, function(_, image)
			{
				if (image.upload_status === "pending") {
					image.upload_status = "aborted";
					image_info.xhr.abort();
				}
				else if (image.upload_status === "success") {
					image.upload_status = "removed";
					uploaded_list.push( {
						id      : image.repo_id,
						section : "-1"
					});
				}
			});

			self.delete_repository_files(uploaded_list);
		};

		// aca realizo el upload del file al repo
		var upload = function(upload_uri, blob_or_file, image_info)
		{
			image_info.xhr = new XMLHttpRequest();
			image_info.xhr.open("POST", upload_uri, true);

			image_info.xhr.onload = function(e)
			{
				if (e.target.status === 200) {
					upload_success(image_info); // success
				}
				else {
					upload_error(image_info); // error
				}
			};
			image_info.xhr.setRequestHeader("Content-Type", blob_or_file.type || "application/octet-stream");
			image_info.xhr.send(blob_or_file);
		};

		// recorro la lista de imagenes embebidas y las mando subir al repositorio.
		$.each(embedded_images, function(_, image_info)
		{
			var file_name = image_info.name;
			var extension = inConcert.Utils.getFileExtension( file_name ) || "png";
			var encoded_id = encodeURIComponent( image_info.repo_id );
			var upload_uri = format("{0}/inconcert/apps/repository/new/?id={1}&section={2}&contentType={3}&format={4}&fileName={5}&proxyType=MAIL&vcc={6}" ,
				URI_DOMAIN,
				encoded_id,
				"-1", /* section */
				image_info.content_type,
				extension, /* format */
				file_name,
				inConcert.Utils.md5(inConcert.app.get_vcc_id().toLowerCase())
			);
			image_info.upload_status = "pending"; // lo dejo en estado pending mientras se realiza el request
			var base64_data = image_info.src;
			// Aca esta la magia, paso base64 a binario para que posteriormente sea subido al repositorio.
			// A esta conversion la realiza "canavas to blob":
			//		https://github.com/blueimp/JavaScript-Canvas-to-Blob
			//		/inconcert/lib/canvas-to-blob/js/canvas-to-blob.js
			//
			try {
				var blob = window.dataURLtoBlob && window.dataURLtoBlob(base64_data);
				upload(upload_uri, blob, image_info);
			}
			catch(e) {
				// si falla el encoded tengo que marcar la imagen como error de encoded, para que luego sea eliminada del array antes de ser enviado el mail.
				count_atob_err++;
				image_info.upload_status = "error.atob_encoded";
				console.warn("MailClientWindow.prototype.uploadEmbeddedImages. Unable to upload image. Incorrect base64: " + base64_data);
				if ( count_atob_err === embedded_images.length ) {
					// Como fallaron el encode de todos los mensajes embebidos, invoco al callback.
					// Si al menos existiera una imagen bien encodeada y subida al repo, la invocacion al callback se realiza en la funcion "upload_success"
					callback(true, embedded_images); // xxxx
				}
			}
		});
	};

	/*
		Funcion que obtiene el ID del la imagen en el Repo.
			1 - Obtiene el valor desde el atributo "repo_id" del elemento <IMG>.
			2 - Si no existe el atributo, parsea la URL que se encuentra en el atributo "src" para obtener el el ID de la imagen.
		El caso 2 se da cuando se realiza el copy/paste, en este caso el atributo "repo_id", NO se pega en el HTML destino.
	*/
	MailClientWindow.prototype.getImgRepoId = function($img)
	{
		if (!$img) {
			return;
		}

		let repo_id = $img.attr("repo_id");
		if (repo_id) {
			return repo_id;
		}

		let src = $img.attr("src");
		let params = src.replace(URI_REPOSITORY_GET, "");
		const urlParams = new URLSearchParams(params);
		repo_id = urlParams.get("id");
	
		return repo_id;
	};

	MailClientWindow.prototype.geEmbeddedImagesInBody = function($body, generate_id)
	{
		const self = this;

		generate_id = typeof generate_id === "undefined" ? true : false;
		const $images = $body.find("img[src*='base64']");
		// obtengo los <IMG con el attr SRC de base64 y los agrego al array
		let embedded_images = [];
		let images_by_id = {};

		$.each($images, function(_, img)
		{
			const $img = $(img);
			let repo_id = self.getImgRepoId($img);
			if (repo_id === null || !images_by_id[repo_id]) {
				let image_info = {
					content_type: "image/jpeg",
					name        : $img.attr("title") || "NO_NAME",
					src         : $img.attr("src")
				};
				if (generate_id) {
					repo_id = repo_id || inConcert.Utils.generateGuidWithSeparator();
					$img.attr("repo_id", repo_id);
					image_info.repo_id = repo_id;
					images_by_id[repo_id] = true;
				}
				embedded_images.push(image_info);
			}
		});
		return embedded_images;
	};

	MailClientWindow.prototype.hasUploadingAttachments = function() {
		return $(".mailAttachDownload").length > 0;
	};

	MailClientWindow.prototype.send = function(callback = $.noop)
	{
		var self = this;

		if (this.sending) {
			return callback(false, "already sending");
		}

		this.hideStatusMessage();
		if (!this.canExecuteInteractionAction() || this.hasUploadingAttachments()) {
			return callback(false, "can not send mail");
		}
		var response = this.validateEmailFields();
		if (!response) {
			return callback(false, "invalid destinations");
		}

		var $send = $("#sendMail");
		$send.prop("disabled", true);
		this.sending = true;
		this.displayStatusMessage("<%=_T('Sending...')%>", "Process", true);

		// (**) tengo que agregar en un div contenedor solo para que el $.each pueda iterar entre los elementos
		var $container = $("<div>" + response.body + "</div>");
		// el send se realiza en callback(success) del upload de imagenes embebidas
		var send = function(uploaded_images)
		{
			// con la lista de imagenes subidas "uploaded_images", reemplazo el atributo "src" donde se encuentra el base64
			//  por la uri del repo donde se encuentra el mismo.
			if ( uploaded_images && uploaded_images.length > 0 ) {
				$.each(uploaded_images, function(_, image)
				{
					var $image = $container.find("img[repo_id='" + image.repo_id + "']");
					if ( $image.length > 0 ) {
						$image.attr("src", image.get_uri);
					}
				});
				response.body = $container.html(); // aca ya esta sin el "DIV" que se agrego arriba (**)
				self.__set_body_content(response.body);
			}

			//The callback should receive a boolean (successful or not) and a message to display (not used when successful)
			
			const isPending = self.m_interaction.isBeingComposed();
			const has_draft = self.m_interaction.hasDraft();
			self.m_interaction.send(response, function(successful, message)
			{
				self.hideStatusMessage();
				$send.prop("disabled", false);
				self.sending = false;
				if (successful) {
					//this goes to base class window to decide how to close itself
					self.m_forwardOrReplyAttachmentsEvent = null;
					self.__setComposingMail(false);
					
					if (has_draft) {
						//Eliminar draft
						self.m_interaction.deleteDrafts(function (success, message, interactionId) {
							if (success) {
								self.__update_draft_text(false, interactionId);
							}
							else {
								inConcert.logger.debug(message);
							}
						}, isPending);
					}

					self.updateSideBar();
					self.close();
					callback(true);
				}
				else {
					//if failed but was properly created, disallow change of campaign/account
					if (self.m_interaction.state !== "PENDING") {
						self.disableCampaignAccountsControls();
					}
					self.displayStatusMessage(message, "Error");
					callback(false, message);
				}
			});
		};

		// obtengo las imagenes embebidas que existen en el body y luego las mando a subir
		var embedded_images = this.geEmbeddedImagesInBody($container);
		// subo las imagenes embebidas
		this.uploadEmbeddedImages(embedded_images, function(success, uploaded_list)
		{
			if (!success ) {
				// si el upload de las imagenes falla, cancelo todo y muestro el error correspondiente
				self.hideStatusMessage();
				$send.prop("disabled", false);
				self.sending = false;
				var err = uploaded_list;
				self.displayStatusMessage(err, "Error");
				return callback(false, "error uploading embeed images");
			}

			// aca elimino del array de imagenes subidas, las imagenes que han fallado en encode del base64
			uploaded_list = uploaded_list || [];
			for ( var i = uploaded_list.length - 1; i >= 0; i--) {
				var image_info = uploaded_list[i];
				if (image_info.upload_status === "error.atob_encoded") {
					uploaded_list.splice(i, 1);
				}
			}

			// si las imagenes fueron subido correctamente al repo, tengo que recorrer las mismas y agregarlas al array de upload files que se encuentra en la ventana del cliente, marcandolo como "content-disposition=inline"
			var uploaded_files = [];
			$.each(uploaded_list, function(_, image_info)
			{
				var uploaded_file = {};
				uploaded_file.id = image_info.repo_id;
				uploaded_file.name = image_info.name;
				uploaded_file.type = image_info.content_type;
				uploaded_file["content-disposition"] = "inline";
				uploaded_files.push(uploaded_file);
			});
			response.attachments = (response.attachments || []).concat(uploaded_files);
			self.m_uploadedFiles = self.m_uploadedFiles.concat(uploaded_files);
			// aca por fin envio el mail.
			send(uploaded_list);
		});
	};

	MailClientWindow.prototype.getForwardSubject = function()
	{
		var event = this.get_current_event();
		if ( !event ) {
			return "";
		}
		return format("<%=_T('Fw: {0}')%>", event.subject || "");
	};

	MailClientWindow.prototype.forwardCurrentMail = function()
	{
		var self = this;
		if (!this.canExecuteInteractionAction()) {
			return;
		}
		var index = this.m_currentEvent;
		this.m_forwardOrReplyAttachmentsEvent = this.m_interaction.getAttachmentsEvent(index); // para el forward van los "inline" y los "attachments"
		self.__setComposingMail(true);
		this.updateSideBar();
		this.m_action = "forward";

		$("body").removeClass("mailClientRead").addClass("mailClientWrite");
		$("#mailDate").hide();

		this.setEventExpander(true);
		this.createHtmlEditor( index );

		$("#mailSubject").val(this.getForwardSubject(index)).removeAttr("readonly");

		var campaignAddress = inConcert.app.getCampaignAddresses(this.m_interaction.campaign, "MAIL");
		if (!campaignAddress.length) {
			this.displayStatusMessage("<%=_T('Unable to forward e-mail, the campaign has no defined e-mail addresses')%>", "Error");
			this.discardMail();
			return;
		}
		var current_agent = true;
		this.setPeopleFrom(current_agent);
		$("#mailTo").val("").prop("readonly", false );
		$("#mailCC").val("").prop("readonly", false );
		$("#mailBCC").val("").prop("readonly", false );

		// este evento click, es solo para los deletes de los current events cuando se realiza forward
		$("#mailAttachments").find(".butMailAttachDelete")
			.unbind("click")
			.bind("click", function() {
				// aca oculto el attach eliminado, y lo agrego en la lista de current attach eliminados
				var $attach = $( this ).parent();
				var attach_info = $attach.find(".mailAttachItem").data("attachment");
				self.m_deletedCurrentEventAttachments[ attach_info.id ] = true;
				$attach.hide();
			}).show();

		this.hideCCSection();
		$("#mailTo").focus();
	};

	MailClientWindow.prototype.onNewEvents = function()
	{
		var events_count = this.m_interaction.getEventCount();
		var new_events_count = events_count - this.events_count;

		inConcert.logger.debug("new_events_count: " + new_events_count);

		if ( new_events_count === 0 ) {
			return;
		}

		var waiting_time_ms = 5000;
		this.displayStatusMessage("<%=_T('Has received new e-mails')%>", "Info", true, waiting_time_ms );
		if ( !this.is_visible_related_events() ) {
			return;
		}
		var all_members;
		inConcert.app.getTeamMembers( function( members ) { all_members = members; }, true );
		if (!all_members) {
			return;
		}

		var all_events = this.m_interaction.getEventList();
		var new_events = [];
		var new_event = all_events[ events_count - 1 ];
		if ( new_event ) {
			new_events.push( new_event );
		}
		var $list = $("#relatedInteractionsList");
		this.fillRelatedEvents( new_events, all_members, $list );
	};

	MailClientWindow.prototype.openCurrentEvent = function(callback = $.noop) {
		this.readCurrentMail();
		this.updateNavegationTooltip();
		
		if (this.__loadDraftMessage) {
			this.__loadDraftMessage = false;
			this.loadDraftMessage();
		}
		
		MailClientWindow.uber.openCurrentEvent.call( this, callback );
	};

	MailClientWindow.prototype.$getControlsHeaderMoreInfo = function()
	{
		return $("#mail_header_section").find("[mail_link_more='true']");
	};

	//Switchs to reply mode
	MailClientWindow.prototype.replyCurrentMail = function( replyAll )
	{
		var self = this;
		if (!this.canExecuteInteractionAction()) {
			return;
		}
		var index = this.m_currentEvent;
		this.m_forwardOrReplyAttachmentsEvent = this.m_interaction.getAttachmentsEvent(index, "inline"); // para el reply van solo los inline
		self.__setComposingMail(true);
		this.updateSideBar();
		$("body").removeClass("mailClientRead").removeClass("clientReadOnly").addClass("mailClientWrite");
		$("#mailDate").hide();

		this.setEventExpander(true);
		this.createHtmlEditor(index);

		$("#mailSubject").val(this.getReplySubject(index)).prop("readonly", false );
		var current_agent = true;
		this.setPeopleFrom(current_agent);

		var addedAddresses = {};
		var mailTo = "";
		var event = this.m_interaction.getEvent(index);
		if (event.agent || event.creator === "bot") {
			mailTo = this.m_interaction.getTo(index);
			addedAddresses[mailTo] = true;
		}
		else {
			mailTo = this.m_interaction.getFrom(index);
			addedAddresses[event.from] = true;
		}
		$("#mailTo").val(mailTo).prop("readonly", false );
		$("#mailCC").prop("readonly", false );
		$("#mailBCC").prop("readonly", false );

		if ( replyAll ) {
			this.m_action = "reply_all";
			var destinatariesOnCC = "";
			var fromAddress = this.m_interaction.getAddressByAccount().toLowerCase();
			//get all people addressed in the TO and CC fields, iterating and removing our own account
			var addDestinatariesFn = function(destinatariesList)
			{
				for (var email in destinatariesList) {
					if ( destinatariesList.hasOwnProperty(email) ) {
						email = email.toLowerCase();
						if ( email !== fromAddress) {
							if (!addedAddresses[email]) {
								addedAddresses[email] = true;
								destinatariesOnCC += destinatariesOnCC.length > 0 ? "; " : "";
								destinatariesOnCC += self.m_interaction.formatAddress(email, destinatariesList[email]);
							}
						}
					}
				}
			};
			var to = this.m_interaction.getTo(index, true);
			addDestinatariesFn(to);
			var cc = this.m_interaction.getCC(index, true);
			addDestinatariesFn(cc);

			$("#mailCC").val( destinatariesOnCC );
			//move that people to the CC field
			if (destinatariesOnCC.length > 0) {
				this.showCCSection();
			}
			else {
				this.hideCCSection();
			}
		}
		else {
			this.m_action = "reply";
			this.hideCCSection();
			this.hideBCCSection();
		}
		this.toggleAttachmentSection(false);
		$("#mailBodyContent").wysiwyg("shell").getElement().focus();
	};

	MailClientWindow.prototype.deleteAttachment = function( attachId, callback ) {
		callback = callback || $.noop;

		var uri = format(
			"{0}/inconcert/apps/repository/erase/?id={1}&section={2}"   , /* uri         */
			URI_DOMAIN                                                   , /* domain      */
			encodeURIComponent( attachId )                               , /* attach guid */
			"-1"                                                           /* section     */);

		inConcert.invoke({
			url: uri,
			dataType: "json",
			contentType: "application/json",
			success: function( response ) {
				callback( response );
			},
			error: function( /* error */ ) {
				callback( false );
			}
		});
	};

	MailClientWindow.prototype.delete_repository_files = function(remove_list)
	{
		if ( !$.isArray(remove_list) || remove_list.length === 0 ) {
			return;
		}
		inConcert.invoke({
			url: URI_DOMAIN + "/inconcert/apps/repository/multipleserase/",
			type: "POST",
			data: JSON.stringify( remove_list ),
			dataType: "json",
			contentType: "application/json",
			success: $.noop,
			error: function( /* response, error */ ) {}
		});
	};

	MailClientWindow.prototype.deleteAttachments = function()
	{
		if (!this.m_uploadedFiles || this.m_uploadedFiles.length === 0) {
			return;
		}
		var remove_list = [];
		for (var i = 0; i < this.m_uploadedFiles.length; i++) {
			if (!this.m_uploadedFiles[i].sharedFile) {
				remove_list.push( {
					id:      this.m_uploadedFiles[i].id,
					section: "-1"
				});
			}
		}
		this.m_uploadedFiles = [];
		this.delete_repository_files(remove_list);
	};

	//Discard current mail
	MailClientWindow.prototype.discardMail = function()
	{
		// aca oculto la cruz de delete de los attach de un forward
		$("#mailAttachments").find(".butMailAttachDelete").unbind("click").hide();
		this.m_deletedCurrentEventAttachments = {};
		this.m_forwardOrReplyAttachmentsEvent = null;
		this.deleteAttachments();
		this.toggleAttachmentSection(false);
		this.hideCCSection();
		this.hideBCCSection();
		this.destroySpellChecker();
		this.readCurrentMail();
	};

	MailClientWindow.prototype.destroySpellChecker = function()
	{
		if ( !this.m_spellchecker ) {
			return;
		}
		this.m_spellchecker.destroy();
		this.m_spellchecker = null;
	};

	MailClientWindow.prototype.allowedPrint = function()
	{
		let app_id = this.from_app;
		if (app_id === "interaction") {
			if (!this.m_interaction.isAssigned()) {
				return false;
			}
			let logged_user = inConcert.app.getLoggedUserId();
			let owner = this.m_interaction.getConversationOwner();
			if (logged_user !== owner) {
				return false;
			}
		}
		return this.hasPermissionToAction("MAIL_PRINT", app_id);
	};

	/*
		Cuando llega el button por parametro la impresion fue invocada desde la opcion "print" del menu contextual.
		Cuando el mismo no llega por parametro, la impresion fue invocada desde el Agent Api (funcion __sendDraftTextMessage)
	*/
	MailClientWindow.prototype.print = function($menu_print_button)
	{
		const $print = $menu_print_button || $("#mailPrint");
		const allowed_print = this.allowedPrint();
		let href;

		if (!allowed_print) {
			const message_error = "<%=_T('You not has permission for print.')%>";
			this.displayStatusMessage( message_error, "Error" , false, 5000 );
			// Cuando no se encuentra permitda la impresion (puede ser por permisos cambiados en tiempo real),
			// se debe de asignar al href el siguiente valor para que NO se ejecute el valor del link.
			href = "javascript:void(0);";
		}
		else {
			/*
			ATENCION: MUY IMPORTANTE !!!
				En el attr "rel" asignamos el valor "noreferrer" para que el popup/tab que realiza la impresion, se ejecute en un thread diferente
				al que se ejecuta la la ventana padre.
				Con esto lo que logramos es que cuando se abre el dialogo de impresion del sistema, el flujo de la ventana donde correr i6 no se detenga,
				y el no exista desconexion por timeout contra el server.

				Referencia: http://stackoverflow.com/questions/39574000/how-to-open-a-new-tab-in-separate-thread-with-javascript-chrome
			*/
			$print.attr("rel", "noreferrer");
			$print.attr("target", "_blank");
			let interaction_id = this.m_interaction.id;
			let server_event = this.m_currentEvent + 1; // los eventos en lua (array) comienzan en 1 y en javascript en 0, por eso incremento en 1. 
			href = "/inconcert/apps/agent/interactions/print/" + interaction_id + "/" + server_event;
		}

		$print.attr("href", href);
	};


	MailClientWindow.prototype.createSpellChecker = function()
	{
		// reference: http://jquery-spellchecker.badsyntax.co/bootstrap-wysihtml5.html
		if ( !$.hasOwnProperty("SpellChecker")) {
			return false;
		}
		var self = this;
		this.destroySpellChecker();
		var $body = this.$getBodyEditor();

		var validate_spellchecker_permissions = function()
		{
			// add to dictionary permission
			var allowed_add_to_dictionary = self.hasPermissionToAction("SPELL_CHECKER_ADD_WORD");
			var $add_dictionary = $body.find(".ignore-forever");
			if ( allowed_add_to_dictionary ) {
				$add_dictionary.show();
			}
			else {
				$add_dictionary.hide();
			}
		};

		var vcc = inConcert.app.get_vcc_id();
		var language = this.get_language();

		// create the html spellchecker
		this.m_spellchecker = new $.SpellChecker(
			$body,
			{
				lang: language,
				parser: "html",
				webservice: {
					path: URI_DOMAIN + "/inconcert/apps/spellchecker/spellcheck/" + vcc,
					driver: "PSpell"
				},
				local: {
					requestError:  "<%=_T('There was an error processing the request.')%>",
					ignoreWord:    "<%=_T('Ignore word')%>",
					ignoreAll:     "<%=_T('Ignore all')%>",
					ignoreForever: "<%=_T('Add to dictionary')%>",
					loading:       "<%=_T('Loading...')%>",
					noSuggestions: "<%=_T('(No suggestions)')%>"
				},
				suggestBox: {
					position: "below"
					// TODO: FIX-ME #3948 Aca se debe ajustar la posicion del popup de palabras sugeridas.
					//  Hay que realizar correctamente el posicionamiento, modificando el codigo comentado, ver si no hay algo en los CSS que este informando mal el alto del body (creo que viene por ahi),
					//   y cuando se vaya a realizar esto, hay que eliminar la linea -> position: "below"
					/*
					position: function()
					{
						var element = this.wordElement.data("firstElement") || this.wordElement;
						var offset = element.offset();
						var boxOffset = this.config.suggestBox.offset;
						var containerHeight = this.container.outerHeight();

						var positionAbove = (offset.top - containerHeight - boxOffset);
						var positionBelow = (offset.top + element.outerHeight() + boxOffset);

						var left = offset.left;
						var top = positionBelow;

						var $win = $(window);
						var window_height = $win.height();
						var window_scrolltop = $win.scrollTop();

						if ( (window_height + window_scrolltop) < (positionBelow + containerHeight) ) {
							top = positionAbove;
						}
						this.container.css({ top: top, left: left });
					}
					*/
				}
			}
		);

		var waiting_time_ms = 5000;
		// Bind spellchecker handler functions
		this.m_spellchecker.on("check.success", function() {
			self.displayStatusMessage("<%=_T('Successfully spelling.')%>", "Ok", false, waiting_time_ms );
		});

		this.m_spellchecker.on("check.fail", function( /*errors*/ ) {
			var message = "<%=_T('There are incorrect words.')%>";
			self.displayStatusMessage( message, "Info", false, waiting_time_ms );
			validate_spellchecker_permissions();
		});
		return true;
	};
	
	let attachmentStatusClasses = {
		download: "mailAttachDownload",
		done : "mailAttachDone",
		error : "mailAttachError"
	};
	
	MailClientWindow.prototype.makeAttachmentNode = function(fileName, status) {
		status = status || "download";
		return $("".concat(
			"<div class='mailAttachUpload'>",
				"<a class='mailAttachItem ", attachmentStatusClasses[status] , "' href='javascript:void(0);'>",
					"<span class='mailAttachItemLabel'>", fileName, "</span>",
					"<span></span>",
				"</a>",
				"<i class='butMailAttachDelete'></i>",
			"</div>"));
	};
	
	MailClientWindow.prototype.appendAttachmentNode = function($node) {
		$("#mailAttachments").append($node);
	};
	
	MailClientWindow.prototype.bindAttachmentNodeDelete = function($node, onDelete) {
		let $delete = $node.find(".butMailAttachDelete");
		$delete.off("click");
		if (onDelete) {
			$delete
				.show()
				.on("click", onDelete);
		}
		else {
			$delete.hide();
		}
	};
	
	MailClientWindow.prototype.setAttachmentNodeStatus = function($node, status) {
		let $item = $node.find(".mailAttachItem");
		$.each(attachmentStatusClasses, function(classStatus, className) {
			$item.toggleClass(className, classStatus === status);
		});
	};
	
	MailClientWindow.prototype.setAttachmentNodeLabel = function($node, label) {
		$node.find(".mailAttachItemLabel").text(label || "");
	};

	MailClientWindow.prototype.__onDeleteAttachment = function( attachGuid, $attach ) {
		const self = this;
		for (let i = 0; i < self.m_uploadedFiles.length; i++) {
			if (self.m_uploadedFiles[i].id === attachGuid) {
				self.m_uploadedFiles.splice(i, 1);
				break;
			}
		}
		// if attach was correctly uploaded must be removed from server now
		const $attach_item = $attach.find(".mailAttachItem");
		if ( $attach_item.hasClass("mailAttachDone") )
		{
			self.deleteAttachment( attachGuid, function( success ) {
				if ( !success ) {
					const waiting_time_ms = 3000;
					self.displayStatusMessage("<%=_T('Error deleting uploaded file.')%>", "Error", false, waiting_time_ms );
					return;
				}
				// only remove the node when delete is succesful
				$attach.remove();
				// if upload has failed and no files left disable attachment section again
				if ( $("#mailAttachments").children().length === 0 ) {
					self.toggleAttachmentSection(false);
				}
			});
		}
		else if ($attach_item.hasClass("mailAttachError"))
		{
			// remove the node no matter what
			$attach.remove();
			// if upload has failed and no files left disable attachment section again
			if ( $("#mailAttachments").children().length === 0 ) {
				self.toggleAttachmentSection(false);
			}
		}
	};

	MailClientWindow.prototype.uploadAttachmentFile = function(file)
	{
		const self = this;
		if (!file) {
			return;
		}

		const $file_picker_ctrl = $("#addNewAttachment");

		// Check file size. Do not upload files we know will fail.
		// 64 MB is what the cookbook uses. Needs a knob.
		if (file.size > 64 * 1024 * 1024) {
			const waiting_time_ms = 3000;
			this.displayStatusMessage("<%=_T('File is too big to be attached.')%>", "Error", false, waiting_time_ms );
			return;
		}

		this.toggleAttachmentSection(true);

		const attachGuid = inConcert.Utils.generateGuid();
		const extension = inConcert.Utils.getFileExtension( file.name );
		const contentType = file.type;

		const uri = format("{0}/inconcert/apps/repository/new/?id={1}&section={2}&contentType={3}&format={4}&fileName={5}&proxyType=MAIL&vcc={6}" ,
			URI_DOMAIN,
			encodeURIComponent( attachGuid ),
			"-1", /* section */
			contentType,
			extension, /* format */
			file.name,
			inConcert.Utils.md5(inConcert.app.get_vcc_id().toLowerCase())
		);

		// create tracking node
		const $attachNode = this.makeAttachmentNode(file.name);

		const xhr = new XMLHttpRequest();
		xhr.open('POST', uri, true);
		xhr.onload = function(e) {
			self.setAttachmentNodeLabel($attachNode, file.name);
			if (e.target.status === 200) { // success
				self.setAttachmentNodeStatus($attachNode, "done");
				let uploaded_file = {};
				uploaded_file.id = attachGuid;
				uploaded_file.name = file.name;
				uploaded_file.type = contentType;
				uploaded_file["content-disposition"] = "attachment";
				self.m_uploadedFiles.push(uploaded_file);
			}
			else { // error
				self.setAttachmentNodeStatus($attachNode, "error");
				if ($("#mailAttachments").children().length === 0) {
					self.toggleAttachmentSection(false);
				}
			}
			$file_picker_ctrl.val("");
		};

		// show the uploaded percentage
		xhr.upload.onprogress = function(e) {
			let percentage = "";
			if ( e && !isNaN( e.loaded ) && !isNaN( e.total ) ) {
				percentage = Math.floor( e.loaded / e.total * 100 ) + "%";
			}
			self.setAttachmentNodeLabel($attachNode, percentage);
		};


		self.bindAttachmentNodeDelete($attachNode, function() {
			self.__onDeleteAttachment(attachGuid, $attachNode);
		});
		self.appendAttachmentNode($attachNode);

		xhr.send(file);
	};

	MailClientWindow.prototype.canSendAttachment = function() {
		if (!this.m_interaction) {
			return false;
		}
		if (this.is_monitor) {
			return false;
		}
		if (!(this.m_interaction.isAssigned())) {
			return false;
		}
		return true;
	};

	MailClientWindow.prototype.getFileOptions = function() {
		const self = this;

		let options = MailClientWindow.uber.getFileOptions.call(this); //invokes the base class method

		delete options.device;

		options.attach = {
			Id: "attach",
			name : "<%=_T('From PC')%>",
			order : 2,
			description : true,
			className: "actionAttachFileFromPC",
			bind_click: function(_, $option) {
				const $file_button = $option.find("[id='" + "menuAddNewAttachment" + "']");
				self.initAttachmentsUploader(true, $file_button);
			},
			$get_menu_item_html: function(option) {
				const $option = $("".concat(
					"<div option='" + option.Id + "' class='interactionMenuItem " + (option.className || "") + "'>",
						option.name,
						"<input id='menuAddNewAttachment' type='file' hidden />",
					"</div>"
				));
				return  $option;
			}
		};

		return options;
	};

	/**
	Initializes the control to upload attachments. It uses an XHR request to upload the file.
	*/
	MailClientWindow.prototype.initAttachmentsUploader = function(enabled, $file_button)
	{
		const self = this;

		if (!enabled) {
			return;
		}

		// The <input type="file"> button cannot be styled by css
		// So we have a another element (a div) which can be styled. In that element's click handler,
		// we forward the click event to the actual <input>
		// Note: to do that (the forwarding), we need the raw DOM element instead of the jQuery wrapped one
		// because $file_picker_ctrl.click() somehow causes a stack overflow (it seems to retrigger the
		// 'click' on the parent).
		// See: http://ericbidelman.tumblr.com/post/14636214755/making-file-inputs-a-pleasure-to-look-at
		//
		const $file_picker_ctrl = $file_button || $("#addNewAttachment");
		const $parent = $file_picker_ctrl.parent();

		$parent.unbind("click");
		$file_picker_ctrl.unbind("change");

		$parent.click(function() {
			$file_picker_ctrl[0].click();
			self.disposeFilesMenu();
		});

		$file_picker_ctrl.change(function(e) {
			const file = e.target.files[0];
			self.uploadAttachmentFile(file);
		});
	};

	MailClientWindow.prototype.startDownloadFile = function(file, attachLink, callback = $.noop)
	{
		let uri = format(
			"{0}{1}?id={2}&section={3}&part={4}", /* full uri (domain + uri get + params) */
			URI_DOMAIN                          , /* domain                               */
			URI_REPOSITORY_GET                  , /* repo uri get                         */
			encodeURIComponent(file.id)         , /* file id                              */
			"-1"                                , /* section */
			"0"                                   /* part                                 */
		);

		attachLink
			.removeClass("mailAttachDone mailAttachError")
			.addClass("mailAttachDownload");

		$.fileDownload( uri,
			{
				httpMethod : "GET",
				cookieName : "download_started" // cookie exist in repository!!
			})
			.done(function() {
				attachLink.removeClass("mailAttachDownload").addClass("mailAttachDone");
				callback( true );
			})
			.fail(function(response) {
				attachLink.removeClass("mailAttachDownload").addClass("mailAttachError");
				try {
					response = JSON.parse( response ).result;
				} catch ( err ) {}
				callback( false, response );
			});
	};

	/*
		If flag is true attachment section is created if not exist
		If flag is fals attachment section is removed if existent
		If flag is undefined it just toggles according to current state
	*/
	MailClientWindow.prototype.toggleAttachmentSection = function(shown)
	{
		const $attachments = $("#mailAttachmentsSection");
		if (shown) {
			$attachments.show();
			return;
		}
		$attachments.hide();
	};

	MailClientWindow.prototype.showCCSection = function()
	{
		$("#mailCCSection").show();
		var $mail_to = $("#mailTo");
		if ( $mail_to.val().trim().length === 0 ) {
			$mail_to.focus();
		}
		else {
			$("#mailCC").focus();
		}
		this.adjustBodyTop();
	};

	MailClientWindow.prototype.hideCCSection = function()
	{
		$("#mailCCSection").hide();
		$("#mailTo").focus();
		this.adjustBodyTop();
	};

	MailClientWindow.prototype.showBCCSection = function()
	{
		$("#mailBCCSection").show();
		this.adjustBodyTop();
	};

	MailClientWindow.prototype.hideBCCSection = function()
	{
		$("#mailBCCSection").hide();
		this.adjustBodyTop();
	};

	MailClientWindow.prototype.toggle_cc_bcc_sections = function()
	{
		var is_visible_cc  = $("#mailCCSection").is(":visible");
		var is_visible_bcc = $("#mailBCCSection").is(":visible");

		if ( is_visible_cc && is_visible_bcc ) {
			if ( this.m_action === "reply_all" ) {
				var cc = this.m_interaction.getCC( this.m_currentEvent );
				if ( !cc || cc.length === 0 ) {
					this.hideCCSection();
				}
			}
			else {
				this.hideCCSection();
			}
			this.hideBCCSection();
		}
		else {
			this.showCCSection();
			this.showBCCSection();
		}
	};

	MailClientWindow.prototype.adjustBodyTop = function() {
		const $mailBody = $("#mailBodySection");

		if (!this.m_mailBodyOriginalTop) {
			/*
				La primera vez que ingresa a esta funcion, cacheo la posicion top original del body (css/masterClientMail.css clase: mailBody)
				Asi luego la utilizo como base para realizar los calculos y reposicionar el top segun se visualice o no el "cc" y el "bcc"
			*/
			this.m_mailBodyOriginalTop = parseInt($mailBody.css("top"), 10);
		}
		const $cc = $("#mailCCSection");
		const $bcc = $("#mailBCCSection");

		let increment = 0;
		if ($cc.is(":visible")) {
			increment = increment + $cc.height();
		}
		if ($bcc.is(":visible")) {
			increment = increment + $bcc.height();
		}

		const $mailBodyHtml = $("#mailBodyHTML");
		let newTop = this.m_mailBodyOriginalTop + increment + (5 + 26);
		$mailBody.css("top", newTop);
		$mailBodyHtml.css("top", newTop);
	};

	MailClientWindow.prototype.setContent = function(content, callback = $.noop) {
		$("#mailBodySection").hide();
		$("#mailBodyHTML").children().remove();
		$("#mailBodyHTML").show().append("<iframe src='/inconcert/apps/agent/pages/mailBody.<%=_L%>.html' width='100%' height='100%' frameborder='0' scrolling='yes' id='iframeMailContent'></iframe>");

		content = content || "";
		var setContentFn = function() {
			const $body = $("#iframeMailContent").contents().find("#iframe_mail_body");
			if ($body.length === 0) {
				window.setTimeout(setContentFn, 10);
				return;
			}
			$body.html(content);
			// Bindeo el click de los links, para que los mismos se abran en una nueva ventana.
			inConcert.Utils.bind_href($body);
			callback();
		};
		setContentFn();
	};

	//Esta funcion obtiene el bcc, y excluye el outgoing bcc configurado en la cuenta de la campana
	MailClientWindow.prototype.get_bcc = function( event_index )
	{
		var it = this.m_interaction;

		var campaign = this.getCampaign( it.campaign );
		if ( !campaign ) {
			return "";
		}

		var account_id = it.getAccount() || "";
		var account = campaign.getAccount( account_id, "MAIL" );
		if ( !account ) {
			return "";
		}
		var outgoing_bcc = account.outgoingBCC;
		if ( !outgoing_bcc ) {
		// si no tiene outgoing bcc configurado en la cuenta de la campana
			return it.getBCC( event_index );
		}

		outgoing_bcc = outgoing_bcc.toLowerCase();
		var bcc_object_list = it.getBCC( event_index, true /* return as table */ );

		var bcc_addresses = "";
		var added_bcc_addresses = {};
		$.each( bcc_object_list, function( email, email_desc )
		{
			email = email.toLowerCase();
			if ( email !== outgoing_bcc ) {
				if ( !added_bcc_addresses[email] ) {
					added_bcc_addresses[email] = true;
					bcc_addresses += bcc_addresses.length > 0 ? "; " : "";
					bcc_addresses += it.formatAddress( email, email_desc );
				}
			}
		});

		return bcc_addresses;
	};

	MailClientWindow.prototype.getEventContentHTML = function(event_index)
	{
		if (isNaN(event_index)) {
			return "";
		}
		var content_html = this.m_interaction.getHTMLBody(event_index);
		return content_html;
	};

	MailClientWindow.prototype.updateSideBar = function()
	{
		MailClientWindow.uber.updateSideBar.call(this); //invokes the base class method
		if (!this.sidebar) {
			return;
		}
		if (this.m_composingMail) {
			this.sidebar.showActions();
		}
		else {
			this.sidebar.hideActions();
		}
	};

	//Loads interaction data in read mode
	MailClientWindow.prototype.readCurrentMail = function(callback = $.noop)
	{
		var self = this;
		$("body").removeClass("mailClientWrite").addClass("mailClientRead");

		var event_index = this.m_currentEvent;
		var current_event = this.m_interaction.getEvent(event_index);

		var readonly = this.m_interaction.hasPendingLabel();
		if (!readonly) {
			readonly = this.m_interaction.state !== "PENDING";
		}
		this.updateNavigationButtons();
		self.__setComposingMail(false);
		this.updateSideBar();
		this.m_action = "read";
		$("#creationDate").text(this.m_interaction.getLocaleDateString( event_index )).prop("readonly", readonly );
		var content_html = this.getEventContentHTML(event_index);
		this.setContent(content_html, callback);

		let subject = this.m_interaction.getSubject(event_index);
		if (subject) {
			$("#mailSubject").val(subject).prop("readonly", readonly);
		}

		this.setPeopleFrom();
		this.setTOLabel();

		// "TO" needs to show contact info
		var eventTO = this.m_interaction.getTo(event_index);
			
		if (this.m_interaction.isFinished() && this.m_interaction.accountHasContactManagementEnabled( this.from_app )){
			// FIX-ME: Removing this and appending html this
			// ways seems wrong.
			var container = $("#mailTo").parent();
			$("#mailTo").remove();
			container.append("<a id='mailTo' class='mailToLink' href='javascript:void(0);'>" + inConcert.Utils.escapeAttrib(eventTO) + "</a>");
			$("#mailTo").click(function(){
				var contactWindow = new inConcert.ContactWindow({
					from_app: self.from_app,
					interaction : self.m_interaction,
					name : '',
					avatar : self.m_interaction.getEventUserAvatarURI( current_event ),
					addresses : [_.findWhere(current_event.addresses, {"source" : "to"}).address]
				});
				contactWindow.show();
			});
		} else {
			$("#mailTo").val(eventTO).prop("readonly", readonly );
		}

		if (this.m_interaction.isInternalMessage(event_index)) {
			$("#mailReply").hide();
			$("#mailReplyAll").hide();
		}
		else {
			$("#mailReply").show();
			$("#mailReplyAll").show();
		}

		this.updateReadActionsButtons();

		// cc addresses
		var cc = this.m_interaction.getCC( event_index );
		$("#mailCC").val( cc ).prop("readonly", readonly );
		if (cc.length > 0) {
			this.showCCSection();
		}
		else {
			this.hideCCSection();
		}

		// bcc addresses
		var bcc = this.get_bcc( event_index );
		$("#mailBCC").val( bcc ).prop("readonly", readonly );
		if ( bcc.length > 0 ) {
			this.showBCCSection();
		}
		else {
			this.hideBCCSection();
		}

		var content_disposition = "attachment";
		var attachments = this.m_interaction.getAttachmentsEvent( event_index, content_disposition );
		if (attachments && attachments.length > 0) {
			this.toggleAttachmentSection(true);
			$("#mailAttachments").children().remove();
			var realAttachs = 0;
			for (var i = 0; i < attachments.length; i++) {
				if (attachments[i].name && attachments[i].name.length > 0) {
					realAttachs++;
					var $attach = $(
						"<div class='mailAttachUpload'>" +
							"<a class='mailAttachItem' href='javascript:void(0);'>" + inConcert.Utils.escapeAttrib( attachments[i].name ) + "</a>" +
							"<i class='butMailAttachDelete' hidden></i>" +
						"</div>"
					);
					$attach.find(".mailAttachItem").data("attachment", attachments[i] );
					$("#mailAttachments").append( $attach );
				}
			}
			if (!realAttachs) {
				this.toggleAttachmentSection(false);
			}
			$("#mailAttachments").find("a").click(function() {
				if ( self.m_composingMail ) {
					return;
				}
				var $attach = $(this);
				var attach = $attach.data("attachment");
				self.startDownloadFile( attach, $attach );
			});
			//indicate how many real attachments the mail has
			$("#mailAttachmentsSection .mailLabel").text(format("<%=_T('Attachs.({0}):')%>", realAttachs));
		}
		else {
			this.toggleAttachmentSection(false);
		}
		$("#elapsedTime").text(this.m_interaction.currentTime());
		this.setEventExpander(true);
		this.adjustBodyTop();
	};

	MailClientWindow.prototype.clientUpdate = function() {
		this.updateReadActionsButtons();
	};

	MailClientWindow.prototype.onInteractionGone = function(interaction) {
		MailClientWindow.uber.onInteractionGone.call( this, interaction ); //invokes the base class method
		$("body").addClass("clientReadOnly");
		this.updateReadActionsButtons("hide");
	};

	MailClientWindow.prototype.setTooltips = function()
	{
		MailClientWindow.uber.setTooltips.call( this );
		// tooltips in navegation
		$("#butRelatedPrevious").inconcert_tooltip();
		$("#butRelatedNext").inconcert_tooltip();
	};

	// navigation in events ----------
	//
	MailClientWindow.prototype.updateNavegationTooltip = function()
	{
		// update tooltip navigation
		var all_events_length = this.m_interaction.getAllRelatedEventCount(); // todos
		var events_length = this.m_interaction.getEventCount(); // sin los autoresponse
		var current_event = this.m_currentEvent + 1; // el index corresponde al indice del array por eso sumo 1
		var has_autoresponse = all_events_length > events_length;

		if ( has_autoresponse ) {
			// si hay autoresponse tengo que calcular diferente ya que el indice del current event se corre
			current_event = current_event - ( all_events_length - events_length );
			if ( current_event <= 0 ) {
				current_event = 1;
			}
		}

		var previous = current_event - 1;
		var next = current_event + 1;

		$("#butRelatedPrevious").attr("tooltip", format("<%=_T('Go to {0} of {1}')%>", previous, events_length ) );
		$("#butRelatedNext").attr("tooltip", format("<%=_T('Go to {0} of {1}')%>", next, events_length ) );
	};

	MailClientWindow.prototype.getAttachmentOptions = function() {
		var options = MailClientWindow.uber.getAttachmentOptions.call(this);
		options.image.description = false;
		options.video.description = false;
		options.audio.description = false;
		options.file.description = false;
		return options;
	};
	
	MailClientWindow.prototype.__hasAttachmentPreview = function(/*option, data*/) {
		return false;
	};

	MailClientWindow.prototype.sendSharedFile = function(option, sharedFile) {
		let self = this;
		if (option.embeed) {
			return self.addSharedImage(sharedFile);
		}
		self.toggleAttachmentSection(true);
		let $attachNode = self.makeAttachmentNode(sharedFile.Name, "done");
		self.appendAttachmentNode($attachNode);
		
		self.m_uploadedFiles.push({
			id : sharedFile.RepositoryId,
			name : sharedFile.Name,
			type : sharedFile.ContentType,
			"content-disposition" : "attachment",
			sharedFile : sharedFile.Id
		});
		
		self.bindAttachmentNodeDelete($attachNode, function() {
			self.m_uploadedFiles = self.m_uploadedFiles.filter(function(file) {
				return file.sharedFile !== sharedFile.Id;
			});
			$attachNode.remove();
			if ( $("#mailAttachments").children().length === 0 ) {
				self.toggleAttachmentSection(false);
			}
		});
	};
	
	MailClientWindow.prototype.showSharedImageSelector = function() {
		let option = this.getAttachmentOptions().image;
		option.Id = "image";
		option.embeed = true;
		this.__showSharedFileSelector(option);
	};
	
	MailClientWindow.prototype.addSharedImage = function(sharedImage) {
		let html = format("<img src='{0}' repo_id='{1}' shared_file='{2}' title='{3}' style='max-width:100%;max-height:20em;'><br><br>",
			inConcert.app.getRepositoryAPI().getUrl({ id : sharedImage.RepositoryId, part: 0, relative : true }),
			sharedImage.RepositoryId, sharedImage.Id, sharedImage.Name);
		this.__setExternalMessage(html);
	};
	
	MailClientWindow.prototype.__makeDraftMessageDestinationAddress = function(address) {
		if ($.isPlainObject(address)) {
			return address.name? format("'{0}' <{1}>", address.name, address.address) : address.address;
		}
		else {
			return address || "";
		}
	};
	
	MailClientWindow.prototype.__makeDraftMessageDestination = function(destination) {
		let self = this;
		if ($.isArray(destination)) {
			return destination.reduce(function(text, address) {
				let raw = self.__makeDraftMessageDestinationAddress(address);
				return text + ((text && raw)? ";" : "") + raw;
			}, "");
		}
		else if (destination) {
			return self.__makeDraftMessageDestinationAddress(destination);
		}
		else {
			return "default";
		}
	};
	
	MailClientWindow.prototype.__makeDraftMessageDestinations = function(message) {
		return {
			to : this.__makeDraftMessageDestination(message.to),
			cc : this.__makeDraftMessageDestination(message.cc),
			bcc : this.__makeDraftMessageDestination(message.bcc)
		};
	};
	
	MailClientWindow.prototype.__addDraftTextMessage = function(message, callback) {
		const self = this;
		if (!message.replyEvent && message.replyEvent !== 0) {
			message.replyEvent = this.m_interaction.events? this.m_interaction.events.length - 1 : -1;
		}
		const $discardCurrentMail = $("#discardCurrentMail");
		if ($discardCurrentMail && typeof this.inflating !== "undefined") {
			$discardCurrentMail.click();
		}
		this.navigateEvent(message.replyEvent, function() {
			if (message.print) {
				return callback(true);
			}
			else if (message.mailReply) {
				$("#mailReply").click();
			}
			else if (message.mailReplyAll) {
				$("#mailReplyAll").click();
			}
			else if (message.mailForward) {
				$("#mailForward").click();
			}
			if (message.subject) {
				$("#mailSubject").val(message.subject);
			}
			if (message.attachments) {
				for (var i = 0; i < message.attachments.length; i++) {
					var attach = message.attachments[i];
					const $attachNode = self.makeAttachmentNode(attach.name);
					self.setAttachmentNodeStatus($attachNode, "done");
					let uploaded_file = {};
					uploaded_file.id = attach.id;
					uploaded_file.name = attach.name;
					uploaded_file.type = attach.fileType;
					uploaded_file["content-disposition"] = "attachment";
					self.m_uploadedFiles.push(uploaded_file);
				}
			}
			self.__prepend_body_content(message.body || message.text || "");
			let destinations = self.__makeDraftMessageDestinations(message);
			if (destinations.to !== "default") {
				$("#mailTo").val(destinations.to);
			}
			if (destinations.cc !== "default") {
				$("#mailCC").val(destinations.cc);
				if (destinations.cc && $("#mailCC").is(":hidden")) {
					$("#toggleBCCSection").click();
				}
			}
			if (destinations.bcc !== "default") {
				$("#mailBCC").val(destinations.bcc);
				if (destinations.bcc && $("#mailBCC").is(":hidden")) {
					$("#toggleBCCSection").click();
				}
			}
			if(self.m_interaction.interactionData.fromAPI) {
				$("#mailBodySection").show();
				$("#mailBodyHTML").hide();
			}
			callback(true);
		});
	};

	MailClientWindow.prototype.__sendDraftTextMessage = function(message, callback)
	{
		if (message.print) {
			if (this.allowedPrint()) {
				this.print();
				let url = $("#mailPrint").attr("href");
				if (url) {
					window.open(url);
					callback(true);
				}
				else {
					callback(false, "can not print mail");
				}
			}
			else {
				callback(false, "can not print mail");
			}
		}
		else {
			this.send(callback);
		}
	};

	MailClientWindow.prototype.onSidebarIntialisedLayout = function()
	{
		const self = this;

		var callback = function() {
			MailClientWindow.uber.onSidebarIntialisedLayout.call(self); //invokes the base class method
			self.__initialize_draft();
		};

		if (!this.m_interaction || this.m_interaction.isBeingComposed()) {
			callback();
			return;
		}
		/*
			Tengo que esperar, a que el layout se haya inicializado, para poder visualizar el body del mail.
			Ref #8533
		*/
		this.navigateToBestEvent(callback);
	};
	
	MailClientWindow.prototype.firstLoadDraftMessage = function() {
		if(this.m_interaction.interactionData.fromAPI && this.m_interaction.state === "PENDING") {
			MailClientWindow.uber.firstLoadDraftMessage.call(this); //invokes the base class method
		}
		else {
			this.__loadDraftMessage = true;
		}
	};

	MailClientWindow.prototype.getTitle = function()
	{
		let window_title = "";
		if ( this.m_interaction && !this.m_interaction.isBeingComposed() ) {
			window_title = this.m_interaction.getClientName();
			if ( !window_title || window_title.length === 0 ) {
				var first_outgoing_event = this.m_interaction.getFirstOutgoingEvent();
				if ( first_outgoing_event ) {
					window_title = this.m_interaction.get_agent_name( first_outgoing_event );
				}
			}
		}
		if ( !window_title || window_title.length === 0 ) {
			window_title = this.getWindowDefaultTitle();
		}
		return window_title;
	};

	MailClientWindow.prototype.onMinimize = function(/*event*/)
	{	
		MailClientWindow.uber.onMinimize.call(this);
		if (this.m_interaction.hasDraft()) {	
			this.updateWindowClasses("hasDraft");
			let title = this.getTitle() + "<span class='draftText'>(<%=_T('Draft')%>)</span>";
			this.setTitle(title);
		}
	};

	MailClientWindow.prototype.onMaximize = function(/*event*/)
	{
		MailClientWindow.uber.onMaximize.call(this);
		this.updateWindowClasses();
		let title = this.getTitle();
		this.setTitle(title);
	};

	MailClientWindow.prototype.onRestore = function(/*event*/)
	{
		MailClientWindow.uber.onRestore.call(this);
		this.updateWindowClasses();
		let title = this.getTitle();
		this.setTitle(title);
	};

	// -------------------------------------

	//when jquery says so, start it all up
	$(document).ready(function() {
		window.hook_environment();
		inConcert.i18n.updateScreen();

		var callback = function() {
			var client = new MailClientWindow(window.interaction,
				{
					archive   : window.archive,
					from_app  : window.from_app,
					monitor   : window.monitor,
					queued    : window.queued,
					waiting   : window.waiting,
					readonly  : window.readonly || window.archive || window.monitor || window.queued || window.waiting,
					reply     : window.reply,
					spam      : window.spam,
					// TODO: #9778 - CANY - Docked.
					panel_name: window.panel_name
				}
			);

			window.onReady(client);

			if (window && window.view && window.view instanceof inConcert.ClientWindowProxy) {
				window.view.apply(client);
			}

			window.view = client;
		};

		if(window.interaction.isBeingComposed()) {
			callback();
		}
		else {
			window.interaction.inflate(callback);
		}
	});

}());
