/*jshint browser:true, quotmark:false, smarttabs:true, esversion: 11*/
/*global $, window, inConcert, AirLoginDialog*/

/**
This module has the MainWindow, which corresponds to pages/inboxMaster.html
*/

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	"use strict";

	const _T = inConcert.i18n.getString;
	const format = inConcert.Utils.format;
	let m_main;

	/**
	Class MainWindow.

	Main window from which everything happens.
	@param application
	*/
	function MainWindow (application)
	{
		window.id = "MainWindow";
		MainWindow.uber.constructor.call(this, {move : ["inboxMainHeader", "inboxMainFooter"]});

		const self = this;
		this.m_panels = {}; //the panels are indexed by type
		this.m_application = application;
		this.m_initialized = false;
		this.m_resize_handle = null;

		m_main = this;

		this.m_start_close = "no";

		$("#headerCloseButton").click(function() {
			if (inConcert.app.canExit()){
				inConcert.app.logout(function() {
					window.location = "/inconcert/apps/agent/login/";
				},"MainWindow-CloseButton");
			}
			else {
				self.showPopupMessage({
					message : "<%=_T('You shouldnt logout while still talking or chatting with someone!')%>",
					messageTitle: "<%=_T('Be polite and wrapup your conversations')%>",
					popUpTitle: "<%=_T('Not so fast!')%>",
					messageType: "Error",
					callback: function() {
						$("#tabAssignedActiveInteractions").click();
					}
				});
			}
		});

		$("#closeStateChangerPopup").click(function() {
			const menu = $(this).parent().parent();
			menu.find(".quickViewPanel").remove();
			menu.hide();
		});
		self.dropdownMenuInteractions = new inConcert.PopupWindow({parentNode:"dropdownMenuButtonInteractions", ownerId: "quickInteractionContainer"});
		const fnRePosition = function() {
			const botonRect = $("#userLoggedAvatarContainer")[0].getBoundingClientRect();
			const $container = $("#quickInteractionContainer");
			let right = botonRect.width + 41;
			let top = $("#userLoggedContainer").height();
			$container.css({ top: top, right: right, left: "auto"});
			let total = self.getTotalIframesWindowsDialogs() || 3;
			let zindex = (total * 10) + 1001;
			self.dropdownMenuInteractions.m_inlineStyleValues["z-index"] = zindex;
		};
		$("#dropdownMenuButtonInteractions").
		off("click")
		.on("click", function() {

			if ($("#quickInteractionContainer").is(":visible")) {
				self.dropdownMenuInteractions.finalize();
			}
			else {
				self.dropdownMenuInteractions.show({
					callbacks: {
						onOpen: function()
						{
							$("#quickInteractionContainer").show();
							fnRePosition();
						},
						onClose: function() {
							const $container = $("#quickInteractionContainer");
							$container.hide();
							$container.css("z-index", "");
						}
					}
				});
			}
		});

		$(window).resize(function() {
			if (self.m_resize_handle) {
				window.clearTimeout(self.m_resize_handle);
				self.m_resize_handle = null;
				return;
			}
			self.m_resize_handle = window.setTimeout(function() {
				self.onWindowResize();
				self.m_resize_handle = null;
			}, 100);
		});

		this.FLOATING_RIGHT_TOOLBAR_THRESHOLD = 900;
		this.FLOATING_RIGHT_SHORTTOOLBAR_THRESHOLD = 540;

		inConcert.app.messagebus.subscribe("/interactions/updated", function(interaction, old_interaction) {
			self.onInteractionUpdated(interaction, old_interaction);
		});

		inConcert.app.messagebus.subscribe("/windows/change_state", function( new_state, dialog ) {
			self.__windows_change_state( new_state, dialog );
		});

		inConcert.app.messagebus.subscribe("/loaded_user_permissions/updated", function( success ) {
			self.refreshPermissionGUI();
		});
	}

	//derive the main window class according to execution context
	inConcert.Utils.inherit( MainWindow, inConcert.Window );

	//Define functions and objects in the namespace in case they don't already exist
	if (typeof inConcert.MainWindow !== 'function') {
		inConcert.MainWindow = MainWindow;
	}

	/*
		Esta funcion se encarga de llevar una lista de todas las ventanas abiertas en todas las apps.
		Ya sea una ventana de FB, Twitter, chat, etc, como personal settings, change avatar etc.
		Esta lista contiene el id de la ventana y el estado actual: "restore", "minimize" "maximize", "hide"
		Si dentro de la lista de ventanas se encuentra alguna minimizada, agrega al BODY la clase "withMinimizedInteractions"
			para que las ventanas aparezcan minimizadas debajo de todo.
		En el caso de que no exista ninguna ventana minimizada esta clase es eliminada
	*/
	MainWindow.prototype.__windows_change_state = function( new_state, dialog )
	{
		if ( !new_state || !dialog ) {
			return;
		}
		//console.debug("MainWindow.prototype.__windows_change_state " + new_state + ", " + dialog.__window_id);
		this.windows = this.windows || {};
		if ( new_state === "create" ) {
			this.windows[ dialog.__window_id ] = { window_id: dialog.__window_id, state: new_state };
			return;
		}
		var current_window = this.windows[ dialog.__window_id ];
		if ( !current_window ) {
			return;
		}
		if ( current_window.state === new_state ) {
			return;
		}
		if ( new_state === "close" ) {
			delete this.windows[ dialog.__window_id ];
		}
		else {
			current_window.state = new_state;
		}
		var has_minimized_windows = false;
		$.each(this.windows, function( _, window_info )
		{
			if ( window_info.state === "minimize" ) {
				has_minimized_windows = true;
				return false; //break
			}
		});
		//console.debug("MainWindow.prototype.__windows_change_state has_minimized_windows: " + has_minimized_windows);
		var $body = $("body");
		$body.removeClass("withMinimizedInteractions");
		if ( has_minimized_windows ) {
			$body.addClass("withMinimizedInteractions");
		}
	};

	/*
		Metodo que realiza reload de los iFrames de Helpdesk (FullApp e Interactions->tickets) invocando a la clase de Helpdesk que corresponda.
			El mismo se utiliza desde el AgentApi
				Ej (secuencia de invocacion):
					(HelpDeskReloadApps) /shared/api/agent_api.js
						(__HelpDeskReloadApps) /apps/agent/scripts/agent_api_server.js
							(HelpDeskReloadApps) /apps/agent/scripts/main_window.js (esta funcion)
		Primero se fija en que App se encuentra (helpdesk, interaction) para realizar primero el reload a este iFrame y luego a la otra.
		Si no se encuentra en ninguna de estas 2 apps, comenzara el reload por la app Helpdesk (fullApp)
	*/
	MainWindow.prototype.HelpDeskReloadApps = function(callback = $.noop)
	{
		/* funcion que agrega el resultado del reload de la clase Helpdesk y luego invoca al callback relacionado*/
		let response = {results: []};
		const Response = (fn, resp) => {
			response.results.push(resp);
			fn(response);
		};
		/* Tickets: funcion que realiza el reload */
		const ReloadTickets = (fn)=> {
			if (!this.m_panels.conversations_panel)
				return Response(fn, {
					success: false, type: "there is no instance of the inConcert.ConversationsPanel class"
				});
			this.m_panels.conversations_panel.ReloadHelpdeskTickets(resp => Response(fn, resp));
		};
		/* FullApp: funcion que realiza el reload */
		const ReloadFullApp = (fn)=> {
			if (!inConcert.helpdesk_dashboard)
				return Response(fn, {
					success: false, type: "there is no instance of the inConcert.helpdesk.dashboard class"
				});
			inConcert.helpdesk_dashboard.reload(resp => Response(fn, resp));
		};
		// definicion de modulos por app
		const helpdeskModulesByApp = {
			helpdesk: {
				moduleId     : "full_app",
				nextReloadApp: "interaction", // el siguiente reload, despues que termina el actual
				reload       : ReloadFullApp
			},
			interaction: {
				moduleId     : "tickets",
				nextReloadApp: "helpdesk",    // el siguiente reload, despues que termina el actual
				reload       : ReloadTickets
			}
		};
		// 1. obtengo los modulos segun la app actual de OCC
		const currentApp = inConcert.app.getCurrentApp();
		const firstModuleToReload = helpdeskModulesByApp[currentApp] || helpdeskModulesByApp.helpdesk;
		const nextModuleToReload = helpdeskModulesByApp[firstModuleToReload.nextReloadApp];
		// 2. comienzo el reload relacionado
		firstModuleToReload.reload(()=> {
			nextModuleToReload.reload(callback);
		});
	};

	//called from the base class when the window is closed, in this case we should close all the opened windows, save settings and call logout
	MainWindow.prototype.onWindowClosed = function() {
		if (!inConcert.app.reloadForUpdate) {
			inConcert.app.logout("","MainWindow-onWindowClosed");
		}
		//TODO this is due to bad this on callback REVIEW!
		if (this.m_panels && this.m_panels.conversations_panel) {
			this.m_panels.conversations_panel.close_all_clients();
		}
		if (this.personalSettingsLoader) {
			this.personalSettingsLoader.close();
		}
		if (this.analyticsLoader) {
			this.analyticsLoader.close();
		}
		if (this.changeAvatarLoader) {
			this.changeAvatarLoader.close();
		}
		if ( inConcert.configuration ) {
			inConcert.configuration.on_shutdown();
		}

		if ( inConcert.report_design ) {
			inConcert.report_design.on_shutdown();
		}

		if ( inConcert.outbound_engine ) {
			inConcert.outbound_engine.on_shutdown();
		}

	};

	MainWindow.prototype.destroy = function() {

		if (this.apps_popup) {
			this.apps_popup.destroy();
			this.apps_popup = null;
			delete this.apps_popup;
		}
	};

	MainWindow.prototype.isInitialized = function() {
		return this.m_initialized;
	};

	//Creates the main window
	MainWindow.prototype.create = $.noop;

	// Before the window is closed. It can return false to prevent the window from being closed.
	MainWindow.prototype.onWindowClosing = function()
	{
		var self = this;

		if (!inConcert.app.canExit()) {
			this.showPopupMessage({
				message : "<%=_T('You shouldnt logout while still talking or chatting with someone!')%>",
				messageTitle: "<%=_T('Be polite and wrapup your conversations')%>",
				popUpTitle: "<%=_T('Not so fast!')%>",
				messageType: "Error",
				callback: function() {
					$("#tabAssignedActiveInteractions").click();
				}
			});
			this.activate();
			return false;
		}

		switch(this.m_start_close) {
			case "no":
				// start the close procedure
				this.m_start_close = "closing";
				window.setTimeout(function() {
					inConcert.app.getMainWindow().hide();

					inConcert.app.changeUserState("OFFLINE", function()
					{
						inConcert.app.uploadLog(function()
						{
							inConcert.app.logout(function() {
								// now, proceed as normal
								self.m_start_close = "ready";
								MainWindow.uber.onClose();
							},"MainWindow-Close Windows");
						});
					});
				}, 0);
			break;

			case "closing":
				// go away, we're in the middle of the close procedure.
			break;

			case "ready":
				// now we're ready for the window to be properly closed.
				return true;
		}

		// always refuse to be closed, except when in the "ready" state
		return false;
	};

	// //Called when the user has succesfully logged in, just forwards bootstrap request to the app
	// MainWindow.prototype.onLoggedIn = function(callbackFn) {
		// this.m_application.bootstrap(callbackFn);
	// };

	//Positionates the contact panel in the specified contact id
	MainWindow.prototype.showContact = function(contactID) {
		if (contactID) {
			this.m_panels.shared.get_tab("CONTACTS").showContact(contactID);
		}
	};

	// Positionates the contact panel in the specified contact id
	// and save the contact updated to the interaction
	MainWindow.prototype.assignContact = function(interaction, contact) {
		window.alert("MainWindow.prototype.assignContact - DEPRECATED");
		var mainWindow = inConcert.app.getMainWindow();
		interaction.contact = contact.Id;
		mainWindow.getPanel("AssignedInteractions").updateInteractionNode(interaction);
		var address = interaction.buildContactAddress();
		if (!address) {
			interaction.contact = "";
			return;
		}
		contact.updateAddress(address, address);
		var panel = m_main.getPanel("Contacts");
		panel.saveContact(contact, function(updatedContact) {
			var contactName = contact.Name || contact.Id;
			if (!updatedContact) {
				interaction.contact = "";
				mainWindow.showBannerMessage({
					message: format("<%=_T('The address {0} could not be added to {1}.')%>", interaction.getDisplayAddress(), contactName),
					messageType: "Alert",
					timeout: 5000
				});
				return;
			}
			mainWindow.showBannerMessage({
				message: format("<%=_T('The address {0} has been added to {1}.')%>", interaction.getDisplayAddress(),  contactName),
				messageType: "Info",
				timeout: 5000
			});
			var currentContactId = panel.m_handler.find('#contactDetailNameHeader').attr("contactId");
			if (currentContactId === updatedContact.Id) {
				var addressListPanel = panel.m_handler.find('.contentContactDetail > .contactDetailAddresses');
				panel.appendAddressNode(addressListPanel, panel.getAddressNode(updatedContact, address));
			}

			var set_contact_callback = function( success, messageError ) {
				if (success) {
					return;
				}
				m_main.showBannerMessage(
					{
						message: messageError,
						messageType:"Error",
						timeout : 4000
					},
					null,
					true
				);
			};
			var app_id = "interaction";
			interaction.setContact(
				updatedContact.Id,
				contactName,
				set_contact_callback,
				app_id
			);
		});
	};

	MainWindow.prototype.updatedContact = function(contact) {
		this.m_panels.shared.get_tab("CONTACTS").updatedContact(contact);
	};

	MainWindow.prototype.onShutdown = function() {
		//this can be called multiple times but is idempotent
		inConcert.app.logout("","MainWindow-onShutdown");
	};

	//Updates campaign counters by forwarding each campaign counters to the campaign panel
	MainWindow.prototype.updateCampaignCounters = function()
	{
		var campaigns = inConcert.app.getCampaigns();
		var campaign_panel = this.m_panels.shared.get_tab("CAMPAIGNS");

		var all_campaigns_counters = { queued: { agent : 0, campaign : 0 }, waiting : 0, abandoned : { agent : 0, campaign : 0 } };

		for (var i = 0; i < campaigns.length; i++) {
			var campaign = campaigns[i];
			var campaign_counters = campaign_panel.updateCounters(campaign);
			if ( campaign_counters ) {
				all_campaigns_counters.queued.campaign = all_campaigns_counters.queued.campaign + campaign_counters.totalUnassignedQueued;
				all_campaigns_counters.waiting = all_campaigns_counters.waiting + campaign_counters.totalWaiting;
				all_campaigns_counters.abandoned.campaign = all_campaigns_counters.abandoned.campaign + campaign_counters.totalUnassignedAbandoned;
			}
		}
		all_campaigns_counters.queued.agent = inConcert.app.getInteractionManager().countQueuedInteractions();
		all_campaigns_counters.abandoned.agent = inConcert.app.getInteractionManager().countAbandonedInteractions();
		
		this.m_panels.conversations_panel.update_counters(all_campaigns_counters);
	// TODO: ver si aca tengo que refrescar los waiting de forma individual, o ya se actualizo en la linea de arriba!
		var queued_panel = this.m_panels.conversations_panel.get_child_panel("queued");
		//Refresh panel before take any decisions here
		queued_panel.refreshPanel();

		// quickbar show/hide
		var allowed_app_interaction = inConcert.permissions.to_application("interaction");
		this.refreshQuickbar(allowed_app_interaction);

		//Counters already changed, it's time enable/disable quickbar take next interaction button
		var queuedCount = all_campaigns_counters.queued.agent + all_campaigns_counters.queued.campaign;
		if (!allowed_app_interaction || queuedCount === 0) {
			this.disableQuickTakeNextButton();
		}
		else {
			this.enableQuickTakeNextButton();
		}
	};

	//Receives updated interactions and refreshes the panels
	MainWindow.prototype.updateInteractions = function(interactions)
	{
		var panels = this.m_panels.conversations_panel.get_childs_panel();
		$.each(panels, function(panel_id, panel) {
			var info = panel.updateInteractions(interactions);
			panel.notify_desktop_new_interactions(info);
		});
		this.updateUnreadCounters();
		var assignedPanel = this.m_panels.conversations_panel.get_child_panel("assigned");
		if (assignedPanel) {
			assignedPanel.update_labels_counters();
		}
	};

	MainWindow.prototype.switchInteractions = function(oldId, interaction) {
		this.m_panels.conversations_panel.switch_window_interaction(oldId, interaction);
	};

	MainWindow.prototype.updateCampaignAccounts = function()
	{
		var campaigns = inConcert.app.getCampaigns();
		var campaign_panel = this.m_panels.shared.get_tab("CAMPAIGNS");
		var initializing_proxies;

		$.each(campaigns, function(undefined, campaign)
		{
			initializing_proxies = campaign_panel.updateAccountsStatus(campaign);
		});

		// update quickbar status
		if ( initializing_proxies === 'running' ) {
			// si es running debo marcar como conectado, con esto en la quickbar se marca como "waiting" los iconos.
			this.updateChannelStatus( undefined, "CONNECTING" );
		}
		else {
			this.updateChannelStatus();
		}

		//assigned interactions
		this.m_panels.conversations_panel.get_child_panel("assigned").onCampaignAccountUpdated();
	};

	MainWindow.prototype.updateChannelStatus = function( channel, status, message )
	{
		status = status || (status === undefined);

		var accounts_types;
		if ( channel ) { // update specific account type
			accounts_types = {};
			accounts_types[ channel ] = 0;
		} else {
		// update all accounts type
			accounts_types = { "CALL": 0, "CHAT": 0, "FACEBOOK": 0, "INSTAGRAM": 0, "YOUTUBE": 0, "APPSTORE": 0, "GOOGLEPLAY": 0, "RCS": 0, "IM": 0, "MAIL": 0, "SMS": 0, "WHATSAPP": 0, "TWITTER": 0, "WEBCONTACT": 0, "VOICEMAIL": 0 };
		}

		// count the active accounts by types on campaigns
		var countActiveCampaignAccountType = function(campaign)
		{
			$.each(accounts_types, function(proxy)
			{
				accounts_types[proxy] = accounts_types[proxy] + campaign.countActiveAccountsByType(proxy);
			});
		};

		var campaigns = inConcert.app.getCampaigns();
		$.each(campaigns, function(undefined, campaign) {
			countActiveCampaignAccountType(campaign);
		});

		// TODO: Revisar si la funcion 'get_tooltip_message' ya no NO cumple funcion. Preguntar a quien la haya dejado de utilizar.
		//
		// const get_tooltip_message = function( channel, only_inbound )
		// {
		// 	var type = only_inbound ? "ONLY_INBOUND" : "INBOUND_AND_OUTBOUND";

		// 	var messages = {
		// 		ONLY_INBOUND: {
		// 			CALL       : "<%=_T('Call Conversation')%>" ,
		// 			CHAT       : "<%=_T('Web Chat Conversation')%>" ,
		// 			FACEBOOK   : "<%=_T('Facebook Conversation')%>" ,
		// 			INSTAGRAM  : "<%=_T('Instagram Conversation')%>" ,
		// 			YOUTUBE	   : "<%=_T('Youtube Conversation')%>" ,
		// 			APPSTORE   : "<%=_T('Appstore Review')%>" ,
		// 			GOOGLEPLAY : "<%=_T('Googleplay Review')%>" ,
		// 			RCS		   : "<%=_T('Rcs Conversation')%>" ,
		// 			IM         : "<%=_T('IM Conversation')%>" ,
		// 			MAIL       : "<%=_T('E-mail Conversation')%>" ,
		// 			SMS        : "<%=_T('SMS Conversation')%>" ,
		// 			WHATSAPP   : "<%=_T('Whatsapp Conversation')%>" ,
		// 			TWITTER    : "<%=_T('Twitter Conversation')%>" ,
		// 			WEBCONTACT : "<%=_T('Web Contact Conversation')%>",
		// 			VOICEMAIL  : "<%=_T('Voice Mail Conversation')%>"
		// 		},
		// 		INBOUND_AND_OUTBOUND: {
		// 			CALL       : "<%=_T('New Call Conversation')%>" ,
		// 			CHAT       : "<%=_T('Web Chat Conversation')%>" ,
		// 			FACEBOOK   : "<%=_T('Facebook Conversation')%>" ,
		// 			INSTAGRAM  : "<%=_T('Instagram Conversation')%>" ,
		// 			YOUTUBE	   : "<%=_T('Youtube Conversation')%>" ,
		// 			APPSTORE   : "<%=_T('Appstore Review')%>" ,
		// 			GOOGLEPLAY : "<%=_T('Googleplay Review')%>" ,
		// 			RCS		   : "<%=_T('Rcs Conversation')%>" ,
		// 			IM         : "<%=_T('New IM Conversation')%>" ,
		// 			MAIL       : "<%=_T('New E-mail Conversation')%>" ,
		// 			SMS        : "<%=_T('New SMS Conversation')%>" ,
		// 			WHATSAPP   : "<%=_T('New Whatsapp Conversation')%>" ,
		// 			TWITTER    : "<%=_T('Twitter Conversation')%>" ,
		// 			WEBCONTACT : "<%=_T('Web Contact Conversation')%>",
		// 			VOICEMAIL  : "<%=_T('Voice Mail Conversation')%>"
		// 		}
		// 	};

		// 	return messages[ type ][ channel ];
		// };

		/*
			Id de Permisos para crear interacciones salientes desde la quickbar.
		*/
		const creation_types = {
			"CALL"       : "int_quick_create_call",
			"CHAT"       : false,
			"FACEBOOK"   : true,
			"IM"         :"int_quick_create_im",
			"INSTAGRAM"  : true,
			"MAIL"       : "int_quick_create_mail",
			"SMS"        : "int_quick_create_sms",
			"RCS"        : "int_quick_create_rcs",
			"TWITTER"    : true,
			"WHATSAPP"   : "int_quick_create_whatsapp",
			"YOUTUBE"    : true,
			"APPSTORE"   : true,
			"GOOGLEPLAY" : true
		};

		// Este es el permiso del API, para sabe si tiene o no permisos para crear interacciones salientes.
		const has_permission_for_create_conversations = inConcert.permissions.to_action_on_any_campaign("interaction", "int_create_outgoing");
		/*
			Esta funcion se encarga de validar si el boton para el canal tiene permisos para activarse o si queda readonly (solo para entrantes).
		*/
		var must_create_outgoing = function(channel)
		{
			if (!has_permission_for_create_conversations) {
				// Si no tiene permisos en el API, si o si el boton tiene que quedar en modo readonly (solo para entrantes).
				return false;
			}

			let enabled_creation_button = false;
			let permission_id = creation_types[channel];
			// Si el channel tiene asociado algun permiso de creacion
			if (typeof(permission_id) === "string") {
				let channel_campaigns = [];
				for (let i = 0; i < campaigns.length; i++) {
					let campaign = campaigns[i];
					// Agrego solo las campanas que tenga configurado alguna cuenta del channel actual
					let belongs_to_channel = campaign.has_accounts_of_proxy(channel, true, true);
					if (belongs_to_channel) {
						channel_campaigns.push(campaign);
					}
				}
				// Ahora valido si que el boton del canal en la quickbar, tiene permisos para estar en modo NO readonly, es decir puede crear salientes.
				const has_validate_all_overrides = true;
				enabled_creation_button = inConcert.permissions.to_action_on_any_campaign("interaction", permission_id, channel_campaigns, has_validate_all_overrides);
			}

			return enabled_creation_button;
		};

		const allowed_app_interaction = inConcert.permissions.to_application("interaction");
	
		// update one accounts status in quickbar
		var update = function( current_channel, new_status )
		{
			var tooltipConfig = {
				alignment: "left-bottom",
				message:    undefined
			};
			var $state = $("#mainButton" + current_channel);
			$state.removeClass("butInteractionDisabled butInteractionOnlyInbound butInteractionWaiting butInteractionConflict");

			if ( new_status )
			{
				switch ( new_status )
				{
					case "DISABLED":
						$state.addClass("butInteractionDisabled");
						tooltipConfig = {detach: true};
						break;

					case "HIDDEN":
						$state.hide();
						return;

					case "CONNECTING":
						$state.addClass("butInteractionWaiting");
						tooltipConfig.message = "<%=_T('Initializing channel')%>";
						break;

					default: //READY
						var only_inbound = false;

						if (!allowed_app_interaction || !must_create_outgoing(current_channel)) {
						// si no tiene permisos sobre la aplicacion "interaction", no puede realizar salientes
							$state.addClass("butInteractionOnlyInbound");
							only_inbound = true;
						}

						// tooltipConfig.message = get_tooltip_message( current_channel, only_inbound );
						break;
				}
			}
			else {
				// CONFLICT
				$state.addClass("butInteractionConflict");
				tooltipConfig.message = message || "<%=_T('Channel in conflict')%>";
			}

			$state.show().inconcert_tooltip( tooltipConfig );
		};

		// update all accounts status in quickbar
		var updateAllChannelsStatus = function()
		{
			$.each( accounts_types, function( proxy, enabled_count )
			{
				var new_status;
				if ( status === "CONNECTING" ) {
					new_status = status;
				}
				else {
					if (!inConcert.app.hasLicensesForChannel(proxy) ) {
						new_status = "HIDDEN";
					}
					else if (enabled_count === 0 ) {
						new_status = "DISABLED";
					} 
					else {
						new_status = !status ? "READY" : status; // status = parameter
					}
				}
				update(proxy, new_status);
			});
		};

		updateAllChannelsStatus();
	};

	MainWindow.prototype.open_chat_window = function (interaction, old_interaction)
	{
		if ( !interaction || (interaction.type !== "CHAT" && interaction.type !== "INTERNAL_CHAT") ) {
			return false;
		}
		if (( !interaction.isTaken() || !interaction.belongsToMe() ) && !interaction.isBotInternalConsultation()) {
			return false;
		}
		if (interaction.isBotInternalConsultation()) {
			return false;
		}
		if (old_interaction) {
			if (old_interaction.state !== "TAKEN" && old_interaction.state === interaction.state) {
				// la ventana se abre unicamente si el state de la interaccion ha cambiado a "take".
				return false;
			}
		}

		var assigned_panel = inConcert.app.getMainWindow().getPanel("AssignedInteractions");

		var create_minimized = false; // por defecto no abre minimizada la ventana de chat
		// Si llega a existir alguna de chat abierta en TAKEN, y que alguna de ellas se encuentre abierta (restore o maximized)
		//  la ventana de chat que se esta por crear, se debe de crear minimizada.
		//
		$.each(assigned_panel.m_windows || {},
			function( window_int_id, window_session )
			{
				if ( window_int_id === interaction.id || !window_session ) {
					return true; //continue
				}
				if ( window_session.client ) {
					// (1) si el client window ya se ha creado ( es decir, llego el evento onready de -> iframe_node[0].contentWindow.onReady)
					// valida contra el cliente para saber si es que tiene que abrir minizada la nueva ventana
					if ( window_session.client.m_interaction && window_session.client.m_interaction.isTaken() ) {
						if ( !window_session.client.isMinimized() ) {
							create_minimized = true; // la abro minimizada
							return false; //break
						}
					}
				}
				else {
					// (2) si el cliente aun no se ha terminado de crear (es decir no se ha cumplido "1"), porque el iframe aun no se ha terminado de crear.
					// tiene que validar si la ventana se tenia que validar si la ventana ya existente, se tenia que crear en modo normal (no minimizada)
					//
					// Esto se da si simultaneamente llegan mas de un chat a la vez al cliente.
					// Sino en un transcurso normal, teniendo previamente la venta de chat creada, si llega una nueva pasaria x (1)
					if ( window_session.create_minimized === false ) {
						var window_interaction = inConcert.app.getInteractionManager().findInteraction( window_int_id );
						if ( window_interaction && window_interaction.isTaken() ) {
							create_minimized = true; // la abro minimizada
							return false; //break
						}
					}
				}
			}
		);

		var params = { create_minimized: create_minimized };
		assigned_panel.openInteractionClient(interaction, null, params);

		return true;
	};

	MainWindow.prototype.onInteractionUpdated = function (interaction, old_interaction)
	{
		this.open_chat_window(interaction, old_interaction);
	};

	MainWindow.prototype.onCampaignUpdated = function(campaign) {
		var campaign_panel = this.m_panels.shared.get_tab("CAMPAIGNS");
		campaign_panel.deleteCampaign(campaign);
		campaign_panel.addCampaign(campaign);
	};

	MainWindow.prototype.onCampaignAdded = function(campaign) {
		this.m_panels.shared.get_tab("CAMPAIGNS").addCampaign(campaign);
		var message = format("<%=_T('You now belong to the campaign {0}')%>", campaign.getDescription());
		m_main.showBannerMessage({ message : message, messageType : "Info", timeout : 5000});
	};

	MainWindow.prototype.onCampaignRemoved = function(campaign) {
		this.m_panels.conversations_panel.get_child_panel("queued").cleanupCampaignInteractions(campaign);
		//this.m_panels.conversations_panel.get_child_panel("waiting").cleanupCampaignInteractions(campaign);
		this.m_panels.conversations_panel.get_child_panel("spam").cleanupCampaignInteractions(campaign);
		this.m_panels.shared.get_tab("CAMPAIGNS").deleteCampaign(campaign);
		var message = format("<%=_T('You were removed from the campaign {0}')%>", campaign.getDescription());
		m_main.showBannerMessage({ message : message, messageType : "Alert", timeout : 5000});
	};

	MainWindow.prototype.updateUnreadCounters = function() {
		const manager = inConcert.app.getInteractionManager();
		if (!manager) {
			return;
		}
		let total = 0;
		let summary = manager.summarizeUnreadInteractions();
		const $quickInteractionContainer = $("#quickInteractionContainer").children();
		if (Object.keys(summary).length === 0) {
			$quickInteractionContainer.each(function(_, element) {
				$(element).find('.quickInboxNewEvents').text('');
			});
		}
		else {
			for (let interactionType in summary) {
				if (summary.hasOwnProperty(interactionType)) {
					total = total + summary[interactionType];
					$("#mainButton" + interactionType).find('.quickInboxNewEvents').text(summary[interactionType]);
				}
			}
		}
		const $quickInboxTotalNewEvent = $("#quickInboxTotalNewEvent");
		if (total === 0) {
			$quickInboxTotalNewEvent.text('').hide();
		}
		else {
			$quickInboxTotalNewEvent.text(total).show();
		}
	};

	MainWindow.prototype.show_hide_quick_take_next_button = function()
	{
		var is_enabled_in_queued = inConcert.permissions.to_action_on_any_campaign( "interaction", "int_queued_take", undefined /*action_type opcional*/, undefined /*campaign_id opcional*/, true /*no_log  opcional */ );

		var is_enabled_in_quick  = inConcert.permissions.to_action_on_any_campaign( "interaction", "int_quick_take" , undefined /*action_type opcional*/, undefined /*campaign_id opcional*/, true /*no_log  opcional */ );

		var can_show = is_enabled_in_queued && is_enabled_in_quick;
		var $cmd = $("#quickTakeQueuedContainer");

		if (can_show) {
			$cmd.show();
		}
		else {
			$cmd.hide();
		}
	};

	MainWindow.prototype.disableQuickTakeNextButton = function()
	{
		$("#quickTakeQueued").prop("disabled", true)
			.addClass("butInteractionDisabled")
			.unbind("click")
			.inconcert_tooltip({detach: true});

		this.show_hide_quick_take_next_button();
	};

	MainWindow.prototype.enableQuickTakeNextButton = function()
	{
		var showError = function(message_error) {
			inConcert.app.m_mainWindow.showBannerMessage({ message: message_error, messageType:"Error", timeout: 5000 });
		};

		$("#quickTakeQueued")
			.prop("disabled", false)
			.removeClass("butInteractionDisabled")
			.unbind("click")
			.bind("click",function()
			{
				var is_enabled_in_quick = inConcert.permissions.to_action_on_any_campaign( "interaction", "int_quick_take" , undefined /*action_type opcional*/, undefined /*campaign_id opcional*/, true /*no_log  opcional */ );
				if (!is_enabled_in_quick) {
					showError("<%=_T('You not has permission for this action.')%>");
					return;
				}

				inConcert.app.getInteractionManager()
					.takeNextBest(null, null, function(success, message_error)
					{
						if (success) {
							return;
						}
						showError(message_error);
					});
			})
			.inconcert_tooltip({alignment: "left-bottom"});

		this.show_hide_quick_take_next_button();
	};

	//Closes the main window
	MainWindow.prototype.close = function() {
		window.close();
	};

	MainWindow.prototype.showContactSelectorDialog = function(addressType, contactType, parentNode, onContactSelectedFn, interaction, parameters) {
		var selector;
		switch(contactType) {
			case "CONTACTS":
				selector = new inConcert.ContactSelector($("#contactSelector"), addressType, parameters);
				break;
			case "TEAMMEMBERS":
				selector = new inConcert.TeamMemberSelector($("#contactSelector"), addressType, interaction, parameters);
				break;
			case "FAVORITES":
				throw "Favorites contact selector type not done yet";
			default:
				throw "Unknown contact selector type";
		}
		selector.show({
			onContactSelectedFn: function(contact, address) {
				if (onContactSelectedFn && typeof(onContactSelectedFn) === "function") {
					onContactSelectedFn(contact, address);
				}
			},
			parentNode: parentNode,
			callbacks :	parameters.callbacks
		});
	};

	//Creates the dialog to ask for user credentials or registering
	MainWindow.prototype.showLoginDialog = function(previousCredentails) {
		// TODO: Sacar este AirLoginDialog
		var main = this;
		var loginDialog = new AirLoginDialog(previousCredentails);
		loginDialog.create({
			onLogin: function(credentials, store, onLoginResponse) {
				main.m_application.login(credentials, store, onLoginResponse);
			},
			onRegister: function() {
				main.m_application.register();
			},
			parentWindow:main
		});
	};

	//retrieve a panel by its id
	MainWindow.prototype.getPanel = function(panelId)
	{
		const self = this;
		if (!this.isInitialized()) {
			return false;
		}
		let mapped_panels = {
			'AssignedInteractions' : function() {
				return self.m_panels.conversations_panel.get_child_panel("assigned");
			},
			'QueuedInteractions' : function() {
				return self.m_panels.conversations_panel.get_child_panel("queued");
			},
			'WaitingInteractions' : function() {
				return self.m_panels.conversations_panel.get_child_panel("waiting");
			},
			'AbandonedInteractions' : function() {
				return self.m_panels.conversations_panel.get_child_panel("abandoned");
			},
			'SpamInteractions' : function() {
				return self.m_panels.conversations_panel.get_child_panel("spam");
			},
			'Campaigns' : function() {
				return self.m_panels.shared.get_tab("CAMPAIGNS");
			},
			'Contacts' : function() {
				return self.m_panels.shared.get_tab("CONTACTS");
			},
			'shared' : function() {
				return self.m_panels.shared;
			},
			'conversations_panel' : function() {
				return self.m_panels.conversations_panel;
			},
			// TODO: #9778 - CANY - Docked.
			'global_search' : function() {
				return self.m_panels.conversations_panel;
			}
		};
		// esto es asi ya que desde otros luggares se invoca con este otro nombre de panel
		// TODO: #9778 - CANY - Docked.
		mapped_panels.abandoned = mapped_panels.AbandonedInteractions;
		mapped_panels.waiting = mapped_panels.WaitingInteractions;
		const panel = mapped_panels[panelId];
		if ( !panel ) {
			throw 'MainWindow.prototype.getPanel - Unrecognized panel id"' + panelId + '"';
		}
		return panel();
	};

	// TODO: #9778 - CANY - Docked.
	MainWindow.prototype.get_interaction_panel_info = function(interaction, force_navigation_panel_id)
	{
		return this.m_panels.conversations_panel.get_interaction_panel_info(interaction, force_navigation_panel_id);
	};

	// TODO: #9778 - CANY - Docked.
	MainWindow.prototype.get_panel_info = function(panel_id)
	{
		return this.m_panels.conversations_panel.get_panel_info(panel_id);
	};

	MainWindow.prototype.totalOpenedPanels = function() {
		var total = 0;
		for (var id in this.m_panels) {
			if (this.m_panels.hasOwnProperty(id)) {
				var panel = this.m_panels[id];
				if (panel.isVisible()) {
					total++;
				}
			}
		}
		return total;
	};

	MainWindow.prototype.centerUpperToolbar = function() {
	};

	MainWindow.prototype.onWindowMoved = function(bounds) {
		if (bounds && bounds.x > 0 && bounds.y > 0) {
			inConcert.Storage.put("applicationPosition", bounds);
		}
	};

	MainWindow.prototype.onMaximized = function() {
		this.centerUpperToolbar();
	};

	MainWindow.prototype.onMaximizing = function()
	{
		if (this.m_panels.conversations_panel.get_child_panel("assigned").isVisible()) {
			$("body").removeClass("inboxFloatingRight");
			$("body").removeClass("inboxFloatingRightShort");
			return true;
		}
		else {
			return false;
		}
	};

	MainWindow.prototype.onWindowResize = function() {
		if (this.isMinimized()) {
			return;
		}

		if (this.isMaximized()) {
			return;
		}
		// trigger onResize method on each panel.
		for (var id in this.m_panels) {
			if (this.m_panels.hasOwnProperty(id)) {
				this.m_panels[id].onResize();
			}
		}
	};

	MainWindow.prototype.belongsEventTargetToContainer = function(containerId, target)
	{
		const $container = $(`#${containerId || ""}`);
		if (!$container || !target) {
			return false;
		}

		return $container.is(target) || $container.has(target).length;
	};

	MainWindow.prototype.get_navigation = function()
	{
		return this.m_panels.conversations_panel.get_navigation();
	};

	MainWindow.prototype.global_search_apply_filter = function(is_new_search)
	{
		const navigation = this.get_navigation();
		navigation.global_search_apply_filter(is_new_search);
	};

	MainWindow.prototype.get_selected_folder_id = function()
	{
		const navigation = this.get_navigation();
		const folder_id = navigation.get_selected_folder_id();
		return folder_id;
	};

	MainWindow.prototype.__mustShowAllPanels = function()
	{
		if (this.m_panels.shared.has_permission_to_show_panel()) {
			return true;
		}
		return false;
	};

	MainWindow.prototype.select_folder = function(folder_id, is_live = false)
	{
		const navigation = this.get_navigation();
		navigation.select_folder(folder_id, is_live = false);
	};

	MainWindow.prototype.select_live_folder = function()
	{
		this.select_folder('live', true);
	};

	MainWindow.prototype.repositionPanels = function()
	{
		const $body = $("body");
		$body.removeClass("inboxConditionOnePanel inboxConditionTwoPanels");
		this.m_panels.conversations_panel.show();

		if (this.__mustShowAllPanels()) {
			this.m_panels.shared.show();
			$body.addClass("inboxConditionTwoPanels");
		}
		else {
			$body.addClass("inboxConditionOnePanel");
		}
	};

	MainWindow.prototype.hideBannerMessage = function() {
		$("#upperMessageBanner").hide();
	};

	MainWindow.prototype.showGenericWaiting = function(message) {
		if (this.m_timeout_waiting) {
			window.clearTimeout(this.m_timeout_waiting);
			this.m_timeout_waiting = null;
		}
		message = message || "";
		var $waiting = $("#genericWaiting");
		$waiting.find(".loadingWaitingMessage").text(message);
		$waiting.show();
	};

	MainWindow.prototype.hideGenericWaiting = function( timeout_ms ) {
		var self = this;
		if (this.m_timeout_waiting) {
			return;
		}
		var hide = function() {
			$("#genericWaiting").hide();
			self.m_timeout_waiting = null;
		};
		if (!timeout_ms) {
			hide();
			return;
		}
		this.m_timeout_waiting = window.setTimeout(function() {
			hide();
		}, timeout_ms);
	};

	MainWindow.prototype.showBannerMessage = function(options, onBannerClosed, hasLog)
	{
		var self = this;
		options = $.extend({messageType: "Error", message :""}, options);
		var banner = $("#upperMessageBanner");
		var bannerHidden = false;

		var hideBanner = function()
		{
			if(bannerHidden === false) {
				banner.hide();
				bannerHidden = true;
				// hide overlay
				self.hideGenericWaiting();
				if (typeof(onBannerClosed) === "function") {
					onBannerClosed();
				}
			}
		};

		banner.removeClass("messageAlert messageInfo messageProcess messageOk messageError").addClass("message" + options.messageType);
		banner.find('.genericMessagesTextAux').text(options.message);

		var close_button = banner.find('.genericMessagesClose');
		if (options.hideClose){
			close_button.hide();
		}
		else{
			close_button.unbind('click').click(function() {
				hideBanner();
			});
		}
		if (options.showOverlay){
			this.showGenericWaiting();
		}

		if (options.timeout) {
			window.setTimeout(function() {
				hideBanner();
			}, options.timeout);
		}
		banner.show();

		if (hasLog) {
			inConcert.logger.warn(options);
		}
	};

	MainWindow.prototype.hidePopupMessage = function() {
		$("#messagePopupBox").hide();
	};

	MainWindow.prototype.showPopupMessage = function(options) {
		options = $.extend({messageType: "Error", messageTitle: "", popUpTitle:"", message :""}, options);
		var popup = $("#messagePopupBox");

		// update popup data
		popup.find('.messagePopUpMarkUp').removeClass("messageAlert messageInfo messageError").addClass("message" + options.messageType);
		popup.find('.popUpTitle').text(options.popUpTitle);
		popup.find('.messageTitle').text(options.messageTitle);
		popup.find('.popUpContent').text(options.message);
		var popupActions = popup.find('.messagePopUpBottom');
		popupActions.children().remove();

		// create actions buttons if any.
		if (options.actions) {
			popup.find('.popUpButClose').unbind("click").hide();
			for (var action in options.actions) {
				if (options.actions.hasOwnProperty(action)) {
					var $action = $('<input class="butBig' + options.actions[action].type +'" value="' + options.actions[action].caption + '"/>');
					$action.click(options.actions[action].exec);
					popupActions.append($action);
				}
			}
		}
		else {
			// hide close button if it was specified inside the options
			if (options.hideCloseButton){
				popup.find('.popUpButClose').hide();
			}
			else {
				// otherwise, just update close behaviour
				popup.find('.popUpButClose').unbind("click").bind("click", function() {
					popup.hide();
					if (options.callback) {
						options.callback();
					}
				});
			}
		}

		// all done, show popup.
		popup.show();
	};

	//refresh the current view for timers and similar
	//in case interactions updates are received update them on the corresponding views
	MainWindow.prototype.refresh = function()
	{
		var current_app = inConcert.app.getCurrentApp();
		if (current_app === "interaction") {
			var allowed_app_interaction = inConcert.permissions.to_application("interaction");

			this.m_panels.conversations_panel.refresh_panel(allowed_app_interaction);
			this.m_panels.shared.get_tab("CONTACTS").refreshPanel();
		}
	};

	/*
		Esta funcion es la encargada de realizar las actualizaciones correspondientes a permisos en la GUI de la app "Interaction".
		La misma es invocada cuando se inicia la interfaz grafica, y si los permisos cambian realtime.
		Dentro contiene una funcion recursiva en un timeout, que se invoca cada 10 segundos para actualizar la misma (sin que cambien los permisos)
		Esto ultimo es por si a alguien se le ocurre cambiar con el instrospector el DOM.
	*/
	MainWindow.prototype.refreshPermissionGUI = function()
	{
		var self = this;

		// funcion que elimina el timeout
		var clear = function() {
			if (self.m_timeout_refresh_permission_gui) {
				window.clearTimeout(self.m_timeout_refresh_permission_gui);
				self.m_timeout_refresh_permission_gui = null;
			}
		};

		// funcion recursiva que se ejecuta dentro el timeout cada 10 segundos.
		var refresh;
		refresh = function() {
			clear();
			var current_app = inConcert.app.getCurrentApp();
			if (current_app === "interaction") {
				var allowed_app_interaction = inConcert.permissions.to_application("interaction");
				self.refreshQuickbar(allowed_app_interaction);
				self.m_panels.conversations_panel.refresh_permission_gui(allowed_app_interaction);
			}
			self.m_timeout_refresh_permission_gui = window.setTimeout(function() {
				refresh();
			}, 10000); // aca invocamos a la funcion en un timeout cada 10 segundos.
		};

		refresh();
	};

	MainWindow.prototype.refreshQuickbar = function(allowed_app_interaction)
	{
		const self = this;

		const $body = $("body");
		const $quick_take_best = $("#quickTakeQueuedContainer");
		const $quick_main_container = $("#quickInteractionActionsContainer");

		var hide_quickbar = function() {
			// Cuando NO tiene permisos sobre la quickbar, tengo que ocultar tanto el container de acciones por channel, como el boton take queued
			$body.removeClass("withoutQuickbar")
				.addClass("withoutQuickbar");
			$("#interactionsMenuTop").hide();
			$quick_take_best.hide();
			$quick_main_container.hide();
		};
		var show_quickbar = function() {
			// Cuando SI tiene permisos sobre la quickbar, visualizo el container de acciones por channel, y mando a refrescar los status,
			// para que deje enabled/disabled los botones de los channels dependiendo de si tiene o no permisos de creacion.
			$body.removeClass("withoutQuickbar");
			$quick_main_container.show();
			$("#interactionsMenuTop").show();
			self.updateChannelStatus();
		};

		if (!allowed_app_interaction) {
			hide_quickbar();
			return;
		}

		// Si no tiene permisos para toda la quickbar, oculto las acciones de creation y take best
		if (!inConcert.permissions.to_action_on_any_campaign("interaction", "int_view_quick")) {
			hide_quickbar();
			return;
		}
		// Si no tiene permisos para take best desde la quickbar, oculto la accion
		if (!inConcert.permissions.to_action_on_any_campaign("interaction", "int_quick_take")) {
			$quick_take_best.hide();
		}
		else {
			$quick_take_best.show();
			$quick_main_container.show();
		}

		show_quickbar();
	};

	//remove an interaction from specific panel
	MainWindow.prototype.removeInteractionFromPanel = function(interaction, panel)
	{
		var current_app = inConcert.app.getCurrentApp();
		if ( current_app === "interaction" ) {
			if (this.m_panels.conversations_panel && this.m_panels.conversations_panel.panels[panel]) {
				this.m_panels.conversations_panel.panels[panel].removeInteraction(interaction);
			}
		}
	};

	MainWindow.prototype.getIntegrationFrame = function() {
		return $("#integrationFrame");
	};

	//find a panel by internal id
	MainWindow.prototype.findPanel = function(nativeId) {
		for (var id in this.m_panels) {
			if (this.m_panels.hasOwnProperty(id)) {
				if (typeof(this.m_panels[id].getId) === "function" && this.m_panels[id].getId() === nativeId) {
					return this.m_panels[id];
				}
			}
		}
		return null;
	};

	MainWindow.prototype.updateUserInformation = function() {
		$('#userLoggedAvatarContainer > .userName').text(inConcert.app.getLoggedUserName());
	};

	//setup personalization of the console
	MainWindow.prototype.loadPersonalInfo = function() {
		var self = this;
		var defaultMessage = "<%=_T('Share a quick message')%>";

		var avatarURI = inConcert.app.getBaseUrl() + "/users/get_avatar/" + inConcert.app.getLoggedUserId();
		$('#userLoggedAvatar > img').attr("src", avatarURI);

		var has_permission_to_change_nick = inConcert.permissions.to_action( "settings", "changenick" );

		var $nick = $('#userLoggedNick > input');
		if ( has_permission_to_change_nick ) {
			$nick
				.prop( "disabled", false )
				.show()
				.focus(function() {
					this.previousMessage = $(this).val();
					var that = this;
					window.setTimeout(function(){$(that).select();},10);
				})
				.blur(function() {
					var $this = $(this);
					var currentMessage = $this.val();
					if ( currentMessage !== defaultMessage ) {
						inConcert.app.setLoggedUserMessage( currentMessage,
							function( success, err )
							{
								if ( !success ) {
									var message;
									if ( err === 403 ) { // forbidden
										message = "<%=_T('Unable to change nick. You dont have permission.')%>";
									}
									else {
										message = "<%=_T('Unable to change nick.')%>";
									}
									inConcert.app.m_mainWindow.showBannerMessage({
										message : message,
										timeout : 5000
									});
									var lastMessage = inConcert.app.getLoggedUserMessage();
									$this.val( lastMessage );
									return;
								}
							}
						);
					}
				})
				.keyup(function(event) {
					if (event.keyCode === 13) {
						this.blur();
					}
					else if (event.keyCode === 27) {
						$(this).val(this.previousMessage);
						this.blur();
					}
				});

			var lastMessage = inConcert.app.getLoggedUserMessage();
			var message = (lastMessage && lastMessage.length > 0) ? lastMessage : defaultMessage;
			$nick.val( message );
		}
		else {
			$nick.prop( "disabled", true ).hide();
		}

		self.updateUserInformation();

		// Set user status
		var status_id = inConcert.app.m_currentStatus;
		self.setUserStatus(status_id);
	};

	MainWindow.prototype.getStateQualifier = function(stateId)
	{
		var qualifier = "OffLine";
		if (stateId && stateId.length > 0) {
			var upperState = stateId.toUpperCase();
			if (upperState !== "OFFLINE") {
				if (upperState === "NOACD") {
					qualifier = "NoACD";
				}
				else {
					var away = inConcert.app.isAwayState(stateId);
					var busy = inConcert.app.isBusyState(stateId);
					var custom =  inConcert.app.isCustomState(stateId);
					if (custom) {
						qualifier = "Custom";
					}
					else if (away || busy) {
						qualifier = busy ? 'Busy' : 'Away';
					}
					else {
						qualifier = 'Available';
					}
				}
			}
		}
		return qualifier;
	};

	MainWindow.prototype.setUserStatus = function(status_id, message)
	{
		const self = this;

		const $state_container = $("#userStatusBtnContainer > .userStatusButton");
		const $state = $state_container.find(".userStatus");

		var allowed_app_interaction = inConcert.permissions.to_application("interaction");
		if (!allowed_app_interaction) {
			$state.parent().remove();
			return;
		}

		var status_name = this.formattedUserStatus(status_id);
		$state.html(status_name);
		// Set status class
		$("#userLoggedContainer")
			.removeClass()
			.addClass("userLogged" + this.getStateQualifier(status_id));
		//state changer
		$state.parent()
			.unbind('click')
			.bind('click', function() {
				self.onStateChangeSelected();
			});
		$("#closeStateChangerPopup")
			.unbind('click')
			.bind('click', function() {
				self.userStatePopup.finalize();
			});

		// (***) Aca iniciamos el Tooltip con el tiempo transcurrido en el estado actual
		$state.attr("state_start_time", new Date().getTime());
		$state_container.inconcert_tooltip({
			className: 'genericTooltip',
			alignment: 'left-bottom',
			bind_selector: '.userStatus',
			["remove-delay"]: 5000,
			message: function() {
				const timestamp = parseInt($state.attr("state_start_time"), 10);
				const state_start_time_obj = new Date(timestamp);
				const eplapsed_time_sec = inConcert.Utils.getTimeDifference(state_start_time_obj, true /*rawSeconds*/);
				let time_type;
				if (eplapsed_time_sec < 60) {
					time_type = "<%=_T('seconds')%>";
				}
				else if (eplapsed_time_sec < 3600) {
					time_type = "<%=_T('minutes')%>";
				}
				else {
					time_type = "<%=_T('hours')%>";
				}
				const formatted_elapsed_time = inConcert.Utils.formatSecondsAsTime(eplapsed_time_sec);
				const state_start_time = inConcert.Utils.getStringFromDate(state_start_time_obj, false, "date-short-year-short-hour");

				let message = format("<%=_T('In the current state for {0} {1} from {2}.')%>", formatted_elapsed_time, time_type, state_start_time);
				return message;
			}
		});

		this.userStatePopup = new inConcert.PopupWindow({ownerId: "stateChangerPopup"});
		if (message) {
			m_main.showBannerMessage({
				messageType : "Info",
				message     : message,
				timeout     : 5000
			});
		}
	};

	MainWindow.prototype.getTotalIframesWindowsDialogs = function()
	{
		if (inConcert.DialogManager && inConcert.DialogManager.m_sortedIframes ) {
			return inConcert.DialogManager.m_sortedIframes.length;
		}
	};

	MainWindow.prototype.onUserStateChanged = function(stateId) {
		const $state = $('#userStatusBtnContainer > .userStatusButton > .userStatus');
		$state.html(this.formattedUserStatus(stateId));
		$("#userLoggedContainer").removeClass().addClass('userLogged' + this.getStateQualifier(stateId));
		// actualizo el start time del estado que sera utilizado en el tooltip del tiempo transcurrido en el estado actual (***)
		const now = new Date();
		const state_start_time = now.getTime();
		$state.attr("state_start_time", state_start_time);
	};

	MainWindow.prototype.formattedUserStatus = function(status)
	{
		var state;
		if (typeof status === "string") {
			var availablesStates = inConcert.app.m_availableStates;
			state = availablesStates[status];
			if (!state) {
				return "";
			}
		}
		else if (typeof status === "object") {
			state = status;
		}
		else {
			return "";
		}
		var translatedState = _T(state.Description);
		if (typeof state.ShortMessage !== "string") {
			return translatedState;
		}
		return translatedState + format(" <span class='stateSubtitle'>({0})</span>", _T(state.ShortMessage));
	};

	/*
		Triggered when the user clicks on status changer.
	*/
	MainWindow.prototype.onStateChangeSelected = function()
	{
		var self = this;

		var $state_changer_popup = $("#stateChangerPopup");
		// hide popup if it's visible.
		if ($state_changer_popup.is(":visible")) {
			this.userStatePopup.finalize();
			return;
		}

		$state_changer_popup.find('.quickViewPanel').remove();

		const $intUserStatesScroll = $('#intUserStatesScroll');
		// Get and build markup for each state.
		inConcert.app.getAvailableStates().forEach(function(state)
		{
			if (typeof(state.Enabled) !== "undefined") {
				const isDisabled = state.Enabled.toString() === "0";
				if (isDisabled) {
					return;
				}
			}
			var add = true;
			if (state.Id.toUpperCase() === "ACTIVE" || state.Id.toUpperCase() === "NOACD") {
				var allowed_app_interaction = inConcert.permissions.to_application( "interaction" );
				if (!allowed_app_interaction) {
					add = false;
				}
			}
			if (add) {
				var stateClass = "quickStatus" + self.getStateQualifier(state.Id);
				$intUserStatesScroll.append(
					'<div state_id="' + state.Id + '" class="quickViewPanel ' + stateClass + '" title="' + self.formattedUserStatus(state) + '">' +
						self.formattedUserStatus(state) +
					'</div>');
			}
		});

		var change_state = function( new_status, callback )
		{
			callback = callback || $.noop;
			inConcert.app.changeUserState(new_status,
				function(response)
				{
					var newState;
					if (response.success) {
						newState = new_status;
						callback();
					}
					else {
						newState = inConcert.app.m_currentStatus;
						var stateName;
						var stateObject = inConcert.app.getStateObject(new_status);
						if (stateObject) {
							stateName = stateObject.Description;
						}
						else {
							stateName = new_status;
						}
						var msg_err;
						switch (response.errType)
						{
							case "USER_HAS_ASSIGNED_LIVE_INTERACTIONS":
								msg_err = format("<%=_T('Unable change to {0} state. You have live conversations assigned.')%>", stateName);
							break;

							case "HAS_NOT_PERMISSION":
								msg_err = "<%=_T('Unable change state. You not has permission for this action.')%>";
							break;

							default: // Cualquier otro error lo visualizamos como mensaje generico.
								msg_err = format("<%=_T('Can not change to state {0}')%>", stateName);
							break;
						}
						m_main.showBannerMessage({message: msg_err, messageType: "Error", timeout: 3000},
							null,
							true
						);
					}
					self.onUserStateChanged(newState);
					self.userStatePopup.finalize();
				}
			);
		};

		var assigned_panel = inConcert.app.getMainWindow().getPanel("AssignedInteractions");
		var close_waiting_windows = function(callback)
		{
			var timemout_ms = 0;
			var closed_waiting_windows = 0;
			$.each(assigned_panel.m_windows || {},
				function( interaction_id, window_session )
				{
					if (window_session && window_session.client) {
						if (window_session.client.m_interaction && window_session.client.m_interaction.state === "WAITING") {
							window_session.client.close();
							closed_waiting_windows++;
							if (closed_waiting_windows === 1) { timemout_ms = 500; }
							else if (closed_waiting_windows > 1) { timemout_ms = timemout_ms + 50; }
						}
					}
				}
			);
			window.setTimeout(function() { callback(); }, timemout_ms);
		};

		// State Row inside popup handler.
		$state_changer_popup.find('.quickViewPanel').click(function()
		{
			const new_status = $(this).attr("state_id");
			if (new_status.toUpperCase() === "ACTIVE") {
				// si se cambia a active, previamente tengo que cerrar las ventanas en waiting que se encuentren abiertas. #6190
				close_waiting_windows(function() {

					var callbackActive = function () {
						const folder_id = self.get_selected_folder_id();
						if (folder_id === "waiting" || folder_id === "global_search") {
							const is_new_search = false;
							window.setTimeout(function () {
								if (folder_id === "waiting") {
									let current_panel = self.m_panels.conversations_panel.get_child_panel("waiting");
									current_panel.apply_filter(is_new_search);
								}
								else {
									self.global_search_apply_filter(is_new_search);
								}
							}, 1500);
						}
					};

					if (!inConcert.app.has_allegro_integration() && inConcert.app.getCurrentStatus() !== "ACTIVE" && (inConcert.app.getCampaignsByAddress("CALL") || []).length > 0) {
						
						if (!inConcert.app.isWebMicrophonePermissionGranted()) {
							// No se puede cambiar a activo si no se han confirmado los permisos de microfono.
							self.showBannerMessage({
								messageType: "Info",
								message: "<%=_T('Can not change to state active.')%>" + " " +  "<%=_T('Microphone permissions are required to make and receive phone calls.')%>",
								timeout: 7000
							}, undefined, true);
							return;
						}

						const clientCall = inConcert.app.getInteractionManager().getChannelClient("CALL");
						if (clientCall && !clientCall.isPeerRegistered()) {
							self.showBannerMessage({
								messageType: "Info",
								message: "<%=_T('Can not change to state active.')%>" + " " +  "<%=_T('The phone client is not registered.')%>",
								timeout: 7000
							}, undefined, true);
							return;
						}

					}
					
					
					change_state(new_status, callbackActive);
				});
			}
			else {
				change_state(new_status);
			}
		});

		const fnRePosition = function() {
			const botonRect = $("#userLoggedAvatarContainer")[0].getBoundingClientRect();
			let right = botonRect.width + 120.5;
			let top = $("#userLoggedContainer").height();
			$("#stateChangerPopup").css({ top: top, right: right, width: "fit-content"});
			let total = self.getTotalIframesWindowsDialogs() || 3;
			let zindex = (total * 10) + 1001;
			self.userStatePopup.m_inlineStyleValues["z-index"] = zindex;
		};

		// Show popup
		this.userStatePopup.show({
			ownerId: "stateChangerPopup",
			eventName: "user_state",
			callbacks: {
				onOpen: function() {
					$("#userStatusBtnContainer").addClass("menu-show");
					$state_changer_popup.show();
					fnRePosition();
				},
				onClose: function() {
					$("#userStatusBtnContainer").removeClass("menu-show");
					$state_changer_popup.hide();
					$('#userLoggedName').removeClass('status-show');
					$("#stateChangerPopup").css("z-index", "");
				}
			}
		});
		
	};

	MainWindow.prototype.setupToolbarHandlers = function()
	{
		var self = this;

		//open twitter searches new window
		$("#showTwitterSearchButton").click(function() {
			var windowSize = {width:800 , height:600};
			var windowOptions = {page : "twitterSearches", size : windowSize};
			inConcert.app.getWindowManager().createWindow(windowOptions);
		});

		//creates an outgoing interaction for a specific channel with no related contact
		$('#mainButtonMAIL, #mainButtonIM, #mainButtonSMS, #mainButtonRCS, #mainButtonWHATSAPP, #mainButtonCALL').click(function()
		{

			if (self.dropdownMenuInteractions.isVisible()) {
				self.dropdownMenuInteractions.finalize();
			}
			
			const $statusNode = $("#" + $(this).attr('id'));
			if ($statusNode.hasClass('butInteractionDisabled') || $statusNode.hasClass('butInteractionOnlyInbound')) {
				return;
			}
			let creation_types = {
				"CALL"    : {permission_id: "int_quick_create_call"    , is_live: true },
				"IM"      : {permission_id: "int_quick_create_im"      , is_live: true },
				"MAIL"    : {permission_id: "int_quick_create_mail"    , is_live: false},
				"SMS"     : {permission_id: "int_quick_create_sms"     , is_live: false},
				"RCS"     : {permission_id: "int_quick_create_rcs"     , is_live: false},
				"WHATSAPP": {permission_id: "int_quick_create_whatsapp", is_live: false}
			};
			let proxyType = $(this).attr("id").match("mainButton(.+)")[1];
			let info = creation_types[proxyType];
			if (!info) {
				throw("Urecognized type " + proxyType );
			}
			let has_permission_quick_create = inConcert.permissions.to_action_on_any_campaign("interaction", info.permission_id);
			if (!has_permission_quick_create) {
				self.showBannerMessage({
					message: "<%=_T('You not have permissions for create outgoing conversations from quickbar.')%>",
					timeout: 5000
				});
				return;
			}

			let panel = m_main.getPanel("AssignedInteractions");
			/*
				Es necesario que la creacion de la ventana de la interaccion saliente, se realice en un pequeno timeout.
				Esto es asi, para que las ventanas que se encuentren abiertas, se minimicen (solo estas), ya que sino
				cuando se levantaba la ventana saliente (la del cick en la quickbar), le llega el evento de minimizar,
				y esto hacia que el html no estuviera renderizado correctamente y ciertos plugines o markup no se instanciaba correctamente (ej GoldenLayout)
			*/
			window.setTimeout(function() {
				let response = panel.createOutgoingInteraction(proxyType, null, "", inConcert.app.getCampaigns());
				if (!response.success) {
					if (response.conflicts) {
						self.showBannerMessage({
							message: response.conflicts.message,
							timeout: 5000
						});
						return;
					}
					inConcert.app.m_mainWindow.showPopupMessage({
						message: "<%=_T('If you want to communicate using this channel, please set up an account in one of your campaigns.')%>",
						messageTitle: format("<%=_T('No groups with channel {0}')%>", proxyType),
						popUpTitle: "<%=_T('Oops')%>",
						messageType: "Error"
					});
					return;
				}
				// TODO: #9778 - CANY - Docked.
				const mw = inConcert.app.getMainWindow();
				mw.jumpToApp("interaction");
				let methodName = info.is_live ? "move_to_live_panel" : "move_to_active_panel";
				inConcert.app.getMainWindow().getPanel('conversations_panel')[methodName]();
			}, 250);
		});

		this.updateCampaignAccounts();
		this.centerUpperToolbar();
	};

	MainWindow.prototype.setupConfigurationHandlers = function()
	{
		var self = this;

		// avatar -------
		// var $avatar_img = $("#userLoggedAvatar > img");
		var $avatar_edit = $("#imgIcoAvatarEdit");
		var has_permission = inConcert.permissions.to_action("settings", "changeavatar");

		if (has_permission) {
			$avatar_edit
				.bind("click", function () {
					self.changeAvatar();
				})
				.show();
			// $avatar_img
			// 	.bind("click", function () {
			// 		self.changeAvatar();
			// 	});
		}
		else {
			$avatar_edit.unbind("click").hide();
			// $avatar_img.unbind("click");
			$avatar_edit.removeClass("icoAvatarEdit");
		}
		//------------

		var $settings_item = $("#managePersonalSettings");
		var validateItemsPermissions = function()
		{
			// var allowed_app_analytics = inConcert.permissions.to_application("analytics");
			// $("#openAnalytics")[allowed_app_analytics ? "show" : "hide"]();
			var has_permission_to_change_settings = inConcert.permissions.to_action("settings", "change_personal_settings");
			$settings_item[has_permission_to_change_settings ? "show" : "hide"]();
		};

		const fnRePosition = function() {
			let right = 22.5;
			let top = $("#userLoggedContainer").height();
			$("#managementMenu").css({ top: top, right: right, width: "fit-content"});
			let total = self.getTotalIframesWindowsDialogs() || 3;
			let zindex = (total * 10) + 1001;
			self.managementPopup.m_inlineStyleValues["z-index"] = zindex;

		};
		$("#userLoggedAvatarContainer").click(function()
		{
			if ($("#managementMenu").is(":visible")) {
				self.managementPopup.finalize();
			}
			else {
				self.managementPopup.show({
					ownerId: "managementMenu",
					eventName: "management",
					callbacks: {
						onOpen: function()
						{
							validateItemsPermissions();
							$("#managementMenu").show();
							$("#userLoggedAvatarContainer").addClass("menu-show");
							fnRePosition();

						},
						onClose: function() {
							$("#managementMenu").hide();
							$("#userLoggedAvatarContainer").removeClass("menu-show");
							$("#managementMenu").css("z-index", "");
						}
					}
				});
			}
		});

		$("#managementMenu").find(".quickMenuBlindClose").click(function() {
			self.managementPopup.finalize();
		});

		// $("#openAnalytics").click(function()
		// {
			// var has_permission = inConcert.permissions.to_application("analytics");
			// if (!has_permission) {
				// inConcert.logger.warn({
					// action_name:      "MainWindow.prototype.setupConfigurationHandlers",
					// action_type:      "analytics",
					// message:          "You not has permission to application analytics"
				// });
				// return;
			// }
			// if (self.analyticsLoader && self.analyticsLoader.isValid()) {
				// self.analyticsLoader.activate();
				// self.managementPopup.finalize();
				// return;
			// }
			// var windowSize = {width:1024 , height:768};
			// var windowOptions = {page : "analyticsWrapper", size : windowSize, title : "<%=_T('Analytics Dashboard')%>" };
			// self.analyticsLoader  = inConcert.app.getWindowManager().createWindow(windowOptions);

			// self.managementPopup.finalize();
		// });

		$settings_item.click(function(e) {
			e.stopPropagation();
			let has_permission = inConcert.permissions.to_action( "settings", "change_personal_settings" );
			if ( !has_permission ) {
				self.showBannerMessage({
					message: "<%=_T('Unable to edit personal settings. You dont have permission.')%>",
					timeout: 5000
				});
				self.managementPopup.finalize();
				return;
			}
			if (self.personalSettingsLoader && self.personalSettingsLoader.isValid()) {
				self.personalSettingsLoader.activate();
				self.managementPopup.finalize();
				return;
			}

			let windowSize = {width: 815, height: 630};
			let resizable = true;
			let maximizable = true;
			if (inConcert.app.is_system_vcc()) {
				// Si se abre desde la app provisioning, el tamano de la ventana es mas acotada y no necesita resize.
				windowSize.width = 620;
				windowSize.height = 490;
				resizable = false;
				maximizable = false;
			}
			let windowOptions = { page : "personalSettings", size : windowSize, windowID: "personalSettings", title: "<%=_T('Personal settings')%>", resizable: resizable, maximizable: maximizable };
			self.personalSettingsLoader = inConcert.app.getWindowManager().createWindow(windowOptions);
			self.managementPopup.finalize();
		});

		$("#closePasswordExpirationDaysButton").click(function() {
			$("#passwordExpirationDaysContainer").hide();
		});

		this.managementPopup = new inConcert.PopupWindow({ownerId: "managementMenu"});
	};

	MainWindow.prototype.refreshAvatar = function() {
		var avatarURI = inConcert.app.getBaseUrl() + "/users/get_avatar/" + inConcert.app.getLoggedUserId()+"?uid=" + new Date().getTime();
		$('#userLoggedAvatar > img').attr("src", avatarURI);
	};

	MainWindow.prototype.changeAvatar = function() {
		var hasPermission = inConcert.permissions.to_action( "settings", "changeavatar" );
		if ( !hasPermission ) {
			this.showBannerMessage({
				message :  "<%=_T('Unable to change avatar. You dont have permission.')%>",
				timeout :  5000
			});
			return;
		}
		if ( this.changeAvatarLoader && this.changeAvatarLoader.isValid()) {
			this.changeAvatarLoader.activate();
			return;
		}
		var windowOptions = {page : "changeAvatar", size : {width: 496 , height: 500}, windowID: "changeAvatar", title: "<%=_T('Change Avatar')%>", resizable: false, maximizable: false };
		this.changeAvatarLoader = inConcert.app.getWindowManager().createWindow(windowOptions);
	};

	// interaction, contactName, contactCampaigns and contactAddress are optional parameters.
	MainWindow.prototype.createContact = function(onContactCreatedFn, interaction, contactName, contactCampaigns, contactAddress) {
		this.m_panels.shared.get_tab("CONTACTS").createNewContact(onContactCreatedFn, interaction, contactName, contactCampaigns, contactAddress);
	};

	MainWindow.prototype.getCurrentSkin = function() {
		return inConcert.app.getWindowManager().getCurrentSkin();
	};

	//
	// FIX-ME: This is a hack, it's done here because I need to access from application.js
	// when a message arrive, and I need to duplicate logic because I can't access
	// calculateCampaignClass from outside the panel.
	//
	// Given a campaign id and a bool status, update the campaign node.
	MainWindow.prototype.setCampaignLiveStatus = function(campaign_id, enabled) {
		//go and search for the campaign
		$('#int_panel_campaigns #int_panel_campaigns_scroll').children().each(function(undefined,e){
			var campaign_node = $(e);

			if ( campaign_node.attr('campaignid') === campaign_id ){
				// update campaign data
				var stateMode = enabled ? "Active" : "";

				var expandMode = "Open";
				var campaignClass = 'boxGroup' + stateMode + expandMode;
				var currentClass = campaign_node.attr("class");

				// update class
				campaign_node.removeClass(currentClass).addClass(campaignClass);
				campaign_node.find('.formCheckbox').prop('checked', enabled);

				// set tooltip
				var tooltip = enabled ? "<%=_T('Pause (avoid live conversations)')%>" : "<%=_T('Activate (live conversations)')%>";
				campaign_node.find('.campaignStateTooltip').attr("tooltip", tooltip);
			}
		});
	};

	MainWindow.prototype.setSkin = function(skin) {
		inConcert.app.getWindowManager().setSkin(skin);
		inConcert.app.getWindowManager().applySkin(window);
	};

	MainWindow.prototype.resetPanels = function() {
		this.m_panels.conversations_panel.reset();
	};

	MainWindow.prototype.initializeDashboardsApps = function()
	{
		const self = this;
		this.apps_popup = new inConcert.DashboardAppsPopup({
			owner : self
		});
		
		self.apps_popup.data.parentNode
		.off("click")
		.on("click", function() {
			const $container = $(`#${self.apps_popup.ownerId}`);
			const positions_options = {
				top_position_config : {
					top_magic_number : $(this).height() + 6
				}
			};
			if ($container && $container.is(":visible")) {
				//Si esta visible, reposicionar
				self.apps_popup.setPosition(positions_options);
				self.apps_popup.applyZindex();
				return;
			}
			self.apps_popup.show();
			self.apps_popup.setPosition(positions_options);
			self.apps_popup.applyZindex();
		});

		//------------------------------------------------------------
		//Show password will expire soon notification, when apply
		let passwordExpirationDays = inConcert.app.PasswordExpirationDays();
		if(passwordExpirationDays)
		{
			var has_permission_to_change_settings = inConcert.permissions.to_action("settings", "change_personal_settings");
			var passwordExpirationDaysText = inConcert.Utils.format("<%=_T('Your password will expire in {0} days.')%>", passwordExpirationDays);
			passwordExpirationDaysText = "<strong>" + passwordExpirationDaysText + "</strong>" + (has_permission_to_change_settings ? " <%=_T('Change it from your personal settings')%>" : " <%=_T('Contact your administrator')%>");
			$("#passwordExpirationDaysText").html(passwordExpirationDaysText).parent().show();
		}

		this.m_initialized = true;
	};

	MainWindow.prototype.jumpToApp = function(app_id = "")
	{
		app_id = app_id.toLowerCase();
		const availables_apps = {
			"configuration": true,
			"interaction"  : true,
			"reports"      : true,
			"supervisor"   : true,
			"report_design": true
		};
		if (!availables_apps[app_id]) {
			throw "Error - Jump to app. Unrecognized app " + app_id;
		}
		const current_app = inConcert.app.getCurrentApp().toLowerCase();
		if (current_app === app_id ) {
			return;
		}

		const $tab_app = $("#dashboard_apps_menu").find("li[dashboard_item='" + app_id + "']");
		$tab_app.trigger("click");
	};

	MainWindow.prototype.startAppSupervisor = function() {
		if (inConcert.SupervisorApp) {
			inConcert.supervisor = new inConcert.SupervisorApp();
		}
	};

	MainWindow.prototype.getAppSupervisor = function() {
		return inConcert.supervisor;
	};

	MainWindow.prototype.startAppConfiguration = function() {
		let apps = {};

		// administration
		if (inConcert.adm_config) {
			apps.administration_config = new inConcert.adm_config();
		}

		// provisioning
		if (inConcert.provisioning_config) {
			apps.provisioning_config = new inConcert.provisioning_config();
		}

		// configuration
		if (inConcert.conf_dashboard) {
			inConcert.configuration = new inConcert.conf_dashboard({
				apps: apps,
				permissions: inConcert.permissions,
				main_html: "/inconcert/apps/configuration/pages/conf_main.<%=_L%>.html#grunt-cache-bust"
			});
		}
	};

	MainWindow.prototype.startAppReportDesign = function() {
		if (inConcert.rpt_config && inConcert.conf_dashboard) {
			let apps = {};
			apps.report_design_config = new inConcert.rpt_config();
			inConcert.report_design = new inConcert.conf_dashboard({
				apps: apps,
				permissions: inConcert.permissions,
				prefix_id: "conf_report_",
				dashboard: $("#report_design_dashboard"),
				main_html: "/inconcert/apps/reports/pages/conf_report_main.<%=_L%>.html#grunt-cache-bust"
			});
		}
	};

	MainWindow.prototype.startAppOutboundEngine = function() {
		
		if (inConcert.outbound_engine_config && inConcert.conf_dashboard) {
			let apps = {};
			apps.outbound_engine_config = new inConcert.outbound_engine_config();
			inConcert.outbound_engine = new inConcert.conf_dashboard({
				apps: apps,
				dashboard: $("#outbound_engine_dashboard"),
				prefix_id: "conf_oe_",
				permissions: inConcert.permissions,
				main_html: "/inconcert/apps/outbound_engine/pages/conf_oe_main.<%=_L%>.html#grunt-cache-bust"
			});
		}
	};

	MainWindow.prototype.startAppSocialNetworks = function() {
		if (!inConcert.SocialNetworksApp) {
			return;
		}
		inConcert.social_networks = new inConcert.SocialNetworksApp();
	};

	MainWindow.prototype.startAppCampaignAnalytics = function() {
		if (!inConcert.CampaignAnalyticsApp) {
			return;
		}
		inConcert.campaign_analytics = new inConcert.CampaignAnalyticsApp();
	};

	MainWindow.prototype.startAppWebDesigner = function() {
		
		if (inConcert.web_designer_config && inConcert.conf_dashboard) {
			let apps = {};
			apps.web_designer_config = new inConcert.web_designer_config();
			inConcert.web_designer = new inConcert.conf_dashboard({
				apps: apps,
				dashboard: $("#web_designer_dashboard"),
				prefix_id: "conf_web_designer_",
				permissions: inConcert.permissions,
				main_html: "/inconcert/apps/web_designer/pages/conf_web_designer_main.<%=_L%>.html#grunt-cache-bust"
			});
		}
	};

	MainWindow.prototype.startAppKnowledgeBase = function() {
		
		if (inConcert.knowledge_base_config && inConcert.conf_dashboard) {
			let apps = {};
			apps.knowledge_base_config = new inConcert.knowledge_base_config();
			inConcert.knowledge_base = new inConcert.conf_dashboard({
				apps: apps,
				dashboard: $("#knowledge_base_dashboard"),
				prefix_id: "conf_knowledge_base_",
				permissions: inConcert.permissions,
				main_html: "/inconcert/apps/knowledge_base/pages/conf_knowledge_base_main.<%=_L%>.html#grunt-cache-bust"
			});
		}
	};

	MainWindow.prototype.startAppFlowDesigner = function() {
		
		if (inConcert.flow_designer_config && inConcert.conf_dashboard) {
			let apps = {};
			apps.process_flows_config = new inConcert.flow_designer_config();
			inConcert.flow_designer = new inConcert.conf_dashboard({
				apps: apps,
				dashboard: $("#flow_designer_dashboard"),
				prefix_id: "conf_flow_designer_",
				permissions: inConcert.permissions,
				main_html: "/inconcert/apps/flow_designer/pages/conf_flow_designer_main.<%=_L%>.html#grunt-cache-bust"
			});
		}
	};

	MainWindow.prototype.startAppCognitiveServices = function() {
		
		if (inConcert.cognitive_services_config && inConcert.conf_dashboard) {
			let apps = {};
			apps.cognitive_services_config = new inConcert.cognitive_services_config();
			inConcert.cognitive_services = new inConcert.conf_dashboard({
				apps: apps,
				dashboard: $("#cognitive_services_dashboard"),
				prefix_id: "conf_cognitive_services_",
				permissions: inConcert.permissions,
				main_html: "/inconcert/apps/cognitive_services/pages/conf_cognitive_services_main.<%=_L%>.html#grunt-cache-bust"
			});
		}
	};

	MainWindow.prototype.startAppBatchManagement = function() {
		if (!inConcert.BatchManagementApp) {
			return;
		}
		inConcert.batch_management = new inConcert.BatchManagementApp();
	};

	MainWindow.prototype.startHelpDeskDashboard = function() {
		if (!inConcert.helpdesk) {
			return;
		}
		inConcert.helpdesk_dashboard = new inConcert.helpdesk.dashboard({
			container_id: "helpdesk_dashboard"
		});
	};

	MainWindow.prototype.startAppsLauncher = function() {
		if (!inConcert.apps_launcher) {
			return;
		}
		inConcert.apps_launcher_dashboard = new inConcert.apps_launcher.dashboard({
			container_id: "apps_launcher_dashboard"
		});
	};

	MainWindow.prototype.startAppMonitoring = function() {
		if (!inConcert.app_monitoring) {
			return;
		}
		inConcert.monitoring = new inConcert.app_monitoring.dashboard({
			container_id: "monitoring_dashboard"
		});
	};

	//initializes the GUI, basically it consists of getting a grip to every panel available
	MainWindow.prototype.initGUI = function()
	{
		const self = this;

		const windowManager = inConcert.app.getWindowManager();
		this.loadPersonalInfo();

		//load panels
		this.m_panels.shared = new inConcert.SharedPanel("int_shared_panel");
		this.m_panels.conversations_panel = new inConcert.ConversationsPanel("int_conversations_panel");

		this.m_panels.conversations_panel.initialize(function(callback = $.noop) {
			/* ---START-------------------------------------
				Start de las diferentes Apps.
				Deben iniciarse antes que el dashboard.
			*/
			self.setupConfigurationHandlers();
			self.setupToolbarHandlers();
			self.startAppConfiguration();
			self.startAppSupervisor();
			self.startAppSocialNetworks();
			self.startAppBatchManagement();
			self.startHelpDeskDashboard();
			self.startAppCampaignAnalytics();
			self.startAppsLauncher();
			self.startAppReportDesign();
			self.startAppOutboundEngine();
			self.startAppMonitoring();
			self.startAppWebDesigner();
			self.startAppFlowDesigner();
			self.startAppCognitiveServices();
			self.startAppKnowledgeBase();
			// ---END----------------------------------------
			self.initializeDashboardsApps(); // dashaboard menu options
			self.repositionPanels();
			callback();
		});

		const allowed_app_interaction = inConcert.permissions.to_application("interaction");
		if (!allowed_app_interaction) {
			$("body").removeClass("withoutQuickbar").addClass("withoutQuickbar");
			$("#interactionsMenuTop").remove();
		}
		else {
			this.refreshQuickbar(allowed_app_interaction);
		}

		this.refreshPermissionGUI();

		//--------------------------
		var changeState = function(state) {
			inConcert.app.changeUserState(state, function(response) {
				if (response.success) {
					self.onUserStateChanged(inConcert.app.getCurrentStatus());
				}
			});
		};

		const initialState = inConcert.app.getInitialState();
		if (initialState && initialState !== inConcert.app.getCurrentStatus() && allowed_app_interaction) {
			changeState(initialState);
		}

		const storage = inConcert.Storage;
		if (storage.get("setAwayWhenIdle", "boolean")) {
			windowManager.setIdleThreshold(storage.get("idleThreshold", "number"));
		}

		windowManager.onUserIdle = function() {
			if (storage.get("setAwayWhenIdle", "boolean") && inConcert.app.getCurrentStatus() === "ACTIVE") {
				self.m_awayFromActive = true;
				changeState("AWAY");
			}
		};

		windowManager.onUserPresent = function() {
			if (storage.get("setAwayWhenIdle", "boolean") && self.m_awayFromActive) {
				// self.m_previousState should be set
				self.m_awayFromActive = false;
				changeState("ACTIVE");
			}
		};

		windowManager.onApplicationGainFocus = $.noop;

		setTimeout(function(){
			self.onWindowResize();
		}, 50);

		//only now set window visible if in a native window environment
		this.show();
	};

	//when jquery says so, start it all up
	$(document).ready(function() {
		inConcert.i18n.updateScreen();
		inConcert.permissions = new inConcert.perm();
		inConcert.app = new inConcert.Application({});
		inConcert.app.start();
		inConcert.app.bootstrap(function(success, error) {
			if (success) {
				var desktop_notifications = inConcert.app.getDesktopNotifications();
				desktop_notifications.start_listening();
			}
			else {
				let result = "FAIL";
				let title = "<%=_T('Login Error')%>";
				let description = "<%=_T('Error loading app. You should exit and login again.')%>";
				if (error && error.loginError) {
					switch(error.loginError) {
						case "no available peer":
							result = "NO_AVAILABLE_PEER";
							description = "<%=_T('You are assigned to a telephony campaign but there are not available peers. Please contact a supervisor.')%>";
							break;
						case "no available license":
							result = "NO_AVAILABLE_LICENSE";
							description = "<%=_T('Concurrent user limit reached.')%>";
							break;
					}
				}
				if (inConcert.app.agent_api_server) {
					inConcert.app.agent_api_server.UserLoginFailed(result, description);
				}
				if (inConcert.app.isEmbedded()) {
					window.location.reload();
				}
				else {
					$.inconcertPopup.Show(
						title,
						description,
						$.inconcertPopup.ButtonSets.BS_OK,
						function(/*response*/) {
							window.location.reload();
						},
						undefined,
						{
							popup_id        : "agent_inconcertPopup",
							container_class : "confirmationPopUp",
							overlay_class   : "confirmationOverlay"
						}
					);
				}
			}
		});
		// Inhabilita la combinacion de teclas "Ctrl+S" en la ventana principal.
		inConcert.Utils.disableCtrlSHotKeys();
	});
}());
