/*jshint browser:true, quotmark:false, smarttabs:true, esversion: 11 */
/*global $, window, inConcert, CryptoJS */

// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	"use strict";

	const _T = inConcert.i18n.getString;
	const format = inConcert.Utils.format;
	let m_telephoneCodes = null;

	/**
	Class PersonalSettings.
	*/
	function PersonalSettings ()
	{
		PersonalSettings.uber.constructor.call(this,
			{
				move: ["topDragBar", "bottomDragBar"],
				resizer: "resizeGripper",
				close: "closePersonalSettings"
			},
			{ disableDblClickMaximize: false }
		);

		document.title = "<%=_T('Personal Settings')%>";
		m_telephoneCodes = inConcert.app.getInstanceTelephonyCodes();
		this.m_idleDefault = 2;   // el mismo valor por defecto que se encuentra en el LUA 'agent.lua' en la propiedad 'idleThreshold' de la funcion 'AgentStartup'
		this.m_tabs_config = null;
		this.m_layout_hnd = null;
		this.m_grid_layouts = null;
		// inicializo la clase
		this.__init();
	}

	inConcert.Utils.inherit(PersonalSettings, inConcert.Dialog);

	let m_passwordChanged = false;

	var displayErrorMessage = function(errorMessage) {
		$("#personalSettingsConfiguration")
			.find(".messageError")
			.show()
			.find(".messageText")
			.text(errorMessage);
	};

	var hideErrorMessage = function() {
		$("#personalSettingsConfiguration")
			.find(".messageError")
			.hide()
			.find(".messageText")
			.text("");
	};

	var showFieldError = function(controlId, message) {
		const $ctrl = $("#" + controlId);
		const $ctrl_err_mark = $("#" + controlId + "_validation");

		window.setTimeout(function() {
			$ctrl.focus();
		}, 50);

		$ctrl_err_mark.show();
		displayErrorMessage(message);

		window.setTimeout(function() {
			$ctrl_err_mark.hide();
			hideErrorMessage();
		}, 5000);

		return false;
	};

	/*******************************************************************
	 *
	 */
	var showAreaName = function(country, areaCode) {
		var $area = $("#lblAreaName");
		m_telephoneCodes.findAreaByCode(
			country,
			areaCode,
			function( area ) {
				if (area.success) {
					$area.text(area.name).show();
				}
				else {
					$area.text("").hide();
				}
			}
		);
	};

	/*******************************************************************
	 * Metodo que valida los valores de los controles a ingresar!!!
	 */
	var validate = function(callback = $.noop)
	{
		$(".popUpFormField_validation").hide();
		hideErrorMessage();

		// Primer Nombre
		if ($("#settingsNewUserFirstName").val().length === 0) {
			showFieldError("settingsNewUserFirstName", "<%=_T('First name is required.')%>");
			callback( false /* error*/ );
			return;
		}
		// Primer Apellido
		if ($("#settingsNewUserFirstSurname").val().length === 0) {
			showFieldError("settingsNewUserFirstSurname", "<%=_T('First surname is required.')%>");
			callback( false /* error*/ );
			return;
		}

		const has_allegro_integration = inConcert.app.has_allegro_integration();

		// Contrasena
		if (m_passwordChanged) {
			if ($("#settingsNewPassword").val() !==  $("#settingsNewPasswordConfirm").val()) {
				showFieldError("settingsNewPasswordConfirm", "<%=_T('Passwords do not match.')%>");
				callback( false /* error*/ );
				return;
			}
			// Resto de validaciones se realizan en el server
		}

		const is_system_vcc = inConcert.app.is_system_vcc();

		// Allegro Integration
		if (!has_allegro_integration && !is_system_vcc) {
		// Validacion que se realiza unicamente SI NO tenemos integracion con Allegro
			var initialState = $("#settingsNewInitialState").val();
			if (initialState.trim().length === 0) {
				showFieldError("settingsNewInitialState", "<%=_T('Initial state is required.')%>");
				callback( false /* error*/ );
				return;
			}
			if (initialState.toUpperCase() === "ACTIVE") {
				var allowed_app_interaction = inConcert.permissions.to_application( "interaction" );
				if (!allowed_app_interaction) {
					showFieldError("settingsNewInitialState", "<%=_T('You must have permissions to Interaction application to use Available system state.')%>");
					callback( false /* error*/ );
					return;
				}
			}
		}


		if (is_system_vcc) {
		// SI es un VCC del sistema
			callback(true /* success*/);
			return;
		}

		// SI NO es un VCC del sistema
		var country = $("#cboCountry").val();
		if (country.trim().length === 0 ) {
			showFieldError("cboCountry", "<%=_T('Country is required.')%>");
			callback( false /* error*/ );
			return;
		}

		if ($("#chkAwayWhenIdle").prop("checked")) {
			var idleThreshold = parseInt($("#idleThreshold").val(), 10);
			if (isNaN(idleThreshold) || idleThreshold <= 0) {
				showFieldError("idleThreshold", "<%=_T('Idle time should be a number greater than 0')%>");
				callback( false /* error*/ );
				return;
			}
		}

		// Check area code. If it is empty and the country table does not provide an entry for it,
		// it is required. Else, just check for existence.
		var areaCode = $("#txtArea").val();

		m_telephoneCodes.findAreaByCode(
			country,
			areaCode,
			function( area ) {
				if (!area.success) {
					if(areaCode === "") {
						showFieldError("txtArea", "<%=_T('Area code is required.')%>");
					}
					else {
						showFieldError("txtArea", "<%=_T('Area code does not exist.')%>");
					}
					callback( false /* error*/ );
				}
				else {
					callback( true /* success*/ );
				}
			}
		);

	};
	PersonalSettings.prototype.makePasswordsCheckUI = function(passwordChecks)
	{
		if (typeof passwordChecks !== "object") {
			return [];
		}
		const makePasswordCheckUI = function(cssClass, text) {
			return $(
				"<div class='" + cssClass + "'>" +
					text +
				"</div>"
			);
		};

		let translatedPasswordChecks = [];
		//passwordLen always goes
		let passwordLenText = inConcert.Utils.format("<%=_T('Be between {0} and {1} characters long')%>", passwordChecks.LEN.range[0], passwordChecks.LEN.range[1]);

		let passwordLenClass = passwordChecks.LEN.success === true ? "passwordCheckStatusPass" : (passwordChecks.LEN.success === false ? "passwordCheckStatusFail" : "passwordCheckStatusInitial");
		translatedPasswordChecks.push(makePasswordCheckUI(passwordLenClass, passwordLenText));

		if(passwordChecks.LOWERS.range[0] > 0) {
			let minLowerCharsText = inConcert.Utils.format("<%=_T('Include at least {0} lowercase characters')%>", passwordChecks.LOWERS.range[0]);
			let minLowerCharsLenClass = passwordChecks.LOWERS.success === true ? "passwordCheckStatusPass" : (passwordChecks.LOWERS.success === false ? "passwordCheckStatusFail" : "passwordCheckStatusInitial");
			translatedPasswordChecks.push(makePasswordCheckUI(minLowerCharsLenClass, minLowerCharsText));
		}
		if(passwordChecks.UPPERS.range[0] > 0) {
			let minUpperCharsText = inConcert.Utils.format("<%=_T('Include at least {0} uppercase characters')%>", passwordChecks.UPPERS.range[0]);
			let minUpperCharsLenClass = passwordChecks.UPPERS.success === true ? "passwordCheckStatusPass" : (passwordChecks.UPPERS.success === false ? "passwordCheckStatusFail" : "passwordCheckStatusInitial");
			translatedPasswordChecks.push(makePasswordCheckUI(minUpperCharsLenClass, minUpperCharsText));
		}
		if(passwordChecks.NUMBERS.range[0] > 0){
			let minNumericCharsText = inConcert.Utils.format("<%=_T('Include at least {0} numeric characters')%>", passwordChecks.NUMBERS.range[0]);
			let minNumericCharsLenClass = passwordChecks.NUMBERS.success === true ? "passwordCheckStatusPass" : (passwordChecks.NUMBERS.success === false ? "passwordCheckStatusFail" : "passwordCheckStatusInitial");
			translatedPasswordChecks.push(makePasswordCheckUI(minNumericCharsLenClass, minNumericCharsText));
		}
		if(passwordChecks.SYMBOLS.range[0] > 0) {
			let minSpecialCharsText = inConcert.Utils.format("<%=_T('Include at least {0} special characters')%>", passwordChecks.SYMBOLS.range[0]);
			let minSpecialCharsLenClass = passwordChecks.SYMBOLS.success === true ? "passwordCheckStatusPass" : (passwordChecks.SYMBOLS.success === false ? "passwordCheckStatusFail" : "passwordCheckStatusInitial");
			translatedPasswordChecks.push(makePasswordCheckUI(minSpecialCharsLenClass, minSpecialCharsText));
		}
		if(passwordChecks.HISTORY.range[0] > 0) {
			let historyText = inConcert.Utils.format("<%=_T('Not have been used in the last {0} passwords')%>", passwordChecks.HISTORY.range[0]);
			let historyClass = passwordChecks.HISTORY.success === true ? "passwordCheckStatusPass" : (passwordChecks.HISTORY.success === false ? "passwordCheckStatusFail" : "passwordCheckStatusInitial");
			translatedPasswordChecks.push(makePasswordCheckUI(historyClass, historyText));
		}
		return translatedPasswordChecks;
	};

	PersonalSettings.prototype.save = function(setting, callback = $.noop)
	{
		const self = this;

		inConcert.logger.action({action_name : "save_settings", action_type:"agent"});
		hideErrorMessage();

		const $waiting = $("#personalSettingsWaiting");
		$waiting.show();

		const url = inConcert.app.getBaseUrl() + "/save_settings/";
		inConcert.invoke({
			url: url,
			type: "POST",
			data: JSON.stringify(setting),
			dataType: "json",
			contentType: "application/json",
			success: function( response ) {
				$waiting.hide();
				if (response.result) {
					inConcert.app.updateSettings(setting);
					inConcert.app.m_mainWindow.updateUserInformation();
					callback(true, function() {
						// Se agrega un timeout al cerrar la ventana, que sino el plugin de jquery se manca con el self en IE
						//  http://redmine.inconcert/issues/7109
						window.setTimeout(function() {
							self.m_layout_has_changed = false;
							self.close();
						});
					});
				}
				else {
					if (typeof response.error === "string") {
						if (response.error === "NOT ALLOWED INITIAL AVAILABLE SYSTEM STATE") {
							showFieldError("settingsNewInitialState", "<%=_T('You must have permissions to Interaction application to use Available system state.')%>");
							callback(false);
							return;
						}
					}
					// Estos codigos de error se generan en la funcion "ValidateUser" del controller "users.lua" de la App "agent"
					const translated_errs = {
						"PASSWORD IS REQUIRED"          : (err)=> "<%=_T('Password is required.')%>",
						"INVALID CHARACTERS IN PASSWORD": (err)=> "<%=_T('The new password value must not contain reserved characters (#, &, ?, =).')%>",
						"INVALID JWT PASSWORD"          : (err)=> "<%=_T('The new password has a invalid JWT.')%>",
						"PASSWORD EXTERNAL AUTH ERROR"  : (err)=> {
							$("#password_checks_list").html("");
							if (typeof err.fieldValue === "object") {
								for(let checkElem of self.makePasswordsCheckUI(err.fieldValue)) {
									$("#password_checks_list").append(checkElem);
								}
							}
							return "<%=_T('Invalid password')%>";
						}
					};
					const errType = (response.error.msg || "").toUpperCase();
					const errFn = translated_errs[errType];
					if (!errFn) {
						displayErrorMessage(response.error.msg || "<%=_T('Oops, theres been an error saving your settings.')%>");
						return callback(false);
					}
					const msgErr = errFn(response.error);
					showFieldError("settingsNewPassword", msgErr);
					callback(false);
				}
			},
			error: function( response ) {
				$waiting.hide();
				var message_error;
				if ( response.status === 403 ) {
					message_error = "<%=_T('You dont have permission to save personal settings.')%>";
				}
				else {
					message_error = format("<%=_T('Oops, theres been an error saving your settings: {0}')%>", response.status );
				}
				displayErrorMessage( message_error );
				callback(false);
			}
		});
	};

	PersonalSettings.prototype.onWindowClosed = function()
	{
		PersonalSettings.uber.onWindowClosed.call( this );
		this.dispose();
	};

	PersonalSettings.prototype.load_user_date_formats = function()
	{
		const formats = [
			{ id: "dd/mm/yy", name: "dd/mm/yy" },
			{ id: "dd/mm/yyyy", name: "dd/mm/yyyy" },
			{ id: "mm/dd/yyyy", name: "mm/dd/yyyy" },
			{ id: "yyyy-mm-dd", name: "yyyy-mm-dd" }
		];

		const $format_container = $("#settingsUserDateFormat");
		const is_blank_first_item = false;
		const attr_id = "id";
		const attr_name = "name";

		inConcert.Utils.fill_combo( $format_container, is_blank_first_item, formats, attr_id, attr_name );
		$("#settingsUserDateFormat").val(inConcert.app.getLoggedUserDateFormat() || "dd/mm/yy");
	};

	PersonalSettings.prototype.dispose = function() {
		if (this.m_layout_hnd) {
			this.m_layout_hnd.dispose();
		}
	};

	PersonalSettings.prototype.validate_and_save = function()
	{
		const self = this;

		validate(function(success)
		{
			if (!success) {
				self.m_tabs_config.switch_tab("user_info");
				return;
			}

			const setting = {
				personal: {
					Name: $("#settingsNewUserFirstName").val(),
					LastName: $("#settingsNewUserLastName").val(),
					FirstSurname: $("#settingsNewUserFirstSurname").val(),
					LastSurname: $("#settingsNewUserSurname").val(),
					countryCode: $("#cboCountry").val(),
					areaCode:  $("#txtArea").val(),
					DateFormat :  $("#settingsUserDateFormat").val(),
					EnterAsKeyToSend: ($("#config_advanced_container_action_keyboard").prop("checked")) ? true : false,
					idleThreshold      : $("#chkAwayWhenIdle").prop("checked")        ? $("#idleThreshold").val() : "-1",
					soundNotifications : $("#chkSoundNotifications").prop("checked")  ? "1" : "0",
					visualNotifications: $("#chkVisualNotifications").prop("checked") ? "1" : "0",
				}
			};

			// el diseno de layouts
			self.setClientsLayout(setting);

			// Allegro integration
			if ( !inConcert.app.has_allegro_integration() ) {
				// el initial state se agrega unicamente SI NO tenemos integracion con Allegro.
				setting.personal.initialState = $("#settingsNewInitialState").val();
			}
			if (m_passwordChanged) {
				setting.personal.Password = inConcert.Utils.md5(inConcert.app.getLoggedUserId().toLowerCase() + $("#settingsNewPassword").val());
				setting.personal.PasswordHasChanged = true;	//Le indicamos al server que hemos cambiado la password, para que valide tambien si hemos puesto la misma que la actual
				
				// Ademas de enviar el md5, enviamos el jwt para poder hacer las validaciones en el server
				var header = {
					"alg": "HS256",
					"typ": "JWT"
				};

				var stringifiedHeader = CryptoJS.enc.Utf8.parse(JSON.stringify(header));
				var encodedHeader = inConcert.Utils.base64url(stringifiedHeader);

				let data = {
				  "password": $("#settingsNewPassword").val()
				};

				var stringifiedData = CryptoJS.enc.Utf8.parse(JSON.stringify(data));
				var encodedData = inConcert.Utils.base64url(stringifiedData);

				var token = encodedHeader + "." + encodedData;
				var secret = inConcert.app.getLoggedUserToken();

				var signature = CryptoJS.HmacSHA256(token, inConcert.Utils.md5(secret));
				signature = inConcert.Utils.base64url(signature);

				var signedToken = token + "." + signature;
				setting.personal.jwtToken = signedToken;
			}

			// almaceno
			self.save(setting, function(success, callback = $.noop) {
				if (success) {
					const idleThreshold = parseInt($("#idleThreshold").val(), 10);
					inConcert.app.getWindowManager().setIdleThreshold(idleThreshold);
				}
				callback();
			});
		});
	};

	/*******************************************************************
	 * Metodo que setea los eventos de los controles!!
	 */
	PersonalSettings.prototype.setEvents = function() {
		const self = this;

		$("#cboCountry").change(function() {
			$("#txtArea").val("").change();
		});
		var findAreaTimeout;
		$("#txtArea")
			.change(function() {
				showAreaName($("#cboCountry").val(), $("#txtArea").val());
			}).keydown(function() {
				clearTimeout(findAreaTimeout);
			}).keyup(function() {
				findAreaTimeout = setTimeout(function() {
					showAreaName($("#cboCountry").val(), $("#txtArea").val());
				}, 600);
			})
			.bind("paste", null, function() {
				showAreaName($("#cboCountry").val(), $("#txtArea").val());
			});

		$("#saveUserSettings").click(function() {
			self.validate_and_save();
		});

		$("#cancelSaveUserSettings").click(function() {
			if (!self.canCloseWindow()) {
				return;
			}
			var currentSkin = inConcert.app.getMainWindow().getCurrentSkin();
			if (currentSkin !== self.previousSkin) {
				inConcert.app.getMainWindow().setSkin(self.previousSkin);
			}
			self.close();
		});

		$("#settingsNewPassword").focus(function() {
			var that = this;
			setTimeout(function(){$(that).select();},10);
		});

		$("#personalSettingsConfiguration").find(".messageError .messageClose").click(function() {
			hideErrorMessage();
			return false;
		});

		// Esta funcion muestra el textbox de confirmacion de pwd, y unbindea los eventos de los controles que la llaman
		var showPasswordConfirm = function() {
			if (m_passwordChanged) {
				return;
			}
			m_passwordChanged = true;
			$("#passwordConfirmSection").show();
		};
		//---------------
		// eventos (paste, keypress, keydown) del text de pwd que habilitan el textbox de confirmación de pwd
		//
		$("#settingsNewPassword")
			.bind("paste", null, showPasswordConfirm)	//evento paste, ya sea desde el menu contextual o presionando ctrl-v
			.bind("keypress", showPasswordConfirm)		//este evento captura las teclas presionandas que modifican el valor, menos las teclas "backspace" y "delete"
			.bind("keydown", function(event) {
				/*
					En el evento keydown capturo y proceso las teclas que se han presionado y que modifican el texto existente.
						-keyCode 8  = tecla "backspace" y -keyCode 46 = tecla "delete"
				*/
				if (event.keyCode === 8 || event.keyCode === 46) {
					showPasswordConfirm();
				}
			}
		);

		// enable/disable away controls when this checkbox changes
		$("#chkAwayWhenIdle").change(function() {
			var $parentClass = $(this).parent().find(".idleThreshold_label");
			var $idleThreshold = $("#idleThreshold");
			if($(this).prop("checked")) {
				$parentClass.removeClass("disabled");
				var value = $idleThreshold.attr("beforeValue");
				if (!value) {
					value = self.m_idleDefault;
				}
				$idleThreshold.removeAttr("disabled").val(value).focus();
			} else {
				$parentClass.addClass("disabled");
				$idleThreshold.attr("beforeValue", $idleThreshold.val());
				$idleThreshold.val("").prop("disabled", true);
			}
		});

		var isDigitKey = function (e) {
			var key = e.keyCode;
			return ((key >= 48 && key <= 57) || (key >= 96 && key <= 105)) && !(e.altKey || e.shiftKey || e.ctrlKey);
		};

		// do not allow the user to paste garbage into this control
		$("#idleThreshold").bind("paste", function(e) {
			var text = e.originalEvent.clipboardData.getData("text/plain");
			for (var i = 0; i < text.length; ++i) {
				if ( text[i] < "0" || text[i] > "9") {
					return false; // parseInt is not good enough
				}
			}
		})
		.bind("keydown", function(e) {
			if (e.keyCode === 40 || e.keyCode === 38) {
			// allow to increment or decrement using arrow keys
				var currentValue = $(this).val().length > 0 ? parseInt($(this).val(), 10) : 0;
				var sum = (e.keyCode === 40 && currentValue > 1) ? -1 : (e.keyCode === 38 && currentValue < 99) ? 1 : 0;
				$(this).val(currentValue + sum);
				return false;
			}
			return isDigitKey(e) || e.keyCode === 8 || e.keyCode === 39 || e.keyCode === 37;
		})
		.bind("keypress", function(e) {
			return isDigitKey(e);
		});

		$("#settingsNewPassword")
			.off("focusin")
			.on("focusin", function() {
				$("#password_checks_container").toggle(!inConcert.app.HasExternalUsersAuth());
			});
		$("#settingsNewPassword")
			.off("focusout")
			.on("focusout", function() {
				$("#password_checks_container").toggle(false);
			});		

		this.m_grid_layouts.set_events();
	};

	/**
	Load initial values in form controls
	*/
	PersonalSettings.prototype.loadingControls = function()
	{
		const self = this;

		var addComboValue = function(combo, id, description, selected) {
			var option = $("<option></option>")
				.attr("value", id)
				.text(_T(description));
			if (selected) {
				option.attr("selected", "selected");
			}
			combo.append(option);
		};

		var buildCountriesList = function(countries_map) {
			var countries_list = [];
			for (var country_id in countries_map) {
				if (countries_map.hasOwnProperty(country_id)) {
					countries_list.push( { code: country_id, name: _T(countries_map[country_id].name) });
				}
			}
			return inConcert.Utils.sortArray(countries_list, "name");
		};

		// countries
		var $cbo_countries = $("#cboCountry");
		addComboValue($cbo_countries, "", "", true);
		m_telephoneCodes.loadCountries( function( success, countries )
		{
			if ( !success ) {
				return;
			}
			var countries_list = buildCountriesList( countries );
			for (var i = 0; i < countries_list.length; i++) {
				var country = countries_list[i];
				addComboValue($cbo_countries, country.code, country.name, false);
			}
			$cbo_countries.val(inConcert.app.getCountryCode());
			$("#txtArea").val(inConcert.app.getAreaCode()).change();
		});

		var cmbSkin = $("#cboSkin");
		var currentSkin = inConcert.app.getMainWindow().getCurrentSkin();
		this.previousSkin = currentSkin;
		addComboValue(cmbSkin, "skinLight", "<%=_T('Light')%>", currentSkin === "skinLight");
		//addComboValue(cmbSkin, "skinDark", "<%=_T('Dark')%>", currentSkin === "skinDark");
		addComboValue(cmbSkin, "skinNew", "<%=_T('Nuevo Skin')%>", currentSkin === "skinNew");
		cmbSkin.change(function() {
			var newSkin = $(this).val();
			inConcert.app.getMainWindow().setSkin(newSkin);
		});

		// other information
		$("#settingsNewUserFirstName").val(inConcert.app.getLoggedUserFirstName()).focus();
		$("#settingsNewUserLastName").val(inConcert.app.getLoggedUserLastName());
		$("#settingsNewUserFirstSurname").val(inConcert.app.getLoggedUserFirstSurname());
		$("#settingsNewUserSurname").val(inConcert.app.getLoggedUserLastSurname());
		this.load_user_date_formats();

		// key send button status
		if (inConcert.app.getSendKeyStatus())
			$("#config_advanced_container_action_keyboard").prop("checked",true);

		//inicializo el plugin password strength
		$("#settingsNewPassword").inconcert_password_strength({
			veredictWords: {
				 empty: {
					text: "<%=_T('Empty')%>",
					className: "passwordMessage passwordEmpty"
				 },
				 very_weak: {
					text: "<%=_T('Very Weak')%>",
					className: "passwordMessage passwordVeryWeak"
				 },
				 weak: {
					text: "<%=_T('Weak')%>",
					className: "passwordMessage passwordWeak"
				 },
				 mediocre: {
					text: "<%=_T('Mediocre')%>",
					className: "passwordMessage passwordMediocre"
				 },
				 strong: {
					text: "<%=_T('Strong')%>",
					className: "passwordMessage passwordStrong"
				 },
				 stronger: {
					text: "<%=_T('Stronger')%>",
					className: "passwordMessage passwordStronger"
				 }
			}
		});

		$("#chkSoundNotifications").prop("checked", inConcert.app.getUserSoundNotifications()).trigger("change");
		$("#chkVisualNotifications").prop("checked", inConcert.app.getUserVisualNotifications()).trigger("change");

		const idleThreshold = inConcert.app.getUserdIdleThreshold();
		if (idleThreshold > -1) {
			$("#chkAwayWhenIdle").prop("checked", true).trigger("change");
			$("#idleThreshold").val(idleThreshold);
		}

		//Show password will expire soon notification, when apply
		let passwordExpirationDays = inConcert.app.PasswordExpirationDays();
		if(passwordExpirationDays)
		{
			var passwordExpirationDaysText = inConcert.Utils.format("<%=_T('Your password will expire in {0} days.')%>", passwordExpirationDays);
			$("#settingsNewPassword_passwordExpirationDaysText").text(passwordExpirationDaysText).parent().show();
		}		

	// Allegro Integration!!!
	//
		if ( inConcert.app.has_allegro_integration() ) {
			$("#settings_initial_state_section").hide();
		}
		else {
			$("#settings_initial_state_section").show();

		// Informacion que se carga unicamente SI NO tenemos integracion con Allegro
			// states
			addComboValue($("#settingsNewInitialState"), "", "", true);
			var states = inConcert.app.getAvailableStates();
			for (var si = 0; si < states.length; si++) {
				var state = states[si];
				addComboValue($("#settingsNewInitialState"), state.Id, state.Description, false);
			}
			$("#settingsNewInitialState").val(inConcert.app.getInitialState());
		}

		if (this.__isSystemVcc()) {
			$("#config_tab_user_info").find("[user_type='not-system']").hide();
			this.m_tabs_config.hide_tab_with_content("layouts");
		}

		this.__load_user_layouts(function(success, layouts) {
			if (!success) {
				return;
			}
			self.m_grid_layouts.initialize($.noop, layouts);
		});

		// Password checks
		var vccInfo = inConcert.app.get_vcc_info();
		$("#settingsNewUserFirstName").prop("disabled", vccInfo.HasExternalUsersSync).parent().find(".form-label").addClass("disabled");
		$("#settingsNewUserFirstSurname").prop("disabled", vccInfo.HasExternalUsersSync).parent().find(".form-label").addClass("disabled");
		$("#settingsNewPassword").prop("disabled", vccInfo.HasExternalUsersSync).parent().find(".form-label").addClass("disabled");
		$("#settingsNewPasswordConfirm").prop("disabled", vccInfo.HasExternalUsersSync).parent().find(".form-label").addClass("disabled");

		if(!vccInfo.HasExternalUsersSync && vccInfo.passwordChecks) {
			$("#password_checks_list").html("");
			for(var checkElem of this.makePasswordsCheckUI(vccInfo.passwordChecks)) {
				$("#password_checks_list").append(checkElem);
			}
		}
	};

/*
Sample:
	{"MAIL":{"Id":"MAIL","MAIL":{"isClosable":true,"reorderEnabled":true,"title":"","openPopouts":[],"settings":{"constrainDragToContainer":true,"hasHeaders":true,"reorderEnabled":true,"popoutWholeStack":false,"blockedPopoutsThrowError":true,"responsiveMode":"onload","closePopoutsOnUnload":true,"showMaximiseIcon":true,"selectionEnabled":true,"showPopoutIcon":false,"showCloseIcon":true},"labels":{"maximise":"Maximizar","close":"close","popin":"pop in","popout":"open in new window","tabDropdown":"Pestañas Adicionales","minimise":"Minimizar"},"content":[{"type":"row","title":"","reorderEnabled":true,"content":[{"isClosable":true,"title":"","reorderEnabled":true,"type":"column","content":[{"isClosable":true,"title":"","reorderEnabled":true,"height":100,"content":[{"isClosable":false,"type":"component","title":"Conversa..","reorderEnabled":true,"id":"main","tooltip":"Conversación","componentState":{"id":"main"},"componentName":"testComponent"}],"activeItemIndex":0,"type":"stack"}],"width":60},{"type":"stack","title":"","reorderEnabled":true,"content":[{"isClosable":true,"type":"component","title":"Informac..","reorderEnabled":true,"id":"client_sidebar_context_tab","tooltip":"Información de contexto","componentState":{"id":"client_sidebar_context_tab"},"componentName":"testComponent"},{"isClosable":true,"type":"component","title":"Conversa..","reorderEnabled":true,"id":"client_sidebar_conversations_tab","tooltip":"Conversaciones anteriores","componentState":{"id":"client_sidebar_conversations_tab"},"componentName":"testComponent"},{"isClosable":true,"type":"component","title":"Historial","reorderEnabled":true,"id":"client_sidebar_browsing_tab","tooltip":"Historial","componentState":{"id":"client_sidebar_browsing_tab"},"componentName":"testComponent"},{"isClosable":true,"type":"component","title":"Plantillas","reorderEnabled":true,"id":"client_sidebar_canned_tab","tooltip":"Plantillas","componentState":{"id":"client_sidebar_canned_tab"},"componentName":"testComponent"},{"isClosable":true,"type":"component","title":"Integrac..","reorderEnabled":true,"id":"client_sidebar_integration_tab","tooltip":"Integración","componentState":{"id":"client_sidebar_integration_tab"},"componentName":"testComponent"},{"isClosable":true,"type":"component","title":"QnA Maker","reorderEnabled":true,"id":"client_sidebar_qna_maker_tab","tooltip":"QnA Maker","componentState":{"id":"client_sidebar_qna_maker_tab"},"componentName":"testComponent"},{"isClosable":true,"type":"component","title":"Formular..","reorderEnabled":true,"id":"client_sidebar_contactform_tab","tooltip":"Formulario de contacto","componentState":{"id":"client_sidebar_contactform_tab"},"componentName":"testComponent"}],"isClosable":true,"activeItemIndex":0,"width":40}],"isClosable":true}],"dimensions":{"headerHeight":30,"dragProxyHeight":200,"dragProxyWidth":300,"minItemHeight":10,"borderWidth":5,"minItemWidth":10}},"Name":"MAIL"}};
*/
	PersonalSettings.prototype.setClientsLayout = function(user_settings) {
		let layouts = this.m_grid_layouts.get_items();
		layouts = inConcert.Utils.arrayToMap(layouts || [], 
			function(layout) {
				return layout;
			},
			function(layout) {
				return layout.Id;
			}
		);
		user_settings.personal.layouts = layouts;
	};

	PersonalSettings.prototype.__isSystemVcc = function()
	{
		return inConcert.app.is_system_vcc();
	};

	/*
	*/
	PersonalSettings.prototype.__init = function()
	{
		this.__setWindowResizingLimits();
		this.__initializeTabs();
		this.__initializeLayoutGrid();
		$("#settingsNewPasswordSection").toggle(!inConcert.app.HasExternalUsersAuth());
	};

	/*
	*/
	PersonalSettings.prototype.__setWindowResizingLimits = function()
	{
		// Estas dimensiones deben de coincidir con lo que se ha definido en "main_window.js" funcion "setupConfigurationHandlers"
		let DEFAULT_WINDOW_HEIGHT_PX = 630;
		let DEFAULT_WINDOW_WIDTH_PX = 815;
		if (this.__isSystemVcc()) {
			// Queda por compoatibilidad, ya que para el VCC System, NO se puede realizar resize de la ventana.
			DEFAULT_WINDOW_HEIGHT_PX = 490;
			DEFAULT_WINDOW_WIDTH_PX = 620;
		}
		this.setHorizontalResizingLimits(DEFAULT_WINDOW_WIDTH_PX);
		this.setVerticalResizingLimits(DEFAULT_WINDOW_HEIGHT_PX);
	};
	/*
		Crea los Tabs de las campanas que se visualiza en modo Pantalla Completa
	*/
	PersonalSettings.prototype.__initializeTabs = function()
	{

		// Instancia de la clase de los TABs de la App "Configuration"
		this.m_tabs_config = new inConcert.conf_form_tab({
			frm            : this,
			container_id   : "config_tabs_container",
			must_initialize: true,
			tabs_list      : {
				user_info: {
					id              : "user_info",
					name            : "<%=_T('User Information')%>",
					tab_container_id: "config_tab_user_info",
					selected        : true,
					on_show         : function() {
						//self.__on_show_tab_campaign("user_info");
					}
				},
				layouts: {
					id              : "layouts",
					name            : "<%=_T('Layouts')%>",
					tab_container_id: "config_tab_layout",
					selected        : false,
					on_show         : function() {
						//self.__on_show_tab_campaign("layouts");
					}
				},
				advanced: {
					id              : "advanced",
					name            : "<%=_T('Advanced')%>",
					tab_container_id: "config_tab_advanced",
					selected        : false,
					on_show         : function() {
						//self.__on_show_tab_campaign("layouts");
					}
				}
			}
		});

		this.m_tabs_config.show_tabs_container();
	};

	PersonalSettings.prototype.__initializeLayoutMng = function(current_layout)
	{
		if(this.m_layout_has_changed !== true)
		{
			this.m_layout_has_changed = false;
		}
		this.m_layout_hnd = new inConcert.LayoutUserDesign({
			owner       : this,
			container_id: "config_layout_design",
			parent      : {frm: this, is_readonly: false}
		});

		this.m_layout_hnd.set_readonly(false);
		this.m_layout_hnd.set_layout(current_layout || null);
		this.m_layout_hnd.initialize();
		let current_layouts = this.m_grid_layouts.get_items();
		this.m_layout_hnd.set_availables_layouts_types(current_layouts);
		this.m_layout_hnd.edit();
	};

	PersonalSettings.prototype.__initializeLayoutGrid = function()
	{
		const self = this;

		const all_channels = inConcert.app.load_vcc_channels();
		var get_layout_name = function(row)
		{
			let original_value = row.Id;
			let translated_value = all_channels[original_value] ? all_channels[original_value].name : "<%=_T('All')%>";
			return {
				original_value: original_value,
				value: translated_value
			};
		};

		this.m_grid_layouts = new inConcert.conf_form_paginator({
			owner: this,
			frm: this,
			row_id: "Id",
			display_name: "Name",
			rows_per_page: 11,
			show_at_least: 10,
			paginator_type: "ALL_ROWS_CLIENT_SIDE",
			without_paginator: true,
			loading_waiting_msg: "<%=_T('Please wait, loading Layouts ...')%>",
			ids: {
				paginator: "config_layout_paginator",
				grid: "config_layout_list",
				not_results_found: "config_layout_not_results_found",
				container_new_btn: "config_tab_layout_new_container",
				new_item: "config_tab_layout_new",
			//popup0
				popup: "config_layout_design_popup",
				popup_msg: "config_layout_design_message",
				popup_create: "config_layout_design_update",
				popup_close: "config_layout_design_close",
				popup_input_id: "config_layout_design_name",
			},
			strings: {
				not_results_found: "<%=_T('No Layouts found.')%>",
				can_remove       : "<%=_T('Do you want to delete Layout: {0} ?')%>",
				button_create    : "<%=_T('Add')%>"
			},
			columns: [
				{id: "_number", name: "", type: "counter"},
				{id: "Name"   , name: "<%=_T('Name')%>", get_value: get_layout_name},
				{
					id: "_action",
					name: "",
					system_action: {
						edit: true,
						erase: true
					}
				}
			],
			callbacks: {
				save: function(callback) {
					self.__add_or_update_layout(callback);
				},
				edit: function(row_data) {
					hideErrorMessage();
					self.__on_edit_layout(row_data);
				},
				on_removed_row: function(/*row, callback = $.noop*/) {
					self.m_layout_has_changed = true;
				},
				on_closed_popup: function() {
					hideErrorMessage();
					self.__on_close_edition_layout();
				},
				on_show_popup_msg: function(msg, type, response_object) {
					const ctrl_id = response_object.$ctrl.attr("id");
					showFieldError(ctrl_id, response_object.error_message);
				},
				on_hide_popup_msg: function() {
					window.setTimeout(function() {
						hideErrorMessage();
					}, 100);
				}
			}
		});
	};

	PersonalSettings.prototype.__on_edit_layout = function(row_data)
	{
		$("#setting_actions").hide();
		$("#config_tab_layout_grid").hide();
		this.m_tabs_config.enable_disable_all_tabs(false);
		$("#layout_actions").show();
		this.__initializeLayoutMng(row_data);
	};

	PersonalSettings.prototype.__on_close_edition_layout = function()
	{
		$("#layout_actions").hide();
		$("#config_tab_layout_grid").show();
		$("#setting_actions").show();
		this.m_tabs_config.enable_disable_all_tabs(true);
		this.m_layout_hnd.dispose();
	};

	PersonalSettings.prototype.__add_or_update_layout = function(callback = $.noop)
	{
		let layout_info = this.m_layout_hnd.validate();
		if (!layout_info.success) {
			let err = layout_info;
			callback(false, null, err);
			return;
		}
		this.m_layout_has_changed = true;
		let layout = {
			[layout_info.layout_id]: layout_info.layout,
			// Estos 2 atributos NO son necesarios server side, pero SI los necesito para visualizar info en la grilla.
			Id: layout_info.layout_id,
			Name: layout_info.layout_id
		};
		callback(true, layout);
	};

	PersonalSettings.prototype.__load_user_layouts = function(callback = $.noop)
	{
		let layouts = [];
		let user_custom_config = inConcert.app.loadLoggedUserClientsLayoutConfig();
		if (user_custom_config) {
			// el map de layouts se pasa a un array ordenado alfabeticamente
			layouts = inConcert.Utils.hash_to_sorted_array(user_custom_config,
				function(layout) {
					return layout.Name;
				}
			);
		}
		callback(true, layouts);
	};
	
	PersonalSettings.prototype.canCloseWindow = function(/*params*/) {
		const self = this;
		if (this.m_layout_has_changed) {
			self.m_tabs_config.switch_tab("layouts");
			const message = "<%=_T('The layout design was modified. Do you want to save before exit?')%>";
			this.confirmOrCancel(message, function(response) {
				switch (response)
				{
					case "yes":
						self.validate_and_save();
						break;
					case "no":
						self.m_layout_has_changed = false;
						self.close();
						break;
					case "cancel":
						//Nothing to do
						break;
				}
			});
			return false;
		}
		return true;
	};	

	PersonalSettings.prototype.confirmOrCancel = function(message, callback = $.noop)
	{
		$.inconcertPopup.Show(
			"<%=_T('Confirm')%>",
			message,
			$.inconcertPopup.ButtonSets.BS_YESNOCANCEL,
			function(response) {
				window.setTimeout(function() {
					callback(response.toLowerCase());
				});
			},
			undefined,
			{
				popup_id       : "conf_inconcertPopup",
				container_class: "confirmationPopUp",
				overlay_class  : "confirmationOverlay"
			}
		);
	};	

	PersonalSettings.prototype.confirm = function(message, callback = $.noop)
	{
		$.inconcertPopup.Show(
			"<%=_T('Confirm')%>",
			message,
			$.inconcertPopup.ButtonSets.BS_YESNO,
			function(response) {
				if (response.toLowerCase() !== "yes") {
					callback(false);
					return;
				}
				window.setTimeout(function() {
					callback(true);
				});
			},
			undefined,
			{
				popup_id       : "conf_inconcertPopup",
				container_class: "confirmationPopUp",
				overlay_class  : "confirmationOverlay"
			}
		);
	};

	/* =====================================
		(INICIO) Funciones utilizadas por las clases "conf_form_tab" y "conf_form_paginator"
	=====================================*/

	PersonalSettings.prototype.show_waiting = $.noop;
	PersonalSettings.prototype.hide_waiting = $.noop;

	PersonalSettings.prototype.is_readonly = function()
	{
		return false;
	};

	PersonalSettings.prototype.hide_messages = function()
	{
		return false;
	};

	PersonalSettings.prototype.onMaximize = function(/*event*/)
	{
		PersonalSettings.uber.onMaximize.call(this);
		this.refreshLayout();
	};

	PersonalSettings.prototype.onRestore = function(/*event*/)
	{
		PersonalSettings.uber.onRestore.call(this);
		this.refreshLayout();
	};

	PersonalSettings.prototype.onResizeStop = function(/*event*/)
	{
		PersonalSettings.uber.onResizeStop.call(this);
		this.refreshLayout();
	};

	PersonalSettings.prototype.refreshLayout = function()
	{
		if (this.m_layout_hnd) {
			this.m_layout_hnd.refreshLayout();
		}
	};

	/* =====================================
		(FIN)
	=====================================*/

	$(document).ready(function() {
		window.hook_environment();
		inConcert.i18n.updateScreen();
		var settings = new PersonalSettings();
		window.onReady(settings);
		settings.setEvents();
		settings.loadingControls();
	});
}());
