/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
  strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 11 */

/* global $, inConcert, window */

/*
Handling of the overlaying popup with the labels section of each interaction
*/

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	"use strict";

	//	const HEIGHT_MAGIC_NUMBER = 10;
	const WIDTH_MAGIC_NUMBER = 4;
	const TOP_MAGIC_NUMBER   = 25;

	function PopupWindow( data )
	{
		this.opened = false;
		this.clickEventName = "click.popup_";
		this.ownerId = "";
		this.canReposition = false;
		this.canMove = false;
		this.closeOnFocusLost = true;
		this.offset = null;
		this.owner_object = undefined;
		this.pubsub_on_scroll_handle = null;
		this.overlay = false;
		this.onNotAutomaticClose = $.noop;
		this.m_inlineStyleValues = {
			["z-index"]: 1029
		};
		this.setData( data );
	}

	if (typeof(inConcert.PopupWindow) !== "function") {
		inConcert.PopupWindow = PopupWindow;
	}

	PopupWindow.prototype.ready = $.noop;

	PopupWindow.prototype.isVisible = function(){
		var isVisible = $("#" + this.ownerId).is(":visible");
		return isVisible;
	};

	PopupWindow.prototype.getOwnerObject = function() { return this.owner_object; };

	PopupWindow.prototype.getData = function() { return this.data; };

	PopupWindow.prototype.setData = function( data )
	{
		var self = this;
		this.offset = null;

		if ( !data ) {
			return;
		}
		this.setPopupInlineStyleValues(data);
		this.owner_object = data.owner_object;
		this.data = data;
		if (data.eventName !== undefined) {
			this.clickEventName = this.clickEventName + data.eventName;
		}
		if (data.ownerId !== undefined) {
			this.ownerId = data.ownerId;
		}
		if (data.canReposition !== undefined) {
			this.canReposition = data.canReposition;
		}
		if (data.canMove !== undefined) {
			this.canMove = data.canMove;
		}
		if (data.closeOnFocusLost !== undefined) {
			this.closeOnFocusLost = data.closeOnFocusLost;
		}
		if (data.overlay !== undefined) {
			this.overlay = data.overlay;
		}
		// set callbacks functions (onOpen, onClose,..)
		this.onOpen = function() {
			$("#" + self.ownerId).show();
		};
		this.onClose = function() {
			self.hide();
		};
		var callbacks = data.callbacks;
		if ( callbacks ) {
			if (typeof(callbacks.onOpen) === "function") {
				this.onOpen = callbacks.onOpen;
			}
			if (typeof(callbacks.onClose) === "function") {
				this.onClose = callbacks.onClose;
			}
			if (typeof(callbacks.onNotAutomaticClose) === "function") {
				this.onNotAutomaticClose = callbacks.onNotAutomaticClose;
			}
		}
	};

	PopupWindow.prototype.setPopupInlineStyleValues = function( {inlineStyleValues} )
	{
		if (!inlineStyleValues) {
			return;
		}
		for (let name in inlineStyleValues) {
			if (inlineStyleValues.hasOwnProperty(name)) {
				this.m_inlineStyleValues[name] = inlineStyleValues[name];
			}
		}
	};

	//------------
	// metodos y variables que sirven para el movimiento de la ventana
	var theMovement;
	var position;
	var browser;
	var cursorStartX;
	var cursorStartY;
	var startX;
	var startY;

	var avoidEvents = function(event) {
		// Funcion que evita que se ejecuten eventos adicionales
		if (browser === 0) {
			window.event.cancelBubble = true;
			window.event.returnValue = false;
		}
		if (browser === 1) {
			event.preventDefault();
		}
	};

	var onMovement = function(event) {  
		var xCurrent, yCurrent;
		if (browser === 0) {    
			xCurrent = window.event.clientX + document.documentElement.scrollLeft + document.body.scrollLeft;
			yCurrent = window.event.clientY + document.documentElement.scrollTop + document.body.scrollTop;
		}  
		if (browser === 1) {
			xCurrent=event.clientX + window.scrollX;
			yCurrent=event.clientY + window.scrollY;
		}
		theMovement.style.left = (startX + xCurrent - cursorStartX) + "px";
		theMovement.style.top = (startY + yCurrent - cursorStartY) + "px";
		avoidEvents(event);
	};

	var endMovement = function() {
		if (browser === 0) {
			document.detachEvent("onmousemove", onMovement);
			document.detachEvent("onmouseup", endMovement);
		}
		else if (browser === 1) {
			document.removeEventListener("mousemove", onMovement, true);
			document.removeEventListener("mouseup", endMovement, true); 
		}
	};

	PopupWindow.prototype.initializeMove = function() {
		position =0; theMovement = null;
		// IE
		if (navigator.userAgent.indexOf("MSIE") >= 0) {
			browser = 0;
		}
		// Otros
		else {
			browser = 1;
		}
	};

	PopupWindow.prototype.startMove = function(event, id)
	{
		theMovement = document.getElementById(id);
		/* Si el elemento que se le hizo click es texto (nodeType=3) se toma como target
		el elemento padre */
		if (theMovement.nodeType === 3) {
			theMovement = theMovement.parentNode;
		}
		// Obtengo la position del cursor
		if (browser === 0) {
			cursorStartX = window.event.clientX + document.documentElement.scrollLeft + document.body.scrollLeft;
			cursorStartY = window.event.clientY + document.documentElement.scrollTop + document.body.scrollTop;
			document.attachEvent("onmousemove", onMovement);
			document.attachEvent("onmouseup", endMovement);
		}
		if (browser === 1) {    
			cursorStartX = event.clientX + window.scrollX;
			cursorStartY = event.clientY + window.scrollY;
			document.addEventListener("mousemove", onMovement, true); 
			document.addEventListener("mouseup", endMovement, true);
		}
		startX = parseInt(theMovement.style.left, 10);
		startY = parseInt(theMovement.style.top, 10);
		// Currentizo el position del elemento
		theMovement.style.zIndex =++ position;
		avoidEvents(event);
	};
	//--fin metodos que implementan el movimiento de la ventana.-----------------------------

	PopupWindow.prototype.setClosedAutomatic = function(close) {
		this.closeOnFocusLost = close;
	};

	/* --------------------------------------------------------------------------------------------------------------
		evento onclick en el body, sirve para saber en que elemento se ha dado click (dentro o fuera del elemento html contenedor del popup)
			1 - Si se ha dado click en el elemento buscado o en un elemento padre que descendiente del id buscado, no debe realizar ninguna accion y salir de la funcion.
			2 - Si se ha dado click fuera no va a encontrar el id buscado y se ha de cerrar el popup.

		PARA REALIZAR ESTO, VA BUSCANDO EN LOS ID DE LOS ELEMENTOS, DESDE EL ELEMENTO QUE SE HA DADO
		CLICK Y SIGUIENDO POR SU PADRE, Y SOBRE EL PADRE DEL PADRE, Y ASI SUSCESIVAMENTE HASTA REALIZAR
		LA VALIDACION.
	*/
	PopupWindow.prototype.closeEvent = function(e)
	{
		if (!this.closeOnFocusLost) {
			this.onNotAutomaticClose();
			return;
		}
		var target = e.target || e.srcElement;
		if (!target) {
			return;
		}
		var elementIdToFind = this.ownerId;
		do {
			if (target.id === elementIdToFind) {
			// El click se ha producido dentro del elemento, no se hace nada.
				return;
			}
			target = target.parentNode;
		} while (target);
		// Se ha cliqueado fuera del elemento, se cierra el popup.
		if (this.close) {
			//el evento "close" se llama si se ha implementado en la clase hija.
			this.close();
		}
		else {
			//el evento "finalize" es un metodo de esta clase que se llama si en la clase hija no se implemento el metodo "close".
			const params = undefined;
			const is_auto_close = true;
			this.finalize(params, is_auto_close);
		}
	};

	PopupWindow.prototype.setPosition = function(positions_options = {})
	{
		if (!this.data) {
			return;
		}

		const $parentNode = this.data.parentNode;
		const $container = this.data.container;
		if (!$parentNode || !$container) {
			return;
		}

		let top_position_config = positions_options.top_position_config || {};

		this.offset = this.offset || $parentNode.offset();
		var popup_width = $container.width() + WIDTH_MAGIC_NUMBER;
		var popup_left_position = this.offset.left;
		var window_width = window.innerWidth;

		var MIN_LEFT_POSITION = 7;
		var calculate_recursive_left_position = function(left_position)
		{
			if (left_position <= MIN_LEFT_POSITION) {
				left_position = left_position + WIDTH_MAGIC_NUMBER;
				return calculate_recursive_left_position(left_position);
			}
			return left_position;
		};

		if ( window_width <= ( popup_left_position + popup_width ) ) {
			// Si el ancho del popup + posicion es mayor que la posicion de la ventana, ajusto la posicion
			//
			//  El tamano del owner es utilizado para que la parte derecha del popup, quede alineado con la parte derecha del Owner.
			//
			//  Si al realizar lo anterior, la parte izquierda del popup es menor a la posicion minima izquierda (MIN_LEFT_POSITION),
			//   Calculo en una funcion recurisva que la posicion left del popup supere la posicion minima izquierda.
			var owner_width = $parentNode.width() + WIDTH_MAGIC_NUMBER;
			var suggested_popup_left_position = (this.offset.left - popup_width) + owner_width;
			popup_left_position = calculate_recursive_left_position(suggested_popup_left_position);
		}

		if (this.data.align === "LEFT") {
			popup_left_position = popup_left_position - popup_width;
			var new_popup_left_position = calculate_recursive_left_position(popup_left_position);
			popup_left_position = new_popup_left_position;
		}

		let my_top_magic_number = top_position_config.top_magic_number || TOP_MAGIC_NUMBER;

		var popup_height = $container.height();// + (HEIGHT_MAGIC_NUMBER + (top_position_config.height_magic_number || 0));
		var popup_top_position = this.offset.top + my_top_magic_number;
		var window_height = window.innerHeight;
		let popup_bottom_position = (popup_top_position + popup_height);

		if (window_height <= popup_bottom_position) {
			// Si la posicion bottom del popup queda fuera del alto de la ventana (es mayor), hay que ajustar la posicion del popup de la siguiente forma:
			//  (1) - Ubicando al popup arriba del boton (padre) que ha abierto dicho popup.
			//  (2) - Si al realizar el punto (1) la parte top del popup queda fuera de la ventana, es decir que es inferior a 0 (cero), comienzo a calcular 
			//        la posicion del popup en una funcion recursiva, que va decrementado a la posicion actual del popup N pixeles, hasta que el mismo no quede fuera
			//        de la ventana, ya sea TOP o BOTTOM. (he dicho!!)

			let iter = 0;
			let suggested_popup_top_position = this.offset.top - popup_height;
			var calculate_recursive_top_position = function()
			{
				if ( suggested_popup_top_position <= 0 ) {
					iter++;
					suggested_popup_top_position = popup_top_position - (my_top_magic_number * iter);
					return calculate_recursive_top_position();
				}
				return suggested_popup_top_position;
			};

			popup_top_position = calculate_recursive_top_position();

			// Con este objeto recibido por parametro, podremos ajustar de forma manual el popup.
			// Esto es en el caso que el calculo generico no sea exactamente correcto.
			// Quien debe proporcionar las coordenadas es quien ha instanciado la clase.
			// Ej: Client sidebar-> canned

			if (top_position_config.increment) {
				popup_top_position = popup_top_position + top_position_config.increment;
			}
			else if (top_position_config.decrement) {
				popup_top_position = popup_top_position - top_position_config.decrement;
			}
			popup_bottom_position = popup_height + popup_top_position;
		}

		// Ajuste si la posicion BOTTOM del Popup super al Alto de la ventana.
		if (window_height < popup_bottom_position) {
			let overflow = (popup_bottom_position - window_height) + my_top_magic_number;
			popup_top_position = popup_top_position - overflow;
		}

		// console.log("1 --> window_width: " + window_width + ", window_height: " + window_height);
		// console.log("2 --> popup: " + $container.attr("id") + ", popup_width: " + popup_width + ", popup_height: " + popup_height);
		// console.log("3 --> popup: " + $container.attr("id") + ", positions - left: " + popup_left_position + ", top: " + popup_top_position);

		const position_style = `left: ${popup_left_position}px; top: ${popup_top_position}px !important;`;
		$container.attr("style", position_style).show();
	};

	PopupWindow.prototype.onChangePopupHeight = function(positions_options = {})
	{
		if (!this.data) {
			return;
		}

		const $parentNode = this.data.parentNode;
		const $container = this.data.container;
		if (!$parentNode || !$container) {
			return;
		}

		let top_position_config = positions_options.top_position_config || {};
		this.offset = this.offset || $parentNode.offset();
		let my_top_magic_number = top_position_config.top_magic_number || TOP_MAGIC_NUMBER;

		let popup_height = $container.height();// + (HEIGHT_MAGIC_NUMBER + (top_position_config.height_magic_number || 0));
		let popup_top_position = this.offset.top + my_top_magic_number;
		let window_height = window.innerHeight;
		let popup_bottom_position = (popup_top_position + popup_height);

		// Ajuste si la posicion BOTTOM del Popup super al Alto de la ventana.
		if (window_height < popup_bottom_position) {
			let overflow = (popup_bottom_position - window_height) + my_top_magic_number;
			popup_top_position = popup_top_position - overflow;
		}

		// console.log("1 --> window_height: " + window_height);
		// console.log("2 --> popup: " + $container.attr("id") + ", popup_height: " + popup_height);

		const position_style = `top: ${popup_top_position}px !important;`;
		$container.attr("style", position_style).show();
	};

	PopupWindow.prototype.applyZindex = function()
	{
		// apply z-index
		var $owner = $("#" + this.ownerId);
		if ($owner.length === 0) {
			return;
		}
		var style = $owner.attr("style");
		style = (style || "").toLowerCase().trim();
		if ( style.indexOf("z-index") === -1 ) {
			var separator = "";
			var length = style.length;
			if ( length > 0 ) {
				var last_char = style.substring(length-1, length);
				if ( last_char !== ";") {
					separator = ";";
				}
			}
			let zIndexValue = this.m_inlineStyleValues["z-index"];
			style = style + separator + `z-index: ${zIndexValue}!important`;
			$owner.attr("style", style);
		}
	};

	PopupWindow.prototype.show = function(data)
	{
		var self = this;

		this.setData(data);
		this.opened = true;
		var $owner = $("#" + this.ownerId);

		if ($owner.length > 0) {
			$("body").trigger(this.clickEventName);
		}
		if (this.overlay) {
			$("#overlay_" + this.ownerId).remove();
			if (!$(".inconcertPopupWindowOverlay:visible").length) { 
				$("body").append("<div id='overlay_" + this.ownerId + "' class='inconcertPopupWindowOverlay'></div>");
			}
		}
		if (typeof(this.initialize) === "function") {
			this.initialize(); // call derived class
		}
		this.onOpen();
		if (this.canReposition) {
			const positions_options = data && data.positions_options ? data.positions_options : undefined;
			if (positions_options && positions_options.show_in_timeout_ms) {
				// Hay veces que es necesario realizar el setPosition en un timeout, como por ejemplo (telefonia-tansfer-member en el popup de transferencias, cuando se utiliza en interaction_actions)
				window.setTimeout(function() {
					self.setPosition(positions_options);
				}, positions_options.show_in_timeout_ms);
			}
			else {
				this.setPosition(positions_options);
			}
		}
		this.applyZindex();
		if (this.canMove) {
			this.initializeMove();
			$owner.mousedown(function(e) {
				self.startMove(e, this.id);
			}).mouseover(function() {
				this.style.cursor="move";
			});
		}
		if (this.closeOnFocusLost) {
			window.setTimeout(function() {
				$("body")
					.off( self.clickEventName )
					.on( self.clickEventName, function(e) {
						self.closeEvent(e);
					});
				self.ready();
			}, 50);
		}
		else {
			this.ready();
		}

		this.eventsSuscribe();
	};

	PopupWindow.prototype.eventsSuscribe = function()
	{
		var self = this;
		this.pubsub_on_scroll_handle = inConcert.app.messagebus.subscribe(
			"/application/popup_window/on_scroll",
			function() {
				// Si se realiza scroll de algun DIV que publique el evento, el popup se debe cerrar.
				// Ej: el evento scroll del panel del supervisor "Supervisor.prototype.create"
				window.setTimeout(function() {
					self.hide();
				});
			}
		);
	};

	PopupWindow.prototype.eventsUnSuscribe = function()
	{
		if (this.pubsub_on_scroll_handle) {
			inConcert.app.messagebus.unsubscribe(this.pubsub_on_scroll_handle);
			this.pubsub_on_scroll_handle = null;
		}
	};

	PopupWindow.prototype.hide = function(callback = $.noop)
	{
		if (this.overlay) {
			$("#overlay_" + this.ownerId).remove();
		}
		const effect_timeout_ms = 10;
		this.opened = false;
		this.eventsUnSuscribe();
		$("#" + this.ownerId).hide(effect_timeout_ms,
			function() {
				window.setTimeout(function() {
					callback();
				});
			})
		; // se necesita un timeout minimo para que se ejecute el callback en orden
	};

	/*******************************************************************
	 * Metodo que cierra el dialogo con los labels
	 */
	PopupWindow.prototype.finalize = function(params, is_auto_close)
	{
		this.opened = false;
		$("body").unbind(this.clickEventName);
		if (typeof this.onClose === "function") {
			this.onClose(params, is_auto_close);
		}
	};
}());
