/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, esversion: 11 */

/* global $, inConcert */

// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	/**
		Class QueuedInteractionsPanel.
		Queued conversations panel subclass, here goes queued.
	*/

	'use strict';

	const _T = inConcert.i18n.getString;
	const format = inConcert.Utils.format;
	const DEFAULT_TOP_QUEUED = 100;
	const MAX_QUEUED_INTERACTIONS = 500; // idem al valor de la variable del controller "interactions" de la app "agent" en la funcion "SearchInteractionsHelper"

	function QueuedInteractionsPanel (id, parent_panel)
	{
		//the category should be defined before the base constructor in order to be used properly
		this.m_parent_panel = parent_panel;
		this.m_category = 'queued';
		this.m_width = 256;
		this.m_queuedList = [];
		this.m_filter = false;
		this.m_topQueued = DEFAULT_TOP_QUEUED;
		this.m_main = inConcert.app.getMainWindow();
		this.m_views = {
			QUEUED : 0
		};
		this.m_currentView = this.m_views.QUEUED; //one of queued
		this.m_has_bulk_actions = true;
		QueuedInteractionsPanel.uber.constructor.call(this, {id: id, name: "<%=_T('Queued Conversations')%>"});
		this.m_dockToLeft = false;
		this.updateBatchActions();
	}

	inConcert.Utils.inherit(QueuedInteractionsPanel, inConcert.InteractionPanel);

	//Define functions and objects in the namespace in case they don't already exist
	if (typeof inConcert.QueuedInteractionsPanel !== 'function') {
		inConcert.QueuedInteractionsPanel = QueuedInteractionsPanel;
	}

	QueuedInteractionsPanel.prototype.can_reposition = function() {
		return false;
	};

	QueuedInteractionsPanel.prototype.get_prefix = function( /*interaction*/ )
	{
		return 'queued_interaction_';
	};

	QueuedInteractionsPanel.prototype.allowed_bulk_actions = function( /*interaction*/ )
	{
		const current_app = inConcert.app.getCurrentApp();
		if(current_app === "supervisor")
		{
			return false;
		}
		return QueuedInteractionsPanel.uber.allowed_bulk_actions.call( this);
	};

	QueuedInteractionsPanel.prototype.show = function( callback ) {
		this.refresh_queued();
		$('#panelQueuedInteractions').show();
		this.on_show(callback);
	};

	QueuedInteractionsPanel.prototype.hide = function( callback ) {
		$('#panelQueuedInteractions').hide();
		this.on_hide(callback);
	};

	QueuedInteractionsPanel.prototype.toggleSearchPanel = function()
	{
		$("#interactionQueryBuilderContainer").toggle();
	};

	QueuedInteractionsPanel.prototype.set_events = function()
	{
		$('#btnSearchOnQueuedSimpleList').unbind('click').click(this.toggleSearchPanel);
		$('#int_search_queuedPillContainer').find(".inlineMessagesContainer").unbind('click').click(this.toggleSearchPanel);
		QueuedInteractionsPanel.uber.set_events.call( this );
	};

	QueuedInteractionsPanel.prototype.get_query_search = function()
	{
		var result = inConcert.app.getMainWindow().m_panels.conversations_panel.queryBuilderObj.getFilter();

		if(result) {
			//labels
			if (this.m_parent_panel) {
				var selected_labels = this.m_parent_panel.get_selected_labels(true /*simple item*/);
				if ( $.isArray(selected_labels) && selected_labels.length > 0 ) {
					var labelsRule = {
						id : "labels",
						operator : "in",
						value : selected_labels
					};
					result.rules.push(labelsRule);
				}
			}
		}

		return result;
	};

	QueuedInteractionsPanel.prototype.searchQueued = function(callback)
	{
		var self = this;
		callback = callback || $.noop;

		if (this.searchingOnQueued) {
			return;
		}
		var query = this.get_query_search();
		if(!query){
			return;
		}

		this.searchingOnQueued = true;
		this.m_filter = false;

		var $waiting = $('#queuedPanelSearchWaiting');
		var $message = $('#interaction_queued_search_not_found_msg');

		var filter = { query : query };

		$waiting.show();
		$message.hide();

		//Pills
		var $pillContainer = $('#int_search_queued').find("#int_search_queuedPillContainer");
		var $noPill = $pillContainer.find(".inlineMessagesContainer");
		var $pillGroup = $pillContainer.find(".form-group-pill-container");
		$pillGroup.empty();
		if(query && query.rules && $.isArray(query.rules) && query.rules.length > 0)
		{
			for (var i = 0; i < query.rules.length; i++) {
				// #12450: Dont add labels to Pills
				if(query.rules[i].id === "labels") {
					continue;
				}				
				var ruleId = "queued_pill_" + i;
				var rulePillText = _T(query.rules[i].label || query.rules[i].id) + " | " + _T(query.rules[i].operator) + " | " + "<strong>" + _T(query.rules[i].value) + "</strong>";
				const $lbl = "" +
					"<label id='" + ruleId + "_label' class='form-group-pill'>" +
						"<span>" + rulePillText + "</span>" +
						"<button id='" + ruleId + "_remove' class='pillButtonDelete'></button>" +
					"</label>";
				$pillGroup.append($lbl);
				$pillGroup.show();
				$noPill.hide();

				$("#" + ruleId + "_remove").click(function() {
					var ruleIdx = parseInt(this.id.replace("queued_pill_", ""));
					//Eliminar del queryBuilder
					if(inConcert.app.getMainWindow().m_panels.conversations_panel.queryBuilderObj.deleteRule(ruleIdx))
					{
						//Eliminar del contenedor de pills
						$(this).parent().remove();
					}
				});
			}
		}
		else
		{
			$pillGroup.hide();
			$noPill.show();
		}		

		var onComplete = function(success) {
			self.searchingOnQueued = false;
			self.updateBatchActions();
			$waiting.hide();
			callback(success);
		};
		var addInteraction = function(interaction) {
			if (!interaction) {
				return;
			}
			var parmams_interaction = {
				prefix: self.get_prefix( interaction ),
				allowed_bulk_actions: false
			};
			interaction.expanded = true;
			var template_tpl = null;
			var $node = self.$build_interaction_node(interaction, template_tpl, parmams_interaction);
			self.updateInteractionData(interaction, $node);
			self.m_interactions[interaction.id] = $node;
			self.onInteractionAdded(interaction, $node);
		};

		inConcert.app.getInteractionManager().searchQueuedInteractions(filter, function(success, result)
		{
			if (success) {
				self.clearListContainer();
				self.m_queuedList = [];
				var length = result.interactions.length;
				if (length === 0) {
					$message.show();
					self.m_filter = filter;
					onComplete(success);
				}
				else {
					self.m_topQueued = (MAX_QUEUED_INTERACTIONS * 2); // TODO: cuando se realice la parte del paginado de queueds, esta estrategia debe volar.
					var loopTop = DEFAULT_TOP_QUEUED + 10; // tiene que ser mayor al default
					var added = 0;
					var add = function(start, end) {
						var i;
						for (i = start; i < end; i++) {
							var interaction = result.interactions[i];
							addInteraction(interaction);
						}
						added = i;
						if (added < length) {
							window.setTimeout(function() {
								start = i + 1;
								end =  Math.min(length, (end + loopTop));
								add(start, end);
							}, 100);
						}
						else {
							self.m_filter = filter;
							onComplete(success);
						}
					};
					var start = 0;
					var end = Math.min(length, loopTop);
					add(start, end);
				}
			}
			else {
				self.m_main.showBannerMessage({
					message:     "<%=_T('Error server finding queued conversations.')%>",
					messageType: 'Error',
					timeout:      5000
				});
				onComplete(success);
			}
		});
	};

	QueuedInteractionsPanel.prototype.has_applied_filter = function()
	{
		var result = this.get_query_search();
		return result ? result.rules.length > 0 : false;
	};

	QueuedInteractionsPanel.prototype.refresh_queued = function()
	{
		var self = this;
		var $not_found_msg = $('#interaction_queued_search_not_found_msg');
		var $waiting = $('#queuedPanelSearchWaiting');

		var callback = function() {
			$waiting.hide();
			$not_found_msg[self.m_queuedList.length === 0 ? 'show' : 'hide']();

			$("#interactionQueryBuilder-btn-clean-all").prop("disabled", !inConcert.app.getMainWindow().m_panels.conversations_panel.queryBuilderObj.hasRules());

			if (self.selectedInteractionBeforeClean && self.selectedInteractionBeforeClean.length > 0 && self.m_queuedList.length > 0) {
				//Vuelvo a seleccionar las interacciones
				let selectedInteractions = inConcert.Utils.arrayToMap(self.selectedInteractionBeforeClean,
					(int) => int,
					(int) => int.id,
				);
				$.each(self.m_queuedList, function (_, item_value) {
					if (selectedInteractions[item_value.id]) {
						const $node = self.$findInteraction(item_value);
						if ($node) {
							$node.find('[int_batch_selector="true"]').prop("checked", true);
						}
					}
				});
				if (self.pendingInteractionBeforeClean.length > 0) {
					self.mark_interactions_as_pending(self.pendingInteractionBeforeClean, true);
				}
				delete self.selectedInteractionBeforeClean;
				delete self.pendingInteractionBeforeClean;
			}
			self.updateBatchActions();
		};
		//Obtengo las interacciones seleccionadas y pending antes de limpiar.
		this.selectedInteractionBeforeClean = this.getSelectedInteractions().interactions;
		this.pendingInteractionBeforeClean = this.selectedInteractionBeforeClean.filter((int)=>int.pending);
		var has_applied_filter = this.has_applied_filter();
		if ( has_applied_filter ) {
			// si tiene filtros aplicados desde el txt de busqueda o en la navegacion, refresca contra la URL de busqueda que va contra ES
			this.searchQueued(callback);
		}
		else {
			// si NO tiene filtros aplicados desde el txt de busqueda o labels seleccionados en la navegacion, refresca contra la URL que obtiene todas las encoladas
			this.m_filter = false;
			this.m_queuedList = [];
			this.m_interactions = {};
			this.m_topQueued = DEFAULT_TOP_QUEUED;
			this.clearListContainer();
			
			//Pills
			var $pillContainer = $('#int_search_queued').find("#int_search_queuedPillContainer");
			var $noPill = $pillContainer.find(".inlineMessagesContainer");			
			var $pillGroup = $pillContainer.find(".form-group-pill-container");
			$pillGroup.empty();	
			$pillGroup.hide();
			$noPill.show();	

			$waiting.show();
			$not_found_msg.hide();
			inConcert.app.getInteractionManager().refreshQueued(callback);
		}
	};

	QueuedInteractionsPanel.prototype.apply_filter = function()
	{
		this.refresh_queued();
	};

	QueuedInteractionsPanel.prototype.reset = function() {
		this.m_windows = {};
		this.m_queuedList = [];
		this.m_filter = false;
		this.clearListContainer();
		this.loadAllInteractions();
	};

	QueuedInteractionsPanel.prototype.refreshPanel = function()
	{
		this.refresh();
		this.updateTakeButtonsStatus();
	};

	QueuedInteractionsPanel.prototype.interactionPassesFilter = function( interaction )
	{
		var has_applied_filter = this.has_applied_filter();
		if ( !has_applied_filter ) {
			return true;
		}
		var query = (this.m_filter || {}).query;

		var validate_value_in_list = function( val, list, get_value )
		{
			var can_add_from_list = false;
			val = val.toLowerCase();
			$.each(list, function( _, item_value )
			{
				if ( val === get_value(item_value) ) {
					// si lo agrega
					can_add_from_list = true;
					return false;
				}
			});
			return can_add_from_list;
		};

		// 1 - el filtro
		if ( typeof(query) === "object" )
		{
			var can_add = false;

			$.each(query, function(name, value)
			{
				if ( name === "state" ) {
					if ( value === "QUEUED" ) {
						can_add = true;
					}
					else {
						// // no lo agrega
						can_add = false;
						return false; // break
					}
				}
				else if ( name === "type" ) {
					var int_type = interaction.type.toLowerCase();
					can_add = validate_value_in_list( int_type, value, function( item_value ) {
						return item_value.toLowerCase();
					});
				}
				else if ( name === "campaign" ) {
					var domain = "@" + inConcert.app.getDomain();
					var int_camp = interaction.campaign.toLowerCase();
					can_add = validate_value_in_list( int_camp, value, function( item_value ) {
						return ( item_value + domain ).toLowerCase();
					});
				}
				else if ( name === "label" ) {
					var filter_labels = value;
					var count_filter_labels = filter_labels.length;
					if ( count_filter_labels === 0 ) {
						can_add = true;
					}
					else {
						var interaction_labels = interaction.getLabels() || {};
						var count_interaction_labels =  Object.keys( interaction_labels ).length;
						if ( count_filter_labels > count_interaction_labels ) {
							// no lo agrega
							can_add = false;
							return false; // break
						}
						for (var i = 0; i < count_filter_labels; i++) {
							var filtered_label = filter_labels[i].toUpperCase();
							var has_label = interaction_labels.hasOwnProperty(filtered_label);
							if ( has_label ) {
								can_add = true;
							}
							else {
								// no lo agrega
								can_add = false;
								return false; // break
							}
						}
					}
				}

				if ( !can_add ) {
					return false; // break
				}
			});

			if ( !can_add ) {
				// no lo agrega
				return false;
			}
		}
		// is ok: lo agrega
		return true;
	};

	QueuedInteractionsPanel.prototype.interactionBelongsToPanel = function(interaction)
	{
		return interaction.isQueued();
	};

	//triggered when an conversation is removed from the panel, in this case cleanup the client window associated
	QueuedInteractionsPanel.prototype.onInteractionRemoved = function( interaction )
	{
		QueuedInteractionsPanel.uber.onInteractionRemoved.call( this, interaction );

		var queuedList = this.m_queuedList;
		var TOP_QUEUED = this.m_topQueued;
		var TOP_SHOWN = TOP_QUEUED / 2;
		var index;

		for (var i = 0; i < queuedList.length; i++) {
			if (queuedList[i].id === interaction.id) {
				queuedList.splice(i, 1);
				index = i;
				break;
			}
		}
		if (typeof(index) === 'undefined') {
			return;
		}

		//if it's on screen remove it
		var $interaction = this.$findInteraction( interaction );
		if ( $interaction && $interaction.length > 0 ) {
			$interaction.remove();
		}
		//If we are on simple list, if shifted and have more buffered, bring one more to the screen
		if (index < TOP_SHOWN && queuedList.length >= TOP_SHOWN) {
			var new_interaction = queuedList[TOP_SHOWN - 1];
			if (new_interaction) {
				var node = this.m_interactions[new_interaction.id];
				if (node) {
					this.m_listContainer.append(node);
					if (this.onNodeInserted) {
						this.onNodeInserted(node, new_interaction);
					}
				}
				else {
					inConcert.logger.warn({
						action_name:      'QueuedInteractionsPanel.onInteractionRemoved',
						action_type:      'interaction',
						interaction_id:   new_interaction.id,
						interaction_type: new_interaction.type,
						message:          'Error adding queued conversation. The DOM node is undefined.'
					});
				}
			}
			else {
				inConcert.logger.warn({
					action_name:      'QueuedInteractionsPanel.onInteractionRemoved',
					action_type:      'interaction',
					interaction_id:   interaction.id,
					interaction_type: interaction.type,
					message:          'Error removing conversation. The conversation is undefined.'
				});
			}
		}

		var has_applied_filter = this.has_applied_filter();
		if ( has_applied_filter ) {
			this.refresh_queued();
		}
		else {
			////if under threshold, trigger refresh of queued conversations
			if (queuedList.length <= TOP_SHOWN && !this.m_filter) {
				this.refresh_queued();
			}
		}
	};

	QueuedInteractionsPanel.prototype.show_hide_quick_take_next_button = function()
	{
		var $cmd = $('#butTakeNextQueued');
		var is_enabled_in_queued = inConcert.permissions.to_action_on_any_campaign( 'interaction', 'int_queued_take', undefined /* action_type opcional */, undefined /*  campaign_id opcional */, true /*no_log  opcional */ );
		var is_enabled_in_queued_panel = inConcert.permissions.to_action_on_any_campaign( 'interaction', 'int_view_take' , undefined /* action_type opcional */, undefined /*  campaign_id opcional */, true /*no_log  opcional */ );
		var can_show = is_enabled_in_queued && is_enabled_in_queued_panel;
		if ( can_show ) {
			$cmd.show();
		}
		else {
			$cmd.hide();
		}
	};

	QueuedInteractionsPanel.prototype.enableTakeNextButton = function()
	{
		var self = this;

		$('#butTakeNextQueued')
			.prop('disabled', false)
			.unbind('click')
			.bind('click', function()
			{
				var is_enabled_in_queued_panel = inConcert.permissions.to_action_on_any_campaign('interaction', 'int_view_take' , undefined /*action_type opcional*/, undefined /*campaign_id opcional*/, true /*no_log  opcional */ );
				if (!is_enabled_in_queued_panel) {
					inConcert.app.m_mainWindow.showBannerMessage({ message: "<%=_T('You not has permission for this action.')%>", messageType: "Error", timeout: 5000 });
					return;
				}
				inConcert.app.getInteractionManager()
					.takeNextBest(null, null, function(success, interactionId)
					{
						if (success) {
							return;
						}
						var message_error = interactionId;
						self.showMessage(message_error, 'Error', 5000);
					});
			});

		this.show_hide_quick_take_next_button();
	};

	QueuedInteractionsPanel.prototype.disableTakeNextButton = function() {
		$('#butTakeNextQueued').prop('disabled', true).unbind('click');
		this.show_hide_quick_take_next_button();
	};

	QueuedInteractionsPanel.prototype.updateTakeButtonsStatus = function()
	{
		var count_queued_interactions = this.count_queued_interactions();
		//If there is no conversations to take, disable take next buttons
		if (count_queued_interactions === 0 ) {
			this.disableTakeNextButton();
		}
		else{
			this.enableTakeNextButton();
		}
	};

	QueuedInteractionsPanel.prototype.count_queued_interactions = function() {
		return Object.keys(this.m_interactions || {}).length;
	};

	QueuedInteractionsPanel.prototype.onInteractionAdded = function(interaction, $new_interaction)
	{
		var self = this;

		//cancel the insert/update if the $new_interaction id already exists on the queued list
		var $interaction = this.$findInteraction( interaction );
		if ( $interaction.length > 0 ) {
			return;
		}

		if ( !this.interactionPassesFilter(interaction) ) {
			inConcert.app.getInteractionManager().discard(interaction);
			this.removeInteraction(interaction);
			return;
		}

		var index;
		var TOP_QUEUED = this.m_topQueued;
		var TOP_SHOWN = TOP_QUEUED / 2;
		var queuedList = this.m_queuedList;

		//this is not an efficient insertion order but it's easy and works
		for (var i = 0; i < queuedList.length; i++) {
			var queuedInteraction = queuedList[i];
			if (this.sortingFunction(queuedInteraction, interaction)) {
				queuedList.splice(i, 0, interaction);
				index = i;
				break;
			}
		}

		//if didn't insert and have room left, push at the end
		if (typeof(index) === 'undefined') {
			if (queuedList.length < TOP_QUEUED) {
				queuedList.push(interaction);
				index = queuedList.length - 1;
			//discard, it's at 41st place
			} else {
				inConcert.app.getInteractionManager().discard(interaction);
				this.removeInteraction(interaction);
				//nothing else to do, return
				return;
			}
		}
		else {
			//if the new element was inserted and I'm over the top, discard the last one
			var queued_length = queuedList.length;
			if (queued_length > TOP_QUEUED)
			{
				var queued_interaction = queuedList[queued_length - 1];
				inConcert.app.getInteractionManager().discard(queued_interaction);
				this.removeInteraction(queued_interaction);
			}
		}

		//show it on screen if under threshold
		if (this.m_currentView === this.m_views.QUEUED ) {
			if (index < TOP_SHOWN) {
				var children = this.m_listContainer.children();

				var insert_in_list_fn = function( $new_node )
				{
					if ( index === children.length || children.length === 0 ) {
						self.m_listContainer.append( $new_node );
					}
					else {
						var $current_node = $(children[index]);
						var current_id = $current_node.attr('id');
						var $child = $new_node.find( '#' + current_id );
						if ( $child.length > 0 ) {
							self.m_listContainer.append( $new_node );
						}
						else {
							$current_node.before( $new_node );
						}
					}
				};

				// inserts the conversation or group conversations in the list panel
				this.insertSortedInteractionInPanelList(
					interaction,
					$new_interaction,
					insert_in_list_fn
				);

				this.m_interaction_gui.toggleInteractionLabels($new_interaction.find('[int_labels_disposition="true"]'));

				if (this.onNodeInserted) {
					this.onNodeInserted( $new_interaction, interaction );
				}
				//remove the last if queue was full
				if (children.length >= TOP_SHOWN) {
					$(children[children.length - 1]).remove();
				}
			}
		}

	};

	QueuedInteractionsPanel.prototype.sortingFunction = function(a, b) {
		//live conversations get priority over offline ones
		if (a.isLive() && !b.isLive()) {
			return false;
		}
		if (!a.isLive() && b.isLive()) {
			return true;
		}
		//own conversations get priority
		if (a.belongsToMe() && !b.belongsToMe()) {
			return false;
		}
		if (!a.belongsToMe() && b.belongsToMe()) {
			return true;
		}
		//check priority first
		if (a.priority === b.priority) {
			return a.lastModified() > b.lastModified();
		}
		return a.priority < b.priority;
	};

	//message: message to show             [mandatory]
	//type: type of the message (the icon) [optional, default = 'Info'] 'Info' | 'Alert' | 'Error'
	//timeout: time to fade out            [optional, default = 5000]
	QueuedInteractionsPanel.prototype.showMessage = function(message, type, timeout)
	{
		var self = this;
		if (!message || (message === '')) {
			//no message to show
			return;
		}
		var allowed_types = {
			'Info' : true, 
			'Alert' : true, 
			'Error' : true
		};

		if (!allowed_types[type]){
			//default type
			type = 'Info';
		}

		//If there is a previous message,
		//replace its text with the new message and restart timer
		//else create new div
		if ( this.lastTimer ) {
			clearTimeout(this.lastTimer);
			$('.queuedQuotaMessage').text(message);
		} else {
			var $parent = $('#panelQueuedInteractions .boxContentScrollArea');
			//no need to escape
			var markup = '<div class="boxContentListMessages queuedQuotaMessage queuedMessage'+ type +'">' + message + '</div>';
			$parent.prepend(markup); //as first child (on top)
		}
		this.lastTimer = setTimeout(function()
		{
			var $messageBox = $('.queuedQuotaMessage');
			$messageBox.fadeOut();
			$messageBox.remove();
			self.lastTimer = undefined;
		}, timeout || 5000);

	};

	QueuedInteractionsPanel.prototype.onNodeInserted = function($new_interaction, interaction)
	{
		if (!$new_interaction) {
			inConcert.logger.warn({
				action_name:      'QueuedInteractionsPanel.onNodeInserted',
				action_type:      'interaction',
				message:          'Error inserting node in the DOM. The node is undefined.'
			});
			return;
		}
		if (!interaction) {
			inConcert.logger.warn({
				action_name:      'QueuedInteractionsPanel.onNodeInserted',
				action_type:      'interaction',
				message:          'Error inserting node in the DOM. The conversation is undefined.'
			});
			return;
		}
		$new_interaction.data('interaction', interaction);
		$('#interaction_queued_search_not_found_msg').hide();
		const self = this;
		this.updateBatchButtonsStatus();
		$new_interaction.find('[int_batch_selector]').bind('click', function(e) {
			self.onNodeSelected(interaction, $new_interaction);
			// TODO: revisar tema de bulling. Se esta ejecutando y abriendo la ventana.
			e.stopPropagation();
			$("body").trigger('click');
		});
	};

	QueuedInteractionsPanel.prototype.clearListContainer = function() {
		this.m_listContainer.children().remove();
	};

	QueuedInteractionsPanel.prototype.cleanupCampaignInteractions = function(campaign) {
		for (var id in this.m_interactions) {
			if (this.m_interactions.hasOwnProperty(id)) {
				var node = this.m_interactions[id];
				var interaction = node.data('interaction');
				if (interaction && interaction.campaign === campaign.Id) {
					node.remove();
					this.removeInteraction(interaction);
				}
			}
		}
	};

	QueuedInteractionsPanel.prototype.updateAssignedCounter = function(counter) {
		if ( this.m_parent_panel ) {
			this.m_parent_panel.update_counters( {
				queued : { agent : counter }
			});
		}
	};

	QueuedInteractionsPanel.prototype.takeSelectedNodes = function(onNodesTaked)
	{
		var self = this;
		var interactions = this.getSelectedInteractions().interactions;
		var interactionCallbacks = {
			onFinished : function(result){
				result = self.__extractBatchMessageInformation(result, 'taken');
				self.m_main.showBannerMessage({
					message:     result.message      ,
					messageType: result.message_type ,
					timeout:     7000
				});
			},
			onPartial : function(successfulInteractions, unsuccessfulInteractions){
				if (successfulInteractions && successfulInteractions.length > 0){
					successfulInteractions.forEach(function(interaction){
						var $node = self.__getInboxInteractionNode(interaction.id);
						if ( $node && $node.length > 0 ) {
							self.updateInteractionNode($node);
						}
					});
				}
				if (unsuccessfulInteractions && unsuccessfulInteractions.length > 0){
					var errorMessage = format("<%=_T('{0} conversations failed to taked')%>", unsuccessfulInteractions.length);
					self.m_main.showBannerMessage({	message : errorMessage, messageType : 'Error', timeout : 5000});
				}
			},
			onFailure: function(errorMessage){
				self.m_main.showBannerMessage({	message : errorMessage, messageType : 'Error', timeout : 5000});
			}
		};

		var args = [];
		if ( interactions.length > 0 ){
			var interactionBatchFunction = inConcert.app.getInteractionManager().batchTake;
			var interactionBatchData = new self.BatchData(interactionBatchFunction, interactions, args, interactionCallbacks);
			self.processBatchInteractions(interactionBatchData, function(){
				if (typeof(onNodesTaked) === 'function'){
					onNodesTaked.apply(self);
				}
			});
		}
	};

	QueuedInteractionsPanel.prototype.$get_panel_container_buttons = function()
	{
		return $("#panelQueuedContainerButtons");
	};

	QueuedInteractionsPanel.prototype.get_batch_actions_prefix = function()
	{
		return 'interaction_queued_';
	};
	
	QueuedInteractionsPanel.prototype.$getChkSelectAllConversations = function()
	{
		return $("#int_queued_grid_header").find("[selector='select-all-conversations']");
	};

	QueuedInteractionsPanel.prototype.getBatchButtons = function()
	{
		return [
			{ actionType : 'TAKE'    , selector : '#batchActionTake' },
			{ actionType : 'FINISH'  , selector : '#batchActionFinish'   },
			{ actionType : 'TRANSFER', selector : '#batchActionTransfer' },
			{ actionType : 'LABEL'   , selector : '#batchActionLabel'    }
		];
	};

}());
