/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:single, esversion: 6 */

/* global $, inConcert */

// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	/**
		Class SharedPanel.
		Panel subclass.
	*/
	'use strict';

	function SharedPanel (id)
	{
		this.permissions_tabs = {
			'campaigns': {
				permission_id   : 'int_view_campaigns',
				id_tab_dom      : 'int_panel_shared_tab_campaigns',
				id_container_dom: 'int_panel_campaigns',
				selected_weight : 1
			},
			'contacts': {
				permission_id   : 'int_view_contact_panel',
				id_tab_dom      : 'int_panel_shared_tab_contacts',
				id_container_dom: 'int_panel_contacts',                   // el panel de contactos y tema utilizan el mismo contenedor
				selected_weight : 2
			},
			'my_team': {
				permission_id   : 'int_view_my_teams',
				id_tab_dom      : 'int_panel_shared_tab_team',
				id_container_dom: 'int_panel_contacts',
				selected_weight : 3
			}
		};
		SharedPanel.uber.constructor.call(this, id, '<%=_T("Shared")%>'); // call a constructor of base class
		this.m_width = 256;
		this.m_visible = true;
		this.m_current_tab = null;
		this.m_tabs = {
			CAMPAIGNS : new inConcert.CampaignPanel('int_panel_campaigns'),
			CONTACTS  : new inConcert.ContactPanel('int_panel_contacts')
		};
		this.m_main = inConcert.app.getMainWindow();
	}

	inConcert.Utils.inherit( SharedPanel, inConcert.Panel );

	if (typeof inConcert.SharedPanel !== 'function') {
		inConcert.SharedPanel = SharedPanel;
	}

	SharedPanel.prototype.get_tab = function(id) { return this.m_tabs[id]; };

	SharedPanel.prototype.set_events = function()
	{
		var self = this;
		SharedPanel.uber.set_events.call( this ); //invokes the base class method
		const $panel = self.__$get_panel();

		$('#int_panel_handler_button')
			.off('click')
			.on('click', function() {
				if ( $panel.hasClass('isPanelHide') ) {
					$panel.removeClass('isPanelHide');  
				} else {
					$panel.addClass('isPanelHide');  
				}
    	});

		$('#int_panel_shared_tab_campaigns')
			.off('click')
			.on('click', function() {
				if ( self.switch_tab( 'campaigns' ) ) {
					self.m_tabs.CAMPAIGNS.select_tab();
				}
				if ( $panel.hasClass('isPanelHide') ) {
					$panel.removeClass('isPanelHide');  
				}
		});

		$('#int_panel_shared_tab_contacts')
			.off('click')
			.on('click', function() {
				const $int_panel_contacts = $('#int_panel_contacts_title');
				$int_panel_contacts.text('<%=_T("Contacts")%>');
				if ( self.switch_tab( 'contacts' ) ) {
					self.m_tabs.CONTACTS.select_tab('contacts');
				}
				if ( $panel.hasClass('isPanelHide') ) {
					$panel.removeClass('isPanelHide');  
				}
		});

		$('#int_panel_shared_tab_team')
			.off('click')
			.on('click', function() {
				const $int_panel_contacts = $('#int_panel_contacts_title');
				$int_panel_contacts.text('<%=_T("My Team")%>');
				if ( self.switch_tab( 'my_team' ) ) {
					self.m_tabs.CONTACTS.select_tab('my_team');
				}
				if ( $panel.hasClass('isPanelHide') ) {
					$panel.removeClass('isPanelHide');  
				}
		});
	};

	SharedPanel.prototype.switch_tab = function( select_tab_id )
	{
		var self = this;
		if ( this.m_current_tab === select_tab_id) {
			return false;
		}
		if ( !this.permissions_tabs[select_tab_id] ) {
			throw('SharedPanel.prototype.switch_tab ERROR. Unrecognized tab id: "' + select_tab_id + '"');
		}
		$.each( this.permissions_tabs, function( tab_id, info )
		{
			var $tab = $('#' + info.id_tab_dom);
			if ( $tab.length === 0 ) {
				return true; // continue
			}
			if ( tab_id === select_tab_id ) {
				self.m_current_tab = select_tab_id;
				$tab.removeClass('boxTabItem').addClass('boxTabSelected');
			}
			else {
				$tab.removeClass('boxTabSelected').addClass('boxTabItem');
			}
		});

		return true;
	};

	SharedPanel.prototype.__$get_panel = function()
	{
		return $('#int_shared_panel');
	};

	SharedPanel.prototype.destroyDOMPanel = function()
	{
		const $panel = this.__$get_panel();
		$panel.remove();
	};

	SharedPanel.prototype.show = function()
	{
		const $panel = this.__$get_panel();
		if ($panel.length > 0) {
			$panel.show();
		}
	};

	SharedPanel.prototype.hide = function()
	{
		const $panel = this.__$get_panel();
		if ($panel.length > 0) {
			$panel.hide();
		}
	};
}());
