/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
  strict:false, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark: false,
  smarttabs:true, esversion: 11 */

/* global inConcert, _ , EmojiPicker, _*/


/*
	Handling GUI events to interact with sms client session
*/

// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

///////////////////Class Definition

	var m_maxCharsPerMessage = 160;
	var FORMAT = inConcert.Utils.format;

	var SCROLL_TO = inConcert.Utils.scroll_to;
	var TIME_SELECTED_COMMENT_MS = 5000;

	var LNK_THREAD_MARK_AS_READ_CLASS   = "threadActionsMarkAsRead";
	var LNK_THREAD_MARK_AS_UNREAD_CLASS = "threadActionsMarkAsUnread";
	var LNK_ORIGIN_MARK_AS_READ_CLASS   = "originActionsMarkAsRead";
	var LNK_ORIGIN_MARK_AS_UNREAD_CLASS = "originActionsMarkAsUnread";

	/**
	Class SmsClientWindow.
	Sms Client Handler
	@param interaction
	@param options
	*/
	function SmsClientWindow (interaction, options) {
		this.m_interaction = interaction;
		this.setHorizontalResizingLimits(480);

		var success = SmsClientWindow.uber.constructor.call( this, options );
		if (!success) {
			return;
		}
		
		if (options.readonly || options.monitor) {
			this.toggleSendMessage(false);
		}
		else {
			this.bindSendMessage();
		}
		
		//load avatar for the contact
		var avatar_uri = inConcert.app.getBaseUrl() + interaction.getAvatarURI();
		$('#contactFromAvatar').attr("data", avatar_uri);
		
		//initiative class
		$("#mainContainer").addClass(interaction.initiative === "OUTBOUND" ? 
			"smsOutboundClientContainer" : "smsInboundClientContainer" );
	}

	inConcert.Utils.inherit( SmsClientWindow, inConcert.ClientWindow );
	
///////////////////Client Methods

	SmsClientWindow.prototype.initialize = function( options )
	{
		SmsClientWindow.uber.initialize.call( this, options );
		
		if (this.m_interaction.state === "PENDING" && this.m_interaction && this.m_interaction.contactAddress) {
			$("#setAddress").val(this.m_interaction.contactAddress);
		}
	};

	SmsClientWindow.prototype.getMenuContextualActions = function()
	{
		const actions = {
		// acciones solo para el cliente
			attachfile   : true,
			mark_all_read_unread: true,
		// acciones sobre la interaccion
			comment      : true,
			label        : true,
			emit_ticket  : true,
			search_ticket: true,
			queue        : true,
			transfer     : true
		// estas NO tienen que visualizarse en el menu, ya que las mismas se encuentran en la toolbar
		};
		return actions;
	};

	SmsClientWindow.prototype.getWindowDefaultTitle = function() {
		return "<%=_T('Sms Window')%>";
	};
	
	SmsClientWindow.prototype.setTitle = function( window_title )
	{
		if (window_title || !this.m_interaction || this.m_interaction.isBeingComposed()) {
			return SmsClientWindow.uber.setTitle.call( this, window_title );
		}
		var name = this.m_interaction.getClientName();
		var address = this.m_interaction.getHumanReadableAddress();
		const visualize_address = inConcert.app.canVisualizeAddress(null, this.m_interaction.campaign, null, true);
		if (name) {
			if (address && address !== name && visualize_address) {
				name += " (" + address + ")";
			}
		}
		else {
			name = !visualize_address? "<%=_T('Hidden Address')%>" : address;
		}
		return SmsClientWindow.uber.setTitle.call( this, name );
	};

	SmsClientWindow.prototype.getScrollBasicInfo = function() {
		return {
			$scroll : $("#thread_container"), // el contenedor de la lista con el scroll
			increase_position : -170 // este es un magic number que sirve unicamente en el caso en que "destination='item_position'",
									// decrementa un valor para poder ajustarlo a la pantalla correctamente.
		};
	};

	SmsClientWindow.prototype.getCaptionRelatedButton = function()
	{
		return FORMAT( "<%=_T('{0} Unread')%>", this.m_interaction.getUnreadEventCount() );
	};

	SmsClientWindow.prototype.afterRenderRelatedMarkup = function() {
		$("#unreadTitle").html(`<%=_T('Unread messages')%> <span>(${this.m_interaction.getUnreadEventCount()})</span>`);
	};

	SmsClientWindow.prototype.getRelatedEvents = function() {
		return this.m_interaction.getUnreadEvents();
	};

	SmsClientWindow.prototype.canShowRelatedSections = function()
	{
		return this.m_interaction.getUnreadEventCount() > 0;
	};

	SmsClientWindow.prototype.closeRelated = function( original_index )
	{
		var self = this;
		var $container = $( "#thread_container" );

		var on_ready = function()
		{
			if ( self.m_currentEvent === 0 ) {
				// cuando se cierra el tab de unread al post origen lo dejo marcado como leido!!!
				var current_event = self.m_interaction.getEvent( self.m_currentEvent );
				if ( self.isReadEvent( current_event, self.m_currentEvent ) ) {
					var id = current_event.message.id; //TODO
					var $post = $container.find( "[data-id='" + id + "']" );
					if ( $post && $post.length > 0 ) {
						self.on_readed_event( true, $post, true );
					}
				}
			}
			else {
				// cuando esta listo, siempre tiene que quedar en el evento cero (el post original m_currentEvent = 0)
				// ya que el input para comentar siempre que se cierra el related es sobre el post original
				self.m_currentEvent = 0;
			}
			self.hideStatusMessage();
			self.focusSendMessageText();
		};

		SmsClientWindow.uber.closeRelated.call( this, original_index, function()
		{
			if ( original_index === undefined ) {
				on_ready();
				return;
			}
			var current_event = self.m_interaction.get_event_by_original_index( original_index );
			if ( current_event === false || !current_event.message || !current_event.message.id ) {
				on_ready();
				return;
			}
			var id = current_event.message.id;
			var $message = $container.find( "[data-id='" + id + "']" );
			if ( !$message || $message.length === 0 ) {
				on_ready();
				return;
			}

			var $mark_read_unread = $message.find( "[mark_as_read_unread]" );
			if ( $mark_read_unread.length > 0 ) {
				$mark_read_unread.hide();
			}

			if ( self.isEditable() ) {
				if ( !$message.hasClass( "unread" ) ) {
					$message.addClass( "unread" );
				}
			}
			if ( !$message.hasClass( "selected" ) ) {
				$message.addClass( "selected" );
			}

			var scroll_data = self.getScrollBasicInfo();
			scroll_data.destination = "item_position";
			scroll_data.$item = $message;
			scroll_data.callback = function()
			{
				window.setTimeout( function() {
					self.on_readed_event( true /* success */, $message, true /* is read */ );
				}, TIME_SELECTED_COMMENT_MS );

				on_ready();
			};
			SCROLL_TO( scroll_data );
		});
	};

	SmsClientWindow.prototype.invalidInteraction = function() {
		SmsClientWindow.uber.invalidInteraction.call( this );
		this.toggleSendMessage(false);
	};

	SmsClientWindow.prototype.onInteractionGone = function(interaction) {
		SmsClientWindow.uber.onInteractionGone.call( this, interaction );
		this.enableSendMessageButton(false);
		this.enableSendMessageText(false);
	};

	SmsClientWindow.prototype.showAddressesBook = function() {
		window.alert("showAddressesBook");
	};

	SmsClientWindow.prototype.setEventsHandlers = function() {
		var self = this;

		SmsClientWindow.uber.setEventsHandlers.call( this ); //invokes the base class method

		if (!this.m_interaction.isBeingComposed()) {
			$("#mark_all_read_unread").click( function() {
				self.markAllReadUnread();
			});
		}
		else {
			$("#showAddressesBook").click( function() {
				self.showAddressesBook();
			});

			const visualize_address = inConcert.app.canVisualizeAddress(null, this.m_interaction.campaign, null, true);
			// Si los numeros de phone no se deben mostrar, no agrego el evento de busqueda.
			if(visualize_address){
			const $setAddress = $("#setAddress");
			$setAddress.off("keyup")
				.on("keyup", function(evt) {
					self.search_contact_keypress(evt, $setAddress);
				})
				.off("click")
				.on("click", function() {
					window.setTimeout(function() {
						let value = $setAddress.val().trim();
						self.search_contact(value, $setAddress);
					});
				}
			);}
		}

		var $container = $("#thread_container");

		// mark as read/unread comment link
		var event_selector = "[mark_as_read_unread]";
		$container
			.off("click", event_selector)
			.on("click", event_selector, function () {
				var $mark_read_unread = $( this );

				if ( $mark_read_unread.prop( "disabled" ) ) {
					return;
				}

				var $comment = $mark_read_unread.parent();
				if ( $comment.hasClass( "selected" ) ) {
					return;
				}

				var event_index = $mark_read_unread.data( "event_index" );
				var new_state = $mark_read_unread.data( "event_unread" ) === "unread";
				self.mark_as_read(new_state, event_index, function( success ) {
					self.on_readed_event( success, $comment );
				});
			});

		event_selector = "[forward_event]";
		$container
			.off("click", event_selector)
			.on("click", event_selector, function () {
				if ( !self.canExecuteInteractionAction() ) {
					return;
				}
				self.startForward($(this).data( "event_index" ));
			});

			/* Gestionar contacto si:
		 1- Esta habilitado la administracion de contactos
		 2- Se pueden visualizar los addresses
		*/
		const visualize_address = inConcert.app.canVisualizeAddress(null, this.m_interaction.campaign, null, true);

		if ( this.m_interaction.accountHasContactManagementEnabled( this.from_app ) && visualize_address) {
			//Setup contact manager popup when user click in the contact
			event_selector = ".smsEventAuthor";
			$container
				.off("click", event_selector)
				.on("click", event_selector, function () {
					var event_index = $( this ).data( "event_index" );
					if ( !event_index && event_index !== 0 ) {
						return;
					}
					var event = self.m_interaction.getEvent( event_index );
					if ( !event || event.creator !== "contact" ) {
						return;
					}

					var address_normalized = inConcert.app.getContactManager().normalizeContactAddress(self.m_interaction.getRawAddress( event_index ), "Phone");

					var contact_window = new inConcert.ContactWindow({
						from_app    : self.from_app,
						interaction : self.m_interaction,
						name        : self.m_interaction.getUserName( event ),
						avatar      : self.m_interaction.getEventUserAvatarURI( event ),
						addresses   : [ address_normalized ]
					});
					contact_window.show();
				}
			);
		}
	};

	SmsClientWindow.prototype.clientUpdate = function () {
		if (!this.m_interaction.isValid()) {
			return;
		}
		if ( this.was_being_composed && !this.m_interaction.isBeingComposed() && ( this.m_interaction.events || [] ).length > 0 ) {
			this.close();
			return;
		}

		this.enableSendMessageText(true);
		this.__updateCharsCounter();
		this.set_mark_all_unread_read_button_caption();
	};

	SmsClientWindow.prototype.change_client_state = function( new_state ) {
		new_state = SmsClientWindow.uber.change_client_state.call( this, new_state );

		var $solve_container = $("#sms_solve_container");

		switch( new_state )
		{
			case "archived":
				$solve_container.hide();
			break;

			case "connected":
				$solve_container.show();
			break;

			case "monitoring":
				$solve_container.hide();
			break;

			case "pending":
				$solve_container.hide();
			break;
		}

		return new_state;
	};
	
/*
-----------
Send Message
-----------

TODO: (REFACTORING) Revisar todas estas funciones referentes a "send message", las mismas se repiten con el mismo codigo en los clientes "webchat", "whatsapp", "sms", "rcs".

Seria lo mejor moverlas a la clase "clientWindow" asi no repetimos codigo. Aunque la clase client window, tambien la utiliza "mail" el "phone" etc, y estas funciones no son utiles ahi, habria que reviarlo.

Quizas lo mejor es crear un clase a parte, algo asi tipo "clientSendMessage" y quien la necesite que la instancie. VER!!!

*/
	SmsClientWindow.prototype.$sendMessageText = function() {
		return this.$sendMessageContainer().find("[sendMessageText]");
	};

	SmsClientWindow.prototype.clearSendMessageText = function() {
		var $input = this.$sendMessageText();
		var $input_emoji = $input.siblings(".emoji-wysiwyg-editor");
		if ($input_emoji.length) {
			$input_emoji.html("");
		}
		$input.val("");
	};

	SmsClientWindow.prototype.focusSendMessageText = function() {
		var $input = this.$sendMessageText();
		var $input_emoji = $input.siblings(".emoji-wysiwyg-editor");
		if ($input_emoji.length) {
			inConcert.Utils.placeCaretAtEnd($input_emoji.get(0));
		}
		else {
			$input.focus();
		}
	};

	SmsClientWindow.prototype.enableSendMessageText = function(enabled) {
		this.$sendMessageText().prop("disabled", !enabled);
		$("#setAddress").prop("disabled", !enabled);
	};

	SmsClientWindow.prototype.appendToSendMessageText = function(text) {
		var $input = this.$sendMessageText();
		var $input_emoji = $input.siblings(".emoji-wysiwyg-editor");
		if ($input_emoji.length) {
			$input_emoji.html($input_emoji.html() + " " + text);
		}
		$input.val($input.val() + " " + text);
		this.__updateCharsCounter();
	};
	
	SmsClientWindow.prototype.setSendMessageText = function(text) {
		this.clearSendMessageText();
		this.appendToSendMessageText(text);
	};

	SmsClientWindow.prototype.getSendMessageText = function() {
		return this.$sendMessageText().val();
	};

	SmsClientWindow.prototype.$sendMessageButton = function() {
		return this.$sendMessageContainer().find("[sendMessageButton]");
	};

	SmsClientWindow.prototype.enableSendMessageButton = function(enabled) {
		this.$sendMessageButton().prop("disabled", !enabled);
	};

	SmsClientWindow.prototype.$sendMessageLimit = function() {
		return this.$sendMessageContainer().find("[sendMessageLimit]");
	};

	SmsClientWindow.prototype.setSendMessageLimit = function(value) {
		this.$sendMessageLimit()
			.text(value)
			.toggleClass("smsClientCharacterCounterNegative", value < 0);
	};

	SmsClientWindow.prototype.bindSendMessage = function() {
		var self = this;

		//container
		this.toggleSendMessage(true);

		//button
		this.$sendMessageButton().click(function() {
			self.sendMessage();
		});

		//text
		this.$sendMessageText().change(function() {
			self.__updateCharsCounter();
		});
		this.focusSendMessageText();

		//limit
		this.__updateCharsCounter();

		//address
		if (this.m_interaction.isBeingComposed()) {
			$("#mainContainer").addClass("smsNewMessageClientContainer");
			$("#setAddressContainer").show();
			$("#setAddress").focus();
		}
	};

	SmsClientWindow.prototype.__updateCharsCounter = _.throttle(function() {
		var currentText = this.$sendMessageText().val();
		var currentChars = currentText && currentText.length || 0;
		var remainingChars = m_maxCharsPerMessage - currentChars;
		this.setSendMessageLimit(remainingChars);
		this.enableSendMessageButton(currentChars > 0 && remainingChars >= 0);
	}, 100);

	SmsClientWindow.prototype.validatePhoneNumber = function(number) {
		return inConcert.app.validatePhoneNumber(number, "SMS");
	};

	SmsClientWindow.prototype.sendMessage = function(callback = $.noop) {
		const self = this;
		if (self.__sending) {
			return callback(false, "already sending another message");
		}
		var text = this.getSendMessageText();
		if (!text) {
			return callback(false, "must enter a message text");
		}
		var message = {
			text : text
		};

		function beforeSend() {
			self.__sending = true;
			self.displayStatusMessage("<%=_T('Sending...')%>", "Process", true);
			self.enableSendMessageButton(false);
		}

		function processResult(success, message) {
			if ( success ) {
				self.clearSendMessageText();
				self.hideStatusMessage();
			}
			else {
				self.enableSendMessageButton(true);
				self.displayStatusMessage( message, "Error", true, 5000 );
			}
			self.__sending = false;
			callback(success, message);
		}

		if (this.m_interaction.isBeingComposed()) {
			const $setAddress = $("#setAddress");
			let phoneNumber = $setAddress.val();
			if (!this.validatePhoneNumber(phoneNumber)) {
				this.displayStatusMessage( "<%=_T('Invalid phone number')%>", "Error", true, 5000 );
				return callback(false, "Invalid phone number");
			}
			phoneNumber = inConcert.Utils.cleanPhoneNumber(phoneNumber);
			beforeSend();
			this.m_interaction.createAndSendMessage({
				attention_level_id : self.getSelectedAttentionLevel(),
				address : phoneNumber,
				message : message,
				callback : processResult
			});
		}
		else {
			//mark all as read
			if ($("#mark_all_read_unread").val() === this.MARK_ALL_READ_BTN_TEXT) {
				$("#mark_all_read_unread").click();
			}
			beforeSend();
			this.m_interaction.sendMessage(undefined, message, processResult);
		}
	};

///////////////////Events

	SmsClientWindow.prototype.isReadEvent = function( event, event_index ) {
		if ( typeof event !== "object" ) {
			throw "SmsClientWindow.isReadEvent - event is not a object.";
		}
		if ( event.outgoing_pending ) {
			return true;
		}
		if ( !event_index ) {
			return true;
		}
		return this.m_interaction.isRead( event_index );
	};

	SmsClientWindow.prototype.on_readed_event = function( success, $comment, is_read )
	{
		if ( !success || typeof $comment !== "object" || $comment.length === 0 ) {
			return;
		}

		if ( !this.isEditable() ) {
			$comment.removeClass( "selected" );
			return;
		}

		// thread link class
		var class_mark_as_read = LNK_THREAD_MARK_AS_READ_CLASS;
		var class_mark_as_unread = LNK_THREAD_MARK_AS_UNREAD_CLASS;
		if ( $comment.hasClass( "smsThreadOrigin" ) ) {
		// origin link class
			class_mark_as_read = LNK_ORIGIN_MARK_AS_READ_CLASS;
			class_mark_as_unread = LNK_ORIGIN_MARK_AS_UNREAD_CLASS;
		}

		var new_state, new_caption;
		var $mark_read_unread = $comment.find("[mark_as_read_unread]");
		$mark_read_unread.prop( "disabled", false );
		var state = $mark_read_unread.data( "event_unread" );
		if ( is_read || state === "unread" ) {
			new_state = "read";
			new_caption = "<%=_T('Mark as unread')%>";
			$comment.removeClass( "unread selected" );
			$mark_read_unread.removeClass( class_mark_as_read ).addClass( class_mark_as_unread );
		}
		else {
			new_state = "unread";
			new_caption = "<%=_T('Mark as read')%>";
			$comment.addClass( "unread" );
			$mark_read_unread.removeClass( class_mark_as_unread ).addClass( class_mark_as_read );
		}
		$mark_read_unread.data( "event_unread", new_state ).text( new_caption ).show();
	};

	SmsClientWindow.prototype.get_html_read_unread_link = function( event, event_index, is_thread ) {
		var response = {
			html         : '',
			parent_class : ''
		};

		var lnk_mark_as_read_class = LNK_THREAD_MARK_AS_READ_CLASS;
		var lnk_mark_as_unread_class = LNK_THREAD_MARK_AS_UNREAD_CLASS;
		if ( !is_thread ) {
			lnk_mark_as_read_class = LNK_ORIGIN_MARK_AS_READ_CLASS;
			lnk_mark_as_unread_class = LNK_ORIGIN_MARK_AS_UNREAD_CLASS;
		}

		if ( event.outgoing_pending === true || event.creator === 'agent' ) {
			return response;
		}

		if ( !this.isReadEvent( event, event_index ) ) {
			response.parent_class = ' unread';
			if ( this.isEditable() ) {
				response.html = FORMAT(
					''.concat(
						'<a mark_as_read_unread class="', lnk_mark_as_read_class, '" href="javascript:void(0);"',
								' data-event_index="{0}" data-event_unread="{1}">',
							"<%=_T('Mark as read')%>",
						'</a>'
					),
					event_index,  // data-event_index
					'unread'
				);
			}
		}
		else {
			if ( this.isEditable() ) {
				response.html = FORMAT(
					''.concat(
						'<a mark_as_read_unread class="', lnk_mark_as_unread_class, '" href="javascript:void(0);"',
								' data-event_index="{0}" data-event_unread="{1}">',
							"<%=_T('Mark as unread')%>",
						'</a>'
					),
					event_index,  // data-event_index
					'read'
				);
			}
		}

		return response;
	};
	
	SmsClientWindow.prototype.makeEventForward = function(_, event_index, is_thread) {
		if (!this.isEditable() || !this.m_interaction.canForward()) {
			return;
		}
		return {
			html : FORMAT(
				"".concat(
					'<a forward_event message_control class="{0}" href="javascript:void(0);"',
							' data-event_index="{1}">',
						"<%=_T('Forward')%>",
					'</a>'
				),
				is_thread? "threadActionsForward" : "actionsForward",
				event_index
			)
		};
	};
	
	SmsClientWindow.prototype.makeEventActions = function(event, event_index, is_thread) {
		let actions = [];
		actions.push(this.makeEventForward(event, event_index, is_thread));
		actions.push(this.get_html_read_unread_link(event, event_index, is_thread));
		return actions.reduce(function(content, action) {
			content.html += action && action.html || "";
			content.parent_class += action && action.parent_class || "";
			return content;
		}, { html : "", parent_class : "" });
	};

	SmsClientWindow.prototype.makeMessageStatusMarkup = function(status, reason) {
		var text;
		var className;
		var showReason = false;
		switch(status) {
			case "SENDING":
				text = "<%=_T('Sending')%>";
				className = "deliverySending";
				break;
			case "FAIL":
				text = "<%=_T('Failed')%>";
				className = "deliveryFail";
				showReason = true;
				break;
			/*case "SENT":
				text = "<%=_T('Delivered')%>";
				className = "deliveryOk";
				break;
			case "READ":
				text = "<%=_T('Read')%>";
				className = "deliveryRead";
				break;*/
			default:
				return null;
		}
		var $status = $(''.concat(
			'<div class="deliveryMessageContainer ', className, '" >',
				'<label class="deliveryMessageText" >', text, '</label>',
			'</div>',
			'<div class="deliveryFailReasonContainer" hidden>',
				'<label class="deliveryFailReasonTxt">', reason || "", '</label>',
			'</div>'
		));
		if (showReason && reason) {
			$status.filter(".deliveryMessageContainer").hover(
				function() {
					$(this).siblings(".deliveryFailReasonContainer").show();
				},
				function() {
					$(this).siblings(".deliveryFailReasonContainer").hide();
				}
			);
		}
		return $status;
	};

	SmsClientWindow.prototype.makeEventMarkup = function(event, index)
	{
		const self = this;

		let isFirst = index === 0;
		let id = event.message.id;
		let authorName = this.m_interaction.getUserName( event );
		let authorNumber = this.m_interaction.getUserNumber( event );
		const visualize_address = inConcert.app.canVisualizeAddress(null, this.m_interaction.campaign, null, true); //No mostrar el numero del contacto
		if (authorNumber && authorNumber !== authorName) {
			authorName = FORMAT("{0} ({1})", authorName, (!visualize_address && event.creator === "contact")? "<%=_T('Hidden Address')%>": authorNumber);
		}
		let authorId   = this.m_interaction.get_user_event_id( event );
		let genericAvatar = this.getGenericAvatar();
		let avatar = this.m_interaction.getEventUserAvatarURI(event) || genericAvatar;
		let eventDate = this.m_interaction.getDateObject(index);
		let status = this.m_interaction.getMessageStatus(event);
		let forwarded = this.m_interaction.getForwarded(index);
		let content = this.m_interaction.getMessageContent(event);
		let message_text = inConcert.Utils.linkify( inConcert.Utils.escapeAttrib(content.text), { exclude: { email: true } } );
		let class_container = isFirst? 
			"smsThreadOrigin " :
			"smsThreadSubEvent ";

		switch(event.creator) {
			case "contact":
				class_container += " contactEvent";
				break;
			case "bot":
				class_container += " botEvent";
				break;
			default:
				class_container += " agentEvent";
				break;
		}
		
		if (forwarded) {
			class_container += " forwardedEvent";
		}

		let actions = this.makeEventActions(event, index, !isFirst);
		class_container += actions.parent_class;

		let attr_id =  ' data-id="' + id + '" ';
		let attr_index =  ' data-event_index="' + index + '" ';
		let time = ' data-time="' + eventDate.getTime() + '" ';

		let $message = $( ''.concat(
			'<div class="', class_container, '" ', attr_id, attr_index, time, '>',
				
				forwarded? FORMAT(''.concat(
					'<div class="forwardedEventMark" title="', '<%=_T("Forwarded by agent {0}")%>', '">', 
					'<%=_T("Forwarded")%>', '</div>'), inConcert.Utils.getIdWithoutVCC(forwarded.agent)) : "",

				'<div class="smsEventAvatar">',
					'<object data="', avatar, '" type="image/png" width="24" height="24">',
						'<img src="', genericAvatar, '" width="24" height="24"/>',
					'</object>',
				'</div>',

				'<div class="smsEventMessageContainer">',
					'<a href="javascript:void(0);" class="smsEventAuthor" data-event_index="', index , '" data-event_user_id="', authorId , '">', authorName, '</a>',
					'<label message_text="true" class="smsEventMessage">', message_text, '</label>',
				'</div>',

				'<label event_date="true" class="smsEventDate">', inConcert.Utils.userFormatDate(eventDate, false, true), '</label>',

				actions.html,
			'</div>'
		));

		if (event.creator === "contact") {
			const $message_text = $message.find('[message_text]');
			$message_text
				.off('dblclick')
				.on('dblclick', function() {
					self.setMessageInSideBar(message_text);
				}
			);
		}

		if (status) {
			$message
				.data("status", status)
				.addClass("status" + status)
				.prepend(this.makeMessageStatusMarkup(status, this.m_interaction.getMessageStatusReason(event)));
		}

		return $message;
	};

	SmsClientWindow.prototype.openCurrentEvent = function( callback ) {
		callback = callback || $.noop;
		var self = this;

		var execute_callback = function( success ) {
			self.hideStatusMessage();
			callback( success );
		};

		this.displayStatusMessage( "<%=_T('Please wait, loading messages.')%>", "Process", false );
		var max_append_per_loop = 50;
		var pause_between_loop_ms = 10;
		var index = 0;
		var events = this.m_interaction.events || [];
		var $container = $("#thread_container");
		$container.html("");

		(function addEvents() {
			var added = 0;
			while (index < events.length && added < max_append_per_loop) {
				var $message = self.makeEventMarkup(events[index], index);
				$container.append($message);
				added++;
				index++;
			}
			if (index === events.length) {
				return execute_callback(true);
			}
			window.setTimeout(addEvents, pause_between_loop_ms);
		})();
	};

	SmsClientWindow.prototype.__updateEvent = function(index, event) {
		var id = event.message && event.message.id || null;
		if (!id) {
			return;
		}
		var $container = $( "#thread_container" );
		var $currentMessage = $container.find( "[data-id='" + id + "']" );
		if ($currentMessage.length) {
			var status = this.m_interaction.getMessageStatus(event);
			if (!status || status === $currentMessage.data("status")) {
				return;
			}
		}
		var $message = this.makeEventMarkup(event, index);
		if ($currentMessage.length) {
			$currentMessage.replaceWith($message);
		}
		else {
			$container.append($message);
			var scroll_data = this.getScrollBasicInfo();
			scroll_data.destination = "bottom";
			SCROLL_TO( scroll_data );
		}
	};

	SmsClientWindow.prototype.__updateEvents = function() {
		var self = this;
		$.each(self.m_interaction && self.m_interaction.events || [], function(index, event) {
			self.__updateEvent(index, event);
		});
	};

	SmsClientWindow.prototype.onInteractionUpdated = function(interaction, callback) {
		callback = callback || $.noop;
		var self = this;
		SmsClientWindow.uber.onInteractionUpdated.call(this, interaction, function(status) {
			if (!self.m_interaction.isBeingComposed()) {
				$("#mainContainer").removeClass("smsNewMessageClientContainer");
				$("#setAddressContainer").hide();
			}
			self.__updateEvents();
			callback(status);
		});
	};

	SmsClientWindow.prototype.getInteractionRelatedMarkup = function( properties )
	{
		let readClass = "relatedInteractionUnread";
		let message = inConcert.Utils.escapeAttrib(this.m_interaction.getMessageText(properties.event));
		let message_abstract = inConcert.Utils.linkify( message.length <= 60 ? message : message.substring(0, 60) + '...', {exclude: {email: true}});
		let author_name = this.m_interaction.getUserName(properties.event);
		let genericAvatar = this.getGenericAvatar();
		let avatar = properties.avatarURI || this.m_interaction.getEventUserAvatarURI(properties.event) || genericAvatar;

		return $(''.concat(
			'<div unread_event="true" class="relatedInteraction ', readClass, '">',
				'<div class="interactionAvatar">',
					'<object data="', avatar, '" type="image/png" width="36px" height="36px">',
						'<img src="', genericAvatar, '" width="36px" height="36px"/>',
					'</object>',
				'</div>',
				'<div class="interactionInfoContainer">',
					'<label class="interactionContactName">',
						author_name,
					'</label>',
					'<label class="interactionDate">', this.m_interaction.getLocaleDateString( properties.sorted_index ), '</label>',
					'<label class="interactionAbstract">', message_abstract, '</label>',
				'</div>',
				'<a unread_event_action="true" href="javascript:void(0);" class="interactionButton">',
					'<i class="far fa-chevron-right"></i>',
				'</a>',
			'</div>'));
	};
	
	SmsClientWindow.prototype.iterateEventsToForward = function(index) {
		let self = this;
		$("#thread_container").children().each(function() {
			let $event = $(this);
			let event_index = $event.data("event_index");
			self.enableEventForward($event, event_index, event_index === index);
		});
	};
	
	SmsClientWindow.prototype.addForwardSelector = function($event, $selector) {
		$event.prepend($selector);
	};
	
	SmsClientWindow.prototype.markForwardSelector = function($selector, selected) {
		$selector.parent().parent().toggleClass("forwardSelected", selected);
	};
	
	SmsClientWindow.prototype.getForwardMessagePopupAddressLabel = function() {
		return "<%=_T('Number')%>";
	};
	
	SmsClientWindow.prototype.validateForwardMessageAddress = function(phoneNumber) { 
		if (typeof(phoneNumber) === "object") {
			phoneNumber = phoneNumber.SanitizedId;
		}
		if (!this.validatePhoneNumber(phoneNumber)) {
			this.displayStatusMessage( "<%=_T('Invalid phone number')%>", "Error", true, 5000 );
			return false;
		}
		phoneNumber = inConcert.Utils.cleanPhoneNumber(phoneNumber);
		return {
			chatType : "user",
			address : phoneNumber
		};
	}; 
	
	SmsClientWindow.prototype.getForwardMessageAddressType = function() {
		return "Phone";
	};
	
	SmsClientWindow.prototype.__addDraftTextMessage = function(message, callback) {
		this.setSendMessageText(message.text || "");
		callback(true);
	};
	
	SmsClientWindow.prototype.__sendDraftTextMessage = function(_, callback) {
		this.sendMessage(callback);
	};

///////////////////Iframe Constructor

	//when jquery says so, start it all up
	$(document).ready(function()
	{
		//initializates emoji
		window.emojiPicker = new EmojiPicker({
			emojiable_selector: '[data-emojiable=true]',
			assetsPath: '/inconcert/lib/emoji-picker/lib/img/',
			popupButtonClasses: 'emoji-picker-button',
			norealTime: false,
			onPaste: function(pasted) {
				/*
					Este callback se debe especificar para que el texto que se va pegar desde el portapapeles, no incluya ENTERs que no van en el texto
					cuando el origen es por ejemplo una planilla de Excel.
					Aunque el callback simplemente retorna el mismo valor que ha recibido, quien realiza el laburo es el mismo plugins "EMOJI"
					cuando obtiene la información a pegar desde portapapeles en formato 'text/plain' y luego invoca al callback.
					Archivo: "\lib\inconcert\agent\inconcert\lib\emoji-picker\lib\js\jquery.emojiarea.js"
					Ref Issue #7071
				*/
				return pasted;
			}
		});
		window.emojiPicker.discover();

		window.hook_environment();
		inConcert.i18n.updateScreen();

		var callback = function()
		{
			var client = new SmsClientWindow(window.interaction,
				{
					archive  : window.archive,
					from_app : window.from_app,
					queued   : window.queued,
					waiting  : window.waiting,
					monitor  : window.monitor,
					readonly : window.readonly || window.archive || window.monitor || window.queued || window.waiting
				}
			);

			window.onReady(client);

			if (window && window.view && window.view instanceof inConcert.ClientWindowProxy) {
				window.view.apply(client);
			}
			window.view = client;
		};

		if(window.interaction.isBeingComposed()) {
			callback();
		}
		else {
			window.interaction.inflate(callback);
		}
	});
}());
