/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:single, esversion: 9 */

/* global $, inConcert */

// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	/**
		Class SpamInteractionsPanel.
	*/

	'use strict';

	function SpamInteractionsPanel (id, parent_panel)
	{
		//the category should be defined before the base constructor in order to be used properly
		this.m_parent_panel = parent_panel;
		this.m_category = 'spam';
		this.m_width = 256;
		this.m_main = inConcert.app.getMainWindow();
		this.m_views = {
			SPAM : 0
		};
		this.m_currentView = this.m_views.SPAM; //one of SPAM
		this.apply_permissions();

		SpamInteractionsPanel.uber.constructor.call(this, {id: id, name: "<%=_T('Spam Conversations')%>"});

		this.m_dockToLeft = false;
	}

	inConcert.Utils.inherit(SpamInteractionsPanel, inConcert.InteractionPanel);

	//Define functions and objects in the namespace in case they don't already exist
	if (typeof inConcert.SpamInteractionsPanel !== 'function') {
		inConcert.SpamInteractionsPanel = SpamInteractionsPanel;
	}

	SpamInteractionsPanel.prototype.can_reposition = function() {
		return false;
	};

	SpamInteractionsPanel.prototype.get_prefix = function( /*interaction*/ )
	{
		return 'spam_interaction_';
	};

	SpamInteractionsPanel.prototype.allowed_bulk_actions = function( /*interaction*/ )
	{
		return false;
	};
	// SpamInteractionsPanel.prototype.getInteractionDOMIdForPanel = function( interaction ) {
		// var normalized_id = inConcert.Utils.getNormalizedId( interaction.id );
		// return this.get_prefix() + normalized_id;
	// };

	SpamInteractionsPanel.prototype.set_events = function()
	{
		var self = this;

		//handlers for tab switching

		$('#butDeleteSpam').unbind('click').click(function() {
			self.deleteSpam();
		});
		$('#butNotSpam').unbind('click').click(function() {
			self.markNotSpam();
		});
		$('#int_spam_select_all').unbind('click').click(function() {
			self.selectAllSpam();
		});
	};

	SpamInteractionsPanel.prototype.apply_permissions = function()
	{
		if ( inConcert.permissions.to_action_on_any_campaign( 'interaction', 'int_delete_spam' ) ) {
			$( '#butDeleteSpam' ).show();
		}
		else {
			$( '#butDeleteSpam' ).hide();
		}

		if ( inConcert.permissions.to_action_on_any_campaign( 'interaction', 'int_not_spam' ) ) {
			$( '#butNotSpam' ).show();
		}
		else {
			$( '#butNotSpam' ).hide();
		}
	};

	SpamInteractionsPanel.prototype.apply_filter = function()
	{
		window.alert('this.refresh_queued()');
	};

	SpamInteractionsPanel.prototype.selectAllSpam = function() {
		var checked = $('#int_spam_select_all').prop('checked');
		this.m_handler.find('.boxSpamInteraction > :checkbox').prop('checked', checked);
		this.updateSelectedSpamCounter();
	};

	SpamInteractionsPanel.prototype.deleteSpam = function() {
		if ( !inConcert.permissions.to_action_on_any_campaign( 'interaction', 'int_delete_spam' ) ) {
			return;
		}
		var self = this;
		var interactionIDs = [];
		self.m_handler.find('.boxSpamInteraction > :checked').attr('interactionId', function(undefined, interactionId) {
			interactionIDs.push(interactionId);
		});

		self.m_handler.find('.boxSpamInteraction > :checked').parent().addClass('interactionWaiting');

		$('#butDeleteSpam').addClass('butDeleteSpamExecuting');

		inConcert.app.getInteractionManager().deleteSpam(interactionIDs, function (ok, err) {
			$('#butDeleteSpam').removeClass('butDeleteSpamExecuting');
			if (ok) {
				self.refreshSpamQueue();
			}
			else {
				self.m_main.showBannerMessage({ message : err, timeout : 5000});
				self.m_handler.find('.boxSpamInteraction > :checked').parent().removeClass('interactionWaiting');
			}
		});
	};

	SpamInteractionsPanel.prototype.markNotSpam = function() {
		if ( !inConcert.permissions.to_action_on_any_campaign( 'interaction', 'int_not_spam' ) ) {
			return;
		}
		var self = this;
		var interactionIDs = [];
		var hash_interactions = {};
		this.m_handler.find('.boxSpamInteraction > :checked').attr('interactionId', function(undefined, interactionId) {
			interactionIDs.push(interactionId);
			hash_interactions[interactionId] = true;
		});

		this.m_handler.find('.boxSpamInteraction > :checked').parent().addClass('interactionWaiting');

		$('.butTrustedContact').addClass('butTrustedContactExecuting');

		/*
			Esta funcion es la que refresca el panel de interacciones spam.
			Se realiza dentro de un timeout con reintentos, porque la funcion que realiza 
			el spam retorna ok, pero en el ir / proxy se toma su tiempo para realizar esta tarea.
			y sino la funcion que retorna la lista de spams, para actualizar la misma trae interacciones que se estan procesando
			y parece que han fallado.

			Entonces se agrega en un timeout de 2 segundos los request con 3 reintentos.
		*/
		var max_retries = 3;
		var retries = 0;
		var refresh_list = function() {
				window.setTimeout( function()
				{
					self.refreshSpamQueue( function( result )
					{
						if ( !result) {
							return;
						}
						if ( result.totalCount === 0 ) {
							return;
						}
						var must_retry = false;
						var spam_interactions = result.interactions;
						$.each( spam_interactions || {}, function( undefined, spam_it) {
							if ( hash_interactions[ spam_it.id ] ) {
								must_retry = true;
							}
						});
						if ( must_retry && ( retries < max_retries ) ) {
							retries++;
							refresh_list();
						}
					});
				},
				2000 /* time out ms */
			);
		};

		inConcert.app.getInteractionManager().markNotSpam(interactionIDs, function (ok, err) {
			$('.butTrustedContact').removeClass('butTrustedContactExecuting');
			if (ok) {
				refresh_list();
			}
			else {
				self.m_handler.find('.boxSpamInteraction > :checked').parent().removeClass('interactionWaiting');
				self.m_main.showBannerMessage({ message : err, timeout : 5000});
			}
		});
	};

	SpamInteractionsPanel.prototype.reset = function() {
		this.m_windows = {};
		this.m_main.getPanel('SpamInteractions').clearListContainer();
	};

	SpamInteractionsPanel.prototype.refreshPanel = function()
	{
		this.refresh();
	};

	SpamInteractionsPanel.prototype.interactionPassesFilter = function( /* interaction*/ ) {
		return true;
	};

	SpamInteractionsPanel.prototype.interactionBelongsToPanel = function(interaction) {
		// TODO: esto sirve de algo ?? revisar
		var iAmRequired = interaction.iAmRequired();
		var belongToMe = interaction.belongsToMe();
		return !iAmRequired && !belongToMe && interaction.isQueued() && (!interaction.getFirstAssignedAgent() || interaction.scheduledDate);
	};

	SpamInteractionsPanel.prototype.update_counters_navigation = function( total )
	{
		if ( this.m_parent_panel ) {
			this.m_parent_panel.update_counters( {
				spam : total
			});
		}
	};

	SpamInteractionsPanel.prototype.refreshSpamQueue = function( callback )
	{
		var self = this;
		callback = callback || $.noop;
		if ( !self ) {
			return;
		}
		var panel = self.m_main.getPanel('SpamInteractions');
		var im = inConcert.app.getInteractionManager();

		var spamListContainer = this.m_handler.find('.boxSpamScrollArea');
		spamListContainer.html('');
		this.updateSelectedSpamCounter();

		im.getSpamInteractions(function(result)
		{
			if (!result) {
				self.m_main.showBannerMessage({ message : "<%=_T('Error getting spam conversations. Please retry later')%>", timeout : 5000});
				callback( result );
				self.update_counters_navigation(0);
				return;
			}
			$('#spamCounterTotal').text(result.totalCount);
			self.update_counters_navigation(result.totalCount);

			var has_spam = result.totalCount > 0;
			var $select_all = $('#int_spam_select_all');
			$select_all.prop('disabled', !has_spam);
			if ( !has_spam ) {
				$select_all.prop('checked', false);
			}

			for ( var i = 0; i < result.interactions.length; i++ ) {
				var interaction = im.buildInteractionObject( result.interactions[i] );

				var interactionBox = self.getSpamInteractionMarkup( interaction );
				var $node = $(interactionBox);

				spamListContainer.append( $node );
				if (panel.onSpamNodeInserted) {
					panel.onSpamNodeInserted( $node, interaction );
				}
			}
			callback( result );
		});
	};

	SpamInteractionsPanel.prototype.updateSelectedSpamCounter = function()
	{
		var self = this;
		var count = 0;
		var has_permission_to_delete_spam = true;
		var has_permission_to_not_spam = true;

		var has_permission_fn = function( action_id, campaign_id )
		{
			return inConcert.permissions.to_action(
				'interaction',
				action_id,
				undefined /* category */,
				campaign_id,
				true /* no log */
			);
		};

		var $spams = this.m_handler.find('.boxSpamInteraction > :checked');
		$.each( $spams, function( undefined, spam )
		{
			count++;
			var campaign_id = $( spam ).attr( 'campaign' ); 
			if ( has_permission_to_delete_spam ) {
				has_permission_to_delete_spam = has_permission_fn( 'int_delete_spam', campaign_id );
			}
			if ( has_permission_to_not_spam ) {
				has_permission_to_not_spam = has_permission_fn( 'int_not_spam', campaign_id );
			}
		});

		$('#spamCounterSelected').text( count );

		var disable = function( $button, disabled, has_permission, click_fn )
		{
			if ( disabled ) {
				$button
					.prop('disabled', true )
					.unbind('click')
					.removeClass( 'disabledInfo' )
					.inconcert_tooltip( { detach: true } );
				return;
			}

			if ( has_permission ) {
				$button
					.prop('disabled', false )
					.bind('click', click_fn )
					.removeClass( 'disabledInfo' )
					.inconcert_tooltip( { detach: true } );
			}
			else {
				$button
					.prop('disabled', false )
					.unbind('click')
					.addClass('disabledInfo' )
					.inconcert_tooltip( {message: "<%=_T('You not have permission on all campaigns.')%>", alignment: 'left-top'} );
			}
		};

		var $delete_spam = $('#butDeleteSpam');
		var $not_spam = $('#butNotSpam');

		if ( count > 0 ) {
			disable(
				$delete_spam,
				false,
				has_permission_to_delete_spam,
				function() { self.deleteSpam(); }
			);

			disable(
				$not_spam,
				false,
				has_permission_to_not_spam,
				function() { self.markNotSpam(); }
			);
		}
		else {
			disable( $delete_spam, true );
			disable( $not_spam, true );
		}
	};

	SpamInteractionsPanel.prototype.onSpamNodeInserted = function($new_interaction, interaction) {
		var self = this;
		$new_interaction.dblclick(function(){
			inConcert.app.getMainWindow().getPanel('AssignedInteractions').openInteractionClient(interaction, null, {readonly:true, spam:true});
		});

		$new_interaction.children(':checkbox').click(function(){
			self.updateSelectedSpamCounter();
		});
	};

	SpamInteractionsPanel.prototype.getSpamInteractionMarkup = function(interaction) {
		var contactAddress = interaction.contactAddress;
		var text = interaction.getExtendedInfo();
		var markup = '';
		markup += '<div class="boxSpamInteraction">';
		markup += '		<img src="/inconcert/apps/agent/images/blank.png" class="interactionWaitingAnima" style="display:none"/>';
		markup += '		<input class="interactionCheckbox" type="checkbox" interactionId="' + interaction.id + '" campaign="' + interaction.campaign + '" />';
		markup += '		<img  class="interactionType interactionTypeMAIL" src="/inconcert/apps/agent/images/blank.png" width="16" height="12"/>';
		markup += '		<div class="interactionBasicInfo">';
		markup += '			<div class="interactionContactName">' + contactAddress + '</div>';
		markup += '			<div class="interactionAbstract">' + text + '</div>';
		markup += '		</div>';
		markup += '</div>';

		return markup;
	};

	SpamInteractionsPanel.prototype.cleanupCampaignInteractions = function(campaign) {
		for (var id in this.m_interactions) {
			if (this.m_interactions.hasOwnProperty(id)) {
				var node = this.m_interactions[id];
				var interaction = node.data('interaction');
				if (interaction && interaction.campaign === campaign.Id) {
					node.remove();
					this.removeInteraction(interaction);
				}
			}
		}
	};

// SpamInteractionsPanel.prototype.getInteractionClass = function(interaction) {
	// var interactionClass = 'boxQueuedInteraction' + (interaction.pending ? ' interactionWaiting' : '');
	// //agregar clase para mensajes privados pending
	// if (typeof(interaction.isPrivateMessage) === 'function' && interaction.isPrivateMessage()) {
		// interactionClass += ' privateMessage';
	// }
	// return interactionClass + ' ' + this.getChannelClass(interaction);
// };

}());
