/*global $, inConcert*/
/*jshint browser:true, esversion: 11 */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	"use strict";
	const _T = inConcert.i18n.getString;
	const FORMAT = inConcert.Utils.format;

	function TelephoneCodes(params) {
	/*

		----------------------------
		-     A T E N C I O N      -
		----------------------------

		- Esta variable se actualiza con el contenido del archivo JSON "telehpne_codes.json"
			que se genera cuando se corre el archivo "inconcert_data_loader.lua".
		- Este archivo "telehpne_codes.json" debe ser copiado en el mismo directorio donde se encuentran
			los .js de la aplicacion (inconcert/apps/agent/scripts/).

		let self.m_countries = {
			"1": {
				"areasByCode": {
					"937": [
						"Ohio"
					],
					"848": [
						"New Jersey"
					],
				..
				..
			}
		};
		*/
		this.base_uri         = params.uri + "/telephones";
		this.areas_by_code    = null;
		this.areas_by_name    = null;
		this.countries        = null;
		this.sorted_countries = null;
		this.loadCountries();
	}
	if (typeof(inConcert.TelephoneCodes) !== "function") {
		inConcert.TelephoneCodes = TelephoneCodes;
	}

	TelephoneCodes.prototype.loadCountries = function( callback = $.noop )
	{
		const self = this;
		if ( this.countries ) {
			callback( true, this.countries );
			return;
		}
		inConcert.invoke({
			url: self.base_uri + "/codes/find_all/",
			dataType : "json",
			success: function( countries ) {
				self.countries = countries;
				self.groupAreas();
				callback( true, countries );
			},
			error: function( response, error ) {
				callback( false );
			}
		});
	};

	TelephoneCodes.prototype.groupAreas = function()
	{
		let areas_by_code = {};
		let areas_by_name = {};

		const format_area = function(code, area_name)
		{
			if (code.length === 0) {
				return area_name;
			}
			return code + " - " + area_name;
		};

		const get_code = function(code)
		{
			let code_number = parseInt(code, 10);
			if (isNaN(code_number)) {
				return code;
			}
			return code_number;
		};

		$.each(this.countries, function(country_id, country)
		{
			//aca recorro y agrego todas las areas de una country en un array x codigo 
			areas_by_code[country_id] = areas_by_code[country_id] || [];
			if (country.areasByCode) {
				let grouped_areas = [];
				$.each(country.areasByCode, function(code, areas)
				{
					let code_number = get_code(code);
					// ordena las areas del mismo codigo
					let sorted_areas_by_name = inConcert.Utils.sortArray(areas);
	
					$.each(sorted_areas_by_name, function(i, area_name)
					{
						grouped_areas.push({
							code: code_number,
							name: format_area(code, area_name),
							sort_attr: code_number
						});
					});
				});
				// ahora ordeno todas la areas por codigo
				areas_by_code[country_id] = inConcert.Utils.sortArray(grouped_areas, "sort_attr");
			}

			//aca recorro y agrego todas las areas de una country en un array x name 
			areas_by_name[country_id] = areas_by_name[country_id] || [];
			if (country.areasByName) {
				let grouped_areas = [];
				$.each(country.areasByName, function(area_name, areas)
				{
					// ordeno las areas por area
					let sorted_areas_by_code = inConcert.Utils.sortArray(areas);

					$.each(sorted_areas_by_code, function(i, code)
					{
						grouped_areas.push({
							code: get_code(code),
							name: format_area(code, area_name),
							sort_attr: area_name
						});
					});
				});
				// ahora ordeno todas la areas por area
				areas_by_name[country_id] = inConcert.Utils.sortArray(grouped_areas, "sort_attr");
			}

		});

		this.areas_by_code = areas_by_code;
		this.areas_by_name = areas_by_name;
	};

	TelephoneCodes.prototype.getAreasByCode = function(country_id)
	{
		if ( !this.areas_by_code || typeof( country_id ) === "undefined" ) {
			return false;
		}
		if ( typeof( country_id ) === "string" && country_id.trim().length === 0) {
			return false;
		}
		return this.areas_by_code[ country_id ];
	};

	TelephoneCodes.prototype.getAreasByName = function(country_id)
	{
		if ( !this.areas_by_name || typeof( country_id ) === "undefined" ) {
			return false;
		}
		if ( typeof( country_id ) === "string" && country_id.trim().length === 0 ) {
			return false;
		}
		return this.areas_by_name[ country_id ];
	};

	TelephoneCodes.prototype.findAreaByCode = function( countryId, areaCodeToFind, callback )
	{
		callback = callback || $.noop;

		let result = {
			success: false
		};
		if (typeof( countryId ) === "undefined" || typeof( areaCodeToFind ) === "undefined") {
			callback( result );
			return;
		}
		if ((typeof( countryId ) === "string" && countryId.trim().length === 0) || typeof( areaCodeToFind ) !== "string") {
			callback( result );
			return;
		}

		this.loadCountries( function( success, countries )
		{
			if ( !countries ) {
				callback( result );
				return;
			}
			let country = countries[ countryId ];
			if ( !country || !country.areasByCode ) {
				callback( result );
				return;
			}
			let area = country.areasByCode[ areaCodeToFind ];
			if ( !area ) {
				callback( result );
				return;
			}
			result.success = true;
			result.name = area[0];
			result.code = areaCodeToFind;
			result.areas = area;

			callback( result );
		});
	};

	TelephoneCodes.prototype.fillCountriesCombo = function(options, callback = $.noop)
	{
		const self = this;

		options = options || {};
		const $cboCountries = options.$cboCountries;
		const $cboAreas = options.$cboAreas;

		if (!$cboCountries || !$cboAreas) {
			callback(false);
			return;
		}
		let can_show_country_code = options.can_show_country_code || false;

		this.loadCountries(function(success, countries) { 
			if (!success) {
				callback(false);
				return;
			}
			const build_sorted_countries = function(countries_map) {
				if ($.isArray(self.sorted_countries)) {
					return self.sorted_countries;
				}
				self.sorted_countries = [];
				$.each(countries_map, function(country_id, country) {
					let country_desc = _T(country.name);
					self.sorted_countries.push({code: country_id, name: country_desc});
				});
				return inConcert.Utils.sortArray(self.sorted_countries, "name");
			};

			$cboCountries.html("");
			inConcert.Utils.combo_add_value($cboCountries, "", "", true);
	
			let sorted_countries = build_sorted_countries(countries);
			$.each( sorted_countries, function(_, country) {
				let country_desc = country.name;
				if (can_show_country_code) {
					country_desc = FORMAT("{0} ({1})", country_desc, country.code);
				}
				inConcert.Utils.combo_add_value($cboCountries, country.code, country_desc, false);
			});

			if ($cboAreas) {
				let change_event = "change.telehpone_codes";
				$cboCountries.off(change_event)
					.on(change_event, function() {
						let country_code = $cboCountries.val();
						self.fillAreasCombo(country_code, $cboAreas);
					}
				);
			}
		
			callback(true);
		});
	};

	TelephoneCodes.prototype.fillAreasCombo = function(country_code, $area)
	{
		let areas = this.getAreasByCode(country_code);
		if (!areas) {
			$area.html("");
			return;
		}

		const attr_name = "name";
		const attr_id = "code";
		/*
			En la lista de "areas" hay que manejar diferente al primer elemento "blank" del combo.
			Esto porque en TODOS los combos que utilizamos el elemento "blank" tiene como "value" del <option> el valor "" (vacio).
			Esto funciona bien en todos lados, menos con los "code areas", ya que aca existe un elemento con Id "" que es el "Fixed Lines".
			Esto hace que el combo tenga 2 items con el mismo Id, y aqui el error.
			Para solucionar esto, al elemento "blank" de este combo le ingresamos una palabra clave "__BLANK" que se asignara en el atributo "value" del <option>
		*/
		const is_blank_first_item = function() {
			return {
				id: "__BLANK",
				name: ""
			};
		};
		inConcert.Utils.fill_combo($area, is_blank_first_item, areas, attr_id, attr_name); //, sorted, can_add_item, data_id, id_is_case_sensitive, selected_value);
	};
}());
