/* jshint jquery:true, strict: true, quotmark:false
*/

/* global inConcert, Mustache */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	"use strict";

	var _T = inConcert.i18n.getString;

	/**
	* Class TicketSearchUI
	*/
	function TicketSearchUI (options)
	{
		TicketSearchUI.uber.constructor.call(this);

		this.parent = options.parent;
		this.interaction_id = options.interaction_id;
		this.search_callback = options.search_callback || $.noop;

		this.canReposition = true;

		this.ownerId = "windowTicketSearchContainer";
	}

	inConcert.Utils.inherit( TicketSearchUI, inConcert.PopupWindow );

	if ( typeof inConcert.TicketSearchUI !== "function" ) {
		inConcert.TicketSearchUI = TicketSearchUI;
	}

	TicketSearchUI.prototype.initialize = function ()
	{
		this.parent.children().remove();
		$("#windowLabelContainer").remove();
		var node = $(this.createMarkup());
		$("body").append(node);
		this.data.container = node;
		this.parent.show();
		this.setEvents();
		$("#ticket_search_data").focus();
	};

	TicketSearchUI.prototype.close = function ()
	{
		this.data.container.remove();
		this.finalize();
	};

	TicketSearchUI.prototype.createMarkup = function ()
	{
		return "".concat(
		"<div id='windowTicketSearchContainer' class='systemPopWindow windowSearchPopoverContainer windowContainer'>",
			"<div class='windowTop'>",
				//'<div class="windowTitle windowTickets">', "<%=_T('Search Tickets')%>", '</div>',
				// '<div class="windowControl windowTickets">',
				// 	'<input id="ticket_search_data" class="ticketSearchField windowControlField" type="text"/>',
				// 	'<button id="ticket_search_button" class="butTicketSearch butControlSearch buttonColorGray"></button>',
				// '</div>',

				"<div class='row'>",
					"<div class='col-12'>",
						"<span class='windowTitle'>", "<%=_T('Search Tickets')%>", "</span>",
					"</div>",
				"</div>",
				"<div class='row'>",
					"<div class='col-12'>",
						"<div class='searchFilter'>",
							"<input  id='ticket_search_data' class='ticketSearchField filterField' type='text' placeholder='", "<%=_T('Search')%>...", "' />",
							'<button id="ticket_search_button" class="buttonToggle buttonToggleOutlined search"><i class="far fa-search"></i></button>',
						"</div>",
					"</div>",
				"</div>",
			"</div>",
			"<div class='windowMiddle windowTicketsResultsContent'>",
				"<div class='row'>",
					"<div class='col-12'>",
					"</div>",
				"</div>",
			"</div>",
			"<div class='windowBottom'>",
				"<button id='closeTicketSearchDialog' class='pageButtonGeneric buttonColorBlack'>" + "<%=_T('Close')%>" + "</button>",
			"</div>",
		"</div>",
		);
	};

	TicketSearchUI.prototype.setEvents = function ()
	{
		var self = this;

		$("#closeTicketSearchDialog").click(function(){
			self.close();
		});

		$("#closeTicketSearchDialogTimes").click(function(){
			self.close();
		});

		var searchFn = this.__search.bind(this);
		$("#ticket_search_button").click(searchFn);

		$("#ticket_search_data").keyup(function(evt){
			if(evt.keyCode === 13) {
				searchFn();
				return;
			}
		});
	};

	TicketSearchUI.prototype.__render_results = function (interactions)
	{
		var interTickets = [];
		interactions.forEach(function(interaction){
			var timeStamp = inConcert.Utils.formatDate(interaction.timestampDouble * 1000);
			var interTicket = {
				type : interaction.type,
				timestamp: timeStamp,
				state: interaction.state,
				assignedAgents: interaction.assignedAgents,
				contactName: interaction.interactionData.contactName,
				campaign: interaction.campaign
			}
			interTickets.push(interTicket);
		});
		var template =
			"<ul class='searchResultsList list-unstyled'>" +
			"	{{#.}}" + // for each element in the array
			"	<li class='searchResultsItem'>" +
			"		<div class='resultData'><label class='resultLabel'>" + "<%=_T('Type')%>" + "<%=_T(':')%>" + "</label> <label class='resultInfo interactionType{{type}}'>{{type}}</label></div>" +
			"		<div class='resultData'><label class='resultLabel'>" + "<%=_T('Date')%>" + "<%=_T(':')%>" + "</label> <label class='resultInfo'>{{timestamp}}</label></div>" +
			"		<div class='resultData'><label class='resultLabel'>" + "<%=_T('State')%>" + "<%=_T(':')%>" + "</label> <label class='resultInfo interactionState{{state}}'>{{state}}</label></div>" +
			"		<div class='resultData'><label class='resultLabel'>" + "<%=_T('Assigned to')%>" + "<%=_T(':')%>" + "</label> <label  class='resultInfo'>{{assignedAgents}}</label></div>" +
			"		<div class='resultData'><label class='resultLabel'>" + "<%=_T('Contact')%>" + "<%=_T(':')%>" + "</label> <label class='resultInfo'>{{contactName}}</label></div>" +
			"		<div class='resultData'><label class='resultLabel'>" + "<%=_T('Campaign')%>" + "<%=_T(':')%>" + "</label> <label class='resultInfo'>{{campaign}}</label></div>" +
			"	</li>" +
			"	{{/.}}" +
			"	{{^.}}" + // this renders when the array is empty 
			"		<li class='noResultsAvailable'><span class='inlineMessagesContainer messageInfo d-flex align-items-center windowLabelNoResults'>" + "<%=_T('No matching conversation. Make sure you entered the whole ticket (including prefix and suffix).')%>" + "</span></li>" + 
			"	{{/.}}" +
			"</ul>";
			
		var markup = Mustache.render(template, interTickets);

		$(".windowTicketsResultsContent > div > div").html("").append(markup);

	};

	/**
	* Performs a search based on the value of the INPUT field
	*/
	TicketSearchUI.prototype.__search = function ()
	{
		let self = this;
		let search_data = $("#ticket_search_data").val().trim();
		if ( search_data.length === 0 ) {
			return;
		}
		let api = inConcert.app.getTicketsAPI();
		api.search({ interaction_id : self.interaction_id, tickets : search_data }, function(status, result) {
			if (status && result.status) {
				self.__render_results(result.interactions);
			}
			self.search_callback(result);
		});
	};

}());