/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
  strict:false, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark: false,
  smarttabs:true */

/* global inConcert */


// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	var format = inConcert.Utils.format;
	var URI_DOMAIN;

	var get_selected_account = function() {
		var $preferred_accounts = $( "#webcontact_preferred_accounts" );
		return {
			id    : $preferred_accounts.val() ,
			proxy : $( "option:selected", $preferred_accounts ).attr( "proxy" ).toUpperCase()
		};
	};

	var get_selected_contact_address = function() {
		var $contact_addresses = $( "#webcontact_contact_addresses" );
		return $( "option:selected", $contact_addresses ).data( "raw_address" );
	};

	/**
	Class WebContactClientWindow.
	Webcontact Client Handler
	*/
	function WebContactClientWindow (interaction, options)
	{
		URI_DOMAIN = inConcert.app.getBaseDomain();

		this.m_interaction = interaction;
		this.cached_contact_addresses = null;
		this.setHorizontalResizingLimits(530);
		// Must call constructor after having set m_interaction. The base class depends on that.
		var success = WebContactClientWindow.uber.constructor.call(this, options);

		if (!success) {
			return;
		}
	}

	inConcert.Utils.inherit(WebContactClientWindow, inConcert.ClientWindow);

	WebContactClientWindow.prototype.initialize = function( options )
	{
		WebContactClientWindow.uber.initialize.call( this, options ); //invokes the base class method

		if ( !this.m_interaction) {
			return;
		}

		// set tweety webcontact form controls
		this.set_tweety_form_controls();

		// add outgoing preferred accounts
		this.set_preferred_accounts();

		// set the client state
		this.set_client_state_flap();

		// tooltips
		//$( "#webcontact_contact_to" ).inconcert_tooltip( {message: "<%=_T('Resolve Appointment')%>" } ); // boton contactar

		// action handlers
		this.setEvents();

		// set read web contact
		this.set_read();

		// contact info
		this.updateContactDisplay( this.m_interaction );
	};

	// WebContactClientWindow.prototype.getMenuContextualActions = function()
	// {
	// 	const actions = {
	// 	// acciones sobre la interaccion
	// 		comment      : true,
	// 		label        : true,
	// 		emit_ticket  : true,
	// 		search_ticket: true,
	// 		queue        : true,
	// 		transfer     : true
	// 	// estas NO tienen que visualizarse en el menu, ya que las mismas se encuentran en la toolbar
	// 	};
	// 	return actions;
	// };

	WebContactClientWindow.prototype.getWindowDefaultTitle = function()
	{
		return "<%=_T('Contact Form Window')%>";
	};

	WebContactClientWindow.prototype.set_read = function()
	{
		if ( this.m_interaction.isRead() ) {
			return;
		}
		this.m_interaction.setRead( true );
		window.taskbar.onLocalInteractionUpdate( window.interaction );
	};

	WebContactClientWindow.prototype.it_is_solving = function()
	{
		var it = this.m_interaction;
		if ( !it ) {
			return false;
		}
		var pending_relationship = it.getRelationshipPendingInteraction();
		return pending_relationship !== null; // si tiene interaccion pending asociada, lo esta resolviendo.
	};

	WebContactClientWindow.prototype.getContactInfoCallback = function() {
		var self = this;
		return function() { self.solveAppointment(); };
	};

	WebContactClientWindow.prototype.set_client_state_flap = function()
	{
		var self = this;
		var class_resolving = "interactionResolving";
		var $body = $("body");
		var solving_interaction;
		var pending_relationship = this.m_interaction.getRelationshipPendingInteraction();

		if ( pending_relationship ) {
			var im = inConcert.app.getInteractionManager();
			solving_interaction = im.findInteraction( pending_relationship.id );
		}

		var on_cancel_outgoing_interaction = function() {
			unresolved();
		};

		var resolving = function() {
			$body.addClass( class_resolving );
			$("#client_state_flap").text( "<%=_T('Resolving')%>" );
			self.disable_accounts_selector();
			if ( solving_interaction ) {
				solving_interaction.on( "cancel_outgoing_interaction", on_cancel_outgoing_interaction );
			}
		};

		var unresolved = function() {
			$body.removeClass( class_resolving );
			self.enable_accounts_selector();
			if ( solving_interaction ) {
				solving_interaction.removeListener( "cancel_outgoing_interaction", on_cancel_outgoing_interaction );
			}
		};

		if ( this.it_is_solving() ) {
			resolving();
		}
		else {
			unresolved();
		}
	};

	WebContactClientWindow.prototype.disable_accounts_selector = function()
	{
		// $( "#webcontact_preferred_accounts_lbl").prop( "disabled", true );	// el label
		// $( "#webcontact_preferred_accounts" ).prop( "disabled", true );		// cbo de accounts
		// $( "#webcontact_preferred_accounts_conflict" ).hide();				// imagen de conflicto
		// $( "#webcontact_contact_to" ).prop( "disabled", true );				// btn para contactar
	};

	WebContactClientWindow.prototype.enable_accounts_selector = function()
	{
		$( "#webcontact_preferred_accounts_lbl").prop( "disabled", false );	// el label

		var $preferred_accounts = $( "#webcontact_preferred_accounts" );
		var accounts_length = $preferred_accounts.find( "option" ).length;
		var disabled_cbo = accounts_length === 1;							// si solo existe una account en el cbo, lo dejo disabled
		$preferred_accounts.prop( "disabled", disabled_cbo );				// cbo de accounts

		this.validateCampaignAndAccountOperational();						// aca si la campana es operational activa el cbo
	};

	WebContactClientWindow.prototype.disable_contact_address_selector = function()
	{
		$( "#webcontact_preferred_contact_addresses_lbl").prop( "disabled", true );	// el label
		$( "#webcontact_preferred_contact_addresses" ).prop( "disabled", true );		// cbo de accounts
	};

	WebContactClientWindow.prototype.enable_contact_address_selector = function()
	{
		$( "#webcontact_preferred_contact_addresses_lbl").prop( "disabled", false );	// el label

		var $preferred_contact_addresses = $( "#webcontact_preferred_contact_addresses" );
		var contact_addresses_length = $preferred_contact_addresses.find( "option" ).length;
		var disabled_cbo = contact_addresses_length === 1;							// si solo existe una address en el cbo, lo dejo disabled
		$preferred_contact_addresses.prop( "disabled", disabled_cbo );				// cbo de contact address
	};

	WebContactClientWindow.prototype.set_tweety_form_controls = function()
	{
		var html_content = "";

		var add_html_control = function( name, value )
		{
			html_content = html_content +
				"<div class='webContactData'>" +
					"<label class='webContactLabel'>" + ( name  || "" ) + ":</label>" +
					"<label class='webContactInfo'>"  + ( value || "" ) + "</label>" +
				"</div>";
		};

		// contact name
		var contact_name = this.m_interaction.getGatheredClientName() || '';
		if ( contact_name.length > 0 ) {
			add_html_control( "<%=_T('Contact Name')%>", contact_name );
		}

		// contact id
		var contact_id = this.m_interaction.getClientExternalId() || '';
		if ( contact_id.length > 0 ) {
			add_html_control( "<%=_T('Contact Id')%>", contact_id );
		}

		// add addresses
		var addresses = this.m_interaction.getExternalContactAddresses();
		const visualize_address = inConcert.app.canVisualizeAddress(null, this.m_interaction.campaign, null, true);
		$.each( addresses || {}, function( _, address ) {

			let addressDisplayable = address.displayable;
			if(addressDisplayable !== "" && !visualize_address)
			{
				addressDisplayable = "<%=_T('Hidden Address')%>";
			}
			add_html_control( address.Type, addressDisplayable );
		});

		// add fields
		var fields = this.m_interaction.getReportFields();
		$.each( fields || {}, function( _, field ) {
			if (!field.hidden) {
				add_html_control( field.displayName, field.value );
			}
		});

		if ( html_content.length === 0 ) {
			html_content = "<div class='internalInfoMessage'>" + "<%=_T('No controls found.')%>" + "</div>";
		}

		$( "#webcontact_tweety_form_controls" ).html( html_content );
	};

	WebContactClientWindow.prototype.set_preferred_accounts = function()
	{
		var campaign = this.getCampaign( this.m_interaction.campaign );
		if ( !campaign ) {
			this.disable_accounts_selector();
			return;
		}
		var account_id = this.m_interaction.getAccount();
		var account = campaign.getAccount( account_id, this.m_interaction.type );
		if ( !account ) {
			this.disable_accounts_selector();
			return;
		}
		var preferred_accounts = campaign.getPreferredAccounts( account );
		if ( !preferred_accounts ) {
			this.disable_accounts_selector();
			return;
		}
		var $preferred_accounts = $("#webcontact_preferred_accounts");
		var default_account = preferred_accounts.DefaultAccount;
		var $default_option;
		var added_accounts = 0;

		// fill combo
		$.each( preferred_accounts.Accounts || {}, function( _, acc )
		{
			var preferred_account = campaign.getAccount( acc.id, acc.proxy );
			if ( !preferred_account ) {
				return true; // continue
			}

			var $option = $( "<option></option>" );

			$option
				.attr( "value", preferred_account.id                       )
				.attr( "proxy", preferred_account.proxyType                )
				.text( campaign.getAccountDescription( preferred_account ) );

			if ( default_account ===  preferred_account.id ) {
				$default_option = $option;
			}

			$preferred_accounts.append( $option );
			added_accounts++;
		});

		if ( added_accounts > 0 ) {
			// sorted preferred accounts
			var cbo_id = $preferred_accounts.attr("id") || "";
			inConcert.Utils.sortOptions( cbo_id );

			if ( added_accounts > 1 && $default_option ) {
			// mark a default option
				$default_option.attr( "selected", "selected" );
				var description = $default_option.text();
				description = description + " " + "<%=_T(' ( Default ) ')%>";
				$default_option.text( description );
			}

			this.enable_accounts_selector();
			var selected_account = get_selected_account();
			this.fill_cbo_contact_addresses( selected_account.proxy );
		}
		else {
			this.disable_accounts_selector();
		}
	};

	WebContactClientWindow.prototype.fill_cbo_contact_addresses = function( proxy )
	{
		var self = this;

		proxy = ( proxy || "" ).toUpperCase();

		var fill = function( addresses )
		{
			var $contact_addresses = $("#webcontact_contact_addresses");
			var added_addresses = 0;

			$contact_addresses.html("");

			addresses = addresses || {};

			// fill combo
			$.each( addresses, function( _, address )
			{
				if ( ( address.Type || "" ).toUpperCase() !== proxy ) {
					return true; // continue
				}
				var address_name = address.displayable || address.address;
				var $option = $( "<option></option>" )
									.attr( "value", address.address  )
									.attr( "proxy", proxy            )
									.text( address_name              )
									.data( "raw_address", address    );

				$contact_addresses.append( $option );
				added_addresses++;
			});

			self.cached_contact_addresses = addresses;

			if ( added_addresses > 0 ) {
				self.enable_contact_address_selector();
			}
			else {
				self.disable_contact_address_selector();
			}
		};

		if ( this.cached_contact_addresses ) {
			fill( this.cached_contact_addresses );
			return;
		}

		var contact_manager = inConcert.app.getContactManager();
		var contact_address = this.findFirstContactableAddressOnContactFormData( proxy );
		if ( contact_address ) {
			// 1 - si no me vino un contact id en la interaccion, busco en la direcciones que parseo, el contact form (tweety)
			//   luego de obtenerla, busco el address en la BD por address, por las dudas de que ya se haya registrado en la BD.
			//   luego abro el cliente con la informacion.
			contact_manager.getContactByAddress( contact_address,
				function( contact )
				{
					var hash_added_accounts = {};
					var addresses = [];
					var add = function( address ) {
						if ( hash_added_accounts[ address.displayable ] ) {
							return;
						}
						hash_added_accounts[ address.displayable ] = true;
						addresses.push( address );
					};

					var external_contact_addresses = self.m_interaction.getExternalContactAddresses();
					// primero las que llegaron en el frm
					$.each( external_contact_addresses, function( _, ext_address ) {
						add( ext_address );
					});

					// despues las del contacto
					if ( contact && contact.addresses ) {
						$.each( contact.addresses, function( _, address ) {
							add( address );
						});
					}

					fill( addresses );
				});
		}
		else {
			// 2 - si me vino el contact Id en la interaccion, es un contacto existente en la BD, entonces realizo request
			//  para obtener el contact desde la BD
			var contact_id = this.m_interaction.contact; // email20c7986f218843d88f6ab2dbec5c9747@inconcert
			if ( ( contact_id || "" ).length === 0 ) {
				// error!!!
				fill(); // si esta vacio limpia el cbo
				return;
			}
			contact_manager.getContact(
				contact_id,
				function( contact ) {
					if ( !contact ) {
						// error!!!
						fill(); // si esta vacio limpia el cbo
						return;
					}
					fill( contact.addresses ); // si esta vacio limpia el cbo
				}
			);
		}
	};

	WebContactClientWindow.prototype.validateCampaignAndAccountOperational = function()
	{
		WebContactClientWindow.uber.validateCampaignAndAccountOperational.call( this ); //invokes the base class method

		// aca es la validacion unicamente de las accounts preferidas, configuradas en la account de webcontact
		var $preferred_accounts = $( "#webcontact_preferred_accounts" );
		var account_id  = $preferred_accounts.val();
		if ( !account_id ) {
			this.disable_accounts_selector();
			return;
		}
		var $conflict   = $( "#webcontact_preferred_accounts_conflict" );
		var account_is_operational = this.m_interaction.accountIsOperational( account_id );

		var disabled_contact_to = false;
		if ( !account_is_operational.success ) {
			this.showConflictAccountIcon( account_is_operational.errorMessage, $conflict );
		}
		else {
			this.hideConflictAccountIcon( $conflict );
			var is_solving = this.it_is_solving();
			if ( !is_solving ) {
				disabled_contact_to = false;
			}
		}
		$( "#webcontact_contact_to" ).prop( "disabled", disabled_contact_to ); // boton contact to
	};

	WebContactClientWindow.prototype.setEvents = function()
	{
		var self = this;

		var $preferred_accounts = $( "#webcontact_preferred_accounts" );
		$( "#webcontact_contact_to" ).on("click", function() {
			self.solveAppointment();
		});

		$preferred_accounts.on("change", function() {
			self.validateCampaignAndAccountOperational();
			var proxy = $( "option:selected", $( this ) ).attr( "proxy" );
			self.fill_cbo_contact_addresses( proxy );
		});
	};

	/**
	* Given an account type (mail, phone, twitter, etc), look for the first address within the contact form data that
	* matches the given type.
	*
	* @param {String} account_type
	*/
	WebContactClientWindow.prototype.findFirstContactableAddressOnContactFormData = function( account_type )
	{
		var addresses = this.m_interaction.getExternalContactAddresses();
		var contact_address;
		$.each( addresses || {}, function( _, address )
		{
			if ( address.Type.toUpperCase() === account_type ) {
				contact_address = address;
				return false; // break
			}
		});
		return contact_address;
	};

	WebContactClientWindow.prototype.OpenRelationWindowClient = function( contact )
	{
		var campaign_id = this.m_interaction.campaign;
		var campaign = inConcert.app.getCampaign( campaign_id );
		if ( !campaign ) {
			this.displayStatusMessage(
				format("<%=_T('{0} campaign is not available.')%>", campaign_id),
				"Error",
				false,
				5000);
			return;
		}
		// funcion que abre el cliente relacionado (mail, etc)
		var campaigns       = [ campaign ];
		var account         = get_selected_account();
		var contact_address = get_selected_contact_address();

		var panel = inConcert.app.getMainWindow().getPanel("AssignedInteractions");
		var response = panel.createOutgoingInteraction(
			account.proxy     ,
			contact           ,
			contact_address   ,
			campaigns         ,
			this.m_interaction,
			undefined         , /* window session */
			account.id
		);

		if (response.success) {
			this.close();
		}
		else {
			if (response.conflicts) {
				this.displayStatusMessage(
					response.conflicts.message,
					"Error",
					false,
					5000);
				return;
			}
		}
	};

	WebContactClientWindow.prototype.solveAppointment = function()
	{
		var self = this;
		var it = this.m_interaction;
		var campaign_id = it.campaign;
		var campaign = inConcert.app.getCampaign( campaign_id );

		if ( !campaign ) {
			this.displayStatusMessage(
				format("<%=_T('{0} campaign is not available.')%>", campaign_id),
				"Error",
				false,
				5000);
			return;
		}

		var account = get_selected_account();
		var contact_manager = inConcert.app.getContactManager();
		var contact_address = this.findFirstContactableAddressOnContactFormData( account.proxy );
		if ( contact_address ) {
			// 1 - si no me vino un contact id en la interaccion, busco en la direcciones que parseo, el contact form (tweety)
			//   luego de obtenerla, busco el address en la BD por address, por las dudas de que ya se haya registrado en la BD.
			//   luego abro el cliente con la informacion.
			contact_manager.getContactByAddress(contact_address,
				function( contact ) {
					self.OpenRelationWindowClient( contact );
				});
		}
		else {
			// 2 - si me vino el contact Id en la interaccion, es un contacto existente en la BD, entonces realizo request
			//  para obtener el contact desde la BD
			var contact_id = it.contact;
			if ( !contact_id ) {
				this.displayStatusMessage(
					"<%=_T('Unable to resolve contact form, contact doesnt have email addresses.')%>",
					"Error",
					false,
					5000);
				return;
			}
			contact_manager.getContact(
				contact_id,
				function( contact )
				{
					if ( !contact ) {
						self.displayStatusMessage(
							"<%=_T('Unable to resolve contact form, contact is not available.')%>",
							"Error",
							false,
							5000);
						return;
					}
					self.OpenRelationWindowClient( contact );
				}
			);
		}
	};

	WebContactClientWindow.prototype.dispose = function()
	{
		WebContactClientWindow.uber.dispose.call( this ); //invokes the base class method
	};

	WebContactClientWindow.prototype.invalidInteraction = function() {
		WebContactClientWindow.uber.invalidInteraction.call( this ); //invokes the base class method
	};

	WebContactClientWindow.prototype.onInteractionGone = function(interaction) {
		WebContactClientWindow.uber.onInteractionGone.call( this, interaction ); //invokes the base class method
		$("body").addClass("clientReadOnly");
	};

	WebContactClientWindow.prototype.change_client_state = function( new_state )
	{
		new_state = WebContactClientWindow.uber.change_client_state.call( this, new_state );

		var $solve_container = $("#webcontact_solve_container");

		switch( new_state )
		{
			case "archived":
				$solve_container.hide();
			break;

			case "connected":
				$solve_container.show();
			break;

			case "monitoring":
				$solve_container.hide();
			break;

			case "pending":
				$solve_container.hide();
			break;
		}

		return new_state;
	};

	//when jquery says so, start it all up
	$(document).ready(function() {
		var client;
		window.hook_environment();
		inConcert.i18n.updateScreen();

		client = new WebContactClientWindow(window.interaction,
			{
				archive  : window.archive,
				from_app : window.from_app,
				monitor  : window.monitor,
				queued   : window.queued,
				waiting  : window.waiting,
				readonly : window.readonly || window.archive || window.monitor || window.queued || window.waiting
			}
		);

		window.onReady(client);

		if (window && window.view && window.view instanceof inConcert.ClientWindowProxy) {
			window.view.apply(client);
		}

		window.view = client;
	});

}());
