///////////////////////////////////////////////////////////////////////////////////////////
//helpers
function IsTypeOf(obj, key) {
 return((obj.constructor.toString().indexOf(key) === -1)?false:true);
}

function IsNumeric(sText) {
	return((sText.match(/^\d+$/) === null)?true:false);
}

function IsObject (value) {
	return value && typeof value === 'object' && value.constructor === Object;
}

function IsArray (value) {
	return value && typeof value === 'object' && value.constructor === Array;
}

function IsJson(item) {
    item = typeof item !== "string"
        ? JSON.stringify(item)
        : item;

    try {
        item = JSON.parse(item);
    } catch (e) {
        return false;
    }

    return (typeof item === "object" && item !== null);
}

String.prototype.trim = function(){ return this.replace(/^\s+|\s+$/g,''); };

///////////////////////////////////////////////////////////////////////////////////////////
//Public Object ICControl
function ICControl(callback) {
	var m_callback = callback;
	var m_bridgeready = false;

	function BindEvent(element, eventName, eventHandler) {
		if (element.addEventListener) {
			element.addEventListener(eventName, eventHandler, false);
		} else if (element.attachEvent) {
			element.attachEvent('on' + eventName, eventHandler);
		}
	}

	function ProcessResponse(response) {
		if (IsObject(response.data) === false) 
			return;

		var result = response.data.result;
		var data;
		if (IsObject(result)) {
			data = {type:"object", value:result};
		} else if (IsArray(result)) {
			data = {type:"array", value:result};
		} else if (IsJson(result)) {
		 	data = {type:"object", value:JSON.parse(result)};
		} else if (result === "true" || result === "false") {
		 	data = {type:"boolean", value: result === "true"};
		} else {
			data = {type:"text", value: result};
		}

		if (response.data.messageId === "response_bridgeready") {
			m_bridgeready = true;
			return;
		}

		var message = {};
		message.uniqueId = response.data.uniqueId;
		message.messageId = response.data.messageId;
		message.type = data.type;
		message.data = data.value;
	
		if(typeof(m_callback) === "function") {
			m_callback(message);
		}

	}

	// Send a message to the parent
	function SendMessage(msg) {
		// Make sure you are sending a string, and to stringify JSON
		window.parent.postMessage(msg, '*');
	};

	// Listen to messages from parent window
	BindEvent(window, 'message', ProcessResponse);

	var bridgeready_timer = setInterval(function() {
		if (m_bridgeready) {
			clearInterval(bridgeready_timer);
			return;
		}
		var jMessage = [];
		jMessage.action = "bridgeready";
		jMessage.uniqueId = 0;
		jMessage.parameters = [];

		SendMessage(jMessage);
	}, 1000);

	///////////////////////////////////////////////////////////////////////////////////////////
	//Public Method TakeNextBest
	this.TakeNextBest = function(uniqueId, callback) {

		var jMessage = [];
		jMessage.action = "takenextbest";
		jMessage.uniqueId = uniqueId || 0;
		jMessage.parameters = [];

		SendMessage(jMessage);
	};

	///////////////////////////////////////////////////////////////////////////////////////////
	//Public Method Interaction_Requeue
	this.Interaction_Requeue = function(interactionId, uniqueId, callback) {

		var jMessage = [];
		jMessage.action = "interaction_requeue";
		jMessage.uniqueId = uniqueId || 0;
		jMessage.parameters = [];
		jMessage.parameters.interactionId = interactionId;

		SendMessage(jMessage);
	};

	///////////////////////////////////////////////////////////////////////////////////////////
	//Public Method Interaction_Comment
	this.Interaction_Comment = function(interactionId, comment, uniqueId, callback) {

		var jMessage = [];
		jMessage.action = "interaction_comment";
		jMessage.uniqueId = uniqueId || 0;
		jMessage.parameters = [];
		jMessage.parameters.interactionId = interactionId;
		jMessage.parameters.comment = comment;

		SendMessage(jMessage);
	};	

	///////////////////////////////////////////////////////////////////////////////////////////
	//Public Method Interaction_Get
	this.Interaction_Get = function(interactionId, uniqueId, callback) {

		var jMessage = [];
		jMessage.action = "interaction_get";
		jMessage.uniqueId = uniqueId || 0;
		jMessage.parameters = [];
		jMessage.parameters.interactionId = interactionId;

		SendMessage(jMessage);
	};		

	///////////////////////////////////////////////////////////////////////////////////////////
	//Public Method Interaction_Archive
	this.Interaction_Archive = function(interactionId, disposition_param, comment, uniqueId, callback) {

		var jMessage = [];
		jMessage.action = "interaction_archive";
		jMessage.uniqueId = uniqueId || 0;
		jMessage.parameters = [];
		jMessage.parameters.interactionId = interactionId;
		jMessage.parameters.disposition_param = disposition_param;
		jMessage.parameters.comment = comment;

		SendMessage(jMessage);
	};	

	///////////////////////////////////////////////////////////////////////////////////////////
	//Public Method Interaction_GetParameter
	this.Interaction_GetParameter = function(interactionId, parameter, uniqueId, callback) {

		var jMessage = [];
		jMessage.action = "interaction_getparameter";
		jMessage.uniqueId = uniqueId || 0;
		jMessage.parameters = [];
		jMessage.parameters.interactionId = interactionId;
		jMessage.parameters.parameter = parameter;

		SendMessage(jMessage);
	};

	///////////////////////////////////////////////////////////////////////////////////////////
	//Public Method Interaction_GetDispositionCodes
	this.Interaction_GetDispositionCodes = function(interactionId, uniqueId, callback) {

		var jMessage = [];
		jMessage.action = "interaction_getdispositioncodes";
		jMessage.uniqueId = uniqueId || 0;
		jMessage.parameters = [];
		jMessage.parameters.interactionId = interactionId;

		SendMessage(jMessage);
	};	

	///////////////////////////////////////////////////////////////////////////////////////////
	//Public Method Interaction_GetMessages (chat)
	this.Interaction_GetMessages = function(interactionId, uniqueId, callback) {

		var jMessage = [];
		jMessage.action = "interaction_getmessages";
		jMessage.uniqueId = uniqueId || 0;
		jMessage.parameters = [];
		jMessage.parameters.interactionId = interactionId;

		SendMessage(jMessage);
	};	

	///////////////////////////////////////////////////////////////////////////////////////////
	//Public Method Interaction_GetEventList (facebook, twitter)
	this.Interaction_GetEventList = function(interactionId, uniqueId, callback) {

		var jMessage = [];
		jMessage.action = "interaction_geteventlist";
		jMessage.uniqueId = uniqueId || 0;
		jMessage.parameters = [];
		jMessage.parameters.interactionId = interactionId;

		SendMessage(jMessage);
	};		

	///////////////////////////////////////////////////////////////////////////////////////////
	//Public Method User_GetCompanyInformation
	this.User_GetCompanyInformation = function(uniqueId, callback) {

		var jMessage = [];
		jMessage.action = "user_getcompanyinformation";
		jMessage.uniqueId = uniqueId || 0;
		jMessage.parameters = [];

		SendMessage(jMessage);
	};	

	///////////////////////////////////////////////////////////////////////////////////////////
	//Public Method User_GetCustomInformation
	this.User_GetCustomInformation = function(uniqueId, callback) {

		var jMessage = [];
		jMessage.action = "user_getcustominformation";
		jMessage.uniqueId = uniqueId || 0;
		jMessage.parameters = [];

		SendMessage(jMessage);
	};	

	///////////////////////////////////////////////////////////////////////////////////////////
	//Public Method User_GetCompleteName
	this.User_GetCompleteName = function(uniqueId, callback) {

		var jMessage = [];
		jMessage.action = "user_getcompletename";
		jMessage.uniqueId = uniqueId || 0;
		jMessage.parameters = [];

		SendMessage(jMessage);
	};	

	///////////////////////////////////////////////////////////////////////////////////////////
	//Public Method User_GetFirstName
	this.User_GetFirstName = function(uniqueId, callback) {

		var jMessage = [];
		jMessage.action = "user_getfirstname";
		jMessage.uniqueId = uniqueId || 0;
		jMessage.parameters = [];

		SendMessage(jMessage);
	};	

	///////////////////////////////////////////////////////////////////////////////////////////
	//Public Method User_GetLastName
	this.User_GetLastName = function(uniqueId, callback) {

		var jMessage = [];
		jMessage.action = "user_getlastname";
		jMessage.uniqueId = uniqueId || 0;
		jMessage.parameters = [];

		SendMessage(jMessage);
	};	

	///////////////////////////////////////////////////////////////////////////////////////////
	//Public Method User_GetFirstSurname
	this.User_GetFirstSurname = function(uniqueId, callback) {

		var jMessage = [];
		jMessage.action = "user_getfirstsurname";
		jMessage.uniqueId = uniqueId || 0;
		jMessage.parameters = [];

		SendMessage(jMessage);
	};	

	///////////////////////////////////////////////////////////////////////////////////////////
	//Public Method User_GetLastSurname
	this.User_GetLastSurname = function(uniqueId, callback) {

		var jMessage = [];
		jMessage.action = "user_getlastsurname";
		jMessage.uniqueId = uniqueId || 0;
		jMessage.parameters = [];

		SendMessage(jMessage);
	};	

	///////////////////////////////////////////////////////////////////////////////////////////
	//Public Method User_GetName
	this.User_GetName = function(uniqueId, callback) {

		var jMessage = [];
		jMessage.action = "user_getname";
		jMessage.uniqueId = uniqueId || 0;
		jMessage.parameters = [];

		SendMessage(jMessage);
	};	

	///////////////////////////////////////////////////////////////////////////////////////////
	//Public Method Window_Close
	this.Window_Close = function(uniqueId, callback) {

		var jMessage = [];
		jMessage.action = "window_close";
		jMessage.uniqueId = uniqueId || 0;
		jMessage.parameters = [];

		SendMessage(jMessage);
	};	

}

