module(..., package.seeall, inconcert.orbit.application.Decorate)

local utils = require "shared.apps.utils"
local proxy = require "apps.allegro.proxy"

--[[
	FUNCIONES QUE SE INVOCAN DE FORMA EXTERNA (NO VA CONTRA UNA URI)
	LAS MISMAS SON INVOCADAS DESDE LA APP OUTBOUNDENGINE DE I6
]]--
function LoadPaginatedRoutingGroups (encodedInfo)
	LogInfo("[ALLEGRO.OUTBOUNDENGINE:LoadPaginatedRoutingGroups] Loading Routing Groups.")

	local success, data = proxy.administration.LoadPaginatedRoutingGroups(encodedInfo)
	if success and data then
		LogInfo("[OUTBOUNDENGINE_EXPORTS:LoadPaginatedRoutingGroups] Successfully loaded %d of %s Routing Groups.", #data.rows, data.total_rows)
		return true, data
	end
	-- Error
	local err = data
	local errType = {
		type = "ALLEGRO_INTEGRATION_ERROR",
		err  = type(err) == "table" and err.reason or err or "ERROR_LOADING_ROUTING_GROUPS"
	}
	return false, errType
end

function GetRoutingGroup (routingGroupId)
	LogInfo("[ALLEGRO.OUTBOUNDENGINE:GetRoutingGroup] Retrieving Routing Group: '%s'.", routingGroupId)

	local response, err = proxy.administration.GetRoutingGroup(routingGroupId)

	if response then
		local routingGroup = response.routingGroup
		LogInfo("[OUTBOUNDENGINE_PROCESSES:GetRoutingGroup] Successfully retrieving %s Routing Group. Info: %s", routingGroupId, routingGroup)
		return true, routingGroup
	end
	-- Error
	local errType = {
		type = "ALLEGRO_INTEGRATION_ERROR",
		err  = type(err) == "table" and err.reason or err or "ERROR_RETRIEVING_ROUTING_GROUP"
	}
	LogError("[OUTBOUNDENGINE_PROCESSES:GetRoutingGroup] Error retrieving %s Routing Group. Info: %s", routingGroupId, errType)
	return false, errType
end

function SaveRoutingGroup (encodedInfo)
	LogInfo("[ALLEGRO.OUTBOUNDENGINE:SaveRoutingGroup] Saving Routing Group. %s", encodedInfo)

	local success, data = proxy.administration.SaveRoutingGroup(encodedInfo)
	--LogDebug("---> SaveRoutingGroup success: %s, data: %s", success, data)
	if success then
		LogInfo("[OUTBOUNDENGINE_EXPORTS:SaveRoutingGroup] Successfully saved %s Routing Groups.", encodedInfo)
		return true
	end
	-- Error
	local err = data
	local errType = {
		type = "ALLEGRO_INTEGRATION_ERROR",
		err  = type(err) == "table" and err.reason or err or "ERROR_SAVING_ROUTING_GROUPS"
	}
	return false, errType
end

function DeleteRoutingGroup (encodedInfo)
	LogInfo("[ALLEGRO.OUTBOUNDENGINE:DeleteRoutingGroup] Deleting Routing Group. %s", encodedInfo)

	local success, data = proxy.administration.DeleteRoutingGroup(encodedInfo)
	--LogDebug("---> DeleteRoutingGroup success: %s, data: %s", success, data)
	if success then
		LogInfo("[OUTBOUNDENGINE_EXPORTS:DeleteRoutingGroup] Successfully deleted %s Routing Groups.", encodedInfo)
		return true
	end
	-- Error
	local err = data
	local errType = {
		type = "ALLEGRO_INTEGRATION_ERROR",
		err  = type(err) == "table" and err.reason or err or "ERROR_DELETING_ROUTING_GROUPS"
	}
	return false, errType
end

--[[
	FUNCIONES QUE SE MAPEAN A UNA URI DETERMINADA
]]--
local function __RoutingGroupCheckMaxAssigned (web, routingGroupId)
	LogInfo("[ALLEGRO.OUTBOUNDENGINE:__RoutingGroupCheckMaxAssigned] 1 Check Capacity assigned for routing group: '%s'.", routingGroupId)

	local result, err = proxy.administration.RoutingGroupCheckMaxAssigned(routingGroupId)
	if result then
		return json.encode(result)
	end

	web.status = 404
	-- Error
	local errType = {
		type = "ALLEGRO_INTEGRATION_ERROR",
		err  = type(err) == "table" and err.reason or err or "ERROR_CHECK_MAX_ASSIGNED_ROUTING_GROUPS"
	}
	return errType
end

function MapUris (module, htmlRender, prefix)
	local LogRequest = require("apps.agent.requestFilters").LogRequest
	prefix = "/routinggroup"
	module:dispatch_get(utils.JsonReply .. __RoutingGroupCheckMaxAssigned, prefix .. "/check_max_assigned/([^/]*)")
end
