module(..., package.seeall, inconcert.orbit.application.Decorate)

local proxy = require "apps.allegro.proxy"
local utils = require "shared.apps.utils"

local function __ValidateDialingRule (web, routingGroupId)
	local encodedData = web.input["post_data"]

	LogInfo("[ALLEGRO:__ValidateDialingRule] Asterisk validating Dialing Rule for routing group '%s'. Data: %s.", routingGroupId, encodedData)

	local success, result = proxy.asterisk.ValidateDialingRule(encodedData, routingGroupId)
	LogDebug("[ALLEGRO:__ValidateDialingRule] success: '%s', result: '%s'", success, json.encode(result or {}))
	--result.success = result.ok
	if success then
		LogInfo("[ALLEGRO:__ValidateDialingRule] Successfully Asterisk vakidated Dialing Rule for routing group '%s'. Result.", routingGroupId, json.encode(result))
		return result
	end

	-- Error
	local err = result
	web.status = 500
	local err_type = {
		type = "ALLEGRO_INTEGRATION_ERROR",
		err  = type(err) == "table" and err.reason or err or "ERROR_ASTERISK_VALIDATING_DIALING_RULE"
	}
	LogError("[ALLEGRO:__ValidateDialingRule] Error Asterisk validating Dialing Rule for routing group '%s' Err: %s", routingGroupId, json.encode(err_type))

	return err_type
end

local function __DeleteDialingRule (web, routingGroupId, dialingRuleId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[ALLEGRO:__DeleteDialingRule] User '%s' Deleting Dialing Rule: '%s' in Routing Group: '%s'.", userId, routingGroupId, dialingRuleId)

	local response, err = proxy.asterisk.DeleteDialingRule(routingGroupId, dialingRuleId)
	LogDebug("[ALLEGRO:__DeleteDialingRule] success: '%s', response: '%s'", response, err)

	if response then
		return true, response
	else
		local err = response
		web.status = 500
		local err_type = {
			type = "ALLEGRO_INTEGRATION_ERROR",
			err  = type(err) == "table" and err.reason or err or "ERROR_ASTERISK_DELETING_DIALING_RULE"
		}
		LogError("[ALLEGRO:__DeleteDialingRule] Error Deleting Dialing Rule: '%s' in Routing Group: '%s'. Error: '%s'", routingGroupId, dialingRuleId, json.encode(err_type))
		return false, err_type
	end
end

function MapUris (module, htmlRender, prefix)
	local LogRequest = require("apps.agent.requestFilters").LogRequest
	module:dispatch_post(LogRequest .. utils.JsonValidatorReply .. __ValidateDialingRule, (prefix .. "/validate/dialing_rule/([^/]*)"))
	module:dispatch_delete(utils.JsonReply .. __DeleteDialingRule, (prefix .. "/delete/dialing_rule/([^/]*)/([^/]*)"))
end
