local utils = require "shared.apps.utils"
local permissions = require "shared.permissions.permissions"
local allegro = require "apps.allegro.proxy"
local m_models = require("apps.allegro.model"):GetModels()
local templates = require "apps.allegro.templates"

module(..., package.seeall, inconcert.orbit.application.Decorate)

m_htmlRender = nil
m_appLocation = nil

function MapUris (module, htmlRender, prefix, appLocation)
	m_htmlRender = htmlRender
	m_appLocation = appLocation
end

function HasIntegration (vcc_id) return allegro.HasIntegration(vcc_id) end

local function GetModel (entity)
	if not m_models or not m_models[entity] then
	-- Si el modelo no se ha creado aun, debo crearlo.
	--  Esto sucede en algunas ocasiones si es que me han invocado desde otra app Ej: "Allegro" y aun no se ha creado el modelo.
	--  Entonces tengo que crear el modelo.
		local model = require "apps.allegro.model"
		model.CreateModelTable()
		m_models = model:GetModels()
	end
	return m_models[entity]
end

--[[
	Esta función se encarga de actualizar la información que va a ser enviada a 5.5, para la actualización de la campaña.
	Sin importar que sección se haya querido almacencar desde i6 (Ej: basic info, scheduler, integration, etc.).
	Lo anterior se realiza así, ya que en 5.5 siempre se necesita toda la información de las campañas para poder almacenar cambios en las campañas, ya que
	el LuaOrm, manda a guardar todas las tablas relacionadas sin importar si tiene o no cambios.
]]--
local function __UpdateCampaignMemoryInfo (omnichannel_campaign, vcc_id, allegro_info)

	local count_accounts = #omnichannel_campaign.accounts
	--LogDebug("----> __UpdateCampaignMemoryInfo -> accounts: %d, allegro_info: %s", count_accounts, json.encode(allegro_info or {}))
	local response = { can_update_55 = false, type = "ALLEGRO_INTEGRATION_ERROR", sub_type = "" }

	local function __ValidateTelephonyCampaign()
		if count_accounts > 1 then
		-- NO se va a actualizar la campana en Allegro.
			-- NO ES UN ERROR, si tiene mas de una cuenta quiere decir que no es de telefonia/integracion
			response.type = "NOT_A_ERROR"
		else
			local current_account = omnichannel_campaign.accounts[1]
			if not current_account or current_account.proxyType ~= "CALL" then
			-- NO se va a actualizar la campana en Allegro.
				-- NO ES UN ERROR, si la cuenta actual no es de telefonia, no es un error
				response.type = "NOT_A_ERROR"
			else
			-- SI se va a actualizar la campana en Allegro.
				response.can_update_55 = true
				response.type = ""
			end
		end
		return response.can_update_55
	end

	--[[
		Informacion adicional que tiene que viajar en el POST.
		Ej: La IP del Server Remoto que se utiliza en el history.
	]]--
	if allegro_info.otherInfo then
		omnichannel_campaign.otherInfo = allegro_info.otherInfo
		omnichannel_campaign.otherInfo.action = allegro_info.action_id
	end
	
	--[[
		Las actualizaciones de "BASIC_INFO", "SAVE_SCHEDULE" y "UPDATE_MEMBERS",
			se realizan siempre sin importar si la campana es o no de telefonia.
	]]--
	if allegro_info.action_id == "BASIC_INFO" then
		-- El basic info de la campana se actualiza siempre sin importar si la misma es de integracion con Allegro o no

		local exclude_columns = { Integration = true }
		-- actualizo unicamente los campos discretos (string, number, etc)
		for name, value in pairs(allegro_info.campaign) do
			if type(value) ~= "table" and not exclude_columns[name] then
				omnichannel_campaign[name] = value
			end
		end
		response.can_update_55 = true
		response.type = ""

	elseif allegro_info.action_id == "SAVE_SCHEDULE" then
		-- El scheduler de la campana se actualiza siempre sin importar si la misma es de integracion con Allegro o no
		omnichannel_campaign.scheduler = allegro_info.scheduler
		response.can_update_55 = true
		response.type = ""

	elseif allegro_info.action_id == "UPDATE_MEMBERS" then
		-- Los members de la campana se actualizan siempre sin importar si la misma es de integracion con Allegro o no

		-- Aca actualizo con los atributos que necesita la app "omnichannelintegration" en el mapeo campos para los members
		local new_members = {}
		for member_id, member_info in pairs(allegro_info.members) do
			if (member_info.action or ""):lower() ~= "removed" then
				local full_member_id = utils.AppendVcc(member_id, vcc_id)
				local new_member = {
					Id             = full_member_id,
					Name           = member_id,
					FirstSurname   = member_id,
					LastSurname    = member_id,
					LastName       = member_id,
					UserGroup      = member_info.member.role,
					Role           = member_info.member.role, --xxxx
					apps_permisson = member_info.member.apps_permisson,
					supervisor_workgroups = member_info.member.supervisor_workgroups
				}
				table.insert(new_members, new_member)
			end
		end

-- LogDebug("1 - %s", json.encode(allegro_info.members))
-- LogDebug("2 - %s", json.encode(omnichannel_campaign.Members))
-- LogDebug("3 - %s", json.encode(new_members))

		omnichannel_campaign.Members = new_members
		response.can_update_55 = true
		response.type = ""

--[[
	Las actualizaciones de "SAVE_ACCOUNT", "SAVE_THRESHOLDS", "SAVE_DISPOSITIONS", "SAVE_INTEGRATION" y "REMOVE_INTEGRATION", "SAVE_SCHEDULE"
		se realizan unicamente si la campana es de telefonia.
]]--

	elseif allegro_info.action_id == "SAVE_ACCOUNT" then

		if allegro_info.is_new then
		-- AGREGO UNA NUEVA CUENTA
			omnichannel_campaign.accounts = omnichannel_campaign.accounts or {}
			if #omnichannel_campaign.accounts > 0 then
				if allegro_info.account.proxyType == "CALL" then
					response.sub_type = "ALLOWED_ONLY_ONE_CALL_ACCOUNT"
					return response
				else
				-- NO ES UN ERROR, ESTA AGREGANDO UNA CUENTA QUE NO ES CALL
					response.type = "NOT_A_ERROR"
					return response
				end
			else
				if allegro_info.account.proxyType == "CALL" then
				--- OK --
					table.insert(omnichannel_campaign.accounts, allegro_info.account)
					response.can_update_55 = true
					response.type = ""
				else
				-- NO ES UN ERROR, ESTA AGREGANDO UNA CUENTA QUE NO ES CALL
					response.type = "NOT_A_ERROR"
					return response
				end
			end
		else
		-- ACTUALIZO LA CUENTA
			local current_account = omnichannel_campaign.accounts[1]
			if current_account.proxyType ~= "CALL" and allegro_info.account.proxyType ~= "CALL" then 
				-- NO ES UN ERROR, ESTA ACTUALIZANDO UNA CUENTA QUE NO ES CALL
				response.type = "NOT_A_ERROR"
				return response
			elseif current_account.proxyType == "CALL" and allegro_info.account.proxyType == "CALL" then
			--- OK --
				table.remove(omnichannel_campaign.accounts, 1)
				table.insert(omnichannel_campaign.accounts, allegro_info.account)
				response.can_update_55 = true
				response.type = ""
			else
			-- aca uno de los 2 es CALL
				response.sub_type = "UNABLE_CHANGE_CALL_ACCOUNT"
				return response
			end
		end

	elseif allegro_info.action_id == "SAVE_ACCOUNT_MESSAGES" then
		__ValidateTelephonyCampaign()
		if response.can_update_55 then
			omnichannel_campaign.AccountMessages = allegro_info.AccountMessages
		end

	elseif allegro_info.action_id == "SAVE_THRESHOLDS" then
		__ValidateTelephonyCampaign()
		if response.can_update_55 then
			omnichannel_campaign.Thresholds = allegro_info.thresholds
		end

	elseif allegro_info.action_id == "SAVE_DISPOSITIONS" then
		__ValidateTelephonyCampaign()
		if response.can_update_55 then
			omnichannel_campaign.dispositionCodes = allegro_info.dispositions
		end

	elseif allegro_info.action_id == "SAVE_INTEGRATION" or allegro_info.action_id == "REMOVE_INTEGRATION" then
		__ValidateTelephonyCampaign()
		if response.can_update_55 then
			omnichannel_campaign.Integration = allegro_info.action_id == "SAVE_INTEGRATION" and allegro_info.Integration or nil
		end
	end

	if response.can_update_55 then
		-- Sanitizo los datos que se deben enviar en el POST hacia Allegro
		templates.SanitizeOmnichannelCampaign(omnichannel_campaign)
	end

	return response
end

--[[
  Crea la campana en 5.5
]]--
function Create (omnichannel_campaign, vcc_id, allegro_info)

	assert(type(omnichannel_campaign) == "table")

	local campaign_with_vcc = omnichannel_campaign.Id
	LogInfo("[ALLEGRO.CAMPAIGNS:Create] Creating allegro campaign. VCC '%s'", campaign_with_vcc)

	if not HasIntegration(vcc_id) then
		LogWarning("[ALLEGRO.CAMPAIGNS:Create] Error creating allegro campaign. VCC '%s' Without integration", vcc_id)
		return true -- no es un error
	end

	local full_campaign_id = utils.AppendVcc(utils.GetId(campaign_with_vcc), vcc_id)
	local camp_model = GetModel("Campaigns")

	if camp_model.CampaignExists(full_campaign_id) then
		LogError("[ALLEGRO.CAMPAIGNS:Create] Error creating allegro campaign: %s. Err: Campaign already exist.", full_campaign_id)
		return false, "CAMPAIGN_ALREADY_EXIST"
	end

	camp_model.LoadAllegroIntegrationInfo(omnichannel_campaign)

	--[[
		Informacion adicional que tiene que viajar en el POST.
		Ej: La IP del Server Remoto que se utiliza en el history.
	]]--
	if allegro_info.otherInfo then
		omnichannel_campaign.otherInfo = allegro_info.otherInfo
	end

	local encoded_campaign_omnichannel = json.encode(omnichannel_campaign)
	--LogInfo("[ALLEGRO.CAMPAIGNS:Create] encoded campaigns: %s", encoded_campaign_omnichannel)
--	vcc_id, post_data, url, content_type, fixedUrl, mustIncludeOmnichannelInfo
	local success, allegro_err = allegro.Post(
		vcc_id,
		encoded_campaign_omnichannel,
		"administration/campaigns/create/", -- la URL de 5.5
		nil, -- content type
		nil, -- fixedUrl
		true -- si debe de incluir informacion adicional en el POST
	)

	local err
	if not success then
		err = { type = "ALLEGRO_INTEGRATION_ERROR", sub_type = allegro_err }
		LogWarning("[ALLEGRO.CAMPAIGNS:Create] Error creating allegro campaign: %s. Err: %s.", full_campaign_id, json.encode(err))
	end

	return success, err
end

--[[
	Actualiza la campana en 5.5
]]--
function Update (campaign_with_vcc, vcc_id, allegro_info, ksAgent)
	assert(type(campaign_with_vcc) == "string")
	assert(type(allegro_info) == "table")

	LogInfo("[ALLEGRO.CAMPAIGNS:Update] Updating allegro campaign. VCC '%s'", campaign_with_vcc)

	if not HasIntegration(vcc_id) then
		LogWarning("[ALLEGRO.CAMPAIGNS:Update] Error updating allegro campaign. VCC '%s' Without integration", vcc_id)
		return true -- no es un error
	end

	local full_campaign_id = utils.AppendVcc(utils.GetId(campaign_with_vcc), vcc_id)
	local camp_model = GetModel("Campaigns")
	local omnichannel_campaign = camp_model.GetCampaign(vcc_id, campaign_with_vcc, ksAgent, allegro_info.mustLoadMembers)

	-- local WAIT_SEC = 30
	-- LogDebug("[ALLEGRO.CAMPAIGNS:Update] Updating allegro campaign: %s. PAUSA para desarollo. WAIT_SEC: %s", full_campaign_id, WAIT_SEC)
	-- utils.Sleep(WAIT_SEC)

	if not omnichannel_campaign then
		LogError("[ALLEGRO.CAMPAIGNS:Update] Error updating allegro campaign: %s. Err: Campaign does not exist.", full_campaign_id)
		return false, "CAMPAIGN_DOES_NOT_EXIST"
	end

	-- actualizo informacion de la campana en memoria, para que sea enviada en el request de actualizacion contra allegro
	local response = __UpdateCampaignMemoryInfo( omnichannel_campaign, vcc_id, allegro_info )
	if not response.can_update_55 then
		-- no es un error o si
		LogWarning("[ALLEGRO.CAMPAIGNS:Update] Unable to update allegro, '%s' campaign in memory. Action id: '%s'. Response: '%s'.", full_campaign_id, allegro_info.action_id, json.encode(response))
		return false, response
	end

-- 	omnichannel_campaign.cany = json.decode([[
-- {"bot_no_contacto_v2_solotexto":{"groupId":"ba686740_89f3_406f_bf60_0ef55652e566","languages":{"es":{"groupId":"ba686740_89f3_406f_bf60_0ef55652e566","code":"es","id":"1054446585884329","text":"Toc! Toc! \u00A1Te estuvimos llamando porque tenemos algo incre\u00EDble  que contarte!\uD83D\uDE31 As\u00ED es, tenemos una *promo* *para que nos digas S\u00CD*\n\nDa clic en _*Conocer Oferta*_ \uD83D\uDC47\uD83D\uDC47 si quieres conocer m\u00E1s \uD83D\uDC40","groupName":"ba686740_89f3_406f_bf60_0ef55652e566","quality_score":{"date":1702583687,"score":"UNKNOWN"},"components":[{"text":"Toc! Toc! \u00A1Te estuvimos llamando porque tenemos algo incre\u00EDble  que contarte!\uD83D\uDE31 As\u00ED es, tenemos una *promo* *para que nos digas S\u00CD*\n\nDa clic en _*Conocer Oferta*_ \uD83D\uDC47\uD83D\uDC47 si quieres conocer m\u00E1s \uD83D\uDC40","type":"BODY"},{"text":"WOM Colombia","type":"FOOTER"},{"buttons":[{"text":"Conocer Oferta","type":"QUICK_REPLY"}],"type":"BUTTONS"}],"rejected_reason":"NONE","name":"bot_no_contacto_v2_solotexto","category":"MARKETING","status":"APPROVED","urlButtonParam":false,"language":"es","params":[]}},"available":true,"id":"bot_no_contacto_v2_solotexto","status":"approved","groupName":"ba686740_89f3_406f_bf60_0ef55652e566","category":"MARKETING"},"bot_no_contacto_v1":{"groupId":"ba686740_89f3_406f_bf60_0ef55652e566","languages":{"es":{"groupId":"ba686740_89f3_406f_bf60_0ef55652e566","code":"es","id":"1000165687745711","text":"Toc! Toc! \u00A1Te estuvimos llamando porque tenemos algo incre\u00EDble  que contarte!\uD83D\uDE31 As\u00ED es, tenemos una *promo* *para que nos digas S\u00CD*\n\nDa clic en _*Conocer Oferta*_ \uD83D\uDC47\uD83D\uDC47 si quieres conocer m\u00E1s \uD83D\uDC40","groupName":"ba686740_89f3_406f_bf60_0ef55652e566","quality_score":{"date":1702582132,"score":"UNKNOWN"},"components":[{"mediaUrl":"https://scontent.whatsapp.net/v/t61.29466-34/399428727_1000165691079044_8683295895356577791_n.mp4?ccb=1-7&_nc_sid=8b1bef&_nc_ohc=7mk7VJjZgUsAX9KqhpP&_nc_ht=scontent.whatsapp.net&edm=AH51TzQEAAAA&oh=01_AdQ-kgQHkJorPOQ8fhaP90XgQ1Y0zsI4Rvgbfmbg2gaf9Q&oe=65A2D5F7","format":"VIDEO","type":"HEADER"},{"text":"Toc! Toc! \u00A1Te estuvimos llamando porque tenemos algo incre\u00EDble  que contarte!\uD83D\uDE31 As\u00ED es, tenemos una *promo* *para que nos digas S\u00CD*\n\nDa clic en _*Conocer Oferta*_ \uD83D\uDC47\uD83D\uDC47 si quieres conocer m\u00E1s \uD83D\uDC40","type":"BODY"},{"text":"WOM Colombia","type":"FOOTER"},{"buttons":[{"text":"Conocer Oferta","type":"QUICK_REPLY"}],"type":"BUTTONS"}],"rejected_reason":"NONE","name":"bot_no_contacto_v1","category":"MARKETING","status":"APPROVED","urlButtonParam":false,"language":"es","params":[]}},"available":true,"id":"bot_no_contacto_v1","status":"approved","groupName":"ba686740_89f3_406f_bf60_0ef55652e566","category":"MARKETING"}}
-- 	]])
	local encoded_campaign_omnichannel = json.encode(omnichannel_campaign)
	LogDebug("[ALLEGRO.CAMPAIGNS:Update] --->  encoded_campaign_omnichannel: %s", encoded_campaign_omnichannel)

	local success, allegro_err = allegro.Post(
		vcc_id,
		encoded_campaign_omnichannel,
		"administration/campaigns/update/", -- la URL de 5.5
		nil, -- content type
		nil, -- fixedUrl
		true -- si debe de incluir informacion adicional en el POST
	)

	local err
	if not success then
		err = {
			type = "ALLEGRO_INTEGRATION_ERROR",
			sub_type = allegro_err
		}
		LogWarning("[ALLEGRO.CAMPAIGNS:Update] Error updating allegro campaign: %s. Err: %s.", full_campaign_id, json.encode(err))
	end

	return success, err
end

local function __CanDeleteCampaign (omnichannel_campaign)
	-- TODO: de momento no realizo ninguna validacion, pero aca podria existir alguna validacion determinada.
	--
	return true
end

--[[
  Elimina la campana en 5.5
]]--
function Delete (campaign_with_vcc, vcc_id, allegroOtherInfo)
	assert(type(campaign_with_vcc) == "string")
	assert(type(vcc_id) == "string")

	LogInfo("[ALLEGRO.CAMPAIGNS:Delete] Deleting allegro campaign. VCC '%s'", campaign_with_vcc)

	if not HasIntegration(vcc_id) then
		LogWarning("[ALLEGRO.CAMPAIGNS:Delete] Error deleting allegro campaign. VCC '%s' Without integration", vcc_id)
		return true -- no es un error
	end

	local simple_camp_id = utils.GetId(campaign_with_vcc)
	local full_campaign_id = utils.AppendVcc(simple_camp_id, vcc_id)
	local camp_model = GetModel("Campaigns")
	local omnichannel_campaign = camp_model.GetCampaign(vcc_id, campaign_with_vcc)

	if not omnichannel_campaign then
		LogError("[ALLEGRO.CAMPAIGNS:Delete] Error deleting allegro campaign: %s. Err: Campaign does not exist.", full_campaign_id)
		return false, "CAMPAIGN_DOES_NOT_EXIST"
	end

	local success = __CanDeleteCampaign( omnichannel_campaign )
	if not success then
		-- no es un error o si
		LogWarning("[ALLEGRO.CAMPAIGNS:Delete] Unable to delete '%s' campaign.", full_campaign_id)
		return false, { type = "UNABLE_TO_DELETE_CAMPAIGN" }
	end

	local success, allegro_err = allegro.Post(
		vcc_id,
		json.encode({campaign_id = simple_camp_id, otherInfo = allegroOtherInfo}),
		"administration/campaigns/delete/", -- la URL de 5.5
		nil, -- content type
		nil, -- fixedUrl
		true -- si debe de incluir informacion adicional en el POST
	)

	local err
	if not success then
		err = { type = "ALLEGRO_INTEGRATION_ERROR", sub_type = allegro_result }
		LogWarning("[ALLEGRO.CAMPAIGNS:Delete] Error deleting allegro campaign: %s. Err: %s.", full_campaign_id, json.encode(err))
	end

	return success, err
end
