module(..., package.seeall, inconcert.orbit.application.Decorate)

local proxy = require "apps.allegro.proxy"

local function GetContact(web, contactId)
	local success, response = proxy.outboundEngine.SdkGetContact(contactId)
	if not success then
		LogError("[OUTBOUNDENGINE_SDK:GetContact] Error retrieving %s Contact - %s.", contactId, response)
		web.status = 500
	end
	return response
end

local function GetContactJson(web, contactId)
	local response, err = proxy.outboundEngine.SdkGetContactJson(contactId)
	if not response then
		local reason = type(err) == "table" and err.reason or err or "server error"
		LogError("[OUTBOUNDENGINE_SDK:GetContactJson] Error retrieving %s Contact - %s.", contactId, reason)
		response = { status = false, reason = reason }
	end
	if response.contact then
		return json.encode(response.contact)
	else
		web.status = 500
		return response.reason or "unknown error"
	end
end

local function GetImportStatus(web)
	local importId = web.input["ImportId"]
	local success, response = proxy.outboundEngine.SdkGetImportStatus(importId)
	if not success then
		LogError("[OUTBOUNDENGINE_SDK:GetImportStatus] Error retrieving import %s status - %s.", importId, response)
		web.status = 500
	end
	return response
end

local function GetImportDetail(web)
	local importId = web.input["ImportId"]
	local success, response = proxy.outboundEngine.SdkGetImportDetail(importId)
	if not success then
		LogError("[OUTBOUNDENGINE_SDK:GetImportDetail] Error retrieving import %s detail - %s.", importId, response)
		web.status = 500
	end
	return response
end

local function GetAllOutboundCampaigns(web)
	local success, response = proxy.outboundEngine.SdkGetAllOutboundCampaigns()
	if not success then
		LogError("[OUTBOUNDENGINE_SDK:GetAllOutboundCampaigns] Error retrieving data - %s.", response)
		web.status = 500
	end
	return response
end

local function GetImportFormats(web)
	local success, response = proxy.outboundEngine.SdkGetImportFormats()
	if not success then
		LogError("[OUTBOUNDENGINE_SDK:GetImportFormats] Error retrieving data - %s.", response)
		web.status = 500
	end
	return response
end

local function GetHistoricalInteraction(web, interactionId)
	local success, response = proxy.outboundEngine.SdkGetHistoricalInteraction(interactionId)
	if not success then
		LogError("[OUTBOUNDENGINE_SDK:GetHistoricalInteraction] Error retrieving %s interaction - %s.", interactionId, response)
		web.status = 500
	end
	return response
end

local function SetDisposition(web)
	local encodedData = web.input["post_data"]
	local success, response = proxy.outboundEngine.SdkSetDisposition(encodedData)
	if not success then
		LogError("[OUTBOUNDENGINE_SDK:SetDisposition] Error setting disposition %s - %s.", 
			encodedData, response)
		web.status = 500
	end
	return response
end

local function SetDispositionJson(web)
	local encodedData = web.input["post_data"]
	local success, response = proxy.outboundEngine.SdkSetDispositionJson(encodedData)
	if not success then
		local reason = type(response) == "table" and (response.reason or "server error") or response or "server error"
		LogError("[OUTBOUNDENGINE_SDK:SetDispositionJson] Error setting disposition %s - %s.", 
			encodedData, reason)
		response = { status = false, reason = reason }
	end
	if response.status then
		return json.encode({ status = true })
	else
		web.status = 500
		return response.reason or "unknown error"
	end
end

local function SearchContacts(web)
	local encodedData = web.input["post_data"]
	local success, response = proxy.outboundEngine.SdkSearchContacts(encodedData)
	if not success then
		LogError("[OUTBOUNDENGINE_SDK:SearchContacts] Error searching contacts with criteria %s - %s.", 
			encodedData, response)
		web.status = 500
	end
	return response
end

local function SearchContactsJson(web)
	local encodedData = web.input["post_data"]
	local success, response = proxy.outboundEngine.SdkSearchContactsJson(encodedData)
	if not success then
		local reason = type(response) == "table" and (response.reason or "server error") or response or "server error"
		LogError("[OUTBOUNDENGINE_SDK:SearchContactsJson] Error searching contacts with criteria %s - %s.", 
			encodedData, reason)
		response = { status = false, reason = reason }
	end
	if response.contacts then
		return json.encode(response.contacts)
	else
		web.status = 500
		return response.reason or "unknown error"
	end
end

local function AddContact(web)
	local encodedData = web.input["post_data"]
	local success, response = proxy.outboundEngine.SdkAddContact(encodedData)
	if not success then
		LogError("[OUTBOUNDENGINE_SDK:AddContact] Error adding contact %s - %s.", encodedData, response)
		web.status = 500
	end
	return response
end

local function AddContactJson(web)
	local encodedData = web.input["post_data"]
	local success, response = proxy.outboundEngine.SdkAddContactJson(encodedData)
	if not success then
		local reason = type(response) == "table" and (response.reason or "server error") or response or "server error"
		LogError("[OUTBOUNDENGINE_SDK:AddContactJson] Error adding contact %s - %s.", encodedData, reason)
		response = { status = false, reason = reason }
	end
	if response.status then
		return json.encode({ status = true })
	else
		web.status = 500
		return response.reason or "unknown error"
	end
end

local function RecognizePhone(web)
	local encodedData = web.input["post_data"]
	local success, response = proxy.outboundEngine.SdkRecognizePhone(encodedData)
	if not success then
		LogError("[OUTBOUNDENGINE_SDK:RecognizePhone] Error recognizing phone %s - %s.", encodedData, response)
		web.status = 500
	end
	return response
end

local function RecognizePhoneJson(web)
	local encodedData = web.input["post_data"]
	local success, response = proxy.outboundEngine.SdkRecognizePhoneJson(encodedData)
	if not success then
		local reason = type(response) == "table" and (response.reason or "server error") or response or "server error"
		LogError("[OUTBOUNDENGINE_SDK:RecognizePhoneJson] Error recognizing phone %s - %s.", encodedData, reason)
		response = { status = false, reason = reason }
	end
	if response.phone then
		return json.encode(response.phone)
	else
		web.status = 500
		return response.reason or "unknown error"
	end
end

local function AddContactValues(web)
	local encodedData = web.input["post_data"]
	local success, response = proxy.outboundEngine.SdkAddContactValues(encodedData)
	if not success then
		LogError("[OUTBOUNDENGINE_SDK:AddContactValues] Error adding values %s - %s.", encodedData, response)
		web.status = 500
	end
	return response
end

local function AddContactSkills(web)
	local encodedData = web.input["post_data"]
	local success, response = proxy.outboundEngine.SdkAddContactSkills(encodedData)
	if not success then
		LogError("[OUTBOUNDENGINE_SDK:AddContactSkills] Error adding skills %s - %s.", encodedData, response)
		web.status = 500
	end
	return response
end

local function AddPhone(web)
	local encodedData = web.input["post_data"]
	local success, response = proxy.outboundEngine.SdkAddPhone(encodedData)
	if not success then
		LogError("[OUTBOUNDENGINE_SDK:AddPhone] Error adding phone %s - %s.", encodedData, response)
		web.status = 500
	end
	return response
end

local function AssignAgent(web)
	local encodedData = web.input["post_data"]
	local success, response = proxy.outboundEngine.SdkAssignAgent(encodedData)
	if not success then
		LogError("[OUTBOUNDENGINE_SDK:AssignAgent] Error assigning agent %s - %s.", encodedData, response)
		web.status = 500
	end
	return response
end

local function AddContactToBatch(web)
	local encodedData = web.input["post_data"]
	local success, response = proxy.outboundEngine.SdkAddContactToBatch(encodedData)
	if not success then
		LogError("[OUTBOUNDENGINE_SDK:AddContactToBatch] Error adding contact %s - %s.", encodedData, response)
		web.status = 500
	end
	return response
end

local function CancelContact(web)
	local encodedData = web.input["post_data"]
	local success, response = proxy.outboundEngine.SdkCancelContact(encodedData)
	if not success then
		LogError("[OUTBOUNDENGINE_SDK:CancelContact] Error canceling contact %s - %s.", encodedData, response)
		web.status = 500
	end
	return response
end

local function PauseContact(web)
	local encodedData = web.input["post_data"]
	local success, response = proxy.outboundEngine.SdkPauseContact(encodedData)
	if not success then
		LogError("[OUTBOUNDENGINE_SDK:PauseContact] Error pausing contact %s - %s.", encodedData, response)
		web.status = 500
	end
	return response
end

local function ResumeContact(web)
	local encodedData = web.input["post_data"]
	local success, response = proxy.outboundEngine.SdkResumeContact(encodedData)
	if not success then
		LogError("[OUTBOUNDENGINE_SDK:ResumeContact] Error resuming contact %s - %s.", encodedData, response)
		web.status = 500
	end
	return response
end

local function FindContactInBatch(web)
	local encodedData = web.input["post_data"]
	local success, response = proxy.outboundEngine.SdkFindContactInBatch(encodedData)
	if not success then
		LogError("[OUTBOUNDENGINE_SDK:FindContactInBatch] Error finding contact %s - %s.", encodedData, response)
		web.status = 500
	end
	return response
end

local function CancelPhone(web)
	local encodedData = web.input["post_data"]
	local success, response = proxy.outboundEngine.SdkCancelPhone(encodedData)
	if not success then
		LogError("[OUTBOUNDENGINE_SDK:CancelPhone] Error canceling phone %s - %s.", encodedData, response)
		web.status = 500
	end
	return response
end

local function ChangePriority(web)
	local encodedData = web.input["post_data"]
	local success, response = proxy.outboundEngine.SdkChangePriority(encodedData)
	if not success then
		LogError("[OUTBOUNDENGINE_SDK:ChangePriority] Error changing priority %s - %s.", encodedData, response)
		web.status = 500
	end
	return response
end

local function DisablePhone(web)
	local encodedData = web.input["post_data"]
	local success, response = proxy.outboundEngine.SdkDisablePhone(encodedData)
	if not success then
		LogError("[OUTBOUNDENGINE_SDK:DisablePhone] Error disabling phone %s - %s.", encodedData, response)
		web.status = 500
	end
	return response
end

local function EnablePhone(web)
	local encodedData = web.input["post_data"]
	local success, response = proxy.outboundEngine.SdkEnablePhone(encodedData)
	if not success then
		LogError("[OUTBOUNDENGINE_SDK:EnablePhone] Error enabling phone %s - %s.", encodedData, response)
		web.status = 500
	end
	return response
end

local function FinishContact(web)
	local encodedData = web.input["post_data"]
	local success, response = proxy.outboundEngine.SdkFinishContact(encodedData)
	if not success then
		LogError("[OUTBOUNDENGINE_SDK:FinishContact] Error finishing contact %s - %s.", encodedData, response)
		web.status = 500
	end
	return response
end

local function ReScheduleContact(web)
	local encodedData = web.input["post_data"]
	local success, response = proxy.outboundEngine.SdkReScheduleContact(encodedData)
	if not success then
		LogError("[OUTBOUNDENGINE_SDK:ReScheduleContact] Error rescheduling contact %s - %s.", encodedData, response)
		web.status = 500
	end
	return response
end

local function ReScheduleContactJson(web)
	local encodedData = web.input["post_data"]
	local success, response = proxy.outboundEngine.SdkReScheduleContactJson(encodedData)
	if not success then
		local reason = type(response) == "table" and (response.reason or "server error") or response or "server error"
		LogError("[OUTBOUNDENGINE_SDK:ReScheduleContactJson] Error rescheduling contact %s - %s.", 
			encodedData, reason)
		response = { status = false, reason = reason }
	end
	if response.status then
		return json.encode({ status = true })
	else
		web.status = 500
		return response.reason or "unknown error"
	end
end

local function ReScheduleContactToPhone(web)
	local encodedData = web.input["post_data"]
	local success, response = proxy.outboundEngine.SdkReScheduleContactToPhone(encodedData)
	if not success then
		LogError("[OUTBOUNDENGINE_SDK:ReScheduleContactToPhone] Error rescheduling contact %s - %s.", encodedData, response)
		web.status = 500
	end
	return response
end

local function ReScheduleContactToPhoneJson(web)
	local encodedData = web.input["post_data"]
	local success, response = proxy.outboundEngine.SdkReScheduleContactToPhoneJson(encodedData)
	if not success then
		local reason = type(response) == "table" and (response.reason or "server error") or response or "server error"
		LogError("[OUTBOUNDENGINE_SDK:ReScheduleContactToPhoneJson] Error rescheduling contact %s - %s.", 
			encodedData, reason)
		response = { status = false, reason = reason }
	end
	if response.status then
		return json.encode({ status = true })
	else
		web.status = 500
		return response.reason or "unknown error"
	end
end

local function ReScheduleContactCampaign(web)
	local encodedData = web.input["post_data"]
	local success, response = proxy.outboundEngine.SdkReScheduleContactCampaign(encodedData)
	if not success then
		LogError("[OUTBOUNDENGINE_SDK:ReScheduleContactCampaign] Error rescheduling campaign %s - %s.", encodedData, response)
		web.status = 500
	end
	return response
end

local function ReScheduleContactCampaignJson(web)
	local encodedData = web.input["post_data"]
	local success, response = proxy.outboundEngine.SdkReScheduleContactCampaignJson(encodedData)
	if not success then
		local reason = type(response) == "table" and (response.reason or "server error") or response or "server error"
		LogError("[OUTBOUNDENGINE_SDK:ReScheduleContactCampaignJson] Error rescheduling contact %s - %s.", 
			encodedData, reason)
		response = { status = false, reason = reason }
	end
	if response.status then
		return json.encode({ status = true })
	else
		web.status = 500
		return response.reason or "unknown error"
	end
end

local function RecycleContact(web)
	local encodedData = web.input["post_data"]
	local success, response = proxy.outboundEngine.SdkRecycleContact(encodedData)
	if not success then
		LogError("[OUTBOUNDENGINE_SDK:RecycleContact] Error recycling contact %s - %s.", encodedData, response)
		web.status = 500
	end
	return response
end

local function ExtendBatch(web)
	local encodedData = web.input["post_data"]
	local success, response = proxy.outboundEngine.SdkExtendBatch(encodedData)
	if not success then
		LogError("[OUTBOUNDENGINE_SDK:ExtendBatch] Error extending batch %s - %s.", encodedData, response)
		web.status = 500
	end
	return response
end

local function DeleteBatch(web)
	local encodedData = web.input["post_data"]
	local success, response = proxy.outboundEngine.SdkDeleteBatch(encodedData)
	if not success then
		LogError("[OUTBOUNDENGINE_SDK:DeleteBatch] Error deleting batch %s - %s.", encodedData, response)
		web.status = 500
	end
	return response
end

local function ExpireBatch(web)
	local encodedData = web.input["post_data"]
	local success, response = proxy.outboundEngine.SdkExpireBatch(encodedData)
	if not success then
		LogError("[OUTBOUNDENGINE_SDK:ExpireBatch] Error expiring batch %s - %s.", encodedData, response)
		web.status = 500
	end
	return response
end

local function StopProcess(web)
	local encodedData = web.input["post_data"]
	local success, response = proxy.outboundEngine.SdkStopProcess(encodedData)
	if not success then
		LogError("[OUTBOUNDENGINE_SDK:StopProcess] Error stopping process %s - %s.", encodedData, response)
		web.status = 500
	end
	return response
end

local function StartProcess(web)
	local encodedData = web.input["post_data"]
	local success, response = proxy.outboundEngine.SdkStartProcess(encodedData)
	if not success then
		LogError("[OUTBOUNDENGINE_SDK:StartProcess] Error starting process %s - %s.", encodedData, response)
		web.status = 500
	end
	return response
end

local function StatusProcess(web)
	local encodedData = web.input["post_data"]
	local success, response = proxy.outboundEngine.SdkStatusProcess(encodedData)
	if not success then
		LogError("[OUTBOUNDENGINE_SDK:StatusProcess] Error getting process %s status - %s.", encodedData, response)
		web.status = 500
	end
	return response
end

local function DeleteContactImport(web)
	local encodedData = web.input["post_data"]
	local success, response = proxy.outboundEngine.SdkDeleteContactImport(encodedData)
	if not success then
		LogError("[OUTBOUNDENGINE_SDK:DeleteContactImport] Error deleting contact import %s - %s.", encodedData, response)
		web.status = 500
	end
	return response
end

local function CreateFormatImport(web)
	local encodedData = web.input["post_data"]
	local success, response = proxy.outboundEngine.SdkCreateFormatImport(encodedData)
	if not success then
		LogError("[OUTBOUNDENGINE_SDK:CreateFormatImport] Error creating import format %s - %s.", encodedData, response)
		web.status = 500
	end
	return response
end

local function CreateBatchFromFilter(web)
	local encodedData = web.input["post_data"]
	local success, response = proxy.outboundEngine.SdkCreateBatchFromFilter(encodedData)
	if not success then
		LogError("[OUTBOUNDENGINE_SDK:CreateBatchFromFilter] Error creating batch %s - %s.", encodedData, response)
		web.status = 500
	end
	return response
end

local function CreateFilter(web)
	local encodedData = web.input["post_data"]
	local success, response = proxy.outboundEngine.SdkCreateFilter(encodedData)
	if not success then
		LogError("[OUTBOUNDENGINE_SDK:CreateFilter] Error creating filter %s - %s.", encodedData, response)
		web.status = 500
	end
	return response
end

local function CreateBatchFromImport(web)
	local encodedData = web.input["post_data"]
	local success, response = proxy.outboundEngine.SdkCreateBatchFromImport(encodedData)
	if not success then
		LogError("[OUTBOUNDENGINE_SDK:CreateBatchFromImport] Error creating batch %s - %s.", encodedData, response)
		web.status = 500
	end
	return response
end

function MapUris (module, htmlRender, prefix)
	module:dispatch_get(GetContact, prefix .. "/get_contact/(.+)")
	module:dispatch_get(GetContactJson, prefix .. "/get_contact_json/(.+)")
	module:dispatch_get(GetImportStatus, prefix .. "/get_import_status/(.+)")
	module:dispatch_get(GetImportDetail, prefix .. "/get_import_detail/(.+)")
	module:dispatch_get(GetAllOutboundCampaigns, prefix .. "/get_all_campaigns/(.*)")
	module:dispatch_get(GetImportFormats, prefix .. "/get_import_formats/(.*)")
	module:dispatch_get(GetHistoricalInteraction, prefix .. "/get_historical_interaction/(.+)")
	
	module:dispatch_post(SetDisposition, prefix .. "/set_disposition_code/(.*)")
	module:dispatch_post(SetDispositionJson, prefix .. "/set_disposition_code_json/(.*)")
	module:dispatch_post(SearchContacts, prefix .. "/search_contacts/(.*)")
	module:dispatch_post(SearchContactsJson, prefix .. "/search_contacts_json/(.*)")
	module:dispatch_post(AddContact, prefix .. "/add_contact/(.*)")
	module:dispatch_post(AddContactJson, prefix .. "/add_contact_json/(.*)")
	module:dispatch_post(RecognizePhone, prefix .. "/recognize_phone/(.*)")
	module:dispatch_post(RecognizePhoneJson, prefix .. "/recognize_phone_json/(.*)")
	module:dispatch_post(AddContactValues, prefix .. "/add_contact_values/(.*)")
	module:dispatch_post(AddContactSkills, prefix .. "/add_contact_skills/(.*)")
	module:dispatch_post(AddPhone, prefix .. "/add_phone/(.*)")
	module:dispatch_post(AssignAgent, prefix .. "/assign_agent/(.*)")
	module:dispatch_post(AddContactToBatch, prefix .. "/add_contact_to_batch/(.*)")
	module:dispatch_post(CancelContact, prefix .. "/cancel_contact/(.*)")
	module:dispatch_post(PauseContact, prefix .. "/pause_contact/(.*)")
	module:dispatch_post(ResumeContact, prefix .. "/resume_contact/(.*)")
	module:dispatch_post(FindContactInBatch, prefix .. "/find_contact_in_batch/(.*)")
	module:dispatch_post(CancelPhone, prefix .. "/cancel_phone/(.*)")
	module:dispatch_post(ChangePriority, prefix .. "/change_contact_priority/(.*)")
	module:dispatch_post(DisablePhone, prefix .. "/disable_phone/(.*)")
	module:dispatch_post(EnablePhone, prefix .. "/enable_phone/(.*)")
	module:dispatch_post(FinishContact, prefix .. "/finish_contact/(.*)")
	module:dispatch_post(ReScheduleContact, prefix .. "/reschedule_contact/(.*)")
	module:dispatch_post(ReScheduleContactJson, prefix .. "/reschedule_contact_json/(.*)")
	module:dispatch_post(ReScheduleContactToPhone, prefix .. "/reschedule_contact_to_phone/(.*)")
	module:dispatch_post(ReScheduleContactToPhoneJson, prefix .. "/reschedule_contact_to_phone_json/(.*)")
	module:dispatch_post(ReScheduleContactCampaign, prefix .. "/reschedule_contact_campaign/(.*)")
	module:dispatch_post(ReScheduleContactCampaignJson, prefix .. "/reschedule_contact_campaign_json/(.*)")
	module:dispatch_post(RecycleContact, prefix .. "/recycle_contact/(.*)")
	module:dispatch_post(ExtendBatch, prefix .. "/extendbatch/(.*)")
	module:dispatch_post(DeleteBatch, prefix .. "/delete_batch/(.*)")
	module:dispatch_post(ExpireBatch, prefix .. "/expire_batch/(.*)")
	module:dispatch_post(StopProcess, prefix .. "/stop_process/(.*)")
	module:dispatch_post(StartProcess, prefix .. "/start_process/(.*)")
	module:dispatch_post(StatusProcess, prefix .. "/status_process/(.*)")
	module:dispatch_post(DeleteContactImport, prefix .. "/delete_contact_import/(.*)")
	module:dispatch_post(CreateFormatImport, prefix .. "/create_format_import/(.*)")
	module:dispatch_post(CreateBatchFromFilter, prefix .. "/create_batch_from_filter/(.*)")
	module:dispatch_post(CreateFilter, prefix .. "/save_filter/(.*)")
	module:dispatch_post(CreateBatchFromImport, prefix .. "/create_batch_from_import/(.*)")
end