local proxy = require "apps.allegro.proxy"
local lfs = require "lfs"
local orbit = require "orbit"
require "uuid"
local utils = require "shared.apps.utils"
require "inconcert.orbit.application"
local wsapi = require "wsapi"
wsapi.request = require "wsapi.request"
wsapi.request_stream = require "wsapi.request_stream"
local ltn12 = require "ltn12"
local m_models = require("apps.allegro.model"):GetModels()
local modulename = select(1, ...)
local m_moduleLocation = select(2, ...)

module(modulename, package.seeall, orbit.new, inconcert.orbit.application.Decorate)

--module(..., package.seeall, inconcert.orbit.application.Decorate)

m_htmlRender = nil
m_appLocation = nil

function HasIntegration ( vcc ) return proxy.HasIntegration(vcc) end

function GetModel ()
	local entity = "Repository"
	if not m_models or not m_models[entity] then
	-- Si el modelo no se ha creado aun, debo crearlo.
	--  Esto sucede en algunas ocasiones si es que me han invocado desde otra app Ej: "Allegro" y aun no se ha creado el modelo.
	--  Entonces tengo que crear el modelo.
		local allegroApp = require "apps.allegro"
		allegroApp.CheckStatus()
		m_models = require("apps.allegro.model"):GetModels()
	end
	return m_models[entity]
end

local function __GetWebhandlerInfo ( vcc )
	local allegro_webhandler_info, err = proxy.generic.GetWebhandlerInfo( vcc )

	if not allegro_webhandler_info then
		if not err then
			err = { type = "ALLEGRO_INTEGRATION_ERROR", sub_type = "RETRIEVING_ALLEGRO_CREDENTIALS" }
		end
		LogWarning("[ALLEGRO.REPOSITORY:__GetWebhandlerInfo] Error retrieving allegro credentials. Err: '%s'", err)
		return false, err
	end

	return allegro_webhandler_info
end

--[[
  Postea el archivo al repositorio de 5.5
]]--
local function __PostFileToAllegro (file_info, file_to_upload, vcc, parameters, webhandler_info)

	LogInfo("[ALLEGRO.REPOSITORY:__PostFileToAllegro] Posting file to allegro. File info '%s', file: '%s', parameters: %s.", file_info, file_to_upload, parameters)

	assert(parameters)

	if not webhandler_info then
		webhandler_info = __GetWebhandlerInfo( vcc )
		if not webhandler_info then
			LogWarning("[ALLEGRO.REPOSITORY:__PostFileToAllegro] Unable to retrieve webhandler_info. Error Uploading file to allegro. File info '%s', file: '%s'.", file_info, file_to_upload)
			return false, "ERROR_UPLOADING_FILE_TO_ALLEGRO"
		end
	end

	local url = "repository/add/" .. "?" .. parameters

	local webhandler_ip = webhandler_info.ip
	local webhandler_port = webhandler_info.port
	local cookie = webhandler_info.credentials

	local file_size = file_info.fileSize
	local headers = {
		["Content-Length"] = file_size,
		["Content-Type"] = "application/octet-stream"
	}

	-- LogDebug("--> url       : '%s'", url)
	-- LogDebug("--> file      : '%s'", file_to_upload)
	-- LogDebug("--> headers   : '%s'", json.encode(headers))
	-- LogDebug("--> cookie    : '%s'", cookie)
	-- LogDebug("--> ip        : '%s'", webhandler_ip)
	-- LogDebug("--> port      : '%s'", webhandler_port)

	local allegro_request = require("apps.allegro.request")
	local success, err = allegro_request.Post(url, file_to_upload, headers, cookie, webhandler_ip, webhandler_port)

	if success then
		LogInfo("[ALLEGRO.REPOSITORY:__PostFileToAllegro] Successfully Posted file to allegro. File info: '%s'.", json.encode(file_info))
		return true
	end

	LogWarning("[ALLEGRO.REPOSITORY:__PostFileToAllegro] Error Posting file to allegro. File info: '%s'. Err: '%s'.", json.encode(file_info), err)
	return false, err
end

local function __PostAccountFilesToAllegro (messages_omnichannel, campaign_id, account_id, vcc)
	LogInfo("[ALLEGRO.REPOSITORY:__PostAccountFilesToAllegro] - Start posting files to Allegro. Campaign '%s', account '%s'.", campaign_id, account_id )

	local webhandler_info = __GetWebhandlerInfo( vcc )
	if not webhandler_info then
		LogWarning("[ALLEGRO.REPOSITORY:__PostAccountFilesToAllegro] Unable to retrieve webhandler_info. Error posting files to allegro. Campaign '%s', account '%s'.", campaign_id, account_id)
		return false
	end

	local repository_model = GetModel()

	for _, message in pairs( messages_omnichannel ) do
		if ( message.type or "" ):lower() == "file" and message.fileInformation then
			local file_info = {
				FileId = message.fileInformation.id,
				Campaign = campaign_id
			}
			local temporal_file = repository_model.GetTemporalUploadedFile(file_info)
			if temporal_file then
				local parameters = "id=" .. file_info.FileId .. "&" ..
					"section=1&" ..
					"format=wav&" ..
					"contentType=application/octet-stream&" ..
					"path=MultimediaResources&" ..
					"createIfNotexists=on"
				local success = __PostFileToAllegro( message.fileInformation, temporal_file, vcc, parameters, webhandler_info )
				if success then
					repository_model.DeleteTemporalUploadedFile(file_info)
				end
			else
				LogWarning("[ALLEGRO.REPOSITORY:__PostAccountFilesToAllegro] - Unable to upload temporal file. Temporal file does not exist. Campaign '%s', file id '%s'.", file_info.Campaign, file_info.FileId )
			end
		end
	end

	return true
end


--[[
  Guarda los mm resources en 5.5, funcion invocada desde la aplicacion administration.
]]--
function SaveMMResources ( messages_omnichannel, campaign_id, account_id, vcc )
	assert(type(messages_omnichannel) == "table")

	LogInfo("[ALLEGRO.REPOSITORY:SaveMMResources] - Start saving mm resources to Allegro. Campaign '%s', account '%s', account_messages.", campaign_id, account_id, json.encode(messages_omnichannel) )

	if not HasIntegration(vcc) then
		LogWarning("[ALLEGRO.REPOSITORY:SaveMMResources] Error saving allegro mm resources. VCC '%s' Without integration", vcc)
		return true -- no es un error
	end

	local encoded_messages_omnichannel = json.encode(messages_omnichannel)
	LogInfo("[ALLEGRO.REPOSITORY:SaveMMResources] encoded mm resources: %s", encoded_messages_omnichannel)

	local success, allegro_err = proxy.Post(
		vcc,
		encoded_messages_omnichannel,
		"administration/mmresources/save/" -- la URL de 5.5
	)

	if not success then
		local err = { type = "ALLEGRO_INTEGRATION_ERROR", sub_type = allegro_err }
		LogWarning("[ALLEGRO.REPOSITORY:SaveMMResources] Error saving allegro mm resources: %s. Err: %s.", encoded_messages_omnichannel, json.encode(err))
		return success, err
	end

	success = __PostAccountFilesToAllegro(messages_omnichannel, campaign_id, account_id, vcc)

	return success
end

local function DeleteTemporalFile (info)
	local file_info = {
		FileId   = info.id,
		Campaign = info.campaign
	}
	local success = false
	local repository_model = GetModel()
	local temporal_file = repository_model.GetTemporalUploadedFile(file_info)
	if temporal_file then
		success = repository_model.DeleteTemporalUploadedFile(file_info)
	end
	if success then
		LogInfo("[ALLEGRO.REPOSITORY:DeleteTemporalFile] Successfully removed temporal file. Campaign '%s', file id '%s'.", file_info.Campaign, file_info.FileId)
	else
		LogWarning("[ALLEGRO.REPOSITORY:DeleteTemporalFile] Unable to remove temporal file. Temporal file does not exist. Campaign '%s', file id '%s'.", file_info.Campaign, file_info.FileId )
	end
	return success
end

function OutboundEngineUploadImportFile (encodedInfo, fileToUpload, vcc)
	LogInfo("[ALLEGRO.REPOSITORY:OutboundEngineUploadImportFile] Posting file to allegro. File info '%s'.", encodedInfo)
	--LogDebug("[ALLEGRO.REPOSITORY:OutboundEngineUploadImportFile] Posting file to allegro. fileToUpload '%s'.", fileToUpload)

	webhandler_info = __GetWebhandlerInfo( vcc )
	if not webhandler_info then
		LogWarning("[ALLEGRO.REPOSITORY:OutboundEngineUploadImportFile] Unable to retrieve webhandler_info. Error Uploading file to allegro. File info '%s', file: '%s'.", encodedInfo, fileToUpload)
		return false, "ERROR_UPLOADING_FILE_TO_ALLEGRO"
	end

	local parameters = string.format("importationInfo=%s", encodedInfo)
	local url = "outboundengine/importation/omnichannel/upload/" .. "?" .. parameters
	local webhandler_ip = webhandler_info.ip
	local webhandler_port = webhandler_info.port
	local cookie = webhandler_info.credentials
	local info = json.decode(encodedInfo)
	local fileSize = info.FileInfo.fileSize
	local headers = {
		["Content-Length"] = fileSize,
		["Content-Type"] = "text/plain"
	}
	local timeout_sec = 1200 -- 20 minutos

-- LogDebug("--> url       : '%s'", url)
-- LogDebug("--> file      : '%s'", fileToUpload)
-- LogDebug("--> fileSize  : '%s'", fileSize)
-- LogDebug("--> headers   : '%s'", json.encode(headers))
-- LogDebug("--> cookie    : '%s'", cookie)
-- LogDebug("--> ip        : '%s'", webhandler_ip)
-- LogDebug("--> port      : '%s'", webhandler_port)
-- LogDebug("--> timeout_sec: '%s'", timeout_sec)

	local allegro_request = require("apps.allegro.request")
	local success, err = allegro_request.Post(url, fileToUpload, headers, cookie, webhandler_ip, webhandler_port, timeout_sec)

	if success then
		LogInfo("[ALLEGRO.REPOSITORY:OutboundEngineUploadImportFile] Successfully Posted file to allegro. File info: '%s'.", encodedInfo)
		-- Despues que el archivo fue subido y recibido correctamente en el servidor de 5.5, elimino el archivo temporal.
		DeleteTemporalFile(info.FileInfo)
		return true
	end

	LogWarning("[ALLEGRO.REPOSITORY:OutboundEngineUploadImportFile] Error Posting file to allegro. File info: '%s'. Err: '%s'.", encodedInfo, err)
	return false, err
end

function OutboundEngineStartImport (encodedInfo, vcc)
	LogInfo("[ALLEGRO.REPOSITORY:OutboundEngineStartImport] Posting file to allegro. File info '%s'.", encodedInfo)

	if not webhandler_info then
		webhandler_info = __GetWebhandlerInfo( vcc )
		if not webhandler_info then
			LogWarning("[ALLEGRO.REPOSITORY:OutboundEngineStartImport] Unable to retrieve webhandler_info. Error Uploading file to allegro. File info '%s'.", encodedInfo)
			return false, "ERROR_STARTING_IMPORTATION_TO_ALLEGRO"
		end
	end

	local url = "outboundengine/importation/omnichannel/start/"
	local webhandler_ip = webhandler_info.ip
	local webhandler_port = webhandler_info.port
	local cookie = webhandler_info.credentials
	local headers = {
		["Content-Length"] = fileSize,
		["Content-Type"] = "text/plain"
	}
	local timeout_sec = 1200 -- 20 minutos

-- LogDebug("--> url        : '%s'", url)
-- LogDebug("--> encodedInfo: '%s'", encodedInfo)
-- LogDebug("--> headers    : '%s'", json.encode(headers))
-- LogDebug("--> cookie     : '%s'", cookie)
-- LogDebug("--> ip         : '%s'", webhandler_ip)
-- LogDebug("--> port       : '%s'", webhandler_port)
-- LogDebug("--> timeout_sec: '%s'", timeout_sec)

	local allegro_request = require("apps.allegro.request")
	local success, err = allegro_request.Post(url, encodedInfo, headers, cookie, webhandler_ip, webhandler_port, timeout_sec)

	if success then
		LogInfo("[ALLEGRO.REPOSITORY:OutboundEngineStartImport] Successfully Started Importation. Info: '%s'.", encodedInfo)
		return true
	end

	LogWarning("[ALLEGRO.REPOSITORY:OutboundEngineStartImport] Error Starting Importation. Info: '%s'. Err: '%s'.", encodedInfo, err)
	return false, err
end

--[[
	Retorna un Error WEB si ha fallado el download del archivo.
]]--
local function __DownloadErrorWebResponse ()
	local headers = {}
	headers["Response"] = "defaultError"
	headers["Content-Type"] = "application/octet-stream"
	local MakeEmptyBody = function()
		return coroutine.wrap(function() coroutine.yield("") end)
	end
	return 400, headers, MakeEmptyBody()
end

local function __DownloadSuccessWebResponse (request, reqHeaders, fileName)
	local suggestedFileName = fileName or (uuid.new() .. ".file")

	LogInfo("[ALLEGRO.REPOSITORY:__DownloadSuccessWebResponse] Downloading file: '%s'.", suggestedFileName)

	local ChunkFn = function()
		local function SizeMessage (totalSentFileSizeBytes)
			local totalSentFileMsg
			local totalSentFileSizeKb = totalSentFileSizeBytes / 1024
			if totalSentFileSizeKb <= 1024 then
				totalSentFileMsg = tostring(utils.RoundNumber(totalSentFileSizeKb, 2)) .. " kB"
			else
				local totalSentFileSizeMb = utils.RoundNumber(totalSentFileSizeKb / 1024, 2)
				totalSentFileMsg = tostring(totalSentFileSizeMb) .. " MB"
			end
			return totalSentFileMsg
		end

		return coroutine.wrap(function()
			local i = 0
			local totalSentFileSizeBytes = 0
			local LOG_CHUNKS = 500 -- cada 500 chunks retorna 1 MB, así que logueamos cada 1 MB

			local mySink = function(chunk, err)
				if chunk then
					i = i + 1
					local size = #chunk
					totalSentFileSizeBytes = totalSentFileSizeBytes + size

					if i == 1 or (i == (math.floor(i / LOG_CHUNKS) * LOG_CHUNKS)) then
						local totalSentFileMsg = SizeMessage(totalSentFileSizeBytes)
						LogInfo("[ALLEGRO.REPOSITORY:__DownloadSuccessWebResponse] Chunk number '%d'. Have been sent %s. File: '%s'.", i, totalSentFileMsg, suggestedFileName)
					end

					local ok = coroutine.yield(chunk)
					if not ok then
						LogError("[ALLEGRO.REPOSITORY:__DownloadSuccessWebResponse] The client dropped the connection")
						return nil
					end
				end
				return 1
			end

			local receive = request.receivebody
			rawset(request, "receivebody", receive)
			request:receivebody(reqHeaders, mySink, ltn12.pump.step)
			request:close()

			local totalSentFileMsg = SizeMessage(totalSentFileSizeBytes)
			LogInfo("[ALLEGRO.REPOSITORY:__DownloadSuccessWebResponse] Have completed sended %s. File '%s'.", totalSentFileMsg, suggestedFileName)
		end)
	end

	local headers = {}
	-- NO LO tenemos, además si el valor va, no tenemos feedback en el navegador, mientras va descargando.
--	headers["Content-Length"]      = size
	headers["Response"]            = "ok"
	headers["Content-Disposition"] = [[attachment; filename="]]  .. suggestedFileName .. '"'
	headers["Set-Cookie"]          = "download_started=true; Path=/" -- used by the jquery download plugin

	-- Return an iterator co-routine to send chunks of opened file in case blob is huge.
	return "200 Ok", headers, ChunkFn()
end

function OutboundEngineImportDownloadFailedContacts (importId, update, vcc, fileName)

	LogInfo("[ALLEGRO.REPOSITORY:OutboundEngineImportDownloadFailedContacts] Importation Downloading Failed Contacts from allegro. ImportId '%s', update: '%s', fileName: '%s'.", importId, update, fileName)

	if not webhandler_info then
		webhandler_info = __GetWebhandlerInfo( vcc )
		if not webhandler_info then
			LogWarning("[ALLEGRO.REPOSITORY:OutboundEngineImportDownloadFailedContacts] Unable to retrieve webhandler_info. Error Downloading file from allegro. ImportId '%s', update: '%s'.", importId, update)
			return __DownloadErrorWebResponse()
		end
	end

-- "http://192.168.4.12:8082/inconcert/apps/outboundengine/importation/failedcsv/" + importation.ImportId + "/" + importation["Update#"];

	local url = "outboundengine/importation/failedcsv/" .. importId .. "/" .. update
	local webhandler_ip = webhandler_info.ip
	local webhandler_port = webhandler_info.port
	local cookie = webhandler_info.credentials

	local headers = {}

-- LogDebug("--> url       : '%s'", url)
-- LogDebug("--> headers   : '%s'", json.encode(headers))
-- LogDebug("--> cookie    : '%s'", cookie)
-- LogDebug("--> ip        : '%s'", webhandler_ip)
-- LogDebug("--> port      : '%s'", webhandler_port)

	local allegro_request = require("apps.allegro.request")
	local success, request, code, headers = allegro_request.StreamGet(url, headers, cookie, webhandler_ip, webhandler_port)

	if success then
		LogInfo("[ALLEGRO.REPOSITORY:OutboundEngineImportDownloadFailedContacts] Successfully Downloaded Failed Contacts from allegro. ImportId '%s', update: '%s'.", importId, update)
		return __DownloadSuccessWebResponse(request, headers, fileName)
	end

	local err = request
	LogWarning("[ALLEGRO.REPOSITORY:OutboundEngineImportDownloadFailedContacts] Error Downloading file from allegro. ImportId '%s', update: '%s'. Err: %s", importId, update, err)

	return __DownloadErrorWebResponse()
end

function OutboundEngineExportationDownload (exportationId, vcc, fileName)
	LogInfo("[ALLEGRO.REPOSITORY:OutboundEngineExportationDownload] Exportation Downloading from allegro. ExportationId '%s', fileName: '%s'.", exportationId, fileName)

	if not webhandler_info then
		webhandler_info = __GetWebhandlerInfo(vcc)
		if not webhandler_info then
			LogWarning("[ALLEGRO.REPOSITORY:OutboundEngineExportationDownload] Unable to retrieve webhandler_info. Error Downloading file from allegro. ExportationId '%s'.", exportationId)
			return __DownloadErrorWebResponse()
		end
	end

-- "http://192.168.4.12:8082/inconcert/apps/outboundengine/exportation/download/" + exportation.exportationId;

	local url = "outboundengine/exportation/download/" .. exportationId
	local webhandler_ip = webhandler_info.ip
	local webhandler_port = webhandler_info.port
	local cookie = webhandler_info.credentials
	local headers = {}

-- LogDebug("--> url       : '%s'", url)
-- LogDebug("--> headers   : '%s'", json.encode(headers))
-- LogDebug("--> cookie    : '%s'", cookie)
-- LogDebug("--> ip        : '%s'", webhandler_ip)
-- LogDebug("--> port      : '%s'", webhandler_port)

	local allegro_request = require("apps.allegro.request")
	local success, request, code, headers = allegro_request.StreamGet(url, headers, cookie, webhandler_ip, webhandler_port)

	if success then
		LogInfo("[ALLEGRO.REPOSITORY:OutboundEngineExportationDownload] Successfully Downloaded Exportation from allegro. ExportationId '%s'.", exportationId)
		return __DownloadSuccessWebResponse(request, headers, fileName)
	end

	local err = request
	LogWarning("[ALLEGRO.REPOSITORY:OutboundEngineExportationDownload] Error Downloading file from allegro. ExportationId '%s'. Err: %s", exportationId, err)

	return __DownloadErrorWebResponse()
end

--[[
  Guarda el archivo temporal en redis para que luego sea enviado al repositorio de 5.5
]]--
local function UploadTemporalFile (wsapi_env)
	local response = wsapi.response.new(200, { ["Content-Type"]= "text/plain" })
	response:write("")
	response.headers["Response"] = "defaultError"
	if wsapi_env.REQUEST_METHOD:lower() ~= "post" then
		response.status = 405
		return response:finish()
	end

	local request = wsapi.request_stream.new(wsapi_env)
	local section = tonumber(request.params.section)
	local timestamp = os.time(os.date("!*t")) -- current date in UTC

	local uploaded_file = {
		FileId      = request.params.id,
		Section     = section                                ,
		Format      = request.params.format                  ,
		ContentType = request.params.contentType             ,
		FileName    = utils.SanitizeFileName(request.params.fileName),
		ServerPath  = request.params.path                    ,
		FileSize    = tonumber(wsapi_env.CONTENT_LENGTH) or 0,
		Campaign    = request.params.campaign                ,
		Account     = request.params.account                 ,
		TimeStamp   = timestamp
	}

	LogInfo("[ALLEGRO.REPOSITORY:UploadTemporalFile] Upload temporal file. File info: %s", json.encode(uploaded_file))

	if uploaded_file.FileSize <= 0 then
		LogError("[ALLEGRO.REPOSITORY:UploadTemporalFile] Unable to upload file. Current file size is %d", uploaded_file.FileSize)
		response.headers["Response"] = "paramsError"
		response.status = 500
		response:write("Invalid file. Zero length")
		return response:finish()
	end

	if not uploaded_file.FileId or not uploaded_file.Section or not uploaded_file.Format or not uploaded_file.ContentType then
		LogError("[ALLEGRO.REPOSITORY:UploadTemporalFile] Unable to upload file. Incomplete file information. Info: %s", json.encode(uploaded_file))
		response.headers["Response"] = "paramsError"
		response.status = 500
		response:write("Invalid file data")
		return response:finish()
	end

	-- aca piso la metatabla para poder encargarme yo de la leida del POST data
	setmetatable(request.POST, nil)

	local chunkSize = 2048
	local table_file = {}

	local left = uploaded_file.FileSize
	local blocksize = math.min(chunkSize, left)
	while left > 0 do
		local chunk = wsapi_env.input:read(blocksize)
		if not chunk then break end
		table.insert(table_file, chunk)
		left = left - #chunk
		blocksize = math.min(blocksize, left)
	end

	local temporal_file = table.concat(table_file)
	local repository_model = GetModel()
	local file_key = repository_model.SetTemporalUploadedFile(uploaded_file, temporal_file)

	LogInfo("[ALLEGRO.REPOSITORY:UploadTemporalFile] Successfully uploaded temporal file. File key: '%s', file info: %s", file_key, json.encode(uploaded_file))

	local result = {fileId = uploaded_file.FileId}
	response:write(json.encode(result))

	return response:finish()
end

local function RemoveTemporalFile (web)
	web.headers["Response"] = "defaultError"

	local file = web.GET
	LogInfo("[ALLEGRO.REPOSITORY:RemoveTemporalFile] Removing file: %s.", json.encode(file))

	local success = DeleteTemporalFile(file)

	return json.encode( success )
end

local function RemoveAllTemporalFiles (web)
	web.headers["Response"] = "defaultError"

	LogInfo("[ALLEGRO.REPOSITORY:RemoveAllTemporalFiles] Removing multiples files: '%s'", web.input["post_data"])

	local result = { deleted = {}, failed  = {} }
	local files = json.decode(web.input["post_data"])

	for _, file in ipairs(files or {}) do
		local success = DeleteTemporalFile(file)
		if success then
			table.insert(result.deleted, file)
		else
			table.insert(result.failed, file)
		end
	end

	return json.encode(result)
end

function Retrieve (vcc, id, section, part, isStream)
	--LogDebug("[ALLEGRO.REPOSITORY:Retrieve] vcc: '%s', id: '%s', section: '%s', part: '%s', isStream: '%s'", vcc, id, section, part, isStream)
	local webhandler_info, err = __GetWebhandlerInfo(vcc)
	if not webhandler_info then
		LogWarning("[ALLEGRO.REPOSITORY:Retrieve] Unable to retrieve webhandler_info")
		return false, err
	end
	local url = "mediaserver/action/download/repository/" .. id .. "/" .. section .. "/" .. part .. "/"
	local webhandler_ip = webhandler_info.ip
	local webhandler_port = webhandler_info.port
	local headers = {}
	local cookie = webhandler_info.credentials
	local allegro_request = require "apps.allegro.request"
	if isStream then
		return allegro_request.StreamGet(url, headers, cookie, webhandler_ip, webhandler_port)
	end
	return allegro_request.Get(url, headers, cookie, webhandler_ip, webhandler_port)
end

local function GetFileFromAllegro (params)
	--LogDebug("[ALLEGRO.REPOSITORY:GetFileFromAllegro] params: %s", json.encode(params))
	local vcc = inconcert.authUser.virtualContactCenter
	local success, request, errorCode, headers = Retrieve(vcc, params.id, params.section, params.part, params.isStream)
	if not success then
		local err = request
		return false, {stauts = false, reason = "error getting media", errorCode = errorCode, err = err}
	end
	--LogDebug("[ALLEGRO.REPOSITORY:GetFileFromAllegro]-> request length: '%s'", #(request or ""))
	return true, request
end

local function DownloadFile (wsapi_env)
	local response = wsapi.response.new(200, { ["Content-Type"]= "text/plain" })
	local webParameters = utils.ParseWebQueryString(wsapi_env.QUERY_STRING)
	webParameters.isStream = false -- download

	LogInfo("[ALLEGRO.REPOSITORY:DownloadFile] Downloading Allegro File. params: %s", utils.JSON_Encode(webParameters))

	local success, file = GetFileFromAllegro(webParameters)
	if not success then
		local err = file
		local status = 400
		if (err.err or ""):upper() == "ALLEGRO INTEGRATION DISABLED IN THIS VCC" then
			status = 403
		end
		wsapi_env.status = status
		local response = wsapi.response.new(200, {["Content-Type"]= "text/plain"})
		local encodedErr = json.encode(err)
		response:write(encodedErr)
		response.headers["Response"] = encodedErr
		response.status = status
		return response:finish()
	end

	require "LuaOrm.Date"
	local updateDate = LuaOrm.Date(os.time()):toString("%Y-%m-%d %H:%M:%S")
	local suggestedFileName = string.format("file_id_%s_section_%s_part_%s_date_%s.wav", webParameters.id, webParameters.section, webParameters.part, updateDate)
	local headers = {
		["Response"]            = "defaultError",
		["Content-Type"]        = "application/octet-stream",
		["Content-Length"]      = #(file or ""),
		["Response"]            = "ok",
		["Content-Disposition"] = [[attachment; filename="]] .. suggestedFileName .. '"',
		["Set-Cookie"]          = "download_started=true; Path=/" -- used by the jquery download plugin
	}

	LogInfo("[ALLEGRO.REPOSITORY:DownloadFile] (END) Donwloaded File Name '%s'.", suggestedFileName)

	--local coroutineIter = 0
	return 200, headers, coroutine.wrap(function()
		-- coroutineIter = coroutineIter + 1
		--LogDebug("[ALLEGRO.REPOSITORY:DownloadFile] coroutine.yield - iter:%d", coroutineIter)
		coroutine.yield(file)
	end)
end

--[[

]]--
local function StreamFile (wsapi_env)
	LogInfo("[ALLEGRO.REPOSITORY:StreamFile] Streaming Allegro File.")
	return DownloadFile(wsapi_env)
end

---------------------------------------------------------------------------------
-- URLs routed to functions
---------------------------------------------------------------------------------
function MapUris (module, htmlRender, prefix)
	module:dispatch_wsapi( UploadTemporalFile    , prefix .. "/upload/([^.]*)"     )
	module:dispatch_get  ( RemoveTemporalFile    , prefix .. "/remove/(.*)"        )
	module:dispatch_post ( RemoveAllTemporalFiles, prefix .. "/remove_all/([^.]*)" )
	--
	module:dispatch_wsapi( DownloadFile          , prefix .. "/download/([^.]*)"   )
	module:dispatch_wsapi( StreamFile            , prefix .. "/stream/([^.]*)"     )
end

