module(..., package.seeall, inconcert.orbit.application.Decorate)

local utils = require "shared.apps.utils"
local proxy = require "apps.allegro.proxy"

-- campaign_id is optional
local function __LoadDNISs (web, campaign_id)

	local vcc = inconcert.authUser.virtualContactCenter
	LogInfo("[ALLEGRO.ADMINISTRATION__LoadDNISs] Loading availables DNISs in '%s' vcc for '%s' campaign.", vcc, campaign_id)

	web.headers["Content-Type"] = "application/json"
	local data, err = proxy.administration.LoadDNISs( campaign_id )
	if data then
		LogInfo("[ALLEGRO:__LoadDNISs] Successfully loaded %d dnis.", #data.rows)
		return data.rows
	end

	-- Error
	web.status = 500
	local err_type = {
		type = "ALLEGRO_INTEGRATION_ERROR",
		err  = type(err) == "table" and err.reason or err or "ERROR_LOADING_AVAILABLES_DNIS"
	}
	LogError("[ALLEGRO:__LoadDNISs] Error loading dnis. Err: %s", json.encode(err_type))

	return err_type
end

--[[
	A esta funcion se la deja publica ya que a la misma es invocada desde este mismo controllers, y desde los controllers de la app "outboundengine".
--]]
function LoadProcesses (web)

	local vcc = inconcert.authUser.virtualContactCenter
	LogInfo("[ALLEGRO.ADMINISTRATION.LoadProcesses] Loading processes in '%s' vcc.", vcc)

	web.headers["Content-Type"] = "application/json"
	local data, err = proxy.administration.LoadProcesses()
	if data then
		LogInfo("[ALLEGRO.ADMINISTRATION.LoadProcesses] Successfully loaded %d processes.", #data.rows)
		return data.rows
	end

	-- Error
	web.status = 500
	local err_type = {
		type = "ALLEGRO_INTEGRATION_ERROR",
		err  = type(err) == "table" and err.reason or err or "ERROR_LOADING_PROCESSES"
	}
	LogError("[ALLEGRO.ADMINISTRATION.LoadProcesses] Error loading processes. Err: %s", json.encode(err_type))

	return err_type
end

local function __LoadRoutingGroups (web)

	local vcc = inconcert.authUser.virtualContactCenter
	LogInfo("[ALLEGRO.ADMINISTRATION__LoadRoutingGroups] Loading routing groups in '%s' vcc.", vcc)

	web.headers["Content-Type"] = "application/json"
	local data, err = proxy.administration.LoadRoutingGroups()
	if data then
		LogInfo("[ALLEGRO:__LoadRoutingGroups] Successfully loaded %d routing groups.", #data.rows)
		return data.rows
	end

	-- Error
	web.status = 500
	local err_type = {
		type = "ALLEGRO_INTEGRATION_ERROR",
		err  = type(err) == "table" and err.reason or err or "ERROR_LOADING_ROUTING_GROUPS"
	}
	LogError("[ALLEGRO:__LoadRoutingGroups] Error loading routing groups. Err: %s", json.encode(err_type))

	return err_type
end

local function __LoadOutboundProcesses (web)

	local vcc = inconcert.authUser.virtualContactCenter
	LogInfo("[ALLEGRO.ADMINISTRATION__LoadOutboundProcesses] Loading outbound processes in '%s' vcc.", vcc)

	web.headers["Content-Type"] = "application/json"
	local data, err = proxy.administration.LoadOutboundProcesses()
	if data then
		LogInfo("[ALLEGRO:__LoadOutboundProcesses] Successfully loaded %d outbound processes.", #data.rows)
		return data.rows
	end

	-- Error
	web.status = 500
	local err_type = {
		type = "ALLEGRO_INTEGRATION_ERROR",
		err  = type(err) == "table" and err.reason or err or "ERROR_LOADING_OUTBOUND_PROCESS"
	}
	LogError("[ALLEGRO:__LoadOutboundProcesses] Error loading outbound processes. Err: %s", json.encode(err_type))

	return err_type
end

local function __LoadOutboundProcessesTypes (web)

	local vcc = inconcert.authUser.virtualContactCenter
	LogInfo("[ALLEGRO.ADMINISTRATION__LoadOutboundProcessesTypes] Loading outbound processes types in '%s' vcc.", vcc)

	web.headers["Content-Type"] = "application/json"
	local data, err = proxy.administration.LoadOutboundProcessesTypes()
	if data then
		LogInfo("[ALLEGRO:__LoadOutboundProcessesTypes] Successfully loaded %d outbound processes types.", #data.rows)
		return data.rows
	end

	-- Error
	web.status = 500
	local err_type = {
		type = "ALLEGRO_INTEGRATION_ERROR",
		err  = type(err) == "table" and err.reason or err or "ERROR_LOADING_OUTBOUND_PROCESS_TYPE"
	}
	LogError("[ALLEGRO:__LoadOutboundProcessesTypes] Error loading outbound processes types. Err: %s", json.encode(err_type))

	return err_type
end

local function __LoadVoiceBlaster (web)

	local vcc = inconcert.authUser.virtualContactCenter
	LogInfo("[ALLEGRO.ADMINISTRATION__LoadVoiceBlaster] Loading voice blaster in '%s' vcc.", vcc)

	web.headers["Content-Type"] = "application/json"
	local data, err = proxy.administration.LoadVoiceBlaster()
	if data then
		LogInfo("[ALLEGRO:__LoadVoiceBlaster] Successfully loaded %d voice blaster.", #data.rows)
		return data.rows
	end

	-- Error
	web.status = 500
	local err_type = {
		type = "ALLEGRO_INTEGRATION_ERROR",
		err  = type(err) == "table" and err.reason or err or "ERROR_LOADING_OUTBOUND_VOICE_BLASTER"
	}
	LogError("[ALLEGRO:__LoadVoiceBlaster] Error loading voice blaster. Err: %s", json.encode(err_type))

	return err_type
end

local function __LoadAlgorithms (web)

	local vcc = inconcert.authUser.virtualContactCenter
	LogInfo("[ALLEGRO.ADMINISTRATION__LoadAlgorithms] Loading algorithms in '%s' vcc.", vcc)

	web.headers["Content-Type"] = "application/json"
	local data, err = proxy.administration.LoadAlgorithms()
	if data then
		LogInfo("[ALLEGRO:__LoadAlgorithms] Successfully loaded %d algorithms.", #data.rows)
		return data.rows
	end

	-- Error
	web.status = 500
	local err_type = {
		type = "ALLEGRO_INTEGRATION_ERROR",
		err  = type(err) == "table" and err.reason or err or "ERROR_LOADING_OUTBOUND_ALGORITHMS"
	}
	LogError("[ALLEGRO:__LoadAlgorithms] Error loading algorithms. Err: %s", json.encode(err_type))

	return err_type
end

-- user_id is optional
local function __LoadAvailablesPeers (web, user_id)

	local vcc = inconcert.authUser.virtualContactCenter
	LogInfo("[ALLEGRO.ADMINISTRATION__LoadAvailablesPeers] Loading availables Peers in '%s' vcc for '%s' user.", vcc, user_id)

	web.headers["Content-Type"] = "application/json"
	local data, err = proxy.administration.LoadAvailablesPeers( user_id )
	if data then
		LogInfo("[ALLEGRO:__LoadAvailablesPeers] Successfully loaded %d Peers.", #data.rows)
		return data.rows
	end

	-- Error
	web.status = 500
	local err_type = {
		type = "ALLEGRO_INTEGRATION_ERROR",
		err  = type(err) == "table" and err.reason or err or "ERROR_LOADING_AVAILABLES_PEERS"
	}
	LogError("[ALLEGRO:__LoadAvailablesPeers] Error loading Peers. Err: %s", json.encode(err_type))

	return err_type
end

--[[
data = { rows = { 
		{
			name = "basic1",
			description = "basic1",
			status = "untracked",
			id = "basic1",
			created_at = "2016-08-11 17:18:55",
			version = 0
		},
		..
		..
]]--
local function __LoadTweetyDesignerProjects (web)

	local vcc = inconcert.authUser.virtualContactCenter
	LogInfo("[ALLEGRO.ADMINISTRATION__LoadTweetyDesignerProjects] Loading tweety designer projects in '%s' vcc.", vcc)

	web.headers["Content-Type"] = "application/json"
	local data, err = proxy.administration.LoadTweetyDesignerProjects()

	if data then
		LogInfo("[ALLEGRO:__LoadTweetyDesignerProjects] Successfully loaded %d tweety designer projects.", #data.rows)
		return data.rows
	end

	-- Error
	web.status = 500
	local err_type = {
		type = "ALLEGRO_INTEGRATION_ERROR",
		err  = type(err) == "table" and err.reason or err or "ERROR_LOADING_TWEETY_DESIGNER_PROJECTS"
	}
	LogError("[ALLEGRO:__LoadTweetyDesignerProjects] Error loading tweety designer projects. Err: %s", json.encode(err_type))

	return err_type
end

local function __LoadDialerContactResults (web)

	local vcc = inconcert.authUser.virtualContactCenter
	LogInfo("[ALLEGRO.ADMINISTRATION__LoadDialerContactResults] Loading dialer contact results in '%s' vcc.", vcc)

	web.headers["Content-Type"] = "application/json"
	local data, err = proxy.administration.LoadDialerContactResults()

	if data then
		LogInfo("[ALLEGRO:__LoadDialerContactResults] Successfully loaded %d dialer contact results.", #data.rows)
		return data.rows
	end

	-- Error
	web.status = 500
	local err_type = {
		type = "ALLEGRO_INTEGRATION_ERROR",
		err  = type(err) == "table" and err.reason or err or "ERROR_DIALER_CONTACT_RESULTS"
	}
	LogError("[ALLEGRO:__LoadDialerContactResults] Error loading dialer contact results. Err: %s", json.encode(err_type))

	return err_type
end

local function __GetLoginCredentials (web)

	local vcc = inconcert.authUser.virtualContactCenter
	LogInfo("[ALLEGRO.ADMINISTRATION__GetLoginCredentials] Retrieving allegro login credentials '%s' vcc.", vcc)

	web.headers["Content-Type"] = "application/json"
	local data, err = proxy.administration.GetLoginCredentials()

	if data then
		LogInfo("[ALLEGRO:__GetLoginCredentials] Successfully retrieved allegro login credentials '%s'.", json.encode(data.rows or {}))
		return data.rows
	end

	-- Error
	web.status = 500
	local err_type = {
		type = "ALLEGRO_INTEGRATION_ERROR",
		err  = type(err) == "table" and err.reason or err or "ERROR_RETRIEVING_LOGIN_CREDENTIALS"
	}
	LogError("[ALLEGRO:__GetLoginCredentials] Error retrieving login credentials. Err: %s", json.encode(err_type))

	return err_type
end

local function __LoadAvailablesTrunksForRoutingGroup (web, routingGroupId)

	local vcc = inconcert.authUser.virtualContactCenter
	LogInfo("[ALLEGRO:__LoadAvailablesTrunksForRoutingGroup] Loading availables Trunks in '%s' vcc for '%s' Routing Group.", vcc, routingGroupId)

	web.headers["Content-Type"] = "application/json"
	local data, err = proxy.administration.LoadAvailablesTrunksForRoutingGroup(routingGroupId)
	if data then
		LogInfo("[ALLEGRO:__LoadAvailablesTrunksForRoutingGroup] Successfully loaded %d availables Trunks for '%s' Routing Group.", #data.rows, routingGroupId)
		return data.rows
	end

	-- Error
	web.status = 500
	local err_type = {
		type = "ALLEGRO_INTEGRATION_ERROR",
		err  = type(err) == "table" and err.reason or err or "ERROR_LOADING_AVAILABLES_ROUTING_GROUPS"
	}
	LogError("[ALLEGRO:__LoadAvailablesTrunksForRoutingGroup] Error loading availables Trunks fro '%s' Routing Group. Err: %s", routingGroupId, json.encode(err_type))

	return err_type
end

function MapUris (module, htmlRender, prefix)
	utils.DispatchGet( module, __LoadDNISs                 , ( prefix .. "/dnis/load_all/([^/]*)"                   ))
	utils.DispatchGet( module, LoadProcesses               , ( prefix .. "/processes/load_all/([^/]*)"              ))
	utils.DispatchGet( module, __LoadRoutingGroups         , ( prefix .. "/routinggroup/load_all/([^/]*)"           ))
	utils.DispatchGet( module, __LoadOutboundProcesses     , ( prefix .. "/outboundprocess/load_all/([^/]*)"        ))
	utils.DispatchGet( module, __LoadOutboundProcessesTypes, ( prefix .. "/outboundprocess_type/load_all/([^/]*)"   ))
	utils.DispatchGet( module, __LoadVoiceBlaster          , ( prefix .. "/voiceblaster/load_all/([^/]*)"           ))
	utils.DispatchGet( module, __LoadAlgorithms            , ( prefix .. "/algorithms/load_all/([^/]*)"             ))
	utils.DispatchGet( module, __LoadAvailablesPeers       , ( prefix .. "/peers/availables/([^/]*)"                ))
	utils.DispatchGet( module, __LoadTweetyDesignerProjects, ( prefix .. "/webdesigner/designer/catalogapp/([^/]*)" ))
	utils.DispatchGet( module, __LoadDialerContactResults  , ( prefix .. "/dialer/contactresult/load_all/([^/]*)"   ))
	utils.DispatchGet( module, __GetLoginCredentials       , ( prefix .. "/login/credentials/get/([^/]*)"           ))

	utils.DispatchGet( module, __LoadAvailablesTrunksForRoutingGroup, (prefix .. "/routinggroup/availables/trunks/load_all/([^/]*)"))
end
