module(..., package.seeall, inconcert.orbit.application.Decorate)

local utils = require "shared.apps.utils"
local model_perm = require "shared.permissions.models.permissions"
local models = require("apps.agent.model"):GetModels()
local proxy = require "apps.allegro.proxy"

local function GetAsteriskIp (web)
	web.headers["Content-Type"] = "application/json"
	local ip, err = proxy.asterisk.GetIp()
	if not ip then
		return json.encode({ status = false, reason = err })
	end
	return json.encode({ status = true, ip = ip })
end

local function GetInteractionRecords (web, interactionId)
	web.headers["Content-Type"] = "application/json"
	local records, err = proxy.supervisor.GetInteractionRecords(interactionId)
	if not records then
		return json.encode({ status = false, reason = err })
	end
	return json.encode({ status = true, records = records.records })
end

--[[
	Función que obtiene el detalle de las partes de una interacción de CALL, según el VCC que ha recibido por parámetro.
	Esta función/url es invocado desde la rocka de summarizer, para obtener el detalle de las interaccione de CALL, en un report->schedule con contenido.
]]--
local function GetInteractionRecordsOfVcc (web, interactionId, vcc)
	web.headers["Content-Type"] = "application/json"
	local records, err = proxy.supervisor.GetInteractionRecordsOfVcc(interactionId, vcc)
	if not records then
		return json.encode({ status = false, reason = err })
	end
	return json.encode({ status = true, records = records.records })
end

function StartScreenStreaming(agentId)
	if not utils.HasAllegroIntegration() then
		return false, "Vcc without interaction"
	end
	return proxy.Post(
		inconcert.authUser.virtualContactCenter,
		json.encode({ supervisorId = inconcert.authUser.userName, agentId = agentId }),
		"supervisor/start_screen_streaming/" -- la URL de 5.5
	)
end

function StopScreenStreaming(agentId)
	if not utils.HasAllegroIntegration() then
		return false, "Vcc without interaction"
	end
	return proxy.Post(
		inconcert.authUser.virtualContactCenter,
		json.encode({ supervisorId = inconcert.authUser.userName, agentId = agentId }),
		"supervisor/stop_screen_streaming/" -- la URL de 5.5
	)
end

function MapUris (module, htmlRender, prefix)
	module:dispatch_get(GetAsteriskIp             , prefix .. "/get_asterisk_ip/(.*)"                       )
	module:dispatch_get(GetInteractionRecords     , prefix .. "/get_interaction_records/(.+)"               )
	module:dispatch_get(GetInteractionRecordsOfVcc, prefix .. "/get_interaction_records_of_vcc/([^/]+)/(.*)")
end
