local utils = require "shared.apps.utils"
local permissions = require "shared.permissions.permissions"
local allegro = require "apps.allegro.proxy"
local m_models = require("apps.allegro.model"):GetModels()

module(..., package.seeall, inconcert.orbit.application.Decorate)

m_htmlRender = nil
m_appLocation = nil

function MapUris (module, htmlRender, prefix, appLocation)
	m_htmlRender = htmlRender
	m_appLocation = appLocation
end

function HasIntegration (vcc_id) return allegro.HasIntegration(vcc_id) end

local function GetModel (entity)
	if not m_models or not m_models[entity] then
	-- Si el modelo no se ha creado aun, debo crearlo.
	--  Esto sucede en algunas ocasiones si es que me han invocado desde otra app Ej: "Allegro" y aun no se ha creado el modelo.
	--  Entonces tengo que crear el modelo.
		local model = require "apps.allegro.model"
		model.CreateModelTable()
		m_models = model:GetModels()
	end
	return m_models[entity]
end

local function __UpdateValues (user_omnichannel, vcc)
	local full_user_id = utils.AppendVcc(user_omnichannel.Id, vcc)

	if not user_omnichannel.newUser and not user_omnichannel.Password then
		-- Si estoy modificando el usuario, debo obtener la Password, para que sea enviada a 5.5, ya que ahi siempre se espera la pwd,
		--   ya sea que se esta creando o modificando el usuario.
		user_omnichannel.Password = utils.GetColumnValue {
			keyspace     = "Agent",
			columnFamily = "Users",
			key          = full_user_id,
			column       = "Password"
		}
		if not user_omnichannel.Password then
			return false, {err = "ERROR_RETRIEVING_USER_PASSWORD_FROM_CASSANDRA"}
		end
	end

	user_omnichannel.groups = user_omnichannel.groups or {}

	local keyspace = utils.GetDatasource():CassandraConnection("Agent")

	-- si tiene acceso a las apps
	user_omnichannel.AllegroIntegration = user_omnichannel.AllegroIntegration or {}

	local applicationsActions = permissions.GetApplicationsActions(keyspace, vcc)

	local function HasPermissionToAction (appId, actionId)
	--	LogDebug("--allegro.users.__UpdateValues--------------------------------------------------")
		local has_permission_to_action = nil

		if user_omnichannel.custom_permissions then
		-- 1 - Valido contra los override que llegan del lado del cliente
			if user_omnichannel.custom_permissions.actions then
				if user_omnichannel.custom_permissions.actions[appId] then
					has_permission_to_action = user_omnichannel.custom_permissions.actions[appId][actionId]
					--LogDebug(" 1-  (client side) reviso contra el override que viene client side. appId: '%s', actionId: '%s', has_permission_to_action: '%s'", appId, actionId, has_permission_to_action)
				end
			end
		end
		if has_permission_to_action == nil then
			----2 - Valido contra la informacion de la BD, server side. TODO: Ver si aplica (**).

			-- has_permission_to_action = permissions.IsPermissionEnabledForSpecificUser(
			-- 	appId,
			-- 	actionId,
			-- 	full_user_id,
			-- 	user_omnichannel.UserGroup,
			-- 	nil, nil, nil,
			-- 	keyspace,
			-- 	applicationsActions
			-- )
			--LogDebug(" 2-  (server side) reviso contra los permisos server side. appId: '%s', actionId: '%s', has_permission_to_action: '%s'", appId, actionId, has_permission_to_action)
		end
		--LogDebug(" ---> app: '%s', actionId: '%s', full_user_id: '%s', HAS_PERMISSION_TO_ACTION: '%s'", appId, actionId, full_user_id, has_permission_to_action)
		return has_permission_to_action
	end

	user_omnichannel.AllegroIntegration.allowed_apps = {
		administration = {
			app_id = "administration",
			has_permission = permissions.HasPermissionToApplication("administration", full_user_id, user_omnichannel.UserGroup, nil, keyspace, {applicationsActions = applicationsActions}),
			--[[
				La tabla "isAllowedByOmnichannelAction" es utilizada para realizar validaciones contra acciones de apps
				de i6 que habilitan o inhabilitan una app de Allegro.

				Ej: con la acción "view_flows" de la app "administration" de i6, se habilita o inhabilita la app "designer" de "Allegro"
			
				Estos valores se definen y calculan en Allegro en:
					\Web\inConcert\apps\omnichannelintegration\templates.lua 
					__GetMappedAppsPermissions
					MakeAllegroUser
			]]--
			allowed_actions = {
				{
					actionId = "view_flows",
					has_permission = nil,
					validate_permission = function(self, appId)
						self.has_permission = HasPermissionToAction(appId, self.actionId)
					end
				},
				{
					actionId = "view_apps_launcher",
					has_permission = nil,
					-- relatedAllegroApps = {
					-- 	scriptinglauncher = true
					-- },
					validate_permission = function(self, appId)
						self.has_permission = HasPermissionToAction(appId, self.actionId)
					end
				}
			}
		},
		interaction = {
			app_id = "interaction",
			has_permission = permissions.HasPermissionToApplication("interaction", full_user_id, user_omnichannel.UserGroup, nil, keyspace, {applicationsActions = applicationsActions})
		},
		supervisor = {
			app_id = "supervisor",
			has_permission = permissions.HasPermissionToApplication("supervisor", full_user_id, user_omnichannel.UserGroup, nil, keyspace, {applicationsActions = applicationsActions})
		}
	}

	for _, appInfo in pairs(user_omnichannel.AllegroIntegration.allowed_apps) do
		--LogDebug("1")
		for _, actionInfo in ipairs(appInfo.allowed_actions or {}) do
			--LogDebug("2")
			if actionInfo.validate_permission then
				--LogDebug("3")
				actionInfo:validate_permission(appInfo.app_id)
				if type(actionInfo.has_permission) == "boolean" then
					if user_omnichannel.custom_permissions then
						-- Si en los custom permiso del lado del cliente NO llega la app, tengo que agregarla (**).
						user_omnichannel.custom_permissions.apps = user_omnichannel.custom_permissions.apps or {}
						user_omnichannel.custom_permissions.apps[appInfo.app_id] = true
					end
				end
				actionInfo.validate_permission = nil
			end
		end
	end

	keyspace:Close()

	if not user_omnichannel.Mail and user_omnichannel.AllegroIntegration.allowed_apps.interaction.has_permission then
		-- Si NO llega un "mail" asociado al usuario y tiene permisos sobre la app Interaction (BarAgent), asigno un Mail VACIO, ya que en 5.5 es requerido
		user_omnichannel.Mail = ""
		--LogDebug("[ALLEGRO.USERS:__UpdateValues] Generate random email '%s' to agent '%s'.", user_omnichannel.Mail, full_user_id)
	end

	if user_omnichannel.Clients then
		-- Si estamos almacenando desde "Personal Settings" nos llega el layout definido. Sanitizamos el Usuario a postear Allegro, eliminando esta información.
		user_omnichannel.Clients = nil
	end

	return true
end

--[[
	Guarda un usuario en 5.5
]]--
function Save (user, vcc_id, allegroOtherInfo)
	assert(type(user) == "table")
	allegroOtherInfo = allegroOtherInfo or {}

	if not HasIntegration(vcc_id) then
		LogWarning("[ALLEGRO.USERS:Save] Unable saving allegro user. VCC '%s' Without integration", vcc_id)
		return true -- no es un error
	end

	local user_omnichannel = utils.DeepCopy(user)
	local full_user_id = utils.AppendVcc(user_omnichannel.Id, vcc_id)

	local success, err_type = __UpdateValues(user_omnichannel, vcc_id)
	if not success then
		local err = { type = "ALLEGRO_INTEGRATION_ERROR", sub_type = err_type }
		LogWarning("[ALLEGRO.USERS:Save] Error saving allegro user: %s. Err: %s.", full_user_id, json.encode(err))
		return false, err
	end

	--[[
		Informacion adicional que tiene que viajar en el POST.
		Ej: La IP del Server Remoto que se utiliza en el history.
	]]--
	if allegroOtherInfo then
		user_omnichannel.otherInfo = allegroOtherInfo
	end

	local encoded_user_omnichannel = json.encode(user_omnichannel)
	LogInfo("[ALLEGRO.USERS:Save] Saving Allegro User: %s", full_user_id)
	--LogDebug("----> [ALLEGRO.USERS:Save] encoded user: %s", encoded_user_omnichannel)

	local success, allegro_err = allegro.Post(
		vcc_id,
		encoded_user_omnichannel,
		"administration/users/save/", -- la URL de 5.5
		nil, -- content type
		nil, -- fixedUrl
		true -- si debe de incluir informacion adicional en el POST
	)

	local err
	if not success then
		err = { type = "ALLEGRO_INTEGRATION_ERROR", sub_type = allegro_err }
		LogWarning("[ALLEGRO.USERS:Save] Error saving allegro user: %s. Err: %s.", full_user_id, json.encode(err))
	end

	return success, err
end

--[[
***********************************************************************************************************
Funciones/URIs utilizadas desde 5.5 via la app "omnichannel integration".

Estas funciones se invocan desde el modulo LUA "integrationapi.lua", cuando desde "Active Directory" se actualizan o desactiva (elimina) un usuario.

Se invocan funciones de los controllers de "Users" de "i6", para que actualicen la informacion correspondiente en CASSANDRA,
  y luego termine invocando a 5.5 para que realice las actualizacion que correspondan en la BD SQL.
***********************************************************************************************************
]]--
local templates = require "apps.allegro.templates"

--[[
	Funcion que crea o actualiza informacion de un usuario en i6
]]--
local function UpdateUserFromAllegro (web)
	local postedData = web.input["post_data"]

	LogInfo("[ALLEGRO.USERS:UpdateUserFromAllegro] Updating user: '%s'.", postedData)

	local allegroUser = json.decode(postedData)
	local vcc = allegroUser.VCC
	local isNew = allegroUser.__isNew or false
	local fromApp = "administration"

	local omnichannelUser = templates.MakeOmnichannelUser(vcc, allegroUser, isNew, fromApp)
	if not omnichannelUser then
		web.status = 500
		return "allegro user error, making omnichannel user template"
	end
	web.input["post_data"] = json.encode(omnichannelUser)

	local moduleAgentUsers = require "apps.agent.controllers.users" -- un asco cambiar, TODO: realizar request!!
	local success, errMsg = pcall(moduleAgentUsers.SaveUser, web)
	if errMsg ~= "" then
		LogWarning("[ALLEGRO.USERS:UpdateUserFromAllegro] Error Updating User. err: '%s'.", errMsg)
		return errMsg
	else
		LogInfo("[ALLEGRO.USERS:UpdateUserFromAllegro] Updated User successfully.")
	end

	web.status = 200
	return ""
end

--[[
	Funcion que desactiva un usuario en i6
]]--
local function DeactivateUserFromAllegro (web)
	-- TODO: accedo derecho al controller de administration, deberia cambiarlo para que realice un request
	local postedData = web.input["post_data"]
	local moduleAdmUsers = require "apps.administration.controllers.users"
	return moduleAdmUsers.Deactivate(web)
end

function MapUris (module, htmlRender, prefix)
	local LogRequest = require("apps.agent.requestFilters").LogRequest
	module:dispatch_post(utils.JsonValidatorReply .. LogRequest .. UpdateUserFromAllegro    , prefix .. "/update/(.*)"    )
	module:dispatch_post(utils.JsonValidatorReply .. LogRequest .. DeactivateUserFromAllegro, prefix .. "/deactivate/(.*)")
	m_htmlRender = htmlRender
end