require "json"
require "base64"
require "socket"
local url = require "socket.url"
local request = require "apps.allegro.request"
local TIMEOUT_AFTER_DISCONNECT_SEC = 3 -- tiempo de espera del sleep despues que realiza el cierre de sesion
local utils = require "shared.apps.utils"

module(..., package.seeall, inconcert.orbit.application.Decorate)

local m_cookie
function GetCookie ()
	return m_cookie
end

function SetCookie (cookie, userName, vcc)
	local fullUserId = utils.AppendVcc(userName, vcc)
	--LogDebug("[ALLEGRO.LOGIN:Login] Setting Cookie: '%s', to Logged User: '%s'.", cookie, fullUserId)
	m_cookie = cookie
end

local function __LoginUser (username, callcenterId, password, applicationName, ip, port)
-- "hash=MTExMTExMTE%3D&username=admin&callcenter=vccprueba&app=dashboard"
-- "hash=MTExMTExMTE%3D&username=admin&callcenter=vccprueba&app=dashboard&previousUrl=/inconcert/apps/dashboard/"

	local params = string.format("hash=%s&username=%s&callcenter=%s&app=%s",
		url.escape(base64.encode(password)),
		url.escape(username),
		callcenterId,
		url.escape(applicationName)
	)
	LogInfo("__LoginUser: '%s'", params)

	local url = "login/login" 
	local headers = {
		["Content-Type"] = 'application/x-www-form-urlencoded; charset=utf-8'
	}
	local result, response = request.Post(url, params, headers, m_cookie, ip, port)
	if result then
		local data = json.decode(response)
		if data.status then
			cookie = "SID=".. data.authToken
			SetCookie(cookie, username, callcenterId)
			return true
		else
			return false, data.failCause
		end
	else
		LogError("Error logging in: "..tostring(response))
	end
	return false, response
end

function Login (user, vcc, password, application, ip, port)
	assert(user)
	assert(vcc)
	assert(password)
	assert(ip)
	assert(port)

	user = user:lower()
	vcc = vcc:lower()
	application = (application or "dashboard"):lower()

	LogInfo("[ALLEGRO.LOGIN:Login] Allegro Login in USER: '%s', VCC: '%s', PASSWORD: '%s', IP: '%s', PORT: '%s', APPLICATION: '%s'.", user, vcc, password, ip, port, application)
	local result, reason = __LoginUser(user, vcc, password, application, ip, port)
	LogInfo("[ALLEGRO.LOGIN:Login] Allegro LOGIN result: '%s', reason: '%s'.", result, reason)

	return result, reason
end

function Logout (cookie, ip, port)
	assert(ip)
	assert(port)
	LogInfo("[ALLEGRO.LOGIN:Logout] Allegro waiting ...logout. COOKIE: '%s', IP: '%s', PORT: '%s'.", cookie, ip, port)
	local result = request.Post("login/logout", nil, nil, cookie, ip, port)
	socket.sleep(TIMEOUT_AFTER_DISCONNECT_SEC)
	SetCookie(nil)
	LogInfo(" logout result:  " .. tostring(result) )
	return result
end
