local utils = require "shared.apps.utils"
module(..., package.seeall)

local m_datasource

function GetName ()
	return "Repository"
end

function GetDefinition (datasource)
	m_datasource = datasource
	return _M
end

local function GetDataSource ()
	if not m_datasource then
		local model = require "apps.allegro.model"
		model.BuildModels(true)
	end
	return m_datasource
end

local function __MakeUploadedFileId (file_info)
	assert(type(file_info) == "table")
	assert(type(file_info.FileId) == "string")

	local vcc = inconcert.authUser.virtualContactCenter
	local campaign_id = file_info.Campaign and utils.AppendVcc(utils.GetId(file_info.Campaign), vcc)

	local key = string.format(
		"Allegro:Repository:Campaign:%s:Account:%s:File:%s",
		campaign_id or "",
		file_info.Account or "allegro_call",
		file_info.FileId
	)

	--LogDebug("[__MakeUploadedFileId]------------> key: '%s'", key)
	return key
end

function SetTemporalUploadedFile (file_info, temporal_file)
	assert(file_info)
	assert(temporal_file)

	utils.CollectAllMemory()
	local key = __MakeUploadedFileId(file_info)
	GetDataSource().redis:set(key, temporal_file)
	utils.CollectAllMemory()
	-- La key expira a los 30 minutos
	local MINUTES_30_IN_SEC = 1800
	GetDataSource().redis:expire(key, MINUTES_30_IN_SEC)

	return key
end

function GetTemporalUploadedFile (file_info)
	assert(file_info)

	local key = __MakeUploadedFileId(file_info)
	return GetDataSource().redis:get(key)
end

function DeleteTemporalUploadedFile (file_info)
	assert(file_info)

	local key = __MakeUploadedFileId(file_info)
	GetDataSource().redis:del(key)
	utils.CollectAllMemory()
	-- valida si el archivo se ha eliminado correctamente
	if GetTemporalUploadedFile(file_info) then
		LogWarning("[DeleteTemporalUploadedFile] Unable to delete temporal file: '%s'", key)
		return false
	end
	return true
end
