local utils = require "shared.apps.utils"
local allegroProxy = require "lua_allegroproxyclient"
local sharedView = require "apps.allegro.shared_view"
local m_moduleLocation = select(2, ...)

module(..., package.seeall, inconcert.orbit.application.Decorate)

local m_allegroClient

local function __GetVCC() return inconcert.authUser.virtualContactCenter end

local function __MakeURI (url)
	return string.format("/inconcert/apps/omnichannelintegration/%s", url)
end

function HasIntegration ( vcc_id )
	assert(type(vcc_id) == "string")

	if not utils.HasAllegroIntegration( vcc_id ) then
		LogInfo("[ALLEGROPROXY.HasIntegration] VCC '%s' has not configured Allegro Integration.", vcc_id)
		return false, "NOT_CONFIGURED_ALLEGRO_INTEGRATION"
	end

	return true
end

--[[
]]--
local function MakeResponseErr (data, err)
	--LogDebug("[ALLEGROPROXY.MakeResponseErr] 1-ENTRA data: %s, err: %s", utils.JSON_Encode(data), utils.JSON_Encode(err))
	local allegroErr = {
		errType = "ALLEGRO_INTEGRATION_ERROR",
		rawErr  = {},
		reason  = "unrecognized allegro error",
		result  = "UNRECOGNIZED ALLEGRO ERROR",
		success = false
	}
	if data and type(data) == "table" then
		-- {"result":"INVALID_JSON_DATA","success":false,"errType":"Expected value but found invalid token at character 1"}
		for name, value in pairs(allegroErr) do
			allegroErr[name] = data[name] or value
		end
		allegroErr.rawErr = utils.DeepCopy(data)
	elseif type(err) == "string" then
		--  "STD::EXCEPTION"
		allegroErr.result = err:upper()
		allegroErr.reason  = err
		allegroErr.rawErr = {err}
	end
	if type(allegroErr.result) == "string" then
		allegroErr.result = allegroErr.result:upper()
		--[[
			A esta pisada hay que realizarla porque hay veces que NO llega en el error como se espera, sino que llega un string (html)
			Entonces recupero el error que se encuentra dentro del TAG <PRE>...</PRE>.

			Este error sucede solo cuando en Allegro existe un error en tiempo de ejecucion, ejemplo cuando forzamos lineas de codigo para que se de un error.
		]]--
		local pattern = "<PRE>(.-)</PRE>"
		local match = string.match(allegroErr.result, pattern)
		if match then
			allegroErr.result = match
		end
		allegroErr.result = allegroErr.result:upper()
		-- Reemplazo el valor solo si:
		--		el error es una Excepcion que se solo se da cuando el servidor de allegro o el webhandler se han caido.
		if allegroErr.result == "STD::EXCEPTION" then
			allegroErr.result = "ALLEGRO SERVER OR WEBHANDLER IS DOWN"
		end
	end
	--LogDebug("[ALLEGROPROXY.MakeResponseErr] 2-SALE allegroErr: %s", json.encode(allegroErr))
	return allegroErr
end

--[[
]]--
function ProcessAllegroPostResponse (data, err)
	--LogDebug("[ALLEGROPROXY.ProcessAllegroPostResponse] 0-----> data: '%s', err: '%s'", data, err)
	if not data then
		LogDebug("[ALLEGROPROXY.ProcessAllegroPostResponse] 1- return err")
		return false, MakeResponseErr(data, err)
	end
	local ok, decoded_data = pcall(json.decode, data)
	if ok then
		--LogDebug("[ALLEGROPROXY.ProcessAllegroPostResponse] 2-decode OK")
		data = decoded_data
	end
	if type(data) ~= "table" then
		--LogDebug("[ALLEGROPROXY.ProcessAllegroPostResponse] 3- return err")
		return false, MakeResponseErr(data, err)
	end
	if not data.success and not data.status then
		--LogDebug("[ALLEGROPROXY.ProcessAllegroPostResponse] 4- return err")
		return false, MakeResponseErr(data, err)
	end
	--LogDebug("[ALLEGROPROXY.ProcessAllegroPostResponse] 5- return OK")
	return true, data
end

--[[
]]--
local function __ProcessAllegroGetResponse (data, err)
	--LogDebug("[ALLEGROPROXY.__ProcessAllegroGetResponse] -----> data: '%s', err: '%s'", data, err)
	if not data then
		return false, err
	end
	local ok, decoded_data = pcall(json.decode, data)
	if ok then
		data = decoded_data
	end
	if type(data) ~= "table" then
		return false, err
	end
	if not data.status then
		return false, data.reason
	end
	return data
end

local function __Get (url, fixedUrl, requestId, vcc)
	--LogDebug("[ALLEGROPROXY.__Get] -----> url: '%s', fixedUrl: '%s', requestId: '%s', vcc: '%s'.", url, fixedUrl, requestId, vcc)
	local resp, err = __ProcessAllegroGetResponse(DoGet(url, vcc, fixedUrl, requestId))
	--LogDebug("[ALLEGROPROXY.__Get] -----> resp: '%s', err: '%s'", json.encode(resp), err)
	return resp, err
end

function Post (vcc_id, post_data, url, content_type, fixedUrl, mustIncludeOmnichannelInfo)
	assert(vcc_id)
	assert(post_data)
	assert(url)

	if not HasIntegration( vcc_id ) then
		return { success = true, type = "WITHOUT_ALLEGRO_INTEGRATION" } -- no es un error
	end

	content_type = content_type or "application/json"

	local encoded_result, response = DoPost(url, post_data, content_type, vcc_id, fixedUrl, mustIncludeOmnichannelInfo)
	--LogDebug("[ALLEGROPROXY.Post] -----> encoded_result: '%s', response: '%s'", encoded_result, response)

	return ProcessAllegroPostResponse(encoded_result, response)
end

local function RawGet(url, fixedUrl)
	local data, err = DoGet(url, nil, fixedUrl)
	if not data then
		return false, err
	end
	return true, data
end

local function RawPost (vcc_id, post_data, url, content_type, fixedUrl, mustIncludeOmnichannelInfo)
	assert(vcc_id)
	assert(post_data)
	assert(url)
	content_type = content_type or "application/xml"

	if not HasIntegration( vcc_id ) then
		return false, "has not integration"
	end

	local data, err = DoPost(url, post_data, content_type, vcc_id, fixedUrl, mustIncludeOmnichannelInfo)
	if not data then
		return false, err
	end
	return true, data
end

local function __GetAllegroClient ()
	if not m_allegroClient then
		LogInfo("[ALLEGROPROXY.__GetAllegroClient] Creating Allegro Proxy. Cluster: %s", inconcert.environment.apps.agent.cluster)
		m_allegroClient = allegroProxy.LuaAllegroProxyClient(inconcert.environment.apps.agent.cluster, "webhandler")
		for _, address in pairs(inconcert.environment.apps.agent.allegroproxy) do
			LogInfo("[ALLEGROPROXY.__GetAllegroClient] Add Known Allegro Proxy. Cluster. IP: '%s', Port: '%s'.", address.ip, address.port)
			m_allegroClient:AddKnownAllegroProxy(address.ip, address.port)
		end
	end
	return m_allegroClient
end

function DoGet (path, vcc, fixedUrl, requestId)
	local url = fixedUrl and path or __MakeURI(path)
	vcc = vcc or inconcert.authUser.virtualContactCenter
	--LogDebug("[ALLEGROPROXY.DoGet] vcc:'%s', url: '%s', requestId: '%s'.", vcc, url, requestId)
	return __GetAllegroClient():DoGet(vcc, url, requestId)
end

local function IncludeOmnichannelInfo (body, info)
	assert(body)
	body.__OmnichannelInfo = {
		userId = body.Id or inconcert.authUser.userName,
		vcc = info.vcc,
		otherInfo = info.otherInfo
	}
	--LogDebug("[ALLEGROPROXY.IncludeOmnichannelInfo] Additional Info: '%s'.", json.encode(body.__OmnichannelInfo))
	local ok, newEncodedBody = pcall(json.encode, body)
	if ok then
		return true, newEncodedBody
	end
	LogWarning("[ALLEGROPROXY.IncludeOmnichannelInfo] Unable to encode Additional Info'.")
	return false
end

function DoPost (path, encodedBody, contentType, vcc, fixedUrl, mustIncludeOmnichannelInfo)
	local url = fixedUrl and path or __MakeURI(path)
	vcc = vcc or inconcert.authUser.virtualContactCenter
	local requestId, otherInfo
	local ok, body = pcall(json.decode, encodedBody)
	if ok then
		-- required
		otherInfo = body.otherInfo
		body.otherInfo = nil
		-- optional
		requestId = body.__requestId
		body.__requestId = nil
	else
		LogWarning("[ALLEGROPROXY.DoPost] Unable to decode body'.")
	end
	if mustIncludeOmnichannelInfo then
		local success, newEncodedBody = IncludeOmnichannelInfo(body, {
			vcc = vcc,
			otherInfo = otherInfo
		})
		if success then
			encodedBody = newEncodedBody
		end
	end

	--LogInfo("[ALLEGROPROXY.DoPost] vcc:'%s', url: '%s', bodyLength: '%s', contentType: '%s', requestId: '%s'.", vcc, url, #encodedBody, contentType, requestId)
	--LogDebug("[ALLEGROPROXY.DoPost] encodedBody: '%s'.", encodedBody)

	return __GetAllegroClient():DoPost(vcc, url, encodedBody, contentType, requestId)
end

-- GENERIC
generic = {
	LoginAgentFromToken = function (token)
		local vcc = inconcert.authUser.virtualContactCenter
		local agentId = inconcert.authUser.userName
		return __GetAllegroClient():LoginAgentFromToken(vcc, agentId, token)
	end,
	AuthenticateUser = function (vcc, userId, password)
		return __GetAllegroClient():AuthenticateUser(vcc, userId, password)
	end,
	GetWebhandlerInfo = function (vcc)
		local info, err = __GetAllegroClient():GetWebhandlerInfo(vcc)
		LogInfo("[ALLEGROPROXY:GetWebhandlerInfo] Allegro webhandler Information: '%s', err: '%s'", json.encode(info or {}), err)
		return info, err
	end,
	GetAgentLastKickout = function(userid)
		return __Get("agent/get_agent_lastkickout/" .. userid)
	end
}

-- SUPERVISOR
supervisor = {
	GetInteractionRecords = function(interactionId)    return __Get("supervisor/get_interaction_records/" .. interactionId)                   end,
	GetInteractionRecordsOfVcc = function(interactionId, vcc)
		--[[
			Función que obtiene el detalle de las partes de una interacción de CALL, según el VCC que ha recibido por parámetro.
			Esta función/url es invocada desde la rocka de summarizer, para obtener el detalle de las interacciones de CALL, en un report->schedule con contenido.
		]]--
		local url = "supervisor/get_interaction_records/" .. interactionId
		local fixedUrl, requestId 
		return __Get(url, fixedUrl, requestId, vcc)
	end
}

-- ADMINISTRATION
administration = {
	LoadDNISs = function(simple_campaign_id)           return __Get("resources/dnis/load_all/" .. (simple_campaign_id or ""))                 end,
	LoadProcesses = function()                         return __Get("resources/processes/load_all/")                                          end,
	LoadRoutingGroups = function()                     return __Get("resources/routinggroup/load_all/")                                       end,
	LoadOutboundProcesses = function()                 return __Get("resources/outboundprocess/load_all/")                                    end,
	LoadOutboundProcessesTypes = function()            return __Get("resources/outboundprocess_type/load_all/")                               end,
	LoadVoiceBlaster = function()                      return __Get("resources/voiceblaster/load_all/")                                       end,
	LoadAlgorithms = function()                        return __Get("resources/algorithms/load_all/")                                         end,
	LoadAvailablesPeers = function(simple_user_id)     return __Get("resources/peers/availables/" .. (simple_user_id or ""))                  end,
	LoadTweetyDesignerProjects = function()            return __Get("resources/webdesigner/designer/catalogapp/")                             end,
	LoadDialerContactResults = function()              return __Get("resources/dialer/contactresult/load_all/")                               end,
	-- routing groups
	LoadPaginatedRoutingGroups = function(encodedInfo) return sharedView.Search("administration/routinggroups/load/", encodedInfo)            end,
	GetRoutingGroup = function(routingGroupId)         return __Get("administration/routinggroups/get/" .. (routingGroupId or ""))            end,
	SaveRoutingGroup = function(encodedInfo)           return Post(__GetVCC(), encodedInfo, "administration/routinggroups/save/")             end,
	DeleteRoutingGroup = function(encodedInfo)         return Post(__GetVCC(), encodedInfo, "administration/routinggroups/delete/")           end,
	LoadAvailablesTrunksForRoutingGroup = function(routingGroupId)
		return __Get("administration/routinggroups/trunks/availables/load/" .. (routingGroupId or ""))
	end,
	RoutingGroupCheckMaxAssigned = function(routingGroupId)
		return __Get("administration/routinggroups/check_max_assigned/" .. (routingGroupId or ""))
	end
}

-- ASTERISK
asterisk = {
	GetIp = function (vcc)
		vcc = vcc or inconcert.authUser.virtualContactCenter
		return __GetAllegroClient():GetAsteriskIp(vcc)
	end,
	ValidateDialingRule = function(encodedInfo, routingGroupId)
		return Post(__GetVCC(), encodedInfo, "asterisk/validate/dialing_rule/" .. (routingGroupId or ""))
	end,
	DeleteDialingRule = function(routingGroupId, dialingRuleId)
		routingGroupId = routingGroupId or ""
		dialingRuleId = dialingRuleId or ""
		return __Get("asterisk/delete/dialing_rule/" .. routingGroupId .. "/" .. dialingRuleId)
	end
}

local m_mustIncludeOmnichannelInfo = true

-- OUTBOUND ENGINE
outboundEngine = {
	-- outboundengine contacts
	LoadContacts          = function(encodedInfo)               return sharedView.Search("outboundengine/contacts/load/"          , encodedInfo)                 end,
	LoadContactsToMigrate = function(encodedInfo)               return sharedView.Search("outboundengine/contacts/migration/load/", encodedInfo)                 end,
	GetContact = function(contactId, vccId)
		local url = "outboundengine/contacts/get/" .. (contactId or "")
		local vcc = vccId or nil
		local fixedUrl = nil
		local requestId = nil
		return __Get(url, fixedUrl, requestId, vcc)
	end,
	LoadContactsBatchFromProcess = function(processId) return __Get("outboundengine/contacts/batch/load/" .. (processId or ""))               end,

	-- outboundengine sdk
	SdkSetDisposition = function(encodedInfo)          return RawPost(__GetVCC(), encodedInfo, "/inconcert/apps/outboundengine/sdk/set_disposition_code/", nil, true)  end,
	SdkSetDispositionJson = function(encodedInfo)      return Post(__GetVCC(), encodedInfo, "/inconcert/apps/outboundengine/sdk/set_disposition_code_json/", nil, true)  end,
	SdkSearchContacts = function(encodedInfo)          return RawPost(__GetVCC(), encodedInfo, "/inconcert/apps/outboundengine/sdk/search_contacts/", nil, true)  end,
	SdkSearchContactsJson = function(encodedInfo)      return Post(__GetVCC(), encodedInfo, "/inconcert/apps/outboundengine/sdk/search_contacts_json/", nil, true)  end,
	SdkRecognizePhone = function(encodedInfo)          return RawPost(__GetVCC(), encodedInfo, "/inconcert/apps/outboundengine/sdk/recognize_phone/", nil, true)  end,
	SdkRecognizePhoneJson = function(encodedInfo)      return Post(__GetVCC(), encodedInfo, "/inconcert/apps/outboundengine/sdk/recognize_phone_json/", nil, true)  end,
	SdkAddContact = function(encodedInfo)              return RawPost(__GetVCC(), encodedInfo, "/inconcert/apps/outboundengine/sdk/add_contact/", nil, true)  end,
	SdkAddContactJson = function(encodedInfo)          return Post(__GetVCC(), encodedInfo, "/inconcert/apps/outboundengine/sdk/add_contact_json/", nil, true)  end,
	SdkAddContactValues = function(encodedInfo)        return RawPost(__GetVCC(), encodedInfo, "/inconcert/apps/outboundengine/sdk/add_contact_values/", nil, true)  end,
	SdkAddContactSkills = function(encodedInfo)        return RawPost(__GetVCC(), encodedInfo, "/inconcert/apps/outboundengine/sdk/add_contact_skills/", nil, true)  end,
	SdkAddPhone = function(encodedInfo)                return RawPost(__GetVCC(), encodedInfo, "/inconcert/apps/outboundengine/sdk/add_phone/", nil, true)  end,
	SdkAssignAgent = function(encodedInfo)             return RawPost(__GetVCC(), encodedInfo, "/inconcert/apps/outboundengine/sdk/assign_agent/", nil, true)  end,
	SdkAddContactToBatch = function(encodedInfo)       return RawPost(__GetVCC(), encodedInfo, "/inconcert/apps/outboundengine/sdk/add_contact_to_batch/", nil, true)  end,
	SdkCancelContact = function(encodedInfo)           return RawPost(__GetVCC(), encodedInfo, "/inconcert/apps/outboundengine/sdk/cancel_contact/", nil, true)  end,
	SdkPauseContact = function(encodedInfo)            return RawPost(__GetVCC(), encodedInfo, "/inconcert/apps/outboundengine/sdk/pause_contact/", nil, true)  end,
	SdkResumeContact = function(encodedInfo)           return RawPost(__GetVCC(), encodedInfo, "/inconcert/apps/outboundengine/sdk/resume_contact/", nil, true)  end,
	SdkFindContactInBatch = function(encodedInfo)      return RawPost(__GetVCC(), encodedInfo, "/inconcert/apps/outboundengine/sdk/find_contact_in_batch/", nil, true)  end,
	SdkCancelPhone = function(encodedInfo)             return RawPost(__GetVCC(), encodedInfo, "/inconcert/apps/outboundengine/sdk/cancel_phone/", nil, true)  end,
	SdkChangePriority = function(encodedInfo)          return RawPost(__GetVCC(), encodedInfo, "/inconcert/apps/outboundengine/sdk/change_contact_priority/", nil, true)  end,
	SdkDisablePhone = function(encodedInfo)            return RawPost(__GetVCC(), encodedInfo, "/inconcert/apps/outboundengine/sdk/disable_phone/", nil, true)  end,
	SdkEnablePhone = function(encodedInfo)             return RawPost(__GetVCC(), encodedInfo, "/inconcert/apps/outboundengine/sdk/enable_phone/", nil, true)  end,
	SdkFinishContact = function(encodedInfo)           return RawPost(__GetVCC(), encodedInfo, "/inconcert/apps/outboundengine/sdk/finish_contact/", nil, true)  end,
	SdkReScheduleContact = function(encodedInfo)       return RawPost(__GetVCC(), encodedInfo, "/inconcert/apps/outboundengine/sdk/reschedule_contact/", nil, true)  end,
	SdkReScheduleContactJson = function(encodedInfo)   return Post(__GetVCC(), encodedInfo, "/inconcert/apps/outboundengine/sdk/reschedule_contact_json/", nil, true)  end,
	SdkReScheduleContactToPhone = function(encodedInfo)return RawPost(__GetVCC(), encodedInfo, "/inconcert/apps/outboundengine/sdk/reschedule_contact_to_phone/", nil, true)  end,
	SdkReScheduleContactToPhoneJson = function(encodedInfo) return Post(__GetVCC(), encodedInfo, "/inconcert/apps/outboundengine/sdk/reschedule_contact_to_phone_json/", nil, true)  end,
	SdkReScheduleContactCampaign = function(encodedInfo)    return RawPost(__GetVCC(), encodedInfo, "/inconcert/apps/outboundengine/sdk/reschedule_contact_campaign/", nil, true)  end,
	SdkReScheduleContactCampaignJson = function(encodedInfo)return Post(__GetVCC(), encodedInfo, "/inconcert/apps/outboundengine/sdk/reschedule_contact_campaign_json/", nil, true)  end,
	SdkRecycleContact = function(encodedInfo)          return RawPost(__GetVCC(), encodedInfo, "/inconcert/apps/outboundengine/sdk/recycle_contact/", nil, true)  end,
	SdkExtendBatch = function(encodedInfo)             return RawPost(__GetVCC(), encodedInfo, "/inconcert/apps/outboundengine/sdk/extendbatch/", nil, true)  end,
	SdkDeleteBatch = function(encodedInfo)             return RawPost(__GetVCC(), encodedInfo, "/inconcert/apps/outboundengine/sdk/delete_batch/", nil, true)  end,
	SdkExpireBatch = function(encodedInfo)             return RawPost(__GetVCC(), encodedInfo, "/inconcert/apps/outboundengine/sdk/expire_batch/", nil, true)  end,
	SdkStopProcess = function(encodedInfo)             return RawPost(__GetVCC(), encodedInfo, "/inconcert/apps/outboundengine/sdk/stop_process/", nil, true)  end,
	SdkStartProcess = function(encodedInfo)            return RawPost(__GetVCC(), encodedInfo, "/inconcert/apps/outboundengine/sdk/start_process/", nil, true)  end,
	SdkStatusProcess = function(encodedInfo)           return RawPost(__GetVCC(), encodedInfo, "/inconcert/apps/outboundengine/sdk/status_process/", nil, true)  end,
	SdkDeleteContactImport = function(encodedInfo)     return RawPost(__GetVCC(), encodedInfo, "/inconcert/apps/outboundengine/sdk/delete_contact_import/", nil, true)  end,
	SdkCreateFormatImport = function(encodedInfo)      return RawPost(__GetVCC(), encodedInfo, "/inconcert/apps/outboundengine/sdk/create_format_import/", nil, true)  end,
	SdkCreateBatchFromFilter = function(encodedInfo)   return RawPost(__GetVCC(), encodedInfo, "/inconcert/apps/outboundengine/sdk/create_batch_from_filter/", nil, true)  end,
	SdkCreateFilter = function(encodedInfo)            return RawPost(__GetVCC(), encodedInfo, "/inconcert/apps/outboundengine/sdk/save_filter/", nil, true)  end,
	SdkCreateBatchFromImport = function(encodedInfo)   return RawPost(__GetVCC(), encodedInfo, "/inconcert/apps/outboundengine/sdk/create_batch_from_import/", nil, true)  end,
	SdkGetContact = function(contactId)                return RawGet("/inconcert/apps/outboundengine/sdk/get_contact/" .. (contactId or ""), true) end,
	SdkGetContactJson = function(contactId)            return __Get("/inconcert/apps/outboundengine/sdk/get_contact_json/" .. (contactId or ""), true) end,
	SdkGetImportStatus = function(importId)            return RawGet("/inconcert/apps/outboundengine/sdk/get_import_status/?ImportId=" .. (importId or ""), true)  end,
	SdkGetImportDetail = function(importId)            return RawGet("/inconcert/apps/outboundengine/sdk/get_import_detail/?ImportId=" .. (importId or ""), true)  end,
	SdkGetAllOutboundCampaigns = function()            return RawGet("/inconcert/apps/outboundengine/sdk/get_all_campaigns/", true)  end,
	SdkGetImportFormats = function()                   return RawGet("/inconcert/apps/outboundengine/sdk/get_import_formats/", true)  end,
	SdkGetHistoricalInteraction = function(id)         return RawGet("/inconcert/apps/outboundengine/sdk/get_historical_interaction/" .. (id or ""), true)  end,

	-- outboundengine exports
	StartExportation = function(encodedInfo)           return Post(__GetVCC(), encodedInfo, "outboundengine/exports/start/")                  end,
	LoadExports = function(encodedInfo)                return sharedView.Search("outboundengine/exports/load/", encodedInfo)                  end,
	DeleteExport = function(encodedInfo)               return Post(__GetVCC(), encodedInfo, "outboundengine/exports/delete/")                 end,
	CancelExport = function(encodedInfo)               return Post(__GetVCC(), encodedInfo, "outboundengine/exports/cancel/")                 end,
	DownloadExport = function(encodedInfo)             return Post(__GetVCC(), encodedInfo, "outboundengine/exports/download/")               end,

	-- outboundengine export layout
	LoadPaginatedExportLayouts = function(encodedInfo) return sharedView.Search("outboundengine/exports/layout/paginated/load/", encodedInfo) end,
	SaveExportLayout = function(encodedInfo)           return Post(__GetVCC(), encodedInfo, "outboundengine/exports/layout/save/")            end,
	GetExportLayout = function(layoutId)               return __Get("outboundengine/exports/layout/get/" .. (layoutId or ""))                 end,
	DeleteExportLayout = function(encodedInfo)         return Post(__GetVCC(), encodedInfo, "outboundengine/exports/layout/delete/")          end,
	LoadExportLayouts = function()                     return __Get("outboundengine/exports/layout/load_all/")                                end,

	-- outboundengine imports
	LoadImports = function(encodedInfo)                return sharedView.Search("outboundengine/imports/load/", encodedInfo)                  end,
	GetImport = function(importId)                     return __Get("outboundengine/imports/get/" .. (importId or ""))                        end,
	DeleteImport = function(encodedInfo)               return Post(__GetVCC(), encodedInfo, "outboundengine/imports/delete/")                 end,
	CancelImport = function(encodedInfo)               return Post(__GetVCC(), encodedInfo, "outboundengine/imports/cancel/")                 end,
	DepurateImport = function(encodedInfo)             return Post(__GetVCC(), encodedInfo, "outboundengine/imports/depurate/",
																	nil, nil, m_mustIncludeOmnichannelInfo)                                   end,
	HasSomeImportRunning = function()                  return __Get("outboundengine/imports/has_some_importation_running/")                   end,
--[[
 Esta función se deja comentada (Ref #8690), ya que esta acción NO va por "allegroProxy", sino que va directamente desde el webhandler de i6 al de 5.5
--	StartImport = function(encodedInfo)                return Post(__GetVCC(), encodedInfo, "outboundengine/imports/start/")                  end,
]]--

	-- outboundengine import formats
	LoadImportFormats = function(encodedInfo)          return sharedView.Search("outboundengine/imports/formats/load/", encodedInfo)          end,
	DeleteImportFormat = function(encodedInfo)         return Post(__GetVCC(), encodedInfo, "outboundengine/imports/formats/delete/")         end,
	LoadImportsFormats = function()                    return __Get("outboundengine/imports/formats/load_all/")                               end,
	GetImportFormat = function(formatId)               return __Get("outboundengine/imports/formats/get/" .. (formatId or ""))                end,
	SaveImportFormat = function(encodedInfo)           return Post(__GetVCC(), encodedInfo, "outboundengine/imports/formats/save/")           end,

	-- outboundengine processes
	LoadProcesses = function(encodedInfo)              return sharedView.Search("outboundengine/processes/load/", encodedInfo)                end,
	GetProcess = function(processId, vcc)            return
		__Get(
			"outboundengine/processes/get/" .. (processId or ""),
			nil, -- fixedUrl
			nil, -- requestId
			vcc
		)
	end,
	DeleteProcess = function(encodedInfo)              return
		Post(
			__GetVCC(),
			encodedInfo,
			"outboundengine/processes/delete/",
			nil, -- content type
			nil, -- fixedUrl
			true -- si debe de incluir informacion adicional en el POST
		)
	end,
	StartProcess = function(encodedInfo)               return Post(__GetVCC(), encodedInfo, "outboundengine/processes/start/")                end,
	StopProcess = function(encodedInfo)                return Post(__GetVCC(), encodedInfo, "outboundengine/processes/stop/")                 end,
	SaveProcess = function(encodedInfo)
		return Post(
			__GetVCC(),
			encodedInfo,
			"outboundengine/processes/save/",
			nil, -- content type
			nil, -- fixedUrl
			true -- si debe de incluir informacion adicional en el POST
		)
	end,
	LoadBatchContactFields = function(processId)       return __Get("outboundengine/processes/batch/fields/load_all/" .. (processId or ""))   end,

	-- outboundengine processes - scheduler
	LoadAvailableSchedulers = function()               return __Get("outboundengine/processes/schedulers/available/load_all/")                end,
	GetScheduler = function(schedulerId, vcc) -- vcc no requerido (utilizado para las migraciones de Allegro a Nativo)
		assert(schedulerId)
		return __Get(
			"outboundengine/processes/schedulers/get/" .. schedulerId,
			nil, -- fixedUrl
			nil, -- requestId
			vcc
		)
	end,
	SaveScheduler = function(encodedInfo)
		return Post(
			__GetVCC(),
			encodedInfo,
			"outboundengine/processes/schedulers/save/",
			nil, -- content type
			nil, -- fixedUrl
			true -- si debe de incluir informacion adicional en el POST
		)
	end,

	-- outboundengine processes - filters
	LoadBatchFilters = function(vcc) -- vcc no requerido (utilizado para las migraciones de Allegro a Nativo)
		return __Get(
			"outboundengine/processes/batch/filters/load_all/",
			nil, -- fixedUrl
			nil, -- requestId
			vcc
		)
	end,
	SaveBatchFilter = function(encodedInfo)            return Post(__GetVCC(), encodedInfo, "outboundengine/processes/batch/filters/save/")   end,
	DeleteBatchFilter = function(encodedInfo)          return Post(__GetVCC(), encodedInfo, "outboundengine/processes/batch/filters/delete/") end,

	-- outboundengine contacts batches (abm)
	CreateBatch = function(queryId, encodedInfo)
		-- (*) Aca utiliza el Id de request para que en Allegro caiga en la misma Lua VM (el mismo llega en el encodedInfo en el atributo "__requestId")
		return Post(__GetVCC(), encodedInfo, "outboundengine/processes/batch/contacts/create/" .. queryId)
	end,
	UpdateBatchByQuery = function(queryId, processId, encodedInfo)
		-- (*) Aca utiliza el Id de request para que en Allegro caiga en la misma Lua VM (el mismo llega en el encodedInfo en el atributo "__requestId")
		return Post(__GetVCC(), encodedInfo, "outboundengine/processes/batch/contacts/update/by_query/" .. queryId .. "/" .. processId)
	end,
	UpdateBatchByAction = function(batchId, processId, encodedInfo)
		return Post(__GetVCC(), encodedInfo, "outboundengine/processes/batch/contacts/update/by_batch/" .. batchId .. "/" .. processId)
	end,
	DeleteBatch = function(encodedInfo) 
		return Post(__GetVCC(), encodedInfo, "outboundengine/processes/batch/contacts/delete/")
	end,
	RunBatchOrContactsQuery = function(encodedInfo, rowsPerPage, source)
		-- A este request le llega un GUID como id de request. El mismo va a ser utilizado en (*)
		return Post(__GetVCC(), encodedInfo, "outboundengine/processes/batch_or_contacts/query/run/" .. rowsPerPage .. "/" .. source)
	end,
	LoadContactsFromQuery = function(queryId, start, count, requestId)
		-- (*) Aca utiliza el Id de request para que en Allegro caiga en la misma Lua VM
		local url = "outboundengine/processes/batch/contacts/load/fromquery/%s/?start=%s&count=%s"
		url = string.format(url, queryId, start, count)
		LogInfo("[ALLEGROPROXY.outboundEngine.LoadContactsFromQuery] URL: '%s'.", url)
		return __Get(url, nil, requestId)
	end,
	LoadBatchContactsStatus = function(processId, batchId)
		return __Get("outboundengine/processes/batch/contacts/status/" .. processId .. "/" .. batchId)
	end,

	-- outboundengine system
	LoadEngineServices = function()               return __Get("outboundengine/system/engine/services/load_all/")                                                end,
	LoadEngineTypes = function()                  return __Get("outboundengine/system/engine/types/load_all/")                                                   end,
	LoadResponsesByChannel = function(channel)    return __Get("outboundengine/system/rules/responsesby/channel/" .. (channel or ""))                            end,
	LoadDestinations = function()                 return __Get("outboundengine/system/rules/destinations/load_all/")                                             end,
	-- outboundengine system (imports formats)
	LoadImportFieldsTypes = function()            return __Get("outboundengine/system/imports/fieldstypes/load_all/")                                            end,
	LoadScheduledDateFormats = function()         return __Get("outboundengine/system/imports/scheduleformat/load_all/")                                         end,
	LoadCountryList = function()                  return __Get("outboundengine/system/imports/countries/load_all/")                                              end,
	LoadContactDestinations = function()          return __Get("outboundengine/system/imports/destinations/load_all/")                                           end,
	LoadContactMailDestinations = function()      return __Get("outboundengine/system/imports/maildestinations/load_all/")                                       end,
	LoadContactAddressDestinations = function()   return __Get("outboundengine/system/imports/addressdestinations/load_all/")                                    end,
	LoadCustomColumns = function()                return __Get("outboundengine/system/imports/customcolumns/load_all/")                                          end,
	LoadReservedWords = function()                return __Get("outboundengine/system/imports/reservedwords/load_all/")                                          end,
	LoadImportations = function()                 return __Get("outboundengine/system/imports/load_all/")                                                        end,
	LoadTimeZones = function()                    return __Get("outboundengine/system/timezone/load_all/")                                                       end,
	LoadCountries = function()                    return __Get("outboundengine/system/countries/load_all/")                                                      end,
	LoadAreas = function()                        return __Get("outboundengine/system/areas/load_all/")                                                          end,
	-- outboundengine system (export)
	LoadBatchesByCampaign = function(campaign)    return __Get("outboundengine/system/batches/by_campaign/" .. (campaign or ""))                                 end,
	LoadExportLayoutColumnTypes = function()      return __Get("outboundengine/system/exports/columnstypes/load_all/")                                           end
}

soaIntegration = {
	GetVideoStreamingServiceIp = function(vcc)    return RawGet("/inconcert/apps/soaintegration/get_videostreaming_serviceip/" .. (vcc or ""), true)             end,
	GetVideoStreamingSessionId = function(vcc)    return RawGet("/inconcert/apps/soaintegration/get_videostreaming_sessionid/" .. (vcc or ""), true)             end,
	RecognizePhone = function(encodedInfo)        return RawPost(__GetVCC(), encodedInfo, "/inconcert/apps/soaintegration/recognize_phone/", nil, true)          end,
	ManualCallBack = function(encodedInfo)        return RawPost(__GetVCC(), encodedInfo, "/inconcert/apps/soaintegration/manual_callback/", nil, true)          end,
	DialerCallBack = function(encodedInfo)        return RawPost(__GetVCC(), encodedInfo, "/inconcert/apps/soaintegration/dialer_callback/", nil, true)          end,
	ExecuteFlowState = function(encodedInfo)      return RawPost(__GetVCC(), encodedInfo, "/inconcert/apps/soaintegration/execute_flow_state/", nil, true)       end,
	AddContactsToCampaign = function(encodedInfo) return RawPost(__GetVCC(), encodedInfo, "/inconcert/apps/soaintegration/add_contacts_to_campaign/", nil, true) end,
	ReloadCampaign = function(encodedInfo)        return RawPost(__GetVCC(), encodedInfo, "/inconcert/apps/soaintegration/reload/reload_campaign/", nil, true)   end
}

-- PROVISIONING
provisioning = {
	GetVCC = function(vcc)
		local url = "provisioning/vccs/get/" .. vcc
		--LogDebug("[ALLEGROPROXY.provisioning.GetVCC] --> URL: '%s'.", url)
		return __Get(url, nil, nil, vcc)
	end
}
