/*jshint bitwise:true, browser:true, curly:true, eqeqeq:true, noarg:true, noempty:true, strict:true, undef:true, unused:true, esversion: 9 */
/*jshint quotmark:false*/
/*global inConcert, $*/


if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	"use strict";
	var FORMAT = inConcert.Utils.format;
	var REQUEST_TIMEOUT_MS = 300000; // 5 minutes

	function API( params )
	{
		if(typeof( params ) !== "object") {
			throw( "API.Constructor - expected an object with configuration" );
		}
		if(typeof( params.uri ) !== "string") {
			throw( "API.Constructor - base uri is required" );
		}

		this.uri = params.uri;
		this.asterisk_ip = null;
		this.cached_dialer_contact_results = null;
		this.cached_screen_recording_percentage = null;
		this.m_token = null;
	}

	if (typeof inConcert.AllegroAPI !== "function") {
		inConcert.AllegroAPI = API;
	}


/* -----------------------------------------------------------------------------------------
FUNCIONES GENERICAS
--------------------------------------------------------------------------------------------*/

	/*
	
	*/
	API.prototype.getBaseURI = function() {
		return this.uri;
	};

	API.prototype.getRequestTimeout = function() {
		return REQUEST_TIMEOUT_MS;
	};

	/*
	
	*/
	API.prototype.__Get = function(uri, callback)
	{
		if (typeof(uri) !== "string") {
			throw("API.prototype.__Get: uri variable must be string type.");
		}
		if (typeof(callback) !== "function") {
			throw("API.prototype.__Get: callback variable must be function type.");
		}

		var request_handler = inConcert.invoke({
			url    : uri,
			type   : "GET",
			timeout: REQUEST_TIMEOUT_MS,
			success: function (list, result, server_response)
			{
				if (typeof(server_response) === "object") {
					server_response = JSON.stringify(server_response);
				}
				//console.log(uri + "\nlist: " + list + "\nresult: " + result + "\nserver_response: " + server_response);
				if (result.toLowerCase() === "success") {
					callback(true, list);
				}
				else {
					callback(false, server_response);
				}
			},
			error: function (code, error) {callback(false, error);}
		});
		return request_handler;
	};

	/*
	
	*/
	API.prototype.__Post = function(uri, callback, data)
	{
		if (typeof(uri) !== "string") {
			throw("API.prototype.__Post: uri variable must be string type.");
		}
		if (typeof(callback) !== "function") {
			throw("API.prototype.__Post: callback variable must be function type.");
		}
		if (typeof(data) !== "object") {
			throw("API.prototype.__Post: data variable must be object type.");
		}

		var request_handler = inConcert.invoke({
			url        : uri,
			type       : "POST",
			data       : JSON.stringify(data),
			dataType   : "json",
			contentType: "application/json",
			timeout    : REQUEST_TIMEOUT_MS,
			success    : function (response)    { callback(true, response); },
			error      : function (code, error) { callback(false, error);   }
		});
		return request_handler;
	};

	/*
	
	*/
	API.prototype.__Delete = function(uri, callback, data)
	{
		if (typeof(uri) !== "string") {
			throw("API.prototype.__Delete: uri variable must be string type.");
		}
		if (typeof(callback) !== "function") {
			throw("API.prototype.__Delete: callback variable must be function type.");
		}

		var request_handler = inConcert.invoke({
			url        : uri,
			type       : "DELETE",
			data       : JSON.stringify(data || {}),
			contentType: "application/json",
			timeout    : REQUEST_TIMEOUT_MS,
			success    : function (response) { callback(true, response); },
			error      : function (code, error) {callback(false, error);}
		});
		return request_handler;
	};

	API.prototype.getRequestTimeout = function() { return REQUEST_TIMEOUT_MS; };

/* -----------------------------------------------------------------------------------------
COMMON Requests
--------------------------------------------------------------------------------------------*/

	API.prototype.getAllegroCampaign = function(full_campaign_id) {
		return inConcert.app.get_id( full_campaign_id );
	};

/* -----------------------------------------------------------------------------------------
ADMINISTRATION Requests
--------------------------------------------------------------------------------------------*/

	API.prototype.loadScreenRecordingPercentage = function( callback )
	{
		/*
		 * Esta funcion no es un Request contra el Server, se realiza igual a como esta en Allegro (Su obtencion es client side)
		 */
		callback = callback || $.noop;
		var self = this;

		// Aunque no sea un request contra el server, tiene que estar dentro de un timeout para que funcione de forma asincronica, y cada uno de los request posteriores funcionen de la misma forma segun quien me ha invocado.
		window.setTimeout(function() {
			if (!self.cached_screen_recording_percentage) {
				self.cached_screen_recording_percentage = [];
				var step = 5;
				var start = step;
				var end = 100;
				for (var i = start; i <= end; i += step) {
					var percentage = i.toString();
					self.cached_screen_recording_percentage.push({
						id: percentage,
						description: percentage + " %"
					});
				}
			}
			callback( true, self.cached_screen_recording_percentage);
		});
	};

	API.prototype.loadAfterConversationWorkTime = function( callback )
	{
		/*
		 * Esta funcion no es un Request contra el Server, se realiza igual a como esta en Allegro (Su obtencion es client side)
		 */
		callback = callback || $.noop;

		// Aunque no sea un request contra el server, tiene que estar dentro de un timeout para que funcione de forma asincronica, y cada uno de los request posteriores funcionen de la misma forma en quien me ha invocado.
		window.setTimeout(function() {
			callback( true, [
				{ id: "TimeLimited", description: "<%=_T('Time limited')%>"  },
				{ id: "Unlimited"  , description: "<%=_T('No time Limit')%>" }
			]);
		});
	};

	/*
		campaign_id : puede venir con o sin vcc, da lo mismo, ya que en esta funcion me tengo que quedar sin el VCC
	*/
	API.prototype.loadDNISs = function( campaign_id, callback )
	{
		/*
		 * sample URI:
		 *   http://192.168.4.11:8082/inconcert/apps/allegro/resources/dnis/load_all/camp1
		 */
		callback = callback || $.noop;

		var simple_camp_id = inConcert.app.get_id( campaign_id || "" );
		var uri = FORMAT( "{0}/resources/dnis/load_all/{1}", this.uri, simple_camp_id );

		var request_handler = inConcert.invoke({
			url         : uri,
			type        : "GET",
			contentType : "application/json",
			dataType    : "json",
			timeout     : REQUEST_TIMEOUT_MS,
			success     : function (list) {
				callback( true, list );
			},
			error       : function (code, error) { callback( false, error ); }
		});

		return request_handler;
	};

	API.prototype.loadProcesses = function( callback )
	{
		/*
		 * sample URI:
		 *   http://192.168.4.11:8082/inconcert/apps/allegro/resources/processes/load_all/
		 */
		callback = callback || $.noop;

		var uri = FORMAT( "{0}/resources/processes/load_all/", this.uri );

		var request_handler = inConcert.invoke({
			url         : uri,
			type        : "GET",
			contentType : "application/json",
			dataType    : "json",
			timeout     : REQUEST_TIMEOUT_MS,
			success     : function (list) {
				callback( true, list );
			},
			error       : function (code, error) { callback( false, error ); }
		});

		return request_handler;
	};

	API.prototype.loadRoutingGroups = function( callback )
	{
		/*
		 * sample URI:
		 *   http://192.168.4.11:8082/inconcert/apps/allegro/resources/routinggroup/load_all/
		 */
		callback = callback || $.noop;

		var uri = FORMAT( "{0}/resources/routinggroup/load_all/", this.uri );

		var request_handler = inConcert.invoke({
			url         : uri,
			type        : "GET",
			contentType : "application/json",
			dataType    : "json",
			timeout     : REQUEST_TIMEOUT_MS,
			success     : function (list) {
				callback( true, list );
			},
			error       : function (code, error) { callback( false, error ); }
		});

		return request_handler;
	};

	API.prototype.loadOutboundProcesses = function( callback )
	{
		/*
		 * sample URI:
		 *   http://192.168.4.11:8082/inconcert/apps/allegro/resources/outboundprocess/load_all/
		 */
		callback = callback || $.noop;

		var uri = FORMAT( "{0}/resources/outboundprocess/load_all/", this.uri );

		var request_handler = inConcert.invoke({
			url         : uri,
			type        : "GET",
			contentType : "application/json",
			dataType    : "json",
			timeout     : REQUEST_TIMEOUT_MS,
			success     : function (list) {
				callback( true, list );
			},
			error       : function (code, error) { callback( false, error ); }
		});

		return request_handler;
	};

	API.prototype.loadOutboundProcessesTypes = function( callback )
	{
		/*
		 * sample URI:
		 *   http://192.168.4.11:8082/inconcert/apps/allegro/resources/outboundprocess_type/load_all/
		 */
		callback = callback || $.noop;

		var uri = FORMAT( "{0}/resources/outboundprocess_type/load_all/", this.uri );

		var request_handler = inConcert.invoke({
			url         : uri,
			type        : "GET",
			contentType : "application/json",
			dataType    : "json",
			timeout     : REQUEST_TIMEOUT_MS,
			success     : function (list) {
				callback( true, list );
			},
			error       : function (code, error) { callback( false, error ); }
		});

		return request_handler;
	};

	API.prototype.loadVoiceBlaster = function( callback )
	{
		/*
		 * sample URI:
		 *   http://192.168.4.11:8082/inconcert/apps/allegro/resources/voiceblaster/load_all/
		 */
		callback = callback || $.noop;

		var uri = FORMAT( "{0}/resources/voiceblaster/load_all/", this.uri );

		var request_handler = inConcert.invoke({
			url         : uri,
			type        : "GET",
			contentType : "application/json",
			dataType    : "json",
			timeout     : REQUEST_TIMEOUT_MS,
			success     : function (list) {
				callback( true, list );
			},
			error       : function (code, error) { callback( false, error ); }
		});

		return request_handler;
	};

	API.prototype.loadAlgorithms = function( callback )
	{
		/*
		 * sample URI:
		 *   http://192.168.4.11:8082/inconcert/apps/allegro/resources/algorithms/load_all/
		 */
		callback = callback || $.noop;

		var uri = FORMAT( "{0}/resources/algorithms/load_all/", this.uri );

		var request_handler = inConcert.invoke({
			url         : uri,
			type        : "GET",
			contentType : "application/json",
			dataType    : "json",
			timeout     : REQUEST_TIMEOUT_MS,
			success     : function (list) {
				callback( true, list );
			},
			error       : function (code, error) { callback( false, error ); }
		});

		return request_handler;
	};

	API.prototype.loadAvailablesPeers = function( user_id, callback )
	{
		/*
		 * sample URI:
		 *   http://192.168.4.11:8082/inconcert/apps/allegro/resources/peers/availables/agent1
		 */
		callback = callback || $.noop;

		var simple_user_id = inConcert.app.get_id( user_id || "" );
		var uri = FORMAT( "{0}/resources/peers/availables/{1}", this.uri, simple_user_id );

		var request_handler = inConcert.invoke({
			url         : uri,
			type        : "GET",
			contentType : "application/json",
			dataType    : "json",
			timeout     : REQUEST_TIMEOUT_MS,
			success     : function (list) {
				callback( true, list );
			},
			error       : function (code, error) { callback( false, error ); }
		});

		return request_handler;
	};

	API.prototype.loadCCIProjects = function( callback )
	{
		/*
		 * sample URI:
		 *   http://192.168.4.11:8082/inconcert/apps/allegro/resources/webdesigner/designer/catalogapp/
		 */
		callback = callback || $.noop;

		var uri = FORMAT( "{0}/resources/webdesigner/designer/catalogapp/", this.uri );

		var request_handler = inConcert.invoke({
			url         : uri,
			type        : "GET",
			contentType : "application/json",
			dataType    : "json",
			timeout     : REQUEST_TIMEOUT_MS,
			success     : function (list) {
				callback( true, list );
			},
			error       : function (code, error) { callback( false, error ); }
		});

		return request_handler;
	};

	API.prototype.loadDialerContactResults = function( callback )
	{
		/*
		 * sample URI:
		 *   http://192.168.4.11:8082/inconcert/apps/allegro/resources/dialer/contactresult/load_all/
		 */
		callback = callback || $.noop;
		var self = this;

		if (this.cached_dialer_contact_results) {
			callback(true, this.cached_dialer_contact_results);
			return;
		}

		var uri = FORMAT( "{0}/resources/dialer/contactresult/load_all/", this.uri );

		var request_handler = inConcert.invoke({
			url         : uri,
			type        : "GET",
			contentType : "application/json",
			dataType    : "json",
			timeout     : REQUEST_TIMEOUT_MS,
			success     : function (list) {
				self.cached_dialer_contact_results = list;
				callback( true, list );
			},
			error       : function (code, error) { callback( false, error ); }
		});

		return request_handler;
	};

	API.prototype.loadAvailablesTrunksFromRoutingGroup = function(routing_group_id, callback)
	{
		/*
		 * sample URI:
		 *   http://192.168.4.11:8082/inconcert/apps/allegro/resources/routinggroup/availables/trunks/load_all/ROUTING_GROUP_ID
		 */
		callback = callback || $.noop;

		var uri = FORMAT( "{0}/resources/routinggroup/availables/trunks/load_all/{1}", this.uri, routing_group_id );

		var request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

	API.prototype.asteriskValidateDialingRule = function(data, routing_group_id, callback)
	{
		/*
		 * sample URI:
		 *   http://192.168.4.11:8082/inconcert/apps/allegro/asterisk/validate/dialing_rule/ROUTING_GROUP_ID
		 */
		callback = callback || $.noop;

		var uri = FORMAT( "{0}/asterisk/validate/dialing_rule/{1}", this.uri, routing_group_id );

		var request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	API.prototype.asteriskDeleteDialingRule = function(routing_group_id, dialing_rule_id, callback)
	{
		/*
		 * sample URI:
		 *   http://192.168.4.11:8082/inconcert/apps/allegro/asterisk/delete/dialing_rule/ROUTING_GROUP_ID/DIALING_RULE_ID
		 */
		callback = callback || $.noop;

		var uri = FORMAT("{0}/asterisk/delete/dialing_rule/{1}/{2}",
			this.uri,
			encodeURIComponent(routing_group_id),
			encodeURIComponent(dialing_rule_id)
		);

		var request_handler = this.__Delete(uri, callback);
		return request_handler;
	};

	API.prototype.routingGroupCheckMaxAssigned = function(routing_group_id, callback)
	{
		/*
		 * sample URI:
		 *   http://192.168.4.11:8082/inconcert/apps/allegro/routinggroup/check_max_assigned/ROUTING_GROUP_ID
		 */
		callback = callback || $.noop;

		var uri = FORMAT( "{0}/routinggroup/check_max_assigned/{1}", this.uri, routing_group_id );

		var request_handler = this.__Get(uri, function(success, info) {
			if (success) {
				callback(true, info);
				return;
			}
			// error
			callback(false, info);
		});
		return request_handler;
	};


/* -----------------------------------------------------------------------------------------
SUPERVISOR Requests
--------------------------------------------------------------------------------------------*/

	API.prototype.getAllegroAsteriskIp = function( callback )
	{
		/*
		 * sample URI:
		 *   http://192.168.4.11:8082/inconcert/apps/allegro/supervisor/get_asterisk_ip/
		 */
		var self = this;
		callback = callback || $.noop;

		if ( this.asterisk_ip ) {
			callback(true, this.asterisk_ip);
			return;
		}

		var uri = FORMAT( "{0}/supervisor/get_asterisk_ip/", this.uri );

		var request_handler = inConcert.invoke({
			url         : uri,
			type        : "GET",
			contentType : "application/json",
			dataType    : "json",
			timeout     : REQUEST_TIMEOUT_MS,
			success     : function (response) {
				self.asterisk_ip = response.ip || null;
				callback( true, self.asterisk_ip );
			},
			error       : function (code, error) { callback( false, error ); }
		});

		return request_handler;
	};

	API.prototype.getInteractionRecords = function( interaction_id, callback )
	{
		/*
		 * sample URI:
		 *   http://192.168.4.11:8082/inconcert/apps/allegro/supervisor/get_interaction_records/interaction_id
		 */
		callback = callback || $.noop;

		var uri = FORMAT( "{0}/supervisor/get_interaction_records/{1}", this.uri, interaction_id );

		var request_handler = inConcert.invoke({
			url         : uri,
			type        : "GET",
			contentType : "application/json",
			dataType    : "json",
			timeout     : REQUEST_TIMEOUT_MS,
			success     : function (response) {
				callback( true, response.records );
			},
			error       : function (code, error) { callback( false, error ); }
		});

		return request_handler;
	};

/* -----------------------------------------------------------------------------------------
AGENT Requests
--------------------------------------------------------------------------------------------*/

	API.prototype.__getAllegroUser = function(full_user_id) {
		return inConcert.app.get_id( full_user_id );
	};

	API.prototype.getAllegroLoggedUser = function() {
		return this.__getAllegroUser(inConcert.app.getLoggedUserId());
	};

	API.prototype.getAllegroState = function(state)
	{
		var vcc_info = inConcert.app.get_vcc_info();
		var states_mapping = vcc_info && vcc_info.AllegroIntegration && vcc_info.AllegroIntegration.mapping && vcc_info.AllegroIntegration.mapping || {};
		//PRUEBA
		states_mapping = {
			ACTIVE    : "Activo",
			NOACD     : "Pause",
			LUNCH     : "Break",
			AWAY      : "Personal",
			BATHROOM  : "Toilette",
			BREAK     : "Break"
		};
		//END
		state = inConcert.app.get_id( state );
		return states_mapping[state] || state;
	};

	API.prototype.loginAgentFromToken = function( callback )
	{
		/*
		 * sample URI:
		 *   http://192.168.4.11:8082/inconcert/apps/allegro/agent/login_agent_from_token/
		 * sample BODY:
		 *   { "token" : "MYTOKEN" }
		 */
		var self = this;
		callback = callback || $.noop;

		if (self.m_token) {
			callback(self.m_token);
			return;
		}
		var uri = FORMAT( "{0}/agent/login_agent_from_token/", this.uri);
		var token = inConcert.BaragentPolling.getAuthToken();
		if (!token) {
			callback(false, "token not found");
			return;
		}

		var request_handler = inConcert.invoke({
			url         : uri,
			type        : "POST",
			data        : JSON.stringify({token : token}),
			dataType    : "json",
			contentType : "application/json",
			timeout     : REQUEST_TIMEOUT_MS,
			success     : function (response) {
				if (response.status) {
					self.m_token = token;
					callback(token);
				}
				else {
					callback(false, response.reason);
				}
			},
			error       : function (code, error) { callback( false, error ); }
		});

		return request_handler;
	};

/* -----------------------------------------------------------------------------------------
OUTBOUNDENGINE Requests
--------------------------------------------------------------------------------------------*/

	API.prototype.setDisposition = function( interaction_id, dispositionCode, callback )
	{
		/*
		 * sample URI:
		 *   http://192.168.4.11:8082/inconcert/apps/allegro/outboundengine_sdk/set_disposition_code_json/
		 */		 
		callback = callback || $.noop;
		if (dispositionCode === undefined) {
			callback(true);
			return;
		}

		let uri = FORMAT( "{0}/outboundengine_sdk/set_disposition_code_json/", this.uri);

		let dialer = dispositionCode.dialer || {};
		let processId = (dialer.outboundProcessId === undefined || dialer.outboundProcessId === null)?"":dialer.outboundProcessId;
		let batchId = (dialer.batchId === undefined || dialer.batchId === null)?"":dialer.batchId;
		let contactId = (dialer.contactId === undefined || dialer.contactId === null)?"":dialer.contactId;
		let contactCode = (dialer.contactCode === undefined || dialer.contactCode === null)?0:dialer.contactCode;
		let contactName = (dialer.contactName === undefined || dialer.contactName === null)?"":dialer.contactName;

		let	disposition = {
			interactionId : interaction_id,
			processId : processId,
			batchId	: batchId,
			contactId : contactId,
			contactName : contactName,
			businessCode : dispositionCode.name,
			contactCode : contactCode || 0
		};	

		let request_handler = inConcert.invoke({
			url         : uri,
			type        : "POST",
			data        : JSON.stringify(disposition),
			dataType    : "json",
			contentType : "application/json",
			timeout     : REQUEST_TIMEOUT_MS,
			success     : function (response) {
				if (response.status) {
					callback(true);
				}
				else {
					callback(false, response.reason);
				}
			},
			error       : function (/*code, error*/) {
				callback( false); 
			}
		});

		return request_handler;
	};


/* -----------------------------------------------------------------------------------------
SOA Requests
--------------------------------------------------------------------------------------------*/

	API.prototype.dialerCallBack = function( interaction_id, dialerData, callback )
	{
		/*
		 * sample URI:
		 *   http://192.168.4.11:8082/inconcert/apps/allegro/soaintegration/dialer_callback/
		 */		 
		callback = callback || $.noop;
		if (dialerData === undefined) {
			callback(true);
			return;
		}

		let uri = FORMAT( "{0}/soaintegration/dialer_callback/", this.uri);

		var xml = '<data>' + 
					'<vcc>' + dialerData.vcc + '</vcc>' +
					'<campaignid>' + dialerData.campaignId + '</campaignid>' +
					'<importid>' + dialerData.importId + '</importid>' +
					'<batchid>' + dialerData.batchId + '</batchid>' +
					(dialerData.batchPriority ? '<batchPriority>' + dialerData.batchPriority + '</batchPriority>' : "") +
				'</data>' +
				'<contact>';

		if (dialerData.contactId !== "" && dialerData.contactId !== undefined) {
					xml = xml  + '<contactId>' + dialerData.contactId + '</contactId>';
		}
		xml = xml + '<name>' + dialerData.contactName + '</name>' +
					(dialerData.name ? '<lastManagementResult>' + dialerData.name + '</lastManagementResult>' : "") + 
					(dialerData.contactPriority ? '<newPriority>' + dialerData.contactPriority + '</newPriority>' : "") +
					'<agentId>' + dialerData.agentId + '</agentId>'
					
		if (dialerData.scheduleDate !== undefined && dialerData.scheduleDate !== "" && isNaN(dialerData.scheduleDate.valueOf()) === false) {
		
			var month = dialerData.scheduleDate.getMonth() + 1;
			
			xml = xml + '<year>' + dialerData.scheduleDate.getFullYear() + '</year>' +
						'<month>' + month + '</month>' +
						'<day>' + dialerData.scheduleDate.getDate() + '</day>' +
						'<hour>' + dialerData.scheduleDate.getHours() + '</hour>' +
						'<minute>' + dialerData.scheduleDate.getMinutes() + '</minute>';
		}

		if (dialerData.timeZone !== undefined && dialerData.timeZone !== "") {
			xml = xml + '<timeZone>' + dialerData.timeZone + '</timeZone>';
		}			
		xml = xml + '<phones>' + 
						'<phone>' + 
							'<phoneType>' + dialerData.phoneType + '</phoneType>' +
							(dialerData.countryId ? '<countryId>' + dialerData.countryId + '</countryId>' : '') +
							(dialerData.areaId ? '<areaId>' + dialerData.areaId + '</areaId>' : '') +
							'<ZIP>' + dialerData.zip + '</ZIP>' +
							'<phoneNumber>' + dialerData.number + '</phoneNumber>' +
							'<phoneExtension>' + dialerData.extension + '</phoneExtension>' +
							'<importName>Telefono</importName>' +
						'</phone>' + 
					'</phones>';
					
		if (dialerData.alternativephone !== undefined && dialerData.alternativephone !== "") {		
		xml = xml + '<phones>' + 
						'<alternativephone>' + 
							'<phoneType>' + dialerData.phoneType + '</phoneType>' +
							(dialerData.alternativephone.countryId ? '<countryId>' + dialerData.alternativephone.countryId + '</countryId>' : '') +
							(dialerData.alternativephone.areaId ? '<areaId>' + dialerData.alternativephone.areaId + '</areaId>' : '') +
							(dialerData.alternativephone.zip ? '<ZIP>' + dialerData.alternativephone.zip + '</ZIP>' : '') +
							'<phoneNumber>' + dialerData.alternativephone.number + '</phoneNumber>' +
							(dialerData.alternativephone.extension ? '<phoneExtension>' + dialerData.alternativephone.extension + '</phoneExtension>' : '') +
							'<importName>Telefono</importName>' +
						'</alternativephone>' + 
					'</phones>';
		}
		
		if (dialerData.extraData) {
			xml = xml + '<data>';
			for (var iuff in dialerData.extraData) {
				if (iuff) {
					xml = xml + '<item>' +
									'<name>' + iuff + '</name>' +
									'<value>' + dialerData.extraData[iuff] +'</value>' +
									'<index>1</index>' +
								'</item>';
				}
			}
			xml = xml + '</data>';

			}
		xml = xml + '</contact>';					
		
		let request_handler = inConcert.invoke({
			url         : uri,
			type        : "POST",
			data        : xml,
			dataType    : "json",
			contentType : "application/json",
			timeout     : REQUEST_TIMEOUT_MS,
			success     : function (response) {
				if (response.status && response.code === 0) {
					callback(true);
				}
				else {
					callback(false, response.message);
				}
			},
			error       : function (code, error) { 
				callback( false); 
			}
		});

		return request_handler;	
	};
	
}());
