/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
  strict:true, undef:false, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:false, esversion: 6 */

/* global inConcert, BaragentController*/

/*
Allegro Authentication
*/

// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.

if (!this.inConcert) {
	this.inConcert = {};
}

(function ()
{
	//constructor
	function AllegroAuthenticator() {
		this.m_urlBase = "/inconcert/apps/";
		this.m_baController = new BaragentController();
	}

	//Define functions and objects in the namespace in case they don't already exist
	if (typeof inConcert.AllegroAuth !== 'object') {
		inConcert.AllegroAuth = new AllegroAuthenticator();
	}

	function isAgentLogged(self, params) {
		//optional params
		let callback = params.callback || $.noop;
		
		self.m_baController.IsAgentLogged(
			function(result) {
				var isLogged = result && result.data && result.data.islogged !== "0";
				callback(isLogged);
			},
			function() {
				callback(false);
			}
		);
	}

	function logoutAgent(self, params) {
		//optional params
		let callback = params.callback || $.noop;
		
		function doLogout() {
			self.m_baController.LogoutAgent(
				function() {
					callback(true);
				},
				function() {
					callback(false, '<%=_T("Error closing telephony components")%>');
				}
			);
		}
		isAgentLogged(self, {
			callback : function(isLogged) {
				if (isLogged) {
					doLogout();
				}
				else {
					callback(true);
				}
			}
		});
	}

	function loginAgent(self, params) {
		//required params
		let password = params.password;
		//optional params
		let vcc = (params.vcc || "").toLowerCase();
		let username = (params.username || "").toLowerCase();
		let callback = params.callback || $.noop;
		

		function doLogin() {
			self.m_baController.LoginAgent(vcc, username, password,
				function() {
					callback(true);
				},
				function() {
					callback(false, '<%=_T("Error loading telephony components")%>');
				}
			);
		}

		logoutAgent(self, {
			callback : function(ok, message) {
				if (ok) {
					doLogin();
				}
				else {
					callback(false, message);
				}
			}
		});
	}

	function hasBaragent(self, params) {
		//optional params
		let userAtVCC = (params.userAtVCC || "").toLowerCase();
		let callback = params.callback || $.noop;
		
		let url = self.m_urlBase + "allegro/login/has_baragent/" + encodeURIComponent(userAtVCC);
		inConcert.invoke({
			url: url,
			dataType: "json",
			contentType: "application/json",
			success: function(response) {
				callback(response && response.baragent);
			},
			error: function(/*response, error*/) {
				callback(false);
			}
		});
	}

	AllegroAuthenticator.prototype.login = function(params) {
		//required params
		let password = params.password;
		//optional params
		let userAtVCC = (params.userAtVCC || "").toLowerCase();
		let mfaAuthData = params.mfaAuthData || null;
		let callback = params.callback || $.noop;
		let mfaFinalStepFn = params.mfaFinalStepFn || $.noop;
		
		var self = this;
		this.m_hasBaragent = false;

		function doLogin() {
			var userAtVCC_a = userAtVCC.split("@");
			var username = userAtVCC_a[0];
			var vcc = userAtVCC_a.length > 1? userAtVCC_a[1] : "";
			loginAgent(self, {
				username : username,
				vcc : vcc,
				password : password,
				callback : function(ok, message) {
					if (ok) {
						//baragent login could be slowly and token expire after 10 seconds
						//then we update it after login it and before login i6
						inConcert.InconcertAuth.getAuthToken({
							callback : function() {
								self.m_hasBaragent = true;
								callback(true);
							}
						});
					}
					else {
						callback(false, message);
					}
				}
			});
		}

		hasBaragent(self, {
			userAtVCC : userAtVCC,
			callback : function(has) {
				if (has) {
					//validate authentication before do login
					//it is because we can't capture different error messages from baragent
					//like password invalid
					inConcert.InconcertAuth.validateAuthentication({
						mfaAuthData : mfaAuthData,
						mfaFinalStepFn : mfaFinalStepFn,
						callback : doLogin
					});
				}
				else {
					callback(true);
				}
			}
		});
	};

	//called when i6 login failed
	AllegroAuthenticator.prototype.logoutOnLoginError = function(params) {
		//optional params
		let callback = params.callback || $.noop;
		
		var self = this;
		if (self.m_hasBaragent) {
			logoutAgent(self, {
				callback : function(ok) {
					if (ok && inConcert.BaragentPolling) {
						inConcert.BaragentPolling.stop();
					}
					callback(ok);
				}
			});
		}
		else {
			callback(true);
		}
	};

}());