/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
  strict:true, undef:false, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:false, esversion: 9 */

/* global inConcert, Baragent, BaragentController*/


// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.

if (!this.inConcert) {
	this.inConcert = {};
}

(function ()
{
	"use strict";

	var zeropad = inConcert.Utils.zeropad;
	
	//constructor
	function BaragentActions() {
		this.m_baController = new BaragentController();
		this.m_ba = new Baragent();
	}

	//Define functions and objects in the namespace in case they don't already exist
	if (typeof inConcert.BaragentActions !== 'object') {
		inConcert.BaragentActions = new BaragentActions();
	}

	var systemStates = {
		"ONLINE" : true
	};

	BaragentActions.prototype.__call = function(method, args, success, error) {
		args = args || [];
		args.push(success);
		args.push(error);
		method.apply(this, args);
	};

	BaragentActions.prototype.__callAndResponse = function(method, args, callback) {
		this.__call(method, args,
			function() {
				callback(true);
			},
			function() {
				callback(false);
			}
		);
	};

	BaragentActions.prototype.__callWithId = function(interactionId, method, args, success, error) {
		var self = this;
		function callMethod(id) {
			args = args || [];
			args.unshift(id);
			self.__call(method, args, success, error);
		}
		
		if (interactionId) {
			callMethod(interactionId);
		}
		else {
			this.getInteractionId(callMethod);
		}
	};

	BaragentActions.prototype.__callWithIdAndResponse = function(interactionId, method, args, callback) {
		var self = this;
		function callMethod(id) {
			args = args || [];
			args.unshift(id);
			self.__callAndResponse(method, args, callback);
		}
		
		if (interactionId) {
			callMethod(interactionId);
		}
		else {
			this.getInteractionId(callMethod);
		}
	};

	BaragentActions.prototype.changeState = function(newState, callback) {
		if (!inConcert.app.has_allegro_baragent() || systemStates[newState]) {
			callback(true);
			return;
		}
		var allegroState = inConcert.app.getAllegroAPI().getAllegroState(newState);
		this.__callAndResponse(this.m_ba.ChangeState, [allegroState], callback);
	};

	BaragentActions.prototype.getInteractionId = function(callback, interactionId) {
		if (interactionId) {
			callback(interactionId);
			return;
		}
		this.__call(this.m_ba.GetInteractionId, [""],
			function(result) {
				callback(result && result.data && result.data.interactionId);
			},
			function() {
				callback(false);
			}
		);
	};

	BaragentActions.prototype.getPhoneState = function(callback, interactionId) {
		this.__callWithId(interactionId, this.m_ba.GetPhoneState, [],
			function(result) {
				callback(result && result.data && result.data.phoneState);
			},
			function() {
				callback(false);
			}
		);
	};

	//Interaction methods

	BaragentActions.prototype.answer = function(callback, interactionId) {
		this.__callWithIdAndResponse(interactionId, this.m_ba.Answer, [], callback);
	};

	BaragentActions.prototype.hangup = function(callback) {
		this.__callAndResponse(this.m_ba.Hangup, [""], callback);
	};

	BaragentActions.prototype.wrapup = function(callback, interactionId) {
		this.__callWithIdAndResponse(interactionId, this.m_ba.Wrapup, [], callback);
	};

	BaragentActions.prototype.sendDigits = function(digits, callback, interactionId) {
		this.__callWithIdAndResponse(interactionId, this.m_ba.SendDigits, [digits], callback);
	};

	BaragentActions.prototype.redial = function(phone, callback, interactionId) {
		this.__callWithIdAndResponse(interactionId, this.m_ba.Redial, [phone], callback);
	};

	BaragentActions.prototype.hold = function(callback, interactionId) {
		var self = this;
		self.getInteractionId(function(interactionId) {
			//check state is connected
			self.getPhoneState(function(state) {
				if (state === "HOLD") {
					callback(true);
					return;
				}
				//hold call
				self.__callWithIdAndResponse(interactionId, self.m_ba.HoldById, [], function(ok) {
					if (ok) {
						callback(true);
						return;
					}
					//is has an error then wait and check state
					window.setTimeout(function() {
						self.getPhoneState(function(state) {
							callback(state === "HOLD");
						}, interactionId);
					}, 500);
				});
			}, interactionId);
		}, interactionId);
	};

	BaragentActions.prototype.unhold = function(callback, interactionId) {
		var self = this;
		self.getInteractionId(function(interactionId) {
			self.getPhoneState(function(state) {
				//check state is hold
				if (state !== "HOLD") {
					callback(true);
					return;
				}
				//unhold call
				self.__callWithIdAndResponse(interactionId, self.m_ba.HoldById, [], function(ok) {
					if (ok) {
						callback(true);
						return;
					}
					//is has an error then wait and check state
					window.setTimeout(function() {
						self.getPhoneState(function(state) {
							callback(state !== "HOLD");
						}, interactionId);
					}, 500);
				});
			}, interactionId);
		}, interactionId);
	};

	BaragentActions.prototype.mute = function(callback) {
		this.__callAndResponse(this.m_ba.Mute, [], callback);
	};

	BaragentActions.prototype.unmute = function(callback) {
		this.__callAndResponse(this.m_ba.Mute, [], callback);
	};

	BaragentActions.prototype._switch = function(callback) {
		this.__callAndResponse(this.m_ba.Switch, [], callback);
	};

	BaragentActions.prototype.unswitch = function(callback) {
		this.__callAndResponse(this.m_ba.Switch, [], callback);
	};

	//Recording
	BaragentActions.prototype.startRecording = function(callback, interactionId) {
		this.__callWithIdAndResponse(interactionId, this.m_ba.Recording, ["1"], callback);
	};

	BaragentActions.prototype.stopRecording = function(callback, interactionId) {
		this.__callWithIdAndResponse(interactionId, this.m_ba.Recording, ["0"], callback);
	};

	//Preview
	BaragentActions.prototype.acceptPreview = function(callback, interactionId) {
		this.__callWithIdAndResponse(interactionId, this.m_ba.AcceptPreview, [], callback);
	};

	BaragentActions.prototype.cancelPreview = function(callback, interactionId) {
		this.__callWithIdAndResponse(interactionId, this.m_ba.CancelPreview, [], callback);
	};

	//Transfer
	BaragentActions.prototype.__transfer = function(destination, autocommit, value, callback, interactionId) {
		this.__callWithIdAndResponse(interactionId, this.m_ba.Transfer, [destination, value, autocommit? 1 : 0], callback);
	};

	BaragentActions.prototype.requeue = function(campaign, callback, interactionId) {
		var allegroCampaign = inConcert.app.getAllegroAPI().getAllegroCampaign(campaign);
		this.__transfer(allegroCampaign, false, "COLA", callback, interactionId);
	};

	BaragentActions.prototype.transferToCampaign = function(campaign, callback, interactionId) {
		var allegroCampaign = inConcert.app.getAllegroAPI().getAllegroCampaign(campaign);
		this.__callWithIdAndResponse(interactionId, this.m_ba.Forward, [allegroCampaign, ""], callback);
	};

	BaragentActions.prototype.__canTransferToAgent = function(agent, interactionId, appId, callback = $.noop) {
		const interactionManager = inConcert.app.getInteractionManager();
		let info = {
			interaction: interactionId,
			target: {
				user: agent
			},
			appId: appId
		};
		interactionManager.canTransferToMember(info, function(success, error) {
			callback(success, error);
		});
	};

	BaragentActions.prototype.transferToAgent = function(agent, callback, interactionId, appId) {
		const self = this;
		var transfer = function(id) {
			self.__canTransferToAgent(agent, id, appId, function(success, error) {
				if (!success) {
					callback(false, error);
					return;
				}
				self.__transfer(agent, false, "AGENTE", callback, id);
			});
		};
		if (interactionId) {
			transfer(interactionId);
			return;
		}
		this.getInteractionId(function(id) {
			if (!id) {
				callback(false);
				return;
			}
			transfer(id);
		});
	};

	BaragentActions.prototype.transferToPhone = function(phone, callback, interactionId) {
		this.__transfer(phone, false, "INTERNO", callback, interactionId);
	};

	BaragentActions.prototype.transferCommit = function(callback, interactionId) {
		this.__callWithIdAndResponse(interactionId, this.m_ba.TransferCommit, [], callback);
	};

	BaragentActions.prototype.transferCancel = function(callback, interactionId) {
		this.__callWithIdAndResponse(interactionId, this.m_ba.TransferCancel, [], callback);
	};

	//Conference
	BaragentActions.prototype.__conference = function(destination, autocommit, value, callback, interactionId) {
		this.__callWithIdAndResponse(interactionId, this.m_ba.Conference, [destination, value, autocommit? 1 : 0], callback);
	};

	BaragentActions.prototype.conferenceWithAgent = function(agent, callback, interactionId) {
		this.__conference(agent, false, "AGENTE", callback, interactionId);
	};

	BaragentActions.prototype.conferenceWithPhone = function(phone, callback, interactionId) {
		this.__conference(phone, false, "INTERNO", callback, interactionId);
	};

	BaragentActions.prototype.conferenceCommit = function(callback, interactionId) {
		this.__callWithIdAndResponse(interactionId, this.m_ba.ConferenceCommit, [], callback);
	};

	BaragentActions.prototype.conferenceCancel = function(callback, interactionId) {
		this.__callWithIdAndResponse(interactionId, this.m_ba.ConferenceCancel, [], callback);
	};

	//Voice mail
	BaragentActions.prototype.replayVoicemail = function(callback) {
		this.__callAndResponse(this.m_ba.RePlayVoiceMail, [], callback);
	};

	//Phone methods
	BaragentActions.prototype.makeCall = function(campaign, number, contactId, contactName, callback, interactionId) {
		var allegroCampaign = inConcert.app.getAllegroAPI().getAllegroCampaign(campaign);
		var iml = "";
		var sep1 = String.fromCharCode(2);	//STX
		var sep2 = String.fromCharCode(3);	//ETX
		
		iml += (contactId)? sep1 + sep2 + "CONTACTID" + sep2 + contactId : "";
		iml += (contactName)? sep1 + ((iml==="")? sep2 : "") + "CONTACTNAME" + sep2 + contactName : "";
		iml += (interactionId)? sep1 + ((iml==="")? sep2 : "") + "INTERACTIONID" + sep2 + interactionId : "";

		this.__callAndResponse(this.m_ba.MakeCall, [allegroCampaign, number, iml], callback);
	};

	//queue methods

	BaragentActions.prototype.takeFirst = function(callback) {
		this.__callAndResponse(this.m_ba.TakeFirst, [], callback);
	};

	BaragentActions.prototype.take = function(interactionId, callback) {
		this.__callAndResponse(this.m_ba.TakeById, [interactionId], callback);
	};

	function __getScheduleDateTime(date, utc) {
		var result = { date : "", time : "" };
		if (date) {
			if (utc) {
				result.date = date.getUTCFullYear() + "-" + zeropad(date.getUTCMonth() + 1) + "-" + zeropad(date.getUTCDate());
				result.time = zeropad(date.getUTCHours()) + ":" + zeropad(date.getUTCMinutes()) + ":00";
			}
			else {
				result.date = date.getFullYear() + "-" + zeropad(date.getMonth() + 1) + "-" + zeropad(date.getDate());
				result.time = zeropad(date.getHours()) + ":" + zeropad(date.getMinutes()) + ":00";
			}
		}
		return result;
	}

	//schedule methods
	BaragentActions.prototype.rescheduleCurrent = function(phone, scheduleDate, callback) {
		var allegroDate = __getScheduleDateTime(scheduleDate);
		this.__callAndResponse(this.m_ba.ManualReschedule, [phone, allegroDate.date, allegroDate.time, ""], callback);
	};

	BaragentActions.prototype.callbackCurrent = function(phone, callback) {
		this.rescheduleCurrent(phone, null, callback);
	};

	//abandoned methods
	BaragentActions.prototype.scheduleAbandoned = function(interactionId, campaign, phone, contactId, contactName, scheduleDate, callback) {
		var allegroCampaign = inConcert.app.getAllegroAPI().getAllegroCampaign(campaign);
		var allegroUser = inConcert.app.getAllegroAPI().getAllegroLoggedUser();
		var allegroDate = __getScheduleDateTime(scheduleDate, true);
		var reschedule = allegroDate.date !== "";
		this.__callAndResponse(this.m_ba.CallAbandoned, [interactionId, allegroCampaign, phone, allegroUser,
			contactId, contactName, reschedule, allegroDate.date, allegroDate.time], callback);
	};

	BaragentActions.prototype.callAbandoned = function(interactionId, campaign, phone, contactId, contactName, callback) {
		this.scheduleAbandoned(interactionId, campaign, phone, contactId, contactName, null, callback);
	};
	
	BaragentActions.prototype.getAttribute = function(attribute, callback, interactionId) {
		this.__callWithId(interactionId, this.m_ba.GetInteractionAttributeByID, [attribute],
			function(result) {
				callback(result && result.data && result.data.value);
			},
			function() {
				callback(null);
			}
		);
	};

}());