/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
  strict:true, undef:false, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:false, esversion: 11 */

/* global inConcert*/


// Override interaction_manager functions

if (!this.inConcert) {
    this.inConcert = {};
}

(function ()
{
	//constructor
	function InteractionManager() {}

	//Define functions and objects in the namespace in case they don't already exist
	if (typeof inConcert.AllegroInteractionManager !== 'object') {
		inConcert.AllegroInteractionManager = new InteractionManager();
	}

	InteractionManager.prototype.onBatchSuccess = function(interactions, action, callbackFn) {
		var interactionManager = inConcert.app.getInteractionManager();
		interactionManager.onBatchSuccess({ successful_nodes : interactions, unsuccessful_nodes : [] }, 
			action, callbackFn);
	};

	InteractionManager.prototype.onBatchError = function(interactions, action, callbackFn, error) {
		error = error || {result : "ALLEGRO_ERROR"};
		inConcert.app.getInteractionManager().onBatchError(null, action, error, callbackFn, interactions);
	};

	InteractionManager.prototype.batchArchive = function(interactions, dispositionCode, _, onArchived) {
		var self = this;
		var allegro_api = inConcert.app.getAllegroAPI();
		
		allegro_api.setDisposition(interactions[0].id,  dispositionCode, function(ok) {
			if (ok) {
				if(dispositionCode && dispositionCode.needReschedule)
				{		
					allegro_api.dialerCallBack(interactions[0].id,  dispositionCode, function(ok, error) {
						if (ok) {
							inConcert.BaragentActions.wrapup(function(ok) {
								if (ok) {
									self.onBatchSuccess(interactions, "batchArchive", onArchived);
								}
								else {
									self.onBatchError(interactions, "batchArchive", onArchived);
								}
							});
						}
						else {
							self.onBatchError(interactions, "batchArchive", onArchived, { result: error });
						}
					});
				}
				else
				{					
					inConcert.BaragentActions.wrapup(function(ok) {
						if (ok) {
							self.onBatchSuccess(interactions, "batchArchive", onArchived);
						}
						else {
							self.onBatchError(interactions, "batchArchive", onArchived);
						}
					});
				}
			}
			else {
				self.onBatchError(interactions, "batchArchive", onArchived);
			}
		});
	};

	InteractionManager.prototype.batchRequeue = function(interactions, onRequeued) {
		var self = this;
		inConcert.BaragentActions.requeue(interactions[0].campaign, function(ok) {
			if (ok) {
				self.onBatchSuccess(interactions, "batchRequeue", onRequeued);
			}
			else {
				self.onBatchError(interactions, "batchRequeue", onRequeued);
			}
		});
	};

	//transfer the conversations to another ( team member / campaign account / attention level )
	// app_id: optional -- default value = interaction
	InteractionManager.prototype.batchTransfer = function(interactions, target, onTransferred, appId) {
		const self = this;
		function responseFn(ok, error) {
			if (ok) {
				$.each(interactions, function(_, interaction) {
					interaction.pending = false;
					inConcert.app.m_mainWindow.updateInteractions([interaction]);
				});
				self.onBatchSuccess(interactions, "batchTransfer", onTransferred);
				return;
			}
			// error
			self.onBatchError(interactions, "batchTransfer", onTransferred, error);
		}

		switch(target.to) {
			case "campaign":
				return inConcert.BaragentActions.transferToCampaign(target.campaign_id, responseFn);
			case "member":
				let interactionId;
				if (interactions) {
					interactionId = interactions[0].id;
				}
				return inConcert.BaragentActions.transferToAgent(target.user, responseFn, interactionId, appId);
			case "number":
				return inConcert.BaragentActions.transferToPhone(target.number, responseFn);
			default:
				responseFn(false);
		}
	};

	//transfer the conversations to another ( team member / campaign account / attention level )
	// app_id: optional -- default value = interaction
	InteractionManager.prototype.batchConference = function( interactions, target, onConference ) {
		var self = this;
		function response(ok) {
			if (ok) {
				$.each(interactions, function(_, interaction) {
						interaction.pending = false;
						inConcert.app.m_mainWindow.updateInteractions([interaction]);
				});
				self.onBatchSuccess(interactions, "batchConference", onConference);
			}
			else {
				self.onBatchError(interactions, "batchConference", onConference);
			}
		}
		switch(target.to) {
			case "number":
				return inConcert.BaragentActions.conferenceWithPhone(target.number, response);
			default:
				response(false);
		}
	};

	//dialer are inbound interactions
	InteractionManager.prototype.createCallInteraction = function(id, campaign, account, initiative, contactId, contactName, voicemail, invoked_from) {
		var interactionManager = inConcert.app.getInteractionManager();
		var rawInteraction = {
			id                  : id                               ,
			type                : voicemail? "VOICEMAIL" : "CALL"  ,
			account             : account                          ,
			contact             : contactId                        ,
			interactionData     : { contactName: contactName }     ,
			initiative          : initiative                       ,
			assignedAgents      : [ inConcert.app.getCompleteId() ],
			contactAddress      : ""                               ,
			completeAddress     : null                             ,
			state               : "PENDING"                        ,
			campaign            : campaign
		};
		console.info(`[AllegroInteractionManager.prototype.createCallInteraction] -> Creating Pending Allegro Interaction (invoked_from: ${invoked_from}). rawInteraction: ${JSON.stringify(rawInteraction)}`);
		var interaction = interactionManager.create(rawInteraction, invoked_from);
		if (!interaction) {
			return false;
		}
		interaction.setContactObject({ Id : contactId, Name : contactName });
		return interaction;
	};

	InteractionManager.prototype.canTakeAllegroInteractions = function() {
		return !inConcert.BaragentPolling.hasInteraction();
	};
})();