/*jshint browser:true, quotmark:false, smarttabs:true, esversion: 9 */
/*global $, window, inConcert */

// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	"use strict";

	/**
	Class AppLaunched.
	*/
	function AppLaunched (options)
	{
		options = options || {};
		AppLaunched.uber.constructor.call(this,
			{
				move: ["topDragBar", "bottomDragBar"],
				resizer: "resizeGripper"
			},
		);
		document.title = "<%=_T('Application Launcher')%>";
		this.addClass("launchedAPP");
		this.m_app_uri = null;
		this.render_status = null;
	}

	inConcert.Utils.inherit(AppLaunched, inConcert.Dialog);

	/*******************************************************************
	 * Metodo que setea los eventos de los controles!!
	 */

	AppLaunched.prototype.set_app_launch = function(app_uri) {
		this.m_app_uri = app_uri;
		this.__render(false);
	};

	AppLaunched.prototype.__render = function(must_reload)
	{
		const self = this;

		let log_preffix = "-----> [APP_LAUNCHED_POPUP:__render] ";
		console.debug(log_preffix + "ENTER - must_reload: " + must_reload);

		if (this.render_status === "RENDERING") {
			console.debug(log_preffix + "return already rendering.");
			return;
		}
		const is_enabled_apps_launcher = true; // TODO: Validar contra algo ?
		if (!is_enabled_apps_launcher) {
			console.debug(log_preffix + "disabled.");
			return;
		}

		const $body = $("body");
		const $waiting_container = $body.find("[waiting_container]");
		const $view_container = $body.find("[view_container]");
		const $error_container = $body.find("[error_container]");
		const $error_msg = $error_container.find("[error_msg]");

		var show_waiting = function() {
			console.debug(log_preffix + "-> show_waiting");
			self.render_status = "RENDERING";
			$error_container.hide();
			$view_container.hide();
			$waiting_container.show();
		};
		var show_error = function(err_info) {
			console.debug(log_preffix + "-> show_error");
			self.__on_finish_render("ERROR", function () {
				$waiting_container.hide();
				$view_container.hide();
				const err_msg = self.__get_error_msg(err_info);
				$error_msg.text(err_msg);
				$error_container.show();
			});
		};
		var show = function() {
			console.debug(log_preffix + "-> show");
			self.__on_finish_render("SUCCESS", function () {
				$waiting_container.hide();
				$error_container.hide();
				$view_container.show();
			});
		};

		show_waiting();

		const $retry = $error_container.find("[load_retry]");
		$retry.off("click")
			.on("click", function() {
				self.__render(false);
			});

		const $iframe = $view_container.find("[view_frm]");
		var set_uri_to_iframe = function(uri) {
			console.debug(log_preffix + "-> set_uri_to_iframe");
			$iframe.attr("src", uri);
			$iframe.on("load", function() {
				show();
			});
		};

		if (must_reload) {
			console.debug(log_preffix + "clean iframe.src. setTimeout 1000");
			const current_uri = $iframe.attr("src");
			$iframe.attr("src", "");
			window.setTimeout(function() {
				set_uri_to_iframe(current_uri);
			}, 1000);
		}
		else {
			console.debug(log_preffix + "retrieving uri");
			set_uri_to_iframe(this.m_app_uri);
		}
	};

	AppLaunched.prototype.__on_finish_render = function(render_status, callback = $.noop)
	{
		this.render_status = render_status;
		callback();
	};

	$(document).ready(function() {
		window.hook_environment();
		inConcert.i18n.updateScreen();
		var settings = new AppLaunched();
		window.onReady(settings);
	});
}());
