/*jshint bitwise:true, browser:true, curly:true, eqeqeq:true, noarg:true, noempty:true, strict:true, undef:true, unused:true, esversion: 9 */
/*jshint quotmark:false */
/*global $, inConcert*/

this.inConcert = this.inConcert || {};
this.inConcert.apps_launcher = this.inConcert.apps_launcher || {};

(function () {
	"use strict";

	const FORMAT = inConcert.Utils.format;
	const APP_PERMISSION_ID = "apps_launcher";

	var can_show = function(app_id) {
		if (!app_id) {
			return false;
		}
		const has_permission = inConcert.permissions.to_application(app_id);
		if (!has_permission) {
			return false;
		}
		return true;
	};

	function AppsLauncherBaseGUI (params) {
		this.container_id = params.container_id;
		this.render_status = null;
		this.server_err_info = null;
		this.$dashboard = null;
		this.m_appLaunchedWindows = {};
		this.m_tweety_runtime_uri = inConcert.app.getTweetyRuntimeUrl();
		if (this.can_show()) {
			this.__create_markup();
		}
	}

	AppsLauncherBaseGUI.prototype.get_container_id = function() {
		return this.container_id;
	};

	AppsLauncherBaseGUI.prototype.__create_markup = function() {
		if (this.$dashboard) {
			return;
		}
		const markup = "".concat(
			"<div waiting_container='true' hidden>",
				"<div class='appsLauncherContentListWaiting'>",
					"<div class='appsLauncherListWaitingMessage'>", "<%=_T('Loading, wait moment please')%>", "</div>",
					"<div class='appsLauncherListWaitingAnimation'></div>",
					"<div class='appsLauncherListWaitingOverlay'></div>",
				"</div>",
			"</div>",
			"<div class='appsLauncherPageHeader'>",
				"<div class='pageTitleContainer'>",
					"<i class='far fa-rocket'></i>",
					"<label class='pageTitle'><%=_T('Applications gallery')%></label>",
				"</div>",
			"</div>",
			"<div error_container='true' hidden>",
				"<div class='appsLauncherContentErrorContainer'>",
					"<div class='appsLauncherListErrorMessage'>",
						"<label error_msg='true'></label>",
					"</div>",
					"<div class='appsLauncherListErrorIcon'></div>",
					"<div class='appsLauncherListErrorOverlay'></div>",
				"</div>",
			"</div>",
			"<div class='skinOCCNew' view_container='true' hidden>",
				"<div view_apps_launcher_not_found='true' class='appsLauncherMessagesContainer' hidden>",
					"<div class='inlineMessagesContainer messageInfo'>", "<%=_T('Applications Launchers not found.')%>", "</div>",
				"</div>",
				"<div view_apps_launcher='true' class='appsLauncherMainContainer' hidden></div>",
			"</div>"
		);
		const $markup = $(markup);
		this.$dashboard = $("#" + this.container_id).html($markup);
		this.__render();
	};

	AppsLauncherBaseGUI.prototype.__on_finish_render = function(render_status, callback = $.noop)
	{
		this.render_status = render_status;
		callback();
	};

	AppsLauncherBaseGUI.prototype.__create_apps_markup = function(apps_actions)
	{
		const self = this;

		if (apps_actions === null) {
			return;
		}
		const $view_container = this.$dashboard.find("[view_container]");
		const $apps_launcher = $view_container.find("[view_apps_launcher]");
		$apps_launcher.html("").hide();

		let html_template_panel =  '' +
			'<div class="appLauncherWebdesignerItem">' +
				'<a class="appLauncherWebdesignerLink" execute="true" href="javascript:void(0)">' +
					'<div class="appLauncherWebdesignerInfoContainer">' +
						'<div>' +
							'<label class="appLauncherWebdesignerTitleInput">{0}</label>' +
						'</div>' +
					'</div>' +
				'</a>' +
			'</div>';

		for (let i = 0; i < apps_actions.length; i++) {
			let application = apps_actions[i];
			let html_panel = FORMAT(html_template_panel, application.Description);
			let $panel = $(html_panel);
			$panel.find("[execute]").data("webdesigner_app", application);
			$apps_launcher.append($panel);
		}

		$apps_launcher.find("[execute]")
			.off("click")
			.on("click", function() {
				const $this = $(this);
				const application = $this.data("webdesigner_app");
				self.__open(application);
			});
	};

	AppsLauncherBaseGUI.prototype.__open = function(application)
	{
		const self = this;

		if (!application) {
			//TODO: Visualizar error
			return;
		}
		const data = application.Data;
		if (!data || !data.webdesigner || !data.webdesigner.projectId) {
			//TODO: Visualizar error
			return;
		}
		let uri = this.m_tweety_runtime_uri + "/" + data.webdesigner.projectId;

		const has_permission = inConcert.permissions.to_action("apps_launcher", application.ActionId);
		if (!has_permission) {
			inConcert.app.getMainWindow().showBannerMessage({
				message: "<%=_T('Unable to Launch Application. You dont have permission.')%>",
				timeout: 5000
			});
			return;
		}

		const window_id = "appLauncher_" + application.ActionId;
		if (this.m_appLaunchedWindows && this.m_appLaunchedWindows[window_id]) {
			const client_mng = this.m_appLaunchedWindows[window_id].client_mng;
			if (client_mng) {
				// Lo meto en un timeout el activate, ya que sino el evento "clickoutside" de la clase "DialogManager" "window_dialog.js" lo minimiza.
				window.setTimeout(function() {
					client_mng.activate();
				}, 150);
				return;
			}
		}

		const length = 100;
		const dots = "..";
		let launchedWindowMng;
		let create_maximized = false;
		let size = {width: 800, height: 600};

		if (application.Data.webdesigner.windowMode) {
			create_maximized = application.Data.webdesigner.windowMode.mode === "MAXIMIZED";
			if (!create_maximized && application.Data.webdesigner.windowMode.size) {
				size.width = application.Data.webdesigner.windowMode.size.width;
				size.height = application.Data.webdesigner.windowMode.size.height;
			}
		}

		const windowOptions = {
			url        : "/inconcert/apps/apps_launcher/pages/appLaunched.<%=_L%>.html",
			size       : size,
			create_maximized: create_maximized,
			windowID   : window_id,
			title      : inConcert.Utils.truncateWord(application.Description, length, dots),
			resizable  : true,
			maximizable: true,
			taskbar    : this,
			can_minimize_when_outside: true,
			on_ready: function(client_mng) {
				self.m_appLaunchedWindows[window_id].client_mng = client_mng;
				client_mng.set_app_launch(uri);
			},
			on_close: function(current_window_id) {
				if (self.m_appLaunchedWindows[current_window_id]) {
					delete self.m_appLaunchedWindows[current_window_id];
				}
			}
		};

		launchedWindowMng = inConcert.app.getWindowManager().createWindow(windowOptions);
		this.m_appLaunchedWindows[window_id] = {
			window_mng: launchedWindowMng,
			client_mng: null
		};
	};

	AppsLauncherBaseGUI.prototype.__render = function()
	{
		const self = this;

		let log_preffix = "-----> [AppsLauncherBaseGUI:__render] ";
		console.debug(log_preffix + "ENTER");

		if (this.render_status === "RENDERING") {
			console.debug(log_preffix + "return already rendering.");
			return;
		}
		const can_show_apps_launcher = this.can_show();
		if (!can_show_apps_launcher) {
			console.debug(log_preffix + "no can show");
			return;
		}

		const $waiting_container = this.$dashboard.find("[waiting_container]");
		const $view_container = this.$dashboard.find("[view_container]");
		const $error_container = this.$dashboard.find("[error_container]");
		const $error_msg = $error_container.find("[error_msg]");
		const $apps_launcher = $view_container.find("[view_apps_launcher]");
		const $apps_launcher_not_found = $view_container.find("[view_apps_launcher_not_found]");

		var show_waiting = function() {
			console.debug(log_preffix + "-> show_waiting");
			self.render_status = "RENDERING";
			$error_container.hide();
			$view_container.hide();
			$waiting_container.show();
		};
		var show_error = function(err_info) {
			console.debug(log_preffix + "-> show_error");
			self.__on_finish_render("ERROR", function () {
				$waiting_container.hide();
				$view_container.hide();
				const err_msg = self.__get_error_msg(err_info);
				$error_msg.text(err_msg);
				$error_container.show();
			});
		};
		var show_apps = function(success, app_actions_info) {
			console.debug(log_preffix + "-> show_apps");
			self.__on_finish_render("SUCCESS", function ()
			{
				let app_actions = [];
				for(let id in app_actions_info.Actions || {}) {
					if (app_actions_info.Actions.hasOwnProperty(id)) {
						let action = app_actions_info.Actions[id];
						const has_permission = inConcert.permissions.to_action("apps_launcher", action.ActionId);
						if (has_permission) {
							app_actions.push(action);
						}
					}
				}

				if (app_actions.length === 0) {
					$apps_launcher.html("").hide();
					$apps_launcher_not_found.show();
				}
				else {
					app_actions = inConcert.Utils.sortArray(app_actions, "Description");
					self.__create_apps_markup(app_actions);
					$apps_launcher.show();
					$apps_launcher_not_found.hide();
				}
				$waiting_container.hide();
				$error_container.hide();
				$view_container.show();
			});
		};

		show_waiting();

		inConcert.permissions.get_one_app_actions(
			"apps_launcher",
			function(success, app_actions_info) {
				if (!success) {
					show_error();
					return;
				}
				show_apps(success, app_actions_info);
			}
		);
	};

	AppsLauncherBaseGUI.prototype.__get_error_msg = function(/*err_info*/)
	{
		const default_msg = "<%=_T('There was an error, please try again later.')%>";
		let msg_error = default_msg;
		return msg_error;
	};

	AppsLauncherBaseGUI.prototype.can_show = function() {
		return can_show(this.permissions.app_id);
	};

	AppsLauncherBaseGUI.prototype.show = function() {
		if (!this.$dashboard) {
			return;
		}
		return this.$dashboard.show();
	};

	AppsLauncherBaseGUI.prototype.hide = function() {
		if (!this.$dashboard) {
			return;
		}
		return this.$dashboard.hide();
	};

	AppsLauncherBaseGUI.prototype.render = function() {
		this.show();
		this.__reload();
	};

	AppsLauncherBaseGUI.prototype.__must_reload = function() {
		if (this.render_status === "RENDERING" || this.render_status === "ERROR") {
			return false;
		}
		if (this.server_err_info && this.server_err_info.code === 403) {
			return false;
		}
		return true;
	};

	AppsLauncherBaseGUI.prototype.__reload = function() {
		if (!this.__must_reload()) {
			return;
		}
		this.__render(true);
	};

	AppsLauncherBaseGUI.prototype.on_leave = function() {
		this.hide();
	};

	//-----------------------------------------------------------------------------
	//---------------------------------------------------------------------
	// Launcher
	//
	function LauncherDashboard (params)
	{
		this.permissions = {
			app_id: APP_PERMISSION_ID
		};
		LauncherDashboard.uber.constructor.call(this, params);
	}
	inConcert.Utils.inherit(LauncherDashboard, AppsLauncherBaseGUI);
	if (typeof inConcert.apps_launcher.dashboard !== "object") {
		inConcert.apps_launcher.dashboard = LauncherDashboard;
	}
}());
