/*  jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
  strict:true, undef:true, unused:true, curly:true, browser:true, indent:false, maxerr:50, esversion: 11 */

/*global $, inConcert*/

if (!this.inConcert) {
	this.inConcert = {};
}
if (!inConcert.batchManagement) {
	inConcert.batchManagement = {};
}
if (!inConcert.batchManagementActions) {
	inConcert.batchManagementActions = {};
}

//comment
(function () {
/*jshint validthis:true*/
"use strict";

	var actions = {
		PAUSE	: { 
			name			: "<%=_T('Pause')%>",
			description		: "<%=_T('Pause batch attention. No new interactions will be created but open interactions could be attended.')%>",
			permission		: "batches_pause",
			method			: "pause_batches/",
			order			: 1
		},
		RESUME	: { 
			name			: "<%=_T('Resume')%>",
			description		: "<%=_T('Resume batch attention.')%>",
			permission		: "batches_resume",
			method			: "resume_batches/",
			order			: 1
		},
		CLOSE	: { 
			name			: "<%=_T('Close')%>",
			description		: "<%=_T('Close batch attention. New messages will be omitted.')%>",
			permission		: "batches_close",
			method			: "close_batches/",
			order			: 2
		},
		/*REOPEN	: { 
			name			: "<%=_T('Reopen')%>",
			description		: "<%=_T('Reopen batch attention.')%>",
			permission		: "batches_reopen",
			method			: "reopen_batches/",
			order			: 1
		},*/
		EDIT	: { 
			name			: "<%=_T('Edit')%>",
			description		: "<%=_T('Edit batch text.')%>",
			permission		: "batches_edit",
			method			: "edit_batches/",
			jsMethod		: "__editPopup",
			order			: 3
		},
		RESCHEDULE	: { 
			name			: "<%=_T('Reschedule')%>",
			description		: "<%=_T('Reschedule batch date.')%>",
			permission		: "batches_edit",
			method			: "edit_batches/",
			jsMethod		: "__reschedulePopup",
			order			: 4
		},
		SET_ATTRIBUTES : { 
			name			: "<%=_T('Set Attributes')%>",
			description		: "<%=_T('Attributes can be used to apply interaction rules.')%>",
			permission		: "batches_edit",
			method			: "edit_batches_properties/",
			jsMethod		: "__setAttributesPopup",
			batchPopup		: "ShowSetAttributesPopup",
			order			: 5
		},
		DELETE	: {
			name			: "<%=_T('Delete')%>",
			description		: "<%=_T('Delete batch.')%>",
			permission		: "batches_delete",
			method			: "delete_batches/",
			order			: 6,
			confirmation	: {
				title		: "<%=_T('Confirm')%>",
				text		: "<%=_T('Do you want to delete batch?')%>"
			}
		},
		PIN	: { 
			name			: "<%=_T('Pin')%>",
			description		: "<%=_T('Pin batch. Pinned batches are not closed automatically.')%>",
			permission		: "batches_edit",
			method			: "pin_batches/",
			order			: 7
		},
		UNPIN	: { 
			name			: "<%=_T('Unpin')%>",
			description		: "<%=_T('Unpin batch.')%>",
			permission		: "batches_edit",
			method			: "unpin_batches/",
			order			: 7
		},
		ADD_ADDRESSES : { 
			name			: "<%=_T('Add addresses')%>",
			description		: "<%=_T('Add addresses to batch')%>",
			permission		: "batches_edit",
			jsMethod		: "__addAdressesPopup",
			method			: "add_addresses_to_batch/",
			order			: 7
		},
	};
	if (!inConcert.batchManagementActions.base) {
		inConcert.batchManagementActions.base = actions;
		inConcert.batchManagementActions.all = $.extend({}, actions);
	}

	// Defines a generic object with its common behaviour()
	// constructor
	function Batch (rawData)
	{
		$.extend(true, this, rawData);
	}
	
	if (typeof inConcert.batchManagement.base !== "function") {
		inConcert.batchManagement.base = Batch;
	}
	
	Batch.prototype.GetCharactersLimit = function() {
		return null;
	};
	
	Batch.prototype.GetAccountName = function() {
		return this.account;
	};
	
	Batch.prototype.HasAddresses = function() {
		return false;
	};
	
	Batch.prototype.GetAddressesStatus = function() {
		return this.addressesStatus || {};
	};
	
	Batch.prototype.GetIdString = function() {
		return this.id;
	};
	
	Batch.prototype.GetText = function() {
		return inConcert.app.getBatchesManager().GetBatchText(this.type, this.data);
	};
	
	Batch.prototype.GetContent = function() {
		return inConcert.app.getBatchesManager().GetBatchContent(this.type, this.data);
	};
	
	Batch.prototype.__getProperty = function(key) {
		return this.properties && this.properties[key] || null;
	};
	
	Batch.prototype.GetAttributes = function() {
		return this.__getProperty("attributes") || "";
	};
	
	Batch.prototype.GetId = function() {
		return this.account + ":" + this.id;
	};
	
	Batch.prototype.IsActive = function() {
		return this.state === "ACTIVE";
	};
	
	Batch.prototype.IsPaused = function() {
		return this.state === "PAUSED";
	};
	
	Batch.prototype.IsProcessing = function() {
		return this.state === "PROCESSING";
	};
	
	Batch.prototype.IsOpen = function() {
		return this.IsActive() || this.IsPaused() || this.IsProcessing();
	};
	
	Batch.prototype.IsClosed = function() {
		return this.state === "CLOSED";
	};
	
	Batch.prototype.IsDeleted = function() {
		return this.state === "DELETED";
	};
	
	Batch.prototype.Update = function(data) {
		$.extend( this, data );
	};
	
	Batch.prototype.UpdateFromServer = function(callback) {
		inConcert.app.getBatchesManager().UpdateBatches([this], callback);
	};
	
	Batch.prototype.__getActions = function() {
		return this.type && inConcert.batchManagementActions[this.type.toLowerCase()] || {};
	};
	
	Batch.prototype.__getAction = function(actionName) {
		return this.__getActions()[actionName];
	};
	
	Batch.prototype.CanRunAction = function(actionName) {
		var action = this.__getAction(actionName);
		return action && 
			inConcert.permissions.to_action("batch_management", action.permission, null, this.campaign) &&
			this["__can_" + actionName.toLowerCase()]() || false;
	};
	
	Batch.prototype.__editPopup = function(callback) {
		callback = callback || $.noop;
		var batchesManager = inConcert.app.getBatchesManager();
		var action = this.__getAction("EDIT");
		if (!this.CanRunAction("EDIT")) {
			return callback(false, batchesManager.GetBatchActionErrorMessage(403, action));
		}
		batchesManager.ShowEditPopup(this);
		callback(true);	
	};

	Batch.prototype.__addAdressesPopup = function(callback) {
		callback = callback || $.noop;
		var batchesManager = inConcert.app.getBatchesManager();
		var action = this.__getAction("ADD_ADDRESSES");
		if (!this.CanRunAction("ADD_ADDRESSES")) {
			return callback(false, batchesManager.GetBatchActionErrorMessage(403, action));
		}
		batchesManager.ShowAddAddressesPopup(this);
		callback(true);	
	};
	
	Batch.prototype.__reschedulePopup = function(callback) {
		callback = callback || $.noop;
		var batchesManager = inConcert.app.getBatchesManager();
		var action = this.__getAction("EDIT");
		if (!this.CanRunAction("EDIT")) {
			return callback(false, batchesManager.GetBatchActionErrorMessage(403, action));
		}
		batchesManager.ShowReschedulePopup(this);
		callback(true);	
	};
	
	Batch.prototype.__setAttributesPopup = function(callback) {
		callback = callback || $.noop;
		var batchesManager = inConcert.app.getBatchesManager();
		var action = this.__getAction("SET_ATTRIBUTES");
		if (!this.CanRunAction("SET_ATTRIBUTES")) {
			return callback(false, batchesManager.GetBatchActionErrorMessage(403, action));
		}
		batchesManager.ShowSetAttributesPopup(this);
		callback(true);	
	};
	
	Batch.prototype.RunAction = function(actionName, callback, data, run) {
		callback = callback || $.noop;
		var self = this;
		var action = self.__getAction(actionName);
		if (action && !run && action.jsMethod && self[action.jsMethod]) {
			return self[action.jsMethod](callback);
		}
		var batchesManager = inConcert.app.getBatchesManager();
		batchesManager.RunBatchAction([this], actionName, function(status, result) {
			if (status) {
				if (result === "ABORTED" || result.successful_nodes.length) {
					callback(true);
				}
				else {
					callback(false, batchesManager.GetBatchActionErrorMessage(result.unsuccessful_nodes[0], action));
				}
			}
			else {
				callback(false, result);
			}
		}, data);
	};
	
	Batch.prototype.HasActions = function() {
		var self = this;
		var has = false;
		$.each(self.__getActions(), function(actionName, action) {
			if (action && self.CanRunAction(actionName)) {
				has = true;
				return false;
			}
		});
		return has;
	};
	
	Batch.prototype.GetActions = function() {
		var self = this;
		var actions = [];
		$.each(self.__getActions(), function(actionName, action) {
			if (action && self.CanRunAction(actionName)) {
				action.id = actionName;
				actions.push(action);
			}
		});
		actions.sort(function(a, b) {
			if (a.order === b.order) {
				return a.name.localeCompare(b.name);
			}
			return a.order - b.order;
		});
		return actions;
	};
	
	Batch.prototype.GetNativeProperties = function() {
		let properties = [];
		if (this.IsDeleted()) {
			properties.push({ className : "deleted", tooltip : "<%=_T('Deleted')%><%=_T('.')%>" });
		}
		else if (this.IsClosed()) {
			properties.push({ className : "closed" });
		}
		else {
			if (this.IsPaused()) {
				properties.push({ className : "paused", tooltip : "<%=_T('Paused')%><%=_T('.')%>" });
			}
			if (this.IsPinned()) {
				properties.push({ className : "pinned", tooltip : "<%=_T('Pinned')%><%=_T('.')%>" });
			}
		}
		return properties;
	};
	
	Batch.prototype.__can_edit = function() {
		return this.IsOpen();
	};

	Batch.prototype.__can_add_addresses = function() {
		return this.IsActive() || this.IsProcessing();
	};
	
	Batch.prototype.__can_reschedule = function() {
		return this.IsOpen() && this.IsScheduled();
	};
	
	Batch.prototype.__can_pause = function() {
		return this.IsActive();
	};
	
	Batch.prototype.__can_resume = function() {
		return this.IsPaused();
	};
	
	Batch.prototype.__can_close = function() {
		return this.IsOpen();
	};
	
	Batch.prototype.__can_reopen = function() {
		return this.IsClosed();
	};
	
	Batch.prototype.__can_delete = function() {
		return !this.IsDeleted();
	};
	
	Batch.prototype.__can_set_attributes = function() {
		return this.IsOpen();
	};
	
	Batch.prototype.IsScheduled = function() {
		return this.GetScheduledDate()? 
			(new Date(this.publish_timestamp * 1000) > new Date()) : false;
	};
	
	Batch.prototype.GetScheduledDate = function() {
		return null;
	};
	
	Batch.prototype.DownloadAddressesStatus = function() {
		inConcert.app.getBatchesManager().DownloadBatchAddressesStatus(this);
	};
	
	Batch.prototype.IsPinned = function() {
		return this.properties && this.properties.pinned && true || false;
	};
	
	Batch.prototype.__can_pin = function() {
		return this.IsOpen() && !this.IsPinned();
	};
	
	Batch.prototype.__can_unpin = function() {
		return this.IsOpen() && this.IsPinned();
	};

}());