/*  jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
  strict:true, undef:true, unused:true, curly:true, browser:true, indent:false, maxerr:50, esversion: 11 */

/*global $, inConcert, Mustache*/

if (!this.inConcert) {
	this.inConcert = {};
}

if (!inConcert.batchManagementAddresses) {
	inConcert.batchManagementAddresses = {};
}

(function () {
/*jshint validthis:true*/
"use strict";

	var templates = inConcert.templates.batch_management;
	var format = inConcert.Utils.format;
	function AddressesPopup (batch)
	{
		this.batch = batch;
	}
	
	if (typeof inConcert.batchManagementAddresses.base !== "function") {
		inConcert.batchManagementAddresses.base = AddressesPopup;
	}
	
	AddressesPopup.prototype.__getClassName = function() {
		return "";
	};
	
	AddressesPopup.prototype.Show = function(callback) {
		callback = callback || $.noop;
		const self = this;
		self.Close(function() {
			self.__show(callback);
		});
	};
	
	AddressesPopup.prototype.Close = function(callback) {
		callback = callback || $.noop;
		const self = this;
		if (self.popup && self.popup.isVisible()) {
			self.popup.finalize(function() {
				self.popup = null;
				$("#outgoingBatchesAddressesPopup").remove();
				callback();
			});
		}
		else {
			$("#outgoingBatchesAddressesPopup").remove();
			callback();
		}
	};
	
	AddressesPopup.prototype.__show = function(callback) {
		callback = callback || $.noop;
		const self = this;
		const $menu = $(Mustache.render(templates.add_addresses_popup_container, {
			className : self.__getClassName()
		}));
		$menu.append(self.__make());
		$("body").append($menu);
		self.popup = new inConcert.PopupWindow();
		self.popup.show({
			ownerId       : "outgoingBatchesAddressesPopup",
			eventName     : "outgoingBatchesAddressesPopup",
			canReposition : true,
			closeOnFocusLost : false,
			overlay		  : true,
			container     : $menu,
			callbacks: {
				onOpen: function() {
					$("#outgoingBatchesAddressesPopup").show();
				},
				onClose: function( callback ) {
					callback = callback || $.noop;
					if (self.popup) {
						self.popup.hide(function() {
							callback();
						});
					}
					else {
						callback();
					}
				}
			}
		});
		callback();
	};

	AddressesPopup.prototype.__getTitle = function() {
		return "<%=_T('Add addresses to batch')%>";
	};

	AddressesPopup.prototype.__makeAddressesUploader = function($uploader) {
		const self = this;
		$uploader.parent().click(function() {
			if (self.processing) {
				return;
			}
			$uploader[0].click();
		});
		$uploader.change(function( event ) {
			if (self.processing) {
				return;
			}
			const file = event.target.files[0];
			if ( !file ) {
				return;
			}
			$("#batch_create_addresses_filename").text(file.name);
			let reader = new FileReader();
			reader.onload = function (e) {
				self.__uploadedAddresses = e.target.result;
            };
			reader.readAsText(file);
		});
	};

	AddressesPopup.prototype.__makeAddressesAgentData = function($container) {
		$container.find("#batch_create_address_agent_data_add").click(function() {
			const $item = $(templates.address_agent_data);
			$item.find("[agent_data_delete]").click(function() {
				$(this)
					.parent().parent().parent()
					.remove();
				const $list = $("#batch_create_address_agent_data_list");
				$list.toggle($list.children().length > 1);
			});
			$("#batch_create_address_agent_data_list")
				.append($item)
				.show();
		});
	};

	AddressesPopup.prototype.__makeAddresses = function($body) {
		const $content = $body.find("#batchesAddressesPopupBodyAdd");
		$content.append(templates.create_popup_addresses);
		$content.find("[address_source_selector]")
			.click(function() {
				const source = $(this).attr("address_source_selector");
				const $parent = $(this).closest("[batch_address_content]");
				$parent
					.find("[address_source_data]")
					.hide()
					.filter(format("[address_source_data='{0}']", source))
					.show();
			})
			.first()
			.click();
		$content.find("#batch_create_include_headers").change(function() {
			$("#batch_create_address_column_container, #batch_create_reporting_group_container, #batch_create_address_agent_data_container")
				.toggle($(this).is(":checked"));
		});
		this.__makeAddressesUploader($content.find("#batch_create_addresses_file"));
		this.__makeAddressesAgentData($content.find("#batch_create_address_agent_data_container"));
	};
	
	AddressesPopup.prototype.__make = function() {
		var self = this;
		var $body = $(Mustache.render(templates.add_addresses_popup, {
			title : self.__getTitle(),
			type: self.batch.type.toLowerCase(),
			batchId : self.batch.id
		}));

		$body.find("#batchesAddressesPopupButtonClose")
		.off("click")
		.on("click", function() {
			self.Close();
		});
		//cancel
		$body.find("[cancel]").click(function() {
			self.Close();
		});
		//add addresses
		$body.find("[create]").click(function() {
			self.Create();
		});
		
		self.__makeAddresses($body);
		return $body;
	};

	
	AddressesPopup.prototype.Message = function(text, type) {
		const $message = $("#outgoingBatchesAddressesPopup").find("[message_container]");
		const $text = $message.find("[message_text]");
		$text
			.attr("class", (type || "info") + "Message")
			.html(text);
		$message.toggle(!!text);
	};
	
	AddressesPopup.prototype.Loading = function(loading) {
		$("#outgoingBatchesAddressesPopup").find("[loading]").toggle(!!loading);
	};

	AddressesPopup.prototype.__getAndValidateAddress = function(address_t) {
		return address_t;
	};
	
	AddressesPopup.prototype.__getSeparator = function($separator) {
		var separator = $separator.val();
		switch(separator) {
			case "TAB":
				separator = "\t";
				break;
			case "LINE_BREAK":
				separator = "\n";
				break;
		}
		return separator;
	};
	
	AddressesPopup.prototype.__getAddressHeaders = function(address_s, columnsSeparator, addressColumn, nameColumn) {
		var headers = address_s.split(columnsSeparator);
		var addressFound = false;
		var nameFound = false;
		$.each(headers, function(_, header) {
			if (addressColumn === header) {
				addressFound = true;
			}
			if (nameColumn && nameColumn === header) {
				nameFound = true;
			}
		});
		if (!addressFound) {
			return { __error : "<%=_T('Address column not found in headers.')%>" };
		}
		if (nameColumn && !nameFound) {
			return { __error : "<%=_T('Name column not found in headers.')%>" };
		}
		return headers;
	};
	
	AddressesPopup.prototype.__getAddress = function(address_s, headers, columnsSeparator, addressColumn, nameColumn, reportingGroup, agentData) {
		if (!headers) {
			return { address : address_s };
		}
		var address_a = address_s.split(columnsSeparator);
		if (address_a.length !== headers.length) {
			return { __error : format("<%=_T('Address {0} has different columns number than headers.')%>", address_s) };
		}
		var address = {
			data : {
				data : {},
				reportingGroup : reportingGroup,
				agentData : agentData? {} : null
			}
		};
		$.each(address_a, function(index, value) {
			var name = headers[index];
			if (!name) {
				return;
			}
			if (name === addressColumn) {
				address.address = value;
			}
			else if (nameColumn && name === nameColumn) {
				address.data.name = value;
			}
			address.data.data[name] = value;
			if (agentData && agentData[name]) {
				address.data.agentData[agentData[name]] = value;
			}
		});
		if (!address.address) {
			return { __error : format("<%=_T('Address not found in {0}.')%>", address_s) };
		}
		return address;
	};
	
	AddressesPopup.prototype.__addAddresses = function(data) {
		const self = this;
		const rowsSeparator = self.__getSeparator($("#batch_create_rows_separator"));
		const columnsSeparator = self.__getSeparator($("#batch_create_columns_separator"));
		const includeHeaders = $("#batch_create_include_headers").is(":checked");
		const addressColumn = includeHeaders? $("#batch_create_address_column").val() : null;
		const nameColumn = includeHeaders? $("#batch_create_name_column").val() : null;
		const reportingGroup = includeHeaders? $("#batch_create_reporting_group").val() : null;
		let agentData = null;
		let error = null;

		if (includeHeaders) {
			if (!addressColumn) {
				return "<%=_T('Must enter an address column name.')%>";
			}
			agentData = {};
			$("#batch_create_address_agent_data_list")
				.children("[agent_data_item]")
				.each(function() {
					const column = $(this).find("[agent_data_column]").val();
					if (!column || column.trim() === "") {
						return;
					}
					const name = $(this).find("[agent_data_name]").val();
					if (!name || name.trim() === "") {
						error = "<%=_T('Must enter an agent data item name.')%>";
						return false;
					}
					agentData[column] = name;
				});
			if (error) {
				return error;
			}
		}
		
		let text;
		if ($("#batch_create_addresses_file_selector").prop("checked")) {
			text = self.__uploadedAddresses;
			if (!text) {
				return "<%=_T('Must upload an addresses file.')%>";
			}
		}
		else {
			text = $("#batch_create_addresses_text").val();
			if (!text) {
				return "<%=_T('Must enter an addresses list.')%>";
			}
		}
		text = text.trim();
		let addresses_s = text.split(rowsSeparator);
		let addresses_d = {};
		let addresses = [];
		error = null;
		let headers = null;
		$.each(addresses_s, function(index, address_s) {
			address_s = address_s.trim();
			if (includeHeaders && index === 0) {
				headers = self.__getAddressHeaders(address_s, columnsSeparator, addressColumn, nameColumn);
				if (headers.__error) {
					error = headers.__error;
					return false;
				}
				return;
			}
			if (!address_s) {
				return;
			}
			let address = self.__getAddress(address_s, headers, columnsSeparator, addressColumn, nameColumn, reportingGroup, agentData);
			if (address) {
				if (address.__error) {
					error = address.__error;
					return false;
				}
				address.address = self.__getAndValidateAddress(address.address);
				if (!address.address) {
					error = format("<%=_T('Invalid address: {0}.')%>", address_s);
					return false;
				}
				if (addresses_d[address.address]) {
					error = format("<%=_T('Duplicated address: {0}.')%>", address.address);
					return false;
				}
				addresses_d[address.address] = true;
				addresses.push(address);
			}
			else {
				error = format("<%=_T('Invalid address: {0}.')%>", address_s);
				return false;
			}
		});
		if (error) {
			return error;
		}
		if (!addresses.length) {
			return "<%=_T('Must enter an addresses list.')%>";
		}
		data.addresses = addresses;
	};


	
	AddressesPopup.prototype.__makeData = function() {
		let data = this.batch.data;
		
		let error = this.__addAddresses(data);
		if (error) {
			this.Message(error, "error", 5000);
			return null;
		}
		return data;
	};

	AddressesPopup.prototype.__addDataAsync = function(data, callback) {
		callback(true);
	};
	
	AddressesPopup.prototype.Create = function() {
		const self = this;
		if (self.processing) {
			return;
		}
		self.Message();
		self.processing = true;
		self.Loading(true);
		$("#outgoingBatchesAddressesPopup").find("button").prop("disabled", true);
		
		var onEnd = function() {
			self.processing = false;
			self.Loading(false);
			$("#outgoingBatchesAddressesPopup").find("button").prop("disabled", false);
		};

		var data = self.__makeData();
		if (!data) {
			onEnd();
			return;
		}
		
		self.__addDataAsync(data, function(status) {
			if (!status) {
				onEnd();
				return;
			}
			self.batch.RunAction("ADD_ADDRESSES", function(status, message) {
				onEnd();
				if (status) {
					return self.Close();
				}
				self.Message(message, "error", 8000);
			}, data, true);
		});
	};

}());