/*jshint bitwise:true, browser:true, curly:true, eqeqeq:true, noarg:true, noempty:true, strict:true, undef:true, unused:true, esversion: 9 */
/*jshint quotmark:false*/
/*global $, inConcert, Mustache*/

if (!this.inConcert) {
	this.inConcert = {};
}

if (!inConcert.batchManagementCreate) {
	inConcert.batchManagementCreate = {};
}

(function () {
/*jshint validthis:true*/
"use strict";

	const format = inConcert.Utils.format;
	const templates = inConcert.templates.batch_management;

	function CreatePopup (account)
	{
		CreatePopup.uber.constructor.call(this, account);
	}

	inConcert.Utils.inherit(CreatePopup, inConcert.batchManagementCreate.base);
	if (typeof inConcert.batchManagementCreate.mail !== "function") {
		inConcert.batchManagementCreate.mail = CreatePopup;
	}

	CreatePopup.prototype.__getTitle = function() {
		return "<%=_T('Create E-mails batch on')%>";
	};

	CreatePopup.prototype.__getTextLimit = function() {
		return 5000;
	};

	CreatePopup.prototype.__getClassName = function() {
		return "mailBatch";
	};

	CreatePopup.prototype.__getAttachmentSizeLimitMB = function() {
		return 64;
	};

	CreatePopup.prototype.__makeAfterIdContent = function() {
		return CreatePopup.uber.__makeAfterIdContent.call(this) + templates.create_popup_subject + templates.create_popup_attachments;
	};

	CreatePopup.prototype.Cancel = function(callback) {
		this.__removeAttachments(function() {
			callback();
		});
	};

	CreatePopup.prototype.__removeAttachment = function($attachment) {
		const self = this;
		var id = $attachment.attr("batch_attachment");
		inConcert.app.getBatchesManager().fileUpload.removeFileFromRepository(id, function(status) {
			$attachment.remove();
			if (!status) {
				self.Message("<%=_T('Error deleting file.')%>", "error", 5000);
			}
		});
	};

	CreatePopup.prototype.__removeAttachments = function(callback) {
		var $attachments = $("#batch_attachments").children("");
		if (!$attachments.length) {
			return callback();
		}
		var files = [];
		$attachments.each(function() {
			files.push({
				id : $(this).attr("batch_attachment"),
				section : "-1"
			});
		});
		inConcert.app.getBatchesManager().fileUpload.removeFilesFromRepository(files, function() {
			callback();
		});
	};

	CreatePopup.prototype.__appendAttachment = function(fileData) {
		const self = this;
		var $file = $(Mustache.render(templates.create_popup_attachment, {
			id : fileData.id,
			name : fileData.name
		}));
		$file
			.data("fileData", fileData)
			.appendTo($("#batch_attachments"))
			.children("[batch_attachment_delete]")
			.click(function() {
				$(this).prop("disabled", true);
				self.__removeAttachment($(this).parent());
			});
	};

	CreatePopup.prototype.__makeAttachment = function($body) {
		const self = this;
		$body.find("#batch_add_attachment_button").click(function() {
			$("#batch_add_attachment_input")[0].click();
		});
		$body.find("#batch_add_attachment_input").change(function(event) {
			var file = event.target.files[0];
			$(this).val("");
			if (!file) {
				return;
			}
			var sizeLimit = self.__getAttachmentSizeLimitMB();
			if (file.size > sizeLimit * 1024 * 1024) {
				return self.Message(format("<%=_T('Attachment file size must be less than {0}MB.')%>", sizeLimit),
					"error", 5000);
			}
			var $button = $body.find("#batch_add_attachment_button");
			var buttonContent = $button.html();
			$button
				.prop("disabled", true)
				.addClass("uploadingFile")
				.text("0%");
			inConcert.app.getBatchesManager().fileUpload.addFileToRepository(file,
				function(fileData) {
					$button
						.prop("disabled", false)
						.removeClass("uploadingFile")
						.html(buttonContent);
					if (fileData) {
						self.__appendAttachment(fileData);
					}
					else {
						self.Message("<%=_T('Error uploading file.')%>", "error", 5000);
					}
				},
				function(percentage) {
					$button.text(percentage + "%");
				});
		});
	};

	CreatePopup.prototype.__makeAddresses = function($body) {
		CreatePopup.uber.__makeAddresses.call(this, $body);
		// $body.find("[brackets_icon]")
		// 	.hide();
	};

	CreatePopup.prototype.__makeText = function($body) {
		this.__makeAttachment($body);
	};

	CreatePopup.prototype.__showText = function() {
		const self = this;
		let owner = {
			show_status_message : function(message, type, timeout) {
				self.Message(message, type.toLowerCase(), timeout);
			},
			is_readonly : function() {
				return false;
			}
		};
		this.html_editor = new inConcert.MailManagement({
			owner: owner,
			auto_init: true,
			ids: {
				body     : "batchesCreatePopup_textarea",
				container: "batchesCreatePopup_textcontainer"
			},
			callbacks: {
				loadVariables: function(sorted, callback = $.noop) {
					inConcert.app.loadVariablesForBatches(sorted, function (success, variables) {
						if (!success) {
							throw("Unable to load Variables");
						}
						callback(success, variables);
					});
				}
			}
		});
	};

	CreatePopup.prototype.__addSubject = function(data) {
		data.subject = $("#batch_create_subject").val();
		if (!data.subject) {
			return { text : "<%=_T('Must enter a subject.')%>", field : "batch_create_subject" };
		}
	};

	CreatePopup.prototype.__getEmbeedImages = $.noop;

	CreatePopup.prototype.__addBody = function(data, callback) {
		const self = this;
		self.html_editor.start_upload_images(function(status) {
			if (!status) {
				return callback({ text : "<%=_T('Error uploading embeed images.')%>", field : "batch_create_text" });
			}
			self.html_editor.validate(function(validationResult) {
				if (!validationResult.success) {
					return callback({ 
						text : validationResult.message || "<%=_T('Must enter a message body.')%>", 
						field : "batch_create_text" 
					});
				}
				data.body = self.html_editor.get_body_parts();
				var attachmentImages = self.html_editor.get_repository_images_list();
				$.each(attachmentImages, function(_, image) {
					data.attachments.push(image);
				});
				callback();
			});
		});
	};

	CreatePopup.prototype.__addAttachments = function(data) {
		data.attachments = data.attachments || [];
		$("#batch_attachments").children().each(function() {
			var fileData = $(this).data("fileData");
			data.attachments.push({
				id : fileData.id,
				name : fileData.name,
				type : fileData.type,
				"content-disposition" : "attachment"
			});
		});
	};

	CreatePopup.prototype.__addText = function(data) {
		return	this.__addSubject(data) || this.__addAttachments(data);
	};

	CreatePopup.prototype.__addDataAsync = function(data, callback) {
		const self = this;
		self.__addBody(data, function(error) {
			if (error) {
				self.Message(error, "error", 5000);
				return callback(false);
			}
			callback(true);
		});
	};

	CreatePopup.prototype.__getAndValidateAddress = function(address_t) {
		return inConcert.Utils.validate(address_t, "email").success? address_t : false;
	};

}());