/* jshint smarttabs:true, quotmark:false, esversion: 11 */
/* global $, inConcert */


if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	"use strict";

	const FORMAT = inConcert.Utils.format;
	const REQUEST_TIMEOUT_MS = 300000; // 5 minutes

	function API (params)
	{
		if(typeof(params) !== "object") {
			throw("BatchesMngAPI.Constructor - expected an object with configuration");
		}
		if(typeof(params.uri ) !== "string") {
			throw("BatchesMngAPI.Constructor - base uri is required");
		}
		this.uri = params.uri;
	}

	if (typeof inConcert.BatchesMngAPI !== "function") {
		inConcert.BatchesMngAPI = API;
	}

/********************************************************************************************************
	FUNCIONES GENERICAS
********************************************************************************************************/

	/*
	
	*/
	API.prototype.__Post = function(uri, callback, data)
	{
		if (typeof(uri) !== "string") {
			throw("API.prototype.__Post: uri variable must be string type.");
		}
		if (typeof(callback) !== "function") {
			throw("API.prototype.__Post: callback variable must be function type.");
		}
		if (typeof(data) !== "object") {
			throw("API.prototype.__Post: data variable must be object type.");
		}

		let request_handler = inConcert.invoke({
			url         : uri,
			type        : "POST",
			data        : JSON.stringify(data),
			dataType    : "json",
			contentType : "application/json",
			timeout     : REQUEST_TIMEOUT_MS,
			success     : function (response)    { callback(true, response); },
			error       : function (code, error) { callback(false, error);   }
		});
		return request_handler;
	};

	/********************************************************************************************************
	BATCHES
	********************************************************************************************************/

	API.prototype.searchBatches = function(data, callback = $.noop) {
		/*
		* sample URI:
		*   http://i6.inconcert.cc//inconcert/apps/batch_management/batches/search_batches/
		*/
		let uri = FORMAT("{0}batches/search_batches/", this.uri);

		let request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	API.prototype.getCampaignBatches = function(campaignId, callback = $.noop) {
		let data = {
			view : "ACTIVE_SCHEDULED",
			from : 0,
			rows : 500,
			query: {
				campaign: campaignId
			},
			sort: [{
				publish_timestamp: {
					order: "desc"
				}
			}]
		};

		this.searchBatches(data, function(success, response) {
			if (!success) {
				callback([]);
				return;
			}
			callback(response && response.batches || []);
		});
	};

	API.prototype.isChannelAvailable = function(channel = "") {
		const availableChannels = {
			"MAIL"    : true,
			"RCS"     : true,
			"SMS"     : true,
			"WHATSAPP": true
		};
		return availableChannels[channel] || false;
	};

	API.prototype.WhatsappGetAccountLimitData = function(data, callback = $.noop) {
		/*
		* sample URI:
		*   http://i6.inconcert.cc//inconcert/apps/batch_management/batches/whatsapp_get_account_limit_data/
		*/
		let uri = FORMAT("{0}batches/whatsapp_get_account_limit_data/", this.uri);

		let request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

}());
