/*jshint bitwise:true, browser:true, curly:true, eqeqeq:true, noarg:true, noempty:true, strict:true, undef:true, unused:true, esversion: 11 */
/*jshint quotmark:false*/
/*global $, inConcert, Mustache*/

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
/*jshint validthis:true*/
"use strict";

	const format = inConcert.Utils.format;
	const templates = inConcert.templates.batch_management;

	function batch_management() {
		const self = this;
		this.categories = [
			new inConcert.batchManagementList.total()
		];
		this.archive= new inConcert.batchManagementList.archive();
		this.active = new inConcert.batchManagementList.active();
		this.categories_dictionary = {};
		$.each(this.categories, function(_, category) {
			self.categories_dictionary[category.GetView()] = category;
		});
		this.currentCategory = null;
		this.currentStatsSlice = null;
		this.defaultStatsSlice = inConcert.outgoingBatchesStatisticsSlices.last7days;
	}

	inConcert.BatchManagementApp = batch_management;

	batch_management.prototype.__load_text_max_allowed_rows = function() {
		const replaceText = this.maxAllowedRows();
		const $divTextWarning = $("#outgoingBatchesMessagesInfo");
		let newText = $divTextWarning.text();
		newText = format(newText,replaceText);
		$divTextWarning.text(newText);
	};

	batch_management.prototype.can_show = function() {
		return inConcert.permissions.to_application("batch_management");
	};

	batch_management.prototype.getSelectedTabId = function() {
		return this.m_tabs_config.get_selected_tab_id().toUpperCase();
	};

	batch_management.prototype.__selectCategory = function(category, selectedAccount, callback = $.noop) {
		if (this.currentCategory && this.currentCategory.IsProcessing()) {
			return;
		}
		this.currentCategory = category;
		const $category = $("#outgoingBatchesTab").find(format("[category={0}]", this.getSelectedTabId()));
		$category
			.addClass("selected")
			.siblings()
			.removeClass("selected");
		category.Load(callback, selectedAccount);
		$("#outgoingBatchesSearchContainer").hide();
		$("#outgoingBatchesContainer").show();
	};

	batch_management.prototype.getActiveCategorie = function() {
		return this.active;
	};

	batch_management.prototype.getSelectedCategory = function() {
		let selectedCategory;
		let selectedTabId = this.getSelectedTabId();
		switch (selectedTabId) {
			case "ACTIVE":
				selectedCategory = this.active;
			break;
			case "ARCHIVE":
				selectedCategory = this.archive;
				break;
			default:
				throw("batch_management.prototype.getSelectedCategory - unrecognized category: " + selectedTabId);
		}
		return selectedCategory;
	};

	batch_management.prototype.MoveToCategory = function(categoryId, account, callback = $.noop) {
		const self = this;
		categoryId = (categoryId || "").toUpperCase();
		let category = this.categories_dictionary[categoryId];
		if (category === this.currentCategory) {
			return callback();
		}
		let cateogryInstance;
		let categoryTabId = categoryId.toLowerCase();
		switch (categoryId) {
			case "ACTIVE":
				cateogryInstance = this.active;
			break;
			case "ARCHIVE":
				cateogryInstance = this.archive;
				break;
			default:
				throw(`batch_management.prototype.MoveToCategory - unrecognized category '${categoryId}'`);
		}
		this.__selectCategory(cateogryInstance, account, function() {
			self.m_tabs_config.switch_tab(categoryTabId);
			callback();
		});
	};

	batch_management.prototype.__enterToCategory = function($target) {
		this.__changeTitleByEmail($target);
		if (!$target.attr("category_selector")) {
			return;
		}
		let categorieId = "";
		switch (this.getSelectedTabId()) {
			case "ACTIVE":
				categorieId = this.active;
			break;
			case "ARCHIVE":
				categorieId = this.archive;
				break;
			case "SCHEDULED":
				categorieId = this.scheduled;
				break;
		}
		this.__selectCategory(categorieId);
	};

	batch_management.prototype.__loadCategory = function(category) {
		const self = this;
		let $category = $(Mustache.render(templates.menu_category_item, {
			id         : category.GetView(),
			name       : category.GetName(),
			description: category.GetDescription(),
			className  : category.GetClassName()
		}));
		$category.click(function(e) {
			//check if event is fot this or for an account
			const $target = $(e.target);
			self.__enterToCategory($target);
		});
		return $category;
	};

	batch_management.prototype.__$loadCategories = function() {
		const self = this;
		let $categories = [];
		let $firstCategory;
		$.each(self.categories, function(_, category) {
			if (category.HasPermission()) {
				const $category = self.__loadCategory(category);
				if (!$firstCategory) {
					$firstCategory = $category;
				}
				$categories.push($category);
			}
		});
		$("#outgoingBatchMenu")
			.html("")
			.append($categories);
		return $firstCategory;
	};

	batch_management.prototype.UpdateBatches = function() {
		if (this.currentCategory && this.currentCategory.UpdateBatches) {
			this.currentCategory.UpdateBatches();
		}
	};
	
	batch_management.prototype.RefreshList = function(batchId) {
		if (this.currentCategory && this.currentCategory.Search) {
			this.currentCategory.selectedBatch = batchId;
			this.currentCategory.Search();
		}
	};

	batch_management.prototype.SelectStatsSlice = function(slice) {
		const self = this;
		if (self.currentCategory && self.currentCategory.SelectStatsSlice) {
			self.currentCategory.SelectStatsSlice(slice, function() {
				self.currentStatsSlice = slice;
			});
		}
	};

	batch_management.prototype.GetStatsSlice = function() {
		return this.currentStatsSlice || this.defaultStatsSlice;
	};

	batch_management.prototype.maxAllowedRows = function() {
		return 500;
	};

	batch_management.prototype.__changeTitleByEmail = function (target) {
		let titleName = target[0].innerText;
		const listItems = $("#outgoingBatchAccountList");
		const itemSelected = listItems.find(".selected").attr( "account" );
		const positionCharacter = itemSelected ? itemSelected.indexOf("_") : titleName = '';
		const typeIconTitle = positionCharacter !== '' ? itemSelected.substring(0, positionCharacter) : '';
		let $outgoingBatchTitleIcon = $("#outgoingBatchTitleIcon");
		let titleElementIcon = $("#outgoingBatchesIconPageTitle");
		let titleElement = $("#outgoingBatchesPageTitle");
		
		if (titleName.indexOf("\n") !== -1 ||
			titleName === "" ||
		 	titleName === `<%=_T('Batch Management')%>` ||
		  	titleName === "<%=_T('Total batches')%>" ) {
				titleElementIcon.removeClass('ic-icon-whatsapp-icon')
					.removeClass('ic-icon-sms-icon')
					.removeClass('ic-icon-mail-icon')
					.addClass("fa-layer-group");
				titleElement.text(`<%=_T('All batches')%>`);
				listItems.find(".selected").removeClass('selected');
				$outgoingBatchTitleIcon.addClass('selected');
		}
		else{
			switch (typeIconTitle) {
				case 'MAIL':
					titleElementIcon.removeClass('ic-icon-whatsapp-icon')
							.removeClass('fa-layer-group')
							.removeClass('ic-icon-sms-icon')
							.addClass('ic-icon-mail-icon');
					titleElement.text(titleName);
					$outgoingBatchTitleIcon.removeClass('selected');
					break;
				case 'WHATSAPP':
					titleElementIcon.removeClass('ic-icon-sms-icon')
							.removeClass('ic-icon-mail-icon')
							.removeClass('fa-layer-group')
							.addClass('ic-icon-whatsapp-icon');
						titleElement.text(titleName);
					$outgoingBatchTitleIcon.removeClass('selected');
					break;
				case 'SMS':
					titleElementIcon.removeClass('ic-icon-whatsapp-icon')
							.removeClass('ic-icon-mail-icon')
							.removeClass('fa-layer-group')
							.addClass('ic-icon-sms-icon');
					titleElement.text(titleName);
					$outgoingBatchTitleIcon.removeClass('selected');
					break;
			}
		}
	};

	batch_management.prototype.__initializeTabs = function () {
		const self = this;
		this.m_tabs_config = new inConcert.conf_form_tab({
			frm: this,
			container_id: "outgoingBatchesTab",
			must_initialize: true,
			tabs_list: {
				active: {
					id: "active",
					name: "<%=_T('Active')%>",
					tab_container_id: "outgoingBatchesTabActive",
					selected: true,
					on_show: function () {
						self.__selectCategory(self.active);
					}
				},
				archive: {
					id: "archive",
					name: "<%=_T('Archive')%>",
					tab_container_id: "outgoingBatchesTabArchive",
					selected: false,
					on_show: function () {
						self.__selectCategory(self.archive);
					}
				}
			}
		});
		this.m_tabs_config.show_tabs_container();
	};

	batch_management.prototype.render = function(callback = $.noop) {
		const self = this;
		function show() {
			self.$dashboard.show();
			callback();
		}
		if (this.$dashboard) {
			return show();
		}
		this.load_inner_page("/inconcert/apps/batch_management/pages/main.<%=_L%>.html#grunt-cache-bust", function(content, error) {
			if (!content) {
				inConcert.app.getMainWindow().showBannerMessage({
					message : format("<%=_T('Rendering batches management dashboard failed with error {0}')%>", error),
					messageType : "Error",
					timeout : 5000
				});
				callback();
				return;
			}
			self.$dashboard = $('#batch_management_dashboard').html(content);
			const $firstCategory = self.__$loadCategories();
			self.__initializeTabs();
			self.getSelectedTabId();
			self.__selectCategory(self.categories[0]);
			self.__load_text_max_allowed_rows();
			if ($firstCategory) {
				self.__enterToCategory($firstCategory);
			}
			show();
		});
	};

	batch_management.prototype.load_inner_page = function(page_url, callback) {
		inConcert.invoke({
			url        : page_url,
			type       : "GET",
			contentType: "text/html",
			success: function(response) {
				callback(response);
			},
			error: function(xhr, error) {
				callback(null, error.result);
			}
		});
	};
}());