local m_models = require("apps.campaign_analytics.model"):GetModels()
module(..., package.seeall, inconcert.orbit.application.Decorate)
local utils = require "shared.apps.utils"
m_htmlRender = nil

local LOG_CONTEXT = "campaign_analytics:alarms"

function MapUris(m, htmlRender, prefix)
	m_htmlRender = htmlRender
	local LogRequest = require("apps.agent.requestFilters").LogRequest

	m:dispatch_get(utils.JsonReply..LogRequest..ConfigList, prefix.."/configuration/")
	m:dispatch_get(utils.JsonReply..LogRequest..ConfigGet, prefix.."/configuration/(.+)")
	m:dispatch_post(utils.JsonValidatorReply..LogRequest..ConfigSaveAll, prefix.."/configuration/")
	m:dispatch_put(utils.JsonValidatorReply..LogRequest..ConfigSaveField, prefix.."/configuration/(.+)")

	m:dispatch_get(utils.JsonReply..LogRequest..NotificationsList, prefix.."/notifications/")
	m:dispatch_get(utils.JsonReply..LogRequest..UnreadNotificationsCount, prefix.."/notifications/unread")
	m:dispatch_post(utils.JsonReply..LogRequest..NotificationsReadAll, prefix.."/notifications/read_all/")
	m:dispatch_post(utils.JsonReply..LogRequest..NotificationsRead, prefix.."/notifications/read/(.+)")
end

function ConfigList(web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	LogInfo("[%s:ConfigList] %s ", LOG_CONTEXT, userId)

	local configuration = m_models.Alarms.GetConfiguration(userId)
	return { result = "OK", configuration = configuration }
end

function ConfigGet(web, field)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	LogInfo("[%s:ConfigGet] %s %s", LOG_CONTEXT, userId, field)

	local configuration = m_models.Alarms.GetConfigurationField(userId, field)
	return { result = "OK", configuration = configuration }
end

function ConfigSaveAll(web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local config = web.request.configuration
	if not config then
		web.status = 401
		return { result = "INVALID_REQUEST", reason = "empty configuration" }
	end

	LogInfo("[%s:ConfigSave] %s %s", LOG_CONTEXT, userId, json.encode(config))
	m_models.Alarms.SaveConfiguration(userId, config)
	return { result = "OK" }
end

function ConfigSaveField(web, field)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local config = web.request.configuration
	if not config then
		web.status = 401
		return { result = "INVALID_REQUEST", reason = "empty configuration" }
	end

	LogInfo("[%s:ConfigSaveField] %s %s", LOG_CONTEXT, userId, json.encode(config))
	m_models.Alarms.SavefigurationField(userId, field, config)
	return { result = "OK" }
end

function NotificationsList(web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	LogInfo("[%s:NotificationsList] %s", LOG_CONTEXT, userId)

	local notifications = m_models.Alarms.GetNotifications(userId)
	return { result = "OK", notifications = notifications }
end

function UnreadNotificationsCount(web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	LogInfo("[%s:UnreadNotificationsCount] %s", LOG_CONTEXT, userId)

	local notifications = m_models.Alarms.CountUnreadNotifications(userId)
	return { result = "OK", notifications = notifications }
end

function NotificationsReadAll(web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	LogInfo("[%s:NotificationsReadAll] %s", LOG_CONTEXT, userId)

	m_models.Alarms.ReadAllNotifications(userId)
	return { result = "OK" }
end

function NotificationsRead(web, notificationId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	LogInfo("[%s:NotificationsRead] %s %s", LOG_CONTEXT, userId, notificationId)

	m_models.Alarms.ReadNotification(userId, notificationId)
	return { result = "OK" }
end
