local m_models = require("apps.campaign_analytics.model"):GetModels()
module(..., package.seeall, inconcert.orbit.application.Decorate)
local utils = require "shared.apps.utils"
local helpers = require "helpers"
local permissions = require "shared.permissions.permissions"
m_htmlRender = nil
local m_outboundEngineClient = require "apps.outbound_engine.outboundengineclient"

local LOG_CONTEXT = "campaign_analytics:batch"

--TODO: remove this one when change to omnichannel
local DEFAULT_CHANNEL = "CALL"

local function CheckPermission(permission, method)
	return permissions:CheckPermission("campaign_analytics", permission, method)
end

function MapUris(m, htmlRender, prefix)
	m_htmlRender = htmlRender
	local LogRequest = require("apps.agent.requestFilters").LogRequest

	local function DoGet(permission, request)
		return utils.JsonReply .. LogRequest .. CheckPermission(permission, request)
	end

	local function DoRequest(permission, request)
		return utils.JsonValidatorReply .. LogRequest .. CheckPermission(permission, request)
	end

	m:dispatch_post(DoRequest("campaign_analytics_recycle_batch", RecycleBatch), prefix.."/recycle/(.+)/(.+)")
	m:dispatch_post(DoGet("campaign_analytics_pause_batch", PauseBatch), prefix.."/pause/(.+)/(.+)")
	m:dispatch_post(DoGet("campaign_analytics_pause_batch", ResumeBatch), prefix.."/resume/(.+)/(.+)")
	m:dispatch_post(DoGet("campaign_analytics_delete_batch", DestroyBatch), prefix.."/delete/(.+)/(.+)")
	m:dispatch_put(DoRequest("campaign_analytics_edit_batch", UpdateBatch), prefix.."/update/(.+)/(.+)")
end

local function ProcessResult(web, method, requestStr, result)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	if result.result ~= "OK" then
		web.status = 500
		LogError("[%s:%s] %s %s - %s", LOG_CONTEXT, method, userId, requestStr, json.encode(result))
	end
	return result
end

local function OeMethod(web, method, request)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local mockIndicators = inconcert.environment.apps.campaign_analytics.mock_indicators
	if mockIndicators then
		request.mockIndicators = true
	end
	local requestStr = json.encode(request)

	LogInfo("[%s:%s] %s %s", LOG_CONTEXT, method, userId, requestStr)
	local client = m_outboundEngineClient.GetClient()
	local result = client[method](client, requestStr)
	return ProcessResult(web, method, requestStr, result)
end

local function UpdateDataIfExists(data, updatedData, field)
	if updatedData[field] ~= nil then
		data[field] = updatedData[field]
	end
	return data
end

local function UpdateProcessData(process, updatedData)
	for channel, channelData in pairs(updatedData.channels or {}) do
		process.channels[channel] = UpdateChannelData(process.channels[channel], channelData)
	end
	process = UpdateDataIfExists(process, updatedData, "maxContactRetries")
	process = UpdateDataIfExists(process, updatedData, "maxContactRetriesDay")
	return process
end

local function MakeBatchActionParams(campaignId, batchId)
	local params = {
		vcc = inconcert.authUser.virtualContactCenter,
		campaignId = campaignId,
		admin = utils.AppendVcc(inconcert.authUser.userName, inconcert.authUser.virtualContactCenter),
		scope = {
			batchId = batchId
		}
	}
	return params
end

local function MakeRecycleParams(campaignId, batchId, data)
	local params = MakeBatchActionParams(campaignId, batchId)
	params.configuration = {
		restartRetries = data.restartRetries or false
	}
	params.scope.scheduled = data.scheduled or false
	params.scope.notContacted = data.notContacted or false
	return params
end

local function MakeDestroyBatchParams(campaignId, batchId)
	local params = {
		vcc = inconcert.authUser.virtualContactCenter,
		campaignId = campaignId,
		batchId = batchId
	}
	return params
end

local function MakeUpdateParams(campaignId, batchId, data)
	local params = MakeBatchActionParams(campaignId, batchId)
	params.batchId = batchId
	params.configuration = {}
	params.configuration = UpdateDataIfExists(params.configuration, data, "startDate")
	params.configuration = UpdateDataIfExists(params.configuration, data, "endDate")
	params.configuration = UpdateDataIfExists(params.configuration, data, "batchPriority")
	params.configuration = UpdateDataIfExists(params.configuration, data, "percentage")
	return params
end

function RecycleBatch(web, campaignId, batchId)
	return OeMethod(web, "RecycleContacts", MakeRecycleParams(campaignId, batchId, web.request))
end

function PauseBatch(web, campaignId, batchId)
	return OeMethod(web, "PauseContacts", MakeBatchActionParams(campaignId, batchId))
end

function ResumeBatch(web, campaignId, batchId)
	return OeMethod(web, "ResumeContacts", MakeBatchActionParams(campaignId, batchId))
end

function DestroyBatch(web, campaignId, batchId)
	return OeMethod(web, "DestroyBatch", MakeDestroyBatchParams(campaignId, batchId))
end

function UpdateBatch(web, campaignId, batchId)
	return OeMethod(web, "UpdateBatch", MakeUpdateParams(campaignId, batchId, web.request.batch or {}))
end
