local model_perm = require "shared.permissions.models.permissions"
local m_models = require("apps.campaign_analytics.model"):GetModels()
module(..., package.seeall, inconcert.orbit.application.Decorate)
local utils = require "shared.apps.utils"
local helpers = require "helpers"
m_htmlRender = nil
local m_outboundEngineClient = require "apps.outbound_engine.outboundengineclient"

local LOG_CONTEXT = "campaign_analytics:indicators"

function MapUris(m, htmlRender, prefix)
	m_htmlRender = htmlRender
	local LogRequest = require("apps.agent.requestFilters").LogRequest

	m:dispatch_get(utils.JsonReply..LogRequest..MetadataList, prefix.."/metadata/")
	m:dispatch_get(utils.JsonReply..LogRequest..CampaignsList, prefix.."/campaigns/")
	m:dispatch_get(utils.JsonReply..LogRequest..CampaignDetails, prefix.."/campaigns/(.+)")
	m:dispatch_get(utils.JsonReply..LogRequest..BatchesList, prefix.."/campaign_batches/(.+)")
	m:dispatch_get(utils.JsonReply..LogRequest..BatchDetails, prefix.."/campaign_batch/(.+)/(.+)")
	m:dispatch_get(utils.JsonReply..LogRequest..SkillsList, prefix.."/campaign_skills/(.+)")
	m:dispatch_get(utils.JsonReply..LogRequest..SkillDetails, prefix.."/campaign_skill/(.+)/(.+)")
	m:dispatch_get(utils.JsonReply..LogRequest..TrunkDetails, prefix.."/trunk/(.+)/(.+)")
	m:dispatch_get(utils.JsonReply..LogRequest..DidDetails, prefix.."/dids/(.+)")
	m:dispatch_get(utils.JsonReply..LogRequest..CapabilitiesDetails, prefix.."/capabilities/")
end

local function HasMockIndicators()
	return inconcert.environment.apps.campaign_analytics.mock_indicators
end

local function MockData(fileName)
	local path = string.format(
		"/usr/lib/inconcert/agent/inconcert/apps/campaign_analytics/mock_indicators/%s.lua",
		fileName
	)
	local runner = loadfile(path)
	return runner()
end

local function IndicatorsListFromParams(indicatorsParam, systemParams)
	local params = helpers.utils.SplitString(indicatorsParam or "", ",")
	for _, param in pairs(systemParams or {}) do
		table.insert(params, param)
	end
	return params
end

local function IndicatorsBoolMapFromParams(indicatorsParam)
	local list = IndicatorsListFromParams(indicatorsParam)
	local boolMap = {}
	for _, value in pairs(list) do boolMap[value] = true end
	return boolMap
end

local function BooleanFilter(filter)
	return (filter and filter == "true" or filter == "1") or false
end

local function GetCampaignAlarms(campaignId, userId)
	local client = m_outboundEngineClient.GetClient()
	return m_models.Alarms.GetActiveAlarms(campaignId, userId)
end

local function AddAlarms(request, result)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	result.unreadNotifications = m_models.Alarms.CountUnreadNotifications(userId)
	if request.campaignId then
		result.alarms = GetCampaignAlarms(request.campaignId, userId)
	elseif result.campaigns then
		for _, campaign in pairs(result.campaigns) do
			campaign.alarms = GetCampaignAlarms(campaign.Id, userId)
		end
	end
end

local function OeMethod(web, method, request, avoidAlarms)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	if HasMockIndicators() then
		request.mockIndicators = true
	end
	local requestStr = json.encode(request)

	LogInfo("[%s:%s] %s %s", LOG_CONTEXT, method, userId, requestStr)
	local client = m_outboundEngineClient.GetClient()
	local result = client["Analytics" .. method](client, requestStr)
	if result.result ~= "OK" then
		web.status = 500
		LogError("[%s:%s] %s %s - %s", LOG_CONTEXT, method, userId, requestStr, json.encode(result))
	end
	if not avoidAlarms then
		AddAlarms(request, result)
	end
	return result
end

function MetadataList(web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	LogInfo("[%s:MetadataList] %s ", LOG_CONTEXT, userId)

	local metadata = m_models.Indicators.GetMetadata()
	return { result = "OK", metadata = metadata }
end

local SYSTEM_CAMPAIGN_INDICATORS = {
	"State",
}

local function MakeCampaignsFilter(query)
	return {
		vcc = inconcert.authUser.virtualContactCenter,
		includeInactive = BooleanFilter(query.include_inactive),
		name = query.name or "",
		indicators = IndicatorsListFromParams(query.indicators, SYSTEM_CAMPAIGN_INDICATORS),
		campaigns = IndicatorsBoolMapFromParams(query.campaigns),
	}
end

local function MakeCampaignDetailsFilter(campaignId, query)
	return {
		vcc = inconcert.authUser.virtualContactCenter,
		campaignId = campaignId,
		detailsSlice = query.details_slice or "ByDate",
		historicalSlice = query.historical_slice or "ByHour",
	}
end

local SYSTEM_BATCH_INDICATORS = {
	"StartDate",
	"EndDate",
	"BatchPriority",
	"Percentage",
	"LoadedContacts",
	"EffectiveContacts",
	"ScheduledContacts",
	"RemainingContacts",
	"PausedContacts",
}

local function MakeBatchesFilter(campaignId, query)
	return {
		vcc = inconcert.authUser.virtualContactCenter,
		campaignId = campaignId,
		skillId = query.skill or "",
		includeInactive = BooleanFilter(query.include_inactive),
		indicators = IndicatorsListFromParams(query.indicators, SYSTEM_BATCH_INDICATORS),
	}
end

local function MakeBatchDetailsFilter(campaignId, batchId, query)
	return {
		vcc = inconcert.authUser.virtualContactCenter,
		campaignId = campaignId,
		batchId = batchId,
		skillId = query.skill or "",
		detailsSlice = query.details_slice or "ByDate",
		historicalSlice = query.historical_slice or "ByHour",
	}
end

local function MakeSkillsFilter(campaignId, query)
	return {
		vcc = inconcert.authUser.virtualContactCenter,
		campaignId = campaignId,
		indicators = IndicatorsListFromParams(query.indicators),
	}
end

local function MakeSkillDetailsFilter(campaignId, skillId, query)
	return {
		vcc = inconcert.authUser.virtualContactCenter,
		campaignId = campaignId,
		skillId = skillId,
		detailsSlice = query.details_slice or "ByDate",
		historicalSlice = query.historical_slice or "ByHour",
	}
end

local function MakeTrunkDetailsFilter(campaignId, trunkId, query)
	return {
		vcc = inconcert.authUser.virtualContactCenter,
		campaignId = campaignId,
		trunkId = trunkId,
		detailsSlice = query.details_slice or "ByDate",
	}
end

local function MakeDidDetailsFilter(campaignId, query)
	return {
		vcc = inconcert.authUser.virtualContactCenter,
		campaignId = campaignId,
		detailsSlice = query.details_slice or "ByDate",
	}
end

local function MakeCapabilitiesDetailsFilter(query)
	return {
		vcc = inconcert.authUser.virtualContactCenter,
		showSolved = BooleanFilter(query.show_solved),
		failsSlice = query.fails_slice or "By5Minutes",
	}
end

local function FilterOnlyUserCampaigns(result)
	-- Check if the result is ok and have campaigns
	if result.result == "OK" and next(result.campaigns) ~= nil then
		local keyspace      = utils.GetDatasource():CassandraConnection( "Agent" )
		local hasPermission = model_perm.IsPermissionEnabledForUser("administration", "admin_restricted", keyspace)

		LogInfo("[%s:FilterOnlyUserCampaigns] Checking if campaigns belongs to user: %s", LOG_CONTEXT, inconcert.authUser.userName)

		if hasPermission and not inconcert.authUser.api then
			local vcc           = inconcert.authUser.virtualContactCenter
			local userCampaigns = m_models.Campaigns.GetUserCampaigns(vcc, inconcert.authUser.userName, keyspace)

			for i=#result.campaigns,1,-1 do
				local belongs = false
				
				for campaignId in pairs(userCampaigns) do
					if result.campaigns[i].Id == campaignId then
						belongs = true
						break
					end
				end

				if not belongs then
					LogInfo("[%s:FilterOnlyUserCampaigns] User: %s not belongs to campaign: %s", LOG_CONTEXT, inconcert.authUser.userName, result.campaigns[i].Id)
					table.remove(result.campaigns, i)
				end
			end			
		end

		keyspace:Close()
	end

	return result
end

function CampaignsList(web)
	return FilterOnlyUserCampaigns(OeMethod(web, "CampaignsList", MakeCampaignsFilter(web.input or {})))
end

function CampaignDetails(web, campaignId)
	return OeMethod(web, "CampaignDetails", MakeCampaignDetailsFilter(campaignId, web.input or {}))
end

function BatchesList(web, campaignId)
	return OeMethod(web, "BatchesList", MakeBatchesFilter(campaignId, web.input or {}))
end

function BatchDetails(web, campaignId, batchId)
	return OeMethod(web, "BatchDetails", MakeBatchDetailsFilter(campaignId, batchId, web.input or {}))
end

function SkillsList(web, campaignId)
	return OeMethod(web, "SkillsList", MakeSkillsFilter(campaignId, web.input or {}))
end

function SkillDetails(web, campaignId, skillId)
	return OeMethod(web, "SkillDetails", MakeSkillDetailsFilter(campaignId, skillId, web.input or {}))
end

function TrunkDetails(web, campaignId, trunkId)
	return OeMethod(web, "TrunkDetails", MakeTrunkDetailsFilter(campaignId, trunkId, web.input or {}))
end

function DidDetails(web, campaignId)
	return OeMethod(web, "AccountDidDetails", MakeDidDetailsFilter(campaignId, web.input or {}), true)
end

function CapabilitiesDetails(web)
	if HasMockIndicators() then return MockData("capabilities") end

	return OeMethod(web, "CapabilitiesDetails", MakeCapabilitiesDetailsFilter(web.input or {}))
end
