local m_models = require("apps.campaign_analytics.model"):GetModels()
module(..., package.seeall, inconcert.orbit.application.Decorate)
local utils = require "shared.apps.utils"
local helpers = require "helpers"
local permissions = require "shared.permissions.permissions"
m_htmlRender = nil
local m_outboundEngineClient = require "apps.outbound_engine.outboundengineclient"
local agentCampaignModel = require "apps.agent.models.campaigns"
local adminPackagesModel = require "apps.administration.models.packages"
local agentCampaignController = require "apps.agent.controllers.campaigns"

local LOG_CONTEXT = "campaign_analytics:process"

--TODO: remove this one when change to omnichannel
local DEFAULT_CHANNEL = "CALL"

local function CheckPermission(permission, method)
	return permissions:CheckPermission("campaign_analytics", permission, method)
end

function MapUris(m, htmlRender, prefix)
	m_htmlRender = htmlRender
	local LogRequest = require("apps.agent.requestFilters").LogRequest

	local function DoGet(permission, request)
		return utils.JsonReply .. LogRequest .. CheckPermission(permission, request)
	end

	local function DoRequest(permission, request)
		return utils.JsonValidatorReply .. LogRequest .. CheckPermission(permission, request)
	end

	m:dispatch_post(DoGet("campaign_analytics_start_stop_process", StartProcess), prefix.."/start/(.+)")
	m:dispatch_post(DoGet("campaign_analytics_start_stop_process", StopProcess), prefix.."/stop/(.+)")
	m:dispatch_get(DoGet("campaign_analytics_edit_process", GetProcess), prefix.."/get/(.+)")
	m:dispatch_put(DoRequest("campaign_analytics_edit_process", UpdateProcess), prefix.."/update/(.+)")
	m:dispatch_get(DoGet("campaign_analytics_edit_process", GetProcessDids), prefix.."/get_dids/(.+)")
	m:dispatch_get(DoGet("campaign_analytics_edit_process", GetProcessDidsInUse), prefix.."/get_dids_in_use/(.+)")
	m:dispatch_put(DoRequest("campaign_analytics_edit_process", UpdateProcessDids), prefix.."/update_dids/(.+)")
end

local function ProcessResult(web, method, requestStr, result)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	if result.result ~= "OK" then
		web.status = 500
		LogError("[%s:%s] %s %s - %s", LOG_CONTEXT, method, userId, requestStr, json.encode(result))
	end
	return result
end

local function OeMethod(web, method, request)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)
	local mockIndicators = inconcert.environment.apps.campaign_analytics.mock_indicators
	if mockIndicators then
		request.mockIndicators = true
	end
	local requestStr = json.encode(request)

	LogInfo("[%s:%s] %s %s", LOG_CONTEXT, method, userId, requestStr)
	local client = m_outboundEngineClient.GetClient()
	local result = client[method](client, requestStr)
	return ProcessResult(web, method, requestStr, result)
end

local function MakeStartStopProcessParams(campaignId)
	local params = {
		vcc = inconcert.authUser.virtualContactCenter,
		campaignId = campaignId,
	}
	return params
end

function StartProcess(web, campaignId)
	return OeMethod(web, "StartProcess", MakeStartStopProcessParams(campaignId))
end

function StopProcess(web, campaignId)
	return OeMethod(web, "StopProcess", MakeStartStopProcessParams(campaignId))
end

local function UpdateDataIfExists(data, updatedData, field)
	if updatedData[field] ~= nil then
		data[field] = updatedData[field]
	end
	return data
end

local function UpdateChannelData(channel, updatedData)
	channel = UpdateDataIfExists(channel, updatedData, "engineType")
	channel = UpdateDataIfExists(channel, updatedData, "serviceLevel")
	channel = UpdateDataIfExists(channel, updatedData, "strictServiceLevel")
	channel = UpdateDataIfExists(channel, updatedData, "hasPreview")
	channel = UpdateDataIfExists(channel, updatedData, "correctionManualRate")
	channel = UpdateDataIfExists(channel, updatedData, "desiredAbandonRate")
	return channel
end

local function UpdateProcessData(process, updatedData)
	for channel, channelData in pairs(updatedData.channels or {}) do
		process.channels[channel] = UpdateChannelData(process.channels[channel], channelData)
	end
	process = UpdateDataIfExists(process, updatedData, "maxContactRetries")
	process = UpdateDataIfExists(process, updatedData, "maxContactRetriesDay")
	return process
end

local function DoGetProcess(web, campaignId, method, requestStr)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[%s:%s] %s %s", LOG_CONTEXT, method, userId, requestStr)
	local processId = m_outboundEngineClient.GetClient():GetCampaignProcessId(vcc, campaignId)
	if processId == "" then
		local result = { result = "FAIL", reason = "campaign process not found" }
		return ProcessResult(web, method, requestStr, result)
	end

	local status, process = utils.GetOEApi().native.processes.GetProcess(processId)
	if not status then
		local result = { result = "FAIL", reason = "process configuration not found" }
		return ProcessResult(web, method, requestStr, result)
	end

	process.isNew = nil
	process.id = processId
	return { result = "OK", process = process }
end

local function DoGetProcessAccount(web, campaignId, channel, method, requestStr)
	local result = DoGetProcess(web, campaignId, method, requestStr)
	if not result.process then
		return result
	end

	local channelData = result.process.configuration.channels[channel]
	if not channelData then
		local result = { result = "FAIL", reason = "process channel not found" }
		return ProcessResult(web, method, requestStr, result)
	end

	if not channelData.account then
		local result = { result = "FAIL", reason = "process channel account not found" }
		return ProcessResult(web, method, requestStr, result)
	end

	local vcc = inconcert.authUser.virtualContactCenter
	local campaignAtVCC = utils.AppendVcc(campaignId, vcc)
	local account = agentCampaignModel.GetAccount(channelData.account, campaignAtVCC)
	if not account then
		local result = { result = "FAIL", reason = "account not found" }
		return ProcessResult(web, method, requestStr, result)
	end

	return { result = "OK", account = account }
end

function GetProcess(web, campaignId)
	return DoGetProcess(web, campaignId, "GetProcess", "")
end

function UpdateProcess(web, campaignId)
	local updatedData = web.request.process
	if not updatedData then
		web.status = 401
		return { result = "INVALID_REQUEST", reason = "empty process" }
	end

	local requestStr = json.encode(updatedData)
	local result = DoGetProcess(web, campaignId, "UpdateProcess", requestStr)
	if not result.process then
		return result
	end

	local process = result.process
	process.configuration = UpdateProcessData(process.configuration, updatedData)
	local status, err = utils.GetOEApi().native.processes.SaveProcess(process)
	if status then
		result = { result = "OK", process = process }
	else
		result = { result = "FAIL", reason = err.type }
	end
	return ProcessResult(web, "UpdateProcess", requestStr, result)
end

function GetProcessDids(web, campaignId)
	local result = DoGetProcessAccount(web, campaignId, "CALL", "GetProcessDids", "")
	if not result.account then
		return result
	end

	if type(result.account.didsPool) == "string" then
		result.account.didsPool = json.decode(result.account.didsPool)
	end
	local dids = (result.account.didsPool and result.account.didsPool.dids) or {}
	return ProcessResult(web, "GetProcessDids", "", { result = "OK", dids = dids })
end

local function GetDidOtherAccounts(accountName, accounts)
	local otherAccounts = {}
	for otherAccountName, place in pairs(accounts) do
		if otherAccountName ~= accountName then
			otherAccounts[otherAccountName] = place
		end
	end
	return otherAccounts
end

local function GetOtherAccountsDids(accountName, allDids)
	local otherDids = {}
	for didId, accounts in pairs(allDids) do
		local otherAccounts = GetDidOtherAccounts(accountName, accounts)
		if not utils.TableIsEmpty(otherAccounts) then
			otherDids[didId] = otherAccounts
		end
	end
	return otherDids
end

function GetProcessDidsInUse(web, campaignId)
	local result = DoGetProcessAccount(web, campaignId, "CALL", "GetProcessDidsInUse", "")
	if not result.account then
		return result
	end

	local accountName = result.account.name
	local vcc = inconcert.authUser.virtualContactCenter
	local packages = adminPackagesModel.GetPackages(vcc, "CALL")
	local allDids = agentCampaignModel.GetVccDids(vcc, packages)
	local otherDids = GetOtherAccountsDids(accountName, allDids)
	return ProcessResult(web, "GetProcessDidsInUse", "", { result = "OK", dids = otherDids })
end

function UpdateProcessDids(web, campaignId)
	local updatedData = web.request.dids
	if not updatedData then
		web.status = 401
		return { result = "INVALID_REQUEST", reason = "empty process" }
	end

	local requestStr = json.encode(updatedData)
	local result = DoGetProcessAccount(web, campaignId, "CALL", "UpdateProcessDids", requestStr)
	if not result.account then
		return result
	end

	local vcc = inconcert.authUser.virtualContactCenter
	local campaignAtVCC = utils.AppendVcc(campaignId, vcc)
	local account = result.account
	if type(account.didsPool) == "string" then
		account.didsPool = json.decode(account.didsPool)
	end
	account.didsPool = account.didsPool or {}
	account.didsPool.dids = updatedData
	local success, result = agentCampaignController.UpdateCampaignAccount(account, campaignAtVCC)
	if success then
		result = { result = "OK" }
	end
	return ProcessResult(web, "UpdateProcessDids", requestStr, result)
end
