module(..., package.seeall)

local utils = require "shared.apps.utils"
local m_datasource

function GetName()
	return "Alarms"
end

function GetDefinition(datasource)
	m_datasource = datasource
	return _M
end

local function GetOutboundEngineRedisClient()
	local analyticsModel = require "apps.campaign_analytics.model"
	local oeRedis = analyticsModel:GetOutboundEngineRedisClient()
	return oeRedis
end

function GetConfiguration(userAtVcc, keyspace)
	return utils.GetAllColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "UserCampaignAnalyticsAlarms",
		key = userAtVcc,
	} or {}
end

function GetConfigurationField(userAtVcc, field, keyspace)
	local value = utils.GetColumnValue({
		keyspace = keyspace or "Agent",
		columnFamily = "UserCampaignAnalyticsAlarms",
		key = userAtVcc,
		column = field
	}) or ""
	return value ~= "" and json.decode(value) or nil
end

function SaveConfiguration(userAtVcc, configuration, keyspace)
	utils.SaveRows {
		keyspace = keyspace or "Agent",
		columnFamily = "UserCampaignAnalyticsAlarms",
		rows = {
			[userAtVcc] = configuration,
		}
	}
end

function SavefigurationField(userAtVcc, field, configuration, keyspace)
	utils.SaveRows {
		keyspace = keyspace or "Agent",
		columnFamily = "UserCampaignAnalyticsAlarms",
		rows = {
			[userAtVcc] = { [field] = json.encode(configuration) },
		}
	}
end

function GetActiveAlarms(campaignId, userAtVcc)
	local redis = GetOutboundEngineRedisClient()
	local key = string.format("CampaignAnalyticsAlarms::%s::%s", campaignId, userAtVcc)
	local alarmsStr = redis:get(key) or ""
	return (alarmsStr ~= "" and json.decode(alarmsStr)) or {}
end

function CountUnreadNotifications(userAtVcc)
	local redis = GetOutboundEngineRedisClient()
	local key = string.format("CampaignAnalyticsUnreadNotifications::%s", userAtVcc)
	return redis:zcard(key) or 0
end

local function GetNotificationsList(userAtVcc)
	local redis = GetOutboundEngineRedisClient()
	local key = string.format("CampaignAnalyticsNotifications::%s", userAtVcc)
	local entries = redis:zrevrange(key, 0, -1) or {}
	local list = {}
	for _, entry in pairs(entries) do
		table.insert(list, json.decode(entry))
	end
	return list
end

local function GetUnreadNotificationsList(userAtVcc)
	local redis = GetOutboundEngineRedisClient()
	local key = string.format("CampaignAnalyticsUnreadNotifications::%s", userAtVcc)
	local entries = redis:zrevrange(key, 0, -1) or {}
	local list = {}
	for _, entry in pairs(entries) do
		list[entry] = true
	end
	return list
end

function GetNotifications(userAtVcc)
	local notifications = GetNotificationsList(userAtVcc)
	local unread = GetUnreadNotificationsList(userAtVcc)
	for _, notification in pairs(notifications) do
		notification.unread = unread[notification.key] or false
	end
	return notifications
end

function ReadAllNotifications(userAtVcc)
	local redis = GetOutboundEngineRedisClient()
	local key = string.format("CampaignAnalyticsUnreadNotifications::%s", userAtVcc)
	redis:del(key)
end

function ReadNotification(userAtVcc, notificationId)
	local redis = GetOutboundEngineRedisClient()
	local key = string.format("CampaignAnalyticsUnreadNotifications::%s", userAtVcc)
	redis:zrem(key, notificationId)
end
