/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 11 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

//	"use strict";
	var format = inConcert.Utils.format;
	let API_VERSION;

	//---------------------------------------------------------------------
	// form cognitive_services class - application section
	function adm_form_cognitive_services_section_application(params)
	{
		var success = adm_form_cognitive_services_section_application.uber.constructor.call(this, params);
		if (!success) {
			return;
		}
		this.view_action_id = "view_cognitive_services";  // view action id de permisos
		this.save_action_id  = { create: "save_cognitive_services", update: "save_cognitive_services" }; // save action id de permisos
		this.erase_action_id = null; // delete action id de permisos

		return true;
	}

	inConcert.Utils.inherit(adm_form_cognitive_services_section_application, inConcert.conf_form_section);

	if ( typeof inConcert.adm_form_cognitive_services_section_application !== "function" ) {
		inConcert.adm_form_cognitive_services_section_application = adm_form_cognitive_services_section_application;
	}

	//// Utiliza el de la clase Base de secciones
	// adm_form_cognitive_services_section_application.prototype.dispose = $.noop;

	adm_form_cognitive_services_section_application.prototype.get = function(cognitive_service)
	{
		cognitive_service = cognitive_service || {};
		if ( this.frm.is_new ) {
			cognitive_service.culture =  $("#adm_form_cognitive_services_cultures").val();
			cognitive_service.api_version =  $("#adm_form_cognitive_services_api_versions").val();
		}

		cognitive_service.name = $("#adm_form_cognitive_services_name").val();
		cognitive_service.description = $("#adm_form_cognitive_services_desc").val();

		return cognitive_service;
	};

	adm_form_cognitive_services_section_application.prototype.set = function(cognitive_service)
	{
		const self = this;
		API_VERSION = this.frm.cognitive_service ? this.frm.cognitive_service.api_version : API_VERSION;

		$("#adm_form_cognitive_services_app_id").val(cognitive_service.id);
		$("#adm_form_cognitive_services_name").focus().val(cognitive_service.name);
		$("#adm_form_cognitive_services_desc").val(cognitive_service.description);
		$("#adm_form_cognitive_services_cultures").val(cognitive_service.culture);
		$("#adm_form_cognitive_services_api_versions").val(cognitive_service.api_version || "2.0");

		if (API_VERSION === "2022-10-01-preview") {
	
			this.m_hnd_refresh_layout = setTimeout(function() {
				let $container = $("#conf_application_actions");
				var $features_phrases_section = $container.find("[sub_menu_id='features_phrases']");
				$($features_phrases_section[0]).hide();
				window.clearTimeout(self.m_hnd_refresh_layout);
				self.m_hnd_refresh_layout = null;
			}, 1000);

		} else {

			this.m_hnd_refresh_layout = setTimeout(function() {
				let $container = $("#conf_application_actions");
				var $features_phrases_section = $container.find("[sub_menu_id='features_phrases']");
				$($features_phrases_section[0]).show();
				window.clearTimeout(self.m_hnd_refresh_layout);
				self.m_hnd_refresh_layout = null;
			}, 1000);			
		}

		this.update_app_version_info(cognitive_service.activeVersionInfo);
	};

	/*
		A esta funcion la invoca esta seccion o puede ser invocada desde otras secciones (ej: train and publish)
	*/
	adm_form_cognitive_services_section_application.prototype.update_app_version_info = function(info, callback)
	{
		const self = this;

		callback = callback || $.noop;

		var make_string_date = function(date) {
			return inConcert.Utils.getStringFromDate(inConcert.Utils.parseDateTimeString(date), false, "date-short-year-short-hour");
		};

		const $published_date = $("#adm_form_cognitive_services_app_info_last_published_date");
		const $train_date = $("#adm_form_cognitive_services_app_info_last_train_date");
		const $intent_count = $("#adm_form_cognitive_services_app_info_intent_count");
		const $entity_count = $("#adm_form_cognitive_services_app_info_entity_count");
		const $total_endpoint_hits = $("#adm_form_cognitive_services_app_info_total_endpoint_hits");

		var update = function(version_info)
		{
			if (typeof version_info !== "object") {
				callback();
				self.frm.dashboard.hide_waiting();
				return;
			}
			// 1 - La ultima fecha de PUBLISH
			let last_published_date = "<%=_T('Not published yet')%>";
			if (version_info.lastPublishedDateTime || version_info.lastDeployedDateTime) {
				last_published_date = make_string_date(version_info.lastPublishedDateTime || version_info.lastDeployedDateTime);
			}
			$published_date.val(last_published_date);

			// 2 - La ultima fecha de TRAIN
			let last_trained_date = "<%=_T('Not trained yet')%>";
			if (version_info.lastTrainedDateTime) {
				last_trained_date = make_string_date(version_info.lastTrainedDateTime);
			}
			$train_date.val(last_trained_date);

			const counter_template_msg = "{0}";
			// 3 - La cantidad de intents
			// Aca por defecto el valor es 1, ya que es el Intent None
			let intent_count = format(counter_template_msg, (version_info.intentsCount || 1));
			$intent_count.val(intent_count);

			// 4 - La cantidad de entities
			let entity_count = format(counter_template_msg, (version_info.entitiesCount || 0));
			$entity_count.val(entity_count);

			// 5 - El total endpoint hits
			let total_endpoint_hits = version_info.endpointHitsCount;
			$total_endpoint_hits.val(total_endpoint_hits);

			callback();
			self.frm.dashboard.hide_waiting();
		};

		if (info) {
			update(info);
			return;
		}

		const app = this.frm.get_entity();
		const params = {appId: app.id, version: app.activeVersion};
		const luisApi = inConcert.app.getLuisAPI();

		this.frm.dashboard.show_waiting("<%=_T('Wait moment please')%>");

		luisApi.getAppVersion(params,
			function(success, active_version_info)
			{
				if (!success) {
					self.frm.dashboard.hide_waiting();
					callback();
					return;
				}
				update(active_version_info);
			}
		);
	};

	adm_form_cognitive_services_section_application.prototype.get_validation_def = function(definitions)
	{
		definitions = definitions || [];
		definitions.push(
			{
				control_id: "adm_form_cognitive_services_name",
				messages: {
					required: "<%=_T('Please enter the cognitive service Name.')%>",
					id      : "<%=_T('Only letters, numbers, dashes and dots are allowed in the cognitive service Name.')%>"
				}
			},
			{
				control_id: "adm_form_cognitive_services_desc",
				messages: {
					required: "<%=_T('Please enter the cognitive service Description.')%>"
				}
			},
			{
				control_id: "adm_form_cognitive_services_cultures",
				messages: {
					required: "<%=_T('Please select the Culture.')%>"
				}
			},
			{
				control_id: "adm_form_cognitive_services_api_versions",
				messages: {
					required: "<%=_T('Please select the Api Version.')%>"
				}
			}
		);
	};

	adm_form_cognitive_services_section_application.prototype.attach_load_request = function(request, section)
	{
		if (!$.isArray(request)) {
			return;
		}
		var luisApi = inConcert.app.getLuisAPI();
		var self = this;
		var api_version = "";

		if (self && self.frm && !self.frm.is_new) {			
			if (section && section.frm && section.frm.frm && section.frm.frm.$current_row) {
				api_version = section.frm.frm.$current_row.find("td[column_id='api_version']").text() || "2.0";
			} else {
				api_version = (self.frm.cognitive_service && self.frm.cognitive_service.api_version ? self.frm.cognitive_service.api_version : "2.0");
			}
		}

		request.push(
			// cultures
			{
				request_function: function(success) {
					return luisApi.loadCultures(api_version, success);
				},
				success: function(list) {
					var $cultures = $("#adm_form_cognitive_services_cultures");
					var is_blank_first_item = true;
					var attr_id = "code";
					var attr_name = "name";
					var sorted = true;
					self.frm.fill_combo($cultures, is_blank_first_item, list, attr_id, attr_name, sorted);
				}
			},
			// cultures for intents
			{
				request_function: function(success) {
					return luisApi.loadCulturesForIntents(api_version, success);
				},
				success: function(list) {
					var $cultures = $("#adm_form_cognitive_services_cultures_for_intents");
					var is_blank_first_item = true;
					var attr_id = "code";
					var attr_name = "name";
					var sorted = true;
					self.frm.fill_combo($cultures, is_blank_first_item, list, attr_id, attr_name, sorted);
				}
			},
			// api versions
			{
				request_function: function(success) {
					return luisApi.loadAvailableApiVersions(success);
				},
				success: function(list) {
					var $versions = $("#adm_form_cognitive_services_api_versions");
					var is_blank_first_item = true;
					var attr_id = "code";
					var attr_name = "name";
					var sorted = false;
					self.frm.fill_combo($versions, is_blank_first_item, list, attr_id, attr_name, sorted);
				}
			}
		);
	};

	adm_form_cognitive_services_section_application.prototype.editable = function()
	{
		adm_form_cognitive_services_section_application.uber.editable.call(this);
		this.frm.editable_buttons();
	};

	adm_form_cognitive_services_section_application.prototype.readonly = function()
	{
		adm_form_cognitive_services_section_application.uber.readonly.call(this);
		this.frm.readonly_buttons();
	};

	adm_form_cognitive_services_section_application.prototype.only_modified = function(cognitive_service)
	{
		var $cultures = $("#adm_form_cognitive_services_cultures");
		if (this.frm.is_new) {
			this.frm.readonly_control($cultures, false /* disabled */, true /* is_required */);
		}
		else {
			this.frm.readonly_control($cultures, true /* disabled */);
		}

		var $versions = $("#adm_form_cognitive_services_api_versions");
		if (this.frm.is_new) {
			this.frm.readonly_control($versions, false /* disabled */, true /* is_required */);
		}
		else {
			this.frm.readonly_control($versions, true /* disabled */);
		}

		$("#adm_form_cognitive_services_name").focus();

		var $app_id = $("#adm_form_cognitive_services_app_id");
		this.frm.readonly_control($app_id, true /* disabled */);
		if (cognitive_service) {
			$app_id.val(cognitive_service.id);
		}
		this.show_or_hide_app_info();
	};

	adm_form_cognitive_services_section_application.prototype.show_or_hide_app_info = function()
	{
		var $info_container = $("#adm_form_cognitive_services_app_info");
		if (this.frm.is_new) {
			$info_container.hide();
			return;
		}

		var $train_date = $("#adm_form_cognitive_services_app_info_last_train_date");
		var $published_date = $("#adm_form_cognitive_services_app_info_last_published_date");
		var $intent_count = $("#adm_form_cognitive_services_app_info_intent_count");
		var $entity_count = $("#adm_form_cognitive_services_app_info_entity_count");
		var $total_endpoint_hits = $("#adm_form_cognitive_services_app_info_total_endpoint_hits");

		this.frm.readonly_control($train_date, true /* disabled */);
		this.frm.readonly_control($published_date, true /* disabled */);
		this.frm.readonly_control($intent_count, true /* disabled */);
		this.frm.readonly_control($entity_count, true /* disabled */);
		this.frm.readonly_control($total_endpoint_hits, true /* disabled */);

		$info_container.show();
	};

	adm_form_cognitive_services_section_application.prototype.ready = function()
	{
		this.only_modified();
	};

	adm_form_cognitive_services_section_application.prototype.on_show = function()
	{
		adm_form_cognitive_services_section_application.uber.on_show.call(this, {show: false});

		var is_readonly = this.is_readonly();
		if (is_readonly) {
			this.readonly();
		}
		else {
			this.editable();
		}

		this.set_events();
	};

	adm_form_cognitive_services_section_application.prototype.set_events = function()
	{
		const $api_version = $("#adm_form_cognitive_services_api_versions");
		
		$api_version
			.off("change")
			.on("change", function() {

                const $languaje = $("#adm_form_cognitive_services_cultures");

				let api_version = $api_version.val();
				if (api_version === "2022-10-01-preview") {
                    $languaje.val("en-us"); // The api works properly with the predefined English language, then the language can be configured at the intent and entity level
                    $languaje.parent().hide();
				} else {
                    $languaje.parent().show();
                }
			});

        $api_version.trigger("change");
	};

	adm_form_cognitive_services_section_application.prototype.can_jump_to_other_menunav = function()
	{
		let resp = true;
		const cognitive_service = this.frm.cognitive_service;
		const curr_cognitive_service = this.get(inConcert.Utils.cloneObject(cognitive_service));
			
		if (!cognitive_service || !curr_cognitive_service || inConcert.Utils.diffObjects(curr_cognitive_service, cognitive_service)) {
			resp = false;
		}
		return resp;
	};

	adm_form_cognitive_services_section_application.prototype.before_jump_to_other_menunav = function()
	{
		this.set(this.frm.cognitive_service);
	};

}());
