/*  jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
  strict:true, undef:true, unused:true, curly:true, browser:true, indent:false, maxerr:50, esversion: 11 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	const FORMAT = inConcert.Utils.format;
	const POPUP_ID_ENTITIES = 0;
	const POPUP_ID_PREBUILT_ENTITIES = 1;
	const ERROR_TIMEOUT_MS = 5000;
	let API_VERSION;

	//---------------------------------------------------------------------
	// form cognitive services class - entities section
	function adm_form_cognitive_services_section_entities(params)
	{
		const success = adm_form_cognitive_services_section_entities.uber.constructor.call(this, params);
		if (!success) {
			return;
		}
		//API_VERSION = inConcert.app.get_luis_api_version();

		const self = this;

		this.view_action_id  = "view_cognitive_services"; // view action id de permisos
		this.save_action_id  = "save_cognitive_services"; // save action id de permisos
		this.erase_action_id = "save_cognitive_services"; // delete action id de permisos

		this.is_new = null;
		this.current_item = null;
		this.loading = false;

		this.id_prefix = "adm_form_cognitive_services_entities";
		// entity popup
		this.id_btn_new = this.id_prefix + "_new";
		this.id_new_entity_popup = this.id_prefix + "_new_popup";
		// prebuilt entity popup
		this.id_btn_add_prebuilt_entity = this.id_prefix + "_add_prebuilt";
		this.id_add_prebuilt_entity_popup = this.id_prefix + "_add_prebuilt_popup";

		// combine_component entity popup
		this.id_add_combine_component_entity_popup = this.id_prefix + "_add_combine_component_popup";

		this.m_prebuilt_entities = [];

		var must_show_button = function(row)
		{
			let show = true;
			let values_to_exclude = {typeId: 2}; // PREBUILT
			for (let name in values_to_exclude) {
				if (values_to_exclude.hasOwnProperty(name)) {
					let value = values_to_exclude[name];
					if (row[name] === value) {
						show = false;
						break;
					}
				}
			}
			return show;
		};

		this.form_paginator = new inConcert.conf_form_paginator({
			owner: this,
			frm: this.frm,
			row_id: "id",
			display_name: "name",
			rows_per_page: 11,
			show_at_least: 10,
			paginator_type: "ALL_ROWS_CLIENT_SIDE",
			canEdit: function(row) {
				return must_show_button(row);
			},
			ids: {
				paginator: this.id_prefix + "_paginator",
				grid: this.id_prefix + "_list",
				not_results_found: this.id_prefix + "_not_results_found",
				//popup0 (new entity) (POPUP_ID_ENTITIES)
				new_item: this.id_btn_new,
				popup: this.id_new_entity_popup,
				popup_msg: this.id_new_entity_popup + "_message",
				popup_create: this.id_new_entity_popup + "_create",
				popup_create_and_continue: this.id_new_entity_popup + "_create_and_continue",
				popup_title: this.id_new_entity_popup + "_title",
				popup_close: this.id_new_entity_popup + "_close",
				popup_close_times: this.id_new_entity_popup + "_close_times",
				popup_input_id: this.id_new_entity_popup + "_name",
				popup_waiting: this.id_new_entity_popup + "_waiting",
				//popup1 (add prebuilt entity) (POPUP_ID_PREBUILT_ENTITIES)
				new_item1: this.id_btn_add_prebuilt_entity,
				popup1: this.id_add_prebuilt_entity_popup,
				popup1_msg: this.id_add_prebuilt_entity_popup + "_message",
				popup1_create: this.id_add_prebuilt_entity_popup + "_create",
				popup1_create_and_continue: this.id_add_prebuilt_entity_popup + "_create_and_continue",
				popup1_title: this.id_add_prebuilt_entity_popup + "_title",
				popup1_close: this.id_add_prebuilt_entity_popup + "_close",
				popup1_close_times: this.id_add_prebuilt_entity_popup + "_close_times",
				popup1_input_id: this.id_add_prebuilt_entity_popup + "_name",
				popup1_waiting: this.id_add_prebuilt_entity_popup + "_waiting"
			},
			strings: {
			// popup0 (new entity) (POPUP_ID_ENTITIES)
				not_results_found         : "<%=_T('No Entities found.')%>",
				can_remove                : "<%=_T('Do you want to delete Entity: {0} ?')%>",
				title_new                 : "<%=_T('New Entity')%>",
				title_update              : "<%=_T('Update Entity')%>",
				button_create             : "<%=_T('Create')%>",
				button_create_and_continue: "<%=_T('Create & Continue')%>",
			//popup1 (add prebuilt entity) (POPUP_ID_PREBUILT_ENTITIES)
				not_results_found1        : "<%=_T('No Prebuilt Entities found.')%>",
				can_remove1               : "<%=_T('Do you want to delete Prebuilt Entity: {0} ?')%>",
				title_new1                : "<%=_T('Assign Prebuilt Entities')%>",
				button_create1            : "<%=_T('Assign')%>"
			},
			columns: [
				{id: "_number", name: "", type: "counter"},
				{id: "name"   , name: "<%=_T('Name')%>"  },
				{id: "type"   , name: "<%=_T('Type')%>", get_value: function(item) {
					return self.get_grid_value(item, "typeId");
				}},
				{
					id: "_action",
					name: "",
					system_action: {
						edit: {
							must_show: function(row) {
								return must_show_button(row);
							}
						},
						erase: true
					}
				}
			],
			callbacks: {
				save: function(callback, must_create_and_contiune, current_popup) {
					callback = callback || $.noop;
					self.save(callback, must_create_and_contiune, current_popup);
				},
				edit: function(row, current_popup) {
					self.on_open_popup(row, current_popup);
				},
				remove: function(row, callback = $.noop) {
					self.remove(row, callback);
				},
				on_closed_popup: function(current_popup) {
					self.on_closed_popup(current_popup);
				}
			}
		});

		this.reload_entities_types();

		return true;
	}

	inConcert.Utils.inherit(adm_form_cognitive_services_section_entities, inConcert.conf_form_section);

	if (typeof inConcert.adm_form_cognitive_services_section_entities !== "function") {
		inConcert.adm_form_cognitive_services_section_entities = adm_form_cognitive_services_section_entities;
	}

	adm_form_cognitive_services_section_entities.prototype.reload_entities_types = function()
	{
		if (!API_VERSION) {
			API_VERSION = "2.0";
		}

		const self = this;

		let simple_children_info = {
			enabled: false
		};

		let machine_learner_children_info = {
			enabled: true,
			// Estos IDs se encuentran en (frm_cognitive_services_content_entities.html)
			container_id: self.id_new_entity_popup + "_machine_learner_childrens_container",
			jquery_prefix_id: "#" + self.id_new_entity_popup + "_machine_learner_childrens",
			class_name: machine_learner_children_list,
			instance: null,
			create: function() {
				return self.__create_entity_type_container(this);
			}
		};

		let get_type1_children_info = function() {
			if (API_VERSION === "2.0") {
				return simple_children_info;
			}
			return machine_learner_children_info;
		};

		this.m_entities_types = {};
		
		if (API_VERSION === "2.0" || API_VERSION === "3.0") {
			this.m_entities_types["1"] = {
				id: 1,
				name: function() {
					if (API_VERSION === "2.0") {
						return "<%=_T('Simple')%>";
					}
					return "<%=_T('Machine Learned')%>";
				},
				help: function() {
					if (API_VERSION === "2.0") {
						return "<%=_T('HELP_COGNITIVE_SERVICES_SIMPLE_ENTITY')%>";
					}
					return "<%=_T('HELP_COGNITIVE_SERVICES_MACHINE_LEARNED_ENTITY')%>";
				},
				can_update_memory_info_on_closed_poup: function() {
					if (API_VERSION === "2.0") {
						return false;
					}
					return true;
				},
				children_info: get_type1_children_info(),
				api_luis_methods: {
				// Estos metodos deben coincidir con el nombre de la clase "LuisAPI" -> "/shared/cognitive_services/luis/scripts/luis_api.js"
					create: "createAppEntityMachineLearned",
					rename: "renameAppEntityMachineLearned",
					erase : "deleteAppEntityMachineLearned"
				},
				load: function(callback = $.noop) {
					callback(true);
				},
				can_show_in_combo: true
			};

			this.m_entities_types["2"] = {
				id: 2,
				name: "<%=_T('Prebuilt')%>",
				//help: "<%=_T('HELP_COGNITIVE_SERVICES_PREBUILT_ENTITY')%>",
				can_update_memory_info_on_closed_poup: function() {
					return false;
				},
				children_info: {
					enabled: true,
				// Estos IDs se encuentran en (frm_cognitive_services_content_entities.html)
					container_id: this.id_add_prebuilt_entity_popup + "_available_list_container", 
					jquery_prefix_id: "#" + this.id_add_prebuilt_entity_popup + "_available_list",
					class_name: prebuilt_children_list,
					instance: null,
					create: function() {
						return self.__create_entity_type_container(this);
					}
				},
				api_luis_methods: {
				// Estos metodos deben coincidir con el nombre de la clase "LuisAPI" -> "/shared/cognitive_services/luis/scripts/luis_api.js"
					create: "addAppEntityPrebuilt",
					rename: null,
					erase : "deleteAppEntityPrebuilt"
				},
				load: function(callback = $.noop) {
					callback(true);
				},
				can_show_in_combo: false
			};

			this.m_entities_types["3"] = {
				id: 3,
				name: "<%=_T('Hierarchical')%>",
				help: "<%=_T('HELP_COGNITIVE_SERVICES_HIERARCHICAL_ENTITY')%>",
				can_update_memory_info_on_closed_poup: function() {
					return false;
				},
				children_info: {
					enabled: true,
				// Estos IDs se encuentran en (frm_cognitive_services_content_entities.html)
					container_id: this.id_new_entity_popup + "_hierarchical_childrens_container",
					jquery_prefix_id: "#" + this.id_new_entity_popup + "_hierarchical_childrens",
					class_name: hierarchical_children_list,
					instance: null,
					create: function() {
						return self.__create_entity_type_container(this);
					}
				},
				api_luis_methods: {
				// Estos metodos deben coincidir con el nombre de la clase "LuisAPI" -> "/shared/cognitive_services/luis/scripts/luis_api.js"
					create: "createAppEntityHierarchical",
					rename: "renameAppEntityHierarchical",
					erase : "deleteAppEntityHierarchical"
				},
				load: function(callback = $.noop) {
					callback(true);
				},
				can_show_in_combo: true
			};

			this.m_entities_types["4"] = {
				id: 4,
				name: "<%=_T('Composite')%>",
				help: "<%=_T('HELP_COGNITIVE_SERVICES_COMPOSITE_ENTITY')%>",
				can_update_memory_info_on_closed_poup: function() {
					return false;
				},
				children_info: {
					enabled: true,
				// Estos IDs se encuentran en (frm_cognitive_services_content_entities.html)
					container_id    : this.id_new_entity_popup + "_composite_childrens_container",
					jquery_prefix_id: "#" + this.id_new_entity_popup + "_composite_childrens",
					class_name: composite_children_list,
					instance: null,
					create: function() {
						return self.__create_entity_type_container(this);
					}
				},
				api_luis_methods: {
				// Estos metodos deben coincidir con el nombre de la clase "LuisAPI" -> "/shared/cognitive_services/luis/scripts/luis_api.js"
					create: "createAppEntityComposite",
					rename: "renameAppEntityComposite",
					erase : "deleteAppEntityComposite"
				},
				load: function(callback = $.noop) {
					callback(true);
				},
				can_show_in_combo: true
			};
		} else {
			this.m_entities_types.combineComponents = {
				id: "combineComponents",
				name: "<%=_T('Combine Components')%>",
				help: function() {
					return "<%=_T('The Learned component uses the entity labels you label your utterances with to train a machine learned model.')%>";
				},
				can_update_memory_info_on_closed_poup: function() {
					return false;
				},
				//children_info: simple_children_info,
				children_info: {
					enabled: true,
					// Estos IDs se encuentran en (frm_cognitive_services_content_entities.html)
					container_id: this.id_add_combine_component_entity_popup + "_available_list_container", 
					jquery_prefix_id: "#" + this.id_add_combine_component_entity_popup + "_available_list",
					class_name: combine_component_children_list,
					instance: null,
					create: function() {
						return self.__create_entity_type_container(this);
					}
				},
				api_luis_methods: {
				// Estos metodos deben coincidir con el nombre de la clase "LuisAPI" -> "/shared/cognitive_services/luis/scripts/luis_api.js"
					create: "createAppEntityMachineLearned",
					rename: null,
					erase : "deleteAppEntityMachineLearned"
				},
				load: function(callback = $.noop) {
					callback(true);
				},
				can_show_in_combo: true
			};
		}

		this.m_entities_type_names = {};
		this.m_entities_type_names["1"] = "<%=_T('Simple')%>";
		this.m_entities_type_names["2"] = "<%=_T('Prebuilt')%>";
		this.m_entities_type_names["3"] = "<%=_T('Hierarchical')%>";
		this.m_entities_type_names["4"] = "<%=_T('Composite')%>";
		this.m_entities_type_names.combineComponents = "<%=_T('Combine Components')%>";

		let entities_to_display_in_combo = {}; // lista con la lista de Tipos de Entidades que se visualizaran en el popup de Entidades.

		for (let id in this.m_entities_types) {
			if (this.m_entities_types.hasOwnProperty(id)) {
				let entity_type = this.m_entities_types[id];
				if (entity_type.children_info && typeof(entity_type.children_info.create) === "function") {
					if (entity_type.children_info.create()) {
						entity_type.children_info.instance.initialize();
					}
				}
				// items a visualizar en el combo del popup de Type de Entidades
				if (entity_type.can_show_in_combo) {
					entities_to_display_in_combo[id] = entity_type;
				}
			}
		}

		// cargo el combo del popup de Type entidades
		const $entities_type = $("#" + this.id_new_entity_popup + "_type");
		this.frm.fill_combo($entities_type, false, entities_to_display_in_combo, "id", "name", false);
		$entities_type.trigger("change");

		// aplica cambios en la gui segun la version del API.
		this.toggle_gui_ctrls_by_api_version();
	};

	adm_form_cognitive_services_section_entities.prototype.get_grid_value = function(item, field_id)
	{
		let value = {
			original_value: "",
			value         : "",
			search_value  : ""
		};

		let item_value = item[field_id] || "";
		if (item_value === undefined) {
			return value;
		}

		value.original_value = item_value;
		value.value = item_value;
		value.search_value = item_value;

		if (field_id === "typeId") {
			let entity_type_def = this.__get_entity_type_def(value.original_value);
			value.value = entity_type_def ? (typeof entity_type_def.name === "function" ? entity_type_def.name() : entity_type_def.name) : "No reconocido";
			value.search_value = value.value;
		}

		return value;
	};

	adm_form_cognitive_services_section_entities.prototype.__create_entity_type_container = function(info)
	{
		if (!info) {
			return;
		}
		info.instance = new info.class_name({
			owner    : this,
			prefix_id: info.jquery_prefix_id
		});
		return true;
	};

	adm_form_cognitive_services_section_entities.prototype.__get_entity_type_def = function(type = "")
	{
		type = type.toString();
		let entity_type_def = this.m_entities_types[type];
		if (!entity_type_def) {
            entity_type_def = {};
        }
		entity_type_def.name = this.m_entities_type_names[type];
		return entity_type_def;
	};

	adm_form_cognitive_services_section_entities.prototype.build_header_custom_buttons = function()
	{
		let html_buttons = "";

		if (API_VERSION !== "2022-10-01-preview") {
			html_buttons += "<button id='" + this.id_btn_add_prebuilt_entity + "' class='pageButtonGeneric buttonColorWhite'>" +
				"<%=_T('Assign Prebuilt Entity')%>" +
			"</button>";
		}

		html_buttons += "<button id='" + this.id_btn_new + "' class='pageButtonGeneric buttonColorPrimary'>" +
			"<%=_T('Add Entity')%>" +
		"</button>";

		adm_form_cognitive_services_section_entities.uber.build_header_custom_buttons.call(this, html_buttons);
	};

	adm_form_cognitive_services_section_entities.prototype.remove = function(entity, callback = $.noop)
	{
		const self = this;

		if (!entity) {
			throw("adm_form_cognitive_services_section_entities.prototype.remove. Undefined Entity.");
		}

		var end = function(success, result) {
			let msg;
			if (success) {
				msg = FORMAT("<%=_T('Entity {0} was successfully deleted.')%>", entity.name);
				self.__update_app_version_info(function() {
					self.frm.enable_all_sections();
					self.frm.show_status_message(msg, "Info", ERROR_TIMEOUT_MS);
					self.frm.dashboard.hide_waiting();
				});
			}
			else {
				let server_err_msg = luisApi.getCognitiveErrorMsg(result);
				msg = FORMAT("<%=_T('Error deleting Entity {0}.')%>", entity.name) + " " + server_err_msg;

				self.frm.enable_all_sections();
				self.frm.show_status_message(msg, "Error", ERROR_TIMEOUT_MS);
				self.frm.dashboard.hide_waiting();
			}
			callback(success);
		};

		this.frm.dashboard.show_waiting(FORMAT("<%=_T('Deleting Entity {0}.')%>", entity.name));
		this.disabled_all_sub_menus(true);

		let app_id = this.frm.get_entity_id();
		if (!app_id) {
			end(false);
			return;
		}

		const params = {appId: app_id, entityId: entity.id || entity.name};
		const luisApi = inConcert.app.getLuisAPI();
		const entity_type_def = this.__get_entity_type_def(entity.typeId);
		const api_luis_methods = entity_type_def.api_luis_methods;

		/*
			Metodos ERASE
				MachineLearned: "deleteAppEntityMachineLearned"
				Hierarchical  : "deleteAppEntityHierarchical"
				Composite     : "deleteAppEntityComposite"
				Prebuilt      : "deleteAppEntityPrebuilt"
		*/
		luisApi[api_luis_methods.erase](params,
			function(success, result) {
				end(success, result);
			}
		);
	};

	adm_form_cognitive_services_section_entities.prototype.set_events = function()
	{
		const self = this;
		this.form_paginator.set_events();

		const $entities_type = $("#" + this.id_new_entity_popup + "_type");
		$entities_type
			.off("change")
			.on("change", function() {
				let entity_type = $entities_type.val();

				if (entity_type === null) {
					let attrs = Object.keys(self.m_entities_types);
					let firstAttr = attrs[0];
					entity_type = self.m_entities_types[firstAttr].id;
					$entities_type.val(entity_type);
				}

				self.__on_change_entity_type(entity_type);
			});
	};

	adm_form_cognitive_services_section_entities.prototype.__on_change_entity_type = function(entity_type)
	{
		const entity_type_def = this.__get_entity_type_def(entity_type);
		let disclaimer = "";
		let children_info;

		if (entity_type_def) {
			disclaimer = typeof entity_type_def.help === "function" ? entity_type_def.help() : entity_type_def.help;
			children_info = entity_type_def.children_info;
		}

		const $disclaimer = $("#" + this.id_new_entity_popup + "_type_disclaimer");
		$disclaimer.html(disclaimer);

		// Oculto todos los contenedores especificos de los diferentes tipos de entidades (COMPOSITE, HIERARCHICAL)
		const $popup = $("#" + this.id_new_entity_popup);
		$popup.find("[childrens_container]").hide();

		if (children_info) {
			// Ahora visualizo el contenedor especifico para el tipo de entidad
			const $composite_children_list = $("#" + children_info.container_id);
			if ($composite_children_list.length > 0) {
				$composite_children_list.show();
			}
		}
	};

	adm_form_cognitive_services_section_entities.prototype.__show_or_hide_loading_msg = function(message)
	{
		if (this.is_loading()) {
			this.frm.dashboard.show_waiting(message || "<%=_T('Loading Entities.')%>");
		}
		else {
			this.frm.dashboard.hide_waiting();
		}
	};

	adm_form_cognitive_services_section_entities.prototype.load_entities = function(callback = $.noop)
	{
		const self = this;

		let app_id = this.frm.get_entity_id();
		if (!app_id) {
			callback(false);
			return;
		}
		this.loading = true;
		this.__show_or_hide_loading_msg();

		if (self.frm && self.frm.cognitive_service && 
			self.frm.cognitive_service.activeVersionInfo && 
			self.frm.cognitive_service.activeVersionInfo.entities) {

			self.loading = false;
			callback(true, self.frm.cognitive_service.activeVersionInfo.entities);

		} else {
			const params = {appId: app_id};

			const luisApi = inConcert.app.getLuisAPI();
			luisApi.loadAllEntities(params,
				function(success, entities) {

					self.loading = false;
					if (!success) {
						self.frm.show_status_message("<%=_T('Error retrieving Entities.')%>", "Error", false);
						callback(false);
						return;
					}
					callback(true, entities);
				}
			);
		}
			
	};

	adm_form_cognitive_services_section_entities.prototype.__init_entities_paginator = function()
	{
		const self = this;
		this.load_entities(function(success, entities) {
			if (!success) {
				return;
			}
			self.form_paginator.initialize($.noop, entities);
		});
	};

	adm_form_cognitive_services_section_entities.prototype.initialize = function()
	{

		this.__init_entities_paginator();
	};

	adm_form_cognitive_services_section_entities.prototype.toggle_gui_ctrls_by_api_version = function()
	{
		const $entities_type = $("#" + this.id_new_entity_popup + "_type");
		const $parent = $entities_type.parent();
		if (API_VERSION === "2.0") {
			this.frm.readonly_control($entities_type, false);
			$parent.show();
		}
		else if (API_VERSION === "2022-10-01-preview") {
			this.frm.readonly_control($entities_type, true);
			$parent.hide();
		}
		else {
			this.frm.readonly_control($entities_type, true);
			$parent.parent().show();
		}
	};

	adm_form_cognitive_services_section_entities.prototype.set = function()
	{
		API_VERSION = this.frm.cognitive_service ? this.frm.cognitive_service.api_version : API_VERSION;
		this.initialize();
	};

	adm_form_cognitive_services_section_entities.prototype.on_show = function()
	{
		API_VERSION = this.frm.cognitive_service ? this.frm.cognitive_service.api_version : API_VERSION;
		this.reload_entities_types();
		adm_form_cognitive_services_section_entities.uber.on_show.call(this, {show: false});
		this.frm.dashboard.$get_dom_element_by_id("form_buttons_actions").hide();
		this.set_events();
	};

	adm_form_cognitive_services_section_entities.prototype.on_hide = function()
	{
		adm_form_cognitive_services_section_entities.uber.on_hide.call( this );
		this.form_paginator.hide_popup();
	};

	adm_form_cognitive_services_section_entities.prototype.__load_entity_popup_info = function(callback = $.noop)
	{
		let total_entities_type = Object.keys(this.m_entities_types || {}).length;
		let i = 0;

		var on_load = function() {
			i++;
			if (i === total_entities_type) {
				callback();
			}
		};

		for (let id in this.m_entities_types) {
			if (this.m_entities_types.hasOwnProperty(id)) {
				let entity_type = this.m_entities_types[id];
				entity_type.load(on_load);
			}
		}
	};

	adm_form_cognitive_services_section_entities.prototype.on_open_popup = function(item, current_popup)
	{
		this.is_new = item ? false: true;

		if (API_VERSION === "2022-10-01-preview") {
			this.__on_open_popup_entity_builtin(item);
		}
		else if (!current_popup || current_popup === POPUP_ID_ENTITIES) {
			this.__on_open_popup_entity(item);
		}
		else if (current_popup === POPUP_ID_PREBUILT_ENTITIES) {
			this.__on_open_popup_entity_builtin();
		}
		else {
			throw("adm_form_cognitive_services_section_entities.prototype.on_open_popup - Unrecognized popup " + current_popup);
		}
	};

	adm_form_cognitive_services_section_entities.prototype.__on_open_popup_entity = function(item)
	{
		const self = this;

		this.is_new = item ? false: true;
		this.current_item = item;

		this.__load_entity_popup_info(function() {
			if (self.is_new) {
				// nuevo
				self.clean_up_controls();
				return;
			}
			// editando
			const $name = $("#" + self.id_new_entity_popup + "_name");
			const $type = $("#" + self.id_new_entity_popup + "_type");
			$name.val(item.name);
			$type.val(item.typeId).trigger("change");
			self.frm.readonly_control($type, true);

			let entity_type_def = self.__get_entity_type_def(item.typeId);
			if (entity_type_def.children_info.instance) {
				entity_type_def.children_info.instance.set(item);
			}
			// MACHINE LEARNED
			if (entity_type_def.id === 1) {
				if (API_VERSION !== "2.0") {
					self.form_paginator.update_popup_info({
						caption_btn_update: "<%=_T('Update Name')%>"
					});
				}
			}
			$name.focus();
		});
	};

	adm_form_cognitive_services_section_entities.prototype.__on_open_popup_entity_builtin = function(item)
	{
		const self = this;

		let app_id = this.frm.get_entity_id();
		if (!app_id) {
			return;
		}
		this.form_paginator.show_popup_waiting("<%=_T('Loading Prebuilt Entities.')%>");

		this.clean_up_controls(); // Limpio los controles!!
		const params = {appId: app_id};
		const luisApi = inConcert.app.getLuisAPI();

		this.m_prebuilt_entities = [];
		luisApi.loadAvailablePrebuiltEntityList(params,
			function(success, prebuilt_entities) {
				self.form_paginator.hide_popup_waiting();
				if (!success) {
					self.m_prebuilt_entities = [];
					let err_message = "<%=_T('Error Retrieving Prebuilt Entities.')%>";
					self.form_paginator.show_popup_msg(err_message, "Error", ERROR_TIMEOUT_MS);
					return;
				}
				self.m_prebuilt_entities = prebuilt_entities;

				let type = "2";

				if (API_VERSION === "2022-10-01-preview") {
					type = "combineComponents";
				}

				let entity_type_def = self.__get_entity_type_def(type); // PREBUILT

				if (entity_type_def.children_info.instance) {
					entity_type_def.children_info.instance.show();
				}

				if (item) {
					const $name = $("#" + self.id_new_entity_popup + "_name");
					const $type = $("#" + self.id_new_entity_popup + "_type");
					$name.val(item.name);
					$type.val(item.typeId).trigger("change");
					self.frm.readonly_control($type, true);
					entity_type_def.children_info.instance.set(item);
				}

			});
	};

	adm_form_cognitive_services_section_entities.prototype.on_closed_popup = function(/*current_popup*/)
	{
		if (this.current_item) {
			let entity_type = this.m_entities_types[this.current_item.typeId];
			if (entity_type && entity_type.can_update_memory_info_on_closed_poup()) {
				// Si es la version 3.0 y es MACHINE LEARNED, cuando cierra el popup sin presionar el boton "UPDATE",
				//  o sea, ha presionado el boton "CLOSE" debe actualizar en memoria la info del row de la grilla, con los items del TREE,
				//  ya que este se actualiza de forma automatica (server side), en la medida que se van actualizando los items del TREE.
				// SOLO para el caso de el UPDATE de una endidad de este tipo, ya que el CREATE de una entidad de este tipo, se almacena todo el TREE de una.
				
				if (entity_type.children_info.instance) {
					if (API_VERSION === "2022-10-01-preview") {
						let regex = entity_type.children_info.instance.get_regular_exp();
						let prebuilts = entity_type.children_info.instance.get_prebuilt();
						let list = entity_type.children_info.instance.get_list();

						this.current_item.regex = regex;
						this.current_item.prebuilts = prebuilts;
						this.current_item.list = list;

						this.form_paginator.update_row(this.current_item);

					} else {
						let children = entity_type.children_info.instance.get();
						this.current_item.children = children;
						this.form_paginator.update_row(this.current_item);
					}
				}
				
			}
		}

		for (let id in this.m_entities_types) {
			if (this.m_entities_types.hasOwnProperty(id)) {
				let entity_type = this.m_entities_types[id];
				if (entity_type.children_info && entity_type.children_info.instance) {
					entity_type.children_info.instance.dispose();
				}
			}
		}
		this.is_new = null;
		this.current_item = null;
	};

	adm_form_cognitive_services_section_entities.prototype.hide_all_errors = function()
	{
		this.form_paginator.hide_popup_msg();
	};

	adm_form_cognitive_services_section_entities.prototype.show_ctrl_err = function(err_info)
	{
		if (!err_info) {
			return;
		}
		const $control = err_info.$ctrl;
		if ($control) {
			const error_id = $control.attr("id") + "_validation";
			const $error_indicator = $("#" + error_id);
			if ($error_indicator) {
				$error_indicator.show();
			}
			$control.focus();
		}
		this.form_paginator.show_popup_msg(err_info.msg, "Error", ERROR_TIMEOUT_MS);
	};

	adm_form_cognitive_services_section_entities.prototype.validate_info_to_save = function(current_popup)
	{
		const self = this;

		this.hide_all_errors();
		let response = {success: false, $ctrl: null, item: null, msg: "<%=_T('Unable to save Entity.')%>"};
		let item = {
			//children: [] /* Solo para los casos (MACHINE LEARNED, COMPOSITE, HIERARCHICAL, PREBUILT). Lo dejo aca como referencia, ya que el mismo se agrega mas abajo /*
		};

		var validate_childrens = function() {
			// Cargo los childrens de las entidades (COMPOSITE, HIERARCHICAL, PREBUILT)
			const entity_type_def = self.__get_entity_type_def(item.typeId);
			const instance = entity_type_def.children_info.instance;
			if (instance) {

				if (API_VERSION === "2022-10-01-preview") {
					
					item.regex = instance.get_regular_exp();
					item.prebuilts = instance.get_prebuilt();
					item.list = instance.get_list();

				} else {
					item.children = instance.get();
					// Si es (COMPOSITE, HIERARCHICAL, PREBUILT), valida que al menos tenga 1 hijo
					if (instance.is_required_childrens()) {
						if ((!$.isArray(item.children)|| item.children.length < 1) && API_VERSION !== "2022-10-01-preview") {
							response.msg = instance.get_string("validate_child_required_err");
							const $grid = instance.$get_grid();
							response.$ctrl = $grid;
							self.show_ctrl_err(response);
							return false;
						}
					}
				}
			}
			return true;
		};

		if (current_popup !== POPUP_ID_PREBUILT_ENTITIES) {
			const $name = $("#" + this.id_new_entity_popup + "_name");
			const $type = $("#" + this.id_new_entity_popup + "_type");

			item.name = $name.val();
			item.typeId = $type.val();

			let validation_result = inConcert.Utils.validate(item.name, "required");
			if ( !validation_result.success ) {
				// valida que el name tenga al menos un caracter
				response.msg = "<%=_T('Name is required.')%>";
				response.$ctrl = $name;
				this.show_ctrl_err(response);
				return response;
			}
			validation_result = inConcert.Utils.validate(item.name, "id");
			if (!validation_result.success) {
				// valida los caracteres permitidos
				response.msg = "<%=_T('Only letters, numbers, dashes and dots are allowed in the Name.')%>";
				response.$ctrl = $name;
				this.show_ctrl_err(response);
				return response;
			}
			// Cargo los children de las entidades (COMPOSITE, HIERARCHICAL)
			const success = validate_childrens();
			if (!success) {
				return response;
			}
		}
		else {
			item.typeId = 2; // PREBUILT
			
			if (API_VERSION === "2022-10-01-preview") {
				item.typeId = "combineComponents";
			}
			
			const success = validate_childrens();
			if (!success) {
				return response;
			}
		}

		response.success = true;
		response.msg = "";
		response.item = item;

		return response;
	};

	adm_form_cognitive_services_section_entities.prototype.clean_up_controls = function()
	{
		const $name = $("#" + this.id_new_entity_popup + "_name");
		const $type = $("#" + this.id_new_entity_popup + "_type");
		//this.frm.readonly_control($type, false);
		this.toggle_gui_ctrls_by_api_version();
		$name.val("").focus();
		$type.val(1).trigger("change"); // el 1 es MachineLearned
		// Recorro las grillas de children de los diferentes tipos (MACHINELEARNED, HIERARCHICAL, COMPOSITE, PREBUILT) y las dejo vacias.
		for (let id in this.m_entities_types) {
			if (this.m_entities_types.hasOwnProperty(id)) {
				let entity_type = this.m_entities_types[id];
				if (entity_type.children_info && entity_type.children_info.instance) {
					entity_type.children_info.instance.clean_up_list();
				}
			}
		}
	};

	adm_form_cognitive_services_section_entities.prototype.__update_app_version_info = function(callback = $.noop)
	{
		const app_section = this.frm.get_section_def("application");
		app_section.frm.update_app_version_info(undefined, callback);
	};

	adm_form_cognitive_services_section_entities.prototype.__get_entities_error = function(error, name, version)
	{
		const entityErrors = {
			"The model name \{.*?\} are reserved" : FORMAT("<%=_T('The model name { {0} } are reserved.')%>", name),
			"Cannot find an application version with the version ID" :FORMAT("<%=_T('Cannot find an application version with the version ID {0}.')%>", version || "0.1"),
			"The models: \{.*?\} already exist in the specified application version." :FORMAT("<%=_T('The models: { {0} } already exist in the specified application version.')%>", name),
		  };
		  for (const key in entityErrors) {
			if (entityErrors.hasOwnProperty(key)) {
				const regex = new RegExp(key);
				if (regex.test(error)) {
					return " " + entityErrors[key];
				}
			}
		  }
		  return error;
	};

	adm_form_cognitive_services_section_entities.prototype.save = function(callback, must_create_and_contiune, current_popup)
	{
		const self = this;
		callback = callback || $.noop;

		let response = this.validate_info_to_save(current_popup);
		if (!response.success) {
			callback(false);
			return;
		}

		let app_id = this.frm.get_entity_id();
		if (!app_id) {
			callback(false);
			return;
		}
		let item = response.item;
		const luisApi = inConcert.app.getLuisAPI();
		let params = {
			appId   : app_id,
			name    : item.name
		};

		if (API_VERSION === "2022-10-01-preview") {
			params.prebuilts = item.prebuilts;
			params.regex = item.regex;
			params.list = item.list;
		} else {
			params.children = item.children;
		}

		this.disabled_all_sub_menus(true);

		const entity_type_def = this.__get_entity_type_def(item.typeId);
		const api_luis_methods = entity_type_def.api_luis_methods;
		const $full_screen_waiting = $("#genericWaiting");

		let is_prebuilt = false;
		if (current_popup === 1) {
			this.is_new = true;
			is_prebuilt = true;
		}

		params.is_new = this.is_new || false;
		
		if (this.is_new || API_VERSION === "2022-10-01-preview") {
			let waiting_msg, success_msg, error_msg;
			if (is_prebuilt) {
				waiting_msg = "<%=_T('Assigning Prebuilt Entities.')%>";
				success_msg = "<%=_T('Prebuilt Entities was successfully added.')%>";
				error_msg   = "<%=_T('Error Assigning Prebuilt Entities.')%>";
			}
			else {
				waiting_msg = FORMAT("<%=_T('Creating Entities {0}.')%>", item.name);
				success_msg = FORMAT("<%=_T('Entity {0} was successfully created.')%>", item.name);
				error_msg   = FORMAT("<%=_T('Error creating Entity {0}.')%>", item.name);
			}

			this.frm.dashboard.show_waiting(waiting_msg, $full_screen_waiting);
			/*
				Metodos CREATE
					MachineLearned: "createAppEntityMachineLearned" (v 3.0 PREVIEW)
					Hierarchical  : "createAppEntityHierarchical"
					Composite     : "createAppEntityComposite"
				Metodo ADD
					Prebuilt      : "addAppEntityPrebuilt"
			*/
			luisApi[api_luis_methods.create](params,
				function(success, result)
				{
					let msg_response = {msg: "", type: ""};
					if (success) {
						msg_response.msg = success_msg;
						msg_response.type = "Info";
						let must_add_many = false;

						if (is_prebuilt) {
							must_add_many = true;
							let items = result;
							callback(true, items, msg_response, must_add_many);
						}
						else {
							if (result.entity) {
								item = result.entity;
							}
							else {
								item.id = result.entityId;
							}
							callback(true, item, msg_response, must_add_many);
						}

						self.clean_up_controls();
						self.__update_app_version_info(function() {
							self.frm.dashboard.hide_waiting($full_screen_waiting);
							self.frm.enable_all_sections();
						});
					}
					else {
						self.frm.dashboard.hide_waiting($full_screen_waiting);
						self.frm.enable_all_sections();
						let server_err_msg = luisApi.getCognitiveErrorMsg(result);
						msg_response.msg = error_msg + self.__get_entities_error(server_err_msg, item.name, item.versionId);
						msg_response.type = "Error";
						callback(false, null, msg_response);
					}
				}
			);
		}
		else {
			let waiting_msg = FORMAT("<%=_T('Updating Entities {0}.')%>", item.name);
			this.frm.dashboard.show_waiting(waiting_msg, $full_screen_waiting);

			params.entityId = this.current_item.id;
			/*
				Metodos RENAME
					MachineLearned: "renameAppEntityMachineLearned"
					Hierarchical  : "renameAppEntityHierarchical"
					Composite     : "renameAppEntityComposite"
			*/
			luisApi[api_luis_methods.rename](params,
				function(success, result)
				{
					let msg_response = {msg: "", type: ""};
					self.frm.dashboard.hide_waiting($full_screen_waiting);
					self.frm.enable_all_sections();
					if (success) {
						let old_entity_name = self.current_item.name;
						let children = inConcert.Utils.cloneArray(item.children);
						$.extend(true, self.current_item, item);
						self.current_item.children = children;
						let cloned_entity = inConcert.Utils.cloneObject(self.current_item);
						msg_response.msg = FORMAT("<%=_T('Entity {0} was successfully updated.')%>", item.name);
						msg_response.type = "Info";
						callback(true, self.current_item, msg_response);
						self.__update_related_childs_in_grid(cloned_entity, old_entity_name);
						self.clean_up_controls();
					}
					else {
						let server_err_msg = luisApi.getCognitiveErrorMsg(result);
						msg_response.msg = FORMAT("<%=_T('Error updating Entity {0}.{1}')%>", item.name, server_err_msg);
						msg_response.type = "Error";
						callback(false, null, msg_response);
					}
				}
			);
		}
	};

	adm_form_cognitive_services_section_entities.prototype.get_current_entities = function()
	{
		return this.form_paginator.get_items();
	};

	adm_form_cognitive_services_section_entities.prototype.get_current_prebuilt_entities = function()
	{
		return this.m_prebuilt_entities;
	};

	/*
		Esta funcion se encarga de actualizar en memoria los nombres de los children de cada item de la grilla principal de entities.
		
		Ej:
			Si estoy modificando el Nombre de una Entity SIMLE o HIERARCHICAL, debe recorrer las rows de la grilla "COMPOSITE" y buscar si alguno de sus hijos era la entidad que se esta modificando.
	*/
	adm_form_cognitive_services_section_entities.prototype.__update_related_childs_in_grid = function(updated_entity, old_entity_name)
	{
		const self = this;
		if (!updated_entity) {
			return;
		}
		old_entity_name = old_entity_name.toLowerCase();
		const updated_entity_type_id = updated_entity.typeId.toString();
		let current_entities = this.form_paginator.get_items();

		current_entities.forEach((entity, i) => {
			const entity_type_id = entity.typeId.toString();
			if (entity_type_id === "4") { // En las COMPOSITE
				if (entity.children) {
					entity.children.forEach((child, j) => {
						// MACHINELEARNED O HIERARCHICAL PADRE
						if (child.name.toLowerCase() === old_entity_name) {
							current_entities[i].children[j].name = updated_entity.name;
							self.form_paginator.update_row(current_entities[i]);
						}
						else {
							// HIERARCHICAL HIJO (PADRE::HIJO)
							if (updated_entity_type_id === "3") {
								const separator = "::";
								const separator_pos = child.name.indexOf(separator);
								if (separator_pos > -1) {
									let parent_name = child.name.substring(0, separator_pos);
									parent_name = parent_name.toLowerCase();
									if (parent_name === old_entity_name) {
										let new_child_name = updated_entity.name + child.name.substring(separator_pos);
										current_entities[i].children[j].name = new_child_name;
										self.form_paginator.update_row(current_entities[i]);
									}
									return false;
								}
							}
						}
					});
				}
			}
		});
	};

	//---------------------------------------------------------------------
	// class children list
	//

	const SANITIZE_ID = function(id) { return id.replace("#", ""); };

	function children_list (params)
	{
		const self = this;

		this.owner = params.owner;
		this.prefix_id = params.prefix_id;

		this.children_section         = this.prefix_id + "_container";
		this.children_new             = this.prefix_id + "_list_new_child";
		this.children_new_prebuilt             = this.prefix_id + "_list_new_child_prebuilt";
		this.children_new_regular_exp             = this.prefix_id + "_list_new_child_regular_exp";
		this.children_new_list             = this.prefix_id + "_list_new_child_list";
		this.children_popup           = this.prefix_id + "_list_popup_children";
		this.children_popup_prebuilt       = this.prefix_id + "_list_popup_children_prebuilt";
		this.children_popup_regular_exp    = this.prefix_id + "_list_popup_children_regular_exp";
		this.children_popup_list           = this.prefix_id + "_list_popup_children_list";
		this.children_popup_title     = this.prefix_id + "_list_popup_title";
		this.children_create          = this.prefix_id + "_list_child_create";
		this.children_create_continue = this.prefix_id + "_list_child_create_and_continue";
		this.children_create_continue_prebuilt = this.prefix_id + "_list_child_create_and_continue_prebuilt";
		this.children_create_continue_regular_exp = this.prefix_id + "_list_child_create_and_continue_regular_exp";
		this.children_create_continue_list = this.prefix_id + "_list_child_create_and_continue_list";
		this.children_close           = this.prefix_id + "_list_child_close";
		this.children_message         = this.prefix_id + "_list_popup_message";
		this.children_message_prebuilt         = this.prefix_id + "_list_popup_message_prebuilt";
		this.children_message_regular_exp         = this.prefix_id + "_list_popup_message_regular_exp";
		this.children_message_list         = this.prefix_id + "_list_popup_message_list";
		this.children_list            = this.prefix_id + "_list_children";
		this.children_list_prebuilt        = this.prefix_id + "_list_children_prebuilt";
		this.children_list_regular_exp     = this.prefix_id + "_list_children_regular_exp";
		this.children_list_list            = this.prefix_id + "_list_children_list";
		this.children_name            = this.prefix_id + "_list_popup_name";
		this.children_desc            = this.children_name + "_desc";
		this.children_example         = this.children_name + "_example";

		this.children_name_prebuilt            = this.prefix_id + "_list_popup_name_prebuilt";
		this.children_desc_prebuilt            = this.children_name + "_desc_prebuilt";
		this.children_example_prebuilt         = this.children_name + "_example_prebuilt";

		this.children_regexkey_regular_exp            = this.prefix_id + "_list_popup_name_regular_exp";
		this.children_language_regular_exp            = this.children_name + "_language_regular_exp";
		this.children_pattern_regular_exp         = this.children_name + "_pattern_regular_exp";

		this.children_value_list            = this.prefix_id + "_list_popup_name_list";
		this.children_language_list            = this.children_name + "_language_list";

		this.strings = {
			popup_title                    : "<%=_T('New Children')%>",
			new_item                       : "<%=_T('New Children')%>",
			list_title                     : "<%=_T('Children')%>",
			btn_create                     : "<%=_T('Create')%>",
			btn_create_continue            : "<%=_T('Create & Continue')%>",
			btn_close                      : "<%=_T('Close')%>",
			err_not_permission_to_create   : "<%=_T('Unable to create Children: You dont have permissions for this action.')%>",
			err_not_permission_to_delete   : "<%=_T('Unable to delete Children: You dont have permissions for this action.')%>",
			delete_question                : "<%=_T('Do you want to delete Child ({0}) ?')%>",
			validate_name_required_err     : "<%=_T('Please enter the child name.')%>",
			validate_name_already_exist_err: "<%=_T('Already exist child name {0}.')%>",
			validate_value_required_err     : "<%=_T('Please enter the value.')%>",
			validate_value_already_exist_err: "<%=_T('Already exist child value {0}.')%>",
			validate_child_required_err    : "<%=_T('Must include at least one child.')%>",
			validate_language_required_err     : "<%=_T('Please select the Combine Components Language.')%>",
			validate_pattern_required_err     : "<%=_T('Must include at least one Combine Components Pattern.')%>",
			validate_regexkey_already_exist_err: "<%=_T('Already exist Combine Components Value {0}.')%>",
			validate_regexkey_required_err    : "<%=_T('Must include at least one Combine Components Value.')%>"
		};

		if (params.strings) {
			for (let string_id in this.strings) {
				if (this.strings.hasOwnProperty(string_id)) {
					this.strings[string_id] = params.strings[string_id] || this.strings[string_id];
				}
			}
		}

		const popup_id = SANITIZE_ID(this.children_popup);
		this.popup_instance = new inConcert.PopupWindow({
			ownerId         : popup_id, // container id
			eventName       : popup_id, // evento click personalizado
			closeOnFocusLost: params.closeOnFocusLost,
			callbacks: {
				onClose: function(){
					self.hide_popup();
				}
			}
		});
	}

	children_list.prototype.$get_grid = function() {
		return $(this.children_list);
	};

	children_list.prototype.$get_grid_prebuilt = function() {
		return $(this.children_list_prebuilt);
	};

	children_list.prototype.$get_grid_regular_exp = function() {
		return $(this.children_list_regular_exp);
	};

	children_list.prototype.$get_grid_list = function() {
		return $(this.children_list_list);
	};

	children_list.prototype.get_string = function(name) {
		let string = this.strings[name] || "<%=_T('Error')%>";
		return string;
	};

	children_list.prototype.render = function()
	{
		if (API_VERSION === "2022-10-01-preview") {

			const sanitized_id_param_new_prebuilt = SANITIZE_ID(this.children_new_prebuilt);
			const sanitized_id_param_new_regular_exp = SANITIZE_ID(this.children_new_regular_exp);
			const sanitized_id_param_new_list = SANITIZE_ID(this.children_new_list);
			
			const $html = $("".concat(
			
				"<div id='", SANITIZE_ID(this.children_popup), "' style='display:none;'>",

					"<i id='", sanitized_id_param_new_prebuilt, "_validation' class='formFieldValidation formFieldError' hidden></i>",
					"<div id='", SANITIZE_ID(this.children_popup_prebuilt), "' class='row'>",
						"<div class='col-12'>",
							this.get_popup_ctrls_markup_prebuilt(),
							"<div class='row custom-form-buttons-container-2'>",
								"<div class='col-12'>",
									"<button id='", SANITIZE_ID(this.children_create_continue_prebuilt), "' class='pageButtonGeneric buttonColorWhite'>",
										this.strings.btn_create_continue,
									"</button>",
								"</div>",
								"<div class='col-12'>",
									"<div id='", SANITIZE_ID(this.children_message_prebuilt), "' class='genericFormMessagesContainer messageError' hidden></div>",
								"</div>",
							"</div>",
						"</div>",
					"</div>",
					"<div class='row'>",
						"<i id='", SANITIZE_ID(this.children_list_prebuilt), "_validation' class='formFieldValidation formFieldError' hidden></i>",
						"<div class='col-12'>",
							"<ul id='", SANITIZE_ID(this.children_list_prebuilt), "' class='formList formListAsGrid mt-0'>",
								"<li class='formListTitle'>",
									"<label head='true' class='formListTitleText nameColumn'><%=_T('Entity')%></label>",
								"</li>",
							"</ul>",
						"</div>",
					"</div>",

					"<i id='", sanitized_id_param_new_regular_exp, "_validation' class='formFieldValidation formFieldError' hidden></i>",					
					"<div id='", SANITIZE_ID(this.children_popup_regular_exp), "' class='row'>",
						"<div class='col-12'>",
							this.get_popup_ctrls_markup_regular_exp(),
							"<div class='row custom-form-buttons-container-2'>",
								"<div class='col-12'>",
									"<button id='", SANITIZE_ID(this.children_create_continue_regular_exp), "' class='pageButtonGeneric buttonColorWhite'>",
										this.strings.btn_create_continue,
									"</button>",
								"</div>",
								"<div class='col-12'>",
									"<div id='", SANITIZE_ID(this.children_message_regular_exp), "' class='genericFormMessagesContainer messageError' hidden></div>",
								"</div>",
							"</div>",
						"</div>",
					"</div>",
					"<div class='row'>",
						"<i id='", SANITIZE_ID(this.children_list_regular_exp), "_validation' class='formFieldValidation formFieldError' hidden></i>",
						"<div class='col-12'>",
							"<ul id='", SANITIZE_ID(this.children_list_regular_exp), "' class='formList formListAsGrid mt-0'>",
								"<li class='formListTitle'>",
									"<label head='true' class='formListTitleText nameColumn'><%=_T('Language')%></label>",
									"<label head='true' class='formListTitleText nameColumn'><%=_T('Value')%></label>",
									"<label head='true' class='formListTitleText nameColumn'><%=_T('Pattern')%></label>",
								"</li>",
							"</ul>",
						"</div>",
					"</div>",

					"<i id='", sanitized_id_param_new_list, "_validation' class='formFieldValidation formFieldError' hidden></i>",
					"<div id='", SANITIZE_ID(this.children_popup_list), "' class='row'>",
						"<div class='col-12'>",
							this.get_popup_ctrls_markup_list(),
							"<div class='row custom-form-buttons-container-2'>",
								"<div class='col-12'>",
									"<button id='", SANITIZE_ID(this.children_create_continue_list), "' class='pageButtonGeneric buttonColorWhite'>",
										this.strings.btn_create_continue,
									"</button>",
								"</div>",
								"<div class='col-12'>",
									"<div id='", SANITIZE_ID(this.children_message_list), "' class='genericFormMessagesContainer messageError' hidden></div>",
								"</div>",
							"</div>",
						"</div>",
					"</div>",
					"<div class='row'>",
						"<i id='", SANITIZE_ID(this.children_list_list), "_validation' class='formFieldValidation formFieldError' hidden></i>",
						"<div class='col-12'>",
							"<ul id='", SANITIZE_ID(this.children_list_list), "' class='formList formListAsGrid mt-0'>",
								"<li class='formListTitle'>",
									"<label head='true' class='formListTitleText nameColumn'><%=_T('Language')%></label>",
									"<label head='true' class='formListTitleText nameColumn'><%=_T('Value')%></label>",
								"</li>",
							"</ul>",
						"</div>",
					"</div>",

				"</div>")
			);

			const $section = $(this.children_section);
			$section.html($html);
		} else {

			const sanitized_id_param_new = SANITIZE_ID(this.children_new);

			const $html = $("".concat(
				"<i id='", sanitized_id_param_new, "_validation' class='formFieldValidation formFieldError' hidden></i>",
				"<div class='row'>",
					"<div class='col-12'>",
						"<button id='", sanitized_id_param_new, "' class='pageButtonGeneric buttonColorWhite float-left mbGeneric'>",
							this.strings.new_item,
						"</button>",
					"</div>",
				"</div>",
				"<div id='", SANITIZE_ID(this.children_popup), "' class='row' style='display:none;'>",
					"<div class='col-12'>",
						this.get_popup_ctrls_markup(),
						"<div class='row custom-form-buttons-container-2'>",
							"<div class='col-12'>",
								"<button id='", SANITIZE_ID(this.children_create), "' class='pageButtonGeneric buttonColorWhite'>",
									this.strings.btn_create,
								"</button>",
								"<button id='", SANITIZE_ID(this.children_create_continue), "' class='pageButtonGeneric buttonColorWhite'>",
									this.strings.btn_create_continue,
								"</button>",
								"<button id='", SANITIZE_ID(this.children_close), "' class='pageButtonGeneric buttonColorWhite'>",
									this.strings.btn_close,
								"</button>",
							"</div>",
							"<div class='col-12'>",
								"<div id='", SANITIZE_ID(this.children_message), "' class='genericFormMessagesContainer messageError' hidden></div>",
							"</div>",
						"</div>",
					"</div>",
				"</div>",
				"<div class='row'>",
					"<i id='", SANITIZE_ID(this.children_list), "_validation' class='formFieldValidation formFieldError' hidden></i>",
					"<div class='col-12'>",
						"<ul id='", SANITIZE_ID(this.children_list), "' class='formList formListAsGrid mt-0'>",
							"<li class='formListTitle'>",
								"<label head='true' class='formListTitleText nameColumn'>", this.strings.list_title, "</label>",
							"</li>",
						"</ul>",
					"</div>",
				"</div>")
			);

			const $section = $(this.children_section);
			$section.html($html);
		}
	};

	children_list.prototype.initialize = function() {
		this.render();
		this.set_events();
		this.on_ready();
	};

	children_list.prototype.on_ready = function() {

		if (API_VERSION === "2022-10-01-preview") {
			const $children_new_prebuilt = $(this.children_new_prebuilt);
			const $children_new_regular_exp = $(this.children_new_regular_exp);
			const $children_new_list = $(this.children_new_list);
			if (this.is_allowed_create()) {
				$children_new_prebuilt.show();
				$children_new_regular_exp.show();
				$children_new_list.show();
			}
			else {
				$children_new_prebuilt.hide();
				$children_new_regular_exp.hide();
				$children_new_list.hide();
			}
		} else {
			const $children_new = $(this.children_new);
			if (this.is_allowed_create()) {
				$children_new.show();
			}
			else {
				$children_new.hide();
			}
		}
	};

	children_list.prototype.get_popup_ctrls_markup = function() {
		return "";
	};

	children_list.prototype.dispose = function()
	{
		// TODO: IMPLEMENTAR
		// if (this.popup_instance) {
			// if ( this.popup_instance.isVisible() ) {
				// this.popup_instance.hide();
			// }
			// this.popup_instance = null;
		// }
	};

	children_list.prototype.get_popup_ctrls_markup = function() {
		return "";
	};

	children_list.prototype.is_required_childrens = function()
	{
		return true;
	};

	children_list.prototype.get = function()
	{
		let children = [];
		// carga la lista de children
		$(this.children_list).find("li[item_data]").each(function(_, item_data) {
			const $item_data = $(item_data);
			const $label = $item_data.find("label[child_name]");
			const child = {
				name: $label.text()
			};
			children.push(child);
		});
		return children;
	};

	children_list.prototype.get_prebuilt = function()
	{
		let prebuilts = [];
		// carga la lista de children
		$(this.children_list_prebuilt).find("li[item_data]").each(function(_, item_data) {
			const $item_data = $(item_data);
			const $label = $item_data.find("label[child_category]");
			const child = {
				category: $label.text()
			};
			prebuilts.push(child);
		});
		return prebuilts;
	};

	children_list.prototype.get_regular_exp = function()
	{
		let regex = [];
		// carga la lista de children
		$(this.children_list_regular_exp).find("li[item_data]").each(function(_, item_data) {
			const $item_data = $(item_data);
			const $language = $item_data.find("label[child_language]");
			const $regexkey = $item_data.find("label[child_regexkey]");
			const $pattern = $item_data.find("label[child_pattern]");
			const child = {
				language: $language.text(),
				regexKey: $regexkey.text(),
				regexPattern: $pattern.text()
			};
			regex.push(child);
		});
		return regex;
	};

	children_list.prototype.get_list = function()
	{
		let list = [];
		// carga la lista de children
		$(this.children_list_list).find("li[item_data]").each(function(_, item_data) {
			const $item_data = $(item_data);
			const $language = $item_data.find("label[child_language]");
			const $value = $item_data.find("label[child_value]");
			const child = {
				language: $language.text(),
				value: $value.text()
			};
			list.push(child);
		});
		return list;
	};

	children_list.prototype.set = function(info)
	{
		const self = this;
			
		if (API_VERSION === "2022-10-01-preview") {
			
			this.clean_up_list_prebuilt();
			this.clean_up_list_regular_exp();
			this.clean_up_list_list();

			if (info && $.isArray(info.prebuilts)) {
				$.each(info.prebuilts, function(_, child) {
					self.add_row_prebuilt(child);
				});
			}

			if (info && info.regex && $.isArray(info.regex.expressions)) {
				$.each(info.regex.expressions, function(_, child) {
					self.add_row_regular_exp(child);
				});
			}

			if (info && info.list && $.isArray(info.list.sublists)) {
				$.each(info.list.sublists, function(_, child1) {

                    let language = "";

                    $.each(child1.synonyms, function(_, child2) {

                        language = child2.language;
    
                        $.each(child2.values, function(_, child3) {
                            self.add_row_list({
                                id: child3,
                                value: child3,
                                language: language
                            });
                        });
                    });
				});
			}

		} else {

			this.clean_up_list();

			if (!info || !$.isArray(info.children)) {
				return;
			}
			$.each(info.children, function(_, child) {
				self.add_row(child);
			});
		}
	};

	children_list.prototype.load = function(callback = $.noop)
	{
		callback();
	};

	children_list.prototype.show_popup = function(on_shown = $.noop)
	{
		const self = this;

		this.hide_errors();

		if ( !this.owner.has_permission_to_section() || !this.owner.has_permission_to_save() ) {
			let message = this.strings.err_not_permission_to_create;
			this.owner.form_paginator.show_popup_msg(message, "Error", ERROR_TIMEOUT_MS);
			on_shown(false);
			return;
		}

		// Cada vez que visualizo el popup de add en la grilla, realizo un load, para que actualice los controles (Ej: combo con las entidades hijas para los COMPOSITE)
		this.load(function() {
			self.popup_instance.show();
			// disable actions popup parameters (new, delete, edit)
			self.disable_actions(true);
			//var type;
			if (!self.$row_editing) {
			// si presione el boton para crear uno nuevo
				$(self.children_popup_title).text(self.strings.popup_title);
				$(self.children_create).val(self.strings.btn_create);
				$(self.children_create_continue).show();
				$(self.children_name).focus().val("");
			}
			on_shown(true);
		});
	};

	children_list.prototype.clean_up_list = function()
	{
		$(this.children_list).find("li[item_data]").remove();
	};

	children_list.prototype.clean_up_list_prebuilt = function()
	{
		$(this.children_list_prebuilt).find("li[item_data]").remove();
	};

	children_list.prototype.clean_up_list_regular_exp = function()
	{
		$(this.children_list_regular_exp).find("li[item_data]").remove();
	};

	children_list.prototype.clean_up_list_list = function()
	{
		$(this.children_list_list).find("li[item_data]").remove();
	};

	children_list.prototype.hide_popup = function()
	{
		if (this.popup_instance) {
			this.popup_instance.hide();
		}
		// enable actions popup entities (new, delete)
		this.disable_actions(false);
	};

	children_list.prototype.can_add_row = function(/*child*/)
	{
		return true;
	};

	children_list.prototype.add_row = function(child, apply_effect)
	{
		if (typeof child !== "object") {
			return;
		}
		if (!this.can_add_row(child)) {
			return;
		}

		const item_value = child.name.trim().toLowerCase();
		const item_id = child.id || "";
		var $make_new_row = function()
		{
			var display = "";
			if (apply_effect) {
				display = "style='display:none;'";
			}
			return $(
				"<li item_data='true' class='formListItem' " + display + ">" +
					"<label child_name='true' class='formItemText nameColumn' item_value='" + item_value + "'" + " item_id='" + item_id + "'>" + child.name + "</label>" +
					"<button action_del='true' class='buttonColorGray gridActionButton delete last'>" +
						"<i class='buttons-icons-delete'></i>" +
						"<%=_T('Delete')%>" +
					"</button>" +
				"</li>");
		};

		let added = false;
		const $list = $(this.children_list);
		let $new_row;
		$list.find("li[item_data]").each(function(_, row)
		{
			const $row = $(row);
			const current_item_value = $row.find("label[child_name]").attr("item_value");
			if (item_value < current_item_value) {
				$new_row = $make_new_row();
				$row.before($new_row);
				added = true;
				return false; // break
			}
		});
		if (!added) {
			$new_row = $make_new_row();
			$list.append($new_row);
		}
		if (apply_effect && $new_row) {
			// aca realizo el fade in
			$new_row.fadeIn("slow");
		}
	};

	children_list.prototype.add_row_prebuilt = function(child, apply_effect)
	{
		if (typeof child !== "object") {
			return;
		}
		if (!this.can_add_row(child)) {
			return;
		}

		const item_value = child.category.trim().toLowerCase();
		const item_id = child.id || "";
		var $make_new_row = function()
		{
			var display = "";
			if (apply_effect) {
				display = "style='display:none;'";
			}
			return $(
				"<li item_data='true' class='formListItem' " + display + ">" +
					"<label child_category='true' class='formItemText nameColumn' item_value='" + item_value + "'" + " item_id='" + item_id + "'>" + child.category + "</label>" +
					"<button action_del_prebuilt='true' class='buttonColorGray gridActionButton delete last'>" +
						"<i class='buttons-icons-delete'></i>" +
						"<%=_T('Delete')%>" +
					"</button>" +
				"</li>");
		};

		let added = false;
		const $list = $(this.children_list_prebuilt);
		let $new_row;
		$list.find("li[item_data]").each(function(_, row)
		{
			const $row = $(row);
			const current_item_value = $row.find("label[child_category]").attr("item_value");
			if (item_value < current_item_value) {
				$new_row = $make_new_row();
				$row.before($new_row);
				added = true;
				return false; // break
			}
		});
		if (!added) {
			$new_row = $make_new_row();
			$list.append($new_row);
		}
		if (apply_effect && $new_row) {
			// aca realizo el fade in
			$new_row.fadeIn("slow");
		}
	};

	children_list.prototype.add_row_regular_exp = function(child, apply_effect)
	{
		if (typeof child !== "object") {
			return;
		}
		if (!this.can_add_row(child)) {
			return;
		}

		const item_regexKey = child.regexKey.trim().toLowerCase();
		const item_language = child.language.trim().toLowerCase();
		const item_pattern = child.regexPattern.trim().toLowerCase();

		const item_id = child.id || "";
		var $make_new_row = function()
		{
			var display = "";
			if (apply_effect) {
				display = "style='display:none;'";
			}
			return $(
				"<li item_data='true' class='formListItem' " + display + ">" +
					"<label child_language='true' class='formItemText nameColumn' item_value='" + item_language + "'" + " item_id='" + item_id + "'>" + child.language + "</label>" +
					"<label child_regexkey='true' class='formItemText nameColumn' item_value='" + item_regexKey + "'" + " item_id='" + item_id + "'>" + child.regexKey + "</label>" +
					"<label child_pattern='true' class='formItemText nameColumn' item_value='" + item_pattern + "'" + " item_id='" + item_id + "'>" + child.regexPattern + "</label>" +
					"<button action_del_regular_exp='true' class='buttonColorGray gridActionButton delete last'>" +
						"<i class='buttons-icons-delete'></i>" +
						"<%=_T('Delete')%>" +
					"</button>" +
				"</li>");
		};

		let added = false;
		const $list = $(this.children_list_regular_exp);
		let $new_row;
		$list.find("li[item_data]").each(function(_, row)
		{
			const $row = $(row);
			const current_item_value = $row.find("label[child_regexkey]").attr("item_value");
			if (item_regexKey < current_item_value) {
				$new_row = $make_new_row();
				$row.before($new_row);
				added = true;
				return false; // break
			}
		});
		if (!added) {
			$new_row = $make_new_row();
			$list.append($new_row);
		}
		if (apply_effect && $new_row) {
			// aca realizo el fade in
			$new_row.fadeIn("slow");
		}
	};

	children_list.prototype.add_row_list = function(child, apply_effect)
	{
		if (typeof child !== "object") {
			return;
		}
		if (!this.can_add_row(child)) {
			return;
		}

		const item_value = child.value.trim().toLowerCase();
		const item_language = child.language.trim().toLowerCase();
		const item_id = child.id || "";
		var $make_new_row = function()
		{
			var display = "";
			if (apply_effect) {
				display = "style='display:none;'";
			}
			return $(
				"<li item_data='true' class='formListItem' " + display + ">" +
					"<label child_language='true' class='formItemText nameColumn' item_value='" + item_language + "'" + " item_id='" + item_id + "'>" + child.language + "</label>" +
					"<label child_value='true' class='formItemText nameColumn' item_value='" + item_value + "'" + " item_id='" + item_id + "'>" + child.value + "</label>" +
					"<button action_del_list='true' class='buttonColorGray gridActionButton delete last'>" +
						"<i class='buttons-icons-delete'></i>" +
						"<%=_T('Delete')%>" +
					"</button>" +
				"</li>");
		};

		let added = false;
		const $list = $(this.children_list_list);
		let $new_row;
		$list.find("li[item_data]").each(function(_, row)
		{
			const $row = $(row);
			const current_item_value = $row.find("label[child_value]").attr("item_value");
			if (item_value < current_item_value) {
				$new_row = $make_new_row();
				$row.before($new_row);
				added = true;
				return false; // break
			}
		});
		if (!added) {
			$new_row = $make_new_row();
			$list.append($new_row);
		}
		if (apply_effect && $new_row) {
			// aca realizo el fade in
			$new_row.fadeIn("slow");
		}
	};

	children_list.prototype.hide_errors = function()
	{
		if (API_VERSION === "2022-10-01-preview") {
			$(this.children_message_prebuilt).hide();
			$(this.children_message_regular_exp).hide();
			$(this.children_message_list).hide();
		} else {
			$(this.children_message_prebuilt).hide();
			$(this.children_message_regular_exp).hide();
			$(this.children_message_list).hide();
		}

		$(this.children_section).find(".formFieldError").hide();
	};

	children_list.prototype.already_exist = function(item_value)
	{
		item_value = (item_value || "").trim().toLowerCase();
		const $grid = this.$get_grid();
		const current_child = $grid.find("li[item_data] label[item_value='" + item_value + "']");
		const already_exist = current_child.length > 0;
		return already_exist;
	};

	children_list.prototype.already_exist_prebuild = function(item_value)
	{
		item_value = (item_value || "").trim().toLowerCase();
		const $grid = this.$get_grid_prebuilt();
		const current_child = $grid.find("li[item_data] label[item_value='" + item_value + "']");
		const already_exist = current_child.length > 0;
		return already_exist;
	};

	children_list.prototype.already_exist_regular_exp = function(item_value)
	{
		item_value = (item_value || "").trim().toLowerCase();
		const $grid = this.$get_grid_regular_exp();
		const current_child = $grid.find("li[item_data] label[item_value='" + item_value + "']");
		const already_exist = current_child.length > 0;
		return already_exist;
	};

	children_list.prototype.already_exist_list = function(item_value)
	{
		item_value = (item_value || "").trim().toLowerCase();
		const $grid = this.$get_grid_list();
		const current_child = $grid.find("li[item_data] label[item_value='" + item_value + "']");
		const already_exist = current_child.length > 0;
		return already_exist;
	};

	children_list.prototype.show_error = function(message, $control)
	{
		const self = this;
		$(this.children_message).text(message).show();
		if ($control) {
			const validation_id = $control.attr("id") + "_validation";
			$("#" + validation_id).show();
			$control.focus();
		}
		window.setTimeout(function() {
			self.hide_errors();
		}, ERROR_TIMEOUT_MS);
	};

	children_list.prototype.show_error_prebuilt = function(message, $control)
	{
		const self = this;
		$(this.children_message_prebuilt).text(message).show();
		if ($control) {
			const validation_id = $control.attr("id") + "_validation";
			$("#" + validation_id).show();
			$control.focus();
		}
		window.setTimeout(function() {
			self.hide_errors();
		}, ERROR_TIMEOUT_MS);
	};

	children_list.prototype.show_error_regular_exp = function(message, $control)
	{
		const self = this;
		$(this.children_message_regular_exp).text(message).show();
		if ($control) {
			const validation_id = $control.attr("id") + "_validation";
			$("#" + validation_id).show();
			$control.focus();
		}
		window.setTimeout(function() {
			self.hide_errors();
		}, ERROR_TIMEOUT_MS);
	};

	children_list.prototype.show_error_list = function(message, $control)
	{
		const self = this;
		$(this.children_message_list).text(message).show();
		if ($control) {
			const validation_id = $control.attr("id") + "_validation";
			$("#" + validation_id).show();
			$control.focus();
		}
		window.setTimeout(function() {
			self.hide_errors();
		}, ERROR_TIMEOUT_MS);
	};

	children_list.prototype.validate = function(child)
	{
		const self = this;

		let message;
		const $name = $(this.children_name);
		let result = this.is_valid_data(child.name, "required");
		if (!result) {
			message = this.strings.validate_name_required_err;
			this.show_error(message, $name);
			return false;
		}

		let name = $name.val();
		var already_exist = function()
		{
			const exist = self.already_exist(name);
			if (exist) {
				var message = FORMAT(self.strings.validate_name_already_exist_err, name);
				const $grid = self.$get_grid();
				self.show_error(message, $grid);
				$name.focus();
				return true;
			}
			return false;
		};

		// Valida que el valor ya no exista en la lista
		if (already_exist()) {
			return false;
		}

		// validation successfully
		return true;
	};

	children_list.prototype.validate_prebuilt = function(child)
	{
		const self = this;
		let message;
		const $category = $(this.children_name_prebuilt);

		let result = this.is_valid_data(child.category, "required");

		if (!result) {
			message = this.strings.validate_name_required_err;
			this.show_error_prebuilt(message, $category);
			return false;
		}

		let category = $category.val();

		var already_exist = function()
		{
			const exist = self.already_exist_prebuild(category);

			if (exist) {
				var message = FORMAT(self.strings.validate_name_already_exist_err, category);
				const $grid = self.$get_grid_prebuilt();
				self.show_error_prebuilt(message, $grid);
				$category.focus();
				return true;
			}
			return false;
		};

		// Valida que el valor ya no exista en la lista
		if (already_exist()) {
			return false;
		}

		// validation successfully
		return true;
	};

	children_list.prototype.validate_regular_exp = function(child)
	{
		const self = this;
		let message;
		const $language = $(this.children_language_regular_exp);
		const $regexKey = $(this.children_regexkey_regular_exp);
		const $pattern = $(this.children_pattern_regular_exp);
		
		let result1 = this.is_valid_data(child.language, "required");
		let result2 = this.is_valid_data(child.regexKey, "required");
		let result3 = this.is_valid_data(child.regexPattern, "required");
		
		if (!result1) {
			message = this.strings.validate_language_required_err;
			this.show_error_regular_exp(message, $language);
			return false;
		}

		if (!result2) {
			message = this.strings.validate_regexkey_required_err;
			this.show_error_regular_exp(message, $regexKey);
			return false;
		}

		if (!result3) {
			message = this.strings.validate_pattern_required_err;
			this.show_error_regular_exp(message, $pattern);
			return false;
		}

		let regexKey = $regexKey.val();
		var already_exist = function()
		{
			const exist = self.already_exist_regular_exp(regexKey);
			if (exist) {
				var message = FORMAT(self.strings.validate_regexkey_already_exist_err, regexKey);
				const $grid = self.$get_grid_regular_exp();
				self.show_error_regular_exp(message, $grid);
				$regexKey.focus();
				return true;
			}
			return false;
		};

		// Valida que el valor ya no exista en la lista
		if (already_exist()) {
			return false;
		}

		// validation successfully
		return true;
	};

	children_list.prototype.validate_list = function(child)
	{
		const self = this;
		let message;
		const $value = $(this.children_value_list);
		const $language = $(this.children_language_list);

		let result1 = this.is_valid_data(child.language, "required");
		let result2 = this.is_valid_data(child.value, "required");

		if (!result1) {
			message = this.strings.validate_language_required_err;
			this.show_error_list(message, $language);
			return false;
		}

		if (!result2) {
			message = this.strings.validate_value_required_err;
			this.show_error_list(message, $value);
			return false;
		}

		let value = $value.val();
		var already_exist = function()
		{
			const exist = self.already_exist_list(value);
			if (exist) {
				var message = FORMAT(self.strings.validate_value_already_exist_err, value);
				const $grid = self.$get_grid_list();
				self.show_error_list(message, $grid);
				$value.focus();
				return true;
			}
			return false;
		};

		// Valida que el valor ya no exista en la lista
		if (already_exist()) {
			return false;
		}

		// validation successfully
		return true;
	};

	children_list.prototype.create = function(has_continue)
	{
		this.hide_errors();

		if ( !this.owner.has_permission_to_section() || !this.owner.has_permission_to_save() ) {
			var message = this.strings.popup_title;
			this.owner.form_paginator.show_popup_msg(message, "Error", ERROR_TIMEOUT_MS);
			this.hide_popup();
			return;
		}

		const $name = $(this.children_name);
		const child = {
			name: $name.val().trim()
		};
		// validate parameters
		const result = this.validate(child);
		if ( !result || ( typeof result === "object" && !result.success ) ) {
			return;
		}

		if (!has_continue) {
			this.hide_popup();
		}

		if (!this.$row_editing) {
		// add new parameter
			const apply_effect = true;
			this.add_row(child, apply_effect);
		}
		// Cada vez que visualizo el popup de add en la grilla, realizo un load, para que actualice los controles (Ej: combo con las entidades hijas para los COMPOSITE)
		this.load(function() {
			$name.val("").trigger("change");
			if (!has_continue) {
				return;
			}
			$name.focus();
		});
	};

	children_list.prototype.create_prebuilt = function(has_continue)
	{
		this.hide_errors();

		if ( !this.owner.has_permission_to_section() || !this.owner.has_permission_to_save() ) {
			var message = this.strings.popup_title;
			this.owner.form_paginator.show_popup_msg(message, "Error", ERROR_TIMEOUT_MS);
			this.hide_popup();
			return;
		}

		const $category = $(this.children_name_prebuilt);
		const child = {
			category: $category.val().trim()
		};

		// validate parameters
		const result = this.validate_prebuilt(child);

		if ( !result || ( typeof result === "object" && !result.success ) ) {
			return;
		}

		if (!has_continue) {
			this.hide_popup();
		}

		if (!this.$row_editing) {
		// add new parameter
			const apply_effect = true;
			this.add_row_prebuilt(child, apply_effect);
		}
		// Cada vez que visualizo el popup de add en la grilla, realizo un load, para que actualice los controles (Ej: combo con las entidades hijas para los COMPOSITE)
		this.load(function() {
			$category.val("").trigger("change");
			if (!has_continue) {
				return;
			}
			$category.focus();
		});
	};

	children_list.prototype.create_regular_exp = function(has_continue)
	{
		this.hide_errors();

		if ( !this.owner.has_permission_to_section() || !this.owner.has_permission_to_save() ) {
			var message = this.strings.popup_title;
			this.owner.form_paginator.show_popup_msg(message, "Error", ERROR_TIMEOUT_MS);
			this.hide_popup();
			return;
		}

		const $regexKey = $(this.children_regexkey_regular_exp);
		const $language = $(this.children_language_regular_exp);
		const $pattern = $(this.children_pattern_regular_exp);

		const child = {
			regexKey: $regexKey.val().trim(),
			language: $language.val().trim(),
			regexPattern: $pattern.val().trim()
		};
		// validate parameters
		const result = this.validate_regular_exp(child);
		if ( !result || ( typeof result === "object" && !result.success ) ) {
			return;
		}

		if (!has_continue) {
			this.hide_popup();
		}

		if (!this.$row_editing) {
		// add new parameter
			const apply_effect = true;
			this.add_row_regular_exp(child, apply_effect);
		}
		// Cada vez que visualizo el popup de add en la grilla, realizo un load, para que actualice los controles (Ej: combo con las entidades hijas para los COMPOSITE)
		this.load(function() {
			$pattern.val("");
			$regexKey.val("").trigger("change");
			if (!has_continue) {
				return;
			}
			$regexKey.focus();
		});
	};

	children_list.prototype.create_list = function(has_continue)
	{
		this.hide_errors();

		if ( !this.owner.has_permission_to_section() || !this.owner.has_permission_to_save() ) {
			var message = this.strings.popup_title;
			this.owner.form_paginator.show_popup_msg(message, "Error", ERROR_TIMEOUT_MS);
			this.hide_popup();
			return;
		}

		const $value = $(this.children_value_list);
		const $language = $(this.children_language_list);
		
		const child = {
			value: $value.val().trim(),
			language: $language.val().trim()
		};
		// validate parameters
		const result = this.validate_list(child);
		if ( !result || ( typeof result === "object" && !result.success ) ) {
			return;
		}

		if (!has_continue) {
			this.hide_popup();
		}

		if (!this.$row_editing) {
		// add new parameter
			const apply_effect = true;
			this.add_row_list(child, apply_effect);
		}
		// Cada vez que visualizo el popup de add en la grilla, realizo un load, para que actualice los controles (Ej: combo con las entidades hijas para los COMPOSITE)
		this.load(function() {
			$value.val("").trigger("change");
			if (!has_continue) {
				return;
			}
			$value.focus();
		});
	};

	children_list.prototype.disable_actions = function(disable)
	{
		disable = disable === undefined ? true : disable;

		if (this.owner.is_readonly()) {
			disable = true;
		}
		const $children_new = $(this.children_new);
		$children_new.prop("disabled", disable);
		const $item_data = $(this.children_list).find("li[item_data]");
		// boton delete de la lista de entities
		$item_data.find("button[action_del]").prop("disabled", disable);
		$item_data.find("button[action_del_prebuilt]").prop("disabled", disable);
		$item_data.find("button[action_del_regular_exp]").prop("disabled", disable);
		$item_data.find("button[action_del_list]").prop("disabled", disable);
		this.owner.disable_actions(disable);
	};

	children_list.prototype.is_valid_data = function(value, validation_type)
	{
		const response = this.owner.frm.validation.valid(value, validation_type);
		return response.success;
	};

	children_list.prototype.set_events = function()
	{
		const self = this;
		if (this.children_new === undefined) {
			return;
		}

		// boton new que abre el popup para poder agregar una nueva entity
		$(this.children_new)
			.off("click")
			.on("click", function() {
				self.$row_editing = null;
				self.show_popup();
			});

		$(this.children_create)
			.off("click")
			.on("click", function() {
				const has_continue = false;
				self.create(has_continue);
			});

		$(this.children_create_continue)
			.off("click")
			.on("click", function() {
				var has_continue = true;
				self.create(has_continue);
			});

		$(this.children_create_continue_prebuilt)
			.off("click")
			.on("click", function() {
				var has_continue = true;
				self.create_prebuilt(has_continue);
			});

		$(this.children_create_continue_regular_exp)
			.off("click")
			.on("click", function() {
				var has_continue = true;
				self.create_regular_exp(has_continue);
			});

		$(this.children_create_continue_list)
			.off("click")
			.on("click", function() {
				var has_continue = true;
				self.create_list(has_continue);
			});

		$(this.children_close)
			.off("click")
			.on("click", function() {
				self.hide_popup();
			});
		$(this.children_name)
			.off("keypress")
			.on("keypress", function(e) {
				if (e.which === 13) {
					var has_continue = true;
					self.create(has_continue);
				}
			});

		// grilla botones DELETE
		const $param_list = $(this.children_list);
		// boton delete de la lista de Children
		let event_selector = "button[action_del]";
		$param_list
			.off("click", event_selector)
			.on("click", event_selector, function()
			{
				const $parameter = $(this).parent();
				let message = FORMAT(
					self.strings.delete_question,
					$parameter.find("[child_name='true']").text()
				);
				$.inconcertPopup.Show(
					"<%=_T('Confirm')%>",
					message,
					$.inconcertPopup.ButtonSets.BS_YESNO,
					function( response )
					{
						if (response.toLowerCase() !== "yes") {
							return;
						}
						if ( !self.owner.has_permission_to_section() || !self.owner.has_permission_to_save() ) {
							let err_message = self.strings.err_not_permission_to_delete;
							self.owner.form_paginator.show_popup_msg(err_message, "Error", ERROR_TIMEOUT_MS);
							return;
						}
						$parameter.fadeOut("slow", function() {
							$parameter.remove();
						});
					},
					undefined,
					{
						popup_id       : "conf_inconcertPopup",
						container_class: "confirmationPopUp",
						overlay_class  : "confirmationOverlay"
					}
				);
			}
		);

		const $param_list_prebuilt = $(this.children_list_prebuilt);
		let event_selector_prebuilt = "button[action_del_prebuilt]";
		$param_list_prebuilt
			.off("click", event_selector_prebuilt)
			.on("click", event_selector_prebuilt, function()
			{
				const $parameter = $(this).parent();
				let message = FORMAT(
					self.strings.delete_question,
					$parameter.find("[child_category='true']").text()
				);
				$.inconcertPopup.Show(
					"<%=_T('Confirm')%>",
					message,
					$.inconcertPopup.ButtonSets.BS_YESNO,
					function( response )
					{
						if (response.toLowerCase() !== "yes") {
							return;
						}
						if ( !self.owner.has_permission_to_section() || !self.owner.has_permission_to_save() ) {
							let err_message = self.strings.err_not_permission_to_delete;
							self.owner.form_paginator.show_popup_msg(err_message, "Error", ERROR_TIMEOUT_MS);
							return;
						}
						$parameter.fadeOut("slow", function() {
							$parameter.remove();
						});
					},
					undefined,
					{
						popup_id       : "conf_inconcertPopup",
						container_class: "confirmationPopUp",
						overlay_class  : "confirmationOverlay"
					}
				);
			}
		);

		const $param_list_regular_exp = $(this.children_list_regular_exp);
		let event_selector_regular_exp = "button[action_del_regular_exp]";
		$param_list_regular_exp
			.off("click", event_selector_regular_exp)
			.on("click", event_selector_regular_exp, function()
			{
				const $parameter = $(this).parent();
				let message = FORMAT(
					self.strings.delete_question,
					$parameter.find("[child_regexkey='true']").text()
				);
				$.inconcertPopup.Show(
					"<%=_T('Confirm')%>",
					message,
					$.inconcertPopup.ButtonSets.BS_YESNO,
					function( response )
					{
						if (response.toLowerCase() !== "yes") {
							return;
						}
						if ( !self.owner.has_permission_to_section() || !self.owner.has_permission_to_save() ) {
							let err_message = self.strings.err_not_permission_to_delete;
							self.owner.form_paginator.show_popup_msg(err_message, "Error", ERROR_TIMEOUT_MS);
							return;
						}
						$parameter.fadeOut("slow", function() {
							$parameter.remove();
						});
					},
					undefined,
					{
						popup_id       : "conf_inconcertPopup",
						container_class: "confirmationPopUp",
						overlay_class  : "confirmationOverlay"
					}
				);
			}
		);

		const $param_list_list = $(this.children_list_list);
		let event_selector_list = "button[action_del_list]";
		$param_list_list
			.off("click", event_selector_list)
			.on("click", event_selector_list, function()
			{
				const $parameter = $(this).parent();
				let message = FORMAT(
					self.strings.delete_question,
					$parameter.find("[child_value='true']").text()
				);
				$.inconcertPopup.Show(
					"<%=_T('Confirm')%>",
					message,
					$.inconcertPopup.ButtonSets.BS_YESNO,
					function( response )
					{
						if (response.toLowerCase() !== "yes") {
							return;
						}
						if ( !self.owner.has_permission_to_section() || !self.owner.has_permission_to_save() ) {
							let err_message = self.strings.err_not_permission_to_delete;
							self.owner.form_paginator.show_popup_msg(err_message, "Error", ERROR_TIMEOUT_MS);
							return;
						}
						$parameter.fadeOut("slow", function() {
							$parameter.remove();
						});
					},
					undefined,
					{
						popup_id       : "conf_inconcertPopup",
						container_class: "confirmationPopUp",
						overlay_class  : "confirmationOverlay"
					}
				);
			}
		);
	};

	children_list.prototype.is_allowed_create = function()
	{
		return false;
	};

	children_list.prototype.show_status_message = function(msg, msg_type, timeout_ms)
	{
		timeout_ms = timeout_ms || ERROR_TIMEOUT_MS;
		this.owner.form_paginator.show_popup_msg(msg, msg_type, ERROR_TIMEOUT_MS);
	};

	children_list.prototype.show_waiting = function(msg)
	{
		this.owner.form_paginator.show_popup_waiting(msg);
	};

	children_list.prototype.hide_waiting = function()
	{
		this.owner.form_paginator.hide_popup_waiting();
	};

	//---------------------------------------------------------------------
	// COMPOSITE children list
	//
	function composite_children_list(params)
	{
		composite_children_list.uber.constructor.call(this, params);
	}

	inConcert.Utils.inherit(composite_children_list, children_list);

	composite_children_list.prototype.is_allowed_create = function()
	{
		if (API_VERSION === "2.0") {
			return true;
		}
		//return false;
		//TODO: deberia ser FALSE, pero por ahora queda asi. En una migration esto deberi quedar FALSE.
		return true;
	};

	composite_children_list.prototype.get_popup_ctrls_markup = function()
	{
		const ctrl_id = SANITIZE_ID(this.children_name);
		const markup =
			"<div class='row'>" +
				"<div class='col-6'>" +
					"<div class='form-group'>" +
						"<label for='" + ctrl_id + "' class='required'>" + "<%=_T('Child Name')%>" + "<%=_T(':')%>" + "</label>" +
						"<select id='" + ctrl_id + "' class='form-control'></select>" +
						"<i id='" + ctrl_id + "_validation' class='formFieldValidation formFieldError' hidden ></i>" +
					"</div>" +
				"</div>" +
			"</div>";
		return markup;
	};

	composite_children_list.prototype.load = function(callback = $.noop)
	{
		let children_entities = this.__make_children_entities();
		const $children = $(this.children_name);
		this.owner.frm.fill_combo($children, true, children_entities, "__display_name", "__display_name", false);
		callback();
	};

	composite_children_list.prototype.__make_children_entities = function()
	{
		const self = this;

		let children_entities = [];
		var add = function(entity_id, entity) {
			let exist = self.already_exist(entity_id);
			if (!exist) {
				let cloned_entity = inConcert.Utils.cloneObject(entity);
				cloned_entity.__display_name = entity_id;
				children_entities.push(cloned_entity);
			}
		};

		const allowed_children_entities_type = {
			"1": true, // MACHINELEARNED
			"2": true, // PREBUILT
			"3": true,  // HIERARCHICAL
			"combineComponents": true  // CombineComponents
		};
		let current_entities = this.owner.get_current_entities();

		current_entities.forEach(entity => {
			let type_id = entity.typeId.toString();
			if (allowed_children_entities_type[type_id]) {
				let entity_id = entity.name;
				if (entity.children) {
					// HIERARCHICAL HIJOS
					let hierarchical_child_id;
					entity.children.forEach(hierarchical_child => {
						hierarchical_child_id = FORMAT("{0}::{1}", entity_id, hierarchical_child.name);
						add(hierarchical_child_id, hierarchical_child);
					});
				}
				// MACHINELEARNED o HIERARCHICAL PADRE
				add(entity_id, entity);
			}
		});

		return children_entities;
	};

	//---------------------------------------------------------------------
	// HIERARCHICAL children list
	//
	function hierarchical_children_list(params)
	{
		hierarchical_children_list.uber.constructor.call(this, params);
	}

	inConcert.Utils.inherit(hierarchical_children_list, children_list);

	hierarchical_children_list.prototype.is_allowed_create = function()
	{
		if (API_VERSION === "2.0") {
			return true;
		}
		return false;
	};

	hierarchical_children_list.prototype.get_popup_ctrls_markup = function()
	{
		const ctrl_id = SANITIZE_ID(this.children_name);
		const markup =
			"<div class='row'>" +
				"<div class='col-6'>" +
					"<div class='form-group'>" +
						"<label for='" + ctrl_id + "' class='required'>" + "<%=_T('Child Name')%>" + "<%=_T(':')%>" + "</label>" +
						"<input id='" + ctrl_id + "' class='form-control' placeholder='<%=_T('Enter the name')%>...' type='text' />" +
						"<i id='" + ctrl_id + "_validation' class='formFieldValidation formFieldError' hidden ></i>" +
					"</div>" +
				"</div>" +
			"</div>";
		return markup;
	};

	hierarchical_children_list.prototype.validate = function(child)
	{
		let result = hierarchical_children_list.uber.validate.call(this, child); //invokes the base class method

		if (!result) {
			return false;
		}
		const $name = $(this.children_name);
		result = this.is_valid_data(child.name, "id");
		if (!result) {
			// valida los caracteres permitidos
			let message = "<%=_T('Only letters, numbers, dashes and dots are allowed in the Name.')%>";
			this.show_error(message, $name);
			return false;
		}

		return true;
	};

	//---------------------------------------------------------------------
	// PREBUILT Entities list
	//
	function prebuilt_children_list(params)
	{
		params.strings = {
			popup_title                    : "<%=_T('Assign Prebuilt Entity')%>",
			new_item                       : "<%=_T('Add')%>",
			list_title                     : "<%=_T('Prebuilt Entities')%>",
			btn_create                     : "<%=_T('Add')%>",
			btn_create_continue            : "<%=_T('Add & Continue')%>",
			err_not_permission_to_create   : "<%=_T('Unable to assign Prebuilt Entity: You dont have permissions for this action.')%>",
			err_not_permission_to_delete   : "<%=_T('Unable to remove Prebuilt Entity: You dont have permissions for this action.')%>",
			delete_question                : "<%=_T('Do you want to remove Prebuilt Entity ({0}) ?')%>",
			validate_name_required_err     : "<%=_T('Please select the prebuilt entity.')%>",
			validate_name_already_exist_err: "<%=_T('Already exist prebuilt entity name {0}.')%>",
			validate_child_required_err    : "<%=_T('Must include at least one Prebuilt Entity.')%>"
		};
		params.closeOnFocusLost = false;
		prebuilt_children_list.uber.constructor.call(this, params);
		this.__set_custom_strings();
	}

	inConcert.Utils.inherit(prebuilt_children_list, children_list);

	prebuilt_children_list.prototype.is_allowed_create = function()
	{
		return true;
	};

	prebuilt_children_list.prototype.get_popup_ctrls_markup = function()
	{
		const ctrl_id = SANITIZE_ID(this.children_name);
		const desc_ctrl_id = SANITIZE_ID(this.children_desc);
		const example_ctrl_id = SANITIZE_ID(this.children_example);

		const markup =
			"<div class='row'>" +
				"<div class='col-4'>" +
					"<div class='form-group'>" +
						"<label for='" + ctrl_id + "' class='required'>" + "<%=_T('Prebuilt Entity')%>" + "<%=_T(':')%>" + "</label>" +
						"<select id='" + ctrl_id + "' class='form-control'></select>" +
						"<i id='" + ctrl_id + "_validation' class='formFieldValidation formFieldError' hidden ></i>" +
					"</div>" +
				"</div>" +
				"<div class='col-4'>" +
					"<div class='form-group'>" +
						"<label for='" + desc_ctrl_id + "' class=''>" + "<%=_T('Description')%>" + "<%=_T(':')%>" + "</label>" +
						"<span id='" + desc_ctrl_id + "' class='form-control'></span>" +
					"</div>" +
				"</div>" +
				"<div class='col-4'>" +
					"<div class='form-group'>" +
						"<label for='" + example_ctrl_id + "' class=''>" + "<%=_T('Example')%>" + "<%=_T(':')%>" + "</label>" +
						"<span id='" + example_ctrl_id + "' class='form-control'></span>" +
					"</div>" +
				"</div>" +
			"</div>";
		return markup;
	};

	prebuilt_children_list.prototype.load = function(callback = $.noop)
	{
		let prebuilt_entities = this.__make_children_entities();
		const $children = $(this.children_name);
		this.owner.frm.fill_combo($children, true, prebuilt_entities, "__display_name", "__display_name", false);
		callback();
	};

	prebuilt_children_list.prototype.must_add = function(item_value)
	{
		item_value = (item_value || "").trim().toLowerCase();
		// Primero vailda que el prebuilt a agregar ya no exista en la grilla del popup.
		const already_exist = prebuilt_children_list.uber.already_exist.call(this, item_value); //invokes the base class method
		if (already_exist) {
			return false;
		}
		// Segundo valida que el prebuilt a agregar ya no se encuentre almacenado en en la app (luis)
		let must_add = true;
		let current_entities = this.owner.get_current_entities() || [];
		for (let i = 0; i < current_entities.length; i++) {
			let current_entity = current_entities[i];
			if (current_entity.typeId === 2) { // PREBUILT
				if (item_value === (current_entity.name || "").trim().toLowerCase()) {
					must_add = false;
					break;
				}
			}
		}
		return must_add;
	};

	prebuilt_children_list.prototype.__make_children_entities = function()
	{
		const self = this;

		let available_prebuilt_entities = [];
		var add = function(prebuilt_entity_id, prebuilt_entity) {
			let must_add = self.must_add(prebuilt_entity_id);
			if (must_add) {
				let cloned_prebuilt_entity = inConcert.Utils.cloneObject(prebuilt_entity);
				cloned_prebuilt_entity.__display_name = prebuilt_entity_id;
				available_prebuilt_entities.push(cloned_prebuilt_entity);
			}
		};

		let current_prebuilt_entities = this.owner.get_current_prebuilt_entities();
		current_prebuilt_entities.forEach(current_prebuilt_entity => {
			let current_prebuilt_entity_id = current_prebuilt_entity.name;
			add(current_prebuilt_entity_id, current_prebuilt_entity);
		});

		return available_prebuilt_entities;
	};

	prebuilt_children_list.prototype.set_events = function()
	{
		const self = this;

		prebuilt_children_list.uber.set_events.call(this); //invokes the base class method

		const $desc = $(this.children_desc);
		const $example = $(this.children_example);
		const $name = $(this.children_name);
		$name
			.off("change")
			.on("change", function() {
				const selected_prebuilt_info = self.owner.frm.combo_get_selected_item_data($name);
				let desc = "";
				let examples = "";

				if (selected_prebuilt_info) {
					if (selected_prebuilt_info.description) {
						desc = self._T(selected_prebuilt_info.description);
					}
					if (selected_prebuilt_info.examples) {
						examples = self._T(selected_prebuilt_info.examples);
					}
				}

				$desc.text(desc);
				$example.text(examples);
			});
	};

	prebuilt_children_list.prototype._T = function(value)
	{
		return this.strings[value] || value;
	};

	/*
		Estos strings, son los que retorna el API de LUIS en el metodo "LuisAPI.loadAvailablePrebuiltEntityList"
		Tengo que agregar los strings aca, ya que estos strings llegan dados de esta forma en el request.
	*/
	prebuilt_children_list.prototype.__set_custom_strings = function()
	{
		this.strings = this.strings || {};
		this.strings["Age of a person or thing"] = "<%=_T('Age of a person or thing')%>";
		this.strings["10-month-old, 19 years old, 58 year-old"] = "<%=_T('10-month-old, 19 years old, 58 year-old')%>";
		this.strings["Dates and times, resolved to a canonical form"] = "<%=_T('Dates and times, resolved to a canonical form')%>";
		this.strings["June 23, 1976, Jul 11 2012, 7 AM, 6:49 PM, tomorrow at 7 AM"] = "<%=_T('June 23, 1976, Jul 11 2012, 7 AM, 6:49 PM, tomorrow at 7 AM')%>";
		this.strings["Spacial dimensions, including length, distance, area, and volume"] = "<%=_T('Spacial dimensions, including length, distance, area, and volume')%>";
		this.strings["2 miles, 650 square kilometres, 9,350 feet"] = "<%=_T('2 miles, 650 square kilometres, 9,350 feet')%>";
		this.strings["Email addresses"] = "<%=_T('Email addresses')%>";
		this.strings["user@site.net, user_name@mysite.com.eg, user.Name12@website.net"] = "<%=_T('user@site.net, user_name@mysite.com.eg, user.Name12@website.net')%>";
		this.strings["Automatically extract key phrases to quickly identify the main talking points"] = "<%=_T('Automatically extract key phrases to quickly identify the main talking points')%>";
		this.strings["Paris, cinema, soccer"] = "<%=_T('Paris, cinema, soccer')%>";
		this.strings["Monetary amounts, including currency"] = "<%=_T('Monetary amounts, including currency')%>";
		this.strings["1000.00 US dollars, £20.00, $ 67.5 B"] = "<%=_T('1000.00 US dollars, £20.00, $ 67.5 B')%>";
		this.strings["A cardinal number in numeric or text form"] = "<%=_T('A cardinal number in numeric or text form')%>";
		this.strings["ten, forty two, 3.141, 10K"] = "<%=_T('ten, forty two, 3.141, 10K')%>";
		this.strings["An ordinal number in numeric or text form"] = "<%=_T('An ordinal number in numeric or text form')%>";
		this.strings["first, second, tenth, 1st, 2nd, 10th"] = "<%=_T('first, second, tenth, 1st, 2nd, 10th')%>";
		this.strings['A percentage, using the symbol % or the word "percent"'] = "<%=_T([[A percentage, using the symbol % or the word \'percent\']])%>";
		this.strings["10%, 5.6 percent"] = "<%=_T('10%, 5.6 percent')%>";
		this.strings["US phone numbers"] = "<%=_T('US phone numbers')%>";
		this.strings["123-456-7890, +1 123 456 789, (123)456-789"] = "<%=_T('123-456-7890, +1 123 456 789, (123)456-789')%>";
		this.strings["A temperature in celsius or fahrenheit"] = "<%=_T('A temperature in celsius or fahrenheit')%>";
		this.strings["32F, 34 degrees celsius, 2 deg C"] = "<%=_T('32F, 34 degrees celsius, 2 deg C')%>";
		this.strings["Websites URLs and links"] = "<%=_T('Websites URLs and links')%>";
		this.strings["www.website.com, http://website.net?name=my_name&age=10, https://www.mywebsite.net.eg/page"] = "<%=_T('www.website.com, http://website.net?name=my_name&age=10, https://www.mywebsite.net.eg/page')%>";
	};

	prebuilt_children_list.prototype.show = function() {
		const self = this;

		var on_shown = function(success) {
			if (!success) {
				return;
			}
			const $create = $(self.children_create);
			$create.attr("value", $(self.children_create_continue).attr("value"));

			$create.remove();
			$(self.children_close).remove();
			$(self.children_new).remove();
		};
		// visualizo el popup
		this.show_popup(on_shown);
	};

	//---------------------------------------------------------------------
	// MACHINE LEARNER children list 
	//
	function machine_learner_children_list(params)
	{
		machine_learner_children_list.uber.constructor.call(this, params);
	}

	inConcert.Utils.inherit(machine_learner_children_list, children_list);

	machine_learner_children_list.prototype.render = function()
	{
		const sanitized_id_param_new = SANITIZE_ID(this.children_new);

		const $html = $("".concat(
			"<i id='", sanitized_id_param_new, "_validation' class='formFieldValidation formFieldError' hidden></i>",
			"<button id='", sanitized_id_param_new, "' class='pageButtonGeneric buttonColorWhite'>",
				this.strings.new_item,
			"</button>",
			"<div class='card-block'>",
				"<div class='row'>",
					"<i id='", SANITIZE_ID(this.children_list), "_validation' class='formFieldValidation formFieldError' hidden></i>",
					"<div id='", SANITIZE_ID(this.prefix_id), "_tree' class='col-12'></div>",
				"</div>",
			"</div>")
		);

		const $section = $(this.children_section);
		$section.html($html);
	};

	machine_learner_children_list.prototype.initialize = function()
	{
		machine_learner_children_list.uber.initialize.call(this);
		this.initialize_tree();
	};

	machine_learner_children_list.prototype.initialize_tree = function()
	{
		const self = this;
		let prefix_id = SANITIZE_ID(this.prefix_id);

		this.tree = new inConcert.conf_form_tree({
			owner: this,
			frm: this.owner.frm,
			is_enable_drag_and_drop: false,
			nodes_attrs: {
				id       : "id",
				childrens: "children"
			},
			ids: {
				tree : prefix_id + "_tree",
				popup: prefix_id + "_tree_popup",
				popup_msg: prefix_id + "_tree_popup_msg",
				popup_create: prefix_id + "_tree_popup_create",
				popup_create_and_continue: prefix_id + "_tree_popup_create_and_continue",
				popup_update: prefix_id + "_tree_popup_update",
				popup_close: prefix_id + "_tree_popup_cancel",
				popup_close_times: prefix_id + "_tree_popup_cancel_times",
				popup_title: prefix_id + "_tree_popup_title",
				popup_input_id: prefix_id + "_tree_popup_node_name",
				popup_waiting: prefix_id + "_tree_popup_waiting"
			},
			strings: {
				waiting_add_nodes: "<%=_T('Please wait, adding child entities.')%>",
				waiting_upd_nodes: "<%=_T('Please wait, updating child entities.')%>",
				waiting_del_node : "<%=_T('Please wait, removing child entity.')%>"
			},
			auto_initialize: true,
			callbacks: {
				create: function(node, callback = $.noop) {
					self.create(node, callback);
				},
				update: function(node, callback = $.noop) {
					self.update(node, callback);
				},
				remove: function(node_id, callback = $.noop) {
					self.remove(node_id, callback);
				},
				show_status_message: function(msg, msg_type, timeout_ms) {
					self.show_status_message(msg, msg_type, timeout_ms);
				},
				show_waiting: function(msg) {
					self.show_waiting(msg);
				},
				hide_waiting: function() {
					self.hide_waiting();
				}
			}
		});
	};

	machine_learner_children_list.prototype.is_allowed_create = function()
	{
		return true;
	};

	machine_learner_children_list.prototype.get_popup_ctrls_markup = function()
	{
		const ctrl_id = SANITIZE_ID(this.children_name);
		const markup =
			"<div class='row'>" +
				"<div class='col-12'>" +
					"<div class='form-group'>" +
						"<label for='" + ctrl_id + "' class='required'>" + "<%=_T('Child Name')%>" + "<%=_T(':')%>" + "</label>" +
						"<input id='" + ctrl_id + "' class='form-control' placeholder='<%=_T('Enter the name')%>...' type='text' />" +
						"<i id='" + ctrl_id + "_validation' class='formFieldValidation formFieldError' hidden ></i>" +
					"</div>" +
				"</div>" +
			"</div>";
		return markup;
	};

	machine_learner_children_list.prototype.validate = function(child)
	{
		let result = machine_learner_children_list.uber.validate.call(this, child); //invokes the base class method

		if (!result) {
			return false;
		}
		const $name = $(this.children_name);
		result = this.is_valid_data(child.name, "id");
		if (!result) {
			// valida los caracteres permitidos
			let message = "<%=_T('Only letters, numbers, dashes and dots are allowed in the Name.')%>";
			this.show_error(message, $name);
			return false;
		}

		return true;
	};

	machine_learner_children_list.prototype.is_required_childrens = function()
	{
		return false;
	};

	machine_learner_children_list.prototype.get = function()
	{
		let tree_info = this.tree.get();
		let children = tree_info.nodes;
		return children;
	};

	machine_learner_children_list.prototype.set = function(info)
	{
		this.tree.set({
			root_node_id: info.id,
			children    : info.children
		});
	};

	machine_learner_children_list.prototype.show_popup = function(on_shown = $.noop)
	{
		const self = this;
		this.hide_errors();
		this.load(function() {
			self.tree.show_root_popup();
			on_shown(true);
		});
	};

	machine_learner_children_list.prototype.dispose = function()
	{
		this.clean_up_list();
	};

	machine_learner_children_list.prototype.remove = function(node_id, callback = $.noop)
	{
		if (this.owner.is_new) {
			callback(true);
			return;
		}

		let app_id = this.owner.frm.get_entity_id();
		const params = {appId: app_id, entityId: node_id};
		const luisApi = inConcert.app.getLuisAPI();

		luisApi.deleteAppEntityChildMachineLearned(params,
			function(success, result) {
				if (!success) {
					let server_err_msg = luisApi.getCognitiveErrorMsg(result);
					let msg_err = "<%=_T('Error deleting Entity.')%>" + " " + server_err_msg;
					callback(success, msg_err);
					return;
				}
				callback(true);
			}
		);
	};

	machine_learner_children_list.prototype.create = function(node, callback = $.noop)
	{
		if (this.owner.is_new) {
			callback(true);
			return;
		}

		let app_id = this.owner.frm.get_entity_id();
		const params = {appId: app_id, entityId: node.parent_node_id, name: node.name};
		const luisApi = inConcert.app.getLuisAPI();

		luisApi.createAppEntityChildMachineLearned(params,
			function(success, result) {
				if (!success) {
					let server_err_msg = luisApi.getCognitiveErrorMsg(result);
					let msg_err = "<%=_T('Error creating Entity.')%>" + " " + server_err_msg;
					callback(success, msg_err);
					return;
				}
				callback(true, result.entityId);
			}
		);
	};

	machine_learner_children_list.prototype.update = function(node, callback = $.noop)
	{
		if (this.owner.is_new) {
			callback(true);
			return;
		}

		let app_id = this.owner.frm.get_entity_id();
		const params = {appId: app_id, entityId: node.node_id, name: node.name};
		const luisApi = inConcert.app.getLuisAPI();

		luisApi.renameAppEntityChildMachineLearned(params,
			function(success, result) {
				if (!success) {
					let server_err_msg = luisApi.getCognitiveErrorMsg(result);
					let msg_err = "<%=_T('Error updating Entity.')%>" + " " + server_err_msg;
					callback(success, msg_err);
					return;
				}
				callback(true);
			}
		);
	};

	machine_learner_children_list.prototype.clean_up_list = function()
	{
		if (this.tree) {
			this.tree.remove_all_nodes();
		}
	};

	//---------------------------------------------------------------------
	// Combine Components Entities list
	//
	function combine_component_children_list(params)
	{
		params.strings = {
			popup_title                    : "<%=_T('Assign Combine Components Entity')%>",
			new_item                       : "<%=_T('Add')%>",
			list_title                     : "<%=_T('Combine Components Entities')%>",
			btn_create                     : "<%=_T('Add')%>",
			btn_create_continue            : "<%=_T('Add & Continue')%>",
			err_not_permission_to_create   : "<%=_T('Unable to assign Combine Components Entity: You dont have permissions for this action.')%>",
			err_not_permission_to_delete   : "<%=_T('Unable to remove Combine Components Entity: You dont have permissions for this action.')%>",
			delete_question                : "<%=_T('Do you want to remove Combine Components Entity ({0}) ?')%>",
			validate_name_required_err     : "<%=_T('Please select the Combine Components entity.')%>",
			validate_language_required_err     : "<%=_T('Please select the Combine Components Language.')%>",
			validate_name_already_exist_err: "<%=_T('Already exist Combine Components entity name {0}.')%>",
			validate_child_required_err    : "<%=_T('Must include at least one Combine Components Entity.')%>",
			validate_regexkey_already_exist_err: "<%=_T('Already exist Combine Components value {0}.')%>",
			validate_regexkey_required_err    : "<%=_T('Must include at least one Combine Components Value.')%>"
		};
		params.closeOnFocusLost = false;
		combine_component_children_list.uber.constructor.call(this, params);
		this.__set_custom_strings();
	}

	inConcert.Utils.inherit(combine_component_children_list, children_list);

	combine_component_children_list.prototype.is_allowed_create = function()
	{
		return true;
	};

	combine_component_children_list.prototype.get_popup_ctrls_markup_prebuilt = function()
	{
		const ctrl_id = SANITIZE_ID(this.children_name_prebuilt);
		const desc_ctrl_id = SANITIZE_ID(this.children_desc_prebuilt);
		const example_ctrl_id = SANITIZE_ID(this.children_example_prebuilt);

		const markup =
			"<div class='card-header'>" + 
				"<h5><%=_T('Prebuilt')%></h5>" + 
			"</div>" +
			"<div class='row'>" +						
				"<div class='col-4'>" +
					"<div class='form-group'>" +
						"<label for='" + ctrl_id + "' class='required'>" + "<%=_T('Combine Components Entity')%>" + "<%=_T(':')%>" + "</label>" +
						"<select id='" + ctrl_id + "' class='form-control'></select>" +
						"<i id='" + ctrl_id + "_validation' class='formFieldValidation formFieldError' hidden ></i>" +
					"</div>" +
				"</div>" +
				"<div class='col-4'>" +
					"<div class='form-group'>" +
						"<label for='" + desc_ctrl_id + "' class=''>" + "<%=_T('Description')%>" + "<%=_T(':')%>" + "</label>" +
						"<span id='" + desc_ctrl_id + "' class='form-control'></span>" +
					"</div>" +
				"</div>" +
				"<div class='col-4'>" +
					"<div class='form-group'>" +
						"<label for='" + example_ctrl_id + "' class=''>" + "<%=_T('Example')%>" + "<%=_T(':')%>" + "</label>" +
						"<span id='" + example_ctrl_id + "' class='form-control'></span>" +
					"</div>" +
				"</div>" +
			"</div>"; // +
			//"<div disclaimer='true' class='inlineMessagesContainer messageInfo mt-0'><%=_T('A prebuilt component gets your entity to extract common types such as numbers, dates, times, and others.')%></div>"; 	

		return markup;
	};

	combine_component_children_list.prototype.get_popup_ctrls_markup_regular_exp = function()
	{
		const regexkey_ctrl_id = SANITIZE_ID(this.children_regexkey_regular_exp);
		const language_ctrl_id = SANITIZE_ID(this.children_language_regular_exp);
		const pattern_ctrl_id = SANITIZE_ID(this.children_pattern_regular_exp);

		const markup =
			"<br/><div class='card-header'>" + 
				"<h5><%=_T('Regular expression')%></h5>" + 
			"</div>" +
			"<div class='row'>" +						
				"<div class='col-4'>" +
					"<div class='form-group'>" +
						"<label for='" + language_ctrl_id + "' class='required'>" + "<%=_T('Language')%>" + "<%=_T(':')%>" + "</label>" +
						"<select id='" + language_ctrl_id + "' class='form-control'></select>" +
						"<i id='" + language_ctrl_id + "_validation' class='formFieldValidation formFieldError' hidden ></i>" +
					"</div>" +
				"</div>" +
				"<div class='col-4'>" +
					"<div class='form-group'>" +
						"<label for='" + regexkey_ctrl_id + "' class='required'>" + "<%=_T('Value')%>" + "<%=_T(':')%>" + "</label>" +
						"<input id='" + regexkey_ctrl_id + "' class='form-control'></input>" +
					"</div>" +
				"</div>" +
				"<div class='col-4'>" +
					"<div class='form-group'>" +
						"<label for='" + pattern_ctrl_id + "' class='required'>" + "<%=_T('Pattern')%>" + "<%=_T(':')%>" + "</label>" +
						"<input id='" + pattern_ctrl_id + "' class='form-control'></input>" +
					"</div>" +
				"</div>" +
			"</div>"; // +
			//"<div disclaimer='true' class='inlineMessagesContainer messageInfo mt-0'><%=_T('A regex component matches regular expressions for common patterns. You can associate a key to each expression.')%></div>"; 	

		return markup;
	};

	combine_component_children_list.prototype.get_popup_ctrls_markup_list = function()
	{
		const ctrl_id = SANITIZE_ID(this.children_value_list);
		const language_ctrl_id = SANITIZE_ID(this.children_language_list);

		const markup =
			"<br/><div class='card-header'>" + 
				"<h5><%=_T('List')%></h5>" + 
			"</div>" +
			"<div class='row'>" +						
				"<div class='col-4'>" +
					"<div class='form-group'>" +
						"<label for='" + language_ctrl_id + "' class='required'>" + "<%=_T('Language')%>" + "<%=_T(':')%>" + "</label>" +
						"<select id='" + language_ctrl_id + "' class='form-control'></select>" +
						"<i id='" + language_ctrl_id + "_validation' class='formFieldValidation formFieldError' hidden ></i>" +
					"</div>" +
				"</div>" +
				"<div class='col-4'>" +
					"<div class='form-group'>" +
						"<label for='" + ctrl_id + "' class='required'>" + "<%=_T('Value')%>" + "<%=_T(':')%>" + "</label>" +
						"<input id='" + ctrl_id + "' class='form-control'></input>" +
					"</div>" +
				"</div>" +
			"</div>"; // + 
			//"<div disclaimer='true' class='inlineMessagesContainer messageInfo mt-0'><%=_T('A list component adds additional phrases to match against, and also returns the list key of the match.')%></div>"; 	

		return markup;
	};

	combine_component_children_list.prototype.load = function(callback = $.noop)
	{
		let combine_component_entities = this.__make_children_entities();
		const $children = $(this.children_name_prebuilt);
		this.owner.frm.fill_combo($children, true, combine_component_entities, "__display_name", "__display_name", false);
		
		var $cultures = $("#adm_form_cognitive_services_cultures_for_intents");
		var $options = $cultures.find("option");

		const $language_regular_exp = $(this.children_language_regular_exp);
		$language_regular_exp.empty();
		$language_regular_exp.append($options.clone());

		const $language_list = $(this.children_language_list);
		$language_list.empty();
		$language_list.append($options.clone());
		
		callback();
	};

	combine_component_children_list.prototype.must_add = function(item_value)
	{
		item_value = (item_value || "").trim().toLowerCase();
		// Primero vailda que el Combine Components a agregar ya no exista en la grilla del popup.
		const already_exist = combine_component_children_list.uber.already_exist.call(this, item_value); //invokes the base class method
		if (already_exist) {
			return false;
		}
		// Segundo valida que el Combine Components a agregar ya no se encuentre almacenado en en la app (luis)
		let must_add = true;
		let current_entities = this.owner.get_current_entities() || [];
		for (let i = 0; i < current_entities.length; i++) {
			let current_entity = current_entities[i];
			if (current_entity.typeId === "combineComponents") { // Combine Components
				if (item_value === (current_entity.name || "").trim().toLowerCase()) {
					must_add = false;
					break;
				}
			}
		}
		return must_add;
	};

	combine_component_children_list.prototype.__make_children_entities = function()
	{
		const self = this;

		let available_combine_component_entities = [];
		var add = function(combine_component_entity_id, combine_component_entity) {
			let must_add = self.must_add(combine_component_entity_id);
			if (must_add) {
				let cloned_combine_component_entity = inConcert.Utils.cloneObject(combine_component_entity);
				cloned_combine_component_entity.__display_name = combine_component_entity_id;
				available_combine_component_entities.push(cloned_combine_component_entity);
			}
		};

		let current_combine_component_entities = this.owner.get_current_prebuilt_entities();
		current_combine_component_entities.forEach(current_combine_component_entity => {
			let current_combine_component_entity_id = current_combine_component_entity.name;
			add(current_combine_component_entity_id, current_combine_component_entity);
		});

		return available_combine_component_entities;
	};

	combine_component_children_list.prototype.set_events = function()
	{
		const self = this;

		combine_component_children_list.uber.set_events.call(this); //invokes the base class method

		const $desc = $(this.children_desc_prebuilt);
		const $example = $(this.children_example_prebuilt);
		const $name = $(this.children_name_prebuilt);
		$name
			.off("change")
			.on("change", function() {
				const selected_combine_component_info = self.owner.frm.combo_get_selected_item_data($name);
				let desc = "";
				let examples = "";

				if (selected_combine_component_info) {
					if (selected_combine_component_info.description) {
						desc = self._T(selected_combine_component_info.description);
					}
					if (selected_combine_component_info.examples) {
						examples = self._T(selected_combine_component_info.examples);
					}
				}

				$desc.text(desc);
				$example.text(examples);
			});
	};

	combine_component_children_list.prototype._T = function(value)
	{
		return this.strings[value] || value;
	};

	/*
		Estos strings, son los que retorna el API de LUIS en el metodo "LuisAPI.loadAvailableCombineComponentsEntityList"
		Tengo que agregar los strings aca, ya que estos strings llegan dados de esta forma en el request.
	*/
	combine_component_children_list.prototype.__set_custom_strings = function()
	{
		this.strings = this.strings || {};
		this.strings["Age of a person or thing"] = "<%=_T('Age of a person or thing')%>";
		this.strings["10-month-old, 19 years old, 58 year-old"] = "<%=_T('10-month-old, 19 years old, 58 year-old')%>";
		this.strings["Dates and times, resolved to a canonical form"] = "<%=_T('Dates and times, resolved to a canonical form')%>";
		this.strings["June 23, 1976, Jul 11 2012, 7 AM, 6:49 PM, tomorrow at 7 AM"] = "<%=_T('June 23, 1976, Jul 11 2012, 7 AM, 6:49 PM, tomorrow at 7 AM')%>";
		this.strings["Spacial dimensions, including length, distance, area, and volume"] = "<%=_T('Spacial dimensions, including length, distance, area, and volume')%>";
		this.strings["2 miles, 650 square kilometres, 9,350 feet"] = "<%=_T('2 miles, 650 square kilometres, 9,350 feet')%>";
		this.strings["Email addresses"] = "<%=_T('Email addresses')%>";
		this.strings["user@site.net, user_name@mysite.com.eg, user.Name12@website.net"] = "<%=_T('user@site.net, user_name@mysite.com.eg, user.Name12@website.net')%>";
		this.strings["Automatically extract key phrases to quickly identify the main talking points"] = "<%=_T('Automatically extract key phrases to quickly identify the main talking points')%>";
		this.strings["Paris, cinema, soccer"] = "<%=_T('Paris, cinema, soccer')%>";
		this.strings["Monetary amounts, including currency"] = "<%=_T('Monetary amounts, including currency')%>";
		this.strings["1000.00 US dollars, £20.00, $ 67.5 B"] = "<%=_T('1000.00 US dollars, £20.00, $ 67.5 B')%>";
		this.strings["A cardinal number in numeric or text form"] = "<%=_T('A cardinal number in numeric or text form')%>";
		this.strings["ten, forty two, 3.141, 10K"] = "<%=_T('ten, forty two, 3.141, 10K')%>";
		this.strings["An ordinal number in numeric or text form"] = "<%=_T('An ordinal number in numeric or text form')%>";
		this.strings["first, second, tenth, 1st, 2nd, 10th"] = "<%=_T('first, second, tenth, 1st, 2nd, 10th')%>";
		this.strings['A percentage, using the symbol % or the word "percent"'] = "<%=_T([[A percentage, using the symbol % or the word \'percent\']])%>";
		this.strings["10%, 5.6 percent"] = "<%=_T('10%, 5.6 percent')%>";
		this.strings["US phone numbers"] = "<%=_T('US phone numbers')%>";
		this.strings["123-456-7890, +1 123 456 789, (123)456-789"] = "<%=_T('123-456-7890, +1 123 456 789, (123)456-789')%>";
		this.strings["A temperature in celsius or fahrenheit"] = "<%=_T('A temperature in celsius or fahrenheit')%>";
		this.strings["32F, 34 degrees celsius, 2 deg C"] = "<%=_T('32F, 34 degrees celsius, 2 deg C')%>";
		this.strings["Websites URLs and links"] = "<%=_T('Websites URLs and links')%>";
		this.strings["www.website.com, http://website.net?name=my_name&age=10, https://www.mywebsite.net.eg/page"] = "<%=_T('www.website.com, http://website.net?name=my_name&age=10, https://www.mywebsite.net.eg/page')%>";
	};

	combine_component_children_list.prototype.show = function() {
		const self = this;

		var on_shown = function(success) {
			if (!success) {
				return;
			}
			const $create = $(self.children_create);
			$create.attr("value", $(self.children_create_continue).attr("value"));

			$create.remove();
			$(self.children_close).remove();
			$(self.children_new).remove();
		};
		// visualizo el popup
		this.show_popup(on_shown);
	};

}());
