/*  jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
  strict:true, undef:true, unused:true, curly:true, browser:true, indent:false, maxerr:50, esversion: 11 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	var format = inConcert.Utils.format;
	var sanitize_id = function(id) { return id.replace("#", ""); };

	//---------------------------------------------------------------------
	// form cognitive services class - features_phrases section
	function adm_form_cognitive_services_section_features_phrases(params)
	{
		var success = adm_form_cognitive_services_section_features_phrases.uber.constructor.call(this, params);
		if (!success) {
			return;
		}

		var self = this;

		this.view_action_id  = "view_cognitive_services"; // view action id de permisos
		this.save_action_id  = "save_cognitive_services"; // save action id de permisos
		this.erase_action_id = "save_cognitive_services"; // delete action id de permisos

		this.is_new = null;
		this.current_item = null;
		this.loading = false;

		this.id_btn_new = "adm_form_cognitive_services_features_phrases_new";

		this.form_paginator = new inConcert.conf_form_paginator({
			owner: this,
			frm: this.frm,
			row_id: "id",
			display_name: "name",
			rows_per_page: 11,
			show_at_least: 10,
			ids: {
				paginator: "adm_form_cognitive_services_features_phrases_paginator",
				grid: "adm_form_cognitive_services_features_phrases_list",
				not_results_found: "adm_form_cognitive_services_features_phrases_not_results_found",
				new_item: this.id_btn_new,
				//popup
				popup: "adm_form_cognitive_services_features_phrases_popup",
				popup_msg: "adm_form_cognitive_services_features_phrases_popup_message",
				popup_create: "adm_form_cognitive_services_features_phrases_popup_create",
				popup_create_and_continue: "adm_form_cognitive_services_features_phrases_popup_create_and_continue",
				popup_title: "adm_form_cognitive_services_features_phrases_popup_title",
				popup_close: "adm_form_cognitive_services_features_phrases_popup_close",
				popup_close_times: "adm_form_cognitive_services_features_phrases_popup_close_times",
				popup_input_id: "adm_form_cognitive_services_features_phrases_popup_name"
			},
			strings: {
				not_results_found: "<%=_T('No Features Phrases found.')%>",
				can_remove : "<%=_T('Do you want to delete Feature Phrase: {0} ?')%>",
				title_new: "<%=_T('New Feature Phrase')%>",
				title_update: "<%=_T('Update Feature Phrase')%>",
				button_create             : "<%=_T('Create')%>",
				button_create_and_continue: "<%=_T('Create & Continue')%>",
			},
			columns: [
				{id: "_number"       , name: ""                 , type: "counter"},
				{id: "name"          , name: "<%=_T('Name')%>"                   },
				{id: "phrases"       , name: "<%=_T('Phrase List')%>"            },
				{id: "isExchangeable", name: "<%=_T('Mode')%>", get_value: function(item) {
					return self.get_grid_value(item, "isExchangeable");
				}},
				{id: "isActive"      , name: "<%=_T('Active')%>", type: "boolean", class_name: "booleanColumnTitle"},
				{id: "_action"   , name: "", system_action: {edit: true, erase: true}}
			],
			callbacks: {
				save: function(callback) {
					self.save(callback);
				},
				edit: function(row) {
					self.on_open_popup(row);
				},
				remove: function(row, callback) {
					self.remove(row, callback);
				},
				on_closed_popup: function() {
					self.on_closed_popup();
				},
				load_items: function(page, callback) {
					self.load_items(page, callback /*, paginator_name */);
				}
			}
		});

		this.form_phrases_list = new phrases_list({
			owner    : this,
			prefix_id: "#adm_form_cognitive_services_features_phrases",
			popup_id : "adm_form_cognitive_services_features_phrases_list_popup_phrases"
		});

		return true;
	}

	inConcert.Utils.inherit(adm_form_cognitive_services_section_features_phrases, inConcert.conf_form_section);

	if (typeof inConcert.adm_form_cognitive_services_section_features_phrases !== "function") {
		inConcert.adm_form_cognitive_services_section_features_phrases = adm_form_cognitive_services_section_features_phrases;
	}

	adm_form_cognitive_services_section_features_phrases.prototype.dispose = function()
	{
		// TODO: IMPLEMENTAR
		//this.form_phrases_list.dispose();
	};

	adm_form_cognitive_services_section_features_phrases.prototype.get_grid_value = function(item, field_id)
	{
		var value = {
			original_value: "",
			value         : "",
			search_value  : ""
		};

		var item_value = item[field_id] || "";
		if (item_value === undefined) {
			return value;
		}

		value.original_value = item_value;
		value.value = item_value;
		value.search_value = item_value;

		if (field_id === "isExchangeable") {
			value.value = value.original_value === true ? "<%=_T('Exchangeable')%>" : "<%=_T('Non-Exchangeable')%>";
			value.search_value = value.value;
		}

		return value;
	};

	adm_form_cognitive_services_section_features_phrases.prototype.build_header_custom_buttons = function()
	{
		var html_buttons =
			"<button id='" + this.id_btn_new + "' class='pageButtonGeneric buttonColorPrimary'>" +
				"<%=_T('Add Feature Phrase')%>" +
			"</button>";

		adm_form_cognitive_services_section_features_phrases.uber.build_header_custom_buttons.call(this, html_buttons);
	};

	adm_form_cognitive_services_section_features_phrases.prototype.remove = function(feature_phrase, callback)
	{
		callback = callback || $.noop;
		var self = this;

		if (!feature_phrase) {
			throw("adm_form_cognitive_services_section_features_phrases.prototype.remove. Undefined Feature Phrase.");
		}

		var end = function(success) {
			var type = "Info";
			var msg;
			if (success) {
				msg = format("<%=_T('Feature Phrase {0} was successfully deleted.')%>", feature_phrase.name);
			}
			else {
				msg = format("<%=_T('Error deleting Feature Phrase {0}.')%>", feature_phrase.name);
				type = "Error";
			}
			self.frm.enable_all_sections();
			self.frm.show_status_message(msg, type, 5000);
			self.frm.dashboard.hide_waiting();
			callback(success);
		};

		this.frm.dashboard.show_waiting(format("<%=_T('Deleting Features Phrase {0}.')%>", feature_phrase.name));
		this.disabled_all_sub_menus(true);

		var app_id = this.frm.get_entity_id();
		if (!app_id) {
			end(false);
			return;
		}

		var params = { appId: app_id, phraseId: feature_phrase.id };
		var luisApi = inConcert.app.getLuisAPI();

		luisApi.deleteAppFeaturesPhrase(params,
			function(success) {
				end(success);
			}
		);
	};

	adm_form_cognitive_services_section_features_phrases.prototype.set_events = function()
	{
		this.form_paginator.set_events();
	};

	adm_form_cognitive_services_section_features_phrases.prototype.__show_or_hide_loading_msg = function()
	{
		if (this.is_loading()) {
			this.frm.dashboard.show_waiting("<%=_T('Loading Features Phrases.')%>");
		}
		else {
			this.frm.dashboard.hide_waiting();
		}
	};

	adm_form_cognitive_services_section_features_phrases.prototype.load_items = function(page, callback)
	{
		var self = this;

		callback = callback || $.noop;
		var app_id = this.frm.get_entity_id();
		if (!app_id) {
			callback(false);
			return;
		}

		this.loading = true;
		this.__show_or_hide_loading_msg();

		var paginator_info = this.form_paginator.get_paginator_info();
		var params = {appId: app_id, paginated: {page: page || 1, rows_per_page: paginator_info.rows_per_page}};

		var luisApi = inConcert.app.getLuisAPI();
		luisApi.loadAllFeatures(params,
			function(success, features) {
				self.loading = false;
				if (!success) {
					self.frm.show_status_message("<%=_T('Error retrieving Features.')%>", "Error", false);
					callback(false);
					return;
				}
				callback(true, features.phraselistFeatures);
			});
	};

	adm_form_cognitive_services_section_features_phrases.prototype.initialize = function()
	{
		this.form_paginator.initialize();
		this.form_phrases_list.render();
	};

	adm_form_cognitive_services_section_features_phrases.prototype.set = function()
	{
		this.initialize();
	};

	adm_form_cognitive_services_section_features_phrases.prototype.on_show = function()
	{
		adm_form_cognitive_services_section_features_phrases.uber.on_show.call(this, {show: false});
		this.frm.dashboard.$get_dom_element_by_id("form_buttons_actions").hide();
	};

	adm_form_cognitive_services_section_features_phrases.prototype.set = function()
	{
		this.initialize();
	};

	adm_form_cognitive_services_section_features_phrases.prototype.on_hide = function()
	{
		adm_form_cognitive_services_section_features_phrases.uber.on_hide.call( this );
		this.form_paginator.hide_popup();
	};

	adm_form_cognitive_services_section_features_phrases.prototype.on_open_popup = function(item)
	{
		this.is_new = item ? false: true;
		this.current_item = item;

		var $name = $("#adm_form_cognitive_services_features_phrases_popup_name");
		var $exchangeable = $("#adm_form_cognitive_services_features_phrases_popup_is_exchangeable");
		var $active = $("#adm_form_cognitive_services_features_phrases_popup_is_active");

		if (this.is_new) {
			this.clean_up_controls();
		}
		else {
			$name.val(item.name).focus();
			$exchangeable.prop("checked", item.isExchangeable ? true : false);
			$active.prop("checked", item.isActive ? true : false);
			this.frm.readonly_control($active, false);
			this.form_phrases_list.set(item.phrases);
		}
	};

	adm_form_cognitive_services_section_features_phrases.prototype.on_closed_popup = function()
	{
		this.is_new = null;
		this.current_item = null;
	};

	adm_form_cognitive_services_section_features_phrases.prototype.hide_all_errors = function()
	{
		this.form_paginator.hide_popup_msg();
	};

	adm_form_cognitive_services_section_features_phrases.prototype.show_ctrl_err = function(err_info)
	{
		if (!err_info) {
			return;
		}
		var $control = err_info.$ctrl;
		if ($control) {
			var error_id = $control.attr("id") + "_validation";
			var $error_indicator = $("#" + error_id);
			if ($error_indicator) {
				$error_indicator.show();
			}
			$control.focus();
		}
		this.form_paginator.show_popup_msg(err_info.msg, "Error");
	};

	adm_form_cognitive_services_section_features_phrases.prototype.validate_info_to_save = function()
	{
		this.hide_all_errors();

		var response = {success: false, $ctrl: null, item: null, msg: "<%=_T('Unable to save Features Phrase.')%>"};

		var $name = $("#adm_form_cognitive_services_features_phrases_popup_name");
		var item = {
			name: $name.val()
		};

		var validation_result = inConcert.Utils.validate(item.name, "required");
		if ( !validation_result.success ) {
			// valida que el name tenga al menos un caracter
			response.msg = "<%=_T('Name is required.')%>";
			response.$ctrl = $name;
			this.show_ctrl_err(response);
			return response;
		}
		validation_result = inConcert.Utils.validate(item.name, "id");
		if (!validation_result.success) {
			// valida los caracteres permitidos
			response.msg = "<%=_T('Only letters, numbers, dashes and dots are allowed in the Name.')%>";
			response.$ctrl = $name;
			this.show_ctrl_err(response);
			return response;
		}

		item.phrases = this.form_phrases_list.get();
		item.isExchangeable = $("#adm_form_cognitive_services_features_phrases_popup_is_exchangeable").prop("checked");
		item.isActive = $("#adm_form_cognitive_services_features_phrases_popup_is_active").prop("checked");

		response.success = true;
		response.msg = "";
		response.item = item;

		return response;
	};

	adm_form_cognitive_services_section_features_phrases.prototype.clean_up_controls = function()
	{
		var $name = $("#adm_form_cognitive_services_features_phrases_popup_name");
		var $exchangeable = $("#adm_form_cognitive_services_features_phrases_popup_is_exchangeable");
		var $active = $("#adm_form_cognitive_services_features_phrases_popup_is_active");
		$name.val("").focus();
		$exchangeable.prop("checked", true);
		$active.prop("checked", true);
		this.frm.readonly_control($active, true);
		this.form_phrases_list.clean_up_list();
	};

	adm_form_cognitive_services_section_features_phrases.prototype.save = function(callback)
	{
		var self = this;

		callback = callback || $.noop;

		var response = this.validate_info_to_save();
		if (!response.success) {
			callback(false);
			return;
		}

		var app_id = this.frm.get_entity_id();
		if (!app_id) {
			callback(false);
			return;
		}
		var item = response.item;
		var luisApi = inConcert.app.getLuisAPI();
		var params = { appId: app_id, name: item.name, phrases: item.phrases, isExchangeable: item.isExchangeable, isActive: item.isActive };
		this.disabled_all_sub_menus(true);
		const $full_screen_waiting = $("#genericWaiting");
		let waiting_msg;

		if (this.is_new) {
			waiting_msg = format("<%=_T('Creating Features Phrases {0}.')%>", item.name);
			this.frm.dashboard.show_waiting(waiting_msg, $full_screen_waiting);

			luisApi.createAppFeaturesPhrase(params,
				function(success, result)
				{
					var msg_response = {msg: "", type: ""};
					self.frm.dashboard.hide_waiting($full_screen_waiting);
					self.frm.enable_all_sections();
					if (success) {
						item.id = result.phraseId;
						msg_response.msg = format("<%=_T('Features Phrase {0} was successfully created.')%>", item.name);
						msg_response.type = "Info";
						callback(true, item, msg_response);
						self.clean_up_controls();
					}
					else {
						var server_err_msg = luisApi.getCognitiveErrorMsg(result);
						msg_response.msg = format("<%=_T('Error creating Features Phrase {0}.{1}')%>", item.name, server_err_msg);
						msg_response.type = "Error";
						callback(false, null, msg_response);
					}
				}
			);
		}
		else {
			waiting_msg = format("<%=_T('Updating Features Phrases {0}.')%>", item.name);
			this.frm.dashboard.show_waiting(waiting_msg, $full_screen_waiting);

			params.phraseId = this.current_item.id;

			luisApi.updateAppFeaturesPhrase(params,
				function(success, result)
				{
					var msg_response = {msg: "", type: ""};
					self.frm.dashboard.hide_waiting($full_screen_waiting);
					self.frm.enable_all_sections();
					if (success) {
						$.extend(true, self.current_item, item);
						msg_response.msg = format("<%=_T('Features Phrase {0} was successfully updated.')%>", item.name);
						msg_response.type = "Info";
						callback(true, self.current_item, msg_response);
						self.clean_up_controls();
					}
					else {
						var server_err_msg = luisApi.getCognitiveErrorMsg(result);
						msg_response.msg = format("<%=_T('Error updating Features Phrase {0}.{1}')%>", item.name, server_err_msg);
						msg_response.type = "Error";
						callback(false, null, msg_response);
					}
				}
			);
		}
	};

	adm_form_cognitive_services_section_features_phrases.prototype.disable_actions = function(disable)
	{
		this.form_paginator.disable_actions(disable);
	};

	//---------------------------------------------------------------------
	// class phrases list
	function phrases_list(params)
	{
		var self = this;
		this.owner = params.owner;
		this.prefix_id = params.prefix_id;

		this.phrase_section         = this.prefix_id + "_list_container";
		this.phrase_new             = this.prefix_id + "_list_new_phrase";
		this.phrase_popup           = this.prefix_id + "_list_popup_phrases";
		this.phrase_popup_title     = this.prefix_id + "_list_popup_title";
		this.phrase_create          = this.prefix_id + "_list_phrase_create";
		this.phrase_create_continue = this.prefix_id + "_list_phrase_create_and_continue";
		this.phrase_close           = this.prefix_id + "_list_phrase_close";
		this.phrase_message         = this.prefix_id + "_list_popup_message";
		this.phrase_list            = this.prefix_id + "_list_phrases";
		this.phrase_name            = this.prefix_id + "_list_popup_name";

		this.popup_instance = new inConcert.PopupWindow(
		{
			ownerId: params.popup_id, // container id
			eventName: params.popup_id, // evento click personalizado
			callbacks: {
				onClose: function(){
					self.hide_popup();
				}
			}
		});

	}

	phrases_list.prototype.render = function()
	{
		var sanitized_id_param_new = sanitize_id(this.phrase_new);

		var $html = $("".concat(
			"<i id='", sanitized_id_param_new, "_validation' class='formFieldValidation formFieldError' hidden></i>",
			"<div class='row mbGeneric'>",
				"<div class='col-12'>",
					"<button id='", sanitized_id_param_new, "'  class='pageButtonGeneric buttonColorBlue pageButtonNew float-left'>",
						"<i class='fal fa-plus'></i>",
						"<%=_T('Add Phrase')%>",
					"</button>",
				"</div>",
			"</div>",
			"<div class='card-block' id='", sanitize_id(this.phrase_popup), "' style='display:none;'>",
				"<div class='row mbGeneric'>",
					"<div class='col-12'>",
						"<div class='form-group mb-0'>",
							"<label for='", sanitize_id(this.phrase_name), "' class='required'>", "<%=_T('Phrase')%>", "<%=_T(':')%>", "</label>",
							"<div class='d-flex flex-row'>",
								"<input id='", sanitize_id(this.phrase_name), "' class='form-control' type='text' />",
								"<i id='", sanitize_id(this.phrase_name), "_validation' class='formFieldValidation formFieldError r94' hidden ></i>",

								// "<button id='", sanitize_id(this.phrase_create), "' class='pageButtonGeneric buttonColorWhite mr-2'>",
								// 	"<%=_T('Create')%>",
								// "</button>",
								"<button id='", sanitize_id(this.phrase_create_continue), "' class='pageButtonGeneric buttonColorWhite iconOnlyBtn save align-self-end' title='", "<%=_T('Create & Continue')%>", "'>",
									"<i class='far fa-check-circle'></i>",
								"</button>",
								"<button id='", sanitize_id(this.phrase_close), "' class='pageButtonGeneric buttonColorWhite iconOnlyBtn align-self-end' title='", "<%=_T('Cancel')%>", "'>",
									"<i class='far fa-times-circle'></i>",
								"</button>",
							"</div>",
						"</div>",
					"</div>",
					"<div class='col-12'>",
						"<div id='", sanitize_id(this.phrase_message), "' class='genericFormMessagesContainer messageError mt-3' hidden></div>",
					"</div>",
				"</div>",
			"</div>",
			"<div class='row'>",
				"<div class='col-12'>",
					"<ul id='", sanitize_id(this.phrase_list), "' class='formList formListAsGrid mt-0'>",
						"<li class='formListTitle'>",
							"<label head='true' class='formListTitleText nameColumn'>", "<%=_T('Phrase')%>", "</label>",
						"</li>",
					"</ul>",
				"</div>",
			"</div>",
			"</div>"));

		$(this.phrase_section).html($html);

		this.set_events();
	};

	phrases_list.prototype.dispose = function()
	{
		// TODO: IMPLEMENTAR
		// if (this.popup_instance) {
			// if ( this.popup_instance.isVisible() ) {
				// this.popup_instance.hide();
			// }
			// this.popup_instance = null;
		// }
	};

	phrases_list.prototype.get = function()
	{
		var phrases = [];
		// carga la lista de phrases
		$(this.phrase_list).find("li[item_data]").each(function(_, item_data)
		{
			var $item_data = $(item_data);
			var name = $item_data.find("label[phrase_name]").text();
			phrases.push(name);
		});
		var string_phrases = phrases.join();
		return string_phrases;
	};

	phrases_list.prototype.set = function(string_phrases)
	{
		var self = this;
		this.clean_up_list();
		var phrases = string_phrases.split(",");
		if (!$.isArray(phrases)) {
			return;
		}
		$.each(phrases, function(_, name)
		{
			self.add_row(name);
		});
	};

	phrases_list.prototype.show_popup = function()
	{
		this.hide_errors();

		if ( !this.owner.has_permission_to_section() || !this.owner.has_permission_to_save() ) {
			var message = "<%=_T('Unable to create Phrases: You dont have permissions for this action.')%>";
			this.owner.form_paginator.show_popup_msg( message, "Error", 5000 );
			return;
		}

		this.popup_instance.show();
		// disable actions popup parameters (new, delete, edit)
		this.disable_actions(true);
		//var type;
		if (!this.$row_editing) {
		// si presione el boton para crear uno nuevo
			$(this.phrase_popup_title).text("<%=_T('New Phrase')%>");
			$(this.phrase_create).val("<%=_T('Create')%>");
			$(this.phrase_create_continue).show();
			$(this.phrase_name).focus().val("");
		}
	};

	phrases_list.prototype.clean_up_list = function()
	{
		$(this.phrase_list).find("li[item_data]").remove();
	};

	phrases_list.prototype.hide_popup = function()
	{
		if (this.popup_instance) {
			this.popup_instance.hide();
		}
		// enable actions popup phrases (new, delete)
		this.disable_actions(false);
	};

	phrases_list.prototype.can_add_row = function(/*name*/)
	{
		return true;
	};

	phrases_list.prototype.add_row = function(name, apply_effect)
	{
		if (typeof name !== "string") {
			return;
		}
		if (!this.can_add_row(name)) {
			return;
		}

		name = this.owner.frm.sanitize_value(name);
		var item_value = name.trim().toLowerCase();
		var $make_new_row = function()
		{
			var display = "";
			if (apply_effect) {
				display = "style='display:none;' ";
			}
			return $(
				"<li item_data='true' class='formListItem'" + display + ">" +
					"<label phrase_name='true' class='formItemText nameColumn' item_value='" + name + "'>" + name + "</label>" +
					"<button action_del='true' class='buttonColorGray gridActionButton delete last'>" +
						"<i class='buttons-icons-delete'></i>" +
					"</button>" +
				"</li>");
		};

		var added = false;
		var $list = $(this.phrase_list);
		var $new_row;
		$list.find("li[item_data]").each(function(_, row)
		{
			var $row = $(row);
			var current_item_value = $row.find("label[phrase_name]").attr("item_value");

			if (item_value < current_item_value) {
				$new_row = $make_new_row();
				$row.before($new_row);
				added = true;
				return false; // break
			}
		});
		if (!added) {
			$new_row = $make_new_row();
			$list.append($new_row);
		}
		if (apply_effect && $new_row) {
			// aca realizo el fade in
			$new_row.fadeIn("slow");
		}
	};

	phrases_list.prototype.hide_errors = function()
	{
		$(this.phrase_message).hide();
		$(this.phrase_section).find(".formFieldError").hide();
	};

	phrases_list.prototype.validate = function(name)
	{
		var self = this;

		var show_error = function(message, $control)
		{
			$(self.phrase_message).text(message).show();
			if ($control) {
				var validation_id = $control.attr("id") + "_validation";
				$("#" + validation_id).show();
				$control.focus();
			}
		};
		var message;
		var $name = $(this.phrase_name);
		if (!this.is_valid_data(name, "required")) {
			message = "<%=_T('Please enter the phrase name.')%>";
			show_error(message, $name);
			return false;
		}

		var index = name.indexOf(",");
		if (index > -1) {
			message = "<%=_T('Comma not allowed in Phrases list.')%>";
			show_error(message, $name);
			return false;
		}

		// validation successfully
		return true;
	};

	phrases_list.prototype.create = function( has_continue )
	{
		this.hide_errors();

		if ( !this.owner.has_permission_to_section() || !this.owner.has_permission_to_save() ) {
			var message = "<%=_T('Unable to create Phrases: You dont have permissions for this action.')%>";
			this.owner.form_paginator.show_popup_msg( message, "Error", 5000 );
			this.hide_popup();
			return;
		}

		var name = $(this.phrase_name).val();

		// validate parameters
		var result = this.validate( name );
		if ( !result || ( typeof result === "object" && !result.success ) ) {
			return;
		}

		if (!has_continue) {
			this.hide_popup();
		}

		if (!this.$row_editing) {
		// add new parameter
			var apply_effect = true;
			this.add_row(name, apply_effect);
		}

		$(this.phrase_name).val("");
		if (!has_continue) {
			return;
		}
		$(this.phrase_name).focus();
	};

	phrases_list.prototype.disable_actions = function( disable )
	{
		disable = disable === undefined ? true : disable;

		if (this.owner.is_readonly()) {
			disable = true;
		}

		$(this.phrase_new).prop("disabled", disable);
		var $item_data = $(this.phrase_list).find("li[item_data]");
		// boton delete de la lista de phrases
		$item_data.find("button[action_del]").prop("disabled", disable);
		this.owner.disable_actions(disable);
	};

	phrases_list.prototype.is_valid_data = function(value, validation_type)
	{
		var response = this.owner.frm.validation.valid( value, validation_type );
		return response.success;
	};

	phrases_list.prototype.set_events = function()
	{
		var self = this;
		if ( this.phrase_new === undefined ) {
			return;
		}

		// boton new que abre el popup para poder agregar una nueva phrase
		$(this.phrase_new)
			.off("click")
			.on("click", function() {
				self.$row_editing = null;
				self.show_popup();
			});

		$(this.phrase_create)
			.off("click")
			.on("click", function() {
				var has_continue = false;
				self.create(has_continue);
			});
		$(this.phrase_create_continue)
			.off("click")
			.on("click", function() {
				var has_continue = true;
				self.create(has_continue);
			});
		$(this.phrase_close)
			.off("click")
			.on("click", function() {
				self.hide_popup();
			});
		$(this.phrase_name)
			.off("keypress")
			.on("keypress", function(e) {
				if (e.which === 13) {
					var has_continue = true;
					self.create(has_continue);
				}
			});

		// grilla botones DELETE
		var $param_list = $(this.phrase_list);
		// boton delete de la lista de phrases
		var event_selector = "button[action_del]";
		$param_list
			.off("click", event_selector)
			.on("click", event_selector, function()
			{
				var $parameter = $(this).parent();
				var message = format(
					"<%=_T('Do you want to delete Phrase ({0}) ?')%>",
					$parameter.find("[phrase_name='true']").text()
				);
				$.inconcertPopup.Show(
					"<%=_T('Confirm')%>",
					message,
					$.inconcertPopup.ButtonSets.BS_YESNO,
					function( response )
					{
						if (response.toLowerCase() !== "yes") {
							return;
						}
						if ( !self.owner.has_permission_to_section() || !self.owner.has_permission_to_save() ) {
							var message = "<%=_T('Unable to delete Phrases: You dont have permissions for this action.')%>";
							self.owner.form_paginator.show_popup_msg( message, "Error", 5000 );
							return;
						}
						$parameter.fadeOut("slow", function() {
							$parameter.remove();
						});
					},
					undefined,
					{
						popup_id        : "conf_inconcertPopup",
						container_class : "confirmationPopUp",
						overlay_class   : "confirmationOverlay"
					}
				);
			});
	};
}());
