/*  jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
  strict:true, undef:true, unused:true, curly:true, browser:true, indent:false, maxerr:50, esversion: 11 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	const FORMAT = inConcert.Utils.format;
	const SANITIZE_ID = function(id) { return id.replace("#", ""); };

	const SEPARATOR_HIERARCHICAL = "::";
	const TEMPLATE_HIERARCHICAL = "{0}" + SEPARATOR_HIERARCHICAL + "{1}";

	const SEPARATOR_COMPOSITE = "::";
	const TEMPLATE_COMPOSITE = "{0}" + SEPARATOR_COMPOSITE + "{1}";

	const SEPARATOR_MACHINE_LEARNED = "....";
	const TEMPLATE_MACHINE_LEARNED = "{0}" + SEPARATOR_MACHINE_LEARNED + "{1}";

	let API_VERSION;

	var get_item_name = function(item)
	{
		if (!item) {
			return;
		}
		return item.name || item.entityName;
	};

	let m_entities_by_id = null;
	var find_entity_by_id = function(entity_id)
	{
		if (!entity_id) {
			return;
		}
		if (!m_entities_by_id) {
			return;
		}
		return m_entities_by_id[entity_id.toLowerCase()];
	};

	var find_root_node = null;
	find_root_node = function(current_entity)
	{
		if (!current_entity) {
			return;
		}
		if (!current_entity.__parent_id) {
			return current_entity;
		}
		const parent_entity = find_entity_by_id(current_entity.__parent_id);
		return find_root_node(parent_entity);
	};

	let m_entities_by_name = null;
	var find_entity_by_name = function(entity_name)
	{
		if (!entity_name) {
			return;
		}
		if (!m_entities_by_name) {
			return;
		}
		let entity = m_entities_by_name[entity_name.toLowerCase()];
		if (!entity) {
			return;
		}
		return entity;
	};

	var update_cached_entity = function(entity)
	{
		if (!entity) {
			return;
		}
		if (m_entities_by_id) {
			m_entities_by_id[entity.id.toLowerCase()] = entity;
		}
		if (m_entities_by_name) {
			let entity_name = get_item_name(entity);
			m_entities_by_name[entity_name.toLowerCase()] = entity;
		}
	};

	//---------------------------------------------------------------------
	// form cognitive services class - intents section
	function adm_form_cognitive_services_section_intents(params)
	{
		const success = adm_form_cognitive_services_section_intents.uber.constructor.call(this, params);
		if (!success) {
			return;
		}

		//API_VERSION = inConcert.app.get_luis_api_version();

		const self = this;

		this.view_action_id  = "view_cognitive_services"; // view action id de permisos
		this.save_action_id  = "save_cognitive_services"; // save action id de permisos
		this.erase_action_id = "save_cognitive_services"; // delete action id de permisos

		this.is_new = null;
		this.current_item = null;
		this.loading = false;
		this.m_cached_intents = null;
		this.m_cached_entities = null;

		this.id_btn_new = "adm_form_cognitive_services_intents_new";

		var must_show_button = function(row)
		{
			let show = true;
			let values_to_exclude = {name: "None"};
			for (let name in values_to_exclude) {
				if (values_to_exclude.hasOwnProperty(name)) {
					let value = values_to_exclude[name];
					if (row[name] === value) {
						show = false;
						break;
					}
				}
			}
			return show;
		};
		// Instancia de la clase del Paginado de INTENTS
		this.form_paginator = new inConcert.conf_form_paginator({
			owner: this,
			frm: this.frm,
			row_id: "id",
			display_name: "name",
			rows_per_page: 11,
			show_at_least: 10,
			canEdit: function(row) {
				return must_show_button(row);
			},
			ids: {
				paginator                : "adm_form_cognitive_services_intents_paginator",
				grid                     : "adm_form_cognitive_services_intents_list",
				not_results_found        : "adm_form_cognitive_services_intents_not_results_found",
				new_item                 : this.id_btn_new,
				//popup
				popup                    : "adm_form_cognitive_services_intents_popup",
				popup_msg                : "adm_form_cognitive_services_intents_popup_message",
				popup_create             : "adm_form_cognitive_services_intents_popup_create",
				popup_create_and_continue: "adm_form_cognitive_services_intents_popup_create_and_continue",
				popup_title              : "adm_form_cognitive_services_intents_popup_title",
				popup_close              : "adm_form_cognitive_services_intents_popup_close",
				popup_close_times        : "adm_form_cognitive_services_intents_popup_close_times",
				popup_input_id           : "adm_form_cognitive_services_intents_popup_name"
			},
			strings: {
				not_results_found        : "<%=_T('No Intents found.')%>",
				can_remove               : "<%=_T('Do you want to delete Intent: {0} ?')%>",
				title_new                : "<%=_T('New Intent')%>",
				title_update             : "<%=_T('Update Intent')%>",
				button_create             : "<%=_T('Create')%>",
				button_create_and_continue: "<%=_T('Create & Continue')%>",
			},
			columns: [
				{id: "_number", name: "", type: "counter"    },
				{id: "name"   , name: "<%=_T('Name')%>"      },
				{id: "_action", name: "",
					system_action: {
						edit: {
							must_show: function(row) {
								return must_show_button(row);
							}
						},
						erase: {
							must_show: function(row) {
								return must_show_button(row);
							}
						}
					},
					custom_action: {
						edit_utterances: {
							name: "<%=_T('Utterances')%>",
							execute: function($row /*, $owner*/)
							{
								let intent = $row.data("row_data");
								let count_initialized_list = null;
								self.loading = true;

								// (1) - Se inicia la lista de Utterances
								self.form_utterances_list.set_intent(intent);
								self.form_utterances_list.initialize(function(success)
								{
									if (!success) {
										self.loading = false;
										count_initialized_list = -1;
										//TODO: Ver si hago algo ?
										return;
									}
									if (count_initialized_list === null) {
										count_initialized_list = 1;
									}
								});

								// (2) - Ejecuta la funcion recursiva, con un timeout a la espera que la carga de (1),
								//       haya finalizado. Para asi proceder a realizar la iniciacion de la lista de suggested utterances.
								let init_suggested_utterances_fn;
								let retries;
								let max_retries = 20;

								init_suggested_utterances_fn = function()
								{
									if (max_retries === retries) {
										self.loading = false;
										return;
									}
									if (count_initialized_list === 1) {
										self.form_suggested_utterances_list.set_intent(intent);
										self.form_suggested_utterances_list.initialize(function(success)
										{
											if (!success) {
												self.loading = false;
												//TODO: Ver si hago algo ?
												return;
											}
											if (count_initialized_list === 1) {
												self.loading = false;
												// La primera vez que se inician las 2 grillas, visualizo el TAB de utterances, ya que este callback el jquery.paginator lo invoca tanto cuando se inicia o cuando nos vemos entre las paginas y va al server a buscar info.
												self.show_utterances();
											}
											count_initialized_list = 2;
										});
									}
									else if (count_initialized_list === null) {
										window.setTimeout(function() {
											init_suggested_utterances_fn();
											retries++;
										}, 1000);
									}
									else {
										self.loading = false;
										self.show_utterances();
									}
								};
								// Aca ejecuta la funcion recursiva por primera vez.
								init_suggested_utterances_fn();
							},
							class_name: "buttons-icons-edit-utterances"
						}
					}
				}
			],
			callbacks: {
				save: function(callback) {
					self.save(callback);
				},
				edit: function(row) {
					self.on_open_popup(row);
				},
				remove: function(row, callback) {
					self.remove(row, callback);
				},
				on_closed_popup: function() {
					self.on_closed_popup();
				},
				load_items: function(page, callback) {
					self.load_items(page, callback /*, paginator_name */);
				},
				show_or_hide_loading_msg: function(message) {
					self.show_or_hide_loading_msg(message);
				}
			}
		});
		// Instancia de la clase los TABs
		this.utterances_tabs = new inConcert.conf_form_tab({
			parent         : this,
			frm            : this.frm,
			container_id   : "adm_form_cognitive_services_intents_utterances_tabs_container",
			must_initialize: false,
			tabs_list      : {
				utterances: {
					id              : "utterances",
					name            : "<%=_T('Utterances')%>",
					tab_container_id: "adm_form_cognitive_services_intents_utterances_container",
					selected        : true,
					on_show         : function() {
						// solo se realizara el on_show si el tab container se encuentra visible.
						self.form_utterances_list.on_show();
					}
				},
				suggested: {
					id              : "suggested",
					name            : "<%=_T('Suggested Utterances')%>",
					tab_container_id: "adm_form_cognitive_services_intents_suggested_utterances_container",
					on_show         : function() {
						// solo se realizara el on_show si el tab container se encuentra visible.
						self.form_suggested_utterances_list.on_show();
					}
				}
			}
		});

		// Instancia de la clase de la lista de Utterances
		this.form_utterances_list = new utterances_list({
			owner    : this,
			prefix_id: "#adm_form_cognitive_services_intents_utterances"
		});
		// Instancia de la clase de la lista de Suggested Utterances
		this.form_suggested_utterances_list = new suggested_utterances_list({
			owner    : this,
			prefix_id: "#adm_form_cognitive_services_intents_suggested_utterances"
		});

		return true;
	}

	inConcert.Utils.inherit(adm_form_cognitive_services_section_intents, inConcert.conf_form_section);

	if (typeof inConcert.adm_form_cognitive_services_section_intents !== "function") {
		inConcert.adm_form_cognitive_services_section_intents = adm_form_cognitive_services_section_intents;
	}

	adm_form_cognitive_services_section_intents.prototype.dispose = function()
	{
		this.is_new = null;
		this.current_item = null;
		this.loading = false;
		this.set_cached_entities(null);
		this.set_cached_intents(null);
		this.form_utterances_list.dispose();
		this.form_suggested_utterances_list.dispose();
		this.$get_dom_element_by_id("form_back_btn_container").hide();
	};

	adm_form_cognitive_services_section_intents.prototype.show_utterances = function()
	{
		this.show_or_hide_loading_msg();
		$("#adm_form_cognitive_services_intents_container").hide();
		this.utterances_tabs.show_tabs_container();
		this.utterances_tabs.switch_tab("utterances");
		this.$get_dom_element_by_id("form_back_btn_container").show();
		this.form_suggested_utterances_list.hide();
		this.form_utterances_list.show();
	};

	adm_form_cognitive_services_section_intents.prototype.hide_utterances = function()
	{
		this.form_utterances_list.hide();
		this.form_suggested_utterances_list.hide();
		this.$get_dom_element_by_id("form_back_btn_container").hide();
		this.utterances_tabs.hide_tabs_container();
		$("#adm_form_cognitive_services_intents_container").show();
	};

	/*
		A esta funcion la invoca esta seccion de una seccion externa (ej: train and publish)
		Sirve para que el usuario tenga que ingresar nuevamente a la pantalla de los utterances para validar los nuevos scores.
	*/
	adm_form_cognitive_services_section_intents.prototype.external_update_intents_info = function()
	{
		this.hide_utterances();
	};

	adm_form_cognitive_services_section_intents.prototype.get_grid_value = function(item, field_id)
	{
		var value = {
			original_value: "",
			value         : "",
			search_value  : ""
		};

		var item_value = item[field_id] || "";
		if (item_value === undefined) {
			return value;
		}

		value.original_value = item_value;
		value.value = item_value;
		value.search_value = item_value;

		if (field_id === "utterances") {
			value.value = value.original_value === true ? 0 : 0;
			value.search_value = value.value;
		}

		return value;
	};

	adm_form_cognitive_services_section_intents.prototype.build_header_custom_buttons = function(params)
	{
		const self = this;

		let current_section_id = this.frm.get_current_section_id();

		let html_buttons = "";

		if (current_section_id === "intents") {
			let custom_html = "";
			let button_id, label;
			if (this.form_utterances_list.is_showing()) {
				const selected_tab_id = this.utterances_tabs.get_selected_tab_id();
				if (selected_tab_id === "utterances") {
					params = this.form_utterances_list.make_header_title();
				}
				else if (selected_tab_id === "suggested") {
					params = this.form_suggested_utterances_list.make_header_title();
				}
				else {
					throw("adm_form_cognitive_services_section_intents.prototype.build_header_custom_buttons. Unrecognized TAB " + selected_tab_id);
				}
			}
	
			if (!params) {
				button_id = this.id_btn_new;
				label = "<%=_T('New Intent')%>";
			}
			else {
				button_id = params.button_id;
				label = params.button_label;
				custom_html = params.html;
			}
	
			if (button_id) {
				html_buttons = html_buttons + "<button id='" + button_id + "' class='pageButtonGeneric buttonColorPrimary'>" + label + "</button>";
			}
			if (custom_html) {
				html_buttons = html_buttons + custom_html;
			}
		}

		// invoco a la clase base
		adm_form_cognitive_services_section_intents.uber.build_header_custom_buttons.call(this, html_buttons, function() {
			self.form_paginator.on_header_created();
		});
	};

	adm_form_cognitive_services_section_intents.prototype.remove = function(intent, callback)
	{
		callback = callback || $.noop;
		var self = this;

		if (!intent) {
			throw("adm_form_cognitive_services_section_intents.prototype.remove. Undefined Intent.");
		}

		var end = function(success) {
			var msg;
			if (success) {
				msg = FORMAT("<%=_T('Intent {0} was successfully deleted.')%>", intent.name);
				self.__update_app_version_info(function() {
					self.frm.enable_all_sections();
					self.frm.show_status_message(msg, "Info", 5000);
					self.frm.dashboard.hide_waiting();
				});
			}
			else {
				msg = FORMAT("<%=_T('Error deleting Intent {0}.')%>", intent.name);
				self.frm.enable_all_sections();
				self.frm.show_status_message(msg, "Error", 5000);
				self.frm.dashboard.hide_waiting();
			}
			callback(success);
		};

		this.frm.dashboard.show_waiting(FORMAT("<%=_T('Deleting Intent {0}.')%>", intent.name));
		this.disabled_all_sub_menus(true);

		var app_id = this.frm.get_entity_id();
		if (!app_id) {
			end(false);
			return;
		}

		var params = { appId: app_id, intentId: intent.id };
		var luisApi = inConcert.app.getLuisAPI();
		luisApi.deleteAppIntent(params,
			function(success) {
				end(success);
			}
		);
	};

	adm_form_cognitive_services_section_intents.prototype.set_events = function()
	{
		this.form_paginator.set_events();
	};

	adm_form_cognitive_services_section_intents.prototype.is_loading = function()
	{
		var is_loading = adm_form_cognitive_services_section_intents.uber.is_loading.call(this);
		if (is_loading) {
			return true;
		}
		is_loading = this.form_utterances_list.is_loading();
		if (is_loading) {
			return true;
		}
		is_loading = this.form_suggested_utterances_list.is_loading();
		if (is_loading) {
			return true;
		}
		return false;
	};

	adm_form_cognitive_services_section_intents.prototype.show_or_hide_loading_msg = function(message)
	{
		const is_loading = this.is_loading();
		message = message || "<%=_T('Loading Intents.')%>";
		adm_form_cognitive_services_section_intents.uber.show_or_hide_loading_msg.call(this, message, is_loading);
	};

	adm_form_cognitive_services_section_intents.prototype.load_items = function(page, callback = $.noop)
	{
		const self = this;

		const app_id = this.frm.get_entity_id();
		if (!app_id) {
			callback(false);
			return;
		}
		this.loading = true;
		this.show_or_hide_loading_msg();

		const paginator_info = this.form_paginator.get_paginator_info();
		const params = {appId: app_id, paginated: {page: page || 1, rows_per_page: paginator_info.rows_per_page}};

		if (self.frm && self.frm.cognitive_service && 
				self.frm.cognitive_service.activeVersionInfo && 
				self.frm.cognitive_service.activeVersionInfo.intents) {

				self.loading = false;
				callback(true, self.frm.cognitive_service.activeVersionInfo.intents);

		} else {
			const luisApi = inConcert.app.getLuisAPI();
			luisApi.loadAllIntents(params,
				function(success, intents) {
					self.loading = false;
					if (!success) {
						self.frm.show_status_message("<%=_T('Error retrieving Intents.')%>", "Error", false);
						callback(false);
						return;
					}
					callback(true, intents);
				});
		}
	};

	adm_form_cognitive_services_section_intents.prototype.initialize = function()
	{
		this.form_paginator.initialize();
		this.utterances_tabs.initialize();
	};

	adm_form_cognitive_services_section_intents.prototype.set = function()
	{
		API_VERSION = this.frm.cognitive_service ? this.frm.cognitive_service.api_version : API_VERSION;
		this.initialize();
	};

	adm_form_cognitive_services_section_intents.prototype.on_show = function()
	{
		API_VERSION = this.frm.cognitive_service ? this.frm.cognitive_service.api_version : API_VERSION;
		var params = {
			show: false
		};
		var info;
		let is_showing = false;
		if (this.form_utterances_list.is_showing()) {
			info = this.form_utterances_list.make_header_title();
			params.section_title = info.section_title;
			is_showing = true;
		}
		else if (this.form_suggested_utterances_list.is_showing()) {
			info = this.form_suggested_utterances_list.make_header_title();
			params.section_title = info.section_title;
			
			is_showing = true;
		}
		adm_form_cognitive_services_section_intents.uber.on_show.call(this, params);
		this.frm.dashboard.$get_dom_element_by_id("form_buttons_actions").hide();

		if (is_showing) {
			this.$get_dom_element_by_id("form_back_btn_container").show();
		}
	};

	adm_form_cognitive_services_section_intents.prototype.on_hide = function()
	{
		adm_form_cognitive_services_section_intents.uber.on_hide.call(this);
		this.set_cached_entities(null);
		this.set_cached_intents(null);
		this.form_paginator.hide_popup();
	};

	adm_form_cognitive_services_section_intents.prototype.set_cached_entities = function(entities)
	{
		this.m_cached_entities = entities;
		m_entities_by_id = null;
		m_entities_by_name = null;

		if (!entities) {
			return;
		}

		let entities_by_id = {};
		let entities_by_name = {};

		var append = null;
		append = function(entity, parent_entity, parent_ml_name) {
			if (parent_entity) {
				entity.__parent_id = parent_entity.id;
			}
			if (entity.id) {
				entities_by_id[entity.id.toLowerCase()] = entity;
			}
			if (entity.name) {
				let entity_name = get_item_name(entity);
				if (parent_entity) {
					let parent_entity_name = get_item_name(parent_entity);
					//parent_entity.typeId
					if (parent_ml_name !== undefined) {
						if (parent_ml_name.length > 0) {
							parent_ml_name = parent_ml_name;
						}
						else {
							parent_ml_name = parent_entity_name;
						}

						entity_name = parent_ml_name + "_" + entity_name;
						parent_ml_name = entity_name;
						entity.__full_tree_name = entity_name;
					}
					else {
						entity_name = FORMAT(TEMPLATE_HIERARCHICAL, parent_entity_name, entity_name);
						entity.typeId = "3"; // Tengo que agregar el atributo HIERARCHICAL, ya que en la data que retorna LUIS no viene. Y lo necesito!!
					}
				}
				entities_by_name[entity_name.toLowerCase()] = entity;
			}
			return parent_ml_name;
		};

		var fill = function(entities_list, parent_entity, parent_ml_name)
		{
			(entities_list || []).forEach(entity => {
				let new_parent_ml_name = append(entity, parent_entity, parent_ml_name);
				/* Solo para los HIERARCHICAL se deben mapear los hijos.
					Ya que los COMPOSITE ya aparecen en la lista discreta.
				*/

				const entity_type_id = (entity.typeId || "").toString();
				if (entity_type_id === "3") {
					(entity.children || []).forEach(child => {
						append(child, entity);
					});
				}
				else if (entity_type_id === "1") { // MACHINE LEARNER
					if (new_parent_ml_name === undefined) {
						new_parent_ml_name = "";
					}
					(entity.children || []).forEach(child => {
						let child_parent_ml = append(child, entity, new_parent_ml_name);
						fill(child.children, entity, child_parent_ml);
					});
				}
			});
		};

		fill(entities);

		m_entities_by_id = entities_by_id;
		m_entities_by_name = entities_by_name;
	};

	adm_form_cognitive_services_section_intents.prototype.set_cached_intents = function(intents)
	{
		this.m_cached_intents = intents;
	};

	adm_form_cognitive_services_section_intents.prototype.load_entities = function(callback = $.noop)
	{
		const self = this;

		if (this.m_cached_entities) {
			callback(true, this.m_cached_entities);
			return;
		}

		const app_id = this.frm.get_entity_id();
		const params = {appId: app_id};

		const luisApi = inConcert.app.getLuisAPI();
		luisApi.loadAllEntities(params,
			function(success, entities) {
				if (!success) {
					self.frm.show_status_message("<%=_T('Error retrieving All Entities.')%>", "Error", false);
					callback(false);
					return;
				}
				self.set_cached_entities(entities);
				callback(true, entities);
			});
	};

	adm_form_cognitive_services_section_intents.prototype.load_intents = function(callback = $.noop)
	{
		const self = this;

		if (this.m_cached_intents) {
			callback(true, this.m_cached_intents);
			return;
		}

		const app_id = this.frm.get_entity_id();
		const params = {appId: app_id};

		const luisApi = inConcert.app.getLuisAPI();
		luisApi.loadAllIntents(params,
			function(success, intents) {
				if (!success) {
					self.frm.show_status_message("<%=_T('Error retrieving All Intents.')%>", "Error", false);
					callback(false);
					return;
				}
				self.set_cached_intents(intents);
				callback(true, intents);
			});
	};

	adm_form_cognitive_services_section_intents.prototype.on_open_popup = function(intent)
	{
		this.is_new = intent ? false: true;
		this.current_item = intent;

		$("#adm_form_cognitive_services_intents_popup_name").focus().val(this.is_new ? "" : intent.name);
	};

	adm_form_cognitive_services_section_intents.prototype.on_closed_popup = function()
	{
		this.is_new = null;
		this.current_item = null;
	};

	adm_form_cognitive_services_section_intents.prototype.hide_all_errors = function()
	{
		this.form_paginator.hide_popup_msg();
	};

	adm_form_cognitive_services_section_intents.prototype.show_ctrl_err = function(err_info)
	{
		if (!err_info) {
			return;
		}
		var $control = err_info.$ctrl;
		if ($control) {
			var error_id = $control.attr("id") + "_validation";
			var $error_indicator = $("#" + error_id);
			if ($error_indicator) {
				$error_indicator.show();
			}
			$control.focus();
		}
		this.form_paginator.show_popup_msg(err_info.msg, "Error");
	};

	adm_form_cognitive_services_section_intents.prototype.validate_info_to_save = function()
	{
		this.hide_all_errors();

		var response = {success: false, $ctrl: null, item: null, msg: "<%=_T('Unable to save Intent.')%>"};

		var $name = $("#adm_form_cognitive_services_intents_popup_name");
		var intent = {
			name: $name.val()
		};

		var validation_result = inConcert.Utils.validate(intent.name, "required");
		if ( !validation_result.success ) {
			// valida que el name tenga al menos un caracter
			response.msg = "<%=_T('Name is required.')%>";
			response.$ctrl = $name;
			this.show_ctrl_err(response);
			return response;
		}
		validation_result = inConcert.Utils.validate(intent.name, "id");
		if (!validation_result.success) {
			// valida los caracteres permitidos
			response.msg = "<%=_T('Only letters, numbers, dashes and dots are allowed in the Name.')%>";
			response.$ctrl = $name;
			this.show_ctrl_err(response);
			return response;
		}

		response.success = true;
		response.msg = "";
		response.item = intent;

		return response;
	};

	adm_form_cognitive_services_section_intents.prototype.clean_up_controls = function()
	{
		$("#adm_form_cognitive_services_intents_popup_name").val("").focus();
	};

	adm_form_cognitive_services_section_intents.prototype.__update_app_version_info = function(callback)
	{
		var app_section = this.frm.get_section_def("application");
		app_section.frm.update_app_version_info(undefined, callback);
	};

	adm_form_cognitive_services_section_intents.prototype.save = function(callback)
	{
		var self = this;

		callback = callback || $.noop;

		var response = this.validate_info_to_save();
		if (!response.success) {
			callback(false);
			return;
		}

		var app_id = this.frm.get_entity_id();
		if (!app_id) {
			callback(false);
			return;
		}
		var item = response.item;
		var luisApi = inConcert.app.getLuisAPI();
		var params = { appId: app_id, name: item.name };

		this.disabled_all_sub_menus(true);

		if (this.is_new) {
			this.frm.dashboard.show_waiting(FORMAT("<%=_T('Creating Intents {0}.')%>", item.name));

			luisApi.createAppIntent(params,
				function(success, result)
				{
					var msg_response = {msg: "", type: ""};
					if (success) {
						item.id = result.intentId;
						msg_response.msg = FORMAT("<%=_T('Intent {0} was successfully created.')%>", item.name);
						msg_response.type = "Info";
						callback(true, item, msg_response);
						self.clean_up_controls();
						self.__update_app_version_info(function() {
							self.frm.dashboard.hide_waiting();
							self.frm.enable_all_sections();
						});
					}
					else {
						self.frm.dashboard.hide_waiting();
						self.frm.enable_all_sections();
						var server_err_msg = luisApi.getCognitiveErrorMsg(result);
						msg_response.msg = FORMAT("<%=_T('Error creating Intent {0}.{1}')%>", item.name, server_err_msg);
						msg_response.type = "Error";
						callback(false, null, msg_response);
					}
				}
			);
		}
		else {
			this.frm.dashboard.show_waiting(FORMAT("<%=_T('Updating Intents {0}.')%>", item.name));

			params.intentId = this.current_item.id;

			luisApi.renameAppIntent(params,
				function(success, result)
				{
					var msg_response = {msg: "", type: ""};
					self.frm.dashboard.hide_waiting();
					self.frm.enable_all_sections();
					if (success) {
						$.extend(true, self.current_item, item);
						msg_response.msg = FORMAT("<%=_T('Intent {0} was successfully updated.')%>", item.name);
						msg_response.type = "Info";
						callback(true, self.current_item, msg_response);
						self.clean_up_controls();
					}
					else {
						var server_err_msg = luisApi.getCognitiveErrorMsg(result);
						msg_response.msg = FORMAT("<%=_T('Error updating Intent {0}.{1}')%>", item.name, server_err_msg);
						msg_response.type = "Error";
						callback(false, null, msg_response);
					}
				}
			);
		}
	};

	//---------------------------------------------------------------------
	// class utterances list
	function utterances_list(params)
	{
		this.name = "utterances";
		this.owner = params.owner;
		this.prefix_id = params.prefix_id;

		this.utterance_section = this.prefix_id + "_container";
		this.utterance_new     = this.prefix_id + "_new_utterance";

		this.is_new = null;
		this.current_item = null;
		this.loading = false;
		this.intent = null;
		this.showing = false;
		// Aca mantenemos la lista de palabras seleccionadas que tiene el menu en el cual se asignaran labels/entities.
		this.$selected_words = {};

		var html_title_intent = "<span>{0}</span>";
		this.header_title = FORMAT("<%=_T('Intent {0} / Utterances')%>", html_title_intent);
		this.string_loading = "<%=_T('Loading Utterances.')%>";
		this.string_deletes_success = "<%=_T('Utterance {0} was successfully deleted.')%>";
		this.string_deletes_error   = "<%=_T('Error deleting Utterance {0}.')%>";
		this.string_deletes_waiting = "<%=_T('Deleting Utterance {0}.')%>";
		/*
			En este atributo es donde se encuentran los index de los UTTERANCES etiquetados.
			A partir de el se realiza el proceso de etiquetado de las palabras del UTTERANCE.
		*/
		this.attr_make_utterance_labels = "entityLabels";
		/*
			En este atributo es donde se encuentran los index de los UTTERANCES entity PREDICTIONS.
		*/
		this.attr_make_utterance_predictions = "entityPredictions";

		this.m_entities_types = {
			// SIMPLE (Api 2.0) o MACHINE LEARNED (Api 3.0-preview)
			"1": {
				id: 1,
				name: API_VERSION === "2.0" ? "SIMPLE" : "MACHINE LEARNED",
				item_class: API_VERSION === "2.0" ? "EntitySimple" : "EntityMachineLearned"
			},
			// PREBUILT
			"2": {
				id: 2,
				name: "PREBUILT",
				item_class: "EntityPrebuilt"
			},
			// HIERARCHICAL
			"3": {
				id: 3,
				name: "HIERARCHICAL",
				item_class: "EntityHierarchical"
			},
			// COMPOSITE
			"4": {
				id: 4,
				name: "COMPOSITE",
				item_class: "EntityComposite"
			}
		};

		this.form_paginator = null;
		this.__new_grid();

		this.menu_entities = null;
		this.menu_intents = null;
	}

	utterances_list.prototype.on_closed_popup = function()
	{
		var $tokenized = $(this.prefix_id + "_popup_tokenized");
		$tokenized.html("");
	};

	utterances_list.prototype.__new_grid = function()
	{
		var self = this;

		var sanitized_prefix_id = SANITIZE_ID(this.prefix_id);
		// Instancia de la clase del Paginado de Utterances
		this.form_paginator = new inConcert.conf_form_paginator({
			owner        : this,
			frm          : this.owner.frm,
			row_id       : "id",
			display_name : "text",
			rows_per_page: 11,
			show_at_least: 10,
			ids: {
				new_item                  : SANITIZE_ID(this.utterance_new),
				paginator                 : sanitized_prefix_id + "_paginator",
				grid                      : sanitized_prefix_id + "_list",
				not_results_found         : sanitized_prefix_id + "_not_results_found",
				//popup
				popup                     : sanitized_prefix_id + "_popup",
				popup_msg                 : sanitized_prefix_id + "_popup_message",
				popup_create              : sanitized_prefix_id + "_popup_create",
				popup_create_and_continue : sanitized_prefix_id + "_popup_create_and_continue",
				popup_title               : sanitized_prefix_id + "_popup_title",
				popup_close               : sanitized_prefix_id + "_popup_close",
				popup_close_times         : sanitized_prefix_id + "_popup_close_times",
				popup_input_id            : sanitized_prefix_id + "_popup_text",
				//search
				search_container          : sanitized_prefix_id + "_search_container",
				search_text               : sanitized_prefix_id + "_search_text",
				search_reset              : sanitized_prefix_id + "_reset_text",
				search_apply              : sanitized_prefix_id + "_apply_text"
			},
			strings: {
				not_results_found         : "<%=_T('No Utterances found.')%>",
				can_remove                : "<%=_T('Do you want to delete Utterance: {0} ?')%>",
				title_new                 : "<%=_T('New Utterance')%>",
				title_update              : "<%=_T('Update Utterance Labels')%>",
				button_create             : "<%=_T('Add')%>",
				button_create_and_continue: "<%=_T('Add & Continue')%>",
				button_update             : "<%=_T('Update')%>"
			},
			columns: [
				{id: "_number"      , name: "", type: "counter"},
				{id: "tokenizedText", name: "<%=_T('Utterances Text')%>",
					get_tooltip: function(item) {
						return item.text;
					},
					get_value: function(item) {
						return self.get_grid_value(item, "tokenizedText");
					}
				},
				{id: "value_of_another_intent", name: "<%=_T('Suggested Intent')%>",
					no_search: true,
					get_value: function(item) {
						return self.get_grid_value(item, "value_of_another_intent");
					}
				},
				{id: "predicated_intent", name: "<%=_T('Predicated Intent')%>",
					no_search: true,
					get_value: function(item) {
						return self.get_grid_value(item, "predicated_intent");
					}
				},
				{id: "dataset", name: "<%=_T('Dataset')%>",
					no_search: true,
					get_value: function(item) {
						return self.get_grid_value(item, "dataset");
					}
				},
				{id: "language", name: "<%=_T('Language')%>",
					no_search: true,
					get_value: function(item) {
						return self.get_grid_value(item, "language");
					}
				},
				{
					id: "_action",
					name: "",
					system_action: {edit: {label: "<%=_T('Labels')%>", class_name: "buttons-icons-utterances-labels"}, erase: true},
					custom_action: {
						reassign_intent: {
							name: "<%=_T('Reassign Intent')%>",
							execute: function($row, $owner)
							{
								var current_intent = $row.data("row_data");
								self.__open_menu_intents($row, $owner, current_intent);
							},
							class_name: "buttons-icons-reassign-intent"
						}
					}
				}
			],
			callbacks: {
				save: function(callback) {
					self.save(callback);
				},
				edit: function(row) {
					self.on_open_popup(row);
				},
				remove: function(row, callback) {
					self.remove(row, callback);
				},
				on_closed_popup: function() {
					self.on_closed_popup();
				},
				load_items: function(page, callback) {
					self.load_items(page, callback /*, paginator_name */);
				},
				show_or_hide_loading_msg: function(message) {
					self.show_or_hide_loading_msg(message);
				}
			}
		});
	};

	utterances_list.prototype.initialize = function(callback)
	{
		this.form_paginator.initialize(callback);
	};

	utterances_list.prototype.is_loading = function()
	{
		return this.loading;
	};

	utterances_list.prototype.on_reassigned_utterance = function(callback)
	{
		callback = callback || $.noop;
		var self = this;
		this.form_paginator.refresh_page(function() {
			self.form_paginator.hide_not_results_found();
			callback();
		});
	};

	utterances_list.prototype.dispose = function()
	{
		this.is_new = null;
		this.current_item = null;
		this.loading = false;
		this.intent = null;
		this.showing = false;
		this.menu_entities = null;
		this.menu_intents = null;
		this.$selected_words = {};
		this.form_paginator.dispose();
		this.owner.$get_dom_element_by_id("form_back_btn_container").hide();
	};

	utterances_list.prototype.is_showing = function()
	{
		return this.showing;
	};

	utterances_list.prototype.show = function()
	{
		$(this.utterance_section).show();
		this.on_show();
	};

	utterances_list.prototype.on_show = function()
	{
		if (!this.owner.utterances_tabs.is_visible_tabs_container()) {
			return;
		}
		this.showing = true;
		var params = this.make_header_title();
		this.owner.build_header_custom_buttons(params);
		this.set_events();
	};

	utterances_list.prototype.make_header_title = function()
	{
		var utterances_section_title = this.intent ? FORMAT(this.header_title, this.intent.name) : "";
		this.owner.set_section_title(utterances_section_title);

		return {
			button_id    : SANITIZE_ID(this.utterance_new),
			button_label : "<%=_T('New Utterance')%>",
			section_title: utterances_section_title,
			html         : this.form_paginator.get_header_search_html()
		};
	};

	utterances_list.prototype.hide = function()
	{
		$(this.utterance_section).hide();
		this.on_hide();
	};

	utterances_list.prototype.on_hide = function()
	{
		this.showing = false;
		this.form_paginator.hide_popup();
		this.owner.build_header_custom_buttons();
		this.owner.set_section_title();
	};

	utterances_list.prototype.__on_assigned_label = function(entity_label, parent_entity_label, callback = $.noop)
	{
		this.__add_label_to_selected_words(entity_label, parent_entity_label);
		this.__unmark_selected_words();
		callback();
	};

	utterances_list.prototype.load_entities = function(loaded_fn = $.noop)
	{
		this.owner.load_entities(function(success, entities) {
			if (!success) {
				loaded_fn(false);
				return;
			}
			loaded_fn(true, entities);
		});
	};

	utterances_list.prototype.__cleanup_entities = function(all_entities)
	{
		let entities = [];
		(all_entities || []).forEach(entity => {
			// PREBUILT (Este tipo de entidades no se pueden asignar de forma manual)
			if (entity.typeId !== 2) {
				entities.push(entity);
			}
		});
		return entities;
	};

	utterances_list.prototype.__open_menu_entities = function($word)
	{
		const self = this;

		let menu_callbacks = {
			load: function(loaded_fn = $.noop) {
				self.load_entities(function(success, all_entities) {
					if (!success) {
						loaded_fn(false);
						return;
					}
					var must_add_item = function(entity)
					{
						// PREBUILT (Este tipo de entidades no se pueden asignar de forma manual)
						if (!entity) {
							return false;
						}
						if (entity.typeId === 2) {
							return false;
						}
						if (entity.typeId === undefined) {
							let entity_name = get_item_name(entity);
							let entity_def = find_entity_by_name(entity_name);
							if (!entity_def || entity_def.typeId === 2) {
								return false;
							}
						}
						return true;
					};
					let entities = self.__cleanup_entities(all_entities);
					loaded_fn(true, entities, $word, must_add_item);
				});
			},
			on_success_assigned: function(entity_label, parent_entity_label, on_assigned) {
				self.__on_assigned_label(entity_label, parent_entity_label, on_assigned);
			},
			on_success_removed: function(on_removed = $.noop) {
				self.__remove_label_from_selected_words();
				self.__unmark_selected_words();
				on_removed();
			},
			on_error_assigned: function() {
				//TODO: HACER ALGO ?
			},
		// Calbacks invocados de la clase Base PopupWindow
			onOpen: function() {
				self.__unselect_all_words();
				self.__select_word($word);
				self.__mark_words();
			},
			onClose: function(params) {
				self.__unmark_selected_words();
				params.onClosedFn();
			},
			onNotAutomaticClose: function() {
				// Este callback se invoca cuando el menu no se ha cerrado de forma automatica, cuando se ha dado click fuera de este.
				// Al invocar a la funcion "setClosedAutomatic" con el valor en true, estamos dejando nuevamente el menu para que se cierre de forma automatica.
				self.menu_entities.setClosedAutomatic(true);
			}
		};

		this.menu_entities = new menu_labels_entities({from_app: "cognitive_services", item_class: menu_labels_utterances_entities});
		this.menu_entities.show({
			parentNode  : $word,
			callbacks   : menu_callbacks,
			owner_object: this
		});
	};

	utterances_list.prototype.__open_menu_intents = function($utterance, $owner, current_intent)
	{
		const self = this;

		let utterance = $utterance.data("row_data");

		let menu_callbacks = {
			load: function(loaded_fn)
			{
				loaded_fn = loaded_fn || $.noop;
				self.owner.load_intents(function(success, intents) {
					if (!success) {
						loaded_fn(false);
						return;
					}
					var must_add_item = function(intent_to_add)
					{
						if (!current_intent) {
							return true;
						}
						if (current_intent.intentLabel !== intent_to_add.name) {
							return true;
						}
						return false;
					};
					loaded_fn(true, intents, $utterance, must_add_item);
				});
			},
			on_success_assigned: function(intent, _, on_assigned) {
				on_assigned();
				self.reassign_intent(intent, utterance, function(success) {
					if (success) {
						self.form_paginator.on_removed_row($utterance);
					}
				});
			},
			on_error_assigned: function() {
				//TODO: HACER ALGO ?
			}
		};

		this.menu_intents = new menu_labels_intents({from_app: "cognitive_services", item_class: menu_labels_utterances_intents});
		this.menu_intents.show({
			parentNode  : $owner,
			callbacks   : menu_callbacks,
			owner_object: this
		});
	};

	/*
		Funcion que genera el titulo del menu, a paritr de las palabras seleccionadas.
	*/
	utterances_list.prototype.__set_menu_title = function()
	{
		var title = "";
		$.each(this.$selected_words, function(_, $word) {
			if (title.length > 0) {
				title = title + " ";
			}
			title = title + $word.attr("word");
		});
		this.menu_entities.set_menu_title(title);
	};

	utterances_list.prototype.__word_is_labeled = function($word)
	{
		const entity_label = $word.data("entity_label");
		const is_labeled = (entity_label !== null && entity_label !== undefined);
		return is_labeled;
	};

	utterances_list.prototype.__word_is_predicted = function($word)
	{
		const entity_prediction = $word.data("entity_prediction");
		const is_predicted = (entity_prediction !== null && entity_prediction !== undefined);
		return is_predicted;
	};

	utterances_list.prototype.__word_is_labeled_or_predicted = function($word)
	{
		const is_labeled = this.__word_is_labeled($word);
		if (is_labeled) {
			return true;
		}
		const is_predicted = this.__word_is_predicted($word);
		if (is_predicted) {
			return true;
		}
		return false;
	};

	/*
		Funcion que marca las palabras seleccionadas con Corchetes, <span>, etc.
		Tanto en el evento "over", como cuando se da "click" en una de estas, teniendo el menu abierto.
	*/
	utterances_list.prototype.__mark_words = function($current_word)
	{
		const self = this;

		// 1 - Si el menu fue abierto con una palabra con LABEL o PREDICTION, No puedo marcar la palabra
		if (this.menu_entities) {
			const assigned_label = this.menu_entities.get_assigned_label();
			if (assigned_label) {
				return;
			}
			const assigned_prediction = this.menu_entities.get_assigned_prediction();
			if (assigned_prediction) {
				return;
			}
		}

		// Aca marcamos la palabra
		var mark = function($w, start, end, type) {
			let start_mark = start ? "[" : "";
			let end_mark = end ? "]" : "";
			let word = $w.attr("word");
			let class_start_mark = "";
			let class_end_mark = "";

			if (type === "SELECTED") {
				class_start_mark = "utteranceSelectedStartMark";
				class_end_mark = "utteranceSelectedEndMark";
			}
			else if (type === "SUGGESTED") {
				if (start) {
					class_start_mark = "utteranceSuggestedStartMark";
				}
				if (end) {
					class_end_mark = "utteranceSuggestedEndMark";
				}
			}
			$w.html("<span class='" + class_start_mark + "'>" + start_mark + "</span>" + word + "<span class='" + class_end_mark + "'>" + end_mark + "</span>")
				.addClass("over");
		};

		let total_selected_words = this.__count_selected_words();
		// Si recibo la palabra como parametro, tengo que validar que la misma no tenga una etiqueta/entidad asociada.

		if ($current_word) {
		// Aca ingresa solo en el evento MOUSEOVER, PARA MARCAR LA PALABRA COMO SUGERIDA.

			// Si tiene una entidad (labeled o predicated), no marco la misma con corchetes.
			const is_labeled_or_predicted = this.__word_is_labeled_or_predicted($current_word);
			if (is_labeled_or_predicted) {
				return;
			}
			// Si el popup se encuentra abierto, valida si la palabra puede ser marcada.
			let is_opened_menu_labels = this.__is_opened_menu_labels();
			if (is_opened_menu_labels) {
				const result = this.__can_add_words_to_menu_labels($current_word);
				if (!result.must_add) {
					return;
				}
			}
			let tokenized_current_word = parseInt($current_word.attr("tokenized_index"), 10);
			let current_word_start_mark;
			let current_word_end_mark;

			if (total_selected_words > 0) {
				$.each(this.$selected_words, function(tokenized_selected_word)
				{
					tokenized_selected_word = parseInt(tokenized_selected_word, 10);
					if (current_word_start_mark === undefined) {
						if (tokenized_current_word < tokenized_selected_word) {
							// Agrego corchete al inicio de la palabra y NO agrego chorchete al final de la palabra sugerida.
							current_word_start_mark = true;
							current_word_end_mark = false;
						}
						else if (tokenized_current_word > tokenized_selected_word) {
							// NO Agrego corchete al inicio de la palabra y SI agrego chorchete al final de la palabra sugerida.
							current_word_start_mark = false;
							current_word_end_mark = true;
						}
						else {
							// NO Agrego corchetes al inicio ni al final de la palabra sugerida.
							current_word_start_mark = false;
							current_word_end_mark = false;
						}
					}
				});
			}
			else {
			// Agrego corchetes de inicio y de final a la palabra sugerida.
				current_word_start_mark = true;
				current_word_end_mark = true;
			}

			// Marco la palabra actual.
			mark($current_word, current_word_start_mark, current_word_end_mark, "SUGGESTED");
		}

		// Si tengo palabras seleccionadas, es decir si ya se encuentra el menu abierto marco las mismas.
		if (total_selected_words > 0) {
			let i = 0;
			$.each(this.$selected_words, function(tokenized_selected_word, $selected_word)
			{
				if (total_selected_words === 1) {
					// Si solo tengo una palabra seleccionada, marco corchete abierto y corchete cerrado.
					mark($selected_word, true, true, "SELECTED");
				}
				else {
					// Si tengo mas de una palabra seleccionada
					i++;
					if (i === 1) {
						// Es la primer palabra, marco corchete abierto.
						mark($selected_word, true, false, "SELECTED");
					}
					else if (i === total_selected_words) {
						// Es la ultima palabra marco corchete cerrado.
						mark($selected_word, false, true, "SELECTED");
					}
					else {
						// Si es una palabra del medio la desmarco.
						self.__unmark_word($selected_word);
					}
				}
			});
		}
	};

	utterances_list.prototype.__unmark_word = function($word)
	{
		let is_labeled_or_predicted = this.__word_is_labeled_or_predicted($word);
		if (is_labeled_or_predicted) {
			return;
		}
		const word = $word.attr("word");
		$word.html(word).removeClass("over");
	};

	utterances_list.prototype.__unmark_selected_words = function()
	{
		var self = this;
		$.each(this.$selected_words, function(_, $selected_word) {
			self.__unmark_word($selected_word);
		});
		this.__unselect_all_words();
	};

	utterances_list.prototype.set_events = function()
	{
		const self = this;

		$(this.owner.$get_dom_element_by_id("form_back_btn"))
			.off("click")
			.on("click", function() {
				self.owner.hide_utterances();
			});

		const $tokenized = $(this.prefix_id + "_popup_tokenized");
		const event_selector = "[word]";

		$tokenized
			.off("mouseenter", event_selector)
			.on("mouseenter", event_selector, function()
			{
				const $word = $(this);
				self.__mark_words($word);
			})
			.off("mouseleave", event_selector)
			.on("mouseleave", event_selector, function()
			{
				const $word = $(this);
				if (!self.__is_opened_word($word)) {
					self.__unmark_word($word);
				}
			})
			.off("click", event_selector)
			.on("click", event_selector, function() {
				const $word = $(this);
				self.__open_menu_labels($word);
			});

		this.form_paginator.set_events();
	};

	/*
		Funcion que valida si al menu abierto, se le pueden agregar mas palabras.
	*/
	utterances_list.prototype.__can_add_words_to_menu_labels = function($current_word)
	{
		let response = {
			must_add: false,
			$words  : []
		};

		if (!this.menu_entities) {
			response.must_add = false;
			return response;
		}
		// 1 - Si el menu fue abierto con una palabra LABEL o PREDICTION, NO dejo abrir el menu ME VOY
		const assigned_label = this.menu_entities.get_assigned_label();
		if (assigned_label) {
			response.must_add = false;
			return response;
		}
		const assigned_prediction = this.menu_entities.get_assigned_prediction();
		if (assigned_prediction) {
			response.must_add = false;
			return response;
		}
		// 2 - Si la palabra que se va a etiquetar, ya se encuentra etiquetada, NO dejo abrir el menu ME VOY
		let is_labeled_or_predicted = this.__word_is_labeled_or_predicted($current_word);
		if (is_labeled_or_predicted) {
			return;
		}

		// Obtengo la primer y ultima palabra seleccionada y sus tokenized
		let first_selected_tokenized, last_selected_tokenized;
		let $first_selected_word, $last_selected_word;
		$.each(this.$selected_words, function(tokenized, $word)
		{
			if (!first_selected_tokenized) {
				first_selected_tokenized = tokenized;
				$first_selected_word = $word;
			}
			last_selected_tokenized = tokenized;
			$last_selected_word = $word;
		});

		if (!first_selected_tokenized) {
			return true;
		}
		first_selected_tokenized = parseInt(first_selected_tokenized, 10);
		last_selected_tokenized = parseInt(last_selected_tokenized, 10);

		let current_tokenized = $current_word.attr("tokenized_index");
		current_tokenized = parseInt(current_tokenized, 10);

		// calculo el orden validar "previo" o "siguiente"
		let order, $word;
		if (current_tokenized < first_selected_tokenized) {
			order = "prev";
			$word = $first_selected_word;
		}
		if (current_tokenized > last_selected_tokenized) {
			order = "next";
			$word = $last_selected_word;
		}
		if (!order) {
			response.must_add = false;
			return response;
		}

		// Aca recorro las palabras NO seleccionadas, desde la primera o ultima seleccionada,
		// hasta la palabra que se desea seleccionar, validando que no exista ninguna que se encuentre etiquetada
		// de las palabras que se encuentran entre la primera o ultima seleccionada hasta la que se quiere seleccionar.
		let must_add = false;
		do {
			// Aca va segun el orden "previo" o "siguiente"
			$word = $word[order]();
			if (!$word || $word.length === 0) {
				must_add = false;
				break;
			}

			// Si se encuentra etiquetada o predicated, NO dejo abrir el menu ME VOY
			let is_labeled_or_predicted = this.__word_is_labeled_or_predicted($word);
			if (is_labeled_or_predicted) {
				must_add = false;
				break;
			}

			// Agrego en el array de palabras continuas ha etiquetar
			response.$words.push($word);
			let tokenized = $word.attr("tokenized_index");
			tokenized = parseInt(tokenized, 10);
			// Si llego a la palabra que se desea etiquetar, SI dejo abrir el menu ME VOY
			if (current_tokenized === tokenized) {
				must_add = true;
				break;
			}
		} while (true);

		response.must_add = must_add;
		return response;
	};

	utterances_list.prototype.__is_opened_menu_labels = function()
	{
		return this.menu_entities && this.menu_entities.isVisible();
	};

	/*
		Abre el menu o asigna nuevas palabras si el menu ya se encuentra abierto.
	*/
	utterances_list.prototype.__open_menu_labels = function($word)
	{
		var self = this;

		if (this.__is_opened_menu_labels()) {
			// Invoco a la funcion de la clase base, para que no cierre el menu de forma automatica, al dar click fuera de este.
			this.menu_entities.setClosedAutomatic(false);

			var result = this.__can_add_words_to_menu_labels($word);
			if (result.must_add) {
				$.each(result.$words, function(_, $word_to_add) {
					// agrego las palabras a la lista de palabras seleccionadas.
					self.__select_word($word_to_add);
				});
				// Invoco a la funcion para que actualice las palabras marcadas con corchetes.
				this.__mark_words();
			}
		}
		else {
			// Aca ingresa si el menu no se encuentra abierto, y se va abrir.
			this.__open_menu_entities($word);
		}
	};

	/*
		Funcion que selecciona y agrega palabras a la lista de palabras seleccionadas que el menu asignara etiquetas.
	*/
	utterances_list.prototype.__select_word = function($word)
	{
		var self = this;

		// Funcion que agrega una palabra a la lista.
		var add_word = function($current_word)
		{
			var tokenized = $current_word.attr("tokenized_index");
			var must_remove = false;
			var $words_to_remove = [];
			// En este loop se van agregando las palabras previamente seleccionadas, que se deberan remover
			$.each(self.$selected_words, function(selected_tokenized, $selected_word)
			{
				if (must_remove) {
					$words_to_remove.push($selected_word);
				}
				else {
					// Se flagean las palabras seleccionadas a remover que el token sea mayor al de la palabra a agregar
					if (selected_tokenized === tokenized) {
						must_remove = true;
					}
				}
			});
			// Aca remuevo las palabras flageadas
			$.each($words_to_remove, function(_, $word_to_remove) {
				self.__unselect_word($word_to_remove);
			});
			// Agrego la nueva palabra
			self.$selected_words[tokenized] = $current_word;
		};

		let is_labeled_or_predicted = this.__word_is_labeled_or_predicted($word);
		if (!is_labeled_or_predicted) {
		// Si la palabra que se recibe como parametros, NO tiene una entity asociada, agrego la palabra a la lista.
			add_word($word);
		}
		else {
			// En este loop voy agregando palabras, hasta llegar a la ultima palabra, o hasta que la palabra se encuentre oculta
			do {
				add_word($word);
				$word = $word.next();
				// Llego a la ultima palabra
				if (!$word || $word.length === 0) {
					break;
				}
				// Se encuentra oculta, me voy
				var has_continue = !$word.is(":visible");
				if (!has_continue) {
					break;
				}
			} while (true);
		}

		// actualizo el titulo del menu con las palabras seleccionadas.
		this.__set_menu_title();
	};

	utterances_list.prototype.__is_opened_word = function($word)
	{
		var tokenized_index = $word.attr("tokenized_index");
		if (this.$selected_words[tokenized_index]) {
			return true;
		}
		return false;
	};

	utterances_list.prototype.__unselect_word = function($word)
	{
		var tokenized_index = $word.attr("tokenized_index");
		delete this.$selected_words[tokenized_index];
	};

	utterances_list.prototype.__unselect_all_words = function()
	{
		this.$selected_words = null;
		this.$selected_words = {};
	};

	utterances_list.prototype.show_or_hide_loading_msg = function()
	{
		this.owner.show_or_hide_loading_msg(this.string_loading);
	};

	utterances_list.prototype.load_items = function(page, callback = $.noop)
	{
		const self = this;

		const app_id = this.owner.frm.get_entity_id();
		if (!app_id) {
			callback(false);
			return;
		}

		this.loading = true;
		this.show_or_hide_loading_msg();

		this.load_entities(function(success/*, entities*/) {
			if (!success) {
				callback(false);
				self.loading = false;
				self.show_or_hide_loading_msg();
				return;
			}
			self.loading = true;
			self.show_or_hide_loading_msg();

			const paginator_info = self.form_paginator.get_paginator_info();
			const params = {appId: app_id, intentName: self.intent.name, paginated: {page: page || 1, rows_per_page: paginator_info.rows_per_page}};

			const luisApi = inConcert.app.getLuisAPI();
			luisApi.loadUtterancesByAppIntent(params,
				function(success, utterances) {
					self.loading = false;
					self.show_or_hide_loading_msg();
					if (!success) {
						self.owner.frm.show_status_message("<%=_T('Error retrieving Utterances.')%>", "Error", false);
						callback(false);
						return;
					}
					callback(true, utterances);
				});
		});
	};

	utterances_list.prototype.on_open_popup = function(utterance)
	{
		this.is_new = utterance ? false: true;
		this.current_item = utterance;

		var $container = $(this.prefix_id + "_popup_ctrls_container");
		$container.find("[show-when]").hide();

		if (this.is_new) {
			$container.find("[show-when='new_utterance']").show();
			var $text = $(this.prefix_id + "_popup_text");
			$text.focus().val("");

			if (this.intent && (this.intent.API_VERSION === "2022-10-01-preview" || API_VERSION === "2022-10-01-preview")) {

				var optionNone = $("<option>", { value: "", text: "" });
				var optionTrain = $("<option>", { value: "Train", text: "<%=_T('Train')%>" });
				var optionTest = $("<option>", { value: "Test", text: "<%=_T('Test')%>" });
				
				const $dataset = $(this.prefix_id + "_popup_dataset");
				$dataset.empty();
				$dataset.append(optionNone).append(optionTrain).append(optionTest);
				
				var $cultures = $("#adm_form_cognitive_services_cultures_for_intents");
				var $options = $cultures.find("option");
	
				const $language = $(this.prefix_id + "_popup_language");
				$language.empty();
				$language.append($options.clone());
	
				$container.find("[show-when='new_utterance_api_2022-10-01-preview']").show();
			}
		}
		else {
			this.show_label_assignment_controls();
		}

	};

	utterances_list.prototype.get_entity_def = function(entity)
	{
		if (!entity || !entity.typeId) {
			return;
		}
		const entity_type_id = (entity.typeId || "").toString();
		let entity_def = this.m_entities_types[entity_type_id];
		if (!entity_def) {
			return;
		}
		return entity_def;
	};

	/*
		Lo que se envia a almacenar en la BD
	*/
	utterances_list.prototype.__make_current_entity_label = function(entity_label, parent_entity_label)
	{
		let label = get_item_name(entity_label);
		if (!parent_entity_label) {
		// SIMPLE
			return label;
		}
		let parent_label = get_item_name(parent_entity_label);
		let parent_entity_def = this.get_entity_def(parent_entity_label);
		if (parent_entity_def && parent_entity_def.name === "HIERARCHICAL") {
		// HIERARCHICAL
			const separator_pos = label.indexOf(SEPARATOR_HIERARCHICAL);
			if (separator_pos > -1) {
				return label;
			}
			return FORMAT(TEMPLATE_HIERARCHICAL, parent_label, label);
		}
		// COMPOSITE
		return label;
	};

	/*
		Lo que se visualiza en pantalla
	*/
	utterances_list.prototype.__make_current_display_entity_label = function(entity, parent_entity, show_full_path)
	{
		let label = get_item_name(entity);
		if (!parent_entity) {
		// SIMPLE / MACHINE LEARNED
			return label;
		}
		let parent_label = get_item_name(parent_entity);
		let parent_entity_def = this.find_entity_def(parent_entity);

		if (parent_entity_def && parent_entity_def.name === "MACHINE LEARNED") {
			if (!show_full_path) {
			// MACHINE LEARNED CHILD
				return label;
			}
			let root_node = find_root_node(entity);
			if (root_node) {
				parent_label = get_item_name(root_node);
			}
			// MACHINE LEARNED PARENT + CHILD
			const separator_pos = label.indexOf(SEPARATOR_MACHINE_LEARNED);
			if (separator_pos > -1) {
				return label;
			}
			return FORMAT(TEMPLATE_MACHINE_LEARNED, parent_label, label);
		}

		if (parent_entity_def && parent_entity_def.name === "HIERARCHICAL") {
		// HIERARCHICAL
			const separator_pos = label.indexOf(SEPARATOR_HIERARCHICAL);
			if (separator_pos > -1) {
				return label;
			}
			return FORMAT(TEMPLATE_HIERARCHICAL, parent_label, label);
		}

		if (parent_entity_def && parent_entity_def.name === "COMPOSITE") {
			if (!show_full_path) {
			// COMPOSITE CHILD
				return label;
			}
			// COMPOSITE PARENT + CHILD
			const separator_pos = label.indexOf(SEPARATOR_COMPOSITE);
			if (separator_pos > -1) {
				return label;
			}
			return FORMAT(TEMPLATE_COMPOSITE, parent_label, label);
		}

		// COMPOSITE PARENT
		return label;
	};

	utterances_list.prototype.__add_label = function($word, entity_label, parent_entity_label, total_selected_words, index_selected_word, show_full_path)
	{
		if (!$word) {
			return;
		}

		this.__remove_label($word); // Elimino el label y toda la Data, para luego proceder.
		$word.addClass("utterance-word");
		// Elimino la class (CSS) de tipo PREBUILT !! Ya que ambas (LABELS y PREDICTIONS) NO pueden existir al mismo tiempo
		let prebuilt_entity_def = this.get_entity_def({typeId: "2"});
		if (prebuilt_entity_def) {
			$word.removeClass(prebuilt_entity_def.item_class);
		}

		const is_visible_word = index_selected_word === 1;

		if (!is_visible_word) {
			//  Este es el caso en cual la palabra SI esta labelada y NO es la primer palabra (sino que es una de las siguientes palabras)
			$word.hide();
		}
		else {
			$word.show();
			let text = $word.attr("word");

			const is_labeled = (entity_label !== null && entity_label !== undefined);

			if (!is_labeled) {
				// Este es el caso en el cual la palabra NO se encuentra labelada.
				$word.text(text);
			}
			else {
				// Este es el caso en el cual la palabra SI se encuentra labelada, y SI es la primera palabra que se encuentra labelada.
				//
				text = this.__make_current_display_entity_label(entity_label, parent_entity_label, show_full_path);
				$word.html("<span class='utteranceAssignedLabelStartMark'></span>" + text + "<span class='utteranceAssignedLabelEndMark'></span>");
				// Asigno Clases (CSS) segun el tipo de Entidad
				let child_class;
				let entity_def, parent_entity_def;
				let is_hierarchical_parent = false;

				if (parent_entity_label) {
					parent_entity_def = this.find_entity_def(parent_entity_label);
					if (parent_entity_def) {
						if (parent_entity_def.name === "MACHINE LEARNED") {
							$word.addClass("ParentEntityMachineLearned");
							child_class = "childItem";
						}
						else if (parent_entity_def.name === "COMPOSITE") {
							$word.addClass("ParentEntityComposite");
							child_class = "childItem";
						}
						else if (parent_entity_def.name === "HIERARCHICAL") {
							is_hierarchical_parent = true;
							let assigned_name = get_item_name(entity_label);
							const separator_pos = assigned_name.indexOf(SEPARATOR_HIERARCHICAL);
							if (separator_pos === -1) {
								let parent_entity_name = get_item_name(parent_entity_label);
								let entity_name = FORMAT(TEMPLATE_HIERARCHICAL, parent_entity_name, assigned_name);
								entity_def = this.find_entity_def({name: entity_name});
							}
						}
					}
				}

				entity_def = entity_def ? entity_def : this.find_entity_def(entity_label);

				if (entity_def) {
					$word.addClass(entity_def.item_class);
					$word.attr("entityType", entity_def.name);
					if (!child_class && entity_def.name === "HIERARCHICAL") {
						if (is_hierarchical_parent) {
							child_class = "childItem";
						}
						else {
							child_class = "parentItem";
						}
					}
				}

				if (child_class) {
					$word.addClass(child_class);
				}
			}
		}

		$word.attr("total_selected_words", total_selected_words);
		$word.attr("index_selected_word", index_selected_word);
		$word.data("entity_label", entity_label);
		$word.data("parent_entity_label", parent_entity_label);
	};

	utterances_list.prototype.__remove_label = function($word)
	{
		const text = $word.attr("word");
		$word.text(text)
			.removeClass()
			.addClass("utterance-word")
			.show()
			.attr("total_selected_words", null)
			.attr("index_selected_word", null)
			.data("entity_label", null)
			.data("parent_entity_label", null);

		const is_predicted = this.__word_is_predicted($word);
		if (is_predicted) {
			// Actualizo la palabra con la PREDICTION.
			let entity_prediction = $word.data("entity_prediction");
			let total_predictions_words = parseInt($word.attr("total_predictions_words"), 10);
			let index_prediction_word = parseInt($word.attr("index_prediction_word"), 10);
			this.__add_prediction($word, entity_prediction, total_predictions_words, index_prediction_word);
		}
	};

	utterances_list.prototype.__add_prediction = function($word, entity_prediction, total_predictions_words, index_prediction_word)
	{
		if (!$word) {
			return;
		}

		const is_labeled = this.__word_is_labeled($word);
		if (!is_labeled) {
			this.__remove_prediction($word); // Elimino la prediction y toda la Data, para luego proceder.

			$word.addClass("utterance-word");

			if (index_prediction_word === 1) {
				let text = $word.attr("word");

				if (entity_prediction) {
					text = this.__make_current_display_entity_label(entity_prediction);
					$word.html("<span class='utteranceAssignedPredictionStartMark'></span>" + text + "<span class='utteranceAssignedPredictionEndMark'></span>");
					// Asigno Clases (CSS) segun el tipo de Entidad
					let entity_def;
					entity_def = entity_def ? entity_def : this.find_entity_def(entity_prediction);
					if (entity_def) {
						$word.addClass(entity_def.item_class);
						$word.attr("entityType", entity_def.name);
					}
				}
				else {
					$word.text(text);
				}
			}
			else {
				// <> 1
				$word.hide();
			}
		}

		$word.attr("total_predictions_words", total_predictions_words);
		$word.attr("index_prediction_word", index_prediction_word);
		$word.data("entity_prediction", entity_prediction);
	};

	utterances_list.prototype.__remove_prediction = function($word)
	{
		$word.attr("total_predictions_words", null)
			.attr("index_prediction_word", null)
			.data("entity_prediction", null);

		const is_labeled = this.__word_is_labeled($word);
		if (!is_labeled) {
			const text = $word.attr("word");
			$word.text(text)
				.removeClass()
				.addClass("utterance-word")
				.show();
		}
	};

	utterances_list.prototype.__count_selected_words = function()
	{
		return Object.keys(this.$selected_words || {}).length;
	};

	utterances_list.prototype.__remove_label_from_selected_words = function()
	{
		var self = this;
		var total_selected_words = this.__count_selected_words();
		if (total_selected_words === 0) {
			return;
		}
		$.each(this.$selected_words, function(_, $word) {
			self.__remove_label($word);
		});
	};

	utterances_list.prototype.__add_label_to_selected_words = function(entity_label, parent_entity_label)
	{
		const self = this;
		const total_selected_words = this.__count_selected_words();
		if (total_selected_words === 0) {
			return;
		}
		const show_full_path = true;
		let index_selected_word = 0;
		$.each(this.$selected_words, function(_, $word) {
			index_selected_word++;
			self.__add_label($word, entity_label, parent_entity_label, total_selected_words, index_selected_word, show_full_path);
		});
	};

	utterances_list.prototype.find_entity_def = function(info)
	{
		if (!info) {
			return;
		}
		let entity_name = info.__full_tree_name || get_item_name(info);
		let full_entity = find_entity_by_name(entity_name);
		let entity_def = full_entity ? this.get_entity_def(full_entity) : undefined;
		return entity_def;
	};

	utterances_list.prototype.__make_utterance_labels = function(utterance)
	{
		const self = this;

		if (!utterance) {
			return;
		}
		let $words = [];
		let entities = {};

		/*
			En el atributo 'attr_make_utterance_labels' es donde se encuentran los index de los UTTERANCES o SUGGESTED UTTERANCES etiquetados.
			Los UTTERANCES utilizan el atributo 'entityLabels', mientras que los SUGGESTED UTTERANCES utilizan el atributo 'entityPredictions'.
		*/
		let utterances_labels = utterance[this.attr_make_utterance_labels] || [];
		let total_labels = utterances_labels.length;
		/*
			En el atributo 'attr_make_utterance_predictions' es donde se encuentran los index de los Predictions para los UTTERANCES.
		*/
		let utterances_predictions = utterance[this.attr_make_utterance_predictions] || [];
		let total_predictions = utterances_predictions.length;

		$.each(utterance.tokenizedText || [], function(tokenized_index, text)
		{
			let $word = $("<span></span>");
			$word.attr("word", text);
			$word.attr("tokenized_index", tokenized_index);

			let entity = null;
			let parent_entity;
			let total_label_selected_words = 1;
			let index_label_selected_word = 1;

			// 1 - Voy por los labels
			for (let i = 0; i < total_labels; i++) // (** este loop)
			{
				let label_info = utterances_labels[i];
				if (tokenized_index >= label_info.startTokenIndex && tokenized_index <= label_info.endTokenIndex)
				{
					if (i < (total_labels - 1)) {
						let has_single_label_in_word = true;
						let is_composite_parent = false;
						let next_label_info = utterances_labels[i + 1];

						if (next_label_info.startTokenIndex === label_info.startTokenIndex && next_label_info.endTokenIndex === label_info.endTokenIndex) {
							has_single_label_in_word = false;
							let next_entity_def = self.find_entity_def(next_label_info);
							is_composite_parent = next_entity_def && next_entity_def.name === "COMPOSITE";
						}

						if (is_composite_parent) {
							/*
								Para el caso de los COMPOSITE, cuando tenemos ambos (padre e hijo), vienen como 2 elementos separados en el array (puede venir uno antes que el otro), pero siempre juntos.
								Entonces debo de obtener ambos para poder validar cual es el padre y cual el hijo, aplicar las clases correspondientes al DOM y me tengo que quedar con el hijo,
								que es el que se va a visualizar.
							*/

							i++;
							let next_entity_def = self.find_entity_def(next_label_info);
							if (next_entity_def && next_entity_def.name === "COMPOSITE") {
								entity = label_info;
								parent_entity = next_label_info;
							}
							else {
								entity = next_label_info;
								parent_entity = label_info;
							}

							total_label_selected_words = (label_info.endTokenIndex + 1) - label_info.startTokenIndex;
							index_label_selected_word = (tokenized_index - label_info.startTokenIndex) + 1;

							// Corto el LOOP (** este loop)
							break;

						}
						else {
							if (!has_single_label_in_word) {
								let entity_def = self.find_entity_def(label_info);
								if (entity_def && entity_def.name === "MACHINE LEARNED") {
									let ml_last_entity = label_info;
									let ml_full_name_of_last_entity = get_item_name(ml_last_entity);
									let ml_full_name_of_parent_last_entity;
									let ml_parent_of_last_entity;
		
									for (let j = i + 1; j < total_labels; j++) 	{
										let next_child_machine_learned_label_info = utterances_labels[j];
		
										if (next_child_machine_learned_label_info) {
											if (next_child_machine_learned_label_info.startTokenIndex === label_info.startTokenIndex &&
													next_child_machine_learned_label_info.endTokenIndex === label_info.endTokenIndex) {
												ml_parent_of_last_entity = ml_last_entity;
												ml_full_name_of_parent_last_entity = ml_full_name_of_last_entity;
												ml_last_entity = next_child_machine_learned_label_info;
												ml_full_name_of_last_entity = ml_full_name_of_parent_last_entity + "_" + get_item_name(ml_last_entity);
											}
											else {
												break;
											}
										}
										entity = find_entity_by_name(ml_full_name_of_last_entity) || ml_last_entity;
									}
									parent_entity = find_entity_by_name(ml_full_name_of_parent_last_entity) || ml_parent_of_last_entity;
		
									total_label_selected_words = (label_info.endTokenIndex + 1) - label_info.startTokenIndex;
									index_label_selected_word = (tokenized_index - label_info.startTokenIndex) + 1;
		
									// Corto el LOOP (** este loop)
									break;
								}
							}
						}
					}

					// Casos SIMPLE e HIERARCHICAL
					entity = label_info;
					total_label_selected_words = (label_info.endTokenIndex + 1) - label_info.startTokenIndex;
					index_label_selected_word = (tokenized_index - label_info.startTokenIndex) + 1;

					let entity_def = self.find_entity_def(label_info);
					if (entity_def) {
						if (entity_def.name === "HIERARCHICAL") {
							const entity_name = get_item_name(entity);
							const separator_pos = entity_name.indexOf(SEPARATOR_HIERARCHICAL);
							if (separator_pos > -1) {
								let parent_name = entity_name.substring(0, separator_pos);
								parent_entity = find_entity_by_name(parent_name);
							}
						}
					}

					// Corto el LOOP
					break;
				}
			}

			// Actualizo la palabra con la Entity seleccionada.
			const show_full_path = true;
			self.__add_label($word, entity, parent_entity, total_label_selected_words, index_label_selected_word, show_full_path);

			const is_labeled = (entity !== null && entity !== undefined);
			if (is_labeled) {
				const entity_name = get_item_name(entity);
				entities[entity_name] = true;
			}

			//////--------------
			let prediction = null;
			let total_prediction_selected_words = 1;
			let index_prediction_selected_word = 1;

			for (let i = 0; i < total_predictions; i++)
			{
				let prediction_info = utterances_predictions[i];
				if (tokenized_index >= prediction_info.startTokenIndex && tokenized_index <= prediction_info.endTokenIndex)
				{
					// Casos PREBUILT
					prediction = prediction_info;
					total_prediction_selected_words = (prediction_info.endTokenIndex + 1) - prediction_info.startTokenIndex;
					index_prediction_selected_word = (tokenized_index - prediction_info.startTokenIndex) + 1;
					// Corto el LOOP
					break;
				}
			}

			// Actualizo la palabra con la PREDICTION.
			self.__add_prediction($word, prediction, total_prediction_selected_words, index_prediction_selected_word);

			const has_prediction = (prediction !== null && prediction !== undefined);
			if (has_prediction) {
				const entity_name = get_item_name(prediction);
				entities[entity_name] = true;
			}
			//------------------------
			// agrego la palabra al array de palabras a procesar!!
			$words.push($word);
		});

		return {$words: $words, entities: entities};
	};

	utterances_list.prototype.show_label_assignment_controls = function()
	{
		var utterance = this.current_item;
		if (!utterance) {
			return;
		}
		var $tokenized = $(this.prefix_id + "_popup_tokenized");
		$tokenized.html("");

		var utterance_labels = this.__make_utterance_labels(utterance);
		var $words = utterance_labels.$words;

		$.each($words || [], function(_, $word) {
			$tokenized.append($word);
		});

		var $container = $(this.prefix_id + "_popup_ctrls_container");
		var $assign_labels = $container.find("[show-when='assign_labels']");
		$assign_labels.show();

		if (this.intent && (this.intent.API_VERSION === "2022-10-01-preview" || API_VERSION === "2022-10-01-preview")) {

			var optionNone = $("<option>", { value: "", text: "" });
			var optionTrain = $("<option>", { value: "Train", text: "<%=_T('Train')%>" });
			var optionTest = $("<option>", { value: "Test", text: "<%=_T('Test')%>" });
			
			const $dataset = $(this.prefix_id + "_popup_dataset");
			$dataset.empty();
			$dataset.append(optionNone).append(optionTrain).append(optionTest);
			
			var $cultures = $("#adm_form_cognitive_services_cultures_for_intents");
			var $options = $cultures.find("option");

			const $language = $(this.prefix_id + "_popup_language");
			$language.empty();
			$language.append($options.clone());

			$language.val(utterance.language);
			$dataset.val(utterance.dataset);

			$container.find("[show-when='new_utterance_api_2022-10-01-preview']").show();
		}
	};

	utterances_list.prototype.set_intent = function(intent)
	{
		this.intent = intent || null;
	};

	utterances_list.prototype.get_grid_value = function(item, field_id)
	{
		var value = {
			original_value: "",
			value         : "",
			search_value  : ""
		};

		var item_value = item[field_id] || "";
		if (item_value === undefined) {
			return value;
		}

		value.original_value = item_value;
		value.value = item_value;
		value.search_value = item_value;
		var prediction;

		if (field_id === "predicated_intent") {
			value.value = "<%=_T('Not trained')%>";
			value.search_value = value.value;
			// Se busca en el array el current intent por su label y se visualiza su valor
			if ($.isArray(item.intentPredictions) && item.intentPredictions.length > 0) {
				$.each(item.intentPredictions, function(_, intent_prediction)
				{
					if (item.intentLabel === intent_prediction.name) {
						prediction = intent_prediction;
						return false; // break
					}
				});
				if (prediction && prediction.score > -1) {
					value.value = prediction.score + "<br>" + prediction.name;
					value.search_value = prediction.score + prediction.name;
				}
			}
		}
		else if (field_id === "value_of_another_intent") {
			value.value = "";
			value.search_value = value.value;
			// Se obtiene el primer intent del array y si su intent label es diferente al current intent, se debe de visualizar.
			if ($.isArray(item.intentPredictions) && item.intentPredictions.length > 0) {
				prediction = item.intentPredictions[0]; // obtengo el primer item del array.
				if (item.intentLabel !== prediction.name) {
					if (prediction.score > -1) {
						value.value = prediction.score + "<br>" + prediction.name;
						value.search_value = prediction.score + prediction.name;
					}
				}
			}
		}
		else if (field_id === "tokenizedText") {
			var $tokenized = $("<div></div>");
			var utterance_labels = this.__make_utterance_labels(item);
			var $words = utterance_labels.$words;
			$.each($words || [], function(_, $word) {
				$tokenized.append($word);
			});
			value.value = $tokenized;
			value.search_value = item_value;
			$.each(utterance_labels.entities, function(entity) {
				if (entity) {
					value.search_value = value.search_value + entity;
				}
			});
		}

		return value;
	};

	utterances_list.prototype.hide_all_errors = function()
	{
		this.form_paginator.hide_popup_msg();
	};

	utterances_list.prototype.clean_up_controls = function()
	{
		$(this.prefix_id + "_popup_text").val("").focus();
	};

	utterances_list.prototype.show_ctrl_err = function(err_info)
	{
		if (!err_info) {
			return;
		}
		var $control = err_info.$ctrl;
		if ($control) {
			var error_id = $control.attr("id") + "_validation";
			var $error_indicator = $("#" + error_id);
			if ($error_indicator) {
				$error_indicator.show();
			}
			$control.focus();
		}
		this.form_paginator.show_popup_msg(err_info.msg, "Error");
	};

	utterances_list.prototype.validate_info_to_save = function()
	{
		const self = this;

		this.hide_all_errors();
		let response = {success: false, $ctrl: null, item: null, msg: "<%=_T('Unable to Add Utterance.')%>"};
		let utterance = {};

		if (!this.current_item) {
		// Agregando nuevos UTTERANCES
			const $text = $(this.prefix_id + "_popup_text");
			utterance.text = $text.val();
			const validation_result_text = inConcert.Utils.validate(utterance.text, "required");

			if ( !validation_result_text.success ) {
				// valida que el name tenga al menos un caracter
				response.msg = "<%=_T('Utterance Text is required.')%>";
				response.$ctrl = $text;
				this.show_ctrl_err(response);
				return response;
			}

			if (self.intent && (self.intent.API_VERSION === "2022-10-01-preview" || API_VERSION === "2022-10-01-preview")) {
				
				const $language = $(this.prefix_id + "_popup_language");
				utterance.language = $language.val();
				const validation_result_language = inConcert.Utils.validate(utterance.language, "required");

				if ( !validation_result_language.success ) { 
					// valida que el name tenga al menos un caracter
					response.msg = "<%=_T('Utterance Language is required.')%>";
					response.$ctrl = $language;
					this.show_ctrl_err(response);
					return response;
				}

				const $dataset = $(this.prefix_id + "_popup_dataset");
				utterance.dataset = $dataset.val();
				const validation_result_dataset = inConcert.Utils.validate(utterance.dataset, "required");

				if ( !validation_result_dataset.success ) { 
					// valida que el name tenga al menos un caracter
					response.msg = "<%=_T('Utterance Dataset is required.')%>";
					response.$ctrl = $dataset;
					this.show_ctrl_err(response);
					return response;
				}
			}
		}
		else {
		// Asignando labels/entities a UTTERANCES existentes
			utterance.text = this.current_item.text;
			utterance.entityLabels = [];
			const $tokenized = $(this.prefix_id + "_popup_tokenized");

			let start_word = 0;
			let start = 0;
			const $words = $tokenized.find("[word]");

			if (self.intent && (self.intent.API_VERSION === "2022-10-01-preview" || API_VERSION === "2022-10-01-preview")) {
				
				const $language = $(this.prefix_id + "_popup_language");
				utterance.language = $language.val();
				const validation_result_language = inConcert.Utils.validate(utterance.language, "required");

				if ( !validation_result_language.success ) { 
					// valida que el name tenga al menos un caracter
					response.msg = "<%=_T('Utterance Language is required.')%>";
					response.$ctrl = $language;
					this.show_ctrl_err(response);
					return response;
				}

				const $dataset = $(this.prefix_id + "_popup_dataset");
				utterance.dataset = $dataset.val();
				const validation_result_dataset = inConcert.Utils.validate(utterance.dataset, "required");

				if ( !validation_result_dataset.success ) { 
					// valida que el name tenga al menos un caracter
					response.msg = "<%=_T('Utterance Dataset is required.')%>";
					response.$ctrl = $dataset;
					this.show_ctrl_err(response);
					return response;
				}
			}
			
			/*
				API v2:
					El modo de asignacion de Labels de esta API, es utilizado aun por las entidades HIERARCHICAL y COMPOSITE.
			*/
			var add_labels_api_v2 = function($word, start, end) {
				const entity_label = $word.data("entity_label");
				const parent_entity_label = $word.data("parent_entity_label");

				if (entity_label) {
					utterance.entityLabels.push({
						entityName    : self.__make_current_entity_label(entity_label, parent_entity_label),
						startCharIndex: start,
						endCharIndex  : end
					});
				}

				if (parent_entity_label) {
					let parent_label = get_item_name(parent_entity_label);
					let parent_entity_def = self.find_entity_def({name: parent_label});

					if (parent_entity_def && parent_entity_def.name === "COMPOSITE") {
						// COMPOSITE
						utterance.entityLabels.push({
							entityName    : self.__make_current_entity_label(parent_entity_label),
							startCharIndex: start,
							endCharIndex  : end
						});
					}
				}
			};

			/*
				Esta funcion obtiene la jerarquia de children para los MACHINE LEARNED.
				Recorre de forma recursiva comenzando desde un nivel inferior yendo hacia su padre, abuelo etc, hasta llegar al raiz.
			*/
			let machine_learner_children = null;
			var make_recursive_children_labels_api_v3 = null;
			make_recursive_children_labels_api_v3 = function (entity_label, parent_entity_label, start, end)
			{
				if ((entity_label.readableType || "").toUpperCase() === "ENTITY EXTRACTOR") { // CHILD ENTITY EXTRACTOR or ENTITY EXTRACTOR
					return;
				}

				if (!parent_entity_label) {
					return;
				}

				let child = {
					entityName    : self.__make_current_entity_label(entity_label),
					startCharIndex: start,
					endCharIndex  : end,
					children      : []
				};

				if (machine_learner_children) {
					child.children = machine_learner_children;
				}
				machine_learner_children = [child];

				const grandfather_entity = find_entity_by_id(parent_entity_label.__parent_id);
				make_recursive_children_labels_api_v3(parent_entity_label, grandfather_entity, start, end);
			};

			/*
				API v3-PREVIEW:
					El modo de asignacion de Labels de esta API, es utilizado por los nuevos MACHINE LEARNED, e invoca 'make_recursive_children_labels_api_v3' de forma recursiva,
					para asi obtener los childrens de childrens, etc.
			*/
			var add_labels_api_v3 = function($word, start, end) {
				const entity_label = $word.data("entity_label");
				const parent_entity_label = $word.data("parent_entity_label");

				if (parent_entity_label) {
					// Si el padre NO ES un COMPOSITE (4)
					if (parent_entity_label.typeId !== 4) {
						if (entity_label && entity_label.__i6_info) {
							const first_level_entity = entity_label.__i6_info.first_level_entity;
							if (first_level_entity && first_level_entity.typeId === 1) {// MACHINE LEARNED
								// Si es un machine learned
								machine_learner_children = [];
								make_recursive_children_labels_api_v3(entity_label, parent_entity_label, start, end);
	
								utterance.entityLabels.push({
									entityName    : self.__make_current_entity_label(first_level_entity),
									startCharIndex: start,
									endCharIndex  : end,
									children      : machine_learner_children
								});
	
								return;
							}
						}
					}
				}
				// Invoca a la asignacion del API v2, Si no es un MACHINE LEARNED.
				add_labels_api_v2($word, start, end);
			};

			$.each($words, function(_, item)
			{
				const $word = $(item);
				const total_selected_words = parseInt($word.attr("total_selected_words"), 10);
				const index_selected_word = parseInt($word.attr("index_selected_word"), 10);

				const word = $word.attr("word");
				start_word = utterance.text.indexOf(word, start_word);

				if (total_selected_words === 1 || index_selected_word === 1) {
					start = start_word;
				}

				const end = (start_word + word.length) - 1;

				if (total_selected_words === index_selected_word) {
					if (API_VERSION === "2.0") {
						add_labels_api_v2($word, start, end);
					}
					else {
						add_labels_api_v3($word, start, end);
					}
				}

				start_word = end;
			});
		}

		response.success = true;
		response.msg = "";
		response.item = utterance;

		return response;
	};

	utterances_list.prototype.__update = function(intent, utterance, callback = $.noop)
	{
		const self = this;

		var app_id = this.owner.frm.get_entity_id();
		if (!app_id) {
			callback(false);
			return;
		}

		var params = { appId: app_id, intentName: intent.name, utterance: utterance.text, language: utterance.language, dataset: utterance.dataset };
		var waiting_msg = "<%=_T('Adding Utterance to Intent ({0}).')%>";
		var success_msg = "<%=_T('Utterance was successfully added to Intent ({0}).')%>";
		var error_msg   = "<%=_T('Error adding Utterance to Intent ({0}).')%>";
		var error_msg_duplicate_key   = "<%=_T('There is already an attempt with the same name.')%>";
		
		if (this.current_item) {
			params.entityLabels = utterance.entityLabels;
			waiting_msg = "<%=_T('Assigning Labels to Utterance the Intent ({0}).')%>";
			success_msg = "<%=_T('Labels was successfully assigned to Utterance the Intent ({0}).')%>";
			error_msg   = "<%=_T('Error assigning Labels to Utterance the Intent ({0}).')%>";			
		}

		this.owner.frm.dashboard.show_waiting(FORMAT(waiting_msg, intent.name));
		this.owner.disabled_all_sub_menus(true);

		var luisApi = inConcert.app.getLuisAPI();
		luisApi.addUtteranceAppIntent(params,
			function(success, result)
			{
				var msg_response = {msg: "", type: ""};
				self.owner.frm.dashboard.hide_waiting();
				self.owner.frm.enable_all_sections();
				if (success) {
					var utterance = result;
					msg_response.msg = FORMAT(success_msg, intent.name);
					msg_response.type = "Info";
					callback(true, utterance, msg_response);
					self.clean_up_controls();
				}
				else {
					var server_err_msg = luisApi.getCognitiveErrorMsg(result);

					if (server_err_msg.includes("There is already")) {
						msg_response.msg = error_msg_duplicate_key;
					} else {
						msg_response.msg = FORMAT(error_msg, intent.name);
					}
					
					msg_response.type = "Error";
					callback(false, null, msg_response);
				}
			}
		);
	};

	utterances_list.prototype.save = function(callback = $.noop)
	{
		var response = this.validate_info_to_save();
		if (!response.success) {
			callback(false);
			return;
		}

		let intent = this.intent;
		let utterance = response.item;

		this.__update(intent, utterance, callback);
	};

	utterances_list.prototype.remove = function(utterance, callback)
	{
		callback = callback || $.noop;
		var self = this;

		if (!utterance) {
			throw("utterances_list.prototype.remove. Undefined Utterance.");
		}

		var end = function(success) {
			var type = "Info";
			var msg;
			if (success) {
				msg = FORMAT(self.string_deletes_success, utterance.text);
			}
			else {
				msg = FORMAT(self.string_deletes_error, utterance.text);
				type = "Error";
			}
			self.owner.frm.enable_all_sections();
			self.owner.frm.show_status_message(msg, type, 5000);
			self.owner.frm.dashboard.hide_waiting();
			callback(success);
		};

		this.owner.frm.dashboard.show_waiting(FORMAT(self.string_deletes_waiting, utterance.text));
		this.owner.disabled_all_sub_menus(true);

		var app_id = this.owner.frm.get_entity_id();
		if (!app_id) {
			end(false);
			return;
		}

		this.remove_utterance(utterance, function(success) {
			end(success);
		});
	};

	utterances_list.prototype.remove_utterance = function(utterance, callback)
	{
		callback = callback || $.noop;
		var app_id = this.owner.frm.get_entity_id();
		var params = { appId: app_id, utteranceId: utterance.id };
		var luisApi = inConcert.app.getLuisAPI();
		luisApi.deleteAppUtterance(params,
			function(success) {
				callback(success);
			}
		);
	};

	utterances_list.prototype.reassign_intent = function(intent, utterance, callback)
	{
		callback = callback || $.noop;
		this.__update(intent, utterance, function(success) {
			callback(success);
		});
	};

	//===================================================================================================
	//===================================================================================================
	/**
	Class suggested_utterances_list base.
	@param params
	*/
	function suggested_utterances_list ( params )
	{
		suggested_utterances_list.uber.constructor.call(this, params);
		this.name = "suggested_utterances";

		var html_title_intent = "<span>{0}</span>";
		this.header_title = FORMAT("<%=_T('Intent {0} / Suggested Utterances')%>", html_title_intent);
		this.string_loading = "<%=_T('Loading Suggested Utterances.')%>";
		this.string_deletes_success = "<%=_T('Suggested Utterance {0} was successfully deleted.')%>";
		this.string_deletes_error   = "<%=_T('Error deleting Suggested Utterance {0}.')%>";
		this.string_deletes_waiting = "<%=_T('Deleting Suggested Utterance {0}.')%>";
		/*
			En este atributo es donde se encuentran los index de los SUGGESTED UTTERANCES etiquetados.
			A partir de el se realiza el proceso de etiquetado de las palabras del SUGGESTED UTTERANCE.
		*/
		this.attr_make_utterance_labels = "entityPredictions";
		// Para el caso de los suggested va vacio!!
		this.attr_make_utterance_predictions = null;
		this.reasigned_suggested_utterance = false;
	}

	inConcert.Utils.inherit(suggested_utterances_list, utterances_list);

	suggested_utterances_list.prototype.__new_grid = function()
	{
		var self = this;

		var sanitized_prefix_id = SANITIZE_ID(this.prefix_id);
		// Instancia de la clase del Paginado de Suggested Utterances
		this.form_paginator = new inConcert.conf_form_paginator({
			owner        : this,
			frm          : this.owner.frm,
			row_id       : "text",
			display_name : "text",
			rows_per_page: 11,
			show_at_least: 10,
			must_delete_row_on_update : true,
			ids: {
				new_item                  : SANITIZE_ID(this.utterance_new),
				paginator                 : sanitized_prefix_id + "_paginator",
				grid                      : sanitized_prefix_id + "_list",
				not_results_found         : sanitized_prefix_id + "_not_results_found",
				//popup
				popup                     : sanitized_prefix_id + "_popup",
				popup_msg                 : sanitized_prefix_id + "_popup_message",
				popup_create              : sanitized_prefix_id + "_popup_create",
				popup_create_and_continue : sanitized_prefix_id + "_popup_create_and_continue",
				popup_title               : sanitized_prefix_id + "_popup_title",
				popup_close               : sanitized_prefix_id + "_popup_close",
				popup_close_times         : sanitized_prefix_id + "_popup_close_times",
				popup_input_id            : sanitized_prefix_id + "_popup_text",
				//search
				search_container          : sanitized_prefix_id + "_search_container",
				search_text               : sanitized_prefix_id + "_search_text",
				search_reset              : sanitized_prefix_id + "_reset_text",
				search_apply              : sanitized_prefix_id + "_apply_text"
			},
			strings: {
				not_results_found         : "<%=_T('There are still no expressions suggested by Luis.')%>",
				can_remove                : "<%=_T('Do you want to delete Suggested Utterance: {0} ?')%>",
				title_update              : "<%=_T('Update Suggested Utterance Labels')%>",
				button_create             : "<%=_T('Add')%>",
				button_create_and_continue: "<%=_T('Add & Continue')%>",
				button_update             : "<%=_T('Update')%>"
			},
			columns: [
				{id: "_number"      , name: "", type: "counter"},
				{id: "tokenizedText", name: "<%=_T('Utterances Text')%>",
					get_tooltip: function(item) {
						return item.text;
					},
					get_value: function(item) {
						return self.get_grid_value(item, "tokenizedText");
					}
				},
				{id: "suggested_intent", name: "<%=_T('Suggested Intent')%>",
					no_search: true,
					get_value: function(item) {
						return self.get_grid_value(item, "suggested_intent");
					}
				},
				{id: "dataset", name: "<%=_T('Dataset')%>",
					no_search: true,
					get_value: function(item) {
						return self.get_grid_value(item, "dataset");
					}
				},
				{id: "language", name: "<%=_T('Language')%>",
					no_search: true,
					get_value: function(item) {
						return self.get_grid_value(item, "language");
					}
				},
				{
					id: "_action",
					name: "",
					system_action: {edit: {label: "<%=_T('Labels')%>", class_name: "buttons-icons-utterances-labels"}, erase: true},
					custom_action: {
						assign_intent: {
							name: "<%=_T('Assign to Intent')%>",
							tooltip: "<%=_T('Assign to suggested intent')%>",
							execute: function($row/*, $owner*/) {
								const utterance = $row.data("row_data");
								if (!utterance) {
									self.owner.frm.show_status_message("<%=_T('Unable to assign to suggested intent.')%>", "Error", false);
									return;
								}

								let intent;
								// Se visualiza el valor del primer intent del array
								if ($.isArray(utterance.intentPredictions) && utterance.intentPredictions.length > 0) {
									intent = utterance.intentPredictions[0]; // obtengo el primer item del array.
								}
								if (!intent) {
									self.owner.frm.show_status_message("<%=_T('Unable to assign to suggested intent.')%>", "Error", false);
									return;
								}

								self.__update(intent, utterance, function(success) {
									if (success){
										self.owner.form_utterances_list.on_reassigned_utterance();
										return;
									}
									self.owner.frm.show_status_message("<%=_T('Unable to assign to suggested intent.')%>", "Error", false);
								});
							},
							class_name: "buttons-icons-assign-intent"
						},
						reassign_intent: {
							name: "<%=_T('Reassign Intent')%>",
							execute: function($row, $owner) {
								self.__open_menu_intents($row, $owner);
							},
							class_name: "buttons-icons-reassign-intent"
						}
					}
				}
			],
			callbacks: {
				save: function(callback) {
					self.save(callback);
				},
				edit: function(row) {
					self.on_open_popup(row);
				},
				remove: function(row, callback) {
					self.remove(row, callback);
				},
				on_closed_popup: function() {
					self.on_closed_popup();
				},
				load_items: function(page, callback) {
					self.load_items(page, callback /*, paginator_name */);
				},
				show_or_hide_loading_msg: function(message) {
					self.show_or_hide_loading_msg(message);
				}
			}
		});
	};

	suggested_utterances_list.prototype.load_items = function(page, callback = $.noop)
	{
		const self = this;

		const app_id = this.owner.frm.get_entity_id();
		if (!app_id) {
			callback(false);
			return;
		}

		this.loading = true;
		this.show_or_hide_loading_msg();

		this.load_entities(function(success/*, entities*/) {
			if (!success) {
				callback(false);
				self.loading = false;
				self.show_or_hide_loading_msg();
				return;
			}
			self.loading = true;
			self.show_or_hide_loading_msg();

			const params = {appId: app_id, intentId: self.intent.id};

			const luisApi = inConcert.app.getLuisAPI();
			luisApi.loadSuggestedUtterancesByAppIntent(params,
				function(success, suggested_utterances) {
					self.loading = false;
					self.show_or_hide_loading_msg();
					if (!success) {
					self.owner.frm.show_status_message("<%=_T('Error retrieving Suggested Utterances.')%>", "Error", false);
						callback(false);
						return;
					}
					callback(true, suggested_utterances);
				});
		});
	};

	suggested_utterances_list.prototype.make_header_title = function()
	{
		var utterances_section_title = this.intent ? FORMAT(this.header_title, this.intent.name) : "";
		this.owner.set_section_title(utterances_section_title);

		return {
			section_title: utterances_section_title,
			html         : this.form_paginator.get_header_search_html()
		};
	};

	suggested_utterances_list.prototype.remove_utterance = function(utterance, callback)
	{
		callback = callback || $.noop;
		var app_id = this.owner.frm.get_entity_id();
		var params = { appId: app_id, utteranceName: utterance.text };
		var luisApi = inConcert.app.getLuisAPI();
		luisApi.deleteAppSuggestedUtterance(params,
			function(success) {
				callback(success);
			}
		);
	};

	suggested_utterances_list.prototype.get_grid_value = function(item, field_id)
	{
		// invoco a la clase base
		var value = suggested_utterances_list.uber.get_grid_value.call(this, item, field_id);

		var suggested_fields = {
			suggested_intent: true
		};

		if (!suggested_fields[field_id]) {
			return value;
		}

		var item_value = item[field_id] || "";
		if (item_value === undefined) {
			return value;
		}

		value.original_value = item_value;
		value.value = item_value;
		value.search_value = item_value;

		if (field_id === "suggested_intent") {
			// Se visualiza el valor del primer intent del array
			if ($.isArray(item.intentPredictions) && item.intentPredictions.length > 0) {
				var prediction = item.intentPredictions[0]; // obtengo el primer item del array.
				value.value = prediction.score + "<br>" + prediction.name;
				value.search_value = prediction.score + prediction.name;
			}
		}

		return value;
	};

	suggested_utterances_list.prototype.reassign_intent = function(intent, utterance, callback)
	{
		callback = callback || $.noop;
		var self = this;

		var current_intent = this.intent;
		suggested_utterances_list.uber.reassign_intent.call(this, intent, utterance,
			function(success) {
				if (success) {
					if (current_intent.id === intent.id) {
						self.set_reassigned_utterance(true);
						self.owner.form_utterances_list.on_reassigned_utterance();
					}
				}
				callback(success);
			}
		);
	};

	suggested_utterances_list.prototype.is_reassigned_utterance = function()
	{
		return this.reasigned_suggested_utterance;
	};

	suggested_utterances_list.prototype.set_reassigned_utterance = function(reassigned)
	{
		this.reasigned_suggested_utterance = reassigned;
	};

	//===================================================================================================
	//===================================================================================================
	/**
	Class labels_utterances base.
	@param params
	*/
	function menu_labels_utterances_base ( params )
	{
		this.parent = params.parent;
		this.$tab_container = undefined;
		this.selected_item_id = null;
		this.jquery_txt_search_selector = "input[txt_search='true']";
		this.total_rows = 0;
		this.string_total_message_one  = "<%=_T('{0} row.')%>";  // el por defecto
		this.string_total_message_many = "<%=_T('{0} rows.')%>"; // el por defecto
		this.load_fn = null;
		this.is_loaded = false;
	}

	/*******************************************************************
	* Metodo que ...
	*/
	menu_labels_utterances_base.prototype.initialize = function()
	{
		var initializing = false;
		var result = this.is_enabled();
		if ( result.enabled ) {
			initializing = true;
		}
		else {
			this.disable_assign_button( result.reason );
		}
		this.set_events();
		return initializing;
	};

	/*******************************************************************
	* Metodo que ...
	*/
	menu_labels_utterances_base.prototype.disable = function()
	{
		var $tab_container = this.$get_tab_container();
		var $search = $tab_container.find(this.jquery_txt_search_selector);
		this.hide_list( "<%=_T('Assign Disabled')%>" );
		$search.prop("disabled", true);
	};

	menu_labels_utterances_base.prototype.enable = function()
	{
		var $tab_container = this.$get_tab_container();
		var $search = $tab_container.find(this.jquery_txt_search_selector);
		this.show_list();
		$search.prop("disabled", false).focus();
	};

	/*******************************************************************
	* Metodo que ...
	*/
	menu_labels_utterances_base.prototype.is_enabled = function()
	{
		return { enabled : true };
	};

	/*******************************************************************
	* Metodo que ...
	*/
	menu_labels_utterances_base.prototype.is_visible = function()
	{
		return { visible : true };
	};

	/*******************************************************************
	* Metodo que ...
	* @param callback
	*/
	menu_labels_utterances_base.prototype.load = function( callback )
	{
		if ( !this.load_fn ) {
			throw "menu_labels_utterances_base.load - this.load_fn is required!!";
		}
		callback = callback || $.noop;

		this.load_fn( callback );
	};

	/*******************************************************************
	* Metodo que ...
	* @param callback
	*/
	menu_labels_utterances_base.prototype.on_finish_load = function( callback )
	{
		callback = callback || $.noop;
		this.is_loaded = true;
		this.set_totals();
		if ( this.total_rows === 0 ) {
			this.disable();
		}
		else {
			this.enable();
		}
		this.parent.hide_waiting();
		callback();
	};

	/*******************************************************************
	* Metodo que ...
	*/
	menu_labels_utterances_base.prototype.on_switch_tab = function()
	{
		this.parent.toggle();
		var $tab_container = this.$get_tab_container();
		$tab_container.find(this.jquery_txt_search_selector).focus();
		this.parent.setPosition();
	};

	/*******************************************************************
	* Metodo que ...
	* @param id
	*/
	menu_labels_utterances_base.prototype.get_radio_dom_id = function(entity, parent_entity)
	{
		if (!entity || !entity.id) {
			return;
		}
		let entity_id = entity.id;
		let radio_id = this.prefix_radio_id + inConcert.Utils.getNormalizedId(entity_id);
		if (!parent_entity) {
			return radio_id;
		}
		let parent_entity_id = parent_entity.id;
		radio_id = radio_id + "_" + inConcert.Utils.getNormalizedId(parent_entity_id);
		return radio_id;
	};

	/*******************************************************************
	* Metodo que ...
	*/
	menu_labels_utterances_base.prototype.$get_tab_container = function()
	{
		if ( !this.$tab_container ) {
			this.$tab_container = this.parent.data.container.find(this.jquery_tab_selector);
		}
		return this.$tab_container;
	};

	menu_labels_utterances_base.prototype.get_selected_item_id = function()
	{
		return this.selected_item_id;
	};

	menu_labels_utterances_base.prototype.make_detail = function() { return false; };

	menu_labels_utterances_base.prototype.show_detail = function()
	{
		var $detail_list = this.$get_tab_container().find("[detail_list='true']");
		if ( !$detail_list || $detail_list.length === 0 ) {
			return;
		}
		if ( !this.selected_item_id ) {
			$detail_list.html("");
			return;
		}
		var $item_container = $("#" + this.selected_item_id).parent();
		if ( !$item_container.is(":visible") ) {
			$detail_list.html("");
			return;
		}
		var item_data = $item_container.data("row");
		if ( !item_data ) {
			$detail_list.html("");
			return;
		}
		if ( !this.make_detail(item_data) ) {
			$detail_list.html("");
		}
	};

	/*******************************************************************
	* Metodo que ...
	*/
	menu_labels_utterances_base.prototype.on_selected_item = function( $new_selected_item /* radio buttom */ )
	{
		if ( this.selected_item_id !== null ) {
			$("#" + this.selected_item_id).parent().removeClass("selected");
		}
		this.selected_item_id = $new_selected_item.attr("for");
		$("#" + this.selected_item_id).prop("checked", true).parent().addClass("selected");
		this.show_detail();
		this.parent.enable_assign_button();
	};

	/*******************************************************************
	* Metodo que ...
	*/
	menu_labels_utterances_base.prototype.set_events = function()
	{
		var self = this;
		var $tab_container = this.$get_tab_container();

		// click en la row del item
		$tab_container
			.off("click", this.jquery_item_selector)
			.on("click", this.jquery_item_selector, function () {
				self.on_selected_item($(this)) ;
			});
		// dbl click en la row del item
		$tab_container
			.off("dblclick", this.jquery_item_selector)
			.on("dblclick", this.jquery_item_selector, function () {
				self.parent.enable_assign_button();
				self.parent.assign_label();
			});
		// evento keyup del text de search
		$tab_container
			.off("keyup", this.jquery_txt_search_selector)
			.on("keyup", this.jquery_txt_search_selector, function() {
				var text = $(this).val();
				if ( self.timeout ) {
					window.clearTimeout( self.timeout );
					self.timeout = null;
				}
				self.timeout = window.setTimeout( function() {
					self.filter( text );
					window.clearTimeout( self.timeout );
					self.timeout = null;
				}, 500 );
			});
	};

	/*******************************************************************
	* Metodo que ...
	* @param word
	*/
	menu_labels_utterances_base.prototype.make_search_regexp = function( word )
	{
		if (typeof word !== "string") {
			return "";
		}
		var search_pattern = inConcert.Utils.removeSpecialChars(word);
		return new RegExp(search_pattern.replace(/[^\w\s]/gi, ""), "i");
	};

	/*******************************************************************
	* Metodo que ...
	*/
	menu_labels_utterances_base.prototype.set_totals = function()
	{
		var text = this.total_rows === 1 ? this.string_total_message_one : this.string_total_message_many;
		var $tab_container = this.$get_tab_container();
		$tab_container.find("[total='true']").text(FORMAT(text, this.total_rows));
	};

	menu_labels_utterances_base.prototype.filter = function( text )
	{
		var self = this;
		var $tab_container = this.$get_tab_container();
		var $list_container = $tab_container.find("[list_container='true']");

		if ( text.trim().length === 0 ) {
		// si no hay texto buscado en el filtro, no filtro la row
			$list_container.find(this.jquery_item_selector).show();
			this.show_list();
			return;
		}
		var hidden_rows = 0;
		$list_container.find(this.jquery_item_selector).each(function(_, row)
		{
			var $row = $(row);
			var row_value = $row.attr("search_text");
			var search_regexp = self.make_search_regexp(text);
			if ( row_value.search(search_regexp) === -1 ) {
				$row.hide();
				hidden_rows++;
			}
			else {
				$row.show();
			}
		});
		if ( self.total_rows === hidden_rows ) {
			this.hide_list();
		}
		else {
		// muestra la lista
			this.show_list();
			this.show_detail();
		}
	};

	/*******************************************************************
	* Metodo que ...
	* @param message
	*/
	menu_labels_utterances_base.prototype.hide_list = function( message )
	{
		message = message || this.not_found_default_msg || "<%=_T('Not found.')%>";
		var $tab_container = this.$get_tab_container();
		$tab_container.find("[data_container='true']").hide();
		var $message_container = $tab_container.find("[not_found_container='true']");
		$message_container.find("span").text(message);
		$message_container.show();
		this.parent.disable_assign_button();
	};

	/*******************************************************************
	* Metodo que ...
	*/
	menu_labels_utterances_base.prototype.show_list = function()
	{
		var $tab_container = this.$get_tab_container();
		$tab_container.find("[data_container='true']").show();
		$tab_container.find("[not_found_container='true']").hide();
		this.parent.toggle();
	};

	//===================================================================================================
	//===================================================================================================
	/**
	Class labels_utterances entity hereda de menu_labels_utterances_base.
	* @param params
	*/
	function menu_labels_utterances_items ( params )
	{
		const self = this;
		menu_labels_utterances_items.uber.constructor.call(this, params);
		this.jquery_item_selector = "[row_item]";
		this.jquery_tab_selector = "[tab_container='items']";
		this.not_found_default_msg = "<%=_T('No items found.')%>";
		//this.prefix_radio_id = "labels_utterances_items_";
		// this.string_total_message_one  = "<%=_T('{0} item.')%>";
		// this.string_total_message_many = "<%=_T('{0} items.')%>";
		this.assigned_label = null;
		this.assigned_prediction = null;
		this.load_fn = function( callback ) { self.parent.parent_load(callback); };
	}

	inConcert.Utils.inherit( menu_labels_utterances_items, menu_labels_utterances_base );

	/*******************************************************************
	* Metodo que ...
	* @param entity
	*/
	menu_labels_utterances_items.prototype.make_search_item_text = function(item)
	{
		return get_item_name(item);
	};

	menu_labels_utterances_items.prototype.find_entity_def = function(entity)
	{
		return this.parent.owner_object.find_entity_def(entity);
	};

	/*******************************************************************
	* Metodo que ...
	* @param callback
	*/
	menu_labels_utterances_items.prototype.load = function(callback = $.noop)
	{
		const self = this;

		this.destroy_assigned_label();
		this.destroy_assigned_prediction();

		const waiting_msg = this.string_loading;
		this.parent.show_waiting(waiting_msg);

		// invoco a la clase base
		menu_labels_utterances_items.uber.load.call(
			this,
			// callback del request
			function(success, entities, $word, must_add_item)
			{
				must_add_item = must_add_item || function() {return true;};
				if (!success) {
					self.on_finish_load(callback);
					self.hide_list();
					return;
				}
				if (!entities || entities.length === 0) {
					self.on_finish_load(callback);
					self.hide_list();
					return;
				}

				const $labels_list = self.$get_tab_container().find("[list_container='true']");
				$labels_list.html("");

				// LABEL -------
				let assigned_label = $word.data("entity_label");
				let assigned_parent_label = $word.data("parent_entity_label");
				let selected_entity_dom_id;
				let full_assigned_parent_entity;

				if (assigned_label) {
					self.set_assigned_label(assigned_label);
					self.parent.show_remove_button();

					let display_assigned_name;
					let assigned_name = get_item_name(assigned_label);
					display_assigned_name = assigned_name;

					let full_assigned_name = assigned_label.__full_tree_name || assigned_name;
					let full_assigned_entity = find_entity_by_name(full_assigned_name);

					if (assigned_parent_label) {
						const parant_entity_name = get_item_name(assigned_parent_label);
						let full_assigned_parent_name = assigned_parent_label.__full_tree_name || parant_entity_name;
						full_assigned_parent_entity = find_entity_by_name(full_assigned_parent_name);
						const parent_entity_def = self.find_entity_def(full_assigned_parent_entity); //{name: parant_entity_name});

						const is_composite_parent = parent_entity_def && parent_entity_def.name === "COMPOSITE";
						if (is_composite_parent) {
							let assigned_parent_name = get_item_name(assigned_parent_label);
							assigned_name = FORMAT(TEMPLATE_COMPOSITE, assigned_parent_name, assigned_name);
						}
						else {
							if (API_VERSION !== "2.0") {
								display_assigned_name = full_assigned_name.replace(/_/gi, " -> ");
							}
						}
					}

					// El ID
					selected_entity_dom_id = self.get_radio_dom_id(full_assigned_entity, full_assigned_parent_entity);

					let display_text = FORMAT("<%=_T('Assigned Entity: {0}')%>", display_assigned_name);

					const $assigned_label = $("".concat(
						"<div action='remove_label' label='" + assigned_name + "' title='" + assigned_name + "' row_item='true' search_text='", display_text, "' class='assignedEntitiesTitle'>",
							"<div class='targetItemData'>",
								"<label class='targetItemName'>", display_text, "</label>",
								"<label class='targetItemInfo' style='display:none;'>", display_text, "</label>",
							"</div>",
						"</div>"
					));
					$assigned_label.data("entity_label", assigned_label);
					$labels_list.append($assigned_label);
				}
				else {
					self.parent.hide_remove_button();
				}
				//------------------

				// PREDICTION -------
				let assigned_prediction = $word.data("entity_prediction");
				if (assigned_prediction) {
					self.set_assigned_prediction(assigned_prediction);
				}
				//------------------
				var can_add = function(item)
				{
					if (!must_add_item(item)) {
						return false;
					}
					return true;
				};

				self.total_rows = 0;
				let sorted_entities = inConcert.Utils.sortArray(entities, "name");

				let entities_by_id = {};
				sorted_entities.forEach(entity => {
					entities_by_id[entity.id] = entity;
				});

				var append = function(entity, parent_entity, level)
				{
					if (can_add(entity)) {
						self.total_rows++;
						const entity_dom_id = self.get_radio_dom_id(entity, parent_entity);
						let search_text = self.make_search_item_text(entity);
						let entity_name = get_item_name(entity);
						let display_name = entity_name;

						const full_entity = entities_by_id[entity.id];
						let entity_def;
						if (full_entity) {
							entity_def = self.get_entity_def(full_entity);
						}
						let item_type_class = entity_def ? entity_def.item_class : "";
						if (parent_entity) {
							item_type_class = item_type_class + " childItem";
							if (level) {
								item_type_class = item_type_class + " childItemLevel" + level;
							}
							// Cuando cargo en el tree del popup de asignacion, tengo que asignar el ParentId "__parent_id".
							// Asi puedo recorrer el tree desde un nivel inferior a sus padre, abuelo, etc.
							let cached_entity = find_entity_by_id(entity.id);
							if (cached_entity) {
								cached_entity.__parent_id = parent_entity.id.toLowerCase();
								update_cached_entity(cached_entity);
								entity = cached_entity;
							}
						}
						else {
							item_type_class = item_type_class + " parentItem";
						}
						const $label = $("".concat(
							"<div action='add_label' label='" + entity_name + "' row_item='true' for='", entity_dom_id ,"' search_text='", search_text, "' title='" + entity_name + "' class='targetListItem " + item_type_class + "'>",
								"<div class='form-radiobutton'>",
									"<label for='", entity_dom_id ,"' class='form-radiobutton-label'>",
										"<input id='", entity_dom_id ,"' selector='true' type='radio' name='entity' class='form-radiobutton-input' />",
										"<span>", display_name, "</span>",
									"</label>",
								"</div>",
								"<div class='targetItemData entityItemData'>",
									"<label for='", entity_dom_id ,"' class='targetItemInfo' style='display:none;'>", entity_name, "</label>",
								"</div>",
							"</div>"
						));

						$label.data("entity_label", entity);
						$label.data("parent_entity_label", parent_entity);
						$labels_list.append($label);
					}
				};

				var fill_recursive_children;
				fill_recursive_children = function(entity, level) {
					if (!entity || !entity.children) {
						return;
					}
					// HIERARCHICAL,  COMPOSITE o MACHINE LEARNED
					entity.children.forEach(child => {
						// Agrego cada uno de los hijos del HIERARCHICAL, COMPOSITE o MACHINE LEARNED
						child.__i6_info = entity.__i6_info;
						child.__i6_info.level = level;

						append(child, entity, level);
						let next_level = level + 1;
						fill_recursive_children(child, next_level);
					});
				};

				// Se carga en un foreach que corre dentro de un timeout para que el navegador no quede colgado
				inConcert.Utils.foreach_timeout({
					rows: sorted_entities,
					append_fn : function(entity) {
						entity.__i6_info = {
							level: 0,
							first_level_entity: entity
						};
						const entity_def = self.get_entity_def(entity);
						append(entity);
						if (entity_def && (entity_def.name === "HIERARCHICAL" || entity_def.name === "COMPOSITE" || entity_def.name === "MACHINE LEARNED")) {
							// // HIERARCHICAL,  COMPOSITE o MACHINE LEARNED
							let level = 1;
							fill_recursive_children(entity, level);
						}
					},
					complete_fn : function() {
						if (selected_entity_dom_id) {
							const $selected_entity = $("#" + selected_entity_dom_id);
							if ($selected_entity) {
								$selected_entity.trigger("click");
							}
						}
						self.on_finish_load( callback );
					}
				});
			}
		);
	};

	menu_labels_utterances_items.prototype.get_entity_def = function(entity)
	{
		const entity_def = this.parent.owner_object.get_entity_def(entity);
		return entity_def;
	};

	menu_labels_utterances_items.prototype.is_enabled = function()
	{
		return { enabled : true };
	};

	menu_labels_utterances_items.prototype.__get_selected_entity_label = function()
	{
		const $tab_container = this.$get_tab_container();
		const $chk_label = $tab_container.find("div[list_container='true'] input[name='entity']:checked");
		if ($chk_label.length === 0) {
			return;
		}
		const chk_id = $chk_label.attr("id");
		const selector = FORMAT("[action='add_label'][for='{0}']", chk_id);
		const $label_container = $chk_label.closest(selector);
		const entity_label = $label_container.data("entity_label");
		const parent_entity_label = $label_container.data("parent_entity_label");
		return {
			entity_label: entity_label,
			parent_entity_label: parent_entity_label
		};
	};

	menu_labels_utterances_items.prototype.get_assigned_label = function()
	{
		return this.assigned_label;
	};

	menu_labels_utterances_items.prototype.set_assigned_label = function(assigned_label)
	{
		this.assigned_label = assigned_label;
	};

	menu_labels_utterances_items.prototype.destroy_assigned_label = function()
	{
		this.assigned_label = null;
	};

	menu_labels_utterances_items.prototype.get_assigned_prediction = function()
	{
		return this.assigned_prediction;
	};

	menu_labels_utterances_items.prototype.set_assigned_prediction = function(assigned_prediction)
	{
		this.assigned_prediction = assigned_prediction;
	};

	menu_labels_utterances_items.prototype.destroy_assigned_prediction = function()
	{
		this.assigned_prediction = null;
	};

	//===================================================================================================
	//===================================================================================================
	/**
	Class menu_labels_utterances_entities.
	* @param params
	*/
	function menu_labels_utterances_entities ( params )
	{
		var self = this;
		menu_labels_utterances_entities.uber.constructor.call(this, params);
		//this.jquery_item_selector = "[row_item]";
		//this.jquery_tab_selector = "[tab_container='items']";
		this.not_found_default_msg = "<%=_T('No entities found.')%>";
		this.prefix_radio_id = "labels_utterances_entity_";
		this.string_total_message_one  = "<%=_T('{0} entity.')%>";
		this.string_total_message_many = "<%=_T('{0} entities.')%>";
		this.string_title = "<%=_T('Entities')%>";
		this.string_save = "<%=_T('Assign')%>";
		this.string_remove = "<%=_T('Remove')%>";
		this.string_loading = "<%=_T('Loading entities, wait moment please')%>";

		this.assigned_label = null;
		this.assigned_prediction = null;
		this.load_fn = function( callback ) { self.parent.parent_load(callback); };
	}

	inConcert.Utils.inherit( menu_labels_utterances_entities, menu_labels_utterances_items );

	/*******************************************************************
	*
	*/
	menu_labels_utterances_entities.prototype.make_menu_title = function(value)
	{
		return value ? FORMAT("<%=_T('Assign Entity - Words: {0}')%>", value) : this.string_title;
	};

	//===================================================================================================
	//===================================================================================================
	/**
	Class menu_labels_utterances_intents.
	* @param params
	*/
	function menu_labels_utterances_intents ( params )
	{
		var self = this;
		menu_labels_utterances_intents.uber.constructor.call(this, params);
		//this.jquery_item_selector = "[row_item]";
		//this.jquery_tab_selector = "[tab_container='items']";

		this.not_found_default_msg = "<%=_T('No intents found.')%>";
		this.prefix_radio_id = "labels_utterances_intent_";
		this.string_total_message_one  = "<%=_T('{0} intent.')%>";
		this.string_total_message_many = "<%=_T('{0} intents.')%>";
		this.string_title = "<%=_T('Intents')%>";
		this.string_save = "<%=_T('Reassign Intent')%>";
		this.string_remove = "";
		this.string_loading = "<%=_T('Loading intents, wait moment please')%>";

		this.assigned_label = null;
		this.assigned_prediction = null;
		this.load_fn = function( callback ) { self.parent.parent_load(callback); };
	}

	inConcert.Utils.inherit( menu_labels_utterances_intents, menu_labels_utterances_items );

	/*******************************************************************
	*
	*/
	menu_labels_utterances_intents.prototype.make_menu_title = function(/*value*/)
	{
		return this.string_title;
	};

	//===================================================================================================
	//===================================================================================================
	/**
	Class menu_labels.
	* @param params
	*/
	function menu_labels ( params )
	{
		menu_labels.uber.constructor.call(this);
		this.clickEventName = this.clickEventName + "labels_utterances";
		this.ownerId = "window_labels_utterances_menu_container";
		this.canReposition = true;
		this.from_app = params.from_app;
		this.owner = params.owner;
		this.current_container = null;
		this.count_loading_list = 0;
		// instancias de los handles de las listas
		this.list_handle = {
			items: new params.item_class({parent: this})
		};
	}

	inConcert.Utils.inherit(menu_labels, inConcert.PopupWindow);

	/*******************************************************************
	*
	* @param text
	* @param type
	* @param closeDisabled
	* @param timeout_ms
	*/
	menu_labels.prototype.show_status_message = function(text, type, closeDisabled, timeout_ms )
	{
		var $section = $("#window_labels_utterances_menu_message");
		$section.removeClass().addClass("windowContainerMessages").addClass("message" + type);
		$section.show().find(".windowMessagesTextAux").text(text);
		if (timeout_ms) {
			window.setTimeout(function() {
				$section.hide();
			}, timeout_ms);
		}
		if (closeDisabled) {
			$("#window_labels_utterances_menu_close_message").hide();
		}
		else {
			$("#window_labels_utterances_menu_close_message").show();
		}
	};

	/*******************************************************************
	* funcion que oculta mensaje
	*/
	menu_labels.prototype.hide_status_message = function() {
		$("#window_labels_utterances_menu_message").hide();
	};

	/*******************************************************************
	*
	*/
	menu_labels.prototype.get_assigned_label = function()
	{
		if ( !this.current_container ) {
			return false;
		}
		return this.list_handle[this.current_container].get_assigned_label();
	};

	/*******************************************************************
	*
	*/
	menu_labels.prototype.get_assigned_prediction = function()
	{
		if ( !this.current_container ) {
			return false;
		}
		return this.list_handle[this.current_container].get_assigned_prediction();
	};

	/*******************************************************************
	*
	*/
	menu_labels.prototype.destroy_assigned_label = function()
	{
		if ( !this.current_container ) {
			return;
		}
		this.list_handle[this.current_container].destroy_assigned_label();
	};

	/*******************************************************************
	*
	*/
	menu_labels.prototype.destroy_assigned_prediction = function()
	{
		if ( !this.current_container ) {
			return;
		}
		this.list_handle[this.current_container].destroy_assigned_prediction();
	};

	/*******************************************************************
	*
	*/
	menu_labels.prototype.validate = function()
	{
		if ( !this.current_container ) {
			return false;
		}
		if (this.list_handle[this.current_container].always_enabled) {
			return true;
		}
		var selected_item_id = this.list_handle[this.current_container].get_selected_item_id();
		return selected_item_id !== null;
	};

	/*******************************************************************
	* 
	*/
	menu_labels.prototype.__get_selected_entity_label = function()
	{
		if ( !this.current_container ) {
			return;
		}
		var list_handle = this.list_handle[this.current_container];
		if ( !list_handle ) {
			return;
		}
		return list_handle.__get_selected_entity_label();
	};

	/*******************************************************************
	* 
	* @param message
	*/
	menu_labels.prototype.show_message_in_main_window = function( message )
	{
		inConcert.app.getMainWindow().showBannerMessage(message);
	};

	/*******************************************************************
	* Metodo que setea eventos de los controles
	* @param tab_item
	*/
	menu_labels.prototype.switch_container = function( tab_item )
	{
		var $tab_items = $("#window_labels_utterances_menu_tab_items");
		var $tab_item = $tab_items.find("[tab_item='" + tab_item + "']");
		if ( this.current_container === tab_item ) {
			return;
		}
		this.current_container = tab_item;
		$tab_items.find("[tab_item]").removeClass("selected");
		$tab_item.addClass("selected");
		var $container = $("#window_labels_utterances_menu_container");
		$container.find("[tab_container]").hide();
		$container.find("[tab_container='" + tab_item + "']").show();

		this.list_handle[tab_item].on_switch_tab();
	};

	/*******************************************************************
	* Metodo que muestra u oculta el selector
	* @param tab_item
	*/
	menu_labels.prototype.toggle_selector = function( tab_item )
	{
		var result = this.list_handle[tab_item].is_visible();
		$("#window_labels_utterances_menu_tab_items")
				.find("[tab_item='" + tab_item + "']")
				.toggle(result.visible);
	};

	/*******************************************************************
	* Metodo ...
	*/
	menu_labels.prototype.enable_assign_button = function()
	{
		$("#window_labels_utterances_menu_save").prop("disabled", false);
	};

	menu_labels.prototype.disable_assign_button = function()
	{
		$("#window_labels_utterances_menu_save").prop("disabled", true);
	};

	menu_labels.prototype.show_remove_button = function()
	{
		$("#window_labels_utterances_menu_remove").prop("disabled", false).show();
	};

	menu_labels.prototype.hide_remove_button = function()
	{
		$("#window_labels_utterances_menu_remove").prop("disabled", true).hide();
	};

	menu_labels.prototype.toggle = function()
	{
		var list_handle = this.list_handle[this.current_container];
		var selected_item = list_handle.get_selected_item_id();
		var enabled = list_handle.always_enabled || false;

		if ( !enabled && selected_item ) {
			var $item = $("#" + selected_item);
			var $item_container = $item.parent();
			enabled = $item_container.is(":visible");
		}

		if ( enabled ) {
			this.enable_assign_button();
		}
		else {
			this.disable_assign_button();
		}
	};

	/*******************************************************************
	* Metodo que setea eventos de los controles
	*/
	menu_labels.prototype.assign_label = function()
	{
		const self = this;
		const selected_label_info = this.__get_selected_entity_label();
		if (!selected_label_info || !selected_label_info.entity_label) {
			this.show_status_message("<%=_T('Error in selected entity label item.')%>", "Error", undefined, 2500);
			return;
		}
		this.parent_on_success_assigned(
			selected_label_info.entity_label,
			selected_label_info.parent_entity_label,
			function() {
				self.close();
			}
		);
	};

	/*******************************************************************
	* Metodo que setea eventos de los controles
	*/
	menu_labels.prototype.__remove_label = function()
	{
		var self = this;
		this.parent_on_success_removed(function() {
			self.close();
		});
	};

	/*******************************************************************
	* Metodo que setea eventos de los controles
	*/
	menu_labels.prototype.set_events = function()
	{
		var self = this;

		$("#window_labels_utterances_menu_save")
			.bind("click", function() {
				self.assign_label();
			});

		$("#window_labels_utterances_menu_remove")
			.bind("click", function() {
				self.__remove_label();
			});

		var close_menu = function() { self.close(); };

		// evento que cierra el dialogo y no realiza cambios
		$("#window_labels_utterances_menu_cancel").bind("click", close_menu );
		// evento que cierra el dialogo y no realiza cambios
		$("#window_labels_utterances_menu_close").bind("click", this.close);

		$("#window_labels_utterances_menu_close_message")
			.bind("click", function() {
				self.hide_status_message();
			});

		var event_selector = "[tab_item]";
		$("#window_labels_utterances_menu_tab_items")
				.find(event_selector)
				.each(function() {
					var $this = $( this );
					var tab_item = $this.attr("tab_item");
					self.toggle_selector( tab_item );
				})
				.off("click")
				.on("click", function ()
				{
					var $this = $( this );
					var tab_item = $this.attr("tab_item");
					self.switch_container( tab_item );
				});
	};

	/*******************************************************************
	* Metodo que crea el markup contenedor de la ventana
	*/
	menu_labels.prototype.create_markup = function()
	{
		var list_handle = this.list_handle.items;

		$("#window_labels_utterances_menu_container").remove();
		var $node = $("".concat(
			"<div id='window_labels_utterances_menu_container' class='windowUtterancesLabelsContainer windowContainer' hidden>",

				"<div id='window_labels_utterances_menu_waiting' class='windowWaitingContainer' hidden>",
					"<div message='true' class='windowWaitingMessage'></div>",
					"<div class='windowWaitingAnimation'></div>",
					"<div class='windowWaitingOverlay'></div>",
				"</div>",

//				"<div class='windowTop'>",
//					"<div class='windowTitle ", this.title_class, "'>",
//						"<div id='window_labels_utterances_menu_title' class='windowUtterancesLabelsTitleText'></div>",
//						"<img id='window_labels_utterances_menu_close' src='/inconcert/apps/agent/images/blank.png' class='windowUtterancesLabelsButClose' hidden />",
//					"</div>",
//				"</div>",

				"<div id='window_labels_utterances_menu_message' class='windowContainerMessages messageError' hidden  style='display:none'>",
					"<a id='window_labels_utterances_menu_close_message' class='windowMessagesClose' href='javascript:void(0);'>", "<%=_T('Close')%>", "</a>",
					"<div class='windowMessagesText'>",
						"<label class='windowMessagesTextAux'></label>",
					"</div>",
				"</div>",

				"<div class='windowMiddle'>",
					"<div class='card-block'>",
						// labels_utterances options
						"<div id='window_labels_utterances_menu_tab_items' class='windowTabsMenu' hidden>",
							"<span tab_item='items' class='windowTabsMenuItem'>", list_handle.string_title, "</span>",
						"</div>",
						// items list container
						"<div tab_container='items' class='windowTargetEntityContainer' hidden>",
							"<div class='row'>",
								"<div class='col-12'>",
									"<div class='form-group'>",
										"<input txt_search='true' class='form-control' placeholder='<%=_T('Search')%>...' type='text' disabled='true'/>",
									"</div>",
								"</div>",
							"</div>",
							"<div class='row'>",
								"<div class='col-12'>",
									"<div class='form-group'>",
										"<div data_container='true' class='windowTargetEntityControl' hidden>",
											"<div list_container='true' class='windowTargetEntityScrollArea'></div>",
											"<div class='windowTargetFooter'>",
												"<div total='true' class='windowTargetTotal'></div>",
												"<div detail='true' class='windowTargetDetailContainer'>",
													"<div detail_list='true' class='windowTargetDetailInfo'></div>",
												"</div>",
											"</div>",
										"</div>",
									"</div>",
								"</div>",
							"</div>",
							"<div class='row'>",
								"<div not_found_container='true' class='col-12' hidden>",
									"<span class='inlineMessagesContainer messageInfo d-block'></span>",
								"</div>",
							"</div>",
						"</div>",
					"</div>",
				"</div>",

				"<div class='windowBottom d-flex justify-content-end'>",
					"<input id='window_labels_utterances_menu_cancel' class='pageButtonGeneric buttonColorWhite' type='button' value='", "<%=_T('Cancel')%>", "'/>",
					"<input id='window_labels_utterances_menu_remove' class='pageButtonGeneric buttonColorWhite' disabled='true' type='button' value='", list_handle.string_remove, "' hidden />",
					"<input id='window_labels_utterances_menu_save' class='pageButtonGeneric buttonColorWhite' disabled='true' type='button' value='", list_handle.string_save, "'/>",
				"</div>",

			"</div>"));
		$("body").append($node);
		return $node;
	};

	menu_labels.prototype.set_menu_title = function(selected_words)
	{
		var list_handle = this.list_handle.items;
		var menu_title;
		if (list_handle) {
			menu_title = list_handle.make_menu_title(selected_words);
		}
		$("#window_labels_utterances_menu_title").text(menu_title);
	};

	/*******************************************************************
	* Metodo que ...
	* @param message
	*/
	menu_labels.prototype.show_waiting = function( message )
	{
		this.waiting_progress_count = this.waiting_progress_count || 0;
		if ( this.waiting_progress_count === 5 ) {
			this.waiting_progress_count = -1;
		}
		this.waiting_progress_count++;
		var progress_text = "";
		var progress = "...";
		for ( var i = 1; i <= this.waiting_progress_count; i++ ) {
			progress_text = progress_text + progress;
		}
		message = message || "<%=_T('Loading, wait moment please')%>";
		var $waiting = $("#window_labels_utterances_menu_waiting");
		$waiting.find("[message='true']").text(message + " " + progress_text);
		$waiting.show();
	};

	/*******************************************************************
	* Metodo que ...
	*/
	menu_labels.prototype.hide_waiting = function()
	{
		var count_loaded = 0;
		$.each( this.list_handle, function( _, instance )
		{
			var result = instance.is_enabled();
			if ( result.enabled && instance.is_loaded ) {
				count_loaded++;
			}
		});
		if ( this.count_loading_list === count_loaded ) {
			$("#window_labels_utterances_menu_waiting").hide();
		}
		this.setPosition();
	};

	/*******************************************************************
	* Metodo que visualiza el menu con los labels_utterances e inicializa eventos del menu!
	*/
	menu_labels.prototype.initialize = function()
	{
		var self = this;

		this.data.container = this.create_markup();

		this.parent_load                = this.data.callbacks.load                || $.noop;
		this.parent_on_success_assigned = this.data.callbacks.on_success_assigned || $.noop;
		this.parent_on_success_removed  = this.data.callbacks.on_success_removed  || $.noop;
		this.parent_on_error_assigned   = this.data.callbacks.on_error_assigned   || $.noop;

		this.set_menu_title();
		// set events
		this.set_events();

		var pending_instances_load = [];
		// continuno con los request pendientes
		var on_load = function() {
			if ( pending_instances_load.length === 0 ) {
				return;
			}
			var instance = pending_instances_load.pop();
			if ( instance ) {
				instance.load( on_load );
			}
		};

		// inicializo las instancias de las diferentes listas
		$.each( this.list_handle, function( _, instance ) {
			if ( instance.initialize() ) {
				self.count_loading_list++;
				if (self.count_loading_list === 1) {
					instance.load( on_load );
				}
				else {
					// a los request los dejo como pendientes para que el siguiente request, recien mande a ejecutar, una vez que ha terminado el anterior.
					pending_instances_load.push( instance );
				}
			}
		});
		this.switch_container("items");
	};

	/*******************************************************************
	* Metodo que cierra el menu
	*/
	menu_labels.prototype.close = function()
	{
		var self = this;
		this.data.container.remove();
		this.finalize({
			onClosedFn: function() {
				self.destroy_assigned_label();
				self.destroy_assigned_prediction();
			}
		});
	};

	//===================================================================================================
	//===================================================================================================
	/**
	Class menu_labels_intents.
	@param params
	*/
	function menu_labels_intents ( params )
	{
		menu_labels_intents.uber.constructor.call(this, params);
		this.title_class = "windowUtterancesLabels";
	}
	inConcert.Utils.inherit(menu_labels_intents, menu_labels);

	//===================================================================================================
	//===================================================================================================
	/**
	Class menu_labels_entities.
	@param params
	*/
	function menu_labels_entities ( params )
	{
		menu_labels_entities.uber.constructor.call(this, params);
		this.title_class = "windowAssignEntity";
	}
	inConcert.Utils.inherit(menu_labels_entities, menu_labels);

}());
