/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 9 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

//	"use strict";
	const format = inConcert.Utils.format;

	var format_date = function(time_sec) {
		if (isNaN(time_sec)) {
			return;
		}
		let time_ms = time_sec * 1000;
		return inConcert.Utils.formatDate(time_ms, "date-short-year-short-hour");
	};

	//---------------------------------------------------------------------
	// form cognitive_services class - application section
	function adm_form_rasa_section_application(params)
	{
		var success = adm_form_rasa_section_application.uber.constructor.call(this, params);
		if (!success) {
			return;
		}
		this.view_action_id = "view_cognitive_services";  // view action id de permisos
		this.save_action_id  = { create: "save_cognitive_services", update: "save_cognitive_services" }; // save action id de permisos
		this.erase_action_id = null; // delete action id de permisos

		return true;
	}

	inConcert.Utils.inherit(adm_form_rasa_section_application, inConcert.conf_form_section);

	if ( typeof inConcert.adm_form_rasa_section_application !== "function" ) {
		inConcert.adm_form_rasa_section_application = adm_form_rasa_section_application;
	}

	//// Utiliza el de la clase Base de secciones
	// adm_form_rasa_section_application.prototype.dispose = $.noop;

	adm_form_rasa_section_application.prototype.get = function(rasa_app)
	{
		rasa_app = rasa_app || {};
		if ( this.frm.is_new ) {
			rasa_app.culture =  $("#adm_form_rasa_cultures").val();
		}

		rasa_app.name = $("#adm_form_rasa_name").val();
		rasa_app.description = $("#adm_form_rasa_desc").val();

		return rasa_app;
	};

	adm_form_rasa_section_application.prototype.set = function(rasa_app)
	{
		$("#adm_form_rasa_app_id").val(rasa_app.id);
		$("#adm_form_rasa_name").focus().val(rasa_app.name);
		$("#adm_form_rasa_desc").val(rasa_app.description);
		$("#adm_form_rasa_cultures").val(rasa_app.culture);
		$("#adm_form_rasa_runtime_port").val((rasa_app.runtime || {}).port || "N/A");
		$("#adm_form_rasa_app_info_creation_date").val(format_date(rasa_app.createdDateTime));

		const $rasa_imported_luis_app_section = $("#adm_form_rasa_imported_luis_app_section");
		const $rasa_imported_luis_app_id = $("#adm_form_rasa_imported_luis_app_id");

		if (rasa_app.imported) {
			$rasa_imported_luis_app_id.val(rasa_app.sourceImportedApp.exportedAppId);
			$rasa_imported_luis_app_section.show();
		}
		else {
			$rasa_imported_luis_app_id.val("");
			$rasa_imported_luis_app_section.hide();
		}
		this.update_app_version_info(rasa_app);
	};

	adm_form_rasa_section_application.prototype.get_entity = function()
	{
		return this.frm.get_entity();
	};

	adm_form_rasa_section_application.prototype.set_entity = function(rasa_app)
	{
		this.frm.set_entity(rasa_app);
	};

	/*
		A esta funcion la invoca esta seccion o puede ser invocada desde otras secciones (ej: train and publish)
	*/
	adm_form_rasa_section_application.prototype.update_app_version_info = function(info, callback = $.noop)
	{
		const self = this;

		const $train_date = $("#adm_form_rasa_app_info_last_train_date");
		const $intent_count = $("#adm_form_rasa_app_info_intent_count");
		const $nodes_state = $("#adm_form_rasa_app_info_nodes_state");
		const $nodes_state_container = $("#adm_form_rasa_app_info_nodes_state_contanier");
		const $nodes_state_list_container = $("#adm_form_rasa_app_info_nodes_state_list_container");
		const $nodes_state_list = $("#adm_form_rasa_app_info_nodes_state_list");

		let server_info = inConcert.app.getServerInfo();
		let is_multi_node = server_info  && server_info.isMultiNode;


		var update = function(version_info = {})
		{
			// 1 - La ultima fecha de TRAIN
			let last_trained_date;
			let is_running = false;

			if (version_info.trainInfo && $.isArray(version_info.trainInfo) && version_info.trainInfo.length > 0) {
				let last_train_steps = version_info.trainInfo[version_info.trainInfo.length - 1];
				if ($.isArray(last_train_steps) && last_train_steps.length > 0) {
					let last_train_step = last_train_steps[last_train_steps.length - 1];
					if (last_train_step && last_train_step.isRunning) {
						is_running = true;
						last_trained_date = format_date(last_train_step.startTrainDateTime);
					}
				}
			}

			if (!is_running) {
				if (version_info.lastTrainedDateTime) {
					last_trained_date = format_date(version_info.lastTrainedDateTime);
				}
				else {
					const app = self.frm.get_entity();
					if (app) {
						last_trained_date = format_date(app.lastTrainedDateTime);
					}
				}
			}

			$train_date.val(last_trained_date ||  "<%=_T('Not trained yet')%>");

			if (version_info.intentsCount) {
				const counter_template_msg = "{0}";
				// 2 - La cantidad de intents
				// Aca por defecto el valor es 1, ya que es el Intent None
				let intent_count = format(counter_template_msg, (version_info.intentsCount || 1));
				$intent_count.val(intent_count);
			}

//			let server_info = inConcert.app.getServerInfo();
//			if (!server_info || !server_info.isMultiNode) {
			if (!is_multi_node) {
				$nodes_state_container.hide();
			}
			else {
				$nodes_state_container.show();

				if (is_running) {
					$nodes_state_list_container.hide();
					let nodes_msg = "<%=_T('Training in Progress.')%>";
					$nodes_state.val(nodes_msg);
				}
				else {
					if (version_info.publications) {
						$nodes_state_list_container.hide();
						let nodes_msg = "<%=_T('All nodes updated')%>";
						let nodes_pending_to_update;
						if (server_info.nodesInfo && (server_info.nodesInfo.nodes || []).length > 1) {
							nodes_pending_to_update = info.publications.length;
							if (nodes_pending_to_update > 0) {
								if (nodes_pending_to_update === 1) {
									nodes_msg = format("<%=_T('{0} node remain to be updated.')%>", nodes_pending_to_update);
								}
								else {
									nodes_msg = format("<%=_T('{0} nodes remain to be updated.')%>", nodes_pending_to_update);
								}
								let nodes_text = "";
								for (let i = 0; i < nodes_pending_to_update; i++) {
									let node_pending = info.publications[i];
									if (nodes_text.length > 0) {
										nodes_text = nodes_text + "; ";
									}
									nodes_text = nodes_text + node_pending.nodeIP;
								}
								$nodes_state_list_container.show();
								$nodes_state_list.val(nodes_text);
							}
						}
						$nodes_state.val(nodes_msg);
					}
				}
			}

			callback();
			self.frm.dashboard.hide_waiting();
		};

		update(info);
	};

	adm_form_rasa_section_application.prototype.get_validation_def = function(definitions)
	{
		definitions = definitions || [];
		definitions.push(
			{
				control_id: "adm_form_rasa_name",
				messages: {
					required: "<%=_T('Please enter the cognitive service Name.')%>",
					id      : "<%=_T('Only letters, numbers, dashes and dots are allowed in the cognitive service Name.')%>"
				}
			},
			{
				control_id: "adm_form_rasa_desc",
				messages: {
					required: "<%=_T('Please enter the cognitive service Description.')%>"
				}
			},
			{
				control_id: "adm_form_rasa_cultures",
				messages: {
					required: "<%=_T('Please select the Culture.')%>"
				}
			}
		);
	};

	adm_form_rasa_section_application.prototype.attach_load_request = function(request)
	{
		if (!$.isArray(request)) {
			return;
		}
		const rasaApi = inConcert.app.getRasaAPI();
		const self = this;
		request.push(
			// cultures
			{
				request_function: function(success) {
					return rasaApi.loadCultures(success);
				},
				success: function(list) {
					const $cultures = $("#adm_form_rasa_cultures");
					const is_blank_first_item = true;
					const attr_id = "code";
					const attr_name = "name";
					const sorted = true;
					self.frm.fill_combo($cultures, is_blank_first_item, list, attr_id, attr_name, sorted);
				}
			}
		);
	};

	adm_form_rasa_section_application.prototype.editable = function()
	{
		adm_form_rasa_section_application.uber.editable.call(this);
		this.frm.editable_buttons();
	};

	adm_form_rasa_section_application.prototype.readonly = function()
	{
		adm_form_rasa_section_application.uber.readonly.call(this);
		this.frm.readonly_buttons();
	};

	adm_form_rasa_section_application.prototype.only_modified = function(rasa_app)
	{
		const $cultures = $("#adm_form_rasa_cultures");
		if (this.frm.is_new) {
			this.frm.readonly_control($cultures, false /* disabled */, true /* is_required */);
		}
		else {
			this.frm.readonly_control($cultures, true /* disabled */);
		}
		$("#adm_form_rasa_name").focus();

		const $app_id = $("#adm_form_rasa_app_id");
		this.frm.readonly_control($app_id, true /* disabled */);
		if (rasa_app) {
			$app_id.val(rasa_app.id);
		}
		const $rasa_imported_luis_app_id = $("#adm_form_rasa_imported_luis_app_id");
		this.frm.readonly_control($rasa_imported_luis_app_id, true /* disabled */);

		const $rasa_runtime_port = $("#adm_form_rasa_runtime_port");
		this.frm.readonly_control($rasa_runtime_port, true /* disabled */);

		this.show_or_hide_app_info();
	};

	adm_form_rasa_section_application.prototype.show_or_hide_app_info = function()
	{
		const $info_container = $("#adm_form_rasa_app_info");
		if (this.frm.is_new) {
			$info_container.hide();
			return;
		}

		const $creation_date = $("#adm_form_rasa_app_info_creation_date");
		const $train_date = $("#adm_form_rasa_app_info_last_train_date");
		const $nodes_state = $("#adm_form_rasa_app_info_nodes_state");
		const $published_date = $("#adm_form_rasa_app_info_last_published_date");
		const $intent_count = $("#adm_form_rasa_app_info_intent_count");
		const $entity_count = $("#adm_form_rasa_app_info_entity_count");
		const $total_endpoint_hits = $("#adm_form_rasa_app_info_total_endpoint_hits");
		const $nodes_state_list = $("#adm_form_rasa_app_info_nodes_state_list");

		this.frm.readonly_control($creation_date, true /* disabled */);
		this.frm.readonly_control($train_date, true /* disabled */);
		this.frm.readonly_control($nodes_state, true /* disabled */);
		this.frm.readonly_control($published_date, true /* disabled */);
		this.frm.readonly_control($intent_count, true /* disabled */);
		this.frm.readonly_control($entity_count, true /* disabled */);
		this.frm.readonly_control($total_endpoint_hits, true /* disabled */);
		this.frm.readonly_control($nodes_state_list, true /* disabled */);

		$info_container.show();
	};

	adm_form_rasa_section_application.prototype.ready = function()
	{
		this.only_modified();
	};

	adm_form_rasa_section_application.prototype.on_show = function()
	{
		adm_form_rasa_section_application.uber.on_show.call(this, {show: false});

		const is_readonly = this.is_readonly();
		if (is_readonly) {
			this.readonly();
		}
		else {
			this.editable();
		}
	};

	adm_form_rasa_section_application.prototype.can_jump_to_other_menunav = function()
	{
		let resp = true;
		const old_rasa_app = this.frm.rasa_app;
		const curr_rasa_app = this.get(inConcert.Utils.cloneObject(old_rasa_app));
			
		if (!old_rasa_app || !curr_rasa_app || inConcert.Utils.diffObjects(curr_rasa_app, old_rasa_app)) {
			resp = false;
		}
		return resp;
	};

	adm_form_rasa_section_application.prototype.before_jump_to_other_menunav = function()
	{
		this.set(this.frm.rasa_app);
	};

}());
