/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:false, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 11 */

/* global $, inConcert, CInconcertPlayer */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
//	"use strict";

	const format = inConcert.Utils.format;

	//---------------------------------------------------------------------
	// dashboard class
	function conf_dashboard(params)
	{
		if (typeof params !== "object" || typeof params.apps !== "object" || typeof params.permissions !== "object") {
			throw("Invalid app configuration!!");
		}
		const self = this;
		this.prefix_id = params.prefix_id || "conf_";
		this.$dashboard = params.dashboard || $("#configuration_dashboard");
		this.created_dashboard = false;
		this.group_permission = false;
		this.app_counts = 0;
		this.current_app = null;
		this.current_action = null;
		this.api       = this.api      || new inConcert.conf_api();
		this.view      = this.view     || new inConcert.conf_view( this ); // allow child classes to override the view
		this.realtime  = this.realtime || new inConcert.conf_realtime();
		this.permissions = params.permissions;
		this.apps_list = {};
		this.popups_windows = {};
		this.main_html = params.main_html;
		this.timeout_handle_show_back_button = null;
		this.timeout_handle_hide_message = null;
		this.timeout_handle_show_message = null;

		// add apps config to dashboard
		$.each(params.apps, function( _, config ) {
			self.add_app(config);
		});

		var menu_id = this.prefix_id + "applications_menu ul";
		var menu_event_name = this.prefix_id + "_menu";
		this.menu_popup_instance = new inConcert.PopupWindow({
			ownerId  : menu_id,         // container id
			eventName: menu_event_name // evento click personalizado
		});
	}

	if ( typeof inConcert.conf_dashboard !== "function" ) {
		inConcert.conf_dashboard = conf_dashboard;
	}

	conf_dashboard.prototype.get_dom_element_by_id = function(id)
	{
		return this.prefix_id + id;
	};

	conf_dashboard.prototype.$get_dom_element_by_id = function(id)
	{
		return $("#" + this.prefix_id + id);
	};

	conf_dashboard.prototype.add_app_css = function( app_id )
	{
		if ( !app_id || !this.apps_list[ app_id ] ) {
			return;
		}
		var add_css = this.apps_list[ app_id ].css || [];
		$.each( add_css , function( _, config ) {
			inConcert.Utils.css.add( config );
		});
	};

	conf_dashboard.prototype.remove_app_css = function( app_id )
	{
		if ( !app_id || !this.apps_list[ app_id ] ) {
			return;
		}
		var remove_css = this.apps_list[ app_id ].css || [];
		$.each( remove_css , function( _, config ) {
			inConcert.Utils.css.remove( config.id );
		});
	};

	conf_dashboard.prototype.remove_all_apps_css = function()
	{
		var self = this;
		$.each( this.apps_list || {} , function( app_id ) {
			self.remove_app_css( app_id );
		});
	};

	conf_dashboard.prototype.update_current_app_css = function()
	{
		this.remove_all_apps_css();
		this.add_app_css( this.current_app );
		this.apply_body_css_classes();
	};

	conf_dashboard.prototype.apply_body_css_classes = function()
	{
		const windowManager = inConcert.app.getWindowManager();
		const skin_class_name = this.get_skin();
		windowManager.applySkin(window, skin_class_name);
	};

	conf_dashboard.prototype.get_skin = function(app_id)
	{
		app_id = app_id || this.current_app;
		if (!app_id) {
			return false;
		}
		let app = this.apps_list[app_id];
		if (!app) {
			return false;
		}
		if (typeof app.get_skin !== "function") {
			return false;
		}
		return app.get_skin();
	};

	conf_dashboard.prototype.add_app = function( config )
	{
		if (typeof config !== "object" || typeof config.get_app_config !== "function") {
			throw("Invalid app configuration!!");
		}
		var app_config = config.get_app_config();
		if ( !app_config.can_show() ) {
			return;
		}
		this.apps_list[app_config.id] = app_config;
	};

	conf_dashboard.prototype.get_api = function()
	{
		return this.api;
	};

	conf_dashboard.prototype.get_current_app = function()
	{
		if (!this.apps_list || !this.current_app) {
			return;
		}

		return this.apps_list[this.current_app];
	};

	conf_dashboard.prototype.get_current_view_id = function()
	{
		return this.current_action;
	};

	conf_dashboard.prototype.add_popup_window = function( id, popup )
	{
		if ( typeof id !== "string" || typeof popup !== "object" ) {
			return;
		}
		this.popups_windows = this.popups_windows || {};
		this.popups_windows[ id ] = popup;
	};

	conf_dashboard.prototype.remove_popup_window = function( id )
	{
		if ( typeof id !== "string" || !this.popups_windows ) {
			return;
		}
		var popup = this.popups_windows[ id ];
		if ( !popup ) {
			return;
		}
		if ( typeof popup.close === "function" ) {
			popup.close();
		}
		delete this.popups_windows[ id ];
	};

	conf_dashboard.prototype.remove_all_popups_windows = function()
	{
		var self = this;
		$.each( this.popups_windows || {}, function( id ) {
			self.remove_popup_window( id );
		});
	};

	conf_dashboard.prototype.get_popups_windows = function()
	{
		return this.popups_windows;
	};

	conf_dashboard.prototype.get_current_action_obj = function(current_action)
	{
		if (!this.apps_list || !this.current_app) {
			return;
		}
		if (!this.apps_list[this.current_app]) {
			return;
		}
		return this.apps_list[this.current_app].navigation[current_action || this.current_action];
	};

	conf_dashboard.prototype.hide_selected_app_in_menu = function()
	{
		this.$get_dom_element_by_id( "applications_menu ul li[app_id]" ).show();
		this.$get_dom_element_by_id( "applications_menu ul li[app_id='" + this.current_app + "']" ).hide();
	};

	conf_dashboard.prototype.toggle_app_menu = function( callback )
	{
		const $applications_menu = this.$get_dom_element_by_id("applications_menu");
		$applications_menu.removeClass("show");
		if ( this.menu_popup_instance.isVisible() ) {
			callback = callback || $.noop;
			this.menu_popup_instance.hide( callback );
		}
		else {
			$applications_menu.addClass("show");
			this.hide_selected_app_in_menu(); // hide selected application in menu options
			this.menu_popup_instance.show();
		}
	};

	conf_dashboard.prototype.toggle_navigation = function(force_action = "NONE")
	{
		let result = {};
		const $navigation = this.$get_dom_element_by_id("navigation");
		if (force_action === "SHOW-NAVIGATION") {
			$navigation.show();
			result.visible = true;
			return result;
		}
		else if (force_action === "HIDE-NAVIGATION") {
			$navigation.hide();
			result.visible = false;
			return result;
		}
		let is_visible_navigation = $navigation.is(":visible");
		if (is_visible_navigation) {
			$navigation.hide();
			result.visible = false;
		}
		else {
			$navigation.show();
			result.visible = true;
		}
		return result;
	};

	conf_dashboard.prototype.open_app = function( app_id, selected_action, config, is_reopen)
	{
		if ( this.current_app === app_id && selected_action === undefined ) {
			return;
		}
		this.current_app = app_id || "";
		if ( !this.apps_list[ app_id ] ) {
			return;
		}
		this.update_current_app_css();
		var app_name = this.apps_list[ app_id ].name;
		this.$get_dom_element_by_id( "applications_menu label" ).text( app_name );
		var first_action = this.build_app_navigation();
		selected_action = selected_action || first_action;
		this.open_view( app_name, selected_action, config, is_reopen);
	};

	conf_dashboard.prototype.open_view = function( _ /* app id */, action_id, config, is_reopen)
	{
		is_reopen = is_reopen || false;
		const self = this;

		self.view.render(action_id,
			function( success ) {
				if ( !success ) {
					return;
				}
				inConcert.Utils.inline_manual_update();
				if ( config && config.open_frm ) {
					self.render_form( undefined, config );
				}
			},
			is_reopen, config
		);
	};

	conf_dashboard.prototype.reopen_view = function(config)
	{
		const current_action_id = this.get_current_view_id();
		const is_reopen = true;
		this.open_view(this.current_app, current_action_id, config, is_reopen);
	};

	// parametros opcionales
	conf_dashboard.prototype.render_form = function( $row, extra_parameters )
	{
		this.view.render_form( $row, extra_parameters );
	};

	conf_dashboard.prototype.set_events = function()
	{
		var self = this;
		if (this.app_counts > 1) {
			this.$get_dom_element_by_id("applications_menu")
				.off("click")
				.on("click", function() {
					self.toggle_app_menu();
				});
			this.$get_dom_element_by_id("applications_menu ul li")
				.off("click")
				.on("click", function() {
					var app_id = $(this).attr("app_id");
					self.toggle_app_menu( function() {
						self.open_app(app_id);
					});
				});
		}

		var $menu_items = this.$get_dom_element_by_id("application_actions");
		// items del menu de navegacion
		var event_selector = "li[app_id][action_id]";
		$menu_items
			.off("click", event_selector)
			.on("click", event_selector, function() {
				var action_id = $(this).attr("action_id");
				self.open_view( self.current_app, action_id );
			});

		// boton BACK del menu (opciones con submenu)
		event_selector = "button[type_button='BACK']";
		$menu_items
			.off("click", event_selector)
			.on("click", event_selector, function(event) {
				const section_frm = self.view.form.get_frm_section(self.view.form.current_section);
				self.confirm_jump_menu_nav({
					fn_can_confirm    : function() {
						//Debo mostrar el confirmar, si no puedo ir a otra item del nav
						if (!section_frm) {
							return false;
						}

						if (section_frm.can_jump_to_other_menunav()) {
							section_frm.before_jump_to_other_menunav();
							return false;
						}
						return true;
					},
					fn_process : function (success) {
						if (success === false) {
							return;
						}
						if (success && section_frm) {
							section_frm.before_jump_to_other_menunav();
						}
						
						self.view.form.close();
					}});
				event.stopPropagation(); // TODO: Ver de quitar el stop. Este stop lo agregue porque cuando doy click me ejecuta tambien el click del menu principal de la navegacion.
			});

		// items del submenu de navegacion/frm
		event_selector = "ul[submenu='true'] > li";
		$menu_items
			.off("click", event_selector)
			.on("click", event_selector, function (event) {
				var $sub_menu_item = $(this);
				var sub_menu_id = $sub_menu_item.attr("sub_menu_id");
				const section_frm = self.view.form.get_frm_section(self.view.form.current_section);
				self.confirm_jump_menu_nav({
					fn_can_confirm    : function() {
						const current_menu_id = self.view.form.current_section;
						const new_menu_id   = sub_menu_id;
						const val_require_params = current_menu_id && new_menu_id && new_menu_id !== "" && current_menu_id !== "";
						if (!val_require_params || current_menu_id === new_menu_id || !section_frm ){
							return false;
						}
						//Debo mostrar el confirmar, si no puedo ir a otra item del nav
						if (section_frm.can_jump_to_other_menunav()) {
								section_frm.before_jump_to_other_menunav();
							return false;
						}
						return true;
					},
					fn_process : function (success) {
						if (success === false) {
							return;
						}
						if (success) {
							section_frm.before_jump_to_other_menunav();
						}
						self.view.form.select_section(sub_menu_id);
					}
				});
				event.stopPropagation(); // TODO: Ver de quitar el stop. Este stop lo agregue porque cuando doy click me ejecuta tambien el click del menu principal de la navegacion.
			});

		// Vistas: cruz que cierra los status messages
		var $message_container = this.$get_dom_element_by_id("container");
		event_selector = "#" + this.prefix_id + "view_message_close";
		$message_container
			.off( "click", event_selector )
			.on(  "click", event_selector, function() {
				self.view.hide_status_message();
			});

		// Formulario (skin 2021): cruz que cierra los status messages
		event_selector = "#" + this.prefix_id + "form_message_close";
		$message_container
			.off( "click", event_selector )
			.on(  "click", event_selector, function() {
				self.view.form.hide_status_message();
			});
	};

	conf_dashboard.prototype.disabled_all_menus = function(disabled)
	{
		var app_id = this.current_app;
		disabled = disabled ? true : false;
		var $menu_list = this.$get_dom_element_by_id("application_actions > li[app_id='" + app_id + "']");
		$menu_list.prop("disabled", disabled);
		if (disabled) {
			$menu_list.addClass("disabled");
		}
		else {
			$menu_list.removeClass("disabled");
		}
	};

	conf_dashboard.prototype.hide_all_menus = function()
	{
		var app_id = this.current_app;
		var $menu_list = this.$get_dom_element_by_id("application_actions > li[app_id='" + app_id + "']");
		$menu_list.hide();
	};

	conf_dashboard.prototype.show_all_menus = function()
	{
		var app_id = this.current_app;
		var $menu_list = this.$get_dom_element_by_id("application_actions > li[app_id='" + app_id + "']");
		$menu_list.show();
	};

	conf_dashboard.prototype.$get_menu_action = function(action_id)
	{
		var app_id = this.current_app;
		action_id = action_id || this.current_action;
		return this.$get_dom_element_by_id("application_actions > li[app_id='" + app_id + "'][action_id='" + action_id + "']");
	};

	conf_dashboard.prototype.enable_menu_item = function(action_id)
	{
		var disabled = false;
		var $menu_action = this.$get_menu_action(action_id);
		$menu_action
			.prop("disabled", disabled)
			.removeClass("disabled");
	};

	conf_dashboard.prototype.show_menu_item = function(action_id)
	{
		var $menu_action = this.$get_menu_action(action_id);
		$menu_action.show();
	};

	conf_dashboard.prototype.disable_menu_item = function(action_id)
	{
		var disabled = true;
		var $menu_action = this.$get_menu_action(action_id);
		$menu_action
			.prop("disabled", disabled)
			.addClass("disabled");
	};

	// funcion privada, que obtiene la lista del sub menu
	var $get_sub_menu = function( dashboard )
	{
		// una accion (opcion "campaign", etc) del menu de navegacion
		var $menu_option = dashboard.$get_menu_action();
		// lista contenedora del submenu
		var $sub_menu = $menu_option.find("ul[submenu='true']");
		return $sub_menu;
	};

	conf_dashboard.prototype.build_app_navigation_sub_menu = function(sections)
	{
		if (typeof sections !== "object") {
			return;
		}
		var $sub_menu = $get_sub_menu(this);
		var sub_menu_length = $sub_menu.find("li").length;
		if (sub_menu_length > 0) {
			return;
		}
		// cada seccion se agrega como una opcion del sub menu
		$.each( sections, function( option_id, option )
		{
			var $option = $("<li class='applicationOptionsSubitem'></li>");

			let html_name = option.name || "";

			if (option.i_class_name && option.i_class_name !== "") {
				//html_name = `<i class="${option.i_class_name}"></i> ${html_name}`;
				html_name = `<i class=" icon  ${option_id} ${option.i_class_name}"></i> ${html_name}`;
			}

			$option.html(html_name);
			$option.attr("sub_menu_id", option_id);

			var initial_state = (option.initial_state || "disabled").toLowerCase();

			if (initial_state === "disabled") {
				$option.prop("disabled", true);
				$option.addClass("disabled");
				$option.show();
			}
			else if (initial_state === "enabled") {
				$option.prop("disabled", false);
				$option.removeClass("disabled");
				$option.show();
			}
			else if (initial_state === "hidden") {
				$option.prop("disabled", false);
				$option.removeClass("disabled");
				$option.hide();
			}
			else if (initial_state === "hidden") {
				throw "conf_dashboard.prototype.build_app_navigation_sub_menu. Unrecognized initial state: " + initial_state;
			}

			$sub_menu.append($option);
		});
	};

	conf_dashboard.prototype.hide_all_sub_menus = function()
	{
		var app_id = this.current_app;
		var $menu_options = this.$get_dom_element_by_id("application_actions > li[app_id='" + app_id + "'] > ul[submenu='true']");
		$menu_options.hide();
	};

	conf_dashboard.prototype.disabled_all_sub_menus = function( disabled )
	{
		disabled = disabled ? true : false;
		var $sub_menu = $get_sub_menu(this);
		var $sub_menu_items = $sub_menu.find("[sub_menu_id]");
		
		if ( disabled) {
			$sub_menu_items.prop("disabled", true);
			$sub_menu_items.removeClass("optionSelected").addClass("disabled");
		}
		else {
			$sub_menu_items.prop("disabled", false);
			$sub_menu_items.removeClass("disabled");
			this.hide_back_button();
		}
	};

	conf_dashboard.prototype.disable_sub_menu_option = function( sub_menu_id, disabled )
	{
		if (typeof sub_menu_id !== "string") {
			return;
		}
		disabled = disabled ? true : false;
		var $sub_menu = $get_sub_menu(this);
		var $sub_menu_item = $sub_menu.find("[sub_menu_id='" + sub_menu_id + "']");
		$sub_menu_item
			.prop("disabled", disabled)
			.removeClass("optionSelected");
		if (disabled) {
			$sub_menu_item.addClass("disabled");
		}
		else {
			$sub_menu_item.removeClass("disabled");
		}
	};

	conf_dashboard.prototype.show_sub_menu_option = function( sub_menu_id )
	{
		if (typeof sub_menu_id !== "string") {
			return;
		}
		var $sub_menu = $get_sub_menu(this);
		var $sub_menu_item = $sub_menu.find("[sub_menu_id='" + sub_menu_id + "']");
		$sub_menu_item.show();
	};

	conf_dashboard.prototype.hide_sub_menu_option = function( sub_menu_id )
	{
		if (typeof sub_menu_id !== "string") {
			return;
		}
		var $sub_menu = $get_sub_menu(this);
		var $sub_menu_item = $sub_menu.find("[sub_menu_id='" + sub_menu_id + "']");
		$sub_menu_item.hide().removeClass("optionSelected");
	};

	conf_dashboard.prototype.mark_sub_menu_option = function(sub_menu_id)
	{
		if (typeof sub_menu_id !== "string") {
			return;
		}
		var $sub_menu = $get_sub_menu(this);
		$sub_menu.find("[sub_menu_id]").removeClass("optionSelected");
		$sub_menu.find("[sub_menu_id='" + sub_menu_id + "']").addClass("optionSelected");
	};

	conf_dashboard.prototype.show_back_button = function()
	{
		const self = this;

		if (!this.view.form.has_sections()) {
			return;
		}
		this.hide_back_button();
		const $current_action = this.$get_menu_action();
		const $back_button = $current_action.find("button[type_button='BACK']");
		const $list_item_label = $current_action.find("[item_name]");
		const elapsed_time_ms = 500;
		let show;
		$list_item_label.hide();
		$back_button.show();
		const action_obj = self.get_current_action_obj();
		const change_nav_sections =
			action_obj && action_obj.view && action_obj.view.change_nav_sections;
		if (change_nav_sections) {
			/*
						  TODO Esta validacion 'change_nav_sections' es para cambios nuevos, definidos por UX/UI en las secciones.
						  Por el momento para navegaciones seleccionadas.
						  Eliminar cuando se haya agregado en todos los FORM que usen secciones.
						  */
			$back_button.html(
				`<i class='fas fa-chevron-left'></i> <label>${self.view.form.get_entity_display_name()}</label>`
			);
			$back_button.removeClass().addClass("goBackDivButton");
		}

		// Para que se pueda visualizar el boton "back", todas las secciones tienen que haber terminado de cargar,
		//   entonces ejecuto la funcion de validacion en un timeout de N milisegundos que va validando si las secciones se han cargado.
		show = function()
		{
			if (self.timeout_handle_show_back_button) {
				window.clearTimeout(self.timeout_handle_show_back_button);
				self.timeout_handle_show_back_button = null;
			}
			if (!self.view.form.is_loading_sections()) {
				//console.log("conf_dashboard.prototype.show_back_button - SHOW");
				$back_button.prop("disabled", false);
				return;
			}
			self.timeout_handle_show_back_button = window.setTimeout(function() {
				show();
			}, elapsed_time_ms);
		};

		show();
	};

	conf_dashboard.prototype.hide_back_button = function()
	{
		//console.log("conf_dashboard.prototype.hide_back_button - HIDE");
		const $current_action = this.$get_menu_action();
		const $back_button = $current_action.find("button[type_button='BACK']");
		$back_button.prop("disabled", true);
		if (!$back_button.is(":visible")) {
			return;
		}
		const $list_item_label = $current_action.find("[item_name]");
		$back_button.hide();
		$list_item_label.show();
	};

	conf_dashboard.prototype.disable_back_button = function(is_disable = false)
	{
		//console.log("conf_dashboard.prototype.disable_back_button - disable:" + is_disable);
		const $current_action = this.$get_menu_action();
		const $back_button = $current_action.find("button[type_button='BACK']");
		$back_button.prop("disabled", is_disable);
		if (is_disable) {
			$back_button.addClass("goBackButton-ds");
		}
		else {
			$back_button.removeClass("goBackButton-ds");
		}
	};

	conf_dashboard.prototype.show_sub_menu = function()
	{
		this.hide_all_sub_menus(); // oculto todos los submenues antes de mostrar el de la current action
		if (!this.view.form.has_sections()) {
			return;
		}
		var $sub_menu = $get_sub_menu(this);
		$sub_menu.show();
		this.disabled_all_menus(true);
		this.enable_menu_item(this.current_action);
		this.hide_all_menus();
		this.show_menu_item(this.current_action);
		this.hide_back_button();
	};

	conf_dashboard.prototype.on_form_ready = function()
	{
		this.show_sub_menu();
		if (this.allow_disable_all_navigation()) {
			this.disabled_all_menus(true);
			this.enable_menu_item(this.current_action);
		}
		this.view.set_in_view(false);
	};

	conf_dashboard.prototype.on_view_ready = function()
	{
		this.disabled_all_sub_menus(true);
		this.hide_all_sub_menus();
		this.show_all_menus();
		this.disabled_all_menus(false);
		this.hide_back_button();
		const action = this.get_current_action_obj();
		if (action && action.view && action.view.on_ready) {
			const result = action.view.on_ready();
			if (result && result.action === "REFRESH_VIEW") {
				this.grid_refresh();
			}
		}
	};

	conf_dashboard.prototype.grid_refresh = function(message, callback = $.noop) {
		this.view.refresh(message, callback);
	};

	conf_dashboard.prototype.build_app_navigation = function()
	{
		var self = this;
		var selected_action;
		var $app_actions_container = this.$get_dom_element_by_id( "application_actions" );
		$app_actions_container.empty();
		var app_id = self.current_app;

		if ( !this.apps_list[ app_id ] ) {
			return;
		}
		var navigation = this.apps_list[ app_id ].navigation;

		var add_view_to_navigation = function( action_id, action )
		{
			var classes = ( action.class_name || "" ) + " " + ( action.selected ? "optionSelected" : "" );
			var i_classes = action.i_class_name || "";
			
			var $html_action = $( 
				"<li app_id='" + app_id + "' action_id='" + action_id + "' class='applicationOptionsItem " + classes + "'>" +
					"<button type_button='BACK' type='button' class='goBackButtonText' style='display:none;'><i class='fas fa-chevron-left'></i> <%=_T('Go back')%></button>" +
					"<span item_name=''>" + "<i class='" + i_classes +  "'></i>" + action.name + "</span>" +
					"<ul submenu='true' style='display:none;'></ul>" +
				"</li>"
			);
			
			$app_actions_container.append( $html_action );
			if ( !selected_action ) {
				selected_action = action_id;
			}
			if ( action.selected ) {
				selected_action = action_id;
			}
		};

		$.each( navigation, function( action_id, action )
		{
			var has_permission = false;
			if ( typeof(action.can_show) === "function" && !action.can_show() ) {
				return true; // continue
			}
			if ( typeof action.permissions.view === "string" ) {
				has_permission = self.permissions.to_navigation( app_id, action.permissions.view );
			}
			else {
				$.each( action.permissions.view || {},
					function( _, view_permission_id ) {
						has_permission = self.permissions.to_navigation( app_id, view_permission_id );
						if ( has_permission ) {
							return false; // break
						}
					});
			}

			if ( has_permission ) {
				add_view_to_navigation( action_id, action );
			}
		});

		return selected_action;
	};

	conf_dashboard.prototype.build_app_menu = function()
	{
		var self = this;
		var $app_menu = this.$get_dom_element_by_id("applications_menu");
		var $app_menu_container = $app_menu.find("ul");
		$app_menu_container.empty().hide();
		self.app_counts = 0;
		var selected_app;

		$.each(this.apps_list, function(app_id, app) {
			var classes = (app.class_name || "");
			var html_menu_item = "<li app_id='{0}' class='applicationListItem {1}'>{2}</li>";
			html_menu_item = format(html_menu_item, app_id, classes, app.name);
			$app_menu_container.append(html_menu_item);

			if (!selected_app) {
				selected_app = app_id;
			}
			if (app.selected) {
				selected_app = app_id;
			}
			self.app_counts++;
		});
		//add/remove open menu items applications
		var $app_menu_selected = $app_menu.find("label");
		if (self.app_counts > 1) {
			if (!$app_menu_selected.hasClass("withAction")) {
				$app_menu_selected.addClass("withAction");
			}
		}
		else {
			if ($app_menu_selected.hasClass("withAction")) {
				$app_menu_selected.removeClass("withAction");
			}
		}
		this.open_app(selected_app);
	};

	conf_dashboard.prototype.show_status_message = function(message, type, timeout_ms, hide_close_button, scrolling, show_custom_button, $custom_message_container, callbacks, wrap_classes)
	{
		var self = this;

		var callback_on_show = $.noop;
		var callback_on_hide = $.noop;
		if (callbacks) {
			callback_on_show = callbacks.on_show || $.noop;
			callback_on_hide = callbacks.on_hide || $.noop;
		}

		var show = function() {
			if (type.toLowerCase() === "error") {
				inConcert.logger.action({message : message, app: "configuration"});
			}
			self.hide_waiting();
			var $message = $custom_message_container || self.$get_dom_element_by_id("view_message");
			if (message) {
				$message.removeClass().addClass("genericFormMessagesContainer").addClass("message" + type);
				$message.show().find(".messageText").html(message);
			}

			if (wrap_classes) {
				const $wrap = $(`<div ${self.prefix_id}message_wrap="true" class="${wrap_classes}"></div>`);
				$message.wrap($wrap);
			}

			if (timeout_ms !== false) {
				timeout_ms = timeout_ms || 3000;
				self.timeout_handle_hide_message = window.setTimeout(function() {
					self.hide_status_message(null, null, $message);
					callback_on_hide();
				}, timeout_ms);
			}

			var $close = self.$get_dom_element_by_id("view_message_close");
			if (hide_close_button) {
				$close.hide();
			}
			else {
				$close.show();
			}
			var $custom = $message.find("[custom_action]");
			if (typeof(show_custom_button) === "object" && typeof(show_custom_button.callback) === "function") {
				$custom
					.html(show_custom_button.message || "")
					.show()
					.off("click")
					.on("click", function() {
						show_custom_button.callback();
					});
			}
			else {
				$custom.hide();
			}
			if (scrolling) {
				self.view.scroll_to("top");
			}

			callback_on_show();
		};

		if (this.timeout_handle_show_message) {
			window.clearTimeout(this.timeout_handle_show_message);
			this.timeout_handle_show_message = null;
		}
		this.timeout_handle_show_message = window.setTimeout(function() {
			show();
		}, 100);
	};

	conf_dashboard.prototype.hide_status_message = function(timeout_ms, callback, $custom_message_container)
	{
		const self = this;
		callback = callback || $.noop;

		if (this.timeout_handle_hide_message) {
			window.clearTimeout(this.timeout_handle_hide_message);
			this.timeout_handle_hide_message = null;
		}

		const remove_message_wrap = function ($message_container) {
			if (!$message_container) {
				return;
			}
			const $parent = $message_container.parent();
			if ($parent.attr(`${self.prefix_id}message_wrap`)) {
				$message_container.unwrap();
			}
		};
		const hide = function($message_container) {
			if (!$message_container) {
				return;
			}
			$message_container.hide();
			$message_container.find("[custom_action]").unbind(); // unbind all events
			remove_message_wrap($message_container);
		};

		const hide_all = function() {
			// se ocultan todos los mensajes, ya a paritr del nuevo diseno "skin 2021", el mensaje ya no esta en unico contenedor, sino que hay uno para la vista y otro para el frm.
			hide(self.$get_dom_element_by_id("view_message")); // contenedor del mensaje de la vista.
			hide(self.$get_dom_element_by_id("form_message")); // contenedor del mensaje del formulario.
			hide($custom_message_container);                   // contenedor personalizado del mensaje.
			// oculto el waiting
			self.hide_waiting();
			// oculto las cruces de errores de validaciones en los formularios.
			const $errors = self.$get_dom_element_by_id("form_data_container .formFieldValidation");
			$errors.hide();
			callback();
		};

		if ( timeout_ms ) {
			window.setTimeout(function() {
				hide_all();
			}, timeout_ms);
		}
		else {
			hide_all();
		}
	};

	conf_dashboard.prototype.show_waiting = function(message, $waiting_container)
	{
		// console.log("conf_dashboard.prototype.show_waiting");
		this.hide_scroll_waiting();
		this.hide_status_message();
		var $waiting;
		if ($waiting_container) {
			$waiting = $waiting_container;
		}
		else {
			$waiting = this.$get_dom_element_by_id("view_waiting");
		}
		$waiting.show().find("[waiting_message]").text(message || "");

		if (this.canDisableNewButton()) {
			this.$get_dom_element_by_id("view_button_new").prop("disabled", true);
		}
	};

	conf_dashboard.prototype.hide_waiting = function($waiting_container)
	{
		// console.log("conf_dashboard.prototype.hide_waiting");
		this.hide_scroll_waiting();
		var $waiting;
		if ($waiting_container) {
			$waiting = $waiting_container;
		}
		else {
			$waiting = this.$get_dom_element_by_id("view_waiting");
		}
		$waiting.hide().find("[waiting_message]").text("");

		if (this.canEnableNewButton()) {
			this.$get_dom_element_by_id("view_button_new").prop("disabled", false);
		}
	};

	conf_dashboard.prototype.canEnableNewButton = function()
	{
		const action = this.get_current_action_obj();
		if (!action || !action.view || typeof action.view.canEnableNewButton !== "function") {
			return true;
		}
		return action.view.canEnableNewButton(this.view);
	};

	conf_dashboard.prototype.canDisableNewButton = function()
	{
		const action = this.get_current_action_obj();
		if (!action || !action.view || typeof action.view.canDisableNewButton !== "function") {
			return true;
		}
		return action.view.canDisableNewButton(this.view);
	};

	conf_dashboard.prototype.show_scroll_waiting = function(message = "")
	{
		const $view_scroll_waiting = this.$get_dom_element_by_id("view_scroll_waiting");
		$view_scroll_waiting.show().find("[type='message']").text(message);
	};

	conf_dashboard.prototype.hide_scroll_waiting = function()
	{
		this.$get_dom_element_by_id("view_scroll_waiting").hide();
	};

	/*
		Recarga la vista cuando ingresa a la app.
		Utilizado para recargar la app si la vista tiene configurada esta opcion en su definicion.
	*/
	conf_dashboard.prototype.reload_current_view = function()
	{
		const action = this.get_current_action_obj();
		if (!action) {
			return;
		}
		if (!action.view || !action.view.reload_when_enter_application) {
			return;
		}
		if (this.is_in_view()) {
			this.reopen_view();
		}
	};

	/*
		Para saber si nos encontramos en una vista o en un form.
	*/
	conf_dashboard.prototype.is_in_view = function()
	{
		return this.view.get_in_view();
	};

	conf_dashboard.prototype.render = function(callback = $.noop)
	{
		const self = this;
		if (this.created_dashboard) {
			this.$dashboard.show();
			this.reload_current_view();
			this.on_show(callback);
			return;
		}
		this.api.load_content_page({
			URI: this.main_html,
			callback: function(success, dashboard_content) {
				if (success && typeof dashboard_content === "string") {
					self.$dashboard.html(dashboard_content);
					self.build_app_menu();
					self.set_events();
					self.created_dashboard = true;
					self.$dashboard.show();
					self.on_show(callback);
				}
				else {
					self.created_dashboard = false;
					callback();
				}
			}
		});
	};

	conf_dashboard.prototype.on_shutdown = function()
	{
		if ( !this.created_dashboard ) {
			return;
		}
		// close all popups windows
		this.remove_all_popups_windows();
		// unsuscribe all listening events
		this.realtime.events_unsuscribe_all();
	};

	conf_dashboard.prototype.on_show = function(callback = $.noop)
	{
		if (!this.created_dashboard) {
			callback();
			return;
		}
		//update css configuration with current app
		this.update_current_app_css();
		//resume events
		this.realtime.resume();

		this.initial_full_screen();
		//invoca al callback para finalizar el render
		callback();
	};

	conf_dashboard.prototype.on_hide = function()
	{
		if ( !this.created_dashboard ) {
			return;
		}
		//remove css all configuration apps
		this.remove_all_apps_css();
		// pause events
		this.realtime.pause();
	};

	conf_dashboard.prototype.on_leave = function(callback = $.noop)
	{
		let curr_nav = this.get_current_action_obj();
		// Cambia de app, para la navegacion actual si tiene on_leave, ejecuto
		if (curr_nav && typeof curr_nav.on_leave === "function") {
			curr_nav.on_leave(this, this.view);
		}
		const must_init_full_screen = (curr_nav?.view?.must_init_full_screen) || function() {
			return false;
		};
		if (must_init_full_screen(this)) {
			//Si tiene must_init_full_screen, al entrar se oculta la navegacion, por lo que al salir, debe ser restaurado
			inConcert.app.toggleConfNavigation(true, this.$get_dom_element_by_id("grid_scroll"), this.$get_dom_element_by_id("navigation"));	
		}
		callback();
	};

	conf_dashboard.prototype.initial_full_screen = function()
	{
		const current_action = this.get_current_action_obj() || {};
		const must_init_full_screen = (current_action?.view?.must_init_full_screen) || function() {
			return false;
		};
		inConcert.app.toggleConfNavigation(!must_init_full_screen(this), this.$get_dom_element_by_id("grid_scroll"), this.$get_dom_element_by_id("navigation"));
	};

	conf_dashboard.prototype.can_show = function()
	{
		var can_show = false;
		$.each( this.apps_list, function( _, config ) {
			can_show = config.can_show();
			if ( can_show ) {
				return false; // break
			}
		});
		return can_show;
	};

	conf_dashboard.prototype.sanitize_value = function(value)
	{
		if (value === null || value === undefined) {
			return "";
		}
		if (typeof value === "number") {
			value = value.toString();
		}
		else if (typeof value !== "string") {
			return value;
		}
		// 1 - Si es una keyword del sistema NO los visualiza
		if (value === "__BLANK") {
			return "";
		}
		// 2 - Elimino todo el markup HTML y me quedo con el texto
		value = inConcert.Utils.htmlBodyToPlainText(value);
		// 3 - Convierto los Enters en </BR>
		value = inConcert.Utils.nl2br(value);
		return value;
	};

	conf_dashboard.prototype.escape = function(value)
	{
		if (typeof value !== "string") {
			return value;
		}
		// let escaped_value = _.escape(value);
		let escaped_value = value.replace(/\"/g, ""); // value.replace("'", "");
		escaped_value = escaped_value.replace(/\'/g, ""); //.replace('"', "");

		if (escaped_value !== value) {
			console.debug("--ESCAPED VALUE-----------------");
			console.debug("--- original value - " + value);
			console.debug("--- escaped  value - " + escaped_value);
			console.debug("---------------------------------" );
		}
		return escaped_value;
	};

	conf_dashboard.prototype.confirm = function(message, callback = $.noop, message_confirm = "<%=_T('Confirm')%>")
	{
		this.hide_status_message();
		$.inconcertPopup.Show(
			message_confirm,
			message,
			$.inconcertPopup.ButtonSets.BS_YESNO,
			function(response) {
				if (response.toLowerCase() !== "yes") {
					callback(false);
					return;
				}
				window.setTimeout(function() {
					callback(true);
				});
			},
			undefined,
			{
				popup_id       : "conf_inconcertPopup",
				container_class: "confirmationPopUp",
				overlay_class  : "confirmationOverlay"
			}
		);
	};

	conf_dashboard.prototype.set_title_img = function($item_img)
	{
		const action = this.get_current_action_obj();
		if (!action) {
			return;
		}
		$item_img.removeClass().addClass(action.i_class_name || "");
	};

	/* -----------------------------------------------------------------------------------
		AUDIO PLAYER
	----------------------------------------------------------------------------------- */

	/*
		Sample URI: "https://testubu.i6.inconcertcc.com/inconcert/apps/repository/download/?id=84072f21-5fde-4c2e-b4ce-04f2d94946d0&section=1&part=1";
	*/
	conf_dashboard.prototype.initialize_player = function(file_info, callback = $.noop)
	{
		const self = this;
		if (!this.must_shown_player()) {
			return;
		}
		this.m_player = this.m_player || {};
		this.stop_players();

		let dom_player_id = file_info.dom_player_id;
		let uri = this.make_repository_uri(file_info);

		this.m_player.instance = new CInconcertPlayer({
			containerId: dom_player_id,
			url        : uri,
			fileFormat : file_info.format,
			success: function() {
				// Callback cuando el audio se ha cargado correctamente.
				callback(true);
			},
			error: function() {
				// Callback cuando se produce un error en la carga del audio.
				const control_msg_desc = $("#" + file_info.dom_player_id).attr("control_msg_desc") || "";
				self.show_status_message(format("<%=_T('Error loading audio: {0}')%>", control_msg_desc), "Error", 5000);
				callback(false);
			}
		});
		this.m_player.instance.onEnded(function() {
			// Dejo el Callback como referencia por si tenemos que realizar algo cuando termina de ser reproducido el audio.
			const $play = $("#" + (self.m_player || {}).dom_owner_id);
			self.toggle_player_button($play, true /*play action*/);
		});
		this.m_player.instance.load();
	};

	conf_dashboard.prototype.must_shown_player = function()
	{
		return true;
	};

	conf_dashboard.prototype.stop_players = function()
	{
		if (!this.m_player) {
			return;
		}
		if (this.m_player.instance) {
			this.m_player.instance.stop();
		}
	};

	conf_dashboard.prototype.toggle_player_button = function($play, is_play)
	{
		if ($play.length === 0) {
			return;
		}
		const $i = $play.find("i");
		$i.removeClass("buttons-icons-stop buttons-icons-start");

		if (is_play) {
			$play.attr("title", "<%=_T('Play')%>");
			$i.addClass("buttons-icons-start");
		}
		else {
			$play.attr("title", "<%=_T('Stop')%>");
			$i.addClass("buttons-icons-stop");
		}
	};

	/*
		https://testubu.i6.inconcertcc.com/inconcert/apps/repository/download/?id=1F099DA7AFE67360746FFBE5D6825460&section=1
	*/
	conf_dashboard.prototype.make_repository_uri = function(file_info)
	{
		let uri = format(
			"{0}/inconcert/apps/repository/download/?id={1}&section={2}",
			inConcert.app.getBaseDomain(),
			encodeURIComponent(file_info.id),
			file_info.section /* section */
		);
		return uri;
	};

	conf_dashboard.prototype.make_player_container = function($play)
	{
		const dom_owner_id = $play.attr("id");
		let dom_player_id = dom_owner_id + "_player_container";
		const $player_container = $("#" + dom_player_id);
		if ($player_container.length > 0) {
			// already exist
			return dom_player_id;
		}
		let html_player = "<div id='" + dom_player_id + "' class='clientPlayContainer' style='display:none;' ></div>";
		$("body").append(html_player);
		return dom_player_id;
	};

	conf_dashboard.prototype.player_start = function($play)
	{
		const self = this;
		const dom_owner_id = $play.attr("id");
		if ((this.m_player || {}).dom_owner_id === dom_owner_id) {
			this.destroy_players();
			this.toggle_player_button($play, true /*play action*/);
			return;
		}
		this.destroy_players();
		this.toggle_player_button($play, false /*stop action*/);
		const file_info = $play.data("file_info");
		let dom_player_id = this.make_player_container($play);
		file_info.dom_player_id = dom_player_id;
		this.initialize_player(file_info, function(success) {
			if (success) {
				self.m_player.dom_owner_id = dom_owner_id;
				self.m_player.instance.play();
			}
		});
	};

	conf_dashboard.prototype.destroy_players = function()
	{
		if (!this.m_player) {
			return;
		}
		if (this.m_player.instance) {
			this.m_player.instance.destroy();
			const $play = $("#" + (this.m_player || {}).dom_owner_id);
			this.toggle_player_button($play, true /*play action*/);
		}
		this.m_player = null;
	};

	conf_dashboard.prototype.player_hide = function(/*control_id*/)
	{
		if (!this.m_player) {
			return;
		}
		const player = this.m_player.instance;
		if (!player) {
			return;
		}
		player.hide();
	};

	conf_dashboard.prototype.allow_disable_all_navigation = function()
	{
		/*
			Se agrega flag para definir si la nav de una app se deshabilitara cuando entre en modo edicion.
			Esto porque se va a empezar por modulos a permitir saltar en items de navegacion, mediante confirmacion.
		*/

		const curr_app = this.get_current_app();
		return curr_app && curr_app.allow_disable_all_navigation;
	};

	conf_dashboard.prototype.confirm_jump_menu_nav = function(params)
	{
		const fn_process = params && params.fn_process || $.noop;
		const fn_can_confirm = params && params.fn_can_confirm || function () {
			return false;
		};

		if (!this.allow_disable_all_navigation() && fn_can_confirm()) {
			this.confirm("<%=_T('Any unsaved changes will be lost!')%>", function (success) {
				fn_process(success);
			}, "<%=_T('Are you sure you want to leave?')%>");
		}
		else{
			fn_process();
		}
	};
}());

