/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 11 */

/* global inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

//	"use strict";

	//---------------------------------------------------------------------
	// form section class
	function conf_form_section(params)
	{
		this.frm = params.frm;
		this.section_id = params.section_id;
		this.save_action_id = null;
		this.erase_action_id = null;
		this.is_read_only = false;
		this.loading = false;
		this.loading_message = "<%=_T('Please wait.')%>";

		return true;
	}

	if ( typeof inConcert.conf_form_section !== "function" ) {
		inConcert.conf_form_section = conf_form_section;
	}

	var has_permission_to_action = function( self, action_id, campaign_id /*optional*/, category /*optional*/, all_overrides /*optional*/ )
	{
		if ( typeof action_id !== "string" ) {
			return false;
		}

		campaign_id = campaign_id || undefined;
		if ( self.frm.get_permissions_type() === "campaign" ) {
			var entity = self.get_entity_instance();
			if ( entity !== null && entity !== undefined ) {
				campaign_id = entity.Id;
			}
		}
		var dashboard = self.frm.dashboard;
		var app_id = dashboard.current_app;
		category = category || undefined;
		all_overrides = all_overrides || undefined;

		return dashboard.permissions.to_action( app_id, action_id, category, campaign_id, true /* no_log */, all_overrides );
	};

	conf_form_section.prototype.get = $.noop;

	conf_form_section.prototype.set = $.noop;

	conf_form_section.prototype.get_entity_instance = function() { return this.frm.get_entity_instance(); };

	conf_form_section.prototype.init = $.noop;

	conf_form_section.prototype.dispose = $.noop;

	conf_form_section.prototype.get_validation_def = $.noop;

	conf_form_section.prototype.attach_load_request = $.noop;

	conf_form_section.prototype.set_events = $.noop;

	conf_form_section.prototype.before_jump_to_other_menunav = $.noop;
	
	conf_form_section.prototype.set_edit_mode = function(isEditMode) {
		this.frm.set_edit_mode(isEditMode);
	};

	conf_form_section.prototype.can_jump_to_other_menunav = function() {
		return this.frm.can_jump_to_other_menunav();
	};

	conf_form_section.prototype.set_load = function(loading) { this.loading = loading; };

	conf_form_section.prototype.is_loading = function() {
		if (this.loading) {
			return true;
		}
		return this.is_loading_another_sections();
	};

	conf_form_section.prototype.realtime_update = $.noop;

	conf_form_section.prototype.readonly = $.noop;

	conf_form_section.prototype.ready = $.noop;

	conf_form_section.prototype.validate_entity = function() { return {success: true}; };

	conf_form_section.prototype.set_readonly = function( readonly ) { this.is_read_only = readonly; };

	conf_form_section.prototype.isAllegro = function() { return inConcert.app.has_allegro_integration(); };

	conf_form_section.prototype.is_readonly = function()
	{
		var is_readonly = !this.has_permission_to_save();
		this.set_readonly( is_readonly );
		return is_readonly;
	};

	conf_form_section.prototype.is_loading_another_sections = function() {
		return this.frm.is_loading_sections();
	};

	conf_form_section.prototype.$get_dom_element_by_id = function( id )
	{
		return this.frm.$get_dom_element_by_id( id );
	};

	conf_form_section.prototype.show_waiting = function( message, $waiting_container )
	{
		this.frm.show_waiting( message, $waiting_container );
	};

	conf_form_section.prototype.hide_waiting = function()
	{
		this.frm.hide_waiting();
	};

	conf_form_section.prototype.show_or_hide_loading_msg = function(message, is_loading)
	{
		if (is_loading || this.is_loading()) {
			this.show_waiting(message || "<%=_T('Loading.')%>");
		}
		else {
			this.hide_waiting();
		}
	};

	conf_form_section.prototype.validate = function( save_callback, validation_def )
	{
		return this.frm.validate( save_callback, validation_def );
	};

	conf_form_section.prototype.valid = function( value, type, options )
	{
		return this.frm.valid( value, type, options );
	};

	conf_form_section.prototype.disable_control = function( $control, disabled, is_required, is_disable_lbl, keep_value )
	{
		if ( this.is_readonly() ) {
			this.readonly_control( $control, true, false );
		}
		else {
			this.frm.disable_control( $control, disabled, is_required, is_disable_lbl, keep_value );
		}
	};

	conf_form_section.prototype.disabled_all_sub_menus = function(disabled)
	{
		this.frm.disabled_all_sub_menus(disabled);
	};

	conf_form_section.prototype.mark_sub_menu_option = function(section_id)
	{
		this.frm.mark_sub_menu_option(section_id);
	};

	conf_form_section.prototype.readonly_control = function( $control, readonly, is_required )
	{
		this.frm.readonly_control( $control, readonly, is_required );
	};

	conf_form_section.prototype.fill_combo = function($combo, is_blank_first_item, list, field_id, field_name, sorted, can_add_item, data_id, id_is_case_sensitive, selected_value, tooltip)
	{
		id_is_case_sensitive = id_is_case_sensitive || false;
		this.frm.fill_combo($combo, is_blank_first_item, list, field_id, field_name, sorted, can_add_item, data_id, id_is_case_sensitive, selected_value, tooltip);
	};

	conf_form_section.prototype.has_permission_to_section = function()
	{
		var category = "VIEW";
		var all_overrides = true;

		return has_permission_to_action( this, this.view_action_id, undefined, category, all_overrides );
	};

	conf_form_section.prototype.has_permission_to_save = function()
	{
		let save_action_id = this.save_action_id;
		if ( typeof save_action_id === "object" ) {
			save_action_id = this.frm.is_creating() ? save_action_id.create : save_action_id.update;
		}
		if ( typeof save_action_id !== "string" ) {
			return false;
		}
		return has_permission_to_action( this, save_action_id );
	};

	conf_form_section.prototype.has_permission_to_delete = function()
	{
		return has_permission_to_action( this, this.erase_action_id );
	};

	conf_form_section.prototype.build_header_custom_buttons = function(html_buttons, callback) {
		this.frm.build_header_custom_buttons(html_buttons, callback);
	};

	conf_form_section.prototype.destroy_header_custom_buttons = function() {
		this.frm.destroy_header_custom_buttons();
	};

	conf_form_section.prototype.format_section_title = function(section_title)
	{
		if (typeof section_title !== "string") {
			const section = this.frm.get_section_def(this.section_id);
			if(typeof section === "object") {
				section_title = section.name;
			}
		}
		return section_title || "";
	};

	conf_form_section.prototype.before_show = function(callback = $.noop)
	{
		callback();
	};

	conf_form_section.prototype.on_show = function(param)
	{
		var section_title;
		var must_show_required_message = true;

		if (this.is_loading()) {
			this.frm.dashboard.show_waiting(this.loading_message);
		}

		if (typeof param === "object")
		{
			if (param.hasOwnProperty("show")) {
				must_show_required_message = param.show;
			}

			if (param.hasOwnProperty("section_title")) {
				section_title = param.section_title;
			}
		}

		this.build_header_custom_buttons();
		this.$get_dom_element_by_id("form_buttons_actions").show();
		this.show_or_hide_required_message(must_show_required_message);

		this.set_section_title(section_title);
		this.frm.set_tagline();
		this.$get_dom_element_by_id("form_back_btn_container").hide();
	};

	conf_form_section.prototype.set_section_title = function(section_title)
	{
		section_title = this.format_section_title(section_title);
		if (!section_title) {
			return;
		}
		this.$get_dom_element_by_id("form_section_title").html(section_title);
		this.frm.change_title_section(this.section_id);

	};

	conf_form_section.prototype.get_section_title = function()
	{
		return this.$get_dom_element_by_id("form_section_title").text();
	};

	conf_form_section.prototype.on_hide = $.noop;

	conf_form_section.prototype.show_or_hide_required_message = function(show)
	{
		// header required message
		var $required_message = this.$get_dom_element_by_id("form_container .requiredFormMessage");
		show = show === undefined ? true : show;
		if ( !show || this.is_readonly() ) {
			$required_message.hide();
		}
		else {
			$required_message.show();
		}
	};

	conf_form_section.prototype.show_status_message = function( message, type, timeout_ms, scrolling, show_custom_button, $custom_message_container, callbacks )
	{
		this.frm.show_status_message( message, type, timeout_ms, scrolling, show_custom_button, $custom_message_container, callbacks );
	};

	conf_form_section.prototype.hide_messages = function( timeout_ms /* no requerido */, callback /* no requerido */ )
	{
		this.frm.hide_messages( timeout_ms, callback );
	};

	conf_form_section.prototype.readonly = function()
	{
		if ( typeof this.section_id !== "string" ) {
			return;
		}
		var selector = " > [form_section='" + this.section_id + "']";
		this.frm.readonly_all_crtls( selector );
	};

	conf_form_section.prototype.editable = function()
	{
		if ( typeof this.section_id !== "string" ) {
			return;
		}
		var selector = " > [form_section='" + this.section_id + "']";
		this.frm.editable_all_crtls( selector );
	};

	conf_form_section.prototype.$get_section_container = function( section_id )
	{
		section_id = section_id || this.section_id;
		if ( typeof section_id !== "string" ) {
			return;
		}
		var $section = this.$get_dom_element_by_id("form_data_container > [form_section='" + section_id + "']");
		if ( $section.length === 0 ) {
			return;
		}
		return $section;
	};

	conf_form_section.prototype.add_css_class = function( class_name )
	{
		var $section = this.$get_section_container();
		if ( !$section ) {
			return;
		}
		$section.removeClass( class_name ).addClass( class_name );
	};

	conf_form_section.prototype.remove_css_class = function( class_name )
	{
		var $section = this.$get_section_container();
		if ( !$section ) {
			return;
		}
		$section.removeClass( class_name );
	};

	conf_form_section.prototype.disable_actions = function(disable, prefix_id)
	{
		prefix_id = prefix_id || this.prefix_id;
		if (!prefix_id) {
			return;
		}
		disable = disable === undefined ? true : disable;
		$(prefix_id + "_save").prop("disabled", disable);
	};

	conf_form_section.prototype.validate_field = function( $field, type, message, options, callback_err, timeout_ms, $custom_message_container )
	{
		return this.frm.validation.validate_field( $field, type, message, options, callback_err, timeout_ms, $custom_message_container );
	};

	conf_form_section.prototype.show_popup_message = function(info)
	{
		this.frm.show_popup_message(info);
	};

	conf_form_section.prototype.hide_popup_message = function()
	{
		this.frm.hide_popup_message();
	};

	conf_form_section.prototype.confirm = function(message, callback = $.noop)
	{
		this.frm.confirm(message, callback);
	};

	conf_form_section.prototype.fill_timezones = function(timezones, $combo, is_blank_first_item)
	{
		this.frm.fill_timezones(timezones, $combo, is_blank_first_item);
	};

	conf_form_section.prototype.fill_combo_app_languages = function($app_language, is_blank_first_item = true)
	{
		this.frm.fill_combo_app_languages($app_language, is_blank_first_item);
	};

	conf_form_section.prototype.fill_countries = function(countries, $combo)
	{
		this.frm.fill_countries(countries, $combo);
	};

	conf_form_section.prototype.fill_combo_areas = function( country_code, $area, is_blank_first_item ) {
		this.frm.fill_combo_areas(country_code, $area, is_blank_first_item);
	};

	conf_form_section.prototype.__get_selected_area = function($ctrl) {
		return this.frm.__get_selected_area($ctrl);
	};
}());
