/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 11 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

//	"use strict";

	const ENABLED_TAB_CLASS = "formTabsMenuItem";
	const DISABED_TAB_CLASS = "formTabsMenuItem-ds";

	//---------------------------------------------------------------------
	// form tabs class
	function conf_form_tab( params )
	{
		params = params || {};
		params.callbacks = params.callbacks || {};
		this.frm             = params.frm;
		this.tabs_list       = params.tabs_list;
		this.container_id    = "#" + params.container_id;
		this.container_class = params.container_class || "formTabsContainer";
		this.list_class      = params.list_class || "formTabsMenu";
		this.all_tabs_hidden = params.all_tabs_hidden || false;
		this.selected_tab_id = null;
		this.count_tabs      = 0;
		let must_initialize   = typeof(params.must_initialize) === "boolean" ? params.must_initialize: true;
		this.callbacks = {
			on_show_tab: params.callbacks.on_show_tab || $.noop,
			on_hide_all_tabs: params.callbacks.on_hide_all_tabs || $.noop,
			on_add_tab: params.callbacks.on_add_tab || $.noop,
			on_remove_tab: params.callbacks.on_remove_tab || $.noop,
		};
		if (must_initialize) {
			this.initialize();
		}

		return true;
	}

	if ( typeof inConcert.conf_form_tab !== "function" ) {
		inConcert.conf_form_tab = conf_form_tab;
	}

	conf_form_tab.prototype.switch_tab = function(select_tab_id, on_changed, force_switch = false)
	{
		on_changed = on_changed || $.noop;

		const self = this;

		if (!this.is_enabled_tab(select_tab_id)) {
			return;
		}
		let selected_tab_id = this.get_selected_tab_id();
		if (!force_switch && selected_tab_id === select_tab_id) {
			return;
		}

		var show = function() {
			self.show_tab(select_tab_id);
			let $tab = self.__$tab(select_tab_id);
			$tab.removeClass("tabsMenuItemSelect").addClass("tabsMenuItemSelect");
			let tab_container_id = self.tabs_list[ select_tab_id ].tab_container_id;
			$( "#" + tab_container_id ).show();
			self.selected_tab_id = select_tab_id;
			let config = self.tabs_list[self.selected_tab_id];
			window.setTimeout(function() {
				(config.on_show || $.noop)(config);
				self.callbacks.on_show_tab(select_tab_id);
				on_changed();
			});
		};

		// hide all tabs
		this.hide_all_tabs_panels();
		// show tab
		show();

		return true;
	};

	conf_form_tab.prototype.show_tab_with_content = function(select_tab_id, on_changed, force_switch = false)
	{
		this.switch_tab(select_tab_id, on_changed, force_switch);
	};

	conf_form_tab.prototype.hide_tab_with_content = function(select_tab_id)
	{
		this.hide_tab(select_tab_id);
		var tab_container_id = this.tabs_list[ select_tab_id ].tab_container_id;
		$( "#" + tab_container_id ).hide();
		(this.tabs_list[select_tab_id].on_hide || $.noop)();
		var total_visible_tabs = this.total_visible_tabs();
		if (total_visible_tabs === 0) {
			this.selected_tab_id = null;
			this.callbacks.on_hide_all_tabs();
		}
		else {
			if (this.selected_tab_id === select_tab_id) {
				this.select_first_visible_tab();
			}
		}
	};

	conf_form_tab.prototype.total_visible_tabs = function()
	{
		var self = this;
		var total = 0;
		$.each(this.tabs_list || {}, function(tab_id) {
			if (self.is_visible_tab(tab_id)) {
				total++;
			}
		});
		return total;
	};

	conf_form_tab.prototype.get_selected_tab_id = function()
	{
		return this.selected_tab_id ;
	};

	conf_form_tab.prototype.$get_tabs = function()
	{
		return $( this.container_id );
	};

	conf_form_tab.prototype.__$get_tabs_list = function()
	{
		var $tabs = this.$get_tabs();
		return $tabs.find("[tab_list]");
	};

	conf_form_tab.prototype.is_visible_tabs_container = function()
	{
		if ( this.count_tabs <= 0 ) {
			return false;
		}
		var $tabs = this.$get_tabs();
		return $tabs.is(":visible");
	};

	conf_form_tab.prototype.show_tabs_container = function()
	{
		if ( this.count_tabs <= 0 ) {
			return;
		}
		var $tabs = this.$get_tabs();
		$tabs.show();
	};

	conf_form_tab.prototype.hide_tabs_container = function()
	{
		var $tabs = this.$get_tabs();
		$tabs.hide();
		this.hide_all_tabs_panels();
	};

	conf_form_tab.prototype.hide_all_tabs_panels = function()
	{
		var self = this;
		// hide all tabs
		$.each(this.tabs_list || {}, function(tab_id, tab_config)
		{
			self.__$tab(tab_id).removeClass("tabsMenuItemSelect");
			var $tab_container = $("#" + tab_config.tab_container_id);
			$tab_container.hide();
			if (self.selected_tab_id === tab_id) {
				(tab_config.on_hide || $.noop)();
			}
		});
		this.selected_tab_id = null;
		this.callbacks.on_hide_all_tabs();
	};

	conf_form_tab.prototype.on_show = $.noop;
	conf_form_tab.prototype.on_hide = $.noop;

	conf_form_tab.prototype.editable = function( tab_id )
	{
		if ( typeof tab_id !== "string" ) {
			return;
		}
		var tab_container_id = this.tabs_list[ tab_id ].tab_container_id;
		var selector = " #" + tab_container_id;
		this.frm.editable_all_crtls( selector );
	};

	conf_form_tab.prototype.readonly = function( tab_id )
	{
		if ( typeof tab_id !== "string" ) {
			return;
		}
		var tab_container_id = this.tabs_list[ tab_id ].tab_container_id;
		var selector = " #" + tab_container_id;
		this.frm.disable_all_crtls( selector );
	};

	conf_form_tab.prototype.set_caption_tab = function(tab_id, caption = "...")
	{
		let $tab = this.__$tab(tab_id);
		if ($tab.length > 0) {
			$tab.html(caption);
		}
	};

	conf_form_tab.prototype.get_tab_config = function(tab_id)
	{
		if ( typeof tab_id !== "string" ) {
			return;
		}
		var tab_config = this.tabs_list[ tab_id ];
		return tab_config;
	};

	conf_form_tab.prototype.add_tab = function(tab_config, selected)
	{
		var exist_tab = this.is_tab(tab_config.id);
		if (exist_tab) {
			return false;
		}
		let $list = this.__$get_tabs_list();
		let tab_classes = tab_config.classes || ENABLED_TAB_CLASS;
		let $tab = $("<li tab_id='" + tab_config.id + "' class='" + tab_classes + "'></li>");
		// let $tab = $(Mustache.render(inConcert.templates.process_flows.tab_element, { id : tab_config.id }));		
		
		$list.append($tab);

		let caption = typeof tab_config.name === "function" ? tab_config.name(tab_config) : (tab_config.name || tab_config.id);
		this.set_caption_tab(tab_config.id, caption);

		this.count_tabs++;
		if (!this.tabs_list.hasOwnProperty(tab_config.id)) {
			this.tabs_list[tab_config.id] = tab_config;
		}

		if (!this.all_tabs_hidden && selected) {
			this.switch_tab($tab.attr("tab_id"));
		}

		this.callbacks.on_add_tab(tab_config.id);

		return $tab;
	};

	conf_form_tab.prototype.remove_tab = function(tab_id)
	{
		if (!tab_id) {
			return;
		}
		let $tab = this.__$tab(tab_id);
		$tab.remove();
		if (this.tabs_list.hasOwnProperty(tab_id)) {
			delete this.tabs_list[tab_id];
		}
		let selected_tab_id = this.get_selected_tab_id();
		if (selected_tab_id === tab_id) {
			this.select_first_visible_tab();
		}

		this.callbacks.on_remove_tab(tab_id);
	};

	conf_form_tab.prototype.create_tabs = function()
	{
		var self = this;

		this.hide_tabs_container();

		var $tabs = this.$get_tabs();
		$tabs.removeClass().addClass(this.container_class);

		// var $list = $("<ul tab_list='true' class='" + this.list_class + "'></ul>");
		var $list = $("<ul tab_list='true' class='nav nav-tabs' id='tabs-configuration' role='tablist'></ul>");
		$tabs.html("").append($list);

		var $selected_tab;
		$.each(this.tabs_list || {}, function(_, tab_config)
		{
			var $tab = self.add_tab(tab_config);
			if ($tab && (!$selected_tab || tab_config.selected)) {
				$selected_tab = $tab;
			}
		});

		if (!this.all_tabs_hidden && $selected_tab) {
			this.switch_tab($selected_tab.attr("tab_id"));
		}

		$.each(this.tabs_list || {}, function(_, tab_config)
		{
			if (tab_config.initial_state === "disabled") {
				self.disable_tab(tab_config.id);
			}
			else {
				self.enable_tab(tab_config.id);
			}
		});
	};

	conf_form_tab.prototype.__$tab = function( tab_id )
	{
		var $tabs = this.$get_tabs();
		var $tab = $tabs.find("li[tab_id='" + tab_id + "']");
		return $tab;
	};

	conf_form_tab.prototype.is_tab = function( tab_id )
	{
		if (!tab_id) {
			return false;
		}
		let $tab = this.__$tab(tab_id);
		return $tab.length > 0;
	};

	conf_form_tab.prototype.is_enabled_tab = function( tab_id )
	{
		var $tab = this.__$tab(tab_id);
		if ( $tab.length === 0 ) {
			return false;
		}
		return !$tab.hasClass(DISABED_TAB_CLASS);
	};

	conf_form_tab.prototype.enable_tab = function( tab_id )
	{
		var $tab = this.__$tab(tab_id);
		if ( $tab.length === 0 ) {
			return;
		}
		if ( !$tab.hasClass(DISABED_TAB_CLASS) ) {
			return;
		}
		$tab.removeClass(DISABED_TAB_CLASS);
	};

	conf_form_tab.prototype.disable_tab = function( tab_id )
	{
		var $tab = this.__$tab(tab_id);
		if ( $tab.length === 0 ) {
			return;
		}
		if ( $tab.hasClass(DISABED_TAB_CLASS) ) {
			return;
		}
		$tab.addClass(DISABED_TAB_CLASS);
	};

	conf_form_tab.prototype.enable_disable_all_tabs = function( enabled )
	{
		const self = this;
		$.each( this.tabs_list || {}, function(tab_id) {
			self[enabled ? "enable_tab" : "disable_tab"](tab_id);
		});
	};

	conf_form_tab.prototype.show_tab = function( tab_id )
	{
		var $tab = this.__$tab(tab_id);
		if ( $tab.length === 0 ) {
			return;
		}
		$tab.show();
	};

	conf_form_tab.prototype.hide_tab = function( tab_id )
	{
		var $tab = this.__$tab(tab_id);
		if ( $tab.length === 0 ) {
			return;
		}
		$tab.hide();
	};

	conf_form_tab.prototype.is_visible_tab = function(tab_id)
	{
		var $tab = this.__$tab(tab_id);
		if ( $tab.length === 0 ) {
			return false;
		}
		return $tab.is(":visible");
	};

	conf_form_tab.prototype.select_first_visible_tab = function()
	{
		var self = this;
		$.each(this.tabs_list || {}, function(tab_id) {
			if (self.is_visible_tab(tab_id)) {
				self.show_tab_with_content(tab_id);
				return false; // break
			}
		});
	};

	conf_form_tab.prototype.set_events = function()
	{
		var self = this;
		var $tabs = this.$get_tabs();
		$tabs
			.off( "click", "li" )
			.on( "click", "li", function() {
				self.switch_tab( $( this ).attr( "tab_id" ) );
			});
	};

	conf_form_tab.prototype.initialize = function()
	{
		if ( !this.tabs_list ) {
			return;
		}
		this.set_events();
		this.create_tabs();
	};
}());
