/*  jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
  strict:true, undef:true, unused:true, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 9 */

/* global $, inConcert, dhtmlXTreeObject */

if (!this.inConcert) {
	this.inConcert = {};
}

(function ()
{
	/*jshint validthis:true*/
	"use strict";

	const FORMAT = inConcert.Utils.format;
	const ROOT_NODE_ID = "0";
	const SON_BRANCH = 0;
	const SIBLING_BRANCH = 1;

	//---------------------------------------------------------------------
	//
	function conf_form_tree(params)
	{
		const self = this;

		if (typeof(params) !== "object") {
			throw("conf_form_tree. params is undefined.");
		}
		if (typeof(params.frm) !== "object") {
			throw("conf_form_tree. params.frm is undefined.");
		}
		if (typeof(params.owner) !== "object") {
			throw("conf_form_tree. params.owner is undefined.");
		}

		this.root_node_id = null;
		this.tree = null;
		this.frm   = params.frm;
		this.owner = params.owner;
		this.is_enable_drag_and_drop = params.is_enable_drag_and_drop || false;

		// El atributo 'nodes_attrs' es necesario.
		if (typeof params.nodes_attrs !== "object") {
			throw("conf_form_tree error. Parameter 'nodes_attrs' is required.");
		}
		if (typeof params.nodes_attrs.childrens !== "string") {
			throw("conf_form_tree error. Parameter 'params.nodes_attrs.attr_childrens' is required.");
		}
		if (params.nodes_attrs) {
			this.attr_node_id   = params.nodes_attrs.id;
			this.attr_childrens = params.nodes_attrs.childrens;
		}

		// ids
		this.id_tree = params.ids.tree;
		// ids popup0
		// this.id_new_item                  = params.ids.new_item;
		// this.id_disclaimer                = params.ids.disclaimer;
		this.id_popup                     = params.ids.popup;
		this.id_popup_msg                 = params.ids.popup_msg;
		this.id_popup_create              = params.ids.popup_create;
		this.id_popup_create_and_continue = params.ids.popup_create_and_continue;
		this.id_popup_update              = params.ids.popup_update;
		this.id_popup_title               = params.ids.popup_title;
		this.id_popup_close               = params.ids.popup_close;
		this.id_popup_input               = params.ids.popup_input_id;
		this.id_popup_waiting             = params.ids.popup_waiting;

		// strings
		this.strings = {
			waiting_add_nodes: params.strings.waiting_add_nodes,
			waiting_upd_nodes: params.strings.waiting_upd_nodes,
			waiting_del_node : params.strings.waiting_del_node
		};

		let callbacks = params.callbacks || {};
		this.callbacks = {
			create: callbacks.create || function(_, callback) {
				callback(true);
			},
			update: callbacks.update || function(_, callback) {
				callback(true);
			},
			remove: callbacks.remove || function(_, callback) {
				callback(true);
			},
			on_removed_node: callbacks.on_removed_node || $.noop,
			show_waiting: callbacks.show_waiting || function(msg) {
				self.frm.show_waiting(msg);
			},
			hide_waiting: callbacks.hide_waiting || function() {
				self.frm.hide_waiting();
			},
		};

		if (params.auto_initialize) {
			this.initialize();
			this.set_events();
		}

		this.plain_tree = {}; // agrega todos los nodos del tree en modo plano ordenado por su id

		return true;
	}

	if (typeof inConcert.conf_form_tree !== "function") {
		inConcert.conf_form_tree = conf_form_tree;
	}

	conf_form_tree.prototype.initialize = function(force)
	{
		this.closed = false;
		if (force) {
			this.tree = null;
			$("#" + this.id_tree).html("");
		}
		else {
			if (this.tree) {
				return;
			}
		}
		this.tree_state = "unloaded";
		this.saving = false; // flag utilizada para no enviar mas de un request de save, si se encuentra uno en curso.
		// initialize tree
		var image_paths = function() {
			let skin_names = {
				skinDark  : "dark",
				skinLight : "light",
				skinNew   : "new"
			};
			let current_skin = inConcert.app.getMainWindow().getCurrentSkin() || "";
			let skin_name = skin_names[current_skin] || skin_names.skinLight;
			return "/inconcert/apps/agent/images/" + skin_name + "/tree/28/";
		}();
		this.tree = new dhtmlXTreeObject(this.id_tree, "100%", "100%", 0);
		this.tree.setImagePath(image_paths);
		this.tree.setStdImages("csh_books/leaf.png","csh_books/folderOpen.png","csh_books/folderClosed.png");
		this.tree.enableDragAndDrop(this.is_enable_drag_and_drop);
		this.tree.enableHighlighting(true);
		this.tree.setOnDblClickHandler(function(/*node_id*/) {
// 			let on_save = self.update_node;
// self.show_node_popup( null, node_id, on_save );
		});
		this.tree.setDragHandler(function(/*sourceId, targetId*/) {
			return true;
		});
		this.tree._getUserData = function(node_id) {
			let json_node_data = this.getUserData(node_id, "node");
			return JSON.parse(json_node_data);
		};
		//initialize events
		this.set_events();

		this.clean_popup();
	};

	conf_form_tree.prototype.dispose = function()
	{
		if (this.tree) {
			this.tree.deleteChildItems(ROOT_NODE_ID);
			this.tree.destructor();
			this.tree = null;
		}
		this.plain_tree = {};
		this.closed = true;
		this.root_node_id = null;
	};

	conf_form_tree.prototype.get_root_node_id = function()
	{
		return this.root_node_id || ROOT_NODE_ID;
	};

	conf_form_tree.prototype.set_root_node_id = function(root_node_id)
	{
		this.root_node_id = root_node_id || ROOT_NODE_ID;
	};

	conf_form_tree.prototype.clean_popup = function()
	{

	};

	/*
	 * add child node to tree
	 */
	conf_form_tree.prototype.add_child_node_to_tree = function(node, node_id, parent_id)
	{
		var name = this.get_markup_node_name(node.name, node_id, parent_id);
		// The SELECT parameter remains selected the created element.
		this.tree.insertNewChild(parent_id, node_id, name, 0, 0, 0, 0, "SELECT");
		this.tree.setUserData(node_id, "node", JSON.stringify(node));

		this.plain_tree[node_id] = node;
	};

	/*
	 * add sibling node to tree
	 */
	conf_form_tree.prototype.add_sibling_node_to_tree = function(node, node_id, parent_id)
	{
		var name = this.get_markup_node_name(node.name, node_id, parent_id);
		// The SELECT parameter remains selected the created element.
		this.tree.insertNewNext( parent_id, node_id, name, 0, 0, 0, 0, "SELECT");
		this.tree.setUserData(node_id, "node", JSON.stringify(node));

		this.plain_tree[node_id] = node;
	};

	conf_form_tree.prototype.get_markup_node_name = function(name, node_id, parent_id)
	{
		let html_node = "".concat(
			"<div node_name='", name, "'>", name, "</div>",
			"<div class='resultButtonContainer' node_id='", node_id, "' parent_id='", parent_id, "'>",
				"{0}", // los buttons
			"</div>");

		let html_buttons = "";

		html_buttons = "".concat(
			"<button delete='true' class='buttonColorGray gridActionButton butResultDelete delete'><%=_T('Delete')%></button>",
			"<button edit='true' class='buttonColorGray gridActionButton butResultEdit edit'><%=_T('Edit')%></button>",
			"<button new='true' class='buttonColorBlack gridActionButton butResultAdd new'><%=_T('New')%></button>");


		html_node = FORMAT( html_node, html_buttons );

		return html_node;
	};

	conf_form_tree.prototype.remove_all_nodes = function()
	{
		this.tree.deleteChildItems(ROOT_NODE_ID);
		this.plain_tree = {};
		this.root_node_id = null;
	};

	/*
	 	Fill the tree with all nodes
		Sample of the expected nodes on the json structure:
			[
				{
					name: "No Compro",
				},
				{
					name: "No compro porque ha fallecido!!",
				},
				{
					name: "Compro",
					childs: [
						{
							name: "Efectivo",
						},
						{
							name: "Tarjeta",
							childs: [
								{name: "Visa"  },
								{name: "Master"}
							]
						}
					]
				}
			]
	 */
	conf_form_tree.prototype.set = function(tree_info)
	{
		const self = this;

		let tree_nodes = tree_info.children;
		if (!tree_nodes) {
			return;
		}
		if (this.tree_state !== "unloaded" && this.tree_state !== "loaded") {
			return;
		}
		this.tree_state = "loading";
		tree_nodes = $.isArray(tree_nodes) ? tree_nodes : JSON.parse(tree_nodes);
		this.remove_all_nodes();
		this.set_root_node_id(tree_info.root_node_id);

		var can_continue = function()
		{
			if ( !self.closed ) {
				return true;
			}
			// si se ha cerrado la ventana del frm de campana no continuo con la carga
			//console.log("abort !!! abm fill tree: Me voy -> self.closed: " + self.closed);
			self.hide_waiting();
			self.tree_state = "canceled";
			return false;
		};

		// recursive function that adds nodes on the tree
		let timeout_counter = 0;
		let pending_childs = 0;
		let total_created = 0;

		var fill = function(tree, parent_id)
		{
			// Lo agrego dentro de la funcion foreach_timeout para que el navegador no quede colgado.
			// Como los timeout se tiran por cada invocacion a funcion recursiva del tree, tengo
			//   un contador "timeout_counter", que se va incrementando en cada invocacion a la funcion
			//   y va decrementando cuando el loop timeout culmina para cada invocacion.
			// Entonces con esto, cuando el contador quede en cero nuevamente, ahi ya termino la carga total del tree.
			if ( !can_continue() ) { return; }
			timeout_counter++;
			inConcert.Utils.foreach_timeout(
			{
				rows : tree,
				max_append_per_loop: 200,
				pause_between_loop_ms: 1,
				append_fn : function(node, created)
				{
					if ( !can_continue() ) { return { cancel: true }; }
					total_created++;
					let node_id;
					if (self.attr_node_id) {
						node_id = node[self.attr_node_id];
					}
					else {
						node_id = inConcert.Utils.generateGuid();
					}
					let first_child = created;
					if (first_child === SON_BRANCH) {
						self.add_child_node_to_tree(node, node_id, parent_id);
					}
					else {
						self.add_sibling_node_to_tree(node, node_id, parent_id);
					}
					//console.log("1 append_fn abm fill tree: " + node_id);
					parent_id = node_id;
					if (node[self.attr_childrens]) {
						// Al loop que carga los childs los agrego en un timeout, e incremento la variable de pending en 1.
						// recien cargo los childs cuando termino la carga de la rama actual (timeout_counter <= 0), o sea el padre.
						// La carga de cada rama child se realiza de forma independiente, es decir, hasta que no termine una, no comienza con la otra.
						var pending_fn = function() {
							window.setTimeout(function() {
								if (timeout_counter <= 0) {
									//console.log("2.1 append_fn abm fill tree voy por los hijos: " + node_id + ", pending_childs: " + pending_childs);
									pending_childs--;
									fill(node[self.attr_childrens], node_id);
								}
								else {
									pending_fn();
								}
							});
						};
						pending_childs++;
						//console.log("2 append_fn abm fill tree voy por los hijos: " + node_id + ", pending_childs: " + pending_childs + ", timeout_counter: " + timeout_counter);
						pending_fn();
					}
				},
				waiting_fn : function(/*message*/)
				{
					let waiting_message = self.strings.waiting_add_nodes || "<%=_T('Please wait, adding {0} nodes.')%>";
					waiting_message = FORMAT(waiting_message, total_created);
					self.show_waiting( waiting_message );
				},
				complete_fn : function()
				{
					//console.log("3 complete_fn - timeout_counter " + timeout_counter);
					if ( !can_continue() ) { return; }
					timeout_counter--;
					//console.log("3.1 complete_fn - pending_childs: " + pending_childs + ", timeout_counter: " + timeout_counter);
					if ( timeout_counter <= 0 && pending_childs <= 0 ) {
						//console.log("4 complete_fn - abm ENDDDDDDDDDDD ");
						// aca termino la carga total del tree
						self.tree.closeAllItems(ROOT_NODE_ID);
						self.hide_waiting();
						self.tree_state = "loaded";
					}
				}
			});
		};

		// start fill
		fill(tree_nodes, ROOT_NODE_ID);
	};

	/*
	 * retrieves the NODES of the tree, and builds the list to be stored.
	 */
	conf_form_tree.prototype.get = function()
	{
		const self = this;

		// recursive function tha adds nodes on the list
		var __MakeTree = function (current_node_id, nodes)
		{
			let nodes_ids = self.get_sub_items(current_node_id);
			if (!nodes_ids || (nodes_ids.length === 1 && nodes_ids[0].length === 0)) {
				return nodes;
			}
			for (let i = 0; i < nodes_ids.length; i++) {
				let node_id = nodes_ids[i];
				let node_info = self.tree._getUserData(node_id);
				let number_of_childs = self.tree.hasChildren(node_id);
				if (number_of_childs > 0) {
					node_info[self.attr_childrens] = __MakeTree(node_id, []);
				}
				else {
					node_info[self.attr_childrens] = null;
				}
				nodes.push(node_info);
			}
			return nodes;
		};

		let tree_info = {
			nodes: __MakeTree(ROOT_NODE_ID, []) // start making tree
		};
		//tree_info.levels = levels;

		return tree_info;
	};

	conf_form_tree.prototype.set_events = function()
	{
		const self = this;
		/////////
		const $tree = $("#" + this.id_tree);
		let event_selector = "[edit='true']";
		$tree
			.off("click", event_selector)
			.on("click", event_selector, function() {
				let current_node_id = self.tree.getSelectedItemId();
				let on_save = self.update;
				let node_info = {
					node_id: current_node_id,
				//	plugin_node_id: current_node_id,
					branch: null, // SON_BRANCH or SIBLING_BRANCH or NULL (edit)
				};
				self.show_popup(node_info, on_save);
			}
		);

		event_selector = "[new='true']";
		$tree
			.off("click", event_selector)
			.on("click", event_selector, function() {
				let current_node_id = $(this).parent().attr("node_id");
				if (!current_node_id) {
					throw("Node id does not exist on tree markup!!");
				}
				let on_save = self.create;
				let node_info = {
					node_id: current_node_id,
				//	plugin_node_id: current_node_id,
					branch: SON_BRANCH, // SON_BRANCH or SIBLING_BRANCH or NULL (edit)
				};
				self.show_popup(node_info, on_save, true /* is_new */);
			}
		);

		event_selector = "[delete='true']";
		$tree
			.off("click", event_selector)
			.on("click", event_selector, function() {
				let node_id = $(this).parent().attr("node_id");
				if (!node_id) {
					throw("Node does not exist on tree markup!!");
				}
				self.delete_node(node_id);
			}
		);
	};

	conf_form_tree.prototype.show_root_popup = function()
	{
		// si el nodo raiz tiene hijos (resultados o categorias), agrega en la rama hijos, sino en la rama hermanos
		const has_children = this.tree.hasChildren(ROOT_NODE_ID) === 0;
		let current_branch = has_children ? SON_BRANCH : SIBLING_BRANCH;
		let on_save = this.create;
		let node_info = {
			node_id: this.get_root_node_id(),
			//plugin_node_id: ROOT_NODE_ID,
			is_root_node: true,
			branch: current_branch, // SON_BRANCH or SIBLING_BRANCH or NULL (edit)
		};
		this.show_popup(node_info, on_save, true /* is_new */ );
	};

	/*
	 * displays the node on popup controls
	 */
	conf_form_tree.prototype.show_popup = function(node_info, on_save/*, is_new*/)
	{
		const self = this;
		let timeout_ms = false; //infinite

		let current_branch = node_info.branch;
		let current_node_id = node_info.node_id;

		const $popup = $("#" + this.id_popup);
		const $popup_title = $("#" + this.id_popup_title);
		const $popup_create = $("#" + this.id_popup_create);
		const $popup_create_and_continue = $("#" + this.id_popup_create_and_continue);
		const $popup_update = $("#" + this.id_popup_update);
		const $name = $("#" + this.id_popup_input);

		var on_ready = function(success, msg_err)
		{
			if (!success) {
				self.show_status_message(msg_err, "Error", timeout_ms);
				return;
			}

			//  - Si el parametro << current_branch >> no es NULL, se esta creando un resultado.
			// - Si el mismo es NULL, se esta actualizando un resultado o categoria.
			//
			// Y segun esto se deben visualizar/ocultar diferentes controles como tambien actualizarlos.
			//
			let name = "";
			const is_new_result = current_branch !== null;
			if (is_new_result) {
				$popup_title.text("<%=_T('New')%>");
				$popup_create.show();
				$popup_create_and_continue.show();
				$popup_update.hide();
			}
			else {
				// se actualizan tanto las categorias como los resultados.
				let popup_title = "<%=_T('Update')%>";
				$popup_title.text(popup_title);
				$popup_create.hide();
				$popup_create_and_continue.hide();
				const $update_button = $popup_update;
				if (self.frm.is_readonly()) {
					$update_button.hide();
				}
				else {
					$update_button.show();
				}
				const node = self.tree._getUserData(current_node_id);
				if (node) {
					name = node.name;
				}
			}

//			self.show_bread_crumbs(current_node_id);
//			self.hide_popup_errors();

			$popup.show();
			$name.val(name).focus();
		};

		// eventos del popup
		this.set_events_popup(function(callback = $.noop) {
			const node = self.get_node_from_controls(current_branch, current_node_id);
			if (!self.validate_node(node, node_info)) {
				callback(false);
				return false;
			}
			return on_save(self, node, node_info, callback);
		});

		// realizo los request y carga de los combos y despues procedo a visualizar el popup.
		this.load_popup_data(on_ready);
	};

	/*
	 * Realiza la carga de los combos y lista mediante request al server.
	 */
	conf_form_tree.prototype.load_popup_data = function(callback = $.noop)
	{
		callback(true);
	};

	conf_form_tree.prototype.set_events_popup = function(on_save)
	{
		const self = this;

		const $name = $("#" + this.id_popup_input);

		$("#" + this.id_popup_create)
			.off("click")
			.on("click", function() {
				on_save(function(is_created) {
					if (is_created) {
						self.close_popup();
					}
				});
			}
		);
		$("#" + this.id_popup_create_and_continue)
			.off("click")
			.on("click", function() {
				on_save(function(is_created) {
					if (is_created) {
						$name.val("").focus();
					}
				});
			}
		);
		$("#" + this.id_popup_update)
			.off("click")
			.on("click", function() {
				on_save(function(is_updated) {
					if (is_updated) {
						self.close_popup();
					}
				});
			}
		);
		$("#" + this.id_popup_close)
			.off("click")
			.on("click", function() {
				self.close_popup();
			});
	};

	conf_form_tree.prototype.get_sub_items = function(node_id)
	{
		if (!node_id) {
			return false;
		}
		let sub_items = this.tree.getSubItems(node_id);
		if (!sub_items) {
			return false;
		}
		let nodes_ids = sub_items.split(this.tree.dlmtr);
		if (!nodes_ids) {
			return false;
		}
		return nodes_ids;
	};

	/*
	 * get the last sub node id
	 */
	conf_form_tree.prototype.get_last_sub_node_id = function(node_id)
	{
		let nodes_ids = this.get_sub_items(node_id);
		if (!nodes_ids) {
			return false;
		}
		return nodes_ids[nodes_ids.length - 1];
	};

	/*
	 *	create and add node to the tree
	 */
	conf_form_tree.prototype.create_node = function(self, node, raw_node, callback = $.noop)
	{
		let node_id = node.id || inConcert.Utils.generateGuid();
		let current_node_id = raw_node.node_id;
		let parent_node_id = raw_node.is_root_node ? ROOT_NODE_ID : current_node_id;

		if (raw_node.branch === SIBLING_BRANCH) {
			// crea un hermano
			parent_node_id = self.get_last_sub_node_id(parent_node_id);
			if (parent_node_id === false) {
				callback(false);
				return false;
			}
			self.add_sibling_node_to_tree(node, node_id, parent_node_id);
		}
		else {
			// crea un hijo
			self.add_child_node_to_tree(node, node_id, parent_node_id);
		}

		callback(true);
		return true;
	};

	/*
	 *	update node on the tree
	 */
	conf_form_tree.prototype.update_node = function(self, node, node_info, callback = $.noop)
	{
		let node_id = node_info.node_id;
		let parent_id = self.tree.getParentId(node_id);
		let name = self.get_markup_node_name(node.name, node_id, parent_id);
		self.tree.setItemText(node_id, name);
		self.tree.setUserData(node_id, "node", JSON.stringify(node));
		self.plain_tree[node_id] = node;
		callback(true);
		return true;
	};

	/*
	 * empty popup controls
	 */
	conf_form_tree.prototype.clean_popup_controls = function()
	{
		const $name = $("#" + this.id_popup_input);
		$name.val("");
	};

	conf_form_tree.prototype.close_popup = function()
	{
		//this.show_or_hide_required_message(false);
		this.hide_messages();
		$("#" + this.id_popup).hide();
	};

	/*
	 *	retrieve results from popup controls
	 */
	conf_form_tree.prototype.get_node_from_controls = function(current_branch, current_node_id)
	{
		/*
			{
				name:               "No Compro",  (required)
				id:              false,       (required)
			}
		 */
		const $name = $("#" + this.id_popup_input);
		let node = {
			name: $name.val().trim()
		};
		// si se esta actualizando y es categoria (tiene hijos), unicamente retorna la descripcion del node.
		if (current_branch === null && this.tree.hasChildren(current_node_id) > 0) {
			return node;
		}

		return node;
	};

	/*
	 * valid node on the popup controls
	 */
	conf_form_tree.prototype.validate_node = function(node, node_info)
	{
		const self = this;

		let current_branch = node_info.branch;
		let current_node_id = node_info.node_id;

		var show_field_error = function(control_id, message)
		{
			$("#" + control_id + "_validation").show();
			$("#" + control_id).focus();
			let message_type = "Error";
			self.show_status_message(message, message_type);
		};

		this.hide_messages();
		if (!node) {
			var message_type = "Error";
			this.show_status_message("<%=_T('Error building the node.')%>", message_type);
			return false;
		}

		const dom_id_name = this.id_popup_input;
		// validate node name
		if (node.name.trim().length === 0) {
			show_field_error(dom_id_name, "<%=_T('Please enter the Name.')%>");
			return false;
		}

		var already_exists = function() {
			let parent_id;
			if (node_info.is_root_node) {
				parent_id = ROOT_NODE_ID;
			}
			else {
				if (current_branch === null) { // editing
					parent_id = self.tree.getParentId(current_node_id) || "0";
				} else { // new
					parent_id = current_node_id;
				}
			}

			//var sibling_ids = self.tree.getSubItems(parent_id).split(self.tree.dlmtr);
			let sibling_ids = self.get_sub_items(parent_id);
			if (sibling_ids) {
				sibling_ids = sibling_ids.length === 1 && sibling_ids[0] === "" ? [] : sibling_ids;

				for (var sibling_id_index = 0; sibling_id_index < sibling_ids.length; sibling_id_index++) {
					var sibling_result_name = JSON.parse(self.tree.getUserData(sibling_ids[sibling_id_index], "node")).name;
	
					if (sibling_result_name && node.name) {
						if (sibling_result_name.length === node.name.length && sibling_result_name.search(new RegExp(node.name, "i")) === 0) {
							return true;
						}
					}
				}
			}
			return false;
		};

		var current_result;
		var is_new_result = current_branch !== null;

		if (is_new_result) {
		// si se va a agregar un nuevo resultado
			if (already_exists()) {
				show_field_error(dom_id_name, "<%=_T('Unable to create. Already exists name.')%>");
				return false;
			}
		}
		else {
		// si se esta editando un resultado y se le esta cambiando el nombre, valida que este nombre nuevo ya no exista
			current_result = this.tree._getUserData(current_node_id);

			if ((current_result && current_result.name) && node.name) {
				if ((current_result.name.length !== node.name.length) || (current_result.name.search(new RegExp(node.name, "i"))) !== 0) {
					if (already_exists()) {
						show_field_error(dom_id_name, "<%=_T('Unable to update. Already exists name.')%>");
						return false;
					}
				}
			}
		}

		//si tiene resultados (hijos), es categoria y unicamente se valida el nombre.
		if (current_branch === null && this.tree.hasChildren(current_node_id) > 0) {
			return true;
		}

		return true;
	};

	/*
	 * remove the tree node
	 */
	conf_form_tree.prototype.delete_node = function(node_id)
	{
		const self = this;
		this.hide_messages();
		let node = this.tree._getUserData(node_id);
		let childs = this.tree.getAllSubItems(node_id).split(this.tree.dlmtr);
		const is_final_node = (childs.length === 1 && childs[0].length === 0) ? true : false;

		var message = function() {
			let message = is_final_node ?
				"<%=_T('Do you want to delete node {0} ?')%>" :
				"<%=_T('Do you want to delete parent node {0} ?')%>";
			return FORMAT(message, node.name);
		}();

		jQuery.inconcertPopup.Show(
			"<%=_T('Confirm')%>",
			message,
			jQuery.inconcertPopup.ButtonSets.BS_YESNO,
			function(response) {
				if (response.toLowerCase() !== "yes") {
					return;
				}
				self.remove(node_id, function(success) {
					if (success) {
						self.tree.deleteItem(node_id);
						delete self.plain_tree[node_id];
					}
				});
			},
			undefined,
			{
				popup_id       : "conf_inconcertPopup",
				container_class: "confirmationPopUp",
				overlay_class  : "confirmationOverlay"
			}
		);
	};

	conf_form_tree.prototype.remove = function(node_id, callback = $.noop)
	{
		const self = this;

		let waiting_msg = self.strings.waiting_del_node || "<%=_T('Please wait, removing {0} node.')%>";
		this.show_waiting(waiting_msg);

		this.callbacks.remove(node_id, function(success, msg_err) {
			self.hide_waiting();
			if (!success) {
				self.show_status_message(msg_err, "Error");
				callback(false);
				return;
			}
			self.on_removed_node(node_id, callback);
		});
	};

	conf_form_tree.prototype.on_removed_node = function(node_id, callback = $.noop)
	{
		callback(true);
		// invoca al callback de la clase que me ha instanciado.
		let node = this.plain_tree[node_id];
		this.callbacks.on_removed_node(node_id, node);
	};

	conf_form_tree.prototype.show_status_message = function(message, message_type)
	{
		message_type = message_type || "Error";
		const $custom_message_container = $("#" + this.id_popup_msg);
		const timeout_ms = 5000;
		const scrolling = false;
		const show_custom_button = false;

		this.frm.show_status_message(message, message_type, timeout_ms, scrolling, show_custom_button, $custom_message_container);
	};

	conf_form_tree.prototype.hide_messages = function(callback)
	{
		const $custom_message_container = $("#" + this.id_popup_msg);
		this.frm.hide_messages(undefined, callback, $custom_message_container);
	};

	conf_form_tree.prototype.show_waiting = function(msg)
	{
		this.callbacks.show_waiting(msg);
	};

	conf_form_tree.prototype.hide_waiting = function()
	{
		this.callbacks.hide_waiting();
	};

	conf_form_tree.prototype.show_popup_waiting = function(msg)
	{
		const $waiting = $("#" + this.id_popup_waiting);
		$waiting.find("[waiting_message]").text(msg || "");
		$waiting.show();
	};

	conf_form_tree.prototype.hide_popup_waiting = function()
	{
		const $waiting = $("#" + this.id_popup_waiting);
		$waiting.hide();
	};

	conf_form_tree.prototype.create = function(self, node, raw_node, callback = $.noop)
	{
		let waiting_msg = self.strings.waiting_add_nodes || "<%=_T('Please wait, adding node.')%>";
		self.show_popup_waiting(waiting_msg);

		node.parent_node_id = raw_node.node_id;
		self.callbacks.create(node, function(success, result) {
			self.hide_popup_waiting();
			if (!success) {
				const msg_err = result;
				self.show_status_message(msg_err, "Error");
				callback(false);
				return;
			}
			let new_node_id = result;
			node.id = node.id || new_node_id;
			self.on_created_node(self, node, raw_node, callback);
		});
	};

	conf_form_tree.prototype.on_created_node = function(self, node, raw_node, callback)
	{
		self.create_node(self, node, raw_node, callback);
	};

	conf_form_tree.prototype.update = function(self, node, raw_node, callback = $.noop)
	{
		let waiting_msg = self.strings.waiting_upd_nodes || "<%=_T('Please wait, updating node.')%>";
		self.show_popup_waiting(waiting_msg);

		node.node_id = raw_node.node_id;
		self.callbacks.update(node, function(success, result) {
			self.hide_popup_waiting();
			if (!success) {
				const msg_err = result;
				self.show_status_message(msg_err, "Error");
				callback(false);
				return;
			}
			self.on_update_node(self, node, raw_node, callback);
		});
	};

	conf_form_tree.prototype.on_update_node = function(self, node, raw_node, callback)
	{
		self.update_node(self, node, raw_node, callback);
	};

}());
