/*jshint quotmark:false, esversion: 11*/
/*global $, inConcert*/

/*
Ha
*/
// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	"use strict";

	const format = inConcert.Utils.format;

	//---------------------------------------------------------------------
	// form validation class
	function conf_form_validation( params )
	{
		/* jshint validthis: true*/
		const self = this;
		this.current_requests = [];
		this.async_running = 0;
		this.errors = 0;
		this.frm = params.form;
		this.show_status_message = function( message, type, timeout_ms, scrolling, show_custom_button, $custom_message_container ) {
			self.frm.show_status_message( message, type, timeout_ms, scrolling, show_custom_button, $custom_message_container );
		};
		this.hide_errors = function() {
			self.frm.hide_messages();
		};
		return true;
	}

	if ( typeof inConcert.conf_form_validation !== "function" ) {
		inConcert.conf_form_validation = conf_form_validation;
	}

	conf_form_validation.prototype.mark_ctrl_error = function($field)
	{
		if (!$field || $field.length === 0) {
			return;
		}
		const validation_id = $field.attr("id") + "_validation";
		const $validation_field = $("#" + validation_id);
		if (!$validation_field || $validation_field.length === 0) {
			return;
		}
		$validation_field.show();
	};

	conf_form_validation.prototype.show_validation_error = function($field, message, timeout_ms, scrolling, show_custom_button, $custom_message_container, focus = true)
	{
		this.cleanup_request();
		this.mark_ctrl_error($field);
		if (message) {
			this.show_status_message(message, "Error", timeout_ms, scrolling, show_custom_button, $custom_message_container);
		}
		if (focus && $field) {
			$field.focus();
		}
	};

	conf_form_validation.prototype.valid = function( value, type, options )
	{
		return inConcert.Utils.validate( value, type, options );
	};

	conf_form_validation.prototype.validate_field = function( $field, type, message, options, callback_err, timeout_ms, $custom_message_container, must_validate_disabled )
	{
		callback_err = callback_err || $.noop;
		if (!must_validate_disabled && $field.is(":disabled") === true) {
			return true;
		}
		var value = $field.val();
		var result = this.valid( value, type, options );
		var success;
		if ( typeof result === "object" ) {
			success = result.success;
		}
		else {
			success = result;
		}
		if ( !success ) {
			timeout_ms = timeout_ms || false;
			var scrolling, show_custom_button;
			message = result.message || message;
			this.show_validation_error( $field, message, timeout_ms, scrolling, show_custom_button, $custom_message_container );
			callback_err();
		}
		return success;
	};

	conf_form_validation.prototype.remote_validation = function( $field, type, callback )
	{
		var self = this;
		callback = callback || $.noop;

		this.async_running++;
		var uri = type.uri( $field.val() );
		var request_timeout_ms = type.timeout_ms || undefined;
		var custom_success_callback = type.success;

		var success_callback = function(success) {
			if (self.errors === 0) {
				if (!success) {
					self.errors++;
				}
				callback($field, type);
			}
		};

		var error_callback = function() {
			self.cleanup_request();
			self.async_running = 0;
			self.errors = 0;
			self.hide_errors();
			var message = "<%=_T('Server error. Unable to validate information.')%>";
			var timeout_ms = false; //infinite
			self.show_status_message(message, "Error", timeout_ms);
		};

		this.current_requests.push(
			inConcert.invoke({
				url: uri,
				type: "GET",
				dataType: "json",
				contentType: "application/json",
				timeout: request_timeout_ms,
				success: function(response) {
					self.async_running--;
					if (typeof(custom_success_callback) === "function") {
						custom_success_callback(response, function(success) {
							success_callback(success);
						});
					}
					else {
						success_callback(response.available);
					}
				},
				error: function(/*xhr, textStatus, errorThrown*/) {
					error_callback();
				}
			})
		);
	};

	conf_form_validation.prototype.cleanup_request = function() 
	{
		$.each( this.current_requests || [], function( _, request ) {
			request.abort();
		});
		this.current_requests = [];
		this.async_running = 0;
	};

	conf_form_validation.prototype.start = function(definitions, callback, timeout_ms, $custom_message_container)
	{
		const self = this;

		callback = callback || $.noop;

		this.hide_errors();
		this.cleanup_request();
		this.errors = 0;

		if (!$.isArray(definitions) || definitions.length === 0) {
			callback( true );
			return true;
		}

		var show_validation_error = function($field, message) {
			let scrolling, show_custom_button;
			self.show_validation_error($field, message, timeout_ms, scrolling, show_custom_button, $custom_message_container);
		};
 
		var validate_field = function($field, type, message, options, callback_err, my_timeout_ms, $my_custom_message_container, must_validate_disabled) {
			my_timeout_ms = my_timeout_ms || timeout_ms;
			$my_custom_message_container = $my_custom_message_container || $custom_message_container;
			return self.validate_field($field, type, message, options, callback_err, my_timeout_ms, $my_custom_message_container, must_validate_disabled);
		};

		var finish = function($field, type) {
			if (self.async_running > 0) {
				return;
			}
			var success = self.errors === 0;
			if (!success) {
				self.cleanup_request();
				if ($field) { // solo para los remotos
					show_validation_error($field, type.message);
				}
			}
			callback(success);
		};

		$.each( definitions, function( _, def )  {
			if ( typeof(def.can_validate) === "function" && !def.can_validate() ) {
				return true; // continue in loop
			}
			var $field = $("#" + def.control_id);
			if ($field.length === 0){
				throw("conf_form_validation.prototype.start. Invalid DOM Id '" + def.control_id + "' or element does not exist.");
			}

			const validate_malicious_code = function ($field, def) {
				if ($field && !def.skip_validate_malicious_code && !validate_field($field, "html_malicious_code", "<%=_T('Possible malicious code in the field value.')%>")) {
					// field with error
					self.errors++;
					if ( typeof( def.on_error ) === "function" ) {
						def.on_error(type, message);
					}
					return false;
				}

				return true;
			};

			if ( def.optional ) {
				var value = $field.val();
				if( value.length === 0 ) {
					return true; // continue in loop
				}
			}

			var custom_async_validation = function(d, m) {
				// Funcion anonima utilizada para las custom async validation
				// la misma se utiliza para que no existan posibles errores al generar funciones dentro de loop
				return function() {
					m.valid($field, function(result) {
						if (!result.success) {
							// field with error
							self.errors++;
							show_validation_error($field, result.message);
							if ( typeof( d.on_error ) === "function" ) {
								d.on_error(null, result.message);
							}
						}
						else {
							self.async_running--;
							finish($field, result);
						}
					});
				};
			};

			for (var type in def.messages) { if (def.messages.hasOwnProperty(type)) {
				var message = def.messages[type];
				if (typeof(message) === "string") {
					if (!validate_field($field, type, message)) {
						// field with error
						self.errors++;
						if ( typeof( def.on_error ) === "function" ) {
							def.on_error(type, message);
						}
						return false; // break in loop
					}

					let malicious_code = validate_malicious_code($field, def);
					if (!malicious_code) {
						return false;
					}
					
				}
				else {
					if (typeof(message) === "object") {
						if (message.type === "remote") {
							self.remote_validation($field, message, finish);
						}
						else if (type === "custom") {
							let malicious_code = validate_malicious_code($field, def);
							if (!malicious_code) {
								return false;
							}

							if ( message.async ) {
								self.async_running++;
								window.setTimeout(custom_async_validation(def, message));
							}
							else {
								var response = message.valid($field);
								if (!response.success) {
									// field with error
									self.errors++;
									show_validation_error($field, response.message);
									if ( typeof( def.on_error ) === "function" ) {
										def.on_error(null, response.message);
									}
									return false;  // break in loop
								}
							}
						}
						else if (type === "range") {
							let between = typeof message.between === "function" ? message.between() : message.between;
							let options = {between: between, get_value: message.get_value};
							let message_error;
							let must_validate_disabled = message.must_validate_disabled || false;
							if (typeof(message.message) === "function") {
								message_error = message.message(options.between);
							}
							else {
								message_error = format(message.message, options.between.from, options.between.to);
							}

							if (!validate_field($field, type, message_error, options, undefined, undefined, undefined, must_validate_disabled)) {
								// field with error
								self.errors++;
								if (typeof(def.on_error) === "function") {
									def.on_error(type, message_error);
								}
								return false; // break in loop
							}

							let malicious_code = validate_malicious_code($field, def);
							if (!malicious_code) {
								return false;
							}
						}
						else {
							throw("validation type '" + message.type + "' not implemented!!");
						}
					}
				}
			}}
		});

		finish();
	};
}());
