/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, esversion: 9 */

/* global $, inConcert*/

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

//	const FORMAT = inConcert.Utils.format;

	//---------------------------------------------------------------------
	// conf_filter_tree
	function conf_filter_tree(params)
	{
		if (!params) {
			throw("conf_filter_tree - 'params' is required");
		}
		if (!params.owner) {
			throw("conf_filter_tree - 'params.owner' is required");
		}
		if (!params.$container) {
			throw("conf_filter_tree - 'params.$container' is required");
		}
		if (!params.$parent_container) {
			throw("conf_filter_tree - 'params.$parent_container' is required");
		}
		if (!params.tree) {
			throw("conf_filter_tree - 'params.tree' is required");
		}

		this.owner             = params.owner;
		this.$container        = params.$container;
		this.$parent_container = params.$parent_container;
		this.initial_state     = params.initial_state || {colapsed: true, selected_all: false};
		this.title             = params.title || "...";
		this.tree              = params.tree;

		let callbacks          = params.callbacks || {};
		this.callbacks = {
			on_select_option: function(selected, item) {
				(callbacks.on_select_option || $.noop)(selected, item);
			},
			on_colapsed_uncolapsed: function(action) {
				(callbacks.on_colapsed_uncolapsed || $.noop)(action);
			}
		};

		 if (params.auto_init) {
			this.initialize();
		}

		return true;
	}

	if (typeof inConcert.conf_filter_tree !== "function") {
		inConcert.conf_filter_tree = conf_filter_tree;
	}

	conf_filter_tree.prototype.initialize = function()
	{
		// build main html
		this.build_main_html();
		// set events
		this.set_events();
		// intial state (colapsed/uncolapsed)
		if (this.initial_state.colapsed) {
			this.colapse_uncolapse("colapse");
		}
		else {
			this.colapse_uncolapse("uncolapse");
		}
	};

	conf_filter_tree.prototype.dispose = function()
	{
		//TODO: implementar, si es necesario
	};

	conf_filter_tree.prototype.set_events = function()
	{
		const self = this;

		const $colapsed_container = this.$container.find("[colapsed_container]");

		let event_selector = "[colapsed_action]";
		$colapsed_container
			.off("click")
			.on("click", event_selector, function() {
				let with_animation = true;
				self.colapse_uncolapse("uncolapse", with_animation);
			});

		const $uncolapsed_container = this.$container.find("[uncolapsed_container]");
		event_selector = "[uncolapsed_action]";
		$uncolapsed_container
			.off("click")
			.on("click", event_selector, function() {
				let with_animation = true;
				self.colapse_uncolapse("colapse", with_animation);
			});

		event_selector = "[item_selector]";
		$uncolapsed_container.find("[uncolapsed_items_container]")
			.off("click")
			.on("click", event_selector, function() {
				const $this = $(this);
				self.on_select_option($this);
			});
	};

	conf_filter_tree.prototype.on_select_option = function($chk)
	{
		const is_root = $chk.attr("is_root") === "1";
		const is_checked = $chk.prop("checked");
		const $uncolapsed_container = this.$container.find("[uncolapsed_container]");

		const has_childs = $chk.attr("has_childs");
		// 1 - Si el item que se esta desmarcando tiene hijos, se deben desmarcar sus hijos tambien.
		if (has_childs === "1") {
			const dom_item_id = $chk.attr("id");
			const $childs_list_container = $uncolapsed_container.find("[childs_list_id='" + dom_item_id + "']");
			const $childs_checkboxes_tree = $childs_list_container.find("[item_selector]");

			if (!is_checked) {
				$childs_checkboxes_tree.prop("checked", false);
			}
			else {
				$childs_checkboxes_tree.prop("checked", true);
			}
		}

		// funcion recursiva, que marca en true al padre, abuelo, etc, si es que algun hijo se ha marcado como true.
		let markup_unmarkup_parents;
		let $root;
		markup_unmarkup_parents = function($current_chk) {
			const parent_chk_dom_id = $current_chk.attr("parent_chk_dom_id");
			if (!parent_chk_dom_id) {
				return;
			}
			const $parent = $("#" + parent_chk_dom_id);
			if (!$parent) {
				return;
			}
			let is_root_parent = $parent.attr("is_root") === "1";
			if (is_root_parent) {
				if (!$root) {
					$root = $parent;
				}
				return;
			}
			const $childs_list_container = $uncolapsed_container.find("[childs_list_id='" + parent_chk_dom_id + "']");
			const $childs_checkboxes_tree = $childs_list_container.find("[item_selector][parent_chk_dom_id='" + parent_chk_dom_id + "']");
			let total_checked = 0;
			$.each($childs_checkboxes_tree, function(_, chk) {
				const $child_chk = $(chk);
				if ($child_chk.prop("checked")) {
					total_checked++;
				}
			});
			if (total_checked > 0) {
				$parent.prop("checked", true);
			}
			if ($current_chk.attr("is_root") !== "1") {
				markup_unmarkup_parents($parent);
			}
		};
		// 2 - start de marcar recursivamente (el padre, abuelo, etc)
		if (!is_root) {
			// 2.1 - en esta funcion NO se marca el root
			markup_unmarkup_parents($chk);
			if ($root) {
				// 2.2 - marco o desmarco el root, (lo marco si están todos sus hijos marcados, de lo contrario lo desmarco)
				const dom_root_id = $root.attr("id");
				const $root_childs_list_container = $uncolapsed_container.find("[childs_list_id='" + dom_root_id + "']");
				const $root_childs_checkboxes_tree = $root_childs_list_container.find("[item_selector]");
				let is_all_childs_checked = false;
				$.each($root_childs_checkboxes_tree, function(_, chk) {
					const $chk = $(chk);
					if ($chk.prop("checked")) {
						is_all_childs_checked = true;
					}
					else {
						is_all_childs_checked = false;
						return false; // break
					}
				});
				$root.prop("checked", is_all_childs_checked);
			}
		}

		// 3 - valores a invocar al callback de quien me haya instanciado.
		const item_id = $chk.attr("item_id");
		const item_label = $chk.attr("item_label");
		const entity_id = $chk.attr("entity_id");
		let item = {
			id       : item_id,
			name     : item_label,
			entity_id: entity_id
		};
		this.callbacks.on_select_option(is_checked, item);
	};

	conf_filter_tree.prototype.colapse_uncolapse = function(action, with_animation)
	{
		this.$parent_container.removeClass("colapsed uncolapsed animate");

		const $colapsed_container = this.$container.find("[colapsed_container]");
		const $uncolapsed_container = this.$container.find("[uncolapsed_container]");

		if ("uncolapse" === action) {
			this.$parent_container.addClass("uncolapsed");
			$colapsed_container.hide();
			$uncolapsed_container.show();
		}
		else if ("colapse" === action) {
			this.$parent_container.addClass("colapsed");
			$uncolapsed_container.hide();
			$colapsed_container.show();
		}
		else {
			throw "conf_filter_tree.colapse_uncolapse - Unrecognized action " + action;
		}

		if (with_animation) {
			this.$parent_container.addClass("animate");
		}

		this.callbacks.on_colapsed_uncolapsed(action);
	};

	conf_filter_tree.prototype.build_main_html = function()
	{
		const $html = $("" +
			"<div colapsed_container='true' class='pageTreeListFilterContainer'>" +
				"<button colapsed_action='true' class='treeCollapseButton openButton'>" +
					"<i class='fas fa-sliders-h'></i>" +
				"</button>" +
			"</div>" +
			"<div uncolapsed_container='true' class='pageTreeListFilterContainer'>" +
				"<div uncolapsed_title_container='true' class='uncolapsedTitleContainer'>" +
					"<label uncolapsed_title='true'><%=_T('Filter')%></label>" +
					"<button uncolapsed_action='true' class='treeCollapseButton closeButton'>" +
						"<i class='far fa-times'></i>" +
					"</button>" +
				"</div>" +
				"<ul uncolapsed_items_container='true' class='treeCollapseContainer'></ul>" +
			"</div>"
		);

		const $html_tree = this.$get_tree_html();
		const html_tree = $html_tree.html();
		$html.find("[uncolapsed_items_container]").html(html_tree);
		this.$container.html($html);

		if (this.initial_state.selected_all) {
			const $uncolapsed_items_container = this.$container.find("[uncolapsed_items_container]");
			const $chk_list = $uncolapsed_items_container.find("[item_selector]");
			$chk_list.prop("checked", true);
		}
	};

	/*
		let html = "" +
			"<ul>" +
				"<li>" +
					"<label>Todos los archivos<span>(501)</span></label>" +
					"<input type='checkbox'>" +
				"</li>" +
					"<li>" +
						"<label>Español</label>" +
						"<input type='checkbox'>" +
					"</li>" +
						"<li>" +
							"<label>Mensajes de Cuenta</label>" +
							"<input type='checkbox'>" +
						"</li>" +
						"<li>" +
							"<label>Diseñador de Flujos</label>" +
							"<input type='checkbox'>" +
						"</li>" +
						"<li>" +
							"<label>Agentes</label>" +
							"<input type='checkbox'>" +
						"</li>" +
					"<li>" +
						"<label>Ingles</label>" +
						"<input type='checkbox'>" +
					"</li>" +
			"</ul>";
	*/
	conf_filter_tree.prototype.$get_tree_html = function()
	{
		let fill_tree, $make_items_container_html, $make_item_html;

		$make_items_container_html = function(/*item*/) {
			return $("<ul childs='true'></ul>");
		};

		$make_item_html = function(item, is_root, parent_chk_dom_id) {
			let $list;
			let chk_dom_id = "chk_" + inConcert.Utils.generateGuid();
			let has_childs = (item.childs || []).length > 0;

			if (has_childs) {
				// genero los hijos
				let parent_chk_dom_id = chk_dom_id;
				$list = fill_tree(item.childs, false, parent_chk_dom_id);
			}

			let root_val = is_root ? "1" : "0";
			let childs_val = has_childs ? "1" : "0";

			// el <li>
			let $item = $("<li></li>");
			$item.attr("item_id", item.value);
			$item.attr("item_label", item.label);
			$item.attr("entity_id", item.entity_id);
			$item.attr("is_root", root_val);
			$item.attr("has_childs", childs_val);
//			$item.data("item", item);

			item.chk_dom_id = chk_dom_id;
			item.parent_chk_dom_id = parent_chk_dom_id;

			// el <label>
			let $lbl = $("<label></label>");
			$lbl.text(item.label);
			$lbl.attr("for", chk_dom_id);
			$item.append($lbl);
	
			// el <input checkbox>
//			if (!is_root) {
				let $chk = $("<input item_selector='true' type='checkbox' />");
				$chk.attr("id", chk_dom_id);
				$chk.attr("parent_chk_dom_id", parent_chk_dom_id);
				$chk.attr("item_id", item.value);
				$chk.attr("item_label", item.label);
				$chk.attr("entity_id", item.entity_id);
				$chk.attr("is_root", root_val);
				$chk.attr("has_childs", childs_val);
				$item.append($chk);
//			}

			// agrego los hijos <ul>.. al item padre <li>...
			$item.append($list);

			return $item;
		};

		fill_tree = function(items, is_root, parent_chk_dom_id) {
			if ((items || []).length === 0) {
				return;
			}
			let $list;
			items.forEach((item, i) => {
				if (i === 0) {
					$list = $make_items_container_html(item);
				}
				let $item = $make_item_html(item, is_root, parent_chk_dom_id);
				$list.append($item);
			});
			if ($list) {
				$list.attr("childs_list_id", parent_chk_dom_id);
			}
			return $list;
		};

		let $html = fill_tree([this.tree], true);
		return $html;
	};

	// conf_filter_tree.prototype.mark_selected_items = function(selected_items, checked)
	// {
	// 	const $uncolapsed_items_container = this.$container.find("[uncolapsed_items_container]");
	// 	const $chk_list = $uncolapsed_items_container.find("[item_selector]");

	// 	$.each($chk_list, function(_, chk) {
	// 		let $chk = $(chk);
	// 		$chk.prop("checked", false);
	// 	});

	// 	if (selected_items.length === 0) {
	// 		return;
	// 	}

	// 	$.each($chk_list, function(_, chk) {
	// 		let $chk = $(chk);
	// 		let entity_id = ($chk.attr("entity_id") || "").toLowerCase();
	// 		let item_id = ($chk.attr("item_id") || "").toLowerCase();

	// 		let selected_item = selected_items.find(function(item) {
	// 			if (item.value.toLowerCase() !== item_id) {
	// 				return false;
	// 			}
	// 			if (item.name.toLowerCase() !== entity_id) {
	// 				return false;
	// 			}
	// 			return true;
	// 		});
	// 		if (selected_item) {
	// 			$chk.prop("checked", checked);
	// 		}
	// 	});
	// };

	conf_filter_tree.prototype.get_plain_selected_items = function()
	{
		let plain_selected_items = [];
		let recursirve_selected_items_fn;

		recursirve_selected_items_fn = function(items, is_root) {
			if ((items || []).length === 0) {
				return;
			}
			items.forEach((item) => {
				if (!is_root) {
					if (item.chk_dom_id) {
						const $chk = $("#" + item.chk_dom_id);
						if ($chk.prop("checked")) {
							let selected_item = {
								id       : $chk.attr("item_id"),
								name     : $chk.attr("item_label"),
								entity_id: $chk.attr("entity_id")
							};
							plain_selected_items.push(selected_item);
						}
					}
				}
				recursirve_selected_items_fn(item.childs, false);
			});
		};

		recursirve_selected_items_fn([this.tree], true);

		return plain_selected_items;
	};

	conf_filter_tree.prototype.get_tree_selected_items = function()
	{
		let recursirve_selected_items_fn;
		recursirve_selected_items_fn = function(items) {
			if ((items || []).length === 0) {
				return;
			}
			items.forEach((item) => {
				if (item.chk_dom_id) {
					const $chk = $("#" + item.chk_dom_id);
					item.selected = $chk.prop("checked");
				}
				recursirve_selected_items_fn(item.childs, false);
			});
		};
		let tree_items = inConcert.Utils.cloneObject(this.tree);
		recursirve_selected_items_fn([tree_items], true);
		return tree_items;
	};

	conf_filter_tree.prototype.get_filter_values = function(type = "tree") {
		let filter_values;
		if (type === "tree") {
			filter_values = this.get_tree_selected_items();
		}
		else {
			filter_values = this.get_plain_selected_items();
		}
		return filter_values;
	};

	conf_filter_tree.prototype.is_colapsed = function() {
		const $uncolapsed_container = this.$container.find("[uncolapsed_container]");
		const is_visible_tree_filter = $uncolapsed_container.is(":visible");
		return !is_visible_tree_filter;
	};

	//-----------------------------------------------------------
}());
