/*jshint quotmark:false, -W014, esversion: 11*/
/*global $, jQuery, inConcert*/
/*
Ha
*/
// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	/*jshint validthis:true*/
	"use strict";

	const FORMAT = inConcert.Utils.format;

	function conf_view( dashboard )
	{
		this.dashboard = dashboard;
		this.hash_main_rows = {};
		this.initialize();
	}

	if ( typeof inConcert.conf_view !== "function" ) {
		inConcert.conf_view = conf_view;
	}

	conf_view.prototype.initialize = function()
	{
		this.m_in_view = null;
		this.current_query = null;
		this.current_search = null;
		this.grid_custom_action = null;
		this.enabled_paginated = true;
		this.stored_scroll_pos = null;
		this.m_edit_mode = false;
		this.build_grid_markup();
		this.clean_grid();
	};

	conf_view.prototype.build_grid_markup = function()
	{
		const markup = this.get_markup_definition_by_type();
		if (markup) {
			const $grid_container = this.$get_dom_element_by_id("view_grid_container");
			let info = {
				grid_dom_id: this.get_dom_element_by_id("view_grid"), // conf_view_grid
				grid_class : "viewGrid",
				thead_class: "viewGridHeader",
				tbody_class: "viewGridBody"
			};
			let html_grid = markup.container(info);
			$grid_container.html(html_grid);
		}
	};

	conf_view.prototype.get_dom_element_by_id = function(id)
	{
		return this.dashboard.get_dom_element_by_id(id);
	};

	conf_view.prototype.$get_dom_element_by_id = function(id)
	{
		return this.dashboard.$get_dom_element_by_id(id);
	};

	conf_view.prototype.is_active_paginated = function()
	{
		return this.enabled_paginated;
	};

	conf_view.prototype.enable_paginated = function(enabled)
	{
		this.enabled_paginated = enabled;
	};

	conf_view.prototype.update_row_counters = function()
	{
		const self = this;
		const $rows = this.$get_rows();
		let i = 0;
		$rows.each(function(_, row) {
			const $row = $(row);
			const row_type = $row.attr("row_type");
			if ("main" === row_type) {
				let selector = "{0}[column_type='counter']";
				selector = self.replace_markup_by_type(selector, "column"); // TD O LI
				const $cell = $row.find(selector);
				if ($cell.length > 0) {
					i++;
					$cell.text(i);
				}
			}
		});
	};

	conf_view.prototype.$get_subgrid_row = function( $main_row )
	{
		var main_row_id = $main_row.attr( "row_id" );
		let selector = "{0}[row_id='" + main_row_id + "'][row_type='subgrid']";
		selector = this.replace_markup_by_type(selector, "row"); // TR O UL
		var $subgrid_row = $main_row.parent().find(selector);
		return $subgrid_row;
	};

	/*
		Para saber si nos encontramos en una vista o en un form.
	*/
	conf_view.prototype.get_in_view = function()
	{
		return this.m_in_view;
	};

	/*
		Para marcar si estamos en una vista o en un form.
	*/
	conf_view.prototype.set_in_view = function(in_view)
	{
		this.m_in_view = in_view;
	};

	conf_view.prototype.set_edit_mode = function(isEditMode)
	{
	 	this.m_edit_mode = isEditMode;
	};

	conf_view.prototype.isEditMode = function()
	{
		return this.m_edit_mode;
	};

	conf_view.prototype.erase_grid_row = function( $main_row )
	{
		var self = this;
		if ( !$main_row || $main_row.length === 0 ) {
			return;
		}
		var is_main_row;
		var main_row_id = $main_row.attr( "row_id" );
		var has_subgrid = this.has_subgrid();
		if ( has_subgrid.success ) {
		// elimina de la subgrilla
			var $subgrid_row = this.$get_subgrid_row( $main_row );
			$subgrid_row.fadeOut( "slow", function() {
				is_main_row = false;
				self.m_paginator.remove( $subgrid_row, is_main_row );
			});
		}

		// elimina la row principal
		$main_row.fadeOut( "slow", function() {
			delete self.hash_main_rows[ main_row_id ];
			is_main_row = true;
			self.m_paginator.remove( $main_row, is_main_row );
			var action_type = "DECREMENT";
			self.show_total_rows( null, action_type );
			self.update_row_counters();
		});
	};

	conf_view.prototype.show_status_message = function( message, type, timeout_ms, scrolling, show_custom_button, $custom_message_container, callbacks, wrap_classes)
	{
		var close = false;
		scrolling = scrolling || true; // default is true
		this.dashboard.show_status_message(message, type, timeout_ms, close, scrolling, show_custom_button, $custom_message_container, callbacks, wrap_classes);
	};

	conf_view.prototype.hide_status_message = function(timeout_ms, callback, $custom_message_container)
	{
		this.dashboard.hide_status_message(timeout_ms, callback, $custom_message_container);
	};

	conf_view.prototype.toggle_split_screen = function(force_action)
	{
		force_action = force_action || "NONE";
		const $body = $("body");
		let result = this.dashboard.toggle_navigation(force_action);
		if (!result.visible) {
			$body.addClass("splittedScenarioConfiguration");
		}
		else {
			$body.removeClass("splittedScenarioConfiguration");
			this.form.hide_header_custom_actions();
			this.form.hide_custom_actions();
			this.destroy_split_info();
		}
	};

	conf_view.prototype.execute_after_erase = function(action, $row)
	{
		let action_to_execute;
		let callback_after_erase = $.noop;
		if (action && action.view.actions && action.view.actions.erase) {
			action_to_execute = action.view.actions.erase.after_erase;
			callback_after_erase = action.view.actions.erase.callback_after_erase || $.noop;
		}

		action_to_execute = action_to_execute || "remove-row"; // default vaue
		if (action_to_execute === "remove-row") {
			if ($row) {
				this.erase_grid_row( $row );
				callback_after_erase( $row, this );
			}
		}
		else if (action_to_execute === "refresh-view") {
			this.refresh(undefined, function (_) {
				callback_after_erase( $row, this );
			});
		}
		else {
			throw("Configuration - unrecognized action: " + action_to_execute);
		}
	};

	conf_view.prototype.erase = function( action, $row )
	{
		const self = this;
		let view = action.view;
		let row_id = $row.attr( "row_id" );
		let display_name = this.sanitize_value($row.attr( "row_name" ));

		this.hide_status_message();

		const get_delete_message = function( message )
		{
			if ( !message ) {
				if (view.messages && view.messages.erase) {
					message = view.messages.erase;
				}
				else {
					message = "<%=_T('Do you want to delete {0} ?')%>";
				}
			}
			message = FORMAT( message, display_name );
			return message;
		};

		const erase = function(erase_data)
		{
			self.dashboard.show_waiting(FORMAT("<%=_T('Please wait, deleting {0}')%>", display_name));

			const callback = function( success, error )
			{
				self.dashboard.hide_waiting();
				let message, message_type, timeout_ms;
				if (success) {
					message = FORMAT("<%=_T('{0} was successfully deleted.')%>", display_name);
					message_type = "Info";
					self.execute_after_erase(action, $row);
				}
				else {
					let default_message = FORMAT("<%=_T('Error deleting {0}.')%>", display_name);

					if (typeof view.messages.err === "function") {
						message = view.messages.err("delete", error, display_name, self);
						if (typeof(message) === "object" && message.can_show_popup_message) {
							self.show_popup_message({
								messages: {
									title  : message.title || "",
									message: message.message,
									detail : message.detail || "",
									type   : message.msg_type || "Error"
								}
							});
							return;
						}
					}
					else {
						let err_messages = {
							FORBIDDEN  : FORMAT( "<%=_T('Unable to delete {0}. You dont have permission.')%>", display_name)
						};
						if ( error ) {
							if (typeof error === "object") {
								message = err_messages[ error.result.toUpperCase() ];
							}
							else {
								message = error ? err_messages[ error.toUpperCase() ] : error;
							}
						}
					}
					if (typeof(message) === "object" && message.not_show_message) {
						return;
					}
					message = message || default_message;
					message_type = "Error";
					timeout_ms = false; //infinite
				}
				self.show_status_message( message, message_type, timeout_ms );
			};

			let uri_erase = view.uri.erase;
			let post_data = [ row_id ];
			if (erase_data) {
				post_data = [{ id : row_id, data : erase_data }];
			}
			if ( typeof view.erase_values === "function" ) {
				post_data = view.erase_values( post_data );
			}
			switch ( typeof uri_erase )
			{
				case "string":
					const api = self.dashboard.get_api();
					api.erase({
						uri                : uri_erase,
						entities_to_delete : post_data,
						callback           : callback,
						request            : view.request
					});
				break;
				case "function":
					uri_erase( row_id, self, callback );
				break;
				default:
					throw("Configuration - erase URI is invalid. URI: " + typeof(uri_erase) );
			}
		};

		const eraseConfirmation = function( success, message )
		{
			self.dashboard.hide_waiting();
			if ( !success ) {
				let message_type, timeout_ms;
				message = FORMAT(message || "<%=_T('Server validation deleting error {0}.')%>", display_name);
				message_type = "Error";
				timeout_ms = false; //infinite
				self.show_status_message(message, message_type, timeout_ms);
				return;
			}

			$.inconcertPopup.Show(
				"<%=_T('Confirm')%>",
				get_delete_message( message ),
				jQuery.inconcertPopup.ButtonSets.BS_YESNO,
				function( response )
				{
					if (response.toLowerCase() !== "yes") {
						return;
					}
					if (typeof view.erase_previous_confirmation === "function") {
						view.erase_previous_confirmation(row_id, function(status, erase_data) {
							if (status) {
								erase(erase_data);
							}
							else if (erase_data) {
								let message_type, timeout_ms;
								message = erase_data;
								message_type = "Error";
								timeout_ms = false; //infinite
								self.show_status_message(message, message_type, timeout_ms);
								return;
							}
						});
					}
					else {
						erase();
					}
				},
				undefined,
				{
					popup_id        : "conf_inconcertPopup",
					container_class : "confirmationPopUp",
					overlay_class   : "confirmationOverlay"
				}
			);
		};

		if (typeof view.erase_previous_validation === "function") {
			self.dashboard.show_waiting("<%=_T('Please wait')%>");
			view.erase_previous_validation(row_id, eraseConfirmation);
		}
		else {
			eraseConfirmation(true);
		}
	};

	conf_view.prototype.dispose = function()
	{
		const current_action = this.dashboard.get_current_action_obj();
		if ( current_action ) {
			this.events_unsuscribe();
		}
		if (this.form) {
			this.form.dispose();
		}
		this.vs_destroy();
		this.tree_filter_destroy();
		this.destroy_split_info();
		this.disable_auto_reload();
	};

	conf_view.prototype.unmark_all_rows = function()
	{
		const $rows = this.$get_rows();
		$rows.removeClass("selected");
	};

	conf_view.prototype.mark_row = function($row)
	{
		if (!$row) {
			return;
		}
		$row.addClass("selected");
	};

	conf_view.prototype.unmark_row = function($row)
	{
		if (!$row) {
			return;
		}
		$row.removeClass("selected");
	};

	conf_view.prototype.render = function( action_id, callback, is_reopen, config)
	{
		const self = this;

		callback = callback || $.noop;
		is_reopen = is_reopen || false;

		this.cancel_search();
		var dashboard = this.dashboard;

		const current_view_id = dashboard.get_current_view_id();
		if ( !is_reopen && current_view_id === action_id ) {
			callback(true);
			return;
		}
		
		const fn_process_render = function () {
			var $app_actions = self.$get_dom_element_by_id("application_actions");
			$app_actions.find(".applicationOptionsItem").removeClass("optionSelected");
			$app_actions.find("[action_id='" + action_id + "']").addClass("optionSelected");
	
			self.dispose();
			dashboard.current_action = action_id || dashboard.current_action;
			var $view_container = self.$get_dom_element_by_id("container");
			var $view_external_container = self.$get_dom_element_by_id("container_external_apps");
	
			// cleaning the view container without destroying the paginator
			// may turn into a performance issue
			if (self.m_paginator) {
				self.m_paginator = self.m_paginator.destroy();
			}
			$view_container.html("");
			var action = dashboard.get_current_action_obj();
			if (!action) {
				$view_container.text("<%=_T('Not has permissions to actions on app!!')%>");
				callback( false );
				return;
			}
			if ( typeof action.view.render === "function" ) {
				action.view.render(
					action.id,
					$view_external_container,
					dashboard,
					function() {
						$view_container.hide();
						$view_external_container.show();
						callback( true );
					});
			}
			else {
				$view_external_container.hide();
				var pages_uri = action.view.render || "/inconcert/apps/configuration/pages/view_content_default.<%=_L%>.html#grunt-cache-bust";
				var api = dashboard.get_api();
	
				api.load_content_page({
					URI: pages_uri,
					callback: function(success, content) {
						if (success && typeof content === "string") {
							$view_container.html(content).show();
							self.ready(action, config);
							callback( true );
						}
					}
				});
			}
		};
		
		var current_view_action = dashboard.get_current_action_obj(current_view_id);
		if (current_view_action && current_view_action.view && typeof current_view_action.view.confirm_jump_menu_nav === "function") {
			return current_view_action.view.confirm_jump_menu_nav(fn_process_render, dashboard);
		}

		dashboard.confirm_jump_menu_nav({
			fn_can_confirm : function() {
				if(!self.form || self.m_in_view){
					//No muestro confirmar
					return false;
				}
				if ( self.form.can_jump_to_other_menunav()) {
					//No muestro confirmar
					self.form.before_jump_to_other_menunav();
					return false;
				}
				return true;
			},
			fn_process : function (success) {
				if (success == false) {
					return;
				}
				if (success) {
					self.form.before_jump_to_other_menunav();
				}
				fn_process_render();
			}
		});		
	};

	// valida si tiene permisos de navigation la current view ( item de la navegacion )
	conf_view.prototype.has_permission_to_navigation = function()
	{
		var self = this;
		var action = this.dashboard.get_current_action_obj();
		var app_id = this.dashboard.current_app;

		if ( typeof action.permissions.view === "string" ) {
			return this.dashboard.permissions.to_navigation( app_id, action.permissions.view );
		}
		else {
			var has_permission = false;
			$.each( action.permissions.view || {},
				function( _, view_permission_id ) {
					has_permission = self.dashboard.permissions.to_navigation( app_id, view_permission_id );
					if ( has_permission ) {
						return false; // break
					}
				});
			return has_permission;
		}
	};

	/*
	 * Esta funcion genera el markup necesario para las acciones de cada row de la grilla
	 */
	conf_view.prototype.get_row_action_html = function( row, column, action )
	{
		const self = this;
		if (!row || !column || !action) {
			return "";
		}
		const get_button_html = function( action_id, label, is_enabled, additional_class, tooltip, action_btn_separator)
		{
			/*
			 * Valores del parametro is_enabled:
			 *    1  = La accion se debe visualizar y estar Enabled.
			 *    0  = La accion se debe visualizar y estar Disabled.
			 *    -1 = La accion no se genera en el markup. (No aparece el boton de la accion).
			 */
			if (is_enabled === -1) {
				return "";
			}
			label = label || "";
			let disabled_attr = (is_enabled === 1 ? "" : "disabled='true'");
			let classes = "buttonColorGray gridActionButton";
			classes = classes + ( additional_class ? " " + additional_class : "" );

			let i_classes = "";
			i_classes = i_classes + ( additional_class ? " buttons-icons-" + additional_class : "" );
			let button_dom_id = inConcert.Utils.generateGuid();
			let button_html =
				"<button  id='" + button_dom_id + "' type='button' " + disabled_attr + " action='" + action_id + "' title='" + tooltip + "' class='" + classes + "'>" +
					"<i class='" + i_classes + "'></i>" +
					// label +
				"</button>";

			if (action_btn_separator) {
				button_html = button_html + "<span class='action-buttons-separator'></span>";
			}

			return button_html;
		};

		let no_log = true;
		// permission to action ( permisos sobre vcc )
		const has_permission_to_action = function( app_id, perm_action_id, action_type ) {
			return self.dashboard.permissions.to_action( app_id, perm_action_id, action_type, undefined, no_log );
		};
		// permission to action campaign ( campaign es opcional )
		const has_permission_to_action_campaign = function( app_id, perm_action_id, action_type ) {
			let campaign_id = action.permissions.get_campaign_id( action.view, row );
			return self.dashboard.permissions.to_action( app_id, perm_action_id, action_type, campaign_id, no_log );
		};

		let has_permission_to_action_vcc_or_cmp;
		if ( action.permissions.permission_type === "campaign" ) {
			has_permission_to_action_vcc_or_cmp = has_permission_to_action_campaign;
		}
		else {
			has_permission_to_action_vcc_or_cmp = has_permission_to_action;
		}

		let app_id = this.dashboard.current_app;
		let is_enabled = column.is_enabled || function() {return 1;/*Enable x defecto*/};

		let button_class_name;

		// edit button
		let html_edit_button = "";
		let html_delete_button = "";
		let html_split_screen_button = "";
		if (column.system_action) {
			if (column.system_action.edit) {
				let row_id = row[ action.view.row_id ];
				// navegacion sobre el item del menu (skill, states, etc)
				let has_navigation_permission = this.has_permission_to_navigation();
				if (has_navigation_permission) {
					if (!this.form.has_sections() || this.form.count_enabled_sections( row_id ) > 0) {
						let caption_edit_button = "<%=_T('Edit')%>";
						if (!has_permission_to_action_vcc_or_cmp(app_id, action.permissions.save, "SAVE")) {
							caption_edit_button =  "<%=_T('View')%>";
							button_class_name = "view";
						}
						else {
							button_class_name = "edit";
						}
						let is_enabled_edit = is_enabled("EDIT", row);
						let tooltip = caption_edit_button;
						if (typeof column.system_action.edit === "object") {
							caption_edit_button = column.system_action.edit.name || "";
							tooltip = column.system_action.edit.tooltip || tooltip;
							button_class_name = column.system_action.edit.class_name || button_class_name;
						}
						html_edit_button = get_button_html("edit", caption_edit_button, is_enabled_edit, button_class_name, tooltip);
					}
				}
			}
	
			// delete button
			if (column.system_action.erase) {
				if (has_permission_to_action_vcc_or_cmp( app_id, action.permissions.erase, "DELETE")) {
					let is_enabled_delete = is_enabled("DELETE", row);
					button_class_name = "delete";
					let caption_button = "<%=_T('Delete')%>";
					let tooltip = caption_button;
					if (typeof column.system_action.erase === "object") {
						caption_button = column.system_action.erase.name || "";
						tooltip = column.system_action.erase.tooltip || tooltip;
						button_class_name = column.system_action.erase.class_name || button_class_name;
					}
					html_delete_button = get_button_html("delete", caption_button, is_enabled_delete, button_class_name, tooltip);
				}
			}
	
			// split screen button
			if (column.system_action.split_screen) {
				let is_enabled = 1;
				button_class_name = "split_screen";
				let caption_button = "<%=_T('Split Screen')%>";
				let tooltip = caption_button;
				if (typeof column.system_action.split_screen === "object") {
					caption_button = column.system_action.split_screen.name || "";
					tooltip = column.system_action.split_screen.tooltip || tooltip;
					button_class_name = column.system_action.split_screen.class_name || button_class_name;
				}
				html_split_screen_button = get_button_html("split_screen", caption_button, is_enabled, button_class_name, tooltip);
			}
		}
		
		// custom buttons
		let html_custom_buttons = "";
		if (typeof column.custom_action === "object")
		{
			let hasExecuteActions = false;

			$.each( column.custom_action, function( custom_action_id, custom_action )
			{
				hasExecuteActions = typeof custom_action.execute === "function" ? true : false;
				if (!custom_action.permission || inConcert.permissions.to_action(app_id, custom_action.permission)) {
					let is_enabled_custom = is_enabled(custom_action_id, row);
					button_class_name = custom_action.class_name || undefined;
					let action_btn_separator = custom_action.action_separator;
					let caption = custom_action.name;
					let tooltip;
					if (typeof(custom_action.tooltip) === "function") {
						tooltip = custom_action.tooltip(row);
					}
					else {
						tooltip = custom_action.tooltip;
					}
					tooltip = tooltip || caption;
					html_custom_buttons = html_custom_buttons + get_button_html( custom_action_id, caption, is_enabled_custom, button_class_name, tooltip, action_btn_separator);
					//TODO: #9335 - revisar merge mcwarren con cany
					//if(custom_action.class_name === 'export'){
						//html_custom_buttons = html_custom_buttons + get_button_html( custom_action_id, caption, is_enabled_custom, button_class_name, tooltip ) + "<span class='action-buttons-separator'></span>";
					//} else {
						//html_custom_buttons = html_custom_buttons + get_button_html( custom_action_id, caption, is_enabled_custom, button_class_name, tooltip );
					//}
				}
			});

			if (hasExecuteActions)
				this.grid_custom_action = column.custom_action;
		}

		// <td> o <li>
		const markup = this.get_markup_definition_by_type();
		let html_action =
			"<{0} column_type='actions'>" +
				html_custom_buttons + (column.system_action?
				"<span class='action-buttons-separator'></span>" +
				html_edit_button    +
				html_delete_button  +
				html_split_screen_button :"") +
			"</{0}>";

		return FORMAT(html_action, markup.body.column);
	};

	conf_view.prototype.get_row_classes = function( entity, action, $row )
	{
		var classes = "";
		var has_subgrid = this.has_subgrid();
		// clases para la subgrid
		if ( has_subgrid.success ) {
			var colapsed = $row.prop( "colapsed-sub-grid" );
			classes = classes + "rowHeaderForNestedTable " + ( colapsed ? "colapsed" : "uncolapsed" );
		}
		// clases particulares para el entity, si es que se especifica en la configuration
		var rows_def = action.view.rows;
		if ( !rows_def || typeof rows_def.get_row_classes !== "function" ) {
			return classes;
		}
		var specific_row_classes = rows_def.get_row_classes( entity );
		if ( typeof specific_row_classes !== "string" ) {
			return classes;
		}
		return classes + " " + specific_row_classes;
	};

	// esta funcion es la encargada de cargar los TD de los rows con sus datos.
	conf_view.prototype.$fill_row_columns_content = function( $row, item, columns, rows_number_per_record, row_count )
	{
		const self = this;

		let is_readonly_frm = false;
		let disabled_attr = "disabled='true'";

		rows_number_per_record = rows_number_per_record || 1;
		let action = this.dashboard.get_current_action_obj();

		const make_html_header_lables_inside_fields = function(column) {
			if (!self.header_lables_inside_fields) {
				return "";
			}
			if (!column || !column.name) {
				return "";
			}
			return FORMAT("<label cell_lbl='true'>{0}</label>", column.name);
		};

		const get_cell_class_name = function(column) {
			if (!column || !column.cell_class_name) {
				return "";
			}
			return column.cell_class_name;
		};

		// La definicion del markup dependiendo de si usa <TABLE..> o <DIV..> para la creacion de la grilla.
		const markup = this.get_markup_definition_by_type();

		$.each( columns, function(_, column)
		{
			let $html_col = null;
			let column_type = column.type || "";
			let cell_class_name = get_cell_class_name(column);

			if (column_type === "counter" && typeof row_count === "number") {
				self.has_counter_column = true;
				$html_col = $(FORMAT(
					"<{0} column_id='" + column.id + "' column_type='" + column_type + "' class='" + cell_class_name + "'>" + row_count + "</{0}>",
					markup.body.column
				));
				item.__number = row_count;
			}
			else if (column_type === "row_selector" || column_type === "cell_checkbox") {
				let is_checked_fn = column.is_checked || function(row) {
					if (!row || !column.id || !row.hasOwnProperty(column.id)) {
						return false;
					}
					let check_val = row[column.id];
					switch (typeof(check_val)) {
						case "boolean":
							return check_val;
						case "string":
							return parseInt(check_val, 10) === 1 ? true: false;
						case "number":
							return check_val === 1 ? true: false;
					}
					return false;
				};
				let checked_attr = is_checked_fn(item) ? "checked" : "";
				let html_header_inside = make_html_header_lables_inside_fields(column);
				$html_col = $(FORMAT(
					"<{0} column_id='" + column.id + "' column_type='" + column_type + "' class='" + cell_class_name + "'>" +
						html_header_inside +
						"<input ctrl_type='checkbox' class='formCheckbox' type='checkbox' " + checked_attr + " " + (is_readonly_frm ? disabled_attr : "") + " />" +
					"</{0}>",
					markup.body.column
				));
			}
			else if (column.system_action || column.custom_action) {
				$html_col = $( self.get_row_action_html( item, column, action ) );
			}
			else {
				var original_value, cell_value, class_name, column_class_name, image_url;

				// get value
				if ( typeof column.get_value === "function" ) {
				// lo obtiene de una funcion
					var response = column.get_value( item );
					original_value = response.original_value;
					cell_value = response.value !== undefined ? response.value : "";
					class_name = response.class_name !== undefined ? response.class_name : "";
					column_class_name = response.column_class_name !== undefined ? response.column_class_name : "";
					image_url = response.image_url;
				}
				else {
				// lo obtiene directo de los datos
					original_value = typeof item[ column.id ] !== "undefined" ? item[ column.id ] : "";
					cell_value = original_value;
				}

				// Sanitizo el valor a visualizar en la grilla (elimino markup html, etc)
				if (!column.render_html_value) {
					cell_value = self.sanitize_value(cell_value);
				}
				// El tooltip
				let tooltip_attr = "";
				if (!column.without_tooltip) {
					let tooltip = column.render_html_value ? (original_value || cell_value) : self.sanitize_value(cell_value);
					// get_tooltip value
					if (typeof column.get_tooltip === "function" ) {
					// lo obtiene de una funcion
						tooltip = column.get_tooltip(item);
					}
					tooltip_attr = FORMAT(" title='{0}' ", tooltip);
				}

				switch ( column_type )
				{
					case "image":
						var image_cell_value = "<i class='" + class_name + "'></i>";
						if (column.image_type === "image_with_text") {
							cell_value = image_cell_value + cell_value;
						}
						else {
							cell_value = image_cell_value;
						}
					break;
					
					case "chip":
						var chip_value = "<span class='icchip " + class_name + "'>"+cell_value+"</span>";
						cell_value = chip_value;
						break;

					case "progressbar":
						cell_value = cell_value && `
							<div class='progress-container' title='${cell_value.tooltip || ""}'>
								<div class='progress-percentage'>${cell_value.percentage}%</div>
								<div class='progress-bar-container'>
									<div class='progress-bar-background'>
										<div class='progress-bar' style='width:${cell_value.percentage}%;'></div>
									</div>
									<div class='progress-remaining-time'> <%=_T('Remaining time')%> ${cell_value.remainingTime}</div>
								</div>
							</div>`;
						break;

					case "boolean":
						if ( cell_value === true || parseInt( cell_value, 10 ) === 1 ) {
							class_name = "fa fa-check-circle";
						}
						else {
							class_name = "fa fa-times-circle";
						}
						cell_value = "<i class='" + class_name + "'></i>";
						break;

					case "textWithIcon":
						if(cell_value.alerted === true){
							cell_value = `${cell_value.text}
								<div class='alertedColumnIndicator'>
									<i class='far fa-exclamation-triangle'></i>
								</div>`;
						}
						else{
							cell_value = `${cell_value.text}`;
						}
						break;

					case "date":
						let date_config = column.date_config || {};
						let date_object = inConcert.Utils.getDateFromString(cell_value);
						if (date_object instanceof Date) {
							cell_value = inConcert.Utils.getStringFromDate(date_object, date_config.utc, date_config.type_date, date_config.format_date, date_config.separator_date);
						}
						break;

					case "timestampSec":
						if (!isNaN(Number(cell_value))) {
							let time_ms = Number(cell_value) * 1000;
							cell_value = inConcert.Utils.formatDate(time_ms, "date-short-year-short-hour");
						}
						break;
					
					case "multi_image":
						if (Array.isArray(cell_value)) {
							let new_cell_value = "";
							for (let index = 0; index < cell_value.length; index++) {
								const obj = cell_value[index];
								new_cell_value += "<i title='"+ obj.tooltip +"'class='" + obj.class_name + "'></i>";
							}
							cell_value = new_cell_value;
						}
						break;
				}

				// si no se ha asignado un valor y se ha definido un valor por defecto
				if (!cell_value) {
					cell_value = column.default_value ? column.default_value : (cell_value || "");
				}

				if (column_class_name) {
					cell_class_name = cell_class_name + (cell_class_name ? " " : "") + column_class_name;
				}

				$html_col = $(FORMAT(
					"<{0} " + tooltip_attr + " column_id='" + column.id + "' column_type='" + column_type + "' class='" + cell_class_name + "'>" + cell_value + "</{0}>",
					markup.body.column
				));

				if (image_url) {
					$html_col.find("i")
						.addClass("withImage")
						.append("<img src='" + image_url + "'>");
				}

				$html_col.data("original_value", original_value || "");
			}

			if ($html_col) {
				$row.append($html_col);
			}
		});

		$row.data( "row_data", item );

		return $row;
	};

	conf_view.prototype.$get_rows = function()
	{
		if (!this.m_paginator) {
			return;
		}
		return this.m_paginator.$get_rows();
	};

	conf_view.prototype.get_selected_items = function()
	{
		let $rows = this.$get_rows();
		let selected_items = [];
		if (!$rows) {
			return selected_items;
		}
		$.each($rows, function(_, row)
		{
			let $row = $(row);
			let $row_selector = $row.find("[column_type='row_selector']");
			let $checkbox = $row_selector.find("input[type='checkbox']");
			if ($checkbox.prop("checked")) {
				let item = $row.data("row_data");
				let cloned_item = inConcert.Utils.cloneObject(item);
				let $col_input = $row.find("[column_type='cell_input']");
				if ($col_input.length > 0) {
					let $input = $col_input.find("input[type='text']");
					let column_id = $col_input.attr("column_id");
					let value = $input.val();
					cloned_item[column_id] = value;
				}
				selected_items.push(cloned_item);
			}
		});
		return selected_items;
	};

	conf_view.prototype.can_make_sub_grid = function( rows )
	{
		var has_subgrid = this.has_subgrid();
		if ( !has_subgrid.success ) {
			return has_subgrid;
		}

		var count_rows = Object.keys( rows || {} ).length;
		if ( count_rows === 0 ) {
			has_subgrid.success = false;
			return has_subgrid;
		}

		return has_subgrid;
	};

	conf_view.prototype.make_sub_grid = function( sub_rows, $row_parent, colspan )
	{
		const self = this;

		// create container
		const markup = this.get_markup_definition_by_type();
		let columns_markup = FORMAT("<{0}></{0}>", markup.head.column);
		// "<td></td>" o "<li></li>"
		$row_parent.append(columns_markup); // td vacio

		const has_subgrid = this.can_make_sub_grid( sub_rows );

		if ( !has_subgrid.success ) {
			const action = this.dashboard.get_current_action_obj();
			let message;
			if ( typeof( action.view.sub_grid.messages ) === "function" ) {
				message = action.view.sub_grid.messages( "no_records_found" );
			}
			message = message || "<span class='messageInfoText1'><%=_T('No results found.')%></span> <span class='messageInfoText2'><%=_T('There are no items created or matching your search yet.')%></span>";
			let column_html = "<{0} colspan='" + colspan + "'><div class='inlineMessagesContainer messageInfo m-3'>" + message + "</div></{0}>";
			// reemplazo los {0} por th o div
			column_html = FORMAT(column_html, markup.head.column);

			$row_parent.append(column_html);
			return;
		}

		let info = {
			grid_class : "nestedTable"
		};
		let html_sub_grid = markup.container(info);
		let column_html = ""+
			"<{0} class='nestedTableContainer' colspan='" + colspan + "' >" +
				html_sub_grid +
			"</{0}>";

		// reemplazo los {0} por th o div
		column_html = FORMAT(column_html, markup.head.column);
		const $sub_grid_container = $(column_html);

		// create header
		const columns = has_subgrid.columns;
		const $columns = this.$get_header_columns( columns );
		$sub_grid_container.find( "thead" ).append( $columns );

		$row_parent.append( $sub_grid_container );

		// create data
		const count_rows = Object.keys( sub_rows || {} ).length;
		if ( count_rows === 0 ) {
			return;
		}

		// reordena las sub_rows
		if ( typeof has_subgrid.sort === "function" ) {
			sub_rows = has_subgrid.sort( sub_rows );
		}
		const is_array = $.isArray( sub_rows );

		const row_parent_id = $row_parent.attr( "row_id" );
		// aca carga los sub_rows de la subgrilla con sus datos
		const $tbody = $sub_grid_container.find( "tbody" );
		let row_number = 0;

		// "<tr></tr>" o "<ul></ul>"
		let row_markup = FORMAT("<{0}></{0}>", markup.body.row);

		$.each( sub_rows, function( row_id, sub_row )
		{
			sub_row.__id = is_array ? ( sub_row.Id || row_id ) : row_id; // el id de la sub row se agrega en este attr
			const $sub_row = $(row_markup);
			$sub_row.attr( "row_id", row_id );
			$sub_row.attr( "row_name", row_id );
			$sub_row.attr( "row_type", "child" );
			$sub_row.attr( "row_parent_id", row_parent_id );
			row_number++;
			self.$fill_row_columns_content( $sub_row, sub_row, columns, undefined, row_number );
			$tbody.append( $sub_row );
		});

		return true;
	};

	conf_view.prototype.$get_rows_html = function(entity, row_count)
	{
		if ( entity === undefined ) {
			return;
		}

		var action = this.dashboard.get_current_action_obj();
		var columns = action.view.columns;

		if ( !$.isArray(columns) || columns.length === 0 ) {
			return;
		}

		var row_id = entity[ action.view.row_id ];
		if ( this.hash_main_rows[ row_id ]) {
			return;
		}

		this.hash_main_rows[ row_id ] = true;
		var has_subgrid = this.has_subgrid();
		var rows_number_per_record = has_subgrid.success ? 2 : 1; // si tiene 2 filas por registro ( el segundo es para la subgrilla)
		var display_name = entity[ action.view.display_name ];

		var $new_rows = [];
		// aca genero los registros para el row principal (siempre), y el segudo row para la subgrilla (solo si tiene subgrid definida)
		for ( var i = 1; i <= rows_number_per_record; i++ ) {
			// "<tr></tr>" o "<ul></ul>"
			const markup = this.get_markup_definition_by_type();
			let row_markup = FORMAT("<{0}></{0}>", markup.body.row);
			let $row = $(row_markup);
			$row.attr( "row_id", row_id );
			$row.attr( "row_name", display_name );
			if ( i === 1 ) {
			// la uno es la main row
				$row.attr( "row_type", "main" );
				if ( has_subgrid.success ) {
					$row.prop( "colapsed-sub-grid", action.view.sub_grid.colapsed ? true : false );
				}
				$row.addClass( this.get_row_classes( entity, action, $row ) );
			}
			else {
			// aca ingresa solo si tiene subgrilla
				$row.attr( "row_type", "subgrid" );
				$row.addClass( "rowForNestedTable" );
				$row.prop( "colapsed", action.view.sub_grid.colapsed ? true : false );
				var sub_rows = entity[ action.view.sub_grid.data_source ];
				var colspan = columns.length - 1;
				this.make_sub_grid( sub_rows, $row, colspan );
			}
			$new_rows.push( $row );
		}

		// aca carga los datos de las columnas de la grilla principal
		var $main_row = $new_rows[ 0 ];
		this.$fill_row_columns_content( $main_row, entity, columns, rows_number_per_record, row_count );

		return $new_rows;
	};

	conf_view.prototype.has_subgrid = function()
	{
		let has_subgrid = {
			success : false,
			columns : null,
			sort    : undefined
		};
		let action = this.dashboard.get_current_action_obj();
		if ( action.view.sub_grid ) {
			has_subgrid.columns = action.view.sub_grid.columns;
			has_subgrid.sort    = action.view.sub_grid.sort;
		}
		has_subgrid.success = $.isArray( has_subgrid.columns ) && has_subgrid.columns.length > 0;
		return has_subgrid;
	};

	conf_view.prototype.is_created_markup_with_divs = function()
	{
		let action = this.dashboard.get_current_action_obj();
		if (action && action.view && action.view.markup_with_divs) {
			return true;
		}
		return false;
	};

	conf_view.prototype.$tbody = function()
	{
		const $grid = this.$grid();
		let selector = this.is_created_markup_with_divs() ? "[tbody='true']" : "tbody";
		const $tbody = $grid.find(selector);
		return $tbody;
	};

	conf_view.prototype.$thead = function()
	{
		const $grid = this.$grid();
		let selector = this.is_created_markup_with_divs() ? "[thead='true']" : "thead";
		const $thead = $grid.find(selector);
		return $thead;
	};

	conf_view.prototype.get_markup_definition_by_type = function()
	{
		if (!this.m_markup_by_type) {
			this.m_markup_by_type = {
				div: {
					container: function(info) {
						let grid_dom_id = info.grid_dom_id || inConcert.Utils.generateGuid();
						return "" +
							"<div id='" + grid_dom_id + "' table='true' class='" + (info.grid_class || "") + "'>" +
								"<div thead='true' class='" + (info.thead_class || "") + "'></div>" +
								"<div tbody='true' class='" + (info.tbody_class || "") + "'></div>" +
							"</div>";
					},
					head: {
						row: "ul",
						column: "li"
					},
					body: {
						row: "ul",
						column: "li"
					}
				},
				table: {
					container: function(info) {
						let grid_dom_id = info.grid_dom_id || inConcert.Utils.generateGuid();
						return "" +
							"<table id='" + grid_dom_id + "' cellpadding='0' cellspacing='0' border='0' class='" + (info.grid_class || "") + "'>" +
								"<thead class='" + (info.thead_class || "") + "'></thead>" +
								"<tbody class='" + (info.tbody_class || "") + "'></tbody>" +
							"</table>";
					},
					head: {
						row: "tr",
						column: "th"
					},
					body: {
						row: "tr",
						column: "td"
					}
				}
			};
		}

		let type = this.is_created_markup_with_divs() ? "div" : "table";
		return this.m_markup_by_type[type];
	};

	/*
		Return: "tr[row_id]" or ul[row_id] / tr[row_id='algo'] ...
	*/
	conf_view.prototype.__jquery_row_selector = function(with_value)
	{
		const markup = this.get_markup_definition_by_type();
		if (with_value) {
			return FORMAT("{0}[row_id='{1}']", markup.body.row, with_value);
		}
		return FORMAT("{0}[row_id]", markup.body.row);
	};

	/*
		Return: "td[column_type]" or li[column_type] / td[column_type='algo'] ...
	*/
	conf_view.prototype.__jquery_column_selector = function(with_value)
	{
		const markup = this.get_markup_definition_by_type();
		if (with_value) {
			return FORMAT("{0}[column_type='{1}']", markup.body.column, with_value);
		}
		return FORMAT("{0}[column_type]", markup.body.column);
	};

	conf_view.prototype.replace_markup_by_type = function(value, type)
	{
		const markup = this.get_markup_definition_by_type();
		if (type === "column") {
			return FORMAT(value, markup.body.column);
		}
		return FORMAT(value, markup.body.row);
	};

	conf_view.prototype.total_rows_has_been_selected = function()
	{
		const $grid_body = this.$tbody();
		let selector = "{0}[column_type='row_selector'] input[type='checkbox']:checked";
		selector = this.replace_markup_by_type(selector, "column");
		const $first_selected_row = $grid_body.find(selector);
		return $first_selected_row.length;
	};

	conf_view.prototype.disabled_batch_actions = function(disabled)
	{
		const action_obj = this.dashboard.get_current_action_obj();
		let header_actions = action_obj.view.header_actions;
		if (!header_actions) {
			return;
		}
		let system_actions = header_actions.system_actions;
		if (!system_actions) {
			return;
		}
		if (!disabled) {
			const total_selected_rows  = this.total_rows_has_been_selected();
			if (total_selected_rows === 0) {
				disabled = true;
			}
			else {
				disabled = false;
			}
		}
		const $btn_menu_actions = this.$get_dom_element_by_id("view_page_header_content_grid_menu_actions");
		$btn_menu_actions.prop("disabled", disabled);
	};

	conf_view.prototype.disabled_header_actions = function(disabled = true)
	{
		const action_obj = this.dashboard.get_current_action_obj();
		let header_actions = action_obj.view.header_actions;

		if (header_actions) {
			let system_actions = header_actions.system_actions;
			if (system_actions) {
				// change grid mode
				if (system_actions.items_mode) {
					const $change_grid_mode = this.$get_dom_element_by_id("view_page_header_content_grid_mode");
					if (disabled) {
						$change_grid_mode.find(`[item_mode_id]`).prop("disabled", true);
					}
					else{
						$change_grid_mode.find(`[item_mode_id!="${system_actions.items_mode}"]`).prop("disabled", true);
					}
					
				}
				//TODO: (*) Revisar ya que los rows se elimina y se vuelven a cargar al presionar el filtro.
				//
				//// btn menu batch actions
				//this.disabled_batch_actions(disabled);
				this.disabled_batch_actions(true); // queda forzado que siempre queda disabled luego de aplciar el filtro (*)
			}
		}

		const $thead = this.$thead();
		const $select_all = $thead.find("[header_column_type='row_selector']");
		if ($select_all.length > 0) {
			$select_all.prop("checked", false);
		}
	};

	conf_view.prototype.show_total_rows = function(row_count, action_type)
	{
		const $total_rows = this.$get_dom_element_by_id("view_total_rows");
		const action_obj = this.dashboard.get_current_action_obj();

		if (action_obj && action_obj.view.header && action_obj.view.header.hide_total_rows) {
		// Ingresa aca, si la vista tiene configurado que no visualice el total de rows en el paginado (caso de las vistas que van contra 5.5)
		//  ya que de ahi no es posible obtener el total exacto
			$total_rows.hide();
			row_count = this.count_main_rows();
		}
		else {
		// Aca si visualiza el total de rows en el paginado (caso normal de las vistas de i6)
			if (typeof row_count !== "number") {
				row_count = parseInt($total_rows.attr("total"), 10);
			}
			// incremento o decremento el total de rows
			switch(action_type) {
				case "INCREMENT":
					row_count++;
					break;
				case "DECREMENT":
					row_count--;
					break;
			}
			let message = row_count === 1 ? "<%=_T('{0} total row')%>" : "<%=_T('{0} total rows')%>";
			message = "/  " + FORMAT(message, row_count);
			$total_rows
				.text(message)
				.attr("total", row_count);
		}

		const $container = this.$get_dom_element_by_id( "view_paginator_container" );
		if (row_count > 0) {
			this.$grid().show();
			this.$get_dom_element_by_id("view_not_results_found").hide();
			$total_rows.show();
			let paginator_info = this.get_paginator_info();
			$container[paginator_info.hide_container?"hide":"show"]();
			this.disabled_header_actions(false);
		}
		else {
			this.hide_total_rows();
			this.$grid().hide();

			if (action_obj && action_obj.view && action_obj.view.messages && action_obj.view.messages.nothing) {
				this.$get_dom_element_by_id("view_not_results_found").html("<span class='messageInfoText2'>" + action_obj.view.messages.nothing + "</span>").show();
			}
			else {
				this.$get_dom_element_by_id("view_not_results_found").html("<span class='messageInfoText1'><%=_T('No results found.')%></span> <span class='messageInfoText2'><%=_T('There are no items created or matching your search yet.')%></span>").show();
			}

			$container.hide();
			this.disabled_header_actions(true);
		}

		this.__update_info_selected_rows();
	};

	conf_view.prototype.hide_total_rows = function()
	{
		this.$get_dom_element_by_id("view_total_rows").hide();
		this.$get_dom_element_by_id("view_not_results_found").hide();
	};

	conf_view.prototype.start_search = function(message)
	{
		if (message !== "AUTORELOAD")
			this.dashboard.show_waiting(message || "<%=_T('Searching')%>");
		this.hide_total_rows();
		this.clean_grid();
		this.unbind_filter();
	};

	conf_view.prototype.end_search = function(row_count, total_unfiltered_rows)
	{
		let total_rows = total_unfiltered_rows || row_count;
		this.dashboard.hide_waiting();
		this.show_total_rows(total_rows);
		this.initialize_filter();
	};

	conf_view.prototype.cancel_search = function()
	{
		if ( this.current_search && this.current_search.request ) {
			this.current_search.request.abort();
		}
	};

	conf_view.prototype.err_search_messages = function(err)
	{
		const err_messages = {
			FORBIDDEN: "<%=_T('Error retrieving view information. You do not have permissions for this action.')%>"
		};

		const type = "Error";
		const default_message = "<%=_T('Server error, retrieving view information.')%>";
		let message;

		const action = this.dashboard.get_current_action_obj();
		if (action.view.callbacks && typeof action.view.callbacks.get_search_err === "function") {
			message = action.view.callbacks.get_search_err(err);
		}
		if (!message) {
			if (typeof err === "object") {
				if (typeof err.result === "string") {
					err = err.result;
				}
				else {
					err = "";
				}
				message = err_messages[err.toUpperCase()] || default_message;
			}
		}

		const timeout_ms = false; //infinite
		this.show_status_message(message, type, timeout_ms);
		this.end_search();
	};

	conf_view.prototype.search_execute = function(filter, on_search_completed = $.noop)
	{
		const self = this;

		this.cancel_search();
		const action = self.dashboard.get_current_action_obj();
		if (typeof action.view.uri !== "object") {
			//TODO: throw -- por ahora lo dejo asi porque autoattendands esta vacio
			this.end_search();
			return;
		}

		const callback = function(success, response)
		{
			if ( success ) {
				if (!self.current_search) {
					return;
				}
				self.current_search = null;

				// cope with array responses
				if ( $.isArray(response) ) {
					response = {
						rows      : response,
						last      : "",
						total_rows: response.length
					};
					response.total_unfiltered_rows = response.total_rows;
				}

				let rows = response.rows;
				if (!rows || rows.length === 0) {
					on_search_completed(success, rows, "", 0);
					return;
				}
				on_search_completed(success, rows, response.last, response.total_rows, response.total_unfiltered_rows, response.warnings);
			}
			else {
				let err;
				try {
					err = JSON.parse(response);
				}
				catch(e) {
					err = response;
				}
				on_search_completed(success);
				self.current_search = null;
				self.err_search_messages(err);
			}
		};

		this.current_search = {
			view: this.dashboard.get_current_view_id(),
			request: null
		};

		if ( typeof action.view.uri.search === "function" ) {
			this.current_search.request = action.view.uri.search(callback, filter, this);
		}
		else {
			let api = this.dashboard.get_api();
			this.current_search.request = api.search({
				uri      : action.view.uri.search,
				filter   : filter,
				callback : callback,
				request  : action.view.request
			});
		}
	};

	conf_view.prototype.get_paginator_info = function()
	{
		var info = {
			show_at_least: 12, // default
			rows_per_page: 15 // default
		};

		var action = this.dashboard.get_current_action_obj();
		if ( action && action.view.paginator ) {
			info.show_at_least = action.view.paginator.show_at_least || info.show_at_least;
			info.rows_per_page = action.view.paginator.rows_per_page || info.rows_per_page;
			info.paged_text_format = action.view.paginator.paged_text_format;
			info.hide_container = action.view.paginator.hide_container || false;
		}

		// el limite del row del request tiene que ser mayor que numero de rows por pagina. Esto es asi, porque el paginator de Juan asi lo necesita.
		info.request_limit_rows = info.rows_per_page + 1;

		return info;
	};

	conf_view.prototype.get_special_values = function()
	{
		var action = this.dashboard.get_current_action_obj();
		var filter = action.view.filter;
		if ( !filter ) {
			return;
		}
		var special_values = filter.special_values;
		if ( !special_values ) {
			return;
		} 
		return typeof( special_values ) === "function" ? special_values() : special_values;
	};

	conf_view.prototype.fill_grid = function(rows, last, total_rows, total_unfiltered_rows)
	{
		const self = this;

		if (!rows) {
			this.end_search(total_rows, total_unfiltered_rows);
			return;
		}

		this.page_start = last;
		let $grid = this.$tbody();
		let selector = "{0}[row_type='main']";
		selector = this.replace_markup_by_type(selector, "row"); // TR O UL
		let row_count = $grid.find(selector).length + 1;

		const create_row = function( row, on_row_created )
		{
			let $new_rows = self.$get_rows_html( row, row_count ); // es un array de jquery rows html
			if ( $new_rows && $new_rows.length > 0 ) {
				row_count++;
				on_row_created( $new_rows );
			}
		};

		let $initial_rows = [];
		$.each( rows, function( _, row )
		{
			create_row( row, function( $new_rows ) {
				// $new_rows es un array de jquery rows html
				$initial_rows = $initial_rows.concat( $new_rows );
			});
		});

		let has_subgrid = this.has_subgrid();
		let rows_number_per_record = has_subgrid.success ? 2 : 1; // si tiene 2 filas por registro ( el segundo es para la subgrilla )

		// paginator definition
		let action = this.dashboard.get_current_action_obj();
		let paginator_info = this.get_paginator_info();
		let $container = this.$get_dom_element_by_id( "view_paginator_container" );
		let paged_text_format = paginator_info.paged_text_format || "<%=_T('{page_row_start}-{page_row_end} of {row_count}')%>";

		if (paginator_info.hide_container) {
			$container.hide();
		}

		this.m_paginator = jQuery.Paginator.init(
		{
			pages_container        : $container,
			data_container         : $grid,
			previous               : "",
			next                   : "",
			show_at_least          : paginator_info.show_at_least,
			rows_per_page          : paginator_info.rows_per_page,
			rows_number_per_record : rows_number_per_record,
			items                  : $initial_rows,
			row_count              : total_rows,
			text_format            : paged_text_format,
			translated_many        : "<%=_T('many')%>",
			callback: function( _, need_data, on_done )
			{
				if (!need_data) {
					self.end_search(total_rows, total_unfiltered_rows);
					return on_done();
				}

				window.setTimeout(function() {
					let current_page_number = self.m_paginator.get_current_page_number() || 0;
					current_page_number++;

					let filter = {
						last           : self.page_start,
						limit          : paginator_info.request_limit_rows,
						numberPage     : current_page_number,
						query          : self.current_query,
						special_values : self.get_special_values(),
						columns        : action.view.source
					};
					self.dashboard.show_scroll_waiting( "<%=_T('Please wait, retrieving information.')%>" );
					self.search_execute(filter, function(success, rows, last)
					{
						self.dashboard.hide_scroll_waiting();

						if (!success || !rows || $.isEmptyObject( rows )) {
							self.m_paginator.no_more_pages();
							self.end_search(total_rows, total_unfiltered_rows);
							on_done();
							return;
						}

						const append_row = function( $rows ) {
							if ( !$.isArray( $rows ) || $rows.length === 0 ) {
								return;
							}
							$.each($rows, function(_, row) {
								const $row = $(row);
								self.m_paginator.append( $row );
							});
						};

						// in case elements where added or removed from the grid
						row_count = $grid.find(selector).length + 1;
						$.each(rows, function(_, row) {
							create_row(row, append_row);
						});

						self.page_start = last;
						self.end_search(total_rows, total_unfiltered_rows);

						//if is the total_rows, set no_more_pages
						
						if (self.m_paginator.get_total_page_rows() === total_rows) {
							self.m_paginator.no_more_pages();
						} 
						on_done();
					}, 100);
				});
			}
		});
	};

	conf_view.prototype.clean_grid = function()
	{
		if (this.m_paginator) {
			this.m_paginator = this.m_paginator.destroy();
		}
		this.$tbody().html("");
		this.hash_main_rows = {};
	};

	conf_view.prototype.count_main_rows = function()
	{
		let count_rows = Object.keys(this.hash_main_rows || {}).length;
		return count_rows;
	};

	conf_view.prototype.vs_get_filter_values = function()
	{
		let action = this.dashboard.get_current_action_obj();
		let filter = action.view.filter;

		let filter_values = {OR: [], AND: []};
		if (typeof filter !== "object" || typeof filter.fields !== "object") {
			return filter_values;
		}

		let vs_filter_values = this.m_visual_search_filter.get_filter_values();

		if (vs_filter_values.length === 0) {
			let filter_value = null;
			for (let field_id in filter.fields) {
				if (filter.fields.hasOwnProperty(field_id)) {
					//if (filter.fields.logical_operator === "OR") {
						filter_value = filter_value || {};
						let value = "";
						filter_value[field_id] = value;
					//}
				}
			}
			if (filter_value) {
				filter_values.OR.push(filter_value);
			}
		}
		else {
			for (let i = 0; i < vs_filter_values.length; i++) {
				let OR_filter_value = null;
				let AND_filter_value = null;
				let vs_filter_value = vs_filter_values[i];
				for (let field_id in filter.fields) {
					if (filter.fields.hasOwnProperty(field_id)) {
						let filter_definition = filter.fields[field_id];
					//	if (filter_definition.logical_operator === "OR") {
					//		OR_filter_value = OR_filter_value || {};
							let value = vs_filter_value[field_id] || null;
							if (value) {
								if (filter_definition.transform_name_value) {
									if (filter_definition.logical_operator === "OR") {
										OR_filter_value = OR_filter_value || {};
										OR_filter_value = filter_definition.transform_name_value(value, OR_filter_value);
									}
									else if (filter_definition.logical_operator === "AND") {
										AND_filter_value = AND_filter_value || {};
										AND_filter_value = filter_definition.transform_name_value(value, AND_filter_value);
									}
								}
								else {
									if (filter_definition.logical_operator === "OR") {
										OR_filter_value = OR_filter_value || {};
										OR_filter_value[field_id] = value;
									}
									else if (filter_definition.logical_operator === "AND") {
										AND_filter_value = AND_filter_value || {};
										AND_filter_value[field_id] = value;
									}
								}
							}
					//	}
					}
				}
				if (OR_filter_value) {
					filter_values.OR.push(OR_filter_value);
				}
				if (AND_filter_value) {
					filter_values.AND.push(AND_filter_value);
				}
			}
		}

		return filter_values;
	};

	/*
	Sample return values:
		filter_values = {
			"operator": "OR",
			"items": [
				{
					"Language": "es",
					"childs": {
						"operator": "AND",
						"items": [
						{
							"AgentVisible": "1"
						},
						{
							"AccountMessagesVisible": "1"
						},
						{
							"FlowDesignerVisible": "1"
						}
						]
					}
				}
			]
		};
	*/
	conf_view.prototype.tree_get_filter_values = function()
	{
		let action = this.dashboard.get_current_action_obj();
		let filter = action.view.filter;

		if (typeof filter !== "object" || typeof filter.fields !== "object") {
			return {};
		}

		if (!this.m_tree_filter) {
			return {};
		}

		if (this.m_tree_filter.is_colapsed()) {
			return {};
		}

		let template_filter_field = {
			"childs": {
				"operator": "OR",
				"items": []
			}
		};

		let template_def = {
			"operator": "OR",
			"items": []
		};

		let filter_values = inConcert.Utils.cloneObject(template_def);
		let tree_filter_values = this.m_tree_filter.get_filter_values();
		let recursirve_selected_items_fn;

		const can_include_filter_value = function(filter_definition, filter_item) {
			if (typeof filter_definition.can_include_filter_value === "function") {
				return filter_definition.can_include_filter_value(filter_item);
			}
			return filter_item.selected;
		};

		recursirve_selected_items_fn = function(current_filter, level, row_filter_values) {
			if (!current_filter) {
				return;
			}
			(current_filter.childs || []).forEach((filter_item, i) => {
				let filter_field;
				let filter_definition = filter.fields[filter_item.entity_id];
				if (can_include_filter_value(filter_definition, filter_item)) {
					filter_field = inConcert.Utils.cloneObject(template_filter_field);
					if ((filter_item.childs || []).length === 0) {
						delete filter_field.childs;
					}
					if (filter_definition && filter_definition.transform_name_value) {
						filter_field = filter_definition.transform_name_value(filter_item, filter_field);
					}
					else {
						filter_field[filter_item.entity_id] = filter_item.value;
					}

					if (filter_definition.logical_operator) {
						row_filter_values.operator = filter_definition.logical_operator;
					}

					recursirve_selected_items_fn(filter_item, level + 1, filter_field.childs);
					row_filter_values.items.push(inConcert.Utils.cloneObject(filter_field));
				}
			});
		};
		recursirve_selected_items_fn(tree_filter_values, 0, filter_values);

		return filter_values;
	};

	conf_view.prototype.get_filter_values = function(query_value = "")
	{
		const self = this;

		let action = this.dashboard.get_current_action_obj();
		let filter = action.view.filter;

		if (typeof filter !== "object" || typeof filter.fields !== "object") {
			return {};
		}

		let tree_filter_values;
		if (this.m_used_tree_filter) {
			tree_filter_values = this.tree_get_filter_values();
		}

		if (this.m_used_filter_visual_search) {
			let vs_filter_values = this.vs_get_filter_values();
			let filter = {
				tree: tree_filter_values,
				AND : vs_filter_values.AND,
				OR  : vs_filter_values.OR
			};
			return filter;
		}

		// obtiene el valor del campo del filtro.
		const get_filter_value = function(field_id, field) {
			if (!field.control) {
			// si no tiene un control custom definido, retorna el valor del textbox principal
				return query_value;
			}
			const $control = self.$get_dom_element_by_id("filter_custom_" + field_id);
			if (!$control) {
				return query_value;
			}
			if (typeof field.get_value === "function") {
			// si se ha definido la funcion, retorna el valor personalizado
				return field.get_value($control);
			}
			//retorna los valores por defecto segun el control.
			if (field.control.toLowerCase() === "checkbox") {
			// retorna el valor del checkbox
				if ($control.prop("checked")) {
					return "1";
				}
				else {
					return "0";
				}
			}
			return query_value;
		};

		let filter_values = {};
		$.each(filter.fields, function(field_id, field) {
			filter_values[field_id] = get_filter_value(field_id, field);
		});

		return filter_values;
	};

	conf_view.prototype.search = function(query_value, message, callback = $.noop)
	{
		const self = this;

		if (message !== "AUTORELOAD") {
			this.start_search(message);
		}
		this.current_query = this.get_filter_values(query_value);
		this.page_start = "";
		const action = this.dashboard.get_current_action_obj();
		const paginator_info = this.get_paginator_info();

		const filter = {
			last          : "",
			limit         : paginator_info.request_limit_rows,
			numberPage    : 1,
			query         : this.current_query,
			special_values: this.get_special_values(),
			columns       : action.view.source
		};

		this.search_execute(filter, function(success, rows, last, total_rows, total_unfiltered_rows, warnings) {
			if (success) {
				if (message === "AUTORELOAD") {
					self.start_search(message);
				}
				self.fill_grid(rows, last, total_rows, total_unfiltered_rows);
			}
			callback(success);
			if (action.view.on_search) {
				action.view.on_search(self, success, rows, warnings);
			}
		});
	};

	conf_view.prototype.refresh = function(message, callback = $.noop)
	{
		message = message || "<%=_T('Refresh')%>";
		this.search( undefined, message, callback );
	};

	conf_view.prototype.$get_header_columns = function( columns )
	{
		const self = this;
		if ( !$.isArray( columns ) || columns.length === 0 ) {
			return "";
		}
		// "<tr></tr>" o "<ul></ul>"
		const markup = this.get_markup_definition_by_type();
		let columns_markup = FORMAT("<{0}></{0}>", markup.head.row);
		let $columns = $(columns_markup);

		$.each( columns, function( _, column )
		{
			if ( column.colspan ) {
				return true; // break
			}

			// El tooltip
			let tooltip_attr_header = "";
			if ( column.get_tooltip_header ) {
				tooltip_attr_header = column.get_tooltip_header;
			}
			tooltip_attr_header = FORMAT(" title='{0}' ", tooltip_attr_header);

			let column_name = column.name || "";
			let class_name;
			if ( column.system_action || column.custom_action ) {
				class_name = (column.class_name || "gridColumnActions");
			}
			else {
				class_name = (column.class_name || "");
			}

			let header_column_html;
			let col_type = column.type || "";
			if (col_type === "row_selector") {
				self.m_has_row_selector = true;
				let row_selector_class_name = "formCheckbox" + (class_name ? " " + class_name : "");
				header_column_html = "<{0} "+ tooltip_attr_header +">" + "<input class='" + row_selector_class_name + "' type='checkbox' header_column_type='" + col_type + "' />" + "</{0}>";
			}
			else {
				header_column_html = "<{0}" + " class='" + class_name + "' "+ tooltip_attr_header +"><span header_column_type='" + col_type + "'>" + column_name + "</span>" + "</{0}>";
			}
			// reemplazo los {0} por th o div
			header_column_html = FORMAT(header_column_html, markup.head.column);

			$columns.append($(header_column_html));
		});

		return $columns;
	};

	conf_view.prototype.__is_configured_action = function(action_id)
	{
		var action = this.dashboard.get_current_action_obj();
		var columns = action.view.columns;

		if ( !$.isArray( columns ) || columns.length === 0 ) {
			return false;
		}

		var is_configured = false;

		$.each( columns, function( _, column )
		{
			if (column.id === "_action") {
				if (column.system_action && column.system_action[action_id]) {
					is_configured = true;
					return false; // break
				}
			}
		});

		return is_configured;
	};

	conf_view.prototype.create_header_columns = function( columns )
	{
		var $columns = this.$get_header_columns( columns );
		this.$thead().append($columns);
		return true;
	};

	conf_view.prototype.new_form = function(action)
	{
		if (!action.form) {
			return;
		}
		var params = {
			action: action,
			dashboard: this.dashboard
		};
		return new action.form.instance(params);
	};

	conf_view.prototype.scroll_to = function(destination, callback)
	{
		var self = this;
		callback = callback || function() {self.enable_paginated(true);};

		this.enable_paginated(false);
		var $scroll = this.$grid_scroll || this.$get_dom_element_by_id("grid_scroll");

		var animation = {scrollTop: 0};

		switch(destination) {
			case "top":
				animation.scrollTop = 0;
			break;

			case "bottom":
				var scroll_height = $scroll.prop("scrollHeight");
				var height = $scroll.height();
				animation.scrollTop = scroll_height - height;
			break;

			case "stored_position":
				animation.scrollTop = this.stored_scroll_pos;
			break;

			default:
				throw("scroll error");
		}

		var time_ms = 250;
		$scroll.animate(animation, time_ms, "linear",
			function() {
				callback();
				self.enable_paginated(true);
			});
	};

	conf_view.prototype.on_scroll = function()
	{
		this.form.on_scroll();
	};

	conf_view.prototype.update_row_source_data = function(entity, $current_row)
	{
		let current_entity = $current_row.data("row_data") || {};
		current_entity = $.extend(true, current_entity, entity);
		$current_row.data("row_data", current_entity);
		return current_entity;
	};

	conf_view.prototype.update_row = function(entity, $current_row, not_show_grid_message)
	{
		const self = this;

		const action = this.dashboard.get_current_action_obj();
		let value = entity[action.view.display_name];

		var show_message = function() {
			if (not_show_grid_message) {
				return;
			}
			const template_messages = {
				is_new   : "<%=_T('{0} was successfully added.')%>",
				is_update: "<%=_T('{0} was successfully updated.')%>"
			};

			let message, template;
			if (action.view.messages) {
				if (entity.isNew && action.view.messages.created) {
					if (typeof(action.view.messages.created) === "function") {
						message = action.view.messages.created(value);
					}
					else {
						template = action.view.messages.created;
					}
				}
				else if (!entity.isNew && action.view.messages.updated) {
					if (typeof(action.view.messages.updated) === "function") {
						message = action.view.messages.updated(value);
					}
					else {
						template = action.view.messages.updated;
					}
				}
			}

			if (!message) {
				template = template || (entity.isNew ? template_messages.is_new : template_messages.is_update);
				message = FORMAT(template, value);
			}

			self.show_status_message(message, "Info");
		};

		if (!$current_row) {
		// don't add new rows, reload the list instead
			const $txt_search = this.$get_dom_element_by_id("filter_text");
			this.search($txt_search.val(), function() {
				if (value !== undefined) {
					window.setTimeout(function() {
						show_message();
					}, 500);
				}
			});
		}
		else {
		// update row
			//Muevo el scroll a la posicion almacenada (la ubicacion donde se encuentra el registro que estamos editando)
			//  esto se realiza asi, porque el scroll es el mismo para la grilla que para el formulario, entonces
			//  si tenemos un frm muy largo en cual tenemos que scrollear hasta el final para presionar el boton "update"
			//  cuando regresamos a la grilla, es probable que la row que estamos editando, no la estemos visualizando.
			//  Ergo, con esto nos posicionamos donde estaba antes de editar la misma, y asi se puede divisar el cambio.

			const has_subgrid = this.has_subgrid();
			//const $grid = this.$get_dom_element_by_id("view_grid tbody");
			const $grid = this.$tbody();

			this.scroll_to("stored_position", function()
			{
				const row_classes = self.get_row_classes( entity, action, $current_row );
				$current_row.removeClass().addClass( row_classes );

				// realizo un fadeOut, para ocultar el row con los datos viejos y luego realizo un fadeIn con los datos nuevos.
				$current_row.fadeOut("slow", function()
				{
					// actualizo el source data de la row
					entity = self.update_row_source_data( entity, $current_row );
					// aca actualizo cada celda de la row.
					let row_number;
					let selector = "{0}[column_id='_number']";
					selector = self.replace_markup_by_type(selector, "column");
					const $col_number = $current_row.find(selector);
					if ( $col_number.length > 0 ) {
						row_number = parseInt( $col_number.text(), 10 );
					}
					selector = "{0}";
					selector = self.replace_markup_by_type(selector, "column"); // TD O LI
					$current_row.find(selector).remove();
					self.$fill_row_columns_content( $current_row, entity, action.view.columns, undefined, row_number );
					// aca realizo el fade in
					$current_row.fadeIn("slow");
					// actualizo la los valores de la subgrilla (la construyo de cero)
					if (has_subgrid.success) {
						const row_id = $current_row.attr("row_id");
						const $sub_grid = $grid.find("[row_id='" + row_id + "'][row_type='subgrid']");
						if ($sub_grid.length > 0) {
							$sub_grid.html("");
							const sub_rows = entity[action.view.sub_grid.data_source];
							const columns = action.view.columns;
							const colspan = columns.length - 1;
							self.make_sub_grid(sub_rows, $sub_grid, colspan);
						}
					}
				});
			});
			if (value !== undefined) {
				$current_row.attr("row_name", value);
				show_message();
			}
		}
	};

	conf_view.prototype.update_view = function(entity, $current_row, not_show_grid_message)
	{
		var action = this.dashboard.get_current_action_obj();
		if (typeof action.view.on_update_view === "function") {
			action.view.on_update_view(entity, $current_row);
		}
		this.update_row(entity, $current_row, not_show_grid_message);
	};

	conf_view.prototype.get_filter_fields_by_type = function(filter_fields, type)
	{
		let filter_fields_by_type = {};
		for (let key in filter_fields) {
			if (filter_fields.hasOwnProperty(key)) {
				let filter_field = filter_fields[key];
				if ((filter_field.type || "").toUpperCase() === type.toUpperCase()) {
					filter_fields_by_type[key] = inConcert.Utils.cloneObject(filter_field);
				}
			}
		}
		return filter_fields_by_type;
	};

	conf_view.prototype.vs_make_filter_fields = function(filter_fields)
	{
		let vs_make_filter_fields = this.get_filter_fields_by_type(filter_fields, "VISUAL-SEARCH");
		return vs_make_filter_fields;
	};

	conf_view.prototype.vs_init = function($filter_container, filter)
	{
		const self = this;
		this.m_used_filter_visual_search = true;
		let vs_filter_fields = this.vs_make_filter_fields(filter.fields);

		this.m_visual_search_filter = new inConcert.VisualSearchMng({
			owner        : this,
			auto_init    : true,
			$container   : $filter_container,
			filter_fields: vs_filter_fields,
			not_duplicate_filters : filter.not_duplicate_filters,
			callbacks : {
				on_search: function() {
					self.vs_search();
				},
				on_changed_filter(filter_values) {
					self.vs_changed_filter(filter_values);
					if (typeof filter.on_changed_filter === "function") {
						filter.on_changed_filter();
					}
				}
			}
		});
	};

	/*
		La ejecucion de la consulta desde VisualSearch se debe de realizar en un timeout, ya que sino al agregar un filtro y perder el foco con el mouse (click sobre el boton para ejecutar la consulta)
			el control de VisualSearch no culmina de actualizar los datos del filtro y la ultima condicion agregada no se obtiene y por ende no se filtra por esta ultima condicion.
	*/
	conf_view.prototype.vs_search = function()
	{
		const self = this;
		if (this.m_vs_search_timeout_hnd) {
			window.clearTimeout(this.m_vs_search_timeout_hnd);
			this.m_vs_search_timeout_hnd = null;
		}
		this.m_vs_search_timeout_hnd = window.setTimeout(function() {
			self.search();
		}, 200);
	};

	conf_view.prototype.vs_changed_filter = function(filter_values)
	{
		// if (!this.m_tree_filter) {
		// 	return;
		// }
		// this.m_tree_filter.mark_selected_items(filter_values, true);
	};

	conf_view.prototype.vs_destroy = function()
	{
		if (!this.m_visual_search_filter) {
			return;
		}
		this.m_visual_search_filter.dispose();
		this.m_visual_search_filter = null;
		this.m_used_filter_visual_search = false;
		this.m_used_tree_filter = false;
	};

	conf_view.prototype.tree_filter_init = function(tree_filter_info) //, filter_fields)
	{
		const self = this;

		if (!tree_filter_info || !tree_filter_info.enabled) {
			return;
		}

		const $tree_filter_container = tree_filter_info.conteiner_id ? $("#" + tree_filter_info.conteiner_id) : this.$get_dom_element_by_id("view_page_tree_filter_content");
		const $parent_container = this.$get_dom_element_by_id("view_page_content");

		this.m_used_tree_filter = true;
		this.m_tree_filter = new inConcert.conf_filter_tree({
			owner            : this,
			auto_init        : true,
			$container       : $tree_filter_container,
			$parent_container: $parent_container,
			initial_state    : tree_filter_info.initial_state,
			title            : tree_filter_info.title,
			tree             : tree_filter_info.get_tree(),
			callbacks : {
				on_select_option: function(selected, item) {
					self.tree_filter_on_select_option(selected, item);
				},
				on_colapsed_uncolapsed: function(action) {
					self.tree_filter_on_colapsed_uncolapsed(action);
				}
			}
		});
	};

	conf_view.prototype.tree_filter_search = function()
	{
		this.search();
	};

	conf_view.prototype.tree_filter_on_colapsed_uncolapsed = function(/*action*/)
	{
		this.tree_filter_search();
	};

	conf_view.prototype.tree_filter_on_select_option = function(selected, item)
	{
		if (!this.m_visual_search_filter) {
			return;
		}

		this.tree_filter_search();

		// let tree_filter_selected_items = this.m_tree_filter.get_selected_items();
		// let selected_items = [];
		// tree_filter_selected_items.forEach(item => {
		// 	selected_items.push({name: item.entity_id, value: item.id});
		// });
		// this.m_visual_search_filter.set_filter_values(selected_items, true);
	};

	conf_view.prototype.tree_filter_destroy = function()
	{
		if (!this.m_tree_filter) {
			return;
		}
		this.m_tree_filter.dispose();
		this.m_tree_filter = null;
	};

	conf_view.prototype.initialize_filter = function(view)
	{
		const self = this;
		const $header_filter_container = this.$get_dom_element_by_id("filter_container");

		if (view) {
			if (!view.filter) {
				$header_filter_container.attr("style", "display: none !important");
			}
			else {
				const filter_type = view.filter.type;
				if (filter_type) {
					let search_info = filter_type.search;
					if (search_info.id === "visual-search") {
						let conteiner_destination = search_info.conteiner_destination ? search_info.conteiner_destination.type : null;
						let $filter_container;
						if (conteiner_destination === "body") {
							$header_filter_container.hide();
							$filter_container = this.$get_dom_element_by_id("view_page_header_filter_container");
						}
						else {
							$filter_container = $header_filter_container;
						}
						this.vs_init($filter_container, view.filter);
						this.tree_filter_init(filter_type.tree_filter); //, view.filter.fields);
						return;
					}
				}
				// let conteiner_destination = view.filter.conteiner_destination ? view.filter.conteiner_destination.type : null;
				// if (view.filter.type === "visual-search") {
				// 	let $filter_container;
				// 	if (conteiner_destination === "body") {
				// 		$header_filter_container.hide();
				// 		$filter_container = this.$get_dom_element_by_id("view_page_header_filter_container");
				// 	}
				// 	else {
				// 		$filter_container = $header_filter_container;
				// 	}
				// 	this.vs_init($filter_container, view.filter);
				// 	return;
				// }
				// else {
				// 	// if (view.filter.conteiner_destination && view.filter.conteiner_destination.body === "body") {
						
				// 	// }
				// }
				var $filter_container_custom = this.$get_dom_element_by_id("filter_container_custom")
					.html('')
					.hide();
				if (view.filter.fields) {
				// se recorre la lista de campos a filtrar, y si tienen definicion asociada, agrega el control correspondiente.
					$.each(view.filter.fields, function(field_id, field) {
						if (!field.control) {
							return true; // continue
						}
						var element_id = "conf_filter_custom_"  + field_id;
						var name = field.name || field_id;
						var default_value = field.default_value || "";

						var $input_field;
						var label_position = "before";
						let classes_filter_parent = "filterData";
						switch(field.control) {
							case "checkbox":
								label_position = "after";
								$input_field = $('<input type="checkbox" class="filterCheckbox" id="' + element_id + '" ' + default_value + '/>');
								break;
							case "switchbox":
								label_position = "";
								classes_filter_parent = "form-check form-switch";
								$input_field = $(`
									<label for="${element_id}" class="form-check-label form-check-label-title">
										<input id="${element_id}" class="form-check-input" type="checkbox" ${default_value} />
										<span></span>
										${name}
									</label>
								`);
								break;
							case "select":
								$input_field = $('<select class="filterCombo" id="' + element_id + '"></select>');
								field.options.forEach( function(option) {
									$input_field.append($(
										'<option value="' + option.value + '"' + (option.value === default_value ? "selected" : "") + '>' +
											option.label +
										'</option>'));
								});
								break;
							default:
								$input_field = $('<input type="text" class="filterField" id="' + element_id + '"/>');
								break;
						}

						
						var $container = $(`<div class="${classes_filter_parent}"></div>`);
						var $label = $('<label for="' + element_id + '" class="filterLabel">' + name + '</label>');
						
						$container.append($input_field);
						switch (label_position) {
							case "before":
								$input_field.before.call($input_field, $label);
								break;
							case "after":
								$input_field.after.call($input_field, $label);
								break;
							default:
								break;
						}

						$filter_container_custom.append($container);
					});
				}
				if (!$filter_container_custom.is(":empty")) {
					$filter_container_custom.show();
				}
				$header_filter_container.show();
			}
		}

		var $txt_search = this.$get_dom_element_by_id("filter_text");
		var trigger_search = function() {
			var text = $txt_search.val();
			self.search(text);
		};

		$txt_search
			.prop("disabled", false)
			.focus()
			.off("keyup")
			.on("keyup", function(event) {
				// If user press ctrl/tab/command/arrows don't do anything
				var filtered_control_keys = inConcert.Utils.getControlCharacters();
				if (filtered_control_keys[event.keyCode]) {
					return;
				}
				// if user press Enter trigger the search inmediatly
				if (event.keyCode === 13) {
					trigger_search();
					return;
				}
			});

		var $btn_search = this.$get_dom_element_by_id("filter_button");
		$btn_search
			.prop("disabled", false)
			.off("click")
			.on("click", function() {
				trigger_search();
			});

		var $btn_reset = this.$get_dom_element_by_id("filter_reset");
		$btn_reset
			.off("click")
			.on("click", function() {
				$txt_search.val("");
				trigger_search();
			});

		$header_filter_container.removeClass("twoButtonsFilter");
		var filter_value = $txt_search.val();
		if (typeof filter_value === "undefined") {
			return;
		}
		var filtered = filter_value.trim().length > 0;
		if (filtered) {
			$btn_reset.prop("disabled", false);
			$btn_reset.show();
			$header_filter_container.addClass("twoButtonsFilter");
		}
		else {
			$btn_reset.prop("disabled", false);
			$btn_reset.hide();
		}
	};

	conf_view.prototype.unbind_filter = function()
	{
		this.$get_dom_element_by_id("filter_text")
			.unbind("keyup")
			.prop("disabled", true);

		this.$get_dom_element_by_id("filter_button")
			.unbind("click")
			.prop("disabled", true);

		this.$get_dom_element_by_id("filter_reset")
			.unbind("click")
			.prop("disabled", true);
	};

	conf_view.prototype.on_show = function()
	{
		this.dashboard.on_view_ready();
		this.form.events_unsuscribe();
		this.toggle_split_screen("SHOW-NAVIGATION");
		this.initial_full_screen();
		this.set_in_view(true);
		this.enable_auto_reload();
	};

	conf_view.prototype.on_hide = $.noop;

	conf_view.prototype.initial_full_screen = function()
	{
		const current_action = this.dashboard.get_current_action_obj() || {};
		const must_init_full_screen = (current_action?.view?.must_init_full_screen) || function() {
			return false;
		};
		inConcert.app.toggleConfNavigation(!must_init_full_screen(this), this.$get_dom_element_by_id("grid_scroll"), this.$get_dom_element_by_id("navigation"));
	};

	conf_view.prototype.set_split_info = function($button, $row)
	{
		if (this.m_currentSplitInfo) {
			this.m_currentSplitInfo.$button.prop("disabled", false);
			this.unmark_row(this.m_currentSplitInfo.$row);
		}
		if (!$button || !$row) {
			this.m_currentSplitInfo = null;
			return;
		}
		$button.prop("disabled", true);
		this.mark_row($row);
		this.m_currentSplitInfo = {
			$button: $button,
			$row   : $row
		};
	};

	conf_view.prototype.destroy_split_info = function()
	{
		this.set_split_info(null, null);
	};

	// grid buttons actions ---------------
	conf_view.prototype.grid_execute_action = function(action_id, $row, extra_parameters, $button)
	{
		const self = this;
		if (typeof action_id !== "string") {
			return;
		}

		let action = this.dashboard.get_current_action_obj();

		var has_permission_to_create = function()
		{
			var permission = { enabled: true };

			permission.enabled = self.has_permission_to_navigation();
			if ( !permission.enabled ) {
				permission.type = "view";
				return permission;
			}

			// a esa validacion, hoy lo utiliza solo web designer ya que es una app externa y no tengo como controlar
			if ( action.permissions.save && action.permissions.verify_edit_permission ) {
				var no_log = true;
				permission.enabled = self.dashboard.permissions.to_action( self.dashboard.current_app, action.permissions.save, "SAVE", undefined, no_log );
				permission.type = "edit";
			}

			return permission;
		};

		action_id = action_id.toLowerCase();
		switch(action_id)
		{
			case "split_screen":
			// system action
				if ($row.find( ".gridActionButton[action='split_screen']" ).length === 0) {
					return;
				}
				extra_parameters = extra_parameters || {};
				extra_parameters.split_screen = true;
				this.set_split_info($button, $row);
				this.toggle_split_screen("HIDE-NAVIGATION", $button);
				/*
					ATENCION: En la accion "split_screen" NO Existe ni debe existin NUNCA un "break" y tiene que estar antes de la accion "edit",
						ya que se realiza el split de la misma (se oculta la navegacion, y luego se realiza un edit del frm)
				*/

			/* falls through */ // para que desactivar validacion de jshint -> Expected a 'break' statement before 'case'.
			case "edit":
			// system action

				// El caso del edit se puede ejecutar al realizar dblclick sobre la row, entonces tengo que validar si el mismo existe en la configuracion.
				if ("split_screen" !== action_id) {
					if (!this.__is_configured_action("edit")) {
						return;
					}
				}

				var permission = has_permission_to_create();
				if ( !permission.enabled ) {
					var name = ( $row.attr( "row_name" ) || "" ).toUpperCase();
					var default_message = FORMAT(
						(
							permission.type === "view" ?
							"<%=_T('Unable to view {0}: You dont have permission for this action.')%>" :
							"<%=_T('Unable to edit {0}: You dont have permission for this action.')%>"
						),
						name
					);
					var view = action.view;
					var message;
					if ( typeof view.messages.err === "function" ) {
						var error = { result: "FORBIDDEN" };
						message = view.messages.err(permission.type, error, name, self) ;
						if (typeof(message) === "object" && message.can_show_popup_message) {
							self.show_popup_message({
								messages: {
									title  : message.title || "",
									message: message.message,
									detail : message.detail || "",
									type   : message.msg_type || "Error"
								}
							});
							return;
						}
					}
					message = message || default_message;
					this.show_status_message( message, "Error", 5000 );
					return;
				}

				var $edit = $row.find( ".gridActionButton[action='edit']" );
				if ( $edit === 0 ) {
					return;
				}
				if ( $edit.prop( "disabled" ) ) {
					return;
				}

				this.render_form( $row, extra_parameters );
				return;

			case "delete":
			// system action
				if ( $row.find( ".gridActionButton[action='delete']" ).length === 0 ) {
					return;
				}
				this.erase( action, $row );
				return;

			case "colapse/uncolapse":
			// system action
				var colapsed = $row.prop( "colapsed-sub-grid" );
				$row.prop( "colapsed-sub-grid", !colapsed );
				if ( colapsed ) {
					$row.removeClass( "colapsed" ).addClass( "uncolapsed" );
				}
				else {
					$row.removeClass( "uncolapsed" ).addClass( "colapsed" );
				}
				var $subgrid_row = this.$get_subgrid_row( $row );
				if ( $subgrid_row.length > 0 ) {
					$subgrid_row.prop( "colapsed", !colapsed );
					$subgrid_row[ colapsed ? "show" : "hide" ]();
				}
				return;

			default:
			// custom actions
				if (typeof this.grid_custom_action === "object") {
					var custom_action = this.grid_custom_action[action_id];
					if (typeof custom_action === "object" && typeof custom_action.execute === "function") {
						custom_action.execute(action_id, $row, this, $button);
						return;
					}
				}
		}
		throw("Configuration view action not defined " + action_id);
	};

	conf_view.prototype.render_popup = function( params )
	{
		this.form.popup_render( params );
	};

	conf_view.prototype.events_suscribe = function()
	{
		var self = this;
		var action = this.dashboard.get_current_action_obj();
		if ( typeof action.view.realtime !== "object" ) {
			return;
		}
		var app_id = this.dashboard.current_app;
		var realtime_config = action.view.realtime;
		$.each( realtime_config || {}, function( event_id, config ) {
			self.dashboard.realtime.event_suscribe( app_id, event_id, config );
		});
	};

	conf_view.prototype.events_unsuscribe = function()
	{
		var self = this;
		var action = this.dashboard.get_current_action_obj();
		if ( typeof action.view.realtime !== "object" ) {
			return;
		}
		var app_id = this.dashboard.current_app;
		var realtime_config = action.view.realtime;
		$.each( realtime_config || {}, function( event_id ) {
			self.dashboard.realtime.event_unsuscribe( app_id, event_id );
		});
		this.form.events_unsuscribe();
	};

	conf_view.prototype.render_form = function( $row, extra_parameters )
	{
		this.form.render( $row, extra_parameters );
	};

	conf_view.prototype.execute_batch_action = function(action_id, callback = $.noop)
	{
		this.form.execute_batch_action(action_id, callback);
	};

	conf_view.prototype.can_show_contextual_batch_menu = function() {
		let selected_grid_items = this.get_selected_items();
		if (!selected_grid_items || selected_grid_items.length < 1) {
			return false;
		}
		return true;
	};

	conf_view.prototype.toggle_contextual_menu = function($parent, menu_options, can_show_contextual_menu, callback, menu_config) {
		if (this.m_contextual_menu && this.m_contextual_menu.isVisible()) {
			this.destroy_contextual_menu();
		}
		this.show_contextual_menu($parent, menu_options, can_show_contextual_menu, callback, menu_config);
	};

	conf_view.prototype.destroy_contextual_menu = function() {
		const self = this;
		if (!this.m_contextual_menu) {
			return;
		}
		this.m_contextual_menu.finalize(function() {
			self.m_contextual_menu = null;
		});
	};

	conf_view.prototype.can_show_contextual_menu = function(can_show_contextual_menu) {
		can_show_contextual_menu = can_show_contextual_menu || function() {return true;};
		return can_show_contextual_menu();
	};

	conf_view.prototype.show_contextual_menu = function($parent, menu_options, can_show_contextual_menu, callback, menu_config) {
		const self = this;
		callback = callback = $.noop;

		if (!this.can_show_contextual_menu(can_show_contextual_menu)) {
			callback();
			return;
		}

		const menu_id = this.dashboard.prefix_id + "view_contextual_menu_actions";
		$('#' + menu_id).remove();

		let menu_classes = "interactionMenuBlind" + (menu_config && menu_config.classes? " " + menu_config.classes:"");
		const $menu = $(''.concat(
			`<div id="${menu_id}" class="${menu_classes}" hidden></div>`
		));

		$.each(menu_options, function(_, option) {
			const $option = $("".concat(
				"<div option='" + option.Id + "' class='interactionMenuItem " + option.Id + "'>",
					option.name,
				"</div>"
			));
			$option.click(function() {
				window.setTimeout(function() {
					option.execute($menu);
					self.destroy_contextual_menu();
				}, 50);
			});
			$menu.append($option);
		});

		$('body').append($menu);
		this.m_contextual_menu = new inConcert.PopupWindow({ownerId: menu_id});
		this.m_contextual_menu.show(
		{
			ownerId       : menu_id,
			eventName     : menu_id,
			canReposition : true,
			container     : $menu,
			parentNode    : $parent,
			callbacks     : 
			{
				onClose   : function() {
					$menu.hide();
					self.m_contextual_menu = null;
					delete self.m_contextual_menu;
				},
			},
//			align         : "LEFT"
		});
		callback();
	};

	conf_view.prototype.get_batch_menu_options = function() {
		return this.form.get_batch_menu_options();
	};

	conf_view.prototype.get_new_menu_options = function(callback = $.noop) {
		return this.form.get_new_menu_options(callback);
	};

	conf_view.prototype.set_events = function(action)
	{
		const self = this;

		var has_permission_to_create = function() {
			var no_log = true;
			return self.dashboard.permissions.to_action(self.dashboard.current_app, action.permissions.save, "SAVE", undefined, no_log);
		};

		var can_show_new_button = function() {
			if ( action.view.header ) {
				if ( action.view.header.hide_new_button ) {
					return false;
				}
			}
			return has_permission_to_create();
		};

		var $conf_view_button_new = this.$get_dom_element_by_id("view_button_new");
		// check permission to save
		if (can_show_new_button()) {
			// new button
			$conf_view_button_new.show()
				.off("click")
				.on("click", function() {
					const $this = $(this);
					if ( !has_permission_to_create() ) {
						var name = ($this.attr("value" ) || "" ).toLowerCase();
						var error_message = FORMAT( "<%=_T('Unable to create {0}: You dont have permission for this action.')%>", name );
						self.show_status_message( error_message, "Error", 5000 );
						return;
					}
					if (action.view.new_action && action.view.new_action.with_contextual_menu) {
						// menu contextual.
						const menu_options = self.get_new_menu_options(function(option_id, info) {
							self.render_form(undefined, {action_id: option_id, info: info});
						});
						self.toggle_contextual_menu($this, menu_options, undefined, undefined, {classes: "NewButton"});
						return;
					}
					// ejecucion por defecto, renderiza el frm.
					self.render_form();
				});
			var button_name = action.view.new_button_name;
			if (button_name) {
				$conf_view_button_new.attr("value", button_name);
				$conf_view_button_new.html(`<i class="fal fa-plus"></i>${button_name}`);
				
			}
		}
		else {
			$conf_view_button_new.hide();
		}

		//extra buttons
		$conf_view_button_new.siblings("[custom_button]").remove();
		$.each(action.view.global_buttons || [], function(_, button) {
			if (button.enabled && !button.enabled()) {
				return;
			}
			var $button = $conf_view_button_new.clone();
			if (button.className) {
				$button
					.removeClass("buttonColorOrange")
					.addClass(button.className);
			}
			$button
				.attr("id", "view_button_" + button.id)
				// .attr("value", button.name)
				.html(button.name)
				.prop("custom_button", true)
				.addClass("pageButton" + button.id.to_camel())
				.show()
				.off("click")
				.on("click", function() {
					button.run(self);
				});
			if (button.before) {
				$conf_view_button_new.before($button);
			}
			else {
				$conf_view_button_new.after($button);
			}
		});

		if (action.view.header_actions) {
			// system actions
			let system_actions = action.view.header_actions.system_actions;
			if (system_actions) {
				// btn menu batch actions
				if (system_actions.batch_actions) {
					const $btn_menu_actions = this.$get_dom_element_by_id("view_page_header_content_grid_menu_actions");
					$btn_menu_actions.off("click")
						.on("click", function() {
							const $owner = $(this);
							const menu_options = self.get_batch_menu_options();
							self.toggle_contextual_menu($owner, menu_options, function() {
								return self.can_show_contextual_batch_menu();
							});
						}
					);
				}
			}
		}

		const grid_execute_action = function(action_id, $row, $button) {
			self.stored_scroll_pos = self.$grid_scroll.scrollTop();
			self.grid_execute_action(action_id, $row, null, $button);
		};

		// ATENCION: aca se setea el evento "click" que tambien funciona como "dblclick"
		//  se calcula los click mediante un timer, y se determina asi si se quiso dar "click" o "dblclick"
		//  Esto se hizo asi, porque si se bindean los eventos por separado "click" y "dblclick", siempre se ejecutan los 2 al realizar "dblclick"
		//    y si solo queremos dar "dblclick" primero se nos va a ejecutar el "click" y luego el "dblclick".
		//
		//  1 - "click" sobre la row colapsa/uncolapasa la subgrilla.
		//  2 - "dblclick" sobre la row edita el registro.
		//  
		var has_subgrid = this.has_subgrid();
		var timeout_click;
		var clicks = 0;

		let $tbody = this.$tbody();
		let event_selector = this.__jquery_row_selector();
		event_selector = event_selector + "[row_type='main']";

		$tbody
			.off("click", event_selector)
			.on("click", event_selector, function( event ) {
				event.stopPropagation();
				clicks++;
				var $row = $( this );
				var timeout_ms = 300;
				if ( !timeout_click ) {
					timeout_click = window.setTimeout( function()
					{
						if ( clicks === 2 ) {
							var has_navigation_permission = self.has_permission_to_navigation();
							if ( !has_navigation_permission ) {
								return;
							}
							grid_execute_action( "edit", $row );
						}
						else {
							if ( has_subgrid.success ) {
								grid_execute_action( "colapse/uncolapse", $row );
							}
						}
						window.clearTimeout( timeout_click );
						timeout_click = undefined;
						clicks = 0;
					}, timeout_ms );
				}
			});

		// row actions
		event_selector = "button[action]";
		$tbody
			.off("click", event_selector)
			.on("click", event_selector, function( event ) {
				event.stopPropagation();
				var $button = $(this);
				var $row = $button.parent().parent();
				var action_id = $button.attr("action");
				grid_execute_action(action_id, $row, $button);
			});

		// grid scroll event
		var $grid_scroll = this.$get_dom_element_by_id("grid_scroll");
		$grid_scroll
			.scroll(function(event) {
				event.stopPropagation();
				self.on_scroll();
			});

		// celdas selectors de cada row (ej: tr[row_id][row_type='main'] td[column_type='row_selector'] [type='checkbox'])
		let main_event_selector = this.__jquery_row_selector();
		main_event_selector = main_event_selector + "[row_type='main']";
		let column_selector = this.__jquery_column_selector("row_selector");
		event_selector = main_event_selector + " " + column_selector + " [type='checkbox']";

		$tbody
			.off("click", event_selector)
			.on("click", event_selector, function() {
				self.__toggle_checkbox_select_all_rows();
				const $chk = $(this);
				self.disabled_batch_actions();
				const is_selected = $chk.prop("checked");
				self.on_selected_row($chk, is_selected);
			});

		// header
		const $thead = this.$thead();
		let col_event_selector = "[header_column_type]";
		// click sobre las columnas del header
		$thead
			.off("click", col_event_selector)
			.on("click", col_event_selector, function(e) {
				e.stopPropagation();
				const $this = $(this);
				const header_column_type = $this.attr("header_column_type");
				if (header_column_type === "row_selector") {
				// click sobre el boton select all
					const is_selected = $this.prop("checked");
					self.__toggle_select_rows(is_selected);
					self.disabled_batch_actions();
				}
			});
	};

	conf_view.prototype.on_selected_row = function($chk, is_selected)
	{
		const column_id = $chk.parent().attr("column_id");
		if (column_id) {
			const action_obj = this.dashboard.get_current_action_obj();
			const column = (action_obj.view.columns || []).find((element) => element.id === column_id);
			if (column && column.on_selected_row) {
				column.on_selected_row($chk.parent().parent(), is_selected);
			}
		}
		this.__update_info_selected_rows();
	};

	conf_view.prototype.__update_info_selected_rows = function()
	{
		if (!this.m_has_row_selector) {
			return;
		}

		// pagina - detalle
		const $detail_selected_rows = this.$get_dom_element_by_id("view_page_header_content_grid_detail_selected_rows");
		const $detail_selected_rows_box = this.$get_dom_element_by_id("view_page_header_content_grid_detail_selected_rows").parent();

		let selected_rows = this.get_selected_items();
		let total_selected = (selected_rows || []).length;
		let total_msg;
		const action_obj = this.dashboard.get_current_action_obj();
		let view_messages = (action_obj && action_obj.view && action_obj.view.messages) ? action_obj.view.messages : {};

		if (total_selected > 0) {
			let template = "<strong>{0}</strong>";
			let formatted_total = FORMAT(template, total_selected);
			total_msg = FORMAT(view_messages.total_selected_rows || "<%=_T('Total selected rows: {0}.')%>", formatted_total);

			const MAX_LENGTH = 500;
			let detail_selected_rows = this.__make_detail_of_selected_rows(selected_rows);
			let truncated_detail_selected_rows = inConcert.Utils.truncateWord(detail_selected_rows, MAX_LENGTH);
			let formatted_detail = FORMAT(template, truncated_detail_selected_rows);
			let detail_msg = FORMAT("<%=_T('Detail: {0}')%>", formatted_detail);

			$detail_selected_rows_box.removeClass('hideSnackBar');

			// pagina - detalle
			$detail_selected_rows
				.html(detail_msg)
				.attr("title", detail_selected_rows)
				.show();
		}
		else {
			total_msg = view_messages.no_selected_rows || "<%=_T('You currently have no selected rows.')%>";
			// pagina - detalle
			$detail_selected_rows.html("").hide();
			$detail_selected_rows_box.addClass('hideSnackBar');
		}

		//pagina - total
		const $total_selected_rows = this.$get_dom_element_by_id("view_page_header_content_grid_total_selected_rows");
		$total_selected_rows.html(total_msg);
	};

	conf_view.prototype.__make_detail_of_selected_rows = function(selected_rows)
	{
		if (!selected_rows) {
			return "";
		}

		let action = this.dashboard.get_current_action_obj();
		let detail_selected_rows = "";

		for (let i = 0; i < selected_rows.length; i++) {
			let selected_row = selected_rows[i];
			let name = selected_row[action.view.row_id] || "";
			if (detail_selected_rows.length > 0) {
				detail_selected_rows = detail_selected_rows + ", ";
			}
			detail_selected_rows = detail_selected_rows + name;
		}

		return detail_selected_rows;
	};

	conf_view.prototype.__toggle_select_rows = function(selected)
	{
		const $rows = this.$get_rows();
		$.each($rows, function(_, row) {
			const $row = $(row);
			const $row_selector = $row.find("[column_type='row_selector']");
			const $checkbox = $row_selector.find("input[type='checkbox']");
			$checkbox.prop("checked", selected);
		});

		this.__update_info_selected_rows();
	};

	conf_view.prototype.__toggle_checkbox_select_all_rows = function()
	{
		if (!this.m_has_row_selector) {
			return;
		}
		if (this.m_toggle_select_all_rows_status === "RUNNING") {
			return;
		}
		this.m_toggle_select_all_rows_status = "RUNNING";

		let selected_all = true;
		const $rows = this.$get_rows();

		$.each($rows, function(_, row) {
			const $row = $(row);
			const $row_selector = $row.find("[column_type='row_selector']");
			const $checkbox = $row_selector.find("input[type='checkbox']");
			const is_selected = $checkbox.prop("checked");
			if (!is_selected) {
				selected_all = false;
				return false; // break
			}
		});

		const $thead = this.$thead();
		const $select_all = $thead.find("[header_column_type='row_selector']");
		$select_all.prop("checked", selected_all);

		this.m_toggle_select_all_rows_status = "COMPLETED";
	};

	const m_grid_view_modes = {
		"list"   : {id: "list"   , grd_class_name: "listGrid"   , img_class_name: "far fa-th"   , change_tooltip: "<%=_T('Change to list mode')%>", text: "<%=_T('List')%>"},
		"gallery": {id: "gallery", grd_class_name: "galleryGrid", img_class_name: "far fa-list", change_tooltip: "<%=_T('Change to gallery mode')%>", text:"<%=_T('Gallery')%>"},
	};

	conf_view.prototype.__make_html_system_actions_items_mode = function(cb_on_click_mode = $.noop) {
		const $container = this.$get_dom_element_by_id("view_page_header_content_grid_mode_container");
		const self = this;
		$.each(m_grid_view_modes, function(_, modeInfo) {
			const $mode = $(`
			<button id="${self.dashboard.prefix_id}view_page_header_content_grid_mode_${modeInfo.id}" item_mode_id="${modeInfo.id}" title="${modeInfo.change_tooltip}" class="gridModeButton buttonColorWhite ${modeInfo.grd_class_name}" type="button" disabled>
					<i class="${modeInfo.img_class_name}"></i>
					<span class="${modeInfo.id}Text">${modeInfo.text}</span>
			</button>
			`);
			$mode.off("click").on('click', function(){
				const $view_grid_header_title =  self.$get_dom_element_by_id("view_grid_header_title");
				$view_grid_header_title.html(modeInfo.text);
				const $this = $(this);
				const mode = $this.attr("item_mode_id");

				$container.find(`[item_mode_id!="${mode}"]`).prop("disabled", false);

				$this.prop("disabled", true);
				cb_on_click_mode(mode);
			});
			
			$container.append($mode);
		});
	};

	conf_view.prototype.set_items_mode = function(items_mode) {
		const mode_id = (items_mode || "list").toLowerCase(); // list mode is default
		const mode = m_grid_view_modes[mode_id];
		if (!mode) {
			throw("conf_view.prototype.set_items_mode - Unrecognized view mode " + mode_id);
		}
		// 1 - cambia la clase de la grilla
		const $grid = this.$grid();
		this.remove_list_items_modes($grid, "grd_class_name");
		$grid.addClass(mode.grd_class_name);
	};

	conf_view.prototype.$grid = function() {
		return this.$get_dom_element_by_id("view_grid");
	};

	conf_view.prototype.remove_list_items_modes = function($ctrl, attr_name) {
		for (let mode_id in m_grid_view_modes) {
			if (m_grid_view_modes.hasOwnProperty(mode_id)) {
				let mode = m_grid_view_modes[mode_id];
				let class_name = mode[attr_name];
				$ctrl.removeClass(class_name);
			}
		}
	};

	conf_view.prototype.ready = function(action, config)
	{
		this.initialize();

		// build sub menu frm app
		this.form = this.new_form(action);
		if ( this.form ) {
			this.dashboard.build_app_navigation_sub_menu( this.form.sections );
			if ( action.only_frm ) {
				this.render_form(undefined, config?.extra_parameters);
				return;
			}
		}

		this.init_header_actions(action.view.header_actions);
		this.$get_dom_element_by_id("view_title").text(action.view.title);
		this.$get_dom_element_by_id("view_tagline").text(action.view.tagline || "");
		this.$get_dom_element_by_id("view_container").addClass(action.view.className || "").show();

		const success = this.create_header_columns(action.view.columns);
		if ( success ) {
			// actualizo la imagen correspondiente al item de la vista.
			this.set_title_img();
			// initialize filter (search)
			this.initialize_filter(action.view);
			// initialize events
			this.set_events(action);
			// events suscribe
			this.events_suscribe();
			// paginated
			this.page_start = "";
			this.$grid_scroll = this.$get_dom_element_by_id("grid_scroll");
			this.on_show();
			this.search(undefined, "<%=_T('Loading')%>");
		}
	};

	conf_view.prototype.init_header_actions = function(header_actions)
	{
		const $header_content = this.$get_dom_element_by_id("view_page_header_content");
		if (!header_actions) {
			$header_content.hide();
			return;
		}
		$header_content.show();

		const $system_actions = this.$get_dom_element_by_id("view_page_header_system_actions");
		const self = this;
		if (header_actions.system_actions) {
			$system_actions.show();
			// change grid mode
			const $change_grid_mode = this.$get_dom_element_by_id("view_page_header_content_grid_mode_container");
			const item_mode_id = header_actions.system_actions.items_mode;
			if (item_mode_id) {
				$change_grid_mode.show();
				this.__make_html_system_actions_items_mode(function(item_mode_id) {
					self.set_items_mode(item_mode_id);
					let on_set_items_mode =  header_actions.system_actions.on_set_items_mode || $.noop;
					on_set_items_mode(item_mode_id, self);
				});

				$change_grid_mode.find(`[item_mode_id="${item_mode_id}"]`).trigger("click");
			}
			else {
				$change_grid_mode.hide();
			}
			// batch actions
			const $btn_menu_actions = this.$get_dom_element_by_id("view_page_header_content_grid_menu_actions");
			if (header_actions.system_actions.batch_actions) {
				if (header_actions.system_actions.batch_actions.caption) {
					const $btn_menu_actions_name = this.$get_dom_element_by_id("view_page_header_content_grid_menu_actions_name");
					$btn_menu_actions_name.text(header_actions.system_actions.batch_actions.caption);
				}
				$btn_menu_actions.show();
				this.disabled_batch_actions(true);
			}
			else {
				$btn_menu_actions.hide();
			}
		}
		else {
			$system_actions.hide();
		}
	};

	conf_view.prototype.set_title_img = function()
	{
		const $item_img = this.$get_dom_element_by_id("view_item_img");
		if ($item_img.length === 0) {
			return;
		}
		this.dashboard.set_title_img($item_img);
	};

	conf_view.prototype.sanitize_value = function(value)
	{
		return this.dashboard.sanitize_value(value);
	};

	conf_view.prototype.escape = function(value)
	{
		return this.dashboard.escape(value);
	};

	conf_view.prototype.show_popup_message = function(info)
	{
		this.form.show_popup_message(info);
	};

	conf_view.prototype.hide_waiting = function()
	{
		this.dashboard.hide_waiting();
	};

	conf_view.prototype.edit_first_row = function()
	{
		let $rows = this.$get_rows();
		if (!$rows) {
			return;
		}
		let $first_row;
		$.each($rows, function(_, row) {
			$first_row = $(row);
			return false; // break
		});
		if (!$first_row) {
			return;
		}
		this.grid_execute_action("edit", $first_row);
	};

	conf_view.prototype.auto_reload_seconds = function()
	{
		return this.dashboard.get_current_action_obj()?.view?.auto_reload_seconds || null;
	};

	conf_view.prototype.enable_auto_reload = function()
	{
		const seconds = this.auto_reload_seconds();
		if (!seconds || this.auto_reload_interval) return;
		const app = inConcert.app.getCurrentApp();
		this.auto_reload_interval = window.setInterval(() => this.auto_reload(app), seconds * 1000);
	};

	conf_view.prototype.disable_auto_reload = function()
	{
		if (!this.auto_reload_seconds() || !this.auto_reload_interval) return;
		window.clearInterval(this.auto_reload_interval);
		this.auto_reload_interval = null;
	};

	conf_view.prototype.auto_reload = function(app)
	{
		if (!this.get_in_view() || app !== inConcert.app.getCurrentApp()) return;
		this.refresh("AUTORELOAD");
	};
}());
