module(..., package.seeall, inconcert.orbit.application.Decorate)

local utils = require "shared.apps.utils"
local Synchronize = require "Synchronize"
local https = Synchronize("luasec_mutex", 
    function()
        return require ("ssl.https")
    end
)
local http = require "socket.http"
local ltn12 = require("ltn12")

m_htmlRender = nil
m_appLocation = nil

local function decodeXmlResponse(response)
	xml = xml or require "pl.xml"
	
	local function loadData(root, xmlData)
		local key = xmlData.tag
		local text = xmlData:get_text()
		local data = ""
		if text ~= "" then
			data = text
		elseif xmlData:first_childtag() then
			data = {}
			for xmlChild in xmlData:children() do
				data = loadData(data, xmlChild)
			end
		end
		if #root > 0 then
			--is an array
			table.insert(root, data)
		elseif root[key] then
			--should be an array
			local firstElement = root[key]
			root = {}
			table.insert(root, firstElement)
			table.insert(root, data)
		else
			--is an object
			root[key] = data
		end
		return root
	end

	local xmlObject = xml.parse(response)
	local data = loadData({}, xmlObject)
	return data
end


local function makeHttpRequest (web)

    local ok, info = pcall(json.decode, web.input["post_data"])
    if not ok then
        LogError("[FLOW_DESIGNER.PROCESS_FLOW_HTTP:MAKEHTTPREQUEST] deserealizando payload")
        web.status = 500
        return false
    end

    if table.getn(info.queryParameters) > 0 then
        info.endpoint = info.endpoint ..[[?]]
        for index, param in ipairs(info.queryParameters) do
            if index == 1 then
                info.endpoint = info.endpoint ..string.format("%s=%s", param["key"],  param["value"])
            else
                info.endpoint = info.endpoint ..string.format("&%s=%s",  param["key"], param["value"])
            end
        end
    end
    local headers={}
    if table.getn(info.headers) > 0 then
        for index, param in ipairs(info.headers) do
            headers[param["key"]] = param["value"]
        end
    end

    if info.bodyType == "text" then
        headers["content-length"] = string.len(info.body)
    else
        if table.getn(info.bodyParameters) > 0 then
			local encodeParameter = function(param)
				local CharToHex = function(c)
					return string.format("%%%02X", string.byte(c))
				end

				param = param:gsub("\n", "\r\n")
				param = param:gsub("([^%w ])", CharToHex)
				param = param:gsub(" ", "+")

				return param
			end
			
			info.body = ""
			for index, param in ipairs(info.bodyParameters) do
				if index == 1 then
					info.body = info.body ..string.format("%s=%s", encodeParameter(param["key"]),  encodeParameter(param["value"]))
				else
					info.body = info.body ..string.format("&%s=%s", encodeParameter(param["key"]),  encodeParameter(param["value"]))
				end
				
			end

			headers["content-length"] = string.len(info.body)
		end
    end

   


    local response = {}
    local result, code, headersResp, statusline
	if string.find(info.endpoint,"https://") then
		result, code, headersResp, statusline = https.request{
			url = info.endpoint,
            method = info.method,
            headers = headers,
            source = ltn12.source.string(info.body),
            sink = ltn12.sink.table(response)
		}
	elseif string.find(info.endpoint,"http://") then
		result, code, headersResp, statusline = http.request{
			url = info.endpoint,
            method = info.method,
            headers = headers,
            source = ltn12.source.string(info.body),
            sink = ltn12.sink.table(response)
		}
	else
		result, code, headersResp, statusline = https.request{
			url = "https://" ..info.endpoint,
            method = info.method,
            headers = headers,
            source = ltn12.source.string(info.body),
            sink = ltn12.sink.table(response)
		}
	end
    -- local result, code, headers, statusline = https.request{
    --     url = info.endpoint,
    --     method = info.method,
    --     headers = headers,
    --     source = ltn12.source.string(info.body),
    --     sink = ltn12.sink.table(response)
    -- }
    web.status = code

    local serialized = table.concat(response, "")
    local result
	if string.len(serialized) > 0 then
		local ok, deserialized = pcall(json.decode,serialized)
		if not ok then
			local xmlOk, deserializedXML = pcall(decodeXmlResponse, serialized)
			if not xmlOk then
				result = serialized
			else
				result = deserializedXML
			end
		else
			result = deserialized
		end
	else
		result = ""
	end
    return result
end


function MapUris(module, htmlRender, prefix, appLocation)
    local LogRequest = require("apps.agent.requestFilters").LogRequest
    module:dispatch_post (utils.JsonReply ..LogRequest ..makeHttpRequest, prefix.."/Makerequest")
    m_htmlRender = htmlRender
    m_appLocation = appLocation
end