require "inconcert.translators"
local lua_cassandra = require "lua_cassandra"
require "redis"
local WrapRedis = require "wrapredis"

local utils = require "shared.apps.utils"

local m_moduleLocation = select(2, ...)

module(..., package.seeall)

local m_dataSource
local m_models = {}
local m_loaded = false

function GetRedisClient ()
	return assert(m_dataSource.redis)
end

--recupera la lista de modelos del directorio models
local function GetModelsFromFilesystem()
	local modelsDir = m_moduleLocation .. "/models"
	if not utils.isDirectory(modelsDir) then
		return {}
	end
	local models = {}
	for file in lfs.dir(modelsDir) do
		if file ~= "." and file ~= ".." then
			local f = modelsDir..'/'..file
			local attr = lfs.attributes (f)
			if attr.mode ~= "directory" then
				local modelName = string.match(file, [[(.+).lua]])
				if (modelName) then
					models[modelName] = modelName
				end
			end
		end
	end
	return models
end

--///////////////////////////////////////////////////////////////////////////////////
-- arma la tabla de modelos definidos
function CreateModelTable() 
	--LogInfo("[process flows] Creating cassandra models")
	if not m_loaded then
		for key, controllerDef in pairs(GetModelsFromFilesystem()) do
			local model = require("apps.flow_designer.models." .. key)
			if model.GetDefinition then
				m_models[model.GetName()] = model.GetDefinition(m_dataSource)
			end
		end
		m_loaded = true
	else
		--if model modules were already loaded just overwrite their inner datasource
		--with the newly changed one
		for modelName, model in pairs(m_models) do
			model.GetDefinition(m_dataSource)
		end
	end
end
--///////////////////////////////////////////////////////////////////////////////////
-- 
function GetModels()
	return m_models
end

--///////////////////////////////////////////////////////////////////////////////////
-- 
function BuildModels (self)
	local redisConfig = utils.GetConnectionConfig("redis", "flow_designer")
	if not redisConfig then
		error("+Unable to initialize flow_designer app, connection to redis failed")
	end
	LogInfo("[FLOW_DESIGNER.MODEL:BuildModels] Connecting to redis on host: '%s', port:'%s'.", redisConfig.host, redisConfig.port)
	--start in nonblocking mode
	local redis = WrapRedis(redisConfig.host, redisConfig.port, redisConfig.pwd)
	if not redis then
		error("-Unable to initialize flow_designer app, connection to redis failed")
	end
	--init local datasource table
	m_dataSource = {
		cassandra = lua_cassandra,
		redis = redis
	}
	utils.SetupConnections(m_dataSource)
	CreateModelTable()
	LogInfo("flow_designer model built")
	return true
end
