module(..., package.seeall)
local utils = require "shared.apps.utils"
local m_datasource

function GetName ()
	return "ProcessFlows"
end

function GetDefinition (datasource)
	m_datasource = datasource
	return _M
end

local function GetModel (entityName)
	assert(type(entityName) == "string")
	if not m_models or not m_models[entityName] then
	-- Si el modelo no se ha creado aun, debo crearlo.
	--  Esto sucede en algunas ocasiones si es que me han invocado desde otra app Ej: "Allegro" y aun no se ha creado el modelo.
	--  Entonces tengo que crear el modelo.
		local model = require "apps.administration.model"
		model:BuildModels()
		m_models = model:GetModels()
	end
	local entityModel = m_models[entityName]
	assert(type(entityModel) == "table")
	return entityModel
end

function GetFlows (vcc, flowId, flowCols, keyspace)
	local flows = utils.GetAllColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "ProcessFlows",
		superColumns = true,
		key = vcc,
		columns = flowCols,
		filterSuperColumns = true
	}
	if flowId then
		return flows and flows[flowId] or {}
	else
		return flows or {}
	end
end

function LoadFlowsByType (vcc, proxyType, keyspace)
	assert(vcc)
	assert(proxyType)

	local flows = utils.GetAllColumns({
		keyspace = keyspace or "Agent",
		columnFamily = "ProcessFlows",
		superColumns = true,
		key = vcc,
		columnCount = 5000,
		columns = {}
	}) or {}

	local flowsByType = {}
	for _, flow in pairs(flows) do
		if flow.Type == proxyType then
			table.insert(flowsByType, flow)
		end
	end

	return flowsByType
end

local function CleanProcessFlowTrace(vcc, flowId)
	m_datasource.redis:del("ProcessFlowTrace:" .. vcc .. ":" .. flowId)
	m_datasource.redis:del("ProcessFlowTraceCount:" .. vcc .. ":" .. flowId)
end

function GetTrace(vcc, flowId)
	return m_datasource.redis:hgetall("ProcessFlowTrace:" .. vcc .. ":" .. flowId) or {}
end

function GetEvents(reference)
	local rawList = m_datasource.redis:lpop("ProcessFlowEvent:" .. reference, 100) or {}
	local list = {}
	for _, eventStr in pairs(rawList) do
		table.insert(list, json.decode(eventStr))
	end
	return list
end

function SaveFlows (vcc, flows, keyspace, otherInfo)
	local sharedFileModel = GetModel("SharedFiles")
	-- 1 - Elimino todas las relaciones para el/los dibujo/s que se van a actualizar (contra sharedfiles)
	if otherInfo then
		for flowId in pairs(flows) do
			sharedFileModel.RemoveSharedFilesFlowsDesginerRelations({vcc = vcc, flowId = flowId}, keyspace)
		end
	end
	-- 2 - Almaceno los Flows
	utils.SaveRows {
		keyspace = keyspace or "Agent",
		columnFamily = "ProcessFlows",
		superColumns = true,
		rows = {
			[vcc] = flows
		}
	}
	-- 3 - Creo nuevamente las relaciones de los dibujos actualizados (contra sharedfiles)
	if otherInfo then
		sharedFileModel.SaveSharedFilesFlowsDesginerRelations(otherInfo, keyspace)
	end
	for flowId in pairs(flows) do
		CleanProcessFlowTrace(vcc, flowId)
	end
end

function CreateTestSession (session)
	local key = "ProcessFlowTestSession:" .. session.id
	m_datasource.redis:set(key, json.encode(session))
	m_datasource.redis:expire(key, 60 * 60)
end

function RemoveFlows (vcc, flows_to_remove, keyspace)
	-- 1 - Remuevo los Flows
	utils.RemoveColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "ProcessFlows",
		superColumns = true,
		rows = {
			[vcc] = flows_to_remove
		}
	}

	-- 2 - Remuevo las relaciones de los dibujos (contra sharedfiles)
	local sharedFileModel = GetModel("SharedFiles")
	for _, flowId in ipairs(flows_to_remove) do
		sharedFileModel.RemoveSharedFilesFlowsDesginerRelations({vcc = vcc, flowId = flowId}, keyspace)
	end

	for _, flowId in pairs(flows_to_remove) do
		CleanProcessFlowTrace(vcc, flowId)
	end
end

function GetAccountsByFlow (vcc, flowId, keyspace)
	flowId = utils.AppendVcc( flowId, vcc )
	return utils.GetAllColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "AccountsByProcessFlow",
		key = flowId
	}
end
