
local function empty(value)
	return not value or value == ""
end

local function valueOr (value, default)
	return not empty(value) and value or default
end

local function valueOrNil (value)
	return valueOr(value, "nil")
end

local function booleanValue (value)
	return value and "true" or "false"
end

local function invalidParameter (reason)
	return 400, {result = "INVALID_PROCESS", reason = reason}
end

local function invalidParameterState (state, reason)
	reason = "state " .. state .. " " .. text
	return 400, {result = "INVALID_PROCESS", reason = reason, state = state}
end

local function invalidParameterActivity (state, activity, reason)
	reason = "state " .. state .. " activity " .. activity .. " " .. reason
	return 400, {result = "INVALID_PROCESS", reason = reason, state = state, activity = activity}
end

local function scapeText (text)
	return text:gsub("'", "\\'"):gsub('"', '\\"')
end

local function getNode (model, id)
	return model[id]
end

local function getConnections (model, node)
	if type(node) == "string" then
		node = getNode(model, node)
	end
	return node and node.connections or {}
end

local function getConnection (model, node, index)
	index = index or 1
	local connections = getConnections(model, node)
	return #connections >= index and connections[index] or nil
end

local function addText (object, key, withoutSpace)
	local value = object[key]
	if not value then
		return ""
	end
	if not withoutSpace then
		value = value .. " "
	end
	return value
end

local function makeCondition (condition)
	if not condition.field or not condition.comparator or not condition.value then
		return invalidParameter("invalid condition")
	end
	local text = ""
	text = text .. addText(condition, "logic")
	text = text .. addText(condition, "begin_parenths")
	text = text .. addText(condition, "field")
	text = text .. addText(condition, "comparator")
	text = text .. addText(condition, "value")
	text = text .. addText(condition, "end_parenths")
	return 200, text
end

local function makeConditions (conditions)
	local text = ""

	for _, condition in pairs(conditions) do

		if condition ~= nil then
			local status, textCondition = makeCondition(condition)
			if status ~= 200 then
				return status, textCondition
			end
			text = text .. textCondition
		end
			
	end
	if text ~= "" then
		text = "return " .. text
	end
	return 200, text
end

local function makeDialerAddresses(addresses)
	local result = "{\n"
	for _, address in pairs(addresses) do
		result = result .. string.format(
[[
{
	Type = "Phone",
	Channels = { "CALL" },
	Kind = "%s",
	Number = %s,
},
]],
			address.phoneType,
			valueOrNil(address.phoneNumber)
		)
	end
	result = result .. "\n}"
	return result
end

local function makeDialerNameValues(values, requiredType)
	local result = "{\n"
	for _, value in pairs(values) do
		if value.attrType == requiredType then
			result = result .. string.format(
[[
{
	Name = "%s",
	Value = %s,
},
]],
				value.attrName,
				valueOrNil(value.attrValue)
			)
		end
	end
	result = result .. "\n}"
	return result
end

return {
	empty = empty,
	valueOr = valueOr,
	valueOrNil = valueOrNil,
	booleanValue = booleanValue,
	invalidParameter = invalidParameter,
	invalidParameterState = invalidParameterState,
	invalidParameterActivity = invalidParameterActivity,
	scapeText = scapeText,
	getNode = getNode,
	getConnections = getConnections,
	getConnection = getConnection,
	addText = addText,
	makeCondition = makeCondition,
	makeConditions = makeConditions,
	makeDialerAddresses = makeDialerAddresses,
	makeDialerNameValues = makeDialerNameValues,
}
