
local function MakeParamsExecutable (className, method, constructor)
	local params = constructor and "" or "self"
	for _, param in pairs(method.parameters or {}) do
		params = params .. (params ~= "" and ", " or "") .. param.name
	end
	return 200, params
end

local function MakeMethodExecutable (className, method)
	local status, paramsCode = MakeParamsExecutable(className, method)
	if status ~= 200 then
		return status, paramsCode
	end
	local code = string.format("function(%s)\n%s\nend", paramsCode, method.code)
	return 200, code
end

local function MakeConstructorExecutable (class)
	local status, paramsCode = MakeParamsExecutable(className, class.Editor.classConstructor, true)
	if status ~= 200 then
		return status, paramsCode
	end

	local bodyCode = "local self = {"
	--methods
	for _, method in pairs(class.Editor.methods) do
		bodyCode = bodyCode .. string.format("\n\t%s = classHelpers.%s.%s,", method.name, class.Name, method.name)
	end
	--attributes
	for _, attr in pairs(class.Editor.attributes) do
		bodyCode = bodyCode .. string.format("\n\t%s = %s,", attr.name, attr.value ~= "" and attr.value or "nil")
	end
	bodyCode = bodyCode .. "\n}"
	--code
	bodyCode = bodyCode .. "\n" .. class.Editor.classConstructor.code

	local code = string.format("function(%s)\n%s\nreturn self\nend", paramsCode, bodyCode)
	return 200, code
end

local function MakeExecutable (class)
	local code = ""
	code = code .. string.format("classHelpers.%s = {", class.Name)

	--methods
	for _, method in pairs(class.Editor.methods) do
		local status, methodCode = MakeMethodExecutable(class.Name, method)
		if status ~= 200 then
			return status, methodCode
		end
		code = code .. string.format("\n%s = %s,", method.name, methodCode)
	end

	--constructor
	local status, constructorCode = MakeConstructorExecutable(class)
	if status ~= 200 then
		return status, constructorCode
	end
	code = code .. string.format("\nnew = %s,", constructorCode)
	
	code = code .. "\n}"
	
	class.Executable = code
	return 200, { result = "OK" }
end

return {
	MakeExecutable = MakeExecutable
}
