
local function MakeExecutableActivity (processName, stateName, activityId, data, activitiesProcessors, aux)
	if not data or not data.type then
		return aux.invalidParameterActivity(stateName, activityId, "invalid")
	end
	local processor = activitiesProcessors[data.type]
	if not processor then
		return aux.invalidParameterActivity(stateName, activityId, "has not processor")
	end
	local status, code = processor.MakeExecutable(stateName, activityId, data, aux)
	if status ~= 200 then
		return status, code
	end
	local activity = {
		code = code,
		type = data.type,
		result = aux.valueOr(data.result, nil)
	}
	return 200, activity
end

local function MakeExecutable (processName, stateId, draw, activitiesProcessors, aux)
	local stateData = draw.states and draw.states[stateId:gsub("state_", "")]
	local name = stateData and stateData.name or stateId
	local model = stateData and stateData.model
	if not stateData or not model then
		return aux.invalidParameter("state " .. stateId .. " model not found")
	end
	local state = {
		label = name,
		activities = {}
	}
	--initial node
	local initialConnection = aux.getConnection(model, "initial")
	if not initialConnection then
		return aux.invalidParameterState(name, "initial condition not found")
	end
	local initialNodeId = initialConnection.to
	if not initialNodeId or not aux.getNode(model, initialNodeId) then
		return aux.invalidParameterState(name, "initial condition invalid")
	end
	--activities
	for activityId, activity in pairs(model) do
		--if activityId ~= "initial" then
		if activityId ~= "initial" and not (string.find(activityId, "activity_") and activity.data == nil) then
			--process activity
			local status, activityData = MakeExecutableActivity(processName, name, activityId, activity.data, activitiesProcessors, aux)
			if status ~= 200 then
				return status, activityData
			end
			--process transitions
			local transitions = {}
			for index, connection in pairs(aux.getConnections(model, activityId)) do
				local connectionData = connection.data or {}
				--process transition conditions
				local status, conditionData = aux.makeConditions(connectionData.conditions)
				if status ~= 200 then
					return status, conditionData
				end
				table.insert(transitions, {
					index = index,
					condition = conditionData,
					to = connection.to,
					label = aux.empty(connectionData.description) and ("transition_" .. index) or connectionData.description,
					port_id = connectionData.port_id,
				})
			end
			activityData.initial = initialNodeId == activityId
			activityData.label = aux.empty(activity.data.description) and activityId or activity.data.description
			activityData.transitions = transitions
			state.activities[activityId] = activityData
		end
	end
	--add transitions to name
	for _, activity in pairs(state.activities) do
		for _, transition in pairs(activity.transitions) do
			transition.to_label = state.activities[transition.to].label
		end
	end
	return 200, state
end

return {
	MakeExecutable = MakeExecutable
}
