/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 11 */

/* global inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function ()
{
	"use strict";

	const format = inConcert.Utils.format;

	
//------------------------------------------------------
	// flow_designer navigation config
	var processflows_config = function()
	{
		var get_message_error = function(type, err, name)
		{
			var message;
			if (type === "delete") {
				var err_messages = {
					"FLOW_BEING_USED_FOR_ACCOUNT" : function() {
						var accounts = "";
						$.each(err.accounts || [], function(_, account) {
							accounts += (accounts? ", " : "") + format("{0} ({1})", account.name, account.campaign);
						});
						return format(
							"<%=_T('Unable to delete {0} process flow. Flow to delete is currently in use for {1}.')%>",
							name,
							accounts
						);
					},
					"FORBIDDEN": function() {
						return format( "<%=_T('Unable to delete {0} process flow. You dont have permission.')%>", name );
					}
				};

				var err_id;
				if (typeof err === "object") {
					err_id = err.result || "";
				}
				else {
					err_id = err;
				}
				err_id = err_id.toUpperCase();

				if ( err_messages[err_id] ) {
					message = err_messages[err_id]();
				}
				else {
					message = format( "<%=_T('Error deleting {0} flow.')%>", name );
				}
			}
			return message;
		};

		var flowAPI = inConcert.app.getFlowDesignerAPI();

		// execute custom action
		var execute_custom_action = function( action_id, $row, view )
		{
			var project_id = $row.attr( "row_id" );
			var display_name = $row.attr("row_name");
			switch ( action_id ) {
				case "clone":
					if ( !view.form.has_permission_to_save() ) {
						view.show_status_message(
							format( "<%=_T('Unable to clone {0} flow. You dont have permission.')%>", display_name),
							"Error",
							5000
						);
						return;
					}
					var popup_params = {
						type: "clone",
						source_id: project_id
					};

					view.render_popup( popup_params );
					break;

				case "export":
					if ( !view.form.has_permission_to_save() ) {
						view.show_status_message(
							format( "<%=_T('Unable to export {0} project. You dont have permission.')%>", display_name),
							"Error",
							5000
						);
						return;
					}
					flowAPI.exportFlow( project_id );
					break;
			}
		};

		var can_show_classes = function() {
			return inConcert.permissions.to_action("flow_designer", "view_flow_classes");
		};

		var show_classes = function(view) {
			if (!can_show_classes()) {
				const error_message = "<%=_T('Unable to show classes IDE: You dont have permission.')%>";
				view.show_status_message( error_message, "Error", 5000 );
				return;
			}
			const classes_editor = new inConcert.adm_process_flow_classes_editor();
			$("#conf_flow_designer_grid_scroll").hide();
			
			classes_editor.Show(
				$.noop,
				function() {
					$("#conf_flow_designer_grid_scroll").show();
				}
			);
		};

		var can_show_datasources = function() {
			return inConcert.permissions.to_action("flow_designer", "view_flow_datasources");
		};

		var show_datasources = function(view) {
			if (!can_show_datasources()) {
				const error_message = "<%=_T('Unable to show datasources: You dont have permission.')%>";
				view.show_status_message( error_message, "Error", 5000 );
				return;
			}
			const form = view.form
			const datasources_editor = new inConcert.adm_process_flow_datasources_editor(undefined,form
			);
			$("#conf_flow_designer_grid_scroll").hide();
			datasources_editor.Show(
				$.noop,
				function() {
					$("#conf_flow_designer_grid_scroll").show();
				}
			);
		};

		// this is the object that dashboard.get_current_action_obj manipulates
		return {
			id: "flow_designer",
			name: "<%=_T('Bots & Automation')%>",
			permissions: {
				view           : "view_flows",
				save           : "save_flows",
				clone          : "save_flows",
				erase          : "delete_flows",
				"export"       : "export_flows",
				permission_type: "vcc"
			},
			class_name: "itemProcessFlows",
			i_class_name: "far fa-robot",
			view: {
				render: "/inconcert/apps/flow_designer/pages/view_flow_designer_content_default.<%=_L%>.html#grunt-cache-bust",
				title: "<%=_T('Bots & Automation')%>",
				tagline: "<%=_T('Tagline for Process Flows')%>",
				new_button_name: "<%=_T('New flow')%>",
				row_id: "Id",
				display_name: "Name",
				reload_when_enter_application: true,
				paginator: {
					paged_text_format : "<%=_T('{page_row_start}-{page_row_end} of {many}')%>"
				},
				header: {
					hide_new_button: false,
					hide_total_rows: true
				},
				global_buttons : [
					{
						id : "classes",
						name : "<%=_T('Classes IDE')%>",
						className : "buttonColorWhite pageButtonNoIcon",
						enabled : can_show_classes,
						run : function(view) {
							show_classes(view);
						},
						before: false
					},
					{
						id : "datasources",
						name : "<%=_T('Datasources')%>",
						className : "buttonColorWhite",
						enabled : can_show_datasources,
						run : function(view) {
							show_datasources(view);
						},
						before : false
					}
				],
				messages: {
					erase: "<%=_T('Do you want to delete flow {0} ?')%>",
					err: get_message_error
				},
				uri: {
					erase: function(flow_id, _, callback) {
						var flows = [];
						flows.push(flow_id);
						return flowAPI.deleteFlows(flows, callback);
					},
					search: function(callback, filter) {
						return flowAPI.searchFlows(filter, callback);
					}
				},
				source: ["Id", "Type", "Name", "Description", "InUse", "Accounts"],
				columns: [
					{id: "_number"    , name: "", type: "counter"},
					{id: "Type"       , name: "", type: "image", 
						class_name : "flowDesignerIconColumnHeader", 
						get_value: function( flow ) {
							return { class_name : "flowType" + flow.Type, column_class_name : "flowDesignerIconColumn" };
						}
					},
					{id: "Name"       , name: "<%=_T('Name')%>"},
					{id: "Description", name: "<%=_T('Description')%>"},
					{id: "InUse", 
						name: "<%=_T('In Use')%>", 
						type: "boolean", 
						class_name: "booleanColumnTitle",  
						get_tooltip: function( flow ) { 
							let accounts = "";
							const accountList = flow.Accounts;
							$.each(accountList || [], function(_, account) {
								account.assignment_type = (account.assignment_type || "").replace("on_schedule", "<%=_T('Active Process')%>");
								account.assignment_type = account.assignment_type.replace("out_of_schedule", "<%=_T('Inactive Process')%>");
								accounts += (accounts? "\n" :  "") + format("{0} ({1}) [{2}]", account.campaign, account.name, account.assignment_type);
							});
							if (accounts === "") {
								return "<%=_T('No')%>";
							}
							return accounts;
						}
					},
					{id: "_action"    , name: "" ,
						class_name: "gridColumnFourActions",
						system_action: { edit: true, erase: true },
						custom_action: {
							clone: { // test 1
								name: "<%=_T('Clone')%>",
								execute: execute_custom_action,
								class_name: "clone"
							},
							"export": {
								name: "<%=_T('Export')%>",
								execute: execute_custom_action,
								class_name: "export"
							}
						}
					}
				],
				filter: {fields: {"Name": {control: null}, "Description": {control: null}}},
				realtime: null,
				must_init_full_screen: ()=> true
			},
			form: {instance: inConcert.adm_form_process_flows}
		};
	};

	//------------------------------------------------------
	// get all navigation config
	const get_navigation_config = function()
	{
		return {
            flow_designer                 : processflows_config(),
		};
	};

	function flow_designer_config() {}

	if (typeof inConcert.flow_designer_config !== "function") {
		inConcert.flow_designer_config = flow_designer_config;
	}

	flow_designer_config.prototype.get_app_config = function()
	{
		return {
			id         : "flow_designer",
			name       : "<%=_T('Bots & Automation')%>",
			selected   : false,
			class_name : "appAdministration",
			main_html  : "/inconcert/apps/flow_designer/pages/conf_flow_designer_main.<%=_L%>.html#grunt-cache-bust",
			navigation : get_navigation_config(),
			allow_disable_all_navigation : true,
			get_skin: function() {
				return "skinOCCNew";
			},
			can_show   : function() {
				return inConcert.permissions.to_category("flow_designer", "view");
			},
			css : [
				{ id: "adm_messages_css"                            , href: "/inconcert/apps/administration/css/administrationRules.css#grunt-cache-bust"                       },
				{ id: "adm_flow_css"                                , href: "/inconcert/apps/flow_designer/css/skin-2021/flowDesigner.css#grunt-cache-bust"                    },
				{ id: "adm_popover_css"                             , href: "/inconcert/apps/flow_designer/css/skin-2021/flowDesignerPopover.css#grunt-cache-bust"             },
				{ id: "adm_classes_css"                             , href: "/inconcert/apps/flow_designer/css/skin-2021/flowDesignerClasses.css#grunt-cache-bust"             },
				{ id: "adm_datasources_css"                         , href: "/inconcert/apps/flow_designer/css/skin-2021/flowDesignerDatasources.css#grunt-cache-bust"         },
				{ id: "skin_occ_internal_forms_css"                 , href: "/inconcert/apps/configuration/css/skin-2021/masterInternalPageForms.css#grunt-cache-bust"          },
                { id: "skin_occ_adm_messenger_css"                 	, href: "/inconcert/apps/administration/css/skin-2021/administrationPageMessenger.css#grunt-cache-bust"     },
                { id: "skin_occ_master_buttons_css"                 , href: "/inconcert/apps/agent/css/skin-2021/masterButtons.css#grunt-cache-bust"                            },
				{ id: "skin_occ_inbox_master_css"                   , href: "/inconcert/apps/agent/css/skin-2021/inboxMaster.css#grunt-cache-bust"                              },
                { id: "skin_occ_messages_tooltips_css"              , href: "/inconcert/apps/agent/css/skin-2021/genericMessagesAndTooltips.css#grunt-cache-bust"               },
				{ id: "skin_occ_adm_master_css"                     , href: "/inconcert/apps/administration/css/skin-2021/administrationMaster.css#grunt-cache-bust"            },
				{ id: "skin_occ_inbox_master_popups_css"            , href: "/inconcert/apps/agent/css/skin-2021/inboxMasterPopUps.css#grunt-cache-bust"                        },
				{ id: "skin_occ_master_client_window_css"           , href: "/inconcert/apps/agent/css/skin-2021/masterWindowCommon.css#grunt-cache-bust"                       },
				{ id: "skin_occ_custom_visual_search_css"           , href: "/inconcert/apps/agent/css/skin-2021/customVisualSearch.css#grunt-cache-bust"                       },
				{ id: "skin_occ_mxgraph_css"           				, href: "/inconcert/apps/agent/css/mxgraph.css#grunt-cache-bust"                       },
				{ id: "skin_occ_adm_shared_files_css"              	, href: "/inconcert/apps/administration/css/skin-2021/sharedFiles.css#grunt-cache-bust"                     },
				
			]
		};
	};
}());
