/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double */

/* global $, inConcert, Mustache */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	var format = inConcert.Utils.format;
	var templates = inConcert.templates.process_flows;

	function adm_process_flow_actions_bar(main) {
		this.main = main;
	}

	if (typeof inConcert.adm_process_flow_actions_bar !== "function") {
		inConcert.adm_process_flow_actions_bar = adm_process_flow_actions_bar;
	}

	adm_process_flow_actions_bar.prototype.GetActions = function() {
		return [];
	};

	adm_process_flow_actions_bar.prototype.__makeAction = function(action) {
		var self = this;
		action.withText = !action.withoutText;
		var $action = $(Mustache.render(templates.action_button, action));
		$action.click(function() {
			self.RunAction($(this).attr("action_button"));
		});
		return $action;
	};

	adm_process_flow_actions_bar.prototype.__make = function() {
		var self = this;
		var $actions = [];
		self.actions_d = {};
		var $container = null;
		$.each(self.GetActions(), function(_, action) {
			var $action = self.__makeAction(action);
			if (action.sectionBegin) {
				if ($container) {
					$actions.push($container);
				}
				$container = $(format("<div class='{0}'></div>", action.sectionBegin));
			}
			if ($container) {
				$container.append($action);
			}
			else {
				$actions.push($action);
			}
			self.actions_d[action.id] = action;
		});
		if ($container) {
			$actions.push($container);
		}
		return $actions;
	};

	adm_process_flow_actions_bar.prototype.GetEnabledActions = function() {
		var self = this;
		var actions = [];
		var $actions = self.main.$element("editor_actions");
		$.each(self.GetActions(), function(_, action) {
			var $action = $actions.find(format("[action_button={0}]", action.id));
			if (!$action.is(":disabled")) {
				actions.push(action);
			}
		});
		return actions;
	};

	adm_process_flow_actions_bar.prototype.__updateButtonState = function(action, $actions, selected) {
		var self = this;
		var $action = $actions.find(format("[action_button={0}]", action.id));
		var enabled = true;
		//validate selection
		if (action.selection && !selected.total || action.singleSelection && selected.total > 1) {
			enabled = false;
		}
		//validate nodes only selection
		if (enabled && action.selection && !action.connectors && selected.connectors) {
			enabled = false;
		}
		if (enabled && action.enabled) {
			enabled = action.enabled(self, selected);
		}
		$action.prop("disabled", !enabled);
	};

	adm_process_flow_actions_bar.prototype.UpdateButtonsState = function() {
		var self = this;
		if (!self.editor) {
			return;
		}
		var $actions = self.main.$element("editor_actions");
		var selected = self.editor.CountSelectedNodes();
		$.each(self.GetActions(), function(_, action) {
			self.__updateButtonState(action, $actions, selected);
		});
	};

	adm_process_flow_actions_bar.prototype.Show = function(editor) {
		this.$actions = this.__make();
		this.main.$element("editor_actions").append(this.$actions);
		this.editor = editor;
		this.UpdateButtonsState();
	};

	adm_process_flow_actions_bar.prototype.Close = function() {
		$.each(this.$actions || [], function(_, $action) {
			$action.remove();
		});
		this.$actions = null;
		this.editor = null;
	};

	adm_process_flow_actions_bar.prototype.RunAction = function(actionId) {
		var self = this;
		if (self.main.__isProcessing() || !self.actions_d[actionId]) {
			return;
		}
		var $action = self.main.$element("editor_actions").find(format("[action_button={0}]", actionId));
		if ($action.is(":disabled")) {
			return;
		}
		var action = self.actions_d[actionId];
		if (action.confirmation) {
			var confirmation = action.confirmation;
			if ($.isPlainObject(action.confirmation)) {
				var selected = self.editor.CountSelectedNodes();
				confirmation = !selected.nodes? confirmation.connectors : confirmation.nodes;
			}
			if (!confirmation) {
				return self.editor["__" + actionId]();
			}
			$.inconcertPopup.Show(
				"<%=_T('Confirmation')%>",
				confirmation,
				$.inconcertPopup.ButtonSets.BS_YESNO,
				function(response) {
					if (response.toLowerCase() === "yes") {
						self.editor["__" + actionId]();
					}
				},
				undefined,
				{
					popup_id        : "conf_inconcertPopup",
					container_class : "confirmationPopUp",
					overlay_class   : "confirmationOverlay"
				}
			);
		}
		else {
			self.editor["__" + actionId]();
		}
	};

}());
