/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 9 */

/* global inConcert, ace, Mustache */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	var format = inConcert.Utils.format;
	var templates = inConcert.templates.process_flows;

	var id = "ask_for_help";

/*
 * BASE
 */

	function adm_process_flow_activity(editor, node) {
		adm_process_flow_activity.uber.constructor.call(this, editor, node);
		
		var self = this;

		this.errorMessageOptions = {
			id: "errorMessage",
			title : "<%=_T('Customize error message')%>",
			channels : { 
				//CALL : true, 
				CHAT : true, 
				//FACEBOOK_BOT : true,
				//INSTAGRAM_BOT : true,
				//TWITTER_BOT : true,
				//MAIL : true,
				//SMS : true,
				WHATSAPP : true 
			},
			checkboxInput: false,
			labelClass : "flowDesignerActivityLabelBold",
			description: "<%=_T('The error message will be sent when the assistant enters an invalid option or the attention time expires')%>",
			help_info: true,
			showMaxLength: true,
			maxLength: 1000
		};

		if (!this.errorMessageOptions.channels[editor.main.data.Type]) {
			this.errorMessageOptions = null;
		} else {
            if(editor.main.data.Type !== "CALL")
            {
                this.errorMessageOptions.message_types = self.errorMessageOptions.message_types || [];
                this.errorMessageOptions.message_types.push({id : "text", name : "<%=_T('Text')%>"});
            }
        }
  
		this.output_options = [];

		this.messageDefinition = {
			id         : "question",
			name       : "<%=_T('Input variable')%>",
			placeholder: "<%=_T('Enter input variable')%>",
			description: "<%=_T('It corresponds to the question that the supervisor must answer to the bot')%>",
			required   : true,
			variable   : true,
			not_extended: true,
			controlContainerClass: "position-relative",
			span: true,
			spanClass: "ic_variable_icon_container",
			variableIconClass: "ic_variable_icon far fa-at",
			form_group: true,
			help_info: true
		};
		
		this.llmProfileDefinition = { 
			id : "llmProfile",
			name : "<%=_T('Large Language Model (LLM)')%>",
			description : "<%=_T('Large language models are a category of artificial intelligence models capable of understanding and generating text with a high degree of sophistication and accuracy')%>",
			not_extended : false,
			required : true,
			emptyLabel : "<%=_T('')%>",
			form_group: true,
			help_info: true,
			options: []
		};

		this.timeoutRetryDefinition = {
			id : "timeoutRetry",
			name : "<%=_T('Waiting time limit (seconds)')%>",
			description: "<%=_T('The time to wait for a response from the supervisor. If you exceed this limit, the request for help will end automatically and the conversation will continue with the defined flow')%>",
			not_extended : true,
			required : true,
			fieldClass : "col_4",
			options : this.__getRange(180, 900),
			validate : {
				range : {
					type : "integer",
					allowVariables : true,
					min : 180,
					max : 900
				}
			},
			emptyLabel : "<%=_T('Nothing')%>",
			defaultValue: 300,
			form_group: true,
			help_info: true
		};

		this.resultDefinition = {
			id : "result",
			name : "<%=_T('Result')%>",
			placeholder : "<%=_T('Enter result')%>...",
			required : true, 
			validate : {
				type : "id",
				message : "<%=_T('Only letters, numbers, dashes and dots are allowed in flow name.')%>"
			},
			description : "<%=_T('The variable where the message received by the supervisor is stored: { data: *message received*}')%>",
			form_group: true,
			help_info: true
		};
		
		this.allowStoringInKnowledgeBaseDefinition = {
			id: "allowStoringInKnowledgeBase",
			name: "<%=_T('Allow Storing in Knowledge Base')%>",
			description: "<%=_T('Enable the response given by the supervisor to be stored in a knowledge base')%>",
			not_extended: true,
			form_group: false,
			envolving_label_class: "form-check-label form-check-label-title",
			envolving_label: true,
			without_label: true,
			controlContainerClass: "form-check mt-0",
			checkBoxClass: "form-check-input",
			help_info: true,
			change: function () {
				self.__changeAllowStoringInKnowledgeBase($(this).prop("checked"));
			}
		};
		
		this.knowledgeBaseDefinition = {
			id : "knowledgeBase",
			name : "<%=_T('Knowledge Base')%>",
			description : "<%=_T('The knowledge base in which the response can be stored')%>",
			not_extended : false,
			required : true,
			emptyLabel : "<%=_T('')%>",
			form_group: true,
			help_info: true
		};

		this.type = editor.main.data.Type;
		this.messageId = "message";
		
		this.messageOptions = {
			title : "<%=_T('Customize waiting message')%>",
			labelClass : "flowDesignerActivityLabelBold",
			description: "<%=_T('It is an automated message that the bot will say while escalating the internal query to a person')%>",
			help_info: true
		};

		this.messagesEditor = new inConcert.adm_process_flow_messages[this.type.toLowerCase()](this);
		
		this.focusControl = this.messageDefinition;
	}

	inConcert.Utils.inherit( adm_process_flow_activity, inConcert.adm_process_flow_activities.base );

	if (typeof inConcert.adm_process_flow_activities[id] !== "function") {
		inConcert.adm_process_flow_activities[id] = adm_process_flow_activity;
	}

	adm_process_flow_activity.prototype.id = id;
	adm_process_flow_activity.prototype.name = "<%=_T('Ask for help')%>";
	adm_process_flow_activity.prototype.description = "<%=_T('Ask for help')%>";
	adm_process_flow_activity.prototype.group = "interaction";
	adm_process_flow_activity.prototype.channels = { 
		//"CALL" : true, 
		"CHAT" : true, 
		//"FACEBOOK_BOT" : true,
		//"INSTAGRAM_BOT" : true,
		//"TWITTER_BOT" : true,
		//"MAIL" : true,
		//"SMS" : true,
		"WHATSAPP" : true
	};
	adm_process_flow_activity.prototype.image = "ico-send-wait.svg";

	adm_process_flow_activity.prototype.GetMigrationData = function(data, editor) {
		let type = editor.main.data.Type;
		let messages = inConcert.adm_process_flow_messages[type.toLowerCase()];
		return messages.prototype.GetMigrationData(messages, type, {}, data);
	};

	adm_process_flow_activity.prototype.GetActivityDeclaredVariables = function(data, editor) {
		let type = editor.main.data.Type;
		let messages = inConcert.adm_process_flow_messages[type.toLowerCase()];
		return messages.prototype.GetActivityDeclaredVariables(type, data, editor);
	};

	adm_process_flow_activity.prototype.dataStyle = function(data) {
		if (data) {
			var type = data.message && data.message.msgtype || "text";
			if (type === "attachment") {
				type = data.message.fileType || "attachment";
			}
			return id + "_" + type;
		}
		return null;
	};
	
	adm_process_flow_activity.prototype.dataStyles = function(type) {
		var types = inConcert.adm_process_flow_messages_type[type.toLowerCase() + "_sorted"] || [];
		var styles = [];
		$.each(types, function(_, type) {
			styles.push({
				id : id + "_" + type,
				imageName : format("ico-send-wait-{0}-message.svg", type)
			});
		});
		["image", "video", "audio", "file"].forEach(function(fileType) {
			styles.push({
				id : id + "_" + fileType,
				imageName : format("ico-send-wait-{0}-message.svg", fileType)
			});
		});
		return styles;
	};
	
	adm_process_flow_activity.prototype.__hasKeyboardEvents = function() {
		return false;
	};
	
	adm_process_flow_activity.prototype.__isFullScreen = function() {
		return false;
	};

	adm_process_flow_activity.prototype.__changeAllowStoringInKnowledgeBase = function(checked) {

        if (checked) {
			this.knowledgeBaseDefinition.required = true;
			let knowledgeBaseDefinitionValue = this.__getValue(this.knowledgeBaseDefinition);

			this.__toggleControl(this.knowledgeBaseDefinition, false);
			this.__showControl(this.knowledgeBaseDefinition);

			if (knowledgeBaseDefinitionValue === "" || knowledgeBaseDefinitionValue === null || knowledgeBaseDefinitionValue === " ") {
				this.__setValue(this.knowledgeBaseDefinition, undefined);
			}
             
		} else {
			this.knowledgeBaseDefinition.required = false;
			this.__toggleControl(this.knowledgeBaseDefinition, true);
			this.__hideControl(this.knowledgeBaseDefinition);
			this.__setValue(this.knowledgeBaseDefinition, undefined);
		}
	};

	adm_process_flow_activity.prototype.__makeFormContent = function($content) {

		inConcert.adm_process_flow_messages_type.chat_sorted = [
			"text"
		];

		if (!this.data.message) {
			this.data.message = {};
		}

		$content.unshift(this.__makeDivFlex(this.__makeDivFlex(this.__makeInputFlex(this.resultDefinition, this.data[this.resultDefinition.id]),{ class: "col-12" }),{ class: "row" }));
		
		$content.push(
			this.__makeDivFlex(
			  [
				this.__makeDivFlex(this.__makeInputFlex(this.messageDefinition,this.data[this.messageDefinition.id]),{ class: "col-12" })
			  ],{ class: "row" }
			)
		);

		$content.push(
			this.__makeDivFlex(
			  [
				this.__makeDivFlex(this.__makeComboFlex(this.llmProfileDefinition,this.data[this.llmProfileDefinition.id] || (this.llmProfileDefinition.options.length === 2 ? this.llmProfileDefinition.options[1].id : undefined)),{ class: "col-6" }),
				this.__makeDivFlex(this.__makeInputFlex(this.timeoutRetryDefinition,this.data[this.timeoutRetryDefinition.id]),{ class: "col-6" })
			  ],{ class: "row" }
			)
		);

		$content.push(
			this.__makeHiddenDivFlex(
			  [
				this.__makeDivFlex(
				  this.__makeDivFlex(
					this.__makeCheckbox(
					  this.allowStoringInKnowledgeBaseDefinition,
					  !(this.data[this.knowledgeBaseDefinition.id] === null || this.data[this.knowledgeBaseDefinition.id] === undefined || (this.data[this.knowledgeBaseDefinition.id] + "").length === 0)
					),
					{ class: "form-group" }
				  ),{ class: "col-6" }
				)
			  ],{ class: "row" }
			)
		);
		
		$content.push(this.__makeRowFlex(this.__makeComboFlex(this.knowledgeBaseDefinition, this.data[this.knowledgeBaseDefinition.id])));

		return $content;
	};
	
	adm_process_flow_activity.prototype.__load = function(callback) {
		let self = this;

		self.editor.main.LoadLlmProfiles(function(services) {

			if (services.length === 0) {
				callback();

				setTimeout(function() {
					self.messagesEditor.ShowErrorScreen("<%=_T('You do not have an associated Long Language Model (LLM)')%>",
						"<%=_T('Please check with your inConcert administrator to use this activity.')%>");
				}, 500);

			} else {
				self.llmProfileDefinition.options = services;
				if ((self.llmProfileDefinition.options.length === 0) || 
					(self.llmProfileDefinition.options.length > 0 && self.llmProfileDefinition.options[0].id !== "")) {
					self.llmProfileDefinition.options.unshift({ id: "", name: "" });
				}

				const qnV2API = inConcert.app.getQnAV2API();
				qnV2API.getAllKnowledgeBases(0, 1000, "", function(success, response){
					if(!success){
						callback(false, "<%=_T('Error geting Knowledge Fusion Bases.')%>");
						return;
					}
					const formatedBases = response.data.map(base => {
						return { id: base.id, name: base.name };
					});

					self.knowledgeBaseDefinition.options = formatedBases;

					if ((self.knowledgeBaseDefinition.options.length === 0) || 
					(self.knowledgeBaseDefinition.options.length > 0 && self.knowledgeBaseDefinition.options[0].id !== "")) {
						self.knowledgeBaseDefinition.options.unshift({ id: "", name: "" });
					}

					if (self.data[self.knowledgeBaseDefinition.id] === null || self.data[self.knowledgeBaseDefinition.id] === undefined || (self.data[self.knowledgeBaseDefinition.id] + "").length === 0) {
						setTimeout(function() {
							self.__changeAllowStoringInKnowledgeBase(false);
						}, 500);						
					} else {
						self.__setValue(self.allowStoringInKnowledgeBaseDefinition, true);
						setTimeout(function() {
							self.__changeAllowStoringInKnowledgeBase(true);
						}, 500);
					}

					callback();
				});
			}
		});
	};

	adm_process_flow_activity.prototype.__getClassName = function() {
		return "popOver640w flowDesignerGlobalModal tabsPopOver botActivity_ask_for_help";
	};

	adm_process_flow_activity.prototype.__makeContent = function() {
		let $content = this.__makeFormContent(adm_process_flow_activity.uber.__makeContent.call(this));

		let $outputs;
		let $error_message;

		$error_message = this.__makeErrorMessage();
		this.data.type = id;
		return this.messagesEditor.Make($content, this.data[this.messageId], {}, $outputs, $error_message);	
	};
	
	adm_process_flow_activity.prototype.__makeErrorMessage = function() {

		if (!this.errorMessageOptions) {
			return null;
		}
		this.errorMessageOptions.always_enabled = true;
		this.errorMessageEditor = new inConcert.adm_process_flow_messages_embeed(
			this.messagesEditor, this.errorMessageOptions);
			
		let errorMessageObj = this.data.errorMessage;
		if (errorMessageObj && errorMessageObj.msgtype && !this.errorMessageOptions.message_types.find(msgtype => msgtype.id === errorMessageObj.msgtype)) {
			errorMessageObj = {};
		}
		return this.errorMessageEditor.Make(errorMessageObj || {});
	};

	adm_process_flow_activity.prototype.__makeValidationEditor = function(visible) {
		let self = this;
		let currentValue = self.data && self.data.message && self.data.message.validation || "";
		let $container = $(templates.validation_custom_disclaimer, {});
		self.validationEditor = ace.edit($container.find("[validation_code]")[0]);
		self.validationEditor.getSession().setMode(format("ace{0}mode{0}lua", "/"));
		self.validationEditor.renderer.setScrollMargin(8, 8);
		self.validationEditor.getSession().setValue(currentValue);

        if (!visible) {
            $container.toggle(false);
        }

		return $container;
	};
	
	adm_process_flow_activity.prototype.__afterAppendBody = function() {
		if (this.errorMessageOptions) {
			this.errorMessageEditor.AfterAppendBody();
		}
		this.messagesEditor.AfterAppendBody();
	};
	
	adm_process_flow_activity.prototype.__getAndValidateForm = function(data) {		
		this.__addValue(data, this.messageDefinition);
		this.__addValue(data, this.llmProfileDefinition);		
		this.__addValue(data, this.timeoutRetryDefinition);		
		this.__addValue(data, this.resultDefinition);
		this.__addValue(data, this.knowledgeBaseDefinition);
	};
	
	adm_process_flow_activity.prototype.__getAndValidateAsync = function(data, callback) {
		let self = this;		
		self.__getAndValidateMessage(data, function() {
            self.__getAndValidateForm(data);
			self.__getAndValidateErrorMessage(data, function() {
				callback(data);
			});
		});
	};
	
	adm_process_flow_activity.prototype.__getAndValidateMessage = function(data, callback) {
		let self = this;
		this.messagesEditor.AddValue(data, this.messageId, function() {
			data.message = data.message || {};
			self.__addValue(data.message, self.knowledgeBaseDefinition);
			callback(data);
		});
	};
	
	adm_process_flow_activity.prototype.__getAndValidateErrorMessage = function(data, callback) {
		if (!this.errorMessageEditor) {
			return callback(data);
		}
		data.errorMessage = {};
		this.errorMessageEditor.AddValue(data.errorMessage);
		if(this.getAndValidateErrorMessage && typeof this.getAndValidateErrorMessage === "function") {
			this.getAndValidateErrorMessage(data.errorMessage);
		}
		this.errorMessageEditor.AddValueAsync(data.errorMessage, function() {
			data.error = data.error || data.errorMessage.error;
			if ($.isEmptyObject(data.errorMessage)) {
				data.errorMessage = null;
			}
			callback();
		});
		
	};
		
/*
 * CALL
 */

	function adm_process_flow_activity_call(editor, node) {
		
		adm_process_flow_activity_call.uber.constructor.call(this, editor, node);
		
		this.ttsEngine = undefined;
		this.ttsVoices = undefined;
		
		this.errorMessageOptions.message_types = [
			{
				id : "attachment",
				name : "<%=_T('Attachment')%>"
			}
		];

		this.voiceDefinition = {
			id : "voiceId",
			name : "<%=_T('Voice')%>",
			description : "<%=_T('TTS voice to use.')%>",
			not_extended : false,
			fieldClass : "Short",
			sort : true,
			emptyLabel : "<%=_T('')%>",
            form_group: true
		};

	}

	inConcert.Utils.inherit( adm_process_flow_activity_call, adm_process_flow_activity );
	adm_process_flow_activity_call.prototype.Override("CALL", adm_process_flow_activity);
	
	adm_process_flow_activity_call.prototype.__load = function(callback) {
		const self = this;

		this.editor.main.LoadTtsAsrEngines(function(success, ttsasrEngines) {
			self.ttsasrEngines = success ? ttsasrEngines : undefined;
			if(self.ttsasrEngines && $.isArray(self.ttsasrEngines)) {
				self.ttsasrEngines.forEach(function(engine) {
					if((engine.providerType === "TTS" || engine.providerType === "TTS-ASR") && engine.voices && $.isArray(engine.voices) && engine.voices.length > 0) {
						self.ttsEngine = engine.engineId;
						self.ttsVoices = [];
						engine.voices.forEach(function(voice) {
							let voice_to_show = {
								id : voice.id,
								name : voice.LanguageName + " - " + voice.Variant + " - " + voice.Gender + " - " + voice.VoiceName
							};
							self.ttsVoices.push(voice_to_show);
						});
					}

				});

				self.isTtsEnabled = (self.ttsVoices && $.isArray(self.ttsVoices) && self.ttsVoices.length > 0);
				
				if(self.isTtsEnabled)
				{
					self.errorMessageOptions.message_types.push({id : "text", name : "<%=_T('Text')%>"});
				}
				
				self.voiceDefinition.options = self.ttsVoices;			
			}

			callback();
		});
	};
	
	adm_process_flow_activity_call.prototype.__makeFormContent = function($content) {
		$content = adm_process_flow_activity_call.uber.__makeFormContent.call(this, $content);

		$content.unshift(
			this.__makeDivFlex([
				this.__makeDivFlex(
					this.__makeComboFlex(this.voiceDefinition, this.data.message && this.data.message[this.voiceDefinition.id]), { class: "col-12" })
			] ,{ class: "row" })
		);

		return $content;
	};
	
	adm_process_flow_activity_call.prototype.__afterAppendBody = function() {
		adm_process_flow_activity_call.uber.__afterAppendBody.call(this);
		
		if(this.isTtsEnabled)
		{
			this.AfterSelectType();
		}
		else
		{
			this.__hideControl(this.voiceDefinition);
		}
	};
	
	adm_process_flow_activity_call.prototype.__getAndValidateFormMessage = function(message) {
		adm_process_flow_activity_call.uber.__getAndValidateFormMessage.call(this, message);

		this.__addValue(message, this.voiceDefinition);
		
		message.ttsEngine = this.ttsEngine;
	};
	
	adm_process_flow_activity_call.prototype.getAndValidateErrorMessage = function(errorMessage) {
		if(errorMessage.msgtype === "text" && this.isTtsEnabled) {
			errorMessage.voiceId = this.__getValue(this.voiceDefinition);
		}
	};
	
	adm_process_flow_activity_call.prototype.AfterSelectType = function() {
		let currentMessageType = this.messagesEditor.currentType;
		let errorMessageActive = $("[embeed_message_toggle]").is(":checked");
		let currentErrorMessageType = $("[control='msgtype']").val();
		if(currentMessageType === "text" || (errorMessageActive && currentErrorMessageType === "text"))
		{
			this.__showControl(this.voiceDefinition);
		}
		else {
			this.__hideControl(this.voiceDefinition);
		}
	};
	
	adm_process_flow_activity.prototype.htmlBody = function(data) {

		let image = "";

		if (data) {
			let type = data.message && data.message.msgtype || "text";
			if (type === "attachment") {
				type = data.message.fileType || "attachment";
			}
			image = id + "_" + type;
		}

		return Mustache.render(inConcert.templates.process_flows.activity_template, {
			image: `custom_icon_${image}`, 
			description: ((data.description && data.description !== "")?data.description:"<%=_T('Ask for help')%>") 
		});
	};

	adm_process_flow_activity.prototype.ports = function(data, node) {
		let ports = [];
		let all_outputs = { ports: [] };
		let acumulative_outputs = { ports: [] };

		// genero una lista con todos los puertos
		this.__generate_all_port_labels(data, all_outputs);

		// siempre hay que agregar este por defecto
		ports.push({
			direction : "in", // in/out
			type : "", // error/success/option
			option_label : "Input" // error/success/option
		});

		if (data.message.voiceId !== undefined) { // es un IVR
		
			// agrego puertos
			ports.push(...this.__generate_ports(data, node, all_outputs, acumulative_outputs)); // debe ir siempre primero

			// agrego un puerto por defecto
			ports.push(...this.__generate_default_port(data, node, acumulative_outputs));

			// agrego una opción para los errores
			ports.push(...this.__generate_error_ports(data, node, true));

		} else { // es un CHAT
		
			// agrego puertos
			ports.push(...this.__generate_ports(data, node, all_outputs, acumulative_outputs)); // debe ir siempre primero

			// agrego un puerto por defecto
			ports.push(...this.__generate_default_port(data, node, acumulative_outputs));

		}

		return ports;
	};

	adm_process_flow_activity.prototype.__generate_all_port_labels = function(data, all_outputs) {
		
        if (!data.ports) {
            data.ports = [];
        }

		let ports = [];
		
		data.ports.forEach(p => {

			// agrego outputs
			if (p.outputs.length > 0) {
				p.outputs.forEach(i => {
					ports.push(i);
				});
			} 
			
		});

		all_outputs.ports.push(...ports.filter(function(value, index, self) {
			return self.indexOf(value) === index;
		}));
	};

	adm_process_flow_activity.prototype.__generate_ports = function(data, node, _, acumulative_outputs) {
            
        if (!data.ports) {
            data.ports = [];
        }

		let ports = [];
		
		if (data.ports && data.ports.length > 0) {

			data.ports.forEach(p => {

				if (p.outputs.length > 0) {

					let existe = node?node.children.find(c => c.id === p.port_id):undefined;
				
					let conditions = [];

					// agrego las condiciones para los errores
					conditions.push({ logic : conditions.length > 0?"and":undefined, field: data.result + ".eventname", comparator: "~=", value: "\"Timeout\"" });
					conditions.push({ logic : conditions.length > 0?"and":undefined, field: data.result + ".eventname", comparator: "~=", value: "\"ChannelClosed\"" });
					conditions.push({ logic : conditions.length > 0?"and":undefined, field: data.result + ".eventname", comparator: "~=", value: "\"Error\"" });

					let contador = 0;

					p.outputs.forEach(i => {
						contador++;

						// incluyo aquí todos los ports que voy agregando
						acumulative_outputs.ports.push(i);

						let begin_parenths = "(";

						conditions.push({ 
							begin_parenths: (contador === 1?begin_parenths:undefined),
							logic : (contador === 1?"and":"or"), //logic, //conditions.length > 0?"and":undefined, 
							field: "decodeUnicodeEscape(" + data.result + ".event.message.text)",
							comparator: "==", 
							value: "decodeUnicodeEscape(\""+ i +"\")" ,
							end_parenths: (contador === p.outputs.length?")":undefined)
						});
					
					});
					
					ports.push({
						direction : "out", // in/out/none
						type : "option", // error/success/option
						option_label : p.name, // error/success/option
						channels: "1", // 0, 1 o n
						id: (existe?existe.id:p.port_id),
						preset_value: {
							description: "", //p.name
							conditions: conditions
						}
					});
				}

			});
		}
		
		return ports;
	};

	adm_process_flow_activity.prototype.__generate_default_port = function(data, node, acumulative_outputs) {
            
        if (!data.ports) {
            data.ports = [];
        }
        
		let ports = [];
		
		let existe = node?node.children.find(c => c.id.includes("_default")):undefined;
		let conditions = [];
		
		// agrego las condiciones para los errores
		conditions.push({ logic : conditions.length > 0?"and":undefined, field: data.result + ".eventname", comparator: "~=", value: "\"Timeout\"" });
		conditions.push({ logic : conditions.length > 0?"and":undefined, field: data.result + ".eventname", comparator: "~=", value: "\"ChannelClosed\"" });
		conditions.push({ logic : conditions.length > 0?"and":undefined, field: data.result + ".eventname", comparator: "~=", value: "\"Error\"" });

		if (acumulative_outputs.ports.length > 0) {

			let contador = 0;

			acumulative_outputs.ports.forEach(i => {

				contador++;

				conditions.push({ 
					begin_parenths: (contador === 1?"(":undefined),
					logic : ("and"), //logic, //conditions.length > 0?"and":undefined, 
					field: "decodeUnicodeEscape(" + data.result + ".event.message.text)",
					comparator: "~=", 
					value: "decodeUnicodeEscape(\""+ i +"\")" ,
					end_parenths: (contador === acumulative_outputs.ports.length?")":undefined)
				});

			});
						
		}

		ports.push({
			direction : "out", // in/out
			type : "option", // error/success/option
			option_label : "<%=_T('Default output')%>", // error/success/option
			channels: "1", // 0, 1 o n
			id: (existe?existe.id:inConcert.Utils.generateGuid() + "_default"),  // le asigno un id por defecto
			preset_value: {
				description: "", //<%=_T('Default output')%>
				conditions: conditions
			}
		});
		
		return ports;
	};

	adm_process_flow_activity.prototype.__generate_error_ports = function(data, node, unificar_errores) {
            
        if (!data.ports) {
            data.ports = [];
        }
        
		let ports = [];
		
		if (unificar_errores) {

			let existe = node?node.children.find(c => c.id.includes("_" + "Error")):undefined;

			let portId = (existe?existe.id:inConcert.Utils.generateGuid() + "_Error");

			ports.push({
				direction : "out", // in/out
				type : "error", // error/success/option
				option_label : "Error", // error/success/option
				channels: "1", // 0, 1 o n
				id: portId,  // le asigno un id por defecto
				preset_value: {
					description: "", // Error
					conditions: [
						{ field: data.result + ".eventname", comparator: "==", value: "\""+ "Error" +"\"" },
						{ logic: "or", field: data.result + ".eventname", comparator: "==", value: "\""+ "Timeout" +"\"" },
						{ logic: "or", field: data.result + ".eventname", comparator: "==", value: "\""+ "ChannelClosed" +"\"" }
					],
					port_id: portId
				}
			});

		} else {

			let errores_admitidos = ["Error","Timeout","ChannelClosed"];
			errores_admitidos.forEach(error_type => {
				let existe = node?node.children.find(c => c.id.includes("_" + error_type)):undefined;
				let portId = (existe?existe.id:inConcert.Utils.generateGuid() + "_" + error_type);
				ports.push({
					direction : "out", // in/out
					type : "error", // error/success/option
					option_label : error_type, // error/success/option
					channels: "1", // 0, 1 o n
					id: portId,  // le asigno un id por defecto
					preset_value: {
						description: "", //error_type
						conditions: [
							{ field: data.result + ".eventname", comparator: "==", value: "\""+ error_type +"\"" }
						],
						port_id: portId
					}
				});
			});

		}

		return ports;
	};

	adm_process_flow_activity.prototype.__getAndValidate = function() {	
		const data = adm_process_flow_activity.uber.__getAndValidate.call(this);
		return data;
	};

}());
