/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 9 */

/* global inConcert, ace, Mustache */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";
	var format = inConcert.Utils.format;
	var templates = inConcert.templates.process_flows;

	function viewer(data) {
		viewer.uber.constructor.call(this);
		this.data = data;
        this.isNew = true;
        this.isNewHidden = true;
	}

	inConcert.Utils.inherit( viewer, inConcert.adm_process_flow_popup );
	
	viewer.prototype.__getPopupId = function() {
		return "flowDesignerViewPopup";
	};

	viewer.prototype.__getClassName = function() {
        if (this.data.examples === "") {
            return "popOver520w flowDesignerGlobalModal fullWhitePopOver botActivity_code_block_class_desc";
        } else if (this.data.code){
            return "popOver520w flowDesignerGlobalModal fullWhitePopOver botActivity_code_block_class_desc meth";
        } else {
            return "popOver520w flowDesignerGlobalModal fullWhitePopOver botActivity_code_block_class_desc attr";
        }		
	};

	viewer.prototype.__getTitle = function() {
		return this.data.title;
	};

	viewer.prototype.__makeCode = function() {
		if (!this.data.code) {
			return;
		}
		var $container = $(templates.classes_tree_view_code);
		var codeEditor = ace.edit($container.find("[code_block]")[0]);
		codeEditor.getSession().setMode(format("ace{0}mode{0}lua", "/"));
		codeEditor.getSession().setValue(this.data.code);
		codeEditor.setReadOnly(true);
		return $container;
	};

	viewer.prototype.__makeValue = function() {
		if (!this.data.value) {
			return;
		}
		var $container = $(Mustache.render(templates.classes_tree_view_value, { text : this.data.value }));
		return $container;
	};

	viewer.prototype.__makeDescription = function() {
		if (!this.data.description) {
			return;
		}
		var $container = $(Mustache.render(templates.classes_tree_view_description, { text : this.data.description }));
		return $container;
	};

	viewer.prototype.__makeExamples = function() {
		if (!this.data.examples) {
			return;
		}
		var $container = $(templates.classes_tree_view_examples);
		var codeEditor = ace.edit($container.find("[code_block]")[0]);
		codeEditor.getSession().setMode(format("ace{0}mode{0}lua", "/"));
		codeEditor.getSession().setValue(this.data.examples);
		codeEditor.setReadOnly(true);
		return $container;
	};

	viewer.prototype.__makeContent = function() {
		var $content = [];
		$content.push(this.__makeCode());
		$content.push(this.__makeValue());
		$content.push(this.__makeDescription());
		$content.push(this.__makeExamples());
		return $content;
	};

	viewer.prototype.__hasCloseButton = function() {
		return true;
	};

	viewer.prototype.__makeActions = function() {
		return null;
	};

	var id = "code_block";

    //activity
	let guid;
	let outputList = [];
	let $listConditions = [];
	let lastGuid;

	function adm_process_flow_activity(editor, node) {
		adm_process_flow_activity.uber.constructor.call(this, editor, node);

		let self = this; 
		self.drawVersion = editor.main.__getVersion(); 
		editor.main.LoadVariablesSet();

        var data = editor.GetNodeData(node);
		this.isNew = !data;
		this.data = data || {outputs: []};
		this.frm = editor.main.frm;

        $listConditions = [];
		outputList = [];
		lastGuid = "";

		this.autocompleteVariableDefinition = {
			id: "autocompleteVariable",
			placeholder: "<%=_T('Search Variable')%>...",
			description: "<%=_T('Select an existing variable.')%>",
			with_label: false,
			with_dots: false,
			required: false,
			not_extended: true,
			form_group: true,
			options : Array.from(editor.main.variables),
			change : function () {
				self.addVariableToCodeInput();
			}
		};

		this.resultDefinition = {
			id : "result",
			name : "<%=_T('Result')%>",
			required : true, 
			validate : {
				type : "id",
				message : "<%=_T('Only letters, numbers, dashes and dots are allowed in flow name.')%>"
			},
			placeholder: "* <%=_T('Enter result')%>...", 
			description : "<%=_T('Variable name to store result ({ status : boolean, data : returned result|error message }). Code is ran inside a function so with return can store a result and with error can throw an exception.')%>",
            form_group: true
		};
	}

	inConcert.Utils.inherit( adm_process_flow_activity, inConcert.adm_process_flow_activities.base );

	if (typeof inConcert.adm_process_flow_activities[id] !== "function") {
		inConcert.adm_process_flow_activities[id] = adm_process_flow_activity;
	}

	adm_process_flow_activity.prototype.id = id;
	adm_process_flow_activity.prototype.name = "<%=_T('Code Block')%>";
	adm_process_flow_activity.prototype.description = "<%=_T('Defines a code block.')%>";
	adm_process_flow_activity.prototype.group = "variables";
	adm_process_flow_activity.prototype.image = "ico-codeblock.svg";

	
	adm_process_flow_activity.prototype.__hasKeyboardEvents = function() {
		return false;
	};
	
    adm_process_flow_activity.prototype.__getClassName = function() {        
		return "popOver940w flowDesignerGlobalModal tabsPopOver botActivity_code_block withTransitions";
	};
	
	adm_process_flow_activity.prototype.__isFullScreen = function() {
		return false;
	};
	
	adm_process_flow_activity.prototype.__load = function(callback) {
		var self = this;
		self.editor.main.GetClasses(function(classes) {
			var classes_a = [];
			var selectedClasses = self.editor.main.data.Classes || {};
			$.each(classes, function(classId, classData) {
				if (!selectedClasses[classId]) {
					return;
				}
				classes_a.push(classData);
			});
			classes_a.sort(function(a, b) {
				return a.Name.localeCompare(b.Name);
			});
			self.classes = classes;
			self.classes_a = classes_a;
			callback();
		});
	};

	adm_process_flow_activity.prototype.__makeAttribute = function(classId, attributeId, attribute) {
		var self = this;
		var $attribute = $(Mustache.render(templates.classes_tree_attribute, {
			class_id : classId,
			id : attributeId,
			name : attribute.name,
			description : self.__makeAttributeTooltip(classId, attributeId)
		}));
		
		$attribute.click(function() {
			$(this)
				.addClass("selected")
				.siblings()
				.removeClass("selected");
		});
		$attribute.find("[snippet_attribute]").click(function() {
			self.__addSnippet(self.__makeAttributeSnippet($(this).attr("class_id"), $(this).attr("snippet_attribute")));
		});
		$attribute.find("[view_attribute]").click(function() {
			self.__viewDetails(self.__makeAttributeView($(this).attr("class_id"), $(this).attr("view_attribute")), $(this));
		});
		
		return $attribute;
	};

	adm_process_flow_activity.prototype.__makeMethod = function(classId, methodId, method) {
		var self = this;
		var $method = $(Mustache.render(templates.classes_tree_method, {
			class_id : classId,
			id : methodId,
			name : method.name || "new",
			description : self.__makeMethodTooltip(classId, methodId)
		}));
		$method.click(function() {
			$(this)
				.addClass("selected")
				.siblings()
				.removeClass("selected");
		});
		$method.find("[snippet_method]").click(function() {
			self.__addSnippet(self.__makeMethodSnippet($(this).attr("class_id"), $(this).attr("snippet_method")));
		});
		$method.find("[view_method]").click(function() {
			self.__viewDetails(self.__makeMethodView($(this).attr("class_id"), $(this).attr("view_method")), $(this));
		});
		
		return $method;
	};

	adm_process_flow_activity.prototype.__makeAttributes = function(classData) {
		var self = this;
		var attributes_a = [];
		$.each(classData.Editor.attributes, function(attributeId, attribute) {
			attributes_a.push({ id : attributeId, data : attribute });
		});
		attributes_a.sort(function(a, b) {
			return a.data.name.localeCompare(b.data.name);
		});
		var $attributes = [];
		$.each(attributes_a, function(_, attribute) {
			$attributes.push(self.__makeAttribute(classData.Id, attribute.id, attribute.data));
		});
		return $attributes;
	};

	adm_process_flow_activity.prototype.__makeMethods = function(classData) {
		var self = this;
		var methods_a = [];
		$.each(classData.Editor.methods, function(methodId, method) {
			methods_a.push({ id : methodId, data : method });
		});
		methods_a.sort(function(a, b) {
			return a.data.name.localeCompare(b.data.name);
		});
		var $methods = [];
		$methods.push(self.__makeMethod(classData.Id, "constructor", classData.Editor.classConstructor));
		$.each(methods_a, function(_, method) {
			$methods.push(self.__makeMethod(classData.Id, method.id, method.data));
		});
		return $methods;
	};

	adm_process_flow_activity.prototype.__makeClass = function(classData) {
		var self = this;
		var $class = $(Mustache.render(templates.classes_tree_class, {
			id : classData.Id,
			name : classData.Name,
			description : self.__makeClassTooltip(classData.Id)
		}));

		var $attributes = self.__makeAttributes(classData);
		$class.find("[attributes_list]").append($attributes);
		if (!$attributes.length) {
			$class.find("[attributes_label]").remove();
		}

		var $methods = self.__makeMethods(classData);
		$class.find("[methods_list]").append($methods);

		$class.click(function(e) {
			var $target = $(e.target);
			if (!$target.attr("class_selector")) {
				return;
			}
			var $titleDiv = $(this).children("[class_selector_title]");
			var minHeight = $titleDiv.offset().top + $titleDiv.height() + 15;
			if (minHeight < e.pageY) {
				return;
			}

			var selected = $(this).hasClass("selected");
			self.$get("[tree]").find(".selected").removeClass("selected");
			$(this).toggleClass("selected", !selected);
		});
		$class.find("[snippet_class]").click(function() {
			self.__addSnippet(self.__makeClassSnippet($(this).attr("snippet_class")));
		});
		$class.find("[view_class]").click(function() {
			self.__viewDetails(self.__makeClassView($(this).attr("view_class")), $(this));
		});
		
		return $class;
	};

	adm_process_flow_activity.prototype.__makeClassesTree = function() {
		var self = this;
		var $tree = $(templates.classes_tree);
		var $classes = [];
		$.each(self.classes_a, function(_, classData) {
			$classes.push(self.__makeClass(classData));
		});
		$tree.find("[tree]").append($classes);
		$tree.find("[tree_toggle]").click(function() {
			self.$get("[tree_container], [content]").toggleClass("collapsed");
		});
		return $tree;
	};

	adm_process_flow_activity.prototype.__addSnippet = function(snippet) {
		this.codeEditor.getSession().insert(this.codeEditor.getCursorPosition(), snippet);
	};

	adm_process_flow_activity.prototype.__makeParametersSnippet = function(method) {
		var snippet = "";
		$.each(method.parameters || [], function(_, parameter) {
			snippet += (snippet? ", " : "") + parameter.name;
		});
		return snippet;
	};

	adm_process_flow_activity.prototype.__makeClassSnippet = function(classId) {
		var classData = this.classes[classId];
		var snippet = format("local myObject = classHelpers.{0}.new({1})", classData.Name, 
			this.__makeParametersSnippet(this.classes[classId].Editor.classConstructor));
		return snippet;
	};

	adm_process_flow_activity.prototype.__makeMethodSnippet = function(classId, methodId) {
		if (methodId === "constructor") {
			return this.__makeClassSnippet(classId);
		}
		var method = this.classes[classId].Editor.methods[methodId];
		var snippet = format("myObject:{0}({1})", method.name, this.__makeParametersSnippet(method));
		return snippet;
	};

	adm_process_flow_activity.prototype.__makeAttributeSnippet = function(classId, attributeId) {
		var attribute = this.classes[classId].Editor.attributes[attributeId];
		var snippet = format("myObject.{0}", attribute.name);
		return snippet;
	};

	adm_process_flow_activity.prototype.__makeClassTooltip = function(classId) {
		var classData = this.classes[classId];
		var tooltip = format("{0}\n{1}", this.__makeClassSnippet(classId), classData.Description);
		return tooltip;
	};
	
	adm_process_flow_activity.prototype.__makeMethodTooltip = function(classId, methodId) {
		if (methodId === "constructor") {
			return this.__makeClassTooltip(classId);
		}
		var method = this.classes[classId].Editor.methods[methodId];
		var tooltip = format("{0}\n{1}", this.__makeMethodSnippet(classId, methodId), method.description);
		return tooltip;
	};

	adm_process_flow_activity.prototype.__makeAttributeTooltip = function(classId, attributeId) {
		var attribute = this.classes[classId].Editor.attributes[attributeId];
		var tooltip = format("{0}\n{1}\n{2}", this.__makeAttributeSnippet(classId, attributeId), 
			format("<%=_T('Default value: {0}')%>", attribute.value || "nil"), attribute.description);
		return tooltip;
	};

	adm_process_flow_activity.prototype.__viewDetails = function(data, $container) {
		var popup = new viewer(data);

        if (data.examples === "") {
            popup.ShowWithContainer(undefined, $container);
        } else {            
            popup.ShowWithContainer(undefined, $container.parent());            
        }
		
	};

	adm_process_flow_activity.prototype.__makeClassView = function(classId) {
		var classData = this.classes[classId];
		var view = {
			title : classData.Name,
			description : classData.Description,
			examples : classData.Examples
		};
		return view;
	};

	adm_process_flow_activity.prototype.__makeMethodView = function(classId, methodId) {
		var method = methodId === "constructor"? this.classes[classId].Editor.classConstructor : 
				this.classes[classId].Editor.methods[methodId];
		var view = {
			title : this.__makeMethodSnippet(classId, methodId),
			description : method.description,
			examples : method.examples,
			code : method.code
		};
		return view;
	};

	adm_process_flow_activity.prototype.__makeAttributeView = function(classId, attributeId) {
		var attribute = this.classes[classId].Editor.attributes[attributeId];
		var view = {
			title : this.__makeAttributeSnippet(classId, attributeId),
			description : attribute.description,
			examples : attribute.examples,
			value : attribute.value
		};
		return view;
	};

	adm_process_flow_activity.prototype.__makeCodeEditor = function($contentBase) { 
		
        let self = this;
        let $container = $(Mustache.render(templates.code_block_content, {}));
        let $code_block = $(Mustache.render(templates.code_block, {}));

		$container.find("[content]").append(
			this.__makeDivFlex(
			this.__makeDivFlex(
				[
				this.__makeDivFlex(
					this.__makeInputFlex(
					this.resultDefinition,
					this.data[this.resultDefinition.id]
					),
					{ class: "col-6" }
				),
				this.__makeDivFlex(
					this.__makeComboFlex(
					this.autocompleteVariableDefinition,
					this.data[this.autocompleteVariableDefinition.id]
					),
					{ class: "col-6 col-autocompletecombo" }
				),
				],
				{ class: "row" }
			),
			{ class: "boxed-input-container" }
			)
		);


        $container.find("[content]").append($code_block);
        
        $($container.find("[code_block]")[0]).parent().addClass("DarkCodeBlockContainer");

		self.codeEditor = ace.edit($code_block.find("[code_block]")[0]);
		self.codeEditor.getSession().setMode(format("ace{0}mode{0}lua", "/"));
		self.codeEditor.renderer.setScrollMargin(8, 8);
		self.codeEditor.getSession().setValue(self.data.code || "");

        const $tabs = $(templates.code_block_tabs);
        
        $contentBase.push($container);
        let $codeBlock_content = $contentBase;
        
        $tabs.filter("#tabContent-codeBlock").find("#codeBlock")
            .append($codeBlock_content);
        
        if (self.drawVersion === "2") {

            $tabs.filter("#tabContent-codeBlock").find("#transitions")
                .append(this.__makeConditions(this.__makeOutputs()));

            const outputs = this.data.outputs;

            setTimeout(function() { 
                self.__disableFirstLogicOperator();
                self.__disableLastDeleteOutputButton();
            }, 100);

            $.each(outputs, function(_, output) {
                output.conditions = [];
            });

        } else {
            $tabs.find("[href='#transitions']").parent().hide();       
        }
        
        return $tabs;		
	};
	
	adm_process_flow_activity.prototype.__afterAppendBody = function() { 
		this.codeEditor.focus();
		this.__makeAutocompleteVariableDropDownInput();
		this.__addAutocompleteToInput();
	};

	adm_process_flow_activity.prototype.__addAutocompleteToInput = function () {
		const self = this;
		self.editor.main.LoadVariablesSet();

		self.$get("[flow_designer_variable_input]").each(function() { 
			self.frm.make_autocomplete(
				$(this),
				Array.from(self.editor.main.variables)
			);
		});

		$("[id*='ui-id']").css("z-index", 119);
	};

	adm_process_flow_activity.prototype.__makeAutocompleteVariableDropDownInput = function () {
		const $autocompleteInput = $("[control=autocompleteVariable]");
		$autocompleteInput.siblings("label").remove();
		$autocompleteInput.select2( {
			language: {
				noResults: function(){
					return "<%=_T('No results found.')%>";
				}
			},
			placeholder: "VARIABLES",
			templateSelection: function() {
				return "VARIABLES";
			  }
		});
	};

	adm_process_flow_activity.prototype.addVariableToCodeInput = function () {
		const self = this;
		const codeEditor = self.codeEditor;
		const $selectedVariable = $("[control=autocompleteVariable] option:selected");		
		const actualValue = codeEditor.getValue();
		const variableValue = $selectedVariable.text();
		const valueWithVariable = actualValue + " {{" + variableValue + "}}";
		codeEditor.setValue(valueWithVariable);
		self.$get("[control=autocompleteVariable]").val(null);
	};

	adm_process_flow_activity.prototype.__makeContent = function() { 
        let $contentBase = adm_process_flow_activity.uber.__makeContent.call(this);
		return $contentBase;
	};

	adm_process_flow_activity.prototype.__makeBody = function() {
		var $popup = adm_process_flow_activity.uber.__makeBody.call(this);
        var $content = this.__makeCodeEditor([]);

        $popup.find("[content]")
			.removeAttr("content")
            .parent()
            .append($content);

        $popup.find("#popup_content").remove();

        $popup.find("[content]")
            .parent()
            .append(this.__makeClassesTree());

        $popup.find(".card.ovf-a").remove();

        return $popup;

	};

	adm_process_flow_activity.prototype.__getAndValidate = function() { 
        let data = adm_process_flow_activity.uber.__getAndValidate.call(this);
		this.__addValue(data, this.resultDefinition);
		data.code = this.codeEditor.getValue();
		if (!data.code) {
			data.error = "<%=_T('Must enter a code.')%>";
		}
		else {
			$.each(this.codeEditor.getSession().getAnnotations(), function(_, annotation) {
				if (annotation.type === "error") {
					data.error = "<%=_T('Invalid lua code.')%>";
					return false;
				}
			});
		}

        data.outputs = data.outputs || [];

		if (this.drawVersion === "2") {

			if (!data.error) {
				if (this.__getAndValidateOutputTitles(data)) {
                    this.__getAndValidateConditions(data);
				}
			}
		}

		return data;
	};
	
	adm_process_flow_activity.prototype.htmlBody = function (data) {
		return Mustache.render(inConcert.templates.process_flows.activity_template, {
			image: `custom_icon_code_block_text`,
			description: ((data.description && data.description !== "") ? data.description : "<%=_T('Code Block')%>")
		});
	};

    // se arman las transiciones
    adm_process_flow_activity.prototype.__makeOutputs = function() {
		const self = this;
		guid = (inConcert.Utils.generateGuid()).toLowerCase();
		let outputs = self.data.outputs || [];
		let $outputs = $(Mustache.render(templates.code_block_transitions, {id: guid}));
		let $listOutputs = [];

		$.each(outputs, function(_, output) {
            if (output && output.title) {
                $listOutputs.push($outputs.find("[div_variables]").append(self.__makeOutput(output, undefined, $outputs)));
            }
		});
		
		if (!$listOutputs.length) {
			const $initialOutput = self.__makeOutput(undefined, guid, $outputs);
			const $initialCondition = self.__makeCondition($initialOutput, guid);
			$initialCondition.find("[logic]").prop("disabled", "disabled");
			$initialCondition.find("[remove]").prop("disabled", "disabled");
			$listOutputs.push($outputs.find("[div_variables]").append($initialOutput));
			$listConditions.push($initialCondition);

			$outputs.find(`#condition_list_${guid}`).find("[condition_list]")
				.sortable({
					cursor : "move"
				})
			.append($initialCondition);

			self.__addAutocompleteToInput();
		} 

		$outputs.find("[add_new_output]").click(function () {
			const $newOutput = self.__makeOutput(undefined, undefined, $outputs);
			const outputId = $newOutput.attr("id"); 
			const $newCondition = self.__makeCondition($newOutput, outputId);
			$listConditions.push($newCondition);
			$newCondition.find("[logic]").prop("disabled", "disabled");
			$newCondition.find("[remove]").prop("disabled", "disabled");
			$outputs.find(`#condition_list_${outputId}`).find("[condition_list]")
				.sortable({
					cursor : "move"
				})
			.append($newCondition);

			self.__addAutocompleteToInput();

            setTimeout(function() { 
                self.__disableFirstLogicOperator();
                self.__disableLastDeleteOutputButton();
            }, 100);
		});

		return $outputs;
	};

	adm_process_flow_activity.prototype.__makeConditions = function($outputs) {
		const self = this;
		const outputs = self.data.outputs || [];
		let outputId;
		let $conditionList;

		$.each(outputs, function(_, output) {
			outputId = output.port_id;
            output.conditions = self.__getValidateInitialParenths(output.conditions);
			$.each(output.conditions, function(_, condition) {
				$conditionList = $outputs.find(`#condition_list_${outputId}`).find("[condition_list]");
				let $condition = self.__makeCondition(condition, outputId);

				self.__disableFirstLogicOperator();

				// $conditionList.append($condition);
				$listConditions.push($conditionList);	
				$outputs.find(`#condition_list_${outputId}`).find("[condition_list]")
					.sortable({
						cursor : "move"
					})
				.append($condition);

				self.__addAutocompleteToInput();
			});
		});

		return $outputs;
	};

    adm_process_flow_activity.prototype.__getValidateInitialParenths = function(conditions) {

		let parenths_bk = [];
        let parenths = [];

        let leftEqual_last = false;
        let rightEqual_last = false;

        try {
            conditions.forEach(c => {
                c.begin_parenths.trim().split("").forEach(p => {
                    parenths_bk.push(p);
                    parenths.push(p);
                });
                c.end_parenths.trim().split("").forEach(p => {
                    parenths_bk.push(p);
                    parenths.push(p);
                });
            });

            leftEqual_last = false;
            rightEqual_last = false;

            let index = -1;
            parenths.forEach(p1 => {
                index++;
                if (p1 === ")") {
					let i = index;
                    for(i=index;i>0;i--) {
                        if (parenths[i] === "(") {

                            let leftEqual = true;
                            for (let ix = (parenths_bk.length / 2) - 1; ix >= 0; ix--) {
                                if (parenths[ix] !== "(" && parenths[ix] !== "") {
                                    leftEqual = false;
                                    break;
                                }
                            }

                            // Validar hacia la derecha
                            let rightEqual = true;
                            for (let iy = (parenths_bk.length / 2) + 1; iy < parenths.length; iy++) {
                                if (parenths[iy] !== ")" && parenths[iy] !== "") {
                                    rightEqual = false;
                                    break;
                                }
                            }

                            leftEqual_last = leftEqual;
                            rightEqual_last = rightEqual;

                            if (!(leftEqual && rightEqual)) {
                                parenths[i] = "";
                                parenths[index] = "";

                                break;
                            } else {
                                break;
                            }
                            
                        }
                    }
                }
            });

            const parenths_filtered = parenths.filter((e) => e !== "");

            if (leftEqual_last && rightEqual_last && parenths_filtered[0] !== parenths_filtered[parenths.length - 1] && conditions.length >= 1) {
                conditions[0].begin_parenths = conditions[0].begin_parenths.substring(1);
                conditions[conditions.length - 1].end_parenths = conditions[conditions.length - 1].end_parenths.substring(1);
            }
        } catch (e) {
            console.error(e);
        }

		if (conditions.length === 1) {
			const c_conditions = conditions.slice(); // Clonamos el arreglo original
			const firstElement = c_conditions[0];
			const hasOpeningParenthesis = (firstElement.begin_parenths || "").length > 0;
			const hasClosingParenthesis = (firstElement.end_parenths || "").length > 0;
		  
			if (hasOpeningParenthesis && hasClosingParenthesis) {
			  const firstElementWithoutOpeningParenthesis = {
				...firstElement,
				begin_parenths: (firstElement.begin_parenths || "").slice(1),
				end_parenths: (firstElement.end_parenths || "").slice(1)
			  };
		  
			  const conditionsWithoutParentheses = [
				firstElementWithoutOpeningParenthesis
			  ];

			  let openParentheses = 0;
			  let closedParentheses = 0;
		  
			  for (let i = 0; i < conditionsWithoutParentheses.length; i++) {
				const { begin_parenths, end_parenths } = conditionsWithoutParentheses[i];
		  
				if (begin_parenths.length > 0) {
                    openParentheses += begin_parenths.length;
				}
		  
				if (end_parenths.length > 0) {
                    closedParentheses += end_parenths.length;
				}
		  
				if (closedParentheses > openParentheses) {               
                    return conditions;
				}
			  }
		  
			  if (openParentheses === closedParentheses) {
				return conditionsWithoutParentheses;
			  }
			}
		} else if (conditions.length > 1) {
			const c_conditions = conditions.slice(); // Clonamos el arreglo original
			const firstElement = c_conditions[0];
			const lastElement = c_conditions[c_conditions.length - 1];
			const hasOpeningParenthesis = (firstElement.begin_parenths || "").length > 0;
			const hasClosingParenthesis = (lastElement.end_parenths || "").length > 0;
		  
			if (hasOpeningParenthesis && hasClosingParenthesis) {
			  const firstElementWithoutOpeningParenthesis = {
				...firstElement,
				begin_parenths: (firstElement.begin_parenths || "").slice(1),
			  };
		  
			  const ultimoElementoSinParentesisFin = {
				...lastElement,
				end_parenths: (lastElement.end_parenths || "").slice(0, -1),
			  };

			  const conditionsWithoutParentheses = [
				firstElementWithoutOpeningParenthesis,
				...c_conditions.slice(1, -1),
				ultimoElementoSinParentesisFin,
			  ];

			  let openParentheses = 0;
			  let closedParentheses = 0;
		  
			  for (let i = 0; i < conditionsWithoutParentheses.length; i++) {
				const { begin_parenths, end_parenths } = conditionsWithoutParentheses[i];
		  
				if (begin_parenths.length > 0) {
                    openParentheses += begin_parenths.length;
				}
		  
				if (end_parenths.length > 0) {
                    closedParentheses += end_parenths.length;
				}
		  
				if (closedParentheses > openParentheses) {
                    return conditions;
				}
			  }
		  
			  if (openParentheses === closedParentheses) {
				return conditionsWithoutParentheses;
			  }
			}
		}

        return conditions;

	};

	adm_process_flow_activity.prototype.__makeOutput = function (output, initialGuid, $outputs) {

		const self = this;

		let $variable_output;
		let outputListLength;
		(initialGuid) ? guid = initialGuid : guid = (inConcert.Utils.generateGuid()).toLowerCase();
		(output) ? outputList.push(output.port_id) : outputList.push(guid);
		(outputList.length !== 0) ? outputListLength = outputList.length : outputListLength = 2;

		if (output) { 
			$variable_output = $(Mustache.render(templates.code_block_transition_output, {id: output.port_id}));
			$outputs.find("[div_variables]").append($variable_output);
			$variable_output.find("[output_name]").val(output.title || "");
		} 
		else {	
			$variable_output = $(Mustache.render(templates.code_block_transition_output, {id: guid}));
			$outputs.find("[div_variables]").append($variable_output);
		}

		$variable_output.find("[output_name]").on("keyup change", function () {
			// UPDATE OUTPUT NAME
			const id = $(this).attr("id");
			const idGuid = id.substring(30, id.length);
			$(`[condition_link_name_${idGuid}]`).html($(this).val());
			Object.keys(self.data.outputs).forEach((key) => {
				if (self.data.outputs[key].port_id === idGuid) {
					self.data.outputs[key].title = $(this).val();
				}
			});
		});

		$variable_output.find("[output_remove]").click(function () {
			// DELETE
			var $row = $(this).parent().parent().parent();
			var outputId = $(this).attr("id");
			const remove = outputList.find(id => id === outputId);
			outputList = $.grep(outputList, function(value) {
				return value !== remove;
			});
			$(`#${remove}`).remove();				
			$row.remove();
			$("[condition_link_order]").each(function(index) { 
				$(this).html(`${index + 1}. `); 
			});

            setTimeout(function() { 
                self.__disableFirstLogicOperator();
                self.__disableLastDeleteOutputButton();
            }, 100);
		});		

		if (output) {
			self.__makeConditionLink(outputListLength, output.port_id, $outputs, output.title);
		}
		else {
			self.__makeConditionLink(outputListLength, guid, $outputs);

            self.data.outputs = self.data.outputs || [];
			self.data.outputs.push( { port_id : guid, title: $($(`[condition_link_name_${guid}]`)), conditions: [] } );
		}
			
		return $variable_output;
	};

	adm_process_flow_activity.prototype.__makeCondition = function(condition, guid) {
		var self = this;
		var $condition = $(Mustache.render(templates.code_block_transition_output_condition, {id: guid}));

		if (condition) {
			$condition.find("[logic]").val(condition.logic || "");
			$condition.find("[begin_parenths]").val(condition.begin_parenths || "");           
            let indexLeftParenth = (condition.field?condition.field:"").indexOf("(");
            let indexRightParenth = (condition.field?condition.field:"").indexOf(")");
            let variableName = (condition.field?condition.field:"").slice(indexLeftParenth + 1, indexRightParenth);
            if ((self.data.result + ".data") === variableName) {
                variableName = variableName.replace(".data", "");
            }

            let indexLeftParenthValue = (condition.value?condition.value:"").indexOf("(");
            let indexRightParenthValue = (condition.value?condition.value:"").indexOf(")");
            let variableValue = (condition.value?condition.value:"").slice(indexLeftParenthValue + 1, indexRightParenthValue);

			if ((self.data.result + ".data") === variableValue) {
                variableValue = variableValue.replace(".data", "");
            }

			$condition.find("[field_name]").val(variableName || "");
			$condition.find("[comparator]").val(condition.comparator || "");
			$condition.find("[field_value]").val(variableValue || "");
			$condition.find("[end_parenths]").val(condition.end_parenths || "");
		}
		$condition.find("[add]").click(function() {
			self.__disableNextKeyboardEvent();
			var $row = $(this).parent().parent();
			var $newRow = self.__makeCondition(undefined, guid);
			$row.after($newRow);
			self.__addAutocompleteToInput();
			$newRow.find("[logic]").focus();
			self.__disableFirstLogicOperator();
		});
		$condition.find("[remove]").click(function() {
			self.__disableNextKeyboardEvent();
			var $row = $(this).parent().parent();
			var $siblings = $row.siblings();
			if ($siblings.length) {
				if ($row.prev().length) {
					$row.prev().find("[logic]").focus();
				}
				else {
					$row.next().find("[logic]").focus();
				}
				$row.remove();
			}
			self.__disableFirstLogicOperator();
		});

		return $condition;
	};

	adm_process_flow_activity.prototype.__makeConditionLink = function(outputListLength, guid, $outputs, title) {
		var self = this;
		let linkTitle;
		(title) ? linkTitle = title : linkTitle = ""; 
		let outputLink =`<li condition_link id='${guid}' class='list-inline-item flowDesignerContentChip'>
							<span condition_link_order>${outputListLength}. </span> 
							<span class="flowDesignerContentChipName" condition_link_name_${guid}>${linkTitle}</span>
						</li>`;
		$(outputLink).insertBefore($outputs.find("#default_output"));
		
		$outputs.find(`#${guid}`).click(function () {
			if (this.tagName === "DIV") {
				return;
			}
			$("[condition_link]").each(function() { 
				$(this).removeClass("selected");
			});
			$(this).addClass("selected");

			try {
				self.$get(".tab-pane.fade.active.show").scrollTo(self.$get(`[activity_output_${guid}]`), {
                    offset: -150,
                    duration: 750
                  });
			}
			catch (e) {}		
		});
	};

    adm_process_flow_activity.prototype.__getAndValidateOutputTitles = function(data) {

		var self = this;
		let outputNameList = [];
		let validated = true;

		self.$get("[output]").each(function() {
			let outputName = $(this).find("[output_name]").val();
			if (outputName && outputName !== "") {
				outputNameList.push(outputName);
			}
		});

		const orderedOutputNames = outputNameList.sort(); 

		self.$get("[output]").each(function(_) {
			
			self.__getAndValidateOutput($(this), data);

			var namesDuplicated = [];
			for (var i = 0; i < orderedOutputNames.length - 1; i++) {
				if (orderedOutputNames[i + 1] === orderedOutputNames[i]) {
					namesDuplicated.push(orderedOutputNames[i]);
				}
			}
			if (namesDuplicated.length > 0) {
				data.error = {
					message : "<%=_T('Output names must be different from each other')%>"
				};	
				validated = false;
				return false;
			}
		});
		
		return validated;
	};

    adm_process_flow_activity.prototype.__getAndValidateOutput = function($output, data) {
		const id = $output.attr("id");
		const name = $output.find("[output_name]").val();
		const output = {
			port_id : id || undefined, 
			title: name, 
			conditions: [] 
		};
		data.outputs.push(output);
		return true;
	};

    adm_process_flow_activity.prototype.__disableFirstLogicOperator = function (_) {
        var self = this;
		self.$get("[condition_list]").each(function(_, item) {
			let lastOutputId;
			$(item).find("[condition]").each(function(index2) {
				let outputId = $(this).attr("id");
				$(this).find("[logic]").prop("disabled", false);
				$(this).find("[remove]").prop("disabled", false);
				if (index2 === 0 || lastOutputId !== outputId) {
					$(this).find("[logic]").prop("disabled", "disabled");
					$(this).find("[logic]").val("");
					$(this).find("[remove]").prop("disabled", $(item).find("[condition]").length > 1 ? false : "disabled");
				}
				lastOutputId = outputId;
			});
			
		});
	};

    adm_process_flow_activity.prototype.__disableLastDeleteOutputButton = function () {
		let deleteOutputs = $("[div_variables]").find("[output_remove]");
		if(deleteOutputs.length < 2) {
			$(deleteOutputs[0]).prop("disabled", true);
		}
		else {
			deleteOutputs.each(function(index) {
				$(deleteOutputs[index]).prop("disabled", false);
			});	
		}
	};

	adm_process_flow_activity.prototype.__getAndValidateConditions = function(data) {

		var self = this;
        var parenths = 0;
		self.__disableFirstLogicOperator();
		self.$get("[condition]").each(function(index) {
            parenths += self.__getAndValidateCondition(index, $(this), data);
		});

        if (!data.error && parenths !== 0) {
            data.error = {
                message : "<%=_T('Error validating parenths.')%>"
            };
        }
        parenths = 0;
        if (data.error) {
            return false;
        }
	};

	adm_process_flow_activity.prototype.__getAndValidateCondition = function(index, $condition, data) {

		const guid = $condition.find("[condition_output]").attr("id");
        const condition = {
            title: $condition.parent().parent().parent().parent().find("[output_name]").val() || undefined,
			logic : $condition.find("[logic]").val() || undefined,
			begin_parenths : $condition.find("[begin_parenths]").val() || undefined,
			field : $condition.find("[field_name]").val() || undefined,
			comparator : $condition.find("[comparator]").val() || undefined,
			value : $condition.find("[field_value]").val() || undefined,
			end_parenths : $condition.find("[end_parenths]").val() || undefined
		};
		//empty
		if (!condition.logic && !condition.begin_parenths && !condition.field && !condition.comparator && 
			!condition.value && !condition.end_parenths && !condition.title) 
		{
			return 0;
		}
		//validation
		var error = null;
		if (!condition.title) {
			error = "<%=_T('Error on output name.')%>";
		}
        if (!condition.value) {
			error = "<%=_T('Must enter a value.')%>";
		}
		if (!condition.comparator) {
			error = "<%=_T('Must enter a comparator.')%>";
		}
		if (!condition.field) {
			error = "<%=_T('Must enter a field.')%>";
		}
		if (index > 0 && (!condition.logic && guid === lastGuid)) {
			error = "<%=_T('Must enter a logical operator.')%>";
		}
		lastGuid = guid;
		if (error) {
			data.error = {
				message : error,
				$control : $condition
			};
		}

		data.outputs.find((output) => output.port_id === guid).conditions.push(condition);
		return (condition.begin_parenths || "").length - (condition.end_parenths || "").length;
	};

	adm_process_flow_activity.prototype.ports = function(data, node) { 
		

		let ports = [];
		let acumulative_outputs = { ports: [] };

		// siempre hay que agregar este por defecto
		ports.push({
			direction : "in", // in/out
			type : "", // error/success/option
			option_label : "Input" // error/success/option
		});

		// agrego puertos
		ports.push(...this.__generate_ports(data, node, acumulative_outputs)); // debe ir siempre primero

		// agrego un puerto por defecto
		ports.push(...this.__generate_default_port(data, node, acumulative_outputs));

		// agrego una opción para los errores
		ports.push(...this.__generate_error_ports(data, node, true));

		data.outputs = data.outputs || [];
		data.outputs.forEach(output => {
            output.conditions = this.__getValidateInitialParenths(output.conditions);
        });
		
		return ports;
	};

	adm_process_flow_activity.prototype.__generate_ports = function(data, node, acumulative_outputs) { 
		
		let ports = [];

		if (data.outputs && data.outputs.length > 0) {

			data.outputs.forEach(b => {

                if (b && b.title) {
                    let existe = node?node.children.find(c => c.id === b.port_id):undefined;
                    let portId = (existe?existe.id:b.port_id);

                    let conditions = [];
                    let contador = 0;
                    conditions.push({ logic : conditions.length > 0?"and":undefined, field: data.result + ".status", comparator: "==", value: "true" });
                    
                    b.conditions.forEach(c => {
                        contador++;
                        c.begin_parenths = (contador === 1?("(" + (c.begin_parenths || "")):(c.begin_parenths || ""));
                        c.logic = (contador === 1?"and":c.logic.toLowerCase());
						c.field = c.field.replace("tostring(", "decodeUnicodeEscape(");

                        if (data.result === c.field) {
                            c.field = ((c.field.includes("decodeUnicodeEscape"))?(c.field):("decodeUnicodeEscape(" + c.field + ".data)"));
                        } else {
                            c.field = ((c.field.includes("decodeUnicodeEscape"))?(c.field):("decodeUnicodeEscape(" + c.field + ")"));
                        }
                        
						if (data.result === c.value) {
                            c.value = ((c.value.includes("decodeUnicodeEscape"))?(c.value):("decodeUnicodeEscape(" + c.value + ".data)"));
                        } else {
                            c.value = ((c.value.includes("decodeUnicodeEscape"))?(c.value):("decodeUnicodeEscape(" + c.value + ")"));
                        }

                        c.end_parenths = (contador === b.conditions.length?(")" + (c.end_parenths || "")):(c.end_parenths || ""));
                    });

                    conditions.push(...b.conditions);

                    acumulative_outputs.ports.push(b.title?b.title:b.message); // esto hay que replantearlo

                    ports.push({
                        direction : "out", // in/out/none
                        type : "option", // error/success/option
                        option_label : b.title?b.title:b.message, // error/success/option
                        channels: "1", // 0, 1 o n
                        id: portId,
                        preset_value: {
                            description: "", //p.name
                            conditions: conditions,
                            port_id: portId
                        }
                    });
                }

			});
		}
		
		return ports;
	};
	
	adm_process_flow_activity.prototype.__generate_default_port = function(data, node, acumulative_outputs) { 

		let ports = [];
		
		let existe = node?node.children.find(c => c.id.includes("_default")):undefined;
		let conditions = [];

		conditions.push({ logic : conditions.length > 0?"and":undefined, field: data.result + ".status", comparator: "==", value: "true" });
		
		if (acumulative_outputs.ports.length > 0) {

			let contador = 0;

			acumulative_outputs.ports.forEach(i => {

				contador++;
				data.result = data.result.replace("tostring(", "decodeUnicodeEscape(");
				conditions.push({ 
					begin_parenths: (contador === 1?"(":undefined),
					logic : "and", //logic, //conditions.length > 0?"and":undefined, 
					field: ((data.result.includes("decodeUnicodeEscape"))?(data.result):("decodeUnicodeEscape(" + data.result + ".data)")),
					comparator: "~=", 
					value: ((i.includes("decodeUnicodeEscape"))?(i):("decodeUnicodeEscape(" + i + ")")),
					end_parenths: (contador === acumulative_outputs.ports.length?")":undefined)
				});

			});
			
		}

        let portId = (existe?existe.id:inConcert.Utils.generateGuid() + "_default");
		ports.push({
			direction : "out", // in/out
			type : "option", // error/success/option
			option_label : "<%=_T('Default output')%>", // error/success/option
			channels: "1", // 0, 1 o n
			id: portId,  // le asigno un id por defecto
			preset_value: {
				description: "", //<%=_T('Default output')%>
				conditions: conditions,
				port_id: portId
			},
			intern_value: undefined
		});
		
		return ports;
	};

	adm_process_flow_activity.prototype.__generate_error_ports = function(data, node, _) { 
		
		let ports = [];
		
		let existe = node?node.children.find(c => c.id.includes("_" + "Error")):undefined;
		let portId = (existe?existe.id:inConcert.Utils.generateGuid() + "_Error");

		ports.push({
			direction : "out", // in/out
			type : "error", // error/success/option
			option_label : "Error", // error/success/option
			channels: "1", // 0, 1 o n
			id: portId,  // le asigno un id por defecto
			preset_value: {
				description: "", // Error
				conditions: [
					{ field: data.result + ".status", comparator: "~=", value: "true" }
				],
				port_id: portId
			}
		});

		return ports;
	};

}());
