/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 9 */

/* global inConcert, Mustache */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	const FLOW_ID = "execute_luis";
	const COGNITIVE_SERVICES_ID = "cognitiveServicesApp";
	const FORMAT = inConcert.Utils.format;

	function adm_process_flow_activity(editor, node) {
		adm_process_flow_activity.uber.constructor.call(this, editor, node);

		var self = this;
		self.frm = editor.main.frm;

		let botName = this.editor.main.GetBotType();
		let translatedBotName;
		if (botName === "LUIS") { 
			translatedBotName = FORMAT("<%=_T('NLU {0} App')%>", botName);
		}
		else {
			translatedBotName = "<%=_T('NLU App')%>";
		}

		this.cognitiveServicesDefinition = {
			id         : COGNITIVE_SERVICES_ID,
			name       : translatedBotName,
			description: translatedBotName,
			sort       : true,
			required   : true,
			form_group: true,
			change: function () {
				self.__selectCognitiveService($(this).val());
			}
		};

		this.cognitiveServiceIntents = [];

		this.messageDefinition = {
			id         : "message",
			name       : "<%=_T('Input variable')%>",
			placeholder: "<%=_T('Enter input variable')%>",
			description: FORMAT("<%=_T('Message to be sent to {0} api.')%>", botName),
			required   : true,
			variable   : true,
			not_extended: true,
			controlContainerClass: "position-relative",
			span: true,
			spanClass: "ic_variable_icon_container",
			variableIconClass: "ic_variable_icon far fa-at",
			form_group: true
		};

		this.resultDefinition = {
			id         : "result",
			name       : "<%=_T('Variable for result')%>",
			placeholder: "<%=_T('Enter variable for result')%>",
			description: FORMAT("<%=_T('Variable name to store result (object with {0} response).')%>", botName),
			required   : true,
			form_group: "true"
		};

		this.scoringDefinition = { 
			id         : "luis_scoring",
			name       : "<%=_T('Scoring')%>", 
			description: FORMAT("<%=_T('Variable name to store result (object with {0} response).')%>", botName)
		};

		this.focusControl = this.cognitiveServicesDefinition;
	}

	inConcert.Utils.inherit( adm_process_flow_activity, inConcert.adm_process_flow_activities.base );

	if (typeof inConcert.adm_process_flow_activities[FLOW_ID] !== "function") {
		inConcert.adm_process_flow_activities[FLOW_ID] = adm_process_flow_activity;
	}

	adm_process_flow_activity.prototype.__selectCognitiveService = function (app) { 
		var self = this;
		self.__loadIntents(app, function() {
			setTimeout(function() { 
				const $metadata = $(inConcert.templates.process_flows.execute_luis_metadata);
				self.loadIntents($metadata, function() {});
			}, 100);
		});
	};

	adm_process_flow_activity.prototype.id = FLOW_ID;
	adm_process_flow_activity.prototype.name = "<%=_T('Execute NLU API')%>";
	adm_process_flow_activity.prototype.description = "<%=_T('Executes a NLU API method.')%>";
	adm_process_flow_activity.prototype.group = "middleware";
	adm_process_flow_activity.prototype.image = "ico-execute-luis.svg";

	adm_process_flow_activity.prototype.GetMigrationData = function(data) {
		return {
			cognitiveServicesApp: data[COGNITIVE_SERVICES_ID]
		};
	};

	adm_process_flow_activity.prototype.MigrateData = function(data, migration) {

		if (data[COGNITIVE_SERVICES_ID] && migration.cognitiveServicesApp &&
			migration.cognitiveServicesApp[data[COGNITIVE_SERVICES_ID]]
		) {
			data[COGNITIVE_SERVICES_ID] = migration.cognitiveServicesApp[data[COGNITIVE_SERVICES_ID]];
		}
	};

	adm_process_flow_activity.prototype.__load = function(callback) { 
		const self = this;
		const drawVersion = self.editor.main.__getVersion();
		this.editor.main.LoadBotApps(function(services) {

			self.cognitiveServicesDefinition.options = services;

			if (services.length >= 1 && drawVersion === "2") {
				self.__loadIntents(self.data.cognitiveServicesApp?self.data.cognitiveServicesApp:services[0].Id, callback);
			} else {
				callback();
			}
		});

	};

	adm_process_flow_activity.prototype.__loadIntents = function(app_id, callback) { 

		var self = this;

		self.editor.main.LoadBotLuisIntents(app_id, function(intents) { 

			self.cognitiveServiceIntents = intents;


			callback();
		});
	};

	adm_process_flow_activity.prototype.__getClassName = function() {
		const drawVersion = this.editor.main.__getVersion(); 
		let flowDesignerPopupClasses = "popOver940w flowDesignerGlobalModal tabsPopOver botActivity_execute_luis";
		if (drawVersion !== "2") {
			flowDesignerPopupClasses = "popOver940w flowDesignerGlobalModal botActivity_execute_luis";
		}

		return flowDesignerPopupClasses;
	};

	adm_process_flow_activity.prototype.__afterAppendBody = function() { 
		const self = this;
		const $inputs = self.$get("span[class=ic_variable_icon_container");
		self.__addAutocompleteToInput($inputs);
	};

	adm_process_flow_activity.prototype.__makeContent = function() { 

		const $content = adm_process_flow_activity.uber.__makeContent.call(this); 
		const drawVersion = this.editor.main.__getVersion(); 

		if (drawVersion === "2") {
			
			const $contentMensaje = adm_process_flow_activity.uber.__makeContent.call(this);

			$contentMensaje.unshift(
				this.__makeDivFlex(
				  [
					this.__makeDivFlex(
					  [
						this.__makeDivFlex(
						  [
							this.__makeDivFlex(
							  this.__makeComboFlex(
								this.cognitiveServicesDefinition,
								this.data[this.cognitiveServicesDefinition.id]
							  ),
							  { class: "col-6" }
							),
							this.__makeDivFlex(
							  this.__makeInputFlex(
								this.messageDefinition,
								this.data[this.messageDefinition.id]
							  ),
							  { class: "col-6" }
							),
						  ],
						  { class: "row" }
						),

						this.__makeDivFlex(
							[
							  this.__makeDivFlex(
								this.__makeInputFlex(
								  this.resultDefinition,
								  this.data[this.resultDefinition.id]
								),
								{ class: "col-12" }
							  ),
							],
							{ class: "row" }
						  ),
			  
						this.__makeScoring(this.data[this.scoringDefinition.id]),
					  ],
					  { class: "card-block" }
					),
				  ],
				  { class: "card no-card" }
				)
			  );

			this.__makeRow(this.__makeComboFlex(this.cognitiveServiceIntents, this.data[this.cognitiveServiceIntents.id])); 


			let $contentConfiguration = [];

			this.__processing(true); /* #12989 */

			$contentConfiguration.unshift(this.__makeIntents());
			

			const $tabs = $(inConcert.templates.process_flows.luis_tabs);

			$tabs.filter("#tabContent-nlu").find("#message")
				.append($contentMensaje);
			$tabs.filter("#tabContent-nlu").find("#configuration")
				.append($contentConfiguration);
			
			setTimeout(function() { 
				this.tabs = new inConcert.conf_form_tab({
					parent       : $content,
					container_id : "flow_designer_luis_tabs",
					tabs_list    : {
						message : {
							id               : "message",
							name             : "<%=_T('Configuration')%>",
							tab_container_id : "flow_designer_luis_tabs_message",
							selected         : true,
							initial_state    : "enabled"
						},
						configuration : {
							id               : "configuration",
							name             : "<%=_T('Outputs')%>", 
							tab_container_id : "flow_designer_luis_tabs_configuration",
							initial_state    : "enabled"
						}
					}
				});

				this.tabs.show_tabs_container();
				this.tabs.switch_tab("message");

			}, 500);

			return $tabs;

		} else {
			$content.unshift(
				this.__makeDivFlex(
				  [
					this.__makeDivFlex(
					  [
						this.__makeDivFlex(
							[
							  this.__makeDivFlex(
								this.__makeComboFlex(
								  this.cognitiveServicesDefinition,
								  this.data[this.cognitiveServicesDefinition.id]
								),
								{ class: "col-12" }
							  ),
							],
							{ class: "row" }
						  ),
						  this.__makeDivFlex(
							[
							  this.__makeDivFlex(
								this.__makeInputFlex(
								  this.messageDefinition,
								  this.data[this.messageDefinition.id]
								),
								{ class: "col-12" }
							  ),
							],
							{ class: "row" }
						  ),
						  this.__makeDivFlex(
							[
							  this.__makeDivFlex(
								this.__makeInputFlex(
								  this.resultDefinition,
								  this.data[this.resultDefinition.id]
								),
								{ class: "col-12" }
							  ),
							],
							{ class: "row" }
						  )
					  ],
					  { class: "card-block" }
					),
				  ],
				  { class: "card no-card" }
				)
			  );
			return $content;
		}

	};

	adm_process_flow_activity.prototype.__makeScoring = function(scoring) { 
		
		const $metadata = $(inConcert.templates.process_flows.execute_luis_scoring);

		let $slider_value = $metadata.find(".ui-slider-scoring-value");
		$metadata.find("[control=global_scoring]").slider({
			range: "min",
			min: 0,
			max: 1,
			value: scoring?scoring:0.50,
			step: 0.01,
			create: function() {
				$slider_value.text(Math.floor(($(this).slider("value")) * 100) + "%"); /* #12989 */
				$metadata.find(".ui-slider-scoring-input-value").val($(this).slider("value"));
			},
			slide: function(_, ui) {

				// $handle.text("");
				$slider_value.text(Math.floor((ui.value) * 100) + "%"); /* #12989 */
				$metadata.find(".ui-slider-scoring-input-value").val(ui.value);
			}
		});
		
		return $metadata;
	};

	adm_process_flow_activity.prototype.__getAndValidate = function(flexible) { 
		
		const data = adm_process_flow_activity.uber.__getAndValidate.call(this);
		const drawVersion = this.editor.main.__getVersion(); 

		if (drawVersion === "2") {

			if (!data.error) {
				this.__getAndValidateFields(data, flexible);
			}
			
			this.__addValue(data, this.cognitiveServiceIntents);
			this.__addValue(data, this.scoringDefinition);
			
		}

		this.__addValue(data, this.cognitiveServicesDefinition);
		this.__addValue(data, this.messageDefinition);
		this.__addValue(data, this.resultDefinition);

		return data;
	};

	adm_process_flow_activity.prototype.__getAndValidateField = function($field, data) { /* #12989 */

		data.ports = data.ports || [];

		let field = {
			name: $field.find("[field_name]").val() || undefined,
			port_id: $field.attr("template_intent") || undefined,
			intents: []
		};

		//validation
		let error = null;
		if (field.name) {
			if (!field.name) {
				error = "<%=_T('Must enter a name.')%>";
			}
			let name = inConcert.Utils.htmlBodyToPlainText(field.name);
			if (name.trim().length === 0) {
				error = "<%=_T('Invalid name.')%>";
			}
		} 
		else {
			error = "<%=_T('Must enter a name.')%>";
		}

		$field.find("[ic-data-bind=intent]").each(function() {
			let $field_i = $(this);

			let value = $field_i.text();
			let is_template = $field_i.attr("ic-role");

			if (is_template !== "template" && is_template !== "template-qr") {
				field.intents.push(value);
			}
		});

		// esta validación es para que al menos agregue un intent de LUIS en al menos 1 puerto																					   
		if (!field.intents || field.intents.length === 0) {
			error = "<%=_T('You must add at least 1 Intent')%>"; /* #12989 */
		}

		if (error) {
			data.error = {
				message : error,
				$control: $field
			};
		}

		data.ports.push(field);

	};

	adm_process_flow_activity.prototype.__getAndValidateFields = function(data, flexible) { 

		const self = this;
		this.$get("[field]").each(function() {
			self.__getAndValidateField($(this), data);

			if (data.error && !flexible) {
				return false;
			}
		});

		// valido el slide scoring 
		this.__addValue(data, this.scoringDefinition); 
	};

	adm_process_flow_activity.prototype.__makeIntents = function() { 
		
		const self = this;
		let ports = [];

		if (this.data.ports) {
			ports = this.data.ports || [];			
		}

		const $metadata = $(inConcert.templates.process_flows.execute_luis_metadata);

		$metadata.find("[add_new_port]").click(function() {
			self.__disableNextKeyboardEvent();
			let $newRow = self.__makeField($metadata);
			$("#output_list").append($newRow);
			$newRow.find("[field_name]").focus();
			// actualizo el evento de drag and drop
			self.updateDragListener($metadata);
		});

		const $intentsList = $metadata.find("[fields]");
		const $list = [];

		$.each(ports, function(_, field) {
			$list.push(self.__makeField($metadata,field));
		});
		if (!$list.length) {
			$list.push(self.__makeField($metadata));
		}

		$intentsList.append($list);

		setTimeout(function() { 
			self.loadIntents($metadata, function() {});
		}, 100);

		// $metadata.find("[add_output]").click(function() { /* #12989 */
		// 	self.__disableNextKeyboardEvent();
		// 	let $row = $(".flowDesignerLuisMetadataFieldsList").children(":last");
		// 	let $newRow = self.__makeField($metadata);
		// 	$row.after($newRow);
		// 	$row.parent().scrollTo($newRow);
		// 	$newRow.find("[field_name]").focus();
		// 	// actualizo el evento de drag and drop
		// 	self.updateDragListener($metadata);
		// });
		return $metadata;
	};

	adm_process_flow_activity.prototype.__makeField = function($metadata, field) { /* #12989 */

		field = field || {};
		let self = this;
		let $field = $(Mustache.render(inConcert.templates.process_flows.execute_luis_metadata_intent_field, { 
			guid : field.port_id?field.port_id:inConcert.Utils.generateGuid(), 
			hidden : true,
            placeholder: "<%=_T('Enter the name of the output port')%>"
		}));

		if ($(".flowDesignerLuisMetadataField").length === 0 || field.name !== undefined) {
            $field.find("[field_name]").val(field.name || "Port " + ($(".flowDesignerLuisMetadataField").length + 1));
        }

		$field.find("[remove]").click(function() {
			
			self.__disableNextKeyboardEvent();
			let $row = $(this).parent().parent().parent().parent();
			let $siblings = $row.siblings();

			// acualizo la definición de los puertos

			let current_data = self.__getAndValidate(true); 
			self.data.ports = current_data.ports;

			// elimino el puerto actual en caso de que exista
			for (let index = 0; index < self.data.ports.length; index++) {
				const p = self.data.ports[index];
				
				if ($field.attr("template_intent") === p.port_id) {
					self.data.ports.splice(index, 1);
					break;
				}
			}
			
			if ($siblings.length) {
				if ($row.prev().length) {
					$row.prev().find("[field_name]").focus();
				}
				else {
					$row.next().find("[field_name]").focus();
				}
			}
			else {
				var $newRow = self.__makeField($metadata);
				$row.after($newRow);
				$newRow.find("[field_name]").focus();
			}
			$row.remove();

			// actualizo los intents
			self.loadIntents($metadata, function() {});
			
		});
		return $field;
	};

	/* MANEJO DE DRAG & DROP */

	adm_process_flow_activity.prototype.loadIntents = function($metadata, callback) /* #12989 */
	{
		const self = this;

		try {

			let context = {
				listIntentsAvail    : $("#rd_luis_available_intents_list"),
				listIntentsInUse    : $metadata.find("[ic-data-bind=intents]")
			};

			this.context = $.extend(context, this.context);

			function renderSummary (callback) {

				self.setupIntentsSelector({
					callback  : function() {
						self.updateDragListener($metadata);
						callback(true);
					},
					update : function() {

						context.listIntentsAvail
							.children()
							.off("mouseenter")
							.one("mouseenter",function(){
								$(this).addClass("sorting-initialize");
								context.listIntentsAvail.sortable("refresh");
							});

						context.listIntentsInUse
							.children()
							.off("mouseenter")
							.one("mouseenter",function(){
								$(this).addClass("sorting-initialize");
								context.listIntentsInUse.sortable("refresh");
							});
					}
				});
			} 

			renderSummary(function(success, errorThrown) {
				if (success) {
					callback(true, errorThrown);
				} else {
					callback(false, errorThrown);
				}
			});

		} finally {
			self.__processing(false);
		}			
	};

	adm_process_flow_activity.prototype.setupIntentsSelector = function(params) { 

		var self = this;
		let constructor_callback = params.callback || $.noop;
		let update               = params.update   || $.noop;
		let $list_container      = $("#rd_luis_intent_selector").children("[ic-role='list-container']");
		let $intents_available   = $list_container.find("[ic-role='intents-available']");

		// limpio los intents existentes antes de hacer todo esto
		let template = $intents_available.children("[ic-role='template']");
		$intents_available.empty().append(template);
		
		// también limpio los puertos existentes
		$("#output_list").find(".ui-intents-receive").each(function () {
			let $intents_receive = $(this).find("[ic-data-bind='intents']");
			let template = $intents_receive.children("[ic-role='template']");
			$intents_receive.empty().append(template);
		});


		// agrego los intents a los puertos
		inConcert.Utils.foreach_timeout({
			rows : self.cognitiveServiceIntents,
			max_append_per_loop: 2,
			pause_between_loop_ms: 10,
			append_fn: function( item )
			{
				let existe = false;

				if (self.data.ports) {

					self.data.ports.forEach(p => {
						if (p.intents) {
							p.intents.forEach(i => {

								if (i === item.name) {

									existe = true;
									$("#output_list").find(".ui-intents-receive").each(function () {
										if ($(this).attr("template_intent") === p.port_id) {

											let existe_intent = false;

											if (!existe_intent) {
												// let $intents_receive = $(this).children("[ic-data-bind='intents']");
												let $intents_receive = $(this);

												let $template_html_receive = $("<div>").append(
													$intents_receive.children("[ic-role='template']")
														.clone()
														.show()
														.removeAttr("ic-role")
														.attr("ic-data-bind", "intent")
													).html();
			
													let item_html = $template_html_receive.replace("{name}", item.name);
												
												return $(item_html).appendTo($intents_receive);
											} else {
												return;
											}											
										}
									});
								}
							});
						}
					});

				}

				if (!existe) {

					let existe_intent = false;

					if (!existe_intent) { 

						let $template_html_available = $("<div>").append(
							$intents_available.children("[ic-role='template']")
								.clone()
								.show()
								.removeAttr("ic-role")
								.attr("ic-data-bind", "intent")
							).html();

							let item_html = $template_html_available.replace("{name}", item.name);	
						return $(item_html).appendTo($intents_available);
					} else {
						return;
					}
					
				}
			},
			complete_fn: function()
			{
				constructor_callback();
				update();
			}
		});

		return {
			reset: function() {}
		};		
	};

	adm_process_flow_activity.prototype.updateDragListener = function($metadata) { 

		let context = {
			listIntentsAvail    : $("#rd_luis_available_intents_list"), 
			listIntentsInUse    : $metadata.find("[ic-data-bind=intents]")
		};

		context.listIntentsAvail.sortable({
			cursor: "move",
			connectWith: ".ui-intents-receive",
			items: ".sorting-initialize",
			receive: function( event, ui ) {
				//--ui.item.children('[ic-inner-data-bind="alias"]').hide();
			}
		});

		context.listIntentsInUse.sortable({
			cursor: "move",
			connectWith: "#" + context.listIntentsAvail.uniqueId().attr("id"),
			items: ".sorting-initialize",
			receive: function( event, ui ) {
				//--ui.item.children('[ic-inner-data-bind="alias"]').show();
			}
		});
		
	};


	adm_process_flow_activity.prototype.htmlBody = function(data) {
		return Mustache.render(inConcert.templates.process_flows.activity_template, {
			image: `custom_icon_${FLOW_ID}`, 
			description: ((data.description && data.description !== "")?data.description:"<%=_T('Execute Luis API')%>") 
		});
	};

	adm_process_flow_activity.prototype.ports = function(data, node) {
			

		let ports = [];

		// siempre hay que agregar este por defecto
		ports.push({
			direction : "in", // in/out
			type : "", // error/success/option
			option_label : "Input" // error/success/option
		});

		// agrego puertos
		if (data.ports && data.ports.length > 0) {
			data.ports.forEach(p => {

				let existe = node?node.children.find(c => c.id === p.port_id):undefined;	
				let conditions = [];

				// agrego las condiciones para los errores
				conditions.push({ logic : conditions.length > 0?"and":undefined, field: data.result + ".eventname", comparator: "~=", value: "\"Error\"" });

				// agrego condición para el scoring
				conditions.push({ logic : conditions.length > 0?"and":undefined, field: data.result + ".topScoringIntent.score", comparator: ">=", value: data.luis_scoring });

				let contador_intents = 0;

				p.intents.forEach(i => {
					contador_intents++;

					conditions.push({ 
						begin_parenths: (contador_intents === 1?"(":undefined),
						logic : (contador_intents === 1?"and":"or"), //logic, //conditions.length > 0?"and":undefined, 
						field: data.result + ".topScoringIntent.intent", 
						comparator: "==", 
						value: "\""+ i +"\"" ,
						end_parenths: (contador_intents === p.intents.length?")":undefined)
					});

				});

				let portId = (existe?existe.id:p.port_id);
				ports.push({
					direction : "out", // in/out/none
					type : "option", // error/success/option
					option_label : p.name, // error/success/option
					channels: "1", // 0, 1 o n
					id: portId,
					preset_value: {
						description: "", //p.name
						conditions: conditions,
						port_id: portId
					}
				});
			});
		}

		// Genero un puerto por defecto
		let existe = node?node.children.find(c => c.id.includes("_default")):undefined;
		let conditions = [];

		// agrego las condiciones para los errores
		conditions.push({ logic : conditions.length > 0?"and":undefined, field: data.result + ".eventname", comparator: "~=", value: "\"Error\"" });

		// agrego condición para cuando no se cumpla con el scoring
		conditions.push({ logic : conditions.length > 0?"and":undefined, field: data.result + ".topScoringIntent.score", comparator: "<", value: data.luis_scoring });

		// recorro los intents de todos los puertos para crear una condición excluyente a todas
		let contador_intents = 0;
		let total_intents = 0;

		data.ports.forEach(p1 => {
			total_intents += p1.intents.length;
		});

		data.ports.forEach(p => {
			p.intents.forEach(i => {
				contador_intents++;

				conditions.push({ 
					begin_parenths: (contador_intents === 1?"(":undefined),
					logic : (contador_intents === 1?"or":"and"),
					field: data.result + ".topScoringIntent.intent", 
					comparator: "~=", 
					value: "\""+ i +"\"" ,
					end_parenths: (contador_intents === total_intents?")":undefined)
				});

			});
		});
		let portId = (existe?existe.id:inConcert.Utils.generateGuid() + "_default");
		ports.push({
			direction : "out", // in/out
			type : "option", // error/success/option
			option_label : "<%=_T('Default output')%>", // error/success/option
			channels: "1", // 0, 1 o n
			id: portId,  // le asigno un id por defecto
			preset_value: {
				description: "", //<%=_T('Default output')%>
				conditions: conditions,
				port_id: portId
			}
		});

		// agrego una opción para los errores
		let unificar_errores = true;
		
		if (unificar_errores) {

			let existe = node?node.children.find(c => c.id.includes("_" + "Error")):undefined;
			let portId = (existe?existe.id:inConcert.Utils.generateGuid() + "_" + "Error");
			ports.push({
				direction : "out", // in/out
				type : "error", // error/success/option
				option_label : "Error", // error/success/option
				channels: "1", // 0, 1 o n
				id: portId,  // le asigno un id por defecto
				preset_value: {
					description: "", //"Error"
					conditions: [
						{ field: data.result + ".eventname", comparator: "==", value: "\""+ "Error" +"\"" }
					],
					port_id: portId
				}
			});

		} else {

			let errores_admitidos = ["Error"];
			errores_admitidos.forEach(error_type => {
				let existe = node?node.children.find(c => c.id.includes("_" + error_type)):undefined;
				let portId = (existe?existe.id:inConcert.Utils.generateGuid() + "_" + error_type);
				ports.push({
					direction : "out", // in/out
					type : "error", // error/success/option
					option_label : error_type, // error/success/option
					channels: "1", // 0, 1 o n
					id: portId,  // le asigno un id por defecto
					preset_value: {
						description: "", //error_type
						conditions: [
							{ field: data.result + ".eventname", comparator: "==", value: "\""+ error_type +"\"" }
						],
						port_id: portId
					}
				});
			});

		}
		return ports;
	};

}());
