/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 9 */

/* global inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";
	var id = "http_rest_full";
	const templates = inConcert.templates.process_flows;

	function adm_process_flow_activity(editor, node) {
		adm_process_flow_activity.uber.constructor.call(this, editor, node);

		//Inicializamos metadata si no existe
		this.data.ports = this.data.ports || [];
		this.data.metadata = this.data.metadata || {};
		this.data.metadata.query = this.data.metadata.query || [];
		this.data.metadata.headers = this.data.metadata.headers || [];
		this.data.metadata.mapping = this.data.metadata.mapping || [];
		this.data.metadata.body = this.data.metadata.body || "";
		this.data.metadata.bodyFormat = this.data.metadata.bodyFormat || "json";
		this.data.metadata.bodyType = this.data.metadata.bodyType || "text";
		this.data.metadata.bodyParameters = this.data.metadata.bodyParameters || [];

		this.data.result = this.data.result || `G_${inConcert.Utils.generateGuid()}`;


		this.MethodEnum = [
			"GET", "POST", "HEAD", "DELETE", "PUT", "PATCH"
		]
		this.HeadersEnum = [
			"Accept",
			"Accept-Charset",
			"Accept-Encoding",
			"Accept-Language",
			"Authorization",
			"Content-Encoding",
			"Content-Language",
			"Content-Lenght",
			"Content-Type",
			"Cookie",
			"Proxy-Authorization",
			{
				key:"<%=_T('Custom')%>",
				value:"Custom"
			}

		]

	}

	inConcert.Utils.inherit(adm_process_flow_activity, inConcert.adm_process_flow_activities.base);

	if (typeof inConcert.adm_process_flow_activities[id] !== "function") {
		inConcert.adm_process_flow_activities[id] = adm_process_flow_activity;
	}


	adm_process_flow_activity.prototype.__getClassName = function () {
		return "flowDesignerPopupRestful tabsPopOver popOver563w flowDesignerGlobalModal";
	};

	/*
		El evento de la actividad cuando se pulsa Enter
	*/
	adm_process_flow_activity.prototype.__onEnter = function () {
		let self = this;
		let defaultExecute = function(){
			if (self.$get(":focus").is("textarea")) {
				return;
			}
			self.Save();
		}
		self.alternativeEnterHandler ? self.alternativeEnterHandler():defaultExecute();
	};
	/*
		El evento de la actividad cuando se pulsa Escape
	*/
	adm_process_flow_activity.prototype.__onEscape = function () {
		let self = this;
		let defaultExecute = function(){
			self.Close();
		}

		self.alternativeEscapeHandler ? self.alternativeEscapeHandler():defaultExecute();
	};

	adm_process_flow_activity.prototype.id = id;
	adm_process_flow_activity.prototype.name = "<%=_T('HTTP API RESTful')%>";
	adm_process_flow_activity.prototype.description = "<%=_T('HTTP API RESTful.')%>";
	adm_process_flow_activity.prototype.group = "middleware";
	adm_process_flow_activity.prototype.image = "ico-jump-to-state.svg";

	/*
		Genera el contenido principal de la actividad.
		Consiste en 3 tabs, Request, Response y Outputs.

		En Request, está la información general y de petición
		En Response la configuración de mapping de la respuesta
		En Outputs, el ruteo de salidas segun el status que devuelva la API
	*/
	adm_process_flow_activity.prototype.__makeContent = function () {

		var $content = adm_process_flow_activity.uber.__makeContent.call(this);

		let principalInput = this.__makeMethodAndUrl();
		$content.unshift(principalInput);
		$content.push(this.__makeRequestMetadata(principalInput));

		let $tabs = $(templates.http_restfull_tabs);//Template de tabs

		$tabs.filter("#tabContent-restfull").find("#flow_designer_http_api_restfull_request_tab").append($content);
		$tabs.filter("#tabContent-restfull").find("#flow_designer_http_api_restfull_response_tab").append(this.__makeResponseMetadata());
		$tabs.filter("#tabContent-restfull").find("#flow_designer_http_api_restfull_ports_tab").append(this.__makePortsConfiguration());

		return $tabs;
	};

	/*
		Crea el formulario que incluye la información general, esto es el verbo
		de la petición y la url
	*/
	adm_process_flow_activity.prototype.__makeMethodAndUrl = function (value) {
		let self = this;
		const guid = inConcert.Utils.generateGuid();

		let $field = $(Mustache.render(templates.http_restfull_metadata_input, { guid: guid }));

		let el = document.createElement("option");
		el.textContent = "";
		el.value = "";
		$field.find("[field_method]")[0].appendChild(el);
		$.each(this.MethodEnum, function (_, option) {
			el = document.createElement("option");
			el.textContent = option;
			el.value = option;
			$field.find("[field_method]")[0].appendChild(el);
		});
		$field.find("[field_method]").change((ev)=>{
			let method = $field.find("[field_method]").val();
			let haveBody = (method !== "GET") && (method !== "");
			self.__showBodyParameters(haveBody);
			if(haveBody){
				let contentType = self.$get("[flow_designer_http_api_body_request_type_radio_set]").find("input[name=Type]:checked").val();
				if(contentType === "text"){
					contentType = self.$get("[select_content_type_body_parameters]").val();
				}
				self.__makeAutoContentType(contentType);
			}
			
		})

		
		$field.find("[field_method]").val(this.data.method|| "");
		$field.find("[field_url]").val(this.data.url || "");

		return $field;
	};

	/*
		Crea el formulario donde se puede configurar el ruteo entre el status
		del response de la API y una salida de la actividad.
	*/
	adm_process_flow_activity.prototype.__makePortsConfiguration = function(){
		let self = this;
		//console.log("data", this.data)
		let $field = $(Mustache.render(templates.http_restfull_tab_ports,{}));

		if(this.data.ports.length > 0){
			this.data.ports.forEach((port)=>{
				$field.find("[ports_container]").append(self.__makePortField(port,$field));
			})
		}else{
			$field.find("[ports_container]").append(self.__makePortField({
				id: inConcert.Utils.generateGuid(),
				name:'<%=_T("Success")%>',
				status:[200,204]
			}, $field));
		}

		$field.find("[add_new_port]").click(function(){
			$field.find("[ports_container]").append(self.__makePortField(undefined,$field));
		})

		$field.find("[status_field]").val(this.data.result);

		//Damos acción al mensaje informativo closeable
		if(this.data.dragAndDropDisclaimerHide){
			$field.find("[disclaimer]").remove();
		}else{
			$field.find("[close_message]").click(function(){
				$field.find("[disclaimer]").remove();
			})
		}
		

		return $field;
	};
	/*
		Crea un formulario de puerto nuevo para la parte de configuración de puertos
	*/
	adm_process_flow_activity.prototype.__makePortField = function(port, parent){
		let self = this;
		let searchNextName = function(){
			let previousPorts = self.__getAndValidatePortsConfiguration({}).ports
			let tempNum = 1;
			let tempName = `Port ${tempNum}`;
			let free = false;
			while(!free){
				if(previousPorts.find((port)=>{
					return port.name === tempName;
				})){
					tempNum++;
					tempName = `Port ${tempNum}`;
				}else{
					free = true;
				}
			}
			return tempName;
		}
		port = port || {
			id:inConcert.Utils.generateGuid(),
			name: searchNextName(),//Debería ir siendo autoincremental
			status:[]
		};

		let $field = $(Mustache.render(templates.http_restfull_port_field,port));

		let otherPorts = parent.find("[field]");
		if(otherPorts.length === 0){
			//Desabilitamos el botón de eliminar si este va a ser el único puerto
			$field.find("[delete]").attr("disabled",true);
		}else{
			//Habilitamos todos los delete, pues este no será el único puerto que haya
			parent.each((_,field)=>{
				$(field).find("[delete]").attr("disabled",false);
			})
		}

		let empty_label_callback = function(){
			if($field.find("[status_list]").find("[template]").length > 0){
				$field.find("[status_list]").find("[label_empty]").remove();
			}else{
				let label_empty = document.createElement("label");
				label_empty.classList.add("required");
				label_empty.innerText = "<%=_T([[Add Status Code]])%>";
				label_empty.setAttribute("label_empty","");
				$field.find("[status_list]").append(label_empty);
			}
		}

		$field.find("[field_name]").val(port.name);

		$field.find("[add]").click(function(){
			$field.find("[status_list]").append(self.__makePortStatus(undefined,empty_label_callback));
			empty_label_callback();
		})

		$field.find("[delete]").click(function(){
			let fields = self.$get("[ports_container]").find("[field]");
			if(fields.length > 1){
				$field.remove();
				if((fields.length -1) === 1){
					//Si solo queda un puerto, deshabilitamos el puerto que queda
					self.$get("[ports_container]").find("[field]").find("[delete]").attr("disabled",true);
				}
			}
			
		})

		$field.on("dragover", function(ev){
			ev.originalEvent.preventDefault();
			ev.originalEvent.dataTransfer.effectAllowed = "move";
		})
		$field.on("drop", function(ev){
			ev.originalEvent.preventDefault();
			let status = ev.originalEvent.dataTransfer.getData("text/plain");
			//console.log("DROP STATUS", status)
			$field.find("[status_list]").append(self.__makePortStatus(status,empty_label_callback));
			empty_label_callback();
		})

		port.status.forEach((status)=>{
			$field.find("[status_list]").append(self.__makePortStatus(status,empty_label_callback));
		})

		empty_label_callback();

		return $field;
	};

	/*
		Crea un campo de status (arrastrable) que se puede mover entre puertos
	*/
	adm_process_flow_activity.prototype.__makePortStatus = function(status, onDeleteCallback){
		status = status || 200;
		let localBuffer = {
			prevValue: status
		};
		let $field = $(Mustache.render(templates.http_restfull_port_status))
		$field.find("[field_status]").val(status);
		$field.find("[delete_status]").click(function(){
			$field.remove();
			onDeleteCallback();
		})

		$field.find("[field_status]").change((ev)=>{
			let newValue = $field.find("[field_status]").val();
			if(parseInt(newValue) <= 599 && parseInt(newValue) >= 100 ){
				localBuffer.prevValue = newValue;
			}else{
				$field.find("[field_status]").val(localBuffer.prevValue);
			}
		})
		$field.find("[field_status]").focus((ev)=>{
			localBuffer.prevValue = $field.find("[field_status]").val();
		})

		$field.attr("draggable", true);
		$field.on("dragstart", function(ev){
			let element = $(this).clone().html(`<span>${status}</span>`);
			element.css(
				{
					"width":"100px",
					"height":"50px",
					"position": "absolute",
					"pointerEvents": "none",
					"font-weight": 600
				}
			).appendTo($field);
			ev.originalEvent.dataTransfer.effectAllowed  = "move";
			//ev.originalEvent.dataTransfer.items.push($field);
			ev.originalEvent.dataTransfer.setDragImage(element[0],10,25);
			ev.originalEvent.dataTransfer.setData("text/plain",$field.find("[field_status]").val())
			setTimeout(()=>{

				element.remove();
				$field.remove();
				onDeleteCallback();
			})
			
		})

		return $field;
	};

	/*
		Asiste la cabecera Content-Type dependiendo del tipo de cuerpo que se
		selecciona
	*/
	adm_process_flow_activity.prototype.__makeAutoContentType = function(type){
		let translator = {
			"json":"application/json",
			"xml/html": "application/xml",
			"plain": "text/plain",
			"form": "application/x-www-form-urlencoded"
		}
		let self = this;
		let existContentTypeHeader = false;
		self.$get("[new_request_header]").each((_,field)=>{
			let selectPointer = $(field).find("[select_header]");
			if(selectPointer.val().toLowerCase() === "custom"){
				//Comprobamos el input de tipo text
				if($(field).find("[input_custom_header]").val().toLowerCase() === "content-type"){
					existContentTypeHeader = true;
					$(field).find("[input_header]").val(translator[type]);
				}
			}else if(selectPointer.val().toLowerCase() === "content-type"){
				existContentTypeHeader = true;
				$(field).find("[input_header]").val(translator[type]);
			}
		})
		if(!existContentTypeHeader){
			//Hay que añadirlo
			let $newRow = self.__makeRequestHeader({key:"Content-Type",value:translator[type]});
			self.$get("[add_new_request_header]").parent().after($newRow);
		}
	}

	/*
		Esta función se llama al inicio de carga de la actividad y
		carga los valores que pueda tener guardados de antemano con respecto a
		la request:
		query parameters, headers y body.
	*/
	adm_process_flow_activity.prototype.__makeRequestMetadata = function ($field) {
		const self = this;

		const $metadata = $(templates.http_restfull_request_card);

		let setRequestBodyMode = function({code}){
			if(code){
				//Vista de modo code
				$metadata.find("#flow_designer_http_api_body_request_text_type_radio").attr("checked",true);
				$metadata.find("#flow_designer_http_api_body_request_form_type_radio").attr("checked",false);
				$metadata.find("#select_content_type_body_parameters_container").css({display:"inherit"});
				$metadata.find("[body_parameters_type_form_btn]").removeClass("d-flex justify-content-end").addClass("d-none");
				$metadata.find("[add_new_body_parameter]").css({display:"none"});
				$metadata.find("#flow_designer_add_http_api_body_request_parameters_container").css({display:"none"});
				$metadata.find("#flow_designer_add_http_api_code_block_container").css({display:"inherit"});
			}else{
				//Vista de modo formulario
				$metadata.find("#flow_designer_http_api_body_request_text_type_radio").attr("checked",false);
				$metadata.find("#flow_designer_http_api_body_request_form_type_radio").attr("checked",true);
				$metadata.find("#select_content_type_body_parameters_container").css({display:"none"});
				$metadata.find("[body_parameters_type_form_btn]").removeClass("d-none").addClass("d-flex justify-content-end");
				$metadata.find("[add_new_body_parameter]").css({display:"inherit"});
				$metadata.find("#flow_designer_add_http_api_body_request_parameters_container").css({display:"inherit"});
				$metadata.find("#flow_designer_add_http_api_code_block_container").css({display:"none"});
			}
		}

		let setAceModeSwitchMetadata = function(selectValue, session){
			switch(selectValue){
				case "json":
					setRequestBodyMode({code:true});
					session.setMode(inConcert.Utils.format("ace{0}mode{0}javascript", "/"));
					break;
				case "xml/html":
					setRequestBodyMode({code:true});
					session.setMode(inConcert.Utils.format("ace{0}mode{0}xml", "/"));
					break;
				case "form":
					setRequestBodyMode({code:false});
					break;
				case "plain":
					setRequestBodyMode({code:true});
					session.setMode();
					break;
			}
		}

		//Inicializamos ACE en el codeblock de body parameters
		let codeEditor = ace.edit($metadata.find("[code_block]")[0]);
		let codeSession = codeEditor.getSession();
		setAceModeSwitchMetadata(self.data.metadata.bodyFormat, codeSession);
		setRequestBodyMode({code:self.data.metadata.bodyType==="text"});
		codeEditor.renderer.setScrollMargin(8, 8);
		//codeSession.setMode(inConcert.Utils.format("ace{0}mode{0}javascript", "/"));
		if(self.data.metadata.bodyFormat === "json"){
			if(self.data.metadata.body !== ""){
				codeSession.setValue(JSON.stringify(JSON.parse(self.data.metadata.body),null, '\t'));
			}
		}else if(self.data.metadata.bodyFormat === "form"){
			if(self.data.metadata.body !== ""){
				let vars = self.data.metadata.body.split("&");
				if(Array.isArray(vars)){
					vars.forEach((variable)=>{
						if(variable.split("=").length > 1){
							$metadata.find("#flow_designer_add_http_api_body_request_parameters_container").append(
								self.__makeBodyRequestFormTypeInput({key: variable.split("=")[0], value: variable.split("=")[1]})
							);
						}
					});
				}

			}
		}else{
			codeSession.setValue(self.data.metadata.body);
		}




		//Establecemos el valor del tipo de body parameters
		$metadata.find("[select_content_type_body_parameters]").val(self.data.metadata.bodyFormat);
		//Añadimos el evento de cambio cuando se cambia el content type de body parameters
		$metadata.find("[select_content_type_body_parameters]").change(()=>{
			let contentFormat = $metadata.find("[select_content_type_body_parameters]").val();
			setAceModeSwitchMetadata(contentFormat,codeSession);
			self.__makeAutoContentType(contentFormat);
		})

		$metadata.find("[flow_designer_http_api_body_request_type_radio_set]").change((ev)=>{
			let val = $metadata.find("input[name=Type]:checked").val();
			setRequestBodyMode({code:val==="text"});
			self.__makeAutoContentType(val==="text"?$metadata.find("[select_content_type_body_parameters]").val():val);
			//console.log(ev);
		})

		$metadata.find("[add_new_body_parameter]").click(function(){
			$metadata.find("#flow_designer_add_http_api_body_request_parameters_container").append(
				self.__makeBodyRequestFormTypeInput()
			);

		})

		//const $list = [];
		self.__showBodyParameters = function(show){
			$metadata.find("[body_parameters_container]").css("display",show?"inherit":"none");
			$metadata.find("[body_parameters_separator]").css("display",show?"inherit":"none");
		}

		self.__showBodyParameters(this.data.method && this.data.method !== "GET" && this.data.method !== "");

		$metadata.find("[add_new_request_header]").click(function () {
			self.__disableNextKeyboardEvent();
			var $row = $(this).parent();
			var $newRow = self.__makeRequestHeader();
			$row.after($newRow);
			$row.parent().scrollTo($newRow);
			$newRow.find("[field_name]").focus();
		});

		$metadata.find("[add_new_query_parameters]").click(function () {
			self.__disableNextKeyboardEvent();
			var $row = $(this).parent();
			var $newRow = self.__makeQueryParameters();
			$row.after($newRow);
			$row.parent().scrollTo($newRow);
			$newRow.find("[field_name]").focus();
		});



		if (self.data.metadata.query.length > 0) {
			self.data.metadata.query.forEach((parameter)=>{
				let $newRow = self.__makeQueryParameters(parameter);
				($metadata.find("[add_new_query_parameters]")).parent().after($newRow);
			});
		}

		if (self.data.metadata.headers.length > 0) {
			self.data.metadata.headers.forEach((header)=>{
				let $newRow = self.__makeRequestHeader(header);
				($metadata.find("[add_new_request_header]")).parent().after($newRow);
			});
		}

		if(self.data.metadata.bodyParameters.length > 0){
			self.data.metadata.bodyParameters.forEach((bodyParameter)=>{
				$metadata.find("#flow_designer_add_http_api_body_request_parameters_container").append(
					self.__makeBodyRequestFormTypeInput(bodyParameter)
				);
			});
		}
		// Crear por defecto un parametro vacío si no tiene parametros, de momento está quitado
		// Pues es posible que no quieras mandar un body, y aun que tengas seleccionado el tipo
		// Formulario, es posible que quieras dejarlo vacío
		// else{
		// 	$metadata.find("#flow_designer_add_http_api_body_request_parameters_container").append(
		// 		self.__makeBodyRequestFormTypeInput()
		// 	);
		// }

		return $metadata;
	};

	/*
		Crea un nuevo input clave/valor para los Request Body de tipo formulario
	*/
	adm_process_flow_activity.prototype.__makeBodyRequestFormTypeInput = function(value){
		let self = this;
		const $metadata = $(templates.http_restfull_bodyRequest_form_type_variable_input);

		$metadata.find("[delete_parameter]").click(function(){
			$metadata.remove();
		});

		if(value){
			$metadata.find("[input_request_body_form_type_variable_key]").val(value.key);
			$metadata.find("[input_request_body_form_type_variable_value]").val(value.value);
		}

		return $metadata
	};

	/*
		Crea y da funcionalidad al formulario de mapping donde se pueden configurar
		los mapeos por tres vías, manual, JSON manual o lanzando la petición diseñada
		a la API
	*/
	adm_process_flow_activity.prototype.__makeResponseMetadata = function(value){
		let self = this;
		const $metadata = $(templates.http_restfull_response_card);

		$metadata.find("[add_answer_http_response]").click(function () {

			self.__makeOpenAuthorizationPopup((mapping)=>{
				self.data.metadata.mapping = mapping;
				let $row = $metadata.find("[response_mapping_parameters]");
				//Eliminamos los mapeos que haya anteriormente;
				$row.find("[new_mapping_parameter]").remove();
				mapping.forEach((interface)=>{
					let $newRow = self.__makeMappingParametter(interface);
					$row.append($newRow);
				})

			});
		});

		$metadata.find("[add_answer_json_example]").click(function () {

			self.__makeOpenPopupManualJSON((mapping)=>{
				self.data.metadata.mapping = mapping;
				let $row = $metadata.find("[response_mapping_parameters]");
				//Eliminamos los mapeos que haya anteriormente;
				$row.find("[new_mapping_parameter]").remove();
				mapping.forEach((interface)=>{
					let $newRow = self.__makeMappingParametter(interface);
					$row.append($newRow);
				})

			});
		});

		$metadata.find("[add_answer_manually]").click(function () {
			let $row = $metadata.find("[response_mapping_parameters]");
			let $newRow = self.__makeMappingParametter({index:"",name:"",manual:true, mask:[]});
			$row.append($newRow);
		});

		if (self.data.metadata.mapping.length > 0) {
			self.data.metadata.mapping.forEach((interface)=>{
				let $newRow = self.__makeMappingParametter(interface);
				
				$metadata.find("[response_mapping_parameters]").append($newRow);
			});
		}

		return $metadata;
	};

	/*
		Crea un input para los headers, donde se puede seleccionar el tipo y darle un
		valor. Tambien crea la lógica necesaria cuando se selecciona un header
		de tipo personalizado.
	*/
	adm_process_flow_activity.prototype.__makeRequestHeader = function (value) {
		const guid = inConcert.Utils.generateGuid();
		var self = this;
		let $field = $(Mustache.render(templates.http_restfull_metadata_request_header, { guid: guid }));
		let el = document.createElement("option");
		el.textContent = "";
		el.value = "";
		$field.find("[select_header]")[0].appendChild(el);
		$.each(this.HeadersEnum, function (_, option) {
			el = document.createElement("option");
			el.textContent = option.key || option;
			el.value = option.value || option;
			$field.find("[select_header]")[0].appendChild(el);
		});

		$field.setModeManual = function(manual){
			$field.find("[select_header]").css("display",manual?"none":"inherit");
			$field.find("[input_custom_header]").css("display",manual?"inherit":"none");
		}

		$field.find("[select_header]").change(function(ev){
			if($field.find("[select_header]").val() === "Custom"){
				$field.setModeManual(true);
			}
		});

		$field.find("[delete_header]").click(function () {
			self.__disableNextKeyboardEvent();
			let $row = $(this).parent().parent().parent();
			$row.remove();
		});

		if(value){
			if(self.HeadersEnum.find((headerType)=>{return headerType === value.key})){
				//No tiene un valor custom
				$field.find("[select_header]").val(value.key);
			}else{
				//Es un valor custom
				$field.find("[select_header]").val("Custom");
				$field.find("[input_custom_header]").val(value.key);
				$field.setModeManual(true);
			}

			$field.find("[input_header]").val(value.value);
		}

		return $field;
	};

	/*
		Genera un parametro clave/valor para una interface del mapping
	*/
	adm_process_flow_activity.prototype.__makeMappingParametter = function(interface){
		//let self = this;
		let $field = $(Mustache.render(templates.http_restfull_metadata_mapping_parameters, { }));

		$field.find("[delete_parameters]").click(function () {
			let $row = $(this).parent().parent().parent();
			$row.remove();
		});

		if(interface){
			$field.find("[input_variable]").val(interface.name);
			$field.find("[input_path]").val(interface.index);
			if(interface.manual){
				$field.find("[input_path]").prop("disabled", false);
			}
		}

		return $field;
	};

	/*
		Genera un formulario clave/valor para los query parameters de la request
	*/
	adm_process_flow_activity.prototype.__makeQueryParameters = function (value) {
		const guid = inConcert.Utils.generateGuid();
		var self = this;
		let $field = $(Mustache.render(templates.http_restfull_metadata_query_parameters, { guid: guid }));

		$field.find("[delete_parameters]").click(function () {
			self.__disableNextKeyboardEvent();
			let $row = $(this).parent().parent().parent();		
			$row.remove();
		});

		if(value){
			$field.find("[input_parameters]").val(value.key);
			$field.find("[input1_parameters]").val(value.value);
		}

		return $field;

	};

	/*
		Inicializa el popup que solicita autorización al usuario para realizar una request
		en el mapeo de respuestas
	*/
	adm_process_flow_activity.prototype.__makeOpenAuthorizationPopup = function (mappingCallback) {

		const self = this;
		let data = self.__getAndValidate();
		//console.log("DATA AFTER VALIDATE", data)
		if(data.error){
			self.__showError(data.error);
		}else{
			self.__clearErrors();
			//console.log(data);
			self.$authorizationPopup = self.__makeAuthorizationPopup(data);
			$("body").append(self.$authorizationPopup);
			self.authorizationPopup = new inConcert.PopupWindow();
			self.authorizationPopup.show({
				ownerId: "flowDesignerPopup_req",
				eventName: "flowDesignerPopup_req",
				canReposition: true,
				closeOnFocusLost: false,
				overlay: true,
				container: self.$authorizationPopup,
				callbacks: {
					onOpen: function () {
						self.$authorizationPopup.show();
					},
					onClose: function (callback) {
						callback = callback || $.noop;
						if (self.authorizationPopup) {
							self.authorizationPopup.hide(function () {
								callback();
							});
						}
						else {
							callback();
						}
					}
				}
			});

			self.$authorizationPopup.find("#btnx_cancel").click(function () {
				self.__closeAuthorizationPopup();
			});
			self.$authorizationPopup.find("#btn_cancel").click(function () {
				self.__closeAuthorizationPopup();
			});

			self.alternativeEscapeHandler = () =>{
				self.__closeAuthorizationPopup();
			}

			let makeHttpRequestAction = function () {
				data.error = undefined;
				data = self.$authorizationPopup.__getAndValidateAuthorizationVariables(data)
				if(data.error){
					self.$authorizationPopup.__showError(data.error);
				}else{
					self.$authorizationPopup.__clearErrors();

					self.__closeAuthorizationPopup();
					self.__makeRequest(data,function(response, indexes){
						self.__makeopenPopupResponse(response,indexes,mappingCallback,data.metadata.mapping);
					},function(code, error){
						if(code.status === 200){
							self.__showError({
								message:error.result
							});
						}else{
							self.__showError({
								message:`<%=_T('The api didn’t sent a valid response')%>: ${code.status}`
							});
						}
					});
				}
			};

			self.alternativeEnterHandler = () =>{
				makeHttpRequestAction();
			};
			self.$authorizationPopup.find("#btn_openPopupResponse").click(makeHttpRequestAction);
		}


	};

	/*
		Inicializa el popup que permite introducir un JSON manualmente para luego mapearlo
		como respuesta
	*/
	adm_process_flow_activity.prototype.__makeOpenPopupManualJSON = function (mappingCallback) {
		const self = this;

		let data = self.__getAndValidate();
		//console.log("DATA AFTER VALIDATE", data)
		if(data.error){
			self.__showError(data.error);
		}else{
			self.__clearErrors();
			self.$manualJSONPopup = self.__makeManualJSONPopup();
			$("body").append(self.$manualJSONPopup);
			self.manualJSONPopup = new inConcert.PopupWindow();
			self.manualJSONPopup.__clearErrors = function(){
				self.$manualJSONPopup.find("[message_container]").find("[message_text]").text("");
				self.$manualJSONPopup.find("[message_container]").css("display","none");
			}
			self.manualJSONPopup.__showError = function(error){
				self.$manualJSONPopup.find("[message_container]").css("display","inline-block");
				self.$manualJSONPopup.find("[message_container]").find("[message_text]").text(error.message);
			}

			self.manualJSONPopup.show({
				ownerId: "flowDesignerPopup_manual_JSON",
				eventName: "flowDesignerPopup_manual_JSON",
				canReposition: true,
				closeOnFocusLost: false,
				overlay: true,
				container: self.$manualJSONPopup,
				callbacks: {
					onOpen: function () {
						self.$manualJSONPopup.show();
						let codeEditor = ace.edit(self.$manualJSONPopup.find("[code_block]")[0]);
						let codeSession = codeEditor.getSession();
						codeSession.setMode(inConcert.Utils.format("ace{0}mode{0}javascript", "/"));
						codeEditor.renderer.setScrollMargin(8, 8);
						codeSession.setValue(JSON.stringify({},null, '\t'));

						let AcceptJSONResponseAction = function () {
							self.manualJSONPopup.__clearErrors();
							let deserialized;
							try {
								deserialized = JSON.parse(codeSession.getValue());
							} catch (error) {
								self.manualJSONPopup.__showError({message:error.message});
							}
							if(deserialized){
								self.__closeManualJSONPopup();
								let response = deserialized;
								//console.log("Simulated Response", response);
								let indexes = self.__buildIndexes(response,{
									addFinalEmptyIndex:false,
									emptyObjectsInOneLine:false
								});
								self.__makeopenPopupResponse(response,indexes,mappingCallback,data.metadata.mapping);

							}

						};
						self.alternativeEnterHandler = () =>{
							AcceptJSONResponseAction();
						}
						self.$manualJSONPopup.find("#btn_AcceptJSONResponse").click(AcceptJSONResponseAction);
					},
					onClose: function (callback) {
						callback = callback || $.noop;
						if (self.manualJSONPopup) {
							self.manualJSONPopup.hide(function () {
								callback();
							});
						}
						else {
							callback();
						}
					}
				}
			});

			self.$manualJSONPopup.find("#btn_cancel_response").click(function () {
				self.__closeManualJSONPopup();
			});
			self.alternativeEscapeHandler = () =>{
				self.__closeManualJSONPopup();
			};
		}

	};

	/*
		Construye los indices correspondientes a una determinada respuesta de API (Objeto JSON)
	*/
	adm_process_flow_activity.prototype.__buildIndexes = function(response, configuration){
		if(!configuration){configuration = {}};
		let indexes = [];
		let rec = (part, base) =>{
			indexes.push(base);
			if(Array.isArray(part)){
				//La parte es un array
				for(let i = 0; i < part.length; i++){
					rec(part[i],`${base}[${i}]`)
				}
				if(configuration.addFinalEmptyIndex){
					if(configuration.emptyObjectsInOneLine){
						if(part.length > 0){
							indexes.push("");
						}
					}else{
						indexes.push("");
					}
				}


			}else{
				if(typeof(part) === "object"){
					for(let key in part){
						rec(part[key], `${base}.${key}`);
					}
					if(configuration.addFinalEmptyIndex){
						if(configuration.emptyObjectsInOneLine){
							if(Object.keys(part).length > 0){
								indexes.push("");
							}
						}else{
							indexes.push("");
						}
					}
				}
			}
		}
		rec(response, "data");
		return indexes;
	};
	/*
		Realiza una petición HTTP al controlador con la configuración que esté establecida en la
		actividad
	*/
	adm_process_flow_activity.prototype.__makeRequest = function (data, successClbk, errorClbk) {
		let self = this;

		let payload = {
			method: data.method,
			endpoint: data.url,
			headers: data.metadata.headers,
			queryParameters: data.metadata.query,
			body: data.metadata.body,
			bodyParameters: data.metadata.bodyParameters,
			bodyType: data.metadata.bodyType
		}

		let uri = inConcert.app.getFlowDesignerUrl() + "/process_flow_http/" + "Makerequest"

		inConcert.invoke({
			url: uri,
			type: "POST",
			timeout: 30000,
			dataType: "json",
			contentType: "application/json",
			data: JSON.stringify(payload),
			success: function (response) {
				successClbk(response, self.__buildIndexes(response, {
					addFinalEmptyIndex:false,
					emptyObjectsInOneLine:false
				}))
			 },
			error: function (code, error) { errorClbk(code, error) }
		});
	};

	/*
		Muestra el arbol de seleccionables del mapeo de una respuesta
	*/
	adm_process_flow_activity.prototype.__showResponseWithCheckbox = function(response, indexes){
		let self = this;
		let $container = self.$authorizationPopup1.find("[code_block]")[0];
		//Selecciona el rango entre dos elementos del arbol omitiendo los que son una repetición de array
		let fillSelectBetween = function(from, to){
			let point = from;
			while(point.uuid !== to.uuid){
				if(!point.parent.inArray && !(point.inArray)){
					point.parent.select(true);
				}
				point = point.parent;
			}
		}
		///Valida que las selecciones del mapeo sean coherentes escalando en el arbol
		let validateUp = function(context){
			//Buscamos si ya hay un padre seleccionado en su camino hacia arriba y
			//por lo tanto estamos en una mascara

			//Devuelve el siguiente padre seleccionado o el propio contexto
			let recSearch = function(context){
				if(context.parent){
					if(context.parent.selected){
						return context.parent;
					}else{
						return recSearch(context.parent);
					}
				}
				return undefined;
			}
			//console.log("FROM UP",context, "TO UP", recSearch(context)||context)
			fillSelectBetween(context,recSearch(context)||context)
		}
		///Valida que las selecciones del mapeo sean coherentes hacia abajo del arbol
		let validateDown = function(context){
			//Buscamos los hijos que puedan estar seleccionados para rellenar entre ellos

			let recSearch = function(context){
				//Recorremos el nivel
				let farestSelecteds = [];
				context.childrens.forEach((children)=>{

					if(children.selected){
						//Si esta seleccionado, pero tiene hijos, puede que este
						//no sea el mas lejano
						if(children.childrens.length > 0){
							let farest = recSearch(children);
							//Si tiene mas lejanos, los añadimos como lejanos
							if(farest.length > 0){
								farestSelecteds = [...farestSelecteds, ...farest]
							}else{
								//Si no añadimos este hijo, pues es el mas lejano
								farestSelecteds.push(children);
							}

						}else{
							//No tiene hijos y esta seleccionado, es el mas lejano seguro
							farestSelecteds.push(children);
						}
					}else{
						//No esta seleccionado, pero al tener hijos, puede que haya lejanos
						//seleccionados
						if(children.childrens.length > 0){
							//El hijo tiene mas hijos
							farestSelecteds = [...farestSelecteds, ...recSearch(children)]
						}
					}

				})
				return farestSelecteds;
			}

			let farest = recSearch(context);
			farest.forEach((farest)=>{
				fillSelectBetween(farest,context);
			})
		}

		let generateElement = function(context){
			let $el = $(Mustache.render( templates.http_restfull_response_mapping_checkbox, context));

			$el.attachElement = function(child){
				$el.find(`[child_container_${context.uuid}]`)[0].appendChild(child);
			}

			$el.processContext = function(){
				if(context.collapsed){
					$el.find(`[collapse_button_${context.uuid}]`).addClass("minimized");
					$el.find(`[child_container_${context.uuid}]`).addClass("hidding");
				}else{
					$el.find(`[collapse_button_${context.uuid}]`).removeClass("minimized");
					$el.find(`[child_container_${context.uuid}]`).removeClass("hidding");
				}

				if(context.selected){
					$el.addClass("selected");
				}else{
					$el.removeClass("selected");
				}


				if(context.isProperty){
					$el.find(`[collapse_button_${context.uuid}]`).addClass("hidding");
					$el.find(`[close_character_${context.uuid}]`).addClass("hidding");
					$el.find(`[value_assignation_${context.uuid}]`).removeClass("hidding");
				}else{
					$el.find(`[collapse_button_${context.uuid}]`).removeClass("hidding");
					$el.find(`[close_character_${context.uuid}]`).removeClass("hidding");
					$el.find(`[value_assignation_${context.uuid}]`).addClass("hidding");
				}
			}

			$el.find(`[collapse_button_${context.uuid}]`).click(function(){
				context.collapsed = !context.collapsed;
				$el.processContext();
			});

			context.select = function(value){
				context.selected = value;
				$el.find(`[check_element_${context.uuid}]`).prop('checked', context.selected);
				$el.processContext();
			}

			$el.find(`[check_element_${context.uuid}]`).click(function(ev){
				context.select(!context.selected);
				if(context.onSelectedChange){
					context.onSelectedChange(context.selected);
				}
			});

			context.print = $el.processContext;
			$el.processContext();
			return $el;
		}

		let nextValidParentIsSelected = function(context){
			let recSearch = function(el){
				if(!el.parent.inArray){
					return el.parent.selected;
				}else{
					return recSearch(el.parent);
				}
			}
			if(context.parent){
				return recSearch(context);
			}else{
				return false;
			}
		}
		let hasFirstLevelChildrenSelected = function(context){
			let selected = false;

			for(let i = 0; i < context.childrens.length; i++){
				if(context.childrens[i].selected){
					selected = true;
					break;
				}
			}

			return selected;
		}
		let __globalCount = 0;
		let rec = function(part,key,el,parent){
			el = el || {};
			el.uuid = inConcert.Utils.generateGuid();
			el.parent = parent;
			el.index = indexes[__globalCount];
			el.onSelectedChange = function(selected){
				/*
				Hay cuatro posibilidades para que el evento no sea valido,
				Que se esté interactuando con un elemento "inArray" donde
				su valor no puede estar seleccionado si el siguiente padre válido está seleccionado

				O bien que esté rodeado de seleccionados, esto es que tenga un hijo
				seleccionado y que su padre "no inArray" esté seleccionado

				O bien, que se seleccione un objeto "isArray" donde algun elemento "inArray" está seleccionado

				O bien, que tanto alguno de sus hijos directos como su padre directo esten seleccionados y se pretenda
				deseleccionar
				*/

				let validChange = true;
				if(
					(el.inArray && nextValidParentIsSelected(el) && selected) || //Primer Caso
					(nextValidParentIsSelected(el) && hasFirstLevelChildrenSelected(el)) || //Segundo Caso
					(selected && hasFirstLevelChildrenSelected(el) && el.isArray) || //Tercer Caso
					(!selected && el.parent && el.parent.selected && hasFirstLevelChildrenSelected(el)) //Cuarto caso
				){
					validChange = false;
				}

				if(validChange){
					if(el.parent && selected){
						validateUp(el);
					}
					if(el.childrens.length > 0 && selected){
						validateDown(el);
					}
				}else{
					el.select(!selected);
				}
			}
			__globalCount++;
			el.childrens = [];
			if(parent && parent.childrens){
				parent.childrens.push(el);
			}

			let component;
			if(Array.isArray(part)){
				//Estamos recorriendo un array
				el.key = key;
				el.isArray = true;
				el.value = "[";
				el.close_character = "]";
				component = generateElement(el);
				for(let i = 0; i < part.length; i++){
					component.attachElement(rec(part[i],i,{collapsed:i>0, inArray:true},el)[0]);
				}
			}else if(typeof(part) === "object"){
				//Es un objeto
				el.key = key;
				el.isObject = true;
				el.value = "{";
				el.close_character = "}";
				component = generateElement(el);
				for(let k in part){
					component.attachElement(rec(part[k],k,null,el)[0])
				}
			}else{
				//Es una propiedad
				el.isProperty = true;
				el.key = key;
				if(typeof(part) === "string"){
					el.value = `"${part}"`
				}else{
					el.value = part;
				}
				component = generateElement(el);
			}
			return [component[0],el];
		}
		let componentConstruction = rec(response,"data")
		$container.appendChild(componentConstruction[0]);
		return componentConstruction[1];
	};

	/*
		Pinta una configuración de mapeo ya establecida sobre un contexto de respuesta
		concreto.
	*/
	adm_process_flow_activity.prototype.__paintMapping = function(context, mapping){
		//console.log("MAPPING", mapping)
		//console.log("GENERAL PAINT", context, mapping)
		let recPaint = function(context, interface, result){
			result[interface.absoluteIndex] = false;
			if(context.index === interface.absoluteIndex){
				result[interface.absoluteIndex] = true;
				if(interface.manual){
					context.manual = true;
				}
				if(interface.name){
					context.name = interface.name;
				}
				context.select(true)
			}else{
				if(context.childrens){
					for(let i = 0; i < context.childrens.length; i++){
						recPaint(context.childrens[i],interface, result);
						if(result[interface.absoluteIndex]){
							break;
						}
					}
				}
			}
			if(interface.mask){
				for(let i = 0; i < interface.mask.length; i++){
					recPaint(context,interface.mask[i],result);
				}
			}
		}

		mapping.forEach((interface)=>{
			let result = {};
			recPaint(context, interface,result);

			let pathsNoEncontrados = Object.keys(result).filter((localResult)=>{
				return result[localResult] === false;
			})
			 if(pathsNoEncontrados.length > 0){
			 	//console.log("HUERFANO PARA:", interface);
				// console.log("ESTADISTICA DE HORFANDAD:",{
				// 	noEnc: pathsNoEncontrados,
				// 	porc: `${Math.round((pathsNoEncontrados.length / Object.keys(result).length) * 100)}%`
				// })
			}
		})
	};

	/*
		Muestra el dialogo de árbol que se vé tras realizar una petición de api.
		El cuál permite seleccionar el mapeo sobre una respuesta real.
	*/
	adm_process_flow_activity.prototype.__makeopenPopupResponse = function (response, indexes, mappingCallback,beforeMapping) {
		const self = this;

		self.$authorizationPopup1 = self.__makePopupResponse();
		$("body").append(self.$authorizationPopup1);

		self.authorizationPopup1 = new inConcert.PopupWindow();
		let mapContext;
		self.authorizationPopup1.show({
			ownerId: "flowDesignerPopup_response",
			eventName: "flowDesignerPopup_response",
			canReposition: true,
			closeOnFocusLost: false,
			overlay: true,
			container: self.$authorizationPopup1,
			callbacks: {
				onOpen: function () {
					//self.__showResponseWithAce(response, indexes);
					$("#flowDesignerPop_response_waiting").show();
					setTimeout(()=>{
						mapContext = self.__showResponseWithCheckbox(response, indexes);
						if(beforeMapping){
							self.__paintMapping(mapContext, beforeMapping);
						}
						$("#flowDesignerPop_response_waiting").hide();
					},0);
				},
				onClose: function (callback) {
					callback = callback || $.noop;
					if (self.authorizationPopup1) {
						self.authorizationPopup1.hide(function () {
							callback();
						});
					}
					else {
						callback();
					}
				}
			}
		});

		self.$authorizationPopup1.find("#btn_cancel_response").click(function () {
			self.__closePopupResponse();
		});

		self.alternativeEscapeHandler = () =>{
			self.__closePopupResponse();
		}

		self.__buildMapping = function(context){
			/*
				El mapping esta compuesto de los indices base y de sus posibles mascaras
				Por lo que el contexto se debe recorrer nivel a nivel
				incluyendo en el mapeo los objetos
				{
					index: indice,
					mask:[
						{
							index: indice,
							mask:[...]
						}
					]
				}
			*/
			let map = [];

			let recBuilder = function(localContext, parentInterface){
				if(localContext.selected){
					//Este nivel está seleccionado, por lo que inicializamos su interfaz si no tiene
					//una interface padre:
					let interface = {
						index: localContext.index,
						absoluteIndex: localContext.index,
						isArray: localContext.isArray,
						manual: localContext.manual,
						mask:[]
					}

					if(!parentInterface){
						//Al estar en una interfaz padre, mantenemos el posible nombre de la variable
						//Tanto para luego rellenar inputs como para el preview
						interface.name = localContext.name,
						map.push(interface)
					}else{
						//Tiene interface padre, por lo que esto iría dentro de la mascara
						//Hay que limpiar los indicadores de iteración salvo del último elemento del path
						let indexArray = interface.index.split(".");
						//Esto quita en los paths absolutos el indice del array si lo tiene en los niveles que no son el último
						//Está quitado para luego poder reconstruir los mapeos ya guardados con anterioridad, no afecta al resto de
						//la construcción
						// indexArray = indexArray.map((level, i)=>{
						// 	return (i < indexArray.length - 1 ? level.replace(/\[\d+\]/,""): level);
						// })
						//Generamos un path absoluto del elemento
						interface.absoluteIndex = indexArray.join(".");
						//Obtenemos el array del indice absoluto del padre
						let parentIndexArray = parentInterface.absoluteIndex.split(".");
						//Restamos al indice del elemento el indice absoluto del padre
						indexArray.splice(0,parentIndexArray.length);
						//Establecemos como indice el resultado
						interface.index = indexArray.join(".");
						//console.log(indexArray, "CLEAN");

						parentInterface.mask.push(interface);
					}
					parentInterface = interface;
				}
				localContext.childrens.forEach((children)=>{
					recBuilder(children, parentInterface);
				})
			}
			recBuilder(context);
			return map;
		};

		self.__previewResult = function(map, response){
			let baseResponse = {data: response};
			let result = {};
			let recBuilder = function(partialMap, partialResponse,partialResult){
				//Recorremos cada una de las interfaces de ese nivel del map
				partialMap.forEach((interface)=>{
					//Es probable, que el indice apunte directamente a un elemento no raiz
					//del resultado, para ello tenemos que ir hasta ese punto
					if(partialResponse){
						let localPartialResponse = partialResponse;
						let indexArray = interface.index.split(".");
						indexArray.forEach((index) =>{
							if(/\[\d+\]/.test(index)){
								//El indice es referente a un elemento de dentro de un array
								let arrayName = index.split(/\[\d+\]/)[0];
								let indexIterancePoint = parseInt(index.substr(index.indexOf("[") + 1).replace("]",""));
								try {
									localPartialResponse = localPartialResponse[arrayName][indexIterancePoint];
								} catch (error) {
									localPartialResponse = undefined;
								}

							}else{
								try {
									localPartialResponse = localPartialResponse[index];
								} catch (error) {
									localPartialResponse = undefined;
								}

							}
						})
						if(localPartialResponse){
							if(interface.isArray){
								//Debemos extraer por cada iteración del response en este indice el contenido
								partialResult[interface.name || interface.index] = new Array(localPartialResponse.length);
								//localPartialResponse es por lo tanto un array:
								localPartialResponse.forEach((singleLocalPartialResponse, i) =>{
									//Vamos añadiendo el resultado al array de resultados
									if(interface.mask.length > 0){
										//El resultado está enmascarado
										//Por lo que debemos construir el interior del elemento a partir de las interfaces declaradas dentro
										partialResult[interface.name || interface.index][i] = partialResult[interface.name || interface.index][i] || {};
										recBuilder(interface.mask,singleLocalPartialResponse,partialResult[interface.name || interface.index][i]);
									}else{
										//El resultado es total
										partialResult[interface.name || interface.index][i] = singleLocalPartialResponse
									}
								})
							}else{
								//En este punto nuestro localPartialResponse ya apunta a la parte de la respuesta que interesa
								//para este indice, queda generar el enmascarado
								if(interface.mask.length > 0){
									//El resultado está enmascarado
									//Por lo que debemos construir el interior del elemento a partir de las interfaces declaradas dentro
									partialResult[interface.name || interface.index] = partialResult[interface.name || interface.index] || {};
									recBuilder(interface.mask,localPartialResponse,partialResult[interface.name || interface.index]);
								}else{
									//El resultado es total
									partialResult[interface.name || interface.index] = localPartialResponse
								}

							}
						}
					}
				})
			}
			recBuilder(map,baseResponse,result)
			return result;
		}

		let selectMappingAction = function () {
			//console.log("BUILDING MAPPING")
			mappingCallback(self.__buildMapping(mapContext));
			self.__closePopupResponse();
		};

		self.alternativeEnterHandler = () =>{
			selectMappingAction();
		};

		self.$authorizationPopup1.find("#btn_selectMapping").click(selectMappingAction);
		self.$authorizationPopup1.find("#btn_previewResult").click(function () {
			let map = self.__buildMapping(mapContext)
			let preview = self.__previewResult(map,response)

			self.$previewMappingPopup = self.__makePopupPreviewMapping();
			$("body").append(self.$previewMappingPopup);

			self.previewMappingPopup = new inConcert.PopupWindow();
			self.previewMappingPopup.show({
				ownerId: "flowDesignerPopup_preview_mapping",
				eventName: "flowDesignerPopup_preview_mapping",
				canReposition: true,
				closeOnFocusLost: false,
				overlay: true,
				container: self.$previewMappingPopup,
				callbacks: {
					onOpen: function () {
							self.$previewMappingPopup.show();
							let codeEditor = ace.edit(self.$previewMappingPopup.find("[code_block]")[0]);
							let codeSession = codeEditor.getSession();
							codeSession.setMode(inConcert.Utils.format("ace{0}mode{0}javascript", "/"));
							codeEditor.renderer.setScrollMargin(8, 8);
							codeSession.setValue(JSON.stringify(preview,null, '\t'));
							codeEditor.setReadOnly(true);
					},
					onClose: function (callback) {
						callback = callback || $.noop;
						if (self.previewMappingPopup) {
							self.previewMappingPopup.hide(function () {
								callback();
							});
						}
						else {
							callback();
						}
					}
				}
			});

			self.$previewMappingPopup.find("#btn_AcceptPreview").click(function () {
				self.__closePopupPreviewMapping(selectMappingAction,self.__closePopupResponse);
			});
			self.alternativeEscapeHandler = () =>{};
			self.alternativeEnterHandler = () =>{
				self.__closePopupPreviewMapping(selectMappingAction,self.__closePopupResponse);
			};
			//console.log("SELECT MAPPING, TO BUILD!", mapContext)
		});
	};
	/*
		Retorna el elemento correspondiente al dialogo de previsualización de una 
		respuesta aplicando un mapeo.
	*/
	adm_process_flow_activity.prototype.__makePopupPreviewMapping = function(){
		let $popup = $(templates.http_restfull_preview_mapping_popup);
		return $popup;
	};
	/*
		Retorna el elemento correspondiente al dialogo que permite indicar
		manualmente un JSON para posteriormente mapearlo
	*/
	adm_process_flow_activity.prototype.__makeManualJSONPopup = function(){
		let $popup = $(templates.http_restfull_manual_json_popup);
		return $popup;
	};
	/*
		Retorna el elemento correspondiente al Popup que muestra la respuesta de la 
		API.
	*/
	adm_process_flow_activity.prototype.__makePopupResponse = function () {
		let $popup = $(templates.http_restfull_response_popup);
		return $popup;
	};

	/*
		Retorna el elemento correspondiente al popup de autorización que se usa
		antes de realizar una petición a la API, también genera el formulario
		necesario en este popup, cuando existen variables en las diferentes
		partes de la configuración
	*/
	adm_process_flow_activity.prototype.__makeAuthorizationPopup = function (data) {
		/*
			Si existen variables declaradas en:
			- URL
			- Header (key o value)
			- Query parameters (key o value)

			Debe pedirse antes un relleno de información de estas
		*/
		let vars = [];
		let __containsVariableTemplate = function(data){
			//console.log(data);
			let reg = /\{\{[^{]+\}\}/g;
			if(reg.test(data.url)){
				vars.push({key:data.url.match(reg), pointer: data, propertie:"url"})
			}
			if(data.metadata.bodyType === "text"){
				if(reg.test(data.metadata.body)){
					vars.push({key:data.metadata.body.match(reg), pointer: data.metadata, propertie:"body"})
				}
			}else{
				data.metadata.bodyParameters.forEach((parameter)=>{
					if(reg.test(parameter.key)){
						vars.push({key:parameter.key.match(reg), pointer: parameter, propertie:"key"})
					}
					if(reg.test(parameter.value)){
						vars.push({key:parameter.value.match(reg), pointer: parameter, propertie:"value"})
					}
				})
			}

			data.metadata.headers.forEach((header)=>{
				if(reg.test(header.key)){
					vars.push({key:header.key.match(reg), pointer: header, propertie:"key"})
				}
				if(reg.test(header.value)){
					vars.push({key:header.value.match(reg), pointer: header, propertie:"value"})
				}
			})
			data.metadata.query.forEach((query)=>{
				if(reg.test(query.key)){
					vars.push({key:query.key.match(reg), pointer: query, propertie:"key"})
				}
				if(reg.test(query.value)){
					vars.push({key:query.value.match(reg), pointer: query, propertie:"value"})
				}
			})

		}

		let __makeVariableInputs = function(variables, parent){
			variables.forEach((variable)=>{
				variable.key.forEach((key)=>{
					let variableElement = $(Mustache.render(templates.http_restfull_authorization_variable_input, variable))
					variableElement.find("[input_authorization_variable_key]").val(key);
					parent.find("[variable_inputs_container]").append(variableElement);
				})

			})
		}

		__containsVariableTemplate(data)
		//console.log(vars);

		let $popup = $(Mustache.render(templates.http_restfull_authorization_request_popup,data));

		if(vars.length > 0){
			__makeVariableInputs(vars, $popup)
		}

		$popup.__getAndValidateAuthorizationVariables = function(data){
			//Debemos sustituir los campos de data, query y methods
			//Primero obtenemos todos los valores, y si alguno estuviese mal,
			//retornamos un error
			let porpuse = {};
			$popup.find("[variable_inputs_container]").find("[http_restfull_authorization_variable]").each((_,el)=>{
				let key = $(el).find("[input_authorization_variable_key]").val();
				let value = $(el).find("[input_authorization_variable_value]").val();
				if(value && value.trim()!==""){
					porpuse[key] = value;
				}else{
					data.error = {
						message: "<%=_T('All variables must be indicated')%>"
					}
				}
			})

			if(!data.error){
				vars.forEach((variable)=>{
					variable.key.forEach((key)=>{
						variable.pointer[variable.propertie] = variable.pointer[variable.propertie].replace(key, porpuse[key]);
					})
				})
			}

			return data;
		}

		$popup.__clearErrors = function(){
			$popup.find("[message_container]").find("[message_text]").text("");
			$popup.find("[message_container]").css("display","none");
		}
		$popup.__showError = function(error){
			$popup.find("[message_container]").css("display","inline-block");
			$popup.find("[message_container]").find("[message_text]").text(error.message);
		}

		return $popup;
	};

	/*
		Cierra el Popup de preview de una respuesta tras pasarla con un mapeo
	*/
	adm_process_flow_activity.prototype.__closePopupPreviewMapping = function (parentEnterEvent, parentEscapeEvent) {
		let self = this;
		// if (self.authorizationPooling) {
		// 	window.clearInterval(self.authorizationPooling);
		// 	self.authorizationPooling = null;
		// }
		if(self.previewMappingPopup && self.previewMappingPopup.isVisible()){
			self.previewMappingPopup.finalize(function () {
				self.previewMappingPopup = null;
				self.$previewMappingPopup.remove();
			});
		}
		else {
			self.$previewMappingPopup.remove();
		}
		self.alternativeEscapeHandler = parentEscapeEvent;
		self.alternativeEnterHandler = parentEnterEvent;
	};


	/*
		Cierra el popup que muestra la respuesta dada por la API
	*/
	adm_process_flow_activity.prototype.__closePopupResponse = function () {
		let self = this;
		// if (self.authorizationPooling) {
		// 	window.clearInterval(self.authorizationPooling);
		// 	self.authorizationPooling = null;
		// }
		if (self.authorizationPopup1 && self.authorizationPopup1.isVisible()) {
			self.authorizationPopup1.finalize(function () {
				self.authorizationPopup1 = null;
				self.$authorizationPopup1.remove();
			});
		}
		else {
			self.$authorizationPopup1.remove();
		}
		self.alternativeEscapeHandler = undefined;
		self.alternativeEnterHandler = undefined;
	};

	/*
		Cierra el popup que permite introducir un JSON manualmente
	*/
	adm_process_flow_activity.prototype.__closeManualJSONPopup = function(){
		let self = this;
		if (self.manualJSONPopup && self.manualJSONPopup.isVisible()) {
			self.manualJSONPopup.finalize(function () {
				self.manualJSONPopup = null;
				self.$manualJSONPopup.remove();
			});
		}
		else {
			self.$manualJSONPopup.remove();
		}
		self.alternativeEscapeHandler = undefined;
		self.alternativeEnterHandler = undefined;
	};

	/*
		Cierra el popup que pide autorización para realizar una petición a la API
	*/
	adm_process_flow_activity.prototype.__closeAuthorizationPopup = function () {
		let self = this;
		if (self.authorizationPooling) {
			window.clearInterval(self.authorizationPooling);
			self.authorizationPooling = null;
		}
		if (self.authorizationPopup && self.authorizationPopup.isVisible()) {
			self.authorizationPopup.finalize(function () {
				self.authorizationPopup = null;
				self.$authorizationPopup.remove();
			});
		}
		else {
			self.$authorizationPopup.remove();
		}
		self.alternativeEscapeHandler = undefined;
		self.alternativeEnterHandler = undefined;
	};

	/*
		Obtiene y valida todos los campos de configuración de la actividad
	*/
	adm_process_flow_activity.prototype.__getAndValidate = function () {

		var data = adm_process_flow_activity.uber.__getAndValidate.call(this);

		if (!data.error) {
			this.__getAndValidateFields(data);
		}
		return data;
	};


	/*
		Incluye en el objeto indicado data, toda la información validada configurada
		para la parte de puertos y status (Outputs)
	*/
	adm_process_flow_activity.prototype.__getAndValidatePortsConfiguration = function(data){

		let self = this;
		data.ports = data.ports || [];
		let fields = this.$get("[ports_container]").find("[field]");
		if(fields.length > 0){
			fields.each((_, field)=>{
				let name = $(field).find("[field_name]").val() || undefined;
				if(!name || name.trim().length === 0){
					data.error = {
						message: `<%=_T('Invalid Port Name')%>.`,
						$control:$(field)
					}
				}else{
					data.ports.push({
						id: $(field).attr("ic-restfull-port-id"),
						name,
						status: self.__getAndValidatePortStatus(field, data)
					})
				}
			});
		};
		let status_variable = this.$get("[status_field]").val();
		if( status_variable && status_variable.trim()!==""){
			data.result = this.$get("[status_field]").val();
		}else{
			data.error = {
				message:`<%=_T('Response Status must have a valid variable value')%>.`,
				$control:this.$get("[status_field]")
			}
		}

		if(this.$get("[disclaimer]").length === 0){
			data.dragAndDropDisclaimerHide = true;
		}


		return data;
	};

	/*
		Obtiene y valida un status concreto dentro de un puerto
	*/
	adm_process_flow_activity.prototype.__getAndValidatePortStatus = function(field, data){
		let statuses = [];
		//console.log(field);
		$(field).find("[field_status]").each((_,statusField)=>{
			let status = parseInt($(statusField).val());
			if(isNaN(status)){
				data.error = {
					message: '<%=_T("At least one status dont have a valid value. It must be numeric")%>.'
				}
				return false;
			}else{
				let anotherStatusesEquals = data.ports.find((port)=>{
					if(port.status && Array.isArray(port.status)){
						return port.status.find((statusInPort)=>{
							return statusInPort === status
						})
					}else{
						return false
					}
				});

				if(!anotherStatusesEquals && !statuses.find((samePortStatus)=>{return samePortStatus === status})){
					statuses.push(status);
				}else{
					data.error = {
						message: '<%=_T("Exists a duplicated status")%>.'
					}
					return false;
				}

			}

		})
		if(statuses.length > 0){
			return statuses;
		}else{
			data.error = {
				message: '<%=_T("At least one port dont have any valid status. It must have at least one")%>.'
			}
			return false;
		}


	};

	/*
		Obtiene y valida un header indicado en el apartado de Request Headers
	*/
	adm_process_flow_activity.prototype.__getAndValidateRequestHeader = function ($field, data) {
		let field = {
			key: $field.find("[select_header]").val() || undefined,
			value: $field.find("[input_header]").val() || undefined
		};

		if(field.key === "Custom"){
			field.key = $field.find("[input_custom_header]").val() || undefined
		}

		let validate = function(prop, name){
			if(!prop){
				data.error = {
					message: `<%=_T('Must enter a ${name}')%>.`,
					$control:$field
				}
			}else{
				if(prop.trim().length === 0){
					data.error = {
						message: `<%=_T('Invalid ${name}')%>.`,
						$control:$field
					}
				}
			}

		}

		validate(field.key,"name");
		validate(field.value,"value");

		data.metadata.headers.push(field);
	};

	/*
		Obtiene y valida un parametro indicado en el Request Body cuando es de tipo Form
	*/
	adm_process_flow_activity.prototype.__getAndValidateRequestBodyFormTypeInput = function ($field, data) {

		let field = {
			key: $field.find("[input_request_body_form_type_variable_key]").val() || undefined,
			value: $field.find("[input_request_body_form_type_variable_value]").val() || undefined
		};

		//console.log("FIELD", field)

		let validate = function(prop, name){
			if(!prop){
				data.error = {
					message: `<%=_T('Must enter a ${name} in all Request Body Parameters')%>.`,
					$control:$field
				}
			}else{
				if(prop.trim().length === 0){
					data.error = {
						message: `<%=_T('Invalid ${name} in a Request Body Parameter')%>.`,
						$control:$field
					}
				}
			}

		}

		validate(field.key,"name");
		validate(field.value,"value");

		data.metadata.bodyParameters.push(field);
	};
	/*
		Obtiene y valida un parametro indicado en el apartado de Query Parameters
	*/
	adm_process_flow_activity.prototype.__getAndValidateQueryParameters = function ($field, data) {
		let field = {
			key: $field.find("[input_parameters]").val() || undefined,
			value: $field.find("[input1_parameters]").val() || undefined
		};


		let validate = function(prop, name){
			if(!prop){
				data.error = {
					message: `<%=_T('Must enter a ${name}')%>.`,
					$control:$field
				}
			}else{
				if(prop.trim().length === 0){
					data.error = {
						message: `<%=_T('Invalid ${name}')%>.`,
						$control:$field
					}
				}
			}

		}

		validate(field.key,"name");
		validate(field.value,"value");

		data.metadata.query.push(field);
	};

	/*
		Obtiene y verifica la información básica, esto es el metodo y la url
	*/
	adm_process_flow_activity.prototype.__getAndValidateBasicConfiguration = function(data){
		let method_value = this.$get("[field_method]").val();
		let url_value = this.$get("[field_url]").val();
		if(!method_value || method_value === ""){
			data.error = {
				message: "<%=_T('Method must be selected')%>.",
				$control: this.$get("[field_method]")
			}
		}else{
			data.method = method_value;
		}
		if(!url_value || url_value === ""){
			data.error = {
				message: "<%=_T('Url must be indicated')%>.",
				$control: this.$get("[field_url]")
			}
		}else if(!/[(http(s)?):\/\/(www\.)?a-zA-Z0-9@:%._\+~#=]{2,256}\.[a-z]{2,6}\b([-a-zA-Z0-9@:%_\+.~#?&//=]*)/.test(url_value)){
			data.error = {
				message: "<%=_T('Url must be a valid url direction')%>.",
				$control: this.$get("[field_url]")
			}
		}else{
			data.url = url_value;
		}

	};

	/*
		Obtiene y verifica los mapeos indicados en la parte de Response, ya sean generados de 
		forma manual o habiendose indicado en el diálogo anterior.
	*/
	adm_process_flow_activity.prototype.__getAndValidateMappingParameters = function ($field, data) {
		let name = $field.find("[input_variable]").val() || undefined;
		let path = $field.find("[input_path]").val() || undefined
		let manual = !($field.find("[input_path]").prop("disabled"));
		if(!name){
			data.error = {
				message: `<%=_T('Must enter a variable name')%>.`,
				$control:$field
			}
		}else{
			if(name.trim().length === 0){
				data.error = {
					message: `<%=_T('Invalid Variable name')%>.`,
					$control:$field
				}
			}
			if(data.metadata.mapping.find((interface)=>{
				return interface.name === name;
			})){
				data.error = {
					message: `<%=_T('Duplicated Variable name')%>.`,
					$control:$field
				}
			}
		}

		if(!data.error){
			let interface = this.data.metadata.mapping.find((interface)=>{return interface.index === path})

			if(!interface && !manual){
				data.error = {
					message: `<%=_T('Invalid Path')%>.`,
					$control:$field
				}
			}else{
				if(manual){
					interface = {};
					if(!path){
						data.error = {
							message: `<%=_T('Must enter a path value')%>.`,
							$control:$field
						}
					}else{
						if(path.trim().length === 0){
							data.error = {
								message: `<%=_T('Invalid path value')%>.`,
								$control:$field
							}
						}
					}
					interface.index = path;
					interface.absoluteIndex = path;
					interface.mask = [];
					interface.manual = true;
				}
				interface.name = name;
				//console.log(interface, "INTERFACE TO SAVE")
				data.metadata.mapping.push(interface);
			}
		}
	};

	/*
		Dispara la validación de cada una de las partes de la actividad
	*/
	adm_process_flow_activity.prototype.__getAndValidateFields = function (data) {

		const self = this;
		//Inicializamos metadata si no existe
		data.ports = data.ports || [];
		data.metadata = data.metadata || {};
		data.metadata.query = data.metadata.query || [];
		data.metadata.headers = data.metadata.headers || [];
		data.metadata.mapping = data.metadata.mapping || [];
		data.metadata.body = data.metadata.body || "";
		data.metadata.bodyFormat = data.metadata.bodyFormat || "json";
		data.metadata.bodyType = data.metadata.bodyType || "text";
		data.metadata.bodyParameters = data.metadata.bodyParameters || [];
		data.result = data.result ||  `G_${inConcert.Utils.generateGuid()}`;

		self.__getAndValidateBasicConfiguration(data);
		self.__getAndValidatePortsConfiguration(data);

		data.metadata.bodyType = this.$get("[flow_designer_http_api_body_request_type_radio_set]").find("input[name=Type]:checked").val();

		if(data.method !== "GET"){
			if(data.metadata.bodyType === "text"){
				data.metadata.bodyFormat = this.$get("[select_content_type_body_parameters]").val();
				let codeEditor = ace.edit(this.$get("[code_block]")[0]);
				let codeSession = codeEditor.getSession();
				codeEditor.renderer.setScrollMargin(8, 8);
				if(codeSession.getValue().trim() !== ""){
					if(data.metadata.bodyFormat === "json"){
						try {
							let bodyDeserialized = JSON.parse(codeSession.getValue());
							data.metadata.body = JSON.stringify(bodyDeserialized);
						} catch (error) {
							error.message = `<%=_T("Request body has errors")%>.`;
							data.error = error;
							return false;
						}
					}else if(data.metadata.bodyFormat === "xml/html"){
						//El tipo es XML
						let domParser = new DOMParser();
						let dom = domParser.parseFromString(codeSession.getValue(),'application/xml');
						//console.log(dom);
						if(dom.getElementsByTagName("parsererror").length > 0){
							data.error = {message:`Body error: ${dom.getElementsByTagName("parsererror")[0].getElementsByTagName("div")[0].innerText}`}
							return false;
						}else{
							data.metadata.body = codeSession.getValue();
						}

					}else{
						//Es texto plano
						data.metadata.body = codeSession.getValue();
					}
				}
			}else{
				this.$get("#flow_designer_add_http_api_body_request_parameters_container").find("[http_restfull_request_body_form_type_variable]").each((_,inputField)=>{
					self.__getAndValidateRequestBodyFormTypeInput($(inputField),data);
					if(data.error){
						return false;
					}
				})
			}


		}



		this.$get("[new_request_header]").each(function () {
			self.__getAndValidateRequestHeader($(this), data);
			if (data.error) {
				return false;
			}
		});

		this.$get("[new_query_parameters]").each(function () {
			self.__getAndValidateQueryParameters($(this), data);
			if (data.error) {
				return false;
			}
		});

		this.$get("[new_mapping_parameter]").each(function () {

			self.__getAndValidateMappingParameters($(this), data);
			if (data.error) {
				return false;
			}
		});
	};

	adm_process_flow_activity.prototype.htmlBody = function (data) {
		return Mustache.render(inConcert.templates.process_flows.activity_template, {
			image: `custom_icon_api_rest`,
			description: ((data.description && data.description !== "") ? data.description : "<%=_T('HTTP API RESTful')%>")
		});
	};

	/*
		Genera la configuración de puertos de la actividad
	*/
	adm_process_flow_activity.prototype.ports = function (data, node) {
		 //console.log("ports.data: ", data);
		 //console.log("ports.node: ", node);
		let ports = [];

		// siempre hay que agregar este por defecto
		ports.push({
			direction: "in", // in/out
			type: "", // error/success/option
			option_label: "Input" // error/success/option
		});
		let usedStatuses = [];
		// console.log("BROTHERS", node.parent.children.filter((childNode)=>{
		// 	try {
		// 		return JSON.parse(childNode.value).type === "http_rest_full"
		// 	} catch (error) {
		// 		return false;
		// 	}	
		// }).map((brother)=>{
		// 	return JSON.parse(brother.value);
		// }))
		
		data.ports.forEach((declaredPort)=>{
			//Vamos declarando los puertos configurados:
			//let existe = node ? node.children.find(c => c.id.includes(declaredPort.id)) : undefined;
			//console.log(declaredPort);
			let existe = node ? node.children.find(c => c.id.includes(declaredPort.id)) : undefined;
			if(!existe){
				declaredPort.id = inConcert.Utils.generateGuid()
			};
			ports.push({
				direction: "out", // in/out
				//type: (declaredPort.status.find((status)=>{return status >= 200 && status < 300})?"success":"option"), // error/success/option
				type: "option",
				option_label: declaredPort.name, // error/success/option
				channels: "1", // 0, 1 o n
				id: declaredPort.id,
				//id: existe ? existe.id: inConcert.Utils.generateGuid(),
				preset_value: {
					description: "",
					conditions: declaredPort.status.map((status, index)=>{
						usedStatuses.push(status);
						return {
							logic: index > 0 ? "or":undefined,
							field: data.result,
							comparator: "==",
							value: status
						}
					})
				},
				intern_value: undefined
			})
		})

		// agrego puerto único por defecto
		let existe = node ? node.children.find(c => c.id.includes("_default")) : undefined;
		ports.push({
			direction: "out", // in/out
			type: "error", // error/success/option
			option_label: "<%=_T('Error')%>", // error/success/option
			channels: "1", // 0, 1 o n
			id: (existe ? existe.id : inConcert.Utils.generateGuid() + "_default"),  // le asigno un id por defecto
			preset_value: {
				description: "",
				conditions: usedStatuses.map((status, index)=>{
					return {
						logic: index > 0 ? "and":undefined,
						field: data.result,
						comparator: "~=",
						value: status
					}
				})
			},
			intern_value: undefined
		});



		return ports;
	};

}());
