/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 9 */

/* global $, inConcert, Mustache */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	var templates = inConcert.templates.process_flows;
	var id = "reporting_data";

	function adm_process_flow_activity(editor, node) {
		var self = this;
		adm_process_flow_activity.uber.constructor.call(this, editor, node);

		this.frm = editor.main.frm;
		
		this.groupDefinition = {
			id : "group",
			name : "<%=_T('Reporting Group')%>",
			placeholder: "<%=_T('Enter reporting group')%>...",
			description : "<%=_T('Name of the reporting group. Empty if do not want to use data in reports.')%>",
			variable : true,
			not_extended : true,
			span: true,
			spanClass: "ic_variable_icon_container",
			controlContainerClass: "position-relative",
			variableIconClass: "ic_variable_icon far fa-at",
			form_group: true
		};
		
		this.applicationDefinition = {
			id : "applicationDefinition",
			name : "<%=_T('Application')%>",
			description : "<%=_T('Application')%>",
			required : false,
			not_extended : true,
			form_group: true,
			change : function() {
				self.__selectApplication($(this).val());
			}						
		};		
		
		this.applicationProperties = {
			id : "applicationProperties",
			name : "<%=_T('Property')%>",
			description : "<%=_T('Property')%>",
			required : false,
			not_extended : true,
			form_group: true	
		};			

		this.fieldTypeDefinition = {
			id : "fieldTypeDefinition",
			name : "<%=_T('Type')%>",
			description : "<%=_T('Field type.')%>",
			required : false,
			not_extended : true,
			form_group: true,
			options : [
				{
					id : "integer",
					name : "<%=_T('Integer')%>",
					disclaimer : "<%=_T('Integer')%>"
				},
				{
					id : "numeric",
					name : "<%=_T('Numeric')%>",
					disclaimer : "<%=_T('Numeric')%>"
				},
				{
					id : "string",
					name : "<%=_T('Text')%>",
					disclaimer : "<%=_T('Text')%>"
				},
				{
					id : "datetime",
					name : "<%=_T('Datetime')%>",
					disclaimer : "<%=_T('Datetime')%>"
				},
				{
					id : "date",
					name : "<%=_T('Date')%>",
					disclaimer : "<%=_T('Date')%>"
				},
				{
					id : "time",
					name : "<%=_T('Time')%>",
					disclaimer : "<%=_T('Time')%>"
				},
				{
					id : "boolean",
					name : "<%=_T('Boolean')%>",
					disclaimer : "<%=_T('Boolean')%>"
				}
			].filter(function(value) {
				return !value.channels || value.channels[editor.main.data.Type] || false;
			}),
			change : function() {
				self.__selectDestination($(this).val());
			}
		};

		this.focusFirst = true; 
	}

	inConcert.Utils.inherit( adm_process_flow_activity, inConcert.adm_process_flow_activities.base );

	if (typeof inConcert.adm_process_flow_activities[id] !== "function") {
		inConcert.adm_process_flow_activities[id] = adm_process_flow_activity;
	}

	adm_process_flow_activity.prototype.id = id;
	adm_process_flow_activity.prototype.name = "<%=_T('Manage Process Data')%>";
	adm_process_flow_activity.prototype.description = "<%=_T('Add or update data to be used in reports.')%>";
	adm_process_flow_activity.prototype.group = "interaction";
	adm_process_flow_activity.prototype.image = "ico-reporting_data.svg";

	adm_process_flow_activity.prototype.__makeField = function(field) {
		field = field || {};
		var self = this;
		var $field = $(Mustache.render(templates.manage_process_field, { guid : inConcert.Utils.generateGuid(), hidden : false }));

		//Carga del combo de aplicaciones
		let el = document.createElement("option");
		el.textContent = "";
		el.value = "";
		$field.find("[field_application]")[0].appendChild(el);
		$.each(this.applicationDefinition.options, function(_, option) {
			if (typeof option === "string") {
				option = { id : option, name : option };
			}
			el = document.createElement("option");
			el.textContent = option.name;
			el.value = option.id;				
			$field.find("[field_application]")[0].appendChild(el);
		});
		//Seleccionamos en el combo de aplicaciones la aplicacion asociada al field
		self.__selectApplication(field, $field);		
			
		$field.find("[field_name]").val(field.name || "");
		$field.find("[field_value]").val(field.value || "");	
		
		$field.find("[field_sendToReport]").prop("checked", field.sendToReport || false);
		$field.find("[field_sendToAgent]").prop("checked", field.sendToAgent || false);
		$field.find("[field_hidden]").prop("checked", field.hidden || false);
		//El checkbox hidden solo se usa para campos que se notifican al agente
		$field.find("[field_hidden]").prop("disabled", !field.sendToAgent);

		//El checkbox type solo se usa para campos que se reportan
		$field.find("[field_searchable]").prop("disabled", !field.sendToReport);
		$field.find("[field_searchable]").prop("checked", field.searchable && field.sendToReport);

		//Carga del combo de tipos
		el = document.createElement("option");
		el.textContent = "";
		el.value = "";
		$field.find("[field_type]")[0].appendChild(el);
		$.each(this.fieldTypeDefinition.options, function(_, option) {
			if (typeof option === "string") {
				option = { id : option, name : option };
			}
			el = document.createElement("option");
			el.textContent = option.name;
			el.value = option.id;				
			$field.find("[field_type]")[0].appendChild(el);
		});
		//Seleccionamos en el combo de tipos el tipo asociado al field
		self.__selectType(field, $field);

		//El combo type solo se usa para campos que se reportan y son searchables
		$field.find("[field_type]").prop("disabled", !field.sendToReport || !field.searchable);
		$field.find("[field_type]").val(field.sendToReport ? field.type : "");
		
		$field.find("[field_application]").change(function() {
			field.sendToApplication = $(this).val();
			self.__selectApplication(field, $field);	
			field.name = "";		
			$field.find("[field_name]").val("");	
		});
		
		$field.find("[field_name_property]").change(function() {
			field.name = $(this).val();		
			$field.find("[field_name]").val(field.name || "");
		});		
		
		$field.find("[field_sendToAgent]").click(function(){
			$field.find("[field_hidden]").prop("disabled", !this.checked);
			if(!this.checked)
			{
				$field.find("[field_hidden]").prop("checked", false);
			}
		});		
		
		$field.find("[field_sendToReport]").click(function(){
			$field.find("[field_searchable]").prop("disabled", !this.checked);
			$field.find("[field_searchable]").prop("checked", field.searchable && this.checked);
			$field.find("[field_type]").prop("disabled", !this.checked || !field.searchable);
			$field.find("[field_type]").val(this.checked && field.searchable ? field.type : "");
		});		
		
		$field.find("[field_searchable]").click(function(){
			$field.find("[field_type]").prop("disabled", !this.checked);
			$field.find("[field_type]").val(this.checked ? field.type : "");
		});	

		$field.find("[add]").click(function() {
			self.__disableNextKeyboardEvent();
			var $row = $(this).parent().parent();
			var $newRow = self.__makeField();
			$row.after($newRow);
			$row.parent().scrollTo($newRow);
			$newRow.find("[field_name]").focus();
		});
		$field.find("[remove]").click(function() {
			self.__disableNextKeyboardEvent();
			var $row = $(this).parent().parent();
			var $siblings = $row.siblings();
			if ($siblings.length) {
				if ($row.prev().length) {
					$row.prev().find("[field_name]").focus();
				}
				else {
					$row.next().find("[field_name]").focus();
				}
			}
			else {
				var $newRow = self.__makeField();
				$row.after($newRow);
				$newRow.find("[field_name]").focus();
			}
			$row.remove();
		});
		return $field;
	};

	adm_process_flow_activity.prototype.__makeFields = function() {
		var self = this;
		var fields = self.data.fields || [];
		var $fields = $(templates.report_fields);
		var $fieldsList = $fields.find("[fields]");
		var $list = [];

		$.each(fields, function(_, field) {
			$list.push(self.__makeField(field));
		});
		if (!$list.length) {
			$list.push(self.__makeField());
		}
		$fieldsList.append($list);
		return $fields;
	};

	adm_process_flow_activity.prototype.__afterAppendBody = function() { 
		const self = this;
		const $inputs = self.$get("span[class=ic_variable_icon_container");
		self.__addAutocompleteToInput($inputs);
	};

	adm_process_flow_activity.prototype.__makeContent = function() {
		var $content = adm_process_flow_activity.uber.__makeContent.call(this);

		$content.unshift(
			this.__makeDivFlex(
			  [
				this.__makeDivFlex(
				  [
					this.__makeRowFlex(this.__makeInputFlex(this.groupDefinition, this.data[this.groupDefinition.id])),
					this.__makeFields()
				  ],
				  { class: "card-block" }
				),
			  ],
			  { class: "card no-card" }
			)
		  );
			
		return $content;
	};

	adm_process_flow_activity.prototype.__getAndValidateField = function($field, data, fields_d) {
		fields_d = fields_d || {};
		var field = {
			name : $field.find("[field_name]").val() || undefined,
			value : $field.find("[field_value]").val() || undefined,
			sendToApplication : $field.find("[field_application]").val() || undefined,
			sendToReport : $field.find("[field_sendToReport]").prop("checked"),
			sendToAgent : $field.find("[field_sendToAgent]").prop("checked"),
			hidden : $field.find("[field_hidden]").prop("checked"),
			searchable: $field.find("[field_searchable]").prop("checked"),
			type: $field.find("[field_type]").val() || undefined
		};
		//validation
		var error = null;
		if (!field.name) {
			error = "<%=_T('Must enter a name.')%>";
		}
		else if (fields_d[field.name]) {
			error = "<%=_T('Name must be unique.')%>";
		}
		else if (!field.value) {
			error = "<%=_T('Must enter a value.')%>";
		}
		else if (field.sendToReport && field.searchable && !field.type) {
			error = "<%=_T('Must enter a type.')%>";
		}
		if (error) {
			data.error = {
				message : error,
				$control : $field
			};
		}
		data.fields.push(field);
		fields_d[field.name] = true;
	};

	adm_process_flow_activity.prototype.__clearErrors = function() {
		adm_process_flow_activity.uber.__clearErrors.call(this);
		this.$get("[entry_validation]").hide();
	};

	adm_process_flow_activity.prototype.__getAndValidateFields = function(data) {
		var self = this;
		data.fields = [];
		var fields_d = {};
		self.$get("[field]").each(function() {
			self.__getAndValidateField($(this), data, fields_d);
			if (data.error) {
				return false;
			}
		});
		if (!data.error && !data.fields.length) {
			data.error = {
				message : "<%=_T('Must define almost one field.')%>"
			};
		}
	};

	adm_process_flow_activity.prototype.__getAndValidate = function() {
		var data = adm_process_flow_activity.uber.__getAndValidate.call(this);
		if (!data.error) {
			this.__addValue(data, this.groupDefinition);
			this.__getAndValidateFields(data);
		}
		return data;
	};

	adm_process_flow_activity.prototype.__load = function(callback) {
		var self = this;
		// El combo de Aplicaciones lo rellenamos con las AppsNotifications
		self.editor.main.getAppsNotifications(function(success, response) {
			if(success && response.rows && Array.isArray(response.rows))
			{
				const vccInfo = inConcert.app.get_vcc_info();
				var appsNotifications = [];
				$.each(response.rows, function(_, row) {
					var appData = {};
					//Si se tiene integracion con la Application, recuperamos los datos del provissioning
					$.each(vccInfo.AppsIntegrations, function(dsId, dsData) {
						if(dsId === row.Id)
						{
							appData = dsData;
						}
					});
					appsNotifications.push({
						id : row.Id,
						name : row.Id,
						data : appData
					});
				});
				appsNotifications.sort(function(a, b) {
					return a.name.localeCompare(b.name);
				});
				self.applicationDefinition.options = appsNotifications;
			}
			callback();
		});
	};			
	
	adm_process_flow_activity.prototype.__selectApplication = function(field, $field) {
		var self = this;
		if(field.sendToApplication)
		{
			$field.find("[field_application]")[0].value = field.sendToApplication;
			var apps = this.applicationDefinition.options.filter(function(app) {
				return app.id === field.sendToApplication;
			});				
			
			if(apps && Array.isArray(apps) && apps.length === 1)
			{
				self.editor.main.getAppIntegrationProperties(apps[0], function(success, response) {	
					self.__reloadProperties(field, $field, success, response);
				});			
			}
			else
			{
				//La app ya no eciste o no esta activa
				field.sendToApplication = "";
				$field.find("[field_name]").hidden = false;
				$field.find("[field_name_property]").hidden = true;	
			}
		}
		else
		{
			$field.find("[field_name]").hidden = false;
			$field.find("[field_name_property]").hidden = true;	
		}		
	};

	adm_process_flow_activity.prototype.__selectType = function(field, $field) {
		if(field.type)
		{
			$field.find("[field_type]")[0].value = field.type;
		}		
	};
	
	adm_process_flow_activity.prototype.__reloadProperties = function(field, $field, success, response) {					
		if(success && response.status)
		{						
			if(response.data && Array.isArray(response.data))
			{
				var properties = response.data.filter(function(property) {
					return property.scope === "contact" && (property.editable === true || property.readOnly !== true);
				});	
				var properties_a = [];
				properties_a.push({
					id : "",
					name : "",
					data : undefined
				});
				properties.forEach(function(property)
				{
					properties_a.push({
						id : property.id,
						name : property.id,
						data : property
					});
				});
				properties_a.sort(function(a, b) {
					return a.name.localeCompare(b.name);
				});			
				$.each(properties_a, function(_, property) {
					let el = document.createElement("option");
					el.textContent = property.name;
					el.value = property.id;				
					$field.find("[field_name_property]").appendChild(el);
					//Seleccionamos en el combo de properties el valor indicado en el field
					if(field.name === property.name)
					{
						$field.find("[field_name_property]").value = field.name;
					}
				});							
				$field.find("[field_name]").hidden = true;
				$field.find("[field_name_property]").hidden = false;							
			}
			else
			{
				$field.find("[field_name]").hidden = false;
				$field.find("[field_name_property]").hidden = true;	
			}
		}
		else
		{
			$field.find("[field_name]").hidden = false;
			$field.find("[field_name_property]").hidden = true;	
		}
	};
	
	adm_process_flow_activity.prototype.__getClassName = function() {
		return "popOver940w flowDesignerGlobalModal botActivity_reporting_data popOverFixedHeight";
	};


	// adm_process_flow_activity.prototype.htmlBody = function(data) {
	// 	return Mustache.render(inConcert.templates.process_flows.reporting_data_activity_template, {
	// 		image: `custom_icon_reporting_data`, 
	// 		description: ((data.description && data.description !== "")?data.description:"<%=_T('Manage Process Data')%>") 
	// 	});
	// };

	adm_process_flow_activity.prototype.htmlBody = function (data) {

		var image = "";

		if (data) {
			var type = data.message && data.message.msgtype || "text";
			if (type === "attachment") {
				type = data.message.fileType || "attachment";
			}
			image = id + "_" + type;
		}

		return Mustache.render(inConcert.templates.process_flows.activity_template, {
			image: `custom_icon_reporting_data`, 
			description: ((data.description && data.description !== "")?data.description:"<%=_T('Manage Process Data')%>") 
		});
	};

	adm_process_flow_activity.prototype.ports = function(data, node) {
		

		let ports = [];

		// siempre hay que agregar este por defecto
		ports.push({
			direction : "in", // in/out
			type : "option", // error/success/option
			option_label : "Input" // error/success/option
		});

		// agrego puerto único por defecto
		let existe = node?node.children.find(c => c.id.includes("_default")):undefined;
		const portId = (existe?existe.id:inConcert.Utils.generateGuid() + "_default");
		ports.push({
			direction : "out", // in/out
			type : "option", // error/success/option/none
			option_label : "<%=_T('Default output')%>", // error/success/option
			channels: "n", // 0, 1 o n 
			id: portId,  // le asigno un id por defecto
			preset_value: undefined,
			intern_value: undefined
		});

		return ports;
	};

}());
