/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 9 */

/* global inConcert, Mustache */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	var id = "send_to_cc";

	function adm_process_flow_activity(editor, node) {
		adm_process_flow_activity.uber.constructor.call(this, editor, node);

		var self = this;
		self.frm = editor.main.frm;

		this.destinationDefinition = {
			id: "destination",
			name: "<%=_T('Destination')%>",
			description: "<%=_T('Transfer destination.')%>",
			required: true,
			not_extended: true,
			form_group: true,
			options: [
				{
					id: "contact_center",
					name: "<%=_T('Contact Center')%>",
					disclaimer: "<%=_T('Send interaction to current campaign contact center.')%>"
				},
				{
					id: "agent",
					name: "<%=_T('Agent')%>",
					disclaimer: "<%=_T('Send interaction to a specific agent.')%>"
				},
				{
					id: "campaign",
					name: "<%=_T('Campaign')%>",
					disclaimer: "<%=_T('Send interaction to another campaign.')%>",
					channels: { CALL: true, CHAT: true, MAIL: true, SMS: true, WHATSAPP: true }
				},
				{
					id: "campaign_agent",
					name: "<%=_T('Campaign and Agent')%>",
					disclaimer: "<%=_T('Send interaction to a specific agent in another campaign.')%>",
					channels: { CALL: true, CHAT: true, MAIL: true, SMS: true, WHATSAPP: true }
				},
				{
					id: "number",
					name: "<%=_T('Number')%>",
					disclaimer: "<%=_T('Send interaction to a specific phone number.')%>",
					channels: { CALL: true }
				}
			].filter(function (value) {
				return !value.channels || value.channels[editor.main.data.Type] || false;
			}),
			change: function () {
				self.__selectDestination($(this).val());
			}
		};

		this.campaignDefinition = {
			id: "toCampaign",
			name: "<%=_T('Campaign')%>",
			description: "<%=_T('Id of campaign to send interaction.')%>",
			required: true,
			not_extended: true,
			variable: true,
			span: true,
			spanClass: "ic_variable_icon_container",
			variableIconClass: "ic_variable_icon far fa-at",
			form_group: true
		};

		this.accountDefinition = {
			id: "toAccount",
			name: "<%=_T('Account')%>",
			description: "<%=_T('Id or name of account to send interaction.')%>",
			required: true,
			not_extended: true,
			variable: true,
			span: true,
			spanClass: "ic_variable_icon_container",
			variableIconClass: "ic_variable_icon far fa-at",
			form_group: true
		};

		this.attentionLevelDefinition = {
			id: "toAttentionLevel",
			name: "<%=_T('Attention Level')%>",
			description: "<%=_T('Id or name of attention level to send interaction.')%>",
			not_extended: true,
			variable: true,
			span: true,
			spanClass: "ic_variable_icon_container",
			variableIconClass: "ic_variable_icon far fa-at",
			form_group: true
		};

		this.agentDefinition = {
			id: "requiredAgent",
			name: "<%=_T('Agents')%>",
			description: "<%=_T('Id of agent to send interaction.')%>",
			required: true,
			not_extended: true,
			variable: true,
			span: true,
			spanClass: "ic_variable_icon_container",
			variableIconClass: "ic_variable_icon far fa-at",
			form_group: true
		};

		this.countryCodeDefinition = {
			id: "countryCode",
			name: "<%=_T('Country')%>",
			description: "<%=_T('Phone number country code.')%>",
			not_extended: true,
			emptyLabel: "<%=_T('Global')%>",
			form_group: true,
			change: function () {
				self.__selectCountryCode($(this).val());
			}
		};

		this.areaCodeDefinition = {
			id: "areaCode",
			name: "<%=_T('Area')%>",
			description: "<%=_T('Phone number area code.')%>",
			not_extended: true,
			emptyLabel: "<%=_T('Nothing')%>",
			form_group: true
		};

		this.numberDefinition = {
			id: "number",
			name: "<%=_T('Number')%>",
			description: "<%=_T('Phone number to send interaction.')%>",
			disclaimer: "<%=_T('Global numbers must start with +.')%>",
			required: true,
			not_extended: true,
			variable: true,
			span: true,
			spanClass: "ic_variable_icon_container",
			variableIconClass: "ic_variable_icon far fa-at",
			form_group: true
		};

		this.preferredAgentsDefinition = {
			id: "preferredAgents",
			name: "<%=_T('Preferred Agents')%>",
			description: "<%=_T('An array with preferred agents to send interaction.')%>",
			not_extended: true,
			variable: true,
			span: true,
			spanClass: "ic_variable_icon_container",
			variableIconClass: "ic_variable_icon far fa-at",
			form_group: true
		};

		this.forbiddenAgentsDefinition = {
			id: "forbiddenAgents",
			name: "<%=_T('Forbidden Agents')%>",
			description: "<%=_T('An array with forbidden agents to send interaction.')%>",
			not_extended: true,
			variable: true,
			span: true,
			spanClass: "ic_variable_icon_container",
			variableIconClass: "ic_variable_icon far fa-at",
			form_group: true
		};

		this.skipFlowDefinition = {
			id: "skipFlow",
			name: "<%=_T('Skip flow')%>",
			description: "<%=_T('Skip destination campaign flow and go direct to contact center.')%>",
			not_extended: true,
			form_group: false,
			envolving_label_class: "form-check-label form-check-label-title",
			envolving_label: true,
			without_label: true,
			controlContainerClass: "form-check mt-0"

		};

		this.priorityDefinition = {
			id: "toPriority",
			name: "<%=_T('Priority')%>",
			description: "<%=_T('Priority value to send interaction.')%>",
			not_extended: true,
			required: false,
			defaultValue: "0",
			variable: true,
			span: true,
			spanClass: "ic_variable_icon_container",
			variableIconClass: "ic_variable_icon far fa-at",
			fieldClass: "Short",
			form_group: true,
			validate: {
				range: {
					type: "integer",
					allowVariables: true,
					min: 0,
					max: 99
				}
			},
			change: function () {
				if ($("[control='toPriority']").val().trim() === "") {
					$("[control='toPriority']").val("0");
				}
			}
		};
		this.actionPriorityDefinition = {
			id: "toActionPriority",
			name: "<%=_T('Action priority')%>",
			not_extended: true,
			required: true,
			form_group: true,
			options: [
				{
					id: "Overwrite",
					name: "<%=_T('Overwrite value')%>"
				},
				{
					id: "Increase",
					name: "<%=_T('Increase value')%>"
				}
			]
		};

		this.enableACDWaitThresholdDefinition = {
			id: "enableACDWaitThreshold",
			name: "<%=_T('ACD Wait Threshold')%>",
			description: "<%=_T('Enable timeout waiting for agent and actions in case of timeout.')%>",
			not_extended: true,
			form_group: false,
			envolving_label_class: "form-check-label form-check-label-title",
			envolving_label: true,
			without_label: true,
			controlContainerClass: "form-check mt-0",
			checkBoxClass: "form-check-input",
			change: function () {
				self.__enableACDTimeoutControls();
			}
		};

		this.ACDWaitThresholdDefinition = {
			id: "ACDWaitThreshold",
			name: "<%=_T('Time (sec)')%>",
			description: "<%=_T('Timeout waiting for agent.')%>",
			required: true,
			not_extended: true,
			variable: true,
			span: true,
			spanClass: "ic_variable_icon_container",
			variableIconClass: "ic_variable_icon far fa-at",
			form_group: true
		};

		this.actionOnTimeoutDefinition = {
			id: "actionOnTimeout",
			name: "<%=_T('Action on timeout')%>",
			description: "<%=_T('Interaction action on ACD timeout.')%>",
			required: true,
			not_extended: true,
			form_group: true,
			options: [
				{
					id: "TakeAccountConfig",
					name: "<%=_T('Take Account Config')%>"
				},
				{
					id: "ContinueExecution",
					name: "<%=_T('Continue Execution')%>"
				},
				{
					id: "Enqueue",
					name: "<%=_T('Enqueue')%>"
				},
				{
					id: "Finalize",
					name: "<%=_T('Finalize Conversation')%>"
				},
				{
					id: "TransferToCampaign",
					name: "<%=_T('Transfer to campaign')%>"
				},
			],
			change: function () {
				if ($("[control='actionOnTimeout']").val() === "TransferToCampaign") {
					self.__showControl(self.onTimeoutCampaignDefinition);
					self.__showControl(self.onTimeoutAccountDefinition);
					self.onTimeoutCampaignDefinition.required = true;
					self.onTimeoutAccountDefinition.required = true;
				}
				else {
					self.__hideControl(self.onTimeoutCampaignDefinition);
					self.__hideControl(self.onTimeoutAccountDefinition);
					self.onTimeoutCampaignDefinition.required = false;
					self.onTimeoutAccountDefinition.required = false;
				}
			}
		};

		this.onTimeoutCampaignDefinition = {
			id: "toCampaignOnTimeout",
			name: "<%=_T('Transfer to campaign on timeout')%>",
			description: "<%=_T('Id of campaign to send interaction on timeout.')%>",
			required: true,
			not_extended: true,
			variable: true,
			span: true,
			spanClass: "ic_variable_icon_container",
			variableIconClass: "ic_variable_icon far fa-at",
			form_group: true
		};

		this.onTimeoutAccountDefinition = {
			id: "toAccountOnTimeout",
			name: "<%=_T('Transfer to account on timeout')%>",
			description: "<%=_T('Id or name of account to send interaction on timeout.')%>",
			required: true,
			not_extended: true,
			variable: true,
			span: true,
			spanClass: "ic_variable_icon_container",
			variableIconClass: "ic_variable_icon far fa-at",
			form_group: true
		};

		this.resultDefinition = {
			id: "result",
			name: "<%=_T('Result')%>",
			placeholder: "<%=_T('Enter result')%>...",
			description: "<%=_T('Variable name to store result: object { status : boolean, event : object|nil with result event, eventname : Transferred|Error|ChannelClosed indicating event name (ChannelClosed if client closes channel while is transferring) }.')%>",
			form_group: true
		};

		this.focusControl = this.destinationDefinition;
	}

	inConcert.Utils.inherit(adm_process_flow_activity, inConcert.adm_process_flow_activities.base);

	if (typeof inConcert.adm_process_flow_activities[id] !== "function") {
		inConcert.adm_process_flow_activities[id] = adm_process_flow_activity;
	}

	adm_process_flow_activity.prototype.id = id;
	adm_process_flow_activity.prototype.name = "<%=_T('Transfer')%>";
	adm_process_flow_activity.prototype.description = "<%=_T('Transfer interaction.')%>";
	adm_process_flow_activity.prototype.group = "interaction";
	adm_process_flow_activity.prototype.image = "ico-send-to-contact-center.svg";

	adm_process_flow_activity.prototype.dataStyle = function (data) {
		return id + "_" + (data.destination || "contact_center");
	};

	adm_process_flow_activity.prototype.dataStyles = function () {
		return [
			{ id: id + "_contact_center", imageName: "ico-send-to-contact-center.svg" },
			{ id: id + "_agent", imageName: "ico-send-to-agent.svg" },
			{ id: id + "_campaign", imageName: "ico-send-to-campaign.svg" },
			{ id: id + "_campaign_agent", imageName: "ico-send-to-campaign-agent.svg" },
			{ id: id + "_number", imageName: "ico-send-to-number.svg" }
		];
	};

	adm_process_flow_activity.prototype.__load = function (callback) {
		const self = this;
		self.editor.main.LoadTelephonyCodes(function () {
			self.countryCodeDefinition.options = self.editor.main.GetTelephonyCountryCodes();
			self.areaCodeDefinition.options = [];
			callback();
		});
	};

	adm_process_flow_activity.prototype.__getClassName = function() { 
		return "popOver640w flowDesignerGlobalModal botActivity_transfer";
	};

	adm_process_flow_activity.prototype.__makeContent = function () { 
		var $content = adm_process_flow_activity.uber.__makeContent.call(this);

		$content.unshift(
			this.__makeDivFlex(
			  [
				this.__makeDivFlex(
				  [
					this.__makeHiddenDivFlex(
					  [
						this.__makeDivFlex(
						  this.__makeComboFlex(
							this.destinationDefinition,
							this.data[this.destinationDefinition.id]
						  ),
						  { class: "col-12" }
						),
					  ],
					  { class: "row" }
					),
					this.__makeHiddenDivFlex(
					  [
						this.__makeDivFlex(
						  this.__makeComboFlex(
							this.countryCodeDefinition,
							this.data[this.countryCodeDefinition.id]
						  ),
						  { class: "col-6" }
						),
						this.__makeDivFlex(
						  this.__makeComboFlex(
							this.areaCodeDefinition,
							this.data[this.areaCodeDefinition.id]
						  ),
						  { class: "col-6" }
						),
					  ],
					  { class: "row" }
					),
					this.__makeHiddenDivFlex(
					  [
						this.__makeDivFlex(
						  this.__makeInputFlex(
							this.numberDefinition,
							this.data[this.numberDefinition.id]
						  ),
						  { class: "col-12" }
						),
					  ],
					  { class: "row" }
					),
					this.__makeHiddenDivFlex(
					  [
						this.__makeDivFlex(
						  this.__makeInputFlex(
							this.campaignDefinition,
							this.data[this.campaignDefinition.id]
						  ),
						  { class: "col-6" }
						),
						this.__makeDivFlex(
						  this.__makeInputFlex(
							this.accountDefinition,
							this.data[this.accountDefinition.id]
						  ),
						  { class: "col-6" }
						),
					  ],
					  { class: "row" }
					),
					this.__makeHiddenDivFlex(
					  [
						this.__makeDivFlex(
						  this.__makeInputFlex(
							this.attentionLevelDefinition,
							this.data[this.attentionLevelDefinition.id]
						  ),
						  { class: "col-12" }
						),
					  ],
					  { class: "row" }
					),
					this.__makeHiddenDivFlex(
					  [
						this.__makeDivFlex(
						  this.__makeInputFlex(
							this.agentDefinition,
							this.data[this.agentDefinition.id]
						  ),
						  { class: "col-12" }
						),
					  ],
					  { class: "row" }
					),
					this.__makeHiddenDivFlex(
					  [
						this.__makeDivFlex(
						  this.__makeInputFlex(
							this.preferredAgentsDefinition,
							this.data[this.preferredAgentsDefinition.id]
						  ),
						  { class: "col-6" }
						),
						this.__makeDivFlex(
						  this.__makeInputFlex(
							this.forbiddenAgentsDefinition,
							this.data[this.forbiddenAgentsDefinition.id]
						  ),
						  { class: "col-6" }
						),
					  ],
					  { class: "row" }
					),
					this.__makeHiddenDivFlex(
					  [
						this.__makeDivFlex(
						  this.__makeInputFlex(
							this.priorityDefinition,
							this.data[this.priorityDefinition.id]
						  ),
						  { class: "col-6" }
						),
						this.__makeDivFlex(
						  this.__makeComboFlex(
							this.actionPriorityDefinition,
							this.data[this.actionPriorityDefinition.id]
						  ),
						  { class: "col-6" }
						),
					  ],
					  { class: "row" }
					),
					this.__makeHiddenDivFlex(
					  [
						this.__makeDivFlex(
						  this.__makeDivFlex(
							this.__makeCheckbox(
							  this.enableACDWaitThresholdDefinition,
							  this.data[this.enableACDWaitThresholdDefinition.id]
							),
							{ class: "form-group" }
						  ),
						  { class: "col-6" }
						),
					  ],
					  { class: "row" }
					),
					this.__makeHiddenDivFlex(
					  [
						this.__makeDivFlex(
						  this.__makeComboFlex(
							this.actionOnTimeoutDefinition,
							this.data[this.actionOnTimeoutDefinition.id]
						  ),
						  { class: "col-6" }
						),
						this.__makeDivFlex(
						  //////////////////////
						  this.__makeInputFlex(
							this.ACDWaitThresholdDefinition,
							this.data[this.ACDWaitThresholdDefinition.id]
						  ),
						  { class: "col-6" }
						),
						this.__makeDivFlex(
						  this.__makeDivFlex(
							this.__makeCheckbox(
							  this.skipFlowDefinition,
							  this.data[this.skipFlowDefinition.id]
							),
							{ class: "form-group" }
						  ),
						  { class: "col-6" }
						),
					  ],
					  { class: "row" }
					),
					this.__makeHiddenDivFlex(
					  [
						this.__makeDivFlex(
						  this.__makeInputFlex(
							this.onTimeoutCampaignDefinition,
							this.data[this.onTimeoutCampaignDefinition.id]
						  ),
						  { class: "col-6" }
						),
						this.__makeDivFlex(
						  this.__makeInputFlex(
							this.onTimeoutAccountDefinition,
							this.data[this.onTimeoutAccountDefinition.id]
						  ),
						  { class: "col-6" }
						),
					  ],
					  { class: "row" }
					),
					this.__makeDivFlex(
					  [
						this.__makeDivFlex(
						  this.__makeInputFlex(
							this.resultDefinition,
							this.data[this.resultDefinition.id]
						  ),
						  { class: "col-12" }
						),
					  ],
					  { class: "row" }
					),
				  ],
				  { class: "card-block" }
				),
			  ],
			  { class: "card no-card" }
			)
		  );		  

		return $content;
	};

	adm_process_flow_activity.prototype.__afterAppendBody = function () {
		const self = this;
		this.__selectDestination(this.data[this.destinationDefinition.id] || "contact_center", true);
		this.__enableACDTimeoutControls(this.data[this.enableACDWaitThresholdDefinition.id]);
		const $inputs = self.$get("span[class=ic_variable_icon_container");
		self.__addAutocompleteToInput($inputs);
	};

	adm_process_flow_activity.prototype.__selectDestination = function (destination, initial) {
		switch (destination) {
			case "contact_center":
				this.__selectContactCenter(initial);
				break;
			case "agent":
				this.__selectAgent(initial);
				break;
			case "campaign":
				this.__selectCampaign(initial);
				break;
			case "campaign_agent":
				this.__selectCampaignAgent(initial);
				break;
			case "number":
				this.__selectNumber(initial);
				break;
		}
		this.__enableACDTimeoutControls();
	};

	adm_process_flow_activity.prototype.__selectContactCenter = function () {
		this.__hideControl(this.campaignDefinition);
		this.__hideControl(this.accountDefinition);
		this.__hideControl(this.agentDefinition);
		this.__hideControl(this.countryCodeDefinition);
		this.__hideControl(this.areaCodeDefinition);
		this.__hideControl(this.numberDefinition);
		this.__hideControl(this.skipFlowDefinition);
		this.__showControl(this.attentionLevelDefinition);
		this.__showControl(this.preferredAgentsDefinition);
		this.__showControl(this.forbiddenAgentsDefinition);
		this.__showControl(this.priorityDefinition);
		this.__showControl(this.actionPriorityDefinition);
		this.__showControl(this.enableACDWaitThresholdDefinition);
	};

	adm_process_flow_activity.prototype.__selectAgent = function () {
		this.__hideControl(this.campaignDefinition);
		this.__hideControl(this.accountDefinition);
		this.__hideControl(this.countryCodeDefinition);
		this.__hideControl(this.areaCodeDefinition);
		this.__hideControl(this.numberDefinition);
		this.__hideControl(this.preferredAgentsDefinition);
		this.__hideControl(this.forbiddenAgentsDefinition);
		this.__hideControl(this.skipFlowDefinition);
		this.__showControl(this.enableACDWaitThresholdDefinition);
		this.__showControl(this.agentDefinition);
		this.__showControl(this.attentionLevelDefinition);
		this.__showControl(this.priorityDefinition);
		this.__showControl(this.actionPriorityDefinition);
	};

	adm_process_flow_activity.prototype.__selectCampaign = function () {
		this.__hideControl(this.agentDefinition);
		this.__hideControl(this.countryCodeDefinition);
		this.__hideControl(this.areaCodeDefinition);
		this.__hideControl(this.numberDefinition);
		this.__hideControl(this.enableACDWaitThresholdDefinition);
		this.__hideControl(this.priorityDefinition);
		this.__hideControl(this.actionPriorityDefinition);
		this.__showControl(this.campaignDefinition);
		this.__showControl(this.accountDefinition);
		this.__showControl(this.attentionLevelDefinition);
		this.__showControl(this.preferredAgentsDefinition);
		this.__showControl(this.forbiddenAgentsDefinition);
		this.__showControl(this.skipFlowDefinition);
	};

	adm_process_flow_activity.prototype.__selectCampaignAgent = function () {
		this.__hideControl(this.countryCodeDefinition);
		this.__hideControl(this.areaCodeDefinition);
		this.__hideControl(this.numberDefinition);
		this.__hideControl(this.preferredAgentsDefinition);
		this.__hideControl(this.forbiddenAgentsDefinition);
		this.__hideControl(this.enableACDWaitThresholdDefinition);
		this.__hideControl(this.priorityDefinition);
		this.__hideControl(this.actionPriorityDefinition);
		this.__showControl(this.campaignDefinition);
		this.__showControl(this.accountDefinition);
		this.__showControl(this.agentDefinition);
		this.__showControl(this.attentionLevelDefinition);
		this.__showControl(this.skipFlowDefinition);
	};

	adm_process_flow_activity.prototype.__selectNumber = function (initial) {
		this.__hideControl(this.campaignDefinition);
		this.__hideControl(this.accountDefinition);
		this.__hideControl(this.preferredAgentsDefinition);
		this.__hideControl(this.forbiddenAgentsDefinition);
		this.__hideControl(this.skipFlowDefinition);
		this.__hideControl(this.enableACDWaitThresholdDefinition);
		this.__hideControl(this.agentDefinition);
		this.__hideControl(this.attentionLevelDefinition);
		this.__hideControl(this.priorityDefinition);
		this.__hideControl(this.actionPriorityDefinition);

		this.__showControl(this.countryCodeDefinition);
		this.__showControl(this.areaCodeDefinition);
		this.__showControl(this.numberDefinition);
		if (initial) {
			this.__selectCountryCode(this.__getValue(this.countryCodeDefinition), true);
		}
	};

	adm_process_flow_activity.prototype.__enableACDTimeoutControls = function (enable) {
		let isCheckedACDWaitThreshold = $("[control='enableACDWaitThreshold']").is(":checked");
		let hasDestinationACDWaitThreshold = $("[control='destination']").val() === "contact_center" || $("[control='destination']").val() === "agent";
		enable = enable !== undefined ? enable : isCheckedACDWaitThreshold && hasDestinationACDWaitThreshold;
		this.__toggleControl(this.ACDWaitThresholdDefinition, enable);
		this.ACDWaitThresholdDefinition.required = enable;
		this.__toggleControl(this.actionOnTimeoutDefinition, enable);
		this.actionOnTimeoutDefinition.required = enable;
		this.__toggleControl(this.onTimeoutCampaignDefinition, enable && $("[control='actionOnTimeout']").val() === "TransferToCampaign");
		this.onTimeoutCampaignDefinition.required = enable && $("[control='actionOnTimeout']").val() === "TransferToCampaign";
		this.__toggleControl(this.onTimeoutAccountDefinition, enable && $("[control='actionOnTimeout']").val() === "TransferToCampaign");
		this.onTimeoutAccountDefinition.required = enable && $("[control='actionOnTimeout']").val() === "TransferToCampaign";
	};

	adm_process_flow_activity.prototype.__selectCountryCode = function (countryCode, initial) {
		this.areaCodeDefinition.options = this.editor.main.GetTelephonyAreaCodes(countryCode);
		let areaCode = initial ? this.data[this.areaCodeDefinition.id] : null;
		this.__replaceControl(this.areaCodeDefinition, this.__makeComboFlex(this.areaCodeDefinition, areaCode));
	};

	adm_process_flow_activity.prototype.__getAndValidate = function () {
		var data = adm_process_flow_activity.uber.__getAndValidate.call(this);
		this.__addValue(data, this.resultDefinition);
		this.__addValue(data, this.destinationDefinition);
		switch (data[this.destinationDefinition.id]) {
			case "contact_center":
				this.__getAndValidateContactCenter(data);
				break;
			case "agent":
				this.__getAndValidateAgent(data);
				break;
			case "campaign":
				this.__getAndValidateCampaign(data);
				break;
			case "campaign_agent":
				this.__getAndValidateCampaignAgent(data);
				break;
			case "number":
				this.__getAndValidateNumber(data);
				break;
		}
		return data;
	};

	adm_process_flow_activity.prototype.__getAndValidateContactCenter = function (data) {
		this.__addValue(data, this.actionOnTimeoutDefinition);
		this.__addValue(data, this.onTimeoutCampaignDefinition);
		this.__addValue(data, this.onTimeoutAccountDefinition);
		this.__addValue(data, this.ACDWaitThresholdDefinition);
		this.__addValue(data, this.enableACDWaitThresholdDefinition);
		this.__addValue(data, this.forbiddenAgentsDefinition);
		this.__addValue(data, this.preferredAgentsDefinition);
		this.__addValue(data, this.attentionLevelDefinition);
		this.__addValue(data, this.priorityDefinition);
		this.__addValue(data, this.actionPriorityDefinition);
	};

	adm_process_flow_activity.prototype.__getAndValidateAgent = function (data) {
		this.__addValue(data, this.actionOnTimeoutDefinition);
		this.__addValue(data, this.onTimeoutCampaignDefinition);
		this.__addValue(data, this.onTimeoutAccountDefinition);
		this.__addValue(data, this.ACDWaitThresholdDefinition);
		this.__addValue(data, this.enableACDWaitThresholdDefinition);
		this.__addValue(data, this.agentDefinition);
		this.__addValue(data, this.attentionLevelDefinition);
		this.__addValue(data, this.priorityDefinition);
		this.__addValue(data, this.actionPriorityDefinition);
	};

	adm_process_flow_activity.prototype.__getAndValidateCampaign = function (data) {
		this.__addValue(data, this.skipFlowDefinition);
		this.__addValue(data, this.forbiddenAgentsDefinition);
		this.__addValue(data, this.preferredAgentsDefinition);
		this.__addValue(data, this.attentionLevelDefinition);
		this.__addValue(data, this.accountDefinition);
		this.__addValue(data, this.campaignDefinition);
	};

	adm_process_flow_activity.prototype.__getAndValidateCampaignAgent = function (data) {
		this.__addValue(data, this.skipFlowDefinition);
		this.__addValue(data, this.agentDefinition);
		this.__addValue(data, this.accountDefinition);
		this.__addValue(data, this.campaignDefinition);
		this.__addValue(data, this.attentionLevelDefinition);
	};

	adm_process_flow_activity.prototype.__getAndValidateNumber = function (data) {
		this.__addValue(data, this.countryCodeDefinition);
		this.__addValue(data, this.areaCodeDefinition);
		this.__addValue(data, this.numberDefinition);
	};

	/*
	 * CALL
	 */

	function adm_process_flow_activity_call(editor, node) {
		adm_process_flow_activity_call.uber.constructor.call(this, editor, node);
	}

	inConcert.Utils.inherit(adm_process_flow_activity_call, adm_process_flow_activity);
	adm_process_flow_activity_call.prototype.Override("CALL", adm_process_flow_activity);

	adm_process_flow_activity_call.prototype.__load = function (callback) {
		let enqueueIndex = this.actionOnTimeoutDefinition.options.findIndex((action) => action.id === "Enqueue");
		if (this.actionOnTimeoutDefinition.options[enqueueIndex]) {
			this.actionOnTimeoutDefinition.options.splice(enqueueIndex, 1);
		}
		callback();
	};

	/*
	 * CHAT
	 */

	function adm_process_flow_activity_chat(editor, node) {
		adm_process_flow_activity_chat.uber.constructor.call(this, editor, node);
	}

	inConcert.Utils.inherit(adm_process_flow_activity_chat, adm_process_flow_activity);
	adm_process_flow_activity_chat.prototype.Override("CHAT", adm_process_flow_activity);

	adm_process_flow_activity_chat.prototype.__load = function (callback) {
		let enqueueIndex = this.actionOnTimeoutDefinition.options.findIndex((action) => action.id === "Enqueue");
		if (this.actionOnTimeoutDefinition.options[enqueueIndex]) {
			this.actionOnTimeoutDefinition.options.splice(enqueueIndex, 1);
		}
		callback();
	};

	// adm_process_flow_activity.prototype.htmlBody = function (data) { 
	// 	return Mustache.render(inConcert.templates.process_flows.transfer_activity_template, {
	// 		image: `custom_icon_${id + "_" + (data.destination || "contact_center")}`,
	// 		description: ((data.description && data.description !== "") ? data.description : "<%=_T('Transfer')%>")
	// 	});
	// };

	adm_process_flow_activity.prototype.htmlBody = function (data) {

		var image = "";

		if (data) {
			var type = data.message && data.message.msgtype || "text";
			if (type === "attachment") {
				type = data.message.fileType || "attachment";
			}
			image = id + "_" + type;
		}

		return Mustache.render(inConcert.templates.process_flows.activity_template, {
			image: `custom_icon_${id + "_" + (data.destination || "contact_center")}`,
			description: ((data.description && data.description !== "") ? data.description : "<%=_T('Transfer')%>")
		});
	};

	adm_process_flow_activity.prototype.ports = function (data, node) { 


		let ports = [];

		// siempre hay que agregar este por defecto
		ports.push({
			direction: "in", // in/out
			type: "option", // error/success/option
			option_label: "Input" // error/success/option
		});

		return ports;
	};
	
}());
