/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 9 */

/* global $, inConcert, Mustache */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

  var format = inConcert.Utils.format;
	var templates = inConcert.templates.process_flows;

	var id = "variable";

	//values

	function variable_value(editor, $list) {
		this.editor = editor;
		this.$list = $list;
	}

	variable_value.prototype.id = null;
	variable_value.prototype.name = null;

	variable_value.prototype.Make = function () {
		throw "not implemented";
	};

	variable_value.prototype.Get = function () {
		throw "not implemented";
	};

	variable_value.prototype.Set = function () {
		throw "not implemented";
	};

	var variable_values = [];

	function variable_expression(editor, $list) {
		variable_expression.uber.constructor.call(this, editor, $list);
	}

	inConcert.Utils.inherit(variable_expression, variable_value);

	variable_expression.prototype.id = "expression";
	variable_expression.prototype.name = "<%=_T('Expression')%>";
	variable_values.push(variable_expression);

	variable_expression.prototype.Make = function ($container) {
		var self = this;
		$container.html(templates.variable_expression);
		$container.find("[expression_value]")
			.focus(function () {
				$(this).addClass("valueFocus");
				self.$list.scrollTo($(this));
			})
			.focusout(function () {
				$(this).removeClass("valueFocus");
			});
	};

	variable_expression.prototype.GetAndValidate = function ($container, data) {
		data.value = $container.find("[expression_value]").val();
		if (!data.value) {
			$container.find("[expression_value_validation]").show();
			return "<%=_T('Must enter a value.')%>";
		}
		// $container.find("[variable]").each(function () {
		// 	error = self.__getAndValidateEntry($(this), data, entries_d);
		// 	if (error) {
		// 		$(this).find("[entry_validation]").show();
		// 		return false;
		// 	}
		// });
	};

	variable_expression.prototype.Set = function ($container, data) {
		$container.find("[expression_value]").val(data.value || "");
	};

	function variable_object(editor, $list) {
		variable_object.uber.constructor.call(this, editor, $list);
	}

	inConcert.Utils.inherit(variable_object, variable_value);

	variable_object.prototype.id = "object";
	variable_object.prototype.name = "<%=_T('Object')%>";
	variable_values.push(variable_object);

	variable_object.prototype.__makeEntry = function (data) {
		data = data || {};
		var self = this;
		var $entry = $(templates.variable_object_entry);
		$entry.find("[entry_name]").val(data.name || "");

		$entry.find("[entry_value]")
			.focus(function () {
				$(this).addClass("valueFocus");
				self.$list.scrollTo($(this));
			})
			.focusout(function () {
				$(this).removeClass("valueFocus");
			})
			.val(data.value || "");

		$entry.find("[entry_add]").click(function () {
			self.editor.__disableNextKeyboardEvent();
			var $row = $(this).closest("li");
			var $newRow = self.__makeEntry();
			$row.after($newRow);
			self.$list.scrollTo($newRow);
			$newRow.find("[entry_name]").focus();
		});
		$entry.find("[entry_remove]").click(function () {
			self.editor.__disableNextKeyboardEvent();
			var $row = $(this).closest("li");
			var $siblings = $row.siblings();
			if ($siblings.length) {
				if ($row.prev().length) {
					$row.prev().find("[entry_name]").focus();
				}
				else {
					$row.next().find("[entry_name]").focus();
				}
			}
			else {
				var $newRow = self.__makeEntry();
				$row.after($newRow);
				$newRow.find("[entry_name]").focus();
			}
			$row.remove();
		});
		return $entry;
	};

	variable_object.prototype.Make = function ($container) {
		$container
			.html(templates.variable_object)
			.find("[object]")
			.sortable({
				cursor: "move"
			})
			.append(this.__makeEntry());
	};

	variable_object.prototype.__getAndValidateEntry = function ($entry, data, entries_d) {
		var entry = {
			name: $entry.find("[entry_name]").val(),
			value: $entry.find("[entry_value]").val()
		};
		if (!entry.name) {
			return ["<%=_T('Must enter a name.')%>", "name"];
		}
		if (entries_d[entry.name]) {
			return ["<%=_T('Entry name can not be repeated.')%>", "name"];
		}
		if (!entry.value) {
			return ["<%=_T('Must enter a value.')%>", "value"];
		}
		data.entries.push(entry);
		entries_d[entry.name] = true;
	};

	variable_object.prototype.GetAndValidate = function ($container, data) {
		var self = this;
		data.entries = [];
		var entries_d = {};
		var error = null;
		var errorType = null;
		$container.find("[object_entry]").each(function () {
			error = self.__getAndValidateEntry($(this), data, entries_d);
			if (error) {
				errorType = error[1];
				error = error[0];
				if (errorType === "value") {
					$(this).find("[entry_validation]").show();
					$container.find("[variable_name]").hide();
					return false;
				} 
				else if (errorType === "name") {
					$(this).find("[entry_name_validation]").show();
					$container.find("[variable_name]").hide();
					return false;
				}
			}
		});
		return error;
	};

	variable_object.prototype.Set = function ($container, data) {
		var self = this;
		var entries = data.entries || [];
		if (!entries.length) {
			return;
		}
		var $entries = [];
		$.each(entries, function (_, entry) {
			$entries.push(self.__makeEntry(entry));
		});
		$container.find("[object]")
			.html("")
			.append($entries);
	};

	function variable_array(editor, $list) {
		variable_array.uber.constructor.call(this, editor, $list);
	}

	inConcert.Utils.inherit(variable_array, variable_value);

	variable_array.prototype.id = "array";
	variable_array.prototype.name = "<%=_T('Array')%>";
	variable_values.push(variable_array);

	variable_array.prototype.__makeEntry = function (data) {
		data = data || {};
		var self = this;
		var $entry = $(templates.variable_array_entry);

		$entry.find("[entry_value]")
			.focus(function () {
				$(this).addClass("valueFocus");
				self.$list.scrollTo($(this));
			})
			.focusout(function () {
				$(this).removeClass("valueFocus");
			})
			.val(data.value || "");

		$entry.find("[entry_add]").click(function () {
			self.editor.__disableNextKeyboardEvent();
			var $row = $(this).closest("li");
			var $newRow = self.__makeEntry();
			$row.after($newRow);
			self.$list.scrollTo($newRow);
			$newRow.find("[entry_value]").focus();
		});
		$entry.find("[entry_remove]").click(function () {
			self.editor.__disableNextKeyboardEvent();
			var $row = $(this).closest("li");
			var $siblings = $row.siblings();
			if ($siblings.length) {
				if ($row.prev().length) {
					$row.prev().find("[entry_value]").focus();
				}
				else {
					$row.next().find("[entry_value]").focus();
				}
			}
			else {
				var $newRow = self.__makeEntry();
				$row.after($newRow);
				$newRow.find("[entry_value]").focus();
			}
			$row.remove();
		});

		return $entry;
	};

	variable_array.prototype.Make = function ($container) {
		$container
			.html(templates.variable_array)
			.find("[array]")
			.sortable({
				cursor: "move"
			})
			.append(this.__makeEntry());
	};

	variable_array.prototype.__getAndValidateEntry = function ($entry, data) {
		var entry = {
			value: $entry.find("[entry_value]").val()
		};
		if (!entry.value) {
			return "<%=_T('Must enter a value.')%>";
		}
		data.entries.push(entry);
	};

	variable_array.prototype.GetAndValidate = function ($container, data) {
		var self = this;
		data.entries = [];
		var error = null;
		$container.find("[array_entry]").each(function () {
			error = self.__getAndValidateEntry($(this), data);
			if (error) {
				$(this).find("[entry_validation]").show();
				return false;
			}
		});
		return error;
	};

	variable_array.prototype.Set = function ($container, data) {
		var self = this;
		var entries = data.entries || [];
		if (!entries.length) {
			return;
		}
		var $entries = [];
		$.each(entries, function (_, entry) {
			$entries.push(self.__makeEntry(entry));
		});
		$container.find("[array]")
			.html("")
			.append($entries);
	};

	//activity

	function adm_process_flow_activity(editor, node) {
		adm_process_flow_activity.uber.constructor.call(this, editor, node);

		this.focusFirst = true;
	}

	inConcert.Utils.inherit(adm_process_flow_activity, inConcert.adm_process_flow_activities.base);

	if (typeof inConcert.adm_process_flow_activities[id] !== "function") {
		inConcert.adm_process_flow_activities[id] = adm_process_flow_activity;
	}

	adm_process_flow_activity.prototype.id = id;
	adm_process_flow_activity.prototype.name = "<%=_T('Declare variables')%>";
	adm_process_flow_activity.prototype.description = "<%=_T('Variables declaration.')%>";
	adm_process_flow_activity.prototype.group = "variables";
	adm_process_flow_activity.prototype.image = "ico-variables.svg";

	adm_process_flow_activity.prototype.__getClassName = function() { 
		return "popOver640w flowDesignerGlobalModal botActivity_variable popOverFixedHeight";
	};

	adm_process_flow_activity.prototype.__afterAppendBody = function() {
		const self = this;
		self.__addAutocompleteToInput();
		self.__disableFirstRemoveButton();
	};

	adm_process_flow_activity.prototype.__addAutocompleteToInput = function () {
		const self = this;
		const main = self.editor.main;
		main.LoadVariablesSet();

		function split( val, splitValue ) {
			return val.split( splitValue );
		}
		  
		function extractLast( term, splitValue ) {
			return split( term, splitValue ).pop();
		}
	
		self.$get("[flow_designer_textarea_variable]").each(function() {
			let $textArea = $(this);

			if ($textArea.autocomplete( "instance" )) {
				return;
			}
			$textArea.on( "keydown", function( event ) {
				if ( event.keyCode === $.ui.keyCode.TAB &&
					$( this ).autocomplete( "instance" ).menu.active ) {
					event.preventDefault();
				}
				})
				.autocomplete({
					minLength: 1,
					// search: function( event, ui ) {},
					source: function( request, response ) {
						const typedValue = extractLast(request.term, " ");
						if (!typedValue.startsWith("{")) {
							return [];
						}
						let searchValue = extractLast(request.term, "{");
						var matcher = new RegExp( "^" + $.ui.autocomplete.escapeRegex( searchValue ), "i" );
						let resultList = $.grep( Array.from(main.variables), function( item ){
							return matcher.test( item );
						});
						response( resultList );
						},
					focus: function() {
						// prevent value inserted on focus
						return false;
					},
					select: function( event, ui ) {
						var terms = split( this.value, " " );
						// remove the current input
						terms.pop();
						// add the selected item
						terms.push( "{{" + ui.item.value + "}}" );
						// add placeholder to get the comma-and-space at the end
						terms.push( "" );
						this.value = terms.join( " " );
						return false;
					}
				});
		});

		$("[id*='ui-id']").css("z-index", 119);
		// return variableList;
	};

	adm_process_flow_activity.prototype.__moveCursorToEnd = function($myDiv) {
		const divContentEditable = $myDiv[0];
		const selection = window.getSelection();
		const range = document.createRange();
		selection.removeAllRanges();
		range.selectNodeContents(divContentEditable);
		range.collapse(false);
		selection.addRange(range);
		divContentEditable.focus();	
	};

	adm_process_flow_activity.prototype.__disableFirstRemoveButton = function () {
		var self = this;
		if(self.$get("[variable]").length === 1) {
			self.$get("[remove]").prop("disabled", "disabled");

		}
		else {
			self.$get("[remove]").prop("disabled", false);
		}
	};

	adm_process_flow_activity.prototype.__makeVariable = function (variable) {
		variable = variable || {};
		var self = this;
		var $variable = $(Mustache.render(templates.variable, { options: self.variableValuesOptions }));

		$variable.find("[name]").val(variable.name || "");

		var type = variable.type || "expression";
		$variable.find("[value_type]")
			.val(variable.type || "expression")
			.change(function () {
				self.variableValues[$(this).val()].Make($(this).parent().parent().parent().find("[value_container]"));
				self.__addAutocompleteToInput();
			})
			.change();

		self.variableValues[type].Set($variable.find("[value_container]"), variable);

		$variable.find("[remove]").click(function () {
			self.__disableNextKeyboardEvent();
			var $row = $(this).closest("li");
			var $siblings = $row.siblings();
			if ($siblings.length) {
				if ($row.prev().length) {
					$row.prev().find("[name]").focus();
				}
				else {
					$row.next().find("[name]").focus();
				}
			}
			else {
				var $newRow = self.__makeVariable();
				$row.after($newRow);
				$newRow.find("[name]").focus();
			}
			$row.remove();
			self.__disableFirstRemoveButton();
		});
		return $variable;
	};

	adm_process_flow_activity.prototype.__makeVariables = function () {
		var self = this;
		var variables = self.data.variables || [];
		var $variables = $(templates.variables_container);
		var $variablesList = $variables.find("[variables]");
		var $list = [];

		self.variableValues = {};
		self.variableValuesOptions = "";

		$variables.find("[add]").click(function () {
			self.__disableNextKeyboardEvent();
			var $row = $(this).parent().parent().parent().siblings().find("[variables]");
			var $newRow = self.__makeVariable();
			$row.append($newRow);
			$row.parent().scrollTo($newRow);
			$newRow.find("[name]").focus();
			self.__addAutocompleteToInput();
			self.__disableFirstRemoveButton();
		});

		$.each(variable_values, function (_, value) {
			self.variableValues[value.prototype.id] = new value(self, $variablesList);
			self.variableValuesOptions += format("<option value='{0}'>{1}</option>",
				value.prototype.id, value.prototype.name);
		});

		$.each(variables, function (_, variable) {
			$list.push(self.__makeVariable(variable));
		});
		if (!$list.length) {
			$list.push(self.__makeVariable());
		}
		$variablesList
			.sortable({
				cursor: "move"
			})
			.append($list);
		return $variables;
	};

	adm_process_flow_activity.prototype.__makeContent = function () {
		const self = this; 
		let $content = adm_process_flow_activity.uber.__makeContent.call(this);
		$content.unshift(this.__makeVariables());
		window.setInterval(function() {
			self.__disableFirstRemoveButton();
		}, 100);
		return $content;
	};

	adm_process_flow_activity.prototype.__getAndValidateVariable = function ($variable, data) {
		var variable = {
			name: $variable.find("[name]").val() || undefined,
			type: $variable.find("[value_type]").val() || undefined
		};
		//validation
		var error = null;
		if (!variable.name) {
			error = "<%=_T('Must enter a name.')%>";
			$variable.find("[variable_name]").show();
		}
		else if (variable.name.indexOf("__") === 0) {
			error = "<%=_T('Name can not start with double underscore.')%>";
			$variable.find("[variable_name]").show();
		}
		if (!variable.type) {
			error = "<%=_T('Must enter a type.')%>";
			$variable.find("[variable_name]").show();
		}
		error = error || this.variableValues[variable.type].GetAndValidate($variable.find("[value_container]"), variable);
		if (error) {
			data.error = {
				message: error,
				$control: $variable
			};
		}
		data.variables.push(variable);
	};

	adm_process_flow_activity.prototype.__clearErrors = function () {
		adm_process_flow_activity.uber.__clearErrors.call(this);
		this.$get("[entry_validation]").hide();
		this.$get("[entry_name_validation]").hide();
		this.$get("[variable_name]").hide();
	};

	adm_process_flow_activity.prototype.__getAndValidateVariables = function (data) {
		var self = this;
		data.variables = [];
		self.$get("[variable]").each(function () {
			self.__getAndValidateVariable($(this), data);
			if (data.error) {
				return false;
			}
		});
		if (!data.error && !data.variables.length) {
			data.error = {
				message: "<%=_T('Must define almost one variable.')%>"
			};
		}
	};

	adm_process_flow_activity.prototype.__getAndValidate = function () {
		var data = adm_process_flow_activity.uber.__getAndValidate.call(this);
		if (!data.error) {
			this.__getAndValidateVariables(data);
		}
		return data;
	};
	// agregado por Erick Gonzalez
	adm_process_flow_activity.prototype.htmlBody = function (data) {

		var image = "";

		if (data) {
			var type = data.message && data.message.msgtype || "text";
			if (type === "attachment") {
				type = data.message.fileType || "attachment";
			}
			image = id + "_" + type;
		}

		return Mustache.render(inConcert.templates.process_flows.activity_template, {
			image: `custom_icon_${image}`,
			description: ((data.description && data.description !== "") ? data.description : "<%=_T('Declare variables')%>")
		});
	};

	adm_process_flow_activity.prototype.ports = function (data, node) {


		var ports = [];

		// siempre hay que agregar este por defecto
		ports.push({
			direction: "in", // in/out
			type: "", // error/success/option
			option_label: "Input" // error/success/option
		});

		// agrego un puerto por defecto si es necesario
		if (ports.length === 1) {
			var existe = node ? node.children.find(c => c.id.includes("_default")) : undefined;
			let portId = (existe ? existe.id : inConcert.Utils.generateGuid() + "_default");
			ports.push({
				direction: "out", // in/out
				type: "option", // error/success/option
				option_label: "<%=_T('Default output')%>", // error/success/option
				channels: "n", // 1 o n
				id: portId, // le asigno un id por defecto
				preset_value: undefined,
				intern_value: undefined
			});
		}

		return ports;
	};
}());
