/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
    strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 9 */

/* global $, inConcert, Mustache */

if (!this.inConcert) {
    this.inConcert = {};
}

(function () {
    var templates = inConcert.templates.process_flows;

    var id = "variable_config";

    //activity
    let guid;
    let outputList = [];
    let $listConditions = [];
    let lastGuid;
    
    
    function adm_process_flow_activity(editor, node) {
        adm_process_flow_activity.uber.constructor.call(this, editor, node);
        this.editor = editor;
        this.frm = editor.main.frm;
        this.node = node;
        var data = editor.GetNodeData(node);
        this.isNew = !data;
        this.data = data || {outputs: []};
        $listConditions = [];
        outputList = [];
        lastGuid = "";


        this.focusFirst = true;

        this.descriptionDefinition = {
            id : "description",
            name : "<%=_T('Description')%>",
            description : "<%=_T('Activity description.')%>",
            placeholder : "<%=_T('Enter description')%>..."
        };
    }

    inConcert.Utils.inherit(adm_process_flow_activity, inConcert.adm_process_flow_activities.base);

    if (typeof inConcert.adm_process_flow_activities[id] !== "function") {
        inConcert.adm_process_flow_activities[id] = adm_process_flow_activity;
    }

    adm_process_flow_activity.prototype.id = id;
    adm_process_flow_activity.prototype.name = "<%=_T('Conditions')%>";
    adm_process_flow_activity.prototype.description = "<%=_T('Conditions')%>";
    adm_process_flow_activity.prototype.group = "variables";
    adm_process_flow_activity.prototype.image = "ico-variables.svg";

    adm_process_flow_activity.prototype.__afterAppendBody = function() {
        const self = this;
        self.__addAutocompleteToInput();
    };


    adm_process_flow_activity.prototype.__makeOutputs = function() {
        const self = this;
        guid = (inConcert.Utils.generateGuid()).toLowerCase();
        let outputs = self.data.outputs || [];
        let $outputs = $(Mustache.render(templates.variable_config_markup, {condition_id: guid}));
        let $listOutputs = [];

        $.each(outputs, function(_, output) {
            if (output && output.title) {
                $listOutputs.push($outputs.find("[div_variables]").append(self.__makeOutput(output, undefined, $outputs)));
            }
        });
        
        if (!$listOutputs.length) {
            const $initialOutput = self.__makeOutput(undefined, guid, $outputs);
            const $initialCondition = self.__makeCondition($initialOutput, guid);
            $initialCondition.find("[logic]").prop("disabled", "disabled");
            $initialCondition.find("[remove]").prop("disabled", "disabled");
            $listOutputs.push($outputs.find("[div_variables]").append($initialOutput));
            $listConditions.push($initialCondition);
            
            $outputs.find(`#condition_list_${guid}`).find("[condition_list]")
                .sortable({
                    cursor : "move"
                })
            .append($initialCondition);

            self.__addAutocompleteToInput();
        } 

        $outputs.find("[add_new_output]").click(function () {
            const $newOutput = self.__makeOutput(undefined, undefined, $outputs);
            const outputId = $newOutput.attr("id"); 
            const $newCondition = self.__makeCondition($newOutput, outputId);
            $listConditions.push($newCondition);
            $newCondition.find("[logic]").prop("disabled", "disabled");
            $newCondition.find("[remove]").prop("disabled", "disabled");
            $outputs.find(`#condition_list_${outputId}`).find("[condition_list]")
                .sortable({
                    cursor : "move"
                })
            .append($newCondition);
        
            self.__addAutocompleteToInput();

            setTimeout(function() { 
                self.__disableFirstLogicOperator();
                self.__disableLastDeleteOutputButton();
            }, 100);
        });

        return $outputs;
    };

    adm_process_flow_activity.prototype.__makeConditions = function($outputs) {
        const self = this;
        const outputs = self.data.outputs || [];
        let outputId;
        let $conditionList;
        let lastOutputId;

        $.each(outputs, function(_, output) {
            outputId = output.port_id;
            output.conditions = self.__getValidateInitialParenths(output.conditions);
            $.each(output.conditions, function(index, condition) {
                $conditionList = $outputs.find(`#condition_list_${outputId}`).find("[condition_list]");
                let $condition = self.__makeCondition(condition, outputId);

                // DESHABILITO EL PRIMER SELECT DE OPERADOR LOGICO PARA LA PRIMERA CONDICION DE CADA OUTPUT
                $condition.find("[logic]").prop("disabled", false);
                if (index === 0 || lastOutputId !== outputId) {
                    $condition.find("[logic]").prop("disabled", "disabled");
                    $condition.find("[logic]").val("");
                    $condition.find("[remove]").prop("disabled", "disabled");
                }
                lastOutputId = outputId;
                // FIN

                // $conditionList.append($condition);
                $listConditions.push($conditionList);	
                $outputs.find(`#condition_list_${outputId}`).find("[condition_list]")
                    .sortable({
                        cursor : "move"
                    })
                .append($condition);

                self.__addAutocompleteToInput();
            });
        });

        return $outputs;
    };

    adm_process_flow_activity.prototype.__getValidateInitialParenths = function(conditions) {

		let parenths_bk = [];
        let parenths = [];

        let leftEqual_last = false;
        let rightEqual_last = false;

        try {
            conditions.forEach(c => {
                c.begin_parenths.trim().split("").forEach(p => {
                    parenths_bk.push(p);
                    parenths.push(p);
                });
                c.end_parenths.trim().split("").forEach(p => {
                    parenths_bk.push(p);
                    parenths.push(p);
                });
            });

            leftEqual_last = false;
            rightEqual_last = false;

            let index = -1;
            parenths.forEach(p1 => {
                index++;
                if (p1 === ")") {
					let i = index;
                    for(i=index;i>0;i--) {
                        if (parenths[i] === "(") {

                            let leftEqual = true;
                            for (let ix = (parenths_bk.length / 2) - 1; ix >= 0; ix--) {
                                if (parenths[ix] !== "(" && parenths[ix] !== "") {
                                    leftEqual = false;
                                    break;
                                }
                            }

                            // Validar hacia la derecha
                            let rightEqual = true;
                            for (let iy = (parenths_bk.length / 2) + 1; iy < parenths.length; iy++) {
                                if (parenths[iy] !== ")" && parenths[iy] !== "") {
                                    rightEqual = false;
                                    break;
                                }
                            }

                            leftEqual_last = leftEqual;
                            rightEqual_last = rightEqual;

                            if (!(leftEqual && rightEqual)) {
                                parenths[i] = "";
                                parenths[index] = "";

                                break;
                            } else {
                                break;
                            }
                            
                        }
                    }
                }
            });

            const parenths_filtered = parenths.filter((e) => e !== "");

            if (leftEqual_last && rightEqual_last && parenths_filtered[0] !== parenths_filtered[parenths.length - 1] && conditions.length >= 1) {
                conditions[0].begin_parenths = conditions[0].begin_parenths.substring(1);
                conditions[conditions.length - 1].end_parenths = conditions[conditions.length - 1].end_parenths.substring(1);
            }
        } catch (e) {
            console.error(e);
        }

		if (conditions.length === 1) {
			const c_conditions = conditions.slice(); // Clonamos el arreglo original
			const firstElement = c_conditions[0];
			const hasOpeningParenthesis = (firstElement.begin_parenths || "").length > 0;
			const hasClosingParenthesis = (firstElement.end_parenths || "").length > 0;
		  
			if (hasOpeningParenthesis && hasClosingParenthesis) {
			  const firstElementWithoutOpeningParenthesis = {
				...firstElement,
				begin_parenths: (firstElement.begin_parenths || "").slice(1),
				end_parenths: (firstElement.end_parenths || "").slice(1)
			  };
		  
			  const conditionsWithoutParentheses = [
				firstElementWithoutOpeningParenthesis
			  ];

			  let openParentheses = 0;
			  let closedParentheses = 0;
		  
			  for (let i = 0; i < conditionsWithoutParentheses.length; i++) {
				const { begin_parenths, end_parenths } = conditionsWithoutParentheses[i];
		  
				if (begin_parenths.length > 0) {
                    openParentheses += begin_parenths.length;
				}
		  
				if (end_parenths.length > 0) {
                    closedParentheses += end_parenths.length;
				}
		  
				if (closedParentheses > openParentheses) {               
                    return conditions;
				}
			  }
		  
			  if (openParentheses === closedParentheses) {
				return conditionsWithoutParentheses;
			  }
			}
		} else if (conditions.length > 1) {
			const c_conditions = conditions.slice(); // Clonamos el arreglo original
			const firstElement = c_conditions[0];
			const lastElement = c_conditions[c_conditions.length - 1];
			const hasOpeningParenthesis = (firstElement.begin_parenths || "").length > 0;
			const hasClosingParenthesis = (lastElement.end_parenths || "").length > 0;
		  
			if (hasOpeningParenthesis && hasClosingParenthesis) {
			  const firstElementWithoutOpeningParenthesis = {
				...firstElement,
				begin_parenths: (firstElement.begin_parenths || "").slice(1),
			  };
		  
			  const ultimoElementoSinParentesisFin = {
				...lastElement,
				end_parenths: (lastElement.end_parenths || "").slice(0, -1),
			  };

			  const conditionsWithoutParentheses = [
				firstElementWithoutOpeningParenthesis,
				...c_conditions.slice(1, -1),
				ultimoElementoSinParentesisFin,
			  ];

			  let openParentheses = 0;
			  let closedParentheses = 0;
		  
			  for (let i = 0; i < conditionsWithoutParentheses.length; i++) {
				const { begin_parenths, end_parenths } = conditionsWithoutParentheses[i];
		  
				if (begin_parenths.length > 0) {
                    openParentheses += begin_parenths.length;
				}
		  
				if (end_parenths.length > 0) {
                    closedParentheses += end_parenths.length;
				}
		  
				if (closedParentheses > openParentheses) {
                    return conditions;
				}
			  }
		  
			  if (openParentheses === closedParentheses) {
				return conditionsWithoutParentheses;
			  }
			}
		}

        return conditions;

	};

    adm_process_flow_activity.prototype.__makeOutput = function (output, initialGuid, $outputs) {

        const self = this;
        
        let $variable_output;
        let outputListLength;
        (initialGuid) ? guid = initialGuid : guid = (inConcert.Utils.generateGuid()).toLowerCase();
        (output) ? outputList.push(output.port_id) : outputList.push(guid);
        (outputList.length !== 0) ? outputListLength = outputList.length : outputListLength = 2;

        if (output) {
            $variable_output = $(Mustache.render(templates.variable_output, {condition_id: output.port_id}));
            $outputs.find("[div_variables]").append($variable_output);
            $variable_output.find("[output_name]").val(output.title || "");
        } 
        else {	
            $variable_output = $(Mustache.render(templates.variable_output, {condition_id: guid}));
            $outputs.find("[div_variables]").append($variable_output);
        }

        $variable_output.find("[output_name]").on("keyup change", function () {
            // UPDATE OUTPUT NAME
            const id = $(this).attr("id");
            const idGuid = id.substring(30, id.length);
            $(`[condition_link_name_${idGuid}]`).html($(this).val());
            Object.keys(self.data.outputs).forEach((key) => {
                if (self.data.outputs[key].port_id === idGuid) {
                    self.data.outputs[key].title = $(this).val();
                }
            });
        });

        $variable_output.find("[output_remove]").click(function () {
            // DELETE
            var $row = $(this).parent().parent().parent();
            var outputId = $(this).attr("id");
            const remove = outputList.find(id => id === outputId);
            outputList = $.grep(outputList, function(value) {
                return value !== remove;
            });
            $(`#${remove}`).remove();				
            $row.remove();
            $("[condition_link_order]").each(function(index) { 
                $(this).html(`${index + 1}. `); 
            });

            setTimeout(function() { 
                self.__disableFirstLogicOperator();
                self.__disableLastDeleteOutputButton();
            }, 100);
        });		

        if (output) {
            self.__makeConditionLink(outputListLength, output.port_id, $outputs, output.title);
        }
        else {
            self.__makeConditionLink(outputListLength, guid, $outputs);

            self.data.outputs = self.data.outputs || [];
			self.data.outputs.push( { port_id : guid, title: $($(`[condition_link_name_${guid}]`)), conditions: [] } );
        }
            
        return $variable_output;
    };

    adm_process_flow_activity.prototype.__makeCondition = function(condition, guid, variable1Guid, variable2Guid) {
        var self = this;
        var $condition = $(Mustache.render(templates.variable_condition, {conditionId: guid, variable1Guid: variable1Guid, variable2Guid: variable2Guid}));

        if (condition) {
            $condition.find("[logic]").val(condition.logic || "");
            $condition.find("[begin_parenths]").val(condition.begin_parenths || "");
            
            let indexLeftParenth = (condition.field?condition.field:"").indexOf("(");
            let indexRightParenth = (condition.field?condition.field:"").indexOf(")");
            let variableName = (condition.field?condition.field:"").slice(indexLeftParenth + 1, indexRightParenth);
            //variableName = variableName.replace(".data", "");

            let indexLeftParenthValue = (condition.value?condition.value:"").indexOf("(");
            let indexRightParenthValue = (condition.value?condition.value:"").indexOf(")");
            let variableValue = (condition.value?condition.value:"").slice(indexLeftParenthValue + 1, indexRightParenthValue);

            $condition.find("[field_name]").val(variableName || "");
            $condition.find("[comparator]").val(condition.comparator || "");
            $condition.find("[field_value]").val(variableValue || "");
            $condition.find("[end_parenths]").val(condition.end_parenths || "");
        }
        $condition.find("[add]").click(function() {
            self.__disableNextKeyboardEvent();
            var $row = $(this).parent().parent();
            var $newRow = self.__makeCondition(undefined, guid);
            $row.after($newRow);
            self.__addAutocompleteToInput();
            $newRow.find("[logic]").focus();
        });
        $condition.find("[remove]").click(function() {
            self.__disableNextKeyboardEvent();
            var $row = $(this).parent().parent();
            var $siblings = $row.siblings();
            if ($siblings.length) {
                if ($row.prev().length) {
                    $row.prev().find("[logic]").focus();
                }
                else {
                    $row.next().find("[logic]").focus();
                }
                $row.remove();
            }
        });

        return $condition;
    };

    adm_process_flow_activity.prototype.__makeConditionLink = function(outputListLength, guid, $outputs, title) {
        var self = this;
        let linkTitle;
        (title) ? linkTitle = title : linkTitle = ""; 
        let outputLink =`<li condition_link id='${guid}' class='list-inline-item flowDesignerContentChip'>
                            <span condition_link_order>${outputListLength}. </span> 
                            <span class="flowDesignerContentChipName" condition_link_name_${guid}>${linkTitle}</span>
                        </li>`;
        $(outputLink).insertBefore($outputs.find("#default_output"));
        
        $outputs.find(`#${guid}`).click(function () {
            if (this.tagName === "DIV") {
                return;
            }
            $("[condition_link]").each(function() { 
                $(this).removeClass("selected");
            });
            $(this).addClass("selected");

            try {
                //self.$get(".tab-pane.fade.active.show").scrollTo(self.$get(`[activity_output_${guid}]`), {
                self.$get(".internalPopUpMiddle").scrollTo(self.$get(`[output_${guid}]`), {
                    offset: -150,
                    duration: 750
                });
            }
            catch (e) {
            }		
        });
    };

    adm_process_flow_activity.prototype.__getAndValidateOutputTitles = function(data) {

		var self = this;
		let outputNameList = [];
		let validated = true;

		self.$get("[output]").each(function() {
			let outputName = $(this).find("[output_name]").val();
			if (outputName && outputName !== "") {
				outputNameList.push(outputName);
			}
		});

		const orderedOutputNames = outputNameList.sort(); 

		self.$get("[output]").each(function(_) {
			
			self.__getAndValidateOutput($(this), data);

			var namesDuplicated = [];
			for (var i = 0; i < orderedOutputNames.length - 1; i++) {
				if (orderedOutputNames[i + 1] === orderedOutputNames[i]) {
					namesDuplicated.push(orderedOutputNames[i]);
				}
			}
			if (namesDuplicated.length > 0) {
				data.error = {
					message : "<%=_T('Output names must be different from each other')%>"
				};	
				validated = false;
				return false;
			}
		});
		
		return validated;
	};

    adm_process_flow_activity.prototype.__getAndValidate = function() {
        let data = adm_process_flow_activity.uber.__getAndValidate.call(this);
        data = this.data;
        data.error = null;
        data.outputs = [];
        data.type = "variable_config";
        this.__addValue(data, this.descriptionDefinition);
        if (this.__getAndValidateOutputTitles(data)) {
            this.__getAndValidateConditions(data);
        }
        return data;
    };

    /*adm_process_flow_activity.prototype.__getAndValidateOutputs = function(data) {
        var self = this;
        let outputNameList = [];
        let validated = true;

        self.$get("[output]").each(function() {
            let outputName = $(this).find("[output_name]").val();
            if (outputName || outputName !== "") {
                outputNameList.push(outputName);
            }
        });

        const orderedOutputNames = outputNameList.sort(); 

        self.$get("[output]").each(function() {
            
            if ($(this).find("[output_name]").val() === "") {
                data.error = {
                    message : "<%=_T('Error on output name.')%>"
                };	
                validated = false;
                return false;
            }

            self.__getAndValidateOutput($(this), data);

            var namesDuplicated = [];
            for (var i = 0; i < orderedOutputNames.length - 1; i++) {
                if (orderedOutputNames[i + 1] === orderedOutputNames[i]) {
                    namesDuplicated.push(orderedOutputNames[i]);
                }
            }
            if (namesDuplicated.length > 0) {
                data.error = {
                    message : "<%=_T('Output names must be different from each other')%>"
                };	
                validated = false;
                return false;
            }
        });
        
        return validated;
    };*/

    adm_process_flow_activity.prototype.__getAndValidateOutput = function($output, data) {
        const id = $output.attr("id");
        const name = $output.find("[output_name]").val();
        const output = {
            port_id : id || undefined, 
            title: name, 
            conditions: [] 
        };
        data.outputs.push(output);
        return true;
    };

    adm_process_flow_activity.prototype.__disableFirstLogicOperator = function () {
        var self = this;
        let lastOutputId;
        self.$get("[condition]").each(function(index) {
            let outputId = $(this).attr("id");
            $(this).find("[logic]").prop("disabled", false);
            $(this).find("[remove]").prop("disabled", false);
            if (index === 0 || lastOutputId !== outputId) {
                $(this).find("[logic]").prop("disabled", "disabled");
                $(this).find("[logic]").val("");
                $(this).find("[remove]").prop("disabled", "disabled");
            }
            lastOutputId = outputId;
        });		
    };

    adm_process_flow_activity.prototype.__disableLastDeleteOutputButton = function () {
        let deleteOutputs = $("#popup_content").find("[output_remove]");
        if(deleteOutputs.length < 2) {
            $(deleteOutputs[0]).prop("disabled", true);
        }
        else {
            deleteOutputs.each(function(index) {
                $(deleteOutputs[index]).prop("disabled", false);
            });	
        }
    };

    adm_process_flow_activity.prototype.__getAndValidateConditions = function(data) {

        var self = this;
        var parenths = 0;
		self.__disableFirstLogicOperator();
		self.$get("[condition]").each(function(index) {
            parenths += self.__getAndValidateCondition(index, $(this), data);
		});

        if (!data.error && parenths !== 0) {
            data.error = {
                message : "<%=_T('Error validating parenths.')%>"
            };
        }
        parenths = 0;
        if (data.error) {
            return false;
        }
    };

    adm_process_flow_activity.prototype.__getAndValidateCondition = function(index, $condition, data) {

        const guid = $condition.find("[condition_output]").attr("id");
        const condition = {
            title: $condition.parent().parent().parent().parent().find("[output_name]").val() || undefined,
            logic : $condition.find("[logic]").val() || undefined,
            begin_parenths : $condition.find("[begin_parenths]").val() || undefined,
            field : $condition.find("[field_name]").val() || undefined,
            comparator : $condition.find("[comparator]").val() || undefined,
            value : $condition.find("[field_value]").val() || undefined,
            end_parenths : $condition.find("[end_parenths]").val() || undefined
        };
        //empty
		if (!condition.logic && !condition.begin_parenths && !condition.field && !condition.comparator && 
			!condition.value && !condition.end_parenths && !condition.title) 
		{
			return 0;
		}        
        //validation
        var error = null;
        if (!condition.title) {
			error = "<%=_T('Error on output name.')%>";
		}
        if (!condition.value) {
            error = "<%=_T('Must enter a value.')%>";
        }
        if (!condition.comparator) {
            error = "<%=_T('Must enter a comparator.')%>";
        }
        if (!condition.field) {
            error = "<%=_T('Must enter a field.')%>";
        }
        if (index > 0 && (!condition.logic && guid === lastGuid)) {
            error = "<%=_T('Must enter a logical operator.')%>";
        }
        lastGuid = guid;
        if (error) {
            data.error = {
                message : error,
                $control : $condition
            };
        }
        
        data.outputs.find((output) => output.port_id === guid).conditions.push(condition);
        return (condition.begin_parenths || "").length - (condition.end_parenths || "").length;
    };

    adm_process_flow_activity.prototype.__addAutocompleteToInput = function () {
        const self = this;
        self.editor.main.LoadVariablesSet();

        self.$get("[flow_designer_variable_input]").each(function() { 
            self.frm.make_autocomplete(
                $(this),
                Array.from(self.editor.main.variables)
            );
        });

        $("[id*='ui-id']").css("z-index", 119);
    };


    adm_process_flow_activity.prototype.__makeContent = function () {
        const self = this;
        const $content = adm_process_flow_activity.uber.__makeContent.call(this);
        let $outputs = this.__makeOutputs();
        $outputs = this.__makeConditions($outputs);
        const outputs = this.data.outputs;
        $content.push($outputs);
        window.setInterval(function() {
            self.__disableFirstLogicOperator();
            self.__disableLastDeleteOutputButton();
        }, 100);
        
        $.each(outputs, function(_, output) {
            output.conditions = [];
        });

        return $content;
    };

    adm_process_flow_activity.prototype.__getClassName = function() {
        return "flowDesignerPopupSetVariables popOver940w flowDesignerGlobalModal popOverFixedHeight botActivity_variable_config";
    };
    
    adm_process_flow_activity.prototype.htmlBody = function (data) {

        var image = "";

        if (data) {
            var type = data.message && data.message.msgtype || "text";
            if (type === "attachment") {
                type = data.message.fileType || "attachment";
            }
            image = id + "_" + type;
        }

        return Mustache.render(inConcert.templates.process_flows.activity_template, {
            image: `custom_icon_${image}`,
            description: ((data.description && data.description !== "") ? data.description : "<%=_T('Declare variables')%>")
        });
    };

    adm_process_flow_activity.prototype.ports = function(data, node) { 
		

		let ports = [];
		let acumulative_outputs = { ports: [] };

		// siempre hay que agregar este por defecto
		ports.push({
			direction : "in", // in/out
			type : "", // error/success/option
			option_label : "Input" // error/success/option
		});

		// agrego puertos
		ports.push(...this.__generate_ports(data, node, acumulative_outputs)); // debe ir siempre primero

		// agrego un puerto por defecto
		//ports.push(...this.__generate_default_port(data, node, acumulative_outputs));

		// agrego una opción para los errores
		//ports.push(...this.__generate_error_ports(data, node, true));

		data.outputs = data.outputs || [];
		data.outputs.forEach(output => {
            output.conditions = this.__getValidateInitialParenths(output.conditions);
        });
		
		return ports;
	};

	adm_process_flow_activity.prototype.__generate_ports = function(data, node, acumulative_outputs) { 
		
		let ports = [];

		if (data.outputs && data.outputs.length > 0) {

			data.outputs.forEach(b => {

                if (b && b.title) {
                    let existe = node?node.children.find(c => c.id === b.port_id):undefined;
                    let portId = (existe?existe.id:b.port_id);

                    let conditions = [];
                    let contador = 0;
                    
                    b.conditions.forEach(c => {
                        contador++;
                        c.begin_parenths = (contador === 1?("(" + (c.begin_parenths || "")):(c.begin_parenths || ""));
                        c.logic = (contador === 1?"":c.logic.toLowerCase());

                        c.field = c.field.replace("tostring(", "decodeUnicodeEscape(");
						c.value = c.value.replace("tostring(", "decodeUnicodeEscape(");

                        c.field = ((c.field.includes("decodeUnicodeEscape"))?(c.field):("decodeUnicodeEscape(" + c.field + ")"));
                        c.value = ((c.value.includes("decodeUnicodeEscape"))?(c.value):("decodeUnicodeEscape(" + c.value + ")"));
                        c.end_parenths = (contador === b.conditions.length?(")" + (c.end_parenths || "")):(c.end_parenths || ""));
                    });

                    conditions.push(...b.conditions);

                    acumulative_outputs.ports.push(b.title?b.title:b.message); // esto hay que replantearlo

                    ports.push({
                        direction : "out", // in/out/none
                        type : "option", // error/success/option
                        option_label : b.title?b.title:b.message, // error/success/option
                        channels: "1", // 0, 1 o n
                        id: portId,
                        preset_value: {
                            description: "", //p.name
                            conditions: conditions,
                            port_id: portId
                        }
                    });
                }

			});
		}
		
		return ports;
	};
	
    adm_process_flow_activity.prototype.__cancel = function(callback) {
        if (this.isNew) {
            this.editor.CancelConnection(this.connector);
        }
        callback();
    };

    adm_process_flow_activity.prototype.__save = function(data, callback) {
        this.editor.AddOrUpdateActivity(this.node, data);
        callback(true);
    };

}());
