/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double */

/* global $, inConcert, Mustache */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	var format = inConcert.Utils.format;
	var templates = inConcert.templates.process_flow_classes;
	
	var actions = [
		{
			id : "duplicate",
			name : "<%=_T('Duplicate')%>",
			description : "<%=_T('Duplicate attribute.')%>",
			selection : true,
			singleSelection : true,
			permission : "save_flow_classes"
		},
		{
			id : "delete",
			name : "<%=_T('Delete')%>",
			description : "<%=_T('Delete attribute.')%>",
			permission : "save_flow_classes",
			confirmation : "<%=_T('Do you want to delete attribute?')%>"
		},
		{
			id : "rename",
			name : "<%=_T('Rename')%>",
			description : "<%=_T('Rename attribute.')%>",
			permission : "save_flow_classes"
		}
	];

	function adm_process_flow_attribute_editor(main, classEditor, id, data) {
		adm_process_flow_attribute_editor.uber.constructor.call(this, main);
		this.classEditor = classEditor;
		this.id = id;
		this.data = data;
		
		this.descriptionDefinition = {
			id : "description",
			name : "<%=_T('Description')%>",
			description : "<%=_T('Attribute description.')%>"
		};
		this.examplesDefinition = {
			id : "examples",
			name : "<%=_T('Examples')%>",
			description : "<%=_T('Attribute extended description and usage examples.')%>"
		};
		
		this.focusFirst = true;
	}

	inConcert.Utils.inherit( adm_process_flow_attribute_editor, inConcert.adm_process_flow_class_editor_base );

	if (typeof inConcert.adm_process_flow_attribute_editor !== "function") {
		inConcert.adm_process_flow_attribute_editor = adm_process_flow_attribute_editor;
	}
	
	adm_process_flow_attribute_editor.prototype.__getClassName = function() {
		return "attributeEditorContainer";
	};
	
	adm_process_flow_attribute_editor.prototype.GetTitle = function() {
		return format("<%=_T('Class {0} - Attribute {1}')%>", this.classEditor.data.Name, this.data.name);
	};
	
	adm_process_flow_attribute_editor.prototype.IsReadonly = function() {
		return this.classEditor.IsReadonly();
	};
	
	adm_process_flow_attribute_editor.prototype.__makeValue = function() {
		var self = this;
		var $container = $(Mustache.render(templates.editor_value_container, {
			name : "<%=_T('Value')%>",
			description : "<%=_T('Attribute initial value.')%>"
		}));
		$container.find("[attribute_value]")
			.val(self.data.value || "")
			.prop("readonly", self.IsReadonly())
			.change(function() {
				self.__updateTooltip();
			});
		return $container;
	};
	
	adm_process_flow_attribute_editor.prototype.__makeContent = function() {
		var $content = [];
		$content.push(this.__makeValue());
		$content.push(this.__makeDescription());
		$content.push(this.__makeExamples());
		return $content;
	};
	
	adm_process_flow_attribute_editor.prototype.__getValue = function(data, errors) {
		data.value = this.$get("[attribute_value]").val() || "";
	};
	
	adm_process_flow_attribute_editor.prototype.GetData = function(errors) {
		var data = {
			name : this.data.name,
			description : this.__getDescription(),
			examples : this.__getExamples()
		};
		this.__getValue(data, errors);
		if (data.error) {
			if (errors) {
				this.__showError(data.error);
				this.main.toolbar.SelectAttribute(this.classEditor.data.Id, this.id, true);
				throw data.error;
			}
			else {
				data.error = undefined;
			}
		}
		return data;
	};
	
	adm_process_flow_attribute_editor.prototype.GetActions = function() {
		return actions;
	};
	
	adm_process_flow_attribute_editor.prototype.__duplicate = function() {
		var data = JSON.parse(JSON.stringify(this.GetData(false)));
		var popup = new inConcert.adm_process_flow_duplicate_attribute_popup(this.main, this.classEditor, data);
		popup.Show();
	};
	
	adm_process_flow_attribute_editor.prototype.__delete = function() {
		this.classEditor.DeleteAttribute(this.id);
	};
	
	adm_process_flow_attribute_editor.prototype.__rename = function() {
		var popup = new inConcert.adm_process_flow_rename_attribute_popup(this.main, this.classEditor, this);
		popup.Show();
	};
	
	adm_process_flow_attribute_editor.prototype.Rename = function(name) {
		this.data.name = name;
		this.main.UpdateTitle(this);
		this.main.toolbar.UpdateAttribute({
			class_id : this.classEditor.data.Id,
			id : this.id,
			name : this.data.name,
			method : true
		});
	};
	
	adm_process_flow_attribute_editor.prototype.__updateTooltip = function() {
		this.classEditor.UpdateAttributeTooltip(this.id);
	};

}());
