/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double */

/* global $, inConcert, Mustache, _ */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	var format = inConcert.Utils.format;
	var templates = inConcert.templates.process_flow_classes;
	
	var actions = [
		{
			id : "duplicate",
			name : "<%=_T('Duplicate')%>",
			description : "<%=_T('Duplicate Class')%>",
			selection : true,
			singleSelection : true,
			permission : "save_flow_classes"
		},
		{
			id : "delete",
			name : "<%=_T('Delete')%>",
			description : "<%=_T('Delete Class')%>",
			permission : "delete_flow_classes",
			confirmation : "<%=_T('Do you want to delete class?')%>"
		},
		{
			id : "rename",
			name : "<%=_T('Rename')%>",
			description : "<%=_T('Rename Class')%>",
			permission : "save_flow_classes"
		}
	];
	
	var reservedWords = {
		"new" : true
	};

	function adm_process_flow_class_editor(main, data) {
		adm_process_flow_class_editor.uber.constructor.call(this, main);
		this.data = data;
		this.data.Editor = this.data.Editor || {
			classConstructor : {
				parameters : [],
				code : "",
				description : "",
				examples : ""
			},
			methods : {},
			attributes : {}
		};
		if (!this.data.Editor.classConstructor && this.data.Editor.constructor) {
			this.data.Editor.classConstructor = this.data.Editor.constructor;
		}
		if ($.isArray(this.data.Editor.methods)) {
			this.data.Editor.methods = {};
		}
		if ($.isArray(this.data.Editor.attributes)) {
			this.data.Editor.attributes = {};
		}
		
		this.descriptionDefinition = {
			id : "Description",
			name : "<%=_T('Description')%>",
			description : "<%=_T('Class description.')%>"
		};
		this.examplesDefinition = {
			id : "Examples",
			name : "<%=_T('Examples')%>",
			description : "<%=_T('Class extended description and usage examples.')%>"
		};
		
		this.focusFirst = true;
	}

	inConcert.Utils.inherit( adm_process_flow_class_editor, inConcert.adm_process_flow_class_editor_base );

	if (typeof inConcert.adm_process_flow_class_editor !== "function") {
		inConcert.adm_process_flow_class_editor = adm_process_flow_class_editor;
	}
	
	adm_process_flow_class_editor.prototype.__getClassName = function() {
		return "classEditorContainer";
	};
	
	adm_process_flow_class_editor.prototype.GetTitle = function() {
		return format("<%=_T('{0}')%>", this.data.Name);
	};
	
	adm_process_flow_class_editor.prototype.IsSystem = function() {
		return !!this.data.System;
	};
	
	adm_process_flow_class_editor.prototype.IsReadonly = function() {
		return this.main.readonly || this.IsSystem();
	};
	
	adm_process_flow_class_editor.prototype.__makeContent = function() {
		var $content = [];
		$content.push(this.__makeDescription());
		$content.push(this.__makeExamples());
		return $content;
	};
	
	adm_process_flow_class_editor.prototype.GetAttributes = function() {
		var self = this;
		var attributes = [];
		$.each(this.data.Editor.attributes, function(id, attribute) {
			attributes.push({
				class_id : self.data.Id,
				id : id,
				name : attribute.name,
				description : self.__makeAttributeTooltip(id)
			});
		});
		attributes.sort(function(a, b) {
			return a.name.localeCompare(b.name);
		});
		return attributes;
	};
	
	adm_process_flow_class_editor.prototype.GetMethods = function() {
		var self = this;
		var methods = [];
		$.each(this.data.Editor.methods, function(id, method) {
			methods.push({
				class_id : self.data.Id,
				id : id,
				name : method.name,
				description : self.__makeMethodTooltip(id),
				method : true
			});
		});
		methods.sort(function(a, b) {
			return a.name.localeCompare(b.name);
		});
		methods.unshift({
			class_id : self.data.Id,
			id : "constructor",
			name : "new",
			description : self.__makeMethodTooltip("constructor"),
			constructor : true
		});
		return methods;
	};
	
	adm_process_flow_class_editor.prototype.GetAttributeEditor = function(attributeId) {
		var attribute = this.data.Editor.attributes[attributeId];
		if (!attribute) {
			return null;
		}
		if (!attribute.editor) {
			attribute.editor = new inConcert.adm_process_flow_attribute_editor(this.main, this, attributeId, attribute);
		}
		return attribute.editor;
	};
	
	adm_process_flow_class_editor.prototype.GetMethodEditor = function(methodId) {
		var method = methodId === "constructor"? this.data.Editor.classConstructor : this.data.Editor.methods[methodId];
		if (!method) {
			return null;
		}
		if (!method.editor) {
			method.editor = new inConcert.adm_process_flow_method_editor(this.main, this, methodId, method);
		}
		return method.editor;
	};
	
	adm_process_flow_class_editor.prototype.__getEditorData = function(data, errors) {
		return data.editor? data.editor.GetData(errors) : data;
	};
	
	adm_process_flow_class_editor.prototype.GetData = function(errors) {
		var self = this;
		if (!self.$body) {
			var data = {};
			$.each(self.data, function(k, v) {
				if (v !== undefined && v !== null) {
					data[k] = v;
				}
			});
			return data;
		}
		try {
			//self data
			var data = {
				Id : self.data.Id,
				Name : self.data.Name,
				Description : self.__getDescription(),
				Examples : self.__getExamples(),
				Editor : {
					methods : {},
					attributes : {}
				}
			};
			if (data.error) {
				if (errors) {
					this.__showError(data.error);
					this.main.toolbar.SelectClass(this.data.Id, true);
					throw data.error;
				}
				else {
					data.error = undefined;
				}
			}
			//attributes
			$.each(self.data.Editor.attributes, function(attributeId, attributeData) {
				data.Editor.attributes[attributeId] = self.__getEditorData(attributeData, errors);
			});
			//constructor
			data.Editor.classConstructor = self.__getEditorData(self.data.Editor.classConstructor, errors);
			//methods
			$.each(self.data.Editor.methods, function(methodId, methodData) {
				data.Editor.methods[methodId] = self.__getEditorData(methodData, errors);
			});
			
			return data;
		}
		catch(e) {
			return null;
		}
	};
	
	adm_process_flow_class_editor.prototype.HasUnsavedChanges = function() {
		return !this.IsReadonly() && (!this.savedData || !_.isEqual(this.GetData(false), this.savedData));
	};
	
	adm_process_flow_class_editor.prototype.IsNew = function() {
		return !this.savedData;
	};
	
	adm_process_flow_class_editor.prototype.SetSavedData = function(data) {
		this.savedData = JSON.parse(JSON.stringify(data));
	};
	
	adm_process_flow_class_editor.prototype.ClearErrors = function() {
		var self = this;
		if (!self.$body) {
			return;
		}
		self.__clearErrors();
		//attributes
		$.each(self.data.Editor.attributes, function(_, attributeData) {
			if (attributeData.editor) {
				attributeData.editor.__clearErrors();
			}
		});
		//constructor
		if (self.data.Editor.classConstructor.editor) {
			self.data.Editor.classConstructor.editor.__clearErrors();
		}
		//methods
		$.each(self.data.Editor.methods, function(_, methodData) {
			if (methodData.editor) {
				methodData.editor.__clearErrors();
			}
		});
	};
	
	adm_process_flow_class_editor.prototype.AddAttribute = function(id, data) {
		this.data.Editor.attributes[id] = data;
		this.main.toolbar.AddAttribute({
			class_id : this.data.Id,
			id : id,
			name : data.name,
			description : this.__makeAttributeTooltip(id)
		});
	};
	
	adm_process_flow_class_editor.prototype.AddMethod = function(id, data) {
		this.data.Editor.methods[id] = data;
		this.main.toolbar.AddMethod({
			class_id : this.data.Id,
			id : id,
			name : data.name,
			description : this.__makeMethodTooltip(id),
			method : true
		});
	};
	
	adm_process_flow_class_editor.prototype.ValidatePropertyName = function(name, id) {
		//check reserved words
		if (reservedWords[name]) {
			return "<%=_T('This name is a reserved word.')%>";
		}
		var found = false;
		//check attributes
		$.each(this.data.Editor.attributes, function(attributeId, attribute) {
			if (attribute.name === name && attributeId !== id) {
				found = true;
				return false;
			}
		});
		if (found) {
			return "<%=_T('Already exists an attribute with same name.')%>";
		}
		//check methods
		$.each(this.data.Editor.methods, function(methodId, method) {
			if (method.name === name && methodId !== id) {
				found = true;
				return false;
			}
		});
		if (found) {
			return "<%=_T('Already exists a method with same name.')%>";
		}
	};
	
	adm_process_flow_class_editor.prototype.GetActions = function() {
		return actions;
	};
	
	adm_process_flow_class_editor.prototype.__duplicate = function() {
		var data = JSON.parse(JSON.stringify(this.GetData(false)));
		var popup = new inConcert.adm_process_flow_duplicate_class_popup(this.main, data);
		popup.Show();
	};
	
	adm_process_flow_class_editor.prototype.__delete = function() {
		this.main.DeleteClass(this.data.Id);
	};
	
	adm_process_flow_class_editor.prototype.__rename = function() {
		var popup = new inConcert.adm_process_flow_rename_class_popup(this.main, this);
		popup.Show();
	};
	
	adm_process_flow_class_editor.prototype.Rename = function(name) {
		this.data.Name = name;
		this.main.UpdateTitle(this);
		this.main.toolbar.UpdateClass(this);
	};
	
	adm_process_flow_class_editor.prototype.DeleteAttribute = function(attributeId) {
		var self = this;
		var oldMethods = self.data.Editor.attributes;
		self.data.Editor.attributes = {};
		$.each(oldMethods, function(id, data) {
			if (id !== attributeId) {
				self.data.Editor.attributes[id] = data;
			}
		});
		this.main.toolbar.RemoveAttribute(this.data.Id, attributeId);
		this.main.toolbar.SelectClass(this.data.Id, true);
	};
	
	adm_process_flow_class_editor.prototype.DeleteMethod = function(methodId) {
		var self = this;
		var oldMethods = self.data.Editor.methods;
		self.data.Editor.methods = {};
		$.each(oldMethods, function(id, data) {
			if (id !== methodId) {
				self.data.Editor.methods[id] = data;
			}
		});
		this.main.toolbar.RemoveMethod(this.data.Id, methodId);
		this.main.toolbar.SelectClass(this.data.Id, true);
	};
	
	adm_process_flow_class_editor.prototype.MakeTooltip = function() {
		return this.$container()? this.__getDescription() : this.data.Description;
	};
	
	adm_process_flow_class_editor.prototype.__updateTooltip = function() {
		this.main.toolbar.UpdateClassTooltip(this.data.Id, this.MakeTooltip());
	};
	
	adm_process_flow_class_editor.prototype.__makeParametersTooltip = function(data) {
		var snippet = "";
		$.each(data.parameters || [], function(_, parameter) {
			snippet += (snippet? ", " : "") + parameter.name;
		});
		return snippet;
	};
	
	adm_process_flow_class_editor.prototype.__makeMethodTooltip = function(methodId) {
		var method = methodId === "constructor"? this.data.Editor.classConstructor : this.data.Editor.methods[methodId];
		var data = method.editor? method.editor.GetData() : method;
		
		var snippet;
		if (methodId === "constructor") {
			snippet = format("local myObject = classHelpers.{0}.new({1})", this.data.Name, 
				this.__makeParametersTooltip(data));
		}
		else {
			snippet = format("myObject:{0}({1})", data.name, this.__makeParametersTooltip(data));
		}
		var tooltip = format("{0}\n{1}", snippet, data.description);
		return tooltip;
	};
	
	adm_process_flow_class_editor.prototype.UpdateMethodTooltip = function(methodId) {
		this.main.toolbar.UpdateMethodTooltip(this.data.Id, methodId, this.__makeMethodTooltip(methodId));
	};
	
	adm_process_flow_class_editor.prototype.__makeAttributeTooltip = function(attributeId) {
		var attribute = this.data.Editor.attributes[attributeId];
		var data = attribute.editor? attribute.editor.GetData() : attribute;
		var snippet = format("myObject.{0}", data.name);
		var defaultValue = format("<%=_T('Default value: {0}')%>", data.value || "nil");
		var tooltip = format("{0}\n{1}\n{2}", snippet, defaultValue, data.description);
		return tooltip;
	};
	
	adm_process_flow_class_editor.prototype.UpdateAttributeTooltip = function(attributeId) {
		this.main.toolbar.UpdateAttributeTooltip(this.data.Id, attributeId, this.__makeAttributeTooltip(attributeId));
	};

}());
