/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double */

/* global $, inConcert, Mustache, ace */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	var format = inConcert.Utils.format;
	var templates = inConcert.templates.process_flow_classes;
	
	var actions = [
		{
			id : "duplicate",
			name : "<%=_T('Duplicate')%>",
			description : "<%=_T('Duplicate method.')%>",
			selection : true,
			singleSelection : true,
			permission : "save_flow_classes"
		},
		{
			id : "delete",
			name : "<%=_T('Delete')%>",
			description : "<%=_T('Delete method.')%>",
			permission : "save_flow_classes",
			confirmation : "<%=_T('Do you want to delete method?')%>"
		},
		{
			id : "rename",
			name : "<%=_T('Rename')%>",
			description : "<%=_T('Rename method.')%>",
			permission : "save_flow_classes"
		}
	];
	
	var reservedWords = {
		"this" : true,
		"self" : true
	};

	function adm_process_flow_method_editor(main, classEditor, id, data) {
		adm_process_flow_method_editor.uber.constructor.call(this, main);
		this.classEditor = classEditor;
		this.id = id;
		this.data = data;
		this.constructor = id === "constructor";
		
		this.descriptionDefinition = {
			id : "description",
			name : "<%=_T('Description')%>",
			description : "<%=_T('Method description.')%>"
		};
		this.examplesDefinition = {
			id : "examples",
			name : "<%=_T('Examples')%>",
			description : "<%=_T('Method extended description and usage examples.')%>"
		};
	}

	inConcert.Utils.inherit( adm_process_flow_method_editor, inConcert.adm_process_flow_class_editor_base );

	if (typeof inConcert.adm_process_flow_method_editor !== "function") {
		inConcert.adm_process_flow_method_editor = adm_process_flow_method_editor;
	}
	
	adm_process_flow_method_editor.prototype.__getClassName = function() {
		return "methodEditorContainer";
	};
	
	adm_process_flow_method_editor.prototype.GetTitle = function() {
		return format("<%=_T('Class {0} - {1}')%>", this.classEditor.data.Name, 
			this.constructor? "<%=_T('Constructor')%>" : format("<%=_T('Method {0}')%>", this.data.name));
	};
	
	adm_process_flow_method_editor.prototype.IsReadonly = function() {
		return this.classEditor.IsReadonly();
	};
	
	adm_process_flow_method_editor.prototype.__makeParameter = function(parameter, readonly) {
		var self = this;
		parameter = parameter || { name : "" };
		var $parameter = $(Mustache.render(templates.editor_param, {
			name : parameter.name,
			id : inConcert.Utils.generateGuid(),
			readonly : !!readonly
		}));
		if (readonly) {
			$parameter.find("button, input").remove();
		}
		else {
			$parameter.find("[param_edit]").click(function() {
				self.__editParameter($(this).attr("param_edit"));
			});
			$parameter.find("[param_delete]").click(function() {
				self.__deleteParameter($(this).attr("param_delete"));
			});
			$parameter.find("[param_confirm]").click(function() {
				self.__confirmParameter($(this).attr("param_confirm"));
			});
			$parameter.find("[param_cancel]").click(function() {
				self.__cancelParameter($(this).attr("param_cancel"));
			});
			$parameter.dblclick(function() {
				if ($(this).hasClass("modeView")) {
					self.__editParameter($(this).attr("param"));
				}
			});
		}
		
		return $parameter;
	};
	
	adm_process_flow_method_editor.prototype.__addParameter = function() {
		var $parameter = this.__makeParameter();
		this.$get("[parameters_list]").append($parameter);
		this.__editParameter($parameter.attr("param"));
	};
	
	adm_process_flow_method_editor.prototype.__getParameter = function(id) {
		return this.$get(format("[param='{0}']", id));
	};
	
	adm_process_flow_method_editor.prototype.__editParameter = function(id) {
		var $parameter = this.__getParameter(id);
		$parameter
			.removeClass("modeView")
			.addClass("modeEdit")
			.find("[param_value]")
			.val($parameter.attr("param_name"))
			.focus();
	};
	
	adm_process_flow_method_editor.prototype.__cancelParameter = function(id) {
		var $parameter = this.__getParameter(id);
		if (!$parameter.attr("param_name")) {
			$parameter.remove();
			return;
		}
		$parameter
			.removeClass("modeEdit")
			.addClass("modeView");
	};
	
	adm_process_flow_method_editor.prototype.__validateParameter = function($parameter, edit, errors) {
		var data = {
			name : edit? $parameter.find("[param_value]").val() : $parameter.attr("param_name")
		};
		if (!errors) {
			return data;
		}
		if (!data.name) {
			data.error = "<%=_T('Parameter name is required.')%>";
		}
		if (!inConcert.Utils.isValid(data.name, "variable")) {
			data.error = "<%=_T('Invalid parameter name.')%>";
		}
		else if (reservedWords[data.name]) {
			data.error = "<%=_T('This name is a reserved word.')%>";
		}
		else {
			var $nameParameter = this.$get(format("[param_name='{0}']", data.name));
			if ($nameParameter.length && $nameParameter.attr("param") !== $parameter.attr("param")) {
				data.error = "<%=_T('Already exists a parameter with same name.')%>";
			}
		}
		return data;
	};
	
	adm_process_flow_method_editor.prototype.__confirmParameter = function(id) {
		var $parameter = this.__getParameter(id);
		var data = this.__validateParameter($parameter, true, true);
		if (data.error) {
			inConcert.app.m_mainWindow.showBannerMessage({ message: data.error, messageType : "Error", timeout: 5000 });
			return;
		}
		$parameter.replaceWith(this.__makeParameter(data));
		this.__updateTooltip();
	};
	
	adm_process_flow_method_editor.prototype.__deleteParameter = function(id) {
		this.__getParameter(id).remove();
		this.__updateTooltip();
	};
	
	adm_process_flow_method_editor.prototype.__makeParameters = function(readonly) {
		var self = this;
		var $container = $(Mustache.render(templates.editor_params_container, {
			name : "<%=_T('Parameters')%>",
			description : "<%=_T('Method parameters.')%>"
		}));
		var $list = $container.find("[parameters_list]");
		if (!readonly) {
			$list.sortable({
				cursor : "move"
			});
		}
		$.each(self.data.parameters || [], function(_, parameter) {
			$list.append(self.__makeParameter(parameter, readonly));
		});
		if (readonly) {
			$container.find("[parameters_add]").remove();
		}
		else {
			$container.find("[parameters_add]").click(function() {
				self.__addParameter();
			});
		}
		return $container;
	};
	
	adm_process_flow_method_editor.prototype.__makeCode = function(readonly) {
		var $container = $(Mustache.render(templates.editor_code_container, {
			name : "<%=_T('Code')%>",
			description : "<%=_T('Method code.')%>"
		}));
		this.codeEditor = ace.edit($container.find("[code_block]")[0]);
		this.codeEditor.getSession().setMode(format("ace{0}mode{0}lua", "/"));
		this.codeEditor.getSession().setValue(this.data.code || "");
		this.codeEditor.setReadOnly(readonly);
		
		return $container;
	};
	
	adm_process_flow_method_editor.prototype.__afterAppendBody = function() {
		this.codeEditor.focus();
	};
	
	adm_process_flow_method_editor.prototype.__makeContent = function() {
		var readonly = !!this.IsReadonly();
		var $content = [];
		$content.push(this.__makeParameters(readonly));
		$content.push(this.__makeCode(readonly));
		$content.push(this.__makeDescription());
		$content.push(this.__makeExamples());
		return $content;
	};
	
	adm_process_flow_method_editor.prototype.__getParameters = function(data, errors) {
		var self = this;
		data.parameters = [];
		self.$get("[param]").each(function() {
			var paramData = self.__validateParameter($(this), false, errors);
			if (paramData.error) {
				data.error = paramData.error;
				return false;
			}
			data.parameters.push(paramData);
		});
	};
	
	adm_process_flow_method_editor.prototype.__getCode = function(data, errors) {
		data.code = this.codeEditor.getValue();
		if (!errors) {
			return;
		}
		$.each(this.codeEditor.getSession().getAnnotations(), function(_, annotation) {
			if (annotation.type === "error") {
				data.error = "<%=_T('Invalid lua code.')%>";
				return false;
			}
		});
	};
	
	adm_process_flow_method_editor.prototype.GetData = function(errors) {
		var data = {
			description : this.__getDescription(),
			examples : this.__getExamples()
		};
		if (!this.constructor) {
			data.name = this.data.name;
		}
		this.__getParameters(data, errors);
		this.__getCode(data, errors);
		if (data.error) {
			if (errors) {
				this.__showError(data.error);
				this.main.toolbar.SelectMethod(this.classEditor.data.Id, this.id, true);
				throw data.error;
			}
			else {
				data.error = undefined;
			}
		}
		return data;
	};
	
	adm_process_flow_method_editor.prototype.GetActions = function() {
		return this.constructor? [] : actions;
	};
	
	adm_process_flow_method_editor.prototype.__duplicate = function() {
		var data = JSON.parse(JSON.stringify(this.GetData(false)));
		var popup = new inConcert.adm_process_flow_duplicate_method_popup(this.main, this.classEditor, data);
		popup.Show();
	};
	
	adm_process_flow_method_editor.prototype.__delete = function() {
		this.classEditor.DeleteMethod(this.id);
	};
	
	adm_process_flow_method_editor.prototype.__rename = function() {
		var popup = new inConcert.adm_process_flow_rename_method_popup(this.main, this.classEditor, this);
		popup.Show();
	};
	
	adm_process_flow_method_editor.prototype.Rename = function(name) {
		this.data.name = name;
		this.main.UpdateTitle(this);
		this.main.toolbar.UpdateMethod({
			class_id : this.classEditor.data.Id,
			id : this.id,
			name : this.data.name,
			method : true
		});
	};
	
	adm_process_flow_method_editor.prototype.__updateTooltip = function() {
		this.classEditor.UpdateMethodTooltip(this.id);
	};

}());
