/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double */

/* global $, inConcert, Mustache */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	var format = inConcert.Utils.format;
	var templates = inConcert.templates.process_flow_classes;

	function adm_process_flow_classes_editor(flowDesigner) {
		if (flowDesigner) {
			this.flowDesigner = flowDesigner;
			this.currentEditor = flowDesigner.currentEditor;
		}
	}

	if (typeof inConcert.adm_process_flow_classes_editor !== "function") {
		inConcert.adm_process_flow_classes_editor = adm_process_flow_classes_editor;
	}
	
	var globalActions = [
		{
			id : "save",
			name : "<%=_T('Save')%>",
			description : "<%=_T('Save Classes')%>",
			permission : "save_flow_classes"
		},
		{
			id : "export",
			name : "<%=_T('Export')%>",
			description : "<%=_T('Export Classes')%>",
			permission : "export_flow_classes"
		},
		{
			id : "import",
			name : "<%=_T('Import')%>",
			description : "<%=_T('Import Classes')%>",
			permission : "save_flow_classes"
		}
	];
	
	var fixedActions = [
		{
			id : "new_class",
			name : "<%=_T('New Class')%>",
			description : "<%=_T('New Class')%>",
			permission : "save_flow_classes"
		}
	];

	adm_process_flow_classes_editor.prototype.$element = function(name, $parent) {
		$parent = $parent || this.$container;
		return name? $parent.find(format("[{0}]", name)) : $parent;
	};
	
	adm_process_flow_classes_editor.prototype.__loadClasses = function(callback) {
		callback = callback || $.noop;
		var flowAPI = inConcert.app.getFlowDesignerAPI();
		flowAPI.getClasses(function(success, classes) {
			if (!success) {
				var message = "<%=_T('Error loading classes.')%>";
				inConcert.app.m_mainWindow.showBannerMessage({ message: message, messageType : "Error", timeout: 5000 });
				return callback(false);
			}
			var classes_a = [];
			$.each(classes, function(_, classData) {
				classes_a.push(classData);
			});
			classes_a.sort(function(a, b) {
				return a.Name.localeCompare(b.Name);
			});
			callback(classes_a);
		});
	};
	
	adm_process_flow_classes_editor.prototype.AddClass = function(classData, sorted, select) {
		var classEditor =  new inConcert.adm_process_flow_class_editor(this, classData);
		this.classes[classData.Id] = classEditor;
		this.toolbar.AddClass(classEditor, sorted);
		if (select) {
			this.toolbar.SelectClass(classData.Id, true);
		}
		return classEditor;
	};
	
	adm_process_flow_classes_editor.prototype.__make = function(callback) {
		callback = callback || $.noop;
		var self = this;
		self.readonly = !inConcert.permissions.to_action("flow_designer", "save_flow_classes");
		self.$container = $(templates.base);
		self.$element("close_button").click(function() {
			self.Close();
		});
		self.$element("minimize_button").click(function() {
			self.Minimize();
		});
		self.__loadClasses(function(classes) {
			if (!classes) {
				return callback();
			}
			self.__makeGlobalActions();
			self.__makeToolbar();
			self.classes = {};
			$.each(classes, function(_, classData) {
				classData.Executable = undefined;
				var classEditor = self.AddClass(classData, true);
				classEditor.SetSavedData(classData);
			});
			callback();
		});
	};
	
	adm_process_flow_classes_editor.prototype.__makeGlobalAction = function(action) {
		var self = this;
		var $action = $(Mustache.render(templates.action_button, action));
		$action.click(function() {
			self["__" + $(this).attr("action_button")]();
		});
		return $action;
	};
	
	adm_process_flow_classes_editor.prototype.__makeGlobalActions = function() {
		var self = this;
		//global
		var $actions = [];
		$.each(globalActions, function(_, action) {
			if (action.permission && !inConcert.permissions.to_action("flow_designer", action.permission)) {
				return;
			}
			action.withText = !action.withoutText;
			$actions.push(self.__makeGlobalAction(action));
		});
		self.$element("global_actions").append($actions);
		//fixed
		$actions = [];
		$.each(fixedActions, function(_, action) {
			if (action.permission && !inConcert.permissions.to_action("flow_designer", action.permission)) {
				return;
			}
			action.withText = !action.withoutText;
			action.fixedButton = true;
			$actions.push(self.__makeGlobalAction(action));
		});
		self.$element("editor_actions").append($actions);
	};
	
	adm_process_flow_classes_editor.prototype.__makeToolbar = function() {
		this.toolbar = new inConcert.adm_process_flow_classes_toolbar(this);
	};
	
	adm_process_flow_classes_editor.prototype.Show = function(callback, onClose) {
		callback = callback || $.noop;
		var self = this;
		self.onClose = onClose || $.noop;
		
		function show() {
			$("#flow_designer_dashboard").append(self.$container);
			callback();
		}
		
		if (self.$container) {
			return show();
		}
		self.__make(function() {
			show();
		});
	};
	
	adm_process_flow_classes_editor.prototype.Close = function(callback) {
		callback = callback || $.noop;
		var self = this;
		
		function close() {
			if (self.$container) {
				self.$container.remove();
				self.$container = null;
				// self.currentEditor.main.__keepDesignerScreenMode();
			}
			self.onClose();
			callback();
		}
		
		if (!self.__hasUnsavedChanges()) {
			return close();
		}
		$.inconcertPopup.Show(
			"<%=_T('Classes IDE')%>",
			"<%=_T('Do you want to save changes?')%>",
			$.inconcertPopup.ButtonSets.BS_SAVECLOSE,
			function(response) {
				switch(response.toLowerCase()) {
					case "save":
						return self.__save(function(status) {
							if (status) {
								close();
							}
						});
					case "close":
						return close();
				}
			},
			undefined,
			{
				popup_id        : "conf_inconcertPopup",
				container_class : "popUp380w confirmationPopUp",
				overlay_class   : "confirmationOverlay"
			}
		);
	};

	adm_process_flow_classes_editor.prototype.__process = function(process) {
		this.processing = !!process;
		this.$element("processing").toggle(this.processing);
	};

	adm_process_flow_classes_editor.prototype.__isProcessing = function() {
		return !!this.processing;
	};
	
	adm_process_flow_classes_editor.prototype.__save = function(callback) {
		callback = callback || $.noop;
		var self = this;
		if (self.__isProcessing()) {
			return callback(false);
		}
		//clear errors
		inConcert.app.m_mainWindow.hideBannerMessage();
		$.each(self.classes, function(_, classEditor) {
			classEditor.ClearErrors();
		});
		//get data
		var data = {};
		var newClasses = [];
		var error = false;
		$.each(self.classes, function(classId, classEditor) {
			if (classEditor.IsReadonly()) {
				return;
			}
			var classData = classEditor.GetData(true);
			if (!classData) {
				error = true;
				return false;
			}
			data[classId] = classData;
			if (classEditor.IsNew()) {
				newClasses.push(classEditor.data.Id);
			}
		});
		if (error) {
			return callback(false);
		}
		//save
		self.__process(true);
		var flowAPI = inConcert.app.getFlowDesignerAPI();
		flowAPI.saveClasses(data, function(success, error) {
			self.__process(false);
			if (success) {
				$.each(self.classes, function(classId, classEditor) {
					if (data[classId]) {
						classEditor.SetSavedData(data[classId]);
					}
				});
				if (self.flowDesigner) {
					self.flowDesigner.AddClasses(newClasses);
					self.flowDesigner.SetClasses(data);
				}
				var message = "<%=_T('Classes saved successfully.')%>";
				inConcert.app.m_mainWindow.showBannerMessage({ message: message, messageType : "Ok", timeout: 1500 });
			}
			else {
				var message;
				switch(error && error.result) {
					case "FORBIDDEN":
						message = "<%=_T('Unable to save process flow classes. You dont have permission.')%>";
						break;
					default:
						message = "<%=_T('Error saving classes.')%>";
						break;
				}
				inConcert.app.m_mainWindow.showBannerMessage({ message: message, messageType : "Error", timeout: 5000 });
			}
			callback(success);
		});
	};
	
	adm_process_flow_classes_editor.prototype.__removeClass = function(classId) {
		var self = this;
		self.toolbar.RemoveClass(classId);
		var oldClasses = self.classes;
		self.classes = {};
		$.each(oldClasses, function(id, editor) {
			if (id !== classId) {
				self.classes[id] = editor;
			}
		});
		if (self.flowDesigner) {
			self.flowDesigner.RemoveClass(classId);
		}
		self.__showEditor();
	};
	
	adm_process_flow_classes_editor.prototype.DeleteClass = function(classId) {
		var self = this;
		if (self.__isProcessing()) {
			return;
		}
		self.__process(true);
		var flowAPI = inConcert.app.getFlowDesignerAPI();
		flowAPI.deleteClasses([ classId ], function(success, error) {
			self.__process(false);
			if (success) {
				self.__removeClass(classId);
				var message = "<%=_T('Class deleted successfully.')%>";
				inConcert.app.m_mainWindow.showBannerMessage({ message: message, messageType : "Ok", timeout: 1500 });
			}
			else {
				var message;
				switch(error && error.result) {
					case "FORBIDDEN":
						message = "<%=_T('Unable to delete process flow class. You dont have permission.')%>";
						break;
					case "CLASS_IN_USE":
						var flows_s = "";
						$.each(error.flows || [], function(_, flowData) {
							flows_s += (flows_s? ", " : "") + flowData.name;
						});
						message = format("<%=_T('Class is currently in use by these flows: {0}.')%>", flows_s);
						break;
					default:
						message = "<%=_T('Error deleting class.')%>";
						break;
				}
				inConcert.app.m_mainWindow.showBannerMessage({ message: message, messageType : "Error", timeout: 5000 });
				return;
			}
		});
	};
	
	adm_process_flow_classes_editor.prototype.__hasUnsavedChanges = function() {
		var has = false;
		$.each(this.classes, function(_, classEditor) {
			has = has || classEditor.HasUnsavedChanges();
			if (has) {
				return false;
			}
		});
		return has;
	};
	
	adm_process_flow_classes_editor.prototype.__export = function() {
		var self = this;

		function doExport() {
			var popup = new inConcert.adm_process_flow_export_classes_popup(self);
			popup.Show();
		}
		
		if (self.__isProcessing()) {
			return;
		}

		//save validation
		if (!self.__hasUnsavedChanges()) {
			return doExport();
		}

		$.inconcertPopup.Show(
			"<%=_T('Confirmation')%>",
			"<%=_T('Must save your changes before export classes.')%>",
			$.inconcertPopup.ButtonSets.BS_SAVECANCEL,
			function(response) {
				switch(response.toLowerCase()) {
					case "save":
						return self.__save(function(status) {
							if (status) {
								doExport();
							}
						});
				}
			},
			undefined,
			{
				popup_id        : "conf_inconcertPopup",
				container_class : "popUp380w confirmationPopUp",
				overlay_class   : "confirmationOverlay"
			}
		);
	};
	
	adm_process_flow_classes_editor.prototype.ExportClasses = function(classes, callback) {
		inConcert.app.getFlowDesignerAPI().exportFlowClasses(classes, function() {
			callback(true);
		});
	};
	
	adm_process_flow_classes_editor.prototype.__import = function() {
		var self = this;

		function doImport() {
			var popup = new inConcert.adm_process_flow_import_classes_popup(self);
			popup.Show();
		}
		
		if (self.__isProcessing()) {
			return;
		}

		//save validation
		if (!self.__hasUnsavedChanges()) {
			return doImport();
		}

		$.inconcertPopup.Show(
			"<%=_T('Confirmation')%>",
			"<%=_T('Must save your changes before import classes.')%>",
			$.inconcertPopup.ButtonSets.BS_SAVECANCEL,
			function(response) {
				switch(response.toLowerCase()) {
					case "save":
						return self.__save(function(status) {
							if (status) {
								doImport();
							}
						});
				}
			},
			undefined,
			{
				popup_id        : "conf_inconcertPopup",
				container_class : "popUp380w confirmationPopUp",
				overlay_class   : "confirmationOverlay"
			}
		);
	};
	
	adm_process_flow_classes_editor.prototype.ImportClasses = function(classes, callback) {
		var self = this;
		$.each(classes, function(_, flowClass) {
			if (self.classes[flowClass.Id]) {
				self.__removeClass(flowClass.Id);
			}
			self.AddClass(flowClass, false, true);
		});
		self.__save(callback);
	};
	
	adm_process_flow_classes_editor.prototype.__new_class = function() {
		if (this.__isProcessing()) {
			return;
		}
		var popup = new inConcert.adm_process_flow_new_class_popup(this);
		popup.Show();
	};
	
	adm_process_flow_classes_editor.prototype.NewAttribute = function(classId) {
		if (this.__isProcessing()) {
			return;
		}
		var classEditor = this.classes[classId];
		if (!classEditor) {
			return;
		}
		var popup = new inConcert.adm_process_flow_new_attribute_popup(this, classEditor);
		popup.Show();
	};
	
	adm_process_flow_classes_editor.prototype.NewMethod = function(classId) {
		if (this.__isProcessing()) {
			return;
		}
		var classEditor = this.classes[classId];
		if (!classEditor) {
			return;
		}
		var popup = new inConcert.adm_process_flow_new_method_popup(this, classEditor);
		popup.Show();
	};
	
	adm_process_flow_classes_editor.prototype.__showEditor = function(editor) {
		if (this.currentEditor) {
			this.currentEditor.Close();
		}
		this.currentEditor = editor || null;
		if (this.currentEditor) {
			this.currentEditor.Show();
			this.__setTitle(this.currentEditor.GetTitle());
		}
		else {
			this.__setTitle("");
		}
	};
	
	adm_process_flow_classes_editor.prototype.__setTitle = function(title) {
		this.$element("context_name").text(" " + title);
	};
	
	adm_process_flow_classes_editor.prototype.UpdateTitle = function(editor) {
		if (this.currentEditor === editor) {
			this.__setTitle(this.currentEditor.GetTitle());
		}
	};
	
	adm_process_flow_classes_editor.prototype.ShowClassEditor = function(classId) {
		this.__showEditor(this.classes[classId]);
	};
	
	adm_process_flow_classes_editor.prototype.ShowAttributeEditor = function(classId, attributeId) {
		var classEditor = this.classes[classId];
		if (!classEditor) {
			return;
		}
		var attributeEditor = classEditor.GetAttributeEditor(attributeId);
		this.__showEditor(attributeEditor);
	};
	
	adm_process_flow_classes_editor.prototype.ShowMethodEditor = function(classId, methodId) {
		var classEditor = this.classes[classId];
		if (!classEditor) {
			return;
		}
		var methodEditor = classEditor.GetMethodEditor(methodId);
		this.__showEditor(methodEditor);
	};

	adm_process_flow_classes_editor.prototype.GetClassFromName = function(name) {
		var classId;
		$.each(this.classes, function(id, classEditor) {
			if (classEditor.data.Name === name) {
				classId = id;
				return false;
			}
		});
		return classId;
	};

}());