/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double */

/* global $, inConcert, Mustache */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	var templates = inConcert.templates.process_flow_classes;
	var format = inConcert.Utils.format;

	function adm_process_flow_classes_toolbar(main) {
		this.main = main;
		this.__make();
	}

	if (typeof inConcert.adm_process_flow_classes_toolbar !== "function") {
		inConcert.adm_process_flow_classes_toolbar = adm_process_flow_classes_toolbar;
	}
	
	adm_process_flow_classes_toolbar.prototype.__make = function() {
		
	};
	
	adm_process_flow_classes_toolbar.prototype.__addSorted = function($classes, $item, name) {
		var itemName = $item.attr(name);
		var $before = $item;
		$classes.each(function() {
			if ($(this).attr(name).localeCompare(itemName) > 0) {
				return false;
			}
			$before = $(this);
		});
		if ($before === $item) {
			$classes.first().before($item);
		}
		else {
			$before.after($item);
		}
	};
	
	adm_process_flow_classes_toolbar.prototype.SelectClass = function(classId, selected) {
		var $class = this.main.$element("toolbar_class=" + classId);
		
		$class
			.addClass("selected")
			.toggleClass("extended", selected)
			.removeClass("childSelected");
		
		$class.siblings("[toolbar_class]")
			.removeClass("childSelected")
			.removeClass("selected")
			.removeClass("extended");
		
		$class.find(".selected")
			.removeClass("selected");

		$class.parent()
			.scrollTo($class);

		this.main.ShowClassEditor(classId);
	};
	
	adm_process_flow_classes_toolbar.prototype.SelectAttribute = function(classId, attributeId, selected) {
		var $class = this.main.$element("toolbar_class=" + classId);
		$class
			.addClass("childSelected")
			.find(".selected")
			.removeClass("selected");
	
		var $attribute = this.main.$element("toolbar_attribute=" + attributeId);
		$attribute
			.toggleClass("selected", selected)
			.parent()
			.scrollTo($attribute);

		this.main.ShowAttributeEditor(classId, attributeId);
	};
	
	adm_process_flow_classes_toolbar.prototype.SelectMethod = function(classId, methodId, selected) {
		var $class = this.main.$element("toolbar_class=" + classId);
		$class
			.addClass("childSelected")
			.find(".selected")
			.removeClass("selected");
	
		var $method = this.main.$element("toolbar_method=" + methodId);
		$method
			.toggleClass("selected", selected)
			.parent()
			.scrollTo($method);

		this.main.ShowMethodEditor(classId, methodId);
	};
	
	adm_process_flow_classes_toolbar.prototype.__makeAttribute = function(attribute) {
		var self = this;
		var $attribute = $(Mustache.render(templates.toolbar_attribute, attribute));
		$attribute.click(function() {
			self.SelectAttribute($(this).attr("class_id"), $(this).attr("toolbar_attribute"));
		});
		return $attribute;
	};
	
	adm_process_flow_classes_toolbar.prototype.__makeAttributes = function(attributes) {
		var self = this;
		var $attributes = [];
		$.each(attributes, function(_, attribute) {
			$attributes.push(self.__makeAttribute(attribute));
		});
		return $attributes;
	};
	
	adm_process_flow_classes_toolbar.prototype.__makeMethod = function(method) {
		var self = this;
		var $method = $(Mustache.render(templates.toolbar_method, method));
		$method.click(function() {
			self.SelectMethod($(this).attr("class_id"), $(this).attr("toolbar_method"));
		});
		return $method;
	};
	
	adm_process_flow_classes_toolbar.prototype.__makeMethods = function(methods) {
		var self = this;
		var $methods = [];
		$.each(methods, function(_, method) {
			$methods.push(self.__makeMethod(method));
		});
		return $methods;
	};
	
	adm_process_flow_classes_toolbar.prototype.__insertClass = function($item, sorted) {
		var $container = this.main.$element("tool_bar_list");
		var $classes = $container.children("[toolbar_class]");
		if (sorted || !$classes.length) {
			$container.append($item);
		}
		else {
			this.__addSorted($classes, $item, "class_name");
		}
	};
	
	adm_process_flow_classes_toolbar.prototype.AddClass = function(classEditor, sorted) {
		var self = this;
		var readonly = classEditor.IsReadonly();
		
		var $item = $(Mustache.render(templates.toolbar_class, {
			id : classEditor.data.Id,
			name : classEditor.data.Name,
			description : classEditor.MakeTooltip(),
			readonly : !!readonly,
			system : classEditor.IsSystem()
		}));
		
		$item.click(function(e) {
			var $target = $(e.target);
			if (!$target.attr("class_selector")) {
				return;
			}
			var $titleDiv = $(this).children("[class_selector_title]");
			var minHeight = $titleDiv.offset().top + $titleDiv.height() + 15;
			if (minHeight < e.pageY) {
				return;
			}
				
			self.SelectClass($(this).attr("toolbar_class"));
		});
		
		if (readonly) {
			$item.find("[add_attribute]").remove();
			$item.find("[add_method]").remove();
		}
		else {
			$item.find("[add_attribute]").click(function() {
				self.main.NewAttribute($(this).attr("add_attribute"));
			});
			$item.find("[add_method]").click(function() {
				self.main.NewMethod($(this).attr("add_method"));
			});
		}
		
		$item.find("[attributes_list]").prepend(self.__makeAttributes(classEditor.GetAttributes()));
		$item.toggleClass("withoutAttributes", $item.find("[toolbar_attribute]").length === 0);
		$item.find("[methods_list]").prepend(self.__makeMethods(classEditor.GetMethods()));
		
		self.__insertClass($item, sorted);
	};
	
	adm_process_flow_classes_toolbar.prototype.RemoveClass = function(classId) {
		this.main.$element(format("toolbar_class={0}", classId)).remove();
	};
	
	adm_process_flow_classes_toolbar.prototype.UpdateClass = function(classEditor) {
		this.RemoveClass(classEditor.data.Id);
		this.AddClass(classEditor);
	};
	
	adm_process_flow_classes_toolbar.prototype.AddAttribute = function(attribute) {
		var $attribute = this.__makeAttribute(attribute);
		var $container = this.main.$element(format("toolbar_class={0}", attribute.class_id));
		$container.removeClass("withoutAttributes");
		var $list = $container.find("[attributes_list]");
		var $attributes = $list.children("[toolbar_attribute]");
		if (!$attributes.length) {
			$list.prepend($attribute);
		}
		else {
			this.__addSorted($attributes, $attribute, "attribute_name");
		}
		this.SelectAttribute(attribute.class_id, attribute.id, true);
	};
	
	adm_process_flow_classes_toolbar.prototype.RemoveAttribute = function(classId, attributeId) {
		var $container = this.main.$element(format("toolbar_class={0}", classId));
		$container.toggleClass("withoutAttributes", $container.find("[toolbar_attribute]").length <= 1);
		var $list = $container.find("[attributes_list]");
		$list
			.find(format("[toolbar_attribute={0}]", attributeId))
			.remove();
	};
	
	adm_process_flow_classes_toolbar.prototype.UpdateAttribute = function(attribute) {
		this.RemoveAttribute(attribute.class_id, attribute.id);
		this.AddAttribute(attribute);
	};
	
	adm_process_flow_classes_toolbar.prototype.AddMethod = function(method) {
		var $attribute = this.__makeMethod(method);
		var $container = this.main.$element(format("toolbar_class={0}", method.class_id))
			.find("[methods_list]");
		var $methods = $container.children("[method_name]");
		if (!$methods.length) {
			$container
				.find("[toolbar_method=constructor]")
				.after($attribute);
		}
		else {
			this.__addSorted($methods, $attribute, "method_name");
		}
		this.SelectMethod(method.class_id, method.id, true);
	};
	
	adm_process_flow_classes_toolbar.prototype.RemoveMethod = function(classId, methodId) {
		this.main.$element(format("toolbar_class={0}", classId))
			.find(format("[toolbar_method={0}]", methodId))
			.remove();
	};
	
	adm_process_flow_classes_toolbar.prototype.UpdateMethod = function(method) {
		this.RemoveMethod(method.class_id, method.id);
		this.AddMethod(method);
	};
	
	adm_process_flow_classes_toolbar.prototype.UpdateClassTooltip = function(classId, tooltip) {
		this.main.$element(format("toolbar_class={0}", classId))
			.find("[class_selector_title]")
			.attr("title", tooltip);
	};
	
	adm_process_flow_classes_toolbar.prototype.UpdateMethodTooltip = function(classId, methodId, tooltip) {
		this.main.$element(format("toolbar_class={0}", classId))
			.find(format("[toolbar_method={0}]", methodId))
			.attr("title", tooltip);
	};
	
	adm_process_flow_classes_toolbar.prototype.UpdateAttributeTooltip = function(classId, attributeId, tooltip) {
		this.main.$element(format("toolbar_class={0}", classId))
			.find(format("[toolbar_attribute={0}]", attributeId))
			.attr("title", tooltip);
	};

}());
