/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	var templates = inConcert.templates.process_flows;

	function adm_process_flow_connection_popup(editor, connector) {
		adm_process_flow_connection_popup.uber.constructor.call(this);
		this.editor = editor;
		this.connector = connector;
		var data = editor.GetNodeData(connector);
		this.isNew = !data;
		this.data = data || {};
		this.frm = editor.main.frm;

		this.descriptionDefinition = {
			id : "description",
			name : "<%=_T('Description')%>",
			placeholder : "<%=_T('Enter description')%>...",
			description : "<%=_T('Transition description.')%>"
		};
		this.focusFirst = true;
	}

	inConcert.Utils.inherit( adm_process_flow_connection_popup, inConcert.adm_process_flow_popup );

	if (typeof inConcert.adm_process_flow_connection_popup !== "function") {
		inConcert.adm_process_flow_connection_popup = adm_process_flow_connection_popup;
	}

	adm_process_flow_connection_popup.prototype.__getClassName = function() {
		return "flowDesignerPopupSetTransition popOver940w flowDesignerGlobalModal";
	};

	adm_process_flow_connection_popup.prototype.__getTitle = function() {
		let descriptionTitle = this.isNew? "<%=_T('New transition')%>" : "<%=_T('Edit transition')%>";
		if (this.data.description !== undefined && this.data.description !== "") {
			if (!this.isNew && (this.data.description !== "<%=_T('Edit transition')%>" || this.data.description !== "<%=_T('New transition')%>"))
				return this.data.description
		}
		return descriptionTitle;
	};

	adm_process_flow_connection_popup.prototype.__afterAppendBody = function() {
		const self = this;
		self.__addAutocompleteToInput();
	};

	adm_process_flow_connection_popup.prototype.__addAutocompleteToInput = function () {
		const self = this;
		self.editor.main.LoadVariablesSet();

		self.$get("[flow_designer_variable_input]").each(function() { 
			self.frm.make_autocomplete(
				$(this),
				Array.from(self.editor.main.variables)
			);
		});

		$("[id*='ui-id']").css("z-index", 119);
	};

	adm_process_flow_connection_popup.prototype.__makeCondition = function(condition) {
		var self = this;
		var $condition = $(templates.condition);
		if (condition) {
			$condition.find("[logic]").val(condition.logic || "");
			$condition.find("[begin_parenths]").val(condition.begin_parenths || "");
			$condition.find("[field]").val(condition.field || "");
			$condition.find("[comparator]").val(condition.comparator || "");
			$condition.find("[field_value]").val(condition.value || "");
			$condition.find("[end_parenths]").val(condition.end_parenths || "");
		}
		$condition.find("[add]").click(function() {
			self.__disableNextKeyboardEvent();
			var $row = $(this).closest("li");
			var $newRow = self.__makeCondition();
			$row.after($newRow);
			$newRow.find("[logic]").focus();
		});
		$condition.find("[remove]").click(function() {
			self.__disableNextKeyboardEvent();
			var $row = $(this).closest("li");
			var $siblings = $row.siblings();
			if ($siblings.length) {
				if ($row.prev().length) {
					$row.prev().find("[logic]").focus();
				}
				else {
					$row.next().find("[logic]").focus();
				}
			}
			else {
				var $newRow = self.__makeCondition();
				$row.after($newRow);
				$newRow.find("[logic]").focus();
			}
			$row.remove();
		});
		return $condition;
	};

	adm_process_flow_connection_popup.prototype.__makeConditions = function() {
		var self = this;
		var conditions = self.data.conditions || [];
		var $conditions = $(templates.conditions_container);
		var $list = [];
		$.each(conditions, function(_, condition) {
			$list.push(self.__makeCondition(condition));
		});
		if (!$list.length) {
			$list.push(self.__makeCondition());
		}
		$conditions.find("[conditions]")
			.sortable({
				cursor : "move"
			})
			.append($list);
		return $conditions;
	};

	adm_process_flow_connection_popup.prototype.__makeContent = function() { 
		const self = this;
		var $content = [];

		$content.unshift(
			this.__makeDivFlex(
			  [
				this.__makeDivFlex(
				  [
					this.__makeConditions()
				  ],
				  { class: "card-block" }
				),
			  ],
			  { class: "card no-card" }
			)
		  );

		window.setInterval(function() {
			self.__disableFirstLogicOperator();
		}, 500);

		return $content;
	};

	adm_process_flow_connection_popup.prototype.__disableFirstLogicOperator = function () {
		var self = this;
		let lastOutputId;
		self.$get("[condition]").each(function(index) {
			let outputId = $(this).attr('id');
			$(this).find("[logic]").prop('disabled', false);
			$(this).find("[remove]").prop('disabled', false);
			if (index === 0 || lastOutputId !== outputId) {
				$(this).find("[logic]").prop('disabled', 'disabled');
				$(this).find("[logic]").val("")
				$(this).find("[remove]").prop('disabled', 'disabled');
			}
			lastOutputId = outputId;
		});		
	}

	adm_process_flow_connection_popup.prototype.__getAndValidateCondition = function(index, $condition, data) {
		var condition = {
			logic : index > 0? ($condition.find("[logic]").val() || undefined) : undefined,
			begin_parenths : $condition.find("[begin_parenths]").val() || undefined,
			field : $condition.find("[field]").val() || undefined,
			comparator : $condition.find("[comparator]").val() || undefined,
			value : $condition.find("[field_value]").val() || undefined,
			end_parenths : $condition.find("[end_parenths]").val() || undefined
		};
		//empty
		if (!condition.logic && !condition.begin_parenths && !condition.field && !condition.comparator && 
			!condition.value && !condition.end_parenths) 
		{
			return 0;
		}
		//validation
		var error = null;
		if (!condition.value) {
			error = "<%=_T('Must enter a value.')%>";
		}
		if (!condition.comparator) {
			error = "<%=_T('Must enter a comparator.')%>";
		}
		if (!condition.field) {
			error = "<%=_T('Must enter a field.')%>";
		}
		if (index > 0 && !condition.logic) {
			error = "<%=_T('Must enter a logical operator.')%>";
		}
		if (error) {
			data.error = {
				message : error,
				$control : $condition
			};
		}
		data.conditions.push(condition);
		return (condition.begin_parenths || "").length - (condition.end_parenths || "").length;
	};

	adm_process_flow_connection_popup.prototype.__getAndValidateConditions = function(data) {
		var self = this;
		data.conditions = [];
		var parenths = 0;
		self.__disableFirstLogicOperator();
		self.$get("[condition]").each(function(index) {
			parenths = self.__getAndValidateCondition(index, $(this), data);
			if (!data.error && parenths !== 0) {
				data.error = {
					message : "<%=_T('Error validating parenths.')%>"
				};
			}
			parenths = 0;
			if (data.error) {
				return false;
			}
		});
		// if (!data.error && parenths !== 0) {
		// 	data.error = {
		// 		message : "<%=_T('Error validating parenths.')%>"
		// 	};
		// }
	};

	adm_process_flow_connection_popup.prototype.__getAndValidate = function() {
		var data = {};
		this.__addValue(data, this.descriptionDefinition);
		this.__getAndValidateConditions(data);
		return data;
	};

	adm_process_flow_connection_popup.prototype.__cancel = function(callback) {
		if (this.isNew) {
			this.editor.CancelConnection(this.connector);
		}
		callback();
	};

	adm_process_flow_connection_popup.prototype.__save = function(data, callback) {
		this.editor.AddOrUpdateConnection(this.connector, data, this.isNew, false);
		callback(true);
	};

}());
