/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double */

/* global $, inConcert, Mustache */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	var templates = inConcert.templates.process_flows;

	var groups = [
		{
			id : "interaction",
			name : "<%=_T('Conversation')%>"
		},
		{
			id : "variables",
			name : "<%=_T('Code')%>"
		},
		{
			id : "middleware",
			name : "<%=_T('Middleware')%>"
		},
		{
			id : "dialer",
			name : "<%=_T('Dialer')%>"
		}
	];

	function adm_process_flow_toolbar(main) {
		this.main = main;
	}

	if (typeof inConcert.adm_process_flow_toolbar !== "function") {
		inConcert.adm_process_flow_toolbar = adm_process_flow_toolbar;
	}

	adm_process_flow_toolbar.prototype.__loadActivities = function() {
		var self = this;
		self.activities = {};
		var activitiesByGroup = {};

		$.each(inConcert.adm_process_flow_activities, function(id, activity) {
			if (id === "base") {
				return;
			}
			activity = activity.prototype.GetActivity(self.main.data.Type);
			if (activity.prototype.channels && !activity.prototype.channels[self.main.data.Type]) {
				return;
			}
			if (activity.prototype.available && !activity.prototype.available()) {
				return;
			}
			self.activities[id] = activity;
			activitiesByGroup[activity.prototype.group] = activitiesByGroup[activity.prototype.group] || [];
			if (activity.prototype.deprecated) {
				return;
			}
			activitiesByGroup[activity.prototype.group].push(activity);
		});
		
		var activitiesByGroup_a = [];
		$.each(groups, function(_, group) {
			if (!activitiesByGroup[group.id]) {
				return;
			}
			activitiesByGroup_a.push({
				group : group,
				activities : activitiesByGroup[group.id]
			});
		});
		return activitiesByGroup_a;
	};

	adm_process_flow_toolbar.prototype.__makeActivity = function(activity) {
		var self = this;
		var $activity = $(Mustache.render(templates.toolbar_activity, activity.prototype));
		$activity.click(function() {
			self.main.NewActivity(self.activities[$(this).attr("toolbar_activity")]);
		});
		return $activity;
	};

	adm_process_flow_toolbar.prototype.__makeGroup = function(group) {
		var self = this;
		var $group = $(Mustache.render(templates.toolbar_category, group.group));
		$group.click(function(e) {
			var $target = $(e.target);
			if (!$target.attr("category_selector")) {
				return;
			}
			var $titleDiv = $(this).children("[category_selector_title]");
			var minHeight = $titleDiv.offset().top + $titleDiv.height() + 15;
			if (minHeight < e.pageY) {
				return;
			}
				
			$(this)
				.toggleClass("selected")
				.siblings("[toolbar_category]")
				.removeClass("selected");
			$(this)
				.parent()
				.scrollTo($(this));
		});
		var $activities = [];
		group.activities.sort(function(a, b) {
			return a.prototype.name.localeCompare(b.prototype.name);
		});
		$.each(group.activities, function(_, activity) {
			$activities.push(self.__makeActivity(activity));
		});
		$group.find("[activities_list]").append($activities);
		return $group;
	};

	adm_process_flow_toolbar.prototype.Make = function() {
		var self = this;
		var $groups = [];
		$.each(self.__loadActivities(), function(_, group) {
			$groups.push(self.__makeGroup(group));
		});
		return $groups;
	};

}());
