/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double */

/* global $, inConcert, Mustache, _ */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	var format = inConcert.Utils.format;
	var templates = inConcert.templates.process_flows;
	
	var log_level = [
		"DEBUG", 
		"INFO", 
		"WARNING", 
		"ERROR"
	];
	
	var available_logs = {};
	$.each(log_level, function(index, level) {
		available_logs[level] = {};
		$.each(log_level, function(anotherIndex, anotherLevel) {
			available_logs[level][anotherLevel] = anotherIndex >= index;
		});
	});

	function adm_process_flow_trace(main) {
		this.main = main;
	}

	if (typeof inConcert.adm_process_flow_trace !== "function") {
		inConcert.adm_process_flow_trace = adm_process_flow_trace;
	}

	adm_process_flow_trace.prototype.Make = function($container, $bottomBar) {
		var self = this;
		var $body = $(templates.trace);
		var drawType = self.main.__getDrawType();

		self.SetBottomBar($bottomBar);

		self.SetStartTestFlag(false);
		self.SetOpenTestFlag(false);
		
		$container.resizable({
			handles: "n",
			minHeight: 50,
			maxHeight: $("#inboxMainContainerTop").height() - 50
		});
		
		$container.find("[close_button]").click(function() {
			self.Close();
		});
		
		if (drawType.toLowerCase() !== "chat") {
			$bottomBar.find("#flowDesignerShowEventTrace").hide();
		}

		$bottomBar.find("#flowDesignerShowTrace").click(function() {
			self.__toggle();
		});

		$bottomBar.find("#flowDesignerShowEventTrace").click(function() {
			$("#flowDesignerShowEventTrace").css("pointer-events", "none");
			self.TestStart();
		});
		
		$body.find("#flowDesignerTraceMinimize").click(function() {
			self.__hide();
		});
		
		$body.find("#flowDesignerTraceRefresh").click(function() {
			self.__refresh();
		});
		
		$body.find("#flowDesignerTraceDownload").click(function() {
			self.__download();
		});
		
		$body.find("#flowDesignerTraceInteractions").change(function() {
			self.__showTrace();
		});
		
		$body.find("#flowDesignerTraceStates").change(function() {
			self.__showTrace();
		});
		
		var options = "";
		$.each(log_level, function(_, level) {
			options += format("<option value='{0}'>{0}</option>", level);
		});
		$body.find("#flowDesignerTraceLogLevel")
			.html(options)
			.change(function() {
				self.__showTrace();
			});
		
		$body.find("#flowDesignerTraceText")
			.on("paste", function() {
				self.__showTrace();
			})
			.on("keyup", function() {
				self.__showTrace();
			});
		
		return $body;
	};

	adm_process_flow_trace.prototype.SetBottomBar = function(bottomBar) {
		this.bottomBar = bottomBar;
	};

	adm_process_flow_trace.prototype.GetBottomBar = function() {
		return this.bottomBar;
	};

	adm_process_flow_trace.prototype.SetIntervalID = function(intervalID) {
		this.intervalID = intervalID;
	};

	adm_process_flow_trace.prototype.GetIntervalID = function() {
		return this.intervalID;
	};

	adm_process_flow_trace.prototype.GetStartTestFlag = function() {
		return this.start;
	};

	adm_process_flow_trace.prototype.SetStartTestFlag = function(bool) {
		return this.start = bool;
	};

	adm_process_flow_trace.prototype.GetOpenTestFlag = function() {
		return this.open;
	};

	adm_process_flow_trace.prototype.SetOpenTestFlag = function(bool) {
		return this.open = bool;
	};

	adm_process_flow_trace.prototype.GetTestSession = function() {
		return this.sessionId;
	};

	adm_process_flow_trace.prototype.SetTestSession = function(session) {
		return this.sessionId = session;
	};

	adm_process_flow_trace.prototype.GetChatURL = function() {
		return this.chatURL;
	};

	adm_process_flow_trace.prototype.SetChatURL = function(url) {
		return this.chatURL = url;
	};

	adm_process_flow_trace.prototype.GetTestVariables = function() {
		return this.variables;
	};

	adm_process_flow_trace.prototype.SetTestVariables = function(variables) {
		return this.variables = variables;
	};

	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////// 
	
	adm_process_flow_trace.prototype.AppendVariableMarkup = function(session,drawVersion) {
		var self = this;
		var variable_markup = $(Mustache.render(templates.add_variable_markup, {}));
		let dontShowVariableForm =  self.main.getCookieValue("dontShowVariablesForm");
		let chatIframe = $("#chat_test_iframe_frame");
		
		if (chatIframe) chatIframe.hide();

		 if (dontShowVariableForm === "true") {
			self.__testStartActions(drawVersion, session);
		} else {
			$("#flowDesignerContainer").find("#flowDesignerDrawContainer").append(variable_markup);
	
			$("#flowDesignerContainer").find("[no_start_test]").click(function(){
				//INICIAR
				document.cookie = "dontShowVariablesForm=" + $("#flow_designer_variables_not_show").is(":checked");
				self.__testStartActions(drawVersion, session);
				$("#flowDesignerContainer").find("[add_variables_message_container]").hide();
			});
	
			$("#flowDesignerContainer").find("[add_variable]").click(function(){
				document.cookie = "dontShowVariablesForm=" + $("#flow_designer_variables_not_show").is(":checked");
				self.AppendVariableForm(session, drawVersion, chatIframe);
				$("#flowDesignerContainer").find("[add_variables_message_container]").hide();
			});

			$("#flowDesignerContainer").find("[cancel_test]").click(function(){
                variablesCount = 1;
                $("#flowDesignerContainer").find("[add_variables_message_container]").hide();
                $("#flowDesignerShowEventTrace").css("pointer-events", "");
                $("#flowDesignerContainer").find("#flowDesignerToolBar").removeClass("hidden");
                self.main.SetRunningTest(false);
                self.main.__set_draw_offset();
                self.__ToggleEventsTracePooling(session);
            });
		}
	};
	
	adm_process_flow_trace.prototype.DeleteVariableFromForm = function(buttonId) { 
		var id = buttonId.slice(30, buttonId.length);
		var elem = $(`#variable_card_${id}`);
		elem.remove();
	};
	
	adm_process_flow_trace.prototype.SaveVariablesFromForm = function() { 
		let formFields = $(".form-control");
		let visibleFields = [];
		let variableArray = [];
		let count = 1;
		let type;
		let name;
		let value;

		$.each(formFields, function(){ 
            if($(this).is(":visible")) {
                visibleFields.push($(this));
            }
		});

		$.each(visibleFields, function(index , field) {
			if (!field) {
				return;
			}
			let fieldId = visibleFields[index][0].id;
			let fieldValue = visibleFields[index][0].value;

				if (fieldId.includes("type")) {
					type = fieldValue.toLowerCase();
				} else if (fieldId.includes("name")) {
					name = fieldValue.toLowerCase();
				}
				else if (fieldId.includes("value")) {
					value = fieldValue.toLowerCase();
				}
			
			
			if (((count % 3 === 0) && (type === "custom")) || ((count % 2 === 0) && (type !== "custom"))) {
				
				if (type !== "custom") name = type;
				let data = {t:type,n:name,a:name,v:value};
				variableArray.push(data);
				type = "";
				name = "";
				value = "";
				count = 0;
			}
			count++;
		});
		return variableArray;
	};	
	
	var variablesCount = 1;
	
	adm_process_flow_trace.prototype.AppendVariableForm = function(session, drawVersion, chatIframe) {
		var self = this;
		var variableTypes = "<option value='ID'><%=_T('Id')%></option>"+
							"<option value='NAME'><%=_T('Name')%></option>"+
							"<option value='PHONE'><%=_T('Phone')%></option>"+
							"<option value='EMAIL'><%=_T('E-mail')%></option>"+
							"<option value='CUSTOM'><%=_T('Custom')%></option>";
		var add_variable_form = $(Mustache.render(templates.add_variable_form, {}));
		
		$("#flowDesignerContainer").find("#flowDesignerDrawContainer").append(add_variable_form);

		// AGREGAR VARIABLE
		$("#flowDesignerContainer").find("[add_variable_cards]").append(
			$(Mustache.render(templates.new_variable_card, {count: variablesCount}))
		);

		$("#flowDesignerContainer").find(`[select_variable_type_${variablesCount}]`)
			.html(variableTypes)
			.change(function() {
				let variableNumber = this.getAttribute("varialbeNumber");
				if (this.value === "CUSTOM") {
					$(`#flow_designer_test_row_var_custom_name_${variableNumber}`).show();
					$(`#flow_designer_test_row_var_custom_value_${variableNumber}`).show();
				} else {
					$(`#flow_designer_test_row_var_custom_name_${variableNumber}`).hide();
					$(`#flow_designer_test_row_var_custom_value_${variableNumber}`).show(); 
				}
		});

		$(`#flow_designer_test_delete_var_${variablesCount}`).click(function(){
			self.DeleteVariableFromForm(this.id);
		});

		variablesCount++;
		

		$("#flowDesignerContainer").find("[add_new_variable]").click(function(){
			// AGREGAR VARIABLE
			$("#flowDesignerContainer").find("[add_variable_cards]").append(
				$(Mustache.render(templates.new_variable_card, {count: variablesCount}))
			);

			$("#flowDesignerContainer").find(`[select_variable_type_${variablesCount}]`)
				.html(variableTypes)
				.change(function() {
					let variableNumber = this.getAttribute("varialbeNumber");
					if (this.value === "CUSTOM") {
						$(`#flow_designer_test_row_var_custom_name_${variableNumber}`).show();
						$(`#flow_designer_test_row_var_custom_value_${variableNumber}`).show();
					} else {
						$(`#flow_designer_test_row_var_custom_name_${variableNumber}`).hide();
						$(`#flow_designer_test_row_var_custom_value_${variableNumber}`).show(); 
					}
			});
			
			$(`#flow_designer_test_delete_var_${variablesCount}`).click(function(){
				self.DeleteVariableFromForm(this.id);
			});

			variablesCount++;
		});
		

		$("#flowDesignerContainer").find("[save_variable]").click(function(){
			// GUARDAR E INICIAR
			$("#flowDesignerContainer").find("[create_variables_form]").hide();
			variablesCount = 1;
			let variables = self.SaveVariablesFromForm();
			self.SetTestVariables(variables);
			self.__testStartActions(drawVersion, session);
		});
		
		$("#flowDesignerContainer").find("[cancel_variable]").click(function(){
			variablesCount = 1;
			$("#flowDesignerContainer").find("[create_variables_form]").hide();
			$("#flowDesignerShowEventTrace").css("pointer-events", "");
			$("#flowDesignerContainer").find("#flowDesignerToolBar").removeClass("hidden");
			self.main.SetRunningTest(false);
			self.main.__set_draw_offset();
			self.__ToggleEventsTracePooling(session);
		});
	};

	adm_process_flow_trace.prototype.AppendVariableStartTest = function() {
		var add_variable_start_test = $(Mustache.render(templates.add_variable_start_test, {}));
		$("#flowDesignerContainer").find("#flowDesignerDrawContainer").append(add_variable_start_test);
		$("#flowDesignerContainer").find("[test_start_variable]").click(function(){
			$("#flowDesignerContainer").find("[test_variables_form]").hide();
		});
	};
	
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////// 

	adm_process_flow_trace.prototype.TestStart = function() {
		const start = this.GetStartTestFlag();
		if (start)
			return this.__ToggleEventsTracePooling(this.GetTestSession());
		
		// if (openTest) self.__ToggleEventsTracePooling(session);	
		// if (!openTest) self.__createTestSession();

		this.__createTestSession();	
	
		this.__toggleEventsContainer();
	};

	adm_process_flow_trace.prototype.__ToggleEventsTracePooling = function(session) {
		const self = this;
		var drawVersion = self.main.__getVersion(); 
		let start = self.GetStartTestFlag();
		start = !start;
		self.SetStartTestFlag(start);
		self.SetTestSession(session);
				
		if (start) {
			self.main.SetRunningTest(true);
			self.main.__set_draw_offset();
			self.AppendVariableMarkup(session, drawVersion);
		} else {
			self.__testStopActions(session);
		}

		$("#flowDesignerContainer").find("[chat_test_iframe_close]").click(function(){
			//STOP AND CLOSE CHAT
			self.__closeTestBot(session);
			self.main.StopReanimateNode();
			$("#chat_test_iframe_frame").remove();
			self.main.__toggleDisableStateTabs(false);
			clearInterval(self.GetIntervalID());
			self.main.SetPreviousPortOut(undefined);
			self.main.SetPortOutId(undefined);
			self.main.SetTransitionNode(undefined);
			self.main.SetStateTransitionNode(undefined);
			self.main.SetLastStateNode(undefined);
			self.main.SetTraceNodeList(undefined);
			self.main.SetTLastState(undefined);
			self.main.SetLastStateTransition(undefined);
			self.main.SetPreviousNode(undefined);
			(drawVersion === "2") ? self.main.__clearTraceEvents() : self.main.__clearTraceEventsV1();
			$("#flowDesignerContainer").find("#flowDesignerToolBar").removeClass("hidden");
			$("#flowDesignerContainer").find("#flowDesignerDrawContainer").removeClass("startTestActive");
			$("#flowDesignerShowEventTrace").html("<%=_T('Start Test')%>");
			$("#flowDesignerShowEventTrace").removeClass("selected");
			self.SetStartTestFlag(false);
			self.SetOpenTestFlag(false);
			self.main.SetRunningTest(false);
			self.main.__set_draw_offset();
		});
	};

	adm_process_flow_trace.prototype.prepareIFrame = function(chatURL) {
        var ifrm = document.createElement("iframe");
        ifrm.setAttribute("src", chatURL);
		ifrm.setAttribute("id", "chat_test_iframe_frame");
		ifrm.setAttribute("width", "100%");
		ifrm.setAttribute("height", "100%");
		ifrm.setAttribute("scrolling", "no");
		ifrm.setAttribute("allow", "autoplay;camera;microphone;display-capture;geolocation");
		$("#draw_chat_test").append(ifrm);
		return ifrm;
    }

	adm_process_flow_trace.prototype.__testStartActions = function(drawVersion, session) {
		var self = this;

		self.__closeTestBot(session);
		self.main.StopReanimateNode();
		self.main.SetStateTransitionNode(undefined);
		self.main.SetPreviousNode(undefined);
		self.main.SetLastStateNode(undefined);
		self.main.SetLastStateTransition(undefined);
		self.main.SetTraceNodeList(undefined);
		self.main.SetTLastState(undefined);
		self.main.__toggleDisableStateTabs(true);
		(drawVersion === "2") ? self.main.__clearTraceEvents() : self.main.__clearTraceEventsV1();
		(drawVersion === "2") ? self.main.__makeAllGrey() : self.main.__makeAllGreyV1();
		if (drawVersion === "2") self.main.__animateInitialEdges();
		self.SetIntervalID(window.setInterval(function() {
			self.__refreshGetBotEvents(session);
		}, 1000));
		let chatURL = self.GetChatURL();
		$("#chat_test_iframe_frame").remove();
		self.prepareIFrame(chatURL);
		$("#flowDesignerShowEventTrace").html("<%=_T('Stop Test')%>");
		$("#flowDesignerShowEventTrace").addClass("selected");
		$("#flowDesignerShowEventTrace").css("pointer-events", "none");
		$("#flowDesignerContainer").find("#flowDesignerToolBar").addClass("hidden");
		$("#flowDesignerContainer").find("#flowDesignerDrawContainer").addClass("startTestActive");
		window.setTimeout(function(){
			$("#flowDesignerShowEventTrace").css("pointer-events","");
		}, 4500);
		self.SetOpenTestFlag(true);
		self.main.SetRunningTest(true);
		self.main.__set_draw_offset();
	}

	adm_process_flow_trace.prototype.__testStopActions = function(session) {
		var self = this;

		// STOP TEST
		self.__closeTestBot(session);
		self.main.StopReanimateNode();
		$("#flowDesignerShowEventTrace").css("pointer-events", "");
		self.main.__toggleDisableStateTabs(false);
		clearInterval(self.GetIntervalID());
		self.main.SetStateTransitionNode(undefined);
		self.main.SetStateTransitionNode(undefined);
		self.main.SetPreviousPortOut(undefined);
		self.main.SetPortOutId(undefined);
		self.main.SetTransitionNode(undefined);
		self.main.SetLastStateNode(undefined);
		self.main.SetLastStateTransition(undefined);
		self.main.SetTraceNodeList(undefined);
		self.main.SetTLastState(undefined);
		self.main.SetPreviousNode(undefined);
		$("#flowDesignerContainer").find("#flowDesignerToolBar").removeClass("hidden");
		$("#flowDesignerContainer").find("#flowDesignerDrawContainer").removeClass("startTestActive");
		$("#flowDesignerShowEventTrace").html("<%=_T('Start Test')%>");
		$("#flowDesignerShowEventTrace").removeClass("selected");
		self.main.SetRunningTest(false);
		self.main.__set_draw_offset();
	}
	

	adm_process_flow_trace.prototype.__StopEventsTracePooling = function(intervalID) {
		clearInterval(intervalID);
	};
	
	adm_process_flow_trace.prototype.__toggle = function() {
		if (this.main.$element("trace_container").is(":visible")) {
			this.__hide();
		}
		else {
			this.__show();
			if (this.main.$element("event_trace_container").is(":visible")) {
				// this.__toggleEventsContainer();
			}
		}
	};

	adm_process_flow_trace.prototype.__toggleEventsContainer = function() {
		if (this.main.$element("event_trace_container").is(":visible")) {
			this.__hideEvents();
		}
		else {
			this.__showEvents();
			if (this.main.$element("trace_container").is(":visible")) {
				this.__toggle();
			}
		}
	};
	
	adm_process_flow_trace.prototype.__show = function() {
		this.main.$element("trace_container").show();
		this.main.$element("trace_container").addClass("show");
		this.main.$element("show_trace").addClass("selected");
		if (!this.trace) {
			this.__refresh();
		}
	};

	adm_process_flow_trace.prototype.__showEvents = function() {
		this.main.$element("draw_chat_test").show();

		// hide toolbar
		this.main.$element("tool_bar").hide();

		if (!this.trace) {
			this.__refresh();
		}
	};
	
	adm_process_flow_trace.prototype.__hide = function() {
		this.main.$element("trace_container").removeClass("show");
		this.main.$element("trace_container").show(1).delay(1000).hide(1);
		this.main.$element("show_trace").removeClass("selected");
	};

	adm_process_flow_trace.prototype.__hideEvents = function() {
		this.main.$element("draw_chat_test").hide();
		
		// show toolbar
		this.main.$element("tool_bar").show();
		this.main.$element("draw_container").addClass("withToolbar");
	};
	
	adm_process_flow_trace.prototype.__loading = function(loading) {
		this.loading = !!loading;
		this.main.$element("trace_loading").toggle(!!loading);
	};

	adm_process_flow_trace.prototype.__refreshGetBotEvents = function(session) {
		var self = this;
		if (self.__isLoading()) {
			return;
		}
		self.main.GetEvent(session, function(trace) {
			if (trace) {
				self.__loadEvents(trace);
			}
		});
		self.main.ReanimateNode();
	};

	adm_process_flow_trace.prototype.__createTestSession = function() {
		var self = this;
		self.main.CreateTestSession(function(success,session){
			self.SetTestSession(session);
			self.__getSandboxCampaign(session);
		});
	}

	adm_process_flow_trace.prototype.__closeTestBot = function(session) {
		var self = this;
		if (!session) return;

		try {
			self.main.CloseTestBot(session);
		} catch(err)
		{
			console.log();
		}
	}

	adm_process_flow_trace.prototype.__getSandboxCampaign = function(session) {
		var self = this;
		self.main.GetSandboxWebchatInformation(function(sandbox) {
			if (sandbox) {
				let chatURL = self.MakeChatUrl(sandbox, session);
				self.SetChatURL(chatURL);
				self.SetOpenTestFlag(true);
				self.__ToggleEventsTracePooling(session);
			}
		});
	};

	adm_process_flow_trace.prototype.MakeChatUrl = function(sandbox, session) {
		var self = this;
		let address = "";
		let paramVariables = [];
		let variables = self.GetTestVariables();
		
		paramVariables.push({ t: "custom", n: "test_bot", a: "test_bot" , v: session }); // Agrego el Bot a las variables
		
		if (variables) {
			$.each(variables, function (index, variable) {
				paramVariables.push(variable);
			});
		}
		
		
		if(document.location.protocol === "https:") {
			address = "https://" + sandbox.host + (sandbox.public_ssl_port !== 443 ? ":" + sandbox.public_ssl_port : "");
		}
		else {
			address = "http://" + sandbox.host + (sandbox.public_port !== 80 ? ":" + sandbox.public_port : "");
		}
		let chatURL = address + "/inconcert/apps/webdesigner/"+ sandbox.account.TweetyAppName + "?token="+sandbox.account.Token;
		chatURL += "&notify=9b153a09-ae1a-f9e9-4fde-9b92f9ca626d&dev=false&type=popup";
		chatURL += "&params=" + encodeURIComponent(JSON.stringify(paramVariables));
		return chatURL;
	};
	
	adm_process_flow_trace.prototype.__isLoading = function() {
		return !!this.loading;
	};
	
	adm_process_flow_trace.prototype.__refresh = function(callback) {
		callback = callback || $.noop;
		var self = this;
		if (self.__isLoading()) {
			return;
		}
		self.__loading(true);
		self.main.GetTrace(function(trace) {
			self.__loading(false);
			if (trace) {
				self.__load(trace);
			}
			callback();
		});
	};
	
	adm_process_flow_trace.prototype.__load = function(trace) {
		var self = this;
		self.trace = {};
		var options = "";
		var trace_a = [];
		$.each(trace, function(interactionId, item) {
			var events = JSON.parse(item);
			trace_a.push({
				id : interactionId,
				events : events,
				firstEvent : events[0],
				lastEvent : events[events.length - 1]
			});
		});
		trace_a.sort(function(a, b) {
			return b.firstEvent.timestamp - a.firstEvent.timestamp;
		});
		var nowDate = new Date().getDate();
		$.each(trace_a, function(_, item) {
			self.trace[item.id] = item.events;
			options += format("<option value='{0}'><%=_T('{1} to {2} ({0})')%></option>", item.id, 
			self.__makeDate(item.firstEvent, nowDate, true), self.__makeDate(item.lastEvent, nowDate, true));
		});
		self.main.$element("trace_interactions").html(options);
		self.__showTrace();
	};

	adm_process_flow_trace.prototype.__loadEvents = function(trace) {
		var self = this;
		var drawVersion = self.main.__getVersion();
		(drawVersion === "2") ? self.main.__handleEventTraceV2(trace) :  self.main.__handleEventTrace(trace);
		// self.__showTrace();
	};
	
	adm_process_flow_trace.prototype.__makeActivityLabel = function(action) {
		return format("<%=_T('Activity {0} ({1})')%>", action.label, action.activityType);
	};
	
	adm_process_flow_trace.prototype.__makeEventLabel = function(action) {
		return format("<%=_T('Event {0} to {1}')%>", action.label, action.to_label);
	};
	
	adm_process_flow_trace.prototype.__makeTransitionLabel = function(action) {
		return format("<%=_T('Transition {0} to {1}')%>", action.label, action.to_label);
	};
	
	adm_process_flow_trace.prototype.__makeActionLabel = function(action) {
		switch(action.type) {
			case "activity":
				return this.__makeActivityLabel(action);
			case "event":
				return this.__makeEventLabel(action);
			case "transition":
				return this.__makeTransitionLabel(action);
		}
	};
	
	adm_process_flow_trace.prototype.__makeText = function(item) {
		var actionLabel = this.__makeActionLabel(item.action || {});
		return (actionLabel? (actionLabel + " - ") : "") + item.message;
	};
	
	adm_process_flow_trace.prototype.__makeClassName = function(item) {
		switch(item.type) {
			case "ERROR":
				return "levelError";
			case "WARNING":
				return "levelWarning";
			case "INFO":
				return "levelInfo";
			default:
				return "levelDebug";
		}
	};
	
	adm_process_flow_trace.prototype.__makeDate = function(item, nowDate, withoutMs) {
		var date = new Date(item.timestamp * 1000);
		var date_s = inConcert.Utils.getStringFromDate(date, false, 
			date.getDate() === nowDate? "full_time" : "full_datetime");
		if (withoutMs) {
			date_s = date_s.split(".")[0];
		}
		return date_s;
	};
	
	adm_process_flow_trace.prototype.__makeItem = function(index, item, logLevel, text, nowDate) {
		var self = this;
		if (available_logs[logLevel][item.type] === false) {
			return;
		}
		var data = {
			index : index,
			text : self.__makeText(item),
			className : self.__makeClassName(item),
			date : self.__makeDate(item, nowDate),
			state : item.state || "",
			level : item.type
		};
		if (text && data.state.toLowerCase().indexOf(text) < 0 && data.date.toLowerCase().indexOf(text) < 0 && data.text.toLowerCase().indexOf(text) < 0) {
			return;
		}
				
		var $item = $(Mustache.render(templates.trace_item, data));
		
		$item
			.click(function() {
				$(this)
					.addClass("selected")
					.siblings()
					.removeClass("selected");
			})
			.dblclick(function() {
				self.__showItemContext(self.trace[self.main.$element("trace_interactions").val()]
					[$(this).attr("trace_item")]);
			});
		
		return $item;
	};
	
	adm_process_flow_trace.prototype.__showTrace = _.throttle(function() {
		var self = this;
		var $items = [];
		var interactionId = self.main.$element("trace_interactions").val();
		var logLevel = self.main.$element("trace_loglevel").val();
		var state = self.main.$element("trace_states").val();
		var text = (self.main.$element("trace_text").val() || "").toLowerCase();
		var nowDate = new Date().getDate();
		
		self.selectedTrace = interactionId && self.trace[interactionId] || [];
		var states_d = {};
		var stateOptions = "<option value=''><%=_T('All states')%></option>";
		
		$.each(self.selectedTrace, function(index, item) {
			$items.push(self.__makeItem(index, item, logLevel, text, nowDate));
			if (item.state && !states_d[item.state]) {
				states_d[item.state] = true;
				stateOptions += format("<option value='{0}'>{0}</option>", item.state);
			}
		});
	
		//filter by state
		if (state && states_d[state]) {
			$.each($items, function(index, $item) {
				if (!$item) {
					return;
				}
				if ($item.attr("trace_state") !== state) {
					$items[index] = null;
				}
				else {
					$item.find("[state_part]").remove();
				}
			});
		}
		
		self.main.$element("trace_states")
			.html(stateOptions)
			.val(state);
		
		self.main.$element("trace_list")
			.html("")
			.append($items);
	}, 100);
	
	adm_process_flow_trace.prototype.__showStateContext = function(stateId) {
		this.main.ShowState(stateId);
	};
	
	adm_process_flow_trace.prototype.__showEventContext = function(from, to) {
		this.main.ShowStateEvent(from, to);
	};
	
	adm_process_flow_trace.prototype.__showActivityContext = function(stateId, activityId) {
		this.main.ShowStateActivity(stateId, activityId);
	};
	
	adm_process_flow_trace.prototype.__showTransitionContext = function(stateId, from, to) {
		this.main.ShowStateTransition(stateId, from, to);
	};
	
	adm_process_flow_trace.prototype.__showItemContext = function(item) {
		var stateId = item.state && this.main.GetStateFromName(item.state);
		if (!stateId) {
			return;
		}
		switch(item.action && item.action.type) {
			case "activity":
				return this.__showActivityContext(stateId, item.action.id);
			case "transition":
				return this.__showTransitionContext(stateId, item.action.activity, item.action.to);
			case "event":
				return this.__showEventContext(stateId, item.action.to);
			default:
				return this.__showStateContext(stateId);
		}
	};
	
	adm_process_flow_trace.prototype.__download = function() {
		var log = "";
		this.main.$element("trace_item").each(function() {
			var level = $(this).attr("trace_level") || "";
			var date = $(this).find("[date_part]").text() || "";
			var state = $(this).find("[state_part]").text() || "";
			var text = $(this).find("[text_part]").text() || "";
			log += format("[{0}]\t{1}\t{2}\t{3}\n", level, date, state, text);
		});
		var uriContent = "data:application/octet-stream," + encodeURIComponent(log);
		var element = document.createElement("a");
		element.setAttribute("href", uriContent);
		element.setAttribute("download", this.main.data.Name + ".log");
		element.style.display = "none";
		document.body.appendChild(element);
		element.click();
		document.body.removeChild(element);
	};

}());
