local utils = require "shared.apps.utils"
local permissions = require "shared.permissions.permissions"
local helpDeskLogin = require "apps.helpdesk.login"
local m_request = require "apps.helpdesk.request"
local cjson = require "cjson"

module(..., package.seeall, inconcert.orbit.application.Decorate)

m_htmlRender = nil
m_appLocation = nil

local m_enabledUpdateUserInHelpDesk = true-- flag para cuando estamos desarrollando. (SI, esta en false, NO realiza REQUEST CONTRA HELPDESK)

------------------------------------------
--[[
	URL PARA GUARDAR USUARIOS
	------------------------
	https://168.62.161.232:2443/ichelpdeskdesarrollo/rest/wsuserocc
]]--
local m_updateUserUriTemplate = "rest/wsuserocc"

local function MakeUpdateUserSuffixURI ()
	local loggedUserId = utils.GetCurrentLoggedUser()
	LogInfo("[HELPDESK.USERS:MakeUpdateUserSuffixURI] Making Update User URL, User: '%s'\nurl base: '%s'.", loggedUserId, m_updateUserUriTemplate)
	return true, m_updateUserUriTemplate
end
------------------------------------------

local function UpdateValues (omnichannelUser, vcc)
	local fullUserId = utils.AppendVcc(omnichannelUser.Id, vcc)

	local function GetWorkgroupId ()
		local workgroupId = omnichannelUser.WorkgroupId and omnichannelUser.WorkgroupId or ""
		return workgroupId
	end

	local tempUser = {
		VCC                 = omnichannelUser.VCC,
		Id                  = omnichannelUser.Id,
		Name                = omnichannelUser.Name,
		LastName            = omnichannelUser.LastName,
		LastSurname         = omnichannelUser.LastSurname,
		FirstSurname        = omnichannelUser.FirstSurname,
		Enabled             = omnichannelUser.Status,
		isNew               = omnichannelUser.isNew,
		newUser             = omnichannelUser.newUser,
		LanguageId          = omnichannelUser.LanguageId,
		UserGroup           = omnichannelUser.UserGroup,
		VccGroup            = omnichannelUser.VccGroup,
		TimeZoneId          = omnichannelUser.TimeZoneId,
		areaCode            = omnichannelUser.areaCode,
		countryCode         = omnichannelUser.countryCode,
		WorkgroupId         = GetWorkgroupId(),
		Mail                = omnichannelUser.Mail,
		groups              = omnichannelUser.groups or {}, -- las campañas
		Password            = omnichannelUser.Password,
		HelpDeskIntegration = omnichannelUser.HelpDeskIntegration or {}
	}

	if not tempUser.newUser and not tempUser.Password then
		-- Si estoy modificando el usuario, debo obtener la Password, para que sea enviada a HelpDesk
		tempUser.Password = utils.GetColumnValue {
			keyspace     = "Agent",
			columnFamily = "Users",
			key          = fullUserId,
			column       = "Password"
		}
		if not tempUser.Password then
			return false, {err = "ERROR_RETRIEVING_USER_PASSWORD_FROM_CASSANDRA"}
		end
	end

	local keyspace = utils.GetDatasource():CassandraConnection("Agent")
	local applicationsActions = permissions.GetApplicationsActions(keyspace, vcc)

	-- si tiene acceso a las apps
	tempUser.HelpDeskIntegration.AllowedApps = {
		administration = {
			has_permission = permissions.HasPermissionToApplication("administration", fullUserId, tempUser.UserGroup, nil, keyspace, {applicationsActions = applicationsActions})
		},
		interaction = {
			has_permission = permissions.HasPermissionToApplication("interaction", fullUserId, tempUser.UserGroup, nil, keyspace, {applicationsActions = applicationsActions})
		},
		supervisor = {
			has_permission = permissions.HasPermissionToApplication("supervisor", fullUserId, tempUser.UserGroup, nil, keyspace, {applicationsActions = applicationsActions})
		},
		helpdesk_tickets = {
			has_permission = permissions.IsPermissionEnabledForSpecificUser("apps_integrations", "helpdesk_tickets_in_app_interaction", fullUserId, tempUser.UserGroup, omnichannelUser.custom_permissions, nil, nil, keyspace, applicationsActions)
		},
		helpdesk_full_app = {
			has_permission = permissions.IsPermissionEnabledForSpecificUser("apps_integrations", "helpdesk_full_access", fullUserId, tempUser.UserGroup, omnichannelUser.custom_permissions, nil, nil, keyspace, applicationsActions)
		}
	}

	keyspace:Close()

	return true, tempUser
end

--[[
	Guarda un usuario en HelpDesk
]]--

function Save (user, vccId)
	assert(type(user) == "table")

	local enabled, vccHelpDeskConfig = utils.HasHelpDeskIntegration(vccId)
	if not enabled then
		LogError("[HELPDESK.USERS.Save] Unable saving HelpDesk user. VCC '%s' Without integration", vccId)
		return true -- no es un error
	end

	local omnichannelUser = utils.DeepCopy(user)
	local fullUserId = utils.AppendVcc(omnichannelUser.Id, vccId)

	local function ProcesHelpdeskError(requestError)
		local info = requestError and ((requestError.response or requestError.Error) or "{}")
		local full_sub_type = utils.JSON_Decode(info)
		local errorMessage = full_sub_type and ((full_sub_type.error and full_sub_type.error.message) or full_sub_type.Description)
		return errorMessage
	end

	local success, response = UpdateValues(omnichannelUser, vccId)
	if not success then
		local errType = response
		local err = {type = "HELPDESK_INTEGRATION_ERROR", sub_type = ProcesHelpdeskError(errType)}
		LogWarning("[HELPDESK.USERS:Save] Error saving HelpDesk user: %s. Err: %s.", fullUserId, cjson.encode(err))
		return false, err
	end

	omnichannelUser = response
--local encodedOmnichannelUser = cjson.encode(omnichannelUser)
--LogDebug("[HELPDESK.USERS:Save] ENCODED user: %s", encodedOmnichannelUser)

	if not m_enabledUpdateUserInHelpDesk then
	-- Esto es por si estamos desarrollando y no queremos que actualice el mismo en HELPDESK.
	-- TODO: cambiar, poner algo en un flag en REDIS o en algún lugar.
		LogInfo("[HELPDESK.USERS:Save] Update users in HELPDESK is disabled.")
		return true
	end

	local success, suffixURI = MakeUpdateUserSuffixURI()
	if not success then
		local errType = suffixURI
		local err = { type = "HELPDESK_INTEGRATION_ERROR", sub_type = ProcesHelpdeskError(errType) }
		LogWarning("[HELPDESK.USERS:Save] Error saving HelpDesk user: %s. Err: %s.", fullUserId, cjson.encode(err))
		return false, err
	end

	local saveSuccess, saveResponse
	local try = 0

	local function SaveInHelpDesk ()
		try = try + 1
		local noGetFromCache = try > 1

		LogInfo("[HELPDESK.USERS.Save] Saving HelpDesk User '%s'. Try: '%d'.", fullUserId, try)

		local success, jwtUserToken = helpDeskLogin.GetJWTUserToken(vccHelpDeskConfig, noGetFromCache, vccId)
		if not success then
			local err = jwtUserToken
			LogError("[HELPDESK.USERS:Save] Error Retrieving JWT Token. User: '%s' Err: '%s'.", fullUserId, err)
			saveSuccess = false
			saveResponse = {type = "HELPDESK_INTEGRATION_ERROR", sub_type = ProcesHelpdeskError(err)}
			return
		end

		local headers = m_request.GetHeaderOAuthAuthorization(jwtUserToken)
		local helpdeskSuccess, helpdeskResponse = m_request.Post(
			suffixURI, -- la suffix URL de HelpDesk
			omnichannelUser,
			nil,
			vccHelpDeskConfig,
			headers
		)

		saveResponse = helpdeskResponse
		--LogDebug("[HELPDESK.USERS:Save]-----> success '%s', helpdeskResponse.Success '%s'", success, helpdeskResponse.Success)

		if not helpdeskSuccess or not helpdeskResponse.Success then
			saveSuccess = false
			saveResponse = {type = "HELPDESK_INTEGRATION_ERROR", sub_type = ProcesHelpdeskError(helpdeskResponse)}
			LogWarning("[HELPDESK.USERS:Save] Error saving HelpDesk user: %s. Err: '%s'", fullUserId, cjson.encode(saveResponse))
			--LogDebug("[HELPDESK.USERS:Save] helpdeskResponse.response: '%s'", helpdeskResponse.response)
			if helpdeskResponse.response then
				local helpdeskErr = type(helpdeskResponse.response) == "string" and json.decode(helpdeskResponse.response) or helpdeskResponse.response
				--LogDebug("[HELPDESK.USERS:Save] helpdeskErr: '%s'", cjson.encode(helpdeskErr))
				if helpdeskErr and helpdeskErr.error and tonumber(helpdeskErr.error.code) == 112 then
					if try == 1 then
						SaveInHelpDesk()
					end
				end
			end
		else
			saveSuccess = true
			LogInfo("[HELPDESK.USERS.Save] Success Saved HelpDesk User '%s'.", fullUserId)
		end
	end

	SaveInHelpDesk()

	return saveSuccess, saveResponse
end

function MapUris (module, htmlRender, prefix, appLocation)
	local p = prefix
	m_htmlRender = htmlRender
	m_appLocation = appLocation
end
