require "inconcert.translators"
local lua_cassandra = require "lua_cassandra"
require "redis"
local WrapRedis = require "wrapredis"
local utils = require "shared.apps.utils"
local m_moduleLocation = select(2, ...)

module(..., package.seeall)

local m_dataSource
local m_models = {}
local m_loaded = false

--///////////////////////////////////////////////////////////////////////////////////
--

--recupera la lista de modelos del directorio models
local function GetModelsFromFilesystem ()
	local modelsDir = m_moduleLocation .. "/models"
	if not utils.isDirectory(modelsDir) then
		return {}
	end
	local models = {}
	for file in lfs.dir(modelsDir) do
		if file ~= "." and file ~= ".." then
			local f = modelsDir..'/'..file
			local attr = lfs.attributes (f)
			if attr.mode ~= "directory" then
				local modelName = string.match(file, [[(.+).lua]])
				if (modelName) then
					models[modelName] = modelName
				end
			end
		end
	end
	return models
end

--///////////////////////////////////////////////////////////////////////////////////
-- arma la tabla de modelos definidos
function CreateModelTable (force)
	--LogInfo("[HELPDESK.MODEL:CreateModelTable] Creating cassandra models")

	local datasource = GetDatasource()
	if force or not m_loaded then
		for key, controllerDef in pairs(GetModelsFromFilesystem()) do
			local model = require("apps.helpdesk.models." .. key)
			if model.GetDefinition then
				m_models[model.GetName()] = model.GetDefinition(m_dataSource)
			end
		end
		m_loaded = true
	else
		--if model modules were already loaded just overwrite their inner datasource
		--with the newly changed one
		for modelName, model in pairs(m_models) do
			model.GetDefinition(m_dataSource)
		end
	end
end

--///////////////////////////////////////////////////////////////////////////////////
-- 
function GetModels ()
	return m_models
end

local function InitDatasource (force)
	local redisConfig = utils.GetConnectionConfig("redis", "helpdesk")
	if not redisConfig then
		error("+Unable to initialize helpdesk app, connection to redis failed")
	end
	LogInfo("[HELPDESK.MODEL:InitDatasource] Connecting to redis on host: '%s', port:'%s'.", redisConfig.host, redisConfig.port)
	--start in nonblocking mode
	local redis = WrapRedis(redisConfig.host, redisConfig.port, redisConfig.pwd)
	if not redis then
		error("-Unable to initialize helpdesk app, connection to redis failed")
	end
	--init local datasource table
	m_dataSource = {
		cassandra = lua_cassandra,
		redis = redis
	}
	return true
end

--///////////////////////////////////////////////////////////////////////////////////
-- 
function BuildModels (force)
	InitDatasource(force)
	utils.SetupConnections(m_dataSource)
	CreateModelTable(force)
	LogInfo("[HELPDESK.MODEL:BuildModels] model built")
	return true
end

function GetDatasource ()
	if not m_dataSource then
		InitDatasource()
	end
	return m_dataSource
end

function GetRedisClient ()
	return assert(m_dataSource.redis)
end
