
local m_moduleLocation = select(2, ...)
local _, m_widgetId = m_moduleLocation:match("(.*/)(.*)") -- sample
local m_models = require("apps.integrations.model"):GetModels(m_widgetId)
module(..., package.seeall, inconcert.orbit.application.Decorate)
local utils = require "shared.apps.utils"
m_htmlRender = nil
m_appLocation = nil
local m_modelName = "SampleOther"

function GetMapUris ()
	--LogDebug("[INTEGRATIONS.WIDGETS.SAMPLE.OTHERS:GetMapUris] -------------------------------------------------------------------------------------> ")
	local definition = {
		{url = "/get/(.+)" , methodType = "GET" , fn = Get    },
		{url = "/list_all/", methodType = "POST", fn = ListAll},
		{url = "/save/"    , methodType = "POST", fn = Save   },
		{url = "/delete/"  , methodType = "POST", fn = Delete }
	}
	return definition
end

local function GetModel (entityName)
	--LogDebug("[INTEGRATIONS.SAMPLE.OTHERS:GetModel]---> entityName %s", entityName)
	assert(type(entityName) == "string")
	if not m_models or not m_models[entityName] then
		local model = require "apps.integrations.model"
		model.CreateModelTable()
		m_models = model:GetModels(m_widgetId)
	end
	local entityModel = m_models[entityName]
	assert(type(entityModel) == "table")
	return entityModel
end

function Get (web, otherId)
	local vcc = inconcert.authUser.virtualContactCenter
	LogInfo("[INTEGRATIONS.SAMPLE.OTHERS:Get] ----> Retrieving otherId %s, user %s, vcc %s", otherId, (inconcert.authUser.userName.."@"..vcc), vcc)

	local keyspace = utils.GetDatasource():CassandraConnection("Integrations")
	local user = GetModel(m_modelName).Get(vcc, otherId, keyspace)
	keyspace:Close()

	if utils.TableIsEmpty(user) then
		web.status = 404
		return json.encode({result = "NOT_FOUND"})
	end

	user.Password = nil
	return json.encode(user)
end

function ListAll (web)
	local postData = web.request
	LogInfo("[INTEGRATIONS.SAMPLE.OTHERS.ListAll] ---> List Others -  posted data: '%s'", web.input["post_data"])
	local keyspace = utils.GetDatasource():CassandraConnection("Integrations")
	local users = GetModel(m_modelName).List(keyspace)
	keyspace:Close()
	local function CanAdd (user)
		if utils.TableIsEmpty(user) then
			return false
		end
		return true
	end
	local filteredUsers = {}
	for _, user in pairs(users or {}) do
		if CanAdd(user) then
			table.insert(filteredUsers, user)
		end
	end
	LogInfo("[INTEGRATIONS.SAMPLE.OTHERS.ListAll] Others %s", json.encode(filteredUsers))
	return filteredUsers
end

function Save (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName .. "@" .. vcc
	local ok, user = pcall(json.decode, web.input["post_data"])

	LogInfo("[INTEGRATIONS.SAMPLE.OTHERS.SAVE] ---> Saving Other - user '%s', vcc '%s'. posted data: '%s'", userId, vcc, web.input["post_data"])

	if (user.Id or ""):len() == 0 then
		web.status = 400
		local err = {result = "INVALID_DATA", type = "MSSING FIELD ID"}
		LogError("[INTEGRATIONS.SAMPLE.OTHERS.SAVE] ERROR Saving Other - user '%s', vcc '%s'. err: '%s'", userId, vcc, json.encode(err))
		return err
	end
	if (user.Name or ""):len() == 0 then
		web.status = 400
		local err = {result = "INVALID_DATA", type = "MSSING FIELD NAME"}
		LogError("[INTEGRATIONS.SAMPLE.OTHERS.SAVE] ERROR Saving Other - user '%s', vcc '%s'. err: '%s'", userId, vcc, json.encode(err))
		return err
	end

	user.Vcc = vcc
	local keyspace = utils.GetDatasource():CassandraConnection("Integrations")
	GetModel(m_modelName).Save(vcc, user, keyspace)
	keyspace:Close()
	LogInfo("[INTEGRATIONS.SAMPLE.OTHERS.SAVE] SUCCESS Saved Other - user '%s', vcc '%s'.", userId, vcc)

	return {result = "OK"}
end

function Delete (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = inconcert.authUser.userName .. "@" .. vcc
	local ok, user = pcall(json.decode, web.input["post_data"])

	LogInfo("[INTEGRATIONS.SAMPLE.OTHERS.Delete] ---> Deleting Other - user '%s'", web.input["post_data"])

	if (user.Id or ""):len() == 0 then
		web.status = 400
		local err = {result = "INVALID_DATA", type = "MSSING FIELD ID"}
		LogError("[INTEGRATIONS.SAMPLE.OTHERS.Delete] ERROR Deleting Other - err: '%s'", json.encode(err))
		return err
	end

	local keyspace = utils.GetDatasource():CassandraConnection("Integrations")
	local currentUser = GetModel(m_modelName).Get(vcc, user.Id, keyspace)
	if utils.TableIsEmpty(currentUser) then
		keyspace:Close()
		web.status = 404
		local err = {result = "NOT_FOUND", type = "USER NOT FOUND"}
		LogError("[INTEGRATIONS.SAMPLE.OTHERS.Delete] ERROR Deleting Other - err: '%s'", json.encode(err))
		return err
	end

	local fullUserId = utils.AppendVcc(user.Id, vcc)
	GetModel(m_modelName).Remove(fullUserId, keyspace)
	keyspace:Close()
	LogInfo("[INTEGRATIONS.SAMPLE.OTHERS.Delete] SUCCESS Deleted Other - user '%s'.", fullUserId)

	return {result = "OK"}
end
