local lua_cassandra = require "lua_cassandra"
require "redis"
local WrapRedis = require "wrapredis"
local utils = require "shared.apps.utils"
local m_moduleLocation = select(2, ...)

module(..., package.seeall)

local m_dataSource
local m_models = {}
local m_loaded = false

function GetRedisClient ()
	return assert(m_dataSource.redis)
end

local function GetWidgetsDirectories (widgetDirectoryBase)
	--LogDebug("[INTEGRATIONS.MODEL:GetWidgetsDirectories] widgetDirectoryBase: %s", widgetDirectoryBase)
	local directories = utils.GetDirectories(widgetDirectoryBase)
	local widgetsDirectories = {}
	for _, directory in ipairs(directories) do
		directory = directory:lower()
		widgetsDirectories[directory] = {url = directory}
	end
	--LogDebug("[INTEGRATIONS.MODEL:GetWidgetsDirectories] widgetsDirectories: %s", json.encode(widgetsDirectories))
	return widgetsDirectories
end

--recupera la lista de modelos del directorio models
local function LoadModelsFromFilesystem (modelsDir, models)
	--LogDebug("[INTEGRATIONS.MODEL:LoadModelsFromFilesystem] modelsDir: %s", modelsDir)
	local files = utils.GetFiles(modelsDir)
	for _, file in ipairs(files) do
		local modelName = string.match(file, [[(.+).lua]])
		--LogDebug("[INTEGRATIONS.MODEL:LoadModelsFromFilesystem] --==========> modelName: %s", modelName)
		if modelName then
			models[modelName] = modelName
		end
	end
end

--///////////////////////////////////////////////////////////////////////////////////
-- arma la tabla de modelos definidos
function CreateModelTable ()
	--LogDebug("[INTEGRATIONS.MODEL:CreateModelTable] ===> Creating INTEGRATIONS Model.")
	if not m_loaded then
		local function CreateModel (modelsDir, currentModels, moduleLocation)
			--LogDebug("[INTEGRATIONS.MODEL:CreateModelTable] ======> modelsDir: %s, modelsDir: %s", modelsDir, moduleLocation)
			local models = {}
			LoadModelsFromFilesystem(modelsDir, models)
			for key, controllerDef in pairs(models) do
				local requireModuleName = moduleLocation .. "." .. key
				--LogDebug("[INTEGRATIONS.MODEL:CreateModelTable]       -- requireModuleName: %s", requireModuleName)
				local model = require(requireModuleName) --require("apps.integrations.models." .. key)
				local modelName = model.GetName()
				--LogDebug("[INTEGRATIONS.MODEL:CreateModelTable]         -- modelName: %s", modelName)
				if model.GetDefinition then
					currentModels[modelName] = model.GetDefinition(m_dataSource)
				else
					LogWarning("[INTEGRATIONS.MODEL:CreateModelTable]     ERROR  CreateModel--==========> modelName: %s, model.GetDefinition: %s", modelName, model.GetDefinition)
				end
			end
		end

		-- 1 - cargo el modelo de la app "Integrations"
		local integrationsModelsDir = m_moduleLocation .. "/models"
		local moduleLocation = "apps.integrations.models"
		CreateModel(integrationsModelsDir, m_models, moduleLocation)

		--2 cargo el modelo de cada uno de los widgets
		local widgetsBaseDir = integrationsModelsDir .. "/widgets"
		local widgetsModelBasePath = moduleLocation .. ".widgets"
		local widgetsDirectories = GetWidgetsDirectories(widgetsBaseDir)

		for widgetDir, info in pairs(widgetsDirectories or {}) do
			widgetDir = widgetDir:lower()
			local fullWidgetDir = widgetsBaseDir .. "/" .. widgetDir
			local widgetsModelPath = widgetsModelBasePath .. "." .. widgetDir
			m_models[widgetDir] = {}
			CreateModel(fullWidgetDir, m_models[widgetDir], widgetsModelPath)
		end

		m_loaded = true
	else
		--if model modules were already loaded just overwrite their inner datasource
		--with the newly changed one
		local ReloadModel
		Reload = function (currentModels) 
			for modelName, currentModel in pairs(currentModels) do
				--LogDebug("[INTEGRATIONS.MODEL:CreateModelTable]     --> modelName: %s, model: %s", modelName, currentModel)
				if type(currentModel) == "table" then
					--Reload(currentModel)
				elseif type(currentModel) == "function" and modelName == "GetDefinition"  then
					--model.GetDefinition(m_dataSource)
					currentModel(m_dataSource)
				end
			end
		end
		Reload(m_models)
	end
end
--///////////////////////////////////////////////////////////////////////////////////
-- 
function GetModels (self, widgetId)
	if type(widgetId) == "string" then
		-- specific widget model
		return m_models[widgetId:lower()]
	end
	-- all widgets model
	return m_models
end

--///////////////////////////////////////////////////////////////////////////////////
-- 
function BuildModels (self)
	local redisConfig = utils.GetConnectionConfig("redis", "integrations")
	if not redisConfig then
		error("+Unable to initialize integrations app, connection to redis failed")
	end
	LogInfo("[INTEGRATIONS.MODEL:BuildModels] Connecting to redis on host: '%s', port:'%s'.", redisConfig.host, redisConfig.port)
	--start in nonblocking mode
	local redis = WrapRedis(redisConfig.host, redisConfig.port, redisConfig.pwd)
	if not redis then
		error("-Unable to initialize integrations app, connection to redis failed")
	end
	--init local datasource table
	m_dataSource = {
		cassandra = lua_cassandra,
		redis = redis
	}
	utils.SetupConnections(m_dataSource)
	CreateModelTable()
	LogInfo("Integrations model built")
	return true
end
