module(..., package.seeall)

local utils = require "shared.apps.utils"
local m_datasource
local m_helpers = require "helpers"

function GetName ()
	return "VCCs"
end

function GetDefinition (datasource)
	m_datasource = datasource
	return _M
end

local function Get (vcc, keyspace, columns)
	assert(vcc)
	local vccInfo = utils.GetAllColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "VCCs",
		key = vcc,
		columns = columns
	} or {}
	if m_helpers.utils.HasLicenseSystem() then
		local options = utils.Get_m_helpers_agentservice_options()
		vccInfo = m_helpers.agentservice.GetVccWithLicenseInfo(options, vccInfo)
	end
	return vccInfo
end

local function GetAppsIntegrations (vcc, keyspace)
	assert(vcc)
	local vccInfo = Get(vcc, keyspace)
	if not vccInfo then
		return false
	end
	return vccInfo.AppsIntegrations
end

function GetWidgetConfig (vcc, widgetId, keyspace)
	assert(vcc)
	assert(widgetId)
	widgetId = widgetId:lower()
	local vccInfo = Get(vcc, keyspace)
	--LogDebug("[[INTEGRATIONS:integrations.models.vccs] 1 - vcc: '%s', widget: '%s', vccInfo: %s", vcc, widgetId, vccInfo)
	if not vccInfo then
		return false
	end
	--LogDebug("[[INTEGRATIONS:integrations.models.vccs] 2 - vcc: '%s', widget: '%s', AppsIntegrations: %s", vcc, widgetId, vccInfo.AppsIntegrations)
	if type(vccInfo.AppsIntegrations) ~= "table" then
		return false
	end
	--LogDebug("[[INTEGRATIONS:integrations.models.vccs] 3 - vcc: '%s', widget: '%s', AppsIntegrations: %s", vcc, widgetId, json.encode(vccInfo.AppsIntegrations))
	local widgetConfig = vccInfo.AppsIntegrations[widgetId]
	if not widgetConfig then
		return false
	end
	--LogDebug("[[INTEGRATIONS:integrations.models.vccs] 4 - vcc: '%s', widget: '%s', widgetConfig: %s", vcc, widgetId, json.encode(widgetConfig))
	if tonumber(widgetConfig.IsWidget or "0") ~= 1 then
		return false
	end
	-- ok
	return widgetConfig
end
