--[[
	Este archivo deberia sufrir muy pocos cambios entre los diferentes Widgets.
	Lo que se debe modificar siempre es el atributo "displayName".
	Si necesitamos incluir algun JavaScript o CSS extra en los archivos del Widget (iFrame)
]]--

local m_moduleLocation = select(2, ...)
local _, m_widgetId = m_moduleLocation:match("(.*/)(.*)") -- sample
local utils = require "shared.apps.utils"
require "cosmo"

module(..., package.seeall)

local _T, _L = utils.MakeTextTranslator("integrations")
local params = {WIDGET_DIRECTORY = m_widgetId, L_NUMBER = _L}

local definitions = {
	--[[
		status: (PRODUCTION o DEVELOPMENT)
			Este atributo acepta 2 valores.
				PRODUCTION: Si asignamos este valor el Widget quedara disponible para su utilizacion.

				DEVELOPMENT: Si asignamos este valor el Widget NO quedara disponible para su utilizacion,
					con la excepcion de que si en la variable de entorno INCONCERT_ENV_MODE asignamos este valor, todos los widgets estarán disponibles.
					Esta variables nos sirve para ir desarrollando.
	]]--
	status = "DEVELOPMENT",
	name = "$WIDGET_DIRECTORY",
	displayName = _T("Sample"), -- valor que se visualizara en las diferentes configuracions de OCC.
	--[[
		iFrame:
			Definicion de los archivos CSS y JavaScript que se han de insertar en la pagina del Widget.
	]]--
	html = {
		head = {
			css = cosmo.fill([[
				<link rel="stylesheet" type="text/css" href="/inconcert/apps/integrations/css/widgets/$WIDGET_DIRECTORY/main.css" />
			]], {WIDGET_DIRECTORY = m_widgetId}),
			js = cosmo.fill([[
				<script type="text/javascript" src="/inconcert/apps/integrations/scripts/widgets/$WIDGET_DIRECTORY/main.$L_NUMBER.js"></script>
				<script type="text/javascript" src="/inconcert/apps/integrations/scripts/widgets/$WIDGET_DIRECTORY/api.$L_NUMBER.js"></script>
			]], params)
		}
	},
	--[[
		Provisioning:
			Definicion del HTML que se va a renderizar en la App Provisioning/Integrations, bajo la configuracion del Widget.
			Tambien se define el include del archivo JavaScript que se ralizara en el inboxMaster.html de la App "provisioning"
	]]--
	provisioning = {
		html = {
			body = {
				vcc = {
					content = "",
					fileName = "provisioningVccWidget.html" -- nombre del html
				}
			},
			-- nombre del js
			js = cosmo.fill([[
				<script type="text/javascript" src="/inconcert/apps/integrations/scripts/widgets/$WIDGET_DIRECTORY/provisioningVccWidget.$L_NUMBER.js"></script>
			]], params)
		},
		--[[
			Nombre de la clase que quedara expuesta y que sera instanciada en las clases de la app "provisionig-vcc-basic"
			El nombre de esta clase tiene que coincidir con el nombre que se definira en el archivo "provisioningVccWidget".

			Ejemplo del archivo "provisioningVccWidget.js":
				..
				if ( typeof inConcert.prov_form_vccs_widget_sample !== "function" ) {
					inConcert.prov_form_vccs_widget_sample = widget_sample;
				}
				..
		]]--
		jsClassName = cosmo.fill([[inConcert.prov_form_vccs_widget_$WIDGET_DIRECTORY]], params) --inConcert.prov_form_vccs_widget_sample
	}
}

-----------------
return {
	Get = function() return definitions end
}