local utils = require "shared.apps.utils"
local relational_db = require("shared.relational_databases.sql"):Init()
require "LuaOrm.Date"
local model_vccs = require "apps.agent.models.vccs"
local m_helpers = require "helpers"
module(..., package.seeall, inconcert.orbit.application.Decorate)

local function loadDataSourse()
	local mysql_config = inconcert.environment.apps.jobs.mysql
	if not mysql_config or not mysql_config.ndb_nodes then
		return
	end
	

	local ndb_nodes_table = {}
	for key, value, state in mysql_config["ndb_nodes"]:childs() do
		if state ~= "deleted" then
			table.insert(ndb_nodes_table, {host = value.host })
		end
	end
	-- validar que existe al menos un host
	if ndb_nodes_table[1] and mysql_config.ndb_nodes[1].host then
		return {
			ndb_nodes = ndb_nodes_table,
			Username = mysql_config.user or "root",
			Password = mysql_config.password or "",
			Port = mysql_config.port,
			Type = "mysql"
		}
	end
end

local ConnectionData = function (dbName)
	
	local dataSource = loadDataSourse()
	if not dataSource then
		return
	end
	local ndb_nodes =  dataSource.ndb_nodes
	dataSource.Database = dbName or ""
	
	local connectionData
	for _, value in ipairs(ndb_nodes) do
		dataSource.Server = value.host
		connectionData = relational_db.Connect(dataSource)
		if connectionData then
			return connectionData
		end
		relational_db.Disconnect(connectionData)
	end
	
	return connectionData
end

local function ReturnError(web, status, err, connectionData)
	web.status = status or 500
	if connectionData then
		relational_db.Disconnect(connectionData)
	end
	return err
end

local function GetTablesInfo(connectionData, dbNameMysql)
	local sql_query_tables = "select TABLE_NAME, COLUMN_NAME, COLUMN_DEFAULT, IS_NULLABLE, DATA_TYPE, COLUMN_TYPE, COLUMN_KEY, EXTRA from information_schema.columns where table_schema = '".. dbNameMysql .."' order by table_name;"
	return relational_db.Query(connectionData, sql_query_tables)
end

local function CleanUpProcedure(web)
	local keyspace = utils.GetDatasource():CassandraConnection( "Agent" ) 
	local all_db = utils.GetAdmModel("Databases").GetDatabases(nil, keyspace)
	--LogInfo("[JOBS.CLEANUP_DATABASES.CleanUpProcedure], databases: '%s'", json.encode(all_db or {}))

	if not next(all_db) then
		return {result = "OK"}
	end
	local fnDebugging = function (key)
		local splitKey = m_helpers.utils.SplitString(key, "_")
		local vcc = splitKey[1]
		local vccInfo = model_vccs.Get(vcc)
		local configDB = vccInfo.Databases
		local connectionData = ConnectionData(key)
		if not connectionData then
			return ReturnError(web, nil, {result = "ERROR_CONNECTION"})
		end
		local result_query_tables, err = GetTablesInfo(connectionData, key)
		if err then
			return ReturnError(web, nil, {result = "ERROR"}, connectionData)
		end
		local analized_tables = {}
		for _, value in ipairs(result_query_tables.rows or {}) do
		
			if not analized_tables[value.TABLE_NAME] then
				local now = LuaOrm.Date()
				local limitTimeStamp = (now:add(unpack({0,0, -((configDB.TimeStamp or 360) -1) }))):toString("%Y-%m-%d")
				local sql_query_count = "select count(*) as count from ".. value.TABLE_NAME .. ";"
				local result_count, err_count = relational_db.Query(connectionData, sql_query_count)
				if err_count then
					return ReturnError(web, nil, {result = "ERROR"}, connectionData)
				end
				local count = result_count.rows[1].count
				local limit = math.abs(count - (configDB.RowsNumber or 3000))
				local sql_query_del = "DELETE FROM ".. value.TABLE_NAME .. " where ic_timestamp < '".. limitTimeStamp .. "' and " .. count .." > " .. tostring(configDB.RowsNumber or 3000) .." ORDER BY ic_timestamp ASC LIMIT " .. tostring(limit) ..";"
				--LogInfo("[JOBS.CLEANUP_DATABASES.CleanUpProcedure], sql_query_del: '%s'", sql_query_del)
				local result, err = relational_db.Query(connectionData, sql_query_del)
				if err then
					return ReturnError(web, nil, {result = "ERROR"}, connectionData)
				end
			end
			analized_tables[value.TABLE_NAME] = true
		end
		relational_db.Disconnect(connectionData)
	end

	for key, value in pairs(all_db) do
		local error = fnDebugging(key)
		if  error then
			keyspace:Close()
			return error
		end
	end
	keyspace:Close()
end

function MapUris(module, prefix)
	-- close queued interactions
	module:dispatch_post(CleanUpProcedure, prefix .. "/dbtablescleanup/(.*)")
	module:dispatch_get (CleanUpProcedure, prefix .. "/dbtablescleanup/(.*)")
end