require "LuaOrm.Date"
require "cjson"
local ir = require "lua_iragentlink"
local agentService = require "lua_agentserviceclient"
local utils = require "shared.apps.utils"
require "socket"
local moses = require "moses"
local http = require "socket.http"
local allegroProxy = require "apps.allegro.proxy"

module(..., package.seeall, inconcert.orbit.application.Decorate)

local redis, agentredis, cassandra, irLink
local m_agentclient, m_campaigns, m_camp_accounts, m_users, m_usersByCampaign, m_vccs, m_camp_ACDWaitThreshold
local m_RedisCampaignAgents = {}
local m_RedisAgentCampaigns = {}

local m_runningStats
local function IncrementStat(stat)
	m_runningStats[stat] = (m_runningStats[stat] or 0) + 1
end

--[[
	The flag m_doExecute is used before each write operation to support a "diagnose but do not fix" mode.
	Mainly for debugging purposes when you don't want to hell break loose
--]]
local m_doExecute = true

local SECONDS_IN_A_DAY = 86400
local SECONDS_IN_ONE_HOUR = 3600
local SECONDS_IN_TWO_HOUR = 7200
local SECONDS_IN_EIGHT_HOUR = 28800
local SECONDS_IN_TEN_MINUTES = 600

local esCaida = false
local senconds_caida =  -1

local validStatesOnES = {
	["WAITING"] =true,
	["QUEUED"] =true,
	["LOCKED"] = true,
	["TAKEN"] = true,
	["WRAPUP"] = true,
	["CANCELED"] = true,
	["FINISHED"] = true,
	["SPAM"] = true,
	["ABANDONED"] = true,
	["OPEN"] = true,
	["TRANSFER QUEUED"] = true,
}
local validChannels = {"MAIL", "CALL", "TWITTER", "FACEBOOK", "INSTAGRAM", "YOUTUBE", "APPSTORE", "GOOGLEPLAY", "RCS", "SMS", "WHATSAPP", "CHAT", "IM", "WEBCONTACT", "VOICEMAIL"}

function MapUris(module, prefix)
	module:dispatch_post(RepairAll, prefix.."/repair_job/(.*)")
	module:dispatch_get(RepairAll, prefix.."/repair_job/(.*)")
end

function CollectAllMemory(originFunction)
	local n = 0
	repeat
		local count1 = math.floor(collectgarbage("count"))
		collectgarbage()
		local count2 = math.floor(collectgarbage("count"))
		n = n + 1
	until count1 == count2
	LogDebug("Repair job: CollectAllMemory for %s took : %s collect passes", originFunction, n)
end

local function LoadUsersByCampaign(campaignAtVCC, keyspace)
	m_usersByCampaign[campaignAtVCC] = {}
	local campaignMembers = keyspace:GetSliceRange(campaignAtVCC,
										   "CampaignMembers",
										   "", {
												column_names = {},
												slice_range = {
													start = "",
													finish = "",
													reversed = false,
													count = 10000} --more than 1000 members on the campaign and this is gonna blow
												}
											)
	if campaignMembers then
		for i, column in ipairs(campaignMembers) do
			table.insert(m_usersByCampaign[campaignAtVCC], column.name)
		end
	end
end

local function LoadCampaignACDWaitThreshold (campaign_id_with_vcc)

	if type(campaign_id_with_vcc) == "string" then
	-- specific campaign accounts
		local params = {
			keyspace = "Agent",
			superColumns = false,
			columnFamily = "CampaignThresholds"
		}
		for _,channel in pairs(validChannels) do
			params.key = channel .. ":" .. campaign_id_with_vcc
			local campaignThreshold = utils.GetAllColumns( params ) or {}
			
			local acdWait
			if channel == "CALL" then
				acdWait = campaignThreshold.ACDWaitThreshold_INBOUND and tonumber(campaignThreshold.ACDWaitThreshold_INBOUND) or -1
			else
				acdWait = campaignThreshold.ACDWaitThreshold and tonumber(campaignThreshold.ACDWaitThreshold) or -1
			end
			if not m_camp_ACDWaitThreshold[campaign_id_with_vcc] then
				m_camp_ACDWaitThreshold[campaign_id_with_vcc] = {}
			end
			m_camp_ACDWaitThreshold[campaign_id_with_vcc][channel] = acdWait
			
		end
	end

end

local function LoadCampaignAccounts (campaign_id_with_vcc)
	local params = {
		keyspace = "Agent",
		superColumns = true,
		columnFamily = "CampaignAccounts"
	}

	if type(campaign_id_with_vcc) == "string" then
	-- specific campaign accounts
		params.key = campaign_id_with_vcc
	else
	-- all campaigns accounts
		params.startKey = ""
		params.endKey = ""
	end

	local accounts = utils.GetAllColumns( params ) or {}

	m_camp_accounts[campaign_id_with_vcc] = m_camp_accounts[campaign_id_with_vcc] or {}
	for _, account in pairs(accounts) do
		table.insert(m_camp_accounts[campaign_id_with_vcc], account)
	end
end

--[[
	Builds a list of all campaign ids on the database
--]]
local function LoadAllCampaigns()
	m_campaigns = {}
	m_camp_accounts = {}
	m_usersByCampaign = {}
	m_camp_ACDWaitThreshold = {}
	local keyspace = cassandra:CassandraConnection("Agent")
	--TODO this should be paginating cause ain't working with more than 1000 campaigns configured
	local campaignList = keyspace:GetRangeSlices("Campaigns",  --column family
										   "",  --supercolumn
										   "", --startkey
										   "", --endkey
												{ --predicate
												column_names = {},
												slice_range = {
															start = "",
															finish = "",
															reversed = false,
															count = 100}
												},
											10000 --total keys
							)
	if campaignList then
		for campaignId, campaignRow in campaignList:Iterator() do
			table.insert(m_campaigns, campaignId)
			LoadUsersByCampaign(campaignId, keyspace)
			LoadCampaignAccounts(campaignId)
			LoadCampaignACDWaitThreshold(campaignId)
		end
	end
	keyspace:Close()
end

local function LoadAllUsers()
	m_users = {}
	local keyspace = cassandra:CassandraConnection("Agent")
	--TODO this should be paginating cause ain't working with more than 1000 users configured
	local userList = keyspace:GetRangeSlices("Users",  --column family
										   "",  --supercolumn
										   "", --startkey
										   "", --endkey
												{ --predicate
												column_names = {},
												slice_range = {
															start = "",
															finish = "",
															reversed = false,
															count = 100}
												},
											10000 --total keys
							)
	if userList then
		for userId, userRow in userList:Iterator() do
			table.insert(m_users, userId)
		end
	end
	keyspace:Close()
end

local function LoadAllVCCs()
	m_vccs = {}
	local keyspace = cassandra:CassandraConnection("Agent")
	--TODO this should be paginating cause ain't working with more than 1000 users configured
	local vccList = keyspace:GetRangeSlices("VCCs",  --column family
										   "",  --supercolumn
										   "", --startkey
										   "", --endkey
												{ --predicate
												column_names = {},
												slice_range = {
															start = "",
															finish = "",
															reversed = false,
															count = 100}
												},
											1000 --total keys
							)
	if vccList then
		for vccId, vccRow in vccList:Iterator() do
			table.insert(m_vccs, vccId)
		end
	end
	keyspace:Close()
end

--[[
	Creates a permutation on a set template string similar to SetName:<CAMPAIGN>:<CHANNEL>:<AGENT>
	and returns all the possible sets formed with that pattern using the currently defined campaigns and agents on the database.
	Campaigns are processed before agents, so only the agents of each campaign are processed for those campaigns.
	Example:
		Having campaigns: product@inconcert and support@inconcert with users guille@inconcert and nacho@inconcert, and types CALL and FACEBOOK
		BuildPermutationSets("Counter:<CHANNEL>:<CAMPAIGN>:<AGENT>") returns {
			["Counter:CALL:product@inconcert:guille@inconcert"] = {
				AGENT = "guille@inconcert",
				CHANNEL = "CALL",
				CAMPAIGN = "product@inconcert"
			},
			["Counter:CALL:support@inconcert:guille@inconcert"] = {
				AGENT = "guille@inconcert",
				CHANNEL = "CALL",
				CAMPAIGN = "support@inconcert"
			},
			["Counter:CALL:product@inconcert:nacho@inconcert"] = {
				AGENT = "nacho@inconcert",
				CHANNEL = "CALL",
				CAMPAIGN = "product@inconcert"
			},
			["Counter:CALL:support@inconcert:nacho@inconcert"] = {
				AGENT = "nacho@inconcert",
				CHANNEL = "CALL",
				CAMPAIGN = "support@inconcert"
			},
			["Counter:FACEBOOK:product@inconcert:guille@inconcert"] = {
				AGENT = "guille@inconcert",
				CHANNEL = "FACEBOOK",
				CAMPAIGN = "product@inconcert"
			},
			["Counter:FACEBOOK:support@inconcert:guille@inconcert"] = {
				AGENT = "guille@inconcert",
				CHANNEL = "FACEBOOK",
				CAMPAIGN = "support@inconcert"
			},
			["Counter:FACEBOOK:product@inconcert:nacho@inconcert"] = {
				AGENT = "nacho@inconcert",
				CHANNEL = "FACEBOOK",
				CAMPAIGN = "product@inconcert"
			},
			["Counter:FACEBOOK:support@inconcert:nacho@inconcert"] = {
				AGENT = "nacho@inconcert",
				CHANNEL = "FACEBOOK",
				CAMPAIGN = "support@inconcert"
			}
		}
--]]
-- Tables to copy are only one level deep.
local function FlatCopy(a)
	local b = {}
	for k,v in pairs(a) do
		b[k] = v
	end
	return b
end

local function InteractionOnCassandra (interactionId)
	local keyspace = cassandra:CassandraConnection("Interactions")
	local detail = {}
	local interactionList = keyspace:GetSliceRange(interactionId,
		"Interactions",
		"", {
			column_names = {},
			slice_range = {
				start = "",
				finish = "",
				reversed = false,
				count = 200
			}
		}
	)
	keyspace:Close()
	
	local originalInteraction = {}
	if interactionList then
		for _, column in ipairs(interactionList or {}) do
			if column.name == "original" then
				local detail = json.decode(column.value)
				return detail
			end
		end
	end
	return
end

local function make_compatible_logger ()
	return {
		debug = LogDebug,
		info = LogInfo,
		warn = LogWarning,
		error = LogError,
		fatal = LogFatal,
	}
end

local function GetFromRepository (document_id, section, part)
	local Repository = require "icc_repository"
	local config_repository = {}
	config_repository.host = inconcert.environment.apps.jobs.cassandra.ip
	config_repository.port = inconcert.environment.apps.jobs.cassandra.port
	Repository.init({
		cassandra = config_repository,
		logger = make_compatible_logger (),
		repositoryNodes = inconcert.environment.apps.jobs.repsrvnodes
	})
	-- false means "get me the document on the repository right away instead of streaming it"
	local result, conversation = Repository.retrieve_file(document_id, section, part, false)
	return result, conversation
end

local function search_elastic_search (query, vccId, withPIT)
	local string_data = json.encode(query)
	local response_body = {}
	local index_name = "vcc-" .. vccId
	local urlString = "/_search"
	if not withPIT then
		urlString = "/"..index_name.."/_search"
	end

	LogDebug("Query for elasticsearch: (vcc: %s) %s", vccId, string_data)

	local ok, response_code, response_headers, response_status_line
	ok, response_code, response_headers, response_status_line = http.request({
		host = inconcert.environment.apps.agent.elasticsearch.host,
		port = inconcert.environment.apps.agent.elasticsearch.port,
		url = urlString,
		headers = { ["Content-Length"] = tostring(#string_data); ["Content-Type"] = "application/json" },
		method = "GET",
		source = ltn12.source.string(string_data),
		sink = ltn12.sink.table(response_body),
		redirect = false
	})
	if not ok then
		if response_body then
			return false, response_code
		end
		if response_code then
			return false, response_code
		end
		if response_headers then
			return false, json.encode(response_headers)
		end
		if response_status_line then
			return false, json.encode(response_status_line)
		end
		return false, ""
	end
	response_body = table.concat(response_body)
	if response_headers["content-type"]:match("^application/json") then
		local response = json.decode(response_body)
		if response.error then
			return false, response_body
		else
			if response and response.hits then
				LogInfo("[search_elastic_search] Query took %s ms and returned %s hits out of %s", response.took or 0, #response.hits.hits or "", response.hits.total.value or 0)
			end
			return true, response_body
		end
	end
	return false,response_body
end

local function ValidateElasticSearch(interactionId)
	local findInteraction = false
	local findOnCassandra = false
	local res, detail = pcall(json.decode, redis:get("InteractionDetail:"..interactionId))
	if res and detail then
		findInteraction = true
	else
		-- Si la interaccion no esta en Redis la busco el Cassandra
		detail = InteractionOnCassandra(interactionId)
		if detail then
			findInteraction = true
			findOnCassandra = true
		end
	end
	
	if findInteraction then
		-- Solo chequeo si la interaccion esta en un estado valido para ES.
		if detail.state and validStatesOnES[detail.state] then
			-- Obtengo la Interaccion desde ES para comparar con la de Redis
			local index_name = ("vcc-%s"):format(assert(detail.VCC))
			local response_body = {}
			local ok, response_code, response_headers, response_status_line
			
			ok, response_code, response_headers, response_status_line = http.request({
				host = inconcert.environment.apps.agent.elasticsearch.host,
				port = inconcert.environment.apps.agent.elasticsearch.port,
				url = ("/%s/_doc/%s"):format(index_name, detail.id),
				headers = { ["Content-Type"] = "application/json" },
				method = "GET",
				sink = ltn12.sink.table(response_body),
				redirect = false
			})
			if not ok then
				LogError("Repair job: Error obtaining interaction from Elasstic Search. %s\n%s\n%s\n%s", response_code, json.encode(response_headers), response_status_line,json.encode(response_body))
			else
				response_body = table.concat(response_body)
				if response_headers["content-type"]:match("^application/json") then
					local jsonResponse = json.decode(response_body)
					local existInteraction = jsonResponse.found
					local jsonInteraction = jsonResponse._source
					-- si no existe la interaccion en ES la creo
					if not existInteraction or not jsonInteraction or not jsonInteraction.state then
						local options = {}
						if detail.type == "CHAT" then
							-- Si el tipo de interaccion es CHAT tengo que obtener lo conversado del repositorio.
							local content = {}
							local result, conversation = GetFromRepository(detail.id, 0, 1)

							if result then
								LogDebug("Repair job: Retrieved conversation log: %s", conversation)
								conversation = json.decode(conversation)
								for _, message in ipairs(conversation.messages) do
									table.insert(content, message.text)
								end
							else
								LogWarning("Repair job: Could not obtain conversation with id '%s'", detail.id)
								if detail.interactionData.firstMessage then
									table.insert(content, detail.interactionData.firstMessage)
								end
							end
							options.additional_content = table.concat(content, " ")
						end 
						-- Si es un abandono y esta no esta en Redis no debe estar en ES
						if detail.state == "ABANDONED" and findOnCassandra then
							LogDebug("Repair job: InteractionID: %s on state ABANDONED and not in Redis. Not create in ES", detail.id)
						else
							LogDebug("Repair job: InteractionID: %s not exist on Elastic Search. Create it.", detail.id)
							if m_doExecute then
								irLink:SendCommand({
									command = "StoreInElasticSearch",
									proxyType = "AGENT",
									interaction = detail,
									options = options
								})
							end
						end
					else
						-- Si la interaccion existe, esta en estado Abandoned y no esta en Redis, tengo que borrarla.
						if detail.state == "ABANDONED" and findOnCassandra then
							local index_name = ("vcc-%s"):format(assert(detail.VCC))
							local urlString = "/"..index_name.."/_doc/"..detail.id
							local ok, response_code, response_headers, response_status_line
							local response_body = {}
							ok, response_code, response_headers, response_status_line = http.request({
								host = inconcert.environment.apps.agent.elasticsearch.host,
								port = inconcert.environment.apps.agent.elasticsearch.port,
								url = urlString,
								method = "DELETE",
								sink = ltn12.sink.table(response_body),
								redirect = false
							})
						else
							--Si los estados difieren y el timestampDouble de la interaccion es mayor a 5 mins actualizo en ElasticSearch
							local newModified = LuaOrm.Date(detail.timestamp)
							local now = LuaOrm.Date()
							if detail.state ~= jsonInteraction.state and (now:time() - newModified:time() > 300) then
								LogDebug("Repair job: InteractionID: %s, Interaction type: %s Redis state: %s, ElasticSearch state: %s. Update interaction on ElasticSearch", detail.id, detail.type, detail.state,jsonInteraction.state)
								local options = {}
								if detail.type == "CHAT" then
									-- Si el tipo de interaccion es CHAT tengo que obtener lo conversado del repositorio.
									local content = {}
									local result, conversation = GetFromRepository(detail.id, 0, 1)

									if result then
										LogDebug("Repair job: Retrieved conversation log: %s", conversation)
										conversation = json.decode(conversation)
										for _, message in ipairs(conversation.messages) do
											table.insert(content, message.text)
										end
									else
										LogWarning("Repair job: Could not obtain conversation with id '%s'", detail.id)
										if detail.interactionData.firstMessage then
											table.insert(content, detail.interactionData.firstMessage)
										end
									end
									options.additional_content = table.concat(content, " ")
								end
								if m_doExecute then
									irLink:SendCommand({
										command = "StoreInElasticSearch",
										proxyType = "AGENT",
										interaction = detail,
										options = options
									})
								end
							end
						end
					end
				else
					LogError("Repair job: Error obtaining interaction from Elasstic Search. %s\n%s\n%s\n%s", response_code, json.encode(response_headers), response_status_line,json.encode(response_body))
				end
			end
		end
	end
end

local function BuildPermutationSets(baseSet)
	-- Gets all the users of a campaign. Users should be loaded on m_usersByCampaign before using.
	local function UsersByCampaign(campaign)
		-- If I have the users of the campaign loaded, use them
		if m_usersByCampaign[campaign] then
			return m_usersByCampaign[campaign]
		end
		LogWarning("Repair job: Campaign %s has no loaded users, how's that possible?", campaign)
		return {}
	end

	-- Get values for a pivot. They may depend on the value of previous processed pivots.
	local function GetValues(pivot, previousValues)
		
		if pivot == "CAMPAIGN" then
			return m_campaigns

		elseif pivot == "ACCOUNT" then
			local accounts = {}
			if previousValues.CAMPAIGN then
				for _, account in pairs( m_camp_accounts[previousValues.CAMPAIGN] or {} ) do
					table.insert(accounts, account.id)
				end
			end
			return accounts

		elseif pivot == "ATTENTION_LEVEL" then
			local assigned_als = {}
			if previousValues.CAMPAIGN then
				for _, account in pairs( m_camp_accounts[previousValues.CAMPAIGN] or {} ) do
					if (previousValues.ACCOUNT and account.id == previousValues.ACCOUNT) or (not previousValues.ACCOUNT) then
						for _, assigned_al in pairs(account.AttentionLevels or {} ) do
							table.insert(assigned_als, assigned_al.attention_level_id)
						end
					end
				end
			end
			return assigned_als

		elseif pivot == "CHANNEL" then
			return validChannels

		elseif pivot == "AGENT" then
			if previousValues.CAMPAIGN then
				return UsersByCampaign(previousValues.CAMPAIGN)
			else
				return m_users
			end

		else
			--unknown, consider not a pattern
			return {pivot}
		end
	end

	--[[
	  Iterates on the resultSet, expanding a specific pivot to all its possible values. 
	  Supposing baseSet is "Counter:<CHANNEL>:<CAMPAIGN>:<AGENT>", with <CHANNEL> and <AGENT> already processed, resultSet is:
	  {
	      ["Counter:SMS:<CAMPAIGN>:fernickk@inconcert"] = { CHANNEL = "SMS", AGENT = "fernickk@inconcert" }, 
	      ["Counter:MAIL:<CAMPAIGN>:fernickk@inconcert"] = { CHANNEL = "MAIL", AGENT = "fernickk@inconcert" }, 
	      etc
	  }
	]]
	local function ProcessPivot(pivot, resultSet)
		local newResultSet = {}
		for result, values in pairs(resultSet) do
			if string.match(result, "<"..  pivot ..">") then
				--replace <pivot> for all its possible values
				for _, pivotValue in pairs(GetValues(pivot, values)) do
					local newKey = string.gsub(result, "<"..  pivot ..">", pivotValue)
					newResultSet[newKey] = FlatCopy(values)
					newResultSet[newKey][pivot] = pivotValue
				end
			end
		end
		return next(newResultSet) and newResultSet or resultSet
	end

	-- If existing, this pivots should be processed first, in this order.
	local preferredPivots = {
		"CAMPAIGN",
		"ACCOUNT",
		"ATTENTION_LEVEL",
		"AGENT",
		"CHANNEL"
	}

	local resultSet = {}
	resultSet[baseSet] = {}

	local pivots = {}
	for pivot in string.gmatch(baseSet, "<([^>]+)>") do
		pivots[pivot] = true
	end
	-- First process the preferred pivots, if they exist on the baseSet
	for _, pivot in ipairs(preferredPivots) do
		if pivots[pivot] then
			resultSet = ProcessPivot(pivot, resultSet)
			pivots[pivot] = nil
		end
	end
	-- Then process all remaining pivots
	for pivot in pairs(pivots) do
		resultSet = ProcessPivot(pivot, resultSet)
	end
	return resultSet
end

local function CheckRequirements(detail, requirements)
	if not requirements then
		return true
	end
	if requirements.CHANNEL and requirements.CHANNEL ~= detail.type then
		return false
	end
	if requirements.CAMPAIGN and requirements.CAMPAIGN ~= detail.campaign then
		return false
	end
	return true
end

local function InteractionHasAttentionLevel(detail)
	return detail.attention_level
end

-- An interaction is closed if it's CANCELED or FINISHED
local function InteractionIsClosed(detail, requirements)
	if not CheckRequirements(detail, requirements) then
		return false
	end
	return detail.state == "CANCELED" or detail.state == "FINISHED" or detail.state == "DISCARD" or detail.state == "INCONSISTENT" or (detail.state == "ABANDONED" and not detail.callbackIsAllowed)
end

-- An interaction is open if it's not closed (duh...)
local function InteractionIsOpen(detail, requirements)
	if not CheckRequirements(detail, requirements) then
		return false
	end
	return not InteractionIsClosed(detail)
end

-- An interaction is spam if it's SPAM
local function InteractionIsSpam(detail, requirements)
	if not CheckRequirements(detail, requirements) then
		return false
	end
	return detail.state == "SPAM"
end

local function InteractionHasQueuedState(detail) 
	return detail.state == "QUEUED" or detail.state == "RESERVED" or detail.state == "TRANSFER QUEUED"
end

local function InteractionHasRequiredAgent(detail)
	return detail.requiredAgent and detail.requiredAgent ~= ""
end

local function InteractionIsQueued(detail, requirements)
	if not CheckRequirements(detail, requirements) then
		return false
	end
	return InteractionHasQueuedState(detail) and not InteractionHasRequiredAgent(detail)
end

local function InteractionIsQueuedToAgent(detail, requirements)
	if not requirements or not requirements.AGENT then
		return false
	end
	if not InteractionIsQueued(detail, requirements) then
		return false
	end
	for _, reservedAgent in pairs(detail.reservedAgents) do
		if reservedAgent == requirements.AGENT then
			return true
		end
	end
	return false
end

local function InteractionIsQueuedWithoutAgent(detail, requirements)
	if not InteractionIsQueued(detail, requirements) then
		return false
	end
	if not detail.reservedAgents or #detail.reservedAgents == 0 then
		return true
	end

	return false
end

-- Interaction ABANDONED -------------------------------------------------------------------------------
local function InteractionIsAbandoned(detail, requirements)
	if not CheckRequirements(detail, requirements) then
		return false
	end
	return detail.state == "ABANDONED" and detail.callbackIsAllowed
end

local function InteractionIsAbandonedToAgent(detail, requirements)
	if not requirements or not requirements.AGENT then
		return false
	end
	if not InteractionIsAbandoned(detail, requirements) then
		return false
	end
	for _, reservedAgent in pairs(detail.reservedAgents) do
		if reservedAgent == requirements.AGENT then
			return true
		end
	end
	return false
end

local function InteractionIsAbandonedWithoutAgent(detail, requirements)
	if not InteractionIsAbandoned(detail, requirements) then
		return false
	end
	if not detail.reservedAgents or #detail.reservedAgents == 0 then
		return true
	end

	return false
end

-- Interaction OPEN. CallBacks -----------------------------------------------------------------------------
local function InteractionIsOnStateOpen(detail, requirements)
	if not CheckRequirements(detail, requirements) then
		return false
	end
	return detail.state == "OPEN"
end

local function InteractionIsOnStateOpenToAgent(detail, requirements)
	if not requirements or not requirements.AGENT then
		return false
	end
	if not InteractionIsOnStateOpen(detail, requirements) then
		return false
	end
	return detail.requiredAgent == requirements.AGENT
end

local function InteractionIsOnStateOpenWithoutAgent(detail, requirements)
	if not InteractionIsOnStateOpen(detail, requirements) then
		return false
	end
	if not detail.requiredAgent or detail.requiredAgent == "" then
		return true
	end

	return false
end

-- An interaction is assigned if it's QUEUED or RESERVED with required agent, or if it's not closed, queued or spam and has an assigned agent
local function InteractionIsAssigned(detail, requirements)
	if not CheckRequirements(detail, requirements) then
		return false
	end
	if InteractionHasQueuedState(detail) then
		return InteractionHasRequiredAgent(detail)
	end
	return not InteractionIsClosed(detail) and not InteractionIsSpam(detail) and not InteractionIsAbandoned(detail) and not InteractionIsOnStateOpen(detail) and (next(detail.assignedAgents) or next(detail.reservedAgents))
end

-- Same as InteractionIsAssigned, but checks that the AGENT in requirements is actually assigned
local function InteractionIsAssignedToAgent(detail, requirements)
	if not requirements or not requirements.AGENT then
		return false
	end
	if not InteractionIsAssigned(detail, requirements) then
		return false
	end
	if InteractionHasQueuedState(detail) then
		return detail.requiredAgent == requirements.AGENT
	end
	for _, assignedAgent in pairs(detail.assignedAgents) do
		if assignedAgent == requirements.AGENT then
			return true
		end
	end
	-- En telefonia una interaccion puede estar LOCKED con reservedAgents
	for _, reservedAgent in pairs(detail.reservedAgents) do
		if reservedAgent == requirements.AGENT then
			return true
		end
	end
	return false
end

-- An interaction is waiting if it's not anything else
local function InteractionIsWaiting(detail, requirements)
	if not CheckRequirements(detail, requirements) then
		return false
	end
	return not InteractionIsClosed(detail) and not InteractionIsQueued(detail) and not InteractionIsSpam(detail) and not InteractionIsAssigned(detail) and not InteractionIsAbandoned(detail) and not InteractionIsOnStateOpen(detail)
end

local function InteractionIsCreated(detail, requirements)
	if not CheckRequirements(detail, requirements) then
		return false
	end
	return detail.state == "CREATED"
end

local function InteractionIsProcessFlow(detail, requirements)
	return InteractionIsCreated(detail, requirements) and detail.processFlow and true or false
end

--[[
	Iterates over all the possible existing sets on redis to validate each member interaction
	is in a state that justifies belonging to the current set, if it's not, they're removed from the set
--]]
local function RepairInteractionSets()

	--[[
		Removes an interaction from a specified set if it's doesn't pass a specific validation criteria
		@set             The set from which to remove the interaction
		@interactionID   The id of the interaction to check against
		@setType         The type of set in redis (zset or set).
		@setKey          (optional) The key of the element to be removed from the set. If not set, it's the interactionID
		@setDetails      (optional) A table with specific data about the set, such as CHANNEL, CAMPAIGN or AGENT. Empty if not set
		@validator       A function that should be called with the interaction detail and the set details to validate it belongs there
	--]]
	local function RemoveInteractionIfNotValid(params)
		local set = params.set
		local interactionID = params.interactionID
		local setType = params.setType
		local setKey = params.setKey or interactionID
		local setDetail = params.setDetails or {}
		local validator = params.validator
		local removedKey = false
		
		local key = "InteractionDetail:"..interactionID
		local options = { watch = key, cas = true, retry = 0 }
		local result = pcall(redis.transaction, redis, options, function(trans)
			local res, detail = pcall(json.decode, trans:get("InteractionDetail:"..interactionID))
			if res and detail then
				if validator(detail, setDetail) then
					return
				end
			end
			LogError("Repair job: Removing interaction %s from set %s it was on state %s", interactionID, set, detail and detail.state or "UNDECODABLE")
			IncrementStat("danglingInteractions")
			if not m_doExecute then
				return
			end
			trans:multi()
			if setType == "zset" then
				trans:zrem(set, setKey)
			elseif setType == "set" then
				trans:srem(set, setKey)
			end
			removedKey = true
			return true
		end)
		-- Si se elimina alguna subKey hay que chequear el status del lado de ElasticSeach.
		if removedKey then
			ValidateElasticSearch(interactionID)
		end
		
		return result
	end

	--[[
		Remove all interactions from a specific set that don't pass a specific validation criteria
		@set                      The set to check
		@setDetails               (optional) A table with specific data about the set, such as CHANNEL, CAMPAIGN or AGENT. Empty if not set
		@validator                A function that should be called with the interaction detail and the set details to validate it belongs there
		@interactionIDPattern     (optional) If the members of the sets are not the interactionIDs, this pattern indicates how to extract them
	--]]
	local function CheckInteractionSet(params)
		local set = params.set
		local setDetails = params.setDetails or {}
		local validator = params.validator
		local interactionIDPattern = params.interactionIDPattern

		local setType = redis:type(set)
		if setType == "none" then
			return -- No interactions on this set
		end

		IncrementStat("repairedSets")
		if setType ~= "zset" and setType ~= "set" then
			LogError("Repair job: Set %s has invalid type %s, please check your schema now!", set, setType)
			IncrementStat("invalidSets")
			return
		end

		local keys = setType == "zset" and redis:zrange(set, 0, -1) or redis:smembers(set)
		for _, key in pairs(keys) do
			local interactionID = key
			if interactionIDPattern and interactionIDPattern ~= "" then
				interactionID = key:match(interactionIDPattern) or key
			end
			RemoveInteractionIfNotValid {
				set = set, 
				interactionID = interactionID,
				setType = setType,
				setKey = key,
				setDetails = setDetails,
				validator = validator
			}
		end
	end

	--[[
		For each set in sets, removes all of its interactions that don't pass a specific validation criteria
		@sets                     The sets to check: this is a table where the keys are the sets, 
		                          and the values are tables with the set details (CHANNEL, CAMPAIGN and so on)
		@validator                A function that should be called with the interaction detail and the set details to validate it belongs there
		@interactionIDPattern     (optional) If the members of the sets are not the interactionIDs, this pattern indicates how to extract them
	--]]
	local function CheckInteractionSets(params)
		local sets = params.sets
		local validator = params.validator
		local interactionIDPattern = params.interactionIDPattern
		for set, setDetails in pairs(sets) do
			CheckInteractionSet {
				set = set,
				setDetails = setDetails,
				validator = validator,
				interactionIDPattern = interactionIDPattern
			}
		end
	end

	local function keySetToCheck(keyType, InteractionIDPattern)
		
		local validator1 
		local validator2 
		local validator3 
		if keyType == "Abandoned" then
			validator1 = InteractionIsAbandoned
			validator2 = InteractionIsAbandonedToAgent
			validator3 = InteractionIsAbandonedWithoutAgent
		elseif keyType == "Queued" then
			validator1 = InteractionIsQueued
			validator2 = InteractionIsQueuedToAgent
			validator3 = InteractionIsQueuedWithoutAgent
		elseif keyType == "Assigned" then
			validator1 = InteractionIsAssigned
			validator2 = InteractionIsAssignedToAgent
		elseif keyType == "ProcessFlow" then
			validator1 = InteractionIsProcessFlow
		elseif keyType == "Waiting" then
			validator1 = InteractionIsWaiting
		elseif keyType == "Open" then
			validator1 = InteractionIsOnStateOpen
			validator2 = InteractionIsOnStateOpenToAgent
			validator3 = InteractionIsOnStateOpenWithoutAgent
		end
		
		LogDebug("Repair job processing SET '" .. keyType .. "InteractionsByCampaign:<CAMPAIGN>'")
		CheckInteractionSets { sets = BuildPermutationSets(keyType .. "InteractionsByCampaign:<CAMPAIGN>"), validator = validator1, interactionIDPattern = InteractionIDPattern }
		
		LogDebug("Repair job processing SET '" .. keyType .. "InteractionsByTypeAndCampaign:<CHANNEL>:<CAMPAIGN>'")
		CheckInteractionSets { sets = BuildPermutationSets(keyType .. "InteractionsByTypeAndCampaign:<CHANNEL>:<CAMPAIGN>"), validator = validator1, interactionIDPattern = InteractionIDPattern }
		
		LogDebug("Repair job processing SET '" .. keyType .. "InteractionsByCampaignAccount:<CAMPAIGN>:<ACCOUNT>'")
		CheckInteractionSets { sets = BuildPermutationSets(keyType .. "InteractionsByCampaignAccount:<CAMPAIGN>:<ACCOUNT>"), validator = validator1, interactionIDPattern = InteractionIDPattern }
		
		LogDebug("Repair job processing SET '" .. keyType .. "InteractionsByCampaignAccountAttentionLevel:<CAMPAIGN>:<ACCOUNT>:<ATTENTION_LEVEL>'")
		CheckInteractionSets { sets = BuildPermutationSets(keyType .. "InteractionsByCampaignAccountAttentionLevel:<CAMPAIGN>:<ACCOUNT>:<ATTENTION_LEVEL>"), validator = 
			function( detail, requirements )
				local attention_level = InteractionHasAttentionLevel(detail)
				if not attention_level then
					return false
				end
				return validator1(detail, requirements)
			end, interactionIDPattern = InteractionIDPattern }
		
		LogDebug("Repair job processing SET '" .. keyType .. "InteractionsByCampaignAccountAttentionLevel:<CAMPAIGN>:<ACCOUNT>:na'")
		CheckInteractionSets { sets = BuildPermutationSets(keyType .. "InteractionsByCampaignAccountAttentionLevel:<CAMPAIGN>:<ACCOUNT>:na"), validator = validator1, interactionIDPattern = InteractionIDPattern }

		LogDebug("Repair job processing SET '" .. keyType .. "InteractionsByAttentionLevelTypeAndCampaign:<ATTENTION_LEVEL>:<CHANNEL>:<CAMPAIGN>'")
		CheckInteractionSets { sets = BuildPermutationSets(keyType .. "InteractionsByAttentionLevelTypeAndCampaign:<ATTENTION_LEVEL>:<CHANNEL>:<CAMPAIGN>"), validator = 
			function( detail, requirements )
				local attention_level = InteractionHasAttentionLevel(detail)
				if not attention_level then
					return false
				end
				return validator1(detail, requirements)
			end, interactionIDPattern = InteractionIDPattern}
		
		LogDebug("Repair job processing SET '" .. keyType .. "InteractionsByAttentionLevelTypeAndCampaign:na:<CHANNEL>:<CAMPAIGN>'")
		CheckInteractionSets { sets = BuildPermutationSets(keyType .. "InteractionsByAttentionLevelTypeAndCampaign:na:<CHANNEL>:<CAMPAIGN>"), validator = validator1, interactionIDPattern = InteractionIDPattern }

		LogDebug("Repair job processing SET 'Inbound" .. keyType .. "InteractionsByTypeAndCampaign:<CHANNEL>:<CAMPAIGN>'")
		CheckInteractionSets { sets = BuildPermutationSets("Inbound"..keyType .. "InteractionsByTypeAndCampaign:<CHANNEL>:<CAMPAIGN>"), validator = validator1, interactionIDPattern = "" }
		
		LogDebug("Repair job processing SET 'Inbound" .. keyType .. "InteractionsByCampaignAccount:<CAMPAIGN>:<ACCOUNT>'")
		CheckInteractionSets { sets = BuildPermutationSets("Inbound"..keyType .. "InteractionsByCampaignAccount:<CAMPAIGN>:<ACCOUNT>"), validator = validator1, interactionIDPattern = "" }
		
		LogDebug("Repair job processing SET 'Inbound" .. keyType .. "InteractionsByCampaignAccountAttentionLevel:<CAMPAIGN>:<ACCOUNT>:<ATTENTION_LEVEL>'")
		CheckInteractionSets { sets = BuildPermutationSets("Inbound"..keyType .. "InteractionsByCampaignAccountAttentionLevel:<CAMPAIGN>:<ACCOUNT>:<ATTENTION_LEVEL>"), validator = 
			function( detail, requirements )
				local attention_level = InteractionHasAttentionLevel(detail)
				if not attention_level then
					return false
				end
				return validator1(detail, requirements)
			end, interactionIDPattern = ""}
		
		LogDebug("Repair job processing SET 'Outbound"..keyType .. "InteractionsByTypeAndCampaign:<CHANNEL>:<CAMPAIGN>'")
		CheckInteractionSets { sets = BuildPermutationSets("Outbound"..keyType .. "InteractionsByTypeAndCampaign:<CHANNEL>:<CAMPAIGN>"), validator = validator1, interactionIDPattern = "" }

		LogDebug("Repair job processing SET 'Outbound" .. keyType .. "InteractionsByCampaignAccount:<CAMPAIGN>:<ACCOUNT>'")
		CheckInteractionSets { sets = BuildPermutationSets("Outbound"..keyType .. "InteractionsByCampaignAccount:<CAMPAIGN>:<ACCOUNT>"), validator = validator1, interactionIDPattern = "" }

		LogDebug("Repair job processing SET 'Outbound" .. keyType .. "InteractionsByTypeAgentAndCampaign:<CHANNEL>:<AGENT>:<CAMPAIGN>'")
		CheckInteractionSets { sets = BuildPermutationSets("Outbound" .. keyType .. "InteractionsByTypeAgentAndCampaign:<CHANNEL>:<AGENT>:<CAMPAIGN>"), validator = validator1, interactionIDPattern = "" }
		
		LogDebug("Repair job processing SET 'Outbound" .. keyType .. "InteractionsByCampaignAccountAttentionLevel:<CAMPAIGN>:<ACCOUNT>:<ATTENTION_LEVEL>'")
		CheckInteractionSets { sets = BuildPermutationSets("Outbound"..keyType .. "InteractionsByCampaignAccountAttentionLevel:<CAMPAIGN>:<ACCOUNT>:<ATTENTION_LEVEL>"), validator = 
			function( detail, requirements )
				local attention_level = InteractionHasAttentionLevel(detail)
				if not attention_level then
					return false
				end
				return validator1(detail, requirements)
			end, interactionIDPattern = "" }

		LogDebug("Repair job processing SET '" .. keyType .. "InteractionsByAgent:<AGENT>'")
		CheckInteractionSets { sets = BuildPermutationSets(keyType .. "InteractionsByAgent:<AGENT>"), validator = validator2, interactionIDPattern = InteractionIDPattern }
		
		LogDebug("Repair job processing SET '" .. keyType .. "InteractionsByAgentAndCampaign:<AGENT>:<CAMPAIGN>'")
		CheckInteractionSets { sets = BuildPermutationSets(keyType .. "InteractionsByAgentAndCampaign:<AGENT>:<CAMPAIGN>"), validator = validator2, interactionIDPattern = InteractionIDPattern }
		
		LogDebug("Repair job processing SET '" .. keyType .. "InteractionsByAgentCampaignAccount:<AGENT>:<CAMPAIGN>:<ACCOUNT>'")
		CheckInteractionSets { sets = BuildPermutationSets(keyType .. "InteractionsByAgentCampaignAccount:<AGENT>:<CAMPAIGN>:<ACCOUNT>"), validator = validator2, interactionIDPattern = InteractionIDPattern }
		
		LogDebug("Repair job processing SET '" .. keyType .. "InteractionsByTypeAgentAndCampaign:<CHANNEL>:<AGENT>:<CAMPAIGN>'")
		CheckInteractionSets { sets = BuildPermutationSets(keyType .. "InteractionsByTypeAgentAndCampaign:<CHANNEL>:<AGENT>:<CAMPAIGN>"), validator = validator2, interactionIDPattern = InteractionIDPattern }
		
		LogDebug("Repair job processing SET '" .. keyType .. "InteractionsByAgentCampaignAccountAttentionLevel:<AGENT>:<CAMPAIGN>:<ACCOUNT>:<ATTENTION_LEVEL>'")
		CheckInteractionSets { sets = BuildPermutationSets(keyType .. "InteractionsByAgentCampaignAccountAttentionLevel:<AGENT>:<CAMPAIGN>:<ACCOUNT>:<ATTENTION_LEVEL>"), validator =
			function( detail, requirements )
				local attention_level = InteractionHasAttentionLevel(detail)
				if not attention_level then
					return false
				end
				return validator2(detail, requirements)
			end, interactionIDPattern = InteractionIDPattern}
			
		LogDebug("Repair job processing SET '" .. keyType .. "InteractionsByAgentCampaignAccountAttentionLevel:<AGENT>:<CAMPAIGN>:<ACCOUNT>:na'")
		CheckInteractionSets { sets = BuildPermutationSets(keyType .. "InteractionsByAgentCampaignAccountAttentionLevel:<AGENT>:<CAMPAIGN>:<ACCOUNT>:na"), validator = validator2, interactionIDPattern = InteractionIDPattern }

		LogDebug("Repair job processing SET 'Inbound" .. keyType .. "InteractionsByTypeAgentAndCampaign:<CHANNEL>:<AGENT>:<CAMPAIGN>'")
		CheckInteractionSets { sets = BuildPermutationSets("Inbound"..keyType .. "InteractionsByTypeAgentAndCampaign:<CHANNEL>:<AGENT>:<CAMPAIGN>"), validator = validator2, interactionIDPattern = "" }
		
		-- Unassigned
		if keyType == "Abandoned" or keyType == "Queued" or keyType == "Open" then
			LogDebug("Repair job processing SET 'Unassigned" .. keyType .. "InteractionsByCampaign:<CAMPAIGN>'")
			CheckInteractionSets { sets = BuildPermutationSets("Unassigned"..keyType .. "InteractionsByCampaign:<CAMPAIGN>"), validator = validator3, interactionIDPattern = InteractionIDPattern }
			
			LogDebug("Repair job processing SET 'Unassigned" .. keyType .. "InteractionsByTypeAndCampaign:<CHANNEL>:<CAMPAIGN>'")
			CheckInteractionSets { sets = BuildPermutationSets("Unassigned"..keyType .. "InteractionsByTypeAndCampaign:<CHANNEL>:<CAMPAIGN>"), validator = validator3, interactionIDPattern = InteractionIDPattern }
			
			LogDebug("Repair job processing SET 'Unassigned" .. keyType .. "InteractionsByCampaignAccount:<CAMPAIGN>:<ACCOUNT>'")
			CheckInteractionSets { sets = BuildPermutationSets("Unassigned"..keyType .. "InteractionsByCampaignAccount:<CAMPAIGN>:<ACCOUNT>"), validator = validator3, interactionIDPattern = InteractionIDPattern }
			
			LogDebug("Repair job processing SET 'Unassigned" .. keyType .. "InteractionsByCampaignAccountAttentionLevel:<CAMPAIGN>:<ACCOUNT>:<ATTENTION_LEVEL>'")
			CheckInteractionSets { sets = BuildPermutationSets("Unassigned"..keyType .. "InteractionsByCampaignAccountAttentionLevel:<CAMPAIGN>:<ACCOUNT>:<ATTENTION_LEVEL>"), validator = 
				function( detail, requirements )
					local attention_level = InteractionHasAttentionLevel(detail)
					if not attention_level then
						return false
					end
					return validator3(detail, requirements)
				end, interactionIDPattern = InteractionIDPattern }
			
			LogDebug("Repair job processing SET 'Unassigned" .. keyType .. "InteractionsByCampaignAccountAttentionLevel:<CAMPAIGN>:<ACCOUNT>:na'")
			CheckInteractionSets { sets = BuildPermutationSets("Unassigned"..keyType .. "InteractionsByCampaignAccountAttentionLevel:<CAMPAIGN>:<ACCOUNT>:na"), validator = validator3, interactionIDPattern = InteractionIDPattern }

			LogDebug("Repair job processing SET 'Unassigned" .. keyType .. "InteractionsByAttentionLevelTypeAndCampaign:<ATTENTION_LEVEL>:<CHANNEL>:<CAMPAIGN>'")
			CheckInteractionSets { sets = BuildPermutationSets("Unassigned"..keyType .. "InteractionsByAttentionLevelTypeAndCampaign:<ATTENTION_LEVEL>:<CHANNEL>:<CAMPAIGN>"), validator = 
				function( detail, requirements )
					local attention_level = InteractionHasAttentionLevel(detail)
					if not attention_level then
						return false
					end
					return validator3(detail, requirements)
				end, interactionIDPattern = InteractionIDPattern}
			
			LogDebug("Repair job processing SET 'Unassigned" .. keyType .. "InteractionsByAttentionLevelTypeAndCampaign:na:<CHANNEL>:<CAMPAIGN>'")
			CheckInteractionSets { sets = BuildPermutationSets("Unassigned"..keyType .. "InteractionsByAttentionLevelTypeAndCampaign:na:<CHANNEL>:<CAMPAIGN>"), validator = validator3, interactionIDPattern = InteractionIDPattern }

			
			LogDebug("Repair job processing SET 'InboundUnassigned" .. keyType .. "InteractionsByTypeAndCampaign:<CHANNEL>:<CAMPAIGN>'")
			CheckInteractionSets { sets = BuildPermutationSets("InboundUnassigned"..keyType .. "InteractionsByTypeAndCampaign:<CHANNEL>:<CAMPAIGN>"), validator = validator3, interactionIDPattern = "" }
			
			LogDebug("Repair job processing SET 'InboundUnassigned" .. keyType .. "InteractionsByCampaignAccount:<CAMPAIGN>:<ACCOUNT>'")
			CheckInteractionSets { sets = BuildPermutationSets("InboundUnassigned"..keyType .. "InteractionsByCampaignAccount:<CAMPAIGN>:<ACCOUNT>"), validator = validator3, interactionIDPattern = "" }
			
			LogDebug("Repair job processing SET 'InboundUnassigned" .. keyType .. "InteractionsByCampaignAccountAttentionLevel:<CAMPAIGN>:<ACCOUNT>:<ATTENTION_LEVEL>'")
			CheckInteractionSets { sets = BuildPermutationSets("InboundUnassigned"..keyType .. "InteractionsByCampaignAccountAttentionLevel:<CAMPAIGN>:<ACCOUNT>:<ATTENTION_LEVEL>"), validator = 
				function( detail, requirements )
					local attention_level = InteractionHasAttentionLevel(detail)
					if not attention_level then
						return false
					end
					return validator3(detail, requirements)
				end, interactionIDPattern = ""}
			
			LogDebug("Repair job processing SET 'OutboundUnassigned" .. keyType .. "InteractionsByTypeAndCampaign:<CHANNEL>:<CAMPAIGN>'")
			CheckInteractionSets { sets = BuildPermutationSets("OutboundUnassigned"..keyType .. "InteractionsByTypeAndCampaign:<CHANNEL>:<CAMPAIGN>"), validator = validator3, interactionIDPattern = "" }

			LogDebug("Repair job processing SET 'OutboundUnassigned" .. keyType .. "InteractionsByCampaignAccount:<CAMPAIGN>:<ACCOUNT>'")
			CheckInteractionSets { sets = BuildPermutationSets("OutboundUnassigned"..keyType .. "InteractionsByCampaignAccount:<CAMPAIGN>:<ACCOUNT>"), validator = validator3, interactionIDPattern = "" }

			LogDebug("Repair job processing SET 'OutboundUnassigned" .. keyType .. "InteractionsByCampaignAccountAttentionLevel:<CAMPAIGN>:<ACCOUNT>:<ATTENTION_LEVEL>'")
			CheckInteractionSets { sets = BuildPermutationSets("OutboundUnassigned"..keyType .. "InteractionsByCampaignAccountAttentionLevel:<CAMPAIGN>:<ACCOUNT>:<ATTENTION_LEVEL>"), validator = 
				function( detail, requirements )
					local attention_level = InteractionHasAttentionLevel(detail)
					if not attention_level then
						return false
					end
					return validator3(detail, requirements)
				end, interactionIDPattern = ""}
		end
	end
	
	-- Open interactions
	LogDebug("Repair job processing SET 'AllActiveInteractions'")
	CheckInteractionSets { sets = { AllActiveInteractions = {} }, validator = InteractionIsOpen }
	LogDebug("Repair job processing SET 'InteractionsByCampaign:<CAMPAIGN>'")
	CheckInteractionSets { sets = BuildPermutationSets("InteractionsByCampaign:<CAMPAIGN>"), validator = InteractionIsOpen }
	LogDebug("Repair job processing SET 'InteractionsByTypeAndCampaign:<CHANNEL>:<CAMPAIGN>'")
	CheckInteractionSets { sets = BuildPermutationSets("InteractionsByTypeAndCampaign:<CHANNEL>:<CAMPAIGN>"), validator = InteractionIsOpen }

	-- Queued interactions
	LogDebug("Repair job Checking Queued permutation set.")
	local setInteractions = redis:keys("*Queued*")
	if setInteractions and #setInteractions > 0 then
		local queuedInteractionIDPattern = "^[^@]*@(.*)$"
		keySetToCheck("Queued",queuedInteractionIDPattern)
	else
		LogDebug("Repair job Not exists Queued permutation set. Continue.")
	end
	
	-- Assigned interactions
	LogDebug("Repair job Checking Assigned permutation set.")
	setInteractions = redis:keys("*Assigned*")
	if setInteractions and #setInteractions > 0 then
		local assignedInteractionIDPattern = "^(.+)#.+#.+$"
		keySetToCheck("Assigned",assignedInteractionIDPattern)
	else
		LogDebug("Repair job Not exists Assigned permutation set. Continue.")
	end

	-- Spam interactions
	LogDebug("Repair job processing SET 'SpamInteractionsToExpire'")
	CheckInteractionSets { sets = { SpamInteractionsToExpire = {} }, validator = InteractionIsSpam }
	LogDebug("Repair job processing SET 'SpamInteractionsByCampaign:<CAMPAIGN>'")
	CheckInteractionSets { sets = BuildPermutationSets("SpamInteractionsByCampaign:<CAMPAIGN>"), validator = InteractionIsSpam }
	LogDebug("Repair job processing SET 'SpamInteractionsByTypeAndCampaign:<CHANNEL>:<CAMPAIGN>'")
	CheckInteractionSets { sets = BuildPermutationSets("SpamInteractionsByTypeAndCampaign:<CHANNEL>:<CAMPAIGN>"), validator = InteractionIsSpam }

	-- ProcessFlow interactions
	LogDebug("Repair job Checking ProcessFlow permutation set.")
	setInteractions = redis:keys("*ProcessFlowInteractions*")
	if setInteractions and #setInteractions > 0 then
		keySetToCheck("ProcessFlow","")
	else
		LogDebug("Repair job Not exists ProcessFlow permutation set. Continue.")
	end

	-- Waiting interactions
	LogDebug("Repair job Checking Waiting permutation set.")
	setInteractions = redis:keys("*Waiting*")
	if setInteractions and #setInteractions > 0 then
		keySetToCheck("Waiting","")
	else
		LogDebug("Repair job Not exists Waiting permutation set. Continue.")
	end
	
	-- Abandoned Interaction
	LogDebug("Repair job Checking Abandoned permutation set.")
	setInteractions = redis:keys("*Abandoned*")
	if setInteractions and #setInteractions > 0 then
		keySetToCheck("Abandoned","")
	else
		LogDebug("Repair job Not exists Abandoned permutation set. Continue.")
	end
	
	-- Open interactions
	LogDebug("Repair job Checking Open permutation set.")
	setInteractions = redis:keys("*Open*")
	if setInteractions and #setInteractions > 0 then
		keySetToCheck("Open","")
	else
		LogDebug("Repair job Not exists Open permutation set. Continue.")
	end

	CollectAllMemory("RepairInteractionSets")

end

local function GetAgentClient()
	if (not m_agentclient) then
		local host = inconcert.environment.apps.agent.agentservice.host
		local port = inconcert.environment.apps.agent.agentservice.port
		m_agentclient = agentService.LuaAgentServiceClient(inconcert.environment.apps.agent.cluster)
		m_agentclient:AddKnownAgentService(host, port)
	end
	return m_agentclient
end

local function CancelRequestByInteractionOnAgentService(interactionId)
	if m_doExecute then
		local debug = GetAgentClient()
		GetAgentClient():CancelRequestsByInteraction(interactionId)
	end
end

local function CancelRequestByIdOnAgentService(requestId)
	if m_doExecute then
		local debug = GetAgentClient()
		GetAgentClient():CancelRequestById(requestId)
	end
	IncrementStat("cancelledRequests")
end

-- Returns true if the interaction has asynchronous nature
local function IsOffline(interaction)
	return interaction.type == "MAIL" or interaction.type == "TWITTER" or interaction.type == "FACEBOOK" or interaction.type == "INSTAGRAM" or interaction.type == "YOUTUBE" or
		interaction.type == "WEBCONTACT" or interaction.type == "VOICEMAIL" or interaction.type == "SMS" or interaction.type == "WHATSAPP" or interaction.type == "RCS" or 
		interaction.type == "APPSTORE" or interaction.type == "GOOGLEPLAY"
end

--[[
	Leaves the interaction in a working state if possible
--]]
local function RepairBrokenInteraction(interaction)
	IncrementStat("brokenInteractions")
	if IsOffline(interaction) then
		-- Asynchronous interactions on a broken state should be requeued
		local removedAgents = {}
		local function addToList(agentList)
			for _, v in pairs(agentList) do
				table.insert(removedAgents, v)
			end
		end
		addToList(interaction.assignedAgents or {})
		addToList(interaction.reservedAgents or {})
		if m_doExecute then
			irLink:SendCommand({
				command = "STUpdateInteraction",
				proxyType = "STATETEST",
				campaignAtVCC = interaction.campaign,
				interactionID = interaction.id,
				newState = { action = "QUEUED"},
				removedAgents = removedAgents
			})
		end
		CancelRequestByInteractionOnAgentService(interaction.id)
		IncrementStat("requeuedInteractions")
	else
		-- Synchronous interactions on a broken state should be closed
		if m_doExecute then
			irLink:SendCommand({
				command = "InteractionClean",
				proxyType = "AGENT",
				interactionID = interaction.id,
			})
		end
		IncrementStat("canceledInteractions")
	end
end

local function SendPendingInteractionToSummarizer(interaction)
	irLink:SendCommand({
		command = "SendPendingInteractionToSummarizer",
		proxyType = "AGENT",
		interactionID = interaction.id,
	})
end

local function EnsuringKeySets (trans, detail,typeSet)
	
	local valueType1 = ""
	if typeSet == "Queued" then
		valueType1 = detail.startTimestampReadable.."@"..detail.id
	elseif typeSet == "Abandoned" or typeSet == "Open" then
		valueType1 = detail.id
	end
	local valueType2 = detail.id
	
	trans:zadd(typeSet.."InteractionsByCampaign:"..detail.campaign, detail.priority, valueType1)
	trans:zadd(typeSet.."InteractionsByTypeAndCampaign:"..detail.type..":"..detail.campaign, detail.priority, valueType1)
	trans:zadd(typeSet.."InteractionsByCampaignAccount:"..detail.campaign..":"..detail.account, detail.priority, valueType1)

	local attention_level = InteractionHasAttentionLevel(detail)
	if attention_level then
		trans:sadd(typeSet.."InteractionsByAttentionLevelTypeAndCampaign:"..attention_level.id..":"..detail.type..":"..detail.campaign, valueType1)
		trans:zadd(typeSet.."InteractionsByCampaignAccountAttentionLevel:"..detail.campaign..":"..detail.account..":"..attention_level.id, detail.priority, valueType1)
	else
		trans:sadd(typeSet.."InteractionsByAttentionLevelTypeAndCampaign:na:"..detail.type..":"..detail.campaign, valueType1)
		trans:zadd(typeSet.."InteractionsByCampaignAccountAttentionLevel:"..detail.campaign..":"..detail.account..":na", detail.priority, valueType1)
	end
	if detail.initiative then
		if detail.initiative == "INBOUND" then
			trans:zadd("Inbound"..typeSet.."InteractionsByTypeAndCampaign:"..detail.type..":"..detail.campaign, detail.timestampDouble or 0, valueType2)
			trans:zadd("Inbound"..typeSet.."InteractionsByCampaignAccount:"..detail.campaign..":"..detail.account, detail.timestampDouble or 0, valueType2)
			if attention_level then
				trans:zadd("Inbound"..typeSet.."InteractionsByCampaignAccountAttentionLevel:"..detail.campaign..":"..detail.account..":"..attention_level.id, detail.timestampDouble or 0, valueType2)
			end
		elseif detail.initiative == "OUTBOUND" then
			trans:zadd("Outbound"..typeSet.."InteractionsByTypeAndCampaign:"..detail.type..":"..detail.campaign, detail.timestampDouble or 0, valueType2)
			trans:zadd("Outbound"..typeSet.."InteractionsByCampaignAccount:"..detail.campaign..":"..detail.account, detail.timestampDouble or 0, valueType2)
			if attention_level then
				trans:zadd("Outbound"..typeSet.."InteractionsByCampaignAccountAttentionLevel:"..detail.campaign..":"..detail.account..":"..attention_level.id, detail.timestampDouble or 0, valueType2)
			end
		end
	end
	-- Keys To Agent
	local agents = {}
	if typeSet == "Open" then
		table.insert(agents, detail.requiredAgent)
	else
		agents = detail.reservedAgents
	end
	if #agents > 0 then
		local valueType = typeSet == "Queued" and valueType1 or valueType2
		for _, agent in ipairs(agents) do
			trans:zadd(typeSet.."InteractionsByAgent:"..agent, detail.priority, valueType)
			trans:zadd(typeSet.."InteractionsByAgentAndCampaign:"..agent..":"..detail.campaign, detail.priority, valueType)
			trans:zadd(typeSet.."InteractionsByAgentCampaignAccount:"..agent..":"..detail.campaign..":"..detail.account, detail.priority, valueType)
			trans:zadd(typeSet.."InteractionsByTypeAgentAndCampaign:"..detail.type..":"..agent..":"..detail.campaign, detail.priority, valueType)

			if attention_level then
				trans:zadd(typeSet.."InteractionsByAgentCampaignAccountAttentionLevel:"..agent..":"..detail.campaign..":"..detail.account..":"..attention_level.id, detail.priority, valueType)
			end
			
			if detail.initiative then
				if detail.initiative == "INBOUND" then
					trans:zadd("Inbound"..typeSet.."InteractionsByTypeAgentAndCampaign:"..detail.type..":"..agent..":"..detail.campaign, detail.timestampDouble or 0, valueType)
				elseif detail.initiative == "OUTBOUND" then
					trans:zadd("Outbound"..typeSet.."InteractionsByTypeAgentAndCampaign:"..detail.type..":"..agent..":"..detail.campaign, detail.timestampDouble or 0, valueType)
				end
			end
		end
	else
		trans:zadd("Unassigned"..typeSet.."InteractionsByCampaign:"..detail.campaign, detail.priority, valueType1)
		trans:zadd("Unassigned"..typeSet.."InteractionsByTypeAndCampaign:"..detail.type..":"..detail.campaign, detail.priority, valueType1)
		trans:zadd("Unassigned"..typeSet.."InteractionsByCampaignAccount:"..detail.campaign..":"..detail.account, detail.priority, valueType1)
		if attention_level then
			trans:sadd("Unassigned"..typeSet.."InteractionsByAttentionLevelTypeAndCampaign:"..attention_level.id..":"..detail.type..":"..detail.campaign, valueType1)
			trans:zadd("Unassigned"..typeSet.."InteractionsByCampaignAccountAttentionLevel:"..detail.campaign..":"..detail.account..":"..attention_level.id, detail.priority, valueType1)
		else
			trans:sadd("Unassigned"..typeSet.."InteractionsByAttentionLevelTypeAndCampaign:na:"..detail.type..":"..detail.campaign, valueType1)
			trans:zadd("Unassigned"..typeSet.."InteractionsByCampaignAccountAttentionLevel:"..detail.campaign..":"..detail.account..":na", detail.priority, valueType1)
		end
		if detail.initiative then
			if detail.initiative == "INBOUND" then
				trans:zadd("InboundUnassigned"..typeSet.."InteractionsByTypeAndCampaign:"..detail.type..":"..detail.campaign, detail.timestampDouble or 0, valueType2)
				trans:zadd("InboundUnassigned"..typeSet.."InteractionsByCampaignAccount:"..detail.campaign..":"..detail.account, detail.timestampDouble or 0, valueType2)
				if attention_level then
					trans:zadd("InboundUnassigned"..typeSet.."InteractionsByCampaignAccountAttentionLevel:"..detail.campaign..":"..detail.account..":"..attention_level.id, detail.timestampDouble or 0, valueType2)
				end
			elseif detail.initiative == "OUTBOUND" then
				trans:zadd("OutboundUnassigned"..typeSet.."InteractionsByTypeAndCampaign:"..detail.type..":"..detail.campaign, detail.timestampDouble or 0, valueType2)
				trans:zadd("OutboundUnassigned"..typeSet.."InteractionsByCampaignAccount:"..detail.campaign..":"..detail.account, detail.timestampDouble or 0, valueType2)
				if attention_level then
					trans:zadd("OutboundUnassigned"..typeSet.."InteractionsByCampaignAccountAttentionLevel:"..detail.campaign..":"..detail.account..":"..attention_level.id, detail.timestampDouble or 0, valueType2)
				end
			end
		end
	end
end	

--[[
	If an interaction does not belong to a specific set, it gets added to it
	There are no retries, if the interaction is modified during the update we just fail and move on
	(presumably if it's still broken it will be taken care of in the next run)
--]]
local function EnsureInteractionIsOnLiveSets(interaction)
	LogDebug("Repair job: Ensuring interaction %s belongs to the right live sets", interaction.id)

	if not m_doExecute then
		return
	end

	local key = "InteractionDetail:"..interaction.id
	local options = { watch = key, cas = true, retry = 0 }

	local result = pcall(redis.transaction, redis, options, function(trans)
		local res, detail = pcall(json.decode, trans:get(key))
		if not res or not detail then
			LogWarning("Repair job: Unable to get interaction %s not a valid json anymore", interaction.id)
			return
		end
		if InteractionIsClosed(detail) then
			-- Closed interactions should not be on any list
			return
		end
		-- Add to open interaction sets
		trans:multi()
		trans:sadd("AllActiveInteractions", detail.id) -- This one, unlike all the others, is an unsorted set
		trans:zadd("InteractionsByCampaign:"..detail.campaign, detail.timestampDouble or 0, detail.id)
		trans:zadd("InteractionsByTypeAndCampaign:"..detail.type..":"..detail.campaign, detail.timestampDouble or 0, detail.id)

		if InteractionIsQueued(detail) then
			EnsuringKeySets(trans, detail, "Queued")
		
		elseif InteractionIsOnStateOpen(detail) then  --OPEN Interaction, CallBacks
			EnsuringKeySets(trans, detail, "Open")
		
		elseif InteractionIsAbandoned(detail) then  --Abandoned Interaction
			EnsuringKeySets(trans, detail, "Abandoned")
			
		elseif InteractionIsSpam(detail) then
			trans:zadd("SpamInteractionsToExpire", detail.timestampDouble or 0, detail.id)
			trans:zadd("SpamInteractionsByCampaign:"..detail.campaign, detail.timestampDouble or 0, detail.id)
			trans:zadd("SpamInteractionsByTypeAndCampaign:"..detail.type..":"..detail.campaign, detail.timestampDouble or 0, detail.id)

		elseif InteractionIsAssigned(detail) then
			trans:zadd("AssignedInteractionsByCampaign:"..detail.campaign, detail.timestampDouble or 0, detail.id)
			trans:zadd("AssignedInteractionsByCampaignAccount:"..detail.campaign..":"..detail.account, detail.timestampDouble or 0, detail.id)
			trans:zadd("AssignedInteractionsByTypeAndCampaign:"..detail.type..":"..detail.campaign, detail.timestampDouble or 0, detail.id)

			local attention_level = InteractionHasAttentionLevel(detail)
			if attention_level then
				trans:zadd("AssignedInteractionsByCampaignAccountAttentionLevel:"..detail.campaign..":"..detail.account..":"..attention_level.id, detail.timestampDouble or 0, detail.id)
				trans:sadd("AssignedInteractionsByAttentionLevelTypeAndCampaign:"..attention_level.id..":"..detail.type..":"..detail.campaign, detail.id)
			else
				trans:zadd("AssignedInteractionsByCampaignAccountAttentionLevel:"..detail.campaign..":"..detail.account..":na", detail.timestampDouble or 0, detail.id)
				trans:sadd("AssignedInteractionsByAttentionLevelTypeAndCampaign:na:"..detail.type..":"..detail.campaign, detail.id)
			end

			if detail.initiative then
				if detail.initiative == "INBOUND" then
					trans:zadd("InboundAssignedInteractionsByTypeAndCampaign:"..detail.type..":"..detail.campaign, detail.timestampDouble or 0, detail.id)
					trans:zadd("InboundAssignedInteractionsByCampaignAccount:"..detail.campaign..":"..detail.account, detail.timestampDouble or 0, detail.id)
					if attention_level then
						trans:zadd("InboundAssignedInteractionsByCampaignAccountAttentionLevel:"..detail.campaign..":"..detail.account..":"..attention_level.id, detail.timestampDouble or 0, detail.id)
					end
				elseif detail.initiative == "OUTBOUND" then
					trans:zadd("OutboundAssignedInteractionsByTypeAndCampaign:"..detail.type..":"..detail.campaign, detail.timestampDouble or 0, detail.id)
					trans:zadd("OutboundAssignedInteractionsByCampaignAccount:"..detail.campaign..":"..detail.account, detail.timestampDouble or 0, detail.id)
					if attention_level then
						trans:zadd("OutboundAssignedInteractionsByCampaignAccountAttentionLevel:"..detail.campaign..":"..detail.account..":"..attention_level.id, detail.timestampDouble or 0, detail.id)
					end
				end
			end
			
			if InteractionHasQueuedState(detail) then
				detail.assignedAgents = { detail.requiredAgent } -- This is only to add it on the next loop, should not be saved as detail
			end
			for _, agent in pairs(detail.assignedAgents) do
				local composeKey = detail.id .. "#" .. detail.type  .. "#" .. detail.campaign
				trans:zadd("AssignedInteractionsByAgent:"..agent, detail.timestampDouble or 0, composeKey)
				trans:zadd("AssignedInteractionsByAgentAndCampaign:"..agent..":"..detail.campaign, detail.timestampDouble or 0, detail.id)
				trans:zadd("AssignedInteractionsByAgentCampaignAccount:"..agent..":"..detail.campaign..":"..detail.account, detail.timestampDouble or 0, detail.id)
				trans:zadd("AssignedInteractionsByTypeAgentAndCampaign:"..detail.type..":"..agent..":"..detail.campaign, detail.timestampDouble or 0, detail.id)

				if attention_level then
					trans:zadd("AssignedInteractionsByAgentCampaignAccountAttentionLevel:"..agent..":"..detail.campaign..":"..detail.account..":"..attention_level.id, detail.timestampDouble or 0, detail.id)
				end
				
				if detail.initiative then
					if detail.initiative == "INBOUND" then
						trans:zadd("InboundAssignedInteractionsByTypeAgentAndCampaign:"..detail.type..":"..agent..":"..detail.campaign, detail.timestampDouble or 0, detail.id)
					elseif detail.initiative == "OUTBOUND" then
						trans:zadd("OutboundAssignedInteractionsByTypeAgentAndCampaign:"..detail.type..":"..agent..":"..detail.campaign, detail.timestampDouble or 0, detail.id)
					end
				end
			end

		elseif InteractionIsProcessFlow(detail) then -- Process flow
			trans:zadd("ProcessFlowInteractionsByCampaign:"..detail.campaign, detail.timestampDouble or 0, detail.id)
			trans:zadd("ProcessFlowInteractionsByTypeAndCampaign:"..detail.type..":"..detail.campaign, detail.timestampDouble or 0, detail.id)
			trans:zadd("ProcessFlowInteractionsByCampaignAccount:"..detail.campaign..":"..detail.account, detail.timestampDouble or 0, detail.id)
			
			local attention_level = InteractionHasAttentionLevel(detail)
			if attention_level then
				trans:zadd("ProcessFlowInteractionsByCampaignAccountAttentionLevel:"..detail.campaign..":"..detail.account..":"..attention_level.id, detail.timestampDouble or 0, detail.id)
				trans:sadd("ProcessFlowInteractionsByAttentionLevelTypeAndCampaign:"..attention_level.id..":"..detail.type..":"..detail.campaign, detail.id)
			else
				trans:zadd("ProcessFlowInteractionsByCampaignAccountAttentionLevel:"..detail.campaign..":"..detail.account..":na", detail.timestampDouble or 0, detail.id)
				trans:sadd("ProcessFlowInteractionsByAttentionLevelTypeAndCampaign:na:"..detail.type..":"..detail.campaign, detail.id)
			end
			
			if detail.initiative then
				if detail.initiative == "INBOUND" then
					trans:zadd("InboundProcessFlowInteractionsByTypeAndCampaign:"..detail.type..":"..detail.campaign, detail.timestampDouble or 0, detail.id)
					trans:zadd("InboundProcessFlowInteractionsByCampaignAccount:"..detail.campaign..":"..detail.account, detail.timestampDouble or 0, detail.id)
					if attention_level then
						trans:zadd("InboundProcessFlowInteractionsByCampaignAccountAttentionLevel:"..detail.campaign..":"..detail.account..":"..attention_level.id, detail.timestampDouble or 0, detail.id)
					end
				elseif detail.initiative == "OUTBOUND" then
					trans:zadd("OutboundProcessFlowInteractionsByTypeAndCampaign:"..detail.type..":"..detail.campaign, detail.timestampDouble or 0, detail.id)
					trans:zadd("OutboundProcessFlowInteractionsByCampaignAccount:"..detail.campaign..":"..detail.account, detail.timestampDouble or 0, detail.id)
					if attention_level then
						trans:zadd("OutboundProcessFlowInteractionsByCampaignAccountAttentionLevel:"..detail.campaign..":"..detail.account..":"..attention_level.id, detail.timestampDouble or 0, detail.id)
					end
				end
			end

			if detail.type == "FACEBOOK" then
				local page = detail.interactionData.page
				trans:zadd("ProcessFlowInteractionsByCampaignAccountPage:"..detail.campaign..":"..detail.account..":"..page, detail.timestampDouble or 0, detail.id)
			end
			
		elseif InteractionIsWaiting(detail) then -- Waiting
			trans:zadd("WaitingInteractionsByCampaign:"..detail.campaign, detail.timestampDouble or 0, detail.id)
			trans:zadd("WaitingInteractionsByTypeAndCampaign:"..detail.type..":"..detail.campaign, detail.timestampDouble or 0, detail.id)
			trans:zadd("WaitingInteractionsByCampaignAccount:"..detail.campaign..":"..detail.account, detail.timestampDouble or 0, detail.id)
			
			local attention_level = InteractionHasAttentionLevel(detail)
			if attention_level then
				trans:zadd("WaitingInteractionsByCampaignAccountAttentionLevel:"..detail.campaign..":"..detail.account..":"..attention_level.id, detail.timestampDouble or 0, detail.id)
				trans:sadd("WaitingInteractionsByAttentionLevelTypeAndCampaign:"..attention_level.id..":"..detail.type..":"..detail.campaign, detail.id)
			else
				trans:zadd("WaitingInteractionsByCampaignAccountAttentionLevel:"..detail.campaign..":"..detail.account..":na", detail.timestampDouble or 0, detail.id)
				trans:sadd("WaitingInteractionsByAttentionLevelTypeAndCampaign:na:"..detail.type..":"..detail.campaign, detail.id)
			end
			
			if detail.initiative then
				if detail.initiative == "INBOUND" then
					trans:zadd("InboundWaitingInteractionsByTypeAndCampaign:"..detail.type..":"..detail.campaign, detail.timestampDouble or 0, detail.id)
					trans:zadd("InboundWaitingInteractionsByCampaignAccount:"..detail.campaign..":"..detail.account, detail.timestampDouble or 0, detail.id)
					if attention_level then
						trans:zadd("InboundWaitingInteractionsByCampaignAccountAttentionLevel:"..detail.campaign..":"..detail.account..":"..attention_level.id, detail.timestampDouble or 0, detail.id)
					end
				elseif detail.initiative == "OUTBOUND" then
					trans:zadd("OutboundWaitingInteractionsByTypeAndCampaign:"..detail.type..":"..detail.campaign, detail.timestampDouble or 0, detail.id)
					trans:zadd("OutboundWaitingInteractionsByCampaignAccount:"..detail.campaign..":"..detail.account, detail.timestampDouble or 0, detail.id)
					if attention_level then
						trans:zadd("OutboundWaitingInteractionsByCampaignAccountAttentionLevel:"..detail.campaign..":"..detail.account..":"..attention_level.id, detail.timestampDouble or 0, detail.id)
					end
				end
			end
		else
			LogWarning("Repair job: Interaction %s on state %s not Ensuring keys.", detail.id,detail.state)
		end
		return true
	end)
	return result
end

--[[
	For an interaction supposed to be WAITING validates it's corresponding request on the AgentService are alive and well
	If some part of the puzzle is missing cancel the conversation
--]]
function ValidateWaitingRequests(interaction)
	LogDebug("Repair job: Ensuring interaction %s is a valid WAITING conversation", interaction.id)

	local function HasRequestOnAS()
		local interactionRequests = agentredis:smembers("AgentService::InteractionRequests:"..interaction.id)
		if not interactionRequests or #interactionRequests == 0 then
			LogWarning("Repair job: No requests in AS for interaction %s", interaction.id)
			return false
		end
		for _, requestID in ipairs(interactionRequests) do
			
			local request = agentredis:hgetall("AgentService::RequestInfo:"..requestID)
			if request and request.Interaction == interaction.id then
				LogDebug("Repair job: Request %s on state %s found for WAITING conversation %s", requestID, request.Status, interaction.id)
				
				-- Primero chequeo si estoy en situacion de caida. Si es asi valido simplemente el tiempo de creacion para saber si es valido.
				local tiempoEspera = request.QueuedTime or request.AssignedTime or -1
				if esCaida and tonumber(tiempoEspera) > 0 and tonumber(tiempoEspera) < (LuaOrm.Date():time() - senconds_caida)  then
					LogDebug("Repair job: Crash state and old request. Clear interaction '%s' and request '%s'", interaction.id,requestID)
				else
					local campWithVcc = request.Campaign .. "@" .. request.VCC
					local acdWaitOnCampaign = m_camp_ACDWaitThreshold[campWithVcc][request.Type] or -1
					-- Chequeo si el tiempo de espera es superior al tiempo definido en campa�a
					if request.Status == "Waiting" and tonumber(tiempoEspera) > 0 and tonumber(acdWaitOnCampaign) > 0 and tonumber(tiempoEspera) < (LuaOrm.Date():time() - acdWaitOnCampaign)  then
						LogDebug("Repair job: Waiting time is longer than the one defined in the campaign. Clear interaction '%s' and request '%s'", interaction.id,requestID)
					else
						--if waiting or accepted less than ten minutes ago
						if request.Status == "Waiting" or ((request.Status == "Accepted" or request.Status == "TransactionalAccepted") and tonumber(request.AssignedTime) > LuaOrm.Date():time() - SECONDS_IN_TEN_MINUTES) then
							return true
						else
							LogWarning("Repair job: Requests %s of interaction %s not fetching, it's %s, assigned since %s", requestID, interaction.id, tostring(request.Status), tostring(request.AssignedTime))
						end
					end
				end
			else
				LogWarning("Repair job: Requests %s in AS should be associated with interaction %s, but it's not", requestID, interaction.id)
			end
		end
		return false
	end

	local validRequest = HasRequestOnAS()
	if validRequest then 
		return true
	end

	-- Check the interaction is still waiting. It may be it already moved away.
	local res, detail = pcall(json.decode, redis:get("InteractionDetail:"..interaction.id))
	if not res or not detail then
		LogWarning("Repair job: Unable to get interaction %s not a valid json anymore", interaction.id)
		return false
	end
	if detail.state ~= "WAITING" then
		return false -- It's not waiting anymore
	end

	if not detail.inSchedule and 
	redis:hget("OutOfHoursUnAssignedInteractions", detail.campaign .. "::" .. detail.type .. "::" .. detail.id) then
		LogWarning("Repair job: Waiting interaction %s, not inSchedule, will be assigned in another job.", interaction.id)
		return true -- Se asignara en otro job
	end
	
	-- Si es de tipo CALL y hay Integracion OCC consulto contra Allegro.
	if interaction.type == "CALL" and detail.state == "WAITING" then
		if allegroProxy.HasIntegration(detail.VCC) then
			local info = allegroProxy.DoGet("resources/interaction/get_status/" .. detail.id, detail.VCC)
			if info then
				--LogDebug("Repair job: Info obtenida desde Allegro: %s",info)
				local ok, jsonInfo = pcall(json.decode,info)
				if ok then
					if jsonInfo.status and jsonInfo.interaction then
						local interAllegro = jsonInfo.interaction
						if interAllegro.State and interAllegro.State == "ABANDONED" then
							LogWarning("Repair job: Allegro Interaction '%s' is ABANDONED.Set it..",detail.id)
							--Si esta Abandoned y y obtuve datos de OMRequestErrors Actualizo con eso.
							if m_doExecute then
								local ok, jsonData = pcall(json.decode,interAllegro.JsonData)
								if not ok then
									jsonData = {}
								end
								irLink:SendCommand({
									command = "UpdateAllegroInteraction",
									proxyType = "CALL",
									campaignAtVCC = interAllegro.Campaign .. interAllegro.Vcc,
									interactionID = detail.id,
									data = jsonData,

									timestamp = tonumber(interAllegro.UnixTimeStamp),
									state = interAllegro.State,
									agent = interAllegro.Agent,
									VCC = detail.VCC
								})
							end
						else
							if interAllegro.EdoCan0245  ~= "ESTABLISHED" and interAllegro.EdoAct0245 ~= "QUEUED" then
								--La ejecucion contra Allegro fue correcta pero la interaccion no esta en Waiting.
								IncrementStat("danglingWaits")
								RepairBrokenInteraction(detail)
							end
						end
					end
				end
			end
		else --Si es call, no es de ALLEGRO y no hay un request en AS entonces retorno false.
			if not validRequest then
				LogWarning("Repair job: Call interaction '%s' is waiting without valid request on AS.", interaction.id)
				IncrementStat("danglingWaits")
				RepairBrokenInteraction(detail)
				return false
			end
		end
		return true
	end
	
	LogWarning("Repair job: Waiting interaction %s has no agent requests on agent service. Repairing...", interaction.id)
	IncrementStat("danglingWaits")
	RepairBrokenInteraction(detail)
	return false
end

local function ResetStats()
	m_runningStats = {
		processedInteractions = 0,
		undecodableInteractions = 0,
		ttlSets = 0,
		brokenInteractions = 0,
		canceledInteractions = 0,
		requeuedInteractions = 0,
		outOfSetInteractions = 0,
		danglingWaits = 0,
		invalidSets = 0,
		danglingInteractions = 0,
		repairedSets = 0,
		elapsedTime = 0,
		danglingWaits = 0,
		cancelledRequests = 0,
		sendToSummarizer = 0
	}
end

local function PrintRunStats()
	local f_time = function(seconds)
		return string.format("%02d", math.floor(seconds / 3600)) .. ":" ..
					 string.format("%02d", math.floor((seconds % 3600) / 60)) .. ":" ..
					 string.format("%02d", math.floor(seconds % 60))
	end
	local f_number = function(number)
		return string.format("% 9d", number)
	end
	LogDebug("Repair job: Finished successfully.")
	LogDebug([[_
						INTERACTION REPAIR STATS:
						----------------------------
						Processed Interactions:    %s
						Undecodable Interactions:  %s
						Finished w/o Expiration:   %s
						Broken Interactions:       %s
						Canceled Interactions:     %s
						Requeued Interactions:     %s
						Out of Set Interactions:   %s
						Dangling WAITS:            %s
						Invalid Schema Sets:       %s
						Dangling Interactions:     %s
						Summarizer Interactions:   %s
						Analized Sets:             %s

						Cancelled Requests:        %s

						Elapsed Time: %s
					]], f_number(m_runningStats.processedInteractions),
							f_number(m_runningStats.undecodableInteractions),
							f_number(m_runningStats.ttlSets),
							f_number(m_runningStats.brokenInteractions),
							f_number(m_runningStats.canceledInteractions),
							f_number(m_runningStats.requeuedInteractions),
							f_number(m_runningStats.outOfSetInteractions),
							f_number(m_runningStats.danglingWaits),
							f_number(m_runningStats.invalidSets),
							f_number(m_runningStats.danglingInteractions),
							f_number(m_runningStats.sendToSummarizer),
							f_number(m_runningStats.repairedSets),
							f_number(m_runningStats.cancelledRequests),
							f_time(m_runningStats.elapsedTime))
end

--[[
	This function iterates over the complete InteractionDetail:* keys and checks for the following situations
	* FINISHED and CANCELED interactions should have an expire TTL set
	* TAKEN and TRANSFER TAKEN conversations should have an owner and belong to all the corresponding sets
	* QUEUED and SPAM conversations should belong to all the corresponding sets
	* WAITING should have a request on the Agent Service
	* Interactions in other states for more than a day are flagged as broken

	- Broken interactions are CANCELED if live, QUEUED if offline
	- Interactions not belonging to their corresponding set get added right away
--]]
local function ValidateInteractions()
	local processWaitingForTimeout = redis:zrangebyscore("InstanceProcessTimeoutEvents", "-inf", "+inf")
	local allInteractions = redis:keys("InteractionDetail:*")
	for _, interactionIDKey in pairs(allInteractions) do
		IncrementStat("processedInteractions")
		local res, detail = pcall(json.decode, redis:get(interactionIDKey))
		if not res or not detail then
			LogError("Repair job: Interaction %s is undecodable, removing from keys", interactionIDKey)
			if m_doExecute then
				redis:del(interactionIDKey)
			end
			IncrementStat("undecodableInteractions")
		else
			LogDebug("Repair job: processing interaction %s on state %s last modified on %s", interactionIDKey, detail.state, detail.timestamp)
			local modified = LuaOrm.Date(detail.timestamp)
			local isRepeared = false
			local now = LuaOrm.Date()
			if InteractionIsClosed(detail) then
				if detail.pendingSummarizer then
					if now:time() - modified:time() > SECONDS_IN_EIGHT_HOUR then
						LogError("Repair job: Interaction %s from campaign %s was finalized but is pending to be sent to summarizer, sending it", detail.id, detail.campaign)
						SendPendingInteractionToSummarizer(detail)
						IncrementStat("sendToSummarizer")
					end
				elseif redis:ttl(interactionIDKey) == -2 then
					LogError("Repair job: Interaction %s from campaign %s was finalized but had no TTL, expiring on 1 day", detail.id, detail.campaign)
					if m_doExecute then
						redis:expire(interactionIDKey, SECONDS_IN_A_DAY)
					end
					IncrementStat("ttlSets")
				end
			elseif detail.state == "TAKEN" or detail.state == "TRANSFER TAKEN" or detail.state == "CONFERENCE TAKEN" or detail.state == "CONFERENCE" then
				if not next(detail.assignedAgents) then
					LogWarning("Repair job: Interaction %s on state %s has no assigned agent, repairing", detail.id, detail.state)
					RepairBrokenInteraction(detail)
					isRepeared = true 
				else
					-- Si la interaccion es sincronica y lleva mas de 8 horas en TAKEN se cancela. Tambien se cancela si estoy en situacion de caida y el tiempo supera lo pasado por parametro.
					if ((detail.type == "CHAT" or detail.type == "CALL") and ( (now:time() - modified:time() > SECONDS_IN_EIGHT_HOUR) or (esCaida and (now:time() - modified:time()  > senconds_caida)) ) ) then
						LogWarning("Repair job: Interaction %s from campaign %s and type %s was on state %s for %s seconds, repairing.", detail.id, detail.campaign, detail.type, detail.state, now:time() - modified:time())
						RepairBrokenInteraction(detail)
						isRepeared = true 
					else
						EnsureInteractionIsOnLiveSets(detail)
					end
				end
			elseif detail.state == "WAITING" then
				if ValidateWaitingRequests(detail) then
					EnsureInteractionIsOnLiveSets(detail)
				else
					isRepeared = true 
				end
			elseif detail.state == "QUEUED" then
				-- Si es CALL o CHAT y lleva 2 horas se cancela 
				if ( (detail.type == "CHAT" or detail.type == "CALL") and ( (now:time() - modified:time() > SECONDS_IN_TWO_HOUR) ) ) then
					LogWarning("Repair job: Interaction %s from campaign %s and type %s was on state %s for %s seconds, repairing.", detail.id, detail.campaign, detail.type, detail.state, now:time() - modified:time())
					RepairBrokenInteraction(detail)
					isRepeared = true 
				else
					EnsureInteractionIsOnLiveSets(detail)
				end
			elseif detail.state == "SPAM" or detail.state == "OPEN" or detail.state == "ABANDONED" then
				EnsureInteractionIsOnLiveSets(detail)
			else
				local waitingForTimeOut = false
				if detail.state == "CREATED" then
					--Busco si la interaccion esta en un Bot esperando por un mensaje.
					local entryProcess = detail.type .. ":" .. detail.id
					for _, entry in pairs(processWaitingForTimeout) do
						if entry == entryProcess then
							waitingForTimeOut = true
							break
						end
					end
				end		
				if not waitingForTimeOut and ((now:time() - modified:time() > SECONDS_IN_ONE_HOUR) or ( esCaida and (now:time() - modified:time()  > senconds_caida) )) then
					LogError("Repair job: Interaction %s from campaign %s and type %s was in intermediate state %s for %s seconds, repairing", detail.id, detail.campaign, detail.type, detail.state, now:time() - modified:time())
					RepairBrokenInteraction(detail)
					isRepeared = true 
				else
					EnsureInteractionIsOnLiveSets(detail)
				end
			end
			-- Luego de validar el status en redis voy a validar a ElasticSearch
			if not isRepeared then
				ValidateElasticSearch(detail.id)
			end
			
		end
	end
	CollectAllMemory("ValidateInteractions")
end


--[[
	Recorro los rows de Elastic Search que se encuentran en un estado Activo: WAITING, QUEUED, TAKEN 
	Y valido que esten en el estado correcto.
--]]
local function RepairInteractionOnElasticSearch ()
	--Para cada VCC recorro las interacciones en ES
	local esConnectionConfig = inconcert.environment.apps.agent.elasticsearch

	for _, vccId in pairs(m_vccs) do
		LogDebug("Repair job: processing interactions for vcc %s on Elastic Search", vccId)
		
		local query = {
				query = {
					bool = {
						filter = { }
					}
				}
			}
		
		local channels = { terms = { ["type"] = validChannels } }
		local states  = { terms = { ["state"] = {"WAITING","QUEUED","LOCKED","TAKEN","WRAPUP","ABANDONED","OPEN","TRANSFER QUEUED"} } }
		table.insert(query.query.bool.filter, channels)
		table.insert(query.query.bool.filter, states)
		--Primero valido si son mas de 10000. Si es asi obtengo un PIT
		local ok, resultCount = utils.GetElasticSearchInstance():CountElasticSearch(query, vccId, esConnectionConfig)
		local pitID 
		local withPIT = false
		if ok then
			local countRows = json.decode(resultCount)
			if countRows.count and countRows.count > 9000 then
				-- si tengo mas de 9000 entonces utilizo PIT
				LogDebug("Repair job: Count documents for vcc '%s' : %s . Get PIT.",vccId, countRows.count)
				ok, pitID = utils.GetElasticSearchInstance():PitElasticSearch(vccId, esConnectionConfig)
				if not ok then
					LogError("Repair job: Error get PIT from ElasticSearch.")
				else
					withPIT = true
				end
			end
		end
		
		query.fields = {"state"}
		query.size = 1000
		if withPIT then
			query.sort = {}
			query.sort["_shard_doc"] = {}
			query.sort["_shard_doc"].order = "asc"
			query.pit = {}
			query.pit.id = pitID
			query.pit.keep_alive = "3m"
		else
			query.from = 0
		end
		local remainingInteraction = true
		local totalRow = 0
		local searchAfter = ""
		while remainingInteraction do
			if withPIT and searchAfter ~= "" then
				query.search_after = searchAfter
			end
			local ok, resultQuery = search_elastic_search(query,vccId,withPIT)
			searchAfter = ""
			if ok then
				local result = json.decode(resultQuery)
				if result.hits then
					for i=1, #result.hits.hits do
						local interact = result.hits.hits[i]
						if pitID then
							searchAfter = interact.sort
						end
						totalRow = totalRow + 1
						if interact then
							local interactionId = interact._id
							local interactionState = ""
							for _, state in pairs(interact.fields.state) do
								interactionState = state
							end
							LogDebug("Repair job: ES processing interaction %s on state %s ", interactionId, interactionState)
							
							local findInteraction = false
							local findOnCassandra = false
							local interactionDetail = {}
							-- Busco en Redis
							local stringData = redis:get("InteractionDetail:"..interactionId)
							--print("stringData:"..stringData)
							if not stringData or #stringData == 0 then
								-- Si no esta lo busco en Cassandra.
								interactionOnCassandra = InteractionOnCassandra(interactionId)
								if interactionOnCassandra then
									interactionDetail = interactionOnCassandra
									findInteraction = true
									findOnCassandra = true
								end
							else
								interactionDetail = json.decode(stringData)
								findInteraction = true
							end
							if findInteraction and interactionDetail and interactionDetail.state then
								local originalState = interactionDetail.state
								local newModified = LuaOrm.Date(interactionDetail.timestamp)
								local now = LuaOrm.Date()
								-- Si la interaccion existe, esta en estado Abandoned y no esta en Redis, tengo que borrarla.
								if originalState == "ABANDONED" and findOnCassandra then
									local index_name = ("vcc-%s"):format(assert(interactionDetail.VCC))
									local urlString = "/"..index_name.."/_doc/"..interactionDetail.id
									local ok, response_code, response_headers, response_status_line
									local response_body = {}
									ok, response_code, response_headers, response_status_line = http.request({
										host = inconcert.environment.apps.agent.elasticsearch.host,
										port = inconcert.environment.apps.agent.elasticsearch.port,
										url = urlString,
										method = "DELETE",
										sink = ltn12.sink.table(response_body),
										redirect = false
									})
								else
									if validStatesOnES[originalState] and originalState ~= interactionState and (now:time() - newModified:time() > 300) then
										LogDebug("Repair job: InteractionID: %s, Interaction type: %s Redis state: %s, ElasticSearch state: %s. Update interaction on ElasticSearch", interactionDetail.id, interactionDetail.type, interactionDetail.state,interactionState)
										local options = {}
										if interactionDetail.type == "CHAT" then
											-- Si el tipo de interaccion es CHAT tengo que obtener lo conversado del repositorio.
											local content = {}
											local result, conversation = GetFromRepository(interactionDetail.id, 0, 1)

											if result then
												--LogDebug("Repair job: Retrieved conversation log: %s", conversation)
												conversation = json.decode(conversation)
												for _, message in ipairs(conversation.messages) do
													table.insert(content, message.text)
												end
											else
												LogWarning("Repair job: Could not obtain conversation with id '%s'", interactionDetail.id)
												if interactionDetail.interactionData.firstMessage then
													table.insert(content, interactionDetail.interactionData.firstMessage)
												end
											end
											options.additional_content = table.concat(content, " ")
										end
										if m_doExecute then
											irLink:SendCommand({
												command = "StoreInElasticSearch",
												proxyType = "AGENT",
												interaction = interactionDetail,
												options = options
											})
										end
									end
								end
							else
								LogDebug ("Repair job: interaction %s not exists. Set INCONSISTENT state on ES.", interactionId)
								local index_name = ("vcc-%s"):format(assert(vccId))
								local response_body = {}
								local ok, response_code, response_headers, response_status_line
								
								ok, response_code, response_headers, response_status_line = http.request({
									host = inconcert.environment.apps.agent.elasticsearch.host,
									port = inconcert.environment.apps.agent.elasticsearch.port,
									url = ("/%s/_doc/%s"):format(index_name, interactionId),
									headers = { ["Content-Type"] = "application/json" },
									method = "GET",
									sink = ltn12.sink.table(response_body),
									redirect = false
								})
								if not ok then
									LogError("Repair job: Error obtaining interaction from Elasstic Search. %s\n%s\n%s\n%s", response_code, json.encode(response_headers), response_status_line,json.encode(response_body))
								else
									response_body = table.concat(response_body)
									if response_headers["content-type"]:match("^application/json") then
										local jsonResponse = json.decode(response_body)
										local existInteraction = jsonResponse.found
										local jsonInteraction = jsonResponse._source
										if existInteraction and jsonInteraction  then
											jsonInteraction.state = "INCONSISTENT"
											response_body = {}
											local string_data = json.encode(jsonInteraction)
											ok, response_code, response_headers, response_status_line = http.request({
												host = inconcert.environment.apps.agent.elasticsearch.host,
												port = inconcert.environment.apps.agent.elasticsearch.port,
												url = ("/%s/_doc/%s"):format(index_name,interactionId),
												headers = { ["Content-Length"] = tostring(#string_data);["Content-Type"] = "application/json" },
												method = "POST",
												source = ltn12.source.string(string_data),
												sink = ltn12.sink.table(response_body),
												redirect = false
											})
											if not ok then
												LogError("Repair job: Error setting interaction to Elasstic Search. %s\n%s\n%s\n%s", response_code, json.encode(response_headers), response_status_line,json.encode(response_body))
											end
										end
									else	
										LogError("Repair job: Error obtaining interaction from Elasstic Search. %s\n%s\n%s\n%s", response_code, json.encode(response_headers), response_status_line,json.encode(response_body))
									end
								end
							end
						end
					end
					if #result.hits.hits < 500 then
						remainingInteraction = false
					else
						if not pitID then
							query.from = query.from + query.size + 1
						end
					end
				else
					remainingInteraction = false
				end
				result = nil
			else
				LogError("Repair job: Error get interactions from Elastic Search: %s",resultQuery)
				remainingInteraction = false
			end
			if totalRow > 50000 then
				-- Solo loopeo hasta 50.000 para no generar problemas en ES.
				remainingInteraction = false
				LogWarning("Repair job: ES documents exceed 50,000. vcc: %s", vccId)
			end
			resultQuery = nil
		end
		LogDebug ("Repair job: Processing '%d' documents for vcc: %s", totalRow,vccId)
		
	end
	CollectAllMemory("RepairInteractionOnElasticSearch")
end


--[[
	The core of the logic is:
	1) Validate all the iteractions are by themselves valid, if not, clean them up
	2) Validate all the sets have valid and not-dangling interactions, if not, clean them up
--]]
local function RepairInteractions()
	LogDebug("Repair job: Interaction repair job validating interactions...")
	ValidateInteractions()
	LogDebug("Repair job: Interaction repair job repairing interaction sets...")
	RepairInteractionSets()
	LogDebug("Repair job: Interaction repair job repairing interaction on ElasticSearch...")
	RepairInteractionOnElasticSearch()
	CollectAllMemory("RepairInteractions")
end

local function RepairAgentService()
	--[[
		AgentService::RequestInfo:REQUEST is a hash that keeps track of many request attributes.
		Requests have the following fields. Unless noted on the field, they are required.

		* Interaction: Interaction ID that originated the request. The validity of the request can be verified by consulting InteractionDetail:ID.
		               The request should be on set AgentService::InteractionRequests:INTERACTION (as REQUEST).
		* Type: Channel type.
		* Campaign: Campaign ID (without VCC)
		* VCC: VCC ID
		* CreationDate: Date when the request was created, which is used to prioritize the request on the waiting queues. On "YYYY-mm-dd HH:MM:SS.FFF" format.
		                This field is essential if Status is Waiting, because is part of the element on AgentService::RequestQueue:TYPE:CAMPAIGN@VCC.
		* Status: Status of the request. Possible statuses are Waiting, Accepted, Rejected, Canceled and Finished.
		          - Waiting: The request is waiting for an agent. This should map up with an opened interaction.
		                     Requests on Waiting should also be on a sorted set AgentService::RequestQueue:TYPE:CAMPAIGN@VCC (as CREATIONDATE@REQUEST, score -PRIORITY),
		                     and on set AgentService::PendingRequests (as REQUEST).
		          - Accepted: The request is assigned to the agent. This should map up with an interaction on TAKEN, LOCKED, WRAPUP or TRANSFER* (or WAITING or RESERVED for a brief time).
		                      Requests on Accepted should also be on a hash AgentService::AgentInteractionRequests:AGENT@VCC (key INTERACTION, value REQUEST),
		                      on set AgentService::AgentAttentions:AGENT@VCC (as REQUEST),
		                      and on set AgentService::AssignedRequests (as REQUEST).
		          - Rejected|Canceled|Finished: The request is closed, for different causes. This doesn't need to map with the interaction, because
		                                        the interaction may be attended by another agent, but the RequestInfo itself should be expiring.
		* AssignedAgent: Only valid (and required) when Status is Accepted. The ID (without VCC) of the agent attending the request.
		* Priority: Integer with the priority of the request. Higher values go first on waiting queues. Only relevant if Status is Waiting. If it's missing,
		            it may ne assumed to be 0.

		The following fields are irrelevant to the repair process:

		* AssignedUsingAddress: Only valid when Status is Accepted. If present (value doesn't matter), it means that the agent is using an address with this request.
		* AssignedAgentAddress: Only valid when AssignedUsingAddress exists. The address used.
		* AssignedAgentResourceAddress: Only valid when AssignedUsingAddress exists. The resource where the address used exists.
		* CloseReason: Only valid when Status is Rejected or Canceled. Explains the reason why it was rejected/canceled.
		* DedicationAtEnqueuing: This is used for statistics, and means how many agents were paying attention when this request entered the waiting queue.
		                         This field may not exist, in which case it means it never entered the waiting queue (was assigned upon request).
		* EnqueuedPosition: In which place this request was placed when it entered the waiting queue.
		                         This field may not exist, in which case it means it never entered the waiting queue (was assigned upon request).
		* QueuedTime: Seconds since epoch of the time where the request entered the waiting queue.
		              This field may not exist, in which case it means it never entered the waiting queue (was assigned upon request).
		* AssignedTime: Same as QueuedTime, but of the time when the request was assigned. Since all requests are assigned once (if assigned), it may not exist
		                if the request was never (or not yet) assigned.
		* OnlyFromList: If present (value doesn't matter), it means that it can only be assigned to agents on AgentService::RequestAgents:REQUEST.
		                If not present, it means that it can be assigned to any agent BUT those on AgentService::RequestAgents:REQUEST.
		* OverrideBreak: If present (value doesn't matter) and the request is on the waiting queue, it may be taken by agents on a state that's overrideable (like BUSY).
		* ClientIP/ClientPort: Two fields that indicate the IR that is waiting for a reply on this request.
		* InWrapup: It means that the interaction is in wrapup, which considers time differently than when it's not.

		Along with AgentService::RequestInfo:REQUEST, AgentService::RequestAgents:REQUEST is a set of agents which meaning depends on the field 'OnlyFromList'.
		This key may not exist if the RequestInfo counterpart does not, same with its expiring. A RequestInfo may not have RequestAgents, though.
	]]

	LogDebug("Repair job: Repairing agent requests")

	local allRequests = agentredis:keys("AgentService::RequestInfo:*")
	for _, requestKey in pairs(allRequests) do
		local request = requestKey:match("^AgentService::RequestInfo:(.*)$")
		LogDebug("Repair job: Checking request %s...", request)
		local options = { watch = requestKey, cas = true, retry = 0 }

		pcall(agentredis.transaction, agentredis, options, function(trans)
			local requestInfo = trans:hgetall(requestKey)
			if not next(requestInfo) then
				return true -- It was removed after watching
			end

			local function RemoveRequest()
				IncrementStat("cancelledRequests")
				if not m_doExecute then
					return true
				end
				trans:multi()
				trans:del(requestKey)
				trans:del("AgentService::RequestAgents:"..request)
				trans:srem("AgentService::PendingRequests", request)
				trans:srem("AgentService::AssignedRequests", request)
				if requestInfo.Interaction then
					trans:srem("AgentService::InteractionRequests:"..requestInfo.Interaction, request)
				end
				if requestInfo.CreationDate and requestInfo.Type and requestInfo.Campaign and requestInfo.VCC then
					trans:zrem("AgentService::RequestQueue:"..requestInfo.Type..":"..requestInfo.Campaign.."@"..requestInfo.VCC, requestInfo.CreationDate.."@"..request)
				end
				if requestInfo.AssignedAgent and requestInfo.VCC then
					trans:srem("AgentService::AgentAttentions:"..requestInfo.AssignedAgent.."@"..requestInfo.VCC, request)
				end
				return true
			end

			local neededFields = { "Interaction", "Type", "Campaign", "VCC", "Status" }
			for _, field in ipairs(neededFields) do
				if not requestInfo[field] then
					LogError("Repair job: Request %s lacks %s, removing.", request, field)
					return RemoveRequest()
				end
			end
			local interaction = requestInfo.Interaction
			local channel = requestInfo.Type
			local campaign = requestInfo.Campaign
			local VCC = requestInfo.VCC
			local status = requestInfo.Status

			if status == "Waiting" then
				if not requestInfo.CreationDate then
					LogError("Repair job: Request %s is waiting, but it has no CreationDate, removing.", request)
					return RemoveRequest()
				end

				local result, interactionDetail = pcall(json.decode, redis:get("InteractionDetail:"..interaction)) -- This doesn't use trans, because it queries the regular redis
				if not result or type(interactionDetail) ~= "table" or not interactionDetail.state then
					LogError("Repair job: Unable to get interaction %s for request %s, cancelling.", interaction, request)
					CancelRequestByIdOnAgentService(request)
					return true
				end

				if interactionDetail.state == "CANCELED" or interactionDetail.state == "FINISHED" then
					LogError("Repair job: Request %s is not consistent with closed interaction %s, cancelling.", request, interaction)
					CancelRequestByIdOnAgentService(request)
					return true
				end

				if not m_doExecute then
					return true
				end
				trans:multi()
				trans:persist(requestKey)
				trans:persist("AgentService::RequestAgents:"..request)
				trans:sadd("AgentService::InteractionRequests:"..requestInfo.Interaction, request)

				-- if priority is other than 0, negate it, because that is how it is stored in the sorted set
				-- don't negate 0, because it yields -0 and screws everything in the AgentService
				local priority = tonumber(requestInfo.Priority) or 0
				priority = (priority ~= 0 and -priority or priority)
				
				trans:zadd("AgentService::RequestQueue:"..channel..":"..campaign.."@"..VCC, priority, requestInfo.CreationDate.."@"..request)
				trans:sadd("AgentService::PendingRequests", request)
				return true
			end

			if status == "Accepted" or status == "TransactionalAccepted" then
				if not requestInfo.AssignedAgent then
					LogError("Repair job: Request %s is assigned, but it has no AssignedAgent, removing.", request)
					return RemoveRequest()
				end

				local validStates = {
					["LOCKED"] = true,
					["TAKEN"] = true,
					["TRANSFER LOCKED"] = true,
					["TRANSFER TAKEN"] = true,
					["CONFERENCE LOCKED"] = true,
					["CONFERENCE TAKEN"] = true,
					["CONFERENCE"] = true,
					["WRAPUP"] = true,
				}
				local invalidStates = {
					["QUEUED"] = true,
					["CANCELED"] = true,
					["FINISHED"] = true
				}

				-- It may take a bit for an interaction to update to the next status after a request is accepted,
				-- so we retry the checking until a valid (or invalid) state is reached, or it times out after 6 seconds.
				local rightDetail
				for i = 1, 30 do
					local result, interactionDetail = pcall(json.decode, redis:get("InteractionDetail:"..interaction)) -- This doesn't use trans, because it queries the regular redis

					if not result or type(interactionDetail) ~= "table" or not interactionDetail.state then
						LogError("Repair job: Unable to get interaction %s for request %s, cancelling.", interaction, request)
						CancelRequestByIdOnAgentService(request)
						return true
					end

					if validStates[interactionDetail.state] then
						rightDetail = interactionDetail
						break
					end

					if invalidStates[interactionDetail.state] then
						break
					end
					
					socket.sleep(0.2)
				end

				if not rightDetail then
					LogError("Repair job: Request %s is not consistent with the state of interaction %s, cancelling.", request, interaction)
					CancelRequestByIdOnAgentService(request)
					return true
				end

				local agentOnInteraction = false
				for _, reservedAgent in ipairs(rightDetail.reservedAgents) do
					if reservedAgent == requestInfo.AssignedAgent.."@"..VCC then
						agentOnInteraction = true
						break
					end
				end
				if not agentOnInteraction then
					for _, assignedAgent in ipairs(rightDetail.assignedAgents) do
						if assignedAgent == requestInfo.AssignedAgent.."@"..VCC then
							agentOnInteraction = true
							break
						end
					end
				end
				if not agentOnInteraction then
					LogError("Repair job: Request %s is not consistent with the reserved/assigned agents of interaction %s, cancelling.", request, interaction)
					CancelRequestByIdOnAgentService(request)
					return true
				end

				if not m_doExecute then
					return true
				end
				trans:multi()
				trans:persist(requestKey)
				trans:persist("AgentService::RequestAgents:"..request)
				trans:sadd("AgentService::InteractionRequests:"..requestInfo.Interaction, request)
				trans:sadd("AgentService::AgentInteractionRequests:"..requestInfo.AssignedAgent.."@"..requestInfo.VCC, requestInfo.Interaction, request)
				trans:sadd("AgentService::AgentAttentions:"..requestInfo.AssignedAgent.."@"..requestInfo.VCC, request)
				trans:sadd("AgentService::AssignedRequests", request)
				return true
			end

			-- It's canceled, rejected or finished
			if not m_doExecute then
				return true
			end
			local expiring = trans:ttl(requestKey)
			trans:multi()
			if not expiring then
				trans:expire(requestKey, 3600)
				trans:expire("AgentService::RequestAgents:"..request, 3600)
			end
			trans:srem("AgentService::PendingRequests", request)
			trans:srem("AgentService::AssignedRequests", request)
			trans:srem("AgentService::InteractionRequests:"..requestInfo.Interaction, request)
			trans:zrem("AgentService::RequestQueue:"..requestInfo.Type..":"..requestInfo.Campaign.."@"..requestInfo.VCC, requestInfo.CreationDate.."@"..request)
			if requestInfo.AssignedAgent and requestInfo.VCC then
				trans:srem("AgentService::AgentAttentions:"..requestInfo.AssignedAgent.."@"..requestInfo.VCC, request)
			end
			return true
		end)
	end
	
	--[[
		After repairing all the RequestInfo, we should look into all the hashes, sets and sorted sets where they may be, and check their contents are valid.

		* AgentService::InteractionRequests:INTERACTION    - set, contains REQUEST of Waiting or Accepted requests
		* AgentService::RequestQueue:TYPE:CAMPAIGN@VCC     - sorted set, contains CREATIONDATE@REQUEST of Waiting requests
		* AgentService::PendingRequests                    - set, contains REQUEST of Waiting requests
		* AgentService::AgentInteractionRequests:AGENT@VCC - hash, maps INTERACTION to REQUEST of Accepted requests
		* AgentService::AgentAttentions:AGENT@VCC          - set, contains REQUEST of Accepted requests
		* AgentService::AssignedRequests                   - set, contains REQUEST of Accepted requests
		* AgentService::RequestAgents:REQUEST              - set, should only exist if the request exists.
	]]

	-- AgentService::InteractionRequests:INTERACTION    - set, contains REQUEST of Waiting or Accepted requests
	local allInteractionRequests = agentredis:keys("AgentService::InteractionRequests:*")
	for _, interactionRequestsKey in pairs(allInteractionRequests) do
		local interaction = interactionRequestsKey:match("^AgentService::InteractionRequests:(.*)$")
		LogDebug("Repair job: Checking requests of interaction %s...", interaction)
		local options = { watch = interactionRequestsKey, cas = true, retry = 3 }

		pcall(agentredis.transaction, agentredis, options, function(trans)
			local requests = trans:smembers(interactionRequestsKey)
			local requestsToRemove = {}

			local function CheckForRemoval(request)
				local requestInteraction = trans:hget("AgentService::RequestInfo:"..request, "Interaction")
				if interaction ~= requestInteraction then
					LogError("Repair job: Request %s is listed as a request of interaction %s, but its info points to interaction %s.", request, interaction, tostring(requestInteraction))
					return true
				end
				local requestStatus = trans:hget("AgentService::RequestInfo:"..request, "Status")
				if requestStatus ~= "Waiting" and requestStatus ~= "Accepted" and requestStatus ~= "TransactionalAccepted" then
					LogError("Repair job: Request %s is listed as a request of interaction %s, but since the status is %s, it shouldn't be listed.", request, interaction, tostring(requestStatus))
					return true
				end
			end

			for _, request in pairs(requests) do
				if CheckForRemoval(request) then
					table.insert(requestsToRemove, request)
				end
			end
			if m_doExecute and #requestsToRemove > 0 then
				trans:multi()
				for _, request in ipairs(requestsToRemove) do
					trans:srem(interactionRequestsKey, request)
				end
			end
			return true
		end)
	end

	-- AgentService::RequestQueue:TYPE:CAMPAIGN@VCC     - sorted set, contains CREATIONDATE@REQUEST of Waiting requests
	local allRequestQueues = agentredis:keys("AgentService::RequestQueue:*:*@*")
	for _, requestQueueKey in pairs(allRequestQueues) do
		local channel, campaign, VCC = requestQueueKey:match("^AgentService::RequestQueue:([^:]*):(.*)@([^@]*)$")
		LogDebug("Repair job: Checking request queue of %s at campaign %s@%s...", channel, campaign, VCC)
		local options = { watch = requestQueueKey, cas = true, retry = 3 }

		pcall(agentredis.transaction, agentredis, options, function(trans)
			local dateRequests = trans:zrange(requestQueueKey, 0, -1)
			local dateRequestsToRemove = {}

			local function CheckForRemoval(dateRequest)
				local date, request = dateRequest:match("^(.*)@(.*)$")
				if not date or not request then
					LogError("Repair job: Date@Request %s is not on the right format.", dateRequest)
					return true
				end
				local requestChannel = trans:hget("AgentService::RequestInfo:"..request, "Type")
				if channel ~= requestChannel then
					LogError("Repair job: Request %s is listed as a request of type %s, but its info says type %s.", request, tostring(channel), tostring(requestChannel))
					return true
				end
				local requestCampaign = trans:hget("AgentService::RequestInfo:"..request, "Campaign")
				if campaign ~= requestCampaign then
					LogError("Repair job: Request %s is listed as a request of campaign %s, but its info says campaign %s.", request, tostring(campaign), tostring(requestCampaign))
					return true
				end
				local requestVCC = trans:hget("AgentService::RequestInfo:"..request, "VCC")
				if VCC ~= requestVCC then
					LogError("Repair job: Request %s is listed as a request of VCC %s, but its info says VCC %s.", request, tostring(VCC), tostring(requestVCC))
					return true
				end
				local requestStatus = trans:hget("AgentService::RequestInfo:"..request, "Status")
				if requestStatus ~= "Waiting" then
					LogError("Repair job: Request %s is listed as a request waiting, but since the status is %s, it shouldn't be listed.", request, tostring(requestStatus))
					return true
				end
			end

			for _, dateRequest in pairs(dateRequests) do
				if CheckForRemoval(dateRequest) then
					table.insert(dateRequestsToRemove, dateRequest)
				end
			end
			if m_doExecute and #dateRequestsToRemove > 0 then
				trans:multi()
				for _, dateRequest in ipairs(dateRequestsToRemove) do
					trans:zrem(requestQueueKey, dateRequest)
				end
			end
			return true
		end)
	end

	-- AgentService::PendingRequests                    - set, contains REQUEST of Waiting requests
	LogDebug("Repair job: Checking pending requests...")
	local pendingRequestsKey = "AgentService::PendingRequests"
	local options = { watch = pendingRequestsKey, cas = true, retry = 3 }

	pcall(agentredis.transaction, agentredis, options, function(trans)
		local requests = trans:smembers(pendingRequestsKey)
		local requestsToRemove = {}

		local function CheckForRemoval(request)
			local requestStatus = trans:hget("AgentService::RequestInfo:"..request, "Status")
			if requestStatus ~= "Waiting" then
				LogError("Repair job: Request %s is listed as a pending request, but since the status is %s, it shouldn't be listed.", request, tostring(requestStatus))
				return true
			end
		end

		for _, request in pairs(requests) do
			if CheckForRemoval(request) then
				table.insert(requestsToRemove, request)
			end
		end
		if m_doExecute and #requestsToRemove > 0 then
			trans:multi()
			for _, request in ipairs(requestsToRemove) do
				trans:srem(pendingRequestsKey, request)
			end
		end
		return true
	end)

	-- AgentService::AgentInteractionRequests:AGENT@VCC - hash, maps INTERACTION to REQUEST of Accepted requests
	local allAgentInteractionRequests = agentredis:keys("AgentService::AgentInteractionRequests:*@*")
	for _, agentInteractionRequestsKey in pairs(allAgentInteractionRequests) do
		local agent, VCC = agentInteractionRequestsKey:match("^AgentService::AgentInteractionRequests:(.*)@([^@]*)$")
		LogDebug("Repair job: Checking requests assigned to %s@%s by interaction...", agent, VCC)
		local options = { watch = agentInteractionRequestsKey, cas = true, retry = 3 }

		pcall(agentredis.transaction, agentredis, options, function(trans)
			local interactionRequests = trans:hgetall(agentInteractionRequestsKey)
			local interactionsToRemove = {}

			local function CheckForRemoval(interaction, request)
				local requestInteraction = trans:hget("AgentService::RequestInfo:"..request, "Interaction")
				if interaction ~= requestInteraction then
					LogError("Repair job: Request %s is listed as a request of interaction %s, but its info points to interaction %s.", request, interaction, tostring(requestInteraction))
					return true
				end
				local requestVCC = trans:hget("AgentService::RequestInfo:"..request, "VCC")
				if VCC ~= requestVCC then
					LogError("Repair job: Request %s is listed as a request of VCC %s, but its info points to VCC %s.", request, VCC, tostring(requestVCC))
					return true
				end
				local requestStatus = trans:hget("AgentService::RequestInfo:"..request, "Status")
				if requestStatus ~= "Accepted" and requestStatus ~= "TransactionalAccepted" then
					LogError("Repair job: Request %s is listed as an assigned request, but since the status is %s, it shouldn't be listed.", request, tostring(requestStatus))
					return true
				end
				local requestAgent = trans:hget("AgentService::RequestInfo:"..request, "AssignedAgent")
				if agent ~= requestAgent then
					LogError("Repair job: Request %s is listed as a request of agent %s, but its info points to agent %s.", request, agent, tostring(requestAgent))
					return true
				end
			end

			for interaction, request in pairs(interactionRequests) do
				if CheckForRemoval(interaction, request) then
					table.insert(interactionsToRemove, interaction)
				end
			end
			if m_doExecute and #interactionsToRemove > 0 then
				trans:multi()
				for _, interaction in ipairs(interactionsToRemove) do
					trans:hdel(agentInteractionRequestsKey, interaction)
				end
			end
			return true
		end)
	end

	-- AgentService::AgentAttentions:AGENT@VCC          - set, contains REQUEST of Accepted requests
	local allAgentRequests = agentredis:keys("AgentService::AgentAttentions:*@*")
	for _, agentRequestsKey in pairs(allAgentRequests) do
		local agent, VCC = agentRequestsKey:match("^AgentService::AgentAttentions:(.*)@([^@]*)$")
		LogDebug("Repair job: Checking requests assigned to %s@%s...", agent, VCC)
		local options = { watch = agentRequestsKey, cas = true, retry = 3 }

		pcall(agentredis.transaction, agentredis, options, function(trans)
			local requests = trans:smembers(agentRequestsKey)
			local requestsToRemove = {}

			local function CheckForRemoval(request)
				local requestVCC = trans:hget("AgentService::RequestInfo:"..request, "VCC")
				if VCC ~= requestVCC then
					LogError("Repair job: Request %s is listed as a request of VCC %s, but its info points to VCC %s.", request, VCC, tostring(requestVCC))
					return true
				end
				local requestStatus = trans:hget("AgentService::RequestInfo:"..request, "Status")
				if requestStatus ~= "Accepted" and requestStatus ~= "TransactionalAccepted" then
					LogError("Repair job: Request %s is listed as an assigned request, but since the status is %s, it shouldn't be listed.", request, tostring(requestStatus))
					return true
				end
				local requestAgent = trans:hget("AgentService::RequestInfo:"..request, "AssignedAgent")
				if agent ~= requestAgent then
					LogError("Repair job: Request %s is listed as a request of agent %s, but its info points to agent %s.", request, agent, tostring(requestAgent))
					return true
				end
			end

			for _, request in pairs(requests) do
				if CheckForRemoval(request) then
					table.insert(requestsToRemove, request)
				end
			end
			if m_doExecute and #requestsToRemove > 0 then
				trans:multi()
				for _, request in ipairs(requestsToRemove) do
					trans:srem(agentRequestsKey, request)
				end
			end
			return true
		end)
	end

	-- AgentService::AssignedRequests                   - set, contains REQUEST of Accepted requests
	LogDebug("Repair job: Checking assigned requests...")
	local assignedRequestsKey = "AgentService::AssignedRequests"
	local options = { watch = assignedRequestsKey, cas = true, retry = 3 }

	pcall(agentredis.transaction, agentredis, options, function(trans)
		local requests = trans:smembers(assignedRequestsKey)
		local requestsToRemove = {}

		local function CheckForRemoval(request)
			local requestStatus = trans:hget("AgentService::RequestInfo:"..request, "Status")
			if requestStatus ~= "Accepted" and requestStatus ~= "TransactionalAccepted" then
				LogError("Repair job: Request %s is listed as an assigned request, but since the status is %s, it shouldn't be listed.", request, tostring(requestStatus))
				return true
			end
		end

		for _, request in pairs(requests) do
			if CheckForRemoval(request) then
				table.insert(requestsToRemove, request)
			end
		end
		if m_doExecute and #requestsToRemove > 0 then
			trans:multi()
			for _, request in ipairs(requestsToRemove) do
				trans:srem(assignedRequestsKey, request)
			end
		end
		return true
	end)

	-- AgentService::RequestAgents:REQUEST              - set, should only exist if the request exists.
	local allRequestAgents = agentredis:keys("AgentService::RequestAgents:*")
	for _, requestAgentsKey in pairs(allRequestAgents) do
		local request = requestAgentsKey:match("^AgentService::RequestAgents:(.*)$")
		LogDebug("Repair job: Checking request %s, which has agents listed...", request)
		local options = { watch = requestAgentsKey, cas = true, retry = 3 }

		pcall(agentredis.transaction, agentredis, options, function(trans)
			if not trans:exists("AgentService::RequestInfo:"..request) then
				LogError("Repair job: Request %s has agents listed, but the request itself does not exist.", request)
				if m_doExecute then
					trans:multi()
					trans:del(requestAgentsKey, request)
				end
			end
			return true
		end)
	end

	--[[
		Some Agent Service keys on Redis contain counters related to how many requests are in a particular situation.
		A more experienced Redis user would not have done those, but would have counted collections instead, since counting collections in Redis is
		just as fast as getting string values, and in case of inconsistencies, it's easier to get what's missing.
		A refactoring is in order. Meanwhile, this job makes sure the numbers on the counters are consistent with the elements on the collections. 

		* AgentService::AgentAddress:TYPE:AGENT@VCC - This hash keeps the address of TYPE associated with the agent. It has some keys, but the only one
		                                              important here is "References", which indicates how many interactions are using this address.
		                                              This is done in order to prevent an agent from switching addresses while using them.
		                                              This job should make sure that the number of references equals the number of requests of this type
		                                              this agent has assigned, but only those with "AssignedAgentAddress" and "AssignedAgentResourceAddress"
		                                              on "AgentService::RequestInfo:REQUEST" that match "Address" and "ResourceAddress" on this hash.
		* AgentService::AgentBusySlots:TYPE:AGENT@VCC - This string counts how many requests of channel TYPE are assigned to the agent.
		* AgentService::AgentBusySlotsOnCampaign:AGENT@VCC:CAMPAIGN@VCC - Same as AgentBusySlots, but separated by campaign instead of channel.
		* AgentService::AgentWrapupSlotsOnCampaign:AGENT@VCC:CAMPAIGN@VCC - Same as AgentBusySlotsOnCampaign, but only the requests with "InWrapup" on their
		                                                                    RequestInfo. Since it's a subset of AgentBusySlotsOnCampaign, the number must be
		                                                                    smaller or equal.
		* AgentService::AgentSyncBusySlots:AGENT@VCC - Same as AgentBusySlots, for synchronous types (i.e. IsOffline returns false for those types).
	]]

	-- Collect all keys and separate them by agent.
	local keysByAgent = {}

	local function ExtractKeysByAgent(keyPattern, keyMatching, keyType)
		local keys = agentredis:keys(keyPattern)
		for _, key in pairs(keys) do
			local agent, VCC = key:match(keyMatching)
			keysByAgent[VCC] = keysByAgent[VCC] or {}
			keysByAgent[VCC][agent] = keysByAgent[VCC][agent] or {}
			-- This key is set for agent and VCC. When collecting data, all keys with data will be updated, while existing keys without data will be dealt with.
			-- The nature of the updating (or dealing with) depends on the keyType.
			keysByAgent[VCC][agent][key] = keyType
		end
	end

	ExtractKeysByAgent("AgentService::AgentAddress:*:*@*", "^AgentService::AgentAddress:[^:]*:(.*)@([^@]*)$", "AgentAddress")
	ExtractKeysByAgent("AgentService::AgentBusySlots:*:*@*", "^AgentService::AgentBusySlots:[^:]*:(.*)@([^@]*)$", "Counter")
	ExtractKeysByAgent("AgentService::AgentBusySlotsOnCampaign:*@*:*@*", "^AgentService::AgentBusySlotsOnCampaign:(.*)@([^@]*):[^:]*@[^@]*$", "Counter")
	ExtractKeysByAgent("AgentService::AgentWrapupSlotsOnCampaign:*@*:*@*", "^AgentService::AgentWrapupSlotsOnCampaign:(.*)@([^@]*):[^:]*@[^@]*$", "Counter")
	ExtractKeysByAgent("AgentService::AgentBusySlotsOnCampaignByInitiative:*@*:*@*", "^AgentService::AgentBusySlotsOnCampaignByInitiative:(.*)@([^@]*):[^:]*@[^@]*$", "Counter")
	ExtractKeysByAgent("AgentService::AgentWrapupSlotsOnCampaignByInitiative:*@*:*@*", "^AgentService::AgentWrapupSlotsOnCampaignByInitiative:(.*)@([^@]*):[^:]*@[^@]*$", "Counter")
	ExtractKeysByAgent("AgentService::AgentSyncBusySlots:*@*", "^AgentService::AgentSyncBusySlots:(.*)@([^@]*)$", "Counter")
	ExtractKeysByAgent("AgentService::AgentAttentions:*@*", "^AgentService::AgentAttentions:(.*)@([^@]*)$", "Collection")

	-- Iterate through all the agents, checking AgentService::AgentAttentions:*@*, counting requests for each indicator and deleting dangling ones.

	for VCC, VCCAgents in pairs(keysByAgent) do
		for agent, keys in pairs(VCCAgents) do
			local agentRequestsKey = "AgentService::AgentAttentions:"..agent.."@"..VCC
			LogDebug("Repair job: Regenerating AgentService indicators for agent %s@%s...", agent, VCC)
			local options = { watch = agentRequestsKey, cas = true, retry = 3 }

			pcall(agentredis.transaction, agentredis, options, function(trans)
				local existingKeys = FlatCopy(keys) -- Copied, since due to CAS this may be retried.
				local requests = trans:smembers(agentRequestsKey)
				local addresses = {}
				local busySlots = {}
				local busySlotsOnCampaign = {}
				local wrapupSlotsOnCampaign = {}
				
				local busySlotsOnCampaignByInitiative = {}
				local wrapupSlotsOnCampaignByInitiative = {}
				
				local syncBusySlots = 0
				local requestsToDeleteAddress = {}

				for _, request in pairs(requests) do
					local interactionId = trans:hget("AgentService::RequestInfo:"..request, "Interaction")
					local channel = trans:hget("AgentService::RequestInfo:"..request, "Type")
					local campaign = trans:hget("AgentService::RequestInfo:"..request, "Campaign")
					local inWrapup = trans:hexists("AgentService::RequestInfo:"..request, "InWrapup")
					local usingAddress = trans:hexists("AgentService::RequestInfo:"..request, "AssignedUsingAddress")
					if usingAddress then
						local address = trans:hget("AgentService::RequestInfo:"..request, "AssignedAgentAddress")
						local resourceAddress = trans:hget("AgentService::RequestInfo:"..request, "AssignedAgentResourceAddress")
						if not addresses[channel] then
							addresses[channel] = {
								address = trans:hget("AgentService::AgentAddress:"..channel..":"..agent.."@"..VCC, "Address"),
								resourceAddress = trans:hget("AgentService::AgentAddress:"..channel..":"..agent.."@"..VCC, "ResourceAddress")
							}
						end
						if address == addresses[channel].address and resourceAddress == addresses[channel].resourceAddress then
							addresses[channel].references = (addresses[channel].references or 0) + 1
						else
							LogWarning("Repair job: Agent %s@%s has address %s@%s registered for channel %s, but request %s says it's using address %s@%s. Cleaning address of request.",
									   agent, VCC, tostring(address), tostring(resourceAddress), channel, request, tostring(addresses[channel].address), tostring(addresses[channel].resourceAddress))
							table.insert(requestsToDeleteAddress, request)
						end
					end
					busySlots[channel] = (busySlots[channel] or 0) + 1
					busySlotsOnCampaign[campaign] = busySlotsOnCampaign[campaign] or {}
					busySlotsOnCampaign[campaign][channel] = (busySlotsOnCampaign[campaign][channel] or 0) + 1
					if inWrapup then
						wrapupSlotsOnCampaign[campaign] = wrapupSlotsOnCampaign[campaign] or {}
						wrapupSlotsOnCampaign[campaign][channel] = (wrapupSlotsOnCampaign[campaign][channel] or 0) + 1
					end
					if not IsOffline(channel) then
						syncBusySlots = syncBusySlots + 1
					end
					
					local result, interactionDetail = pcall(json.decode, redis:get("InteractionDetail:"..interactionId)) -- This doesn't use trans, because it queries the regular redis
					
					if result and type(interactionDetail) == "table" then
						local initiative = interactionDetail.initiative
						
						busySlotsOnCampaignByInitiative[campaign] = busySlotsOnCampaignByInitiative[campaign] or {}
						busySlotsOnCampaignByInitiative[campaign][channel..":"..initiative] = (busySlotsOnCampaignByInitiative[campaign][channel..":"..initiative] or 0) + 1
						
						wrapupSlotsOnCampaignByInitiative[campaign] = wrapupSlotsOnCampaignByInitiative[campaign] or {}
						wrapupSlotsOnCampaignByInitiative[campaign][channel..":"..initiative] = (wrapupSlotsOnCampaignByInitiative[campaign][channel..":"..initiative] or 0) + 1
					end				
					
				end

				if m_doExecute then
					trans:multi()
					for channel, address in pairs(addresses) do
						local key = "AgentService::AgentAddress:"..channel..":"..agent.."@"..VCC
						trans.hset(key, "References", tostring(address.references or 0))
						existingKeys[key] = nil
					end
					for channel, slots in pairs(busySlots) do
						local key = "AgentService::AgentBusySlots:"..channel..":"..agent.."@"..VCC
						trans:set(key, tostring(slots))
						existingKeys[key] = nil
					end
					for campaign, data in pairs(busySlotsOnCampaign) do
						local key = "AgentService::AgentBusySlotsOnCampaign:"..agent.."@"..VCC..":"..campaign.."@"..VCC
						existingKeys[key] = nil
						for channel, slots in pairs(data) do
							trans:hset(key, channel ,tostring(slots))
						end
					end
					for campaign, data in pairs(busySlotsOnCampaignByInitiative) do
						local key = "AgentService::AgentBusySlotsOnCampaignByInitiative:"..agent.."@"..VCC..":"..campaign.."@"..VCC
						existingKeys[key] = nil
						for channel, slots in pairs(data) do
							trans:hset(key, channel ,tostring(slots))
						end
					end
					for campaign, data in pairs(wrapupSlotsOnCampaign) do
						local key = "AgentService::AgentWrapupSlotsOnCampaign:"..agent.."@"..VCC..":"..campaign.."@"..VCC
						existingKeys[key] = nil
						for channel, slots in pairs(data) do
							trans:hset(key, channel, tostring(slots))
						end
					end
					for campaign, data in pairs(wrapupSlotsOnCampaignByInitiative) do
						local key = "AgentService::AgentWrapupSlotsOnCampaignByInitiative:"..agent.."@"..VCC..":"..campaign.."@"..VCC
						existingKeys[key] = nil
						for channel, slots in pairs(data) do
							trans:hset(key, channel, tostring(slots))
						end
					end
					local key = "AgentService::AgentSyncBusySlots:"..agent.."@"..VCC
					trans:set(key, tostring(syncBusySlots))
					existingKeys[key] = nil
					for key, keyType in pairs(existingKeys) do
						if keyType == "AgentAddress" then
							trans:hdel(key, "References")
						elseif keyType == "Counter" then
							trans:del(key)
						end
					end
				end
				return true
			end)
		end
	end
	
	LogDebug("Repair job: Repairing QueueStatistics keys")
	local allQueueStatistics = agentredis:keys("AgentService::QueueStatistics:*")
	for _, queueStatistics in pairs(allQueueStatistics) do
		local options = { watch = queueStatistics, cas = true, retry = 3 }
		pcall(agentredis.transaction, agentredis, options, function(trans)
			local samples = trans:hget(queueStatistics, "Samples")
			local advanceSquareSum = trans:hget(queueStatistics, "AdvanceSquareSum")
			local advanceSum = trans:hget(queueStatistics, "AdvanceSum")
			if samples then
				LogDebug("Repair job: Analize key: %s. Samples: %s, AdvanceSquareSum: %s, AdvanceSum: %s",queueStatistics, samples, advanceSquareSum, advanceSum)
				if tonumber(samples) > 100000 or tonumber(advanceSquareSum) < 0 or tonumber(advanceSum) < 0 then
					LogDebug("Repair job: Reset samples on key: %s",queueStatistics)
					if m_doExecute then
						trans:multi()
						trans:hset(queueStatistics, "Samples","0")
						trans:hset(queueStatistics, "AdvanceSquareSum","0")
						trans:hset(queueStatistics, "AdvanceSum","0")
					end
				end
			end
			return true
		end)
	end
	CollectAllMemory("RepairAgentService")
end 


local function ProcessCampaignUser(campaign, user)
	local campaignID = campaign:match("^(.*)@[^@]*$")
	local userID = user:match("^(.*)@[^@]*$")
	local timeStamp = os.time()
	-- Chequeo status del agente
	local stateAgent = agentredis:hget("AgentService::Agent:" .. user, "State");
	if stateAgent == "OFFLINE" then
		if not m_RedisCampaignAgents[campaign] then
			m_RedisCampaignAgents[campaign] = {}
			local campAgents =  agentredis:smembers("AgentService::CampaignAgents:"..campaign)
			for _,agent in pairs(campAgents) do
				m_RedisCampaignAgents[campaign][agent] = true 
			end
		end
		if not m_RedisCampaignAgents[campaign][userID] then
			LogDebug("Repair job: [ProcessCampaignUser] El usuario: %s NO pertenece a la campaña: %s. Key: 'AgentService::CampaignAgents:*' ", userID, campaign )
			agentredis:sadd("AgentService::CampaignAgents:"..campaign, userID)
			agentredis:sadd("AgentService::AgentCampaigns:"..user, campaignID)
			agentredis:hset("AgentService::AgentCampaignTime:"..user, campaignID, tostring(timeStamp));
			agentredis:hset("AgentService::AgentActiveCampaignActiveTime:"..user, campaignID, tostring(timeStamp));
		end
		if not m_RedisAgentCampaigns[user] then
			m_RedisAgentCampaigns[user] = {}
			--LogDebug("Repair job: [REDIS] Obtengo las campañas del usuario: %s", user)
			local agentCamp = agentredis:smembers("AgentService::AgentCampaigns:"..user)
			for _,camp in pairs(agentCamp) do
				m_RedisAgentCampaigns[user][camp] = true 
			end
		end
		if not m_RedisAgentCampaigns[user][campaignID] then
			LogDebug("Repair job: [ProcessCampaignUser] El usuario: %s NO tiene asignada la campaña : %s. Key: 'AgentService::AgentCampaigns:*' ", user, campaign )
			agentredis:sadd("AgentService::AgentCampaigns:"..user, campaignID)
		end
	end
end

local function RepairMembersOfCampaign()
	for campaign, members in pairs(m_usersByCampaign) do
		--LogDebug("Repair job: Cheching members of campaign: %s.",campaign) 
		for _, user in pairs(members) do
			ProcessCampaignUser(campaign,user)
		end
	end
end

local S3_INTERACTIONS_BATCH_SIZE = 50

local function GetSendingInteractionsToS3Step()
	local nextInteractions =
		redis:hrandfield("PendingInteractionsS3Process", S3_INTERACTIONS_BATCH_SIZE, "withvalues")
	return nextInteractions or {}
end

local function ReprocessSendingInteractionsToS3Step()
	local nextInteractions = GetSendingInteractionsToS3Step()
	if utils.TableIsEmpty(nextInteractions) then
		LogDebug("Repair job: [ReprocessSendingInteractionsToS3Step] - > no pending interactions.")
		return false
	end

	local result = irLink:SendCommand({
		command = "ReprocessSendingInteractionsToS3",
		proxyType = "AGENT",
		interactions = nextInteractions
	}) or { result = "FAIL", reason = "unknown error" }
	if result.result ~= "OK" then
		LogError("Repair job: [ReprocessSendingInteractionsToS3Step] - > abort on fail: %s.",
				 result.reason)
		return false
	end

	LogDebug("Repair job: [ReprocessSendingInteractionsToS3Step] - > %s.",
			 json.encode(result))
	return true
end

local function ReprocessSendingInteractionsToS3()
	while ReprocessSendingInteractionsToS3Step() do
		LogDebug("Repair job: [ReprocessSendingInteractionsToS3] - > iterate sending...")
	end
end

function RunScriptCode(force)
	ResetStats()
	local start = os.time()

	redis = inconcert.environment.apps.jobs.redis
	agentredis = inconcert.environment.apps.jobs.agentredis
	cassandra = inconcert.environment.apps.jobs.cassandra

	if not force and not utils.CanRunJob(redis, "RepairAll", SECONDS_IN_A_DAY) then
		return "{}"
	end

	LogDebug("Repair job: Interaction repair job loading all campaigns...")
	LoadAllCampaigns()
	LogDebug("Repair job: Interaction repair job loading all users...")
	LoadAllUsers()
	LogDebug("Repair job: Interaction repair job loading all vccs...")
	LoadAllVCCs()
	RepairAgentService()
	LogDebug("Repair job: Check Members of Campaigns...")
	RepairMembersOfCampaign()
	RepairInteractions()

	ReprocessSendingInteractionsToS3()

	local ends = os.time()
	m_runningStats.elapsedTime = ends - start
	PrintRunStats()

	--lend the GC a hand and cleanup this stuff for the memory to be freed
	m_campaigns = nil
	m_camp_accounts = nil
	m_users = nil
	m_usersByCampaign = nil
	m_vccs = nil
	m_camp_ACDWaitThreshold = nil
	--return result, usefull for tests
	return json.encode(m_runningStats, true)
end

function RepairAll(web)

	local params = web.GET or {}
	if params and params.tiempo then
		local tiempo = tonumber(params.tiempo)
		if tiempo then
			senconds_caida = params.tiempo * 60
			esCaida = true
			LogDebug("Repair job: Parametros recibos desde la web: senconds_caida : %s", senconds_caida)
		end
	end
	
	--irLink = irLink.CirLink:new(inconcert.environment.apps.agent.cluster)
	irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	if not inconcert.environment.apps.jobs.irLink then
		LogError("Repair job: Not InteractionRouter detected")
		return false
	else
		local ir_host, ir_port = string.match(inconcert.environment.apps.jobs.irLink, "(.*):(.*)")
		LogDebug("Repair job: ir_host: %s, ir_port: %s",ir_host,ir_port)
	
		local result = RunScriptCode(not params.scheduled)
	
		CollectAllMemory("RepairAll")
		return result
	end
end
