module(..., package.seeall, inconcert.orbit.application.Decorate)

local PM = require("provisioningmanager")
local utils = require "shared.apps.utils"
local Synchronize = require "Synchronize"
local https = Synchronize("luasec_mutex", function()
	return require ("ssl.https")
end)
local ltn12 = require("ltn12")
local apiQnAMaker = require "qna_maker" -- rocka

m_htmlRender = nil
m_appLocation = nil

local function GetAllKnowledgeBases (web)
	local vcc = inconcert.authUser.virtualContactCenter

	local qnaMakerSubscriptionsKeysV2, qnaMakerTotalSubscriptionsV2 = apiQnAMaker.LoadVCCSubscriptionsV2(vcc)

	local vccApiKeyQnaV2
	local vccBaseUriQnaV2

	if qnaMakerTotalSubscriptionsV2 == 0 then
		LogInfo("[KNOWLEDGE_BASE.QNA_V2:LoadVccConfig] VCC '%s' has no assigned QnA Maker V2 Subscriptions keys.", vcc)
	else
		local key, value = next(qnaMakerSubscriptionsKeysV2)
		vccBaseUriQnaV2 = value.subscriptionName
		vccApiKeyQnaV2 = value.subscriptionKey
	end

	local baseUri = [[https://]].. vccBaseUriQnaV2 ..[[/rasa/api/qna/knowledgebase]]
	local verb = "GET"
	local apikey = vccApiKeyQnaV2
	local uri = string.format(baseUri)
	
	if web.GET.page then
		uri = uri ..string.format("?page=%s", web.GET.page)
	end	
	
	if web.GET.size then
		uri = uri ..string.format("&size=%s", web.GET.size)
	end
	
	if web.GET.order then
		uri = uri ..string.format("&order=%s", web.GET.order)
	end
	
	LogInfo("[KNOWLEDGE_BASE.QNA_V2:GetAllKnowledgeBases] Requesting to url [%s] %s and apikey %s", verb, uri, apikey)

	local response = {}
	local result, code, headers, statusline = https.request{
		url = uri,
		method = verb,
		headers = {
			["apikey"] = apikey,
			["Content-Type"] = "application/json"
		},
		sink = ltn12.sink.table(response)
	}

	LogInfo("[KNOWLEDGE_BASE.QNA_V2:GetAllKnowledgeBases] Requested to url [%s] %s and apikey %s with status %s", verb, uri, apikey, code)

     web.status = code
	 return table.concat(response)
end

local function SearchKnowledgeBases (web, filter)

	local vcc = inconcert.authUser.virtualContactCenter
	local qnaMakerSubscriptionsKeysV2, qnaMakerTotalSubscriptionsV2 = apiQnAMaker.LoadVCCSubscriptionsV2(vcc)

	local vccApiKeyQnaV2
	local vccBaseUriQnaV2

	if qnaMakerTotalSubscriptionsV2 == 0 then
		LogInfo("[KNOWLEDGE_BASE.QNA_V2:SearchKnowledgeBases] VCC '%s' has no assigned QnA Maker V2 Subscriptions keys.", vcc)
	else
		local key, value = next(qnaMakerSubscriptionsKeysV2)
		vccBaseUriQnaV2 = value.subscriptionName
		vccApiKeyQnaV2 = value.subscriptionKey
	end

	local baseUri = [[https://]].. vccBaseUriQnaV2 ..[[/rasa/api/qna/knowledgebase?page=0&size=1000]]
	local verb = "GET"
	local apikey = vccApiKeyQnaV2
	local uri = string.format(baseUri)
	
	LogInfo("[KNOWLEDGE_BASE.QNA_V2:SearchKnowledgeBases] Requesting to url [%s] %s and apikey %s", verb, uri, apikey)

	local response = {}
	local result, code, headers, statusline = https.request{
		url = uri,
		method = verb,
		headers = {
			["apikey"] = apikey,
			["Content-Type"] = "application/json"
		},
		sink = ltn12.sink.table(response)
	}

	LogInfo("[KNOWLEDGE_BASE.QNA_V2:SearchKnowledgeBases] Requested to url [%s] %s and apikey %s with status %s", verb, uri, apikey, code)

	local encodedResponse = table.concat(response)
	local parsedResponse = json.decode(encodedResponse)
	local responseFiltered = {}

	for item_index, item in pairs(parsedResponse.data) do
		if string.find(string.lower(item.name), string.lower(filter), 1, true) then
			table.insert(responseFiltered, item)
		end
	end

	web.status = code
	return { data = responseFiltered, count = #responseFiltered }
end

local function GetKnowledgeBasesById (web, kbId)
	local vcc = inconcert.authUser.virtualContactCenter

	local qnaMakerSubscriptionsKeysV2, qnaMakerTotalSubscriptionsV2 = apiQnAMaker.LoadVCCSubscriptionsV2(vcc)

	local vccApiKeyQnaV2
	local vccBaseUriQnaV2

	if qnaMakerTotalSubscriptionsV2 == 0 then
		LogInfo("[KNOWLEDGE_BASE.QNA_V2:LoadVccConfig] VCC '%s' has no assigned QnA Maker V2 Subscriptions keys.", vcc)
	else
		local key, value = next(qnaMakerSubscriptionsKeysV2)
		vccBaseUriQnaV2 = value.subscriptionName
		vccApiKeyQnaV2 = value.subscriptionKey
	end

	local baseUri = [[https://]].. vccBaseUriQnaV2 ..[[/rasa/api/qna/knowledgebase/]].. kbId
	local verb = "GET"
	local apikey = vccApiKeyQnaV2
	local uri = string.format(baseUri)
	
	LogInfo("[KNOWLEDGE_BASE.QNA_V2:GetKnowledgeBasesById] Requesting to url [%s] %s and apikey %s", verb, uri, apikey)

	local response = {}
	local result, code, headers, statusline = https.request{
		url = uri,
		method = verb,
		headers = {
			["apikey"] = apikey,
			["Content-Type"] = "application/json"
		},
		sink = ltn12.sink.table(response)
	}

	LogInfo("[KNOWLEDGE_BASE.QNA_V2:GetKnowledgeBasesById] Requested to url [%s] %s and apikey %s with status %s", verb, uri, apikey, code)

     web.status = code
	 return table.concat(response)
end

local function AddKnowledgeBase (web)
	local vcc = inconcert.authUser.virtualContactCenter

	local ok, newBase = pcall(json.decode, web.input["post_data"])
	local qnaMakerSubscriptionsKeysV2, qnaMakerTotalSubscriptionsV2 = apiQnAMaker.LoadVCCSubscriptionsV2(vcc)

	local vccApiKeyQnaV2
	local vccBaseUriQnaV2

	if qnaMakerTotalSubscriptionsV2 == 0 then
		LogInfo("[KNOWLEDGE_BASE.QNA_V2:AddKnowledgeBase] VCC '%s' has no assigned QnA Maker V2 Subscriptions keys.", vcc)
	else
		local key, value = next(qnaMakerSubscriptionsKeysV2)
		vccBaseUriQnaV2 = value.subscriptionName
		vccApiKeyQnaV2 = value.subscriptionKey
	end

	local baseUri = [[https://]].. vccBaseUriQnaV2 ..[[/rasa/api/qna/knowledgebase?eventStream=disabled]]
	local verb = "POST"
	local apikey = vccApiKeyQnaV2
	local uri = string.format(baseUri)
	local jsonDataStr = json.encode(newBase)
	
	LogInfo("[KNOWLEDGE_BASE.QNA_V2:AddKnowledgeBase] Requesting to url [%s] %s and apikey %s", verb, uri, apikey)

	local response = {}
	local result, code, headers, statusline = https.request{
		url = uri,
		method = verb,
		headers = {
			["apikey"] = apikey,
			["Content-Type"] = "application/json",
			["Content-Length"] = #jsonDataStr,
		},
		source = ltn12.source.string(jsonDataStr),
		sink = ltn12.sink.table(response)
	}

	LogInfo("[KNOWLEDGE_BASE.QNA_V2:AddKnowledgeBase] Requested to url [%s] %s and apikey %s with status %s", verb, uri, apikey, code)

     web.status = code
	 return table.concat(response)
end

local function UpdateKnowledgeBase (web)
	local vcc = inconcert.authUser.virtualContactCenter

	local ok, newBase = pcall(json.decode, web.input["post_data"])
	local qnaMakerSubscriptionsKeysV2, qnaMakerTotalSubscriptionsV2 = apiQnAMaker.LoadVCCSubscriptionsV2(vcc)

	local vccApiKeyQnaV2
	local vccBaseUriQnaV2

	if qnaMakerTotalSubscriptionsV2 == 0 then
		LogInfo("[KNOWLEDGE_BASE.QNA_V2:UpdateKnowledgeBase] VCC '%s' has no assigned QnA Maker V2 Subscriptions keys.", vcc)
	else
		local key, value = next(qnaMakerSubscriptionsKeysV2)
		vccBaseUriQnaV2 = value.subscriptionName
		vccApiKeyQnaV2 = value.subscriptionKey
	end

	local baseUri = [[https://]].. vccBaseUriQnaV2 ..[[/rasa/api/qna/knowledgebase?eventStream=disabled]]
	local verb = "PUT"
	local apikey = vccApiKeyQnaV2
	local uri = string.format(baseUri)
	local jsonDataStr = json.encode(newBase)
	
	LogInfo("[KNOWLEDGE_BASE.QNA_V2:UpdateKnowledgeBase] Requesting to url [%s] %s and apikey %s", verb, uri, apikey)

	local response = {}
	local result, code, headers, statusline = https.request{
		url = uri,
		method = verb,
		headers = {
			["apikey"] = apikey,
			["Content-Type"] = "application/json",
			["Content-Length"] = #jsonDataStr,
		},
		source = ltn12.source.string(jsonDataStr),
		sink = ltn12.sink.table(response)
	}

	LogInfo("[KNOWLEDGE_BASE.QNA_V2:UpdateKnowledgeBase] Requested to url [%s] %s and apikey %s with status %s", verb, uri, apikey, code)

     web.status = code
	 return table.concat(response)
end

local function AddKnowledgeBaseFromVideo (web)
	
	 local vcc = inconcert.authUser.virtualContactCenter
	 local qnaMakerSubscriptionsKeysV2, qnaMakerTotalSubscriptionsV2 = apiQnAMaker.LoadVCCSubscriptionsV2(vcc)
 
	 local vccApiKeyQnaV2
	 local vccBaseUriQnaV2
 
	 if qnaMakerTotalSubscriptionsV2 == 0 then
		 LogInfo("[KNOWLEDGE_BASE.QNA_V2:AddKnowledgeBaseFromVideo] VCC '%s' has no assigned QnA Maker V2 Subscriptions keys.", vcc)
	 else
		 local key, value = next(qnaMakerSubscriptionsKeysV2)
		 vccBaseUriQnaV2 = value.subscriptionName
		 vccApiKeyQnaV2 = value.subscriptionKey
	 end
 
	 local baseUri = [[https://]].. vccBaseUriQnaV2 ..[[/rasa/api/qna/knowledgebase/video?eventStream=disabled]]
	 local verb = "POST"
	 local apikey = vccApiKeyQnaV2
	 local uri = string.format(baseUri)
 
	 LogInfo("[KNOWLEDGE_BASE.QNA_V2:AddKnowledgeBaseFromVideo] Requesting to url [%s] %s and apikey %s", verb, uri, apikey)
 
	 local args = utils.ParseQueryString(web.vars.QUERY_STRING)

	 local response = {}
	 local result, code, headers, statusline = https.request{
		 url = uri,
		 method = verb,
		 headers = {
			 ["apikey"] = apikey,
			 ["Content-Type"] = "video/mp4",
			 ["Content-Length"] = tostring(#web.input["post_data"]),
			 ["name"] = args["name"] or "",
			 ["language"] = args["language"] or "es"
		 },
		 source = ltn12.source.string(web.input["post_data"]),
		 sink = ltn12.sink.table(response)
	 }
 
	 LogInfo("[KNOWLEDGE_BASE.QNA_V2:AddKnowledgeBaseFromVideo] Requested to url [%s] %s and apikey %s with status %s", verb, uri, apikey, code)
 
	  web.status = code
	  return table.concat(response)

end

local function AddKnowledgeBaseFromPDF (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local qnaMakerSubscriptionsKeysV2, qnaMakerTotalSubscriptionsV2 = apiQnAMaker.LoadVCCSubscriptionsV2(vcc)

	local vccApiKeyQnaV2
	local vccBaseUriQnaV2

	if qnaMakerTotalSubscriptionsV2 == 0 then
		LogInfo("[KNOWLEDGE_BASE.QNA_V2:AddKnowledgeBaseFromPDF] VCC '%s' has no assigned QnA Maker V2 Subscriptions keys.", vcc)
	else
		local key, value = next(qnaMakerSubscriptionsKeysV2)
		vccBaseUriQnaV2 = value.subscriptionName
		vccApiKeyQnaV2 = value.subscriptionKey
	end

	local baseUri = [[https://]].. vccBaseUriQnaV2 ..[[/rasa/api/qna/knowledgebase/pdf?eventStream=disabled]]
	local verb = "POST"
	local apikey = vccApiKeyQnaV2
	local uri = string.format(baseUri)

	LogInfo("[KNOWLEDGE_BASE.QNA_V2:AddKnowledgeBaseFromPDF] Requesting to url [%s] %s and apikey %s", verb, uri, apikey)

	local args = utils.ParseQueryString(web.vars.QUERY_STRING)

	local response = {}
	local result, code, headers, statusline = https.request{
		url = uri,
		method = verb,
		headers = {
			["apikey"] = apikey,
			["Content-Type"] = "application/pdf",
			["Content-Length"] = tostring(#web.input["post_data"]),
			["name"] = args["name"] or "",
			["language"] = args["language"] or "es"
		},
		source = ltn12.source.string(web.input["post_data"]),
    	sink = ltn12.sink.table(response)
	}

	LogInfo("[KNOWLEDGE_BASE.QNA_V2:AddKnowledgeBaseFromPDF] Requested to url [%s] %s and apikey %s with status %s", verb, uri, apikey, code)

     web.status = code
	 return table.concat(response)
end

local function UpdateKnowledgeBaseQuestions (web, kbId)
	local vcc = inconcert.authUser.virtualContactCenter
	local ok, updatedBase = pcall(json.decode, web.input["post_data"])
	local qnaMakerSubscriptionsKeysV2, qnaMakerTotalSubscriptionsV2 = apiQnAMaker.LoadVCCSubscriptionsV2(vcc)

	local vccApiKeyQnaV2
	local vccBaseUriQnaV2

	if qnaMakerTotalSubscriptionsV2 == 0 then
		LogInfo("[KNOWLEDGE_BASE.QNA_V2:LoadVccConfig] VCC '%s' has no assigned QnA Maker V2 Subscriptions keys.", vcc)
	else
		local key, value = next(qnaMakerSubscriptionsKeysV2)
		vccBaseUriQnaV2 = value.subscriptionName
		vccApiKeyQnaV2 = value.subscriptionKey
	end

	local baseUri = [[https://]].. vccBaseUriQnaV2 ..[[/rasa/api/qna/knowledgebase/]].. kbId .. [[/version]]
	local verb = "PUT"
	local apikey = vccApiKeyQnaV2
	local uri = string.format(baseUri)
	local jsonDataStr = json.encode(updatedBase)
	
	LogInfo("[KNOWLEDGE_BASE.QNA_V2:UpdateKnowledgeBaseQuestions] Requesting to url [%s] %s and apikey %s", verb, uri, apikey)

	local response = {}
	local result, code, headers, statusline = https.request{
		url = uri,
		method = verb,
		headers = {
			["apikey"] = apikey,
			["Content-Type"] = "application/json",
			["Content-Length"] = #jsonDataStr,
		},
		source = ltn12.source.string(jsonDataStr),
		sink = ltn12.sink.table(response)
	}

	LogInfo("[KNOWLEDGE_BASE.QNA_V2:UpdateKnowledgeBaseQuestions] Requested to url [%s] %s and apikey %s with status %s", verb, uri, apikey, code)

     web.status = code
	 return table.concat(response)
end

local function ChangeActiveVersion (web, kbId)
	local vcc = inconcert.authUser.virtualContactCenter

	local qnaMakerSubscriptionsKeysV2, qnaMakerTotalSubscriptionsV2 = apiQnAMaker.LoadVCCSubscriptionsV2(vcc)

	local vccApiKeyQnaV2
	local vccBaseUriQnaV2

	if qnaMakerTotalSubscriptionsV2 == 0 then
		LogInfo("[KNOWLEDGE_BASE.QNA_V2:LoadVccConfig] VCC '%s' has no assigned QnA Maker V2 Subscriptions keys.", vcc)
	else
		local key, value = next(qnaMakerSubscriptionsKeysV2)
		vccBaseUriQnaV2 = value.subscriptionName
		vccApiKeyQnaV2 = value.subscriptionKey
	end

	local baseUri = [[https://]].. vccBaseUriQnaV2 ..[[/rasa/api/qna/knowledgebase/]].. kbId ..[[/active]]
	local verb = "PUT"
	local apikey = vccApiKeyQnaV2
	local uri = string.format(baseUri)
	
	LogInfo("[KNOWLEDGE_BASE.QNA_V2:ChangeActiveVersion] Requesting to url [%s] %s and apikey %s", verb, uri, apikey)

	local response = {}
	local result, code, headers, statusline = https.request{
		url = uri,
		method = verb,
		headers = {
			["apikey"] = apikey,
			["Content-Type"] = "application/json"
		},
		sink = ltn12.sink.table(response)
	}

	LogInfo("[KNOWLEDGE_BASE.QNA_V2:ChangeActiveVersion] Requested to url [%s] %s and apikey %s with status %s", verb, uri, apikey, code)

     web.status = code
	 return table.concat(response)
end

local function DeleteKnowledgeBase (web, kbId)
	local vcc = inconcert.authUser.virtualContactCenter

	local qnaMakerSubscriptionsKeysV2, qnaMakerTotalSubscriptionsV2 = apiQnAMaker.LoadVCCSubscriptionsV2(vcc)

	local vccApiKeyQnaV2
	local vccBaseUriQnaV2

	if qnaMakerTotalSubscriptionsV2 == 0 then
		LogInfo("[KNOWLEDGE_BASE.QNA_V2:LoadVccConfig] VCC '%s' has no assigned QnA Maker V2 Subscriptions keys.", vcc)
	else
		local key, value = next(qnaMakerSubscriptionsKeysV2)
		vccBaseUriQnaV2 = value.subscriptionName
		vccApiKeyQnaV2 = value.subscriptionKey
	end

	local baseUri = [[https://]].. vccBaseUriQnaV2 ..[[/rasa/api/qna/knowledgebase/]].. kbId 
	local verb = "DELETE"
	local apikey = vccApiKeyQnaV2
	local uri = string.format(baseUri)
	
	LogInfo("[KNOWLEDGE_BASE.QNA_V2:DeleteKnowledgeBase] Requesting to url [%s] %s and apikey %s", verb, uri, apikey)

	local response = {}
	local result, code, headers, statusline = https.request{
		url = uri,
		method = verb,
		headers = {
			["apikey"] = apikey,
			["Content-Type"] = "application/json"
		},
		sink = ltn12.sink.table(response)
	}

	LogInfo("[KNOWLEDGE_BASE.QNA_V2:DeleteKnowledgeBase] Requested to url [%s] %s and apikey %s with status %s", verb, uri, apikey, code)

     web.status = code
	 return table.concat(response)
end

local function GetRequestStatus (web, requestId)

	local vcc = inconcert.authUser.virtualContactCenter
	local qnaMakerSubscriptionsKeysV2, qnaMakerTotalSubscriptionsV2 = apiQnAMaker.LoadVCCSubscriptionsV2(vcc)

	local vccApiKeyQnaV2
	local vccBaseUriQnaV2

	if qnaMakerTotalSubscriptionsV2 == 0 then
		LogInfo("[KNOWLEDGE_BASE.QNA_V2:GetRequestStatus] VCC '%s' has no assigned QnA Maker V2 Subscriptions keys.", vcc)
	else
		local key, value = next(qnaMakerSubscriptionsKeysV2)
		vccBaseUriQnaV2 = value.subscriptionName
		vccApiKeyQnaV2 = value.subscriptionKey
	end

	local baseUri = [[https://]].. vccBaseUriQnaV2 ..[[/rasa/api/qna/knowledgebase/request/]].. requestId ..[[?eventStream=disabled]]
	local verb = "GET"
	local apikey = vccApiKeyQnaV2
	local uri = string.format(baseUri)
	
	LogInfo("[KNOWLEDGE_BASE.QNA_V2:GetRequestStatus] Requesting to url [%s] %s and apikey %s", verb, uri, apikey)

	local response = {}
	local result, code, headers, statusline = https.request{
		url = uri,
		method = verb,
		headers = {
			["apikey"] = apikey,
			["Content-Type"] = "application/json"
		},
		sink = ltn12.sink.table(response)
	}

	LogInfo("[KNOWLEDGE_BASE.QNA_V2:GetRequestStatus] Requested to url [%s] %s and apikey %s with status %s", verb, uri, apikey, code)

     web.status = code
	 return table.concat(response)
end

function MapUris(module, htmlRender, prefix, appLocation)
    local LogRequest = require("apps.agent.requestFilters").LogRequest

    module:dispatch_get (utils.JsonReply ..LogRequest ..GetAllKnowledgeBases, prefix.."/knowledgebases/get_all(.+)")
    module:dispatch_get (utils.JsonReply ..LogRequest ..GetKnowledgeBasesById, prefix.."/knowledgebases/get/(.+)")
    module:dispatch_post (utils.JsonReply ..LogRequest ..AddKnowledgeBase, prefix.."/knowledgebases/create/")
	module:dispatch_post (utils.JsonReply ..LogRequest ..UpdateKnowledgeBase, prefix.."/knowledgebases/update/")
	module:dispatch_post (AddKnowledgeBaseFromVideo, prefix.."/knowledgebases/create/video/")
	module:dispatch_post (AddKnowledgeBaseFromPDF, prefix.."/knowledgebases/create/pdf/")
	module:dispatch_put (utils.JsonReply ..LogRequest ..UpdateKnowledgeBaseQuestions, prefix.."/knowledgebases/(.+)/version/")
	module:dispatch_put (utils.JsonReply ..LogRequest ..ChangeActiveVersion, prefix.."/knowledgebases/(.+)/active/")
	module:dispatch_delete (utils.JsonReply ..LogRequest ..DeleteKnowledgeBase, prefix.."/knowledgebases/delete/(.+)")
	module:dispatch_get (utils.JsonReply ..LogRequest ..GetRequestStatus, prefix.."/knowledgebases/request/(.+)")
	module:dispatch_get (utils.JsonReply ..LogRequest ..SearchKnowledgeBases, prefix.."/knowledgebases/search/(.+)")
	
    m_htmlRender = htmlRender
    m_appLocation = appLocation

end
