/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, esversion: 9 */
/* global inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

//	"use strict";

	const FORMAT = inConcert.Utils.format;
	const ERROR_TIMEOUT_MS = 5000;

	var make_string_date = function(date) {
		if (!date) {
			return "";
		}
		return inConcert.Utils.getStringFromDate(inConcert.Utils.parseDateTimeString(date), false, "date-short-year-short-hour");
	};

	//---------------------------------------------------------------------
	// MetaData Management
	function MetaDataMng(params)
	{
		if (!params) {
			throw("MetaDataMng - 'params' is required");
		}
		if (!params.owner) {
			throw("MetaDataMng - 'params.owner' is required");
		}
		if (!params.prefix) {
			throw("MetaDataMng - 'params.prefix' is required");
		}
		if (!params.container_id) {
			throw("MetaDataMng - 'params.container_id' is required");
		}

		this.owner = params.owner;
		this.owner_name = params.owner_name || "";
		this.prefix = params.prefix + "_";
		this.container_id = params.container_id;
		this.m_grid = null;

		return true;
	}

	MetaDataMng.prototype.initialize = function()
	{
		this.__make_popup_html();
		this.__init_grid();
	};

	MetaDataMng.prototype.__make_popup_html = function()
	{
		let html = '' +
		'<div id="' + this.prefix + 'new_container" class="card-block">' +
			'<div class="card-header d-flex flex-row justify-content-between">' +
				'<h5><%=_T([[Metadata Config]])%></h5>' +
				'<i id="' + this.prefix + 'new_list_validation" class="formFieldValidation formFieldError" hidden></i>' +
				'<button id="' + this.prefix + 'new" class="pageButtonGeneric buttonColorWhite pageButtonNew mt-1 mb-4">' +
					'<i class="far fa-plus"></i>' +
					'<%=_T([[New Metadata]])%>' +
				'</button>' +
			'</div>' +
		'</div>' +

		'<div id="' + this.prefix + 'container" class="card-block">' +
			'<div id="' + this.prefix + 'popup" class="" style="display:none;">' +
				'<div class="row">' +
					'<div class="col-6">' +
						'<div class="form-group">' +
							'<label for="' + this.prefix + 'popup_name" class="required"><%=_T([[Name]])%><%=_T([[:]])%></label>' +
							'<input id="' + this.prefix + 'popup_name" class="form-control" type="text" />' +
							'<i id="' + this.prefix + 'popup_name_validation" class="formFieldValidation formFieldError" hidden></i>' +
						'</div>' +
					'</div>' +
					'<div class="col-6">' +
						'<div class="form-group">' +
							'<label for="' + this.prefix + 'popup_value" class="required"><%=_T([[Value]])%><%=_T([[:]])%></label>' +
							'<input id="' + this.prefix + 'popup_value" class="form-control" type="text" />' +
							'<i id="' + this.prefix + 'popup_value_validation" class="formFieldValidation formFieldError" hidden></i>' +
						'</div>' +
					'</div>' +
				'</div>' +
				'<div class="row custom-form-buttons-container-2">' +
					'<div class="col-12">' +
						'<button id="' + this.prefix + 'popup_close" class="pageButtonGeneric buttonColorWhite">' +
							'<%=_T([[Cancel]])%>' +
						'</button>' +
						'<button id="' + this.prefix + 'popup_create_and_continue" class="pageButtonGeneric buttonColorWhite">' +
							'<%=_T([[Confirm & Continue]])%>' +
						'</button>' +
						'<button id="' + this.prefix + 'popup_create" class="pageButtonGeneric buttonColorBlack">' +
							'<%=_T([[Confirm]])%>' +
						'</button>' +
					'</div>' +
				'</div>' +
				"<div id='" + this.prefix + "popup_message' class='genericFormMessagesContainer messageError' hidden>" +
					"<label class='messageText'></label>" +
				"</div>" +
			'</div>' +
			'<div class="row">' +
				'<div class="col-12">' +
					'<div id="' + this.prefix + 'not_results_found" class="inlineMessagesContainer messageInfo mt-0 mb-3" style="display:none"></div>' +
					'<table id="' + this.prefix + 'list" cellpadding="0" cellspacing="0" border="0" class="viewGrid mt-0 mb-3">' +
						'<thead class="viewGridHeader"></thead>' +
						'<tbody class="viewGridBody"></tbody>' +
					'</table>' +
				'</div>' +
				'<div id="' + this.prefix + 'paginator"></div>' +
			'</div>' +
		'</div>';

		const $container = $("#" + this.container_id);
		$container.html(html);
	};

	MetaDataMng.prototype.__init_grid = function()
	{
		const self = this;

		this.m_grid = new inConcert.conf_form_paginator({
			owner: this.owner,
			frm: this.owner,
			row_id: "name",
			display_name: "name",
			rows_per_page: 2000,
			show_at_least: 1999,
			paginator_type: "ALL_ROWS_CLIENT_SIDE",
			without_paginator: true,
			use_instance_of_inconcert_popup: true, // con este flag el popup se instancia desde el popup inconcert popup window.
			ids: {
				paginator: this.prefix + "paginator",
				grid: this.prefix + "list",
				not_results_found: this.prefix + "not_results_found",
				container_new_btn: this.prefix + "new_container",
				new_item: this.prefix + "new",
				disclaimer: this.prefix + "disclaimer",
			//popup0
				popup: this.prefix + "popup",
				popup_msg: this.prefix + "popup_message",
				popup_create: this.prefix + "popup_create",
				popup_create_and_continue: this.prefix + "popup_create_and_continue",
				popup_title: this.prefix + "popup_title",
				popup_close: this.prefix + "popup_close",
				popup_input_id: this.prefix + "popup_question",
				popup_waiting: this.prefix + "popup_waiting"
			},
			strings: {
				not_results_found        : "<%=_T('No MetaData found.')%>",
				title_new                 : "<%=_T('Add MetaData')%>",
				title_update              : "<%=_T('Update MetaData')%>",
				title_readonly            : "<%=_T('View MetaData')%>",
				button_create             : "<%=_T('Add')%>",
				button_create_and_continue: "<%=_T('Add & Continue')%>",
				can_remove                : "<%=_T('Do you want to delete MetaData: {0} ?')%>"
			},
			columns: [
				{id: "_number", name: "", type: "counter"},
				{id: "name"   , name: "<%=_T('Name')%>"  },
				{id: "value"  , name: "<%=_T('Value')%>" },
				{
					id: "_action",
					name: "",
					system_action: {
						edit: true,
						erase: true
					}
				}
			],
			callbacks: {
				save: function(callback, must_create_and_contiune, current_popup) {
					callback = callback || $.noop;
					self.add_or_update(callback, must_create_and_contiune, current_popup);
				},
				edit: function(row_data) {
					self.open_popup(row_data);
				},
				on_removed_row: function(/*row, callback = $.noop*/) {},
 				on_closed_popup: function() {
 					self.on_closed_popup();
 				}
			}
		});
	};

	MetaDataMng.prototype.on_closed_popup = function(/*current_popup*/)
	{
		this.m_is_new_meta_data = null;
		this.m_current_meta_data = null;
		this.owner.m_grd_questions_answers.disable_actions(false);
	};

	MetaDataMng.prototype.add_or_update = function(callback, _, current_popup)
	{
		callback = callback || $.noop;

		let response = this.validate(current_popup);
		if (!response.success) {
			callback(false);
			return;
		}

		let msg_response = {
			msg : "",
			type: "Info"
		};

		if (this.m_is_new_meta_data) {
			msg_response.msg = "<%=_T('MetaData was successfully added.')%>";
			msg_response.$msg_container = $("#adm_form_qna_maker_updating_questions_edit_popup_message");
			let must_add_many = false;
			callback(true, response.item, msg_response, must_add_many);
		}
		else {
			msg_response.msg = "<%=_T('MetaData was successfully updated.')%>";
			this.m_current_meta_data = inConcert.Utils.cloneObject(response.item);
			callback(true, this.m_current_meta_data, msg_response);
		}

		this.clean_up_controls();
	};

	MetaDataMng.prototype.clean_up_controls = function()
	{
		const $name = $("#" + this.prefix + "popup_name");
		this.owner.readonly_control($name, false, true);
		const $value = $("#" + this.prefix + "popup_value");
		$name.val("").focus();
		$value.val("");
	};

	MetaDataMng.prototype.open_popup = function(item)
	{
		this.m_is_new_meta_data = item ? false: true;
		this.m_current_meta_data = item;

		this.owner.m_grd_questions_answers.disable_actions(true);

		this.clean_up_controls();
		if (this.m_is_new_meta_data) {
			// nuevo
			return;
		}
		// editando
		const $name = $("#" + this.prefix + "popup_name");
		$name.val(item.name);
		this.owner.readonly_control($name, true);

		const $value = $("#" + this.prefix + "popup_value");
		$value.val(item.value);

		$name.focus();
	};

	MetaDataMng.prototype.validate = function()
	{
		const self = this;

		this.hide_all_errors();
		let response = {success: false, $ctrl: null, item: null, msg: "<%=_T('Unable to Add MetaData.')%>"};

		let item = {};
		let is_new = false;
		if (this.m_current_meta_data) {
			item = this.m_current_meta_data;
		}
		else {
			is_new = true;
		}

		// NAME validation -----
		//
		// Required validation
		const $name = $("#" + this.prefix + "popup_name");
		let name = $name.val();
		let validation_result = inConcert.Utils.validate(name, "required");
		if (!validation_result.success) {
			// valida que el name tenga al menos un caracter
			response.msg = "<%=_T('Name is required.')%>";
			response.$ctrl = $name;
			this.show_ctrl_err(response);
			return response;
		}

		validation_result = inConcert.Utils.validate(name, "id_without_dots");
		if (!validation_result.success) {
			// valida que el name tenga caracteres validos
			response.msg = "<%=_T('Only letters, numbers and dashes are allowed in the Name.')%>";
			response.$ctrl = $name;
			this.show_ctrl_err(response);
			return response;
		}

		name = this.m_grid.sanitize_value(name);
		if (name.trim().length === 0) {
			// sanitiza el name y valida que no haya quedado en 0 caracteres.
			response.msg = "<%=_T('Invalid name.')%>";
			response.$ctrl = $name;
			this.show_ctrl_err(response);
			return response;
		}

		// Already exist validation
		let current_all_meta_data = this.m_grid.get_items();
		current_all_meta_data = inConcert.Utils.arrayToMap(current_all_meta_data || [],
			function(data) {
				return data.value;
			},
			function(data) {
				// el ID
				return (data.name || "").toLowerCase();
			}
		);
		var show_already_error = function() {
			response.msg = "<%=_T('Name already exists.')%>";
			response.$ctrl = $name;
			self.show_ctrl_err(response);
			return response;
		};
		let lower_name = name.toLowerCase();
		if (is_new) {
			const already = current_all_meta_data[lower_name];
			if (already) {
				return show_already_error();
			}
		}
		else {
			if (item.name.toLowerCase() !== lower_name) {
				const already = current_all_meta_data[lower_name];
				if (already) {
					return show_already_error();
				}
			}
		}

		// VALUE validation -----
		//
		// Required validation
		const $value = $("#" + this.prefix + "popup_value");
		let value = $value.val();
		validation_result = inConcert.Utils.validate(value, "required");
		if (!validation_result.success) {
			// valida que el name tenga al menos un caracter
			response.msg = "<%=_T('Value is required.')%>";
			response.$ctrl = $value;
			this.show_ctrl_err(response);
			return response;
		}

		value = this.m_grid.sanitize_value(value);
		if (value.trim().length === 0) {
			// valida que el name tenga al menos un caracter
			response.msg = "<%=_T('Invalid value.')%>";
			response.$ctrl = $value;
			this.show_ctrl_err(response);
			return response;
		}

		item.name = lower_name;
		item.value = value.toLowerCase();

		response.success = true;
		response.msg = "";
		response.item = item;

		return response;
	};
//" + this.prefix + "
	MetaDataMng.prototype.hide_all_errors = function()
	{
		this.m_grid.hide_popup_msg();
	};

	MetaDataMng.prototype.show_ctrl_err = function(err_info)
	{
		if (!err_info) {
			return;
		}
		const $control = err_info.$ctrl;
		if ($control) {
			const error_id = $control.attr("id") + "_validation";
			const $error_indicator = $("#" + error_id);
			if ($error_indicator) {
				$error_indicator.show();
			}
			$control.focus();
		}
		this.m_grid.show_popup_msg(err_info.msg, "Error"); //, ERROR_TIMEOUT_MS);
	};

	MetaDataMng.prototype.dispose = function()
	{
		//TODO: implementar, si es necesario
	};

	MetaDataMng.prototype.get = function()
	{
		let meta_data = this.m_grid.get_items();
		return meta_data;
	};

	MetaDataMng.prototype.set = function(meta_data)
	{
		this.m_grid.initialize($.noop, meta_data);
	};

	MetaDataMng.prototype.empty_controls = function()
	{
		this.m_grid.remove_all_rows();
	};

	MetaDataMng.prototype.set_events = function()
	{
		this.m_grid.set_events();
	};

	MetaDataMng.prototype.close = function()
	{
		this.m_grid.hide_popup();
	};

	//---------------------------------------------------------------------
	// form qna_maker class
	function adm_form_qna_maker(params)
	{
		const self = this;
		const success = adm_form_qna_maker.uber.constructor.call(this, params);
		if (!success) {
			return;
		}
		this.view_title        = "<%=_T('QnA Maker - View Knowledge base')%>";
		this.new_title         = "<%=_T('QnA Maker - Creating Knowledge base')%>";
		this.update_title      = "<%=_T('QnA Maker - Editing Knowledge base')%>";
		this.update_ok_message = "<%=_T('QnA Maker - Knowledge base was successfully updated.')%>";
		this.html_file_name    = "/inconcert/apps/knowledge_base/qna_maker/frm/render/";
		this.rows_per_page = 10;
		this.show_at_least = this.rows_per_page - 1;
		this.m_grd_questions_answers = null;
		this.m_is_new_question = null;
		this.m_current_question = null;
		this.m_added_questions = [];
		this.m_deleted_questions_ids = [];
		this.m_updated_questions = [];

		this.__initialize_grid();

		// Instancia de la clase los TABs
		this.main_tabs = new inConcert.conf_form_tab({
			parent        : this,
			frm           : this.frm,
			container_id  : "adm_form_qna_maker_updating_tabs",
			must_intialize: false,
			tabs_list   : {
				edit: {
					id              : "edit",
					name            : "<%=_T('Questions/Answers')%>",
					tab_container_id: "adm_form_qna_maker_updating_edit_tab_container",
					selected        : true,
					on_show         : function() {
						self.on_show_tab("edit");
					}
				},
				publish: {
					id              : "publish",
					name            : "<%=_T('Publish')%>",
					tab_container_id: "adm_form_qna_maker_updating_publish_tab_container",
					on_show         : function() {
						self.on_show_tab("publish");
					}
				},
				settings: {
					id              : "settings",
					name            : "<%=_T('General')%>",
					tab_container_id: "adm_form_qna_maker_updating_settings_tab_container",
					on_show         : function() {
						self.on_show_tab("settings");
					}
				}
			}
		});

		this.metadata_mng = new MetaDataMng({
			owner       : this,
			prefix      : "adm_form_qna_maker_updating_questions_edit_popup_metadata",
			container_id: "adm_form_qna_maker_updating_questions_edit_popup_metadata_panel"
		});

		return true;
	}

	inConcert.Utils.inherit( adm_form_qna_maker, inConcert.conf_form );

	if ( typeof inConcert.adm_form_qna_maker !== "function" ) {
		inConcert.adm_form_qna_maker = adm_form_qna_maker;
	}

	adm_form_qna_maker.prototype.initialize = function()
	{
		this.main_tabs.initialize();
		this.main_tabs.show_tabs_container();
		this.metadata_mng.initialize();
	};

	adm_form_qna_maker.prototype.__initialize_grid = function()
	{
		const self = this;
		this.m_grd_questions_answers = new inConcert.conf_form_paginator({
			paginator_name: "QUESTIONS_ANSWERS",
			owner: this,
			frm: this,
			row_id: "id",
			display_name: function(row) {
				const display_name = "<%=_T('{0}, Answer: {1}')%>";
				const question = $.isArray(row.questions) && row.questions[0] || row.questions || "";
				const length = 20;
				const dots = "..";
				return FORMAT(display_name, inConcert.Utils.truncateWord(question, length, dots), inConcert.Utils.truncateWord(row.answer, length, dots));
			},
			rows_per_page: this.rows_per_page,
			show_at_least: this.show_at_least,
			paginator_type: "ALL_ROWS_CLIENT_SIDE",
			add_with_enter: false,
			ids: {
				grid: "adm_form_qna_maker_updating_questions_list",
				paginator: "adm_form_qna_maker_updating_questions_paginator",
				not_results_found: "adm_form_qna_maker_updating_questions_not_results_found",
				//popup0 (add qna pair)
				new_item: "adm_form_qna_maker_updating_questions_add_qna_pair",
				popup: "adm_form_qna_maker_updating_questions_edit_popup",
				popup_msg: "adm_form_qna_maker_updating_questions_edit_popup_message",
				popup_create: "adm_form_qna_maker_updating_questions_edit_popup_create",
				popup_create_and_continue: "adm_form_qna_maker_updating_questions_edit_popup_create_and_continue",
				popup_title: "adm_form_qna_maker_updating_questions_edit_popup_title",
				popup_close: "adm_form_qna_maker_updating_questions_edit_popup_close",
				popup_input_id: "adm_form_qna_maker_updating_questions_edit_popup_question",
				popup_waiting: "adm_form_qna_maker_updating_questions_edit_popup_waiting",
				//search
				search_container: "adm_form_qna_maker_updating_questions_search_container",
				search_text     : "adm_form_qna_maker_updating_questions_search_text",
				search_reset    : "adm_form_qna_maker_updating_questions_reset_text",
				search_apply    : "adm_form_qna_maker_updating_questions_apply_text"
			},
			strings: {
				//popup0 (add qna pair)
				not_results_found         : "<%=_T('No Questions Found.')%>",
				can_remove                : "<%=_T('Do you want to delete Question: {0} ?')%>",
				title_new                 : "<%=_T('QnA Maker - Add Pairs of Questions & Answer')%>",
				title_update              : "<%=_T('QnA Maker - Update Pairs of Questions & Answer')%>",
				title_readonly            : "<%=_T('QnA Maker - View Pairs of Questions & Answer')%>",
				button_create             : "<%=_T('Add')%>",
				button_create_and_continue: "<%=_T('Add & Continue')%>"
			},
			columns: [
				{id: "_number"  , name: "", type: "counter"},
				{id: "questions", name: "<%=_T('Question')%>",
					get_value: function(item) {
						return self.__get_grid_value(item, "questions");
					}
				},
				{id: "answer"  , name: "<%=_T('Answer')%>"},
				{id: "metadata", name: "<%=_T('Metadata Tags')%>",
					get_value: function(item) {
						return self.__get_grid_value(item, "metadata");
					}
				},
				{id: "id"    , name: "<%=_T('Question Id')%>"},
				{
					id: "_action",
					name: "",
					system_action: {
						edit: {
							must_show: function() {
								return true;
							}
						},
						erase: true
					}
				}
			],
			callbacks: {
				save: function(callback, must_create_and_contiune, current_popup) {
					callback = callback || $.noop;
					self.add_or_update_question(callback, must_create_and_contiune, current_popup);
				},
				edit: function(row, current_popup) {
					self.on_open_popup(row, current_popup);
				},
				remove: function(row, callback = $.noop) {
					self.remove(row, callback);
				},
				on_closed_popup: function(current_popup) {
					self.on_closed_popup(current_popup);
				}
			}
		});
	};

	adm_form_qna_maker.prototype.hide_all_errors = function()
	{
		this.m_grd_questions_answers.hide_popup_msg();
	};

	adm_form_qna_maker.prototype.show_ctrl_err = function(err_info)
	{
		if (!err_info) {
			return;
		}
		const $control = err_info.$ctrl;
		if ($control) {
			const error_id = $control.attr("id") + "_validation";
			const $error_indicator = $("#" + error_id);
			if ($error_indicator) {
				$error_indicator.show();
			}
			$control.focus();
		}
		this.m_grd_questions_answers.show_popup_msg(err_info.msg, "Error", ERROR_TIMEOUT_MS);
	};

	adm_form_qna_maker.prototype.on_open_popup = function(item)
	{
		this.m_is_new_question = item ? false: true;
		this.m_current_question = !this.m_is_new_question ? inConcert.Utils.cloneObject(item) : item;

		if (this.is_readonly()) {
			$("#adm_form_qna_maker_updating_questions_edit_popup_type_disclaimer").hide();
		}

		this.clean_up_controls();
		if (this.m_is_new_question) {
			// NUEVO
			//metadata
			this.metadata_mng.set([]);
			return;
		}
		// EDITANDO

		// respuesta
		const $answer = $("#adm_form_qna_maker_updating_questions_edit_popup_answer");
		$answer.val(item.answer);
		// preguntas
		const $question = $("#adm_form_qna_maker_updating_questions_edit_popup_question");
		let questions = item.questions.join("\n");
		$question.val(questions);
		//metadata
		this.metadata_mng.set(item.metadata);

		$question.focus();
	};

	adm_form_qna_maker.prototype.on_closed_popup = function(/*current_popup*/)
	{
		this.m_is_new_question = null;
		this.m_current_question = null;
		this.metadata_mng.close();
	};

	adm_form_qna_maker.prototype.add_or_update_question = function(callback, _, current_popup)
	{
		callback = callback || $.noop;

		let response = this.validate_question_to_add_or_update(current_popup);
		if (!response.success) {
			callback(false);
			return;
		}

		let msg_response = {
			msg : "",
			type: "Info"
		};

		if (this.m_is_new_question) {
			this.m_added_questions.push(response.item);
			msg_response.msg = "<%=_T('Question was successfully added.')%>";
			let must_add_many = false;
			callback(true, response.item, msg_response, must_add_many);
		}
		else {
			let cloned_question = inConcert.Utils.cloneObject(response.item);
			if (cloned_question.id < 0) {
				for (let i = 0; i < this.m_added_questions.length; i++) {
					if (this.m_added_questions[i].id === cloned_question.id) {
						this.m_added_questions[i] = cloned_question;
						break;
					}
				}
			}
			else {
				let must_add = true;
				for (let i = 0; i < this.m_updated_questions.length; i++) {
					let updated_question = this.m_updated_questions[i];
					if (updated_question.id === cloned_question.id) {
						must_add = false;
						this.m_updated_questions[i].new_question_info = cloned_question;
						break;
					}
				}
				if (must_add) {
					let old_question_info = inConcert.Utils.cloneObject(this.m_current_question);
					this.m_updated_questions.push({
						id: cloned_question.id,
						new_question_info: cloned_question,
						old_question_info: old_question_info
					});
				}
			}
			msg_response.msg = "<%=_T('Question was successfully updated.')%>";
			this.m_current_question.answer = cloned_question.answer;
			this.m_current_question.questions = cloned_question.questions;
			this.m_current_question.metadata = cloned_question.metadata;
			callback(true, this.m_current_question, msg_response);
		}

		this.clean_up_controls();
	};

	adm_form_qna_maker.prototype.clean_up_controls = function()
	{
		this.metadata_mng.set([]);
		const $question = $("#adm_form_qna_maker_updating_questions_edit_popup_question");
		const $answer = $("#adm_form_qna_maker_updating_questions_edit_popup_answer");
		$question.val("").focus();
		$answer.val("");
	};

	adm_form_qna_maker.prototype.validate_question_to_add_or_update = function()
	{
		this.hide_all_errors();
		let response = {success: false, $ctrl: null, item: null, msg: "<%=_T('Unable to Add Question.')%>"};

		let item = {};
		if (this.m_current_question) {
			item.id = this.m_current_question.id;
		}
		else {
			// Genero un numero randomico negativo, para los nuevos.
			// Solo se utiliza client side, ya que en la grilla por cada row, se necesita tener un id.
			// Server side no tiene que viajar este valor.
			item.id = (Math.floor((Math.random() * 10000) +1) * -1);
		}

		// 1 - Preguntas N (array de strings)
		let questions_string = $("#adm_form_qna_maker_updating_questions_edit_popup_question").val();
		let validation_result = inConcert.Utils.validate(questions_string, "required");
		if (!validation_result.success) {
			// valida que el name tenga al menos un caracter
			response.msg = "<%=_T('Question is required.')%>";
			response.$ctrl = $("#adm_form_qna_maker_updating_questions_edit_popup_question");
			this.show_ctrl_err(response);
			return response;
		}
		let questions_array = questions_string.split("\n");
		item.questions = [];
		questions_array.forEach(question => {
			if (question.trim().length > 0) {
				item.questions.push(question);
			}
		});

		// 2 - Respuesta 1 (string)
		item.answer = $("#adm_form_qna_maker_updating_questions_edit_popup_answer").val();
		validation_result = inConcert.Utils.validate(item.answer, "required");
		if ( !validation_result.success ) {
			// valida que el name tenga al menos un caracter
			response.msg = "<%=_T('Answer is required.')%>";
			response.$ctrl = $("#adm_form_qna_maker_updating_questions_edit_popup_answer");
			this.show_ctrl_err(response);
			return response;
		}

		// 3 - Metadata N (array de objetos name/value)
		item.metadata = this.metadata_mng.get();

		response.success = true;
		response.msg = "";
		response.item = item;

		return response;
	};

	adm_form_qna_maker.prototype.__get_grid_value = function(item, field_id)
	{
		let value = {
			original_value: "",
			value         : "",
			search_value  : "",
			class_name    : ""
		};

		let item_value = item[field_id];
		if (item_value === undefined) {
			item_value = "";
		}

		value.original_value = item_value;
		value.value = item_value;
		value.search_value = item_value;

		if (field_id === "questions") {
			if ($.isArray(item_value) && item_value.length > 0) {
				let concated_value = "";
				item_value.forEach((val, i) => {
					if (i > 0) {
						concated_value = concated_value + "\n";
					}
					concated_value = concated_value + val;
				});
				value.value = concated_value;
				value.search_value = concated_value;
			}
		}
		else if (field_id === "metadata") {
			if ($.isArray(item_value) && item_value.length > 0) {
				let concated_value = "";
				item_value.forEach((metadata, i) => {
					if (i > 0) {
						concated_value = concated_value + "\n";
					}
					//TODO: SANITIZAR
					concated_value = concated_value + FORMAT("{0} : {1}", metadata.name, metadata.value);
				});
				value.value = concated_value;
				value.search_value = concated_value;
			}
		}

		return value;
	};

	adm_form_qna_maker.prototype.get = function()
	{
		let knowledge_base = {};

		if (this.is_creating()) {
			knowledge_base.name = $("#adm_form_qna_maker_creating_kb_name").val();
		}
		else {
			const entity = this.get_entity_instance();

			knowledge_base.kbId = entity.inconcertInfo.kbId;
			knowledge_base.name = $("#adm_form_qna_maker_updating_kb_name").val();

			// los nuevos
			knowledge_base.add = [];
			this.m_added_questions.forEach(question => {
				knowledge_base.add.push({
					answer   : question.answer,
					questions: question.questions,
					metadata : question.metadata
				});
			});

			// los que se eliminaron
			knowledge_base["delete"] = this.m_deleted_questions_ids;

			// los que se actualizan
			knowledge_base.update = [];
			this.m_updated_questions.forEach(updated_question =>
			{
			// 1 Questions
				let updated_questions = {
					["add"]: [],
					["delete"]: []
				};

				let new_question_info = updated_question.new_question_info;
				let old_question_info = updated_question.old_question_info;

				for (let i = 0; i < old_question_info.questions.length; i++) {
					let must_delete = true;
					let current_question = old_question_info.questions[i];
					for (let j = 0; j < new_question_info.questions.length; j++) {
						let upd_question = new_question_info.questions[j];
						if (current_question.toLowerCase() === upd_question.toLowerCase()) {
							must_delete = false;
							break;
						}
					}
					if (must_delete) {
						updated_questions["delete"].push(current_question);
					}
				}

				for (let i = 0; i < new_question_info.questions.length; i++) {
					let must_add = true;
					let add_question = new_question_info.questions[i];
					for (let j = 0; j < old_question_info.questions.length; j++) {
						let current_question = old_question_info.questions[j];
						if (add_question.toLowerCase() === current_question.toLowerCase()) {
							must_add = false;
							break;
						}
					}
					if (must_add) {
						updated_questions.add.push(add_question);
					}
				}

			// 2 Metadata
				let updated_metadata = {
					["add"]: [],
					["delete"]: []
				};

				for (let i = 0; i < old_question_info.metadata.length; i++) {
					let must_delete = true;
					let current_metadata = old_question_info.metadata[i];
					for (let j = 0; j < new_question_info.metadata.length; j++) {
						let upd_metadata = new_question_info.metadata[j];
						if (!current_metadata ||
								(current_metadata.name.toLowerCase() === upd_metadata.name.toLowerCase() &&
								 current_metadata.value.toLowerCase() === upd_metadata.value.toLowerCase()
								)) {
							must_delete = false;
							break;
						}
					}
					if (must_delete) {
						updated_metadata["delete"].push(current_metadata);
					}
				}

				for (let i = 0; i < new_question_info.metadata.length; i++) {
					let must_add = true;
					let add_metadata = new_question_info.metadata[i];
					for (let j = 0; j < old_question_info.metadata.length; j++) {
						let current_metadata = old_question_info.metadata[j];
						if (!current_metadata ||
								(current_metadata.name.toLowerCase() === add_metadata.name.toLowerCase() &&
								current_metadata.value.toLowerCase() === add_metadata.value.toLowerCase())
								) {
							must_add = false;
							break;
						}
					}
					if (must_add) {
						updated_metadata.add.push(add_metadata);
					}
				}

				knowledge_base.update.push({
					id       : updated_question.id,
					answer   : new_question_info.answer,
					questions: updated_questions,
					metadata : updated_metadata
				});
			});
		}
		return knowledge_base;
	};

	adm_form_qna_maker.prototype.set = function(success, qna_maker)
	{
		if (!success) {
			// TODO: ALGO ?
			return;
		}
		this.empty_controls();

		if (qna_maker.cognitiveServiceError && qna_maker.cognitiveServiceError.errType === "inconcert" && qna_maker.cognitiveServiceError.code === 404) {
			this.__show_creating_panel();
			this.set_new_flag(true);
			$("#adm_form_qna_maker_creating_kb_name").focus();
		}
		else {
			this.__show_updating_panel();
			this.set_new_flag(false);

			// Set tab "Edit"
			const qna_documents = qna_maker.qnaDocuments || [];
			this.__fill_questions_grid(qna_documents);
			// Set tab "Publish"
			const qna_detail = qna_maker.detail || {};
			this.__show_hide_publish_info(qna_detail);
			// Set tab "Settings"
			const inconert_info = qna_maker.inconcertInfo || {};
			$("#adm_form_qna_maker_updating_kb_id").val(inconert_info.kbId);
			$("#adm_form_qna_maker_updating_kb_name").val(inconert_info.name);
			this.__show_hide_settings_info(qna_detail);
		}

		this.set_title();

		return true;
	};

	adm_form_qna_maker.prototype.build_header_custom_buttons = function()
	{
		const self = this;
		const html_search = this.m_grd_questions_answers.get_header_search_html();

		const html_buttons =
		"<button id='adm_form_qna_maker_updating_questions_add_qna_pair' class='pageButtonGeneric buttonColorWhite pageButtonNew pageButtonLeftSeparator'>" +
			"<i class='fal fa-plus'></i>" +
			"<%=_T('Add QnA Pair')%>" +
		"</button>" +
		html_search;

		adm_form_qna_maker.uber.build_header_custom_buttons.call(this, html_buttons, function() {
			if (self.is_readonly()) {
				self.$get_dom_element_by_id( "form_custom_buttons" ).html( html_search )[ "show" ]();
			}
			self.m_grd_questions_answers.on_header_created();
		});
	};

	adm_form_qna_maker.prototype.build_footer_custom_buttons = function()
	{
		const html_buttons =
		"<button id='adm_form_qna_maker_updating_settings_publish_knowledge_base' hidden class='pageButtonGeneric buttonColorBlack'>" +
			"<%=_T('Publish')%>" + 
		"</button>" +
		"<button id='adm_form_qna_maker_updating_settings_delete_knowledge_base' hidden class='pageButtonGeneric pageButtonNew buttonColorWhite pageButtonRightSeparator pageButtonNoIcon'>" +
			"<%=_T('Delete Knowledge base')%>" +
		"</button>" +
		"<button id='adm_form_qna_maker_updating_settings_edit_knowledge_base' hidden class='pageButtonGeneric buttonColorOrange pageButtonNew'>" +
			"<i class='fal fa-pencil'></i>" +
			"<%=_T('Edit')%>" +
		"</button>";

		adm_form_qna_maker.uber.build_footer_custom_buttons.call(this, html_buttons);
	};

	adm_form_qna_maker.prototype.show_header_custom_buttons = function()
	{
		$("#adm_form_qna_maker_updating_questions_add_qna_pair").show();
	};

	adm_form_qna_maker.prototype.hide_header_custom_buttons = function()
	{
		$("#adm_form_qna_maker_updating_questions_add_qna_pair").hide();
	};

	adm_form_qna_maker.prototype.on_show_tab = function(tab_id)
	{
		// generic (conf_form) save button
		const $save = this.$get_dom_element_by_id("form_button_save");
		const $publish = $("#adm_form_qna_maker_updating_settings_publish_knowledge_base");
		const $delete = $("#adm_form_qna_maker_updating_settings_delete_knowledge_base");
		const $edit = $("#adm_form_qna_maker_updating_settings_edit_knowledge_base");

		if (tab_id === "edit") {
			this.show_or_hide_required_message();
			//this.hide_custom_actions();
			if (this.is_readonly()) {
				this.hide_header_custom_buttons();
			//	this.hide_custom_actions();
				$save.hide();
				$delete.hide();
				$edit.show();
			}
			else {
				this.show_header_custom_buttons();
				$save.show();
				$publish.hide();
				$delete.hide();
				$edit.hide();
			}
		}
		else if (tab_id === "publish") {
			this.hide_header_custom_buttons();
			this.show_or_hide_required_message();
			if (this.is_readonly()) {
		//		this.hide_custom_actions();
				$publish.hide();
				$save.hide();
				$edit.show();
			}
			else {
				this.show_custom_actions();
				$publish.show();
				$save.hide();
				$edit.hide();
			}
			$delete.hide();
		}
		else if (tab_id === "settings") {
			this.hide_header_custom_buttons();
			this.show_or_hide_required_message();
			if (this.is_readonly()) {
		//		this.hide_custom_actions();
				$delete.hide();
				$save.hide();
				$edit.show();
			}
			else {
				this.show_custom_actions();
				$delete.show();
				$save.show();
				$edit.hide();
			}
			$publish.hide();
		}
		else {
			throw("adm_form_qna_maker.prototype.on_show_tab - Unrecognized Tab " + tab_id);
		}
	};

	adm_form_qna_maker.prototype.__fill_questions_grid = function(qna_documents)
	{
		let questions_answers = qna_documents || [];
		this.m_grd_questions_answers.initialize($.noop, questions_answers);
	};

	adm_form_qna_maker.prototype.empty_controls = function()
	{
	// creating
		const $creating_kb_name = $("#adm_form_qna_maker_creating_kb_name");
		$creating_kb_name.val("");

	// updating
		$("#adm_form_qna_maker_updating_kb_id").val("");
		$("#adm_form_qna_maker_updating_kb_name").val("");

		if (this.is_creating()) {
			$creating_kb_name.focus();
		}
	};

	adm_form_qna_maker.prototype.__show_creating_panel = function()
	{
		$("#adm_form_qna_maker_creating_container").show();
		$("#adm_form_qna_maker_updating_container").hide();
		$("#adm_form_qna_maker_error_container").hide();
		// botones publish & delete
		$("#adm_form_qna_maker_updating_settings_publish_knowledge_base").hide();
		$("#adm_form_qna_maker_updating_settings_delete_knowledge_base").hide();
		this.ready_for_create();
		this.hide_header_custom_buttons();
		this.show_or_hide_required_message();
		$("#adm_form_qna_maker_creating_kb_name").focus();
	};

	adm_form_qna_maker.prototype.__show_updating_panel = function()
	{
		this.main_tabs.switch_tab("edit");
		$("#adm_form_qna_maker_updating_container").show();
		$("#adm_form_qna_maker_creating_container").hide();
		$("#adm_form_qna_maker_error_container").hide();
		this.ready_for_update();
		this.show_header_custom_buttons();
		this.show_or_hide_required_message();
	};

	adm_form_qna_maker.prototype.__show_error_panel = function()
	{
		$("#adm_form_qna_maker_error_container").show();
		$("#adm_form_qna_maker_updating_container").hide();
		$("#adm_form_qna_maker_creating_container").hide();
	};

	adm_form_qna_maker.prototype.show_or_hide_required_message = function()
	{
		let must_show = false;
		if (this.is_creating()) {
			must_show = true;
		}
		else {
			const selected_tab_id = this.main_tabs.get_selected_tab_id();
			must_show = selected_tab_id === "settings" ? true : false;
		}
		adm_form_qna_maker.uber.show_or_hide_required_message.call(this, must_show); //invokes the base class method
	};

	adm_form_qna_maker.prototype.uri_get_entity = function()
	{
		return inConcert.app.getBaseDomain() + this.get_uri;
	};

	adm_form_qna_maker.prototype.get_validation_def = function()
	{
		let definitions = [];

		if (this.is_creating()) {
			definitions.push({
				control_id: "adm_form_qna_maker_creating_kb_name",
				messages: {
					required: "<%=_T('Please enter the Knowledge base Name.')%>",
					id:       "<%=_T('Only letters, numbers, dashes and dots are allowed in the Knowledge base Name.')%>"
				}
			});
		}

		return definitions;
	};

	adm_form_qna_maker.prototype.make_save_waiting_msg = function(entity)
	{
		entity = entity || this.get();
		const is_creating = this.is_creating();

		let waiting_message;
		if (is_creating) {
			waiting_message = FORMAT("<%=_T('Please wait, creating knowledge base {0}')%>", entity.name);
		}
		else {
			waiting_message = FORMAT("<%=_T('Please wait, updating knowledge base {0}')%>", entity.name);
		}

		return {
			waiting_message: waiting_message
		};
	};

	adm_form_qna_maker.prototype.save_execute = function(after_save, entity)
	{
		const self = this;

		const is_creating = this.is_creating();

		var callback = function(success, response) {
			self.on_saved(success, entity, after_save, response);
		};

		const qnAMakerAPI = inConcert.app.getQnAMakerAPI();
		if (is_creating) {
			delete entity.isNew;
			qnAMakerAPI.createKnowledgeBase(entity, callback);
		}
		else {
			const kb_id = entity.kbId;
			qnAMakerAPI.updateKnowledgeBase(kb_id, entity, callback);
		}
	};

	adm_form_qna_maker.prototype.__get_knowledge_base_operation_detail = function(info, is_creating, callback = $.noop)
	{
		const self = this;

		let dots = "";
		var show_waiting = function() {
			dots = dots === "" ? "..." : "";
			self.dashboard.show_waiting("<%=_T('Hang tight, this should only take a minute... Updating your Knowledge base...')%>" + dots);
		};

		var recursive_get_operation_detail = function(success, operation_detail)
		{
			show_waiting();
			if (!success) {
				if (!is_creating) {
					let message = "<%=_T('Error Retrieving Knowledge base status.')%>";
					let message_type = "Error";
					let timeout_ms = false; //infinite
					self.show_status_message(message, message_type, timeout_ms);
					let err = operation_detail;
					self.dashboard.hide_waiting();
					callback(false, err);
					return;
				}
				callback(true, {});
				return;
			}

			const operation_state = operation_detail.operationState.toUpperCase();
			if (operation_state === "RUNNING" || operation_state === "NOTSTARTED") {
				show_waiting();
				window.setTimeout(function() {
					show_waiting();
					const qnAMakerAPI = inConcert.app.getQnAMakerAPI();
					qnAMakerAPI.getKnowledgeBaseOperationDetail(operation_detail.operationId, function(success, response) {
						recursive_get_operation_detail(success, response);
					});
				}, 1000);
				return;
			}
			show_waiting();
			callback(true, operation_detail);
		};

		recursive_get_operation_detail(true, info);
	};

	adm_form_qna_maker.prototype.__load_knowledge_base = function(kb_id, is_creating, callback = $.noop)
	{
		const self = this;

		let dots = "";
		var show_waiting = function() {
			dots = dots === "" ? "..." : "";
			self.dashboard.show_waiting("<%=_T('Hang tight, this should only take a minute... Loading your Knowledge base...')%>" + dots);
		};

		show_waiting();

		const qnAMakerAPI = inConcert.app.getQnAMakerAPI();
		qnAMakerAPI.getKnowledgeBase(kb_id, function(success, response)
		{
			if (!success) {
				if (!is_creating) {
					let message = "<%=_T('Error Retrieving Knowledge base.')%>";
					let message_type = "Error";
					let timeout_ms = false; //infinite
					self.show_status_message(message, message_type, timeout_ms);
					let err = response;
					self.dashboard.hide_waiting();
					callback(false, err);
					return;
				}
			}
			callback(true, response);
		});
	};

	adm_form_qna_maker.prototype.__get_knowledge_base_detail = function(kb_id, callback = $.noop)
	{
		const self = this;

		let dots = "";
		var show_waiting = function() {
			dots = dots === "" ? "..." : "";
			self.dashboard.show_waiting("<%=_T('Hang tight, this should only take a minute... Loading your Knowledge base detail...')%>" + dots);
		};

		show_waiting();

		const qnAMakerAPI = inConcert.app.getQnAMakerAPI();
		qnAMakerAPI.getKnowledgeBaseDetail(kb_id, function(success, knowledge_base_detail)
		{
			self.dashboard.hide_waiting();
			if (success) {
				let entity = self.get_entity_instance();
				if (entity) {
					entity.detail = knowledge_base_detail;
					self.set_entity_instance(entity);
					self.__show_hide_publish_info(knowledge_base_detail);
					self.__show_hide_settings_info(knowledge_base_detail);
				}
				callback(true, knowledge_base_detail);
				return;
			}
			let message = "<%=_T('Error Retrieving Knowledge base detail.')%>";
			let message_type = "Error";
			let timeout_ms = false; //infinite
			self.show_status_message(message, message_type, timeout_ms);
			let err = knowledge_base_detail;
			callback(false, err);
		});
	};

	adm_form_qna_maker.prototype.on_saved = function(success, entity, after_save, response)
	{
		const self = this;

		const is_creating = this.is_creating();
		adm_form_qna_maker.uber.on_saved.call(this, success, entity, after_save, response); //invokes the base class method

		if (success) {
			this.m_deleted_questions_ids = [];
			this.m_added_questions = [];
			this.m_updated_questions = [];

			if (is_creating) {
				let qna_maker = entity;
				qna_maker.inconcertInfo = response;
				qna_maker.inconcertInfo.name = qna_maker.name;
				this.set_entity_instance(qna_maker);
				delete qna_maker.name;
				this.set(true, qna_maker);
			}
			this.__get_knowledge_base_operation_detail(response, is_creating, function(success) {
				if (!success) {
					self.dashboard.hide_waiting();
					return;
				}
				self.__load_knowledge_base(entity.kbId, is_creating, function(success, resp) {
					self.dashboard.hide_waiting();
					if (!success) {
						return;
					}
					let qna_maker = resp || {};
					const qna_documents = qna_maker.qnaDocuments || [];
					self.__fill_questions_grid(qna_documents);
					self.__get_knowledge_base_detail(entity.kbId);
				});
			});
		}
		else {
			this.dashboard.hide_waiting();
		}
	};

	adm_form_qna_maker.prototype.err_save_messages = function(err)
	{
		let title = "<%=_T('Unable to save Knowledge base.')%>";
		let server_err_msg = "<%=_T('Unrecognized error. Contact your administrator.')%>";
		let error_detail = "";

		if (err.result && (err.result.toUpperCase() === "FORBIDDEN")) {
			server_err_msg = "<%=_T('You dont have permissions for this action.')%>";
		}
		else {
			const qnAMakerAPI = inConcert.app.getQnAMakerAPI();
			const err_obj = qnAMakerAPI.getCognitiveErrorObject(err);
			if (err_obj) {
				if (err_obj.message) {
					server_err_msg = FORMAT("<%=_T('Error: {0} Code: {1}')%>", err_obj.message, (err_obj.code || "-"));
				}
				if (err_obj.details) {
					for (let i = 0; i < err_obj.details.length; i++) {
						let detail = err_obj.details[i];
						if (i > 0) {
							error_detail = "\n\n" + error_detail + ",";
						}
						error_detail = error_detail + FORMAT("<%=_T('Message: {0}Target: {1}Code:{2}')%>", (detail.message || "") + "\n", (detail.target || "") + "\n", (detail.code || "") + "\n");
					}
				}
			}
		}

		this.show_popup_message({
			messages: {
				title  : title,
				message: server_err_msg,
				detail : error_detail,
				type   : "error"
			}
		});
	};

	adm_form_qna_maker.prototype.__delete_knowledge_base = function()
	{
		const self = this;

		const entity = this.get_entity_instance();
		const kb_id = entity.inconcertInfo.kbId;
		const display_name = entity.inconcertInfo.name;

		this.dashboard.hide_status_message();

		var erase = function()
		{
			self.dashboard.show_waiting(FORMAT("<%=_T('Please wait, deleting Knowledge base {0}')%>", display_name));
			const qnAMakerAPI = inConcert.app.getQnAMakerAPI();

			var callback = function(success, response)
			{
				self.dashboard.hide_waiting();
				let message, message_type, timeout_ms;
				if (success) {
					message = FORMAT("<%=_T('{0} Knowledge base was successfully deleted.')%>", display_name);
					message_type = "Info";
					self.__show_creating_panel();
					self.set_title("<%=_T('QnA Maker - Creating Knowledge base')%>");
					self.set_new_flag(true);
					self.set_entity_instance(null);
					self.m_deleted_questions_ids = [];
					self.m_added_questions = [];
					self.m_updated_questions = [];
				}
				else {
					let err = response;
					let server_err_msg;
					if (err.result && (err.result.toUpperCase() === "FORBIDDEN")) {
						server_err_msg = "<%=_T('You dont have permissions for this action.')%>";
					}
					else {
						server_err_msg = qnAMakerAPI.getCognitiveErrorMsg(err);
					}
					message = FORMAT("<%=_T('Error publishing Knowledge base {0}.')%>", display_name) + " " + server_err_msg;
					message_type = "Error";
					timeout_ms = false; //infinite
				}
				self.show_status_message(message, message_type, timeout_ms);
			};

			qnAMakerAPI.deleteKnowledgeBase(kb_id, callback);
		};

		var eraseConfirmation = function()
		{
			let message = "<%=_T('Do you want to delete Knowledge base {0} ?')%>";
			message = FORMAT( message, display_name );

			$.inconcertPopup.Show(
				"<%=_T('Confirm')%>",
				message,
				jQuery.inconcertPopup.ButtonSets.BS_YESNO,
				function(response) {
					if (response.toLowerCase() !== "yes") {
						return;
					}
					erase();
				},
				undefined,
				{
					popup_id       : "conf_inconcertPopup",
					container_class: "confirmationPopUp",
					overlay_class  : "confirmationOverlay"
				}
			);
		};

		eraseConfirmation();
	};

	adm_form_qna_maker.prototype.__proceed_to_remove_kb_from_cassandra = function()
	{
		const self = this;

		this.dashboard.hide_status_message();

		var erase = function()
		{
			self.dashboard.show_waiting("<%=_T('Please wait, clearing Knowledge base information.')%>");

			var callback = function(success)
			{
				self.dashboard.hide_waiting();
				let message, message_type, timeout_ms;
				if (success) {
					message = "<%=_T('Knowledge base was successfully cleaned.')%>";
					message_type = "Info";
					self.__show_creating_panel();
					self.set_title("<%=_T('QnA Maker - Creating Knowledge base')%>");
					self.set_new_flag(true);
					self.set_entity_instance(null);
					self.m_deleted_questions_ids = [];
					self.m_added_questions = [];
					self.m_updated_questions = [];
					self.ready_for_save();
					self.editable_all_crtls();
					self.readonly_control($("#adm_form_qna_maker_updating_kb_id"), true);
					$("#adm_form_qna_maker_creating_kb_name").focus();
				}
				else {
					message = "<%=_T('Error clearing Knowledge base.')%>";
					message_type = "Error";
					timeout_ms = false; //infinite
				}
				self.show_status_message(message, message_type, timeout_ms);
			};

			const qnAMakerAPI = inConcert.app.getQnAMakerAPI();
			qnAMakerAPI.removeKnowledgeBaseOfOCC(callback);
		};

		erase();
	};

	adm_form_qna_maker.prototype.__show_hide_publish_info = function(qna_detail = {})
	{
		const $publish_never_message = $("#adm_form_qna_maker_updating_settings_publish_never_message");
		const $publish_info = $("#adm_form_qna_maker_updating_publish_info");
		if (!qna_detail.lastPublishedTimestamp) {
			$publish_info.hide();
			$publish_never_message.show();
			return;
		}
		$publish_never_message.hide();
		$publish_info.show();

		const last_published_date = make_string_date(qna_detail.lastPublishedTimestamp);
		const $publish_date = $publish_info.find("[publish_date]");
		$publish_date.text(last_published_date);
	};

	adm_form_qna_maker.prototype.__show_hide_settings_info = function(qna_detail = {})
	{
		// last modified date
		const $creation_date = $("#adm_form_qna_maker_updating_creation_date");
		const creation_date = make_string_date(qna_detail.createdTimestamp);
		$creation_date.val(creation_date);
		// last modified date
		const $last_modified_date = $("#adm_form_qna_maker_updating_last_modified_date");
		const last_modified_date = make_string_date(qna_detail.lastChangedTimestamp);
		$last_modified_date.val(last_modified_date);
	};

	adm_form_qna_maker.prototype.__pusblish_knowledge_base = function()
	{
		const self = this;

		const entity = this.get_entity_instance();
		const kb_id = entity.inconcertInfo.kbId;
		const display_name = entity.inconcertInfo.name;

		this.dashboard.hide_status_message();

		var publish = function()
		{
			self.dashboard.show_waiting(FORMAT("<%=_T('Please wait, publishing Knowledge base {0}')%>", display_name));
			const qnAMakerAPI = inConcert.app.getQnAMakerAPI();

			var callback = function(success, response)
			{
				if (success) {
				// SUCCESS
					let message = FORMAT("<%=_T('{0} Knowledge base was successfully published.')%>", display_name);
					let message_type = "Info";
					let timeout_ms = 5000;

					self.__get_knowledge_base_detail(kb_id, function(success) {
						self.show_status_message(message, message_type, timeout_ms);
						if (success) {
							// TODO: CREO QUE NO HAGO NADA!!!
							return;
						}
						//TODO: HACER ALGO SI HAY ERROR, CREO QUE NO ?
					});
					return;
				}
				// ERROR
				let err = response;
				let server_err_msg;
				if (err.result && (err.result.toUpperCase() === "FORBIDDEN")) {
					server_err_msg = "<%=_T('You dont have permissions for this action.')%>";
				}
				else {
					server_err_msg = qnAMakerAPI.getCognitiveErrorMsg(err);
				}
				let message = FORMAT("<%=_T('Error publishing Knowledge base {0}.')%>", display_name)  + " " + server_err_msg;
				let message_type = "Error";
				let timeout_ms = false; //infinite
				self.show_status_message(message, message_type, timeout_ms);
			};

			// Publico
			qnAMakerAPI.publishKnowledgeBase(kb_id, null, callback);
		};

		// Comienza el proceso de publicacion
		publish();
	};

	adm_form_qna_maker.prototype.set_events = function()
	{
		const self = this;
		adm_form_qna_maker.uber.set_events.call(this); //invokes the base class method

		this.m_grd_questions_answers.set_events();

		$("#adm_form_qna_maker_updating_settings_delete_knowledge_base")
			.off("click")
			.on("click", function() {
				self.__delete_knowledge_base();
			});

		$("#adm_form_qna_maker_updating_settings_publish_knowledge_base")
			.off("click")
			.on("click", function() {
				self.__pusblish_knowledge_base();
			});

		$("#adm_form_qna_maker_error_delete_knowledge_base_of_cassandra")
			.off("click")
			.on("click", function() {
				self.__proceed_to_remove_kb_from_cassandra();
			});

		$("#adm_form_qna_maker_updating_settings_edit_knowledge_base")
			.off("click")
			.on("click", function() {
				self.set_readonly(false);
				self.reopen_view({
					extra_parameters:{
						show_view_readonly: false
					}
				});
			});

		// eventos de la grilla de metadata
		this.metadata_mng.set_events();
	};

	adm_form_qna_maker.prototype.remove = function(question, callback = $.noop)
	{
		if (!question) {
			throw("adm_form_qna_maker.prototype.remove. Undefined question.");
		}
		this.m_deleted_questions_ids.push(question.id);
		callback(true);
	};

	adm_form_qna_maker.prototype.unable_for_save = function(err, /*callback = $.noop*/)
	{
		const self = this;

		adm_form_qna_maker.uber.unable_for_save.call(this, err, function()
		{
			self.__show_error_panel();
			self.hide_header_custom_buttons();

			const $err_404 = $("#adm_form_qna_maker_error_delete_knowledge_base_of_cassandra_message");
			const $delete_knowledge_base_of_cassandra = $("#adm_form_qna_maker_error_delete_knowledge_base_of_cassandra");

			$err_404.hide();
			$delete_knowledge_base_of_cassandra.hide();

			let message = "<%=_T('Server error. Unable to retrieve information.')%>";
			if (typeof err === "object" && typeof err.result === "string") {
				if ((err.result.toUpperCase() === "FORBIDDEN")) {
					message = "<%=_T('Unable to retrieve information. You dont have permissions for this action.')%>";
				}
				else {
					const qnAMakerAPI = inConcert.app.getQnAMakerAPI();
					const err_obj = qnAMakerAPI.getCognitiveErrorObject(err);
					if (err_obj && (err_obj.code || "").toUpperCase() === "KBNOTFOUND") {
						message = "<%=_T('Knowledge base not found in QnA Maker.')%>";
						$err_404.show();
						$delete_knowledge_base_of_cassandra.show();
					}
				}
			}

			const timeout_ms = false; //infinite
			self.show_status_message(message, "Error", timeout_ms);
			self.set(false);
		}); //invokes the base class method
	};

	adm_form_qna_maker.prototype.readonly = function()
	{
		adm_form_qna_maker.uber.readonly.call(this); //invokes the base class method
		$("#adm_form_qna_maker_updating_settings_publish_knowledge_base").hide();
		$("#adm_form_qna_maker_updating_settings_delete_knowledge_base").hide();
		$("#adm_form_qna_maker_updating_questions_edit_popup_create").hide();
		$("#adm_form_qna_maker_updating_questions_edit_popup_create_and_continue").hide();
	};

	adm_form_qna_maker.prototype.no_close = function()
	{
		this.readonly();
		this.set_title(this.view_title);
		$("#adm_form_qna_maker_updating_settings_edit_knowledge_base").show();
		this.hide_header_custom_buttons();
		this.m_grd_questions_answers.hide_actions_rows();
	};

	adm_form_qna_maker.prototype.ready_for_save = function()
	{
		adm_form_qna_maker.uber.ready_for_save.call(this);
		if (this.instance_config?.show_view_readonly !== false) {
			this.no_close();
		}
	};

}());
