/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 11 */

/* global inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function ()
{
	"use strict";

	
//------------------------------------------------------
	// knowledgebase navigation config
	var knowledgebase_config = function()
	{
		return {
			id: "knowledge_fusion",
			name: "<%=_T('Knowledge Fusion')%>",
			can_show: function()
			{
				if (!inConcert.app.has_configured_knowledge_fusion_in_vcc()) {
					return false;
				}
				return true;
			},
			permissions: {
				view: "view_qna_maker",
				save: "save_qna_maker",
				erase: "delete_qna_maker",
				permission_type: "vcc"
			},
			class_name: "itemQnAMaker",
			i_class_name: "far fa-question",
			view: {
				render: "/inconcert/apps/knowledge_base/pages/view_knowledge_fusion_content_default.<%=_L%>.html#grunt-cache-bust",
				must_init_full_screen: () => {
					return (!inConcert.app.has_configured_qna_maker_in_vcc() && inConcert.app.has_configured_knowledge_fusion_in_vcc());
				}
			}
		};
	};

	// QnA Maker navigation config
	var qna_maker_config = function()
	{
		const qnAMakerAPI = inConcert.app.getQnAMakerAPI();

		return {
			id: "qna_maker",
			name: "<%=_T('QnA Maker')%>",
			can_show: function()
			{
				if (!inConcert.app.has_configured_qna_maker_in_vcc()) {
					return false;
				}
				return true;
			},
			permissions: {
				view: "view_qna_maker",
				save: "save_qna_maker",
				erase: "delete_qna_maker",
				permission_type: "vcc"
			},
			class_name: "itemQnAMaker",
			i_class_name: "far fa-question",
			api: {
				get: function(callback) {
					// 1 - Obtengo la knowledge base que tiene el VCC y su QnaDocuments
					qnAMakerAPI.getVccKnowledgeBase(function(success, knowledge_base_info) {
						if (!success) {
							let err = knowledge_base_info;
							if (err.cognitiveServiceError && err.cognitiveServiceError.errType === "inconcert" && err.cognitiveServiceError.code === 404) {
								callback(true, err);
								return;
							}
							callback(false, err);
							return;
						}
						// 2 - Obtengo el detalle de la knowledge base
						let kbId = knowledge_base_info.inconcertInfo.kbId;
						qnAMakerAPI.getKnowledgeBaseDetail(kbId, function(success, knowledge_base_detail) {
							if (!success) {
								let err = knowledge_base_detail;
								if (err.cognitiveServiceError && err.cognitiveServiceError.errType === "inconcert" && err.cognitiveServiceError.code === 404) {
									callback(true, err);
									return;
								}
								callback(false, err);
								return;
							}
							knowledge_base_info.detail = knowledge_base_detail;
							callback(true, knowledge_base_info);
						});
					});
				}
			},
			only_frm: true,
			view: {
				render: "/inconcert/apps/knowledge_base/pages/view_knowledge_base_content_default.<%=_L%>.html#grunt-cache-bust",
				must_init_full_screen: () => {
					return (inConcert.app.has_configured_qna_maker_in_vcc() && !inConcert.app.has_configured_knowledge_fusion_in_vcc());
				}
			},
			form: {
				instance: inConcert.adm_form_qna_maker,
				save_button: {
					update_caption: "<%=_T('Update and Train')%>"
				},
				cancel_button: {
					initial_state : "enabled"
				}
			}
		};
	};

	//------------------------------------------------------
	// get all navigation config
	const get_navigation_config = function()
	{
		return {
			knowledge_fusion          : knowledgebase_config(),
			qna_maker                 : qna_maker_config()
		};
	};

	function knowledge_base_config() {}

	if (typeof inConcert.knowledge_base_config !== "function") {
		inConcert.knowledge_base_config = knowledge_base_config;
	}

	knowledge_base_config.prototype.get_app_config = function()
	{
		return {
			id         : "knowledge_base",
			name       : "<%=_T('Knowledge base')%>",
			selected   : false,
			class_name : "appAdministration",
			main_html  : "/inconcert/apps/knowledge_base/pages/conf_knowledge_base_main.<%=_L%>.html#grunt-cache-bust",
			navigation : get_navigation_config(),
			get_skin: function() {
				return "skinOCCNew";
			},
			can_show   : function() {
				return inConcert.permissions.to_category("knowledge_base", "view")
				&& (inConcert.app.has_configured_qna_maker_in_vcc() || inConcert.app.has_configured_knowledge_fusion_in_vcc());
				
			},
			css : [
				{ id: "adm_messages_css"                            , href: "/inconcert/apps/administration/css/administrationRules.css#grunt-cache-bust"                       },
                { id: "skin_occ_internal_forms_css"                 , href: "/inconcert/apps/configuration/css/skin-2021/masterInternalPageForms.css#grunt-cache-bust"          },
                { id: "skin_occ_kb_css"                             , href: "/inconcert/apps/knowledge_base/css/skin-2021/knowledgeBase.css#grunt-cache-bust"                   },
                { id: "skin_occ_adm_messenger_css"                 	, href: "/inconcert/apps/administration/css/skin-2021/administrationPageMessenger.css#grunt-cache-bust"     },
                { id: "skin_occ_master_buttons_css"                 , href: "/inconcert/apps/agent/css/skin-2021/masterButtons.css#grunt-cache-bust"                            },
				{ id: "skin_occ_inbox_master_css"                   , href: "/inconcert/apps/agent/css/skin-2021/inboxMaster.css#grunt-cache-bust"                              },
                { id: "skin_occ_messages_tooltips_css"              , href: "/inconcert/apps/agent/css/skin-2021/genericMessagesAndTooltips.css#grunt-cache-bust"               },
				{ id: "skin_occ_adm_master_css"                     , href: "/inconcert/apps/administration/css/skin-2021/administrationMaster.css#grunt-cache-bust"            },
				{ id: "skin_occ_inbox_master_popups_css"            , href: "/inconcert/apps/agent/css/skin-2021/inboxMasterPopUps.css#grunt-cache-bust"                        },
				{ id: "skin_occ_master_client_window_css"           , href: "/inconcert/apps/agent/css/skin-2021/masterWindowCommon.css#grunt-cache-bust"                       },
				{ id: "skin_occ_custom_visual_search_css"           , href: "/inconcert/apps/agent/css/skin-2021/customVisualSearch.css#grunt-cache-bust"                       }
			]
		};
	};
}());
