local helpers = require "test_helpers"

local inconcert_devel = assert(os.getenv("INCONCERT_DEVEL"))

local socket = require "socket"
local cosmo = require "cosmo"
local Tasks = require "lua_ostasks"

--ejemplo de invocacion: lua5.1 run.lua --host 192.168.22.210 --port 8082
local args = helpers.CommandLine[[
logstorage testing parameters
    -h,--host (string default 127.0.0.1)        The webserver address
    -p,--port (number default ?)                The webserver port
    -r,--reuse                                  Attempts to connect to a running webserver instead of starting up one
    -u,--pause                                  Pauses the test before stopping helper processes.
    <tests...> (default ?)                      Tests to run (leave in blank to run all tests)
]]

local local_ip = helpers.GetHostAddress()

local config_env

if args.reuse then
	local config_file = assert(io.open("test.conf", "rb"))
	config_env = json.decode(config_file:read("*a"))
	config_file:close()
else
	config_env = {
		webserver = {
			host = args.host or local_ip,
			port = args.port or helpers.RandomPort(),
			code_path = inconcert_devel .. "/sources/inconcertwebhandler/data/lua",
			config_path = inconcert_devel .. "/sources/agent/inconcert/apps/logstorage/unittest/config",
			webroot = inconcert_devel .. "/sources/agent/inconcert/apps"
		},
		storage_directory = helpers.is_windows and os.getenv("TMP") or "/tmp"
	}
end

local launcher = helpers.TestLauncher("logstorage")
for _,module in ipairs{
	"test"
} do
	require(module).AddTests(launcher, config_env)
end


local task_set = Tasks.new(os.getenv("BUILD_TAG") or "logstorage_unittest_tasks")

if not args.reuse then
	local f = assert(io.open("config/server_config.lua.template", "rb"))
	local template = cosmo.fill(f:read("*a"), config_env)
	f:close()
	f = assert(io.open("config/server_config.lua", "wb"))
	f:write(template)
	f:close()
end

helpers.Config.CreateFile(
	("%s/sources/agent/inconcert/apps/logstorage/unittest/config/inconcert/apps_config.lua"):format(inconcert_devel),
	config_env
)

---
-- Wait until the webserver is accepting connections
--
local function StartWebserver (task_set, config)

	local base_path = ("%s/sources/agent/inconcert/apps/logstorage/unittest/"):format(inconcert_devel)
	helpers.Process.StartAndWait(task_set, "WebServer", config.webserver.host, config.webserver.port, {
		("%s/bin/inconcertwebhandler_d"):format(inconcert_devel),
		"--ConfigFile=" .. base_path .. "config/server_config.lua"
	})
end

do
	local f = io.open("deflated_data.bin", "rb")
	config_env.compressed_data = f:read("*a")
	f:close()
end


local ok, err = xpcall(function()

	if not args.reuse then
		StartWebserver(task_set, config_env)
	end

	launcher.Run(args.tests)

end, debug.traceback)

if not ok then
	io.stderr:write(err)
end

if args.pause then
	os.execute("pause")
end

if not args.reuse then
	task_set:killall()
end
