local asserts = require "luatestlauncher.asserts"

local http = require "socket.http"
local ltn12 = require "ltn12"

module(..., luatestlauncher.asserts.testcase, package.seeall)


---
-- Sends a properly compressed log
--
function post_deflated_data (env, launcher)

	local url = ("http://%s:%d/inconcert/apps/logstorage/upload"):format(
		env.webserver.host, env.webserver.port
	)
	local body = env.compressed_data
	local resp = {}
	local response, status, headers, statusline = http.request{
		url = url,
		method = "POST",
		headers = { Host = env.webserver.host,
					Connection = "keep-alive",
					["Content-Length"] = #body,
					["Content-Type"] = "text/plain"
		},
		sink = ltn12.sink.table(resp),
		source = ltn12.source.string(body)
	}
	local response_body = table.concat(resp)
	assert_equal(200, status)
end

---
-- Sends an improperly compressed log
--
function post_improperly_compressed_data (env, launcher)
	local body = ([[random stuff blabla]]):rep(1024)
	local url = ("http://%s:%d/inconcert/apps/logstorage/upload"):format(
		env.webserver.host, env.webserver.port
	)

	local resp = {}
	local response, status, headers, statusline = http.request{
		url = url,
		method = "POST",
		headers = { Host = env.webserver.host,
					Connection = "keep-alive",
					["Content-Length"] = #body,
					["Content-Type"] = "text/plain"
		},
		sink = ltn12.sink.table(resp),
		source = ltn12.source.string(body)
	}
	local response_body = table.concat(resp)
	assert_equal(400, status)
end


---
-- Post a single error
function post_single_error (env, launcher)
	local body = [[data=%7B%22timestamp%22%3A1358184373%2C%22level%22%3A%22ERROR%22%2C%22data%22%3A%7B%22message%22%3A%22blablablabla%22%7D%7D&layout=MyJsonLayout]]
	local url = ("http://%s:%d/inconcert/apps/logstorage/log_error"):format(
		env.webserver.host, env.webserver.port
	)

	local resp = {}
	local response, status, headers, statusline = http.request{
		url = url,
		method = "POST",
		headers = { Host = env.webserver.host,
					Connection = "keep-alive",
					["Content-Length"] = #body,
					["Content-Type"] = "application/x-www-form-urlencoded"
		},
		sink = ltn12.sink.table(resp),
		source = ltn12.source.string(body)
	}
	local response_body = table.concat(resp)
	assert_equal(200, status)
end



function AddTests (launcher, env)

	for k,v in ipairs{
		{"Post deflated data", "post_deflated_data", "LogStorage"},
		{"Post improperly compressed data", "post_improperly_compressed_data", "LogStorage"},
		{"Post a single error", "post_single_error", "LogStorage"},
	}
	do
		local description, name, test_fun, category = v[1], v[2], _M[v[2]], v[3]
		launcher.AddTest(description, {
			name = name, test = function(...) test_fun(env, ...) end
		}, category)
	end
end
