module(..., package.seeall, inconcert.orbit.application.Decorate)
local utils = require "shared.apps.utils"
local permissions = require "shared.permissions.permissions"
local wsapi = require "wsapi"
wsapi.request = require "wsapi.request"
local Fullcontact = require "fullcontact"
local m_moduleLocation = select(2, ...)

local function ExecuteFullcontactMethod (callback)
	local outbound_engine = require "apps.outbound_engine"
	outbound_engine.InitializeFullcontact(callback)
end

local m_blackListAPI
local function GetBlackListApi ()
	m_blackListAPI = m_blackListAPI or Fullcontact.Dialer:GetBlackListApi()
	return m_blackListAPI
end

--[[********************************************************************************************
	BLACKLIST IMPORTATIONS
--********************************************************************************************--]]

local function __RenderImport (web)
	_M.response.DontCache(web)

	local env = { _T = m_htmlRender.extension.makeTextTranslator() }
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[OUTBOUNDENGINE_BLACKLIST:__RenderImport] Render form Imports for user %s,", userId)

	local htmlFile = m_appLocation .. "/pages/frm_blacklist_imports_content.html"
	local content = m_htmlRender:RenderPageLP(htmlFile, web, env)
	if not content then
		LogError("[OUTBOUNDENGINE_BLACKLIST:__RenderImport] HTML Imports does not exist: %s", htmlFile)
	end

	return content
end

local function __Search (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local encodedInfo = web.input["post_data"]

	LogInfo("[OUTBOUNDENGINE_BLACKLIST:__Search] Loading Imports in '%s' vcc%s. Posted Info: '%s'", vcc, utils.MakeLogAllegroVccMsg(vcc), encodedInfo)

	web.headers["Content-Type"] = "application/json"
	local success, data

	local info = web.request
	ExecuteFullcontactMethod(function()
		success, data = GetBlackListApi():ListImportations({vcc = vcc})
	end)
	if success then
		local rows = data or {}
		LogInfo("[OUTBOUNDENGINE_BLACKLIST:__Search] 1 - Filtering rows")
		--LogDebug("[OUTBOUNDENGINE_BLACKLIST.STATES:__Search] ===> 1.1 - rows: %s", json.encode(rows))

		local totalUnfilteredRows = utils.GetRowCount(rows)
		local totalAdded = 0
		local filteredRows = {}

		for id, row in ipairs(rows) do
			row.Id = row.ID
			--row.Description = row.Description or row.Id
			if utils.MatchFilter(row, info) then
				totalAdded = totalAdded + 1
				table.insert(filteredRows, row)
			end
		end
	
		LogInfo("[OUTBOUNDENGINE_BLACKLIST:__Search] 2 - Sorting rows %d", #filteredRows)
		--LogDebug("[OUTBOUNDENGINE_BLACKLIST:__Search] ===> 2.1 - filteredRows: %s", json.encode(filteredRows or {}))
	
		--include only rows that belong to logged user vcc
		local response = {
			rows = filteredRows,
			last = "",
			total_rows = totalAdded,
			total_unfiltered_rows = totalUnfilteredRows
		}
		local responseParams = {
			MakeSortingValue = function(row)
				assert(row)
				return tostring(row.Description or row.Id):lower()
			end
		}
		data = utils.MakeSearchResponse(response, info, responseParams)
	end

	if success and data then
		--LogDebug("[OUTBOUNDENGINE_BLACKLIST:__Search] Loaded Importations. %s", json.encode(data))
		return data
	end

	-- Error
	local err = data
	web.status = 500
	LogError("[OUTBOUNDENGINE_BLACKLIST:__Search] Error loading Imports. Err: %s", json.encode(err))

	return err
end

--[[
Return
]]--
local function __ListSystemAndVcc (web, addressType)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[OUTBOUNDENGINE_BLACKLIST:__ListSystemAndVcc] Retrieving All blacklists, addressType: %s. user %s, vcc %s%s", addressType, userId, vcc, utils.MakeLogAllegroVccMsg(vcc))

	local success, vccAndSystemBlacklists
	ExecuteFullcontactMethod(function()
		success, vccAndSystemBlacklists = GetBlackListApi():ListVccAndSystemImportations({vcc = vcc, addressType = addressType})
	end)
	if success and vccAndSystemBlacklists then
		local encodedVccAndSystemBlacklists = json.encode(vccAndSystemBlacklists)
		--LogDebug("[OUTBOUNDENGINE_BLACKLIST:__ListSystemAndVcc] Retrieved all blacklists addressType: %s. vcc %s, encodedVccAndSystemBlacklists: %s.", addressType, vcc, encodedVccAndSystemBlacklists)
		return encodedVccAndSystemBlacklists
	end

	web.status = 404
	return {result = "NOT_FOUND"}
end

--[[
Return
]]--
local function __GetImport (web, importId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[OUTBOUNDENGINE_BLACKLIST:__GetImport] Retrieving Import %s, user %s, vcc %s%s", importId, userId, vcc, utils.MakeLogAllegroVccMsg(vcc))

	local success, import
	ExecuteFullcontactMethod(function()
		success, import = GetBlackListApi():GetImportation(importId, vcc)
	end)
	if success and import then
		local encodedImport = json.encode(import)
		LogInfo("[OUTBOUNDENGINE_BLACKLIST:__GetImport] Retrieving Import %s vcc %s, data: %s.", importId, vcc, encodedImport)
		return encodedImport
	end

	web.status = 404
	return {result = "NOT_FOUND"}
end

--[[
]]--
local function __DeleteImport (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local encodedInfo = web.input["post_data"]

	LogInfo("[OUTBOUNDENGINE_BLACKLIST:__DeleteImport] Deleting Import in '%s' vcc%s. Posted Info: '%s'", vcc, utils.MakeLogAllegroVccMsg(vcc), encodedInfo)

	web.headers["Content-Type"] = "application/json"

	local success, data
	local importationId = json.decode(encodedInfo)[1]
	ExecuteFullcontactMethod(function()
		success, data = GetBlackListApi():RemoveImport(importationId, vcc, inconcert.authUser.userName)
	end)

	if success then
		LogInfo("[OUTBOUNDENGINE_BLACKLIST:__DeleteImport] Success deleting Import %s.", encodedInfo)
		return true
	end

	-- Error
	local err = data
	web.status = 500
	LogError("[OUTBOUNDENGINE_BLACKLIST:__DeleteImport] Error deleting Import %s. Err: %s", encodedInfo, json.encode(err))

	return err
end

local function __StartImport (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local encodedInfo = web.input["post_data"]

	LogInfo("[OUTBOUNDENGINE_BLACKLIST:__StartImport] Starting Imports in '%s' vcc%s. Posted Info: '%s'", vcc, utils.MakeLogAllegroVccMsg(vcc), encodedInfo)

	web.headers["Content-Type"] = "application/json"
	local success, data

	local info = json.decode(encodedInfo)
	info.VCC = vcc
	local fileInfo = {
		FileId = info.FileInfo.id
	}
	local allegroRepositoryModel = require("apps.allegro.controllers.repository").GetModel()
	local temporalCsvFile = allegroRepositoryModel.GetTemporalUploadedFile(fileInfo)
	if temporalCsvFile then
		info.UserId = inconcert.authUser.userName
		info.VCC = vcc
		info.ScopeType = "vcc"
		ExecuteFullcontactMethod(function()
			-- Comienza la importacion y luego de culminar la misma en callback va a recargar las addresses contra el OE Client.
			success, data = GetBlackListApi():StartImport(info, temporalCsvFile, function(addressesByProcesses)
				--ReloadProcessesContacts(addressesByProcesses)
			end)
		end)
		--LogDebug("[OUTBOUNDENGINE_BLACKLIST:__StartImport] --> ENDED Imports success: %s, data: %s.", success, json.encode(data or {}))
		if success then
			LogInfo("[OUTBOUNDENGINE_BLACKLIST:__StartImport] Success Started Imports in Vcc '%s'. Data: %s.", vcc, encodedInfo)
			allegroRepositoryModel.DeleteTemporalUploadedFile(fileInfo)
			return true
		else
			data.code = data.code or data.type
		end
	else
		LogWarning("[OUTBOUNDENGINE_BLACKLIST:__StartImport] - Unable to load temporal CSV file. Temporal CSV file does not exist.file id '%s'.", fileInfo.FileId)
		data = {reason = "Temporal CSV File does not exist", fileId = fileInfo.FileId, code = -105}
	end

	-- Error
	local err = data
	web.status = 500
	LogError("[OUTBOUNDENGINE_BLACKLIST:__StartImport] Error Starting Imports %s. Err: %s", encodedInfo, json.encode(err))

	return err
end


--[[********************************************************************************************
	UPLOAD
--********************************************************************************************--]]

--[[

Return

]]--
local function __GetAllowedCsvHeaderInfo (web)
	LogInfo("[OUTBOUNDENGINE_BLACKLIST:__GetAllowedCsvHeaderInfo] Loading Blacklist Importation Allowed CSV Header Columns.")
	local _, csvAllowedColumns = GetBlackListApi():GetCsvAllowedColumns()
	return json.encode(csvAllowedColumns)
end

--[[

Return

]]--

local function ValidateCSVFile (fileId, LoadMappingFieldsFn)
	assert(type(fileId) == "string")
	assert(type(LoadMappingFieldsFn) == "function")

	LogInfo("[OUTBOUNDENGINE_BLACKLIST:ValidateCSVFile] Validating CSV File '%s'.", fileId)

	local allegroRepositoryModel = require("apps.allegro.controllers.repository").GetModel()
	local csv = allegroRepositoryModel.GetTemporalUploadedFile({
		FileId = fileId
	})

	--LogDebug("0.4 csv: %s", csv)
	if not csv then
		local msg = "CSV File does not exist"
		return false, {
			type = "CSV FILE DOES NOT EXIST",
			code = -101,
			reason = msg
		}
	end

	--LogDebug("0.5 csv: %s", json.encode(csv))
	local csvHeader = utils.ParseCSVHeader(csv)
	if not csvHeader then
		local msg = "Invalid CSV File"
		return false, {
			type = "INVALID CSV FILE",
			code = -105,
			reason = msg
		}
	end

	--LogDebug("1 csvHeader: %s", json.encode(csvHeader))
	local mapping = LoadMappingFieldsFn()
	--LogDebug("2 mapping: %s", json.encode(mapping))
	local missingRequiredFields = {}

	-- Valido los campos requeridos del mapeo contra el CSV
	for _, fieldInfo in ipairs(mapping) do
		if fieldInfo.IsRequired then
			local existRequiredField = false
			for index, csvFieldName in ipairs(csvHeader) do
				csvFieldName = utils.alltrim(csvFieldName):upper()
				if fieldInfo.Name == csvFieldName then
					existRequiredField = true
					break
				end
			end
			if not existRequiredField then
				table.insert(missingRequiredFields, fieldInfo.Name)
			end
		end
	end

	local totalMissingRequiredFields = #missingRequiredFields
	if totalMissingRequiredFields > 0 then
		local msg = string.format("Missing %d required fields in CSV File.", totalMissingRequiredFields)
		return false, {
			type = "MISSING SOME REQUIRED FIELDS",
			code = -102,
			reason = msg,
			missingRequiredFields = missingRequiredFields
		},
		csvHeader
	end

	allowedHeaderFields = csvHeader
	return true, csv, allowedHeaderFields
end

local function __GetUploadedCSVHeader (web, fileId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[OUTBOUNDENGINE_BLACKLIST:__GetUploadedCSVHeader] --> Retrieving CSV Header file '%s'. User %s, vcc %s", fileId, userId, vcc)

	-- local allegroImports = require "apps.allegro.controllers.outboundengine_imports"
	-- local success, csvHeader = allegroImports.GetUploadedCSVHeader(fileId)
	-- if success and csvHeader then
	-- 	LogDebug("[OUTBOUNDENGINE_BLACKLIST:__GetUploadedCSVHeader]------> csvHeader: %s", json.encode(csvHeader))
	-- 	return json.encode(csvHeader)
	-- end

	-- web.status = 404
	-- return {result = "NOT_FOUND"}

	local success, info, csvHeader = ValidateCSVFile(fileId, function()
		local _, csvAllowedColumns = GetBlackListApi():GetCsvAllowedColumns()
		return csvAllowedColumns
	end)

	local response = {
		result = "OK",
		csvHeader = csvHeader,
		info = {}
	}
	if success then
		return json.encode(response)
	end

	response.result = "ERROR"
	response.info = info

	web.status = 404
	return response
end

--[[********************************************************************************************
	DOWNLOAD
--********************************************************************************************--]]

local function __DownloadFailedAddresses (wsapi_env)
	local vcc = inconcert.authUser.virtualContactCenter
	local request = wsapi.request.new(wsapi_env)
	local importId = request.params["importid"]
	local updateId = request.params["update"]

	LogInfo("[OUTBOUNDENGINE_BLACKLIST:__DownloadFailedAddresses] Downloading Failed Addresses in '%s' vcc%s, importId: '%s', update number: '%s'.", vcc, utils.MakeLogAllegroVccMsg(vcc), importId, updateId)
	--
	local function ReturnErr (err)
		-- Error
		wsapi_env.status = 500
		LogError("[OUTBOUNDENGINE_BLACKLIST:__DownloadFailedAddresses] Error Downloading Failed Addresses. Err: %s", json.encode(err))
		local response = wsapi.response.new(200, {["Content-Type"]= "text/plain"})
		response:write(json.encode(err))
		response.headers["Response"] = "defaultError"
		response.status = err.httpCode or 500
		return response:finish()
	end
	--
	local success, info
	ExecuteFullcontactMethod(function()
		success, info = GetBlackListApi():GetImportFailedAddresses(importId, vcc, updateId)
	end)
	if not success then
		-- error
		local err = info
		return ReturnErr(err)
	end

	-- ok
	require "LuaOrm.Date"
	local updateDate = LuaOrm.Date(info.updateInfo.UpdateDate):toString("%Y-%m-%d %H:%M:%S")
	local suggestedFileName = string.format("%s(%s)_failed_addresses.CSV", importId, updateDate)
	-- nota: estos headers response estan mal! para señalar el status se usa, justamente, el status
	local headers = {}
	headers["Response"]            = "defaultError"
	headers["Content-Type"]        = "application/octet-stream"
	headers["Content-Length"]      = info.csvSize
	headers["Response"]            = "ok"
	headers["Content-Disposition"] = [[attachment; filename="]] .. suggestedFileName .. '"'
	headers["Set-Cookie"]          = "download_started=true; Path=/" -- used by the jquery download plugin

	LogInfo("[OUTBOUNDENGINE_BLACKLIST:__DownloadFailedAddresses] (END) Downloading CSV Failed addresses. CSV File Name '%s'.", suggestedFileName)

	--local coroutineIter = 0
	return 200, headers, coroutine.wrap(function()
		--coroutineIter = coroutineIter + 1
		--LogDebug("[OUTBOUNDENGINE_BLACKLIST.SHARED_FILES:__DownloadFailedAddresses] coroutine.yield - iter:%d", coroutineIter)
		coroutine.yield(info.csvFailedAddresses)
	end)
end

--[[********************************************************************************************
	BLACKLIST ADDRESSES
--********************************************************************************************--]]

local function __SearchAddresses (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local encodedInfo = web.input["post_data"]
	LogInfo("[OUTBOUNDENGINE_BLACKLIST:__SearchAddresses] Loading Blacklist Addresses in '%s' vcc %s. Posted Info: '%s'", vcc, utils.MakeLogAllegroVccMsg(vcc), encodedInfo)
	web.headers["Content-Type"] = "application/json"
	local success, data
	local info = web.request
	assert(type(info.query) == "table")
	info.listName = "addresses" -- tiene que ser el nombre de la lista que retorna ES
	info.query.VCC = {exact = true, values = {vcc}}
	ExecuteFullcontactMethod(function()
		success, data = utils.ExecuteESQueryForConfigurationViews(info, GetBlackListApi(), "SearchAddresses")
	end)
	if success and data then
		LogDebug("[OUTBOUNDENGINE_BLACKLIST:__SearchAddresses] Loaded Blacklist Addresses. %s", json.encode(data))
		return data
	end
	-- Error
	local err = data
	web.status = 500
	LogError("[OUTBOUNDENGINE_BLACKLIST:__SearchAddresses] Error loading Blacklist Addresses. Err: %s", json.encode(err))
	return err
end

---------------------------------------

m_htmlRender = nil
m_appLocation = nil

function MapUris (module, htmlRender, prefix, appLocation)
	local LogRequest = require("apps.agent.requestFilters").LogRequest
	local replyFn = utils.JsonReply
	local replyValFn = utils.JsonValidatorReply

	local function HasPermission (fn, actionId, responseFn)
		responseFn = responseFn or replyFn
		return responseFn .. LogRequest .. permissions:CheckPermission("outbound_engine", actionId, fn)
	end

	local jsonValReply = utils.JsonValidatorReply .. LogRequest
	local jsonReply = utils.JsonReply .. LogRequest
	local p = prefix

	-- RENDER FRM
	module:dispatch_get (__RenderImport                                    , p .. "/frm/render/"                  )
	-- IMPORTS
	module:dispatch_post (jsonValReply .. __Search                         , p .."/search/"                       )
	module:dispatch_get  (jsonReply .. __ListSystemAndVcc                  , p .. "/list_all/(.+)"                )
	module:dispatch_get  (HasPermission(__GetImport, "view_blacklist")     , p .. "/get/(.+)"                     )
	module:dispatch_post (HasPermission(__DeleteImport, "delete_blacklist"), p .. "/delete/"                      )
	module:dispatch_post (HasPermission(__StartImport , "save_blacklist")  , p .. "/start/"                       )
	-- UPLOAD
	module:dispatch_get  (jsonReply .. __GetUploadedCSVHeader              , p .. "/get_uploaded_csv_header/(.+)" )
	module:dispatch_get  (jsonReply .. __GetAllowedCsvHeaderInfo           , p .. "/csv/get_allowed_header_info/" )
	-- DOWNLOAD
	module:dispatch_wsapi(__DownloadFailedAddresses                       , p .. "/download/failed_addresses/(.*)")
	-- ADDRESSES
	module:dispatch_post (jsonValReply .. __SearchAddresses                , p .."/addresses/search/"             )

	m_htmlRender = htmlRender
	m_appLocation = appLocation
end
