module(..., package.seeall, inconcert.orbit.application.Decorate)

local utils = require "shared.apps.utils"
local permissions = require "shared.permissions.permissions"
local allegroExports = require "apps.allegro.controllers.outboundengine_exports"
local wsapi = require "wsapi"
wsapi.request = require "wsapi.request"

local m_moduleLocation = select(2, ...)

local function __Search (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local encodedInfo = web.input["post_data"]

	LogInfo("[OUTBOUNDENGINE_EXPORTS:__Search] Loading Exports in '%s' vcc. Posted Info: '%s'", vcc, encodedInfo)

	web.headers["Content-Type"] = "application/json"
	local success, data = allegroExports.LoadPaginatedExports(encodedInfo)

	if success and data then
		return data
	end

	-- Error
	local err = data
	web.status = 500
	LogError("[OUTBOUNDENGINE_EXPORTS:__Search] Error loading Exports. Err: %s", json.encode(err))

	return err
end

local function __StartExportation (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local encodedInfo = web.input["post_data"]

	LogInfo("[OUTBOUNDENGINE_EXPORTS:__StartExportation] Starting Exportation in '%s' vcc. Posted Info: '%s'", vcc, encodedInfo)

	web.headers["Content-Type"] = "application/json"
	local success, data = allegroExports.StartExportation(encodedInfo)

	if success then
		LogInfo("[OUTBOUNDENGINE_EXPORTS:__StartExportation] Success Started Exportation %s.", encodedInfo)
		return success
	end

	-- Error
	local err = data
	web.status = 500
	LogError("[OUTBOUNDENGINE_EXPORTS:__StartExportation] Error Starting Exportation %s. Err: %s", encodedInfo, json.encode(err))

	return err
end

local function __DeleteExport (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local encodedInfo = web.input["post_data"]

	LogInfo("[OUTBOUNDENGINE_EXPORTS:__DeleteExport] Deleting Exports in '%s' vcc. Posted Info: '%s'", vcc, encodedInfo)

	web.headers["Content-Type"] = "application/json"
	local success, data = allegroExports.DeleteExport(encodedInfo)

	if success then
		LogInfo("[OUTBOUNDENGINE_EXPORTS:__DeleteExport] Success deleting Exports %s.", encodedInfo)
		return success
	end

	-- Error
	local err = data
	web.status = 500
	LogError("[OUTBOUNDENGINE_EXPORTS:__DeleteExport] Error deleting Exports %s. Err: %s", encodedInfo, json.encode(err))

	return err
end

local function __CancelExport (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local encodedInfo = web.input["post_data"]

	LogInfo("[OUTBOUNDENGINE_EXPORTS:__CancelExport] Canceling Exports in '%s' vcc. Posted Info: '%s'", vcc, encodedInfo)

	web.headers["Content-Type"] = "application/json"
	local success, data = allegroExports.CancelExport(encodedInfo)

	if success then
		LogInfo("[OUTBOUNDENGINE_EXPORTS:__CancelExport] Success Cancelled Exports %s.", encodedInfo)
		return success
	end

	-- Error
	local err = data
	web.status = 500
	LogError("[OUTBOUNDENGINE_EXPORTS:__CancelExport] Error Canceling Exports %s. Err: %s", encodedInfo, json.encode(err))

	return err
end

local function __DownloadExport (wsapi_env)
	local vcc = inconcert.authUser.virtualContactCenter
	local request = wsapi.request.new(wsapi_env)
	local exportationId = request.params["exportationId"]
	local fileName = request.params["fileName"]

	LogInfo("[OUTBOUNDENGINE_EXPORTS:__DownloadExport] Downloading Exportation in '%s' vcc, ExportationId: '%s', fileName: '%s'.", vcc, exportationId, fileName)

	local suggestedFileName = fileName or string.format("%s_exportation.CSV", exportationId)
	return allegroExports.DownloadExport(exportationId, suggestedFileName)
end

local function __SearchLayout (web)
	_T = m_htmlRender.extension.makeTextTranslator()
	local vcc = inconcert.authUser.virtualContactCenter
	local encodedInfo = web.input["post_data"]

	LogInfo("[OUTBOUNDENGINE_EXPORTS:__SearchLayout] Loading Export Layouts in '%s' vcc. Posted Info: '%s'", vcc, encodedInfo)

	web.headers["Content-Type"] = "application/json"
	local success, data = allegroExports.LoadPaginatedExportLayouts(encodedInfo)

	if success and data then
		for _, row in pairs(data.rows or {}) do
			for _, subRow in pairs(row.ContactExportLayoutColumns or {}) do
				subRow.DisplayGroup = _T(subRow.DisplayGroup)
				subRow.DisplayType = _T(subRow.DisplayType)
				subRow.DisplayExtraData = _T(subRow.DisplayExtraData)
			end
		end
		return data
	end

	-- Error
	local err = data
	web.status = 500
	LogError("[OUTBOUNDENGINE_EXPORTS:__SearchLayout] Error loading Export Layouts. Err: %s", json.encode(err))

	return err
end

local function __DeleteExportLayout (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local encodedInfo = web.input["post_data"]

	LogInfo("[OUTBOUNDENGINE_EXPORTS:__DeleteExportLayout] Deleting Export Layout in '%s' vcc. Posted Info: '%s'", vcc, encodedInfo)

	web.headers["Content-Type"] = "application/json"
	local success, data = allegroExports.DeleteExportLayout(encodedInfo)

	if success then
		LogInfo("[OUTBOUNDENGINE_EXPORTS:__DeleteExportLayout] Success deleting Export Layout %s.", encodedInfo)
		return success
	end

	-- Error
	local err = data
	web.status = 500
	LogError("[OUTBOUNDENGINE_EXPORTS:__DeleteExportLayout] Error deleting Export Layout %s. Err: %s", encodedInfo, json.encode(err))

	return err
end

local function __SaveExportLayout (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local encodedInfo = web.input["post_data"]

	LogInfo("[OUTBOUNDENGINE_EXPORTS:__SaveExportLayout] Saving Export Layout in '%s' vcc. Posted Info: '%s'", vcc, encodedInfo)

	web.headers["Content-Type"] = "application/json"
	local success, data = allegroExports.SaveExportLayout(encodedInfo)

	if success then
		LogInfo("[OUTBOUNDENGINE_EXPORTS:__SaveExportLayout] Success Saving Export Layout %s.", encodedInfo)
		return success
	end

	-- Error
	local err = data
	web.status = 500
	LogError("[OUTBOUNDENGINE_EXPORTS:__SaveExportLayout] Error Saving Export Layout %s. Err: %s", encodedInfo, json.encode(err))

	return err
end


--[[

Return

]]--
local function __GetExportLayout (web, layoutId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[OUTBOUNDENGINE_EXPORTS:__GetExportLayout] Retrieving Export Layout %s, user %s, vcc %s", layoutId, userId, vcc)

	local success, layout = allegroExports.GetExportLayout(layoutId)
	if success and layout then
		return json.encode(layout)
	end

	web.status = 404
	return {result = "NOT_FOUND"}
end

--[[

Return

]]--
local function __LoadExportLayouts (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[OUTBOUNDENGINE_EXPORTS:__LoadExportLayouts] Loadign All Export Layouts, user %s, vcc %s", userId, vcc)

	local success, layouts = allegroExports.LoadExportLayouts()
	if success and layouts then
		return json.encode(layouts)
	end

	web.status = 404
	return {result = "NOT_FOUND"}
end

local function GetExportationFieldName (columnType)
	_T = m_htmlRender.extension.makeTextTranslator()
	local columnsFieldName = {
		--basic
		['0'] = _T("Multi Row"),
		['1'] = _T("Name"),
		['2'] = _T("Category"),
		['3'] = _T("Campaign"),
		['4'] = _T("Account Officer"),
		['5'] = _T("Account Group"),
		['6'] = _T("Is Client"),
		['7'] = _T("Is Vip"),
		['8'] = _T("Last Update"),
		['9'] = _T("Batches Included"),
		--basic attempts
		['10'] = _T("Total Attempts"),
		['10ABANDONED'] = _T("Abandoned"),
		['10ANSWERING_MACHINE'] = _T("Answering Machine"),
		['10BLACKLISTED_NUMBER'] = _T("Blacklisted Number"),
		['10BUSY'] = _T("Busy"),
		['10CALL_REJECTED'] = _T("Rejected"),
		['10CALL_REJECTED_BY_ENDPOINT'] = _T("Rejected by Endpoint"),
		['10CANT_ROUTE_TO_ENDPOINT'] = _T("Can't Route to Endpoint"),
		['10CAUSE_UNKNOWN'] = _T("Cause Unknown"),
		['10CONFIG_ERROR'] = _T("Config Error"),
		['10CONGESTION'] = _T("Congestion"),
		['10CONNECTED'] = _T("Connected"),
		['10DEAD_CONTACT'] = _T("Dead Contact"),
		['10FAX'] = _T("Fax"),
		['10HANDSHAKING_FAILED'] = _T("Handshaking Failed"),
		['10IN_PROGRESS'] = _T("In Progress"),
		['10INDIRECT_CONTACT'] = _T("Indirect Contact"),
		['10INVALID_USE_NETWORK'] = _T("Invalid Use Network"),
		['10LINK_DOWN'] = _T("Link Down"),
		['10NETWORK_ERROR'] = _T("Network Error"),
		['10NO_ANSWER'] = _T("No Answer"),
		['10NORMAL_CALL_END'] = _T("Normal Call End"),
		['10NUMBER_CHANGED'] = _T("Number Changed"),
		['10PROTOCOL_ERROR'] = _T("Protocol Error"),
		--phone
		['11'] = _T("Full Number"),
		['12'] = _T("Destination"),
		['13'] = _T("Country"),
		['14'] = _T("Area"),
		['15'] = _T("Number"),
		['16'] = _T("Extension"),
		['17'] = _T("Status"),
		--name value
		['18'] = _T("Name Value"),
		--batch
		['19'] = _T("Batch Id"),
		['20'] = _T("Campaign"),
		['21'] = _T("Outbound Process"),
		['22'] = _T("Channel Type"),
		['23'] = _T("Priority"),
		['24'] = _T("Reserved Agent"),
		['25'] = _T("Last Contact Attempt Date"),
		['26'] = _T("Last Destination"),
		['27'] = _T("Last Address"),
		['28'] = _T("Last Interaction"),
		['29'] = _T("Last Results"),
		['30'] = _T("Last Success Contact Date"),
		['31'] = _T("Next Contact Attempt Date"),
		['32'] = _T("Next Destination"),
		['33'] = _T("Next Address"),
		['34'] = _T("Batch Status"),
		['35'] = _T("Batch Cause"),
		--contact
		['36'] = _T("Interaction"),
		['37'] = _T("Destination"),
		['38'] = _T("Address"),
		['39'] = _T("Date"),
		['40'] = _T("Dialing Time"),
		['41'] = _T("Results"),
		['42'] = _T("Test Results"),
		['43'] = _T("Agent"),
		['44'] = _T("Transfer To Agent Time"),
		['45'] = _T("Conversation Time"),
		['46'] = _T("Wait Time"),
		['47'] = _T("Wrapup Time"),
		['48'] = _T("Desertion Time"),
		['49'] = _T("Hold Time"),
		['50'] = _T("Duration Time"),
		['51'] = _T("Answer Time"),
		['52'] = _T("Control Agent Time"),
		['53'] = _T("IVR Time"),
		['54'] = _T("Transfer Time"),
		['55'] = _T("Requeued Time"),
		['56'] = _T("Ringing Time"),
		['57'] = _T("Ring Back Time"),
		['58'] = _T("Status Change"),
		['59'] = _T("Next Attempt"),
		--basic
		['60'] = _T("Last Management Result"),
		-- skills
		['61'] = _T("Skill")
	}
	return columnsFieldName[tostring(columnType)]
end

local function __OpenFrm (web, frmName)
	_M.response.DontCache(web)

	local env = {
		_T = m_htmlRender.extension.makeTextTranslator(),
		GetExportationFieldName = GetExportationFieldName
	}
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[OUTBOUNDENGINE_EXPORTS:__OpenFrm] Open HTML form %s for user %s,", frmName, userId)

	local htmlFile = m_appLocation .. "/pages/" .. frmName
	local content = m_htmlRender:RenderPageLP(htmlFile, web, env)
	if not content then
		LogError("[OUTBOUNDENGINE_EXPORTS:__OpenFrm] HTML form does not exist: %s", htmlFile)
	end

	return content
end

local function __RenderExport (web)
	return __OpenFrm(web, "frm_exports_content.html")
end

local function __RenderLayout (web)
	return __OpenFrm(web, "frm_export_layout_content.html")
end

m_htmlRender = nil
m_appLocation = nil

function MapUris (module, htmlRender, prefix, appLocation)
	local LogRequest = require("apps.agent.requestFilters").LogRequest

	local replyFn = utils.JsonReply
	local replyValFn = utils.JsonValidatorReply

	local function HasPermission (fn, actionId, responseFn)
		responseFn = responseFn or replyFn
		return responseFn .. LogRequest .. permissions:CheckPermission("outbound_engine", actionId, fn)
	end

	local fn = utils.JsonValidatorReply .. LogRequest
	local p = prefix

	-- RENDER FRM
	module:dispatch_get(__RenderExport, p .. "/frm/render/"       ) -- export
	module:dispatch_get(__RenderLayout, p .. "/layout/frm/render/") -- export layout

	-- EXPORTS
	module:dispatch_post(fn .. __Search, p .."/search/")
	module:dispatch_post(HasPermission(__DeleteExport    , "delete_contacts_export")  , p .. "/delete/"  )
	-- EXPORTATION ACTIONS
	module:dispatch_post(HasPermission(__StartExportation, "save_contacts_export")  , p .. "/start/"   )
	module:dispatch_post(HasPermission(__CancelExport    , "cancel_contacts_export"), p .. "/cancel/"  )
	-- DOWNLOAD
	module:dispatch_wsapi(__DownloadExport, p .. "/download/exportation/(.*)")

	-- EXPORT LAYOUT
	module:dispatch_post(fn .. __SearchLayout, p .."/layout/search/")
	module:dispatch_post(HasPermission(__SaveExportLayout, "save_contacts_export_layout"), p .. "/layout/save/"    )
	module:dispatch_get (HasPermission(__GetExportLayout, "view_contacts_export_layout") , p .. "/layout/get/(.+)" )
	module:dispatch_post(HasPermission(__DeleteExportLayout, "delete_contacts_export")   , p .. "/layout/delete/"  )
	module:dispatch_get(replyFn .. LogRequest .. __LoadExportLayouts                     , p .. "/layout/load_all/")

	m_htmlRender = htmlRender
	m_appLocation = appLocation
end
