module(..., package.seeall, inconcert.orbit.application.Decorate)
local utils = require "shared.apps.utils"
local permissions = require "shared.permissions.permissions"
local allegroImports = require "apps.allegro.controllers.outboundengine_imports"
local wsapi = require "wsapi"
wsapi.request = require "wsapi.request"
local Fullcontact = require "fullcontact"
local m_moduleLocation = select(2, ...)

local function ExecuteFullcontactMethod (callback)
	local outbound_engine = require "apps.outbound_engine"
	outbound_engine.InitializeFullcontact(callback)
end

function ListImportFormat (vcc)
	local success, data
	ExecuteFullcontactMethod(function()
		success, data = Fullcontact.Dialer:ListImportFormat({vcc = vcc})
	end)
	return success, data
end

local function __Search (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local encodedInfo = web.input["post_data"]

	LogInfo("[OUTBOUNDENGINE_IMPORTS:__Search] Loading Imports in '%s' vcc%s. Posted Info: '%s'", vcc, utils.MakeLogAllegroVccMsg(vcc), encodedInfo)

	web.headers["Content-Type"] = "application/json"
	local success, data
	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)

	if hasAllegroIntegration then
		success, data =	allegroImports.LoadPaginatedImports(encodedInfo)
	else
		local info = web.request
		ExecuteFullcontactMethod(function()
			success, data = Fullcontact.Dialer:ListImportations({vcc = vcc})
		end)
		if success then
			local rows = data or {}
			LogInfo("[OUTBOUNDENGINE_IMPORTS:__Search] 1 - Filtering rows")
			--LogDebug("[OUTBOUNDENGINE_IMPORTS.STATES:__Search] ===> 1.1 - rows: %s", json.encode(rows))
	
			local totalUnfilteredRows = utils.GetRowCount(rows)
			local totalAdded = 0
			local filteredRows = {}

			for id, row in ipairs(rows) do
				row.Id = row.ID
				--row.Description = row.Description or row.Id
				if utils.MatchFilter(row, info) then
					totalAdded = totalAdded + 1
					table.insert(filteredRows, row)
				end
			end

			LogInfo("[OUTBOUNDENGINE_IMPORTS:__Search] 2 - Sorting rows %d", #filteredRows)
			--LogDebug("[OUTBOUNDENGINE_IMPORTS:__Search] ===> 2.1 - filteredRows: %s", json.encode(filteredRows or {}))
		
			--include only rows that belong to logged user vcc
			local response = {
				rows = filteredRows,
				last = "",
				total_rows = totalAdded,
				total_unfiltered_rows = totalUnfilteredRows
			}
			local responseParams = {
				MakeSortingValue = function(row)
					assert(row)
					return tostring(row.Description or row.Id):lower()
				end
			}
			data = utils.MakeSearchResponse(response, info, responseParams)
		end
	end

	if success and data then
		--LogDebug("[OUTBOUNDENGINE_IMPORTS:__Search] Loaded Importations. %s", json.encode(data))
		return data
	end

	-- Error
	local err = data
	web.status = 500
	LogError("[OUTBOUNDENGINE_IMPORTS:__Search] Error loading Imports. Err: %s", json.encode(err))

	return err
end

--[[

Return

]]--
local function __GetImport (web, importId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[OUTBOUNDENGINE_IMPORTS:__GetImport] Retrieving Import %s, user %s, vcc %s%s", importId, userId, vcc, utils.MakeLogAllegroVccMsg(vcc))

	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)
	local success, import
	if hasAllegroIntegration then
		success, import = allegroImports.GetImport(importId)
	else
		ExecuteFullcontactMethod(function()
			success, import = Fullcontact.Dialer:GetImportation(importId, vcc)
		end)
	end
	if success and import then
		local encodedImport = json.encode(import)
		LogInfo("[OUTBOUNDENGINE_IMPORTS:__GetImport] Retrieving Import %s vcc %s, data: %s.", importId, vcc, encodedImport)
		return encodedImport
	end

	web.status = 404
	return {result = "NOT_FOUND"}
end

local function __DeleteImport (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local encodedInfo = web.input["post_data"]

	LogInfo("[OUTBOUNDENGINE_IMPORTS:__DeleteImport] Deleting Import in '%s' vcc%s. Posted Info: '%s'", vcc, utils.MakeLogAllegroVccMsg(vcc), encodedInfo)

	web.headers["Content-Type"] = "application/json"

	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)
	local success, data
	if hasAllegroIntegration then
		success, data = allegroImports.DeleteImport(encodedInfo)
	else
		local importationId = json.decode(encodedInfo)[1]
		ExecuteFullcontactMethod(function()
			success, data = Fullcontact.Dialer:RemoveImport(importationId, vcc, inconcert.authUser.userName)
		end)
	end

	if success then
		LogInfo("[OUTBOUNDENGINE_IMPORTS:__DeleteImport] Success deleting Import %s.", encodedInfo)
		return true
	end

	-- Error
	local err = data
	web.status = 500
	LogError("[OUTBOUNDENGINE_IMPORTS:__DeleteImport] Error deleting Import %s. Err: %s", encodedInfo, json.encode(err))

	return err
end

local function __StartUpload (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local encodedInfo = web.input["post_data"]

	LogInfo("[OUTBOUNDENGINE_IMPORTS:__StartUpload] Starting Uploading Contact Imports File in '%s' vcc%s. Posted Info: '%s'", vcc, utils.MakeLogAllegroVccMsg(vcc), encodedInfo)

	web.headers["Content-Type"] = "application/json"
	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)

	if not hasAllegroIntegration then
		--LogDebug("[OUTBOUNDENGINE_IMPORTS:__StartUpload] Disabled API Method for vcc %s WITHOUT Allegro Integration.", vcc, utils.MakeLogAllegroVccMsg(vcc))
		return true
	end

	local success, data = allegroImports.StartUpload(encodedInfo)
	if success then
		LogInfo("[OUTBOUNDENGINE_IMPORTS:__StartUpload] Success Uploaded Contact Imports File %s.", encodedInfo)
		return true
	end

	-- Error
	local err = data
	web.status = 500
	LogError("[OUTBOUNDENGINE_IMPORTS:__StartUpload] Error Uploading Contact Imports %s. Err: %s", encodedInfo, json.encode(err))

	return err
end

--[[
	Funcion que invoca recarga los contactos por procesos.
]]--
local function ReloadProcessesContacts (contactsByProcesses)
	LogDebug("[OUTBOUNDENGINE_IMPORTS:ReloadProcessesContacts] Reload Process contacts. contactsByProcesses: %s.", utils.JSON_Encode(contactsByProcesses))
	if not contactsByProcesses then
		return
	end
	local m_outboundEngineClient = require "apps.outbound_engine.outboundengineclient"
	for processId, contacts in pairs(contactsByProcesses) do
		local info = {
			processId = processId,
			activeReload = true,
			scope = {
				contacts = contacts
			}
		}
		m_outboundEngineClient.processes:ReloadContacts(info)
	end
	return true
end

local function __StartImport (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local encodedInfo = web.input["post_data"]
	LogInfo("[OUTBOUNDENGINE_IMPORTS:__StartImport] Starting Imports in '%s' vcc%s. Posted Info: '%s'", vcc, utils.MakeLogAllegroVccMsg(vcc), encodedInfo)
	web.headers["Content-Type"] = "application/json"
	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)
	local success, data 
	if hasAllegroIntegration then
		success, data = allegroImports.StartImport(encodedInfo)
		if success then
			LogInfo("[OUTBOUNDENGINE_IMPORTS:__StartImport] Success Started Imports in Vcc '%s' with Allegro Integration. Data: %s.", vcc, encodedInfo)
			return true
		end
	else
		local info = json.decode(encodedInfo)
		info.VCC = vcc
		local fileInfo = {
			FileId = info.FileInfo.id
		}
		local allegroRepositoryModel = require("apps.allegro.controllers.repository").GetModel()
		local temporalCsvFile = allegroRepositoryModel.GetTemporalUploadedFile(fileInfo)
		if temporalCsvFile then
			info.UserId = inconcert.authUser.userName
			info.VCC = vcc
			info.FormatId = info.FormatId or info.Format
			info.Format = nil
			-- blacklists
			local blacklists = {}
			if info.Blacklists and info.Blacklists.enabled then
				blacklists = info.Blacklists.scopes or {}
			end
			info.Blacklists = blacklists
			--
			ExecuteFullcontactMethod(function()
				-- Comienza la importacion y luego de culminar la misma en callback va a recargar los contactos contra el OE Client.
				success, data = Fullcontact.Dialer:StartImport(info, temporalCsvFile, function(contactsByProcesses)
					ReloadProcessesContacts(contactsByProcesses)
				end)
			end)
			--LogDebug("[OUTBOUNDENGINE_IMPORTS:__StartImport] --> ENDED Imports success: %s, data: %s.", success, json.encode(data or {}))
			if success then
				LogInfo("[OUTBOUNDENGINE_IMPORTS:__StartImport] Success Started Imports in Vcc '%s'. Data: %s.", vcc, encodedInfo)
				allegroRepositoryModel.DeleteTemporalUploadedFile(fileInfo)
				return true
			else
				data.code = data.code or data.type
			end
		else
			LogWarning("[OUTBOUNDENGINE_IMPORTS:__StartImport] - Unable to load temporal CSV file. Temporal CSV file does not exist.file id '%s'.", fileInfo.FileId)
			data = {reason = "Temporal CSV File does not exist", fileId = fileInfo.FileId, code = -105}
		end
	end
	-- Error
	local err = data
	web.status = 500
	LogError("[OUTBOUNDENGINE_IMPORTS:__StartImport] Error Starting Imports %s. Err: %s", encodedInfo, json.encode(err))
	return err
end

local function __CancelImport (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local encodedInfo = web.input["post_data"]

	LogInfo("[OUTBOUNDENGINE_IMPORTS:__CancelImport] Canceling Imports in '%s' vcc%s. Posted Info: '%s'", vcc, utils.MakeLogAllegroVccMsg(vcc), encodedInfo)

	web.headers["Content-Type"] = "application/json"
	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)
	local success, data

	if hasAllegroIntegration then
		success, data = allegroImports.CancelImport(encodedInfo)
		if success then
			LogInfo("[OUTBOUNDENGINE_IMPORTS:__CancelImport] Success Cancelled Imports %s.", encodedInfo)
			return success
		end
	else
		data = {type = "7", reason = "THIS ACTION IS NOT AVAILABLE"}
	end

	-- Error
	local err = data
	web.status = 500
	LogError("[OUTBOUNDENGINE_IMPORTS:__CancelImport] Error Canceling Imports %s. Err: %s", encodedInfo, json.encode(err))

	return err
end

local function __DepurateImport (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local encodedInfo = web.input["post_data"]

	LogInfo("[OUTBOUNDENGINE_IMPORTS:__DepurateImport] Depurating Imports in '%s' vcc%s. Posted Info: '%s'", vcc, utils.MakeLogAllegroVccMsg(vcc), encodedInfo)

	web.headers["Content-Type"] = "application/json"

	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)
	local success, data 

	if hasAllegroIntegration then
		success, data = allegroImports.DepurateImport(encodedInfo)
		if success then
			LogInfo("[OUTBOUNDENGINE_IMPORTS:__DepurateImport] Success Depurated Imports %s.", encodedInfo)
			return success
		end
	else
		local importationId = json.decode(encodedInfo).importId
		ExecuteFullcontactMethod(function()
			success, data = Fullcontact.Dialer:DepurateImport(importationId, vcc, inconcert.authUser.userName)
		end)
	end

	if success then
		LogInfo("[OUTBOUNDENGINE_IMPORTS:__DepurateImport] Success Depurating Import %s, result: %s.", encodedInfo, data or {})
		--LogDebug("[OUTBOUNDENGINE_IMPORTS:__DepurateImport] Success Depurating Import %s, result: %s.", encodedInfo, json.encode(data or {}))
		return {
			totalDepuratedContacts = data.totalDepuratedContacts
		}
	end

	-- Error
	local err = data
	web.status = 500
	LogError("[OUTBOUNDENGINE_IMPORTS:__DepurateImport] Error Depurating Imports %s. Err: %s", encodedInfo, json.encode(err))

	return err
end

--[[

]]--
local function __DownloadFailedContacts (wsapi_env)
	local vcc = inconcert.authUser.virtualContactCenter
	local request = wsapi.request.new(wsapi_env)
	local importId = request.params["importid"]
	local updateId = request.params["update"]

	LogInfo("[OUTBOUNDENGINE_IMPORTS:__DownloadFailedContacts] Downloading Failed Contacts in '%s' vcc%s, importId: '%s', update number: '%s'.", vcc, utils.MakeLogAllegroVccMsg(vcc), importId, updateId)

	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)
	if hasAllegroIntegration then
		local suggestedFileName = string.format("%s(%s)_failed_contacts.CSV", importId, updateId)
		return allegroImports.DownloadFailedContacts(importId, updateId, suggestedFileName)
	end
	--
	local function ReturnErr (err)
		-- Error
		wsapi_env.status = 500
		LogError("[OUTBOUNDENGINE_IMPORTS:__DownloadFailedContacts] Error Downloading Failed Contacts. Err: %s", json.encode(err))
		local response = wsapi.response.new(200, {["Content-Type"]= "text/plain"})
		response:write(json.encode(err))
		response.headers["Response"] = "defaultError"
		response.status = err.httpCode or 500
		return response:finish()
	end
	--
	local success, info
	ExecuteFullcontactMethod(function()
		success, info = Fullcontact.Dialer:GetImportFailedContacts(importId, vcc, updateId)
	end)
	if not success then
		-- error
		local err = info
		return ReturnErr(err)
	end

	-- ok
	require "LuaOrm.Date"
	local updateDate = LuaOrm.Date(info.updateInfo.UpdateDate):toString("%Y-%m-%d %H:%M:%S")
	local suggestedFileName = string.format("%s(%s)_failed_contacts.CSV", importId, updateDate)
	-- nota: estos headers response estan mal! para señalar el status se usa, justamente, el status
	local headers = {}
	headers["Response"]            = "defaultError"
	headers["Content-Type"]        = "application/octet-stream"
	headers["Content-Length"]      = info.csvSize
	headers["Response"]            = "ok"
	headers["Content-Disposition"] = [[attachment; filename="]] .. suggestedFileName .. '"'
	headers["Set-Cookie"]          = "download_started=true; Path=/" -- used by the jquery download plugin

	LogInfo("[OUTBOUNDENGINE_IMPORTS:__DownloadFailedContacts] (END) Downloading CSV Failed contacts. CSV File Name '%s'.", suggestedFileName)

	local coroutineIter = 0
	return 200, headers, coroutine.wrap(function()
		coroutineIter = coroutineIter + 1
		--LogDebug("[OUTBOUNDENGINE_IMPORTS.SHARED_FILES:__DownloadFailedContacts] coroutine.yield - iter:%d", coroutineIter)
		coroutine.yield(info.csvFailedContacts)
	end)
end

local function __SearchFormats (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local encodedInfo = web.input["post_data"]

	LogInfo("[OUTBOUNDENGINE_IMPORTS:__SearchFormats] Loading Import Formats in '%s' vcc %s. Posted Info: '%s'", vcc, utils.MakeLogAllegroVccMsg(vcc), encodedInfo)

	web.headers["Content-Type"] = "application/json"
	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)

	if hasAllegroIntegration then
		success, data = allegroImports.LoadPaginatedImportFormats(encodedInfo)
	else
		local info = web.request
		ExecuteFullcontactMethod(function()
			success, data = Fullcontact.Dialer:ListImportFormat({vcc = vcc})
		end)
		if success then
			local rows = data
			LogInfo("[OUTBOUNDENGINE_IMPORTS:__SearchFormats] 1 - Filtering rows")
			--LogDebug("[OUTBOUNDENGINE_IMPORTS.STATES:__SearchFormats] ===> 1.1 - rows: %s", json.encode(rows or {}))
	
			local totalUnfilteredRows = utils.GetRowCount(rows)
			local totalAdded = 0
			local filteredRows = {}
			local importFieldsTypes = nil

			for id, row in pairs(rows) do
				row.Id = row.Id or utils.GetId(id)
				row.Description = row.Description or row.Id
				if utils.MatchFilter(row, info) then
					if not importFieldsTypes then
						local systemModule = require "apps.outbound_engine.controllers.system"
						local success, list = systemModule.LoadImportFieldsTypesByTelI6("INDEXED LIST")
						if success then
							importFieldsTypes = list
						end
					end
					--LogDebug("1 - importFieldsTypes %s", json.encode(importFieldsTypes))
					for _, field in ipairs(row.ContactImportFormatFields or {}) do
						--LogDebug("   - field.Type %s", field.Type)
						if field.Type and importFieldsTypes and importFieldsTypes[tostring(field.Type)] then
							local fieldType = importFieldsTypes[tostring(field.Type)]
							field.DisplayType = fieldType.DisplayName or fieldType.TypeName or field.Type
						end
					end
					totalAdded = totalAdded + 1
					table.insert(filteredRows, row)
				end
			end
		
			LogInfo("[OUTBOUNDENGINE_IMPORTS:__SearchFormats] 2 - Sorting rows %d", #filteredRows)
			--LogDebug("[OUTBOUNDENGINE_IMPORTS:__SearchFormats] ===> 2.1 - filteredRows: %s", json.encode(filteredRows or {}))
		
			--include only rows that belong to logged user vcc
			local response = {
				rows = filteredRows,
				last = "",
				total_rows = totalAdded,
				total_unfiltered_rows = totalUnfilteredRows
			}
			local responseParams = {
				MakeSortingValue = function(row)
					assert(row)
					return tostring(row.Description or row.Id):lower()
				end
			}
			data = utils.MakeSearchResponse(response, info, responseParams)
		end
	end

	if success and data then
		--LogDebug("[OUTBOUNDENGINE_IMPORTS:__SearchFormats] Loaded Import Formats. %s", json.encode(data))
		return data
	end

	-- Error
	local err = data
	web.status = 500
	LogError("[OUTBOUNDENGINE_IMPORTS:__SearchFormats] Error loading Import Formats. Err: %s", json.encode(err))

	return err
end

local function __DeleteImportFormat (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local encodedInfo = web.input["post_data"]

	LogInfo("[OUTBOUNDENGINE_IMPORTS:__DeleteImportFormat] Deleting Import Format in '%s' vcc%s. Posted Info: '%s'", vcc, utils.MakeLogAllegroVccMsg(vcc), encodedInfo)

	web.headers["Content-Type"] = "application/json"
	local success, data
	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)

	if hasAllegroIntegration then
		success, data = allegroImports.DeleteImportFormat(encodedInfo)
	else
		local importFormatId = json.decode(encodedInfo)[1] -- viene en un array por si queremos eliminar en batch
		ExecuteFullcontactMethod(function()
			success, data = Fullcontact.Dialer:RemoveImportFormat(importFormatId, vcc)
		end)
	end

	if success then
		LogInfo("[OUTBOUNDENGINE_IMPORTS:__DeleteImportFormat] Success deleting Import Format %s.", encodedInfo)
		return success
	end

	-- Error
	local err = data
	web.status = 500
	LogError("[OUTBOUNDENGINE_IMPORTS:__DeleteImportFormat] Error deleting Import Format %s. Err: %s", encodedInfo, json.encode(err))

	return err
end

--[[

Return

]]--
local function __LoadImportsFormats (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[OUTBOUNDENGINE_IMPORTS:__LoadImportsFormats] Loading Imports Formats, user %s, vcc %s%s", userId, vcc, utils.MakeLogAllegroVccMsg(vcc))

	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)
	if hasAllegroIntegration then
		local success, importsFormats = allegroImports.LoadImportsFormats()
		if success and importsFormats then
			return json.encode(importsFormats)
		end
	else
		local success, data
		ExecuteFullcontactMethod(function()
			success, data = Fullcontact.Dialer:ListImportFormat({vcc = vcc})
		end)
		if success then
			return json.encode(data)
		end
	end

	web.status = 404
	return {result = "NOT_FOUND"}
end

--[[

Return

]]--
local function __GetImportFormat (web, formatId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[OUTBOUNDENGINE_IMPORTS:__GetImportFormat] Retrieving Import Format %s, user %s, vcc %s%s", formatId, userId, vcc, utils.MakeLogAllegroVccMsg(vcc))

	local success, importFormat
	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)

	if hasAllegroIntegration then
		success, importFormat = allegroImports.GetImportFormat(formatId)
	else
		ExecuteFullcontactMethod(function()
			success, importFormat = Fullcontact.Dialer:GetImportFormat(formatId, vcc)
		end)
	end

	if success and importFormat then
		return json.encode(importFormat)
	end

	web.status = 404
	return {result = "NOT_FOUND"}
end

--[[

Return

]]--
local function __SaveImportFormat (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local encodedInfo = web.input["post_data"]

	LogInfo("[OUTBOUNDENGINE_IMPORTS:__SaveImportFormat] Saving Import Format in '%s' vcc%s. Posted Info: '%s'", vcc, utils.MakeLogAllegroVccMsg(vcc), encodedInfo)

	web.headers["Content-Type"] = "application/json"

	local function responseFn (isOk, result)
		if isOk then
			LogInfo("[OUTBOUNDENGINE_IMPORTS:__SaveImportFormat] Success Saved Import Format %s.", result.Id)
			return result
		end
		-- Error
		local err = result
		web.status = 400
		LogError("[OUTBOUNDENGINE_IMPORTS:__SaveImportFormat] Error Saving Import Format. Err: %s", json.encode(err))
		return err
	end

	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)
	local success, response

	if hasAllegroIntegration then
		success, response = allegroImports.SaveImportFormat(encodedInfo)
	else
		local importFormat = json.decode(encodedInfo)
		if importFormat.__IsNew then
			-- cuando se crea un formato, se valida que el formato con este id ya no exista.
			local currentImportFormat
			ExecuteFullcontactMethod(function()
				success, currentImportFormat = Fullcontact.Dialer:GetImportFormat(importFormat.Id, vcc)
			end)
			if success and currentImportFormat then
				success = false
				local err = {type = "ALREADY_EXIST", formatId = importFormat.Id}
				-- Error - already exist
				return responseFn(false, err)
			end
		end
		importFormat.VCC = vcc
		importFormat.__IsNew = nil
		ExecuteFullcontactMethod(function()
			success, response = Fullcontact.Dialer:SaveImportFormat(importFormat)
		end)
		if success then
			response = importFormat
		end
	end
	-- Ok or Error
	return responseFn(success, response)
end

--[[

Return

]]--
local function __HasSomeImportRunning (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[OUTBOUNDENGINE_IMPORTS:__HasSomeImportRunning] Validating if has some import running. User %s, vcc %s%s", userId, vcc, utils.MakeLogAllegroVccMsg(vcc))

	local hasAllegroIntegration = utils.HasAllegroIntegration(vcc)
	if hasAllegroIntegration then
		local success, isRunning = allegroImports.HasSomeImportRunning()
		if success and isRunning then
			return json.encode(isRunning)
		end
	else
		local response
		ExecuteFullcontactMethod(function()
			response = Fullcontact.Dialer.Realtime:IsAnyProcessBatchInProgressInVCC(vcc)
		end)
		return json.encode(response)
	end

	web.status = 404
	return {result = "NOT_FOUND"}
end

--[[

Return

]]--
local function __GetUploadedCSVHeader (web, fileId)
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[OUTBOUNDENGINE_IMPORTS:__GetUploadedCSVHeader] Retrieving CSV Header file '%s'. User %s, vcc %s", fileId, userId, vcc)

	local success, csvHeader = allegroImports.GetUploadedCSVHeader(fileId)
	if success and csvHeader then
		--LogDebug("---__GetUploadedCSVHeader------> csvHeader: %s", json.encode(csvHeader))
		return json.encode(csvHeader)
	end

	web.status = 404
	return {result = "NOT_FOUND"}
end

local function __RenderImport (web)
	_M.response.DontCache(web)

	local env = { _T = m_htmlRender.extension.makeTextTranslator() }
	local vcc = inconcert.authUser.virtualContactCenter
	local userId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	LogInfo("[OUTBOUNDENGINE_IMPORTS:__RenderImport] Render form Imports for user %s,", userId)

	local htmlFile = m_appLocation .. "/pages/frm_imports_content.html"
	local content = m_htmlRender:RenderPageLP(htmlFile, web, env)
	if not content then
		LogError("[OUTBOUNDENGINE_IMPORTS:__RenderImport] HTML Imports does not exist: %s", htmlFile)
	end

	return content
end

function ListImportations ()
	LogDebug("[OUTBOUNDENGINE_IMPORTS:ListImportations] 1 - Entra")
	local vcc = inconcert.authUser.virtualContactCenter
	local success, data
	ExecuteFullcontactMethod(function()
		success, data = Fullcontact.Dialer:ListImportations({vcc = vcc})
	end)
	LogDebug("[OUTBOUNDENGINE_IMPORTS:ListImportations] 2 - success: %s, data: %s", success, data)
	if not success then
		return {}
	end
	return data
end

m_htmlRender = nil
m_appLocation = nil

function MapUris (module, htmlRender, prefix, appLocation)
	local LogRequest = require("apps.agent.requestFilters").LogRequest
	local replyFn = utils.JsonReply
	local replyValFn = utils.JsonValidatorReply

	local function HasPermission (fn, actionId, responseFn)
		responseFn = responseFn or replyFn
		return responseFn .. LogRequest .. permissions:CheckPermission("outbound_engine", actionId, fn)
	end

	local jsonValReply = utils.JsonValidatorReply .. LogRequest
	local jsonReply = utils.JsonReply .. LogRequest
	local p = prefix

	-- RENDER FRM
	module:dispatch_get(__RenderImport                                                        , p .. "/frm/render/"                  )

	-- IMPORTS
	module:dispatch_post(jsonValReply .. __Search                                             , p .."/search/"                       )
	module:dispatch_get (HasPermission(__GetImport, "view_contacts_import")                   , p .. "/get/(.+)"                     )
	module:dispatch_post(HasPermission(__DeleteImport, "delete_contacts_import")              , p .. "/delete/"                      )
	module:dispatch_post(HasPermission(__StartImport , "save_contacts_import")                , p .. "/start/"                       )
	module:dispatch_post(HasPermission(__CancelImport, "cancel_contacts_import")              , p .. "/cancel/"                      )
	module:dispatch_post(HasPermission(__DepurateImport, "save_contacts_import")              , p .. "/depurate/"                    )

	-- UPLOAD
	module:dispatch_get (jsonReply .. __HasSomeImportRunning                                  , p .. "/has_some_importation_running/")
	module:dispatch_get (jsonReply .. __GetUploadedCSVHeader                                  , p .. "/get_uploaded_csv_header/(.+)" )
	module:dispatch_post(HasPermission(__StartUpload , "save_contacts_import")                , p .. "/upload/"                      )

	-- DOWNLOAD
	module:dispatch_wsapi(__DownloadFailedContacts                                            , p .. "/download/failed_contacts/(.*)")

	-- IMPORT FORMATS
	module:dispatch_post(jsonValReply .. __SearchFormats                                      , p .. "/formats/search/"              )
	module:dispatch_post(HasPermission(__DeleteImportFormat, "delete_contacts_import_formats"), p .. "/formats/delete/"              )
	module:dispatch_get (jsonReply .. __LoadImportsFormats                                    , p .. "/formats/load/"                )
	module:dispatch_get (jsonReply .. __GetImportFormat                                       , p .. "/formats/get/(.+)"             )
	module:dispatch_post(replyFn .. __SaveImportFormat                                        , p .. "/formats/save/"                )

	m_htmlRender = htmlRender
	m_appLocation = appLocation
end
