module(..., package.seeall)
local utils = require "shared.apps.utils"
local m_datasource

local function GetOutboundEngineRedisClient ()
	--LogDebug("[OUTBOUND_ENGINE.processes:GetOutboundEngineRedisClient] Entra")
	local outboundEngineModel = require "apps.outbound_engine.model"
	local oeRedis = outboundEngineModel:GetOutboundEngineRedisClient()
	--LogDebug("[OUTBOUND_ENGINE.processes:GetOutboundEngineRedisClient] Sale - oeRedis:  %s", oeRedis)
	return oeRedis
end

function SanitizeProcessId (vcc, processId)
	assert(vcc)
	assert(processId)
	return utils.AppendVcc(utils.GetId(processId), vcc):lower()
end

function GetName ()
	--LogDebug("[OUTBOUND_ENGINE.processes:GetName] Entra")
	return "OutboundProcesses"
end

function GetDefinition (datasource)
	--LogDebug("[OUTBOUND_ENGINE.processes:GetDefinition] Entra - datasource:  %s", datasource)
	m_datasource = datasource
	return _M
end

function LoadProcesses (vcc, ksOE)
	local processes = utils.GetAllColumns {
		keyspace = ksOE or "OutboundEngine",
		columnFamily = "OutboundProcesses",
		superColumns = true,
		key = vcc,
		columns = {}
	}
	return processes or {}
end

function LoadProcessesAndBatches (vcc, ksOE)
	local processes = LoadProcesses(vcc, ksOE)
	for processId, process in pairs(processes) do
		process.batches = LoadBatchesOfProcess(vcc, processId, ksOE)
	end
	--LogDebug("[LoadProcessesAndBatches]  1----> vcc: %s,  processes: %s", vcc, json.encode(processes or {}))
	return processes
end

function GetProcess (vcc, processId, ksOE)
	processId = SanitizeProcessId(vcc, processId)
	local processes = utils.GetAllColumns {
		keyspace = ksOE or "OutboundEngine",
		columnFamily = "OutboundProcesses",
		superColumns = true,
		key = vcc,
		columns = processId and {processId} or {}
	}
	if processId then
		local process = processes and processes[processId] or {}
		--LogDebug("[GetProcess]  1----> vcc: %s processId: %s, process: %s", vcc, processId, json.encode(process or {}))
		return process
	end
	return processes or {}
end

function LoadBatchesOfProcessDefinition (processId, ksOE)
	local processes = utils.GetAllColumns {
		keyspace = ksOE or "OutboundEngine",
		columnFamily = "OutboundProcessBatches",
		superColumns = true,
		key = processId,
		columns = {}
	}
	return processes or {}
end

function LoadBatchesOfProcess (vcc, processId, ksOE)
	--LogDebug("[OUTBOUND_ENGINE.processes:LoadBatchesOfProcess]  1----> vcc: %s processId: %s.", vcc, processId)
	processId = SanitizeProcessId(vcc, processId)
	local batches = utils.GetOEModel("OutboundProcessBatches").LoadBatches(vcc, processId, ksOE)
	batches = utils.HashToArray(batches, "id")
	--LogDebug("[OUTBOUND_ENGINE.processes:LoadBatchesOfProcess]  2----> batches: %s.", json.encode(batches))
	-- obtengo los statuses de los contactos de cada lote
	local m_outboundEngineClient = require "apps.outbound_engine.outboundengineclient"
	for _, batch in ipairs(batches) do
		if batch.configuration then
			local success, contactsStatuses = m_outboundEngineClient.batches:GetContactsStatuses(processId, batch.configuration.name)
			--LogDebug("[OUTBOUND_ENGINE.processes:LoadBatchesOfProcess]      3----> contactsStatuses: %s.", json.encode(contactsStatuses))
			if success then
				batch.contactsStatuses = contactsStatuses
			end
		end
	end
	--LogDebug("[OUTBOUND_ENGINE.processes:LoadBatchesOfProcess]  4----> batches: %s.", json.encode(batches))
	return batches
end

function GetRunningBatchActions (processId, vcc)
	--LogDebug("[OUTBOUND_ENGINE.processes:GetRunningBatchActions]  1----> vcc: %s processId: %s.", vcc, processId)
	local m_outboundEngineClient = require "apps.outbound_engine.outboundengineclient"
	local isAdmin = true
	local success, runningBatchActions = m_outboundEngineClient.processes:GetRunningBatchActions(processId, vcc, isAdmin)
	--LogDebug("[OUTBOUND_ENGINE.processes:GetRunningBatchActions]      2----> success: '%s', runningBatchActions: %s.", success, json.encode(runningBatchActions))
	if success then
		return runningBatchActions
	end
	return {}
end

function GetStatusCause (processId, vcc)
	LogDebug("[OUTBOUND_ENGINE.processes:GetStatusCause]  1----> vcc: %s processId: %s.", vcc, processId)
	local m_outboundEngineClient = require "apps.outbound_engine.outboundengineclient"
	return m_outboundEngineClient.processes:GetStatusCause(processId, vcc)
end

function GetProcessAndBatches (vcc, processId, ksOE)
	processId = SanitizeProcessId(vcc, processId)
	local process = GetProcess(vcc, processId, ksOE)
	if processId then
		process = process or {}
		process.batches = LoadBatchesOfProcess(vcc, processId, ksOE)
		process.runningBatchActions = GetRunningBatchActions(processId, vcc)
		--LogDebug("[OUTBOUND_ENGINE.processes:GetProcessAndBatches]  2----> vcc: %s processId: %s, process: %s", vcc, processId, json.encode(process or {}))
		return process
	end
	return process or {}
end

local function RemoveProcessSession (processId)
	assert(processId)
	--LogDebug("[OUTBOUND_ENGINE.processes:RemoveProcessSession] Entra - processId: %s", processId)
	local orRedis = GetOutboundEngineRedisClient()
	orRedis:hdel("OutboundProcessSession", processId)
end

local function GetAllProcessSession ()
	local orRedis = GetOutboundEngineRedisClient()
	local allSessions = orRedis:hgetall("OutboundProcessSession") or {}
	--LogDebug("[OUTBOUND_ENGINE.processes:GetAllProcessSession] -----------> oe.models.GetAllProcessSession - allSessions: %s", json.encode(allSessions))
	return allSessions
end

local function CleanUpAllProcessesSessions ()
	local orRedis = GetOutboundEngineRedisClient()
	local allSessions = GetAllProcessSession()
	for id, info in pairs(allSessions) do
		RemoveProcessSession(id)
	end
	GetAllProcessSession()
end

local function InitializeProcessSession (processId)
	assert(processId)
	--LogDebug("[OUTBOUND_ENGINE.processes:InitializeProcessSession] Entra - processId: %s", processId)
	--GetAllProcessSession()
	--CleanUpAllProcessesSessions()
	--RemoveProcessSession(processId)
	local orRedis = GetOutboundEngineRedisClient()
	if orRedis:hexists("OutboundProcessSession", processId) then
		--LogDebug("[OUTBOUND_ENGINE.processes:InitializeProcessSession] 1 - processId: %s", processId)
		return
	end
	orRedis:hset("OutboundProcessSession", processId, utils.JSON_Encode({}))
	--LogDebug("[OUTBOUND_ENGINE.processes:InitializeProcessSession] 2 - processId: %s", processId)
end

function DeleteProcess (vcc, processId, ksOE)
	processId = SanitizeProcessId(vcc, processId)
	utils.RemoveColumns {
		keyspace = ksOE or "OutboundEngine",
		columnFamily = "OutboundProcesses",
		superColumns = true,
		rows = {
			[vcc] = {processId}
		}
	}
	RemoveProcessSession(processId)
	return true, {}
end

function SaveProcess (vcc, process, ksOE)
	assert(vcc)
	assert(process)
	assert(process.id)
	process.id = SanitizeProcessId(vcc, process.id)
	--LogDebug("[OUTBOUND_ENGINE.processes:SaveProcess] -----------> oe.models.SaveProcess - process: %s", json.encode(process))
	utils.SaveRows {
		keyspace = ksOE or "OutboundEngine",
		columnFamily = "OutboundProcesses",
		superColumns = true,
		rows = {
			[vcc] = {
				[process.id] = process
			}
		}
	}
	InitializeProcessSession(process.id)
	return true, {}
end

function GetProcessStatus (vcc, processId, ksOE)
	processId = SanitizeProcessId(vcc, processId)
	local process = GetProcess(vcc, processId, ksOE)
	--LogDebug("[OUTBOUND_ENGINE.processes:GetProcessStatus] ----> vcc: %s processId: %s, process: %s", vcc, processId, utils.JSON_Encode(process))
	if utils.TableIsEmpty(process) then
		return nil
	end
	return process.state
end
