local m_helpers = require "helpers"
local utils = require "shared.apps.utils"
local m_outboundEngineClient = nil
module(..., package.seeall)

local m_fieldsKeys = {
	TABLE_NAME  = "TABLE_",
	FIELD_NAME  = "_FIELDNAME_",
	FIELD_VALUE = "_FIELDVALUE_"
}

local function GetOEClientController ()
	if not m_outboundEngineClient then
		m_outboundEngineClient = require "apps.outbound_engine.outboundengineclient"
	end
	return m_outboundEngineClient
end

local function MakeNestedFieldName (tableName, fieldName, fieldValue)
	local name
	local keyTableName  = m_fieldsKeys.TABLE_NAME
	local keyFieldName  = m_fieldsKeys.FIELD_NAME
	local keyFieldValue = m_fieldsKeys.FIELD_VALUE
	if fieldValue then
		-- Sample: TABLE_Skill_FIELDNAME_SkillId_FIELDVALUE_Habilidad1
		name = string.format("%s%s%s%s%s%s", keyTableName, tableName, keyFieldName, fieldName, keyFieldValue, fieldValue)
	else
		-- Sample: TABLE_Addresses_FIELDNAME_Type_FAX
		name = string.format("%s%s%s%s", keyTableName, tableName, keyFieldName, fieldName)
	end
	--LogDebug("[MakeNestedFieldName] ---> name: %s", name)
	return name
end

local function ParseFieldName (fullFieldName)
	local parsed = {tableName = nil, fieldName = nil, fieldValue = nil, fullFieldName = nil}
	local startPositionTableName, endPositionTableName = string.find(fullFieldName, m_fieldsKeys.TABLE_NAME)
	--LogDebug("%s", tostring(startPositionTableName))
	--LogDebug("%s", tostring(endPositionTableName))

	if not startPositionTableName then
		parsed.fieldName = fullFieldName
		return parsed
	end

	local startPositionFieldName, endPositionFieldName = string.find(fullFieldName, m_fieldsKeys.FIELD_NAME)
	--LogDebug("%s", tostring(startPositionFieldName))
	--LogDebug("%s", tostring(endPositionFieldName))

	if not startPositionFieldName then
		parsed.fieldName = fullFieldName
		return parsed
	end

	local startPositionFieldValue, endPositionFieldValue = string.find(fullFieldName, m_fieldsKeys.FIELD_VALUE)
	--LogDebug("%s", tostring(startPositionFieldValue))
	--LogDebug("%s", tostring(endPositionFieldValue))
	parsed.tableName = string.sub(fullFieldName, endPositionTableName + 1, startPositionFieldName - 1)

	if not startPositionFieldValue then
		parsed.fieldName = string.sub(fullFieldName, endPositionFieldName + 1, startPositionFieldValue)
	else
		parsed.fieldName = string.sub(fullFieldName, endPositionFieldName + 1, startPositionFieldValue -1)
		parsed.fieldValue = string.sub(fullFieldName, endPositionFieldValue + 1)
	end

	parsed.fullFieldName = parsed.tableName .. "." .. parsed.fieldName
	return parsed
end

-----------------------------------------------------------------------------
-- Queries
------------------------------------------------------------------------------
local m_queries = {}
local m_data = {}
-------------

m_queries.MakeOperatorConfig = function (self, line)
	if not line then
		--LogDebug("1 - MakeOperatorConfig" )
		return
	end
	local mapConditionOperators = {
		["="] = {
			operator = "equal" --"term"
		},
		["<>"] = {
			operator = "distinct" -- "term"
		},
		[">"] = {
			operator = "greater" -- "range" "gt"
		},
		[">="] = {
			operator = "greater_equal" --"range" "gte"
		},
		["<"] = {
			operator = "less" -- "range"  "lt"
		},
		["<="] = {
			operator = "less_equal" --"range" "lte"
		},
		["LIKE"] = {
			operator = "contains" -- "match", wildcard
		}
	}
	local operator = line.operator:upper()
	local info = mapConditionOperators[operator]
	assert(info)
	local operatorConfig = {
		operator = info.operator
	}
	--LogDebug("2 - MakeOperatorConfig operatorConfig " .. json.encode(operatorConfig))
	return operatorConfig
end

m_queries.MakeQueryESConditions = function (self, rawData, vcc)
	--LogDebug("[OUTBOUNDENGINE.CLIENT:m_queries.MakeQueryESConditions] --> START Making ES Conditions. vcc: '%s' rawData: %s", vcc, m_helpers.table:JSON_Encode(rawData))
	assert(type(rawData) == "table")
	assert(type(vcc) == "string")
	local sqlParsed = rawData.sqlParsed
	assert(type(sqlParsed) == "table")
	assert(type(sqlParsed.value) == "table")

	rawData.processId = rawData.processId
		and GetOEClientController().processes:MakeProcessId(rawData.processId, vcc)
		or nil

	local systemFieldsDefinitions = m_data:LoadBatchContactFields(nil, vcc)
	local function FindFieldDefinition (tableName, fieldId)
		tableName = tableName or ""
		fieldId = fieldId or ""
		--LogDebug("-FindFieldDefinition---------ENTRA--> tableName: '%s', fieldId: '%s'", tableName, fieldId)
		local fieldDef = {}
		for _, def in ipairs(systemFieldsDefinitions) do
			--LogDebug("       --> def.ParsedFieldName: %s", json.encode(def.ParsedFieldName or {}))
			local parsedFieldTable = def.ParsedFieldName.tableName or ""
			local parsedFieldName = def.ParsedFieldName.fieldName or ""
			--LogDebug("    ----------> IS EQUAL: '%s'. parsedFieldTable: '%s' == '%s' AND -> parsedFieldName '%s' == '%s'", (parsedFieldTable == tableName and parsedFieldName == fieldId), parsedFieldTable, tableName, parsedFieldName, fieldId)
			if parsedFieldTable == tableName then
				--LogDebug("       -table is equal")
				if parsedFieldName == fieldId then
					--LogDebug("           -field is equal")
					fieldDef = def
					break
				end
			end
		end
		--LogDebug("--FindFieldDefinition---....... SALE ----> fieldDef: %s", json.encode(fieldDef))
		return fieldDef
	end

	local function SanitizeValue (originalValue, fieldDef, fieldType, operatorConfig)
		--LogDebug("[SanitizeValue] 1 ---> ENTRA: value: '%s', fieldType: '%s'", originalValue, fieldType)
		local value = originalValue or ""
		local convertLowerOrUpper = (fieldDef.LowerOrUpper or "lower"):lower()
		if convertLowerOrUpper == "lower" then
			value = value:lower()
		elseif convertLowerOrUpper == "upper" then
			value = value:upper()
		end
		if type(fieldDef.Sanitize) == "function" then
			value = fieldDef.Sanitize(value, operatorConfig.operator, rawData)
		end
		if fieldType == "NUMBER" then
			value = tonumber(value)
		elseif fieldType == "STRING" or  fieldType == "TEXT" or  fieldType == "KEYWORD" or fieldType == "DATE" then
			value = value:gsub("'", "")
		end
		if operatorConfig.operator == "contains" then
			value = ("*%s*"):format(value)
		end
		--LogDebug("[SanitizeValue] 2 ---> SALE: value: '%s'", value)
		return value
	end

	local function AddCondition (valueInfo, elascticSearchConditions, fieldDef)
		assert(valueInfo)
		assert(elascticSearchConditions)
		assert(fieldDef)
		--LogDebug("1 - AddCondition -valueInfo:  %s", json.encode(valueInfo))
		local fieldType = (valueInfo.type or ""):upper()
		local operatorConfig = self:MakeOperatorConfig(valueInfo)
		local fieldValue = SanitizeValue(valueInfo.value, fieldDef, fieldType, operatorConfig)
		local condition = {
			type  = operatorConfig.operator,
			name  = valueInfo.fieldName, --"State",
			value = fieldValue -- "ACTIVE"
		}
		--LogDebug("2 - AddCondition -condition:  %s", json.encode(condition))
		table.insert(elascticSearchConditions, condition)
	end

	local function RecursiveFill (sqlWhereCondition, elascticSearchConditions)
		if not sqlWhereCondition or not next(sqlWhereCondition) then
			return
		end
		local conditionType = (sqlWhereCondition.type):upper()
		if conditionType == "COMPARISONBOOLEANPRIMARY" or conditionType == "LIKEPREDICATE" then
			assert(type(sqlWhereCondition.left) == "table")
			assert(type(sqlWhereCondition.right) == "table")
			--LogDebug("----------------------------------")
			--LogDebug("--> sqlWhereCondition: %s", json.encode(sqlWhereCondition))
			local parsed = ParseFieldName(sqlWhereCondition.left.value)
			local operator = sqlWhereCondition.operator
			if conditionType == "LIKEPREDICATE" then
				operator = operator or "LIKE"
			end
			operator = operator or "="
			--LogDebug("--> parsed: %s", json.encode(parsed))

			local fieldDef = FindFieldDefinition(parsed.tableName, parsed.fieldName)
			--LogDebug("----------------------------------")
			if not parsed.fullFieldName then
				--LogDebug("A - simple")
				local valueInfo = {
					fieldName = parsed.fieldName,
					value     = sqlWhereCondition.right.value,
					type      = sqlWhereCondition.right.type,
					operator  = operator
				}
				AddCondition(valueInfo, elascticSearchConditions, fieldDef)
			else
				if parsed.fieldValue then
					if fieldDef.EntityType == "NAME_VALUE_SEARCH" then
						--LogDebug("B1 - Sample: NAME_VALUE_SEARCH.")
						-- name value
						valueInfo = {
							fieldName = parsed.tableName .. "." .. parsed.fieldValue:lower(),
							value     = sqlWhereCondition.right.value,
							type      = fieldDef.Type,
							operator  = operator
						}
						AddCondition(valueInfo, elascticSearchConditions, fieldDef)
					else
						--LogDebug("B2 - Sample: Skills.")
						-- name
						local valueInfo = {
							fieldName = parsed.fullFieldName,
							value     = parsed.fieldValue,
							type      = "string",
							operator  = "="
						}
						AddCondition(valueInfo, elascticSearchConditions, fieldDef)
						-- value
						valueInfo = {
							fieldName = parsed.tableName .. "." .. "Value",
							value     = sqlWhereCondition.right.value,
							type      = sqlWhereCondition.right.type,
							operator  = operator
						}
						AddCondition(valueInfo, elascticSearchConditions, fieldDef)
					end
				else
					--LogDebug("C - Sample: Addresses, etc")
					local valueInfo = {
						fieldName = parsed.fullFieldName,
						value     = sqlWhereCondition.right.value,
						type      = sqlWhereCondition.right.type,
						operator  = operator
					}
					AddCondition(valueInfo, elascticSearchConditions, fieldDef)
				end
			end

		elseif conditionType == "ANDEXPRESSION" or conditionType == "OREXPRESSION" then
			assert(type(sqlWhereCondition.left) == "table")
			assert(type(sqlWhereCondition.right) == "table")
			local condition = {
				type       = conditionType == "ANDEXPRESSION" and "and" or "or",
				conditions = {}
			}
			RecursiveFill(sqlWhereCondition.left, condition.conditions)
			RecursiveFill(sqlWhereCondition.right, condition.conditions)
			table.insert(elascticSearchConditions, condition)

		elseif conditionType == "SIMPLEEXPRPARENTHESES" then
			assert(type(sqlWhereCondition.value) == "table")
			RecursiveFill(sqlWhereCondition.value, elascticSearchConditions)

		elseif conditionType == "EXPRESSIONLIST" then
			assert(type(sqlWhereCondition.value) == "table")
			local expressionList = sqlWhereCondition.value
			for _, expression in ipairs(expressionList) do
				RecursiveFill(expression, elascticSearchConditions)
			end
		else
			assert(false, "Unrecognized SQL Parser Condition Type: " .. conditionType)
		end
	end

	local conditions = {
		type = nil,
		conditions = {}
	}
	RecursiveFill(sqlParsed.value.where, conditions.conditions)
	conditions.type = conditions.type or "and"

	--LogDebug("[OUTBOUNDENGINE.CLIENT:m_queries.MakeQueryESConditions] --> END Maked ES Conditions. conditions: %s", m_helpers.table:JSON_Encode(conditions))
	return true, conditions
end

m_queries.MakeQueryConditions = function (self, rawConditions, vcc)
	return self:MakeQueryESConditions(rawConditions, vcc)
end

--[[
	source values:
			contact_query: Contacts Database
			batch_query" : Current Process Batches
]]--
local function MakeBatchScope (vcc, rawConditions)
	assert(type(vcc) == "string")
	assert(type(rawConditions) == "table")
	--LogDebug("[OUTBOUNDENGINE.CLIENT:m_queries.MakeBatchScope] 1 ENTRA--> vcc: %s, rawConditions --> : %s", vcc, m_helpers.table:JSON_Encode(rawConditions))
	local processId
	local source = rawConditions.source:upper()
	if source == "BATCH_QUERY" then
		processId = rawConditions.processId
		assert(type(processId) == "string", "processId is required for the data source 'database contacts'")
	end
	local scope = {
		vcc              = vcc,
		processId        = processId,
		excludeContacted = rawConditions.excludeContacted,
		contactedDate    = rawConditions.contactedDate
	}
	--LogDebug("[OUTBOUNDENGINE.CLIENT:m_queries.MakeBatchScope] 2 SALE --> vcc: %s, scope --> : %s", vcc, m_helpers.table:JSON_Encode(scope))
	return scope
end

--[[
	source values:
			contact_query: Contacts Database
			batch_query" : Current Process Batches
]]--
m_queries.MakeQuery = function(self, vcc, rawConditions)
	assert(type(vcc) == "string")
	assert(type(rawConditions) == "table")
	assert(type(rawConditions.source) == "string")
	--LogDebug("[OUTBOUNDENGINE.CLIENT:m_queries.MakeQuery] 1 ENTRA--> rawConditions --> : %s", m_helpers.table:JSON_Encode(rawConditions))
	local scope = MakeBatchScope(vcc, rawConditions)
	if (rawConditions.scopeType or ""):upper() == "CONTACTS" then
		scope.contacts = rawConditions.contacts
	else
		local success, conditions = self:MakeQueryConditions(rawConditions, vcc)
		if not success then
			local err = conditions
			return false, err
		end
		scope.conditions = conditions
	end
	--LogDebug("[OUTBOUNDENGINE.CLIENT:m_queries.MakeQuery] 2 SALE --> Query --> : %s", m_helpers.table:JSON_Encode(scope))
	return true, scope
end

------------------------------

m_data.LoadVccCampaigns = function (vcc, keySpaceAgent)
	--LogDebug("--->1 m_data.LoadVccCampaign:")
	local function GetVCCCampaigns (vcc)
		return utils.GetAllColumns {
			keyspace = keySpaceAgent or "Agent",
			columnFamily = "VCCCampaigns",
			key = vcc
		} or {}
	end

	local function GetSpecificColumn (campaignId, specificColumn)
		local campaignAtVcc = utils.AppendVcc(utils.GetId(campaignId), vcc)
		return utils.GetColumnValue({
			keyspace = keySpaceAgent or "Agent",
			columnFamily = "Campaigns",
			key = campaignAtVcc,
			column = specificColumn
		})
	end

	local vccCampaigns = GetVCCCampaigns(vcc)
	local list = {}
	for campaignId in pairs(vccCampaigns) do
		local campaignName = GetSpecificColumn(campaignId, "Description")
		local item = {
			Value = campaignId,
			Name  = campaignName
		}
		table.insert(list, item)
	end
	--LogDebug("---> 2 m_data.LoadVccCampaign - list: %s", json.encode(list))
	return list
end

m_data.LoadVccSkills = function (vcc, keySpaceAgent)
	--LogDebug("--->1 m_data.LoadVccSkills:")
	local skills = utils.GetAllColumns {
		keyspace = keySpaceAgent or "Agent",
		columnFamily = "VCCSkills",
		superColumns = true,
		key = vcc,
		columns = {}
	}
	--LogDebug("---> 2 m_data.LoadVccSkills - skills: %s", json.encode(skills))
	return skills
end

m_data.ListImportFormat = function (vcc)
	--LogDebug("--->1 m_data.ListImportFormat")
	local importsModule = require "apps.outbound_engine.controllers.imports"
	local success, data = importsModule.ListImportFormat(vcc)
	--LogDebug("---> 2 m_data.ListImportFormat -> success: %s, data: %s", success, json.encode(data or {}))
	if success then
		return data
	end
	return {}
end

--[[
	Ejemplo de como se encuentra en Allegro:

[{"Table":"","Name":"Account Officer","Field":"AccountOfficer"},{"Table":"ContactImportAddress","Name":"Address Apartment","Field":"Apt"},{"Table":"ContactImportAddress","Name":"Address Area","Field":"Area"},{"Table":"ContactImportAddress","Name":"Address Country","Field":"Country"},{"Table":"ContactImportAddress","Name":"Address Number","Field":"Number"},{"Table":"ContactImportAddress","Name":"Address Street","Field":"Street"},{"Table":"ContactImportAddress","Name":"Address Type","Values":[{"Value":"HOME","Name":"Home"},{"Value":"WORK","Name":"Work"}],"Field":"Type"},{"Table":"OutboundProcessContact","Name":"Batch Id","Values":[],"Field":"BatchId"},{"Table":"","Name":"Campaign","Values":[{"Value":"camptelborrame","Name":"a"},{"Value":"campwebchat","Name":"campaña de webchat"},{"Value":"camptel2","Name":"NO BORRAR POR FAVOR!!!  ES LA QUE FUNCIONA1"},{"Value":"campmail","Name":"campaña de mail"},{"Value":"qasdf","Name":"asdf"},{"Value":"sms","Name":"sms"}],"Field":"Campaign"},{"Table":"","Name":"Category","Field":"Category"},{"Table":"","Name":"Contact Id","Field":"Id"},{"Table":"OutboundProcessContact","Name":"Contact Result","Values":[{"Name":"Abandoned","Value":"-4"},{"Name":"Answering Machine","Value":"14"},{"Name":"Busy","Value":"3"},{"Name":"Call Rejected","Value":"4"},{"Name":"Call Rejected by Endpoint","Value":"10"},{"Name":"Can't Route to Endpoint","Value":"1"},{"Name":"Configuration Error","Value":"-3"},{"Name":"Congestion","Value":"8"},{"Name":"Connected","Value":"0"},{"Name":"Fax","Value":"15"},{"Name":"Handshaking Failed","Value":"9"},{"Name":"In Progress","Value":"7"},{"Name":"Indirect Contact","Value":"17"},{"Name":"Invalid Use of Network","Value":"6"},{"Name":"Link Down","Value":"-10"},{"Name":"Network Error","Value":"11"},{"Name":"No Answer","Value":"-1"},{"Name":"Normal Call Clearing","Value":"2"},{"Name":"Number in Black List","Value":"-2"},{"Name":"PREVIEW_CANCELED","Value":"-20"},{"Name":"Protocol Error","Value":"12"},{"Name":"The Contact Passed Away","Value":"16"},{"Name":"The Number has Changed","Value":"5"},{"Name":"Unknown Cause","Value":"13"}],"Field":"PreviousContactResult"},{"Table":"OutboundProcessContactRetries","Name":"Daily Retries","Field":"DayRetries"},{"Table":"ContactImportEMail","Name":"Email Address","Field":"Address"},{"Table":"ContactImportEMail","Name":"Email Type","Values":[{"Value":"PERSONAL","Name":"Personal"},{"Value":"WORK","Name":"Work"}],"Field":"Type"},{"Table":"","Name":"Importation","Values":["a","a1","a2","a3","asdf","el_id_cany","emma1","emma10","emma11","emma2","emma3","emma4","emma5","emma6","emma7","emma8","emma9","fasd","iiii1","iiii2","IMPCANY1","import100Contactos","prueba1","TestPuntoYComa"],"Field":"LastUpdate"},{"Table":"OutboundProcessContact","Name":"Last Batch Contact Date","Field":"PreviousContactDate"},{"Table":"OutboundProcessLastContact","Name":"Last Contact Date","Field":"LastContact"},{"Table":"","Name":"Last Management Result","Field":"LastManagementResult"},{"Table":"OutboundProcessContact","Name":"Last Result","Values":[{"Value":"ALL ADDRESSES INVALID","Name":"All addresses are invalid"},{"Value":"AWAITING RULE/CAMPAIGN","Name":"Awaiting rule/campaign"},{"Value":"BATCH COMPLETE","Name":"BATCH COMPLETE"},{"Value":"BATCH ERROR","Name":"BATCH ERROR"},{"Value":"BY RULE","Name":"By rule"},{"Value":"CAMPAIGN/BATCH END","Name":"Campaign/batch end"},{"Value":"MANUAL","Name":"Manual"},{"Value":"MAX DAY RETRIES","Name":"Max days retries"},{"Value":"MAX RETRIES","Name":"Max retries"},{"Value":"NO ADDRESSES","Name":"No addresses"},{"Value":"NO RULE ALLOWS","Name":"No rule allows continuing"}],"Field":"Cause"},{"Table":"","Name":"Name","Field":"Name"},{"Table":"ContactImportPhone","Name":"Phone Area","Field":"Area"},{"Table":"ContactImportPhone","Name":"Phone Country","Field":"Country"},{"Table":"ContactImportPhone","Name":"Phone Extension","Field":"Extension"},{"Table":"ContactImportPhone","Name":"Phone Number","Field":"Number"},{"Table":"ContactImportPhone","Name":"Phone Type","Values":[{"Value":"FAX","Name":"Fax"},{"Value":"HOME","Name":"Home"},{"Value":"CELLULAR","Name":"Mobile"},{"Value":"OFFICE","Name":"Office"}],"Field":"Type"},{"Table":"OutboundProcessContact","Name":"Priority on Batch","Field":"ContactPriority"},{"Table":"OutboundProcessContact","Name":"Reserved Agent","Field":"ReservedActor"},{"Table":"OutboundProcessContact","Name":"Scheduled Date","Field":"ScheduledDate"},{"Table":"ContactImportSkill","Name":"SKILL:skill1","Field":"skill1"},{"Table":"OutboundProcessContact","Name":"Status on Batch","Values":[{"Value":"ACTIVE","Name":"Active"},{"Value":"CANCELED","Name":"Cancelled"},{"Value":"CONTACTED","Name":"Contacted"},{"Value":"MISCONFIGURATION","Name":"Misconfiguration"},{"Value":"PAUSED","Name":"Paused"},{"Value":"SCHEDULED","Name":"Scheduled"}],"Field":"Status"},{"Table":"OutboundProcessContactRetries","Name":"Total Retries","Field":"TotalRetries"}]
]]--
m_data.LoadBatchContactFields = function(self, _T, vcc, keySpaceAgent)
	assert(vcc)
	_T = _T or function (s) return s end

	local data = {
		{
			Table = "",
			Name  = _T("Account Officer"),
			Field = "AccountOfficer",
			Type  = "keyword",
			AllowedOperators = nil, -- all
			AllegroFieldMap = {
				Field = "AccountOfficer" -- el mismo
			}
		},
		{
			Table = "",
			Name  = _T("External Id"),
			Field = "ExternalId",
			Type  = "keyword",
			LowerOrUpper = "lower",
			AllowedOperators = nil, -- all
			AllegroFieldMap = {
				Field = "NONE" -- el mismo
			}
		},
		-- {
		-- 	Table = "",
		-- 	Name = _T("Address Apartment"),
		-- 	Field = "Apt"
		-- },
		-- {
		-- 	Table  = "HomeAddresses",
		-- 	Name   = _T("Address Area"),
		-- 	Field  = "AddressArea",
		-- 	Type   = "keyword"
		-- },
		-- {
		-- 	Table  = "HomeAddresses",
		-- 	Name   = _T("Address Country"),
		-- 	Field  = "AddressCountry",
		-- 	Type   = "keyword"
		-- },
		-- {
		-- 	Table  = "HomeAddresses",
		-- 	Name   = _T("Address Type"),
		-- 	Field  = "AddressSubType",
		-- 	Type   = "keyword"
		-- },
		-- {
		-- 	Table  = "HomeAddresses",
		-- 	Name   = _T("Address Value"),
		-- 	Field  = "Value",
		-- 	Type   = "keyword"
		-- },
		{
			Table = "",
			Name  = _T("Category"),
			Field = "Category",
			Type  = "keyword",
			LowerOrUpper = "none",
			AllowedOperators = nil, -- all
			AllegroFieldMap = {
				Field = "Category" -- el mismo
			}
		},
		{
			Table = "",
			Name = _T("Batch Id"),
			Field = "BatchId",
			Type  = "keyword",
			LowerOrUpper = "lower",
			AllowedOperators = nil, -- all
			AllegroFieldMap = {
				Field = "OutboundProcessContact.BatchId"
			}
		},
		{
			Table = "",
			Name  = _T("Contact Id"),
			Field = "Id",
			Type  = "keyword",
			LowerOrUpper = "lower",
			Sanitize = function(value, operator--[[, rawData]])
				assert(value)
				operator = (operator or ""):lower()
				if operator ~= "contains" then
					value = utils.AppendVcc(utils.GetId(value), vcc)
				end
				return value
			end,
			AllowedOperators = nil, -- all
			AllegroFieldMap = {
				Field = "Id" -- el mismo
			}
		},
		{
			Table  = "",
			Name   = _T("Campaign"),
			Field  = "Campaign",
			Type   = "keyword",
			Values = function()
				return m_data.LoadVccCampaigns(vcc, keySpaceAgent)
			end,
			AllowedOperators = nil, -- all
			AllegroFieldMap = {
				Field = "Campaign" -- el mismo
			}
		},
		{
			Table  = "",
			Name   = _T("Last Contact Result"),
			Field  = "LastInteractionCode",
			Type   = "keyword",
			Values = function()
				local oeSystemAPI = utils.GetOEApi("system") -- Obtengo el API del "outbound_engine".
				local success, list = oeSystemAPI.native.LoadNativeResponsesByChannel("CALL", false)
				if not success then
					-- TODO: log
					return {}
				end
				local result = {}
				for _, info in ipairs(list) do
					local item = {
						Value = info.id,
						Name  = info.name or info.id
					}
					table.insert(result, item)
				end
				return result
			end,
			LowerOrUpper = "upper",
			AllowedOperators = nil, -- all
			AllegroFieldMap = {
				Field = "LastInteractionCode" -- el mismo
			}
		},
		{
			Table  = "",
			Name   = _T("Daily Retries"),
			Field  = "DailyRetries",
			Type   = "integer",
			LowerOrUpper = "none",
			AllowedOperators = {
				["="]    = {enabled = true },
				["<>"]   = {enabled = true },
				["<"]    = {enabled = true },
				[">"]    = {enabled = true },
				["<="]   = {enabled = true },
				[">="]   = {enabled = true },
				["LIKE"] = {enabled = false}
			},
			AllegroFieldMap = {
				Field = "OutboundProcessContactRetries.DayRetries"
			}
		},
		-- {
		-- 	Table = "",
		-- 	Name = _T("Email Address"),
		-- 	Field = "Address"
		-- },
		-- {
		-- 	Table = "",
		-- 	Name = _T("Email Type"),
		-- 	Field = "Type",
		-- 	Values = function()
		-- 		local list = {
		-- 			{Name = _T("PERSONAL"), Value = "PERSONAL"},
		-- 			{Name = _T("WORK")    , Value = "WORK"    }
		-- 		}
		-- 		return list
		-- 	end
		-- },
		-- {
		-- 	Table = "",
		-- 	Name = _T("Last Batch Contact Date"),
		-- 	Field = "PreviousContactDate"
		-- },
		{
			Table   = "",
			Name    = _T("Last Contact Date"),
			Field   = "LastContactDate",
			Type    = "datetime",
			LowerOrUpper = "none",
			AllowedOperators = {
				["="]    = {enabled = true },
				["<>"]   = {enabled = true },
				["<"]    = {enabled = true },
				[">"]    = {enabled = true },
				["<="]   = {enabled = true },
				[">="]   = {enabled = true },
				["LIKE"] = {enabled = false}
			},
			AllegroFieldMap = {
				Field = "LastContactDate" -- el mismo
			}
		},
		{
			Table  = "",
			Name   = _T("Last Management Result"),
			Field  = "LastManagementResult",
			Type   = "keyword",
			LowerOrUpper = "none",
			AllowedOperators = nil, -- all
			AllegroFieldMap = {
				Field = "LastManagementResult" -- el mismo
			}
		},
		{
			Table  = "",
			Name   = _T("State Cause"),
			Field  = "StateCause",
			Type   = "keyword",
			Values = function()
				return {
					{Value = "ACD"                          , Name = _T("ACD")                     },
					{Value = "ADDED_TO_BATCH"               , Name = _T("Added to Batch")          },
					{Value = "AGENT_LOGOUT"                 , Name = _T("Agent Logout")            },
					{Value = "AGENT_UNAVAILABLE"            , Name = _T("Agent Unavailable")       },
					{Value = "API"                          , Name = _T("API")                     },
					{Value = "BATCH_DELETED"                , Name = _T("Batch Deleted")           },
					{Value = "BATCH_ENDS"                   , Name = _T("Batch Ends")              },
					{Value = "BOT"                          , Name = _T("BOT")                     },
					{Value = "CANCELED_BY_RETRIES"          , Name = _T("Canceled By Retries")     },
					{Value = "CANCELED_BY_RULES"            , Name = _T("Canceled By Rules")       },
					{Value = "CONTACT_INTERACTION_NOT_FOUND", Name = _T("Conversation Not Found")  },
					{Value = "CONTACT_WITHOUT_INTERACTION"  , Name = _T("Without Conversation")    },
					{Value = "CONTACTED"                    , Name = _T("Contacted")               },
					{Value = "MANUAL"                       , Name = _T("Manual")                  },
					{Value = "NEW_BATCH"                    , Name = _T("New Batch")               },
					{Value = "NO_ACTIVE_RULE"               , Name = _T("No Active Rule")          },
					{Value = "NO_ADDRESS_DAILY_RETRIES"     , Name = _T("No Address Daily Retries")},
					{Value = "NO_DAILY_RETRIES"             , Name = _T("No Daily Retries")        },
					{Value = "NO_ENABLED_ADDRESSES"         , Name = _T("No Enabled Addresses")    },
					{Value = "NO_RULE_ADDRESSES"            , Name = _T("No Rule Addresses")       },
					{Value = "PAUSED"                       , Name = _T("Paused")                  },
					{Value = "PREVIEW"                      , Name = _T("Preview")                 },
					{Value = "RECYCLED"                     , Name = _T("Recycled")                },
					{Value = "REQUEST_AGENT_FAIL"           , Name = _T("Request Agent Fail")      },
					{Value = "RESERVING_AGENT"              , Name = _T("Reserving Agent")         },
					{Value = "RESUMED"                      , Name = _T("Resumed")                 },
					{Value = "RUNNING"                      , Name = _T("Running")                 },
					{Value = "SCHEDULED_BY_RULES"           , Name = _T("Scheduled By Rules")      },
					{Value = "START_PROCESS"                , Name = _T("Start Process")           },
					{Value = "STOP_PROCESS"                 , Name = _T("Stop Process")            },
					{Value = "TAKEN"                        , Name = _T("Taken")                   },
					{Value = "TIMEOUT_REQUESTING_AGENT"     , Name = _T("Timeout Requesting Agent")},
					{Value = "TIMEOUT_RUNNING"              , Name = _T("Timeout Running")         },
					{Value = "WRAPUP"                       , Name = _T("Wrapup")                  },
				}
			end,
			LowerOrUpper = "upper",
			AllowedOperators = nil, -- all
			AllegroFieldMap = {
				Field = "OutboundProcessContact.Cause"
			}
		},
		{
			Table  = "",
			Name   = _T("Name"),
			Field  = "Name",
			Type   = "text",
			LowerOrUpper = "none",
			AllowedOperators = nil, -- all
			AllegroFieldMap = {
				Field = "Name" -- el mismo
			}
		},
		{
			Table  = "",
			Name   = _T("Importation"),
			Field  = "ImportId",
			Type   = "keyword",
			Values = function()
				local importsModule = require "apps.outbound_engine.controllers.imports"
				local list = {}
				local imports = importsModule.ListImportations()
				for _, item in pairs(imports) do
					table.insert(list, item.ID)
				end
				return list
			end,
			LowerOrUpper = "lower",
			AllowedOperators = nil, -- all
			AllegroFieldMap = {
				Field = "LastUpdate" -- hay que tener cuidado ya que en ES, este campo mapea con una fecha y en el caso de allegro NO tenemos una fecha con la ultima actualizacion
			}
		},
		-- {
		-- 	Table  = "Addresses",
		-- 	Name   = _T("Phone Area"),
		-- 	Field  =  MakeNestedFieldName("Addresses", Area, fieldValue),
		-- 	--Field  = "Area",
		-- 	Type   = "keyword",
		-- 	Nested = true
		-- },
		-- {
		-- 	Table  = "Addresses",
		-- 	Name   = _T("Phone Country"),
		-- 	Field  = "Country",
		-- 	Type   = "keyword",
		-- 	Nested = true
		-- },
		-- {
		-- 	Table  = "Addresses",
		-- 	Name   = _T("Phone Extension"),
		-- 	Field  = "Extension",
		-- 	Type   = "keyword",
		-- 	Nested = true
		-- },
		{
			Table  = "Addresses",
			Name   = _T("Phone Number"),
			Field  =  MakeNestedFieldName("Addresses", "Number"),
			Type   = "keyword",
			Nested = true,
			EntityType = "ADDRESSES",
			AllowedOperators = nil, -- all
			AllegroFieldMap = {
				Field = "ContactImportPhone.Number"
			}
		},
		{
			Table  = "Addresses",
			Name   = _T("Phone Type"),
			Field  =  MakeNestedFieldName("Addresses", "Kind"),
			Type   = "keyword",
			Nested = true,
			EntityType = "ADDRESSES",
			Values = function()
				local list = {
					{Name = _T("FAX")   , Value = "FAX"     },
					{Name = _T("HOME")  , Value = "HOME"    },
					{Name = _T("MOBILE"), Value = "CELLULAR"},
					{Name = _T("OFFICE"), Value = "OFFICE"  }
				}
				return list
			end,
			AllowedOperators = nil, -- all
			AllegroFieldMap = {
				Field = "ContactImportPhone.Type"
			}
		},
		-- {
		-- 	Table = "",
		-- 	Name  = _T("Priority on Batch"),
		-- 	Field = "ContactPriority"                 -- PRIORITY EN FULLCONTACT.ELASTIC_SERACH.LUA -> m_elasiticInfo.contactsDialer.MakeTemplate
		-- },
		{
			Table = "",
			Name  = _T("Agent"),
			Field = "Agent",
			Type  = "keyword",
			LowerOrUpper = "lower",
			AllowedOperators = nil, -- all
			AllegroFieldMap = {
				Field = "OutboundProcessContact.ReservedActor" -- el mismo
			}
		},
		{
			Table = "",
			Name  = _T("Contact Scheduled Date"),
			Field = "ContactScheduleDate",
			Type  = "date",
			LowerOrUpper = "none",
			AllowedOperators = {
				["="]    = {enabled = true },
				["<>"]   = {enabled = true },
				["<"]    = {enabled = true },
				[">"]    = {enabled = true },
				["<="]   = {enabled = true },
				[">="]   = {enabled = true },
				["LIKE"] = {enabled = false}
			},
			AllegroFieldMap = {
				Field = "OutboundProcessLastContact.LastContact"
			}
		},
		{
			Table = "",
			Name  = _T("Manual Scheduled Date"),
			Field = "ManualScheduleDate",
			Type  = "date",
			LowerOrUpper = "none",
			AllowedOperators = {
				["="]    = {enabled = true },
				["<>"]   = {enabled = true },
				["<"]    = {enabled = true },
				[">"]    = {enabled = true },
				["<="]   = {enabled = true },
				[">="]   = {enabled = true },
				["LIKE"] = {enabled = false}
			},
			AllegroFieldMap = {
				Field = "OutboundProcessContact.PreviousContactDate"
			}
		},
		{
			Table = "",
			Name  = _T("Scheduled Date"),
			Field = "ScheduleDate",
			Type  = "date",
			LowerOrUpper = "none",
			AllowedOperators = nil,
			AllowedOperators = {
				["="]    = {enabled = true },
				["<>"]   = {enabled = true },
				["<"]    = {enabled = true },
				[">"]    = {enabled = true },
				["<="]   = {enabled = true },
				[">="]   = {enabled = true },
				["LIKE"] = {enabled = false}
			},
			AllegroFieldMap = {
				Field = "OutboundProcessContact.ScheduledDate"
			}
		},
		{
			Table = "",
			Name = _T("Status on Batch"),
			Field = "State",
			Values = function()
				local list = {
					{Name = _T("SUMMARY NEW CONTACTS"), Value = "FREE"     },
					{Name = _T("SUMMARY SCHEDULED")   , Value = "SCHEDULED"},
					{Name = _T("SUMMARY ACTIVE")      , Value = "ACTIVE"   },
					{Name = _T("SUMMARY FINISHED")    , Value = "FINISHED" },
					{Name = _T("SUMMARY CANCELED")    , Value = "CANCELED" },
					{Name = _T("SUMMARY PENDINGS")    , Value = "PENDING"  },
					{Name = _T("SUMMARY IN PROCESS")  , Value = "PROCESS"  },
					{Name = _T("SUMMARY PAUSED")      , Value = "PAUSED"   }
				}
				return list
			end,
			LowerOrUpper = "upper",
			AllowedOperators = nil, -- all
			AllegroFieldMap = {
				Field = "OutboundProcessContact.Status",
				MakeValue = function (allegroValue)
					if tostring((allegroValue or "")):upper() == "CONTACTED" then
						return "FINISHED" -- native value
					end
					return allegroValue
				end
			}
		},
		{
			Table = "",
			Name  = _T("Total Retries"),
			Field = "TotalRetries",
			Type  = "integer",
			LowerOrUpper = "none",
			AllowedOperators = {
				["="]    = {enabled = true },
				["<>"]   = {enabled = true },
				["<"]    = {enabled = true },
				[">"]    = {enabled = true },
				["<="]   = {enabled = true },
				[">="]   = {enabled = true },
				["LIKE"] = {enabled = false}
			},
			AllegroFieldMap = {
				Field = "OutboundProcessContactRetries.TotalRetries"
			}
		}
	}

	-- cargo los skills
	local skills = m_data.LoadVccSkills(vcc, keySpaceAgent)
	for _, skill in pairs(skills or {}) do
		local skillInfo = {
			Table  = "Skills",
			Name   = string.format("%s: %s", _T("Skill"), skill.Description),
			Field  = MakeNestedFieldName("Skills", "SkillId", skill.Name),
			Type   = "keyword",
			Nested = true,
			EntityType = "SKILLS",
			LowerOrUpper = "none",
			AllowedOperators = nil, -- all
			AllegroFieldMap = {
				Field = function(self, allegroQuery)
					-- LogDebug("[OUTBOUNDENGINE.CLIENT:m_queries.LoadBatchContactFields] --> ENTRA --> Skill Field (def.Name: '%s', def.Field: '%s') -> making allegro migration field name. allegroQuery: %s",
					-- 	self.Name,
					-- 	self.Field,
					-- 	m_helpers.table:JSON_Encode(allegroQuery)
					-- )
					if allegroQuery.isCustomField then -- los isCustomField=false NO son los NAMEVALUES
						--LogDebug("[OUTBOUNDENGINE.CLIENT:m_queries.LoadBatchContactFields.Skill]   --> 1) NO HAGO NADA SKILL")
						return "NONE"
					end
					if not allegroQuery.field then
						--LogDebug("[OUTBOUNDENGINE.CLIENT:m_queries.LoadBatchContactFields.Skill]   --> 2) NO HAGO NADA SKILL")
						return "NONE"
					end
					local result = m_helpers.utils.SplitString(allegroQuery.field or "", ".")
					if #result == 0 then
						--LogDebug("[OUTBOUNDENGINE.CLIENT:m_queries.LoadBatchContactFields.Skill]   --> 3) NO HAGO NADA")
						return "NONE"
					end
					local tableName = result[1]
					if tableName:upper() ~= "CONTACTIMPORTSKILL" then
						--LogDebug("[OUTBOUNDENGINE.CLIENT:m_queries.LoadBatchContactFields.Skill]   --> 4) NO HAGO NADA - tableName: %s", tableName)
						return "NONE"
					end
					local skillName = result[2]
					local currentDefinitionFieldName = MakeNestedFieldName("Skills", "SkillId", skillName)
					if currentDefinitionFieldName ~= self.Field then
						--LogDebug("[OUTBOUNDENGINE.CLIENT:m_queries.LoadBatchContactFields.Skill]   --> 5) NO HAGO NADA - tableName: %s, currentDefinitionFieldName: %s", tableName, currentDefinitionFieldName)
						return "NONE"
					end
					local fieldName = allegroQuery.field
					--LogDebug("[OUTBOUNDENGINE.CLIENT:m_queries.LoadBatchContactFields.Skill]   --> SALE --> Skill (def.Name: '%s', def.Field: '%s') currentDefinitionFieldName: '%s' ->  fieldName: '%s': fieldValue: '%s'",
					-- 	self.Name,
					-- 	self.Field,
					-- 	currentDefinitionFieldName,
					-- 	fieldName,
					-- 	allegroQuery.value
					-- )
					return fieldName
				end
			}
		}
		table.insert(data, skillInfo)
	end

	-- Cargo los NameValuesSearch
	local function SetPropertiesImportInfo (importFormatInfo, searchableDataType, field)
		--LogDebug("[OUTBOUND_ENGINE:LoadBatchContactFields]        1----> searchableDataType: %s, field: %s, importFormatInfo: %s", searchableDataType, json.encode(field), json.encode(importFormatInfo))

		--[[ ----------------------------------------------------------------------------------------------------------
			(**) dejo comentada funcion en la trate de migrar los filtros de tipo NameValues.
			No puede ya no tengo de donde agarrarme para poder crear el nombre del filtro que utiliza Nativo.

			Ejemplo:
				1) Lo que llega de allegro en linea del Query del filtro.
					{"logicalOp":"OR","leftParenth":"","isCustomField":true,"value":"nv1","rightParenth":"","conditionOp":"=","field":"NameValField1"}
				2) La lista de campos disponibles que genera y necesita Nativo:
					{
					"logicalOp": "OR",
					"leftParenth": "",
					"value": "nv1",
					"rightParenth": "",
					"conditionOp": "=",
					"field": "TABLE_NameValuesSearchText_FIELDNAME_Name_FIELDVALUE_NameValueString1"
					}
				donde ese valor "NameValueString1" no me llega cuando creo los formatos de imporatcion desde Allegro y no puedo maperalo.
		---------------------------------------------------------------------------------------------------------------]]--

		-- local function MigrationMakeField (self, allegroQuery)
		-- 	--LogDebug("[OUTBOUNDENGINE.CLIENT:m_queries.LoadBatchContactFields.NameValue] --> ENTRA --> NameValueType: '%s', Table: '%s' Field (def.Name: '%s', def.Field: '%s') -> making allegro migration field name. allegroQuery: %s",
		-- 	-- 	self.EntityType,
		-- 	-- 	self.Table,
		-- 	-- 	self.Name,
		-- 	-- 	self.Field,
		-- 	-- 	m_helpers.table:JSON_Encode(allegroQuery)
		-- 	-- )
		-- 	if not allegroQuery.isCustomField then -- los isCustomField=true son los NAMEVALUES
		-- 		--LogDebug("[OUTBOUNDENGINE.CLIENT:m_queries.LoadBatchContactFields.NameValue]   --> 1) NO HAGO NADA NAMEVALUES")
		-- 		return "NONE"
		-- 	end
		-- 	local fieldName = allegroQuery.field
		-- 	if not fieldName then
		-- 		--LogDebug("[OUTBOUNDENGINE.CLIENT:m_queries.LoadBatchContactFields.NameValue]   --> 2) NO HAGO NADA NAMEVALUES")
		-- 		return "NONE"
		-- 	end
		-- 	local currentDefinitionFieldName = MakeNestedFieldName(
		-- 		"NameValuesSearchText", --self.Table
		-- 		"Name",
		-- 		fieldName
		-- 	)
		-- 	if currentDefinitionFieldName ~= self.Field then
		-- 		--LogDebug("[OUTBOUNDENGINE.CLIENT:m_queries.LoadBatchContactFields.NameValue]   --> 3) NO HAGO NADA NAMEVALUES --> currentDefinitionFieldName: '%s' -> DISTINT ->  def.Field: '%s' ", currentDefinitionFieldName, self.Field)
		-- 		return "NONE"
		-- 	end
		-- 	--LogDebug("[OUTBOUNDENGINE.CLIENT:m_queries.LoadBatchContactFields.NameValue]   --> SALE --> NameValueType: '%s', Table: '%s'  (def.Name: '%s', def.Field: '%s') currentDefinitionFieldName: '%s' ->  fieldName: '%s': fieldValue: '%s'",
		-- 	-- 	self.EntityType,
		-- 	-- 	self.Table,
		-- 	-- 	self.Name,
		-- 	-- 	self.Field,
		-- 	-- 	currentDefinitionFieldName,
		-- 	-- 	fieldName,
		-- 	-- 	allegroQuery.value
		-- 	-- )
		-- 	return fieldName
		-- end

		local searchebleInfo = {
			DATE = {
				Name = _T("Name Value Date"),
				Table = "NameValuesSearchDate",
				Type = "date",
				EntityType = "NAME_VALUE_SEARCH",
				AllowedOperators = {
					["="]    = {enabled = true },
					["<>"]   = {enabled = true },
					["<"]    = {enabled = true },
					[">"]    = {enabled = true },
					["<="]   = {enabled = true },
					[">="]   = {enabled = true },
					["LIKE"] = {enabled = false}
				},
				AllegroFieldMap = {
					NameValueType = "NAME_VALUE_SEARCH_DATE",
					Field = "NONE"
				}
			},
			NUMERIC = {
				Name = _T("Name Value Number"),
				Table = "NameValuesSearchNumeric",
				Type = "double",
				EntityType = "NAME_VALUE_SEARCH",
				AllowedOperators = {
					["="]    = {enabled = true },
					["<>"]   = {enabled = true },
					["<"]    = {enabled = true },
					[">"]    = {enabled = true },
					["<="]   = {enabled = true },
					[">="]   = {enabled = true },
					["LIKE"] = {enabled = false}
				},
				AllegroFieldMap = {
					NameValueType = "NAME_VALUE_SEARCH_NUMERIC",
					Field = "NONE"
				}
			},
			TEXT = {
				Name = _T("Name Value Text"),
				Table = "NameValuesSearchText",
				Type = "keyword",
				EntityType = "NAME_VALUE_SEARCH",
				AllowedOperators = nil, -- all
				AllegroFieldMap = {
					NameValueType = "NAME_VALUE_SEARCH_TEXT",
					Field = "NONE"
					---- (**) lo dejo comentado por si retomo y se valida si realmente se pueden migrar los filtros namevalues.
					-- Field = function(self, allegroQuery)
					-- 	return MigrationMakeField(self, allegroQuery)
					-- end
				}
			}
		}
		local info = searchebleInfo[searchableDataType]
		if not info then
			LogWarning("[OUTBOUND_ENGINE:LoadBatchContactFields] Unrecognized NameValuesSearch Type: %s", searchableDataType)
			return false
		end
		local displayName = info.Name or _T("No Name")
		local dislpayImportFormatName = string.format("(%s: %s)", _T("Import Format"), importFormatInfo.ImportFormatId)
		importFormatInfo.Name  = string.format("%s: %s - %s", displayName, field.Name, dislpayImportFormatName)
		importFormatInfo.Field = MakeNestedFieldName(info.Table, "Name", field.Name)
		importFormatInfo.Table = info.Table
		importFormatInfo.Type  = info.Type
		importFormatInfo.EntityType = info.EntityType
		importFormatInfo.LowerOrUpper = "lower"
		importFormatInfo.AllowedOperators = info.AllowedOperators
		importFormatInfo.AllegroFieldMap = info.AllegroFieldMap
		--LogDebug("[OUTBOUND_ENGINE:LoadBatchContactFields]        2----> searchableDataType: %s, field: %s, importFormatInfo: %s", searchableDataType, json.encode(field), json.encode(importFormatInfo))
		-- ok
		return true
	end

	local importFormats = m_data.ListImportFormat(vcc)
	for _, importFormat in pairs(importFormats) do
		--LogDebug("[OUTBOUND_ENGINE:LoadBatchContactFields]   importFormats--> %s, %s", _, json.encode(importFormat))
		for _1, field in ipairs(importFormat.ContactImportFormatFields or {}) do
			if field.TypeName:upper() == "NAMEVALUE" and field.IsSearchable then
				local importFormatInfo = {Nested = true, ImportFormatId = importFormat.Id}
				local searchableDataType = field.SearchableDataType:upper()
				local mustAdd = SetPropertiesImportInfo(importFormatInfo, searchableDataType, field)
				if mustAdd then
					table.insert(data, importFormatInfo)
				end
			end
		end
	end

	-- Cargo los valores de los combos calculados
	for _, item in ipairs(data) do
		if type(item.Values) == "function" then
			local values = item.Values()
			item.Values = values
		end
		item.ParsedFieldName = ParseFieldName(item.Field)
	end

	--LogDebug("[OUTBOUND_ENGINE:LoadBatchContactFields]  data----->  %s", json.encode(data))
	return data
end

------------------------------
return {
	queries = m_queries,
	data    = m_data
}