/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 9 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	const FORMAT = inConcert.Utils.format;

	//---------------------------------------------------------------------
	// ExportLayout Management
	function ExportLayoutManagement(params)
	{
		if (!params) {
			throw("ExportLayoutManagement - 'params' is required");
		}
		if (!params.owner) {
			throw("ExportLayoutManagement - 'params.owner' is required");
		}
		if (typeof params.owner.show_status_message !== "function") {
			throw("ExportLayoutManagement - 'params.owner.show_status_message' is not a function");
		}

		this.owner               = params.owner;
		this.fields_container_id = params.ids.fields_container;

		this.m_hnd_exp_column_types = new inConcert.outbound_engine_exportation_column_types_def();
		this.m_columnTypesDefinitions = this.m_hnd_exp_column_types.GetColumnsTypesDefinitions();
		this.m_hashBasicEspecialColumns = this.m_hnd_exp_column_types.GetHashBasicEspecialColumns(this.m_columnTypesDefinitions);
		this.m_selectedItems = 0;
		this.m_counterPhoneGroup = 0;
		this.m_counterNameValueGroup = 1;
		this.m_counterSkillGroup = 1;
		this.m_hashColumnTypesById = {};    // hash utilizado para acceder al type (texto) de la columna cuando tenemos el id (numéro de type de columna)
		this.m_hashColumnTypesByName = {};  // hash utilizado para acceder al type (numero) de la columna cuando tenemos el id (nombre de type de columna)
		this.m_hashColumnLayoutByName = {}; // hash utilizado para saber si el nombre de la misma ya fue agregada a layout (es para que no se vuelvan a repetir)
		this.m_hashSortColumns = {};
		this.m_htmlGroupsContent = null;
		this.m_hashDynamicPhoneGroup = {"PhoneExtraData": {groupId: "phone"}}; // hash que contiene los groupos phone dinamicos que se van agregando

		this.m_grd_layouts = null;
		this.__new_grids();

		return true;
	}

	if (typeof inConcert.ExportLayoutManagement !== "function") {
		inConcert.OutboundEngineExportLayout = ExportLayoutManagement;
	}

	ExportLayoutManagement.prototype.initialize = function()
	{
		this.m_grd_layouts.initialize($.noop, []);
		this.m_htmlGroupsContent = {
			contentBasicGroup    : $("#contentBasicGroup").html(),
			contentAttemptsGroup : $("#contentAttemptsGroup").html(),
			contentNameValueGroup: $("#contentNameValueGroup").html(),
			contentSkillGroup    : $("#contentSkillGroup").html(),
			contentPhoneGroup    : $("#contentPhoneGroup").html(),
			contentBatchGroup    : $("#contentBatchGroup").html(),
			contentContactGroup  : $("#contentContactGroup").html()
		};
		// limpio info
		this.__clean_design_form_field();
		// hash utilizado para saber si el nombre de la misma ya fue agregada a layout (es para que no se vuelvan a repetir)
		this.m_hashColumnLayoutByName = {};
	};

	ExportLayoutManagement.prototype.dispose = function()
	{
		//TODO: implementar, si es necesario
	};

	ExportLayoutManagement.prototype.get = function()
	{
		const self = this;
		//funcion que carga a partir de la grilla el array a enviar a guardar.
		var FillExportColumns = function() {
			let columns = [];
			let rows = self.m_grd_layouts.get_items();
			rows.forEach((row, i) => {
				let column = {
					Order: i + 1,
					Type : row.Type,
					Name : row.Name
				};
				let extraDataColumnName = row.ColumnDefinition.extraDataColumnName;
				if (extraDataColumnName !== undefined) {
					if (column.Type === 18 || column.Type === 61) {
						// 18 = namevalue or 61 = skills
						column[extraDataColumnName] = row[extraDataColumnName] || row.fieldName;
					}
					else if (column.Type === 10) {
						// sample: columnsTypes.CONTACT_TOTAL_ATTEMPTS_ABANDONED.extraData = "ABANDONED" (\scripts\outbound_engine_exportation_column_types_def.js)
						column[extraDataColumnName] = row[extraDataColumnName] || "";
					}
					else {
						column[extraDataColumnName] = row[extraDataColumnName] || "";
					}
				}
				columns.push(column);
			});
			return columns;
		};

		const layout = {
			Id: $("#txtId").val(),
			Description: $("#txtDescription").val(),
			ContactExportLayoutColumns: FillExportColumns()
		};
		return layout;
	};

	ExportLayoutManagement.prototype.set = function(layout)
	{
		$("#txtId").val(layout.Id);
		$("#txtDescription").val(layout.Description);
		this.__set_layout_columns(layout.ContactExportLayoutColumns);
		this.m_grd_layouts.fill_rows($.noop, this.m_hashSortColumns);
	};

	ExportLayoutManagement.prototype.save = function(layout, callback = $.noop)
	{
		layout = layout || this.get();
		const outbound_engine_api = inConcert.app.getOutboundEngineAPI();

		outbound_engine_api.exportLayoutSave(layout,
			function(success, resp) {
				if (!success) {
					const err = resp;
					callback(false, err);
					return;
				}
				callback(true, resp);
			}
		);
	};

	ExportLayoutManagement.prototype.__set_layout_columns = function(columns)
	{
		this.m_hashSortColumns = {};
		if (!columns) {
			return;
		}
		let columnsLength = columns.length;
		//los agrego en el hash por la columna order, es para poder ser agregados en orden (segun campo orden)
		for (let i = 0; i < columnsLength; i++) {
			let column = columns[i];
			let key = this.m_hashColumnTypesById[column.Type];
			let columnDefinition = this.m_columnTypesDefinitions[key];
			// 18 = namevalue, 61 = skills
			if (column.Type === 18 || column.Type === 61) {
				let extraDataColumnName = columnDefinition.extraDataColumnName;
				column.fieldName = column[extraDataColumnName];
				column[extraDataColumnName] = "";
			}
			column.groupName = columnDefinition.translateGroup;
			column.ColumnDefinition = columnDefinition;

			this.m_hashSortColumns[columns[i].Order] = column;
		}
	};

	ExportLayoutManagement.prototype.show_status_message = function(options)
	{
		const message   = options.message;
		const type      = options.type || "Error";
		const time_ms   = options.time_ms || 5000;
		const scrolling = options.scrolling || undefined;

		this.owner.show_status_message(
			message,
			type,
			time_ms,
			scrolling
		);
	};

	ExportLayoutManagement.prototype.hide_messages = function()
	{
		this.owner.hide_messages();
	};

	ExportLayoutManagement.prototype.show_waiting = function(message)
	{
		this.owner.show_waiting(message);
	};

	ExportLayoutManagement.prototype.hide_waiting = function()
	{
		this.owner.hide_waiting();
	};

	ExportLayoutManagement.prototype.editable = function()
	{
		$("#out_form_exp_layout_fields_title_container").show();
		$("#out_form_exp_layout_fields_container").show();
		$("#out_form_exp_layout_fields_back_container").show();
		$("#out_form_exp_layout_list_help").show();
		this.m_grd_layouts.editable();
		$("#txtId").focus();
	};

	ExportLayoutManagement.prototype.readonly = function()
	{
		$("#out_form_exp_layout_fields_title_container").hide();
		$("#out_form_exp_layout_fields_container").hide();
		$("#out_form_exp_layout_fields_back_container").hide();
		$("#out_form_exp_layout_list_help").hide();
		this.m_grd_layouts.readonly();
	};

	ExportLayoutManagement.prototype.empty_controls = function()
	{
		this.m_grd_layouts.remove_all_rows();
		$("#txtDescription").val("");
		$("#txtId").val("").focus();
		// limpio info
		this.__clean_design_form_field();
	};

	ExportLayoutManagement.prototype.__clean_design_form_field = function()
	{
		// funcion que luego inicializa las variables y estructuras luego de agregar un layout a la grilla.
		this.m_counterPhoneGroup = 0;
		this.m_counterNameValueGroup = 1;
		this.m_counterSkillGroup = 1;
		this.m_columnTypesDefinitions = this.m_hnd_exp_column_types.GetColumnsTypesDefinitions();
		this.m_selectedItems = 0;
		this.m_hashDynamicPhoneGroup = {"PhoneExtraData": {groupId: "phone"}}; 
		$("#cmdAddLayout").prop("disabled", true).addClass("formButtonAdd-ds");
		for (var divId in this.m_htmlGroupsContent) {
			if (this.m_htmlGroupsContent.hasOwnProperty(divId)) {
				$("#" + divId).html(this.m_htmlGroupsContent[divId]);
			}
		}
		const $fields_container = $("#" + this.fields_container_id);
		$fields_container.find("[columns_phone_group_delete]").css("visibility", "hidden");
		this.hide_messages();
		this.hide_waiting();
	};

	// valida si la columna existe en el hash de columna por su nombre
	ExportLayoutManagement.prototype.__exist_column_in_layout = function(columnName) {
		if (this.m_hashColumnLayoutByName[columnName.toLowerCase()] === undefined) {
			return false;
		}
		return true;
	};

	//function que retorna si se pueden agregar las columans a la grilla.
	//  valida columna a columna.
	ExportLayoutManagement.prototype.__can_add_columns = function()
	{
		const self = this;

		//----------------------------------------------------------------------------
		//----------------------------------------------------------------------------
		// function que muestra/oculta imagen correspondiente a un error al lado de una columna.
		//	tambien inicializa el tooltip correspondiente.
		var ShowError = function(columnType, showHideMethodName, messageError) {
			const $err = $("#tooltip" + columnType);
			if ("show" === showHideMethodName) {
				$err.show().attr("title", messageError);
			}
			else {
				$err.hide().attr("title", "");
			}
		};
		//----------------------------------------------------------------------------
		//----------------------------------------------------------------------------
		//funcion que valida el diseño de campo a campo
		var ValidateDesignField = function(controlId) {
			//----------------------------------------------------------------------------
			// funcion que valida si ya no existe el nombre de columna a agrear en los demas textbox del diseño de layout
			var ExistColumnNameInDesignLayout = function(columnName, columnType) {
				var response = {exist: false};
				for (var key in self.m_columnTypesDefinitions) {
					if (self.m_columnTypesDefinitions.hasOwnProperty(key)) {
						if (columnType.toLowerCase() === key.toLowerCase()) {
							continue;
						}
						var columnDefinition = self.m_columnTypesDefinitions[key];
						if (columnName.toLowerCase() === $("#txt" + columnDefinition.editableFieldId).val().toLowerCase()) {
							response.exist = true;
							response.conflictColumnType = key;
							response.conflictEditableFieldId = columnDefinition.editableFieldId;
							break;
						}
					}
				}
				return response;
			};
			//----------------------------------------------------------------------------
			// funcion que valida si el nombre de la columna a agregar cumpla con:
			//	1 - que no sea vacia
			//	2 - que no se repita el nombre en los diferentes textbox correspondiente a las columnas
			//	3 - que ya no exista en la grilla de layout
			var IsCorrectName = function(columnType, textValue, isColumnName) {
				var response = {success: true, messageError: ""};
				if (textValue.replace(/^\s*|\s*$/g, "").length === 0) {
					response.success = false;
					response.numberError = -1;
					response.messageError = (isColumnName === 0 ? "<%=_T('The field name can not be empty.')%>" : "<%=_T('The column name can not be empty.')%>");
					return response;
				}
				if (isColumnName === 0) {
					return response;
				}
				var responseExist = ExistColumnNameInDesignLayout(textValue, columnType);
				if (responseExist.exist) {
					response.success = false;
					response.numberError = -2;
					response.messageError = "<%=_T('The column name exists in the design.')%>";
					return response;
				}
				if (self.__exist_column_in_layout(textValue)) {
					response.success = false;
					response.numberError = -3;
					response.messageError = "<%=_T('The name of the column and was admitted.')%>";
					return response;
				}
				return response;
			};
			//----------------------------------------------------------------------------
			var response;
			var columnType = controlId;
			controlId = "#txt" + controlId;
			let $ctrl = $(controlId);
			var textValue = $ctrl.val();
			var displayToolTipMethod = "hide";
			if ($ctrl.prop("disabled")) {
				ShowError(columnType, displayToolTipMethod, "");
				return true;
			}
			if ($ctrl.attr("namevalue") === undefined  && $ctrl.attr("skill") === undefined) {
			// para todos los grupos menos los namevalues
				columnType = $ctrl.attr("id").substr(3);
				response = IsCorrectName(columnType, textValue);
			}
			else {
				let textFieldValue, textColumnValue, isColumnName;

				columnType = columnType = $(controlId).attr("skill");
				if (columnType !== undefined) {
					// para los skills
					let skillResponse;
					isColumnName = parseInt($(controlId).attr("iscolumname"), 10);
					if (isColumnName === 1) {
						textFieldValue = $("#" + $(controlId).attr("fieldid")).val();
						textColumnValue = textValue;
					}
					else {
						textFieldValue = textValue;
						textColumnValue = $("#" + $(controlId).attr("columnid")).val();
					}
					skillResponse = IsCorrectName(columnType, textFieldValue, 0);
					response = IsCorrectName(columnType, textColumnValue);
					if (!skillResponse.success) {
						response.success = skillResponse.success;
						response.messageError = skillResponse.messageError + response.messageError;
					}

				} 
				else { 
					// para los namevalues
					columnType = $(controlId).attr("namevalue");
					let nameValueResponse;
					isColumnName = parseInt($(controlId).attr("iscolumname"), 10);
					if (isColumnName === 1) {
						textFieldValue = $("#" + $(controlId).attr("fieldid")).val();
						textColumnValue = textValue;
					}
					else {
						textFieldValue = textValue;
						textColumnValue = $("#" + $(controlId).attr("columnid")).val();
					}
					nameValueResponse = IsCorrectName(columnType, textFieldValue, 0);
					response = IsCorrectName(columnType, textColumnValue);
					if (!nameValueResponse.success) {
						response.success = nameValueResponse.success;
						response.messageError = nameValueResponse.messageError + response.messageError;
					}
				}
			}
			if (!response.success) {
				displayToolTipMethod = "show";
			}
			ShowError(columnType, displayToolTipMethod, response.messageError);
			return response.success;
		};

		//----------------------------------------------------------------------------
		//----------------------------------------------------------------------------
		var groupsError = {};
		var canAdd = 0;
		// recorre columna a columna y las valida contando la cantidad de errores por grupo y total
		for (var key in this.m_columnTypesDefinitions) {
			if (this.m_columnTypesDefinitions.hasOwnProperty(key)) {
				var columnDefinition = this.m_columnTypesDefinitions[key];
				groupsError[columnDefinition.group] = groupsError[columnDefinition.group] || {number: 0, markupId: columnDefinition.markUpIdErrorGroup};
				if (!ValidateDesignField(columnDefinition.editableFieldId)) {
					canAdd++;
					groupsError[columnDefinition.group].number++;
				}
			}
		}

		// Valida en los grupos phoneGroup, que si se ha marcado el option "select one" se agregue datos en el textbox del filtro.
		for (let phoneGroup in this.m_hashDynamicPhoneGroup) {
			if (this.m_hashDynamicPhoneGroup.hasOwnProperty(phoneGroup)) {
				let method = "hide";
				let message = "";
				if ($("#opt" + phoneGroup + "_all").attr("checked")) {
					ShowError(phoneGroup, method, message);
					continue;
				}
				if ($("#txt" + phoneGroup).val().replace(/^\s*|\s*$/g, "").length === 0) {
					canAdd++;
					groupsError[this.m_hashDynamicPhoneGroup[phoneGroup].groupId].number++;
					message = "<%=_T('This field is required.')%>";
					method = "show";
				}
				ShowError(phoneGroup, method, message);
			}
		}
		// muestra los errores por grupo
		for (let group in groupsError) {
			if (groupsError.hasOwnProperty(group)) {
				let message = "";
				let methodName = "hide";
				let error = groupsError[group];
				if (error.number > 0) {
					methodName = "show";
					message = "<%=_T('Group contains %1 %2.')%>".replace(/%1/g, error.number)
								.replace(/%2/g, error.number > 1 ? "<%=_T('errors')%>" : "<%=_T('error')%>");
				}
				$("#" + error.markupId).text(message)[methodName]();
			}
		}

		return canAdd;
	};

	// muestra/oculta un grupo
	ExportLayoutManagement.prototype.collapse_group = function(groupId, elementId)
	{
		const groups = {
			basic:                   {contentGroupId: "contentBasicGroup"},
			subgroup_basic_attempts: {contentGroupId: "contentAttemptsGroup"},
			namevalue:               {contentGroupId: "contentNameValueGroup", buttonNewGroupId: "lnkNewNameValueGroup"},
			skill:                   {contentGroupId: "contentSkillGroup", buttonNewGroupId: "lnkNewSkillGroup"},
			phone:                   {contentGroupId: "contentPhoneGroup", buttonNewGroupId: "lnkNewPhoneGroup"},
			batch:                   {contentGroupId: "contentBatchGroup"},
			contact:                 {contentGroupId: "contentContactGroup"}
		};
		const panelGroup = groups[groupId];
		const panelElement = $("#" + panelGroup.contentGroupId);
		let methodName = "hide";
		let classToRemove = "layoutGroupTitleShow";
		let classToAdd = "layoutGroupTitleHide";
		if (panelElement.css("display").toLowerCase() === "none") {
			methodName = "show";
			const temporalClassName = classToRemove;
			classToRemove = classToAdd;
			classToAdd = temporalClassName;
		}
		panelElement[methodName]();
		$("#" + elementId).removeClass(classToRemove).addClass(classToAdd);
		if (panelGroup.buttonNewGroupId !== undefined) {
			$("#" + panelGroup.buttonNewGroupId)[methodName]();
		}
	};

	ExportLayoutManagement.prototype.__add_columns = function()
	{
		const self = this;

		this.hide_messages();
		let numberErrors = this.__can_add_columns();

		if (numberErrors > 0) {
			this.hide_waiting();
			const message_info = {
				message  : "<%=_T('Layout contains %1 %2 with error.')%>".replace(/%1/g, numberErrors).replace(/%2/g, numberErrors > 1 ? "<%=_T('fields')%>" : "<%=_T('field')%>"),
				scrolling: true
			};
			this.show_status_message(message_info);
			return;
		}

		var add_columns = function() {
			for (let key in self.m_columnTypesDefinitions) {
				if (self.m_columnTypesDefinitions.hasOwnProperty(key)) {
					let columnDefinition = self.m_columnTypesDefinitions[key];
					let columnName = $("#txt" + columnDefinition.editableFieldId).val();
					let temporalName = columnName.replace(/^\s*|\s*$/g, "");
					if (temporalName.length === 0) {
						continue;
					}
					let numberRows = self.m_grd_layouts.count_rows();

					let column = {
						Order           : numberRows + 1,
						Type            : columnDefinition.columnNumberType,
						Name            : columnName,
						groupName       : columnDefinition.translateGroup,
						ColumnDefinition: columnDefinition
					};

					let extraDataColumnName = columnDefinition.extraDataColumnName;
					if (extraDataColumnName !== undefined) {
						//18 = name value
						if (columnDefinition.columnNumberType === 18) {
							column.fieldName = $("#txt" + columnDefinition.editableFieldId).val();
							column.Name = $("#txt" + columnDefinition.editableColumnId).val();
							column[extraDataColumnName] = "";
						}
						else if (columnDefinition.columnNumberType === 10) {
							// 10 = a CONTACT_TOTAL_ATTEMPTS
							column[extraDataColumnName] = columnDefinition.extraData;
						} 
						else if (columnDefinition.columnNumberType === 61){
							// Skills
							column.fieldName = $("#txt" + columnDefinition.editableFieldId).val();
							column.Name = $("#txt" + columnDefinition.editableColumnId).val();
							column[extraDataColumnName] = "";
						}
						else {
							column[extraDataColumnName] = $("#txt" + columnDefinition.extraDataFieldId).val();
						}
					}
					// agrego la columna a la grilla
					self.m_grd_layouts.add_or_update_row(column);
				}
			}
		};

		this.show_waiting("<%=_T('Adding Layout Columns!!')%>");

		// pequeno timeout para tener feedback en el waiting
		window.setTimeout(function(){
			add_columns();
			// limpio info
			self.__clean_design_form_field();
		}, 100);
	};

	ExportLayoutManagement.prototype.set_events = function()
	{
		const self = this;

		if (this.owner.is_readonly()) {
			return;
		}

		this.m_grd_layouts.set_events();

		const $add = $("#cmdAddLayout");
		$add.off("click")
			.on("click", function () {
				self.__add_columns();
			});


		const $fields_container = $("#" + this.fields_container_id);
		let event_selector = "[group] > [divtitle='1']";
		// evento collapse group
		$fields_container
			.off("click", event_selector)
			.on("click", event_selector, function() {
				let group = $(this.parentNode).attr("group");
				self.collapse_group(group, this.id);
			});

		// evento link nuevo phone group
		event_selector = "[id='lnkNewPhoneGroup']";
		$fields_container
			.off("click", event_selector)
			.on("click", event_selector, function()
			{
				self.m_counterPhoneGroup++;
				self.m_columnTypesDefinitions = self.m_hnd_exp_column_types.BuildDynamicGroup(self.m_columnTypesDefinitions, "phone", self.m_counterPhoneGroup);
				self.m_hashDynamicPhoneGroup["PhoneExtraData" + self.m_counterPhoneGroup] = {groupId: "phone"};
				const phoneGroupInformation = {
					number          : {id: "PHONE_COUNTER"    },
					phoneExtraData  : {id: "PHONE_EXTRA_DATA" },
					phone           : {id: "PHONE"            },
					phoneCountry    : {id: "PHONE_COUNTRY"    },
					phoneExtension  : {id: "PHONE_EXTENSION"  },
					phoneArea       : {id: "PHONE_AREA"       },
					phoneDestination: {id: "PHONE_DESTINATION"},
					phoneNumber     : {id: "PHONE_NUMBER"     },
					phoneStatus     : {id: "PHONE_STATUS"     }
				};

				let html = $("#out_form_exp_layout_phone_template").html();
				for (let id in phoneGroupInformation) {
					if (phoneGroupInformation.hasOwnProperty(id)) {
						const info = phoneGroupInformation[id];
						const name_to_replace = "\{" + info.id + "\}";
						const regex = new RegExp(name_to_replace, "ig");
						const value = info.id + self.m_counterPhoneGroup;
						html = html.replace(regex, value);
					}
				}
				const $html = $(html);
				$("#contentPhoneGroup").append($html);
			});

		// evento link nuevo name value group
		event_selector = "[id='lnkNewNameValueGroup']";
		$fields_container
			.off("click", event_selector)
			.on("click", event_selector, function()
			{
				self.m_counterNameValueGroup = self.m_counterNameValueGroup + 2;
				self.m_columnTypesDefinitions = self.m_hnd_exp_column_types.BuildDynamicNameValueGroup(self.m_columnTypesDefinitions, "NAME_VALUE", self.m_counterNameValueGroup);
				const namveValueGroupInformation = {
					namevalue: {id: "NAME_VALUE", first: "NAME_VALUE_FIRST", second: "NAME_VALUE_SECOND"}
				};

				let html = $("#out_form_exp_layout_namevalue_template").html();
				for (let id in namveValueGroupInformation) {
					if (namveValueGroupInformation.hasOwnProperty(id)) {
						const info = namveValueGroupInformation[id];
						// 1 - Reemplazo el Atributo ID
						let name_to_replace = "\{" + info.id + "\}";
						let regex = new RegExp(name_to_replace, "ig");
						let value = info.id;
						html = html.replace(regex, value);
						// 2 - Reemplazo el Atributo FIRST
						name_to_replace = "\{" + info.first + "\}";
						regex = new RegExp(name_to_replace, "ig");
						value = (self.m_counterNameValueGroup - 1);
						html = html.replace(regex, value);
						// 3 - Reemplazo el Atributo SECOND
						name_to_replace = "\{" + info.second + "\}";
						regex = new RegExp(name_to_replace, "ig");
						value = self.m_counterNameValueGroup;
						html = html.replace(regex, value);
					}
				}
				const $html = $(html);
				$("#contentNameValueGroup").append($html);

				const $chk = $("#chk" + (namveValueGroupInformation.namevalue.id + (self.m_counterNameValueGroup - 1)));
				$chk.trigger("click");
			});

		// evento link nuevo skill group
		event_selector = "[id='lnkNewSkillGroup']";
		$fields_container
			.off("click", event_selector)
			.on("click", event_selector, function()
			{
				self.m_counterSkillGroup = self.m_counterSkillGroup + 2;
				self.m_columnTypesDefinitions = self.m_hnd_exp_column_types.BuildDynamicSkillGroup(self.m_columnTypesDefinitions, "SKILL", self.m_counterSkillGroup);
				const skillGroupInformation = {
					skill: {id: "SKILL", first: "SKILL_FIRST", second: "SKILL_SECOND"}
				};
				let html = $("#out_form_exp_layout_skill_template").html();
				for (let id in skillGroupInformation) {
					if (skillGroupInformation.hasOwnProperty(id)) {
						const info = skillGroupInformation[id];
						// 1 - Reemplazo el Atributo ID
						let name_to_replace = "\{" + info.id + "\}";
						let regex = new RegExp(name_to_replace, "ig");
						let value = info.id;
						html = html.replace(regex, value);
						// 2 - Reemplazo el Atributo FIRST
						name_to_replace = "\{" + info.first + "\}";
						regex = new RegExp(name_to_replace, "ig");
						value = (self.m_counterSkillGroup - 1);
						html = html.replace(regex, value);
						// 3 - Reemplazo el Atributo SECOND
						name_to_replace = "\{" + info.second + "\}";
						regex = new RegExp(name_to_replace, "ig");
						value = (self.m_counterSkillGroup);
						html = html.replace(regex, value);
					}
				}
				const $html = $(html);
				$("#contentSkillGroup").append($html);

				const $chk = $("#chk" + (skillGroupInformation.skill.id + (self.m_counterSkillGroup - 1)));
				$chk.trigger("click");
			});

		this.__set_dynamic_events();
	};

	// setea eventos dinamicos a los controles (estos se cargan cada vez que se crea un grupo nuevo en los custom data y phone)
	ExportLayoutManagement.prototype.__set_dynamic_events = function() 
	{
		const self = this;
		const $fields_container = $("#" + this.fields_container_id);

		let clickInCheckedAll = false;
		//----------------------------------------------------------------------------
		// evento click en los checkbox de cada columna
		//
		let event_selector = "[column_selector]";
		$fields_container
			.off("click", event_selector)
			.on("click", event_selector, function()
			{
				var columnId = $(this).attr("key");
				var editableControlId = "txt" + self.m_columnTypesDefinitions[columnId].editableFieldId;
				var disabled = true;
				var suggestedColumnName = "";
				if ($(this).prop("checked")) {
					self.m_selectedItems++;
					//obtiene el nombre sugerido para la columna y si la misma ya fue agregada al layout, se agrega un numero al final
					// hasta llegar al numero 100 como tope (mas es una guasada).
					var getSuggestedColumnName = function() {
						var suggestedColumnName = self.m_columnTypesDefinitions[columnId].suggestedColumnName;
						if (suggestedColumnName === undefined) {
							return "";
						}
						if (!self.__exist_column_in_layout(suggestedColumnName)) {
							return suggestedColumnName;
						}
						for (var i = 1; i < 101; i++) {
							if (!self.__exist_column_in_layout(suggestedColumnName + i)) {
								return suggestedColumnName + i;
							}
						}
						return suggestedColumnName;
					};
					disabled = false;
					var response = self.__truncate_column_name(getSuggestedColumnName());
					suggestedColumnName = response.columnName;
				}
				else {
					self.m_selectedItems--;
				}
				// habilita/inhabilita el boton cuando se presiona un checkbox
				var EnableDisableButton = function() {
					const disabled = self.m_selectedItems === 0;
					$("#cmdAddLayout").prop("disabled", disabled)[(!disabled ? "removeClass" : "addClass")]("formButtonAdd-ds");
				};
				if (self.m_selectedItems <= 1) {
					EnableDisableButton();
				}
				//textbox Column Name
				$("#" + editableControlId).prop("disabled", disabled)[(!disabled ? "removeClass" : "addClass")]("layoutFormField-ds");
				if (columnId.substr(0, 10) !== "NAME_VALUE" && columnId.substr(0, 5) !== "SKILL") {
				// para los tipos distintos a namevalue
					const labelControlId = "lbl" + self.m_columnTypesDefinitions[columnId].editableFieldId;
					//label
					$("#" + labelControlId)[(!disabled ? "removeClass" : "addClass")]("layoutFormLabel-ds");
					//textbox Column Name
					$("#" + editableControlId)
						.focus()
						.val(suggestedColumnName);
				}
				else {
					let editableColumnId;
					if (columnId.substr(0, 5) === "SKILL") {
						// para los namevalue /////////////////
						editableColumnId = "txt" + self.m_columnTypesDefinitions[columnId].editableColumnId;
						//textbox otra
						$("#" + editableColumnId).prop("disabled", disabled)[(!disabled ? "removeClass" : "addClass")]("layoutFormField-ds").val("");
						$("#" + editableControlId).focus().val("");
					}
					else {
						// para los namevalue /////////////////
						editableColumnId = "txt" + self.m_columnTypesDefinitions[columnId].editableColumnId;
						//textbox otra
						$("#" + editableColumnId).prop("disabled", disabled)[(!disabled ? "removeClass" : "addClass")]("layoutFormField-ds").val("");
						$("#" + editableControlId).focus().val("");
					}
				}
				// en este if entra solo si se ha presionado algun checkbox correspondiente a las columnas.
				//  si se presiona el checkbox correspondiente a los grupos, aqui no entra.
				if (!clickInCheckedAll) {
					// funcion que retorna si se han seleccionado todos los elementos de un grupo
					var IsAllSelectedItems = function(group) {
						var checkboxs = $("div[group='" + group + "'] > input.layoutFormCheckbox");
						var checked = 0;
						for (var key in checkboxs) {
							if (checkboxs.hasOwnProperty(key)) {
								var checkbox = checkboxs[key];
								if (typeof(checkbox)!== "object") {
									continue;
								}
								if (checkbox.checked) {
									checked++;
								}
							}
						}
						return checked === checkboxs.length;
					};
					$("#" + self.m_columnTypesDefinitions[columnId].checkboxSelectedGroupId).prop("checked", IsAllSelectedItems($(this.parentNode).attr("group")));
				}
			});
		//----------------------------------------------------------------------------
		// evento que habilita/deshabilita el textbox de extradata para el grupo phone
		event_selector = "[filter_selector]";
		$fields_container
			.off("click", event_selector)
			.on("click", event_selector, function()
			{
				var disabled = $(this).attr("value").toLowerCase() === "all" ? true : false;
				var id = $(this).attr("key");
				var methodName = !disabled ? "removeClass" : "addClass";
				//label
				$("#lbl" + id)[methodName]("layoutFormFilterLabel-ds");
				//textbox
				$("#txt" + id).prop("disabled", disabled)[methodName]("layoutFormField-ds").focus();
				if (disabled) {
					$("#txt" + id).val("");
				}
			});
		//----------------------------------------------------------------------------
		// evento de click en el checkbox de grupo
		event_selector = "[all_columns_selector]";
		$fields_container
			.off("click", event_selector)
			.on("click", event_selector, function()
			{
				clickInCheckedAll = true;
				var group = $(this).attr("group");
				var checked =  $(this).prop("checked");
				var checkboxs = $("div[group='" + group + "'] > input.layoutFormCheckbox");
				for (var key in checkboxs) {
					if (checkboxs.hasOwnProperty(key)) {
						var checkbox = checkboxs[key];
						if (typeof(checkbox)!== "object") {
							continue;
						}
						if (checked !== checkbox.checked) {
							checkbox.click();
						}
					}
				}
				clickInCheckedAll = false;
			});
		//----------------------------------------------------------------------------
		// evento de delete de grupos phone
		event_selector = "[columns_phone_group_delete]";
		$fields_container
			.off("click", event_selector)
			.on("click", event_selector, function()
			{
				var deleteList = [];
				var phoneGroupNumber = $(this).attr("number");
				for (var key in self.m_columnTypesDefinitions) {
					if (self.m_columnTypesDefinitions.hasOwnProperty(key)) {
						if (self.m_columnTypesDefinitions[key].group !== "phone") {
							continue;
						}
						var deleteColumnId = self.m_columnTypesDefinitions[key].columnNameType + phoneGroupNumber;
						if (deleteColumnId === key) {
							deleteList.push(deleteColumnId);
						}
					}
				}
				if (deleteList.length > 0) {
					for (var i = 0; i < deleteList.length; i++) {
						delete self.m_columnTypesDefinitions[deleteList[i]];
					}
				}
				$(this.parentNode.parentNode).html("");
				delete self.m_hashDynamicPhoneGroup["PhoneExtraData" + phoneGroupNumber];
			});
		//----------------------------------------------------------------------------
		// evento de delete de grupos phone
		event_selector = "[column_name_value]";
		$fields_container
			.off("blur", event_selector)
			.on("blur", event_selector, function()
			{
				let fieldValue = $(this).val();
				if (fieldValue.length === 0) {
					return;
				}
				var columnId = "#" + $(this).attr("columnid");
				if ($(columnId).val().length > 0) {
					return;
				}
				$(columnId).val(fieldValue);
			});
	};

	// esta funcion se fija que el largo del campo sugerido no exceda el largo del campo en la tabla de "ContactExportLayoutColumn.Name" en la BD
	ExportLayoutManagement.prototype.__truncate_column_name = function(columnName) {
		const columnLength = 100;
		const response = {truncated: false, columnName: columnName};
		if (columnLength >= columnName.length) {
			return response;
		}
		response.truncated = true;
		response.columnName = columnName.substr(0, columnLength);
		return response;
	};

	ExportLayoutManagement.prototype.__new_grids = function()
	{
		const self = this;

		this.m_grd_layouts = new inConcert.conf_form_paginator({
			paginator_name: "OUTBOUND_LAYOUTS",
			owner: this.owner,
			frm: this.owner,
			row_id: "Name",
			display_name: "Name",
			rows_per_page: 2500,
			show_at_least: 2499,
			paginator_type: "ALL_ROWS_CLIENT_SIDE",
			sortable: true,
			without_paginator: true,
			loading_waiting_msg: "<%=_T('Please wait, loading Layout Columns ...')%>",
			ids: {
				grid: "out_form_exp_layout_list",
				paginator: "out_form_exp_layout_paginator",
				not_results_found: "out_form_exp_layout_not_results_found",
				delete_all: "out_form_exp_layout_list_delete_all"
			},
			strings: {
				not_results_found: "<%=_T('No Layout Columns found.')%>",
				can_remove       : "<%=_T('Do you want to delete Layout Column: {0} ?')%>",
				can_remove_all   : "<%=_T('Do you want to delete All Layout Columns ?')%>"
			},
			columns: [
				{id: "_number"    , name: "", type: "counter"},
				{id: "GroupName", name: "<%=_T('Group')%>", get_value: function(item) {
					return self.get_grid_value(item, "GroupName");
				}},
				{id: "FieldName", name: "<%=_T('Field Name')%>", type: "cell_input",
					validation: [
						{
							required: true,
							message: "<%=_T('The field name can not be empty.')%>"
						}
					],
					get_value: function(item) {
						return self.get_grid_value(item, "FieldName");
					},
					get_column_name: function(item) {
						return self.get_grid_column_name(item, "FieldName");
					},
					is_disable_editable_cell: function(item) {
						const disabled = item.ColumnDefinition.readonlyFieldName;
						return disabled;
					}
				},
				{id: "Name", name: "<%=_T('Column Name')%>", type: "cell_input",
					validation: [
						{
							required: true,
							message: "<%=_T('The column name can not be empty.')%>"
						}
					],
					get_value: function(item) {
						return self.get_grid_value(item, "Name");
					},
					is_disable_editable_cell: function(/*item*/) {
						const disabled = false;
						return disabled;
					}
				},
				{id: "PhoneFilter", name: "<%=_T('Phone Filter')%>", type: "cell_input",
					validation: [
						{
							required: true,
							message: "<%=_T('The phone filter can not be empty.')%>"
						}
					],
					get_value: function(item) {
						return self.get_grid_value(item, "PhoneFilter");
					},
					get_column_name: function(item) {
						return self.get_grid_column_name(item, "PhoneFilter");
					},
					is_disable_editable_cell: function(item) {
						let disabled = true;
						const columnDefinition = item.ColumnDefinition;
						if (columnDefinition) {
							if (columnDefinition.group.toLowerCase() !== "phone") {
								disabled = columnDefinition.readonlyExtraData;
							}
							else {
								let extraData = "";
								if (columnDefinition.extraDataColumnName !== undefined) {
									extraData = item[columnDefinition.extraDataColumnName];
								}
								disabled = (extraData.length === 0) ? true : columnDefinition.readonlyExtraData;
							}
						}
						return disabled;
					}
				},
				{id: "_action", name: "",
					system_action: {erase: true}
				}
			],
			callbacks: {
				on_added_row: function(row, callback = $.noop) {
					self.__on_added_row(row, callback);
				},
				on_removed_row: function(row, callback = $.noop) {
					self.__on_removed_row(row, callback);
				},
				custom_validation_cell_value: function(info, callback) {
					self.__validate_editable_cell_value(info, callback);
				}
			},
			validate: {
				required_msg: "<%=_T('Must add at least one column.')%>",
				required: true
			}
		});
	};

	ExportLayoutManagement.prototype.__validate_editable_cell_value = function(info, callback)
	{
		if (info.column_id !== "Name") {
			callback(true);
			return;
		}

		let new_value = info.value;
		let old_value = info.old_value;

		if (new_value === old_value) {
			callback(false, "rollback");
			return;
		}

		if (this.__exist_column_in_layout(new_value)) {
			let message = FORMAT("<%=_T('Already exist column name: {0}.')%>", new_value);
			callback(false, "rollback", message);
			return;
		}

		callback(true);
	};

	ExportLayoutManagement.prototype.__on_added_row = function(column, callback = $.noop)
	{
		this.m_hashColumnLayoutByName[column.Name.toLowerCase()] = true;
		callback();
	};

	ExportLayoutManagement.prototype.__on_removed_row = function(column, callback = $.noop)
	{
		delete this.m_hashColumnLayoutByName[column.Name.toLowerCase()];
		callback();
	};

	ExportLayoutManagement.prototype.get_grid_value = function(item, column_id)
	{
		const value = {
			original_value: "",
			value         : "",
			search_value  : ""
		};

		let item_value = item[column_id];
		if (item_value === undefined) {
			item_value = "";
		}

		value.original_value = item_value;
		value.value = item_value;
		value.search_value = item_value;

		const item_name = this.m_hashColumnTypesById[item.Type];
		const columnTypeDefinition = this.m_columnTypesDefinitions[item_name];

		let extraData = "";
		if (columnTypeDefinition.extraDataColumnName !== undefined) {
			extraData = item[columnTypeDefinition.extraDataColumnName];
		}

		if (column_id === "GroupName") {
			value.original_value = item.groupName;
			value.value = item.groupName;
			value.search_value = item.groupName;
		}
		else if (column_id === "FieldName") {
			let fieldName = item.fieldName;
			if (fieldName === undefined) {
				let elementId;
				if (item.Type === 10 && extraData.length > 0) {
					elementId = this.m_hashBasicEspecialColumns[extraData].columnNameType;
				}
				else {
					elementId = item_name;
				}
				fieldName = $("#lbl" + elementId).html();
			}
			value.original_value = fieldName;
			value.value = fieldName;
			value.search_value = fieldName;
		}
		else if (column_id === "Name") {
			let columnName = item.Name;
			value.original_value = columnName;
			value.value = columnName;
			value.search_value = columnName;
		}
		else if (column_id === "PhoneFilter") {
			let phoneFilter = (item.Type === 10 ? "" : extraData);
			value.original_value = phoneFilter;
			value.value = phoneFilter;
			value.search_value = phoneFilter;
		}
		return value;
	};

	/*
		Esta funcion es invocada desde la clase "conf_form_paginator", en si los nombres de las columnas a guardar "FieldName" y "PhoneFilter", NO coinciden
		con el nombre a almacenar en la BD, entonces se debe obtener y retornar el nombre especifico, cuando vamos a guardar.
	*/
	ExportLayoutManagement.prototype.get_grid_column_name = function(item, column_id)
	{
		let column_name;
		if (column_id === "FieldName" || column_id === "PhoneFilter") {
			column_name = (item.ColumnDefinition && item.ColumnDefinition.extraDataColumnName) ? item.ColumnDefinition.extraDataColumnName : "fieldName";
		}
		return column_name;
	};

	ExportLayoutManagement.prototype.make_requests_list = function()
	{
		const self = this;

		const outbound_engine_api = inConcert.app.getOutboundEngineAPI();
		const requests = [
			// Columns Types
			{
				request_function: function(success) {
					return outbound_engine_api.systemLoadExportColumnsTypes(success);
				},
				success: function(list) {
					self.__fill_columns_types(list);
				}
			}
		];

		return requests;
	};

	ExportLayoutManagement.prototype.__fill_columns_types = function(columns_types = [])
	{
		// me quedo con la lista de colums types en dos hash (uno por id y otro por nombre)
		for (let i = 0; i < columns_types.length; i++) {
			const column_type = columns_types[i];
			const name = column_type.Name;
			const value = parseInt(column_type.Value, 10);
			this.m_hashColumnTypesById[value] = name;
			this.m_hashColumnTypesByName[name.toLowerCase()] = value;
		}
	};

	ExportLayoutManagement.prototype.get_validation_def = function()
	{
		const self = this;

		let definitions = [
			{
				control_id: "txtId",
				messages: {
					required: "<%=_T('Please enter layout id')%>",
					id      : "<%=_T('Only letters, numbers, dashes and dots are allowed in the layout name.')%>"
				}
			},
			{
				control_id: "txtDescription",
				messages: {
					required: "<%=_T('Please enter the layout description')%>"
				}
			},
			{
				control_id: "out_form_exp_layout_list_container",
				messages: {
					custom: {
						type : "custom",
						valid: function() {
							let response = {
								success: true
							};
							let success = self.m_grd_layouts.start_validation().success;
							if (!success) {
								response.success = false;
								response.message = self.m_grd_layouts.get_validation_message_err("required");
							}
							return response;
						}
					}
				}
			}
		];
		return definitions;
	};
}());
