/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 9 */

/* global inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";
	const FORMAT = inConcert.Utils.format;

	//---------------------------------------------------------------------
	// form exports class
	function form_outbound_engine_exports(params)
	{
		const success = form_outbound_engine_exports.uber.constructor.call(this, params);
		if (!success) {
			return;
		}

		this.view_title = "<%=_T('View Exports {0}')%>";
		this.tagline = "<%=_T('Tagline form Export!!!')%>";
		this.new_title = "<%=_T('New Export')%>";

		this.html_file_name = "/inconcert/apps/outbound_engine/exports/frm/render/";
		this.get_uri =        "/inconcert/apps/outbound_engine/exports/get/";

		this.m_hnd_exp_column_types = new inConcert.outbound_engine_exportation_column_types_def();
		this.m_cached_batches_by_campaigns = {};
		this.m_hash_column_types_by_id = {};
		this.include_save_and_continue = false;
		this.export_started_timestamp = null;

		return true;
	}

	inConcert.Utils.inherit(form_outbound_engine_exports, inConcert.conf_form);

	if (typeof inConcert.form_outbound_engine_exports !== "function") {
		inConcert.form_outbound_engine_exports = form_outbound_engine_exports;
	}

	form_outbound_engine_exports.prototype.initialize = function()
	{
		form_outbound_engine_exports.uber.initialize.call(this); //invokes the base class method
		this.create_datepickers("out_form_exportation_data_sources_container", true);

		const column_types_def = this.m_hnd_exp_column_types.GetColumnsTypesDefinitions();
		for (let column_name in column_types_def) {
			if (column_types_def.hasOwnProperty(column_name)) {
				const column_def = column_types_def[column_name];
				this.m_hash_column_types_by_id[column_def.columnNumberType] = column_def;
			}
		}

		$("#out_form_exportation_id").focus();
	};

	form_outbound_engine_exports.prototype.get = function()
	{
		const campaign    = this.combo_get_selected_item_info($("#out_form_exportation_data_sources_campaign"), "campaign");
		const importation = this.combo_get_selected_item_info($("#out_form_exportation_data_sources_import"), "importation");
		const timezone    = this.combo_get_selected_item_info($("#out_form_exportation_data_sources_timezone"), "timezone");
		const layout      = this.combo_get_selected_item_info($("#out_form_exportation_layout_list"), "layout");
		let batch_id = "";
		let batch_name = "";
		if (campaign.id.length > 0) {
			const batch = this.combo_get_selected_item_info($("#out_form_exportation_data_sources_batch"));
			batch_id = batch.id;
			batch_name = batch.name;
		}
		const $start_date = $("#out_form_exportation_data_sources_start_date");
		const $end_date = $("#out_form_exportation_data_sources_end_date");
		let start_date = "";
		let end_date = "";
		if (!$start_date.prop("disabled")) {
			if ($start_date.val().length > 0) {
				start_date = $start_date.datepicker("getDate");
				start_date = inConcert.Utils.getStringFromDate(start_date, false, "date", "ymd", "-");
			}
			if ($end_date.val().length > 0) {
				end_date = $end_date.datepicker("getDate");
				end_date = inConcert.Utils.getStringFromDate(end_date, false, "date", "ymd", "-");
			}
		}

		let exportation = {
			id                    : $("#out_form_exportation_id").val(),
			layoutId              : layout.id,
			timezone              : timezone.data,
			filter: {
				campaignId        : campaign.id,
				batchId           : batch_id,
				batchName         : batch_name,
				importationId     : importation.id,
				importationName   : importation.name,
				startDate         : start_date,
				endDate           : end_date,
				timeZoneId        : timezone.data.Id,
				timeZoneName      : timezone.data.FormattedTimeZone
			},
			detail: {
				contacted         : $("#out_form_exportation_layout_attempts_info_contacted").prop("checked"),
				abandoned         : $("#out_form_exportation_layout_attempts_info_abandoned").prop("checked"),
				failed            : $("#out_form_exportation_layout_attempts_info_failed").prop("checked"),
				lastContacted     : $("#out_form_exportation_layout_attempts_info_last_contacted").prop("checked"),
				lastContactedBatch: $("#out_form_exportation_layout_attempts_info_last_contacted_batch").prop("checked"),
				oneRowContactData : $("#out_form_exportation_layout_repeat_contact_data").prop("checked")
			}
		};
		return exportation;
	};

	form_outbound_engine_exports.prototype.__fill_timezones = function(list)
	{
		const $combo = $("#out_form_exportation_data_sources_timezone");
		this.fill_timezones(list, $combo, true);
	};

	form_outbound_engine_exports.prototype.__fill_importations = function(list)
	{
		const $combo = $("#out_form_exportation_data_sources_import");
		this.fill_combo($combo, true /* is_blank_first_item */, list, "id" /* attr_id */, "id" /* attr_name */, true /* sorted */, undefined /*can_add_item*/, "importation", /* data_id */);
	};

	form_outbound_engine_exports.prototype.__fill_exports_layouts = function(list)
	{
		const $combo = $("#out_form_exportation_layout_list");
	//	this.fill_combo($combo, true /* is_blank_first_item */, list, "Id" /* attr_id */, "Description" /* attr_name */, true /* sorted */, undefined /*can_add_item*/,  "layout", /* data_id */);
		const self = this;
		$combo.html("");
		this.combo_add_value($combo, "", "", true);
		list.forEach(layout => {
			self.combo_add_value($combo, layout.Id, layout.Description, false, "layout", layout);
		});
	};

	form_outbound_engine_exports.prototype.__load_batches = function(campaign_id)
	{
		const self = this;
		const $combo = $("#out_form_exportation_data_sources_batch");

		var fill = function(list) {
			self.fill_combo($combo, true /* is_blank_first_item */, list, "batchid" /* attr_id */, "batchid" /* attr_name */, true /* sorted */, undefined /*can_add_item*/,  "batch", /* data_id */);
		};

		if (this.m_cached_batches_by_campaigns[campaign_id]) {
			fill(this.m_cached_batches_by_campaigns[campaign_id]);
			return;
		}

		const outbound_engine_api = inConcert.app.getOutboundEngineAPI();
		outbound_engine_api.systemLoadBatchesByCampaign(campaign_id,
			function(success, list) {
				if (!success) {
					const message = "<%=_T('Error retrieving batches.')%>";
					const timeout_ms = 5000;
					self.show_status_message(message, "Error", timeout_ms);
					return;
				}
				self.m_cached_batches_by_campaigns[campaign_id] = list;
				fill(list);
			}
		);
	};

	form_outbound_engine_exports.prototype.load_data = function(callback_complete_requests)
	{
		const self = this;
		const outbound_engine_api = inConcert.app.getOutboundEngineAPI();

		const $campaign = $("#out_form_exportation_data_sources_campaign");
		this.fill_campaigns($campaign, true);

		this.requests = [
			// Importations
			{
				request_function: function(success) {
					return outbound_engine_api.systemLoadImportations(success);
				},
				success: function(list) {
					self.__fill_importations(list);
				}
			},
			// TimeZones
			{
				request_function: function(success) {
					return outbound_engine_api.systemLoadTimeZones(success);
				},
				success: function(list) {
					self.__fill_timezones(list);
				}
			},
			// Export Layouts
			{
				request_function: function(success) {
					return outbound_engine_api.exportLayoutsLoadAll(success);
				},
				success: function(list) {
					self.__fill_exports_layouts(list);
				}
			}
		];

		form_outbound_engine_exports.uber.load_data.call(this, callback_complete_requests); //invokes the base class method
	};

	form_outbound_engine_exports.prototype.set_events = function()
	{
		const self = this;

		const $campaign = $("#out_form_exportation_data_sources_campaign");
		$campaign
			.off("change")
			.on("change", function() {
				const campaign_id = $(this).val();
				self.__on_changed_campaign(campaign_id);
			});

		const $layout = $("#out_form_exportation_layout_list");
		$layout
			.off("change")
			.on("change", function() {
				const layout = self.combo_get_selected_item_info($layout, "layout");
				self.__on_changed_layout(layout.data);
			});

		const $repeat_contact_data = $("#out_form_exportation_layout_repeat_contact_data");
		$repeat_contact_data
			.off("change")
			.on("change", function() {
				const has_repeat = $(this).prop("checked");
				const $repeat_contact_data_info = $("#out_form_exportation_layout_repeat_contact_data_info");
				$repeat_contact_data_info[has_repeat ? "show" : "hide"]();
			});
	};

	form_outbound_engine_exports.prototype.__on_changed_campaign = function(campaign_id)
	{
		this.__load_batches(campaign_id);
		const $batch = $("#out_form_exportation_data_sources_batch");
		const disable = campaign_id.length === 0;
		this.disable_control($batch, disable, false);
	};

	form_outbound_engine_exports.prototype.__disable_filter_dates = function(disabled)
	{
		const $start_date = $("#out_form_exportation_data_sources_start_date");
		const $end_date = $("#out_form_exportation_data_sources_end_date");
		this.disable_control($start_date, disabled, false);
		this.disable_control($end_date, disabled, false);
		$start_date.datepicker("setDate", "");
		$end_date.datepicker("setDate", "");
	};

	form_outbound_engine_exports.prototype.__on_changed_layout = function(layout)
	{
		let showAttemptsInformation = false;
		let htmlPreview = "";
		let disableEnableDates = true;

		if (layout !== undefined) {
			var columnsLength = layout.ContactExportLayoutColumns.length;
			if (layout.hashSortColumns === undefined) {
				for (let i = 0; i < columnsLength; i++) {
					layout.hashSortColumns = layout.hashSortColumns || {};
					layout.hashSortColumns[layout.ContactExportLayoutColumns[i].Order] = layout.ContactExportLayoutColumns[i];
				}
			}
			for (let i = 0; i < columnsLength; i++) {
				var index = i + 1;
				var columnDefinition = layout.hashSortColumns[index];
				htmlPreview += "<div class='layoutPreviewColumn'>" + columnDefinition.Name + "</div>";
				if (!showAttemptsInformation && this.m_hash_column_types_by_id[columnDefinition.Type].group.toLowerCase() === "contact") {
					showAttemptsInformation = true;
				}
				if (showAttemptsInformation || this.m_hash_column_types_by_id[columnDefinition.Type].group.toLowerCase() === ("subgroup_basic_attempts") ) {
					disableEnableDates = false;
				}
			}
		}

		this.__disable_filter_dates(disableEnableDates);

		$("#out_form_exportation_layout_preview").html(htmlPreview);
		$("#out_form_exportation_layout_attempts_info")[showAttemptsInformation ? "show" : "hide"]();
	};

	form_outbound_engine_exports.prototype.get_validation_def = function()
	{
		let definitions = [
			{
				control_id: "out_form_exportation_id",
				messages: {
					required : "<%=_T('Please enter the Exportation Name.')%>",
					id       : "<%=_T('Only letters, numbers, dashes and dots are allowed in the Exportation Name.')%>"
				}
			},

			{
				control_id: "out_form_exportation_data_sources_timezone",
				messages: {
					required : "<%=_T('Please select the timezone.')%>"
				}
			},

			{
				control_id: "out_form_exportation_layout_list",
				messages: {
					required : "<%=_T('Please select the layout.')%>"
				}
			},
		];
		return definitions;
	};

	form_outbound_engine_exports.prototype.make_save_waiting_msg = function(entity)
	{
		entity = entity || this.get();
		return {
			waiting_message: FORMAT("<%=_T('Please wait, Starting Exportation: {0}')%>", entity.id)
		};
	};

	form_outbound_engine_exports.prototype.save_execute = function(after_save, exportation /*, hash_id */)
	{
		const self = this;

		after_save = "SAVE_AND_REFRESH_VIEW";

		let timeout_hnd = null;
		var clear_timeout = function () {
			if (timeout_hnd) {
				window.clearTimeout(timeout_hnd);
				timeout_hnd = null;
			}
		};

		// Funcion que regresa a la vista cuando start de la exportacion se encuentra en progreso o ha finalizado.
		var go_back_grid = function(msg, msg_type = "Info") {
			clear_timeout();
			self.export_started_timestamp = new Date();
			self.hide_waiting();
			self.show_status_message(msg, msg_type, 5000);
			let not_show_grid_message = true;
			self.on_saved(true, exportation, after_save, null, null, not_show_grid_message);
		};

		let timemout_ms = 5000;
		var must_shown_export_err = function() {
			// Para mostrar el error valida que el tiempo transcurrido desde que comenzo la exportacion, haya superado el tiempo del timeout.
			// Es decir que haya comenzado de forma asincronica y que haya regresado a la vista, a partir de ese momento el error ya no deberia visualizarse.
			if (!self.export_started_timestamp) {
				return true;
			}
			let now = new Date();
			let elapsed_time_ms = now.getTime() - self.export_started_timestamp.getTime();
			return timemout_ms > elapsed_time_ms;
		};

		var export_start = function()
		{
			clear_timeout();
			// Si el no ha dado error, regreso a la vista.
			//  El regreso se realiza en un timeout, ya que la exportacion comienza a correr en el server de 5.5 y puedo durar varios minutos.
			//  Y suponemos que si pasaron N segundos la misma ha comenzado correctamente.
			timeout_hnd = window.setTimeout(function() {
				let msg = "<%=_T('Successfully Started Exportation.')%>";
				go_back_grid(msg);
			}, timemout_ms);

			const outbound_engine_api = inConcert.app.getOutboundEngineAPI();
			outbound_engine_api.exportStart(exportation, function(success, response)
			{
				let msg;
				if (success) {
					msg = "<%=_T('Successfully Finished Exportation.')%>";
					go_back_grid(msg);
					return;
				}
				clear_timeout();
				const err = response;
				self.on_saved(false, exportation, after_save, response);
			});
		};

		// Comienza la exportacion
		export_start();
	};

	form_outbound_engine_exports.prototype.__update_buttons = function()
	{
		var $save = this.$get_dom_element_by_id("form_button_save");
		var $save_and_continue = this.$get_dom_element_by_id("form_button_save_and_continue");
		$save.attr("value", "<%=_T('Start')%>");
		$save_and_continue.hide();
	};

	form_outbound_engine_exports.prototype.ready_for_update = function()
	{
		form_outbound_engine_exports.uber.ready_for_update.call( this ); //invokes the base class method
		this.__update_buttons();
	};

	form_outbound_engine_exports.prototype.ready_for_save = function()
	{
		form_outbound_engine_exports.uber.ready_for_save.call( this ); //invokes the base class method
		this.__update_buttons();
	};

	form_outbound_engine_exports.prototype.err_save_messages = function(err)
	{
		let is_custom_message = false;
		if (typeof err === "object") {
			is_custom_message = true;
			let error_type = err.err ? err.err.result : "UNRECOGNIZED_ERR";
			switch(error_type) {
				case -1:
					//exportation failed/not started
					err = "<%=_T('Exportation failed to start, please contact system administrator.')%>";
					break;

				case -100:
					//No existe layout en la bd asociado a la exportacion.
					err = "<%=_T('Not exist layout associated with the exportation.')%>";
					break;

				case -200:
					//El layout no cotiene columnas asociadas.
					err = "<%=_T('The layout does not contain columns associated.')%>";
					break;

				case 3:
					//aborted
					err = "<%=_T('The exportation has been aborted.')%>";
					break;

				case "ALREADY_EXIST":
					err = "<%=_T('Exportation is already in use.')%>";
					const $exp_id = $("#out_form_exportation_id");
					this.show_validation_error($exp_id);
					break;

				default:
					err = "<%=_T('Error Starting Exportation.')%>";
			}
		}
		form_outbound_engine_exports.uber.err_save_messages.call(this, err, is_custom_message); //invokes the base class method
	};
}());
