/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 11 */

/* global inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	const FORMAT = inConcert.Utils.format;

	/**
	Class SaveFilterPopup.
	*/
	function SaveFilterPopup (params)
	{
		SaveFilterPopup.uber.constructor.call( this );
		this.clickEventName = this.clickEventName + "_batch_filter";
		this.ownerId = "out_batch_filter_popup_container";
		this.filterId = this.ownerId + "_filter_id";
		this.saveId = this.ownerId + "_save";
		this.closeId = this.ownerId + "_close";
		this.params = params;
		this.canReposition = true;
	}

	inConcert.Utils.inherit(SaveFilterPopup, inConcert.PopupWindow);

	/*******************************************************************
	 * Metodo que crea el markup contenedor de la ventana
	 */
	SaveFilterPopup.prototype.createMarkup = function()
	{
		$("#" + this.ownerId).remove();

		//create the markup everytime
		var $node = $("".concat(
			"<div id='" + this.ownerId + "' class='systemPopWindow windowContainer' style='display:none'>",
				"<div class='windowTop'>",
					"<div class='windowTitle'>", "<%=_T('Save Filter')%>", "</div>",
				"</div>",
				"<div class='windowMiddle' style='display:none'>",
					"<div class=''></div>",
				"</div>",
				"<div class='windowMiddle'>",
					"<div class='card-block'>",
						"<div class='row'>",
							"<div class='col-12'>",
								"<div class='form-group'>",
									"<label for='", this.filterId, "' class=''>", "<%=_T('Filter Name')%>", "<%=_T(':')%>", "</label>",
									"<input id='", this.filterId, "' class='form-control' type='text' />",
								"</div>",
							"</div>",
						"</div>",
					"</div>",
				"</div>",
				"<div class='windowBottom d-flex justify-content-end'>",
					"<button id='", this.closeId, "' class='pageButtonGeneric buttonColorWhite mr-3'>",
						"<%=_T('Close')%>",
					"</button>",
					"<button id='", this.saveId, "' class='pageButtonGeneric buttonColorBlack'>",
						"<%=_T('Save')%>",
					"</button>",
				"</div>",
			"</div>"));
		$("body").append($node);

		return $node;
	};

	/*******************************************************************
	 *
	 */
	SaveFilterPopup.prototype.initialize = function()
	{
		this.data.container = this.createMarkup();
		this.parent_save = this.data.callbacks.save || $.noop;
		this.setEvents();
	};

	SaveFilterPopup.prototype.ready = function()
	{
		var current_filter = this.params.current_filter;
		$("#" + this.filterId).val(current_filter).focus();
	};

	SaveFilterPopup.prototype.close = function(params) {
		this.data.container.remove();
		this.finalize(params);
	};

	/*******************************************************************
	 * Metodo que setea eventos de los controles
	 */
	SaveFilterPopup.prototype.setEvents = function()
	{
		const self = this;

		var $filter_id =  $("#" + this.filterId);
		//---------------
		$("#" + this.saveId)
			.unbind("click")
			.click(function() {
				var filter_id = $filter_id.val();
				self.parent_save(filter_id, function(success) {
					if (!success) {
						$filter_id.focus();
						return;
					}
					self.close();
				});
			});
		//---------------
		$("#" + this.closeId).unbind("click").click(function() {
			self.close();
		});
	};

	//---------------------------------------------------------------------
	// form process batches class
	function form_outbound_engine_processes_batches(params)
	{
		this.parent = params.parent;

		this.is_new_filter_condition = null;
		this.current_filter_condition = null;

		this.cached_fields_by_desc = null;
		this.cached_fields_by_field = null;
		this.contact_query_result = null;

		// El paginado se realiza un único request de rows por paginas, ya que el paginado se realiza client side.
		this.server_side_contacts_rows_per_page = 200;
		this.form_paginator_contacts = null;
		this.form_paginator_filters = null;
		this.__initialize();

		this.operators = [
			{id: "="   , name: "<%=_T('equal')%>"           },
			{id: "<>"  , name: "<%=_T('distinct')%>"        },
			{id: "<"   , name: "<%=_T('less')%>"            },
			{id: ">"   , name: "<%=_T('greater')%>"         },
			{id: "<="  , name: "<%=_T('less or equal')%>"   },
			{id: ">="  , name: "<%=_T('greater or equal')%>"},
			{id: "Like", name: "<%=_T('contains')%>"        }
		];

		this.logical_operators = [
			{id: "OR" , name: "<%=_T('OR')%>" },
			{id: "AND", name: "<%=_T('AND')%>"}
		];

		this.left_parenths  = [{id: "("}, {id: "(("}, {id: "((("}, {id: "(((("}];
		this.right_parenths = [{id: ")"}, {id: "))"}, {id: ")))"}, {id: "))))"}];

		this.filter_popup = null;
	}

	form_outbound_engine_processes_batches.prototype.__initialize = function()
	{
		this.__new_grids();
	};

	form_outbound_engine_processes_batches.prototype.dispose = function()
	{
		this.contact_query_result = null;
	};

	form_outbound_engine_processes_batches.prototype.initialize_ctrls = function()
	{
		$("#out_form_process_batches_data_source").val("").trigger("change");

		var dom_container_id = "out_form_process_batches_section";
		this.parent.create_datepickers(dom_container_id);

		var $filters = $("#out_form_process_batches_filters");
		$filters.val("");
		this.__change_filter();
		this.form_paginator_contacts.initialize();

		this.__initial_ctrls_values();
	};

	form_outbound_engine_processes_batches.prototype.__new_grids = function()
	{
		const self = this;

		this.form_paginator_filters = new inConcert.conf_form_paginator({
			paginator_name: "OUTBOUND_BATCHES_FILTERS",
			owner: this,
			frm: this.parent,
			display_name: "field",
			rows_per_page: 500,
			show_at_least: 499,
			without_paginator: true,
			loading_waiting_msg: "<%=_T('Please wait, loading Filters ...')%>",
			ids: {
				grid                     : "out_form_process_batches_filters_list",
				paginator                : "out_form_process_batches_filters_paginator",
				not_results_found        : "out_form_process_batches_filters_not_results_found",
				new_item                 : "out_form_process_batches_filters_new",
				container_new_btn        : "out_form_process_batches_filters_container",
				//popup
				popup                    : "out_form_process_batches_filters_popup",
				popup_msg                : "out_form_process_batches_filters_popup_message",
				popup_create             : "out_form_process_batches_filters_popup_create",
				popup_create_and_continue: "out_form_process_batches_filters_popup_create_and_continue",
				popup_title              : "out_form_process_batches_filters_popup_title",
				popup_close              : "out_form_process_batches_filters_popup_close"
			},
			strings: {
				not_results_found         : "<%=_T('No Filters found.')%>",
				can_remove                : "<%=_T('Do you want to delete Filter Condition: {0} ?')%>",
				title_new                 : "<%=_T('New Condition')%>",
				title_update              : "<%=_T('Update Condition')%>",
				button_create             : "<%=_T('Confirm')%>",
				button_create_and_continue: "<%=_T('Confirm & continue')%>",
				button_update             : "<%=_T('Update')%>",
				button_close			  : "<%=_T('Cancel')%>"
			},
			columns: [
				{id: "_number"     , name: "", type: "counter"   },
				{id: "logicalOp"   , name: "<%=_T('Operator')%>" },
				{id: "leftParenth" , name: "<%=_T('(')%>"        },
				{id: "field"       , name: "<%=_T('Field')%>"    , get_value: function(item) {
					var field = self.cached_fields_by_field[item.field];
					if (field) {
						return self.get_grid_filter_value(field, "Id");
					}
					return self.get_grid_filter_value(item, "field");
				}},
				{id: "conditionOp" , name: "<%=_T('Condition')%>", get_value: function(item) {
					return self.get_grid_filter_value(item, "Condition");
				}},
				{id: "value"       , name: "<%=_T('Value')%>"    },
				{id: "rightParenth", name: "<%=_T(')')%>"        },
				{id: "_action", name: "",
					system_action: { edit: true, erase: true }
				}
			],
			callbacks: {
				save: function(callback, must_create_and_contiune) {
					self.add_or_update_filter_condition(callback, must_create_and_contiune);
				},
				edit: function(condition) {
					self.on_edit_filter_condition(condition);
				},
				on_removed_row: function() {
					self.__disabled_filter_actions(false);
				},
				on_closed_popup: function() {
					self.on_closed_filter_condition_popup();
				},
				load_items: function(page, callback) {
					//console.log("1 - grid load_items -> form_paginator_filters");
					self.parent.count_pending_grid = self.parent.count_pending_grid + 1;
					self.load_batches_filters(page, callback);
				},
				on_loaded_items: function(callback) {
					self.parent.__on_loaded_grid(callback);
				}
			}
		});

		// La cantidad de rows que se deben de visualizar en el paginador.
		// El paginado se realiza client side, ya que server side se obtienen una unica pagina con N "server_side_contacts_rows_per_page" registros.
		var client_side_rows_per_page = 10;

		this.form_paginator_contacts = new inConcert.conf_form_paginator({
			paginator_name: "OUTBOUND_BATCHES_CONTACTS",
			owner: this,
			frm: this.parent,
			row_id: "Name",
			display_name: "Name",
			rows_per_page: client_side_rows_per_page,
			show_at_least: client_side_rows_per_page - 1,
			paged_text_format: "row_count",
			loading_waiting_msg: "<%=_T('Please wait, loading Contacts ...')%>",
			ids: {
				grid: "out_form_process_batches_contacts_list",
				paginator: "out_form_process_batches_contacts_paginator",
				not_results_found: "out_form_process_batches_contacts_not_results_found"
			},
			strings: {
				not_results_found: "<%=_T('No Contacts found.')%>"
			},
			columns: [
				{id: "_number"   , name: "", type: "counter"     },
				{id: "ID"        , name: "<%=_T('Identifier')%>" },
				{id: "Name"      , name: "<%=_T('Name')%>"       },
				{id: "Campaign"  , name: "<%=_T('Campaign')%>"   },
				{id: "Provider"  , name: "<%=_T('Provider')%>"   },
				{id: "LastUpdate", name: "<%=_T('Last Import')%>"}
			],
			callbacks: {
				load_items: function(page, callback) {
					//console.log("2 - grid load_items -> form_paginator_contacts");
					self.parent.count_pending_grid = self.parent.count_pending_grid + 1;
					self.load_batches_contacts(page, callback);
				},
				on_loaded_items: function(callback) {
					self.parent.__on_loaded_grid(callback);
				}
			}
		});
	};

	form_outbound_engine_processes_batches.prototype.get_grid_filter_value = function(item, field_id)
	{
		var value = {
			original_value: "",
			value         : "",
			search_value  : ""
		};

		var item_value = item[field_id] || "";
		if (item_value === undefined) {
			return value;
		}

		value.original_value = item_value;
		value.value = item_value;
		value.search_value = item_value;

		if (field_id === "Condition") {
			value.value = this.__operator_as_string(item.conditionOp);
			value.search_value = value.value;
		}

		return value;
	};

	form_outbound_engine_processes_batches.prototype.__operator_as_string = function(condition)
	{
		var condition_string;
		$.each(this.operators, function(_, operator) {
			if (operator.id === condition) {
				condition_string = operator.name;
				return false; //break
			}
		});
		return condition_string || condition;
	};

	form_outbound_engine_processes_batches.prototype.on_closed_filter_condition_popup = function()
	{
		this.is_new_filter_condition = null;
		this.current_filter_condition = null;
		this.__disabled_filter_actions(false);
	};

	form_outbound_engine_processes_batches.prototype.load_batches_filters = function(_, callback)
	{
		callback = callback || $.noop;

		var $filters = $("#out_form_process_batches_filters");
		var filter = this.parent.combo_get_selected_item_data($filters);

		var query = [];
		if (filter) {
			//cargo el filtro seleccionado
			var filter_lines = JSON.parse(filter.FilterSentence);
			query = filter_lines.Query || [];
		}

		callback(true, query);
	};

	form_outbound_engine_processes_batches.prototype.load_batches_contacts = function(_, callback)
	{
		callback = callback || $.noop;
		this.__load_contacts_from_query(function(success, contacts) {
			callback(success, contacts);
		});
	};

	form_outbound_engine_processes_batches.prototype.load_information = function()
	{
		const self = this;

		this.__fill_sources();

		var outbound_engine_api = inConcert.app.getOutboundEngineAPI();

		outbound_engine_api.processLoadBatchFilters(function(success, list) {
			if (!success) {
				self.parent.show_status_message("<%=_T('Error Retrieving Filters Batch')%>", "Error", 5000);
				return;
			}
			self.__fill_filters(list);
			self.initialize_ctrls();
		});
	};

	form_outbound_engine_processes_batches.prototype.load_update_information = function()
	{
		const self = this;

		var outbound_engine_api = inConcert.app.getOutboundEngineAPI();
		var process = this.get_entity_instance();

		outbound_engine_api.processLoadBatchFields(process.Id, function(success, list)
		{
			if (!success) {
				self.parent.show_status_message("<%=_T('Error Retrieving Fields Batch')%>", "Error", 5000);
				return;
			}
			self.cached_fields_by_desc = {};
			self.cached_fields_by_field = {};

			$.each(list, function(_, field)
			{
				var full_field_name = (field.Table ? field.Table + "." : "") + field.Field;
				var field_id = field.Name;

				var cached_field = {
					id    : field_id,
					name  : field.Name,
					field : full_field_name,
					values: field.Values
				};

				self.cached_fields_by_desc[field_id] = cached_field;
				self.cached_fields_by_field[full_field_name] = cached_field;
			});

			self.__fill_filters_fields();
		});

		outbound_engine_api.systemLoadCampaignMembers(process.DisplayCampaign, function(success, list) {
			if (!success) {
				self.parent.show_status_message("<%=_T('Error Retrieving Members of Campaign')%>", "Error", 5000);
				return;
			}
			self.__fill_campaign_members(list);
		});
	};

	form_outbound_engine_processes_batches.prototype.get_entity_instance = function()
	{
		return this.parent.get_entity_instance() || {};
	};

	form_outbound_engine_processes_batches.prototype.__fill_sources = function()
	{
		var $combo = $("#out_form_process_batches_data_source");
		var list = [
			{id: "contact_query", name: "<%=_T('Contacts Database')%>"      },
			{id: "batch_query"  , name: "<%=_T('Current Process Batches')%>"}
		];
		this.parent.fill_combo($combo, true /* is_blank_first_item */, list, "id" /* attr_id */, "name" /* attr_name */);
	};

	form_outbound_engine_processes_batches.prototype.__fill_filters = function(list)
	{
		var $combo = $("#out_form_process_batches_filters");
		this.parent.fill_combo($combo, true /* is_blank_first_item */, list, "Id" /* attr_id */, "Id" /* attr_name */);
	};

	form_outbound_engine_processes_batches.prototype.__fill_actions = function(source)
	{
		var list = [];

		if (source === "contact_query") {
			list.push({id: "createbatch", name: "<%=_T('Create New Batch')%>"});
		}
		else if (source === "batch_query") {
			list.push({id: "cancelcontacts"         , name: "<%=_T('Cancel Contacts')%>"        });
			list.push({id: "pausecontactsquery"     , name: "<%=_T('Pause Contacts')%>"         });
			list.push({id: "resumecontactsquery"    , name: "<%=_T('Resume Contacts')%>"        });
			list.push({id: "activatecontacts"       , name: "<%=_T('Recycle Contacts')%>"       });
			list.push({id: "changepriority"         , name: "<%=_T('Change Priority')%>"        });
			list.push({id: "changeisprogressivetype", name: "<%=_T('Change Progressive Type')%>"});
		}
		var $combo = $("#out_form_process_batches_actions");
		this.parent.fill_combo($combo, true /* is_blank_first_item */, list, "id" /* attr_id */, "name" /* attr_name */);
	};

	form_outbound_engine_processes_batches.prototype.__fill_filters_fields = function()
	{
		const self = this;

		var $logical_operator = $("#out_form_process_batches_filters_field_logical_operator");
		this.parent.fill_combo($logical_operator, true /* is_blank_first_item */, this.logical_operators, "id" /* attr_id */, "name" /* attr_name */);

		var $operator = $("#out_form_process_batches_filters_field_operator");
		this.parent.fill_combo($operator, true /* is_blank_first_item */, this.operators, "id" /* attr_id */, "name" /* attr_name */);

		var $left_parentheses = $("#out_form_process_batches_filters_field_left_parentheses");
		this.parent.fill_combo($left_parentheses, true /* is_blank_first_item */, this.left_parenths, "id" /* attr_id */, "id" /* attr_name */);

		var $rigth_parentheses = $("#out_form_process_batches_filters_field_rigth_parentheses");
		this.parent.fill_combo($rigth_parentheses, true /* is_blank_first_item */, this.right_parenths, "id" /* attr_id */, "id" /* attr_name */);

		// combo autocomplete
		var $field_name = $("#out_form_process_batches_filters_field_name");
		var arr_fields = inConcert.Utils.hash_to_sorted_array(this.cached_fields_by_desc, null, null, {only_key: true});
		this.parent.make_autocomplete(
			$field_name,
			arr_fields,
			function(/*_, value*/) {
				self.__change_filter_field_name();
			}
		);
	};

	form_outbound_engine_processes_batches.prototype.__fill_campaign_members = function(list)
	{
		var $combo = $("#out_form_process_batches_action_activate_assign_agent");
		this.parent.fill_combo($combo, true /* is_blank_first_item */, list, "UserId" /* attr_id */, "UserId" /* attr_name */);
	};

	form_outbound_engine_processes_batches.prototype.__change_filter_field_name = function()
	{
		const $field_name = $("#out_form_process_batches_filters_field_name");
		const field_name = $field_name.val();
		let list = [];

		if (this.cached_fields_by_desc !== undefined && this.cached_fields_by_desc[field_name] !== undefined)
		{
			$.each(this.cached_fields_by_desc[field_name].values || [], function(_, field)
			{
				let item = "";
				if (typeof(field) === "object") {
					item = {
						label : field.Name,
						value: field.Value
					};
				}
				else {
					item = field;
				}
				list.push(item);
			});
		}

		// combo autocomplete
		const $field_value = $("#out_form_process_batches_filters_field_value");
		this.parent.make_autocomplete(
			$field_value,
			list,
			undefined,
			undefined,
			maxItems = list.length > 100? 100: undefined // topear a 100 los items a mostrar
		);
	};

	form_outbound_engine_processes_batches.prototype.hide_all_errors = function()
	{
		this.parent.hide_messages();
	};

	form_outbound_engine_processes_batches.prototype.__show_ctrl_err = function(err_info)
	{
		if (!err_info) {
			return;
		}
		this.parent.show_validation_error(err_info.$ctrl, err_info.message);
	};

	form_outbound_engine_processes_batches.prototype.mark_ctrl_error = function($ctrl)
	{
		if (!$ctrl || $ctrl.length === 0) {
			return;
		}
		this.parent.mark_ctrl_error($ctrl);
	};

	form_outbound_engine_processes_batches.prototype.validate_filter_condition_to_update = function()
	{
		this.form_paginator_filters.hide_popup_msg();

		var response = {success: false, $ctrl: null, item: null, msg: "<%=_T('Unable to Add or Update Filter Condition.')%>"};

		var $logical_operator = $("#out_form_process_batches_filters_field_logical_operator");
		var $left_parentheses = $("#out_form_process_batches_filters_field_left_parentheses");
		var $field_operator = $("#out_form_process_batches_filters_field_operator");
		var $rigth_parentheses = $("#out_form_process_batches_filters_field_rigth_parentheses");

		// field name - autocomplete
		let $field_name = $("#out_form_process_batches_filters_field_name");
		let field_name = this.parent.autocomplete_get_value($field_name);
		let field_description = field_name.label;
		let filed_info = this.__get_field_id_by_desc(field_description);

		// field value - autocomplete
		let $field_value = $("#out_form_process_batches_filters_field_value");
		let field_value = this.parent.autocomplete_get_value($field_value);

		var filter_condition = {
			logicalOp    : $logical_operator.val(),
			leftParenth  : $left_parentheses.val(),
			field        : filed_info.id,
			conditionOp  : $field_operator.val(),
			value        : field_value.value,
			rightParenth : $rigth_parentheses.val(),
			isCustomField: filed_info.is_custom_field
		};

		var validation_result;

		var count_lines = this.form_paginator_filters.count_rows();

		if (this.is_new_filter_condition) {
			// Si estoy creando y NO es la primer linea, el logical operator es requerido.
			if (count_lines > 0) {
				validation_result = inConcert.Utils.validate(filter_condition.logicalOp, "required");
				if ( !validation_result.success ) {
					response.msg = "<%=_T('Logical Operator is required.')%>";
					response.$ctrl = $logical_operator;
					return response;
				}
			}
		}
		else {
			// Si estoy editando y NO es la primer linea, el logical operator es requerido.
			if (this.current_filter_condition.__number > 1) {
				validation_result = inConcert.Utils.validate(filter_condition.logicalOp, "required");
				if ( !validation_result.success ) {
					response.msg = "<%=_T('Logical Operator is required.')%>";
					response.$ctrl = $logical_operator;
					return response;
				}
			}
			// El ID interno que genera el paginador. Esto sirve para que el paginador pueda actualizar la row en la grilla
			filter_condition.id = this.current_filter_condition.id;
		}

		validation_result = inConcert.Utils.validate(filter_condition.field, "required");
		if ( !validation_result.success ) {
			response.msg = "<%=_T('Field Name is required.')%>";
			response.$ctrl = $field_name;
			return response;
		}

		validation_result = inConcert.Utils.validate(filter_condition.conditionOp, "required");
		if ( !validation_result.success ) {
			response.msg = "<%=_T('Operator is required.')%>";
			response.$ctrl = $field_operator;
			return response;
		}

		response.success = true;
		response.msg = "";
		response.item = filter_condition;

		return response;
	};

	form_outbound_engine_processes_batches.prototype.add_or_update_filter_condition = function(callback, must_create_and_contiune)
	{
		callback = callback || $.noop;

		let msg_response = {msg: "", type: ""};
		const response = this.validate_filter_condition_to_update();

		if (!response.success) {
			this.mark_ctrl_error(response.$ctrl);
			msg_response.type = "Error";
			msg_response.msg = response.msg;
			callback(false, response.item, msg_response);
			return;
		}

		this.__cleanup_ctrls_popup();
		this.current_filter_condition = null;

		msg_response.msg = this.is_new_filter_condition ? "<%=_T('Filter Condition was successfully added.')%>" : "<%=_T('Filter Condition was successfully updated.')%>";
		msg_response.type = "Info";

		if (must_create_and_contiune) {
			const $logical_operator = $("#out_form_process_batches_filters_field_logical_operator");
			this.parent.disable_control($logical_operator, false, true);
		}

		callback(true, response.item, msg_response);
	};

	form_outbound_engine_processes_batches.prototype.__change_source = function(source)
	{
		this.__fill_actions(source);
		var $actions = $("#out_form_process_batches_actions");
		$actions.val("").trigger("change");
	};

	form_outbound_engine_processes_batches.prototype.__change_starting_on_date = function()
	{
		var $starting_on = $("#out_form_process_batches_filters_starting_on");
		var enabled = $starting_on.prop("checked");
		var $contacted_date = $("#out_form_process_batches_filters_contacted_date");
		var is_required = enabled;
		this.parent.disable_control($contacted_date, !enabled, is_required);
		if (!enabled) {
			$starting_on.val("");
			$contacted_date.val("");
		}
	};

	form_outbound_engine_processes_batches.prototype.__change_exclude_already = function()
	{
		var $exclude_already = $("#out_form_process_batches_filters_exclude_already");
		var enabled = $exclude_already.prop("checked");
		var $starting_on = $("#out_form_process_batches_filters_starting_on");
		if (!enabled) {
			$starting_on.prop("checked", false);
		}
		this.parent.disable_control($starting_on, !enabled);
		this.__change_starting_on_date();
	};

	form_outbound_engine_processes_batches.prototype.__change_filter = function()
	{
		this.form_paginator_filters.initialize();

		var $filters = $("#out_form_process_batches_filters");
		var filter = this.parent.combo_get_selected_item_data($filters);
		var $exclude_already = $("#out_form_process_batches_filters_exclude_already");
		var $delete_filter = $("#out_form_process_batches_filters_delete");
		var $save = $("#out_form_process_batches_filters_save");

		if (!filter) {
			$save.prop("disabled", true);
			$exclude_already.prop("checked", false);
			this.__change_exclude_already();
			$delete_filter.prop("disabled", true);
			return;
		}

		$save.prop("disabled", false);
		$delete_filter.prop("disabled", false);

		var $starting_on = $("#out_form_process_batches_filters_starting_on");
		var $contacted_date = $("#out_form_process_batches_filters_contacted_date");

		//cargo el filtro seleccionado
		var filter_lines = JSON.parse(filter.FilterSentence);
		if (filter_lines.ExcludeContacted === 1) {
			$exclude_already.prop("checked", true);
			if (filter_lines.ContactedDate) {
				$starting_on.prop("checked", true);
				this.parent.datepicker_set_date($contacted_date, filter_lines.ContactedDate);
			}
			else {
				$starting_on.prop("checked", false);
				$contacted_date.val("");
			}
		}
		else {
			$exclude_already.prop("checked", false);
			$starting_on.prop("checked", false);
			$contacted_date.val("");
		}

		this.__change_exclude_already();
	};

	form_outbound_engine_processes_batches.prototype.__change_action = function(action)
	{
		const $create_batch_container = $("#out_form_process_batches_action_create_batch_container");
		const $activate_contacts_container = $("#out_form_process_batches_action_activate_contacts_container");
		const $change_priority_container = $("#out_form_process_batches_action_change_priority_container");
		const $change_progressive_type_container = $("#out_form_process_batches_action_change_progressive_type_container");

		$create_batch_container.hide();
		$activate_contacts_container.hide();
		$change_priority_container.hide();
		$change_progressive_type_container.hide();

		switch(action) {
			case "createbatch":
				$create_batch_container.show();
			break;
			case "activatecontacts":
				$activate_contacts_container.show();
			break;
			case "changepriority":
				$change_priority_container.show();
			break;
			case "changeisprogressivetype":
					$change_progressive_type_container.show();
			break;
		}
	};

	form_outbound_engine_processes_batches.prototype.__initial_ctrls_values = function()
	{
		// Create New Batch section
		$("#out_form_process_batches_action_create_batch_id").val("");
		$("#out_form_process_batches_action_create_batch_priority").val("1");
		$("#out_form_process_batches_action_create_batch_contact_base_priority").val("100");
		this.parent.datepicker_set_date($("#out_form_process_batches_action_create_batch_start_date"));
		this.parent.datepicker_set_date($("#out_form_process_batches_action_create_batch_end_date"));
		$("#out_form_process_batches_action_create_batch_contact_top").val("");
		$("#out_form_process_batches_action_create_batch_order_by_field").val("");
		$("#out_form_process_batches_action_create_batch_dialing_order").prop("checked", false).trigger("change");
		$("#out_form_process_batches_action_create_batch_inverse_order").prop("checked", false);

		// Recycle Contacts section
		$("#out_form_process_batches_action_activate_schedule").prop("checked", false).trigger("change");
		$("#out_form_process_batches_action_activate_assign").prop("checked", false).trigger("change");
		$("#out_form_process_batches_action_activate_assign_agent").val("");
		$("#out_form_process_batches_action_activate_reset_retries").prop("checked", true);

		// Change Priority section
		$("#out_form_process_batches_action_change_priority_new").val("");
	};

	form_outbound_engine_processes_batches.prototype.__change_create_batch_dialing_order = function(enabled)
	{
		var $batch_order_by_field = $("#out_form_process_batches_action_create_batch_order_by_field");
		var $batch_inverse_order = $("#out_form_process_batches_action_create_batch_inverse_order");

		if (enabled) {
			this.parent.disable_control($batch_order_by_field, false);
			this.parent.disable_control($batch_inverse_order, false);
		}
		else {
			this.parent.disable_control($batch_order_by_field, true, true);
			this.parent.disable_control($batch_inverse_order, true, false);
		}
	};

	form_outbound_engine_processes_batches.prototype.__cleanup_ctrls_popup = function()
	{
		$("#out_form_process_batches_filters_field_left_parentheses").val("");
		$("#out_form_process_batches_filters_field_name").val("");
		$("#out_form_process_batches_filters_field_operator").val("");
		// field value - autocomplete
		let $field_value = $("#out_form_process_batches_filters_field_value");
		this.parent.autocomplete_set_value($field_value, "");
		$("#out_form_process_batches_filters_field_rigth_parentheses").val("");
		$("#out_form_process_batches_filters_field_logical_operator").val("").focus();
	};

	form_outbound_engine_processes_batches.prototype.__disabled_filter_actions = function(disabled)
	{
		$("#out_form_process_batches_filters_new").prop("disabled", disabled);
		$("#out_form_process_batches_filters_retrieve").prop("disabled", disabled);
		var $save = $("#out_form_process_batches_filters_save");
		var $filters = $("#out_form_process_batches_filters");
		this.parent.disable_control($filters, disabled);
		var $delete_filter = $("#out_form_process_batches_filters_delete");

		if (disabled) {
			$save.prop("disabled", true);
			$delete_filter.prop("disabled", true);
		}
		else {
			var filter = this.parent.combo_get_selected_item_data($filters);
			if (!filter) {
				$delete_filter.prop("disabled", true);
			}
			else {
				$delete_filter.prop("disabled", false);
			}
			var filter_lines = this.form_paginator_filters.count_rows();
			if (filter_lines === 0) {
				$save.prop("disabled", true);
			}
			else {
				$save.prop("disabled", false);
			}
		}

		this.form_paginator_filters.disable_grid_actions(disabled);
	};

	form_outbound_engine_processes_batches.prototype.on_edit_filter_condition = function(condition)
	{
		const self = this;

		this.is_new_filter_condition = condition ? false: true;
		this.current_filter_condition = condition;
		this.__cleanup_ctrls_popup();
		this.__disabled_filter_actions(true);

		var logical_op = "", left_parenth = "", condition_op = "", right_parenth = "", field = "", value = "";
		var $logical_operator = $("#out_form_process_batches_filters_field_logical_operator");

		var disable_logical_operator = true;

		if (!this.is_new_filter_condition) {
			logical_op = condition.logicalOp;
			left_parenth = condition.leftParenth;
			condition_op = condition.conditionOp;
			right_parenth = condition.rightParenth;
			var cached_field = this.cached_fields_by_field[condition.field];
			field = cached_field ? cached_field.Id : condition.field;
			value = condition.value;
			if (this.current_filter_condition.__number > 1 || (this.current_filter_condition.__number === 1 && logical_op.length > 0)) {
				disable_logical_operator = false;
			}
		}
		else {
			var count_lines = this.form_paginator_filters.count_rows();
			if (count_lines > 0) {
				disable_logical_operator = false;
			}
		}

		if (disable_logical_operator) {
			this.parent.disable_control($logical_operator, true);
		}
		else {
			this.parent.disable_control($logical_operator, false, true);
		} 

		$logical_operator.val(logical_op);
		$("#out_form_process_batches_filters_field_left_parentheses").val(left_parenth);
		$("#out_form_process_batches_filters_field_operator").val(condition_op);
		$("#out_form_process_batches_filters_field_rigth_parentheses").val(right_parenth);

		/*
			Aca se debe de implementar esta funcion(callack) directamente sobre la clase "conf_form" asi podemos tener el evento "change"
			del "autocomplete" field name.
			Despues de esto recien podes asignar el valor al "autocomplete" field value, ya que sino la lista de este ultimo no queda cargada a tiempo y el valor no queda asignado en el combo.
		*/
		this.parent.autocomplete_custom_change_callback = function() {
			// field value - autocomplete
			let $field_value = $("#out_form_process_batches_filters_field_value");
			self.parent.autocomplete_set_value($field_value, value);
			window.setTimeout(function() {
				self.parent.autocomplete_custom_change_callback = $.noop;
			});
		};
		// field name - autocomplete
		let $field_name = $("#out_form_process_batches_filters_field_name");
		this.parent.autocomplete_set_value($field_name, field);

		$logical_operator.focus();
	};

	form_outbound_engine_processes_batches.prototype.__get_field_id_by_desc = function(field_description)
	{
		var info = {
			id: null, is_custom_field: null
		};
		var field = this.cached_fields_by_desc[field_description];
		if (field !== undefined) {
			info.is_custom_field = false;
			info.id = field.field;
		}
		else {
			info.is_custom_field = true;
			info.id = field_description;
		}
		return info;
	};

	form_outbound_engine_processes_batches.prototype.__validate_expression = function()
	{
		const self = this;

		var response = {
			success: true,
			message: null
		};

		var expression = "";
		var query = {};
		query.values = [];
		query.condition = "";
		query.lines = [];

		var totalLeft = 0;
		var totalRight = 0;
		var isNumeric = function (x) {
		// var is_uri = inConcert.Utils.validate(uri, "url");
		// if (is_uri.success) {
			// return response;
		// }
			var RegExp = /^(-)?(\d*)(\.?)(\d*)$/;
			var result = x.match(RegExp);
			return result;
		};

		var filter_lines = this.form_paginator_filters.get_items();

		$.each(filter_lines, function(i, filter_line)
		{
			var logicalOp = filter_line.logicalOp;
			var leftParenth = filter_line.leftParenth;
			var fieldId = filter_line.field;
			var conditionOp = filter_line.conditionOp;
			var value = filter_line.value;
			var rightParenth = filter_line.rightParenth;

			if (logicalOp.length > 0 && i === 0) {
				response.success = false;
				response.message = "<%=_T('The first condition line cannot start with a logical operator.')%>";
				return false; // break
			}

			var missing_data = "";
			if (logicalOp.length === 0 && i > 0) {
				missing_data = "<%=_T('logical operator.')%>";
				return false; // break
			}
			if (fieldId.length === 0) {
				missing_data = "<%=_T('matching field.')%>";
				return false; // break
			}
			if (conditionOp.length === 0) {
				missing_data = "<%=_T('condition operator.')%>";
				return false; // break
			}
			if (missing_data.length) {
				response.success = false;
				response.message = FORMAT("<%=_T('Please review condition line {0}, you must enter {1}.')%>", i + 1, missing_data);
				return false; // break
			}

			totalLeft += parseInt(leftParenth.length, 10);
			totalRight += parseInt(rightParenth.length, 10);

			var isCustomField;
			var field = self.cached_fields_by_field[fieldId];

			if (field !== undefined) {
				isCustomField = false;
				fieldId = field.field;
			}
			else {
				isCustomField = true;
			}

			var line = {
				logicalOp    : logicalOp,
				leftParenth  : leftParenth,
				field        : fieldId,
				conditionOp  : conditionOp,
				value        : value,
				rightParenth : rightParenth,
				isCustomField: isCustomField
			};

			query.lines.push(line);

			if (line.isCustomField) {
				var modifier = "";
				if (conditionOp === "Like") {
					value = "%" + value + "%";
				}
				if (conditionOp === "<>") {
					conditionOp = "=";
					modifier = " NOT ";
				}
				expression += logicalOp + " " + leftParenth + modifier + " Exists (Select * From ContactImportNameValueData (nolock) Where ContactImportNameValueData.Name =  '" + fieldId;
				var customValue = isNumeric(value) ? "convert(float," + value + ") AND ContactImportNameValueData.DataType='number' ": "'" + value + "'";
				expression += "' AND ContactImportNameValueData.Value " + conditionOp + " " + customValue + " AND ContactImportNameValueData.id = ContactImportContact.Id AND ContactImportNameValueData.VCC = ContactImportContact.VCC) " + rightParenth;
				query.condition += logicalOp + " " + leftParenth + modifier + " Exists (Select * From ContactImportNameValueData (nolock) Where ContactImportNameValueData.Name =  '" + fieldId;
				query.condition += "' AND ContactImportNameValueData.Value " + conditionOp + " " + customValue + " AND ContactImportNameValueData.id = ContactImportContact.Id AND ContactImportNameValueData.VCC = ContactImportContact.VCC) " + rightParenth;
			}
			else {
				if (fieldId === "OutboundProcessContact.Status" && value === "ACTIVE") {
					if (conditionOp === "=") {
						conditionOp = "IS";
					}
					else {
						conditionOp = "IS NOT";
					}
					expression += logicalOp + " " + leftParenth + " " + fieldId + " " + conditionOp + " NULL " + rightParenth;
					query.condition += logicalOp + " " + leftParenth + " " + fieldId + " " + conditionOp + " NULL " + rightParenth;
				}
				else {
					if (fieldId === "OutboundProcessContact.Status" && value === "MISCONFIGURATION") {
						if (conditionOp === "=") {
							expression += logicalOp + " " + leftParenth + " " + "( ISNULL(OutboundProcessContact.ReservedActor,'') <> ''";
							expression += " AND (OutboundProcessContact.Status is null or OutboundProcessContact.Status = 'SCHEDULED') ";
							expression += "AND not exists (Select UC.UserID from CampaignOutboundInteraction COI (nolock), UserCampaign UC (nolock) ";
							expression += "where COI.CampaignId = UC.CampaignId and COI.VCC = UC.VCC ";
							expression += "and OutboundProcessContact.OutboundProcessId = COI.OutboundProcessId and OutboundProcessContact.VCC = COI.VCC";
							expression += " and UC.UserId = OutboundProcessContact.ReservedActor ))";
							expression += rightParenth;

							query.condition += logicalOp + " " + leftParenth + " " + "( ISNULL(OutboundProcessContact.ReservedActor,'') <> ''";
							query.condition += " AND (OutboundProcessContact.Status is null or OutboundProcessContact.Status = 'SCHEDULED') ";
							query.condition += "AND not exists (Select UC.UserID from CampaignOutboundInteraction COI (nolock), UserCampaign UC (nolock) ";
							query.condition += "where COI.CampaignId = UC.CampaignId and COI.VCC = UC.VCC ";
							query.condition += "and OutboundProcessContact.OutboundProcessId = COI.OutboundProcessId and OutboundProcessContact.VCC = COI.VCC";
							query.condition += " and UC.UserId = OutboundProcessContact.ReservedActor ))";
							query.condition += rightParenth;
						}
						else {
							expression += logicalOp + " " + leftParenth + " " + "(ISNULL(OutboundProcessContact.ReservedActor,'') = ''";
							expression += "OR (OutboundProcessContact.Status is not null and OutboundProcessContact.Status <> 'SCHEDULED')  ";
							expression += "OR exists (Select UC.UserID from CampaignOutboundInteraction COI (nolock), UserCampaign UC (nolock) ";
							expression += "where COI.CampaignId = UC.CampaignId and COI.VCC = UC.VCC ";
							expression += "and OutboundProcessContact.OutboundProcessId = COI.OutboundProcessId and OutboundProcessContact.VCC = COI.VCC";
							expression += " and UC.UserId = OutboundProcessContact.ReservedActor ))";
							expression += rightParenth;

							query.condition += logicalOp + " " + leftParenth + " " + "(ISNULL(OutboundProcessContact.ReservedActor,'') = ''";
							query.condition += "OR (OutboundProcessContact.Status is not null and OutboundProcessContact.Status <> 'SCHEDULED')  ";
							query.condition += "OR exists (Select UC.UserID from CampaignOutboundInteraction COI (nolock), UserCampaign UC (nolock) ";
							query.condition += "where COI.CampaignId = UC.CampaignId and COI.VCC = UC.VCC ";
							query.condition += "and OutboundProcessContact.OutboundProcessId = COI.OutboundProcessId and OutboundProcessContact.VCC = COI.VCC";
							query.condition += " and UC.UserId = OutboundProcessContact.ReservedActor ))";
							query.condition += rightParenth;
						}
					}
					else {
						//Si el campo permite NULL y (operador es <> y valor <> '') OR (operador es = y valor = '') agrego la condicion IS NULL
						if ((fieldId === "LastManagementResult" || fieldId === "OutboundProcessContact.ReservedActor" || fieldId === "OutboundProcessContact.Status" || fieldId === "OutboundProcessContact.Cause" || fieldId === "OutboundProcessContact.PreviousContactResult" || fieldId === "OutboundProcessContact.PreviousContactDate" || fieldId === "OutboundProcessContact.ScheduledDate") && ((conditionOp === "=" && value.length === 0) || (conditionOp === "<>" && value.length > 0))) {
							expression += logicalOp + " " + leftParenth + " ( " + fieldId + " " + conditionOp + " " + value + " or " + fieldId + " IS NULL ) " + rightParenth;
							query.condition += logicalOp + " " + leftParenth + " ( " + fieldId + " " + conditionOp + " ? or " + fieldId + " IS NULL ) " + rightParenth;
							query.values.push(value);
						} 
						else {
							const regExpMatch = /^([\w]+)(?:.(\w+))?/;
							fieldId.match(regExpMatch);
							const table =  RegExp.$1;
							const skillName = RegExp.$2;

							if (table === "ContactImportSkill") {
								expression += logicalOp + " " + leftParenth + " " + "( ContactImportSkill.SkillID = '" +  skillName + "'";
								expression += " AND ContactImportSkill.Value " +  conditionOp + " " + value + " ";
								expression += rightParenth;

								query.condition += logicalOp + " " + leftParenth + " " + "( ContactImportSkill.SkillID = '" +  skillName + "'";
								query.condition += " AND ContactImportSkill.Value " +  conditionOp + " ? )";
								query.condition += rightParenth;
								query.values.push(value);
							}
							else {
								expression += logicalOp + " " + leftParenth + " " + fieldId + " " + conditionOp + " " + value + " " + rightParenth;
								query.condition += logicalOp + " " + leftParenth + " " + fieldId + " " + conditionOp + " ? " + rightParenth;
								query.values.push(value);
							}
						}
					}
				}
			}
		});

		if (totalLeft !== totalRight) {
			response.success = false;
			response.message = "<%=_T('Left and right parenthesis do not match.')%>";
		}

		if (!response.success) {
			return response;
		}

		response.success = true;
		response.query = query;
		return response;
	};

	form_outbound_engine_processes_batches.prototype.is_ok_contact_query_result = function()
	{
		let result = {success: false, codeErr: "ERROR RETRIEVING CONTACTS", message: "<%=_T('There are no contacts.')%>"};
		if (!this.contact_query_result || !this.contact_query_result.success || !this.contact_query_result.result) {
			return result;
		}

		const limitRowsInfo = this.contact_query_result.result.limitRowsInfo;
		let exceededTotalLimitRows = false;
		let exceededVCCLimitRows = false;
		if (limitRowsInfo !== undefined) {
			exceededTotalLimitRows = limitRowsInfo.exceededTotalLimit;
			exceededVCCLimitRows = limitRowsInfo.exceededVCCLimit;
		}
		if (exceededTotalLimitRows || exceededVCCLimitRows) {
			result.codeErr = "BATCH EXCEEDS THE ALLOWED LIMIT";
			result.message = "<%=_T('Creating this batch would exceed the allowed contact limit, please delete an existing batch.')%>";
			return result;
		}

		// is ok
		result.success = true;
		result.message = "IS OK";
		result.limitRowsInfo = limitRowsInfo;
		delete result.codeErr;
		return result;
	};

	form_outbound_engine_processes_batches.prototype.__get_query_id = function()
	{
		if (!this.is_ok_contact_query_result().success) {
			return;
		}
		return this.contact_query_result.result.queryid;
	};

	form_outbound_engine_processes_batches.prototype.__load_contacts_from_query = function(callback)
	{
		callback = callback || $.noop;

		var query_id = this.__get_query_id();
		if (!query_id) {
			callback(true, []);
			return;
		}

		var outbound_engine_api = inConcert.app.getOutboundEngineAPI();
		outbound_engine_api.processLoadContactsFromQuery(query_id,
			{
				start: 1,
				count: this.server_side_contacts_rows_per_page
			},
			function(success, contacts) {
				if (success) {
					callback(true, contacts);
				}
				else {
					callback(false, []);
				}
			}
		);
	};

	form_outbound_engine_processes_batches.prototype.__on_executed_query_contacts = function(success)
	{
		const self = this;

		if (!success) {
			// TODO: Error
			this.parent.hide_waiting();
			return;
		}

		let query_result = this.is_ok_contact_query_result();
		if (!query_result.success) {
			this.parent.hide_waiting();
			this.__show_ctrl_err(query_result);
			return;
		}

		this.__load_contacts_from_query(function(is_ok, contacts) {
			if (is_ok) {
				self.form_paginator_contacts.initialize(function() {
						self.parent.hide_waiting();
					},
					contacts
				);
				// Como la grilla de contactos visualiza unicamente 200 contactos (this.server_side_contacts_rows_per_page)
				//   tengo que actualizar el total del footer del paginado, con la informacion real del total de registros/contactos.
				self.form_paginator_contacts.set_footer_total_rows(query_result.limitRowsInfo.totalRowsCurrent);
				self.form_paginator_contacts.update_footer_total_text();
			}
			else {
				self.parent.hide_waiting();
			}
		});
	};

	form_outbound_engine_processes_batches.prototype.__execute_query_contacts = function()
	{
		const self = this;

		this.hide_all_errors();

		var $source = $("#out_form_process_batches_data_source");
		var source = $source.val();
		var validation_result = inConcert.Utils.validate(source, "required");
		var err;
		if ( !validation_result.success ) {
			err = validation_result;
			err.message = "<%=_T('Data Source is required.')%>";
			err.$ctrl = $source;
			this.__show_ctrl_err(err);
			return;
		}

		var response = this.__validate_expression();
		if (!response.success) {
			err = response;
			err.$ctrl = $("#out_form_process_batches_filters_list"); // el header de la lista de filtros
			this.__show_ctrl_err(err);
			return;
		}

		var query = response.query;
		var process = this.get_entity_instance();
		query.ProcessId = process.Id;
		query.ExcludeContacted = $("#out_form_process_batches_filters_exclude_already").prop("checked") ? 1 : 0;
		if ($("#out_form_process_batches_filters_starting_on").prop("checked")) {
			query.ContactedDate = $("#out_form_process_batches_filters_contacted_date").val();
		}

		this.parent.show_waiting("<%=_T('Please wait, Retrieving Contacts....')%>");

		var outbound_engine_api = inConcert.app.getOutboundEngineAPI();

		outbound_engine_api.processRunBatchOrContactsQuery(
			{
				query: query,
				rows_per_page: this.server_side_contacts_rows_per_page,
				source: source
			},
			function(success, query_result) {
				if (success) {
					self.contact_query_result = query_result;
				}
				else {
					self.contact_query_result = null;
				}
				self.__on_executed_query_contacts(success);
			}
		);
	};

	form_outbound_engine_processes_batches.prototype.__clear_filter = function()
	{
		window.alert("TODO: IMPLEMENT-> form_outbound_engine_processes_batches.prototype.__clear_filter");
	};

	form_outbound_engine_processes_batches.prototype.__validate_filter = function(filter_id)
	{
		this.hide_all_errors();

		var response = {
			success: true,
			message: null
		};

		var validation_result = inConcert.Utils.validate(filter_id, "required");
		if ( !validation_result.success ) {
			response.success = false;
			response.message = "<%=_T('Filter Name is required.')%>";
			return response;
		}

		validation_result = this.__validate_expression();
		if (!validation_result.success) {
			return validation_result;
		}

		var filter_lines = validation_result.query.lines;
		if (filter_lines.length === 0) {
			response.success = false;
			response.message = "<%=_T('Filter conditions are required.')%>";
			return response;
		}

		var contacted_date;
		var exclude_contacted = $("#out_form_process_batches_filters_exclude_already").prop("checked") ? 1 : 0;
		if ($("#out_form_process_batches_filters_starting_on").prop("checked")) {
			contacted_date = $("#out_form_process_batches_filters_contacted_date").val();
		}

		response.success = true;
		response.filter = {
			Id              : filter_id,
			Query           : filter_lines,
			ExcludeContacted: exclude_contacted,
			ContactedDate   : contacted_date
		};

		return response;
	};

	form_outbound_engine_processes_batches.prototype.__save_filter = function(filter_id, callback)
	{
		const self = this;
		callback = callback || $.noop;

		var validation_result = this.__validate_filter(filter_id);

		if (!validation_result.success) {
			callback(false);
			this.__show_ctrl_err(validation_result);
			return;
		}

		this.parent.show_waiting(FORMAT("<%=_T('Please wait, Saving {0} Filter')%>", filter_id));

		var outbound_engine_api = inConcert.app.getOutboundEngineAPI();
		outbound_engine_api.processSaveFilter(
			validation_result.filter,
			function(success, result) {
				// Este timeout es solo visual para darle feedback.
				// Se agrega para poder ver el waiting un momento, sino es muy rapido y hace un parpadeo loco.
				window.setTimeout(function() {
					var msg, type;
					if (success) {
						var $filter = $("#out_form_process_batches_filters");
						if (result.result.action === "CREATED FILTER") {
							self.parent.combo_add_value($filter, filter_id, undefined, true, undefined, result.result.filter);
							msg = "<%=_T('Successfully Saved {0} Filter.')%>";
						}
						else {
							self.parent.combo_update_value($filter, filter_id, undefined, filter_id, undefined, result.result.filter);
							msg = "<%=_T('Successfully Updated {0} Filter.')%>";
						}
						type = "Info";
					}
					else {
						msg = "<%=_T('Error Saving {0} Filter.')%>";
						type = "Error";
					}
					self.parent.hide_waiting();
					self.parent.show_status_message(FORMAT(msg, filter_id), type, 5000);
					callback(success);
				}, 500);
			}
		);
	};

	form_outbound_engine_processes_batches.prototype.__open_save_filter_popup = function(btn_parent)
	{
		const self = this;

		if (this.filter_popup) {
			this.filter_popup.close();
			this.filter_popup = null;
		}

		var $filter = $("#out_form_process_batches_filters");
		var filter_id = $filter.val();

		this.filter_popup = new SaveFilterPopup({current_filter: filter_id});
		this.filter_popup.show({
			parentNode: $(btn_parent),
			owner_object: this,
			callbacks: {
				save: function(filter_id, callback) {
					self.__save_filter(filter_id, callback);
				}
			}
		});
	};

	form_outbound_engine_processes_batches.prototype.__delete_filter = function()
	{
		const self = this;

		var $filter = $("#out_form_process_batches_filters");
		var filter_id = $filter.val();

		if (!filter_id) {
			return;
		}

		var remove = function()
		{
			self.parent.show_waiting(FORMAT("<%=_T('Please wait, Deleting {0} Filter')%>", filter_id));

			var outbound_engine_api = inConcert.app.getOutboundEngineAPI();
			outbound_engine_api.processDeleteFilter({
					filterId: filter_id
				},
				function(success) {
					// Este timeout es solo visual para darle feedback.
					// Se agrega para poder ver el waiting un momento, sino es muy rapido y hace un parpadeo loco.
					window.setTimeout(function() {
						var msg, type;
						if (success) {
							msg = "<%=_T('Successfully Deleted {0} Filter.')%>";
							type = "Info";
							self.parent.combo_remove_value($filter, filter_id);
						}
						else {
							msg = "<%=_T('Error Deleting {0} Filter.')%>";
							type = "Error";
						}
						self.parent.hide_waiting();
						self.parent.show_status_message(FORMAT(msg, filter_id), type, 5000);
					}, 500);
				}
			);
		};

		$.inconcertPopup.Show(
			"<%=_T('Confirm')%>",
			FORMAT("<%=_T('Do you want to delete Filter {0} ?')%>", filter_id),
			$.inconcertPopup.ButtonSets.BS_YESNO,
			function(response) {
				if (response.toLowerCase() !== "yes") {
					return;
				}
				remove();
			},
			undefined,
			{
				popup_id       : "conf_inconcertPopup",
				container_class: "confirmationPopUp",
				overlay_class  : "confirmationOverlay"
			}
		);
	};

	form_outbound_engine_processes_batches.prototype.__validate_action_to_execute = function()
	{
		this.hide_all_errors();

		var response = {
			success: true,
			message: null,
			$ctrl  : null
		};

		var $source = $("#out_form_process_batches_data_source");
		var source = $source.val();
		var validation_result = inConcert.Utils.validate(source, "required");
		if ( !validation_result.success ) {
			response.success = false;
			response.message = "<%=_T('Data Source is required.')%>";
			response.$ctrl = $source;
			return response;
		}

		var $actions = $("#out_form_process_batches_actions");
		var action = $actions.val();
		validation_result = inConcert.Utils.validate(action, "required");
		if ( !validation_result.success ) {
			response.success = false;
			response.message = "<%=_T('Action is required.')%>";
			response.$ctrl = $actions;
			return response;
		}

		let query_result = this.is_ok_contact_query_result();
		if (!query_result.success) {
			response.success = false;
			response.message = query_result.message;
			response.$ctrl = null;
			return response;
		}

		var query_id = this.__get_query_id();
		validation_result = inConcert.Utils.validate(query_id, "required");
		if ( !validation_result.success ) {
			response.success = false;
			response.message = "<%=_T('Must recover the contacts.')%>";
			response.$ctrl = $actions;
			return response;
		}

		var process = this.get_entity_instance();

		response.action = {
			ExcludeDuplicates : $("#out_form_process_batches_contacts_duplicated_exclude").prop("checked") ? 1 : 0,
			OutboundProcessId: process.Id
		};

		var validate_priority = function($ctrl, range) {
			var value = $ctrl.val();
			var result = inConcert.Utils.validate(value, "required");
			if ( !result.success ) {
				result.message = "<%=_T('Priority is required.')%>";
				return result;
			}
			result = inConcert.Utils.validate(value, "digits");
			if ( !result.success ) {
				result.message = "<%=_T('Priority must be a number.')%>";
				return result;
			}
			result = inConcert.Utils.validate(value, "range", range);
			if ( !result.success ) {
				result.message = FORMAT("<%=_T('Contact base priority must be a number between {0} and {1}')%>", range.between.from, range.between.to);
				return result;
			}
			return result;
		};

		if (source === "contact_query") {
			var $create_batch_id = $("#out_form_process_batches_action_create_batch_id");
			var $create_batch_priority = $("#out_form_process_batches_action_create_batch_priority");
			var $create_batch_start_date = $("#out_form_process_batches_action_create_batch_start_date");
			var $create_batch_end_date = $("#out_form_process_batches_action_create_batch_end_date");
			var $create_batch_contact_base_priority = $("#out_form_process_batches_action_create_batch_contact_base_priority");
			var $create_batch_contact_top = $("#out_form_process_batches_action_create_batch_contact_top");
			var $create_batch_order_by_field = $("#out_form_process_batches_action_create_batch_order_by_field");
			var $create_order_by_field = $("#out_form_process_batches_action_create_batch_inverse_order");

			response.action.BatchId = $create_batch_id.val();
			response.action.Priority =  $create_batch_priority.val();
			response.action.StartDate = $create_batch_start_date.val();
			response.action.EndDate = $create_batch_end_date.val();
			response.action.PriorityBase = $create_batch_contact_base_priority.val();
			response.action.ContactTarget = $create_batch_contact_top.val().trim().length > 0 ? $create_batch_contact_top.val() : null;

			validation_result = inConcert.Utils.validate(response.action.BatchId, "required");
			if ( !validation_result.success ) {
				response.success = false;
				response.message = "<%=_T('Batch Name is required.')%>";
				response.$ctrl = $create_batch_id;
				return response;
			}

			validation_result = inConcert.Utils.validate(response.action.BatchId, "id");
			if (!validation_result.success) {
				response.success = false;
				response.message = "<%=_T('Only letters, numbers, dashes and dots are allowed in the Batch Name.')%>";
				response.$ctrl = $create_batch_id;
				return response;
			}

			validation_result = validate_priority($create_batch_priority, { between: {from: 1, to: 100} });
			if ( !validation_result.success ) {
				response.success = false;
				response.message = validation_result.message;
				response.$ctrl = $create_batch_priority;
				return response;
			}

			validation_result = validate_priority($create_batch_contact_base_priority, { between: {from: 1, to: 1000000} });
			if ( !validation_result.success ) {
				response.success = false;
				response.message = validation_result.message;
				response.$ctrl = $create_batch_contact_base_priority;
				return response;
			}

			if ($("#out_form_process_batches_action_create_batch_dialing_order").prop("checked")) {
				response.action.OrderField = $create_batch_order_by_field.val();
				response.action.InverseOrder = $create_order_by_field.prop("checked") ? 1 : 0;

				validation_result = inConcert.Utils.validate(response.action.OrderField, "required");
				if ( !validation_result.success ) {
					response.success = false;
					response.message = "<%=_T('Order by Field is required.')%>";
					response.$ctrl = $create_batch_order_by_field;
					return response;
				}
			}
		}
		else {
			response.action.Action = action;

			switch(action) {
				case "cancelcontacts":
					break;
				case "increaseretries":
					break;
				case "changepercentage":
					break;

				case "activatecontacts":
					if ($("#out_form_process_batches_action_activate_schedule").prop("checked")) {
						var $action_activate_schedule_date = $("#out_form_process_batches_action_activate_schedule_date");
						response.action.ActivationDate = $action_activate_schedule_date.val();

						validation_result = inConcert.Utils.validate(response.action.ActivationDate, "required");
						if ( !validation_result.success ) {
							response.success = false;
							response.message = "<%=_T('Activate Date is required.')%>";
							response.$ctrl = $action_activate_schedule_date;
							return response;
						}
					}
					if ($("#out_form_process_batches_action_activate_assign").prop("checked")) {
						var $action_activate_assign_agent = $("#out_form_process_batches_action_activate_assign_agent");
						response.action.ActivationReservedActor = $action_activate_assign_agent.val();
						validation_result = inConcert.Utils.validate(response.action.ActivationReservedActor, "required");
						if ( !validation_result.success ) {
							response.success = false;
							response.message = "<%=_T('Agent is required.')%>";
							response.$ctrl = $action_activate_assign_agent;
							return response;
						}
					}
					response.action.ResetRetries = $("#out_form_process_batches_action_activate_reset_retries").prop("checked");

				break;

				case "changepriority":
					var $action_change_priority_new = $("#out_form_process_batches_action_change_priority_new");
					response.action.Priority = $action_change_priority_new.val();

					validation_result = validate_priority($action_change_priority_new, { between: {from: 1, to: 1000000} });
					if ( !validation_result.success ) {
						response.success = false;
						response.message = validation_result.message;
						response.$ctrl = $action_change_priority_new;
						return response;
					}
				break;

				case "changeisprogressivetype":
					const $action_change_progressive_type = $("#out_form_process_batches_action_change_progressive_type");
					response.action.IsProgressiveType = $action_change_progressive_type.prop("checked") ? 1 : 0;
				break;
			}
		}

		return response;
	};

	form_outbound_engine_processes_batches.prototype.__run_action = function()
	{
		const self = this;
		var response = this.__validate_action_to_execute();

		if (!response.success) {
			this.__show_ctrl_err(response);
			return;
		}

		var callback = function(success/*, response*/) {
			self.parent.hide_waiting();
			if (success) {
				self.parent.show_status_message("<%=_T('Action completed successfully!')%>", "Info", 5000);
				// Dejo que se visualice un instante el mensaje en esta pantalla, y regreso a la pantalla de procesos.
				window.setTimeout(function() {
					self.parent.form_paginator_batches.initialize();
					self.__show_process_section();
				}, 500);
				return;
			}
			self.parent.show_status_message("<%=_T('Unable to execute the batch operation, please contact your administrator.')%>", "Error", 5000);
		};

		var outbound_engine_api = inConcert.app.getOutboundEngineAPI();
		var source = $("#out_form_process_batches_data_source").val();
		var query_id = this.__get_query_id();

		this.parent.show_waiting( "<%=_T('Please wait, Executing Action....')%>" );

		if (source === "contact_query") {
			outbound_engine_api.processCreateBatch(
				{queryId: query_id},
				response.action,
				function(success, response) {
					// pequeno waiting para dar feedback
					window.setTimeout(function() {
						callback(success, response);
					}, 500);
				}
			);
		}
		else if (source === "batch_query") {
			outbound_engine_api.processUpdateBatchByQuery(
				{queryId: query_id, processId: response.action.OutboundProcessId},
				response.action,
				function(success, response) {
					// pequeno waiting para dar feedback
					window.setTimeout(function() {
						callback(success, response);
					}, 500);
				}
			);
		}
		else {
			throw("form_outbound_engine_processes_batches.prototype.__run_action -> Unrecognized '" + source + "'");
		}
	};

	form_outbound_engine_processes_batches.prototype.__show_process_section = function()
	{
		const self = this;
		this.parent.show_process_section(function() {
			self.form_paginator_filters.hide_popup();
		});
	};

	form_outbound_engine_processes_batches.prototype.set_events = function()
	{
		const self = this;

		this.form_paginator_filters.set_events();
		this.form_paginator_contacts.set_events();

		var $source = $("#out_form_process_batches_data_source");
		$source
			.off("change")
			.on("change", function() {
				var source = $(this).val();
				self.__change_source(source);
			});

		var $filters = $("#out_form_process_batches_filters");
		$filters
			.off("change")
			.on("change", function() {
				self.__change_filter();
			});

		var $actions = $("#out_form_process_batches_actions");
		$actions
			.off("change")
			.on("change", function() {
				var action = $(this).val();
				self.__change_action(action);
			});

		var $exclude_already = $("#out_form_process_batches_filters_exclude_already");
		$exclude_already
			.off("change")
			.on("change", function() {
				self.__change_exclude_already();
			});

		var $starting_on = $("#out_form_process_batches_filters_starting_on");
		$starting_on
			.off("change")
			.on("change", function() {
				self.__change_starting_on_date();
			});

		var $retrieve = $("#out_form_process_batches_filters_retrieve");
		$retrieve
			.off("click")
			.on("click", function() {
				self.__execute_query_contacts();
			});

		var $clear = $("#out_form_process_batches_filters_clear");
		$clear
			.off("click")
			.on("click", function() {
				self.__clear_filter();
			});

		var $save = $("#out_form_process_batches_filters_save");
		$save
			.off("click")
			.on("click", function() {
				self.__open_save_filter_popup(this);
			});

		var $delete = $("#out_form_process_batches_filters_delete");
		$delete
			.off("click")
			.on("click", function() {
				self.__delete_filter();
			});

		var $action_execute = $("#out_form_process_batches_action_execute");
		$action_execute
			.off("click")
			.on("click", function() {
				self.__run_action();
			});

		this.parent.$get_dom_element_by_id("form_back_btn_container")
			.off("click")
			.on("click", function() {
				self.__show_process_section();
				$(this).hide();
			});

		var $action_create_batch_dialing_order = $("#out_form_process_batches_action_create_batch_dialing_order");
		$action_create_batch_dialing_order
			.off("change")
			.on("change", function() {
				var enabled = $(this).prop("checked");
				self.__change_create_batch_dialing_order(enabled);
			});

		var $action_activate_schedule = $("#out_form_process_batches_action_activate_schedule");
		$action_activate_schedule
			.off("change")
			.on("change", function() {
				var $schedule_date = $("#out_form_process_batches_action_activate_schedule_date");
				var enabled = $(this).prop("checked");
				if (enabled) {
					self.parent.disable_control($schedule_date, false, true);
				}
				else {
					self.parent.disable_control($schedule_date, true);
				}
			});

		var $action_activate_assign = $("#out_form_process_batches_action_activate_assign");
		$action_activate_assign
			.off("change")
			.on("change", function() {
				var $assign_agent = $("#out_form_process_batches_action_activate_assign_agent");
				var enabled = $(this).prop("checked");
				if (enabled) {
					self.parent.disable_control($assign_agent, false, true);
				}
				else {
					self.parent.disable_control($assign_agent, true);
				}
			});
	};

	//---------------------------------------------------------------------
	// form process rules class
	function form_outbound_engine_processes_rules(params)
	{
		this.parent = params.parent;

		this.edited_rule = null;
		this.is_new_action = false;
		this.current_action = null;

		this.form_paginator_destinations = null;
		this.form_paginator_actions = null;

		this.cached_rules_responses_by_channel = null;
		this.cached_rules_destinations = null;

		this.period_list = {
			1: {id: "1", name: "<%=_T('Seconds')%>"},
			2: {id: "2", name: "<%=_T('Minutes')%>"},
			3: {id: "3", name: "<%=_T('Days')%>"   }
		};

		this.priority_type = {
			1: {id: "1", name: "<%=_T('Priority points')%>"},
			2: {id: "2", name: "<%=_T('Percent')%>"        }
		};

		this.builtin_destinations = {
			ANY_DESTINATION : "<%=_T('ANY_DESTINATION')%>",
			NEXT_ON_LIST    : "<%=_T('NEXT_ON_LIST')%>",
			SAME_ADDRESS    : "<%=_T('SAME_ADDRESS')%>",
			SAME_DESTINATION: "<%=_T('SAME_DESTINATION')%>"
		};

		// en el combo: 3 "", 1 "Progressive", 2 "Not_Progressive"
		this.change_contact_types_list = {
			Progressive: {id: "Progressive", name: "<%=_T('Progressive')%>"},
			Not_Progressive: {id: "Not_Progressive", name: "<%=_T('Not Progressive')%>"}
		};

		this.defined_rule_responses = [
			//PREVIEW_CANCELED
			{Response: -20, LastDestination: "ANY_DESTINATION", NextDestination: "SAME_ADDRESS", 
				InvalidateAddress: 0, CancelInteraction: 0, SecondsToNextAction: 7200, AffectRetries: 0
			},
			//si link down call same destination en 30 minutes
			{Response: -10, LastDestination: "ANY_DESTINATION", NextDestination: "SAME_ADDRESS", 
				InvalidateAddress: 0, CancelInteraction: 0, SecondsToNextAction: 1800, AffectRetries: 1
			},
			//si abandoned call same destination in 300 seconds (5 minutes)
			{Response: -4, LastDestination: "ANY_DESTINATION", NextDestination: "SAME_ADDRESS",
				InvalidateAddress: 0, CancelInteraction: 0, SecondsToNextAction: 300, AffectRetries: 1
			},
			//si blacklist cancel and invalidate
			{Response: -2, LastDestination: "ANY_DESTINATION", NextDestination: "NEXT_ON_LIST", 
				InvalidateAddress: 1, CancelInteraction: 1, AffectRetries: 1
			},
			//si no answer call next destination in 120 minutes
			{Response: -1, LastDestination: "ANY_DESTINATION", NextDestination: "NEXT_ON_LIST",
				InvalidateAddress: 0, CancelInteraction: 0, SecondsToNextAction: 7200, AffectRetries: 1
			},
			//CANT_ROUTE_TO_ENDPOINT
			{Response: 1, LastDestination: "ANY_DESTINATION", NextDestination: "NEXT_ON_LIST", 
				InvalidateAddress: 0, CancelInteraction: 0, SecondsToNextAction: 7200, AffectRetries: 1
			},
			//NORMAL_CALL_END
			{Response: 2, LastDestination: "ANY_DESTINATION", NextDestination: "NEXT_ON_LIST", 
				InvalidateAddress: 0, CancelInteraction: 0, SecondsToNextAction: 7200, AffectRetries: 1
			},
			//si busy call same destination in 30 minutes
			{Response: 3, LastDestination: "ANY_DESTINATION", NextDestination: "SAME_ADDRESS", 
				InvalidateAddress: 0, CancelInteraction: 0, SecondsToNextAction: 1800, AffectRetries: 1
			},
			//CALL_REJECTED
			{Response: 4, LastDestination: "ANY_DESTINATION", NextDestination: "NEXT_ON_LIST", 
				InvalidateAddress: 0, CancelInteraction: 0, SecondsToNextAction: 1800, AffectRetries: 1
			},
			//si number changed cancel and invalidate
			{Response: 5, LastDestination: "ANY_DESTINATION", NextDestination: "NEXT_ON_LIST", 
				InvalidateAddress: 1, CancelInteraction: 1, AffectRetries: 1
			},
			//INVALID_USE_NETWORK
			{Response: 6, LastDestination: "ANY_DESTINATION", NextDestination: "NEXT_ON_LIST", 
				InvalidateAddress: 0, CancelInteraction: 0, SecondsToNextAction: 7200, AffectRetries: 1
			},
			//si congestion call same destination in 30 minutes
			{Response: 8, LastDestination: "ANY_DESTINATION", NextDestination: "SAME_ADDRESS",
				InvalidateAddress: 0, CancelInteraction: 0, SecondsToNextAction: 1800, AffectRetries: 1
			},
			//HANDSHAKING_FAILED
			{Response: 9, LastDestination: "ANY_DESTINATION", NextDestination: "SAME_ADDRESS", 
				InvalidateAddress: 0, CancelInteraction: 0, SecondsToNextAction: 7200, AffectRetries: 1
			},
			//CALL_REJECTED_BY_ENDPOINT
			{Response: 10, LastDestination: "ANY_DESTINATION", NextDestination: "SAME_ADDRESS", 
				InvalidateAddress: 0, CancelInteraction: 0, SecondsToNextAction: 7200, AffectRetries: 1
			},
			//NETWORK_ERROR
			{Response: 11, LastDestination: "ANY_DESTINATION", NextDestination: "SAME_ADDRESS", 
				InvalidateAddress: 0, CancelInteraction: 0, SecondsToNextAction: 7200, AffectRetries: 1
			},
			//PROTOCOL_ERROR
			{Response: 12, LastDestination: "ANY_DESTINATION", NextDestination: "SAME_ADDRESS", 
				InvalidateAddress: 0, CancelInteraction: 0, SecondsToNextAction: 7200, AffectRetries: 1
			},
			//CAUSE_UNKNOWN
			{Response: 13, LastDestination: "ANY_DESTINATION", NextDestination: "NEXT_ON_LIST", 
				InvalidateAddress: 0, CancelInteraction: 1, AffectRetries: 1
			},
			//ANSWERING_MACHINE
			{Response: 14, LastDestination: "ANY_DESTINATION", NextDestination: "NEXT_ON_LIST", 
				InvalidateAddress: 0, CancelInteraction: 0, SecondsToNextAction: 7200, AffectRetries: 1
			},
			//FAX
			{Response: 15, LastDestination: "ANY_DESTINATION", NextDestination: "NEXT_ON_LIST", 
				InvalidateAddress: 0, CancelInteraction: 0, SecondsToNextAction: 7200, AffectRetries: 1
			},
			//DEAD_CONTACT
			{Response: 16, LastDestination: "ANY_DESTINATION", NextDestination: "NEXT_ON_LIST", 
				InvalidateAddress: 0, CancelInteraction: 1, AffectRetries: 1
			},
			//INDIRECT_CONTACT
			{Response: 17, LastDestination: "ANY_DESTINATION", NextDestination: "NEXT_ON_LIST", 
				InvalidateAddress: 0, CancelInteraction: 0, SecondsToNextAction: 7200, AffectRetries: 1
			}
		];
		this.seconds_in_a_day = 24 * 60 * 60;

		this.__initialize();

		return true;
	}

	form_outbound_engine_processes_rules.prototype.__initialize = function()
	{
		const self = this;

		var outbound_engine_api = inConcert.app.getOutboundEngineAPI();

		outbound_engine_api.systemRulesLoadResponsesByChannel(function(success, responsesByChannel) {
			if (success) {
				self.cached_rules_responses_by_channel = responsesByChannel;
			}
		});

		outbound_engine_api.systemRulesLoadDestinations(function(success, destinations) {
			if (success) {
				self.cached_rules_destinations = destinations;
			}
		});

		this.__new_grids();
	};

	form_outbound_engine_processes_rules.prototype.get_parent_process_rules = function()
	{
		return this.parent.__get_rules();
	};

	form_outbound_engine_processes_rules.prototype.set = function(rule)
	{
		const self = this;

		var caption_add_button;
		var rule_schedule_id = rule.ScheduleId || "";

		if (rule_schedule_id) {
			this.edited_rule = rule;
			caption_add_button = "<%=_T('Update Rule')%>";
		}
		else {
			this.edited_rule = null;
			caption_add_button = "<%=_T('Add Rule')%>";
		}
		$("#out_form_process_rules_add_update_rule_to_process").val(caption_add_button);

		this.form_paginator_destinations.initialize();
		this.form_paginator_actions.initialize();

		var process_rules = this.get_parent_process_rules();
		var $schedule = $("#out_form_process_rules_sch");

		// oculto del combo schedule las opciones, segun se edita o se crea una nueva regla
		this.parent.hide_combo_items($schedule, function(cbo_schedule_id)
		{
			var must_hide = false;
			if (self.edited_rule && rule_schedule_id === cbo_schedule_id) {
				return must_hide;
			}
			$.each(process_rules, function(_, process_rule)
			{
				if (process_rule.ScheduleId === cbo_schedule_id) {
					must_hide = true;
					return false; // break
				}
			});
			return must_hide;
		});

		$("#out_form_process_rules_sch").val(rule_schedule_id).trigger("change");
		$("#out_form_process_rules_desc").val(rule.Description || "");
	};

	form_outbound_engine_processes_rules.prototype.__new_grids = function()
	{
		const self = this;

		this.form_paginator_destinations = new inConcert.conf_form_paginator({
			paginator_name: "OUTBOUND_RULES_DESTINATIONS",
			owner: this,
			frm: this.parent,
			row_id: "Name",
			display_name: "DisplayName",
			rows_per_page: 500,
			show_at_least: 499,
			sortable: true,
			without_paginator: true,
			loading_waiting_msg: "<%=_T('Please wait, loading Rules Destination ...')%>",
			ids: {
				grid: "out_form_process_rules_destinations_list",
				paginator: "out_form_process_rules_destinations_paginator",
				not_results_found: "out_form_process_rules_destinations_not_results_found",
			},
			strings: {
				not_results_found: "<%=_T('No Rules Destinations found.')%>"
			},
			columns: [
				{id: "_number"    , name: "", type: "counter"},
				{id: "_select"    , name: "", type: "row_selector", is_checked: function(item) {
					if (!self.edited_rule) {
						// Cuando estoy creando una nueva regla, dejo todos los chk marcados
						return true;
					}
					var selected = false;
					$.each(self.edited_rule.OutboundProcessRuleDestinations || [], function(_, rule) {
						if (rule.Destination === item.Name) {
							selected = true;
							return false; // break
						}
					});
					return selected;
				}},
				{id: "DisplayName", name: "<%=_T('Destination Name')%>", get_value: function(item) {
					return self.get_grid_value(item, "DisplayName");
				}},
				{id: "Retries", name: "<%=_T('Max Retries Per Day')%>", type: "cell_input",
					get_value: function(item) {
						return self.get_grid_value(item, "Retries");
					}
				}
			],
			callbacks: {
				edit: function(rule) {
					self.on_open_rules_popup(rule);
				},
				load_items: function(page, callback) {
					//console.log("3 - grid load_items -> form_paginator_destinations");
					self.load_rules_destinations(page, callback);
				}
			}
		});

		this.form_paginator_actions = new inConcert.conf_form_paginator({
			paginator_name: "OUTBOUND_RULES_ACTIONS",
			owner: this,
			frm: this.parent,
			row_id: "Key",
			display_name: "DisplayValue",
			rows_per_page: 500,
			show_at_least: 499,
			without_paginator: true,
			loading_waiting_msg: "<%=_T('Please wait, loading Rule Actions ...')%>",
			ids: {
				grid: "out_form_process_rules_actions_list",
				paginator: "out_form_process_rules_actions_paginator",
				not_results_found: "out_form_process_rules_actions_not_results_found",
				new_item                 : "out_form_process_rules_actions_new",
				container_new_btn        : "out_form_process_rules_actions_list_container",
				//popup
				popup                    : "out_form_process_rules_actions_popup",
				popup_msg                : "out_form_process_rules_actions_popup_message",
				popup_create             : "out_form_process_rules_actions_popup_create",
				popup_create_and_continue: "out_form_process_rules_actions_popup_create_and_continue",
				popup_title              : "out_form_process_rules_actions_popup_title",
				popup_close              : "out_form_process_rules_actions_popup_close"
			},
			strings: {
				not_results_found         : "<%=_T('No Rule Actions found.')%>",
				can_remove                : "<%=_T('Do you want to delete Rule Action: {0} ?')%>",
				title_new                 : "<%=_T('New Action')%>",
				title_update              : "<%=_T('Update Action')%>",
				button_create             : "<%=_T('Create')%>",
				button_create_and_continue: "<%=_T('Create & Continue')%>"
			},
			columns: [
				{id: "_number"     , name: "", type: "counter"         },
				{id: "DisplayValue", name: "<%=_T('Current Actions')%>"},
				{id: "_action", name: "",
					system_action: { edit: true, erase: true }
				}
			],
			callbacks: {
				save: function(callback, must_create_and_contiune) {
					self.add_or_update_action(callback, must_create_and_contiune);
				},
				edit: function(action) {
					self.on_edit_action(action);
				},
				on_closed_popup: function() {
					self.on_closed_action_popup();
				},
				load_items: function(page, callback) {
					//console.log("4 - grid load_items -> form_paginator_actions");
					self.load_rules_actions(page, callback);
				}
			}
		});
	};

	form_outbound_engine_processes_rules.prototype.on_closed_action_popup = function()
	{
		this.is_new = null;
		this.current_item = null;
	};

	form_outbound_engine_processes_rules.prototype.__cleanup_ctrls_popup = function()
	{
		$("#out_form_process_rules_actions_contacting_result").val("");
		$("#out_form_process_rules_action_next_destination").val("");
		$("#out_form_process_rules_action_call_next_time").val("");
		$("#out_form_process_rules_action_call_next_period").val("");
		$("#out_form_process_rules_action_change_contact_priority").prop("checked", false);
		$("#out_form_process_rules_action_change_contact_priority_value").val("");
		$("#out_form_process_rules_action_change_contact_priority_type").val("");
		$("#out_form_process_rules_action_invalidte_address").prop("checked", false);
		$("#out_form_process_rules_action_affect_retries").prop("checked", false);
		$("#out_form_process_rules_action_execute_process").prop("checked", false);
		$("#out_form_process_rules_action_process").val("");
		$("#out_form_process_rules_action_arguments").val("");
		$("#out_form_process_rules_actions").val("cancel").trigger("change");
		$("#out_form_process_rules_action_change_contact_type").val("");
		$("#out_form_process_rules_actions_return_result").val("").focus();
	};

	/*
		Estas 4 funciones "__make_display_last_destination_attr", "__get_original_last_destination_attr", "__make_display_next_destination_attr", "__get_original_next_destination_attr"
			se encuentran por compatibilidad con 5.5, ya que del lado del server (en la BD) se almacena "" en los campos "LastDestination" y "NextDestination"
			cuando el valor es "ANY_DESTINATION" y "NEXT_ON_LIST", pero del lado del cliente se No se utilizan estos valores "" sino la palabra clave correspondiente.
	*/
	form_outbound_engine_processes_rules.prototype.__make_display_last_destination_attr = function(action)
	{
		var last_destination = action.LastDestination;
		if (last_destination === "") {
			last_destination = "ANY_DESTINATION";
		}
		return last_destination;
	};

	form_outbound_engine_processes_rules.prototype.__get_original_last_destination_attr = function(action)
	{
		var last_destination = action.LastDestination;
		if (last_destination === "ANY_DESTINATION") {
			last_destination = "";
		}
		return last_destination;
	};

	form_outbound_engine_processes_rules.prototype.__make_display_next_destination_attr = function(action)
	{
		var next_destination = action.NextDestination;
		if (next_destination === "") {
			next_destination = "NEXT_ON_LIST";
		}
		return next_destination;
	};

	form_outbound_engine_processes_rules.prototype.__get_original_next_destination_attr = function(action)
	{
		var next_destination = action.NextDestination;
		if (next_destination === "NEXT_ON_LIST") {
			next_destination = "";
		}
		return next_destination;
	};

	form_outbound_engine_processes_rules.prototype.on_edit_action = function(action)
	{
		this.is_new_action = action ? false: true;
		this.current_action = action;
		this.__cleanup_ctrls_popup();

		var $return_result = $("#out_form_process_rules_actions_return_result");
		var $contacting_result = $("#out_form_process_rules_actions_contacting_result");
		var $actions = $("#out_form_process_rules_actions");
		var $next_destination = $("#out_form_process_rules_action_next_destination");
		var $next_time = $("#out_form_process_rules_action_call_next_time");
		var $next_period = $("#out_form_process_rules_action_call_next_period");
		var $contact_priority = $("#out_form_process_rules_action_change_contact_priority");
		var $contact_priority_value = $("#out_form_process_rules_action_change_contact_priority_value");
		var $contact_priority_type = $("#out_form_process_rules_action_change_contact_priority_type");
		var $invalidte_address = $("#out_form_process_rules_action_invalidte_address");
		var $affect_retries = $("#out_form_process_rules_action_affect_retries");
		var $execute_process = $("#out_form_process_rules_action_execute_process");
		var $action_process = $("#out_form_process_rules_action_process");
		var $action_arguments = $("#out_form_process_rules_action_arguments");
		var $change_contact_type = $("#out_form_process_rules_action_change_contact_type");

		if (!this.is_new_action) {
			this.parent.disable_control($return_result, true);
			this.parent.disable_control($contacting_result, true);

			$return_result.val(action.Response);
			$contacting_result.val(this.__make_display_last_destination_attr(action));

			if (action.CancelInteraction === 1) {
				$actions.val("cancel").trigger("change");
			}
			else if (parseInt(action.SecondsToNextAction, 10) === 0) {
				$actions.val("postpone").trigger("change");
			}
			else {
				$actions.val("next").trigger("change");
				var time = parseInt(action.SecondsToNextAction, 10);
				var period = 1;
				if (time >= this.seconds_in_a_day && (time % this.seconds_in_a_day === 0)) {
					period = 3;
					time = (((time / 60) /60) / 24);
				}
				else if (time >= 60 && (time % 60 === 0)) {
					//segundos multiplos de 60 va en minutos
					period = 2;
					time = time / 60;
				}
				$next_time.val(time);
				$next_destination.val(this.__make_display_next_destination_attr(action));
				$next_period.val(period);
			}

			if (action.PriorityMod) {
				$contact_priority.prop("checked", true).trigger("change");
				$contact_priority_value.val(action.PriorityMod);
				$contact_priority_type.val(action.PriorityModIsPerc + 1);
			}

			$invalidte_address.prop("checked", action.InvalidateAddress === 1);
			$affect_retries.prop("checked", action.AffectRetries === 1);

			if (action.ProcessName) {
				$execute_process.prop("checked", true).trigger("change");
				$action_process.val(action.ProcessName);
				$action_arguments.val(action.ProcessArgs);
			}

			// en el combo: 3 "", 1 "Progressive", 2 "Not_Progressive"
			if (action.ChangeContactType === 1)  {
				$change_contact_type.val("Progressive");
			}
			else if (action.ChangeContactType === 0) {
				$change_contact_type.val("Not_Progressive");
			}
			else {
				$change_contact_type.val("");
			}

			$actions.focus();
		}
		else {
			this.parent.disable_control($return_result, false, true);
			this.parent.disable_control($contacting_result, false, true);
			$return_result.focus();
		}
	};

	form_outbound_engine_processes_rules.prototype.validate_action_to_update = function()
	{
		this.form_paginator_actions.hide_popup_msg();

		var response = {success: false, $ctrl: null, item: null, message: "<%=_T('Unable to Add or Update Action.')%>"};

		var $return_result = $("#out_form_process_rules_actions_return_result");
		var $contacting_result = $("#out_form_process_rules_actions_contacting_result");

		var action = {
			Response: $return_result.val(),
			LastDestination: $contacting_result.val()
		};
		action.Key = action.Response + action.LastDestination;

		var validation_result = inConcert.Utils.validate(action.Response, "required");
		if ( !validation_result.success ) {
			response.message = "<%=_T('Return result is required.')%>";
			response.$ctrl = $return_result;
			this.__show_popup_ctrl_err(response);
			return response;
		}

		validation_result = inConcert.Utils.validate(action.LastDestination, "required");
		if ( !validation_result.success ) {
			response.message = "<%=_T('Contacting is required.')%>";
			response.$ctrl = $contacting_result;
			this.__show_popup_ctrl_err(response);
			return response;
		}

		if (this.is_new_action) {
			var current_action = this.form_paginator_actions.find_row_data(action.Key);
			if (current_action) {
				response.message = FORMAT("<%=_T('Rule action already defined for response {0} and destination {1}.')%>", this.__response_as_string(action.Response), action.LastDestination);
				response.$ctrl = $return_result;
				this.__show_popup_ctrl_err(response);
				return response;
			}
		}

		var $actions = $("#out_form_process_rules_actions");
		var action_type = $actions.val();

		if (action_type === "cancel") {
			action.CancelInteraction = 1;
		}
		else if (action_type === "postpone") {
			//para postponer la interaccion simplemente seteo seconds to next action en cero
			action.CancelInteraction = 0;
			action.SecondsToNextAction = 0;
		}
		else if (action_type === "next") {
			var $next_destination = $("#out_form_process_rules_action_next_destination");

			validation_result = inConcert.Utils.validate($next_destination.val(), "required");
			if ( !validation_result.success ) {
				response.message = "<%=_T('Call next destination is required.')%>";
				response.$ctrl = $next_destination;
				this.__show_popup_ctrl_err(response);
				return response;
			}

			var $next_time = $("#out_form_process_rules_action_call_next_time");
			validation_result = inConcert.Utils.validate($next_time.val(), "required");
			if ( !validation_result.success ) {
				response.message = "<%=_T('Call next destination time is required.')%>";
				response.$ctrl = $next_time;
				this.__show_popup_ctrl_err(response);
				return response;
			}

			var $next_period = $("#out_form_process_rules_action_call_next_period");
			validation_result = inConcert.Utils.validate($next_period.val(), "required");
			if ( !validation_result.success ) {
				response.message = "<%=_T('Call next destination period is required.')%>";
				response.$ctrl = $next_period;
				this.__show_popup_ctrl_err(response);
				return response;
			}

			action.CancelInteraction = 0;
			action.NextDestination = $next_destination.val();
			var period = parseInt($next_period.val(), 10);
			var time = parseInt($next_time.val(), 10);
			action.OriginalTimePeriod = time;
			switch (period) {
				case 1: //seconds
				break;
				case 2: //minutes
					time = time * 60;
				break;
				case 3: //days
					time = time * this.seconds_in_a_day;
				break;
			}
			action.Period = period;
			action.SecondsToNextAction = time;
		}
		else {
			throw("error");
		}

		var $contact_priority = $("#out_form_process_rules_action_change_contact_priority");

		if ($contact_priority.prop("checked")) {
			var $contact_priority_value = $("#out_form_process_rules_action_change_contact_priority_value");
			validation_result = inConcert.Utils.validate($contact_priority_value.val(), "required");
			if ( !validation_result.success ) {
				response.message = "<%=_T('Change contact priority value is required.')%>";
				response.$ctrl = $contact_priority_value;
				this.__show_popup_ctrl_err(response);
				return response;
			}

			var $contact_priority_type = $("#out_form_process_rules_action_change_contact_priority_type");
			validation_result = inConcert.Utils.validate($contact_priority_type.val(), "required");
			if ( !validation_result.success ) {
				response.message = "<%=_T('Change contact priority type is required.')%>";
				response.$ctrl = $contact_priority_type;
				this.__show_popup_ctrl_err(response);
				return response;
			}
			action.PriorityMod = $contact_priority_value.val();
			action.PriorityModIsPerc = $contact_priority_type.val() - 1;
		}
		else {
			action.PriorityMod = null;
			action.PriorityModIsPerc = null;
		}

		var $invalidte_address = $("#out_form_process_rules_action_invalidte_address");
		var $affect_retries = $("#out_form_process_rules_action_affect_retries");
		var $execute_process = $("#out_form_process_rules_action_execute_process");

		action.InvalidateAddress = $invalidte_address.prop("checked") === true ? 1 : 0;
		action.AffectRetries = $affect_retries.prop("checked") === true ? 1 : 0;

		if ($execute_process.prop("checked")) {
			var $action_process = $("#out_form_process_rules_action_process");
			validation_result = inConcert.Utils.validate($action_process.val(), "required");
			if ( !validation_result.success ) {
				response.message = "<%=_T('Process to execute is required.')%>";
				response.$ctrl = $action_process;
				this.__show_popup_ctrl_err(response);
				return response;
			}
			var $action_arguments = $("#out_form_process_rules_action_arguments"); 
			action.ProcessName = $action_process.val();
			action.ProcessArgs = $action_arguments.val();
		}

		//en el combo: "", "Progressive", "Not_Progressive"
		const $change_contact_type = $("#out_form_process_rules_action_change_contact_type");
		let contact_type = $change_contact_type.val();

		if (contact_type === "Progressive" ) {
			action.ChangeContactType = 1;
		}
		else if(contact_type === "Not_Progressive" ) {
			action.ChangeContactType = 0;
		}
		else {
			action.ChangeContactType = null;
		}

		action = this.__make_semantic_action(action);

		response.success = true;
		response.message = "";
		response.item = action;

		return response;
	};

	form_outbound_engine_processes_rules.prototype.add_or_update_action = function(callback)
	{
		callback = callback || $.noop;

		var response = this.validate_action_to_update();
		if (!response.success) {
			callback(false);
			return;
		}

		this.__cleanup_ctrls_popup();
		this.current_action = null;

		var item = response.item;
		var msg_response = {msg: "", type: ""};

		msg_response.msg = this.is_new_action ? "<%=_T('Action was successfully added.')%>" : "<%=_T('Action was successfully updated.')%>";
		msg_response.type = "Info";

		callback(true, item, msg_response);
	};

	form_outbound_engine_processes_rules.prototype.on_open_rules_popup = function(action)
	{
		this.is_new_action = action ? false: true;
		this.current_action = action;

		$("#out_form_process_rules_actions_popup_name").focus().val(this.is_new_action ? "" : action.name);
	};

	form_outbound_engine_processes_rules.prototype.get_grid_value = function(item, field_id)
	{
		var value = {
			original_value: "",
			value         : "",
			search_value  : ""
		};

		var item_value = item[field_id] || "";
		if (item_value === undefined) {
			return value;
		}

		value.original_value = item_value;
		value.value = item_value;
		value.search_value = item_value;

		if (field_id === "DisplayName") {
			value.value = item.DisplayName || item.Name;
			value.search_value = value.value;
		}
		else if (field_id === "Retries") {
			value.value = item.Retries || item.TopRetries || "3";
			value.search_value = value.value;
			if (this.edited_rule) {
				$.each(this.edited_rule.OutboundProcessRuleDestinations || [], function(_, rule) {
					if (rule.Destination === item.Name) {
						value.value = rule.Retries || rule.TopRetries || value.value;
						value.search_value = value.value;
						return false; // break
					}
				});
			}
		}

		return value;
	};

	//devuelve la traduccion de un destino tanto sea builtin (ANY_DESTINATION) como dado de alta por el usuario
	form_outbound_engine_processes_rules.prototype.__get_destination_translation = function(destination)
	{
		if (this.builtin_destinations[destination]) {
			return this.builtin_destinations[destination];
		}
		var list = this.cached_rules_destinations || [];
		for (var i = 0; i < list.length; i++) {
			var item = list[i];
			if (item.Name === destination) {
				if (item.DisplayName.length > 0) {
					return item.DisplayName;
				}
				else {
					return item.Name;
				}
			}
		}
		return destination;
	};

	form_outbound_engine_processes_rules.prototype.__response_as_string = function(responseCode)
	{
		var list = this.cached_rules_responses_by_channel || [];
		for (var i = 0; i < list.length; i++) {
			var item = list[i];
			var numericValue = parseInt(item.Value, 10);
			var numericCode = parseInt(responseCode, 10);
			if (numericValue === numericCode) {
				return item.DisplayName;
			}
		}
		return "";
	};

	form_outbound_engine_processes_rules.prototype.__period_as_string = function(periodid)
	{
		var period = this.period_list[periodid.toString()];
		if (period) {
			return period.name;
		}
		// switch(periodid) {
			// case 1:
				// return "<%=_T('Seconds')%>";
			// case 2:
				// return "<%=_T('Minutes')%>";
			// case 3:
				// return "<%=_T('Days')%>";
		// }
		//return "";
		return period;
	};

	form_outbound_engine_processes_rules.prototype.__make_semantic_action = function(action)
	{
		var conditionString;
		var actionString = "";
		var display_last_destination = this.__make_display_last_destination_attr(action);
		var display_next_destination = this.__make_display_next_destination_attr(action);

		if (action.CancelInteraction === 0) {
			if (action.SecondsToNextAction === 0) {
				conditionString = FORMAT("<%=_T('When the interaction returns {0} while contacting {1}, postpone the contact.')%>",
												this.__response_as_string(action.Response), this.__get_destination_translation(display_last_destination));
			}
			else {
				//quiso llamar
				if (display_last_destination === "ANY_DESTINATION") {
					conditionString = FORMAT("<%=_T('When the interaction returns {0}, call {1} in {2} {3}.')%>",
											this.__response_as_string(action.Response), this.__get_destination_translation(display_next_destination), action.OriginalTimePeriod, this.__period_as_string(action.Period));
				}
				else {
					conditionString = FORMAT("<%=_T('When the interaction returns {0} while contacting {1}, call {2} in {3} {4}.')%>",
									this.__response_as_string(action.Response), 
									this.__get_destination_translation(display_last_destination),
									this.__get_destination_translation(display_next_destination),
									action.OriginalTimePeriod, this.__period_as_string(action.Period));
				}
			}
		}
		else {
			//quiso cancelar
			if (display_last_destination === "ANY_DESTINATION") {
				conditionString = FORMAT("<%=_T('When the interaction returns {0}, cancel interaction.')%>", this.__response_as_string(action.Response));
			}
			else {
				conditionString = FORMAT("<%=_T('When the interaction returns {0} while contacting {1}, cancel interaction.')%>", this.__response_as_string(action.Response), this.__get_destination_translation(display_last_destination));
			}
		}

		if (action.InvalidateAddress ) {
			if (action.ProcessName !== undefined && action.ProcessName !== null) {
				if (action.ProcessArgs !== "" && action.ProcessArgs !== null) {
					actionString = FORMAT("<%=_T('Invalidate address and call process {0} with arguments {1}.')%>", action.ProcessName, action.ProcessArgs);
				}
				else {
					actionString = FORMAT("<%=_T('Invalidate address and call process {0} with no arguments.')%>", action.ProcessName);
				}
			}
			else {
				actionString = "<%=_T('Address will be invalidated.')%>";
			}
		}
		else {
			if (action.ProcessName !== undefined && action.ProcessName !== null) {
				if (action.ProcessArgs !== "" && action.ProcessArgs !== null) {
					actionString = FORMAT("<%=_T('Call process {0} with arguments {1}.')%>", action.ProcessName, action.ProcessArgs);
				}
				else {
					actionString = FORMAT("<%=_T('Call process {0} with no arguments.')%>", action.ProcessName);
				}
			}
		}
		var ruleString = conditionString + actionString;

		action.Key = action.Response + display_last_destination;
		action.DisplayValue = ruleString;

		return action;
	};

	form_outbound_engine_processes_rules.prototype.load_rules_actions = function(_, callback)
	{
		const self = this;

		callback = callback || $.noop;

		var filter = function()
		{
			var filtered_list = [];

			var add = function(action)
			{
				action = self.__make_semantic_action(action);
				filtered_list.push(action);
			};

			var rule_actions = self.edited_rule ? self.edited_rule.OutboundProcessRuleResponses : self.defined_rule_responses;
			rule_actions = inConcert.Utils.cloneArray(rule_actions || []);

			$.each(rule_actions, function(_, response)
			{
				if (response.SecondsToNextAction > 0) {
					//tengo que pasar los segundos a humano que es lo que originalmente el flaco especifico
					//aca parto de la base que si es multiplo de dias o de minutos especifico eso (puede pasar que si especifico 120 segundos se le levante despues como 2 minutos)
					var period, originalTimePeriod;
					if (parseInt(response.SecondsToNextAction % self.seconds_in_a_day, 10) === 0) {
						originalTimePeriod = parseInt(response.SecondsToNextAction / self.seconds_in_a_day, 10);
						period = 3;
					}
					else {
						if (parseInt(response.SecondsToNextAction % 60, 10) === 0) {
							originalTimePeriod = parseInt(response.SecondsToNextAction / 60, 10);
							period = 2;
						}
						else {
							originalTimePeriod = parseInt(response.SecondsToNextAction, 10);
							period = 1;
						}
					}
					response.OriginalTimePeriod = originalTimePeriod;
					response.Period = period;
				}
				add(response);
			});

			return filtered_list;
		};

		var list = filter();
		callback(true, list);
	};

	form_outbound_engine_processes_rules.prototype.load_rules_destinations = function(_, callback)
	{
		var cached_rules_destinations = inConcert.Utils.cloneArray(this.cached_rules_destinations);

		if (this.edited_rule) {
			var process_rule_destinations = this.edited_rule.OutboundProcessRuleDestinations || [];

			$.each(process_rule_destinations, function(_, rule_destination)
			{
				//voy a ubicar el destino en la lista por default y ponerlo en el orden que decia la BD que va...
				for (var j = 0; j < cached_rules_destinations.length; j++) {
					if (cached_rules_destinations[j].Name === rule_destination.Destination) {
						var newindex = rule_destination.Priority - 1;
						//swapping al lugar donde va...
						var previous = cached_rules_destinations[newindex];
						cached_rules_destinations[newindex] = cached_rules_destinations[j];
						cached_rules_destinations[newindex].Retries = rule_destination.TopRetries;
						cached_rules_destinations[newindex].Selected = "1";
						cached_rules_destinations[j] = previous;
					}
				}
			});
		}

		callback(true, cached_rules_destinations);
	};

	form_outbound_engine_processes_rules.prototype.attach_load_request = function(request)
	{
		const self = this;

		this.__fill_period();
		this.__fill_priority_value();
		this.__fill_priority_type();
		this.__fill_change_contact_types();

		var outbound_engine_api = inConcert.app.getOutboundEngineAPI();

		// xxxxx
		request.push(
			{
				request_function: function(success) {
					return outbound_engine_api.processLoadAvailableSchedulers(success);
				},
				success: function(list) {
					self.__fill_scheduler(list);
				}
			},
			{
				request_function: function(success) {
					return outbound_engine_api.systemRulesLoadDestinations(success);
				},
				success: function(list) {
					self.__fill_destinations(list);
				}
			},
			{
				request_function: function(success) {
					return outbound_engine_api.systemRulesLoadResponsesByChannel(success);
				},
				success: function(list) {
					self.__fill_actions_result(list);
				}
			},
			{
				request_function: function(success) {
					return outbound_engine_api.processLoadAdministrationProcessess(success);
				},
				success: function(list) {
					self.__fill_administration_processes(list);
				}
			},
			{
				request_function: function(success) {
					var fn = function(result_fn) {
						result_fn(true, [
							{Name: "cancel"  , DisplayName: "<%=_T('Cancel')%>"               },
							{Name: "next"    , DisplayName: "<%=_T('Call Next Destination')%>"},
							{Name: "postpone", DisplayName: "<%=_T('Postpone Contact')%>"     }
						]);
					};
					return fn(success);
				},
				success: function(list) {
					self.__fill_actions_options(list);
				}
			}
		);
	};

	form_outbound_engine_processes_rules.prototype.__fill_scheduler = function(list)
	{
		var $combo = $("#out_form_process_rules_sch");
		this.parent.fill_combo($combo, true /* is_blank_first_item */, list, "Id" /* attr_id */, "Id" /* attr_name */);
	};

	form_outbound_engine_processes_rules.prototype.__fill_actions_result = function(list)
	{
		var $combo = $("#out_form_process_rules_actions_return_result");
		this.parent.fill_combo($combo, true /* is_blank_first_item */, list, "Value" /* attr_id */, "DisplayName" /* attr_name */);
	};

	form_outbound_engine_processes_rules.prototype.__fill_destinations = function(list)
	{
		var $contacting_result = $("#out_form_process_rules_actions_contacting_result");
		var contacting_results = inConcert.Utils.cloneArray(list);
		// lo agrego primero en la lista
		contacting_results.unshift({Name: "ANY_DESTINATION", DisplayName: this.builtin_destinations.ANY_DESTINATION});
		this.parent.fill_combo($contacting_result, true /* is_blank_first_item */, contacting_results, "Name" /* attr_id */, "DisplayName" /* attr_name */);

		var $next_destination = $("#out_form_process_rules_action_next_destination");
		var next_destinations = inConcert.Utils.cloneArray(list);
		// lo agrego primero en la lista
		next_destinations.unshift({Name: "SAME_DESTINATION", DisplayName: this.builtin_destinations.SAME_DESTINATION});
		next_destinations.unshift({Name: "SAME_ADDRESS", DisplayName: this.builtin_destinations.SAME_ADDRESS});
		next_destinations.unshift({Name: "NEXT_ON_LIST", DisplayName: this.builtin_destinations.NEXT_ON_LIST});
		this.parent.fill_combo($next_destination, true /* is_blank_first_item */, next_destinations, "Name" /* attr_id */, "DisplayName" /* attr_name */);
	};

	form_outbound_engine_processes_rules.prototype.__fill_actions_options = function(list)
	{
		var $combo = $("#out_form_process_rules_actions");
		this.parent.fill_combo($combo, false /* is_blank_first_item */, list, "Name" /* attr_id */, "DisplayName" /* attr_name */);
	};

	form_outbound_engine_processes_rules.prototype.__fill_period = function()
	{
		var $combo = $("#out_form_process_rules_action_call_next_period");
		this.parent.fill_combo($combo, true /* is_blank_first_item */, this.period_list, "id" /* attr_id */, "name" /* attr_name */);
	};

	form_outbound_engine_processes_rules.prototype.__fill_change_contact_types = function()
	{
		var $combo = $("#out_form_process_rules_action_change_contact_type");
		this.parent.fill_combo($combo, true /* is_blank_first_item */, this.change_contact_types_list, "id" /* attr_id */, "name" /* attr_name */);
	};

	form_outbound_engine_processes_rules.prototype.__fill_priority_value = function()
	{
		var $combo = $("#out_form_process_rules_action_change_contact_priority_value");
		var list = [];
		//de -100 a 100 saltando de a 10
		for (var i = -100; i <= 100; i += 10) {
			if (i !== 0) {
				list.push({id: i, name: i});
			}
		}
		this.parent.fill_combo($combo, true /* is_blank_first_item */, list, "id" /* attr_id */, "name" /* attr_name */);
	};

	form_outbound_engine_processes_rules.prototype.__fill_priority_type = function()
	{
		var $combo = $("#out_form_process_rules_action_change_contact_priority_type");
		this.parent.fill_combo($combo, true /* is_blank_first_item */, this.priority_type, "id" /* attr_id */, "name" /* attr_name */);
	};

	form_outbound_engine_processes_rules.prototype.__fill_administration_processes = function(list)
	{
		var $combo = $("#out_form_process_rules_action_process");
		this.parent.fill_combo($combo, true /* is_blank_first_item */, list, "Id" /* attr_id */, "Description" /* attr_name */);
	};

	form_outbound_engine_processes_rules.prototype.__get_selected_rules_destinations = function()
	{
		var rules = [];
		var selected_rules = this.form_paginator_destinations.get_selected_items();
		$.each(selected_rules, function(i, selected_rule)
		{
			var rule = {
				Priority   : i + 1,
				Destination: selected_rule.Name,
				TopRetries : selected_rule.Retries || "3"
			};
			rules.push(rule);
		});
		return rules;
	};

	form_outbound_engine_processes_rules.prototype.__get_selected_rules_actions = function()
	{
		const self = this;

		var rule_actions = this.form_paginator_actions.get_items();
		$.each(rule_actions, function(_, rule_action)
		{
			rule_action.LastDestination = self.__get_original_last_destination_attr(rule_action);
			rule_action.NextDestination = self.__get_original_next_destination_attr(rule_action);
		});

		return rule_actions;
	};

	form_outbound_engine_processes_rules.prototype.__get = function()
	{
		var rule = {
			ScheduleId: $("#out_form_process_rules_sch").val(),
			Description: $("#out_form_process_rules_desc").val(),
			OutboundProcessRuleDestinations: this.__get_selected_rules_destinations(),
			OutboundProcessRuleResponses: this.__get_selected_rules_actions()
		};

		if (!this.is_new_rule()) {
			rule.id = this.edited_rule.id; // el id interno que genera la grilla.
			rule.Priority = this.edited_rule.Priority;
		}
		return rule;
	};

	form_outbound_engine_processes_rules.prototype.hide_all_errors = function()
	{
		this.parent.hide_messages();
	};

	form_outbound_engine_processes_rules.prototype.__validate = function()
	{
		this.hide_all_errors();

		var rule = this.__get();

		var response = inConcert.Utils.validate(rule.ScheduleId, "required");
		if (!response.success) {
			response.message = "<%=_T('Unable to add or update Rule. You must add a name to rule.')%>";
			response.$ctrl = $("#out_form_process_rules_sch");
			return response;
		}

		response = inConcert.Utils.validate(rule.Description, "required");
		if (!response.success) {
			response.message = "<%=_T('Unable to add or update Rule. Description is required.')%>";
			response.$ctrl = $("#out_form_process_rules_desc");
			return response;
		}

		if (rule.OutboundProcessRuleDestinations.length === 0) {
			response.success = false;
			response.message = "<%=_T('Unable to add or update Rule. You must select at least one destination for the rule.')%>";
			response.$ctrl = $("#out_form_process_rules_destinations_list");
			return response;
		}

		if (rule.OutboundProcessRuleResponses.length === 0) {
			response.success = false;
			response.message = "<%=_T('You must configure at least one response action for the rule.')%>";
			response.$ctrl = $("#out_form_process_rules_actions_list");
			return response;
		}

		response.rule = rule;
		return response;
	};

	form_outbound_engine_processes_rules.prototype.__show_ctrl_err = function(err_info)
	{
		if (!err_info) {
			return;
		}
		this.parent.show_validation_error(err_info.$ctrl, err_info.message);
	};

	form_outbound_engine_processes_rules.prototype.__show_popup_ctrl_err = function(err_info)
	{
		if (!err_info) {
			return;
		}
		const $err_msg_container = $("#out_form_process_rules_actions_popup_message");
		this.parent.show_validation_error(err_info.$ctrl, err_info.message, undefined, undefined, undefined, $err_msg_container);
	};

	form_outbound_engine_processes_rules.prototype.__add_update_rule = function()
	{
		const self = this;

		this.parent.hide_messages();
		var result = this.__validate();
		if (!result.success) {
			this.__show_ctrl_err(result);
			return;
		}
		var rule = result.rule;

		this.parent.add_update_rule(rule, function() {
			self.__dispose();
		});
	};

	form_outbound_engine_processes_rules.prototype.is_new_rule = function()
	{
		return this.edited_rule === null;
	};

	form_outbound_engine_processes_rules.prototype.__dispose = function()
	{
		this.edited_rule = null;
	};

	form_outbound_engine_processes_rules.prototype.__on_change_rule_action = function(action)
	{
		var $call_section = $("#out_form_process_rules_action_call_section");
		var $contact_priority = $("#out_form_process_rules_action_change_contact_priority");

		switch(action) {
			case "cancel":
				$contact_priority.prop("checked", false);
				$contact_priority.prop("disabled", false);
				$call_section.hide();
			break;
			case "next":
				$contact_priority.prop("disabled", false);
				$call_section.show();
			break;
			case "postpone":
				$contact_priority.prop("checked", true);
				$contact_priority.prop("disabled", true);
				$call_section.hide();
			break;
		}

		this.__change_rule_change_contact_priority();
		this.__change_rule_execute_process();
	};

	form_outbound_engine_processes_rules.prototype.__change_rule_execute_process = function()
	{
		var $action_process = $("#out_form_process_rules_action_process");
		var $action_arguments = $("#out_form_process_rules_action_arguments");
		var $execute_process = $("#out_form_process_rules_action_execute_process");

		if ($execute_process.prop("checked")) {
			this.parent.disable_control($action_process, false, true);
			this.parent.disable_control($action_arguments, false, false);
		}
		else {
			this.parent.disable_control($action_process, true);
			this.parent.disable_control($action_arguments, true);
		}
	};

	form_outbound_engine_processes_rules.prototype.__change_rule_change_contact_priority = function()
	{
		var $contact_priority_value = $("#out_form_process_rules_action_change_contact_priority_value");
		var $contact_priority_type = $("#out_form_process_rules_action_change_contact_priority_type");
		var $contact_priority = $("#out_form_process_rules_action_change_contact_priority");

		if ($contact_priority.prop("checked")) {
			this.parent.disable_control($contact_priority_value, false, true);
			this.parent.disable_control($contact_priority_type, false, false);
		}
		else {
			this.parent.disable_control($contact_priority_value, true);
			this.parent.disable_control($contact_priority_type, true);
		}
	};

	form_outbound_engine_processes_rules.prototype.set_events = function()
	{
		const self = this;

		// eventos de la grilla con paginado
		this.form_paginator_destinations.set_events();
		this.form_paginator_actions.set_events();

		$("#out_form_process_rules_add_update_rule_to_process")
			.off("click")
			.on("click", function() {
				self.__add_update_rule();
			});

		$("#out_form_process_rules_back_process_section")
			.off("click")
			.on("click", function() {
				self.parent.show_process_section();
			});

		// eventos del popup de edicion de acciones de la regla
		$("#out_form_process_rules_actions")
			.off("change")
			.on("change", function() {
				var action = $(this).val();
				self.__on_change_rule_action(action);
			});

		$("#out_form_process_rules_action_execute_process")
			.off("change")
			.on("change", function() {
				self.__change_rule_execute_process();
			});

		$("#out_form_process_rules_action_change_contact_priority")
			.off("change")
			.on("change", function() {
				self.__change_rule_change_contact_priority();
			});

		$("#out_form_process_rules_sch")
			.off("change")
			.on("change", function() {
				var sch_id = $(this).val();
				var is_disabled = sch_id.length === 0;
				$("#out_form_process_rules_edit_sch").prop("disabled", is_disabled);
			});
	};

	//---------------------------------------------------------------------
	/**
	Class BatchInfoPopup.
	*/
	function BatchInfoPopup (params)
	{
		BatchInfoPopup.uber.constructor.call( this );
		this.clickEventName = this.clickEventName + "_batch_filter";
		this.ownerId = "out_batch_filter_popup_container";
		this.waitingId = this.ownerId + "_waitign_id";
		this.messageId = this.ownerId + "_message_id";
		this.closeMessageId = this.ownerId + "_close_message_id";
		this.batchId = this.ownerId + "_batch_id";
		this.statusContainerId = this.ownerId + "_status_container_id";
		this.statusListId = this.ownerId + "_status_list_id";
		this.closeId = this.ownerId + "_close";
		this.params = params;
		this.canReposition = true;
	}

	inConcert.Utils.inherit(BatchInfoPopup, inConcert.PopupWindow);

	/*******************************************************************
	 * Metodo que crea el markup contenedor de la ventana
	 */
	BatchInfoPopup.prototype.createMarkup = function() {

		$("#" + this.ownerId).remove();

		//create the markup everytime
		var $node = $("".concat(
			"<div id='" + this.ownerId + "' class='systemPopWindow windowContactStatusBatches windowContainer' style='display:none'>",

				"<div id='", this.waitingId, "' class='windowWaitingContainer' hidden>",
					"<div message='true' class='windowWaitingMessage'></div>",
					"<div class='windowWaitingAnimation'></div>",
					"<div class='windowWaitingOverlay'></div>",
				"</div>",

				"<div class='windowTop'>",
					"<div class='windowTitle'>", "<%=_T('Contacts Status')%>", "</div>",
				"</div>",

				"<div id='", this.messageId, "' class='windowContainerMessages messageError' style='display:none' hidden>",
					"<a id='", this.closeMessageId, "' class='windowMessagesClose' href='javascript:void(0);'>", "<%=_T('Close')%>", "</a>",
					"<div class='windowMessagesText'>",
						"<label class='windowMessagesTextAux'></label>",
					"</div>",
				"</div>",

				"<div class='windowMiddle' style='display:none'>",
					"<div class=''></div>",
				"</div>",

				"<div class='windowMiddle'>",
					"<div class='card'>",
						"<div class='card-block'>",
							"<div class='row'>",
								"<div class='col-12'>",
									"<div class='form-group'>",
										"<label for='", this.batchId, "' class=''>", "<%=_T('Batch Name')%>", "<%=_T(':')%>", "</label>",
										"<input id='", this.batchId, "' class='form-control formField-ds' type='text' disabled='true' />",
									"</div>",
								"</div>",
							"</div>",
							"<div class='row'>",
								"<div class='col-12'>",
									"<div id='", this.statusContainerId, "' hidden class=''>",
										"<div class='tableContainerBox'>",
											"<table id='", this.statusListId, "' border='0' cellspacing='0' class='viewGrid'>",
												"<thead class='viewGridHeader'>",
													"<tr>",
														"<th>", "<%=_T('Contact State')%>", "</th>",
														"<th align='right'>", "<%=_T('Total')%>", "</th>",
													"</tr>",
												"</thead>",
												"<tbody class='viewGridBody'>",
												"</tbody>",
											"</table>",
										"</div>",
									"</div>",
								"</div>",
							"</div>",
							"<div class='row'>",
								"<div class='col-12'>",
									"<div class='inlineMessagesContainer messageInfo'>",
									"<%=_T('Misconfiguration : active or rescheduled contacts  with reserved agent out of campaign.')%>" ,
								"</div>",
							"</div>",
						"</div>",
						"</div>",
					"</div>",
				"</div>",
				"<div class='windowBottom d-flex justify-content-end'>",
					"<button id='", this.closeId, "' class='pageButtonGeneric buttonColorWhite'>",
						"<%=_T('Close')%>",
					"</button>",
				"</div>",
			"</div>"));
		$("body").append($node);

		return $node;
	};

	/*******************************************************************
	 *
	 */
	BatchInfoPopup.prototype.initialize = function()
	{
		this.data.container = this.createMarkup();
		this.setEvents();
	};

	BatchInfoPopup.prototype.show_waiting = function()
	{
		var message = "<%=_T('Loading, wait moment please')%>";
		var $waiting = $("#"  + this.waitingId);
		$waiting.find("[message='true']").text(message);
		$waiting.show();
	};

	/*******************************************************************
	* Metodo que xxx
	*/
	BatchInfoPopup.prototype.hide_waiting = function()
	{
		$("#" + this.waitingId).hide();
		this.setPosition();
	};

	BatchInfoPopup.prototype.show_status_message = function(text, type, closeDisabled, timeout_ms )
	{
		var $message = $("#" + this.messageId);
		$message.removeClass().addClass("windowContainerMessages").addClass("message" + type);
		$message.show().find(".windowMessagesTextAux").text(text);
		if (timeout_ms) {
			window.setTimeout(function() {
				$message.hide();
			}, timeout_ms);
		}
		var $messageClose = $("#" + this.closeMessageId);
		if (closeDisabled) {
			$messageClose.hide();
		}
		else {
			$messageClose.show();
		}
	};

	/*******************************************************************
	* funcion que oculta mensaje
	*/
	BatchInfoPopup.prototype.hide_status_message = function() {
		var $message = $("#" + this.messageId);
		$message.hide();
	};

	BatchInfoPopup.prototype.make_contacts_status = function(contacts_status)
	{
		var $container = $("#" + this.statusContainerId);

		if (contacts_status.length === 0) {
			$container.hide();
			this.show_status_message("<%=_T('There are no contact statuses.')%>", "Info", true);
			return;
		}

		var $contacts_list = $("#" + this.statusListId);
		$.each(contacts_status, function(_, contact_status) {
			var $contact_status = $("<tr><td>" + contact_status.status + "</td><td align='right'>" + contact_status.total + "</td></tr>");
			$contacts_list.append($contact_status);
		});

		$container.show();
	};

	BatchInfoPopup.prototype.ready = function()
	{
		const self = this;

		this.show_waiting();

		var processId = this.params.processId;
		var batch = this.params.batch;

		$("#" + this.batchId).val(batch.BatchId);

		var outbound_engine_api = inConcert.app.getOutboundEngineAPI();

		outbound_engine_api.processLoadBatchContactStatus(
			{processId: processId, batchId: batch.BatchId},
			function(success, contacts_status)
			{
				// pequeno waiting para dar feedback
				window.setTimeout(function() {
					if (success) {
						self.make_contacts_status(contacts_status);
					}
					else {
						$("#" + self.statusListId).hide();
						self.show_status_message("<%=_T('Error loading contact status.')%>", "Error", true);
					}
					self.hide_waiting();
				}, 500);
			}
		);
	};

	BatchInfoPopup.prototype.close = function(params) {
		this.data.container.remove();
		this.finalize(params);
	};

	/*******************************************************************
	 * Metodo que setea eventos de los controles
	 */
	BatchInfoPopup.prototype.setEvents = function()
	{
		const self = this;

		var $filter_id =  $("#" + this.filterId);
		//---------------
		$("#" + this.saveId)
			.unbind("click")
			.click(function() {
				var filter_id = $filter_id.val();
				self.parent_save(filter_id, function(success) {
					if (!success) {
						$filter_id.focus();
						return;
					}
					self.close();
				});
			});
		//---------------
		$("#" + this.closeId).unbind("click").click(function() {
			self.close();
		});
	};

	//---------------------------------------------------------------------
	// form process class
	function form_outbound_engine_processes_allegro(params)
	{
		var success = form_outbound_engine_processes_allegro.uber.constructor.call(this, params);
		if (!success) {
			return;
		}

		this.new_title = "<%=_T('New Outbound Processes')%>";
		this.update_title = "<%=_T('Update Outbound Processes {0}')%>";
		this.view_title = "<%=_T('View Outbound Processes {0}')%>";
		this.tagline = "<%=_T('Tagline form Outbound Processes!!!')%>";

		this.html_file_name = "/inconcert/apps/outbound_engine/processes/frm/render/";
		this.get_uri =        "/inconcert/apps/outbound_engine/processes/get/";
		this.include_save_and_continue = false;

		this.__entity_id = "Id";

		this.form_paginator_rules = null;
		this.form_paginator_batches = null;
		this.count_pending_grid = 0;

		this.__new_grids();

		this.frm_rules = new form_outbound_engine_processes_rules({
			parent: this
		});

		this.frm_batches = new form_outbound_engine_processes_batches({
			parent: this
		});

		// flag "initialized" utilizado para que no quede en loop el frm cuando se esta creando un nuevo "process",
		//  esto sucede asi, porque se tuvo que sobreescribir la funcion "ready_for_save".
		this.initialized = false;
		this.batch_info_popup = null;

		return true;
	}

	inConcert.Utils.inherit(form_outbound_engine_processes_allegro, inConcert.conf_form);

	if (typeof inConcert.form_outbound_engine_processes_allegro !== "function") {
		inConcert.form_outbound_engine_processes_allegro = form_outbound_engine_processes_allegro;
	}

	form_outbound_engine_processes_allegro.prototype.initialize = $.noop;
	form_outbound_engine_processes_allegro.prototype.dispose = function() {
		this.initialized = false;
	};

	form_outbound_engine_processes_allegro.prototype.get = function()
	{
		var process =  {
			Id              : $("#out_form_process_id").val(),
			EngineService      : $("#out_form_process_engine_service").val(),
			Type          : $("#out_form_process_engine_type").val(),
			MaxRetriesPerDay  : $("#out_form_process_max_retries_per_day").val(),
			MaxRetriesPerContact : $("#out_form_process_max_retries_per_contact").val(),
			TopQueuedInteractions     : 15, // TODO: Ver!!!  esto no se de donde sale
			ContactWindowSize: $("#out_form_process_contact_window_size").val(),
			Status: undefined,
			OutboundProcessRules: this.__get_rules(),
			OutboundProcessBatchs: this.__get_batches()
		};
		return process;
	};

	form_outbound_engine_processes_allegro.prototype.__get_rules = function()
	{
		var rules = this.form_paginator_rules.get_items();
		rules = inConcert.Utils.sortArray(rules, "Priority", "asc");
		return rules;
	};

	form_outbound_engine_processes_allegro.prototype.__get_batches = function()
	{
		var batches = this.form_paginator_batches.get_items();
		return batches;
	};

	form_outbound_engine_processes_allegro.prototype.set = function(success, process)
	{
		if (this.initialized) {
			return;
		}

		this.form_paginator_rules.initialize();
		this.form_paginator_batches.initialize();
		this.frm_batches.load_information();

		if (!success) {
			return;
		}
		$("#out_form_process_id").val(process.Id || "");
		$("#out_form_process_engine_service").val(process.EngineService || "");
		$("#out_form_process_engine_type").val(process.Type || "");
		$("#out_form_process_max_retries_per_day").val(process.MaxRetriesPerDay || 5);
		$("#out_form_process_max_retries_per_contact").val(process.MaxRetriesPerContact || 12);
		$("#out_form_process_contact_window_size").val(process.ContactWindowSize || 12);

		var dom_container_id = "out_form_process_batches_popup";
		this.create_spinners(dom_container_id);
	};

	form_outbound_engine_processes_allegro.prototype.__open_batch_info_popup = function($btn_parent, batch)
	{
		if (this.batch_info_popup) {
			this.batch_info_popup.close();
			this.batch_info_popup = null;
		}

		var process = this.get_entity_instance();
		this.batch_info_popup = new BatchInfoPopup({processId: process.Id, batch: batch});
		this.batch_info_popup.show({
			parentNode: $btn_parent,
			owner_object: this
		});
	};

	form_outbound_engine_processes_allegro.prototype.__new_grids = function()
	{
		const self = this;

		this.count_pending_grid = 0;

		this.form_paginator_rules = new inConcert.conf_form_paginator({
			paginator_name: "OUTBOUND_RULES",
			owner: this,
			frm: this,
			display_name: "ScheduleId",
			rows_per_page: 500,
			show_at_least: 499,
			without_paginator: true,
			sortable: true,
			loading_waiting_msg: "<%=_T('Please wait, loading Rules ... ')%>",
			on_sorted: function() {
				self.__reorder_rules();
			},
			ids: {
				grid: "out_form_process_rules_list",
				paginator: "out_form_process_rules_paginator",
				not_results_found: "out_form_process_rules_not_results_found"
			},
			strings: {
				not_results_found: "<%=_T('You have no rules configured')%>",
				can_remove : "<%=_T('Do you want to delete Rule: {0} ?')%>"
			},
			columns: [
				{id: "Priority"   , name: "<%=_T('Priority')%>", get_value: function(item) {
					return self.get_grid_value(item, "Priority");
				}},
				{id: "ScheduleId"   , name: "<%=_T('Schedule')%>"       },
				{id: "Description", name: "<%=_T('Description')%>"    },
				{id: "_action", name: "",
					system_action: { edit: true, erase: true }
				}
			],
			callbacks: {
				edit: function(rule) {
					self.on_edit_rule_popup(rule);
				},
				load_items: function(page, callback) {
					//console.log("5 - grid load_items -> form_paginator_rules");
					self.count_pending_grid = self.count_pending_grid + 1;
					self.load_rules(page, callback);
				},
				on_loaded_items: function(callback) {
					self.__on_loaded_grid(callback);
				}
			}
		});

		this.form_paginator_batches = new inConcert.conf_form_paginator({
			paginator_name: "OUTBOUND_BATCHES",
			owner: this,
			frm: this,
			row_id: "BatchId", //"id",
			display_name: "BatchId",
			rows_per_page: 500,
			show_at_least: 499,
			without_paginator: true,
			loading_waiting_msg: "<%=_T('Please wait, loading Batches ...')%>",
			ids: {
				grid: "out_form_process_batches_list",
				paginator: "out_form_process_batches_paginator",
				not_results_found: "out_form_process_batches_not_results_found",
				//popup
				popup       : "out_form_process_batches_popup",
				popup_msg   : "out_form_process_batches_popup_message",
				popup_create: "out_form_process_batches_popup_create",
				popup_title : "out_form_process_batches_popup_title",
				popup_close : "out_form_process_batches_popup_close"
			},
			strings: {
				not_results_found: "<%=_T('No Batches found.')%>",
				can_remove       : "<%=_T('Do you want to delete Batch {0} ?')%>",
				title_update     : "<%=_T('Update Batch')%>"
			},
			columns: [
				{id: "_number"    , name: "", type: "counter" },
				{id: "_left_actions", type: "actions", name: "",
					custom_action: {
						INFO: {
							name: "<%=_T('Info')%>",
							execute: function($row, $owner) {
								var batch = self.form_paginator_batches.get_row_data($row);
								self.__open_batch_info_popup($owner, batch);
							},
							class_name: "buttons-icons-information"
						}
					}
				},
				{id: "BatchId"   , name: "<%=_T('Name')%>"},
				{id: "StartDate", name: "<%=_T('Valid From')%>",
					type: "date", date_config: {utc: true, type_date: "date", format_date: "ymd", separator_date: "-"}
				},
				{id: "EndDate", name: "<%=_T('Valid Until')%>",
					type: "date", date_config: {utc: true, type_date: "date", format_date: "ymd", separator_date: "-"}
				},
				{id: "CreationContacts" , name: "<%=_T('Contacts')%>"  },
				{id: "RemainingContacts", name: "<%=_T('Remaining')%>" },
				{id: "Percentage"       , name: "% " + "<%=_T('Percentage')%>", get_value: function(item) {
					return self.get_grid_batch_value(item, "Percentage");
				}},
				{id: "_rigth_actions", type: "actions", name: "",
					system_action: {edit: true, erase: true},
					custom_action: {
						CANCEL: {
							name: "<%=_T('Cancel')%>",
							execute: function($row) {
								var batch = self.form_paginator_batches.get_row_data($row);
								self.__run_batch_action({
									action: "CANCEL",
									batch : batch
								});
							},
							class_name: "buttons-icons-cancel"
						},
						PAUSE: {
							name: "<%=_T('Pause')%>",
							execute: function($row) {
								var batch = self.form_paginator_batches.get_row_data($row);
								self.__run_batch_action({
									action: "PAUSE",
									batch : batch
								});
							},
							class_name: "buttons-icons-voz-hold"
						},
						RESUME: {
							name: "<%=_T('Resume')%>",
							execute: function($row) {
								var batch = self.form_paginator_batches.get_row_data($row);
								self.__run_batch_action({
									action: "RESUME",
									batch : batch
								});
							},
							class_name: "buttons-icons-start"
						}
					},
					is_enabled: function(action_id, batch) {
						return self.__grd_batches_row_status_actions(action_id, batch);
					}
				}
			],
			callbacks: {
				save: function(callback) {
					self.__update_batch(callback);
				},
				edit: function(batch) {
					self.on_edit_batch(batch);
				},
				on_closed_popup: function() {
					self.current_batch = null;
				},
				remove: function(batch, callback) {
					self.__run_batch_action({
						action  : "DELETE",
						batch   : batch,
						callback: callback
					});
				},
				load_items: function(page, callback) {
					//console.log("6 - grid load_items -> form_paginator_batches");
					self.count_pending_grid = self.count_pending_grid + 1;
					self.load_batches(page, callback);
				},
				on_loaded_items: function(callback) {
					self.__on_loaded_grid(callback);
				}
			}
		});
	};

	form_outbound_engine_processes_allegro.prototype.get_grid_batch_value = function(item, field_id)
	{
		let value = {
			original_value: "",
			value         : "",
			search_value  : ""
		};

		let item_value = item[field_id] || "";
		if (item_value === undefined) {
			return value;
		}

		value.original_value = item_value;
		value.value = item_value;
		value.search_value = item_value;

		if (field_id === "Percentage") {
			item_value = item_value || 0;
			if (item_value !== "") {
				item_value = typeof(item_value) === "number" ? item_value.toString() : item_value;
				value.value = item_value + " %";
			}
		}

		return value;
	};

	/*
		Esta funcion se invoca cada vez que termina la carga de una grilla, para verificar si no queda ninguna pending.
		Cuando no queden pending, ahí invoca a la funcion local para dejar listo el frm para ser utilizado.
	*/
	form_outbound_engine_processes_allegro.prototype.__on_loaded_grid = function(callback)
	{
		const self = this;
		callback = callback || $.noop;

		this.count_pending_grid = this.count_pending_grid - 1;
		// console.log("form_outbound_engine_processes_allegro.prototype.__on_loaded_grid " + this.count_pending_grid);
		if (this.count_pending_grid === 0) {
			window.setTimeout(function() {
				self.__ready_for_save();
			});
		}
		callback();
	};

	form_outbound_engine_processes_allegro.prototype.__disabled_period_ctrls = function(disabled)
	{
		var is_required = !disabled;
		var $from = $("#out_form_process_batches_popup_valid_from");
		this.disable_control($from, disabled, is_required);
		var $until = $("#out_form_process_batches_popup_valid_until");
		this.disable_control($until, disabled, is_required);
		this.__enabled_disabled_update_batch();
	};

	form_outbound_engine_processes_allegro.prototype.__disabled_increase_ctrls = function(disabled)
	{
		var is_required = !disabled;
		var $increase = $("#out_form_process_batches_popup_increase_value");
		this.disable_control($increase, disabled, is_required);
		this.__enabled_disabled_update_batch();
	};

	form_outbound_engine_processes_allegro.prototype.__disabled_percentage_ctrls = function(disabled)
	{
		var is_required = !disabled;
		var $percentage = $("#out_form_process_batches_popup_percentage_value");
		this.disable_control($percentage, disabled, is_required);
		this.__enabled_disabled_update_batch();
	};

	form_outbound_engine_processes_allegro.prototype.__enabled_disabled_update_batch = function()
	{
		var $period_enabled = $("#out_form_process_batches_popup_period_enabled");
		var $increase_enabled = $("#out_form_process_batches_popup_increase_value_enabled");
		var $percentage_enabled = $("#out_form_process_batches_popup_change_percentage_value_enabled");
		var disabled = !$period_enabled.prop("checked") && !$increase_enabled.prop("checked") && !$percentage_enabled.prop("checked");
		var $update = $("#out_form_process_batches_popup_create");
		$update.prop("disabled", disabled);
	};

	form_outbound_engine_processes_allegro.prototype.__can_change_batch_period = function()
	{
		return true;
	};

	form_outbound_engine_processes_allegro.prototype.__can_change_batch_increase_values = function()
	{
		if (this.current_batch.Expired) {
			return false;
		}
		const batch_actions_statuses = this.__get_batch_actions_statuses(this.current_batch.ContactsInfo);
		return batch_actions_statuses.edit;
	};

	form_outbound_engine_processes_allegro.prototype.__can_change_batch_percentage_values = function()
	{
		return this.__can_change_batch_increase_values();
	};

	form_outbound_engine_processes_allegro.prototype.on_edit_batch = function(batch)
	{
		this.current_batch = batch;
		// Aca al valor de los dtps los tengo que asignar directo con jQuery y no con la funcion del plguin de datepicker
		// ya que sino, no me toma el rango correctamente de los dtp relacionados.
		$("#out_form_process_batches_popup_valid_from").val(batch.StartDate);
		$("#out_form_process_batches_popup_valid_until").val(batch.EndDate);
		this.create_datepickers("out_form_process_batches_popup");
		const $period_enabled = $("#out_form_process_batches_popup_period_enabled");
		$period_enabled.prop("checked", false);
		this.__disabled_period_ctrls(true);
		if (this.__can_change_batch_period()) {
			$period_enabled.show();
		}
		else {
			$period_enabled.hide();
		}
		// increase value
		$("#out_form_process_batches_popup_increase_value").val(1);
		const $increase_value_enabled = $("#out_form_process_batches_popup_increase_value_enabled");
		$increase_value_enabled.prop("checked", false).trigger("change");
		if (this.__can_change_batch_increase_values()) {
			this.disable_control($increase_value_enabled, false);
		}
		else {
			this.disable_control($increase_value_enabled, true);
		}
		// percentage value
		$("#out_form_process_batches_popup_percentage_value").val(batch.Percentage);
		const $percentage_value_enabled = $("#out_form_process_batches_popup_change_percentage_value_enabled");
		$percentage_value_enabled.prop("checked", false).trigger("change");
		if (this.__can_change_batch_percentage_values()) {
			this.disable_control($percentage_value_enabled, false);
		}
		else {
			this.disable_control($percentage_value_enabled, true);
		}
	};

	form_outbound_engine_processes_allegro.prototype.__update_batch = function(callback)
	{
		const self = this;

		const must_increase_value = $("#out_form_process_batches_popup_increase_value_enabled").prop("checked");
		const must_change_percentage = $("#out_form_process_batches_popup_change_percentage_value_enabled").prop("checked");
		const $percentage = $("#out_form_process_batches_popup_percentage_value");
		let percentage_value = $percentage.val();
		percentage_value = parseInt(percentage_value, 10);

		var validate = function() {
			self.form_paginator_batches.hide_popup_msg();
			let result = {
				success: false,
				type   : "Error",
				msg    : "",
				$ctrl  : undefined
			};

			if (must_change_percentage) {
			// valida que el porcentaje ingresado + la sumatoria de los porcentajes de todos los batches del proceso, NO superen el 100%
				let batches = self.form_paginator_batches.get_items();
				let total_percentage = 0;
				(batches || []).forEach(batch => {
					if (batch.Percentage && batch.Percentage > 0) {
						if (self.current_batch.BatchId !== batch.BatchId) {
							total_percentage = total_percentage + batch.Percentage;
						}
					}
				});
				if (percentage_value && percentage_value > 0) {
					total_percentage = total_percentage + percentage_value;
				}
				if (total_percentage > 100) {
			//		result.success = false;
					result.msg = "<%=_T('The total percentage in the process exceed 100%.')%>";
					result.$ctrl = $percentage;
					return result;
				}
			}
			// success
			result.success = true;
			return result;
		};

		const validation_result = validate();
		if (!validation_result.success) {
			self.mark_ctrl_error(validation_result.$ctrl);
			callback(false, undefined, validation_result);
			return;
		}

		var run_batch_actions = function() {
			if (!must_increase_value && !must_change_percentage) {
				callback(true, self.current_batch);
				self.hide_waiting();
				return;
			}

			var run_percentage_action = function() {
				self.current_batch.Percentage = percentage_value;
				self.__run_batch_action({
					action    : "CHANGEPERCENTAGE",
					batch     : self.current_batch,
					percentage: self.current_batch.Percentage,
					callback: function() {
						callback(true, self.current_batch);
					}
				});
			};

			if (must_increase_value) {
				self.__run_batch_action({
					action : "INCREASERETRIES",
					batch  : self.current_batch,
					retries: $("#out_form_process_batches_popup_increase_value").val(),
					callback: function() {
						if (!must_change_percentage) {
							callback(true, self.current_batch);
							return;
						}
						run_percentage_action();
					}
				});
			}
			else {
				run_percentage_action();
			}
		};

		let start_date = $("#out_form_process_batches_popup_valid_from").datepicker("getDate");
		let end_date = $("#out_form_process_batches_popup_valid_until").datepicker("getDate");

		start_date = inConcert.Utils.getStringFromDate(start_date, false, "full_datetime", "ymd", "-");
		end_date = inConcert.Utils.getStringFromDate(end_date, false, "full_datetime", "ymd", "-");

		var must_change_period = function() {
			if (!$("#out_form_process_batches_popup_period_enabled").prop("checked")) {
				return false;
			}
			return self.current_batch.StartDate !== start_date || self.current_batch.EndDate !== end_date;
		};

		if (must_change_period()) {
		// Si las fechas del batch han cambiado, primero mando a guardar el proceso y luego incremento los reintentos y cambio el porcentaje
			this.show_waiting("<%=_T('Updating Batch')%>");
			this.current_batch.StartDate = start_date;
			this.current_batch.EndDate = end_date;
			this.form_paginator_batches.update_row(this.current_batch);

			let process = this.get();
			const outbound_engine_api = inConcert.app.getOutboundEngineAPI();

			outbound_engine_api.processSave(process, function(success, data)
			{
				if (success) {
					if (data && data.result && data.result.OutboundProcessBatchs) {
						for (let i = 0; i < data.result.OutboundProcessBatchs.length; i++) {
							let batch = data.result.OutboundProcessBatchs[i];
							if (self.current_batch.BatchId === batch.BatchId) {
								// tengo que actualizar la grilla de batches las acciones y las celdas
								self.current_batch.ContactsInfo = batch.contactsStatus;
								self.current_batch.Expired = batch.hasExpiredBatch;
								self.current_batch.Percentage = batch.Percentage;
								self.form_paginator_batches.update_row(self.current_batch);
								break;
							}
						}
					}
					run_batch_actions();
				}
				else {
					self.form_paginator_batches.show_popup_msg("<%=_T('Unable to Save Process.')%>", "Error", 5000 );
					self.hide_waiting();
				}
			});
		}
		else {
		// Si las fechas del batch NO han cambiado, solo incremento los reintentos
			run_batch_actions();
		}
	};

	form_outbound_engine_processes_allegro.prototype.__get_batch_actions_statuses = function(contacts_info)
	{
		var statuses = {
			active: 0, cancelled: 0, contacted: 0, paused: 0, scheduled: 0, others: 0 
		};

		$.each(contacts_info || {}, function(_, info) {
			var status = info.original_status.toLowerCase();
			if (statuses.hasOwnProperty(status)) {
				statuses[status] = info.total;
			}
			else {
				statuses.others = info.total;
			}
		});

		var enabled_actions = { cancel: false, pause: false, resume: false, edit: false };

		// cancel button
		if (statuses.active > 0 || statuses.scheduled > 0 || statuses.paused > 0) {
			enabled_actions.cancel = true;
		}
		// pause button
		if (statuses.active > 0 || statuses.scheduled > 0) {
			enabled_actions.pause = true;
		}
		// resume button
		if (statuses.paused > 0) {
			enabled_actions.resume = true;
		}
		// edit button
		if (statuses.active > 0 || statuses.scheduled > 0 || statuses.paused > 0) {
			enabled_actions.edit = true;
		}

		return enabled_actions;
	};

	form_outbound_engine_processes_allegro.prototype.__grd_batches_row_status_actions = function(action_id, batch)
	{
		if (batch.Expired) {
			if (action_id === "EDIT") {
				return 1;
			}
			if (action_id === "DELETE") {
				var process = this.get_entity_instance() || {};
				if ((process.Status || "").toUpperCase() === "STOPPED") {
					return 1;
				}
				return 0;
			}
		}
		else {
			var buttons_status = this.__get_batch_actions_statuses(batch.ContactsInfo);
			if (action_id === "CANCEL") {
				return buttons_status.cancel ? 1 : 0;
			}
			else if (action_id === "PAUSE") {
				return buttons_status.pause ? 1 : 0;
			}
			else if (action_id === "RESUME") {
				return buttons_status.resume ? 1 : 0;
			}
			else if (action_id === "EDIT") {
				// Siempre va enabled, ya que en este frm se pueden realizar varias acciones (cambiar periodos, incrementar los valores a los contactos, y los porcentajes)
				return 1;
			}
		}

		return -1;
	};

	form_outbound_engine_processes_allegro.prototype.__run_batch_action = function(params)
	{
		const self = this;

		var callback = function(success, response) {
			if (success) {
				self.show_status_message("<%=_T('Action completed successfully!')%>", "Info", 5000);
				if (params.action === "CANCEL" || params.action === "PAUSE" || params.action === "RESUME" || params.action === "INCREASERETRIES" || params.action === "CHANGEPERCENTAGE") {
					params.batch.ContactsInfo = response.result.contactsStatus;
					params.batch.Expired = response.result.hasExpiredBatch;
					self.form_paginator_batches.update_row(params.batch);
				}
			}
			else {
				self.show_status_message("<%=_T('Unable to execute the batch operation, please contact your administrator.')%>", "Error", 5000);
			}
			self.hide_waiting();
			(params.callback || $.noop)(success, response);
		};

		this.show_waiting( "<%=_T('Please wait, Executing Action....')%>" );

		var process = this.get_entity_instance() || {};
		var outbound_engine_api = inConcert.app.getOutboundEngineAPI();

		if (params.action === "DELETE") {
			outbound_engine_api.processDeleteBatch(
				{
					processId: process.Id,
					batchId: params.batch.BatchId
				},
				function(success, response) {
					// pequeno waiting para dar feedback
					window.setTimeout(function() {
						callback(success, response);
					}, 500);
				}
			);
		}
		else {
			var data = {
				BatchId: params.batch.BatchId,
				OutboundProcessId: process.Id
			};
			if (params.action === "CANCEL") {
				data.Action = "cancelcontacts";
			}
			else if (params.action === "PAUSE") {
				data.Action = "pausecontacts";
			}
			else if (params.action === "RESUME") {
				data.Action = "resumecontacts";
			}
			else if (params.action === "INCREASERETRIES") {
				data.Action = "increaseretries";
				data.retries = parseInt(params.retries, 10);
			}
			else if (params.action === "CHANGEPERCENTAGE") {
				data.Action = "changepercentage";
				data.percentage = parseInt(params.percentage, 10);
			}
			else {
				throw("form_outbound_engine_processes_allegro.prototype.__run_batch_action-> Unrecognized action " + params.action);
			}
			outbound_engine_api.processUpdateBatchByAction(
				{
					processId: process.Id,
					batchId: params.batch.BatchId
				},
				data,
				function(success, response) {
					// pequeno waiting para dar feedback
					window.setTimeout(function() {
						callback(success, response);
					}, 500);
				}
			);
		}
	};

	form_outbound_engine_processes_allegro.prototype.load_rules = function(_, callback)
	{
		var process = this.get_entity_instance() || {};
		process.OutboundProcessRules = inConcert.Utils.sortArray(process.OutboundProcessRules, "Priority", "asc");
		callback(true, process.OutboundProcessRules);
	};

	form_outbound_engine_processes_allegro.prototype.load_batches = function(_, callback)
	{
		const self = this;
		var process = this.get_entity_instance() || {};
		var outbound_engine_api = inConcert.app.getOutboundEngineAPI();
		outbound_engine_api.contactsBatchFromProcess(process.Id, function(success, contactsBatchs) {
			callback(success, contactsBatchs);
			// TODO: arreglar el SET EVENTS se tiene que realizar en el set_event del frm.
			self.form_paginator_batches.set_events();
		});
	};

	form_outbound_engine_processes_allegro.prototype.__reorder_rules = function()
	{
		const self = this;
		let rules = this.form_paginator_rules.get_items();
		let new_priority = 0;
		$.each(rules, function(_, rule) {
			if (rule) {
				new_priority++;
				rule.Priority = new_priority;
				self.form_paginator_rules.update_row(rule);
			}
		});
	};

	form_outbound_engine_processes_allegro.prototype.get_grid_value = function(item, field_id)
	{
		var value = {
			original_value: "",
			value         : "",
			search_value  : ""
		};

		var item_value = item[field_id];
		if (item_value === undefined) {
			item_value = "";
		}

		value.original_value = item_value;
		value.value = item_value;
		value.search_value = item_value;

		// GRID OUTBOUND_RULES
		if (field_id === "Priority") {
			if (item_value === "") {
				item_value = 1;
				if (this.frm_rules.is_new_rule()) {
					item_value = this.form_paginator_rules.count_rows();
					item_value = item_value + 1;
				}
				value.original_value = item_value;
				value.value = item_value;
				value.search_value = item_value;
				item.Priority = item_value;
			}
		}
		return value;
	};

	form_outbound_engine_processes_allegro.prototype.load_data = function(callback_complete_requests)
	{
		const self = this;

		var outbound_engine_api = inConcert.app.getOutboundEngineAPI();
		this.requests = [
			{
				request_function: function(success) {
					return outbound_engine_api.systemEngineServicesLoadAll(success);
				},
				success: function(list) {
					self.__fill_engine_services(list);
				}
			},
			{
				request_function: function(success) {
					return outbound_engine_api.systemEngineTypesLoadAll(success);
				},
				success: function(list) {
					self.__fill_engine_types(list);
				}
			}
		];

		this.frm_rules.attach_load_request(this.requests);
		form_outbound_engine_processes_allegro.uber.load_data.call( this, callback_complete_requests ); //invokes the base class method
	};

	form_outbound_engine_processes_allegro.prototype.__fill_engine_services = function(list)
	{
		var $combo = $("#out_form_process_engine_service");
		this.fill_combo($combo, true /* is_blank_first_item */, list, "ServiceId" /* attr_id */, "ServiceId" /* attr_name */);
	};

	form_outbound_engine_processes_allegro.prototype.__fill_engine_types = function(list)
	{
		var $combo = $("#out_form_process_engine_type");
		this.fill_combo($combo, true /* is_blank_first_item */, list, "Value" /* attr_id */, "DisplayName" /* attr_name */);
	};

	form_outbound_engine_processes_allegro.prototype.set_events = function()
	{
		const self = this;

		// eventos de la grilla con paginado
		this.form_paginator_rules.set_events();

		this.frm_rules.set_events();
		this.frm_batches.set_events();

		$("#out_form_process_rules_new")
			.off("click")
			.on("click", function() {
				self.__new_rule();
			});

		$("#out_form_process_batches_management")
			.off("click")
			.on("click", function() {
				self.__show_batches_section();
			});

		$("#out_form_process_rules_edit_sch")
			.off("click")
			.on("click", function() {
				self.__show_schedule_section(false);
			});

		$("#out_form_process_rules_new_sch")
			.off("click")
			.on("click", function() {
				self.__show_schedule_section(true);
			});

		// Eventos del popup de la grilla de edicion de batchmanagement (increase retries, etc)
		//
		$("#out_form_process_batches_popup_period_enabled")
			.off("change")
			.on("change", function() {
				var disabled = !$(this).prop("checked");
				self.__disabled_period_ctrls(disabled);
			});

		$("#out_form_process_batches_popup_increase_value_enabled")
			.off("change")
			.on("change", function() {
				var disabled = !$(this).prop("checked");
				self.__disabled_increase_ctrls(disabled);
			});

		$("#out_form_process_batches_popup_change_percentage_value_enabled")
			.off("change")
			.on("change", function() {
				var disabled = !$(this).prop("checked");
				self.__disabled_percentage_ctrls(disabled);
			});
	};

	form_outbound_engine_processes_allegro.prototype.__hide_rules_section = function()
	{
		$("body").removeClass("popupDisplay");
		$("#out_form_process_schedule_section").hide();
	};

	form_outbound_engine_processes_allegro.prototype.__show_rules_section = function()
	{
		var sub_title = this.frm_rules.is_new_rule() ? "<%=_T('New Rule')%>" : "<%=_T('Edit Rule')%>";
		this.set_sub_title(sub_title);
		this.dashboard.$get_dom_element_by_id("form_buttons_actions").hide();
		$("#out_form_process_section").hide();
		$("#out_form_process_batches_section").hide();
		this.__hide_rules_section();
		$("#out_form_process_rules_section").show();
	};

	form_outbound_engine_processes_allegro.prototype.__new_rule = function()
	{
		this.frm_rules.set({});
		this.__show_rules_section();
	};

	form_outbound_engine_processes_allegro.prototype.__edit_rule = function(rule)
	{
		this.frm_rules.set(rule);
		this.__show_rules_section();
	};

	form_outbound_engine_processes_allegro.prototype.show_process_section = function(callback)
	{
		callback = callback || $.noop;
		this.set_sub_title(); // lo deja empty al sub titulo
		this.form_paginator_rules.cleanup_edited_row();
		this.frm_batches.dispose();
		$("#out_form_process_rules_section").hide();
		$("#out_form_process_batches_section").hide();
		this.__hide_rules_section();
		$("#out_form_process_section").show();
		this.dashboard.$get_dom_element_by_id("form_buttons_actions").show();
		callback();
	};

	form_outbound_engine_processes_allegro.prototype.__show_batches_section = function()
	{
		this.frm_batches.initialize_ctrls();
		var sub_title = "<%=_T('Batches Management')%>";
		this.set_sub_title(sub_title);
		this.dashboard.$get_dom_element_by_id("form_buttons_actions").hide();
		$("#out_form_process_rules_section").hide();
		$("#out_form_process_section").hide();
		this.__hide_rules_section();
		$("#out_form_process_batches_section").show();
		this.$get_dom_element_by_id("form_back_btn_container").show();
	};

	form_outbound_engine_processes_allegro.prototype.__save_scheduler = function(is_new, scheduler, callback)
	{
		const self = this;

		var $sch_id = $("#out_form_process_schedule_name");
		scheduler.Id = $sch_id.val();
		scheduler.__IsNew = is_new;

		var process = this.get_entity_instance();
		if (process && process.Id) {
			scheduler.OutboundProcessId = process.Id;
		}

		var outbound_engine_api = inConcert.app.getOutboundEngineAPI();
		outbound_engine_api.processSaveScheduler(scheduler,
			function(success, err)
			{
				if (success) {
					// pequeno waiting para dar feedback
					window.setTimeout(function() {
						if (is_new) {
							var $sch = $("#out_form_process_rules_sch");
							self.combo_add_value($sch, scheduler.Id, scheduler.Id, true);
							$sch.trigger("change");
						}
						self.__show_rules_section();
						callback(success, err);
					}, 1000);
				}
				else {
					if (typeof err === "object" && typeof err.err === "object") {
						err = ( err.err.result || "").toUpperCase();
					}
					if ("ALREADY_EXIST" === err) {
						// al mensaje lo visualizo en el popup de schedule (conf_form_schedule)
						var $err_msg_container = $("#out_form_process_schedule_popup_message");
						self.show_validation_error($sch_id, undefined, undefined, undefined, undefined, $err_msg_container);
					}
					callback(success, err);
				}
			}
		);
	};

	form_outbound_engine_processes_allegro.prototype.__validate_scheduler = function(_, callback)
	{
		var $sch_id = $("#out_form_process_schedule_name");
		var sch_id = $sch_id.val();
		// al mensaje lo visualizo en el popup de schedule (conf_form_schedule)
		var $err_msg_container = $("#out_form_process_schedule_popup_message");

		var result = inConcert.Utils.validate(sch_id, "required");
		if (!result.success) {
			this.show_validation_error($sch_id, "<%=_T('Schedule Name is required.')%>", undefined, undefined, undefined, $err_msg_container);
			callback(result.success);
			return;
		}

		result = inConcert.Utils.validate(sch_id, "id");
		if (!result.success) {
			this.show_validation_error($sch_id, "<%=_T('Only letters, numbers, dashes and dots are allowed in the Schedule Name.')%>", undefined, undefined, undefined, $err_msg_container);
			callback(result.success);
			return;
		}

		callback(true);
	};

	form_outbound_engine_processes_allegro.prototype.__show_schedule_section = function(is_new)
	{
		const self = this;

		var $schedule = $("#out_form_process_rules_sch");
		var scheduler_id = $schedule.val();
		if (!is_new) {
			if (scheduler_id.length === 0) {
				return;
			}
		}

		var current_title = this.get_sub_title();
		var sub_title = current_title + " - " + (is_new ? "<%=_T('New Scheduler')%>" : "<%=_T('Edit Scheduler')%>");
		this.set_sub_title(sub_title);

		var $sch_id = $("#out_form_process_schedule_name");

		var show = function(schedule_info)
		{
			self.schedule_hnd = new inConcert.conf_form_schedule({
				frm                 : self,
				owner               : self,
				prefix              : "out_form_process_schedule",
				container_id        : "out_form_process_schedule_container",
				err_msg_container_id: "out_form_process_schedule_popup_message",
				buttons_container_id: "out_form_process_schedule_popup_buttons",
				waiting_container_id: "out_form_process_schedule_popup_waiting",
				callbacks: {
					cancel: function() {
						self.__show_rules_section();
					},
					can_configure_special_days: function() {
						return true;
					},
					save: function(scheduler, callback) {
						self.__save_scheduler(is_new, scheduler, callback);
					},
					validate: function(scheduler, callback) {
						self.__validate_scheduler(scheduler, callback);
					}
				}
			});
			self.schedule_hnd.initialize();
			if (schedule_info) {
				// EDITANDO
				$sch_id.val(scheduler_id);
				self.readonly_control($sch_id, true);
				self.schedule_hnd.set(schedule_info);
			}
			else {
				// NUEVO
				self.readonly_control($sch_id, false, true);
				$sch_id.val("").focus();
			}

			$("#out_form_process_rules_section").hide();
			$("#out_form_process_batches_section").hide();
			$("#out_form_process_section").hide();
			//----
			$("body").removeClass("popupDisplay").addClass("popupDisplay");
			$("#out_form_process_schedule_section").show();
		};

		if (is_new) {
			// CREANDO un nuevo scheduler
			show();
			return;
		}

		// EDITANDO un scheduler existente
		this.show_waiting( "<%=_T('Please wait, Loading Scheduler....')%>" );

		var outbound_engine_api = inConcert.app.getOutboundEngineAPI();
		outbound_engine_api.processGetScheduler({ schedulerId: scheduler_id },
			function(success, scheduler)
			{
				if (success) {
					show(scheduler);
				}
				else {
					self.show_status_message( "<%=_T('Error Retrieving schedule.')%>", "Error", 5000 );
				}
				self.hide_waiting();
			}
		);
	};

	form_outbound_engine_processes_allegro.prototype.ready = function()
	{
		var $id = $("#out_form_process_id");
		var $batch_container = $("#out_form_process_batches_list_container");
		if ( this.is_new ) {
			this.readonly_control($id, false /* disabled */, true /* is_required */);
			$id.focus();
			$batch_container.hide();
		}
		else {
			this.readonly_control($id, true /* disabled */ );
			 $("#out_form_process_max_retries_per_day").focus();
			$batch_container.show();
		}
		form_outbound_engine_processes_allegro.uber.ready.call(this); //invokes the base class method
	};

	/*
		Est funcion se invoca cuando ha finalizado la carga de cada grilla del frm, asi el frm queda listo para ser utilizado.
		La misma termina invocando a la funcion "ready_for_save" de la clase base de formularios "conf_form".
		En esa clase de procesos la funcion "ready_for_save" (**) queda empty, para que la misma no se ejecute hasta que sea haya completado la carga de las grillas.
	*/
	form_outbound_engine_processes_allegro.prototype.__ready_for_save = function()
	{
		form_outbound_engine_processes_allegro.uber.ready_for_save.call(this); //invokes the base class method
	};

	/*
		(**) Dejamos a la funcion empty asi no se ejecuta la misma hasta que la carga de las grillas se haya completado.
	*/
	form_outbound_engine_processes_allegro.prototype.ready_for_save = function()
	{
		if (this.is_new) {
			this.__ready_for_save();
		}
	};

	form_outbound_engine_processes_allegro.prototype.ready_for_create = function()
	{
		form_outbound_engine_processes_allegro.uber.ready_for_create.call( this ); //invokes the base class method
		$("#out_form_process_batches_list_container").hide();
		this.set(true, {});
		this.initialized = true;
	};

	form_outbound_engine_processes_allegro.prototype.ready_for_update = function()
	{
		form_outbound_engine_processes_allegro.uber.ready_for_update.call( this ); //invokes the base class method
		this.frm_batches.load_update_information();
		this.readonly_control($("#out_form_process_id"), true /* disabled */ );
		$("#out_form_process_batches_list_container").fadeIn("slow");

		const process = this.get_entity_instance() || {};
		this.readonly_control($("#out_form_process_engine_service"), ((process.Status || "").toUpperCase() === "RUNNING") /* disabled */ );
	};

	form_outbound_engine_processes_allegro.prototype.get_validation_def = function()
	{
		const self = this;

		var definitions = [
			{
				control_id: "out_form_process_id",
				messages: {
					required: "<%=_T('Please enter the Process Name.')%>",
					id      : "<%=_T('Only letters, numbers, dashes and dots are allowed in the Process Name.')%>"
				}
			},
			{
				control_id: "out_form_process_engine_service",
				messages: {
					required: "<%=_T('Please select the Engine Service.')%>"
				}
			},
			{
				control_id: "out_form_process_engine_type",
				messages: {
					required: "<%=_T('Please select the Engine Type.')%>"
				}
			},
			{
				control_id: "out_form_process_max_retries_per_day",
				messages: {
					required: "<%=_T('Please select the Max Retries Per Day.')%>",
					digits  : "<%=_T([[Max Retries Per Day value must be a number]])%>"
				}
			},
			{
				control_id: "out_form_process_max_retries_per_contact",
				messages: {
					required: "<%=_T('Please select the Max Retries Per Contact.')%>",
					digits  : "<%=_T([[Max Retries Per Contact value must be a number]])%>"
				}
			},
			{
				control_id: "out_form_process_contact_window_size",
				messages: {
					required: "<%=_T('Please select the Contact Window Minutes.')%>",
					digits  : "<%=_T([[Contact Window Minutes value must be a number]])%>"
				}
			},
			{
				control_id: "out_form_process_rules_list",
				messages: {
					custom: {
						type:  "custom",
						valid: function() {
							var response = {
								success: true
							};
							var rules = self.__get_rules();
							if (rules.length === 0) {
								response.success = false;
								response.message = "<%=_T('Please Add Rules.')%>";
							}
							return response;
						}
					}
				}
			}
		];
		return definitions;
	};

	form_outbound_engine_processes_allegro.prototype.save_execute = function(after_save, process /*, hash_id */)
	{
		const self = this;

		this.show_waiting( "<%=_T('Please wait, Saving Process....')%>" );

		var outbound_engine_api = inConcert.app.getOutboundEngineAPI();
		outbound_engine_api.processSave(process, function(success, response)
		{
			var err;
			if (success) {
				self.show_status_message( "<%=_T('Successfully Saved Process.')%>", "Info", 5000 );
				self.hide_waiting();
				after_save = "SAVE_AND_NOT_EXIT_AND_REFRESH_VIEW";
			}
			else {
				err = response;
				self.show_status_message( "<%=_T('Unable to Save Process.')%>", "Error", 5000 );
				self.hide_waiting();
			}
			self.on_saved(success, process, after_save, err);
		});
	};

	form_outbound_engine_processes_allegro.prototype.add_update_rule = function(rule, callback)
	{
		callback = callback || $.noop;
		this.form_paginator_rules.add_or_update_row(rule);
		this.show_process_section();
		var message;
		if (this.frm_rules.is_new_rule()) {
			message = "<%=_T('Added Rule to Process.')%>";
		}
		else {
			message = "<%=_T('Updated Rule.')%>";
		}
		this.show_status_message(message, "Info", 5000);
		callback();
	};

	form_outbound_engine_processes_allegro.prototype.on_edit_rule_popup = function(rule)
	{
		this.__edit_rule(rule);
	};

}());
