/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 11 */

/* global inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function ()
{
	"use strict";

	const FORMAT = inConcert.Utils.format;

	const get_request_timeout = function() {
		const outbound_engine_api = inConcert.app.getOutboundEngineAPI();
		return outbound_engine_api.getRequestTimeout();
	};

	/*--------------------------------------------------------------------------------------------------------------------------------
	* ALLERGO CONFIG.
	*
	 --------------------------------------------------------------------------------------------------------------------------------*/

		//------------------------------------------------------
		// contacts navigation config
		const contacts_config_allegro = function(base_uri)
		{
			const frm_instance = inConcert.form_outbound_engine_contacts;
			return {
				id: "contacts",
				name: "<%=_T('Contacts')%>",
				permissions: {
					view: "view_contacts",
					save: "FALSE_SAVE_PERMISSION", // Es un permiso falso, para que la vista y el frm se comporte en modo READONLY.
					permission_type: "vcc"
				},
				class_name: "itemContacts",
				i_class_name: "far fa-database",
				view: {
					request: {
						timeout: get_request_timeout()
					},
					header: {
						hide_new_button: true,
						hide_total_rows: true
					},
					paginator: {
						paged_text_format : "<%=_T('{page_row_start}-{page_row_end} of {many}')%>"
					},
					render: "/inconcert/apps/outbound_engine/pages/view_oe_content_default.<%=_L%>.html#grunt-cache-bust",
					title: "<%=_T('Contacts')%>",
					tagline: "<%=_T('Tagline for Contacts')%>",
					row_id: "ID",
					display_name: "Name",
					uri: {
						search: base_uri + "contacts/search/"
					},
					source: ["ID", "Name", "LastUpdate", "VIP", "IsClient"],
					columns: [
						{id: "_number", name: "", type: "counter"},
						{id: "Name", name: "<%=_T('Name')%>"},
						{id: "ID", name: "<%=_T('Id')%>"},
						{id: "LastUpdate", name: "<%=_T('Last Update')%>"},
						{id: "VIP", name: "<%=_T('VIP')%>", type: "boolean", class_name: "booleanColumnTitle", without_tooltip: true},
						{id: "IsClient", name: "<%=_T('Client')%>", type: "boolean", class_name: "booleanColumnTitle", without_tooltip: true},
						{id: "_action", name: "", system_action: {edit: true}}
					],
					filter: {
						fields: {
							"ID"      : {control: null},
							"Name"    : {control: null},
							"Campaign": {control: null}
						}
					},
					realtime: null
				},
				form: {instance: frm_instance}
			};
		};

		//------------------------------------------------------
		// exports navigation config
		const exports_config_allegro = function(base_uri)
		{
			const frm_instance = inConcert.form_outbound_engine_exports;

			const execute_custom_action = function( action_id, $row, view )
			{
				const outbound_engine_api = inConcert.app.getOutboundEngineAPI();
				let export_id = $row.attr("row_id");
				let has_permission;

				switch ( action_id ) {
					case "cancel":
						has_permission = inConcert.permissions.to_action("outbound_engine" /*app*/, "cancel_contacts_export");
						if (!has_permission) {
							view.dashboard.show_status_message(
								FORMAT("<%=_T('Unable to cancel {0} exportation. You dont have permission.')%>", name),
								"Error",
								5000
							);
							return;
						}

						view.dashboard.show_waiting(FORMAT("<%=_T('Wait moment please, canceling exportation {0}')%>", export_id));

						outbound_engine_api.exportCancel({
								export_id: export_id
							},
							function(success, response)
							{
								let msg, type;
								if (success) {
									msg = "<%=_T('Successfully cancelled {0} exportation.')%>";
									type = "Info";
									let exportation = $row.data("row_data");
									exportation.Status = "ABORTED";
									exportation.DisplayStatus = "Aborted";
									view.update_row(exportation, $row, true);
								}
								else {
									let err = response.err ? response.err.result : "UNRECOGNIZED";
									switch(err) {
										case 3:
											msg = "<%=_T('Unable to cancel {0} exportation. Exportation has aborted.')%>";
											break;

										case 4:
											msg = "<%=_T('Unable to cancel {0} exportation. Exportation has completed.')%>";
											break;

										default:
											msg = "<%=_T('Unable to cancel {0} exportation.')%>";
											break;
									}
									type = "Error";
								}
								view.dashboard.show_status_message(
									FORMAT(msg, export_id),
									type,
									5000
								);
							}
						);
						break;

					case "download":
						has_permission = inConcert.permissions.to_action("outbound_engine" /*app*/, "download_contacts_export");
						if (!has_permission) {
							view.dashboard.show_status_message(
								FORMAT( "<%=_T('Unable to download {0} exportation. You dont have permission.')%>", name),
								"Error",
								5000
							);
							return;
						}

						let row_data = $row.data("row_data");
						view.dashboard.show_waiting(FORMAT("<%=_T('Wait moment please, download exportation {0}')%>", export_id));

						outbound_engine_api.exportDownloadExportation({
								export_id: export_id,
								file_name: row_data.FileName
							},
							function(success/*, err*/)
							{
								let msg, type;
								if (success) {
									msg = "<%=_T('Successfully Started downloaded {0} exportation.')%>";
									type = "Info";
								}
								else {
									msg = "<%=_T('Unable to download {0} exportation.')%>";
									type = "Error";
								}
								view.dashboard.show_status_message(
									FORMAT(msg, export_id),
									type,
									5000
								);
							}
						);
						break;
				}
			};

			// is action enabled
			const is_action_enabled = function(action, row)
			{
				/*
				* Valores a retornar:
				*    1  = La accion se debe visualizar y estar Enabled.
				*    0  = La accion se debe visualizar y estar Disabled.
				*    -1 = La accion no se genera en el markup. (No aparece el boton de la accion).
				*/
				let enabled_statuses;

				if (action.toLowerCase() === "download") {
					enabled_statuses = {
						COMPLETE: true
					};
					if (!enabled_statuses[row.Status]) {
						return 0;
					}
				}
				else if (action.toLowerCase() === "cancel") {
					enabled_statuses = {
						["ASSEMBLING OUTPUT"]: true,
						VALIDATING           : true,
						["QUERYING DB"]      : true
					};
					if (!enabled_statuses[row.Status]) {
						return 0;
					}
				}
				return 1;
			};

			return {
				id: "exports",
				name: "<%=_T('Exportations')%>",
				permissions: {
					view: "view_contacts_export",
					save: "save_contacts_export",
					erase: "delete_contacts_export",
					permission_type: "vcc"
				},
				class_name: "itemExports",
				i_class_name: "far fa-file-import fa-flip-horizontal",
				view: {
					request: {
						timeout: get_request_timeout()
					},
					header: {
						hide_total_rows: true
					},
					paginator: {
						paged_text_format : "<%=_T('{page_row_start}-{page_row_end} of {many}')%>"
					},
					render: "/inconcert/apps/outbound_engine/pages/view_oe_content_default.<%=_L%>.html#grunt-cache-bust",
					title: "<%=_T('Exportations')%>",
					tagline: "<%=_T('Tagline for Exports')%>",
					new_button_name: "<%=_T('New Exportation')%>",
					row_id: "ID",
					display_name: "ID",
					messages: {
						erase: "<%=_T('Do you want to delete Export {0} ?')%>",
						created: function(value) {
							return FORMAT("<%=_T('Successfully Started Exportation: {0}.')%>", value);
						}
						//,
						//err: get_message_error
					},
					uri: {
						erase:  base_uri + "exports/delete/",
						search: base_uri + "exports/search/"
					},
					source: ["ID", "User", "TimeZoneDate", "TimeZoneDate", "RowsExported", "FileName", "DisplayFileSize"],
					columns: [
						{id: "_number"         , name: "", type: "counter"   },
						{id: "ID"              , name: "<%=_T('Name')%>"     },
						{id: "DisplayStatus"   , name: "<%=_T('Status')%>", class_name: "booleanColumnTitle",
							// get_tooltip: function(row) {
								// return "mi tooltip"
							// },
							type: "chip",
							get_value: function(row)
							{
								let resp = {};
								resp.original_value = row.DisplayStatus || row.Status;
								resp.value = resp.original_value;

								const states_def = {
									ABORTED: {
										column_class_name: "exportationAborted processAborted",
										value            : "<%=_T('ABORTED')%>", 
										class_name       : "chipStatus",
									},
									["ASSEMBLING OUTPUT"]: {
										column_class_name: "exportationAssemblingOutput processAssemblingOutput",
										value            : "<%=_T('ASSEMBLING OUTPUT')%>", 
										class_name       : "chipStatus",
									},
									COMPLETE: {
										column_class_name: "exportationCompleted processCompleted",
										value            : "<%=_T('COMPLETE')%>", 
										class_name       : "chipStatus",
									},
									DELETING: {
										column_class_name: "exportationDeleting processDeleting",
										value            : "<%=_T('DELETING')%>", 
										class_name       : "chipStatus",
									},
									PAUSED: {
										column_class_name: "exportationPaused processPaused",
										value            : "<%=_T('PAUSED')%>", 
										class_name       : "chipStatus",
									},
									["QUERYING DB"]: {
										column_class_name: "exportationQueryingDb processQueryingDb",
										value            : "<%=_T('QUERYING DB')%>", 
										class_name       : "chipStatus",
									},
									RUNNING: {
										column_class_name: "exportationRunning processRunning",
										value            : "<%=_T('RUNNING')%>", 
										class_name       : "chipStatus",
									},
									VALIDATING: {
										column_class_name: "exportationValidating processValidating",
										value            : "<%=_T('VALIDATING')%>", 
										class_name       : "chipStatus",
									}
								};

								let state = row.Status;
								let def = states_def[ state ];
								if (!def) {
									resp.column_class_name = state;
									resp.class_name = "chipStatus";
								}
								else {
									resp.column_class_name = def.column_class_name || state;
									resp.class_name = def.class_name || "chipStatus";
									resp.value = def.value || resp.value;
								}

								resp.class_name = "outboundExportationStatus";

								return resp;
							}
						},
						{id: "User"            , name: "<%=_T('User Id')%>"  },
						{id: "TimeZoneDate"    , name: "<%=_T('Date')%>",
							type: "date",
							date_config: {utc: false, type_date: "date", format_date: "ymd", separator_date: "-"},
						},
						{id: "ContactsExported", name: "<%=_T('Contacts')%>" },
						{id: "RowsExported"    , name: "<%=_T('Rows')%>"     },
						{id: "FileName"        , name: "<%=_T('File Name')%>"},
						{id: "DisplayFileSize" , name: "<%=_T('File Size')%>"},
						{id: "_action",
							class_name: "gridColumnThreeActions",
							system_action: {erase: true},
							custom_action: {
								cancel: {
									name: "<%=_T('Cancel')%>",
									execute: execute_custom_action,
									class_name: "cancel",
									permission: "cancel_contacts_export"
								},
								download: {
									name: "<%=_T('Download')%>",
									execute: execute_custom_action,
									class_name: "download",
									permission: "download_contacts_export"
								}
							},
							is_enabled: is_action_enabled
						}
					],
					filter: {
						fields: {
							"ID"    : {control: null},
							"User"  : {control: null},
							"Status": {control: null}
						}
					},
					realtime: null
				},
				form: {instance: frm_instance}
			};
		};

		//------------------------------------------------------
		// export layout navigation config
		const export_layout_config_allegro = function(base_uri)
		{
			const frm_instance = inConcert.form_outbound_engine_export_layout;

			return {
				id: "export_layout",
				name: "<%=_T('Exportations Layout')%>",
				permissions: {
					view: "view_contacts_export_layout",
					save: "save_contacts_export_layout",
					erase: "delete_contacts_export_layout",
					permission_type: "vcc"
				},
				class_name: "itemExportLayout",
				i_class_name: "far fa-file-export",
				view: {
					request: {
						timeout: get_request_timeout()
					},
					header: {
						hide_total_rows: true
					},
					paginator: {
						paged_text_format : "<%=_T('{page_row_start}-{page_row_end} of {many}')%>"
					},
					render: "/inconcert/apps/outbound_engine/pages/view_oe_content_default.<%=_L%>.html#grunt-cache-bust",
					title: "<%=_T('Exportations Layout')%>",
					tagline: "<%=_T('Tagline for Export Layout')%>",
					new_button_name: "<%=_T('New Layout')%>",
					row_id: "Id",
					display_name: "Description",
					messages: {
						erase: "<%=_T('Do you want to delete Export Layout {0} ?')%>"//,
						//err: get_message_error
					},
					uri: {
						erase:  base_uri + "exports/layout/delete/",
						search: base_uri + "exports/layout/search/"
					},
					source: ["Id", "Description"],
					columns: [
						{id: "_number"    , name: "", type: "counter"     },
						{id: "Id"         , name: "<%=_T('Name')%>"       },
						{id: "Description", name: "<%=_T('Description')%>"},
						{id: "_action"    , system_action: {edit: true, erase: true}}
					],
					filter: {
						fields: {
							"Id"         : {control: null},
							"Description": {control: null}
						}
					},
					realtime: null,
					// subgrilla definicion
					sub_grid: {
						data_source: "ContactExportLayoutColumns", // aca vienen los registros de la subgrilla
						colapsed: true,
						messages: function( type ) {
							let message;
							if ( type === "no_records_found" ) {
								message = "<%=_T('No rows found.')%>";
							}
							return message;
						},
						columns: [
							// las columns del colspan
							{id: "_number", name: "", type: "counter"},
							{
								id  : "DisplayGroup",
								name: "<%=_T('Group')%>"
							},
							{
								id  : "DisplayType",
								name: "<%=_T('Field Name')%>"
							},
							{
								id  : "Name",
								name: "<%=_T('Column Name')%>"
							},
							{
								id  : "DisplayExtraData",
								name: "<%=_T('Phone Filter')%>"
							}
						]
					}
				},
				form: {instance: frm_instance}
			};
		};

		//------------------------------------------------------
		// imports navigation config
		const imports_config_allegro = function(base_uri)
		{
			const frm_instance = inConcert.form_outbound_engine_imports;

			const execute_custom_action = function( action_id, $row, view )
			{
				const outbound_engine_api = inConcert.app.getOutboundEngineAPI();
				const import_id = $row.attr("row_id");

				switch (action_id) {
					case "cancel":
						if (!view.form.has_permission_to_save()) {
							view.dashboard.show_status_message(
								FORMAT( "<%=_T('Unable to cancel {0} importation. You dont have permission.')%>", import_id),
								"Error",
								5000
							);
							return;
						}

						view.dashboard.show_waiting(FORMAT("<%=_T('Wait moment please, canceling importation {0}')%>", import_id));

						outbound_engine_api.importCancel({
								import_id: import_id
							},
							function(success, err)
							{
								let msg, type;
								if (success) {
									msg = "<%=_T('Successfully cancelled {0} importation.')%>";
									type = "Info";
								}
								else {
									const err_messages = {
										"ALLEGRO_INTEGRATION_ERROR": "<%=_T('Unable to cancel {0} importation. Allegro integration error.')%>",
										"FORBIDDEN"                : "<%=_T('Unable to cancel {0} importation. You dont have permission.')%>",
										"4"                        : "<%=_T('Unable to cancel {0} importation. Importation is already bulk copying contacts and cannot be cancelled.')%>",
										"5"                        : "<%=_T('Unable to cancel {0} importation. Importation is already merging contacts and cannot be cancelled.')%>",
										"6"                        : "<%=_T('Unable to cancel {0} importation. Importation is already finished contacts and cannot be cancelled.')%>"
									};
									if (typeof err === "object") {
										if (typeof err.err === "object") {
											err = (err.err.result || "").toString();
										}
										else {
											err = err.type || "";
										}
									}
									msg = err_messages[err.toUpperCase()] ||"<%=_T('Unable to cancel {0} importation.')%>";
									type = "Error";
								}
								view.dashboard.show_status_message(
									FORMAT(msg, import_id),
									type,
									5000
								);
							}
						);
						break;

						case "depurate":
							if (!view.form.has_permission_to_save()) {
								view.dashboard.show_status_message(
									FORMAT( "<%=_T('Unable to depurate {0} importation. You dont have permission.')%>", import_id),
									"Error",
									5000
								);
								return;
							}

							let message = FORMAT("<%=_T('Do you want to depurate Import {0} ?')%>", import_id);
							view.dashboard.confirm(message, function(is_ok) {
								if (!is_ok) {
									return;
								}
								view.dashboard.show_waiting(FORMAT("<%=_T('Wait moment please, depurating importation {0}')%>", import_id));

								outbound_engine_api.importDepurate({
										importId: import_id
									},
									function(success, err)
									{
										let msg, type;
										if (success) {
											msg = "<%=_T('Successfully depurated {0} importation.')%>";
											type = "Info";
										}
										else {
											const err_messages = {
												"ALLEGRO_INTEGRATION_ERROR": "<%=_T('Unable to depurate {0} importation. Allegro integration error.')%>",
												"FORBIDDEN"                : "<%=_T('Unable to depurate {0} importation. You dont have permission.')%>",
												"false"                    : "<%=_T('Unable to depurate {0} importation. Importation does not exist.')%>",
												"-2"                       : "<%=_T('Unable to depurate {0} importation. Importation you tried to depurate has all contacts in use. It must be free of use before depuration or deletion.')%>",
												"-3"                       : "<%=_T('Unable to depurate {0} importation. Importation you tried to depurate has no contacts.')%>"
											};
											if (typeof err === "object") {
												if (typeof err.err === "object") {
													err = (err.err.result || "").toString();
												}
												else {
													err = err.type || "";
												}
											}
											msg = err_messages[err.toUpperCase()] ||"<%=_T('Unable to depurate {0} importation.')%>";
											type = "Error";
										}
										view.dashboard.show_status_message(
											FORMAT(msg, import_id),
											type,
											5000
										);
									}
								);
							});
							break;
				}
			};

			// is action enabled
			const is_action_enabled = function( action, row )
			{
				/*
				* Valores a retornar:
				*    1  = La accion se debe visualizar y estar Enabled.
				*    0  = La accion se debe visualizar y estar Disabled.
				*    -1 = La accion no se genera en el markup. (No aparece el boton de la accion).
				*/
				if (action.toLowerCase() === "cancel") {
					let enabled_statuses = {
						RUNNING: true
					};
					if (!enabled_statuses[row.Status]) {
						return 0;
					}
				}
				else if (action.toLowerCase() === "depurate") {
					return 1;
				}
				return 1;
			};

			const get_message_error = function(type, err, name)
			{
				let message;
				if (type === "delete") {
					const err_messages = {
						"ALLEGRO_INTEGRATION_ERROR": "<%=_T('Unable to delete {0} importation. Allegro integration error.')%>",
						"FORBIDDEN"                : "<%=_T('Unable to delete {0} importation. You dont have permission.')%>",
						"-2"                       : "<%=_T('Unable to delete {0} importation. The importation you tried to delete is currently in use.')%>",
						"1205"                     : "<%=_T('Unable to delete {0} importation. It is in use by another process.')%>"
					};
					if (typeof err === "object") {
						if (typeof err.err === "object") {
							err = (err.err.result || "").toString();
						}
						else {
							err = err.type || "";
						}
					}
					message = err_messages[err.toUpperCase()] || "<%=_T('Error deleting {0} importation.')%>";
					message = FORMAT(message, name);
				}
				return message;
			};

			return {
				id: "imports",
				name: "<%=_T('Importations')%>",
				permissions: {
					view: "view_contacts_import",
					save: "save_contacts_import",
					erase: "delete_contacts_import",
					permission_type: "vcc"
				},
				class_name: "itemImports",
				i_class_name: "far fa-folder-upload",
				view: {
					request: {
						timeout: get_request_timeout()
					},
					header: {
						hide_new_button: false,
						hide_total_rows: true
					},
					paginator: {
						paged_text_format : "<%=_T('{page_row_start}-{page_row_end} of {many}')%>"
					},
					render: "/inconcert/apps/outbound_engine/pages/view_oe_content_default.<%=_L%>.html#grunt-cache-bust",
					title: "<%=_T('Importations')%>",
					tagline: "<%=_T('Importations')%>",
					new_button_name: "<%=_T('New Import')%>",
					row_id: "ID",
					display_name: "ID",
					messages: {
						erase: "<%=_T('Do you want to delete Import {0} ?')%>",
						err: get_message_error
					},
					uri: {
						erase:  base_uri + "imports/delete/",
						search: base_uri + "imports/search/"
					},
					source: ["DisplayStatus", "ID", "DBProvider", "Updates"],
					columns: [
						{id: "_number"      , name: "", type: "counter"       },
						{id: "ID"           , name: "<%=_T('Name')%>"         },
						{id: "DisplayStatus", name: "<%=_T('Status')%>", class_name: "booleanColumnTitle",
							type: "chip",
							get_value: function(row)
							{
								let resp = {};
								resp.original_value = row.DisplayStatus || row.Status;
								
								resp.value = resp.original_value;
								const states_def = {
									ABORTED: {
										value            : "<%=_T('Aborted')%>",
										class_name       : "chipStatus",
										column_class_name: "importationAborted processAborted"
									},
									COMPLETE: {
										value            : "<%=_T('Complete')%>",
										class_name       : "chipStatus",
										column_class_name: "importationCompleted processCompleted"
									},
									DELETING: {
										value            : "<%=_T('Deleting')%>",
										class_name       : "chipStatus",
										column_class_name: "importationDeleting processDeleting"
									},
									PAUSED: {
										value            : "<%=_T('Paused')%>",
										class_name       : "chipStatus",
										column_class_name: "importationPaused processPaused"
									},
									RUNNING: {
										value            : "<%=_T('Running')%>",
										class_name       : "chipStatus",
										column_class_name: "importationRunning processRunning"
									}
								};

								const state = row.Status;
								const def = states_def[ state ];
								if (!def) {
									resp.column_class_name = state;
									resp.class_name = "chipStatus";
								}
								else {
									resp.column_class_name = def.column_class_name || state;
									resp.class_name = def.class_name || "chipStatus";
									resp.value = def.value || resp.value;
								}

								return resp;
							}
						},
						{id: "DBProvider"   , name: "<%=_T('Data Provider')%>"},
						{id: "Updates"      , name: "<%=_T('Updates')%>"      },
						{id: "_action",
							class_name: "gridColumnThreeActions",
							system_action: {edit: true, erase: true},
							custom_action: {
								cancel: {
									name: "<%=_T('Cancel')%>",
									execute: execute_custom_action,
									class_name: "cancel"
								},
								depurate: {
									name: "<%=_T('Depurate')%>",
									execute: execute_custom_action,
									class_name: "depurate"
								},
							},
							is_enabled: is_action_enabled
						}
					],
					filter: {
						fields: {
							"ID"                            : {control: null},
							"ContactImportDetail.Date"      : {control: null},
							"Status"                        : {control: null},
							"ContactImportDetail.ImportUser": {control: null}
						}
					},
					realtime: null,
					// subgrilla definicion
					sub_grid: {
						data_source: "ContactImportDetails", // aca vienen los registros de la subgrilla
						colapsed: true,
						messages: function( type ) {
							let message;
							if ( type === "no_records_found" ) {
								message = "<%=_T('No rows found.')%>";
							}
							return message;
						},
						columns: [
							// las columns del colspan
							{id: "_number", name: "", type: "counter"},
							{
								id       : "Update#",
								name     : "<%=_T('Update')%>"
							},
							{
								id       : "Date",
								name     : "<%=_T('Date')%>",
								type     : "date",
								date_config: {utc: false, type_date: "full", format_date: "ymd", separator_date: "-"}
							},
							{
								id       : "ImportUser",
								name     : "<%=_T('Import User')%>"
							},
							{
								id       : "NewContacts",
								name     : "<%=_T('New Contacts')%>"
							},
							{
								id       : "UpdatedContacts",
								name     : "<%=_T('Updated Contacts')%>"
							},
							{
								id       : "InvalidContacts",
								name     : "<%=_T('Invalid Contacts')%>"
							}
						]
					}
				},
				form: {instance: frm_instance}
			};
		};

		//------------------------------------------------------
		// imports formats navigation config
		const import_formats_config_allegro = function(base_uri)
		{
			// En esta vista NO existe un Frm asociado, pero hay que agregar uno por defecto por compatibilidad.
			const frm_instance = inConcert.conf_form;

			return {
				id: "import_formats",
				name: "<%=_T('Importations Format')%>",
				permissions: {
					view: "view_contacts_import_formats",
					erase: "delete_contacts_import_formats",
					permission_type: "vcc"
				},
				class_name: "itemImportFormats",
				i_class_name: "far fa-file-upload",
				view: {
					request: {
						timeout: get_request_timeout()
					},
					header: {
						hide_new_button: true,
						hide_total_rows: true
					},
					paginator: {
						paged_text_format : "<%=_T('{page_row_start}-{page_row_end} of {many}')%>"
					},
					render: "/inconcert/apps/outbound_engine/pages/view_oe_content_default.<%=_L%>.html#grunt-cache-bust",
					title: "<%=_T('Import Formats')%>",
					tagline: "<%=_T('Import Formats')%>",
					row_id: "Id",
					display_name: "Description",
					messages: {
						erase: "<%=_T('Do you want to delete Import Formats {0} ?')%>"
					},
					uri: {
						erase:  base_uri + "imports/formats/delete/",
						search: base_uri + "imports/formats/search/"
					},
					source: ["Id", "Description"],
					columns: [
						{id: "_number"    , name: "", type: "counter"     },
						{id: "Id"         , name: "<%=_T('Name')%>"       },
						{id: "Description", name: "<%=_T('Description')%>"},
						{id: "_action"    , name: "", system_action: {erase: true}}
					],
					filter: {
						fields: {
							"Id"         : {control: null},
							"Description": {control: null},
							"Type"       : {control: null}
						}
					},
					realtime: null,
					// subgrilla definicion
					sub_grid: {
						data_source: "ContactImportFormatFields", // aca vienen los registros de la subgrilla
						colapsed: true,
						messages: function( type ) {
							let message;
							if ( type === "no_records_found" ) {
								message = "<%=_T('No rows found.')%>";
							}
							return message;
						},
						columns: [
							// las columns del colspan
							{id: "_number", name: "", type: "counter"},
							{
								id  : "DisplayType",
								name: "<%=_T('Type')%>"
							},
							{
								id  : "Name",
								name: "<%=_T('Name')%>"
							},
							{
								id  : "MappedField",
								name: "<%=_T('Mapped Field')%>"
							}
						]
					}
				},
				form: {instance: frm_instance}
			};
		};

		//------------------------------------------------------
		// outbound processes navigation config
		const outbound_processes_config_allegro = function(base_uri)
		{
			let timeout_hnd = null;
			const refresh_view = function (view, callback)
			{
				if (timeout_hnd) {
					window.clearTimeout(timeout_hnd);
					timeout_hnd = null;
				}
				timeout_hnd = window.setTimeout(function() {
					view.refresh(undefined, function() {
						callback();
					});
					timeout_hnd = null;
				}, 3000);
			};

			const show_status_message = function (view, msg, type)
			{
				view.show_status_message(
					msg,
					type,
					5000
				);
			};

			const execute_custom_action = function( action_id, $row, view )
			{
				const outbound_engine_api = inConcert.app.getOutboundEngineAPI();

				const process_id = $row.attr( "row_id" );
				switch ( action_id ) {
					case "start":
						if ( !view.form.has_permission_to_save() ) {
							show_status_message(
								view,
								FORMAT( "<%=_T('Unable to start {0} Outbound Processes. You dont have permission.')%>", name),
								"Error"
							);
							return;
						}

						view.dashboard.show_waiting(FORMAT("<%=_T('Wait moment please, starting process {0}')%>", process_id));

						outbound_engine_api.processStart({
								process_id: process_id
							},
							function(success/*, err*/)
							{
								let msg;
								if (success) {
									refresh_view(view, function() {
										msg = "<%=_T('Successfully starting {0} process.')%>";
										show_status_message(
											view,
											FORMAT(msg, process_id),
											"Info"
										);
									});
									return;
								}
								// Error
								msg = "<%=_T('Unable to start {0} process.')%>";
								show_status_message(
									view,
									FORMAT(msg, process_id),
									"Error"
								);
							}
						);
						break;

					case "stop":
						if ( !view.form.has_permission_to_save() ) {
							show_status_message(
								view,
								FORMAT( "<%=_T('Unable to stop {0} Outbound Processes. You dont have permission.')%>", name),
								"Error"
							);
							return;
						}

						view.dashboard.show_waiting(FORMAT("<%=_T('Wait moment please, stopping process {0}')%>", process_id));

						outbound_engine_api.processStop({
								process_id: process_id
							},
							function(success/*, err*/)
							{
								let msg;
								if (success) {
									refresh_view(view, function() {
										msg = "<%=_T('Successfully stopped {0} process.')%>";
										show_status_message(
											view,
											FORMAT(msg, process_id),
											"Info"
										);
									});
									return;
								}
								// Error
								msg = "<%=_T('Unable to stop {0} process.')%>";
								show_status_message(
									view,
									FORMAT(msg, process_id),
									"Error"
								);
							}
						);
						break;
				}
			};

			// is action enabled
			const is_action_enabled = function( action, row )
			{
				/*
				* Valores a retornar:
				*    1  = La accion se debe visualizar y estar Enabled.
				*    0  = La accion se debe visualizar y estar Disabled.
				*    -1 = La accion no se genera en el markup. (No aparece el boton de la accion).
				*/
				let enabled_statuses;

				if ( action.toLowerCase() === "start" ) {
					enabled_statuses = {
						STOPPED: true,
						INVALID: true
					};
					if ( !enabled_statuses[row.Status] ) {
						return 0;
					}
				}
				else if ( action.toLowerCase() === "stop" ) {
					enabled_statuses = {
						RUNNING: true
					};
					if ( !enabled_statuses[row.Status] ) {
						return 0;
					}
				}
				return 1;
			};

			const get_message_error = function(type, err, name)
			{
				let message;
				if (type === "delete") {
					const err_messages = {
						"ALLEGRO_INTEGRATION_ERROR": "<%=_T('Unable to delete {0} process. Allegro integration error.')%>",
						"FORBIDDEN"                : "<%=_T('Unable to delete {0} process. You dont have permission.')%>"
					};
					if (typeof err === "object") {
						err = err.type || "";
					}
					message = err_messages[err.toUpperCase()] || "<%=_T('Error deleting {0} process.')%>";
					message = FORMAT(message, name);
				}
				return message;
			};

			const frm_instance = inConcert.form_outbound_engine_processes_allegro;

			return {
				id: "outbound_processes",
				name: "<%=_T('Outbound Processes')%>",
				permissions: {
					view: "view_outbound_process",
					save: "save_outbound_process",
					erase: "delete_outbound_process",
					permission_type: "vcc"
				},
				class_name: "itemOutboundProcesses",
				i_class_name: "far fa-chart-network",
				view: {
					request: {
						timeout: get_request_timeout()
					},
					header: {
						hide_new_button: false,
						hide_total_rows: true
					},
					paginator: {
						paged_text_format : "<%=_T('{page_row_start}-{page_row_end} of {many}')%>"
					},
					render: "/inconcert/apps/outbound_engine/pages/view_oe_content_default.<%=_L%>.html#grunt-cache-bust",
					title: "<%=_T('Outbound Processes')%>",
					tagline: "<%=_T('Outbound Processes')%>",
					new_button_name: "<%=_T('New Process')%>",
					row_id: "Id",
					display_name: "Id",
					messages: {
						erase: "<%=_T('Do you want to delete Process {0} ?')%>",
						err: get_message_error
					},
					uri: {
						erase:  base_uri + "processes/delete/",
						search: base_uri + "processes/search/"
					},
					columns: [
						{id: "_number"          , name: "", type: "counter"     },
						{id: "Id"               , name: "<%=_T('Name')%>"       },
						{id: "DisplayStatus"    , name: "<%=_T('Status')%>", class_name: "booleanColumnTitle",
							type: "chip",
							get_value: function(row)
							{
								let resp = {};
								resp.original_value = row.DisplayStatus || row.Status;
								resp.value = resp.original_value;

								const states_def = {
									DISABLED: {
										column_class_name: "processDisabled",
										value            : "<%=_T('DISABLED')%>",
										class_name       : "chipStatus",
									},
									INVALID: {
										column_class_name: "processInvalid",
										value            : "<%=_T('INVALID')%>",
										class_name       : "chipStatus",
									},
									RUNNING: {
										column_class_name: "processRunning",
										value            : "<%=_T('RUNNING')%>",
										class_name       : "chipStatus",
									},
									STOPPED: {
										column_class_name: "processStopped",
										value            : "<%=_T('STOPPED')%>",
										class_name       : "chipStatus",
									},
									STOPPING: {
										column_class_name: "processStopping",
										value            : "<%=_T('STOPPING')%>",
										class_name       : "chipStatus",
									}
								};

								const state = row.Status;
								const def = states_def[ state ];
								if (!def) {
									resp.column_class_name = state;
									resp.class_name = "chipStatus";
								}
								else {
									resp.column_class_name = def.column_class_name || state;
									resp.class_name = def.class_name || "chipStatus";
									resp.value = def.value || resp.value;
								}

								return resp;
							}
						},
						{id: "DisplayCampaign"  , name: "<%=_T('Campaign')%>"   },
						{id: "DisplayActiveRule", name: "<%=_T('Active Rule')%>"},
						{id: "ImageActive"      , name: "<%=_T('Active')%>",
							type: "boolean",
							class_name: "booleanColumnTitle",
							get_value: function(process) {
								const has_rule = process.ImageActive.hasRule ? "1" : "0";
								return {
									original_value: process.ImageActive.hasRule,
									value: has_rule
								};
							}
						},
						{id: "DisplayContactsLoaded", name: "<%=_T('Contacts Loaded')%>"},
						{id: "_action",
							class_name: "gridColumnThreeActions",
							system_action: {edit: true, erase: true},
							custom_action: {
								start: {
									name: "<%=_T('Start')%>",
									execute: execute_custom_action,
									class_name: "start"
								},
								stop: {
									name: "<%=_T('Stop')%>",
									execute: execute_custom_action,
									class_name: "stop"
								}
							},
							is_enabled: is_action_enabled
						}
					],
					filter: {
						fields: {
							"Id"           : {control: null},
							"EngineService": {control: null},
							"Type"         : {control: null}
						}
					},
					realtime: null
				},
				form: {instance: frm_instance}
			};
		};


	/*--------------------------------------------------------------------------------------------------------------------------------
	* TEL I6 NATIVA CONFIG
	*
	 --------------------------------------------------------------------------------------------------------------------------------*/

		//------------------------------------------------------
		// imports tel I6 navigation config
		//
		const imports_config = function(base_uri)
		{
			const frm_instance = inConcert.form_outbound_engine_imports;

			const execute_custom_action = function( action_id, $row, view )
			{
				const outbound_engine_api = inConcert.app.getOutboundEngineAPI();
				const import_id = $row.attr("row_id");

				switch (action_id) {
					case "cancel":
						if (!view.form.has_permission_to_save()) {
							view.dashboard.show_status_message(
								FORMAT( "<%=_T('Unable to cancel {0} importation. You dont have permission.')%>", import_id),
								"Error",
								5000
							);
							return;
						}

						view.dashboard.show_waiting(FORMAT("<%=_T('Wait moment please, canceling importation {0}')%>", import_id));

						outbound_engine_api.importCancel({
								import_id: import_id
							},
							function(success, err)
							{
								let msg, type;
								if (success) {
									msg = "<%=_T('Successfully cancelled {0} importation.')%>";
									type = "Info";
								}
								else {
									const err_messages = {
										"FORBIDDEN": "<%=_T('Unable to cancel {0} importation. You dont have permission.')%>",
										"4"        : "<%=_T('Unable to cancel {0} importation. Importation is already bulk copying contacts and cannot be cancelled.')%>",
										"5"        : "<%=_T('Unable to cancel {0} importation. Importation is already merging contacts and cannot be cancelled.')%>",
										"6"        : "<%=_T('Unable to cancel {0} importation. Importation is already finished contacts and cannot be cancelled.')%>",
										"7"        : "<%=_T('Unable to cancel {0} importation. This action is not available.')%>"
									};
									if (typeof err === "object") {
										if (typeof err.err === "object") {
											err = (err.err.result || "").toString();
										}
										else {
											err = err.type || "";
										}
									}
									msg = err_messages[err.toUpperCase()] ||"<%=_T('Unable to cancel {0} importation.')%>";
									type = "Error";
								}
								view.dashboard.show_status_message(
									FORMAT(msg, import_id),
									type,
									5000
								);
							}
						);
						break;

						case "depurate":
							if (!view.form.has_permission_to_save()) {
								view.dashboard.show_status_message(
									FORMAT( "<%=_T('Unable to depurate {0} importation. You dont have permission.')%>", import_id),
									"Error",
									5000
								);
								return;
							}

							let message = FORMAT("<%=_T('Do you want to depurate Import {0} ?')%>", import_id);
							view.dashboard.confirm(message, function(is_ok) {
								if (!is_ok) {
									return;
								}
								view.dashboard.show_waiting(FORMAT("<%=_T('Wait moment please, depurating importation {0}')%>", import_id));

								outbound_engine_api.importDepurate({
										importId: import_id
									},
									function(success, result)
									{
										let msg, type;
										if (success) {
											if (result.totalDepuratedContacts > 0) {
												msg = FORMAT("<%=_T('Successfully depurated {0} importation. {1} contacts have been depurated.')%>", import_id, result.totalDepuratedContacts);
												type = "Info";
											}
											else {
												msg = FORMAT("<%=_T('No contacts depurated on importation {0}.')%>", import_id);
												type = "Warning";
											}
											view.dashboard.show_status_message(
												msg,
												type,
												10000
											);
											return;
										}
										// error
										let err = result;
										let err_messages = {
											"ALLEGRO_INTEGRATION_ERROR": "<%=_T('Unable to depurate {0} importation. Allegro integration error.')%>",
											"FORBIDDEN"                : "<%=_T('Unable to depurate {0} importation. You dont have permission.')%>",
											"false"                    : "<%=_T('Unable to depurate {0} importation. Importation does not exist.')%>",
											"-2"                       : "<%=_T('Unable to depurate {0} importation. Importation you tried to depurate has all contacts in use. It must be free of use before depuration or deletion.')%>",
											"-3"                       : "<%=_T('Unable to depurate {0} importation. Importation you tried to depurate has no contacts.')%>",
											"-11"                      : "<%=_T('Unable to depurate {0} importation. There is another import in use. Import {1}.')%>",
										};
										let errType;
										if (typeof err === "object") {
											if (typeof err.err === "object") {
												errType = (err.err.result || "").toString();
											}
											else {
												errType = err.type || "";
											}
										}
										msg = err_messages[errType.toUpperCase()] ||"<%=_T('Unable to depurate {0} importation.')%>";
										if (errType === "-11") {
											msg = FORMAT(msg, import_id, err.importInProgress);
										}
										else {
											msg = FORMAT(msg, import_id);
										}
										//type = "Error";
										view.dashboard.show_status_message(
											msg,
											"Error",
											5000
										);
									}
								);
							});
							break;
				}
			};

			let states_def = {
				ABORTED: {
					column_class_name: "importationAborted processAborted",
					tooltip          : "<%=_T('ABORTED')%>",
					value            : "<%=_T('ABORTED')%>",
					class_name       : "chipStatus",
					actions_states   : {
						delete          : 0,
						depurate        : 0,
						edit            : 0
					}
				},
				COMPLETE: {
					column_class_name: "importationCompleted processCompleted",
					tooltip          : "<%=_T('COMPLETE')%>",
					value            : "<%=_T('COMPLETE')%>",
					class_name       : "chipStatus",
					actions_states   : {
						delete          : 1,
						depurate        : 1,
						edit            : 1
					}
				},
				DELETING: {
					column_class_name: "importationDeleting processDeleting",
					tooltip          : "<%=_T('DELETING')%>",
					value            : "<%=_T('DELETING')%>",
					class_name       : "chipStatus",
					actions_states   : {
						delete          : 0,
						depurate        : 0,
						edit            : 0
					}
				},
				PAUSED: {
					column_class_name: "importationPaused processPaused",
					tooltip          : "<%=_T('PAUSED')%>",
					value            : "<%=_T('PAUSED')%>",
					class_name       : "chipStatus",
					actions_states   : {
						delete          : 0,
						depurate        : 0,
						edit            : 0
					}
				},
				RUNNING: {
					column_class_name: "importationRunning processRunning",
					tooltip          : "<%=_T('RUNNING')%>",
					value            : "<%=_T('RUNNING')%>",
					class_name       : "chipStatus",
					actions_states   : {
						delete          : 0,
						depurate        : 0,
						edit            : 0
					}
				}
			};

			// is action enabled
			const is_action_enabled = function(action, row)
			{
				/*
				* Valores a retornar:
				*    1  = La accion se debe visualizar y estar Enabled.
				*    0  = La accion se debe visualizar y estar Disabled.
				*    -1 = La accion no se genera en el markup. (No aparece el boton de la accion).
				*/
				let state = ((row.Status || {}).status || "").toUpperCase();
				let def = states_def[state];
				if (!def) {
					return 0;
				}
				action = action.toLowerCase();
				let action_state = def.actions_states[action];
				if (action_state) {
					return action_state;
				}
				return 0;
			};

			const get_message_error = function(type, err, name)
			{
				let message;
				if (type === "delete") {
					let err_messages = {
						"FORBIDDEN": "<%=_T('Unable to delete {0} importation. You dont have permission.')%>",
						"-1"       : "<%=_T('Unable to delete {0} importation. Importation does not exist.')%>",
						"-2"       : "<%=_T('Unable to delete {0} importation. The importation you tried to delete is currently in use.')%>",
						"-3"       : "<%=_T('Unable to delete {0} importation. There are contacts that cannot be removed.')%>",
						"-4"       : "<%=_T('Unable to delete {0} importation. There are contacts that cannot be removed.')%>",
						"-10"      : "<%=_T('Unable to delete {0} importation. There are contacts in the batch: {1}.')%>",
						"-11"      : "<%=_T('Unable to delete {0} importation. There is another import in use. Import {1}.')%>",
						"1205"     : "<%=_T('Unable to delete {0} importation. It is in use by another process.')%>"
					};
					let errType;
					if (typeof err === "object") {
						if (typeof err.err === "object") {
							errType = (err.err.result || "").toString();
						}
						else {
							errType = err.type || "";
						}
					}
					message = err_messages[errType.toUpperCase()] || "<%=_T('Error deleting {0} importation.')%>";
					if (errType === "-10") {
						message = FORMAT(message, name, err.batchId);
					}
					else if (errType === "-11") {
						message = FORMAT(message, name, err.importInProgress);
					}
					else {
						message = FORMAT(message, name);
					}
				}
				return message;
			};

			return {
				id: "imports",
				name: "<%=_T('Importations')%>",
				permissions: {
					view: "view_contacts_import",
					save: "save_contacts_import",
					erase: "delete_contacts_import",
					permission_type: "vcc"
				},
				class_name: "itemImports",
				i_class_name: "far fa-folder-upload",
				view: {
					request: {
						timeout: get_request_timeout()
					},
					header: {
						hide_new_button: false,
						hide_total_rows: true
					},
					paginator: {
						//rows_per_page: 3,
						paged_text_format : "<%=_T('{page_row_start}-{page_row_end} of {many}')%>"
					},
					render: "/inconcert/apps/outbound_engine/pages/view_oe_content_default.<%=_L%>.html#grunt-cache-bust",
					title: "<%=_T('Importations')%>",
					tagline: "<%=_T('Importations')%>",
					new_button_name: "<%=_T('New Import')%>",
					row_id: "ID",
					display_name: "ID",
					messages: {
						erase: "<%=_T('Do you want to delete Import {0} ?')%>",
						err: get_message_error
					},
					uri: {
						erase:  base_uri + "imports/delete/",
						search: base_uri + "imports/search/"
					},
					source: ["Status", "ID", "DBProvider", "Updates", "Detail"],
					columns: [
						{id: "_number", name: "", type: "counter"       },
						{id: "ID"     , name: "<%=_T('Name')%>"         },
						{id: "Status" , name: "<%=_T('Status')%>", class_name: "booleanColumnTitle",
							type: "chip",
							get_tooltip: function(row) {
								let state = row.Status.status;
								let def = states_def[state];
								if (!def) {
									return "";
								}
								return def.tooltip || "";
							},
							get_value: function(row)
							{
								let resp = {};
								resp.original_value = row.DisplayStatus || row.Status;
								resp.value = resp.original_value;
								const state = row.Status.status;
								const def = states_def[ state ];
								if (!def) {
									resp.column_class_name = state;
									resp.class_name = "chipStatus";
								}
								else {
									resp.column_class_name = def.column_class_name || state;
									resp.class_name = def.class_name || "chipStatus";
									resp.value = def.value || resp.value;
								}
								return resp;
							}
						},
						{id: "DBProvider"   , name: "<%=_T('Data Provider')%>"},
						{id: "Updates"      , name: "<%=_T('Updates')%>"      },
						{id: "_action",
							class_name: "gridColumnTwoActions",
							system_action: {edit: true, erase: true},
							custom_action: {
								depurate: {
									name: "<%=_T('Depurate')%>",
									execute: execute_custom_action,
									class_name: "depurate"
								}
							},
							is_enabled: is_action_enabled
						}
					],
					filter: {
						fields: {
							"ID"        : {control: null},
							"Status"    : {control: null},
							"DBProvider": {control: null},
							"User"      : {control: null}
						}
					},
					realtime: null,
					// subgrilla definicion
					sub_grid: {
						data_source: "Detail", // aca vienen los registros de la subgrilla
						colapsed: true,
						messages: function( type ) {
							let message;
							if ( type === "no_records_found" ) {
								message = "<%=_T('No rows found.')%>";
							}
							return message;
						},
						columns: [
							// las columns del colspan
							{id: "_number", name: "", type: "counter"},
							{
								id       : "StartDate",
								name     : "<%=_T('Date')%>",
								type     : "timestampSec",
								date_config: {utc: false, type_date: "full", format_date: "ymd", separator_date: "-"}
							},
							{
								id       : "UserId",
								name     : "<%=_T('Import User')%>",
								get_value: function(row)
								{
									let resp = {};
									resp.original_value = row.UserId;
									resp.value = inConcert.app.get_id(row.UserId);
									return resp;
								}
							},
							{
								id       : "Added",
								name     : "<%=_T('New Contacts')%>"
							},
							{
								id       : "Updated",
								name     : "<%=_T('Updated Contacts')%>"
							},
							{
								id       : "Errors",
								name     : "<%=_T('Invalid Contacts')%>"
							},
							{
								id       : "Warnings",
								name     : "<%=_T('Contacts with Alerts')%>"
							}
						]
					},
					actions: {
						erase: {
							after_erase: "refresh-view",
						}
					}
				},
				form: {instance: frm_instance}
			};
		};

		//------------------------------------------------------
		// contacts teli6 navigation config
		const contacts_config = function(base_uri)
		{
			const frm_instance = inConcert.form_outbound_engine_contacts;

			return {
				id: "contacts",
				name: "<%=_T('Contacts')%>",
				permissions: {
					view: "view_contacts",
					save: "FALSE_SAVE_PERMISSION", // Es un permiso falso, para que la vista y el frm se comporte en modo READONLY.
					permission_type: "vcc"
				},
				class_name: "itemContacts",
				i_class_name: "far fa-database",
				view: {
					request: {
						timeout: get_request_timeout()
					},
					header: {
						hide_new_button: true,
						hide_total_rows: true
					},
					paginator: {
						//rows_per_page: 2,
						paged_text_format : "<%=_T('{page_row_start}-{page_row_end} of {many}')%>"
					},
					render: "/inconcert/apps/outbound_engine/pages/view_oe_content_default.<%=_L%>.html#grunt-cache-bust",
					title: "<%=_T('Contacts')%>",
					tagline: "<%=_T('Tagline for Contacts')%>",
					row_id: "Id",
					display_name: "Name",
					uri: {
						search: base_uri + "contacts/search/"
					},
					source: ["Id", "Name", "ImportId", "IsClient"],
					columns: [
						{id: "_number", name: "", type: "counter"},
						{id: "Name", name: "<%=_T('Name')%>"},
						{id: "Id", name: "<%=_T('Id')%>",
							get_value: function({ Id: contactId })
							{
								const value = inConcert.app.get_id(contactId);
								return {
									original_value: contactId,
									value         : value,
									search_value  : value
								};
							}
						},
						{id: "ImportId", name: "<%=_T('Last Update')%>"},
						{id: "IsClient", name: "<%=_T('Client')%>", type: "boolean", class_name: "booleanColumnTitle", without_tooltip: true},
						{id: "_action", name: "", system_action: {edit: true}}
					],
					filter: {
						fields: {
							"Id"      : {control: null},
							"Name"    : {control: null},
							"ImportId": {control: null}
						}
					},
					realtime: null
				},
				form: {instance: frm_instance}
			};
		};

		//------------------------------------------------------
		// imports formats teli6 navigation config
		const import_formats_config = function(base_uri)
		{
			// En esta vista NO existe un Frm asociado, pero hay que agregar uno por defecto por compatibilidad.
			const frm_instance = inConcert.conf_form;

			return {
				id: "import_formats",
				name: "<%=_T('Importations Format')%>",
				permissions: {
					view: "view_contacts_import_formats",
					erase: "delete_contacts_import_formats",
					permission_type: "vcc"
				},
				class_name: "itemImportFormats",
				i_class_name: "far fa-file-upload",
				view: {
					request: {
						timeout: get_request_timeout()
					},
					header: {
						hide_new_button: true,
						hide_total_rows: true
					},
					paginator: {
						//rows_per_page: 2,
						paged_text_format : "<%=_T('{page_row_start}-{page_row_end} of {many}')%>"
					},
					render: "/inconcert/apps/outbound_engine/pages/view_oe_content_default.<%=_L%>.html#grunt-cache-bust",
					title: "<%=_T('Import Formats')%>",
					tagline: "<%=_T('Import Formats')%>",
					row_id: "Id",
					display_name: "Description",
					messages: {
						erase: "<%=_T('Do you want to delete Import Formats {0} ?')%>"
					},
					uri: {
						erase:  base_uri + "imports/formats/delete/",
						search: base_uri + "imports/formats/search/"
					},
					source: ["Id", "Description", "ContactImportFormatFields"],
					columns: [
						{id: "_number"    , name: "", type: "counter"     },
						{id: "Id"         , name: "<%=_T('Name')%>"       },
						{id: "Description", name: "<%=_T('Description')%>"},
						{id: "_action"    , name: "", system_action: {erase: true}}
					],
					filter: {
						fields: {
							"Id"         : {control: null},
							"Description": {control: null},
							"Type"       : {control: null}
						}
					},
					realtime: null,
					// subgrilla definicion
					sub_grid: {
						data_source: "ContactImportFormatFields", // aca vienen los registros de la subgrilla
						colapsed: true,
						messages: function( type ) {
							let message;
							if ( type === "no_records_found" ) {
								message = "<%=_T('No rows found.')%>";
							}
							return message;
						},
						columns: [
							// las columns del colspan
							{id: "_number", name: "", type: "counter"},
							{
							//	id       : "fieldName",
								id  : "DisplayType",
								name: "<%=_T('Type')%>"
							},
							{
							//	id  : "csvFieldName",
								id  : "Name",
								name: "<%=_T('Name')%>"
							},
							{
								//id  : "mappedFieldValue",
								id  : "MappedField",
								name: "<%=_T('Mapped Field')%>"
							}
						]
					}
				},
				form: {instance: frm_instance}
			};
		};

		//------------------------------------------------------
		// Blacklist teli6 navigation config
		const blacklist_config = function(base_uri)
		{
			const frm_instance = inConcert.form_outbound_engine_blacklist_imports;

			let states_def = {
				ABORTED: {
					column_class_name: "importationAborted processAborted",
					tooltip          : "<%=_T('ABORTED')%>",
					value            : "<%=_T('ABORTED')%>",
					class_name       : "chipStatus",
					actions_states   : {
						delete  : 0,
						depurate: 0,
						edit    : 0
					}
				},
				COMPLETE: {
					column_class_name: "importationCompleted processCompleted",
					tooltip          : "<%=_T('COMPLETE')%>",
					value            : "<%=_T('COMPLETE')%>",
					actions_states   : {
						delete  : 1,
						depurate: 1,
						edit    : 1
					}
				},
				DELETING: {
					column_class_name: "importationDeleting processDeleting",
					tooltip          : "<%=_T('DELETING')%>",
					value            : "<%=_T('DELETING')%>",
					actions_states   : {
						delete  : 0,
						depurate: 0,
						edit    : 0
					}
				},
				PAUSED: {
					column_class_name: "importationPaused processPaused",
					tooltip          : "<%=_T('PAUSED')%>",
					value            : "<%=_T('PAUSED')%>",
					actions_states   : {
						delete  : 0,
						depurate: 0,
						edit    : 0
					}
				},
				RUNNING: {
					column_class_name: "importationRunning processRunning",
					tooltip          : "<%=_T('RUNNING')%>",
					value            : "<%=_T('RUNNING')%>",
					actions_states   : {
						delete  : 0,
						depurate: 0,
						edit    : 0
					}
				}
			};

			// is action enabled
			const is_action_enabled = function(action, row)
			{
				/*
				* Valores a retornar:
				*    1  = La accion se debe visualizar y estar Enabled.
				*    0  = La accion se debe visualizar y estar Disabled.
				*    -1 = La accion no se genera en el markup. (No aparece el boton de la accion).
				*/
				let state = ((row.Status || {}).status || "").toUpperCase();
				let def = states_def[state];
				if (!def) {
					return 0;
				}
				action = action.toLowerCase();
				let action_state = def.actions_states[action];
				if (action_state) {
					return action_state;
				}
				return 0;
			};

			const get_message_error = function(type, err, name)
			{
				let message;
				if (type === "delete") {
					let err_messages = {
						"FORBIDDEN": "<%=_T('Unable to delete {0} importation. You dont have permission.')%>",
						"-1"       : "<%=_T('Unable to delete {0} importation. Importation does not exist.')%>",
						"-2"       : "<%=_T('Unable to delete {0} importation. The importation you tried to delete is currently in use.')%>",
						"-3"       : "<%=_T('Unable to delete {0} importation. There are contacts that cannot be removed.')%>",
						"-4"       : "<%=_T('Unable to delete {0} importation. There are contacts that cannot be removed.')%>",
						"-5"       : "<%=_T('Unable to delete {0} importation. It is in use in some campaigns.')%>",
						"-10"      : "<%=_T('Unable to delete {0} importation. There are contacts in the batch: {1}.')%>",
						"-11"      : "<%=_T('Unable to delete {0} importation. There is another import in use. Import {1}.')%>",
						"1205"     : "<%=_T('Unable to delete {0} importation. It is in use by another process.')%>"
					};
					let errType;
					if (typeof err === "object") {
						if (typeof err.err === "object") {
							errType = err.err.result;
						}
						else {
							errType = err.type;
						}
						errType = (errType || "").toString();
					}
					message = err_messages[errType.toUpperCase()] || "<%=_T('Error deleting {0} importation.')%>";
					if (errType === "-10") {
						message = FORMAT(message, name, err.batchId);
					}
					else if (errType === "-11") {
						message = FORMAT(message, name, err.importInProgress);
					}
					else {
						message = FORMAT(message, name);
					}
				}
				return message;
			};
			
			return {
				id: "blacklist_imports",
				name: "<%=_T('Blacklist Importations')%>",
				permissions: {
					view: "view_blacklist",
					save: "save_blacklist",
					erase: "delete_blacklist",
					permission_type: "vcc"
				},
				class_name: "itemImports",
				i_class_name: "far fa-user-times",
				view: {
					request: {
						timeout: get_request_timeout()
					},
					header: {
						hide_new_button: false,
						hide_total_rows: true
					},
					paginator: {
						//rows_per_page: 3,
						paged_text_format : "<%=_T('{page_row_start}-{page_row_end} of {many}')%>"
					},
					render: "/inconcert/apps/outbound_engine/pages/view_oe_content_default.<%=_L%>.html#grunt-cache-bust",
					title: "<%=_T('Blacklist Importations')%>",
					tagline: "<%=_T('Blacklist Importations')%>",
					new_button_name: "<%=_T('New Import')%>",
					row_id: "ID",
					display_name: "ID",
					messages: {
						erase: "<%=_T('Do you want to delete Import {0} ?')%>",
						err: get_message_error
					},
					uri: {
						erase:  base_uri + "blacklist/delete/",
						search: base_uri + "blacklist/search/"
					},
					source: ["Status", "ID", "Updates", "Detail", "Campaigns", "Description"],
					columns: [
						{id: "_number", name: "", type: "counter"       },
						{id: "ID"     , name: "<%=_T('Name')%>"         },
						{id: "Status" , name: "<%=_T('Status')%>", class_name: "booleanColumnTitle",
							type: "chip",
							get_tooltip: function(row) {
								let state = row.Status.status;
								let def = states_def[state];
								if (!def) {
									return "";
								}
								return def.tooltip || "";
							},
							get_value: function(row)
							{
								let resp = {};
								resp.original_value = row.DisplayStatus || row.Status;
								resp.value = resp.original_value;
								let state = row.Status.status;
								let def = states_def[ state ];
								if (!def) {
									resp.column_class_name = state;
									resp.class_name = "chipStatus";
								}
								else {
									resp.column_class_name = def.column_class_name || state;
									resp.class_name = def.class_name || "chipStatus";
									resp.value = def.value || resp.value;
								}
								return resp;
							}
						},
						{id: "Updates"    , name: "<%=_T('Updates')%>"    },
						{id: "Description", name: "<%=_T('Description')%>"},
						{id: "Campaigns"  , name: "<%=_T('Campaigns')%>", get_value: function(row) {
								let resp = {};
								resp.original_value = row.Campaigns || {};
								let value = "";
								for (let campaignId in row.Campaigns) {
									if (row.Campaigns.hasOwnProperty(campaignId)) {
										if (value.length > 0) {
											value = value + ", ";
										}
										value = value + inConcert.app.get_id(campaignId);
									}
								}
								resp.value = value;
								return resp;
							}
						},
						{
							id: "_action",
							class_name: "gridColumnTwoActions",
							system_action: {edit: true, erase: true},
							is_enabled: is_action_enabled
						}
					],
					filter: {
						fields: {
							"ID"         : {control: null},
							"Status"     : {control: null},
							"User"       : {control: null},
							"Description": {control: null},
						}
					},
					realtime: null,
					// subgrilla definicion
					sub_grid: {
						data_source: "Detail", // aca vienen los registros de la subgrilla
						colapsed: true,
						messages: function( type ) {
							let message;
							if ( type === "no_records_found" ) {
								message = "<%=_T('No rows found.')%>";
							}
							return message;
						},
						columns: [
							// las columns del colspan
							{id: "_number", name: "", type: "counter"},
							{
								id       : "StartDate",
								name     : "<%=_T('Date')%>",
								type     : "timestampSec",
								date_config: {utc: false, type_date: "full", format_date: "ymd", separator_date: "-"}
							},
							{
								id       : "UserId",
								name     : "<%=_T('Import User')%>",
								get_value: function(row)
								{
									let resp = {};
									resp.original_value = row.UserId;
									resp.value = inConcert.app.get_id(row.UserId);
									return resp;
								}
							},
							{
								id       : "Added",
								name     : "<%=_T('New Addresses')%>"
							},
							{
								id       : "Updated",
								name     : "<%=_T('Updated Addresses')%>"
							},
							{
								id       : "Errors",
								name     : "<%=_T('Invalid Addresses')%>"
							}
						]
					},
					actions: {
						erase: {
							after_erase: "refresh-view",
						}
					}
				},
				form: {instance: frm_instance}
			};
		};

		//------------------------------------------------------
		// Blacklist addresses teli6 navigation config
		const blacklist_addresses_config = function(base_uri)
		{
			// En esta vista NO existe un Frm asociado, pero hay que agregar uno por defecto por compatibilidad.
			const frm_instance = inConcert.conf_form;

			return {
				id: "blacklist",
				name: "<%=_T('Blacklist Addresses')%>",
				permissions: {
					view: "view_blacklist",
					save: "FALSE_SAVE_PERMISSION", // Es un permiso falso, para que la vista y el frm se comporte en modo READONLY.
					permission_type: "vcc"
				},
				class_name: "itemBlackList",
				i_class_name: "far fa-ban",
				view: {
					request: {
						timeout: get_request_timeout()
					},
					header: {
						hide_new_button: true,
						hide_total_rows: true
					},
					paginator: {
						//rows_per_page: 2,
						paged_text_format : "<%=_T('{page_row_start}-{page_row_end} of {many}')%>"
					},
					render: "/inconcert/apps/outbound_engine/pages/view_oe_content_default.<%=_L%>.html#grunt-cache-bust",
					title: "<%=_T('Blacklist')%>",
					tagline: "<%=_T('Tagline for Blacklist Addresses')%>",
					row_id: "Key",
					display_name: "Id", //"Name",
					uri: {
						search: base_uri + "blacklist/addresses/search/"
					},
					source: ["Id", "Scope", "Key", "Type", "VCC", "Channels", "AddressInfo", "ImportId"],
					columns: [
						{id: "_number" , name: "", type: "counter"},
						{id: "Id"      , name: "<%=_T('Id')%>", get_value: function(row) {
							if (row.Type.toUpperCase() === "PHONE") {
								let fullNumber = row.AddressInfo.displayable;
								return {
									original_value: fullNumber,
									value         : fullNumber
								};
							}
							// TODO: Implementar para otros tipos de addresses.
							return {
								original_value: row.Id,
								value         : row.Id
							};
						}},
						{id: "Type", name: "<%=_T('Address Type')%>", get_value: function(row) {
								if (row.Type.toUpperCase() === "PHONE") {
									return {
										original_value: row.Type,
										value         : "<%=_T('Phone')%>"
									};
								}
								// TODO: Implementar para otros tipos de addresses.
								return {
									original_value: row.Type,
									value         : row.Type
								};
							}
						},
						{id: "ImportId", name: "<%=_T('Import Name')%>"},
						// No tenemos acciones en esta grilla, pero la definicion tiene que existir igual, para que la ultima columna se vea correctamente.
						{
							id: "_action",
							class_name: "gridColumnTwoActions"
						}
					],
					filter: {
						fields: {
							"Id"      : {control: null},
							"Type"    : {control: null},
							"ImportId": {control: null}
						}
					},
					realtime: null
				},
				form: {instance: frm_instance}
			};
		};

		//------------------------------------------------------
		// outbound processes navigation config
		const outbound_processes_config = function(base_uri)
		{
			let timeout_hnd = null;
			const refresh_view = function (view, callback)
			{
				if (timeout_hnd) {
					window.clearTimeout(timeout_hnd);
					timeout_hnd = null;
				}
				timeout_hnd = window.setTimeout(function() {
					view.refresh(undefined, function() {
						callback();
					});
					timeout_hnd = null;
				}, 3000);
			};

			const show_status_message = function (view, msg, type)
			{
				view.show_status_message(
					msg,
					type,
					5000
				);
			};

			const execute_custom_action = function( action_id, $row, view )
			{
				const outbound_engine_api = inConcert.app.getOutboundEngineAPI();

				let process_id = $row.attr( "row_id" );
				switch ( action_id ) {
					case "start":
						if ( !view.form.has_permission_to_save() ) {
							show_status_message(
								view,
								FORMAT( "<%=_T('Unable to start {0} Outbound Processes. You dont have permission.')%>", name),
								"Error"
							);
							return;
						}

						view.dashboard.show_waiting(FORMAT("<%=_T('Wait moment please, starting process {0}')%>", process_id));

						outbound_engine_api.processStart({
								process_id: process_id
							},
							function(success/*, err*/)
							{
								let msg;
								if (success) {
									refresh_view(view, function() {
										msg = "<%=_T('Successfully starting {0} process.')%>";
										show_status_message(
											view,
											FORMAT(msg, process_id),
											"Info"
										);
									});
									return;
								}
								// Error
								msg = "<%=_T('Unable to start {0} process.')%>";
								show_status_message(
									view,
									FORMAT(msg, process_id),
									"Error"
								);
							}
						);
						break;

					case "stop":
						if ( !view.form.has_permission_to_save() ) {
							show_status_message(
								view,
								FORMAT( "<%=_T('Unable to stop {0} Outbound Processes. You dont have permission.')%>", name),
								"Error"
							);
							return;
						}

						view.dashboard.show_waiting(FORMAT("<%=_T('Wait moment please, stopping process {0}')%>", process_id));

						outbound_engine_api.processStop({
								process_id: process_id
							},
							function(success/*, err*/)
							{
								let msg;
								if (success) {
									refresh_view(view, function() {
										msg = "<%=_T('Successfully stopped {0} process.')%>";
										show_status_message(
											view,
											FORMAT(msg, process_id),
											"Info"
										);
									});
									return;
								}
								// Error
								msg = "<%=_T('Unable to stop {0} process.')%>";
								show_status_message(
									view,
									FORMAT(msg, process_id),
									"Error"
								);
							}
						);
						break;
				}
			};

			// is action enabled
			const is_action_enabled = function( action, row )
			{
				/*
				* Valores a retornar:
				*    1  = La accion se debe visualizar y estar Enabled.
				*    0  = La accion se debe visualizar y estar Disabled.
				*    -1 = La accion no se genera en el markup. (No aparece el boton de la accion).
				*/
				let action_id = action.toLowerCase();

				if (action_id === "start" || action_id === "stop") {
					if (!row.configuration || !row.configuration.campaign) {
						return 0;
					}
					let enabled_statuses;
					let process_state = row.state || "CREATED";
					if (action_id === "start") {
						enabled_statuses = {
							STOPPED: true,
							INVALID: true,
							CREATED: true
						};
						if (!enabled_statuses[process_state]) {
							return 0;
						}
					}
					else {
						enabled_statuses = {
							STARTED: true
						};
						if (!enabled_statuses[process_state]) {
							return 0;
						}
					}
				}
				return 1;
			};

			const get_message_error = function(type, err, name)
			{
				let message;
				if (type === "delete") {
					let value = "";
					let err_messages = {
						"FORBIDDEN"                              : "<%=_T('Unable to delete {0} process. You dont have permission.')%>",
						"PROCESS DOES NOT EXIST"                 : "<%=_T('Unable to delete {0} process. Process does not exist.')%>",
						"PROCESS IS RUNNING"                     : "<%=_T('Unable to delete {0} process. Process is running.')%>",
						"UNABLE TO DELETE PROCESS - BATCH IN USE": "<%=_T('Unable to delete {0} process. The batch {1} is in use.')%>",
						"PROCESS IS ASSIGNED TO ACCOUNT"		 : "<%=_T('Unable to delete {0} process. Process assigned to account {1}.')%>"
					};
					let customInfo;
					if (typeof err === "object") {
						customInfo = err.info;
						if (err.type === "UNABLE TO DELETE PROCESS - BATCH IN USE") {
							value = err.batchName;
						} else if (err.type === "PROCESS IS ASSIGNED TO ACCOUNT") {
							value = err.accountName;
						}
						err = err.type || "";
					}
					message = err_messages[err.toUpperCase()] || "<%=_T('Error deleting {0} process.')%>";
					message = FORMAT(message, name, value);
					if (customInfo) {
						if (customInfo.subType === "ADMIN_ACTIONS_RUNNING") {
							message = `${message} <%=_T('Process has admin actions running.')%>`;
						}
					}
				}
				return message;
			};

			const get_batch_action_tooltip = (actionName) => {
				switch(actionName) {
					case "CancelContacts":
						return "<%=_T('Cancel Contacts')%>";
					case "ChangePriorityContacts":
						return "<%=_T('Change Contacts Priority')%>";
					case "CreateBatch":
						return "<%=_T('Create Batch')%>";
					case "DestroyBatch":
						return "<%=_T('Destroy Batch')%>";
					case "FinishContacts":
						return "<%=_T('Finish Contacts')%>";
					case "PauseContacts":
						return "<%=_T('Pause Contacts')%>";
					case "RecycleContacts":
						return "<%=_T('Recycle Contacts')%>";
					case "ReloadContacts":
						return "<%=_T('Reload Contacts')%>";
					case "RemoveContacts":
						return "<%=_T('Remove Contacts')%>";
					case "ResumeContacts":
						return "<%=_T('Resume Contacts')%>";
					case "ScheduleContacts":
						return "<%=_T('Schedule Contacts')%>";
					case "SetContactsAgent":
						return "<%=_T('Set Contacts Agent')%>";
					case "SetEngineTypeContacts":
						return "<%=_T('Set Contacts Engine Type')%>";
					default:
						return actionName;
				}
			};

			const frm_instance = inConcert.form_outbound_engine_processes;

			return {
				id: "outbound_processes",
				name: "<%=_T('Outbound Processes')%>",
				permissions: {
					view: "view_outbound_process",
					save: "save_outbound_process",
					erase: "delete_outbound_process",
					permission_type: "vcc"
				},
				class_name: "itemOutboundProcesses",
				i_class_name: "far fa-chart-network",
				view: {
					request: {
						timeout: get_request_timeout()
					},
					header: {
						hide_new_button: false,
						hide_total_rows: true
					},
					paginator: {
						paged_text_format : "<%=_T('{page_row_start}-{page_row_end} of {many}')%>"
					},
					render: "/inconcert/apps/outbound_engine/pages/view_oe_content_default.<%=_L%>.html#grunt-cache-bust",
					title: "<%=_T('Outbound Processes')%>",
					tagline: "<%=_T('Outbound Processes')%>",
					new_button_name: "<%=_T('New Process')%>",
					row_id: "id",
					display_name: "id",
					auto_reload_seconds: 10,
					messages: {
						erase: "<%=_T('Do you want to delete Process {0} ?')%>",
						err: get_message_error
					},
					uri: {
						erase:  base_uri + "processes/delete/",
						search: base_uri + "processes/search/"
					},
					columns: [
						{id: "_number"          , name: "", type: "counter"     },
						{id: "id"               , name: "<%=_T('Name')%>",
							get_value: function(row)
							{
								let id_with_vcc = (row.id || "");
								let resp = {
									original_value: id_with_vcc,
									value: inConcert.app.get_id(id_with_vcc)
								};
								return resp;
							}
						},
						{id: "DisplayStatus", name: "<%=_T('Status')%>", class_name: "leftAlign booleanColumnTitle",
							type: "chip",
							get_value: function(row)
							{
								let resp = {};
								resp.original_value = row.state;
								resp.value = resp.original_value;
								/*
									STARTING
									STARTED
									STOPPED
									STOPPING
								*/
								const states_def = {
									DISABLED: {
										value            : "<%=_T('Disabled')%>",
										class_name       : "chipStatus",
										column_class_name: "leftAlign processDisabled"
									},
									INVALID: {
										value            : "<%=_T('Invalid')%>",
										class_name       : "chipStatus",
										column_class_name: "leftAlign processInvalid"
									},
									STARTED: {
										value            : "<%=_T('Running')%>",
										class_name       : "chipStatus",
										column_class_name: "leftAlign processRunning"
									},
									STARTING: {
										value            : "<%=_T('Running')%>",
										class_name       : "chipStatus",
										column_class_name: "leftAlign processRunning"
									},
									STOPPED: {
										value            : "<%=_T('Stopped')%>",
										class_name       : "chipStatus",
										column_class_name: "leftAlign processStopped"
									},
									STOPPING: {
										value            : "<%=_T('Stopped')%>",
										class_name       : "chipStatus",
										column_class_name: "leftAlign processStopping"
									}
								};
								let state = row.state;
								if (!row.configuration || !row.configuration.campaign) {
									state = "INVALID";
								}
								const defaul_class_name = "chipStatus";
								const def = states_def[ state ];
								if (!def) {
									resp.class_name = defaul_class_name;
									resp.column_class_name = state;
								}
								else {
									resp.class_name = def.class_name || defaul_class_name;
									resp.column_class_name =  def.column_class_name || defaul_class_name;
									resp.value = def.value || resp.value;
								}

								return resp;
							}
						},
						{id: "DisplayStatus", name: "<%=_T('Status Detail')%>",type:"textWithIcon", without_tooltip: true,
							get_value: function(row)
							{
								let status;
								let resp = {};
								//console.log(row);
								//console.log(row.statusCause);
								const states_def = {
									OUT_OF_DAY_RANGE: {
										value : "<%=_T('Schedule day is out of range')%>"
									},
									SORTING_CONTACTS: {
										value : "<%=_T('Sorting contacts')%>"
									},
									NOT_RUNNING: {
										value : "<%=_T('Process is not running')%>"
									},
									RUNNING: {
										value : "<%=_T('Running contacts')%>"
									},
									WITHOUT_CAMPAIGN: {
										value : "<%=_T('Process has not campaign')%>"
									},
									NO_AVAILABLE_CHANNELS: {
										value : "<%=_T('No available channels')%>"
									},
									CASSANDRA_DISCONNECTED: {
										value : "<%=_T('Cassandra disconnected')%>"
									},
									NO_VALID_CHANNELS: {
										value : "<%=_T('No valid rule channel found')%>"
									},
									BETWEEN_SCHEDULES: {
										value : "<%=_T('Date is between day schedules')%>"
									},
									ACTIVE_RULE_HASNT_CHANNEL: {
										value : "<%=_T('Active rule hasn´t channel')%>"
									},
									WITHOUT_SCHEDULE: {
										value : "<%=_T('Process has not schedule')%>"
									},
									NO_ACTIVE_RULE: {
										value : "<%=_T('No active rule found')%>"
									},
									BEFORE_SCHEDULE: {
										value : "<%=_T('Date is before day schedule')%>"
									},
									DAY_NOT_CONFIGURED: {
										value : "<%=_T('Schedule day not configured')%>"
									},
									NO_PREFERRED_SKILL_AGENTS: {
										value : "<%=_T('No agents for preferred skills')%>"
									},
									NO_ACTIVE_AGENTS: {
										value : "<%=_T('No active agents')%>"
									},
									NO_ALGORITHM_CHANNELS: {
										value : "<%=_T('No available algorithm channels')%>"
									},
									NO_REMAINING_FREE_CONTACTS: {
										value : "<%=_T('No remaining free contacts')%>"
									},
									NO_ASSIGNED_CONTACTS: {
										value : "<%=_T('No assigned contacts')%>"
									},
									NO_AVAILABLE_AGENTS: {
										value : "<%=_T('No available agents')%>"
									},
									INITIALIZE: {
										value : "<%=_T('Initializing process')%>"
									},
									NOT_READY: {
										value : "<%=_T('Service is not ready yet')%>"
									},
									AFTER_SCHEDULE: {
										value : "<%=_T('Date is after day schedules')%>"
									},
									NO_AVAILABLE_AGENT_CHANNELS: {
										value : "<%=_T('No available agent channels')%>"
									},
									NO_REQUIRED_SKILL_AGENTS: {
										value :  "<%=_T('No agents for required skills')%>"
									},
									ELASTICSEARCH_DISCONNECTED: {
										value :  "<%=_T('Elastic search disconnected')%>"
									},
									ELASTICSEARCH_RECONNECTING: {
										value :  "<%=_T('Process has queued ES tasks')%>"
									}
								};
								if (row.statusCause.result) {
									let resp_text = {};
									resp_text.alerted=false;
									const isAlerted = function(data){
										//return true;
										const alerted_list=[
											"CASSANDRA_DISCONNECTED",
											"NOT_READY",
											"WITHOUT_CAMPAIGN",
											"WITHOUT_SCHEDULE",
											"SORTING_CONTACTS",
											"ELASTICSEARCH_DISCONNECTED",
											"ELASTICSEARCH_RECONNECTING"
										];
										return alerted_list.indexOf(data) !== -1;
									};
									let orig_status = row.statusCause.result;
									status = row.statusCause.result;
									if(states_def[orig_status]){
										resp_text.text=states_def[orig_status].value;
									}
									if(isAlerted(orig_status)){
										resp_text.alerted=true;
									}
									status=resp_text;
								}
								resp.value=status;
								return resp;
							}
						},
						{id: "DisplayAction", name: "<%=_T('Action in progress')%>", without_tooltip: true,
							type:"progressbar",
							get_value: function(row)
							{
								const runningAction = row.runningBatchActions[0]?.statistics;
								if (!runningAction) {
									return { value: null };
								}

								const actionName = row.runningBatchActions[0].action;
								const progressBar = {
									percentage: Math.trunc(runningAction.percentage) || 0,
									remainingTime: "00:00:00",
									tooltip: get_batch_action_tooltip(actionName)
								};
								if (!runningAction.eta) {
									return { value : progressBar };
								}
								const seconds = Math.trunc(runningAction.eta);
								const hours = Math.floor(seconds / 3600);
								const minutes = Math.floor((seconds % 3600) / 60);
								const remainingSeconds = seconds % 60;
								const hh = String(hours).padStart(2, "0");
								const mm = String(minutes).padStart(2, "0");
								const ss = String(remainingSeconds).padStart(2, "0");
								progressBar.remainingTime=`${hh}:${mm}:${ss}`;
								return { value : progressBar };
							}
						},
						{id: "DisplayCampaign", name: "<%=_T('Campaign')%>",
							get_value: function(row)
							{
								let campaign_name;
								let resp = {};
								if (row.configuration) {
									campaign_name = row.configuration.campaign;
									let campaign = inConcert.app.getCampaign(campaign_name);
									if (campaign) {
										campaign_name = campaign.getDescription();
									}
								}
								resp.original_value = campaign_name || "<%=_T('None')%>";
								resp.value = resp.original_value;
								return resp;
							}
						},
						{id: "DisplayAccount", name: "<%=_T('Accounts')%>",
							get_value: function(row)
							{
								let accounts_name;
								let resp = {};
								if (row.configuration && (row.configuration.channels)) {
									let campaign = inConcert.app.getCampaign(row.configuration.campaign);
									const get_account_name = function(account_id, channel) {
										let display_account = account_id;
										if (campaign) {
											let account = campaign.getAccount(account_id);
											if (account) {
												display_account = campaign.getAccountDescription(account) || display_account;
											}
										}
										const translated_channels = {
											CALL    : "<%=_T('Call')%>",
											MAIL    : "<%=_T('E-mail')%>",
											SMS     : "<%=_T('SMS')%>",
											WHATSAPP: "<%=_T('WhatsApp')%>"
										};
										channel = channel.toUpperCase();
										let translated_channel = translated_channels[channel] || channel;
										return `${translated_channel}: ${display_account}`;
									};
									for (let channel in row.configuration.channels) {
										if (row.configuration.channels.hasOwnProperty(channel)) {
											let channel_info = row.configuration.channels[channel];
											if (!accounts_name) {
												accounts_name = get_account_name(channel_info.account, channel);
											}
											else {
												accounts_name = accounts_name + ", " + get_account_name(channel_info.account, channel);
											}
										}
									} 
								}
								resp.original_value = accounts_name || "<%=_T('None')%>";
								resp.value = resp.original_value;
								return resp;
							}
						},
						// {id: "DisplayActiveRule", name: "<%=_T('Active Rule')%>"},
						// {id: "DisplayContactsLoaded", name: "<%=_T('Contacts Loaded')%>"},
						{id: "_action",
							class_name: "gridColumnThreeActions",
							system_action: {edit: true, erase: true},
							custom_action: {
								start: {
									name: "<%=_T('Start')%>",
									execute: execute_custom_action,
									class_name: "start"
								},
								stop: {
									name: "<%=_T('Stop')%>",
									execute: execute_custom_action,
									class_name: "stop"
								}
							},
							is_enabled: is_action_enabled
						}
					],
					filter: {
						fields: {
							"id"           : {control: null},
							// "EngineService": {control: null},
							// "Type"         : {control: null}
						}
					},
					realtime: null,
					reload_when_enter_application: true // para que realice un reload de la vista, cuando ingrese desde otra App (interaction, supervisor, analytics, etc)
				},
				form: {instance: frm_instance}
			};
		};

	//------------------------------------------------------
	// get all navigation config
	const get_navigation_config = function()
	{
		let base_uri = inConcert.app.getOutboundEngineUrl() + "/";
		if (inConcert.app.has_allegro_integration()) {
			// VCCs CON Integracion con Allegro
			return {
				outbound_processes: outbound_processes_config_allegro(base_uri),
				imports           : imports_config_allegro(base_uri),
				import_formats    : import_formats_config_allegro(base_uri),
				contacts          : contacts_config_allegro(base_uri),
				exports           : exports_config_allegro(base_uri),
				export_layout     : export_layout_config_allegro(base_uri)
			};
		}
		// VCCs CON Telefonia Nativa / SIN Integracion Allegro
		return {
			outbound_processes  : outbound_processes_config(base_uri),
			imports             : imports_config(base_uri),
			import_formats      : import_formats_config(base_uri),
			contacts            : contacts_config(base_uri),
			black_list          : blacklist_config(base_uri),
			black_list_addresses: blacklist_addresses_config(base_uri)
		};
	};

	function outbound_engine_config() {}

	if (typeof inConcert.outbound_engine_config !== "function") {
		inConcert.outbound_engine_config = outbound_engine_config;
	}

	outbound_engine_config.prototype.get_app_config = function()
	{
		return {
			id         : "outbound_engine",
			name       : "<%=_T('Outbound Engine')%>",
			selected   : false,
			class_name : "appOutboundEngine",
			main_html  : "/inconcert/apps/outbound_engine/pages/conf_oe_main.<%=_L%>.html#grunt-cache-bust",
			navigation : get_navigation_config(),
			get_skin: function() {
				return "skinOCCNew";
			},
			can_show   : function() {
				// if (!inConcert.app.has_allegro_integration()) {
				// 	return false;
				// }
				return inConcert.permissions.to_category("outbound_engine", "view");
			},
			css : [
				{ id: "outbound_engine_master_css"            , href: "/inconcert/apps/outbound_engine/css/skin-2021/outboundEngineMaster.css#grunt-cache-bust"        },
				// TODO: revisar. Aca se agrego el CSS de administration porque los popups de edicion no se visualizan correctamente "internalPopUpContainer"
				{ id: "adm_master_css"                        , href: "/inconcert/apps/administration/css/skin-2021/administrationMaster.css#grunt-cache-bust"         },
				{ id: "adm_schedule_css"                      , href: "/inconcert/apps/administration/css/skin-2021/administrationScheduleConfig.css#grunt-cache-bust" },
				// Agrega Lolo para SkinOCCNew
				{ id: "skin_occ_internal_forms_css"           , href: "/inconcert/apps/configuration/css/skin-2021/masterInternalPageForms.css#grunt-cache-bust"       },
				{ id: "skin_occ_internal_page_layout_css"     , href: "/inconcert/apps/configuration/css/skin-2021/masterInternalPageLayout.css#grunt-cache-bust"      },
				{ id: "skin_occ_messages_tooltips_css"        , href: "/inconcert/apps/agent/css/skin-2021/genericMessagesAndTooltips.css#grunt-cache-bust"            },
				{ id: "skin_occ_messages_tooltips_css"        , href: "/inconcert/apps/agent/css/skin-2021/genericMessagesAndTooltips.css#grunt-cache-bust"            },
				//Estos hay que ver si se pueden sacar de la inboxMaster.html
				{ id: "skin_occ_master_buttons_css"           , href: "/inconcert/apps/agent/css/skin-2021/masterButtons.css#grunt-cache-bust"                         },
				{ id: "skin_occ_inbox_master_css"             , href: "/inconcert/apps/agent/css/skin-2021/inboxMaster.css#grunt-cache-bust"                           },
				{ id: "skin_occ_master_quick_interactions_css", href: "/inconcert/apps/agent/css/skin-2021/masterQuickInteractions.css#grunt-cache-bust"               },
				{ id: "skin_occ_inbox_master_popups_css"      , href: "/inconcert/apps/agent/css/skin-2021/inboxMasterPopUps.css#grunt-cache-bust"                     },
				{ id: "skin_occ_inbox_master_window_css"      , href: "/inconcert/apps/agent/css/skin-2021/masterWindowCommon.css#grunt-cache-bust"                    },
				{ id: "skin_occ_generic_dialog_master_css"    , href: "/inconcert/apps/agent/css/skin-2021/genericDialogMaster.css#grunt-cache-bust"                   }
			]
		};
	};
}());
