module(..., package.seeall, inconcert.orbit.application.Decorate)
local utils = require "shared.apps.utils"
local json = require "cjson"
local m_helpers = require "helpers"
local modelEnvironment = require("apps.provisioning.models.environment")
require "uuid"
require "LuaOrm.Date"
local m_moduleLocation = select(2, ...)
local wsapi = require "wsapi"
wsapi.request = require "wsapi.request"
m_htmlRender = nil
m_appLocation = nil



local function Render (web)
	_M.response.DontCache(web)
	local env = { _T = m_htmlRender.extension.makeTextTranslator() }
	local userId = inconcert.authUser.userName

	LogInfo("Render form environment for user %s, group %s", userId, inconcert.authUser.userGroup)

	local html_file = m_appLocation .. "/pages/frm_environment_content.html"
	local content = m_htmlRender:RenderPageLP(html_file, web, env)
	if not content then
		LogError("HTML environment does not exist: %s", html_file)
	end
	return content
end

local function Get (web)
	local userId = inconcert.authUser.userName
	LogInfo("[Environment] getting by logged UserId %s", userId)

	local keyspace = utils.GetDatasource():CassandraConnection( "Agent" )
	local installation = modelEnvironment.GetInstalationID() or {}
	local hotfixes = modelEnvironment.GetHotfixes(keyspace)

	local arrHotfixes = {}
	if hotfixes then
		arrHotfixes =  utils.SortArray(utils.HashToArray(hotfixes, "HotfixId"), function(row)
			assert(row)
			local d = LuaOrm.Date(row.timeStamp)
			local dataType = "date"
			return d, dataType
		end, true)
	end
	LogInfo("[Environment] installation %s hotfixes %s", json.encode(installation), json.encode(arrHotfixes))
	
	return {
		Installation = installation,
		Hotfixes = arrHotfixes or {}
	}
end

function MapUris(module, htmlRender, prefix, appLocation)
	local LogRequest = require("apps.agent.requestFilters").LogRequest

	module:dispatch_get(utils.JsonReply .. LogRequest.. Get, prefix .. "/get/")
	
	module:dispatch_get(Render, prefix .. "/frm/render/")

	m_htmlRender = htmlRender
	m_appLocation = appLocation
end