local utils = require "shared.apps.utils"
module(..., package.seeall, inconcert.orbit.application.Decorate)

local LOG_PREFIX = "M I G R A T I O N ==> "
--------------------------------------------------------------------------------------
--[[
	TODO: Mover a helpers.
]]--
local function IsEnabled (value)
	value = value or "0"
	return tostring(value) == "1"
end

--------------------------------------------------------------------------------------
local m_migration = {}
local m_migrationsSteps = {}

--------------------------------------------------------------------------------------
-- STEP ADMINISTRATION - CAMPAIGNS ------
--------------------------------------------------------------------------------------

local m_campaignsStep = {
	id = "campaigns",
	migrationId = nil, -- es el id de la definicion del archivo "/migrations/init.lua" tabla m_migrationsConfig[0,...].id
	description = "Campaigns Step Module Migration"
}

--[[

]]--
m_campaignsStep.Start = function(info)
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CAMPAIGNS.Start] Start Migration Campaigns for vcc '%s'.", info.vccId)
	local vccCampaigns = utils.GetVCCCampaigns(info.vccId, info.ksAgent)
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CAMPAIGNS.Start] --> vcc: %s Campaigns: '%s'.", info.vccId, utils.JSON_Encode(vccCampaigns))
	local runningInfo = {
		stepProcessId = "migrate campaigns",
		campaigns = {}
	}
	-- start running
	info.runningModule.UpdateStepProcess(m_campaignsStep, runningInfo)
	-- comienzo el proceso de migracion
	for campaignId in pairs(vccCampaigns) do
		local isAllegroCampaign, callAllegroAccount = utils.IsAllegroCampaign(campaignId, info.ksAgent, info.vccId)
		LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CAMPAIGNS.Start]    --> campaignId: %s@%s, isAllegroCampaign: %s,  callAllegroAccount: '%s'.", campaignId, info.vccId, isAllegroCampaign, utils.JSON_Encode(callAllegroAccount))
		if isAllegroCampaign then
			local success, result = m_campaignsStep.Update(campaignId, callAllegroAccount, info)
			--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CAMPAIGNS.Start]        --> campaignId: %s@%s, success: %s,  result: '%s'.", campaignId, info.vccId, success, utils.JSON_Encode(result))
			table.insert(runningInfo.campaigns, {
				campaignId  = campaignId,
				accountId   = callAllegroAccount.id,
				success     = success,
				warningList = result
			})
			-- end running current campaign account & others
			info.runningModule.UpdateStepProcess(m_campaignsStep, runningInfo)
		end
	end
	-- ok
	return true
end

--[[

]]--
m_campaignsStep.Update = function(campaignId, allegroAccount, info)
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CAMPAIGNS.Update] Updating allegro campaign : %s.", campaignId)
	-- 1- Actualizo la cuenta de telefonia de la campana.
	local success, result = m_campaignsStep.UpdateAccount(campaignId, allegroAccount, info)
	if not success then
		local err = result
		return false, err
	end
	local response = {}
	local function UpdateResponse (result)
		for _, msg in pairs(result or {}) do table.insert(response, msg) end
	end
	-- 2- Actualizo los mensajes de la cuenta
	local success, result = m_campaignsStep.UpdateAccountMessages(campaignId, allegroAccount, info)
	UpdateResponse(result)
	-- 3- Actualizo Notifications Info (M&S info)
	local success, result = m_campaignsStep.UpdateNotifications(campaignId, allegroAccount, info)
	UpdateResponse(result)
	-- ok
	return true, response
end

--[[

]]--
m_campaignsStep.UpdateAccount = function(campaignId, allegroAccount, info)
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CAMPAIGNS.UpdateAccount] Updating allegroAccount: %s.", allegroAccount.id)
	local success, nativeAccount = m_campaignsStep.MakeCallNativeAccount(campaignId, allegroAccount, info)
	if not success then
		local err = nativeAccount
		return false, err
	end
	local _, agentCampaignsModel = utils.GetControllerOthersApps("agent", {controllerName = "campaigns", modelName = "Campaigns"})
	local campaignIdWithVCC = utils.AppendVcc(campaignId, info.vccId)
	local success, result = agentCampaignsModel.SaveAccount(info.ksAgent,
		campaignIdWithVCC,
		allegroAccount.id,
		nativeAccount,
		false,                    -- isNewAccount,
		{},                       -- allegroOtherInfo,
		info.ksProv,
		false,                    -- mustValidateAllegroAccount -- para que en la funcion NO realice validaciones contra allegro
		false                     -- mustValidateNativeAccount  -- para que en la funcion NO realice validaciones contra los paquetes de tel nativa
	)
	if not success then
		return false, {errType = "UNABLE TO UPDATE ACCOUNT", vccId = info.vccId, campaignId = campaignId, accountId = allegroAccount.id, rawError = result}
	end
	return true
end

--[[
	Genera la Key del Backup de Allegro de los mensajes de las cuentas que quedan como backup para hacer el rollback con la misma key mas el sufijo ~allegro_backup
		Ej: CALL_camptel2@emmatel2_camptel2_camptel2@emmatel2~allegro_backup
]]--
m_campaignsStep.MakeAllegroBackupCFId = function(id)
	assert(type(id) == "string")
	return string.format("%s~allegro_backup", id)
end

--[[
	Lista de mensajes y mapeo Allegro VS Nativo:

		Allegro:
		--------
			01- "EXIT_PROMPT"                       "Thanks for calling, goodbye."
			02- "IVR_PROMPT"                        "This is an interactive voice response audio."
			03- "MOH"                         (02)  "Music on hold."
			04- "OUTOFTIME_PROMPT"                  "We are currently out of business hours ..."
			05- "PLEASEWAIT_PROMPT"                 "Please wait ..."
			06- "RESCHEDULE_PROMPT"                 "We must reschedule your call"
			07- "VMONWAIT_PROMPT"                   "To leave a Voicemail please press 1"
			08- "VOICEMAIL_PROMPT"            (07)  "You have [n] seconds to leave a message."
			09- "VOICEMAILOUTOFTIME_PROMPT"   (03)  "We are currently out of business hours, you can leave a message"
			10- "WAIT_PROMPT"                 (09)   "While you wait [play this music/message]"
			11- "WELCOME_PROMPT"                    "Welcome to ..."
			12- "WERESORRY_PROMPT"                  "We are sorry, we cannot process your call right now."
			13- "YOUMUSTWAIT_PROMPT"                "You must wait [time] to be served."
			14- "YOUREINPOSITION_PROMPT"            "You are in the position [number]."
		Nativo:
		-------
			01- "dialerCallback"                    "Dialer Callback"
			02- "music_on_hold"               (03)  "Music on hold"
			03- "out_of_hours"                (09)  "Out of Hours Message"
			04- "transfer_failure"                  "Transfer failure"
			05- "virtualHold"                       "Virtual Hold"
			06- "voa"                               "VDN of Origin Announcements (VOA) Message."
			07- "voiceMail"                   (08)  "Voice Mail"
			08- "waiting_for_agent_with_ewt"        "Waiting message on ACD"
			09- "waiting_for_agent"           (10)  "Music on ACD"
			10- "welcome"                           "Welcome Message"
]]--
m_campaignsStep.UpdateAccountMessages = function(campaignId, allegroAccount, info)
	local _, admCampaignsModel = utils.GetControllerOthersApps("administration", {controllerName = "campaigns", modelName = "Campaigns"})
	local fullCampaignId = utils.AppendVcc(utils.GetId(campaignId), info.vccId)
	local campaignAccountMessagesId = admCampaignsModel.MakeAccountMessageId(fullCampaignId, allegroAccount.id)

	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CAMPAIGNS.UpdateAccountMessages] Updating Account Messages -> campaignAccountMessagesId: %s.", campaignAccountMessagesId)

	local allegroRepositoryModule = utils.GetControllerOthersApps("allegro", {controllerName = "repository", modelName = nil})
	local mappedMessages = {
		EXIT_PROMPT               = {nativeMessageId = nil                },
		IVR_PROMPT                = {nativeMessageId = nil                },
		MOH                       = {nativeMessageId = "music_on_hold"    },
		OUTOFTIME_PROMPT          = {nativeMessageId = nil                },
		PLEASEWAIT_PROMPT         = {nativeMessageId = nil                },
		RESCHEDULE_PROMPT         = {nativeMessageId = nil                },
		VMONWAIT_PROMPT           = {nativeMessageId = nil                },
		VOICEMAIL_PROMPT          = {nativeMessageId = "voiceMail"        },
		VOICEMAILOUTOFTIME_PROMPT = {nativeMessageId = "out_of_hours"     },
		WAIT_PROMPT               = {nativeMessageId = "waiting_for_agent"},
		WELCOME_PROMPT            = {nativeMessageId = "welcome"          },
		WERESORRY_PROMPT          = {nativeMessageId = nil                },
		YOUMUSTWAIT_PROMPT        = {nativeMessageId = nil                },
		YOUREINPOSITION_PROMPT    = {nativeMessageId = nil                }
	}

	-- 1. Obtengo los mensajes de la cuenta de Allegro
	local allegroAccountMessages = admCampaignsModel.GetAccountMessages(fullCampaignId, allegroAccount.id, info.ksAgent)
	LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CAMPAIGNS.UpdateAccountMessages] campaignAccountMessagesId: %s, allegroAccountMessages: %s.", campaignAccountMessagesId, json.encode(allegroAccountMessages or {}))

	local response = {}
	local nativeAccountMessages = {}

	-- 2 Recorro los mensajes de la cuenta de Allegro.
	for allegroMsgId, message in pairs(allegroAccountMessages) do
		allegroMsgId = allegroMsgId:upper()
		LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CAMPAIGNS.UpdateAccountMessages] allegroMsgId: '%s', message: %s", allegroMsgId, json.encode(message))
		-- 2.1 Si los mensajes de Allegro, mapean con los mensajes Nativos, procedo con la migracion.
		local mappedMessage = mappedMessages[allegroMsgId]
		if mappedMessage and mappedMessage.nativeMessageId then
			local messageType = (message.type or ""):lower()
			if messageType == "file" then
				-- 2.2. Obtengo los archivos de los mensajes de las cuenta de Allegro, realizando request contra el webhandler->repo de Allegro.
				local fileInformation = message.fileInformation or {}
				local part = "1"
				local fileName = string.format("allegro-migration-%s", fileInformation.fileName or fileInformation.id):lower()
				LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CAMPAIGNS.UpdateAccountMessages] Retrieve Allegro file. vcc: '%s', id: %s, section: %s, part: %s, fileName: '%s'.", info.vccId, fileInformation.id, fileInformation.section, part, fileName)
				local success, file, errorCode, headers = allegroRepositoryModule.Retrieve(info.vccId, fileInformation.id, fileInformation.section, part, false)
				if success then
					-- 2.3 - Almaceno el archivo obtenido desde Allegro en el Repositorio Nativo y los agrego en los SharedFiles.
					local repositoryController = require "apps.repository"
					repositoryController.ManualStart(inconcert.environment.apps.agent.cassandra.ip, inconcert.environment.apps.agent.cassandra.port)
					-- 2.4 - Convierte el audio de Allegro a un audio permitido para telefonia de OCC
					local success, convertedFile = repositoryController.ConvertAndRetrieveTelephonyAudioFile(file)
					if success then
						file = convertedFile
					end
					-- 2.5 - Almaceno el SharedFile
					repositoryController.ExternalStoreFile({
						Id       = fileInformation.id,
						FileName = fileName,
						BlobFile = file,
						VCC      = info.vccId,
						Section  = "-1"
					})
					-- 2.6 - Dejo los mensajes con el formato esperado en los mensajes de las cuentas de Allegro.
					nativeAccountMessages[mappedMessage.nativeMessageId] = {
						id = mappedMessage.nativeMessageId,
						type = "file",
						isComposite = true,
						options = {
							{
								Id          = fileName,
								Name        = fileName,
								MessageType = "audio",
								UUID        = utils.Uuid(true),
								type        = "file",
								fileInformation = {
									id          = fileInformation.id,
									fileSize    = #file,
									contentType = "audio/wav",
									fileName    = fileName,
									format      = "wav",
									section     = "-1"
								}
							}
						}
					}
				else
					local message = string.format("Unable to Migrate Allegro Account Message '%s'. Could not retrieve file from allegro. Id: '%s', Section: '%s', Part: '%s'",
						allegroMsgId,
						fileInformation.id,
						fileInformation.section,
						part
					)
					LogWarning(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CAMPAIGNS.UpdateAccountMessages] %s", message)
					-- dejo la traza del warning
					table.insert(response, {
						msgType = "warning",
						message = message
					})
				end
			else
				LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CAMPAIGNS.UpdateAccountMessages] Unable to Migrate Allegro Account Message '%s'. Only supported FILE messages. Current message type: %s", allegroMsgId, messageType)
			end
		else
			LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CAMPAIGNS.UpdateAccountMessages] Unable to Migrate Allegro Account Message '%s' is not supported in Native VCC.", allegroMsgId)
		end
	end

	-- 3. Elimino el backup anterior de los mensajes de la cuenta de Allegro (si es que antes ya se realizo una migracion de este vcc)
	local allegroBackupCampaignAccountMessagesId = m_campaignsStep.MakeAllegroBackupCFId(campaignAccountMessagesId)
	utils.RemoveColumns {
		keyspace = info.ksAgent,
		columnFamily = "CampaignAccountMessages",
		rows = {allegroBackupCampaignAccountMessagesId}
	}

	if not utils.TableIsEmpty(nativeAccountMessages) then
		-- 4. los mensajes de las cuentas de allegro quedan como backup para hacer el rollback con la misma key mas el sufijo ~allegro_backup (ej: CALL_camptel2@emmatel2_camptel2_camptel2@emmatel2~allegro_backup)
		LogDebug(LOG_PREFIX .. [[[PROVISIONING.MIGRATIONS.CAMPAIGNS.UpdateAccountMessages] Adding Messages in Native CampaignAccountMessages And SharedFilesRelations.
			- Backup of Allegro Accounts messages in CF: 'CampaignAccountMessages', allegroKey: '%s'.
			- Native Accounts messages in CF: 'CampaignAccountMessages', nativeKey: '%s'.
				nativeAccountMessages: %s
			]],
			allegroBackupCampaignAccountMessagesId,
			campaignAccountMessagesId,
			nativeAccountMessages
		)

		-- 5. Backup en Cassandra & en HDD temporal
		LogDebug(LOG_PREFIX .. "PROVISIONING.MIGRATIONS.CAMPAIGNS.UpdateAccountMessages] Make backup account messages. BackupId: %s", allegroBackupCampaignAccountMessagesId)
		-- 5.1 en cassandra
		local success = utils.SaveRows {
			keyspace = info.ksAgent or "Agent",
			columnFamily = "CampaignAccountMessages",
			rows = {
				[allegroBackupCampaignAccountMessagesId] = allegroAccountMessages
			}
		}
		-- 5.2 en HDD temporal
		local currentMigrationDir = info.runningModule.currentMigrationDir -- "/tmp/inconcert/migrations/migration_xxxxxxx"
		local directory = ("%s/%s"):format(currentMigrationDir, "backup/accounts"):lower()
		utils.CreateDirectory(directory)
		local fileName = ("camp_account_messages_%s.json"):format(allegroBackupCampaignAccountMessagesId):lower()
		local fullFileName = ("%s/%s"):format(directory, fileName):lower()
		local success, err = utils.CreateFile(fullFileName, json.encode(allegroAccountMessages))

		-- 6. remuevo los mensajes de las cuentas de allegro
		utils.RemoveColumns {
			keyspace = info.ksAgent or "Agent",
			columnFamily = "CampaignAccountMessages",
			rows = {campaignAccountMessagesId}
		}
		-- 7- Almaceno los mensajes de las cuentas en el formato Nativo.
		utils.SaveRows {
			keyspace = info.ksAgent or "Agent",
			columnFamily = "CampaignAccountMessages",
			rows = {
				[campaignAccountMessagesId] = nativeAccountMessages
			}
		}
		-- 8. Almaceno la relacion de los mensajes de las cuentas con los sharedFiles
		local _, sharedFilesModel = utils.GetControllerOthersApps("administration", {controllerName = "shared_files", modelName = "SharedFiles"})
		local info = {
			vcc               = info.vccId,
			campaignId        = fullCampaignId,
			accountId         = allegroAccount.id,
			messagesAccountId = campaignAccountMessagesId,
			messages          = nativeAccountMessages
		}
		sharedFilesModel.SaveSharedFilesAccountMessagesRelations(info, info.ksAgent)
	else
		LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CAMPAIGNS.UpdateAccountMessages] Native VCC without CALL Account Messages.")
	end

	-- ok
	return true, response
end

--[[

]]--
m_campaignsStep.SaveBackupAllegroAccount = function(campaignId, allegroAccount, info)
	local campaignIdWithVCC = utils.AppendVcc(campaignId, info.vccId)
	local allegroBackupCallCampaignId = m_campaignsStep.MakeAllegroBackupCFId(campaignIdWithVCC)

	-- 1. Respaldo en HDD en los directorios temporales (por las dudas)
	local currentMigrationDir = info.runningModule.currentMigrationDir -- "/tmp/inconcert/migrations/migration_xxxxxxx"
	local directory = ("%s/%s"):format(currentMigrationDir, "backup/accounts"):lower()
	utils.CreateDirectory(directory)
	local fileName = ("camp_account_%s.json"):format(allegroBackupCallCampaignId):lower()
	local fullFileName = ("%s/%s"):format(directory, fileName):lower()
	local success, err = utils.CreateFile(fullFileName, json.encode(allegroAccount))

	-- 2. Respaldo en HDD en los directorios temporales (por las dudas)
	utils.SaveRows {
		keyspace     = info.ksAgent or "Agent",
		columnFamily = "CampaignAccounts",
		superColumns = true,
		rows = {
			[allegroBackupCallCampaignId] = {
				[allegroAccount.id] = allegroAccount
			}
		}
	}

	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CAMPAIGNS.SaveBackupAllegroAccount] Saved Backup Allegro Account. campaignId: %s, Allegro account: %s. CF 'CampaignAccounts': key: '%s'. Backup in HDD: %s", campaignId, allegroAccount.id, allegroBackupCallCampaignId, fullFileName)
	return true
end

--[[

]]--
m_campaignsStep.GetBackupAllegroAccount = function(campaignId, accountId, info)
	local campaignIdWithVCC = utils.AppendVcc(campaignId, info.vccId)
	local allegroBackupCallCampaignId = m_campaignsStep.MakeAllegroBackupCFId(campaignIdWithVCC)
	local currentBackup = utils.GetAllColumns({
		keyspace     = info.ksAgent or "Agent",
		superColumns = true,
		columnFamily = "CampaignAccounts",
		key          = allegroBackupCallCampaignId
	})
	--LogDebug("--------------------------> allegroBackupCallCampaignId: '%s' currentBackup: %s", allegroBackupCallCampaignId, json.encode(currentBackup or {}))
	if utils.TableIsEmpty(currentBackup) then
		LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CAMPAIGNS.GetBackupAllegroAccount] --> 1-No exist current backup. campaignId: %sCF 'CampaignAccounts': key: '%s'", campaignId, allegroBackupCallCampaignId)
		return false, "BACKUP ALLEGRO ACCOUNT DOES NOT EXIST"
	end
	local accountAllegroBackup = currentBackup[accountId]
	if not accountAllegroBackup then
		LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CAMPAIGNS.GetBackupAllegroAccount] --> 2-No exist current backup. campaignId: %sCF 'CampaignAccounts': key: '%s', accountId: '%s'", campaignId, allegroBackupCallCampaignId, accountId)
		return false, "BACKUP ALLEGRO ACCOUNT DOES NOT EXIST"
	end
	return true, accountAllegroBackup
end

--[[

]]--
m_campaignsStep.MakeCallNativeAccount = function(campaignId, allegroAccount, info)
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CAMPAIGNS.MakeCallNativeAccount] 1 - Making Native Call Account. campaignId: %s, Allegro account: %s.", campaignId, allegroAccount.id)
--	LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CAMPAIGNS.MakeCallNativeAccount] 2 - Making Native Call Account. Allegro account: %s.", utils.JSON_Encode(allegroAccount))

	local errors = {}
	-- 1 - hago un respaldo de toda la configuracion de la cuenta de Allegro en el atributo "BackupAllegroAccount" y en HDD
	m_campaignsStep.SaveBackupAllegroAccount(campaignId, allegroAccount, info)

	-- 2 - creo la configuracion necesaria para las cuentas de Tel nativa.
	local nativeAccount = utils.DeepCopy(allegroAccount)
	-- generico
	nativeAccount.FlowInformation = {}

	-- entrante
	local allegroIncoming = allegroAccount.incomingConfig or {}
	local allegroIncomingContactCenter = allegroIncoming.ContactCenter
	--
	nativeAccount.incomingConfig = {enabled = false, ContactCenter = {}}

	local nativeIncoming = nativeAccount.incomingConfig
	if allegroIncoming.enabled then
		nativeIncoming.enabled = true
		nativeIncoming.ContactCenter = {
			-- si estan en allegro
			CheckRedial        = allegroIncomingContactCenter.CheckRedial or "0",
			VoicemailOnTimeout = allegroIncomingContactCenter.VoicemailOnTimeout or "0",
			VoicemailOnWait    = IsEnabled(allegroIncomingContactCenter.VoicemailOnWait), -- un boolean
			LifeTimeVoicemail  = allegroIncomingContactCenter.LifeTimeVoicemail or "",
			-- no estan en allegro - valores por defecto
			LeaveVoicemailAtOutOfHours = false
		}
		-- si estan en allegro
		nativeIncoming.AutoAnswerTime = allegroIncoming.AutoAnswerTime
		-- process flows
		if allegroIncoming.SelfServiceProcess then
			local success, flowInfo = info.processFlowsMigrationModule:FindProcessFlowByName(allegroIncoming.SelfServiceProcess, info)
			if success then
				nativeAccount.FlowInformation.on_schedule = {Process = flowInfo.Id}
			else
				table.insert(errors, {
					action      = "making native account - load process flow - on_schedule",
					isOk        = false,
					accountId   = allegroAccount.id,
					allegroVal  = allegroIncoming.SelfServiceProcess,
					err         = flowInfo
				})
			end
		end
		if allegroIncoming.OutOfTimeProcess then
			local success, flowInfo = info.processFlowsMigrationModule:FindProcessFlowByName(allegroIncoming.OutOfTimeProcess, info)
			if success then
				nativeAccount.FlowInformation.out_of_schedule = {Process = flowInfo.Id}
			else
				table.insert(errors, {
					action      = "making native account - load process flow - out_of_schedule",
					isOk        = false,
					accountId   = allegroAccount.id,
					allegroVal  = allegroIncoming.SelfServiceProcess,
					err         = flowInfo
				})
			end
		end
		-- no estan en allegro - valores por defecto
	end

	-- saliente
	local allegroOutgoing = allegroAccount.outgoingConfig or {}
	local allegroOutgoingContactCenter = allegroOutgoing.ContactCenter

	nativeAccount.outgoingConfig = {
		enabled = false,
		ContactCenter = {},
		Blacklists = {
			enabled = false,
			scopes = {}
		}
	}
	local nativeOutgoing = nativeAccount.outgoingConfig
	if allegroOutgoing.enabled then
		nativeOutgoing.enabled = true
		nativeOutgoing.ContactCenter = {
			-- si estan en allegro
			AllowManualCalls = allegroOutgoingContactCenter.AllowManualCalls or "0",
			-- no estan en allegro - valores por defecto
			AllowInvalidNumbers = "0"
		}
	end

	-- paquetes
	if nativeIncoming.enabled or nativeOutgoing.enabled then
		nativeAccount.packages = {}
		local isIncOutPackage = (nativeIncoming.enabled and nativeOutgoing.enabled)
		local isIncPackage    = (nativeIncoming.enabled and not nativeOutgoing.enabled)
		local isOutPackage    = (not nativeIncoming.enabled and nativeOutgoing.enabled)
		local accountType     = isIncOutPackage and "INCOMING & OUTGOING"
								or isIncPackage and "INCOMING"
								or "OUTGOING"
		-- obtengo el paquete desde donde corresponda (entrante y saliente, o entrante o saliente)
		local packages = isIncOutPackage and info.telephony.packages.io_mode or -- paquetes I/O
							isIncPackage and info.telephony.packages.i_mode  or -- paquetes I
							isOutPackage and info.telephony.packages.o_mode     -- paquetes O

		LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CAMPAIGNS.MakeCallNativeAccount] 3----> CampaignId: '%s', Allegro account: '%s' -----> need Package: %s", campaignId, allegroAccount.id, accountType)
		LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CAMPAIGNS.MakeCallNativeAccount] 4----> All posted packages: %s", json.encode(info.telephony.packages or {}))
		LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CAMPAIGNS.MakeCallNativeAccount] 5----> Selected packages: %s", json.encode(packages or {}))

		local _, package = utils.TableIsEmpty(packages)
		if package then
			nativeAccount.packages[package] = ""
			if not isOutPackage then
				-- si es un paquete que NO sea solo saliente lo debo eliminar ya que el mismo NO puede ser reutilizado en otras cuentas/campanas
				-- o sea, los paquetes entrantes o (entrante y saliente) solo pueden estar asociado a una unica cuenta porque tienen el DID asociado
				table.remove(packages, 1)
			end
		else
			local err = {errType = "UNABLE TO UPDATE ACCOUNT - NO AVAILABLE PACKAGES", vccId = info.vccId, campaignId = campaignId, accountId = allegroAccount.id, needPackage = accountType}
			LogWarning(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CAMPAIGNS.MakeCallNativeAccount] Unable to Update Allegro account. Err: %s", utils.JSON_Encode(err))
		end
	end

	-- CCAlgorithm:
	--	 obtengo el algoritmo de distribucion desde la config entrante o sino desde la saliente
	local allegroCCAlgorithm = allegroIncoming.enabled and allegroIncoming.CCAlgorithm or -- entrante
								allegroOutgoing.enabled and allegroOutgoing.CCAlgorithm -- saliente
	local allowedOCCAlgorithms = {
		CIRCULAR = "CIRCULAR",
		QUEUED   = "QUEUED",
		SKILL    = "SKILL"
	}
	nativeAccount.CCAlgorithm = allowedOCCAlgorithms[allegroCCAlgorithm] or "CIRCULAR"
	-- Priority
	nativeAccount.Priority = allegroIncoming.ACDPriority -- TODO: Consultar a Pablo.
	-- CallbackIsAllowed
	nativeAccount.CallbackIsAllowed = allegroAccount.CallbackIsAllowed
	if allegroAccount.CallbackIsAllowed then
		nativeAccount.LifeTimeAppointmentCalls = "5" -- valor por defecto el minimo (entre 5 a 600 segun la configuracion de la GUI)
	end
	-- Language
	nativeAccount.language = "es"

	---------------------------------------------------
	-- TODO: CLEANUP de valores innecesarios.
	---------------------------------------------------

	LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CAMPAIGNS.MakeCallNativeAccount] 3 - Maked Native Call Account: %s.", utils.JSON_Encode(nativeAccount))
	return true, nativeAccount, errors
end

--[[
		SI la cuenta de Allegro tiene Marketing & Sales tengo que:

		 actualizar las siguientes Columns en la CF VCCs:
		  	- AppsIntegrations
			- InteractionNotifications
		 actualizar la Columns en la CF Campaigns:
			- InteractionNotifications
]]--
m_campaignsStep.UpdateNotifications = function(campaignId, allegroAccount, info)
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CAMPAIGNS.UpdateNotifications] Updating vcc: '%s' Campaign '%s' Notifications (M&S) allegroAccount.id: %s, MarketingIntegrationEnabled: %s.", info.vccId, campaignId, allegroAccount.id, allegroAccount.MarketingIntegrationEnabled)
	if tonumber(allegroAccount.MarketingIntegrationEnabled or "0") ~= 1 then
		-- ok - la cuenta NO tiene Marketing & Sales configurado.
		LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CAMPAIGNS.UpdateNotifications] Unable to migrate Notificaciones in Vcc: '%s' Campaign '%s' -> Allegro Account NO configured M&S. allegroAccount.id: %s.", info.vccId, campaignId, allegroAccount.id)
		return true
	end
	--[[

	]]--
	LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CAMPAIGNS.UpdateNotifications] vcc: %s, AppsIntegrations: %s.", info.vccId, json.encode(info.occVccInfo.AppsIntegrations or {}))

	local vccMarketingAndSalesEnabledConfig = [[
		{
			"Enabled": "1",
			"IsChangedByAllegroMigration": "1",
			"Api": {
			  "Url": "https:\/\/sample.mas-testing.inconcertcc.com\/form\/render"
			},
			"Id": "MarketingAndSales",
			"DisplayName": "Marketing and Sales",
			"IsWidget": "0",
			"Name": "MarketingAndSales",
			"Notifications": {
			  "Url": "https:\/\/sample.mas-testing.inconcertcc.com\/public\/occ\/process"
			},
			"AccessToken": {
			  "Token": "the access token",
			  "Url": "https:\/\/sample.mas-testing.inconcertcc.com\/public\/occ\/get_session_token"
			},
			"AppRender": {
			  "Url": "https:\/\/sample.mas-testing.inconcertcc.com\/form\/render"
			}
		}
	]]

	local canUpdateVccAppsIntegrations = false
	if not info.occVccInfo.AppsIntegrations then
		LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CAMPAIGNS.UpdateNotifications] --> 1 debo ACTUALIZAR -> CF VCCs.AppsIntegrations")
		canUpdateVccAppsIntegrations = true
		info.occVccInfo.AppsIntegrations = json.decode([[
			{
				"HelpDesk": {
				  "IsWidget": "0",
				  "Name": "HelpDesk",
				  "Id": "HelpDesk",
				  "Enabled": "0",
				  "DisplayName": "HelpDesk"
				},
				"MarketingAndSales":]] .. vccMarketingAndSalesEnabledConfig ..
				[[,
				"WorkForceManagement": {
				  "IsWidget": "0",
				  "Name": "WorkForceManagement",
				  "Id": "WorkForceManagement",
				  "Enabled": "0",
				  "DisplayName": "Work Force Management"
				}
			  }
		]])
	else
		if info.occVccInfo.AppsIntegrations.MarketingAndSales then
			LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CAMPAIGNS.UpdateNotifications] --> 2")
			if tonumber(info.occVccInfo.AppsIntegrations.MarketingAndSales.Enabled) ~= 1 then
				LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CAMPAIGNS.UpdateNotifications] --> 2.1 debo ACTUALIZAR -> CF VCCs.AppsIntegrations")
				canUpdateVccAppsIntegrations = true
				info.occVccInfo.AppsIntegrations.MarketingAndSales = json.decode(vccMarketingAndSalesEnabledConfig)
			else
				LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CAMPAIGNS.UpdateNotifications] --> 2.2 --> NO ACTUALIZO -> CF VCCs.AppsIntegrations")
			end
		else
			LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CAMPAIGNS.UpdateNotifications] --> 3 debo ACTUALIZAR -> CF VCCs.AppsIntegrations")
			canUpdateVccAppsIntegrations = true
			info.occVccInfo.AppsIntegrations.MarketingAndSales = json.decode(vccMarketingAndSalesEnabledConfig)
		end
	end

	-- 1 - actualizo el VCC Column AppsIntegrations (se almacenara, cuando se actualice el VCC en "init.lua" ->m_migration.Start)
	if canUpdateVccAppsIntegrations then
		LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CAMPAIGNS.UpdateNotifications] --> 4 --> ACTUALIZO -> CF VCCs.AppsIntegrations")
	end

	-- 2 - actualizo el VCC Column InteractionNotifications (se almacenara, cuando se actualice el VCC en "init.lua" ->m_migration.Start)
	LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CAMPAIGNS.UpdateNotifications] --> 5 ACTUALIZO -> CF VCCs.InteractionNotifications")
	info.occVccInfo.InteractionNotifications = utils.JSON_Encode(info.occVccInfo.InteractionNotifications)
	if info.occVccInfo.InteractionNotifications and tostring(info.occVccInfo.InteractionNotifications.Enabled) ~= "1" then
		info.occVccInfo.InteractionNotifications = {Enabled = "1", IsChangedByAllegroMigration = "1"}
	end
	info.occVccInfo.InteractionNotifications = json.encode(info.occVccInfo.InteractionNotifications)

	-- 3 - actualizo la CF Campaigns Column InteractionNotifications
	LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CAMPAIGNS.UpdateNotifications] --> 6 ACTUALIZO -> CF Campaigns.InteractionNotifications")
	local fullCampaignId = utils.AppendVcc(utils.GetId(campaignId), info.vccId)
	local encodedInteractionNotifications = utils.GetColumnValue({
		keyspace     = info.ksAgent,
		columnFamily = "Campaigns",
		key          = fullCampaignId,
		column       = "InteractionNotifications"
	})
	local campaignInteractionNotifications = encodedInteractionNotifications and json.decode(encodedInteractionNotifications) or {}
	if not campaignInteractionNotifications.Apps or not campaignInteractionNotifications.Apps.MarketingAndSales then
		campaignInteractionNotifications.Apps = campaignInteractionNotifications.Apps or {}
		campaignInteractionNotifications.Apps.MarketingAndSales = true
		campaignInteractionNotifications.Apps.IsChangedByAllegroMigration = "1"
	end

	info.ksAgent:InsertColumn(fullCampaignId, "Campaigns", "InteractionNotifications", json.encode(campaignInteractionNotifications))

	-- ok
	local warnings
	if canUpdateVccAppsIntegrations then
		warnings = {}
		-- VCC M&S integration warnings
		table.insert(warnings, {
			msgType = "warning",
			message = "In App Provisioning you have to manually configure in the VCC the M&S Integration URLs in the VCC (Vcc->Basic->Integrations)"
		})
		-- VCC Others API SDK
		table.insert(warnings, {
			msgType = "warning",
			message = "In App Provisioning you have to manually configure in the VCC in the VCC the API SDK values (Vcc->Basic->Others)"
		})
		LogWarning(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CAMPAIGNS.UpdateNotifications] Manual configure: %s", json.encode(warnings))
	end
	return true, warnings
end

----------------------------
-- Rollback
----------------------------

--[[

]]--
m_campaignsStep.Rollback = function(info)
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CAMPAIGNS.Rollback] Rollback Migration Campaigns for vcc '%s'.", info.vccId)
	local vccCampaigns = utils.GetVCCCampaigns(info.vccId, info.ksAgent)
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CAMPAIGNS.Rollback] --> vcc: %s Campaigns: '%s'.", info.vccId, utils.JSON_Encode(vccCampaigns))

	local runningInfo = {
		stepProcessId = "rollback campaign",
		campaigns = {}
	}
	-- start running
	info.runningModule.UpdateStepProcess(m_campaignsStep, runningInfo)

	for campaignId in pairs(vccCampaigns) do
		local accounts = utils.GetCampaignAccounts(campaignId, info.ksAgent, info.vccId)
		--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CAMPAIGNS.Rollback]  1 - campaignId: %s@%s -> accounts: '%s'", campaignId, info.vccId, json.encode(accounts or {}))
		for _, account in pairs(accounts) do
			if account.proxyType == "CALL" then
				local existBackup = m_campaignsStep.GetBackupAllegroAccount(campaignId, account.id, info)
				if existBackup then
					local success, result = m_campaignsStep.RollbackUpdate(campaignId, account, info)
					table.insert(runningInfo.campaigns, {
						campaignId = campaignId,
						accountId = account.id,
						success = success,
						result = result
					})
					-- end running current campaign account
					info.runningModule.UpdateStepProcess(m_campaignsStep, runningInfo)
				end
			end
		end
	end
	-- ok
	return true
end

--[[

]]--
m_campaignsStep.RollbackUpdate = function(campaignId, callNativeAccount, info)
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CAMPAIGNS.RollbackUpdate] Rollback Updating allegro campaign : %s.", campaignId)
	local response = {}
	local function UpdateResponse (result)
		for _, msg in pairs(result or {}) do table.insert(response, msg) end
	end
	-- 1- Actualizo la cuenta de telefonia de la campana.
	local success, result = m_campaignsStep.RollbackAccount(campaignId, callNativeAccount, info)
	if not success then
		local err = result
		return false, err
	end
	UpdateResponse(result)
	-- 2- Actualizo los mensajes de la cuenta
	local success, result = m_campaignsStep.RollbackAccountMessages(campaignId, callNativeAccount, info)
	UpdateResponse(result)
	-- 2- Actualizo Notifications Info (M&S info)
	local success, result = m_campaignsStep.RollbackNotifications(campaignId, info)
	UpdateResponse(result)
	-- ok
	return true, response
end

--[[

]]--
m_campaignsStep.RollbackAccount = function(campaignId, callNativeAccount, info)
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CAMPAIGNS.RollbackAccount] Rollbacking ACCOUNT: %s of campaign: %s.", callNativeAccount.id, campaignId)
	-- 1- obtengo el model campaigns de la app "agent"
	local _, agentCampaignsModel = utils.GetControllerOthersApps("agent", {controllerName = "campaigns", modelName = "Campaigns"})
	-- 2. Remuevo la relacion packages -> campaign -> accounts
	local campaignIdWithVCC = utils.AppendVcc(campaignId, info.vccId)
	local success, result = agentCampaignsModel.RemoveCalli6AccountPackagesRelations(campaignIdWithVCC, callNativeAccount.id, nil, info.ksAgent, info.ksProv)
	-- 3. Restauro en memoria la cuenta de allegro.
	local success, allegroAccount = m_campaignsStep.RollbackMakeCallAllegroAccount(campaignId, callNativeAccount.id, info)
	if not success then
		local err = allegroAccount
		return false, err
	end
	--4 Actualizo la cuenta de la campana
	local success, result = agentCampaignsModel.SaveAccount(info.ksAgent,
		campaignIdWithVCC,
		allegroAccount.id,
		allegroAccount,
		false,                    -- isNewAccount,
		{},                       -- allegroOtherInfo,
		info.ksProv,
		false,                    -- mustValidateAllegroAccount -- para que en la funcion NO realice validaciones contra allegro
		false                     -- mustValidateNativeAccount  -- para que en la funcion NO realice validaciones contra los paquetes
	)
	if not success then
		return false, {errType = "UNABLE TO ROLLBACK ACCOUNT", vccId = info.vccId, campaignId = campaignId, accountId = callNativeAccount.id, rawError = result}
	end
	-- 5. Remuevo la columna 'BackupAllegroAccount' la cual contiene el backup de la cuenta de Call Allegro, la misma se utilizó para restaurar la cuenta en el proceso de rollback.
	-- TODO: Ver si elimino el Rollback luego de restaurar.
	-- ok
	return true
end

--[[

]]--
m_campaignsStep.RollbackMakeCallAllegroAccount = function(campaignId, accountId, info)
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CAMPAIGNS.RollbackMakeCallAllegroAccount] 1 - Making Allegro Call Account. campaignId: %s, Native account: %s.", campaignId, accountId)
	--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CAMPAIGNS.RollbackMakeCallAllegroAccount] 2 - Making Allegro Call Account. Native account: %s.", utils.JSON_Encode(callNativeAccount))
	local existBackup, allegroAccount = m_campaignsStep.GetBackupAllegroAccount(campaignId, accountId, info)
	if not existBackup then
		local err = allegroAccount
		LogWarning(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CAMPAIGNS.RollbackMakeCallAllegroAccount] --> Unable Rollback Allegro Account. Backup does not exist. campaignId: %s. Err: '%s'", campaignId, accountId, err)
		return false, err
	end
	allegroAccount.FlowInformation = {}
	LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CAMPAIGNS.RollbackMakeCallAllegroAccount] 3 - Maked Allegro Call Account: %s.", utils.JSON_Encode(allegroAccount))
	return true, allegroAccount
end

--[[

]]--
m_campaignsStep.RollbackAccountMessages = function(campaignId, callNativeAccount, info)
	local admCampaignsController, admCampaignsModel = utils.GetControllerOthersApps("administration", {controllerName = "campaigns", modelName = "Campaigns"})
	local fullCampaignId = utils.AppendVcc(utils.GetId(campaignId), info.vccId)
	local campaignAccountMessagesId = admCampaignsModel.MakeAccountMessageId(fullCampaignId, callNativeAccount.id)
	local allegroBackupCampaignAccountMessagesId = m_campaignsStep.MakeAllegroBackupCFId(campaignAccountMessagesId)
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CAMPAIGNS.RollbackAccountMessages] Rollbacking Account Messages of campaign: %s, account: %s, campaign account id: '%s', backup campaign account id: '%s'",
		campaignId,
		callNativeAccount.id,
		campaignAccountMessagesId,
		allegroBackupCampaignAccountMessagesId
	)
	-- 1. Obtengo el backup de allegro de los mensajes de las cuentas.
	local fullCampaignIdWithBackupSuffix = m_campaignsStep.MakeAllegroBackupCFId(fullCampaignId)
	local allegroBackupAccountMessages = admCampaignsModel.GetAccountMessages(fullCampaignIdWithBackupSuffix, callNativeAccount.id, info.ksAgent)
	LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CAMPAIGNS.RollbackAccountMessages] allegroBackupCampaignAccountMessagesId: %s, allegroBackupAccountMessages: %s.", allegroBackupCampaignAccountMessagesId, json.encode(allegroBackupAccountMessages or {}))
	if utils.TableIsEmpty(allegroBackupAccountMessages) then
		LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CAMPAIGNS.RollbackAccountMessages] Allegro account NO include Messages. allegroBackupCampaignAccountMessagesId: %s.", allegroBackupCampaignAccountMessagesId)
		return true
	end
	-- 2. Obtengo los mensajes de la cuenta de Nativo
	local nativeAccountMessages = admCampaignsModel.GetAccountMessages(fullCampaignId, callNativeAccount.id, info.ksAgent)
	LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CAMPAIGNS.RollbackAccountMessages] campaignAccountMessagesId: %s, nativeAccountMessages: %s.", campaignAccountMessagesId, json.encode(nativeAccountMessages or {}))
	if utils.TableIsEmpty(allegroBackupAccountMessages) then
		LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CAMPAIGNS.RollbackAccountMessages] Native account NO include Messages. campaignAccountMessagesId: %s.", campaignAccountMessagesId)
		return true
	end
	-- 3. Elimino los mensajes de las cuenta Nativo y las relaciones con los sharedFiles.
	local emptyMessages = {}
	admCampaignsModel.SaveAccountMessages(fullCampaignId, callNativeAccount.id, emptyMessages, info.ksAgent, nil, info.vccId)
	-- 4. Recorro los mensajes de las cuentas Nativo y los agrego en la lista de sharedFiles a eliminar.
	local sharedFilesToRemove = {}
	for nativeMsgId, message in pairs(nativeAccountMessages) do
		nativeMsgId = nativeMsgId:upper()
		LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CAMPAIGNS.RollbackAccountMessages] nativeMsgId: '%s', message: %s", nativeMsgId, json.encode(message))
		local messageType = (message.type or ""):lower()
		if messageType == "file" then
			-- 2.2. Obtengo los archivos de los mensajes de las cuentas de Allegro, realizando request contra el webhandler->repo de Allegro.
			for _, fileInfo in ipairs(message.options or {}) do
				local fileInformation = fileInfo.fileInformation or {}
				table.insert(sharedFilesToRemove, fileInformation.fileName:lower())
			end
		end
	end
	-- 5. Elimino los sharedFiles que estaban asociados a la cuenta Nativa, los elimino y tambien se eliminan del Repositorio Nativo.
	LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CAMPAIGNS.RollbackAccountMessages] sharedFilesToRemove: %s", json.encode(sharedFilesToRemove))
	if #sharedFilesToRemove > 0 then
		local sharedFilesController = utils.GetControllerOthersApps("administration", {controllerName = "shared_files", modelName = "SharedFiles"})
		sharedFilesController.ExternalDeleteSharedFiles(sharedFilesToRemove, info.vccId, info.ksAgent)
	end
	-- 6. Restauro el backup de los mensajes de la cuenta de Allegro.
	admCampaignsModel.SaveAccountMessages(fullCampaignId, callNativeAccount.id, allegroBackupAccountMessages, info.ksAgent, nil, info.vccId)
	-- 7. Elimino el backup de los mensajes de la cuenta de Allegro.
	utils.RemoveColumns {
		keyspace = info.ksAgent,
		columnFamily = "CampaignAccountMessages",
		rows = {allegroBackupCampaignAccountMessagesId}
	}
	--------------------
	-- ok
	return true
end

--[[

]]--
m_campaignsStep.RollbackNotifications = function(campaignId, info)
	info.occVccInfo.AppsIntegrations = utils.JSON_Decode(info.occVccInfo.AppsIntegrations) or {}
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CAMPAIGNS.RollbackNotifications] Rollbacking NOTIFICATIONS of campaign: %s, vcc.AppsIntegrations.MarketingAndSales: %s", campaignId, utils.JSON_Encode(info.occVccInfo.AppsIntegrations))
	if not info.occVccInfo.AppsIntegrations.MarketingAndSales
		or tostring(info.occVccInfo.AppsIntegrations.MarketingAndSales.IsChangedByAllegroMigration) ~= "1" then
		-- ok - No tuvo migracion de M&S
		LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.CAMPAIGNS.RollbackNotifications] Unable to rollback Notificaciones in Vcc: '%s' Campaign '%s' -> Allegro Account NO configured M&S.", info.vccId, campaignId)
		return true
	end
	-- 1. actualizo los valores de las integraciones del VCC
	info.occVccInfo.AppsIntegrations.MarketingAndSales = json.decode([[
		{
			"IsWidget": "0",
			"Name": "MarketingAndSales",
			"Id": "MarketingAndSales",
			"Enabled": "0",
			"DisplayName": "Marketing and Sales"
		  },
	]])
	-- 2. actualizo los valores de las notificaciones del VCC
	if tostring(info.occVccInfo.InteractionNotifications.IsChangedByAllegroMigration) == "1" then
		info.occVccInfo.InteractionNotifications.Enabled = "0"
		info.occVccInfo.InteractionNotifications.IsChangedByAllegroMigration = nil
	end
	-- 3. actualizo los valores de las notificaciones de la Campana
	local fullCampaignId = utils.AppendVcc(utils.GetId(campaignId), info.vccId)
	local encodedInteractionNotifications = utils.GetColumnValue({
		keyspace     = info.ksAgent,
		columnFamily = "Campaigns",
		key          = fullCampaignId,
		column       = "InteractionNotifications"
	})
	local campaignInteractionNotifications = utils.JSON_Decode(encodedInteractionNotifications) or {}
	if campaignInteractionNotifications.Apps
			and campaignInteractionNotifications.Apps.MarketingAndSales
			and tostring(campaignInteractionNotifications.Apps.IsChangedByAllegroMigration) == "1" then
		campaignInteractionNotifications.Apps.MarketingAndSales = false
		campaignInteractionNotifications.Apps.IsChangedByAllegroMigration = nil
		info.ksAgent:InsertColumn(fullCampaignId, "Campaigns", "InteractionNotifications", json.encode(campaignInteractionNotifications))
	end
	-- ok
	return true
end

--
table.insert(m_migrationsSteps, m_campaignsStep)  -- agrego step al array

--------------------------------------------------------------------------------------

--[[

]]--
m_migration.GetSteps = function()
	return m_migrationsSteps
end

--[[

]]--
m_migration.Init = function(config)
	for _, step in ipairs(m_migration.GetSteps()) do
		step.migrationId = config.id
	end
end

------------------------------------------
--
-- Exported functions
--
return {
	Init                    = m_migration.Init,
	GetSteps                = m_migration.GetSteps,
	GetBackupAllegroAccount = m_campaignsStep.GetBackupAllegroAccount
}
