local utils = require "shared.apps.utils"
module(..., package.seeall, inconcert.orbit.application.Decorate)

local LOG_PREFIX = "M I G R A T I O N ==> "
--------------------------------------------------------------------------------------
local m_migration = {}
local m_migrationsSteps = {}

--------------------------------------------------------------------------------------
-- STEP OUTBOUND ENGINE - IMPORT FORMATS ------
--------------------------------------------------------------------------------------

local m_oeImportFormatsStep = {
	id = "processes",
	migrationId = nil, -- es el id de la definicion del archivo "/migrations/init.lua" tabla m_migrationsConfig[0,...].id
	description = "OE Import Formats Step Module Migration"
}

--[[

]]--
m_oeImportFormatsStep.LoadAllegroImportFormats = function(info)
	local moduleAllegroImport = utils.GetControllerOthersApps("allegro", {controllerName = "outboundengine_imports", modelName = nil})
	local requestInfo = {
		vcc = info.vccId,
		last = "",
limit = 1000, -- TODO: paginar
		numberPage = 1,
		query = {Id = "", Description = "", Type = ""},
		columns = {"Id", "Description"}
	}
	local success, data = moduleAllegroImport.LoadPaginatedImportFormats(json.encode(requestInfo))
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_IMPORT_FORMATS.LoadAllegroImportFormats] --> vcc: %s Allegro OE Import Formats request. success: '%s'.", info.vccId, success)
	--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_IMPORT_FORMATS.LoadAllegroImportFormats] --> vcc: %s Allegro OE Import Formats request. data: '%s'.", info.vccId, utils.JSON_Encode(data))
	if not success then
		local err = data
		return false, err
	end
	-- ok
	return true, data
end

--[[

]]--
m_oeImportFormatsStep.Start = function(info)
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_IMPORT_FORMATS.Start] Start Migration OE Import Formats for vcc '%s'.", info.vccId)
	-- 1 - creo el step de running
	local runningInfo = {
		stepProcessId = "migrate oe import formats"
	}
	info.runningModule.UpdateStepProcess(m_oeImportFormatsStep, runningInfo)
	-- 2 - obtengo los import formats desde de allegro
	local success, importFormats = m_oeImportFormatsStep.LoadAllegroImportFormats(info)
	if not success then
		-- 2.1 error, debo actualizar el step de running
		runningInfo.success = false
		runningInfo.errType = "UNABLE TO LOAD OE IMPORT FORMATS"
		runningInfo.err = importFormats
		info.runningModule.UpdateStepProcess(m_oeImportFormatsStep, runningInfo)
		return false
	end
	-- 3 - actualizo el step de running antes de comenzar la migracion de los formatos de importacion
	local allegroImportFormats = importFormats.rows or {}
	runningInfo.total = #allegroImportFormats
	info.runningModule.UpdateStepProcess(m_oeImportFormatsStep, runningInfo)
	-- 4 - comienzo la migracion de los formatos de importacion
	local success, result = m_oeImportFormatsStep.UpdateImportFormats(allegroImportFormats, info)
	if not success then
		-- 4.1 error, debo actualizar el step de running
		runningInfo.success = false
		runningInfo.errType = "UNABLE TO CREATE IMPORT FORMATS"
		runningInfo.err = result
		info.runningModule.UpdateStepProcess(m_oeImportFormatsStep, runningInfo)
		return false
	end
	-- ok
	-- 5 - actualizo el step de running
	runningInfo.success = true
	runningInfo.result = result
	info.runningModule.UpdateStepProcess(m_oeImportFormatsStep, runningInfo)
	return true
end

----------------------------
-- Rollback
----------------------------

--[[

]]--
m_oeImportFormatsStep.Rollback = function(info)
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_IMPORT_CONTACTS.Rollback] Rollback Migration OE Import Formats for vcc '%s'.", info.vccId)
	-- - realizo rollback de todos los import formats
	-- 1 - creo el step de running
	local runningInfo = {
		stepProcessId = "rollback oe import formats"
	}
	info.runningModule.UpdateStepProcess(m_oeImportFormatsStep, runningInfo)
	local success, result = info.fullcontactMigrationModule.importFormats:Rollback(info.vccId, info.ksOE)
	if not success then
		-- 1.1 error, debo actualizar el step de running
		runningInfo.success = false
		runningInfo.errType = "UNABLE TO ROLLBACK IMPORT FORMATS"
		runningInfo.err = result
		info.runningModule.UpdateStepProcess(m_oeImportFormatsStep, runningInfo)
		return false
	end
	-- ok
	-- 2 - actualizo el step de running
	runningInfo.success = true
	runningInfo.result = result
	info.runningModule.UpdateStepProcess(m_oeImportFormatsStep, runningInfo)
	return true
end

--[[
]]--
m_oeImportFormatsStep.UpdateImportFormats = function(allegroImportFormats, info)
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_IMPORT_FORMATS.UpdateImportFormats] Adding Native OE Import Formats from Allegro Import Formats: '%s', vcc: '%s'.", #allegroImportFormats, info.vccId)
	local success, result = info.fullcontactMigrationModule.importFormats:Migrate(info.vccId, allegroImportFormats, info.ksOE)
	if not success then
		local err = result
		LogWarning(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_IMPORT_FORMATS.UpdateImportFormats] Error creating Native Import Formats. Err: %s", json.encode(err))
		return false, err
	end
	local result = {}
	return true, result
end

--
table.insert(m_migrationsSteps, m_oeImportFormatsStep)  -- agrego step al array

--------------------------------------------------------------------------------------

--[[

]]--
m_migration.GetSteps = function()
	return m_migrationsSteps
end

--[[

]]--
m_migration.Init = function(config)
	for _, step in ipairs(m_migration.GetSteps()) do
		step.migrationId = config.id
	end
end

------------------------------------------
--
-- Exported functions
--
return {
	Init     = m_migration.Init,
	GetSteps = m_migration.GetSteps
}
