require "LuaOrm.Date"
local utils = require "shared.apps.utils"
local m_helpers = require "helpers"
module(..., package.seeall, inconcert.orbit.application.Decorate)

local LOG_PREFIX = "M I G R A T I O N ==> "
local m_cachedAllegroSchedulers = nil

--------------------------------------------------------------------------------------
--[[
	TODO: Mover a helpers.
]]--
local function IsEnabled (value)
	value = value or "0"
	return tostring(value) == "1"
end

--[[
	TODO: Mover a helpers.
]]--
local function StringDate (stringDate)
	if not stringDate then
		return ""
	end
	return LuaOrm.Date(stringDate):toString("%Y-%m-%d")
end

--------------------------------------------------------------------------------------
local m_migration = {}
local m_migrationsSteps = {}

--------------------------------------------------------------------------------------
-- STEP OUTBOUND ENGINE - PROCESSES ------
--------------------------------------------------------------------------------------

local m_oeProcessesStep = {
	id = "processes",
	migrationId = nil, -- es el id de la definicion del archivo "/migrations/init.lua" tabla m_migrationsConfig[0,...].id
	description = "OE Processes Step Module Migration (Process + rules + schedulers + destinations + actions + batches + filters)"
}

--[[

]]--
m_oeProcessesStep.Start = function(info)
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.Start] Start Migration OE Processes for vcc '%s'.", info.vccId)

	m_cachedAllegroSchedulers = {}

	------------------------------------------------------------------------------------------
	-- A - los procesos + (reglas: schedulers + destinations + actions) + (batches)

	-- 1 - creo el step de running
	local runningInfo = {
		stepProcessId = "migrate oe process",
		processes = {}
	}
	info.runningModule.UpdateStepProcess(m_oeProcessesStep, runningInfo)
	-- 2 - obtengo los OE process desde allegro
	local success, data = m_oeProcessesStep.LoadAllegroProcesses(info)
	if not success then
		-- 2.1 error, debo actualizar el step de running
		runningInfo.success = false
		runningInfo.errType = "UNABLE TO LOAD OE PROCESSES"
		runningInfo.err = data
		info.runningModule.UpdateStepProcess(m_oeProcessesStep, runningInfo)
		return false
	end

	local oeProcessesList = data.rows or {}
	for _, allegroProcess in ipairs(oeProcessesList) do
		local processId = allegroProcess.Id
		-- obtengo desde Allegro todo el detalle del proceso (config, rules, scheduler, destination, actions, batches)
		local success, process = m_oeProcessesStep.GetAllegroProcess(processId, info.vccId)
		if success then
			-- comienzo la migracion de todo lo anterior
			local success, result = m_oeProcessesStep.UpdateOEProcess(process, info)
			table.insert(runningInfo.processes, {
				id = "migrate oe process",
				processId = processId,
				success = success,
				result = result
			})
		else
			table.insert(runningInfo.processes, {
				processId = processId,
				success = false,
				result = process,
				errType = "UNABLE TO LOAD ONE PROCESS"
			})
		end
	end
	-- Creo en archivo JSON con los procesos y batches que se han migrado y que deberan ser utilizados en el STEP 2 de migracion (los contactos y asignacion a sus batches)
	if info.createdAllegroOeProcessAndBatches and #(info.createdAllegroOeProcessAndBatches.processes or {}) > 0 then
		local fullFileName = ("%s/processes_and_batches_for_step_2.json"):format(info.runningModule.baseDir):lower()
		utils.CreateFile(fullFileName, json.encode(info.createdAllegroOeProcessAndBatches, true))
		LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.Start] The JSON file containing the list of processes and batches for STEP 2 of the migration has been created. FileName '%s'.", fullFileName)
	else
		LogWarning(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.Start] No processes have been migrated.")
	end
	-- actualizo el step de running
	info.runningModule.UpdateStepProcess(m_oeProcessesStep, runningInfo)

	------------------------------------------------------------------------------------------
	-- B - los filtros guardados de los batches
	-- 1 - creo el step de running
	local runningInfo = {
		stepProcessId = "migrate batch filters",
		filters = {}
	}
	info.runningModule.UpdateStepProcess(m_oeProcessesStep, runningInfo)
	-- 2 - obtengo los OE filtros de los batches desde allegro
	local success, data = m_oeProcessesStep.LoadAllegroBatchFilters(info)
	if not success then
		-- 2.1 error, debo actualizar el step de running
		runningInfo.success = false
		runningInfo.errType = "UNABLE TO LOAD OE BATCH FILTERS"
		runningInfo.err = data
		info.runningModule.UpdateStepProcess(m_oeProcessesStep, runningInfo)
		return false
	end

	local oeBatchFilters = data or {}
	for _, filter in ipairs(oeBatchFilters) do
		local filterId = filter.Id
		-- obtengo desde Allegro el deatlle del filtro
		local success, result = m_oeProcessesStep.UpdateBatchFilter(filter, info)
		if success then
			table.insert(runningInfo.filters, {
				id = "migrate oe batch filters",
				filterId = filterId,
				success = success,
				result = result
			})
		else
			table.insert(runningInfo.filters, {
				filterId = filterId,
				success = false,
				result = err,
				errType = "UNABLE TO UPDATE BATCH FILTER"
			})
		end
	end
	-- ok
	return true
end

--[[

]]--
m_oeProcessesStep.LoadAllegroProcesses = function(info)
	local moduleAllegroProcess = utils.GetControllerOthersApps("allegro", {controllerName = "outboundengine_processes", modelName = nil})
	local requestInfo = {
		vcc = info.vccId,
		last = "",
		limit = 1000, -- TODO: paginar
		numberPage = 1,
		query = {Id = "", EngineService = "", Type = ""}
	}
	local success, data = moduleAllegroProcess.LoadPaginatedProcesses(json.encode(requestInfo))
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.LoadAllegroProcesses] --> vcc: %s Allegro OE Processes request. success: '%s'.", info.vccId, success)
	--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.LoadAllegroProcesses] --> vcc: %s Allegro OE Processes request. data: '%s'.", info.vccId, utils.JSON_Encode(data))
	if not success then
		local err = data
		return false, err
	end
	-- ok
	return true, data
end

--[[

]]--
m_oeProcessesStep.GetAllegroProcess = function(processId, vccId)
	assert(type(processId) == "string")
	local moduleAllegroProcess = utils.GetControllerOthersApps("allegro", {controllerName = "outboundengine_processes", modelName = nil})
	local success, data = moduleAllegroProcess.GetProcess(processId, vccId)
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.GetAllegroProcess] 1--> processId: %s vcc: '%s' Allegro OE Process request. success: '%s'.", processId, vccId, success)
	--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.GetAllegroProcess] 2--> processId: %s vcc: '%s' Allegro OE Process request. data: '%s'.", processId, vccId, utils.JSON_Encode(data))
	if not success then
		local err = data
		return false, err
	end
	-- obtengo los schedulers configurados.
	local allegroProcess = data
	allegroProcess.__AllegroSchedulers = {}
	for _, rule in ipairs(allegroProcess.OutboundProcessRules or {}) do
		if rule.ScheduleId then
			local fullSchId = utils.AppendVcc(utils.GetId(rule.ScheduleId), vccId)
			if m_cachedAllegroSchedulers[fullSchId] then
				allegroProcess.__AllegroSchedulers[rule.ScheduleId:lower()] = utils.DeepCopy(m_cachedAllegroSchedulers[fullSchId])
			else
				local success, scheduler = moduleAllegroProcess.GetScheduler(rule.ScheduleId, vccId)
				if success and scheduler then
					m_cachedAllegroSchedulers[fullSchId] = {Scheduler = scheduler, ScheduleId = rule.ScheduleId}
					allegroProcess.__AllegroSchedulers[rule.ScheduleId:lower()] = utils.DeepCopy(m_cachedAllegroSchedulers[fullSchId])
				else
					LogWarning(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.GetAllegroProcess] --> processId: %s vcc: '%s' Allegro OE Process. Unable to retrieve allegro scheuduler: ", processId, vccId, rule.ScheduleId)
				end
			end
		else
			LogWarning(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.GetAllegroProcess] --> processId: %s vcc: '%s' Allegro OE Process without scheduler configured.", processId, vccId)
		end
	end
	--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.GetAllegroProcess] 3--> processId: %s vcc: '%s' Allegro OE Process request. data WITH SCHEDULERS in atributo '__AllegroSchedulers' : '%s'.", processId, vccId, utils.JSON_Encode(allegroProcess))
	-- ok
	return true, allegroProcess
end

--[[

]]--
m_oeProcessesStep.UpdateOEProcess = function(allegroOeProcess, info)
	--LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.UpdateOEProcess] Update Native OE process: '%s', vcc: '%s'.", allegroOeProcess)
	-- 1- Creo el Proceso Nativo (proceso "basico" + reglas + schedulers + destinations + acciones)
	local success, result = m_oeProcessesStep.CreateNativeOEProcess(allegroOeProcess, info)
	if not success then
		local err = result
		return false, err
	end
	-- 2- Creo los Batches del proceso
	local importModule = info.GetStepByModule("oeImportContacts", "importations")
	local success, result = m_oeProcessesStep.CreateNativeOEBatches(allegroOeProcess, info)
	if not success then
		local err = result
		return false, err
	end
	-- 3- Creo los procesos y batches que se han migrado en una tabla.
	--    Estos seran utilizados para crear el archivo "processes_and_batches_for_step_2.json"
	--    que luego sera utilizado en el STEP 2 para migrar contactos.
	info.createdAllegroOeProcessAndBatches = info.createdAllegroOeProcessAndBatches or {
		processes = {}
	}
	local process = {
		processId = allegroOeProcess.Id,
		batches   = {}
	}
	table.insert(info.createdAllegroOeProcessAndBatches.processes, process)
	for _, allegroBatch in ipairs(allegroOeProcess.OutboundProcessBatchs or {}) do
		table.insert(process.batches, {
			batchId = allegroBatch.BatchId
		})
	end
	-- ok
	return true, result
end

--[[

]]--
m_oeProcessesStep.CreateNativeOEProcess = function(allegroOeProcess, info)
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.CreateNativeOEProcess] Creating Native OE process: '%s', vcc: '%s'.", allegroOeProcess, info.vccId)
	local success, nativeOeProcess = m_oeProcessesStep.MakeNativeOEProcess(allegroOeProcess, info)
	if not success then
		local err = nativeOeProcess
		return false, err
	end
	-- \apps\outbound_engine\controllers\processes.lua
	local success, result = utils.GetOEApi().native.processes.SaveProcess(nativeOeProcess, nil, info.ksAgent, info.ksProv, info.ksOE)
	if not success then
		local err = result
		LogWarning(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.CreateNativeOEProcess] Error creating OE Process. Err: %s", json.encode(err))
		return false, result
	end
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.CreateNativeOEProcess] Success created OE Process ... please wait")
	require("socket").sleep(3) -- Pequena pausa necesaria para que el OE cree en cassandra el proceso.
	-- ok
	return true
end

--[[
	-- PROGRESIVO
        1)  NATIVO.hasPreview              = ALLEGRO_ACCOUNT.HasPreview
        2) NATIVO.hangupOnAnsweringMachine = ALLEGRO_ACCOUNT.HangupOnAM

    -- PREDICTIVO
        3) NATIVO.desiredAbandonRate       = ALLEGRO_ACCOUNT.DesiredAbandonRate -- En allegro no es requerido
        4) NATIVO.strictServiceLevel       = ALLEGRO_ACCOUNT.StrictThreshold
        5) NATIVO.abandonedProcess         = ALLEGRO_ACCOUNT.AbandonedPredictiveProcess -> PROCESS_FLOW.Id
        6) NATIVO.serviceLevel             = CampaignThresholds['CALL:campaign@vcc']ServiceLevel_OUTBOUND

    -- AUTOMATIC
        7) NATIVO.process                  = ALLEGRO_ACCOUNT.SelfServiceProcess -> PROCESS_FLOW.Id

    -- PREDICTIVO & AUTOMATIC
        8) NATIVO.answeringMachineProcess  = ALLEGRO_ACCOUNT.AbandonedPredictiveProcess -> PROCESS_FLOW.Id
        9) NATIVO.correctionManualRate     = ALLEGRO_ACCOUNT.CorrectionManualRate

    -- Nuevos en Nativo que no existen en allegro
       10) historicalContactRateFactor

	NOTA:
	-----
		Los puntos 5, 7 y 8 se almacena el Id del process flow, y el mismo es un GUID.
		En Allegro el Id es un nombre, entonces para poder asignar el proceso el mismo debe de existir
		dado de alta (migrado con los procesos de migraciones de flujos, o sino creado manualmente).
		La busqueda se realiza en los procesos por el atributo Name (case sensitive),
		una vez encontrado se obtiene el Id para poder mapearlo al proceso.

		El punto 6) el valor como tal no se almacena en la cuenta de la campana sino que se almacena en la CF de los Thresholds,
		y por ende hay que buscar el valor en esta CF CampaignThresholds.
]]--
m_oeProcessesStep.MakeNativeOEProcess = function(allegroOeProcess, info)
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.MakeNativeOEProcess] 1 - Making Native OE Process. Allegro OE ProcessId: %s, vcc: %s.", allegroOeProcess.Id, info.vccId)

	info.runningModule.UpdateStepProcess(m_oeProcessesStep, {
		stepProcessId = "migrate oe process info",
		processId = allegroOeProcess.Id
	})

	--LogDebug(LOG_PREFIX .. "[[PROVISIONING.MIGRATIONS.OE_PROCESSES.MakeNativeOEProcess] 2 - Making Native OE Process. Allegro OE Process - data: %s.", utils.JSON_Encode(allegroOeProcess))
	local errors = {}
	-- valido que la campana NO sea una campana de Allegro (sirve para las migraciones de VCCs Allegro a Nativos)
	local campaignId = allegroOeProcess.DisplayCampaign or "" -- sale del atributo calculado de LuaORM (modelo de outboundengine tabla OutboundProcess)
	if #campaignId == 0 then
		-- En los procesos de OE Nativo la campana es requerida, asi que si el proceso de Allegro NO esta asignado a una campana, no creo el OE Process en Nativo.
		local err = {type = "UNABLE TO MAKE NATIVE OE PROCESS - ALLEGRO PROCESS IS NOT ASSOCIATED TO SOME CAMPAIGN", processId = allegroOeProcess.Id}
		LogWarning(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.MakeNativeOEProcess] error: %s", json.encode(err))
		return false, err
	end
	local allAccounts = utils.GetCampaignAccounts(campaignId, info.ksAgent, info.vccId)
	--LogDebug(LOG_PREFIX .. "1--> allAccounts: %s", json.encode(allAccounts))
	local noAccounts, migratedNativeAccount = utils.TableIsEmpty(allAccounts)
	--LogDebug(LOG_PREFIX .. "2--> noAccounts: %s", noAccounts)
	--LogDebug(LOG_PREFIX .. "3 -> migratedNativeAccount: %s", json.encode(migratedNativeAccount))
	if noAccounts then
		local err = {type = "UNABLE TO MAKE NATIVE OE PROCESS - CAMPAIGN NOT HAS ACCOUNTS", processId = allegroOeProcess.Id, campaignId = campaignId}
		LogWarning(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.MakeNativeOEProcess] error: %s", json.encode(err))
		return false, err
	end
	if migratedNativeAccount.proxyType ~= "CALL" then
		local err = {type = "UNABLE TO MAKE NATIVE OE PROCESS - IS NOT A CALL ACCOUNT", processId = allegroOeProcess.Id, campaignId = campaignId, accountId = migratedNativeAccount.id}
		LogWarning(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.MakeNativeOEProcess] error: %s", json.encode(err))
		return false, err
	end
	-- ok
	local campaignModule = info.GetStepByModule("campaigns", "campaigns")
	local existBackup, backupAllegroAccount = campaignModule.GetBackupAllegroAccount(campaignId, migratedNativeAccount.id, info)
	assert(existBackup, "Error 'backupAllegroAccount' does not exist")

	local outgoingConfigAccount = backupAllegroAccount.outgoingConfig
	assert(outgoingConfigAccount, "Error 'backupAllegroAccount.outgoingConfig' does not exist")

	local oeConfigAccount = outgoingConfigAccount.OutboundEngine
	assert(oeConfigAccount, "Error 'backupAllegroAccount.outgoingConfig.OutboundEngine' does not exist")

	--
	local nativeOeProcess = m_oeProcessesStep.GetTemplateNativeOEProcess()
	-- Source Allegro Info
	nativeOeProcess.Allegro.OeProcess = allegroOeProcess
	nativeOeProcess.Allegro.OeConfigCampaignAccount = oeConfigAccount

	-- root
	nativeOeProcess.__vcc = info.vccId -- atributo que se va a eliminar en la app OE (no se va a almacenar)
	nativeOeProcess.id = allegroOeProcess.Id
	-- root.configuration
	local fullCampaignId = utils.AppendVcc(utils.GetId(campaignId), info.vccId)
	local callAccountId = migratedNativeAccount.id

	local configuration = nativeOeProcess.configuration
	configuration.campaign = fullCampaignId
	-- retry values (MaxRetriesPerDay o MaxRetriesPerContact)
	local function MakeRetryValue (pValue, attribute)
		local defaultValues = {min = 1, max = 100}
		local value = tonumber(pValue) or defaultValues.max
		if value > defaultValues.max then
			local msg = string.format(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.MakeNativeOEProcess] Allegro '%s' retry attribute with value '%s' is out of range. The value '%s' will be assigned.", attribute, pValue, defaultValues.max)
			LogWarning("%s", msg)
			return defaultValues.max
		end
		if value < defaultValues.min then
			local msg = string.format(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.MakeNativeOEProcess] Allegro '%s' retry attribute with value '%s' is out of range. The value '%s' will be assigned.", attribute, pValue, defaultValues.min)
			LogWarning("%s", msg)
			return defaultValues.min
		end
		return value
	end
	configuration.maxContactRetriesDay = MakeRetryValue(allegroOeProcess.MaxRetriesPerDay, "MaxRetriesPerDay")
	configuration.maxContactRetries = MakeRetryValue(allegroOeProcess.MaxRetriesPerContact, "MaxRetriesPerContact")
	-- root.configuration.channels
	local callChannel = configuration.channels.CALL
	callChannel.account = callAccountId
	callChannel.engineType = oeConfigAccount.OutboundProcessType

	if oeConfigAccount.OutboundProcessType == "PROGRESSIVE" then
	-- PROGRESIVO
		callChannel.hasPreview = IsEnabled(oeConfigAccount.HasPreview)
		callChannel.hangupOnAnsweringMachine = IsEnabled(oeConfigAccount.HangupOnAM)

	elseif oeConfigAccount.OutboundProcessType == "PREDICTIVE" or oeConfigAccount.OutboundProcessType == "AUTOMATIC" then
		if oeConfigAccount.OutboundProcessType == "PREDICTIVE" then
		-- PREDICTIVO (solo)
			-- proceso de abandodanos
			if #(oeConfigAccount.AbandonedPredictiveProcess or "") > 0 then
				local success, flowInfo = info.processFlowsMigrationModule:FindProcessFlowByName(oeConfigAccount.AbandonedPredictiveProcess, info)
				if success then
					callChannel.abandonedProcess = flowInfo.Id
				else
					callChannel.abandonedProcess = "not found in migration"
					table.insert(errors, {
						action      = "creating op process - load process flow",
						isOk        = false,
						processId   = allegroOeProcess.Id,
						processType = "predictive abandoned",
						allegroVal  = oeConfigAccount.AbandonedPredictiveProcess,
						err         = flowInfo
					})
				end
			end
			-- tasa de abandono deseado
			local desiredAbandonRate = nil
			if tostring(oeConfigAccount.DesiredAbandonRate or "-1") ~= "-1" then
				callChannel.desiredAbandonRate = oeConfigAccount.DesiredAbandonRate
			end
			-- (Nativo): Nivel de servicio estricto - (Allegro): Permitir colgar llamada en el umbral
			callChannel.strictServiceLevel = IsEnabled(oeConfigAccount.StrictThreshold)
			-- (Nativo): Nivel de servicio - (Allegro/umbrables): Tiempo de nivel de servicios (segundos)
			local success, channelCampaignThresholds = m_oeProcessesStep:GetChannelCampaignThresholds(fullCampaignId, "CALL", info)
			if success then
				-- TODO: Martin/Pablo - revisar los valores permitidos ya que en Nativo permite un rango entre entre 1 y 10 y en Allegro 1 y 1200
				callChannel.serviceLevel = tonumber(channelCampaignThresholds.ServiceLevel_OUTBOUND or "1")
			else
				callChannel.serviceLevel = 1 -- valor por defecto
				table.insert(errors, {
					action      = "creating op process - load campaign channel thresholds",
					isOk        = false,
					processId   = allegroOeProcess.Id,
					processType = "predictive serviceLevel",
					err         = channelCampaignThresholds
				})
			end
			-- valor nuevo en OCC, no existe el mapeo en Allegro
			callChannel.historicalContactRateFactor = 0.5
		else
		-- AUTOMATIC (solo)
			-- TODO: Proceso estos salen de la migracion de Mario Mori ?
			if #(outgoingConfigAccount.SelfServiceProcess or "") > 0 then
				-- proceso
				local success, flowInfo = info.processFlowsMigrationModule:FindProcessFlowByName(outgoingConfigAccount.SelfServiceProcess, info)
				if success then
					callChannel.process = flowInfo.Id
				else
					callChannel.process = "not found in migration"
					table.insert(errors, {
						action      = "creating op process - load process flow",
						isOk        = false,
						processId   = allegroOeProcess.Id,
						processType = "automatic process",
						allegroVal  = outgoingConfigAccount.SelfServiceProcess,
						err         = flowInfo
					})
				end
			else
				-- TODO: Revisar ya que en Allegro no es requerido.
				callChannel.process = "" -- TODO: Revisar, agrego un valor vacio ya que desde Allegro no es requerido.
			end
		end
		-- PREDICTIVO or AUTOMATIC / VOICEBLASTER (ambos) - proceso de contestador automatico
		if #(oeConfigAccount.AnsweringMachineProcess or "") > 0 then
			local success, flowInfo = info.processFlowsMigrationModule:FindProcessFlowByName(oeConfigAccount.AnsweringMachineProcess, info)
			if success then
				callChannel.answeringMachineProcess = flowInfo.Id
			else
				callChannel.answeringMachineProcess = "not found in migration"
				table.insert(errors, {
					action      = "creating op process - load process flow",
					isOk        = false,
					processId   = allegroOeProcess.Id,
					processType = "automatic-predictive answering machine",
					allegroVal  = oeConfigAccount.AnsweringMachineProcess,
					err         = flowInfo
				})
			end
		end
		-- PREDICTIVO or AUTOMATIC / VOICEBLASTER (ambos) - tasa manual
		if tostring(oeConfigAccount.CorrectionManualRate or "-1") ~= "-1" then
			callChannel.correctionManualRate = oeConfigAccount.CorrectionManualRate
		end
	else
		assert(false, "TODO: QUE HAGO CON ESTE TIPO" .. tostring(oeConfigAccount.OutboundProcessType))
	end

	----------------------------------------------
	-- root.rules (LAS REGLAS)
	local success, nativeRules = m_oeProcessesStep.MakeNativeOEProcessRules(allegroOeProcess, info)
	if success then
		nativeOeProcess.rules = nativeRules
	else
		LogWarning(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.MakeNativeOEProcess] Unable to create native rules for Allegro Process: %s", allegroOeProcess.Id)
	end
	--
	return true, nativeOeProcess, errors
end

--[[
{
  "id": "ID_DE_PROCESO",
  "configuration": {
    "maxContactRetriesDay": 1,
    "maxContactRetries": 2,
    "campaign": "ID_DE_CAMPANA@VCC",
    "channels": {
      "CALL": {
        "account": "ID_DE_CUENTA_DE_LA_CAMPANA@VCC",
        "engineType": "PROGRESSIVE",
      }
    }
  },
  "rules": [
    {
      "name": "NOMBRE_DEL_SCHEDULER",
      "id": "ID_DEL_SCHEDULER",
      "description": "DESCRIPCION_DEL_SCHEDULER",
      "schedule": {
        "scheduleDays": {},
        "scheduleSpecialDays": {}
      },
      "addresses": [
      ],
      "actions": [
      ]
    }
  ],
  "isNew": true
}
]]--
m_oeProcessesStep.GetTemplateNativeOEProcess = function()
	local jsonTemplate = [[
{
  "id": "ID_DE_PROCESO",
  "configuration": {
    "maxContactRetriesDay": 1,
    "maxContactRetries": 2,
    "campaign": "ID_DE_CAMPANA@VCC",
    "channels": {
      "CALL": {
        "account": "ID_DE_CUENTA_DE_LA_CAMPANA@VCC",
        "engineType": "PROGRESSIVE",
      }
    }
  },
  "rules": [],
  "isNew": true,
  "Allegro": []
}
]]
	return json.decode(jsonTemplate)
end

--[[
    {
      "name": "NOMBRE_DEL_SCHEDULER",
      "id": "ID_DEL_SCHEDULER",
      "description": "DESCRIPCION_DEL_SCHEDULER",
      "schedule": {
        "scheduleDays": {},
        "scheduleSpecialDays": {}
      },
      "addresses": [],
      "actions": []
    }
]]--
m_oeProcessesStep.GetTemplateNativeOEProcessRule = function()
	local jsonTemplate = [[
{
	"name": "NOMBRE_DEL_SCHEDULER",
	"id": "ID_DEL_SCHEDULER",
	"description": "DESCRIPCION_DEL_SCHEDULER",
	"schedule": {
		"scheduleDays": {},
		"scheduleSpecialDays": {}
	},
	"addresses": [],
	"actions": []
}
]]
	return json.decode(jsonTemplate)
end

--[[
]]--
m_oeProcessesStep.MakeNativeOEProcessRules = function(allegroProcess, info)
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.MakeNativeOEProcessRules] 1 - Making Native OE Process Rules. vcc: %s.", info.vccId)
	local allegroRules = allegroProcess.OutboundProcessRules or {}
	--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.MakeNativeOEProcessRules] 2 - Making Native OE Process Rules. allegroProcess.OutboundProcessRules: %s.", json.encode(allegroRules))

	info.runningModule.UpdateStepProcess(m_oeProcessesStep, {
		stepProcessId = "migrate oe process rules info",
		processId = allegroProcess.Id,
		totalRules = #allegroRules
	})

	local nativeRules = {}
	for _, allegroRule in ipairs(allegroRules) do
		local nativeRule = m_oeProcessesStep.GetTemplateNativeOEProcessRule()
		-- root
		nativeRule.id = allegroRule.ScheduleId
		nativeRule.name = allegroRule.ScheduleId
		nativeRule.description = allegroRule.Description
		-- root.schedule -> MAPEO DEL SCHEDULER LA REGLA (ALLEGRO a NATIVO)
		local allegroScheduler = allegroProcess.__AllegroSchedulers[allegroRule.ScheduleId:lower()]
		if allegroScheduler then
			nativeRule.schedule = m_oeProcessesStep.MakeNativeOEProcessRulesSchedulers(allegroScheduler.Scheduler, info)
		else
			LogWarning(LOG_PREFIX .. "JAMAS DEBE PASAR ESTO")
		end
		-- root.addresses -> MAPEO DE ADDRESS DESTINATIONS SOBRE LA REGLA (ALLEGRO a NATIVO)
		nativeRule.addresses = m_oeProcessesStep.MakeNativeOEProcessRulesAddressesDestinations(allegroRule.OutboundProcessRuleDestinations, info)
		-- root.actions -> MAPEO DE ACCIONES SOBRE LA REGLA (ALLEGRO a NATIVO)
		nativeRule.actions = m_oeProcessesStep.MakeNativeOEProcessRulesActions(allegroRule.OutboundProcessRuleResponses, info)
		-- inserto en la lista de reglas nativas
		table.insert(nativeRules, nativeRule)
		-- running
		info.runningModule.UpdateStepProcess(m_oeProcessesStep, {
			stepProcessId = "migrated oe process rule",
			processId = allegroProcess.Id,
			ruleId = nativeRule.id,
			schedule = true,
			totalAddresses = #nativeRule.addresses,
			totalActions = #nativeRule.actions
		})
	end
	return true, nativeRules
end

--[[
	------------------------------------------------------------
		MAPEO DE SCHEDULER SOBRE LA REGLA (ALLEGRO a NATIVO)
	------------------------------------------------------------

	-- allegro
		{
			"endTime": "1970-01-01 09:30:00.000",
			"startTime": "1970-01-01 08:00:00.000",
			"enabled": 1,
			"dayOfWeek": 2
		},
	-- nativo
		{
			"startHour": "10:00",
			"dayOfWeek": 2,
			"endTime": "11:00:00",
			"startTime": "10:00:00",
			"endHour": "11:00"
		}
]]--
m_oeProcessesStep.MakeNativeOEProcessRulesSchedulers = function(allegroScheduler, info)
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.MakeNativeOEProcessRulesSchedulers] 1 - Making Native OE Process Schedulers Rules. vcc: %s.", info.vccId)
	--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.MakeNativeOEProcessRulesSchedulers] 2 - Making Native OE Process Schedulers Rules. allegroScheduler: %s.", json.encode(allegroScheduler or {}))

	local nativeScheduler = {
		scheduleDays = {},
		scheduleSpecialDays = {}
	}
	-- 1. scheduleDays
	for _, allegroScheduleDay in ipairs(allegroScheduler.scheduleDays or {}) do
		-- native date format
		local startTime = LuaOrm.Date(allegroScheduleDay.startTime)
		local endTime = LuaOrm.Date(allegroScheduleDay.endTime)
		local nativeScheduleDay = {
			dayOfWeek = tonumber(allegroScheduleDay.dayOfWeek), -- 1
			startHour = startTime:toString("%H:%M"),    -- "10:00"
			startTime = startTime:toString("%H:%M:%S"), -- "10:00:00"
			endHour   = endTime:toString("%H:%M"),      -- "11:00"
			endTime   = endTime:toString("%H:%M:%S")    -- "11:00:00"
		}
		-- incluyo en la lista del dia correspondiente (el indice es el numero del dia en string, Ej: "1" corresponde a enero)
		local index = tostring(allegroScheduleDay.dayOfWeek)
		nativeScheduler.scheduleDays[index] = nativeScheduler.scheduleDays[index] or {}
		table.insert(nativeScheduler.scheduleDays[index], nativeScheduleDay)
	end
	-- 2. scheduleSpecialDays
	for _, allegroScheduleSpecialDay in ipairs(allegroScheduler.scheduleSpecialDays or {}) do
		-- native date format
		local nativeScheduleSpecialDay = utils.DeepCopy(allegroScheduleSpecialDay)
		nativeScheduleSpecialDay.overrides = IsEnabled(nativeScheduleSpecialDay.overrides) -- en Nativo tiene que ser un boolean, en Allegro es 0/1
		nativeScheduleSpecialDay.ranges = utils.DeepCopy(nativeScheduleSpecialDay.scheduleSpecialDayRanges)
		nativeScheduleSpecialDay.scheduleSpecialDayRanges = nil
		for _, specialDay in ipairs(nativeScheduleSpecialDay.ranges) do
			local startTime = LuaOrm.Date(specialDay.startTime)
			local endTime = LuaOrm.Date(specialDay.endTime)
			specialDay.startTime = startTime:toString("%H:%M:%S") -- "10:00:00"
			specialDay.endTime = endTime:toString("%H:%M:%S")     -- "11:00:00"
		end
		-- incluyo en la lista del dia correspondiente (el indice es el anio + mes + dia en string Ej: "2017-07-26")
		local index = tostring(allegroScheduleSpecialDay.year) .. "-" ..
			(tonumber(allegroScheduleSpecialDay.month) < 10 and "0" or "") .. tostring(allegroScheduleSpecialDay.month) .. "-" ..
			(tonumber(allegroScheduleSpecialDay.day) < 10 and "0" or "") .. tostring(allegroScheduleSpecialDay.day)
		nativeScheduler.scheduleSpecialDays[index] = nativeScheduleSpecialDay
	end
	-- 3. scheduleSpecialDayAllYearss
	for _, allegroScheduleSpecialDay in ipairs(allegroScheduler.scheduleSpecialDayAllYearss or {}) do
		-- native date format
		local nativeScheduleSpecialDay = utils.DeepCopy(allegroScheduleSpecialDay)
		nativeScheduleSpecialDay.overrides = IsEnabled(nativeScheduleSpecialDay.overrides) -- en Nativo tiene que ser un boolean, en Allegro es 0/1
		nativeScheduleSpecialDay.ranges = {} -- en Allegro no viene este valor, pero en Nativo tiene que estar vacio
		-- incluyo en la lista del dia correspondiente (el indice es el mes + dia en string Ej: "07-26")
		local index = (tonumber(allegroScheduleSpecialDay.month) < 10 and "0" or "") .. tostring(allegroScheduleSpecialDay.month) .. "-" ..
			(tonumber(allegroScheduleSpecialDay.day) < 10 and "0" or "") .. tostring(allegroScheduleSpecialDay.day)
		nativeScheduler.scheduleSpecialDays[index] = nativeScheduleSpecialDay
	end
	--ok
	--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.MakeNativeOEProcessRulesSchedulers] 3 - Maked Native OE Process Schedulers Rules. nativeScheduler: %s.", json.encode(nativeScheduler or {}))
	return nativeScheduler
end

--[[
	------------------------------------------------------------
		MAPEO DE ADDRESS DESTINATIONS SOBRE LA REGLA (ALLEGRO a NATIVO)
	------------------------------------------------------------
	-- allegro
		{
			"Schedule": "sch1",
			"VCC": "emmatel2",
			"TopRetries": 14,
			"Priority": 1,
			"Destination": "FAX",
			"OutboundProcessId": "process1"
		},
	-- nativo
		{
			"max_retries": 14,
			"kind": "FAX",
			"id": "CHANNEL:CALL-KIND:FAX",
			"channel": "CALL"
		}
]]--
m_oeProcessesStep.MakeNativeOEProcessRulesAddressesDestinations = function(allegroAddressesDestinations, info)
	allegroAddressesDestinations = allegroAddressesDestinations or {}
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.MakeNativeOEProcessRulesAddressesDestinations] 1 - Making Native OE Process AddressesDestinations Rules. vcc: %s.", info.vccId)
	--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.MakeNativeOEProcessRulesAddressesDestinations] 2 - Making Native OE Process AddressesDestinations Rules. allegroAddressesDestinations: %s.", json.encode(allegroAddressesDestinations))
	local nativeAddressesDestinations = {}
	-- 1. asd
	for _, allegroAddress in ipairs(allegroAddressesDestinations) do
		local nativeAddress = {
			id          = string.format("CHANNEL:CALL-KIND:%s", allegroAddress.Destination),
			channel     = "CALL",
			kind        = allegroAddress.Destination,
			max_retries = allegroAddress.TopRetries
		}
		table.insert(nativeAddressesDestinations, nativeAddress)
	end
	--ok
	--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.MakeNativeOEProcessRulesAddressesDestinations] 3 - Maked Native OE Process AddressesDestinations Rules. nativeAddressesDestinations: %s.", json.encode(nativeAddressesDestinations))
	return nativeAddressesDestinations
end

--[[
	------------------------------------------------------------
		MAPEO DE ACCIONES SOBRE LA REGLA (ALLEGRO a NATIVO)
	------------------------------------------------------------

	-- allegro
        {
          "Schedule": "scheEmma1",
          "InvalidateAddress": 0,
          "VCC": "emmatel2",
          "LastDestination": "",
          "Response": 16,
          "OutboundProcessId": "process1",
          "NextDestination": "",
          "AffectRetries": 1,
          "CancelInteraction": 1
        },
	-- nativo
      {
        "cancel": true,
        "translatedResult": "Cancelado",
        "id": "CHANNEL:CALL-RESULT:CANCELED-LAST_ADDRESS_CHANNEL:CALL-LAST_ADDRESS_KIND:ANY",
        "affect_retries": true,
        "result_number": -5,
        "channel": "CALL",
        "next_address": {
          "channel": "SAME",
          "kind": "NEXT"
        },
        "last_address": {
          "channel": "CALL",
          "kind": "ANY"
        },
        "result": "CANCELED",
        "disable_address": {
          "scope": "GLOBAL",
          "channel": "CURRENT"
        }
      },
]]--
m_oeProcessesStep.MakeNativeOEProcessRulesActions = function(allegroRulesActions, info)
	allegroAddressesDestinations = allegroAddressesDestinations or {}
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.MakeNativeOEProcessRulesActions] 1 - Making Native OE Process Rules ACTIONS. vcc: %s.", info.vccId)
	--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.MakeNativeOEProcessRulesActions] 2 - Making Native OE Process Rules ACTIONS. allegroRulesActions: %s.", json.encode(allegroRulesActions))

	local nativeRulesActions = {}
	local _, hashNativeSystemActionsByChannels = utils.GetOEProcessesSystemRulesActions()
	local hashNativeSystemActions = hashNativeSystemActionsByChannels.CALL or {}
	for _, allegroAction in ipairs(allegroRulesActions) do
		local allegroResponseNumber = tostring(allegroAction.Response)
		local nativeSystemAction = hashNativeSystemActions[allegroResponseNumber]
		if nativeSystemAction then
			local nativeAction = m_oeProcessesStep.MakeNativeOEProcessRuleAction(allegroAction, nativeSystemAction, info)
			table.insert(nativeRulesActions, nativeAction)
		else
			LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.MakeNativeOEProcessRulesActions] 3 - Allegro responde action does not exist in Native. allegroResponseNumber: %s.", allegroResponseNumber)
		end
	end
	--ok
	--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.MakeNativeOEProcessRulesActions] 3 - Maked Native OE Process Rules ACTIONS. nativeRulesActions: %s.", json.encode(nativeRulesActions))
	return nativeRulesActions
end

m_oeProcessesStep.MakeNativeOEProcessRuleAction = function(allegroRuleAction, nativeSystemAction, info)
	--LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.MakeNativeOEProcessRuleAction] 1 - Making Native OE Process Rule ACTION. vcc: %s.", info.vccId)
	--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.MakeNativeOEProcessRuleAction] 2 - Making Native OE Process Rule ACTION. allegroRuleAction: %s.", json.encode(allegroRuleAction))
	--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.MakeNativeOEProcessRuleAction] 3 - Making Native OE Process Rule ACTION. nativeSystemAction: %s.", json.encode(nativeSystemAction))
	--[[
		Si se cambia el formato del action ID, ha cambiado en:
			archivo: \usr\lib\inconcert\agent\inconcert\apps\outbound_engine\scripts\form_outbound_engine_processes_rules.js
			funcion: form_outbound_engine_processes_rules.prototype.__make_action_id
		En la funcion MakeActionId debe de quedar con el mismo formato.
	]]--
	local function MakeActionId (action)
		--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.MakeNativeOEProcessRuleAction]       1 --> action: '%s'.", json.encode(action))
		local template = [[CHANNEL:%s-RESULT:%s-LAST_ADDRESS_CHANNEL:%s-LAST_ADDRESS_KIND:%s]]
		--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.MakeNativeOEProcessRuleAction]       2 --> template: '%s'.", template)
		local actionId = string.format(template, action.channel, action.result, action.last_address.channel, action.last_address.kind):upper()
		--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.MakeNativeOEProcessRuleAction]       3 --> actionId: '%s'.", actionId)
		return actionId
	end

	local nativeAction = utils.DeepCopy(nativeSystemAction)
	nativeAction.id = MakeActionId(nativeAction)

	-- mapeo
	nativeAction.result_number    = tonumber(allegroRuleAction.AffectRetries)
	nativeAction.affect_retries   = IsEnabled(allegroRuleAction.AffectRetries)
	nativeAction.cancel           = IsEnabled(allegroRuleAction.CancelInteraction)
	nativeAction.schedule_seconds = tonumber(allegroRuleAction.SecondsToNextAction)
	-- invalid address (InvalidateAddress)
	if IsEnabled(allegroRuleAction.InvalidateAddress) then
		nativeAction.disable_address = nativeAction.disable_address or {
			channel = "CURRENT",
			scope   = "GLOBAL"
		}
	end
	-- last_address
	nativeAction.last_address = nativeAction.last_address or {channel = "CALL", kind = "ANY"}
	if allegroRuleAction.LastDestination and allegroRuleAction.LastDestination ~= "" then
		nativeAction.last_address.kind = (allegroRuleAction.LastDestination):upper()
	end
	-- NEXT ADDRESS Y CAMBIAR TIPO DE CONTACTO (PROGRESIVO/NO PROGRESIVO) (ChangeContactType)
	nativeAction.set_process_type = nil
	nativeAction.next_address = nil
	if not nativeAction.cancel and nativeAction.schedule_seconds ~= 0 then
		-- CAMBIAR TIPO DE CONTACTO
		nativeAction.set_process_type = IsEnabled(allegroRuleAction.ChangeContactType) and "PROGRESIVE" or "PREDICTIVE"
		-- next_address
		nativeAction.next_address = nativeAction.next_address or {
			channel = "SAME"
		}
		nativeAction.next_address.same_address = nil
		local nextDestination = (allegroRuleAction.NextDestination or ""):upper()
		if nextDestination == "SAME_DESTINATION" then
			nativeAction.next_address.kind = "SAME"
			nativeAction.next_address.same_address = nil
		elseif nextDestination == "SAME_ADDRESS" then
			nativeAction.next_address.kind = "SAME"
			nativeAction.next_address.same_address = true
		elseif nextDestination == "" then
			-- PROXIMO EN LA LISTA
			nativeAction.next_address.kind = "NEXT"
			nativeAction.next_address.same_address = nil
		else
			-- FAX, DOMICILIO, CELULAR, OFICINA
			nativeAction.next_address.kind = nextDestination
			nativeAction.next_address.same_address = nil
		end
	end
	--[[
		CAMBIAR PRIORIDAD DEL CONTACTO:
			Puntos de prioridad (PriorityModIsPerc): "0"
			Porcentaje          (PriorityModIsPerc): "1"
			--
			PriorityMod ( valor )
	]]--
	if allegroRuleAction.PriorityMod and allegroRuleAction.PriorityModIsPerc then
		nativeAction.change_priority = nativeAction.change_priority or {}
		nativeAction.change_priority.value = tonumber(allegroRuleAction.PriorityMod)
		nativeAction.change_priority.type = IsEnabled(allegroRuleAction.PriorityModIsPerc) and "PERCENTAGE" or "NUMBER"
	else
		nativeAction.change_priority = nil
	end
	-- Original allegro action
	nativeAction.AllegroInfo = allegroRuleAction
	--ok
	--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.MakeNativeOEProcessRuleAction] 4 - Maked Native OE Process Rule ACTION. nativeAction: %s.", json.encode(nativeAction))
	return nativeAction
end

--[[

]]--
m_oeProcessesStep.CreateNativeOEBatches = function(allegroOeProcess, info)
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.CreateNativeOEBatches] Creating Native OE batches: '%s', vcc: '%s'.", allegroOeProcess.OutboundProcessBatchs, info.vccId)
	info.runningModule.UpdateStepProcess(m_oeProcessesStep, {
		stepProcessId = "start migrate oe process batches",
		processId = allegroOeProcess.Id
	})
	local success, nativeOeBatches = m_oeProcessesStep.MakeNativeOEBatches(allegroOeProcess, info)
	if not success then
		local err = nativeOeBatches
		info.runningModule.UpdateStepProcess(m_oeProcessesStep, {
			stepProcessId = "error migrate oe process batches",
			processId = allegroOeProcess.Id,
			err = err
		})
		return false, err
	end
	local apiOeProcesses = utils.GetOEApi().native.processes -- \apps\outbound_engine\controllers\processes.lua
	local allResults = {}
	for i, nativeBatchConfig in ipairs(nativeOeBatches) do
		local batchId = nativeBatchConfig.configuration.batchName
		info.runningModule.UpdateStepProcess(m_oeProcessesStep, {
			stepProcessId = "creating oe process batch",
			processId = allegroOeProcess.Id,
			batchId = batchId
		})
		LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.CreateNativeOEBatches]      %d->  Creating Native OE BATCH: '%s, PROCESS '%s', vcc: '%s'.", i, batchId, nativeBatchConfig.processId, info.vccId)
		--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.CreateNativeOEBatches]        %d->  nativeBatchConfig: '%s'.", i, json.encode(nativeBatchConfig))
		local success, resp = apiOeProcesses.batches.Create(info.vccId, nativeBatchConfig, info.ksOE)
		if success then
			-- actualizo el batch en el proceso para marcarlo como que es un lote migrado pero aun no se han migrado sus contactos.
			local newNativeBatch = utils.GetOEModel("OutboundProcessBatches").GetBatch(info.vccId, allegroOeProcess.Id, batchId, info.ksOE)
			if not utils.TableIsEmpty(newNativeBatch) then
				-- STEP 1 de migracion
				newNativeBatch.Allegro = {
					isMigratedContacts = false
				}
				utils.GetOEModel("OutboundProcessBatches").SaveBatch(info.vccId, allegroOeProcess.Id, newNativeBatch, info.ksOE)
			end
		end
		local result = {
			action    = "creating batch",
			isOk      = success,
			processId = nativeBatchConfig.processId,
			batchId   = batchId,
			err       = not success and resp or nil
		}
		table.insert(allResults, result)
	end
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.CreateNativeOEBatches] Success created OE Process")
	-- ok
	return true, allResults
end

--[[

]]--
m_oeProcessesStep.CanAssignContactsToBatch = function(nativeBatch)
	assert(nativeBatch)
	if not nativeBatch.Allegro then
		return true, "1-CAN MIGRATE THE BATCH"  -- lo puede migrar
	end
	if not nativeBatch.Allegro.isMigratedContacts then
		return true, "2-CAN MIGRATE THE BATCH"-- lo puede migrar
	end
	-- NO lo puede migrar, el batch ya fue migrado.
	return false, "UNABLE TO MIGRATE - BATCH IS ALREADY MIGRATED"
end

--[[

]]--
m_oeProcessesStep.StartAssignContactsToBatches = function(fullProcessId, nativeOeBatches, info, allResults)
	LogInfo("-----------------------------------------------------")
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.StartAssignContactsToBatches] Start Assigning Contacts to Native OE batches: '%s', fullProcessId: '%s', vcc: '%s'.", #nativeOeBatches, fullProcessId, info.vccId)

	local apiOeProcesses = utils.GetOEApi().native.processes -- \apps\outbound_engine\controllers\processes.lua
	local importModule = info.GetStepByModule("oeImportContacts", "importations")

	local _, storedMigrationInfo = importModule:GetMigratedContactsInfo()
	LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.StartAssignContactsToBatches]     0.5--> storedMigrationInfo: '%s'.", json.encode(storedMigrationInfo))

	local processId = utils.GetId(fullProcessId:lower())
	local storedProcessInfo = storedMigrationInfo[processId]

	--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.StartAssignContactsToBatches]     0.6--> processId: '%s' storedProcessInfo: '%s'.", processId, json.encode(storedProcessInfo or {}))

	if not storedProcessInfo then
		LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.StartAssignContactsToBatches] process '%s', vcc: '%s'. No has file information", processId, info.vccId)
		return false, allResults
	end

	local execGarbageCollectorEveryBatches = 5
	local nextGarbageCollector = 0
	local totalBatchesProcessed = 0

	for i, nativeBatch in ipairs(nativeOeBatches) do
		local mustAssignContactsToBatch, resp = m_oeProcessesStep.CanAssignContactsToBatch(nativeBatch)
		LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.StartAssignContactsToBatches]    1--> '%s', %s -> processId: %s, nativeBatch.configuration: '%s'.", i, resp, fullProcessId, json.encode(nativeBatch.configuration))
		if not mustAssignContactsToBatch then
			LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.StartAssignContactsToBatches]    2--> Unable to migrate Batch '%s' of process: %s'. Current batch is already migrated.", fullProcessId, nativeBatch.configuration.name, fullProcessId)
		else
			-- tabla con la configuracion necesaria para asignar contactos a los batches en la App outboundengine->OEClient
			local batch = m_oeProcessesStep.MakeNativeOEBatch({
					processId  = fullProcessId,
					batchId    = nativeBatch.configuration.name,
					startDate  = nativeBatch.configuration.startDate,
					endDate    = nativeBatch.configuration.endDate,
					priority   = nativeBatch.configuration.batchPriority,
					percentage = nativeBatch.configuration.percentage
				},
				info
			)
			--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.StartAssignContactsToBatches]     2--> '%s', batch: '%s'.", i, json.encode(batch))
			local totalBatches = #(storedProcessInfo.batches or {})
			for j, storedBatchInfo in ipairs(storedProcessInfo.batches or {}) do
				if batch.configuration.batchName:lower() == storedBatchInfo.batchId:lower() then
					-- garbage collector -----------------
					nextGarbageCollector = nextGarbageCollector + 1
					if nextGarbageCollector == execGarbageCollectorEveryBatches then
						-- ejecuto el garbage collector
						nextGarbageCollector = 0
						utils.CollectAllMemory()
					end
					---------------------------------------
					local isOk, jsonData = pcall(info.ksOE.GetColumnValue, info.ksOE, info.vccId:lower(), "TempVccAllegroMigrationOutProcessesBatches", storedBatchInfo.fullKey)
					if isOk and jsonData then
						local storedBatch = json.decode(jsonData)
						local contacts = utils.HashToArray(storedBatch.Contacts or {}, nil, "RAW")

						info.runningModule.UpdateStepProcess(m_oeProcessesStep, {
							stepProcessId = "assign contacts batch",
							processId = fullProcessId,
							batchId = storedBatchInfo.batchId,
							totalContacts = #contacts
						})

						totalBatchesProcessed = totalBatchesProcessed + 1
						LogDebug("")
						LogInfo("----------------------------------------------------------------------------")
						LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.StartAssignContactsToBatches]     Invoke API OEClient.AddContacts -->  %s of %s batches in process --> Process: '%s', Batch: '%s', TotalContacts: %s.", totalBatchesProcessed, totalBatches, fullProcessId, storedBatchInfo.batchId, #contacts)
						LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.StartAssignContactsToBatches]         --> source-> vcc: %s storedBatchInfo.fullKey: %s", info.vccId, storedBatchInfo.fullKey)
						LogInfo("----------------------------------------------------------------------------")
						batch.configuration.batchName = storedBatchInfo.batchId
						batch.query.contacts = contacts
						local success, resp = apiOeProcesses.batches.AddContacts(info.vccId, batch, info.ksOE)
						local result = {
							action    = "add contacts to batch",
							isOk      = success,
							processId = batch.processId,
							batchId   = storedBatchInfo.batchId,
							err       = not success and resp or nil
						}
						table.insert(allResults, result)
						for contactState, data in pairs(storedBatch.ContactsByStates or {}) do
							--LogDebug("    contactState: %s", contactState)
							-- convierte la lista de contactos por estados en arrays
							local contactsByState = utils.HashToArray(data.Contacts, nil, "RAW")
							info.runningModule.UpdateStepProcess(m_oeProcessesStep, {
								stepProcessId = "changing state - contacts batch",
								processId = fullProcessId,
								batchId = storedBatchInfo.batchId,
								state = contactState,
								totalContacts = #contactsByState
							})
							LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.StartAssignContactsToBatches]          ==> Invoke API OEClient.CHANGE_CONTACT_STATE. Process: '%s', Batch: '%s', State: '%s', TotalContactsByState: %s", fullProcessId, storedBatchInfo.batchId, contactState, #contactsByState)
							m_oeProcessesStep.ExecOEMethodChangeBatchState(info, fullProcessId, storedBatchInfo.batchId, contactState, contactsByState, allResults)
						end
						-- actualizo el batch en el proceso para marcarlo como que se han migrado sus contactos. -- STEP 2 de migracion
						nativeBatch.Allegro = {
							isMigratedContacts   = true,
							migratedContactsDate = os.time()
						}
						utils.GetOEModel("OutboundProcessBatches").SaveBatch(info.vccId, fullProcessId, nativeBatch, info.ksOE)
					else
						LogWarning(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.StartAssignContactsToBatches] Batch '%s', process '%s', vcc: '%s'. Contact File does not exist: %s", batch.configuration.batchName, batch.processId, info.vccId, storedBatchInfo.fullKey)
					end
				--else
					--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.StartAssignContactsToBatches] NO Invoke OE for Differents Batches. Batch '%s', process '%s', vcc: '%s'. stored Batch: %s", batch.configuration.batchName, batch.processId, info.vccId, storedBatchInfo.batchId)
				end
			end
		end
	end
	-- ok
	-- ejecuto el garbage collector
	utils.CollectAllMemory()
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.StartAssignContactsToBatches] Success Added contacts to batches of OE Process")

	return true, allResults
end

--[[

]]--
m_oeProcessesStep.AssignContactsToBatches = function(info, allResults)
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.AssignContactsToBatches] START Assign contacts to batches of vcc: '%s'", info.vccId)
	-- para determinar el tiempo de ejecucion
	local _, starElapsedTime = m_helpers.utils.ElapsedTime.Start()
	local apiOeProcesses = utils.GetOEApi().native.processes -- \apps\outbound_engine\controllers\processes.lua
	local nativeProcesses = apiOeProcesses.LoadProcessesAndBatches(info.ksOE, info.vccId)
	-- Actualizo en Cassandra - VCC.AllegroIntegration.LastMigration la ultima pagina procesada, obtenida desde Allegro e insertada en ES.
	local stepProcess = {
		name             = "ASSIGN CONTACTS TO BATCHES",
		totalOEProcesses = utils.GetRowCount(nativeProcesses),
		totalMigrated    = 0,
		startTime        = os.time()
	}
	info.runningModule:UpdateLastMigrationStep2InVcc(info, stepProcess)
	-----
	for processId, currentProcess in pairs(nativeProcesses) do
		local nativeOeBatches = currentProcess.batches or {}
		LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.AssignContactsToBatches]   Assign contacts to '%d' batches Native OE Process '%s' -> vcc: '%s'", #nativeOeBatches, processId, info.vccId)
		m_oeProcessesStep.StartAssignContactsToBatches(processId, nativeOeBatches, info, allResults)
		stepProcess.totalMigrated = stepProcess.totalMigrated + 1
		info.runningModule:UpdateLastMigrationStep2InVcc(info, stepProcess)
	end
	-- fin del tiempo de ejecucion
	local _, endElapsedTime = m_helpers.utils.ElapsedTime.End(starElapsedTime.processId)
	local elapsedTimeDetail = m_helpers.utils.ElapsedTime.GetTimeArray(endElapsedTime.elapsedTimeSec)
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.AssignContactsToBatches] -> END Assign contacts to batches of vcc '%s' - elapsedTime: %s",
		info.vccId,
		elapsedTimeDetail.formattedTime
	)
end

--[[
	Ejecuta metodos contra la App "outboundengine" que a su vez ejecuta metodos contra el OE Client
]]--
m_oeProcessesStep.ExecOEMethodChangeBatchState = function(info, processId, batchId, method, contacts, allResults)
	assert(info)
	assert(processId)
	assert(batchId)
	assert(method)
	assert(contacts)
	assert(allResults)
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.ExecOEMethodChangeBatchState] -> START Execute - OE Method Contacts/Batches. vcc: '%s', processId: '%s', batchId: '%s', method: '%s', total contacts: '%s'.", info.vccId, processId, batchId, method, #contacts)
	local apiOeProcessesBatches = utils.GetOEApi().native.processes.batches -- \apps\outbound_engine\controllers\processes.lua
	local allowedMethodsConfig = {
		CANCELED = {
			actionName = "CANCEL BATCH",
			method     = assert(apiOeProcessesBatches.Cancel)
		},
		FINISHED = {
			actionName = "FINISH BATCH",
			method     = assert(apiOeProcessesBatches.Finish)
		},
		PAUSED = {
			actionName = "PAUSE BATCH",
			method     = assert(apiOeProcessesBatches.Pause)
		}
	}
	local methodConfig = allowedMethodsConfig[method]
	if not methodConfig then
		LogWarning(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.ExecOEMethodChangeBatchState] END Execute - Unable to change Contacts/Batches for method '%s', for this method not allowed for change. OE Batch: '%s' in Process: '%s'.", method, batchId, processId)
		return false
	end
	-- para determinar el tiempo de ejecucion
	local _, starElapsedTime = m_helpers.utils.ElapsedTime.Start()
	-- ejecuto el metodo contra el OE client
	local waitForCompleted = true
	local successExecutedActionWaitSec = 0
	local success, resp = methodConfig.method(info.vccId, processId, batchId, contacts, info.ksOE, waitForCompleted, successExecutedActionWaitSec)
	-- fin del tiempo de ejecucion
	local _, endElapsedTime = m_helpers.utils.ElapsedTime.End(starElapsedTime.processId)
	local elapsedTimeDetail = m_helpers.utils.ElapsedTime.GetTimeArray(endElapsedTime.elapsedTimeSec)

	local result = {
		action    = methodConfig.actionName,
		isOk      = success,
		processId = processId,
		batchId   = batchId,
		err       = not success and resp or nil
	}
	table.insert(allResults, result)
	if not success then
		LogWarning(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.ExecOEMethodChangeBatchState] END Execute - Error '%s' OE Batch: '%s' in Process: '%s'. Err: %s", methodConfig.actionName, batchId, processId, json.encode(result.err or {}))
		return false
	end
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.ExecOEMethodChangeBatchState] -> END Execute - Success Changed Contacts/Batches States. vcc: '%s', processId: '%s', batchId: '%s', method: '%s', total contacts: '%s'. elapsedTime: %s",
		info.vccId,
		processId,
		batchId,
		method,
		#contacts,
		elapsedTimeDetail.formattedTime
	)
	return true
end

--[[
Allegro
    {
      "EffectiveContacts": 0,
      "CreationQuery": "select DISTINCT [ContactImportContact].[VCC], [ContactImportContact].[ID], [ContactImportContact].[Name], [ContactImportContact].[Category], [ContactImportContact].[Campaign], [ContactImportContact].[AccountOfficer], [ContactImportContact].[AccountGroup], [ContactImportContact].[LastManagementResult], [ContactImportContact].[LastUpdate], [ContactImportContact].[VIP], [ContactImportContact].[IsClient] from [ContactImportContact] (nolock) , [ContactImport]  (nolock)   where [ContactImportContact].[VCC] = 120 AND [ContactImportContact].[VIP] = 0 AND [ContactImport].[Status]<>'RUNNING' AND ContactImport.ID = ContactImportContact.LastUpdate AND ContactImport.VCC = ContactImportContact.VCC AND [ContactImportContact].[LastUpdate] = [ContactImport].[ID] AND [ContactImportContact].[VCC] = [ContactImport].[VCC]",
      "VCC": "emmatel2",
      "Expired": true,
      "BatchId": "LoTeUnO",
      "RemainingContacts": 270,
      "EndDate": "2022-12-08 00:00:00.000",
      "OutboundProcessId": "process1",
      "CreationDate": "2022-12-08 16:17:20.000",
      "CancelledContacts": 0,
      "StartDate": "2022-12-07 00:00:00.000",
      "Priority": 1,
      "CreationContacts": 270,
      "Percentage": -1
    }

Nativo

[
	{
		"processId": "proceso1@vcc1",
		"configuration": {
			"batchName": "EmmaVallejo_deMello1",
			"batchPriority": "1",
			"startDate": "2023-03-15",
			"endDate": "",
			"basePriority": "100",
			"percentage": ""
		},
		"query": {
			"values": [],
			"lines": [],
			"sqlSentence": "SELECT * FROM TBL ",
			"sqlParsed": {
			"value": {
				"where": null
			}
			}
		},
		"source": "contact_query"
	}
]
}
]]--
m_oeProcessesStep.MakeNativeOEBatches = function(allegroOeProcess, info)
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.MakeNativeOEBatches] 1 - Making Native OE Batches. Allegro OE ProcessId: %s, vcc: %s.", allegroOeProcess.Id, info.vccId)
	--LogDebug(LOG_PREFIX .. "[[PROVISIONING.MIGRATIONS.OE_PROCESSES.MakeNativeOEBatches] 2 - Making Native OE Batches. Allegro OE Process - data: %s.", utils.JSON_Encode(allegroOeProcess.OutboundProcessBatchs))
	--LogDebug(LOG_PREFIX .. "[[PROVISIONING.MIGRATIONS.OE_PROCESSES.MakeNativeOEBatches] 3 - Making Native OE Batches. Native Contacts for Batches - info.migratedContacts: %s.", utils.JSON_Encode(info.migratedContacts))
	local nativeBatches = {}
	for _, allegroBatch in ipairs(allegroOeProcess.OutboundProcessBatchs or {}) do
		local batchInfo = {
			processId  = allegroBatch.OutboundProcessId,
			batchId    = allegroBatch.BatchId,
			startDate  = allegroBatch.StartDate,
			endDate    = allegroBatch.EndDate,
			priority   = allegroBatch.Priority,
			percentage = allegroBatch.Percentage
		}
		-- tabla con la configuracion necesaria para crear los batches en la App outboundengine->OEClient
		local nativeBatch = m_oeProcessesStep.MakeNativeOEBatch(batchInfo, info)
		table.insert(nativeBatches, nativeBatch)
	end
	-- ok
	--LogDebug(LOG_PREFIX .. "[[PROVISIONING.MIGRATIONS.OE_PROCESSES.MakeNativeOEBatches] 4 - Maked Native OE Batches. nativeBatches: %s.", utils.JSON_Encode(nativeBatches))
	return true, nativeBatches
end

--[[

]]--
m_oeProcessesStep.MakeNativeOEBatch = function(batchInfo, info)
	local processId = utils.AppendVcc(utils.GetId(batchInfo.processId), info.vccId)
	local batchId = (batchInfo.batchId or ""):lower()
	return {
		processId = processId,
		configuration = {
			batchName     = batchId,
			startDate     = StringDate(batchInfo.startDate),
			endDate       = StringDate(batchInfo.endDate),
			batchPriority = batchInfo.priority,
			basePriority  = 100, -- TODO: Martin ver de donde sale
			percentage    = tostring(batchInfo.percentage < 0 and "" or batchInfo.percentage) -- TODO: ver negativo
		},
		source = "contact_query",
		query = {
			scopeType = "CONTACTS",
			contacts = {} -- para crear batches sin contactos.
		}
	}
end

----------------------------
-- Rollback
----------------------------

--[[

]]--
m_oeProcessesStep.Rollback = function(info)
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.Rollback] Rollback Migration OE Processes for vcc '%s'.", info.vccId)

	------------------------------------------------------------------------------------------
	-- A - los procesos + (reglas: schedulers + destinations + actions) + (batches)
	--
	local apiOeProcesses = utils.GetOEApi().native.processes -- \apps\outbound_engine\controllers\processes.lua
	local oeProcesses = apiOeProcesses.LoadProcessesAndBatches(info.ksOE, info.vccId)
	--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.Rollback] --> vcc: %s OE Processes: '%s'.", info.vccId, json.encode(oeProcesses))

	for processId, currentProcess in pairs(oeProcesses) do
		LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.Rollback]  Deleting Native OE Process '%s' -> vcc: '%s'", processId, info.vccId)

		-- 1 - creo el step de running
		local runningInfo = {
			stepProcessId = "rollback oe process",
			processId = processId,
			batches = {}
		}
		info.runningModule.UpdateStepProcess(m_oeProcessesStep, runningInfo)

		-- valido si se pueden eliminar todos los batches del proceso.
		for _, batch in pairs(currentProcess.batches or {}) do
			-- realizo rollback del batch
			local batchName = batch.configuration.name
			LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.Rollback]     -->  Deleting Native OE Batch '%s'  of Process '%s' -> vcc: '%s'. Wait moment !!", batchName, processId, info.vccId)
			local success, result = apiOeProcesses.batches.Destroy(info.vccId, processId, batchName)
			--require("socket").sleep(3) -- Pequena pausa necesaria para que el OE haga sus magias.
			table.insert(runningInfo.batches, {
				id = "rollback oe process batch",
				processId = processId,
				batchName = batchName,
				success = success,
				result = result or {}
			})
		end
		-- 2 actualizo el proceso con el resultado de los batches
		info.runningModule.UpdateStepProcess(m_oeProcessesStep, runningInfo)
		-- 3 realizo rollback del proceso
		local success, result = apiOeProcesses.DeleteProcess(processId, info.vccId, info.ksOE, info.ksAgent)
		table.insert(runningInfo.batches, {
			id = "rollback oe process",
			processId = processId,
			success = success,
			result = result or {}
		})
		-- 4 actualizo el proceso con el resultado de los oe process
		info.runningModule.UpdateStepProcess(m_oeProcessesStep, runningInfo)
	end

	------------------------------------------------------------------------------------------
	-- B - los filtros guardados de los batches
	--
	-- 5 - creo el step de running
	local runningInfo = {
		stepProcessId = "rollback oe batch filters",
		filters = {}
	}
	info.runningModule.UpdateStepProcess(m_oeProcessesStep, runningInfo)
	-- 6 - obtengo los filtros desde allegro
	local apiOeBatchFilters = apiOeProcesses.batches.filters
	local result = apiOeBatchFilters.Load(info.vccId, info.ksOE)
	LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.Rollback] --> vcc: %s OE oeBatchFilters: '%s'.", info.vccId, json.encode(oeBatchFilters))

	if not result then
		local err = result or {}
		LogWarning(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.Rollback] Error loading OE Batch Filters. Err: %s", json.encode(err))
		runningInfo.success = false
		runningInfo.result = err
		runningInfo.errType = "ERROR LOADING ALLEGRO FILTERS"
		-- actualizo el proceso con el resultado de los oe process
		info.runningModule.UpdateStepProcess(m_oeProcessesStep, runningInfo)
	else
		local oeBatchFilters = result or {}
		for _, currentFilter in pairs(oeBatchFilters) do
			local filterId = currentFilter.id
			LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.Rollback] Deleting Native OE Batch Filter '%s' -> vcc: '%s'", filterId, info.vccId)
			local success, result = apiOeBatchFilters.Delete(info.vccId, filterId, info.ksOE)
			table.insert(runningInfo.filters, {
				id = "rollback oe batch filters",
				filterId = filterId,
				success = success,
				result = result or {}
			})
			info.runningModule.UpdateStepProcess(m_oeProcessesStep, runningInfo)
		end
	end
	-- ok
	return true
end

----------------------------------------------------------------
-- FILTERS BATCHES
----------------------------------------------------------------
--[[

]]--
m_oeProcessesStep.LoadAllegroBatchFilters = function(info)
	local moduleAllegroProcess = utils.GetControllerOthersApps("allegro", {controllerName = "outboundengine_processes", modelName = nil})
	local requestInfo = {
		vcc = info.vccId,
		last = "",
		limit = 1000, -- TODO: paginar
		numberPage = 1,
		query = {}
	}
	local success, data = moduleAllegroProcess.LoadBatchFilters(info.vccId)
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.LoadAllegroBatchFilters] --> vcc: %s Allegro OE Batch Filters request. success: '%s'.", info.vccId, success)
	--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.LoadAllegroBatchFilters] --> vcc: %s Allegro OE Batch Filters request. data: '%s'.", info.vccId, utils.JSON_Encode(data))
	if not success then
		local err = data
		return false, err
	end
	-- ok
	return true, data
end

--[[

]]--
m_oeProcessesStep.UpdateBatchFilter = function(allegroBatchFilter, info)
	--LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.UpdateBatchFilter] Update Native OE batch filter: '%s'.", allegroBatchFilter)
	--
	info.runningModule.UpdateStepProcess(m_oeProcessesStep, {
		stepProcessId = "migrate oe batch filters",
		filterId = allegroBatchFilter.Id
	})
	-- 1- Creo el Nativo Batch Filter
	local success, nativeBatchFilter = m_oeProcessesStep.MakeNativeBatchFilter(allegroBatchFilter, info)
	if not success then
		local err = result
		return false, err
	end
	-- 2 - Almaceno el Batch Filter Nativo -- \apps\outbound_engine\controllers\processes.lua
	local success, result = utils.GetOEApi().native.processes.batches.filters.Save(info.vccId, nativeBatchFilter, info.ksOE)
	if not success then
		local err = result
		LogWarning(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.UpdateBatchFilter] Error creating OE batch filter. Err: %s", json.encode(err))
		return false, result
	end
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.UpdateBatchFilter] Success created OE batch filter")
	return true, result
end

--[[

]]--
m_oeProcessesStep.MakeNativeBatchFilter = function(allegroOeBatchFilter, info)
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.MakeNativeBatchFilter] 1 - Making Native OE batch filter. allegroOeBatchFilter: '%s', vcc: '%s'.", allegroOeBatchFilter, info.vccId)
	--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.MakeNativeBatchFilter]    - Making Native OE batch filter. allegroOeBatchFilter: '%s', vcc: '%s'.", json.encode(allegroOeBatchFilter), info.vccId)
	local allegroFilterSentence = json.decode(allegroOeBatchFilter.FilterSentence)
	--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.MakeNativeBatchFilter]    2- Making Native OE batch filter. allegroOeBatchFilter.FilterSentence: '%s', vcc: '%s'.", json.encode(allegroFilterSentence), info.vccId)
	--
	local nativeBatchFilter = {
		id               = allegroFilterSentence.Id or "NONE",
		contactedDate    = allegroFilterSentence.ContactedDate,
		excludeContacted = tostring(allegroFilterSentence.ExcludeContacted == "1") and true or false,
		query            = {}
	}
	--
	local apiOeProcesses = utils.GetOEApi().native.processes -- \apps\outbound_engine\controllers\processes.lua
	local apiOeBatchFilters = apiOeProcesses.batches.filters
	local nativeFieldsDef = apiOeBatchFilters.LoadBatchContactFields(info.vccId, info.ksAgent)

	for _, allegroQuery in ipairs(allegroFilterSentence.Query or {}) do
		local success, nativeQuery = m_oeProcessesStep.MakeNativeBatchFilterQueryLine(allegroQuery, info, nativeFieldsDef)
		if success then
			table.insert(nativeBatchFilter.query, nativeQuery)
		else
			table.insert(nativeBatchFilter.query, allegroQuery)
			-- TODO retornar errores
		end
	end
	---- ok
	--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.MakeNativeBatchFilter] 3 - Maked Native OE batch filter. nativeBatchFilter: '%s', vcc: '%s'.", json.encode(nativeBatchFilter), info.vccId)
	return true, nativeBatchFilter
end

--[[

]]--
m_oeProcessesStep.MakeNativeBatchFilterQueryLine = function(allegroQuery, info, nativeFieldsDef)
	local allegroFieldName = allegroQuery.field
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.MakeNativeBatchFilterQueryLine] 1 - Making Native OE batch filter query line for allegroFieldName: '%s', vcc: '%s'.", allegroFieldName, info.vccId)
	-- LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.MakeNativeBatchFilterQueryLine] 2  - Making Native OE batch filter query line. allegroQuery: '%s', vcc: '%s'.", json.encode(allegroQuery), info.vccId)
	--
	local map = m_oeProcessesStep.LoadMappedNativeFieldsWithAllegro(info, allegroQuery, nativeFieldsDef)
	local nativeFieldDef = map[(allegroFieldName or ""):lower()]
	if not nativeFieldDef then
		local err = {errType = "UNABLE TO MAP BATCH FILTER FIELD - FIELD NOT MATCH IN OCC", allegroFieldName = allegroFieldName}
		LogWarning(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.MakeNativeBatchFilterQueryLine] Unable to make Native OE batch filter query line for allegroFieldName: '%s', vcc: '%s'. Err: %s", allegroFieldName, info.vccId, json.encode(err))
		return false, err
	end
	local nativeQuery = utils.DeepCopy(allegroQuery)
	nativeQuery.field = nativeFieldDef.Field
	if nativeFieldDef.AllegroFieldMap.MakeValue then
		nativeQuery.value = nativeFieldDef.AllegroFieldMap.MakeValue(allegroQuery.value)
	end
	---- ok
	LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.MakeNativeBatchFilterQueryLine] 3 - Maked Native OE batch filter query line. nativeQuery: '%s', vcc: '%s'.", json.encode(nativeQuery), info.vccId)
	return true, nativeQuery
end

--[[

]]--
m_oeProcessesStep.LoadMappedNativeFieldsWithAllegro = function(info, allegroQuery, nativeFieldsDef)
	local map = {}
	for _, def in ipairs(nativeFieldsDef or {}) do
		if def.AllegroFieldMap then
			local fieldName 
			if type(def.AllegroFieldMap.Field) == "string" then
				fieldName = def.AllegroFieldMap.Field
			elseif type(def.AllegroFieldMap.Field) == "function" then
				fieldName = def.AllegroFieldMap.Field(def, allegroQuery)
			end
			if fieldName and fieldName ~= "NONE" then
				map[fieldName:lower()] = def
			end
		end
	end
	--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.LoadMappedNativeFieldsWithAllegro] ----> Created batch filter Map: '%s', vcc: '%s'.", json.encode(map), info.vccId)
	return map
end

----------------------------------------------------------------
-- CAMPAIGNS THRESHOLDS
----------------------------------------------------------------
--[[

]]--
m_oeProcessesStep.GetChannelCampaignThresholds = function(self, campaignIdWithVCC, channel, info)
	local _, modelCampaigns = utils.GetControllerOthersApps("administration", {controllerName = "campaigns", modelName = "Campaigns"})
	local channelCampaignThresholds = modelCampaigns.GetChannelCampaignThresholds(campaignIdWithVCC, "CALL", info.ksAgent)
	if utils.TableIsEmpty(channelCampaignThresholds) then
		local err = {errType = "NOT CALL THRESHOLDS IN VCC", vcc = info.vccId}
		LogWarning(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.GetChannelCampaignThresholds] --> Unable to load campaign '%s' channel '%s' thresholds in vcc '%s'. err: '%s'.", campaignIdWithVCC, channel, info.vccId, json.encode(err))
		return false, err
	end
	LogInfo(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.GetChannelCampaignThresholds] --> Loaded campaign '%s' channel '%s' thresholds in vcc '%s'.", campaignIdWithVCC, channel, info.vccId)
	--LogDebug(LOG_PREFIX .. "[PROVISIONING.MIGRATIONS.OE_PROCESSES.GetChannelCampaignThresholds] ----> vcc: %s Loaded campaign '%s' channel '%s' thresholds in vcc '%s'. Data: %s", campaignIdWithVCC, channel, info.vccId, utils.JSON_Encode(channelCampaignThresholds))
	-- ok
	return true, channelCampaignThresholds
end

----------------------------------------------------------------
-- STEPS DE LA MIGRACION
----------------------------------------------------------------

--
table.insert(m_migrationsSteps, m_oeProcessesStep) -- agrego step al array

--------------------------------------------------------------------------------------

--[[

]]--
m_migration.GetSteps = function()
	return m_migrationsSteps
end

--[[

]]--
m_migration.Init = function(config)
	for _, step in ipairs(m_migration.GetSteps()) do
		step.migrationId = config.id
	end
end

------------------------------------------
--
-- Exported functions
--
return {
	AssignContactsToBatches = m_oeProcessesStep.AssignContactsToBatches,
	GetSteps                = m_migration.GetSteps,
	Init                    = m_migration.Init,
	LoadAllegroProcesses    = m_oeProcessesStep.LoadAllegroProcesses
}