local utils = require "shared.apps.utils"

module(..., package.seeall, inconcert.orbit.application.Decorate)

local DEFINITION_TIMEOUT = 30

m_htmlRender = nil
m_appLocation = nil
local m_helpers = nil
local m_definition = nil

local function GetLuaFiles(folderName)
	local dir = m_htmlRender.extension.appLocation .. "/system_monitoring/" .. folderName
	local luas = {}
	for file in lfs.dir(dir) do
		if file ~= "." and file ~= ".." then
			local f = dir..'/'..file
			local attr = lfs.attributes (f)
			if attr.mode ~= "directory" then
				local fileName = string.match(file, [[(.+).lua]])
				if fileName then
					luas[fileName] = require("apps.provisioning.system_monitoring." .. folderName .. "." .. fileName)
				end
			end
		end
	end
	return luas
end

local function LoadDefinitionCategories()
	return GetLuaFiles("categories")
end

local function LoadDefinitionViews()
	return GetLuaFiles("views")
end

local function LoadDefinition()
	if m_definition ~= nil then return end

	LogDebug("[system_monitoring:LoadDefinition] loading definition")
	m_definition = {
		categories = LoadDefinitionCategories(),
		views = LoadDefinitionViews()
	}
	LogDebug("[system_monitoring:LoadDefinition] definition loaded")
end

local function GetDefinition(web)
	LogDebug("[system_monitoring:GetDefinition] making definition")
	LoadDefinition()
	local result = {}
	local _T = m_htmlRender.extension.makeTextTranslator()
	for categoryId, category in pairs(m_definition.categories) do
		result[categoryId] = category.GetDefinition(_T)
		result[categoryId].views = {}
	end
	for viewId, view in pairs(m_definition.views) do
		local viewDefinition = view.GetDefinition(_T)
		result[viewDefinition.category].views[viewId] = viewDefinition
	end
	LogDebug("[system_monitoring:GetDefinition] definition made")
	return result
end

local function GetDetails(web, viewId, itemId)
	LogDebug("[system_monitoring:GetDetails] running view %s item %s", viewId, itemId)
	LoadDefinition()
	local request = web.request
	local view = m_definition.views[viewId]
	if not view then
		LogWarning("[system_monitoring:GetDetails] view %s not found", viewId)
		web.status = 404
		return { result = "NOT_FOUND", reason = "view not found" }
	end

	local result = view.GetDetails(itemId)
	if result.result ~= "OK" then
		LogError("[system_monitoring:GetDetails] view %s item %s %s", viewId, itemId, json.encode(result))
		web.status = 500
		return result
	end

	LogInfo("[system_monitoring:GetDetails] view %s item %s data found", viewId, itemId)
	return result
end

local function RunView(web, viewId)
	LogDebug("[system_monitoring:RunView] running view %s %s", viewId, web.input["post_data"])
	LoadDefinition()
	local request = web.request
	local _T = m_htmlRender.extension.makeTextTranslator()
	local view = m_definition.views[viewId]
	if not view then
		LogWarning("[system_monitoring:RunView] view %s not found", viewId)
		web.status = 404
		return { result = "NOT_FOUND", reason = "view not found" }
	end

	local result = view.Run(request, _T)
	if result.result ~= "OK" then
		LogError("[system_monitoring:RunView] view %s %s", viewId, json.encode(result))
		web.status = 500
		return result
	end

	LogInfo("[system_monitoring:RunView] view %s %d rows found", viewId, #result.data)
	return result
end

function MapUris (module, htmlRender, prefix, appLocation)
	local LogRequest = require("apps.agent.requestFilters").LogRequest
	local replyFn = utils.JsonReply

	local fn = utils.JsonValidatorReply .. LogRequest
	local p = prefix
	-- RENDER FRM
	module:dispatch_get(Render, p .. "/frm/render/")

	-- Processors i6
	module:dispatch_get (replyFn .. GetDefinition, p .. "/definition/")
	module:dispatch_get (replyFn .. GetDetails, p .. "/details/(.+)/(.+)")
	module:dispatch_post(fn .. RunView, p .. "/run_view/(.+)")

	m_htmlRender = htmlRender
	m_appLocation = appLocation
end
