return {
	{
		name = "Global",
		description = "Global methods",
		methods = {
			{
				name = "GetCallType",
				description = "Get call type. Available types: INCOMING, OUTGOING, OUTGOING_LOCAL, INCOMING_PROXY.",
				snippet = [[local callType = GetCallType()]],
				examples = [[
local callType = GetCallType()
if callType == "INCOMING" then
	--incoming calls
elseif callType == "OUTGOING" then
	--outgoing calls
elseif callType == "OUTGOING_LOCAL" then
	--internal calls assigning a call to an agent
elseif callType == "INCOMING_PROXY" then
	--internal calls between nodes to make a bridge when both legs are in different node.
end
				]]
			},
			{
				name = "GetCallerId",
				description = "Get contact caller id.",
				snippet = [[local callerId = GetCallerId()]],
				examples = [[local callerId = GetCallerId()]]
			},
			{
				name = "SetCallerId",
				description = "Set contact caller id.",
				snippet = [[local status = SetCallerId("CALLER_ID")]],
				examples = [[local status = SetCallerId("1234")]]
			},
			{
				name = "GetCallerName",
				description = "Get contact caller name.",
				snippet = [[local callerName = GetCallerName()]],
				examples = [[local callerName = GetCallerName()]]
			},
			{
				name = "SetCallerName",
				description = "Set contact caller name.",
				snippet = [[local status = SetCallerName("CALLER_NAME")]],
				examples = [[local status = SetCallerName("contact name")]]
			},
			{
				name = "GetDnis",
				description = "Get DNIS for inbound call.",
				types = { "INCOMING" },
				snippet = [[local dnis = GetDnis()]],
				examples = [[local dnis = GetDnis()]]
			},
			{
				name = "SetDnis",
				description = "Set DNIS for inbound call.",
				types = { "INCOMING" },
				snippet = [[local status = SetDnis("DNIS")]],
				examples = [[local status = SetDnis("3001")]]
			},
			{
				name = "GetDialTech",
				description = "Get tech assigned to make outgoing call. Examples: SIP, ZAP, etc.",
				types = { "OUTGOING", "OUTGOING_LOCAL" },
				snippet = [[local dialTech = GetDialTech()]],
				examples = [[local dialTech = GetDialTech()]]
			},
			{
				name = "SetDialTech",
				description = "Set tech assigned to make outgoing call. Examples: SIP, ZAP, etc.",
				types = { "OUTGOING", "OUTGOING_LOCAL" },
				snippet = [[local status = SetDialTech("TECH")]],
				examples = [[local status = SetDialTech("SIP")]]
			},
			{
				name = "GetDialNumber",
				description = "Get number assigned to make outgoing call (phone number to be called on dial).",
				types = { "OUTGOING", "OUTGOING_LOCAL" },
				snippet = [[local number = GetDialNumber()]],
				examples = [[local number = GetDialNumber()]]
			},
			{
				name = "SetDialNumber",
				description = "Set number assigned to make outgoing call (phone number to be called on dial).",
				types = { "OUTGOING", "OUTGOING_LOCAL" },
				snippet = [[local status = SetDialNumber("NUMBER")]],
				examples = [[local status = SetDialNumber("1234")]]
			},
			{
				name = "GetCallTrunk",
				description = "Get call associated trunk.",
				types = { "INCOMING", "OUTGOING" },
				snippet = [[local trunkId = GetCallTrunk()]],
				examples = [[local trunkId = GetCallTrunk()]]
			},
			{
				name = "SetCallTrunk",
				description = "Set trunk assigned to make outgoing call.",
				types = { "OUTGOING" },
				snippet = [[local status, err = SetCallTrunk(TRUNK_ID)]],
				examples = [[local status, err = SetCallTrunk("trunki6_202_193")]]
			},
			{
				name = "GetTrunkCapacity",
				description = "Get trunk assigned capacity (-1 if is unlimited, nil if trunk not found).",
				snippet = [[local capacity = GetTrunkCapacity(TRUNK_ID)]],
				examples = [[local capacity = GetTrunkCapacity("trunki6_202_193")]]
			},
			{
				name = "GetTrunkUsedChannels",
				description = "Get trunk number of channels in use (nil if trunk not found).",
				snippet = [[local usedChannels = GetTrunkUsedChannels(TRUNK_ID)]],
				examples = [[local usedChannels = GetTrunkUsedChannels("trunki6_202_193")]]
			},
			{
				name = "GetTrunkFreeChannels",
				description = "Get trunk number of free channels (-1 if is unlimited, nil if trunk not found).",
				snippet = [[local freeChannels = GetTrunkFreeChannels(TRUNK_ID)]],
				examples = [[local freeChannels = GetTrunkFreeChannels("trunki6_202_193")]]
			},
			{
				name = "GetContextData",
				description = "Get context data from incoming call.",
				snippet = [[local contextData = GetContextData()]],
				examples = [[
local contextData = GetContextData()				
originalCallerID = contextData and contextData.contactAddress or "unknown"
				]]
			},	
			{
				name = "SetContextData",
				description = "Set context data.",
				snippet = [[status = SetContextData({})]],
				examples = [[
local contextData = { data = "value", data2 = "value2" }		
status = SetContextData(contexData)
				]]
			},
			{
				name = "GetInteractionId",
				description = "Get interaction id.",
				snippet = [[local interactionId = GetInteractionId()]],
				examples = [[
local interactionId = GetInteractionId()
				]]
			},
			{
				name = "GetInteraction",
				description = "Get interaction json.",
				snippet = [[local interaction = GetInteraction()]],
				examples = [[
local interaction = GetInteraction()
				]]
			},
			{
				name = "LogDebug",
				description = "Log a message at DEBUG level.",
				snippet = [[LogDebug("MESSAGE")]],
				examples = [[LogDebug("this is a debug message")]]
			},
			{
				name = "LogInfo",
				description = "Log a message at INFO level.",
				snippet = [[LogInfo("MESSAGE")]],
				examples = [[LogInfo("this is an info message")]]
			},
			{
				name = "LogWarning",
				description = "Log a message at WARNING level.",
				snippet = [[LogWarning("MESSAGE")]],
				examples = [[LogWarning("this is a warning message")]]
			},
			{
				name = "LogError",
				description = "Log a message at ERROR level.",
				snippet = [[LogError("MESSAGE")]],
				examples = [[LogError("this is an error message")]]
			},
		},
	},
	{
		name = "channel",
		description = "Asterisk channel manipulation. Allows you to get/set channel context variables and call channel methods.",
		examples = [[
--CONTEXT VARIABLES

--get
local value = channel.myVar:get() --dialplan ${myVar}

--set
channel.myVar:set("value") --dialplan Set(myVar=value)

--CHANNEL METHODS. Full list: https://wiki.asterisk.org/wiki/display/AST/Asterisk+16+Dialplan+Functions

--call method to GET a value. Example: SIPPEER
local peerName = "trunki6_202_193"
local status, value = channel.SIPPEER(peerName, "status"):get() --dialplan SIPPEER(trunki6_202_193,status)
if status then
	LogInfo(string.format("peer %s status is %s", peerName, tostring(value)))
else
	LogError(string.format("error getting peer %s status", peerName))
end

--call method to SET a value. Example: VOLUME
local status = VOLUME(TX):set("3") --dialplan Set(VOLUME(TX)=3)
if status then
	LogInfo("channel volume changed")
else
	LogError("error changing channel volume")
end
		]],
		methods = {
			{
				name = "get",
				description = "Gets a channel context variable value.",
				snippet = [[local value = channel.VAR_NAME:get()]],
				examples = [[
local value = channel.myVar:get()
if value then
	LogInfo(string.format("channel variable value is %s", value))
else
	LogWarning("channel variable value is empty")
end
				]]
			},
			{
				name = "set",
				description = "Sets a channel context variable value.",
				snippet = [[local status = channel.VAR_NAME:set("VALUE")]],
				examples = [[
local status = channel.myVar:set("value")
if status then
	LogInfo("channel variable value set")
else
	LogError("error setting channel variable value")
end
				]]
			},
			{
				name = "CHANNEL",
				description = "Retrieves channel information",
				snippet = [[local status, value = channel.CHANNEL("ITEM"):get()]],
				examples = [[
--Full item options at https://wiki.asterisk.org/wiki/display/AST/Asterisk+16+Function_CHANNEL
local status, peerName = channel.CHANNEL("peername"):get() --dialplan CHANNEL(peername)
if status then
	LogInfo(string.format("channel peername is %s", peerName))
else
	LogError("error getting channel peername")
end
				]]
			},
			{
				name = "SIPPEER",
				description = "Retrieves peer information",
				snippet = [[local status, value = channel.SIPPEER("PEER_NAME", "ITEM"):get()]],
				examples = [[
--Full item options at https://wiki.asterisk.org/wiki/display/AST/Asterisk+16+Function_SIPPEER
local peerName = "trunki6_202_193"
local status, value = channel.SIPPEER(peerName, "status"):get() --dialplan SIPPEER(trunki6_202_193,status)
if status then
	LogInfo(string.format("peer %s status is %s", peerName, tostring(value)))
else
	LogError(string.format("error getting peer %s status", peerName))
end
				]]
			},
			{
				name = "SIP_HEADER",
				description = "Retrieves a SIP INVITE header value for incoming calls",
				snippet = [[local status, value = channel.SIP_HEADER("HEADER_NAME"):get()]],
				examples = [[
--Full doc at: https://wiki.asterisk.org/wiki/display/AST/Asterisk+16+Function_SIP_HEADER

local status, value = channel.SIP_HEADER("From"):get() --dialplan SIP_HEADER(From)
LogInfo(string.format("From header is %s", tostring(value)))
				]]
			},
		},
	},
	{
		name = "app",
		description = "Asterisk app. Allows you to call dialplan applications methods.",
		examples = [[
--Full list: https://wiki.asterisk.org/wiki/display/AST/Asterisk+16+Dialplan+Applications

--Example: Playback
local status = app.playback("demo-congrats") -- dialplan Playback(demo-congrats)
if status then
	LogInfo("audio played")
else
	LogError("error playing audio")
end
		]],
		methods = {
			
		},
	},
	{
		name = "Use Cases",
		description = "Common use cases examples.",
		methods = {
			{
				name = "Override outgoing call trunk",
				description = "Changes outgoing call trunk when current has 100 busy channels.",
				types = { "OUTGOING" },
				examples = [[
local assignedTrunk = GetCallTrunk()
local failoverTrunk = "trunk2" --failover trunk id
local usedChannels = GetTrunkUsedChannels(assignedTrunk)
if usedChannels >= 100 then
	--switch to failover trunk when default trunk has more than 100 channels in use
	local status, err = SetCallTrunk(failoverTrunk)
	if status then
		LogInfo(string.format("Call trunk switched from %s to %s", assignedTrunk, failoverTrunk))
	else
		LogError(string.format("Error changing call trunk from %s to %s - %s", assignedTrunk, failoverTrunk, err))
	end
end
				]]
			},
		},
	},
}