module(..., package.seeall)

local utils = require "shared.apps.utils"
local PM = require("provisioningmanager")
local m_datasource

function GetName ()
	return "Carriers"
end

function GetDefinition (datasource)
	m_datasource = datasource
	PM.SetRedis(datasource.redis)
	return _M
end

function ListAll (ksProv)
	local releaseConn = false
	if not ksProv then
		ksProv = utils.GetDatasource():CassandraConnection("Provisioning")
		releaseConn = true
	end

	local success, carriers = pcall(PM.ListCarriers, ksProv)
	--LogDebug("[provisioning:models.carriers.ListAll]--> success: '%s', Carriers: %s", success, json.encode(carriers or {}))

	if not success then
		local errMsg = carriers
		LogWarning("[provisioning:models.carriers.ListAll] Error retrieving carriers  -> '%s'", errMsg)
	end

	if releaseConn then
		ksProv:Close()
	end
	carriers = carriers or {}

	return success, carriers
end

function Get (carrierId, ksProv)
	--LogDebug("[provisioning:models.carriers.Get] Retrieving CARRIER: '%s'.", vccarrierIdcId)

	local releaseConn = false
	if not ksProv then
		ksProv = utils.GetDatasource():CassandraConnection("Provisioning")
		releaseConn = true
	end

	local success, currentCarrier = pcall(PM.GetCarrier, ksProv, carrierId)
--	LogDebug("[provisioning:models.carriers.Get] Retrieved CARRIER '%s' -> carrierId: '%s', currentCarrier: '%s'.", carrierId, success, json.encode(currentCarrier or {}))

	if not success then
		local errMsg = currentCarrier
		LogWarning("[provisioning:models.carriers.Get] Error retrieving CARRIER '%s' -> '%s'", carrierId, errMsg)
	end

	if releaseConn then
		ksProv:Close()
	end

	if utils.TableIsEmpty(currentCarrier) then
		success = false
		local errMsg = "CARRIER DOES NOT EXIST"
		--LogWarning("[provisioning:models.carriers.Get] Error retrieving CARRIER '%s'. CARRIER does not exist.'", carrierId)
		return false, errMsg
	end

	return success, currentCarrier
end

function Save (carrierId, carrierInfo, ksProv)
	local releaseConn = false
	if not ksProv then
		ksProv = utils.GetDatasource():CassandraConnection("Provisioning")
		releaseConn = true
	end

	-- Clean Up ---
	carrierInfo.isNew = nil

	local success, errMsg = pcall(PM.UpdateCarrier, ksProv, carrierId, carrierInfo)
	if not success then
		LogWarning("[provisioning:models.carriers.Save] Error saving CARRIER '%s' -> '%s'", carrierId, errMsg)
	end

	if releaseConn then
		ksProv:Close()
	end

	return success, errMsg
end

function Delete (carrierId, ksProv)
	local releaseConn = false
	if not ksProv then
		ksProv = utils.GetDatasource():CassandraConnection("Provisioning")
		releaseConn = true
	end

	local success, result = pcall(PM.DeleteCarrier, ksProv, carrierId)
	if not success or not result.success then
		LogWarning("[provisioning:models.carriers.Delete] Error deleting CARRIER '%s' -> '%s'.", carrierId, result.errMsg)
	end

	if releaseConn then
		ksProv:Close()
	end

	return result
end

function ListCarriersWithTrunks (ksProv)
	local releaseConn = false
	if not ksProv then
		ksProv = utils.GetDatasource():CassandraConnection("Provisioning")
		releaseConn = true
	end

	local success, carriersWithTrunks = pcall(PM.ListCarriersWithTrunks, ksProv)
	--LogDebug("[provisioning:models.carriers.ListCarriersWithTrunks]--> success: '%s', carriersWithTrunks: %s", success, json.encode(carriersWithTrunks or {}))

	if not success then
		local errMsg = carriersWithTrunks
		LogWarning("[provisioning:models.carriers.ListCarriersWithTrunks] Error retrieving carriers with trunks -> '%s'", errMsg)
	end

	if releaseConn then
		ksProv:Close()
	end
	carriersWithTrunks = carriersWithTrunks or {}

	return success, carriersWithTrunks
end
