module(..., package.seeall)

local utils = require "shared.apps.utils"
local PM = require("provisioningmanager")
local m_datasource

function GetName ()
	return "DialplanProcessors"
end

function GetDefinition (datasource)
	m_datasource = datasource
	return _M
end

function ListAll (ksProv)
	local releaseConn = false
	if not ksProv then
		ksProv = utils.GetDatasource():CassandraConnection("Provisioning")
		releaseConn = true
	end

	local success, processors = pcall(PM.ListDialplanProcessors, ksProv)
	
	if not success then
		local errMsg = processors
		LogWarning("[provisioning:models.dialplan_processors.ListAll] Error retrieving processors  -> '%s'", errMsg)
	end

	if releaseConn then
		ksProv:Close()
	end
	processors = processors or {}

	for id, processor in pairs(processors) do
		local processorId = utils.GetId(id)
		processor.Id = processorId
	end

	return success, processors
end

function Get (processorId, ksProv)
	LogDebug("[provisioning:models.dialplan_processors.Get] Retrieving processor: '%s'.", processorId)
	local releaseConn = false
	if not ksProv then
		ksProv = utils.GetDatasource():CassandraConnection("Provisioning")
		releaseConn = true
	end

	local success, currentProcessor = pcall(PM.GetDialplanProcessor, ksProv, processorId)
	
	if releaseConn then
		ksProv:Close()
	end

	if not success or utils.TableIsEmpty(currentProcessor) then
		local errMsg = "DIALPLAN PROCESSOR DOES NOT EXIST"
		return false, errMsg
	end

	currentProcessor.Id = currentProcessor.Id or processorId

	return success, currentProcessor
end

function Save (processorId, processorInfo, ksProv)
	local releaseConn = false
	if not ksProv then
		ksProv = utils.GetDatasource():CassandraConnection( "Provisioning" )
		releaseConn = true
	end

	local success, err = PM.UpdateDialplanProcessor(ksProv, processorId, processorInfo)
	if not success then
		LogWarning("[provisioning:models.dialplan_processors.Save] Error saving processor '%s'. --> '%s'", 
			processorId, json.encode(err))
	end

	if releaseConn then
		ksProv:Close()
	end

	return success, err
end

function Delete (processorId, ksProv)
	local releaseConn = false
	if not ksProv then
		ksProv = utils.GetDatasource():CassandraConnection( "Provisioning" )
		releaseConn = true
	end
	local success, result = pcall(PM.DeleteDialplanProcessor, ksProv, processorId)
	if not success or not result.success then
		LogWarning("[provisioning:models.dialplan_processors.Delete] Error deleting processor '%s' --> '%s'", 
			processorId, result.errMsg)
	end

	if releaseConn then
		ksProv:Close()
	end

	return result
end

function GetTrace (processorId, ksProv)
	local releaseConn = false
	if not ksProv then
		ksProv = utils.GetDatasource():CassandraConnection( "Provisioning" )
		releaseConn = true
	end
	local success, result = pcall(PM.GetDialplanProcessorTrace, ksProv, processorId)
	if not success then
		LogWarning("[provisioning:models.dialplan_processors.GetTrace] Error getting processor '%s' trace --> '%s'", 
			processorId, result.errMsg)
	end

	if releaseConn then
		ksProv:Close()
	end

	return success, result
end
