module(..., package.seeall)

local utils = require "shared.apps.utils"
local PM = require("provisioningmanager")
local m_datasource

function GetName ()
	return "Hosts"
end

function GetDefinition (datasource)
	m_datasource = datasource
	PM.SetRedis(datasource.redis)
	return _M
end

function ListAll (ksProv)
	local releaseConn = false
	if not ksProv then
		ksProv = utils.GetDatasource():CassandraConnection("Provisioning")
		releaseConn = true
	end

	local success, hosts = pcall(PM.ListHosts, ksProv)
	hosts = hosts or {}
	if releaseConn then
		ksProv:Close()
	end
	-- sanitize old values
	for id, hostInfo in pairs(hosts) do
		hostInfo.hostId = (hostInfo.hostId or ""):lower()
	end
	--LogDebug("[provisioning:models.hosts.ListAll]--> success: '%s', Hosts: %s", success, json.encode(hosts))

	if not success then
		local errMsg = hosts
		LogWarning("[provisioning:models.hosts.ListAll] Error retrieving hosts  -> '%s'", errMsg)
	end

	return success, hosts
end

function Get (hostId, ksProv)
	--LogDebug("[provisioning:models.hosts.Get] Retrieving HOSTS: '%s'.", hostId)

	local releaseConn = false
	if not ksProv then
		ksProv = utils.GetDatasource():CassandraConnection("Provisioning")
		releaseConn = true
	end

	local success, currentHost = pcall(PM.GetHost, ksProv, hostId)
	--LogDebug("[provisioning:models.hosts.Get] Retrieved HOSTS '%s' -> hostId: '%s', currentHost: '%s'.", hostId, success, json.encode(currentHost or {}))

	if not success then
		local errMsg = currentHost
		LogWarning("[provisioning:models.hosts.Get] Error retrieving HOSTS '%s' -> '%s'", hostId, errMsg)
	end

	if releaseConn then
		ksProv:Close()
	end

	if utils.TableIsEmpty(currentHost) then
		success = false
		local errMsg = "HOSTS DOES NOT EXIST"
		--LogWarning("[provisioning:models.hosts.Get] Error retrieving HOSTS '%s'. HOSTS does not exist.'", hostId)
		return false, errMsg
	end

	return success, currentHost
end

function Save (hostId, hostInfo, ksProv)
	local releaseConn = false
	if not ksProv then
		ksProv = utils.GetDatasource():CassandraConnection("Provisioning")
		releaseConn = true
	end

	-- Clean Up ---
	hostInfo.isNew = nil

	local success, errMsg = PM.UpdateHost(ksProv, hostId, hostInfo)
	if not success then
		LogWarning("[provisioning:models.hosts.Save] Error saving HOSTS '%s' -> '%s'", hostId, errMsg)
	end

	if releaseConn then
		ksProv:Close()
	end

	return success, errMsg
end

function Delete (hostId, ksProv)
	local releaseConn = false
	if not ksProv then
		ksProv = utils.GetDatasource():CassandraConnection("Provisioning")
		releaseConn = true
	end

	local success, result = pcall(PM.DeleteHost, ksProv, hostId)
	if not success or not result.success then
		LogWarning("[provisioning:models.hosts.Delete] Error deleting HOSTS '%s' -> '%s'.", hostId, json.encode(result))
	end

	if releaseConn then
		ksProv:Close()
	end

	return result
end
