module(..., package.seeall)

local utils = require "shared.apps.utils"
local PM = require("provisioningmanager")
local m_datasource

function GetName ()
	return "IceServers"
end

function GetDefinition (datasource)
	m_datasource = datasource
	PM.SetRedis(datasource.redis)
	return _M
end

function ListAll (ksProv)
	local releaseConn = false
	
	if not ksProv then
		ksProv = utils.GetDatasource():CassandraConnection("Provisioning")
		releaseConn = true
	end

	local success, result = pcall(PM.GetIceServersCol, ksProv)

	if releaseConn then
		ksProv:Close()
	end

	return success, result
end

function Get (serverId, ksProv)
	local releaseConn = false
	
	if not ksProv then
		ksProv = utils.GetDatasource():CassandraConnection("Provisioning")
		releaseConn = true
	end

	local success, result = pcall(PM.GetIceServer, serverId, ksProv)

	if releaseConn then
		ksProv:Close()
	end

	return success, result
end

function Save (serverInfo, ksProv)
	local releaseConn   = false
	local iceServerData = { 
			serverId = serverInfo.id and tonumber(serverInfo.id) or os.time(os.date("!*t")),
			user     = serverInfo.user,
			password = serverInfo.password,
			domain   = serverInfo.domain,
			port     = serverInfo.port
	}
	
	if not ksProv then
		ksProv = utils.GetDatasource():CassandraConnection("Provisioning")
		releaseConn = true
	end

	local success, result = pcall(PM.SaveIceServer, iceServerData, ksProv)

	if releaseConn then
		ksProv:Close()
	end

	return success, result
end

function Delete (serverId, ksProv)
	local releaseConn = false

	if not ksProv then
		ksProv = utils.GetDatasource():CassandraConnection("Provisioning")
		releaseConn = true
	end

	local success, result = pcall(PM.DeleteIceServer, serverId, ksProv)

	if releaseConn then
		ksProv:Close()
	end

	return success, result
end

function GetAllIceServers(ksProv)
	local releaseConn = false
	local servers     = {}
	local simple      = nil

	if not ksProv then
		ksProv = utils.GetDatasource():CassandraConnection("Provisioning")
		releaseConn = true
	end

	local success, iceConfigs = pcall(PM.GetIceServersCol, ksProv)

	if releaseConn then
		ksProv:Close()
	end

	if success
	then
		if next(iceConfigs) ~= nil
		then
			if iceConfigs.userConfigs and #iceConfigs.userConfigs > 0
			then
				for _, iceData in ipairs (iceConfigs.userConfigs) do
					if iceData.domain and iceData.port
					then
						local domPort  = iceData.domain .. ":" .. tostring(iceData.port)

						if not simple or simple ~= domPort then
							table.insert(servers, { urls = "stun:" .. domPort })
						end

						simple = not simple and domPort or simple

						if iceData.user and iceData.password
						then
							table.insert(servers, { urls = "turn:" .. domPort                    , username = iceData.user, credential = iceData.password })
							table.insert(servers, { urls = "turn:" .. domPort .. "?transport=tcp", username = iceData.user, credential = iceData.password })
						end
					end
				end
			end

			if iceConfigs.default then
				table.insert(servers, iceConfigs.default)
			else
				table.insert(servers, { urls = "stun:stun.l.google.com:19302"} )
			end
		end
	else
		LogError("[provisioning:models.ice_servers.GetAllIceServers] Error retrieving ice servers  -> '%s'", iceConfigs)
		table.insert(servers, { urls = "stun:stun.l.google.com:19302"} )
	end

	return servers
end
