module(..., package.seeall)

local utils = require "shared.apps.utils"
local PM = require("provisioningmanager")
local m_datasource

function GetName ()
	return "Packages"
end

function GetDefinition (datasource)
	m_datasource = datasource
	return _M
end

function ListAll (ksProv)
	local releaseConn = false
	if not ksProv then
		ksProv = utils.GetDatasource():CassandraConnection("Provisioning")
		releaseConn = true
	end

	local success, packages = pcall(PM.ListPackages, ksProv)
	packages = packages or {}
	--LogDebug("[provisioning:models.packages.ListAll]--> success: '%s', Packages: %s", success, json.encode(packages))
	if releaseConn then
		ksProv:Close()
	end

	if not success then
		local errMsg = packages
		LogWarning("[provisioning:models.packages.ListAll] Error retrieving packages  -> '%s'", errMsg)
	end

	return success, packages
end

function GetFreeVCCPackages (ksProv)
	local releaseConn = false
	if not ksProv then
		ksProv = utils.GetDatasource():CassandraConnection("Provisioning")
		releaseConn = true
	end

	local success, freeVccPackages = pcall(PM.GetFreeVCCPackages, ksProv)
	freeVccPackages = freeVccPackages or {}

	--LogDebug("[provisioning:models.packages.GetFreeVCCPackages]--> success: '%s', freeVccPackages: %s", success, json.encode(freeVccPackages))
	if releaseConn then
		ksProv:Close()
	end

	if not success then
		local errMsg = freeVccPackages
		LogWarning("[provisioning:models.packages.GetFreeVCCPackages] Error retrieving freeVccPackages  -> '%s'", errMsg)
	end

	return success, freeVccPackages
end

function GetVCCPackages (currentVcc, ksProv)
	local releaseConn = false
	if not ksProv then
		ksProv = utils.GetDatasource():CassandraConnection("Provisioning")
		releaseConn = true
	end

	local success, vccPackages = pcall(PM.GetVCCPackages, ksProv, currentVcc)
	vccPackages = vccPackages or {}

	--LogDebug("[provisioning:models.packages.GetVCCPackages]--> success: '%s', Vcc: '%s' vccPackages: %s", success, currentVcc, json.encode(vccPackages))
	if releaseConn then
		ksProv:Close()
	end

	if not success then
		local errMsg = vccPackages
		LogWarning("[provisioning:models.packages.GetVCCPackages] Error retrieving Vcc: '%s' vccPackages  -> '%s'", currentVcc, errMsg)
	end

	return success, vccPackages
end

function Get (packageId, ksProv)
	LogDebug("[provisioning:models.packages.Get] Retrieving PACKAGE: '%s'.", packageId)

	local releaseConn = false
	if not ksProv then
		ksProv = utils.GetDatasource():CassandraConnection("Provisioning")
		releaseConn = true
	end

	local success, currentPackage = pcall(PM.GetPackage, ksProv, packageId)
	if releaseConn then
		ksProv:Close()
	end

	if not success or utils.TableIsEmpty(currentPackage) then
		local errMsg = "PACKAGE DOES NOT EXIST"
		--LogWarning("[provisioning:models.packages.Get] Error retrieving PACKAGE '%s'. PACKAGE does not exist.'", packageId)
		return false, errMsg
	end

	currentPackage.Id = currentPackage.Id or packageId

	return true, currentPackage
end

function Save (packageId, packageInfo, ksProv)
	local releaseConn = false
	if not ksProv then
		ksProv = utils.GetDatasource():CassandraConnection("Provisioning")
		releaseConn = true
	end

	-- Clean Up ---
	packageInfo.isNew = nil

	local success, err = PM.UpdatePackage(ksProv, packageId, packageInfo)
	if not success then
		LogWarning("[provisioning:models.packages.Save] Error saving PACKAGE '%s'. --> '%s'", packageId, json.encode(err or {}))
	end

	if releaseConn then
		ksProv:Close()
	end

	return success, err
end

function Delete (packageId, ksProv)
	local releaseConn = false
	if not ksProv then
		ksProv = utils.GetDatasource():CassandraConnection("Provisioning")
		releaseConn = true
	end
	local success, result = pcall(PM.DeletePackage, ksProv, packageId)
	if not success or not result.success then
		LogWarning("[provisioning:models.packages.Delete] Error deleting packages '%s' --> '%s'", packageId, result.errMsg)
	end

	if releaseConn then
		ksProv:Close()
	end

	return result
end
