module(..., package.seeall)

local utils = require "shared.apps.utils"
local m_datasource

function GetName ()
	return "SystemMonitoring"
end

function GetDefinition (datasource)
	m_datasource = datasource
	return _M
end

local function GetOutboundEngineRedisClient()
	local provisioningModel = require "apps.provisioning.model"
	local oeRedis = provisioningModel:GetOutboundEngineRedisClient()
	return oeRedis
end

function GetVccCampaigns (vcc, keyspace)
	return utils.GetAllColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "VCCCampaigns",
		key = vcc
	} or {}
end

function GetOutboundProcessBatches (processId, ksOE)
	local batches = utils.GetAllColumns {
		keyspace = ksOE or "OutboundEngine",
		columnFamily = "OutboundProcessBatches",
		superColumns = true,
		key = processId,
		columns = {}
	}
	return batches or {}
end

function GetOutboundProcessBatch (processId, batchId, ksOE)
	local batchInfo = utils.GetAllColumns {
		keyspace = ksOE or "OutboundEngine",
		columnFamily = "OutboundProcessBatches",
		superColumns = true,
		key = processId,
		columns = {batchId}
	} or {}
	return batchInfo[batchId]
end

function GetRedisIndicators(scope, section, slice)
	local entriesKey = string.format("Entries%s::%s::By%s", scope, section, slice)
	local redis = GetOutboundEngineRedisClient()
	local keys = redis:smembers(entriesKey) or {}
	local results = {}
	for _, key in pairs(keys) do
		table.insert(results, {
			key = key,
			values = redis:hgetall(key) or {}
		})
	end
	return results
end
