module(..., package.seeall)

local utils = require "shared.apps.utils"
local PM = require("provisioningmanager")
local m_datasource

function GetName ()
	return "Trunks"
end

function GetDefinition (datasource)
	m_datasource = datasource
	return _M
end

function GetDefaultTech ()
	return "SIP"
end

function ListAll (ksProv)
	local releaseConn = false
	if not ksProv then
		ksProv = utils.GetDatasource():CassandraConnection("Provisioning")
		releaseConn = true
	end

	local success, trunks = pcall(PM.ListTrunks, ksProv)
	--LogDebug("[provisioning:models.trunks.ListAll]--> success: '%s', Trunks: %s", success, json.encode(trunks or {}))

	if not success then
		local errMsg = trunks
		LogWarning("[provisioning:models.trunks.ListAll] Error retrieving trunks  -> '%s'", errMsg)
	end

	if releaseConn then
		ksProv:Close()
	end
	trunks = trunks or {}

	return success, trunks
end

function ListTrunksByCarrier (ksProv)
	local releaseConn = false
	if not ksProv then
		ksProv = utils.GetDatasource():CassandraConnection("Provisioning")
		releaseConn = true
	end

	local success, trunksByCarriers = pcall(PM.ListTrunksByCarrier, ksProv)
	--LogDebug("[provisioning:models.trunks.ListTrunksByCarrier]--> success: '%s', trunksByCarriers: %s", success, json.encode(trunksByCarriers or {}))

	if not success then
		local errMsg = trunksByCarriers
		LogWarning("[provisioning:models.trunks.ListTrunksByCarrier] Error retrieving trunks by carriers  -> '%s'", errMsg)
	end

	if releaseConn then
		ksProv:Close()
	end
	trunksByCarriers = trunksByCarriers or {}

	return success, trunksByCarriers
end

function Get (trunkId, ksProv)
	LogDebug("[provisioning:models.trunks.Get] Retrieving TRUNK: '%s'.", trunkId)

	local releaseConn = false
	if not ksProv then
		ksProv = utils.GetDatasource():CassandraConnection("Provisioning")
		releaseConn = true
	end

	local success, currentTrunk = pcall(PM.GetTrunk, ksProv, trunkId)
	LogDebug("[provisioning:models.trunks.Get] Retrieved TRUNK '%s' -> trunkId: '%s', currentTrunk: '%s'.", trunkId, success, json.encode(currentTrunk or {}))
	if releaseConn then
		ksProv:Close()
	end

	if not success or utils.TableIsEmpty(currentTrunk) then
		local errMsg = "TRUNK DOES NOT EXIST"
		--LogWarning("[provisioning:models.trunks.Get] Error retrieving TRUNK '%s'. TRUNK does not exist.'", trunkId)
		return false, errMsg
	end

	currentTrunk.Id = currentTrunk.Id or trunkId

	return true, currentTrunk
end

function Save (trunkId, trunkInfo, ksProv)
	local releaseConn = false
	if not ksProv then
		ksProv = utils.GetDatasource():CassandraConnection( "Provisioning" )
		releaseConn = true
	end
	trunkInfo.tech = trunkInfo.tech or GetDefaultTech()

	-- Clean Up ---
	trunkInfo.isNew = nil

	local success, err = PM.UpdateTrunk(ksProv, trunkId, trunkInfo)
	if not success then
		LogWarning("[provisioning:models.trunks.Save] Error saving trunks '%s'. --> '%s'", trunkId, json.encode(err))
	end

	if releaseConn then
		ksProv:Close()
	end

	return success, err
end

function Delete (trunkId, ksProv)
	local releaseConn = false
	if not ksProv then
		ksProv = utils.GetDatasource():CassandraConnection( "Provisioning" )
		releaseConn = true
	end

	local success, result = pcall(PM.DeleteTrunk, ksProv, trunkId)
	if not success or not result.success then
		LogWarning("[provisioning:models.trunks.Delete] Error deleting trunks '%s'. --> '%s'", trunkId, result.errMsg)
	end

	if releaseConn then
		ksProv:Close()
	end

	return result
end

local MAX_AMD_RESULTS = 10

function StartAmdResult(trunkId, data)
	local redis = m_datasource.redis
	local key = string.format("TrunkAmdResults::%s", trunkId)
	local count = redis:zcard(key) or 0
	if count >= MAX_AMD_RESULTS then
		redis:zremrangebyrank(key, 0, count - MAX_AMD_RESULTS)
	end
	redis:zadd(key, data.timestamp, json.encode(data))
end

function GetAmdResults(trunkId)
	local redis = m_datasource.redis
	local key = string.format("TrunkAmdResults::%s", trunkId)
	local entries = redis:zrevrange(key, 0, -1) or {}
	local list = {}
	for _, entry in pairs(entries) do
		table.insert(list, json.decode(entry))
	end
	return list
end
