module(..., package.seeall)

local utils = require "shared.apps.utils"
local PM = require("provisioningmanager")
local m_datasource

function GetName ()
	return "TrunksGroups"
end

function GetDefinition (datasource)
	m_datasource = datasource
	return _M
end

function ListAll (ksProv)
	local releaseConn = false
	if not ksProv then
		ksProv = utils.GetDatasource():CassandraConnection("Provisioning")
		releaseConn = true
	end

	local success, trunksgroups = pcall(PM.ListTrunkGroups, ksProv)
	--LogDebug("[provisioning:models.trunksgroups.ListAll]--> success: '%s', Trunks: %s", success, json.encode(trunksgroups or {}))

	if not success then
		local errMsg = trunksgroups
		LogWarning("[provisioning:models.trunksgroups.ListAll] Error retrieving trunksgroups  -> '%s'", errMsg)
	end

	if releaseConn then
		ksProv:Close()
	end
	trunksgroups = trunksgroups or {}

	return success, trunksgroups
end

function Get (trunkGroupId, ksProv)
	LogDebug("[provisioning:models.trunksgroups.Get] Retrieving TRUNK: '%s'", trunkGroupId)
	local releaseConn = false
	if not ksProv then
		ksProv = utils.GetDatasource():CassandraConnection("Provisioning")
		releaseConn = true
	end

	local success, currentTrunkGroup = pcall(PM.GetTrunkGroup, ksProv, trunkGroupId)
--	LogDebug("[provisioning:models.trunksgroups.Get] Retrieved TRUNK '%s' -> success: '%s', currentTrunkGroup: '%s'.", trunkGroupId, success, json.encode(currentTrunkGroup or {}))

	if releaseConn then
		ksProv:Close()
	end

	if not success or utils.TableIsEmpty(currentTrunkGroup) then
		local errMsg = "TRUNK GROUP DOES NOT EXIST"
		--LogWarning("[provisioning:models.trunksgroups.Get] Error retrieving TRUNK '%s'. TRUNK does not exist.'", trunkGroupId)
		return false, errMsg
	end

	currentTrunkGroup.Id = currentTrunkGroup.Id or trunkGroupId

	return success, currentTrunkGroup
end

function Save (trunkGroupId, trunkGroupInfo, ksProv)
	local releaseConn = false
	if not ksProv then
		ksProv = utils.GetDatasource():CassandraConnection( "Provisioning" )
		releaseConn = true
	end

	local currentTrunkGroup
	local success, trunkGroup = Get(trunkGroupId, ksProv)
	if success then
		currentTrunkGroup = trunkGroup
	end

	local trunkList = trunkGroupInfo.trunkList
	-- Clean Up ---
	trunkGroupInfo.isNew = nil
	trunkGroupInfo.trunkList = nil

	-- 1 - Almaceno el TrunkGroup
	local success, err = PM.UpdateTrunkGroup(ksProv, trunkGroupId, trunkGroupInfo)
	if not success then
		LogWarning("[provisioning:models.trunksgroups.Save] Error saving trunksgroups '%s'. --> '%s'", trunkGroupId, json.encode(err))
	end

	-- 2 - Si estoy modificando, primero elimino los trunk relacionados
	if currentTrunkGroup then
		for trunkId in pairs(currentTrunkGroup.trunkList or {}) do
			--LogDebug("[provisioning:models.trunksgroups.Save] -REMOVING trunk of '%s' trunksgroups '%s'.", trunkId, trunkGroupId)
			local success, err = PM.RemoveTrunkFromGroup(ksProv, trunkId)
			if not success then
				LogWarning("[provisioning:models.trunksgroups.Save] Error removing trunk of '%s' trunksgroups '%s'. --> '%s'", trunkId, trunkGroupId, json.encode(err))
			end
		end
	end

	-- 3 - Si llegan trunkList a relacionar, tengo que actualizarlos
	for _, trunk in ipairs(trunkList or {}) do
		local trunkId = trunk.TrunkId
		--LogDebug("[provisioning:models.trunksgroups.Save] +ADDING trunk of '%s' trunksgroups '%s'.", trunkId, trunkGroupId)
		local success, err = PM.AddTrunkToGroup(ksProv, trunkId, trunkGroupId)
		if not success then
			LogWarning("[provisioning:models.trunksgroups.Save] Error adding trunk of '%s' trunksgroups '%s'. --> '%s'", trunkId, trunkGroupId, json.encode(err))
		end
	end

	if releaseConn then
		ksProv:Close()
	end

	return success, err
end

function Delete (trunkGroupId, ksProv)
	local releaseConn = false
	if not ksProv then
		ksProv = utils.GetDatasource():CassandraConnection( "Provisioning" )
		releaseConn = true
	end

	local success, result = pcall(PM.DeleteTrunkGroup, ksProv, trunkGroupId)
	if not success or not result.success then
		LogWarning("[provisioning:models.trunksgroups.Delete] Error deleting trunksgroups '%s', --> '%s'", trunkGroupId, result.errMsg)
	end

	if releaseConn then
		ksProv:Close()
	end

	return result
end
