/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, esversion: 9 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

//	const FORMAT = inConcert.Utils.format;

	//---------------------------------------------------------------------
	// Codecs Management
	function ProvisioningCodecsMng(params)
	{
		if (!params) {
			throw("ProvisioningCodecsMng - 'params' is required");
		}
		if (!params.owner) {
			throw("ProvisioningCodecsMng - 'params.owner' is required");
		}
		if (!params.prefix) {
			throw("ProvisioningCodecsMng - 'params.prefix' is required");
		}
		if (!params.container_id) {
			throw("ProvisioningCodecsMng - 'params.container_id' is required");
		}
		if (!params.codecs_list) {
			throw("ProvisioningCodecsMng - 'params.codecs_list' is required");
		}
		this.owner = params.owner;
		this.owner_name = params.owner_name || "";
		this.prefix = params.prefix + "_";
		this.container_id = params.container_id;
		this.codecs_list = params.codecs_list;
		this.fields_per_row = params.fields_per_row || 2;
		this.codecs_list_sep = params.codecs_list_sep || ",";

		return true;
	}

	if (typeof inConcert.ProvisioningCodecsMng !== "function") {
		inConcert.ProvisioningCodecsMng = ProvisioningCodecsMng;
	}

	ProvisioningCodecsMng.prototype.initialize = function()
	{
		this.__make_popup_html();
	};

	ProvisioningCodecsMng.prototype.__make_popup_html = function()
	{
		let html = '' +
			'<div class="card-header">' +
				'<h5><%=_T([[Codecs Config]])%></h5>' +
			'</div>' +
			this.__make_codecs_html();

		const $container = $("#" + this.container_id);
		$container.html(html);
	};

	ProvisioningCodecsMng.prototype.__make_codecs_html = function()
	{
		let html_rows = "";
		let html_fields = "";
		let count_fields = 0;
		let is_added_fields_in_row = false;

		this.codecs_list.forEach(codec_info => {
			let ctrl_id = this.prefix + codec_info.id;
			html_fields = html_fields +
				'<div class="col">' +
					'<div class="form-group">' +
						'<div class="form-check">' +
							'<label for="' + ctrl_id + '" class="form-check-label">' +
								'<input id="' + ctrl_id + '" class="form-check-input" type="checkbox" codec_val="' + codec_info.id + '" />' +
								codec_info.name +
							'</label>' +
						'</div>' +
					'</div>' +
				'</div>';

			count_fields++;
			if (count_fields === this.fields_per_row) {
				html_rows = html_rows +
					'<div class="row">' +
						html_fields +
					'</div>';
				html_fields = "";
				is_added_fields_in_row = true;
				count_fields = 0;
			}
			else {
				is_added_fields_in_row = false;
			}
		});

		if (!is_added_fields_in_row) {
			html_rows = html_rows +
				'<div class="row">' +
					html_fields +
				'</div>';
		}
	
		return html_rows;
	};

	ProvisioningCodecsMng.prototype.dispose = function()
	{
		//TODO: implementar, si es necesario
	};

	ProvisioningCodecsMng.prototype.get = function(entity)
	{
		var self = this;
		
		if (!entity) {
			return;
		}
		const $container = $("#" + this.container_id);
		const $codecs = $container.find("[codec_val]");

		let codecs = "";
		$.each($codecs, function(_, codec) {
			const $codec = $(codec);
			if ($codec.prop("checked")) {
				if (codecs.length > 0) {
					codecs = codecs + self.codecs_list_sep;
				}
				let codec = $codec.attr("codec_val");
				codecs = codecs + codec;
			}
		});

		if (codecs.length > 0) {
			entity.codecs = codecs;
		}
	};

	ProvisioningCodecsMng.prototype.set = function(entity)
	{
		if (!entity) {
			return;
		}
		if ((entity.codecs || "").length === 0) {
			return;
		}

		const $container = $("#" + this.container_id);
		$container.find("[codec_val]").prop("checked", false);

		const mark_codec = function(codec) {
			let selector = "[codec_val='" + codec + "']";
			const $codec = $container.find(selector);
			if ($codec.length > 0) {
				$codec.prop("checked", true);
			}
		};
		let codecs = entity.codecs.toLowerCase().split(this.codecs_list_sep);
		if ($.isArray(codecs)) {
			$.each(codecs, function(_, codec) {
				mark_codec(codec);
			});
		}
	};

	ProvisioningCodecsMng.prototype.empty_controls = function()
	{
		const $container = $("#" + this.container_id);
		$container.find("[codec_val]").prop("checked", false);
	};
}());
