/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, esversion: 9 */

/* global $, inConcert, ace */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	const FORMAT = inConcert.Utils.format;

	//---------------------------------------------------------------------
	// Diaplan Management
	function ProvisioningDialplanProcessorMng(params)
	{
		if (!params) {
			throw("ProvisioningDialplanProcessorMng - 'params' is required");
		}
		if (!params.owner) {
			throw("ProvisioningDialplanProcessorMng - 'params.owner' is required");
		}
		if (!params.prefix) {
			throw("ProvisioningDialplanProcessorMng - 'params.prefix' is required");
		}
		if (!params.container_id) {
			throw("ProvisioningDialplanProcessorMng - 'params.container_id' is required");
		}

		this.owner = params.owner;
		this.owner_name = params.owner_name || "";
		this.prefix = params.prefix + "_";
		this.container_id = params.container_id;

		return true;
	}

	if (typeof inConcert.ProvisioningDialplanProcessorMng !== "function") {
		inConcert.ProvisioningDialplanProcessorMng = ProvisioningDialplanProcessorMng;
	}

	ProvisioningDialplanProcessorMng.prototype.initialize = function()
	{
		this.__make_code_html();
	};

	ProvisioningDialplanProcessorMng.prototype.get_dom_id = function()
	{
		return this.prefix + "code";
	};

	ProvisioningDialplanProcessorMng.prototype.__make_editor_html = function()
	{
		const self = this;

		let $editor = $(FORMAT("".concat(
			"<div class='dialplanCodeBlockContainer'>",
				"<div id='{0}code' class='dialplanCodeBlock'></div>",
				"<div id='{0}trace' class='dialplanTraceContainer'>",
					"<div class='card-block'>",
						"<div class='row'>",
							"<div class='col-3'>",
								"<div class='form-group'>",
									"<label>",
										"<%=_T('Select a conversation')%><%=_T(':')%>",
									"</label>",
									"<select package class='form-control'></select>",
								"</div>",
							"</div>",
							"<div class='col-3'>",
								"<div class='form-group'>",
									"<label></label>",
									"<select call_type class='form-control'></select>",
								"</div>",
							"</div>",
							"<div class='col-6'>",
								"<div class='form-group'>",
									"<label></label>",
									"<select conversation class='form-control'></select>",
								"</div>",
							"</div>",
						"</div>",
						"<div class='row'>",
							"<div class='col-3'>",
								"<div class='form-group'>",
									"<label class=''>",
										"<%=_T('Filter Results By')%><%=_T(':')%>",
									"</label>",
									"<select log_level class='form-control'></select>",
								"</div>",
							"</div>",
							"<div class='col-9'>",
								"<div class='form-group description-form-group'>",
									"<input content type='text' class='form-control' placeholder='Ingrese la descripción…'>",
									"<button refresh class='buttonToggle buttonColorWhite dialplanTraceRefresh'>",
										"<i class='far fa-sync'></i>",
									"</button>",
									"<button download class='buttonToggle buttonColorWhite dialplanTraceDownload'>",
										"<i class='far fa-arrow-to-bottom'></i>",
									"</button>",
								"</div>",
							"</div>",
						"</div>",
						"<div list class='dialplanTraceList'>",
						"</div>",
					"</div>",
				"</div>",
				"<div class='dialplanShowTraceContainer'>",
					"<button show_trace class='pageButtonGeneric buttonColorWhite dialplanShowTraceButton'>",
						"<%=_T('Execution Trace')%>",
					"</button>",
				"</div>",
			"</div>"
		), self.prefix));

		self.codeEditor = ace.edit($editor.find("#" + self.get_dom_id())[0]);
		self.codeEditor.getSession().setMode(FORMAT("ace{0}mode{0}lua", "/"));

		$editor.find("[show_trace]").click(function() {
			$("#" + self.container_id).toggleClass("traceOpen");
			self.codeEditor.resize();
		});

		this.owner.fill_combo($editor.find("[log_level]"), false, ["DEBUG", "INFO", "WARNING", "ERROR"]);

		let types = inConcert.app.getProvisioningAPI().getCallTypes();
		types.unshift({ id : "", name : "<%=_T('All types')%>"});
		this.owner.fill_combo($editor.find("[call_type]"), false, types, "id", "name");

		$editor.find("[log_level]").change(function() {
			self.__fillTrace();
		});
		$editor.find("[content]")
			.on("paste", function() {
				self.__fillTrace();
			})
			.on("keyup", function() {
				self.__fillTrace();
			});

		$editor.find("[package], [call_type], [conversation]").change(function() {
			self.traceFn(function(_, trace) {
				self.__fillTrace(trace);
			});
		});
		$editor.find("[refresh]").click(function() {
			self.traceFn(function(_, trace) {
				self.__fillTrace(trace);
			});
		});

		$editor.find("[download]").click(function() {
			self.__downloadTrace();
		});

		return $editor;
	};

	ProvisioningDialplanProcessorMng.prototype.__make_toolbar_html = function()
	{
		const self = this;
		
		let $toolbar = $(FORMAT("".concat(
			"<div class='dialplanToolbarContainer'>",
				"<div class='dialplanToolbarTitleContainer'>",
					"<button toolbar_collapse class='buttonToggle dialplanToolbarCollapseButton'>",
					"</button>",
					"<label class='dialplanToolbarTitle'>",
						"<%=_T('Helpers')%>",
					"</label>",
				"</div>",
				"<ul id='{0}toolbar_list' class='dialplanToolbarList'",
				"</ul>",
			"</div>"
		), self.prefix));

		$toolbar.find("[toolbar_collapse]").click(function() {
			$("#" + self.container_id).toggleClass("toolbarColapsed");
		});

		return $toolbar;
	};

	ProvisioningDialplanProcessorMng.prototype.__make_code_html = function()
	{
		let $content = [
			this.__make_toolbar_html(),
			this.__make_editor_html()
		];
		let $container = $("#" + this.container_id);
		$container.html("");
		$container.append($content);
	};

	ProvisioningDialplanProcessorMng.prototype.set_events = function()
	{
		
	};

	ProvisioningDialplanProcessorMng.prototype.load_data = function(processorId)
	{
		const self = this;
		const provisioning_api = inConcert.app.getProvisioningAPI();
		
		self.traceFn = function(callback) {
			if (processorId) {
				return provisioning_api.dialplanProcessorsGetTrace(processorId, callback);
			}
			else {
				callback(true, {});
			}
		};
		return [
			{
				request_function: function(success) {
					return provisioning_api.dialplanProcessorsGetHelpers(success);
				},
				success: function(helpers) {
					self.__fillHelpers(helpers);
				}
			},
			{
				request_function: function(success) {
					self.traceFn(success);
				},
				success: function(trace) {
					self.__fillTrace(trace);
				}
			}
		];
	};

	ProvisioningDialplanProcessorMng.prototype.__fillHelpers = function(helpers)
	{
		const self = this;
		let $list = helpers.map(function(helper) {
			return self.__makeHelper(helper);
		});
		$("#" + self.prefix + "toolbar_list").append($list);
	};

	ProvisioningDialplanProcessorMng.prototype.__makeHelper = function(helper)
	{
		const self = this;

		let $helper = $(FORMAT("".concat(
			"<li helper_element class='dialplanToolbarElement'>",
				"<div element_title element_selector='true' class='dialplanToolbarElementTitle'>",
					"<i element_selector='true' class='dialplanToolbarElementIcon'></i>",
					"{0}",
					'<button view_element class="dialplanToolbarElementViewButton buttonToggle" type="button">',
						"<i class='far fa-eye'></i>",
					'</button>',
				"</div>",
				"<ul methods_list class='dialplanToolbarElementMethodsList'>",
				"</ul>",
			"</li>"
		), helper.name));

		$helper.attr("title", helper.description);
		$helper.data("helper", helper);

		let $methods = $helper.find("[methods_list]");
		if (helper.methods.length) {
			$methods.append(helper.methods.map(function(method) {
				return self.__makeHelperMethod(method);
			}));
		}
		else {
			$methods.remove();
		}

		$helper.click(function(e) {
			var $target = $(e.target);
			if (!$target.attr("element_selector")) {
				return;
			}
			var $titleDiv = $(this).children("[element_title]");
			var minHeight = $titleDiv.offset().top + $titleDiv.height() + 15;
			if (minHeight < e.pageY) {
				return;
			}

			var selected = $(this).hasClass("selected");
			$("#" + self.prefix + "toolbar_list").find(".selected").removeClass("selected");
			$(this).toggleClass("selected", !selected);
		});
		$helper.find("[view_element]").click(function() {
			self.__viewHelper($(this).closest("[helper_element]").data("helper"));
		});

		return $helper;
	};

	ProvisioningDialplanProcessorMng.prototype.__makeHelperMethod = function(method)
	{
		const self = this;

		let $method = $(FORMAT("".concat(
			"<li helper_element_method class='dialplanToolbarMethod'>",
				"<i class='dialplanToolbarMethodIcon'></i>",
				"{0}",
				'<button snippet_method class="dialplanToolbarMethodSnippetButton buttonToggle" type="button">',
					"<i class='far fa-plus'></i>",
				'</button>',
				'<button view_method class="dialplanToolbarMethodViewButton buttonToggle" type="button">',
					"<i class='far fa-eye'></i>",
				'</button>',
			"</li>"
		), method.name));

		$method.attr("title", self.__makeHelperMethodTitle(method));
		$method.data("method", method);

		$method.find("[view_method]").click(function() {
			self.__viewHelperMethod($(this).closest("[helper_element_method]").data("method"));
		});
		$method.find("[snippet_method]").click(function() {
			self.__addHelperMethodSnippet($(this).closest("[helper_element_method]").data("method"));
		});
		
		return $method;
	};

	ProvisioningDialplanProcessorMng.prototype.__makeHelperMethodTitle = function(method)
	{
		const api = inConcert.app.getProvisioningAPI();
		let title = "";
		if (method.types) {
			title += FORMAT("<%=_T('Required types')%><%=_T(':')%> {0}\n", method.types.reduce(function(label, type) {
				let typeTable = api.getCallType(type);
				return label + (label? ", " : "") + (typeTable && typeTable.name || type);
			}, ""));
		}
		if (method.snippet) {
			title += method.snippet + "\n";
		}
		title += method.description;
		return title;
	};

	ProvisioningDialplanProcessorMng.prototype.__viewHelper = function(helper)
	{
		return this.__openHelperPopup(helper);
	};

	ProvisioningDialplanProcessorMng.prototype.__viewHelperMethod = function(method)
	{
		return this.__openHelperPopup(method);
	};

	ProvisioningDialplanProcessorMng.prototype.__openHelperPopup = function(data)
	{
		const self = this;
		const popup_id = this.prefix + 'trunks_priority_new_popup';
		const copy_id = this.prefix + '_clipboard';

		let $popup = $(FORMAT("".concat(
			"<div id ='{0}' class='internalPopUpContainer dialplanToolbarViewHelperPopUp' hidden>",
				"<div class='internalPopUpOverlay'></div>",
				"<div class='internalPopUp'>",
					"<div class='internalPopUpTitle'>",
						"<span popup_title_text>{1}</span>",
						"<button close_button class='internalPopUpCloseButton'>",
							"<i class='far fa-times'></i>",
						"</button>",
					"</div>",
					"<div class='internalPopUpMiddle'>",
						"<div class='row'>",
							"<div class='col-12'>",
								"<div class='inlineMessagesContainer messageInfo'>",
									"{2}",
								"</div>",
							"</div>",
						"</div>",
						"<div helper_examples_container class='row'>",
							"<div class='col-12'>",
								"<div class='form-group'>",
									"<div helper_examples class='dialplanToolbarViewHelperExamples'></div>",
								"</div>",
							"</div>",
						"</div>",
					"</div>",
					"<div class='internalPopUpButtons internal'>",
						"<button close_button class='pageButtonGeneric buttonColorWhite float-right'>",
							"<%=_T([[Close]])%>",
						"</button>",
						"<button id='", copy_id, "' class='pageButtonGeneric buttonColorBlue float-right'>",
							"<%=_T([[Copy]])%>",
						"</button>",
					"</div>",
				"</div>",
			"</div>"
		), popup_id, data.snippet || data.name, data.description));

		data.examples = (data.examples || "").trim();
		if (data.examples) {
			this.examplesEditor = ace.edit($popup.find("[helper_examples]")[0]);
			this.examplesEditor.getSession().setMode(FORMAT("ace{0}mode{0}lua", "/"));
			this.examplesEditor.getSession().setValue(data.examples);
			this.examplesEditor.setReadOnly(true);
		}
		else {
			$popup.find("[helper_examples_container]").remove();
		}

		$("#" + popup_id).remove();
		$("body").append($popup);

		this.helperViewPopup = new inConcert.PopupWindow({ownerId: popup_id});
		this.helperViewPopup.show({
			ownerId      : popup_id,
			eventName    : popup_id,
			canReposition: true,
			container    : $popup,
			align        : "LEFT",
			callbacks: {
				onOpen: function() {
					$("#" + popup_id).show();
				},
				onClose: function( onClose ) {
					onClose = onClose || $.noop;
					if (self.helperViewPopup) {
						self.helperViewPopup.hide(function() {
							onClose();
						});
					}
					else {
						onClose();
					}
				}
			}
		});

		$popup.find("[close_button]").click(function() {
			self.helperViewPopup.finalize();
		});

		const $copy = $("#" + copy_id);
		$copy.off("click")
			.on("click", function() {
				let selected_text = self.examplesEditor.getSelectedText();
				if (selected_text.length === 0) {
					// Si no hay un texto seleccionado, tengo que darle el foco al editor y seleccionar todo el texto.
					self.examplesEditor.focus();
					self.examplesEditor.selectAll();
				}
				try {
					let successful = document.execCommand('copy');
					if (!successful) {
						console.warn("1 - error copying dialplan");
					}
					else{
						console.info("success copied dialplan");
					}
				} catch (err) {
					console.error("2 - error copying dialplan");
				}
			}
		);
	};

	ProvisioningDialplanProcessorMng.prototype.__addHelperMethodSnippet = function(method)
	{
		this.codeEditor.getSession().insert(this.codeEditor.getCursorPosition(), 
			method.snippet || (method.examples && method.examples.trim()));
		this.codeEditor.focus();
	};

	ProvisioningDialplanProcessorMng.prototype.__downloadTrace = function() {
		let log = "";
		$(".dialplanTraceLine").each(function() {
			const level = $(this).attr("level");
			const date =  $(this).find(".dialplanTraceLineDate").text() || "";
			const text = $(this).find(".dialplanTraceLineMessage").text() || "";
			log += `[${level}]\t${date}\t${text}\n`;
		});
		const uriContent = `data:application/octet-stream,${encodeURIComponent(log)}`;
		const element = document.createElement("a");
		element.setAttribute("href", uriContent);
		element.setAttribute("download", `${this.processorId || 'new_dialplan'}.log`);
		element.style.display = "none";
		document.body.appendChild(element);
		element.click();
		document.body.removeChild(element);
	};

	ProvisioningDialplanProcessorMng.prototype.__fillTrace = function(trace)
	{
		const self = this;
		const nowDate = new Date().getDate();
		const $trace = $("#" + self.prefix + "trace");
		if (trace) {
			self.__loadTrace($trace, trace, nowDate);
		}
		$trace.find("[list]")
			.html("")
			.append(self.__filterLines($trace).map(function(line) {
				return self.__makeTraceLine(line, nowDate);
			}));
	};

	ProvisioningDialplanProcessorMng.prototype.__loadTrace = function($trace, trace, nowDate)
	{
		const self = this;
		let packages = Object.keys(trace).sort();
		self.trace = packages.reduce(function(result, packageId) {
			$.each(trace[packageId], function(id, conversationTrace_s) {
				let conversationTrace = JSON.parse(conversationTrace_s);
				let conversationDate = new Date(conversationTrace.start_timestamp * 1000);
				let conversationDate_s = inConcert.Utils.getStringFromDate(conversationDate, false, 
					conversationDate.getDate() === nowDate? "full_time" : "full_datetime");
				conversationTrace.package = packageId;
				conversationTrace.id = id;
				conversationTrace.name = FORMAT("{0} ({1})", conversationDate_s, id);
				result[id] = conversationTrace;
			});
			return result;
		}, {});

		packages = packages.map(myPackage => Object({ id : myPackage, name : myPackage }));
		packages.unshift({ id : "", name : "<%=_T('All packages')%>"});
		let $myPackage = $trace.find("[package]");
		let currentPackage = $myPackage.val();
		this.owner.fill_combo($myPackage, false, packages, "id", "name");
		if (currentPackage) {
			$myPackage.val(currentPackage);
		}
		
		let conversations = self.__filterConversations($trace);
		let $conversations = $trace.find("[conversation]");
		let currentConversation = $conversations.val();
		this.owner.fill_combo($conversations, false, conversations, "id", "name");
		if (currentConversation) {
			$conversations.val(currentConversation);
		}
	};

	ProvisioningDialplanProcessorMng.prototype.__filterConversations = function($trace)
	{
		const self = this;
		let myPackage = $trace.find("[package]").val();
		let callType = $trace.find("[call_type]").val();
		return Object.keys(self.trace)
			.filter(function(conversationId) {
				let conversation = self.trace[conversationId];
				return	(!myPackage || conversation.package === myPackage) &&
						(!callType || conversation.type === callType);
			})
			.map(conversationId => self.trace[conversationId])
			.sort(function(a, b) {
				return a.start_timestamp - b.start_timestamp;
			});
	};

	ProvisioningDialplanProcessorMng.prototype.__getLogLevelNumber = function(level)
	{
		switch(level) {
			case "ERROR":
				return 3;
			case "WARNING":
				return 2;
			case "INFO":
				return 1;
			default:
				return 0;
		}
	};

	ProvisioningDialplanProcessorMng.prototype.__filterLines = function($trace) 
	{
		const self = this;
		let conversationId = $trace.find("[conversation]").val();
		let logLevel = self.__getLogLevelNumber($trace.find("[log_level]").val());
		let content = $trace.find("[content]").val().toLowerCase();
		let lines = self.trace[conversationId] && self.trace[conversationId].lines || [];
		return lines.filter(function(line) {
			return	self.__getLogLevelNumber(line.level) >= logLevel &&
					line.message.toLowerCase().indexOf(content) >= 0;
		});
	};

	ProvisioningDialplanProcessorMng.prototype.__makeTraceLine = function(line, nowDate)
	{
		let lineDate = new Date(line.timestamp * 1000);
		let lineDate_s = inConcert.Utils.getStringFromDate(lineDate, false, 
			lineDate.getDate() === nowDate? "full_time" : "full_datetime");
		let $line = $(FORMAT("".concat(
			"<div class='dialplanTraceLine level{0}' level='{0}'>",
				"<label class='dialplanTraceLineDate'>",
					"{1}",
				"</label>",
				"<label class='dialplanTraceLineMessage'>",
					"{2}",
				"</label>",
			"</div>"
		), line.level, lineDate_s, line.message));
		return $line;
	};

	ProvisioningDialplanProcessorMng.prototype.validate = function()
	{
		let response = {
			success: true
		};
		if (!this.codeEditor.getValue()) {
			response.success = false;
			response.message = "<%=_T('Must enter a code.')%>";
			return response;
		}
		$.each(this.codeEditor.getSession().getAnnotations(), function(_, annotation) {
			if (annotation.type === "error") {
				response.success = false;
				response.message = "<%=_T('Invalid lua code.')%>";
				return false;
			}
		});
		return response;
	};
	
	ProvisioningDialplanProcessorMng.prototype.get = function()
	{
		return this.codeEditor.getValue();
	};

	ProvisioningDialplanProcessorMng.prototype.set = function(processorId = "", code = "")
	{
		this.processorId = processorId;
		this.codeEditor.getSession().setValue(code);
		this.codeEditor.focus();
	};

}());
