/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, esversion: 11 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

//	"use strict";
	const format = inConcert.Utils.format;

	//---------------------------------------------------------------------
	// form gateways class
	function form_provisioning_gateways(params)
	{
		let success = form_provisioning_gateways.uber.constructor.call(this, params);
		if (!success) {
			return;
		}
		const self = this;

		this.new_title = "<%=_T('New Gateway')%>";
		this.update_title = "<%=_T('Update Gateway {0}')%>";
		this.view_title = "<%=_T('View Gateway {0}')%>";
		this.tagline = "<%=_T('Tagline form Gateway!!!')%>";

		this.html_file_name = "/inconcert/apps/provisioning/gateways/frm/render/";
		this.save_uri =       "/inconcert/apps/provisioning/gateways/save/";
		this.get_uri =        "/inconcert/apps/provisioning/gateways/get/";

		this.classes_name_frm_container = "gatewayFormPage";
		this.include_save_and_continue = false;
		this.requests = [];
		this.gateway = null;

		const __get_instance = function ( section_id, gateway_id /* optional */ )
		{
			if ( !gateway_id ) {
				gateway_id = self.gateway ? self.gateway.configuration.Id : undefined;
			}

			switch(section_id)
			{
				case "basic":
					return inConcert.form_provisioning_gateways_section_basic;
				case "carrier_trunks":
					return inConcert.form_provisioning_gateways_section_carrier_trunks;
				case "node_trunks":
					return inConcert.form_provisioning_gateways_section_node_trunks;
				case "dialplan_contexts":
					return inConcert.form_provisioning_gateways_section_dialplan_contexts;
				default:
					throw "form_provisioning_gateways.__get_instance. Unrecognized instance: " + section_id;
			}
		};

		// definicion de las secciones del formulario.
		this.sections = {
			"basic": {
				name           : "<%=_T('Configuration')%>",
				action_id      : "view_gateways",
				instance       : function( gateway_id ) { return __get_instance( "basic", gateway_id ); },
				initial_state  : "enabled",
				selected       : true,
				can_show       : function( _section, _gateway_id ) { return true; }
			},
			"carrier_trunks": {
				name           : "<%=_T('Carrier Trunks')%>",
				action_id      : "view_gateways",
				instance       : function( gateway_id ) { return __get_instance( "carrier_trunks", gateway_id ); },
				can_show       : function( _section, _gateway_id ) { return true; }
			},
			"node_trunks": {
				name           : "<%=_T('Internal Trunks')%>",
				action_id      : "view_gateways",
				instance       : function( gateway_id ) { return __get_instance( "node_trunks", gateway_id ); },
				can_show       : function( _section, _gateway_id ) { return true; }
			},
			"dialplan_contexts": {
				name           : "<%=_T('Dialplan')%>",
				action_id      : "dialplan_contexts",
				instance       : function( gateway_id ) { return __get_instance( "dialplan_contexts", gateway_id ); },
				can_show       : function( _section, _gateway_id ) { return true; }
			}
		};
		this.create_sections(this);

		return true;
	}

	inConcert.Utils.inherit( form_provisioning_gateways, inConcert.conf_form );

	if ( typeof inConcert.form_provisioning_gateways !== "function" ) {
		inConcert.form_provisioning_gateways = form_provisioning_gateways;
	}

	form_provisioning_gateways.prototype.get = function()
	{
		if (this.is_new) return this.new_gateway();

		const gateway = this.gateway;
		$.each(this.sections, function(_, section) {
			if (!section.frm) {
				return true; // continue
			}
			section.frm.get(gateway);
		});
		return gateway;
	};

	form_provisioning_gateways.prototype.uri_get_entity = function()
	{
		if (this.is_new) {
			return "";
		}
		var id = this.$current_row.attr("row_id");
		id = inConcert.Utils.getIdWithoutVCC(id);
		var uri = inConcert.app.getBaseDomain() + this.get_uri + id;
		return uri;
	};

	form_provisioning_gateways.prototype.new_gateway = function()
	{
		const defaultGateway = {
			configuration: {},
			nodeTrunks: {},
			carrierTrunks: {},
			dialplanContexts: {}
		};
		return this.sections.basic.frm.get(defaultGateway);
	};

	form_provisioning_gateways.prototype.get_entity_id = function()
	{
		return this.gateway?.configuration?.Id;
	};

	form_provisioning_gateways.prototype.get_entity_instance = function()
	{
		return this.gateway;
	};

	form_provisioning_gateways.prototype.set = function(success, gateway)
	{
		if (!success || !gateway)
			return;

		this.gateway = gateway;
		if (typeof(this.gateway.configuration?.Sections) === "string")
			this.gateway.configuration.Sections = JSON.parse(this.gateway.configuration.Sections);
		if ($.isArray(this.gateway.nodeTrunks)) this.gateway.nodeTrunks = {};
		if ($.isArray(this.gateway.carrierTrunks)) this.gateway.carrierTrunks = {};
		if ($.isArray(this.gateway.dialplanContexts)) this.gateway.dialplanContexts = {};
	};

	form_provisioning_gateways.prototype.__get_server_error_description = function(err)
	{
		let response = { is_custom_message: true, message: undefined, result: err.result };

		switch (err.result) {
			case "INVALID_REQUEST":
				response.is_custom_message = true;
				response.message = "<%=_T('Invalid request.')%>";
			break;
			case "OPEN_CONNECTION_ERROR":
				response.is_custom_message = true;
				response.message = "<%=_T('Error connecting to gateway.')%>";
			break;
			case "LOGIN_ERROR":
				response.is_custom_message = true;
				response.message = "<%=_T('Error doing login.')%>";
			break;
			default:
				response.is_custom_message = true;
				response.message = "<%=_T('Server Error.')%> " + (err.reason || "Unknown error");
		}

		return response;
	};

	form_provisioning_gateways.prototype.err_save_messages = function( err )
	{
		if (typeof err !== "object") {
			form_provisioning_gateways.uber.err_save_messages.call( this, err, false /* custom message*/ );  //invokes the base class method
			return;
		}

		var response = this.__get_server_error_description(err);

		form_provisioning_gateways.uber.err_save_messages.call( this, response.message? response.message: response, response.is_custom_message );

		var error_id = ( err.result || "" ).toUpperCase();
		if ( error_id === "FORBIDDEN" || error_id === "MUST_HAVE_A_NAME" || error_id === "MUST_HAVE_UNIQUE_NAME") {
			$("#adm_form_gateway_name").focus();
		}
		else if ( error_id === "MUST_HAVE_A_DESCRIPTION") {
			$("#adm_form_gateway_desc").focus();
		}
	};

	form_provisioning_gateways.prototype.get_validation_def = function()
	{
		var definitions = [];
		$.each(this.sections, function( _, section )
		{
			if (!section.frm) {
				return true; // continue
			}
			section.frm.get_validation_def(definitions);
		});
		return definitions;
	};

	form_provisioning_gateways.prototype.load_data = function(callback_complete_requests)
	{
		var self = this;
		this.requests = [];

		$.each(this.sections, function( _, section)
		{
			if (!section.frm) {
				return true; // continue
			}
			section.frm.attach_load_request(self.requests);
		});

		form_provisioning_gateways.uber.load_data.call( this, callback_complete_requests ); //invokes the base class method
	};

	form_provisioning_gateways.prototype.save_execute = function( after_edit, gateway /*, hash_id */ )
	{
		const self = this;

		this.dashboard.hide_back_button();
		const callback_response = function(success, response)
		{
			let err;
			if (!success) {
				err = response;
			}
			if ( !after_edit ) {
				after_edit = self.is_new ? "SAVE_AND_NOT_EXIT" : "SAVE_WITHOUT_ACTION";
			}
			
			if (success)
				gateway.configuration = response.configuration;
			self.on_saved(success, gateway, after_edit, err, function() {
				self.dashboard.hide_waiting();
			});
		};

		this.gateway = gateway;
		inConcert.app.getProvisioningAPI().gatewaysSave(gateway.configuration, callback_response);
	};

	form_provisioning_gateways.prototype.on_saved = function( success, entity, after_save, err, callback)
	{
		var self = this;

		callback = callback || $.noop;

		var pending_events = 0;
		var on_saved = function() {
			if ( pending_events === 0 ) {
				form_provisioning_gateways.uber.on_saved.call(
					self,
					success,
					entity.configuration,
					after_save,
					err,
					function(success) {
						callback(success);
					}
				); //invokes the base class method
			}
		};

		on_saved();
		if ( !success )
			return;

		this.gateway.configuration.isNew = null;
		const section = this.sections[ this.current_section ];
		if (section?.frm?.readonly_gateway_name)
			section.frm.readonly_gateway_name();
		if (section?.frm?.refresh)
			section.frm.refresh(this.gateway);
	};

	form_provisioning_gateways.prototype.initialize = function()
	{
		form_provisioning_gateways.uber.initialize.call( this ); //invokes the base class method
	};

	form_provisioning_gateways.prototype.dispose = function()
	{
		form_provisioning_gateways.uber.dispose.call( this ); //invokes the base class method
		this.gateway = null;
	};

	form_provisioning_gateways.prototype.set_events = function()
	{
		form_provisioning_gateways.uber.set_events.call( this ); //invokes the base class method
	};

	form_provisioning_gateways.prototype.ready = function()
	{
		form_provisioning_gateways.uber.ready.call( this ); //invokes the base class method
	};

	form_provisioning_gateways.prototype.unsaved_changes = async function()
	{
		if (!this.gateway?.configuration || this.gateway.configuration.isNew)
			return false;

		const unsaved = Object.keys(this.sections)
			.filter((section) => !!this.sections[section].frm?.unsaved_changes)
			.map((section) => this.sections[section].frm.unsaved_changes())
			.filter((unsavedSection) => unsavedSection);

		if (!unsaved.length)
			return false;

		const result = await $.inconcertPopup.ShowAndReceiveResponse(
			"<%=_T('Unsaved changes')%>",
			"<%=_T('There are unsaved changes. Do you want to continue?')%>",
			$.inconcertPopup.ButtonSets.BS_OKCANCEL,
			undefined,
			{
				popup_id        : "conf_inconcertPopup",
				container_class : "confirmationPopUp",
				overlay_class   : "confirmationOverlay"
			}
		);
		return result !== "Ok";
	}

	form_provisioning_gateways.prototype.close = async function(action)
	{
		if (await this.unsaved_changes())
			return;

		form_provisioning_gateways.uber.close.call(this, action);
	};

}());
