/*jshint bitwise:true, browser:true, curly:true, eqeqeq:true, noarg:true, noempty:true, strict:true, undef:true, unused:true, esversion: 9 */
/*jshint quotmark:false*/
/*global $, inConcert, Mustache*/

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
/*jshint validthis:true*/
"use strict";

	const CONTAINER_ID = "form_provisioning_gateways_monitoring_container";
	const MODULE_NAME = "icc-gateways-monitoring-dashboard";
	const format = inConcert.Utils.format;

	class GatewaysMonitoring {
		constructor() {
			this.module = null;
			window.gatewaysMonitoringInstance = this; //TODO: just for debugging, remove it
		}

		open = async (gatewayId, view) => {
			this.view = view;
			this.gatewayId = gatewayId;
			inConcert.app.gatewayId=gatewayId;
			inConcert.app.getMainWindow().showGenericWaiting("<%=_T('Loading monitor...')%>");
			try {
				await this.load_module();
			} catch(error) {
				inConcert.logger.error({ action: "GatewaysMonitoring", gatewayId, error });
				this.view.show_status_message(
					format("<%=_T('Unable to open {0} gateway monitor.')%>", gatewayId), "Error", 5000);
				this.close();
			}
			inConcert.app.getMainWindow().hideGenericWaiting();
		};

		close = () => {
			this.notify_close();
			this.dispose_container();
			this.module = null;
			this.view = null;
			this.gatewayId = null;
		};

		load_module = async () => {
			this.load_container();
			this.module = document.createElement(MODULE_NAME);
			this.module.setAttribute("data-icc-dev-component",true);
			this.$container()[0].appendChild(this.module);
			await this.wait_for_module();
			this.notify_open();
		};

		load_container = () => {
			$("#conf_container").append(`
				<div
					id="${CONTAINER_ID}"
					class="pageContainer angularAppContainer gatewaysMonitoringContainer"
				>
				</div>
			`);
			$("#conf_view_container").hide();
			this.view.dashboard.disabled_all_menus(true);
			this.view.dashboard.enable_menu_item("gateways");
		};

		dispose_container = () => {
			this.$container().remove();
			$("#conf_view_container").show();
			this.view.dashboard.disabled_all_menus(false);
		};

		$container = () => $(`#${CONTAINER_ID}`);

		render = async (callback = $.noop) => {
			try {
				if (this.loaded)
					this.notify_focus();
				else
					await this.load_module();
				$("#inboxMainContainerTop").hide();
				this.$container().show();
			} catch(err) {
				inConcert.app.m_mainWindow.showBannerMessage({
					message: err,
					timeout: 5000,
				});
			}
			callback();
		}

		wait_for_module = async () => new Promise((resolve, _reject) => {
			const interval = window.setInterval(() => {
				if (!this.module.openView)
					return;

				clearInterval(interval);
				resolve();
			});
		});

		notify_open = () => {
			if (!this.module?.openView) return;

			this.module.openView(this.gatewayId, () => this.close());
		}

		notify_close = () => {
			if (!this.module?.closeView) return;

			this.module.closeView();
		}
	}

	inConcert.form_provisioning_gateways_monitoring = GatewaysMonitoring;
}());