/*  jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
  strict:true, undef:true, unused:true, curly:true, browser:true, indent:false, maxerr:50, esversion: 11 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	var format = inConcert.Utils.format;

	/*---------------------------------------------------------------------
	CLASS TRUNKS FRM.
	---------------
		Esta clase es la encargada de realizar el Alta/Modificacion client/side desde el frm de trunks.
		Aqui se encuentra el nombre, la descripcion.
	*/

	function trunks_frm( params )
	{
		this.parent = params.parent;
	// dom ids
		// frm
		this.id_frm_container   	= "form_provisioning_gateways_carrier_trunks_edition";
		// inputs
		this.id_name      			= "form_provisioning_gateways_carrier_trunks_edition_name";
		this.id_carrier      		= "form_provisioning_gateways_carrier_trunks_edition_carrier";
		this.id_registration    	= "form_provisioning_gateways_carrier_trunks_edition_registration";
		this.id_dialplan_context	= "form_provisioning_gateways_carrier_trunks_edition_dialplan_context";
		// buttons actions
		this.id_frm_actions     = "form_provisioning_gateways_carrier_trunks_edition_actions";
		this.id_create          = "form_provisioning_gateways_carrier_trunks_edition_create";
		this.id_cancel          = "form_provisioning_gateways_carrier_trunks_edition_close";

		this.attributes = new inConcert.ProvisioningGatewayConfigTableMng({
			owner        : this.parent,
			prefix       : "form_provisioning_gateways_carrier_trunks_edition",
			container_id : "form_provisioning_gateways_carrier_trunks_edition_attributes",
			footer_actions_scope : "form_provisioning_gateways_carrier_trunks_edition_attributes",
			footer_actions_popup: "form_provisioning_gateways_carrier_trunks_footer_actions"
		});

		this.set_events();
	}

	trunks_frm.prototype.initialize = function()
	{
		this.__fill_dialplans();
		this.__fill_carriers();
		this.__toggle_registration();
	};

	trunks_frm.prototype.__fill_dialplans = function()
	{
		const dialplans = this.parent.frm.gateway?.dialplanContexts || {};
		const $combo = $(`#${this.id_dialplan_context}`);
		this.parent.frm.fill_combo(
			$combo, true /* is_blank_first_item */, dialplans, "Id" /* attr_id */,
			"Id" /* attr_name */, true /* sorted */, undefined /*can_add_item*/,
			"dialplan", /* data_id */);
	};

	trunks_frm.prototype.__fill_carriers = function()
	{
		const carriers = this.parent.carriers || {};
		const $combo = $(`#${this.id_carrier}`);
		this.parent.frm.fill_combo(
			$combo, true /* is_blank_first_item */, carriers, "id" /* attr_id */,
			"id" /* attr_name */, true /* sorted */, undefined /*can_add_item*/,
			"id", /* data_id */);
	};

	trunks_frm.prototype.__toggle_registration = function()
	{
		const isPjsip = this.parent.frm.gateway?.configuration?.Pjsip === "1";
		$(`#${this.id_registration}_container`).toggle(!isPjsip);
	};

	trunks_frm.prototype.get = function()
	{
		return {
			Id: $(`#${this.id_name}`).val(),
			Name: $(`#${this.id_name}`).val(),
			Carrier: $(`#${this.id_carrier}`).val(),
			RegistrationPath: $(`#${this.id_registration}`).val(),
			DialplanContext: $(`#${this.id_dialplan_context}`).val(),
			Attributes: this.attributes.get()
		};
	};

	trunks_frm.prototype.set = function( trunk )
	{
		trunk = trunk || {};
		$(`#${this.id_name}`).val( trunk.Name );
		$(`#${this.id_carrier}`).val( trunk.Carrier );
		$(`#${this.id_registration}`).val( trunk.RegistrationPath );
		$(`#${this.id_dialplan_context}`).val( trunk.DialplanContext );
		this.attributes.initialize();
		this.attributes.set(trunk.Attributes);
	};

	trunks_frm.prototype.show = function( current_trunk, callback )
	{
		callback = callback || $.noop;
		this.set( current_trunk );
		$("#" + this.id_create).val(current_trunk ? "<%=_T('Update')%>" : "<%=_T('Create')%>");
		$("#" + this.id_frm_container).show();
		$("#" + this.id_frm_actions).show();

		callback();
		this.on_show();
	};

	trunks_frm.prototype.on_show = function()
	{
		//this.parent.frm.add_css_class("pageFormForVisibleButtons");
		var $name = $("#" + this.id_name);
		if ( this.parent.$row_editing ) {
			this.parent.readonly_control( $name, true, false );
			$(`#${this.id_carrier}`).focus();
		}
		else {
			this.parent.readonly_control( $name, false, true );
			$name.focus();
		}
	};

	trunks_frm.prototype.hide = function()
	{
		this.parent.hide_trunk_frm_edition();
	};

	trunks_frm.prototype.on_hide = function()
	{
		$("#" + this.id_frm_container).hide();
		$("#" + this.id_frm_actions).hide();
		//this.parent.frm.remove_css_class("pageFormForVisibleButtons");
	};

	trunks_frm.prototype.validate = function()
	{
		var self = this;

		this.parent.frm.hide_messages();

		const show_error = function(message_error, $control)
		{
			self.parent.frm.show_status_message( message_error, "Error", false );
			if (!$control) return;

			var error_id = $control.attr("id") + "_validation";
			var $error_indicator = $("#" + error_id);
			if ($error_indicator) {
				$error_indicator.show();
			}
			$control.focus();
		};

		const $name = $("#" + this.id_name );
		const $carrier = $("#" + this.id_carrier );
		const $dialplanContext = $("#" + this.id_dialplan_context );
		const trunk = this.get();

		if (!inConcert.Utils.validate( trunk.Name, "required" ).success)
			return show_error("<%=_T('Name is required.')%>", $name);

		if (!inConcert.Utils.validate( trunk.Name, "id_without_dots" ).success)
			return show_error(
				"<%=_T('Only letters, numbers and dashes are allowed in the Name.')%>", $name);

		if (trunk.Name.length > 16)
			return show_error(format(
				"<%=_T('Name must have a maximum of {0} characters.')%>", 16), $name);
		
		if (!inConcert.Utils.validate( trunk.Carrier, "required" ).success)
			return show_error("<%=_T('Carrier is required.')%>", $carrier);
		
		if (!inConcert.Utils.validate( trunk.DialplanContext, "required" ).success)
			return show_error("<%=_T('Dialplan Context is required.')%>", $dialplanContext);

		const attributesValidation = this.attributes.validate();
		if (!attributesValidation.success)
			return show_error(attributesValidation.message);

		const trunkName = trunk.Name.toUpperCase();
		if (!this.parent.$row_editing && this.parent.$get_rows().toArray().some((row) =>
			$(row).attr("trunk_id").toUpperCase() === trunkName
		))
			return show_error("<%=_T('Name already exists.')%>", $name);

		// validation successfully
		$("#" + this.id_frm_container).find(".formFieldValidation").hide();
		return trunk;
	};

	trunks_frm.prototype.set_events = function()
	{
		var self = this;

		$("#" + this.id_create)
			.off("click")
			.on("click", function() {
				self.parent.add_or_update_trunk();
			});
		$("#" + this.id_cancel)
			.off("click")
			.on("click", function() {
				self.hide();
				self.parent.hide_messages();
			});
		
	};
	//
	//-----------------------------------------


	//---------------------------------------------------------------------
	// 
	/*---------------------------------------------------------------------
	CLASS TRUNKS SECTION 
	------------------------
		Esta clase hereda de la clase "conf_form_section" utilizado en las diferentes secciones de la campana.

		Es la encargada de mostrar la lista de trunks, y de realizar el ABM de estos.
	*/
	function form_provisioning_gateways_section_carrier_trunks(params)
	{
		var success = form_provisioning_gateways_section_carrier_trunks.uber.constructor.call(this, params);
		if (!success) {
			return;
		}
		// permisos
		this.view_action_id           = "view_gateways";   // view action id de permisos
		this.save_action_id           = "save_gateway";   // save action id de permisos
		this.erase_action_id          = "delete_gateway"; // delete action id de permisos
		this.has_perm_to_save         = null;
		this.has_perm_to_erase        = null;

		// dom ids
		// 
		this.id_list_container        = "form_provisioning_gateways_carrier_trunks_list_container";
		this.trunks                   = "form_provisioning_gateways_carrier_trunks_list";
		this.id_new_trunk             = "form_provisioning_gateways_carrier_trunks_new";
		this.trunks_not_result_found  = "form_provisioning_gateways_carrier_trunks_not_results_found";

		// selectors - row
		this.selector_row_name        = "[col_name]";
		this.selector_row_carrier     = "[col_carrier]";
		this.selector_row_dialplan    = "[col_dialplan]";

		this.$row_editing = null; // Aca se encuentra el row html de jquery que se esta editando
		
		this.trunks_frm = null; // instancia de la clase que maneja la actualizacion.
		this.carriers = null;

		return true;
	}

	inConcert.Utils.inherit( form_provisioning_gateways_section_carrier_trunks, inConcert.conf_form_section );

	if (typeof inConcert.form_provisioning_gateways_section_carrier_trunks !== "function") {
		inConcert.form_provisioning_gateways_section_carrier_trunks = form_provisioning_gateways_section_carrier_trunks;
	}

	form_provisioning_gateways_section_carrier_trunks.prototype.dispose = function()
	{
		this.has_perm_to_save  = null;
		this.has_perm_to_erase = null;
	};

	// Alta en el hash de todos los trunks
	form_provisioning_gateways_section_carrier_trunks.prototype.add_trunk_to_full_list = function(id, trunk)
	{
		this.frm.gateway.carrierTrunks[id] = trunk;
	};

	// Actualiza en el hash de todos los trunks
	form_provisioning_gateways_section_carrier_trunks.prototype.update_trunk_on_full_list = function(id, trunk)
	{
		if (!this.frm.gateway.carrierTrunks.hasOwnProperty(id)) {
			return;
		}
		this.frm.gateway.carrierTrunks[id] = trunk;
	};

	// Baja en el hash todos los trunks
	form_provisioning_gateways_section_carrier_trunks.prototype.erase_trunk_on_full_list = function(id)
	{
		if (!this.frm.gateway.carrierTrunks.hasOwnProperty(id)) {
			return;
		}
		delete this.frm.gateway.carrierTrunks[id];
	};

	form_provisioning_gateways_section_carrier_trunks.prototype.get_trunk_from_full_list = function(id)
	{
		if (!id) {
			return {};
		}
		return this.frm.gateway.carrierTrunks[id];
	};

	form_provisioning_gateways_section_carrier_trunks.prototype.$get_row = function(id)
	{
		if ( typeof id !== "string" ) {
			id = "";
		}
		return $("#" + this.trunks).find("li[row_data][trunk_id='" + id + "']");
	};

	form_provisioning_gateways_section_carrier_trunks.prototype.$get_rows = function()
	{
		return $("#" + this.trunks).find("li[row_data]");
	};

	// agrega un row ordenado alfabeticamente
	form_provisioning_gateways_section_carrier_trunks.prototype.add_sorted_row = function(new_name, $new_row)
	{
		var self = this;
		var $grid = $("#" + this.trunks);
		var new_sorted_key = this.make_sorted_key(new_name);
		var added = false;
		$grid.find("li[row_data]").each(function( _, row )
		{
			var $row = $(row);
			var id = $row.attr("trunk_id");
			var trunk_data = self.get_trunk_from_full_list(id);
			var current_sorted_key = self.make_sorted_key(trunk_data.Name);
			if (new_sorted_key < current_sorted_key) {
				$row.before($new_row);
				added = true;
				return false; // break
			}
		});
		if (!added) {
			$grid.append($new_row);
		}
	};

	form_provisioning_gateways_section_carrier_trunks.prototype.has_permission_to_delete = function()
	{
		if ( this.has_perm_to_erase !== null ) {
			return this.has_perm_to_erase;
		}
		this.has_perm_to_erase = form_provisioning_gateways_section_carrier_trunks.uber.has_permission_to_delete.call( this );
		return this.has_perm_to_erase;
	};

	form_provisioning_gateways_section_carrier_trunks.prototype.add_row = function(trunk, is_new)
	{
		const self = this;
		if (!$.isPlainObject(trunk)) {
			return;
		}
		const id = trunk.Id;
		const name = trunk.Name || id;
		const carrier = trunk.Carrier;
		const dialplan = trunk.DialplanContext;
		var $row = this.$get_row(id);

		var already_trunk = $row.length > 0;
		if ( already_trunk ) {
		// si el trunk a agregar ya existe en la lista de la GUI me voy
			return;
		}
		var get_actions = function()
		{
			var has_perm_to_save  = self.has_permission_to_save();
			var has_perm_to_erase = self.has_permission_to_delete();
			if ( !has_perm_to_save && ! has_perm_to_erase ) {
				return "";
			}
			var erase_html = "";
			if ( has_perm_to_erase ) { 
				erase_html = "<button action_del='true' class='buttonColorGray gridActionButton delete last' type='button' title='"+ "<%=_T('Delete')%>" + "'><i class='buttons-icons-delete'></i>" + "<%=_T('Delete')%>" + "</button>";
			}
			var save_html = "";
			if ( has_perm_to_save ) {
				save_html = "<button action_edit='true' class='buttonColorGray gridActionButton edit' type='button' title='"+ "<%=_T('Edit')%>" + "'><i class='buttons-icons-edit'></i>" + "<%=_T('Edit')%>" + "</button>";
			}
			return "".concat(save_html , erase_html);
		};
		var $row_html = $("".concat(
			"<li row_data='true' trunk_id='", id, "' class='viewListItem'>",
				"<label col_name='true' class='viewListItemText'>", name, "</label>",
				"<label col_carrier='true' class='viewListItemText'>", carrier, "</label>",
				"<label col_dialplan='true' class='viewListItemText'>", dialplan, "</label>",
				get_actions(),
			"</li>")
		);

		if (!is_new) {
			$("#" + this.trunks).append($row_html);
		}
		else {
			this.add_sorted_row(name, $row_html); // agregar ordenada
		}
		// add trunk in full hash
		this.add_trunk_to_full_list(id, trunk);
		// update message found
		this.show_or_hide_not_results_found();
	};

	form_provisioning_gateways_section_carrier_trunks.prototype.update_row = function( new_row_data )
	{
		if ( !$.isPlainObject( new_row_data ) ) {
			return;
		}
		var $current_row = this.$get_row( new_row_data.Id );
		// name
		var $name = $current_row.find( this.selector_row_name );
		var old_name = $name.text().toLowerCase();
		var new_name = ( new_row_data.Name || "" ).toLowerCase();

		// actualiza la lista completa de trunks
		this.update_trunk_on_full_list( new_row_data.Id, new_row_data );
		$name.text( new_row_data.Name );
		$current_row.find( this.selector_row_carrier ).text(new_row_data.Carrier);
		$current_row.find( this.selector_row_dialplan ).text(new_row_data.DialplanContext);

		if ( old_name !== new_name ) {
			// si el Nombre cambio, tengo que cambiarlo de posicion dentro de la lista (alfabeticamente)
			var $row = $current_row;
			this.$row_editing.remove();
			this.add_sorted_row( new_name, $row );
		}
		this.show_or_hide_not_results_found();
	};

	form_provisioning_gateways_section_carrier_trunks.prototype.make_sorted_key = function(name)
	{
		return ( name || "" ).toLowerCase();
	};

	form_provisioning_gateways_section_carrier_trunks.prototype.show_not_results_found = function( message )
	{
		$("#" + this.trunks_not_result_found)
			.text( message || "<%=_T('No trunks found.')%>" )
			.show();

		$("#" + this.trunks).hide();
	};

	form_provisioning_gateways_section_carrier_trunks.prototype.hide_not_results_found = function()
	{
		$("#" + this.trunks).show();
		$("#" + this.trunks_not_result_found).hide();
	};

	form_provisioning_gateways_section_carrier_trunks.prototype.show_or_hide_not_results_found = function()
	{
		var count = Object.keys( this.frm.gateway.carrierTrunks || {} ).length;
		if ( count > 0 ) {
			this.hide_not_results_found();
		}
		else {
			this.show_not_results_found();
		}
	};

	form_provisioning_gateways_section_carrier_trunks.prototype.set_trunks = function()
	{
		var self = this;

		$("#" + this.trunks).find("li[row_data]").remove();
		const trunks = this.frm.gateway.carrierTrunks;

		if (!$.isPlainObject(trunks)) {
			this.show_or_hide_not_results_found();
			return;
		}

		var sorted_trunks = inConcert.Utils.hash_to_sorted_array(trunks,
			function(trunk) {
				return self.make_sorted_key(trunk.Name);
			}) || {};

		if ( sorted_trunks.length > 0 ) {
			$.each(sorted_trunks, function( _, item )
			{
				item.id = item.Id;
				self.add_row(item);
			});
		}
		this.show_or_hide_not_results_found();
		this.frm.dashboard.hide_waiting();
	};

	form_provisioning_gateways_section_carrier_trunks.prototype.remove_trunks = function(trunks, callback)
	{
		callback = callback || $.noop;

		if ( !$.isArray( trunks ) || trunks.length === 0 ) {
			callback(false);
			return;
		}
		const gatewayId = this.frm.get_entity_id();
		const trunkId = trunks[0];

		this.frm.dashboard.show_waiting("<%=_T('Please wait until trunks are deleted.')%>");
		// lo meto dentro de un timeout minimo, para que se pueda ver el waiting, ya que si la grilla tiene muchos registros
		//	parece que queda la pantalla congelada, porque no da el tiempo para que se vea el waiting
		window.setTimeout(() => {
			const api = inConcert.app.getProvisioningAPI();
			api.gatewayCarrierTrunksDelete(gatewayId, trunkId, (success, error) => {
				var message, type, timeout_ms;

				if ( success ) {
					message = "<%=_T('Trunks have been deleted successfully.')%>";
					type = "Info";
					timeout_ms = 3000;
				}
				else {
					if (typeof error !== "object") {
						error = {result: error};
					}
					switch ((error.result || "").toUpperCase())
					{
						case "INVALID_REQUEST":
							message = "<%=_T('Invalid request.')%>";
							break;
						case "OPEN_CONNECTION_ERROR":
							message = "<%=_T('Error connecting to gateway.')%>";
							break;
						case "LOGIN_ERROR":
							message = "<%=_T('Error doing login.')%>";
							break;
						default:
							message = "<%=_T('Server error.')%> " + (error.reason || "Unknown error");
					}
					type = "Error";
					timeout_ms = false; // infinito
				}

				this.frm.show_status_message(
					message,        /* mensaje          */
					type,           /* tipo de mensaje  */
					timeout_ms      /* timeout ms       */
				);
				callback( success );
			});
		}, 50);
	};

	form_provisioning_gateways_section_carrier_trunks.prototype.build_header_custom_buttons = function()
	{
		var html_buttons =
			"<button id='" + this.id_new_trunk + "' class='pageButtonGeneric buttonColorPrimary pageButtonNew' ><i class='fal fa-plus'></i><%=_T('New')%></button>";

		form_provisioning_gateways_section_carrier_trunks.uber.build_header_custom_buttons.call(this, html_buttons);
	};

	form_provisioning_gateways_section_carrier_trunks.prototype.get_save_trunk_error = function(error)
	{
		switch ((error?.result || error).toUpperCase())
		{
		case "FORBIDDEN":
			return "<%=_T('You dont have permissions to save trunks.')%>";
		case "INVALID_REQUEST":
			return "<%=_T('Invalid request.')%>";
		case "GATEWAY NODE TRUNK EXISTS":
			return "<%=_T('Trunk name already taken by an internal trunk.')%>";
		case "OPEN_CONNECTION_ERROR":
			return "<%=_T('Error connecting to gateway.')%>";
		case "LOGIN_ERROR":
			return "<%=_T('Error doing login.')%>";
		case "INVALID_PATH":
			return format("<%=_T('Format for registration is {0}')%>",
				"[peer?][transport://]user[@domain][:secret[:authuser]]@host[:port][/extension][~expiry]");
		default:
			return "<%=_T('Server error.')%> " + (error?.reason || "Unknown error");
		}
	};

	form_provisioning_gateways_section_carrier_trunks.prototype.add_or_update_trunk = function()
	{
		let trunkData = this.trunks_frm.validate();
		if (!trunkData)
			return;
			
		trunkData.isNew = !this.$row_editing;
		const gatewayId = this.frm.get_entity_id();
		const api = inConcert.app.getProvisioningAPI();
		this.saving = true;
		this.frm.dashboard.show_waiting("<%=_T('Please wait until trunks are saved.')%>");
		api.gatewayCarrierTrunksSave(gatewayId, trunkData, (success, result) => {
			this.saving = false;
			if (!success)
				return this.frm.show_status_message(this.get_save_trunk_error(result), "Error", 5000);

			this.hide_trunk_frm_edition();
			trunkData = {...trunkData, ...result.trunk};
			if (trunkData.isNew)
				this.add_row(trunkData, true);
			else
				this.update_row(trunkData);
			this.$row_editing = null;
			this.frm.show_status_message("<%=_T('Trunks have been saved successfully.')%>", "Info", 3000);
		});
	};

	form_provisioning_gateways_section_carrier_trunks.prototype.show_trunk_frm_edition = function()
	{
		var self = this;
		this.frm.hide_messages();

		var trunk_data;
		if ( this.$row_editing ) {
		// si presione el boton para editar
			var id = this.$row_editing.attr("trunk_id");
			trunk_data = this.get_trunk_from_full_list(id);
		}

		if ( !this.has_permission_to_section() || !this.has_permission_to_save() ) {
			var message;
			if ( !trunk_data ) {
				message = "<%=_T('Unable to view trunk: You dont have permissions for this action.')%>";
			}
			else {
				message = format( "<%=_T('Unable to view {0} trunk: You dont have permissions for this action.')%>", trunk_data.name );
			}
			this.$row_editing = null;
			this.frm.show_status_message( message, "Error", 5000 );
			return;
		}

		// show frm
		this.trunks_frm.show( trunk_data, function() {
			$("#" + self.id_list_container).hide();
			$("#" + self.id_new_trunk).hide();
		});
	};

	form_provisioning_gateways_section_carrier_trunks.prototype.hide_trunk_frm_edition = function()
	{
		this.trunks_frm.on_hide();
		$("#" + this.id_list_container).show();
		$("#" + this.id_new_trunk).show();
		this.$row_editing = null;
	};

	form_provisioning_gateways_section_carrier_trunks.prototype.set_events_frm_abm = function()
	{
		var self = this;
		// new message event
		var event_selector = "#" + this.id_new_trunk;
		var $header_buttons = this.frm.dashboard.$get_dom_element_by_id( "form_custom_buttons" ); //conf_form_custom_buttons
		$header_buttons
			.off("click", event_selector)
			.on("click", event_selector, function() {
				self.show_trunk_frm_edition();
			});
	};

	form_provisioning_gateways_section_carrier_trunks.prototype.set_events = function()
	{
		var self = this;

		var $grid = $("#" + this.trunks);
		var rows_event_selector = "li[row_data]";

		// boton delete de la lista de parametros
		var event_selector = rows_event_selector + " button[action_del]";
		$grid
			.off("click", event_selector)
			.on("click", event_selector, function()
			{
				self.frm.hide_messages();
				self.$row_editing = null;
				var $row = $(this).parent();
				var id = $row.attr("trunk_id");
				var trunk = self.get_trunk_from_full_list(id);
				var message = format(
					"<%=_T('Do you want to delete trunk {0} ?')%>",
					trunk.Name);

				$.inconcertPopup.Show(
					"<%=_T('Confirm')%>",
					message,
					$.inconcertPopup.ButtonSets.BS_YESNO,
					function(response) {
						if (response.toLowerCase() !== "yes") {
							return;
						}
						var trunks_list = [];
						trunks_list.push(trunk.Id);
						// envia a remover al server
						self.remove_trunks(
							trunks_list,
							function(success) {
								if (!success) {
									return;
								}
								// elimino del hash de trunks
								self.erase_trunk_on_full_list(trunk.Id);
								$row.fadeOut("slow", function() {
									$row.remove();
									self.show_or_hide_not_results_found();
								});
							});
					},
					undefined,
					{
						popup_id        : "conf_inconcertPopup",
						container_class : "confirmationPopUp",
						overlay_class   : "confirmationOverlay"
					}
				);
			});

		// doble click sobre la row edita la misma
		$grid
			.off("dblclick", rows_event_selector)
			.on("dblclick", rows_event_selector, function(event) {
				event.stopPropagation();
				self.$row_editing = $(this);
				self.show_trunk_frm_edition();
			});

		// boton edit de la lista de parametros
		event_selector = rows_event_selector + " button[action_edit]";
		$grid
			.off("click", event_selector)
			.on("click", event_selector, function() {
				self.$row_editing = $(this).parent();
				self.show_trunk_frm_edition();
			});

		// eventos del abm del frm
		this.set_events_frm_abm();
	};

	form_provisioning_gateways_section_carrier_trunks.prototype.load_gateway_trunks = function()
	{
		this.set_trunks();
	};

	form_provisioning_gateways_section_carrier_trunks.prototype.initialize = function()
	{
		this.has_perm_to_save  = null;
		this.has_perm_to_erase = null;
		this.trunks_frm = new trunks_frm( {
			parent: this
		});
		this.trunks_frm.initialize();
		this.load_gateway_trunks();
	};

	form_provisioning_gateways_section_carrier_trunks.prototype.set = function()
	{
		this.initialize();
	};

	form_provisioning_gateways_section_carrier_trunks.prototype.on_show = function()
	{
		form_provisioning_gateways_section_carrier_trunks.uber.on_show.call(this, {show: false});
		this.frm.dashboard.$get_dom_element_by_id("form_buttons_actions").hide();
		this.trunks_frm.initialize();
	};

	form_provisioning_gateways_section_carrier_trunks.prototype.on_hide = function()
	{
		form_provisioning_gateways_section_carrier_trunks.uber.on_hide.call( this );
		this.hide_trunk_frm_edition();
	};

	form_provisioning_gateways_section_carrier_trunks.prototype.editable = function()
	{
		form_provisioning_gateways_section_carrier_trunks.uber.editable.call( this );
	};

	form_provisioning_gateways_section_carrier_trunks.prototype.readonly = function() {
		form_provisioning_gateways_section_carrier_trunks.uber.readonly.call( this );
	};

	form_provisioning_gateways_section_carrier_trunks.prototype.attach_load_request = function(requests)
	{
		const api = inConcert.app.getProvisioningAPI();
		requests.push({
			request_function: (success) => api.carriersLoadAll(success),
			success: (list) => this.carriers = list
		});
	};

}());
