/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 11 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

//	"use strict";
	const format = inConcert.Utils.format;

	//---------------------------------------------------------------------
	// form gateways class - basic section
	function form_provisioning_gateways_section_dialplan_contexts( params )
	{
		const success = form_provisioning_gateways_section_dialplan_contexts.uber.constructor.call( this, params );
		if (!success) {
			return;
		}
		this.view_action_id = "view_gateways";  // view action id de permisos
		this.save_action_id  = { create: "save_gateway", update: "save_gateway" }; // save action id de permisos
		this.erase_action_id = null; // delete action id de permisos
		this.contexts = {};
		this.selectedContext = null;
	
		return true;
	}

	inConcert.Utils.inherit( form_provisioning_gateways_section_dialplan_contexts, inConcert.conf_form_section );

	if ( typeof inConcert.form_provisioning_gateways_section_dialplan_contexts !== "function" ) {
		inConcert.form_provisioning_gateways_section_dialplan_contexts = form_provisioning_gateways_section_dialplan_contexts;
	}

	form_provisioning_gateways_section_dialplan_contexts.prototype.initialize = function()
	{
		this.set_events();
	};

	form_provisioning_gateways_section_dialplan_contexts.prototype.set_events = function()
	{
		$("#frm_gateway_content_dialplan_contexts_save")
			.off("click")
			.on("click", () => this.save_contexts());
		$("#frm_gateway_content_dialplan_contexts_add_button")
			.off("click")
			.on("click", () => this.add_context());
		$("#frm_gateway_content_dialplan_contexts_all_tab")
			.off("click")
			.on("click", () => {
				$("#frm_gateway_content_dialplan_contexts_all_tab").addClass("selected");
				$("#frm_gateway_content_dialplan_contexts_unsaved_tab").removeClass("selected");
				this.populate_toolbar();
			});
		$("#frm_gateway_content_dialplan_contexts_unsaved_tab")
			.off("click")
			.on("click", () => {
				$("#frm_gateway_content_dialplan_contexts_unsaved_tab").addClass("selected");
				$("#frm_gateway_content_dialplan_contexts_all_tab").removeClass("selected");
				this.populate_toolbar();
			});

		const doSearch = _.throttle(() => this.populate_toolbar(), 100);
		$("#frm_gateway_content_dialplan_contexts_search")
			.off("keyup")
			.on("keyup", () => doSearch());

		$("#frm_gateway_content_dialplan_contexts_name")
			.off("change")
			.on("change", () => {
				const contextName = $("#frm_gateway_content_dialplan_contexts_name").val();
				const mustToggle = !this.contexts[this.selectedContext].context.Name || !contextName;
				this.contexts[this.selectedContext].context.Name = contextName;
				this.contexts[this.selectedContext].$toolbarContext.find("[name_toolbar]").val(contextName);
				this.contexts[this.selectedContext].$toolbarContext.addClass("edited");
				if (mustToggle)
					this.populate_toolbar();
				this.populate_counters();
			});
	};

	form_provisioning_gateways_section_dialplan_contexts.prototype.get = function(gateway = {})
	{
		return gateway;
	};

	form_provisioning_gateways_section_dialplan_contexts.prototype.set = function(gateway)
	{
		this.set_contexts(gateway.dialplanContexts || {});
	};

	form_provisioning_gateways_section_dialplan_contexts.prototype.set_contexts = function(contexts)
	{
		this.contexts = {};
		this.selectedContext = null;
		$("#frm_gateway_content_dialplan_contexts_search").val("");

		Object.keys(contexts).forEach((contextId) => this.set_context(contexts[contextId]));
		$("#frm_gateway_content_dialplan_contexts_all_tab").trigger("click");
		if (Object.keys(this.contexts).length)
			this.select_last_context();
		else
			this.add_context();
		this.populate_counters();
	};

	form_provisioning_gateways_section_dialplan_contexts.prototype.set_context = function(
		context = { isNew: true, Name: "" })
	{
		const contextGuid = inConcert.Utils.generateGuid();
		const $toolbarContext = $(`
			<li class="dialplanToolbarElement">
				<div class="dialplanToolbarElementTitle">
					${!this.is_readonly() ? (`
						<input
							toolbar_element_check
							class="form-check-input"
							type="checkbox"
						/>
					`) : ""}
					<div class="dialplanToolbarElementName">
						${context.Name || ""}<span class="dialplanToolbarElementError"><%=_T('Error')%></span>
					</div>
				</div>
			</li>
		`);

		$toolbarContext.click(() => this.select_context(contextGuid));
		$toolbarContext.find("[toolbar_element_check]")
			.click((e) => e.stopPropagation())
			.change((e) => {
				$(e.target).closest(".dialplanToolbarElement").toggleClass("selectedToEdit", e.target.checked);
				this.refresh_footer();
			});
		$toolbarContext.toggleClass("edited", !!context.isNew);

		this.contexts[contextGuid] = { $toolbarContext, context };
		return contextGuid;
	};

	form_provisioning_gateways_section_dialplan_contexts.prototype.add_context = function()
	{
		this.select_context(this.set_context());
		this.populate_counters();
	};

	form_provisioning_gateways_section_dialplan_contexts.prototype.remove_contexts = async function(contextIds)
	{
		if (!await this.confirm_remove()) return;

		for (const contextGuid of contextIds) {
			try {
				await this.remove_context(contextGuid);
			} catch(_e) {
				return;
			}
		}
		this.frm.show_status_message("<%=_T('Contexts have been deleted successfully.')%>", "Info", 3000);
		this.refresh_footer();
	};

	form_provisioning_gateways_section_dialplan_contexts.prototype.confirm_remove = async function()
	{
		const result = await $.inconcertPopup.ShowAndReceiveResponse(
			"<%=_T('Remove contexts')%>",
			"<%=_T('Do you want to delete contexts?')%>",
			$.inconcertPopup.ButtonSets.BS_OKCANCEL,
			undefined,
			{
				popup_id        : "conf_inconcertPopup",
				container_class : "confirmationPopUp",
				overlay_class   : "confirmationOverlay"
			}
		);
		return result === "Ok";
		
	};

	form_provisioning_gateways_section_dialplan_contexts.prototype.remove_context = async function(contextGuid)
	{
		const context = this.contexts[contextGuid];
		context.$toolbarContext.removeClass("error");
		if (context.context.isNew) return this.destroy_context(contextGuid);

		await this.remove_context_on_server(contextGuid);
	};

	form_provisioning_gateways_section_dialplan_contexts.prototype.remove_context_on_server = async function(contextGuid)
	{
		const context = this.contexts[contextGuid];
		const contextId = context.context.Id;
		const gatewayId = this.frm.get_entity_id();
		const api = inConcert.app.getProvisioningAPI();
		this.frm.dashboard.show_waiting("<%=_T('Please wait until context is removed.')%>");
		return new Promise((resolve, reject) => (
			api.gatewayDialplanContextsDelete(gatewayId, contextId, (success, error) => {
				if (success) {
					delete this.frm.gateway.dialplanContexts[contextId];
					this.destroy_context(contextGuid);
					resolve();
				} else {
					this.show_remove_error(context.context.Name, error);
					context.$toolbarContext.addClass("error");
					reject();
				}
			})
		));
	};

	form_provisioning_gateways_section_dialplan_contexts.prototype.show_remove_error = function(contextName, error)
	{
		let message;
		switch ((error.result || "").toUpperCase()) {
		case "INVALID_REQUEST":
			message = "<%=_T('Invalid request.')%>";
			break;
		case "GATEWAY CONTEXT ASSIGNED TO NODE TRUNK":
			message = "<%=_T('Context is assigned to an internal trunk.')%>";
			break;
		case "GATEWAY CONTEXT ASSIGNED TO CARRIER TRUNK":
			message = "<%=_T('Context is assigned to a carrier trunk.')%>";
			break;
		case "OPEN_CONNECTION_ERROR":
			message = "<%=_T('Error connecting to gateway.')%>";
			break;
		case "LOGIN_ERROR":
			message = "<%=_T('Error doing login.')%>";
			break;
		default:
			message = "<%=_T('Server error.')%> " + (error.reason || "Unknown error");
		}
		this.frm.show_status_message(`${contextName}: ${message}`, "Error");
	};

	form_provisioning_gateways_section_dialplan_contexts.prototype.destroy_context = function(contextGuid, avoidSelecting = false)
	{
		this.contexts[contextGuid].$toolbarContext.remove();
		delete this.contexts[contextGuid];
		if (this.selectedContext === contextGuid)
			this.selectedContext = null;
		if (!avoidSelecting) {
			if (Object.keys(this.contexts).length)
				this.select_last_context();
			else
				this.add_context();
		}
		this.populate_counters();
	}

	form_provisioning_gateways_section_dialplan_contexts.prototype.select_context = function(contextGuid)
	{
		if (contextGuid === this.selectedContext) return;

		if (this.selectedContext && !this.contexts[this.selectedContext].context.Name)
			this.destroy_context(this.selectedContext, true);
		this.selectedContext = contextGuid;
		const context = this.contexts[contextGuid];
		Object.keys(this.contexts).forEach((contextGuid) =>
			this.contexts[contextGuid].$toolbarContext.removeClass("selected"));
		context.$toolbarContext.addClass("selected");
		this.load_context_code(context);
		this.load_context_trunks(context);
		$("#frm_gateway_content_dialplan_contexts_name").val(context.context.Name);
		this.frm.readonly_control($("#frm_gateway_content_dialplan_contexts_name"), !context.context.isNew, true);
	};

	form_provisioning_gateways_section_dialplan_contexts.prototype.select_last_context = function()
	{
		$("#frm_gateway_content_dialplan_contexts_toolbar_list").children().last().click();
	};

	form_provisioning_gateways_section_dialplan_contexts.prototype.load_context_code = function(context)
	{
		if (!context.$codeEditor)
			this.load_context_editor(context);
		$("#frm_gateway_content_dialplan_contexts_code_container")
			.html("")
			.append(context.$codeEditor);
		context.codeEditor.setReadOnly(this.is_readonly());
		context.codeEditor.resize();
		if (!context.context.isNew)
			context.codeEditor.focus();
	};

	form_provisioning_gateways_section_dialplan_contexts.prototype.load_context_editor = function(context)
	{
		context.$codeEditor = $("<div></div>");
		context.codeEditor = ace.edit(context.$codeEditor[0]);
		context.codeEditor.setTheme(format("ace{0}theme{0}clouds", "/"));
		context.codeEditor.renderer.setScrollMargin(8, 8);
		context.codeEditor.getSession().setMode(format("ace{0}mode{0}dialplan", "/"));
		context.codeEditor.setOptions({
			enableBasicAutocompletion: true,
			enableSnippets: true
		});
		context.codeEditor.getSession().setValue(context.context.Code || "");
		context.codeEditor.getSession().on("change", () => {
			context.$toolbarContext.addClass("edited");
			this.populate_counters();
		});
	};

	form_provisioning_gateways_section_dialplan_contexts.prototype.load_context_trunks = function(context)
	{
		const $selectedTrunks =
			(context.context.OutgoingTrunks || []).sort().map((trunkId) =>
				this.make_selected_trunk(trunkId));
		$("#frm_gateway_content_dialplan_contexts_trunks_list").html("").append($selectedTrunks);
		this.load_trunk_options();
	};

	form_provisioning_gateways_section_dialplan_contexts.prototype.make_selected_trunk = function(trunkId)
	{
		const $selectedTrunk = $(`
			<div trunk_id="${trunkId}" class="dialplanToolbarElementViewButtonsContainer">
				${trunkId}
				${!this.is_readonly() ? (`
					<button remove_trunk class="dialplanToolbarElementViewButton">
						<i class='far fa-times'></i>
					</button>
				`) : ""}
			</div>
		`);
		$selectedTrunk.find("[remove_trunk]").click(() => {
			this.contexts[this.selectedContext].$toolbarContext.addClass("edited");
			$selectedTrunk.remove();
			this.load_trunk_options();
			this.set_context_trunks();
			this.populate_counters();
		});
		return $selectedTrunk;
	};

	form_provisioning_gateways_section_dialplan_contexts.prototype.load_trunk_options = function()
	{
		const $selected = $("#frm_gateway_content_dialplan_contexts_trunks_list");
		const $options = [
				...Object.keys(this.frm.gateway.nodeTrunks),
				...Object.keys(this.frm.gateway.carrierTrunks),
			]
			.sort()
			.filter((trunkId) => !$selected.find(`[trunk_id=${trunkId}]`).length)
			.map(trunkId => this.make_trunk_option(trunkId));
		$("#frm_gateway_content_dialplan_contexts_trunks_drop_list").html("").append($options);
	};

	form_provisioning_gateways_section_dialplan_contexts.prototype.make_trunk_option = function(trunkId)
	{
		const $trunkOption = $(`
			<div class="dropdown-item">${trunkId}</div>
		`);
		$trunkOption.click(() => {
			this.contexts[this.selectedContext].$toolbarContext.addClass("edited");
			$("#frm_gateway_content_dialplan_contexts_trunks_list").append(this.make_selected_trunk(trunkId));
			this.load_trunk_options();
			this.set_context_trunks();
			this.populate_counters();
		});
		return $trunkOption;
	};

	form_provisioning_gateways_section_dialplan_contexts.prototype.set_context_trunks = function()
	{
		this.contexts[this.selectedContext].context.OutgoingTrunks =
			$("#frm_gateway_content_dialplan_contexts_trunks_list")
				.children().toArray().map((trunk) => $(trunk).attr("trunk_id")).sort();
	};

	form_provisioning_gateways_section_dialplan_contexts.prototype.save_contexts = async function()
	{
		const contexts = Object.keys(this.contexts).filter((contextGuid) =>
			this.contexts[contextGuid].$toolbarContext.is(".edited")
		).sort(this.sort_condition());
		if (!contexts.length)
			return;

		for (const contextGuid of contexts) {
			try {
				await this.save_context(contextGuid);
			} catch(_e) {
				return;
			}
		}
		this.frm.show_status_message("<%=_T('Contexts saved successfully.')%>", "Info", 3000);
	};

	form_provisioning_gateways_section_dialplan_contexts.prototype.save_context = async function(contextGuid)
	{
		const context = this.contexts[contextGuid];
		context.$toolbarContext.removeClass("error");
		if (!this.validate_context(contextGuid)) {
			context.$toolbarContext.addClass("error");
			throw "invalid context";
		}

		const contextData = this.get_context_data(contextGuid);
		const gatewayId = this.frm.get_entity_id();
		const api = inConcert.app.getProvisioningAPI();
		this.frm.dashboard.show_waiting("<%=_T('Please wait until context is saved.')%>");
		return new Promise((resolve, reject) => (
			api.gatewayDialplanContextsSave(gatewayId, contextData, (success, error) => {
				if (!success) {
					context.$toolbarContext.addClass("error");
					this.show_save_error(contextData.Name, error || {});
					return reject();
				}

				context.$toolbarContext.removeClass("edited");
				context.context = contextData;
				context.context.isNew = null;
				this.frm.gateway.dialplanContexts[contextData.Id] = contextData;
				if (contextGuid === this.selectedContext)
					this.frm.readonly_control($("#frm_gateway_content_dialplan_contexts_name"), true, true);
				this.populate_toolbar();
				this.populate_counters();
				resolve();
			})
		));
	};

	form_provisioning_gateways_section_dialplan_contexts.prototype.show_save_error = function(contextName, error)
	{
		let message;
		switch ((error.result || "").toUpperCase()) {
			case "INVALID_REQUEST":
				message = "<%=_T('Invalid request.')%>";
				break;
			case "OPEN_CONNECTION_ERROR":
				message = "<%=_T('Error connecting to gateway.')%>";
				break;
			case "LOGIN_ERROR":
				message = "<%=_T('Error doing login.')%>";
				break;
			case "EMPTY_CODE":
				message = "<%=_T('Code is empty (without comments).')%>";
				break;
			case "INVALID_ARROW":
				message = "<%=_T([[Line must have an arrow ('exten => ...')]])%>";
				break;
			case "INVALID_START":
				message = "<%=_T([[Line must starts with 'exten =>']])%>";
				break;
			case "LABEL_MISSING":
				message = "<%=_T([[Label missing trailing '(']])%>";
				break;
			case "CANT_USE_NEXT_FIRST":
				message = "<%=_T([[Can't use 'next' priority on the first entry!]])%>";
				break;
			case "CANT_USE_SAME_FIRST":
				message = "<%=_T([[Can't use 'same' priority on the first entry!]])%>";
				break;
			case "INVALID_PRIORITY_LABEL":
				message = format("<%=_T([[Invalid priority/label: {0}]])%>", error.context?.label || "");
				break;
			case "NO_CLOSING_PARENTHESIS":
				message = format("<%=_T([[No closing parenthesis found? '{0}({1}']])%>",
					error.context?.firstLabel || "", error.context?.lastLabel || "");
				break;
			case "PLEASE_USE_X":
				message = "<%=_T([[The use of '_.' for an extension is strongly discouraged and can have unexpected behavior.  Please use '_X.' instead]])%>";
				break;
			default:
				message = "<%=_T('Server error.')%> " + (error.reason || "Unknown error");
		}
		if (error.context?.index) {
			message = format("<%=_T('Invalid line {0}: {1}')%> ", error.context.index + 1, message);
		}
		this.frm.show_status_message(`${contextName}: ${message}`, "Error", 10000);
	};

	form_provisioning_gateways_section_dialplan_contexts.prototype.validate_context = function(contextGuid)
	{
		return this.validate_context_name(contextGuid) &&
			   this.validate_context_code(contextGuid) &&
			   this.validate_context_trunks(contextGuid);
	};

	form_provisioning_gateways_section_dialplan_contexts.prototype.validate_context_name = function(contextGuid)
	{
		const context = this.contexts[contextGuid];
		const contextName = context.context.Name;

		if (!inConcert.Utils.validate(contextName, "required").success)
			return this.show_validation_error("<%=_T('Name is required.')%>");

		if (!inConcert.Utils.validate(contextName, "id").success)
			return this.show_validation_error(
				"<%=_T('Only letters, numbers, dashes and dots are allowed in the Name.')%>");

		const repeated = Object.keys(this.contexts).some((otherGuid) =>
			otherGuid !== contextGuid && this.contexts[otherGuid].context.Name.toUpperCase() === contextName.toUpperCase()
		);
		if (repeated)
			return this.show_validation_error("<%=_T('Name must be unique.')%>");

		return true;
	};

	form_provisioning_gateways_section_dialplan_contexts.prototype.validate_context_code = function(contextGuid)
	{
		const context = this.contexts[contextGuid];
		if (!context.codeEditor.getSession().getValue())
			return this.show_validation_error("<%=_T('Must enter a code.')%>");

		const hasErrors = context.codeEditor.getSession().getAnnotations().some((annotation) =>
			annotation.type === "error");
		if (hasErrors)
			return this.show_validation_error("<%=_T('Code is invalid.')%>");

		return true;
	};

	form_provisioning_gateways_section_dialplan_contexts.prototype.validate_context_trunks = function(_contextGuid)
	{
		return true;
	};

	form_provisioning_gateways_section_dialplan_contexts.prototype.show_validation_error = function(message, $control)
	{
		this.frm.show_status_message(message, "Error", false);
		if (!$control) return;

		$(`#${$control.attr("id")}_validation`).show();
		$control.focus();
	};

	form_provisioning_gateways_section_dialplan_contexts.prototype.get_context_data = function(contextGuid)
	{
		const context = this.contexts[contextGuid];
		const contextData = {
			Name: context.context.Name,
			Code: context.codeEditor.getSession().getValue(),
			OutgoingTrunks: context.context.OutgoingTrunks,
			isNew: context.context.isNew || null
		};
		contextData.Id = contextData.Name;
		return contextData;
	};

	form_provisioning_gateways_section_dialplan_contexts.prototype.attach_load_request = function( _request )
	{
		
	};

	form_provisioning_gateways_section_dialplan_contexts.prototype.editable = function()
	{
		form_provisioning_gateways_section_dialplan_contexts.uber.editable.call( this );
		this.frm.editable_buttons();
		$("#frm_gateway_content_dialplan_contexts_save").show();
		$("#frm_gateway_content_dialplan_contexts_add_button").show();
		$("#frm_gateway_content_dialplan_contexts_trunks_drop_container").show();
	};

	form_provisioning_gateways_section_dialplan_contexts.prototype.readonly = function()
	{
		form_provisioning_gateways_section_dialplan_contexts.uber.readonly.call( this );
		this.frm.readonly_buttons();
		$("#frm_gateway_content_dialplan_contexts_save").hide();
		$("#frm_gateway_content_dialplan_contexts_add_button").hide();
		$("#frm_gateway_content_dialplan_contexts_trunks_drop_container").hide();
	};

	form_provisioning_gateways_section_dialplan_contexts.prototype.on_show = function()
	{
		form_provisioning_gateways_section_dialplan_contexts.uber.on_show.call(this, {show: false});
		this.initialize();
		this.frm.dashboard.$get_dom_element_by_id("form_buttons_actions").hide();
		const is_readonly = this.is_readonly();
		if (is_readonly) {
			this.readonly();
		}
		else {
			this.editable();
		}
		if (this.selectedContext)
			this.load_context_trunks(this.contexts[this.selectedContext]);
	};

	form_provisioning_gateways_section_dialplan_contexts.prototype.build_header_custom_buttons = function()
	{
		const buttons = `
			<button
				id="frm_gateway_content_dialplan_contexts_save"
				class="pageButtonGeneric buttonColorPrimary"
			>
				<%=_T([[Save contexts]])%>
			</button>
		`;
		form_provisioning_gateways_section_dialplan_contexts.uber.build_header_custom_buttons.call(this, buttons);
	};

	form_provisioning_gateways_section_dialplan_contexts.prototype.unsaved_changes = function()
	{
		return Object.keys(this.contexts).some((context) =>
			!this.contexts[context].context.isNew &&
			this.contexts[context].$toolbarContext.is(".edited"));
	};

	form_provisioning_gateways_section_dialplan_contexts.prototype.populate_toolbar = function()
	{
		const filter= $("#frm_gateway_content_dialplan_contexts_search").val();
		const unsaved = $("#frm_gateway_content_dialplan_contexts_unsaved_tab").hasClass("selected");
		const $items = Object.keys(this.contexts).filter((contextGuid) =>
				(!unsaved || this.contexts[contextGuid].$toolbarContext.is(".edited")) &&
				this.contexts[contextGuid].context.Name &&
				this.contexts[contextGuid].context.Name.indexOf(filter) !== -1
			).sort(this.sort_condition()).map((contextGuid) => this.contexts[contextGuid].$toolbarContext);
		$("#frm_gateway_content_dialplan_contexts_toolbar_empty").toggle($items.length === 0);
		const $list = $("#frm_gateway_content_dialplan_contexts_toolbar_list");
		Object.keys(this.contexts).forEach(
			(contextGuid) => this.contexts[contextGuid].$toolbarContext.detach());
		$items.forEach(($item) => $item.appendTo($list));
	};

	form_provisioning_gateways_section_dialplan_contexts.prototype.refresh_footer = function()
	{
		const checked = Object.keys(this.contexts).filter((contextGuid) =>
			this.contexts[contextGuid].$toolbarContext.find("[toolbar_element_check]").is(":checked")
		).sort(this.sort_condition());
		const $footer = $("#frm_gateway_content_dialplan_contexts_footer_actions");
		if (!checked.length) {
			$footer.hide();
			return;
		}

		$footer.find("[footer_counter]").text(checked.length);
		$footer.find("[footer_cancel]").off("click").on("click", () => {
			checked.forEach((contextGuid) => {
				const $toolbarContext = this.contexts[contextGuid].$toolbarContext;
				$toolbarContext.removeClass("selectedToEdit");
				$toolbarContext.find("[toolbar_element_check]").prop("checked", false);
			});
			this.refresh_footer();
		});
		$footer.find("[footer_delete]").off("click").on("click", () => this.remove_contexts(checked));
		$footer.show();
	};

	form_provisioning_gateways_section_dialplan_contexts.prototype.populate_counters = function()
	{
		const all = Object.keys(this.contexts);
		const unsaved = all.filter((contextGuid) => this.contexts[contextGuid].$toolbarContext.is(".edited"));
		$("#frm_gateway_content_dialplan_contexts_all_tab").text(format(
			"<%=_T('All ({0})')%>", all.length
		));
		$("#frm_gateway_content_dialplan_contexts_unsaved_tab").text(format(
			"<%=_T('Unsaved ({0})')%>", unsaved.length
		));
	};

	form_provisioning_gateways_section_dialplan_contexts.prototype.sort_condition = function()
	{
		return (context1, context2) =>
			this.contexts[context1].context.Name.localeCompare(this.contexts[context2].context.Name);
	};
}());
