/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 11 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";
	const FORMAT = inConcert.Utils.format;
	const _T = inConcert.i18n.getString;

	//---------------------------------------------------------------------
	// form trunks class
	function form_provisioning_trunks(params)
	{
		const success = form_provisioning_trunks.uber.constructor.call(this, params);
		if (!success) {
			return;
		}

		this.new_title = "<%=_T('New IP Trunk')%>";
		this.update_title = "<%=_T('Update IP Trunk {0}')%>";
		this.view_title = "<%=_T('View IP Trunk {0}')%>";
		this.tagline = "<%=_T('Tagline form Trunk!!!')%>";

		this.html_file_name = "/inconcert/apps/provisioning/trunks/frm/render/";
		this.save_uri =       "/inconcert/apps/provisioning/trunks/save/";
		this.get_uri =        "/inconcert/apps/provisioning/trunks/get/";

		this.telephone_codes = inConcert.app.getInstanceTelephonyCodes();
		this.sorted_countries = null;

		this.codecs_mng = new inConcert.ProvisioningCodecsMng({
			owner       : this,
			prefix      : "prov_form_trunk",
			container_id: "prov_form_trunk_codecs_panel",
			fields_per_row: 7,
			codecs_list: [
				{id: "alaw" , name: "alaw" },
				{id: "ulaw" , name: "ulaw" },
				{id: "g729" , name: "g729" },
				{id: "speex", name: "speex"},
				{id: "vp8"  , name: "vp8"  }
			]
		});

		this.amd_mng = new inConcert.ProvisioningAmdMng({
		 	owner       : this,
		 	prefix      : "prov_form_trunk_amd",
		 	container_id: "prov_form_trunk_amd_panel",
		});

		this.fails_detection_mng = new inConcert.ProvisioningFailsDetectionMng({
			owner       : this,
			prefix      : "prov_form_trunk_fails_detection",
			container_id: "prov_form_trunk_fails_detection_panel",
		});

		this.tech_specific_mng = new inConcert.ProvisioningTechSpecificMng({
			owner       : this,
			prefix      : "prov_form_trunk_tech_specific_data",
			container_id: "prov_form_trunk_tech_specific_data_panel"
		});

		this.schema_mng = new inConcert.ProvisioningDialingSchemaMng({
			owner       : this,
			owner_name  : "<%=_T('Trunk')%>",
			prefix      : "prov_form_trunk",
			container_id: "prov_form_trunk_schema_panel"
		});

		return true;
	}

	inConcert.Utils.inherit(form_provisioning_trunks, inConcert.conf_form);

	if (typeof inConcert.form_provisioning_trunks !== "function") {
		inConcert.form_provisioning_trunks = form_provisioning_trunks;
	}

	form_provisioning_trunks.prototype.initialize = function()
	{
		this.codecs_mng.initialize();
		this.amd_mng.initialize();
		this.fails_detection_mng.initialize();
		this.tech_specific_mng.initialize();
		this.schema_mng.initialize();
	};

	form_provisioning_trunks.prototype.get = function()
	{
		
		let trunk = {
			id            : $("#prov_form_trunk_id").val(),
			name          : $("#prov_form_trunk_name").val(),
			carrier       : $("#prov_form_trunk_carrier").val(),
			countryCode   : $("#prov_form_trunk_country_code").val(),
			areaCode      : $("#prov_form_trunk_area_code").val(),
			tech          : "SIP",
			host          : $("#prov_form_trunk_ip").val(),
			hostId        : $("#prov_form_trunk_host").val(),
			callsPerSecond: parseInt($("#prov_form_trunk_dialer_out_calls_per_second").val(), 10),
			registrable   : "no"
		};
		if ($("#prov_form_trunk_technology").val() === "GATEWAY") {
			trunk.gateway = $("#prov_form_trunk_gateway").val();
			trunk.gatewayDialplan = $("#prov_form_trunk_gateway_dialplan").val();
		} else {
			trunk.hostId = $("#prov_form_trunk_host").val();
			const port = $("#prov_form_trunk_port").val();
			if (port.length > 0)
				trunk.port = port;
		}
		// registration host
		if ($("#prov_form_trunk_register_host_ip").prop("checked")) {
			trunk.registrable = "yes";
			trunk.registrationUser = $("#prov_form_trunk_register_username").val();
			trunk.registrationExten = $("#prov_form_trunk_register_extension").val();
		}
		// optionals
		const desc = $("#prov_form_trunk_description").val();
		if (desc.length > 0) {
			trunk.description = desc;
		}
		const capacity = $("#prov_form_trunk_capacity_limit").val();
		if (capacity.length > 0) {
			trunk.capacity = capacity;
		}
		// optionals codecs
		this.codecs_mng.get(trunk);
		// optionals amd
		this.amd_mng.get(trunk);
		// optionals fails detection
		this.fails_detection_mng.get(trunk);
		// password
		if ($("#prov_form_trunk_config_pwd").prop("checked")) {
			trunk.password = $("#prov_form_trunk_pwd").val();
			trunk.authRealm = $("#prov_form_trunk_auth_realm").val();
		}
		// tech specific data
		this.tech_specific_mng.get(trunk);
		// schema
		this.schema_mng.get(trunk);

		const action = this.dashboard.get_current_action_obj();
		const id = action.view.row_id;
		if (!this.is_new) {
			trunk[id] = this.$current_row.attr("row_id");
		}
		return trunk;
	};

	form_provisioning_trunks.prototype.current_gateway_dialplan = function(trunk)
	{
		let gatewayTrunk = this.gateways[trunk.gateway || ""]?.nodeTrunks[trunk.id];
		if (!gatewayTrunk) return "";

		if (typeof(gatewayTrunk) === "string") gatewayTrunk = JSON.parse(gatewayTrunk);
		return gatewayTrunk.DialplanContext || "";
	};

	form_provisioning_trunks.prototype.set = function(success, trunk)
	{
		if (!success) {
			return;
		}
		$("#prov_form_trunk_id").val(trunk.id);
		$("#prov_form_trunk_name").val(trunk.name);
		$("#prov_form_trunk_carrier").val(trunk.carrier);
		$("#prov_form_trunk_technology").val(trunk.gateway ? "GATEWAY" : "SIP");
		this.toggle_trunk_technology();
		$("#prov_form_trunk_ip").val(trunk.host);
		$("#prov_form_trunk_host").val(trunk.hostId.toLowerCase());
		$("#prov_form_trunk_gateway").val(trunk.gateway);
		this.__fill_gateway_dialplans(trunk.gateway);
		this.__fill_combo_areas(trunk.countryCode);
		$("#prov_form_trunk_gateway_dialplan").val(this.current_gateway_dialplan(trunk));
		$("#prov_form_trunk_dialer_out_calls_per_second").val(trunk.callsPerSecond);
		$("#prov_form_trunk_country_code").val(trunk.countryCode);
		this.__fill_combo_areas(trunk.countryCode);
		$("#prov_form_trunk_area_code").val(trunk.areaCode);

		// registration host
		if ((trunk.registrable || "").toLowerCase() === "yes") {
			$("#prov_form_trunk_register_host_ip").prop("checked", true);
			$("#prov_form_trunk_register_username").val(trunk.registrationUser);
			$("#prov_form_trunk_register_extension").val(trunk.registrationExten);
		}
		this.toggle_register_info();

		// optionals
		if (trunk.description) {
			$("#prov_form_trunk_description").val(trunk.description);
		}
		if (trunk.port) {
			$("#prov_form_trunk_port").val(trunk.port);
		}
		if (trunk.capacity) {
			$("#prov_form_trunk_capacity_limit").val(trunk.capacity);
		}
		// optionals codecs
		this.codecs_mng.set(trunk);
		// optionals amd
		this.amd_mng.set(trunk);
		// optionals fails detection
		this.fails_detection_mng.set(trunk);
		// password
		if (trunk.password) {
			$("#prov_form_trunk_config_pwd").prop("checked", true);
			$("#prov_form_trunk_pwd").val(trunk.password);
			$("#prov_form_trunk_auth_realm").val(trunk.authRealm);
		}
		this.toggle_password_info();
		// tech specific data
		this.tech_specific_mng.set(trunk);
		// schema
		this.schema_mng.set(trunk);
	};

	form_provisioning_trunks.prototype.empty_controls = function()
	{
		// basic info
		$("#prov_form_trunk_id").val("").focus();
		$("#prov_form_trunk_name").val("");
		$("#prov_form_trunk_carrier").val("");
		$("#prov_form_trunk_technology").val("SIP");
		this.toggle_trunk_technology();
		$("#prov_form_trunk_ip").val("");
		$("#prov_form_trunk_host").val("");
		$("#prov_form_trunk_gateway").val("");
		$("#prov_form_trunk_gateway_dialplan").val("");
		this.__fill_gateway_dialplans();
		$("#prov_form_trunk_dialer_out_calls_per_second").val("");
		// register info
		$("#prov_form_trunk_register_host_ip").prop("checked", false);
		$("#prov_form_trunk_register_username").val("");
		$("#prov_form_trunk_register_extension").val("");
		this.toggle_register_info();
		// geographic information
		$("#prov_form_trunk_country_code").val("").trigger("change");
		$("#prov_form_trunk_area_code").val("");
		// optionals
		$("#prov_form_trunk_description").val("");
		$("#prov_form_trunk_port").val("");
		$("#prov_form_trunk_capacity_limit").val("");
		// password info
		$("#prov_form_trunk_config_pwd").prop("checked", false);
		$("#prov_form_trunk_pwd").val("");
		$("#prov_form_trunk_auth_realm").val("");
		this.toggle_password_info();
		// codecs
		this.codecs_mng.empty_controls();
		// amd
		this.amd_mng.empty_controls();
		// fails detection
		this.fails_detection_mng.empty_controls();
		// tech specific data
		this.tech_specific_mng.empty_controls();
		// schema
		this.schema_mng.empty_controls();
	};

	form_provisioning_trunks.prototype.err_save_messages = function(err, is_custom_message)
	{
		var get_error_description = function(err_type, raw_data = {})
		{
			is_custom_message = true;
			let message;
			switch (err_type)
			{
				case "TRUNK MUST SPECIFY AN EXISTING CARRIER":
					message = "<%=_T('Trunk must specify an existing carrier.')%>";
				break;

				case "CARRIER DOES NOT EXIST":
					message = "<%=_T('Carrier does not exist.')%>";
				break;

				case "FIELD HOSTID MUST BE SPECIFIED":
					message = "<%=_T('Field hostId must be specified.')%>";
				break;

				case "HOST DOES NOT EXIST":
					message = "<%=_T('Host does not exist.')%>";
				break;

				case "CANNOT CHANGE TRUNK CARRIER WHILE DID IS STILL ASSOCIATED WITH IT":
					message = FORMAT("<%=_T('Cannot change trunk {0} carrier while DID {1} is still associated with it.')%>",
						(raw_data.trunk || ""),
						(raw_data.did || "")
					);
				break;

				case "CANNOT CHANGE TRUNK CARRIER WHILE IT BELONGS TO GROUP":
					message = FORMAT("<%=_T('Cannot change trunk {0} carrier while it belongs to group {1}.')%>",
						(raw_data.trunk || ""),
						(raw_data.currentGroup || "")
					);
				break;

				default:
					is_custom_message = false;
					message = "<%=_T('Server error!!')%>";
			}
			return message;
		};

		switch (typeof err)
		{
			case "object":
				const error_type = err.result.toUpperCase();
				if (error_type === "MUST_HAVE_A_NAME" || error_type === "MUST_HAVE_UNIQUE_NAME") {
					$("#prov_form_trunk_id").focus();
				}
				else {
					err = get_error_description(error_type, err.rawData);
				}
			break;

			case "string":
				err = get_error_description(err.toUpperCase());
			break;
		}

		form_provisioning_trunks.uber.err_save_messages.call(this, err, is_custom_message); //invokes the base class method

	};

	form_provisioning_trunks.prototype.get_validation_def = function()
	{
		const can_validate_register_info = function() {
			const $register_host_ip = $("#prov_form_trunk_register_host_ip");
			return $register_host_ip.prop("checked");
		};
		const can_validate_password_info = function() {
			const $pwd = $("#prov_form_trunk_config_pwd");
			return $pwd.prop("checked");
		};
		
		const definitions = [
			{
				control_id: "prov_form_trunk_id",
				messages: {
					required: "<%=_T('Please enter the Trunk Id.')%>"
				}
			},
			{
				control_id: "prov_form_trunk_name",
				messages: {
					required: "<%=_T('Please enter the Trunk Name.')%>",
					id:       "<%=_T('Only letters, numbers, dashes and dots are allowed in the Trunk Name.')%>"
				}
			},
			{
				control_id: "prov_form_trunk_carrier",
				messages: {
					required: "<%=_T('Please select Carrier.')%>"
				}
			},
			{
				control_id: "prov_form_trunk_host",
				messages: {
					required: "<%=_T('Please select Host.')%>"
				}
			},
			{
				control_id: "prov_form_trunk_technology",
				messages: {
					required: "<%=_T('Please select Technology.')%>"
				}
			},
			{
				control_id: "prov_form_trunk_ip",
				can_validate: function() {
					return $("#prov_form_trunk_technology").val() !== "GATEWAY";
				},
				messages: {
					required: "<%=_T('Please enter Trunk IP/Host/Domain.')%>"
				}
			},
			{
				control_id: "prov_form_trunk_port",
				messages: {
					custom: {
						type : "custom",
						valid: function() {
							let response = {
								success: true
							};
							const value =  $("#prov_form_trunk_port").val();
							if (value !== "" && (!inConcert.Utils.validate(value, "required").success || !inConcert.Utils.validate(value, "number").success)) {
								response.success = false;
								response.message = "<%=_T('Only numbers are allowed in trunk port.')%>";
							}
							return response;
						}
					}
				}
			},
			{
				control_id: "prov_form_trunk_capacity_limit",
				messages: {
					custom: {
						type : "custom",
						valid: function() {
							let response = {
								success: true
							};
							const value =  $("#prov_form_trunk_capacity_limit").val();
							if (value !== "" && (!inConcert.Utils.validate(value, "required").success || !inConcert.Utils.validate(value, "number").success)) {
								response.success = false;
								response.message = "<%=_T('Only numbers are allowed in trunk capacity limit.')%>";
							}
							return response;
						}
					}
				}
			},
			{
				control_id: "prov_form_trunk_gateway",
				can_validate: function() {
					return $("#prov_form_trunk_technology").val() === "GATEWAY";
				},
				messages: {
					required: "<%=_T('Please enter Trunk gateway.')%>"
				}
			},
			{
				control_id: "prov_form_trunk_gateway_dialplan",
				can_validate: function() {
					return $("#prov_form_trunk_technology").val() === "GATEWAY";
				},
				messages: {
					required: "<%=_T('Please enter Trunk dialplan.')%>"
				}
			},
			{
				control_id: `prov_form_trunk_dialer_out_calls_per_second`,
				messages: {
					required: "<%=_T('Please enter outgoing calls per second.')%>",
					range: {
						type   : "range",
						between: {from: 1, to: 3000},
						message: "<%=_T('Outgoing calls per second is out of range. Please enter a value between {0} and {1}.')%>"
					}
				}
			},
			{
				control_id: "prov_form_trunk_register_username",
				can_validate: function() {
					return can_validate_register_info();
				},
				messages: {
					required: "<%=_T('Please enter Register User Name.')%>"
				}
			},
			{
				control_id: "prov_form_trunk_pwd",
				can_validate: function() {
					return can_validate_password_info();
				},
				messages: {
					required: "<%=_T('Please enter Trunk Password.')%>"
				}
			}
		];
		return [
			...definitions,
			...this.amd_mng.get_validation_def(),
			...this.fails_detection_mng.get_validation_def(),
		];
	};

	form_provisioning_trunks.prototype.ready = function()
	{
		form_provisioning_trunks.uber.ready.call(this); //invokes the base class method
		this.only_modified();
	};

	form_provisioning_trunks.prototype.only_modified = function()
	{
		const $id = $("#prov_form_trunk_id");
		if (this.is_new) {
			this.tech_specific_mng.set(); // tengo que iniciarlo en vacio
			this.readonly_control($id, false /* disabled */ , true /* is_required */);
			$id.focus();
		}
		else {
			this.readonly_control($id, true /* disabled */);
			$("#prov_form_trunk_name").focus();
		}
		// register info
		this.toggle_register_info();
		// password info
		this.toggle_password_info();
	};

	form_provisioning_trunks.prototype.set_event_scroll = function(on_scroll = $.noop)
	{
		const self = this;
		let on_scroll_trunks = function() {
			
			if (self.amd_mng && typeof self.amd_mng.on_scroll === "function") {
				self.amd_mng.on_scroll();
			}
			on_scroll();
		}
		form_provisioning_trunks.uber.set_event_scroll.call(this, on_scroll_trunks);
	};

	form_provisioning_trunks.prototype.set_events = function()
	{
		const self = this;
		form_provisioning_trunks.uber.set_events.call(this); //invokes the base class method

		const $country = $("#prov_form_trunk_country_code");
		$country.change(function() {
			const country_code = $(this).val();
			window.setTimeout(function() {
				self.__fill_combo_areas(country_code);
			}, 50);
		});

		$("#prov_form_trunk_register_host_ip").off("change")
			.on("change", function() {
				self.toggle_register_info();
			});

		$("#prov_form_trunk_config_pwd").off("change")
			.on("change", function() {
				self.toggle_password_info();
			});

		$("#prov_form_trunk_technology").off("change")
			.on("change", function() {
				self.toggle_trunk_technology();
			});

		$("#prov_form_trunk_gateway").off("change")
			.on("change", function() {
				self.__fill_gateway_dialplans($(this).val());
			});
		// tech specific
		this.tech_specific_mng.set_events();
		// schema
		this.schema_mng.set_events();
	};

	form_provisioning_trunks.prototype.toggle_register_info = function()
	{
		const must_configure = $("#prov_form_trunk_register_host_ip").prop("checked");
		const $register_user_name = $("#prov_form_trunk_register_username");
		const $register_user_extension = $("#prov_form_trunk_register_extension");

		if (must_configure) {
			this.disable_control($register_user_name, false, true);
			this.disable_control($register_user_extension, false, false);
			return;
		}
		this.disable_control($register_user_name, true);
		this.disable_control($register_user_extension, true);
	};

	form_provisioning_trunks.prototype.toggle_password_info = function()
	{
		const must_configure = $("#prov_form_trunk_config_pwd").prop("checked");
		const $pwd = $("#prov_form_trunk_pwd");
		const $auth_realm = $("#prov_form_trunk_auth_realm");

		if (must_configure) {
			this.disable_control($pwd, false, true);
			this.disable_control($auth_realm, false, false);
			return;
		}
		this.disable_control($pwd, true);
		this.disable_control($auth_realm, true);
	};

	form_provisioning_trunks.prototype.toggle_trunk_technology = function()
	{
		const isGateway = $("#prov_form_trunk_technology").val() === "GATEWAY";
		$("#prov_form_trunk_gateway_div").toggle(isGateway);
		$("#prov_form_trunk_sip_div").toggle(!isGateway);
	};

	form_provisioning_trunks.prototype.validate = function(save_callback, validation_def, tiemout_ms, $custom_message_container)
	{
		return form_provisioning_trunks.uber.validate.call(this, save_callback, validation_def, tiemout_ms, $custom_message_container);
	};

	form_provisioning_trunks.prototype.ready_for_update = function()
	{
		form_provisioning_trunks.uber.ready_for_update.call(this);
		// schema
		this.schema_mng.toggle_ctrls();
	};

	form_provisioning_trunks.prototype.load_data = function(callback_complete_requests)
	{
		const self = this;
		const provisioning_api = inConcert.app.getProvisioningAPI();

		this.__fill_technology([
			{id: "SIP", desc: "<%=_T('SIP')%>"},
			{id: "GATEWAY", desc: "<%=_T('Gateway')%>"}
		]);

		this.requests = [
			// Carriers
			{
				request_function: function(success) {
					return provisioning_api.carriersLoadAll(success);
				},
				success: function(list) {
					self.__fill_carriers(list);
				}
			},
			// Hosts
			{
				request_function: function(success) {
					return provisioning_api.hostsLoadAll(success);
				},
				success: function(list) {
					self.__fill_hosts(list);
				}
			},
			// Countries
			{
				request_function: function(success) {
					return self.telephone_codes.loadCountries(success);
				},
				success: function(countries) {
					self.__fill_countries(countries);
				}
			},
			// trunks amd params
			{
				request_function: function(success) {
					return provisioning_api.trunksAmdLoadAll(success);
				},
				success: function(response) {
					let trunks_amd_list = response && response.rows || [];
					self.amd_mng.setAllTrunksAmd(trunks_amd_list);
				}
			},
			// Gateways
			{
				request_function: function(success) {
					return provisioning_api.gatewaysLoadAll(success);
				},
				success: function(list) {
					self.gateways = list;
					self.__fill_gateways(Object.keys(list).map((id) => ({ Id: id })));
					self.__fill_gateway_dialplans();
				}
			},
		];
		
		form_provisioning_trunks.uber.load_data.call(this, callback_complete_requests); //invokes the base class method
	};
	

	form_provisioning_trunks.prototype.__fill_carriers = function(list)
	{
		const $combo = $("#prov_form_trunk_carrier");
		this.fill_combo($combo, true /* is_blank_first_item */, list, "id" /* attr_id */, "id" /* attr_name */, true /* sorted */, undefined /*can_add_item*/, "carrier", /* data_id */);
	};

	form_provisioning_trunks.prototype.__fill_gateways = function(list)
	{
		const $combo = $("#prov_form_trunk_gateway");
		this.fill_combo($combo, true /* is_blank_first_item */, list, "Id" /* attr_id */, "Id" /* attr_name */, true /* sorted */, undefined /*can_add_item*/, "gateway", /* data_id */);
	};

	form_provisioning_trunks.prototype.__fill_gateway_dialplans = function(gatewayId = "")
	{
		const dialplans = Object.keys(this.gateways[gatewayId]?.dialplanContexts || {}).map((id) => ({ Id: id }));

		const $combo = $("#prov_form_trunk_gateway_dialplan");
		this.fill_combo($combo, true /* is_blank_first_item */, dialplans, "Id" /* attr_id */, "Id" /* attr_name */, true /* sorted */, undefined /*can_add_item*/, "dialplan", /* data_id */);
	};

	form_provisioning_trunks.prototype.__fill_hosts = function(list)
	{
		const $combo = $("#prov_form_trunk_host");
		this.fill_combo($combo,
			true /* is_blank_first_item */,
			list,
			"hostId" /* attr_id */,
			"hostId" /* attr_name */,
			true /* sorted */,
			undefined /*can_add_item*/,
			"host", /* data_id */
			true /* id_is_case_sensitive */
		);
	};

	form_provisioning_trunks.prototype.__fill_technology = function(list)
	{
		const $combo = $("#prov_form_trunk_technology");
		this.fill_combo($combo, true /* is_blank_first_item */, list, "id" /* attr_id */, "desc" /* attr_name */, false /* sorted */, undefined /*can_add_item*/, "trunk_type", /* data_id */);
	};

	form_provisioning_trunks.prototype.__fill_countries = function(countries)
	{
		const self = this;

		var build_sorted_countries = function( countries_map ) {
			if ($.isArray(self.sorted_countries)) {
				return self.sorted_countries;
			}
			self.sorted_countries = [];
			$.each(countries_map, function(country_id, country) {
				self.sorted_countries.push( { code: country_id, name: _T(country.name) });
			});
			return inConcert.Utils.sortArray(self.sorted_countries, "name");
		};

		const $combo = $("#prov_form_trunk_country_code");
		$combo.html("");
		this.combo_add_value($combo, "", "", true);

		let sorted_countries = build_sorted_countries( countries );
		$.each( sorted_countries, function(_, country) {
			self.combo_add_value($combo, country.code, country.name, false);
		});
	};

	form_provisioning_trunks.prototype.__fill_combo_areas = function(country_code)
	{
		let areas = this.telephone_codes.getAreasByCode(country_code);
		const $area = $("#prov_form_trunk_area_code");
		if ( !areas ) {
			$area.html("");
			return;
		}

		const attr_name = "name";
		const attr_id = "code";
		/*
			En la lista de "areas" hay que manejar diferente al primer elemento "blank" del combo.
			Esto porque en TODOS los combos que utilizamos el elemento "blank" tiene como "value" del <option> el valor "" (vacio).
			Esto funciona bien en todos lados, menos con los "code areas", ya que aca existe un elemento con Id "" que es el "Fixed Lines".
			Esto hace que el combo tenga 2 items con el mismo Id, y aqui el error.
			Para solucionar esto, al elemento "blank" de este combo le ingresamos una palabra clave "__BLANK" que se asignara en el atributo "value" del <option>
		*/
		var is_blank_first_item = function() {
			return {
				id: "__BLANK",
				name: ""
			};
		};
		this.fill_combo($area, is_blank_first_item, areas, attr_id, attr_name);
	};

}());