/* jshint smarttabs:true, quotmark:false, esversion: 11 */
/* global $, inConcert */


if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	"use strict";
	const FORMAT = inConcert.Utils.format;
	const REQUEST_TIMEOUT_MS = 1800000; // 30 minutes

	function API (params)
	{
		if(typeof(params) !== "object") {
			throw("API.Constructor - expected an object with configuration");
		}
		if(typeof(params.uri ) !== "string") {
			throw("API.Constructor - base uri is required");
		}

		this.uri = params.uri + "/";
	}

	if (typeof inConcert.ProvisioningAPI !== "function") {
		inConcert.ProvisioningAPI = API;
	}

/********************************************************************************************************
	FUNCIONES GENERICAS
********************************************************************************************************/

	/*
	*/
	API.prototype.__Get = function(uri, callback)
	{
		if (typeof(uri) !== "string") {
			throw("API.prototype.__Get: uri variable must be string type.");
		}
		if (typeof(callback) !== "function") {
			throw("API.prototype.__Get: callback variable must be function type.");
		}

		let request_handler = inConcert.invoke({
			url    : uri,
			type   : "GET",
			timeout: REQUEST_TIMEOUT_MS,
			success: function (list, result, server_response)
			{
				if (typeof(server_response) === "object") {
					server_response = JSON.stringify(server_response);
				}
				
				if (result.toLowerCase() === "success") {
					callback(true, list);
				}
				else {
					callback(false, server_response);
				}
			},
			error  : function (code, error) {callback(false, error);}
		});
		return request_handler;
	};

	/*
	*/
	API.prototype.__Post = function(uri, callback, data)
	{
		if (typeof(uri) !== "string") {
			throw("API.prototype.__Post: uri variable must be string type.");
		}
		if (typeof(callback) !== "function") {
			throw("API.prototype.__Post: callback variable must be function type.");
		}
		if (typeof(data) !== "object") {
			throw("API.prototype.__Post: data variable must be object type.");
		}

		let request_handler = inConcert.invoke({
			url         : uri,
			type        : "POST",
			data        : JSON.stringify(data),
			dataType    : "json",
			contentType : "application/json",
			timeout     : REQUEST_TIMEOUT_MS,
			success     : function (response)    { callback(true, response); },
			error       : function (code, error) { callback(false, error);   }
		});
		return request_handler;
	};

	API.prototype.getRequestTimeout = function() { return REQUEST_TIMEOUT_MS; };

	/*
	*/
	API.prototype.asyncCall = async function(method, ...args)
	{
		return inConcert.Utils.asyncCallObjectMethod(this, method, ...args);
	};

/********************************************************************************************************
	VCCs - BASIC
********************************************************************************************************/
	API.prototype.saveVcc = function(data, callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/vccs/save/
		*/
		let uri = FORMAT("{0}vccs/save/", this.uri);
		let request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

/********************************************************************************************************
	VCCs - Configurations limits
********************************************************************************************************/
	API.prototype.getMaxNumberConfigsAvailable = function(callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/vccs/get_max_configs_available/vcc
		*/
		let uri = FORMAT("{0}vccs/get_max_configs_available/", this.uri);
		let request_handler = this.__Get(uri, callback);
		return request_handler;
	};
/********************************************************************************************************
	VCCs - BOT RASA
********************************************************************************************************/

	API.prototype.rasaUpdateVCC = function(data, callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/vccs/cognitive_services/rasa/update/
		*/
		let uri = FORMAT("{0}vccs/cognitive_services/rasa/update/", this.uri);
		let request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	API.prototype.rasaImportLuisApp = function(data, callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/vccs/cognitive_services/rasa/import/luis/app/
		*/
		let uri = FORMAT("{0}vccs/cognitive_services/rasa/import/luis/app/", this.uri);
		let request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	API.prototype.rasaLoadUnimportedLuisApps = function(data, callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/vccs/cognitive_services/rasa/import/luis/load_unimported_apps/
		*/
		let uri = FORMAT("{0}vccs/cognitive_services/rasa/import/luis/load_unimported_apps/", this.uri);
		let request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

/********************************************************************************************************
	VCCs - BOT LUIS
********************************************************************************************************/

	API.prototype.luisUpdateVCC = function(data, callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/vccs/cognitive_services/luis/update/
		*/
		let uri = FORMAT("{0}vccs/cognitive_services/luis/update/", this.uri);
		let request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

/********************************************************************************************************
	VCCs - LLM
********************************************************************************************************/

API.prototype.llmUpdateVCC = function(data, callback = $.noop)
{
	/*
	* sample URI:
	*   http://i6.inconcert.cc/inconcert/apps/provisioning/vccs/cognitive_services/llm/update/
	*/
	let uri = FORMAT("{0}vccs/cognitive_services/llm/update/", this.uri);
	let request_handler = this.__Post(uri, callback, data);
	return request_handler;
};

/********************************************************************************************************
	VCCs - QNA MAKER
********************************************************************************************************/

	API.prototype.qnaUpdateVCC = function(data, callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/vccs/cognitive_services/qna/update/
		*/
		let uri = FORMAT("{0}vccs/cognitive_services/qna/update/", this.uri);
		let request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

/********************************************************************************************************
	VCCs - PEERs
********************************************************************************************************/
	API.prototype.vccPeersUpdate = function(data, callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/vccs/peers/update/
		*/
		let uri = FORMAT("{0}vccs/peers/update/", this.uri);
		let request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	API.prototype.vccPeersLoadAll = function(vcc_id, callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/vccs/peers/load/vcc_id/
		*/
		let uri = FORMAT("{0}vccs/peers/load/{1}/", this.uri, vcc_id || " ");

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

/********************************************************************************************************
	VCCs - PACKAGES
********************************************************************************************************/
	API.prototype.vccPackagesUpdate = function(data, callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/vccs/packages/update/
		*/
		let uri = FORMAT("{0}vccs/packages/update/", this.uri);
		let request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	API.prototype.vccPackagesLoadAll = function(vcc_id, callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/vccs/packages/load/vcc_id/
		*/
		let uri = FORMAT("{0}vccs/packages/load/{1}/", this.uri, vcc_id || " ");

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

/********************************************************************************************************
	VCCs - TTS/ASR
********************************************************************************************************/
	API.prototype.vccTtsAsrUpdate = function(data, callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/vccs/ttsasr/update/
		*/
		let uri = FORMAT("{0}vccs/ttsasr/update/", this.uri);
		let request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

/********************************************************************************************************
	VCCs - Accounts
********************************************************************************************************/

	API.prototype.vccAccountsLoadAll = function(vcc_id, callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/vccs/accounts/load/vcc_id/
		*/
		let uri = FORMAT("{0}vccs/accounts/load/{1}/", this.uri, vcc_id || " ");

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

/********************************************************************************************************
VCCs - Campaigns
********************************************************************************************************/

	API.prototype.vccCampaignsLoadAll = function(vcc_id, callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/vccs/campaigns/load/vcc_id/
		*/
		let uri = FORMAT("{0}vccs/campaigns/load/{1}/", this.uri, vcc_id || " ");

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

/********************************************************************************************************
	VCCs - User's Groups
********************************************************************************************************/

	API.prototype.vccGroupsLoadAll = function(vcc_id, callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/vccs/groups/load/vcc_id/
		*/
		let uri = FORMAT("{0}vccs/groups/load/{1}/", this.uri, vcc_id || " ");
		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

/********************************************************************************************************
VCCs - Recording Configuration
********************************************************************************************************/

	API.prototype.vccLoadDefaultsRecordingConfiguration = function(callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/vccs/recording_configuration/load_defaults/
		*/
		let uri = FORMAT("{0}vccs/recording_configuration/load_defaults/", this.uri);

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

/********************************************************************************************************
VCCs SUPER USERS
********************************************************************************************************/

	API.prototype.superUserSave = function(data, callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/vccs/super_users/save/
		*/
		const uri = FORMAT("{0}vccs/super_users/save/", this.uri);
		return this.__Post(uri, callback, data);
	};

	API.prototype.superUsersSearch = function(data, callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/vccs/super_users/search/
		*/
		const uri = FORMAT("{0}vccs/super_users/search/", this.uri);
		return this.__Post(uri, callback, data);
	};

/********************************************************************************************************
	CARRIERS
********************************************************************************************************/
	API.prototype.carriersLoadAll = function(callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/carriers/load/
		*/
		let uri = FORMAT("{0}carriers/load/", this.uri);

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

	API.prototype.carriersLoadWithTrunks = function(callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/carriers/load/with_trunks/
		*/
		let uri = FORMAT("{0}carriers/load/with_trunks/", this.uri);

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

/********************************************************************************************************
	DIDs
********************************************************************************************************/
	API.prototype.didsLoadAll = function(callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/dids/load/
		*/
		let uri = FORMAT("{0}dids/load/", this.uri);

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

/********************************************************************************************************
	HOSTS
********************************************************************************************************/
	API.prototype.hostsLoadAll = function(callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/hosts/load/
		*/
		let uri = FORMAT("{0}hosts/load/", this.uri);

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

/********************************************************************************************************
	TRUNKS
********************************************************************************************************/
	API.prototype.trunksLoadAll = function(callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/trunks/load/
		*/
		let uri = FORMAT("{0}trunks/load/", this.uri);

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

	API.prototype.trunksLoadByCarriers = function(callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/trunks/load/by_carriers/
		*/
		let uri = FORMAT("{0}trunks/load/by_carriers/", this.uri);

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

	API.prototype.trunksAmdLoadAll = function(callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/trunks/search/
		*/
		let uri = FORMAT("{0}trunks/search/", this.uri);

		let request_handler = this.__Post(uri, function(success, list) {
			if (success) {
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		}, {columns: ["id", "amdParams"]});
		return request_handler;
	};
	
	API.prototype.trunkAmdStartTest = function(trunkId, data, callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/trunks/amd_start_test/trunkId
		*/
		const uri = `${this.uri}trunks/amd_start_test/${encodeURIComponent(trunkId)}`;

		return this.__Post(uri, function(success, list) {
			if (success) {
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		}, data);
	};

	API.prototype.trunkAmdResults = function(trunkId, callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/trunks/amd_results/trunkId
		*/
		const uri = `${this.uri}trunks/amd_results/${encodeURIComponent(trunkId)}`;

		return this.__Get(uri, function(success, list) {
			if (success) {
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
	};
	
	API.prototype.trunkResetFailsMonitor = function(trunkId, callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/trunks/amd_start_test/trunkId
		*/
		const uri = `${this.uri}trunks/reset_fails_monitor/${encodeURIComponent(trunkId)}`;

		return this.__Post(uri, function(success, list) {
			if (success) {
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		}, {});
	};

/********************************************************************************************************
	TRUNKS GROUPS
********************************************************************************************************/
	API.prototype.trunksGroupsLoadAll = function(callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/trunksgroups/load_all/
		*/
		let uri = FORMAT("{0}trunksgroups/load/", this.uri);

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

/********************************************************************************************************
	Routes
********************************************************************************************************/
	API.prototype.routesLoadAll = function(callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/routes/load_all/
		*/
		let uri = FORMAT("{0}routes/load/", this.uri);

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

	API.prototype.routesLoadTrunksPriorityList = function(callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/routes/load/list/trunks_priority/
		*/
		let uri = FORMAT("{0}routes/load/list/trunks_priority/", this.uri);

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

/********************************************************************************************************
	PEERs
********************************************************************************************************/
	API.prototype.peersLoadAll = function(callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/peers/load/
		*/
		let uri = FORMAT("{0}peers/load/", this.uri);

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

	API.prototype.peersAddRange = function(data, callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/peers/add_range/
		*/
		let uri = FORMAT("{0}peers/add_range/", this.uri);
		let request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	API.prototype.peersUpdateRange = function(data, callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/peers/update_range/
		*/
		let uri = FORMAT("{0}peers/update_range/", this.uri);
		let request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	API.prototype.peerUpdate = function(data, callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/peers/save/
		*/
		let uri = FORMAT("{0}peers/save/", this.uri);
		let request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

/********************************************************************************************************
	Dialplan Processors
********************************************************************************************************/

	API.prototype.dialplanProcessorsLoadAll = function(callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/dialplan_processors/load_all/
		*/
		let uri = FORMAT("{0}dialplan_processors/load/", this.uri);

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

	API.prototype.dialplanProcessorsGetHelpers = function(callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/dialplan_processors/get_helpers/
		*/
		let uri = FORMAT("{0}dialplan_processors/get_helpers/", this.uri);

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

	API.prototype.dialplanProcessorsGetTrace = function(processorId, callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/dialplan_processors/get_trace/my_processor
		*/
		let uri = FORMAT("{0}dialplan_processors/get_trace/{1}", this.uri, processorId);

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

	API.prototype.dialplanProcessorsExport = function(processorId, callback)
	{
		/*
		 * sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/dialplan_processors/export/my_processor
		 */
		callback = callback || $.noop;
		var uri = FORMAT("{0}dialplan_processors/export/{1}", this.uri, processorId);
		window.open(uri, "_blank");
		callback();
	};

/********************************************************************************************************
	TTS/ASR ENGINES
********************************************************************************************************/
	API.prototype.ttsasrEnginesVCCLoad = function(vcc_id, callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/ttsasrengines/vcc/load/vccId
		*/
		let uri = FORMAT("{0}ttsasrengines/vcc/load/{1}", this.uri, vcc_id);

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};
	
	API.prototype.ttsasrEnginesProvisioningLoadAll = function(callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/ttsasrengines/provisioning/load/
		*/
		let uri = FORMAT("{0}ttsasrengines/provisioning/load/", this.uri);

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};
	
	API.prototype.enginesLoadAllProvisionedTtsProfiles = function(callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/ttsasrengines/loadttsprovisioned/
		*/
		let uri = FORMAT("{0}ttsasrengines/loadttsprovisioned/", this.uri);

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};
	
	API.prototype.enginesLoadAllProvisionedAsrProfiles = function(callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/ttsasrengines/loadasrprovisioned/
		*/
		let uri = FORMAT("{0}ttsasrengines/loadasrprovisioned/", this.uri);

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};
	
	API.prototype.ttsasrEnginesLoadUniMRCPLogLevels = function(callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/ttsasrengines/load/loglevels/
		*/
		let uri = FORMAT("{0}ttsasrengines/load/loglevels/", this.uri);

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};
	
	API.prototype.ttsasrDataLoadProviders = function(callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/ttsasrengines/load/providers/
		*/
		let uri = FORMAT("{0}ttsasrengines/load/providers/", this.uri);

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};
	
	API.prototype.ttsasrDataLoadProviderTypes = function(callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/ttsasrengines/load/provider_types/
		*/
		let uri = FORMAT("{0}ttsasrengines/load/provider_types/", this.uri);

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};
	
	API.prototype.LoadUsedTtsAsrProfiles = function(ttsasrEngineId, callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/ttsasrengines/loadusedprofiles/ttsasrEngineId
		*/
		let uri = FORMAT("{0}ttsasrengines/loadusedprofiles/{1}", this.uri, ttsasrEngineId);

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};
	
	API.prototype.ListTtsAsrEnginesInUse = function(callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/ttsasrengines/listttsasrenginesinuse
		*/
		let uri = FORMAT("{0}ttsasrengines/listttsasrenginesinuse/", this.uri);

		let request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};
	
/********************************************************************************************************
	INTEGRATIONS - HELPDESK
********************************************************************************************************/

	API.prototype.helpdeskSynchronize = function(data, callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/vccs/integrations/helpdesk/synchronize/
		*/
		let uri = FORMAT("{0}vccs/integrations/helpdesk/synchronize/", this.uri);
		let request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

/********************************************************************************************************
	MIGRATIONS - ALLEGRO
********************************************************************************************************/

	API.prototype.migrateAllegroVccToNative = function(data, callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/vccs/migrations/allegro/to/native/
		*/
		let uri = FORMAT("{0}vccs/migrations/allegro/to/native/", this.uri);
		let request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

/********************************************************************************************************
	CALL TYPES
********************************************************************************************************/

	let CALL_TYPES = {
		INCOMING : {
			id         : "INCOMING",
			name       : "<%=_T('Incoming')%>",
			description: "<%=_T('Incoming Calls arrived from package DID trunks.')%>"
		},
		OUTGOING : {
			id         : "OUTGOING",
			name       : "<%=_T('Outgoing')%>",
			description: "<%=_T('Outgoing Calls to be made to an external number.')%>"
		},
		OUTGOING_LOCAL : {
			id         : "OUTGOING_LOCAL",
			name       : "<%=_T('Outgoing To Agent')%>",
			description: "<%=_T('Internal Calls made to an agent when conversation is assigned to him.')%>"
		},
		INCOMING_PROXY : {
			id         : "INCOMING_PROXY",
			name       : "<%=_T('Incoming from Host')%>",
			description: "<%=_T('DIALPLAN_CALL_TYPES_INCOMING_PROXY')%>"
		}
	};
	let CALL_TYPES_sorted = [
		"INCOMING",
		"OUTGOING",
		"OUTGOING_LOCAL",
		"INCOMING_PROXY"
	];

	API.prototype.getCallTypes = function()
	{
		return CALL_TYPES_sorted.map(type => CALL_TYPES[type]);
	};

	API.prototype.getCallType = function(type)
	{
		return CALL_TYPES[type];
	};

/********************************************************************************************************
	LDAP
********************************************************************************************************/

	API.prototype.getLdapGroups = function(data, callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/vccs/ldap/loadtree/
		*/
		let uri = FORMAT("{0}vccs/ldap/getldapgroups/", this.uri);
		
		let request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};


/********************************************************************************************************
	AzureAD
********************************************************************************************************/

API.prototype.getAzureAdGroups = function(data, callback = $.noop)
{
	/*
	* sample URI:
	*   http://i6.inconcert.cc/inconcert/apps/provisioning/vccs/azuread/getazureadgroups/
	*/
	let uri = FORMAT("{0}vccs/azuread/getazureadgroups/", this.uri);
	
	let request_handler = this.__Post(uri, callback, data);
	return request_handler;
};


/********************************************************************************************************
	LICENSE
********************************************************************************************************/

API.prototype.LoadAvailableLicenseResources = function(callback = $.noop)
{
	/*
	* sample URI:
	*   http://i6.inconcert.cc/inconcert/apps/provisioning/vccs/get_available_license_resources/vcc_id/
	*/
	let uri = FORMAT("{0}vccs/get_available_license_resources/", this.uri);

	let request_handler = this.__Get(uri, function(success, list) {
		if (success) {
			callback(true, list);
			return;
		}
		// error
		callback(false, list);
	});
	return request_handler;
};

API.prototype.CompanyIsEditable = function(callback = $.noop)
{
	/*
	* sample URI:
	*   http://i6.inconcert.cc/inconcert/apps/provisioning/vccs/company_is_editable
	*/
	let uri = FORMAT("{0}vccs/company_is_editable/", this.uri);

	let request_handler = this.__Get(uri, function(success, response) {
		if (success) {
			callback(true, response);
			return;
		}
		// error
		callback(false, response);
	});
	return request_handler;
};

API.prototype.CompanyIsRequired = function(callback = $.noop)
{
	/*
	* sample URI:
	*   http://i6.inconcert.cc/inconcert/apps/provisioning/vccs/company_is_required
	*/
	let uri = FORMAT("{0}vccs/company_is_required/", this.uri);

	let request_handler = this.__Get(uri, function(success, response) {
		if (success) {
			callback(true, response);
			return;
		}
		// error
		callback(false, response);
	});
	return request_handler;
};

API.prototype.ValidateCompany = function(company, callback = $.noop)
{
	/*
	* sample URI:
	*   http://i6.inconcert.cc/inconcert/apps/provisioning/vccs/validate_company/company
	*/
	let uri = FORMAT("{0}vccs/validate_company/{1}", this.uri, company);

	let request_handler = this.__Get(uri, function(success, result) {
		if (success) {
			callback(true, result);
			return;
		}
		// error
		callback(false, result);
	});
	return request_handler;
};

/********************************************************************************************************
VCCs - S3 Regions
********************************************************************************************************/

API.prototype.loadS3Regions = function(callback = $.noop)
{
	/*
	* sample URI:
	*   http://i6.inconcert.cc/inconcert/apps/provisioning/vccs/load_s3_regions/
	*/
	let uri = FORMAT("{0}vccs/load_s3_regions/", this.uri);

	let request_handler = this.__Get(uri, function(success, list) {
		if (success) {
			callback(true, list);
			return;
		}
		// error
		callback(false, list);
	});
	return request_handler;
};

/********************************************************************************************************
	GATEWAYS
********************************************************************************************************/
	API.prototype.gatewaysLoadAll = function(callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/gateways/load/
		*/
		let uri = FORMAT("{0}gateways/load/", this.uri);

		let request_handler = this.__Get(uri, callback);
		return request_handler;
	};

	API.prototype.gatewaysGet = function(gatewayId, callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/gateways/get/my_gateway
		*/
		let uri = FORMAT("{0}gateways/get/{1}", this.uri, gatewayId);

		let request_handler = this.__Get(uri, callback);
		return request_handler;
	};

	API.prototype.gatewaysSave = function(gateway, callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/gateways/save/
		*/
		let uri = FORMAT("{0}gateways/save/", this.uri);

		let request_handler = this.__Post(uri, callback, [gateway]);
		return request_handler;
	};

	API.prototype.gatewaysDelete = function(gatewayId, callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/gateways/delete/
		*/
		let uri = FORMAT("{0}gateways/delete/", this.uri);

		let request_handler = this.__Post(uri, callback, [gatewayId]);
		return request_handler;
	};

	/********************************************************************************************************
		GATEWAY NODE TRUNKS
	********************************************************************************************************/
	API.prototype.gatewayNodeTrunksLoadAll = function(gatewayId, callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/gateways/node_trunks/load/my_gateway
		*/
		let uri = FORMAT("{0}gateways/node_trunks/load/{1}", this.uri, gatewayId);

		let request_handler = this.__Get(uri, callback);
		return request_handler;
	};

	API.prototype.gatewayNodeTrunksGet = function(gatewayId, trunkId, callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/gateways/node_trunks/get/my_gateway/my_trunk
		*/
		let uri = FORMAT("{0}gateways/node_trunks/get/{1}/{2}", this.uri, gatewayId, trunkId);

		let request_handler = this.__Get(uri, callback);
		return request_handler;
	};

	API.prototype.gatewayNodeTrunksSave = function(gatewayId, trunk, callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/node_trunks/gateways/save/my_gateway
		*/
		let uri = FORMAT("{0}gateways/node_trunks/save/{1}", this.uri, gatewayId);

		let request_handler = this.__Post(uri, callback, [trunk]);
		return request_handler;
	};

	API.prototype.gatewayNodeTrunksDelete = function(gatewayId, trunkId, callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/gateways/node_trunks/delete/my_gateway
		*/
		let uri = FORMAT("{0}gateways/node_trunks/delete/{1}", this.uri, gatewayId);

		let request_handler = this.__Post(uri, callback, [trunkId]);
		return request_handler;
	};

	/********************************************************************************************************
		GATEWAY CARRIER TRUNKS
	********************************************************************************************************/
	API.prototype.gatewayCarrierTrunksLoadAll = function(gatewayId, callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/gateways/carrier_trunks/load/my_gateway
		*/
		let uri = FORMAT("{0}gateways/carrier_trunks/load/{1}", this.uri, gatewayId);

		let request_handler = this.__Get(uri, callback);
		return request_handler;
	};

	API.prototype.gatewayCarrierTrunksGet = function(gatewayId, trunkId, callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/gateways/carrier_trunks/get/my_gateway/my_trunk
		*/
		let uri = FORMAT("{0}gateways/carrier_trunks/get/{1}/{2}", this.uri, gatewayId, trunkId);

		let request_handler = this.__Get(uri, callback);
		return request_handler;
	};

	API.prototype.gatewayCarrierTrunksSave = function(gatewayId, trunk, callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/carrier_trunks/gateways/save/my_gateway
		*/
		let uri = FORMAT("{0}gateways/carrier_trunks/save/{1}", this.uri, gatewayId);

		let request_handler = this.__Post(uri, callback, [trunk]);
		return request_handler;
	};

	API.prototype.gatewayCarrierTrunksDelete = function(gatewayId, trunkId, callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/gateways/carrier_trunks/delete/my_gateway
		*/
		let uri = FORMAT("{0}gateways/carrier_trunks/delete/{1}", this.uri, gatewayId);

		let request_handler = this.__Post(uri, callback, [trunkId]);
		return request_handler;
	};

	/********************************************************************************************************
		GATEWAY DIAPLAN CONTEXTS
	********************************************************************************************************/
	API.prototype.gatewayDialplanContextsLoadAll = function(gatewayId, callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/gateways/dialplan_contexts/load/my_gateway
		*/
		let uri = FORMAT("{0}gateways/dialplan_contexts/load/{1}", this.uri, gatewayId);

		let request_handler = this.__Get(uri, callback);
		return request_handler;
	};

	API.prototype.gatewayDialplanContextsGet = function(gatewayId, contextId, callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/gateways/dialplan_contexts/get/my_gateway/my_context
		*/
		let uri = FORMAT("{0}gateways/dialplan_contexts/get/{1}/{2}", this.uri, gatewayId, contextId);

		let request_handler = this.__Get(uri, callback);
		return request_handler;
	};

	API.prototype.gatewayDialplanContextsSave = function(gatewayId, context, callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/dialplan_contexts/gateways/save/my_gateway
		*/
		let uri = FORMAT("{0}gateways/dialplan_contexts/save/{1}", this.uri, gatewayId);

		let request_handler = this.__Post(uri, callback, [context]);
		return request_handler;
	};

	API.prototype.gatewayDialplanContextsDelete = function(gatewayId, contextId, callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/gateways/dialplan_contexts/delete/my_gateway
		*/
		let uri = FORMAT("{0}gateways/dialplan_contexts/delete/{1}", this.uri, gatewayId);

		let request_handler = this.__Post(uri, callback, [contextId]);
		return request_handler;
	};

	/********************************************************************************************************
		GATEWAY MONITORING
	********************************************************************************************************/

	API.prototype.gatewaysSaveMonitoring = function(gatewayId, data, callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/gateways/monitoring/save/my_gateway
		*/
		let uri = FORMAT("{0}gateways/monitoring/save/{1}", this.uri, gatewayId);

		let request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	API.prototype.gatewaysGetMonitoring = function(gatewayId, callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/gateways/monitoring/get/my_gateway
		*/
		let uri = FORMAT("{0}gateways/monitoring/get/{1}", this.uri, gatewayId);

		let request_handler = this.__Get(uri, callback);
		return request_handler;
	};

	API.prototype.gatewaysMonitoringGrafanaWidgets = function(gatewayId, callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/gateways/monitoring/grafana_widgets/my_gateway
		*/
		let uri = FORMAT("{0}gateways/monitoring/grafana_widgets/{1}", this.uri, gatewayId);

		let request_handler = this.__Get(uri, callback);
		return request_handler;
	};

	API.prototype.gatewaysMonitoringSipCaptureWidgets = function(gatewayId, callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/gateways/monitoring/sip_capture_widgets/my_gateway
		*/
		let uri = FORMAT("{0}gateways/monitoring/sip_capture_widgets/{1}", this.uri, gatewayId);

		let request_handler = this.__Get(uri, callback);
		return request_handler;
	};

	API.prototype.gatewaysMonitoringStopSipCaptureWidgets = function(gatewayId, callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/gateways/monitoring/stop_sip_capture_widgets/my_gateway
		*/
		let uri = FORMAT("{0}gateways/monitoring/stop_sip_capture_widgets/{1}", this.uri, gatewayId);

		let request_handler = this.__Post(uri, callback, {});
		return request_handler;
	};

	API.prototype.gatewaysMonitoringSendConsoleCommand = function(gatewayId, data, callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/gateways/monitoring/send_console_command/my_gateway
		*/
		let uri = FORMAT("{0}gateways/monitoring/send_console_command/{1}", this.uri, gatewayId);

		let request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	API.prototype.gatewaysMonitoringConfigurationFile = function(gatewayId, fileName, callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/gateways/monitoring/configuration_file/my_gateway
		*/
		let uri = FORMAT("{0}gateways/monitoring/configuration_file/{1}", this.uri, gatewayId);

		let request_handler = this.__Post(uri, callback, { fileName });
		return request_handler;
	};

	/********************************************************************************************************
		SYSTEM MONITORING
	********************************************************************************************************/

	API.prototype.getSystemMonitoringDefinition = function(callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/system_monitoring/definition/
		*/
		let uri = FORMAT("{0}system_monitoring/definition/", this.uri);

		let request_handler = this.__Get(uri, callback);
		return request_handler;
	};

	API.prototype.runSystemMonitoringView = function(viewId, request, callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/system_monitoring/run_view/my_view
		*/
		let uri = FORMAT("{0}system_monitoring/run_view/{1}", this.uri, encodeURIComponent(viewId));

		let request_handler = this.__Post(uri, callback, request);
		return request_handler;
	};

	API.prototype.getSystemMonitoringDetails = function(viewId, itemId, callback = $.noop)
	{
		/*
		* sample URI:
		*   http://i6.inconcert.cc/inconcert/apps/provisioning/system_monitoring/details/my_view/my_item
		*/
		let uri = FORMAT("{0}system_monitoring/details/{1}/{2}", this.uri, encodeURIComponent(viewId),
			encodeURIComponent(itemId));

		let request_handler = this.__Get(uri, callback);
		return request_handler;
	};
}());
