/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:false, browser:true, indent:false, maxerr:50, quotmark:double, -W014, esversion: 11 */

/* global inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function ()
{
	"use strict";

	const FORMAT = inConcert.Utils.format;

	var get_request_timeout = function() {
		const provisioning_api = inConcert.app.getProvisioningAPI();
		return provisioning_api.getRequestTimeout();
	};

	//------------------------------------------------------
	// vccs navigation config
	//
	var vccs_config = function(base_uri)
	{
		var get_message_error = function(type, err, name = "")
		{
			let message;
			if (type === "delete") {
				let err_messages = {
					"FORBIDDEN": "<%=_T('Unable to delete {0} VCC. You dont have permission.')%>",
					"UNABLE TO DELETE VCC WITH ALLEGRO INTEGRATION": "<%=_T([[Unable to delete {0} VCC. Can\'t remove Contact Centers with Allegro Integration.]])%>",
					"VCC DOES NOT EXIST": "<%=_T('Unable to delete {0} Contact Center. Contact Center does not exist.')%>"
				};
				if (typeof err === "object") {
					err = err.result || "";
				}
				message = err_messages[err.toUpperCase()] || "<%=_T('Error deleting {0} Contact Center.')%>";
				message = FORMAT(message, name);
			}
			return message;
		};

		// is action enabled
		const is_action_enabled = function(action, row)
		{
			/*
			 * Valores a retornar:
			 *    1  = La accion se debe visualizar y estar Enabled.
			 *    0  = La accion se debe visualizar y estar Disabled.
			 *    -1 = La accion no se genera en el markup. (No aparece el boton de la accion).
			 */
			if (action.toLowerCase() === "synchronize") {
				if (!row.HasHelpDeskIntegration) {
					return -1;
				}
				else {
					if (row.InProgressInfo) {
						return 0;
					}
				}
			}
			return 1;
		};

		var make_detail_error = function(errors) {
			if (!errors) {
				return undefined;
			}
			let detail = "";
			for (let i = 0; i < errors.length; i++) {
				let err = errors[i];
				let err_detail;
				if (err.err) {
					let translated_err;
					if (err.err.result === "DOES NOT HAVE EMAIL") {
						translated_err = "<%=_T([[does not have an assigned email]])%>";
					}
					else if (err.err.result === "CONNECTION TIMEOUT") {
						translated_err = "<%=_T([[connection time against HelpDesk has expired]])%>";
					}
					translated_err = translated_err || err.err.reason;
					err_detail = FORMAT("<%=_T([[{0} - User: '{1}' {2}.]])%>", (i + 1).toString(), err.userId, translated_err);
				}
				else {
					err_detail = FORMAT("<%=_T([[{0} - User: '{1}'.]])%>", (i + 1).toString(), err.userId);
				}
				if (detail) {
					detail = detail + "\n";
				}
				detail = detail + err_detail;
			}
			return detail;
		};

		// execute custom action
		const execute_custom_action = function(action_id, $row, view, _ /*$btn*/, execute_callback = $.noop)
		{
			let row_id = $row.attr("row_id");
			let display_name = $row.attr("row_name");
			let confirmation_message, waiting_message, success_message, error_message;
			action_id = action_id.toLowerCase();

			if (action_id === "synchronize") {
				confirmation_message = "<%=_T('Do you want to synchronize Users of VCC {0} with HelpDesk ?')%>";
				waiting_message = "<%=_T('Please wait, syncing Users of VCC {0} with HelpDesk')%>";
				success_message = "<%=_T('Users of VCC {0} was successfully synchronized with HelpDesk.')%>";
				error_message = "<%=_T('Error synchronizing Users of VCC {0} with HelpDesk.')%>";
			}
			else if (action_id === "migrate_to_native_vcc") {
				confirmation_message = "<%=_T('Do you want to migrate allegro VCC {0} to native VCC ?')%>";
				waiting_message = "<%=_T('Please wait, syncing Users of VCC {0} with HelpDesk')%>";
				success_message = "<%=_T('Users of VCC {0} was successfully synchronized with HelpDesk.')%>";
				error_message = "<%=_T('Error synchronizing Users of VCC {0} with HelpDesk.')%>";
			}

			confirmation_message = FORMAT(confirmation_message, display_name);
			waiting_message = FORMAT(waiting_message, display_name);
			success_message = FORMAT(success_message, display_name);
			error_message = FORMAT(error_message, display_name);

			view.dashboard.hide_status_message();

			$.inconcertPopup.Show(
				"<%=_T('Confirm')%>",
				confirmation_message,
				jQuery.inconcertPopup.ButtonSets.BS_YESNO,
				function(response) {
					if (response.toLowerCase() !== "yes") {
						return;
					}
					view.dashboard.show_waiting(waiting_message);
					var request_response = function(success, resp)
					{
						let message, message_type, timeout_ms;
						if (success) {
							message = success_message;
							message_type = "Info";
						}
						else {
							let detail;
							switch ((resp.result || "").toUpperCase())
							{
								case "FORBIDDEN":
									error_message = "<%=_T('Unable to synchronizing Users of VCC {0}. You dont have permission.')%>";
									error_message = FORMAT(error_message, display_name);
									break;

								case "WITHOUT_HELPDESK_INTEGRATION":
									error_message = "<%=_T('Unable to synchronizing Users of VCC {0}. VCC without HELPDESK integration.')%>";
									error_message = FORMAT(error_message, display_name);
									break;

								case "ALREADY_EXIST_ANOTHER_UPDATE_RUNNING":
									error_message = "<%=_T('Unable to synchronizing Users of VCC {0}. Already exist another update running.')%>";
									error_message = FORMAT(error_message, display_name);
									break;

								case "VALIDATION_ERROR":
									detail = make_detail_error(resp.validation.errors);
									view.show_popup_message({
										messages: {
											title  : FORMAT("<%=_T('Unable to synchronize Users of VCC {0}.')%>", display_name),
											message: "<%=_T('Users do not include the information that HELPDESK needs.')%>",
											detail : detail,
											type   : "warn"
										}
									});
									execute_callback(false, resp);
									return;

								case "ONLY SOME USERS UPDATED":
									detail = make_detail_error(resp.update.errors);
									view.show_popup_message({
										messages: {
											title  : FORMAT("<%=_T('Unable to synchronize all Users of VCC {0}.')%>", display_name),
											message: "<%=_T('Only some Users have been updated.')%>",
											detail : detail,
											type   : "warn"
										}
									});
									execute_callback(false, resp);
									return;
							}

							message = error_message;
							message_type = "Error";
							timeout_ms = false; //infinite
						}
						view.show_status_message(message, message_type, timeout_ms);
						execute_callback(success, resp);
					};

					const provisioning_api = inConcert.app.getProvisioningAPI();

					if (action_id === "synchronize") {
						provisioning_api.helpdeskSynchronize({vcc: row_id},
							function(success, response) {
								request_response(success, response);
							}
						);
					}
					else if (action_id === "migrate_to_native_vcc") {
						provisioning_api.migrateAllegroVccToNative({vcc: row_id},
							function(success, response) {
								request_response(success, response);
							}
						);
					}

				},
				undefined,
				{
					popup_id       : "conf_inconcertPopup",
					container_class: "confirmationPopUp",
					overlay_class  : "confirmationOverlay"
				}
			);
		};

		const frm_instance = inConcert.form_provisioning_vccs;

		return {
			id: "provisioning_vccs",
			name: "<%=_T('Contact Centers')%>",
			permissions: {
				view: "view_vccs",
				save: "save_vcc",
				erase: "delete_vcc",
				permission_type: "vcc"
			},
			class_name: "itemVCCs",
			i_class_name: "far fa-user-headset",
			view: {
				request: {
					timeout: get_request_timeout()
				},
				header: {
					hide_new_button: false,
					hide_total_rows: true
				},
				paginator: {
					paged_text_format : "<%=_T('{page_row_start}-{page_row_end} of {many}')%>"
				},
				render: "/inconcert/apps/configuration/pages/view_content_default.<%=_L%>.html#grunt-cache-bust",
				title: "<%=_T('Contact Centers View')%>",
				tagline: "<%=_T('Contact Centers')%>",
				new_button_name: "<%=_T('New Contact Center')%>",
				row_id: "Id",
				display_name: "Id",
				messages: {
					erase: "<%=_T('Do you want to delete Contact Center {0} ?')%>",
					err: get_message_error
				},
				uri: {
					erase:  base_uri + "vccs/delete/",
					search: base_uri + "vccs/search/"
				},
				source: ["Id", "Name", "Description", "HasAllegroIntegration", "HasHelpDeskIntegration", "HasMarketingAndSalesIntegration", "InProgressInfo"],
				columns: [
					{id: "_number"                        , name: "", type: "counter"     },
					{id: "Id"                             , name: "<%=_T('Name')%>"       },
					{id: "Description"                    , name: "<%=_T('Description')%>"},
					{id: "HasAllegroIntegration"          , name: "<%=_T('Allegro Integration')%>" , type: "boolean", class_name: "booleanColumnTitle"},
					{id: "HasMarketingAndSalesIntegration", name: "<%=_T('M&S Integration')%>"     , type: "boolean", class_name: "booleanColumnTitle"},
					{id: "HasHelpDeskIntegration"         , name: "<%=_T('HelpDesk Integration')%>", type: "boolean", class_name: "booleanColumnTitle"},
					{id: "_action"    , name: "",
						system_action: {
							edit: true,
							//erase: true
						},
						custom_action: {
							synchronize: {
								name      : "<%=_T('Synchronize')%>",
								execute   : execute_custom_action,
								class_name: "synchronize",
								tooltip   : function(row) {
									if (row.InProgressInfo) {
										return "<%=_T('Already exist another update running.')%>";
									}
									return "<%=_T('Synchronize VCC Users with HelpDesk')%>";
								}
							},
							// migrate_to_native_vcc: {
							// 	name      : "<%=_T('Migrate to Native VCC')%>",
							// 	execute   : execute_custom_action,
							// 	class_name: "migrate",
							// 	tooltip   : function(row) {
							// 		if (row.InProgressInfo) {
							// 			return "<%=_T('Already exist another update running.')%>";
							// 		}
							// 		return "<%=_T('Migrate to Native VCC')%>";
							// 	}
							// }
						},
						is_enabled: is_action_enabled
					}
				],
				filter: {
					fields: {
						"Id"                    : {control: null},
						"Description"           : {control: null},
						"HasAllegroIntegration" : {control: null},
						"HasHelpDeskIntegration": {control: null},
						"HasMasIntegration"     : {control: null}
					}
				},
				realtime: null
			},
			form: {instance: frm_instance}
		};
	};

	//------------------------------------------------------
	// carriers navigation config
	//
	var carriers_config = function(base_uri)
	{
		var get_message_error = function(type, err, name = "")
		{
			let message, related_id;
			if (type === "delete") {
				let err_messages = {
					"FORBIDDEN": "<%=_T('Unable to delete {0} Carrier. You dont have permission.')%>",
					"CARRIER DOES NOT EXIST": "<%=_T('Unable to delete {0} CARRIER. CARRIER does not exist.')%>",
					"CANNOT DELETE CARRIER ASSOCIATED TO DID": "<%=_T('Unable to delete {0} CARRIER. Cannot delete CARRIER while DID {1} is still associated with it.')%>",
					"CANNOT DELETE CARRIER ASSOCIATED TO TRUNK": "<%=_T('Unable to delete {0} CARRIER. Cannot delete CARRIER while TRUNK {1} is still associated with it.')%>",
					"CANNOT DELETE CARRIER ASSOCIATED TO TRUNK GROUP": "<%=_T('Unable to delete {0} CARRIER. Cannot delete CARRIER while TRUNK GROUP {1} is still associated with it.')%>"
				};
				if (typeof err === "object") {
					related_id = err.relatedId;
					err = err.result || "";
				}
				related_id = related_id || "";
				message = err_messages[err.toUpperCase()] || "<%=_T('Error deleting {0} CARRIER.')%>";
				message = FORMAT(message, name, related_id);
			}
			return message;
		};

		const frm_instance = inConcert.form_provisioning_carriers;

		return {
			id: "provisioning_carriers",
			name: "<%=_T('Carriers')%>",
			permissions: {
				view: "view_carriers",
				save: "save_carrier",
				erase: "delete_carrier",
				permission_type: "vcc"
			},
			class_name: "itemCarriers",
			i_class_name: "far fa-satellite-dish",
			view: {
				request: {
					timeout: get_request_timeout()
				},
				header: {
					hide_new_button: false,
					hide_total_rows: true
				},
				paginator: {
					paged_text_format : "<%=_T('{page_row_start}-{page_row_end} of {many}')%>"
				},
				render: "/inconcert/apps/configuration/pages/view_content_default.<%=_L%>.html#grunt-cache-bust",
				title: "<%=_T('Carriers View')%>",
				tagline: "<%=_T('Carriers')%>",
				new_button_name: "<%=_T('New Carrier')%>",
				row_id: "id",
				display_name: "id",
				messages: {
					erase: "<%=_T('Do you want to delete Carrier {0} ?')%>",
					err: get_message_error
				},
				uri: {
					erase:  base_uri + "carriers/delete/",
					search: base_uri + "carriers/search/"
				},
				source: ["id", "Name", "description", "prefix"],
				columns: [
					{id: "_number"    , name: "", type: "counter"     },
					{id: "id"         , name: "<%=_T('Name')%>"       },
					{id: "description", name: "<%=_T('Description')%>"},
					{id: "prefix"     , name: "<%=_T('Prefix')%>"     },
					{id: "_action"    , name: "",
						system_action: {edit: true, erase: true}
					}
				],
				filter: {
					fields: {
						"id"         : {control: null},
						"description": {control: null},
						"prefix"     : {control: null}
					}
				},
				realtime: null
			},
			form: {instance: frm_instance}
		};
	};

	//------------------------------------------------------
	// system monitoring config
	//
	var system_monitoring = function(base_uri)
	{
		var get_message_error = function(type, err, name = "")
		{
			let message, related_id;
			if (type === "delete") {
				let err_messages = {
					"FORBIDDEN": "<%=_T('Unable to delete {0} Carrier. You dont have permission.')%>",
					"CARRIER DOES NOT EXIST": "<%=_T('Unable to delete {0} CARRIER. CARRIER does not exist.')%>",
					"CANNOT DELETE CARRIER ASSOCIATED TO DID": "<%=_T('Unable to delete {0} CARRIER. Cannot delete CARRIER while DID {1} is still associated with it.')%>",
					"CANNOT DELETE CARRIER ASSOCIATED TO TRUNK": "<%=_T('Unable to delete {0} CARRIER. Cannot delete CARRIER while TRUNK {1} is still associated with it.')%>",
					"CANNOT DELETE CARRIER ASSOCIATED TO TRUNK GROUP": "<%=_T('Unable to delete {0} CARRIER. Cannot delete CARRIER while TRUNK GROUP {1} is still associated with it.')%>"
				};
				if (typeof err === "object") {
					related_id = err.relatedId;
					err = err.result || "";
				}
				related_id = related_id || "";
				message = err_messages[err.toUpperCase()] || "<%=_T('Error deleting {0} CARRIER.')%>";
				message = FORMAT(message, name, related_id);
			}
			return message;
		};

		const frm_instance = inConcert.form_provisioning_carriers;

		return {
			id: "system_monitoring",
			name: "<%=_T('System Monitoring')%>",
			permissions: {
				view: "view_carriers",
				save: "save_carrier",
				erase: "delete_carrier",
				permission_type: "vcc"
			},
			class_name: "itemCarriers",
			i_class_name: "far fa-file-chart-line",
			view: {
				request: {
					timeout: get_request_timeout()
				},
				header: {
					hide_new_button: false,
					hide_total_rows: true
				},
				paginator: {
					paged_text_format : "<%=_T('{page_row_start}-{page_row_end} of {many}')%>"
				},
				render: "/inconcert/apps/configuration/pages/view_content_system_monitoring.<%=_L%>.html#grunt-cache-bust",
				title: "<%=_T('System Monitoring')%>",
				tagline: "<%=_T('System')%>",
				new_button_name: "<%=_T('New Carrier')%>",
				row_id: "id",
				display_name: "id",
				messages: {
					erase: "<%=_T('Do you want to delete Carrier {0} ?')%>",
					err: get_message_error
				},
				uri: {
					erase:  base_uri + "carriers/delete/",
					search: base_uri + "carriers/search/"
				},
				source: ["id", "Name", "description", "prefix"],
				columns: [
					{id: "_number"    , name: "", type: "counter"     },
					{id: "id"         , name: "<%=_T('Name')%>"       },
					{id: "description", name: "<%=_T('Description')%>"},
					{id: "prefix"     , name: "<%=_T('Prefix')%>"     },
					{id: "_action"    , name: "",
						system_action: {edit: true, erase: true}
					}
				],
				filter: {
					fields: {
						"id"         : {control: null},
						"description": {control: null},
						"prefix"     : {control: null}
					}
				},
				realtime: null
			},
			form: {instance: frm_instance}
		};
	};

	//------------------------------------------------------
	// ice servers navigation config
	//
	var ice_servers_config = function(base_uri)
	{
		var get_message_error = function(type, err, name = "")
		{
			let message, related_id;
			if (type === "delete") {
				let err_messages = {
					"FORBIDDEN": "<%=_T('Unable to delete Ice Server. You dont have permission.')%>",
					"E_NOT_EXISTS": "<%=_T('Unable to delete Ice Server. Ice Server does not exists.')%>"
				};
				if (typeof err === "object") {
					related_id = err.relatedId;
					if ($.isArray(related_id)) {
						related_id = related_id.join(",");
						const dots = "..";
						related_id = inConcert.Utils.truncateWord(related_id, 50, dots);
					}
					err = err.result || "";
				}
				related_id = related_id || "";
				message = err_messages[err.toUpperCase()] || "<%=_T('Error deleting Ice Server.')%>";
				message = FORMAT(message, name, related_id);
			}
			return message;
		};

		const frm_instance = inConcert.form_provisioning_ice_servers;

		return {
			id: "provisioning_ice_servers",
			name: "<%=_T('ICE Server')%>",
			permissions: {
				view: "view_ice_servers",
				save: "save_ice_server",
				erase: "delete_ice_server",
				permission_type: "vcc"
			},
			class_name: "itemHosts",
			i_class_name: "far fa-server",
			view: {
				request: {
					timeout: get_request_timeout()
				},
				header: {
					hide_new_button: false,
					hide_total_rows: true
				},
				paginator: {
					paged_text_format : "<%=_T('{page_row_start}-{page_row_end} of {many}')%>"
				},
				render: "/inconcert/apps/configuration/pages/view_content_default.<%=_L%>.html#grunt-cache-bust",
				title: "<%=_T('ICE Server')%>",
				tagline: "<%=_T('ICE Server')%>",
				new_button_name: "<%=_T('Create ICE Server')%>",
				row_id: "serverId",
				display_name: "serverId",
				messages: {
					erase: "<%=_T('Do you want to delete Ice Server {0} ?')%>",
					nothing: "<%=_T('No ICE Server has been created yet.')%>",
					err: get_message_error
				},
				uri: {
					erase:  base_uri + "ice_servers/delete/",
					search: base_uri + "ice_servers/search/"
				},
				source: ["serverId", "domain", "port", "user"],
				columns: [
					{id: "_number"     , name: "", type: "counter"   },
					{id: "domain"      , name: "<%=_T('Server URL')%>"},
					{id: "port"        , name: "<%=_T('Port')%>"      },
					{id: "user"        , name: "<%=_T('User')%>"      },
					{id: "_action"     , name: "",
						system_action: {edit: true, erase: true}
					}
				],
				filter: {
					fields: {
						"domain" : {control: null}
					}
				},
				realtime: null
			},
			form: {instance: frm_instance}
		};
	};

	//------------------------------------------------------
	// hosts navigation config
	//
	var hosts_config = function(base_uri)
	{
		var get_message_error = function(type, err, name = "")
		{
			let message, related_id;
			if (type === "delete") {
				let err_messages = {
					"FORBIDDEN": "<%=_T('Unable to delete {0} Host. You dont have permission.')%>",
					"HOST DOES NOT EXIST": "<%=_T('Unable to delete {0} HOST. HOST does not exist.')%>",
					"CANNOT DELETE HOST ASSOCIATED TO PEER": "<%=_T('Unable to delete {0} HOST. Cannot delete HOST while PEER {1} is still associated with it.')%>",
					"CANNOT DELETE HOST ASSOCIATED TO TRUNK": "<%=_T('Unable to delete {0} HOST. Cannot delete HOST while TRUNK {1} is still associated with it.')%>"
				};
				if (typeof err === "object") {
					related_id = err.relatedId;
					if ($.isArray(related_id)) {
						related_id = related_id.join(",");
						const dots = "..";
						related_id = inConcert.Utils.truncateWord(related_id, 50, dots);
					}
					err = err.result || "";
				}
				related_id = related_id || "";
				message = err_messages[err.toUpperCase()] || "<%=_T('Error deleting {0} HOST.')%>";
				message = FORMAT(message, name, related_id);
			}
			return message;
		};

		const frm_instance = inConcert.form_provisioning_hosts;

		const MakeTelehponyLabel = function(text) {
			return FORMAT("{0} ({1})", text, "<%=_T('Telephony')%>");
		};
		return {
			id: "provisioning_hosts",
			name: "<%=_T('Hosts')%>",
			permissions: {
				view: "view_hosts",
				save: "save_host",
				erase: "delete_host",
				permission_type: "vcc"
			},
			class_name: "itemHosts",
			i_class_name: "far fa-server",
			view: {
				request: {
					timeout: get_request_timeout()
				},
				header: {
					hide_new_button: false,
					hide_total_rows: true
				},
				paginator: {
					paged_text_format : "<%=_T('{page_row_start}-{page_row_end} of {many}')%>"
				},
				render: "/inconcert/apps/configuration/pages/view_content_default.<%=_L%>.html#grunt-cache-bust",
				title: "<%=_T('Hosts View')%>",
				tagline: "<%=_T('Hosts')%>",
				new_button_name: "<%=_T('New Host')%>",
				row_id: "hostId",
				display_name: "hostId",
				messages: {
					erase: "<%=_T('Do you want to delete Host {0} ?')%>",
					err: get_message_error
				},
				uri: {
					erase:  base_uri + "hosts/delete/",
					search: base_uri + "hosts/search/"
				},
				source: ["hostId", "rtmpServer", "sipServer", "telProxyPort", "description"],
				columns: [
					{id: "_number"     , name: "", type: "counter"     },
					{id: "hostId"      , name: "<%=_T('Host Name')%>"  },
					{id: "description" , name: "<%=_T('Description')%>"},
					{id: "sipServer"   , name: "<%=_T('Private IP')%>" },
					{id: "rtmpServer"  , name: MakeTelehponyLabel("<%=_T('Public Subdomain')%>")},
					{id: "telProxyPort", name: MakeTelehponyLabel("<%=_T('Port')%>")},
					{id: "_action"     , name: "",
						system_action: {edit: true, erase: true}
					}
				],
				filter: {
					fields: {
						"hostId"      : {control: null},
						"rtmpServer"  : {control: null},
						"sipServer"   : {control: null},
						"telProxyPort": {control: null},
						"description" : {control: null}
					}
				},
				realtime: null
			},
			form: {instance: frm_instance}
		};
	};

	//------------------------------------------------------
	// TtsAsr Engines navigation config
	//
	var ttsasrengines_config = function(base_uri)
	{
		var get_message_error = function(type, err, name = "")
		{
			let message, related_id;
			if (type === "delete") {
				let err_messages = {
					"FORBIDDEN": "<%=_T('Unable to delete {0} TTS/ASR Engine. You dont have permission.')%>",
					"TTS/ASR ENGINE DOES NOT EXIST": "<%=_T('Unable to delete {0} TTS/ASR Engine. TTS/ASR Engine does not exist.')%>",
					"CANNOT DELETE TTS/ASR ENGINE ASSOCIATED TO VCC": "<%=_T('Unable to delete {0} TTS/ASR Engine. Cannot delete TTS/ASR Engine while VCC {1} is still associated with it.')%>"
				};
				if (typeof err === "object") {
					related_id = err.relatedId;
					if ($.isArray(related_id)) {
						related_id = related_id.join(",");
						const dots = "..";
						related_id = inConcert.Utils.truncateWord(related_id, 50, dots);
					}
					err = err.result || "";
				}
				related_id = related_id || "";
				message = err_messages[err.toUpperCase()] || "<%=_T('Error deleting {0} TTS/ASR Engine.')%>";
				message = FORMAT(message, name, related_id);
			}
			return message;
		};

		const frm_instance = inConcert.form_provisioning_ttsasrengines;

		return {
			id: "ttsasrengines",
			name: "<%=_T('TTS/ASR Engines')%>",
			permissions: {
				view: "view_ttsasrengines",
				save: "save_ttsasrengines",
				permission_type: "vcc"
			},
			class_name: "itemTtsAsrEngines",
			i_class_name: "far fa-radiation-alt",
			view: {
				request: {
					timeout: get_request_timeout()
				},
				header: {
					hide_new_button: true,
					hide_total_rows: true
				},
				paginator: {
					paged_text_format : "<%=_T('{page_row_start}-{page_row_end} of {many}')%>"
				},
				render: "/inconcert/apps/configuration/pages/view_content_default.<%=_L%>.html#grunt-cache-bust",
				title: "<%=_T('TTS/ASR Engines View')%>",
				tagline: "<%=_T('TTS/ASR Engines')%>",
				new_button_name: "<%=_T('New TTS/ASR Engine')%>",
				row_id: "ttsAsrEngineId",
				display_name: "ttsAsrEngineId",
				messages: {
					erase: "<%=_T('Do you want to delete TTS/ASR Engine {0} ?')%>",
					err: get_message_error
				},
				uri: {
					erase:  base_uri + "ttsasrengines/delete/",
					search: base_uri + "ttsasrengines/search/"
				},
				source: ["ttsAsrEngineId", "description", "enabled"],
				columns: [
					{id: "_number"         , name: "", type: "counter"     },
					{id: "ttsAsrEngineId"  , name: "<%=_T('Id')%>"  },
					{id: "description"     , name: "<%=_T('Description')%>"},
					{id: "enabled"         , name: "<%=_T('Availability')%>" , type: "boolean", class_name: "booleanColumnTitle"},
					{id: "_action"         , name: "",
						system_action: {edit: true, erase: false}
					}
				],
				filter: {
					fields: {
						"ttsAsrEngineId"   : {control: null},
						"description"      : {control: null}
					}
				},
				realtime: null
			},
			form: {instance: frm_instance}
		};
	};

	//------------------------------------------------------
	// peers navigation config
	//
	var peers_config = function(base_uri)
	{
		var get_message_error = function(type, err, name = "")
		{
			let message, related_id;
			if (type === "delete") {
				let err_messages = {
					"FORBIDDEN"          : "<%=_T('Unable to delete {0} IP Phone. You dont have permission.')%>",
					"PEER DOES NOT EXIST": "<%=_T('Unable to delete {0} IP Phone. IP Phone does not exist.')%>",
					"PEER IN USE"        : "<%=_T('Unable to delete {0} IP Phone. Cannot delete IP Phone while VCC {1} is still associated with it.')%>"
				};
				if (typeof err === "object") {
					related_id = err.relatedId;
					err = err.result || "";
				}
				related_id = related_id || "";
				message = err_messages[err.toUpperCase()] || "<%=_T('Error deleting {0} IP Phone.')%>";
				message = FORMAT(message, name, related_id);
			}
			return message;
		};

		const frm_instance = inConcert.form_provisioning_peers;
		const translated_assignation_types = {
			agent : "<%=_T('Agent')%>",
			client: "<%=_T('Client')%>"
		};

		return {
			id: "provisioning_peers",
			name: "<%=_T('IP Phones')%>",
			permissions: {
				view: "view_peers",
				save: "save_peer",
				erase: "delete_peer",
				permission_type: "vcc"
			},
			class_name: "itemPeers",
			i_class_name: "far fa-phone-office",
			view: {
				request: {
					timeout: get_request_timeout()
				},
				header: {
					hide_new_button: false,
					hide_total_rows: true
				},
				global_buttons: [{
					id: "Peersrangeupdate",
					name: "<%=_T('Range Update')%>",
					className : "buttonColorWhite pageButtonNoIcon",
					run: function(view) {
						view.form.edit_range(view);
					},
					before: false
				}],
				paginator: {
					paged_text_format : "<%=_T('{page_row_start}-{page_row_end} of {many}')%>"
				},
				render: "/inconcert/apps/configuration/pages/view_content_default.<%=_L%>.html#grunt-cache-bust",
				title: "<%=_T('IP Phones View')%>",
				tagline: "<%=_T('IP Phones')%>",
				new_button_name: "<%=_T('New IP Phone')%>",
				row_id: "address_id",
				display_name: "address_id",
				messages: {
					erase: "<%=_T('Do you want to delete IP Phone {0} ?')%>",
					err: get_message_error
				},
				uri: {
					erase:  base_uri + "peers/delete/",
					search: base_uri + "peers/search/"
				},
				source: ["id", "hostId", "Name", "tech", "codecs", "name", "resource_address", "address_id", "VCC", "assignationType"],
				columns: [
					{id: "_number"         , name: "", type: "counter"          },
					{id: "address_id"      , name: "<%=_T('Address Id')%>"      },
					{id: "hostId"          , name: "<%=_T('Host Name')%>"       },
					{id: "codecs"          , name: "<%=_T('Codecs')%>"          },
					{id: "resource_address", name: "<%=_T('Resource Address')%>"},
					{id: "VCC"             , name: "<%=_T('Assigned to VCC')%>",
						get_value: function(peer) {
							let info = {
								value: "", original_value: ""
							};
							if (!peer.VCC) {
								return info;
							}
							if (!peer.assignationType) {
								info.value = peer.VCC;
								info.original_value = info.value;
								return info;
							}
							let translated_assignation = translated_assignation_types[peer.assignationType] || peer.assignationType;
							info.value = FORMAT("{0} ({1})", peer.VCC, translated_assignation);
							info.original_value = info.value;
							return info;
						}
					},
					{id: "_action", name: "", system_action: {edit: true, erase: true},}
				],
				filter: {
					fields: {
						"address_id"      : {control: null},
						"hostId"          : {control: null},
						"codecs"          : {control: null},
						"resource_address": {control: null},
						"VCC"             : {control: null}
					}
				},
				realtime: null
			},
			form: {instance: frm_instance}
		};
	};

	//------------------------------------------------------
	// gateways navigation config
	//
	var gateways_config = function(base_uri)
	{
		var get_message_error = function(type, err, name = "")
		{
			let message, related_id;
			if (type === "delete") {
				let err_messages = {
					"FORBIDDEN": "<%=_T('Unable to delete {0} Gateway. You dont have permission.')%>",
					"OPEN_CONNECTION_ERROR": "<%=_T('Unable to delete {0} Gateway. Error connecting to gateway.')%>",
					"LOGIN_ERROR": "<%=_T('Unable to delete {0} Gateway. Error doing login.')%>",
					"GATEWAY DOES NOT EXIST":
						"<%=_T('Unable to delete {0} GATEWAY. GATEWAY does not exist.')%>",
					"CANNOT DELETE GATEWAY ASSOCIATED TO TRUNK":
						"<%=_T('Unable to delete {0} GATEWAY. Cannot delete GATEWAY while TRUNK {1} is still associated with it.')%>",
					
				};
				let errorCode = err;
				if (typeof err === "object") {
					related_id = err.relatedId;
					errorCode = err.result || "";
				}
				related_id = related_id || "";
				message = err_messages[errorCode.toUpperCase()] ||
					"<%=_T('Error deleting {0} GATEWAY.')%> " + (err?.reason || "Unknown error");
				message = FORMAT(message, name, related_id);
			}
			return message;
		};

		const frm_instance = inConcert.form_provisioning_gateways;

		return {
			id: "provisioning_gateways",
			name: "<%=_T('Gateways')%>",
			permissions: {
				view: "view_gateways",
				save: "save_gateway",
				erase: "delete_gateway",
				permission_type: "vcc"
			},
			class_name: "itemGateways",
			i_class_name: "far fa-router",
			view: {
				request: {
					timeout: get_request_timeout()
				},
				header: {
					hide_new_button: false,
					hide_total_rows: true
				},
				paginator: {
					paged_text_format : "<%=_T('{page_row_start}-{page_row_end} of {many}')%>"
				},
				render: "/inconcert/apps/configuration/pages/view_content_default.<%=_L%>.html#grunt-cache-bust",
				title: "<%=_T('Gateways')%>",
				tagline: "<%=_T('Gateways')%>",
				new_button_name: "<%=_T('New Gateway')%>",
				row_id: "Id",
				display_name: "Id",
				messages: {
					erase: "<%=_T('Do you want to delete Gateway {0} ?')%>",
					err: get_message_error
				},
				uri: {
					erase:  base_uri + "gateways/delete/",
					search: base_uri + "gateways/search/"
				},
				source: ["Id", "Description", "Domain"],
				columns: [
					{id: "_number"    , name: "", type: "counter"     },
					{id: "Id"         , name: "<%=_T('Name')%>"       },
					{id: "Description", name: "<%=_T('Description')%>"},
					{id: "Domain"     , name: "<%=_T('IP/Domain')%>"     },
					{id: "_action"    , name: "",
						system_action: {edit: true, erase: true},
						/*custom_action: {
							monitor: {
								tooltip: "<%=_T('Monitor')%>",
								execute: function(_, $row, view, _$button) {
									const gatewayId = $row.attr("row_id");
									monitoring_instance.open(gatewayId, view);
								},
								class_name: "chart"
							},
						}*/
					}
				],
				filter: {
					fields: {
						"Id"         : {control: null},
						"description": {control: null},
						"Domain"     : {control: null}
					}
				},
				realtime: null
			},
			form: {instance: frm_instance}
		};
	};

	//------------------------------------------------------
	// trunks navigation config
	//
	var trunks_config = function(base_uri)
	{
		var get_message_error = function(type, err, name = "")
		{
			let message, related_id;
			if (type === "delete") {
				let err_messages = {
					"FORBIDDEN"                            : "<%=_T('Unable to delete {0} IP Trunk. You dont have permission.')%>",
					"TRUNK DOES NOT EXIST"                 : "<%=_T('Unable to delete {0} IP Trunk. IP Trunks does not exist.')%>",
					"CANNOT DELETE TRUNK ASSOCIATED TO DID": "<%=_T('Unable to delete {0} IP Trunk. Cannot delete IP Trunk while VCC {1} is still associated with it.')%>"
				};
				if (typeof err === "object") {
					related_id = err.relatedId;
					err = err.result || "";
				}
				related_id = related_id || "";
				message = err_messages[err.toUpperCase()] || "<%=_T('Error deleting {0} IP Trunk.')%>";
				message = FORMAT(message, name, related_id);
			}
			return message;
		};

		const frm_instance = inConcert.form_provisioning_trunks;

		return {
			id: "provisioning_trunks",
			name: "<%=_T('IP Trunks')%>",
			permissions: {
				view: "view_trunks",
				save: "save_trunk",
				erase: "delete_trunk",
				permission_type: "vcc"
			},
			class_name: "itemTrunks",
			i_class_name: "far fa-exchange",
			view: {
				request: {
					timeout: get_request_timeout()
				},
				header: {
					hide_new_button: false,
					hide_total_rows: true
				},
				paginator: {
					paged_text_format : "<%=_T('{page_row_start}-{page_row_end} of {many}')%>"
				},
				render: "/inconcert/apps/configuration/pages/view_content_default.<%=_L%>.html#grunt-cache-bust",
				title: "<%=_T('IP Trunks View')%>",
				tagline: "<%=_T('IP Trunks')%>",
				new_button_name: "<%=_T('New IP Trunk')%>",
				row_id: "id",
				display_name: "id",
				messages: {
					erase: "<%=_T('Do you want to delete IP Trunk {0} ?')%>",
					err: get_message_error
				},
				uri: {
					erase:  base_uri + "trunks/delete/",
					search: base_uri + "trunks/search/"
				},
				source: ["id", "hostId", "Name", "name", "gateway", "countryCode", "areaCode", "codecs"],
				columns: [
					{id: "_number"    , name: "", type: "counter"          },
					{id: "id"         , name: "<%=_T('Id')%>"              },
					{id: "name"       , name: "<%=_T('Name')%>"            },
					{id: "hostId"     , name: "<%=_T('Host Name')%>"       },
					{id: "codecs"     , name: "<%=_T('Codecs')%>"          },
					{
						id: "gateway",
						name: "<%=_T('Mode')%>",
					 	get_value : ({gateway}) => gateway
							? { value: "<%=_T('Gateway')%>" } 
							: { value: "<%=_T('SIP')%>" }
					},
					{id: "countryCode", name: "<%=_T('Country Code')%>"    },
					{id: "areaCode"   , name: "<%=_T('Area Code')%>"       },
					{id: "_action"    , name: "",
						system_action: {edit: true, erase: true}
					}
				],
				filter: {
					fields: {
						"id"         : {control: null},
						"hostId"     : {control: null},
						"name"       : {control: null},
						"gateway"    : {control: null},
						"countryCode": {control: null},
						"areaCode"   : {control: null}
					}
				},
				realtime: null
			},
			form: {instance: frm_instance}
		};
	};

	//------------------------------------------------------
	// trunks_groups navigation config
	//
	var trunks_groups_config = function(base_uri)
	{
		var get_message_error = function(type, err, name = "")
		{
			let message, related_id;
			if (type === "delete") {
				let err_messages = {
					"FORBIDDEN"                                  : "<%=_T('Unable to delete {0} Trunk Group. You dont have permission.')%>",
					"TRUNK GROUP DOES NOT EXIST"                 : "<%=_T('Unable to delete {0} Trunk Group. Trunk Group does not exist.')%>",
					"CANNOT DELETE TRUNK GROUP ASSOCIATED TO DID": "<%=_T('Unable to delete {0} Trunk Group. Cannot delete Trunk Group while VCC {1} is still associated with it.')%>"
				};
				if (typeof err === "object") {
					related_id = err.relatedId;
					err = err.result || "";
				}
				related_id = related_id || "";
				message = err_messages[err.toUpperCase()] || "<%=_T('Error deleting {0} Trunk Group.')%>";
				message = FORMAT(message, name, related_id);
			}
			return message;
		};

		const frm_instance = inConcert.form_provisioning_trunks_groups;

		return {
			id: "provisioning_trunks_groups",
			name: "<%=_T('Trunk Group')%>",
			permissions: {
				view: "view_trunks_groups",
				save: "save_trunk_group",
				erase: "delete_trunk_group",
				permission_type: "vcc"
			},
			class_name: "itemTrunkGroups",
			i_class_name: "far fa-sitemap",
			view: {
				request: {
					timeout: get_request_timeout()
				},
				header: {
					hide_new_button: false,
					hide_total_rows: true
				},
				paginator: {
					paged_text_format : "<%=_T('{page_row_start}-{page_row_end} of {many}')%>"
				},
				render: "/inconcert/apps/configuration/pages/view_content_default.<%=_L%>.html#grunt-cache-bust",
				title: "<%=_T('Trunks Groups View')%>",
				tagline: "<%=_T('Trunks Groups')%>",
				new_button_name: "<%=_T('New Trunk Group')%>",
				row_id: "id",
				display_name: "id",
				messages: {
					erase: "<%=_T('Do you want to delete Trunk Group {0} ?')%>",
					err: get_message_error
				},
				uri: {
					erase:  base_uri + "trunksgroups/delete/",
					search: base_uri + "trunksgroups/search/"
				},
				source: ["id", "carrier"],
				columns: [
					{id: "_number"    , name: "", type: "counter"        },
					{id: "id"         , name: "<%=_T('Trunk Group Id')%>"},
					{id: "carrier"    , name: "<%=_T('Carrier')%>"       },
					{id: "_action"    , name: "",
						system_action: {edit: true, erase: true}
					}
				],
				filter: {
					fields: {
						"id"     : {control: null},
						"carrier": {control: null}
					}
				},
				realtime: null
			},
			form: {instance: frm_instance}
		};
	};

	//------------------------------------------------------
	// dids navigation config
	//
	var dids_config = function(base_uri)
	{
		var get_message_error = function(type, err, name = "")
		{
			let message, related_id;
			if (type === "delete") {
				let err_messages = {
					"FORBIDDEN": "<%=_T('Unable to delete {0} DID. You dont have permission.')%>",
					"DID DOES NOT EXIST": "<%=_T('Unable to delete {0} DID. DID does not exist.')%>",
					"CANNOT DELETE DID ASSOCIATED TO PACKAGE": "<%=_T('Unable to delete {0}. Cannot delete DID while Package {1} is still associated with it.')%>"
				};
				if (typeof err === "object") {
					related_id = err.relatedId;
					err = err.result || "";
				}
				related_id = related_id || "";
				message = err_messages[err.toUpperCase()] || "<%=_T('Error deleting {0} DID.')%>";
				message = FORMAT(message, name, related_id);
			}
			return message;
		};

		const frm_instance = inConcert.form_provisioning_dids;

		return {
			id: "provisioning_dids",
			name: "<%=_T('DIDs')%>",
			permissions: {
				view: "view_dids",
				save: "save_did",
				erase: "delete_did",
				permission_type: "vcc"
			},
			class_name: "itemDIDs",
			i_class_name: "far fa-tty",
			view: {
				request: {
					timeout: get_request_timeout()
				},
				header: {
					hide_new_button: false,
					hide_total_rows: true
				},
				paginator: {
					paged_text_format : "<%=_T('{page_row_start}-{page_row_end} of {many}')%>"
				},
				render: "/inconcert/apps/configuration/pages/view_content_default.<%=_L%>.html#grunt-cache-bust",
				title: "<%=_T('DIDs View')%>",
				tagline: "<%=_T('DIDs')%>",
				new_button_name: "<%=_T('New DID')%>",
				row_id: "Id",
				display_name: "Id",
				messages: {
					erase: "<%=_T('Do you want to delete DID {0} ?')%>",
					err: get_message_error
				},
				uri: {
					erase:  base_uri + "dids/delete/",
					search: base_uri + "dids/search/"
				},
				source: ["id", "Id", "Name", "DNIS", "carrier", "trunk", "trunkGroup"],
				columns: [
					{id: "_number"    , name: "", type: "counter"     },
					{id: "Id"         , name: "<%=_T('Id')%>"         },
					{id: "DNIS"       , name: "<%=_T('DNIS')%>"       },
					{id: "carrier"    , name: "<%=_T('Carrier')%>"    },
					{id: "trunk"      , name: "<%=_T('Trunk')%>"      },
					{id: "trunkGroup" , name: "<%=_T('Trunk Group')%>"},
					{id: "_action"    , name: "",
						system_action: {edit: true, erase: true}
					}
				],
				filter: {
					fields: {
						"id"        : {control: null},
						"Id"        : {control: null},
						"DNIS"      : {control: null},
						"carrier"   : {control: null},
						"trunk"     : {control: null},
						"trunkGroup": {control: null}
					}
				},
				realtime: null
			},
			form: {instance: frm_instance}
		};
	};

	//------------------------------------------------------
	// routes navigation config
	//
	var routes_config = function(base_uri)
	{
		var get_message_error = function(type, err, name = "")
		{
			let message, related_id;
			if (type === "delete") {
				let err_messages = {
					"FORBIDDEN": "<%=_T('Unable to delete {0} Route. You dont have permission.')%>",
					"ROUTE DOES NOT EXIST": "<%=_T('Unable to delete {0} Route. Route does not exist.')%>",
					"CANNOT DELETE ROUTE ASSOCIATED TO PACKAGE": "<%=_T('Unable to delete {0}. Cannot delete Route while Package {1} is still associated with it.')%>"
				};
				if (typeof err === "object") {
					related_id = err.relatedId;
					err = err.result || "";
				}
				related_id = related_id || "";
				message = err_messages[err.toUpperCase()] || "<%=_T('Error deleting {0} Route.')%>";
				message = FORMAT(message, name, related_id);
			}
			return message;
		};

		const frm_instance = inConcert.form_provisioning_routes;

		return {
			id: "provisioning_routes",
			name: "<%=_T('Routes')%>",
			permissions: {
				view: "view_routes",
				save: "save_route",
				erase: "delete_route",
				permission_type: "vcc"
			},
			class_name: "itemRoutes",
			i_class_name: "far fa-map-signs",
			view: {
				request: {
					timeout: get_request_timeout()
				},
				header: {
					hide_new_button: false,
					hide_total_rows: true
				},
				paginator: {
					paged_text_format : "<%=_T('{page_row_start}-{page_row_end} of {many}')%>"
				},
				render: "/inconcert/apps/configuration/pages/view_content_default.<%=_L%>.html#grunt-cache-bust",
				title: "<%=_T('Routes View')%>",
				tagline: "<%=_T('Routes')%>",
				new_button_name: "<%=_T('New Route')%>",
				row_id: "Id",
				display_name: "Id",
				messages: {
					erase: "<%=_T('Do you want to delete Route {0} ?')%>",
					err: get_message_error
				},
				uri: {
					erase:  base_uri + "routes/delete/",
					search: base_uri + "routes/search/"
				},
				source: ["Id", "description", "Name"],
				columns: [
					{id: "_number"    , name: "", type: "counter"},
					{id: "Id"         , name: "<%=_T('Id')%>"    },
					{id: "description", name: "<%=_T('Description')%>"  },
					{id: "_action"    , name: "",
						system_action: {edit: true, erase: true}
					}
				],
				filter: {
					fields: {
						"Id"         : {control: null},
						"description": {control: null}
					}
				},
				realtime: null
			},
			form: {instance: frm_instance}
		};
	};

	//------------------------------------------------------
	// dialplan processors navigation config
	//
	var dialplan_processors_config = function(base_uri)
	{
		var get_message_error = function(type, err, name = "")
		{
			let message, related_id;
			if (type === "delete") {
				let err_messages = {
					"FORBIDDEN": "<%=_T('Unable to delete {0} processor. You dont have permission.')%>",
					"PROCESSOR DOES NOT EXIST": "<%=_T('Unable to delete {0} processor. Processor does not exist.')%>",
					"CANNOT DELETE DIALPLAN PROCESSOR ASSOCIATED TO PACKAGE": "<%=_T('Unable to delete {0}. Cannot delete processor while Package {1} is still associated with it.')%>"
				};
				if (typeof err === "object") {
					related_id = err.relatedId;
					err = err.result || "";
				}
				related_id = related_id || "";
				message = err_messages[err.toUpperCase()] || "<%=_T('Error deleting {0} processor.')%>";
				message = FORMAT(message, name, related_id);
			}
			return message;
		};

		const frm_instance = inConcert.form_provisioning_dialplan_processors;

		// execute custom action
		var execute_custom_action = function( action_id, $row, view )
		{
			var project_id = $row.attr( "row_id" );
			var display_name = $row.attr("row_name");
			switch ( action_id ) {
				case "export":
					if ( !view.form.has_permission_to_save() ) {
						view.show_status_message(
							FORMAT( "<%=_T('Unable to export {0} dialplan. You dont have permission.')%>", display_name),
							"Error",
							5000
						);
						return;
					}
					inConcert.app.getProvisioningAPI().dialplanProcessorsExport( project_id );
					break;
			}
		};

		return {
			id: "provisioning_dialplan_processors",
			name: "<%=_T('Dialplan Processors')%>",
			permissions: {
				view: "view_dialplan_processors",
				save: "save_dialplan_processors",
				erase: "delete_dialplan_processors",
				"export": "export_dialplan_processors",
				permission_type: "vcc"
			},
			class_name: "itemDialplanProcessors",
			i_class_name: "far fa-laptop-code",
			view: {
				request: {
					timeout: get_request_timeout()
				},
				header: {
					hide_new_button: false,
					hide_total_rows: true
				},
				paginator: {
					paged_text_format : "<%=_T('{page_row_start}-{page_row_end} of {many}')%>"
				},
				render: "/inconcert/apps/configuration/pages/view_content_default.<%=_L%>.html#grunt-cache-bust",
				title: "<%=_T('Dialplan Processors View')%>",
				tagline: "<%=_T('Dialplan Processors')%>",
				new_button_name: "<%=_T('New Dialplan')%>",
				row_id: "Id",
				display_name: "Id",
				messages: {
					erase: "<%=_T('Do you want to delete Dialplan {0} ?')%>",
					err: get_message_error
				},
				uri: {
					erase:  base_uri + "dialplan_processors/delete/",
					search: base_uri + "dialplan_processors/search/"
				},
				source: ["Id", "description", "Name"],
				columns: [
					{id: "_number"    , name: "", type: "counter"},
					{id: "Id"         , name: "<%=_T('Id')%>"    },
					{id: "description", name: "<%=_T('Name')%>"  },
					{id: "_action"    , name: "",
						system_action: {edit: true, erase: true},
						custom_action: {
							"export": {
								name: "<%=_T('Export')%>",
								execute: execute_custom_action,
								class_name: "export"
							}
						}
					}
				],
				filter: {
					fields: {
						"Id"         : {control: null},
						"description": {control: null}
					}
				},
				realtime: null
			},
			form: {
				instance: frm_instance,
				cancel_button: {
					caption : "<%=_T('Close')%>"
				}
			}
		};
	};

	//------------------------------------------------------
	// packages navigation config
	//
	var packages_config = function(base_uri)
	{
		var get_message_error = function(type, err, name = "") {
			let message, related_id;
			if (type === "delete") {
				let err_messages = {
					"FORBIDDEN": "<%=_T('Unable to delete {0} Package. You dont have permission.')%>",
					"PACKAGE DOES NOT EXIST": "<%=_T('Unable to delete {0} Package. Package does not exist.')%>",
					"CANNOT DELETE PACKAGE ASSOCIATED TO VCC": "<%=_T('Unable to delete {0}. Cannot delete Package while VCC {1} is still associated with it.')%>"
				};
				if (typeof err === "object") {
					related_id = err.relatedId;
					err = err.result || "";
				}
				related_id = related_id || "";
				message = err_messages[err.toUpperCase()] || "<%=_T('Error deleting {0} Package.')%>";
				message = FORMAT(message, name, related_id);
			}
			return message;
		};

		var get_mode_value = function(row) {
			let original_value = row.ioMode || "";
			let value = original_value;
			if (original_value.trim().length > 0) {
				if (value === "I") {
					value = "<%=_T('I')%>";
				}
				else if (value === "O") {
					value = "<%=_T('O')%>";
				}
				else if (value === "I/O") {
					value = "<%=_T('I/O')%>";
				}
			}
			return {
				original_value: original_value, value: value
			};
		};

		const frm_instance = inConcert.form_provisioning_packages;

		return {
			id: "provisioning_packages",
			name: "<%=_T('Packages')%>",
			permissions: {
				view: "view_packages",
				save: "save_package",
				erase: "delete_package",
				permission_type: "vcc"
			},
			class_name: "itemPackages",
			i_class_name: "far fa-tasks",
			view: {
				request: {
					timeout: get_request_timeout()
				},
				header: {
					hide_new_button: false,
					hide_total_rows: true
				},
				paginator: {
					paged_text_format : "<%=_T('{page_row_start}-{page_row_end} of {many}')%>"
				},
				render: "/inconcert/apps/configuration/pages/view_content_default.<%=_L%>.html#grunt-cache-bust",
				title: "<%=_T('Packages View')%>",
				tagline: "<%=_T('Packages')%>",
				new_button_name: "<%=_T('New Package')%>",
				row_id: "Id",
				display_name: "Id",
				messages: {
					erase: "<%=_T('Do you want to delete Package {0} ?')%>",
					err: get_message_error
				},
				uri: {
					erase:  base_uri + "packages/delete/",
					search: base_uri + "packages/search/"
				},
				source: ["Id", "description", "Name", "DID", "capacity", "ioMode", "outgoingRoute", "VCC"],
				columns: [
					{id: "_number"      , name: "", type: "counter"         },
					{id: "Id"           , name: "<%=_T('Id')%>"             },
					{id: "description"  , name: "<%=_T('Description')%>"    },
					{id: "DID"          , name: "<%=_T('DID')%>"            },
					{id: "capacity"     , name: "<%=_T('Capacity')%>"       },
					{id: "ioMode"       , name: "<%=_T('Mode')%>",
						get_value: get_mode_value
					},
					{id: "outgoingRoute", name: "<%=_T('Outgoing Route')%>" },
					{id: "VCC"          , name: "<%=_T('Assigned to VCC')%>"},
					{id: "_action"      , name: "",
						system_action: {edit: true, erase: true}
					}
				],
				filter: {
					fields: {
						"Id"           : {control: null},
						"description"  : {control: null},
						"DID"          : {control: null},
						"capacity"     : {control: null},
						"ioMode"       : {control: null},
						"outgoingRoute": {control: null},
						"VCC"          : {control: null}
					}
				},
				realtime: null
			},
			form: {instance: frm_instance}
		};
	};

	const environment_config = function()
	{
		return {
			id: "environment",
			name: "<%=_T('Environment')%>",
			permissions: {
				view: "view_environment"
			},
			class_name: "itemEnvironment",
			i_class_name: "far fa-cog",
			only_frm: true,
			view: {
				render: "/inconcert/apps/configuration/pages/view_content_default.<%=_L%>.html#grunt-cache-bust"
			},
			form: {
				instance: inConcert.form_provisioning_environment
			}
		};
	};
	
	//------------------------------------------------------
	// get all navigation config
	var get_navigation_config = function()
	{
		const base_uri = inConcert.app.getProvisioningUrl() + "/";
		return {
			vccs               : vccs_config(base_uri),
			carriers           : carriers_config(base_uri),
			system_monitoring  : system_monitoring(base_uri),
			ice_servers        : ice_servers_config(base_uri),
			hosts              : hosts_config(base_uri),
			gateways           : gateways_config(base_uri),
			peers              : peers_config(base_uri),
			trunks             : trunks_config(base_uri),
			trunks_groups      : trunks_groups_config(base_uri),
			dids               : dids_config(base_uri),
			routes             : routes_config(base_uri),
			dialplan_processors: dialplan_processors_config(base_uri),
			packages           : packages_config(base_uri),
			ttsasrt_engines    : ttsasrengines_config(base_uri),
			environment        : environment_config(base_uri)
		};
	};

	function provisioning_config() {}

	if (typeof inConcert.provisioning_config !== "function") {
		inConcert.provisioning_config = provisioning_config;
	}

	provisioning_config.prototype.get_app_config = function()
	{
		return {
			id             : "provisioning",
			name           : "<%=_T('Provisioning')%>",
			selected       : false,
			class_name     : "appProvisioning",
			main_html      : "/inconcert/apps/configuration/pages/conf_main.<%=_L%>.html#grunt-cache-bust",
			navigation     : get_navigation_config(),
			allow_disable_all_navigation : true,
			get_skin: function() {
				return "skinOCCNew";
			},
			can_show       : function() {
				return inConcert.permissions.to_category("provisioning", "view");
			},
			css : [
				//{ id: "internal_forms_css"             , href: "/inconcert/apps/configuration/css/masterInternalPageForms.css#grunt-cache-bust"                 },
				//{ id: "provisioning_master_css"        , href: "/inconcert/apps/provisioning/css/provisioningMaster.css#grunt-cache-bust"                       },
				// Agrega Lolo para SkinOCCNew
				{ id: "skin_occ_internal_forms_css"      , href: "/inconcert/apps/configuration/css/skin-2021/masterInternalPageForms.css#grunt-cache-bust"       },
				//{ id: "skin_occ_internal_page_layout_css", href: "/inconcert/apps/configuration/css/skin-2021/masterInternalPageLayout.css#grunt-cache-bust"      },
				{ id: "skin_occ_provisioning_master_css" , href: "/inconcert/apps/provisioning/css/skin-2021/provisioningMaster.css#grunt-cache-bust"             },
				//Estos hay que ver si se pueden sacar de la inboxMaster.html
				{ id: "skin_occ_master_buttons_css"      , href: "/inconcert/apps/agent/css/skin-2021/masterButtons.css#grunt-cache-bust"                         },
				{ id: "skin_occ_inbox_master_css"        , href: "/inconcert/apps/agent/css/skin-2021/inboxMaster.css#grunt-cache-bust"                           },
				{ id: "skin_occ_master_quick_interactions_css", href: "/inconcert/apps/agent/css/skin-2021/masterQuickInteractions.css#grunt-cache-bust"  },
				{ id: "skin_occ_messages_tooltips_css"   , href: "/inconcert/apps/agent/css/skin-2021/genericMessagesAndTooltips.css#grunt-cache-bust"            },
				{ id: "skin_occ_inbox_master_window_css"  , href: "/inconcert/apps/agent/css/skin-2021/masterWindowCommon.css#grunt-cache-bust"                   },
				{ id: "skin_occ_adm_master_css"          , href: "/inconcert/apps/administration/css/skin-2021/administrationMaster.css#grunt-cache-bust"         },
				{ id: "skin_occ_schedule_config_css"     , href: "/inconcert/apps/administration/css/skin-2021/administrationScheduleConfig.css#grunt-cache-bust" },
				{ id: "skin_occ_inbox_master_popups_css"     , href: "/inconcert/apps/agent/css/skin-2021/inboxMasterPopUps.css#grunt-cache-bust" },
				// TODO: revisar. Aca se agrego el CSS de administration porque los popups de edicion no se visualizan correctamente "internalPopUpContainer"
				// { id: "adm_master_css"                   , href: "/inconcert/apps/administration/css/administrationMaster.css#grunt-cache-bust"                   },
				// { id: "adm_schedule_css"                 , href: "/inconcert/apps/administration/css/administrationScheduleConfig.css#grunt-cache-bust"           }
			]
		};
	};
}());
