/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, esversion: 9 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	const FORMAT = inConcert.Utils.format;

	//---------------------------------------------------------------------
	// Scheduler Management
	function ProvisioningRouteSchedulersMng(params)
	{
		if (!params) {
			throw("ProvisioningRouteSchedulersMng - 'params' is required");
		}
		if (!params.owner) {
			throw("ProvisioningRouteSchedulersMng - 'params.owner' is required");
		}
		if (!params.prefix) {
			throw("ProvisioningRouteSchedulersMng - 'params.prefix' is required");
		}
		if (!params.container_id) {
			throw("ProvisioningRouteSchedulersMng - 'params.container_id' is required");
		}

		this.owner = params.owner;
		this.owner_name = params.owner_name || "";
		this.prefix = params.prefix + "_";
		this.container_id = params.container_id;
		this.schedule_prefix = this.prefix + "popup_schedule";
		this.schedule_container = this.schedule_prefix + "_container";

		this.m_grid_schedulers = null;
		this.m_schedule_hnd = null;
		this.m_trunks_priority_list = null;

		return true;
	}

	if (typeof inConcert.ProvisioningRouteSchedulersMng !== "function") {
		inConcert.ProvisioningRouteSchedulersMng = ProvisioningRouteSchedulersMng;
	}

	ProvisioningRouteSchedulersMng.prototype.initialize = function()
	{
		this.__make_popup_html();
		this.__init_scheduler();
		this.__init_grid();
		this.__init_popup_trunks_priority_list();
	};

	ProvisioningRouteSchedulersMng.prototype.get_grid_dom_id = function()
	{
		return this.prefix + "list";
	};

	ProvisioningRouteSchedulersMng.prototype.__make_popup_html = function()
	{
		let html = '' +
		'<div id="' + this.prefix + 'new_container" class="card-block">' +
			'<div class="card-header card-header-md d-flex flex-row justify-content-between">' +
				'<h5><%=_T([[Schedulers Config]])%></h5>' +
				'<i id="' + this.prefix + 'new_list_validation" class="formFieldValidation formFieldError" hidden></i>' +
				'<button id="' + this.prefix + 'new" class="pageButtonGeneric buttonColorWhite pageButtonNew mt-1"><i class="fal fa-plus"></i><%=_T([[New Scheduler]])%></button>' +
			'</div>' +
			'<div id="' + this.prefix + 'container" class="">' +
				'<div id="' + this.prefix + 'not_results_found" class="inlineMessagesContainer messageInfo" style="display:none"></div>' +
				'<table id="' + this.get_grid_dom_id() + '" cellpadding="0" cellspacing="0" border="0" class="viewGrid">' +
					'<thead class="viewGridHeader"></thead>' +
					'<tbody class="viewGridBody"></tbody>' +
				'</table>' +
				'<div id="' + this.prefix + 'paginator"></div>' +
			'</div>' +
		'</div>' +
		'<div id="' + this.prefix + 'popup" class="internalPopUpContainer schedulePopUpContainer" style="display:none;">' +
			'<div class="internalPopUpOverlay"></div>' +
			'<div class="internalPopUp">' +
				'<div id="' + this.prefix + 'popup_waiting" class="internalWaitingContainer" style="display:none;">' +
					'<div waiting_message="true" class="internalWaitingMessage"></div>' +
					'<div class="internalWaitingAnimation"></div>' +
					'<div class="internalWaitingOverlay"></div>' +
				'</div>' +
				'<div id="' + this.prefix + 'popup_title" class="internalPopUpTitle">' +
					'<span popup_title_text></span>' +
				'</div>' +
				'<div class="internalPopUpMiddle pageFormContainer">' +
					'<div id="' + this.prefix + 'popup_message" class="genericFormMessagesContainer messageError" style="display:none;">' +
						'<label class="messageText"></label>' +
					'</div>' +	
					'<div class="card no-card">' +
						'<div class="card-header">' +
							'<h5><%=_T([[Location]])%></h5>' +
						'</div>' +
						'<div class="card-block">' +
							'<div class="row">' +
								'<div class="col-6">' +
									'<div class="form-group">' +
										'<label for="' + this.prefix + 'popup_country" class="required"><%=_T([[Country]])%><%=_T([[:]])%></label>' +
										'<select id="' + this.prefix + 'popup_country" class="form-control" placeholder="<%=_T([[Enter the country…]])%>"></select>' +
										'<i id="' + this.prefix + 'popup_country_validation" class="formFieldValidation formFieldError" hidden></i>' +
									'</div>' +
								'</div>' +
								'<div class="col-6">' +
									'<div class="form-group">' +
										'<label for="' + this.prefix + 'popup_area" class="required"><%=_T([[Area or locality]])%><%=_T([[:]])%></label>' +
										'<select id="' + this.prefix + 'popup_area" class="form-control" placeholder="<%=_T([[Enter the area code…]])%>"></select>' +
										'<i id="' + this.prefix + 'popup_area_validation" class="formFieldValidation formFieldError" hidden></i>' +
									'</div>' +
								'</div>' +
							'</div>' +
						'</div>' +
						'<hr/>' +
					'</div>' +
					'<div class="card no-card"">' +
						'<div class="card-header">' +
							'<h5><%=_T([[Schedules Trunks]])%></h5>' +
						'</div>' +
						'<div class="card-block">' +
							'<div class="row align-items-end">' +
								'<div class="col-12">' +
								'</div>' +
							'</div>' +
						'</div>' +

						'<div id="' + this.schedule_container + '">' +
						//<!-- el markup se genera en la clase "conf_form_schedule" -->
						'</div>' +
					'</div>' +
				'</div>' +
			'<div class="internalPopUpButtons">' +
				'<button id="' + this.prefix + 'popup_close" class="pageButtonGeneric buttonColorWhite float-right"><%=_T([[Close]])%></button>' +
				'<button id="' + this.prefix + 'popup_create_and_continue" class="pageButtonGeneric buttonColorWhite float-right"><%=_T([[Confirm & continue]])%></button>' +
				'<button id="' + this.prefix + 'popup_create" class="pageButtonGeneric buttonColorBlue float-right"><%=_T([[Confirm]])%></button>' +
			'</div>' +
		'</div>';

		const $container = $("#" + this.container_id);
		$container.html(html);
	};

	ProvisioningRouteSchedulersMng.prototype.__init_popup_trunks_priority_list = function()
	{
		this.trunks_priority_list_popup = new inConcert.PopupWindow({
			ownerId   : this.prefix + "trunks_priority_list", // container id
			eventName : "trunks_priority_list"                // evento click personalizado
		});
	};

	ProvisioningRouteSchedulersMng.prototype.__init_grid = function()
	{
		const self = this;

		this.m_grid_schedulers = new inConcert.conf_form_paginator({
			owner: this.owner,
			frm: this.owner,
			row_id: "UUID",
			display_name: "Country",
			rows_per_page: 2000,
			show_at_least: 1999,
			paginator_type: "ALL_ROWS_CLIENT_SIDE",
			without_paginator: true,
			loading_waiting_msg: "<%=_T('Please wait, loading Schedulers ...')%>",
			ids: {
				paginator: this.prefix + "paginator",
				grid: this.get_grid_dom_id(),
				not_results_found: this.prefix + "not_results_found",
				container_new_btn: this.prefix + "new_container",
				new_item: this.prefix + "new",
			//popup0
				popup: this.prefix + "popup",
				popup_msg: this.prefix + "popup_message",
				popup_create: this.prefix + "popup_create",
				popup_create_and_continue: this.prefix + "popup_create_and_continue",
				popup_title: this.prefix + "popup_title",
				popup_close: this.prefix + "popup_close",
				popup_input_id: this.prefix + "popup_question",
				popup_waiting: this.prefix + "popup_waiting"
			},
			strings: {
				not_results_found: "<%=_T('No technology-specific data were found.')%>",
				can_remove       : "<%=_T('Do you want to delete Scheduler: {0} ?')%>"
			},
			columns: [
				{id: "_number"           , name: "", type: "counter"      },
				{id: "DisplayCountry"    , name: "<%=_T('Country')%>"     },
				//{id: "DisplayCountryCode", name: "<%=_T('Country Code')%>"},
				{id: "DisplayArea"       , name: "<%=_T('Area')%>"        },
				//{id: "DisplayAreaCode"   , name: "<%=_T('Area Code')%>"   },
				{
					id: "_action",
					name: "",
					system_action: {
						edit: true,
						erase: true
					}
				}
			],
			callbacks: {
				save: function(callback, must_create_and_contiune, current_popup) {
					callback = callback || $.noop;
					self.add_or_update_row(callback, must_create_and_contiune, current_popup);
				},
				edit: function(row_data) {
					self.on_open_popup(row_data);
				},
				on_removed_row: function(/*row, callback = $.noop*/) {},
 				on_closed_popup: function() {
 					self.on_closed_popup();
 				}
			},
			validate: {
				required_msg: "<%=_T('Must add at least one Schedule.')%>",
				required: true
			}
		});
	};

	ProvisioningRouteSchedulersMng.prototype.start_grid_validation = function()
	{
		let response = {
			success: true
		};
		let success = this.m_grid_schedulers.start_validation().success;
		if (!success) {
			response.success = false;
			response.message = this.m_grid_schedulers.get_validation_message_err("required");
		}
		return response;
	};

	ProvisioningRouteSchedulersMng.prototype.on_closed_popup = function(/*current_popup*/)
	{
		this.m_is_new_schedule = null;
		this.m_current_schedule = null;
	};

	ProvisioningRouteSchedulersMng.prototype.add_or_update_row = function(callback/*, _, current_popup*/)
	{
		callback = callback || $.noop;

		const self = this;

		this.validate_schedule_to_add_or_update(function(response)
		{
			if (!response.success) {
				callback(false);
				return;
			}
			let msg_response = {
				msg : "",
				type: "Info"
			};
	
			if (self.m_is_new_schedule) {
				msg_response.msg = "<%=_T('Scheduler was successfully added.')%>";
				let must_add_many = false;
				callback(true, response.item, msg_response, must_add_many);
			}
			else {
				msg_response.msg = "<%=_T('Scheduler was successfully updated.')%>";
				self.m_current_schedule = inConcert.Utils.cloneObject(response.item);
				callback(true, self.m_current_schedule, msg_response);
			}
	
			self.clean_up_controls_schedule();
		});
	};

	ProvisioningRouteSchedulersMng.prototype.clean_up_controls_schedule = function()
	{
		const $country = $("#" + this.prefix + "popup_country");
		const $area = $("#" + this.prefix + "popup_area");

		this.owner.readonly_control($country, false, true);
		this.owner.readonly_control($area, false, true);

		$country.val("ANY").focus();
		this.__fill_combo_areas("ANY");
		$area.val("ANY");
		this.__cleanup_temporal_trunk_list();
	};

	ProvisioningRouteSchedulersMng.prototype.on_open_popup = function(item)
	{
		const $title = $("#" + this.prefix + "popup_title").find("[popup_title_text]");
		this.m_is_new_schedule = item ? false: true;
		this.m_current_schedule = item;

		this.clean_up_controls_schedule();
	
		if (this.m_is_new_schedule) {
			$title.text("<%=_T('New Scheduler')%>");
			// nuevo
			this.m_schedule_hnd.initialize();
			this.__set_event_trunks();
			this.fill_trunks_priority_list(this.m_trunks_priority_list);
			const $new_range_every_days = $(`#${this.m_schedule_hnd.prefix}new_range_every_days`);
			$new_range_every_days.text("<%=_T('New schedule')%>");
			$new_range_every_days.removeClass("buttonColorWhite").addClass("buttonColorBlue");
			return;
		}
		// editando
		$title.text("<%=_T('Update Scheduler')%>");
		const $new_range_every_days = $(`#${this.m_schedule_hnd.prefix}new_range_every_days`);
		$new_range_every_days.text("<%=_T('New schedule')%>");
		$new_range_every_days.removeClass("buttonColorWhite").addClass("buttonColorBlue");
		const $country = $("#" + this.prefix + "popup_country");
		const $area = $("#" + this.prefix + "popup_area");

		$country.val(item.Country);

		this.__fill_combo_areas(item.Country);
		$area.val(item.Area);

		this.owner.readonly_control($country, true);
		this.owner.readonly_control($area, true);

		let clientScheduler = item.ClientScheduler;
		this.m_schedule_hnd.set(clientScheduler);
	};

	ProvisioningRouteSchedulersMng.prototype.validate_schedule_to_add_or_update = function(callback = $.noop)
	{
		const self = this;

		this.hide_all_errors();
		let response = {success: false, $ctrl: null, item: null, msg: "<%=_T('Unable to Add Scheduler.')%>"};

		let item = {};
		let is_new = false;
		if (this.m_current_schedule) {
			item = this.m_current_schedule;
		}
		else {
			is_new = true;
		}

		const $country = $("#" + this.prefix + "popup_country");
		let country = $country.val();

		const $area = $("#" + this.prefix + "popup_area");
		let area = $area.val();

		if (is_new) {
			// COUNTRY validation -----
			//
			// Required validation
			let validation_result = inConcert.Utils.validate(country, "required");
			if (!validation_result.success) {
				// valida que el country tenga al menos un caracter
				response.msg = "<%=_T('Country is required.')%>";
				response.$ctrl = $country;
				this.show_ctrl_err(response);
				return callback(response);
			}

			if (country !== "ANY") {
				// AREA validation -----
				//
				// Required validation
				validation_result = inConcert.Utils.validate(area, "required");
				if (!validation_result.success) {
					// valida que el area tenga al menos un caracter
					response.msg = "<%=_T('Area code is required.')%>";
					response.$ctrl = $area;
					this.show_ctrl_err(response);
					return callback(response);
				}
			}

			// Already exist validation
			let current_all_schedule = this.m_grid_schedulers.get_items();
			current_all_schedule = inConcert.Utils.arrayToMap(current_all_schedule || [],
				function(sch) {
					return sch;
				},
				function(sch) {
					// el ID
					return self.make_sch_id(sch.Country, sch.Area);
				}
			);
			var show_already_error = function() {
				response.msg = "<%=_T('Schedule already exist for this country and area.')%>";
				response.$ctrl = $country;
				self.show_ctrl_err(response);
				return response;
			};

			let new_uuid = this.make_sch_id(country, area);
			const already = current_all_schedule[new_uuid];
			if (already) {
				return callback(show_already_error());
			}
			item.UUID = new_uuid;
		}

		this.m_schedule_hnd.validate_scheduler(function(success, clientScheduler) {
			if (!success) {
				let err = clientScheduler;
				response.msg = err.msg;
				self.show_ctrl_err(response);
				return callback(response);
			}

			let total_rows = self.m_grid_schedulers.count_rows();

			let displayCountry = country;
			if (displayCountry === "ANY") {
				if (total_rows > 0) {
					displayCountry = "<%=_T([[Any other country]])%>";
				}
				else {
					displayCountry =  "<%=_T([[All countries]])%>";
				}
			}

			let displayArea = area;
			if (displayArea === "ANY") {
				displayArea = "<%=_T([[All areas]])%>";
			}

			const row_scheduler = self.make_row_scheduler(item, country, displayCountry, area, displayArea, clientScheduler);

			response.success = true;
			response.msg = "";
			response.item = row_scheduler;
	
			callback(response);
		});
	};

	ProvisioningRouteSchedulersMng.prototype.__validate_scheduler = function(scheduler, callback)
	{
		if (!scheduler.scheduleDays || scheduler.scheduleDays.length === 0) {
			let response = {
				msg: "<%=_T('At least one day needs to be set in the schedule.')%>"
			};
			callback(false, response);
			return;
		}
		callback(true);
	};

	ProvisioningRouteSchedulersMng.prototype.__can_add_scheduled_range = function(info, callback)
	{
		const $trunks_priority_list = $("#" + this.prefix + "trunks_priority_list");
		const selected_item = $trunks_priority_list.attr("option_selected");
		if (!selected_item || selected_item === "") {
			info.msg = "<%=_T('Please select Trunk priority list.')%>";
			info.success = false;
			callback(info);
			return;
		}
		if (info.range) {
			info.range.additionalData = this.m_trunks_priority_list[selected_item].Trunks;
		}
		info.success = true;
		callback(info);
	};

	ProvisioningRouteSchedulersMng.prototype.__on_added_scheduled_range = function(callback = $.noop)
	{
		//this.__cleanup_temporal_trunk_list();
		callback();
	};

	ProvisioningRouteSchedulersMng.prototype.__cleanup_temporal_trunk_list = function()
	{
		const $item_empty = $("[trunks_priority_list_ul]").children(':first');
		$item_empty.click();
	};

	ProvisioningRouteSchedulersMng.prototype.hide_all_errors = function()
	{
		this.m_grid_schedulers.hide_popup_msg();
	};

	ProvisioningRouteSchedulersMng.prototype.show_ctrl_err = function(err_info)
	{
		if (!err_info) {
			return;
		}
		const $control = err_info.$ctrl;
		if ($control) {
			const error_id = $control.attr("id") + "_validation";
			const $error_indicator = $("#" + error_id);
			if ($error_indicator) {
				$error_indicator.show();
			}
			$control.focus();
		}
		this.m_grid_schedulers.show_popup_msg(err_info.msg, "Error"); //, ERROR_TIMEOUT_MS);
	};

	ProvisioningRouteSchedulersMng.prototype.dispose = function()
	{
		//TODO: implementar, si es necesario
	};

	ProvisioningRouteSchedulersMng.prototype.dispose = function()
	{
		//TODO: implementar, si es necesario
	};

	/*
	Recibe un json segun el formato de la rocka "provisioningmanager" y lo convierte segun lo espera la clase genercia de schedulers.

		Recibe:
			[
				{
					"0": {
						"from": "0:00",
						"militaryTo": 0,
						"to": "0:00",
						"trunks": {
							"a": "1",
							"asdf": "2"
					}
					}
				},
				...
				{
					"0": {
						"from": "0:00",
						"militaryTo": 0,
						"to": "0:00",
						"trunks": {
							"a": "1500"
						}
					}
				}
			]
		Lo convierte en:

			[
				{
					"startTime": "1970-01-01 00:00:00",
					"endTime": "1970-01-01 00:00:00",
					"startHour": "00:00",
					"endHour": "00:00",
					"dayOfWeek": 1,
					"enabled": 1,
					"additionalData": {
						"a": "1",
						"asdf": "2"
					}
				},
				..
				..
				{
					"startTime": "1970-01-01 00:00:00",
					"endTime": "1970-01-01 00:00:00",
					"startHour": "00:00",
					"endHour": "00:00",
					"dayOfWeek": 7,
					"enabled": 1,
					"additionalData": {
						"a": "1500"
					}
				}
			]
	*/
	ProvisioningRouteSchedulersMng.prototype.make_client_scheduler = function(serverScheduler)
	{
		if (!serverScheduler) {
			return;
		}

		let clientScheduleDays = [];

		let day_number;
		for (let i = 0; i < serverScheduler.length; i++) {
			let serverSchInfo = serverScheduler[i];
			if (serverSchInfo === null) {
				continue;
			}
			day_number = i + 1;
			for (let _ in serverSchInfo) {
				if (serverSchInfo.hasOwnProperty(_)) {
					let serverSchItemInfo = serverSchInfo[_];
					if (serverSchItemInfo !== null) {
						let additionalData = serverSchItemInfo.trunks;
						let scheduler_day = this.m_schedule_hnd.make_scheduler_day(serverSchItemInfo.from, serverSchItemInfo.to, day_number, additionalData);
						clientScheduleDays.push(scheduler_day);
					}
				}
			}
		}

		let clientScheduler = this.m_schedule_hnd.get_scheduler_structure();
		clientScheduler.scheduleDays = clientScheduleDays;
		return clientScheduler;
	};

	ProvisioningRouteSchedulersMng.prototype.make_sch_id = function(country, area)
	{
		return FORMAT("{0}_{1}", country, area).toUpperCase();
	};

	ProvisioningRouteSchedulersMng.prototype.display_any_code = function(code)
	{
		if (!code) {
			return "";
		}
		return code.toUpperCase() === "ANY" ? "-" : code;
	};

	ProvisioningRouteSchedulersMng.prototype.make_row_scheduler = function(current_row_item, country, displayCountry, area, displayArea, clientScheduler)
	{
		let row_item = current_row_item || {};

		const country_info = this.owner.get_country_info(country);
		if (country_info) {
			if (country === displayCountry) {
				if (country_info) {
					displayCountry = country_info.translated_name || displayCountry;
				}
			}
			if (area === displayArea) {
				const area_info = country_info.areasByCode[area];
				if (area_info) {
					displayArea = area_info || displayArea;
				}
			}
		}

		let displayCountryCode = this.display_any_code(country);
		let displayAreaCode = this.display_any_code(area);
		let uuid = this.make_sch_id(country, area);

		row_item.UUID               = uuid;
		row_item.Country            = country;
		row_item.Area               = area;
		row_item.DisplayCountry     = displayCountry;
		row_item.DisplayArea        = displayArea;
		row_item.DisplayCountryCode = displayCountryCode;
		row_item.DisplayAreaCode    = displayAreaCode;
		row_item.ClientScheduler    = clientScheduler;

		return row_item;
	};

	ProvisioningRouteSchedulersMng.prototype.set = function(entity)
	{
		const self = this;
		let schedulers_info = [];

		if (!entity || !entity.schedule) {
			this.m_grid_schedulers.initialize($.noop, schedulers_info);
			return;
		}

		var add_scheduler = function(country, displayCountry, area, displayArea, serverScheduler) {
			const clientScheduler = self.make_client_scheduler(serverScheduler);
			const row_scheduler = self.make_row_scheduler(null, country, displayCountry, area, displayArea, clientScheduler);
			schedulers_info.push(row_scheduler);
		};

		var HasRouteSlot = function (slot) {
			for (let dow in slot || {}) {
				if (slot.hasOwnProperty(dow)) {
					let dayData = slot[dow];
					for (let militaryFrom in dayData || {}) {
						if (dayData.hasOwnProperty(militaryFrom)) {
							return true;
						}
					}
				}
			}
			return false;
		};

		let hasPerCountry;
		for (let country in entity.schedule.perCountry || {}) {
			if (entity.schedule.perCountry.hasOwnProperty(country)) {
				let countryData = entity.schedule.perCountry[country];
				hasPerCountry = true;
				let hasPerArea;

				for (let area in countryData.perArea || {}) {
					if (countryData.perArea.hasOwnProperty(area)) {
						let serverScheduler = countryData.perArea[area];
						let displayCountry = country;
						let displayArea = area;
						add_scheduler(country, displayCountry, area, displayArea, serverScheduler);
						hasPerArea = true;
					}
				}
				if (HasRouteSlot(countryData.anyArea)) {
					let displayCountry = country;
					let displayArea = "";
					let area = "ANY";

					if (hasPerArea) {
						displayArea = "<%=_T([[Any other area]])%>";
					}
					else {
						displayArea = "<%=_T([[All areas]])%>";
					}

					let serverScheduler = countryData.anyArea;
					add_scheduler(country, displayCountry, area, displayArea, serverScheduler);
				}
			}
		}

		if (HasRouteSlot(entity.schedule.anyCountry)) {
			let displayCountry = "";
			let displayArea = "-";
			let country = "ANY";
			let area = "ANY";

			if (hasPerCountry) {
				displayCountry = "<%=_T([[Any other country]])%>";
			}
			else {
				displayCountry = "<%=_T([[All countries]])%>";
			}
			let serverScheduler = entity.schedule.anyCountry;
			add_scheduler(country, displayCountry, area, displayArea, serverScheduler);
		}

		this.m_grid_schedulers.initialize($.noop, schedulers_info);
	};

	ProvisioningRouteSchedulersMng.prototype.empty_controls = function()
	{
		this.m_grid_schedulers.remove_all_rows();
	};

	ProvisioningRouteSchedulersMng.prototype.__get_trunks_priority_list_item_markup = function(item_list){
		item_list = item_list || {};
		let template = "".concat(
			"<li trunks_priority_list_option='{{Id}}' ", !item_list.Id || item_list.Id === ""? "style='display:none'": "", ">",
				"<div>",
	  				"<label trunks_priority_list_option_title='true'>{{Name}}</label>",
				"</div>",
			"</li>"
		);

		const $item = $(Mustache.render(template, item_list));
		$item.attr("option_value", item_list.Id || "");
		const self = this;
		$item.inconcert_tooltip({ 
			className: 'bigTooltip',
			alignment: 'right-bottom',
			["remove-delay"]: 100000,
			content: function(ownerControl) {
			let priority_info_data = self.m_trunks_priority_list[item_list.Id];
			if (!priority_info_data) {
				return;
			}
			const trunks = priority_info_data.Trunks;
			if (!trunks) {
				return;
			}
			let trunks_priority = "";
			for (let trunk_id in trunks) {
				if (trunks.hasOwnProperty(trunk_id)) {
					let priority = trunks[trunk_id];
					trunks_priority = trunks_priority + FORMAT("<div><span>{0}</span> - <span><%=_T([[Priority]])%> {1}</span></div>", trunk_id, priority);
				}
			}
			return trunks_priority;
		}});

		return $item;
	};

	ProvisioningRouteSchedulersMng.prototype.__set_trunks_priority_list_selector_name = function(name, $trunks_priority_list_selector){
		name = name || "<%=_T('Select')%>...";
		$trunks_priority_list_selector = $trunks_priority_list_selector || $("#" + this.prefix + "trunks_priority_list_selector");
		$trunks_priority_list_selector.find("[list_selector_title]").html(name);
	};

	ProvisioningRouteSchedulersMng.prototype.__set_event_trunks = function()
	{
		const $trunks_priority_list_selector = $("#" + this.prefix + "trunks_priority_list_selector");
		const $trunks_priority_list = $("#" + this.prefix + "trunks_priority_list");
		const self = this;
		$trunks_priority_list_selector
		.off("click")
		.on("click",function() {
				if ($trunks_priority_list.is(":visible")) {
					self.trunks_priority_list_popup.hide();
				}
				else {
					// Show popup
					self.trunks_priority_list_popup.show();
				}
		});

		$trunks_priority_list
		.off("click")
		.on("click","[trunks_priority_list_option]", function() {
			const $this = $(this);
			const option_value = $this.attr("option_value");
			const priority_info = self.m_trunks_priority_list[option_value || ""] || {
				Id  : "",
				Name: "<%=_T('Select')%>...",
				Trunks: {},
				Routes: {}
			};
			
			self.__set_trunks_priority_list_selector_name(priority_info.Name);
			$trunks_priority_list.attr("option_selected", priority_info.Id);
			self.show_trunks_priority(priority_info);
			self.trunks_priority_list_popup.hide();
		});

		const $trunks_priority_new = $("#" + this.prefix + "trunks_priority_new");
		$trunks_priority_new
			.off("click")
			.on("click", function() {
				self.show_popup_new_trunks_priority_list();
			});
	};

	ProvisioningRouteSchedulersMng.prototype.set_events = function()
	{
		const self = this;
	
		this.m_grid_schedulers.set_events();

		const $country = $("#" + this.prefix + "popup_country");
		$country.change(function() {
			const country_code = $(this).val();
			window.setTimeout(function() {
				self.__fill_combo_areas(country_code);
			}, 50);
		});

		this.__set_event_trunks();
	};

	ProvisioningRouteSchedulersMng.prototype.make_popup_html_new_trunks_priority_list = function(popup_id)
	{
		let total_list_items = Object.keys(this.m_trunks_priority_list || {}).length;
		let new_list_name = FORMAT('<%=_T([[List {0}]])%>', total_list_items + 1);
		let title_name = "<%=_T([[Creating]])%> ( "+ new_list_name + ")";

		let popup_html = '' +
			'<div id="' + popup_id + '" class="systemPopWindow windowContainer schedulePopupWindow lightPopUpWindow" hidden>' +
				'<div class="windowTop">' +
					'<div class="windowTitle">'+
						'<button id="' + this.prefix + 'popup_trunk_priority_cancel" type="button" class="goBackButtonText"><i class="fas fa-chevron-left"></i></button>' +
						'<div id="' + this.prefix + 'popup_list_title" class="" list_name="' + new_list_name + '">' 
							+ title_name +
							'<span class="fa fa-question-circle" tooltip="<%=_T([[More trunks can be added to the list]])%>"></span>' +
						'</div>' +
					'</div>'+
				'</div>'+
				'<div id="' + this.prefix + 'popup_list_message" class="genericFormMessagesContainer messageError" style="display:none;">' +
					'<label class="messageText"></label>' +
				'</div>' +
				'<div class="windowMiddle">' +
					'<div class="card no-card">' +
						'<div class="card-block">' +
							'<div class="row">' +
								'<div class="col-12">' +
									'<div class="form-group">' +
										'<label for="' + this.prefix + 'popup_trunk" class="required"><%=_T([[Trunk]])%><%=_T([[:]])%></label>' +
										'<select id="' + this.prefix + 'popup_trunk" class="form-control"></select>' +
										'<i id="' + this.prefix + 'popup_trunk_validation" class="formFieldValidation formFieldError" hidden></i>' +
									'</div>' +
								'</div>' +
							'</div>' +
							'<div class="row">' +
								'<div class="col-12">' +
									'<div class="form-group">' +
										'<label for="' + this.prefix + 'popup_priority" class="required"><%=_T([[Priority]])%><%=_T([[:]])%></label>' +
										'<input id="' + this.prefix + 'popup_priority" placeholder="<%=_T([[Enter value]])%>..." class="form-control" type="text" />' +
										'<i id="' + this.prefix + 'popup_priority_validation" class="formFieldValidation formFieldError" hidden></i>' +
									'</div>' +
								'</div>' +
							'</div>' +
							'<div class="row">' +
								'<div class="col-12">' +
									'<div class="form-group">' +
										'<label for="' + this.prefix + 'popup_trunk_list_pre_view" class=""><%=_T("Pre-visualize")%><%=_T([[:]])%>'+
											'<span class="fa fa-question-circle" tooltip="<%=_T([[Trunks assigned to the list.]])%>"></span>' +
										'</label>' +
										'<div id="' + this.prefix + 'popup_trunk_list_pre_view" class="trunksPriorityPreViewList scheduleIntervalItem" hidden></div>' +
									'</div>' +
								'</div>' +
							'</div>' +
						'</div>' +
					'</div>' +
				'</div>' +
				'<div class="windowBottom d-flex justify-content-end">' +
					'<button class="pageButtonGeneric buttonColorWhite" id="' + this.prefix + 'popup_trunk_priority_add"><%=_T([[Add & continue]])%></button>' +
					'<button class="pageButtonGeneric buttonColorBlue" id="' + this.prefix + 'popup_trunk_priority_accept" disabled><%=_T([[Confirm]])%></button>' +
				'</div>' +
			'</div>';

		return popup_html;
	};

	ProvisioningRouteSchedulersMng.prototype.set_events_popup_new_trunks_priority_list = function()
	{
		const self = this;

		const $add = $("#" + this.prefix + "popup_trunk_priority_add");
		$add.off("click")
			.on("click", function() {
				self.popup_add_temporal_trunk_priority();
			});

		const $accept = $("#" + this.prefix + "popup_trunk_priority_accept");
		$accept.off("click")
			.on("click", function() {
				self.popup_update_trunks_priority_list();
			});

		const $cancel = $("#" + this.prefix + "popup_trunk_priority_cancel");
		$cancel.off("click")
			.on("click", function(e) {
				self.popup_cancel_temporal_trunk_priority();
				$(`#${self.schedule_prefix}_range_popup`).show();
				e.stopPropagation();
			});

		const $popup_trunk_list_pre_view = $("#" + this.prefix + "popup_trunk_list_pre_view");
		$(`#${this.prefix}popup_trunk_list_pre_view`)
			.off('click')
			.on('click', '[preViewChipDelete="true"]', function(e) {
				const $this = $(this);
				const $container = $this.parent();
				const trunk_id = $container.attr("trunk");
				delete self.m_temporal_trunks_priority_list[trunk_id.toLowerCase()];
				$container.remove();
				if ($popup_trunk_list_pre_view.find("[trunk]").length === 0) {
					$("#" + self.prefix + "popup_trunk_priority_accept").prop("disabled", true);
				}
				e.stopPropagation();
			});
	};

	ProvisioningRouteSchedulersMng.prototype.popup_cancel_temporal_trunk_priority = function()
	{
		this.m_temporal_trunks_priority_list = null;
		if (this.m_popup) {
			this.m_popup.finalize();
			$("#" + this.prefix + "trunks_priority_info").html("");
		}
	};

	ProvisioningRouteSchedulersMng.prototype.popup_add_temporal_trunk_priority = function()
	{
		this.popup_hide_all_errors();

		const $popup_msg = $("#" + this.prefix + "popup_list_message");
		let msg, ctrl_id;

		// TRUNK validation -----
		//
		// Required validation
		ctrl_id = this.prefix + "popup_trunk";
		const $trunk = $("#" + ctrl_id);
		let trunk = $trunk.val();
		let validation_result = inConcert.Utils.validate(trunk, "required");

		if (!validation_result.success) {
			// valida que el valor tenga al menos un caracter
			msg = "<%=_T('Trunk is required.')%>";
			this.popup_show_ctrl_err(msg, $trunk, $popup_msg);
			return;
		}

		// PRIORITY validation -----
		//
		// Required validation
		ctrl_id = this.prefix + "popup_priority";
		const $priority = $("#" + ctrl_id);
		let priority = $priority.val();
		validation_result = inConcert.Utils.validate(priority, "required");

		if (!validation_result.success) {
			// valida que el valor tenga al menos un caracter
			msg = "<%=_T('Priority is required.')%>";
			this.popup_show_ctrl_err(msg, $priority, $popup_msg);
			return;
		}

		let range = {between: {from: 1, to: 100}};
		validation_result = inConcert.Utils.validate(priority, "range", range);
		if (!validation_result.success) {
			// rango
			msg = FORMAT("<%=_T('Enter Priority between {0} and {1}.')%>", range.between.from, range.between.to);
			this.popup_show_ctrl_err(msg, $priority, $popup_msg);
			return;
		}

		// OK -- agrego al temporal
		const $title =  $("#" + this.prefix + "popup_list_title");
		const list_name = $title.attr("list_name");
		this.m_temporal_trunks_priority_list = this.m_temporal_trunks_priority_list || {};

		this.m_temporal_trunks_priority_list[trunk.toLowerCase()] = {
			ListName: list_name,
			Trunk   : trunk,
			Priority: $priority.val()
		};

		// esta include es temporal (solo visible) ya que si cancela, la misma se elimina
		const $popup_trunk_list_pre_view = $("#" + this.prefix + "popup_trunk_list_pre_view");
		const $current_trunk_priority = $popup_trunk_list_pre_view.find("[trunk='" + trunk + "'");
		const trunk_html = FORMAT("<label>{0}</label>:<span>{1}</span>"+
		"<button preViewChipDelete='true' class='butIntervalDelete' type='button'><i></i></button>" , trunk, priority);
		if ($current_trunk_priority.length > 0) {
			// si el trunk temporal ya se ha agregado a la lista, actualizo su valor
			$current_trunk_priority.html(trunk_html);
		}
		else {
			// si el trunk temporal NO existe en la lista, lo agrego al final de la misma
			let current_trunks_priority = $popup_trunk_list_pre_view.html();
			current_trunks_priority = current_trunks_priority + FORMAT("<div class='trunksPriorityPreViewChip' trunk='{0}'>"+ trunk_html +"</div>", trunk);
			$popup_trunk_list_pre_view.html(current_trunks_priority);
		}
		$(`#${this.prefix}popup_trunk_priority_accept`).prop("disabled", false);
		$priority.val("");
		$trunk.val("").focus();
	};

	ProvisioningRouteSchedulersMng.prototype.popup_hide_all_errors = function()
	{
		const $popup = $("#" + this.prefix + "trunks_priority_new_popup");
		$popup.find(".formFieldValidation").hide();
		this.m_schedule_hnd.hide_errors();
	};

	ProvisioningRouteSchedulersMng.prototype.popup_show_ctrl_err = function(message, $ctrl, $popup_msg)
	{
		if ($ctrl) {
			const error_id = $ctrl.attr("id") + "_validation";
			const $error_indicator = $("#" + error_id);
			if ($error_indicator) {
				$error_indicator.show();
			}
			$ctrl.focus();
		}
		this.m_schedule_hnd.show_error(message, undefined, $popup_msg);
	};

	ProvisioningRouteSchedulersMng.prototype.popup_update_trunks_priority_list = function()
	{
		if (!this.m_temporal_trunks_priority_list) {
			if (this.m_popup) {
				this.m_popup.finalize();
			}
			$(`#${this.schedule_prefix}_range_popup`).show();
			return;
		}

		let key = "";
		let trunks_priority = {};
		let list_name;

		for (let trunk_id in this.m_temporal_trunks_priority_list) {
			if (this.m_temporal_trunks_priority_list.hasOwnProperty(trunk_id)) {
				let info = this.m_temporal_trunks_priority_list[trunk_id];
				list_name = info.ListName;
				if (key.length > 0) {
					key = key + "_";
				}
				key = key + FORMAT("{0}_{1}", trunk_id, info.Priority);
				trunks_priority[trunk_id] = info.Priority;
			}
		}

		let new_item_list = {
			Id    : key,
			Name  : list_name,
			Trunks: trunks_priority,
			Routes: {}
		};

		this.m_trunks_priority_list[key] = new_item_list;
		const $combo = $("[trunks_priority_list_ul]");
		let $item = this.__get_trunks_priority_list_item_markup(new_item_list);
		$combo.append($item);
		$item.click();

		this.m_temporal_trunks_priority_list = null;
		this.m_is_updated_trunks_priority_list = true;
		if (this.m_popup) {
			this.m_popup.finalize();
		}
		$(`#${this.schedule_prefix}_range_popup`).show();
	};

	ProvisioningRouteSchedulersMng.prototype.popup_fill_trunks_list = function()
	{
		const $combo = $("#" + this.prefix + "popup_trunk");
		const list = this.owner.get_trunks() || {};
		var is_blank_first_item = function() {
			return {
				id  : "",
				name: "<%=_T('Select')%>..."
			};
		};
		this.owner.fill_combo($combo, is_blank_first_item, list, "id", "id");
	};

	ProvisioningRouteSchedulersMng.prototype.show_popup_new_trunks_priority_list = function()
	{
		const self = this;
		const popup_id = this.prefix + 'trunks_priority_new_popup';
		const $item_empty = $("[trunks_priority_list_ul]").children(':first');
		$item_empty.click();

		$('#' + popup_id).remove();
		let popup_html = this.make_popup_html_new_trunks_priority_list(popup_id);
		let $popup = $(popup_html);

		//tooltip
		$popup.find("[tooltip]").inconcert_tooltip({
			className: 'bigTooltip',
			alignment: 'right-bottom',
			["remove-delay"]: 100000,
		});

		$('body').append($popup);
		this.set_events_popup_new_trunks_priority_list();
		this.popup_fill_trunks_list();
		this.owner.create_spinners(popup_id);

		this.m_temporal_trunks_priority_list = null;
		this.m_is_updated_trunks_priority_list = false;

		this.m_popup = new inConcert.PopupWindow({ownerId: popup_id});
		this.m_popup.show(
		{
			ownerId      : popup_id,
			eventName    : popup_id,
			container    : $popup,
			callbacks: {
				onOpen: function() {
					const $popup = $('#' + popup_id);
					$popup.show();
					const $trunk = $("#" + self.prefix + "popup_trunk");
					$trunk.focus();
					$(`#${self.schedule_prefix}_range_popup`).hide();
					//$popup.css({top:"53%"});
				},
				onClose: function( onClose ) {
					if (!self.m_is_updated_trunks_priority_list) {
						const $popup_trunk_list_pre_view = $("#" + self.prefix + "popup_trunk_list_pre_view");
						$popup_trunk_list_pre_view.html("");
					}
					self.m_is_updated_trunks_priority_list = false;

					onClose = onClose || $.noop;
					if (self.m_popup) {
						self.m_popup.hide(function() {
							onClose();
						});
					}
					else {
						onClose();
					}
				}
			}
		});
	};

	ProvisioningRouteSchedulersMng.prototype.show_trunks_priority = function(priority_info)
	{
		const $priority_info = $("#" + this.prefix + "trunks_priority_info");
		$priority_info.html("").hide();

		if (!priority_info || priority_info.Id === "") {
			return;
		}
		const trunks = priority_info.Trunks;
		if (!trunks) {
			return;
		}
		let trunks_priority = "";
		for (let trunk_id in trunks) {
			if (trunks.hasOwnProperty(trunk_id)) {
				let priority = trunks[trunk_id];
				trunks_priority = trunks_priority + FORMAT("<div class='trunksPriorityPreViewChip' trunk='{0}'><label>{0}</label> <strong>{1}</strong></div>", trunk_id, priority);
			}
		}
		$priority_info.html(trunks_priority).show();
	};

	ProvisioningRouteSchedulersMng.prototype.__init_scheduler = function()
	{
		const self = this;

		this.m_schedule_hnd = new inConcert.conf_form_schedule({
			frm                 : this.owner,
			owner               : this,
			prefix              : this.schedule_prefix,
			container_id        : this.schedule_container,
			err_msg_container_id: this.prefix + "popup_message",
			buttons_container_id: this.schedule_prefix + "_popup_buttons",
			waiting_container_id: this.schedule_prefix + "_popup_waiting",
			ctrls_customs: {
				skipButIntervalAddDay: true,
				$top_add_range: (function() {
					return '<div class="row">' +
								'<div class="col-12">' +
									'<div class="form-group formVisualCombo">' +
										'<label for="' + self.prefix + 'trunks_priority_list_selector" class="required"><%=_T("Saved trunks list")%><%=_T([[:]])%></label>' +
										'<div class="addTrunkListContainer">' +
											'<div id="' + self.prefix + 'trunks_priority_list_selector" class="form-control">' +
												'<label list_selector_title class="formVisualComboValue pl-0"></label>' +
											'</div>' +
											'<button class="buttonToggle buttonToggleOutlined addPriorityButton" id="' + self.prefix + 'trunks_priority_new" tooltip="<%=_T([[Create property list of trunks.]])%>"><i class="far fa-plus"></i></button>' +
										'</div>' +
										'<div option_selected="" id="' + self.prefix + 'trunks_priority_list" class="formVisualComboList" style="display:none;">' +
											'<ul trunks_priority_list_ul="true">'+
											'</ul>'+
										'</div>' +
										'<i id="' + self.prefix + 'trunks_priority_list_validation" class="formFieldValidation formFieldError" hidden></i>' +
									'</div>' +
								'</div>'+
							'</div>'+
							'<div class="row">' +
								'<div class="col-12">' +
									'<div class="form-group mb-0">' +
										'<label for="' + self.prefix + 'trunks_priority_info" class=""><%=_T("Pre-visualize")%><%=_T([[:]])%>' +
										'</label>' +
										'<div id="' + self.prefix + 'trunks_priority_info" class="trunksPriorityPreViewList"></div>' +
									'</div>' +
								'</div>' +
							'</div>' +
							'<hr/>'
						})()
			},
			callbacks: {
				cancel: function() {
					if (self.m_popup) {
						self.m_popup.finalize();
						self.m_popup = null;
					}
					this.m_schedule_hnd.hide_range_popup();
				},
				can_configure_special_days: function() {
					return false;
				},
				validate: function(scheduler, callback) {
					self.__validate_scheduler(scheduler, callback);
				},
				can_add_scheduled_range: function(info, callback) {
					self.__can_add_scheduled_range(info, callback);
				},
				on_added_scheduled_range: function(callback) {
					self.__on_added_scheduled_range(callback);
				},
				on_open_scheduled_range_popup: function () {
					if (self.m_schedule_hnd.m_newRangePopup) {
						self.m_schedule_hnd.m_newRangePopup.canReposition = false;
						const $RangePopup = $("#" + self.m_schedule_hnd.m_newRangePopup.ownerId);
						//$RangePopup.css({right: "42%", top:"33%"});
						$RangePopup.show();
					}
					self.__cleanup_temporal_trunk_list();
				}
			}
		});

		this.m_schedule_hnd.initialize();
		$("body").removeClass("popupDisplay").addClass("popupDisplay");
	};

	ProvisioningRouteSchedulersMng.prototype.__fill_combo_areas = function(country_code)
	{
		const telephone_codes = this.owner.get_telephone_codes();
		let areas = telephone_codes.getAreasByCode(country_code);
		const $area = $("#" + this.prefix + "popup_area");

		const attr_name = "name";
		const attr_id = "code";
		/*
			En la lista de "areas" hay que manejar diferente al primer elemento "blank" del combo.
			Esto porque en TODOS los combos que utilizamos el elemento "blank" tiene como "value" del <option> el valor "" (vacio).
			Esto funciona bien en todos lados, menos con los "code areas", ya que aca existe un elemento con Id "" que es el "Fixed Lines".
			Esto hace que el combo tenga 2 items con el mismo Id, y aqui el error.
			Para solucionar esto, al elemento "blank" de este combo le ingresamos una palabra clave "__BLANK" que se asignara en el atributo "value" del <option>
		*/
		var is_blank_first_item = function() {
			return {
				id  : "ANY",
				name: "<%=_T('All')%>"
			};
		};

		if (!areas) {
			$area.html("");
			this.owner.fill_combo($area, is_blank_first_item, [], attr_id, attr_name);
			this.owner.readonly_control($area, true);
			return;
		}

		this.owner.readonly_control($area, false, true);
		this.owner.fill_combo($area, is_blank_first_item, areas, attr_id, attr_name);
	};

	ProvisioningRouteSchedulersMng.prototype.fill_countries = function()
	{
		const self = this;

		const $combo = $("#" + this.prefix + "popup_country");
		$combo.html("");
		this.owner.combo_add_value($combo, "ANY", "<%=_T('Alls')%>", true);
		let sorted_countries = this.owner.get_countries();

		$.each(sorted_countries, function(_, country) {
			self.owner.combo_add_value($combo, country.code, country.name, false);
		});
	};

	ProvisioningRouteSchedulersMng.prototype.fill_trunks_priority_list = function(list)
	{
		this.m_trunks_priority_list = list;
		const $combo = $("[trunks_priority_list_ul]");
		
		const default_item = {
			Id  : "",
			Name: "<%=_T('Select')%>...",
			Trunks: {},
			Routes: {}
		};

		let $default_item = this.__get_trunks_priority_list_item_markup(default_item);
		$combo.append($default_item);
		$default_item.click();

		for (let item_id in list) {
			if (list.hasOwnProperty(item_id)) {
				let item = list[item_id];
				let $item = this.__get_trunks_priority_list_item_markup(item);
				$combo.append($item);
			}
		}

		$("#" + this.prefix + "trunks_priority_info").hide();
	};

	ProvisioningRouteSchedulersMng.prototype.get_scheduler_additional_range_info = function(additionalData)
	{
		if (!additionalData) {
			return;
		}
		if (!this.m_trunks_priority_list) {
			return;
		}

		let trunks = additionalData;
		let key = "";
		let tooltip = "";

		for (let trunk_id in trunks) {
			if (trunks.hasOwnProperty(trunk_id)) {
				let priority = trunks[trunk_id];
				if (key.length > 0) {
					key = key + "_";
					tooltip = tooltip + ", ";
				}
				key = key + FORMAT("{0}_{1}", trunk_id, priority);
				tooltip = tooltip + FORMAT("{0}: {1}", trunk_id, priority);
			}
		}

		let priority = this.m_trunks_priority_list[key];
		if (!priority) {
			return;
		}

		return {
			text   : priority.Name || "",
			tooltip: tooltip
		};
	};

	/*
		Segun el formato que retorna la clase del schedulers, lo convierte al mismo formato que espera el controller y la rocka "provisioningmanager", para realizar los inserts sobre los schedulers.
		Lo que convierte es el objeto schedulers "ClientScheduler".

		La clase conf form schedulers retorna:
[
  {
    "UUID": "ANY_ANY",
    "Country": "ANY",
    "Area": "ANY",
    "DisplayCountry": "Todos los paises",
    "DisplayArea": "-",
    "DisplayCountryCode": "-",
    "DisplayAreaCode": "-",
    "ClientScheduler": {
      "scheduleDays": [
        {
          "startTime": "1970-01-01 00:00:00",
          "endTime": "1970-01-01 00:00:00",
          "startHour": "00:00",
          "endHour": "00:00",
          "dayOfWeek": 1,
          "enabled": 1,
          "additionalData": {
            "a": "1",
            "asdf": "2"
          }
        },
		..
		..
		..
        {
          "startTime": "1970-01-01 00:00:00",
          "endTime": "1970-01-01 00:00:00",
          "startHour": "00:00",
          "endHour": "00:00",
          "dayOfWeek": 7,
          "enabled": 1,
          "additionalData": {
            "a": "1500"
          }
        }
      ],
      "scheduleSpecialDays": [],
      "scheduleSpecialDayAllYearss": []
    }
  }
]
		Lo convierte a:

{
  "ANY_ANY_00:00_00:00_{\"a\":\"1\",\"asdf\":\"2\"}": {
    "dow": [
      1
    ],
    "trunks": {
      "a": "1",
      "asdf": "2"
    },
    "from": "00:00",
    "to": "00:00",
    "country": "ANY",
    "area": "ANY"
  },
	..
	..
	..
  "ANY_ANY_00:00_00:00_{\"a\":\"1500\"}": {
    "dow": [
      7
    ],
    "trunks": {
      "a": "1500"
    },
    "from": "00:00",
    "to": "00:00",
    "country": "ANY",
    "area": "ANY"
  }
}
	*/
	ProvisioningRouteSchedulersMng.prototype.get = function()
	{
		var get_structure_to_save = function() {
			return {
				dow    : [],
				trunks : {},
				from   : "",
				to     : "",
				country: "",
				area   : ""
			};
		};

		const template_key = "{0}_{1}_{2}_{3}_{4}";
		let all_schedulers = this.m_grid_schedulers.get_items();
		let schedulers_to_save = {};

		for (let i = 0; i < (all_schedulers || []).length; i++) {
			let current_sch = all_schedulers[i];

			if (current_sch.ClientScheduler) {
				for (let j = 0; j < (current_sch.ClientScheduler.scheduleDays || []).length; j++) {
					let scheduleDay = current_sch.ClientScheduler.scheduleDays[j];
					let trunks = scheduleDay.additionalData;

					let key = FORMAT(template_key,
						current_sch.Country || "ANY",
						current_sch.Area    || "ANY",
						scheduleDay.startHour,
						scheduleDay.endHour,
						JSON.stringify(trunks)
					);

					let temp_sch = schedulers_to_save[key];
					if (!temp_sch) {
						temp_sch = get_structure_to_save();
						temp_sch.dow = [];
						temp_sch.trunks = trunks;
						temp_sch.from = scheduleDay.startHour;
						temp_sch.to = scheduleDay.endHour;
						temp_sch.country = current_sch.Country;
						temp_sch.area = current_sch.Area;
					}
					temp_sch.dow.push(scheduleDay.dayOfWeek);
					schedulers_to_save[key] = temp_sch;
				}
			}
		}

		return schedulers_to_save;
	};

}());
