module(..., package.seeall)

--require "redis"
--local redis = require "wrapredis"
--local cassandra = require "lua_cassandra"
local dbutils = require "shared.apps.utils"
local helpers = require "apps.reports.helpers"
local _       = require "apps.reports.functional"
local m_helpers = require "helpers"
local m_cassandra

function getCassandraConnection(ks)
	local ok, ks = helpers.safeInvoke(m_cassandra.CassandraConnection, function(ks)
		return true, ks
	end, cassandra, ks or "SummarizerData")
	return ks
end

function IsMemberOfCampaign (vcc, userId, campaignId, keyspace)
	local fullUserId = dbutils.AppendVcc(dbutils.GetId(userId), vcc)
	local fullCampaignId = dbutils.AppendVcc(dbutils.GetId(campaignId), vcc)

	local res, column = pcall(keyspace.GetColumn, keyspace, fullCampaignId, "CampaignMembers", fullUserId)
	if res then
		return column and column.value
	end
	return false
end

function GetCampaignMembers (vcc, campaignId, keyspace)
	local fullCampaignId = dbutils.AppendVcc(dbutils.GetId(campaignId), vcc)
	local campaignMembers = dbutils.GetAllColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "CampaignMembers",
		key = fullCampaignId
	} or {}
	--LogDebug("[REPORTS.MODEL.GetCampaignMembers] Campaign '%s' Users: '%s'", campaignId, json.encode(campaignMembers))
	return campaignMembers
end

function GetUserCampaigns (fullUserId, keyspace)
	local userCampaigns = dbutils.GetAllColumns {
		keyspace     = keyspace or "Agent",
		columnFamily = "UserCampaigns",
		key          = fullUserId
	} or {}
	--LogDebug("[REPORTS.MODEL.GetUserCampaigns] User '%s' Campaigns: '%s'", fullUserId, json.encode(userCampaigns))
	return userCampaigns
end

function listVccCampaigns(vcc, keyspace)
	local campaignColumns = dbutils.GetAllColumns({
		keyspace     = keyspace or "Agent",
		columnFamily = "VCCCampaigns",
		key          = vcc
	}) or {}
	return _.map(campaignColumns, function(c, campaignName) -- c is not used
		return campaignName .."@".. vcc
	end)
end

function listCampaignAccounts(campaignId, keyspace)
	return dbutils.GetAllColumns({
		keyspace     = keyspace or "Agent",
		superColumns = true,
		columnFamily = "CampaignAccounts",
		key          = campaignId
	}) or {}
end

function GetAllVCCsAccounts ( keyspace )
	local params = {
		keyspace = keyspace or "Agent",
		superColumns = true,
		columnFamily = "CampaignAccounts",
		startKey = "",
		endKey = ""
	}
	return dbutils.GetAllColumns( params ) or {}
end

function GetVccAccounts (vcc, keyspace)
	assert(vcc, "vcc is required")
	vcc = vcc:lower()
	local all_accounts = GetAllVCCsAccounts( keyspace )
	local vcc_accounts = {}

	for campaign_id, campaign_accounts in pairs( all_accounts ) do
		local _, campaign_vcc = dbutils.SeparateVCC(campaign_id)
		if campaign_vcc and (campaign_vcc:lower() == vcc) then
			vcc_accounts[campaign_id] = campaign_accounts
		end
	end

	return vcc_accounts
end

function GetAttentionLevels (campaignId, keyspace)
	return dbutils.GetAllColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "CampaignAttentionLevels",
		key = campaignId
	} or {}
end

function listCampaignDialerBatches (campaignId, keyspace)
	return dbutils.GetAllColumns({
		keyspace     = keyspace or "Agent",
		superColumns = true,
		columnFamily = "CampaignDialerBatches",
		key          = campaignId
	}) or {}
end

function getCustomMetricsLimit(vcc, keyspace)
	if m_helpers.utils.HasLicenseSystem() then
		local options = dbutils.Get_m_helpers_agentservice_options()	
		local vccInfo = m_helpers.agentservice.GetVccLicense(options, vcc)
		return vccInfo.CustomMetricsLimit or 0	
	end
	local vcc = dbutils.GetAllColumns {
		keyspace     = keyspace or "Agent",
		columnFamily = "VCCs",
		key          = vcc
	} or {}
	return tonumber(vcc.CustomMetricsLimit) or 0	
end

function getCustomFiltersLimit(vcc, keyspace)
	if m_helpers.utils.HasLicenseSystem() then
		local options = dbutils.Get_m_helpers_agentservice_options()			
		local vccInfo = m_helpers.agentservice.GetVccLicense(options, vcc)
		return vccInfo.CustomFiltersLimit or 0
	end
	local vcc = dbutils.GetAllColumns {
		keyspace     = keyspace or "Agent",
		columnFamily = "VCCs",
		key          = vcc
	} or {}
	return tonumber(vcc.CustomFiltersLimit) or 0	
end

function getReportSchedule (scheduleId, keyspace)
	local schedule = dbutils.GetAllColumns {
		keyspace     = keyspace or "SummarizerData",
		columnFamily = "ReportSchedules",
		key          = scheduleId
	} or {}
	--LogDebug("[REPORTS.MODEL.getReportSchedule] schedule: '%s'", json.encode(schedule))
	return schedule
end

function getDbUtils()
	return dbutils
end

function init(config)
	m_cassandra = config.cassandra
	dbutils.SetupConnections(config)
end

function GetCampaign (Id, keyspace, columns)
	return dbutils.GetAllColumns {
		keyspace		= keyspace or "Agent",
		columnFamily	= "Campaigns",
		key				= Id,
		columns			= columns
	} or {}
end