/*jshint bitwise:true, browser:true, curly:true, eqeqeq:true, noarg:true, noempty:true, strict:true, undef:true, unused:true, esversion: 9 */
/*jshint newcap:false, quotmark:false */
/*global $, _, inConcert*/

(function () {
	/*jsh--int validthis:true*/
	"use strict";

	var format = inConcert.Utils.format;
	var helpers = inConcert.reports.helpers;

	function FormEditDimension(dimension, controller, $container, context)
	{
		FormEditDimension.uber.constructor.call(this, "/inconcert/apps/reports/pages/forms/edit_dimension.<%=_L%>.html#grunt-cache-bust", $container, context);
		var self = $.extend(this, {
			dimension : dimension && _.extend({}, dimension, {
				name: helpers.getCustomDimensionName(dimension.name)
			}),
			controller: controller,
			permissions: {
				creation: "save_custom_fields"
			},
			extras    : undefined // extra properties for the dimension
		});
		if (self.dimension && self.dimension.displayName === self.dimension.name) {
			delete self.dimension.displayName;
		}
	}

	inConcert.Utils.inherit(FormEditDimension, inConcert.reports.FormBase);

	FormEditDimension.prototype.onRender = function(callback) {
		var self = this;

		function intGroupControls(callback) {
			var $container = self.container.find('[ic-role="manage-field-groups"]');
			var $buttonNewGroup = self.container.find('[ic-role="new-group"]');
			var $divSelectGroup = $container.children('[ic-role="select-group"]');
			var $divCreateGroup = $container.children('[ic-role="create-group"]');
			var $inputText = $divCreateGroup.find('[ic-role="input-group-name"]');

			function disableControls(options) {
				_.each(self.container.find('[ic-data-bind]'), _.compose(
					_.bind(self.onFieldDisabledChange, self, options.disabled), helpers.future('disabled', options.disabled), _.partial(_.bind, $.fn.prop), $
				));
				self.context.saveButton.prop('disabled', options.disabled);
			}

			$buttonNewGroup.on('click', function() {
				$divSelectGroup.hide();
				$divCreateGroup.show();
				$buttonNewGroup.prop("disabled", true);	
				$buttonNewGroup.toggleClass("active");
				disableControls({ disabled: true });
				$inputText.val('').focus();
			});

			$container.on('click', 'button[ic-role]', function() {
				switch( $(this).attr('ic-role') ) {
					case 'new-group':
						$divSelectGroup.hide();
						$divCreateGroup.show();				
						disableControls({ disabled: true });
						$inputText.val('').focus();
						break;
					case 'save-group':
						var inputText = $inputText.val();
						if (inputText) {
							$divSelectGroup.find('[ic-data-bind="projectId"]')
								.append($('<option></option>').val(inputText).text(inputText))
								.val(inputText)
								.children('option[value=""]').remove();
							self.emit("status", "<%=_T('Group will be stored when saving the Custom Field')%>");
						}
						/*falls through*/
					case 'cancel-group':
						$divSelectGroup.show();
						$divCreateGroup.hide();
						disableControls({ disabled: false });						
						$buttonNewGroup.toggleClass("active");
						$buttonNewGroup.prop("disabled", false);
						break;
				}
			});

			helpers.loadCustomFieldGroups(self.controller, $divSelectGroup.find('[ic-data-bind="projectId"]'))
				.done(_.partial(callback, true))
				.fail(_.partial(callback, false));
		}

		intGroupControls(function(success, errorThrown) {
		  if (!success) {
		    callback(success, errorThrown);
		    return;
		  }
		  
		  self.controller.loadDefaultValues()
			.done(function (defaultValues) {
			  if (self.dimension === null) {
				  self.setTitle("<%=_T('New Custom Field')%>", "", "");
			  } else {
				  var title = self.hasPermissionToSave() ?
					  "<%=_T('Edit Custom Field')%> <span>\"{0}\"</span>" : 
					  "<%=_T('View Custom Field')%> <span>\"{0}\"</span>";
				  self.setTitle(format(title, self.dimension.name));
				  helpers.displayDataModel(self.container, self.dimension);
			  }
			  
			  // Keep `maxLength' synchronized with `type' when `defaultMaxLength' is 
			  // checked.
			
			  var $type = self.container.find('[ic-data-bind="type"]');
			  var $maxLength = self.container.find('[ic-data-bind="maxLength"]');
			  var $defaultMaxLength = self.container.find('[ic-data-bind="defaultMaxLength"]');
			  var DEFAULT_FIELD_MAX_LENGTH = defaultValues.DEFAULT_FIELD_MAX_LENGTH || {};
			
			  var updateMaxLength = function () {
			    var typeValue = $(this).val();
				  var maxLengthValue = typeValue ? 
					  DEFAULT_FIELD_MAX_LENGTH[typeValue.toUpperCase()] || 
					  DEFAULT_FIELD_MAX_LENGTH.STRING : 
					  0;
				
				  $maxLength.val(maxLengthValue);
			  };
			
			  $defaultMaxLength
			  .off("change")
			  .on("change", function () {
				  if ($(this).prop("checked")) {
					  $maxLength.prop("disabled", false);
					
					  $type
					  .off("change.updateMaxLength")
					  .on("change.updateMaxLength", updateMaxLength);
					
					  $type.triggerHandler("change.updateMaxLength");
				  } else {
					  $type.off("change.updateMaxLength");
					  $maxLength.prop("disabled", true);
					  $maxLength.focus();
				  }
			  });
			  
			  if (self.dimension === null) {
				  // Initially `maxLength' is synchronized with `type'.
				  $defaultMaxLength.prop("checked", true);
				  $defaultMaxLength.triggerHandler("change");
			  } else {
				  // Keep `maxLength' synchronized with `type' if `defaultMaxLength' is 
			    // checked.
				  if ($defaultMaxLength.prop("checked")) {
				  	$maxLength.prop("disabled", true);
				  	
				    $type
				    .off("change.updateMaxLength")
				    .on("change.updateMaxLength", updateMaxLength);
				  }
			  }

			  // displayName mirrors the name
			  (function() {
				  var alphanum = _.union(_.range(48, 57+1), _.range(65, 90+1), [ 95 ], _.range(97, 122+1));
				  var $displayName = self.container.find('[ic-data-bind="displayName"]');

				  var onKeyUp = function() {
					  $displayName.attr('placeholder', $(this).val());
				  };

				  self.container.find('[ic-data-bind="name"]')
					  .on('keyup', _.debounce(onKeyUp, 100))
					  .on('keypress', function(e) {
						  if (_.indexOf(alphanum, e.keyCode, true) < 0) {
							  return false;
						  }
						  if (e.keyCode >= 48 && e.keyCode <= 57) {
							  return $(this).val() !== "";
						  }
					  })
					  .trigger('keyup');
			  })();

			  callback(true);
			})
			.fail(function (error) {
			  callback(false, error);
			});
		});
	};

	FormEditDimension.prototype.onShow = function() {
		// mmmhh, nothing here...
	};

	FormEditDimension.prototype.onConfirm = function(callback) {
		var self = this;

		var result = self.validate(_.bind(self.isFieldEnabled, self));
		if (!result.success) {
			return callback(false, result.errorMessage);
		}

		if ( !result.fields.name.match(/^[_a-z][_\w]*$/i) ) {
			return callback(false, "Name should be a valid identifier, it cannot have spaces, only have letters and numbers");
		}

    if ( !result.fields.maxLength.match(/^0*[1-9]\d*$/) ) {
			return callback(false, "<%=_T('Max length must be an integer greater than or equal to 1')%>");
		}

		var dimension = $.extend({
			id: self.dimension && self.dimension.id || undefined
		}, result.fields, self.extras);

		if (!dimension.displayName) {
			dimension.displayName = dimension.name;
		}
		
		dimension.maxLength = Number(dimension.maxLength);

		self.controller.saveDimension(dimension).done(function() {
			callback(true, "<%=_T('Custom Fields saved successfully')%>");
		}).fail(function(errorThrown) {
			var message_error;
			switch ( errorThrown ) {
				case "forbidden":
					message_error = "<%=_T('You dont have permission to save')%>";
				break;
				default:
					message_error = errorThrown;
			}
			callback(false, format("<%=_T('Failed to save Custom Fields: {0}')%>", message_error));
		});
	};


	inConcert.reports.FormEditDimension = FormEditDimension;
}());
