/*jshint bitwise:true, browser:true, curly:true, eqeqeq:true, noarg:true, noempty:true, strict:true, undef:true, unused:true, esversion: 9 */
/*jshint newcap:false, quotmark:false */
/*global $, _, inConcert*/

(function () {
	/*jsh--int validthis:true*/
	"use strict";

	var _T = inConcert.i18n.getString;
	var format = inConcert.Utils.format;
	var helpers = inConcert.reports.helpers;

	function FormEditFilter(filter, controller, $container, context)
	{
		FormEditFilter.uber.constructor.call(this, "/inconcert/apps/reports/pages/forms/edit_filter.<%=_L%>.html#grunt-cache-bust", $container, context);
		$.extend(this, {
			filter    : filter,
			controller: controller,
			permissions: {
				creation: "save_custom_filters"
			},
			extras    : undefined // extra properties for the filter
		});
	}

	inConcert.Utils.inherit(FormEditFilter, inConcert.reports.FormBase);

	FormEditFilter.prototype.onRender = function(callback) {
		var self = this,
			filterType,
			documentType,
			$selectCci				= self.container.find('[ic-data-bind="cciProject"]'),
			$selectWebchat			= self.container.find('[ic-data-bind="webchatProject"]'),
			$selectWebcall			= self.container.find('[ic-data-bind="webcallProject"]'),
			$selectContactForm		= self.container.find('[ic-data-bind="contactformProject"]'),
			$selectEmailSurvey		= self.container.find('[ic-data-bind="surveyProject"]'),
			$selectApi				= self.container.find('[ic-data-bind="apiProject"]'),
			$selectBatchData		= self.container.find('[ic-data-bind="batch_dataProject"]'),
			$selectProcessFlow		= self.container.find('[ic-data-bind="process_flowProject"]'),
			$selectIntegration		= self.container.find('[ic-data-bind="integrationType"]');

		function makeDimensionFilter(options) {
			options = options || {};
			var $excludeWhenIn = options.excludeWhenIn;
			var $hideWhenIn    = options.hideWhenIn;

			var hideElement = function($element) {
				$element.hide();
			};

			var checkAndReturn = function(d) {
				if ($excludeWhenIn && helpers.hasChildren($excludeWhenIn, '[rd-filter-dimension="' + d.id + '"]')) {
					return false;
				}

				if ($hideWhenIn && helpers.hasChildren($hideWhenIn, '[rd-filter-dimension="' + d.id + '"]')) {
					return hideElement;
				}

				return true;
			};

			var checkTweety = documentType.match(/^Interaction(Agent)?$/) && function() {
				var integrationScope = $selectIntegration.val();
				var projects = {
					cci				: $selectCci.val(),
					webchat			: $selectWebchat.val(),
					webcall			: $selectWebcall.val(),
					contactform		: $selectContactForm.val(),
					survey			: $selectEmailSurvey.val(),
					api				: $selectApi.val(),
					batch_data		: $selectBatchData.val(),
					process_flow	: $selectProcessFlow.val()
				};
				return function(d) {
					if (helpers.isScopeCustom(d.scope)) {
						return (d.scope === "cci" || d.scope === "batch_data" || d.scope === "process_flow" || 
								d.scope === integrationScope) && projects[d.scope] === d.projectId;
					}
					return true;
				};
			}() ||  function() {
				return true;
			};

			return (function(d) {
				if (!d) {
					return false;
				}
				if (d.isProjectable === false && filterType === "detail") {
					return false;
				}
				if (d.isGrouper === false && filterType === "summary") {
					return false;
				}
				if (_.contains(["number", "Time", "datetime", "Date"], d.type)) {
					return false;
				}
				if (!helpers.isDimensionCompatible(d, documentType)) {
					return false;
				}
				let belongsToReportType = (d.belongsToReportType || "ARCHIVE").toUpperCase();
				if (belongsToReportType === "LIVE") {
					return false;
				}

				var res = checkTweety(d) && checkAndReturn(d);

				// show consistent displayName for custom fields
				if ( res && helpers.isScopeCustomField(d.scope) ) {
					return _.extend({}, d, {
						displayName: d.name // api.<name>
					});
				}

				return res;
			});
		}

		function displayErrorMessage(fmt, errorThrown) {
			self.emit("error", format(fmt, errorThrown));
		}

		function updateDescription() {
			// TODO
		}

		// hides all elements from the lists passed as arguments that
		// match the item being dropped in the list
		function makeReceive() {
			var $otherLists = _.toArray(arguments);
			return (function(dimension) {
				$otherLists.forEach(function($list) {
					$list.children('[rd-filter-dimension="' + dimension + '"]').hide();
				});
			});
		}

		function makeRemove() {
			var $otherLists = _.toArray(arguments);
			return (function(dimension) {
				$otherLists.forEach(function($list) {
					$list.children('[rd-filter-dimension="' + dimension + '"]').show();
				});
			});
		}

		var has_permission_to_save = self.hasPermissionToSave();

		function connectLists($listItemsAvailable, $listItemsInUse, options) {
			options = $.extend({
				__onReceive: function(/*dimension*/) { },
				__onRemove : function(/*dimension*/) { },
				__canRemove: function(/*dimension*/) { return true; }
			}, options || {});

			if ( !has_permission_to_save ) {
				return;
			}

			$listItemsAvailable.sortable({
				cursor: "move",
				connectWith: '#' + $listItemsInUse.uniqueId().attr('id')
			});
			$listItemsInUse.sortable(_.extend({
				cursor: "move",
				connectWith: '#' + $listItemsAvailable.uniqueId().attr('id'),
				receive: function(event, ui) {
					options.__onReceive(ui.item.attr('rd-filter-dimension'));
					updateDescription();
				},
				remove: function(event, ui) {
					var dimension = ui.item.attr('rd-filter-dimension');
					if (options.__canRemove(dimension)) {
						options.__onRemove(dimension);
						updateDescription();
					} else {
						$(this).sortable('cancel');
						displayErrorMessage(format("<%=_T('You cannot remove <b>{0}</b>, just rearrange it.')%>", dimension));
					}
				}
			}, options || {}));
		}

		function getDimensionById(id) {
			return self.controller.loadDimension(id, {sync: true});
		}

		function makeAppender($target) {
			return function(d) {
				return helpers.appendToList($target, d);
			};
		}

		function getNonCompatibleDimensions($elements) {
			return _.reject($elements, _.compose(
				makeDimensionFilter(), getDimensionById, helpers.future('rd-filter-dimension'), _.partial(_.bind, $.fn.attr), $
			));
		}

		function hasDimensions($list, customSelector) {
			var $children = $list.children('[rd-filter-dimension]:not([ic-role])');
			if (customSelector) {
				$children = $children.filter(customSelector);
			}
			return $children.length > 0 && getNonCompatibleDimensions($children).length > 0;
		}

		function removeElements($list, selector) {
			_.each(getNonCompatibleDimensions($list.children(selector)), function(e) {
				$(e).remove();
			});
		}

		var filterDefinitions = {
			summary: {
				def: {
					$filtersAvailable: $('#rd_summaryFilter_filters_available'),
					$columnsAvailable: $('#rd_summaryFilter_columns_available'),
					$filtersInUse    : self.container.find('[ic-data-bind="filter"]'),
					$columnsInUse    : self.container.find('[ic-data-bind="columns"]'),
					forcedColumns    : [ "Slice" ]
				},
				init: _.once(function() {
					var def = this.def;
					connectLists(def.$filtersAvailable, def.$filtersInUse);
					connectLists(def.$columnsAvailable, def.$columnsInUse, {
						__canRemove: function(dimension) {
							return !_.contains(def.forcedColumns, dimension);
						}
					});
					if (!self.filter) {
						_.each(def.forcedColumns, _.compose(
							makeAppender(def.$columnsInUse), getDimensionById
						));
					}
					return this;
				}),
				displayDimensions: function() {
					var def = this.def;
					self.controller.listDimensions("all").done(function(dimensions) {
						helpers.populateList(
							def.$filtersAvailable,
							dimensions,
							makeDimensionFilter({ excludeWhenIn: def.$filtersInUse }),
							{ sort: true }
						);
						helpers.populateList(
							def.$columnsAvailable,
							dimensions,
							makeDimensionFilter({ excludeWhenIn: def.$columnsInUse }),
							{ sort: true }
						);

						self.extras = undefined;

					}).fail(_.partial(displayErrorMessage, "<%=_T('Failed to list available fields: {0}')%>"));
				},
				hasDimensionsInUse: function() {
					return (
						hasDimensions(this.def.$filtersInUse) ||
						hasDimensions(this.def.$columnsInUse, '[rd-filter-dimension!="Slice"]')
					);
				},
				clearDimensionsInUse: function() {
					removeElements(this.def.$filtersInUse, ':not([ic-role])');
					removeElements(this.def.$columnsInUse, '[rd-filter-dimension!="Slice"]:not([ic-role])');
				}
			},
			detail: {
				def: {
					$mandatoryFieldsAvailable: $('#rd_detailFilter_mandatoryFields_available'),
					$optionalFieldsAvailable : $('#rd_detailFilter_optionalFields_available'),
					$sortFieldsAvailable     : $('#rd_detailFilter_sortFields_available'),
					$mandatoryFieldsInUse    : self.container.find('[ic-data-bind="mandatoryFields"]'),
					$optionalFieldsInUse     : self.container.find('[ic-data-bind="optionalFields"]'),
					$sortFieldsInUse         : self.container.find('[ic-data-bind="sortFields"]'),
					dateFields: [
						{ fieldName: "EndDate", sortField: "StartDate", scopes: ["Interaction", "InteractionAgent", "InteractionAttentionLevel", "InteractionRule", "InteractionVoiceBlaster"] },
						{ fieldName: "To",      sortField: "From",      scopes: ["Agent", "AgentStateLapse"] }
					],
					forcedFields: [
						{ fieldName: "Type", scopes: [ "Interaction", "InteractionAgent", "InteractionAttentionLevel" ] }
					]
				},
				init: _.once(function() {
					var def = this.def;
					connectLists(def.$mandatoryFieldsAvailable, def.$mandatoryFieldsInUse, {
						__onReceive: makeReceive(def.$optionalFieldsAvailable),
						__onRemove : makeRemove(def.$optionalFieldsAvailable),
						__canRemove: function(dimension) {
							return !_.find(def.forcedFields, function(field) {
								return field.fieldName === dimension && _.contains(field.scopes, documentType);
							});
						}
					});
					connectLists(def.$optionalFieldsAvailable, def.$optionalFieldsInUse, {
						__onReceive: makeReceive(def.$mandatoryFieldsAvailable),
						__onRemove : makeRemove(def.$mandatoryFieldsAvailable)
					});
					connectLists(def.$sortFieldsAvailable, def.$sortFieldsInUse, {
						__canRemove: function(dimension) {
							return !(def.currentDateField && dimension === def.currentDateField.sortField);
						}
					});
					return this;
				}),
				displayDimensions: function() {
					var def = this.def;
					self.controller.listDimensions("all").done(function(dimensions) {
						helpers.populateList(
							def.$mandatoryFieldsAvailable,
							dimensions,
							makeDimensionFilter({
								excludeWhenIn: def.$mandatoryFieldsInUse,
								hideWhenIn   : def.$optionalFieldsInUse
							}),
							{ sort: true }
						);
						helpers.populateList(
							def.$optionalFieldsAvailable,
							dimensions,
							makeDimensionFilter({
								excludeWhenIn: def.$optionalFieldsInUse,
								hideWhenIn   : def.$mandatoryFieldsInUse
							}),
							{ sort: true }
						);
						helpers.populateList(
							def.$sortFieldsAvailable,
							dimensions,
							makeDimensionFilter({ excludeWhenIn: def.$sortFieldsInUse }),
							{ sort: true }
						);

						if (!def.currentDateField || !_.contains(def.currentDateField.scopes, documentType)) {
							if ( def.currentDateField ) {
								def.$sortFieldsInUse.children('[rd-filter-dimension="' + def.currentDateField.sortField + '"]').remove();
							}

							def.currentDateField = _.find(def.dateFields, function(item) {
								let contains = _.contains(item.scopes, documentType);
								return contains;
							});

							if (def.currentDateField) {
								let jQuerySelector = '[rd-filter-dimension="' + def.currentDateField.sortField + '"]';
								if (def.$sortFieldsInUse.children(jQuerySelector).length === 0) {
									makeAppender(self.filter ? def.$sortFieldsAvailable : def.$sortFieldsInUse)(
										getDimensionById(def.currentDateField.sortField)
									);
								}
								self.extras = {
									dateField: def.currentDateField.fieldName
								};
							}
						}

						def.forcedFields.forEach(function(item) {
							var $child = def.$mandatoryFieldsInUse.children('[rd-filter-dimension="' + item.fieldName + '"]');
							if (!_.contains(item.scopes, documentType)) {
								$child.remove();
							} else if ($child.length === 0) {
								$child = def.$mandatoryFieldsAvailable.children('[rd-filter-dimension="' + item.fieldName + '"]');
								if ($child.length > 0) {
									$child.detach().appendTo(def.$mandatoryFieldsInUse);
								} else {
									$child = makeAppender(def.$mandatoryFieldsInUse)(
										getDimensionById(item.fieldName)
									);
								}
								makeReceive(def.$optionalFieldsAvailable)(item.fieldName);
							}
						});

					}).fail(_.partial(displayErrorMessage, "<%=_T('Failed to list available fields: {0}')%>"));
				},
				hasDimensionsInUse: function() {
					var def = this.def;
					var excludeSortFieldSelector = def.currentDateField && '[rd-filter-dimension!="' + def.currentDateField.sortField + '"]';
					var excludeForcedFieldsSelector = _.map(def.forcedFields, function(field) {
						return '[rd-filter-dimension!="' + field.fieldName + '"]';
					});
					return (
						hasDimensions(def.$mandatoryFieldsInUse, excludeForcedFieldsSelector.join(',')) ||
						hasDimensions(def.$optionalFieldsInUse) ||
						hasDimensions(def.$sortFieldsInUse, excludeSortFieldSelector)
					);
				},
				clearDimensionsInUse: function() { // clear dimensions in use
					var def = this.def;
					removeElements(def.$mandatoryFieldsInUse, ':not([ic-role])');
					removeElements(def.$optionalFieldsInUse, ':not([ic-role])');
					var filterOut = def.currentDateField ? '[rd-filter-dimension!="' + def.currentDateField.sortField + '"]' : '';
					removeElements(def.$sortFieldsInUse, filterOut + ':not([ic-role])');
				}
			}
		};

		function checkCanCleanupDimensions(callback) {
			if (!self.renderComplete) {
				return callback(true);
			}

			var needConfirmation = false;
			_.each(filterDefinitions, function(def, k) {
				if (def.init().hasDimensionsInUse()) {
					if (!needConfirmation && k === filterType) {
						needConfirmation = true;
					} else {
						def.clearDimensionsInUse();
					}
				}
			});

			if (needConfirmation === false) {
				callback(true); // no need to ask, go on
			} else {
				var question = format("{0}<br><br>&nbsp;&nbsp;&nbsp;&nbsp;{1}",
					"<%=_T('This action will discard the fields that are not compatible with this setting.')%>",
					"<%=_T('Continue?')%>"
				);
				helpers.askForConfirmation(question, "<%=_T('About to change filter attributes')%>")
					.done(function() {
						filterDefinitions[filterType].clearDimensionsInUse();
						callback(true);
					}).fail(_.partial(callback, false));
			}
		}

		function displayDimensions() {
			filterDefinitions[filterType]
				.init()
				.displayDimensions();
			updateDescription();
		}

		const $filterType = self.container.find('[ic-data-bind="type"]'); // combo "Tipo"
		const $scope = self.container.find('[ic-data-bind="scope"]');     // combo "Scope"

		filterType = $filterType.val();
		$filterType.off('change')
			.on("change", function() {
				const $this = $(this);

				checkCanCleanupDimensions(function(ok) {
					if (ok === false) {
						$this.val(filterType);
					} else {
						filterType = $this.val();
						self.switchUseCaseByAttr('rd-for-filter-type', filterType).done(function() {
							if (self.hNavigation) {
								self.context.saveButton.prop('disabled', !self.hNavigation.allVisited());
							}
						});
						displayDimensions();
					}

					// Se filtran las categorias (scope) segun el tipo de filtro "summary" o "detail"
					let currentScope = $scope.val().toLowerCase();
					let currentFilterType = $this.val().toLowerCase();
					let firstVisibleScope, isHiddenCurrentScope;
					const $scope_options = $scope.find("option");

					$.each($scope_options, function(_, option) {
						const $option = $(option);
						let currentOptionValue = $option.attr("value");
						const belognTo = ($option.attr("belong-to-type") || "all").toLowerCase();
						if (belognTo === "all" || belognTo === currentFilterType) {
							$option.show();
							if (!firstVisibleScope) {
								firstVisibleScope = currentOptionValue;
							}
						}
						else {
							$option.hide();
							if (currentOptionValue.toLowerCase() === currentScope) {
								isHiddenCurrentScope = true;
							}
						}
					});

					if (isHiddenCurrentScope) {
						$scope.val(firstVisibleScope).trigger("change");
					}
				});
			}
		);

		documentType = $scope.val();
		$scope.off('change')
			.on('change', function() {
				const $this = $(this);
				checkCanCleanupDimensions(function(ok) {
					if (ok === false) {
						$this.val(documentType);
					} else {
						documentType = $this.val();
						let scope = $this.children(':selected').attr('rd-scope');
						self.switchUseCaseByAttr(
							'rd-for-filter-scope',
							scope
						).done(function() {
							displayDimensions();
						});
					}
				});
			}
		);

		$filterType.trigger("change");

		helpers.configureIntegration({
			container : self.container.find('[ic-role="configure-integration"]'),
			controller: self.controller,
			tweetyApi : inConcert.app.getTweetyAPI(),
			beforeChange: function(category, newScope, newValue, callback) {
				checkCanCleanupDimensions(callback);
			},
			afterChange: function(category, newScope/*, newValue*/) {
				displayDimensions();
				// TODO: Do it in the helper
				self.switchUseCaseByAttr('rd-for-integration-type', newScope, this.container.find('[rd-integration-category="'+ category +'"]'));
			}
		}).done(function(errorThrown) {
			if (errorThrown) {
				self.emit("error", format("<%=_T('Failed to load Integration Projects: {0}')%>", _T(errorThrown)));
			}

			if (self.filter === null) {
				self.setTitle("<%=_T('New Filter')%>", "", "");
			} else {
				var filterTypeTranslated = self.filter.type === "summary" ? "<%=_T('Summary')%>" : "<%=_T('Detail')%>";

				var title = self.hasPermissionToSave() ?
					"<%=_T('Edit Filter')%> <span>\"{0}\"</span>" : 
					"<%=_T('View Filter')%> <span>\"{0}\"</span>";

				self.setTitle(format(title, filterTypeTranslated), "", self.filter.description);

				helpers.displayDataModel(self.container, self.filter, {
					disabledFields : [ "type", "scope" ],
					onFieldDisabled: _.bind(self.onFieldDisabledChange, self, true),
					listIterator   : function(propertyName, $field, item) {
						if ( item === "VCC" ) {
							return false;
						}
					}
				});
			}

			if (!self.filter) {
				_.defer(displayDimensions);
			}

			helpers.setupTextFiltering(self.container, '[ic-role="filter-by-content"]');

			callback(true);
		});
	};

	FormEditFilter.prototype.onShow = function() {
		this.initNavigation({
			allVisited: (this.filter !== null)
		});
	};

	FormEditFilter.prototype.onConfirm = function(callback) {
		var self = this;

		var result = self.validate(_.bind(self.isFieldEnabled, self));
		if (!result.success) {
			return callback(false, result.errorMessage);
		}

		var filter = $.extend({
			id: self.filter && self.filter.id || undefined
		}, result.fields, self.extras);

		self.controller.saveFilter(filter).done(function() {
			callback(true, "<%=_T('Filter saved successfully')%>");
		}).fail(function(errorThrown) {
			var message_error;
			switch ( errorThrown ) {
				case "forbidden":
					message_error = "<%=_T('You dont have permission to save')%>";
				break;
				default:
					message_error = errorThrown;
			}
			callback(false, format("<%=_T('Failed to save filter: {0}')%>", message_error));
		});
	};

	inConcert.reports.FormEditFilter = FormEditFilter;
}());
