/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, esversion: 9 */

/* global _, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function ()
{
	"use strict";

	const _T = inConcert.i18n.getString;
	const format = inConcert.Utils.format;
	let reports_dashboard  = null;
	let reports_designer   = null;
	let reports_controller = null;
	let html_container     = null;

	const confirm_jump_menu_nav = function (cb_process_render = $.noop, navigation) {
		
		if (!reports_designer || !reports_designer.currentForm ) {
			return cb_process_render();
		}

		if ( !reports_designer.currentForm.hasPermissionToSave() ) {
			reports_designer.currentForm.close();
			return cb_process_render();
		}
		else {
			navigation.confirm_jump_menu_nav({
				fn_can_confirm    : function() {
					return true;
				},
				fn_process : function (success) {
					if (!success) {
						return;
					}
					reports_designer.currentForm.close();
					cb_process_render();
				}});
		}
	};

	var translate_export_format = function(value = "") {
		value = value.toLowerCase();
		const translated_words = {
			["csv"]     : "<%=_T('CSV')%>",
			["database"]: "<%=_T('DB')%>",
			["pdf"]     : "<%=_T('PDF')%>"
		};
		let translated_word = translated_words[value];
		return translated_word || value;
	};

	var translate_send_type = function(value = "") {
		value = value.toLowerCase();
		const translated_words = {
			["database"]: "<%=_T('DATABASE')%>",
			["ftp"]     : "<%=_T('FTP')%>",
			["mail"]    : "<%=_T('E-MAIL')%>",
			["sftp"]    : "<%=_T('SFTP')%>"
		};
		let translated_word = translated_words[value];
		return translated_word || value;
	};

	const translate_range = function(value = "") {
		const translated_words = {
			["SinceLastReport"]: "<%=_T('Since Last Execution')%>",
			["Range"]          : "<%=_T('Exact Interval')%>",
			["LastDay"]        : "<%=_T('Last Day')%>",
			["LastWeek"]       : "<%=_T('Last Week')%>",
			["LastMonth"]      : "<%=_T('Last Month')%>",
			["DaysBefore"]     : "<%=_T('Last n Days')%>"
		};
		let translated_word = translated_words[value];
		return translated_word || value;
	};

	var init_designer = function( conf_dashboard )
	{
		if ( !reports_designer ) {
			// se instancia el dashbaord de reports y luego se agregan los metodos al dashboard de configuration, para que
			//   queden disponibles en el report designer
			reports_dashboard = new inConcert.reports.external.dashboard( {
				ids: {
					message       : "conf_rpts_message"       ,
					close_message : "conf_rpts_close_message" ,
					waiting       : "conf_rpts_waiting"
				}
			});

			conf_dashboard.get_controller = function() {
				if (!reports_controller) {
					reports_controller = inConcert.reports.m_ReportsController || new inConcert.reports.ReportsController( inConcert.app.getBaseDomain() );
					inConcert.reports.m_ReportsController = reports_controller;
				}
				return reports_controller;
			};
			conf_dashboard.block_right_panel   = _.bind(reports_dashboard.block_right_panel, reports_dashboard);
			conf_dashboard.unblock_right_panel = _.bind(reports_dashboard.unblock_right_panel, reports_dashboard);
			conf_dashboard.load_inner_page     = _.bind(reports_dashboard.load_inner_page, reports_dashboard);
			conf_dashboard.show_message        = _.bind(reports_dashboard.show_message, reports_dashboard);
			conf_dashboard.hide_message        = _.bind(reports_dashboard.hide_message, reports_dashboard);

			// creo instancia de report designer y le paso como parametro el dashobard de configuration
			reports_designer = new inConcert.reports.external.designer( conf_dashboard );
		}
		reports_designer.enter();
	};

	var render = function( menu_option_id, $view_container, dashboard, callback )
	{
		callback = callback || $.noop;
		var pages_uri = "/inconcert/apps/reports/pages/designer.<%=_L%>.html#grunt-cache-bust";
		var api = dashboard.get_api();

		var start = function() {
			init_designer( dashboard );
			reports_designer.menuChangeExternalOption( menu_option_id );
			callback();
		};

		if ( typeof html_container === "string" ) {
			start();
		}
		else {
			api.load_content_page({
				URI: pages_uri,
				callback: function( success, content ) {
					if ( success && typeof content === "string" ) {
						html_container = content;
						$view_container.html( html_container );
						start();
					}
				}
			});
		}
	};

	var custom_reports = function()
	{
		return {
			id: "reports",
			name: "<%=_T('Reports')%>",
			permissions: {
				view    : "view_custom_reports",
				permission_type: "vcc"
			},
			class_name: "reportItem",
			i_class_name: "far fa-file-chart-line",
			view: {
				render: render,
				confirm_jump_menu_nav: confirm_jump_menu_nav,
				title: "<%=_T('Custom Reports View')%>",
				tagline: "<%=_T('Tagline for custom reports')%>"
			},
			form: {instance: inConcert.conf_form}
		};
	};

	var metrics = function()
	{
		return {
			id: "metrics",
			name: "<%=_T('Metrics')%>",
			permissions: {
				view    : "view_custom_metrics",
				permission_type: "vcc"
			},
			class_name: "metricsItem",
			i_class_name: "far fa-chart-bar",
			view: {
				render: render,
				confirm_jump_menu_nav: confirm_jump_menu_nav,
				title: "<%=_T('Metrics View')%>",
				tagline: "<%=_T('Tagline for metrics')%>"
			},
			form: {instance: inConcert.conf_form}
		};
	};

	var filters = function()
	{
		return {
			id: "filters",
			name: "<%=_T('Filters')%>",
			permissions: {
				view    : "view_custom_filters",
				permission_type: "vcc"
			},
			class_name: "filtersItem",
			i_class_name: "far fa-filter",
			view: {
				render: render,
				confirm_jump_menu_nav: confirm_jump_menu_nav,
				title: "<%=_T('Filters View')%>",
				tagline: "<%=_T('Tagline for filters')%>"
			},
			form: {instance: inConcert.conf_form}
		};
	};

	var custom_fields = function()
	{
		return {
			id: "dimensions",
			name: "<%=_T('Custom Fields')%>",
			permissions: {
				view    : "view_custom_fields",
				permission_type: "vcc"
			},
			class_name: "fieldsItem",
			i_class_name: "far fa-poll-h",
			view: {
				render: render,
				confirm_jump_menu_nav: confirm_jump_menu_nav,
				title: "<%=_T('Custom Fields View')%>",
				tagline: "<%=_T('Tagline for custom fields')%>"
			},
			form: {instance: inConcert.conf_form}
		};
	};

	var schedules = function( base_uri )
	{
		var schedule_config;
		var swap_button_action = function( new_action, $row )
		{
			if ( !$row ) {
				return;
			}
			var columns = schedule_config.view.columns;
			var column_custom_action = null;
			$.each( columns, function( _, column ) {
				if (column.custom_action) {
					column_custom_action = column.custom_action;
					return false; // break
				}
			});
			if ( column_custom_action ) {
				var action_config = column_custom_action[ new_action ];
				var current_action = ( new_action === "deactivate" ? "activate" : "deactivate" );
				var $action_button = $row.find( "input[action='" + current_action + "']" );
				$action_button
					.attr( "action", new_action )
					.val( action_config.name );
			}
		};

		// execute custom action
		var execute_custom_action = function(action_id, $row, view, _, execute_callback = $.noop)
		{
			var row_id = $row.attr("row_id");
			var display_name = $row.attr("row_name");
			var uri = base_uri;
			var confirmation_message, waiting_message, success_message, error_message;

			action_id = action_id.toLowerCase();
			if (action_id === "activate") {
				uri = uri + "schedules/activate/";
				confirmation_message = "<%=_T('Do you want to activate Schedule {0} ?')%>";
				waiting_message = "<%=_T('Please wait, activating Schedule {0}')%>";
				success_message = "<%=_T('Schedule {0} was successfully activated.')%>";
				error_message = "<%=_T('Error activating Schedule {0}.')%>";
			}
			else if (action_id === "deactivate") {
				uri = uri + "schedules/deactivate/";
				confirmation_message = "<%=_T('Do you want to deactivate Schedule {0} ?')%>";
				waiting_message = "<%=_T('Please wait, deactivating Schedule {0}')%>";
				success_message = "<%=_T('Schedule {0} was successfully deactivated.')%>";
				error_message = "<%=_T('Error deactivating Schedule {0}.')%>";
			}
			else  if (action_id === "execute") {
				uri = uri + "schedules/execute/";
				confirmation_message = "<%=_T('Do you want to Execute Schedule {0} ?')%>";
				waiting_message = "<%=_T('Please wait, Executing Schedule {0}')%>";
				success_message = "<%=_T('Schedule {0} was successfully Executed.')%>";
				error_message = "<%=_T('Error Executing Schedule {0}.')%>";
			}
			else if(action_id === "export_files_not_uploaded"){
				uri = uri + "schedules/export_not_uploaded_files_record/";
				confirmation_message = "<%=_T('Do you want to export the record of files not uploaded to ftp ?')%>";
				waiting_message = "<%=_T('Please wait, exporting record.')%>";
				success_message = "<%=_T('The export was successful.')%>";
				error_message = "<%=_T('Unable to export record.')%>";
			}
			else {
				throw ("Unrecognized action " + action_id);
			}
			confirmation_message = format(confirmation_message, display_name);
			waiting_message = format(waiting_message, display_name);
			success_message = format(success_message, display_name);
			error_message = format(error_message, display_name);

			view.dashboard.hide_status_message();

			$.inconcertPopup.Show(
				"<%=_T('Confirm')%>",
				confirmation_message,
				jQuery.inconcertPopup.ButtonSets.BS_YESNO,
				function(response) {
					if (response.toLowerCase() !== "yes") {
						return;
					}
					view.dashboard.show_waiting(waiting_message);
					var request_response = function(success, error)
					{
						var message, message_type, timeout_ms;
						if (success) {
							message = success_message;
							message_type = "Info";
							var new_state, new_action_id;
							let row_data = $row.data("row_data") || {};

							if ( action_id === "deactivate" ) {
								new_state = "inactive";
								new_action_id = "activate";
								swap_button_action( new_action_id, $row );
								row_data.State = new_state;
								view.update_row(row_data, $row);
							}
							else if ( action_id === "activate" ) {
								new_state = "active";
								new_action_id = "deactivate";
								swap_button_action( new_action_id, $row );
								row_data.State = new_state;
								view.update_row(row_data, $row);
							}
							else if ( action_id === "execute" ) {
								// TODO: Se necesita realizar algo ?
							}
							else if ( action_id === "execute" ) {
								// TODO: Se necesita realizar algo ?
							}
							else {
								throw ("Unrecognized action " + action_id);
							}
						}
						else {
							let translated_actions = {
								activate  : "<%=_T('Activate')%>",
								deactivate: "<%=_T('Deactivate')%>",
								execute   : "<%=_T('Execute')%>"
							};

							let translated_action = translated_actions[action_id] || "<%=_T('Apply action in ')%>";

							switch (error.result.toUpperCase())
							{
								case "FORBIDDEN":
									error_message = "<%=_T('Unable to {0} {1} schedule. You dont have permission.')%>";
									error_message = format(error_message, translated_action, display_name);
									break;

								case "CURRENT_SCHEDULE_IS_RUNNING":
									error_message = "<%=_T('Unable to {0} {1} schedule. Current schedule is running.')%>";
									error_message = format(error_message, translated_action, display_name);
									break;

								case "SCHEDULER_DOES_NOT_EXIST":
									error_message = "<%=_T('Unable to {0} {1} schedule. Scheduler does not exist.')%>";
									error_message = format(error_message, translated_action, display_name);
									break;

							// TODO: agregar mensajes de errores para estos tipos.
								// deactiviating

								//activating
							}

							message = error_message;
							message_type = "Error";
							timeout_ms = false; //infinite
						}
						view.dashboard.show_status_message(message, message_type, timeout_ms);
						execute_callback(success, error);
					};

					if(action_id === "export_files_not_uploaded"){
						const rowData = $row.data("row_data");
						return $.fileDownload(base_uri + "schedules/export_not_uploaded_files_record/", {
							httpMethod: "GET",
							cookieName: "download_started",
							data: {
								data: JSON.stringify(rowData.data.filesNotUploaded)
							}
						}).done(function () {
							view.dashboard.show_status_message(success_message, "Info", 6000);
							execute_callback(true);
						}).fail(function (error) {
							view.dashboard.show_status_message(error_message, "Error", 6000);
							execute_callback(false, error);
						});
					}

					// ejecuto el request para el cambio de estado
					inConcert.invoke({
						url: uri,
						type: "POST",
						timeout: 1800000, // 30 minutes
						data: JSON.stringify({
							Id: row_id
						}),
						dataType: "json",
						contentType: "application/json",
						success: function() {
							request_response(true);
						},
						error: function(_, text_status) {
							var err;
							try {
								err = JSON.parse(text_status);
								err = err.result;
							}
							catch(e) {
								err = text_status;
							}
							request_response(false, err);
						}
					});

				},
				undefined,
				{
					popup_id        : "conf_inconcertPopup",
					container_class : "confirmationPopUp",
					overlay_class   : "confirmationOverlay"
				}
			);
		};

		// is action enabled
		var is_action_enabled = function( action, row )
		{
			/*
			 * Valores a retornar:
			 *    1  = La accion se debe visualizar y estar Enabled.
			 *    0  = La accion se debe visualizar y estar Disabled.
			 *    -1 = La accion no se genera en el markup. (No aparece el boton de la accion).
			 */

			var has_permission_to_activate_or_deactivate = function() {
				return inConcert.permissions.to_action("reports" /*app*/, "activate_schedule");
			};
			var has_permission_to_execute = function() {
				return inConcert.permissions.to_action("reports" /*app*/, "execute_schedule");
			};

			switch( action.toLowerCase() )
			{
				case "edit":
					if ( row.State === "running" ) {
						return 0;
					}
					return 1;

				case "delete":
					if ( row.State === "running" ) {
						return 0;
					}
					return 1;

				case "activate":
					if ( !has_permission_to_activate_or_deactivate() ) {
						return -1;
					}
				// si el schedule se encuentra available, desactiva el boton para activarlo.
					if ( row.State === "inactive" ) {
						return 1;
					}
				// si el schedule se encuentra available, desactiva el boton para activarlo.
					if ( row.State === "expired" || row.State === "running" ) {
						return 0;
					}
					return -1;

				case "deactivate":
					if ( !has_permission_to_activate_or_deactivate() ) {
						return -1;
					}
					if ( row.State === undefined ) {
						return 1;
					}
				// si el schedule se encuentra desactivado, desactiva el boton para desactivar el schedule.
					if ( row.State !== "active" ) {
						return -1;
					}
					break;
				case "execute":
					if (!has_permission_to_execute()) {
						return -1;
					}
					if (row.State === "running") {
						return 0;
					}
					// si el schedule se encuentra available, desactiva el boton para ejecutar.
					if (row.State === "inactive") {
						return 0;
					}
					return 1;
				case "export_files_not_uploaded": //FilesNotUpdated to FTP
					if (row.data && row.data.filesNotUploaded && row.success) {
						return 1;
					}
					return -1;
			}
			return 1;
		};

		var get_message_error = function( type, err, name )
		{
			var message;
			if (type === "delete") {
				var err_messages = {
					"SCHEDULE_DOES_NOT_EXIST"    : "<%=_T('Unable to delete {0} schedule. This schedule does not exist.')%>",
					"FORBIDDEN"                  : "<%=_T('Unable to delete {0} schedule. You dont have permission.')%>",
					"CURRENT_SCHEDULE_IS_RUNNING": "<%=_T('Unable to delete {0} schedule. Current schedule is running.')%>"
				};
				if (typeof err === "object") {
					err = err.result || "";
				}
				message = err_messages[err.toUpperCase()] || "<%=_T('Error deleting {0} schedule.')%>";
				message = format( message, name );
			}
			return message;
		};

		var get_formatted_filter = function( sch ) {
			var formatted_filter = "";
			var i = 1;
			var exclude = { "withContent": true, "Type": true };
			$.each( sch.Params || {}, function( name, current_value ) {
				if ( exclude[ name ] ) {
					return true; // continue
				}
				var field = _T(name) + "<%=_T(':')%>" + " ";
				formatted_filter = formatted_filter + ( ( i > 1  ? ", " : "" ) + field + current_value );
				i++;
			});
			return formatted_filter;
		};

		var get_value = function( sch, type )
		{
			var report_info = sch.__report_info;
			if ( typeof report_info !== "object" ) {
				report_info = JSON.parse( report_info || "{}" );
			}

			var resp = {};

			switch( type )
			{
				case "name":
					resp.original_value = report_info.name;
					if (report_info.isCustomReport) {
						resp.value = "<%=_T('Custom')%>" + " - " + report_info.name;
					}
					else {
						resp.value = (report_info.id ? (report_info.id + " - ") : "") + report_info.label;
					}
					var formatted_filter = get_formatted_filter( sch );
					if ( formatted_filter.length > 0 ) {
						resp.value = resp.value + " ( " + formatted_filter + " ) ";
					}
				break;

				case "type":
					resp.original_value = report_info.type;
					resp.value = report_info.typeLabel;
				break;

				case "rang":
					if (sch.QueryRange) {
						resp.original_value = sch.QueryRange;
						resp.value = translate_range(sch.QueryRange.type);
					}
					else {
						resp.original_value = {};
						resp.original_value = "";
					}
				break;

				case "peri":
					resp.original_value = {
						RepeatPolicy     : sch.RepeatPolicy,
						RepeatWeekday    : sch.RepeatWeekday,
						RepeatDay        : sch.RepeatDay,
						RepeatHour       : sch.RepeatHour,
						CustomRepeatDays : sch.CustomRepeatDays
					};
					// si no viene repeat policy es One time
					resp.value = sch.RepeatPolicy || "<%=_T('One Time')%>";
				break;

				case "export_format":
					resp.original_value = sch.ExportFormat || "";
					resp.value = translate_export_format(sch.ExportFormat);
				break;

				case "send_type":
					resp.original_value = sch.SendType || "";
					resp.value = translate_send_type(sch.SendType);
				break;

				case "state":
					resp.original_value = sch.State || {};
					var states_def = {
						active: {
							text: "<%=_T('Active')%>",
							class_name: "gridChipActive"
						},
						inactive: {
							text: "<%=_T('Inactive')%>",
							class_name: "gridChipInactive"
						},
						expired: {
							text: "<%=_T('Expired')%>",
							class_name: "gridChipExpired"
						},
						running: {
							text: "<%=_T('Running')%>",
							class_name: "gridChipRunning"
						}
					};
					var state = sch.State || "expired";
					var def = states_def[ state ];
					if ( !def ) {
						resp.value = state;
						resp.class_name = state;
					}
					else {
						resp.value = def.text || state;
						resp.class_name = def.class_name || state;
					}
				break;
			}

			return resp;
		};

		var get_history_value = function( sch_history, type )
		{
			let resp = {};
			let data = (sch_history.data || {});
			let translated_word;

			switch( type )
			{
				case "execution_date":
					resp.original_value = sch_history.__id; // en id viene el timestamp
					resp.value = sch_history.__id;
				break;

				case "execution_by":
					resp.original_value = data.executionType || "BY SCHEDULER";
					let translated_words = {
						["BY SCHEDULER"]  : "<%=_T('BY SCHEDULER')%>",
						["MANUAL BY USER"]: "<%=_T('MANUAL BY USER')%>"
					};
					translated_word = translated_words[resp.original_value];
					if (data.userId) {
						let simple_user_id = inConcert.app.get_id(data.userId || "");
						translated_word = translated_word + ": " + simple_user_id;
					}
					resp.value = translated_word;
				break;

				case "send_type":
					resp.original_value = data.sendType || "";
					translated_word = translate_send_type(resp.original_value);
					resp.value = translated_word;
				break;

				case "reason":
					resp.original_value = "";
					if (typeof (data.reason || "") === "object") {
						const data_reason = data.reason;
						if (data.reason.result === "NOT_UPLOAD_ALL_FILES") {
							resp.original_value = format("<%=_T('Error, {0}% ({1} - {2}) or more of the files were not uploaded to ftp.')%>", data_reason.percentageNotUploaded, (data_reason.totalFilesToUpload - data_reason.counterUploadedFiles).toFixed(), data_reason.totalFilesToUpload);
						}
					}else{
						resp.original_value = data.reason || "";
					}

					resp.value = resp.original_value;
				break;

				case "success":
					resp.original_value = sch_history.success;
					resp.value = sch_history.success;
				break;
			}

			return resp;
		};

		var get_row_classes = function( row )
		{
			var row_classes = {
				active   : "",
				expired  : "expired",
				inactive : "deactivated",
				running  : "running"
			};
			var state = row.State || "";
			return row_classes[ state ] || "";
		};

		var get_history_tooltip = function( sch_history, type )
		{
			let tooltip = "";
			switch( type )
			{
				case "success":
					tooltip = sch_history.success? "<%=_T('Yes')%>": "<%=_T('No')%>";
				break;
			}

			return tooltip;
		};

		schedule_config = {
			id: "schedules",
			name: "<%=_T('Schedules')%>",
			permissions: {
				view    : "view_schedule",
				save    : "save_schedule",
				erase   : "delete_schedule",
				permission_type: "vcc"
			},
			class_name: "schedulesItem",
			i_class_name: "far fa-calendar-day",
			view: {
				render: "/inconcert/apps/reports/pages/view_report_content_default.<%=_L%>.html#grunt-cache-bust",
				title: "<%=_T('Schedules')%>",
				tagline: "<%=_T('Tagline for tickets')%>",
				new_button_name: "<%=_T('Create schedule')%>",
				row_id: "Id",
				display_name: "Report",
				messages: {
					erase: "<%=_T('Do you want to delete schedule {0} ?')%>",
					err: get_message_error
				},
				uri: {
					erase  : base_uri + "schedules/delete/"   ,
					search : base_uri + "schedules/get_all/"
				},
				paginator: {
					show_at_least : 8,
					rows_per_page : 10
				},

				// estos son los atributos que el server va a retornar por cada entidad
				source: [
					"Id", "Report", "__report_info", "Params", "QueryRange", "RepeatPolicy", "RepeatWeekday", "RepeatDay",
					"RepeatHour", "CustomRepeatDays", "State", "ExportFormat", "SendType", "__history"
				],

				// definiciones espcificas por cada row ( clases particulares a aplicar, etc )
				rows: {
					get_row_classes: get_row_classes
				},

				// columnas de la grilla principal
				columns: [
					{ id: "_number"      , name: "" } , //, type: "counter" //, rowspan: true },
					{ id: "ReportName"   , name: "<%=_T('Report Name')%>"  , get_value: function( sch ) { return get_value( sch, "name"  ); } } ,
					{ id: "Range"        , name: "<%=_T('Range')%>"        , get_value: function( sch ) { return get_value( sch, "rang"  ); } } ,
					{ id: "Period"       , name: "<%=_T('Periodicity')%>"  , get_value: function( sch ) { return get_value( sch, "peri"  ); } } ,
					{ id: "State"        , name: "<%=_T('State')%>"        , get_value: function( sch ) { return get_value( sch, "state" ); } , type: "chip" } ,
					{ id: "ExportFormat" , name: "<%=_T('Export Format')%>", get_value: function( sch ) { return get_value( sch, "export_format"); } } ,
					{ id: "SendType"     , name: "<%=_T('Send Type')%>"    , get_value: function( sch ) { return get_value( sch, "send_type"); } } ,
					{ id: "_action"      , name: "" ,
						system_action: { edit: true, erase: true },
						custom_action: {
							activate: {
								name: "<%=_T('Activate')%>",
								execute: execute_custom_action,
								class_name: "activate"
							},
							deactivate: {
								name: "<%=_T('Deactivate')%>",
								execute: execute_custom_action,
								class_name: "deactivate"
							},
							execute: {
								name: "<%=_T('Execute')%>",
								execute: execute_custom_action,
								class_name: "execute"
							},
							export_files_not_uploaded: { //Export record of Files Not Updated to FTP
								execute: execute_custom_action,
							}
						},
						is_enabled: is_action_enabled
					}
				],
				// subgrilla definicion
				sub_grid: {
					data_source: "__history", // aca vienen los registros de la subgrilla
					colapsed: true,
					messages: function( type ) {
						var message;
						if ( type === "no_records_found" ) {
							message = "<%=_T('No executions found.')%>";
						}
						return message;
					},
					sort: function( rows ) {
						var sorted_rows = inConcert.Utils.hash_to_sorted_array(
							rows,
							function( row ) {
								var timestamp = parseInt( row.__timestamp || 0, 10 );
								return timestamp;
							},
							undefined,
							{ order: "desc" }
						);
						return sorted_rows;
					},
					// columnas de la subgrilla
					columns: [
						// las columns del colspan
						{id: "_number", name: "", type: "counter"},
						{
							id         : "History",
							name       : "<%=_T('Execution Date')%>",
							get_value  : function( sch_history ) { return get_history_value( sch_history, "execution_date" ); }
						},
						{
							id         : "ExecBy",
							name       : "<%=_T('Execution By')%>",
							get_value  : function( sch_history ) { return get_history_value( sch_history, "execution_by" ); }
						},
						{
							id         : "SendType",
							name       : "<%=_T('Send Type')%>",
							get_value  : function( sch_history ) { return get_history_value( sch_history, "send_type" ); }
						},
						{
							id: "_action"    , name: "", class_name:"imageWarningColumn",
							custom_action:{
								export_files_not_uploaded: { //Export record of Files Not Updated to FTP
									name: "<%=_T('Export')%>",
									class_name: "warning",
									tooltip: function(row) {
										if (row.data && row.data.filesNotUploaded) {
											const totalFiles = row.data.filesNotUploaded.totalFilesToUpload;
											const totalFilesNotUploaded = row.data.filesNotUploaded.totalFilesToUpload - row.data.filesNotUploaded.counterUploadedFiles;
											return format("<%=_T('Some files ({0} - {1}) were not uploaded to ftp successfully. Press to export the record.')%>", totalFilesNotUploaded, totalFiles);
										}
										return "";
									}
								}
							},
							is_enabled: is_action_enabled
						},
						{
							id         : "Success",
							name       : "<%=_T('Successfully')%>",
							type       : "boolean",
							class_name : "booleanColumnTitle",
							get_value  : function( sch_history ) { return get_history_value( sch_history, "success" ); },
							get_tooltip: function( sch_history ) { return get_history_tooltip( sch_history, "success" ); }
						},
						{
							id         : "Reason",
							name       : "<%=_T('Reason')%>",
							get_value  : function( sch_history ) { return get_history_value( sch_history, "reason" ); }
						}
					]
				},
				filter: { fields: { "Report": { control: null } } },
				realtime: null,
				on_update_view: function( _, $row )
				{
					// siempre que se actualiza un schedule, el mismo pasa a estar activo, entonces el boton de la accion
					//  pasa a ser "deactivate".
					var new_action = "deactivate";
					swap_button_action( new_action, $row );
				}
			},
			form: { instance: inConcert.rd_form_schedule }
		};

		return schedule_config;
	};

	//------------------------------------------------------
	// get all navigation config
	var get_navigation_config = function()
	{
		var base_uri = inConcert.app.getBaseDomain() + "/inconcert/apps/reports/";
		return {
			custom_reports : custom_reports()      ,
			metrics        : metrics()             ,
			filters        : filters()             ,
			custom_fields  : custom_fields()       ,
			schedules      : schedules( base_uri )
		};
	};

	function rpt_config() {}

	rpt_config.prototype.get_app_config = function()
	{
		return {
			id         : "reports"               ,
			name       : "<%=_T('Reports Designer')%>"  ,
			selected   : false                   ,
			class_name : "appReportsDesigner"    ,
			main_html  : "/inconcert/apps/reports/pages/conf_report_main.<%=_L%>.html#grunt-cache-bust",
			navigation : get_navigation_config(),
			get_skin: function() {
				return "skinOCCNew";
			},
			can_show   : function() {
				return inConcert.permissions.to_category( "reports", "view" );
			},
			css        : [
//				{ id: "internal_layout_css"                      , href: "/inconcert/apps/configuration/css/masterInternalPageLayout.css#grunt-cache-bust"          } ,
//				{ id: "internal_forms_css"                       , href: "/inconcert/apps/configuration/css/masterInternalPageForms.css#grunt-cache-bust"           } ,
				{ id: "reports_designer_css"                     , href: "/inconcert/apps/reports/css/skin-2021/reportsDesigner.css#grunt-cache-bust"               },
				// Agrega Lolo para SkinOCCNew
				{ id: "skin_occ_internal_forms_css"              , href: "/inconcert/apps/configuration/css/skin-2021/masterInternalPageForms.css#grunt-cache-bust"  },
				{ id: "skin_occ_internal_page_layout_css"        , href: "/inconcert/apps/configuration/css/skin-2021/masterInternalPageLayout.css#grunt-cache-bust" },
				{ id: "skin_occ_inbox_master_popups_css"     , href: "/inconcert/apps/agent/css/skin-2021/inboxMasterPopUps.css#grunt-cache-bust" },
				//Estos hay que ver si se pueden sacar de la inboxMaster.html
				{ id: "skin_occ_master_buttons_css"              , href: "/inconcert/apps/agent/css/skin-2021/masterButtons.css#grunt-cache-bust"                    },
				{ id: "skin_occ_inbox_master_css"                , href: "/inconcert/apps/agent/css/skin-2021/inboxMaster.css#grunt-cache-bust"                      },
				{ id: "skin_occ_master_quick_interactions_css", href: "/inconcert/apps/agent/css/skin-2021/masterQuickInteractions.css#grunt-cache-bust"  },
				{ id: "skin_occ_messages_tooltips_css", href: "/inconcert/apps/agent/css/skin-2021/genericMessagesAndTooltips.css#grunt-cache-bust"  },
			]
		};
	};

	if (typeof inConcert.rpt_config !== "function") {
		inConcert.rpt_config = rpt_config;
	}
}());
