/*jshint bitwise:true, browser:true, curly:false, eqeqeq:true, noarg:true, noempty:true, strict:true, undef:true, unused:true, -W014, esversion: 11 */
/*jshint quotmark:false*/
/*global $, _, inConcert*/
 
this.inConcert = this.inConcert || {};
this.inConcert.reports = this.inConcert.reports || {};

(function () {
	"use strict";

	const app_name = "viewer";
	const _T = inConcert.i18n.getString;
	const format = inConcert.Utils.format;
	const helpers = inConcert.reports.helpers;

	const view_categories = {
		TEAM : {
			className	: "categoryTeam",
			label		: "<%=_T('Team')%>",
			tooltip		: "<%=_T('Team, agent states, internal chat.')%>",
			index		: 1
		},
		CALL : {
			className	: "categoryCall",
			label		: "<%=_T('Call')%>",
			tooltip		: "<%=_T('Telephony conversations.')%>",
			index		: 2
		},
		CHAT : {
			className	: "categoryChat",
			label		: "<%=_T('Chat')%>",
			tooltip		: "<%=_T('Web chat conversations.')%>",
			index		: 2
		},
		FACEBOOK : {
			className	: "categoryFacebook",
			label		: "<%=_T('Facebook')%>",
			tooltip		: "<%=_T('Facebook conversations.')%>",
			index		: 2
		},
		INSTAGRAM : {
			className	: "categoryInstagram",
			label		: "<%=_T('Instagram')%>",
			tooltip		: "<%=_T('Instagram conversations.')%>",
			index		: 2
		},
		YOUTUBE : {
			className	: "categoryYoutube",
			label		: "<%=_T('Youtube')%>",
			tooltip		: "<%=_T('Youtube conversations.')%>",
			index		: 2
		},
		APPSTORE : {
			className	: "categoryYoutube",
			label		: "<%=_T('Appstore')%>",
			tooltip		: "<%=_T('Appstore reviews.')%>",
			index		: 2
		},
		GOOGLEPLAY : {
			className	: "categoryYoutube",
			label		: "<%=_T('Googleplay')%>",
			tooltip		: "<%=_T('Googleplay reviews.')%>",
			index		: 2
		},
		RCS : {
			className	: "categoryRcs",
			label		: "<%=_T('Rcs')%>",
			tooltip		: "<%=_T('Rcs conversations.')%>",
			index		: 2
		},
		IM : {
			className	: "categoryIM",
			label		: "<%=_T('IM')%>",
			tooltip		: "<%=_T('IM conversations.')%>",
			index		: 2
		},
		MAIL : {
			className	: "categoryMail",
			label		: "<%=_T('E-mail')%>",
			tooltip		: "<%=_T('E-mail conversations.')%>",
			index		: 2
		},
		SMS : {
			className	: "categorySMS",
			label		: "<%=_T('SMS')%>",
			tooltip		: "<%=_T('SMS conversations.')%>",
			index		: 2
		},
		WHATSAPP : {
			className	: "categoryWHATSAPP",
			label		: "<%=_T('Whatsapp')%>",
			tooltip		: "<%=_T('Whatsapp conversations.')%>",
			index		: 2
		},
		TWITTER : {
			className	: "categoryTwitter",
			label		: "<%=_T('Twitter')%>",
			tooltip		: "<%=_T('Twitter conversations.')%>",
			index		: 2
		},
		WEBCONTACT : {
			className	: "categoryWebForms",
			label		: "<%=_T('Contact Form')%>",
			tooltip		: "<%=_T('Contact form and related conversations.')%>",
			index		: 2
		},
		VOICEMAIL : {
			className	: "categoryVoicemail",
			label		: "<%=_T('Voice Mail')%>",
			tooltip		: "<%=_T('Voice Mail conversations.')%>",
			index		: 2
		},
		MANAGEMENT : {
			className	: "categoryManagement",
			label		: "<%=_T('Management')%>",
			tooltip		: "<%=_T('Management results, rules and conversations by contact.')%>",
			index		: 3
		},
		DIALER : {
			className	: "categoryDialer",
			label		: "<%=_T('Dialer')%>",
			tooltip		: "<%=_T('Dialer campaigns, batches, contacts and statistics')%>",
			index		: 3
		},
		CUSTOM : {
			className	: "categoryCustom",
			label		: "<%=_T('Custom')%>",
			tooltip		: "<%=_T('Custom')%>",
			index		: 4
		}
	};
	inConcert.reports.view_categories = view_categories;
	
	const typesOrder = {
		summary				: 1,
		detail				: 2,
		livedetail			: 2,
		dialerbatch			: 2,
		realtime			: 3,
		dialercontacts		: 3
	};
	
	const typesClassNames = {
		detail				: "detail",
		livedetail			: "detail",
		summary				: "summary",
		realtime			: "realtime",
		dialerbatch			: "dialerbatch",
		dialercontacts		: "dialercontacts"
	};

	const resultSizes = [
		15,
		30,
		50
	];

	function ReportsViewer(dashboard)
	{
		var self = $.extend(this, {
			m_dashboard					: dashboard,
			m_controller				: dashboard.get_controller(),
			m_currentReport				: null,
			m_currentReportTag			: ""
		});
		self.m_currentCategory = null;

		self.initialize_once = _.once(function() {
			var setupExportFormats = function() {
				var $cmdExportReport = $('#cmd_export_report')
					.prop('disabled', true);

				self.m_controller.loadExportFormats().done(function(formats) {
					if (formats.length === 0) {
						self.m_dashboard.show_message("messageInfo", "<%=_T('Report exportation will not be available, no exportation sources found')%>");
						return;
					}

					var $exportFormats = $('#export_options_list').on("click", "li", function() {
						self.export_report($(this).attr('rd-export-format'));
					});

					var rowCount = helpers.populateList($exportFormats, formats);
					if (rowCount === 1) {
						$cmdExportReport.on("click", function() {
							$exportFormats.children('[ic-role!="template"]').click();
						});
					} else {
						$cmdExportReport.on("click", function()
						{
							$exportFormats.children('[rd-export-format="pdf"]')[
								self.m_currentReport.vcc === "system"
								? "hide"
								: "show"
							]();
							$exportFormats.show();
							window.setTimeout(function () {
								$("body").one("click", function () { $exportFormats.hide(); });
							}, 0);
						});
					}

					$cmdExportReport.prop('disabled', false);

				}).fail(function(errorThrown) {
					self.m_dashboard.show_message("messageError", "<%=_T('Failed to load export formats: {0}')%>", errorThrown);
				});
			};

			self.m_controller.on("reportAdded", function() {
				self.initReportsAndCategories();
			});

			self.m_controller.on("reportUpdated", function() {
				self.initReportsAndCategories();
			});

			self.m_controller.on("reportDeleted", function() {
				self.initReportsAndCategories();
			});

			// setup handler for the exec button
			$('#cmd_exec_report').on("click", function() {
				self.execute_report();
			});
			
			$("#reportsViewerShowHome").on("click", function() {
				self.__toggleHome(true);
			});

			// pressing ENTER on an input field triggers execute
			$('#report_filters').on("keypress", '.form-control', function(e) {
				if (e.keyCode === 13) {
					$('#cmd_exec_report').click();
				}
			});

			$('#report_results_grid').on('click', '.rowHeaderForNestedTable', function() {
				var $header = $(this);
				$header.next().toggle(200).promise().done(function() {
					$header.toggleClass('collapsed');
				});
			});

			var $reschedule = $('#cmd_reschedule_report');
			if ( inConcert.permissions.to_action( 'reports', 'save_schedule', 'SAVE' ) ) {
				$reschedule.show();
				$reschedule.on("click", function() {
					self.add_scheduler_report();
				});
			}
			else {
				$reschedule.hide();
				$reschedule.off("click");
			}
			
			//Collapse menu and filter
			$("#reportsViewerCollapseMenu")
				.off("click")
				.on("click", function() {
					self.collapseExpandMenu();
				});
			$("#reportTitleContainer")
				.off("click")
				.on("click", function() {
					self.collapseExpandFilter();
				});

			//filter
			$("#reportsResultsSizeSelector").html(_.reduce(resultSizes, function(memo, size) {
				return memo + format("<option value='{0}'>{0}</option>", size);
			}, ""));

			//charts
			$("#reportsLineChartsSelector").click(function() {
				$("#reportsTotalChartsSelector").removeClass("selected");
				$("#reportsTotalChartsContainer").removeClass("selected");
				
				$(this).addClass("selected");
				$("#reportsLineChartsContainer").addClass("selected");
			});
			$("#reportsTotalChartsSelector").click(function() {
				$("#reportsLineChartsSelector").removeClass("selected");
				$("#reportsLineChartsContainer").removeClass("selected");
				
				$(this).addClass("selected");
				$("#reportsTotalChartsContainer").addClass("selected");
				
				self.__showTotalCharts();
			});

			this.initReportsAndCategories();
			setupExportFormats();
		});
	}

	/*!
	 * App method
	 */
	ReportsViewer.prototype.enter = function() {
		this.initialize_once();
	};

	ReportsViewer.prototype.collapseExpandMenu = function() {
		var $menu = $("#reportMenuContainer");
		//var $button = $("#reportsViewerCollapseMenu");
		var collapse = $menu.hasClass("expanded");
		$menu
			.removeClass(collapse? "expanded" : "collapsed")
			.addClass(collapse? "collapsed" : "expanded");
		//$button
		//	.text(collapse? "<%=_T('Expand Menu')%>" : "<%=_T('Collapse Menu')%>");
	};

	ReportsViewer.prototype.collapseExpandFilter = function() {
		var $filter = $("#reportFilterContainer");
		var collapse = $filter.hasClass("expanded");
		$filter
			.removeClass(collapse? "expanded" : "collapsed")
			.addClass(collapse? "collapsed" : "expanded");
	};

	ReportsViewer.prototype.__groupReportsByCategory = function(categories, reports) {
		var groupedReports = {};
		//load categories
		$.each(categories, function(_, category) {
			var categoryView = view_categories[category.name];
			if (categoryView) {
				$.extend(true, category, categoryView);
			}
			groupedReports[category.name] = { category : category, reports : [] };
		});
		//add reports
		$.each(reports, function(_, report) {
			if (!groupedReports[report.view_category]) {
				return;
			}
			groupedReports[report.view_category].reports.push(report);
		});
		//sort reports
		$.each(groupedReports, function(_, category) {
			category.reports.sort(function(a, b) {
				//first by type
				var typeOrder = typesOrder[a.type] - typesOrder[b.type];
				if (typeOrder !== 0) {
					return typeOrder;
				}
				//then by label
				return (a.label || a.name || "").localeCompare(b.label || b.name || "");
			});
		});
		//sort categories
		var groupedReports_a = [];
		$.each(categories, function(_, category) {
			groupedReports_a.push(groupedReports[category.name]);
		});
		groupedReports_a.sort(function(a, b) {
			//first by index
			var index = (a.category.index || 1000) - (b.category.index || 1000);
			if (index !== 0) {
				return index;
			}
			//then by label
			return (a.category.label || "").localeCompare(b.category.label || "");
		});
		return groupedReports_a;
	};

	ReportsViewer.prototype.initReportsAndCategories = function() {
		const self = this;
		$("#waiting_for_list").show();
		self.m_controller.getCategories(function(status, categories) {
			if (!status) {
				$("#waiting_for_list").hide();
				return;
			}
			self.m_controller.getAllReports(function(status, reports) {
				if (!status) {
					$("#waiting_for_list").hide();
					return;
				}
				self.__loadReports(categories, reports);
			});
		});
	};

	ReportsViewer.prototype.__toggleHome = function(show) {
		$("#reportsViewerHome").toggle(!!show);
		$("#reportsViewerContainer").toggle(!show);
		$("#reportsViewerShowHome").toggleClass("selected", !!show);
		$("#reportsViewerShowHome").prop("disabled", !!show);
		if (show) {
			$("#reports_categories_container").find(".selected").removeClass("selected");
			$("#reportsViewerHome").find(".selected").removeClass("selected");
		}
	};
	
	ReportsViewer.prototype.__loadReports = function(categories, reports) {
		const self = this;
		var groupedReports = self.__groupReportsByCategory(categories, reports);
		var html = "";
		var htmlHome = "";
		var currentReport = self.m_currentReport && self.m_currentReport.get_id();

		$.each(groupedReports, function(_, group) {
			var category = group.category;
			var reports = group.reports;

			html += format(
				"<li class='reportsCategoryItem {0}' category='{1}' title='{2}' category_selector='true'>" +
					"<div class='categoryItemTitle' category_selector='true'>" +
						"<i class='categoryItemIcon' category_selector='true'></i>" +
						// "<label class='categoryItemTextBack' category_selector='true'><%=_T('Go back')%></label>" +
						"<div category_selector='true' class='goBackButtonText categoryItemTextBack'><i class='fas fa-chevron-left'></i> <%=_T('Go back')%></div>" +
						"<label class='categoryItemText' category_selector='true'>{3}</label>" +
					"</div>" +	
					"<ul class='reportsList selBefore'>",
				category.className, category.name, category.tooltip, category.label
			);

			htmlHome += format(
				"<div class='reportCategoryBox {0}' category='{1}' title='{2}' category_selector='true'>" +
					"<div class='categoryBoxTitle' category_selector='true'>" +
						"<i class='categoryBoxIcon' category_selector='true'></i>" +
						"<label class='categoryBoxTitleText' category_selector='true'>{3}</label>" +
					"</div>" +	
					"<ul class='reportBoxList'>",
				category.className, category.name, category.tooltip, category.label
			);

			$.each(reports, function(_, report) {
				html += format(
						"<li class='reportsListItem {0}' type='{1}' report_id='{2}' title='{3}'>" +
							"<label class='listItemText'>{4}</label>" +
						"</li>",
					typesClassNames[report.type],
					report.type,
					report.id,
					report.description || report.label || report.name,
					report.label || report.name
				);

				htmlHome += format(
						"<li class='reportBoxListItem {0}' type='{1}' report_id='{2}' category_name='{3}' title='{4}'>" +
							"<label class='boxListItemText'>{5}</label>" +
						"</li>",
					typesClassNames[report.type],
					report.type,
					report.id,
					category.name,
					report.description || report.label || report.name,
					report.label || report.name
				);
			});
			html += "</ul>" +
				"</li>";

			htmlHome += "</ul>" +
				"</div>";
		});

		const $homeBtn = $("#reportsViewerShowHome");
		const $categories = $("#reports_categories_container");

		const onEnterTreeCategory = ($category) => {
			$categories.addClass("reportsActiveMenuCategory");
			$category.addClass("selected").siblings().hide().removeClass("selected");
			$homeBtn.hide();
			self.m_currentCategory = $category;
		};

		const onQuitTreeCategory = () => {
			const $category = self.m_currentCategory;
			if (!$category) {
				return;
			}
			$categories.removeClass("reportsActiveMenuCategory");
			$category.removeClass("selected").siblings().show();
			// el Back de una categoria nos deja posicionado en HOME
			$homeBtn.trigger("click").show();
			self.m_currentCategory = null;
		};

		$categories.html(html)
			.children("li")
			//categories
			.click(function(e) {
				const $target = $(e.target);
				if (!$target.attr("category_selector")) {
					return;
				}
				const $category = $(this);
				const $titleDiv = $category.children("div.categoryItemTitle");
				const minHeight = $titleDiv.offset().top + $titleDiv.height() + 15;
				if (minHeight < e.pageY) {
					return;
				}
				if ($category.hasClass("selected")) {
					onQuitTreeCategory();
				}
				else {
					onEnterTreeCategory($category);
				}
			});

		const $home = $("#reportsViewerHomeBody");
		$home.html(htmlHome)
			.children("div")
			//categories
			.click(function(e) {
				var $target = $(e.target);
				if (!$target.attr("category_selector")) {
					return;
				}
				var $category = $(this);
				if ($category.hasClass("selected")) {
					$category.removeClass("selected");
				}
				else {
					$category
						.addClass("selected")
						.siblings()
						.removeClass("selected");
				}
			});

		$categories
			.find("li[report_id]")
			//reports
			.click(function() {
				if (self.m_currentReport) {
					$categories
						.find(format("li[report_id={0}]", self.m_currentReport.get_id()))
						.removeClass("selected");
				}
				var $category = $(this).parent().parent();
				if (!$category.hasClass("selected")) {
					$category
						.addClass("selected")
						.siblings()
						.removeClass("selected");
				}

				var $elementHome = $home.find(format("li[report_id={0}]", $(this).attr("report_id")));
				$elementHome
						.addClass("selected")
						.siblings()
						.removeClass("selected");

				var $categoryHome = $elementHome.parent().parent();
				if (!$categoryHome.hasClass("selected")) {
					$categoryHome
						.addClass("selected")
						.siblings()
						.removeClass("selected");
				}
				
				self.__toggleHome(false);
				$(this).addClass("selected");
				self.enterReport( $(this).attr("type"), $(this).attr("report_id"));

				//Quitamos el selected del resto de reportes
				$categories.find(format("li[report_id!={0}].reportsListItem", $(this).attr("report_id"))).each(function() {
					$(this).removeClass("selected");
				});
			});

		$home.find("li[report_id]")
			//reports
			.click(function() {
				const $this = $(this);
				// 1 - regreso a home en el tree, sin importar si estaba dentro o no de una categoria.
				onQuitTreeCategory();
				// 2 - selecciono en el tree la categoria que han selecccionado en el dashboard.
				const category_name = $this.attr("category_name");
				let selector = format("li[category={0}]", category_name);
				const $category = $categories.find(selector);
				onEnterTreeCategory($category);
				// 3 - seleccione en el tree el report de la categoria
				const report_id = $this.attr("report_id");
				selector = format("li[report_id={0}]", report_id);
				$categories.find(selector).click();
			});

		if (currentReport) {
			var $report = $categories.find(format("li[report_id={0}]", currentReport));
			if ($report.length) {
				$report.click();
			}
			else {
				currentReport = null;
				this.m_currentReport = null;
			}
		}
		self.__toggleHome(!currentReport);

		$("#waiting_for_list").hide();
	};

	/*!
	 * App method
	 */
	ReportsViewer.prototype.leave = $.noop;

	ReportsViewer.prototype.enterReport = function(report_type, report_id) {
		const self = this;

		self.cleanup_report_view();
		self.m_dashboard.block_right_panel({
			with_animation: true,
			with_message  : "<%=_T('Please wait ... Downloading report metadata..')%>"
		});

		self.m_currentReportTag = report_type + "/" + report_id;
		self.m_controller.loadReport(report_type, report_id).done(function(report) {
			report.update_filters(function() {
				self.load_report_view(report);
			});
		}).fail(function(errorMessage) {
			self.m_dashboard.unblock_right_panel();
			self.m_dashboard.show_message("messageError", errorMessage);
		});
	};

	ReportsViewer.prototype.setup_input_fields = function(report) {
		const $report_filters = $('#report_filters');
		let filters = report.getFilters();

		$.each(filters, function(filter_id, filter) {
			var $input_field;
			switch (filter.type) {
				case "boolean":
					$input_field = $('<input type="checkbox"></input>')
						.addClass('filterField')
						.prop('checked', true)
						.data('get_value', function($this) {
							return $this.prop('checked');
						});
					break;
				case "string":
					$input_field = $('<input></input>')
						.addClass('filterField');
					if (filter.options) {
						filter.options.sort();
						$input_field
							.autocomplete({
								source: filter.options,
								minLength: 0
							})
							.focus(function(){
								$(this).data("uiAutocomplete").search($(this).val());
							});
					}
					break;
				case "date":
					var date_format = "yy-mm-dd";
					var current_date = $.datepicker.formatDate(date_format, new Date());
					$input_field = $('<input></input>')
						.attr('id', 'dateFilter_' + filter_id)
						.addClass('filterField')
						.datepicker({ dateFormat: date_format, maxDate: 0 })
						.datepicker('setDate', current_date)
						.text(current_date);
					if (filter.time) {
						$input_field.data('get_value', function($this) {
							return $this.val() === "" ? "" : $this.val() + " " + filter.time;
						});
					}
					if (filter.before_than || filter.after_than) {
					// setup a custom validation function in order to check date order
						$input_field.data('custom_validator', function($this) {
							var validation_result = { success: true };

							var check_date_order = function($before, $after) {
								if ($before.datepicker('getDate') > $after.datepicker('getDate')) {
									validation_result = {
										success: false,
										error  : format("<%=_T('{0} must be greater than {1}')%>", _T($after.data('name')), _T($before.data('name')))
									};
								}
								return validation_result.success;
							};

							if (filter.before_than && !check_date_order($this, $('#dateFilter_' + filter.before_than)) ) {
								return validation_result;
							}
							if (filter.after_than && !check_date_order($('#dateFilter_' + filter.after_than), $this) ) {
								return validation_result;
							}

							return validation_result;
						});

						var bind_date_pickers = function($before, $after) {
							$before.datepicker("option", "onClose", function(selectedDate) {
								$after.datepicker( "option", "minDate", selectedDate );
							});
							$after.datepicker("option", "onClose", function(selectedDate) {
								$before.datepicker( "option", "maxDate", selectedDate );
							});
						};

						if (filter.after_than) {
							var $before = $('#dateFilter_' + filter.after_than);
							if ($before.length > 0) {
								bind_date_pickers($before, $input_field);
							}
						}
						if (filter.before_than) {
							var $after = $('#dateFilter_' + filter.before_than);
							if ($after.length > 0) {
								bind_date_pickers($input_field, $after);
							}
						}
					}
					break;
				case "enum":
					$input_field = $('<select></select>')
						.addClass('');
					if (!filter.required) {
						$input_field.append('<option value="">' + "<%=_T('All')%>" + '</option>');
					}
					_.each(filter.domain, function(option) {
						$input_field.append(
							$('<option></option>').attr('value', option.value)
								.text(option.label || option.value)
						);
						if (option.selected) {
							$input_field.val(option.value);
						}
					});
					break;
				default:
					throw "Don't know how to represent filter type " + filter.type;
			}

			$input_field.addClass('form-control')
				.data('filter_id', filter_id)
				.data('required', filter.required)
				.data('name', filter.name);

			var $field_label = $('<label></label>')
				.addClass('')
				.text(filter.label);

			var $field_container = $('<div></div>')
				.attr('title', filter.description || "")
				.addClass('col')
				.append($field_label)
				.append($input_field);

			if (filter.preset) { // It would be awesome to show them two but like a navigation toolbar
				$field_label.prop('disabled', true);
				$input_field.prop('disabled', true).val(filter.value);
				$field_container.hide();
			}

			var $last = $report_filters.children('.col:not(.fixedFilter):last');
			if ($last.length > 0) {
				$field_container.insertAfter($last);
			} else {
				$field_container.insertBefore($report_filters.children(':first'));
			}
		});
	};

	ReportsViewer.prototype.prepare_report = function(report) {
		const self = this;

		// initialize environment for execution
		report.prepare();

		var show_validation_error = function($field, message) {
			self.m_dashboard.show_message("messageError", message);
			$field.focus();
		};

		// and read all input fields
		var $input_fields = $('#report_filters').find('.form-control:not(.notQueryField)');
		for (var i = 0; i < $input_fields.length; ++i) {
			var $current_field = $($input_fields[i]);

			var custom_validator = $current_field.data('custom_validator');
			if  ( typeof(custom_validator) === "function" ) {
				var validation_result = custom_validator($current_field);
				if (!validation_result.success) {
					show_validation_error($current_field, validation_result.error);
					return false;
				}
			}

			var field_value;
			// in case the control does not have a val() property
			var get_value = $current_field.data('get_value');
			if  ( typeof(get_value) === "function" ) {
				field_value = get_value($current_field);
			} else {
				field_value = $current_field.val();
			}

			if ($current_field.data('required') && field_value === "") {
				show_validation_error($current_field, format("<%=_T('Field {0} is required')%>", $current_field.data('name')));
				return false;
			}

			if (field_value !== "") {
				// now the fields can be added to the report
				report.setFilterValue(
					$current_field.data('filter_id'), field_value
				);
			}
		}

		return true;
	};

	ReportsViewer.prototype.export_report = function(export_type)
	{
		if(!this.m_currentReport) {
			return;
		}

		const self = this;
		var report = this.m_currentReport;
		var $cmd_export_report = $('#cmd_export_report').prop('disabled', true);

		var restore_button = function() {
			$cmd_export_report
				.val("<%=_T('Export')%>")
				.prop('disabled', false)
				.removeClass('filterButtonProgress');
		};

		var on_finished = function(status_mesage) {
			$cmd_export_report.val(status_mesage);
			window.setTimeout(restore_button, 500);
		};

		this.m_dashboard.hide_message();
		$('#export_options_list').hide();

		report.backup_state();

		if (!this.prepare_report(report)) {
			restore_button();
			return report.restore_state();
		}

		this.m_controller.exportReport(report, export_type, function(success, errorThrown) {
			on_finished( success ? "<%=_T('Done!')%>" : "<%=_T('Error')%>");
			if (success) {
				self.m_dashboard.show_message("messageInfo", "<%=_T('Report download successfully started.')%>", undefined, "class_wrap_info_message" /*wrap_classes*/);
			} else {
				errorThrown = errorThrown || "";
				var errMsg = errorThrown.length > 0 ? errorThrown : "<%=_T('Failed to export report.')%>";
				self.m_dashboard.show_message("messageError", errMsg, undefined, "class_wrap_error_message" /*wrap_classes*/);
			}
		});

		report.restore_state();
	};

	ReportsViewer.prototype.execute_report = function() {
		if(!this.m_currentReport) {
			return;
		}

		const self = this;
		var report = self.m_currentReport;
		var $report_no_results   = $('#report_no_results');
		var $report_results_grid = $('#report_results_grid');
		var $grid_head           = $report_results_grid.children('thead');

		self.cleanup_execution_state();

		if (!self.prepare_report(report)) {
			return;
		}

		// called when the request starts
		var disable_controls = function(options) {
			if ( options.first_time ) {
				self.m_dashboard.block_right_panel({with_animation: true}); // block everything!
			} else if ( options.updating ) {
				$('#waiting_for_scroll').show();
			}
		};

		// called when the requests ends
		var enable_controls = function(options) {
			if ( options.first_time ) {
				if ( options.empty ) {
					$report_no_results.show();
				} else if (!options.error) {
					$report_results_grid.show();
				}
				self.m_dashboard.unblock_right_panel();
			} 

			if ( options.updating ) {
				$('#waiting_for_scroll').hide();
			}

			if ( options.done && options.empty && self.m_paginator ) {
				self.m_paginator.no_more_pages();
			}
		};

		// called when the requests is complete, must fill the grid with the received data
		var row_count = 0;
		var nestedGridHead; // for threaded reports
		var total_threaded_rows = 0;
		var rows_per_page = parseInt($("#reportsResultsSizeSelector").val()) || 15;
		var rows2Request = rows_per_page + 5;

		function onExecFailure(options, errorThrown) {
			self.m_dashboard.show_message("messageError", errorThrown);
			return enable_controls($.extend({ error:true }, options));
		}

		function onExecComplete(options, result) {
			var processedRows = 0;
			var colspan = result.headers.length; // colspan for nested columns
			var simpleDateFilter, simpleDateColumn;

			if (self.m_currentReport.is_single_day_filter()) {
				simpleDateColumn = self.m_currentReport.get_slice_date_column();
				simpleDateFilter = self.m_currentReport.get_single_day_date_filter();
			}
			else if (self.m_currentReport.is_single_year_filter()) {
				simpleDateColumn = self.m_currentReport.get_slice_date_column();
				simpleDateFilter = self.m_currentReport.get_single_year_date_filter();
			}

			var chartsEnabled = report.can_show_charts();
			var lineChartsData = chartsEnabled? report.make_line_charts_data(result.columnsMetadata || []) : null;
			$('#reportsChartsDisclaimer').toggle(!chartsEnabled && report.has_charts());

			function formatCell(cell) {
				if ( typeof(cell) === "string" ) {
					cell = inConcert.Utils.escapeAttrib(cell);
				}
				if ( typeof(cell) === "number" && Math.floor(cell) !== cell ) {
					cell = cell.toFixed(2);
				}
				return cell;
			}
			function formatCellContent(cell, metadata) {
				let name = metadata.name || "";
				switch(name) {
					case "transferDirection":
						let iconClass = "transferDirectionIcon ";
						switch(cell) {
							case "<%=_T('Origin')%>":
								iconClass += "originDirection";
								break;
							case "<%=_T('Destination')%>":
								iconClass += "destinationDirection";
								break;
							case "<%=_T('Current')%>":
								iconClass += "currentDirection";
								break;
						}
						cell = "<i class='" + iconClass + "'></i>" + cell;
						break;
				}
				return cell;
			}
			function getHeaderTypeClass(type) {
				switch(type) {
					case "number":
					case "default":
					case "Decimal":
						return "NumberHeader";
					case "string":
						return "StringHeader";
					case "Time":
						return "TimeHeader";
					case "Date":
					case "DateTime":
						return "DateHeader";
					case "percentage":
					case "Percentage":
						return "PercentageHeader";
					case "boolean":
						return "BooleanHeader";
					case "enum":
						return "EnumHeader";
					default:
						return "StringHeader";
				}
			}
			function getHeaderNameClass(name) {
				switch(name) {
					case "transferDirection":
						return "TransferDirectionHeader";
					default:
						return "";
				}
			}
			function getHeaderClass(metadata, defaultType) {
				let type = metadata.type || defaultType || "number";
				let name = metadata.name || "";
				return getHeaderTypeClass(type) + " " + getHeaderNameClass(name);
			}
			function getCellTypeClass(type) {
				switch(type) {
					case "number":
					case "default":
					case "Decimal":
						return "NumberCell";
					case "string":
						return "StringCell";
					case "Time":
						return "TimeCell";
					case "Date":
					case "DateTime":
						return "DateCell";
					case "percentage":
					case "Percentage":
						return "PercentageCell";
					case "boolean":
						return "BooleanCell";
					case "enum":
						return "EnumCell";
					default:
						return "StringCell";
				}
			}
			function getCellNameClass(name) {
				switch(name) {
					case "transferDirection":
						return "TransferDirectionCell";
					default:
						return "";
				}
			}
			function getCellClass(metadata, defaultType) {
				let type = metadata.type || defaultType || "number";
				let name = metadata.name || "";
				return getCellTypeClass(type) + " " + getCellNameClass(name);
			}
			function makeHeaderHtml(headers, head, columnsMetadata) {
				var i = 0;
				columnsMetadata = columnsMetadata || [];
				return _.reduce(headers, function(memo, header) {
					var metadata = i < columnsMetadata.length && columnsMetadata[i] || {};
					i++;

					return memo +
						'<th class="' + getHeaderClass(metadata) + '" ' + (metadata.description? ('title="' + metadata.description + '" ') : '') + '>' +
							header +
						'</th>';
				}, head === undefined ? '' : '<th>' + head + '</th>');
			}
			function makeRowHtml(cells, head, columnsMetadata) {
				report.add_line_charts_row(lineChartsData);
				var i = 0;
				columnsMetadata = columnsMetadata || [];
				return _.reduce(cells, function(memo, cell) {
					var metadata = i < columnsMetadata.length && columnsMetadata[i] || {};
					i++;

					var text = formatCell(cell, metadata);
					var content = formatCellContent(text, metadata);

					if (metadata.name) {
						if (metadata.name === simpleDateColumn) {
							text = text.replace(simpleDateFilter, "");
						}
						report.add_line_charts_value(lineChartsData, metadata.name, metadata.type, cell, text);
					}

					return memo +
						'<td title="' + text + '" class="' + getCellClass(metadata) + '" ' + '>' +
							content +
						'</td>';
				}, head === undefined ? '' : '<td>' + head + '</td>');
			}
			function makeRow(cells) {
				var rowContent = makeRowHtml(cells, ++row_count, result.columnsMetadata);
				if ( !report.threaded || _.isEmpty( result.threadingInfo.records[ processedRows ]) ) {
					processedRows++;
					return $('<tr>' + rowContent + '</tr>');
				}

				total_threaded_rows = total_threaded_rows + result.threadingInfo.records[ processedRows ].length;
				processedRows++;

				// TODO: Switch to a list or something
				return $(
					'<tr class="rowHeaderForNestedTable">' + rowContent + '</tr>' +
					'<tr class="rowForNestedTable">' +
						'<td></td>' +
						'<td class="nestedTableContainer" colspan="' + colspan + '">' +
							'<table border="0" cellspacing="0" cellpadding="0" class="nestedTable">' +
								nestedGridHead +
								'<tbody thread="true"></tbody>' +
							'</table>' +
						'</td>' +
					'</tr>'
				);
			}

			var count_main_items = result.rows.length;
			if ( count_main_items === 0 ) {
				enable_controls($.extend({ done:true, empty:true }, options));
				return;
			}

			/*
				A T E N C I O N:
					los valores de las variables "max_append_per_loop" y "pause_between_loop_ms"
					lo que hacen es:
						- procesar en batch de a 10 thread (max_append_per_loop)
						- inserta estos 10 thread (max_append_per_loop), y luego antes de continuar con el segundo batch
							ejecuta un timeout de 120 ms (pause_between_loop_ms)

				Estos 2 valores fueron probados y testeados para que el mensaje del waiting pueda ser visualizado correctamente en el navegador!!!
				y para que el navegador quede liberado entre loops y no pierda conexion.
			*/
			var max_append_per_loop = 10;
			var pause_between_loop_ms = 120;
			var total_added = 0;
			var pending_timemouts = 0;
			var waiting_dots = '';

			var update_waiting = function()
			{
				switch( waiting_dots.length ) {
					case 0:
						waiting_dots = '.';
					break;

					case 1:
						waiting_dots = '..';
					break;

					case 2:
						waiting_dots = '...';
					break;

					case 3:
						waiting_dots = '';
					break;
				}
				self.m_dashboard.block_right_panel({
					with_animation: true,
					with_message  : format( "<%=_T('Please wait ... loading {0} of {1} threaded conversations {2}')%>", total_added, total_threaded_rows, waiting_dots )
				});
			};

			var append_sub_rows = function( $parent_element, thread_rows, added )
			{
				pending_timemouts++;

				// timeout que contiene la funcion que realiza el loop por batch de los rows a agregar en la subgrilla (threads).
				window.setTimeout( function() 
				{
					var added_in_loop = 0;
					var html_thread_rows = '';
					var count_thread = thread_rows.length;
					// aca realizo el batch de a unos N rows ( max_append_per_loop )
					for ( var i = added; i < count_thread; i++ )
					{
						var thread_row = thread_rows[ i ];
						html_thread_rows = html_thread_rows + '<tr>' + makeRowHtml( thread_row, undefined, result.threadingInfo.columnsMetadata ) + '</tr>';
						added_in_loop++;
						added++;
						if ( added_in_loop === max_append_per_loop ) {
							break;
						}
					}
					total_added = total_added + added_in_loop;
					update_waiting(); // actualizo el waiting y seguidamente tengo que realizar otro timeout para que se pueda visualizar el mensaje minetras se actualiza el DOM con los rows

					// este es el timeout que se ejecuta para agregar los rows al DOM
					window.setTimeout( function() 
					{
						if ( html_thread_rows.length > 0 ){
							var $thread = $parent_element.find( 'tbody[thread="true"]' );
							$( html_thread_rows ).appendTo( $thread );
						}

						if ( added < count_thread ) {
							// si aun quedan subrows por cargar invoca la misma funcion recursiva
							append_sub_rows( $parent_element, thread_rows, added );
						}
						pending_timemouts--;

						if ( pending_timemouts === 0 ) {
							update_waiting();
							// este timeout tambien es utilizado para poder divisar correctamente el ultimo mensaje del waiting.
							window.setTimeout( function() {
								enable_controls($.extend({
									done: ( false )
								}, options));
							}, pause_between_loop_ms);
						}

					}, pause_between_loop_ms);

				}, pause_between_loop_ms);
			};

			let subgrid_records = result.threadingInfo ? result.threadingInfo.records : undefined;

			if ( options.first_time ) {
				// setup header before appending any row
				$grid_head.html( makeHeaderHtml( result.headers, '', result.columnsMetadata ) );

				if (report.threaded) {
					nestedGridHead = '<thead>' +
										makeHeaderHtml(result.threadingInfo.headers, undefined, result.threadingInfo.columnsMetadata) +
									'</thead>';
				}

				var main_items = _.map( result.rows, makeRow );
				self.m_paginator = $.Paginator.init({
					pages_container : $('#report_paginator'),
					data_container  : $report_results_grid.children('tbody'),
					previous        : "",
					next            : "",
					show_at_least   : 12,
					rows_per_page   : rows_per_page,
					items           : main_items, // initial items
					row_count       : result.length < rows2Request ? result.length : undefined,
					text_format     : "<%=_T('{page_row_start}-{page_row_end} of {many}')%>",
					translated_many : "<%=_T('many')%>",
					callback: function(page, need_data, on_done) {
						function _on_done() {
							self.show_line_charts(lineChartsData, page, rows_per_page);
							on_done();
							enable_controls({updating:true});
						}

						disable_controls({updating:true});
						if (!need_data) {
							window.setTimeout(function() {
								_on_done();
							}, 300);
							return;
						}
						
						self.m_controller.update(report, rows2Request)
							.done(_.partial(onExecComplete, {updating:true}))
							.fail(_.partial(onExecFailure,  {updating:true}))
							.always(_on_done);
					},
					on_append_one_row : function( $row, main_index ) {
						let lastpage_subgrid_rows = self.m_paginator ? self.m_paginator.__lastpage_subgrid_records : subgrid_records;
						if ( !report.threaded || _.isEmpty( lastpage_subgrid_rows[ main_index ] ) ) {
							if ( main_index === count_main_items - 1 ) {
								enable_controls($.extend({
									done: ( count_main_items < rows2Request )
								}, options));
							}
							return;
						}
						var added = 0;
						append_sub_rows( $row, lastpage_subgrid_rows[ main_index ], added );
					}
				});
				// Debo asignar al paginador, los rows de las subgrilla de la ultima pagina, para que en el callback "on_append_one_row" utilice los rows del scope que corresponde. #9047
				self.m_paginator.__lastpage_subgrid_records = subgrid_records;
			} else {
				// Debo asignar al paginador, los rows de las subgrilla de la ultima pagina, para que en el callback "on_append_one_row" utilice los rows del scope que corresponde. #9047
				self.m_paginator.__lastpage_subgrid_records = subgrid_records;

				$.each( result.rows, function( index, cells ) {
					var $row = makeRow( cells );
					self.m_paginator.append( $row, index );
				});
			}
		}

		var options = {first_time:true};
		disable_controls( options );
		self.m_controller.execute(report, rows2Request)
			.done(_.partial(onExecComplete, options))
			.fail(_.partial(onExecFailure, options));
	};

	ReportsViewer.prototype.show_line_charts = function(data, page, rowsPerPage)
	{
		const self = this;
		if (!data || !data.rowsCount) {
			return;
		}

		$('#reportsViewerContainer').addClass('withCharts');
		$('#reportsChartsContainer').show();
		inConcert.charts.cleanView("reportsCharts");
		$('#reportsLineChartsContainer').html('');
		$('#reportsLineChartsSelector').click();

		var first = (page - 1) * rowsPerPage;
		var last = page * rowsPerPage;
		var report = self.m_currentReport;
		$.each(report.lineCharts, function(index, chart) {
			self.__show_line_chart(index, chart.name, data.chartsIndicators[chart.name], 
				data.chartsRows[chart.name].slice(first, last));
		});
	};

	ReportsViewer.prototype.__show_line_chart = function(index, name, indicators, rows) 
	{
		var $container = $(format("".concat(
			"<div class='reportLineChartContainer' report-line-chart='{0}'>",
				"<div chart-title class='reportLineChartTitle'>",
					"{1}",
				"</div>",
				"<div chart-draw class='reportLineChartDraw'>",
				"</div>",
			"</div>"
		), index, name));
		$("#reportsLineChartsContainer").append($container);
		window.setTimeout(function() {
			inConcert.charts.multiline($container.find("[chart-draw]"), indicators, rows, {
				viewName : "reportsCharts",
				xAxis : false,
				yAxis : false,
				margin : {
					left : 0,
					right : 0, 
					top : 0, 
					bottom : 0 
				}
			});
		}, 50);
	};

	ReportsViewer.prototype.__showTotalCharts = function()
	{
		const self = this;
		if (self.m_totalCharts || !self.m_currentReport || !self.m_currentReport.has_total_charts()) {
			return;
		}
		var report = self.m_currentReport;
		
		function onExecFailure(errorThrown) {
			self.m_dashboard.show_message("messageError", errorThrown);
		}

		function onExecComplete(result) {
			var data = report.make_total_charts_data(result.columnsMetadata, result.rows[0]);
			$.each(report.totalCharts, function(index, chart) {
				self.__show_total_chart(index, chart.name, chart.type || "BARS", data[chart.name]);
			});
		}

		self.m_dashboard.block_right_panel({with_animation: true});
		self.m_controller.totals(report)
			.done(_.partial(onExecComplete))
			.fail(_.partial(onExecFailure))
			.always(function() {
				self.m_dashboard.unblock_right_panel();
				self.m_totalCharts = true;
			});
	};

	ReportsViewer.prototype.__show_total_chart = function(index, name, type, totals) 
	{
		var $container = $(format("".concat(
			"<div class='reportTotalChartContainer' report-total-chart='{0}'>",
				"<div chart-title class='reportTotalChartTitle'>",
					"{1}",
				"</div>",
				"<div chart-draw class='reportTotalChartDraw'>",
				"</div>",
			"</div>"
		), index, name));
		$("#reportsTotalChartsContainer").append($container);
		window.setTimeout(function() {
			var $draw = $container.find("[chart-draw]");
			switch(type) {
				case "BARS":
					$draw.addClass("reportsBarChart");
					inConcert.charts.bars($draw, totals, {
						viewName : "reportsTotalCharts",
						percentages : totals.length > 1,
						margin : {
							left : 0,
							right : 0,
							top : 0,
							bottom : 0
						}
					});
					break;
				case "PIE":
					$draw.addClass("reportsPieChart");
					inConcert.charts.pie($draw, totals, {
						viewName : "reportsTotalCharts",
						percentages : totals.length > 1,
						margin : {
							left : 0,
							right : 0,
							top : 0,
							bottom : 0
						}
					});
			}
		}, 50);
	};

	ReportsViewer.prototype.add_scheduler_report = function()
	{
		const self = this;

		if ( !inConcert.permissions.to_action( "reports", "save_schedule", "SAVE" ) ) {
			this.m_dashboard.show_message( "messageError", "<%=_T('Unable to reschedule: You dont have permission for this action.')%>");
			return;
		}

		if( !this.m_currentReport ) {
			return;
		}
		var report = this.m_currentReport;
		if ( !this.prepare_report( report ) ) {
			return;
		}

		var report_data = report.exec_data;
		if( !report_data ) {
			return;
		}

		// crea y retorna los datos con la estructura que necesita el ABM de schedulers
		var make_schedule_data = function()
		{
			var filter = report_data.filter || report_data.queryFields;
			if ( !filter ) {
				return false;
			}
			var from, to;

			if ( report.type === "summary" ) {
				from = filter.DateFrom;
				to   = filter.DateTo;
				delete filter.DateFrom;
				delete filter.DateTo;
			}
			else {
				from = filter.startDate;
				to   = filter.endDate;
				delete filter.startDate;
				delete filter.endDate;
			}

			return {
				ReportType : report.type,
				Report     : report.id,
				QueryRange : { type : "Range", from: from, to: to },
				Params     : filter
			};
		};

		var schedule = make_schedule_data();
		if( !schedule ) {
			return;
		}

		// me muevo a la app "report_design" y luego en un timeout me muevo dentro del menu a la configuracion de "reports"
		// y luego a la opcion de la navegacion que corresponde a "schedule"
		var mw = inConcert.app.getMainWindow();
		mw.jumpToApp( "report_design" );

		window.setTimeout( function()
		{
			// una vez que en el formulario se presiona "save" o "cancel" nos regresa nuevamente a la app "reports" (viewer)

			var config = {
				open_frm        : true,
				data            : schedule,
				callback_cancel : function() { mw.jumpToApp( "reports" ); },
				callback_save   : function( success )
				{
					mw.jumpToApp( "reports" );
					var type, message;
					if ( success ) {
						type    = "messageInfo";
						message = "<%=_T('The Report was successfully scheduled.')%>";
					}
					else {
						type    = "messageError";
						message = "<%=_T('Failed to scheduled Report.')%>";
					}
					self.m_dashboard.show_message( type, message );
				}
			};

			// aca abro la app que se va a configurar y en que vista voy a quedar posicionado
			var app_id   = "reports";
			var view_id  = "schedules";
			inConcert.report_design.open_app(app_id, view_id, config, true);
		}, 1000);
	};

	ReportsViewer.prototype.cleanup_report_view = function() {
		this.cleanup_execution_state();

		// reset title and filter
		$('#report_header_title').text('');
		$('#report_header_subtitle').text('');
		$('#report_filters').children('.col:not(.fixedFilter)').remove();
		$('#reportsChartsDisclaimer').hide();

		// cleanup table headers
		$('#report_results_grid').children('thead').html('');

		// miscelaneous cleanups in order to handle corner cases
		$('#cmd_exec_report').prop('disabled', false);
	};

	ReportsViewer.prototype.cleanup_execution_state = function() {
		this.m_dashboard.hide_message();

		if (this.m_paginator) {
			this.m_paginator = this.m_paginator.destroy();
		}
		$('#report_no_results').hide();
		$('#waiting_for_scroll').hide();
		$('#report_results_grid').hide();
		$('#report_results_grid').children('tbody').html('');
		
		// cleanup charts
		this.m_totalCharts = false;
		$('#reportsViewerContainer').removeClass('withCharts');
		$('#reportsChartsContainer').hide();
		inConcert.charts.cleanView("reportsCharts");
		inConcert.charts.cleanView("reportsTotalCharts");
		$('#reportsLineChartsContainer').html('');
		$('#reportsTotalChartsContainer').html('');
		$('#reportsLineChartsSelector').click();
	};

	ReportsViewer.prototype.load_report_view = function(report) {
		const self = this;

		if (self.m_currentReportTag !== report.get_type() + "/" + report.get_id()) {
			return;
		}

		self.m_currentReport = report;

		// setup title
		var report_title = report.get_name() !== "" ? report.get_name() : report.get_id();
		$('#report_header_title').text(report_title)
			.attr('tooltip', report.get_id())
			.inconcert_tooltip();
		$('#report_header_subtitle').text(report.get_description());

		//setup charts
		$('#reportsChartsDisclaimer').toggle(report.has_charts());

		// setup filters (aka input fields)
		self.setup_input_fields(report);

		// finally, the report is ready to use
		self.m_dashboard.unblock_right_panel();
	};

	inConcert.reports.dashboard.registerApp ({
		name       : app_name,
		permissions: "view",
		factory    : function(dashboard) {
			return new ReportsViewer(dashboard);
		}
	});

}());
