/*jshint quotmark:false*/
/*global $, inConcert*/

if (!this.inConcert) {
	this.inConcert = {};
}
if (!inConcert.snPosts) {
	inConcert.snPosts = {};
}
if (!inConcert.snPostsActions) {
	inConcert.snPostsActions = {};
}

(function () {
/*jshint validthis:true*/
"use strict";

	var actions = {
		PAUSE	: { 
			name			: "<%=_T('Pause')%>",
			description		: "<%=_T('Pause post attention. No new interactions will be created but open interactions could be attended.')%>",
			permission		: "sn_pause",
			method			: "pause_posts/",
			order			: 1
		},
		RESUME	: { 
			name			: "<%=_T('Resume')%>",
			description		: "<%=_T('Resume post attention.')%>",
			permission		: "sn_resume",
			method			: "resume_posts/",
			order			: 1
		},
		CLOSE	: { 
			name			: "<%=_T('Close')%>",
			description		: "<%=_T('Close post attention. New messages will be omitted.')%>",
			permission		: "sn_close",
			method			: "close_posts/",
			order			: 2
		},
		REOPEN	: { 
			name			: "<%=_T('Reopen')%>",
			description		: "<%=_T('Reopen post attention.')%>",
			permission		: "sn_reopen",
			method			: "reopen_posts/",
			order			: 1
		},
		EDIT	: { 
			name			: "<%=_T('Edit')%>",
			description		: "<%=_T('Edit post text.')%>",
			permission		: "sn_edit",
			method			: "edit_posts/",
			jsMethod		: "__editPopup",
			order			: 3
		},
		RESCHEDULE	: { 
			name			: "<%=_T('Reschedule')%>",
			description		: "<%=_T('Reschedule post date.')%>",
			permission		: "sn_edit",
			method			: "edit_posts/",
			jsMethod		: "__reschedulePopup",
			order			: 4
		},
		SET_ATTRIBUTES : { 
			name			: "<%=_T('Set Attributes')%>",
			description		: "<%=_T('Attributes can be used to apply interaction rules.')%>",
			permission		: "sn_edit",
			method			: "edit_posts_properties/",
			jsMethod		: "__setAttributesPopup",
			batchPopup		: "ShowSetAttributesPopup",
			order			: 5
		},
		DELETE	: { 
			name			: "<%=_T('Delete')%>",
			description		: "<%=_T('Delete social network post.')%>",
			permission		: "sn_delete",
			method			: "delete_posts/",
			order			: 6,
			confirmation	: {
				title		: "<%=_T('Confirm')%>",
				text		: "<%=_T('Do you want to delete post?')%>"
			}
		},
		PIN	: { 
			name			: "<%=_T('Pin')%>",
			description		: "<%=_T('Pin post. Pinned posts are not closed automatically.')%>",
			permission		: "sn_edit",
			method			: "pin_posts/",
			order			: 7
		},
		UNPIN	: { 
			name			: "<%=_T('Unpin')%>",
			description		: "<%=_T('Unpin post.')%>",
			permission		: "sn_edit",
			method			: "unpin_posts/",
			order			: 7
		}
	};
	if (!inConcert.snPostsActions.base) {
		inConcert.snPostsActions.base = actions;
		inConcert.snPostsActions.all = $.extend({}, actions);
	}

	// Defines a generic object with its common behaviour()
	// constructor
	function Post (rawData)
	{
		$.extend(true, this, rawData);
	}
	
	if (typeof inConcert.snPosts.base !== "function") {
		inConcert.snPosts.base = Post;
	}
	
	Post.prototype.GetCharactersLimit = function() {
		return null;
	};
	
	Post.prototype.GetAccountName = function() {
		return this.account;
	};
	
	Post.prototype.GetIdString = function() {
		return this.id;
	};
	
	Post.prototype.GetText = function() {
		return inConcert.app.getSNPostsManager().GetPostText(this.type, this.data);
	};
	
	Post.prototype.GetContent = function() {
		return inConcert.app.getSNPostsManager().GetPostContent(this.type, this.data);
	};
	
	Post.prototype.__getProperty = function(key) {
		return this.properties && this.properties[key] || null;
	};
	
	Post.prototype.GetAttributes = function() {
		return this.__getProperty("attributes") || "";
	};
	
	Post.prototype.GetId = function() {
		return this.account + ":" + this.id;
	};
	
	Post.prototype.IsActive = function() {
		return this.state === "ACTIVE";
	};
	
	Post.prototype.IsPaused = function() {
		return this.state === "PAUSED";
	};
	
	Post.prototype.IsOpen = function() {
		return this.IsActive() || this.IsPaused();
	};
	
	Post.prototype.IsClosed = function() {
		return this.state === "CLOSED";
	};
	
	Post.prototype.IsDeleted = function() {
		return this.state === "DELETED";
	};
	
	Post.prototype.Update = function(data) {
		$.extend( this, data );
	};
	
	Post.prototype.UpdateFromServer = function(callback) {
		inConcert.app.getSNPostsManager().UpdatePosts([this], callback);
	};
	
	Post.prototype.__getActions = function() {
		return this.type && inConcert.snPostsActions[this.type.toLowerCase()] || {};
	};
	
	Post.prototype.__getAction = function(actionName) {
		return this.__getActions()[actionName];
	};
	
	Post.prototype.CanRunAction = function(actionName) {
		var action = this.__getAction(actionName);
		return action && 
			inConcert.permissions.to_action("social_networks", action.permission, null, this.campaign) &&
			this["__can_" + actionName.toLowerCase()]()
			|| false;
	};
	
	Post.prototype.__editPopup = function(callback) {
		callback = callback || $.noop;
		var snManager = inConcert.app.getSNPostsManager();
		var action = this.__getAction("EDIT");
		if (!this.CanRunAction("EDIT")) {
			return callback(false, snManager.GetBatchActionErrorMessage(403, action));
		}
		snManager.ShowEditPopup(this);
		callback(true);	
	};
	
	Post.prototype.__reschedulePopup = function(callback) {
		callback = callback || $.noop;
		var snManager = inConcert.app.getSNPostsManager();
		var action = this.__getAction("EDIT");
		if (!this.CanRunAction("EDIT")) {
			return callback(false, snManager.GetBatchActionErrorMessage(403, action));
		}
		snManager.ShowReschedulePopup(this);
		callback(true);	
	};
	
	Post.prototype.__setAttributesPopup = function(callback) {
		callback = callback || $.noop;
		var snManager = inConcert.app.getSNPostsManager();
		var action = this.__getAction("SET_ATTRIBUTES");
		if (!this.CanRunAction("SET_ATTRIBUTES")) {
			return callback(false, snManager.GetBatchActionErrorMessage(403, action));
		}
		snManager.ShowSetAttributesPopup(this);
		callback(true);	
	};
	
	Post.prototype.RunAction = function(actionName, callback, data, run) {
		callback = callback || $.noop;
		var self = this;
		var action = self.__getAction(actionName);
		if (action && !run && action.jsMethod && self[action.jsMethod]) {
			return self[action.jsMethod](callback);
		}
		var snManager = inConcert.app.getSNPostsManager();
		snManager.RunBatchAction([this], actionName, function(status, result) {
			if (status) {
				if (result === "ABORTED" || result.successful_nodes.length) {
					callback(true);
				}
				else {
					callback(false, snManager.GetBatchActionErrorMessage(result.unsuccessful_nodes[0], action));
				}
			}
			else {
				callback(false, result);
			}
		}, data);
	};
	
	Post.prototype.HasActions = function() {
		var self = this;
		var has = false;
		$.each(self.__getActions(), function(actionName, action) {
			if (action && self.CanRunAction(actionName)) {
				has = true;
				return false;
			}
		});
		return has;
	};
	
	Post.prototype.GetActions = function() {
		var self = this;
		var actions = [];
		$.each(self.__getActions(), function(actionName, action) {
			if (action && self.CanRunAction(actionName)) {
				action.id = actionName;
				actions.push(action);
			}
		});
		actions.sort(function(a, b) {
			if (a.order === b.order) {
				return a.name.localeCompare(b.name);
			}
			return a.order - b.order;
		});
		return actions;
	};
	
	Post.prototype.GetNativeProperties = function() {
		let properties = [];
		if (this.IsDeleted()) {
			properties.push({ className : "deleted", tooltip : "<%=_T('Deleted')%><%=_T('.')%>" });
		}
		else if (this.IsClosed()) {
			properties.push({ className : "closed" });
		}
		else {
			if (this.IsPaused()) {
				properties.push({ className : "paused", tooltip : "<%=_T('Paused')%><%=_T('.')%>" });
			}
			if (this.IsPinned()) {
				properties.push({ className : "pinned", tooltip : "<%=_T('Pinned')%><%=_T('.')%>" });
			}
		}
		return properties;
	};
	
	Post.prototype.__can_edit = function() {
		return this.IsOpen();
	};
	
	Post.prototype.__can_reschedule = function() {
		return this.IsOpen() && this.IsScheduled();
	};
	
	Post.prototype.__can_pause = function() {
		return this.IsActive();
	};
	
	Post.prototype.__can_resume = function() {
		return this.IsPaused();
	};
	
	Post.prototype.__can_close = function() {
		return this.IsOpen();
	};
	
	Post.prototype.__can_reopen = function() {
		return this.IsClosed();
	};
	
	Post.prototype.__can_delete = function() {
		return !this.IsDeleted();
	};
	
	Post.prototype.__can_set_attributes = function() {
		return this.IsOpen();
	};
	
	Post.prototype.IsScheduled = function() {
		return this.GetScheduledDate()? 
			(new Date(this.publish_timestamp * 1000) > new Date()) : false;
	};
	
	Post.prototype.GetScheduledDate = function() {
		return null;
	};
	
	Post.prototype.IsPinned = function() {
		return this.properties && this.properties.pinned && true || false;
	};
	
	Post.prototype.__can_pin = function() {
		return this.IsOpen() && !this.IsPinned();
	};
	
	Post.prototype.__can_unpin = function() {
		return this.IsOpen() && this.IsPinned();
	};

}());