/*  jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
  strict:true, undef:true, unused:true, curly:true, browser:true, indent:false, maxerr:50, esversion: 11 */

/*global $, inConcert*/

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
/*jshint validthis:true*/
"use strict";

	var path = "/inconcert/apps/social_networks/";
	var format = inConcert.Utils.format;
	
	var POST_REQUEST_POOLING = 1000;
	var POST_REQUEST_TIMEOUT = 90000;
	
	var POST_ELASTIC_POOLING = 1000;
	var POST_ELASTIC_TIMEOUT = 10000;

	function PostsManager () {
		const self = this;
		this.dataProcessors = {};
		this.m_api = inConcert.app.getSocialNetworksAPI();
		this.fileUpload = new inConcert.FileUpload();
		$.each(inConcert.snPostsData, function(type, constructor) {
			if (type !== "base") {
				self.dataProcessors[type] = new constructor();
			}
		});
	}
	
	if (typeof inConcert.SNPostsManager !== "function") {
		inConcert.SNPostsManager = PostsManager;
	}

	PostsManager.prototype.__makeUrl = function(method, controller) {
		controller = controller || "posts";
		return inConcert.app.getBaseDomain() + path + controller + "/" + method;
	};
	
	PostsManager.prototype.BuildPost = function(data) {
		var postClass = inConcert.snPosts[data.type.toLowerCase()];
		if (!postClass) {
			return null;
		}
		return new postClass(data);
	};
	
	PostsManager.prototype.BuildPostStatistics = function(data) {
		var postClass = inConcert.snStatistics.post[data.post.type.toLowerCase()];
		if (!postClass) {
			throw "unsupported post: " + JSON.stringify(data);
		}
		return new postClass(data);
	};
	
	PostsManager.prototype.BuildAccountStatistics = function(data) {
		var statsClass = inConcert.snStatistics.account[data.type.toLowerCase()];
		if (!statsClass) {
			throw "unsupported account: " + JSON.stringify(data);
		}
		
		return new statsClass(data);
	};
	
	PostsManager.prototype.ShowCreatePopup = function(account) {
		var popupClass = inConcert.snPostsCreate[account.type.toLowerCase()];
		if (!popupClass) {
			throw "unsupported create post: " + JSON.stringify(account);
		}
		
		var popup = new popupClass(account);
		popup.Show();
	};
	
	PostsManager.prototype.ShowEditPopup = function(post) {
		var popupClass = inConcert.snPostsEdit[post.type.toLowerCase()];
		if (!popupClass) {
			throw "unsupported edit post: " + JSON.stringify(post);
		}
		
		var popup = new popupClass(post);
		popup.Show();
	};
	
	PostsManager.prototype.ShowReschedulePopup = function(post) {
		var popupClass = inConcert.snPostsReschedule[post.type.toLowerCase()];
		if (!popupClass) {
			throw "unsupported edit post: " + JSON.stringify(post);
		}
		
		var popup = new popupClass(post);
		popup.Show();
	};
	
	PostsManager.prototype.ShowSetAttributesPopup = function(post) {		
		var popup = new inConcert.snPostsSetAttributes(post);
		popup.Show();
	};
	
	PostsManager.prototype.CanCreateNewPost = function(campaignId, accountId) {
		if (!inConcert.permissions.to_action("social_networks", "sn_create", null, campaignId)) {
			return false;
		}
		var campaign = inConcert.app.getCampaign(campaignId);
		if (!campaign) {
			return false;
		}
		var account = campaign.getAccount(accountId);
		if (!account) {
			return false;
		}
		return account.state && account.state !== "0";
	};
	
	/**
	 * Search posts by conditions
	 *
	 * @public
	 *
	 * @param  {Object}			conditions
	 *		   {string}			conditions.view				- ACTIVE|SCHEDULED|ARCHIVE		required
	 *		   {object}			conditions.query											required
	 *		   {string|array}	conditions.query.type		- SN Post type					optional
	 *		   {string|array}	conditions.query.campaign									optional
	 *		   {string|array}	conditions.query.account									optional
	 *		   {string|array}	conditions.query.page										optional
	 *		   {string|array}	conditions.query.created_by_user							optional
	 *		   {string|array}	conditions.query.content	- Post text						optional
	 *		   {array}			conditions.sort				- Sort conditions				optional	Sample: [ { publish_timestamp : { order : "desc" } } ]
	 *		   {bool}			conditions.exact			- Find by text exactly			optional
	 *		   {bool}			conditions.highlight		- Return highlights				optional
	 *		   {integer}		conditions.rows				- Number of rows to return		optional
	 *		   {integer}		conditions.from				- Index of first row 			optional
	 *
	 * @param  {Function}		callback					- Callback function
	 */

	PostsManager.prototype.SearchPosts = function(conditions, callback = $.noop) {
		const self = this;

		return this.m_api.searchPosts(conditions, function(success, response) {
			if (!success) {
				callback(false, response);
				return;
			}
			let posts = [];
			$.each(response.posts || [], function(_, post) {
				posts.push(self.BuildPost(post));
			});
			response.posts = posts;
			callback(response);
		});
	};

	// PostsManager.prototype.SearchPosts = function(conditions, callback) {
	// 	callback = callback || $.noop;
	// 	var self = this;
	// 	return inConcert.invoke({
	// 		url: this.__makeUrl("search_posts/"),
	// 		type: "POST",
	// 		data: JSON.stringify(conditions),
	// 		dataType: "json",
	// 		contentType: "application/json",
	// 		success: function(response) {
	// 			var posts = [];
	// 			$.each(response.posts || [], function(_, postData) {
	// 				var post = self.BuildPost(postData);
	// 				if (post) {
	// 					posts.push(post);
	// 				}
	// 			});
	// 			response.posts = posts;
	// 			callback(response);
	// 		},
	// 		error: function(undefined, error) {
	// 			callback(false, error);
	// 		}
	// 	});
	// };
	
	// PostsManager.prototype.GetCampaignPosts = function(campaignId, callback) {
	// 	this.SearchPosts({
	// 		view : "ACTIVE_SCHEDULED",
	// 		from : 0,
	// 		rows : 500,
	// 		query : {
	// 			campaign : campaignId
	// 		},
	// 		sort : [{
	// 			publish_timestamp: { 
	// 				order: "desc" 
	// 			}
	// 		}]
	// 	}, function(response) {
	// 		callback(response && response.posts || []);
	// 	});
	// };

	PostsManager.prototype.GetBatchActionErrorMessage = function(result, action) {
		if (result.status === 403) {
			return format("<%=_T('You have no permissions to {0}.')%>", action? action.name : "");
		}
		result.result = result.result || {};
		if (result.result.reason === "interactions open") {
			return "<%=_T('Post has open interactions.')%>";
		}
		if (result.result.result === "OPEN_LIMIT") {
			switch(action.id) {
				case "REOPEN":
					return "<%=_T('Open posts limit exceeded.')%>";
			}
		}
		return "<%=_T('Server error running action.')%>";
	};
	
	/**
	 * Run batch action
	 *
	 * @public
	 *
	 * @param  {Array<Post>}	posts						- Posts to apply action
	 * @param  {string}			actionName					- Name of action to run
	 * @param  {Function}		callback					- Callback function
	 * @param  {object}			data						- Extra data to send in batch action
	 * @param  {bool}		fromList						- Indicates if action is run from list
	 */	
	PostsManager.prototype.RunBatchAction = function(posts, actionName, callback, data, fromList) {
		callback = callback || $.noop;
		data = data || {};
		var self = this;
		var batch = [];
		var error = null;
		var action = inConcert.snPostsActions.all[actionName];
		if (!action) {
			callback(false, "<%=_T('Unexpected error running action.')%>");
			return;
		}
		$.each(posts, function(_, post) {
			if (!post.CanRunAction(actionName)) {
				callback(false, format("<%=_T('You have no permissions to {0}.')%>", action? action.name : actionName));
				error = true;
				return false;
			}
			var postData = {
				proxyType		: post.type,
				campaignAtVCC	: post.campaign,
				account			: post.account,
				postId			: post.id
			};
			$.each(data, function(k, v) {
				postData[k] = v;
			});
			batch.push(postData);
		});
		if (error) {
			return;
		}
		
		if (fromList && action.batchPopup) {
			self[action.batchPopup](posts);
			callback(true, "ABORTED");
			return;
		}
		
		function onFinished(status, result) {
			self.UpdatePosts(posts, function() {
				callback(status, result);
			});
		}
		
		function runAction() {
			inConcert.invoke({
				url: self.__makeUrl(action.method),
				type: "POST",
				data: JSON.stringify(batch),
				dataType: "json",
				contentType: "application/json",
				success: function(response) {
					onFinished(true, response);
				},
				error: function(/*_, error*/) {
					onFinished(false, "<%=_T('Server error running action.')%>");
				}
			});
		}
		
		if (action.confirmation) {
			$.inconcertPopup.Show(
				action.confirmation.title,
				action.confirmation.text,
				$.inconcertPopup.ButtonSets.BS_YESNO,
				function(response) {
					if (response.toLowerCase() !== "yes") {
						return callback(true, "ABORTED");
					}
					runAction();
				},
				undefined,
				{
					popup_id        : "conf_inconcertPopup",
					container_class : "confirmationPopUp",
					overlay_class   : "confirmationOverlay"
				}
			);
		}
		else {
			runAction();
		}
	};
	
	PostsManager.prototype.__getPostRequestError = function(actionName, request) {
		if (!request.status) {
			return "<%=_T('Unexpected error.')%>";
		}
		switch(actionName) {
			case "CREATE":
				switch(request.status) {
					case "TIMEOUT":
						return "<%=_T('Timeout creating post.')%>";
					default:
						return "<%=_T('Error creating post.')%>";
				}
		}
		return "<%=_T('Unexpected error.')%>";
	};
 	
	PostsManager.prototype.__processPostRequest = function(requestId, callback, time) {
		callback = callback || $.noop;
		time = time || 0;
		var self = this;
		if (time > POST_REQUEST_TIMEOUT) {
			return callback(false, self.__getPostRequestError("CREATE", { status : "TIMEOUT" }));
		}
		time += POST_REQUEST_POOLING;
		inConcert.invoke({
			url: this.__makeUrl("get_post_request/" + encodeURIComponent(requestId)),
			type: "GET",
			dataType: "json",
			contentType: "application/json",
			success: function(response) {
				var request = response.request || {};
				switch(request.status) {
					case "OK":
						return callback(true, request.message);
					case "PENDING":
						return window.setTimeout(function() {
							self.__processPostRequest(requestId, callback, time);
						}, POST_REQUEST_POOLING);
					default:
						return callback(false, self.__getPostRequestError("CREATE", request));
				}
			},
			error: function(/*_, error*/) {
				callback(false, self.__getPostRequestError("CREATE", {}));
			}
		});
	};
 	
	PostsManager.prototype.__existsInElastic = function(postId, callback, time) {
		callback = callback || $.noop;
		time = time || 0;
		var self = this;
		if (time > POST_ELASTIC_TIMEOUT) {
			return callback(false);
		}
		time += POST_ELASTIC_POOLING;
		inConcert.invoke({
			url: this.__makeUrl("exists_in_elastic/" + encodeURIComponent(postId)),
			type: "GET",
			dataType: "json",
			contentType: "application/json",
			success: function(response) {
				if (response.exists) {
					return callback(true);
				}
				window.setTimeout(function() {
					self.__existsInElastic(postId, callback, time);
				}, POST_REQUEST_POOLING);
			},
			error: function(/*_, error*/) {
				callback(false);
			}
		});
	};
	
	PostsManager.prototype.MakeImageUploadUrl = function(account) {
		return inConcert.app.getBaseDomain() + path + 
			format("image_upload/?type={0}&campaign={1}&account={2}&page={3}" ,
				encodeURIComponent(account.type),
				encodeURIComponent(account.campaign),
				encodeURIComponent(account.account),
				encodeURIComponent(account.page));
	};
	
	PostsManager.prototype.__createPostFromImage = function(data, callback) {
		var url = this.__makeUrl("post_image/");
		var file = data.post.file;
		data.post.file = {
			name : file.name,
			type : file.type,
			extension : inConcert.Utils.getFileExtensionFromMimeType(file.type)
		};
		this.fileUpload.uploadImage(url, file, data, function(status, data) {
			if (status) {
				callback(true, JSON.parse(data));
			}
			else {
				callback(false, "<%=_T('Error uploading image.')%>");
			}
		});
	};
	
	PostsManager.prototype.CreatePost = function(account, data, isScheduled, callback) {
		callback = callback || $.noop;
		var self = this;
		if (!self.CanCreateNewPost(account.campaign, account.account)) {
			return callback(false, "<%=_T('You have no permissions to create posts.')%>");
		}
		var postData = {
			proxyType : account.type,
			campaignAtVCC : account.campaign,
			account : account.account,
			page : account.page || "",
			post : data
		};
		
		function processResult(status, data) {
			if (status && data) {
				self.__existsInElastic(data, function() {
					inConcert.social_networks.MoveToCategory("ACTIVE", account, function() {
						inConcert.social_networks.RefreshList(data);
						callback(status, data);
					});
				});
			}
			else {
				callback(status, data);
			}
		}
		
		function processResponse(status, response) {
			if (status) {
				if (response.requestId) {
					return self.__processPostRequest(response.requestId, function(status, data) {
						if (status) {
							data = account.account + ":" + data;
						}
						processResult(status, data);
					});
				}
				processResult(true, response.postId);
			}
			else {
				processResult(false, response);
			}
		}
		
		if (data.file) {
			return this.__createPostFromImage(postData, processResponse);
		}
		
		return inConcert.invoke({
			url: this.__makeUrl("new_post/"),
			type: "POST",
			data: JSON.stringify(postData),
			dataType: "json",
			contentType: "application/json",
			timeout: 300000, // 5 minutes
			success: function(response) {
				processResponse(true, response);
			},
			error: function(_, error) {
				var message = "<%=_T('Server error running action.')%>";
				switch(error && error.result || "") {
					case "OPEN_LIMIT":
						message = "<%=_T('Open posts limit exceeded.')%>";
						break;
					case "FAIL_DOWN_FILE_URL":
						message = "<%=_T('Error getting file from url.')%>";
						break;
				}
				processResult(false, message);
			}
		});
	};
	
	PostsManager.prototype.UpdatePosts = function(posts, callback) {
		callback = callback || $.noop;
		var ids = [];
		$.each(posts, function(_, post) {
			ids.push(post.GetId());
		});
		return inConcert.invoke({
			url: this.__makeUrl("get_posts/"),
			type: "POST",
			data: JSON.stringify(ids),
			dataType: "json",
			contentType: "application/json",
			success: function(response) {
				$.each(posts, function(_, post) {
					var data = response[post.GetId()];
					if (data) {
						post.Update(data);
					}
				});
				//update posts list
				inConcert.social_networks.UpdatePosts();
				callback(true);
			},
			error: function(_, error) {
				//update posts list
				inConcert.social_networks.UpdatePosts();
				callback(false, error);
			}
		});
	};
	
	PostsManager.prototype.GetInstagramPost = function(campaign, account, permalink, callback) {
		callback = callback || $.noop;
		return inConcert.invoke({
			url: this.__makeUrl("get_instagram_post/" + encodeURIComponent(permalink)),
			type: "POST",
			data: JSON.stringify({
				campaignAtVCC : campaign,
				account : account,
				permalink : permalink
			}),
			dataType: "json",
			contentType: "application/json",
			success: function(response) {
				callback(response);
			},
			error: function(_, error) {
				callback(error);
			}
		});
	};
	
	PostsManager.prototype.SearchTweets = function(campaign, account, conditions, callback) {
		callback = callback || $.noop;
		return inConcert.invoke({
			url: this.__makeUrl("search_tweets/"),
			type: "POST",
			data: JSON.stringify({
				campaignAtVCC : campaign,
				account : account,
				conditions : conditions
			}),
			dataType: "json",
			contentType: "application/json",
			success: function(response) {
				callback(true, response.tweets && JSON.parse(response.tweets) || [], response.limit, response.reset_time);
			},
			error: function(_, error) {
				callback(false, error);
			}
		});
	};
	
	PostsManager.prototype.RunBatchActionFromSearch = function(data, tweets, action, callback) {
		callback = callback || $.noop;
		var self = this;
		if (!self.HasPermission(data.campaignAtVCC, "sn_twitter_search")) {
			return callback(false, format("<%=_T('You have no permissions to {0}.')%>", action.name));
		}
		var batchData = [];
		$.each(tweets, function(_, tweet) {
			var tweetData = {};
			$.each(data, function(k, v) {
				tweetData[k] = v;
			});
			tweetData.message = tweet;
			tweetData.postId = tweet.id_str;
			batchData.push(tweetData);
		});
		inConcert.invoke({
			url: self.__makeUrl(action.method),
			type: "POST",
			data: JSON.stringify(batchData),
			dataType: "json",
			contentType: "application/json",
			success: function(response) {
				callback(true, response);
			},
			error: function(/*_, error*/) {
				callback(false, "<%=_T('Server error running action.')%>");
			}
		});
	};
	
	PostsManager.prototype.RunActionFromSearch = function(data, tweet, action, callback) {
		callback = callback || $.noop;
		var self = this;
		this.RunBatchActionFromSearch(data, [tweet], action, function(status, result) {
			if (status) {
				if (result === "ABORTED" || result.successful_nodes.length) {
					callback(true);
				}
				else {
					callback(false, self.GetBatchActionErrorMessage(result.unsuccessful_nodes[0], action));
				}
			}
			else {
				callback(false, result);
			}
		});
	};
	
	PostsManager.prototype.HasPermission = function(campaign, permission) {
		if (campaign) {
			return inConcert.permissions.to_action("social_networks", permission, null, campaign);
		}
		return inConcert.permissions.to_action_on_any_campaign("social_networks", permission);
	};
	
	PostsManager.prototype.GetAccount = function(campaignId, accountId, pageId) {
		var campaign = inConcert.app.getCampaign(campaignId);
		if (!campaign) {
			return null;
		}
		var account = campaign.getAccount(accountId);
		if (!account) {
			return null;
		}
		let pages, pageName;
		switch(account.proxyType) {
			case "FACEBOOK":
				pages = account.pagesNames;
				if (typeof pages === "string") {
					pages = JSON.parse(pages);
				}
				pageName = pages[pageId];
				if (!pageName) {
					return null;
				}
				return { 
					name		: pageName,
					className	: "accountItemFacebook",
					type		: account.proxyType,
					campaign	: campaignId,
					account		: account.id,
					page		: pageId
				};
			case "INSTAGRAM":
				pages = account.pagesNames;
				if (typeof pages === "string") {
					pages = JSON.parse(pages);
				}
				pageName = pages[pageId];
				if (!pageName) {
					return null;
				}
				return { 
					name		: pageName,
					className	: "accountItemInstagram",
					type		: account.proxyType,
					campaign	: campaignId,
					account		: account.id,
					page		: pageId
				};
			case "TWITTER":
				return {
					name		: account.screen_name, 
					className	: "accountItemTwitter",
					type		: account.proxyType,
					campaign	: campaignId,
					account		: account.id,
					page		: ""
				};
			case "YOUTUBE":
				return {
					name		: account.screen_name, 
					className	: "accountItemYoutube",
					type		: account.proxyType,
					campaign	: campaignId,
					account		: account.id,
					page		: ""
				};
		}
		return null;
	};
	
	PostsManager.prototype.GetAccountName = function(campaignId, accountId, pageId) {
		var account = this.GetAccount(campaignId, accountId, pageId);
		return account? account.name : "";
	};
	
	PostsManager.prototype.__getCampaignAccounts = function(campaign, permission) {
		var self = this;
		var accounts = [];
		if (permission && !self.HasPermission(campaign.Id, permission)) {
			return accounts;
		}
		$.each(campaign.accounts, function(_, account) {
			let pages;
			switch(account.proxyType) {
				case "FACEBOOK":
					pages = account.pagesNames;
					if (typeof pages === "string") {
						pages = JSON.parse(pages);
					}
					$.each(pages, function(pageId, pageName) {
						accounts.push({ 
							name		: pageName,
							className	: "accountItemFacebook",
							type		: account.proxyType,
							account		: account.id,
							page		: pageId
						});
					});
					break;
				case "INSTAGRAM":
					pages = account.pagesNames;
					if (typeof pages === "string") {
						pages = JSON.parse(pages);
					}
					$.each(pages, function(pageId, pageName) {
						accounts.push({ 
							name		: pageName,
							className	: "accountItemInstagram",
							type		: account.proxyType,
							account		: account.id,
							page		: pageId
						});
					});
					break;
				case "TWITTER":
					accounts.push({ 
						name		: account.screen_name, 
						className	: "accountItemTwitter",
						type		: account.proxyType,
						account		: account.id,
						page		: ""
					});
					break;
				case "YOUTUBE":
					accounts.push({ 
						name		: account.screen_name, 
						className	: "accountItemYoutube",
						type		: account.proxyType,
						account		: account.id,
						page		: ""
					});
					break;
			}
		});
		return accounts;
	};
	
	PostsManager.prototype.GetAccounts = function(sorted, permission) {
		var self = this;
		var accounts = [];
		var campaigns = inConcert.app.getCampaigns();
		$.each(campaigns, function(_, campaign) {
			var campaignAccounts = self.__getCampaignAccounts(campaign, permission);
			$.each(campaignAccounts, function(_, account) {
				account.campaign = campaign.Id;
				accounts.push(account);
			});
		});
		if (sorted) {
			accounts.sort(function(a, b) {
				if (a.type !== b.type) {
					return a.type.localeCompare(b.type);
				}
				return a.name.localeCompare(b.name);
			});
		}
		return accounts;
	};
	
	PostsManager.prototype.GetDataProcessor = function(type) {
		var processor = this.dataProcessors[type.toLowerCase()];
		if (!processor) {
			throw "unsupported type: " + type;
		}
		return processor;
	};
	
	PostsManager.prototype.GetPostText = function(type, data) {
		return this.GetDataProcessor(type).GetText(data);
	};
	
	PostsManager.prototype.GetPostContent = function(type, data) {
		return this.GetDataProcessor(type).GetContent(data);
	};
	
	PostsManager.prototype.LoadConfiguration = function(callback) {
		callback = callback || $.noop;
		var self = this;
		
		var processResult = function(configuration) {
			self.configuration = !$.isEmptyObject(configuration)? configuration : {};
			callback();
		};
		
		return inConcert.invoke({
			url: this.__makeUrl("get/", "configuration"),
			type: "GET",
			dataType: "json",
			contentType: "application/json",
			success: function(response) {
				processResult(response);
			},
			error: function(/*_, error*/) {
				processResult();
			}
		});
	};
	
	PostsManager.prototype.GetConfiguration = function() {
		if (!this.configuration) {
			throw "configuration not loaded";
		}
		return this.configuration;
	};
	
	PostsManager.prototype.UpdateConfiguration = function(data, callback) {
		callback = callback || $.noop;
		var self = this;
		
		if (!self.configuration) {
			throw "configuration not loaded";
		}
		$.each(data || {}, function(k, v) {
			self.configuration[k] = v;
		});
		
		return inConcert.invoke({
			url: self.__makeUrl("save/", "configuration"),
			type: "POST",
			data : JSON.stringify({ configuration : self.configuration }),
			dataType: "json",
			contentType: "application/json",
			success: function() {
				callback(true);
			},
			error: function(/*_, error*/) {
				callback(false);
			}
		});
	};

}());