/*jshint quotmark:false*/
/*global $, inConcert, _, Mustache*/

if (!this.inConcert) {
	this.inConcert = {};
}

if (!inConcert.snPostsEdit) {
	inConcert.snPostsEdit = {};
}

(function () {
/*jshint validthis:true*/
"use strict";

	var templates = inConcert.templates.social_networks;

	function SetAttributesPopup (post)
	{
		if ($.isArray(post) && post.length === 1) {
			post = post[0];
		}
		this.post = post;
	}
	
	if (typeof inConcert.snPostsSetAttributes !== "function") {
		inConcert.snPostsSetAttributes = SetAttributesPopup;
	}
	
	SetAttributesPopup.prototype.__getClassName = function() {
		var type = "";
		if ($.isArray(this.post)) {
			$.each(this.post, function(_, post) {
				if (type === "") {
					type = post.type;
				}
				else if (type !== post.type) {
					type = "";
					return false;
				}
			});
		}
		else {
			type = this.post.type;
		}
		switch(type) {
			case "FACEBOOK":
				return "fbPost";
			case "INSTAGRAM":
				return "instagramPost";
			case "YOUTUBE":
				return "youtubePost";
			case "TWITTER":
				return "twPost";
			default:
				return "multiplePost";
		}
	};
	
	SetAttributesPopup.prototype.Show = function(callback) {
		callback = callback || $.noop;
		var self = this;
		self.Close(function() {
			self.__show(callback);
		});
	};
	
	SetAttributesPopup.prototype.Close = function(callback) {
		callback = callback || $.noop;
		var self = this;
		if (self.popup && self.popup.isVisible()) {
			self.popup.finalize(function() {
				self.popup = null;
				$("#snSetAttributesPopup").remove();
				callback();
			});
		}
		else {
			$("#snSetAttributesPopup").remove();
			callback();
		}
	};
	
	SetAttributesPopup.prototype.__show = function(callback) {
		callback = callback || $.noop;
		var self = this;
		var $menu = $(Mustache.render(templates.set_attributes_popup_container, {
			className : self.__getClassName()
		}));
		$menu.append(self.__make());
		$("body").append($menu);
		self.popup = new inConcert.PopupWindow();
		self.popup.show({
			ownerId       : "snSetAttributesPopup",
			eventName     : "snSetAttributesPopup",
			canReposition : true,
			closeOnFocusLost : false,
			overlay		  : true,
			container     : $menu,
			callbacks: {
				onOpen: function() {
					$("#snSetAttributesPopup").show();
				},
				onClose: function( callback ) {
					callback = callback || $.noop;
					if (self.popup) {
						self.popup.hide(function() {
							callback();
						});
					}
					else {
						callback();
					}
				}
			}
		});
		callback();
	};
	
	SetAttributesPopup.prototype.__make = function() {
		var self = this;
		var sn = inConcert.app.getSNPostsManager();
		var $body = $(Mustache.render(templates.set_attributes_popup, {
			name : $.isArray(this.post)? 
				"<%=_T('multiple posts')%>" : 
				sn.GetAccountName(self.post.campaign, self.post.account, self.post.page)
		}));
		$body.find("#snPopupSetAttributeButtonClose")
		.off("click")
		.on("click", function() {
			self.Close();
		});
		//attributes
		$body.find("[post_attributes]")
			.val(self.__getOriginalText())
			.change(function() {
				self.__onTextChange();
			});
		//cancel
		$body.find("[cancel]").click(function() {
			self.Close();
		});
		//update
		$body.find("[update]").click(function() {
			const $buttonParent = $("#snButtonAction");
			self.Update();
			$buttonParent.prop("disabled", true);
		});
		
		return $body;
	};
	
	SetAttributesPopup.prototype.__getText = function() {
		return $("#snSetAttributesPopup").find("[post_attributes]").val() || "";
	};
	
	SetAttributesPopup.prototype.__getOriginalText = function() {
		return !$.isArray(this.post) && this.post.GetAttributes() || "";
	};
	
	SetAttributesPopup.prototype.__onTextChange = _.throttle(function() {
		
	}, 100);
	
	SetAttributesPopup.prototype.Message = function(text, type) {
		var $message = $("#snSetAttributesPopup").find("[message_container]");
		var $text = $message.find("[message_text]");
		$text
			.attr("class", (type || "info") + "Message")
			.html(text);
		$message.toggle(!!text);
	};
	
	SetAttributesPopup.prototype.Loading = function(loading) {
		$("#snSetAttributesPopup").find("[loading]").toggle(!!loading);
	};
	
	SetAttributesPopup.prototype.__makeData = function() {
		return {
			attributes : this.__getText()
		};
	};
	
	SetAttributesPopup.prototype.Update = function() {
		var self = this;
		if (self.processing) {
			return;
		}
		self.Message();
		self.processing = true;
		self.Loading(true);
		$("#snSetAttributesPopup").find("button").prop("disabled", true);
		
		var data = {
			properties : self.__makeData(),
			action : "SET_ATTRIBUTES"
		};
		
		function processResult(status, message) {
			self.processing = false;
			self.Loading(false);
			$("#snSetAttributesPopup").find("button").prop("disabled", false);
			if (status) {
				return self.Close();
			}
			self.Message(message, "error");
		}
		
		if ($.isArray(self.post)) {
			var snManager = inConcert.app.getSNPostsManager();
			snManager.RunBatchAction(self.post, "SET_ATTRIBUTES", processResult, data);
		}
		else {
			self.post.RunAction("SET_ATTRIBUTES", processResult, data, true);
		}
	};

}());