/*jshint quotmark:false*/
/*global $, inConcert*/

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
/*jshint validthis:true*/
"use strict";

	var format = inConcert.Utils.format;

	var actions = $.extend({}, inConcert.snPostsActions.base, {
		HIDE	: { 
			name			: "<%=_T('Hide')%>",
			description		: "<%=_T('Hide video from youtube channel playlists.')%>",
			permission		: "sn_hide",
			method			: "hide_posts/",
			order			: 6
		},
		PRIVATE	: { 
			name			: "<%=_T('Private')%>",
			description		: "<%=_T('Set video as private.')%>",
			permission		: "sn_hide",
			method			: "private_posts/",
			order			: 6
		},
		SHOW	: { 
			name			: "<%=_T('Show')%>",
			description		: "<%=_T('Show video in youtube channel.')%>",
			permission		: "sn_hide",
			method			: "show_posts/",
			order			: 6
		},
		/*DISABLE_COMMENTS	: { 
			name			: "<%=_T('Disable Comments')%>",
			description		: "<%=_T('Disable video comments.')%>",
			permission		: "sn_hide",
			method			: "private_posts/",
			order			: 7
		},
		ENABLE_COMMENTS	: { 
			name			: "<%=_T('Enable Comments')%>",
			description		: "<%=_T('Enable video comments.')%>",
			permission		: "sn_hide",
			method			: "show_posts/",
			order			: 7
		},*/
		RESCHEDULE	: null
	});
	
	if (!inConcert.snPostsActions.youtube) {
		inConcert.snPostsActions.youtube = actions;
		$.each(actions, function(actionId, action) {
			inConcert.snPostsActions.all[actionId] = inConcert.snPostsActions.all[actionId] || action;
		});
	}

	function YoutubePost (rawData)
	{
		YoutubePost.uber.constructor.call(this, rawData);
	}

	inConcert.Utils.inherit(YoutubePost, inConcert.snPosts.base);
	if (typeof inConcert.snPosts.youtube !== "function") {
		inConcert.snPosts.youtube = YoutubePost;
	}
	
	YoutubePost.prototype.__getScreenName = function() {
		var campaign = inConcert.app.getCampaign(this.campaign);
		if (campaign) {
			var account  = campaign.getAccount(this.account);
			if (account && account.screen_name) {
				return account.screen_name;
			}
		}
		return inConcert.app.getSNPostsManager().GetDataProcessor("YOUTUBE").GetScreenName(this.data);
	};
	
	YoutubePost.prototype.GetAccountName = function() {
		return this.__getScreenName() || YoutubePost.uber.GetAccountName.call(this);
	};
	
	YoutubePost.prototype.GetNativeProperties = function() {
		let properties = YoutubePost.uber.GetNativeProperties.call(this);
		if (!this.IsDeleted()) {
			if (this.IsLive()) {
				properties.push({ className : "liveVideo", tooltip : "<%=_T('Live video')%><%=_T('.')%>" });
			}
			if (this.IsHidden()) {
				properties.push({ className : "hidden", tooltip : "<%=_T('Hidden video')%><%=_T('.')%>" });
			}
			if (this.IsPrivate()) {
				properties.push({ className : "private", tooltip : "<%=_T('Private video')%><%=_T('.')%>" });
			}
			if (this.AreCommentsDisabled()) {
				properties.push({ className : "commentsDisabled", tooltip : "<%=_T('Comments disabled')%><%=_T('.')%>" });
			}
		}
		return properties;
	};
	
	YoutubePost.prototype.IsHidden = function() {
		return this.data && this.data.privacyStatus === "unlisted";
	};
	
	YoutubePost.prototype.IsPrivate = function() {
		return this.data && this.data.privacyStatus === "private";
	};
	
	YoutubePost.prototype.AreCommentsDisabled = function() {
		return this.data && this.data.commentsStatus === "disabled";
	};
	
	YoutubePost.prototype.IsLive = function() {
		return this.data && this.data.postType === "live";
	};
	
	YoutubePost.prototype.__can_hide = function() {
		return !this.IsDeleted() && !this.IsHidden();
	};
	
	YoutubePost.prototype.__can_private = function() {
		return !this.IsDeleted() && !this.IsPrivate();
	};
	
	YoutubePost.prototype.__can_show = function() {
		return !this.IsDeleted() && (this.IsHidden() || this.IsPrivate());
	};
	
	YoutubePost.prototype.__can_disable_comments = function() {
		return !this.IsDeleted() && !this.AreCommentsDisabled();
	};
	
	YoutubePost.prototype.__can_enable_comments = function() {
		return !this.IsDeleted() && this.AreCommentsDisabled();
	};
	
	YoutubePost.prototype.GetEmbedHtml = function(callback, properties) {
		let url = format("{0}//www.youtube.com/embed/{1}?enablejsapi=1&origin={2}",
			window.location.protocol,
			this.id,
			window.location.origin);
		let html = format("".concat(
			"<iframe type='text/html' width='368' height='182' src='{0}' frameborder='0'>",
			"</iframe>"
		), url);;
		callback({ html : html });
	};

}());