local utils = require "shared.apps.utils"
local permissions = require "shared.permissions.permissions"
require "LuaOrm.Date"

module(..., package.seeall, inconcert.orbit.application.Decorate)
local m_htmlRender = nil

function MapUris (module, prefix, htmlRender)
	m_htmlRender = htmlRender
end

local function GetTranslatedTime (text)
	assert(type(text) == "string")
	local _T = m_htmlRender.extension.makeTextTranslator()
	local translatedTime = text
	if text == "SECOND" then
		translatedTime = _T("Seconds")
	elseif text == "MINUTE" then
		translatedTime = _T("Minutes")
	elseif text == "HOUR" then
		translatedTime = _T("Hours")
	elseif text == "DAY" then
		translatedTime = _T("Days")
	elseif text == "MONTH" then
		translatedTime = _T("Months")
	elseif text == "YEAR" then
		translatedTime = _T("Years")
	end
	return translatedTime
end

--[[
ATENCION: 
 En los indicadores el atributo "scope" se define de la siguiente manera:
 
   campaign: "NOT DEPENDS ON CONFIGURED CHANNELS"  / "DEPENDS ON CONFIGURED CHANNELS" / "NOT ALLOWED"
   channels: "ALL" / nil, {INBOUND_CALL = true, OUTBOUND_CALL = true, ...}
   att_levels = "CHANNEL DEPEND", "NOT CHANNEL DEPEND", nil

   campaign:
      true = El indicador se incluye a nivel campana.
      false o nil = El indicador NO se incluye a nivel campana

   channels:
      "ALL" = El indicador se incluye para todos los canales.
      nil   = El indicador NO se incluye para ningun canal.
      table = {INBOUND_CALL = true, OUTBOUND_CALL = true, ...} Se incluye para los canales que se ha definido.

    att_levels:
      "CHANNEL DEPEND": Se incluye para todos los canales existentes, siempre y cuando el indicador exista definido para la Cuenta.
      "NOT CHANNEL DEPEND": El indicador se incluye sin importar si el mismo se encuentra definido en alguna cuenta.
      nil   = El indicador NO se incluye para ningun AL.
]]--

--------------------------------------------------------------
-- (Summary) Indicadores de Estados de los agentes en la campana
--
local function LoadSummaryIndicatorsByAgentStates (indicators)
	local vcc = inconcert.authUser.virtualContactCenter
	local _T = m_htmlRender.extension.makeTextTranslator()
	local category_type = "agents_by_status"
	local category_id = "agents_by_status"
	local category_desc = _T("Agents By Status")
	local panel = "summary"
	local panel_type = "summary_panel"
	local ind_type = "integer"
	local sections = {"AGENTS"}

	-- Al poner el scope "channels" como nil quiere decir que para este scope, no se visualizaran estos indicadores.
	local default_scope = { campaign = "NOT DEPENDS ON CONFIGURED CHANNELS", channels = nil, att_levels = "NOT CHANNEL DEPEND" }
	if utils.HasAllegroIntegration(vcc) then
		-- Si es un VCC de Telefonía de Allegro, el scope channels tiene que discriminarse y visualizarse ya que en TelAllegro es una única cuenta en toda la campaña
		default_scope.channels = { INBOUND_CALL = true, OUTBOUND_CALL = true }
	end

	local id = "total_agents_idle"
	indicators[id] = {
		id= id, type = ind_type,
		sections = sections,
		label = _T("Idle"), tooltip = _T("Idle"),
		panel = panel, panel_type = panel_type, panel_type = panel_type, category_id = category_id, category_desc = category_desc, category_type = category_type, scope = default_scope
	}

	id = "total_agents_break"
	indicators[id] = {
		id= id, type = ind_type,
		sections = sections,
		label = _T("Break"), tooltip = _T("Break"),
		panel = panel, panel_type = panel_type, panel_type = panel_type, category_id = category_id, category_desc = category_desc, category_type = category_type, scope = default_scope
	}

	id = "total_agents_busy"
	indicators[id] = {
		id= id, type = ind_type,
		sections = sections,
		label = _T("Busy"), tooltip = _T("Busy"),
		panel = panel, panel_type = panel_type, panel_type = panel_type, category_id = category_id, category_desc = category_desc, category_type = category_type, scope = default_scope
	}

	id = "total_agents_busy_cross_campaign"
	indicators[id] = {
		id= id, type = ind_type,
		sections = sections,
		label = _T("Busy In Another"), tooltip = _T("Busy In Another"),
		panel = panel, panel_type = panel_type, panel_type = panel_type, category_id = category_id, category_desc = category_desc, category_type = category_type, scope = default_scope
	}

	id = "total_agents_attending"
	indicators[id] = {
		id= id, type = ind_type,
		sections = sections,
		label = _T("Attending"), tooltip = _T("Attending"),
		panel = panel, panel_type = panel_type, panel_type = panel_type, category_id = category_id, category_desc = category_desc, category_type = category_type, scope = default_scope
	}

	id = "total_agents_attending_cross_campaign"
	indicators[id] = {
		id= id, type = ind_type,
		sections = sections,
		label = _T("Attending Another"), tooltip = _T("Attending Another"),
		panel = panel, panel_type = panel_type, panel_type = panel_type, category_id = category_id, category_desc = category_desc, category_type = category_type, scope = default_scope
	}

	id = "total_agents_reserved_cross_campaign"
	indicators[id] = {
		id= id, type = ind_type,
		sections = sections,
		label = _T("Reserved Another"), tooltip = _T("Reserved Another"),
		panel = panel, panel_type = panel_type, panel_type = panel_type, category_id = category_id, category_desc = category_desc, category_type = category_type, scope = default_scope
	}

	id = "total_agents_reserved"
	indicators[id] = {
		id= id, type = ind_type,
		sections = sections,
		label = _T("Reserved"), tooltip = _T("Reserved"),
		panel = panel, panel_type = panel_type, panel_type = panel_type, category_id = category_id, category_desc = category_desc, category_type = category_type, scope = default_scope
	}

	id = "total_agents_noacd"
	indicators[id] = {
		id= id, type = ind_type,
		sections = sections,
		label = _T("Online No ACD"), tooltip = _T("Online No ACD"),
		panel = panel, panel_type = panel_type, panel_type = panel_type, category_id = category_id, category_desc = category_desc, category_type = category_type, scope = default_scope
	}

	id = "total_agents_logged"
	indicators[id] = {
		id= id, type = ind_type,
		sections = sections,
		label = _T("Logged"), tooltip = _T("Logged"),
		panel = panel, panel_type = panel_type, panel_type = panel_type, category_id = category_id, category_desc = category_desc, category_type = category_type, scope = default_scope
	}

	id = "total_agents_offline"
	indicators[id] = {
		id= id, type = ind_type,
		sections = sections,
		label = _T("Offline"), tooltip = _T("Offline"),
		panel = panel, panel_type = panel_type, panel_type = panel_type, category_id = category_id, category_desc = category_desc, category_type = category_type, scope = default_scope
	}
end

--------------------------------------------------------------
-- (Summary) Indicadores Estados de las conversaciones - totales por campana
--
local function LoadSummaryIndicatorsByTotalsConversations (indicators)
	local _T = m_htmlRender.extension.makeTextTranslator()
	local category_type = "total_conversations_states"
	local category_id = "conversations_states"
	local category_desc = _T("Total Conversations States")
	local panel = "summary"
	local panel_type = "summary_panel"
	local ind_type = "integer"
	local default_scope = { campaign = "NOT DEPENDS ON CONFIGURED CHANNELS", channels = "ALL", att_levels = "CHANNEL DEPEND" }

	local id = "active_campaign_total"
	indicators[id] = {
		id= id, type = ind_type,
		sections = {"CONVERSATIONS"},
		label = _T("Total Active Conversations"), tooltip = _T("Total Active Conversations"),
		panel = panel, panel_type = panel_type, category_id = category_id, category_desc = category_desc, category_type = category_type,
		scope = default_scope
	}

	id = "queued_campaign_total"
	indicators[id] = {
		id= id, type = ind_type,
		sections = {"CONVERSATIONS"},
		label = _T("Total Queued Conversations"), tooltip = _T("Total Queued Conversations"),
		panel = panel, panel_type = panel_type, category_id = category_id, category_desc = category_desc, category_type = category_type,
		scope = default_scope
	}

	id = "waiting_campaign_total"
	indicators[id] = {
		id= id, type = ind_type,
		sections = {"CONVERSATIONS"},
		label = _T("Total Waiting Conversations"), tooltip = _T("Total Waiting Conversations"),
		panel = panel, panel_type = panel_type, category_id = category_id, category_desc = category_desc, category_type = category_type,
		scope = default_scope
	}

	id = "total_campaign_total"
	indicators[id] = {
		id= id, type = ind_type,
		sections = {"CONVERSATIONS"},
		label = _T("Total Conversations"), tooltip = _T("Total Conversations"),
		panel = panel, panel_type = panel_type, category_id = category_id, category_desc = category_desc, category_type = category_type,
		scope = default_scope
	}

	id = "abandoned_campaign_total"
	indicators[id] = {
		id= id, type = "percentage",
		label = _T("Abandoned %"), tooltip = _T("Abandoned %"),
		panel = panel, panel_type = panel_type, category_id = category_id, category_desc = category_desc, category_type = category_type,
		scope = default_scope
	}

	id = "totalGoals_campaign_total"
	indicators[id] = {
		id= id, type = ind_type,
		sections = {"CONVERSATIONS"},
		label = _T("Total Goals Conversations"), tooltip = _T("Total Goals Conversations"),
		panel = panel, panel_type = panel_type, category_id = category_id, category_desc = category_desc, category_type = category_type,
		scope = default_scope
	}

	--In allegro these indicators are global ------------------
	local allegroScope = { campaign = "DEPENDS ON CONFIGURED CHANNELS", channels = {INBOUND_CALL = true, OUTBOUND_CALL = true}, att_levels = "CHANNEL DEPEND" }

	id = "attentionTime_campaign_total"
	indicators[id] = {
		id= id, type = ind_type,
		sections = {"CONVERSATIONS"},
		label = _T("Total Time in Conversations"), tooltip = _T("Total Time in Conversations"),
		panel = panel, panel_type = panel_type, category_id = category_id, category_desc = category_desc, category_type = category_type,
		scope = allegroScope,
		filter = "ONLY_ALLEGRO"
	}

	id = "occupationTime_campaign_total"
	indicators[id] = {
		id= id, type = "datetime",
		label = _T("Total Operation Occupancy Time"), tooltip = _T("Total Operation Occupancy Time"),
		panel = panel, panel_type = panel_type, category_id = category_id, category_desc = category_desc, category_type = category_type,
		scope = allegroScope,
		filter = "ONLY_ALLEGRO"
	}

	id = "occupationRate_campaign_total"
	indicators[id] = {
		id= id, type = "percentage",
		label = _T("Total Operator Occupancy %"), tooltip = _T("Total Operator Occupancy %"),
		panel = panel, panel_type = panel_type, category_id = category_id, category_desc = category_desc, category_type = category_type,
		scope = allegroScope,
		filter = "ONLY_ALLEGRO"
	}
end

local function GetChannelsDefinition()
	local _T = m_htmlRender.extension.makeTextTranslator()
	-- Con el atributo "order", es con en el orden que se visualizaran las categorias del lado del cliente.
	local channels_definition = {
		INBOUND_CALL  = { enabled = true,  label = _T("Inbound call") , order = 1  },
		OUTBOUND_CALL = { enabled = true,  label = _T("Outbound call"), order = 2  },
		CHAT          = { enabled = true,  label = _T("Web Chat")     , order = 3  },
		FACEBOOK      = { enabled = true,  label = _T("Facebook")     , order = 4  },
		IM            = { enabled = false, label = _T("IM")           , order = 5  },
		INSTAGRAM     = { enabled = true,  label = _T("Instagram")    , order = 6  },
		YOUTUBE	      = { enabled = true,  label = _T("Youtube")      , order = 7  },
		MAIL          = { enabled = true , label = _T("Mail")         , order = 8  },
		SMS           = { enabled = true,  label = _T("SMS")          , order = 9  },
		RCS           = { enabled = true,  label = _T("Rcs")          , order = 10  },
		WHATSAPP      = { enabled = true,  label = _T("Whatsapp")     , order = 11 },
		TWITTER       = { enabled = true,  label = _T("Twitter")      , order = 12 },
		VOICEMAIL     = { enabled = true,  label = _T("Voicemail")    , order = 13 },
		WEBCONTACT    = { enabled = true,  label = _T("Contact Form") , order = 14 },
		APPSTORE      = { enabled = true,  label = _T("Appstore")     , order = 15  },
		GOOGLEPLAY    = { enabled = true,  label = _T("Googleplay")   , order = 16  }
	}

	return channels_definition
end

--------------------------------------------------------------
-- 
--
local function GetCategoryStatesChannelDescription (channel_id)
	local _T = m_htmlRender.extension.makeTextTranslator()
	local channels_definition = GetChannelsDefinition()
	local channel_def = channels_definition[channel_id]
	local channel_lbl = channel_def.label or ""

	return {
		category_type = "conversations_states_by_channel",
		category_id = string.format("conversations_states_by_%s_channel", channel_id),
		category_desc = string.format(_T("Conversations States By %s Channel"), channel_lbl)
	}
end

local function GetCategoryMembersChannelDescription (channel_id)
	local _T = m_htmlRender.extension.makeTextTranslator()
	local channels_definition = GetChannelsDefinition()
	local channel_def = channels_definition[channel_id]
	local channel_lbl = channel_def.label or ""

	return {
		category_type = "members_conversations_by_channel",
		category_id = string.format("members_conversations_by_%s_channel", channel_id),
		category_desc = string.format(_T("Users Conversations By %s Channel"), channel_lbl)
	}
end

--------------------------------------------------------------
-- (Summary) Indicadores Estados de las conversaciones por canales
--
local function LoadSummaryIndicatorsByChannelsConversations (indicators)
	local _T = m_htmlRender.extension.makeTextTranslator()

	local ind_definitions = {
		active  = { label = _T("Active"), channels = "ALL", sections = {"CONVERSATIONS"} },
		queued  = { label = _T("Queued"), channels = "ALL", sections = {"CONVERSATIONS"} },
		waiting = { label = _T("Waiting"),channels = "ALL", sections = {"CONVERSATIONS"} },
		total   = { label = _T("Total"),  channels = "ALL", sections = {"CONVERSATIONS"} },

		abandoned = {
			label = _T("Abandoned %"),
			type  = "percentage",
			channels = { INBOUND_CALL = true, OUTBOUND_CALL = true, CHAT = true, FACEBOOK = false, INSTAGRAM = false, YOUTUBE = false, APPSTORE = false, GOOGLEPLAY = false, RCS = false, MAIL = false, TWITTER = false, VOICEMAIL = false, WEBCONTACT = false, SMS = false, WHATSAPP = false }
		},
		totalGoals = {
			label = _T("Total Goals"),
			sections = {"CONVERSATIONS"},
			channels = { INBOUND_CALL = true, OUTBOUND_CALL = true, CHAT = true, FACEBOOK = true, INSTAGRAM = true, YOUTUBE = true, APPSTORE = true, GOOGLEPLAY = true, RCS = true, MAIL = true, TWITTER = true, VOICEMAIL = false, WEBCONTACT = true, SMS = true, WHATSAPP = true }
		},
		serviceLevel = {
			label = _T("Service Level"),
			type  = "percentage",
			channels = { INBOUND_CALL = true, OUTBOUND_CALL = false, CHAT = true, FACEBOOK = true, INSTAGRAM = true, YOUTUBE = true, APPSTORE = true, GOOGLEPLAY = true, RCS = true, MAIL = true, TWITTER = true, VOICEMAIL = false, WEBCONTACT = true, SMS = true, WHATSAPP = true }
		},
		answerTime = {
			label = _T("Average Answer Time"),
			type  = "datetime",
			channels = { INBOUND_CALL = true, OUTBOUND_CALL = false, CHAT = true, FACEBOOK = true, INSTAGRAM = true, YOUTUBE = true, APPSTORE = true, GOOGLEPLAY = true, RCS = true, MAIL = true, TWITTER = true, VOICEMAIL = false, WEBCONTACT = true, SMS = true, WHATSAPP = true }
		},
		abandonTime = {
			label = _T("Average Abandon Time"),
			type  = "datetime",
			channels = { INBOUND_CALL = true, OUTBOUND_CALL = false, CHAT = true, FACEBOOK = false, INSTAGRAM = false, YOUTUBE = false, APPSTORE = false, GOOGLEPLAY = false, RCS = false, MAIL = false, TWITTER = false, VOICEMAIL = false, WEBCONTACT = false, SMS = false, WHATSAPP = false }
		},
		inboundAverageHandlingTime = {
			label = _T("Inbound Average Handling Time"),
			type  = "datetime",
			channels = { INBOUND_CALL = true, OUTBOUND_CALL = false, CHAT = true, FACEBOOK = true, INSTAGRAM = true, YOUTUBE = true, APPSTORE = true, GOOGLEPLAY = true, RCS = true, MAIL = true, TWITTER = true, VOICEMAIL = false, WEBCONTACT = true, SMS = true, WHATSAPP = true }
		},
		outboundContactRate = {
			label = _T("Contact Rate %"),
			type  = "percentage",
			channels = { INBOUND_CALL = false, OUTBOUND_CALL = true, CHAT = false, FACEBOOK = false, INSTAGRAM = false, YOUTUBE = false, APPSTORE = false, GOOGLEPLAY = false, RCS = false, MAIL = false, TWITTER = false, VOICEMAIL = false, WEBCONTACT = false, SMS = false, WHATSAPP = false }
		},
		outboundAverageHandlingTime = {
			label = _T("Outbound Average Handling Time"),
			type  = "datetime",
			channels = { INBOUND_CALL = false, OUTBOUND_CALL = true, CHAT = false, FACEBOOK = false, INSTAGRAM = false, YOUTUBE = false, APPSTORE = false, GOOGLEPLAY = false, RCS = false, MAIL = false, TWITTER = false, VOICEMAIL = false, WEBCONTACT = false, SMS = false, WHATSAPP = false }
		},
		occupationRate = {
			label = _T("Occupancy"),
			type  = "percentage",
			channels = { INBOUND_CALL = false, OUTBOUND_CALL = false, CHAT = true, FACEBOOK = true, INSTAGRAM = true, YOUTUBE = true, APPSTORE = true, GOOGLEPLAY = true, RCS = true, MAIL = true, TWITTER = true, VOICEMAIL = false, WEBCONTACT = true, SMS = true, WHATSAPP = true }
		},
		inboundOldestWaiting = {
			label = _T("Longer Waiting time"),
			type  = "datetime",
			channels = { INBOUND_CALL = true, OUTBOUND_CALL = false, CHAT = true, FACEBOOK = true, INSTAGRAM = true, YOUTUBE = true, APPSTORE = true, GOOGLEPLAY = true, RCS = true, MAIL = true, TWITTER = true, VOICEMAIL = false, WEBCONTACT = true, SMS = true, WHATSAPP = true }
		},

	-- solo para las live
		attentionTime = {
			label = _T("Time in Conversations"),
			type  = "datetime",
			channels = { INBOUND_CALL = false, OUTBOUND_CALL = false, CHAT = true, FACEBOOK = false, INSTAGRAM = false, YOUTUBE = false, APPSTORE = false, GOOGLEPLAY = false, RCS = false, MAIL = false, TWITTER = false, VOICEMAIL = false, WEBCONTACT = false, SMS = false, WHATSAPP = false }
		},
		occupationTime = {
			label = _T("Operator Occupancy"),
			type  = "datetime",
			channels = { INBOUND_CALL = false, OUTBOUND_CALL = false, CHAT = true, FACEBOOK = false, INSTAGRAM = false, YOUTUBE = false, APPSTORE = false, GOOGLEPLAY = false, RCS = false, MAIL = false, TWITTER = false, VOICEMAIL = false, WEBCONTACT = false, SMS = false, WHATSAPP = false }
		},

	-- solo para las offline
		answerTimeReopen = {
			label = _T("Reopen Average Answer Time"),
			type  = "datetime",
			channels = { INBOUND_CALL = false, OUTBOUND_CALL = false, CHAT = false, FACEBOOK = true, INSTAGRAM = true, YOUTUBE = true, APPSTORE = true, GOOGLEPLAY = true, RCS = true, MAIL = true, TWITTER = true, VOICEMAIL = false, WEBCONTACT = true, SMS = true, WHATSAPP = true }
		},
		inboundAverageHandlingTimeReopen = {
			label = _T("Reopen Average Handling Time"),
			type  = "datetime",
			channels = { INBOUND_CALL = false, OUTBOUND_CALL = false,CHAT = false, FACEBOOK = true, INSTAGRAM = true, YOUTUBE = true, APPSTORE = true, GOOGLEPLAY = true, RCS = true, MAIL = true, TWITTER = true, VOICEMAIL = false, WEBCONTACT = true, SMS = true, WHATSAPP = true }
		}
	}

	local CanAdd = function(def)
		return true
	end

	local Add = function(ind_id, ind_def, channel_id, channel_def)
		if not CanAdd(ind_def) then
			return false
		end

		local category_def = GetCategoryStatesChannelDescription(channel_id)
		local label = ind_def.label
		local channel_label = channel_def.label .. " " .. label
		local default_scope = { campaign = "DEPENDS ON CONFIGURED CHANNELS", channels = {[channel_id] = true}, att_levels = "CHANNEL DEPEND" }
		local lowerChannel = string.lower(channel_id)
		local full_id = ind_id .. "_" .. lowerChannel

		indicators[full_id] = {
			id = full_id,
			type = ind_def.type or "integer",
			sections = ind_def.sections,
			label = label,
			channel_label = channel_label,
			tooltip = channel_label,
			panel = "summary",
			panel_type = "summary_panel",
			scope = default_scope,
			category_type = category_def.category_type,
			category_id = category_def.category_id,
			category_desc = category_def.category_desc
		}

		return true
	end

	local channels_definition = GetChannelsDefinition()
	for channel_id, channel_def in pairs(channels_definition) do
		if channel_def.enabled then
			for ind_id, def in pairs(ind_definitions) do
				if def.channels == "ALL" or def.channels[channel_id] then
					Add(ind_id, def, channel_id, channel_def)
				end
			end
		end
	end
end

--------------------------------------------------------------
-- (Team Members) Indicadores de los team members (tiempos en los estados)
--
local function LoadMembersIndicators (indicators)
	local vcc = inconcert.authUser.virtualContactCenter
	local _T = m_htmlRender.extension.makeTextTranslator()
	local category_type = "members"
	local category_id = "members"
	local category_desc = _T("Max Team Users States Time")
	local panel = "members"
	local panel_type = "members_panel"
	local ind_type = "datetime"
	local operators = {"GREATER"}

	-- Al poner el scope "channels" como nil quiere decir que para este scope, no se visualizaran estos indicadores.
	local default_scope = { campaign = "NOT DEPENDS ON CONFIGURED CHANNELS", channels = nil, att_levels = "NOT CHANNEL DEPEND" }
	if utils.HasAllegroIntegration(vcc) then
		-- Si es un VCC de Telefonía de Allegro, el scope channels tiene que discriminarse y visualizarse ya que en TelAllegro es una única cuenta en toda la campaña.
		default_scope.channels = { INBOUND_CALL = true, OUTBOUND_CALL = true }
	end

	--[[
		ATENCION:
			La tabla "eval", es por la cual se evaluan los campos client side en el archivo "sup_alarms_notifications.js"
			funcion "SupAlarmsNotifications.prototype.eval_notifications_alarms".

			El atributo "eval_type" puede contener 3 valores:
				"FIELD_VS_FIELD": Evalua el valor de la alarma contra el valor de la entidad sin modificar ningun valor.
				"SCOPE":          Valida si la alarma y la entidad pueden ser evaluados segun el scope "channels" "att_levels"
				"ALARM":          Realiza validaciones de menor, mayor, igual, distinto etc.
	]]--
	local id = "max_member_noacd_time"
	indicators[id] = {
		id= id, type = ind_type,
		label = _T("Max User Online No ACD Time"), tooltip = _T("Max User Online No ACD Time"),
		panel = panel, panel_type = panel_type, category_id = category_id, category_desc = category_desc, category_type = category_type, scope = default_scope,
		operators = operators,
		eval = {
			fields = {
				{name = "state", type = "string", operator = "EQUAL", value = "NOACD", eval_type = "FIELD_VS_FIELD"},
				{name = "stateTimeTimestamp", type = "member-date", eval_type = "ALARM"}
			}
		},
		other_values = {
			member_state = "NOACD"
		}
	}

	id = "max_member_idle_time"
	indicators[id] = {
		id= id, type = ind_type,
		label = _T("Max User Idle Time"), tooltip = _T("Max User Idle Time"),
		panel = panel, panel_type = panel_type, category_id = category_id, category_desc = category_desc, category_type = category_type, scope = default_scope,
		operators = operators,
		eval = {
			fields = {
				{name = "state", type = "string", operator = "EQUAL", value = "IDLE", eval_type = "FIELD_VS_FIELD"},
				{name = "stateTimeTimestamp", type = "member-date", eval_type = "ALARM"}
			}
		},
		other_values = {
			member_state = "IDLE"
		}
	}

	id = "max_member_any_break_time"
	indicators[id] = {
		id= id, type = ind_type,
		label = _T("Max User Break Time"), tooltip = _T("Max User Break Time"),
		panel = panel, panel_type = panel_type, category_id = category_id, category_desc = category_desc, category_type = category_type, scope = default_scope,
		operators = operators,
		eval = {
			fields = {
				{name = "state", type = "string", operator = "EQUAL", value = "BREAK", eval_type = "FIELD_VS_FIELD"},
				{name = "stateTimeTimestamp", type = "member-date", eval_type = "ALARM"}
			}
		},
		other_values = {
			member_state = "ANY_BREAK" -- todos los estados break posibles, los del sistema (break, launch, away, etc) + los custom del vcc
		}
	}
end

--------------------------------------------------------------
-- (Team Members Conversations) Indicadores de los team members (cantidad de interacciones de un member, o las interaccioens de un member que ha superado tanto tiempo en un estado (taken o wrapup))
-- Estas alarmas se procesa algunas client side y otras server side
--
local function LoadMembersIndicatorsByChannelsConversations (indicators)
	local _T = m_htmlRender.extension.makeTextTranslator()

	-- Función encargada de formatear los mensajes server side (tooltip o message a mostrar en las notificaciones)
	local function GetNotificationMessage (self, info)
--LogDebug("===> GetNotificationMessage info: '%s'", json.encode(info))
		local totalInteractions = "%s"
		local channel = "%s"
		local exceed = info.alarmConfig.value
		local translatedTime = GetTranslatedTime(info.alarmConfig.section)
		local message = self.notificationMessage:format(totalInteractions, channel, tostring(exceed), tostring(translatedTime))
--LogDebug("===> GetNotificationMessage message: '%s'", message)
		return message
	end

	local ind_definitions = {
	-- (PARA TODOS LOS CANALES) ------------------
	--
		-- (****) TOTALES de las interacciones en TAKEN de un member.
		member_with_conversations_in_taken = {
			label = _T("User with conversations in Taken"), type = "integer", channels = "ALL", sections = {"CONVERSATIONS"},
			eval = {
				fields = {
					{name = "ID", type = "string", operator = "EQUAL", alarm_attr_name_value = "agentId", eval_type = "EVAL_AGAINST_NOTIFICATION"} -- evaluar contra la notificacion, no contra la alarma
				}
			},
			filter = "CAMPAIGN"
		},
		-- (******) TOTALES de las interacciones en TAKEN que tenga un member, que superen un tiempo determinado en el estado TAKEN.
		member_with_conversations_time_in_taken = {
			label = _T("Taken time conversations"), type = "datetime", channels = "ALL",
			operators = {"GREATER"},
			attributes = {interactionStateTime = "assignedTime"},-- Este es el atributo por el cual se calcula el tiempo que la interaccion se encuentra en taken, (el cálculo se realiza server side)
			notificationMessage = _T("Has %s taken %s conversations that exceed %s %s."), -- Ej: Has 2 taken MAIL conversations that exceed 10 Minutes.
			GetNotificationMessage = function(self, info)
				return GetNotificationMessage(self, info)
			end,
			eval = {
				fields = {
					{name = "ID", type = "string", operator = "EQUAL", alarm_attr_name_value = "agentId", eval_type = "EVAL_AGAINST_NOTIFICATION"} -- evaluar contra la notificacion, no contra la alarma
				}
			},
			filter = "CAMPAIGN"
		},
	-- (SOLO PARA LOS CANALES LIVE "CHAT", "CALL"  ------------------
	--
		-- (****) TOTALES de las interacciones en WRAPUP de un member.
		member_with_conversations_in_wrapup = {
			label = _T("User with conversations in WrapUp"),
			type  = "integer",
			sections = {"CONVERSATIONS"},
			eval = {
				fields = {
					{name = "ID", type = "string", operator = "EQUAL", alarm_attr_name_value = "agentId", eval_type = "EVAL_AGAINST_NOTIFICATION"} -- evaluar contra la notificacion, no contra la alarma
				}
			},
			filter = "CAMPAIGN",
			channels = { INBOUND_CALL = true, OUTBOUND_CALL = true, CHAT = true, FACEBOOK = false, INSTAGRAM = false, YOUTUBE = false, APPSTORE = false, GOOGLEPLAY = false, RCS = false, MAIL = false, TWITTER = false, VOICEMAIL = false, WEBCONTACT = false, SMS = false, WHATSAPP = false }
		},
		-- (******) TOTALES de las interacciones en WRAPUP que tenga un member, que superen un tiempo determinado en el estado WRAPUP.
		member_with_conversations_time_in_wrapup = {
			label = _T("WrapUp time conversations"),
			type  = "datetime",
			operators = {"GREATER"},
			attributes = {interactionStateTime = "wrapupTime"}, -- Este es el atributo por el cual se calcula el tiempo que la interaccion se encuentra en wrapup, (el cálculo se realiza server side)
			notificationMessage = _T("Has %s wrapup %s conversations that exceed %s %s."), -- Ej: Has 2 wrapup CHAT conversations that exceed 10 Minutes.
			GetNotificationMessage = function(self, info)
				return GetNotificationMessage(self, info)
			end,
			eval = {
				fields = {
					{name = "ID", type = "string", operator = "EQUAL", alarm_attr_name_value = "agentId", eval_type = "EVAL_AGAINST_NOTIFICATION"} -- evaluar contra la notificacion, no contra la alarma
				}
			},
			filter = "CAMPAIGN",
			channels = { INBOUND_CALL = true, OUTBOUND_CALL = true, CHAT = true, FACEBOOK = false, INSTAGRAM = false, YOUTUBE = false, APPSTORE = false, GOOGLEPLAY = false, RCS = false, MAIL = false, TWITTER = false, VOICEMAIL = false, WEBCONTACT = false, SMS = false, WHATSAPP = false }
		}
	}

	local CanAdd = function(def)
		return true
	end

	local Add = function(ind_id, ind_def, channel_id, channel_def)
		if not CanAdd(ind_def) then
			return false
		end

		local category_def = GetCategoryMembersChannelDescription(channel_id)
		local label = ind_def.label
		local channel_label = channel_def.label .. " " .. label
		local default_scope = { campaign = "DEPENDS ON CONFIGURED CHANNELS", channels = {[channel_id] = true}, att_levels = nil }
		local lowerChannel = string.lower(channel_id)
		local full_id = ind_id .. "_" .. lowerChannel

		indicators[full_id] = {
			id = full_id,
			type = ind_def.type or "integer",
			sections = ind_def.sections,
			label = label,
			channel_label = channel_label,
			tooltip = channel_label,
			panel = "members",
			panel_type = "members_panel",
			scope = default_scope,
			category_type = category_def.category_type,
			category_id = category_def.category_id,
			category_desc = category_def.category_desc,
			eval = ind_def.eval,
			operators = ind_def.operators,
			attributes = ind_def.attributes,
			notificationMessage = ind_def.notificationMessage,
			GetNotificationMessage = ind_def.GetNotificationMessage,
			filter = ind_def.filter
		}

		return true
	end

	local channels_definition = GetChannelsDefinition()
	for channel_id, channel_def in pairs(channels_definition) do
		if channel_def.enabled then
			for ind_id, def in pairs(ind_definitions) do
				if def.channels == "ALL" or def.channels[channel_id] then
					Add(ind_id, def, channel_id, channel_def)
				end
			end
		end
	end
end

--------------------------------------------------------------
-- (Interactions) Indicadores de las interacciones
--
local function LoadConversationsIndicators (indicators)
	local _T = m_htmlRender.extension.makeTextTranslator()
	local category_type = "conversation_time_on_state"
	local category_id = "conversation_time_on_state"
	local category_desc = _T("Conversation Time on State")
	local panel_type = "conversations_panel"
	local default_scope = { campaign = "NOT DEPENDS ON CONFIGURED CHANNELS", channels = "ALL", att_levels = "CHANNEL DEPEND" }
	local ind_type = "datetime"
	local operators = {"LESS", "GREATER"}

	--[[
		ATENCION:
			La tabla "eval", es por la cual se evaluan los campos client side en el archivo "sup_alarms_notifications.js"
			funcion "SupAlarmsNotifications.prototype.eval_notifications_alarms".

			El atributo "eval_type" puede contener 3 valores:
				"FIELD_VS_FIELD": Evalua el valor de la alarma contra el valor de la entidad sin modificar ningun valor.
				"SCOPE":          Evalua si la alarma y la entidad pueden ser evaluados segun el scope "channels" "att_levels"
				"ALARM":          Realiza validaciones de menor, mayor, igual, distinto etc.
	]]--

	--[[
		Las interacciones se validan contra la fecha en fecha del ultimo estado 'timestampDouble',
		menos los appointments que van contra la fecha de creacion.
	]]--

	local id = "conversation_queued_time"
	indicators[id] = {
		id= id, type = ind_type,
		label = _T("Time in Queued State"), tooltip = _T("Time in Queued State"),
		panel = "queued conversations", panel_type = panel_type, category_id = category_id, category_desc = category_desc, category_type = category_type, scope = default_scope,
		operators = operators,
		eval = {
			fields = {
				{name = "state", type = "string", operator = "EQUAL", value = "QUEUED", eval_type = "FIELD_VS_FIELD"},
				{name = "account", type = "string", eval_type = "SCOPE"},
				{name = "attention_level", type = "string", eval_type = "SCOPE"},
				{name = "timestampDouble", type = "date", eval_type = "ALARM"}
			}
		}
	}

	id = "conversation_taken_time"
	indicators[id] = {
		id= id, type = ind_type,
		label = _T("Time in Taken State"), tooltip = _T("Time in Taken State"),
		panel = "monitoring conversations", panel_type = panel_type, category_id = category_id, category_desc = category_desc, category_type = category_type, scope = default_scope,
		operators = operators,
		eval = {
			fields = {
				{name = "state", type = "string", operator = "EQUAL", value = "TAKEN", eval_type = "FIELD_VS_FIELD"},
				{name = "account", type = "string", eval_type = "SCOPE"},
				{name = "attention_level", type = "string", eval_type = "SCOPE"},
				{name = "timestampDouble", type = "date", eval_type = "ALARM"}
			}
		}
	}

	id = "conversation_waiting_time"
	indicators[id] = {
		id= id, type = ind_type,
		label = _T("Time in Waiting State"), tooltip = _T("Time in Waiting State"),
		panel = "waiting conversations", panel_type = panel_type, category_id = category_id, category_desc = category_desc, category_type = category_type, scope = default_scope,
		operators = operators,
		eval = {
			fields = {
				{name = "state", type = "string", operator = "EQUAL", value = "WAITING", eval_type = "FIELD_VS_FIELD"},
				{name = "account", type = "string", eval_type = "SCOPE"},
				{name = "attention_level", type = "string", eval_type = "SCOPE"},
				{name = "timestampDouble", type = "date", eval_type = "ALARM"}
			}
		}
	}

	id = "conversation_wrapup_time"
	indicators[id] = {
		id= id, type = ind_type,
		label = _T("Time in Wrap Up State"), tooltip = _T("Time in Wrap Up State"),
		panel = "monitoring conversations", panel_type = panel_type, category_id = category_id, category_desc = category_desc, category_type = category_type, scope = default_scope,
		operators = operators,
		eval = {
			fields = {
				{name = "state", type = "string", operator = "EQUAL", value = "WRAPUP", eval_type = "FIELD_VS_FIELD"},
				{name = "account", type = "string", eval_type = "SCOPE"},
				{name = "attention_level", type = "string", eval_type = "SCOPE"},
				{name = "timestampDouble", type = "date", eval_type = "ALARM"}
			}
		}
	}

	id = "conversation_abandoned_time"
	indicators[id] = {
		id= id, type = ind_type,
		label = _T("Time in Abandoned State"), tooltip = _T("Time in Abandoned State"),
		panel = "abandoned conversations", panel_type = panel_type, category_id = category_id, category_desc = category_desc, category_type = category_type, scope = default_scope,
		operators = operators,
		filter = "ONLY_ALLEGRO",
		eval = {
			fields = {
				{name = "state", type = "string", operator = "EQUAL", value = "ABANDONED", eval_type = "FIELD_VS_FIELD"},
				{name = "account", type = "string", eval_type = "SCOPE"},
				{name = "attention_level", type = "string", eval_type = "SCOPE"},
				{name = "timestampDouble", type = "date", eval_type = "ALARM"}
			}
		}
	}

	--[[
	APPOINTMENT
		Los appointments se validan la fecha de creacion 'startTimestamp'
	]]--
	id = "conversation_open_time"
	indicators[id] = {
		id= id, type = ind_type,
		label = _T("Time in Appointment State"), tooltip = _T("Time in Appointment State"),
		panel = "appointment conversations", panel_type = panel_type, category_id = category_id, category_desc = category_desc, category_type = category_type, scope = default_scope,
		operators = operators,
		filter = "ONLY_ALLEGRO",
		eval = {
			fields = {
				{name = "state", type = "string", operator = "EQUAL", value = "OPEN", eval_type = "FIELD_VS_FIELD"},
				{name = "account", type = "string", eval_type = "SCOPE"},
				{name = "attention_level", type = "string", eval_type = "SCOPE"},
				{name = "startTimestamp", type = "date", eval_type = "ALARM"}
			}
		}
	}
end

local function LoadCampaignIndicators ()
	local indicators = {}

	-- Summary indicators
	LoadSummaryIndicatorsByAgentStates(indicators)
	LoadSummaryIndicatorsByTotalsConversations(indicators)
	LoadSummaryIndicatorsByChannelsConversations(indicators)
	-- Members indicators
	LoadMembersIndicators(indicators) -- Estos Indicadores, salen del request de summary y demás.
	LoadMembersIndicatorsByChannelsConversations(indicators) -- Estos indicadores salen del request de Members
	-- Converstaions indicators
	LoadConversationsIndicators(indicators)

	return indicators
end

local function MakeIndicatorsByScope (list)
	local by_scope = {
		campaign = {},
		channels = {
			INBOUND_CALL = {}, OUTBOUND_CALL = {}, CHAT = {}, FACEBOOK = {}, INSTAGRAM = {}, YOUTUBE = {}, APPSTORE = {}, GOOGLEPLAY = {}, RCS = {}, IM = {}, MAIL = {}, SMS = {}, WHATSAPP = {}, TWITTER = {}, VOICEMAIL = {}, WEBCONTACT = {}
		},
		att_levels = {}
	}

	local added_in_scopes

	for indicator_id, indicator in pairs(list) do

		added_in_scopes = 0

		if indicator.scope then

			if indicator.scope.campaign ~= "NOT ALLOWED" then
				by_scope.campaign[indicator_id] = indicator
				added_in_scopes = added_in_scopes + 1
			end

			if indicator.scope.channels == "ALL" then
				for channel in pairs(by_scope.channels) do
					by_scope.channels[channel][indicator_id] = indicator
				end
				added_in_scopes = added_in_scopes + 1

			elseif type(indicator.scope.channels) == "table" then
				for channel in pairs(indicator.scope.channels or {}) do
					if by_scope.channels[channel] then
						by_scope.channels[channel][indicator_id] = indicator
						added_in_scopes = added_in_scopes + 1
					else
						LogWarning("[SUPERVISOR:MakeIndicatorsByScope] Unable to scope '%s' indicator. Unrecognized '%s' channel.", indicator_id, channel)
					end
				end
			end

			if indicator.scope.att_levels then
				by_scope.att_levels[indicator_id] = indicator
				added_in_scopes = added_in_scopes + 1
			end

		end

		if added_in_scopes == 0 then
			LogWarning("[SUPERVISOR:MakeIndicatorsByScope] Unable to scope '%s' indicator. This indicator has no defined scope.", indicator_id)
		end
	end

	return by_scope
end

local function LoadCategoriesIndicators ()
	local _T = m_htmlRender.extension.makeTextTranslator()

	-- Con el atributo "order", es el orden en el que se visualizaran las categorias del lado del cliente.
	local categories = {
		["agents_by_status"] = {
			desc = _T("Agents By Status"),
			category_type = "agents_by_status",
			order = 1
		},
		["members"] = {
			desc = _T("Max Team Users States Time"),
			category_type = "members",
			order = 2
		},
		["conversation_time_on_state"] = {
			desc = _T("Conversation Time on State"),
			category_type = "conversation_time_on_state",
			order = 3
		},
		-- Esta es la categoria de Totales de todos los channels
		["conversations_states"] = {
			desc = _T("Total Conversations States"),
			category_type = "total_conversations_states",
			order = 4
		}
	}

	local total_order = utils.Cardinality(categories)
	local channels_definition = GetChannelsDefinition()

	local function AddCategory (categoryDef)
		total_order = total_order + 1
		categories[categoryDef.category_id] = {
			desc = categoryDef.category_desc,
			category_type = categoryDef.category_type,
			order = total_order
		}
	end
	-- Ahora van los totales por canales.
	for channel_id, channel_def in pairs(channels_definition) do
		local stateCategoryDef = GetCategoryStatesChannelDescription(channel_id)
		AddCategory(stateCategoryDef)
		local membersCategoryDef = GetCategoryMembersChannelDescription(channel_id)
		AddCategory(membersCategoryDef)
	end

	return categories
end

--[[
	Funcion global invocada para obtener la lista de indicadores a retornar (ej: al lado del cliente)
]]--
function LoadSystemIndicators ()
	LogInfo("[SUPERVISOR:LoadSystemIndicators] Load system indicators.")

	local vcc = inconcert.authUser.virtualContactCenter
	local list = LoadCampaignIndicators()

	-- Al poner el scope "channels" como nil quiere decir que para este scope, no se visualizaran estos indicadores.
	local default_scope = { campaign = "NOT DEPENDS ON CONFIGURED CHANNELS", channels = nil, att_levels = "NOT CHANNEL DEPEND" }
	if utils.HasAllegroIntegration(vcc) then
		-- Si es un VCC de Telefonía de Allegro, el scope channels tiene que discriminarse y visualizarse ya que en TelAllegro es una única cuenta en toda la campaña.
		default_scope.channels = { INBOUND_CALL = true, OUTBOUND_CALL = true }
	end

	-- Agregamos a la lista de indicadores uno inventado (no configurable) para kickout
	local kickout_indicator = {
		category_desc = "Max Team Users States Time",
		category_id = "members",
		category_type = "members",
		id = "kickout",
		label = "kickout_indicator_label",
		panel = "members",
		panel_type = "members_panel",
		tooltip = "kickout_tooltip",
		type = "integer",
		scope = default_scope,
		filter = "NOT_CONFIGURABLE",
		other_values = {
			member_state = "OFFLINE"
		}
	}
	list.kickout = kickout_indicator

	local by_scope = MakeIndicatorsByScope(list)

	local indicators = {
		categories = LoadCategoriesIndicators(),
		list = list,
		by_scope = by_scope
	}
	--LogDebug("[SUPERVISOR:LoadSystemIndicators] indicators: %s", json.encode(indicators))
	return indicators
end

----------------------------------------------------------------------------
--[[
	Aqui estan las funciones para evaluar las alarms en tiempo real.
]]--
----------------------------------------------------------------------------
local function ConvertTimeInSeconds (value, dateType)
	assert(type(value) == "number")
	assert(type(dateType) == "string")
	if dateType == "MINUTE" then
		value = value * 60
	elseif dateType == "HOUR" then
		value = value * 60 * 60
	elseif dateType == "DAY" then
		value = value * 60 * 60 * 24
	elseif dateType == "MONTH" then
		value = value * 60 * 60 * 24 * 30 --la hacemos facil por ahora
	elseif dateType == "YEAR" then
		value = value * 60 * 60 * 24 * 365
	end
	return value
end

local function GetAlarmValue (alarm)
	local value = tonumber(alarm.value)
	if alarm.section then
		value = ConvertTimeInSeconds(value, alarm.section)
	end
	return value
end

local function EvaluateAlarm (indicatorId, alarmConfig, indicator, scope)
	local status = false
	local alarmValue = GetAlarmValue(alarmConfig)
	local indicatorText = type(indicator) == "table" and indicator.text or indicator
	local indicatorValue = type(indicator) == "table" and indicator.source or tonumber(indicator)

	--LogDebug("[SUPERVISOR:EvaluateAlarm] --> IndicatorId: '%s', indicatorText: '%s', indicatorValue: '%s', '%s' alarmValue: '%s'", indicatorId, indicatorText, indicatorValue, alarmConfig.operator, alarmValue)

	local areNumericValues = type(alarmValue) == "number" and type(indicatorValue) == "number"
	if areNumericValues then
		if alarmConfig.operator == "EQUAL" then
			status = indicatorValue == alarmValue
		elseif alarmConfig.operator == "DISTINCT" then
			status = indicatorValue ~= alarmValue
		elseif alarmConfig.operator == "LESS" then
			status = indicatorValue < alarmValue
		elseif alarmConfig.operator == "GREATER" then
			status = indicatorValue > alarmValue
		elseif alarmConfig.operator == "LESS_OR_EQUAL" then
			status = indicatorValue <= alarmValue
		elseif alarmConfig.operator == "GREATER_OR_EQUAL" then
			status = indicatorValue >= alarmValue
		end
	else
		LogWarning("[SUPERVISOR:EvaluateAlarm] Some values are not numerical. IndicatorId; '%s', indicatorValue: '%s' '%s' alarmValue: '%s'", indicatorId, indicatorValue, alarmConfig.operator, alarmValue)
	end

	if status then
		local alarmNotification = {
			indicator_id = indicatorId,
			values = {
				current_value = indicatorText, current_value_source = indicatorValue
			}
		}
		if alarmConfig.indicatorDefinition then
			-- Acá se calcula el mensaje que se visualizará client side (tooltip y popup de notificaciones)
			-- Por ahora solo se utiliza en los indicadores de Members y sus interacciones (cantidad y tiempo de un estado determinado. En estos "LoadMembersIndicatorsByChannelsConversations")
			if alarmConfig.indicatorDefinition.GetNotificationMessage and alarmConfig.indicatorDefinition.notificationMessage then
				alarmNotification.notificationMessage = alarmConfig.indicatorDefinition:GetNotificationMessage({
					alarmConfig = alarmConfig,
					alarmValue = alarmValue,
					alarmNotification = alarmNotification
				})
			end
		end
		for k, v in pairs(scope) do
			if v and v ~= "" then
				alarmNotification[k] = v
			end
		end
		return true, alarmNotification
	end

	return false
end

local function GetSummaryIndicatorValue (indicator)
	indicator = indicator or {}
	return {
		text   = indicator.ofWorkgroups or indicator.total or "0",
		source = indicator.ofWorkgroups_source or indicator.total_source or 0
	}
end

local function ProcessAgentStatusAlarms (indicators, alarms, result, scope)
	for _, indicator in pairs(indicators or {}) do
		if alarms[indicator.ID] and alarms[indicator.ID].enabled then
			local status, alarm = EvaluateAlarm(indicator.ID, alarms[indicator.ID], GetSummaryIndicatorValue(indicator), scope)
			if status then
				table.insert(result, alarm)
			end
		end
	end
end

local function ProcessChannelSummaryAlarms (channel, indicators, alarms, result, scope)
	for _, indicator in pairs(indicators or {}) do
		if alarms[indicator.ID] and alarms[indicator.ID].enabled then
			local status, alarm = EvaluateAlarm(indicator.ID, alarms[indicator.ID], GetSummaryIndicatorValue(indicator), scope)
			if status then
				table.insert(result, alarm)
			end
		end
	end
end

-- Esta funcion es la misma, pero para que quede mas clara su invocacion, asigna la misma con un nuevo nombre.
local ProcessAllChannelsSummaryAlarms = ProcessChannelSummaryAlarms

local function ProcessAgentStateTimeAlarms (stateTimes, alarms, result, scope)
	for stateId, maxTime in pairs(stateTimes or {}) do
		local indicatorId = "max_member_" .. stateId:lower() .. "_time"
		if alarms[indicatorId] and alarms[indicatorId].enabled then
			local status, alarm = EvaluateAlarm(indicatorId, alarms[indicatorId], maxTime, scope)
			if status then
				table.insert(result, alarm)
			end
		end
	end
end

local function ProcessInteractionStateTimeAlarms (stateTimes, alarms, result, scope)
--LogDebug("-a1---------ProcessInteractionStateTimeAlarms --> stateTimes: '%s'", json.encode(stateTimes or {}))
--LogDebug("-b1---------ProcessInteractionStateTimeAlarms --> alarms: '%s'", json.encode(alarms or {}))
	for stateId, maxTime in pairs(stateTimes or {}) do
		local indicatorId = "conversation_" .. stateId:lower() .. "_time"
		if alarms[indicatorId] and alarms[indicatorId].enabled then
			local status, alarm = EvaluateAlarm(indicatorId, alarms[indicatorId], maxTime, scope)
			if status then
				table.insert(result, alarm)
			end
		end
	end
end

--[[
	Funcion global invocada para procesar y evaluar las alarmas de los INDICADORES, y que las mismas sean notificadas al cliente.
]]--
function ProcessAllIndicatorsAlarms (info, alarmsResult, panelResult, campaignResults)
	local vcc = inconcert.authUser.virtualContactCenter
	local fullCampaignId = utils.AppendVcc(info.campaignId, vcc)
	local fullUserId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	--LogInfo("[SUPERVISOR:ProcessAllIndicatorsAlarms] Processing alarms for '%s' campaign, user '%s'.", fullCampaignId, fullUserId)

	--LogDebug("[SUPERVISOR:ProcessAllIndicatorsAlarms] 1 campaign: '%s', info.alarms '%s'.", fullCampaignId, json.encode(info.alarms or {}))
	-- LogDebug("[SUPERVISOR:ProcessAllIndicatorsAlarms] 2 campaign: '%s', panelResult.realtimeSummary: %s", fullCampaignId, json.encode(panelResult.realtimeSummary or {}))
	--LogDebug("[SUPERVISOR:ProcessAllIndicatorsAlarms] 3 campaign: '%s', campaignResults.maxStateTimes: %s", fullCampaignId, json.encode(campaignResults.maxStateTimes or {}))
	--LogDebug("[SUPERVISOR:ProcessAllIndicatorsAlarms] 4 campaign: '%s', campaignResults.maxInteractionTimes: %s", fullCampaignId, json.encode(campaignResults.maxInteractionTimes or {}))

	local function CanProcessAlarms ()
		if utils.TableIsEmpty(info.alarms) then
		-- 1 No tiene alarmas configuradas
			--LogInfo("[SUPERVISOR:ProcessAllIndicatorsAlarms] Not configured alarms on campaign '%s'.", fullCampaignId)
			return false
		end
		-- 2 Si, puede procesar las alarmas
		return true
	end

	if not CanProcessAlarms() then
		return false
	end

	-- Se cargan las alarmas
	local alarmsScope = {
		campaign_id = fullCampaignId,
		account_id  = info.account or "",
		al_id       = info.attention_level or ""
	}

	for _, summary in pairs(panelResult.realtimeSummary) do
		if summary.ID == "agents_by_status" then
			--LogInfo("[SUPERVISOR:ProcessAllIndicatorsAlarms] Processing Realtime Indicators: ID '%s', type '%s', description: '%s'.", summary.ID, summary.type, summary.description)
			ProcessAgentStatusAlarms(summary.indicators, info.alarms, alarmsResult, alarmsScope)

		elseif summary.type == "channel" then
			--LogInfo("[SUPERVISOR:ProcessAllIndicatorsAlarms] Processing Realtime Indicators: ID '%s', type '%s', description: '%s'.", summary.ID, summary.type, summary.description)
			ProcessChannelSummaryAlarms(summary.ID, summary.indicators, info.alarms, alarmsResult, alarmsScope)

		elseif summary.type == "entity" and summary.ID == "campaignTotal" then
			--LogInfo("[SUPERVISOR:ProcessAllIndicatorsAlarms] Processing Realtime Indicators: ID '%s', type '%s', description: '%s'.", summary.ID, summary.type, summary.description)
			ProcessAllChannelsSummaryAlarms(summary.ID, summary.indicators, info.alarms, alarmsResult, alarmsScope)
		else
			LogWarning("[SUPERVISOR:ProcessAllIndicatorsAlarms] Unrecognized Realtime Indicators: ID '%s', type '%s', description: '%s'.", summary.ID, summary.type, summary.description)
		end
	end

	--LogInfo("[SUPERVISOR:ProcessAllIndicatorsAlarms] Processing Indicators Agents States Time.")
	ProcessAgentStateTimeAlarms(campaignResults.maxStateTimes, info.alarms, alarmsResult, alarmsScope)

	--LogInfo("[SUPERVISOR:ProcessAllIndicatorsAlarms] Processing Indicators Conversation Time on State.")
	ProcessInteractionStateTimeAlarms(campaignResults.maxInteractionTimes or {}, info.alarms, alarmsResult, alarmsScope)

	return true
end

--[[

Esta función es la encargada de validar las alarmas definidas para los Indicadores sobre las interacciones de los Members (LoadMembersIndicatorsByChannelsConversations).
La info sale del request de members->supervisorshared y NO desde campaign-summary->supervisorshared, como lo hacen todas las demás alarmas.

[
  {
    "ID": "agent1@emma",
    "workgroup": "",
    "sessionOccupiedTime": 205.59200000763,
    "defaultCampaign": "campmail1@emma",
    "channels": {
      "CHAT": {
        "used": 1,
        "available": 1,
        "maximum": 2
      }
    },
    "oldestInteractionAssignment": {
      "CHAT": "2018-11-13 15:49:28.476"
    },
    "interactions": {
      "CHAT": {
        "1712fcb8-8cda-46f4-b5a2-26ef4542fd2a": {
          "state": "Wrapup",
          "type": "CHAT",
          "assignedTime": "2018-11-13 15:49:28.476",
          "initiative": "INBOUND",
          "assignedTimeTimestamp": 1542124168.476
        }
      }
    },
    "isLogged": true,


	Indicadores
	 Ej: member_with_conversations_in_taken_mail, member_with_conversations_in_wrapup_chat, etc
]]--
local function ProcessMembersConversationsAlarms (agents, alarms, alarmsResult, scope, campaignTimezone)
--LogDebug("-a---------ProcessMembersConversationsAlarms --> agents: '%s'", json.encode(agents or {}))
--LogDebug("-b---------ProcessMembersConversationsAlarms --> alarms: '%s'", json.encode(alarms or {}))

	local function MakeIndicatorId (templateIndicatorId, interaction, channel)
		local myChannel = channel:lower()
		if myChannel == "call" then
			-- El indicador de los canales de call, tiene que in precedido de la initiative (inbound/outbound)
			myChannel = ("%s_%s"):format(interaction.initiative:lower(), myChannel)
		end
		local state = interaction.state:lower()
		local indicatorId = templateIndicatorId:format(state, myChannel)
--LogDebug("----[MakeIndicatorId] indicatorId: '%s'", indicatorId)
		return indicatorId
	end

	-- (*) En este hash se almacenan los agentes, y los indicadores (por estado "taken/wrapup" y el canal de la interacciones) que se deben de notificar con sus alarmas.
	local agentsInteractionsByChannels = {}

	for i, agent in ipairs(agents or {}) do
		local interactions = agent.interactions or {}
--LogDebug(" %d - AgentId: '%s', isLogged: '%s', interactions: %s" , i, agent.ID, agent.isLogged, interactions)
		for channel, interactionsByChannel in pairs(interactions) do
--LogDebug("    - channel '%s', interactionsByChannel: '%s'" , channel, interactionsByChannel)
			if type(interactionsByChannel) == "table" then
				for interactionId, interaction in pairs(interactionsByChannel) do
					--[[
						Aca se agrupan las ALARMAS de los indicadores (****) de la funcion "LoadMembersIndicatorsByChannelsConversations"
						Que son los indicadores de cantidad de interacciones que tengan un member en un estado taken/wrapup y canal determinado.
					--]]
					local indicatorId = MakeIndicatorId("member_with_conversations_in_%s_%s", interaction, channel)
--LogDebug(" 1--> %s alarms[indicatorId]: %s", indicatorId, json.encode(alarms[indicatorId] or {}))
					if alarms[indicatorId] then
						if alarms[indicatorId].enabled then
							agentsInteractionsByChannels[agent.ID] = agentsInteractionsByChannels[agent.ID] or {}
							agentsInteractionsByChannels[agent.ID][indicatorId] = agentsInteractionsByChannels[agent.ID][indicatorId] or {totals = 0, alarm = alarms[indicatorId], channel = channel}
							agentsInteractionsByChannels[agent.ID][indicatorId].totals = agentsInteractionsByChannels[agent.ID][indicatorId].totals + 1
						end
					end

					--[[
						Aca se agrupan las ALARMAS de los indicadores (******) de la funcion "LoadMembersIndicatorsByChannelsConversations"
						Que son los indicadores del tiempo que se encuentran las interacciones de un member en un estado taken/wrapup y canal determinado.
					--]]
					local indicatorId = MakeIndicatorId("member_with_conversations_time_in_%s_%s", interaction, channel)
--LogDebug(" 2--> %s alarms[indicatorId]: %s", indicatorId, json.encode(alarms[indicatorId] or {}))
					if alarms[indicatorId] and alarms[indicatorId].enabled then
						agentsInteractionsByChannels[agent.ID] = agentsInteractionsByChannels[agent.ID] or {}
						agentsInteractionsByChannels[agent.ID][indicatorId] = agentsInteractionsByChannels[agent.ID][indicatorId] or {interactions = {}, alarm = alarms[indicatorId], channel = channel, evaluateList = true}
						interaction.Id = interactionId
--LogDebug(" 2.1--> agentsInteractionsByChannels[%s][%s]: %s", agent.ID, indicatorId, json.encode(agentsInteractionsByChannels[agent.ID][indicatorId]))
						table.insert(agentsInteractionsByChannels[agent.ID][indicatorId].interactions, interaction)
					end
				end
			end
		end
	end

--LogDebug("-c---------ProcessMembersConversationsAlarms --> agentsInteractionsByChannels: '%s'", json.encode(agentsInteractionsByChannels))

	-- Aca se obtiene la defincion de los Indicadores de los members
	local membersIndicatorsDef = {}
	LoadMembersIndicatorsByChannelsConversations(membersIndicatorsDef)

	-- Esta es la fecha y hora actual, que se utilizara para validar el tiempo transcurrido de las interacciones en un estado determinado.
	local nowUTCTime = LuaOrm.Date()
	local nowUTCString = LuaOrm.Date():toString()
	nowUTCTimeSec = nowUTCTime:time()
--LogDebug("----> LuaOrm.Date():toString():  %s", nowUTCString)

	-- (**) Este es un hash con las alarmas que se tienen que validar como lista (lista de todas las interacciones que tenga cada uno de los members)
	local hashAlarmsByEvalList = {}

	-- Recorro el hash (*)
	for agentId, indicators in pairs(agentsInteractionsByChannels) do
		for indicatorId, indicator in pairs(indicators) do
			indicator.alarm.indicatorDefinition = membersIndicatorsDef[indicatorId]
			if not indicator.evaluateList then
			-- Indicadores (member_with_conversations_in_taken y member_with_conversations_in_wrapup)
			-- Si no se evalua como lista (es decir no se evaluan todas las interacciones), mando evaular la cantidad de interacciones que se encuentra en el atributo "totals"
				local status, alarm = EvaluateAlarm(indicatorId, indicator.alarm, indicator.totals, scope)
				if status then
					alarm.agentId = agentId
					table.insert(alarmsResult, alarm)
				end
			else
			-- Indicadores (member_with_conversations_time_in_taken y member_with_conversations_time_in_wrapup)
			-- Evalua como lista (es decir tengo que evaluar interacción por interacción)

				for _, interaction in ipairs(indicator.interactions) do
					--local indicatorId = ("member_with_conversations_in_%s_%s"):format(interaction.state:lower(), channel:lower())
					local interactionStateTime = indicator.alarm.indicatorDefinition.attributes and indicator.alarm.indicatorDefinition.attributes.interactionStateTime
--LogDebug(" 0.5--> %s interactionStateTime: %s", indicatorId, interactionStateTime)
					if interactionStateTime then
--LogDebug("----> interaction.%s: '%s'", interactionStateTime, interaction[interactionStateTime])
						if interaction[interactionStateTime] then
						-- Paso a UTC para realizar las validaciones
							local interactionStateUTCTime = utils.ConvertDateToUTCDate(interaction[interactionStateTime], nil, nil, true, campaignTimezone)
							local interactionStateUTCTimeSec = interactionStateUTCTime:time()
							local elapsedTimeSec = os.difftime(nowUTCTimeSec, interactionStateUTCTimeSec) -- tiempo transcurrido
							local alarmValue = GetAlarmValue(indicator.alarm)
--LogDebug(" 1--> %s alarms[indicatorId]: %s", indicatorId, json.encode(alarms[indicatorId] or {}))
--LogDebug("----> interactionStateUTCTime:  %s", interactionStateUTCTime)
--LogDebug("--c.1--> Now:  %d, Assigned Time: %d, Elapsed Time (sec) %d", nowUTCTimeSec, interactionStateUTCTimeSec, elapsedTimeSec)
--LogDebug("--c2--> (EN SEGUNDOS) alarm Value: %d, elapsed Time: %d", alarmValue, elapsedTimeSec)
--LogDebug("--c3--> campaignTimezone '%s'", json.encode(campaignTimezone))

							-- Mando a evaluar
							local status, alarm = EvaluateAlarm(indicatorId, indicator.alarm, elapsedTimeSec, scope)
							if status then
								-- Tengo que notificar, entonces almaceno en el hash (**)
								alarm.agentId = agentId
								hashAlarmsByEvalList[agentId] = hashAlarmsByEvalList[agentId] or {}
								hashAlarmsByEvalList[agentId][indicatorId] = hashAlarmsByEvalList[agentId][indicatorId] or {channel = indicator.channel, alarms = {}} or hashAlarmsByEvalList[agentId][indicatorId]
								table.insert(hashAlarmsByEvalList[agentId][indicatorId].alarms, alarm)
							end
						else
							LogWarning("[SUPERVISOR.ALARMS:ProcessMembersConversationsAlarms] Unable to process Alarm, attribute '%s' does not exist in Interaction: '%s' for Members Indicators '%s' definitions.", interactionStateTime, json.encode(interaction), indicatorId)
						end
					else
						LogWarning("[SUPERVISOR.ALARMS:ProcessMembersConversationsAlarms] Unable to process Alarm, attribute 'interactionStateTime' does not exist in Members Indicators '%s' definitions.", indicatorId)
					end
				end
			end
		end
	end

--LogDebug("-d---------ProcessMembersConversationsAlarms --> hashAlarmsByEvalList: '%s'", json.encode(hashAlarmsByEvalList or {}))

	-- Ahora tengo que recorrer el hash (**) y solo obtener la primera interacción de cada Agente y notificar esta.
	for agentId, agents in pairs(hashAlarmsByEvalList) do
		for indicatorId, indicatorInfo in pairs(agents) do
			local totalInteractions = #indicatorInfo.alarms
			local firstAlarm = indicatorInfo.alarms[1]
			if firstAlarm.notificationMessage then
				local channelDef = GetChannelsDefinition()[indicatorInfo.channel:upper()]
				local channelName = channelDef and channelDef.label or indicatorInfo.channel
				-- Aca tengo que reemplazar información del mensaje, la cantidad de interacciones y el canal
				firstAlarm.notificationMessage = firstAlarm.notificationMessage:format(totalInteractions, channelName)
			end
--LogDebug("--> Notification Alarm for agentId: '%s', totalInteractions: %s, alarm: %s", agentId, totalInteractions, json.encode(firstAlarm))
			firstAlarm.values.current_value = totalInteractions
			firstAlarm.values.current_value_source = totalInteractions
			table.insert(alarmsResult, firstAlarm)
		end
	end
--LogDebug("-e---------ProcessMembersConversationsAlarms --> alarmsResult: '%s'", json.encode(alarmsResult or {}))
end

--[[
	Funcion global invocada para procesar y evaluar las alarmas de los MEMBERS, y que las mismas sean notificadas al cliente.
]]--
function ProcessAllMembersAlarms (info, alarmsResult, panelResult, campaignResults)
	local vcc = inconcert.authUser.virtualContactCenter
	local fullCampaignId = utils.AppendVcc(info.campaignId, vcc)
	local fullUserId = utils.AppendVcc(inconcert.authUser.userName, vcc)

	--LogInfo("[SUPERVISOR:ProcessAllMembersAlarms] Processing alarms for '%s' campaign, user '%s'.", fullCampaignId, fullUserId)

	--LogDebug("[SUPERVISOR:ProcessAllMembersAlarms] 1 campaign: '%s', info.alarms '%s'.", fullCampaignId, json.encode(info.alarms or {}))
	--LogDebug("[SUPERVISOR:ProcessAllMembersAlarms] 2 campaign: '%s', panelResult: %s", fullCampaignId, json.encode(panelResult or {}))
	--LogDebug("[SUPERVISOR:ProcessAllMembersAlarms] 3 campaign: '%s', campaignResults: %s", fullCampaignId, json.encode(campaignResults or {}))

	local function HasKickouts()
		--Si el supervisor no quiere ver las alarmas de kickouts, directamente devolvemos false
		if not info.showKickouts then
			--LogDebug("[SUPERVISOR:ProcessAllMembersAlarms] HasKickouts return false because user '%s' dont want to see kickouts alarms", fullUserId)
			return false
		end
		for agent in pairs(campaignResults.agents) do
			if campaignResults.agents[agent].kickouts and info.showKickouts then
				--LogDebug("[SUPERVISOR:ProcessAllMembersAlarms] HasKickouts return true")
				return true
			end
		end
		--LogDebug("[SUPERVISOR:ProcessAllMembersAlarms] HasKickouts return false")
		return false
	end

	local function CanProcessAlarms ()
		if utils.TableIsEmpty(info.alarms) then
		-- 1 No tiene alarmas configuradas
			--LogInfo("[SUPERVISOR:ProcessAllMembersAlarms] Not configured alarms on campaign '%s'.", fullCampaignId)
			return false
		end
		-- 2 Si, puede procesar las alarmas
		return true
	end

	if CanProcessAlarms() then
		-- Se cargan las alarmas
		local alarmsScope = {
			campaign_id = fullCampaignId,
			account_id  = info.account or "",
			al_id       = info.attention_level or ""
		}
		-- Asigno los valores de la campana que no lleguen en el campaignResults desde la supervisorshared (timezone)
		SetCampaignInfo(info.campaignId, vcc, campaignResults)

		local campaignTimezone = campaignResults and utils.GetTimeZoneInstance():GetEnvironmentFormat(campaignResults.timezone)
		--LogInfo("[SUPERVISOR:ProcessAllMembersAlarms] Processing Members Alarms.")
		ProcessMembersConversationsAlarms(campaignResults.agents, info.alarms, alarmsResult, alarmsScope, campaignTimezone)
	end

	if HasKickouts() then
		--LogInfo("[SUPERVISOR:ProcessAllMembersAlarms] Processing Members Kickouts.")
		-- kickout alarms
		for agent in pairs(campaignResults.agents) do
			if campaignResults.agents[agent].kickouts and info.showKickouts then
				for kickout in pairs(campaignResults.agents[agent].kickouts or {}) do
					local jKickout = json.decode(campaignResults.agents[agent].kickouts[kickout])
					local kickoutAlarm = 
					{
						indicator_id = "kickout",
						member_id = campaignResults.agents[agent].ID,
						userid = jKickout.UserID,
						timestamp = jKickout.timestamp,
						account_id = info.account or "",
						campaign_id = fullCampaignId,
						agentId = campaignResults.agents[agent].ID
					}
					table.insert(alarmsResult, kickoutAlarm)
				end
			end
		end
	elseif not CanProcessAlarms() then
		return false
	end

	--LogInfo("[SUPERVISOR:ProcessAllMembersAlarms] alarmsResult '%s'", json.encode(alarmsResult))

	return true
end

function HasInteractionTimes (info)
	if utils.TableIsEmpty(info.alarms) then
		return false
	end
	local interactionTimesIndicators = {
		"conversation_queued_time",
		"conversation_taken_time",
		"conversation_waiting_time",
		"conversation_wrapup_time",
		"conversation_abandoned_time",
		"conversation_open_time"
	}
	for _, indicatorId in pairs(interactionTimesIndicators) do
		if info.alarms[indicatorId] and info.alarms[indicatorId].enabled then
			return true
		end
	end
	return false
end

--[[
	Funcion que asigna los valores de la campana que no lleguen en el objeto resultante desde supervisorshared
	Ej: timezone de la campana
]]--
function SetCampaignInfo (campaignIdWithVcc, vcc, campaignInfo, keyspace)
	assert(type(campaignIdWithVcc) == "string")
	assert(type(vcc) == "string")
	assert(type(campaignInfo) == "table")

	--LogDebug("[SUPERVISOR:SetCampaignInfo] 1 (entra) campaignInfo.campaignIdWithoutVcc: %s", campaignInfo.campaignIdWithoutVcc)
	--LogDebug("[SUPERVISOR:SetCampaignInfo] 2 (entra) campaignInfo.campaignIdWithVcc: %s", campaignInfo.campaignIdWithVcc)
	--LogDebug("[SUPERVISOR:SetCampaignInfo] 3 (entra) campaignInfo.timezone: %s", campaignInfo.timezone)
	--LogDebug("[SUPERVISOR:SetCampaignInfo] 4 (entra) campaignInfo.vcc: %s", campaignInfo.vcc)

	local campaignIdWithoutVcc = utils.GetId(campaignIdWithVcc)
	campaignIdWithVcc = utils.AppendVcc(campaignIdWithoutVcc, vcc)

	if not campaignInfo.vcc then
		campaignInfo.vcc = vcc
	end
	if not campaignInfo.campaignIdWithoutVcc then
		campaignInfo.campaignIdWithoutVcc = campaignIdWithoutVcc
	end
	if not campaignInfo.campaignIdWithVcc then
		campaignInfo.campaignIdWithVcc = campaignIdWithVcc
	end
	if not campaignInfo.timezone then
		local dbCampaign = utils.GetCachedCampaignInfo(campaignIdWithVcc, keyspace)
		if dbCampaign then
			campaignInfo.timezone = dbCampaign.ValidationTimeZone
		end
	end

	--LogDebug("[SUPERVISOR:SetCampaignInfo] 4 (sale) campaignInfo.campaignIdWithoutVcc: %s", campaignInfo.campaignIdWithoutVcc)
	--LogDebug("[SUPERVISOR:SetCampaignInfo] 5 (sale) campaignInfo.campaignIdWithVcc: %s", campaignInfo.campaignIdWithVcc)
	--LogDebug("[SUPERVISOR:SetCampaignInfo] 6 (sale) campaignInfo.timezone: %s", campaignInfo.timezone)
	--LogDebug("[SUPERVISOR:SetCampaignInfo] 7 (sale) campaignInfo.vcc: %s", campaignInfo.vcc)
end