module(..., package.seeall)

local utils = require "shared.apps.utils"
local m_datasource

function GetName()
	return "Users"
end

function GetDefinition (datasource)
	m_datasource = datasource
	return _M
end

function Get (vcc, userId, keyspace, columns)
	vcc = vcc or inconcert.authUser.virtualContactCenter
	local simpleUserId = utils.GetId(userId)
	local fullUserId = utils.AppendVcc(simpleUserId, vcc)

	local user = utils.GetAllColumns {
		keyspace     = keyspace or "Agent",
		columnFamily = "Users",
		key          = fullUserId,
		columns      = columns -- no requerido
	}
	return user
end

function GetSupervisorWorkgroups (vcc, userId, keyspace)
	local user = utils.GetAllColumns {
		keyspace = keyspace or "Agent",
		columnFamily = "Users",
		key = userId.."@"..vcc,
		columns = {"SupervisorWorkgroups"},
	}
	return user and user.SupervisorWorkgroups or {}
end

function BelongToCampaign (fullCampaignId, fullUserId, keyspace)
	local closeKeyspace = false
	if not keyspace then
		keyspace = m_datasource.cassandra:CassandraConnection("Agent")
		closeKeyspace = true
	end

	local res, column = pcall(keyspace.GetColumn, keyspace, fullCampaignId, "CampaignMembers", fullUserId)
	if closeKeyspace then
		keyspace:Close()
	end

	if res then
		local belong = column and column.value or false
		--LogDebug("[SUPERVISOR.USERS:BelongToCampaign] CampaignMembers -> fullCampaignId:'%s' fullUserId: '%s' result: '%s'.", fullCampaignId, fullUserId, belong)
		return belong
	end

	return false
end
