/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
  strict:false, undef:false, unused:false, curly:true, browser:true, indent:false, maxerr:50, quotmark:false, devel:true */

/*global $, jQuery
inconcert EJSC AsyncCall dhtmlXComboFromSelect
prevPageButton nextPageButton UpdatePaginationState UpdateSerieSelectorState
CreateEJSSerie privateSelf Redraw, EJSC, privateSelf
*/

var inconcert = inconcert || {};

//
// Codigos de errores definidos
//
var INCONCERTCHART_NO_DIV_ERROR							= 1;
var INCONCERTCHART_INCORRECT_DIV_ERROR					= 2;
var INCONCERTCHART_INCORRECT_SERIE_ERROR				= 3;
var INCONCERTCHART_SOURCE_TYPE_ERROR					= 4;
var INCONCERTCHART_SERVER_RESPONSE_ERROR				= 5;
var INCONCERTCHART_SERVER_DATA_RESPONSE_ERROR			= 6;
var INCONCERTCHART_NO_MAIN_SERIE_ERROR					= 7;
var INCONCERTCHART_SECONDARY_SERIE_ALREADY_EXISTS_ERROR = 8;
var INCONCERTCHART_SECONDARY_SERIE_NOT_EXISTS_ERROR		= 9;
var INCONCERTCHART_INVALID_PARAMETER_TYPE				= 10;
var INCONCERTCHART_MALFORMED_PARAMETER					= 11;
var INCONCERTCHART_INVALID_FILTER_VALUE					= 12;
var INCONCERTCHART_CREDENTIALS_ARE_NO_LONGER_VALID		= 13;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Estas funciones y objetos seran utilizados como herramienta para los llamados asincronicos que se necesiten
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
var AsyncCallId;
var AsyncCallObjects;


//Esta funcion recibe una funcion a ejecutar y un tiempo en milisegundos
//Luego de pasado ese tiempo, ejecutara la funcion asincronicamente
function AsyncCall(funcion, time) {
	if (AsyncCallId === undefined) {
		AsyncCallId = 0;
	}
	AsyncCallId++;

	if (AsyncCallObjects === undefined) {
		AsyncCallObjects = {}; 
	}
	AsyncCallObjects[AsyncCallId] = funcion;

	setTimeout('AsyncCallExecute(' + AsyncCallId + ')', time);
}
//Esta funcion deberia ser privada... pero si fuera asi no se podria ejecutar con setTimeout
function AsyncCallExecute(i) {
	if (typeof (AsyncCallObjects[i]) === "function") {
		AsyncCallObjects[i]();
		delete AsyncCallObjects[i];
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////


//Constructor de inconcert.Chart
// options = {
//		div : "divContenedor", //obligatorio
//		Series : {
//			Main : {
//				Properties : {},
//				DataSource : {}
//			},
//			Secondary : {
//				Serie1 : { //Reemplazar Serie1 por el nombre de la serie
//					DataSource : {},
//					Properties : {}
//				},
//				Serie2: {...},
//			},
//			Properties : {}
//		},
//		Filters : {
//			TopLeft : {
//				Properties : {},
//				FilterValues : [{name:"name", value:"value"}, ...],
//			},
//			TopRight : {IDEM},
//			BottomLeft : {IDEM},
//			BottomRight : {IDEM},
//		},
//		Pagination : {},
//		ChartProperties : {},
//		OnError			: bool function (errorCode, description), //(ver notas del manejo de errores al final)
//}
inconcert.Chart = function (options) {
	var self = this;

	if (!options) {
		alert ("No options where specified");
		throw "No options where specified";
	}

	//Esta funcion asignara al objeto dest, las propiedades del objeto orig que esten en propsArray y que no sean undefined
	function assignProperties (dest, orig, propsArray) {
		for (var i = 0; i < propsArray.length; i++) {
			if (orig[propsArray[i]] !== undefined) {
				dest[propsArray[i]] = orig[propsArray[i]];
			}
		}
	}

	//
	// Funciones de error
	//
	if (typeof (options.OnError) === "function") {
		var OnErrorFunction = options.OnError;
	}
	function handleError (erroCode, descr) {
		if (!(OnErrorFunction && OnErrorFunction (erroCode, descr) === false)) {
			alert ("ErrorCode: " + erroCode + " \r\nDescription: " + descr);
			throw "ErrorCode: " + erroCode + " \r\nDescription: " + descr;
		}
	}

	// Implementacion de un Map Objeto-Objeto ya que esto no se puede
	// hacer directo con los arrays ni objetos de javascript
	function Object2ObjectMap () {
		var arr = [];

		this.Set = function (key, value) {
			for (var i = 0; i < arr.length; i++) {
				if (arr[i].key === key) {
					//estaba ya ingresado, solo seteo
					arr[i].value = value;
					return;
				}
			}
			//No existia antes, lo ingreso al array
			arr.push ({key: key, value: value});
		};

		this.UnSet = function (key) {
			for (var i = 0; i < arr.length; i++) {
				if (arr[i].key === key) {
					arr.splice (i, 1);
					return;
				}
			}
		};

		this.Get = function (key) {
			for (var i = 0; i < arr.length; i++) {
				if (arr[i].key === key) {
					return arr[i].value;
				}
			}
			//si no existe no devuelvo nada (ie. undefined)
		};
	}

	//
	// Funcion para agregar tooltips
	//

	function AddTooltip (selector, text) {
		$(selector).tooltip({
			track: false,
			bodyHandler: function() {
				return text;
			},
			delay: 0,
			fade: 150,
			showURL: false,
			opacity: 1,
			fixPNG: true,
			showBody: " - ",
			//extraClass: "contextualHelp",
			extraClass: "imageTooltip",
			top: -5,
			left: 10
		});
	}

	//
	// Estructura de divs contenedores del chart
	//
	if (typeof (options.div) !== "string") {
		handleError (INCONCERTCHART_NO_DIV_ERROR, "DIV not specified");
	}
	var panelContainer = $("#" + options.div);
	if (!panelContainer) {
		handleError (INCONCERTCHART_INCORRECT_DIV_ERROR, "Incorrect DIV: " + options.div);
		return;
	}
	panelContainer.addClass("formIndicatorPanelContainerExt");

	function createChartDiv (parent, style, id) {
		if (id === undefined) {
			id = options.div + style;
		}
		parent.append ("<div id = '" + id + "' class = '" + style + "'> </div>");
		return $("#" + id);
	}

	//Esta indentado asi para mostrar la jerarquia que sigue el html generado
	var panelTitle							= createChartDiv (panelContainer, "formIndicatorPanelTitle");
		var panelTitleText					= createChartDiv (panelTitle, "formIndicatorPanelTitleText");
	var panelContent						= createChartDiv (panelContainer, "formIndicatorPanelContent");
		var panelChart						= createChartDiv (panelContent, "indicatorContentPanelChart");
			var chartDraw					= createChartDiv (panelChart, "indicatorChartDraw");
				var chartDrawContainer		= createChartDiv (chartDraw, "indicatorChartDrawContainer");

	//
	//Funcion callback para procesar el evento de click sobre un punto
	//
	function doPointSelection (point, serie) {
		var current_serie = privateSelf.InternalSerie.Get(serie);
		if (current_serie && typeof (current_serie.Properties.onClick) === "function") {
			current_serie.Properties.onClick(point.x, point.y, point.userdata);
		}
	}

	//
	// Objeto chart privado
	//
	var ejsChart = new EJSC.Chart (options.div + "indicatorChartDrawContainer", {
		show_legend: false,
		drawing_message: "<%=_T('Please wait...')%>",
		show_messages: false,
		auto_zoom : "y",
		message_timeouts : {
			progress : 300
		},
		title : "",
		show_titlebar: false,
		axis_left: {
			caption: "",
			size: 0,
			/*grid: {
				color: "#F00",
				opacity: 100,
				show: true
			},*/
			//array onNeedsTicks( float min, float max, EJSC.Chart chart, EJSC.Axis axis)
			onNeedsTicks : options.ChartProperties && options.ChartProperties.yNeedsTicks || null,
			border: {
				show: false
			},
			formatter: new EJSC.StringFormatter({
				onNeedsFormat: options.ChartProperties && options.ChartProperties.yFormatter || null
			})
		},
		axis_bottom: {
			caption: "",
			size: 0,
			/*grid: {
				color: "#F00",
				opacity: 100,
				show: true
			},*/
			//array onNeedsTicks( float min, float max, EJSC.Chart chart, EJSC.Axis axis)
			onNeedsTicks : options.ChartProperties && options.ChartProperties.xNeedsTicks || null,
			border: {
				show: false
			},
			formatter: new EJSC.StringFormatter({
				onNeedsFormat: options.ChartProperties && options.ChartProperties.xFormatter || null
			})
		},
		background: {
			color: "#FFF",
			opacity: 100,
			includeTitle: false
		},
		onDblClickPoint: doPointSelection,
		onAfterBuild : options.onAfterBuild && options.onAfterBuild || null
	});

	//
	// Aca se mantienen los datos privados del objeto
	// Se inicializa con los valores por defecto que debe tener
	// Los que estan inicializados con null son unicamente un placeholder
	// para que se entienda que estaran ahi, mas abajo seran inicializados
	// con sus respectivos constructores y valores default
	//
	var privateSelf = {
		ejsChart : ejsChart,
		Series : {
			Main : null, //Se creara con new PrivateSerie mas abajo
			Secondary : {}, //Sera un hash entre el nombre de la serie y la PrivateSerie
			showSeriesSelector : false,
			title: "",
			normalize: false,
			showDelta: false
		},
		Filters : {
			TopLeft		: null, //Se crearan con new PrivateFilter mas abajo
			TopRight	: null,
			BottomLeft	: null,
			BottomRight	: null
		},
		Pagination : {
			showButtons: false,
			pageSize : 10,
			cache : true
		},
		Properties : {
			canFetchData: true,
			streched: false,
			extended: true,
			title: "",
			footer: "",
			footerExtra: "",
			showButtonExpandPanel: true,
			showButtonClose: false,
			allowZoom : false,
			refreshTime : undefined,
			dateFormat : undefined,
			useUTC : true,
			verticalDateFormat: undefined,
			minExtreme: undefined,
			maxExtreme: undefined
		},
		//Este objeto sera usado como mapa entre una serie de ejsChart y el objeto privateSerie
		//para poder procesar los callback de usuario definidos sobre cada serie
		InternalSerie: new Object2ObjectMap(),
		DestroyChart : function () {
			privateSelf.Properties.refreshTime = 0; //para que no ocurra el refresco asincronico
			$("#" + options.div).html("");
			$("#" + options.div).attr("class", "");
			$("#" + options.div).hide();

			//esto esta comentado asi ya que sino al eliminar una grafica
			//las otras graficas misteriosamente dejan de responder al resize
			//es un bug de ejsCharts?
			//ejsChart.remove();
		},
		FetchData: function (forceUpdate) {
			privateSelf.Series.Main.UpdateSerieData (privateSelf.Series.Main.Data.ActualPage, forceUpdate);
		},
		FetchSecondarySerieData: function (forceUpdate, pageNumber) {
			for (var k in privateSelf.Series.Secondary) {
				if (privateSelf.Series.Secondary.hasOwnProperty(k)) {
					var serie = privateSelf.Series.Secondary[k];
					if (typeof (pageNumber) === "number") {
						serie.UpdateSerieData (pageNumber, forceUpdate);
					}
					else {
						serie.UpdateSerieData (serie.Data.ActualPage, forceUpdate);
					}
				}
			}
		}
	};

	//
	// Objeto que manteniene los datos privados de una serie
	// Se inicializa con los valores por defecto
	//
	function PrivateSerie (serie) {
		var thisSerie = this;

		this.DataSource = {};
		this.Properties = {
			name : " ",
			hintString : "[Y]",
			type : "bar",
			beforeChangeType: undefined,
			visible: true,
			userShowAverage : true,
			showDeleteButton: false,
			beforeDelete: undefined,
			color : "#e05c0e",
			actualColor : "#e05c0e",
			colorEnd : "#adadad",
			blinkTime: 0,
			normalize: false,
			showDelta: false
		};
		this.Data = {
			Pages : [],
			ActualPage : 0,
			TotalPages : 1,
			TotalData : 0
		};

		this.SetDataSource = function (dataSource) {
			assignProperties (thisSerie.DataSource, dataSource, ["source","params"]);
			this.UpdateSerieData (undefined, true);
		};

		//Elimina la serie del chart
		this.Destroy = function () {
			thisSerie.Properties.blinkTime = 0;
			var oldSerie = thisSerie.ejsSerie;
			privateSelf.InternalSerie.UnSet (oldSerie);
			//Antes de remover la serie, misteriosamente tengo que dejarla vacia y hacer reload
			//sino explota cuando esta contiene valores undefined
			ejsChart.clearSelectedPoint();
			oldSerie.getDataHandler ().setArray ([]);
			oldSerie.reload();
			ejsChart.removeSeries (oldSerie, true);

			//Elimino el average si existia
			if (thisSerie.ejsAverageSerie) {
				ejsChart.removeSeries(thisSerie.ejsAverageSerie, true);
				thisSerie.ejsAverageSerie = undefined;
			}
			delete privateSelf.Series.Secondary [thisSerie.Properties.name];
		};

		this.UpdateProperties = function (properties) {

			if (properties.type !== undefined && properties.type !== thisSerie.Properties.type && thisSerie.Properties.type !== "pie" && properties.type !== "pie") {
				//Hay que actualizar el tipo de la serie, es decir, tengo que eliminar la serie vieja y crearla de nuevo

				var oldSerie = thisSerie.ejsSerie;
				var newSerie = CreateEJSSerie (properties.type);
				ejsChart.addSeries (newSerie);
				privateSelf.InternalSerie.UnSet (oldSerie);
				//ingreso la serie en el mapa para poder procesar los eventos de doble click
				privateSelf.InternalSerie.Set (newSerie, thisSerie);
				thisSerie.ejsSerie = newSerie;
				thisSerie.Properties.type = properties.type;
				//Antes de remover la serie, misteriosamente tengo que dejarla vacia y hacer reload
				//sino explota cuando esta contiene valores undefined
				ejsChart.clearSelectedPoint();
				oldSerie.getDataHandler ().setArray ([]);
				oldSerie.reload();
				ejsChart.removeSeries (oldSerie, true);
				thisSerie.UpdateSerieData (thisSerie.Data.ActualPage);
			}

			assignProperties(thisSerie.Properties, properties, [
				"name",
				"visible",
				"drawPoints",
				"lineWidth",
				"onClick",
				"hintString",
				"showAverage",
				"userShowAverage",
				"color",
				"colorEnd",
				"colorPool",
				"showDeleteButton",
				"beforeDelete",
				"beforeChangeType"
			]);

			if (thisSerie.Properties.showAverage && thisSerie.ejsAverageSerie === undefined) {
				//No existia la serie, debo crearla
				thisSerie.ejsAverageSerie = new EJSC.LineSeries(new EJSC.ArrayDataHandler([]), {lineOpacity: 100, lineWidth: 2});
				ejsChart.addSeries (thisSerie.ejsAverageSerie);
				thisSerie.ejsAverageSerie.setColor(thisSerie.Properties.color);
			}
			if (thisSerie.Properties.showAverage === false && thisSerie.ejsAverageSerie !== undefined) {
				//Existia y la quiero quitar
				ejsChart.removeSeries(thisSerie.ejsAverageSerie, true);
				thisSerie.ejsAverageSerie = undefined;
			}

			if (thisSerie.Properties.visible) {
				thisSerie.ejsSerie.show();
			}
			else {
				thisSerie.ejsSerie.hide();
			}

			thisSerie.ejsSerie.hint_string = thisSerie.Properties.hintString;
			thisSerie.ejsSerie.setColor (thisSerie.Properties.color);
			thisSerie.Properties.actualColor = thisSerie.Properties.color;
			if (thisSerie.Properties.colorPool) {
				thisSerie.ejsSerie.defaultColors = thisSerie.Properties.colorPool; 
			}
			thisSerie.ejsSerie.setTitle (thisSerie.Properties.name);
			thisSerie.ejsSerie.drawPoints = thisSerie.Properties.drawPoints || false;
			thisSerie.ejsSerie.lineWidth = thisSerie.Properties.lineWidth || thisSerie.ejsSerie.lineWidth;

			thisSerie.ejsSerie.reload ();

			if (!thisSerie.Properties.blinkTime && properties.blinkTime > 0) {
				//El usuario seteo un blink time asi que inicializo timer
				thisSerie.Properties.blinkTime = properties.blinkTime;
				var blink = function (alert) {
					thisSerie.Properties.actualColor = alert && thisSerie.Properties.blinkTime > 0 ? thisSerie.Properties.colorEnd : thisSerie.Properties.color;
					thisSerie.ejsSerie.setColor(thisSerie.Properties.actualColor);

					if (thisSerie.Properties.blinkTime) {
						AsyncCall (function () {
							if (thisSerie.Properties.blinkTime > 0) {
								blink(!alert);
							}
						}, thisSerie.Properties.blinkTime);
					}
				};

				blink(true);
			}
			if (thisSerie.Properties.blinkTime > 0 && properties.blinkTime === 0) {
				thisSerie.Properties.blinkTime = 0;
			}
		};


		//Dado un tipo de serie, devuelve un nuevo objeto EJSCSerie
		function CreateEJSSerie (type) {
			var ejsSerie;
			if (type === "bar") {
				ejsSerie = new EJSC.BarSeries(new EJSC.ArrayDataHandler([]), {opacity: 75, lineOpacity: 100, lineWidth: 1});
			}
			else if (type === "line") {
				ejsSerie = new EJSC.LineSeries(new EJSC.ArrayDataHandler([]), {lineOpacity: 85, lineWidth: 2});
			}
			else if (type === "pie") {
				ejsSerie = new EJSC.PieSeries(new EJSC.ArrayDataHandler([]), {opacity: 85, position: "left"});
			}
			else {
				handleError (INCONCERTCHART_INCORRECT_SERIE_ERROR, "Serie type '" + type +"' is incorrect");
			}
			ejsSerie.title = thisSerie.Properties.name;
			ejsSerie.hint_string = thisSerie.Properties.hintString;
			ejsSerie.color = thisSerie.Properties.color;
			return ejsSerie;
		}
		if (serie && serie.Properties) {
			assignProperties(thisSerie.Properties, serie.Properties, ["type"]);
		}
		this.ejsSerie = CreateEJSSerie (this.Properties.type);
		ejsChart.addSeries (this.ejsSerie);
		//ingreso la serie en el mapa para poder procesar los eventos de doble click
		privateSelf.InternalSerie.Set(this.ejsSerie, this);

		//Esta funcion actualiza los datos de la serie en funcion del source especificado
		//utiliza el numero de pagina (si no se especifica es 0)
		//forceUpdate es para no utilizar la cache
		this.UpdateSerieData = function (pageNumber, forceUpdate) {
			var source = thisSerie.DataSource.source;

			function DataIsOk (data) {
				if (!data) {
					return false;
				}
				for (var i = 0; i < data.length; i++) {
					if (typeof (data[i][0]) !== "string" && typeof (data[i][0]) !== "number"){
						return false;
					}
					if (typeof(data[i][1]) !== "number" && typeof (data[i][1]) !== "undefined"){
						//Los datos undefined son validos para las graficas de tipo line
						//Cuando no es de tipo line, la funcion AdaptData lo corregira debidamente
						return false;
					}
				}
				return true;
			}

			function ConvertXToDate (data) {
				for (var i = 0; i < data.length; i++) {
					var millisec = Date.parse (data[i][0]);
					if (!isNaN (millisec)) {
						data[i][0] = millisec;
					}
				}
			}
			//No es lo mejor del mundo, pero anda
			function ConvertYToDate (data) {
				for (var i = 0; i < data.length; i++) {
					var millisec = data[i][1] * 10; //El tiempo en tickcounts viene dado en centesimas de segundo por lo tanto lo transformo a milesimas
					if (!isNaN (millisec)) {
						data[i][1] = millisec;
					}
				}
			}

			//Adapta los datos segun las particularidades de cada tipo de serie
			function AdaptData (data) {
				var isNull = true;
				for (var i = 0; i < data.length; i++) {
					if (typeof (data[i][1]) === "number") {
						isNull = false;
						break;
					}
				}
				//Si la serie es toda null, entonces devuelvo un array vacio para que se pueda procesar bien
				if (isNull) {
					return [];
				}

				if (privateSelf.Series.showDelta) {
					//TODO: [[1325161001000, 4], [1325161062000, 6], [1325161121000, 6.5]]
					var newData = [];
					for (var delta_idx = 1; delta_idx < data.length; delta_idx++) {
						var x1 = data[delta_idx - 1][0];
						var y1 = data[delta_idx - 1][1];
						var x2 = data[delta_idx][0];
						var y2 = data[delta_idx][1];

						//se multiplica por 1000 ya que el tiempo esta expresado en milisegundos
						var delta = 1000 * (y2 - y1) / (x2 - x1);
						newData.push([x2, delta]);
					}
					data = newData;
				}

				if (privateSelf.Series.normalize) {
					//Calculo el maximo de la serie
					var max;
					for (var max_idx = 0; max_idx < data.length; max_idx++) {
						if (typeof(max) === "undefined" || data[max_idx][1] > max) {
							max = data[max_idx][1];
						}
					}
					//Normalizo
					if (typeof(max) !== "undefined") {
						for (var n_idx = 0; n_idx < data.length; n_idx++) {
							if (typeof (data[n_idx][1]) === "number") {
								data[n_idx][1] = 100 * data[n_idx][1] / max;
							}
						}
					}
				}

				var ret = [];
				if (thisSerie.Properties.type === "pie") {
					//en caso de que la grafica sea de tipo pie hay que reacomodar los datos
					//es decir invertir el X con el Y
					//ademas no se permiten los undefined o null
					for (var pie_idx = 0; pie_idx < data.length; pie_idx++) {
						if (typeof (data[pie_idx][1]) === "number") {
							ret.push ([data[pie_idx][1], data[pie_idx][0], data[pie_idx][2]]);
						}
					}
					return ret;
				}
				else if (thisSerie.Properties.type === "bar") {
					//en caso que me venga undefined o null del server
					//quito el punto ya que este tipo de grafica
					//no soporta que los datos sean undefined o null
					for (var bar_idx = 0; bar_idx < data.length; bar_idx++) {
						if (typeof (data[bar_idx][1]) === "number") {
							ret.push ([data[bar_idx][0], data[bar_idx][1], data[bar_idx][2]]);
						}
					}
					return ret;
				}
				else {
					return data;
				}
			}

			if (forceUpdate) {
				//Limpio la cache!
				thisSerie.Data.Pages = [];
			}

			ejsChart.clearSelectedPoint();

			//Si el origen de datos es una URL y esta habilitada la cache y ademas ya tengo los datos => uso la cache
			if (typeof (source) === "string" && privateSelf.Pagination.cache && thisSerie.Data.Pages[pageNumber]) {
				thisSerie.ejsSerie.getDataHandler().setArray (AdaptData(thisSerie.Data.Pages[pageNumber]));
				thisSerie.ejsSerie.reload();
				return;
			}

			//Si no puedo usar la cache, voy a buscar al source definido
			if (typeof (source) === "string") {
				//es una URL

				if (privateSelf.Properties.canFetchData === false) {
					//Se deshabilito el fetch de datos desde el server
					return;
				}

				var params = {};
				if (!forceUpdate && thisSerie.QueryId && pageNumber !== undefined) {
					//El server ya tiene guardado en sus sesion que es lo que estoy filtrando
					params.pageNumber = pageNumber;
					params.queryId = thisSerie.QueryId;
				}
				params.pageSize = privateSelf.Pagination.pageSize;

				params.filters = {};
				if (privateSelf.Filters.TopLeft) {
					params.filters [privateSelf.Filters.TopLeft.Properties.name] = privateSelf.Filters.TopLeft.Properties.value;
				}
				if (privateSelf.Filters.TopRight) {
					params.filters [privateSelf.Filters.TopRight.Properties.name] = privateSelf.Filters.TopRight.Properties.value;
				}
				if (privateSelf.Filters.BottomLeft) {
					params.filters [privateSelf.Filters.BottomLeft.Properties.name] = privateSelf.Filters.BottomLeft.Properties.value;
				}
				if (privateSelf.Filters.BottomRight) {
					params.filters [privateSelf.Filters.BottomRight.Properties.name] = privateSelf.Filters.BottomRight.Properties.value;
				}
				params.params = thisSerie.DataSource.params;
				params.globalParams = privateSelf.Series.globalParams;

				//Ejecuto el callback del usuario si corresponde
				if (thisSerie === privateSelf.Series.Main && typeof (privateSelf.Series.beforeSendRequestUserCallback) === "function") {
					privateSelf.Series.beforeSendRequestUserCallback (forceUpdate, params.pageNumber);
				}

				jQuery.ajax({
					url: source,
					type: 'POST',
					data: JSON.stringify (params),
					dataType: "json",
					async: true,
					contentType: "application/json",
					success: function (response) {
						if (!response) {
							handleError (INCONCERTCHART_SERVER_RESPONSE_ERROR, "<%=_T('Server response is null')%>");
							return;
						}
						thisSerie.QueryId = response.queryId;

						if (response.totalPages !== undefined) {
							thisSerie.Data.TotalPages = response.totalPages;
						}
						else {
							thisSerie.Data.TotalPages = 1;
						}
						if (!response.data) {
							handleError (INCONCERTCHART_SERVER_RESPONSE_ERROR, "<%=_T('Server do not send data')%>");
							return;
						}
						if (response.totalData !== undefined) {
							thisSerie.Data.TotalData = response.totalData;
						}
						else {
							thisSerie.Data.TotalData = response.data.length;
						}
						if (response.pageNumber !== undefined) {
							thisSerie.Data.ActualPage = response.pageNumber;
						}
						else {
							thisSerie.Data.ActualPage = 0;
						}

						if (!DataIsOk(response.data)) {
							handleError(INCONCERTCHART_SERVER_DATA_RESPONSE_ERROR, "<%=_T('Server data response is incorrect')%>");
							return;
						}

						if (typeof (privateSelf.Properties.dateFormat) === "string") {
							ConvertXToDate(response.data);
						}
						if (typeof (privateSelf.Properties.verticalDateFormat) === "string") {
							ConvertYToDate(response.data);
						}
						thisSerie.Data.Pages[thisSerie.Data.ActualPage] = response.data;

						//Seteo los extremos del eje inferior para que se ajusten a los datos recibidos de la serie principal
						if (thisSerie === privateSelf.Series.Main) {
							var extremes = self.Series.Main.GetExtremes ();
							ejsChart.axis_bottom.setExtremes();
							if (typeof (extremes.min) === "number" && typeof (extremes.max) === "number" && extremes.min < extremes.max) {
								ejsChart.axis_bottom.setExtremes(extremes.min, extremes.max);
							}
						}

						thisSerie.ejsSerie.getDataHandler ().setArray (AdaptData(response.data));
						thisSerie.ejsSerie.reload();

						if (thisSerie === privateSelf.Series.Main && typeof (privateSelf.Series.onServerResponseUserCallback) === "function") {
							//Si soy la serie principal y tengo definido un callback, ejecuto dicho callback
							privateSelf.Series.onServerResponseUserCallback (thisSerie.Data.ActualPage, thisSerie.Data.TotalPages, thisSerie.Data.TotalData);
						}
					},
					error : function (error) {
						if (error.status === 12007) {
							handleError (INCONCERTCHART_SERVER_RESPONSE_ERROR, "<%=_T('Unable to communicate with server, please verify server name')%>");
						}
						else if (error.status === 403) {
							handleError (INCONCERTCHART_CREDENTIALS_ARE_NO_LONGER_VALID, "<%=_T('You have been logged out from the server, please try logging in again')%>");
						}
						else {
							handleError (INCONCERTCHART_SERVER_RESPONSE_ERROR, "<%=_T('Unable to communicate with server, please check your network connection')%>");
						}
					}
				});
				return;
			}
			else if (typeof (source) === "object") {
				//es un array
				if (!DataIsOk(source)) {
					handleError(INCONCERTCHART_SERVER_DATA_RESPONSE_ERROR, "<%=_T('Data source array is incorrect')%>");
					return;
				}
				thisSerie.Data.Pages[0] = source;
			}
			else if (typeof (source) === "function") {
				//es una funcion.. que noticia
				var datos = source (thisSerie.DataSource.params);
				if (!DataIsOk(datos)) {
					handleError(INCONCERTCHART_SERVER_DATA_RESPONSE_ERROR, "<%=_T('Function return is incorrect')%>");
					return;
				}
				thisSerie.Data.Pages[0] = datos;
			}
			else {
				handleError (INCONCERTCHART_SOURCE_TYPE_ERROR, "Source type '" + typeof (source) +"' not supported");
				return;
			}

			if (typeof (privateSelf.Properties.dateFormat) === "string") {
				ConvertXToDate(thisSerie.Data.Pages[0]);
			}
			if (typeof (privateSelf.Properties.verticalDateFormat) === "string") {
				ConvertYToDate(thisSerie.Data.Pages[0]);
			}
			thisSerie.ejsSerie.getDataHandler ().setArray (AdaptData(thisSerie.Data.Pages[0]));
			thisSerie.ejsSerie.reload();

		};

		if (serie && serie.DataSource) {
			this.SetDataSource (serie.DataSource);
		}
		if (serie && serie.Properties) {
			this.UpdateProperties(serie.Properties);
		}
	}

	//
	// Objeto que mantiene los datos privados de un filtro
	// Se inicializa con los valores por defecto
	//
	function PrivateFilter (div, position, filter) {
		var thisFilter = this;
		this.Properties = {
			name : position,
			value : undefined,
			title : "",
			onClick : undefined,
			type : "links"
		};
		this.Values = []; //Array de {name, value}

		function FilterChanges (name, value) {
			var onClick = thisFilter.Properties.onClick;
			if (typeof (onClick) === "function" && onClick (name, value) === false) {
				return false; //El usuario cancelo
			}
			thisFilter.Properties.value = value;
			privateSelf.FetchData(true);
			Redraw();

			return false;
		}

		function Redraw () {

			$("#" + div).html("");
			if (thisFilter.Properties.title !== "") {
				$("#" + div).append("<label>" + thisFilter.Properties.title + ": </label>");
			}
			if (thisFilter.Properties.type === "links") {
				//Creo un div nuevo contenedor de los links
				var divLinkContainer = div + "LinkContainer";
				$("#" + div).append('<div id="' + divLinkContainer + '" class="chartFilterCriteriaContainer"></div>');

				var attachLinkCallback = function (name, value) {
					return function () {
						return FilterChanges (name, value);
					};
				};
				for (var i = 0; i < thisFilter.Values.length; i++) {
					var clase = "chartFilterCriteria";
					if (thisFilter.Values[i].value === thisFilter.Properties.value) {
						clase += " chartFilterCriteriaSelected";
					}
					$("#" + divLinkContainer).append("<a class='" + clase + "'id='" + div + thisFilter.Values[i].value + "lnk'>" + thisFilter.Values[i].name + "</a> ");
					$("#" + div + thisFilter.Values[i].value + "lnk").click (attachLinkCallback(thisFilter.Values[i].name, thisFilter.Values[i].value));
				}
			}
			else if (thisFilter.Properties.type === "combo") {
				var html = '<select class="formGroupSelect" style="width:150px;" id="' + div + 'Combo">';
				//Ingreso las opciones del filtro
				thisFilter.Values.forEach(function(filter_value){
					html += '<option value="' + filter_value.value + '">' + filter_value.name + '</option>';
				});
				html += "</select>";
				$("#" + div).append(html);

				var dhtmlxCombo = dhtmlXComboFromSelect(div + "Combo");
				dhtmlxCombo.DOMelem.id = div + "Combo";
				dhtmlxCombo.readonly(true);
				dhtmlxCombo.selectOption(dhtmlxCombo.getIndexByValue(thisFilter.Properties.value));

				dhtmlxCombo.attachEvent("onChange", function () {
					var name = dhtmlxCombo.getSelectedText();
					var value = dhtmlxCombo.getSelectedValue();
					return FilterChanges (name, value);
				});
			}
			else {
				handleError(INCONCERTCHART_INVALID_PARAMETER_TYPE, "Invalid filter type '" + thisFilter.Properties.type + "'");
			}
		}

		this.UpdateProperties = function (props) {
			if (!props) {
				handleError (INCONCERTCHART_MALFORMED_PARAMETER, "No properties where specified");
				return;
			}
			assignProperties (thisFilter.Properties, props, ["name","title","onClick", "type"]);
			Redraw();
		};

		this.SetActualValue = function (value) {
			for (var i = 0; i < thisFilter.Values.length; i++) {
				if (thisFilter.Values[i].value === value) {
					FilterChanges (thisFilter.Values[i].name, thisFilter.Values[i].value);
					return;
				}
			}
			handleError (INCONCERTCHART_INVALID_FILTER_VALUE, "Invalid filter value '" + value +"'");
		};

		/**
		Optimizacion:
		Esto podria ser mas optimo y solamente agregar
		directamente al html o al combo sin llamar al redraw!!
		**/
		this.AddFilterValue = function (filter) {
			if (!(filter && typeof (filter.name) === "string" && typeof (filter.value) !== "undefined")) {
				handleError(INCONCERTCHART_MALFORMED_PARAMETER, "Parameter 'filter' is malformed");
				return;
			}
			thisFilter.Values.push (filter);
			Redraw();
		};

		if (filter && filter.Properties) {
			this.UpdateProperties (filter.Properties);
		}
		if (filter && filter.FilterValues) {
			for (var i = 0; i < filter.FilterValues.length; i++) {
				var filtro = filter.FilterValues[i];
				if (!(filtro && typeof (filtro.name) === "string" && typeof (filtro.value) === "string")) {
					handleError(INCONCERTCHART_MALFORMED_PARAMETER, "Parameter 'filter.FilterValues' is malformed");
					return;
				}
				thisFilter.Values.push (filtro);
			}
			Redraw();
		}
	}

	//
	// API
	//
	this.Series = {
		Main : {
			/*src = {
				source	: "url" | [[x1,y1], [x2,y2], ...] | function (params), //obligatorio (ver descripcion al final del archivo) (si es funcion debe devolver un array bienformado con los datos actualizados cada vez que es llamado)
				params  : {param1: value1, param2: value2, ...}, //parametros a enviar por JSON a la url, por defecto undefined
			}*/
			SetDataSource		: function (src) {
				if (!privateSelf.Series.Main) {
					handleError (INCONCERTCHART_NO_MAIN_SERIE_ERROR, "There is no main serie");
					return;
				}
				privateSelf.Series.Main.SetDataSource (src);
			},
			GetDataSource		: function () {
				if (!privateSelf.Series.Main) {
					handleError (INCONCERTCHART_NO_MAIN_SERIE_ERROR, "There is no main serie");
					return;
				}
				return privateSelf.Series.Main.DataSource;
			},

			/*prop = {
				type		: "bar" | "pie" | "line", //por defecto "bar"
				visible		: true | false, //por defecto true
				onClick		: function (xValue, yValue, userData), //funcion que se ejecutara cuando se haga click en un valor de la serie, por defecto undefined
				hintString	: "HintString", //template del hint a mostrar cuando el usuario pasar por encima de un punto, se puede usar [x], [y], [chart_title], [series_title]
				showAverage : true | false, //por defecto false
				userShowAverage: true | false, //por defecto true
				color		: "#RRGGBB", //en hexadecimal
				colorEnd	: "#RRGGBB",  //en hexadecimal
				blinkTime   : time //intervalo de tiempo en el cual se alternan los colores color y colorEnd, si es 0, esta desactivado
			}*/
			UpdateProperties	: function (prop) {
				if (!privateSelf.Series.Main) {
					handleError (INCONCERTCHART_NO_MAIN_SERIE_ERROR, "There is no main serie");
					return;
				}
				privateSelf.Series.Main.UpdateProperties(prop);
			},

			//Devuelve el array con los datos que se estan graficando
			GetData				: function () {
				if (!privateSelf.Series.Main) {
					handleError (INCONCERTCHART_NO_MAIN_SERIE_ERROR, "There is no main serie");
					return;
				}
				return privateSelf.Series.Main.Data.Pages[privateSelf.Series.Main.Data.ActualPage];
			},
			GetExtremes			: function () { //Devuelve la pareja {min, max} (segun X) de lo que se esta mostrando
				if (!privateSelf.Series.Main) {
					handleError (INCONCERTCHART_NO_MAIN_SERIE_ERROR, "There is no main serie");
					return;
				}

				var actualPage = privateSelf.Series.Main.Data.ActualPage;
				var datos = privateSelf.Series.Main.Data.Pages[actualPage];

				var min, max;

				if (datos) {
					for (var i = 0; i < datos.length; i++) {
						if ((datos[i][0] < min)) {
							min = datos[i][0];
						}
						if ((datos[i][0] > max)) {
							max = datos[i][0];
						}
					}
				}
				return {min: min, max: max};
			}
		},
		Secondary: {
			AddSerie			: function (name, src, prop) {
				if (typeof (name) !== "string") {
					handleError (INCONCERTCHART_INVALID_PARAMETER_TYPE, "Parameter name must be a string");
					return;
				}
				if (privateSelf.Series.Secondary [name]) {
					handleError (INCONCERTCHART_SECONDARY_SERIE_ALREADY_EXISTS_ERROR, "Secondary serie named '" + name + "' already exists");
					return;
				}

				if (prop && !prop.name) {
					prop.name = name;
				}
				privateSelf.Series.Secondary [name] = new PrivateSerie ({
					DataSource : src,
					Properties : prop
				});
			},
			RemoveSerie			: function (name) {
				if (typeof (name) !== "string") {
					handleError (INCONCERTCHART_INVALID_PARAMETER_TYPE, "Parameter name must be a string");
					return;
				}
				if (!privateSelf.Series.Secondary [name]) {
					return;
				}
				privateSelf.Series.Secondary [name].Destroy ();
			},
			SetDataSource		: function (name, src) {
				if (typeof (name) !== "string") {
					handleError (INCONCERTCHART_INVALID_PARAMETER_TYPE, "Parameter name must be a string");
					return;
				}
				if (!privateSelf.Series.Secondary [name]) {
					handleError (INCONCERTCHART_SECONDARY_SERIE_NOT_EXISTS_ERROR, "Serie named '" + name + "' not exists");
					return;
				}
				privateSelf.Series.Secondary [name].SetDataSource (src);
			},
			UpdateProperties	: function (name, prop)	{
				if (typeof (name) !== "string") {
					handleError (INCONCERTCHART_INVALID_PARAMETER_TYPE, "Parameter name must be a string");
					return;
				}
				if (!privateSelf.Series.Secondary [name]) {
					handleError (INCONCERTCHART_SECONDARY_SERIE_NOT_EXISTS_ERROR, "Serie named '" + name + "' not exists");
					return;
				}
				privateSelf.Series.Secondary [name].UpdateProperties (prop);
			},
			UpdateSecondarySerieData: function (forceUpdate, pageNumber) {
				privateSelf.FetchSecondarySerieData (forceUpdate, pageNumber);
			},
			GetSecondarySeriesNames: function () {
				var result = [];
				for (var name in privateSelf.Series.Secondary) {
					if (privateSelf.Series.Secondary.hasOwnProperty (name)) {
						result.push (name);
					}
				}
				return result;
			},
			GetSecondarySerieProperties: function (name) {
				if (typeof (name) !== "string") {
					handleError (INCONCERTCHART_INVALID_PARAMETER_TYPE, "Parameter name must be a string");
					return;
				}
				if (!privateSelf.Series.Secondary [name]) {
					handleError (INCONCERTCHART_SECONDARY_SERIE_NOT_EXISTS_ERROR, "Serie named '" + name + "' not exists");
					return;
				}
				return privateSelf.Series.Secondary [name].Properties;
			},
			GetSecondarySerieDataSource: function (name) {
				if (typeof (name) !== "string") {
					handleError (INCONCERTCHART_INVALID_PARAMETER_TYPE, "Parameter name must be a string");
					return;
				}
				if (!privateSelf.Series.Secondary [name]) {
					handleError (INCONCERTCHART_SECONDARY_SERIE_NOT_EXISTS_ERROR, "Serie named '" + name + "' not exists");
					return;
				}
				return privateSelf.Series.Secondary [name].DataSource;
			}
		},
		/*prop = {
			showSeriesSelector  : true | false, //default es false
			title				: string, //default es ""
			globalParameters	: {}, //default es undefined
			normalize			: true | false, //default es false
		}*/
		UpdateProperties		: function (prop) {
			if (prop && typeof (prop.showSeriesSelector) === "boolean") {
				privateSelf.Series.showSeriesSelector = prop.showSeriesSelector;
				UpdateSerieSelectorState();
			}
			if (prop && typeof (prop.title) === "string") {
				privateSelf.Series.title = prop.title;
			}
			if (prop && typeof (prop.globalParameters) === "object") {
				privateSelf.Series.globalParams = prop.globalParameters;
			}
			if (prop && typeof (prop.normalize) === "boolean") {
				privateSelf.Series.normalize = prop.normalize;
				privateSelf.FetchData(); //Update de los datos
			}
			if (prop && typeof (prop.showDelta) === "boolean") {
				privateSelf.Series.showDelta = prop.showDelta;
				privateSelf.FetchData(); //Update de los datos
			}
		}
	};

	this.Filters = {
		TopLeft : {
			/*prop = {
				name	: "Range",
				title	: "ej: Range",
				onClick : bool function (name, value) //si devuelve false, no se aplica el filtro
				type	: "combo" | "links"
			}*/
			UpdateProperties : function (prop) {
				privateSelf.Filters.TopLeft.UpdateProperties(prop);
			},
			/*filter = {
				name	: "ej: Week",
				value	: SEMANA
			}*/
			AddFilterValue : function (filterValue) {
				privateSelf.Filters.TopLeft.AddFilterValue (filterValue);
			},
			GetActualValue : function () {
				return privateSelf.Filters.TopLeft.Properties.value;
			},
			SetActualValue : function (value) {
				privateSelf.Filters.TopLeft.SetActualValue (value);
			}
		},
		TopRight : {
			UpdateProperties : function (prop) {
				privateSelf.Filters.TopRight.UpdateProperties(prop);
			},
			AddFilterValue : function (filterValue) {
				privateSelf.Filters.TopRight.AddFilterValue (filterValue);
			},
			GetActualValue : function () {
				return privateSelf.Filters.TopRight.Properties.value;
			},
			SetActualValue : function (value) {
				privateSelf.Filters.TopRight.SetActualValue (value);
			}
		},
		BottomLeft : {
			UpdateProperties : function (prop) {
				privateSelf.Filters.BottomLeft.UpdateProperties(prop);
			},
			AddFilterValue : function (filterValue) {
				privateSelf.Filters.BottomLeft.AddFilterValue (filterValue);
			},
			GetActualValue : function () {
				return privateSelf.Filters.BottomLeft.Properties.value;
			},
			SetActualValue : function (value) {
				privateSelf.Filters.BottomLeft.SetActualValue (value);
			}
		},
		BottomRight : {
			UpdateProperties : function (prop) {
				privateSelf.Filters.BottomRight.UpdateProperties(prop);
			},
			AddFilterValue : function (filterValue) {
				privateSelf.Filters.BottomRight.AddFilterValue (filterValue);
			},
			GetActualValue : function () {
				return privateSelf.Filters.BottomRight.Properties.value;
			},
			SetActualValue : function (value) {
				privateSelf.Filters.BottomRight.SetActualValue (value);
			}
		}
	};


	var updateChart = function UpdateChart () { //Esta es privada
		if (privateSelf.Properties.refreshTime > 0) {
			//Solo actua cuando el tiempo de refresco es > 0

			privateSelf.FetchData();

			//Seteo nuevamente el timer ya que sino no arranca solo
			AsyncCall (UpdateChart, privateSelf.Properties.refreshTime);
		}
	};

	/*properties = {
		canFetchData	: true | false, //si esta en false, no se van a buscar datos al server, por defecto es true
		streched		: true | false, //muestra la grafica ocupando el 100% del div (sin ejes ni titulo) por defecto false
		extended		: true | false, //muestra la grafica extendida, sino se muestra utilizando la mitad del espacio, por defecto es true. El valor de streched tiene mas prioridad
		title			: string, //por defecto vacio
		showButtonExpandPanel: true | false, //por defecto true
		showButtonClose	: true | false, //por defecto false,
		onBeforeClose	: bool function (), //si devuelve false, se cancela el close
		onExpandPanel	: function (bool expand),
		footer			: string, //por defecto vacio
		footerExtra		: string, //por defecto vacio
		allowZoom		: true | false, //por defecto false
		refreshTime		: milliseconds, // tiempo de refresco de la grafica en milisegundos
		dateFormat		: "formato de la fecha, ej: DD/MM/YYYY", //por defecto undefined
		verticalDateFormat: "formato de la fecha del eje vertical, ej: DD/MM/YYYY", //por defecto undefined
		minExtreme		: min, //extremo minimo del eje Y (por defecto es undefined)
		maxExtreme		: max, //extremo maximo del eje Y (por defecto es undefined)
	}*/
	this.UpdateProperties = function (properties) {
		assignProperties (privateSelf.Properties, properties, ["canFetchData","title", "footer", "footerExtra", "allowZoom", "dateFormat", "verticalDateFormat", "useUTC", "minExtreme", "maxExtreme", "streched", "extended", "showButtonExpandPanel", "showButtonClose"]);

		if (privateSelf.Properties.streched === true) {
			ejsChart.axis_left.hide();
			ejsChart.axis_bottom.hide();
		}
		else {
			ejsChart.axis_left.show();
			ejsChart.axis_bottom.show();
		}

		panelTitleText.html (privateSelf.Properties.title);
		privateSelf.ejsChart.setTitle(privateSelf.Properties.title);

		ejsChart.allow_zoom = privateSelf.Properties.allowZoom;

		if (typeof (privateSelf.Properties.minExtreme) === "number" || typeof (privateSelf.Properties.maxExtreme) === "number") {
			ejsChart.axis_left.setExtremes(privateSelf.Properties.minExtreme, privateSelf.Properties.maxExtreme);
		}
		else {
			ejsChart.axis_left.setExtremes(undefined, undefined);
		}

		if (typeof (privateSelf.Properties.dateFormat) === "string") {
			ejsChart.axis_bottom.formatter = new EJSC.DateFormatter({format_string: privateSelf.Properties.dateFormat, useUTC: privateSelf.Properties.useUTC});
		}

		if (typeof (privateSelf.Properties.verticalDateFormat) === "string") {
			ejsChart.axis_left.formatter = new EJSC.DateFormatter({format_string: privateSelf.Properties.verticalDateFormat, useUTC: privateSelf.Properties.useUTC});
		}

		if (typeof (properties.refreshTime) === "number") {
			if (privateSelf.Properties.refreshTime === undefined || privateSelf.Properties.refreshTime === 0) {
				//Inicializo el timer ya que antes no estaba corriendo
				AsyncCall (updateChart, properties.refreshTime);
			}
			privateSelf.Properties.refreshTime = properties.refreshTime;
		}

		if (typeof (properties.onBeforeClose) === "function") {
			privateSelf.Properties.onBeforeClose = properties.onBeforeClose;
		}

		if (typeof (properties.onExpandPanel) === "function") {
			privateSelf.Properties.onExpandPanel = properties.onExpandPanel;
		}
		if (properties.canFetchData === true) {
			privateSelf.FetchData ();
		}

		UpdateSerieSelectorState ();
	};

	this.Redraw = function () {
		ejsChart.redraw (true);
	};

	this.GetContainerDiv = function () {
		return options.div;
	};

	//Destruye la grafica y vacia el div contenedor
	this.DestroyChart = function () {
		privateSelf.DestroyChart();
	};

	//
	// Funciones de actualizacion de estado de html
	//

	//Actualiza el estado de la leyenda (ConfigContentCompare)
	function UpdateSerieSelectorState() {

		var drawContainer = $("#" + options.div + "indicatorChartDrawContainer");
		var ejschartDiv = drawContainer.find(".ejsc-chart");
		//Actualizo la leyenda cuando el tipo de la serie principal es pie
		if (privateSelf.Series.Main && privateSelf.Series.Main.Properties.type === "pie") {
			drawContainer.addClass("indicatorChartTypeCake");

			var pieLegend = ejschartDiv.find (".indicatorChartCakeReferences");
			var idLegend = options.div + "indicatorChartCakeReferences";

			if (pieLegend.length === 0) { //si no existia lo creo
				pieLegend = createChartDiv (ejschartDiv, "indicatorChartCakeReferences", idLegend);
			}
			pieLegend.html("");
			var points = privateSelf.Series.Main.ejsSerie.getPoints();
			for (var i = 0; i < points.length; i++) {
				//Reemplazo todo lo que no sea letras y numeros por un "_"
				var labelID = points[i].label.replace(/[^A-Za-z0-9]/g, "_"); 
				var pointLegend = createChartDiv (pieLegend, "indicatorChartPointLegend", options.div + "indicatorChartPointLegend" + labelID);
				pointLegend.append ('<img class="indicatorChartSerieReference" src="/inconcert/shared/inconcertcharts/images/blank.gif" style="background-color:' + points[i].__color + '" />');
				pointLegend.append ('<span>' + points[i].label + '</span>');
			}
			privateSelf.ejsChart.axis_bottom.hide();
			privateSelf.ejsChart.axis_left.hide();
		}
		else {
			//Si existe, elimino el div de la leyenda de pie
			ejschartDiv.find (".indicatorChartCakeReferences").remove();
			drawContainer.removeClass("indicatorChartTypeCake");
			privateSelf.ejsChart.axis_bottom.show();
			privateSelf.ejsChart.axis_left.show();
		}

	}

	//
	// Inicializo el chart segun la configuracion pasada en options
	//

	//Properties options
	if (options.ChartProperties) {
		this.UpdateProperties(options.ChartProperties);
	}

	//Main Serie
	if (typeof (options.Series) !== "undefined" && typeof (options.Series.Main) !== "undefined") {
		privateSelf.Series.Main = new PrivateSerie (options.Series.Main);
	}
	else { //inicializo con los valores default
		privateSelf.Series.Main = new PrivateSerie ();
	}

	//Secondary Series
	if (options.Series && options.Series.Secondary) {
		for (var serieName in options.Series.Secondary) {
			if (options.Series.Secondary.hasOwnProperty(serieName)) {
				var initOptions = options.Series.Secondary[serieName];

				if (!initOptions.Properties) {
					initOptions.Properties = {
						name: serieName
					};
				}
				else {
					initOptions.Properties.name = serieName;
				}

				privateSelf.Series.Secondary[serieName] = new PrivateSerie (initOptions);
			}
		}
	}

	//Series Properties
	if (options.Series && options.Series.Properties) {
		this.Series.UpdateProperties(options.Series.Properties);
	}

	//////////////////////////////////////////////////////////////////

	UpdateSerieSelectorState();
};



//JSON que envia el control hacia el server la primera vez que quiere paginado, o siempre si no quiere paginado:
// {
//	pageSize : pageSize, //Si es undefined, no se realiza paginado del lado del server
//	filters : {
//		name1	: value1,
//		name2	: value2,
//		...
//	}
//	params : {
//		PARAMETROS DEL USUARIO SEGUN LA SERIE QUE SE QUIERA VER
//	},
//	globalParams : {

//		PARAMETROS GLOBALES DEL CHART
//	}
// }

//JSON que envia el control hacia el server cuando ya setupeo el paginado y quiere pedir otra pagina:
// {
//	pageNumber : numeroPagina,
//	queryId : GUID, //esto lo devuelve el server
//	filters : {
//		name1  : value1,
//		name2  : value2,
//		...
//	},
//	params : {
//		PARAMETROS DEL USUARIO SEGUN LA SERIE QUE SE QUIERA VER
//	},
//	globalParams : {
//		PARAMETROS GLOBALES DEL CHART
//	}
// }

//JSON que debe devolver el server:
//{
//	pageNumber	: numeroPagina, //opcional, se asume 0 por defecto
//	totalPages	: totalDePaginas, //opcional (se asume 1 por defecto)
//	totalData	: totalDePuntos, //total de punto de todas las paginas sumadas (opcional, se asume 1 pagina, por lo tanto el total es lo que esta en data unicamente)
//	queryId		: GUID, //opcional (se asume undefined por defecto, o sea, no se pagina)
//	data		: [
//					[X, Y, UserData], // "X" puede ser texto, fecha o numerico, "Y" solo numerico. UserData es opcional y puede ser cualquier cosa JSONizable
//					...
//				]
//}

//Notas:	Cuando ocurra un error, se llamara a la funcion onError especificada en el constructor, pasandole como parametro el codigo de error y una descripcion del mismo.
//			Si no esta especificada una funcion, se lanzara una excepcion al producirse cualquier error. La funcion debe devolver false para el codigo de error en cuestion si no quiere que se propague la excepcion
