/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
  strict:true, undef:true, unused:true, curly:true, browser:true, indent:false, maxerr:50, esversion: 9 */

/* global $, inConcert */

// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.

if (!this.inConcert) {
	this.inConcert = {};
}

/**
 * @namespace
 * @name SupAlarmsNotifications
 */
(function () {

	'use strict';

	const format = inConcert.Utils.format;
	const MAX_NOTIFICATIONS = 50;
	const STORAGE_KEY = "SUPERVISOR_ALARMS";

	/**
	 * Creates a new supervisor alarms notifications instance.
	 *
	 * @author Ellio de Mello (cany@inconcertcc.com)
	 * @constructor
	 * @param params object
	 */
	function SupAlarmsNotifications( params )
	{
		this.m_dashboard = params.dashboard;
		this.container_id = params.container_id;

		this.popup_id = this.container_id + "_alarms_popup";
		this.alarms_list_id = this.container_id + "_alarms_list_popup";
		this.close_popup_id = this.container_id + "_close_alarms_popup";

		this.live_alarms = {}; // hash con la lista de alarmas que se encuentran realtime.
		this.historical_alarms = []; // Un array que contiene el historico de alarmas (aca se encuentran tanto las live como las viejas), el tope maximo del array se define en MAX_NOTIFICATIONS
		this.total_live_alarms = 0;
		this.unread_alarms = 0;
		this.m_queued_alarms = null;

		this.m_operators_labels = {
			EQUAL   : "<%=_T('Equal To')%>",
			DISTINCT: "<%=_T('Distinct From')%>",
			LESS    : "<%=_T('Less Than')%>",
			GREATER : "<%=_T('Greater Than')%>"
		};

		this.m_sections_labels = {
			AGENTS       : "<%=_T('Agents')%>",
		//
			CONVERSATIONS: "<%=_T('Conversations')%>",
		//
			PERCENTAGE   : "<%=_T('%')%>",
		//
			SECOND       : "<%=_T('Seconds')%>",
			MINUTE       : "<%=_T('Minutes')%>",
			HOUR         : "<%=_T('Hours')%>",
			DAY          : "<%=_T('Days')%>",
			MONTH        : "<%=_T('Months')%>"
		};

		//this.__clean_up_storage();
		this.__intialize();
	}

	if (typeof inConcert.sup_alarms_notifications !== "function") {
		inConcert.sup_alarms_notifications = SupAlarmsNotifications;
	}

	SupAlarmsNotifications.prototype.__intialize = function()
	{
		this.__build_markup();
		this.__set_events();
		this.__load_historical_alarms();
	};

	/*
		Retorna la defincion de indicadores que existen server side.
	*/
	SupAlarmsNotifications.prototype.__get_system_indicators = function()
	{
		return this.m_dashboard.get_system_indicators();
	};

	SupAlarmsNotifications.prototype.__get_indicator = function(id)
	{
		var indicators = this.__get_system_indicators();
		if (!indicators) {
			return;
		}
		return indicators.list[id];
	};

	SupAlarmsNotifications.prototype.__get_operator_label = function(operator)
	{
		return this.m_operators_labels[operator] || "";
	};

	SupAlarmsNotifications.prototype.__load_historical_alarms = function()
	{
		var self = this;
		var user_id = inConcert.app.getCompleteId();
		var stored_alarms = inConcert.Storage.get(STORAGE_KEY);

		if (!stored_alarms || !stored_alarms[user_id]) {
			return;
		}

		var is_live_alarm = false;
		this.historical_alarms = stored_alarms[user_id].historical || [];

		$.each(this.historical_alarms, function(_, alarm) {
			self.__add_alarm(alarm, is_live_alarm);
		});
	};

	SupAlarmsNotifications.prototype.__get_campaign_id_in_full_screen = function()
	{
		return this.m_dashboard.get_campaign_id_in_full_screen();
	};

	SupAlarmsNotifications.prototype.__save_to_storage = function()
	{
		var user_id = inConcert.app.getCompleteId();
		var stored_alarms = inConcert.Storage.get(STORAGE_KEY);

		stored_alarms = stored_alarms || {};
		stored_alarms[user_id] = {
			historical: this.historical_alarms
		};

		inConcert.Storage.put(STORAGE_KEY, stored_alarms);
	};

	/*
		Esta funcion no se invocada desde ningun lado.
		Pero si quiero eliminar el storage para realizar algun tipo de prueba,
		la invoco desde algun evento (Ej click del boton close popup)
	*/
	SupAlarmsNotifications.prototype.__clean_up_storage = function()
	{
		var user_id = inConcert.app.getCompleteId();

		var stored_alarms = {};
		stored_alarms[user_id] = {
			historical: []
		};

		inConcert.Storage.put(STORAGE_KEY, stored_alarms);
	};

	SupAlarmsNotifications.prototype.__build_markup = function()
	{
		if ($("#" + this.popup_id).length > 0) {
			return;
		}
		var $popup = $(
			"<div id='" + this.popup_id + "' class='alarmsPopup' hidden>" +
				'<div class="card-header">' +
					"<span><%=_T('Notifications')%></span>"+
					"<a style='display:none;'><%=_T('Mark as read')%></a>"+
				'</div>'+
				'<div class="card-block ">' +
					"<ul id='" + this.alarms_list_id + "' class='alarmsArea btnStatesInLi'>"+
					"</ul>" +
					'<div class="inlineMessagesContainer messageInfo">'+
						"<%=_T('You have no new notifications.')%>"+
					'</div>'+
				'</div>' +
				//'<div class="internalPopUpButtons d-flex justify-content-end">' +
				//	"<button id='" + this.close_popup_id + "' type='button' class='pageButtonGeneric buttonColorWhite'>" + "<%=_T('Close')%>" + "</button>" +
				//'</div>' +
			"</div>"
		);

		$("#" + this.container_id).append($popup);
	};

	SupAlarmsNotifications.prototype.__mark_alarm_as_read = function(alarm_id)
	{
		var alarm = this.live_alarms[alarm_id];
		if (!alarm) {
			return;
		}
		var $alarm = this.__$find_alarm(alarm);
		if (!$alarm.hasClass("unread")) {
			return;
		}
		$alarm.removeClass("unread");

		if (this.live_alarms[alarm_id]) {
			this.live_alarms[alarm_id].unread = false;
		}
		this.unread_alarms--;
		if (this.unread_alarms < 0) {
			this.unread_alarms = 0;
		}

		if (alarm.indicator_id === "kickout") {
			this.__move_live_alarm_to_historical(alarm_id);
		}

		this.__set_alarms_number();
	};

	SupAlarmsNotifications.prototype.get_totals_alarms = function()
	{
		return {
			total_live: this.total_live_alarms,
			unread    : this.unread_alarms
		};
	};

	SupAlarmsNotifications.prototype.__set_alarms_number = function()
	{
		const $notif_button = $("#sup_toolbar_options [action_id='campaign_alarms']"); // el <li> contenedor
		$notif_button.removeClass("unread");
		// marcar como unread el item de la lista <li>
		if (this.unread_alarms > 0) {
			$notif_button.addClass("unread");
		}

		// el <div> contenedor de los numbers
		const $alarm_number_container = $notif_button.find("[alarm_number_container]");
		if (this.total_live_alarms > 0) {
			$alarm_number_container.show();
		}
		else {
			$alarm_number_container.hide();
		}

		// unread number
		const $alarm_unread_number = $notif_button.find("[alarm_unread_number]");
		$alarm_unread_number
			.attr("alarm_unread_number", this.unread_alarms)
			.html(this.unread_alarms || "0");

		// total live number
		const $alarm_live_number = $notif_button.find("[alarm_live_number]");
		$alarm_live_number
			.attr("alarm_live_number", this.total_live_alarms)
			.html(this.total_live_alarms || "0");
	};

	/*
		Funcion que invoca a la clase de notificaciones de escritorio y auditivas.
	*/
	SupAlarmsNotifications.prototype.notify_desktop = function(campaign_id, total_new_alarms)
	{
		const desktop_notifications = inConcert.app.getDesktopNotifications();
		desktop_notifications.notify_supervisor_alarm(campaign_id, total_new_alarms);
	};

	SupAlarmsNotifications.prototype.update_notifications = function()
	{
		this.__set_alarms_number();
	};

	SupAlarmsNotifications.prototype.__set_events = function()
	{
		var self = this;

		$("#" + this.close_popup_id)
			.off("click")
			.on("click", function(e) {
				e.stopPropagation();
				self.toggle_popup();
			});

		var event_selector = "[alarm_id][alarm_from='LIVE']";
		$("#" + this.alarms_list_id)
			.off("click", event_selector)
			.on("click", event_selector, function(e) {
				e.stopPropagation();
				var alarm_id = $(this).attr("alarm_id");
				self.__mark_alarm_as_read(alarm_id);
			});

		event_selector = "[goto_alarm]";
		$("#" + this.alarms_list_id)
			.off("click", event_selector)
			.on("click", event_selector, function(e) {
				e.stopPropagation();
				var alarm_id = $(this).attr("goto_alarm");
				var alarm = self.live_alarms[alarm_id];
				if (!alarm) {
					return;
				}
				var indicator = self.__get_indicator(alarm.indicator_id);
				if (!indicator) {
					return;
				}
				self.__goto_panel(indicator.panel, alarm);
			});
	};

	SupAlarmsNotifications.prototype.__goto_panel = function(panel, alarm)
	{
		var self = this;

		var campaign_id = alarm.campaign_id;

		var goto_panel = function()
		{
			let full_screen = self.m_dashboard.is_full_screen_mode();
			if (full_screen) {
				let campaign_id_in_full_screen = self.__get_campaign_id_in_full_screen();
				if (campaign_id !== campaign_id_in_full_screen) {
					self.m_dashboard.full_screen_goto_tab_campaign(campaign_id);
				}
			}
			var params = {
				campaign_id: campaign_id,
				must_execute_new_search: true
			};

			// Sirve tanto en modo simple screen como en full screen
			var $back = self.m_dashboard.$back_button(campaign_id);

			switch(panel) {
				case "summary":
					$back.trigger("click");
				break;
				case "members":
					$back.trigger("click");
					// Si es la alarma de kickout, extraer el member id y pasarlo a una nueva funcion que haga el filtrado si es la alarma de kickout,
					// extraer el member id y pasarlo a una nueva funcion que haga el filtrado
					var member_id = "";
					var member_state = "";
					var indicator = self.__get_indicator(alarm.indicator_id);
					if (indicator) {
						if(indicator.id === "kickout")
						{
							member_id = alarm.member_id;
						}
						else
						{
							member_state = indicator.other_values.member_state;
						}
					}
					var must_clean_member_text = true;
					if (member_id !== "") {
						self.m_dashboard.members_change_member_filter(campaign_id, member_id);
					}
					else {
						self.m_dashboard.members_change_status_filter(campaign_id, member_state, 'desc', must_clean_member_text);
					}
				break;
				case "monitoring conversations":
					$back.trigger("click");
					self.m_dashboard.open_view("monitoring", params);
				break;
				case "queued conversations":
					$back.trigger("click");
					self.m_dashboard.open_view("queued", params);
				break;
				case "waiting conversations":
					$back.trigger("click");
					self.m_dashboard.open_view("waiting", params);
				break;
				case "abandoned conversations":
					$back.trigger("click");
					self.m_dashboard.open_view("abandoned", params);
				break;
				case "appointment conversations":
					$back.trigger("click");
					self.m_dashboard.open_view("appointment", params);
				break;
			}
		};

		var campaign_config = this.m_dashboard.get_campaign_selected_config(campaign_id);
		if (campaign_config.account_id === alarm.account_id && campaign_config.al_id === alarm.al_id) {
			goto_panel();
		}
		else {
			campaign_config.account_id = alarm.account_id;
			campaign_config.al_id = alarm.al_id;
			this.m_dashboard.change_campaign_selected_config(campaign_config, function(success)
			{
				if (!success) {
					// TODO: Hacer algo?... no deberia pasar nunca esto
					throw("SupAlarmsNotifications.prototype.__goto_panel. Error changing campaign selector.");
				}
				// Agrego la invocacion a la funcion en un pequeno timeout para que no haya inconvenientes.
				// No deberian existir.... pero.... asi nos aseguramos de darle tiempo a los paneles
				window.setTimeout(function() {
					goto_panel();
				}, 100);
			});
		}
	};

	SupAlarmsNotifications.prototype.__can_show_notification = function(campaign_id)
	{
		let campaign_id_in_full_screen = this.__get_campaign_id_in_full_screen();
		if (!campaign_id_in_full_screen || 'dashboard' === campaign_id_in_full_screen) {
			// 1 - Visualizo el item de la lista si NO esta en modo full screen, o si esta en modo full screen y posicionado en el dashboard (lista de campanas)
			return true;
		}
		else {
			if (campaign_id_in_full_screen.toUpperCase() === campaign_id.toUpperCase()) {
			// 2 - Visualizo el item de la lista, si se encuentra en modo Full Screen y estamos posicionados en el TAB de esta campana
				return true;
			}
			else {
			// 3 - NO lo visualizo el item de la lista, si se encuentra en modo Full Screen y NO estamos posicionados en el TAB de esta campana
				return false;
			}
		}
	};

	SupAlarmsNotifications.prototype.toggle_popup = function()
	{
		const self = this;

		var bind_event_outside;
		const $popup = $("#" + this.popup_id);
		var $alarms_list = $("#" + this.alarms_list_id);

		var show = function()
		{
			bind_event_outside();
			const $alarms = $alarms_list.find('[alarm_campaign_id]');
			// Recorro las campanas del selector y:
			$.each($alarms, function(_, alarm) {
				let $alarm = $(alarm);
				let alarm_campaign_id = $alarm.attr('alarm_campaign_id');
				if (self.__can_show_notification(alarm_campaign_id)) {
					$alarm.show();
				}
				else {
					$alarm.hide();
				}
			});
			$popup.show();
		};
		var hide = function() {
			$popup.unbind('clickoutside');
			$popup.hide();
		};

		bind_event_outside = function() {
			// Se agrega en un pequeno timeout el bindeo para que se biendee despues de visualizarlo correctamente, sino el mismo se cierra
			window.setTimeout(function() {
				$popup.bind('clickoutside', function(_) {
						hide();
					}
				);
			});
		};
		if ($popup.is(":visible")) {
			hide();
		}
		else {
			let has_permission = this.m_dashboard.has_permission_to_any_campaign("sup_view_alarms_config");
			if (!has_permission) {
				inConcert.app.getMainWindow().showBannerMessage({ 
					message: "<%=_T('NOT HAS PERMISSIONS TO SHOW ALARMS NOTIFICATIONS')%>",
					timeout: 5000
				});
				return;
			}
			show();
		}
	};

	SupAlarmsNotifications.prototype.__can_move_live_alarm_to_historical = function(alarm_id)
	{
		if (!this.live_alarms[alarm_id]) {
			return false;
		}
		// TODO: hacer algo mas ?
		return true;
	};

	/*
		Cuando la alarma deja de ser realtime y pasa a ser historica
	*/
	SupAlarmsNotifications.prototype.__move_live_alarm_to_historical = function(alarm_id)
	{
		if (!this.__can_move_live_alarm_to_historical(alarm_id)) {
			return false;
		}

		this.__mark_alarm_as_read(alarm_id);
		var alarm = this.live_alarms[alarm_id];
		this.total_live_alarms--;
		delete this.live_alarms[alarm_id];

		var $alarm = this.__$find_alarm(alarm);
		if ($alarm.length > 0) {
			$alarm.attr("alarm_from", "HISTORICAL")
				.removeClass("notifyLive")
				.addClass("notifyHistorical");
			$alarm.find("[goto_alarm]").remove();

			// Mueve en la lista del DOM, el elemento LIVE despues del ultimo elemento LIVE existente,
			// es decir, antes del primer HISTORICO.
			// Es una funcion recursiva, que va buscando el primer elemento historico.
			var total_alarms = this.__$count_alarm();
			var iter = 0;
			var move = function($current_alarm) {
				iter++;
				var $after_alarm = $current_alarm.next();
				var alarm_from = $after_alarm.attr("alarm_from");
				if (alarm_from === "LIVE") {
					if (total_alarms === iter + 1) {
						// Muevo el elemento que era LIVE y ahora es HISTORICO, despues del ultimo LIVE, ya que no existe ningun historico.
						$alarm.insertAfter($after_alarm);
						return;
					}
					else {
						// invoco nuevamente a la funcion recursiva, para que continue buscando el primer elemento historico
						return move($after_alarm);
					}
				}
				else if (alarm_from === "HISTORICAL") {
					// si es historico el elemento siguiente, y era la primer invocacion a la funcion
					// no tiene que mover el item en la lista.
					if (iter > 1) {
						// Muevo el elemento que era LIVE y ahora es HISTORICO, antes del primer historico existente.
						$alarm.insertBefore($after_alarm);
						return; // movido
					}
				}
				return; // no movido
			};
			move($alarm);
		}

		this.__set_alarms_number();
		return true;
	};

	/*
		Funcion encargada de encolar las alarmas de la campana.
		Encola los diferentes tipos de alarmas que llegan por los diferentes request.
		Espera a que lleguen todos los posibles requests para la campana, y ahi recien notifica las alarmas.
	*/
	SupAlarmsNotifications.prototype.queue_alarms_notifications = function(campaign_id, new_alarms_of_one_camp, notification_request_type)
	{
		// Aca se encuentran los diferentes tipos de request soportados (el que llega desde el request contra la URL de supervisor summary, etc, y del que llega del request contra los members)
		const allowed_notifications_types = {REQUEST_SUPERVISOR_PANEL: true, REQUEST_MEMBERS_PANEL: true};
		if (!allowed_notifications_types[notification_request_type]) {
			throw("SupAlarmsNotifications.prototype.queue_alarms_notifications - Unrecognized notification_request_type: " + notification_request_type);
		}
		// 1 - Para una campana determinada encolo las alarmas para un request dado
		this.m_queued_alarms = this.m_queued_alarms || {};
		this.m_queued_alarms[campaign_id] = this.m_queued_alarms[campaign_id] || {};
		this.m_queued_alarms[campaign_id][notification_request_type] = new_alarms_of_one_camp;

		// 2 - Recorro la lista de alarmas encoladas para la campana y las unifico en un unico array que contiene todas las notificaciones para esta campana.
		let total_campaign_queued = 0;
		let merged_campaign_alarms = [];
		for (let type in this.m_queued_alarms[campaign_id]) {
			if (this.m_queued_alarms[campaign_id].hasOwnProperty(type)) {
				total_campaign_queued++;
				let queued_alarms_type = this.m_queued_alarms[campaign_id][type];
				merged_campaign_alarms = merged_campaign_alarms.concat(queued_alarms_type);
			}
		}
		// 3 - Valido si la cantidad de request encolados para la campana ha llegado al total de request soportados por campana.
		//     En caso de haber llegado al total soportado por la campana, se notifican las alarmas para esta campana.
		let total_allowed_notifications_type = inConcert.Utils.objectCount(allowed_notifications_types);
		if (total_allowed_notifications_type === total_campaign_queued) {
			this.notify_alarms(campaign_id, merged_campaign_alarms);
			delete this.m_queued_alarms[campaign_id];
			return merged_campaign_alarms;
		}
	};

	/*
		Funcion publica que la invoca el dashboard para refrescar las interacciones en el popup.
		Esta funcion es invocada por cada request de cada campana.
	*/
	SupAlarmsNotifications.prototype.notify_alarms = function(campaign_id, new_alarms_of_one_camp)
	{
		const self = this;

		if (!new_alarms_of_one_camp) {
			return;
		}

		// Se agregan las nuevas alarmas
		let is_live_alarm = true;
		let has_new_alarms = {};
		let total_added_new_alarms = 0;

		$.each(new_alarms_of_one_camp, function(_, alarm) {
			// Si es alarma kickout, y ya está en el histórico de alarmas, no la notificamos de nuevo
			let kickoutAlarmExistsInHistorical = false;
			if (alarm.indicator_id === "kickout") {
				let kickoutAlarmId = "INDICATOR:kickout:CAMP:" + alarm.campaign_id + ":ACCOUNT:" + alarm.account_id + ":AL::AGENT:" + alarm.agentId + ":TIMESTAMP:" + alarm.timestamp;

				$.each(self.historical_alarms, function(_, historicalAlarm) {
					if(historicalAlarm.indicator_id === "kickout" && historicalAlarm.alarm_id === kickoutAlarmId) {
						kickoutAlarmExistsInHistorical = true;
					}
				});
				if (!kickoutAlarmExistsInHistorical) {
					// Condiciones, para mostrar la alarma:
					//	que el timestamp del kickout sea posterior a la fecha de login del supervisor
					//	que el timestamp del kickout sea posterior a show_kickouts_timestamp
					//	que el timestamp del kickout sea posterior a show_offline_timestamp 
					//	que el timestamp del kickout sea posterior al visible_timestamp de la campana
					let kickoutTimestamp = new Date((new Date(alarm.timestamp * 1000)).toUTCString().slice(0, -4));
					let config_options = self.m_dashboard.m_config.m_current_configuration[campaign_id].dashboard.panels.sup_dash_members_panel.options;
					let show_kickouts_timestamp = config_options.show_kickouts_timestamp || 0;
					let show_offline_timestamp = config_options.show_offline_timestamp || 0;
					let campaign_visible_timestamp = self.m_dashboard.m_config.m_current_configuration[alarm.campaign_id].visible_timestamp || 0;

					if (kickoutTimestamp > self.m_dashboard.m_userLoggedUTCdate &&
						alarm.timestamp > show_kickouts_timestamp && alarm.timestamp > show_offline_timestamp && alarm.timestamp > campaign_visible_timestamp) {
						let is_added = self.__add_alarm(alarm, is_live_alarm);
						if (is_added) {
							total_added_new_alarms++;
						}
					}
				}
				else {
					alarm.alarm_id = kickoutAlarmId;
				}
				has_new_alarms[alarm.alarm_id] = alarm;
			}
			else {
				let is_added = self.__add_alarm(alarm, is_live_alarm);
				if (is_added) {
					total_added_new_alarms++;
				}
				has_new_alarms[alarm.alarm_id] = alarm;
			}
		});

		// Almacena en un hash las alarmas que se deben pasar al historico.
		var hash_alarms_to_historical = {};
		$.each(this.live_alarms, function(live_alarm_id, live_alarm)
		{
			if (has_new_alarms[live_alarm_id]) {
			// 1 - Si la alarma live existente, SE ENCUENTRA, dentro de las new alarmas que han llegado en el request
			//     se actualiza la misma en la lista interna y en el markup de notificaciones.
				var message = self.get_alarm_msg(has_new_alarms[live_alarm_id]);
				live_alarm.message = message.full_message;
				live_alarm.tooltip = message.tooltip;
				var $alarm = self.__$find_alarm(live_alarm);
				if ($alarm.length > 0) {
					var $updated_alarm = self.__$make_alarm_notif(live_alarm, true);
					$alarm.replaceWith($updated_alarm);
				}
			}
			else {
			// 2 - Si la alarma live existente, NO SE ENCUENTRA, dentro de las new alarmas que han llegado en el request
				var move_alarm_to_historical = false;
				// 2.1 - Ahora valida que la alarma live a eliminar, sean de la misma campana que la campana del request
				if (campaign_id === live_alarm.campaign_id) {
					move_alarm_to_historical = true;
				}
				else {
					self.m_dashboard.get_config(function(config)
					{
						var campaign_config = config[live_alarm.campaign_id];
						if (!campaign_config || (campaign_config && !campaign_config.visible)){
							move_alarm_to_historical = true;
						}
					});
				}
				if (move_alarm_to_historical) {
					hash_alarms_to_historical[live_alarm_id] = true;
				}
			}
		});

		// Recorro el hash de alarmas a remover, y se van removiendo las mismas.
		$.each(hash_alarms_to_historical, function(current_alarm_id) {
			self.__move_live_alarm_to_historical(current_alarm_id);
		});

		// Notificacion en Desktop & Auditivo
		this.notify_desktop(campaign_id, total_added_new_alarms);

		// Guarda en el storage.
		this.__save_to_storage();

		return total_added_new_alarms;
	};

	SupAlarmsNotifications.prototype.__make_alarm_id = function (alarm)
	{
	//TODO: esta funcion deberia ser unica, ya que existe la misma en "sup_alarms_config.js".
		var alarm_id = "INDICATOR:{0}:CAMP:{1}:ACCOUNT:{2}:AL:{3}";

		var indicator_id = alarm.indicator_id || "";
		var campaign_id = alarm.campaign_id || "";
		var account_id = alarm.account_id || "";
		var al_id = alarm.al_id || "";

		let id = format(alarm_id, indicator_id, campaign_id, account_id, al_id);
		if (alarm.agentId) {
			id = id + ":AGENT:" + alarm.agentId;
		}
		//Si es un Kickout agregamos el timstamp al ID, para que evitar notificarla de nuevo si ya la mandamos al historico
		if(alarm.indicator_id === "kickout") {
			id = id + ":TIMESTAMP:" + alarm.timestamp;
		}
		return id;
	};

	SupAlarmsNotifications.prototype.__can_add_alarm = function(alarm, is_live)
	{
		var alarm_id = this.__make_alarm_id(alarm);
		if (is_live) {
			if (this.live_alarms[alarm_id]) {
				return false;
			}
		}

		return true;
	};

	/*
		{
		  "section": "INBOUND_CAMPAIGN",
		  "id": "CAMPAIGN_INDICATOR_ALARM",
		  "type": "alert",
		  "view": "AGENTS_DASHBOARD_INBOUND",
		  "campaign": "tel1",
		  "indicatorName": "indInCallAgents",
		  "parameters": [
			"tel1",
			"In Call Agents",
			"Equal to",
			"0",
			"0"
		  ],
		  "unread": false,
		  "date": 1482331568849
		}
	*/
	SupAlarmsNotifications.prototype.__add_alarm = function(alarm, is_live)
	{
		let alarm_id = this.__make_alarm_id(alarm);
		alarm.alarm_id = alarm_id;

		if (!this.__can_add_alarm(alarm, is_live)) {
			return false;
		}

		if (is_live) {
			var message = this.get_alarm_msg(alarm);
			alarm.message = message.full_message;
			alarm.date = Date.now();
			alarm.unread = true;

			this.live_alarms[alarm_id] = alarm;
			this.unread_alarms++;
			this.total_live_alarms++;
			this.historical_alarms.unshift(alarm); // agrego en la primer posicion del array
			this.__remove_old_read_alarm();
		}
		else {
			alarm.unread = false;
		}
		this.__append_alarm_to_panel(alarm, is_live);
		this.__set_alarms_number();

		return true;
	};

	// Quita la notificacion mas antigua
	SupAlarmsNotifications.prototype.__remove_old_read_alarm = function()
	{
		var total_alarms = this.historical_alarms.length;

		if (total_alarms === 0 || total_alarms <= MAX_NOTIFICATIONS) {
			return;
		}

		var index_old_read_alarm, old_read_alarm;
		for (var i = (total_alarms - 1); i >= 0; i--) {
			var alarm = this.historical_alarms[i];
			if (!alarm.unread) {
				index_old_read_alarm = i;
				old_read_alarm = alarm;
				break;
			}
		}

		if (index_old_read_alarm !== undefined) {
			this.historical_alarms.splice(index_old_read_alarm, 1); // remuevo del array la alarma segun su posicion
			var $old_read_alarm = this.__$find_alarm(old_read_alarm);
			if ($old_read_alarm.length > 0) {
				$old_read_alarm.remove();
			}
		}
	};

	SupAlarmsNotifications.prototype.__$find_alarm = function(alarm)
	{
		let selector = "";
		if(alarm) {
			selector = "[alarm_id='" + alarm.alarm_id + "'][alarm_date='" + alarm.date + "']";
		}
		const $alarm = $("#" + this.alarms_list_id).find(selector);
		return $alarm;
	};

	SupAlarmsNotifications.prototype.__$count_alarm = function()
	{
		var selector = "li";
		var count = $("#" + this.alarms_list_id).find(selector).length;
		return count;
	};

	SupAlarmsNotifications.prototype.__get_scope_by_alarm = function(alarm)
	{
		if (!alarm.account_id && !alarm.al_id) {
			return "campaign";
		}
		else if (!alarm.al_id) {
			return "account";
		}
		return "att_level";
	};

	/*
		Le pide a la clase la configuracion la alarma que el supervisor ha configurado.
	*/
	SupAlarmsNotifications.prototype.__get_user_alarm = function(alarm)
	{
		var user_alarms = this.m_dashboard.get_alarms_config(alarm.campaign_id);
		if (!user_alarms) {
			return;
		}
		var scope = this.__get_scope_by_alarm(alarm);

		var user_alarms_by_acc_scope;
		var user_alarms_by_scope;

		switch(scope) {
			case "campaign":
				user_alarms_by_scope = user_alarms.indicators.campaign;
			break;

			case "account":
				user_alarms_by_acc_scope = user_alarms.indicators.accounts[alarm.account_id];
				if (!user_alarms_by_acc_scope) {
					return;
				}
				user_alarms_by_scope = user_alarms_by_acc_scope.indicators;
			break;

			case "att_level":
				user_alarms_by_acc_scope = user_alarms.indicators.accounts[alarm.account_id];
				if (!user_alarms_by_acc_scope) {
					return;
				}
				var user_alarms_by_att_level_scope = user_alarms_by_acc_scope.att_levels[alarm.al_id];
				if (!user_alarms_by_att_level_scope) {
					return;
				}
				user_alarms_by_scope = user_alarms_by_att_level_scope.indicators;
			break;
		}

		if (!user_alarms_by_scope) {
			return;
		}

		return user_alarms_by_scope[alarm.indicator_id];
	};

	/*
		Esta funcion formatea el mensaje a mostrar en el popup de notificacion.
	*/
	SupAlarmsNotifications.prototype.get_alarm_msg = function(alarm, calculated_current_value)
	{
		var self = this;

		var msg_by_scope = {};

		msg_by_scope.campaign =
			"<label class='notifyCampaignText'>" +
				"<%=_T('{0}')%>" +
			"</label>" +
			"<label class='notifyItemText'>" +
				"<%=_T('Indicator:')%>" +
				"<strong>" +
					"<%=_T('{1}')%>" +
				"</strong>" +
			"</label>";

		msg_by_scope.account =
			"<label class='notifyCampaignText'>" +
				"<%=_T('{0}')%>" +
			"</label>" +
			"<label class='notifyItemText'>" +
				"<%=_T('Account:')%>" +
				"<strong>" +
					"<%=_T('{1}')%>" +
				"</strong>" +
			"</label>"; +
			"<label class='notifyItemText'>" +
				"<%=_T('Indicator:')%>" +
				"<strong>" +
					"<%=_T('{2}')%>" +
				"</strong>" +
			"</label>";

		msg_by_scope.att_level = 
			"<label class='notifyCampaignText'>" +
				"<%=_T('{0}')%>" +
			"</label>" +
			"<label class='notifyItemText'>" +
				"<%=_T('Account:')%>" +
				"<strong>" +
					"<%=_T('{1}')%>" +
				"</strong>" +
			"</label>"; +
			"<label class='notifyItemText'>" +
				"<%=_T('Att Level:')%>" +
				"<strong>" +
					"<%=_T('{2}')%>" +
				"</strong>" +
			"</label>"; +
			"<label class='notifyItemText'>" +
				"<%=_T('Indicator:')%>" +
				"<strong>" +
					"<%=_T('{3}')%>" +
				"</strong>" +
			"</label>";

			//Agregamos nuevo tipo de mensaje especifico para kickouts
		msg_by_scope.kickout =
			"<label class='notifyCampaignText'>" +
				"<%=_T('{0}')%>" +
			"</label>" +
			"<label class='notifyItemText'>" +
				"<%=_T('Account: {1}')%>" +
			"</label>" +
			"<label class='notifyItemText'>" +
				"<%=_T('{2} was kicked out on {4} by {3}')%>" +
			"</label>";

		var scope = this.__get_scope_by_alarm(alarm);
		var message = {full_message: "", tooltip: ""};

		//Si es un kickout, usamos el tipo de mensaje especifico
		if (alarm.indicator_id === "kickout") {
			message.full_message = msg_by_scope.kickout;
		}
		else {
			message.full_message = msg_by_scope[scope];
		}

		var campaign = inConcert.app.getCampaign(alarm.campaign_id);
		if (!campaign) {
			// TODO: hacer algo //CAMPAIGN NOT FOUND
			return;
		}

		var desc = function() {
			var description = {campaign : "", account: "", att_level: "", indicator: "hola como estas hoy"};

			var indicator = self.__get_indicator(alarm.indicator_id);
			if (indicator) {
				description.indicator = indicator.channel_label || indicator.label || indicator.id;
			}
			description.campaign = (campaign.Name || campaign.Id);

			var account = campaign.getAccount(alarm.account_id);
			if (!account) {
				return description;
			}
			description.account = campaign.getAccountDescription(account);
			if (alarm.al_id) {
				var att_level = campaign.getAttentionLevel(alarm.al_id);
				if (att_level) {
					description.att_level = att_level.name;
				}
			}
			return description;
		}();

		//Si es un kickout, usamos el tipo de mensaje especifico
		if(alarm.indicator_id === "kickout")
		{
			var kickoutTimestamp = (new Date(alarm.timestamp * 1000)).toLocaleString();
			message.full_message = format(message.full_message, desc.campaign, desc.account, alarm.member_id, alarm.userid, kickoutTimestamp);
		}
		else
		{
			switch(scope) {
				case "campaign":
					message.full_message = format(message.full_message, desc.campaign, desc.indicator);
				break;
				case "account":
					message.full_message = format(message.full_message, desc.campaign, desc.account, desc.indicator);
				break;
				case "att_level":
					message.full_message = format(message.full_message, desc.campaign, desc.account, desc.att_level, desc.indicator);
				break;
			}
		}
		
		var user_alarm = this.__get_user_alarm(alarm);
		if (user_alarm) {
			var operator_msg = this.__get_operator_label(user_alarm.operator);
			var section = user_alarm.section;
			var value_type = this.m_sections_labels[section] || "";

			var text_indicator_value = format(
				"<%=_T('Is {0} {1} {2}')%>",
				operator_msg,
				user_alarm.value,
				value_type
			);

			var current_value;
			if (calculated_current_value) {
				current_value = calculated_current_value;
			}
			else {
				current_value = typeof(alarm.values.current_value_source) !== "undefined" ? alarm.values.current_value_source : alarm.values.current_value;
				if (section === "MINUTE") {
					current_value = current_value / 60;
				}
				else if (section === "HOUR") {
					current_value = (current_value / 60) /60;
				}
				else if (section === "DAY") {
					current_value = ((current_value / 60) /60) / 24;
				}
				else if (section === "MONTH") {
					current_value = (((current_value / 60) /60) / 24) / 30;
				}
			}

			current_value = parseInt(current_value, 10);

			var text_current_value = format(
				"<span class='valueLabelAlarm'> (<%=_T('Current value:')%> </span><span class='valueNumberAlarm'>{0} {1})</span>",
				current_value,
				value_type
			);

			if (alarm.notificationMessage) {
				// El mensaje viene dado server side
				message.full_message = message.full_message +
					"<label class='notifyItemText'>" +
						alarm.notificationMessage +
					"</label>";

				message.tooltip = alarm.notificationMessage;
			}
			else {
				// El mensaje se genera client side
				message.full_message = message.full_message +
					"<label class='notifyItemText'>" +
						text_indicator_value +
						"<span class='currentValueITem'>" + text_current_value + "</span>" +
					"</label>";

				message.tooltip = format(
					"<%=_T('{0} {1} {2}')%>",
					desc.indicator,
					text_indicator_value,
					text_current_value
				);
			}
		}

		return message;
	};

	SupAlarmsNotifications.prototype.__$make_alarm_notif = function(alarm, is_live)
	{
		var goto_markup, live_historical_class, alarm_from;

		if (is_live) {
			alarm_from = "LIVE";
			live_historical_class = "notifyLive";
			goto_markup = "<a goto_alarm='" + alarm.alarm_id + "' class='pageButtonGeneric buttonColorWhite' href='javascript:void(0);'>" + "<%=_T('Context')%>" + "</a>";
		}
		else {
			alarm_from = "HISTORICAL";
			live_historical_class = "notifyHistorical";
			goto_markup = "";
		}

		var panel_class = "";
		var indicator = this.__get_indicator(alarm.indicator_id);
		if (indicator) {
			// Mapeo de tipos de panels y clases (css)
			var panel_classes = {
				conversations_panel: "notifyItemConversationsPanel",
				members_panel      : "notifyItemMembersPanel",
				summary_panel      : "notifyItemSummaryPanel"
			};
			panel_class = panel_classes[indicator.panel_type];
			if (!panel_class) {
				throw("SupAlarmsNotifications.prototype.__$make_alarm_notif - Unrecognized panel type: " + indicator.panel_type);
			}
		}
		var alarm_classes = "notifyListItem" + " " +
			(alarm.unread ? "unread" : "") + " " +
			panel_class + " " +
			live_historical_class;

		var $alarm = $(
			"<li alarm_campaign_id='" + alarm.campaign_id + "' alarm_id='" + alarm.alarm_id + "' alarm_date='" + alarm.date + "' alarm_from='" + alarm_from + "' class='" + alarm_classes + "'>" +
				"<div class='notifyColumn'>" +
					alarm.message +
				"</div>" +
				"<div class='notifyColumn notifyColumnBottom'>" +
					"<label class='notifyItemDate'>" + inConcert.Utils.formatDate(alarm.date) + "</label>" +
					goto_markup +
				"</div>" +
			"</li>"
		);

		if (!this.__can_show_notification(alarm.campaign_id)) {
			$alarm.hide();
		}

		$alarm.data("alarm", alarm);

		return $alarm;
	};

	//agrega una notificacion a la lista
	SupAlarmsNotifications.prototype.__append_alarm_to_panel = function(alarm, is_live)
	{
		var $alarm = this.__$find_alarm(alarm);
		if ($alarm.length > 0) {
			return false;
		}

		var $list = $("#" + this.alarms_list_id);
		$alarm = this.__$make_alarm_notif(alarm, is_live);

		if (is_live) {
			$list.prepend($alarm);
		}
		else {
			$list.append($alarm);
		}

		return true;
	};

	/*
		Esta funcion obtiene una alarma live y la retorna.
		A la misma la invocan los panels para saber si existe una alarma para un determinado indicador.
		Ej: summary si la alarma existe, marca uno de sus indicadors "Idle" "Break", etc.
	*/
	SupAlarmsNotifications.prototype.get_notification_alarm_by_indicator = function (info)
	{
		var alarm_id = this.__make_alarm_id(info);
		var alarm = this.live_alarms[alarm_id];
		return alarm;
	};

	/*
	*/
	SupAlarmsNotifications.prototype.__get_notifications_alarms_by_camp_and_category = function (campaign_id, category_id)
	{
		var self = this;

		var categorized_indicators = this.m_dashboard.get_categorized_indicators();
		if (!categorized_indicators) {
			// TODO: hacer algo no se podria dar!!
			return;
		}
		var indicators_by_category = categorized_indicators[category_id];
		if (!indicators_by_category) {
			// TODO: hacer algo no se podria dar!!
			return;
		}

		var live_alarms_by_category = {};

		$.each(this.live_alarms, function(_, notification)
		{
			if (campaign_id === notification.campaign_id) {
				if (indicators_by_category[notification.indicator_id]) {
					var indicator = self.__get_indicator(notification.indicator_id);
					live_alarms_by_category[notification.alarm_id] = {
						indicator: indicator,
						notification: notification,
						user_alarm: self.__get_user_alarm(notification)
					};
				}
			}
		});

		return live_alarms_by_category;
	};

/*
			Estas validaciones se realizan segun la definicion de las alarmas en el archivo "alarms.lua"
eval = {
	fields = {
		{name = "state", type = "string", operator = "EQUAL", value = "OFFLINE"},
		{name = "stateTimeTimestamp", "date"}
	}
}
*/
	SupAlarmsNotifications.prototype.eval_notifications_alarms = function (campaign_id, category_id, entity)
	{
		const self = this;

		let alarms = this.__get_notifications_alarms_by_camp_and_category(campaign_id, category_id);
		let response = {must_show_alarm: false};

		var show_alarm = function(user_alarm, current_value, notification) {
			response.must_show_alarm = true;
			response.indicator_id = user_alarm.indicator_id;
			response.campaign_id = user_alarm.campaign_id;
			response.account_id = user_alarm.account_id;
			response.al_id = user_alarm.al_id;
			if (notification) {
				response.agentId = notification.agentId;
			}
			if (user_alarm.indicator_id === "kickout" && user_alarm.member_id === entity.ID) {
				response.timestamp = user_alarm.timestamp;
			}
			response.current_value = current_value;
		};

		var evaluate = function(operator, user_alarm, entity_value, alarm_value, notification)
		{
			let display_value;
			if (notification) {
				display_value = (notification.values && typeof(notification.values.current_value_source) !== "undefined") ? notification.values.current_value_source : notification.values.current_value;
			}
			display_value = display_value || entity_value;

			if (operator === "GREATER") {
				if (entity_value > alarm_value) {
					show_alarm(user_alarm, display_value, notification);
					return false; // break loop
				}
			}
			else if (operator === "LESS") {
				if (entity_value < alarm_value) {
					show_alarm(user_alarm, display_value, notification);
					return false; // break loop
				}
			}
			else if (operator === "DISTINCT") {
				if (entity_value !== alarm_value) {
					show_alarm(user_alarm, display_value, notification);
					return false; // break loop
				}
			}
			else if (operator === "EQUAL") {
				if (entity_value === alarm_value) {
					show_alarm(user_alarm, display_value, notification);
					return false; // break loop
				}
			}
			else {
				throw("SupAlarmsNotifications.prototype.eval_notifications_alarms-> Unrecognized Operator: '" + operator + "'");
			}
			return true; // continue in loop
		};

		$.each(alarms || {}, function(_, alarm)
		{
			let user_alarm = alarm.user_alarm;

			if (user_alarm && user_alarm.enabled) {
				let indicator = alarm.indicator;

				if (indicator && indicator.eval) {
					$.each(indicator.eval.fields || {}, function(_, field)
					{
						let eval_value = entity[field.name];

						if (field.eval_type === "FIELD_VS_FIELD") {
							if (field.operator === "EQUAL") {
								if (field.value) {
									if (field.value !== eval_value) {
										return false; // break
									}
								}
								else {
									throw("TODO: IMPLEMENT!");
								}
							}
						}
						else if (field.eval_type === "SCOPE") {
							if (user_alarm.scope === "channels" && field.name === "account") {
								if (user_alarm.account_id !== eval_value) {
									return false; // break
								}
							}
							else if (user_alarm.scope === "att_levels" && field.name === "attention_level") {
								if (typeof eval_value !== "object" || user_alarm.al_id !== eval_value.id) {
									return false; // break
								}
							}
						}
						else if (field.eval_type === "EVAL_AGAINST_NOTIFICATION") {
							if (field.alarm_attr_name_value) {
								let notification = alarm.notification;
								let notification_value = notification[field.alarm_attr_name_value];
								let must_continue_in_loop = evaluate(field.operator, user_alarm, eval_value, notification_value, notification);
								if (!must_continue_in_loop) {
									return false; // break
								}
							}
							else {
								throw("TODO: IMPLEMENT!");
							}
						}
						else {
							let entity_value;
							let alarm_value;

							if (field.type === "date" || field.type === "member-date") {
								let elapsed_time_ms, elapsed_time_sec, elapsed_time_min, elapsed_time_hr, elapsed_time_day, elapsed_time_month;

								let date = typeof eval_value === "number" ? eval_value * 1000 : eval_value;
								eval_value = new Date(date);
								let date_time = eval_value.getTime();

								if (field.type === "member-date") {
									let campaign_timestamp = self.m_dashboard.get_campaign_timestamp(user_alarm.campaign_id) * 1000;
									elapsed_time_ms = campaign_timestamp - date_time;
									elapsed_time_sec = elapsed_time_ms / 1000;
								}
								else {
									elapsed_time_sec = inConcert.Utils.getTimeDifference(date_time, true);
								}

								elapsed_time_min = elapsed_time_sec / 60;
								elapsed_time_hr = elapsed_time_min / 60;
								elapsed_time_day = elapsed_time_hr / 24;
								elapsed_time_month = elapsed_time_day / 30;

								if (user_alarm.section === "SECOND") {
									entity_value = elapsed_time_sec;
								}
								else if (user_alarm.section === "MINUTE") {
									entity_value = elapsed_time_min;
								}
								else if (user_alarm.section === "HOUR") {
									entity_value = elapsed_time_hr;
								}
								else if (user_alarm.section === "DAY") {
									entity_value = elapsed_time_day;
								}
								else if (user_alarm.section === "MONTH") {
									entity_value = elapsed_time_month;
								}
								else {
									// TODO: no se deberia dar
									return false; // break
								}
								entity_value = parseFloat(entity_value, 10);
							}

							alarm_value = parseFloat(user_alarm.value, 10);

							let must_continue_in_loop = evaluate(user_alarm.operator, user_alarm, entity_value, alarm_value);
							if (!must_continue_in_loop) {
								return false; // break
							}
						}
					});
				}
			}
			else if (alarm.indicator.id === "kickout" && alarm.notification.member_id === entity.ID)
			{
				user_alarm = {
					indicator_id : alarm.indicator.id,
					campaign_id : alarm.notification.campaign_id,
					account_id : alarm.notification.account_id,
					member_id : alarm.notification.member_id,
					userid : alarm.notification.userid,
					timestamp : alarm.notification.timestamp,
					agentId : alarm.notification.agentId
				};
				show_alarm(user_alarm, "", alarm.notification);
			}
		});

		return response;
	};

}());
