/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
  strict:true, undef:true, curly:true, browser:true, indent:false, maxerr:50, esversion: 11 */

/*global $, inConcert, _ */

// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.
if (!this.inConcert) {
	this.inConcert = {};
}


/**
 * @namespace
 * @name SupervisorDashboardArchive
 */
(function () {

	'use strict';

	const _T = inConcert.i18n.getString;
	var format = inConcert.Utils.format;

	/**
	 * Creates a new archive dashboard instance.
	 *
	 * @author Sebastian Moreno (smoreno@inconcertcc.com)
	 * @constructor
	 * @memberOf SupervisorDashboardArchive
	 * @param dashboard_instance
	 */
	function DashboardArchive(dashboard_instance) {

		DashboardArchive.uber.constructor.call(this, 'archive', "<%=_T('Archive')%>", dashboard_instance);

		this.permission_id = 'sup_view_archive';
		this.navigation_folder_id = 'archive';
		this.m_view_classes_name = 'archiveViewer';
		this.m_templates = inConcert.sup_templates.dashboard;
		this.m_model = dashboard_instance.get_model_manager();
		this.m_dashboard = dashboard_instance;
		this.m_config = dashboard_instance.get_config_manager();
		this.m_request_search_archive = null;
		// Campaign Specific
		this.m_search_already_initialized = {};
		// filters
		this.m_filter_by_campaign = {};
	}

	inConcert.Utils.inherit(DashboardArchive, inConcert.sup_dashboard_interactions_view);

	if (typeof inConcert.sup_dashboard_archive !== 'function') {
		inConcert.sup_dashboard_archive = DashboardArchive;
	}

	DashboardArchive.prototype.__get_interaction_gui_config = function()
	{
		var self = this;

		var config = DashboardArchive.uber.__get_interaction_gui_config.call(this);

		config.callbacks = {
			on_reopen: function( success, interaction )
			{
				if ( !success ) {
					return;
				}
				var campaign_dom_id = self.m_dashboard.get_normalized_id( interaction.campaign );
				var $container = self.get_interactions_container( campaign_dom_id );
				var interaction_dom_id = self.get_interaction_dom_id( interaction );
				self.remove_interaction( $container, interaction_dom_id, true /* remove all interactions of the group */ );
			}
		};

		return config;
	};

	DashboardArchive.prototype.on_change_config_selector = function( campaign_id )
	{
		DashboardArchive.uber.on_change_config_selector.call(this, campaign_id);
	};

	DashboardArchive.prototype.get_query_from_archive_filter = function( campaign_id )
	{
		var self = this;
		
		var search_query = this.m_dashboard.getQueryBuilderFilter(campaign_id);
		if(!search_query) {
			return;
		}

		var campaign_dom_id = this.m_dashboard.get_normalized_id(campaign_id);
		var $dynamic = $('#sup_dash_interaction_viewer_dynamic_' + campaign_dom_id);

		//Pills
		var $pillContainer = $dynamic.find(".form-group-pill-card-outlined");
		var $noPill = $pillContainer.find(".inlineMessagesContainer");
		var $pillGroup = $pillContainer.find(".form-group-pill-container");
		$pillGroup.empty();
		if(search_query.rules && $.isArray(search_query.rules) && search_query.rules.length > 0)
		{
			var addRemoveButtonClick;
			addRemoveButtonClick = function(ruleId) {
				$("#" + ruleId + "_remove").click(function() {
					var ruleIdx = parseInt(this.id.replace("sup_archive" + campaign_dom_id + "_pill_", ""));
					//Eliminar del queryBuilder
					if(self.m_dashboard.queryBuilderObj[campaign_dom_id].deleteRule(ruleIdx))
					{
						//Eliminar del contenedor de pills
						$(this).parent().remove();
					}
				});
			};

			for (var i = 0; i < search_query.rules.length; i++) {
				var ruleId = "sup_archive" + campaign_dom_id + "_pill_" + i;
				var rulePillText = _T(search_query.rules[i].label || search_query.rules[i].id);
				if(search_query.rules[i].id === 'businessdata') {
					for(let j = 0; j < search_query.rules[i].value.length; j++){
						var businessdataName = search_query.rules[i].value[j].name;
						var businessdataOperator = search_query.rules[i].value[j].operator;
						var businessdataType = search_query.rules[i].value[j].type;
						var businessdataValue = search_query.rules[i].value[j].value;
						var businessdataValueTo = search_query.rules[i].value[j].value_to;
						rulePillText += ' | ' + '<strong>' + businessdataName + '</strong>' + ' (' + businessdataType.toLowerCase() + ') | ' + _T(businessdataOperator) + ' | <strong>' + businessdataValue;
						if(businessdataOperator === 'between') {
							rulePillText += ' , ' + businessdataValueTo;
						}
						rulePillText += '</strong>';
					}
				}
				else {
					rulePillText += ' | ' + _T(search_query.rules[i].operator) + ' | ' + '<strong>';
					rulePillText += search_query.rules[i].pill ? search_query.rules[i].pill : _T(search_query.rules[i].value);
					if(search_query.rules[i].operator === 'between') {
						rulePillText += ' , ' + search_query.rules[i].value_to;
					}
					rulePillText += '</strong>';
				}
				const $lbl = "" +
					"<label id='" + ruleId + "_label' class='form-group-pill'>" +
						"<span>" + rulePillText + "</span>" +
						"<button id='" + ruleId + "_remove' class='pillButtonDelete'></button>" +
					"</label>";
				$pillGroup.append($lbl);
				$pillGroup.show();
				$noPill.hide();

				addRemoveButtonClick(ruleId);
			}
		}
		else
		{
			$pillGroup.hide();
			$noPill.show();
		}
		
		//labels
		let navigation = this.get_navigation(campaign_id);
		let selected_labels = navigation.get_selected_labels();
		if (selected_labels && $.isArray(selected_labels) && selected_labels.length > 0) {
			var labelsRule = {
				id : "labels",
				operator : "in",
				value : selected_labels
			};
			search_query.rules.push(labelsRule);
		}
		
		if(!search_query.rules.find(rule => rule.id === "campaign")) {
			//campaign
			var campaignRule = {
				id : "campaign",
				operator : "equal",
				value : campaign_id
			};
			search_query.rules.push(campaignRule);
		}

		search_query.rules = this.m_dashboard.getCommonConfigRules(search_query.rules, campaign_id);

		return search_query;
	};

	DashboardArchive.prototype.__get_prefix = function()
	{
		return 'supervisor_int_archive_';
	};

	DashboardArchive.prototype.process_interactions = function(campaign_id, interactions)
	{
		var self = this;

		var campaign_dom_id = this.m_dashboard.get_normalized_id(campaign_id);
		var interactions_container = this.get_interactions_container(campaign_dom_id);
		var scroll_container = interactions_container.parent();
	
		// detach the node from the DOM to faster append
		var scroll_top = scroll_container.scrollTop();
		var detached_container = interactions_container.detach();

		var parmams_interaction = {
			prefix: this.__get_prefix(),
			allowed_bulk_actions: false,
			columns: [
				{id: "attention_time", hide: false}
			]
		};

		// build markup for every interaction.
		interactions.forEach(function( interaction ) {
			var $new_interaction = self.m_interaction_gui.$build_interaction_node( interaction, self.m_templates.interaction_node_tpl, parmams_interaction );
			// update interaction on panel
			self.m_interaction_gui.update_interaction_data( interaction, $new_interaction );
			self.insert_interaction_in_panel_list( interaction, $new_interaction, detached_container );
		});

		// put back to the DOM and adjust scroll top position.
		scroll_container.append(detached_container);

		detached_container.find("[int_labels_disposition]").each(function(_, element) {
			self.m_interaction_gui.toggleInteractionLabels($(element));
		});
		
		scroll_container.scrollTop(scroll_top);
	};

	// get query from vs box plus campaign.
	DashboardArchive.prototype.set_archive_filter_pattern = function(campaign_id, pattern) {
		
	};

	DashboardArchive.prototype.update_current_pagination = function(campaignId, pageInteractions)
	{
		let pageInfo = this.m_filter_by_campaign[campaignId];
		if (!pageInfo) {
			return;
		}
		let currentPage = pageInfo.page;
		let lastPage = pageInfo.last_page;
		let totalRows = pageInfo.totalRows;

		DashboardArchive.uber.update_current_pagination.call(this, campaignId, currentPage, lastPage, pageInteractions, totalRows);
	};

	DashboardArchive.prototype.search_archive = function ($current_node, query, callback = $.noop)
	{
		const self = this;

		console.info("[DashboardArchive.search_archive] START ");

		if (this.m_request_search_monitor) {
			console.info("[DashboardArchive.search_archive] --> Unable to start search archive. Already exist another request: " + this.m_request_search_archive);
			callback(false);
			return;
		}

		let campaign_id = $current_node.attr('campaign_id');
		let campaign_dom_id = this.m_dashboard.get_normalized_id(campaign_id);
		var $dynamic = $('#sup_dash_interaction_viewer_dynamic_' + campaign_dom_id);
		
		// Dejo disabled la navegacion, hasta que culmine de procesar el request
		let navigation = this.get_navigation(campaign_id);
		if (navigation) {
			navigation.disable_navigation(true);
		}

		const onSearchFinished = function() {
			callback();
			if (navigation) {
				// Dejo enabled la navegacion, ya que ha terminado de procesar el Request.
				navigation.disable_navigation(false);
			}
			self.m_request_search_archive = null;
		};

		// if we don't have any filter, just create the default.
		if (!this.m_filter_by_campaign[campaign_id]){
			this.m_filter_by_campaign[campaign_id] = {};
			this.m_filter_by_campaign[campaign_id].page = 1;
		}

		// override filter if a query was specified
		var filter_query = query ? query : this.get_query_from_archive_filter(campaign_id); // use the previous filter if any

		// defaults value set
		var current_page = this.m_filter_by_campaign[campaign_id].page || 1;
		var last_by_page = this.m_filter_by_campaign[campaign_id].last_by_page || {};

		// last interaction only apply if we are dealing with a page different from the first.
		var last_interaction = last_by_page && current_page > 1 ? last_by_page[current_page - 1] : '';
		var paginating = last_interaction !== '' ? true : false;

		this.m_filter_by_campaign[campaign_id] = {
			query        : filter_query,
			last         : last_interaction,
			last_by_page : last_by_page,
			page         : current_page,
			paginating   : paginating,
			rows         : this.INTERACTIONS_PER_PAGE,
			output       : 'interactions'
		};

		// go and search for archive interactions
		this.m_request_search_archive = this.m_model.search_archive_interactions(this.m_filter_by_campaign[campaign_id],
			function(status, result) {
				if (status) {
					$dynamic.find("[query_builder_clean]").prop("disabled", !self.m_dashboard.queryBuilderObj[campaign_dom_id].hasRules());
					var current_page = self.m_filter_by_campaign[campaign_id].page;
					var page_interactions = result.interactions && result.interactions.length || 0;

					// update pagination numbers
					self.m_filter_by_campaign[campaign_id].totalRows = result.total || 0;
					self.m_filter_by_campaign[campaign_id].last_by_page[current_page] = result.last;
					self.m_filter_by_campaign[campaign_id].last_page = Math.ceil(result.total / self.INTERACTIONS_PER_PAGE);
					self.update_current_pagination(campaign_id, page_interactions);

					// if user changed view, return
					if (self.view_has_changed(campaign_id)) {
						onSearchFinished();
						return;
					}
					// Show empty conversations message.
					self.clear_list(campaign_dom_id);
					if (page_interactions === 0){
						self.toggle_no_conversations_msg(campaign_dom_id, "<%=_T('No archived conversations to show. Try another search')%>");
					}
					else{
						// go and process interactions
						self.process_interactions(campaign_id, result.interactions);
					}
					// all done
					onSearchFinished();
				}
				else {
					inConcert.app.getMainWindow().showBannerMessage({ message : format("<%=_T('Unable to search conversations from archive on campaign: {0}.')%>", campaign_id), timeout : 5000});
					onSearchFinished();
				}
			}
		);
	};

	DashboardArchive.prototype.setup_archive_pagination = function($current_node, campaign_id)
	{
		const self = this;
		let campaign_dom_id = this.m_dashboard.get_normalized_id(campaign_id);

		// setup interactions pagination.
		this.setup_pagination(campaign_id, function(button, on_pagination_finished){
			// acording to the button clicked, set correct page.
			switch (button.id){
				case 'interactions_page_first_' + campaign_dom_id:
					self.m_filter_by_campaign[campaign_id].page = 1;
					break;
				case 'interactions_page_next_'+ campaign_dom_id:
					self.m_filter_by_campaign[campaign_id].page++;
					break;
				case 'interactions_page_prev_'+ campaign_dom_id:
					self.m_filter_by_campaign[campaign_id].page--;
					break;
				case 'interactions_page_last_' + campaign_dom_id:
					let page;
					if (self.m_filter_by_campaign[campaign_id].totalRows > self.MAX_ALLOWED_ROWS_IN_ES) {
						// Si ha llegado al maximo de rows permitidos por ES, me quedo con la pagina previa
						page = self.MAX_ALLOWED_ROWS_IN_ES / self.INTERACTIONS_PER_PAGE;
						page = Math.floor(page) < page ? Math.floor(page) + 1 : page;
					}
					else {
						page = self.m_filter_by_campaign[campaign_id].last_page;
					}
					self.m_filter_by_campaign[campaign_id].page = page;
					break;
				default:
					break;
			}
			// Go and search
			let current_filter = self.get_query_from_archive_filter(campaign_id);
			self.search_archive($current_node, current_filter, function(){
				on_pagination_finished();
			});
		});
	};

	DashboardArchive.prototype.init = function($current_node, member_id)
	{
		DashboardArchive.uber.init.call(this, $current_node, { has_batch_actions : false });

		var self = this;
		var campaign_id = $current_node.attr('campaign_id');
		var campaign_dom_id = this.m_dashboard.get_normalized_id(campaign_id);

		//var $search = $current_node.find('[id="sup_dash_interaction_viewer_dynamic_' + campaign_dom_id + '"]');
		//var $search_box = $search.find('[filter_search]');
		//var $search_button = $search.find('[filter_search_execute]');

		this.showOrHideColumns(campaign_id);

		// setup interactions pagination.
		this.setup_archive_pagination($current_node, campaign_id);

		// Set agent if it was specified.
		if (member_id){
			this.set_archive_filter_pattern(campaign_id, 'agent:' + member_id);
		}

		// before do anything, clear interactions list.
		this.clear_list(campaign_dom_id);
		this.show_waiting(campaign_dom_id, "<%=_T('Searching archive, please wait.')%>"); // show waiting

		// go and search
		this.search_archive($current_node, undefined, function(){
			self.hide_waiting(campaign_dom_id);
			self.on_view_load_finished(campaign_id);
		});
	};

	DashboardArchive.prototype.refresh = function (campaign_id)
	{
		var self = this;
		var campaign_dom_id = this.m_dashboard.get_normalized_id(campaign_id);
		this.show_waiting(campaign_dom_id, "<%=_T('Searching archive, please wait.')%>"); // show waiting
		this.clear_list(campaign_dom_id);
		var $current_node = this.m_dashboard.get_campaign_node(campaign_id);
		// go and search for conversations
		this.search_archive($current_node, undefined, function(){
			self.hide_waiting(campaign_dom_id);
			self.on_view_load_finished(campaign_id);
		});
	};

	DashboardArchive.prototype.showOrHideColumns = function(campaign_id)
	{
		const campaign_dom_id = this.m_dashboard.get_normalized_id(campaign_id);
		const $dynamic = $('#sup_dash_interaction_viewer_dynamic_' + campaign_dom_id);
		$dynamic.find("[header_column_name='attention_time']").show();
	};

}());
