/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
  strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, esversion: 11 */

/* global inConcert, _ */

// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.
if (!this.inConcert) {
	this.inConcert = {};
}

/**
 * @namespace
 * @name SupervisorBotInteractions
 */
(function () {

	'use strict';

	const _T = inConcert.i18n.getString;

	/**
	 * Creates a new dashboard bot_interactions instance.
	 *
	 * @author Mario Mori
	 * @constructor
	 * @memberOf SupervisorBotInteractions
	 * @param dashboard_instance
	 */
	function DashboardBots(dashboard_instance)
	{
		DashboardBots.uber.constructor.call(this, 'bot_interactions', "<%=_T('Bot Interactions')%>", dashboard_instance);

		this.permission_id = 'sup_monitoring_bot_interaction';
		this.navigation_folder_id = 'bot_interactions';
		this.m_view_classes_name = 'bot_interactionsViewer';
		this.m_model = dashboard_instance.get_model_manager();
		// flag to know if i'm searching the bot_interactions.
		this.m_searching_bot_interactions = {};
		this.m_page_bot_interactions = {};
		this.m_dashboard = dashboard_instance;
		// current config
		this.m_config = dashboard_instance.m_config;
	}

	inConcert.Utils.inherit(DashboardBots, inConcert.sup_dashboard_interactions_view);

	if (typeof inConcert.sup_dashboard_bot_interactions !== 'function') {
		inConcert.sup_dashboard_bot_interactions = DashboardBots;
	}

	DashboardBots.prototype.__get_interaction_gui_config = function()
	{
		var self = this;

		var config = DashboardBots.uber.__get_interaction_gui_config.call(this);

		config.allowed_bulk_actions = false;

		config.get_specifics_interaction_classes = function( interaction, $interaction ) {
			return self.get_specifics_interaction_classes( interaction, $interaction );
		};

		return config;
	};

	DashboardBots.prototype.on_change_config_selector = function( campaign_id )
	{
		DashboardBots.uber.on_change_config_selector.call(this, campaign_id);
	};

	DashboardBots.prototype.__must_show_alarm = function (campaign_id, interaction)
	{
		var category = "conversation_time_on_state";
		return this.m_dashboard.eval_notifications_alarms(campaign_id, category, interaction);
	};

	DashboardBots.prototype.get_specifics_interaction_classes = function( interaction, $interaction )
	{
		if ( typeof interaction !== 'object' ) {
			return '';
		}

		var classes = this.__get_alarm_classes(interaction, $interaction);
		return classes;
	};

	DashboardBots.prototype.__get_prefix = function( /*interaction*/ )
	{
		return 'supervisor_int_bot_interactions_';
	};

	DashboardBots.prototype.__get_dom_interaction_params = function( interaction )
	{
		return {
			prefix: this.__get_prefix( interaction ),
			allowed_bulk_actions: false // TODO: agregar permiso ?
		};
	};

	DashboardBots.prototype.search_bot_interactions = function ($current_node, is_new_search, callback_fn = $.noop)
	{
		const self = this;

		let campaign_id = $current_node.attr('campaign_id');
		var search_query = this.m_dashboard.getQueryBuilderFilter(campaign_id);
		if(!search_query) {
			return;
		}

		let campaign_dom_id = this.m_dashboard.get_normalized_id(campaign_id);

		if (this.m_searching_bot_interactions[campaign_id]) {
			return;
		}

		// Dejo disabled la navegacion, hasta que culmine de procesar el request
		let navigation = this.get_navigation(campaign_id);
		if (navigation) {
			navigation.disable_navigation(true);
		}

		this.m_searching_bot_interactions[campaign_id] = true;
		if ( is_new_search ) {
			this.clear_list(campaign_dom_id);
			this.show_waiting(campaign_dom_id, "<%=_T('Searching bot_interactions, please wait.')%>");
		}

		var on_search_completed = function() {
			if ( is_new_search ) {
				self.disable_batch_actions(campaign_id);
			}
			self.m_searching_bot_interactions[campaign_id] = false;
			self.hide_waiting(campaign_dom_id);
			if (navigation) {
				// Dejo enabled la navegacion, ya que ha terminado de procesar el Request.
				navigation.disable_navigation(false);
			}
			callback_fn();
		};

		var page_number = this.m_page_bot_interactions[campaign_id] && this.m_page_bot_interactions[campaign_id].filter.page ? this.m_page_bot_interactions[campaign_id].filter.page : 1;
		var $dynamic = $('#sup_dash_interaction_viewer_dynamic_' + campaign_dom_id);

		//Pills
		var $pillContainer = $dynamic.find(".form-group-pill-card-outlined");
		var $noPill = $pillContainer.find(".inlineMessagesContainer");
		var $pillGroup = $pillContainer.find(".form-group-pill-container");
		$pillGroup.empty();
		if(search_query && search_query.rules && $.isArray(search_query.rules) && search_query.rules.length > 0)
		{
			var addRemoveButtonClick;
			addRemoveButtonClick = function(ruleId) {
				$("#" + ruleId + "_remove").click(function() {
					var ruleIdx = parseInt(this.id.replace("sup_bot_interactions" + campaign_dom_id + "_pill_", ""));
					//Eliminar del queryBuilder
					if(self.m_dashboard.queryBuilderObj[campaign_dom_id].deleteRule(ruleIdx))
					{
						//Eliminar del contenedor de pills
						$(this).parent().remove();
					}
				});
			};			
			for (var i = 0; i < search_query.rules.length; i++) {
				var ruleId = "sup_bot_interactions" + campaign_dom_id + "_pill_" + i;
				var rulePillText = _T(search_query.rules[i].label || search_query.rules[i].id) + " | " + _T(search_query.rules[i].operator) + " | " + "<strong>" + _T(search_query.rules[i].value) + "</strong>";
				const $lbl = "" +
					"<label id='" + ruleId + "_label' class='form-group-pill'>" +
						"<span>" + rulePillText + "</span>" +
						"<button id='" + ruleId + "_remove' class='pillButtonDelete'></button>" +
					"</label>";
				$pillGroup.append($lbl);
				$pillGroup.show();
				$noPill.hide();

				addRemoveButtonClick(ruleId);
			}
		}
		else
		{
			$pillGroup.hide();
			$noPill.show();
		}

		//labels
		let selected_labels = navigation.get_selected_labels();
		if (selected_labels && $.isArray(selected_labels) && selected_labels.length > 0) {
			var labelsRule = {
				id : "labels",
				operator : "in",
				value : selected_labels
			};
			search_query.rules.push(labelsRule);
		}

		if(!search_query.rules.find(rule => rule.id === "campaign")) {
			//campaign
			var campaignRule = {
				id : "campaign",
				operator : "equal",
				value : campaign_id
			};
			search_query.rules.push(campaignRule);
		}

		search_query.rules = this.m_dashboard.getCommonConfigRules(search_query.rules, campaign_id);

		var filter = {
			page               : page_number,
			rows_per_page      : this.INTERACTIONS_PER_PAGE,
			query              : search_query,
			sort               : [{state_timestamp: { order: "asc" }, priority: {order: "desc"}}]
		};

		this.m_page_bot_interactions[campaign_id] = {
			paginating : false,
			filter : filter
		};

		this.m_model.search_bot_interactions(filter, function(success, result) {

			self.m_page_bot_interactions[campaign_id].paginating = false;
			if ( success ) {
				$dynamic.find("[query_builder_clean]").prop("disabled", !self.m_dashboard.queryBuilderObj[campaign_dom_id].hasRules());
				// Show empty list conversations.
				if ( result.interactions.length === 0 && result.total === 0 ) {
					self.clear_list(campaign_dom_id);
					self.toggle_no_conversations_msg(campaign_dom_id, "<%=_T('Bot conversations list is empty. You can try searching conversations by text.')%>");
					on_search_completed();
				}
				else {
					if (self.view_has_changed(campaign_id)){
						on_search_completed();
					}
					else {
						self.remove_no_conversations_msg(campaign_dom_id);
						if ( is_new_search ) {
							self.clear_list(campaign_dom_id);
							var $container = self.get_interactions_container(campaign_dom_id);
							result.interactions.forEach(function(interaction)
							{
								var parmams_interaction = self.__get_dom_interaction_params(interaction);

								self.update_owner(interaction);
								self.update_contact_name(interaction);

								var $interaction_node = self.m_interaction_gui.$build_interaction_node( interaction, self.m_templates.interaction_node_tpl, parmams_interaction );
								self.insert_interaction_in_panel_list( interaction, $interaction_node, $container );
							});
						}
						else {

							// go and process the conversations
							self.process_off_dom_interaction_update(
								campaign_dom_id,
								result.interactions,
								self.m_templates.interaction_node_tpl,
								function(interaction, $interaction_node) {
									// update interaction on panel
									self.update_owner(interaction);
									self.update_contact_name(interaction);
									self.m_interaction_gui.update_interaction_data( interaction, $interaction_node );
								}
							);
						}
					}
					on_search_completed();
				}
				// Update pagination info.
				self.m_page_bot_interactions[campaign_id].totalRows = result.total || 0;
				self.m_page_bot_interactions[campaign_id].last_page = Math.ceil(result.total / self.INTERACTIONS_PER_PAGE);
				self.update_current_pagination(campaign_id, result.interactions.length || 0, result.total || 0);
			}
			else {
				inConcert.app.getMainWindow().showBannerMessage({ message : "<%=_T('Error server finding bot_interactions conversations')%>", timeout : 5000});
				on_search_completed();
			}
		});
	};

	/**
	 * Will refresh the list of bot_interactions conversations
	 *
	 * @async
	 * @public
	 * @param  {Object} $current_node - The current jquery campaign object.
	 * @param callback
	 */
	DashboardBots.prototype.refresh = function($current_node, callback)
	{
		callback = callback || $.noop;

		var campaign_id = $current_node.attr('campaign_id');

		if (this.view_has_changed(campaign_id)) {
			callback();
			return;
		}
		if ( !this.m_page_bot_interactions[campaign_id] ) {
			callback();
			return;
		}
		var current_filter = this.m_page_bot_interactions[campaign_id].filter;
		var campaign_config = this.m_dashboard.get_campaign_selected_config(campaign_id);
		current_filter.account_id = campaign_config.account_id;
		current_filter.attention_level_id = campaign_config.al_id;
		this.m_page_bot_interactions[campaign_id].filter = current_filter;

		var is_new_search = false;
		this.search_bot_interactions($current_node, is_new_search, callback);
	};

	DashboardBots.prototype.get_view_paged_info = function()
	{
		return this.m_page_bot_interactions;
	};

	DashboardBots.prototype.update_owner = function(interaction)
	{
		let internal_consultation_owner = interaction.interactionData.internal_consultation_owner;

		if (internal_consultation_owner &&
			internal_consultation_owner !== undefined &&
			internal_consultation_owner.split("@").length > 1) {
				interaction.assignedAgents = interaction.assignedAgents || [];
				if (interaction.assignedAgents.length === 0) {
					interaction.assignedAgents.push(internal_consultation_owner.match(/(.+)@([^@]+)$/)[1]);
				}
		}
	};
	
	DashboardBots.prototype.update_contact_name = function(interaction, _)
	{
		let contactName = "";

        if (interaction.properties.display_name !== "" && interaction.properties.display_name !== undefined) {
            contactName = interaction.properties.display_name;
        } else {
            contactName = interaction.interactionData.contactName;
        }

		if (interaction.interactionData.internal_consultation_flag && contactName && !contactName.includes("><i class='far fa-hand-paper'></i><")) {
			interaction.interactionData.contactName = "<div class='client-label'><span class='client-name'>" + contactName + "</span><span class='hand-icon'><i class='far fa-hand-paper'></i></span></div>";
			interaction.properties.display_name = "<div class='client-label'><span class='client-name'>" + contactName + "</span><span class='hand-icon'><i class='far fa-hand-paper'></i></span></div>";
		}
	};

	DashboardBots.prototype.update_current_pagination = function(campaignId, pageInteractions)
	{
		let pageInfo = this.m_page_bot_interactions[campaignId];
		if (!pageInfo) {
			return;
		}
		let currentPage = pageInfo.filter.page;
		let lastPage = pageInfo.last_page;
		let totalRows = pageInfo.totalRows;

		DashboardBots.uber.update_current_pagination.call(this, campaignId, currentPage, lastPage, pageInteractions, totalRows);
	};

	DashboardBots.prototype.execute_new_search = function(campaign_id, callback, cleanup_search_text)
	{
		callback = callback || $.noop;

		if (cleanup_search_text) {
			this.cleanup_search_text(campaign_id);
		}
		if ( this.m_page_bot_interactions[campaign_id] ) {
			this.m_page_bot_interactions[campaign_id].filter.page = 1;
		}
		var $current_node = this.m_dashboard.get_campaign_node(campaign_id);
		var is_new_search = true;
		this.search_bot_interactions($current_node, is_new_search, function() {
			callback();
		});
	};

	/**
	 * Will initialize the bot_interactions view, will get the conversations
	 * from the server and build the corresponding nodes.
	 *
	 * @async
	 * @public
	 * @param  {object}   $current_node - The current campaign node (jquery node)
	 * @param  {Function} callback     - A callback to be called once the initialization is finished
	 */
	DashboardBots.prototype.init = function($current_node, callback)
	{
		DashboardBots.uber.init.call(this, $current_node, { has_batch_actions : false });

		var self = this;
		callback = callback || $.noop;
		var campaign_id = $current_node.attr('campaign_id');
		var campaign_dom_id = this.m_dashboard.get_normalized_id(campaign_id);

		// Clear previous conversations (monitoring/archive) and show bot_interactions
		this.clear_list(campaign_dom_id);
		this.show_waiting(campaign_dom_id, 'Searching bot_interactions, please wait.');

		this.setup_view_pagination($current_node, campaign_id);
		var campaign_config = this.m_dashboard.get_campaign_selected_config(campaign_id);

		this.m_page_bot_interactions[campaign_id] = {
			paginating : false,
			filter : {
				campaign_id        : campaign_id,
				account_id         : campaign_config.account_id,
				attention_level_id : campaign_config.al_id,
				page               : 1,
				rows_per_page      : this.INTERACTIONS_PER_PAGE
			}
		};
		this.search_bot_interactions($current_node, true, function() {
			self.hide_waiting(campaign_dom_id);
			self.on_view_load_finished(campaign_id);
			callback();
		});
	};
}());
