/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, esversion: 11 */

/* global $, inConcert, _ , Mustache, inconcert*/

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

//	"use strict";

	const _T = inConcert.i18n.getString;
	const zeropad = inConcert.Utils.zeropad;
	const nid = inConcert.Utils.getNormalizedId;
	const capitalize = inConcert.Utils.capitaliseFirstLetter;
	const HISTORICAL_CHART_REFRESH_INTERVAL_MS = 60000; // 1 minute (min slice 15 min)
	const FORMAT = inConcert.Utils.format;

	function sup_full_screen_charts (params)
	{
		this.dashboard = params.dashboard;
		this.owner = params.owner;
		this.m_templates = this.dashboard.get_html_templates();
		this.m_model = params.dashboard.get_model_manager();
		this.m_config = params.dashboard.get_config_manager();
		this.m_chart_serie_colors = {
			'MAIL'         : '#FF18F6',
			'CHAT'         : '#0FBBBB',
			'WEBCONTACT'   : '#C56DFF',
			'CALL'         : '#9900FF',
			'INBOUND_CALL' : '#25933D', // defino ambos colores para los nuevos canales
			'OUTBOUND_CALL': '#16009B',
			'IM'           : '#F7CF97',
			'INSTAGRAM'    : '#FD03B2',
			'YOUTUBE'      : '#FF0000',
			'APPSTORE'     : '#2072F3',
			'GOOGLEPLAY'   : '#3C4A5F',
			'RCS'          : '#FCC400',
			'SMS'          : '#00B1FD',
			'WHATSAPP'     : '#00CC66',
			'FACEBOOK'     : '#4267B2',
			'TWITTER'      : '#1DA1F2',
			'VOICEMAIL'    : '#FFA449',
			'TOTAL'        : '#333333'
		};
		this.m_charts_instances = {};
		this.m_allowed_realtime_indicators = {"agentstatussummary": true};
		this.m_allowed_historical_indicators = {"interactions": true, "goodservicelevel": true};
		this.m_realtime_indicators_by_campaigns = {};
		this.m_historical_indicators_by_campaigns = {};
//		this.m_campaigns_legends = {};
		this.m_current_campaign = null;
	}

	sup_full_screen_charts.prototype.is_visible = function (campaign_id)
	{
		return this.m_current_campaign === campaign_id;
	};

	sup_full_screen_charts.prototype.show = function (campaign_id)
	{
		const self = this;

		let current_campaign_id = this.get_current_campaign();
		if (current_campaign_id === campaign_id) {
			return;
		}
		let campaign = inConcert.app.getCampaign(campaign_id);
		if (!campaign) {
			return;
		}

		this.dashboard.update_waiting_message("<%=_T('Please wait')%>");
		this.dashboard.show_waiting();
		$("#sup_dashboard_campaigns_list_charts_container").fadeOut("slow");
		this.__set_current_campaign(campaign_id);

		$("#sup_dashboard_campaigns_list_charts_campaign_name").text(campaign.Name);
		this.__cleanup_legends();
		this.__set_slice_info(campaign_id);

		this.__redraw_charts();
		this.__start_historical_charts_refresh_timer();
		window.setTimeout(function() {
			self.dashboard.hide_waiting();
			$("#sup_dashboard_campaigns_list_charts_container").fadeIn("slow");
		}, 500);
	};

	sup_full_screen_charts.prototype.hide = function ()
	{
		this.__stop_refresh();
		$("#sup_dashboard_campaigns_list_charts_container").fadeOut("slow");
		this.__set_current_campaign(null);
	};

	sup_full_screen_charts.prototype.__cleanup_legends = function ()
	{
		var $legends_container = this.__$get_legends_container();
		$legends_container.html("");
	};

	sup_full_screen_charts.prototype.__set_current_campaign = function (campaign_id)
	{
		this.m_current_campaign = campaign_id;
	};

	sup_full_screen_charts.prototype.get_current_campaign = function ()
	{
		return this.m_current_campaign;
	};

	sup_full_screen_charts.prototype.__is_allowed_chart = function (indicator_id, is_realtime)
	{
		indicator_id = (indicator_id || "").toLowerCase();
		if (is_realtime) {
			return this.m_allowed_realtime_indicators[indicator_id];
		}
		return this.m_allowed_historical_indicators[indicator_id];
	};

	sup_full_screen_charts.prototype.__set_campaign_indicators = function (campaign_id, charts_info, is_realtime)
	{
		if (!campaign_id) {
			return;
		}
		if (is_realtime) {
			this.m_realtime_indicators_by_campaigns[campaign_id] = charts_info;
			return;
		}
		this.m_historical_indicators_by_campaigns[campaign_id] = charts_info;
	};

	sup_full_screen_charts.prototype.__get_campaign_indicators = function (campaign_id, is_realtime)
	{
		if (!campaign_id) {
			return;
		}
		if (is_realtime) {
			return this.m_realtime_indicators_by_campaigns[campaign_id];
		}
		return this.m_historical_indicators_by_campaigns[campaign_id];
	};

	sup_full_screen_charts.prototype.__set_slice_info = function (campaign_id)
	{
		const campaign_dom_id = this.owner.get_normalized_id(campaign_id);
		const $campaign_slice = $('#sup_dash_indicators_slice_select_' + campaign_dom_id);
		const $campaign_time_ago = $('#sup_dash_indicators_time_ago_select_' + campaign_dom_id);

		const translated_slice = {
			'15Min': "<%=_T('15 minutes')%>",
			'30Min': "<%=_T('30 minutes')%>",
			'Hour ': "<%=_T('1 hour')%>",
			'Day'  : "<%=_T('1 day')%>"
		};
		const translated_time_ago = {
			'2Hour': "<%=_T('2 hours')%>",
			'4Hour': "<%=_T('4 hours')%>",
			'8Hour': "<%=_T('8 hours')%>",
			'Week' : "<%=_T('1 week')%>"
		};

		const $full_screen_slice = $('#sup_dash_full_screen_indicators_slice');
		const $full_screen_time_ago = $('#sup_dash_full_screen_indicators_time_ago');

		let campaign_slice = $campaign_slice.val();
		let campaign_time_ago = $campaign_time_ago.val();

		$full_screen_slice.val(translated_slice[campaign_slice] || campaign_slice);
		$full_screen_time_ago.val(translated_time_ago[campaign_time_ago] || campaign_time_ago);
	};

	sup_full_screen_charts.prototype.__get_campaign_slice_minutes = function(campaign_id)
	{
		const slice_minutes = {
			'15Min': 15,
			'30Min': 30,
			'Hour' : 60,
			'2Hour': 60 * 2,
			'4Hour': 60 * 4,
			'8Hour': 60 * 8,
			'Day'  : 60 * 24,
			'Week' : 60 * 24 * 7
		};
		let indicator_panel_config = this.__get_indicators_panel_config(campaign_id);
		let slice = indicator_panel_config && indicator_panel_config.slice ? indicator_panel_config.slice : '15Min';
		let slice_minute = slice_minutes[slice];
		return slice_minute * 60 * 1000;
	};

	sup_full_screen_charts.prototype.__get_indicators_panel_config = function (campaign_id)
	{
		// Va contra el panel indicators de la campana
		return this.m_config.get_panel_config(campaign_id, 'sup_dash_indicators_panel') || {slice: 'Hour', time_ago: '8Hour'};
	};

	sup_full_screen_charts.prototype.setup_charts = function (campaign_id, charts_info, is_realtime)
	{
		const self = this;


		this.__set_slice_info(campaign_id);
		this.__set_campaign_indicators(campaign_id, charts_info, is_realtime);

		let $charts = $("#sup_dashboard_campaigns_list_charts");
		charts_info.forEach(indicator =>
		{
			if (!self.__is_allowed_chart(indicator.id, is_realtime)) {
				return true; // continue
			}
			let indicator_id = nid(indicator.id);
			let $chart = $('#sup_dash_full_screen_indicators_' + indicator_id);

			// create chart if does not exists.
			if ($chart.length > 0) {
				return true; // continue
			}

			let data = indicator.data;
			let slice = self.__get_campaign_slice_minutes(campaign_id);
			//slice >= Day
			if (slice >= 60 * 24 * 1000) {
				slice = 0;
			}

			//add slice duration to timestamps so each point represents the end of a slice
			if (data && data[0] && data[0][0] && typeof(data[0][0]) !== 'string') {
				for (let j = 0; j < data.length; j++) {
					data[j][0] += slice;
				}
			}

			var description = indicator.description;
			$chart = self.__create_new_chart_node(indicator_id, description);

			// append node and build
			$charts.append($chart);

			// if the first element of the data array is an string
			// I will assume that will be a bar chart (string is the legend)
			let chart_type = data[0] && data[0][0] && typeof(data[0][0]) === 'string' ? 'bar' : 'line';
			var tooltip = chart_type === 'bar' ? 'label' : 'y';

			let chart_data = {
				indicator_id : indicator_id,
				indicator_container_id : 'sup_dash_full_screen_indicators_container_' + indicator_id,
				description : description,
				data : [], // data
				chart_type : chart_type,
				tooltip : tooltip,
				containsSecondarySeries : indicator.containsSecondarySeries || false
			};

			// build chart in a closure to avoid frezes
			let check_if_ready_to_finish;
			window.setTimeout(self.__build_chart_closure(chart_data, check_if_ready_to_finish), 0);
		});
	};

	sup_full_screen_charts.prototype.__allowed_to_refresh = function (campaign_id)
	{
		let current_tab = this.owner.get_selected_tab_id();
		if (current_tab !== "dashboard") {
			return false;
		}
		let current_campaign_id = this.get_current_campaign();
		if (current_campaign_id === campaign_id) {
			return true;
		}
		return false;
	};

	sup_full_screen_charts.prototype.__process_indicators = function(campaign_id, is_realtime, callback = $.noop)
	{
		const self = this;

		let indicators = this.__get_campaign_indicators(campaign_id, is_realtime);
		// handle desconnections from inConcert server
		if (_.isEmpty(indicators)){
			return callback();
		}

		this.__set_slice_info(campaign_id);

		indicators.forEach(indicator =>
		{
			if (!self.__is_allowed_chart(indicator.id, is_realtime)) {
				return true; // continue
			}

			let slice = self.__get_campaign_slice_minutes(campaign_id);
			//slice >= Day
			if (slice >= 60 * 24 * 1000) {
				slice = 0;
			}

			var data = indicator.data;
			//add slice duration to timestamps so each point represents the end of a slice
			if (data && data[0] && data[0][0] && typeof(data[0][0]) !== 'string') {
				for (var j = 0; j < data.length; j++) {
					data[j][0] += slice;
				}
			}

			var indicator_id = nid(indicator.id);
			self.__update_chart_data(indicator_id, data, indicator.containsSecondarySeries || false);
		});
	};

	// sup_full_screen_charts.prototype.__make_html_campaign_legends_by_internal_list = function()
	// {
		// const self = this;
		// let campaign_id = this.get_current_campaign();
		// if (!campaign_id) {
			// return;
		// }
		// if (!this.m_campaigns_legends || !this.m_campaigns_legends[campaign_id]) {
			// return;
		// }
		// _.each(this.m_campaigns_legends[campaign_id], function(id, name) {
			// self.__add_legend(id, name);
		// });
	// };

	sup_full_screen_charts.prototype.__update_chart_data = function(indicator_id, new_data, contains_secondary_series)
	{
		const self = this;

		let chart_object = this.get_chart_instance(indicator_id);
		if (!chart_object) {
			return;
		}

		let chart_instance = chart_object.chart;
		let previous_data = chart_object.chart_data;

		if (_.isEqual(previous_data, new_data)) {
//			this.__make_html_campaign_legends_by_internal_list();
			chart_instance.Redraw();
			return;
		}
		chart_object.chart_data = new_data;

		if (!contains_secondary_series) {
			chart_instance.Series.Main.SetDataSource({
				source: new_data
			});
		}
		else {
			let secondary_series = chart_object.secondary_series;
//			this.__cleanup_campaign_legend_list();
			// update each serie data
			_.each(new_data, function(serie, channel)
			{
				let series_names = chart_instance.Series.Secondary.GetSecondarySeriesNames();
				if (series_names.length > 0 && _.contains(series_names, channel)) {
					chart_instance.Series.Secondary.SetDataSource(channel, {
						source: serie.data
					});
				} else if (channel === "TOTAL") {
					self.__add_legend(channel, capitalize(_T(channel)));
					chart_instance.Series.Main.SetDataSource({
						source: serie.data
					});
				}
				else {
					self.__add_legend(channel, capitalize(_T(channel)));
					secondary_series[channel] = self.__get_default_chart_properties(serie.data, chart_object.chart_type, chart_object.tooltip, channel);
				//	self.__add_serie_selector(campaign_id, channel, capitalize(_T(channel)));
				//	var $chk_serie = self.__get_serie_selector_item(campaign_id, channel);
				//	if ($chk_serie.is(":checked")) {
						chart_instance.Series.Secondary.AddSerie(channel, {
								source: serie.data
							},
							{
								// TO-DO: Get this from a function
								type : chart_object.chart_type,
								drawPoints : true,
								lineWidth : 2,
								color : self.m_chart_serie_colors[channel],
								hintString: '<div>['+chart_object.tooltip+']</div>'
							}
						);
				//	}
				}
			});

			_.each(previous_data, function(_, channel) {
				if (!new_data[channel]) {
					if (secondary_series[channel]){
						chart_instance.Series.Secondary.RemoveSerie(channel);
					}
				}
			});
		}

		indicator_id = nid(indicator_id);
		let $chart = $('#sup_dash_full_screen_indicators_' + indicator_id);

		var previous_message = $chart.find('.trendChartMessageContainer');
		if (previous_message.length > 0){
			previous_message.remove();
			$chart.find('.indicatorContentPanelChart').show();
		}

		chart_instance.Redraw();
	};

	sup_full_screen_charts.prototype.refresh_realtime_charts = function (campaign_id, info, callback = $.noop)
	{
		let is_realtime = true;
		this.__set_campaign_indicators(campaign_id, info, is_realtime);
		if (this.__allowed_to_refresh(campaign_id)) {
			this.__process_indicators(campaign_id, is_realtime, function(){
				callback();
			});
		}
	};

	sup_full_screen_charts.prototype.__refresh_historical_charts = function (campaign_id, info, callback = $.noop)
	{
		let is_realtime = false;
		this.__set_campaign_indicators(campaign_id, info, is_realtime);
		if (this.__allowed_to_refresh(campaign_id)) {
			this.__process_indicators(campaign_id, is_realtime, function(){
				callback();
			});
		}
	};

	sup_full_screen_charts.prototype.__load_historical_data = function ()
	{
		const self = this;

		let current_tab = this.owner.get_selected_tab_id();
		if (current_tab !== "dashboard") {
			return;
		}

		if (!this.m_current_campaign) {
			return;
		}

		let data = {
			slice: '15Min',
			time_ago: '2Hour',
			campaign_id: this.m_current_campaign
		};

		var campaign_config = this.dashboard.get_campaign_selected_config(data.campaign_id);
		data.account = !this.dashboard.isAllegro(data.campaign_id) && campaign_config.account_name || null;
		if (data.account) {
			data.channel = campaign_config.channel;
		}
		data.attention_level = campaign_config.al_name;

		this.m_model.get_historical_data(data,
			function (indicators, err) {
				if (indicators && !err) {
					self.__refresh_historical_charts(data.campaign_id, indicators);
				}
			}
		);
	};

	sup_full_screen_charts.prototype.__start_historical_charts_refresh_timer = function ()
	{
		const self = this;
		var refresh;
		refresh = function() {
			self.__stop_refresh();
			self.__load_historical_data();
			self.m_on_refresh_chart_timer = window.setTimeout(function() {
				refresh();
			}, HISTORICAL_CHART_REFRESH_INTERVAL_MS);
		};
		refresh();
	};

	sup_full_screen_charts.prototype.__stop_refresh = function ()
	{
		window.clearTimeout(this.m_on_refresh_chart_timer);
		this.m_on_refresh_chart_timer = null;
	};

	sup_full_screen_charts.prototype.__create_new_chart_node = function(indicator_id, description)
	{
		const chart_template = this.m_templates.full_screen_indicator_chart_tpl;
		return $(Mustache.render(chart_template, {
			title: description || indicator_id,
			id: indicator_id
		}));
	};

	sup_full_screen_charts.prototype.__build_chart_closure = function(chart_data, callback = $.noop)
	{
		const self = this;

		return function()
		{
			// update waiting message
//			self.dashboard.update_waiting_message("<%=_T('Building charts')%>");

			let chart = self.get_chart_instance(chart_data.indicator_id);
			if (chart) {
				callback();
				return;
			}

			// build the real chart, this will call the external charts library
			self.__build_chart(chart_data, function() { 
				callback();
			});

			const $chart = $('#' + chart_data.indicator_container_id);
			// Toggle *not enough data* message.
			if (chart_data.data.length === 0){
				let no_data_msg = Mustache.render(self.m_templates.indicator_no_data_tpl, { message: "<%=_T('Not enough data to display chart')%>" });
				$chart.find('.formIndicatorPanelContent').prepend(no_data_msg);
				$chart.find('.indicatorContentPanelChart').hide();
			} else {
				$chart.find('.trendChartMessageContainer').remove();
				$chart.find('.indicatorContentPanelChart').show();
			}

			// update member with the chart from the library
			self.m_charts_instances[chart_data.indicator_id].chart.Redraw();
		};
	};

	sup_full_screen_charts.prototype.__redraw_charts = function ()
	{
		for (var indicator_id in this.m_charts_instances) {
			if (this.m_charts_instances.hasOwnProperty(indicator_id)) {
				this.m_charts_instances[indicator_id].chart.Redraw();
			}
		}
	};

	sup_full_screen_charts.prototype.__$get_legends_container = function(){
		return $('#sup_dashboard_campaigns_list_charts_legend');
	};

	sup_full_screen_charts.prototype.__$get_legend_item = function(legend_name){
		return $('#sup_dash_full_screen_indicators_legend_item_' + legend_name);
	};

	sup_full_screen_charts.prototype.__legend_already_exists = function(legend_name){
		return this.__$get_legend_item(legend_name).length > 0;
	};

	sup_full_screen_charts.prototype.__$get_panel_node = function() {
		return $('#sup_dash_full_screen_indicators');
	};

	sup_full_screen_charts.prototype.__add_legend = function(channel, name)
	{
		let $legends_container = this.__$get_legends_container();
		if (!this.__legend_already_exists(channel)) {
			let html_legend = Mustache.render(this.m_templates.full_screen_indicator_chart_legend, {channel: channel, name : name});
			$legends_container.append(html_legend);
			this.__$get_panel_node().addClass('showingLegends');
			// let campaign_id = this.get_current_campaign();
			// if (campaign_id) {
				// this.m_campaigns_legends[campaign_id] = this.m_campaigns_legends[campaign_id] || {};
				// this.m_campaigns_legends[campaign_id][channel] = name;
			// }
		}
	};

	// sup_full_screen_charts.prototype.__cleanup_campaign_legend_list = function() {
		// let campaign_id = this.get_current_campaign();
		// if (campaign_id) {
			// delete this.m_campaigns_legends[campaign_id];
		// }
	// };

	sup_full_screen_charts.prototype.__build_chart = function (chart_data, onAfterBuild)
	{
		const self = this;

		let main_serie = this.__get_default_chart_properties(chart_data.data, chart_data.chart_type, chart_data.tooltip);
		let secondary_series = {};

		// dealing with secondary series?
		if (chart_data.containsSecondarySeries) {
			// show leyends container
//			this.__cleanup_campaign_legend_list();
			this.__$get_legends_container().show();

			_.each(chart_data.data, function(serie, serie_id)
			{
				// create legend
				self.__add_legend(serie_id, capitalize(_T(serie_id)));
				// build basic chart data
				let serie_data = self.__get_default_chart_properties(serie.data, chart_data.chart_type, chart_data.tooltip, serie_id);
				// we always need to specify a main serie for this chart
				// so we make 'TOTAL' indicator our main serie
				if (serie_id === 'TOTAL'){
					main_serie = serie_data; // override previous declared main serie
				} else {
					secondary_series[serie_id] = serie_data;
					// TOTAL indicator is not shown on serie selection
//self.__add_serie_selector(serie_id, capitalize(_T(serie_id)));
				}
			});
		}

		let chart = new inconcert.Chart({
			div: chart_data.indicator_container_id,
			ChartProperties : {
				title : chart_data.description,
				allowZoom: false,
				useUTC : true,
				minExtreme: 0,
				yFormatter: function(n) {
					let format = self.__chart_Y_formatter(n);
					return format;
				},
				xFormatter: function(timestamp) {
					let date = new Date(timestamp);
					return zeropad(date.getUTCHours()) + ':' + zeropad(date.getUTCMinutes());
				},
				xNeedsTicks: function (min, max) {
					let ticks = self.__chart_X_need_ticks(min, max);
					return ticks;
				}
			},
			Series: {
				Properties: {
					showSeriesSelector: false
				},
				Main: main_serie,
				Secondary: secondary_series
			},
			onAfterBuild: onAfterBuild
		});

		this.m_charts_instances[chart_data.indicator_id] = {
			chart_data       : chart_data.data,
			secondary_series : secondary_series,
			chart_type       : chart_data.chart_type,
			tooltip          : chart_data.tooltip,
			chart            : chart
		};
	};

	sup_full_screen_charts.prototype.__chart_Y_formatter = function(n)
	{
		if (Math.floor(n) !== n){
			return '';
		}
		//Scales the value and adds the corresponding unit prefix
		//so instead of displaying '1000000' this will display '1M'
		const suffix = ['', 'K', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y', '*10^27', '*10^30', '*10^33', '*10^36', '*10^39'];
		const sign = n < 0 ? '-' : '';

		n = Math.abs(n);
		let i = 0;

		while (n >= 1000) {
			n /= 1000;
			i++;
		}
		//round to 2 decimals
		n = Math.round(n * 100) / 100;
		return sign + n + suffix[i];
	};

	sup_full_screen_charts.prototype.__chart_X_need_ticks = function(min, max)
	{
		if (!min || !inConcert.Utils.isNumber(min)) {
			return;
		}
		let campaign_id = this.get_current_campaign();
		if (!campaign_id) {
			return;
		}
		//create ticks so they are slice aligned
		let slice = this.__get_campaign_slice_minutes(campaign_id);
		//slice >= Day
		if (slice >= 60 * 24 * 1000) {
			slice = 0;
		}

		let ticks = [];
		let init = Math.floor(min);
		init = init - (init % slice);
		let tick = init;
		let current_config = this.__get_indicators_panel_config(campaign_id);

		while (tick <= max) {
			if (current_config.slice === 'Day') {
				let date = new Date(tick);
				ticks.push([tick, zeropad(date.getUTCDate()) + '/' + zeropad(date.getUTCMonth() + 1)]);
			}
			else {
				//use default label (specified in xFormatter)
				ticks.push([tick, null]);
			}
			tick = tick + slice;
			if (ticks.length > 20) {
				//we don't want more than 20 labels
				//because it can slow things down
				return ticks;
			}
		}
		return ticks;
	};

	sup_full_screen_charts.prototype.get_chart_instance = function(chart_id) {
		return this.m_charts_instances[chart_id];
	};

	sup_full_screen_charts.prototype.__get_default_chart_properties = function(data, type, tooltip, serie_id){
		return {
			DataSource: {
				source: data
			},
			Properties : {
				type : type,
				drawPoints : true,
				lineWidth : 2,
				color : serie_id ? this.m_chart_serie_colors[serie_id] : '#46b3e6',
				hintString: '<div>['+tooltip+']</div>'
			}
		};
	};

	//---------------------------------------------------------------------
	// form full screen class
	//
	const FULL_SCREEN_VIEW = 'FULL_SCREEN';
	const SIMPLE_SCREEN_VIEW = 'SIMPLE_SCREEN';
	const FULL_SCREEN_CLASS = 'supervisorFullScreenMode';

	function sup_full_screen (params) {
		this.dashboard = params.dashboard;
		this.m_campaign_id_in_full_screen = null;
		this.m_tabs_campaigns = null;
		this.m_timeout_save_tab = null;
		this.m_timeout_grid_update_row = null;
		this.m_grid_campaigns = null;
		this.m_charts = null;
		this.__initialize_tabs();
		this.__initialize_grid();
		this.__initialize_charts();
		this.set_events();
	}

	if (typeof inConcert.sup_full_screen !== "function") {
		inConcert.sup_full_screen = sup_full_screen;
	}

	sup_full_screen.prototype.get_chart_instance = function()
	{
		return this.m_charts;
	};

	sup_full_screen.prototype.__initialize_charts = function()
	{
		this.m_charts = new sup_full_screen_charts({
			dashboard: this.dashboard,
			owner    : this
		});
		//this.m_charts.init();
	};

	// sup_full_screen.prototype.full_screen_setup_charts = function()
	// {
		// this.m_charts = new sup_full_screen_charts({
			// dashboard: this.dashboard
		// });
		// //this.m_charts.init();
	// };

	/*
		Crea los Tabs de las campanas que se visualiza en modo Pantalla Completa
	*/
	sup_full_screen.prototype.__initialize_tabs = function()
	{
		const self = this;
		// TODO: ver de implementar o hacer algo con esto !!
		this.editable_all_crtls = $.noop;
		this.disable_all_crtls = $.noop;

		// Instancia de la clase de los TABs de la App "Configuration"
		this.m_tabs_campaigns = new inConcert.conf_form_tab({
			frm            : this,
			container_id   : "sup_campaigns_tabs_container",
			must_initialize: true,
			tabs_list      : {
				dashboard: {
					id              : "dashboard",
					name            : "<%=_T('Dashboard')%>",
					tab_container_id: "sup_dashboard_container",
					selected        : true,
					on_show: function () {
						self.__on_show_tab_campaign("dashboard");
						self.toggle_dashboard_title_container(true);
					}
				}
			},
			callbacks: {
				on_show_tab: function(select_tab_id) {
					if (select_tab_id === "dashboard") {
						return;
					}
					const tab_config = self.m_tabs_campaigns.get_tab_config(select_tab_id);
					const campaign = inConcert.app.getCampaign(tab_config.id);
					const $navigation_btn = $("#sup_full_screen_header_navigation_btn");
					$navigation_btn.prop("value", tab_config.id);
					$navigation_btn.find('label').text(campaign.Name);

					self.update_campaign_title();
					self.fill_campaigns_navigation(select_tab_id);
					self.dashboard.update_main_dashbord_layout(tab_config.id);
					self.dashboard.enable_disable_btn_campaign_accounts_selector(select_tab_id);
					self.dashboard.campaignBotsSelectFilterOption(select_tab_id);
					const $menu_option_control_panel = $('[action_id="control_panel"]');
					$menu_option_control_panel.removeClass("selected").prop("disabled", false);
					const $menu_option_hide_full_screen = $('[action_id="campaign_hide_full_screen"]');
					$menu_option_hide_full_screen.removeClass("selected").prop("disabled", false);
				},
				on_add_tab: function() {
					self.fill_campaigns_navigation();
				},
				on_remove_tab: function() {
					self.fill_campaigns_navigation();
					const current_campaign = self.get_campaign_id();
					if ( Object.keys(self.m_tabs_campaigns.tabs_list) === 1 || !current_campaign || !self.m_tabs_campaigns.tabs_list[current_campaign]) {
						//Solo anda el dashboard
						self.show_control_panel();
					}
				},
			}
		});
		//Se mantiene oculto pq se va a usar dropdown como selector de campannas
		this.m_tabs_campaigns.hide_tabs_container();

		// Los Tabs correspondientes a cada Campana se agregan cuando las campanas se reenderizan de forma efectiva en el DOM (SupervisorDashboard.__render_campaign->full_screen_add_tab_campaign->this.add_tab_campaign)
	};

	sup_full_screen.prototype.sanitize_value = function(value)
	{
		return this.dashboard.sanitize_value(value);
	};

	sup_full_screen.prototype.__initialize_grid = function()
	{
		const self = this;

		let grid_add_customs_attrs_to_cell = function($cell, column_def, row_data) {
			self.__grid_add_customs_attrs_to_cell($cell, column_def, row_data);
		};

		this.m_grid_campaigns = new inConcert.conf_form_paginator({
			owner: this,
			frm: this,
			row_id: "Id",
			display_name: "Name",
			auto_initialize: true,
			rows_per_page: 8,
			show_at_least: 7,
			without_paginator: false,
			loading_waiting_msg: "<%=_T('Please wait, loading Campaigns ...')%>",
			$custom_waiting_container: $("#genericWaiting"),
			ids: {
				paginator: "sup_dashboard_campaigns_paginator",
				grid: "sup_dashboard_campaigns_list",
				not_results_found: "sup_dashboard_campaigns_not_results_found"
			},
			strings: {
				not_results_found: "<%=_T('No campaigns found.')%>"
			},
			columns: [
				{id: "_number", name: "#", type: "counter"},
				{id: "Name"   , name: "<%=_T('Name')%>"  },
				{id: "scope"   , name: "<%=_T('Scope')%>",
					get_value: function(item) {
						return self.__get_grid_value(item, "scope");
					}
				},
				{id: "slice"   , name: "<%=_T('Slice')%>",
					get_value: function(item) {
						return self.__get_grid_value(item, "slice");
					}
				},
				{id: "_Members"   , name: "<%=_T('Members')%>"}, // Esta columna se actualiza de forma dinamica en la funcion "update_by_members"
				{id: "active_campaign_total"   , name: "<%=_T('Active')%>",
					// (**) Definicion de los indicadores de las columnas, por los cuales se actualizara la celda de la row de forma dinamica en la funcion "update_by_summary". Cuando se realiza la  actualizacion de summary.
					channel_indicators: [
						{channel: "call"      , indicator_id: "active_call"      },
						{channel: "chat"      , indicator_id: "active_chat"      },
						{channel: "facebook"  , indicator_id: "active_facebook"  },
						{channel: "im"        , indicator_id: "active_im"        },
						{channel: "mail"      , indicator_id: "active_mail"      },
						{channel: "sms"       , indicator_id: "active_sms"       },
						{channel: "twitter"   , indicator_id: "active_twitter"   },
						{channel: "webcontact", indicator_id: "active_webcontact"},
						{channel: "whatsapp"  , indicator_id: "active_whatsapp"  },
						{channel: "instagram"  , indicator_id: "active_instagram"  }
					],
					total_indicator_id: "active_campaign_total",
					add_customs_attrs: grid_add_customs_attrs_to_cell
				},
				{id: "queued_campaign_total"   , name: "<%=_T('Queued')%>",
					channel_indicators: [
						{channel: "call"      , indicator_id: "queued_call"      },
						{channel: "chat"      , indicator_id: "queued_chat"      },
						{channel: "facebook"  , indicator_id: "queued_facebook"  },
						{channel: "im"        , indicator_id: "queued_im"        },
						{channel: "mail"      , indicator_id: "queued_mail"      },
						{channel: "sms"       , indicator_id: "queued_sms"       },
						{channel: "twitter"   , indicator_id: "queued_twitter"   },
						{channel: "webcontact", indicator_id: "queued_webcontact"},
						{channel: "whatsapp"  , indicator_id: "queued_whatsapp"  },
						{channel: "instagram"  , indicator_id: "queued_instagram"  }
					],
					total_indicator_id: "queued_campaign_total",
					add_customs_attrs: grid_add_customs_attrs_to_cell
				},
				{id: "waiting_campaign_total"   , name: "<%=_T('Waiting')%>",
					channel_indicators: [
						{channel: "call"      , indicator_id: "waiting_call"      },
						{channel: "chat"      , indicator_id: "waiting_chat"      },
						{channel: "facebook"  , indicator_id: "waiting_facebook"  },
						{channel: "im"        , indicator_id: "waiting_im"        },
						{channel: "mail"      , indicator_id: "waiting_mail"      },
						{channel: "sms"       , indicator_id: "waiting_sms"       },
						{channel: "twitter"   , indicator_id: "waiting_twitter"   },
						{channel: "webcontact", indicator_id: "waiting_webcontact"},
						{channel: "whatsapp"  , indicator_id: "waiting_whatsapp"  },
						{channel: "instagram"  , indicator_id: "waiting_instagram"  }
					],
					total_indicator_id: "waiting_campaign_total",
					add_customs_attrs: grid_add_customs_attrs_to_cell
				},
				{id: "total_campaign_total"   , name: "<%=_T('Total')%>",
					channel_indicators: [
						{channel: "call"      , indicator_id: "total_call"      },
						{channel: "chat"      , indicator_id: "total_chat"      },
						{channel: "facebook"  , indicator_id: "total_facebook"  },
						{channel: "im"        , indicator_id: "total_im"        },
						{channel: "mail"      , indicator_id: "total_mail"      },
						{channel: "sms"       , indicator_id: "total_sms"       },
						{channel: "twitter"   , indicator_id: "total_twitter"   },
						{channel: "webcontact", indicator_id: "total_webcontact"},
						{channel: "whatsapp"  , indicator_id: "total_whatsapp"  },
						{channel: "instagram"  , indicator_id: "total_instagram"  }
					],
					total_indicator_id: "total_campaign_total",
					add_customs_attrs: grid_add_customs_attrs_to_cell
				},
				{id: "abandoned_campaign_total"   , name: "<%=_T('Abandoned %')%>",
					get_value: function(item) {
						return self.__get_grid_value(item, "abandoned_campaign_total");
					},
					channel_indicators: [
						{channel: "call"      , indicator_id: "abandoned_call"      },
						{channel: "chat"      , indicator_id: "abandoned_chat"      },
						{channel: "facebook"  , indicator_id: "abandoned_facebook"  },
						{channel: "im"        , indicator_id: "abandoned_im"        },
						{channel: "mail"      , indicator_id: "abandoned_mail"      },
						{channel: "sms"       , indicator_id: "abandoned_sms"       },
						{channel: "twitter"   , indicator_id: "abandoned_twitter"   },
						{channel: "webcontact", indicator_id: "abandoned_webcontact"},
						{channel: "whatsapp"  , indicator_id: "abandoned_whatsapp"  },
						{channel: "instagram"  , indicator_id: "abandoned_instagram"  }
					],
					total_indicator_id: "abandoned_campaign_total",
					add_customs_attrs: grid_add_customs_attrs_to_cell
				},
				{id: "_action", name: "",
					custom_action: {
						edit: {
							name: "<%=_T('Members')%>",
							execute: function($row /*, $owner*/) {
								var campaign = $row.data("row_data");
								self.__open_popup_members(campaign.Id);
							},
							class_name: "buttons-icons-group-edit"
						},
						view: {
							name: "<%=_T('View')%>",
							execute: function($row /*, $owner*/) {
								const campaign = $row.data("row_data");
								const config = {
									full_screen: true,
									campaign_id: campaign.Id
								};
								self.dashboard.full_screen_toggle_visibility(config);
							},
							class_name: "buttons-icons-view"
						},
						charts: {
							name: "<%=_T('Indicators')%>",
							execute: function($row /*, $owner*/) {
								var campaign = $row.data("row_data");
								self.__toggle_charts(campaign.Id, $row);
							},
							class_name: "buttons-icons-indicators"
						}
					}
				}
			],
			callbacks: {
				load_items: function(_, callback) {
					self.__load_campaigns(callback);
				}
			}
		});
	};

	sup_full_screen.prototype.__grid_add_customs_attrs_to_cell = function($cell, column_def, row_data)
	{
		$cell.removeAttr("indicator_id");
		let campaign_id = row_data.Id;
		let campaign = inConcert.app.getCampaign(campaign_id);
		if (!campaign) {
			return;
		}
		let accounts = campaign.get_accounts();
		if (!accounts || accounts.length === 0) {
			return;
		}
		let indicator_id;
		let total_accounts = accounts.length;
		if (total_accounts === 1) {
			let account = accounts[0];
			column_def.channel_indicators.forEach(channel_indicator => {
				if (account.proxyType.toLowerCase() === channel_indicator.channel.toLowerCase()) {
					indicator_id = channel_indicator.indicator_id;
					return false; // break
				}
			});
		}
		else {
			indicator_id = column_def.total_indicator_id;
		}
		if (indicator_id) {
			$cell.attr("indicator_id", indicator_id);
			this.__show_alarm(campaign_id, $cell, indicator_id);
		}
	};

	sup_full_screen.prototype.is_full_screen = function()
	{
		if (!this.m_campaign_id_in_full_screen) {
			return false;
		}
		return true;
	};

	sup_full_screen.prototype.get_campaign_id = function()
	{
		return this.m_campaign_id_in_full_screen;
	};

	sup_full_screen.prototype.__set_campaign_id = function(campaign_id)
	{
		this.m_campaign_id_in_full_screen = campaign_id;
	};

	sup_full_screen.prototype.show_waiting = function(msg)
	{
		this.dashboard.update_waiting_message(msg);
		this.dashboard.show_waiting();
	};

	sup_full_screen.prototype.hide_waiting = function()
	{
		this.dashboard.hide_waiting();
	};

	sup_full_screen.prototype.is_readonly = function()
	{
		return false;
	};

	sup_full_screen.prototype.__load_campaigns = function(callback = $.noop)
	{
		callback(true, []);
	};

	sup_full_screen.prototype.set_events = function() {
		this.set_event_nav_campaign_items();
	};

	sup_full_screen.prototype.__get_grid_value = function(item, field_id)
	{
		let value = {
			original_value: "",
			value         : "",
			search_value  : "",
			class_name    : ""
		};

		let item_value = item[field_id];
		if (item_value === undefined) {
			item_value = "";
		}

		value.original_value = item_value;
		value.value = item_value;
		value.search_value = item_value;

		let campaign_id = item.Id;
		if (field_id === "slice") {
			let config_manager = this.dashboard.get_config_manager();
			if (config_manager) {
				var summary_config = config_manager.get_panel_config(campaign_id, "sup_dash_summary_panel");
				var slice = summary_config && summary_config.realtime_window ? summary_config.realtime_window : "Last15Minutes";
				var slices_def = this.dashboard.get_slices();
				value.value = slices_def[slice] || item_value;
				value.search_value = value.value;
			}
		}
		else if (field_id === "scope") {
			let campaign = inConcert.app.getCampaign(campaign_id);
			if (campaign) {
				let camp_scope = "<%=_T('Campaign')%>";
				if (!campaign.isAllegro()) {
					let campaign_config = this.dashboard.get_selected_campaign_account_al(campaign_id);
					if (campaign_config) {
						if (campaign_config.account_name) {
							camp_scope = camp_scope + "->" + campaign_config.account_name;
							if (campaign_config.al_name) {
								camp_scope = camp_scope + "->" + campaign_config.al_name;
							}
						}
					}
				}
				value.original_value = camp_scope;
				value.value = camp_scope;
				value.search_value = camp_scope;
			}
		}
		else if (field_id === "abandoned_campaign_total") {
			if (item_value !== "") {
				if (item_value === 0) {
					item_value = "0.00";
				}
				else {
					let suffix = "";
					let round_value = Math.round(item_value * 100) / 100;
					if (round_value === parseInt(item_value, 10)) {
						suffix = ".00";
					}
					item_value = round_value.toString() + suffix;
				}
				value.value = item_value + " %";
			}
		}

		return value;
	};

	sup_full_screen.prototype.__show_alarm = function (campaign_id, $node, indicator_id)
	{
		// (ALARMS) Marco/desmarco el indicador como alarmado.
		var campaign_config = this.dashboard.get_campaign_selected_config(campaign_id);
		var alarm_config = this.dashboard.get_notification_alarm_by_indicator({
			indicator_id: indicator_id,
			campaign_id : campaign_id,
			account_id  : campaign_config.account_id,
			al_id       : campaign_config.al_id
		});
		return this.dashboard.show_alarm($node, alarm_config);
	};

	sup_full_screen.prototype.make_tab_caption = function(campaign_id, alarm_number)
	{
		let tab_config = this.m_tabs_campaigns.get_tab_config(campaign_id);
		if (!tab_config) {
			return;
		}
		let tab_caption = this.__make_tab_caption(tab_config, alarm_number || 0);
		this.m_tabs_campaigns.set_caption_tab(campaign_id, tab_caption);
	};

	/*
		zzzzz
	*/
	sup_full_screen.prototype.notify_alarms = function(campaign_id/*, alarms*/)
	{
		let tab_config = this.m_tabs_campaigns.get_tab_config(campaign_id);
		if (!tab_config) {
			return;
		}
		// el numero de alarmas a mostrar lo sacamos del numero de alarmas activas, porque las alarmas de kickout siguen llegando en el parametro 'alarms' aun cuando se hayan leido
		let alarm_number = 0;
		$.each(this.dashboard.m_alarms_notifications.live_alarms, function(_ /*live_alarm_id*/, live_alarm) {
			if(live_alarm.campaign_id === campaign_id) {
				alarm_number++;
			}
		});
		let tab_caption = this.__make_tab_caption(tab_config, alarm_number);
		this.m_tabs_campaigns.set_caption_tab(campaign_id, tab_caption);
		this.__show_or_hide_header_notify_alarms(campaign_id, alarm_number);
	};

	sup_full_screen.prototype.__show_or_hide_header_notify_alarms = function(campaign_id, alarm_number)
	{
		let current_tab = this.get_selected_tab_id();
		if (current_tab === "dashboard") {
			const totals_alarms = this.dashboard.get_totals_alarms();
			alarm_number = totals_alarms.total_live;
		}
		else if (current_tab !== campaign_id) {
			return;
		}
		let html = '<i class="far fa-bell-exclamation"></i>';
		if (alarm_number > 0) {
			const has_permission = this.dashboard.has_permission_to_any_campaign('sup_view_alarms_config');
			if (has_permission) {
				const html_alarm = this.__make_alarm_notification_markup(alarm_number);
				html = html + html_alarm;
			}
		}
		const $full_screen_campaign_alarms = $('#sup_full_screen_header_container [full_screen_header_buttons] [action_id="campaign_alarms"]');
		$full_screen_campaign_alarms.html(html);
	};

	/*
		zzzzz
	*/
	sup_full_screen.prototype.__update_row_grid = function(row_data)
	{
		const self = this;
		this.m_grid_campaigns.update_row(row_data, function() {
			if (self.m_timeout_grid_update_row) {
				window.clearTimeout(self.m_timeout_grid_update_row);
				this.m_timeout_grid_update_row = null;
			}
			self.m_timeout_grid_update_row = window.setTimeout(function() {
				self.m_grid_campaigns.__update_column_counter();
			}, 500);
		});
	};

	/*
		Esta funcion recibe informacion del panel de Summary y con esta informacion actualiza las columnas correspondientes de la grilla.
	*/
	sup_full_screen.prototype.update_by_summary = function(info)
	{
		const self = this;
		//console.log("sup_full_screen.prototype.update_by_summary --> " + JSON.stringify(info || {}));
		if (!info) {
			return;
		}
		// 1- Obtiene las columnas de la grilla.
		let columns = this.m_grid_campaigns.get_columns(false);
		if (!columns) {
			return;
		}
		// 1.1- Valida que la campana exista
		let campaign = inConcert.app.getCampaign(info.campaign_id);
		if (!campaign) {
			return;
		}
		// 1.2- Valida que al menos tenga 1 cuenta
		let accounts = campaign.get_accounts();
		if (!accounts || accounts.length === 0) {
			return;
		}
		// 2- Obtiene la row correspondiente a la campana a actualizar.
		let $row = this.m_grid_campaigns.$find_row(info.campaign_id);
		if (!$row || $row.length === 0) {
			return;
		}
		let row_data = this.m_grid_campaigns.get_row_data($row);
		if (!row_data) {
			return;
		}
		let summary_data = info.data;
		// Funcion que obtiene los indicadores summary
		var get_summary_indicators = function(id)
		{
			let summary_indicators;
			summary_data.forEach(data => {
				if ((data.ID || "").toLowerCase() === id.toLowerCase()) {
					summary_indicators = data.indicators;
					return false; // break
				}
			});
			return summary_indicators;
		};

		// Funcion que valida si la validacion se debe realizar contra el indicador total de la campana, o si lo tiene que hacer por algun canal especifico.
		var has_to_validate_a_single_channel = function()
		{
			if (campaign.isAllegro()) {
				// En el caso de Campanas de Allegro, siempre valida contra el indicador total de la campana (3.2 'campaignTotal')
				return false;
			}
			let campaign_channels = campaign.get_channels();
			if (campaign_channels.CALL) {
				// En el caso de Campanas con canales de telefonia siempre va contra el total de la campana (3.2 'campaignTotal')
				return false;
			}
			let total_accounts = accounts.length;
			if (total_accounts === 1) {
				// Si la campana tiene una unica cuenta voy contra los indicadores de los canales (3.1.2)
				return true;
			}
			let total_channels = Object.keys(campaign_channels).length;
			if (total_channels === 1) {
				// Si la campana tiene mas de una cuenta, pero todas del mismo canal voy contra los indicadores de los canales (3.1.2)
				return true;
			}
			let campaign_config = self.dashboard.get_selected_campaign_account_al(info.campaign_id);
			if (campaign_config) {
				if (campaign_config.account_name) {
					// Si en el selector de campanas el scope es Account o AL, voy contra los indicadores de los canales (3.1.2)
					return true;
				}
			}
			// Voy contra (3.2 'campaignTotal'), si la campana tiene mas de una cuenta de diferentes canales y no esta monitreando una account o AL especifico
			return false;
		};

		let validate_single_channel = has_to_validate_a_single_channel();
		let have_to_update_row = false;

		// 3 - Recorro las columnas
		columns.forEach(column =>
		{
			// 3.1 Recorro la definicion de los indicadores de tipo channel de la columna (**)
			let new_column_value = null;
			if (validate_single_channel) {
				(column.channel_indicators || []).forEach((column_channel_indicators) =>
				{
					// 3.1.1 Obtengo los indicadores de summary para el channel
					let summary_indicators_channel = get_summary_indicators(column_channel_indicators.channel);
					if (!summary_indicators_channel) {
						return true; // continue
					}
					// 3.1.2 Recorro la lista de indicadores del channel de los datos del summary
					summary_indicators_channel.forEach(summary_indicator_channel =>
					{
						// 3.1.3 Ahora valido que sean los mismos indicadores para poder obtener su valor
						if (summary_indicator_channel.ID.toLowerCase() === column_channel_indicators.indicator_id) {
							new_column_value = (new_column_value || 0) + Number(summary_indicator_channel.total_source);
							return false; // break
						}
					});
					return false; // break
				});
			}
			else {
				// 3.2 Obtengo los indicadores de summary para el Total de la campana
				let summary_indicators_camp_total = get_summary_indicators("campaignTotal");
				// 3.3 Recorro la lista de indicadores del channel de los datos del summary
				(summary_indicators_camp_total || []).forEach(summary_indicator_camp_total =>
				{
					// 3.3.1 Ahora valido que sean los mismos indicadores para poder obtener su valor
					if (summary_indicator_camp_total.ID.toLowerCase() === column.total_indicator_id) {
						new_column_value = (new_column_value || 0) + Number(summary_indicator_camp_total.total_source);
						return false; // break
					}
				});
			}
			if (new_column_value !== null && row_data[column.id] !== new_column_value) {
				// 3.4 Si el valor correspondiente a la columna ha cambiado, actualizo la info de la campana, para luego actualizar en la grilla (4)
				// Tambien flagueo "have_to_update_row" para saber que la row debe ser actualizada
				row_data[column.id] = new_column_value;
				have_to_update_row = true;
			}
		});

		if (have_to_update_row) {
			// 4.1 Si algun dato de la campana ha cambiado, actualizo la row de la grilla.
			this.__update_row_grid(row_data);
		}
	};

	/*
		Esta funcion recibe informacion del panel de Members y con esta informacion actualiza las columnas correspondientes de la grilla.
	*/
	sup_full_screen.prototype.update_by_members = function(info)
	{
		//console.log("sup_full_screen.prototype.update_by_members --> " + JSON.stringify(info || {}));
		if (!info || info.data.total_members === undefined) {
			return;
		}
		// 1- Obtiene la columna que contiene la cantidad de members
		let column = this.m_grid_campaigns.get_column("_Members");
		if (!column) {
			return;
		}
		let row_data = this.m_grid_campaigns.find_row_data(info.campaign_id);
		// 2- Obtiene la row correspondiente a la campana a actualizar.
		if (!row_data) {
			return;
		}
		let total_members = Number(info.data.total_members);
		let have_to_update_row = false;
		if (row_data._Members !== total_members) {
			// 3 Si el valor correspondiente a la columna que ha cambiado, actualizo la info de la campana, para luego actualizar en la grilla (4)
			// Tambien flagueo "have_to_update_row" para saber que la row debe ser actualizada
			row_data._Members = total_members;
			have_to_update_row = true;
		}
		if (have_to_update_row) {
			// 4 Actualizo la row de la grilla.
			this.__update_row_grid(row_data);
		}
	};

	sup_full_screen.prototype.__make_alarm_notification_markup = function(alarm_number)
	{
		let template =
			"<div alarm_number_container='true' class='notifyNumberContainer' style='display:{0};'>" +
//				"<span alarm_unread_number='0' class='notifyLiveUnreadNumber'></span>" +
				"<span alarm_live_number='{1}' class='notifyLiveNumber'>{2}</span>" +
			"</div>";

		let display = "none";
		let alarm_value = "0";
		if (parseInt(alarm_number, 10) > 0) {
			display = "";
			alarm_value = alarm_number;
		}

		return FORMAT(template, display, alarm_value, alarm_value);
	};

	sup_full_screen.prototype.__make_tab_caption = function(tab_config, alarm_number)
	{
		let campaign = tab_config.__campaign;
		let workgroups = this.dashboard.get_workgroup_manager().get_campaign_workgroups(campaign.Id);
		let config = {
			config: this.dashboard.get_campaign_selected_config(campaign.Id)
		};
		let tab_caption = this.dashboard.get_campaign_title(campaign.Id, campaign, workgroups, config);
		let current_tab = this.get_selected_tab_id();
		if (current_tab === campaign.Id) {
			return tab_caption;
		}
		return tab_caption + this.__make_alarm_notification_markup(alarm_number);
	};

	sup_full_screen.prototype.add_tab_campaign = function(campaign, selected = false, on_changed = $.noop)
	{
		const self = this;
		if (typeof campaign !== "object") {
			return false;
		}
		let campaign_config = this.dashboard.get_stored_campaign_config(campaign.Id);
		if (campaign_config && !campaign_config.visible) {
			return false;
		}
		let tab_config = {
			__campaign      : campaign,
			id              : campaign.Id,
			name            : function(config) {
				const alarm_number = 0;
				return self.__make_tab_caption(config, alarm_number);
			},
			tab_container_id: "sup_dash_campaign_" + this.get_normalized_id(campaign.Id),
			on_show         : function(config) {
				let campaign_id = config.id;
				self.__on_show_tab_campaign(campaign_id);
				on_changed();
			}
		};
		this.m_tabs_campaigns.add_tab(tab_config, selected);
		return true;
	};

	sup_full_screen.prototype.remove_tab_campaign = function(campaign_id)
	{
		if (!campaign_id) {
			return;
		}
		this.m_tabs_campaigns.remove_tab(campaign_id);
		if (this.m_started_dashboard_campaigns_charts) {
			delete this.m_started_dashboard_campaigns_charts[campaign_id];
		}
	};

	sup_full_screen.prototype.on_removed_campaign = function(campaign_id, callback = $.noop)
	{
		// remuevo el tab
		this.remove_tab_campaign(campaign_id);
		// elimino la row de la grilla
		this.m_grid_campaigns.remove_dom_row_by_id(campaign_id, callback);
		let chart_instance = this.get_chart_instance();
		if (chart_instance) {
			let chart_campaign_id = chart_instance.get_current_campaign();
			if (chart_campaign_id === campaign_id) {
				chart_instance.hide();
			}
		}
	};

	sup_full_screen.prototype.get_total_grid_rows = function()
	{
		let total_rows = this.m_grid_campaigns.count_rows();
		return total_rows;
	};

	sup_full_screen.prototype.on_added_campaign = function(campaign_id)
	{
		const self = this;
		let campaign = inConcert.app.getCampaign(campaign_id);
		if (!campaign) {
			return false;
		}
		// agrego el tab
		let is_added = this.add_tab_campaign(campaign);
		if (!is_added) {
			return false;
		}
		// agrego el row a la grilla
		let maximum_number_of_monitorable_campaigns = this.dashboard.get_maximum_number_of_monitorable_campaigns();
		let total_rows = this.m_grid_campaigns.count_rows();
		if (total_rows < maximum_number_of_monitorable_campaigns) {
			this.m_grid_campaigns.add_or_update_row(campaign, undefined, undefined, function() {
				self.__update_row_grid(campaign);
			});
			return true;
		}
		return false;
	};

	sup_full_screen.prototype.__open_popup_members = function (campaign_id)
	{
		this.dashboard.open_popup_members(campaign_id);
	};

	sup_full_screen.prototype.goto_tab_campaign = function(tab_id, on_changed = $.noop)
	{
		if (!this.m_tabs_campaigns || !tab_id) {
			return false;
		}
		this.toggle_campaigns_nav(true);
		var exist_tab = this.m_tabs_campaigns.is_tab(tab_id);
		if (exist_tab) {
			const force_switch = true;
			this.m_tabs_campaigns.show_tab_with_content(tab_id, function() {
					on_changed();
				},
				force_switch
			);
			return true;
		}
		var campaign = inConcert.app.getCampaign(tab_id);
		if (!campaign) {
			return false;
		}
		var selected = true;
		this.add_tab_campaign(campaign, selected, on_changed);
	};

	sup_full_screen.prototype.__toggle_charts = function(campaign_id, $row)
	{
		if (this.m_charts.is_visible(campaign_id)) {
			this.m_grid_campaigns.unmark_row($row);
			this.m_charts.hide();
		}
		else {
			this.m_grid_campaigns.unmark_all_rows();
			this.m_grid_campaigns.mark_row($row);
			this.m_charts.show(campaign_id);
		}
	};

	sup_full_screen.prototype.on_change_config_selector = function()
	{
		this.update_campaign_title();

		const selected_tab = this.get_campaign_id();
		if (selected_tab !== "dashboard") {
			return;
		}
		// Por el momento es para actualizar la columna "ALCANCE".
		this.dashboard_refresh_all_client_side_rows();
	};

	sup_full_screen.prototype.dashboard_refresh_all_client_side_rows = function()
	{
		if (!this.m_grid_campaigns) {
			return;
		}
		this.m_grid_campaigns.refresh_all_client_side_rows();
	};

	sup_full_screen.prototype.__on_show_tab_campaign = function(tab_id)
	{
		const self = this;
		this.__set_campaign_id(tab_id);
		this.__update_panel_positions();
		this.update_toolbar_buttons();

		const update_dashboard_rows = function() {
			if (tab_id === "dashboard") {
				// Si entramos al TAB dashboard, refresco todas las rows.
				// Mas que nada para la columna "Slice" ya que este dato no llega en las actualizaciones de summary.
				self.dashboard_refresh_all_client_side_rows();
			}
		};

		const update_toolbar_view = function(){
			if (tab_id !== "dashboard"){
				self.__update_toolbar_view({campaign_id:tab_id, full_screen:true});
			}
		};

		if (this.m_timeout_save_tab) {
			window.clearTimeout(this.m_timeout_save_tab);
			this.m_timeout_save_tab = null;
		}
		this.m_timeout_save_tab = window.setTimeout(function() {
			self.save_config(function() {
				if (tab_id === "dashboard") {
					update_dashboard_rows();
					// Si se esta visualizando una grafica, debo de dejar la row correspondiente a la campana marcada.
					let chart_instance = self.get_chart_instance();
					if (chart_instance) {
						let chart_campaign_id = chart_instance.get_current_campaign();
						if (chart_campaign_id) {
							let $row = self.m_grid_campaigns.$find_row(chart_campaign_id);
							if ($row && $row.length > 0) {
								self.m_grid_campaigns.mark_row($row);
							}
						}
					}
				}
				else {
					// cuando ingreso al tab de una campana especifica (la primera vez), tengo que actualizar las graficas historicas.
					let campaign_id = tab_id;
					self.dashboard_refresh_campaign_charts(campaign_id);
					update_toolbar_view();
				}
			});
		}, 500);
	};

	/*
		Funcion encargada de refrescar las graficas historicas de una campana (la primera vez que ingresa al tab).
	*/
	sup_full_screen.prototype.dashboard_refresh_campaign_charts = function(campaign_id) {
		this.m_started_dashboard_campaigns_charts = this.m_started_dashboard_campaigns_charts || {};
		if (this.m_started_dashboard_campaigns_charts[campaign_id]) {
			return;
		}
		//Se activa flag que permite buscar los indicadores y actualizarlos cuando se refrescan las campannas.
		if (!this.dashboard.m_fire_charts_refresh) {
			this.dashboard.m_fire_charts_refresh = {
				campaign_id : campaign_id,
				enable : true
			};
		}
		
		this.m_started_dashboard_campaigns_charts[campaign_id] = true;
	};

	sup_full_screen.prototype.__update_panel_positions = function() {
		this.dashboard.update_panel_positions();
	};

	sup_full_screen.prototype.__$get_campaign_node = function(campaign_id) {
		return this.dashboard.get_campaign_node(campaign_id);
	};

	sup_full_screen.prototype.__$get_campaign_bots_node = function(campaign_id) {
		return this.dashboard.get_campaign_bots_node(campaign_id);
	};

	sup_full_screen.prototype.__$get_campaign_title_bar_node = function(campaign_id) {
		return this.dashboard.get_campaign_title_bar_node(campaign_id);
	};

	sup_full_screen.prototype.__on_toggle_full_screen = function(campaign_id) {
		this.dashboard.on_toggle_full_screen(campaign_id);
	};

	sup_full_screen.prototype.save_config = function (callback = $.noop)
	{
		if (this.m_timeout_save_tab) {
			window.clearTimeout(this.m_timeout_save_tab);
			this.m_timeout_save_tab = null;
		}
		this.dashboard.save_config(callback);
	};

	sup_full_screen.prototype.update_toolbar_buttons = function ()
	{
		if (!this.is_full_screen()) {
			return;
		}
		const campaign_id = this.get_campaign_id();
		const current_view = this.dashboard.get_current_view_id(campaign_id);
		const $sup_full_screen_header_container = $('#sup_full_screen_header_container');
		this.toggle_btns_toolbar_bots(campaign_id);
		
		$.each($sup_full_screen_header_container.find('[action_id]'), function(_, action)
		{
			let $action = $(action);
			let show_in_tab = $action.attr('show_in_tab');
			let show_in_dashboard_panel = $action.attr('show_in_dashboard_panel');

			if (show_in_tab !== 'BOTH') {
				if (show_in_tab === 'CAMPAIGNS') {
					if (campaign_id === 'dashboard') {
						$action.hide();
						return true;// continue
					}
				}
			}
			
			if (show_in_dashboard_panel === 'BOTH') {
				$action.show();
				return true;// continue
			}
			if (current_view === 'dashboard') {
				if (show_in_dashboard_panel === 'ONLY_CAMPAIGNS') {
					$action.show();
					return true; // continue
				}
			}
			else if (show_in_dashboard_panel === 'ONLY_CONVERSATIONS_PANELS') {
					$action.show();
					return true; // continue
			}
			$action.hide();
		});

		//$("#sup_full_screen_header_navigation_icon")[current_view === "dashboard"? "show": "hide"]();

		const is_view_bot = this.dashboard.__is_campaign_view_bots(campaign_id);
		if (is_view_bot) {
			$('#sup_full_screen_header_container [action_id="campaign_detail"]').hide();
			$('#sup_full_screen_header_container [action_id="campaign_menu_config_view"]').hide();
			$("#sup_full_screen_header_navigation_campaign_accounts_btn").hide();
			$("#sup_dash_full_screen_panel_bots_filter").show();
		}
		
	};

	sup_full_screen.prototype.__update_toolbar_view = function(params){
		const self = this;
		const $sup_dashboard_container = self.__$get_campaign_bots_node(params.campaign_id);
		const $sup_container = $('#sup_container');
		if($sup_dashboard_container.css('display') && $sup_dashboard_container.css('display')!=="none"){
			//Se esta mostrando el dashboard de bots
			$sup_container.removeClass("supBotsDashContainer").addClass("supBotsDashContainer");
			$sup_dashboard_container.removeClass("supBotsDashFullScreen").addClass("supBotsDashFullScreen");
			$sup_dashboard_container.removeClass("supBotsDashSimpleScreen");
			//Debemos informar al dashboard de que está en pantalla completa
			const appForm = $sup_dashboard_container.find('icc-processflow-dashboard')[0];
			appForm.setFullScreenMode(true);
		}else{
			$sup_container.removeClass("supBotsDashContainer");
		}
	};
	sup_full_screen.prototype.toggle_btns_toolbar_bots = function(campaign_id)
	{
		const is_view_bot = this.dashboard.__is_campaign_view_bots(campaign_id);
		$('#sup_full_screen_header_container [action_id="campaign_detail"]')[is_view_bot?"hide":"show"]();
		$('#sup_full_screen_header_container [action_id="campaign_menu_config_view"]')[is_view_bot?"hide":"show"]();
		$("#sup_full_screen_header_navigation_campaign_accounts_btn")[is_view_bot?"hide":"show"]();
		$("#sup_dash_full_screen_panel_bots_filter")[is_view_bot?"show":"hide"]();
	};

	sup_full_screen.prototype.toggle_visibility = function(params)
	{
		const self = this;
		const $body = $('body');
		const $sup_container = $('#sup_container');
		const $sup_container_left_menu = $('#sup_container_left_menu');
		const $sup_full_screen_header_container = $('#sup_full_screen_header_container');
		var on_toggle = function()
		{
			if (params.full_screen) {
				self.update_toolbar_buttons();
			}
			else {
				self.toggle_dashboard_title_container(false);
				self.__set_campaign_id(null);
				self.m_tabs_campaigns.hide_tabs_container();
			}
			$.each(self.dashboard.get_rendered_campaigns(), function(campaign_id)
			{
				let $campaign_node = self.__$get_campaign_node(campaign_id);
				if ($campaign_node.length > 0) {
					let $title = $campaign_node.find('[campaign_title]');				
					const $sup_dashboard_container_iteration = self.__$get_campaign_bots_node(campaign_id);
					if (params.full_screen) {
						$title.hide();						
						if(params.fromBotDashboard){
							$sup_dashboard_container_iteration.removeClass("supBotsDashSimpleScreen");
							$sup_dashboard_container_iteration.removeClass("supBotsDashFullScreen").addClass("supBotsDashFullScreen");
						}
					}
					else {
						$title.show();
						if(params.fromBotDashboard){
							$sup_dashboard_container_iteration.removeClass("supBotsDashFullScreen");
							$sup_dashboard_container_iteration.removeClass("supBotsDashSimpleScreen").addClass("supBotsDashSimpleScreen");
						}
						$campaign_node.show();
					}
					self.__on_toggle_full_screen(campaign_id);
				}
			});

			self.__update_panel_positions();
			self.save_config();
		};

		if (params.full_screen) {
			$body.attr('supervisor-app-visibility-type', FULL_SCREEN_VIEW)
				.removeClass(FULL_SCREEN_CLASS)
				.addClass(FULL_SCREEN_CLASS);

			$sup_full_screen_header_container.show();

			if(params.fromBotDashboard){
				$sup_container.removeClass("supBotsDashContainer").addClass("supBotsDashContainer");
			}
			else{
				$sup_container.removeClass("supBotsDashContainer");
			}
			$sup_container_left_menu.hide();
			this.goto_tab_campaign(params.campaign_id, on_toggle);
		}
		else {
			$body.attr('supervisor-app-visibility-type', SIMPLE_SCREEN_VIEW)
				.removeClass(FULL_SCREEN_CLASS);
			$sup_full_screen_header_container.hide();
			$sup_container_left_menu.show();
			on_toggle();
		}
	};

	// boton de back desde el panel de interacciones al dashboard (summary+indicators+members)
	sup_full_screen.prototype.$full_screen_back_button = function (campaign_id)
	{
		let campaign_dom_id = this.get_normalized_id(campaign_id);
		let $full_screen_back = $('#sup_dash_full_screen_back_dashboard_' + campaign_dom_id);
		return $full_screen_back;
	};

	sup_full_screen.prototype.is_full_screen_mode = function()
	{
		let visibility_type = ($('body').attr('supervisor-app-visibility-type') || '').toUpperCase();
		return visibility_type === FULL_SCREEN_VIEW;
	};

	sup_full_screen.prototype.get_selected_tab_id = function()
	{
		if (!this.m_tabs_campaigns) {
			return;
		}
		return this.m_tabs_campaigns.get_selected_tab_id();
	};

	sup_full_screen.prototype.toggle_campaign_menu = function ()
	{
		//Se llama al evento del menu en vista comprimida.
		let campaign_id = this.get_campaign_id();
		if (!campaign_id) {
			return;
		}
		$("#sup_full_screen_header_container").find('[action_id="campaign_menu"]').removeClass("btnActive").addClass("btnActive");
		let campaign_dom_id = this.get_normalized_id(campaign_id);
		let $campaign_menu = $('#sup_dash_panel_menu_' + campaign_dom_id);
		$campaign_menu.trigger('click');
	};

	sup_full_screen.prototype.toggle_campaign_menu_config_view = function ()
	{
		//Se llama al evento del menu en vista comprimida.
		let campaign_id = this.get_campaign_id();
		if (!campaign_id) {
			return;
		}
		$("#sup_full_screen_header_container").find('[action_id="campaign_menu_config_view"]').removeClass("btnActive").addClass("btnActive");
		let campaign_dom_id = this.get_normalized_id(campaign_id);
		let $campaign_menu = $('#sup_dash_panel_menu_config_view_' + campaign_dom_id);
		$campaign_menu.trigger('click');
	};

	sup_full_screen.prototype.get_normalized_id = function (campaign_id) {
		return this.dashboard.get_normalized_id(campaign_id);
	};

	sup_full_screen.prototype.show_control_panel = function()
	{	
		//Muestro el panel de controles (campigns), muestro icono de back del panel y title.
		this.m_tabs_campaigns.switch_tab("dashboard", undefined, true);
	};

	sup_full_screen.prototype.toggle_dashboard_title_container = function(isShow)
	{
		const $dashboard_title_container = $("#sup_dashboard_title_container");
		const $sup_title_container = $("#sup_title_container");
		//Se muestra el title de panel de control y btn de atras
		if (isShow) {
			const $sup_container_left_menu = $('#sup_container_left_menu');
			$sup_container_left_menu.show();
			const $sup_full_screen_header_container = $('#sup_full_screen_header_container');
			$sup_full_screen_header_container.hide();
			$dashboard_title_container.show();
			$sup_title_container.hide();

			//Deshabilito/Habilito las opciones del menu de navegacion global segun corresponda. 
			const $menu_option_control_panel = $('[action_id="control_panel"]');
			$menu_option_control_panel.removeClass("selected").addClass("selected").prop("disabled", true);
			const $menu_option_hide_full_screen = $('[action_id="campaign_hide_full_screen"]');
			$menu_option_hide_full_screen.removeClass("selected").prop("disabled", false);
		}
		else{
			$dashboard_title_container.hide();
			$sup_title_container.show();
			//Deshabilito/Habilito las opciones del menu de navegacion global segun corresponda.
			const $menu_option_control_panel = $('[action_id="control_panel"]');
			$menu_option_control_panel.removeClass("selected").prop("disabled", false);
			const $menu_option_hide_full_screen = $('[action_id="campaign_hide_full_screen"]');
			$menu_option_hide_full_screen.removeClass("selected").addClass("selected").prop("disabled", true);
		}
	};

	sup_full_screen.prototype.toggle_campaigns_nav = function(isShow)
	{
		const $menu = $("#sup_full_screen_header_navigation_container");
		$menu[isShow?"show":"hide"]();
	};

	sup_full_screen.prototype.fill_campaigns_navigation = function(select_tab_id)
	{
		if (!this.m_tabs_campaigns) {
			return;
		}

		//Se agregan o actualizan la navegacion de campannas
		const $nav_campaigns_menu = $('#sup_full_screen_header_navigation_menu');
		$nav_campaigns_menu.html("");
		let number_campaigns = 0;
		select_tab_id = select_tab_id || this.get_campaign_id();
		$.each(this.m_tabs_campaigns.tabs_list || {}, function(_, tab_config)
		{
			if (tab_config.id === "dashboard" || tab_config.id === (select_tab_id )) {
				return;
			}
			const campaign = inConcert.app.getCampaign(tab_config.id);
			if (!campaign) {
				return;
			}
			number_campaigns += 1;
			const $item = $(("").concat(
				'<a menu_item_action',  
				' campaign_id="', tab_config.id ,'" class="dropdown-item batchActionButton buttonColorGray" type="button">',
				campaign.Name || campaign.id,
				'</a>'
				));

			$nav_campaigns_menu.append($item);
		});

		$("#sup_full_screen_header_navigation_btn").prop("disabled", number_campaigns === 0);

	};

	sup_full_screen.prototype.__get_campaign_title = function(campaign_object, config)
	{
		const is_allegro_campaign = campaign_object.isAllegro();
		let campaign_title = '';

		if (config) {
			if (!is_allegro_campaign) {
				if ( config.account_id ) {
					var account = campaign_object.getAccount(config.account_id);
					var account_name = campaign_object.getAccountDescription(account);
					var channel_class = "accountSelect" + account.proxyType.toUpperCase();

					if ( config.al_id ) {
						var al = campaign_object.getAttentionLevel(config.al_id);
						var al_name = al ? al.name : config.al_id;

						campaign_title = FORMAT('<span class="accountName {0}">{1}</span> <span class="attentionLevelName">{2}</span>', channel_class, account_name, al_name);
					}
					else {
						campaign_title =  FORMAT('<span class="accountNameSelect {0}">{1}</span>', channel_class, account_name);
					}
				}
			}
		}
		return campaign_title;
	};

	sup_full_screen.prototype.__set_campaign_workgrpups = function (full_campaign_id, campaign_object, config) {

		const is_allegro_campaign = campaign_object.isAllegro();
		config = config ||  this.dashboard.get_campaign_selected_config(full_campaign_id);
		let campaign_workgroups = this.dashboard.get_workgroup_manager().get_campaign_workgroups(full_campaign_id) || [];
		const $container = $(`#sup_full_screen_header_navigation_campaign_workgroups`);
		$container.hide();
		if (campaign_workgroups.length > 0) {
			// Si la campana tiene Workgropus, y es de tipo "call allegro" o NO tiene cuentas en el selector de campanas (es decir ha seleccionado solamente campana),
			//	ahi SI debe de mostrar el icono wkg y el tooltip con los workgroups
			if (is_allegro_campaign || !config.account_id) {
				var workgroups_names = '';
				campaign_workgroups.forEach(function(wg_id, index){
					var delimiter = index + 1 === campaign_workgroups.length ? '' : ', ';
					workgroups_names += wg_id + delimiter;
				});
				$container.show();
				$container.attr("title", "<%=_T('Workgroups')%>" + ": " + workgroups_names);
			}
		}
	};

	sup_full_screen.prototype.update_campaign_title = function () {
		
		const current_campaign = this.get_campaign_id();
		if (!this.m_tabs_campaigns || !current_campaign || current_campaign === "dashboard") {
			return;
		}

		const campaign_id = current_campaign;
		const campaign = inConcert.app.getCampaign(campaign_id);
		if (!campaign ) {
			return;
		}
		const $container_name = $(`#sup_full_screen_header_navigation_campaign_name`);
		var campaign_config = this.dashboard.get_campaign_selected_config(campaign_id);
		const title = this.__get_campaign_title(campaign, campaign_config);
		$container_name.html(title);
		this.__set_campaign_workgrpups(campaign_id, campaign, campaign_config);
		this.dashboard.set_chip_name(campaign_id, this.dashboard.get_current_view_id(campaign_id));
	};

	sup_full_screen.prototype.set_event_nav_campaign_items = function () {
		const self = this;
		const $navigation_menu = $("#sup_full_screen_header_navigation_menu");
		$navigation_menu
			.off("click")
			.on("click", "[menu_item_action]",function() {
				const campaign_id = $(this).attr("campaign_id");
				self.m_tabs_campaigns.switch_tab(campaign_id);
			});
	};

	sup_full_screen.prototype.set_chip_name = function (name, campaign_id, current_dashboard) {
		const current_campaign = this.get_campaign_id();
		if ( current_campaign !== campaign_id) {
			return;
		}
		this.__remove_chip_viewer_classes();
		const $chip = $("#sup_full_screen_header_navigation_campaign_child");
		$chip.text(name);
		if ((!name || name === ""))  {
			$chip.hide();
		}
		else{
			$chip.show();
		}
		if (current_dashboard && current_dashboard.m_view_classes_name) {
			this.__add_chip_viewer_classes(current_dashboard.m_view_classes_name);
		}
	};

	sup_full_screen.prototype.__remove_chip_viewer_classes = function() {
		const $container = $("#sup_full_screen_header_container");
		$container.removeClass('archiveViewerTitle queuedViewerTitle waitingViewerTitle abandonedViewerTitle monitoringViewerTitle appointmentViewerTitle global_searchViewerTitle BotsViewer');
	};

	sup_full_screen.prototype.__add_chip_viewer_classes = function( classes_name) {
		const $container = $("#sup_full_screen_header_container");
		$container.addClass(classes_name+"Title");
	};

	sup_full_screen.prototype.show_campaign_bots_filters = function() {
		const current_campaign = this.get_campaign_id();
		if (!this.m_tabs_campaigns || !current_campaign || current_campaign === "dashboard") {
				return;
		}

		this.dashboard.show_campaign_bots_filters(current_campaign, $("#sup_dash_full_screen_panel_bots_filter"));
	};

	sup_full_screen.prototype.show_campaign_accounts = function() {
		const current_campaign = this.get_campaign_id();
		if (!this.m_tabs_campaigns || !current_campaign || current_campaign === "dashboard") {
				return;
		}
		const $btn_parent = $(`#sup_full_screen_header_navigation_campaign_accounts_btn`);
		this.dashboard.show_campaign_accounts_selector(current_campaign, $btn_parent);
	};

}());
