/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
  strict:true, undef:true, unused:true, curly:true, browser:true, indent:false, maxerr:50, esversion: 9 */

/* global inConcert, _ */

// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.

if (!this.inConcert) {
	this.inConcert = {};
}

/**
 * @namespace
 * @name SupervisorWorkgroups
 */
(function () {

	'use strict';

	/**
	 * Creates a new workgroup instance.
	 *
	 * @author Sebastian Moreno (smoreno@inconcertcc.com)
	 * @constructor
	 * @memberOf SupervisorWorkgroups
	 */
	function SupervisorWorkgroups(dashboard_instance) {
		this.m_dashboard = dashboard_instance;
		this.m_workgroups = {};
		this.m_workgroups_ids = {};
	}

	if (typeof inConcert.sup_workgroups !== 'function') {
		inConcert.sup_workgroups = SupervisorWorkgroups;
	}

	SupervisorWorkgroups.prototype.update_workgroups = function(campaign_workgroups)
	{
		var self = this;
		var workgroup_list = {};
		for (var campaign_id in campaign_workgroups){
			
			if (campaign_workgroups.hasOwnProperty(campaign_id)) {
				var workgroup = campaign_workgroups[campaign_id];
				if (_.size(workgroup) !== 0){
					workgroup_list[campaign_id] = [];
					for (var workgroup_id in campaign_workgroups[campaign_id]){
						if (campaign_workgroups[campaign_id].hasOwnProperty(workgroup_id)){
							workgroup_list[campaign_id].push(workgroup_id);
						}
					}
				}
			}
		}
		self.m_workgroups = campaign_workgroups;
		self.m_workgroups_ids = workgroup_list;
	};
	
	SupervisorWorkgroups.prototype.get = function(callback)
	{
		var self = this;
		this.m_dashboard.m_model.get_workgroups(function(campaign_workgroups, error){
			if (!error){
				self.update_workgroups(campaign_workgroups);
				if (typeof(callback) === 'function'){
					callback(self.m_workgroups_ids);
				}
			}
			else{
				if (typeof(callback) === 'function'){
					callback(null, error);
				}
			}
		});
	};

	SupervisorWorkgroups.prototype.strip_vcc_from_campaign = function(campaign_id)
	{
		return campaign_id.replace(/^(.*?)@(.*?)$/, '$1');
	};

	SupervisorWorkgroups.prototype.get_campaign_workgroups = function(campaign_id)
	{
		var campaign_without_vcc = this.strip_vcc_from_campaign(campaign_id);
		return !_.isEmpty(this.m_workgroups_ids) && this.m_workgroups_ids[campaign_without_vcc] ? this.m_workgroups_ids[campaign_without_vcc] : [];
	};

	SupervisorWorkgroups.prototype.get_workgroup_agents = function(campaign_id, workgroup_id)
	{
		var campaign_without_vcc = this.strip_vcc_from_campaign(campaign_id);
		return !_.isEmpty(this.m_workgroups_ids) && this.m_workgroups[campaign_without_vcc][workgroup_id] ? this.m_workgroups[campaign_without_vcc][workgroup_id].Agents : [];
	};

	SupervisorWorkgroups.prototype.get_agents_workgroups_of_campaign = function(campaign_id)
	{
		let agents_workgroups_of_campaign = {};
		let workgroups_of_campaign = this.get_campaign_workgroups(campaign_id);
		for (let i = 0; i < workgroups_of_campaign.length; i++) {
			let workgroup_id = workgroups_of_campaign[i];
			let agents = this.get_workgroup_agents(campaign_id, workgroup_id);
			for (let agent_id in agents) {
				if (agents.hasOwnProperty(agent_id)) {
					if (agents[agent_id] === true) {
						agents_workgroups_of_campaign[agent_id.toLowerCase()] = true;
					}
				}
			}
		}
		return agents_workgroups_of_campaign;
	};
}());
