/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double */

/* global $, inConcert */

// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	"use strict";
    let REQUEST_TIMEOUT_MS = 300000;

	let format = inConcert.Utils.format;

	function adm_form_tweety(params)
	{
		let success = adm_form_tweety.uber.constructor.call(this, params);
		if (!success) {
			return;
		}
		this.tweety_uri = inConcert.app.getTweetyDesignerUrl() + "/";
		return true;
	}

	inConcert.Utils.inherit(adm_form_tweety, inConcert.conf_form);

	if (typeof inConcert.adm_form_tweety !== "function") {
		inConcert.adm_form_tweety = adm_form_tweety;
	}

	adm_form_tweety.prototype.has_permission_to_save = function()
	{
		return this.dashboard.permissions.to_action("web_designer", "web_designer_save", "SAVE");
	};

	adm_form_tweety.prototype.render = function( $row, extra_parameters )
	{
		if ( !this.has_permission_to_save() ) {
			return;
		}
		if ( !$row ) {
			// new project
			if (extra_parameters && extra_parameters.action_id) {
				this.popup_render( { type: extra_parameters.action_id } );
			} else {
				this.popup_render( { type: "new" } );
			}
		}
		else {
			// edit project
			let data = {
				is_new    : false,
				project_id: $row.attr("row_id")
			};
			this.show_tweety( data );
		}
	};

	adm_form_tweety.prototype.show_tweety = function( param )
	{
		let self = this;
		this.hide_messages();
		this.popup_hide();

		let parameters = {};
		if ( param.is_new ) {
			parameters.cmd = "new";
			parameters.type = param.type; 
		}
		else {
			parameters.cmd = "open";
			parameters.value = param.project_id;
		}
		let data = {
			conf: {
				uri           : this.tweety_uri,
				parameters    : parameters,
				instance_id   : inConcert.Utils.generateGuid(),
				project_id    : param.project_id || null,
				instance      : this,
				on_new_project: function( success, data ) {
					if ( success ) {
						data.name = data.description; // TODO: que fabian cambie el attr description por name
						self.dashboard.view.update_view( data );
					}
				},
				on_close_window: function( instance_id ) {
					self.dashboard.remove_popup_window( instance_id );
				}
			}
		};
		let tweety_window = new inConcert.TweetyWindow( data );
		tweety_window.load();
		this.dashboard.add_popup_window( data.conf.instance_id, tweety_window );
	};

	adm_form_tweety.prototype.popup_show = function( params )
	{
		adm_form_tweety.uber.popup_show.call( this, params );
		if ( params ) {
			if ( params.type === "clone") {
				$("#adm_form_tweety_clone_id").focus();
			}
		}
	};

	adm_form_tweety.prototype.popup_html = function( params )
	{
		let title;
		if ( params.type === "clone" ) {
			title = format( "<%=_T('Cloning project {0}')%>", ( params.source_id || "" ) );
			return "".concat(
				"<div class='internalPopUpContainer tweetyPopUp formPopUp popUp520w'>",
					"<div class='internalPopUpOverlay'></div>",
					"<div class='internalPopUp'>",

						// title
						"<div class='internalPopUpTitle'>",
							"<span>", title, "</span>",
							"<button id='adm_form_tweety_clone_cancel_times' class='internalPopUpCloseButton'><i class='far fa-times'></i></button>",
						"</div>",

						"<div class='internalPopUpMiddle pageFormContainer'>",
							
							// mensaje de error
							"<div id='adm_form_tweety_clone_message' class='genericFormMessagesContainer messageError' style='display:none;'></div>",

							// key input
							"<div class='card no-card'>",
								"<div class='card-block'>",
									"<div class='row'>",
										"<div class='col-12'>",
											"<div class='form-group'>",
												"<label for='adm_form_tweety_clone_id' class='required'>", "<%=_T('New project Id')%>", "<%=_T(':')%>", "</label>",
												"<input id='adm_form_tweety_clone_id' class='form-control' type='text' maxlength='50' placeholder='", "<%=_T('Enter identifier')%>...", "' />",
												"<i id='adm_form_tweety_clone_id_validation' class='formFieldValidation formFieldError' hidden ></i>",
											"</div>",
										"</div>",
									"</div>",
									"<div class='row'>",
										"<div class='col-12'>",
											"<div class='form-group'>",
												"<div class='form-check'>",
													"<label for='adm_form_tweety_clone_overwrite' class='form-check-label form-check-label-title'>",
														"<input id='adm_form_tweety_clone_overwrite' class='form-check-input' type='checkbox' />",
														"<%=_T('Overwrite')%>",
													"</label>",
												"</div>",
											"</div>",
										"</div>",
									"</div>",
								"</div>",
							"</div>",
						"</div>",

						// MODAL FOOTER BUTTONS
						"<div class='internalPopUpButtons'>",
							"<button id='adm_form_tweety_clone_execute_and_edit' class='pageButtonGeneric buttonColorBlue float-right'>" + "<%=_T('Clone & Edit')%>" + "</button>",
							"<button id='adm_form_tweety_clone_execute' class='pageButtonGeneric buttonColorBlue float-right'>" + "<%=_T('Clone')%>" + "</button>",
							"<button id='adm_form_tweety_clone_cancel' class='pageButtonGeneric buttonColorWhite float-right'>" + "<%=_T('Cancel')%>" + "</button>",
						"</div>",
					"</div>",
				"</div>");
		}
		else if ( params.type === "new" ) {
			title = "<%=_T('Create project')%>";
			return "".concat(
				"<div class='internalPopUpContainer tweetyPopUp tweetyNew formPopUp popUp520w'>",
					"<div class='internalPopUpOverlay'></div>",
					"<div class='internalPopUp'>",

						// title
						"<div class='internalPopUpTitle'>",
						"<span>", title, "</span>",
							"<button id='adm_form_tweety_new_cancel_times' class='internalPopUpCloseButton closeButtonSmall'><i class='far fa-times'></i></button>",
						"</div>",

						"<div class='internalPopUpMiddle pageFormContainer'>",

							// mensaje de error
							"<div id='adm_form_tweety_new_message' class='genericFormMessagesContainer messageError' style='display:none;'></div>",

							// key input
							"<div id='adm_form_tweety_new_type_container' class='card no-card'>",
								"<div class='card-block'>",

                                    "<div id='adm_form_tweety_new_type_title' class='row adm_form_tweety_new_type_title'>",
                                        "<div class='col-12'>",
                                            "<label><%=_T('Hello!')%> 😊</label>",
                                        "</div>",
                                    "</div>",
                                    "<div id='adm_form_tweety_new_type_subtitle' class='row adm_form_tweety_new_type_subtitle'>",
										"<div class='col-12'>",
                                            "<label><%=_T('What project do you want to create?')%></label>",
										"</div>",
									"</div>",

                                    "<div class='adm_form_tweety_new_type_options'>",
                                        "<div id='adm_form_tweety_new_type_options_container' class='options_container'>",
                                        "</div>",
                                        "<div id='adm_form_tweety_new_type_options_container_fei' class='options_container' style='display:none;'>",
                                        "</div>",
									"</div>",

								"</div>",
							"</div>",

                            "<div id='adm_form_tweety_new_type_chat_server_container' class='adm_form_tweety_new_type_chat_server_container' style='display:none;'>",
								
                                "<div class='internalPopUpButtons'>",
                                    "<button id='adm_form_tweety_new_create_manually' class='pageButtonGenericHeight2 pageButtonGeneric buttonColorWhite'><%=_T('Create manually')%></button>",
                                "</div>",

                                "<div id='adm_form_tweety_new_type_chat_server' class='chatConversation'>",
                                    "<div id='chatReadControl' class='chatReadControl'>",
                                    "</div>",
                                    "<div id='adm_form_tweety_section_loader' class='flowDesignerIAFormBuilderLoaderContainer' style='display:none;'>",
										"<div class='loader'>",
											"<div class='dot'></div>",
											"<div class='dot'></div>",
											"<div class='dot'></div>",
										"</div>",
									"</div>",
								"</div>",
							"</div>",
						"</div>",

						// MODAL FOOTER BUTTONS
						"<div id='adm_form_tweety_new_type_chat_client_container' class='adm_form_tweety_new_type_chat_client_container' style='display: none;'>",
                            "<textarea id='adm_form_tweety_new_type_chat_client_control' class='adm_form_tweety_new_type_chat_client_control' rows='1' placeholder='<%=_T([[Write a message to the bot...]])%>' data-emojiable='converted' data-id='318f3259-5075-4348-b8d3-f8c66983cc4a' data-type='original-input' style='height: 28px;'></textarea>",
                            "<button id='adm_form_tweety_new_open' class='pageButtonGeneric iconOnlyBtnSm buttonColorTertiary iconSendMessage'></button>",
						"</div>",
					"</div>",
				"</div>");
		}
        
		return "";
	};

	adm_form_tweety.prototype.validate_clone_project = function( new_id, params )
	{
		this.popup_hide_errors();

		let show_error = function( $control, message )
		{
			$("#adm_form_tweety_clone_message").text( message ).show();
			if ( $control ) {
				let validation_id = $control.attr("id") + "_validation";
				$("#" + validation_id).show();
				$control.focus();
			}
		};

		let message;
		let $clone = $("#adm_form_tweety_clone_id");
		// requerido
		if ( new_id.trim().length === 0 ) {
			message = "<%=_T('Please enter the new project id.')%>";
			show_error( $clone, message );
			return false;
		}
		// valida que el nuevo nombre no sea igual al nombre origen
		if ( new_id.toLowerCase() === params.source_id.toLowerCase() ) {
			message = "<%=_T('The new project id must be different to source project.')%>";
			show_error( $clone, message );
			return false;
		}
		// valida los caracteres permitidos (reverd words de tweety).
		let reserverd_words = { "default": true, "designer": true };
		if ( reserverd_words[ new_id.trim().toLowerCase() ] ) {
			message = format( "<%=_T('{0} is not allowed to be projects name.')%>", new_id );
			show_error( $clone, message );
			return false;
		}
		// valida los caracteres permitidos.
		let is_filesystem = this.validation.valid( new_id, "filesystem" );
		if ( !is_filesystem.success ) {
			message = "<%=_T('Only letters, numbers, dashes and dots are allowed.')%>";
			show_error( $clone, message );
			return false;
		}

		// validation successfully
		return true;
	};

	adm_form_tweety.prototype.clone_project = function( source_id, has_edit )
	{
		let self = this;
		let dashboard = this.dashboard;
		let view = this.dashboard.view;

		let target_id = $("#adm_form_tweety_clone_id").val();
		let params = {
			source_id: source_id,
			overwrite: $("#adm_form_tweety_clone_overwrite").prop("checked")
		};

		if ( !this.validate_clone_project( target_id, params ) ) {
			return;
		}

		dashboard.hide_status_message();
		dashboard.show_waiting( format( "<%=_T('Please wait, cloning project {0} to {1}.')%>", source_id, target_id ) );

		let tweety_api = inConcert.app.getTweetyAPI();
		tweety_api.clone(
			target_id,
			params,
			function( success, response )
			{
				let message;
				if ( success ) {
					view.refresh();
					message = format( "<%=_T('Project {0} was successfully cloned to {1}.')%>", source_id, target_id );
					let message_type = "Info";
					self.dashboard.show_status_message( message, message_type );
					if ( has_edit ) {
						let data = {
							is_new    : false,
							project_id: target_id
						};
						self.show_tweety( data );
					}
					self.popup_hide();
					return;
				}

				// error
				message = format( "<%=_T('Failed to clone project {0} to {1}.')%>", source_id, target_id );
				if ( typeof response.result === "object" ) {
					if ( response.result.code === 1016 ) {
						message = format( "<%=_T('Unable to clone project {0}. Already exist new project id {1}.')%>", source_id, target_id );
					}
				}
				$("#adm_form_tweety_clone_message").text( message ).show();
				dashboard.hide_waiting();
			}
		);
	};

	adm_form_tweety.prototype.validate_new_project = function()
	{
		this.popup_hide_errors();

		let show_error = function( $control, message )
		{
			$("#adm_form_tweety_new_message").text( message ).show();
			if ( $control ) {
				let validation_id = $control.attr("id") + "_validation";
				$("#" + validation_id).show();
				$control.focus();
			}
		};

		let message;
		let $new_project_type = $("#adm_form_tweety_new_type");
		let project_type = this.form_type;
		// requerido
		if ( !project_type || project_type.length === 0 ) {
			message = "<%=_T('Please select the project type.')%>";
			show_error( $new_project_type, message );
			return false;
		}

		return true;
	};

	adm_form_tweety.prototype.new_project = function()
	{
		if ( !this.validate_new_project() ) {
			return;
		}
		let project_type = this.form_type;
		let data = { is_new: true, type: project_type };
		this.show_tweety( data );
	};

	adm_form_tweety.prototype.popup_set_events = function( params )
	{
		let self = this;

		if ( params.type === "clone" ) {
			$("#adm_form_tweety_clone_execute")
				.off("click")
				.on("click", function() {
					if ( !self.has_permission_to_save() ) {
						let error_message = "<%=_T('Unable to clone project: You not have permission for this action.')%>";
						inConcert.app.m_mainWindow.showBannerMessage({ message: error_message, timeout: 5000 });
						return;
					}
					self.clone_project( params.source_id );
				});

			$("#adm_form_tweety_clone_execute_and_edit")
				.off("click")
				.on("click", function() {
					if ( !self.has_permission_to_save() ) {
						let error_message = "<%=_T('Unable to clone project: You not have permission for this action.')%>";
						inConcert.app.m_mainWindow.showBannerMessage({ message: error_message, timeout: 5000 });
						return;
					}
					self.clone_project( params.source_id, true /* editar despues de clonar */ );
				});

			$("#adm_form_tweety_clone_cancel")
				.off("click")
				.on("click", function() {
					self.popup_hide();
				});

			$("#adm_form_tweety_clone_cancel_times")
				.off("click")
				.on("click", function() {
					self.popup_hide();
				});
		}
		else if ( params.type === "new" ) {
			$("#adm_form_tweety_new_open")
				.off("click")
				.on("click", function() {
					if ( !self.has_permission_to_save() ) {
						let error_message = "<%=_T('Unable to create new project: You not have permission for this action.')%>";
						inConcert.app.m_mainWindow.showBannerMessage({ message: error_message, timeout: 5000 });
						return;
					}
					self.send_bot_message();
				});

			$("#adm_form_tweety_new_cancel")
				.off("click")
				.on("click", function() {
					self.popup_hide();
				});

			$("#adm_form_tweety_new_cancel_times")
				.off("click")
				.on("click", function() {
					self.popup_hide();
				});

		}
	};

    adm_form_tweety.prototype.update_option_events = function()
	{
        let self = this;

        $("#back")
            .off("click")
            .on("click", function() {
                self.form_type = "";
                self.form_subtype = "";
                $("#adm_form_tweety_new_type_options_container").show();
                $("#adm_form_tweety_new_type_options_container_fei").hide();
            });

        $("#bot")
            .off("click")
            .on("click", function() {
                self.form_type = "cci";
                $("#adm_form_tweety_new_type_options_container").hide();
				$("#adm_form_tweety_new_type_options_container_fei").hide();
				$("#adm_form_tweety_new_type_container").hide();          
				$("#adm_form_tweety_new_type_chat_client_container").show();
				$("#adm_form_tweety_new_type_chat_server_container").show();
				self.appendMessage("bot", "<%=_T('Hello!, I am the Inconcert Assistant, please describe in as much detail as possible the form you need me to create for you')%>");				
            });

		$("#manually")
            .off("click")
            .on("click", function() {
                if ( !self.has_permission_to_save() ) {
                    let error_message = "<%=_T('Unable to create new project: You not have permission for this action.')%>";
                    inConcert.app.m_mainWindow.showBannerMessage({ message: error_message, timeout: 5000 });
                    return;
                }
                self.form_type = "cci";
                self.new_project();				
            });

        $("#adm_form_tweety_new_create_manually")
            .off("click")
            .on("click", function() {
                if ( !self.has_permission_to_save() ) {
                    let error_message = "<%=_T('Unable to create new project: You not have permission for this action.')%>";
                    inConcert.app.m_mainWindow.showBannerMessage({ message: error_message, timeout: 5000 });
                    return;
                }
                self.form_type = "cci";
                self.new_project();
            });

        $("#cci")
            .off("click")
            .on("click", function() {
                self.form_type = "cci";

				self.validateCopilotEnabled(function(status, response){ 
					if (status && response.enabled) {
						$("#adm_form_tweety_new_type_options_container").hide();
		                $("#adm_form_tweety_new_type_options_container_fei").show();
					} else {
						self.new_project();
					}
				});					
            });

        $("#survey_form")
            .off("click")
            .on("click", function() {
                if ( !self.has_permission_to_save() ) {
                    let error_message = "<%=_T('Unable to create new project: You not have permission for this action.')%>";
                    inConcert.app.m_mainWindow.showBannerMessage({ message: error_message, timeout: 5000 });
                    return;
                }
                self.form_type = "survey_form";
                self.new_project();
            });

        $("#generic")
            .off("click")
            .on("click", function() {
                if ( !self.has_permission_to_save() ) {
                    let error_message = "<%=_T('Unable to create new project: You not have permission for this action.')%>";
                    inConcert.app.m_mainWindow.showBannerMessage({ message: error_message, timeout: 5000 });
                    return;
                }
                self.form_type = "generic";
                self.new_project();
            });

        $("#contactform")
            .off("click")
            .on("click", function() {
                if ( !self.has_permission_to_save() ) {
                    let error_message = "<%=_T('Unable to create new project: You not have permission for this action.')%>";
                    inConcert.app.m_mainWindow.showBannerMessage({ message: error_message, timeout: 5000 });
                    return;
                }
                self.form_type = "contactform";
                self.new_project();
            });

        $("#chatcampaign")
            .off("click")
            .on("click", function() {
                if ( !self.has_permission_to_save() ) {
                    let error_message = "<%=_T('Unable to create new project: You not have permission for this action.')%>";
                    inConcert.app.m_mainWindow.showBannerMessage({ message: error_message, timeout: 5000 });
                    return;
                }
                self.form_type = "chatcampaign";
                self.new_project();
            });

        $("#webcallcampaign")
            .off("click")
            .on("click", function() {
                if ( !self.has_permission_to_save() ) {
                    let error_message = "<%=_T('Unable to create new project: You not have permission for this action.')%>";
                    inConcert.app.m_mainWindow.showBannerMessage({ message: error_message, timeout: 5000 });
                    return;
                }
                self.form_type = "webcallcampaign";
                self.new_project();
            });
	};

	adm_form_tweety.prototype.popup_load_data = function( params, callback )
	{
		if ( params.type === "new" ) {

			let list = {
				"generic"        : "<%=_T('Generic')%>",
				"survey_form"    : "<%=_T('Survey Form')%>",
				"cci"            : "<%=_T('Contact Center Integration')%>",
				"contactform"    : "<%=_T('Contact Form')%>" ,
				"chatcampaign"   : "<%=_T('Web Chat')%>",
				"webcallcampaign": "<%=_T('Web Call')%>"				
			};

            let html_list = "";
            for (let key in list) {
                if (list.hasOwnProperty(key)) {
                    let buttonText = list[key];
                    let buttonId = key;
                    let buttonHtml = "<button id='" + buttonId + "' class='tweety_option pageButtonGeneric buttonColorSecondary'>" + buttonText + "</button>";                    
                    html_list += buttonHtml;
                }
            }
            $("#adm_form_tweety_new_type_options_container").html(html_list);

			list = {
				"bot"      : "<%=_T('With Bot')%>" ,
				"manually" : "<%=_T('Manually')%>",
				"back"     : "<%=_T('Go back')%>"
			};

            let html_list_fei = "";
            for (let key in list) {
                if (list.hasOwnProperty(key)) {
                    let buttonText = list[key];
                    let buttonId = key;
                    let buttonHtml = "<button id='" + buttonId + "' class='pageButtonGeneric tweety_option" + (buttonId === "back" ? "_back buttonColorTertiary" : " buttonColorSecondary") + "'>" + (buttonId === "back" ? "<i class='far fa-arrow-left'></i>" : "") + buttonText + "</button>";                    
                    html_list_fei += buttonHtml;
                }
            }
            $("#adm_form_tweety_new_type_options_container_fei").html(html_list_fei);
			$("#adm_form_tweety_new_type_options_container_fei > #back").before("<span class='spaceBreak'></span>");

            this.update_option_events();
			callback();
		}
		else {
			callback();
		}
	};

    adm_form_tweety.prototype.__Post = function(uri, callback, jsonData, formData)
	{
		let request_handler = inConcert.invoke({
			url         : uri,
			type        : "POST",
			data        : (jsonData !== undefined ? JSON.stringify(jsonData) : formData),
			contentType : (jsonData !== undefined ? "application/json" : false),
    		processData : (jsonData !== undefined ? true : false),    
			timeout     : REQUEST_TIMEOUT_MS,
			success     : function (response)    { callback(true, response); },
			error       : function (_, error) { callback(false, error);   }
		});

		return request_handler;
	};

	adm_form_tweety.prototype.__Get = function(uri, callback)
	{
		let request_handler = inConcert.invoke({
			url    : uri,
			type   : "GET",
			timeout: REQUEST_TIMEOUT_MS,
			success: function (list, result, server_response)
			{
				if (typeof(server_response) === "object") {
					server_response = JSON.stringify(server_response);
				}
				
				if (result.toLowerCase() === "success") {
					callback(true, list);
				}
				else {
					callback(false, server_response);
				}
			},
			error  : function (_, error) {callback(false, error);}
		});
		return request_handler;
	};


    adm_form_tweety.prototype.send_bot_message = function()
	{

		if ( !this.validate_new_project() ) {
			return;
		}

		let self = this;
		self.conversation = self.conversation || [];

		let client_message = $("#adm_form_tweety_new_type_chat_client_control").val();
		$("#adm_form_tweety_new_type_chat_client_control").prop("disabled", true);
		$("#adm_form_tweety_new_open").prop("disabled", true);
        
        this.appendMessage("client", client_message);

		$("#adm_form_tweety_section_loader").show();

		let userLanguage = inConcert.app.getLoggedUserLanguage();

		self.buildFormDefinitionUsingIA( {
			form_definition: client_message,
			conversation: JSON.stringify(self.conversation),
            initial_message: "<%=_T('Hello!, I am the Inconcert Assistant, please describe in as much detail as possible the form you need me to create for you')%>",
            confirmation_message: "<%=_T('Give me a moment while I process the information provided')%>",
			userLanguage: userLanguage
		}, function(_, response){

            try {

                if (response.result) {
                    self.appendMessage("bot", "<%=_T('There was an error processing your request, please try again in a few minutes')%>");
                    self.activeControls();
                    return;
                }

                self.conversation = JSON.parse(response.conversation);
                self.appendMessage("bot", response.message);
                
                if (response.form_especification) {

                    let form_name = self.parseFormName(response.form_especification);

					if (!form_name) {
						let message = "<%=_T('Please tell me what name you want me to create the form with')%>";
						self.appendMessage("bot", message);
						self.conversation = self.conversation || [];
						self.conversation.push({ role: "assistant", content: message });
						self.activeControls();
					} else {
						setTimeout(function () {										
							self.buildFormUsingIA( {
								form_especification: response.form_especification,
								project_type: self.form_type,
								project_name: form_name
							}, function(status, response){

								if (!status) {
									self.appendMessage("bot", "<%=_T('There was an error processing your request, please try again in a few minutes')%>, " + JSON.stringify(response));
									self.activeControls();
									return false;
								}

								self.web_designer_get_id_handler(form_name, response);
							});
						}, 5000);
					}
						
                } else {
                    self.activeControls();
                }
            } catch(e) {
                self.appendMessage("bot", "<%=_T('There was an error processing your request, please try again in a few minutes')%>, " + e);
                self.activeControls();
            }
                
		});

	};

	adm_form_tweety.prototype.web_designer_get_id_handler = function(form_name, response_build_form)
	{
		let self = this;

		try {
			self.__Post(self.tweety_uri + "uploader/getidhandler", function(status, response) {

				if (!status) {
					self.appendMessage("bot", "<%=_T('There was an error processing your request, please try again in a few minutes')%>, " + JSON.stringify(response));
					return false;
				}

				self.web_designer_upload_handler(response, form_name, response_build_form);

			}, {});
		} catch(e) {
			console.error("Error: ", e);
		}			
	};

	adm_form_tweety.prototype.web_designer_import = function(fileid, form_name)
	{
		let self = this;

		try {
            if (self.isValidFilenameAgtMens(form_name)) {
                self.__Post(self.tweety_uri + "designer/applications/bundle/import/" + fileid + "?appName=" + form_name + "&overwrite=" + false + "&hidden=" + false, function(status, response) {

                    if (!status) {
                        self.appendMessage("bot", "<%=_T('There was an error processing your request, please try again in a few minutes')%>, " + JSON.stringify(response));
                        return false;
                    }

                    response = JSON.parse(response);
                    if(response.status){
                        let data = {
                            is_new    : false,
                            project_id: form_name
                        };
                        self.conversation = [];
                        self.show_tweety( data );
                    } else {
                        let error = (response && response.result && response.result.description) ? response.result.description : JSON.stringify(response);

                        if (error.includes("Error creating directory")) {
                            let message = format("<%=_T('There is already a form with the name {0}, please tell me a different one')%>", form_name);
                            self.appendMessage("bot", message);
                            self.conversation = self.conversation || [];
                            self.conversation.push({ role: "assistant", content: message });
                            self.activeControls();
                        } else {
                            self.appendMessage("bot", "<%=_T('There was an error processing your request, please try again in a few minutes')%>, " + error);
                        }
                    }
                }, undefined, "");
            } else {
                let message = format("<%=_T('The file name is invalid, please enter a different one')%>", form_name);
                self.appendMessage("bot", message);
                self.conversation = self.conversation || [];
                self.conversation.push({ role: "assistant", content: message });
                self.activeControls();
            }                
		} catch(e) {
			console.error("Error: ", e);
		}		
	};

	adm_form_tweety.prototype.isValidFilenameAgtMens = function(str){
        str = str.toUpperCase();
        if(str.length>0){
            if (/^[^\\\/\:\*\?\"\<\>\|\.]+(\.[^\\\/\:\*\?\"\<\>\|\.]+)*$/.test(str)) {
                if (jQuery.inArray(str,["CON","PRN","AUX","NUL","DESIGNER","DEFAULT",
                    "COM1","COM2","COM3","COM4","COM5","COM6","COM7","COM8","COM9",
                    "LPT1","LPT2","LPT3","LPT4","LPT5","LPT6","LPT7","LPT8","LPT9" 
                ])<0){
                    return true;
                }
            }
        }
    }

	adm_form_tweety.prototype.web_designer_upload_handler = function(fileid, form_name, response_build_form)
	{
		let self = this;

		try {

			let formData = new FormData();
			let binaryData = response_build_form.binaryData;
			let arrayBuffer = new ArrayBuffer(binaryData.length);
			let uint8Array = new Uint8Array(arrayBuffer);

			for (let i = 0; i < binaryData.length; i++) {
				uint8Array[i] = binaryData.charCodeAt(i);
			}

			let blob = new Blob([uint8Array], { type: "application/octet-stream" });
			formData.append("fileUpload", blob, form_name);
			
			self.__Post(self.tweety_uri + "uploader/uploadhandler?fileName=" + form_name + ".bundle&fileid=" + fileid, function(status, response) {

				if (!status) {
					self.appendMessage("bot", "<%=_T('There was an error processing your request, please try again in a few minutes')%>, " + JSON.stringify(response));
					return false;
				}

				let get_status = function(){
					self.__Post(self.tweety_uri + "uploader/getinfohandler", function(_, response) {
						let r = parseInt(response, 10);
						if(!isNaN(r) && r !== -1){
							setTimeout(get_status, 500);
						}
						if(r === -1){
							self.web_designer_import(fileid, form_name);
						}
					}, undefined, "fileid=".concat(fileid));
				};
				get_status();
			}, undefined, formData);
		} catch(e) {
			console.error("Error: ", e);
		}			
	};

    adm_form_tweety.prototype.appendMessage = function(type, message)
	{
        let messageHtml = "";
        if (type === "bot") {
            messageHtml = "<div class='chatEvent botEvent'>" +
				"<div class='bot_avatar'>" +
					"<img src='/inconcert/apps/agent/images/botAvatar.png' width='30' height='30'/>" +
				"</div>" +
                "<div class='content'>" +
					"<label class='title'><%=_T('Assistant')%></label>" +
                    "<div class='content_text'>"+ message +"</div>" +
				"</div>" +
			"</div>";
        } else {
            messageHtml = "<div class='chatEvent botEvent'>" +
				"<div class='client_avatar'>" +
					"<img src='"+ inConcert.app.getBaseUrl() + "/users/get_avatar/" + inConcert.app.getLoggedUserId() +"' width='30' height='30'/>" +
				"</div>" +
                "<div class='content'>" +
					"<label class='title'><%=_T('You')%></label>" +
                    "<div class='content_text'>"+ message +"</div>" +
				"</div>" +
			"</div>";
        }
            
        $("#chatReadControl").html($("#chatReadControl").html() + messageHtml);
        $("#chatReadControl").scrollTop($("#chatReadControl").prop("scrollHeight"));
        $("#adm_form_tweety_new_type_chat_client_control").val("");
	};

    adm_form_tweety.prototype.parseFormName = function(form_especification)
	{
        form_especification = JSON.parse(form_especification);

        let formName = (form_especification.form_name || "").toLowerCase();
        formName = formName.toLowerCase();
        formName = formName.replace(/á/g, "u");
        formName = formName.replace(/é/g, "u");
        formName = formName.replace(/í/g, "u");
        formName = formName.replace(/ó/g, "u");
        formName = formName.replace(/ú/g, "u");
        formName = formName.replace(/ /g, "_");
		formName = formName.replace(/'/g, "_");
        return formName;
	};

    adm_form_tweety.prototype.activeControls = function()
	{
        $("#adm_form_tweety_new_type_chat_client_control").prop("disabled", false);
        $("#adm_form_tweety_new_open").prop("disabled", false);
        $("#adm_form_tweety_section_loader").hide();
	};
    
	adm_form_tweety.prototype.buildFormDefinitionUsingIA = function(data, callback)
	{
		let request_handler = this.__Post(this.tweety_uri.replace("/webdesigner", "/web_designer/tweety/build_form_definition"), callback, data);
		return request_handler;
	};

	adm_form_tweety.prototype.validateCopilotEnabled = function(callback)
	{
		let request_handler = this.__Get(this.tweety_uri.replace("/webdesigner", "/web_designer/tweety/validate_copilot_enabled"), callback);
		return request_handler;
	};

	adm_form_tweety.prototype.buildFormUsingIA = function(data, callback)
	{
		let request_handler = this.__Post(this.tweety_uri.replace("/webdesigner", "/web_designer/tweety/build_form"), callback, data);
		return request_handler;
	};
    
}());
