/* ***** BEGIN LICENSE BLOCK *****
 * Cloned from SQL language mode definition (mode-sql.js), for using on inConcert Dialplan syntax highlighter.
 * True syntax validation it's done on a real Asterisk through AMI (Asterisk Manager Interface) over TCP protocol.
 *
 * ***** END LICENSE BLOCK ***** */

ace.define('ace/mode/dialplan', ['require', 'exports', 'module' , 'ace/lib/oop', 'ace/mode/text', 'ace/mode/dialplan_highlight_rules', 'ace/range'], function(require, exports, module) {


var oop = require("../lib/oop");
var TextMode = require("./text").Mode;
var DialplanlHighlightRules = require("./dialplan_highlight_rules").DialplanlHighlightRules;
var Range = require("../range").Range;

var Mode = function() {
    this.HighlightRules = DialplanlHighlightRules;
};
oop.inherits(Mode, TextMode);

(function() {

    this.lineCommentStart = "--";

    this.$id = "ace/mode/sql";
}).call(Mode.prototype);

exports.Mode = Mode;

});

ace.define('ace/mode/dialplan_highlight_rules', ['require', 'exports', 'module' , 'ace/lib/oop', 'ace/mode/text_highlight_rules'], function(require, exports, module) {


var oop = require("../lib/oop");
var TextHighlightRules = require("./text_highlight_rules").TextHighlightRules;

var DialplanlHighlightRules = function() {

    var keywords = (
        "exten|_X|_XX|_XXX|_XXXX"
    );

    var builtinConstants = (
        "true|false|null|sip|zap|iax|iax2|unicall"
    );

    var builtinFunctions = (
        "Set|Dial|SetCallerPres|PlayTones|NoOP|Goto|GotoIf|Authenticate|ODBCquery|BackGround|Meetme|Wait|ChanSpy|Hangup|" +
        "TkPlugEvents|TkParkApp|ChanSpyInconcert|AvoidMasquerading"
    );

    var keywordMapper = this.createKeywordMapper({
        "support.function": builtinFunctions,
        "keyword": keywords,
        "constant.language": builtinConstants
    }, "identifier", true);

    this.$rules = {
        "start" : [ {
            token : "comment",
            regex : ";(?!--).*$"
        },  {
            token : "comment",
            start : ";--(?!-)",
            end : "--;"
        }, {
            token : "string",           // " string
            regex : '".*?"'
        }, {
            token : "string",           // ' string
            regex : "'.*?'"
        }, {
            token : "constant.numeric", // float
            regex : "[+-]?\\d+(?:(?:\\.\\d*)?(?:[eE][+-]?\\d+)?)?\\b"
        }, {
            token : keywordMapper,
            regex : "[a-zA-Z_$][a-zA-Z0-9_$]*\\b"
        }, {
            token : "keyword.operator",
            regex : "\\+|\\-|\\/|\\/\\/|%|<@>|@>|<@|&|\\^|~|<|>|<=|=>|==|!=|<>|="
        }, {
            token : "paren.lparen",
            regex : "[\\(]"
        }, {
            token : "paren.rparen",
            regex : "[\\)]"
        }, {
            token : "text",
            regex : "\\s+"
        } ]
    };
    this.normalizeRules();
};

oop.inherits(DialplanlHighlightRules, TextHighlightRules);

exports.DialplanlHighlightRules = DialplanlHighlightRules;
});

