
import {ExtensionProxy, ExtensionNotFoundError } from './ExtensionProxy';
import {ICCBAMLIBEVENT_RESPONSE_TIMEOUT} from './Constants';
import {setInterval,clearInterval,setTimeout,clearTimeout,browser} from './utils';
class EventManager {
    #eventQueue = [];
    #senderInterval;
    #systemMonitorInterval;
    #extension;
    #senderCallbacks = [];
    #installExtensionHasBenRequired = false;
    constructor(){
    }
    startSender(){
        this.#senderInterval = setInterval(()=> {
            let events = this.#eventQueue.splice(-10,10);
            if(events.length>0){
                for(let i=0; i< this.#senderCallbacks.length;i++){
                    this.#senderCallbacks[i](events);
                }
            }
        },500);
    }
    stopSender(){
        if(this.#senderInterval){
            clearInterval(this.#senderInterval);
            this.#senderInterval = null;
        }
    }
    addSenderCallback(sender_callback){
        this.#senderCallbacks.push(sender_callback);
    }
    async #getExtension() {
        if (this.#extension) {
            return this.#extension;
        } else {
            this.#extension = await ExtensionProxy.waitForExtension(ICCBAMLIBEVENT_RESPONSE_TIMEOUT);
            return this.#extension;
        }
    }
    startSystemMonitor(opts){
        console.log("Start system monitor in browser "+ browser())
        this.#installExtensionHasBenRequired = false;
        this.#systemMonitorInterval = setInterval(()=>{
           this.#getExtension()
           .then((extension) => {
                return extension.getSystemEvents(opts);
           })
           .then((systemEvents) => {
                let l = systemEvents.length;
                for(let i = 0; i< l;i++){
                    this.#eventQueue.unshift(systemEvents.shift());
                }
                if(window.performance && window.performance.memory) {
                    this.#eventQueue.unshift(["chrome.memory",{ts: Date.now(), value:{jsHeapSize: {limit: window.performance.memory.jsHeapSizeLimit,
                                                                                                    total: window.performance.memory.totalJSHeapSize,
                                                                                                    used: window.performance.memory.usedJSHeapSize}}}])
                }
           })
           .catch((error)=> {
            if (error instanceof ExtensionNotFoundError){
                if(!this.#installExtensionHasBenRequired){
                    this.#installExtensionHasBenRequired = true;
                   
                    if(browser()=="chrome" || 
                       browser()=="chromium based edge"){
                        console.log("Attempt to install browser extension");
                        ExtensionProxy.attemptToInstall();
                    }
                }
            }else {
                console.log("[EventManager] Error:" + error)
            }
           })
        },800);
    }
    stopSystemMonitor(){
        if(this.#systemMonitorInterval){
            clearInterval(this.#systemMonitorInterval);
            this.#systemMonitorInterval = null;
        }
    }

    addEvent(eventName, eventData){
        this.#eventQueue.unshift({eventName,eventData});
    }


}

export default EventManager;
