/* jshint smarttabs:true, quotmark:false, esversion: 9 */
/* global $, inConcert */


if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	"use strict";
	const FORMAT = inConcert.Utils.format;
	const REQUEST_TIMEOUT_MS = 300000; // 5 minutes
	const _T = inConcert.i18n.getString;

	function QnAMakerAPI(params)
	{
		if(typeof(params) !== "object") {
			throw("QnAMakerAPI.Constructor - expected an object with configuration");
		}
		if(typeof(params.uri ) !== "string") {
			throw("QnAMakerAPI.Constructor - base uri is required");
		}

		this.uri = params.uri + "/qna_maker";
	}

	if (typeof inConcert.QnAMakerAPI !== "function") {
		inConcert.QnAMakerAPI = QnAMakerAPI;
	}

	QnAMakerAPI.prototype.__getAgentRuntimeURI = function()
	{
		return FORMAT("{0}/qna_maker/runtime", inConcert.app.getBaseUrl());
	};

/********************************************************************************************************
	FUNCIONES GENERICAS
********************************************************************************************************/

	/*
	
	*/
	QnAMakerAPI.prototype.__Get = function(uri, callback = $.noop)
	{
		if (typeof(uri) !== "string") {
			throw("QnAMakerAPI.prototype.__Get: uri variable must be string type.");
		}
		if (typeof(callback) !== "function") {
			throw("QnAMakerAPI.prototype.__Get: callback variable must be function type.");
		}

		const request_handler = inConcert.invoke({
			url    : uri,
			type   : "GET",
			timeout: REQUEST_TIMEOUT_MS,
			success: function (list, result, server_response)
			{
				if (typeof(server_response) === "object") {
					server_response = JSON.stringify(server_response);
				}
				//console.log(uri + "\nlist: " + list + "\nresult: " + result + "\nserver_response: " + server_response);
				if (result.toLowerCase() === "success") {
					callback(true, list);
				}
				else {
					// Ejemplo: code 204 (nocontent) para el caso de getTrain #7167
					callback(false, server_response);
				}
			},
			error  : function (code, error) {callback(false, error);}
		});
		return request_handler;
	};

	/*
	
	*/
	QnAMakerAPI.prototype.__Delete = function(uri, callback, data)
	{
		if (typeof(uri) !== "string") {
			throw("QnAMakerAPI.prototype.__Delete: uri variable must be string type.");
		}
		if (typeof(callback) !== "function") {
			throw("QnAMakerAPI.prototype.__Delete: callback variable must be function type.");
		}

		const request_handler = inConcert.invoke({
			url         : uri,
			type        : "DELETE",
			data        : JSON.stringify(data || {}),
			contentType : "application/json",
			timeout     : REQUEST_TIMEOUT_MS,
			success     : function (response) { callback(true, response); },
			error       : function (code, error) {callback(false, error);}
		});
		return request_handler;
	};

	/*
	
	*/
	QnAMakerAPI.prototype.__Post = function(uri, callback, data)
	{
		if (typeof(uri) !== "string") {
			throw("QnAMakerAPI.prototype.__Post: uri variable must be string type.");
		}
		if (typeof(callback) !== "function") {
			throw("QnAMakerAPI.prototype.__Post: callback variable must be function type.");
		}
		if (typeof(data) !== "object") {
			throw("QnAMakerAPI.prototype.__Post: data variable must be object type.");
		}

		const request_handler = inConcert.invoke({
			url         : uri,
			type        : "POST",
			data        : JSON.stringify(data),
			dataType    : "json",
			contentType : "application/json",
			timeout     : REQUEST_TIMEOUT_MS,
			success     : function (response)    { callback(true, response); },
			error       : function (code, error) { callback(false, error);   }
		});
		return request_handler;
	};

	/*
	
	*/
	QnAMakerAPI.prototype.__Put = function(uri, callback, data)
	{
		if (typeof(uri) !== "string") {
			throw("QnAMakerAPI.prototype.__Put: uri variable must be string type.");
		}
		if (typeof(callback) !== "function") {
			throw("QnAMakerAPI.prototype.__Put: callback variable must be function type.");
		}
		if (typeof(data) !== "object") {
			throw("QnAMakerAPI.prototype.__Put: data variable must be object type.");
		}

		const request_handler = inConcert.invoke({
			url         : uri,
			type        : "PUT",
			data        : JSON.stringify(data),
			dataType    : "json",
			contentType : "application/json",
			timeout     : REQUEST_TIMEOUT_MS,
			success     : function (response)    { callback(true, response); },
			error       : function (code, error) { callback(false, error  ); }
		});
		return request_handler;
	};

	/*
	
	*/
	QnAMakerAPI.prototype.getCognitiveErrorMsg = function(err)
	{
		if (typeof err !== "object") {
			return "";
		}
		if (typeof err.cognitiveServiceError !== "object") {
			return "";
		}
		if (typeof err.cognitiveServiceError.response !== "object") {
			return "";
		}
		if (typeof err.cognitiveServiceError.response !== "object") {
			return "";
		}
		if (typeof err.cognitiveServiceError.response.error !== "object") {
			return "";
		}
		const msg_err = err.cognitiveServiceError.response.error.message;
		if (typeof msg_err !== "string") {
			return "";
		}
		return " " + _T(msg_err);
	};

	/*
	
	*/
	QnAMakerAPI.prototype.getCognitiveErrorObject = function(err)
	{
		if (typeof err !== "object") {
			return;
		}
		if (typeof err.cognitiveServiceError !== "object") {
			return;
		}
		if (typeof err.cognitiveServiceError.response !== "object") {
			return;
		}
		if (typeof err.cognitiveServiceError.response !== "object") {
			return;
		}
		if (typeof err.cognitiveServiceError.response.error !== "object") {
			return;
		}
		return err.cognitiveServiceError.response.error;
	};

	/*
	
	*/
	QnAMakerAPI.prototype.getAnswerRequestInfo = function()
	{
		const vcc_info = inConcert.app.get_vcc_info();
		if (!vcc_info) {
			return;
		}
		if (!vcc_info.QnAMaker) {
			return;
		}
		return vcc_info.QnAMaker.AnswerRequestInfo;
	};

/********************************************************************************************************
	KNOWLEDGEBASE
********************************************************************************************************/
	/*
	
	*/
	QnAMakerAPI.prototype.getKnowledgeBase = function(kb_id, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/knowledge_base/qna_maker/knowledgebases/get/48b866be-f747-45bb-8a80-332a5cf6a0f8
		 */
		const uri = FORMAT("{0}/knowledgebases/get/{1}",
			this.uri,
			encodeURIComponent(kb_id || "")
		);

		const request_handler = this.__Get(uri, function(success, info) {
			if (success) {
				callback(true, info);
				return;
			}
			// error
			callback(false, info);
		});
		return request_handler;
	};

	/*
	
	*/
	QnAMakerAPI.prototype.getKnowledgeBaseDetail = function(kb_id, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/knowledge_base/qna_maker/knowledgebases/get_detail/48b866be-f747-45bb-8a80-332a5cf6a0f8
		 */
		const uri = FORMAT("{0}/knowledgebases/get_detail/{1}",
			this.uri,
			encodeURIComponent(kb_id || "")
		);

		const request_handler = this.__Get(uri, function(success, info) {
			if (success) {
				callback(true, info);
				return;
			}
			// error
			callback(false, info);
		});
		return request_handler;
	};

	/*
	
	*/
	QnAMakerAPI.prototype.getVccKnowledgeBase = function(callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/knowledge_base/qna_maker/knowledgebases/vcc/get/
		 */
		const uri = FORMAT("{0}/knowledgebases/vcc/get/", this.uri);

		const request_handler = this.__Get(uri, function(success, info) {
			if (success) {
				callback(true, info);
				return;
			}
			// error
			callback(false, info);
		});
		return request_handler;
	};

	/*
	
	*/
	QnAMakerAPI.prototype.getKnowledgeBaseOperationDetail = function(operation_id, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/knowledge_base/qna_maker/knowledgebases/get_operation_detail/4b80-8a01-ec7d19582dbe0fd162e9-335
		 */
		const uri = FORMAT("{0}/knowledgebases/get_operation_detail/{1}",
			this.uri,
			encodeURIComponent(operation_id || "")
		);

		const request_handler = this.__Get(uri, function(success, info) {
			if (success) {
				callback(true, info);
				return;
			}
			// error
			callback(false, info);
		});
		return request_handler;
	};

	QnAMakerAPI.prototype.createKnowledgeBase = function(data, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/knowledge_base/qna_maker/knowledgebases/create/
		 */

		const uri = FORMAT("{0}/knowledgebases/create/", this.uri);

		const request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	QnAMakerAPI.prototype.updateKnowledgeBase = function(kb_id, data, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/knowledge_base/qna_maker/knowledgebases/update/48b866be-f747-45bb-8a80-332a5cf6a0f8
		 */

		const uri = FORMAT("{0}/knowledgebases/update/{1}",
			this.uri,
			encodeURIComponent(kb_id || "")
		);

		const request_handler = this.__Put(uri, callback, data);
		return request_handler;
	};

	/*
	
	*/
	QnAMakerAPI.prototype.deleteKnowledgeBase = function(kb_id, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/knowledge_base/qna_maker/knowledgebases/delete/48b866be-f747-45bb-8a80-332a5cf6a0f8
		 */

		const uri = FORMAT("{0}/knowledgebases/delete/{1}",
			this.uri,
			encodeURIComponent(kb_id || "")
		);

		const request_handler = this.__Delete(uri, callback);
		return request_handler;
	};

	QnAMakerAPI.prototype.publishKnowledgeBase = function(kb_id, data, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/knowledge_base/qna_maker/knowledgebases/publish/48b866be-f747-45bb-8a80-332a5cf6a0f8
		 */

		const uri = FORMAT("{0}/knowledgebases/publish/{1}",
			this.uri,
			encodeURIComponent(kb_id || "")
		);

		const request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	/*
	
	*/
	QnAMakerAPI.prototype.removeKnowledgeBaseOfOCC = function(callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/knowledge_base/qna_maker/knowledgebases/remove_occ/
		 */

		const uri = FORMAT("{0}/knowledgebases/remove_occ/", this.uri);
		const request_handler = this.__Delete(uri, callback);
		return request_handler;
	};

	QnAMakerAPI.prototype.getAnswerInfo = function(question, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/agent/qna_maker/runtime/get_answer_info/
		 */
		const agent_runtime_uri = this.__getAgentRuntimeURI();
		const uri = FORMAT("{0}/runtime/get_answer_info/", agent_runtime_uri);
		let data = {
			question: question
		};
		const request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	QnAMakerAPI.prototype.generateAnswer = function(info, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/agent/qna_maker/runtime/generate_answer/
		 */
		const agent_runtime_uri = this.__getAgentRuntimeURI();
		const uri = FORMAT("{0}/generate_answer/", agent_runtime_uri);
		let data = {
			question: info.question,
			metadata: info.metadata
		};
		const request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	QnAMakerAPI.prototype.generateAnswerV2 = function(info, callback = $.noop) // fede
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/agent/qna_maker/runtime/generate_answer_v2/
		 */
		const agent_runtime_uri = this.__getAgentRuntimeURI();
		const uri = FORMAT("{0}/generate_answer_v2/", agent_runtime_uri);
		let data = {
			question: info.question,
			metadata: info.metadata
		};
		const request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	QnAMakerAPI.prototype.getAjaxAnswer = function(question, callback = $.noop)
	{
		/*
		 * sample URI:
		 *
		 */
		var getAnswer = function(success, answer_request_info) {
			if (!success) {
				callback(success, answer_request_info);
				return;
			}
			console.log("QnAMakerAPI.prototype.getAnswer - answer_request_info: " + JSON.stringify(answer_request_info));

			const uri = answer_request_info.uri;
			let data = answer_request_info.postData;
			data.question = question;

			const encoded_post_data = JSON.stringify(data);

			console.log("QnAMakerAPI.prototype.getAnswer - post_data: " + encoded_post_data);

			let parameters = {
				url         : uri,
				type        : "POST",
				data        : encoded_post_data,
				dataType    : "json",
				contentType : "application/json",
				timeout     : REQUEST_TIMEOUT_MS,
				success     : function (response)    { callback(true, response); },
				error       : function (code, error) { callback(false, error);   }
			};

			parameters.beforeSend = function (request) {
				for (let name in answer_request_info.headers) {
					if (answer_request_info.headers.hasOwnProperty(name)) {
						let val = answer_request_info.headers[name];
						request.setRequestHeader(name, val);
					}
				}
			};

			return $.ajax(parameters);
		};

		const answer_request_info = this.getAnswerRequestInfo();
		if (answer_request_info) {
			return getAnswer(true, answer_request_info);
		}
		else {
			return this.getAnswerInfo(question, function(success, answer_request_info) {
				return getAnswer(success, answer_request_info);
			});
		}
	};

}());
