/* jshint smarttabs:true, quotmark:false, esversion: 9 */
/* global $, inConcert */


if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	"use strict";
	const FORMAT = inConcert.Utils.format;
	const REQUEST_TIMEOUT_MS = 300000; // 5 minutes
	const _T = inConcert.i18n.getString;

	function QnaV2Api(params)
	{
		if(typeof(params) !== "object") {
			throw("QnaV2Api.Constructor - expected an object with configuration");
		}
		if(typeof(params.uri ) !== "string") {
			throw("QnaV2Api.Constructor - base uri is required");
		}

		this.uri = params.uri + "/qna_v2";
	}

	if (typeof inConcert.QnaV2Api !== "function") {
		inConcert.QnaV2Api = QnaV2Api;
	}

	// QnaV2Api.prototype.__getAgentRuntimeURI = function()
	// {
	// 	return FORMAT("{0}/qna_maker/runtime", inConcert.app.getBaseUrl());
	// };

/********************************************************************************************************
	FUNCIONES GENERICAS
********************************************************************************************************/

	/*
	
	*/
	QnaV2Api.prototype.__Get = function(uri, callback = $.noop)
	{
		if (typeof(uri) !== "string") {
			throw("QnaV2Api.prototype.__Get: uri variable must be string type.");
		}
		if (typeof(callback) !== "function") {
			throw("QnaV2Api.prototype.__Get: callback variable must be function type.");
		}

		const request_handler = inConcert.invoke({
			url    : uri,
			type   : "GET",
			timeout: REQUEST_TIMEOUT_MS,
			success: function (list, result, server_response)
			{
				if (typeof(server_response) === "object") {
					server_response = JSON.stringify(server_response);
				}
				//console.log(uri + "\nlist: " + list + "\nresult: " + result + "\nserver_response: " + server_response);
				if (result.toLowerCase() === "success") {
					callback(true, list);
				}
				else {
					// Ejemplo: code 204 (nocontent) para el caso de getTrain #7167
					callback(false, server_response);
				}
			},
			error  : function (code, error) {callback(false, error);}
		});
		return request_handler;
	};

	/*
	
	*/
	QnaV2Api.prototype.__Delete = function(uri, callback, data)
	{
		if (typeof(uri) !== "string") {
			throw("QnaV2Api.prototype.__Delete: uri variable must be string type.");
		}
		if (typeof(callback) !== "function") {
			throw("QnaV2Api.prototype.__Delete: callback variable must be function type.");
		}

		const request_handler = inConcert.invoke({
			url         : uri,
			type        : "DELETE",
			data        : JSON.stringify(data || {}),
			contentType : "application/json",
			timeout     : REQUEST_TIMEOUT_MS,
			success     : function (response) { callback(true, response); },
			error       : function (code, error) {callback(false, error);}
		});
		return request_handler;
	};

	/*
	
	*/
	QnaV2Api.prototype.__Post = function(uri, callback, data)
	{
		if (typeof(uri) !== "string") {
			throw("QnaV2Api.prototype.__Post: uri variable must be string type.");
		}
		if (typeof(callback) !== "function") {
			throw("QnaV2Api.prototype.__Post: callback variable must be function type.");
		}
		if (typeof(data) !== "object") {
			throw("QnaV2Api.prototype.__Post: data variable must be object type.");
		}

		const request_handler = inConcert.invoke({
			url         : uri,
			type        : "POST",
			data        : JSON.stringify(data),
			dataType    : "json",
			contentType : "application/json",
			timeout     : REQUEST_TIMEOUT_MS,
			success     : function (response)    { callback(true, response); },
			error       : function (code, error) { callback(false, error);   }
		});
		return request_handler;
	};

	QnaV2Api.prototype.__PostFile = function(uri, callback, data)
	{
		if (typeof(uri) !== "string") {
			throw("QnaV2Api.prototype.__PostFile: uri variable must be string type.");
		}
		if (typeof(callback) !== "function") {
			throw("QnaV2Api.prototype.__PostFile: callback variable must be function type.");
		}
		const request_handler = inConcert.invoke({
			url         : uri,
			type        : "POST",
			data        : data,
			//dataType    : "json",
			contentType : false,
			processData : false,
			timeout     : REQUEST_TIMEOUT_MS,
			success     : function (response)    { callback(true, response); },
			error       : function (code, error) { callback(false, error);   }
		});
		return request_handler;
	};

	/*
	
	*/
	QnaV2Api.prototype.__Put = function(uri, callback, data)
	{
		if (typeof(uri) !== "string") {
			throw("QnaV2Api.prototype.__Put: uri variable must be string type.");
		}
		if (typeof(callback) !== "function") {
			throw("QnaV2Api.prototype.__Put: callback variable must be function type.");
		}
		if (typeof(data) !== "object") {
			throw("QnaV2Api.prototype.__Put: data variable must be object type.");
		}

		const request_handler = inConcert.invoke({
			url         : uri,
			type        : "PUT",
			data        : JSON.stringify(data),
			dataType    : "json",
			contentType : "application/json",
			timeout     : REQUEST_TIMEOUT_MS,
			success     : function (response)    { callback(true, response); },
			error       : function (code, error) { callback(false, error  ); }
		});
		return request_handler;
	};

	/*
	
	*/
	QnaV2Api.prototype.getCognitiveErrorMsg = function(err)
	{
		if (typeof err !== "object") {
			return "";
		}
		if (typeof err.cognitiveServiceError !== "object") {
			return "";
		}
		if (typeof err.cognitiveServiceError.response !== "object") {
			return "";
		}
		if (typeof err.cognitiveServiceError.response !== "object") {
			return "";
		}
		if (typeof err.cognitiveServiceError.response.error !== "object") {
			return "";
		}
		const msg_err = err.cognitiveServiceError.response.error.message;
		if (typeof msg_err !== "string") {
			return "";
		}
		return " " + _T(msg_err);
	};

	/*
	
	*/
	QnaV2Api.prototype.getCognitiveErrorObject = function(err)
	{
		if (typeof err !== "object") {
			return;
		}
		if (typeof err.cognitiveServiceError !== "object") {
			return;
		}
		if (typeof err.cognitiveServiceError.response !== "object") {
			return;
		}
		if (typeof err.cognitiveServiceError.response !== "object") {
			return;
		}
		if (typeof err.cognitiveServiceError.response.error !== "object") {
			return;
		}
		return err.cognitiveServiceError.response.error;
	};

	/*
	
	*/
	QnaV2Api.prototype.getAnswerRequestInfo = function()
	{
		const vcc_info = inConcert.app.get_vcc_info();
		if (!vcc_info) {
			return;
		}
		if (!vcc_info.QnAMaker) {
			return;
		}
		return vcc_info.QnAMaker.AnswerRequestInfo;
	};

/********************************************************************************************************
	KNOWLEDGEBASE
********************************************************************************************************/
	/*
	
	*/

	QnaV2Api.prototype.asyncCall = async function(method, ...args)
	{
		return inConcert.Utils.asyncCallObjectMethod(this, method, ...args);
	};

	QnaV2Api.prototype.getAllKnowledgeBases = function(page, size, order, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/knowledge_base/qna_v2/knowledgebases/get_all/
		 */
        
		callback = callback || $.noop;
		var uri = FORMAT(
            "{0}/knowledgebases/get_all/?page={1}&size={2}&order={3}",
            this.uri,
            page,
            size,
            order
        );

		var request_handler = this.__Get(uri, function(success, knowledgeBases) {
			if (success) {
				callback(true, knowledgeBases);
				return;
			}
			// error
			callback(false, knowledgeBases);
		});
		return request_handler;
	};

	QnaV2Api.prototype.getKnowledgeBasesById = function(kb_id, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/knowledge_base/qna_v2/knowledgebases/get/48b866be-f747-45bb-8a80-332a5cf6a0f8/
		 */
		callback = callback || $.noop;

		var uri = FORMAT("{0}/knowledgebases/get/{1}",
			this.uri,
			encodeURIComponent(kb_id || "")
		);

		var request_handler = this.__Get(uri, function(success, knowledgeBase) {
			if (success) {
				callback(true, knowledgeBase);
				return;
			}
			// error
			callback(false, knowledgeBase);
		});
		
		return request_handler;
	};

	QnaV2Api.prototype.searchKnowledgeBases = function(filter, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/knowledge_base/qna_v2/knowledgebases/search/hello
		 */
		callback = callback || $.noop;

		var uri = FORMAT("{0}/knowledgebases/search/{1}",
			this.uri,
			encodeURIComponent(filter || "")
		);

		var request_handler = this.__Get(uri, function(success, knowledgeBase) {
			if (success) {
				callback(true, knowledgeBase);
				return;
			}
			// error
			callback(false, knowledgeBase);
		});
		
		return request_handler;
	};

	QnaV2Api.prototype.addKnowledgeBase = function(data, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/knowledge_base/qna_v2/knowledgebases/create/
		 */
		callback = callback || $.noop;

		var uri = FORMAT("{0}/knowledgebases/create/", this.uri);

		var request_handler = this.__Post(uri, callback, data);

		return request_handler;
	};

	QnaV2Api.prototype.updateKnowledgeBase = function(data, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/knowledge_base/qna_v2/knowledgebases/update/
		 */
		callback = callback || $.noop;

		var uri = FORMAT("{0}/knowledgebases/update/", this.uri);

		var request_handler = this.__Post(uri, callback, data);

		return request_handler;
	};

	QnaV2Api.prototype.addKnowledgeBaseFromVideo = function(data, name, language, callback)
	{
		// /*
		//  * sample URI:
		//  *   http://i6.inconcert.cc/inconcert/apps/knowledge_base/qna_v2/knowledgebases/create/video/
		//  */
		callback = callback || $.noop;

		var uri = FORMAT("{0}/knowledgebases/create/video/?name=" + encodeURIComponent(name || "") + "&language="+ encodeURIComponent(language || ""), this.uri);

		var request_handler = this.__PostFile(uri, callback, data);

		
		return request_handler;
	};

	QnaV2Api.prototype.addKnowledgeBaseFromPDF = function(data, name, language, callback)
	{
		// /*
		//  * sample URI:
		//  *   http://i6.inconcert.cc/inconcert/apps/knowledge_base/qna_v2/knowledgebases/create/pdf/
		//  */
		callback = callback || $.noop;

		var uri = FORMAT("{0}/knowledgebases/create/pdf/?name=" + encodeURIComponent(name || "") + "&language="+ encodeURIComponent(language || ""), this.uri);

		var request_handler = this.__PostFile(uri, callback, data);

		return request_handler;
	};

	QnaV2Api.prototype.deleteKnowledgeBase = function(kb_id, callback)
	{
		/*
		 * sample URI:
			http://i6.inconcert.cc/inconcert/apps/knowledge_base/qna_v2/knowledgebases/delete/48b866be-f747-45bb-8a80-332a5cf6a0f8/
		 *   
		 */
		callback = callback || $.noop;

		var uri = FORMAT("{0}/knowledgebases/delete/{1}",
			this.uri,
			encodeURIComponent(kb_id || "")
		);

		var request_handler = this.__Delete(uri, callback);
		
		return request_handler;
	};

	QnaV2Api.prototype.updateKnowledgeBaseQuestions = function(kb_id, data, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/knowledge_base/qna_v2/knowledgebases/48b866be-f747-45bb-8a80-332a5cf6a0f8/version/
		 */
		callback = callback || $.noop;

		console.log("UPDATED DATA EN API :", data);

		var uri = FORMAT("{0}/knowledgebases/{1}/version/",
			this.uri,
			encodeURIComponent(kb_id || ""),
		);

		var request_handler = this.__Put(uri, callback, data);

		
		return request_handler;
	};

	QnaV2Api.prototype.changeActiveVersion = function(kb_id, callback)
	{
		/*
		 * sample URI:
			http://i6.inconcert.cc/inconcert/apps/knowledge_base/qna_v2/knowledgebases/48b866be-f747-45bb-8a80-332a5cf6a0f8/active/
		 *  
		 */
		callback = callback || $.noop;

		var uri = FORMAT("{0}/knowledgebases/{1}/active/",
			this.uri,
			encodeURIComponent(kb_id || ""),
		);

		var request_handler = this.__Put(uri, callback, {});
		
		return request_handler;
	};

    QnaV2Api.prototype.getRequestStatus = function(requestId, callback)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/knowledge_base/qna_v2/knowledgebases/request/48b866be-f747-45bb-8a80-332a5cf6a0f8/
		 */
		callback = callback || $.noop;

		var uri = FORMAT("{0}/knowledgebases/request/{1}",
			this.uri,
			encodeURIComponent(requestId || "")
		);

		var request_handler = this.__Get(uri, function(success, response) {
			if (success) {
				callback(true, response);
				return;
			}
			// error
			callback(false, response);
		});
		
		return request_handler;
	};

}());
