/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, esversion: 11 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

//	"use strict";

	const _T = inConcert.i18n.getString;

	//===================================================================================================
	//===================================================================================================
	/**
	Class campaign_selector_popup.
	* @param params

		Es interna de este scope, y es instanciada por la clase "campaign_selector".
		Hereda de la clase "inConcert.PopupWindow"
		La misma genera la lista de campanas, cuentas y attention levels.
		Los lista en forma de tree.

	*/
	function campaign_selector_popup (params)
	{
		campaign_selector_popup.uber.constructor.call(this);

		this.clickEventName = this.clickEventName + "campaignSelector";
		this.ownerId = "window_campaign_selector_popup_container";
		this.canReposition = true;

		this.prefix = "popup_campign_selector_list_";

		this.id_campaigns_list = this.prefix + inConcert.Utils.generateGuid();
		this.id_select = this.prefix + "_select_campaign";
		this.id_close = this.prefix + "_close_popup";
		this.id_waiting = this.prefix + "_waiting";
		this.id_message = this.prefix + "_message";

		this.from_app = params.from_app;
		this.callbacks = params.callbacks;
		this.popup_config = params.popup_config;
	}

	inConcert.Utils.inherit(campaign_selector_popup, inConcert.PopupWindow);

	/*******************************************************************
	* Metodo que crea el markup contenedor de la ventana
	*/
	campaign_selector_popup.prototype.create_markup = function()
	{
		$("#" + this.ownerId).remove();

		var $popup = $("".concat(
			"<div id='", this.ownerId, "' class='systemPopWindow windowContainer scopeSelectorPopup' hidden>",

				"<div id='", this.id_waiting, "' class='windowWaitingContainer' hidden>",
					"<div message='true' class='windowWaitingMessage'></div>",
					"<div class='windowWaitingAnimation'></div>",
					"<div class='windowWaitingOverlay'></div>",
				"</div>",

				"<div popup_header='true' class='windowTop'>",
					"<div class='windowTitle'>",
						"<div title='true' class='TU_CLASE_windowTitleText'></div>",
						"<img src='/inconcert/apps/agent/images/blank.png' class='TU_CLASE_windowButClose' />",
					"</div>",
				"</div>",

				"<div id='", this.id_message, "' class='windowContainerMessages messageError' hidden>",
					"<a close_msg='true' class='windowMessagesClose' href='javascript:void(0);'>",  _T("close"), "</a>",
					"<div class='windowMessagesText'>",
						"<label class='windowMessagesTextAux'></label>",
					"</div>",
				"</div>",

				"<div class='windowMiddle'>",
					"<ul id='", this.id_campaigns_list, "' class='campaignArea'></ul>",
				"</div>",

				"<div class='windowBottom'>",
					//"<input id='", this.id_close, "' class='buttonColorWhite butWindowClose' type='button' value='", _T("Cancel"), "'/>",
					"<input id='", this.id_select, "' class='buttonColorBlue butWindowOk mr-0' disabled='true' type='button' value='", _T("Select"), "'/>",
				"</div>",

			"</div>"));

		$popup.find("[popup_header]")[this.popup_config.shown_header ? "show" : "hide"]();

		$("body").append($popup);

		return $popup;
	};

	campaign_selector_popup.prototype.set_popup_title = function()
	{
		var popup_title = this.popup_config.title || _T("Campaign Selector");
		$("#" + this.ownerId + " [title=true]").text(popup_title);
	};

	/*******************************************************************
	* @param message
	*/
	campaign_selector_popup.prototype.__show_waiting = function(message)
	{
		message = message || _T("Loading, wait moment please");
		var $waiting = $("#" + this.id_waiting);
		$waiting.find("[message='true']").text(message);
		$waiting.show();
	};

	/*******************************************************************
	*/
	campaign_selector_popup.prototype.__hide_waiting = function()
	{
		var $waiting = $("#" + this.id_waiting);
		$waiting.hide();
		this.setPosition();
	};

	/*******************************************************************
	*
	* @param text
	* @param type
	* @param closeDisabled
	* @param timeout_ms
	*/
	campaign_selector_popup.prototype.__show_status_message = function(text, type, closeDisabled, timeout_ms)
	{
		var $message = $("#" + this.id_message);
		$message.removeClass().addClass("windowContainerMessages").addClass("message" + type);
		$message.show().find(".windowMessagesTextAux").text(text);
		if (timeout_ms) {
			window.setTimeout(function() {
				$message.hide();
			}, timeout_ms);
		}
		var $close_msg = $message.find("[close_msg]");
		if (closeDisabled) {
			$close_msg.hide();
		}
		else {
			$close_msg.show();
		}
	};

	/*******************************************************************
	* funcion que oculta mensaje
	*/
	campaign_selector_popup.prototype.__hide_status_message = function() {
		$("#" + this.id_message).hide();
	};

	/*******************************************************************
	* Metodo que setea eventos de los controles
	*/
	campaign_selector_popup.prototype.set_events = function()
	{
		var self = this;

		$("#" + this.id_message).find("[close_msg]")
			.bind("click", function() {
				self.__hide_status_message();
			});

		var close_popup = function() { self.close(); };

		// evento que cierra el dialogo y no realiza cambios
		$("#" + this.id_close).bind("click", close_popup);

		var $campaigns = $("#" + this.id_campaigns_list);

		// evento que obtiene la opcion seleccionada, cierra el popup y retorna setea el valor seleccionado control
		// correspondiente al combo.
		$("#" + this.id_select).bind("click", function()
		{
			var selected;

			var $marked_campaign = $campaigns.find("[campaign_selector='true']:checked");
			// Si ha marcado una campana
			if ($marked_campaign.length > 0) {
				var $marked_account = $marked_campaign.parent().find("[accounts_list] [account_selector]:checked");
				// Si ha marcado una cuenta
				if ($marked_account.length > 0) {
					var account_item_type = $marked_account.attr("account_selector");
					// Si ha marcado una cuenta
					if (account_item_type === "account_item") {
						var $marked_al = $marked_campaign.parent().find("[attention_levels_list] [attention_level_selector]:checked");
						// Si ha marcado un attention level
						if ($marked_al.length > 0) {
							// Si marco una attenion level unicamente
							selected = $marked_al.data("info");
						}
						else {
							// Si marco una cuenta unicamente
							selected = $marked_account.data("info");
						}
					}
					else {
					// Si marco la opcion que incluye todas las cuentas de la campana
						selected = $marked_account.data("info");
					}
				}
				else {
					// Si marco una campana unicamente
					selected = $marked_campaign.data("info");
				}
			}

			if (selected) {
			// Como ha seleccionado una opcion, invoco al callback con valor asignado y cierro el popup.
				self.callbacks.set_selected_option(selected);
				close_popup();
			}
		});
	};

	campaign_selector_popup.prototype.__mark_unmark_campaign_account = function( $radio, mark, from )
	{
		var unmark_account_als = function( $assigned_als )
		{
			$.each($assigned_als, function( _, assigned_al )
			{
				var $assigned_al = $(assigned_al);
				$assigned_al.removeClass('selected');
				var $al_selector = $assigned_al.find('[attention_level_selector]');
				$al_selector.prop('checked', false);
			});
		};

		var hide_account_als = function( $current_account_selector, $current_als_container )
		{
			$current_account_selector.prop('checked', false);
			$current_als_container.hide();
		};

		var $current_account = $radio.parent();
		var $all_campaign_accounts = $current_account.parent().find('[account_item_type]');

		$.each($all_campaign_accounts, function(_, acc)
		{
			var $other_account = $(acc);
			$other_account.removeClass('selected childSelected');
			var $other_account_selector = $other_account.find('[account_selector]');
			var $als_container = $other_account.find('[attention_levels_list]');
			var $assigned_als = $als_container.find('[attention_level_item_type]');
			unmark_account_als( $assigned_als );
			hide_account_als( $other_account_selector, $als_container );
		});

		if ( mark ) {
			$current_account.addClass('selected');
			if ( from === 'attention_level' ) {
				$current_account.addClass('childSelected');
			}
			$current_account.find('[attention_levels_list]').show();
			$radio.prop('checked', true);
		}
	};

	/*******************************************************************
	* Metodo que visualiza el popup con el campaign selector e inicializa eventos del popup!
	* A este metodo lo invoca la clase base "inConcert.PopupWindow"
	*/
	campaign_selector_popup.prototype.initialize = function()
	{
		this.data.container = this.create_markup();
		this.set_popup_title();
		this.set_events();
		this.__load_campaigns();
	};

	campaign_selector_popup.prototype.__normalize_id = function(id)
	{
		return inConcert.Utils.getNormalizedId(id);
	};

	/*******************************************************************
	*
	*/
	campaign_selector_popup.prototype.__load_campaigns = function()
	{
		var self = this;

		this.__show_waiting();

		var $campaigns = $("#" + this.id_campaigns_list);
		$campaigns.html("");

		var unmark_another_campaigns = function(current_campaign_id)
		{
			// oculto la lista de accounts y attentions levels de otras cuentas
			var $accounts_list_of_all_campaigns = $campaigns.find('[accounts_list][campaign_id]');
			$.each($accounts_list_of_all_campaigns, function(_, list) {
				var $list = $(list);
				if ($list.attr("campaign_id") !== current_campaign_id) {
					$list.hide();
				}
			});
		};

		this.popup_config.load_campaigns(function(success, campaigns)
		{
			if (!success) {
				// TODO: agregar lista vacia y visualizar mensaje de error
				self.__show_status_message(_T("Error loading campaigns."), "Error");
				self.__hide_waiting();
				return;
			}

			if (!$.isArray(campaigns) || campaigns.length === 0) {
				// TODO: agregar lista vacia
				self.__hide_waiting();
				return;
			}

			var event_selector = '[campaign_selector="true"]';
			$campaigns
				.off( 'change', event_selector)
				.on(  'change', event_selector, function()
				{
					var $radio = $(this);
					var campaign_id = $radio.attr('campaign_id');
					var is_marked_campaign = true;
					self.__on_selected_item( is_marked_campaign, campaign_id );

					// muestro la lista de accounts y attentions levels
					var $account_list = $campaigns.find('[accounts_list][campaign_id="' + campaign_id + '"]');
					if ( is_marked_campaign ) {
						$account_list.show();
					}
					unmark_another_campaigns(campaign_id);
				});

			var hash_added_campaigns = {};
			var none_camp_dom_id;
			// (**) Si tiene un unico item en el combo (es decir una sola campana y el primer elemento no es un elemento en blanco)
			var has_single_campaign = campaigns.length === 1 && !self.popup_config.blank_first_item;

			$.each(campaigns, function(i, camp)
			{
				var campaign = inConcert.app.getCampaign(camp.Id);
				if (!campaign) {
				// TODO: hacer algo ????
					return true; // continue
				}

				if (i === 0 && self.popup_config.blank_first_item) {
					none_camp_dom_id = "NONE_CAMPAIGNS_" + inConcert.Utils.generateGuid();
					var $blank = $(
						"<li class='campaignListItem'>" +
							"<input id='" + none_camp_dom_id + "' campaign_id='NONE-CAMPAIGN' campaign_selector='true' class='summaryRowCheck campaignRowCheck' type='radio' name='popup_camp_selector_item' />" +
							"<label for='" + none_camp_dom_id + "' class='summaryName campaignRowName'>" + _T("NONE CAMPAIGN") + "</label>" +
						"</li>"
					);
					$blank.find("[id='" + none_camp_dom_id + "']").data("info", {blank: true});

					$campaigns.append($blank);
				}

				var marked = false;
				// Si se da el caso (**), el item correspondiente a la campana (label) debe estar oculto.
				//   En este caso lo que NO debe estar oculto, y SI debe de visualizarse es los items correspondientes a las cuentas y ALs.
				var is_hidden_campaign = has_single_campaign ? true : false;
				self.__add_campaign_selector_node(campaign, marked, is_hidden_campaign);
				hash_added_campaigns[camp.Id] = true;
			});

			// Aca dejamos marcada en el combo/popup la campana/cuenta/al que tiene asignando el control input.
			//
			var selected_campaign_id, selected_account_id, selected_att_level_id;
			var selected_campaign = self.callbacks.get_selected_option(); // Aca le pedimos al control input el valor que tiene seleccionado.
			if (selected_campaign) {
				// Si el input correspondiente al valor del combo seleccionado tiene algun valor seleccionado.
				selected_campaign_id = selected_campaign.campaign_id;
				selected_account_id = selected_campaign.account_id;
				selected_att_level_id = selected_campaign.al_id;
			}
			else {
				// Si el input correspondiente al valor del combo seleccionado NO tiene ningun valor seleccionado.
				if (has_single_campaign) {
					// Si se da el caso (**), tiene que quedar seleccionado el item correspondiente a la unica campana (el item All accounts).
					selected_campaign_id = campaigns[0].Id;
				}
			}

			if (selected_campaign_id && hash_added_campaigns[selected_campaign_id]) {
			// Si tiene una campana seleccionada
				var camp_dom_id = self.__make_camp_dom_id(selected_campaign_id);
				var $campaign = $("#" + camp_dom_id);
				if ($campaign.length > 0) {
					$campaign.trigger("click");
					if (selected_account_id) {
					// Si tiene una cuenta seleccionada
						var dom_account_id = self.__make_account_dom_id(selected_campaign_id, selected_account_id);
						var $account = $("#" + dom_account_id);
						if ($account.length > 0) {
							$account.trigger("click");
							if (selected_att_level_id) {
							// Si tiene un AL seleccionado.
								var dom_att_level_id = self.__make_att_level_dom_id(selected_campaign_id, selected_account_id, selected_att_level_id);
								var $al = $("#" + dom_att_level_id);
								if ($al.length > 0) {
									$al.trigger("click");
								}
							}
						}
					}
				}
			}
			else {
				// Si no tiene ninguna campana seleccionada y la lista en el combo contiene el primer elemento en blanco,
				// lo dejo seleccionado.
				if (self.popup_config.blank_first_item && none_camp_dom_id) {
					var $none_campaign = $("#" + none_camp_dom_id);
					if ($none_campaign.length > 0) {
						$none_campaign.trigger("click");
					}
				}
			}

			self.__hide_waiting();
		});
	};

	campaign_selector_popup.prototype.__add_campaign_selector_node = function(campaign, marked_campaign, is_hidden_campaign)
	{
		var self = this;

		var campaign_id = campaign.Id;
		var camp_dom_id = this.__make_camp_dom_id(campaign_id);
		// Aca lo que se oculta es el label correspondiente a la campana, pero lo que se continua visualizando es la lisa de cuentas y ALs.
		//  Este caso se puede dar cuando tenemos un unica campana y el primer elemento no es un elemento en blanco.
		var hidden_attr = is_hidden_campaign ? "hidden" : "";

		var $campaign = $(
			"<li class='campaignListItem'>" +
				"<input id='" + camp_dom_id + "' campaign_id='" + campaign_id + "' campaign_selector='true' class='summaryRowCheck campaignRowCheck' type='radio' name='popup_camp_selector_item' " + hidden_attr + " />" +
				"<label for='" + camp_dom_id + "' class='summaryName campaignRowName' " + hidden_attr + ">" + campaign.getDescription() + "</label>" +
				"<ul accounts_list='true' campaign_id='" + campaign_id + "' class='accountArea' hidden></ul>" +
			"</li>"
		);

		if (marked_campaign) {
			$campaign.find("[campaign_selector]").prop("checked", true);
		}
		// Se agrega nuevo nodo campana
		var $campaigns = $("#" + this.id_campaigns_list);
		$campaigns.append($campaign);

		var $accounts_list = $campaign.find("[accounts_list]");
		var is_allegro_campaign = campaign.isAllegro();
		var $allegro_account;

		// agrega una cuenta al tree
		var add = function( account )
		{
			if (!is_allegro_campaign) {
				var $all_campaign = self.__$make_header_campaign_accounts_selector_node(campaign);
				if ( $all_campaign ) {
					$accounts_list.append($all_campaign);
				}
			}
			var $account = self.__$make_account_selector_node(campaign, account);
			if ( $account ) {
				$accounts_list.append($account);
				if (is_allegro_campaign) {
					$allegro_account = $account;
				}
			}
		};

		// recorro la lista de cuentas y las agrego
		var accounts = campaign.get_accounts();
		$.each(accounts || {}, function(_, account) { add(account); });
		// append campaign new node
		if ( marked_campaign ) {
			$campaign.find('.campaignRowCheck').trigger('change');
			if ($allegro_account) {
				// si es una allegro account, tengo que dejarla marcada
				var $account_chk = $allegro_account.find('input[account_selector]');
				if ($account_chk.length > 0) {
					$account_chk.trigger('click');
				}
			}
		}
	};

	campaign_selector_popup.prototype.__enable_select_button = function() {
		$("#" + this.id_select).prop("disabled", false);
	};

	campaign_selector_popup.prototype.__$make_header_campaign_accounts_selector_node = function(campaign)
	{
		var self = this;

		var campaign_id = campaign.Id;
		var dom_header_account_id = this.__make_header_account_dom_id(campaign_id);

		if ($('#' + dom_header_account_id).length > 0) {
			return;
		}

		var $header = $(
			"<li class='accountListItem' account_item_type='header'>" +
				"<input id='" + dom_header_account_id + "' class='accountRowRadio' campaign_id='" + campaign_id + "' account_selector='header_item' type='radio' />" +
				"<label for='" + dom_header_account_id + "' class='accountRowName accountRowAll'>" + _T("Campaign (All Accounts)") + "</label>" +
			"</li>"
		);
		$header.find("[id='" + dom_header_account_id + "']").data("info", {campaign: campaign, scope: "campaign", campaign_desc: campaign.getDescription()});

		var event_selector = '[id="' + dom_header_account_id + '"]';
		$header
			.off('click', event_selector)
			.on('click', event_selector, function()
			{
				var $radio = $(this);
				var $header = $radio.parent();
				var marked = $header.hasClass('selected');
				var $marked_account = $header.parent().find('[account_item_type="account"] > [account_selector]:checked');
				$.each($marked_account, function(_, acc_radio)
				{
					var $acc_radio = $(acc_radio);
					self.__mark_unmark_campaign_account( $acc_radio, false /* mark */ );
				});
				if ( !marked ) {
					$header.addClass('selected');
					$radio.prop('checked', true);
				}
				var campaign_id = $radio.attr('campaign_id');
				self.__on_selected_item( true, campaign_id );
			});

		$header.addClass('selected');
		var $header_radio = $header.find('[account_selector]');
		$header_radio.prop('checked', true);

		return $header;
	};

	campaign_selector_popup.prototype.__get_account_desc = function(campaign, account)
	{
		var account_name = campaign.getAccountDescription(account);
		return campaign.isAllegro() ? _T("Phone Account") : account_name;
	};

	campaign_selector_popup.prototype.__$make_account_selector_node = function(campaign, account)
	{
		var self = this;

		var campaign_id = campaign.Id;
		var account_id = account.id;
		var dom_account_id = this.__make_account_dom_id(campaign_id, account_id);
		if ( $('#' + dom_account_id).length > 0 ) {
			return;
		}

		var account_name = campaign.getAccountDescription(account);
		var account_desc = this.__get_account_desc(campaign, account);

		var $account = $(
			"<li class='accountListItem' account_item_type='account'>" +
				"<input id='" + dom_account_id + "' account_id='" + account_id + "' account_name='" + account_name + "' channel='" + account.proxyType + "' class='accountRowRadio' campaign_id='" + campaign_id + "' account_selector='account_item' type='radio' />" +
				"<label for='" + dom_account_id + "' class='accountRowName'>" + account_desc + "</label>" +
				"<ul attention_levels_list='true' account_id='" + account_id + "' class='attentionLevelArea' hidden></ul>" +
			"</li>"
		);
		$account.addClass("accountItem" + account.proxyType.toUpperCase());
		$account.find("[id='" + dom_account_id + "']").data("info", {campaign: campaign, account_id: account_id, scope: "channels", channel: account.proxyType, campaign_desc: campaign.getDescription(), account_desc: account_desc });

		var $als_list = $account.find('[attention_levels_list]');
		$.each(account.AttentionLevels || {}, function(al_id)
		{
			var al_def = campaign.getAttentionLevel(al_id);
			var $al = self.__$make_attention_level_selector_node(campaign, account, al_def);
			$als_list.append($al);
		});

		var event_selector = '[id="' + dom_account_id + '"]';
		$account
			.off('click', event_selector)
			.on('click', event_selector, function()
			{
				var $radio = $(this);
				self.__mark_unmark_campaign_account( $radio, true /* mark */ );
				var campaign_id = $radio.attr('campaign_id');
				self.__on_selected_item( true, campaign_id );
			});

		return $account;
	};

	campaign_selector_popup.prototype.__$make_attention_level_selector_node = function( campaign, account, al )
	{
		var self = this;

		var campaign_id = campaign.Id;
		var account_id = account.id;
		var al_id = al.id;
		var dom_att_level_id = this.__make_att_level_dom_id(campaign_id, account_id, al_id);
		var al_def = campaign.getAttentionLevel(al_id);
		var dom_account_id = this.__make_account_dom_id(campaign_id, account_id);
		var account_desc = this.__get_account_desc(campaign, account);

		var $attention_level = $(
			"<li class='attentionLevelListItem' attention_level_item_type='al'>" +
				"<input id='" + dom_att_level_id + "' class='attentionLevelRowRadio' name='" + dom_account_id + "' attention_level_id='" + al_id + "' attention_level_name='" + al_def.name + "' campaign_id='" + campaign_id + "' account_id='" + account_id + "' attention_level_selector='true' type='radio' />" +
				"<label for='" + dom_att_level_id + "' class='attentionLevelRowName'>" + al_def.name + "</label>" +
			"</li>"
		);
		$attention_level.find("[id='" + dom_att_level_id + "']").data("info", {campaign: campaign, account_id: account_id, al_id: al_id, scope: "att_levels", channel: account.proxyType, campaign_desc: campaign.getDescription(), account_desc: account_desc, att_level_desc: al_def.name });

		var event_selector = '[attention_level_selector]';
		$attention_level
			.off('click', event_selector)
			.on('click', event_selector, function()
			{
				var $radio = $(this);
				var campaign_id = $radio.attr('campaign_id');
				var account_id = $radio.attr('account_id');
				var normalized_account_id = self.__make_account_dom_id(campaign_id, account_id);
				var $account_radio = $('#' + normalized_account_id);
				self.__mark_unmark_campaign_account( $account_radio, true /* mark */, 'attention_level' );
				$radio.prop('checked', true);
				$radio.parent().addClass('selected');
				self.__on_selected_item( true, campaign_id );
			});

		return $attention_level;
	};

	campaign_selector_popup.prototype.__make_camp_dom_id = function(campaign_id)
	{
		var normalized_id = this.__normalize_id(campaign_id);
		return "popup_campign_item_" + normalized_id;
	};

	campaign_selector_popup.prototype.__make_header_account_dom_id = function(campaign_id)
	{
		var normalized_id = this.__normalize_id(campaign_id);
		return "popup_header_account_item_" + normalized_id;
	};

	campaign_selector_popup.prototype.__make_account_dom_id = function(campaign_id, account_id)
	{
		var camp_normalized_id = this.__normalize_id(campaign_id);
		var account_normalized_id = this.__normalize_id(account_id);
		return "popup_account_item_" + camp_normalized_id + "_" + account_normalized_id;
	};

	campaign_selector_popup.prototype.__make_att_level_dom_id = function(campaign_id, account_id, al_id)
	{
		var camp_normalized_id = this.__normalize_id(campaign_id);
		var account_normalized_id = this.__normalize_id(account_id);
		var al_normalized_id = this.__normalize_id(al_id);
		return "popup_att_level_item_" + camp_normalized_id + "_" + account_normalized_id + "_" + al_normalized_id;
	};

	campaign_selector_popup.prototype.__on_selected_item = function(/*add, campaign_id*/) {
		this.__enable_select_button();
	};

	/*******************************************************************
	* Metodo que cierra el popup
	*/
	campaign_selector_popup.prototype.close = function() {
		if (this.data.container) {
			this.data.container.remove();
		}
		this.finalize();
	};

	//===================================================================================================
	//===================================================================================================
	/**
	Class campaign_selector.

	* Description:
			Esta clase se instancia por cualquier clase externa, ya que la misma es expuesta en el scope "inConcert"
			La misma genera los controles del combo (label, input, y una instancia de la clase "campaign_selector_popup")

	* @param params
		Sample @params: 
		this.m_scope_popup = new inConcert.campaign_selector(
		{
			// configuracion y datos de la lista que se utilizaran en la clase "campaign_selector_popup".
			popup_config: {
				shown_header:       booelan,        // Si queremos o no visualizar el titulo en el selector
				title:              string          // El titulo a visualizar en el selector, si el mismo no se ha deifindo se agrega un titulo por defecto.         
				blank_first_item  : booelan,        // Si el combo va a tener el primer item en blanco o no
				include_accounts  : booelan,        // Si el combo incluye cuentas o solo campanas.
				include_att_levels: booelan,        // Si el combo incluye attention levels o solo campanas y cuentas.
				load_campaigns:     function        // Funcion que se encarga de retornar la lista de campanas a visualizar en el popup.
				                                    // La misma debe de tener el siguiente formato -> [{Id: camp1@vcc}, {Id: camp2@vcc}]
				on_selected:        function        // Funcion que es invocada cuando se ha seleccionado un item y se ha presionado el boton "select"
				                                    // La misma recibe un objeto con la opcion seleccioanda.
			},

			from_app:               string          // La app que me ha instanciado (por ahora no se hace nada pero por las dudas)
			container_id:           string          // El DOM id destino, donde se crearan los input y labels del cbo. El elemento debe existir en el DOM.
			selector_label:         string          // Texto a visualizar en el control label
			selector_label_classes: string          // Clase (css) a asignar en el control label
			selector_text_classes:  string          // Clase (css) a asignar en el control input
		});
	*/
	function campaign_selector (params)
	{
		var self = this;

		this.container_id = params.container_id;
		this.selector_id = "open_campign_popup_" + inConcert.Utils.generateGuid();
		this.selector_label = params.selector_label;
		this.selector_label_classes = params.selector_label_classes;
		this.selector_text_classes = params.selector_text_classes;
		this.params = params;

		this.params.callbacks = {
			get_selected_option : function() {
				return self.get_selected_info();
			},
			set_selected_option : function(selected_info) {
				self.__set_selected_info(selected_info);
			}
		};
		this.__initialize();
	}

	if (typeof(inConcert.campaign_selector) !== "function") {
		inConcert.campaign_selector = campaign_selector;
	}

	// -------------------
	// Funciones privadas
	// -------------------

	campaign_selector.prototype.__initialize = function()
	{
		this.__build_markup();
		this.__set_events();
		this.popup = new campaign_selector_popup(this.params);
	};

	campaign_selector.prototype.__build_markup = function()
	{
		var $selector_container = $("#" + this.container_id);
		$selector_container.html("".concat(
			'<label class="', this.selector_label_classes, '">', this.selector_label, '</label>',
			'<input id="', this.selector_id, '" type="text" class="', this.selector_text_classes, '" readonly="true" />'
		));
	};

	campaign_selector.prototype.__set_events = function()
	{
		var self = this;

		$("#" + this.selector_id)
			.off('click')
			.on('click', function() {
				if (self.popup.isVisible()) {
					self.__hide();
				}
				else {
					self.__show();
				}
			});
	};

	campaign_selector.prototype.__show = function()
	{
		var $selector = $("#" + this.selector_id);
		this.popup.show({ parentNode: $selector });
	};

	campaign_selector.prototype.__hide = function()
	{
		this.popup.close();
	};

	campaign_selector.prototype.__make_selected_display_value = function(info)
	{
		if (!info || info.blank) {
			return "";
		}
		// scope campaign
		var display_value = info.campaign_desc;
		if (info.scope === "channels") {
			// scope channels
			display_value = display_value + " > " + info.account_desc;
		}
		else if (info.scope === "att_levels") {
			// scope attention levels
			display_value = display_value + " > " + info.account_desc + " > " + info.att_level_desc;
		}
		return display_value;
	};

	campaign_selector.prototype.__set_selected_info = function(selected_info)
	{
		var self = this;
		var $selector = $("#" + this.selector_id);

		var set = function(display_value)
		{
			$selector.val(display_value || "");
			$selector.data("info", selected_info);
			if (typeof(self.params.popup_config.on_selected) === "function") {
				self.params.popup_config.on_selected(selected_info);
			}
		};

		if (!selected_info) {
			set();
			return;
		}

		var display_value = this.__make_selected_display_value(selected_info);
		set(display_value);
	};

	// -------------------
	// Funciones publicas
	// -------------------
	
	campaign_selector.prototype.get_selected_info = function()
	{
		var $selector = $("#" + this.selector_id);
		var info = $selector.data("info");
		if (!info || info.blank) {
			return;
		}

		var selected_info = {
			campaign_id: info.campaign.Id,
			scope      : info.scope
		};

		if (info.scope !== "campaign") {
			selected_info.account_id = info.account_id;
			selected_info.channel = info.channel;
			if (info.scope === "att_levels") {
				selected_info.al_id = info.al_id;
			}
		}
		selected_info.description = this.__make_selected_display_value(info);

		return selected_info;
	};

	campaign_selector.prototype.enable = function() {
		$("#" + this.selector_id).prop("disabled", false);
	};

	campaign_selector.prototype.disable = function() {
		$("#" + this.selector_id).prop("disabled", true);
	};

}());
