/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
  strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:single, esversion: 11 */

/* global $, inConcert, _ */

// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	'use strict';

	const _T = inConcert.i18n.getString;
	const FORMAT = inConcert.Utils.format;
	const INTERACTIONS_PER_PAGE = 25;
	const nid = inConcert.Utils.getNormalizedId;

	function GlobalSearch(params = {})
	{
		this.queryBuilderObj = params.queryBuilderObj || null;
		this.last_control    = params.last_control || false;
		this.agent_filter    = params.agent_filter || null;
		this.container_id    = params.container_id;
		// TODO: #9778 - CANY - Docked.
		this.panel_name      = params.panel_name || 'global_search';
		if ( typeof this.container_id !== 'string' ) {
			throw('GlobalSearch: params.container_id is required!!');
		}
		this.panel_name_text      = params.panel_name_text || _T('Global search');
		this.$container = $('#' + this.container_id);
		this.MAX_ALLOWED_ROWS_IN_ES = inConcert.app.getServerInfo().maxAllowedRowsInES; // Este valor se obtiene del LUA "/apps/agent/shared/utils.lua", funcion "GetMaxAllowedRowsInES"
		this.interactions_per_page  = (params.interactions_per_page || INTERACTIONS_PER_PAGE);
		this.interactions_per_page  = this.interactions_per_page  > this.MAX_ALLOWED_ROWS_IN_ES ? this.MAX_ALLOWED_ROWS_IN_ES : this.interactions_per_page;
		this.has_interaction_timer  = params.has_interaction_timer === false ? false : true;
		this.callbacks              = params.callbacks || {};
		this.from_app               = params.from_app;
		this.permissions            = params.permissions || {};
		this.navigation             = params.navigation;
		this.filter_by_campaign     = params.filter_by_campaign || {};
		this.custom_facets          = params.facets;
		this.messages               = params.messages || {};
		this.columns_sort           = params.columns_sort || {};
		// TODO: #9778 - CANY - Docked.
		this.css_classes            = params.css_classes || {};
		this.current_header_sort    = null;
		this.method                 = params.method || 'globalSearch';
		// los estados de las interacciones que se visualizaran
		this.allowed_interaction_states = params.interaction_states || ['ABANDONED', 'FINISHED', 'QUEUED', 'LOCKED', 'TAKEN', 'TRANSFER QUEUED', 'WAITING', 'WRAPUP','CANCELED'];
		// las columnas adicionales a visualizar en las interacciones
		this.additional_columns = params.additional_columns || { interaction_state : { show: true }, user_owner : { show: true } };
		// id de los elementos del DOM
		this.id_filter_container            = this.__get_prefix() + '_filter_container';
		this.id_filter                      = this.__get_prefix() + '_filter_apply';
		this.id_filter_refresh              = this.__get_prefix() + '_filter_refresh';
		this.id_filter_not_found_msg        = this.__get_prefix() + '_not_found_msg';
		this.id_filter_waiting              = this.__get_prefix() + '_waiting';
		this.id_filter_waiting_msg          = this.__get_prefix() + '_waiting_msg';
		this.id_filter_interactions         = this.__get_prefix() + '_interactions';
		// TODO: #9778 - CANY - Docked.
		this.iframes_container_id           = params.iframes_container_id;
		// paginated
		this.id_filter_paginated_container  = this.__get_prefix() + '_paginated_container';
		this.id_filter_paginated_text       = this.__get_prefix() + '_paginated_text';
		this.id_filter_paginated_page_first = this.__get_prefix() + '_paginated_first';
		this.id_filter_paginated_page_prev  = this.__get_prefix() + '_paginated_prev';
		this.id_filter_paginated_page_next  = this.__get_prefix() + '_paginated_next';
		this.id_filter_paginated_page_last  = this.__get_prefix() + '_paginated_last';
		this.id_filter_paginated_disclaimer = this.__get_prefix() + '_paginated_disclaimer';

		this.interaction_timer_hnd = null;
		this.current_request = null;
		this.last_page = 0;
		// filters
		this.current_query_filter = null;
		this.facets_by_name = null;
		// lista de agentes indexados por el nombre completo del agente
		this.agents_by_name  = {};

		this.interaction_gui = null;
		// todos los campos de filtros permitidos.
		this.all_facets = {
			account            : { value: 'account',           label: _T('Account')                },
			address            : { value: 'address',           label: _T('Address')                },
			agent              : { value: 'agent',             label: _T('Agent')                  },
			agent_name         : { value: 'agent_name',        label: _T('Agent Name')             },
			attention_level    : { value: 'attention_level',   label: _T('Attention Level')        },
			campaign           : { value: 'campaign',          label: _T('Campaign')               },
			cci                : { value: 'cci',               label: _T('CCI (field=value)')      },
			contact_id         : { value: 'contactExternalId', label: _T('Contact ID')             },
			contact_name       : { value: 'name',              label: _T('Contact Name')           },
			conversation_state : { value: 'state',             label: _T('Conversation State')     },
			conversation_type  : { value: 'type',              label: _T('Conversation Type')      },
			date_from          : { value: 'date_from',         label: _T('Date From (yyyy-mm-dd)') },
			date_to            : { value: 'date_to',           label: _T('Date To(yyyy-mm-dd)')    },
			disposition        : { value: 'disposition',       label: _T('Disposition Code')       },
			frontend           : { value: 'frontend',          label: _T('FEI (field=value)')      },
			text               : { value: 'text',              label: _T('Text')                   },
			ticket             : { value: 'ticket',            label: _T('Ticket')                 },
			has_attachments    : { value: 'has_attachments',   label: _T('Attachment')             },
			initiative         : { value: 'initiative',        label: _T('Initiative')             }
		};

		this.__initialize();
	}

	if (typeof inConcert.GlobalSearch !== 'function') { inConcert.GlobalSearch = GlobalSearch; }

	GlobalSearch.prototype.__has_permission = function( permission_id )
	{
		if ( typeof permission_id === 'string' ) {
			var has_permission = inConcert.permissions.to_action_on_any_campaign( this.from_app, permission_id );
			if ( !has_permission ) {
				return false;
			}
			return true;
		}
		else if( typeof permission_id === 'function' ) {
			return permission_id();
		}
		// si el que me instancia no ha definido permisos, retorno true
		return true;
	};

	GlobalSearch.prototype.__get_prefix = function()
	{
		return this.container_id || '';
	};

	GlobalSearch.prototype.__initialize = function()
	{
		if (inConcert.app.is_enable_docked_windows_interactions()) {
			// TODO: #9778 - CANY - Docked. Revisar !!
			this.__create_docked_windows_markup();
		}
		else {
			this.__create_markup();
		}
		this.__show_no_conversations_msg();
		this.__initialize_interaction_gui();
		this.__initialize_header_sort();
		this.__set_events();
	};

	GlobalSearch.prototype.__initialize_interaction_gui = function()
	{
		const self = this;

		var interaction_actions = this.from_app === 'supervisor' ? inConcert.supervisor_actions : inConcert.attention_actions;

		var generic_callback_response = function( success, _ /* interaction */, timeout_ms, on_finish_search )
		{
			on_finish_search = on_finish_search || $.noop;
			if ( !success ) {
				// TODO: hacer algo en el error ?
				return;
			}
			// Lo meto en un timeout para darle tiempo al IR, ta feo!!! pero ta x ahora queda asi.
			timeout_ms = timeout_ms || 500; //si no llega un timeout especifico, el por default es de 500 ms
			window.setTimeout(function() {
				var is_new_search = false; // el refresh lo realiza sobre la misma pagina
				self.search(is_new_search, on_finish_search);
			}, timeout_ms);
		};

		var interaction_previous_state = null;
		var can_open_client_window = function( interaction, callback )
		{
			callback = callback || $.noop;
			interaction_previous_state = interaction.state;

			if (interaction.isWaiting()) {
				// Si la interaction se encuentra en estado WAITING, se realiza un request contra el server, para saber si la misma ha cambiado de estado,
				//  si llega a ser asi, la ventana del cliente no se abrira, se mostrara un mensaje de error y se refrescara la busqueda.
				// Esto se realiza asi, contra el server a buscar si cambio el estado, porque la interaccion no se encuentra dentro de las listas de interacciones del agente, y por ende la actualizacion de la misma, no llega en el poller.
				//
				var interactionManager = inConcert.app.getInteractionManager();
				interactionManager.getInteraction(interaction.id, function( interaction_server )
				{
					if (!interaction_server) {
					// la interaccion no existe
						self.__show_banner_message(_T('Unable to open conversation. Conversation not found.'));
						callback(false);
						generic_callback_response(true, interaction);
						return;
					}
					if (!interaction_server.isWaiting()) {
					// la interaccion ha cambiado de estado
						self.__show_banner_message(_T('Unable to open conversation. The conversation is not in waiting.'));
						callback(false);
						generic_callback_response(true, interaction);
						return;
					}
					// Puede abrir la ventana con la interaccion en Waiting.
					callback(true);
				});
				// retorno porque la validacion e invocacion de la funcion de callback, se realiza dentro del callback de .getInteraction
				return;
			}
			// Puede abrir la ventana la interaccion NO se encuentra en Waiting.
			callback(true);
		};

		var max_timeout_ms = 2000;

		this.interaction_gui = new inConcert.interaction_gui( interaction_actions,
			{
				from_app  : this.from_app,
				$container: $('#' + this.id_filter_interactions),
				columns   : this.additional_columns,
				owner     : this,
				callbacks : {
					on_comment: function( success, interaction ) { generic_callback_response(success, interaction); },
					on_finish : function( success, interaction ) { generic_callback_response(success, interaction); },
					on_label  : function( success, interaction ) { generic_callback_response(success, interaction); },
					on_reopen : function( success, interaction ) { generic_callback_response(success, interaction); },
					on_requeue: function( success, interaction ) { generic_callback_response(success, interaction); },
					on_taken  : function( success, interaction ) { generic_callback_response(success, interaction); },
					conference: { 
						on_conference_success: function() { generic_callback_response(true);                        },
						on_conference_error  : function() { generic_callback_response(false);                       }
					},
					schedule  : { 
						on_schedule_success: function(_, on_finish) {
							generic_callback_response(true, null, max_timeout_ms, on_finish);
						},
						on_schedule_error  : function() { generic_callback_response(false);                         }
					},
					transfer  : { 
						on_transfer_success: function() { generic_callback_response(true);                          },
						on_transfer_error  : function() { generic_callback_response(false);                         }
					},
					can_open_client_window: function( interaction, callback ) { can_open_client_window( interaction, callback ); },
					on_client_window_close: function( interaction )
					{
						// si cerro la ventana, y la interaccion cambio de estado "WAITING" a algun otro estado, tengo que refrescar la busqueda.
						if (interaction_previous_state === 'WAITING' && !interaction.isWaiting() ) {
							self.__show_banner_message(_T('Window closed because conversation is not in waiting.'));
							generic_callback_response(true, interaction);
						}
						interaction_previous_state = null;
					}
				}
			}
		);
	};

	GlobalSearch.prototype.__start_interaction_timer = function()
	{
		const self = this;
		if (!this.has_interaction_timer) {
			return false;
		}
		this.__stop_interaction_timer();
		this.interaction_timer_hnd = window.setInterval(function()
		{
			// Update en false, llega si las interacciones no se encuentran visibles, o el container no se encuentra visible.
			// Si llega a ser asi, se realiza un stop del timer. Con esto dejamos de ejecutar codigo innecesario.
			// El mismo se vuelve a encender en una nueva busqueda.
			var updated = self.interaction_gui.update_all_interactions_time();
			if (!updated) {
				self.__stop_interaction_timer();
			}
		}, 1000); // cada 1 segundo
	};

	GlobalSearch.prototype.__stop_interaction_timer = function()
	{
		if ( this.interaction_timer_hnd ) {
			window.clearInterval(this.interaction_timer_hnd);
			this.interaction_timer_hnd = null;
		}
	};

	GlobalSearch.prototype.__create_markup = function()
	{
		const $content = $(''.concat(
			'<div class="boxContentTitleArea querySearchContainer">',
				'<div id="', this.id_filter_container, '" class="boxContentFilterArea boxContentGlobalFilterArea">',
					'<h5>' + this.panel_name_text + '</h5>',
					'<div id="', this.id_filter, '" class="assignedInteractionSearch">',
						'<div id="', this.id_filter_refresh, 'PillContainer" class="card-outlined-internal form-group-pill-card-outlined flex-grow-1">',
							//'<div class="inlineMessagesContainer messageInfo" style="display: none;"><label>', this.from_app === 'supervisor' ? '' : _T('Click to start filtering.'), '</label></div>',
							// '<div class="inlineMessagesContainer messageInfo" style="display: none;"><label>', _T('Click to start filtering.'), '</label></div>',
							'<div class="form-group-pill-container" style=""></div>',
						'</div>	',
					'</div>',
					this.from_app === 'supervisor' ? 						
						'<button query_builder_close_times=true class="butFormSearch buttonColorLightOrange" type="button" title="' + _T('Filter') + '"><i class="fas fa-sliders-h"></i><span>' + _T('Filter') + '</span></button>' : 
						'<button id="'+ this.id_filter_refresh + '" class="butFormSearch buttonColorLightOrange" type="button" title="'+ _T('Filter')+ '" ><i class="fas fa-sliders-h"></i><span>'+ _T('Filter') + '</span></button>',
				'</div>',
			'</div>',
			'<div id="', this.id_filter_not_found_msg, '" class="boxContentListMessages"></div>',
			'<div id="', this.id_filter_waiting, '" class="boxContentListWaiting" hidden>',
				'<div id="', this.id_filter_waiting_msg, '" class="listWaitingMessage"></div>',
				'<div class="listWaitingAnimation"></div>',
				'<div class="listWaitingOverlay"></div>',
			'</div>',
			// header
			'<div header="true" class="boxContenTableHeaderArea">',
				'<div class="boxContenTableHeaderAreaAux">',
					'<div class="interactionListColumn interactionColumnTitle columnBatch"></div>',
					'<div header_column_name="type" class="interactionListColumn interactionColumnTitle columnType">',
						'<div class="interactionColumnTitleText">', _T('Type'), '</div>',
					'</div>',
					'<div header_column_name="state" class="interactionListColumn interactionColumnTitle columnState">',
						'<div class="interactionColumnTitleText">', _T('Status'), '</div>',
					'</div>',
					'<div header_column_name="contact_name" class="interactionListColumn interactionColumnTitle columnName">',
						'<div class="interactionColumnTitleText">', _T('Name'), '</div>',
					'</div>',
					'<div header_column_name="contact_address" class="interactionListColumn interactionColumnTitle columnContactAddress">',
						'<div class="interactionColumnTitleText">', _T('Address'), '</div>',
					'</div>',
					'<div header_column_name="contact_id" class="interactionListColumn interactionColumnTitle columnContactId" hidden>',
						'<div class="interactionColumnTitleText">', _T('Contact ID'), '</div>',
					'</div>',
					'<div header_column_name="campaign" class="interactionListColumn interactionColumnTitle columnCampaign" hidden>',
						'<div class="interactionColumnTitleText">', _T('Campaign'), '</div>',
					'</div>',
					'<div header_column_name="account" class="interactionListColumn interactionColumnTitle columnAccount">',
						'<div class="interactionColumnTitleText">', _T('Account'), '</div>',
					'</div>',
					'<div header_column_name="attention_time" class="interactionListColumn interactionColumnTitle columnAttentionTime" hidden>',
						'<div class="interactionColumnTitleText">', _T('Duration'), '</div>',
					'</div>',
					'<div header_column_name="owner" class="interactionListColumn interactionColumnTitle columnOwner">',
						'<div class="interactionColumnTitleText">', _T('Owner'), '</div>',
					'</div>',
					'<div header_column_name="number" class="interactionListColumn interactionColumnTitle columnNumber">',
						'<i class="interactionColumnTitleIco"></i>',
					'</div>',
					'<div header_column_name="events_unanswered" class="interactionListColumn interactionColumnTitle eventsUnansweredColumn columnNumber">' +
						'<div class="interactionColumnTitleIco"></div>' +
					'</div>' ,
					'<div header_column_name="abstract" class="interactionListColumn interactionColumnTitle columnAbstract">',
						'<div class="interactionColumnTitleText">', _T('Subject'), '</div>',
					'</div>',
					'<div header_column_name="creation_date" class="interactionListColumn interactionColumnTitle columnDate">',
						'<div class="interactionColumnTitleText">', _T('Creation Date'), '</div>',
					'</div>',
					'<div header_column_name="last_event_date_or_state_date" class="interactionListColumn interactionColumnTitle columnDate">',
						'<div class="interactionColumnTitleText">', _T('State Date'), '</div>',
					'</div>',
					'<div header_column_name="comments" class="interactionListColumn interactionColumnTitle columnComments" title="', _T('Comments'), ' ">',
						'<i class="interactionColumnTitleIco"></i>',
					'</div>',
					'<div header_column_name="actions" class="interactionListColumn interactionColumnTitle columnActions">',
						'<i class="interactionColumnTitleIco"></i>',
					'</div>',
				'</div>',
			'</div>',
			'<div id="', this.id_filter_interactions, '" class="boxContentScrollArea GlobalSearchResultsContainer"></div>',
			'<div id="', this.id_filter_paginated_container, '" class="interactionListPager">',
				'<div class="listPager">',
					'<div class="pagerButtonsContainer">',
						'<button id="', this.id_filter_paginated_page_first, '" class="pg-button butPagerFirst" go_to="first"></button>',
						'<button id="', this.id_filter_paginated_page_prev , '" class="pg-button butPagerPrev"  go_to="previous"></button>',
						'<button id="', this.id_filter_paginated_page_next , '" class="pg-button butPagerNext"  go_to="next"></button>',
						'<button id="', this.id_filter_paginated_page_last , '" class="pg-button butPagerLast"  go_to="last"' + (this.last_control? '' : 'style="display:none;"') + '></button>',
					'</div>',
					'<div class="pagerTextsContainer">',
						'<label id="', this.id_filter_paginated_text, '" class="pagerText"></label>',
						'<label id="', this.id_filter_paginated_disclaimer, '" class="pagerDisclaimer">' +
							FORMAT(_T('The Panel allows you to display up to {0} conversations. If you need to view more conversations, you must perform a more specific search.'), this.MAX_ALLOWED_ROWS_IN_ES),
						'</label>',
					'</div>',
				'</div>',
			'</div>'
		));

		this.$container.html($content);
	};

	// TODO: #9778 - CANY - Docked.
	//
	GlobalSearch.prototype.__create_docked_windows_markup = function()
	{
		const self = this;
		const get_html_iframe_container = function () {
			if (!self.iframes_container_id) {
				return '';
			}
			return '<div id="' + self.iframes_container_id + '"></div>';
		};
		const $content = $(''.concat(
			'<div conversations_container_list="true" class="', this.css_classes.container_list || '', '">',
				'<div class="boxContentTitleArea querySearchContainer">',
					'<div id="', this.id_filter_container, '" class="boxContentFilterArea boxContentGlobalFilterArea">',
						'<div id="', this.id_filter, '" class="assignedInteractionSearch">',
							'<div id="', this.id_filter_refresh, 'PillContainer" class="card-outlined-internal form-group-pill-card-outlined flex-grow-1">',
								'<div class="inlineMessagesContainer messageInfo" style="display: none;"><label>', this.from_app === 'supervisor' ? '' : _T('Click to start filtering.'), '</label></div>',
								'<div class="form-group-pill-container" style=""></div>',
							'</div>',
						'</div>',
						this.from_app === 'supervisor' ? '' :
							'<button id="', this.id_filter_refresh, '" class="butFormSearch buttonColorLightOrange" type="button" title="', _T('Filter'), '" ><i class=" fas fa-sliders-h"></i></button>',
					'</div>',
				'</div>',
				'<div id="', this.id_filter_not_found_msg, '" class="boxContentListMessages"></div>',
				'<div id="', this.id_filter_waiting, '" class="boxContentListWaiting" hidden>',
					'<div id="', this.id_filter_waiting_msg, '" class="listWaitingMessage"></div>',
					'<div class="listWaitingAnimation"></div>',
					'<div class="listWaitingOverlay"></div>',
				'</div>',
				// header
				'<div header="true" class="boxContenTableHeaderArea">',
					'<div class="boxContenTableHeaderAreaAux">',
						'<div class="interactionListColumn interactionColumnTitle columnBatch"></div>',
						'<div header_column_name="type" class="interactionListColumn interactionColumnTitle columnType">',
							'<div class="interactionColumnTitleText">', _T('Type'), '</div>',
						'</div>',
						'<div header_column_name="state" class="interactionListColumn interactionColumnTitle columnState">',
							'<div class="interactionColumnTitleText">', _T('Status'), '</div>',
						'</div>',
						'<div header_column_name="contact_name" class="interactionListColumn interactionColumnTitle columnName">',
							'<div class="interactionColumnTitleText">', _T('Name'), '</div>',
						'</div>',
						'<div header_column_name="contact_address" class="interactionListColumn interactionColumnTitle columnContactAddress">',
							'<div class="interactionColumnTitleText">', _T('Address'), '</div>',
						'</div>',
						'<div header_column_name="contact_id" class="interactionListColumn interactionColumnTitle columnContactId" hidden>',
							'<div class="interactionColumnTitleText">', _T('Contact ID'), '</div>',
						'</div>',
						'<div header_column_name="campaign" class="interactionListColumn interactionColumnTitle columnCampaign" hidden>',
							'<div class="interactionColumnTitleText">', _T('Campaign'), '</div>',
						'</div>',
						'<div header_column_name="account" class="interactionListColumn interactionColumnTitle columnAccount">',
							'<div class="interactionColumnTitleText">', _T('Account'), '</div>',
						'</div>',
						'<div header_column_name="attention_time" class="interactionListColumn interactionColumnTitle columnAttentionTime" hidden>',
							'<div class="interactionColumnTitleText">', _T('Duration'), '</div>',
						'</div>',
						'<div header_column_name="owner" class="interactionListColumn interactionColumnTitle columnOwner">',
							'<div class="interactionColumnTitleText">', _T('Owner'), '</div>',
						'</div>',
						'<div header_column_name="number" class="interactionListColumn interactionColumnTitle columnNumber">',
							'<i class="interactionColumnTitleIco"></i>',
						'</div>',
						'<div header_column_name="events_unanswered" class="interactionListColumn interactionColumnTitle eventsUnansweredColumn columnNumber">',
							'<div class="interactionColumnTitleIco"></div>' +
						'</div>' ,
						'<div header_column_name="abstract" class="interactionListColumn interactionColumnTitle columnAbstract">',
							'<div class="interactionColumnTitleText">', _T('Subject'), '</div>',
						'</div>',
						'<div header_column_name="creation_date" class="interactionListColumn interactionColumnTitle columnDate">',
							'<div class="interactionColumnTitleText">', _T('Creation Date'), '</div>',
						'</div>',
						'<div header_column_name="last_event_date_or_state_date" class="interactionListColumn interactionColumnTitle columnDate">',
							'<div class="interactionColumnTitleText">', _T('State Date'), '</div>',
						'</div>',
						'<div header_column_name="comments" class="interactionListColumn interactionColumnTitle columnComments" title="', _T('Comments'), ' " >',
							'<i class="interactionColumnTitleIco"></i>',
						'</div>',
						'<div header_column_name="actions" class="interactionListColumn interactionColumnTitle columnActions">',
							'<i class="interactionColumnTitleIco"></i>',
						'</div>',
					'</div>',
				'</div>',
				'<div id="', this.id_filter_interactions, '" class="boxContentScrollArea GlobalSearchResultsContainer"></div>',
				'<div id="', this.id_filter_paginated_container, '" class="interactionListPager">',
					'<div class="listPager">',
						'<div class="pagerButtonsContainer">',
							'<button id="', this.id_filter_paginated_page_first, '" class="pg-button butPagerFirst" go_to="first"></button>',
							'<button id="', this.id_filter_paginated_page_prev , '" class="pg-button butPagerPrev"  go_to="previous"></button>',
							'<button id="', this.id_filter_paginated_page_next , '" class="pg-button butPagerNext"  go_to="next"></button>',
							'<button id="', this.id_filter_paginated_page_last , '" class="pg-button butPagerLast"  go_to="last"', (this.last_control ? '' : 'style="display:none;"'), '></button>',
						'</div>',
						'<div class="pagerTextsContainer">',
							'<label id="', this.id_filter_paginated_text, '" class="pagerText"></label>',
							'<label id="', this.id_filter_paginated_disclaimer, '" class="pagerDisclaimer">',
								FORMAT(_T('The Panel allows you to display up to {0} conversations. If you need to view more conversations, you must perform a more specific search.'), this.MAX_ALLOWED_ROWS_IN_ES),
							'</label>',
						'</div>',
					'</div>',
				'</div>',
			'</div>',
			get_html_iframe_container()
		));

		this.$container.html($content);
	};

	GlobalSearch.prototype.__initialize_header_sort = function()
	{
		const self = this;

		var $header = this.$container.find('[header="true"]');

		$.each(this.columns_sort || {}, function(header_column_name, info)
		{
			var $column = $header.find('[header_column_name="' + header_column_name + '"]');
			if ($column.length > 0) {
				$column.addClass('columnSorteable');
				if (info.hasOwnProperty('default')) {
					self.__set_column_sort($column, info.order);
				}
			}
		});
	};

	GlobalSearch.prototype.__set_column_sort = function($column, order)
	{
		if (!$column || $column.length === 0) {
			return;
		}
		var header_column_name = $column.attr('header_column_name');
		var column_info = this.columns_sort[header_column_name];
		if (!column_info) {
			return;
		}

		if (!order) {
			if (this.current_header_sort) {
				order = this.current_header_sort.order === 'asc' ? 'desc' : 'asc';
			}
			else {
				order = 'asc'; // el por defecto
			}
		}

		this.current_header_sort = { column_id: column_info.id, header_column_name: header_column_name, order: order };

		this.$container.find('[header="true"] div[header_column_name]').removeClass('columnSortAsc columnSortDesc');

		var class_name = order === 'asc' ? 'columnSortAsc' : 'columnSortDesc';
		$column.addClass(class_name);

		if (this.interaction_gui) {
			this.interaction_gui.set_column_sort(this.current_header_sort);
		}
	};

	GlobalSearch.prototype.__get_column_sort = function()
	{
		if (!this.current_header_sort) {
			return;
		}

		// Este es el formato como lo necesita ElasticSearch, y asi debe ser enviado
		//{ { start_timestamp = { order = "asc" }} };
		//
		var column_order = {};
		column_order[this.current_header_sort.column_id] = { order: this.current_header_sort.order }; // este es valor de la columna que se ha seleccionado

		var sort = [ column_order ];

		return sort;
	};

	GlobalSearch.prototype.toggleSearchPanel = function()
	{
		$('#interactionQueryBuilderContainer').toggle();
	};	

	GlobalSearch.prototype.__set_events = function()
	{
		const self = this;

		var $global_search_queryBuilderCloseButon = this.$container.find("[query_builder_close_times]");
		$global_search_queryBuilderCloseButon.on('click', function() {
			self.$container.parent().find(".interactionQueryBuilderContainer").toggle();
			if(self.filter_by_campaign && self.filter_by_campaign.campaign_id) {			
				$("#sup_dash_campaign_" + nid(self.filter_by_campaign.campaign_id)).toggleClass("showingQueryBuilder");
			}			
		});	

		var $refresh = $('#' + this.id_filter_refresh);

		$refresh
			.off('click')
			.on('click', function() {
				self.toggleSearchPanel();
			});

		var $pillLabel = $('#' + this.id_filter_refresh + 'PillContainer').find('.inlineMessagesContainer');

		$pillLabel
			.off('click')
			.on('click', function() {
				if(self.from_app === "supervisor") {
					self.$container.parent().find(".interactionQueryBuilderContainer").toggle();
					if(self.filter_by_campaign && self.filter_by_campaign.campaign_id) {			
						$("#sup_dash_campaign_" + nid(self.filter_by_campaign.campaign_id)).toggleClass("showingQueryBuilder");
					}					
				}
				else {
					self.toggleSearchPanel();
				}					
			});

		var $header_column = this.$container.find('[header="true"] div[header_column_name]');
		$header_column
			.off('click')
			.on('click', function() {
				var $column = $(this);
				var header_column_name = $(this).attr('header_column_name');
				var column_info = self.columns_sort[header_column_name];
				if (!column_info) {
					return;
				}
				self.__set_column_sort($column);
				var is_new_search = true;
				self.search(is_new_search);
			});

		// eventos del paginado
		this.__set_pagination_events();
	};

	GlobalSearch.prototype.__show_waiting = function ()
	{
		var message = this.messages.waiting || _T('Global searching, please wait');
		$('#' + this.id_filter_waiting_msg).text( message );
		$('#' + this.id_filter_waiting).show();
	};

	GlobalSearch.prototype.__hide_waiting = function ()
	{
		$('#' + this.id_filter_waiting).hide();
	};

	GlobalSearch.prototype.__show_no_conversations_msg = function( message )
	{
		$('#' + this.id_filter_not_found_msg).html( "<label>"+message || _T('No conversations found.')+"</label>").show();
		$('#' + this.id_filter_interactions).hide();
		$('#' + this.id_filter_paginated_container).hide();
	};

	GlobalSearch.prototype.__hide_no_conversations_msg = function()
	{
		$('#' + this.id_filter_not_found_msg).hide();
		$('#' + this.id_filter_interactions).show();
		$('#' + this.id_filter_paginated_container).show();
	};

	GlobalSearch.prototype.__show_banner_message = function(error_message, message_type, timeout_ms, has_log)
	{
		message_type = message_type || 'Error';
		timeout_ms = timeout_ms || 5000;
		has_log = typeof(has_log) === 'undefined' ? true : has_log;

		inConcert.app.getMainWindow().showBannerMessage(
			{
				message:     error_message,
				messageType: message_type,
				timeout:     timeout_ms
			},
			null,
			has_log
		);
	};

	GlobalSearch.prototype.__clear_list = function()
	{
		this.__hide_no_conversations_msg();
		$('#' + this.id_filter_interactions).html('');
	};

	GlobalSearch.prototype.__get_parent_campaign_config = function()
	{
		if ( typeof this.filter_by_campaign === 'object' ) {
			if ( this.filter_by_campaign.type === 'UNIQUE_CAMPAIGN' ) {
				return this.navigation.get_parent_campaign_config(this.filter_by_campaign.campaign_id);
			}
		}
		return {}; // Si no ha definido nada en este atributo debe de retorna un objeto vacio, ya que el parent de parent retorna asi (el supervisor)
	};

	GlobalSearch.prototype.__build_custom_facets = function()
	{
		const self = this;
		var custom_facets = [];
		$.each(this.custom_facets || {}, function( facet_id )
		{
			if (self.all_facets.hasOwnProperty(facet_id)) {
				custom_facets.push(self.all_facets[facet_id]);
			}
			else {
				throw('GlobalSearch.prototype.__build_custom_facets: Custom facet "' + facet_id + '" does not exist');
			}
		});
		return custom_facets;
	};

	GlobalSearch.prototype.__abort_search_request = function( /*filter*/ )
	{
		if ( !this.current_request ) {
			return;
		}
		this.current_request.abort();
		this.current_request = null;
	};

	GlobalSearch.prototype.__search_request = function( filter, on_search_finished )
	{
		on_search_finished = on_search_finished || $.noop;
		let self = this;
		self.__abort_search_request( filter );
		inConcert.app.getInteractionManager()[self.method](filter, function(success, result) {
			self.current_request = null;
			if (success) {
				self.last_page = Math.ceil((result.total || 0) / (self.interactions_per_page || 1));
				on_search_finished(success, result.interactions, result.last, result.total);
			}
			else {
				on_search_finished(success, result.result);
			}
		}, self.from_app);
	};

	GlobalSearch.prototype.search = function(is_new_search, on_search_finished = $.noop)
	{
		const self = this;

		if ( !this.__has_permission(this.permissions.global_search) ) {
			let error_message = _T('Unable to search conversations on global search view: You not have permission for this action.');
			this.__clear_list();
			this.__show_banner_message(error_message);
			on_search_finished();
			return;
		}

		let filter_query = this.__get_query_filter();

		//Pills
		var $pillContainer = $('#' + this.id_filter_refresh + 'PillContainer');
		var $noPill = $pillContainer.find('.inlineMessagesContainer');
		var $pillGroup = $pillContainer.find('.form-group-pill-container');
		$pillGroup.empty();
		if(filter_query && filter_query.rules && $.isArray(filter_query.rules) && filter_query.rules.length > 0)
		{
			for (var i = 0; i < filter_query.rules.length; i++) {
				// #12450: Dont add labels to Pills
				if(filter_query.rules[i].id === 'labels') {
					continue;
				}				
				var ruleId = this.id_filter_refresh + '_pill_' + i;
				var rulePillText = _T(filter_query.rules[i].label || filter_query.rules[i].id);
				if(filter_query.rules[i].id === 'businessdata') {
					for(let j = 0; j < filter_query.rules[i].value.length; j++){
						var businessdataName = filter_query.rules[i].value[j].name;
						var businessdataOperator = filter_query.rules[i].value[j].operator;
						var businessdataType = filter_query.rules[i].value[j].type;
						var businessdataValue = filter_query.rules[i].value[j].value;
						var businessdataValueTo = filter_query.rules[i].value[j].value_to;
						rulePillText += ' | ' + '<strong>' + businessdataName + '</strong>' + ' (' + _T(businessdataType).toLowerCase() + ') | ' + _T(businessdataOperator) + ' | <strong>' + _T(businessdataValue);
						if(businessdataOperator === 'between') {
							rulePillText += ' , ' + businessdataValueTo;
						}
						rulePillText += '</strong>';
					}
				}
				else {
					rulePillText += ' | ' + _T(filter_query.rules[i].operator) + ' | ' + '<strong>';
					rulePillText += filter_query.rules[i].pill ? filter_query.rules[i].pill : _T(filter_query.rules[i].value);
					if(filter_query.rules[i].operator === 'between') {
						rulePillText += ' , ' + filter_query.rules[i].value_to;
					}
					rulePillText += '</strong>';
				}				
				const $lbl = '' +
					'<label id="' + ruleId + '_label" class="form-group-pill">' +
						'<span>' + rulePillText + '</span>' +
						'<button id="' + ruleId + '_remove" class="pillButtonDelete"></button>' +
					'</label>';
				$pillGroup.append($lbl);
				$pillGroup.show();
				$noPill.hide();

				$('#' + ruleId + '_remove').click(function() {
					var ruleIdx = parseInt(this.id.replace(self.id_filter_refresh + '_pill_', ''));
					//Eliminar del queryBuilder
					if(self.queryBuilderObj.deleteRule(ruleIdx))
					{
						//Eliminar del contenedor de pills
						$(this).parent().remove();
					}
				});
			}
		}
		else
		{
			$pillGroup.hide();
			$noPill.show();
		}


		if(!filter_query) {
			on_search_finished();
			return;
		}

		// Supervisor has to filtar by campaign
		if(this.from_app === 'supervisor' && this.filter_by_campaign && this.filter_by_campaign.campaign_id && !filter_query.rules.find(rule => rule.id === 'campaign')) {
			//campaign
			var campaignRule = {
				id : 'campaign',
				operator : 'equal',
				value : this.filter_by_campaign.campaign_id
			};
			filter_query.rules.push(campaignRule);
		}

		const on_end = function() {
			self.__start_interaction_timer();
			self.__hide_waiting();
			on_search_finished();
		};

		// if we don't have any filter, just create the default.
		if ( !this.current_query_filter ){
			this.current_query_filter = {};
			this.current_query_filter.page = 1;
		}

		is_new_search = is_new_search || JSON.stringify({a: filter_query}) !== JSON.stringify({a: this.current_query_filter.query});

		if ( is_new_search ) {
		// reset page number
			this.current_query_filter.page = 1;
		}

		// defaults value set
		let current_page = this.current_query_filter.page || 1;
		let last_by_page = this.current_query_filter.last_by_page || {};

		// last interaction only apply if we are dealing with a page different from the first.
		let last_interaction = current_page > 1 ? ((current_page - 1) * this.interactions_per_page) : '';

		// Si ha llegado al maximo de rows permitidos por ES, me quedo con la pagina previa
		if ((last_interaction + this.interactions_per_page) > this.MAX_ALLOWED_ROWS_IN_ES) {
			current_page = this.MAX_ALLOWED_ROWS_IN_ES / this.interactions_per_page;
			this.current_query_filter.page = current_page;
			last_interaction = (current_page - 1) * this.interactions_per_page;
			if (last_interaction < 0) {
				last_interaction = 0;
			}
		}

		this.__show_waiting();
		this.__clear_list();
		let paginating = last_interaction !== '' ? true : false;

		this.current_query_filter = {
			query        : filter_query,
			last         : last_interaction,
			last_by_page : last_by_page,
			page         : current_page,
			paginating   : paginating,
			rows         : this.interactions_per_page,
			output       : 'interactions',
			sort         : this.__get_column_sort()
		};

		// comienza la busqueda de interacciones
		this.__search_request(this.current_query_filter,
			function( success, interactions, last_interaction, total_interactions )
			{
				if (success) {
				// ok
					self.queryBuilderObj.queryBuilderContainer.parent().parent().find('[query_builder_clean]').prop('disabled', !self.queryBuilderObj.hasRules());
					let current_page = self.current_query_filter.page;
					let interactions_in_page = interactions && interactions.length || 0;
					// update pagination numbers
					self.__update_current_pagination(current_page, interactions_in_page, last_interaction, total_interactions);
					self.current_query_filter.last_by_page[current_page] = last_interaction;
					// Show empty conversations message.
					if ( total_interactions === 0 ){
						self.__show_no_conversations_msg(_T('No conversations found.'));
						// all done
						on_end();
					}
					else{
						// go and process interactions
						self.__load_interactions(interactions, function() {
							// all done
							on_end();
						});
					}
					return; // ok
				}
				// error
				self.__show_banner_message(_T('Unable to search conversations from global search.'));
				on_end();
			}
		);
	};

	GlobalSearch.prototype.__load_interactions = function(interactions, callback = $.noop)
	{
		const self = this;

		this.__clear_list();

		var template_tpl = null;
		var parmams_interaction = {
			prefix: this.__get_prefix(),
			allowed_bulk_actions: false
		};

		var $interactions = $('#' + this.id_filter_interactions);

		$.each( interactions || [], function( _, interaction )
		{
			if (interaction) { // en el array que retorna ES pueden llegar a venir interacciones en "undefined", TODO: revisar por que?
				var $new_interaction = self.interaction_gui.$build_interaction_node( interaction, template_tpl, parmams_interaction );
				self.interaction_gui.update_interaction_data( interaction, $new_interaction );
				$interactions.append( $new_interaction );
				self.interaction_gui.toggleInteractionLabels($new_interaction.find(`[int_labels_disposition]`));
			}
		});

		callback();
	};

	GlobalSearch.prototype.__get_search_facet_values = function( type, callback )
	{
		const self = this;

		callback = callback || $.noop;

		this.mapped_accounts_by_desc = {};
		var campaign_config;

		/*
			Funcion que se encarga de agregar al array (lista de id de agentes) cada agente,
				tambien agrega en la lista ordenada por nombre completo.
		*/
		var add_agent = function(arr, agent)
		{
			var user_name_params = {
				include_user_name: true,
				include_user_last_name: true,
				include_user_first_surname: true,
				include_user_last_surname: true,
				include_user_id: true
			};
			var complete_user_name = inConcert.Utils.formatUserName(agent, user_name_params);
			arr.push(complete_user_name);
			self.agents_by_name[complete_user_name.toLowerCase()] = agent.Id;
		};

		var facets_values = {
			campaign: function(_) {
				var campaign_ids = [];
				var campaigns = inConcert.app.getCampaigns();
				for (var i = 0; i < campaigns.length; i++) {
					var campaign_id = campaigns[i].Id.match('([^@]+)@(.+)')[1];
					campaign_ids.push(campaign_id);
				}
				callback(campaign_ids);
			},
			account: function(_)
			{
				var accounts_ids = [];
				var campaigns = [];
				campaign_config = self.__get_parent_campaign_config();

				if ( campaign_config.campaign_id ) {
					campaigns.push( inConcert.app.getCampaign(campaign_config.campaign_id) );
				}
				else {
					campaigns = inConcert.app.getCampaigns();
				}

				var iter_fn = function( _, account) {
					var account_desc = campaign.getAccountDescription( account );
					account_desc = campaign_id + ' ' + account_desc + ' (' + inConcert.app.getTranslatedChannel(account.proxyType) + ')';
					accounts_ids.push(account_desc);
					self.mapped_accounts_by_desc[account_desc] = account.id;
				};

				for (var i = 0; i < campaigns.length; i++)
				{
					var campaign = campaigns[i];
					var campaign_id = campaign.Id.match('([^@]+)@(.+)')[1];
					$.each(campaign.accounts || {}, iter_fn);
				}

				callback(accounts_ids);
			},
			agent : function()
			{
				campaign_config = self.__get_parent_campaign_config();

				if ( !campaign_config.campaign_id ) {
					var contactHandler = function(members)
					{
						var agents = [];
						if ( !members ) {
							callback(agents);
							return;
						}
						for (var i = 0; i < members.length; i++) {
							var agent = members[i];
							agents.push(agent.Id);
						}
						agents.push(inConcert.app.getLoggedUserId());
						callback(agents);
					};
					inConcert.app.getTeamMembers(contactHandler, true /*from cache*/);
					return;
				}

				var campaign = inConcert.app.getCampaign(campaign_config.campaign_id);
				if ( campaign )
				{
					campaign_config = self.__get_parent_campaign_config();
					if ( campaign_config.account_id && campaign_config.al_id ) {
					// 1 - si en el selector de campanas ha seleccionado algun AL debe de visualizar los users de ese AL
						if ( campaign.has_attention_levels_configured_in_account(campaign_config.account_id) ) {
							var al_users = campaign.getUsersInAttentionLevel(campaign_config.al_id);
							var users_ids = [];
							$.each(al_users, function( user_id ) {
								users_ids.push(user_id);
							});
							// Push current member Id.
							users_ids.push(inConcert.app.getLoggedUserId());
							callback(users_ids);
							return;
						}
					}

					// 2 - si en el selector de campanas no ha seleccionado "attention levels", trae todos los members de la campana
					campaign.getTeamMembers(function( members )
						{
							if (members.length === 0){
								callback([]);
							}
							// Extract member's ids.
							var members_ids = _.pluck(members, 'Id');
							// // Push current member Id.
							// members_ids.push(inConcert.app.getLoggedUserId());
							callback(members_ids);
						},
						true // from cache
					);
				}
			},

			agent_name : function()
			{
				campaign_config = self.__get_parent_campaign_config();

				if ( !campaign_config.campaign_id ) {
					var contactHandler = function(members)
					{
						var agents_names = [];
						if ( !members ) {
							callback(agents_names);
							return;
						}

						for (var i = 0; i < members.length; i++) {
							var agent = members[i];
							add_agent(agents_names, agent);
						}

						// Push logged User.
						var logged_user_info = inConcert.app.getLoggedUserInfo();
						add_agent(agents_names, logged_user_info);

						var sorted_agents = inConcert.Utils.sortArray(agents_names);
						callback(sorted_agents, {preserveOrder: true});
					};
					inConcert.app.getTeamMembers(contactHandler, true /*from cache*/);
					return;
				}

				var campaign = inConcert.app.getCampaign(campaign_config.campaign_id);
				if ( campaign )
				{
					campaign_config = self.__get_parent_campaign_config();
					if ( campaign_config.account_id && campaign_config.al_id ) {
					// 1 - si en el selector de campanas ha seleccionado algun AL debe de visualizar los users de ese AL
						if ( campaign.has_attention_levels_configured_in_account(campaign_config.account_id) ) {

							campaign.getTeamMembers(function( members )
								{
									if (!members || members.length === 0){
										callback([]);
										return;
									}

									var agents_by_id = {};
									for (var i = 0; i < members.length; i++) {
										var agent = members[i];
										add_agent([], agent);
										agents_by_id[agent.Id] = agent;
									}

									var al_users = campaign.getUsersInAttentionLevel(campaign_config.al_id);
									var agents_names = [];
									$.each(al_users, function( user_id ) {
										var agent = agents_by_id[user_id];
										add_agent(agents_names, agent);
									});

									// Push current member Id.
									var logged_user_info = inConcert.app.getLoggedUserInfo();
									add_agent(agents_names, logged_user_info);

									var sorted_agents = inConcert.Utils.sortArray(agents_names);
									callback(sorted_agents, {preserveOrder: true});
								},
								true // from cache
							);

							return;
						}
					}

					// 2 - si en el selector de campanas no ha seleccionado "attention levels", trae todos los members de la campana
					campaign.getTeamMembers(function( members )
						{
							var agents_names = [];
							if (!members || members.length === 0){
								callback(agents_names);
								return;
							}

							for (var i = 0; i < members.length; i++) {
								var agent = members[i];
								add_agent(agents_names, agent);
							}

							// Push current member Id.
							var logged_user_info = inConcert.app.getLoggedUserInfo();
							add_agent(agents_names, logged_user_info);

							var sorted_agents = inConcert.Utils.sortArray(agents_names);
							callback(sorted_agents, {preserveOrder: true});
						},
						true // from cache
					);
				}
			},

			attention_level : function()
			{
				var displayed_attention_levels = [];
				campaign_config = self.__get_parent_campaign_config();

				if ( campaign_config.account_id && !campaign_config.al_id ) {
					var campaign = inConcert.app.getCampaign(campaign_config.campaign_id);

					if ( campaign && campaign.has_attention_levels_configured_in_account(campaign_config.account_id)  ) {
						var account = campaign.getAccount(campaign_config.account_id);

						if ( account ) {
							$.each( account.AttentionLevels || {}, function(_, assigned_al) {
								var current_al = campaign.getAttentionLevel(assigned_al.attention_level_id);
								if ( current_al ) {
									displayed_attention_levels.push(current_al.name);
								}
							});
						}
					}
				}

				callback( displayed_attention_levels );
			},

			type: function() {
				var displayed_channels = [];
				$.each( ['CALL', 'CHAT', 'FACEBOOK', 'INSTAGRAM', 'YOUTUBE', 'APPSTORE', 'GOOGLEPLAY', 'RCS', 'IM', 'MAIL', 'SMS', 'WHATSAPP', 'TWITTER', 'WEBCONTACT', 'VOICEMAIL'],
					function( _, channel ) {
						if ( inConcert.app.hasLicensesForChannel( channel ) ) {
							displayed_channels.push( channel );
						}
					});
				callback( displayed_channels );
			},

			state: function() {
				var displayed_states = self.allowed_interaction_states;
				callback( displayed_states );
			},

		// TODO: esto va ?
			// workgroup: function() {
				// callback( self.m_workgroups.get_campaign_workgroups(campaign_id) );
			// }
		};

		if ( facets_values[type] ) {
			facets_values[type]();
			return;
		}

		callback();
	};

	GlobalSearch.prototype.__get_navigation = function ()
	{
		return this.navigation;
	};

	GlobalSearch.prototype.__get_filter_facets = function()
	{
		var facets = [];
		if (this.current_search) {
			facets = this.current_search.searchQuery.facets();
		}
		return facets;
	};

	GlobalSearch.prototype.__get_face_by_name = function(name)
	{
		if (!this.facets_by_name) {
			return '';
		}
		return this.facets_by_name[name.toLowerCase()];
	};

	GlobalSearch.prototype.__get_query_filter = function()
	{
		var result = this.queryBuilderObj.getFilter();
		if(result) {
			//labels
			var navigation = this.__get_navigation();
			if ( navigation ) {
				var selected_labels = navigation.get_selected_labels( true /*simple item*/ );
				if ( $.isArray(selected_labels) && selected_labels.length > 0 ) {
					var labelsRule = {
						id : 'labels',
						operator : 'in',
						value : selected_labels
					};
					result.rules.push(labelsRule);
				}
			}
		}

		return result;
	};

	// PAGINATION -------------------
	//
	GlobalSearch.prototype.__set_pagination_events = function( search_fn )
	{
		search_fn = search_fn || $.noop;
		const self = this;

		var page_buttons = [
			{ selector: '#' + this.id_filter_paginated_page_first , tooltip: _T('First Page')    },
			{ selector: '#' + this.id_filter_paginated_page_next  , tooltip: _T('Next Page')     },
			{ selector: '#' + this.id_filter_paginated_page_prev  , tooltip: _T('Previous Page') },
			{ selector: '#' + this.id_filter_paginated_page_last  , tooltip: _T('Last Page')     }
		];

		page_buttons.forEach(function(button)
		{
			var $button_node = $( button.selector );

			$button_node
				.unbind('click_navigate_to')
				.bind('click_navigate_to', function()
				{
					var $button = $(this);

					if ( self.current_query_filter )
					{
						var go_to = $button.attr('go_to') || '';

						switch(go_to)
						{
							case 'first':
								self.current_query_filter.page = 1;
							break;

							case 'previous':
								self.current_query_filter.page--;
							break;

							case 'next':
								self.current_query_filter.page++;
							break;

							case 'last':
								self.current_query_filter.page = self.last_page;
							break;

							default:
								throw('GlobalSearch.prototype.__set_pagination_events: Unrecognized pagination button: "' + go_to + '"');
						}
					}
					// ejecuto la busqueda de los datos de la pagina seleccionada
					var is_new_search = false;
					self.search(is_new_search);
				});

			$button_node
				.attr('tooltip', button.tooltip)
				.unbind('click')
				.bind('click', function(){
					$(this).trigger('click_navigate_to');
				});
		});
	};

	GlobalSearch.prototype.__update_current_pagination = function(current_page, interactions_in_page, last_interaction, total_interactions)
	{
		let last_page = current_page === 0 ? 0 : current_page -1 ;

		// calculate start number.
		let start = null;
		if (interactions_in_page === 0) {
			start = 0;
		}
		else if (current_page === 1) {
			start = 1;
		}
		else {
			start = ( this.interactions_per_page * last_page ) + 1;
		}

		// calculate finish number
		let finish = null;
		if (this.interactions_per_page > interactions_in_page){
			// handle special (0,1,2) interaction in total.
			if (start <= 1 && interactions_in_page <= 2){
				finish = interactions_in_page;
			}
			// otherwise, just sum up start and interactions in page.
			else {
				finish = (start + interactions_in_page) - 1;
			}
		}
		else {
			finish = this.interactions_per_page * current_page;
		}

		total_interactions = total_interactions || last_interaction;
		let disable_more_pages = false;
		if ((last_interaction + this.interactions_per_page) > this.MAX_ALLOWED_ROWS_IN_ES) {
			// Si llego al maximo de Rows permitidas desactivo la navegacion para obtener mas Rows.
			disable_more_pages = true;
		}
		else {
			disable_more_pages = last_interaction === total_interactions;
		}

		// update navigation page
		$('#' + this.id_filter_paginated_text).text(FORMAT(_T('{0} - {1} of {2} conversations'), start, finish, total_interactions));
		$('#' + this.id_filter_paginated_page_first).prop('disabled', current_page === 1);
		$('#' + this.id_filter_paginated_page_prev).prop('disabled', current_page === 1);
		$('#' + this.id_filter_paginated_page_next).prop('disabled', disable_more_pages);
		$('#' + this.id_filter_paginated_page_last).prop('disabled', disable_more_pages);
	};
}());