/* jshint forin:true, noarg:true, multistr:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
  strict:true, undef:true, curly:true, browser:true, indent:false, maxerr:50, -W083, esversion: 11 */

/* global inConcert, $ */

// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.
if (!this.inConcert) {
	this.inConcert = {};
}

/**
 * @namespace
 * @name QueryBuilder
 */
(function () {
//	"use strict";

	const _T = inConcert.i18n.getString;
	const format = inConcert.Utils.format;

	const filteringEntities = {
		campaign: _T('Account') + ", " + _T('Type') + ", " + _T('Attention Level') + ", " + _T('Agent') + ", " + _T('Agent Name'),
		account: _T('Type') + ", " + _T('Attention Level')
	};

	const rulesAndOrMessages = {
		AND: _T("Result of the conversations that comply with all the rules indicated."),
		OR: _T("Result of the conversations that comply with at least one of the indicated rules.")
	};

	function inconcert_querybuilder (params)
	{
		const self = this;

		this.queryBuilderContainer = params.container;
		this.fromApp = params.fromApp;
		this.supervisorCampaignId = params.supervisorCampaignId;
		this.onReadyToRefresh = params.onReadyToRefresh || $.noop;
		this.onRuleAdded = params.onRuleAdded || $.noop;
		this.onRuleDeleted = params.onRuleDeleted || $.noop;
		this.queryBuilderInitialConfig = {
				condition: 'AND',
				operators: ['equal', 'not_equal', 'contains', 'not_contains', 'less', 'less_or_equal', 'greater', 'greater_or_equal', 'between',
					{ type: 'similar', nb_inputs: 1, multiple: true, apply_to: ['string'] }
				],
				rules: [],
				filters: [],
				allow_groups: false,
				display_errors: true,
				allow_empty: true,
				lang_code: inConcert.app.m_userLanguage,
				select_placeholder: _T("Select an option")
			};

		this.queryBuilderFilters = [
		{
			id: 'campaign',
			label: _T('Campaign'),
			type: 'string',
			operators: ['equal', 'not_equal', 'contains', 'not_contains'],
			input: function(rule, name) {
				return ' \
					<select class="form-control" name="'+ name +'_multiple"> \
					</select> \
					<input type="text" class="form-control" name="'+ name +'_single" style="display:none;"></input>';
			},
		    valueGetter: function(rule) {
		    	var getOption = ['equal', 'not_equal'].indexOf(rule.operator.type) !== -1;
		    	return getOption ? rule.$el.find('.rule-value-container [name$=_multiple]').val() : rule.$el.find('.rule-value-container [name$=_single]').val();
		    },
		    valueSetter: function(rule, value) {}
		}, {
			id: 'account',
			label: _T('Account'),
			type: 'string',
			operators: ['equal', 'not_equal', 'contains', 'not_contains'],
			input: function(rule, name) {
				return ' \
					<select class="form-control" name="'+ name +'_multiple"> \
					</select> \
					<input type="text" class="form-control" name="'+ name +'_single" style="display:none;"></input>';
			},
		    valueGetter: function(rule) {
		    	var getOption = ['equal', 'not_equal'].indexOf(rule.operator.type) !== -1;
		    	return getOption ? rule.$el.find('.rule-value-container [name$=_multiple]').val() : rule.$el.find('.rule-value-container [name$=_single]').val();
		    },
		    valueSetter: function(rule, value) {}
		}, {
			id: 'agent',
			label: _T('Agent'),
			type: 'string',
			operators: ['equal', 'not_equal', 'contains', 'not_contains'],
			input: function(rule, name) {
				return '\
					<select class="form-control" name="'+ name +'_multiple"> \
					</select> \
					<input type="text" class="form-control" name="'+ name +'_single" style="display:none;"></input>';
			},
		    valueGetter: function(rule) {
		    	var getOption = ['equal', 'not_equal'].indexOf(rule.operator.type) !== -1;
		    	return getOption ? rule.$el.find('.rule-value-container [name$=_multiple]').val() : rule.$el.find('.rule-value-container [name$=_single]').val();
		    },
		    valueSetter: function(rule, value) {}
		}, {
			id: 'agentname',
			label: _T('Agent Name'),
			type: 'string',
			operators: ['equal', 'not_equal'],
			input: function(rule, name) {
				return '\
					<select class="form-control" name="'+ name +'_multiple"> \
					</select> \
					<input type="text" class="form-control" name="'+ name +'_single" style="display:none;"></input>';
			},
		    valueGetter: function(rule) {
		    	var getOption = ['equal', 'not_equal'].indexOf(rule.operator.type) !== -1;
		    	return getOption ? rule.$el.find('.rule-value-container [name$=_multiple]').val() : rule.$el.find('.rule-value-container [name$=_single]').val();
		    },
		    valueSetter: function(rule, value) {}
		}, {
			id: 'businessdata',
			label: _T('Business data'),
			type: 'string',
			operators: ['equal', 'not_equal'],	//No usamos este. Usamos un fieldOperator interno, pero es necesario declarar dos operators aqui para evitar problemas
			input: function(rule, name) {
				return '\
					<select class="form-control" name="'+ name +'_fieldType"> \
						<option value="boolean">' + _T('Boolean') + '</option>\
						<option value="date">' + _T('Date') + '</option>\
						<option value="numeric">' + _T('Numeric') + '</option>\
						<option value="text">' + _T('Text') + '</option>\
					</select> \
					<input type="text" class="form-control" name="'+ name +'_fieldName" placeholder="' + _T('Field name') + '"></input> \
					<select class="form-control" name="'+ name +'_fieldOperator"> \
					</select> \
					<input type="text" class="form-control" name="'+ name +'_fieldValue" placeholder="' + _T('Field value') + '"></input> \
					<select class="form-control" name="'+ name +'_fieldValueBoolean"> \
						<option value="true">' + _T('True') + '</option>\
						<option value="false">' + _T('False') + '</option>\
					</select> \
					<input type="text" class="form-control" name="'+ name +'_fieldValueTo"  placeholder="' + _T('To field value') + '" style="display:none;"></input>';
			},
		    valueGetter: function(rule) {
		    	if(rule.filter && rule.filter.id === "businessdata"){
		    		var businessdataValue = {
		    			fieldType : rule.$el.find('.rule-value-container [name$=_fieldType]').val(),
		    			fieldName : rule.$el.find('.rule-value-container [name$=_fieldName]').val(),
		    			fieldOperator : rule.$el.find('.rule-value-container [name$=_fieldOperator]').val()
		    		};
		    		if(businessdataValue.fieldType === "boolean") {
		    			businessdataValue.fieldValue = rule.$el.find('.rule-value-container [name$=_fieldValueBoolean]').val() === "true";
		    		}
		    		else { 
		    			businessdataValue.fieldValue = rule.$el.find('.rule-value-container [name$=_fieldValue]').val();
			    		if(businessdataValue.fieldOperator === "between") {
			    			businessdataValue.fieldValueTo = rule.$el.find('.rule-value-container [name$=_fieldValueTo]').val();
			    		}						
		    		}

		    		return businessdataValue;
		    	}
		    },
		    valueSetter: function(rule, value) {
		    	rule.$el.find('.rule-value-container [name$=_fieldType]').val(value.fieldType);
		    	rule.$el.find('.rule-value-container [name$=_fieldName]').val(value.fieldName);
		    	rule.$el.find('.rule-value-container [name$=_fieldOperator]').val(value.fieldOperator);
	    		if(value.fieldType === "boolean") {
	    			rule.$el.find('.rule-value-container [name$=_fieldValueBoolean]').val(value.fieldValue);
	    		}
	    		else {
	    			rule.$el.find('.rule-value-container [name$=_fieldValue]').val(value.fieldValue);
	    		}		    	
		    	rule.$el.find('.rule-value-container [name$=_fieldValueTo]').val(value.fieldValueTo);
		    },
		    validation: {
		    	callback: function(value, rule) {
					if(!value.fieldType || value.fieldType === "") {
						return "Business Data Type is required";
					}
					if(!value.fieldName || value.fieldName === "") {
						return "Business Data Name is required";						
					}
					if(!value.fieldOperator || value.fieldOperator === "") {
						return "Business Data Operator is required";	
					}
					if(value.fieldValue === undefined || value.fieldValue === "") {
						return "Business Data Value is required";	
					}
					if(value.fieldOperator === "between" && (!value.fieldValueTo || value.fieldValueTo === "")) {
						return "Business Data ValueTo is required";	
					}
					if(value.fieldType === "numeric" && isNaN(Number(value.fieldValue))) {
						return "Business Data Value must be numeric";	
					}
					if(value.fieldType === "numeric" && value.fieldOperator === "between" && isNaN(Number(value.fieldValueTo))) {
						return "Business Data ValueTo must be numeric";
					}
					if(value.fieldOperator === "between" && value.fieldValueTo < value.fieldValue) {
						return "Business Data ValueTo must be greater or equal than Value";
					}
					return true;
		    	}
		    }
		}, {
			id: 'disposition_code',
			label: _T('Disposition Code'),
			type: 'string',
			operators: ['equal', 'not_equal', 'contains', 'not_contains']
		}, {
			id: 'contact_address',
			label: _T('Contact Address'),
			type: 'string',
			operators: ['equal', 'not_equal', 'contains', 'not_contains']
		}, {
			id: 'state',
			label: _T('State'),
			type: 'string',
			input: 'select',
			operators: ['equal', 'not_equal'],
			values: {
				ABANDONED: _T('Abandoned State'),
				FINISHED: _T('Ended'),
				QUEUED: _T('Queued'),
				LOCKED: _T('Ringing'),
				TAKEN: _T('Active'),
				'TRANSFER QUEUED': _T('Transfer queued'),
				WAITING: _T('Waiting'),
				WRAPUP: _T('Wrapping Up'),
				CANCELED: _T('Canceled')
			}
		}, {
			id: 'start_date',
			label: _T('Creation Date'),
			type: 'date',
			operators: ['equal', 'not_equal', 'less', 'less_or_equal', 'greater', 'greater_or_equal', 'between'],
			validation: {
			  format: 'YY-MM-DD'
			},
			plugin: 'datepicker',
			plugin_config: {
			  dateFormat: 'yy-mm-dd'
			}
		}, {
			id: 'contact',
			label: _T('Contact ID'),
			type: 'string',
			operators: ['equal', 'not_equal']
		}, {
			id: 'attention_level',
			label: _T('Attention Level'),
			type: 'string',
			operators: ['equal', 'not_equal'],
			input: function(rule, name) {
				return ' \
					<select class="form-control" name="'+ name +'_multiple"> \
					</select>';
			},
		    valueGetter: function(rule) {
		    	return rule.$el.find('.rule-value-container [name$=_multiple]').val();
		    },
		    valueSetter: function(rule, value) {}			
		}, {
			id: 'contact_name',
			label: _T('Contact Name'),
			type: 'string',
			operators: ['contains', 'not_contains', 'similar']
		}, {
			id: 'content',
			label: _T('Text'),
			type: 'string',
			operators: ['contains', 'not_contains', 'similar']
		}, {
			id: 'ticket',
			label: _T('Ticket'),
			type: 'string',
			operators: ['equal', 'not_equal', 'contains', 'not_contains']
		}, {
			id: 'type',
			label: _T('Conversation Type'),
			type: 'string',
			operators: ['equal', 'not_equal'],
			input: function(rule, name) {
				return ' \
					<select class="form-control" name="'+ name +'_multiple"> \
					</select>';
			},
		    valueGetter: function(rule) {
		    	return rule.$el.find('.rule-value-container [name$=_multiple]').val();
		    },
		    valueSetter: function(rule, value) {}
		}, {
			id: 'events_unanswered',
			label: _T('Unanswered Messages'),
			type: 'string',
			input: 'select',
			operators: ['equal'],
			values: {
				true: _T('True')
			}
		}, {
			id: 'events_unread',
			label: _T('Unread Messages'),
			type: 'string',
			input: 'select',
			operators: ['equal'],
			values: {
				true: _T('True')
			}
		}, {
			id: 'has_attachments',
			label: _T('Attachment'),
			type: 'string',
			input: 'select',
			operators: ['equal'],
			values: {
				true: _T('Yes'),
				false: _T('No')
			}
		}, {
			id: 'initiative',
			label: _T('Initiative'),
			type: 'string',
			input: 'select',
			operators: ['equal'],
			values: {
				OUTBOUND: _T('OUTBOUND'),
				INBOUND: _T('INBOUND')
			}
		}, {
			id: 'has_draft',
			label: _T('Draft'),
			type: 'string',
			input: 'select',
			operators: ['equal'],
			values: {
				true: _T('True')
			}
		}, {
			id: 'id' /*interaction_id*/,
			label: _T('Interaction ID'),
			type: 'string',
			operators: ['equal'],
		}, {
			id: 'need_help',
			label: _T('Need Help'),
			type: 'string',
			input: 'select',
			operators: ['equal'],
			values: {
				true: _T('True')
			}
		}];

		var initPanelQueryBuilderFilters = function(panel, filters) {
			for (let i = 0; i < filters.length; i++) {
				let filter = self.queryBuilderFilters.find(filter => filter.id === filters[i]);
				if(filter){
					panel.filters.push(filter);
				}
			}
			panel.filters.sort((a,b) => (a.label > b.label) ? 1 : ((b.label > a.label) ? -1 : 0));
		};

		this.queryBuilderConfig = {};
		if(params.panels && $.isArray(params.panels)) {
			for (let i = 0; i < params.panels.length; i++) {
				this.queryBuilderConfig[params.panels[i].panelId] = JSON.parse(JSON.stringify(this.queryBuilderInitialConfig));
				initPanelQueryBuilderFilters(this.queryBuilderConfig[params.panels[i].panelId], params.panels[i].filters);
			}
		}
	}

	if (typeof inConcert.inconcert_querybuilder !== 'function') {
		inConcert.inconcert_querybuilder = inconcert_querybuilder;
	}

	var isValidDate = function(dateString) {
		if(!dateString) {
			return false;
		}
		var regEx = /^\d{4}-\d{2}-\d{2}$/;
		if(!dateString.match(regEx)) { return false; }  // Invalid format
		var d = new Date(dateString);
		var dNum = d.getTime();
		if(!dNum && dNum !== 0) { return false; } // NaN value, Invalid date
		return d.toISOString().slice(0,10) === dateString;
	};

	// Add filter to rules selectbox filter, except for a specific rule
	var AddFilterToRules = function(filter, rules, exceptRuleId) {
		if(filter) {
			rules.filter(otherRule => otherRule.id !== exceptRuleId).forEach((otherRule, index, parentRules) => {
				//Get otherRule selectbox current options
				var options = [];
				otherRule.$el.find('.rule-filter-container select').find("option").map((index, option) => {
					options.push({value:option.value, text:option.text});
				});
				//Add deleted filter option
				if(options.filter(option => option.value === filter.id).length === 0)
				{
					options.push({value:filter.id, text:filter.label});
				}
				//Sort options by text
				options.sort((a,b) => (a.text > b.text) ? 1 : ((b.text > a.text) ? -1 : 0));
				//Empty otherRule selectbox
				otherRule.$el.find('.rule-filter-container select').empty();

				//Fill otherRule selectbox
				options.forEach((option, index, options) => {
					otherRule.$el.find('.rule-filter-container select').append($('<option>').val(option.value).text(option.text));
				});
				//Select otherRule selectbox current option 
				otherRule.$el.find('.rule-filter-container select').val(otherRule.filter && otherRule.filter.id);
			});	
		}
	};

	var prefilteredCampaigns = function(rule, campaigns){
		if(rule.parent.condition === "AND") {
			for(let i=0; i < rule.parent.rules.length; i++) {
				if(rule.parent.rules[i].filter.id === "campaign" && rule.parent.rules[i].value  && !rule.parent.rules[i].beingDeleted){
					var value_lower = rule.parent.rules[i].value.toLowerCase();
					var resultCampaigns = [];
					switch(rule.parent.rules[i].operator.type)
					{
						case "equal": //Remove all campaigns except rule value
							var configuredCampaign = campaigns.find(campaign => {
								var campaign_id_lower = campaign.Id.match('([^@]+)@(.+)')[1].toLowerCase();
								return campaign_id_lower === value_lower;
							});
							if(configuredCampaign){
								resultCampaigns.push(configuredCampaign);
							}
							return resultCampaigns;

						case "not_equal": 	//Remove campaign rule value
							resultCampaigns = campaigns.filter(campaign => {
								var campaign_id_lower = campaign.Id.match('([^@]+)@(.+)')[1].toLowerCase();
								return campaign_id_lower !== value_lower;
							});
							return resultCampaigns;

						case "contains": //Remove campaigns containing rule value
							resultCampaigns = campaigns.filter(campaign => {
								var campaign_id_lower = campaign.Id.match('([^@]+)@(.+)')[1].toLowerCase();
								return campaign_id_lower.includes(value_lower);
							});
							return resultCampaigns;

						case "not_contains": //Remove campaigns not containing rule value
							resultCampaigns = campaigns.filter(campaign => {
								var campaign_id_lower = campaign.Id.match('([^@]+)@(.+)')[1].toLowerCase();
								return !campaign_id_lower.includes(value_lower);
							});
							return resultCampaigns;
					}
				}
			}
		}
		return campaigns;
	};

	var prefilteredAccounts = function(rule, accounts){
		if(rule.parent.condition === "AND") {
			for(let i=0; i < rule.parent.rules.length; i++) {
				if(rule.parent.rules[i].filter.id === "account"  && rule.parent.rules[i].value  && !rule.parent.rules[i].beingDeleted){
					var value_lower = rule.parent.rules[i].value.toLowerCase();
					var resultAccounts = [];
					switch(rule.parent.rules[i].operator.type)
					{
						case "equal": //Remove all accounts except rule value
							var configuredAccount = accounts.find(account => {
								var account_id_lower = account.Id.toLowerCase();
								return account_id_lower === value_lower;
							});
							if(configuredAccount){
								resultAccounts.push(configuredAccount);
							}
							return resultAccounts;

						case "not_equal": 	//Remove account rule value
							resultAccounts = accounts.filter(account => {
								var account_id_lower = account.Id.toLowerCase();
								return account_id_lower !== value_lower;
							});
							return resultAccounts;

						case "contains": //Remove accounts containing rule value
							resultAccounts = accounts.filter(account => {
								var account_id_lower = account.Id.toLowerCase();
								return account_id_lower.includes(value_lower);
							});
							return resultAccounts;

						case "not_contains": //Remove accounts not containing rule value
							resultAccounts = accounts.filter(account => {
								var account_id_lower = account.Id.toLowerCase();
								return !account_id_lower.includes(value_lower);
							});
							return resultAccounts;
					}
				}
			}	
		}
		return accounts;
	};	

	inconcert_querybuilder.prototype.setup = $.noop;

	inconcert_querybuilder.prototype.fillAccounts = function(rule) {
		var self = this;
		var accounts = [];
		var campaigns = (this.fromApp === "supervisor" && this.supervisorCampaignId) ? [ inConcert.app.getCampaign(this.supervisorCampaignId) ] : prefilteredCampaigns(rule, inConcert.app.getCampaigns());
		
		var iter_fn = function( _, account) {
			var account_desc = campaign.getAccountDescription( account );
			let translatedChannel = inConcert.app.getTranslatedChannel(account.proxyType);
			account_desc = campaign_id + ' ' + account_desc + ' (' + (translatedChannel) + ')';
			accounts.push({Id:account.id, Name:account_desc});
		};

		for (let i = 0; i < campaigns.length; i++)
		{
			var campaign = campaigns[i];
			var campaign_id = campaign.Id.match('([^@]+)@(.+)')[1];
			$.each(campaign.accounts || {}, iter_fn);
		}

		var $accountList = rule.$el.find('.rule-value-container [name$=_multiple]');
		$accountList.empty();			
		
		var $iccRuleError = rule.$el.find('.error-container label');
		if(!accounts.length) {
			var textError = _T("No accounts available");
			rule.error = textError;
			rule.$el.find('.error-container')[0].title = textError;
			$iccRuleError.text(textError);
			$iccRuleError.show();
			return;
		}
		else {
			rule.error = null;
			$iccRuleError.text("");
			$iccRuleError.hide();
		}

		accounts.sort((a,b) => (a.Name > b.Name) ? 1 : ((b.Name > a.Name) ? -1 : 0));

		for (let j = 0; j < accounts.length; j++) {
			$accountList.append($('<option>', { 
		        value: accounts[j].Id,
		        text : accounts[j].Name
		    }));
		}

		//Si el value no nos viene dado en la rule (bug de queryBuilder?), vemos si lo tenemos en nuestro queryBuilderConfig
		if(!rule.value) {
			var ruleIdx = parseInt(rule.id.replace(self.queryBuilderContainer.prop("id") + "_rule_", ""));
			if(!isNaN(ruleIdx) && self.queryBuilderConfig[self.queryBuilderCurrentPanel].rules && self.queryBuilderConfig[self.queryBuilderCurrentPanel].rules[ruleIdx]) {
				rule.value = self.queryBuilderConfig[self.queryBuilderCurrentPanel].rules[ruleIdx].value;
			}
		}

		if(rule.value) {
			//If rule value still exists on the list, select it
			var ruleValueExists = false;
			var options = $accountList.find("option");
			for(let i=0; i < options.length; i++) {
				if(options[i].value === rule.value){
					ruleValueExists = true;
					break;
				}
			}			
			if(ruleValueExists){
				$accountList.val(rule.value);
			}
			else {	//rule value is changed to the first element on the list
				rule.value = $accountList.val();
			}
		}
		else {
			rule.value = $accountList.val();
		}
	};

	inconcert_querybuilder.prototype.fillTypes = function(rule) {
		var self = this;
		var accounts = [];
		var campaigns = prefilteredCampaigns(rule, inConcert.app.getCampaigns());
		var iter_fn = function( _, account) {
			accounts.push({Id:account.id, proxyType:account.proxyType});
		};

		for (let i = 0; i < campaigns.length; i++)
		{
			var campaign = campaigns[i];
			$.each(campaign.accounts || {}, iter_fn);
		}

		accounts = prefilteredAccounts(rule, accounts);
		var displayed_channels = [];
		for (let j = 0; j < accounts.length; j++)
		{
			var proxyType = accounts[j].proxyType;
			if(!displayed_channels.find(channel => { return channel.Id === proxyType; }) && inConcert.app.hasLicensesForChannel( proxyType )) {
				displayed_channels.push( { Id:proxyType, Name: inConcert.app.getTranslatedChannel(proxyType)} );
				if(proxyType === "CALL") {
					displayed_channels.push( { Id:"VOICEMAIL", Name: inConcert.app.getTranslatedChannel("VOICEMAIL")} );
				}
			}
		}

		var $channelList = rule.$el.find('.rule-value-container [name$=_multiple]');
		$channelList.empty();

		var $iccRuleError = rule.$el.find('.error-container label');
		if(!displayed_channels.length) {
			var textError = _T("No types available");
			rule.error = textError;
			rule.$el.find('.error-container')[0].title = textError;
			$iccRuleError.text(textError);
			$iccRuleError.show();			
			return;
		}
		else {
			rule.error = null;
			$iccRuleError.text("");
			$iccRuleError.hide();
		}

		displayed_channels.sort((a,b) => (a.Name > b.Name) ? 1 : ((b.Name > a.Name) ? -1 : 0));

		for (let k = 0; k < displayed_channels.length; k++) {
			$channelList.append($('<option>', { 
				value: displayed_channels[k].Id ,
				text : displayed_channels[k].Name 
			}));
		}

		//Si el value no nos viene dado en la rule (bug de queryBuilder?), vemos si lo tenemos en nuestro queryBuilderConfig
		if(!rule.value) {
			var ruleIdx = parseInt(rule.id.replace(self.queryBuilderContainer.prop("id") + "_rule_", ""));
			if(!isNaN(ruleIdx) && self.queryBuilderConfig[self.queryBuilderCurrentPanel].rules && self.queryBuilderConfig[self.queryBuilderCurrentPanel].rules[ruleIdx]) {
				rule.value = self.queryBuilderConfig[self.queryBuilderCurrentPanel].rules[ruleIdx].value;
			}
		}

		if(rule.value) {
			//If rule value still exists on the list, select it
			var ruleValueExists = false;
			var options = $channelList.find("option");
			for(let i=0; i < options.length; i++) {
				if(options[i].value === rule.value){
					ruleValueExists = true;
					break;
				}
			}
			if(ruleValueExists){
				$channelList.val(rule.value);
			}
			else {	//rule value is changed to the first element on the list
				rule.value = $channelList.val();
			}			
		}
		else {
			rule.value = $channelList.val();
		}
	};

	inconcert_querybuilder.prototype.fillAttentionLevels = function(rule) {
		var self = this;
		var accounts = [];
		var campaigns = (this.fromApp === "supervisor" && this.supervisorCampaignId) ? [ inConcert.app.getCampaign(this.supervisorCampaignId) ] : prefilteredCampaigns(rule, inConcert.app.getCampaigns());
		var CampaignAtVcc; //With @vcc

		var iter_fn = function( _, account) {
			var AccountAttentionLevels = [];
			for (const index in account.AttentionLevels) {
				if(account.AttentionLevels.hasOwnProperty(index)) {
					var al = account.AttentionLevels[index];
					AccountAttentionLevels.push(al.attention_level_id);
				}
			}
			if(AccountAttentionLevels.length) {
				accounts.push({Id:account.id, Name: account.name, CampaignId: CampaignAtVcc, AttentionLevels:AccountAttentionLevels});
			}
		};

		for (let i = 0; i < campaigns.length; i++)
		{
			let campaign = campaigns[i];
			CampaignAtVcc = campaign.Id;
			$.each(campaign.accounts || {}, iter_fn);
		}

		accounts = prefilteredAccounts(rule, accounts);

		self.allAttentionLevels = {};
		var displayed_als = [];
		for (let j = 0; j < accounts.length; j++) {
			let campaign = inConcert.app.getCampaign(accounts[j].CampaignId);
			for (let k = 0; k < accounts[j].AttentionLevels.length; k++) {
				var attention_level = campaign.getAttentionLevel(accounts[j].AttentionLevels[k]);
				if(attention_level) {
					attention_level.pill = attention_level.name + " (" + accounts[j].Name + ")";
					self.allAttentionLevels[attention_level.id] =attention_level;
					var al = {
						id: attention_level.id,
						name: attention_level.name + " (" + accounts[j].Name + ")"
					};
					displayed_als.push(al);
				}
			}
		}

		var $attentionLevelList = rule.$el.find('.rule-value-container [name$=_multiple]');
		$attentionLevelList.empty();

		var $iccRuleError = rule.$el.find('.error-container label');
		if(!displayed_als.length) {
			var textError = _T("No attention levels available");
			rule.error = textError;
			rule.$el.find('.error-container')[0].title = textError;
			$iccRuleError.text(textError);
			$iccRuleError.show();
			return;
		}
		else {
			rule.error = null;
			$iccRuleError.text("");
			$iccRuleError.hide();
		}

		displayed_als.sort((a,b) => (a.name > b.name) ? 1 : ((b.name > a.name) ? -1 : 0));

		for (let i = 0; i < displayed_als.length; i++) {
			$attentionLevelList.append($('<option>', { 
				value: displayed_als[i].id ,
				text : displayed_als[i].name 
			}));
		}

		//Si el value no nos viene dado en la rule (bug de queryBuilder?), vemos si lo tenemos en nuestro queryBuilderConfig
		if(!rule.value) {
			var ruleIdx = parseInt(rule.id.replace(self.queryBuilderContainer.prop("id") + "_rule_", ""));
			if(!isNaN(ruleIdx) && self.queryBuilderConfig[self.queryBuilderCurrentPanel].rules && self.queryBuilderConfig[self.queryBuilderCurrentPanel].rules[ruleIdx]) {
				rule.value = self.queryBuilderConfig[self.queryBuilderCurrentPanel].rules[ruleIdx].value;
			}
		}

		if(rule.value) {
			//If rule value still exists on the list, select it
			var ruleValueExists = false;
			var options = $attentionLevelList.find("option");
			for(let i=0; i < options.length; i++) {
				if(options[i].value === rule.value){
					ruleValueExists = true;
					break;
				}
			}
			if(ruleValueExists){
				$attentionLevelList.val(rule.value);
			}
			else {
				rule.value = $attentionLevelList.val();
			}
		}
		else {
			rule.value = $attentionLevelList.val();
		}
	};

	inconcert_querybuilder.prototype.fillAgents = function(rule) {
		var self = this;
		var campaigns = (this.fromApp === "supervisor" && this.supervisorCampaignId) ? [ inConcert.app.getCampaign(this.supervisorCampaignId) ] : prefilteredCampaigns(rule, inConcert.app.getCampaigns());
		// Load all m_members for all campaigns
		for (let i = 0; i < campaigns.length; i++) {
			campaigns[i].getTeamMembers($.noop, true);
		}
		var waitAllCampaignsMembersLoadCompletion = function()
		{
			var agents = [];
			for (let i = 0; i < campaigns.length; i++) {
				for(let j = 0; j < campaigns[i].m_members.length; j++) {
					if(!agents.find(agent => { return agent.Id === campaigns[i].m_members[j].Id; })) {
						agents.push( {Id: campaigns[i].m_members[j].Id, Name: campaigns[i].m_members[j].Name});
					}
				}
			}
			//Agregar el usuario actualmente logado
			if(!agents.find(agent => { return agent.Id === inConcert.app.getLoggedUserId(); })) {
				agents.push( {Id: inConcert.app.getLoggedUserId(), Name: inConcert.app.getLoggedUserFirstName()});
			}

			var $memberList = rule.$el.find('.rule-value-container [name$=_multiple]');
			$memberList.empty();

			var $iccRuleError = rule.$el.find('.error-container label');
			if(!agents.length) {
				var textError = _T("No agents available");
				rule.error = textError;
				rule.$el.find('.error-container')[0].title = textError;
				$iccRuleError.text(textError);
				$iccRuleError.show();
				return;
			}
			else {
				rule.error = null;
				$iccRuleError.text("");
				$iccRuleError.hide();
			}

			if(rule.filter.id === "agent") {
				agents.sort((a,b) => (a.Id > b.Id) ? 1 : ((b.Id > a.Id) ? -1 : 0));
			}
			else{
				agents.sort((a,b) => (a.Name > b.Name) ? 1 : ((b.Name > a.Name) ? -1 : 0));
			}
			for (let i = 0; i < agents.length; i++) {
				$memberList.append($('<option>', { 
			        value: agents[i].Id,
			        text : rule.filter.id === "agent" ? agents[i].Id : agents[i].Name + " (" + agents[i].Id + ")"
			    }));
			}

			//Si el value no nos viene dado en la rule (bug de queryBuilder?), vemos si lo tenemos en nuestro queryBuilderConfig
			if(!rule.value) {
				var ruleIdx = parseInt(rule.id.replace(self.queryBuilderContainer.prop("id") + "_rule_", ""));
				if(!isNaN(ruleIdx) && self.queryBuilderConfig[self.queryBuilderCurrentPanel].rules && self.queryBuilderConfig[self.queryBuilderCurrentPanel].rules[ruleIdx]) {
					rule.value = self.queryBuilderConfig[self.queryBuilderCurrentPanel].rules[ruleIdx].value;
				}
			}

			if(rule.value) {
				//If rule value still exists on the list, select it
				var ruleValueExists = false;
				var options = $memberList.find("option");
				for(let i=0; i < options.length; i++) {
					if(options[i].value === rule.value){
						ruleValueExists = true;
						break;
					}
				}
				if(ruleValueExists){
					$memberList.val(rule.value);
				}
				else {
					rule.value = $memberList.val();
				}
			}
			else {
				rule.value = $memberList.val();
			}			
		};
		setTimeout(waitAllCampaignsMembersLoadCompletion, 500);
	};

	inconcert_querybuilder.prototype.switchToPanel = function(panel)
	{
		if(this.queryBuilderCurrentPanel) {
			var queryBuilderCurrentRules = this.queryBuilderContainer.queryBuilder('getRules');
			//Si las rules actuales no son validas, el panel se queda con las que tenia
			if(queryBuilderCurrentRules !== null) {
				this.queryBuilderConfig[this.queryBuilderCurrentPanel].rules = queryBuilderCurrentRules;
			}
			this.queryBuilderContainer.queryBuilder('destroy');
		}

		this.setEvents();
		this.queryBuilderCurrentPanel = panel;
		this.queryBuilderContainer.queryBuilder(this.queryBuilderConfig[panel]);
	};

	inconcert_querybuilder.prototype.setEvents = function()
	{
		var self = this;

		$("#query_builder_close_times").unbind('click').bind('click', function() {
			$("#interactionQueryBuilderContainer").toggle(); 
		});

		this.queryBuilderContainer.on("afterSetRules.queryBuilder", function(e, parent) {	
			var model = self.queryBuilderContainer.queryBuilder('getModel');
			self.queryBuilderContainer.find(".rules-group-header").toggle(model.rules.length >= 2);
		});

		this.queryBuilderContainer.on("afterReset.queryBuilder", function(e, parent) {
			//Despues del reset queda una rule vacia. Al eliminarla se ejecuta el afterDeleteRule, que refresca los datos
			var model = self.queryBuilderContainer.queryBuilder('getModel');
			if(model.rules.length > 0) {
				for(let i = model.rules.length - 1; i >= 0; i--)
				{
					self.deleteRule(i);
				}
			}
		});	

		this.queryBuilderContainer.on("beforeAddRule.queryBuilder", function(e, parent) {	
			var result = self.queryBuilderContainer.queryBuilder('getRules');
			var model = self.queryBuilderContainer.queryBuilder('getModel');
			//Cancel adding new rules if current ones are not valid
			if(!result || !result.valid)
			{
				for(let i = 0; i < model.rules.length; i++)
				{
					if(model.rules[i].error) {
						var errorMessage = "";
						if($.isArray(model.rules[i].error && model.rules[i].error.length && model.rules[i].error[0])) {
							errorMessage = _T(model.rules[i].error[0]);
						}
						else {
							errorMessage = _T(model.rules[i].error);
						}
						var $iccRuleError = model.rules[i].$el.find('.error-container label');
						$iccRuleError.text(errorMessage);
						$iccRuleError.show();
					}
				}
				e.preventDefault();
			}
		});

		this.queryBuilderContainer.on("afterAddRule.queryBuilder", function(e, rule) {
			self.queryBuilderContainer.find(".rules-group-header").toggle(rule.parent.rules.length >= 2);
			rule.$el.prepend("<div class='genericFormMessagesContainer messageError' hidden='' style='display: none;'>");
			self.onRuleAdded();
		});

		this.queryBuilderContainer.on("afterCreateRuleFilters.queryBuilder", function(e, rule) {
			//remove from this rule already selected filters in other rules filter selectbox
			if(rule.parent.condition === "AND") {
				rule.parent.rules.filter(otherRule => otherRule.id !== rule.id).forEach((otherRule, index, parentRules) => {
					if(otherRule.filter && otherRule.filter.id !== "businessdata") {
						rule.$el.find('.rule-filter-container select').find("option[value='" + otherRule.filter.id + "']").remove();
					}
				});	
			}
		});

		this.queryBuilderContainer.on("beforeDeleteRule.queryBuilder", function(e, rule) {				
			//No delete allowed if there is a non valid rule
			let otherRuleWithNoFilter = rule.parent.rules.find(otherRule => otherRule.id !== rule.id && !otherRule.filter);
			if(otherRuleWithNoFilter) {
				var textError = _T("no_filter");
				otherRuleWithNoFilter.error = textError;
				otherRuleWithNoFilter.$el.find('.error-container')[0].title = textError;
				var $iccRuleError = otherRuleWithNoFilter.$el.find('.error-container label');
				$iccRuleError.text(textError);
				$iccRuleError.show();
				e.preventDefault();
				return;
			}

			if(rule.parent.condition === "AND"){
				AddFilterToRules(rule.filter, rule.parent.rules, rule.id);
			}

			if(rule.filter) {
				//Update dependant filters
				switch(rule.filter.id) {
					case "campaign":
						rule.parent.rules.filter(otherRule => otherRule.id !== rule.id).forEach((otherRule, index, parentRules) => {
							for(let i=0; i < otherRule.parent.rules.length; i++) {
								if(otherRule.parent.rules[i].id === rule.id) {
									otherRule.parent.rules[i].beingDeleted = true;
									break;
								}
							}
							if(otherRule.filter && otherRule.filter.id === "account") {
								self.fillAccounts(otherRule);			
							}
							if(otherRule.filter && otherRule.filter.id === "type") {
								self.fillTypes(otherRule);			
							}
							if(otherRule.filter && otherRule.filter.id === "attention_level") {
								self.fillAttentionLevels(otherRule);			
							}
							if(otherRule.filter && (otherRule.filter.id === "agent" || otherRule.filter.id === "agentname")) {
								self.fillAgents(otherRule);			
							}
							for(let i=0; i < otherRule.parent.rules.length; i++) {
								if(otherRule.parent.rules[i].id === rule.id) {
									otherRule.parent.rules[i].beingDeleted = undefined;
									break;
								}
							}
						});		
					break;		
					case "account":
						rule.parent.rules.filter(otherRule => otherRule.id !== rule.id).forEach((otherRule, index, parentRules) => {
							for(let i=0; i < otherRule.parent.rules.length; i++) {
								if(otherRule.parent.rules[i].id === rule.id) {
									otherRule.parent.rules[i].beingDeleted = true;
									break;
								}
							}
							if(otherRule.filter && otherRule.filter.id === "type") {
								self.fillTypes(otherRule);			
							}
							if(otherRule.filter && otherRule.filter.id === "attention_level") {
								self.fillAttentionLevels(otherRule);			
							}
							for(let i=0; i < otherRule.parent.rules.length; i++) {
								if(otherRule.parent.rules[i].id === rule.id) {
									otherRule.parent.rules[i].beingDeleted = undefined;
									break;
								}
							}
						});		
					break;		
				}			
			}
		});

		this.queryBuilderContainer.on("afterDeleteRule.queryBuilder", function(e) {	
			var model = self.queryBuilderContainer.queryBuilder('getModel');
			self.queryBuilderContainer.find(".rules-group-header").toggle(model.rules.length >= 2);
			self.onReadyToRefresh();
		});

		this.queryBuilderContainer.on("afterUpdateGroupCondition.queryBuilder", function(e, group, previousCondition) {
			if(!previousCondition) {
				return;
			}
			self.queryBuilderContainer.find("#rules-and-or-message").find("label").text(rulesAndOrMessages[group.condition]);
			group.rules.forEach((rule, index, parentRules) => {
				if(group.condition === "OR") {
					AddFilterToRules(rule.filter, group.rules, rule.id);
					rule.parent.rules.filter(otherRule => otherRule.id !== rule.id).forEach((otherRule, index, parentRules) => {
						otherRule.error = null;
						var $iccRuleError = otherRule.$el.find('.error-container label');
						$iccRuleError.text("");
						$iccRuleError.hide();
					});
				}
				else {	//AND
					//remove rule.filter from other rules filter selectbox
					rule.parent.rules.filter(otherRule => otherRule.id !== rule.id).forEach((otherRule, index, parentRules) => {
						if(otherRule.filter && rule.filter) {
							//Removes otherRule to avoid filter repeating
							if(otherRule.filter.id === rule.filter.id) {
								var textError = format(_T('There are more than one rules with {0} filter. Remove them to keep just one.'), _T(otherRule.filter.label));
								otherRule.error = textError;
								otherRule.$el.find('.error-container')[0].title = textError;
								var $iccRuleError = otherRule.$el.find('.error-container label');
								$iccRuleError.text(textError);
								$iccRuleError.show();
							}
							else { //Just remove otherRule filter option
								otherRule.$el.find('.rule-filter-container select').find("option[value='" + rule.filter.id + "']").remove();
							}
						}
					});
				}
			});
			self.onReadyToRefresh();
		});

		this.queryBuilderContainer.on("validationError.queryBuilder", function(e, node, error, value) {
			var textError = _T(error);
			var $iccRuleError = node.$el.find('.error-container label');
			$iccRuleError.text(textError);
			$iccRuleError.show();
		});

		this.queryBuilderContainer.on("validateValue.filter.queryBuilder", function(e, value, rule) {
			var isValid = true;
			var textError;
			if(rule.parent.condition === "AND") {
				rule.parent.rules.filter(otherRule => otherRule.id !== rule.id).forEach((otherRule, index, parentRules) => {
					var $iccOtherRuleError = otherRule.$el.find('.error-container label');
					if(otherRule.filter && otherRule.filter.id === rule.filter.id) {
						if(otherRule.filter.id === "businessdata"){
							if(otherRule.value && otherRule.value.fieldName && rule.value && rule.value.fieldName && otherRule.value.fieldName === rule.value.fieldName){
								textError = format(_T('There are more than one rules with {0} filter. Remove them to keep just one.'), _T(otherRule.value.fieldName));
							}
						}
						else {
							textError = format(_T('There are more than one rules with {0} filter. Remove them to keep just one.'), _T(otherRule.filter.label));
						}
						if(textError) {
							otherRule.error = textError;
							otherRule.$el.find('.error-container')[0].title = textError;
							$iccOtherRuleError.text(textError);
							$iccOtherRuleError.show();
							isValid = otherRule.error;
						}
					}
				});
			}
			var $iccRuleError = rule.$el.find('.error-container label');
			if(!value){
				textError = _T("no_filter");
				rule.error = textError;
				rule.$el.find('.error-container')[0].title = textError;
				$iccRuleError.text(textError);
				$iccRuleError.show();
				isValid = rule.error;
			}
			if(isValid === true && rule.filter.id === "businessdata")
			{
				if(!rule.value.fieldName){
					textError = _T("Business data field name is required.");
					rule.error = textError;
					rule.$el.find('.error-container')[0].title = textError;
					$iccRuleError.text(textError);
					$iccRuleError.show();
					isValid = rule.error;
				}
				else if(rule.value.fieldType === "numeric" && (isNaN(rule.value.fieldValue) || rule.value.fieldValue === ""))
				{
					if(rule.value.fieldOperator === "between") {
						textError = _T("Business data from field value must be a number.");
					}
					else {
						textError = _T("Business data field value must be a number.");
					}
					rule.error = textError;
					rule.$el.find('.error-container')[0].title = textError;					
					$iccRuleError.text(textError);
					$iccRuleError.show();
					isValid = rule.error;
				}
				else if(rule.value.fieldType === "numeric" && rule.value.fieldOperator === "between" && (isNaN(rule.value.fieldValueTo) || rule.value.fieldValueTo === ""))
				{
					textError = _T("Business data to field value must be a number.");
					rule.error = textError;
					rule.$el.find('.error-container')[0].title = textError;
					$iccRuleError.text(textError);
					$iccRuleError.show();
					isValid = rule.error;
				}
				else if(rule.value.fieldType === "date" && !isValidDate(rule.value.fieldValue))
				{
					if(rule.value.fieldOperator === "between") {
						textError = _T("Business data from field value must be a date in yyyy-mm-dd format.");
					}
					else {
						textError = _T("Business data field value must be a date in yyyy-mm-dd format.");
					}
					rule.error = textError;
					rule.$el.find('.error-container')[0].title = textError;
					$iccRuleError.text(textError);
					$iccRuleError.show();
					isValid = rule.error;
				}
				else if(rule.value.fieldType === "date" && rule.value.fieldOperator === "between" && !isValidDate(rule.value.fieldValueTo))
				{
					textError = _T("Business data to field value must be a date in yyyy-mm-dd format.");
					rule.error = textError;
					rule.$el.find('.error-container')[0].title = textError;
					$iccRuleError.text(textError);
					$iccRuleError.show();
					isValid = rule.error;
				}
			}
			if(isValid === true && rule.filter.type === "date")
			{
				if((rule.operator.type !== "between" && !isValidDate(rule.value)) || (rule.operator.type === "between" && (!$.isArray(rule.value) || rule.value.length !== 2 || !isValidDate(rule.value[0]) || !isValidDate(rule.value[1])))) {
					textError = _T("Value must be a date in yyyy-mm-dd format.");
					rule.error = textError;
					rule.$el.find('.error-container')[0].title = textError;
					$iccRuleError.text(textError);
					$iccRuleError.show();
					isValid = rule.error;
				}
			}
			if(isValid === true) {
				rule.error = null;
				$iccRuleError.text("");
				$iccRuleError.hide();
			}
			e.value = isValid;
		});


		this.queryBuilderContainer.on("afterUpdateRuleFilter.queryBuilder", function(e, rule, previousFilter) {	
			if(!rule || !rule.filter) {
				return;
			}
			if(rule.parent.condition === "AND") {
				//remove rule.filter from other rules filter selectbox
				rule.parent.rules.filter(otherRule => otherRule.id !== rule.id).forEach((otherRule, index, parentRules) => {
					if(otherRule.filter) {
						otherRule.$el.find('.rule-filter-container select').find("option[value='" + rule.filter.id + "']").remove();
					}
				});
				AddFilterToRules(previousFilter, rule.parent.rules, rule.id);
			}

			//Avoid value autocomplete
			rule.$el.find('.rule-value-container input[type="text"]').each(function() {
				$(this).attr("autocomplete", "off");
			});

			
			var AddRemoveFilteringEntity = function(add) {
				rule.$el.find(".rule-filter-container").find("span").remove();
				if(add) {
					var tooltipText = _T("This entity will pre-filter the options of entities: ") + filteringEntities[rule.filter.id];
					rule.$el.find(".rule-filter-container").prepend("<span class='rule-input-alert far fa-exclamation-circle' title='" + tooltipText + "'></span>");
				}
			};

			switch(rule.filter.id) {
				case "campaign":
					AddRemoveFilteringEntity(true);
					var campaigns = inConcert.app.getCampaigns();
					campaigns.sort((a,b) => (a.Name > b.Name) ? 1 : ((b.Name > a.Name) ? -1 : 0));
					var $campaignList = rule.$el.find('.rule-value-container [name$=_multiple]');
					$campaignList.empty();
					for (let i = 0; i < campaigns.length; i++) {
						var campaign_id = campaigns[i].Id.match('([^@]+)@(.+)')[1];
						$campaignList.append($('<option>', { 
					        value: campaign_id,
					        text : campaigns[i].Name 
					    }));
					}

					//Si el value no nos viene dado en la rule (bug de queryBuilder?), vemos si lo tenemos en nuestro queryBuilderConfig
					if(!rule.value) {
						var ruleIdx = parseInt(rule.id.replace(self.queryBuilderContainer.prop("id") + "_rule_", ""));
						if(!isNaN(ruleIdx) && self.queryBuilderConfig[self.queryBuilderCurrentPanel].rules && self.queryBuilderConfig[self.queryBuilderCurrentPanel].rules[ruleIdx]) {
							rule.value = self.queryBuilderConfig[self.queryBuilderCurrentPanel].rules[ruleIdx].value;
						}
					}

					if(rule.value) {
						$campaignList.val(rule.value);
					}
					else {
						rule.value = $campaignList.val();
					}
				break;

				case "account":
					AddRemoveFilteringEntity(true);
					self.fillAccounts(rule);
				break;

				case "agent":
				case "agentname":
					AddRemoveFilteringEntity(false);
					self.fillAgents(rule);
				break;

				case 'businessdata':
					AddRemoveFilteringEntity(false);
					rule.$el.find('.rule-operator-container [name$=_operator]').hide();
				break;

				case "type":
					AddRemoveFilteringEntity(false);
					self.fillTypes(rule);
				break;	

				case "attention_level":
					self.fillAttentionLevels(rule);
				break;			
				case "events_unread":
				case "has_attachments":
				case "has_draft":
				case "need_help":
				case "id": /*interaction_id*/
				case "initiative":
				case "events_unanswered":
					rule.$el.find(".rule-operator-container").find("span").remove();
					AddRemoveFilteringEntity(false);
					break;
				default:					
					AddRemoveFilteringEntity(false);
			}

			//Update dependant filters
			if(previousFilter){
				switch(previousFilter.id) {
					case "campaign":
						rule.parent.rules.filter(otherRule => otherRule.id !== rule.id).forEach((otherRule, index, parentRules) => {
							if(otherRule.filter && otherRule.filter.id === "account") {
								self.fillAccounts(otherRule);			
							}
							if(otherRule.filter && otherRule.filter.id === "type") {
								self.fillTypes(otherRule);			
							}
							if(otherRule.filter && otherRule.filter.id === "attention_level") {
								self.fillAttentionLevels(otherRule);			
							}
							if(otherRule.filter && (otherRule.filter.id === "agent" || otherRule.filter.id === "agentname")) {
								self.fillAgents(otherRule);			
							}
						});					
					break;
					case "account":
						rule.parent.rules.filter(otherRule => otherRule.id !== rule.id).forEach((otherRule, index, parentRules) => {
							if(otherRule.filter && otherRule.filter.id === "type") {
								self.fillTypes(otherRule);			
							}
							if(otherRule.filter && otherRule.filter.id === "attention_level") {
								self.fillAttentionLevels(otherRule);			
							}
						});					
					break;
				}
			}

			if(rule.filter.type === "date")
			{
				//Esto es porque nuestro layout tiene un z-index de 9 y tapa el datepicker			
				rule.$el.find(".rule-value-container").find(".hasDatepicker").unbind('click').bind('click', function() {
					$("#ui-datepicker-div").css('z-index', 10);
				});		
			}
		});

		this.queryBuilderContainer.on("afterUpdateRuleOperator.queryBuilder", function(e, rule) {
			if(!rule || !rule.filter) {
				return;
			}
			//Avoid autocomplete on inputs regenerated by between operator
			rule.$el.find('.rule-value-container input[type="text"]').each(function() {
				$(this).attr("autocomplete", "off");
			});
			//Show select box or input
			if(['campaign', 'account', 'agent', 'agentname'].indexOf(rule.filter.id) !== -1) {
				var showOptions = ['equal', 'not_equal'].indexOf(rule.operator.type) !== -1;
				rule.$el.find('.rule-value-container [name$=_multiple]').toggle(showOptions);
				rule.$el.find('.rule-value-container [name$=_single]').toggle(!showOptions);
			}
			//Update dependant filters
			switch(rule.filter.id) {
				case "campaign":
					rule.parent.rules.filter(otherRule => otherRule.id !== rule.id).forEach((otherRule, index, parentRules) => {
						if(otherRule.filter && otherRule.filter.id === "account") {
							self.fillAccounts(otherRule);			
						}
						if(otherRule.filter && otherRule.filter.id === "type") {
							self.fillTypes(otherRule);			
						}
						if(otherRule.filter && otherRule.filter.id === "attention_level") {
							self.fillAttentionLevels(otherRule);			
						}
						if(otherRule.filter && (otherRule.filter.id === "agent" || otherRule.filter.id === "agentname")) {
							self.fillAgents(otherRule);			
						}
					});		
				break;		
				case "account":
					rule.parent.rules.filter(otherRule => otherRule.id !== rule.id).forEach((otherRule, index, parentRules) => {
						if(otherRule.filter && otherRule.filter.id === "type") {
							self.fillTypes(otherRule);			
						}
						if(otherRule.filter && otherRule.filter.id === "attention_level") {
							self.fillAttentionLevels(otherRule);			
						}
					});		
				break;		
			}

			if(rule.filter.type === "date")
			{
				//Esto es porque nuestro layout tiene un z-index de 9 y tapa el datepicker			
				rule.$el.find(".rule-value-container").find(".hasDatepicker").unbind('click').bind('click', function() {
					$("#ui-datepicker-div").css('z-index', 10);
				});		
			}

			if((rule.operator.type !== "between" && rule.value) || (rule.operator.type === "between" && $.isArray(rule.value) && rule.value.length === 2 && rule.value[0] && rule.value[1]))
			{
				self.onReadyToRefresh();
			}
		});

		this.queryBuilderContainer.on("afterUpdateRuleValue.queryBuilder", function(e, rule, previousValue) {
			if(!rule || !rule.filter) {
				return;
			}
			switch(rule.filter.id)
			{
				case "campaign":
					rule.parent.rules.filter(otherRule => otherRule.id !== rule.id).forEach((otherRule, index, parentRules) => {
						if(otherRule.filter && otherRule.filter.id === "account") {
							self.fillAccounts(otherRule);			
						}
						if(otherRule.filter && otherRule.filter.id === "type") {
							self.fillTypes(otherRule);			
						}
						if(otherRule.filter && otherRule.filter.id === "attention_level") {
							self.fillAttentionLevels(otherRule);			
						}
						if(otherRule.filter && (otherRule.filter.id === "agent" || otherRule.filter.id === "agentname")) {
							self.fillAgents(otherRule);			
						}
					});
					if(rule.value) {
						self.onReadyToRefresh();
					}
				break;

				case "account":
					rule.parent.rules.filter(otherRule => otherRule.id !== rule.id).forEach((otherRule, index, parentRules) => {
						if(otherRule.filter && otherRule.filter.id === "type") {
							self.fillTypes(otherRule);			
						}
						if(otherRule.filter && otherRule.filter.id === "attention_level") {
							self.fillAttentionLevels(otherRule);			
						}
					});
					if(rule.value) {
						self.onReadyToRefresh();
					}
				break;

				case "businessdata":
					if(!previousValue || rule.value.fieldType !== previousValue.fieldType) {
						rule.$el.find('.rule-value-container [name$=_fieldValue]').val("");
						rule.$el.find('.rule-value-container [name$=_fieldValueTo]').val("");
						rule.value.fieldValue = undefined;
						rule.value.fieldValueTo = undefined;
						var $fieldOperator = rule.$el.find('.rule-value-container [name$=_fieldOperator]');
						$fieldOperator.empty();
						var allowedFieldOperators = [];
						allowedFieldOperators.push({value:"equal",text:_T("equal")});
						allowedFieldOperators.push({value:"not_equal",text:_T("not_equal")});
						if(rule.value.fieldType === "text") {
							allowedFieldOperators.push({value:"contains",text:_T("contains")});
							allowedFieldOperators.push({value:"not_contains",text:_T("not_contains")});
						}
						if(rule.value.fieldType === "date" || rule.value.fieldType === "numeric") {
							allowedFieldOperators.push({value:"less",text:_T("less")});
							allowedFieldOperators.push({value:"less_or_equal",text:_T("less_or_equal")});
							allowedFieldOperators.push({value:"greater",text:_T("greater")});
							allowedFieldOperators.push({value:"greater_or_equal",text:_T("greater_or_equal")});
							allowedFieldOperators.push({value:"between",text:_T("between")});
						}
						for (let i = 0; i < allowedFieldOperators.length; i++) {
							$fieldOperator.append($('<option>', { 
						        value: allowedFieldOperators[i].value,
						        text : allowedFieldOperators[i].text
						    }));
						}
						//Seleccionamos el primero de los operators
						rule.value.fieldOperator = $fieldOperator.val();
						if(rule.value.fieldType === "boolean") {
							//Seleccionamos el primero de los values
							rule.value.fieldValue = rule.$el.find('.rule-value-container [name$=_fieldValueBoolean]').val();
						}
						if(rule.value.fieldType === "date") {
							rule.$el.find('.rule-value-container [name$=_fieldValue]').datepicker({ dateFormat: 'yy-mm-dd' });
							rule.$el.find('.rule-value-container [name$=_fieldValueTo]').datepicker({ dateFormat: 'yy-mm-dd' });
							//Esto es porque nuestro layout tiene un z-index de 9 y tapa el datepicker			
							rule.$el.find(".rule-value-container").find(".hasDatepicker").unbind('click').bind('click', function() {
								$("#ui-datepicker-div").css('z-index', 10);
							});
						}
						else {
							rule.$el.find('.rule-value-container [name$=_fieldValue]').datepicker("destroy");
							rule.$el.find('.rule-value-container [name$=_fieldValueTo]').datepicker("destroy");
						}
					}
					if(!previousValue || rule.value.fieldOperator !== previousValue.fieldOperator) {
						rule.$el.find('.rule-value-container [name$=_fieldValue]').prop("placeholder", rule.value.fieldOperator === "between" ? _T('From field value') : _T('Field value'));
					}
					rule.$el.find('.rule-value-container [name$=_fieldValueBoolean]').toggle(rule.value && rule.value.fieldType === "boolean");
					rule.$el.find('.rule-value-container [name$=_fieldValue]').toggle(rule.value && rule.value.fieldType !== "boolean");
					rule.$el.find('.rule-value-container [name$=_fieldValueTo]').toggle(rule.value && rule.value.fieldType !== "boolean" && rule.value.fieldOperator === "between");

					if(rule.value.fieldName && rule.value.fieldType && rule.value.fieldOperator && rule.value.fieldValue && (rule.value.fieldOperator !== "between" || rule.value.fieldValueTo)) {
						self.onReadyToRefresh();
					}

				break;

				default:
					if((rule.operator.type !== "between" && rule.value) || (rule.operator.type === "between" && $.isArray(rule.value) && rule.value.length === 2 && rule.value[0] && rule.value[1]))
					{
						self.onReadyToRefresh();
					}
			}
		});
	};

	inconcert_querybuilder.prototype.getFilter = function()
	{
		var self = this;
		var queryBuilderFilter = {
			condition: "AND",
			rules: []
		};

		switch(self.queryBuilderCurrentPanel)
		{
			case "active":
				queryBuilderFilter.fixedStateRule = ["LOCKED","TAKEN","WRAPUP"];
				break;
			case "abandoned":
			case "waiting":
			case "queued":
				queryBuilderFilter.fixedStateRule = self.queryBuilderCurrentPanel.toUpperCase();
				break;
			case "appointment":
				queryBuilderFilter.fixedStateRule = "OPEN";
				break;
			case "archive":
				if(self.fromApp === "interaction") {
					queryBuilderFilter.fixedAgentRule = inConcert.app.getCompleteId();
				}
				queryBuilderFilter.fixedStateRule = "FINISHED";
				break;
		}

		var result;
		//try...catch to avoid error on first query (active panel) after login and before queryBuilder initialization
		try
		{
			result = this.queryBuilderContainer.queryBuilder('getRules');
		}
		catch(error)
		{
			return queryBuilderFilter;
		}

		if (!$.isEmptyObject(result) && result.valid && result.rules && Array.isArray(result.rules)) {
			queryBuilderFilter.condition = result.condition;
			var businessdataRules = [];
			for(let i = 0; i < result.rules.length; i++){
				if(result.rules[i].id === "businessdata") {
					var businessdataRule = {
						name: result.rules[i].value.fieldName,
						type: result.rules[i].value.fieldType,
						value: result.rules[i].value.fieldValue,
						operator: result.rules[i].value.fieldOperator
					};
					if(businessdataRule.operator === "between") {
						businessdataRule.value_to = result.rules[i].value.fieldValueTo;
					}
					businessdataRules.push(businessdataRule);
				}
				else {
					var rule = {
						id: result.rules[i].id === "agentname" ? "agent" : result.rules[i].id,
						operator: result.rules[i].operator,
						value: result.rules[i].value,					
					};
					if(rule.operator === "between") {
						rule.value = result.rules[i].value[0];
						rule.value_to = result.rules[i].value[1];
					}
					var filter = this.queryBuilderFilters.find(filter => filter.id === rule.id);
					rule.label = filter ? filter.label : rule.id;
					if(result.rules[i].id === "attention_level") {
						rule.pill = self.allAttentionLevels ? self.allAttentionLevels[rule.value].pill : undefined;
					}
					queryBuilderFilter.rules.push(rule);
				}
			}
			if(businessdataRules.length > 0) {
				queryBuilderFilter.rules.push({
					id: "businessdata",
					label: "Business data",
					operator: "",	//Not used on businessdata
					value: businessdataRules
				});
			}

			if(this.queryBuilderCurrentPanel && this.queryBuilderConfig[this.queryBuilderCurrentPanel]) {
				this.queryBuilderConfig[this.queryBuilderCurrentPanel].rules = queryBuilderFilter.rules;
			}
		}
		else if(this.queryBuilderCurrentPanel && this.queryBuilderConfig[this.queryBuilderCurrentPanel]) {
			//#11730: Si actualizo con una rule indefinida, el array rules queda vacio. En ese caso uso las rules que tenga previamente
			queryBuilderFilter.rules = this.queryBuilderConfig[this.queryBuilderCurrentPanel].rules;
		}
		
		return queryBuilderFilter;
	};	

	inconcert_querybuilder.prototype.deleteRule = function(ruleIdx)
	{
		var model = this.queryBuilderContainer.queryBuilder('getModel');

		if (!$.isEmptyObject(model) && model.rules && Array.isArray(model.rules) && typeof model.rules[ruleIdx] !== 'undefined') {
			return this.queryBuilderContainer.queryBuilder('deleteRule', model.rules[ruleIdx]);
		}
	};

	inconcert_querybuilder.prototype.hasRules = function(enable)
	{
		var model = this.queryBuilderContainer.queryBuilder('getModel');
		return model.rules.length > 0;
	};

	inconcert_querybuilder.prototype.cleanAll = function(enable)
	{
		this.queryBuilderContainer.queryBuilder('reset');
	};

	inconcert_querybuilder.prototype.AddRuleAgent = function(agent)
	{
		this.cleanAll();
		var agentFilter = this.queryBuilderFilters.find(filter => filter.id === "agent");
		var model = this.queryBuilderContainer.queryBuilder('getModel');
		if(agentFilter && model) {
			var rule = model.rules.find(rule => rule.filter.id === "agent");
			if(!rule) {
				rule = this.queryBuilderContainer.queryBuilder('addRule', model);
				rule.filter = agentFilter;
			}
			rule.value = agent;
			var $memberList = rule.$el.find('.rule-value-container [name$=_multiple]');
			$memberList.val(rule.value);
		}
	};

	inconcert_querybuilder.prototype.toggle = function()
	{
		this.queryBuilderContainer.parent().parent().toggle();
	};	

}());