/*  jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
  strict:true, undef:true, unused:true, curly:true, browser:true, indent:false, maxerr:50, esversion: 11 */

/* global $, window, inConcert, _ */

// Create an inConcert object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.
if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	// "use strict";
	const _T = inConcert.i18n.getString;
	const format = inConcert.Utils.format;

	/**
	The interaction is in the process of being created (i.e. we're redacting an email, but we haven't sent it yet)
	* @param interaction
	*/
	function BeingComposed (interaction) {
		return interaction.isBeingComposed();
	}

	function SwitchToApp (app) {
		const $dashboard_apps_tab = $("#dashboard_apps_tab");
		const $app = $dashboard_apps_tab.find("[dashboard_item='" + app + "']");
		return $app.trigger("click");
	}

	var showBannerMessage = function(error_message, message_type, timeout_ms, has_log)
	{
		message_type = message_type || "Error";
		timeout_ms = timeout_ms || 5000;
		has_log = typeof(has_log) === "undefined" ? true : has_log;

		inConcert.app.m_mainWindow.showBannerMessage(
			{
				message:     error_message,
				messageType: message_type,
				timeout:     timeout_ms
			},
			null,
			has_log);
	};

	/**
	Negates the result of calling function 'fn'. (taken from http://eloquentjavascript.net/chapter6.html)
	* @param fn
	*/
	function Not(fn) {
		return function() {
			return !fn.apply(null, arguments);
		};
	}

	//generic functions to query for standard interaction states
	function LIVE(interaction) {
		return interaction.isLive() && !BeingComposed(interaction);
	}

	function CancelInteraction(interaction) {
		inConcert.app.getInteractionManager().cancelInteraction(interaction);
	}


	//---------------------------------------------------------------------
	//
	function generic_buttons( params )
	{
		this.parent = params.parent;
	}

	// visible actions ---
	//
	generic_buttons.prototype.is_visible_monitor = function( /*interaction*/ )
	{
		return false;
	};

	generic_buttons.prototype.is_visible_transfer = function(interaction, not_must_validate_custom_action)
	{
		if (!not_must_validate_custom_action) {
			var is_visible_custom_action = this.parent.is_visible_custom_action('transfer');
			if (!is_visible_custom_action) {
				return false;
			}
		}
		var has_permission = this.parent.has_permission_to_action( interaction, this.parent.TRANSFER );
		if ( !has_permission ) {
			return false;
		}
		return !interaction.isBotInternalConsultation() && !BeingComposed( interaction ) &&
				!interaction.hasPendingLabel() &&
				(interaction.belongsToMe() || interaction.isQueued());
	};

	generic_buttons.prototype.is_visible_transfer_only_in_toolbar = function()
	{
		return false;
	};

	generic_buttons.prototype.is_visible_conference = function( /*interaction*/ )
	{
		return false;
	};

	generic_buttons.prototype.is_visible_conference_only_in_toolbar = function()
	{
		return false;
	};

	generic_buttons.prototype.is_visible_finish = function(interaction, not_must_validate_custom_action)
	{
		if (!not_must_validate_custom_action) {
			var is_visible_custom_action = this.parent.is_visible_custom_action('finish');
			if (!is_visible_custom_action) {
				return false;
			}
		}
		var has_permission = this.parent.has_permission_to_action( interaction, this.parent.FINISH );
		if ( !has_permission ) {
			return false;
		}
		return !BeingComposed( interaction ) &&
				!interaction.hasPendingLabel() &&
				(interaction.belongsToMe() || interaction.isQueued());
	};

	generic_buttons.prototype.is_visible_queue = function(interaction, not_must_validate_custom_action)
	{
		if (!not_must_validate_custom_action) {
			var is_visible_custom_action = this.parent.is_visible_custom_action('queue');
			if (!is_visible_custom_action) {
				return false;
			}
		}
		var has_permission = this.parent.has_permission_to_action( interaction, this.parent.REQUEUE );
		if ( !has_permission ) {
			return false;
		}
		return interaction.state !== "WRAPUP" && !BeingComposed(interaction) && !interaction.hasPendingLabel() && interaction.belongsToMe();
	};

	generic_buttons.prototype.is_visible_cancel = function(interaction, not_must_validate_custom_action)
	{
		if (!not_must_validate_custom_action) {
			var is_visible_custom_action = this.parent.is_visible_custom_action('cancel');
			if (!is_visible_custom_action) {
				return false;
			}
		}
		return BeingComposed( interaction );
	};

	generic_buttons.prototype.is_visible_comment = function(interaction, not_must_validate_custom_action)
	{
		if (!not_must_validate_custom_action) {
			var is_visible_custom_action = this.parent.is_visible_custom_action('comment');
			if (!is_visible_custom_action) {
				return false;
			}
		}
		if ( interaction.isArchived() ) {
			return false;
		}
		var has_permission = this.parent.has_permission_to_action( interaction, this.parent.COMMENT );
		if ( !has_permission ) {
			return false;
		}
		return !interaction.isBotInternalConsultation() && !BeingComposed( interaction ) && !interaction.hasPendingLabel();
	};

	generic_buttons.prototype.is_visible_label = function(interaction, not_must_validate_custom_action)
	{
		if (!not_must_validate_custom_action) {
			var is_visible_custom_action = this.parent.is_visible_custom_action('label');
			if (!is_visible_custom_action) {
				return false;
			}
		}
		if ( interaction.isArchived() ) {
			return false;
		}
		var has_permission = this.parent.has_permission_to_action( interaction, this.parent.LABEL );
		if ( !has_permission ) {
			return false;
		}
		return !interaction.isBotInternalConsultation() && !BeingComposed( interaction );
	};

	generic_buttons.prototype.is_visible_reopen = function(interaction, not_must_validate_custom_action)
	{
		if (!interaction.isArchived()) {
			return false;
		}
		//Si la interaccion tiene relacionada una nueva interaccion que se ha creado a partir de esta, NO visualizamos la accion.
		if (interaction.relatedInteractions?.newInteraction) {
			return false;
		}
		if (!not_must_validate_custom_action) {
			var is_visible_custom_action = this.parent.is_visible_custom_action('reopen');
			if (!is_visible_custom_action) {
				return false;
			}
		}
		// Bots
		if (!interaction.getLastAgentEndsConversation()) {
			return false;
		}
		// Valida si desde la app "interaction/global search", tiene permisos para reabrir interacciones que no le pertenecen
		var reopen_not_belong_to_you = "int_global_search_reopen_interactions_not_belong_to_you";
		var has_permission_reopen_conv_not_belong_to_you = inConcert.permissions.to_action("interaction",
			reopen_not_belong_to_you,
			"global_search_item",
			interaction.campaign,
			true /* no log */
		);
		if (!has_permission_reopen_conv_not_belong_to_you) {
			// Si no tiene permisos para reabrir interacciones que no le pertenecen, valida que la interaccion sea de el.
			var archived_belongs_to_me = interaction.archivedBelongsToMe();
			if (!archived_belongs_to_me) {
				return false;
			}
		}
		var has_permission = this.parent.has_permission_to_action( interaction, this.parent.REOPEN );
		if (!has_permission) {
			return false;
		}
		return true;
	};

	generic_buttons.prototype.is_visible_take = function( interaction, not_must_validate_custom_action)
	{
		if (!not_must_validate_custom_action) {
			var is_visible_custom_action = this.parent.is_visible_custom_action('take');
			if (!is_visible_custom_action) {
				return false;
			}
		}
		if ( !interaction.isQueued() && !interaction.isAppointment() ) {
			return false;
		}
		if (interaction.isAppointment() && !interaction.isAppointmentActive()) {
			return false;
		}
		//in allegro call ring in appointment state
		if (inConcert.app.has_allegro_baragent() && interaction.isAllegro() && interaction.isAppointment()) {
			var phone = inConcert.app.getInteractionManager().getChannelClient("CALL");
			if (!phone || phone.attendingInteraction(interaction)) {
				return false;
			}
		}
		var has_permission = this.parent.has_permission_to_action( interaction, this.parent.TAKE );
		if ( !has_permission ) {
			return false;
		}
		return true;
	};

	generic_buttons.prototype.is_visible_take_bot_interaction = function(interaction)
	{
		if (interaction.state !== "CREATED" || !interaction.interactionData.internal_consultation_flag) {
			return false;
		}

		if (!interaction.isBotInternalConsultation()) {
			return false;
		}
		
		if (interaction.interactionData.internal_consultation_owner) {
			return false;
		}

		return true;
	};

	generic_buttons.prototype.is_visible_schedule = function( /*interaction*/ )
	{
		return false;
	};

	generic_buttons.prototype.is_visible_schedule_only_in_toolbar = function()
	{
		return false;
	};

	function __hasTicketConfig ( interaction ) {
		var campaign = inConcert.app.getCampaign( interaction.campaign );
		if ( !campaign ) {
			return false;
		}
		var account_id = interaction.getAccount();
		var proxy_type = interaction.type;
		return campaign.hasTicketConfiguredInAccount( account_id, proxy_type );
	}

	generic_buttons.prototype.is_visible_emit_ticket = function (interaction, not_must_validate_custom_action)
	{
		if (!not_must_validate_custom_action) {
			var is_visible_custom_action = this.parent.is_visible_custom_action('emit_ticket');
			if (!is_visible_custom_action) {
				return false;
			}
		}
		if(interaction.isBotInternalConsultation() || interaction.isInternal() || !interaction.isTaken() || 
				!__hasTicketConfig(interaction) || interaction.getAssignedTicket()) {
			return false;
		}
		var has_permission = this.parent.has_permission_to_action( interaction, this.parent.TICKET_EMIT );
		if ( !has_permission ) {
			return false;
		}
		return true;
	};

	generic_buttons.prototype.is_visible_search_ticket = function (interaction, not_must_validate_custom_action)
	{
		if (!not_must_validate_custom_action) {
			var is_visible_custom_action = this.parent.is_visible_custom_action('search_ticket');
			if (!is_visible_custom_action) {
				return false;
			}
		}
		if(interaction.isBotInternalConsultation() || interaction.isInternal() || interaction.isBeingComposed() || !__hasTicketConfig(interaction)) {
			return false;
		}
		var has_permission = this.parent.has_permission_to_action( interaction, this.parent.TICKET_SEARCH );
		if ( !has_permission ) {
			return false;
		}
		return true;
	};

	function __getTicketErrorMessage(error) {
		error = error || "";
		switch ( error.toUpperCase() ) {
			case "ACCOUNT_DOES_NOT_EXIST":
				return _T("Account does not exist.");

			case "ACCOUNT_DOES_NOT_HAVE_TICKET_CONFIG":
				return _T("This account dont have ticket configured.");

			case "CAMPAIGN_DOES_NOT_EXIST":
				return _T("Campaign does not exist.");

			case "INTERACTION_DOES_NOT_EXIST":
				return _T("Conversation does not exist.");

			case "UNABLE_TO_EMIT_TICKET":
				return _T("Unable to emit ticket.");

			default:
				return _T("Server error");
		}
	}

	generic_buttons.prototype.emit_ticket = function ( interaction, node, callbacks )
	{
		var error_msg;
		var on_ticket = callbacks && callbacks.on_ticket ? callbacks.on_ticket : $.noop;
		var p = this.parent;
		//permissions
		if(!interaction || interaction.isInternal() || !interaction.isTaken() || interaction.getAssignedTicket()) {
			error_msg = format( _T("Unable to emit ticket.") );
			showBannerMessage( error_msg );
			return false;
		}
		if (!__hasTicketConfig(interaction)) {
			error_msg = format( _T("This account dont have ticket configured.") );
			showBannerMessage( error_msg );
			return false;
		}
		if ( !p.has_permission_on_campaign( interaction, p.TICKET_EMIT ) ) {
			error_msg = format( _T("You dont have permission for this action.") );
			showBannerMessage( error_msg );
			return false;
		}
		interaction.emitTicket(function( ok, ticket ) {
			if(!ok) {
				var err_message = __getTicketErrorMessage( ticket );
				showBannerMessage( err_message );
			}
			on_ticket(ok);
		});
	};

	generic_buttons.prototype.search_ticket = function ( interaction, node, callbacks )
	{
		var error_msg;
		var on_search = callbacks && callbacks.on_search ? callbacks.on_search : $.noop;
		var p = this.parent;
		//permissions
		if(!interaction || interaction.isInternal() || interaction.isBeingComposed()) {
			error_msg = format( _T("Unable to search ticket.") );
			showBannerMessage( error_msg );
			return false;
		}
		if (!__hasTicketConfig(interaction)) {
			error_msg = format( _T("This account dont have ticket configured.") );
			showBannerMessage( error_msg );
			return false;
		}
		if ( !p.has_permission_on_campaign( interaction, p.TICKET_SEARCH ) ) {
			error_msg = format( _T("You dont have permission for this action.") );
			showBannerMessage( error_msg );
			return false;
		}
		// remove the popup if it was already built.
		var previous_container = $("#windowTicketSearchContainer");
		if (previous_container.length !== 0){
			previous_container.remove();
		}

		var $positionNode = node.positionNode;
		var $interactionNode = node.interactionNode;
		var existsNode = $interactionNode && $interactionNode.length > 0;

		inConcert.logger.guiAction( {action_name : "search_ticket", window_id: window.id},interaction );

		var popupTickets = new inConcert.TicketSearchUI({
			parent: $positionNode,
			interaction_id: interaction.id,
			canReposition: true,
			search_callback: function( result ) {
				if ( !result.status ) {
					var err_message = __getTicketErrorMessage( result.code );
					showBannerMessage( err_message );
				}
				on_search(result.status);
			}
		});

		popupTickets.show({
			parentNode: $positionNode,
			callbacks: {
				onOpen: function() {
					if (existsNode) {
						$interactionNode.addClass("selectedToSearchTicket");
					}
				},
				onClose: function() {
					if (existsNode) {
						$interactionNode.removeClass("selectedToSearchTicket");
					}
				}
			}
		});
	};

	// execute actions ---
	//
	generic_buttons.prototype.comment = function( interaction, node, callbacks )
	{
		var on_comment = callbacks && callbacks.on_comment ? callbacks.on_comment : $.noop;
		var p = this.parent;
		if ( !p.has_permission_on_campaign( interaction, p.COMMENT ) ) {
			var error_msg = format( _T("You have not permissions to comment conversations on {0} campaign."), interaction.campaign );
			showBannerMessage( error_msg );
			return false;
		}
		inConcert.logger.guiAction( {action_name : "open_comments_interaction", window_id: window.id},interaction );

		var $positionNode = node.positionNode;
		var $interactionNode = node.interactionNode;
		var existsNode = $interactionNode && $interactionNode.length > 0;

		var comments = new inConcert.CommentsPopup( { from_app: p.app_id } );

		comments.show({
			//will be used for retrieving the old comments
			interaction: interaction,
			//will be used to position the popup
			parentNode: $positionNode,
			canReposition: true,
			callbacks: {
				on_comment : function( success/*, error*/ ) {
					on_comment(success);
				},
				onOpen: function() {
					if (existsNode) {
						$interactionNode.addClass("selectedToComment");
					}
					//mark interaction comments as read
					interaction.readComments();
				},
				onClose: function() {
					if (existsNode) {
						$interactionNode.removeClass("selectedToComment");
					}
				}
			}
		});
	};

	generic_buttons.prototype.label = function( interaction, node, callbacks )
	{
		var on_label = callbacks && callbacks.on_label ? callbacks.on_label : $.noop;
		var p = this.parent;
		if ( !p.has_permission_on_campaign( interaction, p.LABEL ) ) {
			var error_msg = format( _T("You have not permissions to label conversations on {0} campaign."), interaction.campaign );
			showBannerMessage( error_msg );
			return false;
		}
		inConcert.logger.guiAction({action_name : "open_labels_interaction", window_id: window.id}, interaction);
		var $positionNode = node.positionNode;
		var $interactionNode = node.interactionNode;
		var existsNode = $interactionNode && $interactionNode.length > 0;

		var labelPopup = new inConcert.LabelsPopup( { from_app: p.app_id } );

		labelPopup.show({
			interactions: [ interaction ],
			campaign: interaction.getCampaign(),
			parentNode: $positionNode,
			canReposition: true,
			callbacks: {
				on_label : function( success/*, error*/ ) {
					on_label(success);
				},
				onOpen: function() {
					if (existsNode) {
						$interactionNode.addClass("selectedToTag");
					}
				},
				onClose: function() {
					if (existsNode) {
						$interactionNode.removeClass("selectedToTag");
					}
				}
			}
		});
	};

	generic_buttons.prototype.owner_is_client_window = function()
	{
		return this.m_client_window !== undefined;
	};

	generic_buttons.prototype.transfer = function( interaction, node, callbacks, transfer_type )
	{
		var interactions;
		if ($.isArray(interaction)) {
			interactions = interaction;
			interaction = inConcert.Utils.cloneObject(interaction[0]);
		}
		else {
			interactions = [interaction];
		}

		if ( !this.is_visible_transfer(interaction) ) {
			var error_msg = format(_T("You have not permissions to transfer conversations on {0} campaign."), interaction.campaign);
			showBannerMessage(error_msg);
			return false;
		}
		var p = this.parent;
		var self = this;
		var $position_node = node.positionNode;
		var $interaction_node = node.interactionNode;
		var exists_node = $interaction_node && $interaction_node.length > 0;

		var transfer_callbacks = (callbacks && callbacks.transfer) ? callbacks.transfer : null;

		var transfer_popup = new inConcert.transfer_popup({
			from_app: p.app_id,
			transfer_type: transfer_type, //mismo valor que el que existe en la definicion del popup de transfer
			callbacks: self.owner_is_client_window()? {
				showScreenReadOnly: function (show) {
					self.m_client_window.showScreenReadOnly(show);
				}
			} : null
		});

		/*
			Esto es para el popup de transfer desde la ventana de los clientes, NO afecta, o da lo mismo para cuando se utiliza el popup desde la bandeja.
			Para el transfer tiene que cambiar a demanda el magic number, que se utiliza en la clase "popupwindows".
			Esto debido a que con el nuevo diseno se han utilizado posiciones absolutas y no se que mas, que afecta a como realiza el calculo de posicionamiento.
			Y para el caso de telefonia-tansfer-member (**), se debe de modificar de forma diferente este number magic y el ajuste de posiciones se debe de realizar en un timeout.
		*/
		let decrement = this.owner_is_client_window() ? 250: 10;
		var positions_options = {
			top_position_config: {
				//height_magic_number: 50,
				top_magic_number: -55,
				decrement: decrement, // con este valor ajusto el top en N px si es que sale fuera de la pantalla.
				show_in_timeout_ms: 250
			}
		};

		transfer_popup.show(
		{
			interactions : interactions,
			parentNode   : $position_node,
			canReposition: true,
			positions_options: positions_options,
			callbacks    :
			{
				on_transfer_success : function()
				{
					if ( transfer_callbacks && transfer_callbacks.on_transfer_success ) {
						transfer_callbacks.on_transfer_success( interaction.campaign );
					}
				},
				on_before_transfer_execute : function(transfer_to, interactions)
				{
					if ( transfer_callbacks && transfer_callbacks.on_before_transfer_execute ) {
						transfer_callbacks.on_before_transfer_execute( transfer_to, interactions);
					}
				},
				on_transfer_error : function()
				{
					if ( p.app_id === "interaction" ) {
						inConcert.app.getMainWindow().getPanel("AssignedInteractions").updateInteractionNode( interaction );
					}
					if ( transfer_callbacks && transfer_callbacks.on_transfer_error ) {
						transfer_callbacks.on_transfer_error();
					}
				},
				onOpen: function()
				{
					if (exists_node) {
						$interaction_node.addClass("selectedToTransfer");
					}
					if ( transfer_callbacks && transfer_callbacks.onOpen ) {
						transfer_callbacks.onOpen($interaction_node);
					}
				},
				onClose: function()
				{
					if (exists_node) {
						$interaction_node.removeClass("selectedToTransfer");
					}
					if ( transfer_callbacks && transfer_callbacks.onClose ) {
						transfer_callbacks.onClose($interaction_node);
					}
				}
			}
		});
	};

	generic_buttons.prototype.conference = function( interaction, node, callbacks, conference_type )
	{
		if ($.isArray(interaction)) {
			interaction = interaction[0];
		}
		if ( !this.is_visible_conference(interaction) ) {
			var error_msg = format(_T("You have not permissions to put in conference conversations on {0} campaign."), interaction.campaign);
			showBannerMessage(error_msg);
			return false;
		}
		var p = this.parent;
		var $position_node = node.positionNode;
		var $interaction_node = node.interactionNode;
		var exists_node = $interaction_node && $interaction_node.length > 0;

		var conference_callbacks = (callbacks && callbacks.conference) ? callbacks.conference : null;

		var conference_popup = new inConcert.conference_popup({ 
			from_app: p.app_id,
			conference_type: conference_type
		});

		conference_popup.show(
		{
			interactions : [interaction],
			parentNode   : $position_node,
			canReposition: true,
			callbacks    :
			{
				on_conference_success : function()
				{
					if ( conference_callbacks && conference_callbacks.on_conference_success ) {
						conference_callbacks.on_conference_success( interaction.campaign );
					}
				},
				on_conference_error : function()
				{
					if ( p.app_id === "interaction" ) {
						inConcert.app.getMainWindow().getPanel("AssignedInteractions").updateInteractionNode( interaction );
					}
					if ( conference_callbacks && conference_callbacks.on_conference_error ) {
						conference_callbacks.on_conference_error();
					}
				},
				onOpen: function()
				{
					if (exists_node) {
						$interaction_node.addClass("selectedToConference");
					}
					if ( conference_callbacks && conference_callbacks.onOpen ) {
						conference_callbacks.onOpen($interaction_node);
					}
				},
				onClose: function()
				{
					if (exists_node) {
						$interaction_node.removeClass("selectedToConference");
					}
					if ( conference_callbacks && conference_callbacks.onClose ) {
						conference_callbacks.onClose($interaction_node);
					}
				}
			}
		});
	};

	generic_buttons.prototype.finish = function( interaction, node, callbacks )
	{
		var on_finish = callbacks && callbacks.on_finish ? callbacks.on_finish : $.noop;

		if (interaction.hasContactConflict()) {
			showBannerMessage(_T("This conversation contains a conflict, you must first resolve before finish it"), "Alert");
			inConcert.app.getMainWindow().getPanel("AssignedInteractions").openInteractionClient(interaction, null);
			return false;
		}

		var p = this.parent;
		if ( !p.has_permission_on_campaign( interaction, p.FINISH ) ) {
			var error_msg = format(_T("You have not permissions to finish conversations on {0} campaign."), interaction.campaign);
			showBannerMessage(error_msg);
			return false;
		}
		var campaign = inConcert.app.getCampaign(interaction.campaign);
		if (!campaign) {
			throw "Campaign has been deleted";
		}
		var $positionNode = node.positionNode;
		var $interactionNode = node.interactionNode;
		var existsNode = $interactionNode && $interactionNode.length > 0;

		var dispositionCodes = campaign.getDispositionCodes();
		if ( dispositionCodes ) {
			inConcert.logger.guiAction({action_name : "open_disposition_assigned_toolbar", window_id: window.id}, interaction);
			var dispositionCodesPopup = new inConcert.DispositionCodesPopup( { from_app: p.app_id } );

			dispositionCodesPopup.show({
				interactions:     [interaction],
				parentNode:       $positionNode,
				dispositionCodes: dispositionCodes,
				canReposition: true,
				callbacks: {
					on_finish : function( success/*, error*/) {
						on_finish(success);
					},
					onOpen: function() {
						if (existsNode) {
							$interactionNode.addClass("selectedToFinish");
						}
					},
					onClose: function() {
						if (existsNode) {
							 $interactionNode.removeClass("selectedToFinish");
						}
					}
				}
			});
		}
		else {
			var archive_callback = function(success, successfulNodes, unsuccessfulNodes)
			{
				var error_msg;
				if (!success) {
					on_finish(false);
					error_msg = successfulNodes;
					showBannerMessage(error_msg);
				}
				else if (success && $.isArray(successfulNodes) && $.isArray(unsuccessfulNodes)) {

					var unsuccessful_length = unsuccessfulNodes.length;
					if (unsuccessful_length > 0) {
						if (unsuccessful_length === 1) {
							var unsuccessfulNode = unsuccessfulNodes[0];
							switch (unsuccessfulNode.result)
							{
								case "INVALID_REQUEST":
									error_msg =  _T("Invalid request please try again later");
								break;

								case "FORBIDDEN":
									error_msg = _T("You have not permissions to finish conversations on campaign.");
								break;

								case "UNABLE_TO_FINISH_INTERACTION_WITH_PENDING_EVENTS":
									var total_pending_events;
									if (unsuccessfulNode.raw_info) {
										total_pending_events = unsuccessfulNode.raw_info.total_pending_events;
									}
									error_msg = format(_T("Unable to finish conversation. Conversation has {0} sending events."), total_pending_events);
								break;

								default:
									error_msg = _T("Failed finished conversations, please try again later");
							}
						}
						else {
							error_msg = format(_T("{0} conversations failed to finish"), unsuccessfulNodes.length);
						}
						showBannerMessage(error_msg);
						on_finish(false);
					}
					// handle the single finish from supervisor.
					else if (successfulNodes.length === 1){
						if (existsNode) {
							$interactionNode.remove();
						}
						on_finish(true); // finished
					}
				}
				inConcert.app.getMainWindow().getPanel("AssignedInteractions").updateInteractionNode( interaction );
			};

			interaction.archive( undefined, undefined, archive_callback, p.app_id );
			inConcert.app.getMainWindow().getPanel("AssignedInteractions").updateInteractionNode( interaction );
		}
	};

	generic_buttons.prototype.requeue = function( interaction, node, callbacks )
	{
		var on_requeue = callbacks && callbacks.on_requeue ? callbacks.on_requeue : $.noop;

		var p = this.parent;
		if ( !p.has_permission_on_campaign( interaction, p.REQUEUE ) ) {
			var error_msg = format(_T("You have not permissions to requeue conversations on {0} campaign."), interaction.campaign);
			showBannerMessage(error_msg);
			on_requeue(false, error_msg);
			return false;
		}
		inConcert.app.getMainWindow().getPanel("AssignedInteractions").updateInteractionNode(interaction);

		var requeue_callback = function( success, error_msg )
		{
			if ( success ) {
				on_requeue(true, interaction);
				return;
			}
			inConcert.app.getMainWindow().getPanel("AssignedInteractions").updateInteractionNode( interaction );
			showBannerMessage(error_msg);
			on_requeue(false, error_msg);
		};

		interaction.requeue( requeue_callback, p.app_id );
	};

	generic_buttons.prototype.reopen = function( interaction, node, callbacks )
	{
		var on_reopen = callbacks && callbacks.on_reopen ? callbacks.on_reopen : $.noop;

		if ( !this.is_visible_reopen(interaction) ) {
			var error_msg = format(_T("You have not permissions to reopen conversations on {0} campaign."), interaction.campaign);
			showBannerMessage(error_msg);
			on_reopen(false, error_msg);
			return false;
		}

		var p = this.parent;
		var interaction_gui = p.params.parent;
		var $interaction_node = node.interactionNode;
		interaction.pending = true;
		interaction_gui.calculate_interaction_class(interaction, $interaction_node);

		var reopen_callback = function( success, error_msg )
		{
			if ( success ) {
				on_reopen(true, interaction);
				interaction_gui.close_client_window(interaction);
				return;
			}
			// error
			interaction_gui.calculate_interaction_class(interaction, $interaction_node);
			showBannerMessage(error_msg);
			on_reopen(false, error_msg);
		};

		interaction.reopen( reopen_callback, p.app_id );
	};

	generic_buttons.prototype.take = function( interaction, node, callbacks )
	{
		var on_taken = callbacks && callbacks.on_taken ? callbacks.on_taken : $.noop;

		if ( !this.is_visible_take(interaction) ) {
			var error_msg = format(_T("You have not permissions to take conversations on {0} campaign."), interaction.campaign);
			showBannerMessage(error_msg);
			on_taken(false, error_msg);
			return false;
		}
		var p = this.parent;
		var interaction_gui = p.params.parent;
		var $interaction_node = node.interactionNode;
		interaction.pending = true;
		interaction_gui.calculate_interaction_class(interaction, $interaction_node);

		var take_callback = function( success, error_msg )
		{
			if ( success ) {
				on_taken(true, interaction);
				interaction_gui.close_client_window(interaction);
				return;
			}
			// error
			interaction.pending = false;
			p.params.parent.calculate_interaction_class(interaction, $interaction_node);
			showBannerMessage(error_msg);
			on_taken(false, error_msg);
		};

		interaction.take( take_callback, p.app_id );
	};

	generic_buttons.prototype.schedule = function( /*interaction*/ )
	{
		return;
	};

	generic_buttons.prototype.add_contact = function( interaction, node )
	{
		var p = this.parent;
		if ( !p.has_permission_on_campaign( interaction, p.REQUEUE ) ) {
			var error_msg = format(_T("You have not permissions to add contact conversations on {0} campaign."), interaction.campaign);
			showBannerMessage(error_msg);
			return false;
		}
		if (interaction.contact && interaction.contact.length > 0) {
			//TODO, esto falta resolver ya que hay que ir a updatear la interaccion al REDIS a decirle
			//que en realidad ahora tiene un contacto, y si eso no se hace el pibe puede crear N contactos a partir
			//de la interaccion asumiendo que no le esta quedando asociado
			return;
		}
		inConcert.logger.guiAction({action_name : "open_transfer_assigned_toolbar", window_id: window.id},interaction);

		var $positionNode = node.positionNode;
		var $interactionNode = node.interactionNode;
		var existsNode = $interactionNode && $interactionNode.length > 0;

		//this contact selector allows to create new contacts
		var selector = new inConcert.ContactSelector(
			$("#contactSelector"),
			null,
			function(onContactCreatedFn) {
				inConcert.app.getMainWindow().createContact(onContactCreatedFn, interaction);
			},
			function() {
				return interaction.campaign;
			});

		selector.show({
			onContactSelectedFn: function(contact) {
				inConcert.app.getMainWindow().assignContact(interaction, contact);
			},
			parentNode: $positionNode,
			callbacks: {
				onOpen: function() {
					if (existsNode) {
						$interactionNode.addClass("selectedToAddContact");
					}
				},
				onClose: function() {
					if (existsNode) {
						$interactionNode.removeClass("selectedToAddContact");
					}
				}
			}
		});
	};

	generic_buttons.prototype.show_contact = function( interaction )
	{
		inConcert.app.getMainWindow().showContact( interaction.contact );
	};

	//---------------------------------------------------------------------
	//
	function attention_call_buttons( params )
	{
		attention_call_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( attention_call_buttons, generic_buttons );

	attention_call_buttons.prototype.is_visible_answer = function(interaction, not_must_validate_custom_action)
	{
		if (!not_must_validate_custom_action) {
			var is_visible_custom_action = this.parent.is_visible_custom_action('answer');
			if (!is_visible_custom_action) {
				return false;
			}
		}
		var client = inConcert.app.getInteractionManager().getChannelClient("CALL");
		if (LIVE(interaction) && client && client.supports('answer') && interaction.belongsToMe()) {
			if (interaction.state === "LOCKED") {
				return interaction.initiative === "INBOUND" && !interaction.in_redial;
			}
			if ($.inArray(interaction.state, ["TRANSFER LOCKED", "CONFERENCE LOCKED"]) >= 0 && interaction.reservedAgents) {
				// Must check I'm in the list of reserved agent (i.e. the transfer receipt)
				var loggedUserId = inConcert.app.getLoggedUserId();
				for (var i = 0; i < interaction.reservedAgents.length; ++i) {
					if (interaction.reservedAgents[i].match(inConcert.app.idAtVCCRegExp)) {
						var username = RegExp.$1;
						if (username === loggedUserId) {
							return true; // finally, that's me
						}
					}
				}
			}
		}
		return false;
	};

	attention_call_buttons.prototype.is_visible_hangup = function( interaction, not_must_validate_custom_action)
	{
		if (!not_must_validate_custom_action) {
			var is_visible_custom_action = this.parent.is_visible_custom_action('hangup');
			if (!is_visible_custom_action) {
				return false;
			}
		}
		var has_permission = this.parent.has_permission_to_action( interaction, this.parent.HANGUP );
		if ( !has_permission ) {
			return false;
		}

		if (interaction.belongsToMe()) {
			if (interaction.state === "LOCKED") {
				return interaction.isCreatedByAgent() || interaction.in_redial;
			}
			if (interaction.state === "TAKEN") {
				return true;
			}
			let allegro = inConcert.app.has_allegro_baragent() && interaction.isAllegro();
			if (_.contains(["TRANSFER LOCKED", "TRANSFER TAKEN", "CONFERENCE LOCKED", "CONFERENCE TAKEN", "CONFERENCE"], interaction.state)) {
				if (allegro) {
					var username = interaction.getFirstAssignedAgent();
					if (username) {
						return (username !== inConcert.app.getLoggedUserId());
					}
				}
				else {
					return true;
				}
			}
			if (allegro && interaction.state === "OPEN") {
				var phone = inConcert.app.getInteractionManager().getChannelClient("CALL");
				if (phone && phone.attendingInteraction(interaction)) {
					return true;
				}
			}
		}
		return false;
	};

	attention_call_buttons.prototype.is_visible_transfer = function(interaction, not_must_validate_custom_action)
	{
		if (!not_must_validate_custom_action) {
			var is_visible_custom_action = this.parent.is_visible_custom_action('transfer');
			if (!is_visible_custom_action) {
				return false;
			}
		}
		var has_permission = this.parent.has_permission_to_action( interaction, this.parent.TRANSFER );
		if ( !has_permission ) {
			return false;
		}
		var phone = inConcert.app.getInteractionManager().getChannelClient("CALL");
		return ((interaction.state === "TAKEN" && interaction.belongsToMe()) || interaction.isQueued())&&
			phone && phone.supports('transfer');
	};

	attention_call_buttons.prototype.is_visible_conference = function(interaction, not_must_validate_custom_action)
	{
		if (!not_must_validate_custom_action) {
			var is_visible_custom_action = this.parent.is_visible_custom_action('conference');
			if (!is_visible_custom_action) {
				return false;
			}
		}
		var has_permission = this.parent.has_permission_to_action( interaction, this.parent.CONFERENCE );
		if ( !has_permission ) {
			return false;
		}
		var phone = inConcert.app.getInteractionManager().getChannelClient("CALL");
		return (interaction.state === "TAKEN" || (interaction.state === "CONFERENCE" && Object.values(interaction.interactionData.legs).filter(l => l.status === "BRIDGED").length < (interaction.interactionData.maxConferenceParticipants || 3))) &&
			phone && phone.supports('conference') &&
			!BeingComposed(interaction) &&
			interaction.belongsToMe();
	};

	attention_call_buttons.prototype.is_visible_queue = function(interaction, not_must_validate_custom_action)
	{
		if (!not_must_validate_custom_action) {
			var is_visible_custom_action = this.parent.is_visible_custom_action('queue');
			if (!is_visible_custom_action) {
				return false;
			}
		}
		var has_permission = this.parent.has_permission_to_action( interaction, this.parent.REQUEUE );
		if ( !has_permission ) {
			return false;
		}
		var phone = inConcert.app.getInteractionManager().getChannelClient("CALL");
		return interaction.state === "TAKEN" &&
			phone && phone.supports('transfer') &&
			!BeingComposed(interaction) &&
			interaction.belongsToMe();
	};

	attention_call_buttons.prototype.is_visible_finish = function(interaction, not_must_validate_custom_action)
	{
		if (!not_must_validate_custom_action) {
			const is_visible_custom_action = this.parent.is_visible_custom_action('finish');
			if (!is_visible_custom_action) {
				return false;
			}
		}
		const has_permission = this.parent.has_permission_to_action( interaction, this.parent.FINISH );
		if ( !has_permission ) {
			return false;
		}
		//this is an allegro condition
		const im = inConcert.app.getInteractionManager();
		const phone = im.getChannelClient("CALL");
		if (interaction.isAllegro() && interaction.state === "TAKEN" && phone && phone.status() === "WRAPUP") {
			return true;
		}
		//check wrapup state was not set manually by phone client (maybe a requeued)
		if (!interaction.manualState && interaction.state === "WRAPUP" && interaction.belongsToMe()) {
			const phoneState = im.getPhoneState().toLowerCase();
			if (phoneState === "active") {
				return false;
			}
			return true;
		}
		// otherwise
		return false;
	};

	attention_call_buttons.prototype.is_visible_redial = function(interaction, not_must_validate_custom_action)
	{
		if (!not_must_validate_custom_action) {
			var is_visible_custom_action = this.parent.is_visible_custom_action('redial');
			if (!is_visible_custom_action) {
				return false;
			}
		}
		var has_permission = this.parent.has_permission_to_action( interaction, this.parent.REDIAL );
		if ( !has_permission ) {
			return false;
		}
		return !interaction.manualState && interaction.state === "WRAPUP" && interaction.belongsToMe();
	};

	attention_call_buttons.prototype.is_visible_schedule = function(interaction, not_must_validate_custom_action)
	{
		if (!not_must_validate_custom_action) {
			var is_visible_custom_action = this.parent.is_visible_custom_action('schedule');
			if (!is_visible_custom_action) {
				return false;
			}
		}
		var has_permission = this.parent.has_permission_to_action( interaction, this.parent.SCHEDULE );
		if ( !has_permission ) {
			return false;
		}
		return interaction.isScheduleEnabled();
	};

	// execute actions
	attention_call_buttons.prototype.answer = function( interaction )
	{
		var phone = inConcert.app.getInteractionManager().getChannelClient("CALL");
		if (phone) {
			phone.answer(interaction);
		}
	};

	attention_call_buttons.prototype.hangup = function( interaction )
	{
		var has_permission = this.parent.has_permission_to_action( interaction, this.parent.HANGUP );
		if ( !has_permission ) {
			return false;
		}
		var phone = inConcert.app.getInteractionManager().getChannelClient("CALL");
		if (phone) {
			phone.hangup(interaction);
		}
	};

	attention_call_buttons.prototype.redial = function( interaction )
	{
		var has_permission = this.parent.has_permission_to_action( interaction, this.parent.REDIAL );
		if ( !has_permission ) {
			return false;
		}
		var phone = inConcert.app.getInteractionManager().getChannelClient("CALL");
		if (phone && interaction.isPhoneValidToRedial()) {
			phone.redial(interaction);
		}
		else {
			var error_msg = _T("The number to redial is invalid or hidden.");
			showBannerMessage(error_msg);
			return false;
		}
	};

	attention_call_buttons.prototype.schedule = function ( interaction, node, callbacks )
	{
		if ( !this.is_visible_schedule(interaction) ) {
			var error_msg = format(_T("You have not permissions to schedule conversations on {0} campaign."), interaction.campaign);
			showBannerMessage(error_msg);
			return false;
		}
		var p = this.parent;
		var $position_node = node.positionNode;
		var interaction_gui = p.params.parent;
		var $interaction_node = node.interactionNode;
		var schedule_callbacks = (callbacks && callbacks.schedule) ? callbacks.schedule : null;

		var schedule_popup = new inConcert.schedule_popup( { from_app: p.app_id, $interaction: $interaction_node } );

		schedule_popup.show(
		{
			interactions : [interaction],
			parentNode   : $position_node,
			canReposition: true,
			callbacks    :
			{
				before_schedule : function()
				{
					// Dejo en waiting la interaccion en pantalla mientras se procesa
					interaction.pending = true;
					interaction_gui.calculate_interaction_class(interaction, $interaction_node);
				},
				on_schedule_success : function(callback)
				{
					if ( schedule_callbacks && schedule_callbacks.on_schedule_success ) {
					// Caso del panel de Abandonadas (estatico)
						schedule_callbacks.on_schedule_success( interaction.campaign, callback);
					}
					else {
					// Caso del panel de Live (dinamico)
						interaction.pending = false;
						interaction_gui.calculate_interaction_class(interaction, $interaction_node);
						callback();
					}
				},
				on_schedule_error : function()
				{
					interaction.pending = false;
					interaction_gui.calculate_interaction_class(interaction, $interaction_node);
					if ( schedule_callbacks && schedule_callbacks.on_schedule_error ) {
						schedule_callbacks.on_schedule_error();
					}
				},
				onOpen: function()
				{
					if ($interaction_node && $interaction_node.length > 0) {
						$interaction_node.addClass("selectedToSchedule");
					}
					if ( schedule_callbacks && schedule_callbacks.onOpen ) {
						schedule_callbacks.onOpen($interaction_node);
					}
				},
				onClose: function()
				{
					if ($interaction_node && $interaction_node.length > 0) {
						$interaction_node.removeClass("selectedToSchedule");
					}
					if ( schedule_callbacks && schedule_callbacks.onClose ) {
						schedule_callbacks.onClose($interaction_node);
					}
				}
			}
		});
	};

	//---------------------------------------------------------------------
	//
	function attention_voicemail_buttons( params )
	{
		attention_voicemail_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( attention_voicemail_buttons, generic_buttons );

	attention_voicemail_buttons.prototype.is_visible_schedule = function(interaction, not_must_validate_custom_action)
	{
		if (!not_must_validate_custom_action) {
			var is_visible_custom_action = this.parent.is_visible_custom_action('schedule');
			if (!is_visible_custom_action) {
				return false;
			}
		}
		var has_permission = this.parent.has_permission_to_action( interaction, this.parent.SCHEDULE );
		if ( !has_permission ) {
			return false;
		}
		return interaction.isScheduleEnabled();
	};

	// execute actions

	attention_voicemail_buttons.prototype.schedule = function ( interaction, node, callbacks )
	{
		if ( !this.is_visible_schedule(interaction) ) {
			var error_msg = format(_T("You have not permissions to schedule conversations on {0} campaign."), interaction.campaign);
			showBannerMessage(error_msg);
			return false;
		}
		var p = this.parent;
		var $position_node = node.positionNode;
		var interaction_gui = p.params.parent;
		var $interaction_node = node.interactionNode;
		var schedule_callbacks = (callbacks && callbacks.schedule) ? callbacks.schedule : null;

		var schedule_popup = new inConcert.schedule_popup( { from_app: p.app_id, $interaction: $interaction_node } );

		schedule_popup.show(
		{
			interactions : [interaction],
			parentNode   : $position_node,
			canReposition: true,
			callbacks    :
			{
				before_schedule : function()
				{
					// Dejo en waiting la interaccion en pantalla mientras se procesa
					interaction.pending = true;
					interaction_gui.calculate_interaction_class(interaction, $interaction_node);
				},
				on_schedule_success : function(callback)
				{
					if ( schedule_callbacks && schedule_callbacks.on_schedule_success ) {
					// Caso del panel de Abandonadas (estatico)
						schedule_callbacks.on_schedule_success( interaction.campaign, callback);
					}
					else {
					// Caso del panel de Live (dinamico)
						interaction.pending = false;
						interaction_gui.calculate_interaction_class(interaction, $interaction_node);
						callback();
					}
				},
				on_schedule_error : function()
				{
					interaction.pending = false;
					interaction_gui.calculate_interaction_class(interaction, $interaction_node);
					if ( schedule_callbacks && schedule_callbacks.on_schedule_error ) {
						schedule_callbacks.on_schedule_error();
					}
				},
				onOpen: function()
				{
					if ($interaction_node && $interaction_node.length > 0) {
						$interaction_node.addClass("selectedToSchedule");
					}
					if ( schedule_callbacks && schedule_callbacks.onOpen ) {
						schedule_callbacks.onOpen($interaction_node);
					}
				},
				onClose: function()
				{
					if ($interaction_node && $interaction_node.length > 0) {
						$interaction_node.removeClass("selectedToSchedule");
					}
					if ( schedule_callbacks && schedule_callbacks.onClose ) {
						schedule_callbacks.onClose($interaction_node);
					}
				}
			}
		});
	};

	//---------------------------------------------------------------------
	//
	function attention_facebook_buttons( params )
	{
		attention_facebook_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( attention_facebook_buttons, generic_buttons );

	//---------------------------------------------------------------------
	//
	function attention_instagram_buttons( params )
	{
		attention_instagram_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( attention_instagram_buttons, generic_buttons );

	//---------------------------------------------------------------------
	//
	function attention_youtube_buttons( params )
	{
		attention_youtube_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( attention_youtube_buttons, generic_buttons );

	//---------------------------------------------------------------------
	//
	function attention_appstore_buttons( params )
	{
		attention_appstore_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( attention_appstore_buttons, generic_buttons );

	//---------------------------------------------------------------------
	//
	function attention_googleplay_buttons( params )
	{
		attention_googleplay_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( attention_googleplay_buttons, generic_buttons );

	//---------------------------------------------------------------------
	//
	function attention_rcs_buttons( params )
	{
		attention_rcs_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( attention_rcs_buttons, generic_buttons );

	//---------------------------------------------------------------------
	//
	function attention_twitter_buttons( params )
	{
		attention_twitter_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( attention_twitter_buttons, generic_buttons );

	//---------------------------------------------------------------------
	//
	function attention_sms_buttons( params )
	{
		attention_sms_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( attention_sms_buttons, generic_buttons );

	//---------------------------------------------------------------------
	//
	function attention_whatsapp_buttons( params )
	{
		attention_whatsapp_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( attention_whatsapp_buttons, generic_buttons );

	//---------------------------------------------------------------------
	//
	function attention_webcontact_buttons( params )
	{
		attention_webcontact_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( attention_webcontact_buttons, generic_buttons );

	//---------------------------------------------------------------------
	//
	function attention_mail_buttons( params )
	{
		attention_mail_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( attention_mail_buttons, generic_buttons );

	//---------------------------------------------------------------------
	//
	function attention_im_buttons( params )
	{
		attention_im_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( attention_im_buttons, generic_buttons );

	//---------------------------------------------------------------------
	//
	function attention_chat_buttons( params )
	{
		attention_chat_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( attention_chat_buttons, generic_buttons );

	attention_chat_buttons.prototype.is_visible_transfer = function(interaction, not_must_validate_custom_action)
	{
		if (!not_must_validate_custom_action) {
			var is_visible_custom_action = this.parent.is_visible_custom_action('transfer');
			if (!is_visible_custom_action) {
				return false;
			}
		}
		var has_permission = this.parent.has_permission_to_action( interaction, this.parent.TRANSFER );
		if ( !has_permission ) {
			return false;
		}

		return !BeingComposed( interaction ) &&
				interaction.state !== "WRAPUP" &&
				(interaction.belongsToMe() || interaction.isQueued());
	};

	attention_chat_buttons.prototype.is_visible_finish = function(interaction, not_must_validate_custom_action)
	{
		if (!not_must_validate_custom_action) {
			var is_visible_custom_action = this.parent.is_visible_custom_action('finish');
			if (!is_visible_custom_action) {
				return false;
			}
		}
		var has_permission = this.parent.has_permission_to_action( interaction, this.parent.FINISH );
		if ( !has_permission ) {
			return false;
		}
		if ( interaction.state !== "WRAPUP" ) {
			return false;
		}
		if ( BeingComposed( interaction ) ) {
			return false;
		}
		if ( interaction.hasPendingLabel() ) {
			return false;
		}
		if ( !interaction.belongsToMe() ) {
			return false;
		}
		return true;
	};

	attention_chat_buttons.prototype.is_visible_hangup = function(interaction, not_must_validate_custom_action)
	{
		if (!not_must_validate_custom_action) {
			var is_visible_custom_action = this.parent.is_visible_custom_action('hangup');
			if (!is_visible_custom_action) {
				return false;
			}
		}
		var has_permission = this.parent.has_permission_to_action( interaction, this.parent.HANGUP );
		if ( !has_permission ) {
			return false;
		}
		return ( LIVE(interaction) && interaction.belongsToMe() && interaction.state === "TAKEN" );
	};

	// execute actions
	//
	attention_chat_buttons.prototype.hangup = function( interaction )
	{
		var has_permission = this.parent.has_permission_to_action( interaction, this.parent.HANGUP );
		if ( !has_permission ) {
			return false;
		}
		var session = interaction.getSession();
		if ( !session || !session.close() ) {
			var error_msg = _T("Unable to hang up conversation. Conversation it's closed.");
			showBannerMessage( error_msg );
			return false;
		}
		//this code is for showing the spinner
		interaction.pending = true;
		inConcert.app.getMainWindow().getPanel("AssignedInteractions").updateInteractionNode(interaction);
	};

	//---------------------------------------------------------------------
	//
	function supervisor_generic_buttons( params )
	{
		supervisor_generic_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( supervisor_generic_buttons, generic_buttons );

	// visible actions
	//
	supervisor_generic_buttons.prototype.is_visible_monitor = function(interaction, not_must_validate_custom_action)
	{
		if ( !interaction.isMonitorable() ) {
			return false;
		}
		if (!not_must_validate_custom_action) {
			var is_visible_custom_action = this.parent.is_visible_custom_action('monitor');
			if (!is_visible_custom_action) {
				return false;
			}
		}
		var has_permission = this.parent.has_permission_to_action( interaction, this.parent.MONITOR );
		if ( !has_permission ) {
			return false;
		}
		return true;
	};

	supervisor_generic_buttons.prototype.is_visible_transfer = function(interaction, not_must_validate_custom_action)
	{
		if (interaction.isAllegro() || interaction.isBotInternalConsultation()) {
			return false;
		}
		if (!not_must_validate_custom_action) {
			var is_visible_custom_action = this.parent.is_visible_custom_action('transfer');
			if (!is_visible_custom_action) {
				return false;
			}
		}
		if ( interaction.belongsToMe() || interaction.isArchived() || interaction.isWaiting() ) {
			return false;
		}
		var campaign_id, has_show_transfer_button;
		if ( interaction.isQueued() ) {
			var has_transfer_permissions = interaction.hasPermission( 'TRANSFER_QUEUED', 'supervisor' ); // contra conversation management
			if ( !has_transfer_permissions ) {
				return false;
			}
			campaign_id = interaction.campaign;
			// contra el layout para saber si puede visualizarlo
			has_show_transfer_button = inConcert.permissions.to_action( 'supervisor', 'sup_queued_transfer', undefined /* category*/ , campaign_id, true /* no log */ );
			if ( !has_show_transfer_button ) {
				return false;
			}
			return true;
		}
		else { // is TAKEN
			var has_permission = this.parent.has_permission_to_action( interaction, this.parent.TRANSFER );
			if ( !has_permission ) {
				return false;
			}
			campaign_id = interaction.campaign;
			// contra el layout para saber si puede visualizarlo
			has_show_transfer_button = inConcert.permissions.to_action( 'supervisor', 'sup_monitoring_transfer', undefined /* category*/ , campaign_id, true /* no log */ );
			if ( !has_show_transfer_button ) {
				return false;
			}
			return true;
		}
	};

	supervisor_generic_buttons.prototype.is_visible_reopen = function(interaction, not_must_validate_custom_action)
	{
		if ( !interaction.isArchived() ) {
			return false;
		}
		//Si la interaccion tiene relacionada una nueva interaccion que se ha creado a partir de esta, NO visualizamos la accion.
		if (interaction.relatedInteractions) {
			if (interaction.relatedInteractions.newInteraction) {
				return false;
			}
		}
		var p = this.parent;
		if (!not_must_validate_custom_action) {
			var is_visible_custom_action = p.is_visible_custom_action('reopen');
			if (!is_visible_custom_action) {
				return false;
			}
		}
		var has_reopen_permissions = p.has_permission_on_campaign( interaction, 'REOPEN' ); // contra conversation management
		if ( !has_reopen_permissions ) {
			return false;
		}
		var campaign_id = interaction.campaign;
		// contra el layout para saber si puede visualizarlo
		var has_show_reopen_button = inConcert.permissions.to_action( 'supervisor', 'sup_archive_reopen', undefined /* category*/ , campaign_id, true /* no log */ );
		if ( !has_show_reopen_button ) {
			return false;
		}
		return true;
	};

	supervisor_generic_buttons.prototype.is_visible_take = function(interaction, not_must_validate_custom_action)
	{
		if (interaction.isAllegro()) {
			return false;
		}
		if ( !interaction.isQueued() ) {
			return false;
		}
		if (!not_must_validate_custom_action) {
			var is_visible_custom_action = this.parent.is_visible_custom_action('take');
			if (!is_visible_custom_action) {
				return false;
			}
		}
		var allowed_app_interaction = inConcert.permissions.to_application('interaction'); // si no tiene acceso a la app interaction, no lo visualiza
		if ( !allowed_app_interaction ) {
			return false;
		}
		var p = this.parent;
		var has_taken_permissions = p.has_permission_to_action( interaction, p.TAKE ); // contra conversation management
		if ( !has_taken_permissions ) {
			return false;
		}
		var campaign_id = interaction.campaign;
		// contra el layout para saber si puede visualizarlo
		var has_show_taken_button_perm = inConcert.permissions.to_action('supervisor', 'sup_queued_take', undefined /* category*/ , campaign_id, true /* no log */);
		if ( !has_show_taken_button_perm ) {
			return false;
		}
		var campaign = inConcert.app.getCampaign(campaign_id);
		if ( !campaign ) {
			return false;
		}
		var user_id = inConcert.app.getLoggedUserId();
		var user_belongs_some_attention_level = campaign.userBelongsSomeAttentionLevel(user_id);
		var configured_al_in_interaction = interaction.hasAttentionLevel();
		// Si el usuario tiene pertenece a un AL en las campanas y la interaccion NO pertenece a algun AL, no se visualiza la opcion.
		//  y viceversa
		if ( user_belongs_some_attention_level !== configured_al_in_interaction ) {
			return false;
		}
		// Si la interaccion pertenece a un AL, valido si el usuario pertenece al mismo AL
		if ( configured_al_in_interaction ) {
			var attention_level = interaction.getAttentionLevel();
			if ( !campaign.userBelongsToAttentionLevel(user_id, attention_level.id) ) {
				// si no pertenece al mismo AL que la interaccion, no se visualiza la opcion.
			return false;
			}
		}
		return true;
	};

	supervisor_generic_buttons.prototype.is_visible_take_bot_interaction = function(interaction)
	{
		if (interaction.state !== "CREATED" || !interaction.interactionData.internal_consultation_flag) {
			return false;
		}

		if (!interaction.isBotInternalConsultation()) {
			return false;
		}
		
		var user_id = inConcert.app.getLoggedUserId();
		
		if (interaction.interactionData.internal_consultation_owner && interaction.interactionData.internal_consultation_owner !== user_id) {
			return false;
		}

		return true;
	};

	supervisor_generic_buttons.prototype.is_visible_finish = function( /*interaction*/ )
	{
		return false;
	};

	supervisor_generic_buttons.prototype.is_visible_queue = function( /*interaction*/ )
	{
		return false;
	};

	supervisor_generic_buttons.prototype.is_visible_cancel = function( /*interaction*/ )
	{
		return false;
	};

	supervisor_generic_buttons.prototype.is_visible_comment = function(interaction, not_must_validate_custom_action)
	{
		if ( interaction.isBotInternalConsultation() || interaction.belongsToMe() ) {
			return false;
		}
		if (!not_must_validate_custom_action) {
			var is_visible_custom_action = this.parent.is_visible_custom_action('comment');
			if (!is_visible_custom_action) {
				return false;
			}
		}
		var has_permission = supervisor_generic_buttons.uber.is_visible_comment.call( this, interaction );
		if ( !has_permission ) {
			return false;
		}
		var campaign_id = interaction.campaign;
		// contra el layout para saber si puede visualizarlo
		var can_show_comment_button = inConcert.permissions.to_action( 'supervisor', 'sup_monitoring_comment', undefined /* category*/ , campaign_id, true /* no log */ );
		if ( !can_show_comment_button ) {
			return false;
		}
		return true;
	};

	supervisor_generic_buttons.prototype.is_visible_label = function( interaction )
	{
		var is_visible_custom_action = this.parent.is_visible_custom_action('label');
		if (!is_visible_custom_action) {
			return false;
		}
		if ( interaction.belongsToMe() || interaction.isBotInternalConsultation()) {
			return false;
		}
		var has_permission = supervisor_generic_buttons.uber.is_visible_label.call( this, interaction );
		if ( !has_permission ) {
			return false;
		}
		var campaign_id = interaction.campaign;
		// contra el layout para saber si puede visualizarlo
		var has_show_label_button = inConcert.permissions.to_action( 'supervisor', 'sup_monitoring_label', undefined /* category*/ , campaign_id, true /* no log */ );
		if ( !has_show_label_button ) {
			return false;
		}
		return true;
	};

	// execute actions ----
	//
	supervisor_generic_buttons.prototype.monitor = function( interaction )
	{
		if ( interaction.belongsToMe() ) {
			return false;
		}
		var p = this.parent;
		if ( !p.has_permission_on_campaign( interaction, p.MONITOR ) ) {
			var error_msg = format( _T("You have not permissions to monitoring conversations on {0} campaign."), interaction.campaign );
			showBannerMessage( error_msg );
			return false;
		}
		inConcert.app.getMainWindow().getPanel("AssignedInteractions").openInteractionClient( interaction, null, { readonly : true, monitor : true, from_app: p.app_id });
	};

	supervisor_generic_buttons.prototype.show_contact = function( interaction )
	{
		supervisor_generic_buttons.uber.show_contact.call( this, interaction );
		SwitchToApp("interaction");
	};

	//---------------------------------------------------------------------
	//
	function supervisor_call_buttons( params )
	{
		supervisor_call_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( supervisor_call_buttons, supervisor_generic_buttons );

	supervisor_call_buttons.prototype.is_visible_answer = function( /*interaction*/ )
	{
		return false;
	};

	supervisor_call_buttons.prototype.is_visible_hangup = function( /*interaction*/ )
	{
		return false;
	};

	supervisor_call_buttons.prototype.is_visible_redial = function( /*interaction*/ )
	{
		return false;
	};

	// execute actions ---
	supervisor_call_buttons.prototype.answer = function( /*interaction*/ ) { };
	supervisor_call_buttons.prototype.hangup = function( /*interaction*/ ) { };
	supervisor_call_buttons.prototype.redial = function( /*interaction*/ ) { };

	//---------------------------------------------------------------------
	//
	function supervisor_voicemail_buttons( params )
	{
		supervisor_voicemail_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( supervisor_voicemail_buttons, supervisor_generic_buttons );

	//---------------------------------------------------------------------
	//
	function supervisor_facebook_buttons( params )
	{
		supervisor_facebook_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( supervisor_facebook_buttons, supervisor_generic_buttons );
	
	//---------------------------------------------------------------------
	//
	function supervisor_instagram_buttons( params )
	{
		supervisor_instagram_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( supervisor_instagram_buttons, supervisor_generic_buttons );

	//---------------------------------------------------------------------
	//
	function supervisor_youtube_buttons( params )
	{
		supervisor_youtube_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( supervisor_youtube_buttons, supervisor_generic_buttons );

	//---------------------------------------------------------------------
	//
	function supervisor_appstore_buttons( params )
	{
		supervisor_appstore_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( supervisor_appstore_buttons, supervisor_generic_buttons );

	//---------------------------------------------------------------------
	//
	function supervisor_googleplay_buttons( params )
	{
		supervisor_googleplay_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( supervisor_googleplay_buttons, supervisor_generic_buttons );

	//---------------------------------------------------------------------
	//
	function supervisor_rcs_buttons( params )
	{
		supervisor_rcs_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( supervisor_rcs_buttons, supervisor_generic_buttons );

	//---------------------------------------------------------------------
	//
	function supervisor_sms_buttons( params )
	{
		supervisor_sms_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( supervisor_sms_buttons, supervisor_generic_buttons );
	
	//---------------------------------------------------------------------
	//
	function supervisor_whatsapp_buttons( params )
	{
		supervisor_whatsapp_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( supervisor_whatsapp_buttons, supervisor_generic_buttons );

	//---------------------------------------------------------------------
	//
	function supervisor_twitter_buttons( params )
	{
		supervisor_twitter_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( supervisor_twitter_buttons, supervisor_generic_buttons );

	//---------------------------------------------------------------------
	//
	function supervisor_webcontact_buttons( params )
	{
		supervisor_webcontact_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( supervisor_webcontact_buttons, supervisor_generic_buttons );

	//---------------------------------------------------------------------
	//
	function supervisor_mail_buttons( params )
	{
		supervisor_mail_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( supervisor_mail_buttons, supervisor_generic_buttons );

	//---------------------------------------------------------------------
	//
	function supervisor_im_buttons( params )
	{
		supervisor_im_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( supervisor_im_buttons, supervisor_generic_buttons );

	//---------------------------------------------------------------------
	//
	function supervisor_chat_buttons( params )
	{
		supervisor_chat_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( supervisor_chat_buttons, supervisor_generic_buttons );

	supervisor_chat_buttons.prototype.is_visible_hangup = function( interaction )
	{
		var is_visible_custom_action = this.parent.is_visible_custom_action('hangup');
		if (!is_visible_custom_action) {
			return false;
		}
		var has_permission = this.parent.has_permission_to_action( interaction, this.parent.HANGUP );
		if ( !has_permission ) {
			return false;
		}
		return ( LIVE(interaction) && interaction.belongsToMe() && interaction.state === "TAKEN" );
	};

	// execute actions
	//
	supervisor_chat_buttons.prototype.hangup = function( interaction )
	{
		var has_permission = this.parent.has_permission_to_action( interaction, this.parent.HANGUP );
		if ( !has_permission ) {
			return false;
		}
		var session = interaction.getSession();
		if ( !session || !session.close() ) {
			var error_msg = _T("Unable to hang up conversation. Conversation it's closed.");
			showBannerMessage( error_msg );
			return false;
		}
		//this code is for showing the spinner
		interaction.pending = true;
		inConcert.app.getMainWindow().getPanel("AssignedInteractions").updateInteractionNode(interaction);
	};

	//---------------------------------------------------------------------
	//
	function base_actions( params )
	{
		var self = this;

		this.params = params;

		this.COMMENT       = "COMMENT"      ;
		this.FINISH        = "FINISH"       ;
		this.HANGUP        = "HANGUP"       ;
		this.LABEL         = "LABEL"        ;
		this.REQUEUE       = "REQUEUE"      ;
		this.TRANSFER      = "TRANSFER"     ;
		this.CONFERENCE    = "CONFERENCE"   ;
		this.REOPEN        = "REOPEN"       ;
		this.TAKE          = "TAKE"         ;
		this.SCHEDULE      = "SCHEDULE"     ;
		this.TICKET_EMIT   = "TICKET_EMIT"  ;
		this.TICKET_SEARCH = "TICKET_SEARCH";
		// este es de gui del sup, pero tiene que estar aca porque sino me rompeo todo!!!
		this.MONITOR  = {
			id      : "sup_monitoring",
			app_id  : "supervisor",
			category: undefined
		};

		$.each( this.types || {}, function( type, instance ) {
			self.types[ type ] = new instance( { parent: self } );
		});

		this.custom_allowed_actions = this.params ? this.params.custom_allowed_actions : undefined;
		this.actions = {};
		this.initialize();
	}

	base_actions.prototype.get = function( interaction )
	{
		return this.actions[ interaction.type ];
	};

	base_actions.prototype.get_app_id = function()
	{
		return this.app_id;
	};

	base_actions.prototype.execute = function( action_id, interaction, params )
	{
		var actions_by_type = this.get( interaction );
		if ( !actions_by_type ) {
			return false;
		}
		var action = actions_by_type[ action_id ];
		if (!action) {
			return false;
		}
		if ( typeof action.execute !== 'function' ) {
			return false;
		}
		let callbacks;
		action.execute( interaction, callbacks, params );
		return true;
	};

	base_actions.prototype.is_visible_action = function(action_id, interaction, not_must_validate_custom_action)
	{
		var actions_by_type = this.get( interaction );
		if ( !actions_by_type ) {
			return false;
		}
		var action = actions_by_type[ action_id ];
		if (!action) {
			return false;
		}
		return typeof(action.visible) === "function" && action.visible(interaction, not_must_validate_custom_action) || action.visible === true;
	};

	base_actions.prototype.is_visible_custom_action = function(action_id)
	{
		if (typeof this.custom_allowed_actions !== 'object') {
			return true;
		}
		if (!this.custom_allowed_actions.hasOwnProperty(action_id)) {
			return false;
		}
		return this.custom_allowed_actions[action_id];
	};

	base_actions.prototype.count_visible_actions = function(interaction)
	{
		var actions_by_type = this.get(interaction);
		var counter = 0;
		$.each(actions_by_type || {}, function(_, action) {
			var is_visible = typeof(action.visible) === "function" && action.visible(interaction) || action.visible === true;
			if (is_visible) {
				counter++;
			}
		});
		return counter;
	};

	base_actions.prototype.has_permission_on_campaign = function( interaction, action )
	{
		if ( typeof interaction !== "object" || typeof interaction.campaign !== "string" || typeof interaction.type !== "string" ) {
			return false;
		}
		if ( typeof action === "string" ) {
			return interaction.hasPermission( action, this.app_id );
		}
		else {
			var app_id = action.app_id;
			var action_id = action.id;
			var category = action.category;
			var campaign_id = interaction.campaign;
			return inConcert.permissions.to_action( app_id, action_id, category, campaign_id, true /* no log */ );
		}
	};

	base_actions.prototype.has_permission_to_action = function( interaction, action_id )
	{
		if ( typeof interaction !== "object" || typeof interaction.campaign !== "string" || typeof interaction.type !== "string" ) {
			return false;
		}
		var hasPermission = this.has_permission_on_campaign( interaction, action_id );
		if ( !hasPermission ) {
			return false;
		}
		// validate integration override action on attr "campaign.Integration"
		
		var hasAction = interaction.hasIntegrationOverrideActionPopupShow(action_id);
		if (hasAction) {
			return false;
		}
		return true;
	};

	base_actions.prototype.initialize = function()
	{
		this.initialize_call();
		this.initialize_voicemail();
		this.initialize_mail();
		this.initialize_chat();
		this.initialize_im();
		this.initialize_sms();
		this.initialize_whatsapp();
		this.initialize_facebook();
		this.initialize_instagram();
		this.initialize_youtube();
		this.initialize_appstore();
		this.initialize_googleplay();
		this.initialize_rcs();
		this.initialize_webcontact();
		this.initialize_internal_chat();
		this.initialize_twitter();
	};

	base_actions.prototype.initialize_call = function()
	{
		this.REDIAL = "REDIAL";

		var type = this.types.CALL;

		this.actions.CALL = {
			Redial : {
				tooltip: _T("Redial"),
				enabled: function( /*interaction*/ ) {
					return true;
				},
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_redial( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.redial( interaction, this, callbacks ); }
			},
			Hangup : {
				tooltip: function( /*interaction*/ ) {
					return _T("Hang Up");
				},
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_hangup( interaction, not_must_validate_custom_action ); },
				execute: function( interaction ) { type.hangup( interaction, this ); },
				enabled: function( /*interaction*/ ) {
					return true;
				}
			},
			Monitor : {
				tooltip: _T("Monitor"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_monitor( interaction, not_must_validate_custom_action ); },
				execute: function( interaction ) { type.monitor( interaction, this ); }
			},
			Queue : {
				tooltip: _T("Requeue"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_queue( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.requeue( interaction, this, callbacks ); }
			},
			Take : {
				tooltip: _T("Take"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_take( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.take( interaction, this, callbacks ); }
			},
			TakeBotInteraction : {
				tooltip: _T("TakeBotInteraction"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_take_bot_interaction( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.take( interaction, this, callbacks ); }
			},
			Transfer : {
				tooltip: _T("Transfer"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
				submenu: {
					TransferMember: {
						tooltip: _T("Member"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
						execute: function( interaction, callbacks ) {
							type.transfer( interaction, this, callbacks, {members: true} );
						}
					},
					TransferCampaign: {
						tooltip: _T("Campaign"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
						execute: function( interaction, callbacks ) {
							type.transfer( interaction, this, callbacks, {campaigns: true} );
						}
					},
					TransferNumber: {
						tooltip: _T("Number"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
						execute: function( interaction, callbacks ) {
							type.transfer( interaction, this, callbacks, {number: true} );
						}
					},
					TransferAttentionLevel: {
						tooltip: _T("Attention Level"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) { 
							if (inConcert.app.has_allegro_integration()) {
								return false;
							}
							return type.is_visible_transfer( interaction, not_must_validate_custom_action ); 
						},
						execute: function( interaction, callbacks ) {
							type.transfer( interaction, this, callbacks, {attention_levels: true} );
						}
					},
					TransferBot: {
						tooltip: _T("Process"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) { 
							if (inConcert.app.has_allegro_integration()) {
								return false;
							}
							return type.is_visible_transfer( interaction, not_must_validate_custom_action ); 
						},
						execute: function( interaction, callbacks ) {
							type.transfer( interaction, this, callbacks, {bots: true} );
						}
					}
				}
			},
			Finish : {
				tooltip: _T("Finish"),
				enabled: function( /*interaction*/ ) {
					return true;
				},
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_finish( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.finish( interaction, this, callbacks ); }
			},
			Conference : {
				tooltip: _T("Conference"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_conference( interaction, not_must_validate_custom_action ); },
				is_visible_only_in_toolbar: function() {
					if (!type.is_visible_conference_only_in_toolbar) {
						return false;
					}
					return type.is_visible_conference_only_in_toolbar();
				},
				submenu: {
					ConferenceMember: {
						tooltip: _T("Member"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) {
							if (inConcert.app.has_allegro_integration()) {
								return false;
							}
							return type.is_visible_conference( interaction, not_must_validate_custom_action ); 
						},
						execute: function( interaction, callbacks ) {
							type.conference( interaction, this, callbacks, {members: true} );
						}
					},
					ConferenceNumber: {
						tooltip: _T("Number"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_conference( interaction, not_must_validate_custom_action ); },
						execute: function( interaction, callbacks ) {
							type.conference( interaction, this, callbacks, {number: true} );
						}
					}
				}
			},
			Cancel : {
				tooltip: _T("Cancel"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_cancel( interaction, not_must_validate_custom_action ); },
				execute: CancelInteraction
			},
			Answer : {
				tooltip: _T("Answer"),
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_answer( interaction, not_must_validate_custom_action ); },
				execute: function( interaction ) { type.answer( interaction, this ); },
				enabled: function( /*interaction*/ ) {
					return true;
				}
			},
			Schedule : {
				tooltip: _T("Schedule"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_schedule( interaction, not_must_validate_custom_action ); },
				is_visible_only_in_toolbar: function() {
					if (!type.is_visible_schedule_only_in_toolbar) {
						return false;
					}
					return type.is_visible_schedule_only_in_toolbar();
				},
				execute: function( interaction, callbacks, params ) {
					let node;
					if (params && params.positionNode) {
						node = {
							positionNode: params.positionNode
						};
					}
					else {
						node = this;
					}
					type.schedule( interaction, node, callbacks );
				}
			},
			AddComment : {
				tooltip: _T("Add Comment"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_comment( interaction, not_must_validate_custom_action ); },
				execute : function( interaction, callbacks ) { type.comment( interaction, this, callbacks ); }
			},
			Label : {
				tooltip: _T("Assign Label"),
				enabled: Not(BeingComposed),
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_label( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.label( interaction, this, callbacks ); }
			},
			EmitTicket : {
				tooltip: _T("Emit Ticket"),
				enabled: Not(BeingComposed),
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_emit_ticket( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.emit_ticket( interaction, this, callbacks ); }
			},
			SearchTicket : {
				tooltip: _T("Search Ticket"),
				enabled: Not(BeingComposed),
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_search_ticket( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.search_ticket( interaction, this, callbacks ); }
			}
		};
	};

	base_actions.prototype.initialize_voicemail = function()
	{
		var type = this.types.VOICEMAIL;

		this.actions.VOICEMAIL = {
			Monitor : {
				tooltip: _T("Monitor"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_monitor( interaction, not_must_validate_custom_action ); },
				execute: function( interaction ) { type.monitor( interaction, this ); }
			},
			Queue : {
				tooltip: _T("Requeue"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_queue( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.requeue( interaction, this, callbacks ); }
			},
			Take : {
				tooltip: _T("Take"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_take( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.take( interaction, this, callbacks ); }
			},
			Transfer : {
				tooltip: _T("Transfer"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
				submenu: {
					TransferMember: {
						tooltip: _T("Member"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
						execute: function( interaction, callbacks ) {
							type.transfer( interaction, this, callbacks, {members: true} );
						}
					},
					TransferCampaign: {
						tooltip: _T("Campaign"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
						execute: function( interaction, callbacks ) {
							type.transfer( interaction, this, callbacks, {campaigns: true} );
						}
					},
					TransferAttentionLevel: {
						tooltip: _T("Attention Level"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) {
							if (inConcert.app.has_allegro_integration()) {
								return false;
							}
							return type.is_visible_transfer( interaction, not_must_validate_custom_action ); 
						},
						execute: function( interaction, callbacks ) {
							type.transfer( interaction, this, callbacks, {attention_levels: true} );
						}
					}
				}
			},
			Finish : {
				tooltip: _T("Finish"),
				enabled: function( /*interaction*/ ) {
					return true;
				},
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_finish( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.finish( interaction, this, callbacks ); }
			},
			Cancel : {
				tooltip: _T("Cancel"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_cancel( interaction, not_must_validate_custom_action ); },
				execute: CancelInteraction
			},
			Schedule : {
				tooltip: _T("Schedule"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_schedule( interaction, not_must_validate_custom_action ); },
				is_visible_only_in_toolbar: function() {
					if (!type.is_visible_schedule_only_in_toolbar) {
						return false;
					}
					return type.is_visible_schedule_only_in_toolbar();
				},
				execute: function( interaction, callbacks, params ) {
					let node;
					if (params && params.positionNode) {
						node = {
							positionNode: params.positionNode
						};
					}
					else {
						node = this;
					}
					type.schedule( interaction, node, callbacks );
				}
			},
			AddComment : {
				tooltip: _T("Add Comment"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_comment( interaction, not_must_validate_custom_action ); },
				execute : function( interaction, callbacks ) { type.comment( interaction, this, callbacks ); }
			},
			Label : {
				tooltip: _T("Assign Label"),
				enabled: Not(BeingComposed),
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_label( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.label( interaction, this, callbacks ); }
			},
			EmitTicket : {
				tooltip: _T("Emit Ticket"),
				enabled: Not(BeingComposed),
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_emit_ticket( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.emit_ticket( interaction, this, callbacks ); }
			},
			SearchTicket : {
				tooltip: _T("Search Ticket"),
				enabled: Not(BeingComposed),
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_search_ticket( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.search_ticket( interaction, this, callbacks ); }
			}
		};
		
		if (inConcert.app.has_allegro_integration()) {
			this.actions.VOICEMAIL.Answer = {
				tooltip: _T("Answer"),
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_answer( interaction, not_must_validate_custom_action ); },
				execute: function( interaction ) { type.answer( interaction, this ); },
				enabled: function( /*interaction*/ ) {
					return true;
				}
			};
			this.actions.VOICEMAIL.Hangup = {
				tooltip: function( /*interaction*/ ) {
					return _T("Close");
				},
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_hangup( interaction, not_must_validate_custom_action ); },
				execute: function( interaction ) { type.hangup( interaction, this ); },
				enabled: function( /*interaction*/ ) {
					return true;
				}
			};
		}
	};

	base_actions.prototype.initialize_mail = function()
	{
		var type = this.types.MAIL;

		this.actions.MAIL = {
			Monitor : {
				tooltip: _T("Monitor"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_monitor( interaction, not_must_validate_custom_action ); },
				execute: function( interaction ) { type.monitor( interaction, this ); }
			},
			Queue : {
				tooltip: _T("Requeue"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_queue( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.requeue( interaction, this, callbacks ); }
			},
			Take : {
				tooltip: _T("Take"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_take( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.take( interaction, this, callbacks ); }
			},
			Transfer : {
				tooltip: _T("Transfer"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
				submenu: {
					TransferMember: {
						tooltip: _T("Member"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
						execute: function( interaction, callbacks ) {
							type.transfer( interaction, this, callbacks, {members: true} );
						}
					},
					TransferCampaign: {
						tooltip: _T("Campaign"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
						execute: function( interaction, callbacks ) {
							type.transfer( interaction, this, callbacks, {campaigns: true} );
						}
					},
					TransferAttentionLevel: {
						tooltip: _T("Attention Level"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
						execute: function( interaction, callbacks ) {
							type.transfer( interaction, this, callbacks, {attention_levels: true} );
						}
					},
					TransferBot: {
						tooltip: _T("Process"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
						execute: function( interaction, callbacks ) {
							type.transfer( interaction, this, callbacks, {bots: true} );
						}
					}
				}
			},
			Finish : {
				tooltip: _T("Finish"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_finish( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.finish( interaction, this, callbacks ); }
			},
			Cancel : {
				tooltip: _T("Cancel"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_cancel( interaction, not_must_validate_custom_action ); },
				execute: CancelInteraction
			},
			AddComment : {
				tooltip: _T("Add Comment"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_comment( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.comment( interaction, this, callbacks ); }
			},
			Label : {
				tooltip: _T("Assign Label"),
				enabled: Not(BeingComposed),
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_label( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.label( interaction, this, callbacks ); }
			},
			Reopen : {
				tooltip: _T("Reopen"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_reopen( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.reopen( interaction, this, callbacks ); }
			},
			EmitTicket : {
				tooltip: _T("Emit Ticket"),
				enabled: Not(BeingComposed),
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_emit_ticket( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.emit_ticket( interaction, this, callbacks ); }
			},
			SearchTicket : {
				tooltip: _T("Search Ticket"),
				enabled: Not(BeingComposed),
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_search_ticket( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.search_ticket( interaction, this, callbacks ); }
			}
		};
	};

	base_actions.prototype.initialize_chat = function()
	{
		var type = this.types.CHAT;

		this.actions.CHAT = {
			Hangup : {
				tooltip: function( /*interaction*/ ) {
					return _T("Hang Up");
				},
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_hangup( interaction, not_must_validate_custom_action ); },
				execute: function( interaction ) { return type.hangup( interaction ); },
				enabled: true
			},
			Monitor : {
				tooltip: _T("Monitor"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_monitor( interaction, not_must_validate_custom_action ); },
				execute: function( interaction ) { type.monitor( interaction, this ); }
			},
			Queue : {
				tooltip: _T("Requeue"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_queue( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.requeue( interaction, this, callbacks ); }
			},
			Take : {
				tooltip: _T("Take"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_take( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.take( interaction, this, callbacks ); }
			},
			Transfer : {
				tooltip: _T("Transfer"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
				submenu: {
					TransferMember: {
						tooltip: _T("Member"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
						execute: function( interaction, callbacks ) {
							type.transfer( interaction, this, callbacks, {members: true} );
						}
					},
					TransferCampaign: {
						tooltip: _T("Campaign"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
						execute: function( interaction, callbacks ) {
							type.transfer( interaction, this, callbacks, {campaigns: true} );
						}
					},
					TransferAttentionLevel: {
						tooltip: _T("Attention Level"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
						execute: function( interaction, callbacks ) {
							type.transfer( interaction, this, callbacks, {attention_levels: true} );
						}
					},
					TransferBot: {
						tooltip: _T("Process"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
						execute: function( interaction, callbacks ) {
							type.transfer( interaction, this, callbacks, {bots: true} );
						}
					}
				}
			},
			Finish : {
				tooltip: _T("Finish"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_finish( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.finish( interaction, this, callbacks ); }
			},
			Cancel : {
				tooltip: _T("Cancel"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_cancel( interaction, not_must_validate_custom_action ); },
				execute: CancelInteraction
			},
			AddComment : {
				tooltip: _T("Add Comment"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_comment( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.comment( interaction, this, callbacks ); }
			},
			Label : {
				tooltip: _T("Assign Label"),
				enabled: Not(BeingComposed),
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_label( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.label( interaction, this, callbacks ); }
			},
			EmitTicket : {
				tooltip: _T("Emit Ticket"),
				enabled: Not(BeingComposed),
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_emit_ticket( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.emit_ticket( interaction, this, callbacks ); }
			},
			SearchTicket : {
				tooltip: _T("Search Ticket"),
				enabled: Not(BeingComposed),
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_search_ticket( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.search_ticket( interaction, this, callbacks ); }
			}
		};
	};

	base_actions.prototype.initialize_im = function()
	{
		var type = this.types.IM;

		this.actions.IM = {
			Monitor : {
				tooltip: _T("Monitor"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_monitor( interaction, not_must_validate_custom_action ); },
				execute: function( interaction ) { type.monitor( interaction, this ); }
			},
			Queue : {
				tooltip: _T("Requeue"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_queue( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.requeue( interaction, this, callbacks ); }
			},
			Take : {
				tooltip: _T("Take"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_take( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.take( interaction, this, callbacks ); }
			},
			Transfer : {
				tooltip: _T("Transfer"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
				submenu: {
					TransferMember: {
						tooltip: _T("Member"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
						execute: function( interaction, callbacks ) {
							type.transfer( interaction, this, callbacks, {members: true} );
						}
					},
					TransferCampaign: {
						tooltip: _T("Campaign"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
						execute: function( interaction, callbacks ) {
							type.transfer( interaction, this, callbacks, {campaigns: true} );
						}
					},
					TransferAttentionLevel: {
						tooltip: _T("Attention Level"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
						execute: function( interaction, callbacks ) {
							type.transfer( interaction, this, callbacks, {attention_levels: true} );
						}
					}
				}
			},
			Finish : {
				tooltip: _T("Finish"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_finish( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.finish( interaction, this, callbacks ); }
			},
			Cancel : {
				tooltip: _T("Cancel"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_cancel( interaction, not_must_validate_custom_action ); },
				execute: CancelInteraction
			},
			AddComment : {
				tooltip: _T("Add Comment"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_comment( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.comment( interaction, this, callbacks ); }
			},
			Label : {
				tooltip: _T("Assign Label"),
				enabled: Not(BeingComposed),
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_label( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.label( interaction, this, callbacks ); }
			},
			EmitTicket : {
				tooltip: _T("Emit Ticket"),
				enabled: Not(BeingComposed),
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_emit_ticket( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.emit_ticket( interaction, this, callbacks ); }
			},
			SearchTicket : {
				tooltip: _T("Search Ticket"),
				enabled: Not(BeingComposed),
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_search_ticket( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.search_ticket( interaction, this, callbacks ); }
			}
		};
	};

	base_actions.prototype.initialize_sms = function()
	{
		var type = this.types.SMS;

		this.actions.SMS = {
			Monitor : {
				tooltip: _T("Monitor"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_monitor( interaction, not_must_validate_custom_action ); },
				execute: function( interaction ) { type.monitor( interaction, this ); }
			},
			Queue : {
				tooltip: _T("Requeue"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_queue( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.requeue( interaction, this, callbacks ); }
			},
			Take : {
				tooltip: _T("Take"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_take( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.take( interaction, this, callbacks ); }
			},
			Transfer : {
				tooltip: _T("Transfer"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
				submenu: {
					TransferMember: {
						tooltip: _T("Member"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
						execute: function( interaction, callbacks ) {
							type.transfer( interaction, this, callbacks, {members: true} );
						}
					},
					TransferCampaign: {
						tooltip: _T("Campaign"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
						execute: function( interaction, callbacks ) {
							type.transfer( interaction, this, callbacks, {campaigns: true} );
						}
					},
					TransferAttentionLevel: {
						tooltip: _T("Attention Level"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
						execute: function( interaction, callbacks ) {
							type.transfer( interaction, this, callbacks, {attention_levels: true} );
						}
					},
					TransferBot: {
						tooltip: _T("Process"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
						execute: function( interaction, callbacks ) {
							type.transfer( interaction, this, callbacks, {bots: true} );
						}
					}
				}
			},
			Finish : {
				tooltip: _T("Finish"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_finish( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.finish( interaction, this, callbacks ); }
			},
			Cancel : {
				tooltip: _T("Cancel"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_cancel( interaction, not_must_validate_custom_action ); },
				execute: CancelInteraction
			},
			AddComment : {
				tooltip: _T("Add Comment"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_comment( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.comment( interaction, this, callbacks ); }
			},
			Label : {
				tooltip: _T("Assign Label"),
				enabled: Not(BeingComposed),
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_label( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.label( interaction, this, callbacks ); }
			},
			Reopen : {
				tooltip: _T("Reopen"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_reopen( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.reopen( interaction, this, callbacks ); }
			},
			EmitTicket : {
				tooltip: _T("Emit Ticket"),
				enabled: Not(BeingComposed),
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_emit_ticket( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.emit_ticket( interaction, this, callbacks ); }
			},
			SearchTicket : {
				tooltip: _T("Search Ticket"),
				enabled: Not(BeingComposed),
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_search_ticket( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.search_ticket( interaction, this, callbacks ); }
			}
		};
	};

	base_actions.prototype.initialize_rcs = function()
	{
		var type = this.types.RCS;

		this.actions.RCS = {
			Monitor : {
				tooltip: _T("Monitor"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_monitor( interaction, not_must_validate_custom_action ); },
				execute: function( interaction ) { type.monitor( interaction, this ); }
			},
			Queue : {
				tooltip: _T("Requeue"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_queue( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.requeue( interaction, this, callbacks ); }
			},
			Take : {
				tooltip: _T("Take"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_take( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.take( interaction, this, callbacks ); }
			},
			Transfer : {
				tooltip: _T("Transfer"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
				submenu: {
					TransferMember: {
						tooltip: _T("Member"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
						execute: function( interaction, callbacks ) {
							type.transfer( interaction, this, callbacks, {members: true} );
						}
					},
					TransferCampaign: {
						tooltip: _T("Campaign"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
						execute: function( interaction, callbacks ) {
							type.transfer( interaction, this, callbacks, {campaigns: true} );
						}
					},
					TransferAttentionLevel: {
						tooltip: _T("Attention Level"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
						execute: function( interaction, callbacks ) {
							type.transfer( interaction, this, callbacks, {attention_levels: true} );
						}
					}
				}
			},
			Finish : {
				tooltip: _T("Finish"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_finish( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.finish( interaction, this, callbacks ); }
			},
			Cancel : {
				tooltip: _T("Cancel"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_cancel( interaction, not_must_validate_custom_action ); },
				execute: CancelInteraction
			},
			AddComment : {
				tooltip: _T("Add Comment"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_comment( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.comment( interaction, this, callbacks ); }
			},
			Label : {
				tooltip: _T("Assign Label"),
				enabled: Not(BeingComposed),
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_label( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.label( interaction, this, callbacks ); }
			},
			Reopen : {
				tooltip: _T("Reopen"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_reopen( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.reopen( interaction, this, callbacks ); }
			},
			EmitTicket : {
				tooltip: _T("Emit Ticket"),
				enabled: Not(BeingComposed),
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_emit_ticket( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.emit_ticket( interaction, this, callbacks ); }
			},
			SearchTicket : {
				tooltip: _T("Search Ticket"),
				enabled: Not(BeingComposed),
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_search_ticket( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.search_ticket( interaction, this, callbacks ); }
			}
		};
	};

	base_actions.prototype.initialize_whatsapp = function()
	{
		var type = this.types.WHATSAPP;

		this.actions.WHATSAPP = {
			Monitor : {
				tooltip: _T("Monitor"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_monitor( interaction, not_must_validate_custom_action ); },
				execute: function( interaction ) { type.monitor( interaction, this ); }
			},
			Queue : {
				tooltip: _T("Requeue"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_queue( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.requeue( interaction, this, callbacks ); }
			},
			Take : {
				tooltip: _T("Take"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_take( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.take( interaction, this, callbacks ); }
			},
			Transfer : {
				tooltip: _T("Transfer"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
				submenu: {
					TransferMember: {
						tooltip: _T("Member"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
						execute: function( interaction, callbacks ) {
							type.transfer( interaction, this, callbacks, {members: true} );
						}
					},
					TransferCampaign: {
						tooltip: _T("Campaign"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
						execute: function( interaction, callbacks ) {
							type.transfer( interaction, this, callbacks, {campaigns: true} );
						}
					},
					TransferAttentionLevel: {
						tooltip: _T("Attention Level"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
						execute: function( interaction, callbacks ) {
							type.transfer( interaction, this, callbacks, {attention_levels: true} );
						}
					},
					TransferBot: {
						tooltip: _T("Process"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
						execute: function( interaction, callbacks ) {
							type.transfer( interaction, this, callbacks, {bots: true} );
						}
					}
				}
			},
			Finish : {
				tooltip: _T("Finish"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_finish( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.finish( interaction, this, callbacks ); }
			},
			Cancel : {
				tooltip: _T("Cancel"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_cancel( interaction, not_must_validate_custom_action ); },
				execute: CancelInteraction
			},
			AddComment : {
				tooltip: _T("Add Comment"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_comment( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.comment( interaction, this, callbacks ); }
			},
			Label : {
				tooltip: _T("Assign Label"),
				enabled: Not(BeingComposed),
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_label( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.label( interaction, this, callbacks ); }
			},
			Reopen : {
				tooltip: _T("Reopen"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_reopen( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.reopen( interaction, this, callbacks ); }
			},
			EmitTicket : {
				tooltip: _T("Emit Ticket"),
				enabled: Not(BeingComposed),
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_emit_ticket( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.emit_ticket( interaction, this, callbacks ); }
			},
			SearchTicket : {
				tooltip: _T("Search Ticket"),
				enabled: Not(BeingComposed),
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_search_ticket( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.search_ticket( interaction, this, callbacks ); }
			}
		};
	};

	base_actions.prototype.initialize_facebook = function()
	{
		var type = this.types.FACEBOOK;

		this.actions.FACEBOOK = {
			Monitor : {
				tooltip: _T("Monitor"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_monitor( interaction, not_must_validate_custom_action ); },
				execute: function( interaction ) { type.monitor( interaction, this ); }
			},
			Queue : {
				tooltip: _T("Requeue"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_queue( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.requeue( interaction, this, callbacks ); }
			},
			Take : {
				tooltip: _T("Take"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_take( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.take( interaction, this, callbacks ); }
			},
			Transfer : {
				tooltip: _T("Transfer"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
				submenu: {
					TransferMember: {
						tooltip: _T("Member"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
						execute: function( interaction, callbacks ) {
							type.transfer( interaction, this, callbacks, {members: true} );
						}
					},
					TransferCampaign: {
						tooltip: _T("Campaign"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
						execute: function( interaction, callbacks ) {
							type.transfer( interaction, this, callbacks, {campaigns: true} );
						}
					},
					TransferAttentionLevel: {
						tooltip: _T("Attention Level"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
						execute: function( interaction, callbacks ) {
							type.transfer( interaction, this, callbacks, {attention_levels: true} );
						}
					},
					TransferBot: {
						tooltip: _T("Process"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
						execute: function( interaction, callbacks ) {
							type.transfer( interaction, this, callbacks, {bots: true} );
						}
					}
				}
			},
			Finish : {
				tooltip: _T("Finish"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_finish( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.finish( interaction, this, callbacks ); }
			},
			Cancel : {
				tooltip: _T("Cancel"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_cancel( interaction, not_must_validate_custom_action ); },
				execute: CancelInteraction
			},
			AddComment : {
				tooltip: _T("Add Comment"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_comment( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.comment( interaction, this, callbacks ); }
			},
			Label : {
				tooltip: _T("Assign Label"),
				enabled: Not(BeingComposed),
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_label( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.label( interaction, this, callbacks ); }
			},
			Reopen : {
				tooltip: _T("Reopen"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_reopen( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.reopen( interaction, this, callbacks ); }
			},
			EmitTicket : {
				tooltip: _T("Emit Ticket"),
				enabled: Not(BeingComposed),
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_emit_ticket( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.emit_ticket( interaction, this, callbacks ); }
			},
			SearchTicket : {
				tooltip: _T("Search Ticket"),
				enabled: Not(BeingComposed),
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_search_ticket( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.search_ticket( interaction, this, callbacks ); }
			}
		};
	};

	base_actions.prototype.initialize_instagram = function()
	{
		var type = this.types.INSTAGRAM;

		this.actions.INSTAGRAM = {
			Monitor : {
				tooltip: _T("Monitor"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_monitor( interaction, not_must_validate_custom_action ); },
				execute: function( interaction ) { type.monitor( interaction, this ); }
			},
			Queue : {
				tooltip: _T("Requeue"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_queue( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.requeue( interaction, this, callbacks ); }
			},
			Take : {
				tooltip: _T("Take"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_take( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.take( interaction, this, callbacks ); }
			},
			Transfer : {
				tooltip: _T("Transfer"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
				submenu: {
					TransferMember: {
						tooltip: _T("Member"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
						execute: function( interaction, callbacks ) {
							type.transfer( interaction, this, callbacks, {members: true} );
						}
					},
					TransferCampaign: {
						tooltip: _T("Campaign"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
						execute: function( interaction, callbacks ) {
							type.transfer( interaction, this, callbacks, {campaigns: true} );
						}
					},
					TransferAttentionLevel: {
						tooltip: _T("Attention Level"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
						execute: function( interaction, callbacks ) {
							type.transfer( interaction, this, callbacks, {attention_levels: true} );
						}
					}
				}
			},
			Finish : {
				tooltip: _T("Finish"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_finish( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.finish( interaction, this, callbacks ); }
			},
			Cancel : {
				tooltip: _T("Cancel"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_cancel( interaction, not_must_validate_custom_action ); },
				execute: CancelInteraction
			},
			AddComment : {
				tooltip: _T("Add Comment"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_comment( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.comment( interaction, this, callbacks ); }
			},
			Label : {
				tooltip: _T("Assign Label"),
				enabled: Not(BeingComposed),
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_label( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.label( interaction, this, callbacks ); }
			},
			Reopen : {
				tooltip: _T("Reopen"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_reopen( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.reopen( interaction, this, callbacks ); }
			},
			EmitTicket : {
				tooltip: _T("Emit Ticket"),
				enabled: Not(BeingComposed),
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_emit_ticket( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.emit_ticket( interaction, this, callbacks ); }
			},
			SearchTicket : {
				tooltip: _T("Search Ticket"),
				enabled: Not(BeingComposed),
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_search_ticket( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.search_ticket( interaction, this, callbacks ); }
			}
		};
	};

	base_actions.prototype.initialize_youtube = function()
	{
		var type = this.types.YOUTUBE;

		this.actions.YOUTUBE = {
			Monitor : {
				tooltip: _T("Monitor"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_monitor( interaction, not_must_validate_custom_action ); },
				execute: function( interaction ) { type.monitor( interaction, this ); }
			},
			Queue : {
				tooltip: _T("Requeue"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_queue( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.requeue( interaction, this, callbacks ); }
			},
			Take : {
				tooltip: _T("Take"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_take( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.take( interaction, this, callbacks ); }
			},
			Transfer : {
				tooltip: _T("Transfer"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
				submenu: {
					TransferMember: {
						tooltip: _T("Member"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
						execute: function( interaction, callbacks ) {
							type.transfer( interaction, this, callbacks, {members: true} );
						}
					},
					TransferCampaign: {
						tooltip: _T("Campaign"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
						execute: function( interaction, callbacks ) {
							type.transfer( interaction, this, callbacks, {campaigns: true} );
						}
					},
					TransferAttentionLevel: {
						tooltip: _T("Attention Level"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
						execute: function( interaction, callbacks ) {
							type.transfer( interaction, this, callbacks, {attention_levels: true} );
						}
					}
				}
			},
			Finish : {
				tooltip: _T("Finish"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_finish( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.finish( interaction, this, callbacks ); }
			},
			Cancel : {
				tooltip: _T("Cancel"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_cancel( interaction, not_must_validate_custom_action ); },
				execute: CancelInteraction
			},
			AddComment : {
				tooltip: _T("Add Comment"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_comment( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.comment( interaction, this, callbacks ); }
			},
			Label : {
				tooltip: _T("Assign Label"),
				enabled: Not(BeingComposed),
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_label( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.label( interaction, this, callbacks ); }
			},
			Reopen : {
				tooltip: _T("Reopen"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_reopen( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.reopen( interaction, this, callbacks ); }
			},
			EmitTicket : {
				tooltip: _T("Emit Ticket"),
				enabled: Not(BeingComposed),
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_emit_ticket( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.emit_ticket( interaction, this, callbacks ); }
			},
			SearchTicket : {
				tooltip: _T("Search Ticket"),
				enabled: Not(BeingComposed),
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_search_ticket( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.search_ticket( interaction, this, callbacks ); }
			}
		};
	};

	base_actions.prototype.initialize_appstore = function()
	{
		var type = this.types.APPSTORE;

		this.actions.APPSTORE = {
			Monitor : {
				tooltip: _T("Monitor"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_monitor( interaction, not_must_validate_custom_action ); },
				execute: function( interaction ) { type.monitor( interaction, this ); }
			},
			Queue : {
				tooltip: _T("Requeue"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_queue( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.requeue( interaction, this, callbacks ); }
			},
			Take : {
				tooltip: _T("Take"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_take( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.take( interaction, this, callbacks ); }
			},
			Transfer : {
				tooltip: _T("Transfer"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
				submenu: {
					TransferMember: {
						tooltip: _T("Member"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
						execute: function( interaction, callbacks ) {
							type.transfer( interaction, this, callbacks, {members: true} );
						}
					},
					TransferCampaign: {
						tooltip: _T("Campaign"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
						execute: function( interaction, callbacks ) {
							type.transfer( interaction, this, callbacks, {campaigns: true} );
						}
					},
					TransferAttentionLevel: {
						tooltip: _T("Attention Level"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
						execute: function( interaction, callbacks ) {
							type.transfer( interaction, this, callbacks, {attention_levels: true} );
						}
					}
				}
			},
			Finish : {
				tooltip: _T("Finish"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_finish( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.finish( interaction, this, callbacks ); }
			},
			Cancel : {
				tooltip: _T("Cancel"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_cancel( interaction, not_must_validate_custom_action ); },
				execute: CancelInteraction
			},
			AddComment : {
				tooltip: _T("Add Comment"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_comment( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.comment( interaction, this, callbacks ); }
			},
			Label : {
				tooltip: _T("Assign Label"),
				enabled: Not(BeingComposed),
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_label( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.label( interaction, this, callbacks ); }
			},
			Reopen : {
				tooltip: _T("Reopen"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_reopen( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.reopen( interaction, this, callbacks ); }
			},
			EmitTicket : {
				tooltip: _T("Emit Ticket"),
				enabled: Not(BeingComposed),
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_emit_ticket( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.emit_ticket( interaction, this, callbacks ); }
			},
			SearchTicket : {
				tooltip: _T("Search Ticket"),
				enabled: Not(BeingComposed),
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_search_ticket( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.search_ticket( interaction, this, callbacks ); }
			}
		};
	};

	base_actions.prototype.initialize_googleplay = function()
	{
		var type = this.types.GOOGLEPLAY;

		this.actions.GOOGLEPLAY = {
			Monitor : {
				tooltip: _T("Monitor"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_monitor( interaction, not_must_validate_custom_action ); },
				execute: function( interaction ) { type.monitor( interaction, this ); }
			},
			Queue : {
				tooltip: _T("Requeue"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_queue( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.requeue( interaction, this, callbacks ); }
			},
			Take : {
				tooltip: _T("Take"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_take( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.take( interaction, this, callbacks ); }
			},
			Transfer : {
				tooltip: _T("Transfer"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
				submenu: {
					TransferMember: {
						tooltip: _T("Member"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
						execute: function( interaction, callbacks ) {
							type.transfer( interaction, this, callbacks, {members: true} );
						}
					},
					TransferCampaign: {
						tooltip: _T("Campaign"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
						execute: function( interaction, callbacks ) {
							type.transfer( interaction, this, callbacks, {campaigns: true} );
						}
					},
					TransferAttentionLevel: {
						tooltip: _T("Attention Level"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
						execute: function( interaction, callbacks ) {
							type.transfer( interaction, this, callbacks, {attention_levels: true} );
						}
					}
				}
			},
			Finish : {
				tooltip: _T("Finish"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_finish( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.finish( interaction, this, callbacks ); }
			},
			Cancel : {
				tooltip: _T("Cancel"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_cancel( interaction, not_must_validate_custom_action ); },
				execute: CancelInteraction
			},
			AddComment : {
				tooltip: _T("Add Comment"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_comment( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.comment( interaction, this, callbacks ); }
			},
			Label : {
				tooltip: _T("Assign Label"),
				enabled: Not(BeingComposed),
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_label( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.label( interaction, this, callbacks ); }
			},
			Reopen : {
				tooltip: _T("Reopen"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_reopen( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.reopen( interaction, this, callbacks ); }
			},
			EmitTicket : {
				tooltip: _T("Emit Ticket"),
				enabled: Not(BeingComposed),
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_emit_ticket( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.emit_ticket( interaction, this, callbacks ); }
			},
			SearchTicket : {
				tooltip: _T("Search Ticket"),
				enabled: Not(BeingComposed),
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_search_ticket( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.search_ticket( interaction, this, callbacks ); }
			}
		};
	};

	base_actions.prototype.initialize_twitter = function()
	{
		var type = this.types.TWITTER;

		this.actions.TWITTER = {
			Monitor : {
				tooltip: _T("Monitor"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) {
					return type.is_visible_monitor( interaction, not_must_validate_custom_action );
				},
				execute: function( interaction ) {
					type.monitor( interaction, this );
				}
			},
			Queue : {
				tooltip: _T("Requeue"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) {
					return type.is_visible_queue( interaction, not_must_validate_custom_action );
				},
				execute: function( interaction, callbacks ) { type.requeue( interaction, this, callbacks ); }
			},
			Take : {
				tooltip: _T("Take"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_take( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.take( interaction, this, callbacks ); }
			},
			Transfer : {
				tooltip: _T("Transfer"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) {
					return type.is_visible_transfer( interaction, not_must_validate_custom_action );
				},
				submenu: {
					TransferMember: {
						tooltip: _T("Member"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
						execute: function( interaction, callbacks ) {
							type.transfer( interaction, this, callbacks, {members: true} );
						}
					},
					TransferCampaign: {
						tooltip: _T("Campaign"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
						execute: function( interaction, callbacks ) {
							type.transfer( interaction, this, callbacks, {campaigns: true} );
						}
					},
					TransferAttentionLevel: {
						tooltip: _T("Attention Level"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
						execute: function( interaction, callbacks ) {
							type.transfer( interaction, this, callbacks, {attention_levels: true} );
						}
					},
					TransferBot: {
						tooltip: _T("Process"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
						execute: function( interaction, callbacks ) {
							type.transfer( interaction, this, callbacks, {bots: true} );
						}
					}
				}
			},
			Finish : {
				tooltip: _T("Finish"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) {
					return type.is_visible_finish( interaction, not_must_validate_custom_action );
				},
				execute: function( interaction, callbacks ) { type.finish( interaction, this, callbacks ); }
			},
			Cancel : {
				tooltip: _T("Cancel"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) {
					return type.is_visible_cancel( interaction, not_must_validate_custom_action );
				},
				execute: CancelInteraction
			},
			AddComment : {
				tooltip: _T("Add Comment"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) {
					return type.is_visible_comment( interaction, not_must_validate_custom_action );
				},
				execute : function( interaction, callbacks ) { type.comment( interaction, this, callbacks ); }
			},
			Label : {
				tooltip: _T("Assign Label"),
				enabled: Not(BeingComposed),
				visible: function( interaction, not_must_validate_custom_action ) {
					return type.is_visible_label( interaction, not_must_validate_custom_action );
				},
				execute: function( interaction, callbacks ) { type.label( interaction, this, callbacks ); }
			},
			Reopen : {
				tooltip: _T("Reopen"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_reopen( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.reopen( interaction, this, callbacks ); }
			},
			EmitTicket : {
				tooltip: _T("Emit Ticket"),
				enabled: Not(BeingComposed),
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_emit_ticket( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.emit_ticket( interaction, this, callbacks ); }
			},
			SearchTicket : {
				tooltip: _T("Search Ticket"),
				enabled: Not(BeingComposed),
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_search_ticket( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.search_ticket( interaction, this, callbacks ); }
			}
		};
	};

	base_actions.prototype.initialize_webcontact = function()
	{
		var type = this.types.WEBCONTACT;

		this.actions.WEBCONTACT = {
			Monitor : {
				tooltip: _T("Monitor"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_monitor( interaction, not_must_validate_custom_action ); },
				execute: function( interaction ) { type.monitor( interaction, this ); }
			},
			Queue : {
				tooltip: _T("Requeue"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_queue( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.requeue( interaction, this, callbacks ); }
			},
			Take : {
				tooltip: _T("Take"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_take( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.take( interaction, this, callbacks ); }
			},
			Transfer : {
				tooltip: _T("Transfer"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
				submenu: {
					TransferMember: {
						tooltip: _T("Member"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
						execute: function( interaction, callbacks ) {
							type.transfer( interaction, this, callbacks, {members: true} );
						}
					},
					TransferCampaign: {
						tooltip: _T("Campaign"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
						execute: function( interaction, callbacks ) {
							type.transfer( interaction, this, callbacks, {campaigns: true} );
						}
					},
					TransferAttentionLevel: {
						tooltip: _T("Attention Level"),
						enabled: true,
						visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_transfer( interaction, not_must_validate_custom_action ); },
						execute: function( interaction, callbacks ) {
							type.transfer( interaction, this, callbacks, {attention_levels: true} );
						}
					}
				}
			},
			Finish : {
				tooltip: _T("Finish"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_finish( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.finish( interaction, this, callbacks ); }
			},
			Cancel : {
				tooltip: _T("Cancel"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_cancel( interaction, not_must_validate_custom_action ); },
				execute: CancelInteraction
			},
			AddComment : {
				tooltip: _T("Add Comment"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_comment( interaction, not_must_validate_custom_action ); },
				execute : function( interaction, callbacks ) { type.comment( interaction, this, callbacks ); }
			},
			Label : {
				tooltip: _T("Assign Label"),
				enabled: Not(BeingComposed),
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_label( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.label( interaction, this, callbacks ); }
			},
			Reopen : {
				tooltip: _T("Reopen"),
				enabled: true,
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_reopen( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.reopen( interaction, this, callbacks ); }
			},
			EmitTicket : {
				tooltip: _T("Emit Ticket"),
				enabled: Not(BeingComposed),
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_emit_ticket( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.emit_ticket( interaction, this, callbacks ); }
			},
			SearchTicket : {
				tooltip: _T("Search Ticket"),
				enabled: Not(BeingComposed),
				visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_search_ticket( interaction, not_must_validate_custom_action ); },
				execute: function( interaction, callbacks ) { type.search_ticket( interaction, this, callbacks ); }
			}
		};
	};

	base_actions.prototype.initialize_internal_chat = function()
	{
		this.actions.INTERNAL_CHAT = {
			Cancel : {
				tooltip: _T("Close"),
				enabled: true,
				visible: BeingComposed,
				execute: CancelInteraction
			}
		};
	};

	//---------------------------------------------------------------------
	// app "interaction" attention class
	function attention_actions( params )
	{
		this.app_id  = "interaction";

		this.types = {
			APPSTORE   : attention_appstore_buttons  ,
			CALL       : attention_call_buttons      ,
			CHAT       : attention_chat_buttons      ,
			IM         : attention_im_buttons        ,
			FACEBOOK   : attention_facebook_buttons  ,
			GOOGLEPLAY : attention_googleplay_buttons,
			INSTAGRAM  : attention_instagram_buttons ,
			MAIL       : attention_mail_buttons      ,
			RCS        : attention_rcs_buttons       ,
			SMS        : attention_sms_buttons       ,
			TWITTER    : attention_twitter_buttons   ,
			VOICEMAIL  : inConcert.app.has_allegro_integration()?
				attention_call_buttons : attention_voicemail_buttons,
			WEBCONTACT : attention_webcontact_buttons,
			WHATSAPP   : attention_whatsapp_buttons  ,
			YOUTUBE	   : attention_youtube_buttons
		};

		attention_actions.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( attention_actions, base_actions );

	if ( typeof inConcert.attention_actions !== "function" ) {
		inConcert.attention_actions = attention_actions;
	}

	//---------------------------------------------------------------------
	// app "supervisor" class
	function supervisor_actions( params )
	{
		this.app_id  = "supervisor";
		this.types = {
			APPSTORE   : supervisor_appstore_buttons  ,
			CALL       : supervisor_call_buttons      ,
			CHAT       : supervisor_chat_buttons      ,
			IM         : supervisor_im_buttons        ,
			FACEBOOK   : supervisor_facebook_buttons  ,
			GOOGLEPLAY : supervisor_googleplay_buttons,
			INSTAGRAM  : supervisor_instagram_buttons ,
			MAIL       : supervisor_mail_buttons      ,
			RCS        : supervisor_rcs_buttons       ,
			SMS        : supervisor_sms_buttons       ,
			TWITTER    : supervisor_twitter_buttons   ,
			VOICEMAIL  : inConcert.app.has_allegro_integration() ? supervisor_call_buttons : supervisor_voicemail_buttons,
			WEBCONTACT : supervisor_webcontact_buttons,
			WHATSAPP   : supervisor_whatsapp_buttons  ,
			YOUTUBE    : supervisor_youtube_buttons
		};

		supervisor_actions.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( supervisor_actions, base_actions );
	if ( typeof inConcert.supervisor_actions !== "function" ) {
		inConcert.supervisor_actions = supervisor_actions;
	}


	/*------------------------------------------------------------------------------------------
		Aca comienzan las clases y definiciones que se usan desde los menues contextuales en las diferentes
		ventanas de los clientes por App "INTERACTION" y "SUPERVISOR"
	------------------------------------------------------------------------------------------*/

	// -- INICIO CLASES APP INTERACTION CLASES CLIENTS WINDOWS ---------------------------------
	//

	/*--START WEBCHAT - APP INTERACTION -------------------------------------------------------------------
		WEBCHAT MENU ITEMS for CLIENT WINDOW - APP INTERACTION
	*/
	function client_attention_chat_buttons( params )
	{
		this.m_client_window = params.parent.m_client_window;
		client_attention_chat_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( client_attention_chat_buttons, attention_chat_buttons );

	client_attention_chat_buttons.prototype.is_visible_transfer_only_in_toolbar = function()
	{
		return false;
	};

	client_attention_chat_buttons.prototype.is_visible_attach_file = function(interaction, not_must_validate_custom_action)
	{
		if (!not_must_validate_custom_action) {
			var is_visible_custom_action = this.parent.is_visible_custom_action('attachfile');
			if (!is_visible_custom_action) {
				return false;
			}
		}
	
		const can_show_send_attach = this.m_client_window.canSendAttachment();
		const can_show_shared_file = this.m_client_window.canSendSharedFile();
		if (!can_show_send_attach && !can_show_shared_file) {
			return false;
		}

		return !BeingComposed( interaction ) &&
				interaction.state !== "WRAPUP" &&
				interaction.belongsToMe();
	};

	client_attention_chat_buttons.prototype.show_attach_file_menu = function(interaction, node, callbacks, popup_main_menu)
	{
		const on_attach = callbacks && callbacks.on_attach ? callbacks.on_attach : $.noop;
		const can_show_send_attach = this.m_client_window.canSendAttachment();
		const can_show_shared_file = this.m_client_window.canSendSharedFile();

		if (!can_show_send_attach && !can_show_shared_file) {
			const error_msg = _T("You cannot send files.");
			showBannerMessage(error_msg);
			on_attach(false, error_msg);
			return false;
		}

		const $position_node = node.positionNode;
		const options = {align: "LEFT", popup_main_menu: popup_main_menu};
		this.m_client_window.showFilesMenu($position_node, options);
		on_attach(true, interaction);
	};
	//--END WEBCHAT - APP INTERACTION---------------------------------------------------------------------------

	/*--START WHATSAPP - APP INTERACTION-------------------------------------------------------------------
		WHATSAPP MENU ITEMS for CLIENT WINDOW - APP INTERACTION
	*/
	function client_attention_whatsapp_buttons( params )
	{
		this.m_client_window = params.parent.m_client_window;
		client_attention_whatsapp_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( client_attention_whatsapp_buttons, attention_whatsapp_buttons );

	client_attention_whatsapp_buttons.prototype.is_visible_transfer_only_in_toolbar = function()
	{
		return false;
	};

	client_attention_whatsapp_buttons.prototype.is_visible_attach_file = function(interaction, not_must_validate_custom_action)
	{
		if (!not_must_validate_custom_action) {
			var is_visible_custom_action = this.parent.is_visible_custom_action('attachfile');
			if (!is_visible_custom_action) {
				return false;
			}
		}
	
		const can_show_send_attach = this.m_client_window.canSendAttachment();
		const can_show_shared_file = this.m_client_window.canSendSharedFile();
		if (!can_show_send_attach && !can_show_shared_file) {
			return false;
		}

		return !BeingComposed( interaction ) &&
				interaction.state !== "WRAPUP" &&
				interaction.belongsToMe();
	};

	client_attention_whatsapp_buttons.prototype.show_attach_file_menu = function(interaction, node, callbacks, popup_main_menu)
	{
		const on_attach = callbacks && callbacks.on_attach ? callbacks.on_attach : $.noop;
		const can_show_send_attach = this.m_client_window.canSendAttachment();
		const can_show_shared_file = this.m_client_window.canSendSharedFile();

		if (!can_show_send_attach && !can_show_shared_file) {
			const error_msg = _T("You cannot send files.");
			showBannerMessage(error_msg);
			on_attach(false, error_msg);
			return false;
		}

		const $position_node = node.positionNode;
		const options = {align: "LEFT", popup_main_menu: popup_main_menu};
		this.m_client_window.showFilesMenu($position_node, options);
		on_attach(true, interaction);
	};
	//--END WHATSAPP - APP INTERACTION---------------------------------------------------------------------------


	/*--START MAIL - APP INTERACTION-------------------------------------------------------------------
		MAIL MENU ITEMS for CLIENT WINDOW - APP INTERACTION
	*/
	function client_attention_mail_buttons( params )
	{
		this.m_client_window = params.parent.m_client_window;
		client_attention_mail_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( client_attention_mail_buttons, attention_mail_buttons );

	client_attention_mail_buttons.prototype.is_visible_transfer_only_in_toolbar = function()
	{
		return false;
	};

	client_attention_mail_buttons.prototype.is_visible_print = function(interaction, not_must_validate_custom_action)
	{
		if (!not_must_validate_custom_action) {
			var is_visible_custom_action = this.parent.is_visible_custom_action('print');
			if (!is_visible_custom_action) {
				return false;
			}
		}

		const is_composing_mail = this.m_client_window.is_composing_mail();
		if (is_composing_mail) {
			return false;
		}

		const can_show_print = this.m_client_window.allowedPrint();
		if (!can_show_print) {
			return false;
		}

		return true;
	};

	client_attention_mail_buttons.prototype.execute_print = function(interaction, node, callbacks, popup_main_menu, $button)
	{
		const on_print = callbacks && callbacks.on_print ? callbacks.on_print : $.noop;
		const can_show_print = this.m_client_window.allowedPrint();
	
		if (!can_show_print) {
			const error_msg = _T("You cannot print.");
			showBannerMessage(error_msg);
			on_print(false, error_msg);
			return false;
		}
		/*
			La opcion del menu print (link) tiene que ir por parametro, para que utilice este link para imprimir y le asigne el valor al href.

			En el attr "rel" asignamos el valor "noreferrer" para que el popup/tab que realiza la impresion, se ejecute en un thread diferente
			al que se ejecuta la la ventana padre.
			Con esto lo que logramos es que cuando se abre el dialogo de impresion del sistema, el flujo de la ventana donde correr i6 no se detenga,
			y el no exista desconexion por timeout contra el server.

			Referencia: http://stackoverflow.com/questions/39574000/how-to-open-a-new-tab-in-separate-thread-with-javascript-chrome
		*/
		this.m_client_window.print($button);
		on_print(true, interaction);
	};

	client_attention_mail_buttons.prototype.is_visible_attach_file = function(interaction, not_must_validate_custom_action)
	{
		if (!not_must_validate_custom_action) {
			var is_visible_custom_action = this.parent.is_visible_custom_action('attachfile');
			if (!is_visible_custom_action) {
				return false;
			}
		}
	
		const is_composing_mail = this.m_client_window.is_composing_mail();
		if (!is_composing_mail) {
			return false;
		}

		const can_show_send_attach = this.m_client_window.canSendAttachment();
		const can_show_shared_file = this.m_client_window.canSendSharedFile();
		if (!can_show_send_attach && !can_show_shared_file) {
			return false;
		}

		return !BeingComposed( interaction ) &&	interaction.belongsToMe();
	};

	client_attention_mail_buttons.prototype.show_attach_file_menu = function(interaction, node, callbacks, popup_main_menu)
	{
		const on_attach = callbacks && callbacks.on_attach ? callbacks.on_attach : $.noop;
		const can_show_send_attach = this.m_client_window.canSendAttachment();
		const can_show_shared_file = this.m_client_window.canSendSharedFile();

		if (!can_show_send_attach && !can_show_shared_file) {
			const error_msg = _T("You cannot send files.");
			showBannerMessage(error_msg);
			on_attach(false, error_msg);
			return false;
		}

		const $position_node = node.positionNode;
		const options = {align: "LEFT", popup_main_menu: popup_main_menu};
		this.m_client_window.showFilesMenu($position_node, options);
		on_attach(true, interaction);
	};

	//--END MAIL - APP INTERACTION---------------------------------------------------------------------------

	/*--START CALL - APP INTERACTION-------------------------------------------------------------------
		CALL MENU ITEMS for CLIENT WINDOW - APP INTERACTION
	*/
	function client_attention_call_buttons( params )
	{
		this.m_client_window = params.parent.m_client_window;
		client_attention_call_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( client_attention_call_buttons, attention_call_buttons );

	client_attention_call_buttons.prototype.is_visible_conference_only_in_toolbar = function()
	{
		return false;
	};

	client_attention_call_buttons.prototype.is_visible_transfer_only_in_toolbar = function()
	{
		return false;
	};

	client_attention_call_buttons.prototype.is_visible_schedule_only_in_toolbar = function()
	{
		return false;
	};

	//--END CALL - APP INTERACTION---------------------------------------------------------------------------

	/*--START VOICEMAIL - APP INTERACTION-------------------------------------------------------------------
		CALL MENU ITEMS for CLIENT WINDOW - APP INTERACTION
	*/
	function client_attention_voicemail_buttons( params )
	{
		this.m_client_window = params.parent.m_client_window;
		client_attention_voicemail_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( client_attention_voicemail_buttons, attention_voicemail_buttons );

	client_attention_voicemail_buttons.prototype.is_visible_conference_only_in_toolbar = function()
	{
		return false;
	};

	client_attention_voicemail_buttons.prototype.is_visible_transfer_only_in_toolbar = function()
	{
		return false;
	};

	client_attention_voicemail_buttons.prototype.is_visible_schedule_only_in_toolbar = function()
	{
		return false;
	};

	//--END VOICEMAIL - APP INTERACTION---------------------------------------------------------------------------

	/*--START IM - APP INTERACTION-------------------------------------------------------------------
		IM MENU ITEMS for CLIENT WINDOW - APP INTERACTION
	*/
	function client_attention_im_buttons( params )
	{
		this.m_client_window = params.parent.m_client_window;
		client_attention_im_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( client_attention_im_buttons, attention_im_buttons );

	client_attention_im_buttons.prototype.is_visible_transfer_only_in_toolbar = function()
	{
		return false;
	};
	//--END IM - APP INTERACTION---------------------------------------------------------------------------

	/*--START FACEBOOK - APP INTERACTION-------------------------------------------------------------------
		FACEBOOK MENU ITEMS for CLIENT WINDOW - APP INTERACTION
	*/
	function client_attention_facebook_buttons( params )
	{
		this.m_client_window = params.parent.m_client_window;
		client_attention_facebook_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( client_attention_facebook_buttons, attention_facebook_buttons );

	client_attention_facebook_buttons.prototype.is_visible_transfer_only_in_toolbar = function()
	{
		return false;
	};

	//--END FACEBOOK - APP INTERACTION---------------------------------------------------------------------------

	/*--START INSTAGRAM-------------------------------------------------------------------
		INSTAGRAM MENU ITEMS for CLIENT WINDOW - APP INTERACTION
	*/
	function client_attention_instagram_buttons( params )
	{
		this.m_client_window = params.parent.m_client_window;
		client_attention_instagram_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( client_attention_instagram_buttons, attention_instagram_buttons );

	client_attention_instagram_buttons.prototype.is_visible_transfer_only_in_toolbar = function()
	{
		return false;
	};

	//--END INSTAGRAM - APP INTERACTION---------------------------------------------------------------------------

	/*--START SMS-------------------------------------------------------------------
		SMS MENU ITEMS for CLIENT WINDOW - APP INTERACTION
	*/
	function client_attention_sms_buttons( params )
	{
		this.m_client_window = params.parent.m_client_window;
		client_attention_sms_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( client_attention_sms_buttons, attention_sms_buttons );

	client_attention_sms_buttons.prototype.is_visible_transfer_only_in_toolbar = function()
	{
		return false;
	};

	//--END SMS - APP INTERACTION---------------------------------------------------------------------------

	/*--START TWITTER - APP INTERACTION-------------------------------------------------------------------
		TWITTER MENU ITEMS for CLIENT WINDOW - APP INTERACTION
	*/
	function client_attention_twitter_buttons( params )
	{
		this.m_client_window = params.parent.m_client_window;
		client_attention_twitter_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( client_attention_twitter_buttons, attention_twitter_buttons );

	client_attention_twitter_buttons.prototype.is_visible_transfer_only_in_toolbar = function()
	{
		return false;
	};

	//--END TWITTER - APP INTERACTION---------------------------------------------------------------------------

	/*--START WEBCONTACT - APP INTERACTION-------------------------------------------------------------------
		WEBCONTACT MENU ITEMS for CLIENT WINDOW - APP INTERACTION
	*/
	function client_attention_webcontact_buttons( params )
	{
		this.m_client_window = params.parent.m_client_window;
		client_attention_webcontact_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( client_attention_webcontact_buttons, attention_webcontact_buttons );

	client_attention_webcontact_buttons.prototype.is_visible_transfer_only_in_toolbar = function()
	{
		return false;
	};

	//--END WEBCONTACT - APP INTERACTION---------------------------------------------------------------------------

	/*--START YOUTUBE - APP INTERACTION-------------------------------------------------------------------
		YOUTUBE MENU ITEMS for CLIENT WINDOW - APP INTERACTION
	*/
	function client_attention_youtube_buttons( params )
	{
		this.m_client_window = params.parent.m_client_window;
		client_attention_youtube_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( client_attention_youtube_buttons, attention_youtube_buttons );

	client_attention_youtube_buttons.prototype.is_visible_transfer_only_in_toolbar = function()
	{
		return false;
	};

	//--END YOUTUBE - APP INTERACTION---------------------------------------------------------------------------

	/*--START APPSTORE - APP INTERACTION-------------------------------------------------------------------
		APPSTORE MENU ITEMS for CLIENT WINDOW - APP INTERACTION
	*/
	function client_attention_appstore_buttons( params )
	{
		this.m_client_window = params.parent.m_client_window;
		client_attention_appstore_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( client_attention_appstore_buttons, attention_youtube_buttons );

	client_attention_appstore_buttons.prototype.is_visible_transfer_only_in_toolbar = function()
	{
		return false;
	};

	//--END APPSTORE - APP INTERACTION---------------------------------------------------------------------------

	/*--START GOOGLEPLAY - APP INTERACTION-------------------------------------------------------------------
		GOOGLEPLAY MENU ITEMS for CLIENT WINDOW - APP INTERACTION
	*/
	function client_attention_googleplay_buttons( params )
	{
		this.m_client_window = params.parent.m_client_window;
		client_attention_googleplay_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( client_attention_googleplay_buttons, attention_youtube_buttons );

	client_attention_googleplay_buttons.prototype.is_visible_transfer_only_in_toolbar = function()
	{
		return false;
	};

	//--END GOOGLEPLAY - APP INTERACTION---------------------------------------------------------------------------

	/*--START RCS - APP SUPERVISOR-------------------------------------------------------------------
		RCS MENU ITEMS for CLIENT WINDOW - APP INTERACTION
	*/
	function client_attention_rcs_buttons( params )
	{
		this.m_client_window = params.parent.m_client_window;
		client_attention_rcs_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( client_attention_rcs_buttons, attention_youtube_buttons );

	client_attention_rcs_buttons.prototype.is_visible_transfer_only_in_toolbar = function()
	{
		return false;
	};

	//--END RCS - APP INTERACTION---------------------------------------------------------------------------


	/*
		Funcion que inserta a la definicion por canal (en el cliente) la validacion de si visualiza el transfer.
	*/
	var is_visible_transfer_only_in_toolbar = function(type, channel_actions) {
		if (!channel_actions.Transfer) {
			return;
		}
		channel_actions.Transfer.is_visible_only_in_toolbar = function() {
			if (!type.is_visible_transfer_only_in_toolbar) {
				return false;
			}
			return type.is_visible_transfer_only_in_toolbar();
		};
	};

	// ---------------
	var is_visible_mark_all_read_unread = function(self, type, interaction, not_must_validate_custom_action)
	{
		if (!not_must_validate_custom_action) {
			var is_visible_custom_action = type.parent.is_visible_custom_action('mark_all_read_unread');
			if (!is_visible_custom_action) {
				return false;
			}
		}

		const is_editable = self.m_client_window.isEditable();
		if (!is_editable) {
			return false;
		}

		if (interaction.getContactsEvents().length === 0) {
			return false;
		}

		return true;
	};

	var execute_mark_all_read_unread = function(self)
	{
		// al callback lo dejo como referencia por si hay que realizar algo!!
		self.m_client_window.markAllReadUnread(function(/*success*/) {});
	};

	/*
		Funcion que inserta a la definicion por canal (en el cliente) para marcar como read o unread todos los mensajes.
	*/
	var insert_mark_all_read_unread = function(self, type, channel_actions) {
		channel_actions.MarkAllReadUnread = {
			tooltip: function() {
				const info = type.m_client_window.get_mark_all_unread_read_button_caption();
				return info.caption;
			},
			classes: function() {
				const info = type.m_client_window.get_mark_all_unread_read_button_caption();
				return info.action_id;
			},
			enabled: true,
			visible: function( interaction, not_must_validate_custom_action ) {
				return is_visible_mark_all_read_unread(self, type, interaction, not_must_validate_custom_action);
			},
			execute: function( /*interaction, callbacks, popup_main_menu, $button*/) {
				execute_mark_all_read_unread(self);
			}
		};
	};

	//---------------------------------------------------------------------
	// app client window app "interaction" class
	function client_window_attention_actions( params )
	{
		this.app_id = "interaction";
		this.m_client_window = params.parent.owner;

		this.types = {
			APPSTORE   : client_attention_appstore_buttons  ,
			CALL       : client_attention_call_buttons      ,
			CHAT       : client_attention_chat_buttons      ,
			IM         : client_attention_im_buttons        ,
			FACEBOOK   : client_attention_facebook_buttons  ,
			GOOGLEPLAY : client_attention_googleplay_buttons,
			INSTAGRAM  : client_attention_instagram_buttons ,
			MAIL       : client_attention_mail_buttons      ,
			RCS        : client_attention_rcs_buttons       ,
			SMS        : client_attention_sms_buttons       ,
			TWITTER    : client_attention_twitter_buttons   ,
			VOICEMAIL  : inConcert.app.has_allegro_integration() ? client_attention_call_buttons : client_attention_voicemail_buttons,
			WEBCONTACT : client_attention_webcontact_buttons,
			WHATSAPP   : client_attention_whatsapp_buttons  ,
			YOUTUBE	   : client_attention_youtube_buttons
		};

		client_window_attention_actions.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( client_window_attention_actions, base_actions );

	if ( typeof inConcert.client_window_attention_actions !== "function" ) {
		inConcert.client_window_attention_actions = client_window_attention_actions;
	}

	client_window_attention_actions.prototype.initialize_chat = function()
	{
		client_window_attention_actions.uber.initialize_chat.call(this); //invokes the base class method
	
		const type = this.types.CHAT;
		let channel_actions = this.actions.CHAT;

		channel_actions.AttachFile = {
			tooltip: _T("Attach File"),
			enabled: true,
			visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_attach_file( interaction, not_must_validate_custom_action ); },
			execute: function( interaction, callbacks, popup_main_menu) { type.show_attach_file_menu(interaction, this, callbacks, popup_main_menu); },
			submenu: {external: true}
		};

		// Inserto la validacion si el transfer es visible solo en la toolbar
		is_visible_transfer_only_in_toolbar(type, channel_actions);
	};

	client_window_attention_actions.prototype.initialize_whatsapp = function()
	{
		client_window_attention_actions.uber.initialize_whatsapp.call(this); //invokes the base class method
	
		const type = this.types.WHATSAPP;
		let channel_actions = this.actions.WHATSAPP;

		channel_actions.AttachFile = {
			tooltip: _T("Attach File"),
			enabled: true,
			visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_attach_file( interaction, not_must_validate_custom_action ); },
			execute: function( interaction, callbacks, popup_main_menu) { type.show_attach_file_menu(interaction, this, callbacks, popup_main_menu); },
			submenu: {external: true}
		};

		// Inserto la validacion si el transfer es visible solo en la toolbar
		is_visible_transfer_only_in_toolbar(type, channel_actions);
		// Inserta a la definicion por canal (en el cliente) para marcar como read o unread todos los mensajes.
		insert_mark_all_read_unread(this, type, channel_actions);
	};

	client_window_attention_actions.prototype.initialize_mail = function()
	{
		client_window_attention_actions.uber.initialize_mail.call(this); //invokes the base class method
	
		const type = this.types.MAIL;
		let channel_actions = this.actions.MAIL;

		channel_actions.Print = {
			tooltip: _T("Print"),
			enabled: true,
			visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_print( interaction, not_must_validate_custom_action ); },
			execute: function( interaction, callbacks, popup_main_menu, $button) { type.execute_print(interaction, this, callbacks, popup_main_menu, $button); }
		};

		channel_actions.AttachFile = {
			tooltip: _T("Attach File"),
			enabled: true,
			visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_attach_file( interaction, not_must_validate_custom_action ); },
			execute: function( interaction, callbacks, popup_main_menu) { type.show_attach_file_menu(interaction, this, callbacks, popup_main_menu); },
			submenu: {external: true}
		};

		// Inserto la validacion si el transfer es visible solo en la toolbar
		is_visible_transfer_only_in_toolbar(type, channel_actions);
	};

	client_window_attention_actions.prototype.initialize_call = function()
	{
		client_window_attention_actions.uber.initialize_call.call(this); //invokes the base class method
		const type = this.types.CALL;
		const channel_actions = this.actions.CALL;
		// Inserto la validacion si el transfer es visible solo en la toolbar
		is_visible_transfer_only_in_toolbar(type, channel_actions);
	};

	client_window_attention_actions.prototype.initialize_voicemail = function()
	{
		client_window_attention_actions.uber.initialize_voicemail.call(this); //invokes the base class method
		const type = this.types.VOICEMAIL;
		const channel_actions = this.actions.VOICEMAIL;
		// Inserto la validacion si el transfer es visible solo en la toolbar
		is_visible_transfer_only_in_toolbar(type, channel_actions);
	};

	client_window_attention_actions.prototype.initialize_im = function()
	{
		client_window_attention_actions.uber.initialize_im.call(this); //invokes the base class method
		const type = this.types.IM;
		const channel_actions = this.actions.IM;
		// Inserto la validacion si el transfer es visible solo en la toolbar
		is_visible_transfer_only_in_toolbar(type, channel_actions);
	};

	client_window_attention_actions.prototype.initialize_sms = function()
	{
		client_window_attention_actions.uber.initialize_sms.call(this); //invokes the base class method
		const type = this.types.SMS;
		let channel_actions = this.actions.SMS;
		// Inserto la validacion si el transfer es visible solo en la toolbar
		is_visible_transfer_only_in_toolbar(type, channel_actions);
		// Inserta a la definicion por canal (en el cliente) para marcar como read o unread todos los mensajes.
		insert_mark_all_read_unread(this, type, channel_actions);
	};

	client_window_attention_actions.prototype.initialize_facebook = function()
	{
		client_window_attention_actions.uber.initialize_facebook.call(this); //invokes the base class method
		const type = this.types.FACEBOOK;
		let channel_actions = this.actions.FACEBOOK;
		// Inserto la validacion si el transfer es visible solo en la toolbar
		is_visible_transfer_only_in_toolbar(type, channel_actions);
		// Inserta a la definicion por canal (en el cliente) para marcar como read o unread todos los mensajes.
		insert_mark_all_read_unread(this, type, channel_actions);
	};

	client_window_attention_actions.prototype.initialize_instagram = function()
	{
		client_window_attention_actions.uber.initialize_instagram.call(this); //invokes the base class method
		const type = this.types.INSTAGRAM;
		let channel_actions = this.actions.INSTAGRAM;
		// Inserto la validacion si el transfer es visible solo en la toolbar
		is_visible_transfer_only_in_toolbar(type, channel_actions);
		// Inserta a la definicion por canal (en el cliente) para marcar como read o unread todos los mensajes.
		insert_mark_all_read_unread(this, type, channel_actions);
	};

	client_window_attention_actions.prototype.initialize_twitter = function()
	{
		client_window_attention_actions.uber.initialize_twitter.call(this); //invokes the base class method
		const type = this.types.TWITTER;
		let channel_actions = this.actions.TWITTER;
		// Inserto la validacion si el transfer es visible solo en la toolbar
		is_visible_transfer_only_in_toolbar(type, channel_actions);
		// Inserta a la definicion por canal (en el cliente) para marcar como read o unread todos los mensajes.
		insert_mark_all_read_unread(this, type, channel_actions);
	};

	client_window_attention_actions.prototype.initialize_internal_chat = function()
	{
		client_window_attention_actions.uber.initialize_internal_chat.call(this); //invokes the base class method
	};

	client_window_attention_actions.prototype.initialize_webcontact = function()
	{
		client_window_attention_actions.uber.initialize_webcontact.call(this); //invokes the base class method
		const type = this.types.WEBCONTACT;
		let channel_actions = this.actions.WEBCONTACT;
		// Inserto la validacion si el transfer es visible solo en la toolbar
		is_visible_transfer_only_in_toolbar(type, channel_actions);
	};

	client_window_attention_actions.prototype.initialize_youtube = function()
	{
		client_window_attention_actions.uber.initialize_youtube.call(this); //invokes the base class method
		const type = this.types.YOUTUBE;
		let channel_actions = this.actions.YOUTUBE;
		// Inserto la validacion si el transfer es visible solo en la toolbar
		is_visible_transfer_only_in_toolbar(type, channel_actions);
		// Inserta a la definicion por canal (en el cliente) para marcar como read o unread todos los mensajes.
		insert_mark_all_read_unread(this, type, channel_actions);
	};

	client_window_attention_actions.prototype.initialize_appstore = function()
	{
		client_window_attention_actions.uber.initialize_appstore.call(this); //invokes the base class method
		const type = this.types.APPSTORE;
		let channel_actions = this.actions.APPSTORE;
		// Inserto la validacion si el transfer es visible solo en la toolbar
		is_visible_transfer_only_in_toolbar(type, channel_actions);
		// Inserta a la definicion por canal (en el cliente) para marcar como read o unread todos los mensajes.
		insert_mark_all_read_unread(this, type, channel_actions);
	};

	client_window_attention_actions.prototype.initialize_googleplay = function()
	{
		client_window_attention_actions.uber.initialize_googleplay.call(this); //invokes the base class method
		const type = this.types.GOOGLEPLAY;
		let channel_actions = this.actions.GOOGLEPLAY;
		// Inserto la validacion si el transfer es visible solo en la toolbar
		is_visible_transfer_only_in_toolbar(type, channel_actions);
		// Inserta a la definicion por canal (en el cliente) para marcar como read o unread todos los mensajes.
		insert_mark_all_read_unread(this, type, channel_actions);
	};

	client_window_attention_actions.prototype.initialize_rcs = function()
	{
		client_window_attention_actions.uber.initialize_rcs.call(this); //invokes the base class method
		const type = this.types.RCS;
		let channel_actions = this.actions.RCS;
		// Inserto la validacion si el transfer es visible solo en la toolbar
		is_visible_transfer_only_in_toolbar(type, channel_actions);
		// Inserta a la definicion por canal (en el cliente) para marcar como read o unread todos los mensajes.
		insert_mark_all_read_unread(this, type, channel_actions);
	};

	// -- FIN CLASES APP INTERACTION CLASES CLIENTS WINDOWS ---------------------------------

	//
	// -- INICIO CLASES APP SUPERVISOR CLASES CLIENTS WINDOWS ---------------------------------
	//

	/*--START WEBCHAT - APP SUPERVISOR -------------------------------------------------------------------
		WEBCHAT MENU ITEMS for CLIENT WINDOW - APP SUPERVISOR
	*/
	function client_supervisor_chat_buttons( params )
	{
		this.m_client_window = params.parent.m_client_window;
		client_supervisor_chat_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( client_supervisor_chat_buttons, supervisor_chat_buttons );

	client_supervisor_chat_buttons.prototype.is_visible_transfer_only_in_toolbar = function()
	{
		return false;
	};

	//--END WEBCHAT - APP SUPERVISOR---------------------------------------------------------------------------

	/*--START WHATSAPP - APP SUPERVISOR-------------------------------------------------------------------
		WHATSAPP MENU ITEMS for CLIENT WINDOW - APP SUPERVISOR
	*/
	function client_supervisor_whatsapp_buttons( params )
	{
		this.m_client_window = params.parent.m_client_window;
		client_supervisor_whatsapp_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( client_supervisor_whatsapp_buttons, supervisor_whatsapp_buttons );

	client_supervisor_whatsapp_buttons.prototype.is_visible_transfer_only_in_toolbar = function()
	{
		return false;
	};

	//--END WHATSAPP - APP SUPERVISOR---------------------------------------------------------------------------


	/*--START MAIL - APP SUPERVISOR-------------------------------------------------------------------
		MAIL MENU ITEMS for CLIENT WINDOW - APP SUPERVISOR
	*/
	function client_supervisor_mail_buttons( params )
	{
		this.m_client_window = params.parent.m_client_window;
		supervisor_mail_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( client_supervisor_mail_buttons, supervisor_mail_buttons );

	client_supervisor_mail_buttons.prototype.is_visible_transfer_only_in_toolbar = function()
	{
		return false;
	};

	client_supervisor_mail_buttons.prototype.is_visible_print = function(interaction, not_must_validate_custom_action)
	{
		if (!not_must_validate_custom_action) {
			var is_visible_custom_action = this.parent.is_visible_custom_action('print');
			if (!is_visible_custom_action) {
				return false;
			}
		}

		const is_composing_mail = this.m_client_window.is_composing_mail();
		if (is_composing_mail) {
			return false;
		}

		const can_show_print = this.m_client_window.allowedPrint();
		if (!can_show_print) {
			return false;
		}

		return true;
	};

	client_supervisor_mail_buttons.prototype.execute_print = function(interaction, node, callbacks, popup_main_menu, $button)
	{
		const on_print = callbacks && callbacks.on_print ? callbacks.on_print : $.noop;
		const can_show_print = this.m_client_window.allowedPrint();
	
		if (!can_show_print) {
			const error_msg = _T("You cannot print.");
			showBannerMessage(error_msg);
			on_print(false, error_msg);
			return false;
		}
		/*
			La opcion del menu print (link) tiene que ir por parametro, para que utilice este link para imprimir y le asigne el valor al href.

			En el attr "rel" asignamos el valor "noreferrer" para que el popup/tab que realiza la impresion, se ejecute en un thread diferente
			al que se ejecuta la la ventana padre.
			Con esto lo que logramos es que cuando se abre el dialogo de impresion del sistema, el flujo de la ventana donde correr i6 no se detenga,
			y el no exista desconexion por timeout contra el server.

			Referencia: http://stackoverflow.com/questions/39574000/how-to-open-a-new-tab-in-separate-thread-with-javascript-chrome
		*/
		this.m_client_window.print($button);
		on_print(true, interaction);
	};

	//--END MAIL - APP SUPERVISOR---------------------------------------------------------------------------

	/*--START CALL - APP SUPERVISOR-------------------------------------------------------------------
		CALL MENU ITEMS for CLIENT WINDOW - APP SUPERVISOR
	*/
	function client_supervisor_call_buttons( params )
	{
		this.m_client_window = params.parent.m_client_window;
		client_supervisor_call_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( client_supervisor_call_buttons, supervisor_call_buttons );

	client_supervisor_call_buttons.prototype.is_visible_transfer_only_in_toolbar = function()
	{
		return false;
	};

	client_supervisor_call_buttons.prototype.is_visible_conference_only_in_toolbar = function()
	{
		return false;
	};

	client_supervisor_call_buttons.prototype.is_visible_schedule_only_in_toolbar = function()
	{
		return false;
	};

	//--END CALL - APP SUPERVISOR---------------------------------------------------------------------------

	/*--START IM - APP SUPERVISOR-------------------------------------------------------------------
		IM MENU ITEMS for CLIENT WINDOW - APP SUPERVISOR
	*/
	function client_supervisor_im_buttons( params )
	{
		this.m_client_window = params.parent.m_client_window;
		client_supervisor_im_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( client_supervisor_im_buttons, supervisor_im_buttons );

	client_supervisor_im_buttons.prototype.is_visible_transfer_only_in_toolbar = function()
	{
		return false;
	};

	//--END IM - APP SUPERVISOR---------------------------------------------------------------------------

	/*--START FACEBOOK - APP SUPERVISOR-------------------------------------------------------------------
		FACEBOOK MENU ITEMS for CLIENT WINDOW - APP SUPERVISOR
	*/
	function client_supervisor_facebook_buttons( params )
	{
		this.m_client_window = params.parent.m_client_window;
		client_supervisor_facebook_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( client_supervisor_facebook_buttons, supervisor_facebook_buttons );

	client_supervisor_facebook_buttons.prototype.is_visible_transfer_only_in_toolbar = function()
	{
		return false;
	};

	//--END FACEBOOK - APP SUPERVISOR---------------------------------------------------------------------------

	/*--START INSTAGRAM-------------------------------------------------------------------
		INSTAGRAM MENU ITEMS for CLIENT WINDOW - APP SUPERVISOR
	*/
	function client_supervisor_instagram_buttons( params )
	{
		this.m_client_window = params.parent.m_client_window;
		client_supervisor_instagram_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( client_supervisor_instagram_buttons, supervisor_instagram_buttons );

	client_supervisor_instagram_buttons.prototype.is_visible_transfer_only_in_toolbar = function()
	{
		return false;
	};

	//--END INSTAGRAM - APP SUPERVISOR---------------------------------------------------------------------------

	/*--START SMS-------------------------------------------------------------------
		SMS MENU ITEMS for CLIENT WINDOW - APP SUPERVISOR
	*/
	function client_supervisor_sms_buttons( params )
	{
		this.m_client_window = params.parent.m_client_window;
		client_supervisor_sms_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( client_supervisor_sms_buttons, supervisor_sms_buttons );

	client_supervisor_sms_buttons.prototype.is_visible_transfer_only_in_toolbar = function()
	{
		return false;
	};

	//--END SMS - APP SUPERVISOR---------------------------------------------------------------------------

	/*--START TWITTER - APP SUPERVISOR-------------------------------------------------------------------
		TWITTER MENU ITEMS for CLIENT WINDOW - APP SUPERVISOR
	*/
	function client_supervisor_twitter_buttons( params )
	{
		this.m_client_window = params.parent.m_client_window;
		client_supervisor_twitter_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( client_supervisor_twitter_buttons, supervisor_twitter_buttons );

	client_supervisor_twitter_buttons.prototype.is_visible_transfer_only_in_toolbar = function()
	{
		return false;
	};

	//--END TWITTER - APP SUPERVISOR---------------------------------------------------------------------------

	/*--START WEBCONTACT - APP SUPERVISOR-------------------------------------------------------------------
		WEBCONTACT MENU ITEMS for CLIENT WINDOW - APP SUPERVISOR
	*/
	function client_supervisor_webcontact_buttons( params )
	{
		this.m_client_window = params.parent.m_client_window;
		client_supervisor_webcontact_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( client_supervisor_webcontact_buttons, supervisor_webcontact_buttons );

	client_supervisor_webcontact_buttons.prototype.is_visible_transfer_only_in_toolbar = function()
	{
		return false;
	};

	//--END WEBCONTACT - APP SUPERVISOR---------------------------------------------------------------------------

	/*--START WEBCONTACT - APP SUPERVISOR-------------------------------------------------------------------
		WEBCONTACT MENU ITEMS for CLIENT WINDOW - APP SUPERVISOR
	*/
	function client_supervisor_youtube_buttons( params )
	{
		this.m_client_window = params.parent.m_client_window;
		client_supervisor_youtube_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( client_supervisor_youtube_buttons, supervisor_youtube_buttons );

	client_supervisor_youtube_buttons.prototype.is_visible_transfer_only_in_toolbar = function()
	{
		return false;
	};

	//--END WEBCONTACT - APP SUPERVISOR---------------------------------------------------------------------------

	/*--START APPSTORE - APP SUPERVISOR-------------------------------------------------------------------
		APPSTORE MENU ITEMS for CLIENT WINDOW - APP SUPERVISOR
	*/
	function client_supervisor_appstore_buttons( params )
	{
		this.m_client_window = params.parent.m_client_window;
		client_supervisor_appstore_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( client_supervisor_appstore_buttons, supervisor_youtube_buttons );

	client_supervisor_appstore_buttons.prototype.is_visible_transfer_only_in_toolbar = function()
	{
		return false;
	};

	//--END APPSTORE - APP SUPERVISOR---------------------------------------------------------------------------

	/*--START GOOGLEPLAY - APP SUPERVISOR-------------------------------------------------------------------
		GOOGLEPLAY MENU ITEMS for CLIENT WINDOW - APP SUPERVISOR
	*/
	function client_supervisor_googleplay_buttons( params )
	{
		this.m_client_window = params.parent.m_client_window;
		client_supervisor_googleplay_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( client_supervisor_googleplay_buttons, supervisor_youtube_buttons );

	client_supervisor_googleplay_buttons.prototype.is_visible_transfer_only_in_toolbar = function()
	{
		return false;
	};

	//--END GOOGLEPLAY - APP SUPERVISOR---------------------------------------------------------------------------

	/*--START RCS - APP SUPERVISOR-------------------------------------------------------------------
		RCS MENU ITEMS for CLIENT WINDOW - APP SUPERVISOR
	*/
	function client_supervisor_rcs_buttons( params )
	{
		this.m_client_window = params.parent.m_client_window;
		client_supervisor_rcs_buttons.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( client_supervisor_rcs_buttons, supervisor_youtube_buttons );

	client_supervisor_rcs_buttons.prototype.is_visible_transfer_only_in_toolbar = function()
	{
		return false;
	};

	//--END RCS - APP SUPERVISOR---------------------------------------------------------------------------

	//---------------------------------------------------------------------
	// app client window app "supervisor" class
	function client_window_supervisor_actions( params )
	{
		this.app_id = "supervisor";
		this.m_client_window = params.parent.owner;

		this.types = {
			APPSTORE   : client_supervisor_appstore_buttons  ,
			CALL       : client_supervisor_call_buttons      ,
			CHAT       : client_supervisor_chat_buttons      ,
			IM         : client_supervisor_im_buttons        ,
			FACEBOOK   : client_supervisor_facebook_buttons  ,
			GOOGLEPLAY : client_supervisor_googleplay_buttons,
			INSTAGRAM  : client_supervisor_instagram_buttons ,
			MAIL       : client_supervisor_mail_buttons      ,
			SMS        : client_supervisor_sms_buttons       ,
			RCS        : client_supervisor_rcs_buttons       ,
			TWITTER    : client_supervisor_twitter_buttons   ,
			VOICEMAIL  : client_supervisor_call_buttons      ,
			WEBCONTACT : client_supervisor_webcontact_buttons,
			WHATSAPP   : client_supervisor_whatsapp_buttons  ,
			YOUTUBE	   : client_supervisor_youtube_buttons
		};

		client_window_supervisor_actions.uber.constructor.call( this, params );
	}

	inConcert.Utils.inherit( client_window_supervisor_actions, base_actions );

	if ( typeof inConcert.client_window_supervisor_actions !== "function" ) {
		inConcert.client_window_supervisor_actions = client_window_supervisor_actions;
	}

	client_window_supervisor_actions.prototype.initialize_chat = function()
	{
		client_window_supervisor_actions.uber.initialize_chat.call(this); //invokes the base class method
		const type = this.types.CHAT;
		const channel_actions = this.actions.CHAT;
		// Inserto la validacion si el transfer es visible solo en la toolbar
		is_visible_transfer_only_in_toolbar(type, channel_actions);
	};

	client_window_supervisor_actions.prototype.initialize_whatsapp = function()
	{
		client_window_supervisor_actions.uber.initialize_whatsapp.call(this); //invokes the base class method
		const type = this.types.WHATSAPP;
		const channel_actions = this.actions.WHATSAPP;
		// Inserto la validacion si el transfer es visible solo en la toolbar
		is_visible_transfer_only_in_toolbar(type, channel_actions);
	};

	client_window_supervisor_actions.prototype.initialize_mail = function()
	{
		client_window_supervisor_actions.uber.initialize_mail.call(this); //invokes the base class method
	
		const type = this.types.MAIL;
		let channel_actions = this.actions.MAIL;

		channel_actions.Print = {
			tooltip: _T("Print"),
			enabled: true,
			visible: function( interaction, not_must_validate_custom_action ) { return type.is_visible_print( interaction, not_must_validate_custom_action ); },
			execute: function( interaction, callbacks, popup_main_menu, $button) { type.execute_print(interaction, this, callbacks, popup_main_menu, $button); }
		};

		// Inserto la validacion si el transfer es visible solo en la toolbar
		is_visible_transfer_only_in_toolbar(type, channel_actions);
	};

	client_window_supervisor_actions.prototype.initialize_call = function()
	{
		client_window_supervisor_actions.uber.initialize_call.call(this); //invokes the base class method
		const type = this.types.CALL;
		const channel_actions = this.actions.CALL;
		// Inserto la validacion si el transfer es visible solo en la toolbar
		is_visible_transfer_only_in_toolbar(type, channel_actions);
	};

	client_window_supervisor_actions.prototype.initialize_voicemail = function()
	{
		client_window_supervisor_actions.uber.initialize_voicemail.call(this); //invokes the base class method
		const type = this.types.VOICEMAIL;
		const channel_actions = this.actions.VOICEMAIL;
		// Inserto la validacion si el transfer es visible solo en la toolbar
		is_visible_transfer_only_in_toolbar(type, channel_actions);
	};

	client_window_supervisor_actions.prototype.initialize_im = function()
	{
		client_window_supervisor_actions.uber.initialize_im.call(this); //invokes the base class method
		const type = this.types.IM;
		const channel_actions = this.actions.IM;
		// Inserto la validacion si el transfer es visible solo en la toolbar
		is_visible_transfer_only_in_toolbar(type, channel_actions);
	};

	client_window_supervisor_actions.prototype.initialize_sms = function()
	{
		client_window_supervisor_actions.uber.initialize_sms.call(this); //invokes the base class method
		const type = this.types.SMS;
		const channel_actions = this.actions.SMS;
		// Inserto la validacion si el transfer es visible solo en la toolbar
		is_visible_transfer_only_in_toolbar(type, channel_actions);
	};

	client_window_supervisor_actions.prototype.initialize_facebook = function()
	{
		client_window_supervisor_actions.uber.initialize_facebook.call(this); //invokes the base class method
		const type = this.types.FACEBOOK;
		const channel_actions = this.actions.FACEBOOK;
		// Inserto la validacion si el transfer es visible solo en la toolbar
		is_visible_transfer_only_in_toolbar(type, channel_actions);
	};

	client_window_supervisor_actions.prototype.initialize_instagram = function()
	{
		client_window_supervisor_actions.uber.initialize_instagram.call(this); //invokes the base class method
		const type = this.types.INSTAGRAM;
		const channel_actions = this.actions.INSTAGRAM;
		// Inserto la validacion si el transfer es visible solo en la toolbar
		is_visible_transfer_only_in_toolbar(type, channel_actions);
	};

	client_window_supervisor_actions.prototype.initialize_twitter = function()
	{
		client_window_supervisor_actions.uber.initialize_twitter.call(this); //invokes the base class method
		const type = this.types.TWITTER;
		const channel_actions = this.actions.TWITTER;
		// Inserto la validacion si el transfer es visible solo en la toolbar
		is_visible_transfer_only_in_toolbar(type, channel_actions);
	};

	client_window_supervisor_actions.prototype.initialize_internal_chat = function()
	{
		client_window_supervisor_actions.uber.initialize_internal_chat.call(this); //invokes the base class method
	};

	client_window_supervisor_actions.prototype.initialize_webcontact = function()
	{
		client_window_supervisor_actions.uber.initialize_webcontact.call(this); //invokes the base class method
		const type = this.types.WEBCONTACT;
		const channel_actions = this.actions.WEBCONTACT;
		// Inserto la validacion si el transfer es visible solo en la toolbar
		is_visible_transfer_only_in_toolbar(type, channel_actions);
	};

	client_window_supervisor_actions.prototype.initialize_youtube = function()
	{
		client_window_supervisor_actions.uber.initialize_youtube.call(this); //invokes the base class method
		const type = this.types.YOUTUBE;
		const channel_actions = this.actions.YOUTUBE;
		// Inserto la validacion si el transfer es visible solo en la toolbar
		is_visible_transfer_only_in_toolbar(type, channel_actions);
	};

	client_window_supervisor_actions.prototype.initialize_appstore = function()
	{
		client_window_supervisor_actions.uber.initialize_appstore.call(this); //invokes the base class method
		const type = this.types.APPSTORE;
		const channel_actions = this.actions.APPSTORE;
		// Inserto la validacion si el transfer es visible solo en la toolbar
		is_visible_transfer_only_in_toolbar(type, channel_actions);
	};

	client_window_supervisor_actions.prototype.initialize_googleplay = function()
	{
		client_window_supervisor_actions.uber.initialize_googleplay.call(this); //invokes the base class method
		const type = this.types.GOOGLEPLAY;
		const channel_actions = this.actions.GOOGLEPLAY;
		// Inserto la validacion si el transfer es visible solo en la toolbar
		is_visible_transfer_only_in_toolbar(type, channel_actions);
	};

	client_window_supervisor_actions.prototype.initialize_rcs = function()
	{
		client_window_supervisor_actions.uber.initialize_rcs.call(this); //invokes the base class method
		const type = this.types.RCS;
		const channel_actions = this.actions.RCS;
		// Inserto la validacion si el transfer es visible solo en la toolbar
		is_visible_transfer_only_in_toolbar(type, channel_actions);
	};
	// -- FIN CLASES APP SUPERVISOR CLASES CLIENTS WINDOWS ---------------------------------

}());
