/*jshint quotmark:false, esversion: 9*/
/* global jQuery */

/*!
 * Plug-In				:	jQuery ToolTip
 * Version				:	1.0
 * Autor					:	Federico Ramos
 * Descripcion		:	Crea un dropdown que permite selecciones multiples a partir de un div determinado y una lista de opciones
 
 * ----------------------------------------------------------------------------------------------------------
 *
 * Sintaxis
 * ---------
 * HTML		:	<div id="miMultiselect" />
 * 
 * JS		:	var dropdownOptions = [{id: 1, name: "option 1"},{id: 2, name: "option 2"},{id: 3, name: "option 3"}];
                $('#miMultiselect').multipleSelectDropdown(dropdownOptions);
                $('#miMultiselect').getSelectedItems();
*/

(function($) {
    $.fn.selectItemsFromArray = function(itemsArray) {
        var $dropdown = $(this);
    
        itemsArray.forEach(function(item) {
            if (item.id === 'selectAll') {
                $dropdown.find('#selectAll').prop('checked', true);
            } else {
                var $option = $dropdown.find('.option[value="' + item.name + '"]');
                $option.prop('checked', true);
            }
        });
    
        var $btn = $dropdown.find('.dropbtn');
        var selectedOptions = $dropdown.find('.option:checked').map(function() {
            return $(this).val();
        }).get();
        $btn.text(selectedOptions.length > 0 ? selectedOptions.join(', ') : 'Seleccionar opciones');

        var selectAllChecked = $dropdown.find('#selectAll').prop('checked');
        if (selectAllChecked) {
            $btn.text('Todos seleccionados');
        }
    };
    
  $.fn.getSelectedItems = function() {
    var selectedItems = [];

    this.each(function() {
        var $dropdown = $(this);

        var selectAllChecked = $dropdown.find('#selectAll').prop('checked');
        if (selectAllChecked) {
            selectedItems.push({ id: 'selectAll', name: 'Seleccionar todos' });
        }

        $dropdown.find('.option:checked').each(function() {
            var $selectedOption = $(this);
            var itemId = $selectedOption.attr('id') || $selectedOption.val();
            var itemName = $selectedOption.closest('label').text().trim();

            selectedItems.push({ id: itemId, name: itemName });
        });
    });

    return selectedItems;
};

  $.fn.multipleSelectDropdown = function(options) {
      return this.each(function() {
          var $dropdown = $(this);
          var $btn = $('<button class="dropbtn form-control" id="dropbtn">Seleccionar opciones</button>');
          var $dropdownContent = $('<div class="dropdown-content" id="dropdown-content"></div>');

          var dropdownOptions = options || [];
          var dropdownHTML = '';

          dropdownHTML += '<input type="text" class="search-input form-control" placeholder="Buscar...">';
          dropdownHTML += '<div class="form-check"><label class="form-check-label"><input class="form-check-input" type="checkbox" id="selectAll"><strong>Seleccionar todos</strong></label></div>';
          dropdownHTML += '<div class="options-list">';

          dropdownOptions.forEach(function(option) {
              dropdownHTML += '<div class="form-check"><label class="form-check-label"><input type="checkbox" class="option" id="' + option.id + '" value="' + option.name + '">' + option.name + '</label></div>';
          });

          dropdownHTML += '</div>';

          $dropdownContent.html(dropdownHTML);

          $dropdown.append($btn).append($dropdownContent);

          $dropdownContent.on('keyup', '.search-input', function() {
            var searchText = $(this).val().toLowerCase();
            $dropdownContent.find('.form-check-label').each(function() {
                var optionText = $(this).text().toLowerCase();
                var $option = $(this);
                if (optionText.indexOf(searchText) > -1) {
                    $option.closest('.form-check').show();
                } else {
                    $option.closest('.form-check').hide();
                }
             });
            });

          $dropdownContent.on('change', '#selectAll', function() {
              $dropdownContent.find('.option').prop('checked', $(this).prop('checked'));
              $btn.text($(this).prop('checked') ? 'Todos seleccionados' : 'Seleccionar opciones');
          });

          $btn.on('click', function() {
              $dropdownContent.toggle();
          });

          $dropdownContent.on('change', '.option', function() {
              var selectedOptions = $dropdownContent.find('.option:checked').map(function() {
                  return $(this).val();
              }).get();
              $btn.text(selectedOptions.length > 0 ? selectedOptions.join(', ') : 'Seleccionar opciones');
          });

          $(document).on('click', function(event) {
              if (!$dropdown.is(event.target) && !$dropdown.has(event.target).length) {
                  $dropdownContent.hide();
              }
          });
      });
    };
  })(jQuery);
