/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, esversion: 11 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

//	"use strict";


	//===================================================================================================
	//===================================================================================================
	/**
	Class users_selector.

	*/
	const _T = inConcert.i18n.getString;
	const FORMAT = inConcert.Utils.format;
	const LIMIT_FILTER = 300;
	/*---------------------------------------------------------------------
	CLASS USERS SELECTOR POPUP.
	------------------------
	*/
	function users_selector (params)
	{
		const self = this;

		this.container_id               = params.container_id;
		this.id_hide_popup              = this.container_id + "_popup_hide";
		this.id_update_users_list       = this.container_id + "_popup_update_list";
		this.id_err_message             = this.container_id + "_popup_message";
		this.id_waiting                 = this.container_id + "_popup_waiting";
		this.id_search_text             = this.container_id + "_popup_search_text";
		this.id_users_list              = this.container_id + "_popup_search_list";
		this.id_search_not_result_found = this.container_id + "_popup_search_not_results_found";
		this.id_total_rows              = this.container_id + "_popup_total_rows";
		this.id_check_all               = this.container_id + "_popup_check_all";
		this.id_popup_title             = this.container_id + "_popup_title";
		this.id_popup_check_view_mode   = this.container_id + "_check_view_mode";
		this.id_check_all_al            = this.container_id + "_popup_check_all_al";

		this.m_owner = params.owner;
		this.m_listener = params.listener;
		this.m_container_classes = params.container_classes || "";
		//TODO Estos totales se agregan para no contabilizar recorriendo el DOM, para mostrar totales y realizar validaciones independientes.
		this.m_total_rows_filtered = 0; // Total de rows filtrados
		this.m_total_selected_rows_filtered = 0; //Total de rows seleccionados del filtrado. Aca no tiene encuenta los miembros de la campanna.
		this.m_total_campaign_members = 0; //Total de members que pertenecen a la campanna.
		this.m_total_selected_als = 0; //Total de als seleccionados del filtrado. 
		this.m_total_general_members_selected = 0; //Total de rows seleccionados. Aca tiene encuenta los miembros de la campanna.
		this.m_users = null; //Total de usuarios del vcc
		this.m_is_rendered_users = false;
		this.m_info = null;
		this.m_timeout_filter_key_up = null;
		this.m_timeout_filter_search = null;
		this.m_pubsub_new_user_handle = null;

		let callbacks = params.callbacks || {};
		this.m_callbacks = {
			update: function(on_updated = $.noop) {
				self.hide_errors();
				let info = self.get_info();
				if (callbacks.update) {
					callbacks.update(info, function(success, error) {
						self.__on_updated(success, error, on_updated);
					});
				}
				else {
					on_updated(false);
				}
			},
			show_waiting: function(msg) {
				if (callbacks.show_waiting) {
					callbacks.show_waiting(msg);
				}
				else {
					self.show_waiting(msg); // si el show waiting no se da en el callback, ejecuto el propio del popup
				}
			},
			hide_waiting: function() {
				if (callbacks.hide_waiting) {
					callbacks.hide_waiting();
				}
				else {
					self.hide_waiting(); // si el hide waiting no se da en el callback, ejecuto el propio del popup
				}
			}
		};

		this.initialize();
	}

	if (typeof(inConcert.users_selector) !== "function") {
		inConcert.users_selector = users_selector;
	}

	users_selector.prototype.initialize = function() {
		this.__create_markup();
		this.set_events();
		this.enable_listener();
	};

	users_selector.prototype.enable_listener = function() 
	{
		const self = this;
		if (!this.m_listener) {
			return;
		}
		if (this.m_listener.new_user) {
			let $list_container = $("#" + this.id_users_list);
			let pub_new_user_id = this.m_listener.new_user;

			this.m_pubsub_new_user_handle = inConcert.app.messagebus.subscribe(
				pub_new_user_id,
				function(new_user) {
					if (!self.is_rendered_users()) {
						return;
					}
					let user = self.__make_user_data(new_user);
					self.add_row_user(user, $list_container);
					self.m_users = self.m_users || [];
					self.m_users.push(new_user);
				}
			);
		}
	};

	users_selector.prototype.__get_title = function() 
	{
		let default_title = _T("Update users");
		if (!this.m_info || !this.m_info.campaign_id) {
			return default_title;
		}
		let campaign = inConcert.app.getCampaign(this.m_info.campaign_id);
		if (!campaign) {
			return default_title;
		}
		return FORMAT(_T("Update users of campaign: {0}"), campaign.getDescription());
	};

	users_selector.prototype.__update_title = function() {
		const $title = $("#" + this.id_popup_title);
		let title = this.__get_title();
		$title.text(title);
	};

	users_selector.prototype.__create_markup = function()
	{
		const $container = $("#" + this.container_id);
		$container
			.removeClass("")
			.addClass("internalPopUpContainer " + this.m_container_classes)
			.hide();

		$container.html(
			"<div class='internalPopUpOverlay'></div>" +
			"<div class='internalPopUp'>" +
				"<div id='" + this.id_popup_title + "' class='internalPopUpTitle'>" + this.__get_title() + "</div>" +
				"<div class='internalPopUpMiddle'>" +
					"<div id='" + this.id_err_message + "' class='genericFormMessagesContainer messageError' hidden>" +
					"<label class='messageText'></label>" + 
					"</div>" +

					"<div id='" + this.id_waiting + "' class='popupWaitingContainer' hidden>" +
						"<div waiting_message='true' class='popupWaitingMessage'>" + _T("Loading...") + "</div>" +
						"<div class='popupWaitingAnimation'></div>" +
						"<div class='popupWaitingOverlay'></div>" +
					"</div>" +

					"<div class='card-block'>" +
						"<div class='row'>" +
							"<div class='col-12'>" +
								"<div class='d-flex justify-content-between'>" +
									"<div class='form-group mb-0' title='"+ _T('Uncheck to switch view and update attention levels')+"'>" +
										"<div class='form-check'>" +
											"<label for='" + this.id_popup_check_view_mode + "' class='form-check-label form-check-label-title'>" +
												"<input id='" + this.id_popup_check_view_mode + "' class=form-check-input'' type='checkbox' checked>" +
											 _T("Compact") + 
											"</label>" +
										"</div>" +
									"</div>" +
									"<div class='form-group mb-0'>" +
										"<div id='" + this.id_total_rows + "' class='form-list-totals'>" + _T("totals") + "</div>" +					
									"</div>" +
								"</div>" +
							"</div>" +
						"</div>" +
						"<div class='row'>" +
							"<div class='col-12'>" +
								"<div class='form-group search-form-group' title='" + FORMAT(_T('The search is limited to {0} items. Before performing a search, you should save the changes so that they are not lost.'), LIMIT_FILTER) + "'>" +
	//								"<label for='" + this.id_search_text + "' class=''>" + _T("Agent") + _T(":") + "</label>" +
									"<input id='" + this.id_search_text + "' class='form-control' type='text' placeholder='" + _T("Search users") + "...'  />" +
									"<i class='far fa-search'></i>" +				
								"</div>" +
							"</div>" +
						"</div>" +
	
						"<div class='row'>" +
							"<div class='col-4'>" +
								"<div class='form-group mb-0'>" +
									"<div class='form-check'>" +
										"<label for='" + this.id_check_all + "' class='form-check-label form-check-label-title'>" +
											"<input id='" + this.id_check_all + "' class='form-check-input' type='checkbox' />" +
											 _T("Select All") + 
										"</label>" +
									"</div>" +
									"<i class='fa fa-question-circle' title='" + FORMAT(_T('The search is limited to {0} items. Before performing a search, you should save the changes so that they are not lost.'), LIMIT_FILTER) + "'></i>" +
								"</div>" +
							"</div>" +
							"<div class='col-8'>" +
								"<div class='form-group mb-0'>" +
									"<div class='form-check'>" +
										"<label for='" + this.id_check_all_al + "' class='form-check-label form-check-label-title' style='display:none'>" +
											"<input id='" + this.id_check_all_al + "' class='form-check-input' type='checkbox' />" + 
											 _T("Select All Attention Levels") +
										 "</label>" +
									"</div>" +		
								"</div>" +
							"</div>" +
						"</div>" +
	
						"<div class='row'>" +
							"<div class='formListContainer'>" +
								"<ul id='" + this.id_users_list + "' class='formList formListWithCheckbox' style='display:none'>" +
									"<li view_mode_full header='true' class='viewListTitle' style='display:none'>"+
										"<label class='viewListTitleText'>"+ _T("Agent") +"</label>"+
										"<label class='viewListTitleText'>"+ _T("Attention Levels") +"</label>"+
									"</li>"+
								"</ul>" +
							"</div>" +
							"<div id='" + this.id_search_not_result_found + "' class='inlineMessagesContainer messageInfo m-2'>" + _T('No users matched.') + "</div>" +
						"</div>" +
					"</div>" +
				"</div>" +
				"<div class='d-flex justify-content-end internalPopUpButtons'>" +
					"<input id='" + this.id_hide_popup + "' class='pageButtonGeneric buttonColorWhite mr-3' type='button' value='" + _T("Close") + "' />" +
					"<input id='" + this.id_update_users_list + "' class='pageButtonGeneric buttonColorBlue' type='button' value='" + _T("Update") + "' />" +
				"</div>" +
			"</div>"
		);
	};

	users_selector.prototype.is_rendered_users = function() {
		return this.m_is_rendered_users;
	};

	users_selector.prototype.set_info = function(info) {
		this.m_info = info;
	};

	users_selector.prototype.get_info = function() {
		this.m_info = this.m_info || {};
		this.m_info.selected_users = this.get_selected_users();
		return this.m_info;
	};

	users_selector.prototype.__make_user_data = function(user, att_levels)
	{
		const member_without_vcc = inConcert.Utils.getIdWithoutVCC(user.Id);
		const get_user_name = function() {
			if ( !user.Name ) {
				return member_without_vcc;
			}
			if ( typeof user.FirstSurname !== "string" || user.FirstSurname.trim().length === 0 ) {
				return user.Name;
			}
			return user.Name + " " + user.FirstSurname;
		};
		return {
			id        : member_without_vcc,
			name      : get_user_name(),
			att_levels: att_levels
		};
	};

	users_selector.prototype.__add_users = function (params) {
		const users = params.users;
		const att_levels = params.att_levels || [];
		const campaign_id = params.campaign_id;
		const complete_cb = params.complete_cb || $.noop;
		const waiting_message = params.waiting_message || _T("Please wait, adding user, {0} of {1}.");
		const can_append_fn = params.can_append_fn;
		const self = this;
		const $list_container = $("#" + this.id_users_list);
		const $parent = $list_container.parent();
		$list_container.html("");
		this.hide_not_results_found();
		
		this.m_total_rows_filtered = 0;
		this.m_total_selected_rows_filtered = 0;
		this.m_total_selected_als = 0;
		let users_html = "";
		$("#" + this.id_users_list).find("li[user_data]").remove();
		if (!users || users.length === 0) {
			this.show_not_results_found();
			complete_cb(false);
		}

		let campaign_members = {};

		//Itero de forma async (Esto lo hacia antes la funcion add_rows_users).
		//Se acomoda para unificar iteraciones y mejorar performance.
		inConcert.Utils.foreach_timeout({
			rows: users,
			count_total_rows: users.length < LIMIT_FILTER? users.length: LIMIT_FILTER,
			waiting_message: waiting_message,
			can_append: can_append_fn,
			append_fn: function (member) {
				let is_selected;
				if (campaign_id && member.BelongToCampaigns[campaign_id]) {
					is_selected = true;
					campaign_members[member.Id.toLowerCase()] = member;
				}
				const user = self.__make_user_data(member, att_levels);
				users_html += self.add_row_user(user, undefined, is_selected);
			},
			waiting_fn: function (message) {
				self.m_callbacks.show_waiting(message);
			},
			complete_fn: function () {
				if (users_html !== "") {
					$list_container.append($(users_html));
				}

				self.m_is_rendered_users = true;
				$parent.append($list_container);
				self.update_total_rows_msg();
				$("#" + self.id_search_text).focus();
				// pequeno timeout para que el render se realice correctamente.
				window.setTimeout(function () {
					self.set_info({campaign_id: campaign_id, current_users: campaign_members, campaign_att_levels: att_levels});
					complete_cb(true);
				}, 200);
			}
		});
	};

	users_selector.prototype.make_users_markup = function(all_users, att_levels, callback = $.noop, campaign_id)
	{
		this.__set_users(all_users);
		this.__add_users({
			users: all_users,
			att_levels: att_levels,
			campaign_id: campaign_id,
			complete_cb: callback
		});
	};

	users_selector.prototype.__set_users = function(users) {
		this.m_users = users || [];
	};

	users_selector.prototype.mark_users = function (callback = $.noop) {

		const is_all_selected = this.m_total_rows_filtered > 0 && this.m_total_rows_filtered === this.m_total_selected_rows_filtered;
		$("#" + this.id_check_all).prop("checked", is_all_selected);
		this.update_total_rows_msg();

		//Siempre muestro la vista compact
		const $popup_check_view_mode = $("#" + this.id_popup_check_view_mode);
		$popup_check_view_mode.prop('checked', false).click();
		const $popup_check_view_mode_container = $popup_check_view_mode.parent();

		if (this.m_info?.campaign_att_levels?.length > 0) {
			$popup_check_view_mode_container.show();
		} else {
			$popup_check_view_mode_container.hide();
		}
		callback();
	};

	users_selector.prototype.show = function() {
		this.hide_errors();
		this.__update_title();
		$("#" + this.container_id).show();
	};

	users_selector.prototype.hide = function()
	{
		$("#" + this.container_id).hide();
		$("#" + this.id_search_text).val("");
		this.clean_filter();
		this.hide_errors();
	};

	users_selector.prototype.show_not_results_found = function()
	{
		$("#" + this.id_users_list).hide();
		$("#" + this.id_search_not_result_found).show();
		$("#" + this.id_total_rows).hide();
	};

	users_selector.prototype.hide_not_results_found = function()
	{
		$("#" + this.id_search_not_result_found).hide();
		$("#" + this.id_users_list).show();
		$("#" + this.id_total_rows).show();
	};

	users_selector.prototype.show_error_msg = function(msg) {
		const $container = $("#" + this.container_id);
		$container.addClass("showError");
		$("#" + this.id_err_message + " > .messageText").text(msg).show();
	};

	users_selector.prototype.hide_errors = function() {
		$("#" + this.id_err_message).text("").hide();
		const $users_list = $("#" + this.id_users_list);
		const $users_errors = $users_list.find("[user_data] i[error='true']");
		$users_errors.hide();
		const $container = $("#" + this.container_id);
		$container.removeClass("showError");
	};

	users_selector.prototype.show_waiting = function( message )
	{
		this.hide_errors();
		const $waiting = $("#" + this.id_waiting);
		$waiting.find("[waiting_message]").text(message || "");
		$waiting.show();
	};

	users_selector.prototype.hide_waiting = function() {
		$("#" + this.id_waiting).hide();
	};

	users_selector.prototype.get_selected_att_levels = function($member)
	{
		const $att_levels_user = $member.find("[att_level_id]").filter(':checked');
		let array_att_levels = {};

		$att_levels_user.each(function() {
		
			let att_level = $(this).data("att_level");
			array_att_levels[att_level.id] = {
				id:att_level.id,
				name:att_level.name
			};
		});
		return array_att_levels;
	};

	users_selector.prototype.get_selected_users = function( only_visible)
	{
		const self = this;
		let selected_users = {};
		let count = 0;
		const visible = only_visible? ":visible":"";
		$(`#${this.id_users_list} li[user_data]${visible}`).each(function( _, member )
		{
			let $member = $(member);
			if ( !only_visible || (only_visible && $member.is(":visible")) ) {
				let $chk = $member.find("input[selector='1'][type='checkbox']");
				let checked = $chk.prop("checked");
				if ( checked ) {
					count++;
					let user = $chk.data("user_info");
					let att_levels = self.get_selected_att_levels($member);
					selected_users[ user.id ] = {
						id   : user.id,
						name : user.name || user.id,
						att_levels: att_levels
					};
				}
			}
		});
		return {
			count : count,
			users : selected_users
		};
	};

	users_selector.prototype.get_opt_user_id = function( user_id )
	{
		let normalized_id = inConcert.Utils.getNormalizedId( user_id );
		let opt_id = this.id_users_list + "_opt_user_" + normalized_id;
		return opt_id;
	};

	users_selector.prototype.add_row_user = function( user, $list_container, is_selected_user)
	{
		const self = this;

		if ( !user ) {
			return;
		}

		is_selected_user = is_selected_user || false;
		
		let unique_id = user.id.trim().toLowerCase();
		let total_selected_als = 0;
		const make_al_for_user = function(user_id)
		{
			const resp = {html: ""};
			$.each(user.att_levels || [], function(_, att_level)
			{
				const checked = !!att_level.users?.[user_id];
				total_selected_als += checked? 1: 0;
				let dom_id = self.id_users_list + "_att_levels_" + user_id + "_"+ att_level.id;
				dom_id = inConcert.Utils.getNormalizedId(dom_id);
				const data = {
					id  : att_level.id,
					name: att_level.name
				};
				resp.html = resp.html +
					"<li class='formListItem'>" +
						"<input data-att_level='"+ JSON.stringify(data) +"' id='" + dom_id + "' att_level_id='true' class='formListCheck' type='checkbox' " + (!is_selected_user ? "disabled":  checked? "checked" :"") + ">" +
						"<label for='"+ dom_id + "'>" + att_level.name + "</label>" +
					"</li>";
			});
			return resp;
		};

		const make_new_row = function()
		{
			let user_id = user.id;
			let user_name = user.name || user_id;
			let opt_dom_id = self.get_opt_user_id( user_id );
			const alResponse = make_al_for_user(user_id);
			const $check_view_mode = $(`#${self.id_popup_check_view_mode}`);
			const is_checked_view_mode = $check_view_mode.prop("checked");
			let dom_als_list_id = self.id_users_list + "_att_levels_" + user_id;
			dom_als_list_id = inConcert.Utils.getNormalizedId(dom_als_list_id);
			delete user.att_levels;
			const user_html =
				"<li user_id='" + unique_id + "' user_data='true' class='formListItem' search_text='" + unique_id + user_name + "'>" +
					"<div class='formListItemDataContainer'>" +
						"<div class='formListItemLeftContainer'>" +
							"<div class='formItemIcon'>" +
								"<i class='fas fa-id-card-alt'></i>" +
							"</div>" +
							"<div class='formItemText'>" +
								"<label class='formItemID' for='" + opt_dom_id + "' user_id='true' unique_id='" + unique_id + "'>" + user_id  + "</label>" +
								"<label class='formItemUsername' for='" + opt_dom_id + "' user_name='true' user_name='" + user_name + "'>" + user_name + "</label>" +
							"</div>" +
						"</div>" +
						"<input id='" + opt_dom_id + "' data-user_info='"+ JSON.stringify(user) +"' user_id='" + unique_id + "' user_selector='true' class='formListCheck' selector='1' type='checkbox' " + (is_selected_user ? "checked" :"") + "/>" +
						"<i error='true' class='formFieldValidation formFieldError' hidden></i>" +
					"</div>" +
					`<div view_mode_full class='formListAttLevelsContainer' ${is_checked_view_mode?"style='display:none'":""}>` +
						"<ul id='" + dom_als_list_id + "' class='attLevelsList formListWithCheckbox'>" +
						alResponse.html + 
						"</ul>" +
					"</div>" +
				"</li>";
			return user_html;
		};

		const user_html = make_new_row();
		if ($list_container) {
			$list_container.append($(user_html));
		}

		if (is_selected_user) {
			this.m_total_selected_rows_filtered++;
			this.m_total_selected_als += total_selected_als;
		}

		this.m_total_rows_filtered++;
		return user_html;
	};

	users_selector.prototype.update_total_rows_msg = function(total_from) {

		/*
			total_from: Aca es total seleccionado (los miembros de la campanna +|- lo nuevo seleccionado/deseleccionado del filtro (Max LIMIT_FILTER))
			total_to: total de usuarios.
		*/
		total_from = total_from !== undefined ? total_from: this.m_total_general_members_selected; 
		const total_to = this.m_users.length;
		$("#" + this.id_total_rows).text(FORMAT(_T("Selected {0} of {1} users."), total_from, total_to));
	};

	users_selector.prototype.count_rows_filtered = function()
	{
		if ( this.m_total_rows_filtered === 0 ) {
			this.m_total_rows_filtered = $("#" + this.id_users_list).find("li[user_data]").length;
		}
		return this.m_total_rows_filtered;
	};

	users_selector.prototype.set_total_campaign_members = function(total)
	{
		this.m_total_campaign_members = total || 0;
		this.__reload_total_general_members_selected();
	};

	users_selector.prototype.__reload_total_general_members_selected = function()
	{
		this.m_total_general_members_selected = this.m_total_campaign_members;
	};

	users_selector.prototype.count_selected_rows_filtered = function( from_cache )
	{
		if ( !from_cache ) {
			let selected = this.get_selected_users();
			this.m_total_selected_rows_filtered = selected.count;
		}
		return this.m_total_selected_rows_filtered;
	};

	users_selector.prototype.make_search_regexp = function( word )
	{
		if (typeof word !== "string") {
			return "";
		}
		let search_pattern = inConcert.Utils.removeSpecialChars(word);
		return new RegExp(search_pattern.replace(/[^\w\s]/gi, ""), "i");
	};

	users_selector.prototype.clean_filter = function()
	{
		this.filter();
	};

	users_selector.prototype.filter = function(value, callback = $.noop)
	{
		const self = this;

		var end_filtering = function() {
			self.m_callbacks.hide_waiting();
			self.__reload_total_general_members_selected();
			self.update_total_rows_msg();
			callback();
		};

		const $list_container = $("#" + this.id_users_list);
		if (value === undefined) {
		// si no hay texto ha buscar en el filtro, no filtro la row
			this.hide_not_results_found();
			$list_container.find("li[user_data]").show();
			end_filtering();
			return;
		}
		
		var filter = function() {
			self.__add_users({
				users : self.m_users,
				att_levels: self.m_info?.campaign_att_levels,
				campaign_id: self.m_info?.campaign_id,
				waiting_message : _T("Searching..."),
				can_append_fn: function (member) {
					if (value === "") {
						return true;
					}
					const unique_id = member.Id.trim().toLowerCase();
					const user_name = member.Name || member.Id;
					const search_text = unique_id + user_name;
					let search_regexp = self.make_search_regexp(value);
					return search_text.search(search_regexp) !== -1; //Si encuentro la palabra la agrego
				},
				complete_cb: function(){
					if (self.m_total_rows_filtered > 0) {
						self.hide_not_results_found();
					}
					else {
						self.show_not_results_found();
					}
					const is_all_selected = self.m_total_rows_filtered > 0 && self.m_total_rows_filtered === self.m_total_selected_rows_filtered;
					$("#" + self.id_check_all).prop("checked", is_all_selected);
					end_filtering();
				},
			});
		};

		if (this.m_timeout_filter_search) {
			window.clearTimeout(this.m_timeout_filter_search);
			this.m_timeout_filter_search = null;
		}
		// Se agrega la busqueda en un timeout, asi se puede ver el waiting mientras se busca.
		this.m_timeout_filter_search = window.setTimeout(function() {
			window.clearTimeout(self.m_timeout_filter_search);
			self.m_timeout_filter_search = null;
			filter();
		}, 1000);
	};

	users_selector.prototype.__on_updated = function(success, error, on_updated = $.noop)
	{
		const self = this;

		if (success) {
			this.hide();
			on_updated(success, error);
			return;
		}
		// Error actualizando
		let error_message;
		let problematic_users;

		var mark_members_with_error = function()
		{
			if (!$.isArray(problematic_users)) {
				return;
			}
			const $users_list = $("#" + self.id_users_list);

			$.each(problematic_users, function(_, member)
			{
				let unique_id = member.id.trim().toLowerCase();
				const $user = $users_list.find("[user_data][user_id='" + unique_id + "']");
				if ($user.length > 0) {
					const $err = $user.find("i[error='true']");
					if ($user.length > 0) {
						$err.show();
					}
				}
			});
		};
		let raw_error = error;
		if (typeof error === "object") {
			problematic_users = error.problematicUsers;
			error = ( error.result || "").toUpperCase();
		}
		switch (error)
		{
			case "INVALID_REQUEST":
				error_message = _T("Invalid request.");
			break;

			case "SOME_USERS_HAS_INTERACTIONS":
				error_message = _T("Error updating users list, some users has assigned conversations, and can not be removed, please try again later...");
				mark_members_with_error();
			break;

			case "SOME_USERS_ASSIGNED_IN_ATTENTION_LEVELS":
				error_message = _T("Error updating users list, some users has assigned in attention levels, and can not be removed, please try again later...");
				mark_members_with_error();
			break;

			case "USER_HAS_INTERACTIONS_MEMBER_HAVE_NOT_PERMISSION_TO_INTERACTION_APP":
				var count = raw_error.problematicUsers[0].assigned_interactions;
				error_message = FORMAT(
					_T("User has {0} assigned {1}. Error updating users permissions, user does not have permission to interaction app ..."),
					count,
					count > 1 ? _T("conversations") : _T("conversation")
				);
				mark_members_with_error();
			break;

			case "USER_IN_ATTENTION_LEVELS_MEMBER_HAVE_NOT_PERMISSION_TO_INTERACTION_APP":
				error_message = _T("Attention levels conflict. Error updating users permissions, user does not have permission to interaction app ...");
				mark_members_with_error();
			break;

			case "FORBIDDEN":
				error_message = _T("You do not have permissions to save the users list.");
			break;

			case "ALLEGRO_INTEGRATION_ERROR":
				error_message = _T("Allegro integration error.");
			break;

			default:
				error_message = _T("Failed updating users list, please try again in a few minutes...");
		}
		this.show_error_msg(error_message);
		on_updated(success, error);
	};

	users_selector.prototype.compact_view_mode = function(checked)
	{
	
		const $user_list = $("#" + this.id_users_list);
		const $check_all_al = $("#" + this.id_check_all_al);
		const $elements_mode_full = $user_list.find("[view_mode_full]");

		//Se hace en un foreach_waiting, para dar un feedback y que no congele el navegador cuando son muchos rows
		const self = this;
			
			if (checked){
				$user_list.removeClass("viewModeFull");
				$elements_mode_full.hide();
				$check_all_al.prop("checked", false);
				$check_all_al.parent().hide();
				return; 
			}
			else if(this.m_info?.campaign_att_levels?.length > 0){
				$user_list.addClass("viewModeFull");
				$check_all_al.parent().show();
			}else{
				return;
			}

			inConcert.Utils.$foreach_timeout({
				rows: $elements_mode_full,
				append_fn: function (elem) {
					if (checked){
						$(elem).hide();
					}
					else {
						$(elem).show();
					}
					
				},
				waiting_fn: function (message) {
					self.m_callbacks.show_waiting(message);
				},
				complete_fn: function () {
					self.m_callbacks.hide_waiting();
					self.select_check_all_als();
				}
			});
	};

	users_selector.prototype.select_all_attlevels = function(checked_all) {	
		
		if (this.m_total_selected_rows_filtered === 0) {
			return;
		}

		const self = this;
		const $users = $("#" + this.id_users_list).find("li[user_data='true']:visible input[selector='1'][type='checkbox']");
		inConcert.Utils.$foreach_timeout({
			rows: $users ,
			append_fn: function (elem) {
				const $elem = $(elem);
				if ( $elem.prop("checked")) {
					$elem.parent().siblings().find("[att_level_id]").prop("checked", checked_all);
				}
			},
			waiting_fn: function (message) {
				self.m_callbacks.show_waiting(message);
			},
			complete_fn: function () {
				self.m_callbacks.hide_waiting();
				if (checked_all) {
					self.m_total_selected_als = self.m_total_selected_rows_filtered * (self.m_info?.campaign_att_levels.length || 0);
				}
				else{
					self.m_total_selected_als = 0;
				}
			}
		});
	};

	users_selector.prototype.__enable_disable_als = function(checked_all) {	
		const all_als = $("#" + this.id_users_list).find("input[att_level_id][type='checkbox']");
		const $chk_all_als = $("#" + this.id_check_all_al);
		$chk_all_als.prop("disabled", !checked_all);
		all_als.prop("disabled", !checked_all);
		$chk_all_als.prop("checked", false);

		if(!checked_all)
		{
			all_als.prop("checked", false);
		}
	};

	users_selector.prototype.select_check_all_als = function(alsByUsers) {	
		
		if(this.m_info?.campaign_att_levels){
			if (alsByUsers === undefined) {
				alsByUsers = this.m_info.campaign_att_levels.length * this.m_total_selected_rows_filtered;
			}
			if(alsByUsers === 0 && this.m_total_selected_als === 0)
			{
				$("#" + this.id_check_all_al).prop("checked", false);
			}
			else{
				$("#" + this.id_check_all_al).prop("checked", alsByUsers === this.m_total_selected_als );
			}
			
		}
	};

	users_selector.prototype.__select_all_users = function(checked_all) {	
		
		//Selecciono los usuarios usando foreach_timeout, para marcar y procesar los ALS
		// async, para el caso de muchos agentes no se bloquee el navegador
		const self = this;
		let total_rows_checked = 0;
		const $members = $("#" + self.id_users_list).find("li[user_data='true']:visible input[selector='1'][type='checkbox']");

		inConcert.Utils.$foreach_timeout({
			rows: $members, //array de usuarios
			append_fn: function (elem) {
				const $elem = $(elem);
				const is_already_checked = $elem.prop("checked");

				if (checked_all) {
					total_rows_checked++;

					if (!is_already_checked) {
						self.m_total_general_members_selected++;
					}
				}
				else{
					if (is_already_checked) {
						self.m_total_general_members_selected--;
					}
				}

				$elem.prop("checked", checked_all);
				//Obtengos los als del usuario y los marco/desmarco
				const $check_als = $elem.parent().siblings().find("[att_level_id]");
				$check_als.prop("disabled", !checked_all);

				if (!checked_all) {
					$check_als.prop("checked", false);
				}
			},
			waiting_fn: function (message) {
				self.m_callbacks.show_waiting(message);
			},
			complete_fn: function () {
				//actualizo titulo de usuarios seleccionado
				self.update_total_rows_msg();
				//actualizo los usuarios seleccionado y als.
				self.m_total_selected_rows_filtered = total_rows_checked;
				if (!checked_all) {
					self.m_total_selected_als = 0;
				}

				//Si no existe usuario chequeado, marco/desmarco, deshabilito/habilito el check de marcar todos los als
				self.select_check_all_als();
				const $chk_all_als = $("#" + self.id_check_all_al);
				$chk_all_als.prop("disabled", !checked_all);
				self.m_callbacks.hide_waiting();
			}
		});
	};

	

	users_selector.prototype.set_events = function()
	{
		const self = this;

		//evento check view mode compact
		$("#" + this.id_popup_check_view_mode)
		.off("click")
		.on("click", function() {
			const checked = $(this).prop("checked");
			self.compact_view_mode(checked);
		});
		// evento keyup del text de search
		$("#" + this.id_search_text)
			.off("keyup")
			.on("keyup", function() {
				let value = $(this).val();
				if ( self.m_timeout_filter_key_up ) {
					window.clearTimeout( self.m_timeout_filter_key_up );
					self.m_timeout_filter_key_up = null;
				}
				self.m_timeout_filter_key_up = window.setTimeout( function() {
					window.clearTimeout( self.m_timeout_filter_key_up );
					self.m_timeout_filter_key_up = null;
					self.filter(value);
				}, 500 );
			}
		);
		// click sobre el boton update para actualizar la lista
		$("#" + this.id_update_users_list)
			.off("click")
			.on("click", function() {
				self.m_callbacks.update();
			}
		);
		// click en el boton para cerrar el popup y descartar cambios
		$("#" + this.id_hide_popup)
			.off("click")
			.on("click", function() {
				self.hide();
			}
		);
		// click en el check all users
		$("#" + this.id_check_all)
			.off("click")
			.on("click", function() {
				const $this = $(this);
				let checked_all = $this.prop("checked");
				//Confirmar
				$.inconcertPopup.Show(
					_T("Confirm"),
					FORMAT(_T("Are you sure you want to {0} all users?"), checked_all? _T("select"):_T("deselect")),
					$.inconcertPopup.ButtonSets.BS_YESNO,
					function(response) {
						if (response.toLowerCase() === "yes") {
							self.__select_all_users(checked_all);
						}
						else{	
							$this.prop("checked", !checked_all);
						}
					},
					undefined,
					{
						popup_id       : "conf_inconcertPopup",
						container_class: "confirmationPopUp",
						overlay_class  : "confirmationOverlay"
					}
				);
			}
		);
		// evento click de los checkboxes de la lista de users.
		const event_selector = "input[selector='1'][type='checkbox'], [att_level_id]"; //Selector user y als
		const $list_container = $("#" + this.id_users_list);
		
			$list_container.off("click", event_selector)
			.on("click", event_selector, function(e) {

				const is_checked = $(this).prop("checked");
				if ($(e.target).attr("att_level_id")) {
					if (is_checked) {
						self.m_total_selected_als++;
					}
					else{
						self.m_total_selected_als--;
					}
					return self.select_check_all_als();
				}

				let $check = $(this).parent().siblings().find("[att_level_id]");
				const als_checked = $check.filter(":checked").length;
				if ( is_checked ) {
					self.m_total_selected_rows_filtered++;
					self.m_total_general_members_selected++;
					$check.prop("disabled", false);
				}
				else {
					self.m_total_selected_rows_filtered--;
					self.m_total_general_members_selected--;
					$check.prop("disabled", true);
					$check.prop("checked", false);
					self.m_total_selected_als = self.m_total_selected_als - als_checked;
				}
				$("#" + self.id_check_all).prop("checked", self.count_rows_filtered() === self.m_total_selected_rows_filtered );
				self.update_total_rows_msg();
				self.select_check_all_als();
				
				//Deshbilitar o habilitar el selec all de los als
				const $all_als = $("#" + self.id_check_all_al);
				const is_disabled_all_als = $all_als.is(':disabled');
				
				if (self.m_total_selected_rows_filtered > 0 && is_disabled_all_als) {
					$all_als.prop("disabled", false);
				}
				else if (self.m_total_selected_rows_filtered === 0 && !is_disabled_all_als) {
					$all_als.prop("disabled", true);
				}
			}
		);

		let fnConfirm = function(checked_all, $this) {
			$.inconcertPopup.Show(
				_T("Confirm"),
				FORMAT(_T("Are you sure you want to {0} all attention levels?"), checked_all? _T("select"):_T("deselect")),
				$.inconcertPopup.ButtonSets.BS_YESNO,
				function(response) {
					if (response.toLowerCase() === "yes") {	
						self.select_all_attlevels(checked_all);
						$("#" + self.id_check_all_al).prop("checked",checked_all);
					}
					else{
						$this.prop("checked", !checked_all);
					}
				},
				undefined,
				{
					popup_id       : "conf_inconcertPopup",
					container_class: "confirmationPopUp",
					overlay_class  : "confirmationOverlay"
				}
			);
		};
		// click en el check all
		$("#" + this.id_check_all_al)
			.off("click")
			.on("click", function() {
				const $this = $(this);
				let checked_all = $this.prop("checked");
				fnConfirm(checked_all, $this);
			}
		);
			
	};
}());