/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:true, curly:true, browser:true, indent:false, maxerr:50, quotmark:false, esversion: 9 */

/* global $, mejs, swfobject, MediaElementPlayer, inConcert */


(function () {
	"use strict";

	if (!String.prototype.endsWith) {
	  String.prototype.endsWith = function(searchString, position) {
		  var subjectString = this.toString();
		  if (typeof position !== 'number' || !isFinite(position) || Math.floor(position) !== position || position > subjectString.length) {
			position = subjectString.length;
		  }
		  position -= searchString.length;
		  var lastIndex = subjectString.indexOf(searchString, position);
		  return lastIndex !== -1 && lastIndex === position;
	  };
	}

	function CInconcertPlayer(params) {
		function overrideLanguage() {
			var _T = inConcert.i18n.getString;
			mejs.i18n.t = _T;
			$.each(mejs.MepDefaults, function(key, value) {
				if (key.endsWith("Text") && typeof value === "string") {
					mejs.MepDefaults[key] = _T(value);
				}
			});
		}

		function getFileType(fileFormat) {
			switch(fileFormat) {
				case "mp3":
				case "wav":
				case "webm":
				case "ogg":
				case "oga":
				case "aac":
				case "m4a":
					return "audio";
				case "flv":
				case "mp4":
				case "avi":
				case "3gp":
					return "video";
				default:
					throw "unsuported type: " + fileFormat;
			}
		}

		function getContentType(fileType, fileFormat) {
			switch(fileFormat) {
				case "mp3":
				case "wav":
				case "ogg":
				case "oga":
					return "audio/mpeg";
				case "aac":
					return "audio/aac";
				case "m4a":
					return "audio/mp4";
				case "webm":
					return fileType === "audio"? 
						"audio/mpeg" : "video/webm";
				case "flv":
				case "mp4":
				case "avi":
					return "video/mp4";
				case "3gp":
					return "video/3gpp";
				default:
					throw "unsuported type: " + fileFormat;
			}
		}

		//override mediaelement languages function
		overrideLanguage();

		this.containerId = params.containerId;
		this.url = params.url;
		this.fileFormat = params.fileFormat;
		this.fileType = params.fileType || getFileType(this.fileFormat);
		this.contentType = params.contentType || getContentType(this.fileType, this.fileFormat);
		this.isLive = params.isLive || false;
		this.success = params.success || function() {};
		this.error = params.error || function() {};
		this.events = {};
	}
	
	window.CInconcertPlayer = CInconcertPlayer;

	///Methods

	CInconcertPlayer.prototype.load = function() {
		var self = this;

		var usesFlash = false;
		//if (self.isLive && inConcert.Utils.browserDetect().name !== "FIREFOX") {
		//	if (swfobject.hasFlashPlayerVersion("10")) {
		//		usesFlash = true;
		//	}
		//	else {
		//		inConcert.app.enableFlash();
		//	}
		//}

		function makeHtmlTag() {
			var tag = "";
			switch(self.fileType) {
				case "audio":
					tag =	'<audio id="' + self.containerId + '_tag" controls preload="none"' +
							(self.isLive? ' autoplay' : '') +
							'>' +
								'<source src="' + self.url + '" type="' + self.contentType + '" />' +
							'</audio>';
					break;
				case "video":
				case "screen_recording":
					tag =	'<video width="100%" height="100%" id="' + self.containerId + '_tag" controls preload="none"' +
							(self.isLive? ' autoplay' : '') +
							'>' +
								'<source src="' + self.url + '" type="' + self.contentType + '" />' +
							'</video>';
					break;
			}
			return tag;
		}

		function getContainerClass() {
			var containerClass = "inConcertPlayer";
			switch(self.fileType) {
				case "audio":
					containerClass += "Audio";
					break;
				case "video":
				case "screen_recording":
					containerClass += "Video";
					break;
			}
			containerClass += self.isLive? "Live" : "";
			return containerClass;
		}

		function getFeatures() {
			switch(self.fileType) {
				case "audio":
					return self.isLive? 
						  ['volume'] 
						: ['playpause', 'stop', 'current','progress','duration','volume'];
				case "video":
				case "screen_recording":
					return self.isLive? 
						  ['volume'] 
						: ['playpause', 'stop', 'current','progress','duration','volume','fullscreen'];
			}
			return null;
		}

		function makePlayerOptions() {
			var events = [
				"loadeddata",
				"progress",
				"timeupdate",
				"seeked",
				"canplay",
				"play",
				"playing",
				"pause",
				"loadedmetadata",
				"ended",
				"volumechange"
			];
			var options = {
				success: function(player) {
					self.m_player = player;
					window.player = player;
					$.each(events, function(_, event) {
						player.addEventListener(event, function() {
							if (self.events[event]) {
								$.each(self.events[event], function(_, event) {
									event();
								});
							}
						});
					});
					self.success();
				},
				error: function() {
					self.error();
				},
				features: getFeatures(),
				startVolume: 1.0,
				mode: usesFlash ? "shim" : undefined,
				pluginPath: "/inconcert/lib/mediaelement/"
			};
			return options;
		}

		try {
			var $tag = $(makeHtmlTag());
			$tag.on("error", function() {
				self.error();
			});
			var $container = $("#" + self.containerId);
			$container.html($tag);

			new MediaElementPlayer("#" + self.containerId + "_tag", makePlayerOptions());
			$container.children(".mejs-container").addClass(getContainerClass());
		}
		catch(e) {
			self.error();
		}
	};

	CInconcertPlayer.prototype.show = function() {
		$("#" + this.containerId)
				.show()
				.children()
				.show();
		$("#" + this.containerId)
				.find(".mejs-horizontal-volume-slider")
				.show();
	};

	CInconcertPlayer.prototype.hide = function() {
		$("#" + this.containerId)
				.hide()
				.children()
				.hide();
	};

	CInconcertPlayer.prototype.destroy = function() {
		if (this.m_player) {
			this.stop();
			this.m_player = null;
		}
		$("#" + this.containerId).html("");
	};

	CInconcertPlayer.prototype.play = function() {
		this.m_player.play();
	};

	CInconcertPlayer.prototype.pause = function() {
		this.m_player.pause();
	};

	CInconcertPlayer.prototype.stop = function() {
		this.m_player.stop();
	};

	CInconcertPlayer.prototype.isPaused = function() {
		return $("#" + this.containerId + "_tag").prop("paused");
	};
	CInconcertPlayer.prototype.isEnded = function() {
		return $("#" + this.containerId + "_tag").prop("ended");
	};
	CInconcertPlayer.prototype.isPlaying = function() {
		return !this.isPaused() && ! this.isEnded();
	};

	//0 - 1
	CInconcertPlayer.prototype.getVolume = function() {
		return this.m_player.getVolume();
	};
	CInconcertPlayer.prototype.setVolume = function(volume) {
		this.m_player.setVolume(volume);
	};

	//seconds
	CInconcertPlayer.prototype.getCurrentTime = function() {
		return this.m_player.getVolume();
	};
	CInconcertPlayer.prototype.setCurrentTime = function(time) {
		this.m_player.setCurrentTime(time);
	};

	CInconcertPlayer.prototype.isMuted = function() {
		return $("#" + this.containerId + "_tag").prop("muted");
	};
	CInconcertPlayer.prototype.setMuted = function(muted) {
		this.m_player.setMuted(muted);
	};

	CInconcertPlayer.prototype.getSources = function() {
		var sources = [];
		var $container = $("#" + this.containerId);
		if ($container.children("source").length) {
			$container.children("source").each(function() {
				sources.push($(this).attr("src"));
			});
		}
		else {
			sources.push($container.attr("src"));
		}
		return sources;
	};
	CInconcertPlayer.prototype.setSource = function(source) {
		this.m_player.setSrc(source);
	};

	///Events

	CInconcertPlayer.prototype.on = function(methodId, method) {
		this.events[methodId] = this.events[methodId] || [];
		this.events[methodId].push(method);
		return this;
	};

	CInconcertPlayer.prototype.off = function(methodId) {
		this.events[methodId] = null;
		return this;
	};

	CInconcertPlayer.prototype.onLoadedData = function(method) {
		return this.on("loadeddata", method);
	};

	CInconcertPlayer.prototype.onProgress = function(method) {
		return this.on("progress", method);
	};

	CInconcertPlayer.prototype.onTimeUpdate = function(method) {
		return this.on("timeupdate", method);
	};

	CInconcertPlayer.prototype.onSeeked = function(method) {
		return this.on("seeked", method);
	};

	CInconcertPlayer.prototype.onCanPlay = function(method) {
		return this.on("canplay", method);
	};

	CInconcertPlayer.prototype.onPlay = function(method) {
		return this.on("play", method);
	};

	CInconcertPlayer.prototype.onPlaying = function(method) {
		return this.on("playing", method);
	};

	CInconcertPlayer.prototype.onPause = function(method) {
		return this.on("pause", method);
	};

	CInconcertPlayer.prototype.onLoadedMetadata = function(method) {
		return this.on("loadedmetadata", method);
	};

	CInconcertPlayer.prototype.onEnded = function(method) {
		return this.on("ended", method);
	};

	CInconcertPlayer.prototype.onVolumeChange = function(method) {
		return this.on("volumechange", method);
	};

}());