/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, esversion: 11 */

/* global $, inConcert, URLSearchParams */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	const _T = inConcert.i18n.getString;
	const FORMAT = inConcert.Utils.format;
	const MAX_UPLOAD_IMAGES_SIZE_BYTES = 25600; // bytes
	const MAX_UPLOAD_IMAGES_SIZE_KB = MAX_UPLOAD_IMAGES_SIZE_BYTES / 1024; // 25 Kb
	const URI_REPOSITORY_GET = "/inconcert/apps/repository/download/";

	let URI_DOMAIN;

	//---------------------------------------------------------------------
	// E-mail Management
	function MailManagement(params)
	{
		const self = this;

		if (!params) {
			throw("MailManagement - 'params' is required");
		}
		if (!params.owner) {
			throw("MailManagement - 'params.owner' is required");
		}
		if (typeof params.owner.show_status_message !== "function") {
			throw("MailManagement - 'params.owner.show_status_message' is not a function");
		}

		URI_DOMAIN = inConcert.app.getBaseDomain();

		this.messages = {
		// upload file
			err_upload_file_not_exist  : _T("Unable to upload file. File does not exist."),
			err_upload_file_only_images: _T("Unable to upload file. Only images are allowed."),
			err_upload_file_size       : _T("Unable to upload file of {0} Kb size. Max upload file size is {1} Kb."),
			err_upload_files_size      : _T("Unable to upload file of {0} Kb size. Uploaded files exceeding the {1} Kb."),
			err_save_files_size        : _T("Unable to save. Uploaded files exceeding the {0} Kb.")
		};

		this.owner            = params.owner;
		this.id               = params.ids.body;
		this.container_id     = params.ids.container;
		this.disclaimer_id    = this.id + "_disclaimer";
		this.popup_id         = this.id + "_popup";
		this.popup_close_id   = this.id + "_popup_close";
		this.popup_content_id = this.id + "_popup_content";
		this.popup_message_id = this.id + "_popup_message";
		this.popup_update_id  = this.id + "_popup_update";
		this.m_editor_saved_selection = null;
		this.temporal_repository_images = [];
		this.variables = [];

		let callbacks = params.callbacks || {};
		this.callbacks = {
			loadVariables: function(callback) {
				const sorted = true;
				if (!callbacks.loadVariables) {
					inConcert.app.loadVariablesForSignatures(sorted, callback);
					return;
				}
				callbacks.loadVariables(true, callback);
			}
		};

		this.callbacks.loadVariables(function(success, variables) {
			if (!success) {
				throw("Unable to load loadVariables");
			}
			self.variables = variables;
		});

		this.__make_popup_html();
		if (params.auto_init) {
			this.initialize();
			this.set_events();
		}
		return true;
	}

	if (typeof inConcert.MailManagement !== "function") {
		inConcert.MailManagement = MailManagement;
	}

	MailManagement.prototype.initialize = function()
	{
		const self = this;

		this.temporal_repository_images = [];
		/*
		http://www.jqueryrain.com/?0gvIIXi7
		https://github.com/wysiwygjs/wysiwyg.js
		*/
		const $mail = $("#" + this.id);

		$mail.wysiwyg({
			classes: "some-more-classes",
			// 'selection'|'top'|'top-selection'|'bottom'|'bottom-selection'
			toolbar: "top-selection",
			buttons: {

				insertimage: {
					title: _T("Insert image"),
					image: "\uf03e", // <img src="path/to/image.png" width="16" height="16" alt="" />
					showstatic: true,    // wanted on the toolbar
				},
				insertlink: {
					title: _T("Insert link"),
					image: "\uf08e", // <img src="path/to/image.png" width="16" height="16" alt="" />
					classes: 'editor-right-separator',
				},
				// Fontname plugin
				fontname: {
					title: _T("Font"),
					image: "\uf031", // <img src="path/to/image.png" width="16" height="16" alt="" />
					popup: function( $popup/*, $button*/ ) {
							var fonts = inConcert.app.loadFontsList();
							var $list = $("<div/>").addClass("wysiwyg-plugin-list")
													.attr("unselectable","on");
							$.each( fonts, function( _, font ) {
								var $link = $("<a/>").attr("href","#")
													.css( "font-family", font.id )
													.html( font.name )
													.click(function(e) {
														$mail.wysiwyg("shell").fontName(font.id).closePopup();
														// prevent link-href-#
														e.stopPropagation();
														e.preventDefault();
														return false;
													});
								$list.append( $link );
							});
							$popup.append( $list );
						},
					showstatic: true,    // wanted on the toolbar
				},
				// Fontsize plugin
				fontsize: {
					title: _T("Size"),
					image: "\uf034", // <img src="path/to/image.png" width="16" height="16" alt="" />
					popup: function( $popup/*, $button*/ ) {
							// Hack: http://stackoverflow.com/questions/5868295/document-execcommand-fontsize-in-pixels/5870603#5870603
							var list_fontsizes = [];
							var i;
							for( i=8; i <= 11; ++i ) {
								list_fontsizes.push(i+"px");
							}
							for( i=12; i <= 28; i+=2 ) {
								list_fontsizes.push(i+"px");
							}
							list_fontsizes.push("36px");
							list_fontsizes.push("48px");
							list_fontsizes.push("72px");
							var $list = $("<div/>").addClass("wysiwyg-plugin-list")
													.attr("unselectable","on");
							$.each( list_fontsizes, function( _, size ) {
								var $link = $("<a/>").attr("href","#")
													.html( size )
													.click(function(e) {
														$mail.wysiwyg("shell").fontSize(7).closePopup();
														$mail.wysiwyg("container")
																.find("font[size=7]")
																.removeAttr("size")
																.css("font-size", size);
														// prevent link-href-#
														e.stopPropagation();
														e.preventDefault();
														return false;
													});
								$list.append( $link );
							});
							$popup.append( $list );
						}
				},
				// Header plugin
				header: {
					title: _T("Header"),
					image: "\uf1dc", // <img src="path/to/image.png" width="16" height="16" alt="" />
					classes: 'editor-right-separator',
					popup: function( $popup/*, $button*/ ) {
							var list_headers = {
									// Name : Font
									"Header 1": "<h1>",
									"Header 2": "<h2>",
									"Header 3": "<h3>",
									"Header 4": "<h4>",
									"Header 5": "<h5>",
									"Header 6": "<h6>",
									"Code"    : "<pre>"
								};
							var $list = $("<div/>").addClass("wysiwyg-plugin-list")
													.attr("unselectable","on");
							$.each( list_headers, function( name, format ) {
								var $link = $("<a/>").attr("href","#")
														.css( "font-family", format )
														.html( name )
														.click(function(e) {
															$mail.wysiwyg("shell").format(format).closePopup();
															// prevent link-href-#
															e.stopPropagation();
															e.preventDefault();
															return false;
														});
								$list.append( $link );
							});
							$popup.append( $list );
						}
				},
				bold: {
					title: _T("Bold (Ctrl+B)"),
					image: "\uf032", // <img src="path/to/image.png" width="16" height="16" alt="" />
					hotkey: "b"
				},
				italic: {
					title: _T("Italic (Ctrl+I)"),
					image: "\uf033", // <img src="path/to/image.png" width="16" height="16" alt="" />
					hotkey: "i"
				},
				underline: {
					title: _T("Underline (Ctrl+U)"),
					image: "\uf0cd", // <img src="path/to/image.png" width="16" height="16" alt="" />
					hotkey: "u"
				},
				strikethrough: {
					title: _T("Strikethrough (Ctrl+S)"),
					image: "\uf0cc", // <img src="path/to/image.png" width="16" height="16" alt="" />
					hotkey: "s",
					classes: 'editor-right-separator',
				},
				forecolor: {
					title: _T("Text color"),
					image: "\uf1fc" // <img src="path/to/image.png" width="16" height="16" alt="" />
				},
				highlight: {
					title: _T("Background color"),
					image: "\uf576", // <img src="path/to/image.png" width="16" height="16" alt="" />
					classes: 'editor-right-separator',
				},
				alignleft: {
					title: _T("Left"),
					image: "\uf036", // <img src="path/to/image.png" width="16" height="16" alt="" />
					showselection: false    // wanted on selection
				},
				aligncenter: {
					title: _T("Center"),
					image: "\uf037", // <img src="path/to/image.png" width="16" height="16" alt="" />
					//showstatic: true,    // wanted on the toolbar
					showselection: false    // wanted on selection
				},
				alignright: {
					title: _T("Right"),
					image: "\uf038", // <img src="path/to/image.png" width="16" height="16" alt="" />
					showselection: false    // wanted on selection
				},
				alignjustify: {
					title: _T("Justify"),
					image: "\uf039", // <img src="path/to/image.png" width="16" height="16" alt="" />
					showselection: false,    // wanted on selection
					classes: 'editor-right-separator',
				},
				subscript: {
					title: "Subscript",
					image: "\uf12c", // <img src="path/to/image.png" width="16" height="16" alt="" />
					showselection: true    // wanted on selection
				},
				superscript: {
					title: _T("Superscript"),
					image: "\uf12b", // <img src="path/to/image.png" width="16" height="16" alt="" />
					//showstatic: true,    // wanted on the toolbar
					showselection: true    // wanted on selection
				},
				indent: {
					title: _T("Indent"),
					image: "\uf03c", // <img src="path/to/image.png" width="16" height="16" alt="" />
					showselection: false    // wanted on selection
				},
				outdent: {
					title: _T("Outdent"),
					image: "\uf03b", // <img src="path/to/image.png" width="16" height="16" alt="" />
					//showstatic: true,    // wanted on the toolbar
					showselection: false    // wanted on selection
				},
				orderedList: {
					title: _T("Ordered list"),
					image: "\uf0cb", // <img src="path/to/image.png" width="16" height="16" alt="" />
					showselection: false    // wanted on selection
				},
				unorderedList: {
					title: _T("Unordered list"),
					image: "\uf0ca", // <img src="path/to/image.png" width="16" height="16" alt="" />
					showselection: false    // wanted on selection
				},

			// custom buttons
				// add variables
				vars: {
					title: _T("Variables"),
					image: "\uf7ea", // <img src="path/to/image.png" width="16" height="16" alt="" />
					popup: function( $popup/*, $button*/ )
					{
						let variables = self.variables;
						var $list = $("<div/>").addClass("wysiwyg-plugin-list").attr("unselectable","on");
						$.each( variables, function(_, variable) {
							var $link = $("<a/>").attr("href","#")
								.html(variable.name)
								.click(function(e) {
									$mail.wysiwyg("shell").insertHTML(variable.id).closePopup();
									// prevent link-href-#
									e.stopPropagation();
									e.preventDefault();
									return false;
								}
							);
							$list.append($link);
						});
						$popup.append($list);
					},
					showstatic: true// wanted on the toolbar
				},
				// add variables
				edit_html: {
					title: _T("HTML"),
					image: "\uf121", // <img src="path/to/image.png" width="16" height="16" alt="" />
					popup: function( /*$popup, $button*/ )
					{
						self.show_popup_html_edit();
					},
					showstatic: true,    // wanted on the toolbar
				}
			},
			// Submit-Button
			submit: {
				title: _T("Submit"),
				image: "\uf00c" // <img src="path/to/image.png" width="16" height="16" alt="" />
			},
			// Other properties
			selectImage: _T("Click image"),
			placeholderUrl: "www.example.com",
			placeholderEmbed: "<embed/>",
			maxImageSize: [600,200],
			validateUploadImageSize : function( file, e, callback ) {
				self.validate_upload_image( file, e, callback );
			},
			onKeyDown: function( /*key, character, shiftKey, altKey, ctrlKey, metaKey*/ ) {
							// E.g.: submit form on enter-key:
							//if( (key == 10 || key == 13) && !shiftKey && !altKey && !ctrlKey && !metaKey ) {
							//    submit_form();
							//    return false; // swallow enter
							//}
						},
			onKeyPress: function( /*key, character, shiftKey, altKey, ctrlKey, metaKey*/ ) {
						},
			onKeyUp: function( /*key, character, shiftKey, altKey, ctrlKey, metaKey*/ ) {
						},
			onAutocomplete: function( typed, key, character/*, shiftKey, altKey, ctrlKey, metaKey*/ ) {
							if( typed.indexOf("@") === 0 ) // startswith "@"
							{
								var usernames = [
										"Evelyn",
										"Harry",
										"Amelia",
										"Oliver",
										"Isabelle",
										"Eddie",
										"Editha",
										"Jacob",
										"Emily",
										"George",
										"Edison"
									];
								var $list = $("<div/>").addClass("wysiwyg-plugin-list")
														.attr("unselectable","on");
								$.each( usernames, function( _, username ) {
									if( username.toLowerCase().indexOf(typed.substring(1).toLowerCase()) !== 0 ) { // don't count first character "@"
										return;
									}
									var $link = $("<a/>").attr("href","#")
														.text( username )
														.click(function(e) {
															var url = "http://example.com/user/" + username,
																html = "<a href='" + url + "'>@" + username + "</a> ";
															var editor = $mail.wysiwyg("shell");
															// Expand selection and set inject HTML
															editor.expandSelection( typed.length, 0 ).insertHTML( html );
															editor.closePopup().getElement().focus();
															// pre link-href-#
															e.stopPropagation();
															e.preventDefault();
															return false;
														});
									$list.append( $link );
								});
								if( $list.children().length )
								{
									if( key === 13 )
									{
										$list.children(":first").click();
										return false; // swallow enter
									}
									// Show popup
									else if( character || key === 8 ) {
										return $list;
									}
								}
							}
						},
			onImageUpload: function( /*insert_image*/ ) {
							// A bit tricky, because we can't easily upload a file via
							// '$.ajax()' on a legacy browser without XMLHttpRequest2.
							// You have to submit the form into an '<iframe/>' element.
							// Call 'insert_image(url)' as soon as the file is online
							// and the URL is available.
							// Example server script (written in PHP):
							/*
							<?php
							$upload = $_FILES['upload-filename'];
							// Crucial: Forbid code files
							$file_extension = pathinfo( $upload['name'], PATHINFO_EXTENSION );
							if( $file_extension != 'jpeg' && $file_extension != 'jpg' && $file_extension != 'png' && $file_extension != 'gif' )
								die("Wrong file extension.");
							$filename = 'image-'.md5(microtime(true)).'.'.$file_extension;
							$filepath = '/path/to/'.$filename;
							$serverpath = 'http://domain.com/path/to/'.$filename;
							move_uploaded_file( $upload['tmp_name'], $filepath );
							echo $serverpath;
							*/
							// Example client script (without upload-progressbar):
						/*
							var iframe_name = "legacy-uploader-" + Math.random().toString(36).substring(2);
							$("<iframe>").attr("name",iframe_name)
											.load(function() {
												// <iframe> is ready - we will find the URL in the iframe-body
												var iframe = this;
												var iframedoc = iframe.contentDocument ? iframe.contentDocument :
																(iframe.contentWindow ? iframe.contentWindow.document : iframe.document);
												var iframebody = iframedoc.getElementsByTagName("body")[0];
												var image_url = iframebody.innerHTML;
												insert_image( image_url );
												$(iframe).remove();
											})
											.appendTo(document.body);
							var $input = $(this);
							$input.attr("name","upload-filename")
									.parents("form")
									.attr("action","/script.php") // accessing cross domain <iframes> could be difficult
									.attr("method","POST")
									.attr("enctype","multipart/form-data")
									.attr("target",iframe_name)
									.submit();
						*/
						},
			forceImageUpload: false,    // upload images even if File-API is present
			videoFromUrl: function( url ) {
				// Contributions are welcome :-)

				// youtube - http://stackoverflow.com/questions/3392993/php-regex-to-get-youtube-video-id
				var youtube_match = url.match( /^(?:http(?:s)?:\/\/)?(?:[a-z0-9.]+\.)?(?:youtu\.be|youtube\.com)\/(?:(?:watch)?\?(?:.*&)?v(?:i)?=|(?:embed|v|vi|user)\/)([^\?&\"'>]+)/ );
				if( youtube_match && youtube_match[1].length === 11 ) {
					return "<iframe src='//www.youtube.com/embed/" + youtube_match[1] + "' width='640' height='360' frameborder='0' allowfullscreen></iframe>";
				}
				// vimeo - http://embedresponsively.com/
				var vimeo_match = url.match( /^(?:http(?:s)?:\/\/)?(?:[a-z0-9.]+\.)?vimeo\.com\/([0-9]+)$/ );
				if( vimeo_match ) {
					return "<iframe src='//player.vimeo.com/video/" + vimeo_match[1] + "' width='640' height='360' frameborder='0' webkitAllowFullScreen mozallowfullscreen allowFullScreen></iframe>";
				}
				// dailymotion - http://embedresponsively.com/
				var dailymotion_match = url.match( /^(?:http(?:s)?:\/\/)?(?:[a-z0-9.]+\.)?dailymotion\.com\/video\/([0-9a-z]+)$/ );
				if( dailymotion_match ) {
					return "<iframe src='//www.dailymotion.com/embed/video/" + dailymotion_match[1] + "' width='640' height='360' frameborder='0' webkitAllowFullScreen mozallowfullscreen allowFullScreen></iframe>";
				}

				// undefined -> create "<video/>" tag
			}
		});

		this.m_editor_saved_selection = null;
		const $mail_editor = this.$get_body_editor();
		$mail_editor.off("focusin")
			.on("focusin", function() {
				self.m_editor_saved_selection = null;
			}
		);
		$mail_editor.off("focusout")
			.on("focusout", function() {
				// Cuando el editor pierde el foco, almaceno la posicion donde se encuentra el mismo.
				//  Posteriormente puede ser utilizado en el drag & drop de imagenes desde el file system,
				//    ya que sino la imagen es agregada siempre al final del body, en lugar de ser agregado donde se encuentra el cursor del editor.
				self.m_editor_saved_selection = self.editor_save_selection();
			}
		);

		// inicia los eventos para realizar resize de las imagenes.
		this.initialize_image_resize();
		// inicia los eventos para realizar drag & drop de imagenes desde el file system.
		this.initialize_file_system_files_drag_and_drop();
	};


	MailManagement.prototype.__make_popup_html = function()
	{
		var $popup = $("#" + this.popup_id);
		if ($popup.length > 1) {
			return;
		}
		$popup = $(
			'<div id="' + this.popup_id + '" class="internalPopUpContainer signaturePopUp darkBodyPopUp popUp520w" style="display:none;">' +
				'<div class="internalPopUpOverlay"></div>' +
				'<div class="internalPopUp">' +
					'<div class="internalPopUpTitle">' +
					'<span>' + _T('HTML E-mail') + '</span>' +
					'<button id="' + this.popup_close_id + '_times" class="internalPopUpCloseButton"><i class="far fa-times"></i></button>' +
					'</div>' +
					'<div class="internalPopUpMiddle">' +
						'<div class="row">' +
							'<div class="col-12">' +
								'<div id="' + this.popup_message_id + '" class="genericFormMessagesContainer messageError" style="display:none;">FIELD ERROR MESSAGE</div>' +
								'<textarea id="' + this.popup_content_id + '" class="formTextarea"></textarea>' +
							'</div>' +
						'</div>' +
					'</div>' +
					'<div class="internalPopUpButtons internal">' +
						'<button id="' + this.popup_update_id + '" class="pageButtonGeneric buttonColorBlack">' + _T('Confirm') + '</button>' +
						'<button id="' + this.popup_close_id + '" class="pageButtonGeneric buttonColorGray">' + _T('Close') + '</button>' +
					'</div>' +
				'</div>' +
			'</div>'
		);

		$("body").append($popup);
	};

	MailManagement.prototype.dispose = function()
	{
		//TODO: implementar, si es necesario
	};

	MailManagement.prototype.get = function()
	{
		//text area is not coming out with modifications using .val() nor .text()
		var html = document.getElementById(this.id).value;
		html = this.__cleanDomainImages(html);
		return html;
	};

	MailManagement.prototype.__cleanDomainImages = function(html)
	{
		let domain = window.location.protocol + "//" + window.location.host;
		let path = URI_REPOSITORY_GET;
		let fullPath = domain + path;
		while (html.indexOf(fullPath) >= 0) {
			html = html.replace(fullPath, path);
		}
		return html;
	};

	MailManagement.prototype.get_plain_text = function()
	{
		var html = this.get();
		var plain_text = inConcert.Utils.htmlBodyToPlainText(html);
		return plain_text;
	};

	MailManagement.prototype.set = function(mail_html)
	{
		$("#" + this.id).wysiwyg("shell").setHTML(mail_html);
	};

	MailManagement.prototype.get_body_parts = function()
	{
		var html = this.get();
		return inConcert.Utils.makeBodyParts(html, false);
	};

	MailManagement.prototype.set_body_parts = function(parts)
	{
		var html_part;
		$.each(parts, function(_, part)
		{
			if (part.type === "text/html") {
				html_part = part;
				return false; // break
			}
		});
		var mail_html;
		if (html_part) {
			mail_html = html_part.content;
		}
		mail_html = mail_html || "";
		this.set(mail_html);
	};

	MailManagement.prototype.get_repository_relation = function()
	{
		if (!this.owner.get_repository_relation) {
			return;
		}
		return this.owner.get_repository_relation();
	};

	MailManagement.prototype.show_popup_html_edit = function()
	{
		var mail_html = this.get();
		$("#" + this.popup_id).show();
		$("#" + this.popup_content_id).val(mail_html).focus();
	};

	MailManagement.prototype.hide_popup_html_edit = function()
	{
		$("#" + this.popup_message_id).hide();
		$("#" + this.popup_content_id).val("");
		$("#" + this.popup_id).hide();
		$("#" + this.id).wysiwyg("shell").getElement().focus();
	};

	MailManagement.prototype.update = function()
	{
		var modified_html = $("#" + this.popup_content_id).val();
		var validate_html = function()
		{
			var result = {success: true};
			if (modified_html.length === 0) {
				return result;
			}
			/*
			// TODO: realizar una validacion de las sintaxis HTML. Ver si es posible, estuve mirando otros plugins y no validan nada.
			if ( false ) {
				result = {success: false, msg_error: "Invalid HTML"};
			}
			*/

			let validation_result = inConcert.Utils.validate(modified_html, "html_malicious_code");
			if ( !validation_result.success ) {
				// valida codigo malicioso
				result.success = false;
				result.msg_error = _T('Possible malicious code in the field value.');
			}
			return result;
		};
		var response = validate_html();
		if ( !response.success ) {
			var msg_error = response.msg_error;
			$("#" + this.popup_message_id).text(msg_error).show();
			return;
		}
		this.set(modified_html);
		this.hide_popup_html_edit();
	};

	MailManagement.prototype.$get_body_editor = function() {
		var $container = $("#" + this.container_id);
		return $container.find("[contenteditable]");
	};

	MailManagement.prototype.initialize_image_resize = function()
	{
		if (this.owner.is_readonly()) {
			return;
		}
		const $mail_editor = this.$get_body_editor();
		inConcert.Utils.initialize_image_resize($mail_editor);
	};

	MailManagement.prototype.initialize_file_system_files_drag_and_drop = function()
	{
		const self = this;

		if (this.owner.is_readonly()) {
			return;
		}

		const $mail_editor = this.$get_body_editor();
		inConcert.Utils.initialize_file_system_files_drag_and_drop($mail_editor,
			function(file, callback) {
				self.editor_restore_selection();
				self.validate_upload_image(file, undefined, function(success) {
					callback(success);
				});
			}
		);
	};

	MailManagement.prototype.get_uploaded_images_size = function( callback )
	{
		// TODO: realizar la obtencion del tamanio exacto del file como se realizo en el cliente de mail, utilizando "window.dataURLtoBlob"
		// "canavas to blob":
		//   https://github.com/blueimp/JavaScript-Canvas-to-Blob
		//   /inconcert/lib/canvas-to-blob/js/canvas-to-blob.js
		//
		callback = callback || $.noop;
		var total_size = 0;
		var $mail_editor = this.$get_body_editor();
		var $images = $mail_editor.find("img[src*='base64']");

		$.each($images, function(_, img)
		{
			// Aca esta la magia, paso base64 a binario para que posteriormente se pueda obtener el tamano exacto del file.
			// A esta conversion la realiza "canavas to blob":
			//   https://github.com/blueimp/JavaScript-Canvas-to-Blob
			//   /inconcert/lib/canvas-to-blob/js/canvas-to-blob.js
			//
			var $img = $(img);
			var base64_data = $img.attr("src");
			var blob = window.dataURLtoBlob && window.dataURLtoBlob(base64_data);
			total_size = total_size + blob.size; // voy incrementando al current file size, el tamanio de los archivos embebidos.
		});

		callback(total_size);
	};

	MailManagement.prototype.show_status_message = function(options)
	{
		var message = options.message;
		var type    = options.type || "Error";
		var time_ms = options.time_ms || 5000;

		this.owner.show_status_message(
			message,
			type,
			time_ms
		);
	};

	MailManagement.prototype.validate_upload_image = function(file, _, callback = $.noop)
	{
		const self = this;

		if (!file) {
			this.show_status_message({
				message: this.messages.err_upload_file_not_exist,
			});
			callback(false);
			return;
		}
		if (!file.type.match(/^image/i)) {
			this.show_status_message({
				message: this.messages.err_upload_file_only_images,
			});
			callback(false);
			return;
		}

		const file_size_kb = parseInt(file.size / 1024, 10);
		if (file.size > MAX_UPLOAD_IMAGES_SIZE_BYTES) {
			this.show_status_message({
				message: FORMAT(this.messages.err_upload_file_size, file_size_kb, MAX_UPLOAD_IMAGES_SIZE_KB),
			});

			callback(false);
		}
		else {
			this.get_uploaded_images_size(function( uploaded_images_size )
			{
				if ( (uploaded_images_size + file.size) > MAX_UPLOAD_IMAGES_SIZE_BYTES ) {
					self.show_status_message({
						message: FORMAT(self.messages.err_upload_files_size, file_size_kb, MAX_UPLOAD_IMAGES_SIZE_KB)
					});
					callback(false);
					return;
				}
				callback(true);
			});
		}
	};

	MailManagement.prototype.validate = function(callback)
	{
		var self = this;
		callback = callback || $.noop;

		var html_body = this.get();
		var response = inConcert.Utils.validate(html_body, "required");

		if (!response.success) {
			callback(response);
			return;
		}

		this.get_uploaded_images_size(function(uploaded_images_size)
		{
			response = {success: true};
			if (uploaded_images_size > MAX_UPLOAD_IMAGES_SIZE_BYTES) {
				response.message = FORMAT(self.messages.err_save_files_size, MAX_UPLOAD_IMAGES_SIZE_KB);
				response.success = false;
			}
			callback(response);
		});
	};

	MailManagement.prototype.get_embedded_images_in_body = function($body, generate_id)
	{
		generate_id = typeof generate_id === "undefined" ? true : false;
		// obtengo los <IMG con el attr SRC de base64 y los agrego al array
		var embedded_images = [];
		var $images = $body.find("img[src*='base64']");
		$.each($images, function(_, img)
		{
			var $img = $(img);
			var image_info = {
				content_type : "image/jpeg",
				name         : $img.attr("title") || "NO_NAME",
				src          : $img.attr("src")
			};
			if ( generate_id ) {
				var repo_id = inConcert.Utils.generateGuidWithSeparator();
				$img.attr("repo_id", repo_id);
				image_info.repo_id = repo_id;
			}
			embedded_images.push(image_info);
		});
		return embedded_images;
	};

	MailManagement.prototype.start_upload_images = function(callback)
	{
		var self = this;

		callback = callback || $.noop;
		// (**) tengo que agregar en un div contenedor solo para que el $.each pueda iterar entre los elementos
		var $body = $("<div>" + this.get() + "</div>");

		// el replace_img_src se realiza en callback(success) del upload de imagenes embebidas
		var replace_img_src = function(uploaded_images)
		{
			// con la lista de imagenes subidas "uploaded_images", reemplazo el atributo "src" donde se encuentra el base64
			//  por la uri del repo donde se encuentra el mismo.
			if ( uploaded_images && uploaded_images.length > 0 ) {
				$.each(uploaded_images, function(_, image)
				{
					var $image = $body.find("img[repo_id='" + image.repo_id + "']");
					if ( $image.length > 0 ) {
						$image.attr("src", image.get_uri);
					}
				});
				var html = $body.html(); // aca ya esta sin el "DIV" que se agrego arriba (**)
				self.set(html);
				window.setTimeout(function() {
					callback(true);
				}, 500);
			}
			else {
				callback(true);
			}
		};

		// obtengo las imagenes embebidas que existen en el body y luego las mando a subir
		var embedded_images = this.get_embedded_images_in_body($body);
		// subo las imagenes embebidas
		this.upload_embedded_images(embedded_images, function(success, uploaded_list)
		{
			if (!success ) {
			// TODO: hacer algo ??
				// // si el upload de las imagenes falla, cancelo todo y muestro el error correspondiente
				// self.hideStatusMessage();
				// $send.prop("disabled", false);
				// self.sending = false;
				// var err = uploaded_list;
				// self.displayStatusMessage(err, "Error");
				callback(false);
				return;
			}

			// aca elimino del array de imagenes subidas, las imagenes que han fallado en encode del base64
			uploaded_list = uploaded_list || [];
			for ( var i = uploaded_list.length - 1; i >= 0; i--) {
				var image_info = uploaded_list[i];
				if (image_info.upload_status === "error.atob_encoded") {
					uploaded_list.splice(i, 1);
				}
			}

			// si las imagenes fueron subidas correctamente al repo, tengo que recorrer las mismas y agregarlas al array de upload files que se encuentra en la ventana del cliente, marcandolo como "content-disposition=inline"
			////self.temporal_repository_images = [];
			$.each(uploaded_list, function(_, image_info)
			{
				var uploaded_image = {};
				uploaded_image.id = image_info.repo_id;
				uploaded_image.name = image_info.name;
				uploaded_image.type = image_info.content_type;
				uploaded_image["content-disposition"] = "inline";
				self.temporal_repository_images.push(uploaded_image);
			});
			// Aca por fin finalizo el upload, ahora reemplazo los src de base64 por las uri del repositorio.
			replace_img_src(uploaded_list);
		});
	};

	MailManagement.prototype.upload_embedded_images = function(embedded_images, callback)
	{
		var self = this;
		callback = callback || $.noop;

		if ( !$.isArray(embedded_images) || embedded_images.length ===0 ) {
			callback(true);
			return;
		}

		var count_atob_err = 0;
		var count_uploaded = 0;
		// esta funcion es invocada por cada archivo que se sube
		var upload_success = function(image_info)
		{
			// marco el file como success
			image_info.upload_status = "success";
			// asigno la uri de get para el archivo recien subido, la misma sera utilizada para remplazar el attr src del base por la url del repo en el body del mail
			image_info.get_uri = FORMAT(
				"{0}?id={1}&section={2}&part={3}"       , /* full uri (uri get + params) */
				URI_REPOSITORY_GET                      , /* repo uri get                */
				encodeURIComponent( image_info.repo_id ), /* file id                     */
				"-1"                                    , /* section                     */
				"0"                                       /* part                        */
			);
			// si subieron todos los archivos correctamente, invoco al callback para que termine enviando el mail
			count_uploaded++;
			if ( count_uploaded + count_atob_err === embedded_images.length ) {
				callback(true, embedded_images); // si llega aca se subieron los mails correctamente y envio el mail por fin!!
			}
		};

		// En el caso de que algun archivo de los que se estan subiendo falle, tiene que:
		//	1- Abortar los request en curso que aun no se haya completado.
		//	2- Mandar a eliminar del repo los archivos embebidos previamente subidos.
		var upload_error = function(image_info)
		{
			callback(false, _T("Error uploading embed images"));
			image_info.upload_status = "error";
			var uploaded_list = [];

			$.each(embedded_images, function(_, image)
			{
				if (image.upload_status === "pending") {
					image.upload_status = "aborted";
					image_info.xhr.abort();
				}
				else if (image.upload_status === "success") {
					image.upload_status = "removed";
					uploaded_list.push( {
						id      : image.repo_id,
						section : "-1"
					});
				}
			});

			self.delete_repository_files(uploaded_list);
		};

		// aca realizo el upload del file al repo
		var upload = function(upload_uri, blob_or_file, image_info)
		{
			image_info.xhr = new XMLHttpRequest();
			image_info.xhr.open("POST", upload_uri, true);

			image_info.xhr.onload = function(e)
			{
				if (e.target.status === 200) {
					upload_success(image_info); // success
				}
				else {
					upload_error(image_info); // error
				}
			};
			image_info.xhr.setRequestHeader("Content-Type", blob_or_file.type || "application/octet-stream");
			image_info.xhr.send(blob_or_file);
		};

		var relation = this.get_repository_relation();
		if (relation) {
			relation = encodeURIComponent(JSON.stringify(relation));
		}

		// recorro la lista de imagenes embebidas y las mando subir al repositorio.
		$.each(embedded_images, function(_, image_info)
		{
			var file_name = image_info.name;
			var extension = inConcert.Utils.getFileExtension( file_name ) || "png";
			var encoded_id = encodeURIComponent( image_info.repo_id );

			var upload_uri = FORMAT("{0}/inconcert/apps/repository/new/?id={1}&section={2}&contentType={3}&format={4}&fileName={5}",
				URI_DOMAIN,
				encoded_id,
				"-1",      /* section */
				image_info.content_type,
				extension, /* format */
				file_name
			);
			/* json con informacion de la relacion del archivo con la campana, cuenta, mensaje */
			if (relation) {
				upload_uri = upload_uri + FORMAT("&relation={0}", relation);
			}
			image_info.upload_status = "pending"; // lo dejo en estado pending mientras se realiza el request
			var base64_data = image_info.src;
			// Aca esta la magia, paso base64 a binario para que posteriormente sea subido al repositorio.
			// A esta conversion la realiza "canavas to blob":
			//		https://github.com/blueimp/JavaScript-Canvas-to-Blob
			//		/inconcert/lib/canvas-to-blob/js/canvas-to-blob.js
			//
			try {
				var blob = window.dataURLtoBlob && window.dataURLtoBlob(base64_data);
				upload(upload_uri, blob, image_info);
			}
			catch(e) {
				// si falla el encoded tengo que marcar la imagen como error de encoded, para que luego sea eliminada del array antes de ser enviado el mail.
				count_atob_err++;
				image_info.upload_status = "error.atob_encoded";
				console.warn("MailManagement.prototype.upload_embedded_images. Unable to upload image. Incorrect base64: " + base64_data);
				if ( count_atob_err === embedded_images.length ) {
					// Como fallaron el encode de todos los mensajes embebidos, invoco al callback.
					// Si al menos existiera una imagen bien encodeada y subida al repo, la invocacion al callback se realiza en la funcion "upload_success"
					callback(true, embedded_images);
				}
			}
		});
	};

	MailManagement.prototype.delete_repository_files = function(remove_list)
	{
		if ( !$.isArray(remove_list) || remove_list.length === 0 ) {
			return;
		}
		inConcert.invoke({
			url: URI_DOMAIN + "/inconcert/apps/repository/multipleserase/",
			type: "POST",
			data: JSON.stringify( remove_list ),
			dataType: "json",
			contentType: "application/json",
			success: $.noop,
			error: function( /* response, error */ ) {}
		});
	};

	/*
		Funcion que obtiene el ID del la imagen en el Repo.
			1 - Obtiene el valor desde el atributo "repo_id" del elemento <IMG>.
			2 - Si no existe el atributo, parsea la URL que se encuentra en el atributo "src" para obtener el el ID de la imagen.
		El caso 2 se da cuando se realiza el copy/paste, en este caso el atributo "repo_id", NO se pega en el HTML destino.
	*/
	MailManagement.prototype.__get_img_repo_id = function($img)
	{
		if (!$img) {
			return;
		}

		let repo_id = $img.attr("repo_id");
		if (repo_id) {
			return repo_id;
		}

		let src = $img.attr("src");
		let params = src.replace(URI_REPOSITORY_GET, "");
		const urlParams = new URLSearchParams(params);
		repo_id = urlParams.get("id");
	
		return repo_id;
	};

	MailManagement.prototype.get_repository_images_in_body = function()
	{
		const self = this;

		// (**) tengo que agregar en un div contenedor solo para que el $.each pueda iterar entre los elementos
		let $body = $("<div>" + this.get() + "</div>");
		// obtengo los <IMG con el attr SRC de repository y los agrego al array
		let repository_images = [];
		let $images = $body.find("img[src*='repository']");
		let images_by_id = {};

		$.each($images, function(_, img)
		{
			let $img = $(img);
			let repo_id = self.__get_img_repo_id($img);
			if (!images_by_id[repo_id]) {
				images_by_id[repo_id] = true;
				let repository_image = {
					id  : repo_id,
					name: $img.attr("title") || "NO_NAME",
					type: "image/jpeg"
				};
				repository_image["content-disposition"] = "inline";
				repository_images.push(repository_image);
			}
		});
		return repository_images;
	};

	MailManagement.prototype.get_repository_images_list = function()
	{
		// obtengo las imagenes que se han subido al repositorio a partir del body attr src="../repository/.."
		return this.get_repository_images_in_body();
	};

	MailManagement.prototype.show_disclaimer = function()
	{
		var $disclaimer = $("#" + this.disclaimer_id);
		if ($disclaimer.length > 0) {
			$disclaimer.show();
		}
	};

	MailManagement.prototype.hide_disclaimer = function()
	{
		var $disclaimer = $("#" + this.disclaimer_id);
		if ($disclaimer.length > 0) {
			$disclaimer.hide();
		}
	};

	MailManagement.prototype.editable = function()
	{
		this.show_disclaimer();
		$("#" + this.id).wysiwyg("shell").readOnly(false);
		this.show_toolbar(true);
	};

	MailManagement.prototype.readonly = function()
	{
		this.hide_disclaimer();
		$("#" + this.id).wysiwyg("shell").readOnly(true);
		this.show_toolbar(false);
	};

	MailManagement.prototype.show_toolbar = function(must_show)
	{
		var $toolbar = $("#" + this.id).parent().find(".wysiwyg-toolbar");
		if( $toolbar.length > 0 ){
			$toolbar[must_show ? "show" : "hide"]();
		}
	};

	MailManagement.prototype.on_saved = function()
	{
		this.temporal_repository_images = [];
	};

	MailManagement.prototype.cleanup_temp_repository_files = function()
	{
		var images = [];

		$.each(this.temporal_repository_images, function(_, image) {
			images.push({
				id     : image.id,
				section: "-1"
			});
		});

		if (images.length > 0) {
			this.delete_repository_files(images);
		}
	};

	MailManagement.prototype.set_events = function()
	{
		var self = this;

		if (this.owner.is_readonly()) {
			return;
		}

		var $body = $("body");

		var event_selector = "[id='" + this.popup_update_id + "']";
		$body
			.off("click", event_selector)
			.on("click", event_selector, function() {
				self.update();
			});

		event_selector = "[id='" + this.popup_close_id + "']";
		$body
			.off("click", event_selector)
			.on("click", event_selector, function() {
				self.hide_popup_html_edit();
			});

		event_selector = "[id='" + this.popup_close_id + "_times']";
		$body
			.off("click", event_selector)
			.on("click", event_selector, function() {
				self.hide_popup_html_edit();
			});
	};

	/*
		Estas 2 funciones "editor_save_selection" y "editor_restore_selection", son utilizadas para guardar y restaurar la posicion
		  donde se encuentra el cursor en el editor, cuando este pierde el foco.

		save/restore selection
		http://stackoverflow.com/questions/13949059/persisting-the-changes-of-range-objects-after-selection-in-html/13950376#13950376
	*/
	MailManagement.prototype.editor_save_selection = function()
	{
		let selection;
		if (window.getSelection) {
			selection = window.getSelection();
			if (selection.rangeCount > 0) {
				return selection.getRangeAt(0);
			}
		}
		else if (document.selection) {
			selection = document.selection;
			return selection.createRange();
		}
		return null;
	};

	MailManagement.prototype.editor_restore_selection = function()
	{
		if (!this.m_editor_saved_selection) {
			return;
		}
		if (window.getSelection) {
			var sel = window.getSelection();
			sel.removeAllRanges();
			sel.addRange(this.m_editor_saved_selection);
		}
		else if (document.selection) {
			this.m_editor_saved_selection.select();
		}
	};
	//-----------------------------------------------------------
}());
