#!/bin/bash

#
# Note: This script is idempotent
#

# maintainer       "Ellio de Mello"
# description      "Elimina las acciones y permisos del ABM de Marketing & Sales en las campanas"
# repo             "agent"
# dependencies     "cassandra"
# idempotent       "Yes"


. /etc/inconcert/inconcert.conf
. /etc/inconcert/network.conf


RemoveActions() {
	lua << LUA_EOF

local json = require "json"
local cassandra = require "lua_cassandra"
cassandra.seed("$CASSANDRA_IP", $CASSANDRA_PORT)

local m_AppsActions = {
	{
		AppId = "apps_launcher",
		Actions={
			{ActionId = "emma11"},
			{ActionId = "emma4"},
			{ActionId = "emma5"},
			{ActionId = "emma2"},
			{ActionId = "10"},
			{ActionId = "emma9"},
			{ActionId = "emma8"},
			{ActionId = "emma6"},
			{ActionId = "emma7"},
			{ActionId = "emma10"},
			{ActionId = "tito1"},
			{ActionId = "emma1"},
			{ActionId = "emma10"},
			{ActionId = "apps_launcher_dos"},
			{ActionId = "apps_launcher_tres"},
			{ActionId = "apps_launcher_uno"},
			{ActionId = "tres"},
			{ActionId = "dos"},
		}
	}
}

local function RemoveActions()
	print("")
	print("DELETING ACTIONS ---------------------------")
	local keyspace = cassandra:CassandraConnection("Agent")

	for _, appActions in ipairs(m_AppsActions) do
		local appId = appActions.AppId
		local encoded_actions = keyspace:GetColumnValue(appId, "Application", "Actions")
		local actions = json.decode(encoded_actions)

		for _, actionToDelete in ipairs(appActions.Actions) do
			print("- DELETING ACTION " .. tostring(actionToDelete.ActionId) .. ", APP " .. tostring(appId))
			actions[actionToDelete.ActionId] = nil
		end

		encoded_actions = json.encode(actions)
		keyspace:InsertColumn(appId, "Application", "Actions", encoded_actions)
	end

	-- local encoded_actions = keyspace:GetColumnValue("administration", "Application", "Actions")
	-- local actions = json.decode(encoded_actions)

	-- -- MARKETING SALES
	-- -- create Marketing Sales
		-- actions.create_campaign_marketing_sales = nil

		-- -------------------------------------------------------------
		-- -- campaigns Marketing Sales
		-- -- view campaigns Marketing Sales
		-- actions.view_campaigns_marketing_sales = nil

		-- -- save campaigns Marketing Sales
			-- actions.save_campaign_marketing_sales = nil

	-- ------------------------------------------------
	-- encoded_actions = json.encode(actions)
	-- keyspace:InsertColumn("administration", "Application", "Actions", encoded_actions)


	keyspace:Close()

	print("---------------------------")
	print("")
end


RemoveActions()

print("\nDone")

LUA_EOF
}


RemoveGroupPermissions() {
	lua << LUA_EOF

require "json"
cassandra = require "lua_cassandra"
cassandra.seed("$CASSANDRA_IP", $CASSANDRA_PORT)

local m_AppsActions = {
	{
		AppId = "apps_launcher",
		Actions={
			{ActionId = "emma11"},
			{ActionId = "emma4"},
			{ActionId = "emma5"},
			{ActionId = "emma2"},
			{ActionId = "10"},
			{ActionId = "emma9"},
			{ActionId = "emma8"},
			{ActionId = "emma6"},
			{ActionId = "emma7"},
			{ActionId = "emma10"},
			{ActionId = "tito1"},
			{ActionId = "emma1"},
			{ActionId = "emma10"},
			{ActionId = "apps_launcher_dos"},
			{ActionId = "apps_launcher_tres"},
			{ActionId = "apps_launcher_uno"},
			{ActionId = "tres"},
			{ActionId = "dos"},
		}
	}
}

function RemoveGroupPermissions()

	print("")
	print ("Start Removing Groups permissions...")


	local keyspace = cassandra:CassandraConnection("Agent")

	local roles = keyspace:GetRangeSlices("GroupPermission",  --column family
		"",  --supercolumn
		"", --startkey
		"", --endkey
			{ --predicate
			column_names = {},
			slice_range = {
						start = "",
						finish = "",
						reversed = false,
						count = 1000}
			},
		1000 --total keys
	)


	-- local function InsertGroupPermission( groupPermission, appId, actionId, full_roleId )
		-- groupPermission[ full_roleId ] = groupPermission[ full_roleId ] or {}
		-- groupPermission[ full_roleId ][ appId ] = groupPermission[ full_roleId ][ appId ] or {}
		-- groupPermission[ full_roleId ][ appId ][ actionId ] = true
	-- end

	 if roles then
		for role, permission in roles:Iterator() do
			print("role => " .. role)
			--if role == "administrator@system" then
				for k, v in ipairs(permission) do
					local app_id = v.name
					print("  appId => " .. app_id)
					for _, appActions in ipairs(m_AppsActions) do
						if app_id == appActions.AppId then -- app administration
							local permission = json.decode(v.value)
							print("    permission => " .. v.value)
							for _, actionToDelete in ipairs(appActions.Actions) do
								print("- DELETING ROLE " .. tostring(role) .. " PERMISSION, ACTION " .. tostring(actionToDelete.ActionId) .. ", APP " .. tostring(app_id))
								print("   +(antes) action_id value => " .. tostring(permission[actionToDelete.ActionId]))
								permission[actionToDelete.ActionId] = nil
								print("   -(despues) action_id value => " .. tostring(permission[actionToDelete.ActionId]))
							end

							keyspace:InsertColumn(role, 'GroupPermission', app_id, json.encode(permission))
						end
					end
				end
			--end
		end
	end

	keyspace:Close()

	print ("End Removed Groups permissions...")

end

RemoveGroupPermissions()

print("\nDONE Update Group permissions!")


LUA_EOF

}

RemoveCampaignOverridePermissions() {
	lua << LUA_EOF

require "utils" 
local json = require "json"
local cassandra = require "lua_cassandra"
cassandra.seed("$CASSANDRA_IP", $CASSANDRA_PORT)

local m_AppsActions = {
	{
		AppId = "apps_launcher",
		Actions={
			{ActionId = "emma11"},
			{ActionId = "emma4"},
			{ActionId = "emma5"},
			{ActionId = "emma2"},
			{ActionId = "10"},
			{ActionId = "emma9"},
			{ActionId = "emma8"},
			{ActionId = "emma6"},
			{ActionId = "emma7"},
			{ActionId = "emma10"},
			{ActionId = "tito1"},
			{ActionId = "emma1"},
			{ActionId = "emma10"},
			{ActionId = "apps_launcher_dos"},
			{ActionId = "apps_launcher_tres"},
			{ActionId = "apps_launcher_uno"},
			{ActionId = "tres"},
			{ActionId = "dos"},
		}
	}
}

local function GetCampaignPermissions (campaignId, keyspace)
	local encodedPermissions = utils.GetColumnValue({
		keyspace     = keyspace or "Agent",
		columnFamily = "Campaigns",
		key          = campaignId,
		column       = "permissions"
	})
	if encodedPermissions then
		return json.decode(encodedPermissions), encodedPermissions
	end
end

local function FindAllCampaigns (keyspace)
	local results = {}

	--TODO this should be paginating cause ain't working with more than 1000 campaigns configured
	local campaignList = keyspace:GetRangeSlices("Campaigns",  --column family
		"",  --supercolumn
		"", --startkey
		"", --endkey
		{ --predicate
			column_names = {},
			slice_range = {
				start = "",
				finish = "",
				reversed = false,
				count = 1000
			}
		},
		1000 --total keys
	)
	if campaignList then
		for campaignIdWithVCC, campaignRow in campaignList:Iterator() do
			table.insert(results, campaignIdWithVCC)
		end
	end
	return results
end

local function UpdateCampaignPermissions (campId, campPermissions, keyspace)
	utils.SaveRows {
		keyspace = keyspace,
		columnFamily = "Campaigns",
		rows = {
			[campId] = {
				permissions = campPermissions
			}
		}
	}
end

--[[
create_campaign_marketing_sales = nil
view_campaigns_marketing_sales = nil
save_campaign_marketing_sales = nil
]]
local function RemoveCampaignOverridePermissions()
	print("")
	print ("Start Removing CAMPAIGN OVERRIDE permissions...")

	local keyspace = cassandra:CassandraConnection("Agent")
	local campiagns = FindAllCampaigns(keyspace)
	--print(" - campiagns ", json.encode(campiagns))
	for i, campId in ipairs(campiagns) do
		print(tostring(i) .. " - " .. campId)
		local campPermissions, encodedPermissions = GetCampaignPermissions(campId, keyspace)
		if campPermissions then
			----print("encodedPermissions: " .. encodedPermissions)

			for _, appActions in ipairs(m_AppsActions) do
				local appId = appActions.AppId

				-- 1 - Elimino del override de campaign->members
				for memberId, memberInfo in pairs(campPermissions.members or {}) do
					--print("  - memberId " .. tostring(memberId))
					if memberInfo.actions and memberInfo.actions[appId] then
						for _, actionToDelete in ipairs(appActions.Actions) do
							print("- DELETING MEMBER ".. tostring(memberId) ..  " PERMISSION, ACTION " .. tostring(actionToDelete.ActionId) .. ", APP " .. tostring(appId))
							print("   +(antes) action_id value => " .. tostring(memberInfo.actions[appId][actionToDelete.ActionId]))
							memberInfo.actions[appId][actionToDelete.ActionId] = nil
							print("   -(despues) action_id value => " .. tostring(memberInfo.actions[appId][actionToDelete.ActionId]))
						end
					end
				end
				-- 2 - Elimino del override de campaign->roles
				for groupId, groupInfo in pairs(campPermissions.groups or {}) do
					--print("  - groupId " .. tostring(groupId))
					if groupInfo.actions and groupInfo.actions[appId] then
						for _, actionToDelete in ipairs(appActions.Actions) do
							print("- DELETING ROLE ".. tostring(groupId) ..  " PERMISSION, ACTION " .. tostring(actionToDelete.ActionId) .. ", APP " .. tostring(appId))
							print("   +(antes) action_id value => " .. tostring(groupInfo.actions[appId][actionToDelete.ActionId]))
							groupInfo.actions[appId][actionToDelete.ActionId] = nil
							print("   -(despues) action_id value => " .. tostring(groupInfo.actions[appId][actionToDelete.ActionId]))
						end
					end
				end
			end

			-- Ahora manda a Actualizar a Cassandra
			UpdateCampaignPermissions(campId, campPermissions, keyspace)
		end
		print("----------")
	end
	keyspace:Close()
	print ("End Removed CAMPAIGN OVERRIDE permissions...")
	print("")
end


RemoveCampaignOverridePermissions()

LUA_EOF
}


RemoveUsersPermissions() {
	lua << LUA_EOF

require "json"
cassandra = require "lua_cassandra"
cassandra.seed("$CASSANDRA_IP", $CASSANDRA_PORT)

local m_AppsActions = {
	{
		AppId = "apps_launcher",
		Actions={
			{ActionId = "emma11"},
			{ActionId = "emma4"},
			{ActionId = "emma5"},
			{ActionId = "emma2"},
			{ActionId = "10"},
			{ActionId = "emma9"},
			{ActionId = "emma8"},
			{ActionId = "emma6"},
			{ActionId = "emma7"},
			{ActionId = "emma10"},
			{ActionId = "tito1"},
			{ActionId = "emma1"},
			{ActionId = "emma10"},
			{ActionId = "apps_launcher_dos"},
			{ActionId = "apps_launcher_tres"},
			{ActionId = "apps_launcher_uno"},
			{ActionId = "tres"},
			{ActionId = "dos"},
		}
	}
}

function RemoveUsersPermissions()

	print("")
	print ("Start Removing Users permissions...")


	local keyspace = cassandra:CassandraConnection("Agent")

	local users = keyspace:GetRangeSlices("UserPermission",  --column family
		"",  --supercolumn
		"", --startkey
		"", --endkey
			{ --predicate
			column_names = {},
			slice_range = {
						start = "",
						finish = "",
						reversed = false,
						count = 10000}
			},
		1000 --total keys
	)



	 if users then
		for user, userInfo in users:Iterator() do
			print("user => " .. user)

			for k, v in pairs(userInfo) do
				local field = v.name
				if field == "actions" then
					local encodedPermissions =  v.value
					print(encodedPermissions)
					local permissions = json.decode(encodedPermissions)

					for app_id, permission in pairs(permissions) do
						-- print("  appId => " .. app_id)
						--print("  permission => " .. json.encode(permission))

						for _, appActions in ipairs(m_AppsActions) do
							if app_id == appActions.AppId then
								print("  appId => " .. app_id)
								print("    permission => " .. json.encode(permission))

								for _, actionToDelete in ipairs(appActions.Actions) do
									print("- DELETING USER " .. tostring(user) .. " PERMISSION, ACTION " .. tostring(actionToDelete.ActionId) .. ", APP " .. tostring(app_id))
									print("   +(antes) action_id value => " .. tostring(permission[actionToDelete.ActionId]))
									permission[actionToDelete.ActionId] = nil
									print("   -(despues) action_id value => " .. tostring(permission[actionToDelete.ActionId]))
								end

								keyspace:InsertColumn(user, 'UserPermission', "actions", json.encode(permission))
							end
						end
					end
				end
			end
		end
	end

	keyspace:Close()

	print ("End Removed Users permissions...")

end

RemoveUsersPermissions()

print("\nDONE Update Group permissions!")


LUA_EOF

}

echo "Starting migration..."

RemoveActions

RemoveGroupPermissions

RemoveCampaignOverridePermissions

RemoveUsersPermissions

echo "bye bye";