/* jshint smarttabs:true, quotmark:false, esversion: 9 */
/* global $, inConcert */


if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	"use strict";
	const FORMAT = inConcert.Utils.format;
	const REQUEST_TIMEOUT_MS = 300000; // 5 minutes
	const _T = inConcert.i18n.getString;

	function RasaAPI(params)
	{
		if(typeof(params) !== "object") {
			throw("RasaAPI.Constructor - expected an object with configuration");
		}
		if(typeof(params.uri ) !== "string") {
			throw("RasaAPI.Constructor - base uri is required");
		}

		this.uri = params.uri + "/rasa";
		this.cached_cultures = null;
	}

	if (typeof inConcert.RasaAPI !== "function") {
		inConcert.RasaAPI = RasaAPI;
	}

/********************************************************************************************************
	FUNCIONES GENERICAS
********************************************************************************************************/

	/*
	
	*/
	RasaAPI.prototype.__Get = function(uri, callback = $.noop)
	{
		if (typeof(uri) !== "string") {
			throw("RasaAPI.prototype.__Get: uri variable must be string type.");
		}
		if (typeof(callback) !== "function") {
			throw("RasaAPI.prototype.__Get: callback variable must be function type.");
		}

		const request_handler = inConcert.invoke({
			url    : uri,
			type   : "GET",
			timeout: REQUEST_TIMEOUT_MS,
			success: function (list, result, server_response)
			{
				if (typeof(server_response) === "object") {
					server_response = JSON.stringify(server_response);
				}
				//console.log(uri + "\nlist: " + list + "\nresult: " + result + "\nserver_response: " + server_response);
				if (result.toLowerCase() === "success") {
					callback(true, list);
				}
				else {
					// Ejemplo: code 204 (nocontent) para el caso de getTrain #7167
					callback(false, server_response);
				}
			},
			error  : function (code, error) {callback(false, error);}
		});
		return request_handler;
	};

	/*
	
	*/
	RasaAPI.prototype.__Delete = function(uri, callback, data)
	{
		if (typeof(uri) !== "string") {
			throw("RasaAPI.prototype.__Delete: uri variable must be string type.");
		}
		if (typeof(callback) !== "function") {
			throw("RasaAPI.prototype.__Delete: callback variable must be function type.");
		}

		const request_handler = inConcert.invoke({
			url         : uri,
			type        : "DELETE",
			data        : JSON.stringify(data || {}),
			contentType : "application/json",
			timeout     : REQUEST_TIMEOUT_MS,
			success     : function (response) { callback(true, response); },
			error       : function (code, error) {callback(false, error);}
		});
		return request_handler;
	};

	/*
	
	*/
	RasaAPI.prototype.__Post = function(uri, callback, data)
	{
		if (typeof(uri) !== "string") {
			throw("RasaAPI.prototype.__Post: uri variable must be string type.");
		}
		if (typeof(callback) !== "function") {
			throw("RasaAPI.prototype.__Post: callback variable must be function type.");
		}
		if (typeof(data) !== "object") {
			throw("RasaAPI.prototype.__Post: data variable must be object type.");
		}

		const request_handler = inConcert.invoke({
			url         : uri,
			type        : "POST",
			data        : JSON.stringify(data),
			dataType    : "json",
			contentType : "application/json",
			timeout     : REQUEST_TIMEOUT_MS,
			success     : function (response)    { callback(true, response); },
			error       : function (code, error) { callback(false, error);   }
		});
		return request_handler;
	};

	/*
	
	*/
	RasaAPI.prototype.__Put = function(uri, callback, data)
	{
		if (typeof(uri) !== "string") {
			throw("RasaAPI.prototype.__Put: uri variable must be string type.");
		}
		if (typeof(callback) !== "function") {
			throw("RasaAPI.prototype.__Put: callback variable must be function type.");
		}
		if (typeof(data) !== "object") {
			throw("RasaAPI.prototype.__Put: data variable must be object type.");
		}

		const request_handler = inConcert.invoke({
			url         : uri,
			type        : "PUT",
			data        : JSON.stringify(data),
			dataType    : "json",
			contentType : "application/json",
			timeout     : REQUEST_TIMEOUT_MS,
			success     : function (response)    { callback(true, response); },
			error       : function (code, error) { callback(false, error  ); }
		});
		return request_handler;
	};

	/*
	
	*/
	RasaAPI.prototype.getRasaErrorMsg = function(err)
	{
		if (typeof err !== "object") {
			return "";
		}
		if (typeof err.rasaError !== "object") {
			return "";
		}
		const result = err.rasaError.result;
		if (typeof result !== "string") {
			return "";
		}

		const errors = {
		// APPLICATION
			"UNABLE TO CREATE APP - APP ALREADY EXIST"                                 : _T("Application already exist"),
			"UNABLE TO RENAME APP - APP DOES NOT EXIST"                                : _T("Application does not exist"),
			"UNABLE TO DELETE APP - APP DOES NOT EXIST"                                : _T("Unable to delete {0} application. Application does not exist."),
			"UNABLE TO DELETE APP - TRAIN IS RUNNING FOR APP"                          : _T("Unable to delete {0} application. There is running training for the application."),
			"UNABLE TO CREATE DIRECTORY. DIRECTORY ALREADY EXIST"                      : _T("You should consult your administrator"),
			"UNABLE TO REMOVE DIRECTORY. DIRECTORY DOES NOT EXIST"                     : _T("You should consult your administrator"),
			"UNABLE TO TRAIN RASA APP - RASA COMMAND LINE ERROR"                       : _T("You should consult your administrator"),

		// INTENT
			"UNABLE TO CREATE INTENT IN APP - APP DOES NOT EXIST"                      : _T("Application does not exist"),
			"UNABLE TO CREATE INTENT - INTENT ALREADY EXIST"                           : _T("Intent already exist"),
			"UNABLE TO RENAME INTENT - APP DOES NOT EXIST"                             : _T("Application does not exist"),
			"UNABLE TO RENAME INTENT - INTENT DOES NOT EXIST"                          : _T("Intent does not exist"),
			"UNABLE TO RENAME INTENT - INTENT NONE IS THE SYSTEM"                      : _T("Intent None is the system"),
			"UNABLE TO RENAME INTENT - INTENT ALREADY EXIST"                           : _T("Intent already exist"),
			"UNABLE TO DELETE INTENT - APP DOES NOT EXIST"                             : _T("Application does not exist"),
			"UNABLE TO DELETE INTENT - INTENT DOES NOT EXIST"                          : _T("Intent does not exist"),
			"UNABLE TO DELETE INTENT - INTENT NONE IS THE SYSTEM"                      : _T("Intent None is the system"),

		// UTTERANCES
			"UNABLE TO ADD UTTERANCE TO INTENT - APP DOES NOT EXIST"                   : _T("Application does not exist"),
			"UNABLE TO ADD UTTERANCE TO INTENT - INTENT DOES NOT EXIST"                : _T("Intent does not exist"),
			"UNABLE TO ADD UTTERANCE TO INTENT - UTTERANCE ALREADY EXIST"              : _T("Utterance already exist"),
			"UNABLE TO UPDATE LABELS IN UTTERANCE TO INTENT - APP DOES NOT EXIST"      : _T("Application does not exist"),
			"UNABLE TO UPDATE LABELS IN UTTERANCE TO INTENT - INTENT DOES NOT EXIST"   : _T("Intent does not exist"),
			"UNABLE TO UPDATE LABELS IN UTTERANCE TO INTENT - UTTERANCE DOES NOT EXIST": _T("Utterance does not exist"),
			"UNABLE TO DELETE UTTERANCE - APP DOES NOT EXIST"                          : _T("Application does not exist"),
			"UNABLE TO DELETE UTTERANCE - INTENT DOES NOT EXIST"                       : _T("Intent does not exist"),
			"UNABLE TO DELETE UTTERANCE - UTTERANCE DOES NOT EXIST"                    : _T("Utterance does not exist"),

		// ENTITY
			"UNABLE TO CREATE ENTITY - ENTITY ALREADY EXIST"                           : _T("Entity already exist"),
			"UNABLE TO CREATE ENTITY IN APP - APP DOES NOT EXIST"                      : _T("Application does not exist"),
			"UNABLE TO RENAME ENTITY - APP DOES NOT EXIST"                             : _T("Application does not exist"),
			"UNABLE TO RENAME ENTITY - ENTITY ALREADY EXIST"                           : _T("Entity already exist"),
			"UNABLE TO RENAME ENTITY - ENTITY DOES NOT EXIST"                          : _T("Entity does not exist"),
			"UNABLE TO DELETE ENTITY - APP DOES NOT EXIST"                             : _T("Application does not exist"),
			"UNABLE TO DELETE ENTITY - ENTITY DOES NOT EXIST"                          : _T("Entity does not exist"),
		};
		let msg_err = errors[result.toUpperCase()] || result;
		return " " + msg_err;
	};

	RasaAPI.prototype.makeTrainAppResult = function(result)
	{
		if (typeof result !== "object" || !result) {
			return;
		}
		let statuses = {};

		statuses["COMPLETED TRAIN"] = {
			name  : "Success",
			desc  : _T("Trained."),
			action: "TRAINED",
			msg   : _T("App {0} was successfully trained."),
			class_name: "gridNLUChipCompleted"
		};
		statuses["PARTIALLY TRAIN - SOME NODES REMAIN TO UPDATE"] = {
			name  : "SuccessSomeNodesRemainToUpdate",
			desc  : _T("Partially trained."),
			action: "PARTIALLY TRAINED",
			msg   : _T("Application {0} was partially trained."),
			class_name: "gridNLUChipPartiallyActive"
		};
		statuses["FAIL TRAIN"] = {
			name  : "Fail",
			desc  : _T("Fails to Train."),
			action: "ERROR",
			msg   : _T("Error Training App ({0})."),
			class_name: "gridNLUChipError"
		};
		statuses["START RUNNING"] = {
			name  : "InProgress",
			desc  : _T("Starting Training."),
			action: "RUNNING",
			class_name: "gridNLUChipRunning"
		};
		statuses["MAKING NLU FILE"] = {
			name  : "InProgress",
			action: "RUNNING",
			desc  : _T("Training in Progress, Making NLU File."),
			msg: _T("In Progress Training App {0}. Making NLU File."),
			class_name: "gridNLUChipRunning"
		};
		statuses["RUNNING API RASA TRAIN"] = {
			name  : "InProgress",
			action: "RUNNING",
			desc  : _T("Training in Progress, Running Train."),
			msg: _T("Unable to train {0} application. There is running training for the application."),
			class_name: "gridNLUChipRunning"
		};
		statuses["NEVER TRAINED"] = {
			name  : "NeverTrained",
			action: "NEVERTRAINED",
			desc  : _T("Never Trained."),
			msg   : _T("Never Trained App {0}."),
			class_name: "gridNLUChipInactive"
		};

		let status_id = result.statusId;
		let current_status = statuses[status_id];
		current_status.raw_status = result;
		return current_status;
	};

	RasaAPI.prototype.getLastTrainInfo = function(row)
	{
		let last_train_step;
		let trainInfo = row.trainInfo || [];

		if ($.isArray(trainInfo) && trainInfo.length > 0) {
			let last_train_steps = trainInfo[trainInfo.length - 1];
			if ($.isArray(last_train_steps) || last_train_steps.length > 0) {
				last_train_step = last_train_steps[last_train_steps.length - 1];
			}
		}

		return last_train_step;
	};

/********************************************************************************************************
	APPS
********************************************************************************************************/
	/*
	
	*/
	RasaAPI.prototype.loadCultures = function(callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/cognitive_services/rasa/apps/load_cultures/
		 */

		if (this.cached_cultures) {
			callback(true, this.cached_cultures);
			return; 
		}

		const self = this;
		const uri = FORMAT("{0}/apps/load_cultures/", this.uri);

		const request_handler = this.__Get(uri, function(success, list) {
			if (success) {
				self.cached_cultures = list;
				callback(true, list);
				return;
			}
			// error
			callback(false, list);
		});
		return request_handler;
	};

	/*
	
	*/
	RasaAPI.prototype.loadApps = function(callback, trained_only)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/cognitive_services/rasa/apps/load_all/
		 */

		const uri = FORMAT("{0}/apps/load_all/", this.uri);

		var can_include_app = function(app)
		{
			if (!app) {
				return false;
			}
			if (!trained_only) {
				return true;
			}
			if (!$.isArray(app.trainInfo) || app.trainInfo.length === 0) {
				return false;
			}
			let last_train_steps = app.trainInfo[app.trainInfo.length - 1];
			if (!$.isArray(last_train_steps) || last_train_steps.length === 0) {
				return false;
			}
			let last_train_step = last_train_steps[last_train_steps.length - 1];
			if (last_train_step && last_train_step.statusId === "COMPLETED TRAIN") {
				return true;
			}
			return false;
		};

		const request_handler = this.__Post(uri, function(success, list) {
			if (success) {
				let app_list = [];
				$.each(list.rows || [], function(_, app) {
					if (can_include_app(app)) {
						app_list.push(app);
					}
				});
				callback(true, app_list);
				return;
			}
			// error
			let err = list;
			callback(false, err);
		}, {});
		return request_handler;
	};

	RasaAPI.prototype.addApp = function(data, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/cognitive_services/rasa/apps/add/
		 */

		const uri = FORMAT("{0}/apps/add/", this.uri);

		const request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	RasaAPI.prototype.renameApp = function(data, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/cognitive_services/rasa/apps/rename/
		 */

		const uri = FORMAT("{0}/apps/rename/", this.uri);

		const request_handler = this.__Put(uri, callback, data);
		return request_handler;
	};

	/*
	
	*/
	RasaAPI.prototype.deleteApps = function(data, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/cognitive_services/rasa/apps/delete/
		 */

		const uri = FORMAT("{0}/apps/delete/", this.uri);

		const request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	RasaAPI.prototype.trainApp = function(data, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/cognitive_services/rasa/apps/train/
		 */

		const uri = FORMAT("{0}/apps/train/", this.uri);

		const request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

/********************************************************************************************************
	INTENTS
********************************************************************************************************/
	/*
	
	*/
	RasaAPI.prototype.loadAllIntents = function(params, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/cognitive_services/rasa/apps/intents/load_all/d469528f-a521-4035-a9ea-36a346b83df8
		 */
		params = params || {};

		const uri = FORMAT("{0}/apps/intents/load_all/{1}",
			this.uri,
			encodeURIComponent(params.appId || "")
		);

		const request_handler = this.__Get(uri, callback);
		return request_handler;
	};

	/*
	
	*/
	RasaAPI.prototype.createAppIntent = function(params, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/cognitive_services/rasa/apps/intents/create/
		 */

		const uri = FORMAT("{0}/apps/intents/create/", this.uri);

		const data = {
			appId: params.appId,
			name : params.name
		};
		const request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	RasaAPI.prototype.renameAppIntent = function(params, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/cognitive_services/rasa/apps/intents/rename/d469528f-a521-4035-a9ea-36a346b83df8/7b1d9a5f-2804-42a1-9b58-03b07788477a
		 */

		const uri = FORMAT("{0}/apps/intents/rename/{1}/{2}",
			this.uri,
			encodeURIComponent(params.appId    || ""),
			encodeURIComponent(params.intentId || "")
		);

		const data = {
			name: params.name
		};
		const request_handler = this.__Put(uri, callback, data);
		return request_handler;
	};

	/*
	
	*/
	RasaAPI.prototype.deleteAppIntent = function(params, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/cognitive_services/rasa/apps/intents/delete/d469528f-a521-4035-a9ea-36a346b83df8/7b1d9a5f-2804-42a1-9b58-03b07788477a
		 */
		params = params || {};

		const uri = FORMAT("{0}/apps/intents/delete/{1}/{2}",
			this.uri,
			encodeURIComponent(params.appId    || ""),
			encodeURIComponent(params.intentId || "")
		);

		const request_handler = this.__Delete(uri, callback);
		return request_handler;
	};



/********************************************************************************************************
	UTTERANCES
********************************************************************************************************/

	/*
	
	*/
	RasaAPI.prototype.loadUtterancesByAppIntent = function(params, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/cognitive_services/rasa/apps/intents/utterances/load_all/d469528f-a521-4035-a9ea-36a346b83df8/INTENT_ID
		 */
		params = params || {};

		const uri = FORMAT("{0}/apps/intents/utterances/load_all/{1}/{2}",
			this.uri,
			encodeURIComponent(params.appId    || ""),
			encodeURIComponent(params.intentId || "")
		);

		const request_handler = this.__Get(uri, callback);
		return request_handler;
	};

	/*
	
	*/
	RasaAPI.prototype.addUtteranceAppIntent = function(params, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/cognitive_services/rasa/apps/intents/utterances/add/
		 */

		const uri = FORMAT("{0}/apps/intents/utterances/add/", this.uri);

		// requerido
		let data = {
			appId    : params.appId,
			intentId : params.intentId,
			utterance: params.utterance
		};

		const request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	/*
	
	*/
	RasaAPI.prototype.updateLabelsInUtterance = function(params, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/cognitive_services/rasa/apps/intents/utterances/update/
		 */

		const uri = FORMAT("{0}/apps/intents/utterances/update/", this.uri);

		// requerido
		let data = {
			appId    : params.appId,
			intentId : params.intentId,
			utterance: params.utterance
		};

		// opcional
		if (params.entityLabels) {
			data.entityLabels = [];
			$.each(params.entityLabels, function(_, info) {
				data.entityLabels.push({
					entityId      : info.entityId,
					startCharIndex: info.startCharIndex,
					endCharIndex  : info.endCharIndex,
					children      : info.children
				});
			});
		}

		const request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	/*
	
	*/
	RasaAPI.prototype.deleteAppUtterance = function(params, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/cognitive_services/rasa/apps/intents/utterances/delete/f23db312-1ff6-47d5-be25-f6a739daf5ee/-8010780/intentId
		 */
		params = params || {};

		const uri = FORMAT("{0}/apps/intents/utterances/delete/{1}/{2}/{3}",
			this.uri,
			encodeURIComponent(params.appId       || ""),
			encodeURIComponent(params.utteranceId || ""),
			encodeURIComponent(params.intentId    || "")
		);

		const request_handler = this.__Delete(uri, callback);
		return request_handler;
	};

/********************************************************************************************************
	ENTITIES SIMPLES
********************************************************************************************************/

	/*
	
	*/
	RasaAPI.prototype.loadAllEntities = function(params, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/cognitive_services/rasa/apps/entities/load_all/d469528f-a521-4035-a9ea-36a346b83df8
		 */
		params = params || {};

		const uri = FORMAT("{0}/apps/entities/load_all/{1}",
			this.uri,
			encodeURIComponent(params.appId || "")
		);

		const request_handler = this.__Get(uri, callback);
		return request_handler;
	};

	/*
	
	*/
	RasaAPI.prototype.createAppEntity = function(params, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/cognitive_services/rasa/apps/entities/simple/create/
		 */

		const uri = FORMAT("{0}/apps/entities/simple/create/", this.uri);

		const data = {
			appId: params.appId,
			name : params.name
		};

		const request_handler = this.__Post(uri, callback, data);
		return request_handler;
	};

	/*
	
	*/
	RasaAPI.prototype.renameAppEntity = function(params, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/cognitive_services/rasa/apps/entities/simple/rename/d469528f-a521-4035-a9ea-36a346b83df8/entity_id
		 */

		const uri = FORMAT("{0}/apps/entities/simple/rename/{1}/{2}",
			this.uri,
			encodeURIComponent(params.appId    || ""),
			encodeURIComponent(params.entityId || "")
		);

		const data = {
			name: params.name
		};

		const request_handler = this.__Put(uri, callback, data);
		return request_handler;
	};

	/*
	
	*/
	RasaAPI.prototype.deleteAppEntity = function(params, callback = $.noop)
	{
		/*
		 * sample URI:
		 *   http://i6.inconcert.cc/inconcert/apps/cognitive_services/rasa/apps/entities/simple/delete/f23db312-1ff6-47d5-be25-f6a739daf5ee/entity_id
		 */
		params = params || {};

		const uri = FORMAT("{0}/apps/entities/simple/delete/{1}/{2}",
			this.uri,
			encodeURIComponent(params.appId    || ""),
			encodeURIComponent(params.entityId || "")
		);

		const request_handler = this.__Delete(uri, callback);
		return request_handler;
	};

}());
