/*jslint evil:true forin:true*/
/*global $, StereoAudioRecorder

*/

(function () {
	//"use strict";

	function InconcertRecorder() {
		this.microphone = null;
		this.recorder = null;
		this.isIE = !!document.documentMode;
		this.isEdge = navigator.userAgent.indexOf("Edge") !== -1 && (!!navigator.msSaveOrOpenBlob || !!navigator.msSaveBlob);
		this.isSafari = /^((?!chrome|android).)*safari/i.test(navigator.userAgent);
		this.recording = false;
		this.hasRecord = false;
	}
	
	window.InconcertRecorder = InconcertRecorder;

	InconcertRecorder.prototype.validate = function() {
		if (document.location.protocol === "http:") {
			return "HTTPS";
		}
		if (this.isIE) {
			return "IE";
		}
	};

	InconcertRecorder.prototype.initialize = function(callback) {
		this.__initializeMicrophone(callback);
	};

	InconcertRecorder.prototype.__initializeMicrophone = function(callback) {
		callback = callback || $.noop;
		var self = this;
		if (self.microphone) {
			return callback(true);
		}
		if(typeof navigator.mediaDevices === "undefined" || !navigator.mediaDevices.getUserMedia) {
			var error = !!navigator.getUserMedia?
				"This browser does not supports WebRTC getUserMedia API." :
				"This browser seems supporting deprecated getUserMedia API.";
			return callback(false, error);
		}
		navigator.mediaDevices.getUserMedia({
			audio: self.isEdge ? true : {
				echoCancellation: false
			}
		}).then(function(microphone) {
			self.microphone = microphone;
			callback(true);
		}).catch(function(error) {
			callback(false, error);
		});
	};

	InconcertRecorder.prototype.__initializeRecorder = function(callback) {
		callback = callback || $.noop;
		var self = this;
		if (self.recorder) {
			return callback(true);
		}
		var options = {
			type: "audio",
			numberOfAudioChannels: self.isEdge ? 1 : 2,
			checkForInactiveTracks: true,
			bufferSize: 16384
		};
		if(self.isSafari || self.isEdge) {
			options.recorderType = StereoAudioRecorder;
		}
		if(navigator.platform && navigator.platform.toString().toLowerCase().indexOf("win") === -1) {
			options.sampleRate = 48000; // or 44100 or remove this line for default
		}
		if(self.isSafari) {
			options.sampleRate = 44100;
			options.bufferSize = 4096;
			options.numberOfAudioChannels = 2;
		}
		self.recorder = RecordRTC(self.microphone, options);
		callback(true);
	};

	InconcertRecorder.prototype.__destroyMicrophone = function(callback) {
		callback = callback || $.noop;
		if (this.microphone) {
			this.microphone.stop();
			this.microphone = null;
		}
		callback(true);
	};

	InconcertRecorder.prototype.__destroyRecorder = function(callback) {
		callback = callback || $.noop;
		if(this.recorder) {
			this.recorder.destroy();
			this.recorder = null;
		}
		this.recording = false;
		this.hasRecord = false;
		callback(true);
	};

	InconcertRecorder.prototype.destroy = function(callback) {
		callback = callback || $.noop;
		var self = this;
		//first destroy recorder
		self.__destroyRecorder(function(status_r, error_r) {
			//then destroy microphone
			self.__destroyMicrophone(function(status_m, error_m) {
				if (!status_r) {
					callback(status_r, error_r);
				}
				else {
					callback(status_m, error_m);
				}
			});
		});
	};

	InconcertRecorder.prototype.start = function(callback) {
		callback = callback || $.noop;
		var self = this;
		if (!self.microphone) {
			//first initialize microphone
			return self.initialize(function(status, error) {
				if (!status) {
					return callback(status, error);
				}
				self.__start(callback);
			});
		}
		self.__start(callback);
	};

	InconcertRecorder.prototype.__start = function(callback) {
		callback = callback || $.noop;
		var self = this;
		//destroy current recorder
		self.__destroyRecorder(function(status, error) {
			if (!status) {
				return callback(status, error);
			}
			//then intialize newer
			self.__initializeRecorder(function(status, error) {
				if (!status) {
					return callback(status, error);
				}
				self.recorder.startRecording();
				self.recording = true;
				callback(true);
			});
		});
	};

	InconcertRecorder.prototype.stop = function(callback) {
		callback = callback || $.noop;
		var self = this;
		if (!self.recording) {
			return callback(false, "Recording not started.");
		}
		self.recorder.stopRecording(function() {
			self.recording = false;
			self.hasRecord = true;
			callback(true);
		});
	};

	InconcertRecorder.prototype.getRecord = function(callback) {
		callback = callback || $.noop;
		var self = this;
		if (!self.hasRecord) {
			return callback(false, "Has not a record.");
		}
		var blob = self.recorder.getBlob();
		callback(blob, !blob? "Error getting record" : null);
	};

	InconcertRecorder.prototype.__getBlobExtension = function(blob) {
		switch(blob.type) {
			case "audio/webm":
				return ".webm";
			case "audio/ogg":
				return ".ogg";
			default:
				var type_a = blob.type.split(";")[0].split("/");
				if (type_a.size === 2 && type_a[0] === "audio") {
					return type_a[1];
				}
				return ".wav";
		}
	};

	InconcertRecorder.prototype.getRecordFile = function(fileName, callback) {
		callback = callback || $.noop;
		var self = this;
		self.getRecord(function(blob, error) {
			if (!blob) {
				return callback(false, error);
			}
			fileName += self.__getBlobExtension(blob);
			var file = new File([blob], fileName, {
				type: blob.type
			});
			callback(file);
		});
	};

	InconcertRecorder.prototype.downloadFile = function(fileName, callback) {
		callback = callback || $.noop;
		var self = this;
		self.getRecordFile(fileName, function(file, error) {
			if (!file) {
				return callback(false, error);
			}
			if (self.isSafari) {
				self.recorder.getDataURL(function(dataUrl) {
					if (!dataUrl) {
						return callback(false, "Error getting data url");
					}
					self.__saveToDisk(dataUrl, fileName, callback);
				});
			}
			else {
				invokeSaveAsDialog(file);
				callback(true);
			}
		});
	};

	InconcertRecorder.prototype.__saveToDisk = function(dataUrl, fileName, callback) {
		callback = callback || $.noop;
		// for non-IE
		if (!window.ActiveXObject) {
			var save = document.createElement("a");
			save.href = dataUrl;
			save.download = fileName || "unknown";
			save.style = "display:none;opacity:0;color:transparent;";
			(document.body || document.documentElement).appendChild(save);

			if (typeof save.click === "function") {
				save.click();
			} else {
				save.target = "_blank";
				var event = document.createEvent("Event");
				event.initEvent("click", true, true);
				save.dispatchEvent(event);
			}

			(window.URL || window.webkitURL).revokeObjectURL(save.href);
		}
		// for IE
		else if (!!window.ActiveXObject && document.execCommand) {
			var _window = window.open(dataUrl, "_blank");
			_window.document.close();
			_window.document.execCommand("SaveAs", true, fileName || dataUrl);
			_window.close();
		}
		callback(true);
	};

}());
