/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:false, browser:true, indent:false, maxerr:50, quotmark:double, -W014, esversion: 11 */

/* global inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";

	//---------------------------------------------------------------------
	// upload_file
	//---------------------------------------------------------------------

	const _T = inConcert.i18n.getString;
	const format = inConcert.Utils.format;

	// Check file size. Do not upload files we know will fail.
	// 64 MB is what the cookbook uses. Needs a knob.
	//
	//
	// Tamanio maximo de los archivos a subir
	const MAX_FILE_UPLOAD_SIZE_MB = 5;
	const MAX_FILE_UPLOAD_SIZE_BYTES = MAX_FILE_UPLOAD_SIZE_MB * 1024 * 1024;

	function upload_file (params)
	{
		this.container_id = params.container_id;
		this.upload_file_btn_id = this.container_id + "_file_btn";
		this.upload_file_accept_id = this.container_id + "_file_accept";
		this.upload_file_percentaje_id = this.container_id + "_file_percentaje";
		this.upload_file_validation_id = this.container_id + "_file_validation";

		this.must_validate_max_size = typeof(params.must_validate_max_size) === "boolean" ? params.must_validate_max_size : true;
		this.must_deactivate_after_uploading =  typeof(params.must_deactivate_after_uploading) === "boolean" ? params.must_deactivate_after_uploading : true;
		this.availables_extensions = params.availables_extensions;
		this.show_status_message_fn = params.show_status_message_fn;
		this.on_uploading_fn = params.on_uploading_fn || $.noop;
		this.on_uploaded_fn = params.on_uploaded_fn || function(_1, _2, callback) {callback();};
		this.can_upload_file_fn = params.can_upload_file_fn || $.noop;

		this.m_repository_url = inConcert.app.getBaseDomain() + (params.repository_url || "/inconcert/apps/allegro/repository");

		// Este objeto contiene la configuracion del archivos subidos.
		// el mismo es cargado de forma asincronica cuando se edita una configuracion.
		this.m_uploadedFiles = null;
		// Este objeto contiene la configuracion de los archivos subidos, antes de presionar el boton "save".
		// Si se presiona el boton cancel, se debe recorrer este objeto e ir eliminando del repo todos los archivos
		// que aqui se encuentren.
		this.recently_uploaded_files = null;

		if (params.auto_initialize) {
			this.initialize();
		}
		else {
			this.__initialize_data();
		}
		return true;
	}

	if (typeof inConcert.RepositoryUpload !== "function") {
		inConcert.RepositoryUpload = upload_file;
	}

	upload_file.prototype.initialize = function()
	{
		this.__initialize_data();
		this.__build_markup();
		this.__set_events();
	};

	upload_file.prototype.dispose = function()
	{
		this.remove_temporal_uploaded_file();
		this.destroy_data();
	};

	upload_file.prototype.__on_upload = function(success, data, callback)
	{
		callback = callback || $.noop;
		var self = this;

		this.on_uploaded_fn(success, data, function()
		{
			if (success) {
				if (self.must_deactivate_after_uploading) {
					self.__disable_upload();
				}
			}
			callback();
		}); // invoco al callback de quien me ha instanciado
	};

	upload_file.prototype.__on_uploading = function()
	{
		this.on_uploading_fn(); // invoco al callback de quien me ha instanciado
		this.__disable_upload();
	};

	upload_file.prototype.__disable_upload = function()
	{
		// El div contenedor del input file upolaod
		var $upload_file_container = $("#" + this.container_id).find("[upload_file]");
		$upload_file_container.removeClass("butFormUploadFileDisabled").addClass("butFormUploadFileDisabled");

		// El boton de upload
		var $upload = $("#" + this.upload_file_accept_id);
		$upload.prop("disabled", true);

		// El textbox
		var $upload_file_name = $("#" + this.upload_file_btn_id);
		$upload_file_name.prop("disabled", true);
		// El label
		var $parent = $upload_file_name.parent();
		var $label = $parent.find("label[for='" + this.upload_file_btn_id + "']");
		$upload.prop("disabled", true);
		// $label.removeClass("formLabel-rq");
		// $label.addClass("formLabel-ds").prop("disabled", true);
		$label.prop("disabled", true);
	};

	upload_file.prototype.__initialize_data = function()
	{
		this.m_uploadedFiles = {};
		this.recently_uploaded_files = {};
	};

	upload_file.prototype.destroy_data = function()
	{
		this.m_uploadedFiles = {};
		this.recently_uploaded_files = {};
	};

	upload_file.prototype.remove_temporal_uploaded_file = function()
	{
		var txt_id = this.upload_file_btn_id;
		if (this.recently_uploaded_files[txt_id]) {
			this.delete_uploaded_file(this.recently_uploaded_files[txt_id].id);
		}
	};

	upload_file.prototype.__build_markup = function()
	{
		var $container = $("#" + this.container_id);

		$container.html(
			"<div class='row align-items-end'>" +
				"<div class='col-3'>" +
					"<div class='form-group d-flex flex-column align-items-start'>" +
						"<button upload_file='true' class='pageButtonGeneric buttonColorWhite butFormUploadFile'>" +
							"<i class='far fa-upload mr-2'></i>" +	
							"<input id='" + this.upload_file_accept_id + "' type='file' hidden accept='.csv, application/vnd.openxmlformats-officedocument.spreadsheetml.sheet, application/vnd.ms-excel' />" +
							_T("Import CSV") +
						"</button>" +
						"<i id='" + this.upload_file_btn_id + "_validation' class='formFieldValidation formFieldError d-none' hidden></i>" +
						"<label for='" + this.upload_file_btn_id + "' class='required importDoneLabel'>" + _T("File to Import") + _T(":") + "</label>" +
						"<input id='" + this.upload_file_btn_id + "' class='form-control formFieldUpload' type='text' readonly='readonly' value='' />" +
						"<i id='" + this.upload_file_validation_id + "' class='formFieldValidation formFieldError' hidden></i>" +
					"</div>" +
				"</div>" +
				"<div class='col-3'>" +
					"<div class='form-group d-flex flex-column align-items-start'>" +
						// "<label class='importDoneLabel'>&nbsp;</label>" +
						"<label id='" + this.upload_file_percentaje_id + "' class='uploadFilePercentageChip' hidden ></label>" +
					"</div>"+
				"</div>"+
			"</div>"
		);
	};

	upload_file.prototype.get_dom_upload_file_btn_id = function()
	{
		return this.upload_file_btn_id;
	};

	/*
		Funcion que setea eventos.
	*/
	upload_file.prototype.__set_events = function()
	{
		this.__init_files_uploader();
	};

	/*
		Esta funcion envia request al server para eliminar una lista de archivos subidos del repositorio.
	*/
	upload_file.prototype.delete_uploaded_files = function( remove_list )
	{
		var self = this;
		if (!remove_list || remove_list.length === 0) {
			return;
		}
		var uri = this.__make_multi_delete_uri();

		inConcert.invoke({
			url: uri,
			type: "POST",
			data: JSON.stringify(remove_list),
			dataType: "json",
			contentType: "application/json",
			success: function(/*response*/) {},
			error: function(/*response, error*/) {
				self.show_status_message( _T("Error deleting uploaded files"), "Error", 5000 );
			}
		});
	};

	upload_file.prototype.delete_uploaded_file = function( upload_id, callback )
	{
		callback = callback || $.noop;

		if ( !upload_id ) {
			callback( false );
			return;
		}
		var uri = this.__make_delete_uri( upload_id );

		inConcert.invoke({
			url: uri,
			dataType: "json",
			contentType: "application/json",
			success: function( response ) {
				callback( response );
			},
			error: function( /* error */ ) {
				callback( false );
			}
		});
	};

	upload_file.prototype.__make_upload_uri = function( file )
	{
		var file_id = inConcert.Utils.generateGuid();
		var extension = inConcert.Utils.getFileExtension( file.name ) || "";

		var uri = format("{0}/upload/?id={1}&section={2}&contentType={3}&format={4}&fileName={5}" ,
			this.m_repository_url,
			encodeURIComponent( file_id ),
			"1", /* section */
			file.type,
			extension, /* format */
			file.name
		);

		return {
			uri: uri,
			file_id: file_id
		};
	};

	upload_file.prototype.__make_delete_uri = function( upload_id )
	{
		return format(
			"{0}/remove/?id={1}&section={2}", /* uri         */
			this.m_repository_url                                     , /* domain      */
			encodeURIComponent( upload_id )                                   , /* upload guid */
			"1"                                                                /* section     */
		);
	};

	upload_file.prototype.__make_multi_delete_uri = function()
	{
		return this.m_repository_url + "/remove_all/";
	};

	upload_file.prototype.show_status_message = function(message, type, time_ms)
	{
		this.show_status_message_fn(message, type, time_ms);
	};

	upload_file.prototype.get_info_uploaded_files = function()
	{
		var txt_id = this.upload_file_btn_id;
		return this.recently_uploaded_files[txt_id];
	};

	upload_file.prototype.__can_upload_file = function(file, callback)
	{
		callback = callback || $.noop;

		var err;
		//aca valida que la extension del archivo a subir sea de un archivo ....
		var extension = inConcert.Utils.getFileExtension( file.name ) || "";
		if (!this.availables_extensions[extension.toLowerCase()]) {
			err = _T("Unsupported extension file. Only CSV files.");
			callback(false, err);
			return;
		}
		// tamano maximo soportado en el upload
		if (this.must_validate_max_size) {
			if ( file.size > MAX_FILE_UPLOAD_SIZE_BYTES ) {
				err = format(_T("Unable upload file. Max file size until {0} MB."), MAX_FILE_UPLOAD_SIZE_MB);
				callback(false, err);
				return;
			}
		}

		this.can_upload_file_fn(callback);
	};

	upload_file.prototype.__init_files_uploader = function()
	{
		var self = this;

		var waiting_time_ms = 5000;

		// The <input type="file"> button cannot be styled by css
		// So we have a another element (a div) which can be styled. In that element's click handler,
		// we forward the click event to the actual <input>
		// Note: to do that (the forwarding), we need the raw DOM element instead of the jQuery wrapped one
		// because $files_picker_ctrl.click() somehow causes a stack overflow (it seems to retrigger the
		// 'click' on the parent).
		// See: http://ericbidelman.tumblr.com/post/14636214755/making-file-inputs-a-pleasure-to-look-at
		//

		var $files_picker_ctrl = $("#" + this.container_id).find( "[upload_file] > input[type='file']" );
		$files_picker_ctrl.parent().click(function()
		{
			var $file_picker_ctrl = $( this ).find( "input[type='file']" );
			$file_picker_ctrl[0].click();
		});

		var upload = function (file)
		{
			self.__on_uploading();
			// create tracking node
			var txt_id = self.upload_file_btn_id;
			var $file_name = $( "#" + txt_id );
			$file_name.val(file.name);

			var $file_percentage = $("#" + self.upload_file_percentaje_id);
			$file_percentage.text( "0" + "%" ).show();

			var uri_info = self.__make_upload_uri(file);

			var xhr = new XMLHttpRequest();
			xhr.open("POST", uri_info.uri, true);
			xhr.onload = function(e)
			{
				$file_name.val( file.name );
				const errStatus = Number(e.target.status);
				if (errStatus !== 200) {
				// error
					$file_percentage.text( "" ).hide();
					let msgErr;
					if (errStatus === 403)
						msgErr = _T("Error uploading file {0}. You not has permission.");
					else
						msgErr = _T("Error uploading file {0}.");
					self.show_status_message( format( msgErr, file.name ), "Error", waiting_time_ms );
					$files_picker_ctrl.val("");
					self.__on_upload(false);
					return;
				}

				// success
				//
				// si estoy editando, tengo que marcar el archivo existente como delete, para que se elimine al dar "save"
				var currentUploadedFiles = self.m_uploadedFiles[ txt_id ];
				if (currentUploadedFiles && currentUploadedFiles.id && currentUploadedFiles.id !== uri_info.file_id) {
					self.m_uploadedFiles[ txt_id ].changed = true;
				}

				// si existe en recientes lo elimino!!!!
				if ( self.recently_uploaded_files[ txt_id ] ) {
					self.delete_uploaded_file(self.recently_uploaded_files[ txt_id ].id);
				}

				var extension = inConcert.Utils.getFileExtension( file.name ) || "";
				// doy de alta en recientes!!!!
				self.recently_uploaded_files[ txt_id ] = {
					id         : uri_info.file_id,
					section    : "1",
					fileName   : file.name,
					fileSize   : file.size,
					contentType: file.type,
					format     : extension
				};
				window.setTimeout( function() {
					$file_percentage.text("").hide();
				}, waiting_time_ms );

				var data = null;
				try {
					data = JSON.parse(e.target.responseText);
				}
				catch(_) {}

				$files_picker_ctrl.val("");
				self.__on_upload(true, data, function() {
					$file_percentage.text("100%");
				});
			};

			// show the uploaded percentage
			xhr.upload.onprogress = function( e ) {
				var percentage = "";
				if ( e && !isNaN( e.loaded ) && !isNaN( e.total ) ) {
					percentage =  Math.floor( e.loaded / e.total * 100 );
					if (percentage === 100) {
						// al 100 % lo visualiza en el callback de la funcion "xhr.upload.onload"
						return;
					}
					percentage = percentage + "%";
				}
				$file_percentage.text( percentage );
			};

			xhr.send( file );
		};

		$files_picker_ctrl.change(function( e )
		{
			inConcert.logger.guiAction({action_name: "upload_file", window_id: window.id});

			var file = e.target.files[0];
			if (!file) {
				self.__on_upload(false);
				return;
			}

			self.__can_upload_file(file, function(success, err)
			{
				if (success) {
					upload(file);
					return;
				}
				if (err) {
					self.show_status_message(err, "Error", waiting_time_ms );
				}
				self.__on_upload(false);
			});
		});
	};

}());