/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 9 */

/* global inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	//"use strict";
	const _T = inConcert.i18n.getString;
	const FORMAT = inConcert.Utils.format;

	/*
		SharedFilesList, template to make shared files list
	*/
	function SharedFilesList(properties = {})
	{
		this.owner = properties.owner;
//		this.frm = properties.frm;
		this.ownerId = properties.ownerId || false;
		this.option_id = properties.option_id;
		this.agentVisible = properties.agentVisible || false;
		this.fileTypes = properties.fileTypes || null;
		this.fileTypes_d = this.fileTypes? this.fileTypes.reduce(function(map, fileType) {
			map[fileType] = true;
			return map;
		}, {}) : null;
		this.fileTypesFilter = properties.fileTypesFilter !== false;
		this.maxSize = properties.maxSize || null;
		this.filter = properties.filter !== false;
		this.scrollPaginator = properties.scrollPaginator !== false;
		this.scrollPaginatorPerc = properties.scrollPaginatorPerc || 80;
		this.extensionsDisclaimer = properties.extensionsDisclaimer !== false;
		this.sizeLimitDisclaimer = properties.sizeLimitDisclaimer !== false;
		this.m_search_files_type_id = properties.search_files_type_id;
		this.m_abm_files_type_id = properties.abm_files_type_id;
		this.m_visibility = properties.visibility;

		this.autoSelectFirst = properties.autoSelectFirst || false;
		this.multiSelect = properties.multiSelect || false;
		this.unSelect = properties.unSelect || false;
		
		this.delayMs = properties.delayMs || 400;
		this.rows = properties.rows || 40;
		this.className = "sharedFilesList " + (properties.className || "");
		this.messages = properties.messages || {
			searching : _T("Searching files"),
			empty : _T("No files found."),
			error : _T("Error searching files.")
		};
		this.result = properties.result || null;
		this.can_create_new_file = properties.can_create_new_file || false;
		this.create_file_and_quit = properties.create_file_and_quit || false;
		this.$get_save_btn = properties.$get_save_btn || false;
		this.$get_cancel_btn = properties.$get_cancel_btn || false;
		this.$get_cancel_btn_times = properties.$get_cancel_btn_times || false;

		let callbacks = properties.callbacks || {};
		this.callbacks = {
			on_show_maintenance: function() {
				(callbacks.on_show_maintenance || $.noop)();
			},
			on_hide_maintenance: function() {
				(callbacks.on_hide_maintenance || $.noop)();
			}
		};

		this.build_instance_maintenance_mng();
	}

	//Define functions and objects in the namespace in case they don't already exist
	if (typeof inConcert.SharedFilesList !== "function") {
		inConcert.SharedFilesList = SharedFilesList;
	}

	SharedFilesList.prototype.__$get_save_btn = function() {
		if (this.$get_save_btn) {
			return this.$get_save_btn();
		}
	};

	SharedFilesList.prototype.__$get_cancel_btn = function() {
		if (this.$get_cancel_btn) {
			return this.$get_cancel_btn();
		}
	};

	SharedFilesList.prototype.__$get_cancel_btn_times = function() {
		if (this.$get_cancel_btn_times) {
			return this.$get_cancel_btn_times();
		}
	};

	SharedFilesList.prototype.make = function() {
		this.$body = this.makeBody();
		this.makeDisclaimers();
		this.bindEvents();
		return this.$body;
	};

	SharedFilesList.prototype.edit_file = function(file) {
		this.is_creating_new_shared_file = false;
		this.m_maintenanceMng.initialize();
		this.m_maintenanceMng.load_data();
		this.m_maintenanceMng.set_events();
		this.m_maintenanceMng.setMultipleFiles(false);
		this.m_maintenanceMng.set(true, file);
		this.on_show_maintenance();
	};

	SharedFilesList.prototype.start = function(callback = $.noop) {
		this.search(false, callback);
	};

	SharedFilesList.prototype.get_search_files_extensions = function()
	{
		if (!this.m_maintenanceMng) {
			return inConcert.FileUploaderTypes[this.m_search_files_type_id];
		}
		return this.m_maintenanceMng.get_search_files_extensions();
	};

	SharedFilesList.prototype.get_visibility = function()
	{
		return this.m_visibility;
	};

	SharedFilesList.prototype.makeBody = function() {
		const self = this;
		const makeHtmlNewFile = function() {
			if (!self.can_create_new_file) {
				return "";
			}
			let has_permission = inConcert.permissions.to_action("administration", "save_shared_files");
			let title = _T("New Shared File");
			let caption = _T("Upload File");
			let disable_attr = "";
			if (!has_permission) {
				title = _T("forbidden");
				disable_attr = "disabled='true'";
			}
			return "".concat(
				"<span class='action-buttons-separator'></span>",
				"<button shared_files_list_new_file class='pageButtonGeneric buttonColorWhite' ", disable_attr, " title='", title, "'>",
					caption,
				"</button>",
			);
		};

		return $("".concat(
			"<div class='card no-card'>",
				"<div class='card-block ", this.className, "'>",
					"<div id='shared_files_list_container'>",
						"<div class='sharedFilesListFilterContainer boxContentFilterArea'>",
							"<div class='row'>",
								"<div class='col-12 d-flex flex-row justify-content-between'>",
									"<div class='d-flex form-group mb-0'>",
										"<select shared_files_list_filter_type class='form-control mr-2'></select>",
										"<input shared_files_list_filter type='text' placeholder='", _T("Search"), "", _T("..."), "' class='form-control' />",
										"<i class='far fa-search'></i>",
										"<button shared_files_list_refresh class='buttonToggle buttonToggleOutlined search' title='", _T("Search"), "' style='display:none'>",
											"<i class='far fa-search'></i>",
										"</button>",
										"<button shared_files_list_reset class='buttonToggle buttonToggleOutlined search' title='", _T("Reset"), "'>",
											"<i class='far fa-eraser'></i>",
										"</button>",
										makeHtmlNewFile(),
									"</div>",
									"<div class='form-group mb-0 d-flex align-items-center'>",
										"<div extensions_disclaimer class='boxContentExtensionsDisclaimer'></div>",
										"<div size_limit_disclaimer class='boxContentSizeLimitDisclaimer'></div>",
									"</div>", 
								"</div>",
							"</div>",
						"</div>",
						"<div shared_files_list_messages class='inlineMessagesContainer messageInfo' hidden></div>",
						"<div shared_files_list_waiting class='boxContentListWaiting' hidden>",
							"<div class='listWaitingAnimation'></div>",
							"<div shared_files_list_waiting_message class='listWaitingMessage'></div>",
							"<div class='listWaitingOverlay'></div>",
						"</div>",
						"<div shared_files_list class='boxContentScrollArea contentSharedFilesList'></div>",
					"</div>",
					// El contenido de este div se genera en la clase "inConcert.shared_files_maintenance_mng" del archivo "shared_files_maintenance_mng.js"
					"<div id='shared_files_list_abm_container' hidden></div>",
				"</div>",
			"</div>"
		));
	};

	SharedFilesList.prototype.build_instance_maintenance_mng = function() {
		const self = this;
		if (!this.can_create_new_file) {
			return;
		}
		var displayStatusMessage = function (message, msg_type, is_closeable, timeout_ms) {
			self.owner.displayStatusMessage(message, msg_type, is_closeable, timeout_ms);
		};
		var hideStatusMessage = function () {
			self.owner.hideStatusMessage();
		};

		let owner = {
			is_new: function() {
				return self.is_creating_new_shared_file;
			},
			// readonly_control: function($control, readonly, is_required) {
			// 	self.owner.owner.frm.readonly_control($control, readonly, is_required);
			// },
			popup_hide: function() {},
			show_status_message: function(message, msg_type, timeout_ms) {
				displayStatusMessage(message, msg_type, true, timeout_ms);
			},
			hide_status_message: function() {
				hideStatusMessage();
			},
			show_waiting: function(msg) {
				inConcert.app.m_mainWindow.showBannerMessage({
					messageType: "Process",
					message: msg,
					hideClose: true
				});
			},
			hide_waiting: function() {
				inConcert.app.m_mainWindow.hideBannerMessage();
			},
			is_readonly: function() {},
			validation: {
				show_validation_error: function(_, message, timeout_ms) {
					displayStatusMessage(message, "Error", true, timeout_ms);
				}
			}
		};

		this.m_maintenanceMng = new inConcert.shared_files_maintenance_mng({
			instance_type: "INTERNAL",
			owner        : owner,
			container_id : "shared_files_list_abm_container",
			prefix_id    : "shared_files_list_abm_",
			search_files_type_id: this.m_search_files_type_id,
			abm_files_type_id   : this.m_abm_files_type_id,
			visibility   : this.m_visibility,
			callbacks    : {
				on_show: function() {
					self.on_show_maintenance();
				},
				on_close: function() {
					self.on_hide_maintenance();
					if (!self.is_creating_new_shared_file) {
						const $get_cancel_btn = self.__$get_cancel_btn();
						if ($get_cancel_btn) {
							$get_cancel_btn.trigger("click");
						}
						//TODO: #9335 - revisar merge mcwarren con cany
						//const $get_cancel_btn = self.__$get_cancel_btn();
						//const $get_cancel_btn_times = self.__$get_cancel_btn_times();
						//if ($get_cancel_btn) {
							//$get_cancel_btn.trigger("click");
						//} else if ($get_cancel_btn_times) {
							//$get_cancel_btn_times.trigger("click");
						//}
					}
				},
				on_save: function(file) {
					if (self.is_creating_new_shared_file) {
						let new_file = file;
						self.on_add_file_by_maintenance(new_file);
					}
					else {
						let update_file = file;
						self.on_update_file_by_maintenance(update_file);
					}
				}
			}
		});
	};

	SharedFilesList.prototype.on_show_maintenance = function() {
		let $bottom_container, $title_container;
		if (this.ownerId) {
			const $popup = $("#" + this.ownerId);
			$title_container = $popup.find("[title_container]");
			$bottom_container = $popup.find("[bottom_container]");
		}

		const $files_container = $("#shared_files_list_container");
		const $abm_container = $("#shared_files_list_abm_container");

		$files_container.hide();
		if ($bottom_container) {
			$bottom_container.hide();
		}
		if ($title_container) {
			let title = this.is_creating_new_shared_file ? _T("New File") : _T("Update File");
			$title_container.text(title);
		}
		$abm_container.show();

		this.callbacks.on_show_maintenance();
	};

	SharedFilesList.prototype.on_hide_maintenance = function() {
		let $bottom_container, $title_container;
		if (this.ownerId) {
			const $popup = $("#" + this.ownerId);
			$title_container = $popup.find("[title_container]");
			$bottom_container = $popup.find("[bottom_container]");
		}

		const $files_container = $("#shared_files_list_container");
		const $abm_container = $("#shared_files_list_abm_container");

		$abm_container.hide();
		if ($bottom_container) {
			$bottom_container.show();
		}
		if ($title_container) {
			$title_container.text(_T("Select a file"));
		}
		$files_container.show();

		this.callbacks.on_hide_maintenance();
	};

	SharedFilesList.prototype.on_add_file_by_maintenance = function(new_file) {
		const self = this;
		this.addFile(new_file, function(file, $file) {
			if (!file) {
				// TODO: hacer algo ?
				return;
			}
			// Tengo que dejar seleccionado el archivo que hemos agregado.
			$file.trigger("click");
			// Elimino el mensaje de "not result found", para el caso en el cual sea el primer archivo existente en los shared files.
			self.message();
			// Guardo en shared file y cierro el popup con la lista.
			if (self.create_file_and_quit === true && self.ownerId) {
				const $save_btn = self.__$get_save_btn();
				if ($save_btn) {
					$save_btn.trigger("click");
				}
			}
		});
	};

	SharedFilesList.prototype.on_update_file_by_maintenance = function(update_file) {
		const self = this;
		this.updateFile(update_file, function(file, $file) {
			if (!file) {
				// TODO: hacer algo ?
				return;
			}
			// Tengo que dejar seleccionado el archivo que hemos agregado.
			$file.trigger("click");
			// Elimino el mensaje de "not result found", para el caso en el cual sea el primer archivo existente en los shared files.
			self.message();
			// Guardo en shared file y cierro el popup con la lista.
			if (self.create_file_and_quit === true && self.ownerId) {
				const $save_btn = self.__$get_save_btn();
				if ($save_btn) {
					$save_btn.trigger("click");
				}
			}
		});
	};

	SharedFilesList.prototype.$refresh = function() {
		return this.$body.find("[shared_files_list_refresh]");
	};

	SharedFilesList.prototype.$reset = function() {
		return this.$body.find("[shared_files_list_reset]");
	};

	SharedFilesList.prototype.$new_file = function() {
		return this.$body.find("[shared_files_list_new_file]");
	};

	SharedFilesList.prototype.$filter = function() {
		return this.$body.find("[shared_files_list_filter]");
	};

	SharedFilesList.prototype.$filterType = function() {
		return this.$body.find("[shared_files_list_filter_type]");
	};

	SharedFilesList.prototype.$list = function() {
		return this.$body.find("[shared_files_list]");
	};
	
	SharedFilesList.prototype.$message = function() {
		return this.$body.find("[shared_files_list_messages]");
	};

	SharedFilesList.prototype.$waiting = function() {
		return this.$body.find("[shared_files_list_waiting]");
	};

	SharedFilesList.prototype.$waitingMessage = function() {
		return this.$body.find("[shared_files_list_waiting_message]");
	};

	SharedFilesList.prototype.$extensionsDisclaimer = function() {
		return this.$body.find("[extensions_disclaimer]");
	};

	SharedFilesList.prototype.$sizeLimitDisclaimer = function() {
		return this.$body.find("[size_limit_disclaimer]");
	};

	SharedFilesList.prototype.makeDisclaimers = function() {
		this.makeSizeLimitDisclaimer();
		this.makeExtensionsDisclaimer();
	};

	SharedFilesList.prototype.makeSizeLimitDisclaimer = function() {
		if (this.sizeLimitDisclaimer && this.maxSize) {
			this.$sizeLimitDisclaimer().show().text(FORMAT(_T("File size limit: {0}"), inConcert.Utils.fileSizeText(this.maxSize)));
		}
		else {
			this.$sizeLimitDisclaimer().hide();
		}
	};

	SharedFilesList.prototype.makeExtensionsDisclaimer = function() {
		let type = this.getTypeFilter();
		if (this.extensionsDisclaimer && type && type.extensions) {
			this.$extensionsDisclaimer().show()
				.text(FORMAT(_T("Valid extensions: {0}"), 
					inConcert.Utils.arrayToText(type.extensions, function(extension) {
						return "." + extension;
					}))
				);
		}
		else {
			this.$extensionsDisclaimer().hide();
		}
	};

	SharedFilesList.prototype.bindEvents = function() {
		this.bindRefresh();
		this.bindReset();
		this.bindNewFile();
		this.bindFilter();
		this.bindFilterType();
		if (this.scrollPaginator) {
			this.bindScroll();
		}
	};

	SharedFilesList.prototype.bindRefresh = function() {
		const self = this;
		this.$refresh()
			.off("click")
			.on("click", function() {
				self.search();
			});
	};

	SharedFilesList.prototype.bindReset = function() {
		const self = this;
		this.$reset()
			.off("click")
			.on("click", function() {
				self.reset();
			});
	};

	SharedFilesList.prototype.bindNewFile = function() {
		const self = this;
		this.$new_file()
			.off("click")
			.on("click", function() {
				self.new_file();
			});
	};

	SharedFilesList.prototype.bindFilter = function() {
		const self = this;
		this.$filter()
			.off("keyup")
			.on("keyup", function() {
				self.search(true);
			});
	};

	SharedFilesList.prototype.bindFilterType = function() {
		const self = this;
		let $filter = this.$filterType();
		if (this.fileTypes && this.fileTypesFilter) {
			$filter
				.html(this.fileTypes.reduce(
					function(types, type) {
						return types + FORMAT("<option value='{0}'>{1}</option>", type.id, type.name);
					}, self.fileTypes.length > 1 ? "<option value=''>" + _T("All types") + "</option>" : "")
				)
				.prop("disabled", this.fileTypes.length === 1)
				.off("change")
				.on("change", function() {
					self.makeExtensionsDisclaimer();
					self.search(true);
				});
		}
		else {
			$filter.remove();
		}
	};

	SharedFilesList.prototype.bindScroll = function() {
		const self = this;
		this.$list()
			.off("scroll")
			.on("scroll", function() {
				let scrolledPercentage = Math.round((this.scrollTop / (this.scrollHeight - this.clientHeight)) * 100);
				if (!self.result.allResultsFound && scrolledPercentage > self.scrollPaginatorPerc) {
					self.nextPage();
				}
			});
	};

	SharedFilesList.prototype.message = function(message) {
		this.$message()
			.toggle(!!message)
			.text(message || "");
	};

	SharedFilesList.prototype.waiting = function(waiting, message) {
		this.$waiting().toggle(!!waiting);
		this.$waitingMessage().text(message || "");
	};

	SharedFilesList.prototype.startProcess = function(message) {
		this.message();
		this.waiting(true, message);
		this.processing = true;
	};

	SharedFilesList.prototype.stopProcess = function() {
		this.waiting(false);
		this.processing = false;
	};

	SharedFilesList.prototype.isProcessing = function() {
		return !!this.processing;
	};

	SharedFilesList.prototype.getFilter = function() {
		if (!this.filter || this.fixedFilter) {
			return this.fixedFilter || "";
		}
		return this.$filter().val() || "";
	};

	SharedFilesList.prototype.setFilter = function(value) {
		if (!this.filter || this.fixedFilter) {
			return;
		}
		this.$filter().val(value || "");
	};

	SharedFilesList.prototype.getTypeFilter = function() {
		if (!this.fileTypes) {
			return;
		}
		if (!this.fileTypesFilter) {
			return this.fileTypes[0];
		}
		let value = this.$filterType().val();
		return value && this.fileTypes_d[value] || null;
	};

	SharedFilesList.prototype.setTypeFilter = function(value) {
		if (!this.fileTypes || this.fileTypes === 1 || !this.fileTypesFilter) {
			return;
		}
		this.$filterType().val(value || "");
	};

	SharedFilesList.prototype.disableInputs = function() {
		this.$refresh().prop("disabled", true);
		this.$reset().prop("disabled", true);
		this.$new_file().prop("disabled", true);
		this.$filter().prop("disabled", true);
		this.$filterType().prop("disabled", true);
	};

	SharedFilesList.prototype.enableInputs = function() {
		this.$refresh().prop("disabled", false);
		this.$new_file().prop("disabled", false);
		this.$reset().prop("disabled", !!this.getFilter());
		this.$filter().prop("disabled", !this.fixedFilter);
		this.$filterType().prop("disabled", !this.fileTypes || this.fileTypes.length === 1);
	};

	SharedFilesList.prototype.reset = function(callback = $.noop) {
		if (this.isProcessing()) {
			return callback(false, "bussy");
		}
		this.setFilter("");
		this.setTypeFilter("");
		this.search(false, callback);
	};

	SharedFilesList.prototype.new_file = function(callback = $.noop) {
		if (this.isProcessing()) {
			return callback(false, "bussy");
		}
		this.is_creating_new_shared_file = true;
		this.m_maintenanceMng.show({option_id: this.option_id});
	};

	SharedFilesList.prototype.cleanList = function() {
		this.$list().html("");
	};

	SharedFilesList.prototype.search = function(delay, callback = $.noop) {
		const self = this;
		
		function task() {
			if (self.isProcessing()) {
				return callback(false, "bussy");
			}
			self.cleanList();
			self.result = null;
			self.__search(callback);
		}
		
		if (this.delayTimeout) {
			window.clearTimeout(self.delayTimeout);
			self.delayTimeout = null;
		}
		if (delay) {
			this.delayTimeout = window.setTimeout(function() {
				self.delayTimeout = null;
				task();
			}, self.delayMs);
		}
		else {
			task();
		}
	};

	SharedFilesList.prototype.nextPage = function(callback = $.noop) {
		if (this.isProcessing()) {
			return callback(false, "bussy");
		}
		this.__search(callback);
	};

	/*
		{
			"last": "",
			"limit": 6,
			"numberPage": 1,
			"query": {
				"tree": {
				"operator": "OR",
				"items": [
					{
					"childs": {
						"operator": "OR",
						"items": [
						{
							"AgentVisible": "0"
						},
						{
							"AccountMessagesVisible": "1"
						},
						{
							"FlowDesignerVisible": "0"
						}
						]
					},
					"Language": "none"
					},
					{
					"childs": {
						"operator": "OR",
						"items": [
						{
							"AgentVisible": "0"
						},
						{
							"AccountMessagesVisible": "1"
						},
						{
							"FlowDesignerVisible": "0"
						}
						]
					},
					"Language": "en"
					},
					{
					"childs": {
						"operator": "OR",
						"items": [
						{
							"AgentVisible": "0"
						},
						{
							"AccountMessagesVisible": "1"
						},
						{
							"FlowDesignerVisible": "0"
						}
						]
					},
					"Language": "es"
					},
					{
					"childs": {
						"operator": "OR",
						"items": [
						{
							"AgentVisible": "0"
						},
						{
							"AccountMessagesVisible": "1"
						},
						{
							"FlowDesignerVisible": "0"
						}
						]
					},
					"Language": "fr"
					}
				]
				},
				"OR": [
					{
						"Type": "audio"
					}
				]
			}
		}
	*/
	SharedFilesList.prototype.__makeFilter = function() {
		let filter = {
			last : this.result && this.result.last || "",
			limit : this.rows,
			query : {
				tree: {
					// operator: "OR",
					// items: []
				},
				OR: []
			}
		};

		// 1 - el tipo de archivo (audio, image, etc)
		let orQuery;
		let type = this.getTypeFilter();
		if (type) {
			orQuery = orQuery || {};
			orQuery.Type = type.id;
		}

		// 2 - el texto a buscar en los campos (nombre y description)
		let text = this.getFilter();
		if (text) {
			orQuery = orQuery || {};
			orQuery.Name = text;
			orQuery.Description = text;
		}

		// 3 - la visibilidad en (agente, account messages, flow designer)
		let visibility_list = this.get_visibility();
		for (let visibility_name in visibility_list) {
			if (visibility_list.hasOwnProperty(visibility_name)) {
				let enabled = visibility_list[visibility_name];
				if (enabled) {
					let orVisibilityQuery = inConcert.Utils.cloneObject(orQuery || {});
					orVisibilityQuery[visibility_name] = "1";
					orQuery = orVisibilityQuery;
				}
			}
		}

		// 4 - las extensiones soportadas en el tipo de archivo (2) (wav, mp3, etc)
		let serch_files_extensions = this.get_search_files_extensions();
		if (serch_files_extensions && (serch_files_extensions.extensions || []).length > 0) {
			serch_files_extensions.extensions.forEach(extension => {
				let orExtQuery = inConcert.Utils.cloneObject(orQuery || {});
				orExtQuery.Extension = extension;
				filter.query.OR.push(orExtQuery);
			});
		}
		else {
			if (orQuery) {
				filter.query.OR.push(orQuery);
			}
		}

		return filter;
	};


	// SharedFilesList.prototype.getTypeVisibility = function() {
	// 	let visibility = [
	// 		{
	// 		  "childs": {
	// 			"operator": "OR",
	// 			"items": [
	// 			  {
	// 				"AgentVisible": "0"
	// 			  },
	// 			  {
	// 				"AccountMessagesVisible": "1"
	// 			  },
	// 			  {
	// 				"FlowDesignerVisible": "0"
	// 			  }
	// 			]
	// 		  },
	// 		  "Language": "none"
	// 		},
	// 		{
	// 		  "childs": {
	// 			"operator": "OR",
	// 			"items": [
	// 			  {
	// 				"AgentVisible": "0"
	// 			  },
	// 			  {
	// 				"AccountMessagesVisible": "1"
	// 			  },
	// 			  {
	// 				"FlowDesignerVisible": "0"
	// 			  }
	// 			]
	// 		  },
	// 		  "Language": "en"
	// 		},
	// 		{
	// 		  "childs": {
	// 			"operator": "OR",
	// 			"items": [
	// 			  {
	// 				"AgentVisible": "0"
	// 			  },
	// 			  {
	// 				"AccountMessagesVisible": "1"
	// 			  },
	// 			  {
	// 				"FlowDesignerVisible": "0"
	// 			  }
	// 			]
	// 		  },
	// 		  "Language": "es"
	// 		},
	// 		{
	// 		  "childs": {
	// 			"operator": "OR",
	// 			"items": [
	// 			  {
	// 				"AgentVisible": "0"
	// 			  },
	// 			  {
	// 				"AccountMessagesVisible": "1"
	// 			  },
	// 			  {
	// 				"FlowDesignerVisible": "0"
	// 			  }
	// 			]
	// 		  },
	// 		  "Language": "fr"
	// 		}
	// 	  ];
	// 	return visibility;
	// };

	SharedFilesList.prototype.__search = function(callback = $.noop) {
		const self = this;
		const filter = this.__makeFilter();
		this.startProcess(this.messages.searching);
		const filesAPI = inConcert.app.getSharedFilesAPI();
		filesAPI.searchFiles(filter, function(status, data) {
			if (!status) {
				self.message(self.messages.error);
				self.stopProcess();
				callback(false, "searching");
				return;
			}
			self.addFiles(data.rows, data.last, function() {
				self.stopProcess();
				callback(true);
			});
		});
	};

	SharedFilesList.prototype.addFiles = function(files, last, callback = $.noop) {
		const self = this;
		this.result = this.result || {
			files : [],
			files_d : {}
		};
		let currentLength = this.result.files.length;
		this.result.last = last;
		(function addFile(index) {
			if (index === files.length) {
				if (!self.result.files.length) {
					self.message(self.messages.empty);
				}
				if (self.result.files.length === currentLength) {
					self.result.allResultsFound = true;
				}
				return callback();
			}
			self.addFile(files[index], function() {
				addFile(index + 1);
			});
		})(0);
	};

	SharedFilesList.prototype.addFile = function(fileRaw, callback = $.noop) {
		const self = this;
		this.processFile(fileRaw, function(file, error) {
			if (!file) {
				return callback(false, error);
			}
			if (self.result.files_d[file.Id]) {
				return callback(false, "already added");
			}
			let first = !self.result.files.length;
			self.result.files.push(file);
			self.result.files_d[file.Id] = file;
			let $file = first && self.autoSelectFirst? self.makeFileSelectedRow(file) : self.makeFileRow(file);
			self.appendFile(file, $file);
			callback(file, $file);
		});
	};

	SharedFilesList.prototype.updateFile = function(fileRaw, callback = $.noop) {
		const self = this;
		this.processFile(fileRaw, function(file, error) {
			if (!file) {
				return callback(false, error);
			}
			let is_empty = self.result.files.length === 0;
			if (is_empty) {
				return callback(false, "list is emtpy");
			}
			for (let i = 0; i < self.result.files.length; i++) {
				let current_file = self.result.files[i];
				if (file.Id.toLowerCase() === current_file.Id.toLowerCase()) {
					self.result.files[i] = file;
					break;
				}
			}
			self.result.files_d[file.Id] = file;
			let $file = self.makeFileRow(file);
			self.appendFile(file, $file);
			callback(file, $file);
		});
	};

	SharedFilesList.prototype.processFile = function(fileRaw, callback = $.noop) {
		callback(fileRaw);
	};

	SharedFilesList.prototype.appendFile = function(_, $file) {
		this.$list().append($file);
	};

	let filesWithImage = {
		image: true
	};

	SharedFilesList.prototype.getFileImageUrl = function(file) {
		if (filesWithImage[file.Type]) {
			let api = inConcert.app.getRepositoryAPI();
			return api.getUrl({id : file.RepositoryId});
		}
		//return inConcert.app.getBaseUrl() + FORMAT("/images/{0}SharedFile.png", file.Type);
		return inConcert.shared_files_images_fn(file.Type);
	};

	SharedFilesList.prototype.__makeFileRow = function(file) {
		let $file = $(FORMAT("".concat(
			"<div list_file='{0}' class='boxSharedFileItem'>",
				"<div class='sharedFileImage'>",
					"<img src='{1}'>",
				"</div>",
				"<div class='boxSharedFileIdentificator'>",
					"{2}",
				"</div>",
				"<div class='boxSharedFileSize'>",
					"{3}",
				"</div>",
			"</div>"
		), file.Id, this.getFileImageUrl(file), file.Name, inConcert.Utils.fileSizeText(file.Size)));
		$file.prop("title", file.Description);
		return $file;
	};

	SharedFilesList.prototype.makeFileRow = function(file) {
		const self = this;
		const $file = this.__makeFileRow(file);

		var selectFile = function() {
			let f = $file.attr("list_file");
			self.selectFile(f);
		};

		$file.off("click")
			.on("click", function() {
				selectFile();
			}
		);

		// if (this.ownerId) {
		// 	const $save = $("#" + this.ownerId + "_save");
		// 	$file.off("dblclick")
		// 		.on("dblclick", function() {
		// 			selectFile();
		// 			$save.trigger("click");
		// 		}
		// 	);
		// }

		return $file;
	};

	SharedFilesList.prototype.makeFileSelectedRow = function(file) {
		const self = this;
		const $file = this.__makeFileRow(file);
		$file.attr("selected_file", true)
			.addClass("boxSharedFileItemSelect")
			.append(this.makeSharedFileActions(file));
		if (this.unSelect) {
			$file.on("click", function(e) {
				if (self.hasToUnSelectFile($(this), $(e.target))) {
					self.unSelectFile($(this).attr("list_file"));
				}
			});
		}
		return $file;
	};

	SharedFilesList.prototype.hasToUnSelectFile = function(/*$file, $target*/) {
		return true;
	};

	SharedFilesList.prototype.selectFile = function(file) {
		if (!this.multiSelect) {
			this.unSelectFiles();
		}
		file = this.getFile(file);
		if (!file) {
			return;
		}
		this.$fileRow(file.Id).replaceWith(this.makeFileSelectedRow(file));
	};

	SharedFilesList.prototype.unSelectFile = function(file) {
		file = this.getFile(file);
		if (!file) {
			return;
		}
		this.$fileRow(file.Id).replaceWith(this.makeFileRow(file));
	};

	SharedFilesList.prototype.getSelectedFiles = function() {
		const self = this;
		let files = [];
		this.$body.find("[selected_file]").each(function() {
			files.push(self.getFile($(this).attr("list_file")));
		});
		return files;
	};

	SharedFilesList.prototype.getSelectedFile = function() {
		let files = this.getSelectedFiles();
		return files.length? files[0] : null;
	};

	SharedFilesList.prototype.unSelectFiles = function() {
		const self = this;
		this.getSelectedFiles().map(function(file) {
			self.unSelectFile(file);
		});
	};

	SharedFilesList.prototype.$fileRow = function(file) {
		let fileId = typeof(file) === "string"? file : file.Id;
		return this.$body.find(FORMAT("[list_file='{0}']", fileId));
	};

	SharedFilesList.prototype.getFile = function(file) {
		if (!file || typeof(file) !== "string") {
			return file;
		}
		return this.result && this.result.files_d[file] || null;
	};

	SharedFilesList.prototype.makeSharedFileActions = function(/*file*/) {
		let actions = [];
		return actions;
	};

	//--------------------------------------------------------------------
	/*
		SharedFilesPopupList
	*/
	function SharedFilesPopupList(params)
	{
		SharedFilesPopupList.uber.constructor.call( this );
		this.clickEventName = this.clickEventName + "SHARED_FILES";
		this.owner = params.owner;
		this.ownerId = params.popupId || "fileSharedPopupId";
		this.canReposition = false;
		this.closeOnFocusLost = false;
		this.from_app = params.from_app || "administration";
		this.m_search_files_type_id = params.search_files_type_id;
		this.m_abm_files_type_id = params.abm_files_type_id;
		this.m_visibility = params.visibility;
		this.custom_class = params.custom_class;

		let callbacks = params.callbacks || {};
		this.callbacks = {
			on_show: function() {
				(callbacks.on_show || $.noop)();
			},
			on_close: function(selected_file) {
				(callbacks.on_close || $.noop)(selected_file);
			},
			on_show_maintenance: function() {
				(callbacks.on_show_maintenance || $.noop)();
			},
			on_hide_maintenance: function() {
				(callbacks.on_hide_maintenance || $.noop)();
			}
		};
	}

	inConcert.Utils.inherit(SharedFilesPopupList, inConcert.PopupWindow);

	if (typeof inConcert.SharedFilesPopupList !== "function") {
		inConcert.SharedFilesPopupList = SharedFilesPopupList;
	}

	/*******************************************************************
	 * Metodo que crea el markup contenedor de la ventana
	 */
	SharedFilesPopupList.prototype.createMarkup = function()
	{
		$("#" + this.ownerId).remove();

		let typeId = this.option_id; // "audio"; //option.Id
		let popupId = this.ownerId;

		const $node = $("".concat(
			// "<div class='systemPopWindow windowContainer sharedFileSelectorPopup ", typeId, "SharedFileSelector'>",
			// "</div>",
			"<div id='", popupId, "'  class='", (this.custom_class || "internalPopUpContainer formPopUp popUp940w sharedFileSelectorPopup"), "'", typeId, "SharedFileSelector' style='display:none;'>",
			"<div class='internalPopUpOverlay'></div>",
				"<div class='internalPopUp sharedFilePopWindow sharedFileSelectorPopup'>",
					// title
					"<div header_container class='internalPopUpTitle'>",
						"<span title_container >", _T("Select a file"), "</span>",
						"<button id='", popupId, "_cancel_times' class='internalPopUpCloseButton'><i class='far fa-times'></i></button>",
					"</div>",

					"<div class='internalPopUpMiddle pageFormContainer'>",
						"<div id='", popupId, "_message' class='genericFormMessagesContainer messageError' style='display:none;'>",
							"<label class='messageText'></label>",
							// "<a id='", popupId, "_close_message' class='windowMessagesClose' href='javascript:void(0);'>", _T("close"), "</a>",
						"</div>",

						"<div middle_container ></div>",
					"</div>",
						
					// MODAL FOOTER BUTTONS
					"<div bottom_container class='internalPopUpButtons justify-content-end'>" ,
						"<button id='", popupId, "_cancel' class='pageButtonGeneric buttonColorWhite float-right'>" , _T("Cancel"), "</button>" ,
						"<button id='", popupId, "_save' class='pageButtonGeneric buttonColorBlue float-right'>" , _T("Confirm"), "</button>" ,
					"</div>",

				"</div>",
			"</div>"
		));

		$("body").append($node);
		return $node;
	};

	/*******************************************************************
	 *
	 */
	SharedFilesPopupList.prototype.initialize = function()
	{
		this.data.container = this.createMarkup();
//		this.loadInfo();
		this.setEvents();
	};

	SharedFilesPopupList.prototype.close = function(params) {
		if (this.data && this.data.container) {
			this.data.container.remove();
		}
		this.finalize(params);
		let selected_file = params && params.selected_file;
		this.callbacks.on_close(selected_file);
	};

	SharedFilesPopupList.prototype.getAttachmentSizeLimitMb = function() {
		return 16;
	};

	SharedFilesPopupList.prototype.getAttachmentSizeLimit = function() {
		return this.getAttachmentSizeLimitMb() * 1024 * 1024;
	};

	/*******************************************************************
	 *
	 */
	SharedFilesPopupList.prototype.show = function(params)
	{
		this.option_id = params.option_id;
		this.can_create_new_file = params.can_create_new_file || false;

		SharedFilesPopupList.uber.show.call(this, params);

		const option = this.getAttachmentOption(params.option_id);
		const $list = this.__makeSharedFilesList(option);
		const $popup = $("#" + this.ownerId);
		const $middle_container = $popup.find("[middle_container]");
		$middle_container.append($list);
	 };

	 SharedFilesPopupList.prototype.__makeSharedFilesList = function(option)
	 {
		const self = this;
		this.m_sharedFilesList = new inConcert.SharedFilesList({
			fileTypes: [{
				id        : option.Id,
				name      : option.name,
				extensions: option.type.extensions
			}],
			fileTypesFilter      : false,
			maxSize              : this.getAttachmentSizeLimit(),
			agentVisible         : true,
			can_create_new_file  : this.can_create_new_file,
			create_file_and_quit : true,
			ownerId              : this.ownerId,
			owner                : this,
//			frm                  : this.owner.frm,
			option_id            : this.option_id,
			search_files_type_id: this.m_search_files_type_id,
			abm_files_type_id    : this.m_abm_files_type_id,
			visibility           : this.m_visibility,
			$get_save_btn        : function() {
				return self.__$get_save_btn();
			},
			$get_cancel_btn        : function() {
				return self.__$get_cancel_btn();
			},
			$get_cancel_btn_times        : function() {
				return self.__$get_cancel_btn_times();
			},
			callbacks: {
				on_show_maintenance: function() {
					self.callbacks.on_show_maintenance();
				},
				on_hide_maintenance: function() {
					self.callbacks.on_hide_maintenance();
				}
			}
		});
		const $list = this.m_sharedFilesList.make();
		this.m_sharedFilesList.start();
		return $list;
	};

	SharedFilesPopupList.prototype.__$get_save_btn = function()
	{
		const popupId = this.ownerId;
		const $save_btn = $("#" + popupId + "_save");
		return $save_btn;
	};

	SharedFilesPopupList.prototype.__$get_cancel_btn = function()
	{
		const popupId = this.ownerId;
		const $cancel_btn = $("#" + popupId + "_cancel");
		return $cancel_btn;
	};

	SharedFilesPopupList.prototype.__$get_cancel_btn_times = function()
	{
		const popupId = this.ownerId;
		const $cancel_btn_times = $("#" + popupId + "_cancel_times");
		return $cancel_btn_times;
	};

	SharedFilesPopupList.prototype.loadAttachmentOptions = function() {
		const options = {
			image : {
				name :  _T("Image"),
				order : 1,
				description : true,
				type : JSON.parse(JSON.stringify(inConcert.FileUploaderTypes.image))
			},
			video : {
				name :  _T("Video"),
				order : 2,
				description : true,
				type : JSON.parse(JSON.stringify(inConcert.FileUploaderTypes.video))
			},
			audio : {
				name :  _T("Audio"),
				order : 3,
				description : true,
				type : JSON.parse(JSON.stringify(inConcert.FileUploaderTypes.audio))
			},
			file : {
				name :  _T("File"),
				order : 4,
				description : true,
				type : JSON.parse(JSON.stringify(inConcert.FileUploaderTypes.file))
			}
		};
		$.each(options, function(id, option) {
			option.Id = id;
		});
		return options;
	};

	SharedFilesPopupList.prototype.getSortedAttachmentOptions = function() {
		return inConcert.Utils.hash_to_sorted_array(this.loadAttachmentOptions(), function(option) {
			return option.order;
		});
	};

	SharedFilesPopupList.prototype.getAttachmentOption = function(option_id) {
		let options = this.loadAttachmentOptions();
		let option = options[option_id.toLowerCase()];
		if (!option) {
			throw("Unrecognized shared file type " + option_id);
		}
		return option;
	};

	SharedFilesPopupList.prototype.returnSelectedFile = function()
	{
		let selected_file = this.m_sharedFilesList.getSelectedFile();
		if (!selected_file) {
			let msg_err = _T("Must select a file.");
			this.displayStatusMessage(msg_err, "Error", true);
			return;
		}
		this.close({selected_file: selected_file});
	};

	/*******************************************************************
	* Metodo que setea eventos de los controles
	*/
	SharedFilesPopupList.prototype.setEvents = function()
	{
		const self = this;

		const popupId = this.ownerId;
		$("#" + popupId + "_save").unbind("click").click(function() {
			self.returnSelectedFile();
		});
		$("#" + popupId + "_cancel").unbind("click").click(function() {
			self.close();
			self.callbacks.on_close();
		});
		$("#" + popupId + "_cancel_times").unbind("click").click(function() {
			self.close();
			self.callbacks.on_close();
		});
		$("#" + popupId + "_close_message").unbind("click").click(function() {
			self.hideStatusMessage();
		});
	};

	/*******************************************************************
	*
	*/
	 SharedFilesPopupList.prototype.displayStatusMessage = function(text, type, is_closeable, timeout_ms) {
		const popupId = this.ownerId;
		const $section = $("#" + popupId + "_message");
		$section.removeClass().addClass("genericFormMessagesContainer").addClass("message" + type);
		$section.show().find(".messageText").text(text);
		if (timeout_ms) {
			window.setTimeout(function() {
				$section.hide();
			}, timeout_ms);
		}
		const $close_message = $("#" + popupId + "_close_message");
		if (is_closeable) {
			$close_message.show();
		}
		else {
			$close_message.hide();
		}
	};

	/*******************************************************************
	*
	*/
	 SharedFilesPopupList.prototype.hideStatusMessage = function() {
		const popupId = this.ownerId;
		const $section = $("#" + popupId + "_message");
		$section.hide();
	};

	/*******************************************************************
	*
	*/
	SharedFilesPopupList.prototype.edit_file = function(item) {
		this.m_sharedFilesList.edit_file(item);
	};
}());
