/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 11 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	"use strict";

	const _T = inConcert.i18n.getString;
	const FORMAT = inConcert.Utils.format;

	const get_default_src = function(fileType = "") {
		let src_def = {
			audio: "/inconcert/shared/shared_files/images/audioSharedFile.svg",
			file : "/inconcert/shared/shared_files/images/fileSharedFile.svg",
			image: "/inconcert/shared/shared_files/images/imageSharedFile.svg",
			video: "/inconcert/shared/shared_files/images/videoSharedFile.svg"
		};
		let src = src_def[fileType] || "/inconcert/apps/agent/images/blank.png";
		return src;
	};

	//---------------------------------------------------------------------
	// form shared files class
	function MaintenanceMng (params)
	{
		if (!params) {
			throw("SharedFiles MaintenanceMng Error - params is required");
		}
		if (!params.owner) {
			throw("SharedFiles MaintenanceMng Error - params.owner is required");
		}
		if (!params.container_id) {
			throw("SharedFiles MaintenanceMng Error - params.container_id is required");
		}
		if (!params.prefix_id) {
			throw("SharedFiles MaintenanceMng Error - params.prefix_id is required");
		}

		/*
			instance_type (EXTERNAL o INTERNAL)
				EXTERNAL: (Por defecto) - Sirve para saber si la clase depende la app Config-->Adm->ABM de Archivos Compartidos.
				INTERNAL: (Lo nuevo)    - Sirve para saber si la clase NO depende de quien la haya instanciado, es decir se puede instanciar y utilizar desde otras apps u otros lugares.
											Ej: Adm Accounts Messages, Flow Designer
		*/
		this.instance_type          = params.instance_type || "EXTERNAL";
		this.owner                  = params.owner;
		this.container_id           = params.container_id;
		this.prefix_id              = params.prefix_id;
		this.prefix_popup_import_id = params.prefix_id + "importation_";
		this.prefix_popup_csv_id    = params.prefix_id + "csv_";

		this.m_search_files_types  = null;
		if (params.search_files_type_id) {
			this.m_search_files_types = inConcert.FileUploaderTypes[params.search_files_type_id] || null;
		}
		this.m_abm_files_types  = null;
		if (params.abm_files_type_id) {
			this.m_abm_files_types = inConcert.FileUploaderTypes[params.abm_files_type_id] || null;
		}

		this.m_visibility = params.visibility;
		this.m_allowsMultipleFiles  = params.allowsMultipleFiles || false;
		this.m_maximum_number_of_files_in_import = params.maximum_number_of_files_in_import || -1;

		this.m_grdFiles = null;

		let callbacks = params.callbacks || {};
		this.callbacks = {
			on_show: function() {
				(callbacks.on_show || $.noop)();
			},
			on_close: function() {
				(callbacks.on_close || $.noop)();
			},
			on_save: function(new_file) {
				(callbacks.on_save || $.noop)(new_file);
			},
			on_error: function(err) {
				(callbacks.on_error || $.noop)(err);
			}
		};

		this.is_rendering = false;

		return true;
	}

	if (typeof inConcert.shared_files_maintenance_mng !== "function") {
		inConcert.shared_files_maintenance_mng = MaintenanceMng;
	}

	const m_languages = [
		{id: "none", label: _T("Unspecified")},
		{id: "en"  , label: _T("English")    },
		{id: "es"  , label: _T("Spanish")    },
		{id: "fr"  , label: _T("French")     }
	];

	if (!inConcert.shared_files_types) {
		inConcert.shared_files_languages = m_languages;
	}

	const sharedFilesTypes = [
		{id: "image", label: _T("Image")},
		{id: "audio", label: _T("Audio")},
		{id: "video", label: _T("Video")},
		{id: "file" , label: _T("File") }
	];

	if (!inConcert.shared_files_types) {
		inConcert.shared_files_types = sharedFilesTypes;
	}

	const m_visibilty = [
		{id: "agent"           , label: _T("Agent")           },
		{id: "account messages", label: _T("Account Messages")},
		{id: "flow designer"   , label: _T("Process Flows")   }
	];

	if (!inConcert.shared_files_visibilty) {
		inConcert.shared_files_visibilty = m_visibilty;
	}

	if (!inConcert.shared_files_images_fn) {
		inConcert.shared_files_images_fn = get_default_src;
	}

//START STATIC -----------------------------------
//
	MaintenanceMng.prototype.loadSizeLimit = function(status, result, $element_destination) {
		if (!$element_destination) {
			return;
		}
		let id_files_size_limit = (this.prefix_id || "shared_files_maintenance_mng") + "_files_size_limit";
		$("#" + id_files_size_limit).remove();
		if (!status || !result) {
			return;
		}
		let sizeLimit = result.size_limit_mb || 0;
		let usedSize = inConcert.Utils.fileSizeMB(result.total_size);
		let $limit = $("<div id='" + id_files_size_limit + "' class='sharedFilesSizeLimit'></div>");
		$limit.data("limit", sizeLimit);
		$element_destination.after($limit);
		MaintenanceMng.prototype.setSizeLimit(usedSize);
	};

	MaintenanceMng.prototype.reloadSizeLimit = function(callback = $.noop) {
		let filesAPI = inConcert.app.getSharedFilesAPI();
		filesAPI.getUsedSpace(function(status, usedSize) {
			if (status) {
				MaintenanceMng.prototype.setSizeLimit(inConcert.Utils.fileSizeMB(usedSize));
			}
			callback();
		});
	};

	MaintenanceMng.prototype.setSizeLimit = function(usedSize) {
		let $limit = $("#" + (this.prefix_id || "shared_files_maintenance_mng") + "_files_size_limit");
		if (!$limit.length) {
			return;
		}
		let sizeLimit = $limit.data("limit");
		$limit.toggleClass("limitWarning", sizeLimit - usedSize < 10)
			.text(FORMAT(_T("{0} of {1} MB used."), usedSize, sizeLimit));
	};

	MaintenanceMng.prototype.getImageUrl = function(file) {
		if (file.Type === "image") {
			let api = inConcert.app.getRepositoryAPI();
			return api.getUrl({id : file.RepositoryId});
		}
		return get_default_src(file.Type);
	};

//
//END STATIC -----------------------------------

	MaintenanceMng.prototype.__is_internal = function()
	{
		return this.instance_type === "INTERNAL";
	};

	MaintenanceMng.prototype.__make_popup_html = function()
	{
		let prefix_id = this.prefix_id;

		let html = "".concat(
			"<div id='", prefix_id, "internal_message' class='row'>" ,
				"<div class='col-12'>" ,
					"<div id='", prefix_id, "disclaimer' style='display:none;' class='inlineMessagesContainer messageInfo'>" , 
					"</div>" ,
				"</div>" ,
			"</div>" ,
			"<div id='", prefix_id, "new_container' class='row'>",
				"<div class='col-6'>" ,
					"<div class='card-header'>" , 
						"<h5 id='", prefix_id, "total_rows_msg'></h5>",
						"<label id='", prefix_id, "total_err_msg' class='totalErrorsContainer' style='display:none;'></label>",
					"</div>",
				"</div>",
				"<div id='", prefix_id, "show_only_errors_container' style='display:none;' class='col-6 showErrorsCheckboxContainer'>",
					"<input id='", prefix_id, "show_only_errors' class='form-check-input' type='checkbox'>",
					"<label for='", prefix_id, "show_only_errors' class=''>", _T("Show only errors"), "</label>",
				"</div>",
			"</div>",
			"<div class='card-block'>",
				"<div id='", prefix_id, "container'>",
					"<div id='", prefix_id, "popup' class='card-block' style='display:none;'>",
						"<div class='card-block'>",
							"<div class='row'>",
								"<div class='col-6'>",
									"<div class='form-group'>",
										"<label for='", prefix_id, "popup_name' class='required'>", _T("Name"), _T(":"), "</label>",
										"<input id='", prefix_id, "popup_name' class='form-control' type='text' />",
										"<i id='", prefix_id, "popup_name_validation' class='formFieldValidation formFieldError' hidden></i>",
									"</div>",
								"</div>",
								"<div class='col-6'>",
									"<div class='form-group'>",
										"<label for='", prefix_id, "popup_value' class='required'>", _T("Value"), _T(":"), "</label>",
										"<input id='", prefix_id, "popup_value' class='form-control' type='text' />",
										"<i id='", prefix_id, "popup_value_validation' class='formFieldValidation formFieldError' hidden></i>",
									"</div>",
								"</div>",
							"</div>",
							"<div id='", prefix_id, "popup_message' class='genericFormMessagesContainer messageError' style='display:none;'>",
								"<label class='messageText'></label>",
							"</div>",
							"<div class='pageButtonContainer internal'>",
								"<input id='", prefix_id, "popup_create' class='pageButtonGeneric buttonColorBlack' type='button' value='", _T("Create"), "' />",
								"<input id='", prefix_id, "popup_create_and_continue' class='pageButtonGeneric buttonColorBlack' type='button' value='", _T("Create & continue"), "' />",
								"<input id='", prefix_id, "popup_close' class='pageButtonGeneric buttonColorGray' type='button' value='", _T("Close"), "' />",
							"</div>",
						"</div>",
					"</div>",
					"<div id='", prefix_id, "not_results_found' class='inlineMessagesContainer messageInfo' style='display:none'></div>",
					"<div id='", prefix_id, "grid_container'>",
						"<!-- El markup correspondiente a la tabla se genera dentro de la clase 'conf_form_paginator.js' -->",
					"</div>",
					// start - edit popup default values
					"<div id='", prefix_id, "default_values_popup' class='internalPopUpContainer importSharedFilesPopup formPopUp popUp640w' style='display:none;'>",
						"<div class='internalPopUpOverlay'></div>",
						"<div class='internalPopUp'>",
							"<div id='", prefix_id, "default_values_popup_waiting' class='internalWaitingContainer' style='display:none;'>",
								"<div waiting_message='true' class='internalWaitingMessage'></div>",
								"<div class='internalWaitingAnimation'></div>",
								"<div class='internalWaitingOverlay'></div>",
							"</div>",
							"<div id='", prefix_id, "default_values_popup_title' class='internalPopUpTitle'>",
								"<span popup_title_text=''>...</span>",
								"<button popup_close='' class='internalPopUpCloseButton'><i class='far fa-times'></i></button>",
							"</div>",
							"<div class='internalPopUpMiddle pageFormContainer'>",
								"<div class='card no-card'>",
									"<div class='card-block'>",
										"<div id='", prefix_id, "default_values_popup_message' class='genericFormMessagesContainer messageError' style='display:none;'>",
											"<label class='messageText'></label>",
										"</div>",
										"<div class='row'>",
											"<div class='col-12'>",
												"<div id='", prefix_id, "default_values_popup_total_msg' class='inlineMessagesContainer messageInfo'>........................</div>",
											"</div>",
										"</div>",
										"<div class='card-block'>",
											"<div class='row'>",
												"<div class='col-12'>",
													"<div class='form-group'>",
														"<label for='", prefix_id, "default_values_popup_language' class=''>", _T("Language"), _T(":"), "</label>",
														"<select id='", prefix_id, "default_values_popup_language' class='form-control'></select>",
													"</div>",
												"</div>",
											"</div>",
											"<div class='row'>" ,
												"<div class='col-12'>" ,
													"<div class='card-header'>" , 
														"<h5>", _T("Visibility"), " <span class='fa fa-question-circle' title='", _T("Specify where this file can be used"), "'></span></h5>",
													"</div>" ,
												"</div>" ,
											"</div>" ,
											"<div class='row'>",
												"<div class='col-12'>",
													"<div class='form-group'>",
														"<div class='form-check mr-3'>",
															"<label for='", prefix_id, "default_values_popup_agent_visible' class='form-check-label form-check-label-title'>", "<input id='", prefix_id, "default_values_popup_agent_visible' class='form-check-input' type='checkbox'>", _T("Agent"), "</label>",
														"</div>" ,
														"<div class='form-check mr-3'>",
															"<label for='", prefix_id, "default_values_popup_account_visible' class='form-check-label form-check-label-title'>", "<input id='", prefix_id, "default_values_popup_account_visible' class='form-check-input' type='checkbox'>", _T("Account Messages"),  "</label>",
														"</div>" ,
														"<div class='form-check mr-3'>",
															"<label for='", prefix_id, "default_values_popup_flow_visible' class='form-check-label form-check-label-title'>", "<input id='", prefix_id, "default_values_popup_flow_visible' class='form-check-input' type='checkbox'>", _T("Process Flows"),"</label>",
														"</div>" ,
												"</div>",
											"</div>",
		//									"<div class='row'>",
		//										"<div class='col-12'>",
		//											"<label for='", prefix_id, "default_values_popup_language' class=''>", _T("Description"), _T(":"), "</label>",
		//											"<select id='", prefix_id, "default_values_popup_language' class='form-control'></select>",
		//										"</div>",
		//									"</div>",
										"</div>",
										"<div id='", prefix_id, "default_values_popup_metadata_panel'>",
										"</div>",
									"</div>",
								"</div>",
							"</div>",
							"<div class='internalPopUpButtons d-flex justify-content-end'>",
								"<button id='", prefix_id, "default_values_popup_create' class='pageButtonGeneric buttonColorBlack'>",
									_T("Confirm"),
								"</button>",
							"</div>",
						"</div>",
					"</div>",
					// end - edit popup default values
				"</div>",
				"<div id='", prefix_id, "paginator'></div>",
			"</div>"
		);

		return html;
	};

	MaintenanceMng.prototype.__build_html = function()
	{
// if (this.is_rendering) {
// 	return false;
// }
		let content_popup_html = this.__make_popup_html();
		let prefix_id = this.prefix_id;

		let content_html = "".concat(
			"<div class='card'>",
				"<div class='card-block'>",

					// upload container row
					"<div class='row'>",
						"<div class='col-12'>",
							"<div class='form-group d-flex flex-column'>",
								"<div id='", prefix_id, "file_share_container' class='sharedFileUploadContainer'>",
									"<div class='w-100 d-flex flex-column'>",
										"<label class='required'>", _T("Upload File"), _T(":"), "</label>",
										"<div id='", prefix_id, "file_share_control' class='sharedFileUploadControl'>",
											_T("Drag and drop or select one or more files"),
										"</div>",
									"</div>",
								"</div>",
							"</div>",
						"</div>",
					"</div>",

					// Files container
					"<div id='", prefix_id, "simple_file_container' style='display:none;'>",
						"<hr />",
						"<div class='row pt-2'>",
							"<div class='col-6'>",
								"<div class='form-group'>",
									"<label for='", prefix_id, "file_name' class='required'>", _T("Name"), _T(":"), "</label>",
									"<input id='", prefix_id, "file_name' autocomplete='new-user' class='form-control' type='text' placeholder='", _T("Enter name"), "...' />",
									"<i id='", prefix_id, "file_name_validation' class='formFieldValidation formFieldError' hidden></i>",
								"</div>",
							"</div>",
							"<div class='col-6'>",
								"<div class='form-group'>",
									"<label for='", prefix_id, "file_description' class='required'>", _T("Description"), _T(":"), "</label>",
									"<input id='", prefix_id, "file_description' autocomplete='new-user' class='form-control' type='text' placeholder='", _T("Enter identifier"), "...' />",
									"<i id='", prefix_id, "file_description_validation' class='formFieldValidation formFieldError' hidden></i>",
								"</div>",
							"</div>",
						"</div>",
						"<div class='row'>",
							"<div class='col-6'>",
								"<div class='form-group'>",
									"<label for='", prefix_id, "file_type' >", _T("Type"), _T(":"), "</label>",
									"<select id='", prefix_id, "file_type' class='form-control'></select>",
								"</div>",
							"</div>",
							"<div class='col-6'>",
								"<div class='form-group'>",
									"<label for='", prefix_id, "file_language' >", _T("Language"), _T(":"), "</label>",
									"<select id='", prefix_id, "file_language' class='form-control'></select>",
								"</div>",
							"</div>",
						"</div>",
						"<hr />",
						//TODO: RODRIGO AGREGAR EL MARKUP DE VISIBILIDAD.
						"<div class='row'>" ,
							"<div class='col-12'>" ,
								"<div class='card-header'>" , 
									"<h5>", _T("Visibility"), " <span class='fa fa-question-circle' title='", _T("Specify where this file can be used"), "'></span></h5>" ,
								"</div>" ,
							"</div>" ,
						"</div>" , 

						"<div class='row'>",
							"<div class='col-12'>",
								"<div class='form-group'>",
									"<div class='form-check mr-3'>",
										"<label for='", prefix_id, "file_agent_visible' class='form-check-label form-check-label-title'>",
											"<input id='", prefix_id, "file_agent_visible' class='form-check-input' type='checkbox' />",
											_T("Agent"),
										"</label>",
									"</div>",
									"<div class='form-check mr-3 ml-3'>",
										"<label for='", prefix_id, "file_account_visible' class='form-check-label form-check-label-title'>",
											"<input id='", prefix_id, "file_account_visible' class='form-check-input' type='checkbox' />",
											_T("Account Messages"),
										"</label>",
									"</div>",
									"<div class='form-check ml-3'>",
										"<label for='", prefix_id, "file_flow_visible' class='form-check-label form-check-label-title'>",
											"<input id='", prefix_id, "file_flow_visible' class='form-check-input' type='checkbox' />",
											_T("Process Flows"),
										"</label>",
									"</div>",
								"</div>",
							"</div>",
						"</div>",
					"</div>",

					"<div id='", prefix_id, "multi_files_container' style='display:none;'>",
						content_popup_html,
					"</div>",

				"</div>",
			"</div>",
		);

		if (this.__is_internal()) {
			content_html = content_html.concat(
				//"<div class='form___________Row'>",
					// MODAL FOOTER BUTTONS
					"<div class='internalPopUpButtons'>",
						"<button id='", prefix_id, "cancel' class='pageButtonGeneric buttonColorWhite float-right'>" , _T("Cancel") , "</button>" ,
						"<button id='", prefix_id, "save' class='pageButtonGeneric buttonColorBlue float-right'>" , _T("Confirm") , "</button>" ,
					"</div>",
				//"</div>"
			);
		}

		const $content = $(content_html);
		const $container = $("#" + this.container_id);
		$container.html($content);
		this.is_rendering = true;
	};

	MaintenanceMng.prototype.initialize = function(/*params*/)
	{
		this.__build_html();
		this.mapUpdateFiles = null;
		this.__init_grid();
	};

	MaintenanceMng.prototype.__init_grid = function()
	{
		const self = this;

		if (!this.allowsMultipleFiles()) {
			return;
		}

		this.m_grdFiles = new inConcert.conf_form_paginator({
			owner: this.owner,
			frm: this.owner,
			row_id: "RepositoryId", //row_id: "Name",
			display_name: "Name",
			rows_per_page: 20000,
			show_at_least: 19999,
			paginator_type: "ALL_ROWS_CLIENT_SIDE",
			without_paginator: true,
//			use_instance_of_inconcert_popup: true, // con este flag el popup se instancia desde el popup inconcert popup window.
			auto_initialize: true,
			header_hidden: true,
			header_lables_inside_fields: true,
			markup_with_divs: true, // Con este flag la grilla se crea con DIV+UL+LI,etc en lugar de ser creada con TABLE+THEAD+TBODY,etc
			ids: {
				grid_container: this.prefix_id + "grid_container",
				paginator: this.prefix_id + "paginator",
				grid: this.prefix_id + "list",
				not_results_found: this.prefix_id + "not_results_found",
				// new_item: this.prefix_id + "new",
				disclaimer: this.prefix_id + "disclaimer",
			//popup0
				popup: this.prefix_id + "default_values_popup",
				popup_msg: this.prefix_id + "default_values_popup_message",
				popup_create: this.prefix_id + "default_values_popup_create",
				popup_title: this.prefix_id + "default_values_popup_title",
				popup_close: this.prefix_id + "default_values_popup_close",
				popup_input_id: this.prefix_id + "default_values_popup_question",
				popup_waiting: this.prefix_id + "default_values_popup_waiting",
			},
			strings: {
				not_results_found         : _T("No files found."),
				title_new                 : _T("Files Configuration"),
				button_create             : _T("Confirm"),
				can_remove                : _T("Do you want to delete: {0} ?")
			},
			columns: [
				{id: "_error_mark"    , name: "ERR MARK", type: "row_error_mark", cell_class_name: ""},
//				{id: "_number"    , name: "", type: "counter"},
//{id: "_select"    , name: "", type: "row_selector"},

				{id: "Image" , name: "", type: "image", cell_class_name: "col-1", //"sharedFileIconColumnHeader",
					get_value: function(file) {
						let src;
						if (file.Type !== "image") {
							src = get_default_src(file.Type);
						}
						else {
							src = file.blobContent;
							if (!src) {
								if (file.RepositoryId) {
									src = inConcert.shared_files_maintenance_mng.prototype.getImageUrl(file);
								}
								else {
									src = get_default_src(file.Type);
								}
							} 
						}

						return {
						//	class_name       : "fileType" + file.Type,
							column_class_name: "sharedFileIconColumn pr-0",
							value            : "",
						//	type             : "image",
							src              : src
						};
					}
				},

				{id: "Name", name: _T("Name"), type: "cell_input", cell_class_name: "col-12 pr-0 mb-2", grouper: {start: true, class_name: "col"},
					// validation: [
					// 	{
					// 		required: true,
					// 		message: _T("The field name can not be empty.")
					// 	}
					// ],
					// get_value: function(item) {
					// 	return self.get_grid_value(item, "Name");
					// },
					// is_disable_editable_cell: function(item) {
					// 	return true;
					// }
				},
				{id: "Language"   , name: _T("Language")   , type: "cell_combo", cell_class_name: "col-12 pr-0", grouper: {end: true},
					get_value: function(item) {
						return self.get_grid_value(item, "Language");
					},
					is_disable_editable_cell: function() {
						return false;
					}
				},

				{id: "Description", name: _T("Description"), type: "cell_input", cell_class_name: "col-6 d-inline-flex pl-0", grouper: {start: true, class_name: "col", index: 1}},
				{id: "Type"       , name: _T("Type")       , type: "cell_combo", cell_class_name: "col-5  d-inline-flex pl-0",
					get_value: function(item) {
						return self.get_grid_value(item, "Type");
					},
					is_disable_editable_cell: function() {
						return true;
					}
				},
				{id: "AgentVisible"          , name: _T("Agent")           , type: "cell_checkbox", cell_class_name: "col mt-2 px-0", grouper: {middle: true, index: 2}},
				{id: "AccountMessagesVisible", name: _T("Account Messages"), type: "cell_checkbox", cell_class_name: "col mt-2 px-0", grouper: {middle: true, index: 3}},
				{id: "FlowDesignerVisible"   , name: _T("Process Flows")   , type: "cell_checkbox", cell_class_name: "col mt-2 px-0", grouper: {end: true, index: 4}},
				{
					id: "_action",
					name: "",
					cell_class_name: "",
					system_action: {
					//	edit: true,
						erase: true
					}
				},
				{id: "_error_msg"            , name: "ERR MSG", type: "row_error_msg", cell_class_name: "col-12 px-0"},
			],
			callbacks: {
				save: function(callback, _, current_popup) {
					callback = callback || $.noop;
					if (current_popup === 0 || current_popup === "") {
						self.__add_grid_files_with_default_values();
					}
				},
				edit: function(_, current_popup) {
					if (current_popup === 0 || current_popup === "") {
						self.__open_popup_grid_default_files_values();
					}
				},
 				on_closed_popup: function(current_popup) {
					if (current_popup === 0 || current_popup === "") {
						// (**)
						self.__add_grid_files_values(false);
					}
 				},
				on_removed_row: function(row) {
					const on_removed_row = function() {
						// acutlizo informacion
						delete self.m_uploadedFilesList[row.RepositoryId];
						delete self.m_errorsFilesList[row.RepositoryId];
						self.update_totals();
					};
					if (!self.m_uploadedFilesList[row.RepositoryId] && !self.m_errorsFilesList[row.RepositoryId]) {
						// 1 - el caso de Upload x CSV y que aun no se han subido archivos.
						on_removed_row();
						return;
					}
					// 2 - el caso de Upload por Tipo de Archivo, primero envio a eliminar del repo el file.
					self.deleteFiles([row.RepositoryId], function() {
						on_removed_row();
					});
				}
			}
		});
	};

	MaintenanceMng.prototype.__add_grid_files_with_default_values = function()
	{
		const $agent_visible = $("#" + this.prefix_id + "default_values_popup_agent_visible");
		const $account_visible = $("#" + this.prefix_id + "default_values_popup_account_visible");
		const $flow_visible = $("#" + this.prefix_id + "default_values_popup_flow_visible");

		if (!$agent_visible.prop("checked") && !$account_visible.prop("checked") && !$flow_visible.prop("checked")) {
			this.m_grdFiles.show_popup_msg(
				_T("You must select at least one visibility option."),
				"Error",
				5000
			);
			return;
		}
		this.m_grdFiles.hide_popup({
			must_invoke_callback: false // no invoco el callback "on_closed_popup" para que NO se ejecute 2 veces el Add de la grilla (**).
		});
		this.__add_grid_files_values(true);
	};

	MaintenanceMng.prototype.__open_popup_grid_default_files_values = function()
	{
		let total = (this.m_grdFiles.__uploaded_files || []).length;
		const $total_msg = $("#" + this.prefix_id + "default_values_popup_total_msg");
		let msg = FORMAT(_T("{0} files have been uploaded. You can update your general settings."), total);
		$total_msg.text(msg);
	};

	/*
		Funcion encargada de agregar files a la grilla, que se invoca cuando se cierra el popup de valor por defecto (x lote).
		Aca se determina si se deben o no actualizar los valores que se agregaran a la grilla.
	*/
	MaintenanceMng.prototype.__add_grid_files_values = function(must_update_default_batch_values, callback = $.noop)
	{
		const self = this;

		if ((this.m_grdFiles.__uploaded_files || []).length === 0) {
			callback();
			return;
		}
		let uploaded_files = this.m_grdFiles.__uploaded_files;
		let default_values;

		if (must_update_default_batch_values) {
			const $language = $("#" + this.prefix_id + "default_values_popup_language");
			const $agent_visible = $("#" + this.prefix_id + "default_values_popup_agent_visible");
			const $account_visible = $("#" + this.prefix_id + "default_values_popup_account_visible");
			const $flow_visible = $("#" + this.prefix_id + "default_values_popup_flow_visible");

			default_values = {
				Language              : $language.val(),
				AgentVisible          : $agent_visible.prop("checked") ? "1": 0,
				AccountMessagesVisible: $account_visible.prop("checked") ? "1": 0,
				FlowDesignerVisible   : $flow_visible.prop("checked") ? "1": 0
			};
		}

		let waiting_msg = _T("Loading, wait moment please");
		this.show_waiting(waiting_msg);

		// Se carga en un foreach que corre dentro de un timeout para que el navegador no quede colgado
		inConcert.Utils.foreach_timeout({
			rows: uploaded_files,
			max_append_per_loop: 50,
			pause_between_loop_ms: 10,
			waiting_message: waiting_msg + ", {0} of {1}.",
			append_fn : function(uploaded_file) {
				if (default_values) {
					$.extend(true, uploaded_file, default_values);
				}
				let $current_row = self.m_grdFiles.$find_row_by_name(uploaded_file.Name);
				if ($current_row && $current_row.length > 0) {
					let file_info = uploaded_file;
					if (self.m_from === "csv") {
						let csv_file_info = self.m_grdFiles.get_row_data($current_row);
						if (csv_file_info) {
							file_info = csv_file_info;
							file_info.RepositoryId = uploaded_file.RepositoryId;
							file_info.blobContent = uploaded_file.blobContent;
						}
					}
					self.m_grdFiles.update_row(file_info, null, {find_type: "row_name"});
				}
				else {
					self.m_grdFiles.add_or_update_row(uploaded_file);
				}
			},
			waiting_fn : function(message) {
				self.show_waiting(message);
			},
			complete_fn : function() {
				self.hide_waiting();
				self.m_grdFiles.__uploaded_files = null;
				callback();
			}
		});
	};

	MaintenanceMng.prototype.get_grid_value = function(item, column_id)
	{
		const value = {
			original_value: "",
			value         : "",
			search_value  : ""
		};

		let item_value = item[column_id];
		if (item_value === undefined) {
			item_value = "";
		}

		value.original_value = item_value;
		value.value = item_value;
		value.search_value = item_value;

		if (column_id === "Type") {
			value.combo_info = {
				list               : sharedFilesTypes,
				field_id           : "id",
				field_name         : "label",
				is_blank_first_item: false,
				sorted             : true,
				selected_value     : this.m_new_file_type
			};
		}
		else if (column_id === "Language") {
			value.combo_info = {
				list               : m_languages,
				field_id           : "id",
				field_name         : "label",
				is_blank_first_item: false,
				sorted             : true,
				selected_value     : item[column_id] || "none"
			};
		}

		return value;
	};

	MaintenanceMng.prototype.show = function(params = {})
	{
		this.initialize(params);
		this.load_data(params);
		this.set_events();
		if (params.option_id) {
			$("#" + this.prefix_id + "file_type").trigger("change");
		}
		this.callbacks.on_show();
	};

	MaintenanceMng.prototype.fill_combo = function($combo, is_blank_first_item, list, field_id, field_name, sorted, can_add_item, data_id, id_is_case_sensitive)
	{
		return inConcert.Utils.fill_combo($combo, is_blank_first_item, list, field_id, field_name, sorted, can_add_item, data_id, id_is_case_sensitive);
	};

	MaintenanceMng.prototype.show_status_message = function(message, type, timeout_ms, scrolling, show_custom_button, $custom_message_container)
	{
		this.owner.show_status_message(message, type, timeout_ms, scrolling, show_custom_button, $custom_message_container);
	};

	MaintenanceMng.prototype.hide_status_message = function()
	{
		this.owner.hide_status_message();
	};

	MaintenanceMng.prototype.get_search_files_extensions = function()
	{
		return this.m_search_files_types;
	};

	MaintenanceMng.prototype.load_data = function(params = {})
	{
		const self = this;

		let file_type_id = params.option_id;
		this.m_allowsMultipleFiles = params.allowsMultipleFiles || this.m_allowsMultipleFiles;

		let allowed_files_types, uploader_file_types;
		if (this.m_search_files_types) {
			allowed_files_types = [this.m_search_files_types];
		}
		else {
			allowed_files_types = sharedFilesTypes;
		}

		if (this.m_abm_files_types) {
			uploader_file_types = this.m_abm_files_types;
		}
		else {
			uploader_file_types = sharedFilesTypes;
		}

		const $cbo_type = $("#" + this.prefix_id + "file_type");
		this.fill_combo($cbo_type, false, allowed_files_types, "id", "label");

		const $cbo_language = $("#" + this.prefix_id + "file_language");
		this.fill_combo($cbo_language, false, m_languages, "id", "label");

		const $cbo_default_language = $("#" + this.prefix_id + "default_values_popup_language");
		this.fill_combo($cbo_default_language, false, m_languages, "id", "label");

//		let uploader_file_types;
//		if (this.fileTypes) {
//			uploader_file_types =  inConcert.FileUploaderTypes.asterisk_audio; //e ["wav"];
//		}
//		else {
//			uploader_file_types = inConcert.FileUploaderTypes[allowed_files_types[0].id];
//		}

		if (file_type_id) {
			$cbo_type.val(file_type_id);
			$cbo_type.prop("disabled", true);
		}
		else {
			$cbo_type.prop("disabled", false);
		}

		if (this.m_uploader) {
			this.m_uploader.destroy();
			this.m_uploader = null;
		}

		this.m_uploader = new inConcert.FileUploaderControl({
			uploader : {
				$control : $("#" + this.prefix_id + "file_share_control")
			},
			dragger : {
				$control : $("#" + this.prefix_id + "file_share_control")
			},
			preview : {
				$control : $("#" + this.prefix_id + "file_share_container"),
				canRemove: false
			},
			type               : uploader_file_types,
			uploadOnAdd        : true,
			disabled           : this.is_readonly(),
			allowsMultipleFiles: this.m_allowsMultipleFiles,
			sizeLimitMB        : 20 //0.1
			// filesLimit         : this.m_maximum_number_of_files_in_import,
			// overrideFiles      : false
		});

		this.m_uploader.onFileAddError = function(_, result, callback) {
			self.show_status_message(result.reason, "Error", 3000);
			callback();
		};

		this.m_uploader.onConvertFileStart = function() {
			self.show_waiting(_T("Converting file..."));
		};

		this.m_uploader.onConvertFileEnd = function() {
			self.hide_waiting();
		};

		this.m_telephonyAudio = this.m_search_files_types?.telephonyAudio || false;
		this.m_uploader.telephonyAudio = this.m_telephonyAudio;
	};

	MaintenanceMng.prototype.is_readonly = function()
	{
		return this.owner.is_readonly();
	};

	MaintenanceMng.prototype.allowsMultipleFiles = function()
	{
		if (!this.m_allowsMultipleFiles) {
			return false;
		}
		if (this.is_new()) {
			return true;
		}
		return false;
	};

	MaintenanceMng.prototype.setMultipleFiles = function(allow)
	{
		this.m_allowsMultipleFiles = allow;
		this.m_uploader.setMultipleFiles(allow);
		this.updateGUI();
	};

	MaintenanceMng.prototype.updateGUI = function()
	{
		let lbl_drag_and_drop = _T("Drag and drop or select one or more files");
		if (!this.m_allowsMultipleFiles) {
			lbl_drag_and_drop = _T("Drag & drop or select a file");
		}
		const $drag = $("#" + this.prefix_id +  "file_share_control");
		$drag.text(lbl_drag_and_drop);
	};

	MaintenanceMng.prototype.__toggle_files_container = function()
	{
		const $simple_file_container =  $("#" + this.prefix_id + "simple_file_container");
		const $multi_files_container =  $("#" + this.prefix_id + "multi_files_container");
		if (this.allowsMultipleFiles()) {
			$simple_file_container.hide();
			$multi_files_container.show();
			return;
		}
		$multi_files_container.hide();
		$simple_file_container.show();
	};

	MaintenanceMng.prototype.update_totals = function()
	{
		let resp = {
			total_upladed_files: 0,
			total_errors_files: 0
		};
		const $total_upladed_files =  $("#" + this.prefix_id + "total_rows_msg");
		resp.total_upladed_files = Object.keys(this.m_uploadedFilesList || {}).length;

		if (resp.total_upladed_files === 0) {
			$total_upladed_files.text(_T("Uploaded Files"));
		}
		else {
			$total_upladed_files.text(_T("Uploaded Files") + FORMAT(" ({0})", resp.total_upladed_files));
		}

		const $show_only_errors_container = $("#" + this.prefix_id + "show_only_errors_container");
		const $show_only_errors = $("#" + this.prefix_id + "show_only_errors");
		const $total_errors_files =  $("#" + this.prefix_id + "total_err_msg");
		resp.total_errors_files = Object.keys(this.m_errorsFilesList || {}).length;

		if (resp.total_errors_files === 0) {
			$total_errors_files.hide();
			$show_only_errors_container.hide();
			$show_only_errors.prop("checked", true);
			$show_only_errors.trigger("click");
		}
		else {
			let msg_error = resp.total_errors_files === 1 ? _T("{0} Error") : _T("{0} Errors");
			$total_errors_files.text(FORMAT(msg_error, resp.total_errors_files));
			$total_errors_files.show();
			$show_only_errors_container.show();
		}

		return resp;
	};

	MaintenanceMng.prototype.get_visibility = function()
	{
		return this.m_visibility;
	};

	MaintenanceMng.prototype.set_events = function()
	{
		const self = this;

		let isAllowsMultipleFiles = this.allowsMultipleFiles();

		$("#" + this.prefix_id + "show_only_errors")
			.off("click")
			.on("click", function() {
				let only_errors = $(this).prop("checked");
				self.m_grdFiles.show_rows_only_with_errors(only_errors);
			}
		);

		let uploaded_files;
		if (this.is_new() && isAllowsMultipleFiles) {
			uploaded_files = [];
			this.m_grdFiles.__uploaded_files = uploaded_files;
		}

		this.m_uploader.onBatchFilesUploadStart = function() {
			self.show_waiting(_T("Loading, wait moment please"));
			if (self.is_new()) {
				self.m_uploadedFilesList = self.m_uploadedFilesList || {};
				self.m_errorsFilesList = self.m_errorsFilesList || {};
				self.__toggle_files_container();
 			}
		};

		this.m_uploader.onBatchFilesUploadEnd = function() {
			self.hide_waiting();
			if (self.is_new() && isAllowsMultipleFiles) {
				self.__toggle_files_container();
				// Agrego a la grilla, el array de archivos subidos en una propiedad para que puedan ser procesados en el popup donde se asignan los valores en lote por defecto a todos los rows de la grilla.
				//   Funciones: 
				//     __open_popup_grid_default_files_values
				//      __add_grid_files_values
				//
				self.m_grdFiles.__uploaded_files = uploaded_files;
				if (self.m_from === "normal") {
					self.m_grdFiles.show_popup();
					self.update_totals();
				}
				else {
					self.__add_grid_files_values(false, function() {
						self.update_totals();
					});
				}
			}
		};

		let total_added_files = 0;
		this.m_uploader.onFileAdded = function(fileData, callback)
		{
			const readonly_controls = function() {
				if (!self.__is_internal()) {
					return;
				}
				let map = {
					AgentVisible          : $("#" + self.prefix_id + "file_agent_visible"),
					AccountMessagesVisible: $("#" + self.prefix_id + "file_account_visible"),
					FlowDesignerVisible   : $("#" + self.prefix_id + "file_flow_visible")
				};
				let visibility_list = self.get_visibility() || {};
				for (let visibility_name in map) {
					if (map.hasOwnProperty(visibility_name)) {
						let $ctrl = map[visibility_name];
						self.readonly_control($ctrl, true);
						if (visibility_list[visibility_name] === true) {
							$ctrl.prop("checked", true);
						}
						else {
							$ctrl.prop("checked", false);
						}
					}
				}
			};

			if (self.is_new()) {
				let file_name = self.m_uploader.getFileName(fileData.file);
				if (!self.allowsMultipleFiles()) {
					$("#" + self.prefix_id + "file_name").val(file_name);
					$("#" + self.prefix_id + "file_description").val(file_name);
					// procedo a dejar checked la visibilidad y tambien dejarla readonly
					readonly_controls();
				}
				else {
					total_added_files++;
					// agrego los rows en el array que luego seran procesados para agregar en la grilla, una vez que se cierre el popup de valores por defecto.
					//  funcion: __add_grid_files_values
					let row = {
						Order                 : total_added_files,
						Type                  : self.m_new_file_type,
						Name                  : file_name,
						Description           : file_name,
						RepositoryId          : fileData.id,
						Language              : fileData.Language || "none",
						AgentVisible          : fileData.AgentVisible || "0",
						AccountMessagesVisible: fileData.AccountMessagesVisible || "0",
						FlowDesignerVisible   : fileData.FlowDesignerVisible || "0",
						blobContent           : fileData.blobContent,
						AddedFrom             : "normal"
					};

					self.m_uploadedFilesList[row.RepositoryId] = row;
					uploaded_files.push(row);
				}
			}
			else {
				// procedo a dejar checked la visibilidad y tambien dejarla readonly
				readonly_controls();
			}

			callback();
		};

		if (this.__is_internal()) {
			$("#" + this.prefix_id + "save")
				.off("click")
				.on("click", function() {
					self.internal_save();
				}
			);
			$("#" + this.prefix_id + "cancel")
				.off("click")
				.on("click", function() {
					self.close();
				}
			);
		}
	};

	MaintenanceMng.prototype.dispose = function()
	{
		$("#" + this.container_id).html("");
	};

	MaintenanceMng.prototype.close = function()
	{
		this.callbacks.on_close();
		this.dispose();
	};

	MaintenanceMng.prototype.internal_save = function()
	{
		const self = this;

		var callback_save = function(success) {
			if (!success) {
				return;
			}
			let new_file = self.get();
			new_file.isNew = true;
		//	const resp = self.make_save_waiting_msg(new_file);
		//	self.show_waiting(resp.waiting_message);
//			var api = this.dashboard.get_api();

			let data = {};
			data[new_file.Id] = new_file;
			let uri = inConcert.app.getBaseDomain() + "/inconcert/apps/administration/shared_files/save/";

			inConcert.invoke({
				url         : uri,
				type        : "POST",
				data        : JSON.stringify(data),
				dataType    : "json",
				contentType : "application/json",
				success: function(/*response*/) {
					self.callbacks.on_save(new_file);
					self.close();
				},
				error: function(_, response) {
					let reason = response.reason|| response.result;
					if (!reason && $.isArray(response)) {
						let first_error = response[0];
						reason = (first_error || {}).reason;
					}
					let msg_err = _T(reason || "Server error.");
					self.show_validation_error(null, msg_err);
				}
			});
		};

		this.internal_save_validation(callback_save);
	};

	MaintenanceMng.prototype.internal_save_validation = function(callback_save)
	{
		this.hide_status_message();
		let success = this.validate();
		if (!success) {
			callback_save(false);
			return;
		}
		callback_save(true);
	};

	MaintenanceMng.prototype.get = function()
	{
		const self = this;

		let current_file_number = 0;
		var get_dom_ctrl_id = function(suffix) {
			if (current_file_number === 0) {
				suffix = "file_" + suffix;
			}
			else {
				suffix = "multi_files_" + suffix + current_file_number.toString();
			}
			let dom_ctrl_id = self.prefix_id + suffix;
			return dom_ctrl_id;
		};

		var make_file_info = function(fileData, shared_file_id) {
			let file = {
				RepositoryId: fileData.id,
				ContentType : self.m_uploader.getFileType(fileData.file),
				Size        : self.m_uploader.getFileSize(fileData.file),
//				LastUse     : self.currentFile && self.currentFile.LastUse || null,
				is_new      : self.is_new()
			};

			file.Id = shared_file_id;
			file.Name = file.Id;

			if (self.mapUpdateFiles) {
				let mapUdateFile = self.mapUpdateFiles[shared_file_id.toLowerCase()];
				if (mapUdateFile) {
					file.LastUse = mapUdateFile.LastUse || null;
				}
			}

			return file;
		};

		if (this.allowsMultipleFiles()) {
			let files = [];
			let grid_files = this.m_grdFiles.get_items(true);
			let files_data = this.m_uploader.getFiles();

			for (let i = 0; i < files_data.length; i++) {
				current_file_number++;
				let fileData = files_data[i];
				if (!fileData.file) {
					continue;
				}
				let repository_id = (fileData.id || "").toLowerCase();
				let grid_file = grid_files[repository_id];
				if (!grid_file) {
					continue;
				}

				let shared_file_id = grid_file.Name;
				let file = make_file_info(fileData, shared_file_id);
				file.Description = grid_file.Description;
				file.Type = grid_file.Type;
				file.Language = grid_file.Language;
				file.AgentVisible = grid_file.AgentVisible || "0";
				file.AccountMessagesVisible = grid_file.AccountMessagesVisible || "0";
				file.FlowDesignerVisible = grid_file.FlowDesignerVisible || "0";

				files.push(file);
			}
			return files;
		}

		let fileData = this.m_uploader.getFile();
		let shared_file_id = $("#" + get_dom_ctrl_id("name")).val();
		let file = make_file_info(fileData, shared_file_id);
		file.Description = $("#" + get_dom_ctrl_id("description")).val();
		file.Type = $("#" + get_dom_ctrl_id("type")).val();
		file.Language = $("#" + get_dom_ctrl_id("language")).val();
		file.AgentVisible = $("#" + get_dom_ctrl_id("agent_visible")).is(":checked")? "1" : "0";
		file.AccountMessagesVisible = $("#" + get_dom_ctrl_id("account_visible")).is(":checked")? "1" : "0";
		file.FlowDesignerVisible = $("#" + get_dom_ctrl_id("flow_visible")).is(":checked")? "1" : "0";

		return file;
	};

	/*
		A partir de un $control retorna su par $label
	*/
	MaintenanceMng.prototype.__$get_label = function($control)
	{
		const control_id = $control.attr("id");
		const $parent = $control.parent().parent();
		let $label = $parent.find("div[for='" + control_id + "']");
		if ($label.length === 0) {
			$label = $parent.find("label[for='" + control_id + "']");
		}
		return $label;
	};

	MaintenanceMng.prototype.readonly_control = function($control, readonly, is_required)
	{
		is_required = is_required === undefined ? false : is_required;
		if (typeof $control !== "object") {
			return;
		}
		const $label = this.__$get_label( $control );
		if (readonly) {
			$control.prop("disabled", true).addClass("readonly");
			$label.removeClass("required").addClass("formLabel-ds readonly").prop("disabled", true);
		}
		else {
			$control.prop("disabled", false).removeClass("readonly");
			$label.removeClass("formLabel-ds required readonly").prop("disabled", false);
			// if (is_required && !this.is_readonly()) {
			// 	$label.addClass("required");
			// }
		}
	};

	MaintenanceMng.prototype.set = function(success, file)
	{
		if (!success) {
			return;
		}

		this.__toggle_files_container();

		this.mapUpdateFiles = {[file.Id.toLowerCase()]: file};
		$("#" + this.prefix_id + "file_name").val(file.Name);
		$("#" + this.prefix_id + "file_description").val(file.Description);
		$("#" + this.prefix_id + "file_type").val(file.Type);
		$("#" + this.prefix_id + "file_language").val(file.Language || "none");
		$("#" + this.prefix_id + "file_agent_visible").prop("checked", file.AgentVisible === "1");
		$("#" + this.prefix_id + "file_account_visible").prop("checked", file.AccountMessagesVisible === "1");
		$("#" + this.prefix_id + "file_flow_visible").prop("checked", file.FlowDesignerVisible === "1");

		this.readonly_control($("#" + this.prefix_id + "file_name"), true);
		this.readonly_control($("#" + this.prefix_id + "file_type"), true);

		let fileData = {
			id     : file.RepositoryId,
			Section: file.Section,
			Part   : file.Part,
			file: this.m_uploader.makeManualFile({
				name: file.Name,
				type: file.ContentType,
				size: file.Size
			}),
			uploaded: true
		};

		this.m_uploader.edit(fileData, file.Type);
	};

	MaintenanceMng.prototype.empty_controls = function()
	{
		$("#" + this.prefix_id + "file_name").val("").focus();
		$("#" + this.prefix_id + "file_description").val("");
		$("#" + this.prefix_id + "file_type").val(sharedFilesTypes[0].id);
		$("#" + this.prefix_id + "file_language").val("none");
		$("#" + this.prefix_id + "file_agent_visible").prop("checked", false);
		$("#" + this.prefix_id + "file_account_visible").prop("checked", false);
		$("#" + this.prefix_id + "file_flow_visible").prop("checked", false);
	};

	MaintenanceMng.prototype.on_close = function(action)
	{
		if (action === "cancel") {
			this.m_uploader.removeNewFiles();
		}
		else {
			MaintenanceMng.prototype.reloadSizeLimit();
		}
		this.m_from = null;
	};

	MaintenanceMng.prototype.__show_grid_rows_errors = function(errors)
	{
		this.m_errorsFilesList = {};
		for (let i = 0; i < errors.length; i++) {
			let err = errors[i];
			if (err.fileData) {
				let row_info = {
					row_id : err.fileData.RepositoryId,
					msg_err: err.message
				};
				this.m_grdFiles.mark_row_error(row_info);
				this.m_errorsFilesList[err.fileData.RepositoryId] = err.fileData;
			}
		}
		let msg_error = _T("Files could not be uploaded.");
		this.show_status_message(msg_error, "Error", 3000);
	};

	MaintenanceMng.prototype.__hide_all_mark_rows_errors = function()
	{
		this.m_grdFiles.hide_all_mark_rows_errors();
	};

	/*
		Errores sever side.
	*/
	MaintenanceMng.prototype.err_save_messages = function(errors)
	{
		const get_message_error = function(err) {
			let message;
			let error_type = err.result.toUpperCase();

			if (error_type === "NAME_ALREADY_EXISTS") {
				message = _T("Already exists another file with same name.");
			}
			else if (error_type === "HAS_FILE_NOT_SPACE") {
				message = _T("File by itself exceeds the available space.");
			}
			else if (error_type === "HAS_BATCH_FILES_NOT_SPACE") {
				message = _T("File exceeds available space.");
			}
			else if (error_type === "MUST_HAVE_A_DESCRIPTION") {
				message = _T("Description taken, choose another one.");
			}
			else if (error_type === "MUST_HAVE_A_TYPE") {
				message = _T("File must have a file type.");
			}
			else if (error_type === "MUST_HAVE_A_NAME") {
				message = _T("Name already taken, choose another one.");
			}
			else if (error_type === "MUST_HAVE_A_SIZE") {
				message = _T("File doesnt have a size.");
			}
			else if (error_type === "MUST_HAVE_A_CONTENT_TYPE") {
				message = _T("File must have a content type.");
			}
			else if (error_type === "MUST_SELECT_AT_LEAST_ONE_VISIBILITY_OPTION") {
				message = _T("You must select at least one visibility option.");
			}
			else if (error_type === "MUST_HAVE_A_REPOSITORY_ID") {
				message = _T("File must have a repository id.");
			}
			else if (error_type === "THERE ANOTHER PROCESS BATCH IN PROGRESS") {
				message = _T("Unable to complete action. There is another batch process in progress.");
			}
			else if (error_type === "UPLOAD EXCEEDS THE LIMIT OF FILES SUPPORTED") {
				message = FORMAT(_T("The upload must not exceed {0} files. The current upload contains {1} files."), err.maximumNumberRowsInImport, err.fileRows);
			}
			else {
				message = _T("Server error.");
			}
			return message;
		};

		if ($.isArray(errors)) {
			for (let i = 0; i < errors.length; i++) {
				let err = errors[i];
				err.message = get_message_error(err);
			}
			this.__show_grid_rows_errors(errors);
		}
		else if (typeof errors === "object") {
			let message = get_message_error(errors);
			let error_type = errors.result.toUpperCase();
			if (error_type === "NAME_DOES_NOT_EXIST" || error_type === "NAME_ALREADY_EXISTS") {
				$("#" + this.prefix_id + "file_name").focus();
			}
			else if (error_type === "MUST_HAVE_A_DESCRIPTION") {
				$("#" + this.prefix_id + "file_description").focus();
			}
			else if (error_type === "MUST_HAVE_A_TYPE") {
				$("#" + this.prefix_id + "file_type").focus();
			}
			if (message) {
				errors = message;
			}
			this.show_status_message(message, "Error", 3000);
		}
	};

	MaintenanceMng.prototype.get_validation_def = function()
	{
		if (this.allowsMultipleFiles()) {
			return [];
		}
		let definitions = [
			{
				control_id: this.prefix_id + "file_name",
				messages: {
					required: _T("Please enter the file name."),
					id      : _T("Only letters, numbers, dashes and dots are allowed in the file name.")
				}
			},
			{
				control_id: this.prefix_id + "file_description",
				messages: {
					required: _T("Please enter the file description.")
				}
			}
		];
		return definitions;
	};

	MaintenanceMng.prototype.show_validation_error = function($field, message, timeout_ms, scrolling, show_custom_button, $custom_message_container)
	{
		timeout_ms = timeout_ms || 5000;
		this.owner.validation.show_validation_error($field, message, timeout_ms, scrolling, show_custom_button, $custom_message_container);
	};

	MaintenanceMng.prototype.validate_multiupload = function()
	{
		let total_rows = this.m_grdFiles.count_rows();
		if (total_rows === 0) {
			this.show_validation_error(null, _T("You must upload at least one file."));
			return false;
		}
		if (this.m_from === "csv") {
			let rows = this.m_grdFiles.get_items();
			let has_uploaded_at_least_one_file = false;
			for (let i = 0; i < rows.length; i++) {
				let row = rows[i];
				if (row.AddedFrom === "csv" && row.RepositoryId) {
					has_uploaded_at_least_one_file = true;
					break;
				}
			}
			if (!has_uploaded_at_least_one_file) {
				this.show_validation_error(null, _T("You must upload at least one file that you have defined in the Metadata."));
				return false;
			}
		}
		// is ok
		return true;
	};

	MaintenanceMng.prototype.validate = function()
	{
		if (this.allowsMultipleFiles()) {
			this.__hide_all_mark_rows_errors();
			let is_ok = this.validate_multiupload();
			return is_ok;
		}
		let fileData = this.m_uploader.getFile();
		if (!fileData) {
			this.show_validation_error(null, _T("Must upload a file."));
			return false;
		}
		let fileExtension = this.m_uploader.getFileExtension(fileData.file);
		const $file_name = $("#" + this.prefix_id + "file_name");
		let nameExtension = inConcert.Utils.getFileExtension($file_name.val());
		if (fileExtension.toLowerCase() !== nameExtension.toLowerCase()) {
			this.show_validation_error($file_name, _T("File name must include uploaded file extension."));
			return false;
		}
		return true;
	};

	// MaintenanceMng.prototype.readonly_control = function($control, readonly, is_required)
	// {
	// 	this.owner.readonly_control($control, readonly, is_required);
	// };

	MaintenanceMng.prototype.is_new = function()
	{
		return typeof this.owner.is_new === "function" ? this.owner.is_new() : this.owner.is_new;
	};

	MaintenanceMng.prototype.show_waiting = function(msg)
	{
		this.owner.show_waiting(msg);
	};

	MaintenanceMng.prototype.hide_waiting = function()
	{
		this.owner.hide_waiting();
	};

	MaintenanceMng.prototype.ready = function(new_file_type, info)
	{
		const self = this;

		this.m_new_file_type = null;
		if (!this.is_new()) {
			const $name = $("#" + this.prefix_id + "file_name");
			this.readonly_control($name, true);
			$("#" + this.prefix_id + "file_description").focus();
		}
		else {
			this.m_uploadedFilesList = {};
			this.m_errorsFilesList = {};
			this.m_new_file_type = new_file_type;
			let file_type_def = inConcert.FileUploaderTypes[new_file_type];
			this.m_uploader.changeType(file_type_def);
			this.m_from = "normal";
			this.m_telephonyAudio =
				info?.telephonyAudio || this.m_search_files_types?.telephonyAudio || false;
			this.m_uploader.telephonyAudio = this.m_telephonyAudio;
			if (!this.allowsMultipleFiles()) {
				return;
			}
			this.m_grdFiles.hide_disclaimer();
			if (info) {
				if ((info.from || "").toLowerCase() === "csv") {
					this.m_from = "csv";
					this.m_csv_files = info.files;
					this.m_grdFiles.__uploaded_files = this.m_csv_files;
					this.__add_grid_files_values(false, function() {
						self.__toggle_files_container();
						let msg = _T("File upload using CSV file metadata. Drag or drop above to upload files. For the system to synchronize them, the uploaded files must have the same name that has been entered in the Metadata.");
						self.m_grdFiles.show_disclaimer(msg);
					});
				}
			}
		}
	};

	MaintenanceMng.prototype.popup_show = function(params)
	{
		switch(params && params.type || "") {
			case "import_files":
				$("#" + this.prefix_popup_import_id + "_name").focus();
			break;
		}
	};

	MaintenanceMng.prototype.__popup_importation_html = function()
	{
		return "".concat(
			"<div id='", this.prefix_popup_import_id, "' class='internalPopUpContainer importSharedFilesPopup'>",
				"<div class='internalPopUpOverlay'></div>",
				"<div class='internalPopUp'>",
					"<div class='internalPopUpTitle'>",
						"<label>", _T("Import File"), "</label>",
					"</div>",
					"<div class='internalPopUpMiddle'>",
						"<div id='", this.prefix_popup_import_id, "_message' class='genericFormMessagesContainer messageError' style='display:none;'></div>",
						"<div class='row'>",
							"<div id='", this.prefix_popup_import_id, "_container' class='sharedFileUploadContainer'>",
								"<div id='", this.prefix_popup_import_id, "_control' class='sharedFileUploadControl'>",
									_T("Drag and drop or select one or more files"),
								"</div>",
							"</div>",
						"</div>",
						"<hr />",
						"<div form_control class='row pt-2' hidden>",
							"<div class='col-6'>",
								"<div class='form-group'>",
									"<label for='", this.prefix_popup_import_id, "_name' class='required'>", _T("Name"), _T(":"), "</label>",
									"<input id='", this.prefix_popup_import_id, "_name' autocomplete='new-user' class='form-control' type='text' />",
									"<i id='", this.prefix_popup_import_id, "_name_validation' class='formFieldValidation formFieldError' hidden></i>",
								"</div>",
							"</div>",
						"</div>",
						"<div form_control class='row' hidden>",
							"<div class='col-6'>",
								"<div class='form-group'>",
									"<label for='", this.prefix_popup_import_id, "_description' class='required'>", _T("Description"), _T(":"), "</label>",
									"<input id='", this.prefix_popup_import_id, "_description' autocomplete='new-user' class='form-control' type='text' />",
									"<i id='", this.prefix_popup_import_id, "_description_validation' class='formFieldValidation formFieldError' hidden></i>",
								"</div>",
							"</div>",
						"</div>",
						"<div form_control class='row' hidden>",
							"<div class='col-6'>",
								"<div class='form-group'>",
									"<label for='", this.prefix_popup_import_id, "_type' class=''>", _T("Type"), _T(":"), "</label>",
									"<select id='", this.prefix_popup_import_id, "_type' class='form-control'></select>",
								"</div>",
							"</div>",
						"</div>",
						"<div form_control class='row' hidden>",
							"<div class='col-12'>",
								"<div class='form-group'>",
									"<div class='form-check mr-3'>",
										"<label for='", this.prefix_popup_import_id, "_agent_visible' class='form-check-label form-check-label-title'>", 
										"<input id='", this.prefix_popup_import_id, "_agent_visible' class='form-check-input' type='checkbox'>",
										_T("Agent Visible"),
										"<i id='", this.prefix_popup_import_id, "_agent_visible_validation' class='formFieldValidation formFieldError' hidden></i>",
										 "</label>",
									"</div>",
								"</div>",
							"</div>",
						"</div>",
					"</div>",
					"<div class='internalPopUpButtons internal'>",
						"<input id='", this.prefix_popup_import_id, "_import' class='pageButtonGeneric buttonColorBlack' type='button' value='", _T("Import"), "' />",
						"<input id='", this.prefix_popup_import_id, "_cancel' class='pageButtonGeneric buttonColorGray' type='button' value='", _T("Cancel"), "' />",
					"</div>",
				"</div>",
			"</div>"
		);
	};

	MaintenanceMng.prototype.__popup_importation_csv_html = function()
	{
		return "".concat(
			"<div id='", this.prefix_popup_csv_id, "file_csv_panel' class='internalPopUpContainer formPopUp'>",
				"<div class='internalPopUpOverlay'></div>",
				"<div class='internalPopUp'>",
					"<div id='", this.prefix_popup_csv_id, "file_csv_panel_waiting' class='internalWaitingContainer' style='display:none;'>",
						"<div waiting_message='true' class='internalWaitingMessage'></div>",
						"<div class='internalWaitingAnimation'></div>",
						"<div class='internalWaitingOverlay'></div>",
					"</div>",
					"<div class='internalPopUpTitle'>",
						"<span popup_title_text=''>" , _T("Upload From CSV File"), "</span>",
						"<button id='", this.prefix_popup_csv_id, "file_csv_popup_close' popup_close='' class='internalPopUpCloseButton'><i class='far fa-times'></i></button>",
					"</div>",
					"<div class='internalPopUpMiddle pageFormContainer'>",
						"<div class='card no-card'>",
							"<div class='card-header'>",
								"<h5>",
									_T("File Upload Configuration"),
								"</h5>",
							"</div>",
							"<div class='card-block'>",
								"<div id='", this.prefix_popup_csv_id, "file_csv_panel_popup_message' class='genericFormMessagesContainer messageError' hidden>",
									"<label class='messageText'></label>",
								"</div>",
								"<div class='row'>",
									"<div class='col-12'>",
										"<div class='inlineMessagesContainer messageInfo'>/", FORMAT(_T("Select CSV file to upload with a maximum of {0} rows."), this.m_maximum_number_of_files_in_import), "</div>",
									"</div>",
								"</div>",
								"<div class='row'>",
									"<div class='col-3'>",
										"<div class='form-group'>",
											"<label for='", this.prefix_popup_csv_id, "file_csv_panel_file_type' class='required'>", _T("Files Type to Upload"), _T(":"), "</label>",
											"<select id='", this.prefix_popup_csv_id, "file_csv_panel_file_type' class='form-control'></select>",
										"</div>",
									"</div>",
									"<div class='col-6 colButtonUpload'>",										
										"<div id='", this.prefix_popup_csv_id, "file_csv_panel_upload' class='fileSelectionSection'></div>",
									"</div>",
								"</div>",
								// Aca se genera de forma dinamica desde JAVASCRIPT el DOM correspondiente al Upload de archivos.
								"<div id='", this.prefix_popup_csv_id, "file_csv_panel_csv_header_columns' class='row' hidden>",
									"<div class='col-12'>",
										"<div class='form-group'>",
											"<label class=''>", _T("Allowed CSV Columns"), _T(":"), "</label>",
											"<div id='", this.prefix_popup_csv_id, "file_csv_panel_allowed_csv_columns' class='importationHeaders'></div>",
										"</div>",
									"</div>",
								"</div>",
							"</div>",
						"</div>",
					"</div>",
					"<div class='internalPopUpButtons d-flex justify-content-end hiddeAll'>",
						"<button id='", this.prefix_popup_csv_id, "file_csv_panel_back' class='pageButtonGeneric buttonColorWhite'>",
							_T("Close"),
						"</button>",
					"</div>",
				"</div>",
			"</div>"
		);
	};

	MaintenanceMng.prototype.popup_html = function( params )
	{
		let type = params && params.type || "";
		if (type === "import_files") {
			return this.__popup_importation_html();
		}
		else if (type === "import_csv_file") {
			return this.__popup_importation_csv_html();
		}
		return "";
	};

	MaintenanceMng.prototype.popup_hide = function(has_been_imported)
	{
		this.owner.popup_hide(has_been_imported);
	};

	MaintenanceMng.prototype.popup_set_events = function(params)
	{
		const self = this;
		switch(params && params.type || "") {
			case "import_files":
				this.set_import_uploader();
				const $cbo = $("#" + this.prefix_popup_import_id + "_type");
				this.fill_combo($cbo, false, sharedFilesTypes, "id", "label");
				this.readonly_control($cbo, true);

				$("#" + this.prefix_popup_import_id + "_cancel")
					.off("click")
					.on("click", function() {
						self.popup_hide();
					});
				$("#" + this.prefix_popup_import_id + "_import")
					.off("click")
					.on("click", function() {
						self.doImport();
					});
			break;
			case "import_csv_file":
				const jQuerySelector = FORMAT("#{0}file_csv_panel_back, #{0}file_csv_popup_close", this.prefix_popup_csv_id);
				$(jQuerySelector)
					.off("click")
					.on("click", function() {
						self.popup_hide();
					});
			break;

		}
	};

	MaintenanceMng.prototype.popup_on_render = function(params)
	{
		const type = params && params.type || "";
		if (type === "import_csv_file") {
			const $cbo_type = $("#" + this.prefix_popup_csv_id + "file_csv_panel_file_type");
			this.fill_combo($cbo_type, false, sharedFilesTypes, "id", "label");
			this.__load_allowed_csv_columns();
			this.__initialize_upload_csv_ctrl(params);
		}
		//else if (type === "import_files") {
		//	// TODO: Hacer algo si es necesario !!!
		//}
	};

	MaintenanceMng.prototype.__load_allowed_csv_columns = function()
	{
		const self = this;
		if (this.m_allowed_csv_columns) {
			this.__make_allowed_columns();
			return;
		}
		this.__show_waiting_csv_popup();
		const importation_api = inConcert.app.getImportationAPI();
		importation_api.getAllowedSharedFilesHeader({},
			function(success, response) {
				self.__hide_waiting_csv_popup();
				if (success) {
					self.m_allowed_csv_columns = response;
					self.__make_allowed_columns();
					return;
				}
				self.__show_csv_popup_error(_T("Unable to retrieve information from CSV allowed columns ..."));
			}
		);
	};

	MaintenanceMng.prototype.__show_waiting_csv_popup = function(msg)
	{
		const $waiting = $("#" + this.prefix_popup_csv_id + "file_csv_panel_waiting");
		$waiting.show();
		$waiting.find("[waiting_message]").text(msg || _T("Loading..."));
	};

	MaintenanceMng.prototype.__hide_waiting_csv_popup = function()
	{
		const $waiting = $("#" + this.prefix_popup_csv_id + "file_csv_panel_waiting");
		$waiting.hide();
	};

	MaintenanceMng.prototype.__make_allowed_columns = function()
	{
		const $header = $("#" + this.prefix_popup_csv_id + "file_csv_panel_allowed_csv_columns");
		$header.html("");
		$.each(this.m_allowed_csv_columns || [], function(_, column) {
			const $column = $("<div class='importationHeaderItem badge'>" + column.CSVFieldName + "</div>");
			$column.attr("col_name", column.CSVFieldName.toUpperCase());
			if (column.required) {
				$column.addClass("required");
			}
			$header.append($column);
		});
		$("#" + this.prefix_popup_csv_id + "file_csv_panel_csv_header_columns").show();
	};

	MaintenanceMng.prototype.__initialize_upload_csv_ctrl = function(params)
	{
		const self = this;

		this.m_upload_instance = new inConcert.RepositoryUpload({
			auto_initialize: true,
repository_url: "/inconcert/apps/administration/imports",

			container_id: this.prefix_popup_csv_id +"file_csv_panel_upload",
			must_validate_max_size: false,
			availables_extensions: {
				"csv": true
			},
			show_status_message_fn: function(message, type, time_ms) {
				const $custom_message_container = $("#" + self.prefix_popup_csv_id + "file_csv_panel_popup_message");
				self.show_status_message(message, type, time_ms, undefined, undefined, $custom_message_container);
			},
			on_uploading_fn: function(/*success, data*/) {
				//self.on_uploading_fn(success, data);
			},
			on_uploaded_fn: function(success, data, callback) {
				self.__show_waiting_csv_popup();
				self.on_uploaded_csv_file(success, data, function(success, response) {
					callback();
					if (success) {
						const $back = $("#" + self.prefix_popup_csv_id + "file_csv_panel_back");
						$back.trigger("click");
						let file_type = $("#" + self.prefix_popup_csv_id + "file_csv_panel_file_type").val(); // image, video, audio, files
						for (let i = 0; i < (response.jsonValues || []).length; i++) {
							response.jsonValues[i].Type = response.jsonValues[i].Type || file_type;
							response.jsonValues[i].AddedFrom = "csv";
						}
						params.callbacks.uploaded(success, file_type, response.jsonValues);
					}
					self.__hide_waiting_csv_popup();
				});
			},
			can_upload_file_fn: function(callback = $.noop) {
				callback(true);
			}
		});
		this.m_current_upload_info = null;
	};

	MaintenanceMng.prototype.on_uploaded_csv_file = function(success, file_info, callback = $.noop)
	{
		const self = this;

		if (!success) {
			this.m_current_upload_info = null;
			callback();
		}
		else {
			this.m_current_upload_info = file_info;
			const importation_api = inConcert.app.getImportationAPI();

			importation_api.fileSharingLoadUploadedCSVtoJSON(file_info,
				function(success, response)	{
					self.m_csv_header = response.csvHeader;
					self.__mark_csv_columns(self.m_csv_header, "SUCCESS");
					if (!success) {
						self.__show_csv_popup_error(response.info);
					}
					// else {
					// 	//self.m_jsonValues = response.jsonValues;
					// }
					callback(success, response);
				}
			);
		}
	};

	MaintenanceMng.prototype.__show_csv_popup_error = function(current_err)
	{
		let msg;
		if (!current_err) {
			msg = _T("Unable to Upload CSV File.");
		}
		else if (typeof current_err === "string") {
			msg = current_err;
		}
		else {
			let code_error = parseInt(current_err.code, 10);
			switch(code_error) {
				// Client side errors
				case -1:
					msg = _T("CSV File has not been uploaded");
					break;
	
				// Server side errors
				case -100:
					msg = _T("Invalid request");
					break;
	
				case -101:
					msg = _T("CSV File does not exist");
					break;
	
				case -102:
					var missing_fields = current_err.missingRequiredFields || [];
					var total_missing_fields = missing_fields.length;
					msg = total_missing_fields === 1 ?
						_T("Missing {0} required field in CSV File") :
						_T("Missing {0} required fields in CSV File");
					msg = FORMAT(msg, total_missing_fields);
					this.__mark_csv_columns(missing_fields, "ERROR");
					break;
	
				case -103:
					msg = _T("Unable to Upload CSV File. There another uploaded file in progress, try later.");
					break;
	
				case -104:
					msg = _T("The uploaded file must not exceed {0} rows. File contains {1} rows.");
					msg = FORMAT(msg, current_err.maximumNumberRowsInImport, current_err.fileRows);
					break;
	
				case -105:
					msg = _T("Invalid CSV File");
					break;
	
				default:
					msg = _T("Unable to Upload CSV File.");
			}
		}

		const timemout_ms = 5000;
		const $custom_message_container = $("#" + this.prefix_popup_csv_id + "file_csv_panel_popup_message");
		this.show_status_message(msg, "Error", timemout_ms, undefined, undefined, $custom_message_container);
	};

	MaintenanceMng.prototype.__mark_csv_columns = function(list, type)
	{
		const $header = $("#" + this.prefix_popup_csv_id + "file_csv_panel_allowed_csv_columns");
		$.each(list || [], function(_, col_name) {
			const $column = $header.find("[col_name='" + col_name.toUpperCase() + "']");
			if ($column.length > 0) {
				$column.removeClass("error").removeClass("success");
				if (type === "SUCCESS") {
					$column.addClass("success");
				}
				else {
					$column.addClass("error");
				}
			}
		});
	};

	MaintenanceMng.prototype.set_import_uploader = function() {
		const self = this;
		if (this.m_uploader) {
			this.m_uploader.destroy();
			this.m_uploader = null;
		}
		this.m_uploader = new inConcert.FileUploaderControl({
			uploader : {
				$control : $("#" + this.prefix_popup_import_id + "_control")
			},
			dragger : {
				$control : $("#" + this.prefix_popup_import_id + "_control")
			},
			preview : {
				$control : $("#" + this.prefix_popup_import_id + "_container"),
				canRemove : false,
				uploadStatus : false
			},
			type : inConcert.FileUploaderTypes.json,
			uploadOnAdd : false,
			sizeLimitMB: 0.1
//			filesLimit  : this.m_maximum_number_of_files_in_import
		});

		this.m_uploader.onFileAddError = function(_, result, callback) {
			self.popup_error("import_files", result.reason);
			callback();
		};

		this.m_uploader.onFileAdded = function(fileData, callback) {
			self.m_uploader.getFileContent(fileData, function(result) {
				if (!self.m_uploader.__isOk(result) || !result.content) {
					self.popup_error("import_files", _T("Invalid file."));
					return callback();
				}
				self.popup_error("import_files");
				let fileContent = result.content;
				$("#" + self.prefix_popup_import_id + "_name").val(fileContent.Name);
				$("#" + self.prefix_popup_import_id + "_description").val(fileContent.Description);
				$("#" + self.prefix_popup_import_id + "_type").val(fileContent.Type);
				$("#" + self.prefix_popup_import_id + "_agent_visible").prop("checked", fileContent.AgentVisible === "1");
				$("#" + self.prefix_popup_import_id + "").find("[form_control]").show();
				callback();
			});
		};

		this.m_uploader.__loadFilePreviewImage = function(fileData, callback) {
			self.m_uploader.__loadFileContent(fileData, callback);
		};

		this.m_uploader.__hasFilePreviewImage = function(fileData) {
			return fileData.content && fileData.content.Type === "image";
		};

		this.m_uploader.__getFilePreviewType = function(fileData) {
			return fileData.content && fileData.content.Type || "file";
		};

		this.m_uploader.__getFilePreviewImageUrl = function(fileData) {
			return "data:" + fileData.content.ContentType + ";base64," + fileData.content.content;
		};
	};

	MaintenanceMng.prototype.popup_error = function(type, error) {
		let $error;
		switch(type) {
			case "import_files":
				$error = $("#" + this.prefix_popup_import_id + "_message");
				break;
		}
		$error.toggle(!!error).text(error || "");
	};

	MaintenanceMng.prototype.doImport = function() {
		const self = this;
		
		let fileData = self.m_uploader.getFile();
		if (!fileData) {
			self.popup_error("import_files", _T("Must upload a file."));
			return;
		}
		let fileContent = fileData.content;
		if (!fileContent || !fileContent.Id || !fileContent.Name || !fileContent.Description || !fileContent.RepositoryId ||
			!fileContent.ContentType || !fileContent.Size || !fileContent.content
		) {
			self.popup_error("import_files", _T("File uploaded is invalid."));
			return;
		}

		let $name = $("#" + this.prefix_popup_import_id + "_name");
		let $description = $("#" + this.prefix_popup_import_id + "_description");
		let name = $name.val();
		let description = $description.val();
		if (!inConcert.Utils.validate($name, "required")) {
			this.popup_error("import_files", _T("Please enter the file name."));
			return;
		}
		if (!inConcert.Utils.validate($name, "id")) {
			this.popup_error("import_files", _T("Only letters, numbers, dashes and dots are allowed in the file name."));
			return;
		}
		if (inConcert.Utils.getFileExtension(name).toLowerCase() !== inConcert.Utils.getFileExtension(fileContent.Name).toLowerCase()) {
			this.popup_error("import_files", _T("File name must include uploaded file extension."));
			return;
		}
		if (!inConcert.Utils.validate($description, "required")) {
			this.popup_error("import_files", _T("Please enter the file description."));
			return;
		}

		let filesAPI = inConcert.app.getSharedFilesAPI();
		let file = {
			Id : name,
			Name: name,
			Description: description,
			Type: fileContent.Type,
			AgentVisible: $("#" + this.prefix_popup_import_id + "_agent_visible").is(":checked")? "1" : "0",
			RepositoryId : inConcert.Utils.generateGuid(),
			ContentType : fileContent.ContentType,
			Size : parseInt(fileContent.Size)
		};

		this.popup_error("import_files");
		$("#" + this.prefix_popup_import_id + "_import").prop("disabled", true);

		let uploadFile = function() {
			let repositoryApi = inConcert.app.getRepositoryAPI();
			let repositoryFile = {
				id : file.RepositoryId,
				name : file.Name,
				type : file.ContentType,
				format: inConcert.Utils.getFileExtension(file.Name)
			};
			repositoryApi.addBase64(repositoryFile, fileContent.content, function(status) {
				if (!status) {
					self.popup_error("import_files", _T("Error importing file."));
					$("#" + self.prefix_popup_import_id + "_import").prop("disabled", false);
					return;
				}
				filesAPI.saveFile(file, function(/*stauts, result*/) {
					if (status) {
						let has_been_imported = true;
						self.popup_hide(has_been_imported);
					}
					else {
						self.popup_error("import_files", _T("Error importing file."));
						$("#" + self.prefix_popup_import_id + "_import").prop("disabled", false);
					}
				});
			});
		};

		let validateExistentFile = function() {
			filesAPI.getFile({ fileId : file.Id }, function(status, existentFile) {
				if (!status || !existentFile) {
					uploadFile();
				}
				else {
					$.inconcertPopup.Show(
						_T("Confirmation"),
						_T("File already exists. Do you want to overwrite it?"),
						$.inconcertPopup.ButtonSets.BS_YESNO,
						function(response) {
							switch(response.toLowerCase()) {
								case "yes":
									uploadFile();
									break;
								default:
									$("#" + self.prefix_popup_import_id + "_import").prop("disabled", false);
									break;
							}
						},
						undefined,
						{
							popup_id        : "conf_inconcertPopup",
							container_class : "confirmationPopUp",
							overlay_class   : "confirmationOverlay"
						}
					);
				}
			});
		};

		filesAPI.validateFile(file, function(status/*, result*/) {
			if (status) {
				return validateExistentFile();
			}
			else {
				self.popup_error("import_files", _T("Error importing file."));
				$("#" + self.prefix_popup_import_id + "_import").prop("disabled", false);
			}
		});
	};

	MaintenanceMng.prototype.deleteFiles = function(files_to_delete, callback = $.noop) {
		const self = this;
		let filesAPI = inConcert.app.getSharedFilesAPI();
		return filesAPI.deleteFiles(files_to_delete, function(status, result) {
			self.reloadSizeLimit(function() {
				callback(status, result);
			});
		});
	};

	MaintenanceMng.prototype.getFileTypeInfo = function(file_type) {
		if (!file_type) {
			return;
		}
		file_type = file_type.toLowerCase();
		for (let i = 0; i < sharedFilesTypes.length; i++) {
			let file_info = sharedFilesTypes[i];
			if (file_info.id.toLowerCase() === file_type) {
				return file_info;
			}
		}
	};

}());
