/*jshint quotmark:false, esversion: 9*/
/*global $, inConcert*/

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {
	
	//"use strict";

	const format = inConcert.Utils.format;

	function TextTemplate(data) {
		this.$header_control = data.header_control;
		this.header_text = data.header_text || "";
		this.example_header_text = data.example_header_text || "";
		this.example_header_text_value = data.example_header_text_value || "";
		if(this.$header_control) {
			this.$header_control.html(this.__makeHeaderControlHtml());
			this.$header_control.parent().toggle(this.$header_control.html() !== "");
		}
		this.content = data.content;
		this.placeholders = data.placeholders || [];
		this.values = data.values || [];
		this.messages = data.messages || {};
		this.$control = data.control;
		this.$control.html(this.__makeControlHtml());
		this.hsm_buttons = data.hsm_buttons || [];
		this.$button_control = data.button_control;
		if(this.$button_control) {
			this.$button_control.html(this.__makeButtonControlHtml());
			this.$button_control.parent().toggle(this.$button_control.html() !== "");
		}
	}
	
	if (!inConcert.TextTemplate) {
		inConcert.TextTemplate = TextTemplate;
	}
	
	TextTemplate.prototype.__bracketsRegex = function() {
		return new RegExp("{{(.+?)}}", "gi");
	};
	
	TextTemplate.prototype.__loadBrackets = function() {
		var regex = this.__bracketsRegex();
		this.brackets = [];
		var result;
		while ((result = regex.exec(this.content)) !== null) {
			var bracket = result[1];
			this.brackets.push(bracket);
		}
	};
	
	TextTemplate.prototype.__makeHeaderControlHtml = function() {
		let self = this;
		let html = self.header_text;
		let regex = self.__bracketsRegex();
		let brackets = self.header_text.match(regex) || [];
		if(brackets.length) {
			//just one placeholder in the header is allowed
			let bracket = brackets[0];
			let key = bracket.replace("{{", "").replace("}}", "").trim();
			let placeholder = self.example_header_text;
			let value = self.example_header_text_value;
			let bracketHtml = "<p contenteditable='true' class='textTemplateItem' match='" + bracket + "' bracket='" + key + 
				"' placeholder='" + placeholder + "' >" + value + "</p>";
			html = html.replace(bracket, bracketHtml);
		}
		else {
			html = "";
			}
		return html;
	};
	
	TextTemplate.prototype.__makeControlHtml = function() {
		var self = this;
		var regex = self.__bracketsRegex();
		var brackets = self.content.match(regex) || [];
		var html = self.content.replaceAll('\n', '<br>');
		$.each(brackets, function(index, bracket) {
			var key = bracket.replace("{{", "").replace("}}", "").trim();
			var placeholder = index < self.placeholders.length? self.placeholders[index] : key;
			var value = index < self.values.length? self.values[index] : "";
			var bracketHtml = "<p contenteditable='true' class='textTemplateItem' match='" + bracket + "' bracket='" + key + 
				"' placeholder='" + placeholder + "' >" + value + "</p>";
			html = html.replace(bracket, bracketHtml);
		});	
		return html;
	};
	
	TextTemplate.prototype.__makeButtonControlHtml = function() {
		var html = "";
		let ctaButton = this.hsm_buttons.find(button => button.type === "call_to_action" && button.urlButtonParam && !button.otp_type);
		if(ctaButton) {
			html = "<p contenteditable='true' class='textTemplateItem' match='{{1}}' bracket='1' placeholder='" + ctaButton.title + "' ></p>";
		}
		return html;
	};
	
	TextTemplate.prototype.__message = function(message) {
		return this.messages[message] || message;
	};
	
	TextTemplate.prototype.__itemError = function($item, bracket, placeholder, message) {
		$item
			.addClass("validationError")
			.attr("title", message);
		return {
			bracket : bracket,
			placeholder : placeholder,
			message : message
		};
	};
	
	TextTemplate.prototype.__getAndValidateHeader = function(data, $item, required, customValidation) {
		var match = $item.attr("match");
		var bracket = $item.attr("bracket");
		var placeholder = $item.attr("placeholder");
		var value = $item.text() || "";
		var errorMessage = (!value && required && this.__message("required")) ||
				(customValidation && customValidation(bracket, placeholder, value));
		if (!errorMessage) {
			data.header_value_param = value;
		}
		else {
			data.error = this.__itemError($item, bracket, placeholder, errorMessage);
		}
	};
	
	TextTemplate.prototype.__getAndValidateItem = function(data, $item, required, customValidation) {
		var match = $item.attr("match");
		var bracket = $item.attr("bracket");
		var placeholder = $item.attr("placeholder");
		var value = $item.text() || "";
		var errorMessage = (!value && required && this.__message("required")) ||
				(customValidation && customValidation(bracket, placeholder, value));
		if (!errorMessage) {
			data.values.push(value);
			data.valuesByBracket[bracket] = value;
			data.valuesByPlaceholder[placeholder] = value;
			data.text = data.text.replace(match, value);
		}
		else {
			data.error = this.__itemError($item, bracket, placeholder, errorMessage);
		}
	};

	TextTemplate.prototype.__getAndValidateButton = function(data, $item, required, customValidation) {
		let ctaButton = data.hsm_buttons.find(button => button.type === "call_to_action" && button.urlButtonParam);
		const ctaButtonIndex = data.hsm_buttons.map(button => button.urlButtonParam && button.type).indexOf('call_to_action');
		var match = $item.attr("match");
		var bracket = $item.attr("bracket");
		var placeholder = $item.attr("placeholder");
		var value = $item.text() || "";
		var errorMessage = (!value && required && this.__message("required")) ||
				(customValidation && customValidation(bracket, placeholder, value));

		//NOTA: Uso esta function aqui en lugar de inConcert.Utils.validate(string, "url") porque la de Utils deja freezed el browser cuando la url contiene "{"
		var isValidURL = function(string) {
			var res = string.match(/(http(s)?:\/\/.)?(www\.)?[-a-zA-Z0-9@:%._\+~#=]{2,256}\.[a-z]{2,6}\b([-a-zA-Z0-9@:%_\+.~#?&//=]*)/g);
			return (res !== null && Array.isArray(res) && res.length > 0 && res[0] === string);
		};
		if (!errorMessage && ctaButtonIndex >= 0) {
			let fullUrl = ctaButton.url.replace("{{1}}", value);
			errorMessage = isValidURL(fullUrl) ? undefined : this.__message("resource");
		}
		
		if (!errorMessage && ctaButtonIndex >= 0) {
			data.hsm_buttons[ctaButtonIndex].payload = value;
			data.hsm_buttons[ctaButtonIndex].urlButtonParam = true;
		}
		else {
			data.error = this.__itemError($item, bracket, placeholder, errorMessage);
		}
	};
	
	TextTemplate.prototype.get = function(required, customValidation) {
		var self = this;
		var data = {
			values : [],
			valuesByBracket : {},
			valuesByPlaceholder : {},
			template_text : self.content,
			text : self.content
		};
		self.cleanErrors();
		if(self.$header_control) {
			self.$header_control.find(".textTemplateItem").each(function() {
				self.__getAndValidateHeader(data, $(this), required, customValidation);
				if (data.error) {
					return false;
				}
			});
		}
		if(!data.error) {
			self.$control.find(".textTemplateItem").each(function() {
				self.__getAndValidateItem(data, $(this), required, customValidation);
				if (data.error) {
					return false;
				}
			});
		}
		if(!data.error) {
			data.hsm_buttons = self.hsm_buttons;	
			if(self.$button_control) {		
				self.$button_control.find(".textTemplateItem").each(function() {
					self.__getAndValidateButton(data, $(this), required, customValidation);
					if (data.error) {
						return false;
					}
				});
			}
		}
		return data;
	};
	
	TextTemplate.prototype.setHeader = function(value) {
		this.$header_control.find(".textTemplateItem").each(function(index) {
			$(this).text(value);
		});
	};
	
	TextTemplate.prototype.set = function(values) {
		this.$control.find(".textTemplateItem").each(function(index) {
			if (index < values.length) {
				$(this).text(values[index]);
			}
		});
	};
	
	TextTemplate.prototype.setByIndex = function(index, value) {
		this.$control.find(".textTemplateItem").eq(index).text(value);
	};
	
	TextTemplate.prototype.setByBracket = function(bracket, value) {
		this.$control.find(format(".textTemplateItem[bracket={0}]", bracket)).text(value);
	};
	
	TextTemplate.prototype.setByPlaceholder = function(bracket, value) {
		this.$control.find(format(".textTemplateItem[placeholder={0}]", bracket)).text(value);
	};
	
	TextTemplate.prototype.setCtaButton = function(value) {
		this.$button_control.find(".textTemplateItem").text(value);
	};
	
	TextTemplate.prototype.cleanErrors = function() {
		this.$control.find(".textTemplateItem")
			.removeClass("validationError")
			.attr("title", "");
	};
	
	TextTemplate.prototype.cleanValues = function() {
		this.$control.find(".textTemplateItem")
			.removeClass("validationError")
			.text("");
	};
	
	TextTemplate.prototype.clean = function() {
		this.cleanErrors();
		this.cleanValues();
	};

}());