/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:true, browser:true, indent:false, maxerr:50, quotmark:double, esversion: 11 */

/* global $, inConcert */


/*
	Esta clase es instanciada y utilizada desde la configuracion de PROVISIONING-VCC-BASIC.
	La misma debe exponer el nombre de la misma (ej: inConcert.prov_form_vccs_widget_example)

	Los metodos requeridos son:
		get
			Tiene que retornar un objeto con la Config del Widget.
			Esta configuracion quedara almacenada en el VCC.
		set
			Recibe un objeto con la configuracion del Widget que se ha almacenado en el VCC.
		get_validation_def
			Debe retornar un Array con las validaciones que se realizaran en el formulario del VCC.
			A estas validaciones las realiza el VCC.
			Los items Array deben seguir la estructura esperada en los formularios del VCC.
		set_events
			Se invoca cuando la clase que configura el VCC instancia la clase e inicia sus eventos.
			Sirve para que internamente se realicen bindeo de eventos segun los controles de configuracion del Widget.
		ready
			Se invoca cuando la instancia del Frm de provisioning ha termiando.
			Aca se podran realizar diferentes tareas que se necesiten.

*/

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

	//"use strict";

	//const FORMAT = inConcert.Utils.format;

	//---------------------------------------------------------------------
	// example
	function widget_example (params = {})
	{
		this.params = params;
		this.owner = params.owner;
		this.init();
	}

	/*
		Este nombre de coincidir con el valor del atributo "jsClassName" que se ha ingresado en la definicion del Widget en el archivo:
			\apps\integrations\widgets\sample\definitions.lua
		
		Ejemplo del archivo "definitions.lua"
		..
		..
			jsClassName = cosmo.fill([[inConcert.prov_form_vccs_widget_$WIDGET_DIRECTORY]], params) 
		..
		..
	*/
	if ( typeof inConcert.prov_form_vccs_widget_template !== "function" ) {
		inConcert.prov_form_vccs_widget_template = widget_example;
	}

	widget_example.prototype.init = function()
	{
		this.load_data();
	};

	widget_example.prototype.load_data = function()
	{
		const $cbo_type = $("#prov_form_vcc_basic_template_widget_integration_others_type");
		let values = [
			{id: "option2", name: "<%=_T('Option 2')%>"},
			{id: "option1", name: "<%=_T('Option 1')%>"}
		];
		const is_blank_first_item = false;
		const sorted = true;
		inConcert.Utils.fill_combo($cbo_type, is_blank_first_item, values, "id", "name", sorted);
	};

	widget_example.prototype.clean_up = function()
	{
		$("#prov_form_vcc_basic_template_widget_integration_value1").val("");
		$("#prov_form_vcc_basic_template_widget_integration_value2").val("");
		$("#prov_form_vcc_basic_template_widget_integration_others_type").val("option1").trigger("change");
		$("#prov_form_vcc_basic_template_widget_integration_others_option1_val1").val("");
		$("#prov_form_vcc_basic_template_widget_integration_others_option1_val2").val("");
		$("#prov_form_vcc_basic_template_widget_integration_others_option2_val1").val("");
	};

	widget_example.prototype.on_validation_error = function(/*type, message_error*/)
	{
		// TODO: Implementar si es necesario!!!
		//window.alert("widget_example.prototype.on_validation_error " + type + " " + message_error);
	};

	// Funcion requerida ....
	widget_example.prototype.get = function()
	{
		let info = {
			value1: $("#prov_form_vcc_basic_template_widget_integration_value1").val(),
			value2: $("#prov_form_vcc_basic_template_widget_integration_value2").val(),
			others: {
				type: $("#prov_form_vcc_basic_template_widget_integration_others_type").val()
			}
		};
		if (info.others.type === "option1") {
			info.others.val1 = $("#prov_form_vcc_basic_template_widget_integration_others_option1_val1").val();
			info.others.val2 = $("#prov_form_vcc_basic_template_widget_integration_others_option1_val2").val();
		}
		else {
			info.others.val1 = $("#prov_form_vcc_basic_template_widget_integration_others_option2_val1").val();
		}
		return info;
	};

	// Funcion requerida ....
	widget_example.prototype.set = function(info)
	{
		const $cbo_type = $("#prov_form_vcc_basic_template_widget_integration_others_type");
		if (!info) {
			return;
		}
		$("#prov_form_vcc_basic_template_widget_integration_value1").val(info.value1);
		$("#prov_form_vcc_basic_template_widget_integration_value2").val(info.value2);
		if (!info.others) {
			return;
		}
		$cbo_type.val(info.others.type).trigger("change");
		if (info.others.type === "option1") {
			$("#prov_form_vcc_basic_template_widget_integration_others_option1_val1").val(info.others.val1);
			$("#prov_form_vcc_basic_template_widget_integration_others_option1_val2").val(info.others.val2);
		}
		else {
			$("#prov_form_vcc_basic_template_widget_integration_others_option2_val1").val(info.others.val1);
		}
	};

	// Funcion requerida ....
	widget_example.prototype.get_validation_def = function()
	{
		const self = this;

		const can_validate = function () {
			return true;
		};

		const $cbo_type = $("#prov_form_vcc_basic_template_widget_integration_others_type");
		const can_validate_option1 = function() {
			return $cbo_type.val() === "option1";
		};

		const can_validate_option2 = function() {
			return $cbo_type.val() === "option2";
		};

		return [
			{
				control_id: "prov_form_vcc_basic_template_widget_integration_value1",
				can_validate: can_validate,
				messages: {
					required: "<%=_T('Value1 is required.')%>",
					range: {
						between: {from: 1, to: 10},
						message: "<%=_T('Value1 out of range. Please enter value between {0} and {1}.')%>"
					}
				},
				on_callback_error: function(type, message_error) {
					self.on_validation_error(type, message_error);
				}
			},
			{
				control_id: "prov_form_vcc_basic_template_widget_integration_value2",
				can_validate: can_validate,
				messages: {
					required: "<%=_T('Value2 id is required.')%>",
					id:       "<%=_T('Only letters, numbers, dashes and dots are allowed in the Value2.')%>"
				},
				on_callback_error: function(type, message_error) {
					self.on_validation_error(type, message_error);
				}
			},
			// others - option1
			{
				control_id: "prov_form_vcc_basic_template_widget_integration_others_option1_val1",
				can_validate: can_validate_option1,
				messages: {
					required: "<%=_T('Value1 is required in option 1 config.')%>",
					id:       "<%=_T('Only letters, numbers, dashes and dots are allowed in the Value1 option 1 config.')%>"
				},
				on_callback_error: function(type, message_error) {
					self.on_validation_error(type, message_error);
				}
			},
			// others - option2
			{
				control_id: "prov_form_vcc_basic_template_widget_integration_others_option2_val1",
				can_validate: can_validate_option2,
				messages: {
					required: "<%=_T('Value1 is required in option 2 config.')%>",
					id:       "<%=_T('Only letters, numbers, dashes and dots are allowed in the Value1 option 2 config.')%>"
				},
				on_callback_error: function(type, message_error) {
					self.on_validation_error(type, message_error);
				}
			}
		];
	};

	// Funcion requerida ....
	widget_example.prototype.set_events = function()
	{
		const $option1_panel = $("#prov_form_vcc_basic_template_widget_integration_others_option1_panel");
		const $option2_panel = $("#prov_form_vcc_basic_template_widget_integration_others_option2_panel");
		$("#prov_form_vcc_basic_template_widget_integration_others_type")
			.off("change")
			.on("change", function() {
				$option1_panel.hide();
				$option2_panel.hide();

				let value = $(this).val();
				if (value === "option1") {
					$option1_panel.show();
				}
				else if (value === "option2") {
					$option2_panel.show();
				}
				else {
					throw("unrecognized option " + value);
				}
			});
	};

	// Funcion requerida ....
	widget_example.prototype.ready = function()
	{
		let is_new_vcc = this.owner.frm.is_new;
		if (is_new_vcc) {
			this.clean_up();
		}
	};

}());

