local oo = require "loop.base"
local date = require "LuaOrm.Date"
require "LuaOrm.Mapper"
require "inconcert.translators"
local jsonEncoder = require "LuaOrm.Serializers.Json"
local ir = require "lua_iragentlink"
local agentservice = require "apps.agent.agentservice"
local permissions = require "shared.permissions.permissions"
local utf8 = require "icu.utf8"
local http_mapping = require "apps.agent.result_codes_to_http_codes"
local helpers = require "apps.agent.helpers"
local mail = require "apps.agent.mail"
local base64 = require "base64"
local interactionmanip = require "interactionmanip"
local m_channelsInteractionManIp =interactionmanip.RequireChannelsModules() -- lista de modules por canal de la rocka "interactionmanip"
local uuid = require "uuid"
local rpairs = LuaOrm.Associations.AssociationArray.Iterator
local model_perm = require "shared.permissions.models.permissions"
local agentModel = require "apps.agent.model"
local models = agentModel:GetModels()
local utils = require "shared.apps.utils"
local moses = require "moses"
local repositoryController = require("apps.repository")
local m_admModels = nil
local Fullcontact = require "fullcontact"

local wsapi = require "wsapi"
wsapi.request = require "wsapi.request"
wsapi.request_stream = require "wsapi.request_stream"

module(..., package.seeall, inconcert.orbit.application.Decorate)

m_htmlRender = nil

local function GetModel (entityName)
	assert(type(entityName) == "string")
	return utils.GetAgentModel(entityName)
end

local function GetAdmModel (entityName)
	assert(type(entityName) == "string")
	if not m_admModels or not m_admModels[entityName] then
	-- Si el modelo no se ha creado aun, debo crearlo.
	--  Esto sucede en algunas ocasiones si es que me han invocado desde otra app y aun no se ha creado el modelo.
	--  Entonces tengo que crear el modelo.
		local model = require "apps.administration.model"
		model:BuildModels()
		m_admModels = model:GetModels()
	end
	local entityModel = m_admModels[entityName]
	assert(type(entityModel) == "table")
	return entityModel
end

local function StartRepository ()
	repositoryController.ManualStart(inconcert.environment.apps.agent.cassandra.ip, inconcert.environment.apps.agent.cassandra.port)
end

local function ReplyJSON (web, data)
	web.headers["Content-Type"] = "application/json"
	if type(data) == "string" then
		return data
	end
	return json.encode(data)
end

function GetTopInteractions ()
	return 50
end

---
-- Helper function to perform searches (it is also called from the supervisor app (ouch!))
-- Returns the interactions that match the given criteria.
-- El campo "query" es un hash que tiene los campos a buscar. Cada campo puede corresponder
-- a un valor (string) o a un array, en caso de haber mas de un valor.
-- Esto sucede, por ejemplo, cuando se busca por interacciones de tipo CHAT o tipo TWITTER.
-- En query va a llegar { "type": ["TWITTER", "CHAT"] }
-- Lo primero que hacemos es normalizar las claves del campo "query", mapeando los nombres que
-- recibimos acá a los nombres que se tienen en el índice en ElasticSearch.
-- Por ejemplo: disposition -> disposition_code
-- Luego se va armando el query para elasticsearch, que es un json feo feo.
--
-- @tparam table request
-- request.last (en base64)
-- request.rows
-- request.query
-- @tparam table attention_levels An optional array of attention levels to filter for
-- return http status (puaj), table with response
--
-- Request de ejemplo:
-- {"last":"","rows":20,"query":{"type":["TWITTER"],"disposition":["123"],"agent":"agent1@inconcert","state":"FINISHED"},"paginating":true,"exact":false,"output":"interactions"}
--
-- a couple of helpers
-- flattens an array of words into a string (using table.concat)
local function to_string (value)
	return moses.isTable(value) and table.concat(value, " ") or tostring(value)
end

-- wraps a single value in an array (or returns the array)
local function to_table (value)
	return moses.isTable(value) and value or { tostring(value) }
end

local function is_nonempty_string (value)
	return (moses.isString(value) and not moses.isEmpty(value))
end

local function hasLicensesForChannel (channel, ksAgent)
	local agent = require "apps.agent.agent"
	return agent.hasLicensesForChannel(channel, ksAgent)
end

local function SplitString(inputstr, sep)
	if sep == nil then
			sep = "%s"
	end
	local t={} ; i=1
	for str in string.gmatch(inputstr, "([^"..sep.."]+)") do
			t[i] = str
			i = i + 1
	end
	return t
end

function HasPermission ( action_id, campaign_id, app_id, keyspace )
	app_id = app_id or "interaction" -- default

	local actions = {
	-- INTERACTIONS ACTIONS
		CREATE_OUTGOING           = { interaction = "int_create_outgoing"                     , supervisor = false                     },
		COMMENT                   = { interaction = "int_comment"                             , supervisor = "sup_monitoring_comment"  },
		HANGUP                    = { interaction = "int_hang_up"                             , supervisor = false                     }, -- TODO: ver no se esta utilizando!! quizas no va
		FINISH                    = { interaction = "int_finish"                              , supervisor = false                     },
		LABEL                     = { interaction = "int_label"                               , supervisor = "sup_monitoring_label"    },
--		MONITOR                   = { interaction = false                                     , supervisor = "sup_monitoring"          }, -- TODO: NO va aca, no existe validacion de API
		REDIAL                    = { interaction = "int_redial"                              , supervisor = false                     },
		REQUEUE                   = { interaction = "int_requeue"                             , supervisor = false                     },
		REOPEN                    = { interaction = "int_archive_reopen"                      , supervisor = "sup_archive_reopen"      },
		TAKE                      = { interaction = "int_queued_take"                         , supervisor = "sup_queued_take"         },
		TRANSFER                  = { interaction = "int_transfer"                            , supervisor = "sup_monitoring_transfer" },
		TRANSFER_QUEUED           = { interaction = false                                     , supervisor = "sup_queued_transfer"     },
		CHAT_FILE_TRANSFER        = { interaction = "int_chat_file_transfer"                  , supervisor = false                     },
		SCHEDULE		          = { interaction = "int_schedule"                            , supervisor = false                     },
		CONFERENCE		          = { interaction = "int_conference"                          , supervisor = false                     },
		SCREEN_SHARE              = { interaction = "int_screen_share"                        , supervisor = false                     },

	-- SPAM ACTIONS
		MARK_AS_SPAM              = { interaction = "int_mark_as_spam"                        , supervisor = false                     },
		NOT_SPAM                  = { interaction = "int_not_spam"                            , supervisor = false                     },
		DELETE_SPAM               = { interaction = "int_delete_spam"                         , supervisor = false                     },

	-- CONTACT ACTIONS
		SET_CONTACT               = { interaction = "int_save_contact"                        , supervisor = "sup_save_contact"        },

	-- PRINT ACTIONS
		PRINT                     = { interaction = "int_mail_print"                          , supervisor = "sup_mail_print"          },

	-- MOVE MAIL TO FOLDER ACTION
		MOVE_MAIL_TO_FOLDER       = { interaction = "int_mail_move_to_folder"                 , supervisor = "sup_mail_move_to_folder" },

	-- SEARCH INTERACTIONS FILTERS BY CAMPAIGN / ATTENTION LEVELS ....
		FILTER_BY_CROSS_CAMPAIGNS = { interaction = "int_search_conversations_cross_campaigns", supervisor = false                     },
		FILTER_WITHOUT_ATT_LEVEL  = { interaction = "int_search_conversations_without_als"    , supervisor = false                     },
		ASSISTANCE                = { interaction = false                                     , supervisor = "sup_monitoring_bot_interaction"         }
	}

	local has_permission = false
	local apps_actions = actions[ action_id ]

	if not apps_actions then
		LogError( "Interaction permissions: Unrecognized action id '%s' for app '%s'.", action_id, app_id )
		return has_permission
	end

	local action = apps_actions[ app_id ]
	has_permission = model_perm.IsPermissionEnabledForUser( app_id, action, campaign_id, keyspace )

	local user_id = utils.GetCurrentLoggedUser()
	if has_permission then
		LogInfo("User '%s', has permission ENABLED for action '%s' in campaign '%s' for app '%s'.", user_id, action_id, campaign_id, app_id )
	else
		LogWarning("User '%s', has permission DISABLED for action '%s' in campaign '%s' for app '%s'.", user_id, action_id, campaign_id, app_id )
	end

	return has_permission
end


-- Batch Functions
--
--	Given a web request and a batch type (currently just interations)
--	Will validate if the data to process a batch operation is correct.
--
local function ValidateBatchRequest (web, batchType)
	local post_data =  web.input["post_data"]
	local res, request = pcall(json.decode, post_data)
	if not res or not request[batchType] then
		web.status = 400
		LogWarning("Invalid request: %s", post_data)
		return false, ReplyJSON(web, {result = "INVALID_REQUEST", reason = "Invalid request: " .. tostring(post_data)})
	end

	if #request[batchType] == 0 then
		local errorMessage = LogWarning("Batch %s validation failed, batch list empty", batchType)
		web.status = 400
		return false, ReplyJSON(web, {result = string.upper(batchType).."_LIST_EMPTY", reason = errorMessage})
	end
	return true, request, request[batchType]
end

-- Process a batch of interactions or appointments (batch), and for each node will call a function passed as parameter
-- This is done to avoid code repetition and nested ifs
--
-- @param: web (table)
-- @param: action (string) - The performing action (i.e. finish)
-- @param: batchType (string) - The node type (i.e. "interactions" or "appointments")
-- @param: batch (table) - A batch of interactions or appointments.
-- @param: fn (function) - A callback function to apply on each interaction/appointment.
--
-- @return: This will call replyJSON so you need to call this function as "a last thing to do" on the main function
local function ProcessBatchData ( web, action, batchType, batch, fn )
	LogInfo("Processing batch for action: %s with a total of: %d %s", action, #batch, batchType)
	local successful_nodes = {}
	local unsuccessful_nodes = {}
	for _, node in pairs(batch) do
		local ok, args = fn( node.id, node )
		if ok then
			local data = { id = node.id }
			-- insert extra args if exists.
			if args and type(args) == "table" then
				for k, v in pairs(args) do data[k] = v end
			end
			table.insert(successful_nodes, data)
		else
			--something went wrong.
			LogWarning("[INTERACTIONS:ProcessBatchData] Unable to %s interaction %s with error %s", action, node.id, args.error_message)
			table.insert(unsuccessful_nodes, { id = node.id, result = args.error_code, reason = args.error_message, raw_info = args })
		end
	end
	--I give back to the user the list of unsuccessful interactions,
	--including on each of them, the error of because why it failed.
	if #unsuccessful_nodes ~= 0 then
		local err = "BATCH_PROCESS_INCOMPLETE"
		if web.status == 403 then
			err = "FORBIDDEN"
		end
		return ReplyJSON(web, {result = err, reason = "Some "..batchType.." failed to "..action, successful_nodes = successful_nodes, unsuccessful_nodes = unsuccessful_nodes })
	end
	--all nodes processed
	LogInfo("Success processing action: %s for %d %s", action, #batch, batchType)
	return ReplyJSON(web, {successful_nodes = successful_nodes, unsuccessful_nodes = unsuccessful_nodes})
end


-- Helper function to add all interactions in the groups to the batch
local function UnionBatchDataWithGroups(batch)
	local vcc = inconcert.authUser.virtualContactCenter
	local interactionsProcessed = {}

	local function markInteractionsProcessed(groupId, group)
		for id, children in pairs(group or {}) do
			if not interactionsProcessed[id] then
				interactionsProcessed[id] = true
				LogInfo("Adding interaction %s as part of the batch because it belongs to the group %s", id, groupId)
			end
			markInteractionsProcessed(groupId, children)
		end
	end
	local modelInteractions = GetModel("Interactions")
	for _, e in pairs(batch) do
		if not interactionsProcessed[e.id] then
			interactionsProcessed[e.id] = true
			local interaction = modelInteractions.GetInteraction(e.id)
			if not utils.TableIsEmpty(interaction) and interaction.interactionsGroup then
				markInteractionsProcessed(interaction.groupId, interaction.interactionsGroup)
			end
		end
	end
	local newBatch = {}
	for id in pairs(interactionsProcessed) do
		table.insert(newBatch, {id = id})
	end

	return newBatch
end


-- Helper function to remove all interactions that are in the same group from the batch
-- only leaves a representant of each group
local function RemoveInteractionsFromSameGroupInBatch(batch)
	local vcc = inconcert.authUser.virtualContactCenter
	local interactionsProcessed = {}
	local representativeByGroup = {}
	local modelInteractions = GetModel("Interactions")
	for _, e in pairs(batch) do
		local interaction = modelInteractions.GetInteraction(e.id)
		if not utils.TableIsEmpty(interaction) and interaction.interactionsGroup then
			if not representativeByGroup[interaction.groupId] then
				--group is not yet represented
				representativeByGroup[interaction.groupId] = e.id
				interactionsProcessed[e.id] = e
			else
				LogInfo("Interaction %s will be removes from batch because it belongs to the group %s", e.id, interaction.groupId)
			end
		else
			--if interaction has not group, then its added to the batch
			interactionsProcessed[e.id] = e
		end
	end

	local newBatch = {}
	for id, interaction in pairs(interactionsProcessed) do
		table.insert( newBatch, interaction )
	end

	return newBatch
end


-- Esta funcion ordena las interacciones para la app la aplicacion "Supervisor" ( Es raro no deberia estar aca, pero bueno hay que cambiar directamente el request )!!!
--   Este orden es utilizado en los paneles "monitoring" y "queued"
--   Se ordena con el siguiente criterio:
--     1 - las interacciones sincronicas(chat, call, etc)
--     2 - segun su prioridad (atributo "interaction.priority"), cuanto mas alto el valor, primero van las interacciones.
--     3 - a igual prioridad, segun su fecha de creacion (las mas antiguas primero)
--
local function SortSupervisorInteractionsPanel( interactions )
	assert(type(interactions) == "table")
	-- se le agrega un peso a los diferentes tipos de interacciones; las de peso mas alto van antes.
	local channels_order = { CALL = 2, CHAT = 2, IM = 2, FACEBOOK = 1, INSTAGRAM = 1, YOUTUBE = 1, APPSTORE = 1, GOOGLEPLAY = 1, RCS = 1, MAIL = 1, SMS = 1, WHATSAPP = 1, TWITTER = 1, WEBCONTACT = 1, VOICEMAIL = 1 }

	table.sort( interactions, function( a, b )
		-- 1 orden segun tipo de interaccion (channel)
		local a_channel_order = channels_order[a.type] or 0
		local b_channel_order = channels_order[b.type] or 0
		if a_channel_order > b_channel_order then
			return true
		elseif a_channel_order == b_channel_order then
			-- 2 orden segun prioridad
			local a_priority = a.priority or 0
			local b_priority = b.priority or 0
			if a_priority > b_priority then
				return true
			elseif a_priority == b_priority then
				-- 3 orden segun fecha de creacion
				if a.startTimestamp < b.startTimestamp then
					return true
				end
			end
		end
		return false
	end)
end

local function GetTopQueuedInteractions (web)
	_M.response.DontCache(web)

	local request = web.request

	local logged_user = utils.GetCurrentLoggedUser()
	LogInfo("[GetTopQueuedInteractions] Get top queued interactions from agent: %s",  logged_user)

	local userName = inconcert.authUser.userName
	local vcc = inconcert.authUser.virtualContactCenter
	local totalQueuedInteractions = {}

	--first add agent interactions
	local agentInteractions = GetModel("Interactions").GetUserQueuedInteractions(vcc, userName)
	for _, interaction in pairs(agentInteractions) do
		table.insert(totalQueuedInteractions, interaction)
	end

	-- Fetch level ids and build a lookup map, in order to filter by attention level
	local userAttLevels = GetModel("Users").GetUserAttentionLevels(logged_user)
	for _, level in ipairs(userAttLevels) do
		userAttLevels[level] = level
	end

	local totalInteractions = GetTopInteractions() * 1.5 -- esto es para que me retorne 75 que fue lo que se pidio en el mail :)
	local params = {
		campaign_id = nil,
		start       = 0,
		total       = totalInteractions,
		filter      = function(interaction)
			local result = {success = nil}
			if utils.TableIsEmpty(interaction) then
			-- NO agrego la interacción si: Si la misma es vacia o no existe, 
				result.success = false
				return result
			end
			if not interaction.attention_level then
				result.success = true
				return result
			end
			if not utils.TableIsEmpty(userAttLevels) then
				if userAttLevels[interaction.attention_level.id] then
					result.success = true
					return result
				end
			end
			result.success = false
			return result
		end
	}
	local userCampaigns = GetModel("Campaigns").GetUserCampaigns(userName, vcc)
	local modelInteractions = GetModel("Interactions")
	for campaign_id in pairs(userCampaigns) do
		params.campaign_id = campaign_id
		local result = modelInteractions.GetCampaignInteractions(params)
		local queuedInteractions = result.interactions
		for _, interaction in pairs(queuedInteractions) do
			table.insert(totalQueuedInteractions, interaction)
		end
	end

	LogInfo("[GetTopQueuedInteractions] Return '%d' top queued interactions for user '%s'", #totalQueuedInteractions, logged_user)

	interactionmanip.NormalizeInteractions(totalQueuedInteractions)
	if not request.inflate then
		interactionmanip.StripUnwantedMembers(totalQueuedInteractions)
	end

	return {
		interactions = totalQueuedInteractions
	}
end

function SetInteractionContact(web)
	local data = json.decode(web.input["post_data"])
	local contactIdAtVCC = helpers.AddVCC(data.contactId, inconcert.authUser.virtualContactCenter)
	LogInfo("Set interaction contact request - interactionId: '%s', contactId: '%s', contactName '%s'", data.interactionId, contactIdAtVCC, data.contactName)

	local interaction = GetModel("Interactions").GetInteraction(data.interactionId)
	if utils.TableIsEmpty(interaction) then
		web.status = 400
		return ReplyJSON(web, false)
	end
	local app_id = data.app_id
	if not HasPermission( "SET_CONTACT", interaction.campaign, app_id ) then
		web.status = 403
		return "forbidden"
	end

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local successfullyAssignedContact = irLink:SetInteractionContact(data.interactionId, contactIdAtVCC, data.contactName)
	LogInfo("Setted interaction contact: '%s' with name '%s' on interaction: '%s', success: '%s'.", contactIdAtVCC, data.contactName, data.interactionId, tostring(successfullyAssignedContact))

	return ReplyJSON(web, successfullyAssignedContact)
end

function RemoveContactConflict(web)
	local request_id = web.vars.HTTP_X_ICC_RID
	local data = json.decode(web.input["post_data"])
	local userAtVCC = utils.GetCurrentLoggedUser()
	LogInfo("Resolving interaction conflict, interactionId: '%s', user: '%s'", data.interactionId, userAtVCC)

	local interaction = GetModel("Interactions").GetInteraction(data.interactionId)
	if utils.TableIsEmpty(interaction) then
		web.status = 400
		LogWarning("Resolving interaction conflict failed, interactionId: '%s' not found, user: '%s'", data.interactionId, userAtVCC)
		return ReplyJSON(web, false)
	end

	-- TO-DO: Create permissions?
	-- if not HasPermission(interaction.campaign, interaction.type, "remove_contact_conflict", "interaction") then
	-- 	web.status = 403
	-- 	LogWarning("Resolving interaction conflict failed, user does not have permission to resolve conflicts, interactionId: '%s', user: '%s'", data.interactionId, userAtVCC)
	-- 	return ReplyJSON(web, false)
	-- end

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local conflictSolved = irLink:RemoveContactConflict({ id = data.interactionId, rid = request_id }	)
	if not conflictSolved then
		web.status = 500
		LogError("There is a problem resolving interaction conflict, interactionId: '%s', user: '%s'", data.interactionId, userAtVCC)
		return ReplyJSON(web, false)
	end

	LogInfo("Contact conflict solved on interaction: '%s' by user: '%s'", data.interactionId, userAtVCC)
	return ReplyJSON(web, conflictSolved)
end

function DiscardFailedEvent(web)
	local agentAtVCC = utils.GetCurrentLoggedUser()

	LogInfo("Discard failed event request from agent:%s, post_data: '%s'.", agentAtVCC, web.input["post_data"])

	local request = json.decode(web.input["post_data"])

	local interaction = GetModel("Interactions").GetInteraction(request.interactionId)
	if utils.TableIsEmpty(interaction) then
		local error_message = LogWarning("Error discarding failed event '%s'. Interaction '%s' not found.", request.fake_event_id, request.interactionId)
		web.status = 404
		return { result = "INTERACTION_NOT_FOUND", reason = error_message }
	end

	-- TODO: esto por ahora solo sirve para FB, en la medida que se implemente en los demas proxies, hay que aregegar el mismo en tabla "supported_proxies"
	local supported_proxies = { FACEBOOK = true, INSTAGRAM = true, YOUTUBE = true, APPSTORE = true, GOOGLEPLAY = true }
	local proxy_type = interaction.type
	if not supported_proxies[ proxy_type ] then
		local error_message = LogWarning("Error discard failed event. Proxy %s not supported.", proxy_type)
		web.status = 404
		return { result = "PROXY_NOT_SUPPORTED", reason = error_message }
	end

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local response = irLink:SendCommand({
		interactionId    = request.interactionId,
		fake_event_id    = request.fake_event_id,
		RID              = web.vars.HTTP_X_ICC_RID,
		proxyType        = proxy_type,
		agent            = agentAtVCC,
		command          = "DiscardFailedEvent"
	})
	if response.result ~= "OK" then
		LogWarning("Unable discard failed event '%s' of interaction '%s'. Reason: %s", request.fake_event_id, request.interactionId, response.reason)
	end

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(response["result"])
	web.status = http_code or 500 --unknown result code

	return ReplyJSON( web, response )
end

function FailSendingEvent(web)
	local agentAtVCC = utils.GetCurrentLoggedUser()

	LogInfo("Fail sending event request from agent:%s, post_data: '%s'.", agentAtVCC, web.input["post_data"])

	local request = json.decode(web.input["post_data"])

	local interaction = GetModel("Interactions").GetInteraction(request.interactionId)
	if utils.TableIsEmpty(interaction) then
		local error_message = LogWarning("Error fail sending event '%s'. Interaction '%s' not found.", request.fake_event_id, request.interactionId)
		web.status = 404
		return { result = "INTERACTION_NOT_FOUND", reason = error_message }
	end

	-- TODO: esto por ahora solo sirve para FB, en la medida que se implemente en los demas proxies, hay que aregegar el mismo en tabla "supported_proxies"
	local supported_proxies = { FACEBOOK = true, INSTAGRAM = true, YOUTUBE = true, APPSTORE = true, GOOGLEPLAY = true }
	local proxy_type = interaction.type
	if not supported_proxies[ proxy_type ] then
		local error_message = LogWarning("Error fail sending event. Proxy %s not supported.", proxy_type)
		web.status = 404
		return { result = "PROXY_NOT_SUPPORTED", reason = error_message }
	end

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local response = irLink:SendCommand({
		interactionId    = request.interactionId,
		fake_event_id    = request.fake_event_id,
		RID              = web.vars.HTTP_X_ICC_RID,
		proxyType        = proxy_type,
		agent            = agentAtVCC,
		command          = "FailSendingEvent",
		error            = { message = "Agent timeout sending event", code = "TIMEOUT" }
	})
	if response.result ~= "OK" then
		LogWarning("Unable fail sending event '%s' of interaction '%s'. Reason: %s", request.fake_event_id, request.interactionId, response.reason)
	end

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(response["result"])
	web.status = http_code or 500 --unknown result code

	return ReplyJSON( web, response )
end

function SendCustomMessage(web)
	local agentAtVCC = utils.GetCurrentLoggedUser()

	LogInfo("SendCustomMessage from agent:%s, post_data: '%s'.", agentAtVCC, web.input["post_data"])

	local request = json.decode(web.input["post_data"])

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	LogInfo("SendCustomMessage called for id:%s from agent %s - message %s", request.interactionID, agentAtVCC, request.message)
	local response = irLink:SendCustomMessage({
		interactionID    = request.interactionID,
		RID              = web.vars.HTTP_X_ICC_RID,
		proxyType		 = request.proxyType,
		message          = request.message,
		waitForResponse  = request.waitForResponse or false,
		timeout			 = request.timeout or nil
	})
	if response.result ~= "OK" then
		LogWarning("Error sending message to %s. Reason: %s", request.interactionID, response.reason)
	end

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(response["result"])
	web.status = http_code or 500 --unknown result code

	return ReplyJSON( web, response )
end


local configHasChanged
function ConfigHasChanged()
	configHasChanged = true
end

--------------------
--handlers
--------------------

--listings

---
-- recupera una interaccion que se encuentre en redis
--
local function GetInteraction (web)
	local request = web.request
	LogInfo("[AGENT.INTERACTIONS: GetInteraction] get interaction: %s", request.interactionID)

	local interaction = GetModel("Interactions").GetInteraction(request.interactionID)
	if utils.TableIsEmpty(interaction) then
		local error_message = LogWarning("Interaction '%s' not found.", request.interactionID)
		web.status = 404
		return { result = "INTERACTION_NOT_FOUND", reason = error_message }
	end

	interactionmanip.NormalizeInteractions({ interaction })
	if not request.inflate then
		interactionmanip.StripUnwantedMembers({ interaction })
	end
	return interaction
end

local function GetBotInteraction (web)
	local request = web.request
	LogInfo("[AGENT.INTERACTIONS: GetInteraction] get interaction: %s", request.interactionID)
	local interaction = GetModel("Interactions").GetInteraction(request.interactionID)
	
	if utils.TableIsEmpty(interaction) then
		local error_message = LogWarning("Interaction '%s' not found.", request.interactionID)
		web.status = 404
		return { result = "INTERACTION_NOT_FOUND", reason = error_message }
	end

	interactionmanip.NormalizeInteractions({ interaction })
	if not request.inflate then
		interactionmanip.StripUnwantedMembers({ interaction })
	end
	
	if interaction.data and interaction.data.reportingData and interaction.data.reportingData.process_flow_data and interaction.data.reportingData.process_flow_data.fields then
		for _, interaction_rdf in pairs(interaction.data.reportingData.process_flow_data.fields) do
			if interaction_rdf.name == 'internal_consultation_flag' then
				interaction.interactionData.internal_consultation_flag = interaction_rdf.value
			end
			if interaction_rdf.name == 'internal_consultation_question' then
				interaction.interactionData.internal_consultation_question = interaction_rdf.value
			end
			if interaction_rdf.name == 'internal_consultation_id' then
				interaction.interactionData.internal_consultation_id = interaction_rdf.value
			end
			if interaction_rdf.name == 'internal_consultation_timeout' then
				interaction.interactionData.internal_consultation_timeout = interaction_rdf.value
			end
			if interaction_rdf.name == 'internal_consultation_startdate' then
				interaction.interactionData.internal_consultation_startdate = interaction_rdf.value
			end
			if interaction_rdf.name == 'internal_consultation_kb' then
				interaction.interactionData.internal_consultation_kb = interaction_rdf.value
			end
		end
	end
	
	return interaction
end

---
-- recupera una interaccion que se encuentre en cassandra
--
local function GetArchivedInteraction (web)
	local request = web.request
	LogInfo("get archive interaction: %s", request.interactionID)
	local interaction = GetModel("Interactions").GetFromArchive(request.interactionID) or {}
	if utils.TableIsEmpty(interaction) then
		local error_message = LogWarning("Archived interaction '%s' not found.", request.interactionID)
		web.status = 404
		return { result = "INTERACTION_NOT_FOUND", reason = error_message }
	end

	interactionmanip.NormalizeInteractions({ interaction })

	if not request.inflate then
		interactionmanip.StripUnwantedMembers({ interaction })
	end

	return interaction
end

local function FilterUserInteractionsByCampaign (user_interactions, userName, campaignId, keyspace, vcc)
	if not user_interactions then
		return
	end
	vcc = vcc or inconcert.authUser.virtualContactCenter
	local interactions_list = {}
	--get all the campaigns of the requesting user
	local simpleUserId = utils.GetId(userName)
	local userCampaigns = GetModel("Campaigns").GetUserCampaigns(simpleUserId, vcc, keyspace)
	for campIdWithVcc, role in pairs(userCampaigns) do
		if role ~= "invited" then
			local exists = GetModel("Campaigns").CampaignExists(campIdWithVcc, keyspace)
			if exists and (not campaignId or campaignId == campIdWithVcc) then
				--only let pass through the interactions belonging to the campaign
				for i, c in pairs(user_interactions) do
					if c.campaign == campIdWithVcc then
						table.insert(interactions_list, c)
					end
				end
			end
		end
	end
	return interactions_list
end

function RetrieveUserInteractions (userName, campaignId, keyspace, vcc)
	vcc = vcc or inconcert.authUser.virtualContactCenter
	--first get all the interactions for the user
	local user_interactions = GetModel("Interactions").GetUserInteractions(vcc, userName)
	return FilterUserInteractionsByCampaign(user_interactions, userName, campaignId, keyspace, vcc)
end

function CountUserAssignedInteractionsByType (userName, vcc, keyspace)
	vcc = vcc or inconcert.authUser.virtualContactCenter
	local info = {live = 0, active = 0}
	local userInteractions = RetrieveUserInteractions(userName, nil, keyspace, vcc)
	for _, interaction in ipairs(userInteractions or {}) do
		if utils.InteractionIsLive(interaction.type) then
			info.live = info.live + 1
		else
			info.active = info.active + 1
		end
	end
	--LogDebug("[INTERACTIONS:CountUserAssignedInteractionsByType] Info: '%s'", json.encode(info))
	return info
end

-- Note: Should be local but the administration app is using it (but it shouldn't)
function GetUserInteractions (web)
	local request = web.request
	local _T = m_htmlRender.extension.makeTextTranslator()
	local vcc = inconcert.authUser.virtualContactCenter
	local user_name = inconcert.authUser.userName

	LogInfo("Get user interactions called by %s@%s for team member %s, will return for the same campaigns...",  user_name, vcc, request.memberId)
	local ks_agent = utils.GetDatasource():CassandraConnection( "Agent" )
	local interactionList = RetrieveUserInteractions(request.memberId, nil, ks_agent)
	ks_agent:Close()

	interactionmanip.NormalizeInteractions(interactionList)
	if not request.inflate then
		interactionmanip.StripUnwantedMembers(interactionList)
	end

	for k, interaction in pairs(interactionList) do
		interaction.typeLabel = _T(interaction.type)
	end
	return interactionList
end

local function GetAgentsWorkgroups (workgroups, ksAgent)
	assert(ksAgent)
	LogInfo("[INTERACTIONS:GetAgentsWorkgroups] Retrieving Agents of Workgroups %s", json.encode(workgroups or {}))
	if #(workgroups or {}) == 0 then
		return
	end

	local vcc = inconcert.authUser.virtualContactCenter
	local hashAgentsWorkgroups

	for _, workgroupId in ipairs(workgroups) do
		local workgroupInfo = GetModel("Workgroups").GetWorkgroups(vcc, workgroupId, ksAgent)
		--LogDebug("%d --> %s = %s", _, workgroupId, json.encode(workgroupInfo))
		if workgroupInfo.Agents then
			hashAgentsWorkgroups = hashAgentsWorkgroups or {}
			for userId, enabled in pairs(workgroupInfo.Agents) do
				if enabled then
					local fullUserId = (utils.AppendVcc(userId, vcc)):lower()
					hashAgentsWorkgroups[fullUserId] = {
						Id = fullUserId
					}
				end
			end
		end
	end

	local arrayAgentsWorkgroups = {}
	if hashAgentsWorkgroups then
		arrayAgentsWorkgroups = utils.HashToArray(hashAgentsWorkgroups, "Id")
	end

	LogInfo("[INTERACTIONS:GetAgentsWorkgroups] Total %d Agents of Workgroups.", #arrayAgentsWorkgroups)
	--LogDebug("[INTERACTIONS:GetAgentsWorkgroups] Agents of Workgroups: %s", json.encode(hashAgentsWorkgroups or {}))

	return arrayAgentsWorkgroups, hashAgentsWorkgroups
end

---
-- Search for assigned interactions for a given campaign.
-- If no filters are supplied, use Redis (fast path)
-- Else, use Elasticsearch (potentially slow path)
--
local function GetCampaignInteractions (web)
	LogInfo("[INTERACTIONS:GetCampaignInteractions] Request = %s.", web.input["post_data"])

	-- Filter skeleton. We add more specialized filters as needed (plus, a query for text if needed too)
	local query = {
		query = {
			bool = {
				must = {},
				filter = {}
			}
		} 
	}

	local request = web.request

	if request.sort then
		query.sort = request.sort
	else
		query.sort = { { start_timestamp = { order = "asc" }, priority = { order = "desc" } } }
	end

	local MAX_INTERACTIONS = 500
	-- Determine if pagination is needed
	local page = tonumber(request.page) or 0
	if page == 0 then
		-- thay want all interactions
		query.size = MAX_INTERACTIONS
		query.from = 0
	else
		-- they want a specific "page"
		local rows_per_page = tonumber(request.rows_per_page) or 50
		local start =  rows_per_page * (page - 1)
		query.size = rows_per_page
		query.from = start
	end

	local ksAgent = utils.GetDatasource():CassandraConnection("Agent")
	-- if no specific type was requested, search for all licensed types
	if not request.type then
		request.type = moses.filter({ "CALL", "CHAT", "FACEBOOK", "INSTAGRAM", "YOUTUBE", "APPSTORE", "GOOGLEPLAY", "RCS", "IM", "MAIL", "SMS", "WHATSAPP", "TWITTER", "WEBCONTACT", "VOICEMAIL" }, function(_, channel)
			return hasLicensesForChannel(channel, ksAgent)
		end)
	end

	local vcc = inconcert.authUser.virtualContactCenter

	if request.agent then
		-- Convierto el agente en una lista lista de agentes.
		request.agent = to_table(request.agent)
		request.agent = moses.map(request.agent, function(_,member_id)
			return (helpers.AddVCC( helpers.SeparateVCC(member_id), vcc))
		end)
		-- (***) Si en el POST llegan workgrups, debo filtrar los agentes según si pertenece o no al workgroup.
		if request.workgroup then
			local _, hashAgentsWorkgroups = GetAgentsWorkgroups(request.workgroup, ksAgent)
			if hashAgentsWorkgroups then
				local filteredAgentsByWorkgroups = {}
				for _, agentId in pairs(request.agent) do
					if hashAgentsWorkgroups[agentId:lower()] then
						-- Si pertenece al Workgroup, la agrego.
						table.insert(filteredAgentsByWorkgroups, agentId)
					end
				end
				request.agent = filteredAgentsByWorkgroups
			end
		end
	else
	-- (***) Si en el query NO existe un Agent, filtra contra los agentes pertenecientes a los workgroups (si es que el mismo llega en el filtro del POST)
		if request.workgroup then
			local agentsWorkgroups = GetAgentsWorkgroups(request.workgroup, ksAgent)
			request.agent = agentsWorkgroups
		end
	end
	ksAgent:Close()

	-- Map some of the names used in the request to what's defined in elasticsearch
	moses({ campaign_id = "campaign", account_id = "account", name = "contact_name", text = "content", label = "labels" }):each(function(k,v)
		if request[k] then
			request[v] = request[k]
			request[k] = nil
		end
	end)

	-- now add filters
	local must_filters = query.query.bool.filter

	-- the following filters use either 'terms' or 'term' if there are more than one value
	-- ie, type=[ TWITTER, MAIL ]
	moses({ "type", "campaign", "account", "agent" }):eachi(function(_, filter)
		if request[filter] then
			local key_name = moses.isTable(request[filter]) and "terms" or "term"
			table.insert(must_filters, { [key_name] = { [filter] = request[filter] }})
		end
	end)

	if request.attention_levels then
		-- va exclusivamente contra los als (posteados)
		table.insert(must_filters, {
			["or"] = {
				{ terms = { ["attention_level"] = request.attention_levels } }
			}
		})
	end

	-- voy por los labels que tienen que ir entre AND de forma individual
	if request.labels then
		local and_filter = {}
		local key = "labels"
		for _,label_id in pairs(request.labels) do
			table.insert(and_filter, {
				terms = {
					[key] = { utf8.lower(label_id) }
				}
			})
		end

		table.insert(must_filters, {
			["and"] = and_filter
		})
	end

	-- contact address
	if request.address then
		table.insert(must_filters, { term = { ["contact_address"] = utf8.lower(request.address) } })
	end

	local utc_date_from = request.date_from and utils.ConvertDateToUTCDate(request.date_from, "00:00:00")
	local utc_date_to = request.date_to and utils.ConvertDateToUTCDate(request.date_to, "23:59:59") -- Next day

	if utc_date_from or utc_date_to then
		local range = { start_timestamp = { format = "yyyy-MM-dd HH:mm:ss" } }
		if utc_date_from then range.start_timestamp.gte = utc_date_from:toString("%Y-%m-%d %H:%M:%S") end
		if utc_date_to then range.start_timestamp.lte = utc_date_to:toString("%Y-%m-%d %H:%M:%S") end
		table.insert(must_filters, { range = range })
	end

	local fuzzy_query = not request.exact

	moses({ "content", "contact_name", "ticket" }):eachi(function(_, filter)
		if request[filter] then
			local partial_query = query.query.filtered.query
			-- si es convertible a numerico (aun cosas del estilo 123abc), uso wildcards para buscar numeros
			if (filter == "content" or filter == "ticket") and fuzzy_query and tonumber(request[filter]) then
				partial_query.wildcard = partial_query.wildcard or {}
				partial_query.wildcard[filter] = "*"..utf8.lower(request[filter]).."*"
			else
				partial_query.match = partial_query.match or {}
				partial_query.match[filter] = {
					query = utf8.lower(request[filter]),
					fuzziness = fuzzy_query and 1 or nil,
					prefix_length = fuzzy_query and 1 or nil -- para que no se ponga tan creativo con las substituciones
															 -- y no haga reemplazos al comienzo de la palabra
				}
			end
		end
	end)

	table.insert(must_filters, { terms = { ["state"] = request.states or {"LOCKED", "TAKEN", "WRAPUP"} }})

	if #query.query.bool.must == 0 then
		query.query.bool.must = nil
	end
	local modelInteractions = GetModel("Interactions")
	local ok, result = modelInteractions.search_elastic(inconcert.authUser.virtualContactCenter, query)
	if not ok then
		return 500, { result = "REQUEST_FAILED", reason = result }
	end
	if result.timed_out then
		LogWarning("[INTERACTIONS:GetCampaignInteractions] Query against elasticsearch timed out. %s", json.encode(query))
		return 502, { result = "REQUEST_TIMEDOUT", reason = "Query timed out" }
	end
	LogInfo("[INTERACTIONS:GetCampaignInteractions] Query took %s ms and returned %s hits out of %s", result.took, #result.hits.hits, result.hits.total.value)
	--LogDebug("%s", json.encode(result, true))
	local queued_interactions = {}

	-- retrieve the full interaction
	local ks_interactions = utils.GetDatasource():CassandraConnection( "Interactions" )
	moses.each(result.hits.hits, function(_, hit)
		local interaction = modelInteractions.GetInteraction(hit._id, ks_interactions)
		if not utils.TableIsEmpty(interaction) then table.insert(queued_interactions, interaction) end
	end)
	ks_interactions:Close()
	
	interactionmanip.NormalizeInteractions(queued_interactions)

	if not request.inflate then
		interactionmanip.StripUnwantedMembers(queued_interactions)
	end

	return { interactions = queued_interactions, total_interactions = result.hits.total.value }
end


---
-- This function returns dispositions data codes, and the campaign disposition settings.
--
local function BuildDispositionData (clientDisposition, campaign, agentAtVCC)
	local campaignDisposition = GetModel("Campaigns").FindDisposition(campaign.dispositions, clientDisposition.treePath)
	if not campaignDisposition then
		local encodedDisposition = json.encode(clientDisposition or {})
		LogError("Unable to archive interaction. Disposition %s does not exist on campaign %s", encodedDisposition, campaign.Id)
		return false, {result = "DISPOSITION_DOES_NOT_EXIST", reason = ("Disposition %s does not exist on campaign %s"):format(encodedDisposition, campaign.Id)}
	end
	local disposition = {
		agentAtVCC          = agentAtVCC,
		dialerCode          = campaignDisposition.dialerCode,
		contactCode         = campaignDisposition.contactCode,
		isGoal              = campaignDisposition.isGoal,
		finalizationMessage = campaignDisposition.finalizationMessage,
		reopen              = campaignDisposition.reopen,
		cleanupLabels       = campaignDisposition.cleanupLabels,
		name                = clientDisposition.name,
		treePath            = clientDisposition.treePath,
		contactId           = clientDisposition.contactId,
		contactName         = clientDisposition.contactName
	}

--TODO: (Ref #7571) Activar si en algun momento se realiza la parte de schedule de interacciones.
	-- if campaignDisposition.needReschedule then
		-- if not clientDisposition.rescheduleDate or not clientDisposition.rescheduleTo then
			-- local encodedTo = json.encode(clientDisposition.rescheduleTo or {})
			-- LogError("Unable to archive interaction. Incorrect reschedule information. Date: '%s', To: '%s'.", clientDisposition.rescheduleDate, encodedTo)
			-- return false, {result = "INCORRECT_RESCHEDULE_INFORMATION", reason = ("Incorrect reschedule information. Date: '%s', To: '%s'."):format(clientDisposition.rescheduleDate, encodedTo)}
		-- end
		-- disposition.alternativeNumber = clientDisposition.alternativeNumber
		-- disposition.rescheduleDate    = clientDisposition.rescheduleDate
		-- disposition.rescheduleTo      = clientDisposition.rescheduleTo
	-- end
	return disposition, campaignDisposition
end

---
-- If the given campaign has disposition codes, checks that a valid one has been provided.
-- Returns true, a table with disposition data codes and the campaign disposition settings
-- or false and a json encoded error response.
--
local function ValidateCampaignDispositionData (campaign, postDisposition, agentAtVCC)
	local campaignNeedsDisposition = GetModel("Campaigns").HasDispositionCodes(campaign)
	if campaignNeedsDisposition and not postDisposition then
		local errorMessage = LogWarning("[INTERACTIONS:ValidateCampaignDispositionData] Disposition is required for the campaign %s", campaign.Id)
		return false, {result = "DISPOSITION_CODE_REQUIRED", reason = errorMessage}
	end

	local campaignDisposition
	local disposition = {}
	if campaignNeedsDisposition then
		-- build disposition code information
		disposition, campaignDisposition = BuildDispositionData(postDisposition, campaign, agentAtVCC)
		if not disposition then
			local errorMessage = campaignDisposition
			LogWarning("[INTERACTIONS:ValidateCampaignDispositionData] Failed to build disposition code, with error %s", errorMessage)
			return false, {reason = errorMessage}
		end
	end
	return true, disposition, campaignDisposition
end

local function ValidateFinishInfo (info)
	if not info then
		local errorMessage = LogWarning("[INTERACTIONS:ValidateFinishInfo] Archive interactions - Invalid Request")
		return false, {error_code = "Invalid request", error_message = errorMessage}
	end

	local campaignId = info.campaignId
	if not campaignId then
		local errorMessage = LogWarning("[INTERACTIONS:ValidateFinishInfo] Archive interactions - Invalid Request campaign id: '%s'", campaignId )
		return false, {error_code = "Invalid request", error_message = errorMessage}
	end

	local campaign = GetModel("Campaigns").Get(campaignId)
	if not campaign then
		local errorMessage = LogWarning("[INTERACTIONS:ValidateFinishInfo] Unable to archive interactions. Campaign '%s' does not exist", campaignId)
		return false, {error_code = "CAMPAIGN_DOES_NOT_EXIST", error_message = errorMessage}
	end

	local agentAtVCC = utils.GetCurrentLoggedUser()
	-- validate if the campaign needs disposition code
	local res, disposition, campaignDisposition = ValidateCampaignDispositionData(campaign, info.disposition, agentAtVCC)
	if not res then
		-- did not pass campaign disposition code validation
		local err = disposition
		return false, err
	end

	return true, {
		campaignId          = campaignId,
		app_id              = info.app_id or "interaction",
		disposition         = disposition,
		campaignDisposition = campaignDisposition,
		comment             = info.comment or ""
	}
end

local function IsMemberEnabledForTransferStep1 (info)
	local response = {success = true}

	if info.interactionId then
		local interaction = GetModel("Interactions").GetInteraction(info.interactionId)
		info.channel = interaction.type
		info.campaignId = interaction.campaign
		info.interaction = interaction
		response.interaction = info.interaction

		if utils.TableIsEmpty(interaction) then
			response.success = false
			response.reason = "INTERACTION_DOES_NOT_EXIST"
			return response
		end
	end

	assert(info.channel, "Channel is required for transfer member validation.")
	assert(info.campaignId, "campaignId is required for transfer member validation.")

	local vcc = inconcert.authUser.virtualContactCenter
	info.target.user = utils.AppendVcc(utils.GetId(info.target.user), vcc)

	------------------------------------------------
	-- 1) Valido que la interaccion no se transfiera a si mismo
	local loggedUserId = utils.GetCurrentLoggedUser()
	if loggedUserId:lower() == info.target.user:lower() and info.appId ~= "supervisor" then
		--LogDebug("[INTERCTIONS:IsMemberEnabledForTransferStep1] 1 - No se puede incluir a si mismo si NO tiener permisos sobre la App Supervisor. user: %s", loggedUserId)
		response.success = false
		response.reason = "CAN_NOT_TRANSFER_YOURSELF"
		return response
	end
	------------------------------------------------
	-- 2) Valido que el Usuario destino, exista en la BD
	info.member = info.member or GetModel("Users").Get(info.target.user)
	if not info.member or (not info.member.Id and not info.member.Name) then
		--LogDebug("[INTERCTIONS:IsMemberEnabledForTransferStep1] 2 - Usuario no existe. user: %s", info.target.user)
		response.success = false
		response.reason = "MEMBER_DOES_NOT_EXIST"
		return response
	end
	------------------------------------------------
	-- 3) Valido que el Usuario destino, se encuentre en estado ENABLED
	if info.member.Enabled == "0" then
		--LogDebug("[INTERCTIONS:IsMemberEnabledForTransferStep1] 3 - Usuario Disabled. user: %s", info.member.Id)
		response.success = false
		response.reason = "MEMBER_IS_DISABLED"
		return response
	end
	------------------------------------------------
	-- 4) Valido que el Usuario destino, tenga permisos para la APP INTERACTION
	-- REF #17373: Comento todo lo siguiente porque demora excesivamente y genera problemas en produccion. 
	
	--local hasPermissionToApp = model_perm.HasPermissionToApplication("interaction", info.target.user, info.member.UserGroup, nil, nil, nil, info.ksAgent, nil, nil, nil, info.applicationsActions)
	--if not hasPermissionToApp then
	--	--LogDebug("[INTERCTIONS:IsMemberEnabledForTransferStep1] 4 - El Usuario No tiene permisos sobre la App Interaction. user: %s", info.member.Id)
	--	response.success = false
	--	response.reason = "MEMBER_NOT_HAS_PERMISSION_TO_INTERACTION_APPLICATION"
	--	return response
	--end

	-- Ok - Paso 1
	return response
end

local function IsMemberEnabledForTransferStep2 (info)
	local response = {success = true, interaction = info.interaction}

	------------------------------------------------
	-- 5) Validaciones especificas por tipo de CANAL

	local function CanTransferInSynchronousChannels ()
		-- Validaciones para los Canales sincronicos (CHAT Y CALL con y sin Integracion con Allegro)
		-- A) Valida que el Agente a transferir se encuentre en un ESTADO disponible
		local agentStatus
		if info.usersStatuses then
			agentStatus = info.usersStatuses[info.target.user]
			--LogDebug("----> 1 - %s = %s", info.target.user, json.encode(agentStatus or {}))
		else
			agentStatus = agentservice.GetAgentStatus(info.target.user)
			--LogDebug("----> 2 - %s = %s", info.target.user, json.encode(agentStatus or {}))
		end

		if type(agentStatus) == "table" and type(agentStatus.state) == "string" then
			local status = utf8.upper(agentStatus.state)
			if status ~= "ACTIVE" and status ~= "NOACD" then
			-- Aca ingreso si el usuario NO se encuentra en los estados "Disponible (ACTIVE) o En linea No ACD (NOACD)"
				--LogDebug("[INTERCTIONS:IsMemberEnabledForTransferStep2] 1 - El Usuario No se encuentra en Estado ACTIVE o NOACD (AGENT SERVICE). user: %s", info.member.Id)
				response.success = false
				response.reason = "MEMBER_NOT_AVAILABLE"
				return false
			end
		end
		-- Si, puede ser transferida al CANAL
		return true
	end

	local availablesChannels = {
		CHAT = {enabled = true,
			CanTransfer = function()
				return CanTransferInSynchronousChannels()
			end
		},
		CALL = {enabled = true,
			CanTransfer = function()
				if utils.HasAllegroIntegration() then
					local userAllegroInfo = info.member.AllegroIntegration or {}
					--LogDebug("----------------------------2 userAllegroInfo %s", json.encode(userAllegroInfo))
					if utils.TableIsEmpty(userAllegroInfo.peer) then
						-- Falla si el Usuario de un VCC con Allegro NO tiene un telefono configurado.
						--LogDebug("[INTERCTIONS:IsMemberEnabledForTransferStep2] 2 - El Usuario No tiene un PEER configurado para Telefonia de Allegro. user: %s", info.member.Id)
						response.success = false
						response.reason = "ALLEGRO_MEMBER_DOES_NOT_HAVE_A_PHONE_CONFIGURED"
						return false
					end
				end
				return CanTransferInSynchronousChannels()
			end
		},
		FACEBOOK   = {enabled = true, CanTransfer = function() return true end},
		INSTAGRAM  = {enabled = true, CanTransfer = function() return true end},
		YOUTUBE	   = {enabled = true, CanTransfer = function() return true end},
		APPSTORE   = {enabled = true, CanTransfer = function() return true end},
		GOOGLEPLAY = {enabled = true, CanTransfer = function() return true end},
		RCS        = {enabled = true, CanTransfer = function() return true end},
		IM         = {enabled = true, CanTransfer = function() return true end},
		MAIL       = {enabled = true, CanTransfer = function() return true end},
		SMS        = {enabled = true, CanTransfer = function() return true end},
		WHATSAPP   = {enabled = true, CanTransfer = function() return true end},
		TWITTER    = {enabled = true, CanTransfer = function() return true end},
		WEBCONTACT = {enabled = true, CanTransfer = function() return true end},
		VOICEMAIL  = {enabled = true, CanTransfer = function() return true end}
	}
	-- 5.1) Valida que el CANAL se encuentre dentro de los canales disponibles
	local availableChannel = availablesChannels[info.channel]
	if not availableChannel or not availableChannel.enabled then
		--LogDebug("[INTERCTIONS:IsMemberEnabledForTransferStep2] 5 - El canal %s no soporta transferencias. user: %s", info.channel, info.member.Id)
		response.success = false
		response.reason = response.reason or "CHANNEL_DOES_NOT_SUPPORTED_TRANSFERS_TO_MEMBERS"
		return response
	end
	-- 5.2) Valida si la interaccion del CANAL puede ser transferida
	if not availableChannel.CanTransfer() then
		return response
	end
	------------------------------------------------
	-- 6) Se valida la capability del Usuario para el CANAL
	local isAvailable = utils.AvailableUserCapabilityForChannel(info.member, info.channel, info.campaignId, models)
	if not isAvailable then
		--LogDebug("[INTERCTIONS:IsMemberEnabledForTransferStep2] 4 - El Usuario No tiene Capability. user: %s", info.member.Id)
		response.success = false
		response.reason = "MEMBER_HAS_NOT_AVAILABLE_CAPABILITY_FOR_CHANNEL"
		return response
	end

	-- Ok - Paso 2
	return response
end

local function CanTransferToMember (info)
	local response = IsMemberEnabledForTransferStep1(info)
	if not response.success then
		return response
	end
	response = IsMemberEnabledForTransferStep2(info)
	return response
end

local function GetCampaignUsers(web)
	local request = web.request
	local vcc = inconcert.authUser.virtualContactCenter
	local campaignAtVCC = request.campaignAtVCC
	local interactionPermissions = request.interactionPermissions

	local users = GetModel("Interactions").GetCampaignUsers(vcc, campaignAtVCC, interactionPermissions)
	return { result = "OK", users = users }
end

local function GetMembersForTransfer (web)
	local request = web.request
	local appId = request.appId or "interaction"
	local batchInteractions

	if type(request.interactionID) ~= "table" then
		batchInteractions = {request.interactionID}
	else
		batchInteractions = request.interactionID
	end

	local startTime = os.time()
	local user_name = inconcert.authUser.userName
	LogInfo("[GetMembersForTransfer] (START) Retrieving agents to transfer. By User:'%s', Interactions: '%s'.", user_name, json.encode(batchInteractions))

	local ks_agent = utils.GetDatasource():CassandraConnection( "Agent" )
	local ks_interactions = utils.GetDatasource():CassandraConnection( "Interactions" )

	local channel
	-- Validacion de las interacciones del batch
	local function ValidateBatchInteractions ()
		local response = { success = false }
		local channels = {
			CALL       = { count = 0 },
			CHAT       = { count = 0 },
			FACEBOOK   = { count = 0 },
			INSTAGRAM  = { count = 0 },
			YOUTUBE	   = { count = 0 },
			APPSTORE   = { count = 0 },
			GOOGLEPLAY = { count = 0 },
			RCS		   = { count = 0 },
			IM         = { count = 0 },
			MAIL       = { count = 0 },
			SMS        = { count = 0 },
			WHATSAPP   = { count = 0 },
			TWITTER    = { count = 0 },
			WEBCONTACT = { count = 0 },
			VOICEMAIL  = { count = 0 }
		}
		local current_campaign
		local modelInteractions = GetModel("Interactions")
		for current_interaction_index, inter_id in ipairs(batchInteractions) do
			local inter = modelInteractions.GetInteraction(inter_id, ks_interactions)
			-- si la interaccion no existe falla
			if utils.TableIsEmpty(inter) then
				response.error = "INTERACTION_NOT_FOUND"
				response.msg = LogError("[GetMembersForTransfer] Unable transfer intraction '%s' does not exist.", inter_id)
				return response
			end

			batchInteractions[ current_interaction_index ] = inter -- reemplazo el id de la lista por la interaccion 

			-- Valido que las interacciones pertenezcan a la misma campana
			if current_campaign ~= nil and current_campaign ~= inter.campaign then
				response.error = "UNABLE_TRANSFER_DIFFERENTS_CAMPAIGNS"
				response.msg = LogError("[GetMembersForTransfer] Unable transfer between differents campaigns. Campaigns '%s' and '%s'.", current_campaign, inter.campaign)
				return response
			end
			current_campaign = inter.campaign

			-- Valido que el channel sea un channel conocido
			if not channels[ inter.type ] then
				response.error = "UNRECOGNIZED_INTERACTION_TYPE"
				response.msg = LogError("[GetMembersForTransfer] Unrecognized interaction type '%s', for interaction: '%s'.", inter.type, inter.id)
				return response
			end

			-- Valido que las interacciones correspondan todas al mismo channel
			channels[ inter.type ].count = channels[ inter.type ].count + 1
			if channels[ inter.type ].count < current_interaction_index then
				response.error = "UNABLE_TRANSFER_DIFFERENTS_CHANNELS"
				response.msg = LogError("[GetMembersForTransfer] Unable transfer between differents channels. Channels info: '%s'.", json.encode(channels))
				return response
			end

			channel = channel or inter.type
		end

		response.success = true
		return response
	end

	local validationResult = ValidateBatchInteractions()
	ks_interactions:Close()
	if not validationResult.success then
		ks_agent:Close()
		return {}
	end

	-- la variable que se retornara con el resultado al lado del cliente.
	local membersToTransfer = {}
	-- 1) obtengo la primera interaccion del array de interacciones (batch) para procesarla
	local firstInteraction = batchInteractions[1]
	table.remove(batchInteractions, 1) -- la elimino del batch para luego procesar las restantes interacciones del batch (ya que se tratan distintas)
	if not firstInteraction then
		ks_agent:Close()
		LogError("[GetMembersForTransfer] Interaction does not exist. bye bye")
		return membersToTransfer
	end

	local campaignMembers = GetModel("Campaigns").GetCampaignMembers(firstInteraction.campaign, ks_agent) or {}
	LogInfo("[GetMembersForTransfer] Verifying users (enabled, etc) agents: %d",  utils.GetRowCount(campaignMembers))

	-- callback que lo invoca el modelo de usuario para determinar las columnas que se incluiran en el member
	local CanAddMemberColumnFn = function(columnName)
		local allowedColumns = {Name = true, FirstSurname = true, Enabled = true, IsSystemUser = true, AllegroIntegration = true, UserGroup = true, Id = true}
		return allowedColumns[columnName]
	end

	local loggedUserId = utils.GetCurrentLoggedUser():lower()
	-- 3) Recorro la lista de agentes devueltos por el AS en (2), y valido si los usuarios estan Enable, si tienen permisos sobre la app "interaction".

	local hasCache, expireInSec = GetModel("Users").ExistUsersCache(inconcert.authUser.virtualContactCenter)
	if hasCache then
		LogInfo("[GetMembersForTransfer] + Loading Users Info From CACHE. Exipre in %s seconds", expireInSec)
	else
		LogInfo("[GetMembersForTransfer] - Loading Users Info From DATABASE")
	end

	-- obtengo los users statuses
	local vcc = inconcert.authUser.virtualContactCenter
	-- obtengo los applications actions a utilizar en la validacion de permisos, para que se mas perfomance
	local applicationsActions = permissions.GetApplicationsActions(ks_agent, vcc)

	local function MustIncludeUserStep1 (fullUserId, user)
		local info = {
			--interactionId = firstInteraction.id,
			target              = {user = fullUserId},
			appId               = appId,
			campaignId          = firstInteraction.campaign,
			member              = user,
			channel             = channel,
			ksAgent             = ks_agent,
			applicationsActions = applicationsActions
		}
		local canTransfer = IsMemberEnabledForTransferStep1(info)
		--LogDebug("[GetMembersForTransfer-> MustIncludeUser] ---> info: '%s'canTransfer: '%s'", json.encode(info), json.encode(canTransfer))
		if canTransfer.success then
			--LogDebug("[GetMembersForTransfer-> MustIncludeUser] ---> info: '%s'canTransfer: '%s'", json.encode(info), json.encode(canTransfer))
			return true
		end
		return false
	end

	local usersStep1 = {}
	for fullUserId in pairs(campaignMembers) do
		local mustLoadFromCache = true
		local user = GetModel("Users").Get(fullUserId, CanAddMemberColumnFn, ks_agent, mustLoadFromCache)
		user.VCC = vcc
		--LogDebug("--> user: %s", json.encode(user or {}))
		if MustIncludeUserStep1(fullUserId, user) then
			user.__fullUserId = fullUserId
			table.insert(usersStep1, user)
		end
	end
	--LogDebug("[GetMembersForTransfer-> MustIncludeUser] 1 ---> Usuarios que han pasado la primera validacion. usersStep1: '%s'", json.encode(usersStep1))
	--LogDebug("[GetMembersForTransfer-> MustIncludeUser] A ---> Un total de %s Usuarios han pasado la primera validacion.", #usersStep1)

	local isLiveInteraction = utils.InteractionIsLive(firstInteraction.type)
	local usersStatuses

	if isLiveInteraction or request.agentStatus then
		usersStatuses = agentservice.GetAgentsStatuses(usersStep1, vcc)
	end

	--LogDebug("[GetMembersForTransfer-> MustIncludeUser] 2 --> El status contra el AS de los Usuarios que han pasado la primera validacion. IsLiveInteraction: '%s' usersStatuses: '%s'", isLiveInteraction, json.encode(usersStatuses or {}))
	--LogDebug("[GetMembersForTransfer-> MustIncludeUser] B ---> Se ha terminado la validacion del estado de los Usuarios contra el AS.")

	local function MustIncludeUserStep2 (fullUserId, user)
		local info = {
			--interactionId = firstInteraction.id,
			target        = {user = fullUserId},
			appId         = appId,
			campaignId    = firstInteraction.campaign,
			member        = user,
			channel       = channel,
			ksAgent       = ks_agent,
			usersStatuses = isLiveInteraction and usersStatuses or nil
		}
		local canTransfer = IsMemberEnabledForTransferStep2(info)
		--LogDebug("[GetMembersForTransfer-> MustIncludeUser] ---> info: '%s'canTransfer: '%s'", json.encode(info), json.encode(canTransfer))
		if canTransfer.success then
			--LogDebug("[GetMembersForTransfer-> MustIncludeUser] ---> info: '%s'canTransfer: '%s'", json.encode(info), json.encode(canTransfer))
			return true
		end
		return false
	end

	local membersALs = nil
	local function GetMemberAttentionLevels(userId)
		--1. fetch members als
		if not membersALs then
			membersALs = {}
			--1.1 campaign als
			local campaignALs = GetModel("Campaigns").GetAttentionLevels(firstInteraction.campaign, ks_agent)
			if utils.TableIsEmpty(campaignALs) then
				return nil
			end
			--1.2 account als
			local account = GetModel("Campaigns").GetAccount(firstInteraction.account, firstInteraction.campaign, ks_agent)
			local accountALs = account and account.AttentionLevels
			if utils.TableIsEmpty(accountALs) then
				return nil
			end
			--1.3 make members als
			for alId, accountAl in pairs(accountALs) do
				local al = campaignALs[alId]
				local al_j = { id = al.id, name = al.name, distribution = accountAl.distribution }
				for alUserId, _ in pairs(al.users) do
					membersALs[alUserId] = membersALs[alUserId] or {}
					table.insert(membersALs[alUserId], al_j)
				end
			end
		end
		--2. return member als
		return membersALs[userId]
	end

	for _, user in ipairs(usersStep1) do
		if MustIncludeUserStep2(user.__fullUserId, user) then
			local simpleUserId = helpers.SeparateVCC(user.__fullUserId)
			local attentionLevels = GetMemberAttentionLevels(simpleUserId)
			local member = {
				Id = simpleUserId,
				Name = user.Name,
				FirstSurname = user.FirstSurname,
				AttentionLevels = attentionLevels
			}
			
			if usersStatuses
			then
				local agentStatus = usersStatuses[user.__fullUserId]
				if type(agentStatus) == "table" and type(agentStatus.state) == "string" then
					local status    = utf8.upper(agentStatus.state)
					member.IsOnline = (status == "ACTIVE" or status == "NOACD") and true or false
				end
			end

			membersToTransfer[user.__fullUserId] = member
		end
	end
	--LogDebug("[GetMembersForTransfer-> MustIncludeUser] 3 ---> Usuarios que han pasado la segunda validacion. membersToTransfer: '%s'", json.encode(membersToTransfer))

	ks_agent:Close()

	local endTime = os.time()
	local elapsedTime = endTime - startTime

	LogInfo("[GetMembersForTransfer] (END) Retrieving '%d' agents to transfer. By User:'%s', ELAPSED Time '%s' seconds.", utils.GetRowCount(membersToTransfer), user_name, elapsedTime)

	return membersToTransfer
end

local function has_configured_att_levels (account)
	if not account then
		return false
	end
	for _, att_level in pairs(account.AttentionLevels or {}) do
		return true
	end
	return false
end

local function CanTransferToCampaign (interaction, app_id)
	local response = { success = true }

	if not interaction then
		response.success = false
		response.reason = "INTERACTION_DOES_NOT_EXIST"
		return response
	end
	local availables_channels = {
		CALL       = true,
		CHAT       = true,
		FACEBOOK   = false,
		INSTAGRAM  = false,
		YOUTUBE    = false,
		APPSTORE   = false,
		GOOGLEPLAY = false,
		IM         = false,
		MAIL       = true,
		SMS        = true,
		WHATSAPP   = true,
		RCS		   = true,
		TWITTER    = false,
		WEBCONTACT = false,
		VOICEMAIL  = false
	}
	if not availables_channels[interaction.type] then
		response.success = false
		response.reason = "UNAVAILABLE_CHANNEL_TO_TRANSFER"
		return response
	end
	if app_id == "supervisor" and interaction.state == "WRAPUP" then
		response.success = false
		response.reason = "UNABLE_TO_TRANSFER_INTERACTION_IN_WRAPUP_STATE"
		return response
	end

	return response
end

local function IsCampaignAccountForTransfer(transferrerAccountId, transferrerType, accountId, account, campaignActive)
	--la cuenta no es la misma que la actual
	if transferrerAccountId == accountId then
		return false
	end
	--la cuenta esta activa
	if account.state ~= "1" then
		return false
	end
	--la cuenta es del mismo canal
	if account.proxyType ~= transferrerType then
		return false
	end
	if account.proxyType == "CALL" and (not account.incomingConfig or not account.incomingConfig.enabled) then
		return false
	end
	--cuenta de chat inactiva sin webcontact
	if account.proxyType == "CHAT" and not campaignActive and account.outHours_enabled ~= "true" then
		return false
	end

	return true
end

local function GetCampaignsAccountsForTransfer (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local user_name = inconcert.authUser.userName
	local request = web.request
	local batchInteractions

	if type(request.interactionID) ~= "table" then
		batchInteractions = {request.interactionID}
	else
		batchInteractions = request.interactionID
	end

	LogInfo("[GetCampaignsAccountsForTransfer] (START) Retrieving campaigns accounts to transfer. By App: '%s', User:'%s', Interactions: '%s'.", request.app_id, user_name, json.encode(batchInteractions))

	local ks_agent = utils.GetDatasource():CassandraConnection( "Agent" )
	local ks_interactions = utils.GetDatasource():CassandraConnection( "Interactions" )

	local campaigns_accounts = {}
	-- Todas las cuentas de todas las campanas del vcc
	local vcc_accounts = GetModel("Campaigns").GetVccAccounts(vcc, ks_agent)
	if not vcc_accounts then
		ks_agent:Close()
		ks_interactions:Close()
		LogError("[GetCampaignsAccountsForTransfer] (END) Not exist accounts on campaigns")
		return campaigns_accounts
	end
--

	local campaigns_attention_levels = {}
	local function get_campaign_att_level_definition ( campaign_id, att_level_id )
		campaigns_attention_levels[campaign_id] = campaigns_attention_levels[campaign_id] or GetModel("Campaigns").GetAttentionLevels(campaign_id, ks_agent)
		return campaigns_attention_levels[campaign_id][att_level_id] or {}
	end

	-- ahora recorro las interacciones a transferir
	local count = 0
	local modelInteractions = GetModel("Interactions")
	for _, interactionID in ipairs(batchInteractions) do
		local interaction = modelInteractions.GetInteraction(interactionID, ks_interactions)

		if utils.TableIsEmpty(interaction) then
			LogError("[GetCampaignsAccountsForTransfer] '%s' Interaction does not exist.", interactionID)
		else
			local can_transfer = CanTransferToCampaign(interaction, request.app_id)
			if not can_transfer.success then
				count = 0
				campaigns_accounts = {}
				LogError("[GetCampaignsAccountsForTransfer] Transfer disabled. Reason: '%s', interaction id: '%s', type: '%s', state: '%s'.",
					can_transfer.reason,
					interactionID,
					interaction.type,
					interaction.state
				)
				break
			end
			local campaign_id = interaction.campaign
			local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)

			for current_campaign, current_accounts in pairs( vcc_accounts ) do
				-- recorro todas las accounts
				for full_acc_id, current_account in pairs( current_accounts ) do
					local campaign_active = true
					--checks only for chat
					if current_account.proxyType == "CHAT" then
						campaign_active = irLink:IsCampaignActive({ proxyType = current_account.proxyType, campaignAtVCC = current_campaign, accountId =  full_acc_id}).status
					end
					if IsCampaignAccountForTransfer(interaction.account, interaction.type, full_acc_id, current_account, campaign_active) then
						campaigns_accounts[current_campaign] = campaigns_accounts[current_campaign] or {}
						campaigns_accounts[current_campaign].accounts = campaigns_accounts[current_campaign].accounts or {}
						campaigns_accounts[current_campaign].accounts[full_acc_id] = current_account
						count = count + 1

						current_account.info = {}
						if has_configured_att_levels(current_account) then
							-- agrego la descripcion(name) del attention level para que pueda ser visualizado en el cliente.
							-- busco x el id
							for att_level_id, att_level in pairs(current_account.AttentionLevels) do
								local att_level_definition = get_campaign_att_level_definition(current_campaign, att_level_id)
								current_account.AttentionLevels[att_level_id].name = att_level_definition.name or ""
								if att_level.distribution then
									current_account.info.is_default_attention_level = att_level.distribution
									current_account.info.attention_level_name = att_level_definition.name
									--break
								end
							end
						end
					end
				end
			end
		end
	end
	ks_agent:Close()
	ks_interactions:Close()

	LogInfo("[GetCampaignsAccountsForTransfer] (END) Retrieving campaign accounts to transfer. By User:'%s', campaign accounts: '%d'.", user_name, count)

	return campaigns_accounts
end

local function GetAccountsAttLevelsForTransfer(web)

	local vcc = inconcert.authUser.virtualContactCenter
	local user_name = inconcert.authUser.userName
	local request = web.request

	local batchInteractions

	if type(request.interactionID) ~= "table" then
		batchInteractions = {request.interactionID}
	else
		batchInteractions = request.interactionID
	end

	LogInfo("[GetAccountsAttLevelsForTransfer] (START) Retrieving accounts attention levels to transfer. By App: '%s', User:'%s', Interactions: '%s'.", request.app_id, user_name, json.encode(batchInteractions))

	local ks_agent = utils.GetDatasource():CassandraConnection( "Agent" )
	local ks_interactions = utils.GetDatasource():CassandraConnection( "Interactions" )

	-- Todas las cuentas de todas las campanas del vcc
	local vcc_accounts = GetModel("Campaigns").GetVccAccounts(vcc, ks_agent)
	if not vcc_accounts then
		ks_agent:Close()
		ks_interactions:Close()
		LogError("[GetAccountsAttLevelsForTransfer] (END) Not exist accounts on campaigns")
		return {}
	end

	local campaigns_attention_levels = {}
	local function get_campaign_att_level_definition ( campaign_id, att_level_id )
		campaigns_attention_levels[campaign_id] = campaigns_attention_levels[campaign_id] or GetModel("Campaigns").GetAttentionLevels(campaign_id, ks_agent)
		return campaigns_attention_levels[campaign_id][att_level_id]
	end

	-- Filter interactions that don't exist
	local modelInteractions = GetModel("Interactions")
	local interactions = {}
	for _, interaction_id in ipairs(batchInteractions) do
		local interaction = modelInteractions.GetInteraction(interaction_id, ks_interactions)
		if not utils.TableIsEmpty(interaction) then
			table.insert(interactions, interaction)
		end
	end

	if #interactions == 0 then
		ks_agent:Close()
		ks_interactions:Close()
		LogInfo("[GetAccountsAttLevelsForTransfer] No interactions to process")
		return {}
	end

	-- Validacion
	for _, interaction in ipairs(interactions) do
		-- 1- si la interacciones a transferir son de diferente proxy, fallo!!!
		if interaction.type ~= interactions[1].type then
			ks_agent:Close()
			ks_interactions:Close()
			LogError("[GetAccountsAttLevelsForTransfer] Unavailable transfer to attention levels with different proxies. Interaction id: '%s', proxies: '%s' and '%s'.",
				interaction.id, interaction.type, interactions[1].type)
			return {}
		end
		-- 2- si la interacciones a transferir son de diferente account, fallo!!!
		if interaction.account ~= interactions[1].account then
			ks_agent:Close()
			ks_interactions:Close()
			LogError("[GetAccountsAttLevelsForTransfer] Unavailable transfer to attention levels with different accounts. Interaction id: '%s', accounts: '%s' and '%s'.",
				interaction.id, interaction.account, interactions[1].account)
			return {}
		end
		-- 3- si la transferencia se realiza desde la app "supervisor" y la interaccion se encuentra en estado wrapup, fallo
		if request.app_id == "supervisor" and interaction.state == "WRAPUP" then
			ks_agent:Close()
			ks_interactions:Close()
			LogError("[GetAccountsAttLevelsForTransfer] Unavailable transfer to attention levels in app: '%s'. Interaction id: '%s', type: '%s', state: '%s'.",
				request.app_id,
				interaction.id,
				interaction.type,
				interaction.state
			)
			return {}
		end
	end

	local count = 0
	local campaigns_accounts = {}

	for _, interaction in ipairs(interactions) do

		local campaign_id = interaction.campaign
		local account_id = interaction.account
		local account = GetModel("Campaigns").FindAccountById(
			{
				proxyType = interaction.type,
				id = account_id
			},
			ks_agent
		)
		local interaction_att_level_id = interaction.attention_level and interaction.attention_level.id

		if has_configured_att_levels(account) then
			-- agrego la descripcion(name) del attention level para que pueda ser visualizado en el cliente.
			-- busco x el id
			for att_level_id, att_level in pairs(account.AttentionLevels) do

				if interaction_att_level_id == att_level_id then
					-- No es posible transferir la interaccion al mismo AL,
					--  entonces elimino el AL de la lista de accounts a retornar en el request
					account.AttentionLevels[att_level_id] = nil
				else
					local att_level_definition = get_campaign_att_level_definition(campaign_id, att_level_id)
					if att_level_definition then
						att_level.name = att_level_definition.name -- descripcion(name) del att level
					end
				end
			end
			campaigns_accounts[campaign_id] = campaigns_accounts[campaign_id] or {}
			campaigns_accounts[campaign_id].accounts = campaigns_accounts[campaign_id].accounts or {}
			campaigns_accounts[campaign_id].accounts[account_id] = account
			count = count + 1
		end
	end

	ks_agent:Close()
	ks_interactions:Close()

	LogInfo("[GetAccountsAttLevelsForTransfer] (END) Retrieving accounts attention levels to transfer. By User:'%s', attention levels: '%d'.", user_name, count)

	return campaigns_accounts
end

local function GetAccountsBotsForTransfer(web)

	local vcc = inconcert.authUser.virtualContactCenter
	local user_name = inconcert.authUser.userName
	local request = web.request

	local batchInteractions

	if type(request.interactionID) ~= "table" then
		batchInteractions = {request.interactionID}
	else
		batchInteractions = request.interactionID
	end

	LogInfo("[GetAccountsBotsForTransfer] (START) Retrieving accounts bots to transfer. By App: '%s', User:'%s', Interactions: '%s'.", request.app_id, user_name, json.encode(batchInteractions))

	local ks_agent = utils.GetDatasource():CassandraConnection( "Agent" )
	local ks_interactions = utils.GetDatasource():CassandraConnection( "Interactions" )

	-- Filter interactions that don't exist
	local modelInteractions = GetModel("Interactions")
	local interactions = {}
	for _, interaction_id in ipairs(batchInteractions) do
		local interaction = modelInteractions.GetInteraction(interaction_id, ks_interactions)
		if not utils.TableIsEmpty(interaction) then
			table.insert(interactions, interaction)
		end
	end

	if #interactions == 0 then
		ks_agent:Close()
		ks_interactions:Close()
		LogInfo("[GetAccountsBotsForTransfer] No interactions to process")
		return {}
	end

	-- Validacion
	for _, interaction in ipairs(interactions) do
		-- 1- si la interacciones a transferir son de diferente proxy, fallo!!!
		if interaction.type ~= interactions[1].type then
			ks_agent:Close()
			ks_interactions:Close()
			LogError("[GetAccountsBotsForTransfer] Unavailable transfer to bots with different proxies. Interaction id: '%s', proxies: '%s' and '%s'.",
				interaction.id, interaction.type, interactions[1].type)
			return {}
		end
		-- 2- si la interacciones a transferir son de diferente account, fallo!!!
		if interaction.account ~= interactions[1].account then
			ks_agent:Close()
			ks_interactions:Close()
			LogError("[GetAccountsBotsForTransfer] Unavailable transfer to bots with different accounts. Interaction id: '%s', accounts: '%s' and '%s'.",
				interaction.id, interaction.account, interactions[1].account)
			return {}
		end
		-- 3- si la transferencia se realiza desde la app "supervisor" y la interaccion se encuentra en estado wrapup, fallo
		if request.app_id == "supervisor" and interaction.state == "WRAPUP" then
			ks_agent:Close()
			ks_interactions:Close()
			LogError("[GetAccountsBotsForTransfer] Unavailable transfer to bots in app: '%s'. Interaction id: '%s', type: '%s', state: '%s'.",
				request.app_id,
				interaction.id,
				interaction.type,
				interaction.state
			)
			return {}
		end
	end
	
	local account = GetModel("Campaigns").GetAccount(interactions[1].account, interactions[1].campaign, ks_agent)
	local account_bots = account.BotsToTransferFromAgent and type(account.BotsToTransferFromAgent) == "string" and json.decode(account.BotsToTransferFromAgent) or {}
	local _, modelProcessFlows = utils.GetControllerOthersApps("flow_designer", {controllerName = "process_flows", modelName = "ProcessFlows"})
	for _, account_bot in pairs(account_bots) do
		local flow = modelProcessFlows.GetFlows(vcc, account_bot.id, {"Id", "Name", "Description"}, ks_agent)
		account_bot.name = flow.Name
		account_bot.description = flow.Description
	end
	
	ks_agent:Close()
	ks_interactions:Close()

	LogInfo("[GetAccountsBotsForTransfer] (END) Retrieving accounts bots to transfer. By User:'%s', bots: '%d'.", user_name, #account_bots)

	return account_bots
end

---
--
local function TransferInteractions (web)
	local address
	local user_name = inconcert.authUser.userName
	LogInfo("[AGENT:TransferInteractions] Transfer conversations from user:%s, request: %s.", user_name, web.input["post_data"])

	local success, transferRequest, interactions = ValidateBatchRequest(web, "interactions")
	if not success then
		return transferRequest
	end

	local function ErrResponse (err)
		web.status = 404
		return ReplyJSON(web, {result = err.error_code, reason = err.error_message})
	end

	local modelInteractions = GetModel("Interactions")

	-- Validate structure of "transfer" target
	local target = transferRequest.target
	if type(target) ~= "table" then
		local error_message = LogError("[AGENT:TransferInteractions] Unable to transfer. Target info is not a table '%s'.", target)
		return ErrResponse({ error_code = "INVALID_REQUEST", error_message = error_message })
	end

	-- Check that the transfer type is supported
	local supported_transfer = { attention_level = true, campaign = true, member = true, number = true, bot = true }
	if not supported_transfer[target.to] then
		local error_message = LogError("[AGENT:TransferInteractions] Unable to transfer. Unrecognized target transfer to '%s'.", target.to)
		return ErrResponse({ error_code = "TRANSFER_TYPE_NOT_SUPPORTED", error_message = error_message })
	end

	interactions = RemoveInteractionsFromSameGroupInBatch(interactions)
	local app_id = transferRequest.app_id -- optional

	local success, dispositionInfo
	local mustFinalizeWithDispositionCode = false -- flag utilizado para saber si tenemos que finalizar la interaccion con un Dispositon Code

	if target.to == "number" then
		if target.number then
			address = target.number
		else
			local error_message = LogError("[AGENT:TransferInteractions] Unable to transfer. Missing target.number.")
			return ErrResponse({ error_code = "UNABLE_TO_TRANSFER", error_message = error_message })
		end
	end
	if target.to == "member" then
		local firstInteraction = interactions[1]
		local canTransfer = CanTransferToMember({
			interactionId = firstInteraction.id,
			target        = target,
			appId         = app_id
		})
		if not canTransfer.success then
			local error_message = LogError("[AGENT:TransferInteractions] Unable transfer to Member '%s'. Reason: '%s', interaction id: '%s', type: '%s', state: '%s'",
				target.user,
				canTransfer.reason,
				canTransfer.interaction.id,
				canTransfer.interaction.type,
				canTransfer.interaction.state
			)
			return ErrResponse({ error_code = canTransfer.reason, error_message = error_message })
		end

	elseif target.to == "campaign" then
	-- validaciones para tansferir a campana
		local firstInteraction
		for _, int in ipairs(interactions) do
			local interaction = modelInteractions.GetInteraction(int.id)
			if utils.TableIsEmpty(interaction) then
				local error_message = LogError("[AGENT:TransferInteractions] Unable transfer to campaign. '%s' interaction does not exist.", int.id)
				return ErrResponse({ error_code = "INTERACTION_NOT_FOUND", error_message = error_message })
			else
				firstInteraction = firstInteraction or interaction
				local canTransfer = CanTransferToCampaign(interaction, app_id)
				if not canTransfer.success then
					local error_message = LogError("[AGENT:TransferInteractions] Unable transfer to campaign. Reason: '%s', interaction id: '%s', type: '%s', state: '%s'",
						canTransfer.reason,
						int.id,
						interaction.type,
						interaction.state
					)
					return ErrResponse({ error_code = canTransfer.reason, error_message = error_message })
				end
			end
		end

		------------------------------------------------------------------------------------------
		-- proxies asincronicos que admiten disposiciones de finalizacion al transferir x campaña
		--   existe la misma validacion en:
		--             1 - (AGENT) /agent/scripts/transfer_popup.js  - en la funcion para transferir (proxies_that_support_dispositions_when_transferring)
		--             2 - (IR)    /scripts/IR/chatscripts.lua       - en la funcion para transferir (proxiesThatSupportDispositionsWhenTransferring)
		local proxiesThatSupportDispositionsWhenTransferring = {
			MAIL     = true,
			SMS      = true,
			WHATSAPP = true,
			RCS 	 = true
		}
		if proxiesThatSupportDispositionsWhenTransferring[firstInteraction.type] then
			if transferRequest.dispositonInfo then
				mustFinalizeWithDispositionCode = true
			end
		end

		if mustFinalizeWithDispositionCode then
			transferRequest.dispositonInfo.campaignId = firstInteraction.campaign
			success, dispositionInfo = ValidateFinishInfo(transferRequest.dispositonInfo)
			if not success then
				local err = dispositionInfo
				return ErrResponse(err)
			end
		end

	elseif target.to == "attention_level" then
		assert(type(target.att_level_id) == "string")

		-- Make sure they all are of the same type
		local first_proxy, first_account
		for _, int in ipairs(interactions) do
			local interaction = modelInteractions.GetInteraction(int.id)
			if utils.TableIsEmpty(interaction) then
				local error_message = LogError("[AGENT:TransferInteractions] Unable transfer to attention levels. '%s' interaction does not exist.", int.id)
				return ErrResponse({ error_code = "INTERACTION_NOT_FOUND", error_message = error_message })
			end

			-- 1- si existien interacciones de diferente proxy, fallo!!
			first_proxy = first_proxy or interaction.type
			if interaction.type ~= first_proxy then
				local error_message = LogError("[AGENT:TransferInteractions] Unavailable transfer to attention levels with different proxies. Interaction id: '%s', proxies: '%s' and '%s'.", interaction.id, interaction.type, first_proxy)
				return false, { error_code = "UNABLE_TO_TRANSFER", error_message = error_message }
			end

			-- 2- si la interacciones a transferir son de diferente account, fallo!!!
			first_account = first_account or interaction.account
			if interaction.account ~= first_account then
				LogError("[AGENT:TransferInteractions] Unavailable transfer to attention levels with different accounts. Interaction id: '%s', accounts: '%s' and '%s'.", interaction.id, interaction.account, first_account)
				return ErrResponse({ error_code = "UNABLE_TO_TRANSFER", error_message = error_message })
			end

			-- 3- si la transferencia se realiza desde la app "supervisor" y la interaccion se encuentra en estado wrapup, fallo
			if app_id == "supervisor" and interaction.state == "WRAPUP" then
				LogError("[AGENT:TransferInteractions] Unavailable transfer to attention levels in app: '%s'. Interaction id: '%s', type: '%s', state: '%s'.",
					app_id,
					interaction.id,
					interaction.type,
					interaction.state
				)
				return ErrResponse({ error_code = "UNABLE_TO_TRANSFER", error_message = error_message })
			end

			-- 3- si el AL de la interaccion, es el mismo al cual se quiere transferir, fallo!!
			local interaction_att_level_id = interaction.attention_level and interaction.attention_level.id
			if interaction_att_level_id == target.att_level_id then
				local error_message = LogError("[AGENT:TransferInteractions] Cannot transfer the same level of attention. Interaction id: '%s', interaction attention level id: '%s', destination attention level id '%s'.",interaction.id, interaction_att_level_id, target.att_level_id)
				return ErrResponse({ error_code = "UNABLE_TO_TRANSFER", error_message = error_message })
			end
		end
		
	elseif target.to == "bot" then
		assert(type(target.bot_id) == "string")
		
		-- Make sure they all are of the same type
		local first_proxy, first_account
		for _, int in ipairs(interactions) do
			local interaction = modelInteractions.GetInteraction(int.id)
			if utils.TableIsEmpty(interaction) then
				local error_message = LogError("[AGENT:TransferInteractions] Unable transfer to bots. '%s' interaction does not exist.", int.id)
				return ErrResponse({ error_code = "INTERACTION_NOT_FOUND", error_message = error_message })
			end

			-- 1- si existien interacciones de diferente proxy, fallo!!
			first_proxy = first_proxy or interaction.type
			if interaction.type ~= first_proxy then
				local error_message = LogError("[AGENT:TransferInteractions] Unavailable transfer to bots with different proxies. Interaction id: '%s', proxies: '%s' and '%s'.", interaction.id, interaction.type, first_proxy)
				return false, { error_code = "UNABLE_TO_TRANSFER", error_message = error_message }
			end

			-- 2- si la interacciones a transferir son de diferente account, fallo!!!
			first_account = first_account or interaction.account
			if interaction.account ~= first_account then
				LogError("[AGENT:TransferInteractions] Unavailable transfer to bots with different accounts. Interaction id: '%s', accounts: '%s' and '%s'.", interaction.id, interaction.account, first_account)
				return ErrResponse({ error_code = "UNABLE_TO_TRANSFER", error_message = error_message })
			end

			-- 3- si la transferencia se realiza desde la app "supervisor" y la interaccion se encuentra en estado wrapup, fallo
			if app_id == "supervisor" and interaction.state == "WRAPUP" then
				LogError("[AGENT:TransferInteractions] Unavailable transfer to bots in app: '%s'. Interaction id: '%s', type: '%s', state: '%s'.",
					app_id,
					interaction.id,
					interaction.type,
					interaction.state
				)
				return ErrResponse({ error_code = "UNABLE_TO_TRANSFER", error_message = error_message })
			end
		end
	end

	-- create a table for the unsuccessful interactions
	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)

	return ProcessBatchData(web, "transfer", "interactions", interactions, function(interactionId)
		local currentInteraction = modelInteractions.GetInteraction(interactionId)
		if utils.TableIsEmpty(currentInteraction) then
			return false, { error_code = "INTERACTION_NOT_FOUND", error_message = ("Interaction %s does not exist"):format(interactionId) }
		end

		local action = "TRANSFER"
		if app_id == "supervisor" and (currentInteraction.state == "QUEUED" or currentInteraction.state == "TRANSFER QUEUED") then
			action = "TRANSFER_QUEUED"
		end
		LogInfo("Transfer app_id: '%s', currentInteraction.state: '%s', action: '%s'", app_id, currentInteraction.state, action)
		if not HasPermission( action, currentInteraction.campaign, app_id ) then
			return false, { error_code = "FORBIDDEN", error_message = "forbidden: "..web.input["post_data"] }
		end

		local transferInfo = {
			id          = currentInteraction.id,
			type        = currentInteraction.type,
			transferrer = utils.GetCurrentLoggedUser(),
			target      = target,
			address     = address,
			rid         = web.vars.HTTP_X_ICC_RID
		}

		if mustFinalizeWithDispositionCode then -- es opcional, y solo es valido para transferencias a campañas
			transferInfo.wrapupData = dispositionInfo.disposition
			transferInfo.wrapupData.comment = dispositionInfo.comment
		end

		LogInfo("[AGENT:TransferInteractions] Transfer conversations, parameters invoked to IR: '%s'.", json.encode(transferInfo))
		local result = irLink:InteractionReassign(transferInfo)
		if not result then
			return false, { error_code = "UNABLE_TO_TRANSFER", error_message = ("Unable to transfer interaction: %s"):format(currentInteraction.id) }
		end

		-- something is wrong
		if result["result"] ~= "OK" then
			return false, { error_code = result["result"], error_message = result["reason"], error_source = result }
		end

		-- everything looks good.
		return true
	end)
end
		
local function ConferenceInteractions (web)
	local address
	local user_name = inconcert.authUser.userName
	LogInfo("[AGENT:ConferenceInteractions] Conference conversations from user:%s, request: %s.", user_name, web.input["post_data"])

	local success, conferenceRequest, interactions = ValidateBatchRequest(web, "interactions")
	if not success then
		return conferenceRequest
	end

	local function ErrResponse (err)
		web.status = 404
		return ReplyJSON(web, {result = err.error_code, reason = err.error_message})
	end

	-- Validate structure of "conference" target
	local target = conferenceRequest.target
	if type(target) ~= "table" then
		local error_message = LogError("[AGENT:ConferenceInteractions] Unable to conference. Target info is not a table '%s'.", target)
		return ErrResponse({ error_code = "INVALID_REQUEST", error_message = error_message })
	end

	-- Check that the conference type is supported
	local supported_conference = { member = true, number = true }
	if not supported_conference[target.to] then
		local error_message = LogError("[AGENT:ConferenceInteractions] Unable to conference. Unrecognized target conference to '%s'.", target.to)
		return ErrResponse({ error_code = "CONFERENCE_TYPE_NOT_SUPPORTED", error_message = error_message })
	end

	interactions = RemoveInteractionsFromSameGroupInBatch(interactions)
	local app_id = conferenceRequest.app_id -- optional

	if target.to == "number" then
		if target.number then
			address = target.number
		else
			local error_message = LogError("[AGENT:ConferenceInteractions] Unable to conference. Missing target.number.")
			return ErrResponse({ error_code = "UNABLE_TO_CONFERENCE", error_message = error_message })
		end
	elseif target.to == "member" then
		local firstInteraction = interactions[1]
		local canConference = CanTransferToMember({
			interactionId = firstInteraction.id,
			target        = target,
			appId         = app_id
		})
		if not canConference.success then
			local error_message = LogError("[AGENT:ConferenceInteractions] Unable transfer to Member '%s'. Reason: '%s', interaction id: '%s', type: '%s', state: '%s'",
				target.user,
				canConference.reason,
				canConference.interaction.id,
				canConference.interaction.type,
				canConference.interaction.state
			)
			return ErrResponse({ error_code = canConference.reason, error_message = error_message })
		end
	end

	-- create a table for the unsuccessful interactions
	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)

	return ProcessBatchData(web, "conference", "interactions", interactions, function(interactionId)
		local currentInteraction = GetModel("Interactions").GetInteraction(interactionId)
		if utils.TableIsEmpty(currentInteraction) then
			return false, { error_code = "INTERACTION_NOT_FOUND", error_message = ("Interaction %s does not exist"):format(interactionId) }
		end

		local action = "CONFERENCE"
		LogInfo("Conference app_id: '%s', currentInteraction.state: '%s', action: '%s'", app_id, currentInteraction.state, action)
		if not HasPermission( action, currentInteraction.campaign, app_id ) then
			return false, { error_code = "FORBIDDEN", error_message = "forbidden: "..web.input["post_data"] }
		end

		local conferenceInfo = {
			id          = currentInteraction.id,
			type        = currentInteraction.type,
			transferrer = utils.GetCurrentLoggedUser(),
			target      = target,
			address     = address,
			rid         = web.vars.HTTP_X_ICC_RID
		}

		LogInfo("[AGENT:ConferenceInteractions] Conference conversations, parameters invoked to IR: '%s'.", json.encode(conferenceInfo))
		local result = irLink:InteractionConference(conferenceInfo)
		if not result then
			return false, { error_code = "UNABLE_TO_CONFERENCE", error_message = ("Unable to conference interaction: %s"):format(currentInteraction.id) }
		end

		-- something is wrong
		if result["result"] ~= "OK" then
			return false, { error_code = result["result"], error_message = result["reason"], error_source = result }
		end

		-- everything looks good.
		return true
	end)
end

---
--
local function IterativeTakeNextBest (agent_name, vcc, campaignId, interactionType, request_id, app_id, can_take_allegro)
	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local ksAgent = utils.GetDatasource():CassandraConnection("Agent")
	local ksInteractions = utils.GetDatasource():CassandraConnection("Interactions")
	-- Interactions we tried to take are stored here to avoid infinite looping on an 'untakable' interaction
	local take_attempts = {}
	local userCampaigns = {}
	if not campaignId then
		userCampaigns = GetModel("Campaigns").GetUserCampaigns(agent_name, vcc, ksAgent)
	else
		userCampaigns[campaignId] = ""
	end

	-- This process wil result in one of:
	--  * successfully takes an interaction
	--  * has no permission to take any interactions
	--  * can't take interactions because of busy slots and stuff
	-- if both "no permissions" and "can't take" conditions occur at the same time, we give priority to
	-- "can't take". You *could* have taken an interaction, but you're exceeding the limits.

	local ReplyError = function (cant_take, forbidden)
		ksAgent:Close()
		ksInteractions:Close()
		if cant_take then
			return false, "Can't take any interactions (limits exceeded)", "AGENT_UNAVAILABLE"
		elseif forbidden then
			return false, "No permission to take any queued interaction", "FORBIDDEN"
		else
			return false, "No queued interactions matching criteria", "NO_INTERACTIONS_FOUND"
		end
	end

	local ReplyOk = function (best)
		ksAgent:Close()
		ksInteractions:Close()
		return true, best
	end

	local err_forbidden, err_cant_take = false, false
	-- iterate till one interaction is taken or there is no more left
	local modelInteractions = GetModel("Interactions")
	while true do
		LogInfo("Finding next best interaction for user %s on campaign %s from type %s", agent_name, campaignId, interactionType)
		local bestList = {}

		--get for agent
		--TODO: filter by interactionType if we use it
		local agentBests = modelInteractions.GetAgentBests(vcc, agent_name, ksInteractions)
		for id, interaction in pairs(agentBests) do
			local userCampaign = userCampaigns[interaction.campaign]
			if not userCampaign then
				LogInfo("Interaction %s belongs to campaign %s not assigned to agent", interaction.id, interaction.campaign)
			elseif not can_take_allegro and GetModel("Campaigns").IsAllegroCampaign(vcc, userCampaign, ksAgent) then
				LogInfo("Campaign %s can not take more allegro interactions", interaction.campaign)
			elseif not take_attempts[interaction.id] then
				LogInfo("Best for agent found, campaign %s priority %s type %s date %s id %s",
					interaction.campaign, interaction.priority, interaction.type, interaction.timestamp, interaction.id)

				table.insert(bestList, {id = interaction.id, priority = interaction.priority,
										type = interaction.type, timestamp = interaction.timestamp,
										campaign = interaction.campaign, agent = 1})
				take_attempts[interaction.id] = interaction.id
			end
		end

		--get for campaign
		for campaign, role in pairs(userCampaigns) do
			LogInfo("Iterating on campaign %s", campaign)
			if not can_take_allegro and GetModel("Campaigns").IsAllegroCampaign(vcc, campaign, ksAgent) then
				LogInfo("Campaign %s can not take more allegro interactions", campaign)
			else
				local campaignBests = modelInteractions.GetCampaignBests(campaign, interactionType, ksInteractions)
				for id, interaction in pairs(campaignBests) do
					if not take_attempts[interaction.id] then
						LogInfo("Best for campaign %s found, priority %s type %s date %s id %s",
							campaign, interaction.priority, interaction.type, interaction.timestamp, interaction.id)

						table.insert(bestList, {id = interaction.id, priority = interaction.priority,
												type = interaction.type, timestamp = interaction.timestamp,
												campaign = campaign, agent = 0})
						take_attempts[interaction.id] = interaction.id
					end
				end
			end
		end
		if #bestList == 0 then
			return ReplyError(err_cant_take, err_forbidden)
		end
		--sort between all campaigns, first by agent then by priority then by queued date
		table.sort(bestList, function(a, b)
			if a.agent == b.agent then
				if a.priority == b.priority then
					return a.timestamp < b.timestamp
				else
					return a.priority > b.priority
				end
			else
				return a.agent > b.agent
			end
		end)
		--try to take queued request in order until one is succesfull

		for _, best in pairs(bestList) do
			LogInfo("Trying to take interaction '%s' of type '%s'", best.id, best.type)
			if not HasPermission( "TAKE", best.campaign, app_id ) then
				err_forbidden = true
			else
				local campaign_name = helpers.SeparateVCC(best.campaign)
				if agentservice.CanTakeInteraction(agent_name, vcc, best.type, best.id, campaign_name) then
					--allegro interactions are taken in js
					if utils.IsAllegroChannel(best.type) then
						return ReplyOk(best)
					end
					local result = irLink:InteractionGetFromQueue({
						id = best.id,
						agent = agent_name.."@"..vcc,
						type = best.type,
						rid = request_id,
						fromTakeNextBest = true
					})
					if result then
						-- Got an interaction, stop and return that
						return ReplyOk(best)
					end
				else
					err_cant_take = true
				end
			end
		end
	end

	return false, ReplyError(err_cant_take, err_forbidden)
end

function TakeNextBest (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local user_name = inconcert.authUser.userName
	local request_id = web.vars.HTTP_X_ICC_RID
	local request = web.request

	LogInfo("Take next best request, interactionType:%s campaign:%s from agent %s", request.interactionType or "",
		request.campaignId or "", user_name)

	local app_id = request.app_id -- optional
	if request.campaignId and request.interactionType then
		if not HasPermission( "TAKE", request.campaignId, app_id ) then
			web.status = 403
			return { result = "FORBIDDEN" }
		end
	end

	local campaignIdAtVCC = request.campaignId and helpers.AddVCC(request.campaignId, vcc)
	local result, taken_interaction, err = IterativeTakeNextBest(user_name, vcc, campaignIdAtVCC, request.interactionType, request_id, app_id, request.can_take_allegro )

	LogInfo("Take best ending, result: '%s' id: '%s', err: '%s'", result, result and taken_interaction.id or taken_interaction, err or "none")

	if result then
		return { interactionID = taken_interaction.id, type = taken_interaction.type }
	else
		web.status = (err == "NO_INTERACTIONS_FOUND") and 404 or 400
		return { result = err }
	end
end

local CanFinishInteractionByProxyType_d = {
	FACEBOOK 		= true,
	INSTAGRAM 		= true,
	YOUTUBE 		= true,
	APPSTORE 		= true,
	RCS 			= true,
}
local PENDING_EVENTS_TIMEOUT = 5 * 60
local function CanFinishInteractionByProxyType (interaction)
	if CanFinishInteractionByProxyType_d[interaction.type] then
		local total_pending_events = 0
		local now = os.time()
		for _, pending_event in pairs(interaction.pending_events or {}) do
			if pending_event.state == "SENDING" and now - pending_event.timestamp < PENDING_EVENTS_TIMEOUT then
				total_pending_events = total_pending_events + 1
			end
		end
		if total_pending_events > 0 then
			local errorMessage = LogError("[CanFinishInteractionByProxyType] Unable to finish Conversation '%s'. Conversation has %d sending events.", interaction.id, total_pending_events)
			local err = { error_code = "UNABLE_TO_FINISH_INTERACTION_WITH_PENDING_EVENTS", error_message = errorMessage, total_pending_events = total_pending_events }
			return false, err
		end
	end
	return true
end

local function ValidateInteractionToFinish (info)
	local interactionId = info.interactionId
	local campaignId = info.campaignId
	local app_id = info.app_id

	local currentInteraction = GetModel("Interactions").GetInteraction(interactionId)
	if utils.TableIsEmpty(currentInteraction) then
		local errorMessage = LogWarning("[INTERACTIONS:ValidateInteractionToFinish] Interaction %s does not exist", interactionId)
		return false, { error_code = "INTERACTION_NOT_FOUND", error_message = errorMessage }
	end
	if not HasPermission("FINISH", currentInteraction.campaign, app_id) then
		local errorMessage = LogWarning("[INTERACTIONS:ValidateInteractionToFinish] Fobidden archive interaction")
		return false, { error_code = "FORBIDDEN", error_message = errorMessage}
	end

	-- make sure we don't allow to finish interactions with contact conflicts
	if currentInteraction.interactionData and currentInteraction.interactionData.conflict  then
		local errorMessage = LogWarning("[INTERACTIONS:ValidateInteractionToFinish] Interaction needs to resolve the contact conflict before archiving")
		return false, { error_code = "PENDING_CONFLICT", error_message = errorMessage }
	end

	if campaignId ~= currentInteraction.campaign then
		local errorMessage = LogWarning("[INTERACTIONS:ValidateInteractionToFinish] Incorrect Request campaignId '%s'", campaignId)
		return false, { error_code = "INCORRECT_REQUEST_CAMPAIGN", error_message = errorMessage }
	end

	local success, err = CanFinishInteractionByProxyType(currentInteraction)
	if not success then
		return false, err
	end

	return true, {
		currentInteraction = currentInteraction
	}
end

local function FinishInteraction (info)
	local agentAtVCC = utils.GetCurrentLoggedUser() or info.fullUserId or ""
	local appId = info.app_id or "interaction"
	local currentInteraction = info.currentInteraction
	local postedInteraction = info.postedInteraction
	local disposition = info.disposition
	local campaignDisposition = info.campaignDisposition
	local currentComment = info.comment or ""
	local RID = info.RID

	LogInfo("[INTERACTIONS:FinishInteraction] User: '%s' Archiving Interaction. Interaction Id '%s', Disposition '%s', App: '%s'.",
		agentAtVCC,
		(currentInteraction or {}).id,
		(disposition or {}).name,
		appId
	)

	local successResponse = {warning = nil}

	if campaignDisposition and #currentComment > 0 then
		local hasPermissionToComment = HasPermission("COMMENT", currentInteraction.campaign, appId)
		if not hasPermissionToComment then
			LogWarning("[INTERACTIONS:FinishInteraction] Archive with disposition - You have no permissions to comment conversations on campaign '%s'.", currentInteraction.campaign)
			currentComment = ""
			successResponse.warning = { result = "FORBIDDEN_COMMENT", reason = "forbidden"}
		end
	end

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)

	-- Actualizo la informacion de finalizacion en la interaccion, cuando el agente tiene el control
	local agentFinalizationData = postedInteraction and postedInteraction.finalization_data
	if agentFinalizationData then
		local response = irLink:SendCommand({
			interactionID    = currentInteraction.id,
			RID              = RID,
			proxyType        = "AGENT",
			agentAtVCC       = agentAtVCC,
			command          = "AssignAgentFinalizationDataToInteraction",
			finalizationData = agentFinalizationData
		})
		if response.result ~= "OK" then
			LogWarning("[INTERACTIONS:FinishInteraction] Unable to assign agent finalization data to interaction '%s'. Reason: %s", currentInteraction.id, response.reason)
		end
	end

	-- Go and try to archive the interaction
	local archiveResult = irLink:InteractionArchive(currentInteraction.id, agentAtVCC, disposition, currentComment)
	if not archiveResult then
		return false, { error_code = "UNABLE_TO_ARCHIVE", error_message = ("Unable to archive interaction: '%s'"):format(currentInteraction.id) }
	end

	-- change contact state
	if campaignDisposition then
		local contactId = currentInteraction.contactId
		local newState  = campaignDisposition.changeContactState
		if newState and contactId and #contactId > 0 then
			Fullcontact.ChangeState(contactId, inconcert.authUser.virtualContactCenter, newState)
		end
	end

	return true, successResponse
end

local function FinishAssignedAgent (info)
	local agentAtVCC = utils.GetCurrentLoggedUser() or info.fullUserId or ""
	local currentInteraction = info.currentInteraction

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local response = irLink:SendCommand({
		interactionID    = currentInteraction.id,
		interaction      = currentInteraction,
		proxyType        = "CALL",
		agentAtVCC       = agentAtVCC,
		command          = "FinishAssignedAgent"
	})
	if response.result ~= "OK" then
		LogWarning("[INTERACTIONS:FinishAssignedAgent] Unable to unassign agent to interaction '%s'. Reason: %s", currentInteraction.id, response.reason)
	end
end

--[[
	Funcion que se encarga de finalizar las interacciones de CALL de i6 en estado de wrapup.
	Funcion que se encarga de desasignar los agente de las interacciones de CALL de i6 en estado de BOT
	La misma se invoca cuando se produce el logout del Agente.
]]--
function CleanupI6CallInteractions (vcc, userName, ksAgent)
	if utils.HasAllegroIntegration(vcc, ksAgent) then
		return
	end
	local fullUserId = utils.AppendVcc(userName, vcc)
	--LogDebug("[INTERACTIONS:CleanupI6CallInteractions] Clean Up Wrapped Call i6 Interactions for User '%s'.", fullUserId)
	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local ksInteractions = utils.GetDatasource():CassandraConnection("Interactions")
	local userInteractions = GetModel("Interactions").GetUserInteractions(vcc, userName, ksInteractions)
	ksInteractions:Close()
	for interactionId, interaction in pairs(userInteractions or {}) do
		if interaction.type == "CALL" and interaction.state == "WRAPUP" then
			LogInfo("[INTERACTIONS:CleanupI6CallInteractions] Clean Up Wrapped Call i6 Interaction '%s' for User '%s'.", interactionId, fullUserId)
			irLink:AbortInteraction({
				id = interaction.id,
				proxyType = interaction.type,
				enderSystem = "AgentLogout"
			})
		elseif interaction.type == "CALL" and interaction.state == "BOT" then
			LogInfo("[INTERACTIONS:CleanupI6CallInteractions] Unassign User '%s' for Call i6 Interaction '%s'.",fullUserId, interactionId)
			FinishAssignedAgent({
				currentInteraction = interaction,
				fullUserId = fullUserId
			})
		end
	end
end

--[[
	Funcion que se encarga de finalizar las interacciones de CALL y CHAT en estado de taken de un agente que quedaron rotas.
	La misma se invoca cuando se produce el login del Agente.
]]--
function CleanupInteractionsOnAgentStartup (vcc, userName, userInteractions)
	local fullUserId = utils.AppendVcc(userName, vcc)
	--LogDebug("[INTERACTIONS:CleanupInteractionsOnAgentStartup] Clean Up Wrapped Call i6 Interactions for User '%s'.", fullUserId)

	local function hasAgent(agents)
		for _, agent in ipairs(agents or {}) do
			if agent == fullUserId then
				return true
			end
		end
		return false
	end
	
	local mustCleanupInteraction = {
		CHAT = {			
			CanCleanup = function(state, assignedAgents, reservedAgents)
				return state == "TAKEN" and hasAgent(assignedAgents)
			end
		},
		CALL = {			
			CanCleanup = function(state, assignedAgents, reservedAgents)
				return (state == "LOCKED" and hasAgent(reservedAgents)) or (state == "TAKEN" and hasAgent(assignedAgents)) or (state == "WRAPUP" and hasAgent(assignedAgents) and utils.HasAllegroIntegration(vcc))
			end
		}
	}

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	for _, interaction in pairs(userInteractions or {}) do
		if mustCleanupInteraction[interaction.type] and mustCleanupInteraction[interaction.type].CanCleanup(interaction.state, interaction.assignedAgents, interaction.reservedAgents) then
			LogInfo("[INTERACTIONS:CleanupInteractionsOnAgentStartup] Clean Up %s %s Interaction '%s' for User '%s'.", interaction.state, interaction.type, interaction.id, fullUserId)
			irLink:SendCommand({
				command = "InteractionClean",
				proxyType = "AGENT",
				interactionID = interaction.id,
			})			
		end
	end
end

--- Archives/Finishes an interaction
--
function ArchiveInteractions (web)
	local success, request, interactions = ValidateBatchRequest(web, "interactions")
	if not success then
		-- did not pass interaction batch validation
		return request
	end

	local postedData = web.input["post_data"]
	local res, request = pcall(json.decode, postedData)

	--stored disposition
	local storedDisposition = false
	if (not request.disposition or request.disposition == "") and #request.interactions == 1 then
		local interactionId = request.interactions[1].id
		local data = GetModel("Interactions").GetDisposition(interactionId)
		if data ~= nil then
			storedDisposition = true
			request.disposition = data.disposition
			request.comment = data.comment
		end
	end

	local success, info = ValidateFinishInfo(request)
	if not success then
		web.status = 400
		local err = info
		return ReplyJSON(web, err)
	end

	interactions = RemoveInteractionsFromSameGroupInBatch(interactions)

	return ProcessBatchData(web, "archive", "interactions", interactions, function( interactionId, postedInteraction )
		info.interactionId = interactionId
		local success, response = ValidateInteractionToFinish(info)
		if not success then
			local err = response
			return false, err
		end

		info.rid = web.vars.HTTP_X_ICC_RID
		info.currentInteraction  = response.currentInteraction
		info.postedInteraction = postedInteraction

		local success, successResponse = FinishInteraction(info)

		if success and storedDisposition then
			GetModel("Interactions").RemoveDisposition(interactionId)
		end

		-- everything looks good.
		return success, successResponse
	end)
end

--- Set disposition to an open interaction
--
function SetDisposition (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local user_name = inconcert.authUser.userName
	local post_data =  web.input["post_data"]
	local res, request = pcall(json.decode, post_data)
	if not res or not request.interactionID or not request.disposition then
		web.status = 400
		LogWarning("Invalid request: %s", post_data)
		return ReplyJSON(web, {result = "INVALID_REQUEST", reason = "Invalid request: " .. tostring(post_data)})
	end

	GetModel("Interactions").SetDisposition(request.interactionID, {
		disposition = request.disposition,
		comment = request.comment or ""
	})

	return ReplyJSON(web, { status = true })
end

function TakeInteraction (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local user_name = inconcert.authUser.userName
	local request = web.request

	LogInfo("Take interaction request, id:%s type:%s, campaign:%s from agent %s",  request.id, request.type,
		request.campaign, user_name)

	local app_id = request.app_id -- optional

	if not HasPermission( "TAKE", request.campaign, app_id ) then
		web.status = 403
		return { error_code = "FORBIDDEN" }
	end
	local campaign_name = helpers.SeparateVCC(request.campaign)
	if not agentservice.CanTakeInteraction(user_name, vcc, request.type, request.id, campaign_name, web.vars.HTTP_X_ICC_RID) then
		LogError("Take interaction error: Agent '%s@%s' can't take interaction %s on campaign '%s' at the moment. Retry later.", user_name, vcc, request.id, request.campaign)

		-- As usual, error code handling needs to be rewritten
		-- AGENT_UNAVAILABLE means that this very agent is unavailable (busy slots, etc)
		-- The error code could be better, though
		local response = { result = "AGENT_UNAVAILABLE", reason = "interaction limit exceeded" }
		web.status = http_mapping.getHTTPCode(response["result"]) or 500
		return response
	end

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local result = irLink:InteractionGetFromQueue({
		id = request.id,
		agent = utils.GetCurrentLoggedUser(),
		type = request.type,
		rid = web.vars.HTTP_X_ICC_RID
	})

	LogInfo("Take interaction Result: '%s'. Agent '%s@%s' take interaction %s on campaign '%s'.", json.encode( result or {}), user_name, vcc, request.id, request.campaign)

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(result["result"])
	web.status = http_code or 500 --unknown result code

	return result
end

function TakeBotInteraction (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local user_name = inconcert.authUser.userName
	local request = web.request

	LogInfo("Take interaction request, id:%s type:%s, campaign:%s from agent %s",  request.id, request.type,
		request.campaign, user_name)

	local app_id = request.app_id

	if not HasPermission( "ASSISTANCE", request.campaign, app_id ) then
		web.status = 403
		return { error_code = "FORBIDDEN" }
	end
	local campaign_name = helpers.SeparateVCC(request.campaign)

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local result = irLink:InteractionGetFromQueueBotInteraction({
		id = request.id,
		agent = utils.GetCurrentLoggedUser(),
		type = request.type,
		rid = web.vars.HTTP_X_ICC_RID
	})

	LogInfo("Take interaction Result: '%s'. Agent '%s@%s' take interaction %s on campaign '%s'.", json.encode( result or {}), user_name, vcc, request.id, request.campaign)

	local http_code = http_mapping.getHTTPCode(result["result"])
	web.status = http_code or 500

	return result
end

-- creates an outgoing CALL interaction using call API
-- used when agent is configured with external or deskop phone
function CreateOutgoingCALL(web)
	local user_name = utils.GetCurrentLoggedUser()

	LogInfo("Create call conversation from agent:%s, request: %s.", user_name, web.input["post_data"])

	local res, callRequest = pcall(json.decode, web.input["post_data"])

	if not hasLicensesForChannel( "CALL" ) then
		local reason = LogFatal("Unable to create CALL outgoing conversation. Not has licensed for the CALL channel.")
		web.status = 500
		return ReplyJSON(web, {
				agent_err = true,
				result = "NOT_LICENSED",
				reason = reason
			})
	end

	if not res or not callRequest then
		web.status = 400
		return ReplyJSON(web, { result = "INVALID_REQUEST", reason = "Invalid request: ".. web.input["post_data"] })
	end

	if not HasPermission( "CREATE_OUTGOING", callRequest.campaign, "interaction" ) then
		web.status = 403
		return ReplyJSON( web, { result = "FORBIDDEN", reason = "You not have permission for create outgoing call conversation." })
	end

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	LogInfo("Agent %s is calling number %s on campaign %s", user_name, callRequest.number, callRequest.campaign)

	local result = irLink:CallCreate(callRequest.id, user_name, callRequest.campaign, callRequest.number)
	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(result["result"])
	web.status = http_code or 500 --unknown result code

	return ReplyJSON(web, result)
end

function CloseAgentLeg(web)
	local res, closeRequest = pcall(json.decode, web.input["post_data"])
	if not res or not closeRequest then
		web.status = 400
		return ReplyJSON(web, { result = "INVALID_REQUEST", reason = "Invalid request: ".. web.input["post_data"] })
	end

	local user_name = utils.GetCurrentLoggedUser()
	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	LogInfo("Closing %s' leg in interaction %s", user_name, closeRequest.id)

	local result = irLink:CallCloseAgentLeg(closeRequest.id, user_name)
	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(result["result"])
	web.status = http_code or 500 --unknown result code

	return ReplyJSON(web, result)
end

function StartAttendedTransfer (web)
	local res, transferRequest = pcall(json.decode, web.input["post_data"])
	if not res or not transferRequest then
		web.status = 400
		return json.encode({ result = "INVALID_REQUEST", reason = "Invalid request: ".. web.input["post_data"] })
	end

	local vcc = inconcert.authUser.virtualContactCenter
	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	LogInfo("StartAttendedTransfer called for id:%s from agent %s to %s", transferRequest.id, utils.GetCurrentLoggedUser(), transferRequest.phone or transferRequest.agent)

	local result = irLink:InteractionReassign({
		id = transferRequest.id,
		type = "CALL",
		transferrer = utils.GetCurrentLoggedUser(),
		target = {
			to = "member", user = utils.AppendVcc(transferRequest.agent, vcc)
		},
		rid = web.vars.HTTP_X_ICC_RID,
		-- CALL specific
		address = transferRequest.phone
	})
	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(result["result"])
	web.status = http_code or 500 --unknown result code

	return ReplyJSON(web, result)
end

function CommitAttendedTransfer(web)
	local res, transferRequest = pcall(json.decode, web.input["post_data"])
	if not res or not transferRequest then
		web.status = 400
		return json.encode({ result = "INVALID_REQUEST", reason = "Invalid request: ".. web.input["post_data"] })
	end

	LogInfo("CommitAttendedTransfer called for id:%s", transferRequest.id)

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local result = irLink:CallCommitAttendedTransfer(transferRequest.id);

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(result["result"])
	web.status = http_code or 500 --unknown result code

	return ReplyJSON(web, result)
end

function CancelAttendedTransfer(web)
	local res, transferRequest = pcall(json.decode, web.input["post_data"])
	if not res or not transferRequest then
		web.status = 400
		return json.encode({ result = "INVALID_REQUEST", reason = "Invalid request: ".. web.input["post_data"] })
	end

	LogInfo("CancelAttendedTransfer called for id:%s", transferRequest.id)

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local result = irLink:CallCancelAttendedTransfer(transferRequest.id);

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(result["result"])
	web.status = http_code or 500 --unknown result code

	return ReplyJSON(web, result)
end

function MonitorCall(web)
	local res, monitorRequest = pcall(json.decode, web.input["post_data"])
	if not res or not monitorRequest then
		web.status = 400
		return ReplyJSON (web, {
				result = "INVALID_REQUEST",
				reason = "Invalid request: ".. web.input["post_data"]
			})
	end

	if utils.HasAllegroIntegration() then
		return ReplyJSON(web, { result = "OK" })
	end

	local user_name = utils.GetCurrentLoggedUser()
	local vcc = inconcert.authUser.virtualContactCenter
	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	LogInfo("MonitorCall called for id:%s from agent %s", monitorRequest.id, user_name)

	local result = irLink:StartMonitor({
		proxyType = "CALL",
		agentAtVCC = user_name,
		interactionID = monitorRequest.id,
		data = monitorRequest.data,
		RID = web.vars.HTTP_X_ICC_RID,
	})
	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(result["result"])
	web.status = http_code or 500 --unknown result code
	return ReplyJSON(web, result)
end

function GetCampaignAccountsStatus(campaigns)
	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local data = json.encode(campaigns)
	--wait at most 15 seconds for the IR to respond
	--(this does not account for execution time only for finding a suitable server)
	local result = irLink:GetAccountsStatus(data, 15)
	if result then
		GetModel("Campaigns").SaveAccountStatusCache(result)
		return true
	else
		LogWarning("Unable to retrieve accounts status from IR, no IRs available?")
		return false
	end
end

local function ReadInteraction (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local user_name = inconcert.authUser.userName
	local logged_user = utils.GetCurrentLoggedUser()
	local request = web.request

	--This is just in case somebody passes a number instead of a table (like in the old version)
	--note the request.event (without the "s") in the second part of the or
	local events = type(request.events) == "table" and request.events or { request.event }
	local mark_as_read = request.read
	local interaction_id = request.id
	local mark_as = ( mark_as_read and "READ" or "UNREAD" )

	LogInfo("'ReadInteraction' interaction request, id:'%s' read mark:'%s' events:'%s' from agent '%s'", interaction_id, mark_as_read, table.concat( events, ", " ), logged_user)
	local irLink = ir.IRAgentLink( inconcert.environment.apps.agent.cluster )
	local success = irLink:InteractionEventRead( interaction_id, logged_user, mark_as_read, events )
	LogInfo("MARK AS %s is %s", mark_as, ( success and "successfull" or "error" ) )
	return success
end

---
-- TODO: el formato de los responses de esta API es inconsistente
--
function CommentInteraction (web)
	local res, commentRequest = pcall(json.decode, web.input["post_data"])
	if not res or not commentRequest then
		web.status = 400
		return "Invalid request"
	end

	local interaction = GetModel("Interactions").GetInteraction(commentRequest.id)
	if utils.TableIsEmpty(interaction) then
		web.status = 400
		return ReplyJSON(web, false)
	end

	local app_id = commentRequest.app_id
	if not HasPermission( "COMMENT", interaction.campaign, app_id ) then
		web.status = 403
		return ReplyJSON(web, false)
	end

	local userId = utils.GetCurrentLoggedUser()
	LogInfo("[INTERACTIONS:CommentInteraction] Comment interaction request, id:%s comment:%s from agent %s", commentRequest.id, commentRequest.comment, userId)

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local result = irLink:CommentInteraction({
		id = commentRequest.id,
		agent = userId,
		comment = commentRequest.comment,
		rid = web.vars.HTTP_X_ICC_RID
	})

	return ReplyJSON(web, result)
end

function SetCommentsAsRead (web)
	local request = web.request
	local userId = utils.GetCurrentLoggedUser()

	LogInfo("Read comments interaction request, id:%s from agent %s", request.id, userId)
	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local result = irLink:SetCommentsAsRead({ id = request.id, agent = userId, rid = web.vars.HTTP_X_ICC_RID })

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(result["result"])
	web.status = http_code or 500 --unknown result code

	return result
end


---
--
function ReopenInteraction (web)
	local _T = m_htmlRender.extension.makeTextTranslator()
	local currentLoggedUser = utils.GetCurrentLoggedUser()
	local reopenRequest = json.decode(web.input["post_data"])

	LogInfo("[INTERACTIONS:ReopenInteraction] Reopen interaction request, id:%s type:%s, campaign:%s from agent %s",
		reopenRequest.id, reopenRequest.type, reopenRequest.campaign, currentLoggedUser)

	local interaction = GetModel("Interactions").GetFromArchive(reopenRequest.id)
	if not interaction then
		web.status = 500
		return "interaction not found"
	end

	local app_id = reopenRequest.app_id

	if app_id == "interaction" then
		local isMember = GetModel("Campaigns").UserIsMember(interaction.campaign, currentLoggedUser)
		if not isMember then
			LogError("[INTERACTIONS:ReopenInteraction] Unable to reopen conversation '%s'. You does not belong to '%s' campaign.", interaction.id, interaction.campaign)
			local result = {
				reason = "USER DOES NOT BELONG TO CAMPAIGN"
			}
			return ReplyJSON(web, result)
		end
	end

	-- app_id is optional
	if not HasPermission("REOPEN", interaction.campaign, app_id) then
		web.status = 403
		return "forbidden"
	end

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local ignore_level_membership = (
		app_id == "supervisor" and model_perm.IsPermissionEnabledForUser(app_id, "sup_archive_reopen", interaction.campaign)
	)

	local result = irLink:InteractionReopen({
		id = reopenRequest.id,
		agentAtVCC = currentLoggedUser,
		proxyType = reopenRequest.type,
		ignore_attention_level_membership = ignore_level_membership
	})

	if result.result ~= "OK" then
		LogError("[INTERACTIONS:ReopenInteraction] Reopen interaction error. IR has returned %s", json.encode(result))
		result.reasonTraslated = _T(result.reason)
	end

	return ReplyJSON(web, result)
end

function RequeueInteractions(web)
	local agentAtVCC = utils.GetCurrentLoggedUser()
	local success, request, interactions = ValidateBatchRequest(web, "interactions")

	if not success then
		-- did not pass interaction batch validation
		return request
	end

	interactions = RemoveInteractionsFromSameGroupInBatch(interactions)

	local app_id = request.app_id -- optional

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)

	return ProcessBatchData(web, "requeue", "interactions", interactions, function(interactionId)
		local currentInteraction = GetModel("Interactions").GetInteraction(interactionId)
		if utils.TableIsEmpty(currentInteraction) then
			return false, { error_code = "INTERACTION_NOT_FOUND", error_message = string.format("Interaction %s does not exist", interactionId) }
		end

		if not HasPermission( "REQUEUE", currentInteraction.campaign, app_id ) then
			return false, { error_code = "FORBIDDEN", error_message = "forbidden: "..web.input["post_data"] }
		end

		local result = irLink:InteractionRequeue({
			id = currentInteraction.id,
			type = currentInteraction.type,
			rid = web.vars.HTTP_X_ICC_RID,
			transferrer = utils.GetCurrentLoggedUser(),
		})
		if not result then
			return false, { error_code = "UNABLE_TO_REQUEUE", error_message = string.format("Unable to requeue interaction: %s", interactionId) }
		end
		-- something is wrong
		if result["result"] ~= "OK" then
			return false, { error_code = "UNABLE_TO_REQUEUE", error_message = result["result"] }
		end

		--everything looks good.
		return true;
	end)
end

function TakeInteractions(web)
	local vcc = inconcert.authUser.virtualContactCenter
	local user_name = inconcert.authUser.userName
	local success, request, interactions = ValidateBatchRequest(web, "interactions")
	if not success then
		-- did not pass interaction batch validation
		return request
	end

	interactions = RemoveInteractionsFromSameGroupInBatch(interactions)

	local app_id = request.app_id -- optional

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)

	return ProcessBatchData(web, "take", "interactions", interactions, function(interactionId)
		local currentInteraction = GetModel("Interactions").GetInteraction(interactionId)
		if utils.TableIsEmpty(currentInteraction) then
			return false, { error_code = "INTERACTION_NOT_FOUND", error_message = string.format("Interaction %s does not exist", interactionId) }
		end

		if not HasPermission( "TAKE", currentInteraction.campaign, app_id ) then
			return false, { error_code = "FORBIDDEN", error_message = "forbidden: "..web.input["post_data"] }
		end

		local campaign_name = helpers.SeparateVCC(currentInteraction.campaign)
		if not agentservice.CanTakeInteraction(user_name, vcc, currentInteraction.type, currentInteraction.id, campaign_name, web.vars.HTTP_X_ICC_RID) then
			LogError("[TakeInteractions] - Take interaction error: Agent '%s@%s' can't take interaction %s on campaign '%s' at the moment. Retry later.", user_name, vcc, currentInteraction.id, currentInteraction.campaign)
	
			-- As usual, error code handling needs to be rewritten
			-- AGENT_UNAVAILABLE means that this very agent is unavailable (busy slots, etc)
			-- The error code could be better, though
			return false, { error_code = "AGENT_UNAVAILABLE", error_message = "interaction limit exceeded" }
		end

		local result = irLink:InteractionGetFromQueue({
			id = currentInteraction.id,
			type = currentInteraction.type,
			agent = utils.GetCurrentLoggedUser(),
			rid = web.vars.HTTP_X_ICC_RID
		})

		if not result then
			return false, { error_code = "UNABLE_TO_TAKE", error_message = string.format("Unable to take interaction: %s", interactionId) }
		end
		-- something is wrong
		if result["result"] ~= "OK" then
			return false, { error_code = "UNABLE_TO_TAKE", error_message = result["result"] }
		end
	
		return true;
	end)
end

local function CanAddRemoveLabel( label, interactionId, campaign_id, campaign_labels )

	local result = {success = false, interaction_id = interactionId, campaign_id = campaign_id, label = label}

	if label.id == "PENDING" then
		result.error_message = LogWarning("Unable to add/delete 'PENDING' label on interaction: '%s'.", interactionId)
		result.error_code = "UNABLE_UPDATE_PENDING_LABEL"
		return result
	else
		if not campaign_labels then
			campaign_labels = GetModel("Campaigns").GetCampaignLabels(campaign_id) or {}
		end
		local camp_label = campaign_labels[label.id]
		if not camp_label then
			result.error_message = LogWarning("Unable to add/delete '%s' label on interaction: '%s'. Label not configured on campaign.", label.id, interactionId)
			result.error_code = "LABEL_NOT_CONFIGURED_ON_CAMPAIGN"
			return result
		end
		if camp_label.system == "1" then
			local fullUserId = utils.GetCurrentLoggedUser()
			local user_group = inconcert.authUser.userGroup
			local has_permission_to_app_supervisor = permissions.HasPermissionToApplication( "supervisor", fullUserId, user_group, campaign_id )

			if not has_permission_to_app_supervisor then
				result.error_message = LogWarning("Unable to add/delete '%s' SYSTEM label on interaction: '%s', campaign: '%s'. User '%s' has not permission to supervisor application.", label.id, interactionId, campaign_id, fullUserId)
				result.error_code = "USER_NOT_HAS_PERMISSION_FOR_UPDATE_SYSTEM_LABELS"
				return result
			end
		end
	end
	-- success
	result.success = true
	return result
end

local function DeleteLabels(interactionId, labelsToDelete, campaign_id, campaign_labels)
	LogInfo("Deleting labels in irLink- interaction request, interactionId: '%s', labels: %s", interactionId,  json.encode(labelsToDelete))
	local result = {success = true}
	if not labelsToDelete then
		return result
	end
	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	for labelId, label in pairs(labelsToDelete) do
		LogInfo("Deleting label: '%s' on interaction: '%s'.", labelId, interactionId)
		result = CanAddRemoveLabel(label, interactionId, campaign_id, campaign_labels)
		if result.success then
			result.success = irLink:RemoveInteractionLabel(interactionId, labelId)
			LogInfo("-Deleted Label: '%s' on interaction: '%s', success: '%s'.", labelId, interactionId, tostring(result))
			if not result.success then
				break
			end
		else
			break
		end
	end
	return result
end

local function AddLabels(interactionId, newLabels, campaign_id, campaign_labels)

	LogInfo("Adding labels in irLink- interaction request, interactionId: '%s', labels: %s", interactionId,  json.encode(newLabels))
	local result = {success = true}
	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)

	for labelId, label in pairs(newLabels) do
		local encodedLabel = json.encode(label)
		LogInfo("Adding label: '%s' on interaction: '%s', information: '%s'", labelId, interactionId, encodedLabel)
		result = CanAddRemoveLabel(label, interactionId, campaign_id, campaign_labels)
		if result.success then
			result.success = irLink:AddInteractionLabel(interactionId, labelId, encodedLabel)
			LogInfo("+Added Interaction Label: '%s' on interaction: '%s', success: '%s'.", labelId, interactionId, tostring(result))
			if not result.success then
				break
			end
		else
			break
		end
	end
	return result
end

function UpdateInteractionsLabels(web)
	local success, labelRequest, interactions = ValidateBatchRequest(web, "interactions")
	if not success then
		return labelRequest
	end

	interactions = UnionBatchDataWithGroups(interactions)

	local app_id = labelRequest.app_id -- optional

	local cached_campaigns_labels = {}

	return ProcessBatchData(web, "update labels", "interactions", interactions, function(interactionId)

		local currentInteraction = GetModel("Interactions").GetInteraction(interactionId)

		if utils.TableIsEmpty(currentInteraction) then
			web.status = 404
			return false, { error_code = "INTERACTION_NOT_FOUND", error_message = string.format("Interaction %s not found", interactionId) }
		end

		local campaign_id = currentInteraction.campaign

		local hasPermissionToLabel = HasPermission( "LABEL", campaign_id, app_id )
		if not hasPermissionToLabel then
			web.status = 403
			return false, { error_code = "FORBIDDEN", error_message = string.format("Fobidden: Unable to add label to interaction: %s", interactionId) }
		end

		-- send to the user the current interaction labels.
		local current_labels = currentInteraction.labels
		cached_campaigns_labels[campaign_id] = cached_campaigns_labels[campaign_id] or GetModel("Campaigns").GetCampaignLabels(campaign_id)

		local delete_result = DeleteLabels(interactionId, labelRequest.currentLabels, campaign_id, cached_campaigns_labels[campaign_id])
		if not delete_result.success then
			--web.status = 404
			return false, delete_result
		end

		local add_result = AddLabels(interactionId, labelRequest.newLabels, campaign_id, cached_campaigns_labels[campaign_id])
		if not add_result.success then
			--web.status = 404
			return false, delete_result
		end

		-- everything looks good
		return true, { current_labels = current_labels }
	end)
end

function DeleteLabelsInteractions(web)
	local success, labelRequest, interactions = ValidateBatchRequest(web, "interactions")

	if not success then
		return labelRequest
	end

	interactions = UnionBatchDataWithGroups(interactions)
	local app_id = labelRequest.app_id -- optional
	local cached_campaigns_labels = {}

	return ProcessBatchData(web, "delete labels", "interactions", interactions, function(interactionId)
		local currentInteraction = GetModel("Interactions").GetInteraction(interactionId)

		if utils.TableIsEmpty(currentInteraction) then
			return false, { error_code = "INTERACTION_NOT_FOUND", error_message = string.format("Interaction %s not found", interactionId) }
		end

		local campaign_id = currentInteraction.campaign

		local hasPermissionToLabel = HasPermission( "LABEL", currentInteraction.campaign, app_id )
		if not hasPermissionToLabel then
			web.status = 403
			return false, { error_code = "FORBIDDEN", error_message = string.format("Fobidden: Unable to delete label to interaction: %s", interactionId) }
		end

		cached_campaigns_labels[campaign_id] = cached_campaigns_labels[campaign_id] or GetModel("Campaigns").GetCampaignLabels(campaign_id)
		local delete_result = DeleteLabels(interactionId, labelRequest.deleteLabels, campaign_id, cached_campaigns_labels[campaign_id])
		if not delete_result.success then
			return false, delete_result
		end

		-- send to the user the current interaction labels.
		local current_labels = currentInteraction.labels

		-- everything looks good
		return true, { current_labels = current_labels }
	end)
end

local function ValidateCanTakeAppointment (info)
	if not HasPermission("CREATE_OUTGOING", info.campaignAtVCC, info.app_id) then
		LogWarning("[INTERACTIONS:ValidateCanTakeAppointment] Error Taking Appointment. You not have permission for create outgoing conversation.")
		return false, {result = "FORBIDDEN", reason = "You not have permission for create outgoing conversation."}
	end
	local vcc = inconcert.authUser.virtualContactCenter
	local userIdWithoutVcc = inconcert.authUser.userName
	local campaignWithoutVcc = helpers.SeparateVCC(info.campaignAtVCC)
	if not agentservice.CanTakeInteraction(userIdWithoutVcc, vcc, info.proxyType, info.interactionID, info.campaignAtVCC, info.HTTP_X_ICC_RID) then
		LogWarning("INTERACTIONS:ValidateCanTakeAppointment] Error Taking Appointment. You can not take conversation in agent current status.")
		return false, {result = "AGENT_UNAVAILABLE", reason = "You can not take conversation in agent current status."}
	end
	-- success
	return true
end

local function CanTakeAppointment (web)
	local postedData = web.input["post_data"]
	local res, request = pcall(json.decode, postedData)

	if not res or not request then
		LogWarning("[INTERACTIONS:CanTakeAppointment] Error Taking Appointment. Invalid request. Interactions has an invalid structure.")
		web.status = 400
		return {result = "INVALID_REQUEST", reason = "Invalid request. Interactions has an invalid structure"}
	end
	
	request.HTTP_X_ICC_RID = web.vars.HTTP_X_ICC_RID
	local success, response = ValidateCanTakeAppointment(request)
	if not success then
		LogWarning("[INTERACTIONS:CanTakeAppointment] Error Taking Appointment. Err: %s.", response.reason)
		web.status = response.result == "FORBIDDEN" and 403 or 500
		return ReplyJSON(web, response)
	end

	-- OK
	response = {result = "OK"}
	web.status = 200
	return ReplyJSON(web, response)
end

local function TakeAppointment (web)
	local postedData = web.input["post_data"]
	local res, request = pcall(json.decode, postedData)

	if not res or not request then
		LogWarning("[INTERACTIONS:TakeAppointment] Error Taking Appointment. Invalid request. Interactions has an invalid structure.")
		web.status = 400
		return {result = "INVALID_REQUEST", reason = "Invalid request. Interactions has an invalid structure"}
	end
	
	request.HTTP_X_ICC_RID = web.vars.HTTP_X_ICC_RID
	local success, response = ValidateCanTakeAppointment(request)
	if not success then
		LogWarning("[INTERACTIONS:TakeAppointment] Error Taking Appointment. Err: %s.", response.reason)
		web.status = response.result == "FORBIDDEN" and 403 or 500
		return ReplyJSON(web, response)
	end

	local fullUserId = utils.GetCurrentLoggedUser()
	LogInfo("[INTERACTIONS:TakeAppointment] user %s request %s", fullUserId, postedData)

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local response = irLink:TakeAppointment({
		proxyType = request.proxyType,
		agentAtVCC = fullUserId,
		interactionID = request.interactionID,
		RID = web.vars.HTTP_X_ICC_RID
	})

	if response.result ~= "OK" then
		LogWarning("[INTERACTIONS:TakeAppointment] Error Taking Appointment. Response: '%s'. Request %s.", json.encode(response), postedData)
		web.status = 500
		return ReplyJSON(web, response)
	end

	LogInfo("[INTERACTIONS:TakeAppointment] Success Taken Appointment. Response: '%s'. Request %s.", json.encode(response), postedData)

	web.status = 200
	return ReplyJSON(web, response)
end

local function MakeAppointment (web)
	local res, request = pcall(json.decode, web.input["post_data"])
	if not res or not request then
		web.status = 400
		return "Invalid request"
	end
	
	if not HasPermission("SCHEDULE", request.campaignAtVCC, request.app_id) then
		web.status = 403
		return ReplyJSON(web, false)
	end

	local userId = utils.GetCurrentLoggedUser()
	LogInfo("MakeAppointment user %s request %s", userId, web.input["post_data"])

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local result = irLink:MakeAppointment({
		proxyType = request.proxyType,
		campaignAtVCC = request.campaignAtVCC,
		account = request.account,
		agentAtVCC = userId,
		interactionID = request.interactionID,
		addressTable = request.addressTable,
		fromDate = request.fromDate,
		RID = web.vars.HTTP_X_ICC_RID
	})
	return ReplyJSON(web, result)
end

--IM/Chat specific

--[[
	Retrieves the contact status from their IM accounts ands suscribes notifications for those contacts
]]--
function SolveSubscribeIMContactsStatus(contacts)
	local vcc = inconcert.authUser.virtualContactCenter
	local user_name = inconcert.authUser.userName
	local suscriptions = {}
	local modelInteractions = GetModel("Interactions")
	for key, contact in pairs(contacts or {}) do
		for k, address in pairs(contact.addresses) do
			if (address.Type == "IM") then
				LogInfo("Retrieving contact status for buddy %s on procotol %s", address.buddy, address.protocol)
				address.status = modelInteractions.IMGetBuddyStatus(address.buddy, address.protocol)
				if (address.status) then
					address.status.icon = nil
				end
				table.insert(suscriptions, address)
			end
		end
	end
end


--[[
	Request a new IM conversation creation against the IR
]]--
function CreateOutgoingIM(web)
	local vcc = inconcert.authUser.virtualContactCenter
	local user_name = utils.GetCurrentLoggedUser()

	local imRequest = json.decode(web.input["post_data"])

	LogInfo("Create IM conversation request, id:%s campaign:%s protocol:%s buddy:%s from agent %s to contact %s",
			imRequest.id, imRequest.campaign, imRequest.address.protocol, imRequest.address.buddy, user_name, imRequest.contact)

	-- TODO: Validar licencia, discriminada segun IM, etc, por ahora queda solo IM
	if not hasLicensesForChannel( "IM" ) then
		local reason = LogFatal("Unable to create IM outgoing conversation. Not has licensed for the IM channel.")
		web.status = 500
		return ReplyJSON(web, {
				agent_err = true,
				result = "NOT_LICENSED",
				reason = reason
			})
	end

	if not HasPermission( "CREATE_OUTGOING", imRequest.campaign, "interaction" ) then
		web.status = 403
		return ReplyJSON( web, { result = "FORBIDDEN", reason = "You not have permission for create outgoing IM conversation." })
	end

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)

	--this is done in order to avoid api manglers creating interactions from another vcc from the one they're logged in
	local campaignIdAtVCC = helpers.AddVCC(imRequest.campaign, vcc)
	local res = irLink:IMCreateOutgoing(imRequest.id, campaignIdAtVCC, imRequest.address.protocol, imRequest.address.buddy, user_name, imRequest.contact or "", imRequest.parentInteractionId or "")

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(res["result"])
	web.status = http_code or 500 --unknown result code

	return ReplyJSON(web, res)
end


--Facebook
local function GetAccountFacebookPages( web, campaignId, accountId )
	local vcc = inconcert.authUser.virtualContactCenter
	local name = inconcert.authUser.userName
	LogInfo("User '%s' on vcc '%s' retrieving '%s' account facebook pages for campaign '%s'", name, vcc, accountId, campaignId)

	if not hasLicensesForChannel( "FACEBOOK" ) then
		local reason = LogFatal("Unable to retrieve account pages. Not has licensed for the Facebook channel.")
		web.status = 500
		return ReplyJSON(web, {
				agent_err = true,
				result = "NOT_LICENSED",
				reason = reason
			})
	end

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local campaignIdAtVCC = helpers.AddVCC(campaignId, vcc)
	local res = irLink:FacebookGetPages(campaignIdAtVCC, accountId, "")
	local http_code = http_mapping.getHTTPCode(res["result"])
	web.status = http_code or 500 --unknown result code

	return ReplyJSON(web, res)
end

local function CreateOutgoingFacebook(web)
	local vcc = inconcert.authUser.virtualContactCenter
	local user_name = utils.GetCurrentLoggedUser()

	local facebookRequest = json.decode(web.input["post_data"])

	LogInfo("Create facebook request from agent:%s, id:%s campaign:%s account:%s destination:%s", user_name, facebookRequest.id, facebookRequest.campaign, facebookRequest.account, facebookRequest.destination)

	if not hasLicensesForChannel( "FACEBOOK" ) then
		local reason = LogFatal("Unable to create facebook post. Not has licensed for the Facebook channel.")
		web.status = 500
		return ReplyJSON(web, {
				agent_err = true,
				result = "NOT_LICENSED",
				reason = reason
			})
	end

	if not HasPermission( "CREATE_OUTGOING", facebookRequest.campaign, "interaction" ) then
		web.status = 403
		return ReplyJSON( web, { result = "FORBIDDEN", reason = "You not have permission for create outgoing facebook conversation." })
	end

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)

	--this is done in order to avoid api manglers creating interactions from another vcc from the one they're logged in
	local campaignIdAtVCC = helpers.AddVCC(facebookRequest.campaign, vcc)
	local res = irLink:FacebookCreate(facebookRequest.id,
					campaignIdAtVCC,
					user_name,
					facebookRequest.account,
					facebookRequest.destination,
					facebookRequest.destinationType and facebookRequest.destinationType or "CONTACT",
					facebookRequest.destinationName and facebookRequest.destinationName or "",
					facebookRequest.parentInteractionId or "")

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(res["result"])
	web.status = http_code or 500 --unknown result code

	return ReplyJSON(web, res)
end

local function CommentFacebookPost(web)
	local vcc = inconcert.authUser.virtualContactCenter
	local agentAtVCC = utils.GetCurrentLoggedUser()

	LogInfo("Comment facebook post request from agent:%s, post_data: '%s'.", agentAtVCC, web.input["post_data"])

	if not hasLicensesForChannel( "FACEBOOK" ) then
		local reason = LogFatal("Unable to comment facebook post. Not has licensed for the Facebook channel.")
		web.status = 500
		return ReplyJSON(web, {
				agent_err = true,
				result = "NOT_LICENSED",
				reason = reason
			})
	end

	local facebookRequest = json.decode(web.input["post_data"])
	--this is done in order to avoid api manglers creating interactions from another vcc from the one they're logged in
	local campaignIdAtVCC = helpers.AddVCC(facebookRequest.campaign, vcc)

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local res = irLink:FBCommentPost({
		fake_event_id = facebookRequest.fake_event_id,
		rid = web.vars.HTTP_X_ICC_RID,
		campaignAtVCC = campaignIdAtVCC,
		account = facebookRequest.account,
		page = facebookRequest.page,
		interactionID = facebookRequest.interactionID,
		agentAtVCC = agentAtVCC,
		postId = facebookRequest.postId,
		commentId = facebookRequest.commentId,
		message = facebookRequest.message,
	})
	local http_code = http_mapping.getHTTPCode(res["result"])
	web.status = http_code or 500 --unknown result code
	return ReplyJSON(web,  res)
end

--- [Twitter] Will send a direct message
--
-- @tparam table  web
-- @tparam table  web.input
-- @tparam string web.input.post_data  - Data posted on the API
--
-- if we decode post_data json string:
-- table  request
--
local function FacebookPrivateReply(web)
	local _T = m_htmlRender.extension.makeTextTranslator()
	local vcc = inconcert.authUser.virtualContactCenter
	local user_name = utils.GetCurrentLoggedUser()
	local request = json.decode(web.input["post_data"])

	LogInfo("Sending direct message from object from agent: '%s', to Facebook object: '%s', on interaction id: '%s', campaign: '%s', account: '%s', message: '%s'",
			user_name, json.encode(request.object), request.interactionID, request.campaignId, request.account, json.encode(request.message))

	if not hasLicensesForChannel( "FACEBOOK" ) then
		local reason = LogFatal("Unable to direct message. Not has licensed for the Facebook channel.")
		web.status = 500
		return ReplyJSON(web, {
			agent_err = true,
			result = "NOT_LICENSED",
			reason = _T(reason)
		})
	end

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)

	--this is done in order to avoid api manglers creating interactions from another vcc from the one they're logged in
	local campaignIdAtVCC = helpers.AddVCC(request.campaignId, vcc)
	local res = irLink:FBPrivateReply({
		campaignAtVCC = campaignIdAtVCC,
		account = request.account,
		page = request.page,
		interactionID = request.interactionID,
		agentAtVCC = user_name,
		object = request.object,
		index = request.index,
		message = request.message,
	})

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(res["result"])
	web.status = http_code or 500 --unknown result code

	if res.result ~= "OK" then
		LogError("Error sending Facebook direct message from object on interaction id: '%s' -> '%s'", request.interactionID, res.reason)
		res.reason = _T(res.reason)
		return ReplyJSON(web, res)
	end

	LogInfo("Success sending Facebook direct message from object on interaction '%s'", request.interactionID)
	return ReplyJSON(web, res)
end

local function NewFacebookPrivateMessage(web)
	local vcc = inconcert.authUser.virtualContactCenter
	local user_name = inconcert.authUser.userName
	local agentAtVCC = utils.GetCurrentLoggedUser()

	LogInfo("New facebook private message request from agent:%s, post_data: '%s'.", agentAtVCC, web.input["post_data"])

	if not hasLicensesForChannel( "FACEBOOK" ) then
		local reason = LogFatal("Unable to new facebook private message. Not has licensed for the Facebook channel.")
		web.status = 500
		return ReplyJSON(web, {
				agent_err = true,
				result    = "NOT_LICENSED",
				reason    = reason
			})
	end

	local facebookRequest = json.decode(web.input["post_data"])
	--this is done in order to avoid api manglers creating interactions from another vcc from the one they're logged in
	local campaignIdAtVCC = helpers.AddVCC(facebookRequest.campaign, vcc)
	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)

	local res = irLink:FBCreatePrivateMessage({
		fake_event_id = facebookRequest.fake_event_id,
		rid = web.vars.HTTP_X_ICC_RID,
		campaignAtVCC = campaignIdAtVCC,
		account = facebookRequest.account,
		page = facebookRequest.page or "",
		interactionID = facebookRequest.interactionId,
		agentAtVCC = agentAtVCC,
		message = facebookRequest.message
	})

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(res["result"])
	web.status = http_code or 500 --unknown result code

	return ReplyJSON(web, res)
end

local function LikeFacebookObject(web)
	local vcc = inconcert.authUser.virtualContactCenter
	local user_name = inconcert.authUser.userName
	local agentAtVCC = utils.GetCurrentLoggedUser()

	LogInfo("Like facebook object request from agent:%s, post_data: '%s'.", agentAtVCC, web.input["post_data"])

	if not hasLicensesForChannel( "FACEBOOK" ) then
		local reason = LogFatal("Unable like object. Not has licensed for the Facebook channel.")
		web.status = 500
		return ReplyJSON(web, {
				agent_err = true,
				result    = "NOT_LICENSED",
				reason    = reason
			})
	end

	local facebookRequest = json.decode(web.input["post_data"])
	--this is done in order to avoid api manglers creating interactions from another vcc from the one they're logged in
	local campaignIdAtVCC = helpers.AddVCC(facebookRequest.campaign, vcc)
	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)

	local res = irLink:FBLikeObject(
		facebookRequest.fake_event_id,
		facebookRequest.interactionId,
		campaignIdAtVCC,
		facebookRequest.account,
		facebookRequest.page or "",
		facebookRequest.objectId,
		facebookRequest.like and true or false,
		web.vars.HTTP_X_ICC_RID --RID
	)
	
	web.status = res and 200 or 500

	return ReplyJSON(web, res)
end

local function GetFacebookPages(web)
	local vcc = inconcert.authUser.virtualContactCenter
	local user_name = inconcert.authUser.userName

	local request = json.decode(web.input["post_data"])

	LogInfo("Getting facebok pages for account on campaign:%s account:%s", request.campaignId, request.account)

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	--this is done in order to avoid api manglers creating interactions from another vcc from the one they're logged in
	local campaignIdAtVCC = helpers.AddVCC(request.campaignId, vcc)
	local res = irLink:FacebookGetPages(campaignIdAtVCC, request.account)

	if res.result ~= "OK" then
		LogError("Error getting facebok pages for account on campaign:%s account:%s -> '%s'", 
			request.campaignId, request.account, res.reason.message)
		return ReplyJSON(web, { result = res.result, reason = res.reason.message })
	end

	LogInfo("Success getting facebook pages for account '%s'", request.account)

	return ReplyJSON(web, res)
end

local function GetFacebookAvatar(wsapi_env)
	local vcc = inconcert.authUser.virtualContactCenter
	local user_name = inconcert.authUser.userName

	StartRepository()
	return repositoryController.retrieveFile(wsapi_env,
		function(request)
			LogInfo("Getting facebok avatar for user:%s on campaign:%s account:%s page:%s", 
				request.params.userId, request.params.campaignId, request.params.account, request.params.page)
			--check if exists in repository
			local key = "FACEBOOK_AVATAR:" .. request.params.account .. ":" .. request.params.page .. ":" .. request.params.userId
			request.params.id = key
			request.params.section = -1
			request.params.part = -1
			local exists = repositoryController.existsFile(key, -1, -1)
			if exists then
				--if exits then return from repository
				return 200
			end
			--else send request to server
			local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
			local result = irLink:RequestAvatar({
				proxyType = "FACEBOOK",
				campaignAtVCC = request.params.campaignId,
				account = request.params.account,
				page = request.params.page,
				userId = request.params.userId,
				returnPicture = true
			})
			if result.result == "OK" then
				return 200
			else
				local http_code = http_mapping.getHTTPCode(result.result)
				return http_code or 500, result.reason
			end
		end
	)
end

local function CommentInstagramPost(web)
	local vcc = inconcert.authUser.virtualContactCenter
	local agentAtVCC = utils.GetCurrentLoggedUser()

	LogInfo("Comment instagram post request from agent:%s, post_data: '%s'.", agentAtVCC, web.input["post_data"])

	if not hasLicensesForChannel( "INSTAGRAM" ) then
		local reason = LogFatal("Unable to comment instagram post. Not has licensed for the Instagram channel.")
		web.status = 500
		return ReplyJSON(web, {
				agent_err = true,
				result = "NOT_LICENSED",
				reason = reason
			})
	end

	local instagramRequest = json.decode(web.input["post_data"])
	--this is done in order to avoid api manglers creating interactions from another vcc from the one they're logged in
	local campaignAtVCC = helpers.AddVCC(instagramRequest.campaign, vcc)

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local res = irLink:InstagramCommentPost({
		fake_event_id = instagramRequest.fake_event_id,
		interactionId = instagramRequest.interactionId,
		campaignAtVCC = campaignAtVCC,
		account = instagramRequest.account,
		agentAtVCC = agentAtVCC,
		postId = instagramRequest.postId,
		commentId = instagramRequest.commentId,
		message = instagramRequest.message,
		rid = web.vars.HTTP_X_ICC_RID
	})
	local http_code = http_mapping.getHTTPCode(res["result"])
	web.status = http_code or 500 --unknown result code
	return ReplyJSON(web, res)
end

local function NewInstagramPrivateMessage(web)
	local vcc = inconcert.authUser.virtualContactCenter
	local user_name = inconcert.authUser.userName
	local agentAtVCC = utils.GetCurrentLoggedUser()

	LogInfo("New instagram private message request from agent:%s, post_data: '%s'.", agentAtVCC, web.input["post_data"])

	if not hasLicensesForChannel( "INSTAGRAM" ) then
		local reason = LogFatal("Unable to new instagram private message. Not has licensed for the Instagram channel.")
		web.status = 500
		return ReplyJSON(web, {
				agent_err = true,
				result    = "NOT_LICENSED",
				reason    = reason
			})
	end

	local instagramRequest = json.decode(web.input["post_data"])
	--this is done in order to avoid api manglers creating interactions from another vcc from the one they're logged in
	local campaignAtVCC = helpers.AddVCC(instagramRequest.campaign, vcc)
	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)

	local res = irLink:InstagramCreatePrivateMessage({
		fake_event_id = instagramRequest.fake_event_id,
		rid = web.vars.HTTP_X_ICC_RID,
		campaignAtVCC = campaignAtVCC,
		account = instagramRequest.account,
		page = instagramRequest.page or "",
		interactionID = instagramRequest.interactionId,
		agentAtVCC = agentAtVCC,
		message = instagramRequest.message
	})

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(res["result"])
	web.status = http_code or 500 --unknown result code

	return ReplyJSON(web, res)
end

--- [Instagram] Will send a direct message
--
-- @tparam table  web
-- @tparam table  web.input
-- @tparam string web.input.post_data  - Data posted on the API
--
-- if we decode post_data json string:
-- table  request
--
local function InstagramPrivateReply(web)
	local _T = m_htmlRender.extension.makeTextTranslator()
	local vcc = inconcert.authUser.virtualContactCenter
	local user_name = utils.GetCurrentLoggedUser()
	local request = json.decode(web.input["post_data"])

	local request = json.decode(web.input["post_data"])

	LogInfo("Sending direct message from object from agent: '%s', to Instagram object: '%s', on interaction id: '%s', campaign: '%s', account: '%s', message: '%s'",
			user_name, json.encode(request.object), request.interactionID, request.campaignId, request.account, json.encode(request.message))

	if not hasLicensesForChannel( "INSTAGRAM" ) then
		local reason = LogFatal("Unable to direct message. Not has licensed for the Instagram channel.")
		web.status = 500
		return ReplyJSON(web, {
			agent_err = true,
			result = "NOT_LICENSED",
			reason = _T(reason)
		})
	end

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)

	--this is done in order to avoid api manglers creating interactions from another vcc from the one they're logged in
	local campaignIdAtVCC = helpers.AddVCC(request.campaignId, vcc)
	local res = irLink:InstagramPrivateReply({
		campaignAtVCC = campaignIdAtVCC,
		account = request.account,
		page = request.page,
		interactionID = request.interactionID,
		agentAtVCC = user_name,
		object = request.object,
		index = request.index,
		message = request.message,
	})

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(res["result"])
	web.status = http_code or 500 --unknown result code

	if res.result ~= "OK" then
		LogError("Error sending Facebook direct message from object on interaction id: '%s' -> '%s'", request.interactionID, res.reason)
		res.reason = _T(res.reason)
		return ReplyJSON(web, res)
	end

	LogInfo("Success sending Facebook direct message from object on interaction '%s'", request.interactionID)
	return ReplyJSON(web, res)
end

local function LikeInstagramObject(web)
	local vcc = inconcert.authUser.virtualContactCenter
	local user_name = inconcert.authUser.userName
	local agentAtVCC = utils.GetCurrentLoggedUser()

	LogInfo("Like instagram object request from agent:%s, post_data: '%s'.", agentAtVCC, web.input["post_data"])

	if not hasLicensesForChannel( "INSTAGRAM" ) then
		local reason = LogFatal("Unable like object. Not has licensed for the Instagram channel.")
		web.status = 500
		return ReplyJSON(web, {
				agent_err = true,
				result    = "NOT_LICENSED",
				reason    = reason
			})
	end

	local instagramRequest = json.decode(web.input["post_data"])
	--this is done in order to avoid api manglers creating interactions from another vcc from the one they're logged in
	local campaignAtVCC = helpers.AddVCC(instagramRequest.campaign, vcc)
	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local res = irLink:InstagramLikeObject({
		fake_event_id = instagramRequest.fake_event_id,
		rid = web.vars.HTTP_X_ICC_RID,
		campaignAtVCC = campaignAtVCC,
		account = instagramRequest.account,
		interactionID = instagramRequest.interactionId,
		agentAtVCC = agentAtVCC,
		objectId = instagramRequest.objectId,
		like = instagramRequest.like and true or false
	})
	
	web.status = res and 200 or 500

	return ReplyJSON(web, res)
end

local function LikeInstagramMessage(web)
	local vcc = inconcert.authUser.virtualContactCenter
	local user_name = inconcert.authUser.userName
	local agentAtVCC = utils.GetCurrentLoggedUser()

	LogInfo("Like instagram message request from agent:%s, post_data: '%s'.", agentAtVCC, web.input["post_data"])

	if not hasLicensesForChannel( "INSTAGRAM" ) then
		local reason = LogFatal("Unable like message. Not has licensed for the Instagram channel.")
		web.status = 500
		return ReplyJSON(web, {
				agent_err = true,
				result    = "NOT_LICENSED",
				reason    = reason
			})
	end

	local instagramRequest = json.decode(web.input["post_data"])
	--this is done in order to avoid api manglers creating interactions from another vcc from the one they're logged in
	local campaignAtVCC = helpers.AddVCC(instagramRequest.campaign, vcc)
	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local res = irLink:InstagramLikeMessage({
		fake_event_id = instagramRequest.fake_event_id,
		rid = web.vars.HTTP_X_ICC_RID,
		campaignAtVCC = campaignAtVCC,
		account = instagramRequest.account,
		interactionID = instagramRequest.interactionId,
		agentAtVCC = agentAtVCC,
		recipientId = instagramRequest.recipientId,
		messageId = instagramRequest.messageId,
		page = instagramRequest.page,
		like = instagramRequest.like and true or false
	})
	
	web.status = res and 200 or 500

	return ReplyJSON(web, res)
end

local function GetInstagramAvatar(wsapi_env)
	local vcc = inconcert.authUser.virtualContactCenter
	local user_name = inconcert.authUser.userName

	StartRepository()
	return repositoryController.retrieveFile(wsapi_env,
		function(request)
			LogInfo("Getting instagram avatar for user:%s on campaign:%s account:%s page:%s", 
				request.params.userId, request.params.campaignId, request.params.account, request.params.page)
			--check if exists in repository
			local key = "INSTAGRAM_AVATAR:" .. request.params.account .. ":" .. request.params.page .. ":" .. request.params.userId
			request.params.id = key
			request.params.section = -1
			request.params.part = -1
			local exists = repositoryController.existsFile(key, -1, -1)
			if exists then
				--if exits then return from repository
				return 200
			end
			--else send request to server
			local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
			local result = irLink:RequestAvatar({
				proxyType = "INSTAGRAM",
				campaignAtVCC = request.params.campaignId,
				account = request.params.account,
				page = request.params.page,
				userId = request.params.userId,
				returnPicture = true
			})
			if result.result == "OK" then
				return 200
			else
				local http_code = http_mapping.getHTTPCode(result.result)
				return http_code or 500, result.reason
			end
		end
	)
end

local function GetInstagramMedia(wsapi_env)
	local vcc = inconcert.authUser.virtualContactCenter
	local user_name = inconcert.authUser.userName
	local request = wsapi.request.new(wsapi_env)
	local headers = {}
	local status = 404	-- not found by default

	local make_body = function(content)
		return coroutine.wrap(function() coroutine.yield(content) end)
	end

	LogInfo("Getting instagram media for user:%s - media url:'%s'", 
		user_name, request.GET.url)
	--send request to server
	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local result = irLink:GetInstagramMedia({
		proxyType = "INSTAGRAM",
		url = request.GET.url
	})
	
	local b='ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/' -- You will need this for encoding/decoding
	-- B64 decoding
	function dec(data)
	    data = string.gsub(data, '[^'..b..'=]', '')
	    return (data:gsub('.', function(x)
	        if (x == '=') then return '' end
	        local r,f='',(b:find(x)-1)
	        for i=6,1,-1 do r=r..(f%2^i-f%2^(i-1)>0 and '1' or '0') end
	        return r;
	    end):gsub('%d%d%d?%d?%d?%d?%d?%d?', function(x)
	        if (#x ~= 8) then return '' end
	        local c=0
	        for i=1,8 do c=c+(x:sub(i,i)=='1' and 2^(8-i) or 0) end
	            return string.char(c)
	    end))
	end

	if result.result == "OK" then
		return 200, result.headers, make_body(dec(result.content))
	else
		local http_code = http_mapping.getHTTPCode(result.result)
		return http_code or 500, result.reason
	end
end

local function GetInstagramMediaData(web)
	local user_name = inconcert.authUser.userName

	if not hasLicensesForChannel( "INSTAGRAM" ) then
		web.status = 500
		return ReplyJSON(web, {
				result = "NOT_LICENSED"
			})
	end
	local request = json.decode(web.input["post_data"])
	LogInfo("Getting instagram media data for user:%s - media:'%s'", 
		user_name, type(request.media) == "table" and json.encode(request.media) or request.media)
	--send request to server
	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	
	local result = irLink:SendCommand({
		proxyType = "INSTAGRAM",
		command   = "GetMediaData",
		page    = request.page,
		media = request.media
	})

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(result["result"])
	web.status = http_code or 500 --unknown result code
	return ReplyJSON(web, result)
end

local function CommentYoutubePost(web)
	local vcc = inconcert.authUser.virtualContactCenter
	local agentAtVCC = utils.GetCurrentLoggedUser()

	LogInfo("Comment youtube post request from agent:%s, post_data: '%s'.", agentAtVCC, web.input["post_data"])

	if not hasLicensesForChannel( "YOUTUBE" ) then
		local reason = LogFatal("Unable to comment youtube post. Not has licensed for the Youtube channel.")
		web.status = 500
		return ReplyJSON(web, {
				agent_err = true,
				result = "NOT_LICENSED",
				reason = reason
			})
	end

	local youtubeRequest = json.decode(web.input["post_data"])
	--this is done in order to avoid api manglers creating interactions from another vcc from the one they're logged in
	local campaignAtVCC = helpers.AddVCC(youtubeRequest.campaign, vcc)

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local res = irLink:YoutubeCommentPost({
		fake_event_id = youtubeRequest.fake_event_id,
		interactionId = youtubeRequest.interactionId,
		campaignAtVCC = campaignAtVCC,
		account = youtubeRequest.account,
		agentAtVCC = agentAtVCC,
		postId = youtubeRequest.postId,
		commentId = youtubeRequest.commentId,
		message = youtubeRequest.message,
		rid = web.vars.HTTP_X_ICC_RID
	})
	local http_code = http_mapping.getHTTPCode(res["result"])
	web.status = http_code or 500 --unknown result code
	return ReplyJSON(web, res)
end

local function NewYoutubePrivateMessage(web)
	local vcc = inconcert.authUser.virtualContactCenter
	local user_name = inconcert.authUser.userName
	local agentAtVCC = utils.GetCurrentLoggedUser()

	LogInfo("New youtube private message request from agent:%s, post_data: '%s'.", agentAtVCC, web.input["post_data"])

	if not hasLicensesForChannel( "YOUTUBE" ) then
		local reason = LogFatal("Unable to new youtube private message. Not has licensed for the Youtube channel.")
		web.status = 500
		return ReplyJSON(web, {
				agent_err = true,
				result    = "NOT_LICENSED",
				reason    = reason
			})
	end

	local youtubeRequest = json.decode(web.input["post_data"])
	--this is done in order to avoid api manglers creating interactions from another vcc from the one they're logged in
	local campaignAtVCC = helpers.AddVCC(youtubeRequest.campaign, vcc)
	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)

	local res = irLink:YoutubeCreatePrivateMessage({
		fake_event_id = youtubeRequest.fake_event_id,
		rid = web.vars.HTTP_X_ICC_RID,
		campaignAtVCC = campaignAtVCC,
		account = youtubeRequest.account,
		interactionID = youtubeRequest.interactionId,
		agentAtVCC = agentAtVCC,
		message = youtubeRequest.message
	})

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(res["result"])
	web.status = http_code or 500 --unknown result code

	return ReplyJSON(web, res)
end

local function LikeYoutubeObject(web)
	local vcc = inconcert.authUser.virtualContactCenter
	local user_name = inconcert.authUser.userName
	local agentAtVCC = utils.GetCurrentLoggedUser()

	LogInfo("Like youtube object request from agent:%s, post_data: '%s'.", agentAtVCC, web.input["post_data"])

	if not hasLicensesForChannel( "YOUTUBE" ) then
		local reason = LogFatal("Unable like object. Not has licensed for the Youtube channel.")
		web.status = 500
		return ReplyJSON(web, {
				agent_err = true,
				result    = "NOT_LICENSED",
				reason    = reason
			})
	end

	local youtubeRequest = json.decode(web.input["post_data"])
	--this is done in order to avoid api manglers creating interactions from another vcc from the one they're logged in
	local campaignAtVCC = helpers.AddVCC(youtubeRequest.campaign, vcc)
	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local res = irLink:YoutubeLikeObject({
		fake_event_id = youtubeRequest.fake_event_id,
		rid = web.vars.HTTP_X_ICC_RID,
		campaignAtVCC = campaignAtVCC,
		account = youtubeRequest.account,
		interactionID = youtubeRequest.interactionId,
		agentAtVCC = agentAtVCC,
		objectId = youtubeRequest.objectId,
		like = youtubeRequest.like and true or false
	})
	
	web.status = res and 200 or 500

	return ReplyJSON(web, res)
end

local function CommentAppstorePost(web)
	local vcc = inconcert.authUser.virtualContactCenter
	local agentAtVCC = utils.GetCurrentLoggedUser()

	LogInfo("Comment appstore post request from agent:%s, post_data: '%s'.", agentAtVCC, web.input["post_data"])

	if not hasLicensesForChannel( "APPSTORE" ) then
		local reason = LogFatal("Unable to comment appstore post. Not has licensed for the Appstore channel.")
		web.status = 500
		return ReplyJSON(web, {
				agent_err = true,
				result = "NOT_LICENSED",
				reason = reason
			})
	end

	local appstoreRequest = json.decode(web.input["post_data"])
	--this is done in order to avoid api manglers creating interactions from another vcc from the one they're logged in
	local campaignAtVCC = helpers.AddVCC(appstoreRequest.campaign, vcc)

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local res = irLink:AppstoreCommentPost({
		fake_event_id = appstoreRequest.fake_event_id,
		interactionId = appstoreRequest.interactionId,
		campaignAtVCC = campaignAtVCC,
		account = appstoreRequest.account,
		agentAtVCC = agentAtVCC,
		postId = appstoreRequest.postId,
		commentId = appstoreRequest.commentId,
		message = appstoreRequest.message,
		rid = web.vars.HTTP_X_ICC_RID
	})
	local http_code = http_mapping.getHTTPCode(res["result"])
	web.status = http_code or 500 --unknown result code
	return ReplyJSON(web, res)
end

local function CommentGoogleplayPost(web)
	local vcc = inconcert.authUser.virtualContactCenter
	local agentAtVCC = utils.GetCurrentLoggedUser()

	LogInfo("Comment googleplay post request from agent:%s, post_data: '%s'.", agentAtVCC, web.input["post_data"])

	if not hasLicensesForChannel( "GOOGLEPLAY" ) then
		local reason = LogFatal("Unable to comment googleplay post. Not has licensed for the Googleplay channel.")
		web.status = 500
		return ReplyJSON(web, {
				agent_err = true,
				result = "NOT_LICENSED",
				reason = reason
			})
	end

	local googleplayRequest = json.decode(web.input["post_data"])
	--this is done in order to avoid api manglers creating interactions from another vcc from the one they're logged in
	local campaignAtVCC = helpers.AddVCC(googleplayRequest.campaign, vcc)

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local res = irLink:GoogleplayCommentPost({
		fake_event_id = googleplayRequest.fake_event_id,
		interactionId = googleplayRequest.interactionId,
		campaignAtVCC = campaignAtVCC,
		account = googleplayRequest.account,
		agentAtVCC = agentAtVCC,
		postId = googleplayRequest.postId,
		commentId = googleplayRequest.commentId,
		message = googleplayRequest.message,
		rid = web.vars.HTTP_X_ICC_RID
	})
	local http_code = http_mapping.getHTTPCode(res["result"])
	web.status = http_code or 500 --unknown result code
	return ReplyJSON(web, res)
end

--- Extracts the bcc address from mail request
-- also will check if the account has configured automatic bcc.
--
-- @tparam table mailRequest
-- @tparam table mailRequest.bcc
--
-- @treturn string mailBCC
local function ExtractMailBCC(mailRequest, account)
	local mailBCC = mailRequest.bcc or ""
	local mailAccount = GetModel("Campaigns").FindAccountById( {
		proxyType = "MAIL",
		id = account
	})
	if mailAccount and mailAccount.outgoingBCC and #mailAccount.outgoingBCC > 0 then
		mailBCC = mailBCC .. (mailBCC ~= "" and ";" or "") .. mailAccount.outgoingBCC
	end
	return mailBCC
end

--- Will send a new mail, could be over an existing one.
--
-- @tparam table web
-- @tparam table web.input
-- @tparam string web.input.post_data Data posted on the API
function SendMail(web)
	local user_name = utils.GetCurrentLoggedUser()
	local mailRequest = json.decode(web.input["post_data"])
	local RID = web.vars.HTTP_X_ICC_RID

	LogInfo("Send Mail request from agent:%s, id:%s from:%s to:%s subject:%s", user_name, mailRequest.id, mailRequest.from, mailRequest.to, mailRequest.subject)

	if not hasLicensesForChannel( "MAIL" ) then
		local reason = LogFatal("Unable to send e-mail. Not has licensed for the e-mail channel.")
		web.status = 500
		return ReplyJSON(web, {
			result = "NOT_LICENSED",
			reason = reason
		})
	end

	local interaction
	local validate = function()
		local response = { success = false }

		-- Estas validaciones tambien se realizan en el IR en MAIL.lua "SendOutMail".
		-- Igual las dejamos aca
		interaction = GetModel("Interactions").GetInteraction(mailRequest.id)
		if utils.TableIsEmpty(interaction) then
			response.result = "INTERACTION_NOT_FOUND"
			response.error_message = string.format("Error sending interaction e-mail. Interaction id '%s' does not exist..", mailRequest.id)
			return response
		end

		if interaction.state ~= "TAKEN" and interaction.state ~= "TRANSFER TAKEN" then
			response.result = "INTERACTION_NOT_TAKEN"
			response.error_message = string.format("Unable to sending e-mail interaction '%s'. The current state is: '%s'.", mailRequest.id, interaction.state)
			return response
		else
			local owner = interaction.assignedAgents and interaction.assignedAgents[1] or ""
			local loggedUserId = utils.GetCurrentLoggedUser()
			if owner:lower() ~= loggedUserId:lower() then
				response.result = "INTERACTION_NOT_TAKEN_BY_AGENT"
				response.error_message = string.format("Unable to sending e-mail interaction '%s' by user '%s'. The interaction belongs to the user '%s'.", mailRequest.id, loggedUserId, owner)
				return response
			end
		end

		response.success = true
		return response
	end

	local response = validate()
	if not response.success then
		LogError( response.error_message )
		web.status = 500
		return ReplyJSON( web, response )
	end

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local mailBCC = ExtractMailBCC(mailRequest, interaction.account)

	local res = irLink:MailSend(mailRequest.id,
		interaction.campaign,
		user_name,
		mailRequest.from,
		mailRequest.to,
		mailRequest.subject,
		mailRequest.body,
		mailRequest.attachments and json.encode(mailRequest.attachments) or "",
		mailRequest.cc and mailRequest.cc or "",
		mailBCC,
		RID
	)

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(res["result"])
	web.status = http_code or 500 --unknown result code

	return ReplyJSON(web, res)
end

--- Will start a new mail from the scratch.
-- Internally, IR creates the interaction and send the mail.
--
-- @tparam table web
-- @tparam table web.input
-- @tparam string web.input.post_data Data posted on the API
function MailCreateAndSend (web)

	local user_name = utils.GetCurrentLoggedUser()
	local RID = web.vars.HTTP_X_ICC_RID
	local request = web.request

	LogInfo("MailCreateAndSend request from agent: '%s, id: %s campaign: %s account: %s, to: %s, subject: %s", user_name, request.id, request.campaign, request.account, request.from, request.to, request.subject)

	if not hasLicensesForChannel( "MAIL" ) then
		local reason = LogFatal("Unable to create and send e-mail. Not has licensed for the e-mail channel.")
		web.status = 500
		return ReplyJSON(web, {
			result = "NOT_LICENSED",
			reason = reason
		})
	end

	local validate = function()
		local response = { success = false }

		if not HasPermission( "CREATE_OUTGOING", request.campaign, "interaction" ) then
			response.result = "FORBIDDEN"
			response.error_message = "You not have permission for create outgoing e-mail conversation."
			return response
		end

		local interaction = GetModel("Interactions").GetInteraction(request.id)
		if not utils.TableIsEmpty(interaction) then
			response.result = "INTERACTION_ALREADY_EXIST"
			response.error_message = string.format("Error creating interaction e-mail. Interaction id '%s' already exist..", request.id)
			return response
		end

		local to = request.to
		if type( to ) ~= "string" then
			response.result = "TO_ADDRESS_IS_EMPTY"
			response.error_message = "Error sending e-mail. TO e-mail address is empty."
			return response
		end

		local address_separator = ";"
		local position = to:find( address_separator )
		-- Aca valido si llega un ';' que es el separador de direcciones de correo,
		--   1 - Si no llega (position = nil), continua.
		--   2 - Si llega (position = number), valido que despues del ';' no exista ningun caracter (o sea ninguna direccion de correo).
		if position ~= nil and ( position ~= utils.rtrim( to ):len() ) then
			response.result = "TO_ADDRESS_EXCEEDED_LIMIT_OF_ALLOWED_ADDRESSES"
			response.error_message = "Error sending e-mail. Only 1 address is allowed in TO e-mail."
			return response
		end

		response.success = true
		return response
	end

	local response = validate()
	if not response.success then
		LogError( response.error_message )
		web.status = 403
		return ReplyJSON( web, response )
	end

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)

	local mailBCC = ExtractMailBCC(request, request.account)

	--this is done in order to avoid api manglers creating interactions from another vcc from the one they're logged in
	local vcc = inconcert.authUser.virtualContactCenter
	local campaignIdAtVCC = helpers.AddVCC(request.campaign, vcc)
	local res = irLink:MailCreateAndSend({
		interactionID       = request.id,
		campaignAtVCC       = campaignIdAtVCC,
		agentAtVCC          = user_name,
		account             = request.account,
		attention_level_id	= request.attention_level_id,
		contact             = request.contact,
		from                = request.from,
		to                  = request.to,
		cc                  = request.cc or "",
		bcc                 = mailBCC,
		subject             = request.subject,
		body                = request.body,
		attachments         = request.attachments and json.encode(request.attachments) or "",
		parentInteractionId = request.parentInteractionId,
		RID                 = RID
	})

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(res["result"])
	web.status = http_code or 500 --unknown result code

	return ReplyJSON(web, res)
end

-- Mail
local function LoadFoldersMailAccount ( web, campaignId, accountId )
	local vcc = inconcert.authUser.virtualContactCenter
	local name = inconcert.authUser.userName

	LogInfo("[INTERACTIONS:LoadFoldersMailAccount] User '%s' on vcc '%s' Loading Folders for '%s' Account Mail in campaign '%s'", name, vcc, accountId, campaignId)

	if not hasLicensesForChannel( "MAIL" ) then
		local reason = LogFatal("[INTERACTIONS:LoadFoldersMailAccount] Unable to Load e-mail Account Folders. Not has licensed for the e-mail channel.")
		web.status = 500
		return ReplyJSON(web, {
				agent_err = true,
				result = "NOT_LICENSED",
				reason = reason
			})
	end

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local campaignIdAtVCC = helpers.AddVCC(campaignId, vcc)
	local agentAtVCC = utils.GetCurrentLoggedUser()

	local res = irLink:SendCommand({
		RID       = web.vars.HTTP_X_ICC_RID,
		proxyType = "MAIL",
		agent     = agentAtVCC,
		command   = "LoadFoldersAccount",
		campaign  = campaignIdAtVCC,
		account   = accountId
	})

	local http_code = http_mapping.getHTTPCode(res["result"])
	web.status = http_code or 500 --unknown result code

	if res.result == "OK" then
		LogInfo("[INTERACTIONS:LoadFoldersMailAccount] Loaded Folders for '%s' Account Mail in campaign '%s'. HTTP code '%s', result: '%s'.", accountId, campaignIdAtVCC, http_code, json.encode(res))
		return ReplyJSON(web, res.folders)
	else
		LogError("[INTERACTIONS:LoadFoldersMailAccount] Error Loading Folders '%s' Account Mail in campaign '%s'. HTTP code '%s', result: '%s'", accountId, campaignIdAtVCC, http_code, json.encode(res))
		return ReplyJSON(web, res)
	end
end

function MoveMailToFolder (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local name = inconcert.authUser.userName
	local postData = web.input["post_data"]
	local request = json.decode(postData)

	LogInfo("[INTERACTIONS:MoveMailToFolder] User '%s' on vcc '%s' Moving Mail To Folder. Info '%s'", name, vcc, postData)

	if not hasLicensesForChannel("MAIL") then
		local reason = LogFatal("[INTERACTIONS:MoveMailToFolder] Unable to Move e-mail to Folder. Not has licensed for the e-mail channel.")
		web.status = 500
		return ReplyJSON(web, {
				result = "NOT_LICENSED",
				reason = reason
			})
	end

	local appId = request.appId

	if not HasPermission("MOVE_MAIL_TO_FOLDER", nil, appId) then
		web.status = 403
		return ReplyJSON(web, {
				result = "NOT_HAS_PERMISSION",
				reason = "Not has permission to move e-mail to folder"
			})
	end

	local campaignId = request.campaign
	local accountId = request.account
	local interactionId = request.interaction.id
	local fromFolder = request.fromFolder
	local toFolder = request.toFolder
	local finishInfo = request.finishInfo
	local labelInfo = request.labelInfo
	local emailsId = request.emailsId

	if not emailsId then
		LogInfo("[INTERACTIONS:MoveMailToFolder] All mails (events) of '%s' interaction will be moved folder: from '%s'-> to '%s'.", interactionId, fromFolder, toFolder)

		local currentInteraction = GetModel("Interactions").GetInteraction(interactionId)
		if utils.TableIsEmpty(currentInteraction) then
			local reason = LogFatal("[INTERACTIONS:MoveMailToFolder] Unable to Move e-mail to Folder. Not has licensed for the e-mail channel.")
			web.status = 500
			return ReplyJSON({
				result = "INTERACTION_NOT_FOUND",
				reason = ("Interaction %s does not exist"):format(interactionId)
			})
		end

		local CanAddMailEvent = function(mailEvent)
			-- unicamente se agregan eventos entrantes del email
			if mailEvent.creator == "contact" then
				return true
			end
			return false
		end
		emailsId = {}
		for _, mailEvent in ipairs(currentInteraction.events or {}) do
			if CanAddMailEvent(mailEvent) then
				table.insert(emailsId, mailEvent.id)
			end
		end
	end

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local campaignIdAtVCC = helpers.AddVCC(campaignId, vcc)
	local agentAtVCC = utils.GetCurrentLoggedUser()

	local res = irLink:SendCommand({
		RID           = web.vars.HTTP_X_ICC_RID,
		proxyType     = "MAIL",
		agent         = agentAtVCC,
		command       = "MoveToFolder",
		campaign      = campaignIdAtVCC,
		account       = accountId,
		interactionId = interactionId,
		fromFolder    = fromFolder,
		toFolder      = toFolder,
		emailsId      = emailsId
	})

	local http_code = http_mapping.getHTTPCode(res["result"])
	web.status = http_code or 500 --unknown result code

	if res.result == "OK" then
		LogInfo("[INTERACTIONS:MoveMailToFolder] Success Moved Mail to Folder. Info '%s'. HTTP code '%s', result: '%s'.", postData, http_code, json.encode(res))

		-- LABEL
		if labelInfo and labelInfo.must_label_interaction then
			local hasPermissionToLabel = HasPermission("LABEL", campaignIdAtVCC, appId)
			if not hasPermissionToLabel then
				web.status = 403
				return false, ReplyJSON(web, { result = "FORBIDDEN", reason = string.format("Fobidden: Unable to add label to interaction: %s", interactionId) })
			end

			local action = labelInfo.action:lower()
			if action == "add" then
				LogInfo("[INTERACTIONS:MoveMailToFolder] Add Label to Interaction '%s'. Label: '%s'.", interactionId, labelInfo.label.id)

				local newLabels = {}
				newLabels[labelInfo.label.id] = labelInfo.label
				local label_result = AddLabels(interactionId, newLabels, campaignIdAtVCC)
				if not label_result.success then
					LogWarning("[INTERACTIONS:MoveMailToFolder] Unable to Add Label to Interaction '%s'. Label: '%s'.", interactionId, labelInfo.label.id)
					-- TODO: METER UN ROLLBACK AL MOVE TO FOLDER ????
				end
			elseif action == "remove" then
				LogInfo("[INTERACTIONS:MoveMailToFolder] Remove Label to Interaction '%s'. Label: '%s'.", interactionId, labelInfo.label.id)

				local removeLabels = {}
				removeLabels[labelInfo.label.id] = labelInfo.label
				local label_result = DeleteLabels(interactionId, removeLabels, campaignIdAtVCC)
				if not label_result.success then
					LogWarning("[INTERACTIONS:MoveMailToFolder] Unable to Remove Label to Interaction '%s'. Label: '%s'.", interactionId, labelInfo.label.id)
					-- TODO: METER UN ROLLBACK AL MOVE TO FOLDER ????
				end
			end
		end

		return ReplyJSON(web, res)
	else
		LogError("[INTERACTIONS:MoveMailToFolder] Error Moving Mail to Folder. Info '%s'. HTTP code '%s', result: '%s'", postData, http_code, json.encode(res))
		return ReplyJSON(web, res)
	end
end


--- [Twitter] Will reply a tweet on an interaction
--
-- @tparam table  web
-- @tparam table  web.input
-- @tparam string web.input.post_data  - Data posted on the API
--
-- if we decode post_data json string:
-- table  request
-- string request.interactionID  - The interaction id
-- string request.campaignId     - Campaign id (i.e "campaign@vcc")
-- string request.account        - The account id (i.e "TWITTER_campaign@vcc_username")
-- string request.status         - The tweet text (may contain a @username inside)
-- string request.replyTo        - Tweet id to reply to
--
function ReplyTweet(web)
	local _T = m_htmlRender.extension.makeTextTranslator()
	local vcc = inconcert.authUser.virtualContactCenter
	local logged_user = utils.GetCurrentLoggedUser()
	local request = json.decode(web.input["post_data"])

	LogInfo("Tweet from agent: '%s', on interaction id: '%s', campaign: '%s', account: '%s', tweet: '%s', response to: '%s'",
			logged_user, request.interactionID, request.campaignId, request.account, request.status, request.replyTo)

	if not hasLicensesForChannel( "TWITTER" ) then
		local reason = LogFatal("Unable to reply tweet. Not has licensed for the Twitter channel.")
		web.status = 500
		return ReplyJSON(web, {
				agent_err = true,
				result = "NOT_LICENSED",
				reason = _T(reason)
			})
	end

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)

	--this is done in order to avoid api manglers creating interactions from another vcc from the one they're logged in
	local campaignIdAtVCC = helpers.AddVCC(request.campaignId, vcc)
	local res = irLink:ReplyTweet(request.interactionID,
		campaignIdAtVCC,
		logged_user,
		request.account,
		request.status,
		request.replyTo,
		request.exclude_reply_user_ids or ""
	)

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(res["result"])
	web.status = http_code or 500 --unknown result code

	if res.result ~= "OK" then
		LogError("Error replying tweet on interaction id: '%s' -> '%s'", request.interactionID, res.reason.message)
		return ReplyJSON(web, { result = res.result, reason = _T(res.reason.message) })
	end

	LogInfo("Success replying tweet on interaction '%s'", request.interactionID)
	return ReplyJSON(web)
end

--- [Twitter] Will send a tweet
--
-- @tparam table  web
-- @tparam table  web.input
-- @tparam string web.input.post_data  - Data posted on the API
--
-- if we decode post_data json string:
-- table  request
-- string request.interactionID  - The interaction id
-- string request.campaignId     - Campaign id (i.e "campaign@vcc")
-- string request.account        - The account id (i.e "TWITTER_campaign@vcc_username")
-- string request.status         - The tweet text (may contain a @username inside)
--
function Tweet(web)
	local _T = m_htmlRender.extension.makeTextTranslator()
	local vcc = inconcert.authUser.virtualContactCenter
	local user_name = utils.GetCurrentLoggedUser()
	local request = json.decode(web.input["post_data"])

	LogInfo("Tweet from agent: '%s', on interaction id: '%s', campaign: '%s', account: '%s', tweet: '%s'",
			user_name, request.interactionID, request.campaignId, request.account, request.status)

	if not hasLicensesForChannel( "TWITTER" ) then
		local reason = LogFatal("Unable to tweet. Not has licensed for the Twitter channel.")
		web.status = 500
		return ReplyJSON(web, {
				agent_err = true,
				result = "NOT_LICENSED",
				reason = _T(reason)
			})
	end

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)

	--this is done in order to avoid api manglers creating interactions from another vcc from the one they're logged in
	local campaignIdAtVCC = helpers.AddVCC(request.campaignId, vcc)
	local res = irLink:Tweet(request.interactionID,
		campaignIdAtVCC,
		user_name,
		request.account,
		request.status,
		request.attachment_url or ""
	)

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(res["result"])
	web.status = http_code or 500 --unknown result code

	if res.result ~= "OK" then
		LogError("Error tweeting on interaction id: '%s' -> '%s'", request.interactionID, res.reason.message)
		return ReplyJSON(web, { result = res.result, reason = _T(res.reason.message) })
	end

	LogInfo("Success tweeting on interaction '%s'", request.interactionID)
	return ReplyJSON(web)
end

--- [Twitter] Will re-tweet a tweet.
--
-- @tparam table  web
-- @tparam table  web.input
-- @tparam string web.input.post_data  - Data posted on the API
--
-- if we decode post_data json string:
-- table  request
-- string request.interactionID  - The interaction id.
-- string request.campaignId     - Campaign id (i.e "campaign@vcc")
-- string request.account        - The account id (i.e "TWITTER_campaign@vcc_username")
-- string request.tweet_id       - The tweet to be retweeted.
--
function Retweet(web)
	local _T = m_htmlRender.extension.makeTextTranslator()
	local vcc = inconcert.authUser.virtualContactCenter
	local user_name = utils.GetCurrentLoggedUser()
	local request = json.decode(web.input["post_data"])

	LogInfo("ReTweet from agent: '%s', on interaction id: '%s', campaign: '%s', account: '%s', retweeted tweet: '%s'",
			user_name, request.interactionID, request.campaignId, request.account, request.tweet_id)

	if not hasLicensesForChannel( "TWITTER" ) then
		local reason = LogFatal("Unable to retweet. Not has licensed for the Twitter channel.")
		web.status = 500
		return ReplyJSON(web, {
				agent_err = true,
				result = "NOT_LICENSED",
				reason = _T(reason)
			})
	end

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)

	--this is done in order to avoid api manglers creating interactions from another vcc from the one they're logged in
	local campaignIdAtVCC = helpers.AddVCC(request.campaignId, vcc)
	local res = irLink:ReTweet(request.interactionID,
		campaignIdAtVCC,
		user_name,
		request.account,
		request.tweet_id
	)

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(res["result"])
	web.status = http_code or 500 --unknown result code

	if res.result ~= "OK" then
		LogError("Error retweeting on interaction id: '%s' -> '%s'", request.interactionID, res.reason.message)
		return ReplyJSON(web, { result = res.result, reason = _T(res.reason.message) })
	end

	LogInfo("Success retweeting on interaction '%s'", request.interactionID)
	return ReplyJSON(web)
end

--- [Twitter] Will send a direct message
--
-- @tparam table  web
-- @tparam table  web.input
-- @tparam string web.input.post_data  - Data posted on the API
--
-- if we decode post_data json string:
-- table  request
-- string request.interactionID  - The interaction id.
-- string request.campaignId     - Campaign id (i.e "campaign@vcc")
-- string request.account        - The account id (i.e "TWITTER_campaign@vcc_username")
-- string request.tweet_id       - The tweet to be retweeted.
--
function SendDirectMessage(web)
	local _T = m_htmlRender.extension.makeTextTranslator()
	local vcc = inconcert.authUser.virtualContactCenter
	local user_name = utils.GetCurrentLoggedUser()
	local request = json.decode(web.input["post_data"])

	LogInfo("Sending direct message from agent: '%s', to Twitter user: '%s', on interaction id: '%s', campaign: '%s', account: '%s', message: '%s'",
			user_name, request.target_user, request.interactionID, request.campaignId, request.account, request.message)

	if not hasLicensesForChannel( "TWITTER" ) then
		local reason = LogFatal("Unable to direct message. Not has licensed for the Twitter channel.")
		web.status = 500
		return ReplyJSON(web, {
			agent_err = true,
			result = "NOT_LICENSED",
			reason = _T(reason)
		})
	end

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)

	--this is done in order to avoid api manglers creating interactions from another vcc from the one they're logged in
	local campaignIdAtVCC = helpers.AddVCC(request.campaignId, vcc)
	local res = irLink:TwitterSendDirectMessage({
		interactionID = request.interactionID,
		campaignAtVCC = campaignIdAtVCC,
		agentAtVCC = user_name,
		account = request.account,
		targetUser = request.target_user,
		targetUserId = request.target_user_id,
		message = request.message
	})

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(res["result"])
	web.status = http_code or 500 --unknown result code

	if res.result ~= "OK" then
		LogError("Error sending Twitter direct message on interaction id: '%s' -> '%s'", request.interactionID, res.reason.message)
		return ReplyJSON(web, { result = res.result, reason = _T(res.reason.message) })
	end

	LogInfo("Success sending Twitter direct message on interaction '%s'", request.interactionID)
	return ReplyJSON(web)
end

--- [Twitter] Will send a direct message
--
-- @tparam table  web
-- @tparam table  web.input
-- @tparam string web.input.post_data  - Data posted on the API
--
-- if we decode post_data json string:
-- table  request
--
function SendDirectMessageFromTweet(web)
	local _T = m_htmlRender.extension.makeTextTranslator()
	local vcc = inconcert.authUser.virtualContactCenter
	local user_name = utils.GetCurrentLoggedUser()
	local request = json.decode(web.input["post_data"])

	LogInfo("Sending direct message from tweet from agent: '%s', to Twitter user: '%s', on interaction id: '%s', campaign: '%s', account: '%s', message: '%s'",
			user_name, request.target_user, request.interactionID, request.campaignId, request.account, request.message)

	if not hasLicensesForChannel( "TWITTER" ) then
		local reason = LogFatal("Unable to direct message. Not has licensed for the Twitter channel.")
		web.status = 500
		return ReplyJSON(web, {
			agent_err = true,
			result = "NOT_LICENSED",
			reason = _T(reason)
		})
	end

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)

	--this is done in order to avoid api manglers creating interactions from another vcc from the one they're logged in
	local campaignIdAtVCC = helpers.AddVCC(request.campaignId, vcc)
	local res = irLink:TwitterSendDirectMessageFromTweet({
		interactionID = request.interactionID,
		campaignAtVCC = campaignIdAtVCC,
		agentAtVCC = user_name,
		account = request.account,
		targetUser = request.target_user,
		targetUserId = request.target_user_id,
		message = request.message,
		tweet = request.tweet and json.encode(request.tweet) or ""
	})

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(res["result"])
	web.status = http_code or 500 --unknown result code

	if res.result ~= "OK" then
		LogError("Error sending Twitter direct message from tweet on interaction id: '%s' -> '%s'", request.interactionID, res.reason)
		res.reason = _T(res.reason)
		return ReplyJSON(web, res)
	end

	LogInfo("Success sending Twitter direct message from tweet on interaction '%s'", request.interactionID)
	return ReplyJSON(web)
end


--- [Twitter] Will create a new Twitter interaction
--
-- @tparam table  web
-- @tparam table  web.input
-- @tparam string web.input.post_data  - Data posted on the API
--
-- if we decode post_data json string:
-- table  request
-- string request.id             - The interaction id.
-- string request.campaign       - Campaign id (i.e "campaign@vcc")
-- string request.account        - The account id (i.e "TWITTER_campaign@vcc_username")
--
function CreateTwitterInteraction(web)
	local _T = m_htmlRender.extension.makeTextTranslator()
	local vcc = inconcert.authUser.virtualContactCenter
	local user_name = utils.GetCurrentLoggedUser()
	local rid = web.vars.HTTP_X_ICC_RID or "n/a"

	local request = json.decode(web.input["post_data"])

	LogInfo("Create twitter request from agent: '%s', id: '%s', campaign: '%s', account: '%s'", user_name, request.id, request.campaign, request.account)

	if not HasPermission( "CREATE_OUTGOING", request.campaign, "interaction" ) then
		web.status = 403
		return ReplyJSON( web, { result = "FORBIDDEN", reason = _T("You not have permission for create outgoing tweet conversation.") })
	end

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)

	--this is done in order to avoid api manglers creating interactions from another vcc from the one they're logged in
	local campaignIdAtVCC = helpers.AddVCC(request.campaign, vcc)
	local res = irLink:TwitterCreate(request.id,
		campaignIdAtVCC,
		user_name,
		request.account,
		request.destination or "", -- timeline's tweets does not have destination.
		request.parentInteractionId or "",
		rid
	)

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(res["result"])
	web.status = http_code or 500 --unknown result code

	if res.result ~= "OK" then
		LogError("Error creating Twitter interaction id: '%s' -> '%s'", request.id, res.reason.message)
		return ReplyJSON(web, { result = res.result, reason = _T(res.reason.message) })
	end

	LogInfo("Success creating Twitter interaction '%s'", request.id)
	return ReplyJSON(web)
end

local function LikeTwitterObject(web)
	local vcc = inconcert.authUser.virtualContactCenter
	local user_name = inconcert.authUser.userName
	local agentAtVCC = utils.GetCurrentLoggedUser()

	LogInfo("Like twitter object request from agent:%s, post_data: '%s'.", agentAtVCC, web.input["post_data"])

	if not hasLicensesForChannel( "TWITTER" ) then
		local reason = LogFatal("Unable like object. Not has licensed for the Twitter channel.")
		web.status = 500
		return ReplyJSON(web, {
				agent_err = true,
				result    = "NOT_LICENSED",
				reason    = reason
			})
	end

	local request = json.decode(web.input["post_data"])
	--this is done in order to avoid api manglers creating interactions from another vcc from the one they're logged in
	local campaignAtVCC = helpers.AddVCC(request.campaign, vcc)
	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local res = irLink:TwitterLikeObject({
		fake_event_id = request.fake_event_id,
		rid = web.vars.HTTP_X_ICC_RID,
		campaignAtVCC = campaignAtVCC,
		account = request.account,
		interactionID = request.interactionId,
		agentAtVCC = agentAtVCC,
		objectId = request.objectId,
		like = request.like and true or false
	})
	
	local http_code = http_mapping.getHTTPCode(res["result"])
	web.status = http_code or 500 --unknown result code

	return ReplyJSON(web, res)
end

local function GetTwitterContentMedia(wsapi_env)
	local user_name = inconcert.authUser.userName
	local request = wsapi.request.new(wsapi_env)

	local make_body = function(content)
		return coroutine.wrap(function() coroutine.yield(content) end)
	end
	
	--LogInfo("Getting Twitter media content for user:%s - media url:'%s' %s", user_name, request.GET.url, json.encode(request))
	--send request to server
	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local result = irLink:SendCommand({
			proxyType = "TWITTER",
			command = "GetMedia",
			url = request.GET.url,
			campaignAtVCC = request.GET.campaignAtVCC, 
			accountId = request.GET.accountId
	})

	if result.result == "OK" then
		return 200, result.headers or {}, make_body(base64.decode(result.content))
	else
		local http_code = http_mapping.getHTTPCode(result.result)
		return http_code or 500, {}, result.reason
	end
end

--- [Sms] Will send a message
--
-- @tparam table  web
-- @tparam table  web.input
-- @tparam string web.input.post_data  - Data posted on the API
--
-- if we decode post_data json string:
-- table  request
-- string request.interactionID  - The interaction id.
-- string request.campaign		 - Campaign id (i.e "campaign@vcc")
-- string request.account        - The account id (i.e "TWITTER_campaign@vcc_username")
-- object request.message        - The message
--
function SendSms(web)
	local _T = m_htmlRender.extension.makeTextTranslator()
	local vcc = inconcert.authUser.virtualContactCenter
	local user_name = utils.GetCurrentLoggedUser()
	local request = json.decode(web.input["post_data"])

	LogInfo("Sending sms from agent: '%s', on interaction id: '%s', campaign: '%s', account: '%s', message: '%s'",
		user_name, request.interactionID, request.campaign, request.account, json.encode(request.message))

	if not hasLicensesForChannel( "SMS" ) then
		local reason = LogFatal("Unable to send message. Not has licensed for the Sms channel.")
		web.status = 500
		return ReplyJSON(web, {
			agent_err = true,
			result = "NOT_LICENSED",
			reason = _T(reason)
		})
	end

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	
	local res = irLink:SMSSendInteractionMessage({
		interactionID = request.interactionID,
		message = request.message,
		agentAtVCC = user_name
	})

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(res["result"])
	web.status = http_code or 500 --unknown result code

	if res.result ~= "OK" then
		LogError("Error sending sms on interaction id: '%s' -> '%s'", request.interactionID, res.reason)
		return ReplyJSON(web, { result = res.result, reason = _T(res.reason) })
	end

	LogInfo("Success sending sms on interaction '%s'", request.interactionID)
	return ReplyJSON(web)
end

--- [Sms] Will create a new Sms interaction
--
-- @tparam table  web
-- @tparam table  web.input
-- @tparam string web.input.post_data  - Data posted on the API
--
-- if we decode post_data json string:
-- table  request
-- string request.interactionID  - The interaction id.
-- string request.campaign       - Campaign id (i.e "campaign@vcc")
-- string request.account        - The account id (i.e "TWITTER_campaign@vcc_username")
-- object request.message        - The message
--
function SmsCreateAndSend(web)
	local _T = m_htmlRender.extension.makeTextTranslator()
	local vcc = inconcert.authUser.virtualContactCenter
	local user_name = utils.GetCurrentLoggedUser()
	local rid = web.vars.HTTP_X_ICC_RID or "n/a"

	local request = json.decode(web.input["post_data"])

	LogInfo("Create sms request from agent: '%s', id: '%s', campaign: '%s', account: '%s'", 
		user_name, request.interactionID, request.campaign, request.account)

	if not HasPermission( "CREATE_OUTGOING", request.campaign, "interaction" ) then
		web.status = 403
		return ReplyJSON( web, { result = "FORBIDDEN", reason = _T("You not have permission for create outgoing sms conversation.") })
	end

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)

	--this is done in order to avoid api manglers creating interactions from another vcc from the one they're logged in
	local campaignIdAtVCC = helpers.AddVCC(request.campaign, vcc)
	local res = irLink:SMSCreateAndSendInteractionMessage({
		interactionID = request.interactionID,
		parentInteractionId = request.parentInteractionId,
		campaignAtVCC = campaignIdAtVCC,
		account = request.account,
		attention_level_id	= request.attention_level_id,
		phoneNumber = request.phoneNumber,
		contactID = request.contactID,
		message = request.message,
		agentAtVCC = user_name
	})

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(res["result"])
	web.status = http_code or 500 --unknown result code

	if res.result ~= "OK" then
		LogError("Error creating Sms interaction id: '%s' -> '%s'", request.interactionID, res.reason)
		res.reason = _T(res.reason)
		return ReplyJSON(web, res)
	end

	LogInfo("Success creating Sms interaction '%s'", request.interactionID)
	return ReplyJSON(web)
end

local function SmsForward(web)
	local _T = m_htmlRender.extension.makeTextTranslator()
	local vcc = inconcert.authUser.virtualContactCenter
	local user_name = utils.GetCurrentLoggedUser()
	local request = json.decode(web.input["post_data"])

	LogInfo("Forwarding messages from agent: '%s', to address: '%s', on interaction id: '%s', campaign: '%s', account: '%s'",
		user_name, request.address, request.interactionID, request.campaignId, request.account)

	if not hasLicensesForChannel( "SMS" ) then
		local reason = LogFatal("Unable to forward. Not has licensed for the Sms channel.")
		web.status = 500
		return ReplyJSON(web, {
			agent_err = true,
			result = "NOT_LICENSED",
			reason = _T(reason)
		})
	end

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)

	--this is done in order to avoid api manglers creating interactions from another vcc from the one they're logged in
	local campaignIdAtVCC = helpers.AddVCC(request.campaignId, vcc)
	local res = irLink:SMSForward({
		campaignAtVCC = campaignIdAtVCC,
		account = request.account,
		interactionID = request.interactionID,
		agentAtVCC = user_name,
		address = request.address,
		messages = request.messages,
	})

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(res["result"])
	web.status = http_code or 500 --unknown result code

	if res.result ~= "OK" then
		LogError("Error forwarding sms messages to address %s on interaction id: '%s' -> '%s'", 
			request.address, request.interactionID, res.reason)
		res.reason = _T(res.reason)
		return ReplyJSON(web, res)
	end

	LogInfo("Success forwarding sms messages to address %s on interaction '%s'", 
		request.address, request.interactionID)
	return ReplyJSON(web, res)
end

--- [Whatsapp] Will send a message
--
-- @tparam table  web
-- @tparam table  web.input
-- @tparam string web.input.post_data  - Data posted on the API
--
-- if we decode post_data json string:
-- table  request
-- string request.interactionID  - The interaction id.
-- string request.campaign		 - Campaign id (i.e "campaign@vcc")
-- string request.account        - The account id (i.e "TWITTER_campaign@vcc_username")
-- object request.message        - The message
--
function SendWhatsapp(web)
	local _T = m_htmlRender.extension.makeTextTranslator()
	local vcc = inconcert.authUser.virtualContactCenter
	local user_name = utils.GetCurrentLoggedUser()
	local request = json.decode(web.input["post_data"])

	LogInfo("Sending Whatsapp from agent: '%s', on interaction id: '%s', campaign: '%s', account: '%s', message: '%s'",
		user_name, request.interactionID, request.campaign, request.account, json.encode(request.message))

	if not hasLicensesForChannel( "WHATSAPP" ) then
		local reason = LogFatal("Unable to send message. Not has licensed for the Whatsapp channel.")
		web.status = 500
		return ReplyJSON(web, {
			agent_err = true,
			result = "NOT_LICENSED",
			reason = _T(reason)
		})
	end

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	
	local res = irLink:WhatsappSendInteractionMessage({
		interactionID = request.interactionID,
		message = request.message,
		agentAtVCC = user_name
	})

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(res["result"])
	web.status = http_code or 500 --unknown result code

	if res.result ~= "OK" then
		LogError("Error sending Whatsapp on interaction id: '%s' -> '%s'", request.interactionID, res.reason)
		res.reason = _T(res.reason)
		return ReplyJSON(web, res)
	end

	LogInfo("Success sending Whatsapp on interaction '%s'", request.interactionID)
	return ReplyJSON(web)
end

function SendInternalWhatsapp(web)
	local _T = m_htmlRender.extension.makeTextTranslator()
	local vcc = inconcert.authUser.virtualContactCenter
	local user_name = utils.GetCurrentLoggedUser()
	local request = json.decode(web.input["post_data"])

	LogInfo("Sending Whatsapp from agent: '%s', on interaction id: '%s', campaign: '%s', account: '%s', message: '%s'",
		user_name, request.interactionID, request.campaign, request.account, json.encode(request.message))

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	
	local res = irLink:WhatsappSendInternalMessage({
		interactionID = request.interactionID,
		message = request.message,
		agentAtVCC = user_name
	})

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(res["result"])
	web.status = http_code or 500 --unknown result code

	if res.result ~= "OK" then
		LogError("Error sending Whatsapp on interaction id: '%s' -> '%s'", request.interactionID, res.reason)
		res.reason = _T(res.reason)
		return ReplyJSON(web, res)
	end

	LogInfo("Success sending Whatsapp on interaction '%s'", request.interactionID)
	return ReplyJSON(web)
end

--- [Whatsapp] Will create a new Whatsapp interaction
--
-- @tparam table  web
-- @tparam table  web.input
-- @tparam string web.input.post_data  - Data posted on the API
--
-- if we decode post_data json string:
-- table  request
-- string request.interactionID  - The interaction id.
-- string request.campaign       - Campaign id (i.e "campaign@vcc")
-- string request.account        - The account id (i.e "TWITTER_campaign@vcc_username")
-- object request.message        - The message
--
function WhatsappCreateAndSend(web)
	local _T = m_htmlRender.extension.makeTextTranslator()
	local vcc = inconcert.authUser.virtualContactCenter
	local user_name = utils.GetCurrentLoggedUser()
	local rid = web.vars.HTTP_X_ICC_RID or "n/a"

	local request = json.decode(web.input["post_data"])

	LogInfo("Create Whatsapp request from agent: '%s', id: '%s', campaign: '%s', account: '%s'", 
		user_name, request.interactionID, request.campaign, request.account)

	if not HasPermission( "CREATE_OUTGOING", request.campaign, "interaction" ) then
		web.status = 403
		return ReplyJSON( web, { result = "FORBIDDEN", reason = _T("You not have permission for create outgoing Whatsapp conversation.") })
	end

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)

	--this is done in order to avoid api manglers creating interactions from another vcc from the one they're logged in
	local campaignIdAtVCC = helpers.AddVCC(request.campaign, vcc)
	local res = irLink:WhatsappCreateAndSendInteractionMessage({
		interactionID = request.interactionID,
		parentInteractionId = request.parentInteractionId,
		campaignAtVCC = campaignIdAtVCC,
		account = request.account,
		attention_level_id	= request.attention_level_id,
		address = request.address,
		contactID = request.contactID,
		message = request.message,
		agentAtVCC = user_name
	})

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(res["result"])
	web.status = http_code or 500 --unknown result code

	if res.result ~= "OK" then
		LogError("Error creating Whatsapp interaction id: '%s' -> '%s'", request.interactionID, res.reason)
		res.reason = _T(res.reason)
		return ReplyJSON(web, res)
	end

	LogInfo("Success creating Whatsapp interaction '%s'", request.interactionID)
	return ReplyJSON(web)
end

local function WhatsappForward(web)
	local _T = m_htmlRender.extension.makeTextTranslator()
	local vcc = inconcert.authUser.virtualContactCenter
	local user_name = utils.GetCurrentLoggedUser()
	local request = json.decode(web.input["post_data"])

	LogInfo("Forwarding messages from agent: '%s', to address: '%s', on interaction id: '%s', campaign: '%s', account: '%s'",
		user_name, request.address, request.interactionID, request.campaignId, request.account)

	if not hasLicensesForChannel( "WHATSAPP" ) then
		local reason = LogFatal("Unable to forward. Not has licensed for the Whatsapp channel.")
		web.status = 500
		return ReplyJSON(web, {
			agent_err = true,
			result = "NOT_LICENSED",
			reason = _T(reason)
		})
	end

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)

	--this is done in order to avoid api manglers creating interactions from another vcc from the one they're logged in
	local campaignIdAtVCC = helpers.AddVCC(request.campaignId, vcc)
	local res = irLink:WhatsappForward({
		campaignAtVCC = campaignIdAtVCC,
		account = request.account,
		interactionID = request.interactionID,
		agentAtVCC = user_name,
		address = request.address,
		messages = request.messages,
	})

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(res["result"])
	web.status = http_code or 500 --unknown result code

	if res.result ~= "OK" then
		LogError("Error forwarding whatsapp messages to address %s on interaction id: '%s' -> '%s'", 
			request.address, request.interactionID, res.reason)
		res.reason = _T(res.reason)
		return ReplyJSON(web, res)
	end

	LogInfo("Success forwarding whatsapp messages to address %s on interaction '%s'", 
		request.address, request.interactionID)
	return ReplyJSON(web, res)
end

--- [Rcs] Will send a message
--
-- @tparam table  web
-- @tparam table  web.input
-- @tparam string web.input.post_data  - Data posted on the API
--
-- if we decode post_data json string:
-- table  request
-- string request.interactionID  - The interaction id.
-- string request.campaign		 - Campaign id (i.e "campaign@vcc")
-- string request.account        - The account id (i.e "TWITTER_campaign@vcc_username")
-- object request.message        - The message
--
function SendRcs(web)
	local _T = m_htmlRender.extension.makeTextTranslator()
	local vcc = inconcert.authUser.virtualContactCenter
	local user_name = utils.GetCurrentLoggedUser()
	local request = json.decode(web.input["post_data"])

	LogInfo("Sending Rcs from agent: '%s', on interaction id: '%s', campaign: '%s', account: '%s', message: '%s'",
		user_name, request.interactionID, request.campaign, request.account, json.encode(request.message))

	if not hasLicensesForChannel( "RCS" ) then
		local reason = LogFatal("Unable to send message. Not has licensed for the Rcs channel.")
		web.status = 500
		return ReplyJSON(web, {
			agent_err = true,
			result = "NOT_LICENSED",
			reason = _T(reason)
		})
	end

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	
	local res = irLink:RcsSendInteractionMessage({
		interactionID = request.interactionID,
		message = request.message,
		agentAtVCC = user_name
	})

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(res["result"])
	web.status = http_code or 500 --unknown result code

	if res.result ~= "OK" then
		LogError("Error sending Rcs on interaction id: '%s' -> '%s'", request.interactionID, res.reason)
		res.reason = _T(res.reason)
		return ReplyJSON(web, res)
	end

	LogInfo("Success sending Rcs on interaction '%s'", request.interactionID)
	return ReplyJSON(web)
end

--- [Rcs] Will create a new Rcs interaction
--
-- @tparam table  web
-- @tparam table  web.input
-- @tparam string web.input.post_data  - Data posted on the API
--
-- if we decode post_data json string:
-- table  request
-- string request.interactionID  - The interaction id.
-- string request.campaign       - Campaign id (i.e "campaign@vcc")
-- string request.account        - The account id (i.e "TWITTER_campaign@vcc_username")
-- object request.message        - The message
--
function RcsCreateAndSend(web)
	local _T = m_htmlRender.extension.makeTextTranslator()
	local vcc = inconcert.authUser.virtualContactCenter
	local user_name = utils.GetCurrentLoggedUser()
	local rid = web.vars.HTTP_X_ICC_RID or "n/a"

	local request = json.decode(web.input["post_data"])

	LogInfo("Create Rcs request from agent: '%s', id: '%s', campaign: '%s', account: '%s'", 
		user_name, request.interactionID, request.campaign, request.account)

	if not HasPermission( "CREATE_OUTGOING", request.campaign, "interaction" ) then
		web.status = 403
		return ReplyJSON( web, { result = "FORBIDDEN", reason = _T("You not have permission for create outgoing Rcs conversation.") })
	end

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)

	--this is done in order to avoid api manglers creating interactions from another vcc from the one they're logged in
	local campaignIdAtVCC = helpers.AddVCC(request.campaign, vcc)
	local res = irLink:RcsCreateAndSendInteractionMessage({
		interactionID = request.interactionID,
		parentInteractionId = request.parentInteractionId,
		campaignAtVCC = campaignIdAtVCC,
		account = request.account,
		attention_level_id	= request.attention_level_id,
		address = request.address,
		contactID = request.contactID,
		message = request.message,
		agentAtVCC = user_name
	})

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(res["result"])
	web.status = http_code or 500 --unknown result code

	if res.result ~= "OK" then
		LogError("Error creating Rcs interaction id: '%s' -> '%s'", request.interactionID, res.reason)
		res.reason = _T(res.reason)
		return ReplyJSON(web, res)
	end

	LogInfo("Success creating Rcs interaction '%s'", request.interactionID)
	return ReplyJSON(web)
end

local function RcsForward(web)
	local _T = m_htmlRender.extension.makeTextTranslator()
	local vcc = inconcert.authUser.virtualContactCenter
	local user_name = utils.GetCurrentLoggedUser()
	local request = json.decode(web.input["post_data"])

	LogInfo("Forwarding messages from agent: '%s', to address: '%s', on interaction id: '%s', campaign: '%s', account: '%s'",
		user_name, request.address, request.interactionID, request.campaignId, request.account)

	if not hasLicensesForChannel( "RCS" ) then
		local reason = LogFatal("Unable to forward. Not has licensed for the Rcs channel.")
		web.status = 500
		return ReplyJSON(web, {
			agent_err = true,
			result = "NOT_LICENSED",
			reason = _T(reason)
		})
	end

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)

	--this is done in order to avoid api manglers creating interactions from another vcc from the one they're logged in
	local campaignIdAtVCC = helpers.AddVCC(request.campaignId, vcc)
	local res = irLink:RcsForward({
		campaignAtVCC = campaignIdAtVCC,
		account = request.account,
		interactionID = request.interactionID,
		agentAtVCC = user_name,
		address = request.address,
		messages = request.messages,
	})

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(res["result"])
	web.status = http_code or 500 --unknown result code

	if res.result ~= "OK" then
		LogError("Error forwarding Rcs messages to address %s on interaction id: '%s' -> '%s'", 
			request.address, request.interactionID, res.reason)
		res.reason = _T(res.reason)
		return ReplyJSON(web, res)
	end

	LogInfo("Success forwarding Rcs messages to address %s on interaction '%s'", 
		request.address, request.interactionID)
	return ReplyJSON(web, res)
end

function GetESqueryForSearchInteractionsHelper (request, user_attention_levels)
	local _T = m_htmlRender.extension.makeTextTranslator()
	local user_name, vcc = inconcert.authUser.userName, inconcert.authUser.virtualContactCenter

	-- validate the user is filtering for valid campaigns, in case empty add all the campaigns available
	local userCampaigns 
	--apiNotValidate: No validar user_name de api
	if not request.apiNotValidateUserCampaigns then
		userCampaigns = GetModel("Campaigns").GetUserCampaigns(user_name, vcc)
		if not next(userCampaigns) then
			LogInfo("[INTERACTIONS:GetESqueryForSearchInteractionsHelper] User %s does not belong to a campaign, no interactions are returned", user_name)
			return false, 403, { reason = _T("User does not belong to any campaign") }
		end
	end

	local request_query = request.query
	if not request_query then
		return false, 400, { reason = _T("Missing query") }
	end

	LogInfo("[INTERACTIONS:GetESqueryForSearchInteractionsHelper] Request = %s.", json.encode(request))

	local sortByPriority = false
	local includeAllLicensedTypes = true
	local includeCampaign = false
	local canUseAttentionLevelRule = true

	-- Filter skeleton. We add more specialized filters as needed (plus, a query for text if needed too)
	local query = {
		query = {
			bool = {
				filter = {},
				must = {},
				must_not = {},
				should = {}
			}
		}
	}

	--Si tenemos una rule prefijada por estado
	if request_query.fixedStateRule then
		local fixedStateRule = to_table(request_query.fixedStateRule)
		table.insert(query.query.bool.must, { terms = { state = fixedStateRule } })
		-- Si el valor del estado prefinado es FINISHED, la query viene del panel Archive, y el orden a aplicar es el de fecha de creacion de mas reciente a mas antigua
		if request_query.fixedStateRule == "FINISHED" then
			sortByPriority = true			
		elseif request_query.fixedStateRule == "ABANDONED" and not request.fromAPI then
			-- En las consultas desde el panel de abandonadas no se deben devolver las que no tengan posibilidad de rellamada
			table.insert(query.query.bool.must, { term = { callback_is_allowed = true } })
		end
	end

	--Si tenemos una rule prefijada por agent
	if request_query.fixedAgentRule then
		local fixedAgentRule = to_table(request_query.fixedAgentRule)
		table.insert(query.query.bool.must, { terms = { agent = fixedAgentRule } })
	end

	local afirmativeClause
	local negativeClause = query.query.bool.must_not
	--Common condition for all clauses
	if request_query.condition == "AND" then
		afirmativeClause = query.query.bool.must
	else --OR
		afirmativeClause = query.query.bool.should
		query.query.bool.minimum_should_match = 1
	end

	local function AddRule (rule)
		local ruleId = rule.id
		-- va contra los attention levels del usuario + las interacciones que no tienen als (als no posteados)
		if ruleId == "attention_level" and not canUseAttentionLevelRule then
			return
		end

		-- Map some of the names used in the request to what's defined in elasticsearch
		-- {key = received, value = whats's on elastic}
		local mappings = {
			client = "client_id", start_date = "start_timestamp", date_state_to = "state_timestamp", date_state_from = "state_timestamp", utc_date_last_event_from = "last_event_timestamp", utc_date_last_event_to = "last_event_timestamp"
		}
		moses(mappings):each(function(k,v)
			if ruleId == k then
				ruleId = v
			end
		end)

		local isDateRule = {
			--This comes from agent app
			start_timestamp = true,
			--These come from API
			state_timestamp = true,
			last_event_timestamp = true		
		}

		if ruleId == "contact" then
			local contact = Fullcontact.GetByExternalId(rule.value, vcc)
			if contact and contact.Id and contact.Id ~= "" then
				LogDebug("[INTERACTIONS:GetESqueryForSearchInteractionsHelper] - Contact '%s' found by externalId '%s'", contact.Id, rule.value)
				rule.value = contact.Id
			end
		end		

		local ruleOperator = rule.operator
		local ruleValue = (isDateRule[ruleId] or ruleOperator == "contains" or ruleOperator == "not_contains" or ruleOperator == "similar" or ruleOperator == "between") and rule.value or to_table(rule.value)
		local ruleValueTo = rule.value_to

		if ruleId == "type" then
			includeAllLicensedTypes = false
		end

		if ruleId == "campaign" then
			includeCampaign = true
		end

		-- Normalize campaign and agent (make sure @vcc is the correct one)
		if ruleOperator ~= "contains" and ruleOperator ~= "not_contains" and (ruleId == "campaign" or ruleId == "agent") then
			ruleValue = moses.map(ruleValue, function(_,member_id)
				return (helpers.AddVCC( helpers.SeparateVCC(member_id), vcc))
			end)	
		end	

		-- We lowercase these fields. Use them with fields in Elastic that use "reporting_content" analyzer
		if ruleId == "disposition_code" or ruleId == "contact_address" or ruleId == "labels" then
			if moses.isTable(ruleValue) then
				ruleValue = moses.map(ruleValue, function(k,v) return utf8.lower(v) end)
			else
				ruleValue = utf8.lower(ruleValue)
			end
		end

		-- We split these fields by white space
		local hasDispotitionWithSpace
		if (ruleId == "contact_name" or (ruleOperator.contains and ruleId == "disposition_code")) and ruleOperator ~= "similar" then
			local ruleValueTable = {}
			if moses.isTable(ruleValue) then
				for _,value in pairs(ruleValue) do
					local ruleValueSplitted = SplitString(value," ")
					for _,valueSplitted in pairs(ruleValueSplitted) do
						table.insert(ruleValueTable, valueSplitted)
					end
				end
			else
				local ruleValueSplitted = SplitString(ruleValue," ")
				for _,valueSplitted in pairs(ruleValueSplitted) do
					table.insert(ruleValueTable, valueSplitted)
				end
			end
			ruleValue = ruleValueTable
		elseif ruleId == "disposition_code" then
			local ruleValueTable = {}
			local isContains = ruleOperator == "contains" or ruleOperator == "not_contains" or false
			
			if not moses.isTable(ruleValue) then
				ruleValue = {ruleValue}
			end
			for _, value in ipairs(ruleValue) do
				local ruleValueSplitted = SplitString(value," ")
				
				if isContains then
					for _,valueSplitted in pairs(ruleValueSplitted) do
						table.insert(ruleValueTable, valueSplitted)
					end
				end

				if ruleValueSplitted and #ruleValueSplitted > 1 then
					--[[ Si se quiere filtrar uno o mas codigos y al menos 1 tiene espacio, se agrega el 
							subcampo'.lowercase' para filtrar teniendo encuenta espacios.
						Se agrega esta validacion porque para usar filtrar codigos con espacios se debe reindexar y muchos no pueden.
						De esta forma queda obligatorio reindexar si se desea filtrar por espacio.	
					]]--

					hasDispotitionWithSpace = true
					if not isContains then
						break
					end
				end
			end
			if isContains then
				ruleValue = ruleValueTable
			end
		end
		
		if (ruleId == "contact_address") and ruleOperator ~= "similar" then
			local ruleValueTable = {}
			if moses.isTable(ruleValue) then
				for _,value in pairs(ruleValue) do
					scapeValue = string.gsub(value, "@", "\\@")
					table.insert(ruleValueTable, scapeValue)
				end
			else
				scapeValue = string.gsub(ruleValue, "@", "\\@")
				table.insert(ruleValueTable, scapeValue)
			end
			ruleValue = ruleValueTable
		end
		
		if ruleOperator == "equal" then
			if isDateRule[ruleId] then
				local ormDate = LuaOrm.Date(ruleValue)
				local addDay = request_query.isFullDate and 0 or 1
				local addSecond = request_query.isFullDate and 1 or 0
				ruleValue = ormDate:toString("%Y-%m-%d %H:%M:%S")
				ruleValueTo = ormDate:add(0, 0, addDay, 0, 0, addSecond):toString("%Y-%m-%d %H:%M:%S")		
				-- Se transforma las fechas a UTC para el timezone del usuario. 
				if inconcert and inconcert.authUser and inconcert.authUser.timezone then
					ruleValue = utils.ConvertDateToUTCDate( ruleValue, nil, "%Y-%m-%d %H:%M:%S", true, inconcert.authUser.timezone or nil )
					ruleValueTo = utils.ConvertDateToUTCDate( ruleValueTo, nil, "%Y-%m-%d %H:%M:%S", true, inconcert.authUser.timezone or nil )
				end
				table.insert(afirmativeClause, { range = { [ruleId] = { gte = ruleValue, lt = ruleValueTo, format = "yyyy-MM-dd HH:mm:ss" } } })
			elseif ruleId == "contact_address" then
				for _,value in pairs(ruleValue) do
					table.insert(afirmativeClause, { query_string = {
						["fields"] = {ruleId},
						["query"] = value,
						["default_operator"] = "AND"
					}})
				end
			else
				if ruleId == "disposition_code" then
					ruleId = ruleId .. (hasDispotitionWithSpace and ".lowercase" or "")
				elseif ruleId == "disposition_tree_code" then
					ruleId = ruleId .. ".keyword"
				end
				table.insert(afirmativeClause, { terms = { [ruleId] = ruleValue } })
			end
		elseif ruleOperator == "not_equal" then
			if isDateRule[ruleId] then
				local ormDate = LuaOrm.Date(ruleValue)
				local addDay = request_query.isFullDate and 0 or 1
				local addSecond = request_query.isFullDate and 1 or 0
				ruleValue = ormDate:toString("%Y-%m-%d %H:%M:%S")
				ruleValueTo = ormDate:add(0, 0, addDay, 0, 0, addSecond):toString("%Y-%m-%d %H:%M:%S")	
				-- Se transforma las fechas a UTC para el timezone del usuario.
				if inconcert and inconcert.authUser and inconcert.authUser.timezone then
					ruleValue = utils.ConvertDateToUTCDate( ruleValue, nil, "%Y-%m-%d %H:%M:%S", true, inconcert.authUser.timezone or nil )
					ruleValueTo = utils.ConvertDateToUTCDate( ruleValueTo, nil, "%Y-%m-%d %H:%M:%S", true, inconcert.authUser.timezone or nil )
				end
				table.insert(negativeClause, { range = { [ruleId] = { gte = ruleValue, lt = ruleValueTo, format = "yyyy-MM-dd HH:mm:ss" } } })
			elseif ruleId == "contact_address" then
				for _,value in pairs(ruleValue) do
					table.insert(negativeClause, { query_string = {
						["fields"] = {ruleId},
						["query"] = value,
						["default_operator"] = "AND"
					}})
				end
			else
				if ruleId == "disposition_code" then
					ruleId = ruleId .. (hasDispotitionWithSpace and ".lowercase" or "")
				elseif ruleId == "disposition_tree_code" then
					ruleId = ruleId .. ".keyword"
				end
				table.insert(negativeClause, { terms = { [ruleId] = ruleValue } })
			end			
		elseif ruleOperator == "contains" then
			if ruleId == "content" then
				table.insert(afirmativeClause, { match = { [ruleId] = { query = ruleValue } }})
			elseif ruleId == "contact_address" or hasDispotitionWithSpace then
				for _,value in pairs(ruleValue) do
					table.insert(afirmativeClause, { query_string = {
						["fields"] = {ruleId},
						["query"] = value,
						["default_operator"] = "AND"
					}})
				end
			elseif ruleId == "disposition_code" or ruleId == "contact_name" then
				table.insert(afirmativeClause, { terms = { [ruleId] = ruleValue } })
			else
				table.insert(afirmativeClause, { wildcard = { [ruleId] = "*"..ruleValue.."*"} })
			end
		elseif ruleOperator == "not_contains" then
			if ruleId == "content" then
				table.insert(negativeClause, { match = { [ruleId] = { query = ruleValue } }})
			elseif ruleId == "disposition_code" or ruleId == "contact_name" then
				table.insert(negativeClause, { terms = { [ruleId] = ruleValue } })
			elseif ruleId == "contact_address" or hasDispotitionWithSpace then
				for _,value in pairs(ruleValue) do
					table.insert(negativeClause, { query_string = {
						["fields"] = {ruleId},
						["query"] = value,
						["default_operator"] = "AND"
					}})
				end
			else
				table.insert(negativeClause, { wildcard = { [ruleId] = "*"..ruleValue.."*"} })
			end
		elseif ruleOperator == "less" then
			if isDateRule[ruleId] then
				local ormDate = LuaOrm.Date(ruleValue)
				ruleValue = ormDate:toString("%Y-%m-%d %H:%M:%S")
				-- Se transforma las fechas a UTC para el timezone del usuario. 
				if inconcert and inconcert.authUser and inconcert.authUser.timezone then
					ruleValue = utils.ConvertDateToUTCDate( ruleValue, nil, "%Y-%m-%d %H:%M:%S", true, inconcert.authUser.timezone or nil )
				end
				table.insert(afirmativeClause, { range = { [ruleId] = { lt = ruleValue, format = "yyyy-MM-dd HH:mm:ss" } } })
			else
				table.insert(afirmativeClause, { range = { [ruleId] = { lt = ruleValue } } })
			end			
		elseif ruleOperator == "less_or_equal" then
			if isDateRule[ruleId] then
				local ormDate = LuaOrm.Date(ruleValue)
				ruleValue = ormDate:toString("%Y-%m-%d %H:%M:%S")	
				-- Se transforma las fechas a UTC para el timezone del usuario. 
				if inconcert and inconcert.authUser and inconcert.authUser.timezone then
					ruleValue = utils.ConvertDateToUTCDate( ruleValue, nil, "%Y-%m-%d %H:%M:%S", true, inconcert.authUser.timezone or nil )
				end
				table.insert(afirmativeClause, { range = { [ruleId] = { lte = ruleValue, format = "yyyy-MM-dd HH:mm:ss" } } })
			else
				table.insert(afirmativeClause, { range = { [ruleId] = { lte = ruleValue } } })
			end
		elseif ruleOperator == "greater" then
			if isDateRule[ruleId] then
				local ormDate = LuaOrm.Date(ruleValue)
				ruleValue = ormDate:toString("%Y-%m-%d %H:%M:%S")
				-- Se transforma las fechas a UTC para el timezone del usuario. 
				if inconcert and inconcert.authUser and inconcert.authUser.timezone then
					ruleValue = utils.ConvertDateToUTCDate( ruleValue, nil, "%Y-%m-%d %H:%M:%S", true, inconcert.authUser.timezone or nil )
				end
				table.insert(afirmativeClause, { range = { [ruleId] = { gt = ruleValue, format = "yyyy-MM-dd HH:mm:ss" } } })
			else
				table.insert(afirmativeClause, { range = { [ruleId] = { gt = ruleValue } } })
			end
		elseif ruleOperator == "greater_or_equal" then
			if isDateRule[ruleId] then
				local ormDate = LuaOrm.Date(ruleValue)
				ruleValue = ormDate:toString("%Y-%m-%d %H:%M:%S")
				-- Se transforma las fechas a UTC para el timezone del usuario.
				if inconcert and inconcert.authUser and inconcert.authUser.timezone then
					ruleValue = utils.ConvertDateToUTCDate( ruleValue, nil, "%Y-%m-%d %H:%M:%S", true, inconcert.authUser.timezone or nil )
				end
				table.insert(afirmativeClause, { range = { [ruleId] = { gte = ruleValue, format = "yyyy-MM-dd HH:mm:ss" } } })
			else
				table.insert(afirmativeClause, { range = { [ruleId] = { gte = ruleValue } } })
			end			
		elseif ruleOperator == "between" then
			if isDateRule[ruleId] then
				local ormDate = LuaOrm.Date(ruleValue)
				ruleValue = ormDate:toString("%Y-%m-%d %H:%M:%S")
				ormDate = LuaOrm.Date(ruleValueTo)
				ruleValueTo = ormDate:toString("%Y-%m-%d %H:%M:%S")	
				-- Se transforma las fechas a UTC para el timezone del usuario. 
				if inconcert and inconcert.authUser and inconcert.authUser.timezone then
					ruleValue = utils.ConvertDateToUTCDate( ruleValue, nil, "%Y-%m-%d %H:%M:%S", true, inconcert.authUser.timezone or nil )
					ruleValueTo = utils.ConvertDateToUTCDate( ruleValueTo, nil, "%Y-%m-%d %H:%M:%S", true, inconcert.authUser.timezone or nil )
				end			
				table.insert(afirmativeClause, { range = { [ruleId] = { gte = ruleValue, lte = ruleValueTo, format = "yyyy-MM-dd HH:mm:ss" } } })
			else
				table.insert(afirmativeClause, { range = { [ruleId] = { gte = ruleValue, lte = ruleValueTo } } })
			end
		elseif ruleOperator == "in" then
			if ruleId == "disposition_code" then
				ruleId = ruleId .. (hasDispotitionWithSpace and ".lowercase" or "")
			end
			table.insert(afirmativeClause, { terms = { [ruleId] = to_table(ruleValue) } })
		elseif ruleOperator == "similar" then
			-- si es convertible a numerico, uso wildcards para buscar numeros
			if tonumber(ruleValue) then
				afirmativeClause.wildcard = afirmativeClause.wildcard or {}
				afirmativeClause.wildcard[ruleId] = "*"..ruleValue.."*"
			else
				-- Multi-word match
				-- https://www.elastic.co/guide/en/elasticsearch/guide/current/match-multi-word.html
				table.insert(afirmativeClause, {
					match = {
						[ruleId] = {
							query = ruleValue,
							operator = "and",
							fuzziness = 1,
							prefix_length = 1 -- para que no se ponga tan creativo con las substituciones
											  -- y no haga reemplazos al comienzo de la palabra
						}
					}
				})
			end
		else
			LogWarning("[INTERACTIONS:GetESqueryForSearchInteractionsHelper] Unknown rule operator '%s'.", ruleOperator)
			return
		end

		if ruleId == "state" and ruleValue == "FINISHED" then
			sortByPriority = true
		end
	end

	local function AddBusinessDataRule (rule)
		--businessdata: Must be an array of objects with these attributes: 
		--	name: Business data field name
		--	type: Business data field type. allowed types: numeric (both integers and decimals), text (strings), date (allowed formats: "yyyy-MM-dd HH:mm:ss, yyyy-MM-dd, HH:mm:ss, HH:mm"), boolean (true, false)
		--	value: value to compare with Business data field to 
		--	operator: comparision operator to compare value and Business data field. allowed operators: =, <>, >, <, >=, <=, between
		--
		--	For 'between' operator Business data field must include an additional atribute value_to

		local businessdataTypeMapping = {
			numeric = "number",
			text = "string",
			date = "string",
			boolean = "boolean"
		}

		local booleanOperators = {["equal"] = true, ["not_equal"] = true}
		local textOperators = {["equal"] = true, ["not_equal"] = true, ["contains"] = true, ["not_contains"] = true, ["in"] = true}
		local numericAndDateOperators = {["less"] = true, ["greater"] = true, ["less_or_equal"] = true, ["greater_or_equal"] = true, ["between"] = true, ["equal"] = true, ["not_equal"] = true}

		local businessdataTypeOperatorMapping = {
			numeric = numericAndDateOperators,
			date = numericAndDateOperators,
			text = textOperators,
			boolean = booleanOperators
		}

		local businessdataOperatorMapping = {
			["less"] = "lt",
			["greater"] = "gt",
			["less_or_equal"] = "lte",
			["greater_or_equal"] = "gte"
		}

		local function AddBusinessDataFilter (filter)
			local filterFieldName = filter.name and type(filter.name) == "string" and filter.name or nil
			if not filterFieldName then
				return false, "Missing filter name or it is not a string"
			end

			if not businessdataTypeMapping[filter.type] then
				return false, "Unknown business data filter type"
			end

			local filterType = "business_data_" .. filter.type

			local filterValue = filter.value
			if filter.value == nil then
				return false, "Missing business data filter value"
			end

			local filterOperator = filter.operator and type(filter.operator) == "string" and filter.operator or nil
			if not filterOperator then
				return false, "Missing business data filter operator or it is not a string"
			end

			local validOperators = businessdataTypeOperatorMapping[filter.type]
			if not validOperators[filterOperator] then
				return false, "Business data filter operator not allowed for business data filter type"
			end

			local filterValueTo = filter.value_to
			if filterOperator == "between" then
				if not filterValueTo then
					return false, "Business data filter operator between requires value_to"
				end
			end 

			local condition = {
				nested = {
					path = "business_data_" .. filter.type,
					query = {
						bool = {
							must = {},
							must_not = {}
						}
					}
				}
			}
			table.insert(condition.nested.query.bool.must, { match = { [filterType..".FieldName"] = filterFieldName } })

			if filterOperator == "not_equal" then
				table.insert(condition.nested.query.bool.must_not, { match = { [filterType..".Value"] = filterValue } })
			elseif filterOperator == "not_contains" then
				table.insert(condition.nested.query.bool.must_not, { wildcard = { [filterType..".Value"] = "*" .. filterValue .. "*" } })
			else 
				condition.nested.query.bool.must_not = nil
				if filterOperator == "contains" then
					table.insert(condition.nested.query.bool.must, { wildcard = { [filterType..".Value"] = "*" .. filterValue .. "*" } })
				elseif filterOperator == "equal" then
					table.insert(condition.nested.query.bool.must, { match = { [filterType..".Value"] = filterValue } })
				elseif filterOperator == "between" then
					table.insert(condition.nested.query.bool.must, { range = { [filterType..".Value"] = { ["gte"] = filterValue, ["lte"] = filterValueTo } } })
				else
					table.insert(condition.nested.query.bool.must, { range = { [filterType..".Value"] = { [businessdataOperatorMapping[filterOperator]] = filterValue } } })
				end
			end

			return true, condition
		end

		if rule.value and #rule.value then
			for _, businessdataFilter in pairs(rule.value) do
				local addbusinessdataFilterResult, businessdataFilterCondition = AddBusinessDataFilter(businessdataFilter)
				if addbusinessdataFilterResult then
					table.insert(afirmativeClause, businessdataFilterCondition)
				else
					LogWarning("[INTERACTIONS:GetESqueryForSearchInteractionsHelper] Invalid business data filter '%s': %s", json.encode(businessdataFilter), businessdataFilterCondition)
				end
			end
		end
	end

	local function MustFilterWithoutAttentionLevels ()
		if not mustValidateSearchPermission then
			return false
		end
		local mustFilterWithoutAttentionLevels = HasPermission("FILTER_WITHOUT_ATT_LEVEL", nil, appId)
		LogInfo("[INTERACTIONS:GetESqueryForSearchInteractionsHelper] Must Filter Without Attention Levels = %s.", mustFilterWithoutAttentionLevels)
		return mustFilterWithoutAttentionLevels
	end

	local mustFilterWithoutAttentionLevels = MustFilterWithoutAttentionLevels()
	if not mustFilterWithoutAttentionLevels then
		if user_attention_levels then
			-- va contra los attention levels del usuario + las interacciones que no tienen als (als no posteados)
			local condition = {
					["bool"] = {
						["should"] = {
							{
								["bool"] = {
									must_not = {
										exists = { field = "attention_level" } 
									}
								}
							},
							{ terms = { ["attention_level"] = user_attention_levels } },
						}
					}
				}	
			table.insert(query.query.bool.filter, condition)
			canUseAttentionLevelRule = false
		end
	end

	--Process rules
	if request_query.rules and #request_query.rules > 0 then
		for _, rule in pairs(request_query.rules) do
			if rule.id == "businessdata" then
				AddBusinessDataRule(rule)
			else
				AddRule(rule)
			end
		end
	end

	-- Searching the archive, we don't sort by priority
	if sortByPriority then
		query.sort = { { start_timestamp = { order = "desc" } } }
	else
		-- If the client specifies a sorting scheme, use that.
		-- The sorting must be specified according to what ElasticSearch needs, so go read the docs.
		if request.sort then
			query.sort = {}
			for _, sortType in pairs(request.sort) do
				for name, sType in pairs(sortType) do
					local sortItem = {}
					if name == "contact_name" then
						sortItem["contact_name.keyword"] = sType
					else
						sortItem[name] = sType
					end
					table.insert(query.sort,sortItem)
				end
			end
		else
			query.sort = { { start_timestamp = { order = "asc" }, priority = { order = "desc" } } }
		end
	end
	-- Determine if pagination is needed
	-- Con request.rows pide el archivo, el else atiende al resto. TODO: unificar
	if request.rows then
		query.size = request.rows or 20
		query.from = (tonumber(request.last) or 0)
	else
		local page = tonumber(request.page) or 0
		if page == 0 then
			-- they want all interactions
			local MAX_QUEUED_INTERACTIONS = 500
			query.size = MAX_QUEUED_INTERACTIONS
			query.from = 0
		else
			-- they want a specific "page"
			local rows_per_page = tonumber(request.rows_per_page) or 50
			local start =  rows_per_page * (page - 1)
			query.size = rows_per_page
			query.from = start
		end
	end

	-- if no specific type was requested, search for all licensed types
	if includeAllLicensedTypes then
		local licensedTypes = moses.filter({ "CALL", "CHAT", "FACEBOOK", "INSTAGRAM", "YOUTUBE", "APPSTORE", "GOOGLEPLAY", "RCS", "IM", "MAIL", "SMS", "WHATSAPP", "TWITTER", "WEBCONTACT", "VOICEMAIL" }, function(_, channel)
			return hasLicensesForChannel(channel)
		end)
		if licensedTypes then
			table.insert(query.query.bool.must, { terms = { type = to_table(licensedTypes) } })
		end
	end

	local appId = request.appId
	local mustValidateSearchPermission = request.mustValidateSearchPermission

	local function MustFilterByCrossCampaigns ()
		if not mustValidateSearchPermission then
			return false
		end
		local mustFilterByCrossCampaigns = HasPermission("FILTER_BY_CROSS_CAMPAIGNS", nil, appId)
		LogInfo("[INTERACTIONS:GetESqueryForSearchInteractionsHelper] Must Filter by Cross Campaigns %s.", mustFilterByCrossCampaigns)
		return mustFilterByCrossCampaigns
	end

	local mustFilterByCrossCampaigns = MustFilterByCrossCampaigns()
	if not mustFilterByCrossCampaigns then
		-- if no specific campaigns were requestd, default to the user campaigns
		if not includeCampaign and not request.apiNotValidateUserCampaigns then
			table.insert(query.query.bool.must, { terms = { campaign = moses.keys(userCampaigns) } })
		end
	end
	
	--Add pit
	local withPit = false
	if request.pit then
		query["pit"] = request.pit
		withPit = true
	end

	-- This fields can't be empty, so remove them if no query was added
	if #query.query.bool.must == 0 then
		query.query.bool.must = nil
	end
	if #query.query.bool.must_not == 0 then
		query.query.bool.must_not = nil
	end
	if #query.query.bool.should == 0 then
		query.query.bool.should = nil
		query.query.bool.minimum_should_match = nil
	end

	return true, query, nil, withPit
end

--[[
	request:
		El post data con filtros, etc.
	user_attention_levels:
		los niveles de atencion del usuario.
	elasticSearchConnectionConfig:
		conexiones a ES.
	excludeSrtipAttributes     (por defecto no excluye):
		Lista de atributos que NO se necesitan eliminar de la interaccion (que la rocka "interactionmanip" no los elimine).
	returnOnlyElastcSearchInfo (por defecto es false):
		Flag para que la informacion de las interacciones que se retornaran se obtengan desde ES y no vaya ni a Cassandra ni a Redis.
]]--
function SearchInteractionsHelper (request, user_attention_levels, elasticSearchConnectionConfig, excludeSrtipAttributes, returnOnlyElastcSearchInfo)
	returnOnlyElastcSearchInfo = returnOnlyElastcSearchInfo or false
	
	local querySuccess, query_ES, error_message, withPit = GetESqueryForSearchInteractionsHelper(request, user_attention_levels)
	if not querySuccess then
		return query_ES, error_message
	end
	local _T = m_htmlRender.extension.makeTextTranslator()

	-- LogDebug("[INTERACTIONS.SearchInteractionsHelper] 1 ------------> query---------> %s", json.encode(query_ES))
	-- LogDebug("[INTERACTIONS.SearchInteractionsHelper] 2 ------------> request_query-> %s", json.encode(request_query))

	local modelInteractions = GetModel("Interactions")
	local ok, result = modelInteractions.search_elastic(inconcert.authUser.virtualContactCenter, query_ES, withPit, request.fromAPI, elasticSearchConnectionConfig)
	if not ok then
		local result_s = type(result) == "string" and _T(result) or result
		return 500, { result = "REQUEST_FAILED", reason = result_s }
	end
	if result.timed_out then
		LogWarning("[SearchInteractionsHelper] Query against elasticsearch timed out. %s", json.encode(query_ES))
		return 502, { result = "REQUEST_TIMEDOUT", reason = _T("Query timed out") }
	end
	LogInfo("[SearchInteractionsHelper] -> Query took %s ms and returned %s hits out of %s. Return Info ONLY ElastcSearch: '%s'", result.took, #result.hits.hits, result.hits.total.value, returnOnlyElastcSearchInfo)
	--LogDebug("%s", json.encode(result, true))
	local matched_interactions, highlights = {}, {}
	local total_interactions = result.hits.total.value

	if returnOnlyElastcSearchInfo then
		-- (ES): La informacion de las interacciones a retornar es lo que ha retornado ES.
		--
		moses.each(result.hits.hits, function(_, hit)
			local interaction = hit._source
			--LogDebug("-ES-----> id %s - got %s", hit._id, interaction)
			if not utils.TableIsEmpty(interaction) then
				table.insert(matched_interactions, interaction)
			else
				total_interactions = total_interactions - 1
				--LogDebug("------> empty interaction id %s -  %s", hit._id, json.encode(interaction))
			end
			if request.highlight then
				highlights[hit._id] = hit.highlight.content
			end
		end)
	else
		-- (REDIS/CASSANDRA): La informacion de las interacciones a retornar es lo retornado por REDIS o CASSANDRA.
		--
		local ks_interactions = utils.GetDatasource():CassandraConnection("Interactions")
		-- retrieve the full interaction
		moses.each(result.hits.hits, function(_, hit)
			local origin = "redis"
			local interaction = modelInteractions.GetInteraction(hit._id, ks_interactions)
			if not interaction then
				interaction = modelInteractions.GetFromArchive(hit._id, ks_interactions)
				origin = "cassandra"
			end
			--LogDebug("--%s----> id %s - got %s", origin, hit._id, interaction)
			
			if not utils.TableIsEmpty(interaction)  then
				table.insert(matched_interactions, interaction)
			else
				total_interactions = total_interactions - 1
				--LogDebug("------> empty interaction id %s -  %s", hit._id, json.encode(interaction))
			end
			if request.highlight then
				highlights[hit._id] = hit.highlight.content
			end
		end)
		ks_interactions:Close()
		interactionmanip.NormalizeInteractions(matched_interactions)
		if not request.inflate then
			interactionmanip.StripUnwantedMembers(matched_interactions, excludeSrtipAttributes)
		end	
	end

	--LogDebug("1 --> matched_interactions %s", json.encode(matched_interactions))


	--LogDebug("2 --> matched_interactions %s", json.encode(matched_interactions))

	local last = (tonumber(request.last) or 0) + #matched_interactions
	return 200, {
		interactions = matched_interactions,
		total_interactions = total_interactions,
		last = last,
		highlights = request.highlight and highlights
	}
end

---
--
local function SearchInteractions (web)

	local full_user = utils.AppendVcc( inconcert.authUser.userName, inconcert.authUser.virtualContactCenter )
	LogInfo("[Agent.SearchInteractions] User %s is SEARCHING interactions with request '%s'", full_user, web.input["post_data"])

	-- Las busquedas del agente (y del supervisor cuando tiene permisos de interaccion, pero esta usando la aplicacion agent)
	-- se filtran por attention level.
	local user_attn_levels = GetModel("Users").GetUserAttentionLevels( full_user )
	web.request.appId = "interaction"
	local status, response = SearchInteractionsHelper( web.request, user_attn_levels )
	web.status = status

	return response
end

local function SearchArchivedInteractions (web)

	local full_user = utils.AppendVcc( inconcert.authUser.userName, inconcert.authUser.virtualContactCenter )
	LogInfo("[Agent.SearchArchivedInteractions] User %s is SEARCHING ARCHIVED interactions.", full_user)
	if web.request.query then
		web.request.query.state = "FINISHED"
	end
	web.request.mustValidateSearchPermission = true -- debe validar permisos en las busquedas (cross campaign & ALs)
	return SearchInteractions( web )
end

local function SearchQueuedInteractions (web)

	local full_user = utils.AppendVcc( inconcert.authUser.userName, inconcert.authUser.virtualContactCenter )
	LogInfo("[Agent.SearchQueuedInteractions] User %s is SEARCHING QUEUED interactions.", full_user)
	if web.request.query then
		web.request.query.state = { "TRANSFER QUEUED", "QUEUED" }
	end
	web.request.mustValidateSearchPermission = false -- NO debe validar permisos en las busquedas (cross campaign & ALs)
	return SearchInteractions( web )
end

local function SearchWaitingInteractions (web)

	local full_user = utils.AppendVcc( inconcert.authUser.userName, inconcert.authUser.virtualContactCenter )
	LogInfo("[Agent.SearchWaitingInteractions] User %s is SEARCHING WAITING interactions.", full_user)
	if web.request.query then
		web.request.query.state = "WAITING"
	end
	web.request.mustValidateSearchPermission = false -- NO debe validar permisos en las busquedas (cross campaign & ALs)
	return SearchInteractions( web )
end

function SearchBotInteractions (request)

	local full_user = utils.AppendVcc( inconcert.authUser.userName, inconcert.authUser.virtualContactCenter )
	LogInfo("[Agent.SearchBotInteractions] User %s is SEARCHING BOT interactions.", full_user)
	
	local user_attn_levels = GetModel("Users").GetUserAttentionLevels( full_user )
	request.appId = "interaction"

	local modelInteractions = GetModel("Interactions")
	
	local matched_interactions, highlights = {}, {}
	local total_interactions = 0
	local campaignId
	local interactionType
	local needHelp

	for _, r in pairs(request.query.rules) do
		if r.id == "campaign" then
			campaignId = r.value
		end
		if r.id == "type" then
			interactionType = r.value
		end
		if r.id == "need_help" then
			needHelp = r.value
		end
	end

	local process_flow_interactions = modelInteractions.GetProcessFlowInteractions(ks_interactions, campaignId, interactionType, needHelp)

	for _, pfi in pairs(process_flow_interactions) do
		if pfi.data and pfi.data.reportingData and pfi.data.reportingData.process_flow_data and pfi.data.reportingData.process_flow_data.fields then
			for _, pfi_rdf in pairs(pfi.data.reportingData.process_flow_data.fields) do
				if pfi_rdf.name == 'internal_consultation_flag' then
					pfi.interactionData.internal_consultation_flag = pfi_rdf.value
				end
				if pfi_rdf.name == 'internal_consultation_question' then
					pfi.interactionData.internal_consultation_question = pfi_rdf.value
				end
				if pfi_rdf.name == 'internal_consultation_id' then
					pfi.interactionData.internal_consultation_id = pfi_rdf.value
				end
				if pfi_rdf.name == 'internal_consultation_timeout' then
					pfi.interactionData.internal_consultation_timeout = pfi_rdf.value
				end
				if pfi_rdf.name == 'internal_consultation_startdate' then
					pfi.interactionData.internal_consultation_startdate = pfi_rdf.value
				end
				if pfi_rdf.name == 'internal_consultation_kb' then
					pfi.interactionData.internal_consultation_kb = pfi_rdf.value
				end
			end
		end

		table.insert(matched_interactions, pfi)
		total_interactions = total_interactions + 1
	end

	local last = (tonumber(request.last) or 0) + #matched_interactions

	return 200, {
		interactions = matched_interactions,
		total_interactions = total_interactions,
		last = last,
		highlights = request.highlight and highlights
	}

end

local function SearchAbandonedInteractions (web)

	local full_user = utils.AppendVcc( inconcert.authUser.userName, inconcert.authUser.virtualContactCenter )
	LogInfo("[Agent.SearchAbandonedInteractions] User %s is SEARCHING ABANDONED interactions.", full_user)
	if web.request.query then
		web.request.query.state = "ABANDONED"
	end
	web.request.mustValidateSearchPermission = false
	return SearchInteractions( web )
end

local function SearchGlobalInteractions (web)

	local full_user = utils.AppendVcc( inconcert.authUser.userName, inconcert.authUser.virtualContactCenter )
	LogInfo("[Agent.SearchGlobalInteractions] User %s is SEARCHING interactions with request '%s'", full_user, json.encode( web.request ))

	web.request.mustValidateSearchPermission = true
	local status, response = SearchInteractionsHelper( web.request )
	web.status = status

	return response
end

local function CanWorkSpam (interaction_id, appId, markType)
	local result = {success = false, status = 400, reason = "Invalid request"}
	local interaction = GetModel("Interactions").GetInteraction(interaction_id)
	if utils.TableIsEmpty(interaction) then
		LogInfo("[AGENT.CanWorkSpam] Unable to '%s' interaction '%s'. Interaction does not exist.", markType, interaction_id)
		return result
	end
	if interaction.state ~= "TAKEN" and interaction.state ~= "TRANSFER TAKEN" then
		result.status = 403
		result.reason = "forbidden"
		LogInfo("[AGENT.CanWorkSpam] Unable to '%s' interaction '%s'. The current state is: '%s'.", markType, interaction_id, interaction.state)
		return result
	else
		local owner = interaction.assignedAgents and interaction.assignedAgents[1] or ""
		local loggedUserId = utils.GetCurrentLoggedUser()
		if owner:lower() ~= loggedUserId:lower() then
			LogInfo("[AGENT.CanWorkSpam] Unable to '%s' the interaction by user '%s'. The interaction '%s' belongs to the user '%s'.", markType, loggedUserId, interaction_id, owner)
			result.status = 403
			result.reason = "forbidden"
			return result
		end
	end
	-- markType = (MARK_AS_SPAM, NOT_SPAM, DELETE_SPAM)
	if not HasPermission(markType, interaction.campaign, appId) then
		result.status = 403
		result.reason = "forbidden"
		return result
	end

	result.success = true
	result.status = 200
	result.reason = "ok"
	return result
end

--
--spam
function MarkSpam (web)
	local res, request = pcall(json.decode, web.input["post_data"])
	if not res or not request then
		web.status = 400
		return "Invalid request"
	end
	assert(request.interactionIDs, "Parameter interactionIDs is required")
	local vcc = inconcert.authUser.virtualContactCenter
	local user_name = inconcert.authUser.userName

	LogInfo("[AGENT.MarkSpam] User %s on vcc %s wants to mark as spam interactions %s", user_name, vcc, json.encode(request.interactionIDs))

	local appId = request.app_id

	for _, interaction_id in ipairs(request.interactionIDs) do
		local result = CanWorkSpam(interaction_id, appId, "MARK_AS_SPAM")
		if not result.success then
			web.status = result.status
			return result.reason
		end
	end

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local res = irLink:MarkSpam(request.interactionIDs)
	web.status = http_mapping.getHTTPCode(res.result) or 500 --unknown result code

	return ReplyJSON(web, res)
end

function MarkNotSpam (web)
	local res, request = pcall(json.decode, web.input["post_data"])
	if not res or not request then
		web.status = 400
		return "Invalid request"
	end

	local vcc = inconcert.authUser.virtualContactCenter
	local user_name = inconcert.authUser.userName

	LogInfo("[AGENT.MarkNotSpam] User %s on vcc %s wants to mark as no spam interactions %s", user_name, vcc, json.encode(request.interactionIDs))

	local appId = request.app_id
	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)

	for _, interaction_id in ipairs( request.interactionIDs ) do
		local result = CanWorkSpam(interaction_id, appId, "NOT_SPAM")
		if not result.success then
			web.status = result.status
			return result.reason
		end
	end

	local res = irLink:MarkNotSpam(request.interactionIDs)

	web.status = http_mapping.getHTTPCode(res.result) or 500 --unknown result code
	return ReplyJSON(web, res)
end

function DeleteSpam (web)
	local res, request = pcall(json.decode, web.input["post_data"])
	if not res or not request then
		web.status = 400
		return "Invalid request"
	end

	local vcc = inconcert.authUser.virtualContactCenter
	local user_name = inconcert.authUser.userName

	LogInfo("[AGENT.DeleteSpam] User %s on vcc %s wants to delete spam interactions %s", user_name, vcc, json.encode(request.interactionIDs))

	local appId = request.app_id

	for _, interaction_id in ipairs( request.interactionIDs ) do
		local result = CanWorkSpam(interaction_id, appId, "DELETE_SPAM")
		if not result.success then
			web.status = result.status
			return result.reason
		end
	end

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)

	local res = irLink:DeleteSpam(request.interactionIDs)

	web.status = http_mapping.getHTTPCode(res.result) or 500 --unknown result code
	return ReplyJSON(web, res)
end

function GetSpam (web)
	_M.response.DontCache(web)

	local vcc = inconcert.authUser.virtualContactCenter
	local user_name = inconcert.authUser.userName

	LogInfo("User %s on vcc %s wants vcc's spam", user_name, vcc)

	local redis = agentModel:GetRedisClient()

	local interactions = {}
	local spamCount = 0

	local userCampaigns = GetModel("Campaigns").GetUserCampaigns(user_name, vcc)

	for campaign in pairs(userCampaigns) do
		--We sum all spam in all user's campaigns
		local count = redis:zcard("SpamInteractionsByCampaign:"..campaign)
		spamCount = spamCount + count

		local campaignInteractions = redis:zrevrange("SpamInteractionsByCampaign:"..campaign, 0, 29, "WITHSCORES")

		if type(campaignInteractions) == "table" then
			for _, tuple in ipairs(campaignInteractions) do
				table.insert(interactions, {id = tuple[1], timestamp = tuple[2]})
			end
		end
	end

	table.sort(interactions, function(a, b)
		return a.timestamp > b.timestamp --greater is best
	end)

	local modelInteractions = GetModel("Interactions")
	local result = {}
	local count = 0
	for _, int in ipairs(interactions) do
		local interaction = modelInteractions.GetInteraction(int.id)
		if not utils.TableIsEmpty(interaction) then
			table.insert(result, interaction)
			count = count + 1
			if count >= 30 then
				break
			end
		end
	end

	return ReplyJSON(web, {totalCount = spamCount, interactions = result})
end

-- Check if given group is valid
-- To be valid, a group must be a table where keys are interactionIds
-- and  values are tables, that can be empty, or can be groups
local function GroupIsValid(g)
	if type(g) ~= "table" then
		return false
	end
	for k, v in pairs(g) do
		if type(k) ~= "string" or k == "" or not GroupIsValid(v) then
			return false
		end
	end
	return true
end

---
-- Adds one or more interactions to a given group.
-- All interactions must belong to the same vcc as the authenticated user.
-- Also, the group must belong to this same vcc.
-- @param group_id (string) the id of the group
-- @param web.request has a table with the request data
--   web.request.interactions is an object containing the hierarchy of the group
--
function AddInteractionsToGroup (web, group_id)
	local interactions = web.request.interactions
	local vcc = inconcert.authUser.virtualContactCenter
	local rid = web.vars.HTTP_X_ICC_RID or "n/a"

	if group_id == nil or group_id == "" then
		group_id = uuid.new()
	end
	if type(interactions) ~= "table" then
		local msg = LogError("AddInteractionsToGroup: Invalid request. Must have a valid 'interactions' field")
		web.status = 400
		return { result = "INVALID_REQUEST", reason = _T("AddInteractionsToGroup: Invalid request. Must have a valid 'interactions' field") }
	elseif not GroupIsValid(interactions) then
		local msg = LogError("AddInteractionsToGroup: Invalid request. Interactions has an invalid structure: %s", json.encode(interactions))
		web.status = 400
		return { result = "INVALID_REQUEST", reason = _T("AddInteractionsToGroup: Invalid request. Interactions has an invalid structure: ") .. json.encode(interactions) }
	end

	LogInfo("AddInteractionToGroup: interactions %s will be added to group '%s'",
		web.input["post_data"], group_id)

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local result = irLink:AddInteractionsToGroup {rid = rid, interactions = interactions, groupId = group_id, vcc = vcc}

	if result.result == "OK" then
		result.group_id = group_id
	else
		web.status = 400
	end

	return result
end

---
-- Removes one or more interactions from a given group.
-- The group must belong to the same vcc as the authenticated user.
-- @param group_id (string) the id of the group
-- @param web.request has a table with the request data
--   web.request.interactions is either an array or a single interaction id
--
function RemoveInteractionsFromGroup (web, group_id)
	local interactions = web.request.interactions
	local vcc = inconcert.authUser.virtualContactCenter
	local rid = web.vars.HTTP_X_ICC_RID or "n/a"

	if type(interactions) ~= "table" then
		local msg = LogError("RemoveInteractionsFromGroup: Invalid request. Must have a valid 'interactions' field")
		web.status = 400
		return { result = "INVALID_REQUEST", reason = _T("RemoveInteractionsFromGroup: Invalid request. Must have a valid 'interactions' field") }
	elseif not GroupIsValid(interactions) then
		local msg = LogError("RemoveInteractionsFromGroup: Invalid request. Interactions has an invalid structure: %s", json.encode(interactions))
		web.status = 400
		return { result = "INVALID_REQUEST", reason = _T("RemoveInteractionsFromGroup: Invalid request. Interactions has an invalid structure: ") .. json.encode(interactions) }
	end

	LogInfo("RemoveInteractionsFromGroup: interactions '%s' will be removed from group '%s'",
		web.input["post_data"], group_id)

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	return irLink:RemoveInteractionsFromGroup {rid = rid, interactions = interactions, groupId = group_id, vcc = vcc}
end

---
-- Returns a json with all interaction ids that belong to the given group.
-- The group must belong to the same vcc as the authenticated user.
-- @param group_id (string) the id of the group
--
function GetInteractionsInGroup (web, group_id)
	_M.response.DontCache(web)
	local vcc = inconcert.authUser.virtualContactCenter
	local group = GetModel("Interactions").GetInteractionGroup(vcc, group_id)
	if group then
		return { result = "OK", group = group }
	else
		return { result = "FAIL", message = "group not found" }
	end
end

local function Print (web, interactionId, currentEvent)
	return mail.Print(web, m_htmlRender, interactionId, currentEvent)
end

local storeAttachments = {
	CHAT = true
}

local function SendAttachment (wsapi_env)
	local irRequest = nil
	local function sendToIR()
		local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
		local result = irLink:SendAttachment(irRequest)
		if result.result == "OK" then
			return 200, json.encode(result)
		end
		LogError("[SendAttachment] Error sending file %s", json.encode(result))
		return 500, result.reason or "Server error"
	end

	StartRepository()
	return repositoryController.storeFile(wsapi_env, "application/json",
		function(request)
			--make ir request
			irRequest = json.decode(request.params.p)
			if not irRequest.proxyType or not irRequest.campaignAtVCC or not irRequest.account or not irRequest.interactionID then
				LogError("[SendAttachment] Unable to upload file. Incomplete file information.")
				return 400, "Invalid file data"
			end
			if not HasPermission("CHAT_FILE_TRANSFER", irRequest.campaignAtVCC) then
				LogError("[SendAttachment] Unable to upload file. User has not permissions.")
				return 403, "User has not permissions"
			end
			LogInfo("[SendAttachment] campaign %s account %s interaction %s", irRequest.campaignAtVCC, irRequest.account, irRequest.interactionID)
			irRequest.file.id = uuid.new()
			if irRequest.file.extension ~= "" then
				irRequest.file.id = irRequest.file.id .. "." .. irRequest.file.extension
			end
			irRequest.agentAtVCC = utils.GetCurrentLoggedUser()
			--process content
			if storeAttachments[irRequest.proxyType] then	
				--upload file to repository
				request.params.id = irRequest.file.id
				request.params.fileName = irRequest.file.name
				request.params.section = -1
				request.params.format = ""
				request.params.contentType = irRequest.file.type
				--Column Family for file data must be "Files" folowed by proxyType capitalized (i.e. "FilesChat")
				request.params.proxyType = irRequest.proxyType
				request.params.vcc = irRequest.vcc
				return 200
			else
				--read and upload file to cache
				irRequest.data.temporalFile = true
				local fileSize = tonumber(wsapi_env.CONTENT_LENGTH)
				setmetatable(request.POST, nil)
				local chunkSize = 2048
				local table_file = {}
				local left = fileSize
				local blocksize = math.min(chunkSize, left)
				while left > 0 do
					local chunk = wsapi_env.input:read(blocksize)
					if not chunk then break end
					table.insert(table_file, chunk)
					left = left - #chunk
					blocksize = math.min(blocksize, left)
				end
				local fileContent = table.concat(table_file)
				GetModel("Interactions").SetTemporalFile(irRequest.file.id, fileContent)
				local status, result = sendToIR()
				if status == 200 then
					status = true
				end
				return status, result
			end
		end,
		nil,
		function()
			local status, result = sendToIR()
			if status ~= 200 then
				--error adding file to ir: remove file from repository
				repositoryController.deleteFile(irRequest.file.id, -1)
			end
			return status, result
		end
	)
end

local function SendSharedFile(web)
	local agentAtVCC = utils.GetCurrentLoggedUser()

	LogInfo("SendSharedFile from agent:%s, post_data: '%s'.", agentAtVCC, web.input["post_data"])

	local request = json.decode(web.input["post_data"])

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	LogInfo("SendSharedFile called for id:%s from agent %s - message %s", request.interactionID, agentAtVCC, request.message)
	local response = irLink:SendSharedFile({
		interactionID    = request.interactionID,
		agentAtVCC		 = agentAtVCC,
		RID              = web.vars.HTTP_X_ICC_RID,
		proxyType		 = request.proxyType,
		message          = request.message,
		displayName      = request.displayName or nil
	})
	if response.result ~= "OK" then
		LogWarning("Error sending shared file to %s. Reason: %s", request.interactionID, response.reason)
	end

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(response["result"])
	web.status = http_code or 500 --unknown result code

	return ReplyJSON( web, response )
end

local function SendRepositoryFile(web)
	local agentAtVCC = utils.GetCurrentLoggedUser()

	LogInfo("SendRepositoryFile from agent:%s, post_data: '%s'.", agentAtVCC, web.input["post_data"])

	local request = json.decode(web.input["post_data"])
	
	if not HasPermission("CHAT_FILE_TRANSFER", request.campaignAtVCC) then
		LogError("[SendRepositoryFile] Unable to send file. User has not permissions.")
		web.status = 403
		return ReplyJSON( web, { result = "FORBIDDEN", reason = "User has not permissions" })
	end

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	LogInfo("SendRepositoryFile called for id:%s from agent %s - message %s", request.interactionID, agentAtVCC, request.message)
	local response = irLink:SendAttachment({
		campaignAtVCC	 = request.campaignAtVCC,
		account			 = request.account,
		interactionID    = request.interactionID,
		agentAtVCC		 = agentAtVCC,
		RID              = web.vars.HTTP_X_ICC_RID,
		proxyType		 = request.proxyType,
		data			 = request.data,
		file			 = request.file,
	})
	if response.result ~= "OK" then
		LogWarning("Error sending shared file to %s. Reason: %s", request.interactionID, response.reason)
	end

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(response["result"])
	web.status = http_code or 500 --unknown result code

	return ReplyJSON( web, response )
end

local function StartRecord (web)
	local res, request = pcall(json.decode, web.input["post_data"])
	if not res or not request then
		web.status = 400
		return json.encode({ result = "INVALID_REQUEST", reason = "Invalid request: ".. web.input["post_data"] })
	end

	LogInfo("StartRecord called for id:%s", request.id)

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local result = irLink:StartRecord(request.id);

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(result["result"])
	web.status = http_code or 500 --unknown result code

	return ReplyJSON(web, result)
end

local function StopRecord (web)
	local res, request = pcall(json.decode, web.input["post_data"])
	if not res or not request then
		web.status = 400
		return json.encode({ result = "INVALID_REQUEST", reason = "Invalid request: ".. web.input["post_data"] })
	end

	LogInfo("StopRecord called for id:%s", request.id)

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local result = irLink:StopRecord(request.id);

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(result["result"])
	web.status = http_code or 500 --unknown result code

	return ReplyJSON(web, result)
end

local function CanTransferInteractionToMember (web)
	local vcc = inconcert.authUser.virtualContactCenter
	local postData = web.input["post_data"]

	LogInfo("[INTERACTIONS:CanTransferInteractionToMember] Validating if the interaction can be transferred to the user. vcc: '%s', postData: '%s'.", vcc, postData)

	local request = web.request
	if not request.interaction_id or not request.target then
		web.status = 400
		LogError("[INTERACTIONS:CanTransferInteractionToMember] Unable transfer interaction to Member. Invalid request.")
		return ReplyJSON(web, {result = "INVALID_REQUEST", reason = "Invalid request: " .. postData})
	end

	local result = CanTransferToMember({
		interactionId = request.interaction_id,
		target        = request.target,
		appId         = request.app_id
	})
	if not result.success then
		local errorMsg = LogError("[INTERACTIONS:CanTransferInteractionToMember] Unable transfer interaction '%s' to Member '%s'. Reason: '%s'.",
			request.interaction_id,
			request.target.user,
			result.reason
		)
		web.status = 500
		return ReplyJSON(web, {result = result.reason, reason = errorMsg})
	end

	-- OK
	web.status = 200
	return ReplyJSON(web, result)
end

local function Hold (web)
	local res, request = pcall(json.decode, web.input["post_data"])
	if not res or not request then
		web.status = 400
		return json.encode({ result = "INVALID_REQUEST", reason = "Invalid request: ".. web.input["post_data"] })
	end

	LogInfo("Hold called for id:%s", request.id)

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local result = irLink:Hold(request.id);

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(result["result"])
	web.status = http_code or 500 --unknown result code

	return ReplyJSON(web, result)
end

local function Unhold (web)
	local res, request = pcall(json.decode, web.input["post_data"])
	if not res or not request then
		web.status = 400
		return json.encode({ result = "INVALID_REQUEST", reason = "Invalid request: ".. web.input["post_data"] })
	end

	LogInfo("Unhold called for id:%s", request.id)

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local result = irLink:Unhold(request.id);

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(result["result"])
	web.status = http_code or 500 --unknown result code

	return ReplyJSON(web, result)
end

local function SwitchCall(web)
	local res, request = pcall(json.decode, web.input["post_data"])
	if not res or not request then
		web.status = 400
		return json.encode({ result = "INVALID_REQUEST", reason = "Invalid request: ".. web.input["post_data"] })
	end

	local user_name = inconcert.authUser.userName

	LogInfo("SwitchCall called for id:%s by %s", request.id, user_name)

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local result = irLink:SwitchConversation({
		proxyType = "CALL",
		agentAtVCC = user_name,
		interactionID = request.id,
		RID = web.vars.HTTP_X_ICC_RID,
	})

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(result["result"])
	web.status = http_code or 500 --unknown result code

	return ReplyJSON(web, result)
end

local function UnswitchCall(web)
	local res, request = pcall(json.decode, web.input["post_data"])
	if not res or not request then
		web.status = 400
		return json.encode({ result = "INVALID_REQUEST", reason = "Invalid request: ".. web.input["post_data"] })
	end

	local user_name = inconcert.authUser.userName

	LogInfo("UnswitchCall called for id:%s by %s", request.id, user_name)

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local result = irLink:UnswitchConversation({
		proxyType = "CALL",
		agentAtVCC = user_name,
		interactionID = request.id,
		RID = web.vars.HTTP_X_ICC_RID,
	})

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(result["result"])
	web.status = http_code or 500 --unknown result code

	return ReplyJSON(web, result)
end

local function AgentVideoChange (web)
	local res, request = pcall(json.decode, web.input["post_data"])
	if not res or not request then
		web.status = 400
		return json.encode({ result = "INVALID_REQUEST", reason = "Invalid request: ".. web.input["post_data"] })
	end

	local user_name = inconcert.authUser.userName

	LogInfo("AgentVideoChange called for id:%s by %s", request.id, user_name)

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local result = irLink:AgentVideoChange({
		proxyType = "CALL",
		agentAtVCC = user_name,
		interactionID = request.id,
		change = request.change,
		RID = web.vars.HTTP_X_ICC_RID,
	})

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(result["result"])
	web.status = http_code or 500 --unknown result code

	return ReplyJSON(web, result)
end

local function SaveApiAdditionalData (web)
	local res, request = pcall(json.decode, web.input["post_data"])
	if not res or not request then
		web.status = 400
		return "Invalid request"
	end
	assert(request.id, "Parameter id is required")
	assert(request.data, "Parameter data is required")
	local vcc = inconcert.authUser.virtualContactCenter
	local user_name = inconcert.authUser.userName

	LogInfo("[AGENT.SaveApiAdditionalData] User %s on vcc %s wants to save API additional data on interaction %s: '%s'", user_name, vcc, request.id, json.encode(request.data))

	GetModel("Interactions").SaveApiAdditionalData(request.id, json.encode(request.data))

	local result = {}
	result.result = "OK"

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(result["result"])
	web.status = http_code or 500 --unknown result code

	return ReplyJSON(web, result)
end

local function SetPreInteractionData (web)
	local res, request = pcall(json.decode, web.input["post_data"])
	if not res or not request then
		web.status = 400
		return "Invalid request"
	end
	assert(request.id, "Parameter id is required")
	assert(request.data, "Parameter data is required")
	local vcc = inconcert.authUser.virtualContactCenter
	local user_name = inconcert.authUser.userName

	LogInfo("[AGENT.SetPreInteractionData] User %s on vcc %s interaction %s: '%s'", 
		user_name, vcc, request.id, json.encode(request.data))

	GetModel("Interactions").SetPreInteractionData(request.id, json.encode(request.data))

	local result = {}
	result.result = "OK"

	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(result["result"])
	web.status = http_code or 500 --unknown result code

	return ReplyJSON(web, result)
end

local function DialerPreviewCancel(web)
	local _T = m_htmlRender.extension.makeTextTranslator()
	local vcc = inconcert.authUser.virtualContactCenter
	local user_name = utils.GetCurrentLoggedUser()
	local request = json.decode(web.input["post_data"])

	LogInfo("[interactions::DialerPreviewCancel] %s %s",
			user_name, web.input["post_data"])

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local result = irLink:DialerPreviewCancel({
		proxyType = request.proxyType,
		interactionID = request.interactionID,
	})
	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(result["result"])
	web.status = http_code or 500 --unknown result code

	if result.result ~= "OK" then
		LogInfo("[interactions::DialerPreviewCancel] %s %s - %s",
				user_name, web.input["post_data"], json.encode(result))
		result.reason = _T(result.reason)
		return ReplyJSON(web, res)
	end

	LogInfo("[interactions::DialerPreviewCancel] %s %s - success",
			user_name, web.input["post_data"])
	return ReplyJSON(web, res)
end

local function DialerPreviewConfirm(web)
	local _T = m_htmlRender.extension.makeTextTranslator()
	local vcc = inconcert.authUser.virtualContactCenter
	local user_name = utils.GetCurrentLoggedUser()
	local request = json.decode(web.input["post_data"])

	LogInfo("[interactions::DialerPreviewConfirm] %s %s",
			user_name, web.input["post_data"])

	local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
	local result = irLink:DialerPreviewConfirm({
		proxyType = request.proxyType,
		interactionID = request.interactionID,
	})
	--Convert result code to HTTP code
	local http_code = http_mapping.getHTTPCode(result["result"])
	web.status = http_code or 500 --unknown result code

	if result.result ~= "OK" then
		LogInfo("[interactions::DialerPreviewConfirm] %s %s - %s",
				user_name, web.input["post_data"], json.encode(result))
		result.reason = _T(result.reason)
		return ReplyJSON(web, res)
	end

	LogInfo("[interactions::DialerPreviewConfirm] %s %s - success",
			user_name, web.input["post_data"])
	return ReplyJSON(web, res)
end

local function SaveDrafts(web)
	local full_user = utils.AppendVcc( inconcert.authUser.userName, inconcert.authUser.virtualContactCenter )
	local res, request = pcall(json.decode, web.input["post_data"])
	--LogInfo("[interactions::SaveDrafts] %s %s", full_user, web.input["post_data"])
	if not res or not request or not request.id or not request.type or not request.draft then
		web.status = 400
		return ReplyJSON(web, {result = "INVALID_REQUEST"})
	end

	local isPending = request.state and (request.state == "PENDING" or request.state == "CREATED") or false
	if isPending then
		-- pending: Enviar a Cassandra
		local rows = {
			[full_user] = {
				[request.id] = request
			}
		}
		GetModel("Interactions").SaveDraftForPending(rows)
	else
		-- Enviar a IR para add o update
		local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
		local response = irLink:SendCommand({
			interactionId    = request.id,
			RID              = web.vars.HTTP_X_ICC_RID,
			proxyType        = request.type,
			draft			 = request.draft,
			command          = "SaveDraftsConversation"
		})
		if response.result ~= "OK" then
			LogWarning("Unable save draft of interaction '%s'. Reason: %s", request.id, response.reason)
			web.status = 500
			return ReplyJSON(web, response)
		end
	end

	return ReplyJSON(web, {result = "OK"})
end

local function DeleteDrafts(web)
	local full_user = utils.AppendVcc( inconcert.authUser.userName, inconcert.authUser.virtualContactCenter )
	--LogInfo("[interactions::DeleteDrafts] %s %s", full_user, web.input["post_data"])
	
	local res, request = pcall(json.decode, web.input["post_data"])
	if not res or not request or not request.id or not request.type then
		web.status = 400
		return ReplyJSON(web, {result = "INVALID_REQUEST"})
	end

	local isPending = request.isPending
	if isPending then
		-- pending: Enviar a Cassandra
		local rows = {
			[full_user] = {request.id}
		}
		GetModel("Interactions").DeleteDraftForPending(rows)
	else
		-- Enviar a IR para eliminar
		local irLink = ir.IRAgentLink(inconcert.environment.apps.agent.cluster)
		local response = irLink:SendCommand({
			interactionId    = request.id,
			RID              = web.vars.HTTP_X_ICC_RID,
			proxyType        = request.type,
			draft			 = request.draft,
			command          = "RemoveDraftsConversation"
		})
		if response.result ~= "OK" then
			LogWarning("Unable delete draft of interaction '%s'. Reason: %s", request.id, response.reason)
			web.status = 500
			return ReplyJSON(web, response)
		end
	end

	return ReplyJSON(web, {result = "OK"})
end

local function MapListingUris (m, perm, p, Log, jsonValidator, vr, r)
	m:dispatch_post( Log .. vr .. GetInteraction                      , p .. "/get_interaction/(.*)"                                )
	m:dispatch_post( Log .. vr .. GetBotInteraction                   , p .. "/get_bot_interaction/(.*)"                            )
	m:dispatch_post( Log .. vr .. GetArchivedInteraction              , p .. "/get_archived_interaction/(.*)"                       )
	m:dispatch_post( Log .. vr .. GetUserInteractions                 , p .. "/get_user_interactions/(.*)"                          )
	m:dispatch_post( Log .. vr .. GetCampaignInteractions             , p .. "/get_campaign_interactions/([^.]*)"                   )
	m:dispatch_post( Log .. vr .. GetTopQueuedInteractions            , p .. "/get_top_queued_interactions/(.*)"                    )
end

local function MapSharedUris (m, perm, p, Log, jsonValidator, vr, r)
	m:dispatch_post( Log .. ArchiveInteractions                       , p .. "/archive_interactions/(.*)"                           )
	m:dispatch_post( Log .. SetDisposition                            , p .. "/set_disposition/(.*)"                                )
	--take
	m:dispatch_post( Log .. vr .. TakeInteraction                     , p .. "/take_interaction/(.*)"                               )
	m:dispatch_post( Log .. vr .. TakeBotInteraction                  , p .. "/take_bot_interaction/(.*)"                           )
	m:dispatch_post( Log .. vr .. TakeNextBest                        , p .. "/take_next_best/(.*)"                                 )
	--transfer
	m:dispatch_post( Log .. vr .. GetMembersForTransfer               , p .. "/get_members_for_transfer/(.*)"                       )
	m:dispatch_post( Log .. vr .. GetCampaignUsers                    , p .. "/get_campaign_users/(.*)"                             )
	m:dispatch_post( Log .. vr .. GetCampaignsAccountsForTransfer     , p .. "/get_campaigns_accounts_for_transfer/(.*)"            )
	m:dispatch_post( Log .. vr .. GetAccountsAttLevelsForTransfer     , p .. "/get_accounts_attention_levels_for_transfer/(.*)"     )
	m:dispatch_post( Log .. vr .. GetAccountsBotsForTransfer          , p .. "/get_accounts_bots_for_transfer/(.*)"                 )
	m:dispatch_post( Log .. TransferInteractions                      , p .. "/transfer_interactions/(.*)"                          )
	m:dispatch_post( Log .. vr .. CanTransferInteractionToMember      , p .. "/can_transfer_to_member/"                             )
	--reopen
	m:dispatch_post( Log .. ReopenInteraction                         , p .. "/reopen_interaction/(.*)"                             )
	--requeue
	m:dispatch_post( Log .. RequeueInteractions                       , p .. "/requeue_interactions/(.*)"                           )
	--take interactions
	m:dispatch_post( Log .. TakeInteractions                          , p .. "/take_interactions/(.*)"                              )
	--read
	m:dispatch_post( Log .. vr .. ReadInteraction                     , p .. "/read_interaction/(.*)"                               )
	--contact
	m:dispatch_post( Log .. SetInteractionContact                     , p .. "/set_interaction_contact/(.*)"                        )
	-- comments
	m:dispatch_post( Log .. CommentInteraction                        , p .. "/comment_interaction/(.*)"                            )
	m:dispatch_post( Log .. vr .. SetCommentsAsRead                   , p .. "/read_comments/(.*)"                                  )
	--appointment
	m:dispatch_post( Log .. vr .. MakeAppointment                     , p .. "/make_appointment/(.*)"                               )
	m:dispatch_post( Log .. vr .. TakeAppointment                     , p .. "/take_appointment/(.*)"                               )
	m:dispatch_post( Log .. vr .. CanTakeAppointment                  , p .. "/can_take_appointment/(.*)"                           )
	--search
	-- TODO: Leave just 'search' url
	m:dispatch_post( Log .. vr .. SearchArchivedInteractions          , p .. "/search_archive/(.*)"                                 )
	m:dispatch_post( Log .. vr .. SearchQueuedInteractions            , p .. "/search_queued/(.*)"                                  )
	m:dispatch_post( Log .. vr .. SearchWaitingInteractions           , p .. "/search_waiting/(.*)"                                 )
	m:dispatch_post( Log .. vr .. SearchAbandonedInteractions         , p .. "/search_abandoned/(.*)"                               )
	m:dispatch_post( Log .. vr .. SearchGlobalInteractions            , p .. "/global_search/(.*)"                                  )
	--labels
	m:dispatch_post( Log .. UpdateInteractionsLabels                  , p .. "/update_labels_interactions/(.*)"                     )
	m:dispatch_post( Log .. DeleteLabelsInteractions                  , p .. "/delete_labels_interactions/(.*)"                     )
	--conflicts/failed
	m:dispatch_post( Log .. RemoveContactConflict                     , p .. "/remove_contact_conflict/(.*)"                        )
	m:dispatch_post( Log .. DiscardFailedEvent                        , p .. "/discard_failed_event/(.*)"                           )
	m:dispatch_post( Log .. FailSendingEvent                          , p .. "/fail_sending_event/(.*)"                             )
	--custom message
	m:dispatch_post( Log .. SendCustomMessage                         , p .. "/send_custom_message/(.*)"                            )
	--attachment
	m:dispatch_wsapi( Log .. SendAttachment	                          , p .. "/send_attachment/(.*)"                                )
	--shared file
	m:dispatch_post( Log .. SendSharedFile	                          , p .. "/send_shared_file/(.*)"                               )
	--repository file
	m:dispatch_post( Log .. SendRepositoryFile                        , p .. "/send_repository_file/(.*)"                           )
	--conference
	m:dispatch_post( Log .. ConferenceInteractions                    , p .. "/conference_interactions/(.*)"                        )
	--pre interaction
	m:dispatch_post( Log .. SetPreInteractionData                     , p .. "/set_preinteraction_data/(.*)"                        )
end

local function MapInteractionGroupUris(m, perm, p, Log, jsonValidator, vr, r)
	m:dispatch_post( Log .. vr .. AddInteractionsToGroup              , p .. "/group/add/(.*)"                                      )
	m:dispatch_post( Log .. vr .. RemoveInteractionsFromGroup         , p .. "/group/remove_from/(.+)"                              )
	m:dispatch_get(  Log .. r  .. GetInteractionsInGroup              , p .. "/group/get/(.+)"                                      )
end

local function MapCallUris(m, perm, p, Log, jsonValidator, vr, r)
	m:dispatch_post( Log .. CreateOutgoingCALL                        , p .. "/call/create_outgoing/"                               )
	m:dispatch_post( Log .. CloseAgentLeg                             , p .. "/call/close_agent_leg/"                               )
	m:dispatch_post( Log .. StartAttendedTransfer                     , p .. "/call/start_attended_transfer/"                       )
	m:dispatch_post( Log .. CommitAttendedTransfer                    , p .. "/call/commit_attended_transfer/"                      )
	m:dispatch_post( Log .. CancelAttendedTransfer                    , p .. "/call/cancel_attended_transfer/"                      )
	m:dispatch_post( Log .. MonitorCall                               , p .. "/call/monitor/"                                       )
	m:dispatch_post( Log .. StartRecord                               , p .. "/call/start_record/"                                  )
	m:dispatch_post( Log .. StopRecord                                , p .. "/call/stop_record/"                                   )
	m:dispatch_post( Log .. Hold                                      , p .. "/call/hold/"                                          )
	m:dispatch_post( Log .. Unhold                                    , p .. "/call/unhold/"                                        )
	m:dispatch_post( Log .. SwitchCall                                , p .. "/call/switch/"                                        )
	m:dispatch_post( Log .. UnswitchCall                              , p .. "/call/unswitch/"                                      )
	m:dispatch_post( Log .. AgentVideoChange                          , p .. "/call/agent_video_change/"                            )
end

local function MapImUris(m, perm, p, Log, jsonValidator, vr, r)
	m:dispatch_post( Log .. CreateOutgoingIM                          , p .. "/im/create_outgoing/(.*)"                             )
end

local function MapFacebookUris(m, perm, p, Log, jsonValidator, vr, r)
	m:dispatch_get ( Log .. GetAccountFacebookPages                   , p .. "/get_account_facebook_pages/([^/]+)/(.*)"             )
	m:dispatch_post( Log .. CreateOutgoingFacebook                    , p .. "/facebook/create_outgoing/(.*)"                       )
	m:dispatch_post( Log .. CommentFacebookPost                       , p .. "/facebook/comment_post/(.*)"                          )
	m:dispatch_post( Log .. FacebookPrivateReply	                  , p .. "/facebook/private_reply/(.*)"                         )
	m:dispatch_post( Log .. NewFacebookPrivateMessage                 , p .. "/facebook/new_private_message/(.*)"                   )
	m:dispatch_post( Log .. LikeFacebookObject                        , p .. "/facebook/like/(.*)"                                  )
	m:dispatch_post( Log .. GetFacebookPages                          , p .. "/facebook/get_account_pages/(.*)"                     )
	m:dispatch_wsapi(Log .. GetFacebookAvatar                         , p .. "/facebook/get_avatar/(.*)"                            )
end

local function MapInstagramUris(m, perm, p, Log, jsonValidator, vr, r)
	m:dispatch_post ( Log .. GetInstagramMediaData                   , p .. "/instagram/get_media_data/(.*)"             )
	m:dispatch_post( Log .. CommentInstagramPost                      , p .. "/instagram/comment_post/(.*)"                         )
	m:dispatch_post( Log .. InstagramPrivateReply	                  , p .. "/instagram/private_reply/(.*)"                        )
	m:dispatch_post( Log .. NewInstagramPrivateMessage                , p .. "/instagram/new_private_message/(.*)"                  )
	m:dispatch_post( Log .. LikeInstagramObject                       , p .. "/instagram/like/(.*)"                                 )
	m:dispatch_post( Log .. LikeInstagramMessage                      , p .. "/instagram/like_message/(.*)"                         )
	m:dispatch_wsapi(Log .. GetInstagramAvatar                        , p .. "/instagram/get_avatar/(.*)"                           )
	m:dispatch_wsapi(Log .. GetInstagramMedia                         , p .. "/instagram/get_media/(.*)"                            )
end

local function MapYoutubeUris(m, perm, p, Log, jsonValidator, vr, r)
	m:dispatch_post( Log .. CommentYoutubePost                        , p .. "/youtube/comment_post/(.*)"                           )
	m:dispatch_post( Log .. NewYoutubePrivateMessage                  , p .. "/youtube/new_private_message/(.*)"                    )
	m:dispatch_post( Log .. LikeYoutubeObject                         , p .. "/youtube/like/(.*)"                                   )
end

local function MapAppstoreUris(m, perm, p, Log, jsonValidator, vr, r)
	m:dispatch_post( Log .. CommentAppstorePost                       , p .. "/appstore/comment_post/(.*)"                          )
end

local function MapGooglePlayUris(m, perm, p, Log, jsonValidator, vr, r)
	m:dispatch_post( Log .. CommentGoogleplayPost                     , p .. "/googleplay/comment_post/(.*)"                        )
end

local function MapTwitterUris(m, perm, p, Log, jsonValidator, vr, r)
	m:dispatch_post( Log .. Tweet                                     , p .. "/twitter/tweet/(.*)"                                  )
	m:dispatch_post( Log .. ReplyTweet                                , p .. "/twitter/reply_tweet/(.*)"                            )
	m:dispatch_post( Log .. Retweet                                   , p .. "/twitter/retweet/(.*)"                                )
	m:dispatch_post( Log .. SendDirectMessage                         , p .. "/twitter/send_dm/(.*)"                                )
	m:dispatch_post( Log .. SendDirectMessageFromTweet                , p .. "/twitter/send_dm_from_tweet/(.*)"                     )
	m:dispatch_post( Log .. CreateTwitterInteraction                  , p .. "/twitter/create_outgoing/(.*)"                        )
	m:dispatch_post( Log .. LikeTwitterObject                         , p .. "/twitter/like/(.*)"                                   )
	m:dispatch_wsapi(Log .. GetTwitterContentMedia                   , p .. "/twitter/get_content_media/(.*)"                       )
end

local function MapSmsUris(m, perm, p, Log, jsonValidator, vr, r)
	m:dispatch_post( Log .. SendSms                                   , p .. "/sms/send_message/(.*)"                               )
	m:dispatch_post( Log .. SmsCreateAndSend                          , p .. "/sms/create_and_send/(.*)"                            )
	m:dispatch_post( Log .. SmsForward                                , p .. "/sms/forward/(.*)"                                    )
end

local function MapWhatsappUris(m, perm, p, Log, jsonValidator, vr, r)
	m:dispatch_post( Log .. SendWhatsapp                              , p .. "/whatsapp/send_message/(.*)"                          )
	m:dispatch_post( Log .. SendInternalWhatsapp                      , p .. "/whatsapp/send_internal_message/(.*)"                 )
	m:dispatch_post( Log .. WhatsappCreateAndSend                     , p .. "/whatsapp/create_and_send/(.*)"                       )
	m:dispatch_post( Log .. WhatsappForward                           , p .. "/whatsapp/forward/(.*)"                               )
end

local function MapRcsUris(m, perm, p, Log, jsonValidator, vr, r)
	m:dispatch_post( Log .. SendRcs                                   , p .. "/rcs/send_message/(.*)"                               )
	m:dispatch_post( Log .. RcsCreateAndSend                          , p .. "/rcs/create_and_send/(.*)"                            )
	m:dispatch_post( Log .. RcsForward                                , p .. "/rcs/forward/(.*)"                                    )
end

local function MapMailUris(m, perm, p, Log, jsonValidator, vr, r)
	m:dispatch_post( Log .. SendMail                                  , p .. "/mail/send_message/(.*)"                              )
	m:dispatch_post( Log .. jsonValidator .. MailCreateAndSend        , p .. "/mail/create_and_send/(.*)"                           )
	m:dispatch_get ( Log .. LoadFoldersMailAccount                    , p .. "/mail/load_account_folders/([^/]+)/(.*)"              )
	m:dispatch_post( Log .. MoveMailToFolder                          , p .. "/mail/move_to_folder/"                                )
end

local function MapSpamUris(m, perm, p, Log, jsonValidator, vr, r)
	m:dispatch_get(  Log .. GetSpam                                   , p .. "/spam/get_spam/"                                      )
	m:dispatch_post( Log .. MarkSpam                                  , p .. "/spam/mark_as_spam/(.*)"                              )
	m:dispatch_post( Log .. MarkNotSpam                               , p .. "/spam/mark_not_spam/(.*)"                             )
	m:dispatch_post( Log .. DeleteSpam                                , p .. "/spam/delete_spam/(.*)"                               )
end

local function MapPrintUris(m, perm, p, Log, jsonValidator, vr, r)
	m:dispatch_get ( Log .. Print                                     , p .. "/print/([^/]+)/(.*)"                                  )
end

local function MapDialerUris(m, perm, p, Log, jsonValidator, vr, r)
	m:dispatch_post( Log .. DialerPreviewCancel                       , p .. "/dialer/preview_cancel/"                              )
	m:dispatch_post( Log .. DialerPreviewConfirm                      , p .. "/dialer/preview_confirm/"                             )
end

local function MapApiUris(m, perm, p, Log, jsonValidator, vr, r)
	m:dispatch_post( Log .. SaveApiAdditionalData                     , p .. "/save_api_additional_data/(.*)"                       )
end

local function MapDraftsUris(m, perm, p, Log, jsonValidator, vr, r)
	m:dispatch_post( Log .. SaveDrafts                                , p .. "/drafts/save/"                                        )
	m:dispatch_post( Log .. DeleteDrafts                              , p .. "/drafts/delete/"                                      )
end

function MapUris( m, htmlRender, prefix )
	local perm = permissions
	local p = prefix
	local Log = require("apps.agent.requestFilters").LogRequest
	local jsonValidator = utils.JsonValidator
	local vr = utils.JsonValidatorReply
	local r = utils.JsonReply

	MapListingUris(m, perm, p, Log, jsonValidator, vr, r)
	MapSharedUris(m, perm, p, Log, jsonValidator, vr, r)
	MapInteractionGroupUris(m, perm, p, Log, jsonValidator, vr, r)
	MapCallUris(m, perm, p, Log, jsonValidator, vr, r)
	MapImUris(m, perm, p, Log, jsonValidator, vr, r)
	MapFacebookUris(m, perm, p, Log, jsonValidator, vr, r)
	MapInstagramUris(m, perm, p, Log, jsonValidator, vr, r)
	MapYoutubeUris(m, perm, p, Log, jsonValidator, vr, r)
	MapAppstoreUris(m, perm, p, Log, jsonValidator, vr, r)
	MapGooglePlayUris(m, perm, p, Log, jsonValidator, vr, r)
	MapTwitterUris(m, perm, p, Log, jsonValidator, vr, r)
	MapSmsUris(m, perm, p, Log, jsonValidator, vr, r)
	MapWhatsappUris(m, perm, p, Log, jsonValidator, vr, r)
	MapRcsUris(m, perm, p, Log, jsonValidator, vr, r)
	MapMailUris(m, perm, p, Log, jsonValidator, vr, r)
	MapSpamUris(m, perm, p, Log, jsonValidator, vr, r)
	MapPrintUris(m, perm, p, Log, jsonValidator, vr, r)
	MapDialerUris(m, perm, p, Log, jsonValidator, vr, r)
	MapApiUris(m, perm, p, Log, jsonValidator, vr, r)
	MapDraftsUris(m, perm, p, Log, jsonValidator, vr, r)
	m_htmlRender = htmlRender
end
