/* jshint forin:true, noarg:true, jquery:true, noempty:true, eqeqeq:true, bitwise:true,
	strict:true, undef:true, unused:vars, curly:false, browser:true, indent:false, maxerr:50, quotmark:double, -W083, -W014, evil: true, esversion: 11 */

/* global $, inConcert */

if (!this.inConcert) {
	this.inConcert = {};
}

(function () {

//	"use strict";

	const FORMAT = inConcert.Utils.format;

	const jQUERY_SELECTOR                      = (n, v)=> `[${n}='${v}']`;
	const jQUERY_SELECTOR_LDAP_GROUP_FILTER    = (v)=> jQUERY_SELECTOR("LDAP_GROUP_FILTER", v);
	const jQUERY_SELECTOR_LDAP_GROUP_NAME      = (v)=> jQUERY_SELECTOR("LDAP_GROUP_NAME", v);
	const jQUERY_SELECTOR_AZUREAD_GROUP_FILTER = (v)=> jQUERY_SELECTOR("AZUREAD_GROUP_FILTER", v);

	//---------------------------------------------------------------------
	// apps channels limit (clase base)
	function apps_channels_limit (params)
	{
		this.owner = params.owner;
		this.m_total_channels_config = Object.keys(this.m_channels_config).length;
	}

	apps_channels_limit.prototype.init = function()
	{
		this.make_channels_markup();
	};

	apps_channels_limit.prototype.make_channels_markup = function()
	{
		const self = this;
		const $channels = $("#" + this.id_section);
		$channels.html("");

		if (this.owner.frm.is_new) {
			return;
		}

		const is_readonly = false;
		const readonly_class = is_readonly ? "readonly" : "";
		const readonly_prop = is_readonly ? " disabled='true' " : "";
		const all_channels = inConcert.app.load_vcc_channels();

		var must_include_channel = function(channel_id) {
			let channel_info = all_channels[channel_id];
			channel_id = (channel_id || "").toUpperCase();
			if (!self.m_channels_config[channel_id]) {
				return false;
			}
			self.m_channels_config[channel_id].translated_name = channel_info.name;
			return true;
		};

		for (let channel_id in all_channels) {
			if (all_channels.hasOwnProperty(channel_id)) {
				if (!must_include_channel(channel_id)) {
					continue;
				}
				let channel = all_channels[channel_id];//
				let name = channel.name;//
				let channel_license_id = self.id_section + "_opt_channel_" + channel_id;
				let class_name = channel_id;
				let channel_limit_id = self.id_section + "_opt_channel_limit_" + channel_id;

				let $channel = $(
					"<div class='col-3 " + class_name + "' channel_id='" + channel_id + "' list_item='true'>" +
						"<div class='row'>" +
							"<div class='col-12'>" +
								"<div class='form-group customEditable'>" +
								"<label for='" + channel_license_id + "' class='form-check-label form-check-label-title form-check-label-title-customEditable'>"+ 
									name +
								"</label>" +
									"<div class='d-flex align-items-center'>" +
										"<div class='form-check mt-0'>" +
											"<input class='form-check-input " + readonly_class + "' id='"  + channel_license_id + "' channel_id='" + channel_id + "' selector='1' type='checkbox' " + readonly_prop + " />"  +
										"</div>" +
										"<input id='" + channel_limit_id + "' class='form-control' type='text' disabled='true' channel_id='" + channel_id + "' ctrl='limit' />" +
										"<i id='" + channel_limit_id + "_validation' class='formFieldValidation formFieldError' hidden></i>" +
									"</div>" +
								"</div>" +
							"</div>" +
						"</div>" +
						"<i error='true' class='formFieldValidation formFieldError' hidden ></i>" +
					"</div>");
	
				// agrego el channel al markup
				$channels.append($channel);
			}
		}
	};

	apps_channels_limit.prototype.get = function(vcc)
	{
		if (!vcc) {
			return;
		}
		const $container = $("#" + this.id_section);
		let limit_values;
		$("#" + this.id_section + " input[selector='1'][type='checkbox']").each(function(_, channel)
		{
			const $checkbox = $(channel);
			const channel_name = $checkbox.attr("channel_id").toUpperCase(); // sample FACEBOOK
			const enabled = $checkbox.prop("checked");
			if (enabled) {
				const $limit = $container.find("[ctrl][channel_id='" + channel_name + "']");
				limit_values = limit_values || {};
				limit_values[channel_name] = $limit.val();
			}
		});
		if (limit_values) {
			vcc[this.entity_name] = limit_values;
		}
		else {
			delete vcc[this.entity_name]; // Si viene undefined, lo borra y toma los por default quien lo consume (NO queda almacenado en cassandra)
		}
	};

	apps_channels_limit.prototype.set = function(vcc)
	{
		const self = this;
		if (!vcc) {
			return;
		}

		const limit_values = vcc[this.entity_name] || {};
		const $container = $("#" + this.id_section);

		let removed_channels = 0;
		$("#" + this.id_section + " input[selector='1'][type='checkbox']").each(function(_, chk)
		{
			const $checkbox = $(chk);
			let channel_name = $checkbox.attr("channel_id"); // FACEBOOK
			let license_channel_name = channel_name + "_licensed"; // FACEBOOK_licensed

			let vcc_channel_licensed = inConcert.app.hasLicenseSystem() ? vcc[license_channel_name] : vcc[license_channel_name] === "1";
			if (vcc_channel_licensed) {
				const $limit = $container.find("[ctrl][channel_id='" + channel_name + "']");
				let lower_channel_name = channel_name.toUpperCase();
				let limit = limit_values[lower_channel_name];

				if (limit) {
					$checkbox.prop("checked", true).trigger("change");
					$limit.val(limit);
				}
				else {
					self.owner.frm.disable_control($limit, true /* disabled */, false /* is_required */);
				}
			}
			else {
				const $channel = $container.find("[list_item][channel_id='" + channel_name + "']");
				$channel.remove();
				removed_channels++;
			}
		});

		const $not_found_msg = $("#" + this.id_section + "_not_results_found");
		if (removed_channels >= this.m_total_channels_config) {
			$container.hide();
			$not_found_msg.show();
		}
		else {
			$container.show();
			$not_found_msg.hide();
		}
	};

	apps_channels_limit.prototype.can_validate = function()
	{
		const $container = $("#" + this.id_section);
		const $channels_ctrls = $container.find("[ctrl][channel_id]");
		if ($channels_ctrls.length > 0) {
			return true;
		}
		return false;
	};

	apps_channels_limit.prototype.validate = function()
	{
		const self = this;
		let response = {
			success: true
		};
	
		const $container = $("#" + this.id_section);
	
		$("#" + this.id_section + " input[selector='1'][type='checkbox']").each(function(_, channel)
		{
			const $checkbox = $(channel);
			const enabled = $checkbox.prop("checked");
			if (enabled) {
				const channel_name = $checkbox.attr("channel_id").toUpperCase(); // FACEBOOK
				const channel_config = self.m_channels_config[channel_name.toUpperCase()];
				const $limit = $container.find("[ctrl][channel_id='" + channel_name + "']");
	
				if (!self.owner.validate_field($limit, "required")) {
					response.success = false;
					response.message = FORMAT(self.msg_err_required, channel_config.translated_name);
					return false; // break
				}
	
				const range = {between: {from: channel_config.minLimit, to: channel_config.maxLimit}};
				if (!self.owner.validate_field($limit, "range", "", range)) {
					response.success = false;
					response.message = FORMAT(self.msg_err_overflow, channel_config.translated_name, range.between.from, range.between.to);
					return false; // break
				}
			}
		});

		return response;
	};

	apps_channels_limit.prototype.set_events = function()
	{
		const self = this;
		const $container = $("#" + this.id_section);

		$("#" + this.id_section + " input[selector='1'][type='checkbox']")
			.off("change")
			.on("change", function() {
				const $chk = $(this);
				const channel_name = $chk.attr("channel_id").toUpperCase(); // FACEBOOK
				const $limit = $container.find("[ctrl][channel_id='" + channel_name + "']");
				if ($chk.prop("checked")) {
					self.owner.frm.disable_control($limit, false /* disabled */, true /* is_required */);
					if ($limit.val().trim() === "") {
						let lower_channel_name = channel_name.toUpperCase();
						let channel_config = self.m_channels_config[lower_channel_name];
						if (channel_config) {
							$limit.val(channel_config.default || "");
						}
					}
				}
				else {
					self.owner.frm.disable_control($limit, true /* disabled */, false /* is_required */);
				}
			});
	};

	//---------------------------------------------------------------------
	// outgoing batches channels
	function outgoing_batches_channels (params)
	{
		this.id_section       = "prov_form_vcc_basic_outgoing_batches_section";
		this.msg_err_required = "<%=_T('{0} Outgoing batch post limit is required.')%>";
		this.msg_err_overflow = "<%=_T('{0} Outgoing batch post limit value out of range. Please enter value between {1} and {2}.')%>";
		this.entity_name      = "OutgoingBatchesLimit"; // nombre que corresponde con la Columna de la CF VCCs.

		this.m_channels_config = {
			MAIL    : {minLimit: 2, maxLimit: 300, default: 50, translated_name: null},
			SMS     : {minLimit: 2, maxLimit: 300, default: 50, translated_name: null},
			WHATSAPP: {minLimit: 2, maxLimit: 300, default: 50, translated_name: null}
		};

		outgoing_batches_channels.uber.constructor.call(this, params);
	}

	inConcert.Utils.inherit(outgoing_batches_channels, apps_channels_limit);

	//---------------------------------------------------------------------
	// social networks apps channels
	function social_networks_app_channels (params)
	{
		this.id_section       = "prov_form_vcc_basic_social_networks_section";
		this.msg_err_required = "<%=_T('{0} Social network post limit is required.')%>";
		this.msg_err_overflow = "<%=_T('{0} Social network post limit value out of range. Please enter value between {1} and {2}.')%>";
		this.entity_name      = "SocialNetworkPostsLimit"; // nombre que corresponde con la Columna de la CF VCCs.
	
		this.m_channels_config = {
			FACEBOOK : {minLimit: 2, maxLimit: 300, default: 50, translated_name: null},
			INSTAGRAM: {minLimit: 2, maxLimit: 300, default: 50, translated_name: null},
			TWITTER  : {minLimit: 2, maxLimit: 300, default: 50, translated_name: null},
			YOUTUBE  : {minLimit: 2, maxLimit: 300, default: 50, translated_name: null}
		};

		social_networks_app_channels.uber.constructor.call(this, params);
	}

	inConcert.Utils.inherit(social_networks_app_channels, apps_channels_limit);

	//---------------------------------------------------------------------
	// form vcc class - basic section
	function prov_form_vccs_basic( params )
	{
		const success = prov_form_vccs_basic.uber.constructor.call( this, params );
		if (!success) {
			return;
		}

		this.view_action_id = "view_vccs"; // view action id de permisos
		this.save_action_id  = {create: "save_vcc", update: "save_vcc"}; // save action id de permisos
		this.erase_action_id = null; // delete action id de permisos
		this.m_vcc_has_users = false;
		this.m_widgets = {};
		this.telephone_codes = inConcert.app.getInstanceTelephonyCodes();
		this.countries_by_code = null;
		this.sorted_countries = null;
		this.company_is_editable = true;
		this.company_is_required = true;

		//Default values for license max limits where there is no license system. These limits apply to validation definitions
		this.MaxOnlineUsers = 0;
		this.MAX_CustomMetricsLimit = 0;
		this.MAX_CustomFiltersLimit = 0;
		this.MAX_ExportSpaceQuota = 0;
		this.MAX_SharedFilesSpaceQuota = 0;
		this.MAX_AppsIntegrations = {
			HelpDesk: 0
		};

		// outgoing batches
		this.m_outgoing_batches_channels = new outgoing_batches_channels({
			owner: this
		});
		// social networks apps
		this.m_social_networks_app_channel = new social_networks_app_channels({
			owner: this
		});

		// Instancia de la clase los TABs
		this.main_tabs = new inConcert.conf_form_tab({
			parent        : this,
			frm           : this.frm,
			container_id  : "prov_form_vcc_tabs",
			must_intialize: false,
			tabs_list   : {
				luis: {
					id              : "basic",
					name            : "<%=_T('Basic (Required)')%>",
					tab_container_id: "prov_form_vcc_basic_container",
					selected        : true
				},
				licenses: {
					id              : "licenses",
					name            : "<%=_T('Licenses')%>",
					tab_container_id: "prov_form_vcc_licenses_container"
				},
				security: {
					id              : "security",
					name            : "<%=_T('Security')%>",
					tab_container_id: "prov_form_vcc_security_container"
				},
				integrations: {
					id              : "integrations",
					name            : "<%=_T('Integrations')%>",
					tab_container_id: "prov_form_vcc_integrations_container"
				},
				super_users: {
					id              : "super_users",
					name            : "<%=_T('Super Users')%>",
					tab_container_id: "prov_form_vcc_super_users_container"
				},
				others: {
					id              : "others",
					name            : "<%=_T('Other configurations')%>",
					tab_container_id: "prov_form_vcc_others_container"
				}
			}
		});

		// super users
		this.m_grid_super_users = null;

		return true;
	}

	inConcert.Utils.inherit(prov_form_vccs_basic, inConcert.conf_form_section);

	if ( typeof inConcert.prov_form_vccs_basic !== "function" ) {
		inConcert.prov_form_vccs_basic = prov_form_vccs_basic;
	}

	/*

	*/
	let m_commonValidations = {};

	prov_form_vccs_basic.prototype.init = function()
	{
		prov_form_vccs_basic.uber.init.call(this); //invokes the base class method
		this.make_licenses_markup();
		// outgoing batches
		this.m_outgoing_batches_channels.init();
		// social networks apps
		this.m_social_networks_app_channel.init();
		//repository service
		$("#storage_config_container").toggle(inConcert.app.hasRepositoryService());
		// super users
		this.__init_grid_super_users();
	};

	prov_form_vccs_basic.prototype.dispose = function()
	{
		prov_form_vccs_basic.uber.dispose.call(this); //invokes the base class method
		this.m_vcc_has_users = false;
		this.m_oldConfigLimits = null;
	};

	prov_form_vccs_basic.prototype.fill_security_authentication_types = function()
	{
		const vcc = this.frm.get_entity_instance();
		var types = [
			{id: "BASIC", description: "<%=_T('Basic')%>"},
			{id: "MFA", description: "<%=_T('MFA')%>"}
		];
		if (!vcc || !vcc.HasAllegroIntegration) {
			types.push({id: "LDAP", description: "<%=_T('LDAP')%>"});
			types.push({id: "OPENID", description: "<%=_T('OpenId')%>"});
		}
		this.fill_combo($("#prov_form_vcc_basic_security_authentication_type"), false, types, "id", "description");
	};

	prov_form_vccs_basic.prototype.fill_security_authentication_openid_sync_methods = function(openIdEnabled)
	{
		var types = [];
		if(openIdEnabled) {
			types.push({id: "AZUREAD", description: "<%=_T('Azure AD API Graph')%>"});
		}
		this.fill_combo($("#prov_form_vcc_basic_security_authentication_openid_sync_method"), false, types, "id", "description");
	};

	/*
		La funcion 'fill_combo_app_languages' se debe de invocar a la clase BASE ya que esta definida en clase base "conf_form".
	*/
	prov_form_vccs_basic.prototype.fill_all_cbos_app_languages = function()
	{
		//LDAP
		this.fill_combo_app_languages($("#prov_form_vcc_basic_security_authentication_ldap_sync_language"), false);
		//AZURE
		this.fill_combo_app_languages($("#prov_form_vcc_basic_security_authentication_azuread_sync_language"), false);
		//SUPER USERS
		this.fill_combo_app_languages($("#prov_form_vcc_super_users_popup_app_language"), true);
	};

	prov_form_vccs_basic.prototype.fill_security_captcha_types = function()
	{
		const captcha_type_list = [
			{id: "DEFAULT", description: "<%=_T('Default')%>"},
			{id: "CUSTOM" , description: "<%=_T('Custom')%>" }
		];
		const $security_captcha_type = $("#prov_form_vcc_basic_security_captcha_type");
		const is_blank_first_item = false;
		const attr_id = "id";
		const attr_name = "description";
		this.fill_combo($security_captcha_type, is_blank_first_item, captcha_type_list, attr_id, attr_name);
	};

	prov_form_vccs_basic.prototype.fill_security_authentication_mfa_types = function()
	{
		const types = [
			{id: "MAIL", description: "<%=_T('E-mail')%>"}
		];
		this.fill_combo($("#prov_form_vcc_basic_security_authentication_mfa_type"), false, types, "id", "description");
	};

	prov_form_vccs_basic.prototype.fill_security_authentication_mfa_mail_accounts = function(accounts)
	{
		this.fill_combo($("#prov_form_vcc_basic_security_authentication_mfa_mail_account"), false, accounts, "id", "name");
	};

	prov_form_vccs_basic.prototype.fill_security_authentication_ldap_common = function()
	{
		const self = this;

		const can_validate_security_authentication_ldap_fn = function() {
			return $("#prov_form_vcc_basic_security_authentication_type").val() === "LDAP";
		};

		const on_security_error = function() {
			self.switch_tab("security");
		};

		this.ldap_common_validation_definitions = [];
		this.ldap_common_validation_definitions.push(
			{
			// server
				control_id: "prov_form_vcc_basic_security_authentication_ldap_server",
				can_validate: can_validate_security_authentication_ldap_fn,
				messages: {
					required: "<%=_T('Server IP or domain is required.')%>"
				},
				on_error: on_security_error
			},
			{
			// Admin User DN
				control_id: "prov_form_vcc_basic_security_authentication_ldap_adminUserDN",
				can_validate: can_validate_security_authentication_ldap_fn,
				messages: {
					required: "<%=_T('User RDN is required.')%>"
				},
				on_error: on_security_error
			},
			{
			// Admin User Password
				control_id: "prov_form_vcc_basic_security_authentication_ldap_password",
				can_validate: can_validate_security_authentication_ldap_fn,
				messages: {
					required: "<%=_T('Password is required.')%>"
				},
				on_error: on_security_error
			},
			{
			// Base DN
				control_id: "prov_form_vcc_basic_security_authentication_ldap_baseDN",
				can_validate: can_validate_security_authentication_ldap_fn,
				messages: {
					required: "<%=_T('Base DN is required.')%>"
				},
				on_error: on_security_error
			},
			{
			// User Id Attribute
				control_id: "prov_form_vcc_basic_security_authentication_ldap_userIdAttributte",
				can_validate: can_validate_security_authentication_ldap_fn,
				messages: {
					required: "<%=_T('User identifier attribute is required.')%>",
					custom: {
						type : "custom",
						valid: function() {
							let response = {
								success: true
							};
							const $userIdAttributte =  $("#prov_form_vcc_basic_security_authentication_ldap_userIdAttributte");
							let userIdAttributte = $userIdAttributte.val();
							let userIdAttributteRegExp = /^[a-zA-Z0-9\-]+$/;
							if(!userIdAttributte.match(userIdAttributteRegExp)) {
								response.success = false;
								response.message = "<%=_T('Only letters, numbers and mid dashes are allowed in the User identifier attribute.')%>";
							}
							return response;
						}
					}			
				},
				on_error: on_security_error
			},
		);
	};

	prov_form_vccs_basic.prototype.fill_security_authentication_azuread_common = function()
	{
		const self = this;

		const can_validate_security_authentication_azuread_fn = function() {
			return $("#prov_form_vcc_basic_security_authentication_type").val() === "OPENID" && $("#prov_form_vcc_basic_security_authentication_openid_sync_method").val() === "AZUREAD";
		};

		const on_security_error = function() {
			self.switch_tab("security");
		};

		this.azuread_common_validation_definitions = [];
		this.azuread_common_validation_definitions.push(
			{
			// url
				control_id: "prov_form_vcc_basic_security_authentication_openid_configuration_url",
				can_validate: can_validate_security_authentication_azuread_fn,
				messages: {
					required: "<%=_T('OpenId configuration URL is required.')%>",
					url: "<%=_T('OpenId configuration URL must be a valid URL.')%>"
				},
				on_error: on_security_error
			},
			{
			// Client Id
				control_id: "prov_form_vcc_basic_security_authentication_azuread_sync_client_id",
				can_validate: can_validate_security_authentication_azuread_fn,
				messages: {
					required: "<%=_T('Client Id is required.')%>"
				},
				on_error: on_security_error
			},
			{
			// Client secret
				control_id: "prov_form_vcc_basic_security_authentication_azuread_sync_client_secret",
				can_validate: can_validate_security_authentication_azuread_fn,
				messages: {
					required: "<%=_T('Client secret is required.')%>"
				},
				on_error: on_security_error
			},
		);
	};	

	prov_form_vccs_basic.prototype.fill_security_authentication_ldap_groups = function(groups)
	{
		const self = this;

		const $ldap_panel = $("#prov_form_vcc_basic_security_authentication_ldap_panel");

		/*
		*/
		const on_click_fn = function() {
			let selectedFilterId = $(this).attr("id");
			self.validate(function( success ) {
				if (!success) {
					return;
				}
				const $password =  $("#prov_form_vcc_basic_security_authentication_ldap_password");
				const jwtToken  = inConcert.Utils.GetJwtToken($password.val());
				var data = {
					vcc: $("#prov_form_vcc_basic_name").val(),
					server : $("#prov_form_vcc_basic_security_authentication_ldap_server").val(),
					baseDN : $("#prov_form_vcc_basic_security_authentication_ldap_baseDN").val(),
					adminUserDN : $("#prov_form_vcc_basic_security_authentication_ldap_adminUserDN").val(),
					password : jwtToken,
					useTLS : $("#prov_form_vcc_basic_security_authentication_ldap_useTLS").prop("checked")
				};
				var validation_def = {
					control_id: "prov_form_vcc_basic_security_authentication_ldap_server",
					can_validate: true,
					messages: {
						custom: {
							has_error: false,
							error_message: "",
							type : "custom",
							valid: function() {
								let response = {
									success: true
								};
								if(this.has_error) {
									response.success = false;
									response.message = this.error_message;
								}
								return response;
							}
						}
					}
				};
				var definitions = [];
				definitions.push(validation_def);

				inConcert.app.getProvisioningAPI().getLdapGroups(data, function(success, response) {
					if(!success) {
						validation_def.messages.custom.has_error = true;
						validation_def.messages.custom.error_message = response;
						self.validate(function(_) {}, definitions);
						console.log(response);
						return;
					}
					if(!response.success) {
						validation_def.messages.custom.has_error = true;
						validation_def.messages.custom.error_message = response.error;
						self.validate(function(_) {}, definitions);
						console.log(response.error);
						return;
					}
					if(!response.result || response.result.length === 0) {
						validation_def.messages.custom.has_error = true;
						validation_def.messages.custom.error_message =  "<%=_T('No results returned')%>";
						self.validate(function(_) {}, definitions);
						return;
					}

					
					// response.result es un objeto cuyas propiedades son las entradas de ldap, desordenadas y a un mismo nivel
					// La clase GenericTreePopup espera los dispositionCodes como un array, en el que cada elemento tendría un campo string "name" y un campo array "childs"
					var resultAsDispositionCodesFormat = [];
					// Obtiene un array anidado, en forma de arbol, con toda la descendencia de una entrada de ldap
					let getChilds;
					getChilds = function(nodeLevels) {
						if(nodeLevels.length > 0) {
							let node = {
								name: nodeLevels[nodeLevels.length - 1]
							};
							nodeLevels.pop();
							let childs = getChilds(nodeLevels);
							if(childs) {
								node.childs = [];
								node.childs.push(childs);
							}
							return node;
						}
					};
					//Inserta el primer nodo, junto con sus hijos, que no exista ya en arr
					let insertFirstNonExistingNode;
					insertFirstNonExistingNode = function(arr, node) {
						let indexFound = arr.findIndex(e => e.name === node.name);
						if(indexFound >= 0) {
							if(arr[indexFound].childs && node.childs && node.childs[0]) {
								insertFirstNonExistingNode(arr[indexFound].childs, node.childs[0]);
							}
						}
						else {
							arr.push(node);
						}
					};

					// Hacer el reverse de cada propiedad de response.result para poder ordenar 
					let reversedResult = {};
					for (const property in response.result) {
						if (response.result.hasOwnProperty(property)) {
							reversedResult[property.split("").reverse().join("")] = response.result[property];
						}
					}

					// Obtenemos un objeto con sus properties ordenadas descendente, para que aparezcan los descendientesde nivel más profundo antes que su progenitor
					let sortedResult = {};
					Object.keys(reversedResult).sort(function (a, b) {
						if (a < b) {
							return 1;
						}
						if (a > b) {
							return -1;
						}
						// a must be equal to b
						return 0;
					}).map(i=>sortedResult[i]=response.result[i]);

					for (const itemReversed in sortedResult) {
						if (sortedResult.hasOwnProperty(itemReversed)) {
							//Deshacemos el reverse una vez ordenado el resultado
							let item = itemReversed.split("").reverse().join("");
							// Extraer del item el numero de separadores (n) de path de ldap tiene (,). El node debe ir en el subnivel n del array
							let nodeLevels = item.split(",");
							let node = {
								name: nodeLevels[nodeLevels.length - 1]
							};
							nodeLevels.pop();
							let childs = getChilds(nodeLevels);
							if(childs) {
								node.childs = [];
								node.childs.push(childs);
							}

							insertFirstNonExistingNode(resultAsDispositionCodesFormat, node);
						}
					}

					const ldapTreePopup = new inConcert.GenericTreePopup();

					ldapTreePopup.show({
						title: "<%=_T('Select a LDAP level')%>",
						parentNode:       $("#" + selectedFilterId),
						dispositionCodes: resultAsDispositionCodesFormat,
						permissionToComment: false,
						callbacks: {
							on_finish : function( response ) {
								if (response && response.disposition && response.disposition.treePath && Array.isArray(response.disposition.treePath)) {
									// Insertar en el filter correspondiente lo seleccionado en el arbol
									let selectedFilterValue = "";
									response.disposition.treePath.slice().reverse().forEach(element => selectedFilterValue+= (selectedFilterValue.length ? "," : "") + element);
									// No permitir filtros duplicados
									const ldap_filter_validation_definition = {
										control_id: selectedFilterId,
										can_validate: true,
										messages: {
											custom: {
												valid: function() {
													const response = {
														success: true
													};
													$ldap_panel.find("[LDAP_ROW_GROUP_NAME]").each(function(_, item) {
														const $item = $(item);
														const $ldapGroupFilter = $item.find("[ldap_group_filter]");
														const filter = $ldapGroupFilter.val();
														const filterId = $ldapGroupFilter.attr("id");
														if (filterId !== selectedFilterId && filter === selectedFilterValue) {
															response.success = false;
															response.message = "<%=_T('A filter can only be used once')%>";	
															return response;
														}
													});
													return response;
												}
											}
										}
									};

									const ldap_filter_validation_definitions = [];
									ldap_filter_validation_definitions.push(ldap_filter_validation_definition);

									self.validate(function( success ) {
										if(success) {
											$("#" + selectedFilterId).val(selectedFilterValue);
										}
									}, ldap_filter_validation_definitions);
								}
							},
							onOpen: $.noop,
							onClose: $.noop
						}
					});
				});
			}, self.ldap_common_validation_definitions);
		};

		for (const group in groups) {
			if (groups.hasOwnProperty(group)) {
				var groupId = "prov_form_vcc_basic_security_authentication_ldap_group_" + group;
	
				var labelGroup = document.createElement("label");
				labelGroup.setAttribute("for", groupId);
				labelGroup.innerHTML = "<%=_T('Group')%>";
	
				var inputGroup = document.createElement("input");
				inputGroup.classList.add("form-control");
				inputGroup.setAttribute("id", groupId);
				inputGroup.setAttribute("type", "text");
				inputGroup.setAttribute("disabled", "true");
				inputGroup.setAttribute("LDAP_GROUP_NAME", group);
				inputGroup.value = group;
	
				var formGroup = document.createElement("div");
				formGroup.classList.add("form-group");
				formGroup.classList.add("customEditable");
				formGroup.appendChild(labelGroup);
				formGroup.appendChild(inputGroup);
	
				var colGroup = document.createElement("div");
				colGroup.classList.add("col-3");
				colGroup.appendChild(formGroup);
	
				var labelFilter = document.createElement("label");
				labelFilter.setAttribute("for", groupId + "_filter");
				labelFilter.innerHTML = "<%=_T('Filter')%>";
	
				var filterId = groupId + "_filter"; 
				var inputFilter = document.createElement("input");
				inputFilter.classList.add("form-control");
				inputFilter.setAttribute("id", filterId);
				inputFilter.setAttribute("type", "text");
				//inputFilter.setAttribute("disabled", "true");
				inputFilter.setAttribute("LDAP_GROUP_FILTER", group);
	
				var validationFilter = document.createElement("i");
				validationFilter.classList.add("formFieldValidation");
				validationFilter.classList.add("formFieldError");
				validationFilter.setAttribute("id", filterId + "_validation");
	
				var formFilter = document.createElement("div");
				formFilter.classList.add("form-group");
				//formFilter.classList.add("customEditable");
				formFilter.appendChild(labelFilter);
				formFilter.appendChild(inputFilter);
				formFilter.appendChild(validationFilter);
	
				var colFilter = document.createElement("div");
				colFilter.classList.add("col-6");
				colFilter.appendChild(formFilter);

				inputFilter.onclick = on_click_fn;
	
				var row = document.createElement("div");
				row.classList.add("row");
				row.setAttribute("LDAP_ROW_GROUP_NAME", group);
				row.appendChild(colGroup);
				row.appendChild(colFilter);
	
				let $ldap_sync_common = document.getElementById("prov_form_vcc_basic_security_authentication_ldap_sync_common");
				document.getElementById("prov_form_vcc_basic_security_authentication_ldap_sync_panel").insertBefore(row, $ldap_sync_common);
			}
		}
	};

	prov_form_vccs_basic.prototype.fill_security_authentication_azuread_groups = function(groups)
	{
		for (const group in groups) {
			if (groups.hasOwnProperty(group) && group !== "allusers") {
				var groupId = "prov_form_vcc_basic_security_authentication_azuread_group_" + group;
	
				var labelGroup = document.createElement("label");
				labelGroup.setAttribute("for", groupId);
				labelGroup.innerHTML = "<%=_T('OCC Group')%><%=_T(':')%>";
	
				var inputGroup = document.createElement("input");
				inputGroup.classList.add("form-control");
				inputGroup.setAttribute("id", groupId);
				inputGroup.setAttribute("type", "text");
				inputGroup.setAttribute("disabled", "true");
				inputGroup.setAttribute("AZUREAD_GROUP_NAME", group);
				inputGroup.value = group;
	
				var formGroup = document.createElement("div");
				formGroup.classList.add("form-group");
				formGroup.classList.add("customEditable");
				formGroup.appendChild(labelGroup);
				formGroup.appendChild(inputGroup);
	
				var colGroup = document.createElement("div");
				colGroup.classList.add("col-3");
				colGroup.appendChild(formGroup);
	
				var labelFilter = document.createElement("label");
				labelFilter.setAttribute("for", groupId + "_filter");
				labelFilter.innerHTML = "<%=_T('Azure AD Group')%><%=_T(':')%>";
	
				var filterId = groupId + "_filter"; 
				var inputFilter = document.createElement("select");
				inputFilter.classList.add("form-control");
				inputFilter.setAttribute("id", filterId);
				inputFilter.setAttribute("AZUREAD_GROUP_FILTER", group);
	
				var validationFilter = document.createElement("i");
				validationFilter.classList.add("formFieldValidation");
				validationFilter.classList.add("formFieldError");
				validationFilter.setAttribute("id", filterId + "_validation");
	
				var formFilter = document.createElement("div");
				formFilter.classList.add("form-group");
				formFilter.appendChild(labelFilter);
				formFilter.appendChild(inputFilter);
				formFilter.appendChild(validationFilter);
	
				var colFilter = document.createElement("div");
				colFilter.classList.add("col-3");
				colFilter.appendChild(formFilter);
	
				var row = document.createElement("div");
				row.classList.add("row");
				row.setAttribute("AZUREAD_ROW_GROUP_NAME", group);
				row.appendChild(colGroup);
				row.appendChild(colFilter);
	
				let $openid_sync_common = document.getElementById("prov_form_vcc_basic_security_authentication_azuread_sync_common");
				document.getElementById("prov_form_vcc_basic_security_authentication_azuread_sync_panel").insertBefore(row, $openid_sync_common);
			}
		}
	};
	
	prov_form_vccs_basic.prototype.fill_security_authentication_ldap_sync_areas = function( country_code ){
		this.fill_combo_areas(
			country_code,
			$("#prov_form_vcc_basic_security_authentication_ldap_sync_area"),
			false
		);
	};
	
	prov_form_vccs_basic.prototype.fill_security_authentication_azuread_sync_areas = function( country_code ){
		this.fill_combo_areas(
			country_code,
			$("#prov_form_vcc_basic_security_authentication_azuread_sync_area"),
			false
		);
	};

	prov_form_vccs_basic.prototype.fill_security_authentication_ldap_sync_campaigns = function(campaigns)
	{
		var $combo = $("#prov_form_vcc_basic_security_authentication_ldap_sync_default_campaign");
		this.fill_combo($combo, false, campaigns, "id", "description");
	};

	prov_form_vccs_basic.prototype.fill_security_authentication_azuread_sync_campaigns = function(campaigns)
	{
		var $combo = $("#prov_form_vcc_basic_security_authentication_azuread_sync_default_campaign");
		this.fill_combo($combo, false, campaigns, "id", "description");
	};

	prov_form_vccs_basic.prototype.fill_security_authentication_ldap_sync_states = function()
	{
		var $combo = $("#prov_form_vcc_basic_security_authentication_ldap_sync_initial_state");
		this.fill_combo($combo, false, inConcert.app.getAvailableStates(), "Id", "Description");
	};

	prov_form_vccs_basic.prototype.fill_security_authentication_azuread_sync_states = function()
	{
		var $combo = $("#prov_form_vcc_basic_security_authentication_azuread_sync_initial_state");
		this.fill_combo($combo, false, inConcert.app.getAvailableStates(), "Id", "Description");
	};

	prov_form_vccs_basic.prototype.fill_accounts = function(accounts = {})
	{
		this.fill_security_authentication_mfa_mail_accounts(accounts.MAIL || []);
	};

	prov_form_vccs_basic.prototype.fill_mail_controls = function()
	{
		this.m_mailControls = {
			mfa : new inConcert.MailManagement({
				owner: this,
				auto_init: true,
				ids: {
					body     : "prov_form_vcc_basic_security_authentication_mfa_mail_body",
					container: "prov_form_vcc_basic_security_authentication_mfa_mail_body_container"
				},
				callbacks: {
					loadVariables: function(sorted, callback = $.noop) {
						let variables = [];
						// {CODE}, {VCC}, {AGENT}, {AGENTNAME}, {AGENT_COMPLETE_NAME}
						variables.push({name: "<%=_T('User Id')%>"           , id: "{AGENT}"});
						variables.push({name: "<%=_T('User Name')%>"         , id: "{AGENTNAME}"});
						variables.push({name: "<%=_T('User Complete Name')%>", id: "{AGENT_COMPLETE_NAME}"});
						variables.push({name: "<%=_T('Code')%>"              , id: "{CODE}"});
						variables.push({name: "<%=_T('Contact Center')%>"    , id: "{VCC}"});
						if (sorted) {
							variables = inConcert.Utils.sortArray(variables, "name", "asc");
						}
						callback(true, variables);
					}
				}
			})
		};
		//default values
		$("#prov_form_vcc_basic_security_authentication_mfa_mail_subject").val("<%=_T('Omnichannel login code')%>");
		this.m_mailControls.mfa.set("<%=_T('Enter the code {CODE} to login.')%>");
	};
	
	prov_form_vccs_basic.prototype.fill_ldap_common_controls = function()
	{
		this.fill_security_authentication_ldap_common();
		this.fill_security_authentication_ldap_sync_states();
	};
	
	prov_form_vccs_basic.prototype.fill_azuread_common_controls = function()
	{
		this.fill_security_authentication_azuread_common();
		this.fill_security_authentication_azuread_sync_states();
	};
	
	prov_form_vccs_basic.prototype.fill_groups_grid = function(groups = {})
	{
		this.fill_security_authentication_ldap_groups(groups);
		this.fill_security_authentication_azuread_groups(groups);
	};

	/*
		La funcion 'fill_timezones' se debe de invocar a la clase BASE ya que esta definida en clase base "conf_form".
	*/
	prov_form_vccs_basic.prototype.fill_all_ctrls_timezones = function(timezones = {})
	{
		// LDAP
		this.fill_timezones(timezones, $("#prov_form_vcc_basic_security_authentication_ldap_sync_timezone"));
		// AZURED
		this.fill_timezones(timezones, $("#prov_form_vcc_basic_security_authentication_azuread_sync_timezone"));
		// SUPER USER
		this.fill_timezones(timezones, $("#prov_form_vcc_super_users_popup_timezone"));
	};

	prov_form_vccs_basic.prototype.fill_campaigns = function(campaigns = {})
	{
		//Remove internal and not assignable campaigns as default campaign for a user
		delete campaigns.internal_calls;
		delete campaigns.sandbox;
		this.fill_security_authentication_ldap_sync_campaigns(campaigns);
		this.fill_security_authentication_azuread_sync_campaigns(campaigns);
	};

	prov_form_vccs_basic.prototype.fill_states = function()
	{
		this.fill_security_authentication_ldap_sync_states();
	};

	prov_form_vccs_basic.prototype.fill_all_ctrls_coutries = function(countries)
	{
		// LDAP
		this.fill_countries( countries, $("#prov_form_vcc_basic_security_authentication_ldap_sync_country") );
		// AZURE
		this.fill_countries( countries, $("#prov_form_vcc_basic_security_authentication_azuread_sync_country") );
		// SUPER USERS
		this.fill_countries( countries, $("#prov_form_vcc_super_users_popup_country") );
	};

	prov_form_vccs_basic.prototype.fill_recording_configuration = function()
	{
		var criteria = [
			{id: "all", description: "<%=_T('All')%>"},
			{id: "none", description: "<%=_T('None')%>"}
		];
		this.fill_combo($("#prov_form_vcc_basic_recording_call_automatic_recording_criteria"), false, criteria, "id", "description");
		this.fill_combo($("#prov_form_vcc_basic_recording_videocall_automatic_recording_criteria"), false, criteria, "id", "description");
		this.fill_combo($("#prov_form_vcc_basic_recording_voicemail_automatic_recording_criteria"), false, criteria, "id", "description");

		var call_formats = [
			{id: "mp3", description: "<%=_T('mp3')%>"},
			{id: "wav", description: "<%=_T('wav')%>"}
		];
		this.fill_combo($("#prov_form_vcc_basic_recording_call_format"), false, call_formats, "id", "description");
		this.fill_combo($("#prov_form_vcc_basic_recording_voicemail_format"), false, call_formats, "id", "description");

		var video_formats = [
			{id: "webm", description: "<%=_T('webm')%>"},
			{id: "mp4", description: "<%=_T('mp4')%>"}
		];
		this.fill_combo($("#prov_form_vcc_basic_recording_videocall_format"), false, video_formats, "id", "description");

		var channels = [
			{id: "mono", description: "<%=_T('mono')%>"},
			{id: "stereo", description: "<%=_T('stereo')%>"}
		];
		this.fill_combo($("#prov_form_vcc_basic_recording_call_channels"), false, channels, "id", "description");
		this.fill_combo($("#prov_form_vcc_basic_recording_voicemail_channels"), false, channels, "id", "description");

		var modes = [
			{id: "video_only", description: "<%=_T('Video only')%>"},
			{id: "audio_only", description: "<%=_T('Audio only')%>"},
			{id: "all", description: "<%=_T('Video and Audio')%>"}
		];
		this.fill_combo($("#prov_form_vcc_basic_recording_videocall_mode"), false, modes, "id", "description");
	};

	prov_form_vccs_basic.prototype.attach_load_request = function(request)
	{
		const self = this;

		if (inConcert.app.hasRepositoryService()) {
			this.fill_storage_locations();
		}
		this.fill_security_captcha_types();
		this.fill_security_authentication_types();
		this.fill_security_authentication_mfa_types();
		this.fill_all_cbos_app_languages();
		this.fill_mail_controls();
		this.fill_ldap_common_controls();
		this.fill_azuread_common_controls();
		this.fill_recording_configuration();
		if (!$.isArray(request)) {
			return;
		}
		const provisioning_api = inConcert.app.getProvisioningAPI();
		request.push(
			{
				request_function: function(success, vcc_id) {
					return provisioning_api.vccAccountsLoadAll(vcc_id, success);
				},
				success: function(response) {
					self.fill_accounts(response);
				}
			},
			{
				request_function: function(success, vcc_id) {
					return provisioning_api.vccGroupsLoadAll(vcc_id, success);
				},
				success: function(response) {
					self.fill_groups_grid(response);
				}
			},
			{
				request_function: function(success) {
					return inConcert.app.loadTimezones(success);
				},
				success: function(response) {
					self.fill_all_ctrls_timezones(response);
				}
			},
			{
				request_function: function(success) {
					return self.telephone_codes.loadCountries(success);
				},
				success: function(countries) {
					self.fill_all_ctrls_coutries(countries);
				}
			},
			{
				request_function: function(success, vcc_id) {
					return provisioning_api.vccCampaignsLoadAll(vcc_id, success);
				},
				success: function(response) {
					self.fill_campaigns(response);
				}
			},
			{
				request_function: function(success) {
					return provisioning_api.vccLoadDefaultsRecordingConfiguration(success);
				},
				success: function(response) {
					self.m_defaultsRecordingConfiguration = response;
				}
			},
			{
				request_function: function(success) {
					// Se obtienen los rangos disponibles que se pueden configurar como limites de configuracions del VCC, teniendo encuenta los limites maximos del ambiente
					return provisioning_api.getMaxNumberConfigsAvailable(success);
				},
				success: function(response) {
					if (response && response.result === "OK") {
						self.m_maxNumberConfigsAvailable = response.configLimits;
						self.set_config_limits_titles();
					}
					else {
						self.m_maxNumberConfigsAvailable = response.errType;
						self.show_popup_message({
							messages: {
								title  : "<%=_T('Error getting environment configurations limits.')%>",
								message: "",
								type   : "warn"
							}
						});
					}
				}
			}
		);

		if (!this.frm.is_new) {
			request.push({
				request_function: function(success, vcc_id) {
					if (!vcc_id) return;
					return provisioning_api.superUsersSearch({vcc: vcc_id}, success);
				},
				success: function(response) {
					self.fill_super_users(response);
				}
			});
		}

		if (inConcert.app.hasLicenseSystem()) {
			request.push(
				{
					request_function: function(success) {
						return provisioning_api.LoadAvailableLicenseResources(success);
					},
					success: function(response) {
						self.__set_total_max_license_resources(response);
						self.load_licensed_resources(response);
					}
				},
				{
					request_function: function(success) {
						return provisioning_api.CompanyIsEditable(success);
					},
					success: function(response) {
						self.company_is_editable = response.company_is_editable;
					}
				},
				{
					request_function: function(success) {
						return provisioning_api.CompanyIsRequired(success);
					},
					success: function(response) {
						self.company_is_required = response.company_is_required;
					}
				}
			);
		}
		if (inConcert.app.hasRepositoryService()) {
			request.push(
				{
					request_function: function(success) {
						return provisioning_api.loadS3Regions(success);
					},
					success: function(countries) {
						self.fill_S3Regions(countries);
					}
				}
			);
		}
		// cargo la informacion de los widgets, que se obtuvieron al realizar el loggin del usuario "system"
		this.load_widgets(inConcert.app.get_widgets_integrations_info());
	};

	prov_form_vccs_basic.prototype.make_licenses_markup = function()
	{
		const $channelsSpans = $("#prov_form_vcc_licenses_container").find(".fa, .fa-question-circle");
		$channelsSpans.each(function() {
			if(!inConcert.app.hasLicenseSystem()) {
				$( this ).prop("title", "<%=_T('This information must be updated from the command line.')%>");
			}
		});
		const $helpdeskSpan = $("#prov_form_vcc_basic_helpdesk_licenses_section").find(".fa, .fa-question-circle");
		if(!inConcert.app.hasLicenseSystem()) {
			$helpdeskSpan.prop("title", "<%=_T('This information must be updated from the command line.')%>");
		}
		const $recordingConfigurationSpan = $("#prov_form_vcc_basic_recording_panel").find(".fa, .fa-question-circle");
		if(!inConcert.app.hasLicenseSystem()) {
			$recordingConfigurationSpan.prop("title", "<%=_T('This information must be updated from the command line.')%>");
		}

		const $licences = $("#prov_form_vcc_basic_licences_section");
		$licences.html("");

		let is_readonly = inConcert.app.hasLicenseSystem() ? this.is_readonly() : true;

		const readonly_class = is_readonly ? "readonly" : "";
		const readonly_prop = is_readonly ? " disabled='true' " : "";
		const all_channels = inConcert.app.load_vcc_channels();

		const must_include_channel = function(channel_id) {
			channel_id = (channel_id || "").toUpperCase();
			if (channel_id === "RCS") {
				return false;
			}
			return true;
		};

		for (let channel_id in all_channels) {
			if (all_channels.hasOwnProperty(channel_id)) {
				if (!must_include_channel(channel_id)) {
					continue;
				}
				let channel = all_channels[channel_id];
				let name = channel.name;
				let channel_license_id = "prov_form_vcc_basic_licences_section_opt_channel_license_" + channel_id;
				let class_name = channel_id;
				let $license = $(
					"<div class='col-3 " + class_name + "' channel_id='" + channel_id + "' list_item='true'>" +
						"<div class='form-group customEditable'>" +
							"<div class='form-check mt-2 mb-2'>" +
								"<label class='form-check-label form-check-label-title " + readonly_class + "' for='" + channel_license_id + "'>" + 
									"<input class='form-check-input " + readonly_class + "' id='"  + channel_license_id + "' channel_id='" + channel_id + "' selector='1' type='checkbox' " + readonly_prop + " />" +
									"<span>" + name + "</span>" +
								"</label>" +
							"</div>" +
						"</div>" +
						"<i error='true' class='formFieldValidation formFieldError' hidden ></i>" +
					"</div>");
				// agrego la licencia al markup
				$licences.append($license);
			}
		}
	};

	prov_form_vccs_basic.prototype.get = function(vcc = {})
	{
		if (this.frm.is_new) {
			vcc.Id   = $("#prov_form_vcc_basic_name" ).val();
		}
		vcc.Description                 = $("#prov_form_vcc_basic_desc").val();
		vcc.OnlineUsers                 = $("#prov_form_vcc_basic_online_users").val();
		vcc.company                     = $("#prov_form_vcc_basic_company").val();
		vcc.CompanyAddress              = $("#prov_form_vcc_basic_company_address").val();
		vcc.Telephone1                  = $("#prov_form_vcc_basic_tel1").val();
		vcc.Telephone2                  = $("#prov_form_vcc_basic_tel2").val();
		vcc.ContactName                 = $("#prov_form_vcc_basic_contact_name").val();
		vcc.ContactEmail                = $("#prov_form_vcc_basic_contact_email").val();
		vcc.FreeSecondsAfterInteraction = $("#prov_form_vcc_basic_free_seconds_after_interaction").val();
		vcc.CustomMetricsLimit          = $("#prov_form_vcc_basic_custom_metrics_limit").val();
		vcc.CustomFiltersLimit          = $("#prov_form_vcc_basic_custom_filters_limit").val();
		vcc.ExportSpaceQuota            = $("#prov_form_vcc_basic_export_space_quota").val();
		vcc.SharedFilesSpaceQuota       = $("#prov_form_vcc_basic_shared_files_space_quota").val();

		vcc.StorageLifecycle = {
			S3Files         : $("#prov_form_vcc_basic_storage_s3_files_lifecycle").val(),
			S3MetaData      : $("#prov_form_vcc_basic_storage_s3_meta_lifecycle").val(),
			RepositoryCall  : $("#prov_form_vcc_basic_storage_call_files_lifecycle").val(),
			RepositoryMail  : $("#prov_form_vcc_basic_storage_email_files_lifecycle").val(),
			RepositoryOther : $("#prov_form_vcc_basic_storage_other_files_lifecycle").val()
		};

		if (inConcert.app.hasRepositoryService())
		{
			vcc.StorageConfig = {
				StorageLocation: $("#prov_form_vcc_basic_storage_location").val(),
				StorageLocationParams: {}
			};

			if (vcc.StorageConfig.StorageLocation === "S3") {
				vcc.StorageConfig.StorageLocationParams.S3PubKey = $("#prov_form_vcc_basic_storage_location_params_S3PubKey").val();
				vcc.StorageConfig.StorageLocationParams.S3PrivKey = $("#prov_form_vcc_basic_storage_location_params_S3PrivKey").val();
				vcc.StorageConfig.StorageLocationParams.S3Region = $("#prov_form_vcc_basic_storage_location_params_S3Region").val();
				
				// User predefined bucket
				if ($("#prov_form_vcc_basic_storage_location_params_S3BucketCheckBox").prop("checked")) {
					vcc.StorageConfig.StorageLocationParams.S3Bucket = $("#prov_form_vcc_basic_storage_location_params_S3Bucket").val();
					vcc.StorageConfig.StorageLocationParams.IsPredBucket = true;
				}

				// Prefix config
				vcc.InteractionPrefixConfig.TreeString = $("#prov_form_vcc_basic_prefix_config_tree_structure").val();
			}
		}

		this.get_session_timeouts(vcc);
		this.get_security_captcha(vcc);
		this.get_security_authentication(vcc);
		this.get_integrations(vcc);
		this.get_licenses(vcc);
		//screen recording
		this.get_screen_recording(vcc);
		//Video Configuration (webcall)
		this.get_video_config(vcc);
		//Audit Configuration
		this.get_audit_config(vcc);
		// outgoing batches
		this.m_outgoing_batches_channels.get(vcc);
		// social networks apps
		this.m_social_networks_app_channel.get(vcc);
		// user
		this.get_user(vcc);
		// API SDK
		this.get_api_sdk_config(vcc);
		// Databases
		this.get_databases_config(vcc);
		// Telephony native
		this.get_telephony_config(vcc);
		//drafts
		this.get_drafts_config(vcc);
		//drafts
		this.get_phone_validation_config(vcc);
		//Recording Configuration
		if(inConcert.app.hasLicenseSystem()) {
			this.get_recording_configuration(vcc);		
		}
		//Configurations limits
		this.get_configuration_limits(vcc);
		return vcc;
	};

	prov_form_vccs_basic.prototype.get_user = function(vcc)
	{
		const $password =  $("#prov_form_vcc_basic_user_password");
	 	if (this.frm.is_new || !this.m_vcc_has_users || $password.val() !== this.m_original_user_password) {
			const $user = $("#prov_form_vcc_basic_user_id");
			const jwtToken  = inConcert.Utils.GetJwtToken($password.val());
			vcc.__userInfo = {
				userId: $user.val(),
				jwtToken: jwtToken
			};
			if(!this.frm.is_new && $password.val() !== this.m_original_user_password) {
				vcc.__userInfo.Password = inConcert.Utils.md5( vcc.__userInfo.userId.toLowerCase() + $password.val() );
			}
			else {
				vcc.__userInfo.plainPassword = $password.val();
			}
			if (vcc.AppsIntegrations && vcc.AppsIntegrations.HelpDesk && vcc.AppsIntegrations.HelpDesk.Enabled === "1") {
				const $email = $("#prov_form_vcc_basic_user_email");
				vcc.__userInfo.email = $email.val().trim();
			}
	 	}
	};

	prov_form_vccs_basic.prototype.get_telephony_config = function(vcc)
	{
		if (!vcc) {
			return;
		}
		
		if (vcc.HasAllegroIntegration) {
			return;
		}
		
		const DEFAULT_DIAL_TIME_VALUE = 60;
		const DEFAULT_MAX_MANUAL_RATE = 8;

		if (!$("#prov_form_vcc_basic_telephony_dial_time").val()) {
			$("#prov_form_vcc_basic_telephony_dial_time").val(DEFAULT_DIAL_TIME_VALUE);
		}

		if (!$("#prov_form_vcc_basic_telephony_max_manual_rate").val()) {
			$("#prov_form_vcc_basic_telephony_max_manual_rate").val(DEFAULT_MAX_MANUAL_RATE);
		}

		vcc.TelephonyConfig = {
			DialTimeouts: {
				timeoutSec: $("#prov_form_vcc_basic_telephony_dial_time").val()
			},
			ProcessMaxManualRate: $("#prov_form_vcc_basic_telephony_max_manual_rate").val()
		};
	};

	prov_form_vccs_basic.prototype.get_session_timeouts = function(vcc)
	{
		if (!vcc) {
			return;
		}
		const $mobile = $("#prov_form_vcc_basic_session_timeouts_mobile");
		const default_session_timeouts = {
			MOBILE: parseInt($mobile.attr("default_value"), 0)
		};
		const $session_timeouts_enabled = $("#prov_form_vcc_basic_session_timeouts_enabled");
		if (!$session_timeouts_enabled.prop("checked")) {
			vcc.SessionTimeouts = default_session_timeouts;
			return;
		}
		vcc.SessionTimeouts.MOBILE = $mobile.val();
	};

	prov_form_vccs_basic.prototype.get_screen_recording = function(vcc)
	{
		if (!vcc) {
			return;
		}
		const $port = $("#prov_form_vcc_basic_screen_recording_port");
		const $fps = $("#prov_form_vcc_basic_screen_recording_fps");
		const $bitrate = $("#prov_form_vcc_basic_screen_recording_bitrate");
		const default_screen_recording = {
			Enabled: false,
			PORT: parseInt($port.attr("default_value"), 0),
			FPS: parseInt($fps.attr("default_value"), 0),
			BITRATE: parseInt($bitrate.attr("default_value"), 0)
		};
		const $screen_recording_enabled = $("#prov_form_vcc_basic_screen_recording_enabled");
		if (!$screen_recording_enabled.prop("checked")) {
			vcc.ScreenRecording = default_screen_recording;
			return;
		}
		if (!vcc.ScreenRecording) {
			vcc.ScreenRecording={};
		}
		vcc.ScreenRecording.Enabled = true;
		vcc.ScreenRecording.PORT = $port.val();
		vcc.ScreenRecording.FPS = $fps.val();
		vcc.ScreenRecording.BITRATE = $bitrate.val();
	};

	prov_form_vccs_basic.prototype.get_video_config = function(vcc)
	{
		if (!vcc) {
			return;
		}
		const default_video_config = {
			Enabled: false,
			BITRATE: 64000
		};
		const $bitrate = $("#prov_form_vcc_basic_video_bitrate");
		const $video_config_enabled = $("#prov_form_vcc_basic_video_config_enabled");
		if (!$video_config_enabled.prop("checked")) {
			vcc.VideoConfig = default_video_config;
			return;
		}
		if (!vcc.VideoConfig) {
			vcc.VideoConfig={};
		}
		vcc.VideoConfig.Enabled = true;
		vcc.VideoConfig.BITRATE = $bitrate.val();
	};

	prov_form_vccs_basic.prototype.get_drafts_config = function(vcc)
	{
		if (!vcc) {
			return;
		}
		const default_drafts_config = {
			Enabled: false,
			MaxDrafts: 10,
			MaxSizeDrafts: 1024
		};
		const $maxdrafts = $("#prov_form_vcc_basic_drafts_configuration_number");
		const $maxdSizeDrafts = $("#prov_form_vcc_basic_drafts_configuration_max_size");
		const $config_enabled = $("#prov_form_vcc_basic_drafts_configuration_enabled");
		if (!$config_enabled.prop("checked")) {
			vcc.DraftsConfig = default_drafts_config;
			return;
		}
		if (!vcc.DraftsConfig) {
			vcc.DraftsConfig={};
		}
		vcc.DraftsConfig.Enabled = true;
		vcc.DraftsConfig.MaxDrafts = $maxdrafts.val();
		vcc.DraftsConfig.MaxSizeDrafts = $maxdSizeDrafts.val();
	};

	prov_form_vccs_basic.prototype.get_phone_validation_config = function(vcc)
	{
		if (!vcc) return;

		const $format = $("#prov_form_vcc_basic_phone_validation_configuration_format");
		const $enabled = $("#prov_form_vcc_basic_phone_validation_configuration_enabled");
		vcc.PhoneValidation = { Enabled: $enabled.prop("checked") };
		if (!vcc.PhoneValidation.Enabled) return;

		vcc.PhoneValidation.Format = $format.val();
	};

	prov_form_vccs_basic.prototype.get_databases_config = function(vcc)
	{
		if (!vcc) {
			return;
		}
		
		if (!vcc.Databases) {
			vcc.Databases={};
		}
		const $enable = $("#prov_form_vcc_basic_databases_configuration_panel_enabled");
		if ($enable.prop("checked")) {
			const $limit_number = $("#prov_form_vcc_basic_databases_number");
			const $sizing = $("#prov_form_vcc_basic_databases_sizing");
			const $rows_number = $("#prov_form_vcc_basic_databases_rows_number");
			const $timestamp = $("#prov_form_vcc_basic_databases_timestamp");
	
			vcc.Databases.LimitNumber = $limit_number.val();
			vcc.Databases.Sizing = $sizing.val();
			vcc.Databases.RowsNumber = $rows_number.val();
			vcc.Databases.TimeStamp = $timestamp.val();
			vcc.Databases.Enabled = true;
		}
		else{
			vcc.Databases.LimitNumber = 3;
			vcc.Databases.Sizing = 20480;
			vcc.Databases.RowsNumber = 3000;
			vcc.Databases.TimeStamp = 365;
			vcc.Databases.Enabled = false;
		}
	
	};

	prov_form_vccs_basic.prototype.get_configuration_limits = function(vcc)
	{
		if (!vcc) {
			return;
		}

		let MaxCampaigns = $("#prov_form_vcc_basic_config_limits_max_campaigns").val();
		let MaxCampaignsUsers = $("#prov_form_vcc_basic_config_limits_max_campaigns_users").val();
		let MaxAttsLevel = $("#prov_form_vcc_basic_config_limits_max_att_levels").val();
		let MaxAttsLevelUsers = $("#prov_form_vcc_basic_config_limits_max_att_levels_users").val();
		let MaxWorkgroups = $("#prov_form_vcc_basic_config_limits_max_workgroups").val();
		let MaxWorkgroupsUsers = $("#prov_form_vcc_basic_config_limits_max_workgroups_users").val();

		//Si tienen datos entonces agrego 
		if (MaxCampaigns && MaxCampaigns.trim() !== "") {
			vcc.ConfigLimits = {
				MaxCampaigns : parseInt(MaxCampaigns, 10),
				MaxCampaignsUsers : parseInt(MaxCampaignsUsers, 10),
				MaxAttsLevel : parseInt(MaxAttsLevel, 10),
				MaxAttsLevelUsers : parseInt(MaxAttsLevelUsers, 10),
				MaxWorkgroups : parseInt(MaxWorkgroups, 10),
				MaxWorkgroupsUsers : parseInt(MaxWorkgroupsUsers, 10)
			};
		}
		
	};

	prov_form_vccs_basic.prototype.get_audit_config = function(vcc)
	{
		if (!vcc) {
			return;
		}
		if (!vcc.Audit) {
			vcc.Audit = {};
		}
		const $cleanUpPeriod = $("#prov_form_vcc_basic_audit_days");
		const $enable_cleanup = $("#prov_form_vcc_basic_audit_cleanup_enabled");
		if ($enable_cleanup.prop("checked")) {
			vcc.Audit.CleanUpPeriod = $cleanUpPeriod.val();
			vcc.Audit.EnableCleanUp = true;
		}
		else{
			vcc.Audit.CleanUpPeriod = 365;
			vcc.Audit.EnableCleanUp = false;
		}
	};

	prov_form_vccs_basic.prototype.get_api_sdk_config = function(vcc)
	{
		if (!vcc) {
			return;
		}
		if (!vcc.ApiConfiguration) {
			vcc.ApiConfiguration = {};
		}
		vcc.ApiConfiguration.urlsConfig = {
			resources: {
				high: {
					rateLimits: {
						desc: "very high rate limit"
					}
				},
				medium : {
					rateLimits: {
						desc: "high rate limit"
					}
				},
				medium_light : {
					rateLimits: {
						desc: "medium rate limit"
					}
				},
				low : {
					rateLimits: {
						desc: "low rate limit"
					}
				}
			}
		};
		const $enable = $("#prov_form_vcc_basic_api_sdk_enabled");
		vcc.ApiConfiguration.enabled = $enable.prop("checked");

		if (vcc.ApiConfiguration.enabled) {
			// high
			vcc.ApiConfiguration.urlsConfig.resources.high.rateLimits.maxRequests = $("#prov_form_vcc_basic_api_sdk_high_rate_max_requests").val();
			// medium
			vcc.ApiConfiguration.urlsConfig.resources.medium.rateLimits.maxRequests = $("#prov_form_vcc_basic_api_sdk_medium_rate_max_requests").val();
			// medium_light
			vcc.ApiConfiguration.urlsConfig.resources.medium_light.rateLimits.maxRequests = $("#prov_form_vcc_basic_api_sdk_medium_light_rate_max_requests").val();
			// low
			vcc.ApiConfiguration.urlsConfig.resources.low.rateLimits.maxRequests = $("#prov_form_vcc_basic_api_sdk_low_rate_max_requests").val();
			return;
		}
		// high
		vcc.ApiConfiguration.urlsConfig.resources.high.rateLimits.timeLimitSec = undefined;
		vcc.ApiConfiguration.urlsConfig.resources.high.rateLimits.maxRequests = undefined;
		// medium
		vcc.ApiConfiguration.urlsConfig.resources.medium.rateLimits.timeLimitSec = undefined;
		vcc.ApiConfiguration.urlsConfig.resources.medium.rateLimits.maxRequests = undefined;
		// medium_light
		vcc.ApiConfiguration.urlsConfig.resources.medium_light.rateLimits.timeLimitSec = undefined;
		vcc.ApiConfiguration.urlsConfig.resources.medium_light.rateLimits.maxRequests = undefined;
		// low
		vcc.ApiConfiguration.urlsConfig.resources.low.rateLimits.timeLimitSec = undefined;
		vcc.ApiConfiguration.urlsConfig.resources.low.rateLimits.maxRequests = undefined;
	};

	prov_form_vccs_basic.prototype.get_recording_configuration = function(vcc)
	{
		if (!vcc) {
			return;
		}
		const $call_automatic = $("#prov_form_vcc_basic_recording_call_automatic_recording_criteria");
		const $call_format = $("#prov_form_vcc_basic_recording_call_format");
		const $call_channels = $("#prov_form_vcc_basic_recording_call_channels");

		const $videocall_automatic = $("#prov_form_vcc_basic_recording_videocall_automatic_recording_criteria");
		const $videocall_format = $("#prov_form_vcc_basic_recording_videocall_format");
		const $videocall_mode = $("#prov_form_vcc_basic_recording_videocall_mode");

		const $voicemail_automatic = $("#prov_form_vcc_basic_recording_voicemail_automatic_recording_criteria");
		const $voicemail_format = $("#prov_form_vcc_basic_recording_voicemail_format");
		const $voicemail_channels = $("#prov_form_vcc_basic_recording_voicemail_channels");

		vcc.RecordingConfiguration = (typeof vcc.RecordingConfiguration === "string" || vcc.RecordingConfiguration instanceof String) ? JSON.parse(vcc.RecordingConfiguration) : (vcc.RecordingConfiguration || {});
		vcc.RecordingConfiguration.CALL = vcc.RecordingConfiguration.CALL || {};
		vcc.RecordingConfiguration.VIDEO_CALL = vcc.RecordingConfiguration.VIDEO_CALL || {};
		vcc.RecordingConfiguration.VOICEMAIL = vcc.RecordingConfiguration.Voicemail|| {};

		vcc.RecordingConfiguration.CALL.automatic = $call_automatic.val();
		vcc.RecordingConfiguration.CALL.format = $call_format.val();
		vcc.RecordingConfiguration.CALL.channels = $call_channels.val();

		vcc.RecordingConfiguration.VIDEO_CALL.automatic = $videocall_automatic.val();
		vcc.RecordingConfiguration.VIDEO_CALL.format = $videocall_format.val();
		vcc.RecordingConfiguration.VIDEO_CALL.mode = $videocall_mode.val();

		vcc.RecordingConfiguration.VOICEMAIL.automatic = $voicemail_automatic.val();
		vcc.RecordingConfiguration.VOICEMAIL.format = $voicemail_format.val();
		vcc.RecordingConfiguration.VOICEMAIL.channels = $voicemail_channels.val();
		
		vcc.RecordingConfiguration = JSON.stringify(vcc.RecordingConfiguration);
	};

	prov_form_vccs_basic.prototype.set_api_sdk_config = function(vcc)
	{
		const $enable = $("#prov_form_vcc_basic_api_sdk_enabled");
		if (!vcc || !vcc.ApiConfiguration || !vcc.ApiConfiguration.enabled) {
			$enable.prop("checked", false).trigger("change");
			return;
		}
		$enable.prop("checked", true).trigger("change");
		if (vcc.ApiConfiguration.urlsConfig && vcc.ApiConfiguration.urlsConfig.resources) {
			const resources = vcc.ApiConfiguration.urlsConfig.resources;
			const get_rate_limits_info = function(resource_type) {
				let default_info = {maxRequests: 0};
				if (!resources[resource_type]) {
					return default_info;
				}
				if (resources[resource_type].rateLimits) {
					return resources[resource_type].rateLimits;
				}
				return default_info;
			};
			const high_rate_limits = get_rate_limits_info("high");
			const medium_rate_limits = get_rate_limits_info("medium");
			const medium_light_rate_limits = get_rate_limits_info("medium_light");
			const low_rate_limits = get_rate_limits_info("low");
			// high
			$("#prov_form_vcc_basic_api_sdk_high_rate_max_requests").val(high_rate_limits.maxRequests);
			// medium
			$("#prov_form_vcc_basic_api_sdk_medium_rate_max_requests").val(medium_rate_limits.maxRequests);
			// medium_light
			$("#prov_form_vcc_basic_api_sdk_medium_light_rate_max_requests").val(medium_light_rate_limits.maxRequests);
			// low
			$("#prov_form_vcc_basic_api_sdk_low_rate_max_requests").val(low_rate_limits.maxRequests);
		}
	};

	prov_form_vccs_basic.prototype.get_security_captcha = function(vcc)
	{
		if (!vcc) {
			return;
		}
		vcc.Security = vcc.Security || {captcha: {}};

		const $captcha_enabled = $("#prov_form_vcc_basic_security_captcha_enabled");
		vcc.Security.captcha.enabled = $captcha_enabled.prop("checked") ? "1" : "0";
		if (vcc.Security.captcha.enabled !== "1") {
			return;
		}

		const security_captcha_type = $("#prov_form_vcc_basic_security_captcha_type").val();
		if (security_captcha_type !== "CUSTOM") {
			vcc.Security.captcha.publicKey = "";
			vcc.Security.captcha.secretKey = "";
			return;
		}

		const $public_key = $("#prov_form_vcc_basic_security_captcha_public_key");
		const $secret_key = $("#prov_form_vcc_basic_security_captcha_secret_key");
		vcc.Security.captcha.publicKey = $public_key.val();
		vcc.Security.captcha.secretKey = $secret_key.val();
	};

	prov_form_vccs_basic.prototype.get_security_authentication = function(vcc)
	{
		if (!vcc) {
			return;
		}
		vcc.Security = vcc.Security || {};
		const type = $("#prov_form_vcc_basic_security_authentication_type").val();
		vcc.Security.authentication = {
			type : type
		};
		//BASIC
		if(type === "BASIC" || type === "MFA") {
			vcc.Security.authentication.passwordChecks = {};
			vcc.Security.authentication.passwordChecks.minLen = parseInt($("#prov_form_vcc_basic_security_authentication_passwordChecks_minLen").val());
			vcc.Security.authentication.passwordChecks.maxLen = parseInt($("#prov_form_vcc_basic_security_authentication_passwordChecks_maxLen").val());
			vcc.Security.authentication.passwordChecks.expirationDays = parseInt($("#prov_form_vcc_basic_security_authentication_passwordChecks_expirationDays").val());
			vcc.Security.authentication.passwordChecks.blockOnLoginFailures = parseInt($("#prov_form_vcc_basic_security_authentication_passwordChecks_blockOnLoginFailures").val());
			vcc.Security.authentication.passwordChecks.history = parseInt($("#prov_form_vcc_basic_security_authentication_passwordChecks_history").val());
			vcc.Security.authentication.passwordChecks.minUpperChars = parseInt($("#prov_form_vcc_basic_security_authentication_passwordChecks_minUpperChars").val());
			vcc.Security.authentication.passwordChecks.minLowerChars = parseInt($("#prov_form_vcc_basic_security_authentication_passwordChecks_minLowerChars").val());
			vcc.Security.authentication.passwordChecks.minSpecialChars = parseInt($("#prov_form_vcc_basic_security_authentication_passwordChecks_minSpecialChars").val());
			vcc.Security.authentication.passwordChecks.minNumericChars = parseInt($("#prov_form_vcc_basic_security_authentication_passwordChecks_minNumericChars").val());
			vcc.Security.authentication.passwordChecks.expirationNotificationDays = parseInt($("#prov_form_vcc_basic_security_authentication_passwordChecks_expirationNotificationDays").val());
			vcc.Security.authentication.passwordChecks.changeOnFirstLogin = $("#prov_form_vcc_basic_security_authentication_passwordChecks_changeOnFirstLogin").prop("checked");
		}
		//MFA
		vcc.Security.mfa = {
			enabled : type === "MFA" ? "1" : "0"
		};
		if (vcc.Security.mfa.enabled === "1") {
			vcc.Security.mfa.type = $("#prov_form_vcc_basic_security_authentication_mfa_type").val();
			switch(vcc.Security.mfa.type) {
				case "MAIL": {
					vcc.Security.mfa.account = $("#prov_form_vcc_basic_security_authentication_mfa_mail_account").val();
					vcc.Security.mfa.keyLength = parseInt($("#prov_form_vcc_basic_security_authentication_mfa_mail_length").val());
					vcc.Security.mfa.expirationSeconds = 
						parseInt($("#prov_form_vcc_basic_security_authentication_mfa_mail_expiration").val()) * 60;
					vcc.Security.mfa.subject = $("#prov_form_vcc_basic_security_authentication_mfa_mail_subject").val();
					const parts = this.m_mailControls.mfa.get_body_parts();
					vcc.Security.mfa.body = JSON.stringify([ parts[0] ]);
					vcc.Security.mfa.attachments = JSON.stringify(this.m_mailControls.mfa.get_repository_images_list());
					break;
				}
			}
		}
		//LDAP
		vcc.Security.ldap = {
			enabled : type === "LDAP" ? "1" : "0"
		};
		if (vcc.Security.ldap.enabled === "1") {
			vcc.Security.ldap.server = $("#prov_form_vcc_basic_security_authentication_ldap_server").val();
			vcc.Security.ldap.adminUserDN = $("#prov_form_vcc_basic_security_authentication_ldap_adminUserDN").val();
			vcc.Security.ldap.useTLS = $("#prov_form_vcc_basic_security_authentication_ldap_useTLS").prop("checked");
			vcc.Security.ldap.baseDN = $("#prov_form_vcc_basic_security_authentication_ldap_baseDN").val();
			vcc.Security.ldap.userIdAttributte = $("#prov_form_vcc_basic_security_authentication_ldap_userIdAttributte").val();
			
			const $password =  $("#prov_form_vcc_basic_security_authentication_ldap_password"); 
			const jwtToken  = inConcert.Utils.GetJwtToken($password.val());
			vcc.Security.ldap.adminPassword = jwtToken;

			const $ldap_panel = $("#prov_form_vcc_basic_security_authentication_ldap_panel");

			vcc.Security.ldap.sync_enabled = $("#prov_form_vcc_basic_security_authentication_ldap_sync_enabled").prop("checked");
			vcc.Security.ldap.groupFilters = {};
			if(vcc.Security.ldap.sync_enabled) {
				vcc.Security.ldap.timezone = $("#prov_form_vcc_basic_security_authentication_ldap_sync_timezone").val();
				vcc.Security.ldap.language = $("#prov_form_vcc_basic_security_authentication_ldap_sync_language").val();
				vcc.Security.ldap.country = $("#prov_form_vcc_basic_security_authentication_ldap_sync_country").val();
				vcc.Security.ldap.area = $("#prov_form_vcc_basic_security_authentication_ldap_sync_area").val();
				vcc.Security.ldap.default_campaign = $("#prov_form_vcc_basic_security_authentication_ldap_sync_default_campaign").val();
				vcc.Security.ldap.initial_state = $("#prov_form_vcc_basic_security_authentication_ldap_sync_initial_state").val();

				$ldap_panel.find("[LDAP_ROW_GROUP_NAME]").each(function(_, item) {
					const $item = $(item);
					const group = $item.attr("ldap_row_group_name");
					if (group !== "allusers") {
						const $ldapGroupFilter = $item.find("[ldap_group_filter]");
						vcc.Security.ldap.groupFilters[group] = $ldapGroupFilter.val();
					}
				});
			}
			else {
				const group ="allusers";
				const selector = jQUERY_SELECTOR_LDAP_GROUP_FILTER(group);
				const filter = $ldap_panel.find(selector).val();
				vcc.Security.ldap.groupFilters[group] = filter;
			}
		}
		//OPENID
		vcc.Security.openid = {
			enabled : type === "OPENID" ? "1" : "0"
		};
		if (vcc.Security.openid.enabled === "1") {
			vcc.Security.openid.openid_configuration_url = $("#prov_form_vcc_basic_security_authentication_openid_configuration_url").val();
			vcc.Security.openid.sync_method = $("#prov_form_vcc_basic_security_authentication_openid_sync_method").val();
			if(vcc.Security.openid.sync_method === "AZUREAD") {
				vcc.Security.openid.client_id = $("#prov_form_vcc_basic_security_authentication_azuread_sync_client_id").val();

				const $password =  $("#prov_form_vcc_basic_security_authentication_azuread_sync_client_secret");
				const jwtToken  = inConcert.Utils.GetJwtToken($password.val());
				vcc.Security.openid.adminPassword = jwtToken;


				vcc.Security.openid.sync_enabled = $("#prov_form_vcc_basic_security_authentication_azuread_sync_enabled").prop("checked");
				vcc.Security.openid.groupFilters = {};
				vcc.Security.openid.timezone = $("#prov_form_vcc_basic_security_authentication_azuread_sync_timezone").val();
				vcc.Security.openid.language = $("#prov_form_vcc_basic_security_authentication_azuread_sync_language").val();
				vcc.Security.openid.country = $("#prov_form_vcc_basic_security_authentication_azuread_sync_country").val();
				vcc.Security.openid.area = $("#prov_form_vcc_basic_security_authentication_azuread_sync_area").val();
				vcc.Security.openid.default_campaign = $("#prov_form_vcc_basic_security_authentication_azuread_sync_default_campaign").val();
				vcc.Security.openid.initial_state = $("#prov_form_vcc_basic_security_authentication_azuread_sync_initial_state").val();

				const $azuread_panel = $("#prov_form_vcc_basic_security_authentication_azuread_sync_panel");
				$azuread_panel.find("[AZUREAD_ROW_GROUP_NAME]").each(function(_, item) {
					const $item = $(item);
					const group = $item.attr("AZUREAD_ROW_GROUP_NAME");
					if (group !== "allusers") {
						const $filter = $item.find("[AZUREAD_GROUP_FILTER]");
						const $cboSelectedOption = $filter.find("option:selected");
						vcc.Security.openid.groupFilters[group] = {
							id         : $filter.val(),
							displayName: $cboSelectedOption.text()
						};
					}
				});
			}
		}
	};

	/*
vccInfo.AppsIntegrations = {
	HelpDesk = {
		Enabled = "1",
		UserAuthentication = {
			Password = "...",
			User = "..",
			Scope = "...",
			GrantType = "...",
			ClientId = "..."
		},
		Notifications = {
			Url = "https://168.62.161.232:2443/ichelpdeskdesarrollo/notifications"
		},
		Licenses = {
			Users = "1"
		},
		Server = {
			HOST = "https://168.62.161.232:2443/ichelpdeskdesarrollo"
		}
	},
	WorkForceManagement = {Enabled = "1", Notifications = {Url = "EDe YAYAaaaaaaaaaaaaaaaa"}},
	MarketingAndSales = {
		Notifications = {
		  Url = "https://webapp01.inconcertcc.com/public/occ/process"
		},
		AppRender = {
		  Url = "https://webapp01.inconcertcc.com/form/render"
		},
		AccessToken = {
		  Url = "https://webapp01.inconcertcc.com/public/occ/get_session_token"
		},
		Enabled = "1"
	}
*/
	prov_form_vccs_basic.prototype.get_integrations = function(vcc)
	{
		const $helpdesk_integration = $("#prov_form_vcc_basic_helpdesk_integration_enabled");
		const $mas_integration = $("#prov_form_vcc_basic_mas_integration_enabled");
		const $wfm_integration = $("#prov_form_vcc_basic_wfm_integration_enabled");

		vcc.AppsIntegrations = {
			HelpDesk: {
				Enabled: $helpdesk_integration.prop("checked") ? "1" : "0"
			},
			MarketingAndSales: {
				Enabled: $mas_integration.prop("checked") ? "1" : "0"
			},
			WorkForceManagement: {
				Enabled: $wfm_integration.prop("checked") ? "1" : "0"
			}
		};

		// HELPDESK
		vcc.HasHelpDeskIntegration = false; // campo calculado que se utilza unicamente client side para la actualizacion de la celda.
		if (vcc.AppsIntegrations.HelpDesk.Enabled === "1") {
			// cammpo calculado (only client side)
			vcc.HasHelpDeskIntegration = true;
			// notification
			const $helpdesk_notification_url = $("#prov_form_vcc_basic_helpdesk_integration_notification_url");
			vcc.AppsIntegrations.HelpDesk.Notifications = {};
			vcc.AppsIntegrations.HelpDesk.Notifications.Url = $helpdesk_notification_url.val();
			// server
			const $heldpesk_server_url = $("#prov_form_vcc_basic_helpdesk_integration_server_url");
			vcc.AppsIntegrations.HelpDesk.Server = {};
			vcc.AppsIntegrations.HelpDesk.Server.HOST = $heldpesk_server_url.val();
		// Adm Auth Api ------
			// helpdesk auth - userid
			const $helpdesk_auth_adm_user_id = $("#prov_form_vcc_basic_helpdesk_integration_api_adm_user_auth_id");
			vcc.AppsIntegrations.HelpDesk.UserAuthentication = {};
			vcc.AppsIntegrations.HelpDesk.UserAuthentication.User = $helpdesk_auth_adm_user_id.val();
			// helpdesk auth - pwd
			const $helpdesk_auth_adm_pwd = $("#prov_form_vcc_basic_helpdesk_integration_api_adm_user_auth_pwd");
			vcc.AppsIntegrations.HelpDesk.UserAuthentication.Password = $helpdesk_auth_adm_pwd.val();
		// Flow Auth Api ------
			// helpdesk auth - userid
			const $helpdesk_auth_flow_user_id = $("#prov_form_vcc_basic_helpdesk_integration_api_flow_user_auth_id");
			vcc.AppsIntegrations.HelpDesk.ApiUserAuthentication = {};
			vcc.AppsIntegrations.HelpDesk.ApiUserAuthentication.User = $helpdesk_auth_flow_user_id.val();
			// helpdesk auth - pwd
			const $helpdesk_auth_flow_pwd = $("#prov_form_vcc_basic_helpdesk_integration_api_flow_user_auth_pwd");
			vcc.AppsIntegrations.HelpDesk.ApiUserAuthentication.Password = $helpdesk_auth_flow_pwd.val();
			// helpdesk user licences

			if(inConcert.app.hasLicenseSystem()) {
				const $helpdesk_user_licenses = $("#prov_form_vcc_basic_helpdesk_integration_user_licenses");
				vcc.AppsIntegrations.HelpDesk.Licenses = {};
				vcc.AppsIntegrations.HelpDesk.Licenses.Users = $helpdesk_user_licenses.val();
			}
		}

		// M&S
		vcc.HasMarketingAndSalesIntegration = false; // campo calculado que se utilza unicamente client side para la actualizacion de la celda.
		if (vcc.AppsIntegrations.MarketingAndSales.Enabled === "1") {
			// cammpo calculado (only client side)
			vcc.HasMarketingAndSalesIntegration = true;
			// notification
			const $mas_notification_url = $("#prov_form_vcc_basic_mas_integration_notification_url");
			vcc.AppsIntegrations.MarketingAndSales.Notifications = {};
			vcc.AppsIntegrations.MarketingAndSales.Notifications.Url = $mas_notification_url.val();
			// app render
			const $mas_app_url = $("#prov_form_vcc_basic_mas_integration_application_url");
			vcc.AppsIntegrations.MarketingAndSales.AppRender = {};
			vcc.AppsIntegrations.MarketingAndSales.AppRender.Url = $mas_app_url.val();
			// token
			const $mas_token_url = $("#prov_form_vcc_basic_mas_integration_token_url");
			vcc.AppsIntegrations.MarketingAndSales.AccessToken = {};
			vcc.AppsIntegrations.MarketingAndSales.AccessToken.Url = $mas_token_url.val();
			// API
			const $mas_api_url = $("#prov_form_vcc_basic_mas_integration_api_url");
			vcc.AppsIntegrations.MarketingAndSales.Api = {};
			vcc.AppsIntegrations.MarketingAndSales.Api.Url = $mas_api_url.val();
		}

		// WorkForce Management
		vcc.HasWorkForceManagementIntegration = false; // campo calculado que se utilza unicamente client side para la actualizacion de la celda.
		if (vcc.AppsIntegrations.WorkForceManagement.Enabled === "1") {
			// cammpo calculado (only client side)
			vcc.HasWorkForceManagementIntegration = true;
			// notification
			const $wfm_notification_url = $("#prov_form_vcc_basic_wfm_integration_notification_url");
			vcc.AppsIntegrations.WorkForceManagement.Notifications = {};
			vcc.AppsIntegrations.WorkForceManagement.Notifications.Url = $wfm_notification_url.val();
		}

		// Widgets
		for (let widget_id in this.m_widgets) {
			if (this.m_widgets.hasOwnProperty(widget_id)) {
				const $widget_enabled = this.$get_widget_enabled(widget_id);
				if ($widget_enabled.length > 0) {
					vcc.AppsIntegrations[widget_id] = vcc.AppsIntegrations[widget_id] || {};
					vcc.AppsIntegrations[widget_id].Enabled = $widget_enabled.prop("checked") ? "1" : "0";
					let config = {};
					if (vcc.AppsIntegrations[widget_id].Enabled === "1") {
						let widget_config_obj = this.get_widget_instance(widget_id);
						if (typeof (widget_config_obj.get) === "function") {
							config = widget_config_obj.get();
						}
						else {
							let msg = FORMAT("prov_form_vccs_basic.prototype.get_integrations - The Widget '{0}' Provisioning class has to implement the 'get' method.", widget_id);
							console.log(msg);
							window.alert(msg);
						}
					}
					vcc.AppsIntegrations[widget_id].Config = config;
				}
			}
		}
	};

	prov_form_vccs_basic.prototype.$get_widget_enabled = function(widget_id)
	{
		widget_id = widget_id.toLowerCase();
		return $(FORMAT("#prov_form_vcc_basic_widgets_{0}_enabled", widget_id));
	};

	prov_form_vccs_basic.prototype.$get_widget_fields_section = function(widget_id)
	{
		widget_id = widget_id.toLowerCase();
		return $(FORMAT("#prov_form_vcc_basic_{0}_fields_section", widget_id));
	};

	prov_form_vccs_basic.prototype.get_licenses = function(vcc)
	{
		if (!vcc) {
			return;
		}
		$("#prov_form_vcc_basic_licences_section input[selector='1'][type='checkbox']").each(function(_, license)
		{
			const $checkbox = $(license);
			const field_channel_name = $checkbox.attr("channel_id") + "_licensed"; // CALL_licensed
			const enabled = $checkbox.prop("checked") ? "1" : "0";
			vcc[field_channel_name] = enabled;
		});
	};

	prov_form_vccs_basic.prototype.set = function(vcc)
	{
		this.m_vcc_has_users = typeof vcc.hasUsers === "undefined" ? this.m_vcc_has_users : vcc.hasUsers;
		this.m_original_user_password = vcc.adminPwd;

		$("#prov_form_vcc_basic_user_id").val(vcc.adminUser);
		$("#prov_form_vcc_basic_user_password").val(vcc.adminPwd);
		$("#prov_form_vcc_basic_name").val(vcc.Id) ;
		$("#prov_form_vcc_basic_desc").focus().val(vcc.Description);
		$("#prov_form_vcc_basic_online_users").focus().val(vcc.OnlineUsers);
		$("#prov_form_vcc_basic_company").focus().val(vcc.company);
		$("#prov_form_vcc_basic_company_address").focus().val(vcc.CompanyAddress);
		$("#prov_form_vcc_basic_tel1").focus().val(vcc.Telephone1);
		$("#prov_form_vcc_basic_tel2").focus().val(vcc.Telephone2);
		$("#prov_form_vcc_basic_contact_name").focus().val(vcc.ContactName);
		$("#prov_form_vcc_basic_contact_email").focus().val(vcc.ContactEmail);
		$("#prov_form_vcc_basic_free_seconds_after_interaction").focus().val(vcc.FreeSecondsAfterInteraction);
		$("#prov_form_vcc_basic_custom_metrics_limit").focus().val(vcc.CustomMetricsLimit);
		$("#prov_form_vcc_basic_custom_filters_limit").focus().val(vcc.CustomFiltersLimit);
		$("#prov_form_vcc_basic_export_space_quota").focus().val(vcc.ExportSpaceQuota);
		$("#prov_form_vcc_basic_shared_files_space_quota").focus().val(vcc.SharedFilesSpaceQuota);

		// Storage Lifecycle
		$("#prov_form_vcc_basic_storage_s3_files_lifecycle").val((vcc.StorageLifecycle.S3Files) || "");
		$("#prov_form_vcc_basic_storage_s3_meta_lifecycle").val((vcc.StorageLifecycle.S3MetaData) || "");
		$("#prov_form_vcc_basic_storage_call_files_lifecycle").val((vcc.StorageLifecycle.RepositoryCall) || "");
		$("#prov_form_vcc_basic_storage_email_files_lifecycle").val((vcc.StorageLifecycle.RepositoryMail) || "");
		$("#prov_form_vcc_basic_storage_other_files_lifecycle").val((vcc.StorageLifecycle.RepositoryOther) || "");

		if (inConcert.app.hasRepositoryService()) {
			$("#prov_form_vcc_basic_storage_location").val((vcc.StorageConfig && vcc.StorageConfig.StorageLocation) || "cassandra").trigger("change");
			$("#prov_form_vcc_basic_storage_location_params_S3PubKey").val((vcc.StorageConfig && vcc.StorageConfig.StorageLocation && vcc.StorageConfig.StorageLocationParams && vcc.StorageConfig.StorageLocationParams.S3PubKey) || "");
			$("#prov_form_vcc_basic_storage_location_params_S3PrivKey").val((vcc.StorageConfig && vcc.StorageConfig.StorageLocation && vcc.StorageConfig.StorageLocationParams && vcc.StorageConfig.StorageLocationParams.S3PrivKey) || "");
            
			if ((vcc.StorageConfig && vcc.StorageConfig.StorageLocation && vcc.StorageConfig.StorageLocationParams && vcc.StorageConfig.StorageLocationParams.S3Region)) {
				$("#prov_form_vcc_basic_storage_location_params_S3Region").val(vcc.StorageConfig.StorageLocationParams.S3Region);
			}

			if (vcc.StorageConfig && vcc.StorageConfig.StorageLocation && vcc.StorageConfig.StorageLocationParams && 
				vcc.StorageConfig.StorageLocationParams.S3Bucket && vcc.StorageConfig.StorageLocationParams.IsPredBucket) {
				$("#prov_form_vcc_basic_storage_location_params_S3Bucket").val(vcc.StorageConfig.StorageLocationParams.S3Bucket);
				$("#prov_form_vcc_basic_storage_location_params_S3BucketCheckBox").prop("checked",true).trigger("change");
			}

			// Prefix Configuration
			$("#prov_form_vcc_basic_prefix_config_tree_structure").val((vcc.InteractionPrefixConfig && vcc.InteractionPrefixConfig.TreeString) || "");
		}

		this.set_session_timeouts(vcc);
		this.set_security_captcha(vcc);
		this.fill_security_authentication_types();
		this.set_security_authentication(vcc);
		this.set_integrations(vcc);
		this.set_licenses(vcc);
		this.set_screen_recording(vcc);
		this.set_video_config(vcc);
		this.set_audit_config(vcc);
		this.set_configuration_limits(vcc);

		// API SDK
		this.set_api_sdk_config(vcc);
		// outgoing batches
		this.m_outgoing_batches_channels.set(vcc);
		// social networks apps
		this.m_social_networks_app_channel.set(vcc);
		// Databases
		this.set_databases_config(vcc);
		// Telephony native
		this.set_telephony_config(vcc);
		//Drafts
		this.set_drafts_config(vcc);
		//Phone Validation
		this.set_phone_validation_config(vcc);
		//Recording Configuration
		if(inConcert.app.hasLicenseSystem()) {
			this.set_recording_configuration(vcc);		
		}			
	};

	prov_form_vccs_basic.prototype.set_telephony_config = function(vcc)
	{
		if (!vcc) {
			return;
		}
		if (vcc.HasAllegroIntegration) {
			return;
		}
		const DEFAULT_DIAL_TIME_VALUE = 60;
		const DEFAULT_MAX_MANUAL_RATE = 8;
		let dial_time = vcc.TelephonyConfig ? vcc.TelephonyConfig.DialTimeouts.timeoutSec : DEFAULT_DIAL_TIME_VALUE;
		let max_manual_rate = (vcc.TelephonyConfig && vcc.TelephonyConfig.ProcessMaxManualRate) ? vcc.TelephonyConfig.ProcessMaxManualRate : DEFAULT_MAX_MANUAL_RATE;
		$("#prov_form_vcc_basic_telephony_dial_time").val(dial_time);
		$("#prov_form_vcc_basic_telephony_max_manual_rate").val(max_manual_rate);
		
	};

	prov_form_vccs_basic.prototype.set_session_timeouts = function(vcc)
	{
		const $session_timeouts_enabled = $("#prov_form_vcc_basic_session_timeouts_enabled");
		const $mobile = $("#prov_form_vcc_basic_session_timeouts_mobile");

		const default_values = {
			MOBILE: parseInt($mobile.attr("default_value"), 0)
		};

		if (!vcc || !vcc.SessionTimeouts) {
			$session_timeouts_enabled.prop("checked", false);
			this.toggle_session_timeouts();
			return;
		}
		if (parseInt((vcc.SessionTimeouts.MOBILE || "0"), 10) === default_values.MOBILE) {
			$session_timeouts_enabled.prop("checked", false);
			this.toggle_session_timeouts();
			return;
		}

		$session_timeouts_enabled.prop("checked", true);
		this.toggle_session_timeouts();

		$mobile.val(vcc.SessionTimeouts.MOBILE);
	};

	prov_form_vccs_basic.prototype.set_screen_recording = function(vcc)
	{
		const $screen_recording_enabled = $("#prov_form_vcc_basic_screen_recording_enabled");
		const $port = $("#prov_form_vcc_basic_screen_recording_port");
		const $fps = $("#prov_form_vcc_basic_screen_recording_fps");
		const $bitrate = $("#prov_form_vcc_basic_screen_recording_bitrate");
		const is_debian_allegro = vcc && ((!vcc.HasAllegroIntegration && inConcert.app.is_os_debian()) || vcc.HasAllegroIntegration);
		
		if (!vcc || !vcc.ScreenRecording || !vcc.ScreenRecording.Enabled || is_debian_allegro) {
			if (is_debian_allegro) {
				$("#prov_form_vcc_basic_screen_recording_panel").hide();
			}
			$screen_recording_enabled.prop("checked", false);
			this.toggle_screen_recording();
			return;
		}

		$("#prov_form_vcc_basic_screen_recording_panel").show();
		$screen_recording_enabled.prop("checked", true);
		this.toggle_screen_recording();

		$port.val(vcc.ScreenRecording.PORT);
		$fps.val(vcc.ScreenRecording.FPS);
		$bitrate.val(vcc.ScreenRecording.BITRATE);
	};

	prov_form_vccs_basic.prototype.set_video_config = function(vcc)
	{
		const $screen_video_config_enable = $("#prov_form_vcc_basic_video_config_enabled");
		const $bitrate = $("#prov_form_vcc_basic_video_bitrate");

		if (!vcc || !vcc.VideoConfig || !vcc.VideoConfig.Enabled) {
			$screen_video_config_enable.prop("checked", false);
			this.toggle_video_config();
			return;
		}

		$screen_video_config_enable.prop("checked", true);
		this.toggle_video_config();

		$bitrate.val(vcc.VideoConfig.BITRATE);
	};

	prov_form_vccs_basic.prototype.set_drafts_config = function(vcc)
	{
		const $drafts_enable = $("#prov_form_vcc_basic_drafts_configuration_enabled");
		if (!vcc || !vcc.DraftsConfig || !vcc.DraftsConfig.Enabled) {
			$drafts_enable.prop("checked", false);
			this.toggle_drafts_config();
			return;
		}
		const $maxDrafts = $("#prov_form_vcc_basic_drafts_configuration_number");
		const $maxdSizeDrafts = $("#prov_form_vcc_basic_drafts_configuration_max_size");

		$drafts_enable.prop("checked", true);
		this.toggle_drafts_config();

		$maxDrafts.val(vcc.DraftsConfig.MaxDrafts);
		$maxdSizeDrafts.val(vcc.DraftsConfig.MaxSizeDrafts);
	};

	prov_form_vccs_basic.prototype.set_phone_validation_config = function(vcc)
	{
		const $format = $("#prov_form_vcc_basic_phone_validation_configuration_format");
		const $enabled = $("#prov_form_vcc_basic_phone_validation_configuration_enabled");
		const available = !vcc?.HasAllegroIntegration;
		const enabled = (available && vcc?.PhoneValidation?.Enabled) || false;
		const format = (enabled && vcc.PhoneValidation.Format) || "global_plain";
		$enabled.prop("checked", enabled).trigger("change");
		window.setTimeout(() => $enabled.prop("disabled", !available), 100);
		$format.val(format);
	};

	prov_form_vccs_basic.prototype.set_databases_config = function(vcc)
	{
		const $limit_number = $("#prov_form_vcc_basic_databases_number");
		const $sizing = $("#prov_form_vcc_basic_databases_sizing");
		const $rows_number = $("#prov_form_vcc_basic_databases_rows_number");
		const $timestamp = $("#prov_form_vcc_basic_databases_timestamp");
		const $enabled = $("#prov_form_vcc_basic_databases_configuration_panel_enabled");
		if (!vcc || !vcc.Databases || !vcc.Databases.Enabled) {
			$enabled.prop("checked", false);
			this.toggle_databases_config();
			return;
		}
		$enabled.prop("checked", true);
		this.toggle_databases_config();
		$limit_number.val((vcc.Databases.LimitNumber).toString());
		$sizing.val((vcc.Databases.Sizing).toString());
		$rows_number.val((vcc.Databases.RowsNumber).toString());
		$timestamp.val((vcc.Databases.TimeStamp).toString());
	};

	prov_form_vccs_basic.prototype.set_audit_config = function(vcc)
	{

		if (!vcc || !vcc.Audit || !vcc.Audit.EnableCleanUp) {
			$("#prov_form_vcc_basic_audit_cleanup_enabled").prop("checked", false);
			this.toggle_audit_cleanup_config();
			return;
		}
		const $days = $("#prov_form_vcc_basic_audit_days");
		this.toggle_audit_cleanup_config();
		$("#prov_form_vcc_basic_audit_cleanup_enabled").prop("checked", true);
		$days.val(vcc.Audit.CleanUpPeriod);
	};

	prov_form_vccs_basic.prototype.set_configuration_limits = function(vcc)
	{
		if (!vcc || !vcc.ConfigLimits) {
			//Si al editar no tiene configurado limites desmarco como requerido.
			this.set_config_limits_class_required(false);
			return;
		}

		if (!(this.frm && (this.frm.is_new || this.frm.m_vcc && this.frm.m_vcc.isNew))) {
			/*
				Esto porque solo se va a buscar los limites del ambiente cuando se entra a la vista edicion.
				Para el caso en que se edite o se cree, se queda en la misma vista y no se actualiza.
				Mientras no se salga de la vista de edicion, no se remplazan los limites del vcc y se utilizan los ultimos disponibles para comparar el rango.
			*/
			this.m_oldConfigLimits = {
				MaxCampaigns : vcc.ConfigLimits.MaxCampaigns,
				MaxCampaignsUsers : vcc.ConfigLimits.MaxCampaignsUsers,
				MaxAttsLevel : vcc.ConfigLimits.MaxAttsLevel,
				MaxAttsLevelUsers : vcc.ConfigLimits.MaxAttsLevelUsers,
				MaxWorkgroups : vcc.ConfigLimits.MaxWorkgroups,
				MaxWorkgroupsUsers : vcc.ConfigLimits.MaxWorkgroupsUsers
			};

			this.set_config_limits_titles();
		}
		
		$("#prov_form_vcc_basic_config_limits_max_campaigns").val(vcc.ConfigLimits.MaxCampaigns);
		$("#prov_form_vcc_basic_config_limits_max_campaigns_users").val(vcc.ConfigLimits.MaxCampaignsUsers);
		$("#prov_form_vcc_basic_config_limits_max_att_levels").val(vcc.ConfigLimits.MaxAttsLevel);
		$("#prov_form_vcc_basic_config_limits_max_att_levels_users").val(vcc.ConfigLimits.MaxAttsLevelUsers);
		$("#prov_form_vcc_basic_config_limits_max_workgroups").val(vcc.ConfigLimits.MaxWorkgroups);
		$("#prov_form_vcc_basic_config_limits_max_workgroups_users").val(vcc.ConfigLimits.MaxWorkgroupsUsers);
	};

	prov_form_vccs_basic.prototype.set_security_captcha = function(vcc)
	{
		const $security_captcha_type = $("#prov_form_vcc_basic_security_captcha_type");
		const $captcha_enabled = $("#prov_form_vcc_basic_security_captcha_enabled");

		if (!vcc || !vcc.Security || !vcc.Security.captcha) {
			$security_captcha_type.val("DEFAULT").trigger("change");
			$captcha_enabled.prop("checked", false).trigger("change");
			return;
		}
		const enabled = vcc.Security.captcha.enabled === "1";
		$captcha_enabled.prop("checked", enabled);

		const public_key = (vcc.Security.captcha.publicKey || "");
		const secret_key = (vcc.Security.captcha.secretKey || "");

		if (public_key.length > 0 && secret_key.length > 0) {
			$security_captcha_type.val("CUSTOM").trigger("change");
		}
		else {
			$security_captcha_type.val("DEFAULT").trigger("change");
		}

		$security_captcha_type.trigger("change");

		const $public_key = $("#prov_form_vcc_basic_security_captcha_public_key");
		const $secret_key = $("#prov_form_vcc_basic_security_captcha_secret_key");
		$public_key.val(public_key);
		$secret_key.val(secret_key);
	};

	prov_form_vccs_basic.prototype.set_security_authentication = function(vcc)
	{
		const authentication = vcc && vcc.Security && vcc.Security.authentication || {};
		const passwordChecks = authentication.passwordChecks || {};
		$("#prov_form_vcc_basic_security_authentication_type").val(authentication.type);
		switch(authentication.type) {
			case "BASIC":
			case "MFA":
				$("#prov_form_vcc_basic_security_authentication_passwordChecks_minLen").val(passwordChecks.minLen);
				$("#prov_form_vcc_basic_security_authentication_passwordChecks_maxLen").val(passwordChecks.maxLen);
				$("#prov_form_vcc_basic_security_authentication_passwordChecks_expirationDays").val(passwordChecks.expirationDays);
				$("#prov_form_vcc_basic_security_authentication_passwordChecks_blockOnLoginFailures").val(passwordChecks.blockOnLoginFailures);
				$("#prov_form_vcc_basic_security_authentication_passwordChecks_history").val(passwordChecks.history);
				$("#prov_form_vcc_basic_security_authentication_passwordChecks_minUpperChars").val(passwordChecks.minUpperChars);
				$("#prov_form_vcc_basic_security_authentication_passwordChecks_minLowerChars").val(passwordChecks.minLowerChars);
				$("#prov_form_vcc_basic_security_authentication_passwordChecks_minSpecialChars").val(passwordChecks.minSpecialChars);
				$("#prov_form_vcc_basic_security_authentication_passwordChecks_minNumericChars").val(passwordChecks.minNumericChars);
				$("#prov_form_vcc_basic_security_authentication_passwordChecks_expirationNotificationDays").val(passwordChecks.expirationNotificationDays);
				$("#prov_form_vcc_basic_security_authentication_passwordChecks_changeOnFirstLogin").prop("checked", passwordChecks.changeOnFirstLogin);
				
				const mfa = vcc && vcc.Security && vcc.Security.mfa || {};
				if (mfa.enabled === "1") {
					$("#prov_form_vcc_basic_security_authentication_mfa_type").val(mfa.type);
					switch(mfa.type) {
						case "MAIL": {
							$("#prov_form_vcc_basic_security_authentication_mfa_mail_account").val(mfa.account);
							$("#prov_form_vcc_basic_security_authentication_mfa_mail_length").val(mfa.keyLength);
							$("#prov_form_vcc_basic_security_authentication_mfa_mail_expiration").val(mfa.expirationSeconds / 60);
							$("#prov_form_vcc_basic_security_authentication_mfa_mail_subject").val(mfa.subject);
							this.m_mailControls.mfa.set_body_parts(JSON.parse(mfa.body));
							break;
						}
					}
				}
				break;

			case "LDAP":
				const ldap = vcc && vcc.Security && vcc.Security.ldap || {};
				const $ldap_panel = $("#prov_form_vcc_basic_security_authentication_ldap_panel");

				const setLdapValues = (group)=> {
					let selector = jQUERY_SELECTOR_LDAP_GROUP_NAME(group);
					const $item = $ldap_panel.find(selector);
					$item.val(group);
					selector = jQUERY_SELECTOR_LDAP_GROUP_FILTER(group);
					const $filter = $ldap_panel.find(selector);
					$filter.val(ldap.groupFilters ? ldap.groupFilters[group] : "");
				};

				$("#prov_form_vcc_basic_security_authentication_ldap_server").val(ldap.server);
				$("#prov_form_vcc_basic_security_authentication_ldap_adminUserDN").val(ldap.adminUserDN);
				$("#prov_form_vcc_basic_security_authentication_ldap_useTLS").prop("checked", ldap.useTLS);
				$("#prov_form_vcc_basic_security_authentication_ldap_baseDN").val(ldap.baseDN);
				$("#prov_form_vcc_basic_security_authentication_ldap_userIdAttributte").val(ldap.userIdAttributte);
				$("#prov_form_vcc_basic_security_authentication_ldap_password").val(ldap.adminPassword);
				$("#prov_form_vcc_basic_security_authentication_ldap_sync_enabled").prop("checked", ldap.sync_enabled);
				if(ldap.sync_enabled) {
					$("#prov_form_vcc_basic_security_authentication_ldap_sync_timezone").val(ldap.timezone);
					$("#prov_form_vcc_basic_security_authentication_ldap_sync_language").val(ldap.language);
					$("#prov_form_vcc_basic_security_authentication_ldap_sync_country").val(ldap.country);
					this.fill_security_authentication_ldap_sync_areas(ldap.country);
					$("#prov_form_vcc_basic_security_authentication_ldap_sync_area").val(ldap.area);
					$("#prov_form_vcc_basic_security_authentication_ldap_sync_default_campaign").val(ldap.default_campaign);
					$("#prov_form_vcc_basic_security_authentication_ldap_sync_initial_state").val(ldap.initial_state);
					for (const group in ldap.groupFilters) {
						if (ldap.groupFilters.hasOwnProperty(group)) {
							setLdapValues(group);
						}
					}
				}
				else {
					setLdapValues("allusers");
				}
				break;

			case "OPENID":
				const openid = vcc && vcc.Security && vcc.Security.openid || {};
				$("#prov_form_vcc_basic_security_authentication_openid_configuration_url").val(openid.openid_configuration_url);
				$("#prov_form_vcc_basic_security_authentication_openid_sync_method").val(openid.sync_method);
				if(vcc.Security.openid.sync_method === "AZUREAD") {
					$("#prov_form_vcc_basic_security_authentication_azuread_sync_client_id").val(openid.client_id);
					$("#prov_form_vcc_basic_security_authentication_azuread_sync_client_secret").val(openid.adminPassword);
					$("#prov_form_vcc_basic_security_authentication_azuread_sync_enabled").prop("checked", openid.sync_enabled).trigger("change");
					$("#prov_form_vcc_basic_security_authentication_azuread_sync_timezone").val(openid.timezone);
					$("#prov_form_vcc_basic_security_authentication_azuread_sync_language").val(openid.language);
					$("#prov_form_vcc_basic_security_authentication_azuread_sync_country").val(openid.country);
					this.fill_security_authentication_azuread_sync_areas(openid.country);
					$("#prov_form_vcc_basic_security_authentication_azuread_sync_area").val(openid.area);
					$("#prov_form_vcc_basic_security_authentication_azuread_sync_default_campaign").val(openid.default_campaign);
					$("#prov_form_vcc_basic_security_authentication_azuread_sync_initial_state").val(openid.initial_state);
				}
				break;
		}
		this.toggle_security_authentication();
	};

	prov_form_vccs_basic.prototype.set_integrations = function(vcc)
	{
		const $helpdesk_integration = $("#prov_form_vcc_basic_helpdesk_integration_enabled");
		const $mas_integration = $("#prov_form_vcc_basic_mas_integration_enabled");
		const $wfm_integration = $("#prov_form_vcc_basic_wfm_integration_enabled");

		if (!vcc || !vcc.AppsIntegrations) {
			$helpdesk_integration.prop("checked", false).trigger("change");
			$mas_integration.prop("checked", false).trigger("change");
			$wfm_integration.prop("checked", false).trigger("change");
			return;
		}

		// HelpDesk
		if (vcc.AppsIntegrations.HelpDesk && vcc.AppsIntegrations.HelpDesk.Enabled === "1") {
			$helpdesk_integration.prop("checked", true).trigger("change");
			// notification
			if (vcc.AppsIntegrations.HelpDesk.Notifications) {
				const $helpdesk_notification_url = $("#prov_form_vcc_basic_helpdesk_integration_notification_url");
				$helpdesk_notification_url.val(vcc.AppsIntegrations.HelpDesk.Notifications.Url);
			}
			// server
			if (vcc.AppsIntegrations.HelpDesk.Server) {
				const $helpdesk_server_url = $("#prov_form_vcc_basic_helpdesk_integration_server_url");
				$helpdesk_server_url.val(vcc.AppsIntegrations.HelpDesk.Server.HOST);
			}
			// adm auth
			if (vcc.AppsIntegrations.HelpDesk.UserAuthentication) {
				// userid
				const $helpdesk_auth_adm_user_id = $("#prov_form_vcc_basic_helpdesk_integration_api_adm_user_auth_id");
				$helpdesk_auth_adm_user_id.val(vcc.AppsIntegrations.HelpDesk.UserAuthentication.User);
				// pwd
				const $helpdesk_auth_adm_pwd = $("#prov_form_vcc_basic_helpdesk_integration_api_adm_user_auth_pwd");
				$helpdesk_auth_adm_pwd.val(vcc.AppsIntegrations.HelpDesk.UserAuthentication.Password);
			}
			// flow auth
			if (vcc.AppsIntegrations.HelpDesk.ApiUserAuthentication) {
				// userid
				const $helpdesk_auth_flow_user_id = $("#prov_form_vcc_basic_helpdesk_integration_api_flow_user_auth_id");
				$helpdesk_auth_flow_user_id.val(vcc.AppsIntegrations.HelpDesk.ApiUserAuthentication.User);
				// pwd
				const $helpdesk_auth_flow_pwd = $("#prov_form_vcc_basic_helpdesk_integration_api_flow_user_auth_pwd");
				$helpdesk_auth_flow_pwd.val(vcc.AppsIntegrations.HelpDesk.ApiUserAuthentication.Password);
			}
			// user licences
			if (vcc.AppsIntegrations.HelpDesk.Licenses) {
				const $helpdesk_user_licenses = $("#prov_form_vcc_basic_helpdesk_integration_user_licenses");
				$helpdesk_user_licenses.val(vcc.AppsIntegrations.HelpDesk.Licenses.Users);
			}
		}
		else if (inConcert.app.hasLicenseSystem() && (!vcc.MAX_AppsIntegrations || !vcc.MAX_AppsIntegrations.HelpDesk || vcc.MAX_AppsIntegrations.HelpDesk === 0)) {
			this.frm.disable_control($helpdesk_integration, true, false);	
		}

		// M&S
		if (vcc.AppsIntegrations.MarketingAndSales && vcc.AppsIntegrations.MarketingAndSales.Enabled === "1") {
			$mas_integration.prop("checked", true).trigger("change");
			// notification
			if (vcc.AppsIntegrations.MarketingAndSales.Notifications) {
				const $mas_notification_url = $("#prov_form_vcc_basic_mas_integration_notification_url");
				$mas_notification_url.val(vcc.AppsIntegrations.MarketingAndSales.Notifications.Url);
			}
			// app render
			if (vcc.AppsIntegrations.MarketingAndSales.AppRender) {
				const $mas_app_url = $("#prov_form_vcc_basic_mas_integration_application_url");
				$mas_app_url.val(vcc.AppsIntegrations.MarketingAndSales.AppRender.Url);
			}
			// token
			if (vcc.AppsIntegrations.MarketingAndSales.AccessToken) {
				const $mas_token_url = $("#prov_form_vcc_basic_mas_integration_token_url");
				$mas_token_url.val(vcc.AppsIntegrations.MarketingAndSales.AccessToken.Url);
			}
			// API
			if (vcc.AppsIntegrations.MarketingAndSales.Api) {
				const $mas_api_url = $("#prov_form_vcc_basic_mas_integration_api_url");
				$mas_api_url.val(vcc.AppsIntegrations.MarketingAndSales.Api.Url);
			}
		}

		// WorkForce Management
		if (vcc.AppsIntegrations.WorkForceManagement && vcc.AppsIntegrations.WorkForceManagement.Enabled === "1") {
			$wfm_integration.prop("checked", true).trigger("change");
			// notification
			if (vcc.AppsIntegrations.WorkForceManagement.Notifications) {
				const $wfm_notification_url = $("#prov_form_vcc_basic_wfm_integration_notification_url");
				$wfm_notification_url.val(vcc.AppsIntegrations.WorkForceManagement.Notifications.Url);
			}
		}

		// Widgets
		for (let widget_id in this.m_widgets) {
			if (this.m_widgets.hasOwnProperty(widget_id)) {
				let widget_info = vcc.AppsIntegrations[widget_id];
				let widget_config_obj = this.get_widget_instance(widget_id);
				if (widget_info && widget_info.Enabled === "1") {
					const $widget_enabled = this.$get_widget_enabled(widget_id);
					if ($widget_enabled.length > 0) {
						$widget_enabled.prop("checked", true).trigger("change");
						if (typeof (widget_config_obj.set) === "function") {
							widget_config_obj.set(widget_info.Config);
						}
						else {
							let msg = FORMAT("prov_form_vccs_basic.prototype.set_integrations - The Widget '{0}' Provisioning class has to implement the 'set' method.", widget_id);
							console.log(msg);
							window.alert(msg);
						}
					}
				}
				else {
					if (typeof (widget_config_obj.clean_up) === "function") {
						widget_config_obj.clean_up();
					}
					else {
						let msg = FORMAT("prov_form_vccs_basic.prototype.set_integrations - The Widget '{0}' Provisioning class has to implement the 'clean_up' method.", widget_id);
						console.log(msg);
						window.alert(msg);
					}
				}
			}
		}
	};

	prov_form_vccs_basic.prototype.load_licensed_resources = function(resources)
	{
		this.set_license_max_resources(resources);
		this.set_licensed_channels(resources);
	};	

	prov_form_vccs_basic.prototype.set_licensed_channels = function(vcc)
	{
		let self = this;

		if (!vcc) {
			return;
		}
		$("#prov_form_vcc_basic_licences_section input[selector='1'][type='checkbox']").each(function(_, license) {
			const $checkbox = $(license);
			const field_channel_name = $checkbox.attr("channel_id") + "_licensed"; // CALL_licensed
			self.frm.readonly_control($checkbox, vcc[field_channel_name] === false /* disabled */, false /* is_required */);
		});
	};	

	prov_form_vccs_basic.prototype.set_licenses = function(vcc)
	{

		if (!vcc) {
			return;
		}
		$("#prov_form_vcc_basic_licences_section input[selector='1'][type='checkbox']").each(function(_, license) {
			const $checkbox = $(license);
			const field_channel_name = $checkbox.attr("channel_id") + "_licensed"; // CALL_licensed
			if(inConcert.app.hasLicenseSystem()) {
				$checkbox.prop("checked", vcc[field_channel_name] === true);
			}
			else {
				if (vcc[field_channel_name] === "1") {
					$checkbox.prop("checked", true);
				}
			}
		});
		if(inConcert.app.hasLicenseSystem()) {
			this.__set_total_max_license_resources(vcc);
			this.set_license_max_resources(vcc);
		}
	};

	prov_form_vccs_basic.prototype.set_recording_configuration = function(vcc)
	{
		const $call_automatic = $("#prov_form_vcc_basic_recording_call_automatic_recording_criteria");
		const $call_format = $("#prov_form_vcc_basic_recording_call_format");
		const $call_channels = $("#prov_form_vcc_basic_recording_call_channels");

		const $videocall_automatic = $("#prov_form_vcc_basic_recording_videocall_automatic_recording_criteria");
		const $videocall_format = $("#prov_form_vcc_basic_recording_videocall_format");
		const $videocall_mode = $("#prov_form_vcc_basic_recording_videocall_mode");

		const $voicemail_automatic = $("#prov_form_vcc_basic_recording_voicemail_automatic_recording_criteria");
		const $voicemail_format = $("#prov_form_vcc_basic_recording_voicemail_format");
		const $voicemail_channels = $("#prov_form_vcc_basic_recording_voicemail_channels");

		if (!vcc || !vcc.RecordingConfiguration ) {
			$call_automatic.val(this.m_defaultsRecordingConfiguration.CALL.automatic);
			$call_format.val(this.m_defaultsRecordingConfiguration.CALL.format);
			$call_channels.val(this.m_defaultsRecordingConfiguration.CALL.channels);

			$videocall_automatic.val(this.m_defaultsRecordingConfiguration.VIDEO_CALL.automaticc);
			$videocall_format.val(this.m_defaultsRecordingConfiguration.VIDEO_CALL.format);
			$videocall_mode.val(this.m_defaultsRecordingConfiguration.VIDEO_CALL.mode);

			$voicemail_automatic.val(this.m_defaultsRecordingConfiguration.VOICEMAIL.automatic);
			$voicemail_format.val(this.m_defaultsRecordingConfiguration.VOICEMAIL.format);
			$voicemail_channels.val(this.m_defaultsRecordingConfiguration.VOICEMAIL.channels);
			return;
		}

		vcc.RecordingConfiguration = JSON.parse(vcc.RecordingConfiguration);

		$call_automatic.val(vcc.RecordingConfiguration.CALL.automatic);
		$call_format.val(vcc.RecordingConfiguration.CALL.format);
		$call_channels.val(vcc.RecordingConfiguration.CALL.channels);

		$videocall_automatic.val(vcc.RecordingConfiguration.VIDEO_CALL.automatic);
		$videocall_format.val(vcc.RecordingConfiguration.VIDEO_CALL.format);
		$videocall_mode.val(vcc.RecordingConfiguration.VIDEO_CALL.mode);

		$voicemail_automatic.val(vcc.RecordingConfiguration.VOICEMAIL.automatic);
		$voicemail_format.val(vcc.RecordingConfiguration.VOICEMAIL.format);
		$voicemail_channels.val(vcc.RecordingConfiguration.VOICEMAIL.channels);
		
	};

	prov_form_vccs_basic.prototype.switch_tab = function(tab_id)
	{
		this.main_tabs.switch_tab(tab_id);
	};

	prov_form_vccs_basic.prototype.some_widgets_integration_was_configured_fn = function()
	{
		let is_configured = false;
		for (let widget_id in this.m_widgets) {
			if (this.m_widgets.hasOwnProperty(widget_id)) {
				const $widget_enabled = this.$get_widget_enabled(widget_id);
				if ($widget_enabled.length > 0) {
					const is_enabled = $widget_enabled.prop("checked");
					if (is_enabled) {
						is_configured = true;
						break;
					}
				}
			}
		}
		return is_configured;
	};

	prov_form_vccs_basic.prototype.some_integration_was_configured_fn = function()
	{
		// helpdesk
		if ($("#prov_form_vcc_basic_helpdesk_integration_enabled").prop("checked")) {
			return true;
		}
		// m&s
		if ($("#prov_form_vcc_basic_mas_integration_enabled").prop("checked")) {
			return true;
		}
		// wfm
		if ($("#prov_form_vcc_basic_wfm_integration_enabled").prop("checked")) {
			return true;
		}
		// widgets
		if (this.some_widgets_integration_was_configured_fn()) {
			return true;
		}
		// no tiene ninguna integracion configurada.
		return false;
	};

	prov_form_vccs_basic.prototype.get_validation_def = function(definitions = [])
	{
		const self = this;
		const isPredefinedBucket = $("#prov_form_vcc_basic_storage_location_params_S3BucketCheckBox").prop("checked");

		// verificaciones que determinan si se deben validar las secciones (habilitadas/deshabilitadas) .. -------------------------------------------------------------------------------------------
		const can_validate_helpdesk_integration_fn = function() {
			return $("#prov_form_vcc_basic_helpdesk_integration_enabled").prop("checked");
		};

		const can_validate_mas_integration_fn = function() {
			return $("#prov_form_vcc_basic_mas_integration_enabled").prop("checked");
		};

		const can_validate_wfm_integration_fn = function() {
			return $("#prov_form_vcc_basic_wfm_integration_enabled").prop("checked");
		};

		const can_validate_session_timeouts_fn = function() {
			return $("#prov_form_vcc_basic_session_timeouts_enabled").prop("checked");
		};

		const can_validate_security_captcha_fn = function() {
			if (!$("#prov_form_vcc_basic_security_captcha_enabled").prop("checked")) {
				return false;
			}
			const security_captcha_type = $("#prov_form_vcc_basic_security_captcha_type").val();
			return security_captcha_type === "CUSTOM";
		};

		const can_validate_security_authentication_basic_fn = function() {
			return $("#prov_form_vcc_basic_security_authentication_type").val() === "BASIC" ||
				$("#prov_form_vcc_basic_security_authentication_type").val() === "MFA";
		};

		const can_validate_security_authentication_mfa_mail_fn = function() {
			return $("#prov_form_vcc_basic_security_authentication_type").val() === "MFA" &&
				$("#prov_form_vcc_basic_security_authentication_mfa_type").val() === "MAIL";
		};

		const can_validate_security_authentication_ldap_fn = function() {
			return $("#prov_form_vcc_basic_security_authentication_type").val() === "LDAP";
		};

		const can_validate_security_authentication_azuread_fn = function() {
			return $("#prov_form_vcc_basic_security_authentication_type").val() === "OPENID" && $("#prov_form_vcc_basic_security_authentication_openid_sync_method").val() === "AZUREAD";
		};

		const can_validate_outgoing_batches_fn = function() {
			if (self.frm.is_new) {
				return false;
			}
			return self.m_outgoing_batches_channels.can_validate();
		};

		const can_validate_social_networks_fn = function() {
			if (self.frm.is_new) {
				return false;
			}
			return self.m_social_networks_app_channel.can_validate();
		};

		const can_validate_screen_recording_fn = function() {
			return $("#prov_form_vcc_basic_screen_recording_enabled").prop("checked");
		};

		const can_validate_video_config_fn = function() {
			return $("#prov_form_vcc_basic_video_config_enabled").prop("checked");
		};

		const can_validate_drafts_config_fn = function() {
			return $("#prov_form_vcc_basic_drafts_configuration_enabled").prop("checked");
		};

		const can_validate_phone_validation_config_fn = function() {
			return $("#prov_form_vcc_basic_phone_validation_configuration_enabled").prop("checked");
		};

		const can_validate_audit_config_fn = function() {
			return $("#prov_form_vcc_basic_audit_cleanup_enabled").prop("checked");
		};

		const can_validate_api_sdk_fn = function() {
			return $("#prov_form_vcc_basic_api_sdk_enabled").prop("checked");
		};

		const can_validate_databases = function() {
			return $("#prov_form_vcc_basic_databases_configuration_panel_enabled").prop("checked");
		};

		const can_validate_telephony_i6_fn = function() {
			let vcc = self.frm.get_entity_instance();
			if (vcc && vcc.HasAllegroIntegration) {
				return false;
			}
			return true;
		};

		const can_validate_config_limits = function() {
			let vcc = self.frm.get_entity_instance();
			let max_campaigns = $("#prov_form_vcc_basic_config_limits_max_campaigns").val();
			let max_campaigns_users = $("#prov_form_vcc_basic_config_limits_max_campaigns_users").val();
			let max_att_levels = $("#prov_form_vcc_basic_config_limits_max_att_levels").val();
			let max_att_levels_users = $("#prov_form_vcc_basic_config_limits_max_att_levels_users").val();
			let max_workgroups = $("#prov_form_vcc_basic_config_limits_max_workgroups").val();
			let max_workgroups_users = $("#prov_form_vcc_basic_config_limits_max_workgroups_users").val();
			let has_data = max_campaigns !== "" ||  max_campaigns_users !== "" ||  max_att_levels !== "" ||  max_att_levels_users !== "" ||  max_workgroups !== "" ||  max_workgroups_users !== "";
			
			// Si es edicion de un vcc que no tiene configurado limites y no agrega ningun, no se valida.
			// Solo se valida si agregaron algun dato a uno de los input, si es nuevo el vcc o si es viejo y ya tenia limites configurados con anterioridad.
			if (self.frm.is_new || (vcc && vcc.ConfigLimits) || has_data) {
				return true;
			}
			return false;
		};

		// callbacks on_errors .. -------------------------------------------------------------------------------------------
		const on_basic_error = function() {
			self.switch_tab("basic");
		};

		const on_license_error = function() {
			self.switch_tab("licenses");
		};

		const on_security_error = function() {
			self.switch_tab("security");
		};

		const on_integrations_error = function(callback) {
			callback = typeof callback === "function" ? callback : $.noop;
			self.switch_tab("integrations");
			callback();
		};

		const on_others_error = function() {
			self.switch_tab("others");
		};

		const parseIntRegEx = function(val) {
			return parseInt((/^-?\d+$/.test(val) && val) || 0);
		};

		const sumOfMinimums = function() {
			const minOfLengths = 4; //Minimum length for both minLen and maxLen
			const minLowerChars = parseIntRegEx($("#prov_form_vcc_basic_security_authentication_passwordChecks_minLowerChars").val());
			const minUpperChars = parseIntRegEx($("#prov_form_vcc_basic_security_authentication_passwordChecks_minUpperChars").val());
			const minSpecialChars = parseIntRegEx($("#prov_form_vcc_basic_security_authentication_passwordChecks_minSpecialChars").val());
			const minNumericChars = parseIntRegEx($("#prov_form_vcc_basic_security_authentication_passwordChecks_minNumericChars").val());
			const sumOfMin = minLowerChars + minUpperChars + minSpecialChars + minNumericChars;
			return sumOfMin < minOfLengths ? minOfLengths : sumOfMin;
		};

		const getMinLen = function() {
			const minOfLengths = sumOfMinimums(); //Minimum length for both minLen and maxLen
			const minLen = parseIntRegEx($("#prov_form_vcc_basic_security_authentication_passwordChecks_minLen").val());
			return minLen < minOfLengths ? minOfLengths : minLen;
		};

		const getMaxExpirationNotificationDays = function() {
			const $expirationDays = $("#prov_form_vcc_basic_security_authentication_passwordChecks_expirationDays");
			const expirationDays = parseInt($expirationDays.val());
			return expirationDays - 1 < 0 ? 0 : expirationDays - 1;
		};

		const customValidateRangeConfigurationsLimits = function($control, limit_key, range_msg_error, maxNumberConfigsAvailable, old_vcc_config_limits) {

			let response = {
				success: true
			};

			let control_value = parseInt($control.val());

			//Si no se tienen los limites disponibles, se devuelve error.
			if(!maxNumberConfigsAvailable || maxNumberConfigsAvailable === "ERROR_SYSTEM_CONFIG" || typeof maxNumberConfigsAvailable[limit_key] !== "number"){
				response.success = false;
				response.message = "<%=_T('Error getting environment configurations limits.')%>" +" "+"<%=_T('Retrieve the vcc info again in a few minutes...')%>";
				return response;
			}

			old_vcc_config_limits = old_vcc_config_limits || {
				MaxCampaigns : 0,
				MaxCampaignsUsers : 0,
				MaxAttsLevel : 0,
				MaxAttsLevelUsers : 0,
				MaxWorkgroups : 0,
				MaxWorkgroupsUsers : 0
			};

			let to_range = old_vcc_config_limits[limit_key] + maxNumberConfigsAvailable[limit_key];
			if(control_value < 0 || control_value > to_range){
				response.success = false;
				response.message = FORMAT(range_msg_error, 0, to_range);
				return response;
			}

			return response;
		};


		// array con todas las definiciones que se validaran en "conf_form_validation" .. -------------------------------------------------------------------------------------------
		definitions.push(
		// Basic Validations
			{
				control_id: "prov_form_vcc_basic_name",
				messages: {
					required: "<%=_T('Please enter the Contact Center Name.')%>",
					id_without_dots: "<%=_T('Only letters, numbers and dashes are allowed in the Contact Center Name.')%>",
					custom: {
						type : "custom",
						valid: function() {
							let response = {
								success: true
							};
							const $name =  $("#prov_form_vcc_basic_name");
							let name = $name.val();
							let contactCenterNameOptions = self.frm.get_contact_center_name_options();	
							let contactCenterNameLenMsg = FORMAT("<%=_T('The Contact Center Name length must be between {0} and {1}.')%>", contactCenterNameOptions.min_length, contactCenterNameOptions.max_length );
							if (!self.validate_field($name, "string_max_length", "", contactCenterNameOptions) || !self.validate_field($name, "string_min_length", "", contactCenterNameOptions)) {
								response.success = false;
								response.message = contactCenterNameLenMsg;
							}
							else if(name.toLowerCase() === "systemlicense") {								
								response.success = false;
								response.message = "<%=_T('Invalid Contact Center Name.')%>";
							}else{
									let validation_result = inConcert.Utils.validate(name.trim(), "number");
									if (validation_result.success) {
										response.success = false;
										response.message = "<%=_T('The contact center name cannot be all numbers.')%>";
									}
							}

							return response;
						}
					},
				},
				on_error: on_basic_error
			},
			{
				control_id: "prov_form_vcc_basic_desc",
				messages: {
					required: "<%=_T('Please enter the Contact Center description.')%>"
				},
				on_error: on_basic_error
			},
			{
				control_id: "prov_form_vcc_basic_free_seconds_after_interaction",
				messages: {
					custom: {
						type : "custom",
						valid: function() {
							let response = {
								success: true
							};
							const $free_seconds =  $("#prov_form_vcc_basic_free_seconds_after_interaction");
							let free_seconds = $free_seconds.val();
							if (free_seconds === "") {
								return response;
							}
							const range = {between: {from: 0, to: 180}};
							if (!self.validate_field($free_seconds, "range", "", range)) {
								response.success = false;
								response.message = FORMAT("<%=_T('Contact Center Free Time After Conversation value out of range. Please enter value between {0} and {1}.')%>", range.between.from, range.between.to );
							}
							return response;
						}
					}
				},
				on_error: on_basic_error
			},
			{
				control_id: "prov_form_vcc_basic_contact_email",
				messages: {
					email: "<%=_T('Please enter a Valid Contact Email.')%>"
				},
				on_error: on_basic_error
			},
		// User Validations
			{
				control_id: "prov_form_vcc_basic_user_id",
				can_validate: function() {
					if (!self.frm.is_new) {
						if (self.m_vcc_has_users) {
							return false;
						}
					}
					return true;
				},
				messages: {
					custom: {
						type : "custom",
						valid: function() {
							const $user = $("#prov_form_vcc_basic_user_id");
							return m_commonValidations.users.Id.validateFn(self, $user);
						}
					}
				},
				on_error: on_basic_error
			},
			{
				control_id: "prov_form_vcc_basic_user_password",
				can_validate: function() {
					const $password =  $("#prov_form_vcc_basic_user_password");
					if (!self.frm.is_new && self.m_vcc_has_users && $password.val() === self.m_original_user_password) {
						if (self.m_vcc_has_users) {
							return false;
						}
					}
					return true;
				},
				messages: {
					custom: {
						type : "custom",
						valid: function() {
							const $password = $("#prov_form_vcc_basic_user_password");
							return m_commonValidations.users.Password.validateFn(self, $password);
						}
					}
				},
				on_error: on_basic_error
			},
			{
				control_id: "prov_form_vcc_basic_user_email",
				can_validate: function() {
					if (!self.frm.is_new) {
						if (self.m_vcc_has_users) {
							return false;
						}
					}
					return can_validate_helpdesk_integration_fn();
				},
				messages: {
					required: "<%=_T('Please enter the User e-mail address.')%>",
					email   : "<%=_T('Please enter a Valid User e-mail.')%>"
				},
				on_error: on_basic_error
			},

		// Custom Security Captcha
			{
			// public key
				control_id: "prov_form_vcc_basic_security_captcha_public_key",
				can_validate: can_validate_security_captcha_fn,
				messages: {
					required: "<%=_T('Public Key is required for Security Captcha.')%>"
				},
				on_error: on_security_error
			},
			{
			// Secret key
				control_id: "prov_form_vcc_basic_security_captcha_secret_key",
				can_validate: can_validate_security_captcha_fn,
				messages: {
					required: "<%=_T('Secret Key is required for Security Captcha.')%>"
				},
				on_error: on_security_error
			},
		// Basic authentication
			{
			// Minimum upper character count
				control_id: "prov_form_vcc_basic_security_authentication_passwordChecks_minUpperChars",
				can_validate: can_validate_security_authentication_basic_fn,
				messages: {
					required: "<%=_T('Minimum upper character count is required.')%>",
					range: {
						between: { from: 0, to: 5 },
						message: "<%=_T('Minimum upper character count is out of range. Please enter a value between {0} and {1}.')%>"
					}
				},
				on_error: on_security_error
			},
			{
			// Minimum lower character count
				control_id: "prov_form_vcc_basic_security_authentication_passwordChecks_minLowerChars",
				can_validate: can_validate_security_authentication_basic_fn,
				messages: {
					required: "<%=_T('Minimum lower character count is required.')%>",
					range: {
						between: { from: 0, to: 5 },
						message: "<%=_T('Minimum lower character count is out of range. Please enter a value between {0} and {1}.')%>"
					}
				},
				on_error: on_security_error
			},
			{
			// Minimum special character count
				control_id: "prov_form_vcc_basic_security_authentication_passwordChecks_minSpecialChars",
				can_validate: can_validate_security_authentication_basic_fn,
				messages: {
					required: "<%=_T('Minimum special character count is required.')%>",
					range: {
						between: { from: 0, to: 5 },
						message: "<%=_T('Minimum special character count is out of range. Please enter a value between {0} and {1}.')%>"
					}
				},
				on_error: on_security_error
			},
			{
			// Minimum numeric character count
				control_id: "prov_form_vcc_basic_security_authentication_passwordChecks_minNumericChars",
				can_validate: can_validate_security_authentication_basic_fn,
				messages: {
					required: "<%=_T('Minimum numeric character count is required.')%>",
					range: {
						between: { from: 0, to: 5 },
						message: "<%=_T('Minimum numeric character count is out of range. Please enter a value between {0} and {1}.')%>"
					}
				},
				on_error: on_security_error
			},
			{
			// Minimum password length
				control_id: "prov_form_vcc_basic_security_authentication_passwordChecks_minLen",
				can_validate: can_validate_security_authentication_basic_fn,
				messages: {
					required: "<%=_T('Minimum password length is required.')%>",
					range: {
						between: { from: sumOfMinimums(), to: 20 },
						message: "<%=_T('Minimum password length is out of range. Please enter a value between {0} and {1}.')%>"
					}
				},
				on_error: on_security_error
			},
			{
			// Maximum password length
				control_id: "prov_form_vcc_basic_security_authentication_passwordChecks_maxLen",
				can_validate: can_validate_security_authentication_basic_fn,
				messages: {
					required: "<%=_T('Maximum password length is required.')%>",
					range: {
						between: { from: getMinLen(), to: 20 },
						message: "<%=_T('Maximum password length is out of range. Please enter a value between {0} and {1}.')%>"
					}
				},
				on_error: on_security_error
			},
			{
			// Password expiration
				control_id: "prov_form_vcc_basic_security_authentication_passwordChecks_expirationDays",
				can_validate: can_validate_security_authentication_basic_fn,
				messages: {
					required: "<%=_T('Password expiration is required (0 = disabled).')%>",
					range: {
						between: { from: 0, to: 60 },
						message: "<%=_T('Password expiration is out of range. Please enter a value between {0} and {1} days (0 = disabled).')%>"
					}
				},
				on_error: on_security_error
			},
			{
			// Password expiration notification
				control_id: "prov_form_vcc_basic_security_authentication_passwordChecks_expirationNotificationDays",
				can_validate: function() {
					return can_validate_security_authentication_basic_fn() && $("#prov_form_vcc_basic_security_authentication_passwordChecks_expirationDays").val() > 0;
				},
				messages: {
					required: "<%=_T('Password expiration notification is required (0 = disabled).')%>",
					range: {
						between: { from: 0, to: getMaxExpirationNotificationDays() },
						message: "<%=_T('Password expiration notification is out of range. Please enter a value between {0} and {1} days (0 = disabled).')%>"
					}
				},
				on_error: on_security_error
			},			
			{
			// Block on login failures
				control_id: "prov_form_vcc_basic_security_authentication_passwordChecks_blockOnLoginFailures",
				can_validate: can_validate_security_authentication_basic_fn,
				messages: {
					required: "<%=_T('Block on login failures is required (0 = disabled).')%>",
					range: {
						between: { from: 0, to: 10 },
						message: "<%=_T('Block on login failures is out of range. Please enter a value between {0} and {1} (0 = disabled).')%>"
					}
				},
				on_error: on_security_error
			},
			{
			// Password history
				control_id: "prov_form_vcc_basic_security_authentication_passwordChecks_history",
				can_validate: can_validate_security_authentication_basic_fn,
				messages: {
					required: "<%=_T('Password history is required (0 = disabled).')%>",
					range: {
						between: { from: 0, to: 20 },
						message: "<%=_T('Password history is out of range. Please enter a value between {0} and {1} (0 = disabled).')%>"
					}
				},
				on_error: on_security_error
			},
		// Security MFA
			{
			// mail account
				control_id: "prov_form_vcc_basic_security_authentication_mfa_mail_account",
				can_validate: can_validate_security_authentication_mfa_mail_fn,
				messages: {
					required: "<%=_T('An outgoing account is required.')%>"
				},
				on_error: on_security_error
			},
			{
			// Secret key
				control_id: "prov_form_vcc_basic_security_authentication_mfa_mail_length",
				can_validate: can_validate_security_authentication_mfa_mail_fn,
				messages: {
					required: "<%=_T('Security code length is required.')%>",
					range: {
						between: { from: 4, to: 10 },
						message: "<%=_T('Security code length is out of range. Please enter a value between {0} and {1}.')%>"
					}
				},
				on_error: on_security_error
			},
			{
			// Expiration minutes
				control_id: "prov_form_vcc_basic_security_authentication_mfa_mail_expiration",
				can_validate: can_validate_security_authentication_mfa_mail_fn,
				messages: {
					required: "<%=_T('Expiration minutes are required.')%>",
					range: {
						between: { from: 5, to: 30 },
						message: "<%=_T('Expiration minutes are out of range. Please enter a value between {0} and {1} minutes.')%>"
					}
				},
				on_error: on_security_error
			},
			{
			// Subject
				control_id: "prov_form_vcc_basic_security_authentication_mfa_mail_subject",
				can_validate: can_validate_security_authentication_mfa_mail_fn,
				messages: {
					required: "<%=_T('Security e-mail subject is required.')%>"
				},
				on_error: on_security_error
			},
			{
			// Body
				control_id: "prov_form_vcc_basic_security_authentication_mfa_mail_body",
				can_validate: can_validate_security_authentication_mfa_mail_fn,
				messages: {
					valid: function() {
						let response = {
							success: false
						};
						self.m_mailControls.mfa.validate(function(validation_result) {
							response = validation_result;
						});
						return response;
					}
				},
				on_error: on_security_error
			},
		// Licencias
		 	{
		 		control_id: "prov_form_vcc_basic_company",
				can_validate: function() {
					return inConcert.app.hasLicenseSystem() && self.company_is_editable && this.company_is_required;
				},
				messages: {
					required: "<%=_T('Please enter the Company.')%>",
					remote: {
						type: "remote",
						message: "<%=_T('The company is invalid')%>",
						timeout_ms: 5000,
						uri: function(company) {
							return inConcert.app.getBaseDomain() + "/inconcert/apps/provisioning/vccs/validate_company/" +
								encodeURIComponent(company);
						},
						success: function(response, callback)
						{
							var success = response && response.company_is_valid;
							callback(success);
						}
					}
				},
				on_error: on_basic_error
			},
			{
				control_id: "prov_form_vcc_basic_online_users",
				can_validate: function() {
					return inConcert.app.hasLicenseSystem();
				},
				messages: {
					required: "<%=_T('Please enter the Contact Center Online Users.')%>",
					number  : "<%=_T('Only numbers are allowed in the Contact Center Online Users.')%>",
					range: {
						type   : "range",
						between: {from: 0, to: self.MaxOnlineUsers},
						message: "<%=_T('Contact Center Online Users allowed values between {0} to {1}.')%>"
					}
		 		},
				on_error: on_license_error
		 	},
			{
				control_id: "prov_form_vcc_basic_custom_metrics_limit",
				can_validate: function() {
					return inConcert.app.hasLicenseSystem();
				},
				messages: {
					range: {
						type   : "range",
						between: {from: 0, to: self.MAX_CustomMetricsLimit},
						message: "<%=_T('Contact Center Custom Metrics Limit allowed values between {0} to {1}.')%>"
					}
				},
				on_error: on_license_error
			},
			{
			 	control_id: "prov_form_vcc_basic_custom_filters_limit",
				can_validate: function() {
					return inConcert.app.hasLicenseSystem();
				},
				messages: {
					range: {
						type   : "range",
						between: {from: 0, to: self.MAX_CustomFiltersLimit},
						message: "<%=_T('Contact Center Custom Filters Limit allowed values between {0} to {1}.')%>"
					}
				},
				on_error: on_license_error
			},
			{
				control_id: "prov_form_vcc_basic_export_space_quota",
				can_validate: function() {
					return inConcert.app.hasLicenseSystem();
				},
				messages: {
					range: {
						type   : "range",
						between: {from: 0, to: self.MAX_ExportSpaceQuota},
						message: "<%=_T('Contact Center Export Space Quota allowed values between {0} to {1}.')%>"
					}
				},
				on_error: on_license_error
			},
			{
				control_id: "prov_form_vcc_basic_shared_files_space_quota",
				can_validate: function() {
					return inConcert.app.hasLicenseSystem();
				},
				messages: {
					range: {
						type   : "range",
						between: {from: 0, to: self.MAX_SharedFilesSpaceQuota},
						message: "<%=_T('Contact Center Shared Files Space Quota allowed values between {0} to {1}.')%>"
					}
				},
				on_error: on_license_error
			},
		// HelpDesk Integrations Validations
			{
			// notification url
				control_id: "prov_form_vcc_basic_helpdesk_integration_notification_url",
				can_validate: can_validate_helpdesk_integration_fn,
				messages: {
					required: "<%=_T('Notification URL is required for HelpDesk Integration.')%>"
				},
				on_error: on_integrations_error
			},
			// server url
			{
				control_id: "prov_form_vcc_basic_helpdesk_integration_server_url",
				can_validate: can_validate_helpdesk_integration_fn,
				messages: {
					required: "<%=_T('Application URL is required for HelpDesk Integration.')%>"
				},
				on_error: on_integrations_error
			},
		// Adm auth api
			// adm auth - user Id
			{
				control_id: "prov_form_vcc_basic_helpdesk_integration_api_adm_user_auth_id",
				can_validate: can_validate_helpdesk_integration_fn,
				messages: {
					required: "<%=_T('Administration API Authorization User is required for HelpDesk Integration.')%>",
					id:       "<%=_T('Only letters, numbers, dashes and dots are allowed in the HelpDesk Administration API Authorization User.')%>"
				},
				on_error: on_integrations_error
			},
			// adm auth - pwd
			{
				control_id: "prov_form_vcc_basic_helpdesk_integration_api_adm_user_auth_pwd",
				can_validate: can_validate_helpdesk_integration_fn,
				messages: {
					required: "<%=_T('Administration API Authorization Password is required for HelpDesk Integration.')%>"
				},
				on_error: on_integrations_error
			},
		// Flow auth api
			// flow auth - user Id
			{
				control_id: "prov_form_vcc_basic_helpdesk_integration_api_flow_user_auth_id",
				can_validate: can_validate_helpdesk_integration_fn,
				messages: {
					required: "<%=_T('API Authorization User is required for HelpDesk Integration.')%>",
					id:       "<%=_T('Only letters, numbers, dashes and dots are allowed in the HelpDesk API Authorization User.')%>"
				},
				on_error: on_integrations_error
			},
			// flow auth - pwd
			{
				control_id: "prov_form_vcc_basic_helpdesk_integration_api_flow_user_auth_pwd",
				can_validate: can_validate_helpdesk_integration_fn,
				messages: {
					required: "<%=_T('API Authorization Password is required for HelpDesk Integration.')%>"
				},
				on_error: on_integrations_error
			},
			// user licenses
			{
				control_id: "prov_form_vcc_basic_helpdesk_integration_user_licenses",
				can_validate: function() {
					return can_validate_helpdesk_integration_fn;
				},
				messages: {
					required: "<%=_T('User Licenses is required for HelpDesk Integration.')%>",
					range: {
						between: {from: 0, to: (self.MAX_AppsIntegrations && self.MAX_AppsIntegrations.HelpDesk) || 0 },
						message: "<%=_T('User Licenses is value out of range. Please enter value between {0} and {1}.')%>"
					}
				},
				on_error: on_integrations_error
			},

		// M&S Integrations Validations

			// notification url
			{
				control_id: "prov_form_vcc_basic_mas_integration_notification_url",
				can_validate: can_validate_mas_integration_fn,
				messages: {
					required: "<%=_T('Notification URL is required for M&S Integration.')%>"
				},
				on_error: on_integrations_error
			},
			// app url
			{
				control_id: "prov_form_vcc_basic_mas_integration_application_url",
				can_validate: can_validate_mas_integration_fn,
				messages: {
					required: "<%=_T('Application URL is required for M&S Integration.')%>"
				},
				on_error: on_integrations_error
			},
			// token url
			{
				control_id: "prov_form_vcc_basic_mas_integration_token_url",
				can_validate: can_validate_mas_integration_fn,
				messages: {
					required: "<%=_T('Access Token URL is required for M&S Integration.')%>"
				},
				on_error: on_integrations_error
			},
			// API url
			{
				control_id: "prov_form_vcc_basic_mas_integration_api_url",
				can_validate: can_validate_mas_integration_fn,
				messages: {
					required: "<%=_T('API URL is required for M&S Integration.')%>"
				}
			},

		// WorkForce Management Integrations Validations

			// notification url
			{
				control_id: "prov_form_vcc_basic_wfm_integration_notification_url",
				can_validate: can_validate_wfm_integration_fn,
				messages: {
					required: "<%=_T('Notification URL is required for WorkForce Management Integration.')%>"
				},
				on_error: on_integrations_error
			},

		// Native Telephony Configuration
			{
				control_id: "prov_form_vcc_basic_telephony_dial_time",
				can_validate: can_validate_telephony_i6_fn,
				messages: {
					range: {
						between: {from: 10, to: 120},
						message: "<%=_T('Telephony configuration. Please enter dial time value between {0} and {1}.')%>"
					}
				},
				on_error: on_others_error
			},
			{
				control_id: "prov_form_vcc_basic_telephony_max_manual_rate",
				can_validate: can_validate_telephony_i6_fn,
				messages: {
					range: {
						between: {from: 1, to: 20},
						message: "<%=_T('Telephony configuration. Please enter max manual rate value between {0} and {1}.')%>"
					}
				},
				on_error: on_others_error
			},

		// API SDK
			{
				control_id: "prov_form_vcc_basic_api_sdk_enabled",
				can_validate: function() {
					let is_configured = self.some_integration_was_configured_fn();
					if (is_configured) {
						return true;
					}
					return false;
				},
				messages: {
					custom: {
						type : "custom",
						valid: function() {
							let response = {
								success: true
							};
							const is_enabled_api_sdk =  $("#prov_form_vcc_basic_api_sdk_enabled").prop("checked");
							if (is_enabled_api_sdk) {
								return response;
							}
							response.success = false;
							response.message = "<%=_T('If you enable Integrations you must also configure the API SDK on the VCC.')%>";
							return response;
						}
					}
				},
				on_error: on_others_error
			},
			{
				control_id: "prov_form_vcc_basic_api_sdk_low_rate_max_requests",
				can_validate: can_validate_api_sdk_fn,
				messages: {
					required: "<%=_T('API SDK - rate limit: Low max request is required.')%>",
					range: {
						between: {from: 1, to: null},
						message: "<%=_T('API SDK - Low rate limit error. The minimum value allowed is {0}.')%>"
					}
				},
				on_error: on_others_error
			},
			{
				control_id: "prov_form_vcc_basic_api_sdk_medium_light_rate_max_requests",
				can_validate: can_validate_api_sdk_fn,
				messages: {
					required: "<%=_T('API SDK - rate limit: MediumLight max request is required.')%>",
					range: {
						between: {from: 1, to: null},
						message: "<%=_T('API SDK - MediumLight rate limit error. The minimum value allowed is {0}.')%>"
					}
				},
				on_error: on_others_error
			},
			{
				control_id: "prov_form_vcc_basic_api_sdk_medium_rate_max_requests",
				can_validate: can_validate_api_sdk_fn,
				messages: {
					required: "<%=_T('API SDK - rate limit: Medium max request is required.')%>",
					range: {
						between: {from: 1, to: null},
						message: "<%=_T('API SDK - Medium rate limit error. The minimum value allowed is {0}.')%>"
					}
				},
				on_error: on_others_error
			},
			{
				control_id: "prov_form_vcc_basic_api_sdk_high_rate_max_requests",
				can_validate: can_validate_api_sdk_fn,
				messages: {
					required: "<%=_T('API SDK - rate limit: High max request is required.')%>",
					range: {
						between: {from: 1, to: null},
						message: "<%=_T('API SDK - High rate limit error. The minimum value allowed is {0}.')%>"
					}
				},
				on_error: on_others_error
			},

		// Screen Recording Port
			{
				control_id: "prov_form_vcc_basic_screen_recording_port",
				can_validate: can_validate_screen_recording_fn,
				messages: {
					required: "<%=_T('Screen Recording Port is required.')%>",
					range: {
						between: {from: 1024, to: 49151},
						message: "<%=_T('Screen Recording Port is value out of range. Please enter value between {0} and {1}.')%>"
					}
				},
				on_error: on_others_error
			},
		// Screen Recording Fps
			{
				control_id: "prov_form_vcc_basic_screen_recording_fps",
				can_validate: can_validate_screen_recording_fn,
				messages: {
					required: "<%=_T('Screen Recording Fps is required.')%>",
					range: {
						between: {from: 5, to: 30},
						message: "<%=_T('Screen Recording Fps is value out of range. Please enter value between {0} and {1}.')%>"
					}
				},
				on_error: on_others_error
			},
		// Screen Recording Bitrate
			{
				control_id: "prov_form_vcc_basic_screen_recording_bitrate",
				can_validate: can_validate_screen_recording_fn,
				messages: {
					required: "<%=_T('Screen Recording Bitrate is required.')%>",
					range: {
						between: {from: 96000, to: 14000000},
						message: "<%=_T('Screen Recording Bitrate is value out of range. Please enter value between {0} and {1}.')%>"
					}
				},
				on_error: on_others_error
			},
		// Video Bitrate (webCall)
			{
				control_id: "prov_form_vcc_basic_video_bitrate",
				can_validate: can_validate_video_config_fn,
				messages: {
					required: "<%=_T('Video Bitrate is required.')%>",
					range: {
						between: {from: 100, to: 14000000},
						message: "<%=_T('Video Bitrate is value out of range. Please enter value between {0} and {1}.')%>"
					}
				},
				on_error: on_others_error
			},
		//Audits
			{
				control_id: "prov_form_vcc_basic_audit_days",
				can_validate: can_validate_audit_config_fn,
				messages: {
					required: "<%=_T('Period for Clean up is required.')%>",
					range: {
						between: {from: 1, to: 365},
						message: "<%=_T('Period value out of range. Please enter value between {0} and {1}.')%>"
					}
				},
				on_error: on_others_error
			},

		// Maximum Session Timeout
			{
			// MOBILE
				control_id: "prov_form_vcc_basic_session_timeouts_mobile",
				can_validate: can_validate_session_timeouts_fn,
				messages: {
					required: "<%=_T('Mobile Maximum Session Timeout is required.')%>",
					range: {
						between: {from: 30, to: 10800},
						message: "<%=_T('Mobile Maximum Session Timeout is value out of range. Please enter value between {0} and {1} seconds.')%>"
					}
				},
				on_error: on_others_error
			},

		// Storage Configurations
			{
				control_id: "prov_form_vcc_basic_storage_location_params_S3PubKey",
				can_validate: function() {
					return inConcert.app.hasRepositoryService() && $("#prov_form_vcc_basic_storage_location").val() === "S3";
				},
				messages: {
					required: "<%=_T('Please enter the S3 Public Key.')%>"
				},
				on_error: on_others_error
			},
			{
				control_id: "prov_form_vcc_basic_storage_location_params_S3PrivKey",
				can_validate: function() {
					return inConcert.app.hasRepositoryService() && $("#prov_form_vcc_basic_storage_location").val() === "S3";
				},
				messages: {
					required: "<%=_T('Please enter the S3 Private Key.')%>"
				},
				on_error: on_others_error
			},
			{
				control_id: "prov_form_vcc_basic_storage_location_params_S3Bucket",
				can_validate: function() {
					return inConcert.app.hasRepositoryService() && $("#prov_form_vcc_basic_storage_location").val() === "S3" && isPredefinedBucket;
				},
				messages: {
					required: "<%=_T('Please enter the name of the S3 predefined bucket.')%>"
				},
				on_error: on_others_error
			},

		// Storage Lifecycle
            {
				control_id: "prov_form_vcc_basic_storage_s3_files_lifecycle",
				can_validate: function() {
					return !isPredefinedBucket;
				},
				messages: {
					required: "<%=_T('Please enter the lifecycle of S3 files.')%>",
					range: {
						between: {from: 1, to: 36500},
						message: "<%=_T('S3 files lifecycle configuration. Please enter lifecycle value between {0} and {1}.')%>"
					}
				},
				on_error: on_others_error
			},
			{
				control_id: "prov_form_vcc_basic_storage_s3_meta_lifecycle",
				can_validate: function() {
					return true;
				},
				messages: {
					required: "<%=_T('Please enter the lifecycle of S3 metadata.')%>",
					range: {
						between: {from: 1, to: 36500},
						message: "<%=_T('S3 Metadata lifecycle configuration. Please enter lifecycle value between {0} and {1}.')%>"
					}
				},
				on_error: on_others_error
			},
			{
				control_id: "prov_form_vcc_basic_storage_call_files_lifecycle",
				can_validate: function() {
					return true;
				},
				messages: {
					required: "<%=_T('Please enter the lifecycle of Cassandra call files.')%>",
					range: {
						between: {from: 1, to: 90},
						message: "<%=_T('Cassandra call files lifecycle configuration. Please enter lifecycle value between {0} and {1}.')%>"
					}
				},
				on_error: on_others_error
			},
			{
				control_id: "prov_form_vcc_basic_storage_email_files_lifecycle",
				can_validate: function() {
					return true;
				},
				messages: {
					required: "<%=_T('Please enter the lifecycle of Cassandra e-mail files.')%>",
					range: {
						between: {from: 1, to: 90},
						message: "<%=_T('Cassandra e-mail files lifecycle configuration. Please enter lifecycle value between {0} and {1}.')%>"
					}
				},
				on_error: on_others_error
			},
			{
				control_id: "prov_form_vcc_basic_storage_other_files_lifecycle",
				can_validate: function() {
					return true;
				},
				messages: {
					required: "<%=_T('Please enter lifecycle of the other Cassandra files.')%>",
					range: {
						between: {from: 1, to: 90},
						message: "<%=_T('Cassandra Other files lifecycle configuration. Please enter lifecycle value between {0} and {1}.')%>"
					}
				},
				on_error: on_others_error
			},
            
		// Outgoing Batches Limit
			{
				control_id: "prov_form_vcc_basic_outgoing_batches_section",
				can_validate: can_validate_outgoing_batches_fn,
				messages: {
					custom: {
						type : "custom",
						valid: function() {
							return self.m_outgoing_batches_channels.validate();
						}
					}
				},
				on_error: on_others_error
			},

		// Social Networks Post Limit
			{
				control_id: "prov_form_vcc_basic_social_networks_section",
				can_validate: can_validate_social_networks_fn,
				messages: {
					custom: {
						type : "custom",
						valid: function() {
							return self.m_social_networks_app_channel.validate();
						}
					}
				},
				on_error: on_others_error
			},
			// Databases
			{
				control_id: "prov_form_vcc_basic_databases_number",
				can_validate: can_validate_databases,
				messages: {
					required: "<%=_T('The limit number of databases is required.')%>",
					range: {
						between: {from: 1, to: 3},
						message: "<%=_T('The limit number of databases is out of range. Please enter value between {0} and {1}.')%>"
					}
				},
				on_error: on_others_error
			},
			{
				control_id: "prov_form_vcc_basic_databases_sizing",
				can_validate: can_validate_databases,
				messages: {
					required: "<%=_T('Database sizing is required.')%>",
					range: {
						between: {from: 1, to: 20480},
						message: "<%=_T('Database sizing is out of range. Please enter value between {0} and {1}.')%>"
					}
				},
				on_error: on_others_error
			},
			{
				control_id: "prov_form_vcc_basic_databases_rows_number",
				can_validate: can_validate_databases,
				messages: {
					required: "<%=_T('Maximum number of rows is required.')%>",
					range: {
						between: {from: 1, to: 3000},
						message: "<%=_T('Maximum number of rows is out of range. Please enter value between {0} and {1}.')%>"
					}
				},
				on_error: on_others_error
			},
			{
				control_id: "prov_form_vcc_basic_databases_timestamp",
				can_validate: can_validate_databases,
				messages: {
					required: "<%=_T('Timestamp is required.')%>",
					range: {
						between: {from: 1, to: 365},
						message: "<%=_T('Timestamp is out of range. Please enter value between {0} and {1}.')%>"
					}
				},
				on_error: on_others_error
			},
			// Drafts
			{
				control_id: "prov_form_vcc_basic_drafts_configuration_number",
				can_validate: can_validate_drafts_config_fn,
				messages: {
					required: "<%=_T('The maximum number of drafts allowed is required.')%>",
					digits  : "<%=_T('Only integers are permitted.')%>",
					range: {
						between: {from: 3, to: 10},
						message: "<%=_T('The maximum number of drafts allowed is out of range. Please enter value between {0} and {1}.')%>"
					}
				},
				on_error: on_others_error
			},
			{
				control_id: "prov_form_vcc_basic_drafts_configuration_max_size",
				can_validate: can_validate_drafts_config_fn,
				messages: {
					required: "<%=_T('The maximum size allowed of drafts is required.')%>",
					digits  : "<%=_T('Only integers are permitted.')%>",
					range: {
						between: {from: 0, to: 1024},
						message: "<%=_T('The maximum size allowed of drafts is out of range. Please enter value between {0} and {1}.')%>"
					}
				},
				on_error: on_others_error
			},
			// Phone Validation
			{
				control_id: "prov_form_vcc_basic_phone_validation_configuration_format",
				can_validate: can_validate_phone_validation_config_fn,
				messages: {
					required: "<%=_T('Phone numbers format is required.')%>",
				},
				on_error: on_others_error
			},
			// VCC Configuration limits
			{
				control_id: "prov_form_vcc_basic_config_limits_max_campaigns",
				can_validate: can_validate_config_limits,
				messages: {
					required: "<%=_T('The maximum number of campaigns is required.')%>",
					number  : "<%=_T('Only numbers are allowed in the maximum number of campaigns.')%>",
					custom: {
						type : "custom",
						valid: function(){
							return customValidateRangeConfigurationsLimits(
							$("#prov_form_vcc_basic_config_limits_max_campaigns"), 
							"MaxCampaigns",
							"<%=_T('The maximum number of campaigns is out of range. Please enter value between {0} and {1}.')%>",
							self.m_maxNumberConfigsAvailable,
							self.m_oldConfigLimits
							);
						}
					}
				},
				on_error: on_others_error
			},
			{
				control_id: "prov_form_vcc_basic_config_limits_max_campaigns_users",
				can_validate: can_validate_config_limits,
				messages: {
					required: "<%=_T('The maximum number of campaigns/users is required.')%>",
					number  : "<%=_T('Only numbers are allowed in the maximum number of campaigns/users.')%>",
					custom: {
						type : "custom",
						valid: function(){
							return customValidateRangeConfigurationsLimits(
							$("#prov_form_vcc_basic_config_limits_max_campaigns_users"), 
							"MaxCampaignsUsers",
							"<%=_T('The maximum number of campaigns/users is out of range. Please enter value between {0} and {1}.')%>",
							self.m_maxNumberConfigsAvailable,
							self.m_oldConfigLimits
							);
						}
					}
				},
				on_error: on_others_error
			},
			{
				control_id: "prov_form_vcc_basic_config_limits_max_att_levels",
				can_validate: can_validate_config_limits,
				messages: {
					required: "<%=_T('The maximum number of attentions levels is required.')%>",
					number  : "<%=_T('Only numbers are allowed in the maximum number of attentions levels.')%>",
					custom: {
						type : "custom",
						valid: function(){
							return customValidateRangeConfigurationsLimits(
							$("#prov_form_vcc_basic_config_limits_max_att_levels"), 
							"MaxAttsLevel",
							"<%=_T('The maximum number of att. levels is out of range. Please enter value between {0} and {1}.')%>",
							self.m_maxNumberConfigsAvailable,
							self.m_oldConfigLimits
							);
						}
					}
				},
				on_error: on_others_error
			},
			{
				control_id: "prov_form_vcc_basic_config_limits_max_att_levels_users",
				can_validate: can_validate_config_limits,
				messages: {
					required: "<%=_T('The maximum number of att. levels/users is required.')%>",
					number  : "<%=_T('Only numbers are allowed in the maximum number of att. levels/users.')%>",
					custom: {
						type : "custom",
						valid: function(){
							return customValidateRangeConfigurationsLimits(
							$("#prov_form_vcc_basic_config_limits_max_att_levels_users"), 
							"MaxAttsLevelUsers",
							"<%=_T('The maximum number of att. levels/users is out of range. Please enter value between {0} and {1}.')%>",
							self.m_maxNumberConfigsAvailable,
							self.m_oldConfigLimits
							);
						}
					}
				},
				on_error: on_others_error
			},
			{
				control_id: "prov_form_vcc_basic_config_limits_max_workgroups",
				can_validate: can_validate_config_limits,
				messages: {
					required: "<%=_T('The maximum number of workgroups is required.')%>",
					number  : "<%=_T('Only numbers are allowed in the maximum number of workgroups.')%>",
					custom: {
						type : "custom",
						valid: function(){
							return customValidateRangeConfigurationsLimits(
							$("#prov_form_vcc_basic_config_limits_max_workgroups"), 
							"MaxWorkgroups",
							"<%=_T('The maximum number of workgroups is out of range. Please enter value between {0} and {1}.')%>",
							self.m_maxNumberConfigsAvailable,
							self.m_oldConfigLimits
							);
						}
					}
				},
				on_error: on_others_error
			},
			{
				control_id: "prov_form_vcc_basic_config_limits_max_workgroups_users",
				can_validate: can_validate_config_limits,
				messages: {
					required: "<%=_T('The maximum number of workgroups/users is required.')%>",
					number  : "<%=_T('Only numbers are allowed in the maximum number of workgroups/users.')%>",
					custom: {
						type : "custom",
						valid: function(){
							return customValidateRangeConfigurationsLimits(
							$("#prov_form_vcc_basic_config_limits_max_workgroups_users"), 
							"MaxWorkgroupsUsers",
							"<%=_T('The maximum number of workgroups/user is out of range. Please enter value between {0} and {1}.')%>",
							self.m_maxNumberConfigsAvailable,
							self.m_oldConfigLimits
							);
						}
					}
				},
				on_error: on_others_error
			},
		// Recording Configuration
			{
			// call automatic recording criteria
				control_id: "prov_form_vcc_basic_recording_call_automatic_recording_criteria",
				can_validate: function() {
					return inConcert.app.hasLicenseSystem();
				},	
				messages: {
					required: "<%=_T('A Call automatic recording criteria is required.')%>"
				},
				on_error: on_others_error
			},
			{
			// call format
				control_id: "prov_form_vcc_basic_recording_call_format",
				can_validate: function() {
					return inConcert.app.hasLicenseSystem();
				},	
				messages: {
					required: "<%=_T('A Call format is required.')%>"
				},
				on_error: on_others_error
			},
			{
			// call channels
				control_id: "prov_form_vcc_basic_recording_call_channels",
				can_validate: function() {
					return inConcert.app.hasLicenseSystem();
				},	
				messages: {
					required: "<%=_T('A Call channels is required.')%>"
				},
				on_error: on_others_error
			},
			{
			// videocall automatic recording criteria
				control_id: "prov_form_vcc_basic_recording_videocall_automatic_recording_criteria",
				can_validate: function() {
					return inConcert.app.hasLicenseSystem();
				},	
				messages: {
					required: "<%=_T('A Videocall automatic recording criteria is required.')%>"
				},
				on_error: on_others_error
			},
			{
			// videocall format
				control_id: "prov_form_vcc_basic_recording_videocall_format",
				can_validate: function() {
					return inConcert.app.hasLicenseSystem();
				},	
				messages: {
					required: "<%=_T('A Videocall format is required.')%>"
				},
				on_error: on_others_error
			},
			{
			// videocall mode
				control_id: "prov_form_vcc_basic_recording_videocall_mode",
				can_validate: function() {
					return inConcert.app.hasLicenseSystem();
				},	
				messages: {
					required: "<%=_T('A Videocall mode is required.')%>"
				},
				on_error: on_others_error
			},
			{
			// voicemail automatic recording criteria
				control_id: "prov_form_vcc_basic_recording_voicemail_automatic_recording_criteria",
				can_validate: function() {
					return inConcert.app.hasLicenseSystem();
				},	
				messages: {
					required: "<%=_T('A Voicemail automatic recording criteria is required.')%>"
				},
				on_error: on_others_error
			},
			{
			// voicemail format
				control_id: "prov_form_vcc_basic_recording_voicemail_format",
				can_validate: function() {
					return inConcert.app.hasLicenseSystem();
				},	
				messages: {
					required: "<%=_T('A Voicemail format is required.')%>"
				},
				on_error: on_others_error
			},
			{
			// voicemail channels
				control_id: "prov_form_vcc_basic_recording_voicemail_channels",
				can_validate: function() {
					return inConcert.app.hasLicenseSystem();
				},	
				messages: {
					required: "<%=_T('A Voicemail channels is required.')%>"
				},
				on_error: on_others_error
			},					
		);
		if (can_validate_security_authentication_ldap_fn()) {
			this.ldap_get_validation_def(definitions);
		}
		if (can_validate_security_authentication_azuread_fn()) {
			this.azuread_get_validation_def(definitions);
		}

		// Widgets
		const on_widget_err = function(def) {
			return function(type, message_error) {
				on_integrations_error(function() {
					def.on_callback_error(type, message_error);
				});
			};
		};
		for (let widget_id in this.m_widgets) {
			if (this.m_widgets.hasOwnProperty(widget_id)) {
				const $widget_enabled = this.$get_widget_enabled(widget_id);
				if ($widget_enabled.length > 0) {
					const is_enabled = $widget_enabled.prop("checked");
					if (is_enabled) {
						let widget_config_obj = this.get_widget_instance(widget_id);
						if (typeof (widget_config_obj.get_validation_def) === "function") {
							let widget_validation_def = widget_config_obj.get_validation_def();
							if ($.isArray(widget_validation_def) && widget_validation_def.length > 0) {
								widget_validation_def.forEach(def => {
									def.on_error = on_widget_err(def);
									definitions.push(def);
								});
							}
						}
						else {
							let msg = FORMAT("prov_form_vccs_basic.prototype.get_validation_def - The Widget '{0}' Provisioning class has to implement the 'get_validation_def' method.", widget_id);
							console.log(msg);
							window.alert(msg);
						}
					}
				}
			}
		}
	};

	prov_form_vccs_basic.prototype.editable = function()
	{
		prov_form_vccs_basic.uber.editable.call( this );
		this.frm.editable_buttons();
		this.main_tabs.initialize();
		this.main_tabs.show_tabs_container();
	};

	prov_form_vccs_basic.prototype.readonly = function()
	{
		prov_form_vccs_basic.uber.readonly.call( this );
		this.frm.readonly_buttons();
		this.main_tabs.initialize();
		this.main_tabs.show_tabs_container();
	};

	prov_form_vccs_basic.prototype.only_modified = function(vcc, _, response)
	{
		const self = this;
		
		if (response && response.code === "VCC WAS COMPLETELY UPDATED" && inConcert.app.hasLicenseSystem()) {
			const provisioning_api = inConcert.app.getProvisioningAPI();
			let refresh_unassigned_license_resources = function(success, resources) {
				if(success) {
					self.load_licensed_resources(resources);
				}
			};
			provisioning_api.LoadAvailableLicenseResources(refresh_unassigned_license_resources);
		}

		// Administrator & API SDK User Messages
		if (response && response.code === "PARTIAL UPDATED VCC") {
			let message;

			(response.warningInfo || []).forEach((warning, i)=> {
				message = !message ? "" : message + "\n";
				if (warning.subCode === "UNABLE TO CREATE USER") {
					self.m_vcc_has_users = false;
					if (warning.result === "HELPDESK_INTEGRATION_ERROR") {
						message = "<%=_T('Administrator User could not be created. The integration with HelpDesk has rejected it. Please validate the configuration.')%>";
					}
					else {
						message = "<%=_T('Administrator User could not be created !!')%>";
					}
				}
				else if (warning.subCode === "UNABLE TO CREATE API USER") {
					message = "<%=_T('API SDK User could not be created !!')%>";
				}
				else if (warning.subCode === "UNABLE TO CREATE SUPER USER") {
					message = "<%=_T('Super User could not be created !!')%>";
				}
				else {
					message = `<%=_T('Unrecognized warning')%>  ${i}`;
				}
			});

			this.show_popup_message({
				messages: {
					title  : "<%=_T('The Contact Center has been updated successfully.')%>",
					message: message || "<%=_T('Some errors were found while updating the Contact Center. Consult the administrator.')%>",
					type   : "warn"
				}
			});
		}

		const $user_id = $("#prov_form_vcc_basic_user_id");
		const $name = $("#prov_form_vcc_basic_name");

		this.frm.readonly_control($("#prov_form_vcc_basic_company"), !this.company_is_editable /* disabled */, this.company_is_editable && this.company_is_required /* required */);

		// security captcha type
		const $security_captcha_type = $("#prov_form_vcc_basic_security_captcha_type");

		if (this.frm.is_new) {
			this.frm.readonly_control($name, false /* disabled */, true /* is_required */);
			$name.focus();
			$security_captcha_type.val("DEFAULT").trigger("change");
			$("#prov_form_vcc_basic_security_authentication_type").val("BASIC").trigger("change");
			if(!this.company_is_editable) {
				$("#prov_form_vcc_basic_company").val("******");
			}
		}
		else {
			this.frm.readonly_control($user_id, true /* disabled */);
			this.frm.readonly_control($name, true /* disabled */);
			$("#prov_form_vcc_basic_desc").focus();
			vcc = vcc || this.frm.get_entity_instance();
			if (vcc && vcc.HasAllegroIntegration) {
				const $call_license = $("#prov_form_vcc_basic_licences_section").find("[list_item][channel_id='CALL']");
				$call_license.remove();
				// telephony i6 configuration
				$("#prov_form_vcc_basic_teli6_section").remove();

				// Disable users licensed
				this.frm.readonly_control($("#prov_form_vcc_basic_online_users"), true, false);
			}

			if (vcc && vcc.ConfigLimits) {
				this.set_config_limits_class_required(true);
			}
		}
		// session timeouts
		this.toggle_session_timeouts();
		// security captcha enabled
		const $captcha_enabled = $("#prov_form_vcc_basic_security_captcha_enabled");
		$captcha_enabled.trigger("change");

		this.toggle_security_authentication();

		// security mfa enabled
		//this.toggle_security_authentication_mfa_enabled();
		// helpdesk integration
		this.toggle_helpdesk_integration();
		// mas integration
		this.toggle_mas_integration();
		// muestro u oculto el mensaje de invalid url de access token de MAS
		const $mas_token_url_invalid = $("#prov_form_vcc_basic_mas_integration_token_url_invalid");
		$mas_token_url_invalid.hide();
		if (vcc && vcc.AppsIntegrations) {
			if (vcc.AppsIntegrations.MarketingAndSales && vcc.AppsIntegrations.MarketingAndSales.AccessToken) {
				if (!vcc.AppsIntegrations.MarketingAndSales.AccessToken.Token) {
					$mas_token_url_invalid.show();
				}
			}
		}
		// wfm integration
		this.toggle_wfm_integration();

		// Widgets
		for (let widget_id in this.m_widgets) {
			if (this.m_widgets.hasOwnProperty(widget_id)) {
				this.toggle_widget_integration(widget_id);
				let widget_config_obj = this.get_widget_instance(widget_id);
				if (typeof (widget_config_obj.ready) === "function") {
					widget_config_obj.ready();
				}
				else {
					let msg = FORMAT("prov_form_vccs_basic.prototype.only_modified - The Widget '{0}' Provisioning class has to implement the 'ready' method.", widget_id);
					console.log(msg);
					window.alert(msg);
				}
			}
		}
		if(!inConcert.app.hasLicenseSystem()) {
			this.__disable_licenses_ctrls();
			this.__disable_recording_configuration_ctrls();
		}
		// screen recording
		this.toggle_screen_recording();
		this.toggle_screen_recording_panel();
		// video Config (webcall)
		this.toggle_video_config();
		// Audit cleanUp
		this.toggle_audit_cleanup_config();
		// Databases config
		this.toggle_databases_config();
		// api sdk
		this.toggle_api_sdk_config();
		//Drafts
		this.toggle_drafts_config();
		//Phone validation
		this.toggle_phone_validation_config();
		// Super Users
		this.load_super_users();
	};

	prov_form_vccs_basic.prototype.__disable_recording_configuration_ctrls = function()
	{
		//call
		this.frm.readonly_control($("#prov_form_vcc_basic_recording_call_automatic_recording_criteria"), true);
		this.frm.readonly_control($("#prov_form_vcc_basic_recording_call_format"), true);
		this.frm.readonly_control($("#prov_form_vcc_basic_recording_call_channels"), true);
		//video call
		this.frm.readonly_control($("#prov_form_vcc_basic_recording_videocall_automatic_recording_criteria"), true);
		this.frm.readonly_control($("#prov_form_vcc_basic_recording_videocall_format"), true);
		this.frm.readonly_control($("#prov_form_vcc_basic_recording_videocall_mode"), true);
		//voice mail
		this.frm.readonly_control($("#prov_form_vcc_basic_recording_voicemail_automatic_recording_criteria"), true);
		this.frm.readonly_control($("#prov_form_vcc_basic_recording_voicemail_format"), true);
		this.frm.readonly_control($("#prov_form_vcc_basic_recording_voicemail_channels"), true);
	};

	prov_form_vccs_basic.prototype.__disable_licenses_ctrls = function()
	{
		let self = this;
		// usuarios en linea
		this.frm.readonly_control($("#prov_form_vcc_basic_online_users"), true);
		// otros (metricas, filtros, quotas)
		this.frm.readonly_control($("#prov_form_vcc_basic_custom_metrics_limit"), true);
		this.frm.readonly_control($("#prov_form_vcc_basic_custom_filters_limit"), true);
		this.frm.readonly_control($("#prov_form_vcc_basic_export_space_quota"), true);
		this.frm.readonly_control($("#prov_form_vcc_basic_shared_files_space_quota"), true);
		// canales
		$("#prov_form_vcc_basic_licences_section [selector='1'][type='checkbox']").each(function(_, license) {
			const $checkbox = $(license);
			self.frm.readonly_control($checkbox, true);
		});
	};

	prov_form_vccs_basic.prototype.ready = function()
	{
		this.only_modified();
	};

	prov_form_vccs_basic.prototype.on_show = function()
	{
		prov_form_vccs_basic.uber.on_show.call(this, {show: false});
		const is_readonly = this.is_readonly();
		if (is_readonly) {
			this.readonly();
		}
		else {
			this.editable();
		}
	};

	prov_form_vccs_basic.prototype.toggle_helpdesk_integration = function()
	{
		const is_required = true;
		// helpdesk integration enabled
		const is_enabled_helpdesk_integration = $("#prov_form_vcc_basic_helpdesk_integration_enabled").prop("checked");
		// helpdesk notificaiton url
		const $heldpesk_notification_url = $("#prov_form_vcc_basic_helpdesk_integration_notification_url");
		this.frm.disable_control($heldpesk_notification_url, !is_enabled_helpdesk_integration, is_required);
		// helpdesk server url
		const $helpdesk_server_url = $("#prov_form_vcc_basic_helpdesk_integration_server_url");
		this.frm.disable_control($helpdesk_server_url, !is_enabled_helpdesk_integration, is_required);
	// Adm auth api ------------
		// helpdesk adm auth - userid
		const $helpdesk_auth_adm_user_id = $("#prov_form_vcc_basic_helpdesk_integration_api_adm_user_auth_id");
		this.frm.disable_control($helpdesk_auth_adm_user_id, !is_enabled_helpdesk_integration, is_required);
		// helpdesk adm auth - pwd
		const $helpdesk_auth_adm_pwd = $("#prov_form_vcc_basic_helpdesk_integration_api_adm_user_auth_pwd");
		this.frm.disable_control($helpdesk_auth_adm_pwd, !is_enabled_helpdesk_integration, is_required);

	// Flow auth api ------------
		// helpdesk flow auth - userid
		const $helpdesk_auth_flow_user_id = $("#prov_form_vcc_basic_helpdesk_integration_api_flow_user_auth_id");
		this.frm.disable_control($helpdesk_auth_flow_user_id, !is_enabled_helpdesk_integration, is_required);
		// helpdesk flow auth - pwd
		const $helpdesk_auth_flow_pwd = $("#prov_form_vcc_basic_helpdesk_integration_api_flow_user_auth_pwd");
		this.frm.disable_control($helpdesk_auth_flow_pwd, !is_enabled_helpdesk_integration, is_required);

		// helpdesk user licences
		const $helpdesk_user_licenses = $("#prov_form_vcc_basic_helpdesk_integration_user_licenses");
		this.frm.disable_control($helpdesk_user_licenses, !(is_enabled_helpdesk_integration), is_required);

		// show / hide config fields sections.
		const $basic = $("#prov_form_vcc_basic_helpdesk_basic_section");
		const $auth_adm_section = $("#prov_form_vcc_basic_helpdesk_api_auth_adm_section");
		const $auth_flow_section = $("#prov_form_vcc_basic_helpdesk_api_auth_flow_section");
		const $licenses = $("#prov_form_vcc_basic_helpdesk_licenses_section");
		// first admin user email
		const $user_email_panel = $("#prov_form_vcc_basic_user_email_panel");

		if (is_enabled_helpdesk_integration) {
			$basic.show();
			$auth_adm_section.show();
			$auth_flow_section.show();
			$licenses.show();
			if (this.frm.is_new || !this.m_vcc_has_users) {
				$user_email_panel.show();
			}
		}
		else {
			$basic.hide();
			$auth_adm_section.hide();
			$auth_flow_section.hide();
			$licenses.hide();
			$user_email_panel.hide();
		}
	};

	prov_form_vccs_basic.prototype.toggle_api_sdk_config = function()
	{
		// api sdk enabled
		const is_enabled_api_sdk = $("#prov_form_vcc_basic_api_sdk_enabled").prop("checked");
		// show / hide config fields sections.
		const $api_sdk_panel = $("#prov_form_vcc_basic_api_sdk_panel");
		if (is_enabled_api_sdk) {
			$api_sdk_panel.show();
		}
		else {
			$api_sdk_panel.hide();
		}
		this.frm.disable_control($("#prov_form_vcc_basic_api_sdk_high_rate_limit_time"), true, false);
		this.frm.disable_control($("#prov_form_vcc_basic_api_sdk_medium_rate_limit_time"), true, false);
		this.frm.disable_control($("#prov_form_vcc_basic_api_sdk_medium_light_rate_limit_time"), true, false);
		this.frm.disable_control($("#prov_form_vcc_basic_api_sdk_low_rate_limit_time"), true, false);

		const system_configuration = inConcert.app.getSystemConfiguration("API_SDK");
		const resources = system_configuration.URLsDefaultConfig.resources;
		// time limit en sengundos (txt readonly)
		$("#prov_form_vcc_basic_api_sdk_high_rate_limit_time").val(resources.high.rateLimits.timeLimitSec);
		$("#prov_form_vcc_basic_api_sdk_medium_rate_limit_time").val(resources.low.rateLimits.timeLimitSec);
		$("#prov_form_vcc_basic_api_sdk_medium_light_rate_limit_time").val(resources.low.rateLimits.timeLimitSec);
		$("#prov_form_vcc_basic_api_sdk_low_rate_limit_time").val(resources.low.rateLimits.timeLimitSec);
		// tooltip
		const $api_sdk_enabled_tooltip = $("#prov_form_vcc_basic_api_sdk_enabled_tooltip");
		let title = $api_sdk_enabled_tooltip.prop("title");
		title = FORMAT(title, system_configuration.Cache.vccApiConfiguration.expiredTimeSec);
		$api_sdk_enabled_tooltip.prop("title", title);
	};

	prov_form_vccs_basic.prototype.toggle_mas_integration = function()
	{
		const is_required = true;
		// mas integration enabled
		const is_enabled_mas_integration = $("#prov_form_vcc_basic_mas_integration_enabled").prop("checked");
		// mas notificaiton url
		const $mas_notification_url = $("#prov_form_vcc_basic_mas_integration_notification_url");
		this.frm.disable_control($mas_notification_url, !is_enabled_mas_integration, is_required);
		// mas app url
		const $mas_app_url = $("#prov_form_vcc_basic_mas_integration_application_url");
		this.frm.disable_control($mas_app_url, !is_enabled_mas_integration, is_required);
		// mas token url
		const $mas_token_url = $("#prov_form_vcc_basic_mas_integration_token_url");
		this.frm.disable_control($mas_token_url, !is_enabled_mas_integration, is_required);
		// mas API url
		const $mas_api_url = $("#prov_form_vcc_basic_mas_integration_api_url");
		this.frm.disable_control($mas_api_url, !is_enabled_mas_integration, is_required);
		// show / hide config fields sections.
		const $basic = $("#prov_form_vcc_basic_mas_fields_section");
		if (is_enabled_mas_integration) {
			$basic.show();
		}
		else {
			$basic.hide();
		}
	};

	prov_form_vccs_basic.prototype.toggle_wfm_integration = function()
	{
		const is_required = true;
		// wfm integration enabled
		const $wfm_notification_url = $("#prov_form_vcc_basic_wfm_integration_notification_url");
		// wfm notificaiton url
		const is_enabled_wfm_integration = $("#prov_form_vcc_basic_wfm_integration_enabled").prop("checked");
		this.frm.disable_control($wfm_notification_url, !is_enabled_wfm_integration, is_required);
		// show / hide config fields sections.
		const $basic = $("#prov_form_vcc_basic_wfm_fields_section");
		if (is_enabled_wfm_integration) {
			$basic.show();
		}
		else {
			$basic.hide();
		}
	};

	prov_form_vccs_basic.prototype.toggle_widget_integration = function(widget_id)
	{
		const $widget_enabled = this.$get_widget_enabled(widget_id);
		if ($widget_enabled.length === 0) {
			return;
		}
		const is_enabled_widget_integration = $widget_enabled.prop("checked");
		// show / hide config fields sections.
		const $fields_section = this.$get_widget_fields_section(widget_id);
		if (is_enabled_widget_integration) {
			$fields_section.show();
		}
		else {
			$fields_section.hide();
		}
	};

	prov_form_vccs_basic.prototype.toggle_session_timeouts = function()
	{
		// session timeouts enabled
		const is_enabled_session_timeouts = $("#prov_form_vcc_basic_session_timeouts_enabled").prop("checked");
		// mobile
		const $session_timeouts_mobile = $("#prov_form_vcc_basic_session_timeouts_section");
		$session_timeouts_mobile[is_enabled_session_timeouts ? "show" : "hide"]();
	};

	prov_form_vccs_basic.prototype.toggle_security_captcha_enabled = function()
	{
		const is_required = false;
		// security captcha enabled
		const is_enabled_security_captcha = $("#prov_form_vcc_basic_security_captcha_enabled").prop("checked");
		// security captcha type
		const $security_captcha_type = $("#prov_form_vcc_basic_security_captcha_type");
		this.frm.disable_control($security_captcha_type, !is_enabled_security_captcha, is_required);
		$security_captcha_type.trigger("change");
		// catpcha fields config
		const $security_captcha_basic_fields_config = $("#prov_form_vcc_basic_security_captcha_basic_fields_config");
		$security_captcha_basic_fields_config[is_enabled_security_captcha ? "show" : "hide"]();
	};

	prov_form_vccs_basic.prototype.toggle_security_captcha_custom_enabled = function()
	{
		const is_required = true;
		// security captcha enabled
		const is_enabled_security_captcha = $("#prov_form_vcc_basic_security_captcha_enabled").prop("checked");
		// security captcha type
		const $security_captcha_type = $("#prov_form_vcc_basic_security_captcha_type");
		const security_captcha_type = $security_captcha_type.val();
		const is_enabled_custom_security_captcha = is_enabled_security_captcha && security_captcha_type === "CUSTOM";
		// catpcha fields config
		const $security_captcha_basic_fields_config = $("#prov_form_vcc_basic_security_captcha_basic_fields_config");
		$security_captcha_basic_fields_config.find("[captcha_custom_config]")[is_enabled_custom_security_captcha ? "show" : "hide"]();
		// captcha public_key
		const $public_key = $("#prov_form_vcc_basic_security_captcha_public_key");
		this.frm.disable_control($public_key, !is_enabled_custom_security_captcha, is_required);
		// captcha secret_key
		const $secret_key = $("#prov_form_vcc_basic_security_captcha_secret_key");
		this.frm.disable_control($secret_key, !is_enabled_custom_security_captcha, is_required);
	};

	prov_form_vccs_basic.prototype.toggle_security_authentication = function()
	{
		const type = $("#prov_form_vcc_basic_security_authentication_type").val();
		const passwordChecksEnabled = type === "BASIC" || type === "MFA";
		const mfaEnabled = type === "MFA";
		const ldapEnabled = type === "LDAP";
		const openIdEnabled = type === "OPENID";
		$("#prov_form_vcc_basic_security_authentication_passwordChecks_panel").toggle(passwordChecksEnabled);

		const $expirationDays = $("#prov_form_vcc_basic_security_authentication_passwordChecks_expirationDays");
		const $expirationNotificationDays = $("#prov_form_vcc_basic_security_authentication_passwordChecks_expirationNotificationDays");
		this.frm.disable_control($expirationNotificationDays, passwordChecksEnabled && $expirationDays.val() === "0", passwordChecksEnabled);
		$expirationNotificationDays.val(passwordChecksEnabled && $expirationDays.val() === "0" ? "0" : $expirationNotificationDays.val());

		$("#prov_form_vcc_basic_security_authentication_mfa_panel").toggle(mfaEnabled);
		$("#prov_form_vcc_basic_security_authentication_ldap_panel").toggle(ldapEnabled);

		this.toggle_security_authentication_ldap_sync_enabled();

		$("#prov_form_vcc_basic_security_authentication_openid_panel").toggle(openIdEnabled);
		this.fill_security_authentication_openid_sync_methods(openIdEnabled);
		$("#prov_form_vcc_basic_security_authentication_openid_sync_method").trigger("change");	
	};	

	prov_form_vccs_basic.prototype.toggle_security_authentication_ldap_sync_enabled = function()
	{
		const is_enabled_ldap_sync = $("#prov_form_vcc_basic_security_authentication_ldap_sync_enabled").prop("checked");
		const $ldap_panel = $("#prov_form_vcc_basic_security_authentication_ldap_panel");
		$ldap_panel.find("[LDAP_ROW_GROUP_NAME]").each(function(_, element) {
			const $groupRow = $(element);
			const group = $groupRow.attr("ldap_row_group_name");
			const showGroup = (group === "allusers" && !is_enabled_ldap_sync) || (group !== "allusers" && is_enabled_ldap_sync);
			$groupRow.toggle(showGroup);
		});
		$("#prov_form_vcc_basic_security_authentication_ldap_sync_common").toggle(is_enabled_ldap_sync);
	};

	prov_form_vccs_basic.prototype.toggle_security_authentication_mfa_enabled = function()
	{
		const enabled = $("#prov_form_vcc_basic_security_authentication_type").val() === "MFA";
		this.frm.disable_control($("#prov_form_vcc_basic_security_authentication_mfa_type"), !enabled, true);
		if (enabled) {
			const $type = $("#prov_form_vcc_basic_security_authentication_mfa_type");
			$type.val($type.val() || $type.children(":first").val());
			this.toggle_security_authentication_mfa_type();
		}
		else {
			$("#prov_form_vcc_basic_security_authentication_mfa_panel").children("[mfa_section]").hide();
		}
	};

	prov_form_vccs_basic.prototype.toggle_security_authentication_mfa_type = function()
	{
		const type = $("#prov_form_vcc_basic_security_authentication_mfa_type").val();
		$("#prov_form_vcc_basic_security_authentication_mfa_panel")
			.children("[mfa_section]")
			.hide()
			.filter(FORMAT("[mfa_section={0}]", type))
				.show();
		if (type === "MAIL") {
			const $mailAccount = $("#prov_form_vcc_basic_security_authentication_mfa_mail_account");
			$mailAccount.val($mailAccount.val() || $mailAccount.children(":first").val());
		}
	};
	
	prov_form_vccs_basic.prototype.toggle_video_config = function()
	{
		const is_enabled = $("#prov_form_vcc_basic_video_config_enabled").prop("checked");
		$("#prov_form_vcc_basic_video_configuration_section")[is_enabled ? "show" : "hide"]();
	};

	prov_form_vccs_basic.prototype.toggle_drafts_config = function()
	{
		const is_enabled = $("#prov_form_vcc_basic_drafts_configuration_enabled").prop("checked");
		$("#prov_form_vcc_basic_drafts_configuration_section")[is_enabled ? "show" : "hide"]();
	};

	prov_form_vccs_basic.prototype.toggle_phone_validation_config = function()
	{
		const is_enabled = $("#prov_form_vcc_basic_phone_validation_configuration_enabled").prop("checked");
		$("#prov_form_vcc_basic_phone_validation_configuration_section")[is_enabled ? "show" : "hide"]();
	};

	prov_form_vccs_basic.prototype.toggle_audit_cleanup_config = function()
	{
		const is_enabled = $("#prov_form_vcc_basic_audit_cleanup_enabled").prop("checked");
		$("#prov_form_vcc_basic_audit_configuration_section")[is_enabled ? "show" : "hide"]();
	};

	prov_form_vccs_basic.prototype.toggle_databases_config = function()
	{
		const is_enabled = $("#prov_form_vcc_basic_databases_configuration_panel_enabled").prop("checked");
		$("#prov_form_vcc_basic_databases_configuration_panel_container")[is_enabled ? "show" : "hide"]();
	};

	prov_form_vccs_basic.prototype.toggle_screen_recording = function()
	{
		const is_enabled = $("#prov_form_vcc_basic_screen_recording_enabled").prop("checked");
		$("#prov_form_vcc_basic_screen_recording_section")[is_enabled ? "show" : "hide"]();
	};

	prov_form_vccs_basic.prototype.toggle_screen_recording_panel = function()
	{
		const $screen_panel = $("#prov_form_vcc_basic_screen_recording_panel");
		const vcc = this.frm.get_entity_instance();
		if (vcc && (vcc.HasAllegroIntegration || (!vcc.HasAllegroIntegration && inConcert.app.is_os_debian()))) {
			$screen_panel.hide();
			$("#prov_form_vcc_basic_screen_recording_enabled").prop("checked", false);
			this.toggle_screen_recording();
		}
		else{
			$screen_panel.show();
		}
	};

	prov_form_vccs_basic.prototype.toggle_security_authentication_azuread_sync_enabled = function()
	{
		const self = this;

		//clean filters
		const $azuread_panel = $("#prov_form_vcc_basic_security_authentication_azuread_sync_panel");
		$azuread_panel.find("[AZUREAD_ROW_GROUP_NAME]").each(function(_, item) {
			const $item = $(item);
			const group = $item.attr("AZUREAD_ROW_GROUP_NAME");
			const selector = jQUERY_SELECTOR_AZUREAD_GROUP_FILTER(group);
			const $filter = $azuread_panel.find(selector);
			$filter.empty();
		});

		let validation_def = {
			control_id: "prov_form_vcc_basic_security_authentication_openid_configuration_url",
			can_validate: true,
			messages: {
				custom: {
					has_error: false,
					error_message: "",
					type : "custom",
					valid: function() {
						let response = {
							success: true
						};
						if(this.has_error) {
							response.success = false;
							response.message = this.error_message;
						}
						return response;
					}
				}
			}
		};
		const definitions = [];
		definitions.push(validation_def);

		const $azuread_sync_country = $("#prov_form_vcc_basic_security_authentication_azuread_sync_country");
		const $chk = $("#prov_form_vcc_basic_security_authentication_azuread_sync_enabled");
		if (!$chk.prop("checked")) {
			$azuread_sync_country.trigger("change");
			return;
		}

		this.validate(function( success ) {
				if (!success) {
					$chk.prop("checked", false);
					return;
				}
				const $password =  $("#prov_form_vcc_basic_security_authentication_azuread_sync_client_secret"); 
				const jwtToken  = inConcert.Utils.GetJwtToken($password.val());
				const data = {
					vcc: $("#prov_form_vcc_basic_name").val(),
					openid_configuration_url : $("#prov_form_vcc_basic_security_authentication_openid_configuration_url").val(),
					client_id : $("#prov_form_vcc_basic_security_authentication_azuread_sync_client_id").val(),
					client_secret : jwtToken
				};
				self.frm.dashboard.show_waiting("<%=_T('Recovering Azure AD groups')%>");
				inConcert.app.getProvisioningAPI().getAzureAdGroups(data, function(success, response) {
					self.frm.dashboard.hide_waiting();
					if(!success) {
						validation_def.messages.custom.has_error = true;
						validation_def.messages.custom.error_message = response;
						self.validate(function(_) {}, definitions);
						console.log(response);
						$chk.prop("checked", false);
						return;
					}
					if(!response.success) {
						validation_def.messages.custom.has_error = true;
						validation_def.messages.custom.error_message = "<%=_T('Azure AD Error:')%>" + " (" + response.error + ")";
						self.validate(function(_) {}, definitions);
						console.log(response.error);
						$chk.prop("checked", false);
						return;
					}
					if(!response.result || response.result.length === 0) {
						validation_def.messages.custom.has_error = true;
						validation_def.messages.custom.error_message =  "<%=_T('No results returned')%>";
						self.validate(function(_) {}, definitions);
						$chk.prop("checked", false);
						return;
					}
					
					const vcc = self.frm.get_entity_instance();
					const groupFilters = vcc && vcc.Security && vcc.Security.openid && vcc.Security.openid.groupFilters || {};

					$azuread_panel.find("[AZUREAD_ROW_GROUP_NAME]").each(function(_, item) {
						const $item = $(item);
						const group = $item.attr("AZUREAD_ROW_GROUP_NAME");
						const selector = jQUERY_SELECTOR_AZUREAD_GROUP_FILTER(group);
						const $filter = $azuread_panel.find(selector);
						self.fill_combo($filter, false, response.result, "id", "displayName");
						$filter.val(groupFilters[group] && groupFilters[group].id);
					});
				});
			}, 
			this.azuread_common_validation_definitions
		);
		$azuread_sync_country.trigger("change");
	};

	prov_form_vccs_basic.prototype.set_events = function()
	{
		const self = this;

		prov_form_vccs_basic.uber.set_events.call(this); //invokes the base class method

		// session timeouts enabled
		$("#prov_form_vcc_basic_session_timeouts_enabled")
			.off("change")
			.on("change", function() {
				self.toggle_session_timeouts();
			});

		// authentication type
		$("#prov_form_vcc_basic_security_authentication_type")
			.off("change")
			.on("change", function() {
				self.toggle_security_authentication();
			});

		// openid sync method
		$("#prov_form_vcc_basic_security_authentication_openid_sync_method")
			.off("change")
			.on("change", function() {
				$("#prov_form_vcc_basic_security_authentication_azuread_sync_panel").toggle($(this).val() === "AZUREAD");
			});

		const $ldap_panel = $("#prov_form_vcc_basic_security_authentication_ldap_panel");
		// ldap sync enabled
		$("#prov_form_vcc_basic_security_authentication_ldap_sync_enabled")
			.off("change")
			.on("change", function() {
				self.toggle_security_authentication_ldap_sync_enabled();
				if ($(this).prop("checked")) {
					const selector = jQUERY_SELECTOR_LDAP_GROUP_FILTER("allusers");
					$ldap_panel.find(selector).val("");
				}
				else {
					$ldap_panel.find("[LDAP_ROW_GROUP_NAME] [LDAP_GROUP_FILTER]").val("");
				}
				$("#prov_form_vcc_basic_security_authentication_ldap_sync_country").trigger("change");
			});

		// azuread sync enabled 
		$("#prov_form_vcc_basic_security_authentication_azuread_sync_enabled")
			.off("change")
			.on("change", function() {
				self.toggle_security_authentication_azuread_sync_enabled();
			});

		// ldap country
		$("#prov_form_vcc_basic_security_authentication_ldap_sync_country")
			.off("change")
			.on("change", function() {
				self.fill_security_authentication_ldap_sync_areas($(this).val());
			});

		// Azure AD country
		$("#prov_form_vcc_basic_security_authentication_azuread_sync_country")
			.off("change")
			.on("change", function() {
				self.fill_security_authentication_azuread_sync_areas($(this).val());
			});

		// captcha enabled
		$("#prov_form_vcc_basic_security_captcha_enabled")
			.off("change")
			.on("change", function() {
				self.toggle_security_captcha_enabled();
			});

		// captcha custom type enabled
		$("#prov_form_vcc_basic_security_captcha_type")
			.off("change")
			.on("change", function() {
				self.toggle_security_captcha_custom_enabled();
			});

		// expirationDays
		$("#prov_form_vcc_basic_security_authentication_passwordChecks_expirationDays")
			.off("blur")
			.on("blur", function() {
				let $expirationNotificationDays = $("#prov_form_vcc_basic_security_authentication_passwordChecks_expirationNotificationDays");
				self.frm.disable_control($expirationNotificationDays, $( this ).val() === "0", true);
				$expirationNotificationDays.val($( this ).val() === "0" ? "0" : $expirationNotificationDays.val());
			});

		// mfa custom type enabled
		$("#prov_form_vcc_basic_security_authentication_mfa_type")
			.off("change")
			.on("change", function() {
				self.toggle_security_authentication_mfa_type();
			});

		// helpdesk integration
		$("#prov_form_vcc_basic_helpdesk_integration_enabled")
			.off("change")
			.on("change", function() {
				self.toggle_helpdesk_integration();
			});

		// mas integration
		$("#prov_form_vcc_basic_mas_integration_enabled")
			.off("change")
			.on("change", function() {
				self.toggle_mas_integration();
			});

		// wfm integration
		$("#prov_form_vcc_basic_wfm_integration_enabled")
			.off("change")
			.on("change", function() {
				self.toggle_wfm_integration();
			});

		// Widgets
		const makeChangeWidgetFn = function(widget_id) {
			return function() {
				self.toggle_widget_integration(widget_id);
			};
		};
		for (let widget_id in this.m_widgets) {
			if (this.m_widgets.hasOwnProperty(widget_id)) {
				const $widget_enabled = this.$get_widget_enabled(widget_id);
				if ($widget_enabled.length > 0) {
					$widget_enabled
						.off("change")
						.on("change", makeChangeWidgetFn(widget_id));

					let widget_config_obj = this.get_widget_instance(widget_id);
					if (typeof (widget_config_obj.set_events) === "function") {
						widget_config_obj.set_events();
					}
					else {
						let msg = FORMAT("prov_form_vccs_basic.prototype.set_events - The Widget '{0}' Provisioning class has to implement the 'set_events' method.", widget_id);
						console.log(msg);
						window.alert(msg);
					}
				}
			}
		}

		// outgoing batches
		this.m_outgoing_batches_channels.set_events();
		// social networks apps
		this.m_social_networks_app_channel.set_events();

		// screen recording enabled
		$("#prov_form_vcc_basic_screen_recording_enabled")
			.off("change")
			.on("change", function() {
				self.toggle_screen_recording();
			});
		
		// video config (webcall) enabled
		$("#prov_form_vcc_basic_video_config_enabled")
			.off("change")
			.on("change", function() {
				self.toggle_video_config();
			});
		// Audit CleanUp Enable
		$("#prov_form_vcc_basic_audit_cleanup_enabled")
			.off("change")
			.on("change", function() {
				self.toggle_audit_cleanup_config();
			});

		// Databases COnfig
		$("#prov_form_vcc_basic_databases_configuration_panel_enabled")
			.off("change")
			.on("change", function() {
				self.toggle_databases_config();
			});

		// API SDK
		$("#prov_form_vcc_basic_api_sdk_enabled")
			.off("change")
			.on("change", function() {
				self.toggle_api_sdk_config();
			});

		// Drafts
		$("#prov_form_vcc_basic_drafts_configuration_enabled")
			.off("change")
			.on("change", function() {
				self.toggle_drafts_config();
			});

		// Phone validation
		$("#prov_form_vcc_basic_phone_validation_configuration_enabled")
			.off("change")
			.on("change", function() {
				self.toggle_phone_validation_config();
			});

		if(inConcert.app.hasRepositoryService()) {
			// repository service
			$("#prov_form_vcc_basic_storage_location")
				.off("change")
				.on("change", function() {
					self.toggle_S3_storage_location();
				});

			// Predefined bucket checkbox
			$("#prov_form_vcc_basic_storage_location_params_S3BucketCheckBox")
				.off("change")
				.on("change", function() {
					const $chk                 = $(this);
					const $s3Lifecycle         = $("#prov_form_vcc_basic_storage_s3_flc");
					const $predefinedBucket    = $("#prov_form_vcc_basic_storage_location_params_S3Bucket");

					if ($chk.prop("checked")) {
						$s3Lifecycle.toggle(false);
						self.frm.disable_control($predefinedBucket   , false, true);
					}
					else {
						$s3Lifecycle.toggle(true);
						self.frm.disable_control($predefinedBucket   ,  true, false);
					}
				});
		}
	};

	prov_form_vccs_basic.prototype.ldap_get_validation_def = function (definitions) {
		const self = this;
		this.ldap_common_validation_definitions.forEach(definition => definitions.push(definition));

		const on_error_fn = function() { self.switch_tab("security"); };
		const $ldap_panel = $("#prov_form_vcc_basic_security_authentication_ldap_panel");

		const get_group_definition = function(group) {
			let groupId = "prov_form_vcc_basic_security_authentication_ldap_group_" + group;
			let filterId = groupId + "_filter";
			return {
				control_id: filterId,
				can_validate: true,
				messages: {
					required: FORMAT("<%=_T('Filter for group {0} is required.')%>", group),
					custom: {
						valid: function() {
							const response = {
								success: true
							};
							const filters = [];
							$ldap_panel.find("[LDAP_ROW_GROUP_NAME]").each(function(_, item) {
								const $item = $(item);
								const selectedGroup = $item.attr("ldap_row_group_name");
								const $ldapGroupFilter = $item.find("[ldap_group_filter]");
								const selectedFilterValue = $ldapGroupFilter.val();
								if(selectedFilterValue !== "") {
									const selectedFilterId = $ldapGroupFilter.attr("id");
									if(filters[selectedFilterValue] && filters[selectedFilterValue] !== selectedFilterId) {
										response.success = false;
										response.message = FORMAT("<%=_T('Selected filter is in use by role {0}')%>", selectedGroup);
										return false;
									}
									filters[selectedFilterValue] = selectedFilterId;
								}
							});
							return response;
						}
					}
				},
				on_error: on_error_fn
			};
		};

		let ldap_sync_validation_definitions = [];
		if($("#prov_form_vcc_basic_security_authentication_ldap_sync_enabled").prop("checked")) {

			const can_validate_security_authentication_ldap_sync_fn = function() {
				return $("#prov_form_vcc_basic_security_authentication_type").val() === "LDAP";
			};

			$ldap_panel.find("[LDAP_GROUP_NAME]").each(function(_, item) {
				const $item = $(item);
				const group = $item.val();
				if (group === "allusers")
					return;
				ldap_sync_validation_definitions.push(
					get_group_definition(group)
				);
			});

			ldap_sync_validation_definitions.push(
				{
				// timezone
					control_id: "prov_form_vcc_basic_security_authentication_ldap_sync_timezone",
					can_validate: can_validate_security_authentication_ldap_sync_fn,
					messages: {
						required: "<%=_T('Time Zone is required.')%>"
					},
					on_error: on_error_fn
				},
				{
				// language
					control_id: "prov_form_vcc_basic_security_authentication_ldap_sync_language",
					can_validate: can_validate_security_authentication_ldap_sync_fn,
					messages: {
						required: "<%=_T('Language is required.')%>"
					},
					on_error: on_error_fn
				},
				{
				// country
					control_id: "prov_form_vcc_basic_security_authentication_ldap_sync_country",
					can_validate: can_validate_security_authentication_ldap_sync_fn,
					messages: {
						required: "<%=_T('Country is required.')%>"
					},
					on_error: on_error_fn
				},
				{
				// area
					control_id: "prov_form_vcc_basic_security_authentication_ldap_sync_area",
					can_validate: can_validate_security_authentication_ldap_sync_fn,
					messages: {
						required: "<%=_T('Area code is required.')%>"
					},
					on_error: on_error_fn
				},
				{
				// campaign
					control_id: "prov_form_vcc_basic_security_authentication_ldap_sync_default_campaign",
					can_validate: can_validate_security_authentication_ldap_sync_fn,
					messages: {
						required: "<%=_T('Default Campaign is required.')%>"
					},
					on_error: on_error_fn
				},
				{
				// state
					control_id: "prov_form_vcc_basic_security_authentication_ldap_sync_initial_state",
					can_validate: can_validate_security_authentication_ldap_sync_fn,
					messages: {
						required: "<%=_T('Initial State is required.')%>"
					},
					on_error: on_error_fn
				}
			);
		}
		else {
			ldap_sync_validation_definitions.push(
				get_group_definition("allusers")
			);
		}

		ldap_sync_validation_definitions.forEach(definition => definitions.push(definition));
	};

	prov_form_vccs_basic.prototype.azuread_get_validation_def = function (definitions) {
		const self = this;
		this.azuread_common_validation_definitions.forEach(definition => definitions.push(definition));

		const on_error_fn = function() { self.switch_tab("security"); };
		const $azuread_panel = $("#prov_form_vcc_basic_security_authentication_azuread_sync_panel");

		const get_group_definition = function(group) {
			let groupId = "prov_form_vcc_basic_security_authentication_azuread_group_" + group;
			let filterId = groupId + "_filter"; 
			return {
				control_id: filterId,
				can_validate: true,
				messages: {
					required: FORMAT("<%=_T('Azure AD Group for group {0} is required.')%>", group),
					custom: {
						valid: function() {
							const response = {
								success: true
							};
							const filters = [];
							$azuread_panel.find("[AZUREAD_ROW_GROUP_NAME]").each(function(_, item) {
								const $item = $(item);
								const selectedGroup = $item.attr("AZUREAD_ROW_GROUP_NAME");
								const $filter = $item.find("[azured_group_filter]");
								const selectedFilterValue = $filter.val();
								if(selectedFilterValue !== "") {
									const selectedFilterId = $filter.attr("id");
									if(filters[selectedFilterValue] && filters[selectedFilterValue] !== selectedFilterId) {
										response.success = false;
										response.message = FORMAT("<%=_T('Selected Azure AD Group is in use by role {0}')%>", selectedGroup);
										return false;
									}
									filters[selectedFilterValue] = selectedFilterId;
								}
							});
							return response;
						}
					}
				},
				on_error: on_error_fn
			};
		};
	
		const can_validate_security_authentication_azuread_sync_fn = function() {
			return $("#prov_form_vcc_basic_security_authentication_type").val() === "OPENID"  && $("#prov_form_vcc_basic_security_authentication_openid_sync_method").val() === "AZUREAD";
		};

		$azuread_panel.find("[AZUREAD_ROW_GROUP_NAME]").each(function(_, item) {
			const $item = $(item);
			const group = $item.attr("AZUREAD_ROW_GROUP_NAME");
			definitions.push(
				get_group_definition(group)
			);
		});

		definitions.push(
			{
			// timezone
				control_id: "prov_form_vcc_basic_security_authentication_azuread_sync_timezone",
				can_validate: can_validate_security_authentication_azuread_sync_fn,
				messages: {
					required: "<%=_T('Time Zone is required.')%>"
				},
				on_error: on_error_fn
			},
			{
			// language
				control_id: "prov_form_vcc_basic_security_authentication_azuread_sync_language",
				can_validate: can_validate_security_authentication_azuread_sync_fn,
				messages: {
					required: "<%=_T('Language is required.')%>"
				},
				on_error: on_error_fn
			},
			{
			// country
				control_id: "prov_form_vcc_basic_security_authentication_azuread_sync_country",
				can_validate: can_validate_security_authentication_azuread_sync_fn,
				messages: {
					required: "<%=_T('Country is required.')%>"
				},
				on_error: on_error_fn
			},
			{
			// area
				control_id: "prov_form_vcc_basic_security_authentication_azuread_sync_area",
				can_validate: can_validate_security_authentication_azuread_sync_fn,
				messages: {
					required: "<%=_T('Area code is required.')%>"
				},
				on_error: on_error_fn
			},
			{
			// campaign
				control_id: "prov_form_vcc_basic_security_authentication_azuread_sync_default_campaign",
				can_validate: can_validate_security_authentication_azuread_sync_fn,
				messages: {
					required: "<%=_T('Default Campaign is required.')%>"
				},
				on_error: on_error_fn
			},
			{
			// state
				control_id: "prov_form_vcc_basic_security_authentication_azuread_sync_initial_state",
				can_validate: can_validate_security_authentication_azuread_sync_fn,
				messages: {
					required: "<%=_T('Initial State is required.')%>"
				},
				on_error: on_error_fn
			}
		);
	};

	prov_form_vccs_basic.prototype.load_widgets = function(widgets)
	{
		const $widgets_integration_panel = $("#prov_form_vcc_basic_widgets_integration_panel");
		this.m_widgets = widgets;
		if (!this.m_widgets || Object.keys(this.m_widgets).length === 0) {
			$widgets_integration_panel.hide();
			return;
		}
		$widgets_integration_panel.show();
		this.make_widgets_markup();
	};

	prov_form_vccs_basic.prototype.make_widgets_markup = function() {
		const $template = $("#prov_form_vcc_basic_widgets_template_panel");
		const html_template = $template.html();
		if (!html_template) {
			return;
		}
		const $widget_section = $("#prov_form_vcc_basic_widgets_integration_panel");
		let has_widget_config = false;
		for (let widget_id in this.m_widgets) {
			if (this.m_widgets.hasOwnProperty(widget_id)) {
				let widget_info = this.m_widgets[widget_id];
				// markup
				let html_widget_config = widget_info.provisioning.html.body.vcc.content;
				let html = FORMAT(html_template, widget_id, widget_info.displayName, html_widget_config);
				let current_html = $widget_section.html();
				$widget_section.html(current_html + html);
				// js class instance
				let widget_class_name = widget_info.provisioning.jsClassName;
				let widget_class = eval(widget_class_name);
				widget_info.provisioning.instanceVccConfig = new widget_class({owner: this});
			}
		}
		if (has_widget_config) {
			$widget_section.show();
		}
	};

	prov_form_vccs_basic.prototype.get_widget_instance = function(widget_id) {
		let widget_info = this.m_widgets[widget_id];
		if (!widget_info || !widget_info.provisioning) {
			return;
		}
		return widget_info.provisioning.instanceVccConfig;
	};

	prov_form_vccs_basic.prototype.__set_total_max_license_resources = function(resources) {
		this.MaxOnlineUsers = resources.MaxOnlineUsers + (resources.OnlineUsers || 0);
		this.MAX_CustomMetricsLimit = resources.MAX_CustomMetricsLimit + (resources.CustomMetricsLimit || 0);
		this.MAX_CustomFiltersLimit = resources.MAX_CustomFiltersLimit + (resources.CustomFiltersLimit || 0);
		this.MAX_ExportSpaceQuota = resources.MAX_ExportSpaceQuota + (resources.ExportSpaceQuota || 0);
		this.MAX_SharedFilesSpaceQuota = resources.MAX_SharedFilesSpaceQuota + (resources.SharedFilesSpaceQuota || 0);
		if(resources.MAX_AppsIntegrations) {
			for (let app in this.MAX_AppsIntegrations) {
				if (this.MAX_AppsIntegrations.hasOwnProperty(app) && resources.MAX_AppsIntegrations.hasOwnProperty(app)) {
					this.MAX_AppsIntegrations[app] = resources.MAX_AppsIntegrations[app] + ((resources.AppsIntegrations && resources.AppsIntegrations[app] && resources.AppsIntegrations[app].Licenses && resources.AppsIntegrations[app].Licenses.Users && !isNaN(parseInt(resources.AppsIntegrations[app].Licenses.Users)) && parseInt(resources.AppsIntegrations[app].Licenses.Users))  || 0);
				}
			}
		}
	};

	prov_form_vccs_basic.prototype.set_config_limits_class_required = function(required) {
		let workgroups_users_label = $("#prov_form_vcc_basic_config_limits_max_workgroups_users_label");
		let workgroups_label = $("#prov_form_vcc_basic_config_limits_max_workgroups_label");
		let att_levels_users_label = $("#prov_form_vcc_basic_config_limits_max_att_levels_users_label");
		let att_levels_label = $("#prov_form_vcc_basic_config_limits_max_att_levels_label");
		let campaigns_users_label = $("#prov_form_vcc_basic_config_limits_max_campaigns_users_label");
		let campaigns_label = $("#prov_form_vcc_basic_config_limits_max_campaigns_label");

		if (required) {
			workgroups_users_label.removeClass("required").addClass("required");
			workgroups_label.removeClass("required").addClass("required");
			att_levels_users_label.removeClass("required").addClass("required");
			att_levels_label.removeClass("required").addClass("required");
			campaigns_users_label.removeClass("required").addClass("required");
			campaigns_label.removeClass("required").addClass("required");
		}
		else{
			workgroups_users_label.removeClass("required");
			workgroups_label.removeClass("required");
			att_levels_users_label.removeClass("required");
			att_levels_label.removeClass("required");
			campaigns_users_label.removeClass("required");
			campaigns_label.removeClass("required");
		}
	};

	prov_form_vccs_basic.prototype.set_config_limits_titles = function() {
		const self = this;
		if (!this.m_maxNumberConfigsAvailable) {
			return;
		}
		this.m_oldConfigLimits = this.m_oldConfigLimits || {
			MaxCampaigns : 0,
			MaxCampaignsUsers : 0,
			MaxAttsLevel : 0,
			MaxAttsLevelUsers : 0,
			MaxWorkgroups : 0,
			MaxWorkgroupsUsers : 0
		};
		const getMessage = function (attrName) {
			return FORMAT("(<%=_T('Limit {0}')%>)", self.m_oldConfigLimits[attrName] + self.m_maxNumberConfigsAvailable[attrName]);
		};
		$("#prov_form_vcc_basic_config_limits_max_workgroups_users_free_limit").text(getMessage("MaxWorkgroupsUsers"));
		$("#prov_form_vcc_basic_config_limits_max_workgroups_free_limit").text(getMessage("MaxWorkgroups"));
		$("#prov_form_vcc_basic_config_limits_max_att_levels_users_free_limit").text(getMessage("MaxAttsLevelUsers"));
		$("#prov_form_vcc_basic_config_limits_max_att_levels_free_limit").text(getMessage("MaxAttsLevel"));
		$("#prov_form_vcc_basic_config_limits_max_campaigns_users_free_limit").text(getMessage("MaxCampaignsUsers"));
		$("#prov_form_vcc_basic_config_limits_max_campaigns_free_limit").text(getMessage("MaxCampaigns"));
	};

	prov_form_vccs_basic.prototype.set_license_max_resources = function(resources) {
		$("#prov_form_vcc_basic_online_users_unassigned").text(FORMAT("<%=_T('({0} unassigned)')%>", (resources && resources.MaxOnlineUsers) || 0));

		$("#prov_form_vcc_basic_custom_metrics_limit_unassigned").text(FORMAT("<%=_T('({0} unassigned)')%>", (resources && resources.MAX_CustomMetricsLimit) || 0));

		$("#prov_form_vcc_basic_custom_filters_limit_unassigned").text(FORMAT("<%=_T('({0} unassigned)')%>", (resources && resources.MAX_CustomFiltersLimit) || 0));

		$("#prov_form_vcc_basic_export_space_quota_unassigned").text(FORMAT("<%=_T('({0} unassigned)')%>", (resources && resources.MAX_ExportSpaceQuota) || 0));

		$("#prov_form_vcc_basic_shared_files_space_quota_unassigned").text(FORMAT("<%=_T('({0} unassigned)')%>", (resources && resources.MAX_SharedFilesSpaceQuota) || 0));

		// Buscar cada App y setear el label de unassigned correspondiente
		for (let app in resources.MAX_AppsIntegrations) {
			if (resources.MAX_AppsIntegrations.hasOwnProperty(app)) {
				const $app_user_licenses = $("#prov_form_vcc_basic_" + app.toLowerCase() + "_integration_user_licenses_unassigned");
				$app_user_licenses.text(FORMAT("<%=_T('({0} unassigned)')%>", (resources && resources.MAX_AppsIntegrations && resources.MAX_AppsIntegrations[app]) || 0));
			}
		}
	};

	prov_form_vccs_basic.prototype.fill_storage_locations = function()
	{
		const locations = [
			{id: "cassandra", description: "Cassandra"},
			{id: "S3", description: "Amazon S3"}
		];
		this.fill_combo($("#prov_form_vcc_basic_storage_location"), false, locations, "id", "description");
	};

	prov_form_vccs_basic.prototype.toggle_S3_storage_location = function()
	{
		// S3 storage enabled
		const is_enabled_storage_location_params = $("#prov_form_vcc_basic_storage_location").val() === "S3";
		$("#prov_form_vcc_basic_storage_location_params").toggle(is_enabled_storage_location_params);
		$("#prov_form_vcc_basic_storage_location_params_S3Region_disclaimer").toggle(is_enabled_storage_location_params);
		$("#prov_form_vcc_basic_prefix_config_container").toggle(is_enabled_storage_location_params);
		$("#prov_form_vcc_basic_prefix_config_disclaimer").toggle(is_enabled_storage_location_params);
	};

	prov_form_vccs_basic.prototype.fill_S3Regions = function(regions = {})
	{
		var $combo = $("#prov_form_vcc_basic_storage_location_params_S3Region");
		this.fill_combo($combo, false, regions, "id", "description", true);
	};

	//---------------------------------------------------------------------
	// SUPER USERS.
	//---------------------------------------------------------------------
	prov_form_vccs_basic.prototype.fill_super_users = function(superUsers = {})
	{
		this.m_grid_super_users.fill_rows($.noop, superUsers);
	};

	prov_form_vccs_basic.prototype.load_super_users = function(superUsers = {})
	{
		const data = {
			vcc: $("#prov_form_vcc_basic_name").val()
		};
		inConcert.app.getProvisioningAPI()
			.superUsersSearch(data, (success, response)=> {
				if (!success) throw("Error");
				this.fill_super_users(response);
			}
		);
	};

	prov_form_vccs_basic.prototype.validate_super_user_to_add_or_update = function()
	{
		this.hide_super_users_popup_errors();
		let response = {success: false, $ctrl: null, item: null, msg: "<%=_T('Unable to Add Super User.')%>"};

		let superUser = {};
		let is_new = true;
		if (this.m_current_super_user) {
			is_new = false;
			superUser = this.m_current_super_user;
		}

		/*
			funcion que realiza validacion de campos requeridos
		*/
		const ValidateField = ($ctrl, fieldId, errMsg, type = "required")=> {
			superUser[fieldId] = $ctrl.val().trim();
			const validation_result = inConcert.Utils.validate(superUser[fieldId], type);
			if (!validation_result.success) {
				response.msg = errMsg;
				response.$ctrl = $ctrl;
				this.show_super_user_popup_ctrl_err(response);
				return response;
			}
			// ok
			return {success: true};
		};

		// VCC ID validation -----
		if (ValidateField(
				$("#prov_form_vcc_basic_name"),
				"vccId",
				"<%=_T('VCC Name is required in the Basic tab configuration.')%>"
			).success === false
		) return response; // error

		// ID validation (usa el mismo codigo de validacion que se utiliza para dar de alta el primer usuario admin del vcc)
		//
		const $user_id = $("#prov_form_vcc_super_users_popup_id");
		superUser.Id = $user_id.val();
		let validation_result = m_commonValidations.users.Id.validateFn(this, $user_id);
		if (!validation_result.success) {
			response.msg = validation_result.message;
			response.$ctrl = $user_id;
			this.show_super_user_popup_ctrl_err(response);
			return response;
		}

		// Already exist validation
		if (is_new) {
			let current_all_super_users = this.m_grid_super_users.get_items();
			current_all_super_users = inConcert.Utils.arrayToMap(current_all_super_users || [],
				function(superUser) {
					return superUser;
				},
				function(superUser) {
					// el ID
					return (superUser.Id || "").toLowerCase();
				}
			);
			const show_already_error = ()=> {
				response.msg = "<%=_T('Id already exist.')%>";
				response.$ctrl = $user_id;
				this.show_super_user_popup_ctrl_err(response);
				return response;
			};
			let lower_id = $user_id.val().toLowerCase();
			const already = current_all_super_users[lower_id];
			if (already) {
				return show_already_error();
			}
		}

		// User LanguageId validation -----
		//
		// Required validation
		if (ValidateField(
				$("#prov_form_vcc_super_users_popup_app_language"),
				"LanguageId",
				"<%=_T('User language is required.')%>"
			).success === false
		) return response; // error

		// User Name validation -----
		//
		// Required validation
		//
		if (ValidateField(
				$("#prov_form_vcc_super_users_popup_name"),
				"Name",
				"<%=_T('User name is required.')%>"
			).success === false
		) return response; // error

		// User FirstSurname validation -----
		//
		// Required validation
		if (ValidateField(
				$("#prov_form_vcc_super_users_popup_first_surname"),
				"FirstSurname",
				"<%=_T('User first surname is required.')%>"
			).success === false
		) return response; // error

		// User Password validation -----
		//
		if (is_new || this.m_super_user_password_has_changed) {
			const $password = $("#prov_form_vcc_super_users_popup_pwd");
			superUser.plainPassword = $password.val();
			const $password_confirmation = $("#prov_form_vcc_super_users_popup_pwd_confirmation");
			if (superUser.plainPassword !== $password_confirmation.val()) {
				response.success = false;
				response.msg = "<%=_T('Passwords do not match.')%>";
				response.$ctrl = $password_confirmation;
				this.show_super_user_popup_ctrl_err(response);
				return response;
			}
			const mustValidateSecurity = false; // para que realice validaciones de que la pwd es requerida, pero que no realice validaciones de seguridad (ldap, etc)
			validation_result = m_commonValidations.users.Password.validateFn(this, $password, mustValidateSecurity);
			if (!validation_result.success) {
				response.msg = validation_result.message;
				response.$ctrl = $password;
				this.show_super_user_popup_ctrl_err(response);
				return response;
			}
		}

		// E-mail validation -----
		//
		const $email = $("#prov_form_vcc_super_users_popup_mail");
		// Required validation
		if (ValidateField(
				$email,
				"Mail",
				"<%=_T('Please enter the e-mail address.')%>"
			).success === false
		) return response; // error
		// valid email -----
		if (ValidateField(
				$email,
				"Mail",
				"<%=_T('Please enter a valid e-mail address.')%>",
				"email" // tipo de validacion
			).success === false
		) return response; // error

		// User TimeZoneId validation -----
		//
		// Required validation
		if (ValidateField(
				$("#prov_form_vcc_super_users_popup_timezone"),
				"TimeZoneId",
				"<%=_T('User timezone is required.')%>"
			).success === false
		) return response; // error

		// User countryCode validation -----
		//
		// Required validation
		if (ValidateField(
				$("#prov_form_vcc_super_users_popup_country"),
				"countryCode",
				"<%=_T('User country is required.')%>"
			).success === false
		) return response; // error

		// User areaCode validation -----
		//
		// Required validation
		const $area = $("#prov_form_vcc_super_users_popup_area");
		// const areaErrMsg = "<%=_T('User area is required.')%>";
		superUser.areaCode = this.__get_selected_area($area);
		// el area puede venir con esta clave cuando NO se ha seleccionado una en el cbo
		if (superUser.areaCode === "__BLANK") {
			// valida que el area del usuario
			response.msg =  "<%=_T('User area is required.')%>";
			response.$ctrl = $area;
			this.show_super_user_popup_ctrl_err(response);
			return response;
		}

		// OK
		response.success = true;
		response.msg = "";
		response.item = superUser;

		return response;
	};

	prov_form_vccs_basic.prototype.save_super_user = function(must_create_and_contiune, _, callback = $.noop)
	{
		const data = this.validate_super_user_to_add_or_update();
		if (!data.success)
			return callback(false);
		/*
		*/
		const getMsgErr = ({rawError} = {})=> {
			const default_err = "<%=_T('Unable save Super User.')%>";
			if (!rawError)
				return default_err;

			const fieldId = rawError.fieldId || "<%=_T('unrecognized')%>";

			switch (rawError.result) {
				case "UNABLE ADD SUPER USER - REACHED THE MAXIMUM ALLOWED":
					return FORMAT("<%=_T([[Unable to Save Super User. You have reached the maximum number of super users allowed in the VCC. Maximum allowed: {0}.]])%>", rawError.maxAllowed);
			
				case "FILE IS REQUIRED":
					return FORMAT("<%=_T([[Field '{0}' is required.]])%>", rawError.fieldId);

				case "VALUE NOT ALLOWED":
					// errores especificos
					switch (rawError.msg) {
						//mail errors
						case "MAIL ALREADY EXISTS":
							return "<%=_T('E-mail address taken by another user, please choose another one.')%>";
						case "MAIL IDENTIFIER IN USE":
							return "<%=_T('E-mail address taken by another user, please choose another one.')%>";
						case "MAIL IS REQUIRED IN VCC WITH HELPDESK INTEGRATION":
							return "<%=_T('E-mail is required in VCC with HelpDesk Integration.')%>";
					}
					// default errors
					return FORMAT("<%=_T([[Value is not allowed in field '{0}'.]])%>", fieldId);
			}

			return rawError.reason || rawError.msg || default_err;
		};

		this.m_grid_super_users.show_popup_waiting("<%=_T('Please wait ...')%>");
		// Un timeout para dar feedback ya que puede ser muy rapida la actualizacion. Al menos en mi servidor.
		window.setTimeout(()=> {
			inConcert.app.getProvisioningAPI()
				.superUserSave(data.item, (success, response)=> {
					this.m_grid_super_users.hide_popup_waiting();
					if (!success) {
						response.msg = getMsgErr(response);
						this.show_super_user_popup_ctrl_err(response);
						return callback(false);
					}
					const msg_response = {
						msg : "<%=_T('Super User was successfully saved.')%>",
						type: "Info"
					};
					callback(true, data.item, msg_response);
					this.clean_up_controls_super_users();
				}
			);
		}, 1500);
	};

	prov_form_vccs_basic.prototype.clean_up_controls_super_users = function()
	{
		const $id = $("#prov_form_vcc_super_users_popup_id");
		this.readonly_control($id, false /* disabled */, true /* required */);
		$id.val("").focus();
		$("#prov_form_vcc_super_users_popup_name").val("");
		$("#prov_form_vcc_super_users_popup_pwd").val("");
		$("#prov_form_vcc_super_users_popup_pwd_confirmation").val("");
		$("#prov_form_vcc_super_users_popup_first_surname").val("");
		$("#prov_form_vcc_super_users_popup_app_language").val("");
		$("#prov_form_vcc_super_users_popup_mail").val("");
		$("#prov_form_vcc_super_users_popup_timezone").val("");
		$("#prov_form_vcc_super_users_popup_country").val("").trigger("change");
		$("#prov_form_vcc_super_users_popup_area").val("__BLANK");
	};

	prov_form_vccs_basic.prototype.hide_super_users_popup_errors = function()
	{
		this.m_grid_super_users.hide_popup_msg();
	};

	prov_form_vccs_basic.prototype.show_super_user_popup_ctrl_err = function(err_info)
	{
		if (!err_info) {
			return;
		}
		const $control = err_info.$ctrl;
		if ($control) {
			const error_id = $control.attr("id") + "_validation";
			const $error_indicator = $("#" + error_id);
			if ($error_indicator) {
				$error_indicator.show();
			}
			$control.focus();
		}
		this.m_grid_super_users.show_popup_msg(err_info.msg, "Error"); //, ERROR_TIMEOUT_MS);
	};

	prov_form_vccs_basic.prototype.on_open_super_user_popup = function(superUser)
	{
		// super popups events
		this.on_events_popup_super_users();

		this.readonly_control($("#prov_form_vcc_super_users_popup_pwd_confirmation"), true /* disabled */);
		this.m_super_user_password_has_changed = false;
		this.m_current_super_user = superUser;
		this.clean_up_controls_super_users();
		if (!this.m_current_super_user) {
			// Nuevo Usuario
			return;
		}
		// Editando Usuario
		delete this.m_current_super_user.plainPassword; // la debo de remover
		const $id = $("#prov_form_vcc_super_users_popup_id");
		$id.val(superUser.Id);
		this.readonly_control($id, true /* disabled */);
		$("#prov_form_vcc_super_users_popup_app_language").val(superUser.LanguageId).focus();
		$("#prov_form_vcc_super_users_popup_pwd").val("********");
		$("#prov_form_vcc_super_users_popup_name").val(superUser.Name);
		$("#prov_form_vcc_super_users_popup_first_surname").val(superUser.FirstSurname);
		$("#prov_form_vcc_super_users_popup_mail").val(superUser.Mail);
		$("#prov_form_vcc_super_users_popup_timezone").val(superUser.TimeZoneId);
		$("#prov_form_vcc_super_users_popup_country")
			.val(superUser.countryCode)
			.trigger("change", ()=> {
				// (*aca) el callback que se invoca cuando se invoca el trigger
				$("#prov_form_vcc_super_users_popup_area").val(superUser.areaCode);
			}
		);
	};

	prov_form_vccs_basic.prototype.on_closed_super_user_popup = function()
	{
		this.m_current_super_user = null;
		this.off_events_popup_super_users();
	};

	prov_form_vccs_basic.prototype.__init_grid_super_users = function()
	{
		this.m_grid_super_users = new inConcert.conf_form_paginator({
			owner: this,
			frm: this.frm,
			auto_initialize: true,
			row_id: "Id",
			display_name: "Id",
			rows_per_page: 20,
			show_at_least: 19,
			paginator_type: "ALL_ROWS_CLIENT_SIDE",
			without_paginator: false,
			loading_waiting_msg: "<%=_T('Please wait, loading Super Users ...')%>",
			ids: {
				// paginator
				paginator: "prov_form_vcc_super_users_paginator",
				grid: "prov_form_vcc_super_users_list",
				not_results_found: "prov_form_vcc_super_users_not_results_found",

				// header actions
				container_new_btn: "prov_form_vcc_super_users_actions_container",

				// popup0
				new_item: "prov_form_vcc_super_users_create",
				popup: "prov_form_vcc_super_users_popup",
				popup_msg: "prov_form_vcc_super_users_popup_message",
				popup_create: "prov_form_vcc_super_users_popup_create",
				popup_create_and_continue: "prov_form_vcc_super_users_popup_create_and_continue",
				popup_title: "prov_form_vcc_super_users_popup_title",
				popup_close: "prov_form_vcc_super_users_popup_close",
				popup_waiting: "prov_form_vcc_super_users_popup_waiting",
			},
			strings: {
				not_results_found: "<%=_T('No Super Users found.')%>",
				// popup0
				title_new                  : "<%=_T('Create Super user')%>",
				title_update               : "<%=_T('Edit Super user')%>",
				button_create              : "<%=_T('Confirm')%>",
				button_create_and_continue : "<%=_T('Confirm & continue')%>",
				button_update              : "<%=_T('Confirm')%>"
			},
			columns: [
				{id: "_number"     , name: "", type: "counter"       },
				{id: "Id"          , name: "<%=_T('Id')%>"           },
				{id: "Name"        , name: "<%=_T('Name')%>"         },
				{id: "FirstSurname", name: "<%=_T('Last Name')%>"},
				{
					id  : "_action",
					name: "",
					system_action: {
						edit : true,
						erase: false
					}
				}
			],
			callbacks: {
				save: (callback, must_create_and_contiune, current_popup)=> {
					callback = callback || $.noop;
					this.save_super_user(must_create_and_contiune, current_popup, callback);
				},
				edit: (row_data)=> {
					this.on_open_super_user_popup(row_data);
				},
				on_closed_popup: ()=> {
					this.on_closed_super_user_popup();
				},
				on_loaded_items: (callback)=> {
					this._on_loaded_grd_super_users(callback);
				},
				on_added_row: (/*item*/)=> {
					this._on_loaded_grd_super_users();
				}
			}
		});
	};

	prov_form_vccs_basic.prototype._on_loaded_grd_super_users = function(callback = $.noop)
	{
		const totalSuperUsers = this.m_grid_super_users.count_rows();
		$("#prov_form_vcc_super_users_help")[totalSuperUsers > 0 ? "show" : "hide"]();
		callback();
	};

	prov_form_vccs_basic.prototype.on_events_popup_super_users = function()
	{
		const self = this;
		const $area = $("#prov_form_vcc_super_users_popup_area");

		let timeoutHnd = null;
		// countries
		$("#prov_form_vcc_super_users_popup_country")
			.off("change")
			.on("change", function() {
				if (timeoutHnd) window.clearTimeout(timeoutHnd);
				timeoutHnd = null;
				// Acceder al callback
				const callback = arguments[1];
				const country_code = $(this).val();
				timeoutHnd = window.setTimeout(function() {
					timeoutHnd = null;
					self.fill_combo_areas(country_code, $area, true);
					if (country_code === "")
						self.readonly_control($area, true /*disabled*/);
					else
						self.readonly_control($area, false /*disabled*/, true /*required*/);
					// Verificar si se proporcionó un callback y lo ejecuto cuando se invoco el trigger (*aca)
					if (typeof callback === "function")
						callback();
				}, 50);
			});

		// Esta funcion muestra el textbox de confirmacion de pwd.
		const show_pwd_confirm = function() {
			if (self.m_super_user_password_has_changed) return;
			self.m_super_user_password_has_changed = true;
			self.readonly_control($("#prov_form_vcc_super_users_popup_pwd_confirmation"), false /* disabled */, true, /*required*/);
		};
		//---------------
		// eventos (paste, keypress, keydown) del text de pwd que habilitan el textbox de confirmación de pwd
		$("#prov_form_vcc_super_users_popup_pwd")
			.on("paste", null, show_pwd_confirm)	//evento paste, ya sea desde el menu contextual o presionando ctrl-v
			.on("keypress", show_pwd_confirm)		//este evento captura las teclas presionandas que modifican el valor, menos las teclas "backspace" y "delete"
			.on("keydown", function(event) {
				/*
					En el evento keydown capturo y proceso las teclas que se han presionado y que modifican el texto existente.
						-keyCode 8  = tecla "backspace" y -keyCode 46 = tecla "delete"
				*/
				if (event.keyCode === 8 || event.keyCode === 46)
					show_pwd_confirm();
			}
		);
	};

	prov_form_vccs_basic.prototype.off_events_popup_super_users = function()
	{
		// countries
		$("#prov_form_vcc_super_users_popup_country").off("change");
		//---------------
		// eventos (paste, keypress, keydown) del text de pwd que habilitan el textbox de confirmación de pwd
		$("#prov_form_vcc_super_users_popup_pwd")
			.off("paste")
			.off("keypress")
			.off("keydown");
	};

	/*
		Validacion en comun utilizada tanto desde la pantalla Basic (Tab del vcc) de usuarios, como en el popup de Super Usuarios.
	*/
	m_commonValidations.users = {
		Id: {
			validateFn: (section, $user)=> {
				const response = {
					success: true
				};
				if (!section.validate_field($user, "required")) {
					// error
					response.success = false;
					response.message = "<%=_T('User Id is required.')%>";
					return response;
				}
				if (!section.validate_field($user, "id")) {
					// error
					response.success = false;
					response.message = "<%=_T('Only letters, numbers, dashes and dots are allowed in the user Id.')%>";
					return response;
				}
				const range = {between: {from: 8, to: 255}};
				const user_id_length = $user.val().length.toString();
				const validation_result = inConcert.Utils.validate(user_id_length, "range", range);
				if (!validation_result.success) {
					// error
					response.success = false;
					response.message = FORMAT("<%=_T('Length of User Id must be between {0} and {1} characters.')%>", range.between.from, range.between.to );
					return response;
				}
				// is ok
				return response;
			}
		},
		Password: {
			validateFn: (section, $password, mustValidateSecurity = true)=> {
				const response = {
					success: true
				};
				if (!section.validate_field($password, "required")) {
					// error
					response.success = false;
					response.message = "<%=_T('Password is required.')%>";
					return response;
				}
				if (!mustValidateSecurity) {
					// is ok
					return response;
				}
				const type = $("#prov_form_vcc_basic_security_authentication_type").val();
				const passwordChecksEnabled = type === "BASIC" || type === "MFA";
				const minUpperChars = parseInt($("#prov_form_vcc_basic_security_authentication_passwordChecks_minUpperChars").val());
				const minLowerChars = parseInt($("#prov_form_vcc_basic_security_authentication_passwordChecks_minLowerChars").val());
				const minSpecialChars = parseInt($("#prov_form_vcc_basic_security_authentication_passwordChecks_minSpecialChars").val());
				const minNumericChars = parseInt($("#prov_form_vcc_basic_security_authentication_passwordChecks_minNumericChars").val());
				const minLen = parseInt($("#prov_form_vcc_basic_security_authentication_passwordChecks_minLen").val());
				const maxLen = parseInt($("#prov_form_vcc_basic_security_authentication_passwordChecks_maxLen").val());
				const passwordChecksValid = !isNaN(minUpperChars) && !isNaN(minLowerChars) && !isNaN(minSpecialChars) && !isNaN(minNumericChars) && !isNaN(minLen) && !isNaN(maxLen);
				if (passwordChecksEnabled && passwordChecksValid) {
					const password = $password.val();
					const numUpper = password.length - password.replace(/[A-Z]/g, "").length; 
					const numLower = password.length - password.replace(/[a-z]/g, "").length; 
					const numSpecial = password.length - password.replace(/[^a-zA-Z0-9]/g, "").length; 
					const numNumbers = password.length - password.replace(/[0-9]/g, "").length; 
					if (password.length < minLen || password.length > maxLen || numUpper < minUpperChars || numLower < minLowerChars || numSpecial < minSpecialChars || numNumbers < minNumericChars) {
						// error
						response.success = false;
						response.message = "<%=_T('Password does not meet security requirements.')%>";
						return response;
					}
				}
				// is ok
				return response;
			}
		}
	};

}());
